$applicationName = $null
$releaseDate = $null
$buildNuget = $null
$wpfOrSl = $null
$wpfSlList = "WPF", "SL"

"------"
"Before proceeding with this script, please make sure to read the 'Protecting the Assemblies.rtf' file in this directory."
"------"

DO
{
    $applicationName = Read-Host -Prompt 'Please, enter your application name'
} While ([String]::IsNullOrWhiteSpace($applicationName))

DO
{
    $userInput = Read-Host -Prompt 'Choose whether to rebuild the WPF or Silverlight assembly - enter (WPF/SL)'
    $wpfOrSl = $userInput.ToUpper()
} While (!($wpfSlList -contains $wpfOrSl))

DO
{
    $releaseDate = Read-Host -Prompt 'Specify ReleaseDate in the following format - MM/DD/YYYY'
} While (!(($releaseDate) -match '(0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])[- /.](19|20)\d\d'))

DO
{
    $userInput = Read-Host -Prompt 'Build NuGet package (y/n)'
    $userInputToLower = $userInput.ToLower()
    if ($userInputToLower.Equals('y')) {
        $buildNuget = $true
    }
    elseif($userInputToLower.Equals('n')){
        $buildNuget =  $false
    }
} While ($null -eq $buildNuget)

$scriptDir = split-path -parent $MyInvocation.MyCommand.Definition
$sourceCodeDir = (get-item $scriptDir).parent.parent.parent.FullName
$batFileDir =  Join-Path -Path $scriptDir -ChildPath \BatFiles
$nuspecsDir = Join-Path -Path $scriptDir -ChildPath \Nuspecs

$assemblyProtectionFileName  = Join-Path -Path $sourceCodeDir -ChildPath \Documents\Licensing\AssemblyProtection.cs
Set-ItemProperty $assemblyProtectionFileName -name IsReadOnly -value $false

$updatedContent = 
	[System.Io.File]::ReadAllText($assemblyProtectionFileName) `
	-replace "(?ms)\/\/\/\/return.*?true;", "return ValidatePassPhrase();" `
	-replace "(internal const string ApplicationName = )(.*?)(;)", "`$1""$applicationName""`$3"
[System.Io.File]::WriteAllText($assemblyProtectionFileName, $updatedContent)

$nugetOutputDirectory = $null
$controlsDllPath = $null
$nugetCommandPropertiesString = $null
$nuspecFile = $null
$fullVersion = $null
$version = $null

if($wpfOrSl.Equals("WPF")){
    $buildControlsFileName = "$batFileDir\Build_Documents_Core.bat Release $releaseDate WPF" 
    cmd.exe /C $buildControlsFileName
    $buildControlsFileName = "$batFileDir\Build_Documents_Core.bat Release45 $releaseDate WPF" 
    cmd.exe /C $buildControlsFileName

    $controlsDllPath = Join-Path -Path $sourceCodeDir -ChildPath \Binaries\WPF40\Dev\Telerik.Windows.Documents.Core.dll
    $fullVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo($controlsDllPath).FileVersion
    $version = $fullVersion.subString(0, $fullVersion.LastIndexOf('.'))
    
    $nugetCommandPropertiesString = "DocumentProcesingVersion=`"" + $version +  "`";PartialID=`"Net40`";PartialTitle=`"WPF`";DocumentProcesingPartialID=`"Net40`";DPLWpfNetCorePartialID=`"`";SourceDir=`"" + $sourceCodeDir + "`";BinariesSubDirnet40=`"Binaries\WPF40\Dev`""
    $nuspecFile = Join-Path -Path $nuspecsDir -ChildPath \Telerik.Windows.Documents.Core.nuspec
}
else{
    $buildControlsFileName = "$batFileDir\Build_Documents_Core.bat Release $releaseDate SL" 
    cmd.exe /C $buildControlsFileName
    
    $controlsDllPath = Join-Path -Path $sourceCodeDir -ChildPath \Binaries\Silverlight5\Dev\Telerik.Windows.Documents.Core.dll
    $fullVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo($controlsDllPath).FileVersion
    $version = $fullVersion.subString(0, $fullVersion.LastIndexOf('.'))
    
    $nugetCommandPropertiesString = "DocumentProcesingVersion=`"" + $version +  "`";PartialID=`"Net40`";PartialTitle=`"Silverlight`";DocumentProcesingPartialID=`"Net40`";SourceDir=`"" + $sourceCodeDir + "`";BinariesSubDirsl5=`"Binaries\Silverlight5\Dev`""
    $nuspecFile = Join-Path -Path $nuspecsDir -ChildPath \Telerik.Windows.Documents.Core_SL.nuspec
}

$nugetOutputDirectory = Join-Path -Path $scriptDir -ChildPath \NugetOutput
if($buildNuget){ 
    $nugetExePath = Join-Path -Path $sourceCodeDir -ChildPath \Build\Tools  
       
    $nugetCommand = "$($nugetExePath)\nuget.exe pack $($nuspecFile) -Version $($version) -OutputDirectory $($nugetOutputDirectory) -Properties $($nugetCommandPropertiesString)"
    cmd.exe /C $nugetCommand
}