﻿using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Common.FormatProviders
{
    /// <summary>
    /// Represents base type for binary format provider.
    /// </summary>
    /// <typeparam name="T">The type of the document.</typeparam>
    public abstract class BinaryFormatProviderBase<T> : FormatProviderBase<T>, IBinaryFormatProvider<T>
    {
        internal virtual bool ShouldDisposeMemoryStreamOnImport
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The document.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000:Dispose objects before losing scope")]
        public T Import(byte[] input)
        {
            Guard.ThrowExceptionIfNull(input, "input");

            MemoryStream inputStream = new MemoryStream(input);

            if (this.ShouldDisposeMemoryStreamOnImport)
            {
                using (inputStream)
                {
                    return this.Import(inputStream);
                }
            }
            else
            {
                return this.Import(inputStream);
            }
        }

        /// <summary>
        /// Exports the specified document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns>The result byte array.</returns>
        public byte[] Export(T document)
        {
            Guard.ThrowExceptionIfNull(document, "document");

            using (MemoryStream outputStream = new MemoryStream())
            {
                this.Export(document, outputStream);
               
                return outputStream.ToArray();
            }
        }
    }
}
