﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Telerik.Windows.Documents.Common.FormatProviders
{
    /// <summary>
    /// Represents interface for format provider.
    /// </summary>
    public interface IFormatProvider<T>
    {
        /// <summary>
        /// Gets the supported extensions.
        /// </summary>
        /// <value>The supported extensions.</value>
        IEnumerable<string> SupportedExtensions { get; }

        /// <summary>
        /// Gets a value indicating whether format provider can import.
        /// </summary>
        /// <value>The value indicating whether can import.</value>
        bool CanImport { get; }

        /// <summary>
        /// Gets a value indicating whether format provider can export.
        /// </summary>
        /// <value>The value indicating whether can export.</value>
        bool CanExport { get; }

        /// <summary>
        /// Imports the specified input stream.
        /// </summary>
        /// <param name="input">The input stream.</param>
        /// <returns>The imported document.</returns>
        T Import(Stream input);

        /// <summary>
        /// Exports the specified document to the output stream.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <param name="output">The output stream.</param>
        void Export(T document, Stream output);
    }
}
