﻿using System.Xml;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Import
{
    internal class OpenXmlPartReader : IOpenXmlReader
    {
        private readonly XmlReader reader;

        public OpenXmlPartReader(XmlReader reader)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");

            this.reader = reader;
        }

        XmlNodeType IOpenXmlReader.GetNodeType()
        {
            if (this.reader == null)
            {
                return XmlNodeType.None;
            }
            return this.reader.NodeType;
        }

        bool IOpenXmlReader.IsEmptyElement()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.IsEmptyElement;
        }

        bool IOpenXmlReader.HasAttributes()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.HasAttributes;
        }

        bool IOpenXmlReader.MoveToFirstAttribute()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.MoveToFirstAttribute();
        }

        bool IOpenXmlReader.MoveToNextAttribute()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.MoveToNextAttribute();
        }

        bool IOpenXmlReader.IsElementOfType(XmlNodeType type)
        {
            if (this.reader == null)
            {
                return false;
            }

            return this.reader.NodeType == type;
        }

        bool IOpenXmlReader.Read()
        {
            if (this.reader == null)
            {
                return false;
            }

            if (this.reader.Read())
            {
                return true;
            }

            return false;
        }

        void IOpenXmlReader.SkipElement()
        {
            if (this.reader == null)
            {
                return;
            }

            this.reader.Skip();
        }

        bool IOpenXmlReader.GetElementName(out string elementName)
        {
            elementName = string.Empty;

            if (this.reader == null)
            {
                return false;
            }

            if (((IOpenXmlReader)this).IsElementOfType(XmlNodeType.Element))
            {
                elementName = this.reader.LocalName;
                return true;
            }

            return false;
        }

        bool IOpenXmlReader.GetElementValue(out string value)
        {
            if (this.reader == null)
            {
                value = string.Empty;

                return false;
            }
            else
            {
                value = this.reader.Value;

                return true;
            }
        }

        bool IOpenXmlReader.GetAttributeNameAndValue(out string attributeName, out string namespaceURI, out string value)
        {
            attributeName = string.Empty;
            value = string.Empty;
            namespaceURI = string.Empty;

            if (this.reader == null)
            {
                return false;
            }

            if (((IOpenXmlReader)this).IsElementOfType(XmlNodeType.Attribute))
            {
                attributeName = this.reader.LocalName;
                value = this.reader.Value;
                namespaceURI = this.reader.NamespaceURI;

                return true;
            }

            return false;
        }
    }
}
