﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core
{
    internal class OpenXmlChildElement<T> : IOpenXmlChildElement where T : OpenXmlElementBase
    {
        private readonly string elementName;
        private readonly string uniquePoolId;

        private T element;

        public OpenXmlChildElement(string elementName)
            : this(elementName, elementName)
        {

        }

        public OpenXmlChildElement(string elementName, string uniqueElementName)
        {
            Guard.ThrowExceptionIfNull(elementName, "elementName");
            Guard.ThrowExceptionIfNull(uniqueElementName, "uniqueElementName");

            this.elementName = elementName;
            this.uniquePoolId = uniqueElementName;
        }

        public OpenXmlChildElement(T element)
        {
            Guard.ThrowExceptionIfNull(element, "element");

            this.element = element;
            this.elementName = this.element.ElementName;
        }

        public T Element
        {
            get
            {
                return this.element;
            }
            set
            {
                if (this.element != value)
                {
                    this.element = value;
                }
            }
        }

        string IOpenXmlChildElement.ElementName
        {
            get
            {
                return this.elementName;
            }
        }

        // TODO: This property is for cases when the same element has to be created with a different namespace or for the case when different elements have the same element names.
        // It is used for the creation of elements and in most cases will be the same as ElementName unless when necessary. The solution is not ideal and in the future it would be best
        // to be replaced by a FullName (= Namespace + ElementName) property which will replace both UniquePoolId and ElementName. Take care to ensure this will be indeed enough
        // to identify the element uniquely, as there are elements that have the same name and same namespace, see Chart and ChartReference.
        string IOpenXmlChildElement.UniquePoolId
        {
            get
            {
                return this.uniquePoolId;
            }
        }

        bool IOpenXmlChildElement.HasElement
        {
            get
            {
                return this.Element != null;
            }
        }

        void IOpenXmlChildElement.SetElement(OpenXmlElementBase element)
        {
            this.Element = (T)element;
        }

        OpenXmlElementBase IOpenXmlChildElement.GetElement()
        {
            return this.Element;
        }

        void IOpenXmlChildElement.ClearElement()
        {
            this.Element = null;
        }
    }
}
