﻿using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core
{
    internal class SpaceAttribute : OpenXmlAttributeBase
    {
        private const string Preserve = "preserve";

        private readonly OpenXmlElementBase element;

        public SpaceAttribute(OpenXmlElementBase element)
            : base("space", new OpenXmlNamespace("", "xml", ""))
        {
            Guard.ThrowExceptionIfNull(element, "element");

            this.element = element;
        }

        public string Value
        {
            get;
            set;
        }

        public override bool HasValue
        {
            get
            {
                return !string.IsNullOrEmpty(this.Value);
            }
        }

        public override string GetValueAsString()
        {
            return this.Value;
        }

        public override void SetStringValue(Documents.FormatProviders.OpenXml.Import.IOpenXmlImportContext context, string value)
        {
            this.Value = value;
        }

        public override bool ShouldExport()
        {
            string innerText = this.element.InnerText;
            if (!string.IsNullOrEmpty(innerText))
            {
                if (char.IsWhiteSpace(innerText.First()) || char.IsWhiteSpace(innerText.Last()))
                {
                    this.Value = Preserve;

                    return true;
                }
            }

            return false;
        }

        public override void ResetValue()
        {
            this.Value = null;
        }
    }
}
