﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal abstract class AxisElementBase : ChartElementBase
    {
        private readonly OpenXmlChildElement<AxisIdElement> axisId;
        private readonly OpenXmlChildElement<ScalingElement> scaling;
        private readonly OpenXmlChildElement<DeleteElement> delete;
        private readonly OpenXmlChildElement<AxisPositionElement> axisPosition;
        private readonly OpenXmlChildElement<MajorGridlinesElement> majorGridlines;
        private readonly OpenXmlChildElement<NumberFormatElement> numberFormat;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;
        private readonly OpenXmlChildElement<CrossesAxisElement> crossesAxis;

        public AxisElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.axisId = this.RegisterChildElement<AxisIdElement>(OpenXmlElementNames.AxisIdElementName);
            this.scaling = this.RegisterChildElement<ScalingElement>(OpenXmlElementNames.ScalingElementName);
            this.delete = this.RegisterChildElement<DeleteElement>(OpenXmlElementNames.DeleteElementName);
            this.axisPosition = this.RegisterChildElement<AxisPositionElement>(OpenXmlElementNames.AxisPositionElementName);
            this.majorGridlines = this.RegisterChildElement<MajorGridlinesElement>(OpenXmlElementNames.MajorGridlinesElementName);
            this.numberFormat = this.RegisterChildElement<NumberFormatElement>(OpenXmlElementNames.NumberFormatElementName, OpenXmlUniquePoolIds.NumberFormatElementUniquePoolId);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName, OpenXmlUniquePoolIds.ChartShapePropertiesUniquePoolId);
            this.crossesAxis = this.RegisterChildElement<CrossesAxisElement>(OpenXmlElementNames.CrossesAxisElementName);
        }

        public AxisIdElement AxisIdElement
        {
            get
            {
                return this.axisId.Element;
            }
            set
            {
                this.axisId.Element = value;
            }
        }

        public ScalingElement ScalingElement
        {
            get
            {
                return this.scaling.Element;
            }
            set
            {
                this.scaling.Element = value;
            }
        }

        public AxisPositionElement AxisPositionElement
        {
            get
            {
                return this.axisPosition.Element;
            }
            set
            {
                this.axisPosition.Element = value;
            }
        }

        public CrossesAxisElement CrossesAxisElement
        {
            get
            {
                return this.crossesAxis.Element;
            }
            set
            {
                this.crossesAxis.Element = value;
            }
        }

        public DeleteElement DeleteElement
        {
            get
            {
                return this.delete.Element;
            }
            set
            {
                this.delete.Element = value;
            }
        }

        public NumberFormatElement NumberFormatElement
        {
            get
            {
                return this.numberFormat.Element;
            }
            set
            {
                this.numberFormat.Element = value;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
            set
            {
                this.shapeProperties.Element = value;
            }
        }

        public MajorGridlinesElement MajorGridlinesElement
        {
            get
            {
                return this.majorGridlines.Element;
            }
            set
            {
                this.majorGridlines.Element = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Axis axis, int axisIndex, int crossingAxisIndex, AxisPosition position)
        {
            this.CreateElement(this.axisId);
            this.AxisIdElement.Value = axisIndex;

            this.CreateElement(this.scaling);
            this.ScalingElement.CopyPropertiesFrom(axis);

            this.CreateElement(this.delete);
            this.DeleteElement.Value = !axis.IsVisible;

            this.CreateElement(this.axisPosition);
            this.AxisPositionElement.Value = position;

            if (axis.MajorGridlines.Outline.Fill != null || axis.MajorGridlines.Outline.Width != null)
            {
                this.CreateElement(this.majorGridlines);
                this.MajorGridlinesElement.CopyPropertiesFrom(context, axis.MajorGridlines);
            }

            if (!string.IsNullOrEmpty(axis.NumberFormat))
            {
                this.CreateElement(this.numberFormat);
                this.NumberFormatElement.FormatCode = axis.NumberFormat;
            }

            this.CreateElement(this.shapeProperties);
            this.ShapePropertiesElement.CopyPropertiesFrom(context, axis);

            this.CreateElement(this.crossesAxis);
            this.CrossesAxisElement.Value = crossingAxisIndex;
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, Axis axis, AxisGroupName groupName)
        {
            int thisId;
            int otherId;

            if (this.AxisIdElement != null)
            {
                thisId = this.AxisIdElement.Value;
                this.ReleaseElement(this.axisId);
            }
            else
            {
                throw new InvalidOperationException("The axId element is required.");
            }

            if (this.ScalingElement != null)
            {
                this.ScalingElement.CopyPropertiesTo(axis);
                this.ReleaseElement(this.scaling);
            }

            if (this.AxisPositionElement != null)
            {
                this.ReleaseElement(this.axisPosition);
            }

            if (this.CrossesAxisElement != null)
            {
                otherId = this.CrossesAxisElement.Value;
                context.RegisterAxisGroup(groupName, thisId, otherId);

                this.ReleaseElement(this.crossesAxis);
            }
            else
            {
                throw new InvalidOperationException("The crossAx element determines how the axes cross one another and is required.");
            }

            if (this.DeleteElement != null)
            {
                axis.IsVisible = !this.DeleteElement.Value;
                this.ReleaseElement(this.delete);
            }

            if (this.NumberFormatElement != null)
            {
                axis.NumberFormat = this.NumberFormatElement.FormatCode;
                this.ReleaseElement(this.numberFormat);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, axis);
                this.ReleaseElement(this.shapeProperties);
            }

            if (this.MajorGridlinesElement != null)
            {
                this.MajorGridlinesElement.CopyPropertiesTo(context, axis.MajorGridlines);
                this.ReleaseElement(this.majorGridlines);
            }
        }
    }
}
