﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class BarChartElement : ChartGroupWithAxesElementBase<BarSeriesGroup>
    {
        private readonly OpenXmlChildElement<BarDirectionElement> barDirection;
        private readonly OpenXmlChildElement<BarGroupingElement> grouping;
        private readonly OpenXmlChildElement<OverlapElement> overlap;

        public BarChartElement(OpenXmlPartsManager partsManager) 
            : base(partsManager)
        {
            this.barDirection = this.RegisterChildElement<BarDirectionElement>(OpenXmlElementNames.BarDirectionElementName);
            this.grouping = this.RegisterChildElement<BarGroupingElement>(OpenXmlElementNames.GroupingElementName, OpenXmlUniquePoolIds.BarGroupingUniquePoolId);
            this.overlap = this.RegisterChildElement<OverlapElement>(OpenXmlElementNames.OverlapElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.BarChartElementName;
            }
        }

        public BarDirectionElement BarDirectionElement
        {
            get
            {
                return this.barDirection.Element;
            }
            set
            {
                this.barDirection.Element = value;
            }
        }

        public BarGroupingElement GroupingElement
        {
            get
            {
                return this.grouping.Element;
            }
            set
            {
                this.grouping.Element = value;
            }
        }

        public OverlapElement OverlapElement
        {
            get
            {
                return this.overlap.Element;
            }
            set
            {
                this.overlap.Element = value;
            }
        }

        public void CopyPropertiesFrom()
        {
            this.CreateElement(this.barDirection);
            this.BarDirectionElement.Value = this.SeriesGroup.BarDirection;

            this.CreateElement(this.grouping);

            this.GroupingElement.Value = this.SeriesGroup.Grouping;

            if (this.SeriesGroup.Grouping == SeriesGrouping.Stacked ||
                this.SeriesGroup.Grouping == SeriesGrouping.PercentStacked)
            {
                this.CreateElement(this.overlap);
                this.OverlapElement.Value = OverlapElement.FullOverlap;
            }
        }

        public override void CopyPropertiesTo(IOpenXmlImportContext context, SeriesGroup seriesGroup)
        {
            base.CopyPropertiesTo(context, seriesGroup);

            BarSeriesGroup barSeriesGroup = seriesGroup as BarSeriesGroup;

            if(this.GroupingElement != null)
            {
                barSeriesGroup.Grouping = this.GroupingElement.Value;
                this.ReleaseElement(this.grouping);
            }

            if(this.BarDirectionElement != null)
            {
                barSeriesGroup.BarDirection = this.BarDirectionElement.Value;
                this.ReleaseElement(this.barDirection);
            }
        }
    }
}
