﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class ChartElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<PlotAreaElement> plotArea;
        private readonly OpenXmlChildElement<TitleElement> title;
        private readonly OpenXmlChildElement<LegendElement> legend;

        public ChartElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.plotArea = this.RegisterChildElement<PlotAreaElement>(OpenXmlElementNames.PlotAreaElementName);
            this.title = this.RegisterChildElement<TitleElement>(OpenXmlElementNames.TitleElementName);
            this.legend = this.RegisterChildElement<LegendElement>(OpenXmlElementNames.LegendElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ChartElementName;
            }
        }

        public PlotAreaElement PlotAreaElement
        {
            get
            {
                return this.plotArea.Element;
            }
            set
            {
                this.plotArea.Element = value;
            }
        }

        public TitleElement TitleElement
        {
            get
            {
                return this.title.Element;
            }
            set
            {
                this.title.Element = value;
            }
        }

        public LegendElement LegendElement
        {
            get
            {
                return this.legend.Element;
            }
            set
            {
                this.legend.Element = value;
            }
        }

        public void CopyPropertiesFrom(DocumentChart chart)
        {
            if (chart.Title != null)
            {
                this.CreateElement(this.title);
                this.TitleElement.CopyPropertiesFrom(chart.Title);
            }

            if (chart.Legend != null)
            {
                this.CreateElement(this.legend);
                this.LegendElement.CopyPropertiesFrom(chart.Legend);
            }

            this.CreateElement(this.plotArea);
            this.PlotAreaElement.CopyPropertiesFrom(chart);
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, DocumentChart chart)
        {
            if (this.TitleElement != null)
            {
                chart.Title = this.TitleElement.CreateChartTitle(context);
                this.ReleaseElement(this.title);
            }

            if (this.LegendElement != null)
            {
                chart.Legend = new Legend();
                this.LegendElement.CopyPropertiesTo(chart.Legend);
                this.ReleaseElement(this.legend);
            }

            if (this.PlotAreaElement != null)
            {
                this.PlotAreaElement.CopyPropertiesTo(context, chart);
                this.ReleaseElement(this.plotArea);
            }
        }
    }
}
