﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal abstract class ChartGroupElementBase : ChartElementBase
    {
        private readonly List<SeriesInfo> importedSeriesInfos;

        private DocumentChart chart;
        private SeriesGroup seriesGroup;

        public ChartGroupElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.importedSeriesInfos = new List<SeriesInfo>();
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public DocumentChart Chart
        {
            get
            {
                return this.chart;
            }
            set
            {
                this.chart = value;
            }
        }

        public SeriesGroup SeriesGroup
        {
            get
            {
                return this.seriesGroup;
            }
            set
            {
                this.seriesGroup = value;
            }
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IOpenXmlExportContext context)
        {
            foreach (SeriesBase series in this.SeriesGroup.Series)
            {
                int seriesIndex = this.GetSeriesIndex(series);
                SeriesElement seriesElement = this.CreateElement<SeriesElement>(OpenXmlElementNames.SeriesElementName);
                seriesElement.CopyPropertiesFrom(context, series, seriesIndex);
                yield return seriesElement;
            }
        }

        private int GetSeriesIndex(SeriesBase series)
        {
            int index = 0;

            foreach (SeriesGroup seriesGroup in this.Chart.SeriesGroups)
            {
                foreach (SeriesBase currentSeries in seriesGroup.Series)
                {
                    if (currentSeries == series)
                    {
                        return index;
                    }

                    index++;
                }
            }

            throw new InvalidOperationException("Series does not belong to this chart.");
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase childElement)
        {
            if (childElement.ElementName == OpenXmlElementNames.SeriesElementName)
            {
                SeriesElement seriesElement = childElement as SeriesElement;
                SeriesInfo seriesInfo = new SeriesInfo();
                seriesElement.CopyPropertiesTo(context, seriesInfo);
                this.importedSeriesInfos.Add(seriesInfo);
            }
        }

        public virtual void CopyPropertiesTo(IOpenXmlImportContext context, SeriesGroup seriesGroup)
        {
            foreach (SeriesInfo seriesInfo in this.importedSeriesInfos)
            {
                SeriesBase series = seriesGroup.Series.Add();

                series.Title = seriesInfo.Title;

                series.HorizontalSeriesData = seriesInfo.Categories;
                series.VerticalSeriesData = seriesInfo.Values;
                series.Outline = seriesInfo.Outline;

                BubbleSeries bubbleSeries = series as BubbleSeries;
                if (bubbleSeries != null)
                {
                    bubbleSeries.BubbleSizes = seriesInfo.BubbleSizes;
                }

                ISupportMarker supportMarkerSeries = series as ISupportMarker;
                if (supportMarkerSeries != null)
                {
                    supportMarkerSeries.Marker = seriesInfo.Marker;
                }

                ISupportSmooth supportSmoothSeries = series as ISupportSmooth;
                if (supportSmoothSeries != null)
                {
                    supportSmoothSeries.IsSmooth = seriesInfo.IsSmooth;
                }
            }
        }

        protected override void ClearOverride()
        {
            this.importedSeriesInfos.Clear();
            this.chart = null;
            this.seriesGroup = null;
        }
    }
}
