﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal abstract class ChartGroupWithAxesElementBase<T> : ChartGroupElementBase<T>
        where T : SeriesGroup, ISupportAxes
    {
        private const int ObligatoryNumberOfAxes = 2;

        private int? importedCatAxisId;
        private int? importedValAxisId;

        public ChartGroupWithAxesElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IOpenXmlExportContext context)
        {
            foreach (OpenXmlElementBase element in base.EnumerateChildElements(context))
            {
                yield return element;
            }

            AxisIdElement horizontalAxisId = this.CreateElement<AxisIdElement>(OpenXmlElementNames.AxisIdElementName);
            horizontalAxisId.Value = this.SeriesGroup.AxisGroupName == AxisGroupName.Primary ? PlotAreaElement.PrimaryCategoryAxisIndex : PlotAreaElement.SecondaryCategoryAxisIndex;
            yield return horizontalAxisId;

            AxisIdElement verticalAxisId = this.CreateElement<AxisIdElement>(OpenXmlElementNames.AxisIdElementName);
            verticalAxisId.Value = this.SeriesGroup.AxisGroupName == AxisGroupName.Primary ? PlotAreaElement.PrimaryValueAxisIndex : PlotAreaElement.SecondaryValueAxisIndex;
            yield return verticalAxisId;
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase childElement)
        {
            base.OnAfterReadChildElement(context, childElement);

            if (childElement.ElementName == OpenXmlElementNames.AxisIdElementName)
            {
                AxisIdElement axisIdElement = childElement as AxisIdElement;

                // According to the specification, the axis elements (when the group supports them)
                // are always two. While this isn't in the specification, testing with Excel shows
                // that the first element is the category axis and the second is the value axis.
                if (!this.importedCatAxisId.HasValue)
                {
                    this.importedCatAxisId = axisIdElement.Value;
                }
                else if (!this.importedValAxisId.HasValue)
                {
                    this.importedValAxisId = axisIdElement.Value;
                }
                else
                {
                    throw new InvalidOperationException("The number of axes for this seriesGroup is not valid.");
                }
            }
        }

        public override void CopyPropertiesTo(IOpenXmlImportContext context, SeriesGroup seriesGroup)
        {
            base.CopyPropertiesTo(context, seriesGroup);

            // According to the specification, the axis elements (when the group supports them) are always two.
            if (!this.importedCatAxisId.HasValue || !this.importedValAxisId.HasValue)
            {
                throw new InvalidOperationException("The number of axes for this seriesGroup is not valid.");
            }

            ISupportAxes seriesGroupWithAxes = seriesGroup as ISupportAxes;
            context.RegisterSeriesGroupAwaitingAxisGroupName(seriesGroupWithAxes, this.importedCatAxisId.Value, this.importedValAxisId.Value);
        }

        protected override void ClearOverride()
        {
            base.ClearOverride();

            this.importedCatAxisId = null;
            this.importedValAxisId = null;
        }
    }
}
