﻿using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class ChartReferenceElement : ChartElementBase
    {
        private readonly OpenXmlAttribute<string> relationId;

        public ChartReferenceElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.relationId = this.RegisterAttribute<string>("id", OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ChartReferenceElementName;
            }
        }

        public override IEnumerable<OpenXmlNamespace> Namespaces
        {
            get
            {
                yield return OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace;
            }
        }

        public string RelationshipId
        {
            get
            {
                return this.relationId.Value;
            }
            set
            {
                this.relationId.Value = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, ChartShape chart)
        {
            ChartPart chartPart = context.GetChartPartForChart(chart);
            this.RelationshipId = this.PartsManager.CreateRelationship(this.Part.Name, chartPart.Name, OpenXmlRelationshipTypes.ChartRelationshipType);
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, ChartShape chartShape)
        {
            string target = this.PartsManager.GetRelationshipTarget(this.Part.Name, this.RelationshipId);
            string chartPartName = this.Part.GetResourceName(target);

            ChartPart chartPart = this.PartsManager.GetPart<ChartPart>(chartPartName);
            context.RegisterChartPartForChart(chartShape, chartPart);
        }
    }
}
