﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class ChartSpaceElement : OpenXmlPartRootElementBase
    {
        private readonly OpenXmlChildElement<RoundedCornersElement> roundedCorners;
        private readonly OpenXmlChildElement<ChartElement> chart;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;

        public ChartSpaceElement(OpenXmlPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
            this.roundedCorners = this.RegisterChildElement<RoundedCornersElement>(OpenXmlElementNames.RoundedCornersElementName);
            this.chart = this.RegisterChildElement<ChartElement>(OpenXmlElementNames.ChartElementName, OpenXmlUniquePoolIds.ChartElementUniquePoolId);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName, OpenXmlUniquePoolIds.ChartShapePropertiesUniquePoolId);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ChartSpaceElmentName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.ChartDrawingMLNamespace;
            }
        }

        public RoundedCornersElement RoundedCornersElement
        {
            get
            {
                return this.roundedCorners.Element;
            }
            set
            {
                this.roundedCorners.Element = value;
            }
        }

        public ChartElement ChartElement
        {
            get
            {
                return this.chart.Element;
            }
            set
            {
                this.chart.Element = value;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
            set
            {
                this.shapeProperties.Element = value;
            }
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            ChartShape chartShape = context.GetChartForChartPart(this.Part as ChartPart);

            this.CreateElement(this.chart);
            this.ChartElement.CopyPropertiesFrom(chartShape.Chart);

            this.CreateElement(this.roundedCorners);
            this.RoundedCornersElement.Value = false;

            if (chartShape.Fill != null || chartShape.Outline != null)
            {
                this.CreateElement(this.shapeProperties);
                this.ShapePropertiesElement.CopyPropertiesFrom(context, chartShape);
            }
        }

        protected override void OnAfterRead(IOpenXmlImportContext context)
        {
            ChartShape chartShape = context.GetChartForChartPart(this.Part as ChartPart);

            // When there is a chart sheet there will be no chart registered as they are still not implemented and just skipped.
            if (chartShape == null)
            {
                return;
            }

            if (this.ChartElement != null)
            {
                this.ChartElement.CopyPropertiesTo(context, chartShape.Chart);
                this.ReleaseElement(this.chart);
            }

            if (this.RoundedCornersElement != null)
            {
                this.ReleaseElement(this.roundedCorners);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, chartShape);
                this.ReleaseElement(this.shapeProperties);
            }
        }
    }
}
