﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class LegendElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<LegendPositionElement> legendPosition;
        private readonly OpenXmlChildElement<OverlayElement> overlay;

        public LegendElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.legendPosition = this.RegisterChildElement<LegendPositionElement>(OpenXmlElementNames.LegendPositionElementName);
            this.overlay = this.RegisterChildElement<OverlayElement>(OpenXmlElementNames.OverlayElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.LegendElementName;
            }
        }

        public LegendPositionElement LegendPositionElement
        {
            get
            {
                return this.legendPosition.Element;
            }
            set
            {
                this.legendPosition.Element = value;
            }
        }

        public OverlayElement OverlayElement
        {
            get
            {
                return this.overlay.Element;
            }
            set
            {
                this.overlay.Element = value;
            }
        }

        public void CopyPropertiesFrom(Legend chartLegend)
        {
            this.CreateElement(this.legendPosition);
            this.LegendPositionElement.Value = chartLegend.Position;

            this.CreateElement(this.overlay);
            this.OverlayElement.Value = false;
        }

        public void CopyPropertiesTo(Legend legend)
        {
            if(this.LegendPositionElement != null)
            {
                legend.Position = this.LegendPositionElement.Value;
                this.ReleaseElement(this.legendPosition);
            }

            if(this.OverlayElement != null)
            {
                this.ReleaseElement(this.overlay);
            }
        }
    }
}
