﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class MarkerElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<SymbolElement> symbol;
        private readonly OpenXmlChildElement<SizeElement> size;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;

        public MarkerElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.symbol = this.RegisterChildElement<SymbolElement>(OpenXmlElementNames.SymbolElementName);
            this.size = this.RegisterChildElement<SizeElement>(OpenXmlElementNames.SizeElementName);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName, OpenXmlUniquePoolIds.ChartShapePropertiesUniquePoolId);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.MarkerElementName;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return false;
            }
        }

        public SymbolElement SymbolElement
        {
            get
            {
                return this.symbol.Element;
            }
        }

        public SizeElement SizeElement
        {
            get
            {
                return this.size.Element;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
        }

        internal void CopyPropertiesFrom(IOpenXmlExportContext context, Marker marker)
        {
            this.CreateElement(this.symbol);
            this.SymbolElement.Value = marker.Symbol;

            this.CreateElement(this.size);
            this.SizeElement.Value = marker.Size;

            this.CreateElement(this.shapeProperties);
            this.ShapePropertiesElement.CopyPropertiesFrom(context, marker);
        }

        internal Marker CreateMarker(IOpenXmlImportContext context)
        {
            Marker marker = new Marker();

            marker.Symbol = this.SymbolElement.Value;
            this.ReleaseElement(this.symbol);

            if (this.SizeElement != null)
            {
                marker.Size = this.SizeElement.Value;
                this.ReleaseElement(this.size);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, marker);
                this.ReleaseElement(this.shapeProperties);
            }

            return marker;
        }
    }
}
