﻿using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class NumberLiteralsElement : LiteralsDataElementBase
    {
        private readonly OpenXmlChildElement<DataPointElement> dataPoint;
        private NumericChartData data;
        private List<double> dataPoints;

        public NumberLiteralsElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.dataPoint = this.RegisterChildElement<DataPointElement>(OpenXmlElementNames.DataPointElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.NumberLiteralsElementName;
            }
        }

        public DataPointElement DataPointElement
        {
            get
            {
                return this.dataPoint.Element;
            }
            set
            {
                this.dataPoint.Element = value;
            }
        }

        protected override bool ShouldExport(IOpenXmlExportContext context)
        {
            return this.data != null;
        }

        public void CopyPropertiesFrom(NumericChartData numericChartData)
        {
            this.data = numericChartData;
        }

        protected override IEnumerable<string> EnumerateDataAsString()
        {
            foreach (double dataPoint in this.data.NumericLiterals)
            {
                yield return dataPoint.ToString();
            }
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase childElement)
        {
            base.OnAfterReadChildElement(context, childElement);

            DataPointElement dataPointElement = childElement as DataPointElement;
            if (dataPointElement != null)
            {
                if (this.dataPoints == null)
                {
                    this.dataPoints = new List<double>();
                }

                this.dataPoints.Add(double.Parse(dataPointElement.ValueElement.InnerText));
                this.ReleaseElement(this.dataPoint);
            }
        }

        public IChartData CreateLiteralChartData()
        {
            NumericChartData result = new NumericChartData(this.dataPoints);

            this.dataPoints = null;

            return result;
        }
    }
}
