﻿using System;
using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class RichTextElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<BodyPropertiesElement> bodyProperties;

        private string exportedPlainText;
        private StringBuilder importedPlainText;

        public RichTextElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.bodyProperties = this.RegisterChildElement<BodyPropertiesElement>(OpenXmlElementNames.BodyPropertiesElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.RichTextElementName;
            }
        }

        public BodyPropertiesElement BodyPropertiesElement
        {
            get
            {
                return this.bodyProperties.Element;
            }
            set
            {
                this.bodyProperties.Element = value;
            }
        }

        public void CopyPropertiesFrom(TextTitle textTitle)
        {
            this.CreateElement(this.bodyProperties);

            this.exportedPlainText = textTitle.Text;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IOpenXmlExportContext context)
        {
            ParagraphElement paragraphElement = this.CreateElement<ParagraphElement>(OpenXmlUniquePoolIds.ParagraphElementUniquePoolId);
            paragraphElement.CopyPropertiesFrom(this.exportedPlainText);
            yield return paragraphElement;
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase childElement)
        {
            if(this.importedPlainText == null)
            {
                this.importedPlainText = new StringBuilder();
            }

            if (childElement.ElementName == OpenXmlElementNames.ParagraphElementName)
            {
                ParagraphElement paragraphElement = childElement as ParagraphElement;
                string paragraphText = paragraphElement.GetPlainText();

                importedPlainText.AppendLine(paragraphText);
            }
        }

        public string GetPlainText()
        {
            this.importedPlainText.Remove(this.importedPlainText.Length - Environment.NewLine.Length, Environment.NewLine.Length);
            return this.importedPlainText.ToString();
        }

        protected override void ClearOverride()
        {
            this.importedPlainText = null;
        }
    }
}
