﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class ScalingElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<MinAxisValueElement> min;
        private readonly OpenXmlChildElement<MaxAxisValueElement> max;

        public ScalingElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.min = this.RegisterChildElement<MinAxisValueElement>(OpenXmlElementNames.MinAxisValueElementName);
            this.max = this.RegisterChildElement<MaxAxisValueElement>(OpenXmlElementNames.MaxAxisValueElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ScalingElementName;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public MinAxisValueElement MinAxisValueElement
        {
            get
            {
                return this.min.Element;
            }
            set
            {
                this.min.Element = value;
            }
        }

        public MaxAxisValueElement MaxAxisValueElement
        {
            get
            {
                return this.max.Element;
            }
            set
            {
                this.max.Element = value;
            }
        }

        public void CopyPropertiesFrom(Axis axis)
        {
            if (axis.Min.HasValue)
            {
                this.CreateElement(this.min);
                this.MinAxisValueElement.Value = axis.Min.Value;
            }

            if (axis.Max.HasValue)
            {
                this.CreateElement(this.max);
                this.MaxAxisValueElement.Value = axis.Max.Value;
            }
        }

        public void CopyPropertiesTo(Axis axis)
        {
            if (this.MinAxisValueElement != null)
            {
                axis.Min = this.MinAxisValueElement.Value;
                this.ReleaseElement(this.min);
            }

            if (this.MaxAxisValueElement != null)
            {
                axis.Max = this.MaxAxisValueElement.Value;
                this.ReleaseElement(this.max);
            }
        }
    }
}
