﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal abstract class SeriesDataElementBase : ChartElementBase
    {
        private readonly OpenXmlChildElement<StringReferenceElement> stringReference;
        private readonly OpenXmlChildElement<NumberReferenceElement> numberReference;
        private readonly OpenXmlChildElement<StringLiteralsElement> stringLiterals;
        private readonly OpenXmlChildElement<NumberLiteralsElement> numberLiterals;

        public SeriesDataElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.stringReference = this.RegisterChildElement<StringReferenceElement>(OpenXmlElementNames.StringReferenceElementName);
            this.numberReference = this.RegisterChildElement<NumberReferenceElement>(OpenXmlElementNames.NumberReferenceElementName);
            this.stringLiterals = this.RegisterChildElement<StringLiteralsElement>(OpenXmlElementNames.StringLiteralsElementName);
            this.numberLiterals = this.RegisterChildElement<NumberLiteralsElement>(OpenXmlElementNames.NumberLiteralsElementName);
        }

        public StringReferenceElement StringReferenceElement
        {
            get
            {
                return this.stringReference.Element;
            }
            set
            {
                this.stringReference.Element = value;
            }
        }

        public NumberReferenceElement NumberReferenceElement
        {
            get
            {
                return this.numberReference.Element;
            }
            set
            {
                this.numberReference.Element = value;
            }
        }

        public StringLiteralsElement StringLiteralsElement
        {
            get
            {
                return this.stringLiterals.Element;
            }
            set
            {
                this.stringLiterals.Element = value;
            }
        }

        public NumberLiteralsElement NumberLiteralsElement
        {
            get
            {
                return this.numberLiterals.Element;
            }
            set
            {
                this.numberLiterals.Element = value;
            }
        }

        public void CopyPropertiesFrom(IChartData chartData)
        {
            switch (chartData.ChartDataType)
            {
                case ChartDataType.Formula:
                    {
                        FormulaChartData formulaChartData = chartData as FormulaChartData;

                        this.CreateElement(this.numberReference);
                        this.NumberReferenceElement.CopyPropertiesFrom(formulaChartData);
                        break;
                    }

                case ChartDataType.NumericLiteral:
                    {
                        NumericChartData numericChartData = chartData as NumericChartData;

                        this.CreateElement(this.numberLiterals);
                        this.NumberLiteralsElement.CopyPropertiesFrom(numericChartData);

                        break;
                    }

                case ChartDataType.StringLiteral:
                    {
                        StringChartData stringChartData = chartData as StringChartData;

                        this.CreateElement(this.stringLiterals);
                        this.StringLiteralsElement.CopyPropertiesFrom(stringChartData);

                        break;
                    }

                default:
                    throw new NotSupportedException();
            }
        }

        public IChartData CreateChartData(IOpenXmlImportContext context)
        {
            IChartData result = null;

            if (this.NumberReferenceElement != null)
            {
                result = this.NumberReferenceElement.CreateFormulaChartData(context);
                this.ReleaseElement(this.numberReference);
            }
            else if (this.StringReferenceElement != null)
            {
                result = this.StringReferenceElement.CreateFormulaChartData(context);
                this.ReleaseElement(this.stringReference);
            }
            else if (this.NumberLiteralsElement != null)
            {
                result = this.NumberLiteralsElement.CreateLiteralChartData();
                this.ReleaseElement(this.numberLiterals);
            }
            else if (this.StringLiteralsElement != null)
            {
                result = this.StringLiteralsElement.CreateLiteralChartData();
                this.ReleaseElement(this.stringLiterals);
            }

            return result;
        }
    }
}
