﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class SeriesElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<SeriesIndexElement> index;
        private readonly OpenXmlChildElement<SeriesOrderElement> order;
        private readonly OpenXmlChildElement<SeriesCategoriesElement> categories;
        private readonly OpenXmlChildElement<SeriesValuesElement> values;
        private readonly OpenXmlChildElement<SeriesXValuesElement> xValues;
        private readonly OpenXmlChildElement<SeriesYValuesElement> yValues;
        private readonly OpenXmlChildElement<SeriesBubbleSizeElement> bubbleSize;
        private readonly OpenXmlChildElement<TxElement> title;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;
        private readonly OpenXmlChildElement<MarkerElement> marker;
        private readonly OpenXmlChildElement<SmoothElement> smooth;

        public SeriesElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.index = this.RegisterChildElement<SeriesIndexElement>(OpenXmlElementNames.SeriesIndexElementName);
            this.order = this.RegisterChildElement<SeriesOrderElement>(OpenXmlElementNames.SeriesOrderElementName);
            this.title = this.RegisterChildElement<TxElement>(OpenXmlElementNames.TxElementName, OpenXmlUniquePoolIds.SeriesTextElementUniquePoolId);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName, OpenXmlUniquePoolIds.ChartShapePropertiesUniquePoolId);
            this.marker = this.RegisterChildElement<MarkerElement>(OpenXmlElementNames.MarkerElementName);
            this.categories = this.RegisterChildElement<SeriesCategoriesElement>(OpenXmlElementNames.SeriesCategoriesElementName);
            this.values = this.RegisterChildElement<SeriesValuesElement>(OpenXmlElementNames.SeriesValuesElementName);
            this.xValues = this.RegisterChildElement<SeriesXValuesElement>(OpenXmlElementNames.SeriesXValuesElementName);
            this.yValues = this.RegisterChildElement<SeriesYValuesElement>(OpenXmlElementNames.SeriesYValuesElementName);
            this.bubbleSize = this.RegisterChildElement<SeriesBubbleSizeElement>(OpenXmlElementNames.SeriesBubbleSizeElementName);
            this.smooth = this.RegisterChildElement<SmoothElement>(OpenXmlElementNames.SmoothElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.SeriesElementName;
            }
        }

        public SeriesIndexElement Index
        {
            get
            {
                return this.index.Element;
            }
            set
            {
                this.index.Element = value;
            }
        }

        public SeriesOrderElement Order
        {
            get
            {
                return this.order.Element;
            }
            set
            {
                this.order.Element = value;
            }
        }

        public SeriesCategoriesElement SeriesCategoriesElement
        {
            get
            {
                return this.categories.Element;
            }
            set
            {
                this.categories.Element = value;
            }
        }

        public SeriesValuesElement SeriesValuesElement
        {
            get
            {
                return this.values.Element;
            }
            set
            {
                this.values.Element = value;
            }
        }

        public SeriesXValuesElement SeriesXValuesElement
        {
            get
            {
                return this.xValues.Element;
            }
            set
            {
                this.xValues.Element = value;
            }
        }

        public SeriesYValuesElement SeriesYValuesElement
        {
            get
            {
                return this.yValues.Element;
            }
            set
            {
                this.yValues.Element = value;
            }
        }

        public SeriesBubbleSizeElement SeriesBubbleSizeElement
        {
            get
            {
                return this.bubbleSize.Element;
            }
            set
            {
                this.bubbleSize.Element = value;
            }
        }

        public TxElement TxElement
        {
            get
            {
                return this.title.Element;
            }
            set
            {
                this.title.Element = value;
            }
        }

        public SmoothElement SmoothElement
        {
            get
            {
                return this.smooth.Element;
            }
            set
            {
                this.smooth.Element = value;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
            set
            {
                this.shapeProperties.Element = value;
            }
        }

        public MarkerElement MarkerElement
        {
            get
            {
                return this.marker.Element;
            }
            set
            {
                this.marker.Element = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, SeriesBase series, int index)
        {
            this.CreateElement(this.index);
            this.Index.Value = index;

            this.CreateElement(this.order);
            this.Order.Value = index;

            this.CreateElement(this.shapeProperties);
            this.ShapePropertiesElement.CopyPropertiesFrom(context, series);

            if (series.Title != null)
            {
                this.CreateElement(this.title);
                this.TxElement.CopyPropertiesFrom(series.Title);
            }

            ISupportMarker supportMarkerSeries = series as ISupportMarker;
            if (supportMarkerSeries != null && supportMarkerSeries.Marker != null)
            {
                this.CreateElement(this.marker);
                this.MarkerElement.CopyPropertiesFrom(context, supportMarkerSeries.Marker);
            }

            this.CopyPropertiesFromCategorySeries(series);
            this.CopyPropertiesFromPointSeries(series);
            this.CopyPropertiesFromBubbleSeries(series);

            ISupportSmooth supportSmoothSeries = series as ISupportSmooth;
            if (supportSmoothSeries != null)
            {
                this.CreateElement(this.smooth);
                this.SmoothElement.Value = supportSmoothSeries.IsSmooth;
            }
        }

        private void CopyPropertiesFromCategorySeries(SeriesBase series)
        {
            if (!(series is CategorySeriesBase))
            {
                return;
            }

            if (series.HorizontalSeriesData != null)
            {
                this.CreateElement(this.categories);
                this.SeriesCategoriesElement.CopyPropertiesFrom(series.HorizontalSeriesData);
            }

            if (series.VerticalSeriesData != null)
            {
                this.CreateElement(this.values);
                this.SeriesValuesElement.CopyPropertiesFrom(series.VerticalSeriesData);
            }
        }

        private void CopyPropertiesFromPointSeries(SeriesBase series)
        {
            if (!(series is PointSeriesBase))
            {
                return;
            }

            if (series.HorizontalSeriesData != null)
            {
                this.CreateElement(this.xValues);
                this.SeriesXValuesElement.CopyPropertiesFrom(series.HorizontalSeriesData);
            }

            if (series.VerticalSeriesData != null)
            {
                this.CreateElement(this.yValues);
                this.SeriesYValuesElement.CopyPropertiesFrom(series.VerticalSeriesData);
            }
        }

        private void CopyPropertiesFromBubbleSeries(SeriesBase series)
        {
            BubbleSeries bubbleSeries = series as BubbleSeries;
            if (bubbleSeries == null)
            {
                return;
            }

            if (bubbleSeries.BubbleSizes != null)
            {
                this.CreateElement(this.bubbleSize);
                this.SeriesBubbleSizeElement.CopyPropertiesFrom(bubbleSeries.BubbleSizes);
            }
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, SeriesInfo seriesInfo)
        {
            if (this.Order != null)
            {
                this.ReleaseElement(this.order);
            }

            if (this.Index != null)
            {
                this.ReleaseElement(this.index);
            }

            if (this.TxElement != null)
            {
                Title title = this.TxElement.CreateChartTitle(context);
                seriesInfo.Title = title;
                this.ReleaseElement(this.title);
            }

            if (this.SeriesCategoriesElement != null)
            {
                IChartData categoriesData = this.SeriesCategoriesElement.CreateChartData(context);
                seriesInfo.Categories = categoriesData;
                this.ReleaseElement(this.categories);
            }

            if (this.SeriesValuesElement != null)
            {
                IChartData valuesData = this.SeriesValuesElement.CreateChartData(context);
                seriesInfo.Values = valuesData;
                this.ReleaseElement(this.values);
            }

            if (this.MarkerElement != null)
            {
                seriesInfo.Marker = this.MarkerElement.CreateMarker(context);
                this.ReleaseElement(this.marker);
            }

            if (this.SmoothElement != null)
            {
                seriesInfo.IsSmooth = this.SmoothElement.Value;
                this.ReleaseElement(this.smooth);
            }

            if (this.SeriesXValuesElement != null)
            {
                seriesInfo.Categories = this.SeriesXValuesElement.CreateChartData(context);
                this.ReleaseElement(this.xValues);
            }

            if (this.SeriesYValuesElement != null)
            {
                seriesInfo.Values = this.SeriesYValuesElement.CreateChartData(context);
                this.ReleaseElement(this.yValues);
            }

            if (this.SeriesBubbleSizeElement != null)
            {
                seriesInfo.BubbleSizes = this.SeriesBubbleSizeElement.CreateChartData(context);
                this.ReleaseElement(this.bubbleSize);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, seriesInfo);
                this.ReleaseElement(this.shapeProperties);
            }
        }
    }
}
