﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal class TitleElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<ChartTextElement> chartTextElement;
        private readonly OpenXmlChildElement<OverlayElement> overlay;

        public TitleElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.chartTextElement = this.RegisterChildElement<ChartTextElement>(OpenXmlElementNames.TxElementName, OpenXmlUniquePoolIds.ChartTextElementUniquePoolId);
            this.overlay = this.RegisterChildElement<OverlayElement>(OpenXmlElementNames.OverlayElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.TitleElementName;
            }
        }

        public ChartTextElement ChartTextElement
        {
            get
            {
                return this.chartTextElement.Element;
            }
            set
            {
                this.chartTextElement.Element = value;
            }
        }

        public OverlayElement OverlayElement
        {
            get
            {
                return this.overlay.Element;
            }
            set
            {
                this.overlay.Element = value;
            }
        }

        public void CopyPropertiesFrom(Title chartTitle)
        {
            this.CreateElement(this.chartTextElement);
            this.ChartTextElement.CopyPropertiesFrom(chartTitle);

            this.CreateElement(this.overlay);
            this.OverlayElement.Value = false;
        }

        public Title CreateChartTitle(IOpenXmlImportContext context)
        {
            Title result = null;

            if (this.ChartTextElement != null)
            {
                result = this.ChartTextElement.CreateChartTitle(context);
                this.ReleaseElement(this.chartTextElement);
            }

            if (this.OverlayElement != null)
            {
                this.ReleaseElement(this.overlay);
            }

            return result;
        }
    }
}
