﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart
{
    internal abstract class TxElement : ChartElementBase
    {
        private readonly OpenXmlChildElement<StringReferenceElement> stringReference;

        public TxElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.stringReference = this.RegisterChildElement<StringReferenceElement>(OpenXmlElementNames.StringReferenceElementName);
        }

        public StringReferenceElement StringReferenceElement
        {
            get
            {
                return this.stringReference.Element;
            }
            set
            {
                this.stringReference.Element = value;
            }
        }

        public void CopyPropertiesFrom(Title title)
        {
            if (title.TitleType == TitleType.Text)
            {
                this.CopyPropertiesFromTextTitle(title as TextTitle);
            }
            else if (title.TitleType == TitleType.Formula)
            {
                this.CreateElement(this.stringReference);
                this.StringReferenceElement.CopyPropertiesFrom((title as FormulaTitle).Formula);
            }
            else
            {
                throw new NotSupportedException();
            }
        }

        protected abstract void CopyPropertiesFromTextTitle(TextTitle title);

        public Title CreateChartTitle(IOpenXmlImportContext context)
        {
            Title result = null;

            if(this.StringReferenceElement != null)
            {
                FormulaChartData chartData = this.StringReferenceElement.CreateFormulaChartData(context);
                result = new FormulaTitle(chartData);
                this.ReleaseElement(this.stringReference);
            }
            else
            {
                result = this.CreateTextChartTitle(context);
            }

            return result;
        }

        protected abstract Title CreateTextChartTitle(IOpenXmlImportContext context);
    }
}
