﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
#if !SILVERLIGHT && !NETSTANDARD
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class GraphicFrameElement : DrawingElementBase
    {
        private readonly OpenXmlNamespace ns;

        private readonly OpenXmlChildElement<NonVisualGraphicFramePropertiesElement> nonVisualDrawingProperties;
        private readonly OpenXmlChildElement<TransformElement> transform;
        private readonly OpenXmlChildElement<GraphicElement> graphic;

        public GraphicFrameElement(OpenXmlPartsManager partsManager, OpenXmlNamespace ns)
            : base(partsManager)
        {
            this.ns = ns;

            this.nonVisualDrawingProperties = this.RegisterChildElement<NonVisualGraphicFramePropertiesElement>(OpenXmlElementNames.NonVisualGraphicFramePropertiesElementName);
            this.transform = this.RegisterChildElement<TransformElement>(OpenXmlElementNames.TransformElementName, OpenXmlUniquePoolIds.SpreadsheetDrawingTransformElementUniquePoolId);
            this.graphic = this.RegisterChildElement<GraphicElement>(OpenXmlElementNames.GraphicElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.GraphicFrame;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public NonVisualGraphicFramePropertiesElement NonVisualDrawingPropertiesElement
        {
            get
            {
                return this.nonVisualDrawingProperties.Element;
            }
            set
            {
                this.nonVisualDrawingProperties.Element = value;
            }
        }

        public TransformElement TransformElement
        {
            get
            {
                return this.transform.Element;
            }
            set
            {
                this.transform.Element = value;
            }
        }

        public GraphicElement GraphicElement
        {
            get
            {
                return this.graphic.Element;
            }
            set
            {
                this.graphic.Element = value;
            }
        }

#if !SILVERLIGHT && !NETSTANDARD
        public void CopyPropertiesFrom(IOpenXmlExportContext context, ChartShape chartShape)
        {
            this.CreateElement(this.nonVisualDrawingProperties);
            this.NonVisualDrawingPropertiesElement.CopyPropertiesFrom(chartShape);

            this.CreateElement(this.transform);
            this.TransformElement.CopyPropertiesFrom(context, chartShape);

            this.CreateElement(this.graphic);
            this.GraphicElement.CopyPropertiesFrom(context, chartShape);
        }
#endif

        public ShapeBase CreateShapeAndCopyPropertiesTo(IOpenXmlImportContext context)
        {
            ShapeBase result = null;

            if (this.GraphicElement != null)
            {
                result = this.GraphicElement.CreateShapeAndCopyPropertiesTo(context);
                this.ReleaseElement(this.graphic);
            }

            if (this.NonVisualDrawingPropertiesElement != null && result != null)
            {
                this.NonVisualDrawingPropertiesElement.CopyPropertiesTo(result);
                this.ReleaseElement(this.nonVisualDrawingProperties);
            }

            if (this.TransformElement != null && result != null)
            {
                this.TransformElement.CopyPropertiesTo(context, result);
                this.ReleaseElement(this.transform);
            }

            return result;
        }
    }
}
