﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class NonVisualGraphicFramePropertiesElement : DrawingElementBase
    {
        private readonly OpenXmlNamespace ns;
        private readonly OpenXmlChildElement<NonVisualDrawingPropertiesElement> nonVisualDrawingProperties;
        private readonly OpenXmlChildElement<NonVisualGraphicFrameDrawingPropertiesElement> nonVisualGraphicFrameDrawingProperties;

        public NonVisualGraphicFramePropertiesElement(OpenXmlPartsManager partsManager, OpenXmlNamespace ns)
            : base(partsManager)
        {
            this.ns = ns;

            this.nonVisualDrawingProperties = this.RegisterChildElement<NonVisualDrawingPropertiesElement>(OpenXmlElementNames.NonVisualDrawingPropertiesElementName);
            this.nonVisualGraphicFrameDrawingProperties = this.RegisterChildElement<NonVisualGraphicFrameDrawingPropertiesElement>(OpenXmlElementNames.NonVisualGraphicFrameDrawingPropertiesElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.NonVisualGraphicFramePropertiesElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public NonVisualDrawingPropertiesElement NonVisualDrawingPropertiesElement
        {
            get
            {
                return this.nonVisualDrawingProperties.Element;
            }
            set
            {
                this.nonVisualDrawingProperties.Element = value;
            }
        }

        public NonVisualGraphicFrameDrawingPropertiesElement NonVisualGraphicFrameDrawingPropertiesElement
        {
            get
            {
                return this.nonVisualGraphicFrameDrawingProperties.Element;
            }
            set
            {
                this.nonVisualGraphicFrameDrawingProperties.Element = value;
            }
        }

#if !SILVERLIGHT
        public void CopyPropertiesFrom(ShapeBase shape)
        {
            this.CreateElement(this.nonVisualDrawingProperties);
            this.NonVisualDrawingPropertiesElement.Id = shape.Id;
            this.NonVisualDrawingPropertiesElement.Name = shape.Name;

            this.CreateElement(this.nonVisualGraphicFrameDrawingProperties);
        }
#endif

        public void CopyPropertiesTo(ShapeBase shape)
        {
            if (this.NonVisualDrawingPropertiesElement != null)
            {
                shape.Id = this.NonVisualDrawingPropertiesElement.Id;
                shape.Name = this.NonVisualDrawingPropertiesElement.Name;
                this.ReleaseElement(this.nonVisualDrawingProperties);
            }
        }
    }
}
