﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common
{
    internal class NonVisualPictureDrawingPropertiesElement : OpenXmlElementBase
    {
        private readonly OpenXmlNamespace ns;
        private readonly OpenXmlChildElement<PictureLockingElement> pictureLocking;
        private readonly IntOpenXmlAttribute preferRelativeResize;

        public NonVisualPictureDrawingPropertiesElement(OpenXmlPartsManager partsManager, OpenXmlNamespace ns)
            : base(partsManager)
        {
            Guard.ThrowExceptionIfNull(ns, "ns");

            this.ns = ns;
            this.pictureLocking = this.RegisterChildElement<PictureLockingElement>(OpenXmlElementNames.PictureLocksElementName);
            this.preferRelativeResize = this.RegisterAttribute(new IntOpenXmlAttribute("preferRelativeResize"));
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.NonVisualPictureDrawingPropertiesElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public PictureLockingElement PictureLockingElement
        {
            get
            {
                return this.pictureLocking.Element;
            }
            set
            {
                this.pictureLocking.Element = value;
            }
        }

        public int PreferRelativeResize
        {
            get
            {
                if (this.preferRelativeResize.HasValue)
                {
                    return this.preferRelativeResize.Value;
                }
                else
                {
                    return 1;
                }
            }
            set
            {
                this.preferRelativeResize.Value = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.pictureLocking);

            if (!image.PreferRelativeToOriginalResize)
            {
                this.PreferRelativeResize = 0;
            }

            this.PictureLockingElement.NoChangeAspect = image.LockAspectRatio ? 1 : 0;
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            image.PreferRelativeToOriginalResize = this.PreferRelativeResize == 0 ? false : true;

            if (this.PictureLockingElement != null)
            {
                image.LockAspectRatio = this.PictureLockingElement.NoChangeAspect == 1 ? true : false;
            }
            else
            {
                image.LockAspectRatio = false;
            }

            this.ReleaseElement(this.pictureLocking);
        }
    }
}
