﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common
{
    internal class NonVisualPicturePropertiesElement : OpenXmlElementBase
    {
        private readonly OpenXmlNamespace ns;

        private readonly OpenXmlChildElement<NonVisualDrawingPropertiesElement> nonVisualDrawingPropertiesElement;
        private readonly OpenXmlChildElement<NonVisualPictureDrawingPropertiesElement> nonVisualPictureDrawingPropertiesElement;

        public NonVisualPicturePropertiesElement(OpenXmlPartsManager partsManager, OpenXmlNamespace ns)
            : base(partsManager)
        {
            Guard.ThrowExceptionIfNull(ns, "ns");

            this.ns = ns;

            this.nonVisualDrawingPropertiesElement = this.RegisterChildElement<NonVisualDrawingPropertiesElement>(OpenXmlElementNames.NonVisualDrawingPropertiesElementName);
            this.nonVisualPictureDrawingPropertiesElement = this.RegisterChildElement<NonVisualPictureDrawingPropertiesElement>(OpenXmlElementNames.NonVisualPictureDrawingPropertiesElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.NonVisualPicturePropertiesElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public NonVisualPictureDrawingPropertiesElement NonVisualPictureDrawingPropertiesElement
        {
            get
            {
                return this.nonVisualPictureDrawingPropertiesElement.Element;
            }
            set
            {
                this.nonVisualPictureDrawingPropertiesElement.Element = value;
            }
        }

        public NonVisualDrawingPropertiesElement NonVisualDrawingPropertiesElement
        {
            get
            {
                return this.nonVisualDrawingPropertiesElement.Element;
            }
            set
            {
                this.nonVisualDrawingPropertiesElement.Element = value;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.nonVisualPictureDrawingPropertiesElement);
            this.CreateElement(this.nonVisualDrawingPropertiesElement);

            this.NonVisualDrawingPropertiesElement.Id = image.Id;
            this.NonVisualDrawingPropertiesElement.Name = image.Name;

            this.NonVisualPictureDrawingPropertiesElement.CopyPropertiesFrom(context, image);
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            image.Name = this.NonVisualDrawingPropertiesElement.Name;
            this.NonVisualPictureDrawingPropertiesElement.CopyPropertiesTo(context, image);

            this.ReleaseElement(this.nonVisualPictureDrawingPropertiesElement);
            this.ReleaseElement(this.nonVisualDrawingPropertiesElement);
        }
    }
}
