﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common
{
    internal class OffsetElement : OpenXmlElementBase
    {
        private readonly IntOpenXmlAttribute x;
        private readonly IntOpenXmlAttribute y;

        private readonly string elementName;
        private readonly OpenXmlNamespace ns;

        public OffsetElement(OpenXmlPartsManager partsManager, string elementName, OpenXmlNamespace ns)
            : base(partsManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(ns, "ns");

            this.elementName = elementName;
            this.ns = ns;

            this.x = this.RegisterAttribute(new IntOpenXmlAttribute("x", true));
            this.y = this.RegisterAttribute(new IntOpenXmlAttribute("y", true));
        }

        public override string ElementName
        {
            get
            {
                return this.elementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public int X
        {
            get
            {
                return this.x.Value;
            }
            set
            {
                this.x.Value = value;
            }
        }

        public int Y
        {
            get
            {
                return this.y.Value;
            }
            set
            {
                this.y.Value = value;
            }
        }
    }
}
