﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class SizeElement : OpenXmlElementBase
    {
        private readonly string elementName;
        private readonly OpenXmlNamespace ns;

        private readonly ConvertedOpenXmlAttribute<double> extentHeight;
        private readonly ConvertedOpenXmlAttribute<double> extentWidth;

        public SizeElement(OpenXmlPartsManager partsManager, string elementName, OpenXmlNamespace ns)
            : base(partsManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(ns, "ns");

            this.elementName = elementName;
            this.ns = ns;

            this.extentHeight = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("cy", Converters.EmuOrUniversalMeasureToDipConverter, true));
            this.extentWidth = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("cx", Converters.EmuOrUniversalMeasureToDipConverter, true));
        }

        public override string ElementName
        {
            get
            {
                return this.elementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public double ExtentWidth
        {
            get
            {
                return this.extentWidth.Value;
            }
            set
            {
                this.extentWidth.Value = value;
            }
        }

        public double ExtentHeight
        {
            get
            {
                return this.extentHeight.Value;
            }
            set
            {
                this.extentHeight.Value = value;
            }
        }
    }
}
