﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Pictures;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;
#if !SILVERLIGHT && !NETSTANDARD
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class GraphicDataElement : DrawingElementBase
    {
        private readonly OpenXmlAttribute<string> uri;

        private readonly OpenXmlChildElement<PictureElement> picture;
#if !SILVERLIGHT && !NETSTANDARD
        private readonly OpenXmlChildElement<ChartReferenceElement> chartReference;
#endif

        public GraphicDataElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.uri = this.RegisterAttribute<string>("uri", true);

            this.picture = this.RegisterChildElement<PictureElement>(OpenXmlElementNames.PictureElementName);
#if !SILVERLIGHT && !NETSTANDARD
            this.chartReference = this.RegisterChildElement<ChartReferenceElement>(OpenXmlElementNames.ChartReferenceElementName, OpenXmlUniquePoolIds.ChartReferenceElementUniquePoolId);
#endif
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.GraphicDataElementName;
            }
        }

        public string Uri
        {
            get
            {
                return this.uri.Value;
            }
            set
            {
                this.uri.Value = value;
            }
        }

        public PictureElement PictureElement
        {
            get
            {
                return this.picture.Element;
            }
        }

#if !SILVERLIGHT && !NETSTANDARD
        public ChartReferenceElement ChartReferenceElement
        {
            get
            {
                return this.chartReference.Element;
            }
        }
#endif

        public void CopyPropertiesFrom(IOpenXmlExportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            Image image = shape as Image;
            if (image != null)
            {
                this.CreateElement(this.picture);
                this.Uri = this.PictureElement.Namespace.Value;
                this.PictureElement.CopyPropertiesFrom(context, image);
            }

#if !SILVERLIGHT && !NETSTANDARD
            ChartShape chartShape = shape as ChartShape;
            if (chartShape != null)
            {
                this.CreateElement(this.chartReference);
                this.Uri = this.ChartReferenceElement.Namespace.Value;
                this.ChartReferenceElement.CopyPropertiesFrom(context, chartShape);
            }
#endif
        }

        public ShapeBase CreateShapeAndCopyPropertiesTo(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ShapeBase result = null;

            if (this.PictureElement != null)
            {
                Image image = new Image();

                this.PictureElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.picture);

                result = image;
            }

#if !SILVERLIGHT && !NETSTANDARD
            if (this.ChartReferenceElement != null && context.SupportCharts)
            {
                ChartShape chartShape = new ChartShape();
                chartShape.Chart = new DocumentChart();
                
                this.ChartReferenceElement.CopyPropertiesTo(context, chartShape);
                this.ReleaseElement(this.chartReference);

                result = chartShape;
            }
#endif

            return result;
        }
    }
}
