﻿using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class ParagraphElement : DrawingElementBase
    {
        private string exportedPlainText;
        private StringBuilder importedPlainText;

        private readonly OpenXmlChildElement<RichTextRunElement> run;

        public ParagraphElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.run = this.RegisterChildElement<RichTextRunElement>(OpenXmlElementNames.RichTextRunElementName, OpenXmlUniquePoolIds.RichTextRunElementUniquePoolId);

        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ParagraphElementName;
            }
        }

        protected override bool ShouldExport(IOpenXmlExportContext context)
        {
            return this.exportedPlainText != null;
        }

        public RichTextRunElement TextRunElement
        {
            get
            {
                return this.run.Element;
            }
            set
            {
                this.run.Element = value;
            }
        }

        public void CopyPropertiesFrom(string text)
        {
            this.exportedPlainText = text;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IOpenXmlExportContext context)
        {
            RichTextRunElement runElement = this.CreateElement<RichTextRunElement>(OpenXmlUniquePoolIds.RichTextRunElementUniquePoolId);
            runElement.CopyPropertiesFrom(this.exportedPlainText);
            yield return runElement;
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase childElement)
        {
            if (this.importedPlainText == null)
            {
                this.importedPlainText = new StringBuilder();
            }

            if (childElement.ElementName == OpenXmlElementNames.RichTextRunElementName)
            {
                RichTextRunElement runElement = childElement as RichTextRunElement;
                string runText = runElement.GetPlainText();

                this.importedPlainText.Append(runText);

                this.ReleaseElement(this.run);
            }
        }

        public string GetPlainText()
        {
            return this.importedPlainText.ToString();
        }

        protected override void ClearOverride()
        {
            this.importedPlainText = null;
            this.exportedPlainText = null;
        }
    }
}
