﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing
{
    internal class StretchElement : DrawingElementBase
    {
        private readonly OpenXmlChildElement<FillRectElement> fillRect;

        public StretchElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.fillRect = this.RegisterChildElement<FillRectElement>(OpenXmlElementNames.FillRectElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.StretchElementName;
            }
        }

        public FillRectElement FillRectElement
        {
            get
            {
                return this.fillRect.Element;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            this.CreateElement(this.fillRect);
        }

        internal void CopyPropertiesTo(Import.IOpenXmlImportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            if (this.FillRectElement != null)
            {
                this.ReleaseElement(this.fillRect);
            }
        }
    }
}
