﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Pictures
{
    internal class PictureElement : PictureElementBase
    {
        private readonly OpenXmlChildElement<NonVisualPicturePropertiesElement> nonVisualPictureProperties;
        private readonly OpenXmlChildElement<PictureFillElementBase> pictureFill;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;

        public PictureElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.nonVisualPictureProperties = this.RegisterChildElement<NonVisualPicturePropertiesElement>(OpenXmlElementNames.NonVisualPicturePropertiesElementName);
            this.pictureFill = this.RegisterChildElement<PictureFillElementBase>(OpenXmlElementNames.PictureFillElementName);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName, OpenXmlUniquePoolIds.DrawingShapePropertiesUniquePoolId);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.PictureElementName;
            }
        }

        public NonVisualPicturePropertiesElement NonVisualPicturePropertiesElement
        {
            get
            {
                return this.nonVisualPictureProperties.Element;
            }
            set
            {
                this.nonVisualPictureProperties.Element = value;
            }
        }

        public PictureFillElementBase PictureFillElement
        {
            get
            {
                return this.pictureFill.Element;
            }
            set
            {
                this.pictureFill.Element = value;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
            set
            {
                this.shapeProperties.Element = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.nonVisualPictureProperties);
            this.NonVisualPicturePropertiesElement.CopyPropertiesFrom(context, image);

            this.CreateElement(this.pictureFill);
            this.PictureFillElement.CopyPropertiesFrom(context, image);

            this.CreateElement(this.shapeProperties);
            this.ShapePropertiesElement.CopyPropertiesFrom(context, image);
        }

        public Image CopyPropertiesTo(IOpenXmlImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.PictureFillElement != null)
            {
                this.PictureFillElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.pictureFill);
            }

            if (this.NonVisualPicturePropertiesElement != null)
            {
                this.NonVisualPicturePropertiesElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.nonVisualPictureProperties);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.shapeProperties);
            }

            return image;
        }
    }
}
