﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Pictures
{
    internal abstract class PictureFillElementBase : PictureElementBase
    {
        private readonly OpenXmlChildElement<BlipElement> blip;
        private readonly OpenXmlChildElement<StretchElement> stretch;

        public PictureFillElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.blip = this.RegisterChildElement<BlipElement>(OpenXmlElementNames.BlipElementName);
            this.stretch = this.RegisterChildElement<StretchElement>(OpenXmlElementNames.StretchElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.PictureFillElementName;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public BlipElement BlipElement
        {
            get
            {
                return this.blip.Element;
            }
            set
            {
                this.blip.Element = value;
            }
        }

        public StretchElement StretchElement
        {
            get
            {
                return this.stretch.Element;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.blip);

            if (image.ImageSource == null)
            {
                return;
            }

            this.BlipElement.RelationshipId = this.CreateRelationshipIdFromImageSource(context, image.ImageSource);

            this.CreateElement(this.stretch);
            this.StretchElement.CopyPropertiesFrom(context, image);
        }

        public void CopyPropertiesTo(IOpenXmlImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            if (this.BlipElement != null)
            {
                image.ImageSource = this.CreateImageSourceFromRelationshipId(context, this.BlipElement.RelationshipId);

                if (image.ImageSource == null)
                {
                    return;
                }

                this.ReleaseElement(this.blip);
            }

            if (this.StretchElement != null)
            {
                this.StretchElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.stretch);
            }
        }

        protected abstract string CreateRelationshipIdFromImageSource(IOpenXmlExportContext context, ImageSource imageSource);

        protected abstract ImageSource CreateImageSourceFromRelationshipId(IOpenXmlImportContext context, string relationshipId);
    }
}
