﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Contexts;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Relationships
{
    internal class RelationshipElement : RelationshipElementBase
    {
        private readonly OpenXmlAttribute<string> targetAttribute;
        private readonly OpenXmlAttribute<string> typeAttribute;
        private readonly OpenXmlAttribute<string> idAttribute;
        private readonly OpenXmlAttribute<string> targetModeAttribute;

        public RelationshipElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.idAttribute = this.RegisterAttribute<string>("Id", true);
            this.typeAttribute = this.RegisterAttribute<string>("Type", true);
            this.targetAttribute = this.RegisterAttribute<string>("Target", true);
            this.targetModeAttribute = this.RegisterAttribute<string>("TargetMode", string.Empty, false);
        }

        public string Target
        {
            get
            {
                return this.targetAttribute.Value;
            }
            set
            {
                this.targetAttribute.Value = value;
            }
        }

        public string Type
        {
            get
            {
                return this.typeAttribute.Value;
            }
            set
            {
                this.typeAttribute.Value = value;
            }
        }

        public string Id
        {
            get
            {
                return this.idAttribute.Value;
            }
            set
            {
                this.idAttribute.Value = value;
            }
        }

        public string TargetMode
        {
            get
            {
                return this.targetModeAttribute.Value;
            }
            set
            {
                this.targetModeAttribute.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.RelationshipElementName;
            }
        }

        public void SetRelationshipInfo(RelationshipInfo relationshipInfo)
        {
            Guard.ThrowExceptionIfNull(relationshipInfo, "relationshipInfo");

            if (!string.IsNullOrEmpty(relationshipInfo.Target))
            {
                if (this.Part.Level == OpenXmlPartLevel.Level2 && relationshipInfo.TargetMode != TargetModes.External)
                {
                    string relativeTarget = OpenXmlPartsManager.GetRelationshipRelativeTarget(this.Part.Name, relationshipInfo.Target);

                    this.Target = relativeTarget;
                }
                else
                {

                    this.Target = relationshipInfo.Target;
                }
            }

            if (!string.IsNullOrEmpty(relationshipInfo.TargetMode))
            {
                this.TargetMode = relationshipInfo.TargetMode;
            }

            this.Id = relationshipInfo.Id;
            this.Type = relationshipInfo.Type;
        }

        public RelationshipInfo CreateRelationshipInfo()
        {
            RelationshipInfo res = new RelationshipInfo();
            res.Target = this.Target;
            res.Id = this.Id;
            res.TargetMode = this.TargetMode;
            res.Type = this.Type;

            return res;
        }
    }
}