﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders.OpenXml.Utilities;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Contexts;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Relationships
{
    internal class RelationshipsElement : OpenXmlPartRootElementBase
    {
        private readonly Dictionary<string, RelationshipInfo> relationships;

        public RelationshipsElement(OpenXmlPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
            this.relationships = new Dictionary<string, RelationshipInfo>();
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.RelationshipsElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.RelationshipsNamespace;
            }
        }

        public string CreateRelationship(string target, string type, string targetMode)
        {
            Guard.ThrowExceptionIfNull(target, "target");
            Guard.ThrowExceptionIfNullOrEmpty(type, "type");

            string relationshipId = OpenXmlHelper.CreateRelationshipId(this.relationships.Count + 1);
            RelationshipInfo relationship = new RelationshipInfo();
            relationship.Id = relationshipId;
            relationship.Target = target;
            relationship.TargetMode = targetMode;
            relationship.Type = type;

            this.relationships[relationshipId] = relationship;

            return relationshipId;
        }

        public string GetRelationshipId(string name)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            return this.relationships.Values.FirstOrDefault(r => name.EndsWith(PathHelper.NormalizePath(r.Target))).Id;
        }

        //public string GetRelationshipId(string target)
        //{
        //    Guard.ThrowExceptionIfNull(target, "target");

        //    return this.relationships.Values.FirstOrDefault(r => OpenXmlHelper.NormalizePartName(target).EndsWith(OpenXmlHelper.NormalizePartName(r.Target))).Id;
        //}

        public string GetRelationshipTarget(string id)
        {
            Guard.ThrowExceptionIfNull(id, "id");

            RelationshipInfo relationship;
            if (this.relationships.TryGetValue(id, out relationship))
            {
                return relationship.Target;
            }

            return null;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (RelationshipInfo relationshipInfo in this.relationships.Values)
            {
                RelationshipElement relationshipElement = this.CreateElement<RelationshipElement>(OpenXmlElementNames.RelationshipElementName);
                relationshipElement.SetRelationshipInfo(relationshipInfo);

                yield return relationshipElement;
            }
        }

        protected override void OnAfterReadChildElement(IOpenXmlImportContext context, OpenXmlElementBase element)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            RelationshipInfo res = ((RelationshipElement)element).CreateRelationshipInfo();
            this.relationships[res.Id] = res;
        }

        protected override void ClearOverride()
        {
            this.relationships.Clear();
        }
    }
}