﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class ColorSchemeElement : ThemeElementBase
    {
        private readonly OpenXmlAttribute<string> name;

        private readonly OpenXmlChildElement<Dark1Element> dark1;
        private readonly OpenXmlChildElement<Light1Element> light1;
        private readonly OpenXmlChildElement<Dark2Element> dark2;
        private readonly OpenXmlChildElement<Light2Element> light2;
        private readonly OpenXmlChildElement<Accent1Element> accent1;
        private readonly OpenXmlChildElement<Accent2Element> accent2;
        private readonly OpenXmlChildElement<Accent3Element> accent3;
        private readonly OpenXmlChildElement<Accent4Element> accent4;
        private readonly OpenXmlChildElement<Accent5Element> accent5;
        private readonly OpenXmlChildElement<Accent6Element> accent6;
        private readonly OpenXmlChildElement<HlinkElement> hyperlink;
        private readonly OpenXmlChildElement<FollowedHyperlinkElement> followedHyperlink;

        public ColorSchemeElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.name = this.RegisterAttribute<string>("name", true);

            this.dark1 = this.RegisterChildElement<Dark1Element>(OpenXmlElementNames.Dark1ElementName);
            this.light1 = this.RegisterChildElement<Light1Element>(OpenXmlElementNames.Light1ElementName);
            this.dark2 = this.RegisterChildElement<Dark2Element>(OpenXmlElementNames.Dark2ElementName);
            this.light2 = this.RegisterChildElement<Light2Element>(OpenXmlElementNames.Light2ElementName);
            this.accent1 = this.RegisterChildElement<Accent1Element>(OpenXmlElementNames.Accent1ElementName);
            this.accent2 = this.RegisterChildElement<Accent2Element>(OpenXmlElementNames.Accent2ElementName);
            this.accent3 = this.RegisterChildElement<Accent3Element>(OpenXmlElementNames.Accent3ElementName);
            this.accent4 = this.RegisterChildElement<Accent4Element>(OpenXmlElementNames.Accent4ElementName);
            this.accent5 = this.RegisterChildElement<Accent5Element>(OpenXmlElementNames.Accent5ElementName);
            this.accent6 = this.RegisterChildElement<Accent6Element>(OpenXmlElementNames.Accent6ElementName);
            this.hyperlink = this.RegisterChildElement<HlinkElement>(OpenXmlElementNames.HlinkElementName);
            this.followedHyperlink = this.RegisterChildElement<FollowedHyperlinkElement>(OpenXmlElementNames.FollowedHyperlinkElementName);
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ColorSchemeElementName;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public Dark1Element Dark1Element
        {
            get
            {
                return this.dark1.Element;
            }
            set
            {
                this.dark1.Element = value;
            }
        }

        public Light1Element Light1Element
        {
            get
            {
                return this.light1.Element;
            }
            set
            {
                this.light1.Element = value;
            }
        }

        public Dark2Element Dark2Element
        {
            get
            {
                return this.dark2.Element;
            }
            set
            {
                this.dark2.Element = value;
            }
        }

        public Light2Element Light2Element
        {
            get
            {
                return this.light2.Element;
            }
            set
            {
                this.light2.Element = value;
            }
        }

        public Accent1Element Accent1Element
        {
            get
            {
                return this.accent1.Element;
            }
            set
            {
                this.accent1.Element = value;
            }
        }

        public Accent2Element Accent2Element
        {
            get
            {
                return this.accent2.Element;
            }
            set
            {
                this.accent2.Element = value;
            }
        }

        public Accent3Element Accent3Element
        {
            get
            {
                return this.accent3.Element;
            }
            set
            {
                this.accent3.Element = value;
            }
        }

        public Accent4Element Accent4Element
        {
            get
            {
                return this.accent4.Element;
            }
            set
            {
                this.accent4.Element = value;
            }
        }

        public Accent5Element Accent5Element
        {
            get
            {
                return this.accent5.Element;
            }
            set
            {
                this.accent5.Element = value;
            }
        }

        public Accent6Element Accent6Element
        {
            get
            {
                return this.accent6.Element;
            }
            set
            {
                this.accent6.Element = value;
            }
        }

        public HlinkElement HyperlinkElement
        {
            get
            {
                return this.hyperlink.Element;
            }
            set
            {
                this.hyperlink.Element = value;
            }
        }

        public FollowedHyperlinkElement FollowedHyperlinkElement
        {
            get
            {
                return this.followedHyperlink.Element;
            }
            set
            {
                this.followedHyperlink.Element = value;
            }
        }

        public ThemeColorScheme CreateColorScheme(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            Color text1Color = this.Dark1Element.ToColor(context);
            Color background1Color = this.Light1Element.ToColor(context);
            Color text2Color = this.Dark2Element.ToColor(context);
            Color background2Color = this.Light2Element.ToColor(context);
            Color accent1Color = this.Accent1Element.ToColor(context);
            Color accent2Color = this.Accent2Element.ToColor(context);
            Color accent3Color = this.Accent3Element.ToColor(context);
            Color accent4Color = this.Accent4Element.ToColor(context);
            Color accent5Color = this.Accent5Element.ToColor(context);
            Color accent6Color = this.Accent6Element.ToColor(context);
            Color hyperlinkColor = this.HyperlinkElement.ToColor(context);
            Color followedHyperlinkColor = this.FollowedHyperlinkElement.ToColor(context);

            this.ReleaseElement(this.dark1);
            this.ReleaseElement(this.light1);
            this.ReleaseElement(this.dark2);
            this.ReleaseElement(this.light2);
            this.ReleaseElement(this.accent1);
            this.ReleaseElement(this.accent2);
            this.ReleaseElement(this.accent3);
            this.ReleaseElement(this.accent4);
            this.ReleaseElement(this.accent5);
            this.ReleaseElement(this.accent6);
            this.ReleaseElement(this.hyperlink);
            this.ReleaseElement(this.followedHyperlink);

            return new ThemeColorScheme(this.Name,
                background1Color,
                text1Color,
                background2Color,
                text2Color,
                accent1Color,
                accent2Color,
                accent3Color,
                accent4Color,
                accent5Color,
                accent6Color,
                hyperlinkColor,
                followedHyperlinkColor);
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Name = context.Theme.ColorScheme.Name;

            this.CreateElement(this.dark1);
            this.CreateElement(this.light1);
            this.CreateElement(this.dark2);
            this.CreateElement(this.light2);
            this.CreateElement(this.accent1);
            this.CreateElement(this.accent2);
            this.CreateElement(this.accent3);
            this.CreateElement(this.accent4);
            this.CreateElement(this.accent5);
            this.CreateElement(this.accent6);
            this.CreateElement(this.hyperlink);
            this.CreateElement(this.followedHyperlink);
        }
    }
}
