﻿using System;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal abstract class ColorSchemeElementBase : ThemeElementBase
    {
        private readonly OpenXmlChildElement<RgbHexColorModelElement> rgbHexColorModel;
        private readonly OpenXmlChildElement<SystemColorElement> systemColor;

        public ColorSchemeElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.rgbHexColorModel = this.RegisterChildElement<RgbHexColorModelElement>(OpenXmlElementNames.RgbHexColorModelElementName);
            this.systemColor = this.RegisterChildElement<SystemColorElement>(OpenXmlElementNames.SystemColorElementName);
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public abstract ThemeColorType ThemeColorType { get; }

        public SystemColorElement SystemColorElement
        {
            get
            {
                return this.systemColor.Element;
            }
            set
            {
                this.systemColor.Element = value;
            }
        }

        public RgbHexColorModelElement RgbHexColorModelElement
        {
            get
            {
                return this.rgbHexColorModel.Element;
            }
            set
            {
                this.rgbHexColorModel.Element = value;
            }
        }

        public Color ToColor(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.SystemColorElement != null)
            {
                Color color = this.SystemColorElement.GetColor(context);
                this.ReleaseElement(this.systemColor);

                return color;
            }

            if (this.RgbHexColorModelElement != null)
            {
                Color color = this.RgbHexColorModelElement.GetColor();
                this.ReleaseElement(this.rgbHexColorModel);

                return color;
            }

            throw new ArgumentException("Color type not supported.");
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.rgbHexColorModel);

            this.RgbHexColorModelElement.SetColor(context.Theme.ColorScheme[this.ThemeColorType].Color);
        }
    }
}
