﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class FontSchemeElement : ThemeElementBase
    {
        private readonly OpenXmlAttribute<string> name;

        private readonly OpenXmlChildElement<MajorFontElement> majorFont;
        private readonly OpenXmlChildElement<MinorFontElement> minorFont;

        public FontSchemeElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.name = this.RegisterAttribute<string>("name", true);

            this.majorFont = this.RegisterChildElement<MajorFontElement>(OpenXmlElementNames.MajorFontElementName);
            this.minorFont = this.RegisterChildElement<MinorFontElement>(OpenXmlElementNames.MinorFontElementName);
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.FontSchemeElementName;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public MajorFontElement MajorFontElement
        {
            get
            {
                return this.majorFont.Element;
            }
            set
            {
                this.majorFont.Element = value;
            }
        }

        public MinorFontElement MinorFontElement
        {
            get
            {
                return this.minorFont.Element;
            }
            set
            {
                this.minorFont.Element = value;
            }
        }

        public ThemeFontScheme CreateFontScheme(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string latinMajorFontName = this.MajorFontElement.GetLatinFontName(context);
            string eastAsianMajorFontName = this.MajorFontElement.GetEastAsianFontName(context);
            string complexScriptMajorFontName = this.MajorFontElement.GetComplexScriptFontName(context);

            string latinMinorFontName = this.MinorFontElement.GetLatinFontName(context);
            string eastAsianMinorFontName = this.MinorFontElement.GetEastAsianFontName(context);
            string complexScriptMinorFontName = this.MinorFontElement.GetComplexScriptFontName(context);

            this.ReleaseElement(this.majorFont);
            this.ReleaseElement(this.minorFont);

            return new ThemeFontScheme(this.Name, latinMajorFontName, latinMinorFontName, eastAsianMajorFontName, eastAsianMinorFontName, complexScriptMajorFontName, complexScriptMinorFontName);
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Name = context.Theme.FontScheme.Name;

            this.CreateElement(this.majorFont);
            this.CreateElement(this.minorFont);
        }
    }
}
