﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal abstract class FontSchemeElementBase : ThemeElementBase
    {
        private readonly OpenXmlChildElement<LatinFontElement> latinFont;
        private readonly OpenXmlChildElement<EastAsianFontElement> eastAsianFont;
        private readonly OpenXmlChildElement<ComplexScriptFontElement> complexScriptFont;

        public FontSchemeElementBase(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.latinFont = this.RegisterChildElement<LatinFontElement>(OpenXmlElementNames.LatinFontElementName);
            this.eastAsianFont = this.RegisterChildElement<EastAsianFontElement>(OpenXmlElementNames.EastAsianFontElementName);
            this.complexScriptFont = this.RegisterChildElement<ComplexScriptFontElement>(OpenXmlElementNames.ComplexScriptFontElementName);
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public LatinFontElement LatinFontElement
        {
            get
            {
                return this.latinFont.Element;
            }
            set
            {
                this.latinFont.Element = value;
            }
        }

        public EastAsianFontElement EastAsianFontElement
        {
            get
            {
                return this.eastAsianFont.Element;
            }
            set
            {
                this.eastAsianFont.Element = value;
            }
        }

        public ComplexScriptFontElement ComplexScriptFontElement
        {
            get
            {
                return this.complexScriptFont.Element;
            }
            set
            {
                this.complexScriptFont.Element = value;
            }
        }

        protected abstract ThemeFontType ThemeFontType { get; }

        public string GetLatinFontName(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string fontName = this.LatinFontElement.GetFontName(context);
            this.ReleaseElement(this.latinFont);

            return fontName;
        }

        public string GetEastAsianFontName(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string fontName = this.EastAsianFontElement.GetFontName(context);
            this.ReleaseElement(this.eastAsianFont);

            return fontName;
        }

        public string GetComplexScriptFontName(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string fontName = this.ComplexScriptFontElement.GetFontName(context);
            this.ReleaseElement(this.complexScriptFont);

            return fontName;
        }

        protected override OpenXmlElementBase CreateElement(string elementName)
        {
            if (elementName == "font")
            {
                return null;
            }

            return base.CreateElement(elementName);
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.latinFont);
            this.CreateElement(this.eastAsianFont);
            this.CreateElement(this.complexScriptFont);

            this.LatinFontElement.SetFontName(context.Theme.FontScheme[this.ThemeFontType][FontLanguageType.Latin]);
            this.EastAsianFontElement.SetFontName(context.Theme.FontScheme[this.ThemeFontType][FontLanguageType.EastAsian]);
            this.ComplexScriptFontElement.SetFontName(context.Theme.FontScheme[this.ThemeFontType][FontLanguageType.ComplexScript]);
        }
    }
}
