﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class FormatSchemeElement : ThemeElementBase
    {
        private readonly OpenXmlAttribute<string> name;

        private readonly OpenXmlChildElement<FillStyleListElement> fillStyleList;
        private readonly OpenXmlChildElement<LineStyleListElement> lineStyleList;
        private readonly OpenXmlChildElement<EffectStyleListElement> effectStyleList;
        private readonly OpenXmlChildElement<BackgroundFillStyleListElement> backgroundFillStyleList;

        public FormatSchemeElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.name = this.RegisterAttribute<string>("name", false);

            this.fillStyleList = this.RegisterChildElement<FillStyleListElement>(OpenXmlElementNames.FillStyleListElementName);
            this.lineStyleList = this.RegisterChildElement<LineStyleListElement>(OpenXmlElementNames.LineStyleListElementName);
            this.effectStyleList = this.RegisterChildElement<EffectStyleListElement>(OpenXmlElementNames.EffectStyleListElementName);
            this.backgroundFillStyleList = this.RegisterChildElement<BackgroundFillStyleListElement>(OpenXmlElementNames.BackgroundFillStyleListElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.FormatSchemeElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Name = context.Theme.Name;

            this.CreateElement(this.fillStyleList);
            this.CreateElement(this.lineStyleList);
            this.CreateElement(this.effectStyleList);
            this.CreateElement(this.backgroundFillStyleList);
        }

        protected override void OnAfterRead(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.fillStyleList);
            this.ReleaseElement(this.lineStyleList);
            this.ReleaseElement(this.effectStyleList);
            this.ReleaseElement(this.backgroundFillStyleList);
        }
    }
}
