﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class OutlineElement : ThemeElementBase
    {
        private readonly IntOpenXmlAttribute width;
        private readonly OpenXmlChildElement<SolidFillElement> solidFill;
        private readonly OpenXmlChildElement<NoFillElement> noFill;

        public OutlineElement(OpenXmlPartsManager partsManager)
            : base(partsManager)
        {
            this.width = this.RegisterAttribute(new IntOpenXmlAttribute("w", false));
            this.solidFill = this.RegisterChildElement<SolidFillElement>(OpenXmlElementNames.SolidFillElementName);
            this.noFill = this.RegisterChildElement<NoFillElement>(OpenXmlElementNames.NoFillElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.OutlineElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public SolidFillElement SolidFillElement
        {
            get
            {
                return this.solidFill.Element;
            }
            set
            {
                this.solidFill.Element = value;
            }
        }

        public NoFillElement NoFillElement
        {
            get
            {
                return this.noFill.Element;
            }
            set
            {
                this.noFill.Element = value;
            }
        }

        public int Width
        {
            get
            {
                return this.width.Value;
            }
            set
            {
                this.width.Value = value;
            }
        }

        public void SetDefaultProperties()
        {
            this.CreateElement(this.solidFill);
            this.SolidFillElement.SetDefaultProperties();
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, Outline shapeOutline)
        {
            if (shapeOutline.Width.HasValue)
            {
                this.Width = (int)Unit.DipToEmu(Unit.PointToDip(shapeOutline.Width.Value));
            }

            if (shapeOutline.Fill != null)
            {
                switch (shapeOutline.Fill.ShapeFillType)
                {
                    case FillType.Solid:
                        this.CreateElement(this.solidFill);
                        this.SolidFillElement.CopyPropertiesFrom(context, shapeOutline.Fill as SolidFill);
                        break;
                    case FillType.NoFill:
                        this.CreateElement(this.noFill);
                        break;
                    default:
                        throw new NotSupportedException("This type of fill is not supported.");
                }
            }
        }

        public void CopyPropertiesTo(Outline shapeOutline)
        {
            if (this.width.HasValue)
            {
                shapeOutline.Width = Unit.DipToPoint(Unit.EmuToDip(this.Width));
            }

            if (this.SolidFillElement != null)
            {
                shapeOutline.Fill = this.SolidFillElement.CreateSolidFill();
            }
            else if (this.NoFillElement != null)
            {
                shapeOutline.Fill = new NoFill();
            }
        }
    }
}
