﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class SchemeColorElement : ThemeElementBase
    {
        private readonly OpenXmlAttribute<string> value;

        private readonly OpenXmlChildElement<LuminanceModulationElement> luminanceModulation;
        private readonly OpenXmlChildElement<LuminanceOffsetElement> luminanceOffset;
        private readonly OpenXmlChildElement<TintElement> tint;

        public SchemeColorElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.value = this.RegisterAttribute<string>("val", false);

            this.luminanceModulation = this.RegisterChildElement<LuminanceModulationElement>(OpenXmlElementNames.LuminanceModulationElementName);
            this.luminanceOffset = this.RegisterChildElement<LuminanceOffsetElement>(OpenXmlElementNames.LuminanceOffsetElementName);
            this.tint = this.RegisterChildElement<TintElement>(OpenXmlElementNames.TintElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.SchemeColorElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public string Value
        {
            get
            {
                return this.value.Value;
            }
            set
            {
                this.value.Value = value;
            }
        }

        public LuminanceModulationElement LuminanceModulationElement
        {
            get
            {
                return this.luminanceModulation.Element;
            }
            set
            {
                this.luminanceModulation.Element = value;
            }
        }

        public LuminanceOffsetElement LuminanceOffsetElement
        {
            get
            {
                return this.luminanceOffset.Element;
            }
            set
            {
                this.luminanceOffset.Element = value;
            }
        }

        public TintElement TintElement
        {
            get
            {
                return this.tint.Element;
            }
            set
            {
                this.tint.Element = value;
            }
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, ThemableColor themableColor)
        {
            // The reason why the attribute is a string attribute and not a converted attribute of ThemeColorType
            // is that we need to set the attribute directly to a string that is not represented in the ThemeColorType enum.
            // See comment in SetDefaultProperties method of SolidFIllElement.
            this.Value = Converters.ThemeColorTypeToStringConverter.ConvertToString(themableColor.ThemeColorType);

            double? tint = null;

            if (themableColor.TintAndShade.HasValue)
            {
                tint = themableColor.TintAndShade.Value;
            }
            else if (themableColor.ColorShadeType.HasValue)
            {
                tint = context.Theme.ColorScheme.GetTintAndShade(themableColor.ThemeColorType, themableColor.ColorShadeType.Value);
            }

            if (tint.HasValue)
            {
                double lumOff;
                double lumMod;
                ColorsHelper.GetLuminanceModulationAndOffsetFromTint(tint.Value, out lumMod, out lumOff);

                this.CreateElement(this.luminanceModulation);
                this.LuminanceModulationElement.Value = (int)(lumMod * 100000);

                if (lumOff != 0)
                {
                    this.CreateElement(this.luminanceOffset);
                    this.LuminanceOffsetElement.Value = (int)(lumOff * 100000);
                }
            }
        }

        public ThemableColor CreateColor()
        {
            double? tint = null;

            if (this.TintElement != null)
            {
                tint = this.TintElement.Value;
            }
            else if (this.LuminanceModulationElement != null)
            {
                int lumMod = this.LuminanceModulationElement.Value;
                int lumOff = 0;

                if (this.LuminanceOffsetElement != null)
                {
                    lumOff = this.LuminanceOffsetElement.Value;
                }

                tint = ColorsHelper.GetTintFromLuminanceModulationAndOffset((double)lumMod / 100000, (double)lumOff / 100000);
            }

            ThemeColorType themeColorType;

            try
            {
                themeColorType = Converters.ThemeColorTypeToStringConverter.ConvertFromString(this.Value);
            }
            catch (NotImplementedException)
            {
                return null;
            }

            if (tint.HasValue)
            {
                return new ThemableColor(themeColorType, tint.Value);
            }
            else
            {
                return new ThemableColor(themeColorType);
            }
        }
    }
}
