﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class SolidFillElement : ThemeElementBase
    {
        private readonly OpenXmlChildElement<SchemeColorElement> schemeColor;
        private readonly OpenXmlChildElement<RgbHexColorModelElement> rgbHexColor;

        public SolidFillElement(OpenXmlPartsManager partsManager) : base(partsManager)
        {
            this.schemeColor = this.RegisterChildElement<SchemeColorElement>(OpenXmlElementNames.SchemeColorElementName);
            this.rgbHexColor = this.RegisterChildElement<RgbHexColorModelElement>(OpenXmlElementNames.RgbHexColorModelElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.SolidFillElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public SchemeColorElement SchemeColorElement
        {
            get
            {
                return this.schemeColor.Element;
            }
            set
            {
                this.schemeColor.Element = value;
            }
        }

        public RgbHexColorModelElement RgbHexColorModelElement
        {
            get
            {
                return this.rgbHexColor.Element;
            }
            set
            {
                this.rgbHexColor.Element = value;
            }
        }

        public void SetDefaultProperties()
        {
            // "phClr" stands for "placeholder color". It is supposed to be part of the ThemeColorType enum, however, it should not be public at this point and
            // this is why it is not included. The reason it is used in this method is that it will allow for the shapes (including charts) to have color.
            // When the FillStyleList is exported with <a:noFill />, instead of <a:solidFill> <a:schemeClr val="phClr"/> </a:solidFill>, the shapes are transparent.
            // The placeholder color allows them to pick color from the currently chosen theme.

            this.CreateElement(this.schemeColor);
            this.SchemeColorElement.Value = "phClr";
        }

        public void CopyPropertiesFrom(IOpenXmlExportContext context, SolidFill solidFill)
        {
            if (solidFill.Color.IsFromTheme)
            {
                this.CreateElement(this.schemeColor);
                this.SchemeColorElement.CopyPropertiesFrom(context, solidFill.Color);
            }
            else
            {
                this.CreateElement(this.rgbHexColor);
                this.RgbHexColorModelElement.SetColor(solidFill.Color.LocalValue);
            }
        }

        public SolidFill CreateSolidFill()
        {
            if (this.SchemeColorElement != null)
            {
                ThemableColor color = this.SchemeColorElement.CreateColor();
                this.ReleaseElement(this.schemeColor);
                return new SolidFill(color);
            }
            else if (this.RgbHexColorModelElement != null)
            {
                Color color = this.RgbHexColorModelElement.GetColor();
                this.ReleaseElement(this.schemeColor);
                return new SolidFill(color);
            }

            return null;
        }
    }
}
