﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class ThemeElement : OpenXmlPartRootElementBase
    {
        private readonly OpenXmlAttribute<string> name;
        private readonly OpenXmlChildElement<ThemeElementsElement> themeElements;

        public ThemeElement(OpenXmlPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
            this.name = this.RegisterAttribute<string>("name", string.Empty);

            this.themeElements = this.RegisterChildElement<ThemeElementsElement>(OpenXmlElementNames.ThemeElementsElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ThemeElementName;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public ThemeElementsElement ThemeElementsElement
        {
            get
            {
                return this.themeElements.Element;
            }
            set
            {
                this.themeElements.Element = value;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Name = context.Theme.Name;

            this.CreateElement(this.themeElements);
        }

        protected override void OnAfterRead(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.ThemeElementsElement != null)
            {
                ThemeColorScheme colorScheme = this.ThemeElementsElement.CreateColorScheme(context);
                ThemeFontScheme fontScheme = this.ThemeElementsElement.CreateFontScheme(context);
                context.Theme = new DocumentTheme(this.Name, colorScheme, fontScheme);
            }

            this.ReleaseElement(this.themeElements);
        }
    }
}
