﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme
{
    internal class ThemeElementsElement : ThemeElementBase
    {
        private readonly OpenXmlChildElement<ColorSchemeElement> colorScheme;
        private readonly OpenXmlChildElement<FontSchemeElement> fontScheme;
        private readonly OpenXmlChildElement<FormatSchemeElement> formatScheme;

        public ThemeElementsElement(OpenXmlPartsManager partsManager)
            : base(partsManager)

        {
            this.colorScheme = this.RegisterChildElement<ColorSchemeElement>(OpenXmlElementNames.ColorSchemeElementName);
            this.fontScheme = this.RegisterChildElement<FontSchemeElement>(OpenXmlElementNames.FontSchemeElementName);
            this.formatScheme = this.RegisterChildElement<FormatSchemeElement>(OpenXmlElementNames.FormatSchemeElementName);
        }

        public override string ElementName
        {
            get
            {
                return OpenXmlElementNames.ThemeElementsElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.DrawingMLNamespace;
            }
        }

        public ColorSchemeElement ColorSchemeElement
        {
            get
            {
                return this.colorScheme.Element;
            }
            set
            {
                this.colorScheme.Element = value;
            }
        }

        public FontSchemeElement FontSchemeElement
        {
            get
            {
                return this.fontScheme.Element;
            }
            set
            {
                this.fontScheme.Element = value;
            }
        }

        public ThemeColorScheme CreateColorScheme(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ThemeColorScheme colorScheme = this.ColorSchemeElement.CreateColorScheme(context);
            this.ReleaseElement(this.colorScheme);

            return colorScheme;
        }

        public ThemeFontScheme CreateFontScheme(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ThemeFontScheme fontScheme = this.FontSchemeElement.CreateFontScheme(context);
            this.ReleaseElement(this.fontScheme);

            return fontScheme;
        }

        protected override void OnBeforeWrite(IOpenXmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.colorScheme);
            this.CreateElement(this.fontScheme);
            this.CreateElement(this.formatScheme);
        }

        protected override void OnAfterRead(IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.formatScheme);
        }
    }
}
