﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.ContentTypes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Pictures;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Relationships;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Theme;
using Telerik.Windows.Documents.Utilities;
using DrawingSizeElement = Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing.SizeElement;
#if !SILVERLIGHT && !NETSTANDARD
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Chart;
#endif

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1506:AvoidExcessiveClassCoupling")]
    internal static class OpenXmlElementsFactory
    {
        private static readonly Dictionary<string, Func<OpenXmlPartsManager, OpenXmlElementBase>> elementNameToFactoryMethod;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1506:AvoidExcessiveClassCoupling")]
        static OpenXmlElementsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<OpenXmlPartsManager, OpenXmlElementBase>>();

            RegisterFactoryMethod(OpenXmlElementNames.DefaultElementName, (pm) => new DefaultElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.OverrideElementName, (pm) => new OverrideElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.RelationshipElementName, (pm) => new RelationshipElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ThemeElementsElementName, (pm) => new ThemeElementsElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ColorSchemeElementName, (pm) => new ColorSchemeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.RgbHexColorModelElementName, (pm) => new RgbHexColorModelElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent1ElementName, (pm) => new Accent1Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent2ElementName, (pm) => new Accent2Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent3ElementName, (pm) => new Accent3Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent4ElementName, (pm) => new Accent4Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent5ElementName, (pm) => new Accent5Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Accent6ElementName, (pm) => new Accent6Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Dark1ElementName, (pm) => new Dark1Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Dark2ElementName, (pm) => new Dark2Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FollowedHyperlinkElementName, (pm) => new FollowedHyperlinkElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.HlinkElementName, (pm) => new HlinkElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Light1ElementName, (pm) => new Light1Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.Light2ElementName, (pm) => new Light2Element(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FontSchemeElementName, (pm) => new FontSchemeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MajorFontElementName, (pm) => new MajorFontElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MinorFontElementName, (pm) => new MinorFontElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LatinFontElementName, (pm) => new LatinFontElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.EastAsianFontElementName, (pm) => new EastAsianFontElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ComplexScriptFontElementName, (pm) => new ComplexScriptFontElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FormatSchemeElementName, (pm) => new FormatSchemeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.BackgroundFillStyleListElementName, (pm) => new BackgroundFillStyleListElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FillStyleListElementName, (pm) => new FillStyleListElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.NoFillElementName, (pm) => new NoFillElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.EffectStyleListElementName, (pm) => new EffectStyleListElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.EffectStyleElementName, (pm) => new EffectStyleElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.EffectContainerElementName, (pm) => new EffectContainerElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LineStyleListElementName, (pm) => new LineStyleListElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.OutlineElementName, (pm) => new OutlineElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SystemColorElementName, (pm) => new SystemColorElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.PictureElementName, (pm) => new PictureElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualPicturePropertiesElementName, (pm) => new NonVisualPicturePropertiesElement(pm, OpenXmlNamespaces.PictureDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualPictureDrawingPropertiesElementName, (pm) => new NonVisualPictureDrawingPropertiesElement(pm, OpenXmlNamespaces.PictureDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.BlipElementName, (pm) => new BlipElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.DrawingShapePropertiesUniquePoolId, (pm) => new ShapePropertiesElement(pm, OpenXmlNamespaces.PictureDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.GraphicElementName, (pm) => new GraphicElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.GraphicDataElementName, (pm) => new GraphicDataElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.GraphicFrame, (pm) => new GraphicFrameElement(pm, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualGraphicFramePropertiesElementName, (pm) => new NonVisualGraphicFramePropertiesElement(pm, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualGraphicFrameDrawingPropertiesElementName, (pm) => new NonVisualGraphicFrameDrawingPropertiesElement(pm, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualDrawingPropertiesElementName, (pm) => new NonVisualDrawingPropertiesElement(pm, OpenXmlElementNames.NonVisualDrawingPropertiesElementName, OpenXmlNamespaces.PictureDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.DrawingTransformElementUniquePoolId, (pm) => new TransformElement(pm, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.SpreadsheetDrawingTransformElementUniquePoolId, (pm) => new TransformElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.ExtentsElementName, (pm) => new DrawingSizeElement(pm, OpenXmlElementNames.ExtentsElementName, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.OffsetElementName, (pm) => new OffsetElement(pm, OpenXmlElementNames.OffsetElementName, OpenXmlNamespaces.DrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.PresetGeometryElementName, (pm) => new PresetGeometryElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FillRectElementName, (pm) => new FillRectElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.StretchElementName, (pm) => new StretchElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.PictureLocksElementName, (pm) => new PictureLockingElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SolidFillElementName, (pm) => new SolidFillElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SchemeColorElementName, (pm) => new SchemeColorElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.BodyPropertiesElementName, (pm) => new BodyPropertiesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ParagraphElementName, (pm) => new ParagraphElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.RichTextRunElementName, (pm) => new RichTextRunElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.TextElementName, (pm) => new TextElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ParagraphElementUniquePoolId, (pm) => new ParagraphElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.RichTextRunElementUniquePoolId, (pm) => new RichTextRunElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.TextElementUniquePoolId, (pm) => new TextElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LuminanceModulationElementName, (pm) => new LuminanceModulationElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LuminanceOffsetElementName, (pm) => new LuminanceOffsetElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.TintElementName, (pm) => new TintElement(pm));
#if !SILVERLIGHT && !NETSTANDARD
            RegisterFactoryMethod(OpenXmlElementNames.ChartElementName, (pm) => new ChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.PlotAreaElementName, (pm) => new PlotAreaElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.BarChartElementName, (pm) => new BarChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LineChartElementName, (pm) => new LineChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.PieChartElementName, (pm) => new PieChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.DoughnutChartElementName, (pm) => new DoughnutChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.AreaChartElementName, (pm) => new AreaChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.BarDirectionElementName, (pm) => new BarDirectionElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.AxisIdElementName, (pm) => new AxisIdElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesElementName, (pm) => new SeriesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesCategoriesElementName, (pm) => new SeriesCategoriesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesValuesElementName, (pm) => new SeriesValuesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesXValuesElementName, (pm) => new SeriesXValuesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesYValuesElementName, (pm) => new SeriesYValuesElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesBubbleSizeElementName, (pm) => new SeriesBubbleSizeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SmoothElementName, (pm) => new SmoothElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.StringReferenceElementName, (pm) => new StringReferenceElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.NumberReferenceElementName, (pm) => new NumberReferenceElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.StringLiteralsElementName, (pm) => new StringLiteralsElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.NumberLiteralsElementName, (pm) => new NumberLiteralsElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesOrderElementName, (pm) => new SeriesOrderElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SeriesIndexElementName, (pm) => new SeriesIndexElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ValueElementName, (pm) => new ValueElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ValueElementUniquePoolId, (pm) => new ValueElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.DataPointElementName, (pm) => new DataPointElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.CategoryAxisElementName, (pm) => new CategoryAxisElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ValueAxisElementName, (pm) => new ValueAxisElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.DateAxisElementName, (pm) => new DateAxisElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ScalingElementName, (pm) => new ScalingElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.AxisPositionElementName, (pm) => new AxisPositionElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.CrossesAxisElementName, (pm) => new CrossesAxisElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.DeleteElementName, (pm) => new DeleteElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.BarGroupingUniquePoolId, (pm) => new BarGroupingElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.GroupingUniquePoolId, (pm) => new GroupingElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.OverlapElementName, (pm) => new OverlapElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.RoundedCornersElementName, (pm) => new RoundedCornersElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ChartTextElementUniquePoolId, (pm) => new ChartTextElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.SeriesTextElementUniquePoolId, (pm) => new SeriesTextElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.TitleElementName, (pm) => new TitleElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.RichTextElementName, (pm) => new RichTextElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LegendElementName, (pm) => new LegendElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.LegendPositionElementName, (pm) => new LegendPositionElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.HoleSizeElementName, (pm) => new HoleSizeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.FormulaElementName, (pm) => new FormulaElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.FormulaElementUniquePoolId, (pm) => new FormulaElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ChartElementUniquePoolId, (pm) => new ChartElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ChartReferenceElementUniquePoolId, (pm) => new ChartReferenceElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.NumberFormatElementName, (pm) => new NumberFormatElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.NumberFormatElementUniquePoolId, (pm) => new NumberFormatElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.OverlayElementName, (pm) => new OverlayElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MinAxisValueElementName, (pm) => new MinAxisValueElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MaxAxisValueElementName, (pm) => new MaxAxisValueElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MajorGridlinesElementName, (pm) => new MajorGridlinesElement(pm));
            RegisterFactoryMethod(OpenXmlUniquePoolIds.ChartShapePropertiesUniquePoolId, (pm) => new ShapePropertiesElement(pm, OpenXmlNamespaces.ChartDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.ScatterChartElementName, (pm) => new ScatterChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.BubbleChartElementName, (pm) => new BubbleChartElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.MarkerElementName, (pm) => new MarkerElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SizeElementName, (pm) => new Elements.Chart.SizeElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.SymbolElementName, (pm) => new SymbolElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.ScatterStyleElementName, (pm) => new ScatterStyleElement(pm));
#endif
        }

        public static OpenXmlElementBase CreateInstance(string elementName, OpenXmlPartsManager partsManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");

            return elementNameToFactoryMethod[elementName](partsManager);
        }

        public static bool CanCreateInstance(string elementName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");

            return elementNameToFactoryMethod.ContainsKey(elementName);
        }

        private static void RegisterFactoryMethod(string elementName, Func<OpenXmlPartsManager, OpenXmlElementBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(elementName, factoryMethod);
        }
    }
}
