﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model
{
    internal class OpenXmlPartsFactory
    {
        private static readonly Dictionary<string, Func<OpenXmlPartsManager, string, OpenXmlPartBase>> elementNameToFactoryMethod;

        static OpenXmlPartsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<OpenXmlPartsManager, string, OpenXmlPartBase>>();

            RegisterFactoryMethod(OpenXmlContentTypeNames.RelationshipContentType, (pm, pn) => new RelationshipsPart(pm, pn));
            RegisterFactoryMethod(OpenXmlContentTypeNames.ThemeContentType, (pm, pn) => new ThemePart(pm, pn));
#if !SILVERLIGHT && !NETSTANDARD
            RegisterFactoryMethod(OpenXmlContentTypeNames.ChartContentType, (pm, pn) => new ChartPart(pm, pn));
#endif
        }

        public static OpenXmlPartBase CreateInstance(string partType, OpenXmlPartsManager partsManager, string partName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNullOrEmpty(partName, "partName");

            return elementNameToFactoryMethod[partType](partsManager, partName);
        }

        public static bool CanCreateInstance(string partType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");

            return elementNameToFactoryMethod.ContainsKey(partType);
        }

        private static void RegisterFactoryMethod(string partType, Func<OpenXmlPartsManager, string, OpenXmlPartBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(partType, factoryMethod);
        }
    }
}
