﻿using System.Collections.Generic;
using System.Globalization;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model
{
    internal static class OpenXmlRelationshipTypes
    {
        public static readonly string ThemeRelationshipType = @"http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme";
        public static readonly string ImageRelationshipType = @"http://schemas.openxmlformats.org/officeDocument/2006/relationships/image";
        public static readonly string ChartRelationshipType = @"http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart";

        private static readonly Dictionary<string, string> extensionToRelationshipType;

        static OpenXmlRelationshipTypes()
        {
            extensionToRelationshipType = new Dictionary<string, string>();

            RegisterRelationshipType("jpg", ImageRelationshipType);
            RegisterRelationshipType("jpeg", ImageRelationshipType);
            RegisterRelationshipType("gif", ImageRelationshipType);
            RegisterRelationshipType("tif", ImageRelationshipType);
            RegisterRelationshipType("tiff", ImageRelationshipType);
            RegisterRelationshipType("bmp", ImageRelationshipType);
            RegisterRelationshipType("png", ImageRelationshipType);
            RegisterRelationshipType("icon", ImageRelationshipType);
            RegisterRelationshipType("wmf", ImageRelationshipType);
            RegisterRelationshipType("emf", ImageRelationshipType);
        }

        public static string GetRelationshipTypeByExtension(string extension)
        {
            extension = PathExtension.StripExtension(extension);

            return extensionToRelationshipType[extension.ToLower(CultureInfo.InvariantCulture)];
        }

        private static void RegisterRelationshipType(string extension, string relationshipType)
        {
            extensionToRelationshipType[extension] = relationshipType;
        }
    }
}
