﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types
{
    internal class MarkerStyleToStringConverter : IStringConverter<MarkerStyle>
    {
        private static readonly Dictionary<string, MarkerStyle> stringToMarkerStyle = new Dictionary<string, MarkerStyle>();
        private static readonly Dictionary<MarkerStyle, string> markerStyleToString = new Dictionary<MarkerStyle, string>();

        static MarkerStyleToStringConverter()
        {
            RegisterPair("circle", MarkerStyle.Circle);
            RegisterPair("dash", MarkerStyle.Dash);
            RegisterPair("diamond", MarkerStyle.Diamond);
            RegisterPair("dot", MarkerStyle.Dot);
            RegisterPair("none", MarkerStyle.None);
            RegisterPair("plus", MarkerStyle.Plus);
            RegisterPair("square", MarkerStyle.Square);
            RegisterPair("star", MarkerStyle.Star);
            RegisterPair("triangle", MarkerStyle.Triangle);
            RegisterPair("x", MarkerStyle.X);
            RegisterPair("auto", MarkerStyle.Auto);
        }

        private static void RegisterPair(string name, MarkerStyle value)
        {
            stringToMarkerStyle.Add(name, value);
            markerStyleToString.Add(value, name);
        }

        public MarkerStyle ConvertFromString(string value)
        {
            // Not supported
            if (value == "picture")
            {
                return MarkerStyle.Circle;
            }

            return stringToMarkerStyle[value];
        }

        public string ConvertToString(MarkerStyle value)
        {
            return markerStyleToString[value];
        }
    }
}
