using System;
using System.Collections.Generic;
using System.Globalization;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types
{
    internal class UniversalMeasureToDipConverter : IStringConverter<double>
    {
        private static readonly Dictionary<string, Func<double, double>> unitToConvertMethodMapping;

        static UniversalMeasureToDipConverter()
        {
            unitToConvertMethodMapping = new Dictionary<string, Func<double, double>>();

            RegisterConvertMethod("cm", Unit.CmToDip);
            RegisterConvertMethod("mm", Unit.MmToDip);
            RegisterConvertMethod("in", Unit.InchToDip);
            RegisterConvertMethod("pt", Unit.PointToDip);
            RegisterConvertMethod("pc", Unit.PicaToDip);
            RegisterConvertMethod("pi", Unit.PicaToDip);
        }

        public static bool IsValidUnit(string unit)
        {
            return unitToConvertMethodMapping.ContainsKey(unit);
        }

        public double ConvertFromString(string value)
        {
            if (value != null && value.Length > 1)
            {
                string unit = value.Substring(value.Length - 2);

                double units;
                if (double.TryParse(value.Substring(0, value.Length - 2), NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign, CultureInfo.InvariantCulture, out units))
                {
                    return Convert(unit, units);
                }
            }

            return 0;
        }

        public string ConvertToString(double units)
        {
            throw new NotSupportedException();
        }

        private static void RegisterConvertMethod(string unit, Func<double, double> convertMethod)
        {
            unitToConvertMethodMapping[unit] = convertMethod;
        }

        private static double Convert(string unit, double units)
        {
            return unitToConvertMethodMapping[unit](units);
        }
    }
}