﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Telerik.Windows.Documents.Common.FormatProviders.OpenXml.Utilities
{
    internal static class PathHelper
    {
        public const string RelationDirectory = "_rels";

        private const string DirectoryMarker = "/";
        private const string ParentDirectoryMarker = "..";

        public static string NormalizePath(string path)
        {
            path = path.Replace("\\", DirectoryMarker);
            path = path.Replace("//", DirectoryMarker);

            return path;
        }

        public static string NormalizePathForExport(string path)
        {
            path = path.Replace("\\", DirectoryMarker);
            path = path.Replace("//", DirectoryMarker);

            if (path.StartsWith(DirectoryMarker))
            {
                path = path.Substring(1);
            }

            return path;
        }

        public static string NormalizePathFromImport(string path)
        {
            path = path.Replace("\\", DirectoryMarker);
            path = path.Replace("//", DirectoryMarker);

            if (!path.StartsWith(DirectoryMarker))
            {
                path = DirectoryMarker + path;
            }

            return path;
        }

        public static string Combine(string root, string path)
        {
            path = NormalizePath(path);
            if (path.StartsWith(DirectoryMarker))
            {
                return path;
            }
            else
            {
                root = NormalizePath(Path.GetDirectoryName(root));

                Stack<string> pathStack = new Stack<string>(root.Split(new string[] { DirectoryMarker }, StringSplitOptions.RemoveEmptyEntries));
                string[] pathParts = path.Split(new string[] { DirectoryMarker }, StringSplitOptions.RemoveEmptyEntries);

                foreach (string part in pathParts)
                {
                    if (part == ParentDirectoryMarker)
                    {
                        pathStack.Pop();
                    }
                    else
                    {
                        pathStack.Push(part);
                    }
                }

                StringBuilder result = new StringBuilder();
                while (pathStack.Count > 0)
                {
                    result.Insert(0, DirectoryMarker + pathStack.Pop());
                }

                return result.ToString();
            }
        }
    }
}
