﻿using System.IO;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Common.FormatProviders
{
    /// <summary>
    /// Represents base class for text base format provider.
    /// </summary>
    /// <typeparam name="T">The type of the T.</typeparam>
    public abstract class TextBasedFormatProviderBase<T> : FormatProviderBase<T>, ITextBasedFormatProvider<T>
    {
        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The result document.</returns>
        public T Import(string input)
        {
            Guard.ThrowExceptionIfNull(input, "input");

            using (MemoryStream inputStream = new MemoryStream())
            {
                using (StreamWriter writer = new StreamWriter(inputStream))
                {
                    writer.Write(input);
                    writer.Flush();
                    inputStream.Position = 0;

                    return this.Import(inputStream);
                }
            }
        }

        /// <summary>
        /// Exports the specified document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns>The result string.</returns>
        public string Export(T document)
        {
            Guard.ThrowExceptionIfNull(document, "document");

            using (MemoryStream outputStream = new MemoryStream())
            {
                this.Export(document, outputStream);
                outputStream.Seek(0, SeekOrigin.Begin);

                using (StreamReader reader = new StreamReader(outputStream))
                {
                    return reader.ReadToEnd();
                }
            }
        }
    }
}
