﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Namespaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Attributes
{
    internal abstract class XmlAttribute
    {
        private readonly string name;
        private readonly XmlNamespace ns;
        private readonly bool isRequired;

        public XmlAttribute(string name, XmlNamespace ns, bool isRequired = false)
        {
            this.name = name;
            this.ns = ns;
            this.isRequired = isRequired;
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public XmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        public bool IsRequired
        {
            get
            {
                return this.isRequired;
            }
        }

        public abstract bool HasValue { get; }

        public abstract string GetValue();

        public abstract void SetValue(string value);

        public abstract bool ShouldExport();

        public abstract void ResetValue();
    }

    internal class XmlAttribute<T> : XmlAttribute
    {
         private T defaultValue;
        private T value;
        private bool isSet;

        public XmlAttribute(string name, XmlNamespace ns, T defaultValue, bool isRequired = false)
            : base(name, ns, isRequired)
        {
            this.defaultValue = defaultValue;
            this.value = defaultValue;
        }

        public T DefaultValue
        {
            get
            {
                return this.defaultValue;
            }
            protected set
            {
                this.defaultValue = value;
                if (!this.isSet)
                {
                    this.ResetValue();
                }
            }
        }

        public T Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if (!ObjectExtensions.EqualsOfT(this.value, value))
                {
                    this.value = value;
                }

                this.isSet = true;
            }
        }

        public override bool HasValue
        {
            get
            {
                return this.isSet || !ObjectExtensions.EqualsOfT(this.value, this.defaultValue);
            }
        }

        public override string GetValue()
        {
            if (object.ReferenceEquals(this.Value, null))
            {
                return string.Empty;
            }

            return this.Value.ToString();
        }

        public override void SetValue(string value)
        {
            Guard.ThrowExceptionIfNull(value, "value");

            this.Value = (T)Convert.ChangeType(value, typeof(T), System.Globalization.CultureInfo.InvariantCulture);
            this.isSet = true;
        }

        public override bool ShouldExport()
        {
            return this.IsRequired || this.HasValue;
        }

        public override void ResetValue()
        {
            this.value = this.defaultValue;
            this.isSet = false;
        }
    }
}