﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.FormatProviders.OpenXml.Model
{
    internal abstract class ElementsPoolBase<TElement>
    {
        private readonly Dictionary<string, Stack<TElement>> store;
        private readonly object syncRoot;

        public ElementsPoolBase()
        {
            this.store = new Dictionary<string, Stack<TElement>>();
            this.syncRoot = new object();
        }

        public TElement CreateElement(string elementName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");

            TElement element;
            Stack<TElement> elementsStack;
            if (!this.store.TryGetValue(elementName, out elementsStack) || elementsStack.Count == 0)
            {
                element = this.CreateInstance(elementName);
            }
            else
            {
                lock (this.syncRoot)
                {
                    if (elementsStack.Count == 0)
                    {
                        element = this.CreateInstance(elementName);
                    }
                    else
                    {
                        element = elementsStack.Pop();
                    }
                }
            }

            return element;
        }

        public void ReleaseElement(string elementName, TElement element)
        {
            Guard.ThrowExceptionIfNull(element, "element");
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");

            Stack<TElement> elementsStack;
            if (!this.store.TryGetValue(elementName, out elementsStack))
            {
                lock (this.syncRoot)
                {
                    if (!this.store.TryGetValue(elementName, out elementsStack))
                    {
                        elementsStack = new Stack<TElement>();
                        this.store[elementName] = elementsStack;
                    }
                }
            }

            this.store[elementName].Push(element);
        }

        protected abstract TElement CreateInstance(string elementName);
    }
}
