﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Common.Model.Data
{
    internal class ResourceManager : IEnumerable<IResource>
    {
        private readonly Dictionary<IResource, int> resourceInstances;
        
        private int nextId;

        public ResourceManager()
        {
            this.resourceInstances = new Dictionary<IResource, int>();
            this.nextId = 1;
        }

        public void RegisterResource(IResource resource)
        {
            Guard.ThrowExceptionIfNull(resource, "resource");

            if (resource.Owner != null && resource.Owner != this)
            {
                throw new ArgumentException("The resource is associated with another owner.", "resource");
            }

            if (!this.resourceInstances.ContainsKey(resource))
            {
                this.resourceInstances[resource] = 1;
                resource.Id = this.nextId++;
                resource.Owner = this;
            }
            else
            {
                this.resourceInstances[resource]++;
            }
        }

        public void ReleaseResource(IResource resource)
        {
            Guard.ThrowExceptionIfNull(resource, "resource");

            if (this.resourceInstances.ContainsKey(resource))
            {
                int instances = --this.resourceInstances[resource];
                if (instances == 0)
                {
                    this.resourceInstances.Remove(resource);
                    resource.Owner = null;
                }
            }
        }

        public IEnumerator<IResource> GetEnumerator()
        {
            return this.resourceInstances.Keys.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.resourceInstances.Keys.GetEnumerator();
        }
    }
}
