﻿using System;
using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Common.Model.Data
{
    internal class ResourcesFactory
    {
        private static readonly Dictionary<string, Func<Stream, string, IResource>> extensionToResourceMapping;

        static ResourcesFactory()
        {
            extensionToResourceMapping = new Dictionary<string, Func<Stream, string, IResource>>();

            RegisterFactoryMethod("jpg", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("jpeg", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("png", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("bmp", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("tiff", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("tif", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("gif", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("icon", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("wmf", (stream, extension) => new ImageSource(stream, extension));
            RegisterFactoryMethod("emf", (stream, extension) => new ImageSource(stream, extension));
        }

        public static IResource CreateInstance(Stream stream, string extension)
        {
            extension = extension.ToLowerInvariant();

            return extensionToResourceMapping[extension](stream, extension);
        }

        public static bool CanCreateInstance(string extension)
        {
            Guard.ThrowExceptionIfNullOrEmpty(extension, "extension");

            extension = extension.ToLowerInvariant();
            return extensionToResourceMapping.ContainsKey(extension);
        }

        private static void RegisterFactoryMethod(string extension, Func<Stream, string, IResource> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(extension, "extension");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            extensionToResourceMapping.Add(extension, factoryMethod);
        }
    }
}
