﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
#if WPF || SILVERLIGHT
using System.Windows.Media;
#else
using Telerik.Documents.Media;
#endif

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// A converter for the ThemableColor class. 
    /// </summary>
    public class ThemableColorTypeConverter : TypeConverter
    {
        private static readonly Dictionary<string, Color> colorNameToColor;

        static ThemableColorTypeConverter()
        {
            colorNameToColor = new Dictionary<string, Color>();

            colorNameToColor.Add("Black", Colors.Black);
            colorNameToColor.Add("Blue", Colors.Blue);
            colorNameToColor.Add("Brown", Colors.Brown);
            colorNameToColor.Add("Cyan", Colors.Cyan);
            colorNameToColor.Add("DarkGray", Colors.DarkGray);
            colorNameToColor.Add("Gray", Colors.Gray);
            colorNameToColor.Add("Green", Colors.Green);
            colorNameToColor.Add("LightGray", Colors.LightGray);
            colorNameToColor.Add("Magenta", Colors.Magenta);
            colorNameToColor.Add("Orange", Colors.Orange);
            colorNameToColor.Add("Purple", Colors.Purple);
            colorNameToColor.Add("Red", Colors.Red);
            colorNameToColor.Add("Transparent", Colors.Transparent);
            colorNameToColor.Add("White", Colors.White);
            colorNameToColor.Add("Yellow", Colors.Yellow);
        }

        /// <summary>
        /// Returns whether this converter can convert an object of the given type
        /// to the type of this converter, using the specified context.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext" />
        /// that provides a format context.</param>
        /// <param name="sourceType">A <see cref="T:System.Type" /> that represents the type
        /// you want to convert from.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return (sourceType == typeof(string));
        }

        /// <summary>
        /// Returns whether this converter can convert the object to the specified
        /// type, using the specified context.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext" />
        /// that provides a format context.</param>
        /// <param name="destinationType">A <see cref="T:System.Type" /> that represents the
        /// type you want to convert to.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return (destinationType == typeof(string));
        }

        /// <summary>
        /// Converts the given object to the type of this converter, using the specified
        /// context and culture information.
        /// </summary>
        /// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext" />
        /// that provides a format context.</param>
        /// <param name="culture">The <see cref="T:System.Globalization.CultureInfo" /> to
        /// use as the current culture.</param>
        /// <param name="value">The <see cref="T:System.Object" /> to convert.</param>
        /// <exception cref="T:System.NotSupportedException">The conversion cannot be performed.
        /// </exception>
        /// <returns>
        /// An <see cref="T:System.Object" /> that represents the converted value.
        /// </returns>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            string color = (string)value;

            if (colorNameToColor.ContainsKey(color))
            {
                return new ThemableColor(colorNameToColor[color]);
            }

            color = color.Substring(1, color.Length - 1);

            UInt32 hexColor = UInt32.Parse(color, System.Globalization.NumberStyles.HexNumber);

            byte a = (byte)(hexColor >> 24);
            byte r = (byte)(hexColor >> 16);
            byte g = (byte)(hexColor >> 8);
            byte b = (byte)(hexColor);

            return new ThemableColor(Color.FromArgb(a, r, g, b));
        }
    }
}