﻿using System;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of axes in the chart.
    /// </summary>
    public class AxisGroup
    {
        private Axis categoryAxis;
        private Axis valueAxis;

        /// <summary>
        /// Initializes a new instance of the <see cref="AxisGroup" /> class.
        /// </summary>
        /// <param name="categoryAxis">The category axis.</param>
        /// <param name="valueAxis">The value axis.</param>
        public AxisGroup(Axis categoryAxis, Axis valueAxis)
        {
            this.CategoryAxis = categoryAxis;
            this.ValueAxis = valueAxis;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AxisGroup" /> class.
        /// </summary>
        public AxisGroup()
        {
            this.CategoryAxis = new CategoryAxis();
            this.ValueAxis = new ValueAxis();
        }

        /// <summary>
        /// Gets or sets the axis agains which the categories of the series are plotted.
        /// </summary>
        public Axis CategoryAxis
        {
            get
            {
                return this.categoryAxis;
            }
            set
            {
                if (this.categoryAxis != value)
                {
                    if (value == null)
                    {
                        throw new ArgumentNullException("value", "The axis group cannot have axis with value null.");
                    }

                    if (this.categoryAxis != null)
                    {
                        this.categoryAxis.AxisChanged -= this.Axis_AxisChanged;
                    }

                    this.categoryAxis = value;

                    this.categoryAxis.AxisChanged += this.Axis_AxisChanged;

                    this.OnCollectionChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the axis agains which the values of the series are plotted.
        /// </summary>
        public Axis ValueAxis
        {
            get
            {
                return this.valueAxis;
            }
            set
            {
                if (this.valueAxis != value)
                {
                    if (value == null)
                    {
                        throw new ArgumentNullException("value", "The axis group cannot have axis with value null.");
                    }

                    if (this.valueAxis != null)
                    {
                        this.valueAxis.AxisChanged -= this.Axis_AxisChanged;
                    }

                    this.valueAxis = value;

                    this.valueAxis.AxisChanged += this.Axis_AxisChanged;

                    this.OnCollectionChanged();
                }
            }
        }

        private void Axis_AxisChanged(object sender, EventArgs e)
        {
            this.OnAxesChanged();
        }

        internal event EventHandler CollectionChanged;

        private void OnCollectionChanged()
        {
            if (this.CollectionChanged != null)
            {
                this.CollectionChanged(this, EventArgs.Empty);
            }
        }

        internal event EventHandler AxesChanged;

        private void OnAxesChanged()
        {
            if (this.AxesChanged != null)
            {
                this.AxesChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public AxisGroup Clone()
        {
            return new AxisGroup(this.CategoryAxis, this.ValueAxis);
        }
    }
}
