﻿using System;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a marker for the chart series.
    /// </summary>
    public class Marker
    {
        internal static readonly byte DefaultSize = 5;
        private byte size;
        private MarkerStyle symbol;
        private Fill fill;

        /// <summary>
        /// Initializes a new instance of the <see cref="Marker" /> class.
        /// </summary>
        public Marker()
        {
            this.size = DefaultSize;
            this.Outline = new Outline();
            this.Outline.ShapeOutlineChanged += this.Outline_ShapeOutlineChanged;
        }

        /// <summary>
        /// Gets or sets the marker symbol.
        /// </summary>
        public MarkerStyle Symbol
        {
            get
            {
                return this.symbol;
            }
            set
            {
                if (this.symbol != value)
                {
                    this.symbol = value;
                    this.OnChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the marker size. The alowed values are between 2 and 72.
        /// </summary>
        public byte Size
        {
            get
            {
                return this.size;
            }
            set
            {
                if (this.size != value)
                {
                    Guard.ThrowExceptionIfLessThan(2, value, "value");
                    Guard.ThrowExceptionIfGreaterThan(72, value, "value");

                    this.size = value;
                    this.OnChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the marker fill.
        /// </summary>
        public Fill Fill
        {
            get
            {
                return this.fill;
            }
            set
            {
                if (this.fill != value)
                {
                    this.fill = value;
                    this.OnChanged();
                }
            }
        }

        /// <summary>
        /// Gets the marker outline.
        /// </summary>
        public Outline Outline { get; private set; }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public Marker Clone()
        {
            Marker newMarker = new Marker();

            newMarker.Symbol = this.Symbol;
            newMarker.Size = this.Size;
            newMarker.Fill = this.Fill.Clone();
            newMarker.Outline = this.Outline.Clone();

            return newMarker;
        }

        internal event EventHandler Changed;

        private void OnChanged()
        {
            if (this.Changed != null)
            {
                this.Changed(this, EventArgs.Empty);
            }
        }

        private void Outline_ShapeOutlineChanged(object sender, EventArgs e)
        {
            this.OnChanged();
        }
    }
}
