﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series of type area.
    /// </summary>
    public class AreaSeriesGroup : SeriesGroup<AreaSeries>, ISupportAxes, ISupportGrouping
    {
        private const SeriesType areaSeriesType = SeriesType.Area;

        private SeriesGrouping grouping;
        private AxisGroupName axisGroup;

        /// <summary>
        /// Initializes a new instance of the <see cref="AreaSeriesGroup" /> class.
        /// </summary>
        public AreaSeriesGroup()
            : base(areaSeriesType)
        {
        }

        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return areaSeriesType;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating which group of axes the series group is associated with.
        /// </summary>
        public AxisGroupName AxisGroupName
        {
            get
            {
                return this.axisGroup;
            }
            set
            {
                if (this.axisGroup != value)
                {
                    this.axisGroup = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the value indicating how the series are grouped.
        /// </summary>
        public SeriesGrouping Grouping
        {
            get
            {
                return this.grouping;
            }
            set
            {
                if (this.grouping != value)
                {
                    this.grouping = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        internal override void CloneProperties(SeriesGroup group)
        {
            base.CloneProperties(group);

            AreaSeriesGroup newSeriesGroup = (AreaSeriesGroup)group;

            newSeriesGroup.Grouping = this.Grouping;
            newSeriesGroup.AxisGroupName = this.AxisGroupName;
        }
    }
}
