﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series of type bar.
    /// </summary>
    public class BarSeriesGroup : SeriesGroup<BarSeries>, ISupportAxes, ISupportGrouping
    {
        private const SeriesType barSeriesType = SeriesType.Bar;

        private AxisGroupName axisGroup;
        private BarDirection barDirection;
        private SeriesGrouping grouping;

        /// <summary>
        /// Initializes a new instance of the <see cref="BarSeriesGroup" /> class.
        /// </summary>
        public BarSeriesGroup()
            : base(barSeriesType)
        {

        }

        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return barSeriesType;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating which group of axes the series group is associated with.
        /// </summary>
        public AxisGroupName AxisGroupName
        {
            get
            {
                return this.axisGroup;
            }
            set
            {
                if (this.axisGroup != value)
                {
                    this.axisGroup = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the value indicating how the series are grouped.
        /// </summary>
        public SeriesGrouping Grouping
        {
            get
            {
                return this.grouping;
            }
            set
            {
                if (this.grouping != value)
                {
                    this.grouping = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the direction of the bars.
        /// </summary>
        public BarDirection BarDirection
        {
            get
            {
                return this.barDirection;
            }
            set
            {
                if (this.barDirection != value)
                {
                    this.barDirection = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        internal override void CloneProperties(SeriesGroup group)
        {
            base.CloneProperties(group);

            BarSeriesGroup newSeriesGroup = (BarSeriesGroup)group;
            newSeriesGroup.BarDirection = this.BarDirection;
            newSeriesGroup.Grouping = this.Grouping;
            newSeriesGroup.AxisGroupName = this.AxisGroupName;
        }
    }
}