﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series of type bubble.
    /// </summary>
    public class BubbleSeriesGroup : SeriesGroup<BubbleSeries>, ISupportAxes
    {
        private const SeriesType seriesType = SeriesType.Bubble;

        private AxisGroupName axisGroup;

        /// <summary>
        /// Initializes a new instance of the <see cref="BubbleSeriesGroup" /> class.
        /// </summary>
        public BubbleSeriesGroup()
            : base(seriesType)
        {
        }

        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return seriesType;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating which group of axes the series group is associated with.
        /// </summary>
        public AxisGroupName AxisGroupName
        {
            get
            {
                return this.axisGroup;
            }
            set
            {
                if (this.axisGroup != value)
                {
                    this.axisGroup = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        internal override int DataRangesPerSeries
        {
            get
            {
                return 3;
            }
        }

        internal override bool HasCategories
        {
            get
            {
                return false;
            }
        }

        internal override void OnSeriesGroupChanged()
        {
            base.OnSeriesGroupChanged();
        }

        internal override void CloneProperties(SeriesGroup group)
        {
            base.CloneProperties(group);

            BubbleSeriesGroup newSeriesGroup = (BubbleSeriesGroup)group;
            newSeriesGroup.AxisGroupName = this.AxisGroupName;
        }
    }
}
