﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a base class for the category series.
    /// </summary>
    public abstract class CategorySeriesBase : SeriesBase
    {
        private IChartData values;
        private IChartData categories;

        /// <summary>
        /// Gets or sets the data for the values of the series.
        /// </summary>
        public IChartData Values
        {
            get
            {
                return this.values;
            }
            set
            {
                if (this.values != value)
                {
                    this.values = value;
                    this.OnSeriesChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the data for the categories of the series.
        /// </summary>
        public IChartData Categories
        {
            get
            {
                return this.categories;
            }
            set
            {
                if (this.categories != value)
                {
                    this.categories = value;
                    this.OnSeriesChanged();
                }
            }
        }

        internal sealed override IChartData VerticalSeriesData
        {
            get
            {
                return this.Values;
            }
            set
            {
                this.Values = value;
            }
        }

        internal sealed override IChartData HorizontalSeriesData
        {
            get
            {
                return this.Categories;
            }
            set
            {
                this.Categories = value;
            }
        }

        internal override void CloneProperties(SeriesBase series)
        {
            base.CloneProperties(series);

            CategorySeriesBase categorySeries = (CategorySeriesBase)series;

            if (this.values != null)
            {
                categorySeries.values = this.values.Clone();
            }

            if (this.categories != null)
            {
                categorySeries.categories = this.categories.Clone();
            }
        }
    }
}
