﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents formula chart data. This type of chart data holds a formula which refers to the actual data.
    /// </summary>
    public abstract class FormulaChartData : IChartData
    {
        private readonly string formula;

        /// <summary>
        /// Initializes a new instance of the <see cref="FormulaChartData" /> class.
        /// </summary>
        /// <param name="formula">The formula which refers to the chart data.</param>
        protected FormulaChartData(string formula)
        {
            this.formula = formula;
        }

        /// <summary>
        /// Gets the type of chart data.
        /// </summary>
        public ChartDataType ChartDataType
        {
            get
            {
                return ChartDataType.Formula;
            }
        }

        /// <summary>
        /// Gets the formula which refers to the chart data.
        /// </summary>
        public string Formula
        {
            get
            {
                return this.formula;
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public abstract IChartData Clone();
    }
}
