﻿using System;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a series of type line.
    /// </summary>
    public class LineSeries : CategorySeriesBase, ISupportMarker, ISupportSmooth
    {
        private Marker marker;
        private bool isSmooth;

        /// <summary>
        /// Gets the type of the series.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return SeriesType.Line;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating if the series line will be smooth.
        /// </summary>
        public bool IsSmooth
        {
            get
            {
                return this.isSmooth;
            }
            set
            {
                if (this.isSmooth != value)
                {
                    this.isSmooth = value;
                    this.OnSeriesChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the series marker.
        /// </summary>
        public Marker Marker
        {
            get
            {
                return this.marker;
            }
            set
            {
                if (this.marker != null)
                {
                    this.marker.Changed -= this.Marker_Changed;
                }

                this.marker = value;

                if (this.marker != null)
                {
                    this.marker.Changed += this.Marker_Changed;
                }

                this.OnSeriesChanged();
            }
        }

        internal override void CloneProperties(SeriesBase series)
        {
            base.CloneProperties(series);

            LineSeries lineSeries = (LineSeries)series;
            lineSeries.isSmooth = this.isSmooth;
            if (this.marker != null)
            {
                lineSeries.marker = this.marker.Clone();
            }
        }

        private void Marker_Changed(object sender, EventArgs e)
        {
            this.OnSeriesChanged();
        }
    }
}