﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series of type line.
    /// </summary>
    public class LineSeriesGroup : SeriesGroup<LineSeries>, ISupportAxes, ISupportGrouping
    {
        private const SeriesType lineSeriesType = SeriesType.Line;

        private SeriesGrouping grouping;
        private AxisGroupName axisGroup;

        /// <summary>
        /// Initializes a new instance of the <see cref="LineSeriesGroup" /> class.
        /// </summary>
        public LineSeriesGroup()
            : base(lineSeriesType)
        {

        }

        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return lineSeriesType;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating which group of axes the series group is associated with.
        /// </summary>
        public AxisGroupName AxisGroupName
        {
            get
            {
                return this.axisGroup;
            }
            set
            {
                if (this.axisGroup != value)
                {
                    this.axisGroup = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the value indicating how the series are grouped.
        /// </summary>
        public SeriesGrouping Grouping
        {
            get
            {
                return this.grouping;
            }
            set
            {
                if (this.grouping != value)
                {
                    this.grouping = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        internal override void CloneProperties(SeriesGroup group)
        {
            base.CloneProperties(group);

            LineSeriesGroup newSeriesGroup = (LineSeriesGroup)group;
            newSeriesGroup.Grouping = this.Grouping;
            newSeriesGroup.AxisGroupName = this.AxisGroupName;
        }
    }
}