﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents the numeric literals chart data. This type of chart data holds a series of numbers as data.
    /// </summary>
    public class NumericChartData : IChartData
    {
        private readonly IEnumerable<double> numericLiterals;

        /// <summary>
        /// Initializes a new instance of the <see cref="NumericChartData" /> class.
        /// </summary>
        public NumericChartData(IEnumerable<double> numericLiterals)
        {
            this.numericLiterals = numericLiterals;
        }

        /// <summary>
        /// Gets the type of chart data.
        /// </summary>
        public ChartDataType ChartDataType
        {
            get
            {
                return ChartDataType.NumericLiteral;
            }
        }

        /// <summary>
        /// Gets the numeric chart data.
        /// </summary>
        public IEnumerable<double> NumericLiterals
        {
            get
            {
                return this.numericLiterals;
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public IChartData Clone()
        {
            return new NumericChartData(new List<double>(this.NumericLiterals));
        }
    }
}
