﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a base class for the point series.
    /// </summary>
    public abstract class PointSeriesBase : SeriesBase
    {
        private IChartData xValues;
        private IChartData yValues;

        /// <summary>
        /// Gets or sets the data for the X values of the series.
        /// </summary>
        public IChartData XValues
        {
            get
            {
                return this.xValues;
            }
            set
            {
                if (this.xValues != value)
                {
                    this.xValues = value;
                    this.OnSeriesChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the data for the Y values of the series.
        /// </summary>
        public IChartData YValues
        {
            get
            {
                return this.yValues;
            }
            set
            {
                if (this.yValues != value)
                {
                    this.yValues = value;
                    this.OnSeriesChanged();
                }
            }
        }

        internal sealed override IChartData VerticalSeriesData
        {
            get
            {
                return this.YValues;
            }
            set
            {
                this.YValues = value;
            }
        }

        internal sealed override IChartData HorizontalSeriesData
        {
            get
            {
                return this.XValues;
            }
            set
            {
                this.XValues = value;
            }
        }

        internal override void CloneProperties(SeriesBase series)
        {
            base.CloneProperties(series);

            PointSeriesBase pointSeries = (PointSeriesBase)series;

            if (this.xValues != null)
            {
                pointSeries.xValues = this.xValues.Clone();
            }

            if (this.yValues != null)
            {
                pointSeries.yValues = this.yValues.Clone();
            }
        }
    }
}
