﻿#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a series of type scatter.
    /// </summary>
    public class ScatterSeries : PointSeriesBase, ISupportMarker, ISupportSmooth
    {
        private bool isSmooth;
        private Marker marker;

        /// <summary>
        /// Gets the type of the series.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return SeriesType.Scatter;
            }
        }

        /// <summary>
        /// Gets the style of the scatter series.
        /// </summary>
        public ScatterStyle ScatterStyle
        {
            get
            {
                bool hasMarker = this.Marker != null && this.Marker.Fill != null && !(this.Marker.Fill is NoFill);

                bool hasLine = false;
                if (this.Outline != null && (this.Outline.Fill != null || this.Outline.Width.HasValue))
                {
                    hasLine = !(this.Outline.Fill is NoFill);
                }

                if (hasLine && !this.isSmooth && !hasMarker)
                {
                    return ScatterStyle.Line;
                }
                else if (hasLine && !this.isSmooth && hasMarker)
                {
                    return ScatterStyle.LineMarker;
                }
                else if (!hasLine && hasMarker)
                {
                    return ScatterStyle.Marker;
                }
                else if (hasLine && this.isSmooth && !hasMarker)
                {
                    return ScatterStyle.Smooth;
                }
                else if (hasLine && this.isSmooth && hasMarker)
                {
                    return ScatterStyle.SmoothMarker;
                }

                return ScatterStyle.None;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating if the series line will be smooth.
        /// </summary>
        public bool IsSmooth
        {
            get
            {
                return this.isSmooth;
            }
            set
            {
                if (this.isSmooth != value)
                {
                    this.isSmooth = value;
                    this.OnSeriesChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the series marker.
        /// </summary>
        public Marker Marker
        {
            get
            {
                return this.marker;
            }
            set
            {
                if (this.marker != null)
                {
                    this.marker.Changed -= this.Marker_Changed;
                }

                this.marker = value;

                if (this.marker != null)
                {
                    this.marker.Changed += this.Marker_Changed;
                }

                this.OnSeriesChanged();
            }
        }

        internal override void CloneProperties(SeriesBase series)
        {
            base.CloneProperties(series);

            ScatterSeries scatterSeries = (ScatterSeries)series;

            scatterSeries.isSmooth = this.isSmooth;
            if (this.marker != null)
            {
                scatterSeries.marker = this.marker.Clone();
            }
        }

        private void Marker_Changed(object sender, System.EventArgs e)
        {
            this.OnSeriesChanged();
        }
    }
}
