﻿namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series of type scatter.
    /// </summary>
    public class ScatterSeriesGroup : SeriesGroup<ScatterSeries>, ISupportAxes
    {
        private const SeriesType seriesType = SeriesType.Scatter;

        private AxisGroupName axisGroup;
        private ScatterStyle? scatterStyle;

        /// <summary>
        /// Initializes a new instance of the <see cref="ScatterSeriesGroup" /> class.
        /// </summary>
        public ScatterSeriesGroup()
            : base(seriesType)
        {
        }

        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public override SeriesType SeriesType
        {
            get
            {
                return seriesType;
            }
        }

        /// <summary>
        /// Gets or sets the value indicating which group of axes the series group is associated with.
        /// </summary>
        public AxisGroupName AxisGroupName
        {
            get
            {
                return this.axisGroup;
            }
            set
            {
                if (this.axisGroup != value)
                {
                    this.axisGroup = value;
                    this.OnSeriesGroupChanged();
                }
            }
        }

        internal ScatterStyle ScatterStyle
        {
            get
            {
                if (!this.scatterStyle.HasValue)
                {
                    this.scatterStyle = this.GetScatterStyle();
                }

                return this.scatterStyle.Value;
            }
        }

        internal override int DataRangesPerSeries
        {
            get
            {
                return 2;
            }
        }

        internal override bool HasCategories
        {
            get
            {
                return false;
            }
        }

        internal override void OnSeriesGroupChanged()
        {
            this.scatterStyle = null;
            base.OnSeriesGroupChanged();
        }

        internal override void CloneProperties(SeriesGroup group)
        {
            base.CloneProperties(group);

            ScatterSeriesGroup newSeriesGroup = (ScatterSeriesGroup)group;
            newSeriesGroup.AxisGroupName = this.AxisGroupName;
        }

        private ScatterStyle? GetScatterStyle()
        {
            foreach (ScatterSeries series in this.Series)
            {
                if (series.ScatterStyle == ScatterStyle.LineMarker)
                {
                    return ScatterStyle.LineMarker;
                }
            }

            return ScatterStyle.SmoothMarker;
        }
    }
}
