﻿using System;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a base class for the series.
    /// </summary>
    public abstract class SeriesBase
    {
        private Title title;
        private Outline outline;

        /// <summary>
        /// Initializes a new instance of the <see cref="SeriesBase" /> class.
        /// </summary>
        protected SeriesBase()
        {
            this.outline = new Outline();
            this.outline.ShapeOutlineChanged += this.Outline_ShapeOutlineChanged;
        }

        /// <summary>
        /// Gets the type of the series.
        /// </summary>
        public abstract SeriesType SeriesType
        {
            get;
        }

        /// <summary>
        /// Gets or sets the title of the series.
        /// </summary>
        public Title Title
        {
            get
            {
                return this.title;
            }
            set
            {
                if (this.title != value)
                {
                    this.title = value;
                    this.OnSeriesChanged();
                }
            }
        }

        /// <summary>
        /// Gets the series outline.
        /// </summary>
        public Outline Outline
        {
            get
            {
                return this.outline;
            }
            internal set
            {
                if (this.outline != value)
                {
                    this.outline = value;
                    this.OnSeriesChanged();
                }
            }
        }

        internal abstract IChartData HorizontalSeriesData { get; set; }

        internal abstract IChartData VerticalSeriesData { get; set; }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public SeriesBase Clone()
        {
            SeriesBase series = (SeriesBase)Activator.CreateInstance(this.GetType());

            this.CloneProperties(series);

            return series;
        }

        internal virtual void CloneProperties(SeriesBase series)
        {
            if (series.title != null)
            {
                series.title = this.title.Clone();
            }

            if (series.outline != null)
            {
                series.outline = this.outline.Clone();
            }
        }

        internal event EventHandler SeriesChanged;

        internal void OnSeriesChanged()
        {
            if (this.SeriesChanged != null)
            {
                this.SeriesChanged(this, EventArgs.Empty);
            }
        }

        private void Outline_ShapeOutlineChanged(object sender, EventArgs e)
        {
            this.OnSeriesChanged();
        }
    }
}