﻿using System;
using System.Collections.Generic;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    internal static class SeriesFactory
    {
        private static readonly Dictionary<SeriesType, Func<SeriesBase>> seriesConstructors;

        static SeriesFactory()
        {
            seriesConstructors = new Dictionary<SeriesType, Func<SeriesBase>>();

            seriesConstructors.Add(SeriesType.Bar, () =>
            {
                return new BarSeries();
            });

            seriesConstructors.Add(SeriesType.Line, () =>
            {
                return new LineSeries();
            });

            seriesConstructors.Add(SeriesType.Pie, () =>
            {
                return new PieSeries();
            });

            seriesConstructors.Add(SeriesType.Area, () =>
            {
                return new AreaSeries();
            });

            seriesConstructors.Add(SeriesType.Scatter, () =>
            {
                ScatterSeries series = new ScatterSeries();
                series.Outline.Fill = new NoFill();

                return series;
            });

            seriesConstructors.Add(SeriesType.Bubble, () =>
            {
                return new BubbleSeries();
            });
        }

        internal static SeriesBase GetSeries(SeriesType seriesType)
        {
            if (!seriesConstructors.ContainsKey(seriesType))
            {
                throw new NotSupportedException("This type of chart is not supported.");
            }

            return seriesConstructors[seriesType]();
        }

        internal static SeriesBase GetSeries(SeriesType seriesType, IChartData xValuesData, IChartData valuesData, IChartData bubbleSizesData, Title title = null)
        {
            SeriesBase newSeries = GetSeries(seriesType);
            newSeries.Title = title;

            if (seriesType == SeriesType.Bubble)
            {
                BubbleSeries bubbleSeries = (BubbleSeries)newSeries;
                bubbleSeries.XValues = xValuesData;
                bubbleSeries.YValues = valuesData;
                bubbleSeries.BubbleSizes = bubbleSizesData;
            }
            else
            {
                newSeries.VerticalSeriesData = valuesData;
                newSeries.HorizontalSeriesData = xValuesData;
            }

            return newSeries;
        }
    }
}
