﻿using System;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a base class for a group of series.
    /// </summary>
    public abstract class SeriesGroup
    {
        /// <summary>
        /// Gets the type of series this group holds.
        /// </summary>
        public abstract SeriesType SeriesType
        {
            get;
        }

        /// <summary>
        /// Gets the collection of series of the group.
        /// </summary>
        public SeriesCollection Series
        {
            get
            {
                return this.SeriesInternal;
            }
        }

        internal abstract SeriesCollection SeriesInternal { get; }

        internal virtual int DataRangesPerSeries
        {
            get
            {
                return 1;
            }
        }

        internal virtual bool HasCategories
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public SeriesGroup Clone()
        {
            SeriesGroup series = (SeriesGroup)Activator.CreateInstance(this.GetType());

            this.CloneProperties(series);

            return series;
        }

        internal virtual void CloneProperties(SeriesGroup group)
        {
            foreach (SeriesBase series in this.Series)
            {
                group.Series.Add(series.Clone());
            }
        }

        internal abstract event EventHandler SeriesGroupChanged;
    }
}
