﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a group of series.
    /// </summary>
    public class SeriesGroupCollection : IEnumerable<SeriesGroup>
    {
        private readonly List<SeriesGroup> innerList;

        /// <summary>
        /// Initializes a new instance of the <see cref="SeriesGroupCollection" /> class.
        /// </summary>
        public SeriesGroupCollection()
        {
            this.innerList = new List<SeriesGroup>();
        }

        /// <summary>
        /// Adds a new group of series to the collection.
        /// </summary>
        /// <param name="seriesGroup">The new group of series.</param>
        public void Add(SeriesGroup seriesGroup)
        {
            if (seriesGroup == null)
            {
                throw new ArgumentNullException("seriesGroup");
            }

            this.innerList.Add(seriesGroup);
            seriesGroup.SeriesGroupChanged += this.SeriesGroup_SeriesGroupChanged;
            this.OnCollectionChanged();
        }

        /// <summary>
        /// Removes the specified group of series from the collection,
        /// </summary>
        /// <param name="seriesGroup"></param>
        public void Remove(SeriesGroup seriesGroup)
        {
            if (this.innerList.Contains(seriesGroup))
            {
                seriesGroup.SeriesGroupChanged -= this.SeriesGroup_SeriesGroupChanged;
                this.innerList.Remove(seriesGroup);
                this.OnCollectionChanged();
            }
        }

        /// <summary>
        /// Enumerates the elements of the collection.
        /// </summary>
        public IEnumerator<SeriesGroup> GetEnumerator()
        {
            return this.innerList.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.innerList.GetEnumerator();
        }

        private void SeriesGroup_SeriesGroupChanged(object sender, EventArgs e)
        {
            this.OnSeriesGroupsChanged();
        }

        internal event EventHandler CollectionChanged;

        private void OnCollectionChanged()
        {
            if (this.CollectionChanged != null)
            {
                this.CollectionChanged(this, EventArgs.Empty);
            }
        }

        internal event EventHandler SeriesGroupsChanged;

        private void OnSeriesGroupsChanged()
        {
            if (this.SeriesGroupsChanged != null)
            {
                this.SeriesGroupsChanged(this, EventArgs.Empty);
            }
        }
    }
}
