﻿using System;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents a base class for a group of series.
    /// </summary>
    /// <typeparam name="T">The type of the series that the group can hold.</typeparam>
    public abstract class SeriesGroup<T> : SeriesGroup
        where T : SeriesBase
    {
        private readonly SeriesCollection<T> series;

        /// <summary>
        /// Initializes a new instance of the <see cref="SeriesGroup" /> class.
        /// </summary>
        protected SeriesGroup(SeriesType seriesType)
        {
            this.series = new SeriesCollection<T>(seriesType);
            this.series.CollectionChanged += this.Series_CollectionChanged;
            this.series.SeriesChanged += this.Series_SeriesChanged;
        }

        /// <summary>
        /// Gets the collection of series of the group. The series are represented by the <see cref="SeriesBase" /> class.
        /// </summary>
        public new SeriesCollection<T> Series
        {
            get
            {
                return this.SeriesInternal as SeriesCollection<T>;
            }
        }

        internal override SeriesCollection SeriesInternal
        {
            get
            {
                return this.series;
            }
        }

        private void Series_SeriesChanged(object sender, EventArgs e)
        {
            this.OnSeriesGroupChanged();
        }

        private void Series_CollectionChanged(object sender, EventArgs e)
        {
            this.OnSeriesGroupChanged();
        }

        internal override event EventHandler SeriesGroupChanged;

        internal virtual void OnSeriesGroupChanged()
        {
            if (this.SeriesGroupChanged != null)
            {
                this.SeriesGroupChanged(this, EventArgs.Empty);
            }
        }
    }
}