﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represents the string literals chart data. This type of chart data holds a series of strings as data.
    /// </summary>
    public class StringChartData : IChartData
    {
        private readonly IEnumerable<string> stringLiterals;

        /// <summary>
        /// Initializes a new instance of the <see cref="StringChartData" /> class.
        /// </summary>
        public StringChartData(IEnumerable<string> stringLiterals)
        {
            this.stringLiterals = stringLiterals;
        }

        /// <summary>
        /// Gets the type of chart data.
        /// </summary>
        public ChartDataType ChartDataType
        {
            get
            {
                return ChartDataType.StringLiteral;
            }
        }

        /// <summary>
        /// Gets the string chart data.
        /// </summary>
        public IEnumerable<string> StringLiterals
        {
            get
            {
                return this.stringLiterals;
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public IChartData Clone()
        {
            return new StringChartData(new List<string>(this.StringLiterals));
        }
    }
}
