﻿using System;

namespace Telerik.Windows.Documents.Model.Drawing.Charts
{
    /// <summary>
    /// Represent the formula chart title. This type of title holds a formula which refers to the actual title.
    /// </summary>
    public class FormulaTitle : Title
    {
        private readonly FormulaChartData formula;

        /// <summary>
        /// Initializes a new instance of the <see cref="FormulaTitle" /> class.
        /// </summary>
        public FormulaTitle(FormulaChartData formulaChartData)
        {
            if (formulaChartData == null)
            {
                throw new ArgumentNullException("formulaChartData", "The title data cannot be null.");
            }

            this.formula = formulaChartData;
        }

        /// <summary>
        /// Gets the type of the title.
        /// </summary>
        public override TitleType TitleType
        {
            get
            {
                return TitleType.Formula;
            }
        }

        /// <summary>
        /// Gets the formula referring to the actual title.
        /// </summary>
        public FormulaChartData Formula
        {
            get
            {
                return this.formula;
            }
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public override Title Clone()
        {
            return new FormulaTitle(this.Formula.Clone() as FormulaChartData);
        }
    }
}
