﻿using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Utilities;

#if (WPF || SILVERLIGHT) && !NETCORE
namespace Telerik.Windows.Documents.Spreadsheet.Theming
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// Represents a theme.
    /// </summary>
    public class DocumentTheme : NamedObjectBase
    {
        private readonly ThemeColorScheme colorScheme;
        private readonly ThemeFontScheme fontScheme;

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentTheme" /> class.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="colorScheme">The color scheme.</param>
        /// <param name="fontScheme">The font scheme.</param>
        public DocumentTheme(string name, ThemeColorScheme colorScheme, ThemeFontScheme fontScheme)
            : base(name)
        {
            Guard.ThrowExceptionIfNull(colorScheme, "colorScheme");
            Guard.ThrowExceptionIfNull(fontScheme, "fontScheme");

            this.colorScheme = colorScheme;
            this.fontScheme = fontScheme;
        }

        /// <summary>
        /// Gets the color scheme.
        /// </summary>
        /// <value>The color scheme.</value>
        public ThemeColorScheme ColorScheme
        {
            get
            {
                return this.colorScheme;
            }
        }

        /// <summary>
        /// Gets the font scheme.
        /// </summary>
        /// <value>The font scheme.</value>
        public ThemeFontScheme FontScheme
        {
            get
            {
                return this.fontScheme;
            }
        }

        /// <summary>
        /// Creates deep copy of this document theme.
        /// </summary>
        /// <returns>The cloned document theme.</returns>
        public DocumentTheme Clone()
        {
            return new DocumentTheme(this.Name, this.ColorScheme.Clone(), this.FontScheme.Clone());
        }

        /// <inheritdoc />
        public override bool Equals(object obj)
        {
            DocumentTheme other = obj as DocumentTheme;

            if (other == null)
            {
                return false;
            }

            return this.Name.Equals(other.Name) &&
                this.ColorScheme.Equals(other.ColorScheme) &&
                this.FontScheme.Equals(other.FontScheme);
        }

        /// <inheritdoc />
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.Name.GetHashCode(), this.ColorScheme.GetHashCode(), this.FontScheme.GetHashCode());
        }
    }
}