﻿using System;

#if (WPF || SILVERLIGHT) && !NETCORE
namespace Telerik.Windows.Documents.Model.Drawing.Theming
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// Represents the outline of an object.
    /// </summary>
    public class Outline
    {
        private Fill fill;
        private double? width;

        /// <summary>
        /// Gets or sets the fill of the outline.
        /// </summary>
        public Fill Fill
        {
            get
            {
                return this.fill;
            }
            set
            {
                if (this.fill != value)
                {
                    this.fill = value;
                    this.OnShapeOutlineChanged();
                }
            }
        }

        /// <summary>
        /// Gets ot sets the width of the line in points.
        /// </summary>
        public double? Width
        {
            get
            {
                return this.width;
            }
            set
            {
                if (this.width != value)
                {
                    this.width = value;
                    this.OnShapeOutlineChanged();
                }
            }
        }

        internal Outline()
        {
        }

        internal Outline(Fill fill, double? width)
        {
            this.Fill = fill;
            this.Width = width;
        }

        /// <summary>
        /// Creates a deep copy of the object.
        /// </summary>
        /// <returns>The deep copy of the object.</returns>
        public Outline Clone()
        {
            Fill clonedFill = null;
            if (this.Fill != null)
            {
                this.Fill.Clone();
            }

            return new Outline(clonedFill, this.Width);
        }

        internal event EventHandler ShapeOutlineChanged;

        internal void OnShapeOutlineChanged()
        {
            if (this.ShapeOutlineChanged != null)
            {
                this.ShapeOutlineChanged(this, EventArgs.Empty);
            }
        }
    }
}
