﻿using System.Collections.ObjectModel;
using Telerik.Windows.Documents.Media;

#if (WPF || SILVERLIGHT) && !NETCORE
namespace Telerik.Windows.Documents.Spreadsheet.Theming
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// Describes the color and font schemes for the predefined document themes.
    /// </summary>
    public static class PredefinedThemeSchemes
    {
        private static readonly DocumentTheme defaultTheme;
        private static readonly ReadOnlyCollection<ThemeColorScheme> colorSchemes;
        private static readonly ReadOnlyCollection<ThemeFontScheme> fontSchemes;

        /// <summary>
        /// The default document theme.
        /// </summary>
        public static DocumentTheme DefaultTheme
        {
            get
            {
                return defaultTheme;
            }
        }

        /// <summary>
        /// Predefined color schemes.
        /// </summary>
        public static ReadOnlyCollection<ThemeColorScheme> ColorSchemes
        {
            get
            {
                return colorSchemes;
            }
        }

        /// <summary>
        /// Predefined font schemes.
        /// </summary>
        public static ReadOnlyCollection<ThemeFontScheme> FontSchemes
        {
            get
            {
                return fontSchemes;
            }
        }

        static PredefinedThemeSchemes()
        {
            fontSchemes = new ReadOnlyCollection<ThemeFontScheme>(new ThemeFontScheme[]
            {
                new ThemeFontScheme("Office", "Cambria", "Calibri"),
                new ThemeFontScheme("Office 2", "Calibri", "Cambria"),
                new ThemeFontScheme("Office Classic", "Arial", "Times New Roman"),
                new ThemeFontScheme("Office Classic 2", "Arial", "Arial"),
                new ThemeFontScheme("Adjacency", "Cambria", "Calibri"),
                new ThemeFontScheme("Angels", "Franklin Gothic Medium", "Franklin Gothic Book"),
                new ThemeFontScheme("Apex", "Lucida Sans", "Book Antiqua"),
                new ThemeFontScheme("Apothecary", "Book Antiqua", "Century Gothic"),
                new ThemeFontScheme("Aspect", "Verdana", "Verdana"),
                new ThemeFontScheme("Austin", "Century Gothic", "Century Gothic"),
                new ThemeFontScheme("Black Tie", "Garamond", "Garamond"),
                new ThemeFontScheme("Civic", "Georgia", "Georgia"),
                new ThemeFontScheme("Clarity", "Arial", "Arial"),
                new ThemeFontScheme("Composite", "Calibri", "Calibri"),
                new ThemeFontScheme("Concourse", "Lucida Sans Unicode", "Lucida Sans Unicode"),
                new ThemeFontScheme("Couture", "Garamond", "Garamond"),
                new ThemeFontScheme("Elemental", "Palatino Linotype", "Palatino Linotype"),
                new ThemeFontScheme("Equity", "Franklin Gothic Book", "Perpetua"),
                new ThemeFontScheme("Essential", "Arial Black", "Arial"),
                new ThemeFontScheme("Executive", "Century Gothic", "Palatino Linotype"),
                new ThemeFontScheme("Flow", "Calibri", "Constantia"),
                new ThemeFontScheme("Foundry", "Rockwell", "Rockwell"),
                new ThemeFontScheme("Grid", "Franklin Gothic Medium", "Franklin Gothic Medium"),
                new ThemeFontScheme("Hardcover", "Book Antiqua", "Book Antiqua"),
                new ThemeFontScheme("Horizon", "Arial Narrow", "Arial Narrow"),
                new ThemeFontScheme("Median", "Tw Cen MT", "Tw Cen MT"),
                new ThemeFontScheme("Metro", "Consolas", "Corbel"),
                new ThemeFontScheme("Module", "Corbel", "Corbel"),
                new ThemeFontScheme("Newsprint", "Impact", "Times New Roman"),
                new ThemeFontScheme("Opulent", "Trebuchet MS", "Trebuchet MS"),
                new ThemeFontScheme("Oriel", "Century Schoolbook", "Century Schoolbook"),
                new ThemeFontScheme("Origin", "Bookman Old Style", "Gill Sans MT"),
                new ThemeFontScheme("Paper", "Constantia", "Constantia"),
                new ThemeFontScheme("Perspective", "Arial", "Arial"),
                new ThemeFontScheme("Pushpin", "Constantia", "Franklin Gothic Book"),
                new ThemeFontScheme("Slipstream", "Trebuchet MS", "Trebuchet MS"),
                new ThemeFontScheme("Solstice", "Gill Sans MT", "Gill Sans MT"),
                new ThemeFontScheme("Technic", "Franklin Gothic Book", "Arial"),
                new ThemeFontScheme("Thatch", "Tw Cen MT", "Tw Cen MT"),
                new ThemeFontScheme("Trek", "Franklin Gothic Medium", "Franklin Gothic Book"),
                new ThemeFontScheme("Urban", "Trebuchet MS", "Georgia"),
                new ThemeFontScheme("Verve", "Century Gothic", "Century Gothic"),
                new ThemeFontScheme("Waveform", "Candara", "Candara")
            });

            colorSchemes = new ReadOnlyCollection<ThemeColorScheme>(new ThemeColorScheme[]
            {
                CreateThemeColorScheme("Office",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#EEECE1",
                    text2: "#1F497D",
                    accent1: "#4F81BD",
                    accent2: "#C0504D",
                    accent3: "#9BBB59",
                    accent4: "#8064A2",
                    accent5: "#4BACC6",
                    accent6: "#F79646",
                    hyperlink: "#0000FF",
                    followedHyperlink: "#800080"),

                CreateThemeColorScheme("Grayscale",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#F8F8F8",
                    text2: "#000000",
                    accent1: "#DDDDDD",
                    accent2: "#B2B2B2",
                    accent3: "#969696",
                    accent4: "#808080",
                    accent5: "#5F5F5F",
                    accent6: "#4D4D4D",
                    hyperlink: "#5F5F5F",
                    followedHyperlink: "#919191"),

                CreateThemeColorScheme("Adjacency",
                    background1: "#FFFFFF",
                    text1: "#2F2B20",
                    background2: "#DFDCB7",
                    text2: "#675E47",
                    accent1: "#A9A57C",
                    accent2: "#9CBEBD",
                    accent3: "#D2CB6C",
                    accent4: "#95A39D",
                    accent5: "#C89F5D",
                    accent6: "#B1A089",
                    hyperlink: "#D25814",
                    followedHyperlink: "#849A0A"),

                CreateThemeColorScheme("Angels",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#CDD7D9",
                    text2: "#434342",
                    accent1: "#797B7E",
                    accent2: "#F96A1B",
                    accent3: "#08A1D9",
                    accent4: "#7C984A",
                    accent5: "#C2AD8D",
                    accent6: "#506E94",
                    hyperlink: "#5F5F5F",
                    followedHyperlink: "#969696"),

                CreateThemeColorScheme("Apex",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#C9C2D1",
                    text2: "#69676D",
                    accent1: "#CEB966",
                    accent2: "#9CB084",
                    accent3: "#6BB1C9",
                    accent4: "#6585CF",
                    accent5: "#7E6BC9",
                    accent6: "#A379BB",
                    hyperlink: "#410082",
                    followedHyperlink: "#932968"),

                 CreateThemeColorScheme("Apothecary",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#ECEDD1",
                    text2: "#564B3C",
                    accent1: "#93A299",
                    accent2: "#CF543F",
                    accent3: "#B5AE53",
                    accent4: "#848058",
                    accent5: "#E8B54D",
                    accent6: "#786C71",
                    hyperlink: "#CCCC00",
                    followedHyperlink: "#B2B2B2"),

                 CreateThemeColorScheme("Aspect",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E3DED1",
                    text2: "#323232",
                    accent1: "#F07F09",
                    accent2: "#9F2936",
                    accent3: "#1B587C",
                    accent4: "#4E8542",
                    accent5: "#604878",
                    accent6: "#C19859",
                    hyperlink: "#6B9F25",
                    followedHyperlink: "#B26B02"),

                 CreateThemeColorScheme("Austin",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#CAF278",
                    text2: "#3E3D2D",
                    accent1: "#94C600",
                    accent2: "#71685A",
                    accent3: "#FF6700",
                    accent4: "#909465",
                    accent5: "#956B43",
                    accent6: "#FEA022",
                    hyperlink: "#E68200",
                    followedHyperlink: "#FFA94A"),

                 CreateThemeColorScheme("Black Tie",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E3DCCF",
                    text2: "#46464A",
                    accent1: "#6F6F74",
                    accent2: "#A7B789",
                    accent3: "#BEAE98",
                    accent4: "#92A9B9",
                    accent5: "#9C8265",
                    accent6: "#8D6974",
                    hyperlink: "#67AABF",
                    followedHyperlink: "#B1B5AB"),

                 CreateThemeColorScheme("Civic",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#C5D1D7",
                    text2: "#646B86",
                    accent1: "#D16349",
                    accent2: "#CCB400",
                    accent3: "#8CADAE",
                    accent4: "#8C7B70",
                    accent5: "#8FB08C",
                    accent6: "#D19049",
                    hyperlink: "#00A3D6",
                    followedHyperlink: "#694F07"),

                 CreateThemeColorScheme("Clarity",
                    background1: "#FFFFFF",
                    text1: "#292934",
                    background2: "#F3F2DC",
                    text2: "#D2533C",
                    accent1: "#93A299",
                    accent2: "#AD8F67",
                    accent3: "#726056",
                    accent4: "#4C5A6A",
                    accent5: "#808DA0",
                    accent6: "#79463D",
                    hyperlink: "#0000FF",
                    followedHyperlink: "#800080"),

                 CreateThemeColorScheme("Composite",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E7ECED",
                    text2: "#5B6973",
                    accent1: "#98C723",
                    accent2: "#59B0B9",
                    accent3: "#DEAE00",
                    accent4: "#B77BB4",
                    accent5: "#E0773C",
                    accent6: "#A98D63",
                    hyperlink: "#26CBEC",
                    followedHyperlink: "#598C8C"),

                 CreateThemeColorScheme("Concourse",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DEF5FA",
                    text2: "#464646",
                    accent1: "#2DA2BF",
                    accent2: "#DA1F28",
                    accent3: "#EB641B",
                    accent4: "#39639D",
                    accent5: "#474B78",
                    accent6: "#7D3C4A",
                    hyperlink: "#FF8119",
                    followedHyperlink: "#44B9E8"),

                 CreateThemeColorScheme("Couture",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#D0CCB9",
                    text2: "#37302A",
                    accent1: "#9E8E5C",
                    accent2: "#A09781",
                    accent3: "#85776D",
                    accent4: "#AEAFA9",
                    accent5: "#8D878B",
                    accent6: "#6B6149",
                    hyperlink: "#B6A272",
                    followedHyperlink: "#8A784F"),

                 CreateThemeColorScheme("Elemental",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#ACCBF9",
                    text2: "#242852",
                    accent1: "#629DD1",
                    accent2: "#297FD5",
                    accent3: "#7F8FA9",
                    accent4: "#4A66AC",
                    accent5: "#5AA2AE",
                    accent6: "#9D90A0",
                    hyperlink: "#9454C3",
                    followedHyperlink: "#3EBBF0"),

                 CreateThemeColorScheme("Equity",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E9E5DC",
                    text2: "#696464",
                    accent1: "#D34817",
                    accent2: "#9B2D1F",
                    accent3: "#A28E6A",
                    accent4: "#956251",
                    accent5: "#918485",
                    accent6: "#855D5D",
                    hyperlink: "#CC9900",
                    followedHyperlink: "#96A9A9"),

                 CreateThemeColorScheme("Essential",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#C8C8B1",
                    text2: "#D1282E",
                    accent1: "#7A7A7A",
                    accent2: "#F5C501",
                    accent3: "#526DB0",
                    accent4: "#989AAC",
                    accent5: "#DC5924",
                    accent6: "#B4B392",
                    hyperlink: "#CC9900",
                    followedHyperlink: "#969696"),

                 CreateThemeColorScheme("Executive",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E4E9EF",
                    text2: "#2F5897",
                    accent1: "#6076B4",
                    accent2: "#9C5252",
                    accent3: "#E68422",
                    accent4: "#846648",
                    accent5: "#63891F",
                    accent6: "#758085",
                    hyperlink: "#3399FF",
                    followedHyperlink: "#B2B2B2"),

                 CreateThemeColorScheme("Flow",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DBF5F9",
                    text2: "#04617B",
                    accent1: "#0F6FC6",
                    accent2: "#009DD9",
                    accent3: "#0BD0D9",
                    accent4: "#10CF9B",
                    accent5: "#7CCA62",
                    accent6: "#A5C249",
                    hyperlink: "#F49100",
                    followedHyperlink: "#85DFD0"),

                 CreateThemeColorScheme("Foundry",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#EAEBDE",
                    text2: "#676A55",
                    accent1: "#72A376",
                    accent2: "#B0CCB0",
                    accent3: "#A8CDD7",
                    accent4: "#C0BEAF",
                    accent5: "#CEC597",
                    accent6: "#E8B7B7",
                    hyperlink: "#DB5353",
                    followedHyperlink: "#903638"),

                 CreateThemeColorScheme("Grid",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#CCD1B9",
                    text2: "#534949",
                    accent1: "#C66951",
                    accent2: "#BF974D",
                    accent3: "#928B70",
                    accent4: "#87706B",
                    accent5: "#94734E",
                    accent6: "#6F777D",
                    hyperlink: "#CC9900",
                    followedHyperlink: "#C0C0C0"),

                 CreateThemeColorScheme("Hardcover",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#ECE9C6",
                    text2: "#895D1D",
                    accent1: "#873624",
                    accent2: "#D6862D",
                    accent3: "#D0BE40",
                    accent4: "#877F6C",
                    accent5: "#972109",
                    accent6: "#AEB795",
                    hyperlink: "#CC9900",
                    followedHyperlink: "#B2B2B2"),

                 CreateThemeColorScheme("Horizon",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DC9E1F",
                    text2: "#1F2123",
                    accent1: "#7E97AD",
                    accent2: "#CC8E60",
                    accent3: "#7A6A60",
                    accent4: "#B4936D",
                    accent5: "#67787B",
                    accent6: "#9D936F",
                    hyperlink: "#646464",
                    followedHyperlink: "#969696"),

                 CreateThemeColorScheme("Median",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#EBDDC3",
                    text2: "#775F55",
                    accent1: "#94B6D2",
                    accent2: "#DD8047",
                    accent3: "#A5AB81",
                    accent4: "#D8B25C",
                    accent5: "#7BA79D",
                    accent6: "#968C8C",
                    hyperlink: "#F7B615",
                    followedHyperlink: "#704404"),

                 CreateThemeColorScheme("Metro",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#D6ECFF",
                    text2: "#4E5B6F",
                    accent1: "#7FD13B",
                    accent2: "#EA157A",
                    accent3: "#FEB80A",
                    accent4: "#00ADDC",
                    accent5: "#738AC8",
                    accent6: "#1AB39F",
                    hyperlink: "#EB8803",
                    followedHyperlink: "#5F7791"),

                 CreateThemeColorScheme("Module",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#D4D4D6",
                    text2: "#5A6378",
                    accent1: "#F0AD00",
                    accent2: "#60B5CC",
                    accent3: "#E66C7D",
                    accent4: "#6BB76D",
                    accent5: "#E88651",
                    accent6: "#C64847",
                    hyperlink: "#168BBA",
                    followedHyperlink: "#680000"),

                 CreateThemeColorScheme("Newsprint",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DEDEE0",
                    text2: "#303030",
                    accent1: "#AD0101",
                    accent2: "#726056",
                    accent3: "#AC956E",
                    accent4: "#808DA9",
                    accent5: "#424E5B",
                    accent6: "#730E00",
                    hyperlink: "#D26900",
                    followedHyperlink: "#D89243"),

                 CreateThemeColorScheme("Opulent",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#F4E7ED",
                    text2: "#B13F9A",
                    accent1: "#B83D68",
                    accent2: "#AC66BB",
                    accent3: "#DE6C36",
                    accent4: "#F9B639",
                    accent5: "#CF6DA4",
                    accent6: "#FA8D3D",
                    hyperlink: "#FFDE66",
                    followedHyperlink: "#D490C5"),

                 CreateThemeColorScheme("Oriel",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#FFF39D",
                    text2: "#575F6D",
                    accent1: "#FE8637",
                    accent2: "#7598D9",
                    accent3: "#B32C16",
                    accent4: "#F5CD2D",
                    accent5: "#AEBAD5",
                    accent6: "#777C84",
                    hyperlink: "#D2611C",
                    followedHyperlink: "#3B435B"),

                 CreateThemeColorScheme("Origin",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DDE9EC",
                    text2: "#464653",
                    accent1: "#727CA3",
                    accent2: "#9FB8CD",
                    accent3: "#D2DA7A",
                    accent4: "#FADA7A",
                    accent5: "#B88472",
                    accent6: "#8E736A",
                    hyperlink: "#B292CA",
                    followedHyperlink: "#6B5680"),

                 CreateThemeColorScheme("Paper",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#FEFAC9",
                    text2: "#444D26",
                    accent1: "#A5B592",
                    accent2: "#F3A447",
                    accent3: "#E7BC29",
                    accent4: "#D092A7",
                    accent5: "#9C85C0",
                    accent6: "#809EC2",
                    hyperlink: "#8E58B6",
                    followedHyperlink: "#7F6F6F"),

                 CreateThemeColorScheme("Perspective",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#FF8600",
                    text2: "#283138",
                    accent1: "#838D9B",
                    accent2: "#D2610C",
                    accent3: "#80716A",
                    accent4: "#94147C",
                    accent5: "#5D5AD2",
                    accent6: "#6F6C7D",
                    hyperlink: "#6187E3",
                    followedHyperlink: "#7B8EB8"),

                 CreateThemeColorScheme("Pushpin",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#CCDDEA",
                    text2: "#465E9C",
                    accent1: "#FDA023",
                    accent2: "#AA2B1E",
                    accent3: "#71685C",
                    accent4: "#64A73B",
                    accent5: "#EB5605",
                    accent6: "#B9CA1A",
                    hyperlink: "#D83E2C",
                    followedHyperlink: "#ED7D27"),

                 CreateThemeColorScheme("Slipstream",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#B4DCFA",
                    text2: "#212745",
                    accent1: "#4E67C8",
                    accent2: "#5ECCF3",
                    accent3: "#A7EA52",
                    accent4: "#5DCEAF",
                    accent5: "#FF8021",
                    accent6: "#F14124",
                    hyperlink: "#56C7AA",
                    followedHyperlink: "#59A8D1"),

                 CreateThemeColorScheme("Solstice",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#E7DEC9",
                    text2: "#4F271C",
                    accent1: "#3891A7",
                    accent2: "#FEB80A",
                    accent3: "#C32D2E",
                    accent4: "#84AA33",
                    accent5: "#964305",
                    accent6: "#475A8D",
                    hyperlink: "#8DC765",
                    followedHyperlink: "#AA8A14"),

                 CreateThemeColorScheme("Technic",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#D4D2D0",
                    text2: "#3B3B3B",
                    accent1: "#6EA0B0",
                    accent2: "#CCAF0A",
                    accent3: "#8D89A4",
                    accent4: "#748560",
                    accent5: "#9E9273",
                    accent6: "#7E848D",
                    hyperlink: "#00C8C3",
                    followedHyperlink: "#A116E0"),

                 CreateThemeColorScheme("Thatch",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DFE6D0",
                    text2: "#1D3641",
                    accent1: "#759AA5",
                    accent2: "#CFC60D",
                    accent3: "#99987F",
                    accent4: "#90AC97",
                    accent5: "#FFAD1C",
                    accent6: "#B9AB6F",
                    hyperlink: "#66AACD",
                    followedHyperlink: "#809DB3"),

                 CreateThemeColorScheme("Trek",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#FBEEC9",
                    text2: "#4E3B30",
                    accent1: "#F0A22E",
                    accent2: "#A5644E",
                    accent3: "#B58B80",
                    accent4: "#C3986D",
                    accent5: "#A19574",
                    accent6: "#C17529",
                    hyperlink: "#AD1F1F",
                    followedHyperlink: "#FFC42F"),

                 CreateThemeColorScheme("Urban",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#DEDEDE",
                    text2: "#424456",
                    accent1: "#53548A",
                    accent2: "#438086",
                    accent3: "#A04DA3",
                    accent4: "#C4652D",
                    accent5: "#8B5D3D",
                    accent6: "#5C92B5",
                    hyperlink: "#67AFBD",
                    followedHyperlink: "#C2A874"),

                 CreateThemeColorScheme("Verve",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#D2D2D2",
                    text2: "#666666",
                    accent1: "#FF388C",
                    accent2: "#E40059",
                    accent3: "#9C007F",
                    accent4: "#68007F",
                    accent5: "#005BD3",
                    accent6: "#00349E",
                    hyperlink: "#17BBFD",
                    followedHyperlink: "#FF79C2"),

                 CreateThemeColorScheme("Waveform",
                    background1: "#FFFFFF",
                    text1: "#000000",
                    background2: "#C6E7FC",
                    text2: "#073E87",
                    accent1: "#31B6FD",
                    accent2: "#4584D3",
                    accent3: "#5BD078",
                    accent4: "#A5D028",
                    accent5: "#F5C040",
                    accent6: "#05E0DB",
                    hyperlink: "#0080FF",
                    followedHyperlink: "#5EAEFF")
            });

            ThemeColorScheme colorScheme = ColorSchemes[0];
            ThemeFontScheme fontScheme = FontSchemes[0];

            defaultTheme = new DocumentTheme(colorScheme.Name, colorScheme, fontScheme);
        }

        private static ThemeColorScheme CreateThemeColorScheme(
            string name,
            string background1,
            string text1,
            string background2,
            string text2,
            string accent1,
            string accent2,
            string accent3,
            string accent4,
            string accent5,
            string accent6,
            string hyperlink,
            string followedHyperlink)
        {
            return new ThemeColorScheme(name,
                ColorsHelper.HexStringToColor(background1),
                ColorsHelper.HexStringToColor(text1),
                ColorsHelper.HexStringToColor(background2),
                ColorsHelper.HexStringToColor(text2),
                ColorsHelper.HexStringToColor(accent1),
                ColorsHelper.HexStringToColor(accent2),
                ColorsHelper.HexStringToColor(accent3),
                ColorsHelper.HexStringToColor(accent4),
                ColorsHelper.HexStringToColor(accent5),
                ColorsHelper.HexStringToColor(accent6),
                ColorsHelper.HexStringToColor(hyperlink),
                ColorsHelper.HexStringToColor(followedHyperlink));
        }
    }
}