﻿using Telerik.Windows.Documents.Utilities;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif

#if (WPF || SILVERLIGHT) && !NETCORE
namespace Telerik.Windows.Documents.Spreadsheet.Theming
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// Represents a color in a theme.
    /// </summary>
    public class ThemeColor
    {
        private readonly ThemeColorType themeColorType;
        private readonly Color color;

        /// <summary>
        /// Initializes a new instance of the <see cref="ThemeColor" /> class.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <param name="themeColorType">Type of the theme color.</param>
        public ThemeColor(Color color, ThemeColorType themeColorType)
        {
            this.color = color;
            this.themeColorType = themeColorType;
        }

        /// <summary>
        /// Gets the type of the theme color.
        /// </summary>
        /// <value>The type of the theme color.</value>
        public ThemeColorType ThemeColorType
        {
            get
            {
                return this.themeColorType;
            }
        }

        /// <summary>
        /// Gets the color.
        /// </summary>
        /// <value>The color.</value>
        public Color Color
        {
            get
            {
                return this.color;
            }
        }

        /// <summary>
        /// Creates deep copy of this theme color.
        /// </summary>
        /// <returns>The cloned theme color.</returns>
        public ThemeColor Clone()
        {
            return new ThemeColor(this.Color, this.ThemeColorType);
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            ThemeColor other = obj as ThemeColor;
            if (other == null)
            {
                return false;
            }

            return this.color == other.color &&
                this.themeColorType == other.themeColorType;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.color.GetHashCode(), this.themeColorType.GetHashCode());
        }
    }
}