﻿#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#endif
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Utilities;

#if (WPF || SILVERLIGHT) && !NETCORE
namespace Telerik.Windows.Documents.Spreadsheet.Theming
#else
namespace Telerik.Documents.Common.Model
#endif
{
    /// <summary>
    /// Represents the font in a theme.
    /// </summary>
    public class ThemeFont
    {
        private readonly FontFamily fontFamily;
        private readonly FontLanguageType fontLanguageType;

        /// <summary>
        /// Initializes a new instance of the <see cref="ThemeFont" /> class.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <param name="fontLanguageType">The type of font language.</param>
        public ThemeFont(FontFamily fontFamily, FontLanguageType fontLanguageType)
        {
            Guard.ThrowExceptionIfNull(fontFamily, "fontFamily");

            this.fontFamily = fontFamily;
            this.fontLanguageType = fontLanguageType;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ThemeFont" /> class.
        /// </summary>
        /// <param name="fontName">Name of the font.</param>
        /// <param name="fontLanguageType">Type of the font language.</param>
        public ThemeFont(string fontName, FontLanguageType fontLanguageType)
            : this(new FontFamily(fontName), fontLanguageType)
        {
        }

        /// <summary>
        /// Gets the font family.
        /// </summary>
        /// <value>The font family.</value>
        public FontFamily FontFamily
        {
            get
            {
                return this.fontFamily;
            }
        }

        /// <summary>
        /// Gets the type of the font language.
        /// </summary>
        /// <value>The type of font language.</value>
        public FontLanguageType FontLanguageType
        {
            get
            {
                return this.fontLanguageType;
            }
        }

        /// <inheritdoc />
        public override bool Equals(object obj)
        {
            ThemeFont other = obj as ThemeFont;
            if (other == null)
            {
                return false;
            }

            return ObjectExtensions.EqualsOfT(this.fontFamily, other.fontFamily) &&
                this.fontLanguageType == other.fontLanguageType;
        }

        /// <inheritdoc />
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.fontFamily.GetHashCode(), this.fontLanguageType.GetHashCode());
        }
    }
}
