﻿using System;
using System.IO;

/*
The Bouncy Castle Cryptographic C#® API

License:

The Bouncy Castle License
Copyright (c) 2000-2011 The Legion Of The Bouncy Castle (http://www.bouncycastle.org)
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sub license, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
 */

namespace Telerik.Windows.Documents.Common.Model.Protection
{
    /*
       Contributors:

       The following people have contributed to the C# Bouncy Castle Cryptography Package.
       Thanks, may your castles never deflate!

       Kaiser Yang <kaiseryang@yahoo.com> - initial port of the lightweight API and ASN.1 library. Finding BigInteger loop problem.
       Asier Murciego <a.murciego@captiva-sys.es> - Further patching to BigInteger library.
       Megan Woods <megan.woods@widestreet.com.au> - X509 certificate generation, RSA/DSA digest signature classes.
       David Del Vecchio <ddelvecc@virginia.edu> - patches to RSA Pkcs1 Signature generation OID issues, help with clarifications on DateTime and certificates.
       Nelson Fernandez <nelson-bc@kpanic.com.ar> - patches to allow compilation under mono.
       Paulo Soares <psoares@consiste.pt> - patches to X509CertificateParser, C# port of JZlib plus inflater/deflater streams, C# port of Apache BZip2 classes.
       Pawel Niewiadomski <11110000b@gmail.com> - patches for X509 and CMS, unit test for time classes.
       Jesper Johansen <jesper@hc.jay.net> - bug fix for DerT61String encodings.
       Adam Sternberg <agsternberg@gmail.com> - identified problem with generation of PGP public keyrings.
       Kirill Zhuklinets <zhuklinets_k@gaz-is.ru> - initial submission of bulk of Asn1.Esf classes (RFC 3126).
       Dr Andrew Gray <andrew.gray@rcrt.co.uk> - identified problem with BigInteger.ModPow for negative exponents.
       Mauricio Ulate <mulate@gmail.com> - identified problem with non-ASCII pass phrases in PGP.
       John Allberg <John.Allberg@teliasonera.com> - initial implementation of CryptoApiRandomGenerator.
       Mattias Öhrn <mattias.ohrn@gmail.com> - identified problem with Pkcs12Store.Save and provided fix.
       Jen Andre <jandre@gmail.com> - initial implementation of case-insensitive searches for PGP keyrings.
       #Cyrille37# <cyrille37@gmail.com> - identified problem with BigInteger.ModInverse for negative values.
       David Reis Jr <davidreis@yahoo.com> - bug fix for X509CrlStoreSelector handling of NextUpdate, fix handling of null parameters for DSA in key factories, initial port of Pkix namespace and supporting tests.
       Ivan Peev <ivan.peev@cozyroc.com> - bug fix for version string displayed in PGP armored output.
       Hector Ornelas Aciga <hector.ornelas@sat.gob.mx> - patch to add support for PKCS#5 Scheme 2 keys.
       Tom Van Holle <tvh@dsoft.be> - patch to add new class: Pkcs10CertificationRequestDelaySigned.
    */

    internal interface IEncoder
    {
        int Encode(byte[] data, int off, int length, Stream outStream);

        int Decode(byte[] data, int off, int length, Stream outStream);

        int DecodeString(string data, Stream outStream);
    }
}
