﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Data
{
    internal class OrderedSet<T> : ICollection<T>
    {
        private readonly HashSet<T> set;
        private readonly List<T> store;

        public OrderedSet()
        {
            this.store = new List<T>();
            this.set = new HashSet<T>();
        }

        public T this[int index]
        {
            get
            {
                return this.store[index];
            }
        }

        public void Add(T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            this.AddOverride(item);
        }

        public void Insert(int index, T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            this.store.Insert(index, item);
            this.set.Add(item);
        }

        public void InsertRange(int index, IEnumerable<T> items)
        {
            Guard.ThrowExceptionIfNull(items, "items");

            this.store.InsertRange(index, items);
            foreach (T item in items)
            {
                this.set.Add(item);
            }
        }

        public void AddRange(IEnumerable<T> items)
        {
            Guard.ThrowExceptionIfNull(items, "items");

            foreach (T item in items)
            {
                this.Add(item);
            }
        }

        public void Clear()
        {
            this.store.Clear();
            this.set.Clear();
        }

        public bool Contains(T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            return this.set.Contains(item);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            Guard.ThrowExceptionIfNull(array, "array");

            this.set.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        public bool Remove(T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            bool result = this.store.Remove(item);
            if (!this.store.Contains(item))
            {
                this.set.Remove(item);
            }

            return result;
        }

        public IEnumerator<T> GetEnumerator()
        {
            return store.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        protected virtual void AddOverride(T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            this.store.Add(item);
            this.set.Add(item);
        }
    }
}
