﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Core.Data
{
    internal class ResourceIndexedTable<T> : IEnumerable<T>
    {
        private readonly bool uniqueResources;
        private readonly int startIndex;
        private readonly QueueDictionary<T, int> resourceToIndex;
        private readonly QueueDictionary<int, T> indexToResource;

        public ResourceIndexedTable(bool uniqueResources = true, int startIndex = 0)
        {
            this.startIndex = startIndex;
            this.uniqueResources = uniqueResources;
            if (uniqueResources)
            {
                this.resourceToIndex = new QueueDictionary<T, int>();
            }

            this.indexToResource = new QueueDictionary<int, T>();
        }

        public T this[int index]
        {
            get
            {
                return this.indexToResource[index];
            }
            set
            {
                this.indexToResource[index] = value;
            }
        }
        
        public int Add(T resource)
        {
            int index = this.startIndex + this.indexToResource.Count;
            return this.Add(resource, index);
        }

        public int Add(T resource, int index)
        {
            int actualIndex;
            if (this.uniqueResources)
            {
                if (!this.resourceToIndex.TryGetValue(resource, out actualIndex))
                {
                    this.resourceToIndex.Add(resource, index);
                    this.indexToResource.Add(index, resource);
                    actualIndex = index;
                }
            }
            else
            {
                actualIndex = index;
                this.indexToResource.Add(index, resource);
            }

            return actualIndex;
        }

        public void AddRange(IEnumerable<T> resources)
        {
            foreach (var resource in resources)
            {
                this.Add(resource);
            }
        }

        public bool Contains(T resource)
        {
            return this.resourceToIndex.ContainsKey(resource);
        }

        public int GetIndex(T resource)
        {
            return this.resourceToIndex[resource];
        }

        public IEnumerator<T> GetEnumerator()
        {
            foreach (var keyValuePair in this.indexToResource)
            {
                yield return keyValuePair.Value;
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
}