﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Data.StateMachine
{
    internal class State<TArgs>
    {
        private readonly string name;
        private readonly Action<TArgs> onStartAction;
        private readonly Action<TArgs> onStopAction;
        private readonly Action<TArgs> onEnterAction;
        private readonly Action<TArgs> onLeaveAction;
        private readonly Action<TArgs> onStayAction;

        public State(string name, Action<TArgs> onEnterAction = null, Action<TArgs> onLeaveAction = null, Action<TArgs> onStayAction = null, Action<TArgs> onStartAction = null, Action<TArgs> onStopAction = null)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            this.name = name;
            this.onStartAction = onStartAction;
            this.onStopAction = onStopAction;
            this.onStayAction = onStayAction;
            this.onEnterAction = onEnterAction;
            this.onLeaveAction = onLeaveAction;
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public void Start(TArgs arguments)
        {
            ExecuteActionIfNotNull(this.onStartAction, arguments);
        }

        public void Stop(TArgs arguments)
        {
            ExecuteActionIfNotNull(this.onStopAction, arguments);
        }

        public void Enter(TArgs arguments)
        {
            ExecuteActionIfNotNull(this.onEnterAction, arguments);
        }

        public void Leave(TArgs arguments)
        {
            ExecuteActionIfNotNull(this.onLeaveAction, arguments);
        }

        public void Stay(TArgs arguments)
        {
            ExecuteActionIfNotNull(this.onStayAction, arguments);
        }

        private static void ExecuteActionIfNotNull(Action<TArgs> action, TArgs arguments)
        {
            if (action != null)
            {
                action(arguments);
            }
        }

        public override string ToString()
        {
            return this.name;
        }
    }
}
