﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Data.StateMachine
{
    internal class StateMachine<TArgs>
    {
        private readonly TransitionsCollection<TArgs> transitions;
        private readonly StatesCollection<TArgs> states;

        public StateMachine()
        {
            this.transitions = new TransitionsCollection<TArgs>();
            this.states = new StatesCollection<TArgs>();
        }
        
        public StatesCollection<TArgs> States
        {
            get
            {
                return this.states;
            }
        }

        public TransitionsCollection<TArgs> Transitions
        {
            get
            {
                return this.transitions;
            }
        }

        public State<TArgs> CurrentState
        {
            get;
            private set;
        }

        public void Start(State<TArgs> initialState, TArgs arguments)
        {
            this.CurrentState = initialState;
            this.CurrentState.Start(arguments);
            this.GoToNextState(arguments);
        }

        public bool GoToNextState(TArgs arguments)
        {
            IList<Transition<TArgs>> transitions = this.Transitions.GetAllTransitionsForState(this.CurrentState);

            for (int i = 0; i < transitions.Count; i++)
            {
                Transition<TArgs> transition = transitions[i];

                if (transition.Predicate(arguments))
                {
                    return this.GoToState(transition.ToState, arguments);
                }
            }

            return false;
        }

        public bool GoToState(State<TArgs> state, TArgs arguments)
        {
            if (this.CurrentState == state)
            {
                this.CurrentState.Stay(arguments);
            }
            else
            {
                this.CurrentState.Leave(arguments);
                this.CurrentState = state;
                this.CurrentState.Enter(arguments);
            }

            return true;
        }

        public void Stop(TArgs arguments)
        {
            this.CurrentState.Stop(arguments);
            this.CurrentState = null;
        }
    }
}
