﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Data.StateMachine
{
    internal class TransitionsCollection<TArgs>
    {
        // Used only for distinct values in the list.
        private readonly Dictionary<State<TArgs>, HashSet<Transition<TArgs>>> uniqueStore;
        private readonly Dictionary<State<TArgs>, List<Transition<TArgs>>> store;

        public TransitionsCollection()
        {
            this.uniqueStore = new Dictionary<State<TArgs>, HashSet<Transition<TArgs>>>();
            this.store = new Dictionary<State<TArgs>, List<Transition<TArgs>>>();
        }

        public void AddTransition(Transition<TArgs> transition)
        {
            HashSet<Transition<TArgs>> uniqueTransitions;
            List<Transition<TArgs>> transitions;

            if (!this.uniqueStore.TryGetValue(transition.FromState, out uniqueTransitions))
            {
                uniqueTransitions = new HashSet<Transition<TArgs>>();
                this.uniqueStore[transition.FromState] = uniqueTransitions;

                transitions = new List<Transition<TArgs>>();
                this.store[transition.FromState] = transitions;
            }
            else
            {
                transitions = this.store[transition.FromState];
            }

            if (uniqueTransitions.Add(transition))
            {
                transitions.Add(transition);
            }
        }

        public IList<Transition<TArgs>> GetAllTransitionsForState(State<TArgs> state)
        {
            return this.store[state];
        }
    }
}
