﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Core.Data
{
    internal class ValueMapper<TFrom, TTo>
    {
        private readonly Dictionary<TFrom, TTo> fromToToType;
        private readonly Dictionary<TTo, TFrom> toToFromType;

        private ValueBox<TFrom> defaultFromValue;
        private ValueBox<TTo> defaultToValue;

        public ValueMapper()
        {
            this.fromToToType = new Dictionary<TFrom, TTo>();
            this.toToFromType = new Dictionary<TTo, TFrom>();
        }

        public ValueMapper(TFrom defaultFromValue, TTo defaultToValue)
            : this()
        {
            this.defaultFromValue = new ValueBox<TFrom>(defaultFromValue);
            this.defaultToValue = new ValueBox<TTo>(defaultToValue);
        }

        public int Count
        {
            get
            {
                return this.fromToToType.Count;
            }
        }

        public ValueBox<TFrom> DefaultFromValue
        {
            get
            {
                return this.defaultFromValue;
            }
            set
            {
                this.defaultFromValue = value;
            }
        }

        public ValueBox<TTo> DefaultToValue
        {
            get
            {
                return this.defaultToValue;
            }
            set
            {
                this.defaultToValue = value;
            }
        }

        public IEnumerable<TFrom> FromValues
        {
            get
            {
                return this.fromToToType.Keys;
            }
        }

        public IEnumerable<TTo> ToValues
        {
            get
            {
                return this.toToFromType.Keys;
            }
        }
        
        public void AddPair(TFrom from, TTo to)
        {
            this.fromToToType[from] = to;
            this.toToFromType[to] = from;
        }

        public TFrom GetFromValue(TTo value)
        {
            TFrom result;
            if (!this.toToFromType.TryGetValue(value, out result))
            {
                if (this.defaultFromValue == null)
                {
                    throw new InvalidOperationException("Not supported to value: " + value.ToString());
                }

                result = this.defaultFromValue.Value;
            }

            return result;
        }


        public bool TryGetFromValue(TTo value, out TFrom result)
        {
            return this.toToFromType.TryGetValue(value, out result);
        }

        public bool TryGetToValue(TFrom value, out TTo result)
        {
            return this.fromToToType.TryGetValue(value, out result);
        }

        public TTo GetToValue(TFrom value)
        {
            TTo result;
            if (!this.fromToToType.TryGetValue(value, out result))
            {
                if (this.defaultToValue == null)
                {
                    throw new InvalidOperationException("Not supported from value: " + value.ToString());
                }

                result = this.defaultToValue.Value;
            }

            return result;
        }

        public bool ContainsToValue(TTo to)
        {
            return this.toToFromType.ContainsKey(to);
        }

        public bool ContainsFromValue(TFrom from)
        {
            return this.fromToToType.ContainsKey(from);
        }
    }
}