﻿namespace Telerik.Documents.Core.Fonts
{
    /// <summary>
    /// Represents a family of fonts.
    /// </summary>
    public class FontFamily
    {
        private readonly string familyName;

        /// <summary>
        /// Initializes a new instance of the <see cref="FontFamily"/> class.
        /// </summary>
        /// <param name="familyName"></param>
        public FontFamily(string familyName)
        {
            this.familyName = familyName;
        }

        /// <summary>
        /// Gets the family name.
        /// </summary>
        public string Source
        {
            get
            {
                return this.familyName;
            }
        }

        /// <summary>
        /// Determines whether the specified object is equal to the current object.
        /// </summary>
        /// <param name="obj">The object to compare with the current object.</param>
        /// <returns>true if the specified object is equal to the current object; otherwise, false.</returns>
        public override bool Equals(object obj)
        {
            FontFamily other = obj as FontFamily;

            if (other == null)
            {
                return false;
            }

            return this.familyName.ToLowerInvariant() == other.familyName.ToLowerInvariant();
        }

        /// <summary>
        /// Serves as the default hash function.
        /// </summary>
        /// <returns>A hash code for the current object.</returns>
        public override int GetHashCode()
        {
            if (this.familyName == null)
            {
                return 0;
            }

            return this.familyName.ToLowerInvariant().GetHashCode();
        }

        public override string ToString()
        {
            return this.familyName;
        }
    }
}
