﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Documents.Core.Fonts
{
    internal struct FontStretch : IFormattable
    {
        private int _stretch;

        private int RealStretch
        {
            get
            {
                return this._stretch + 5;
            }
        }

        internal FontStretch(int stretch)
        {
            this._stretch = stretch - 5;
        }

        public static int Compare(FontStretch left, FontStretch right)
        {
            return left._stretch - right._stretch;
        }

        private string ConvertToString(string format, IFormatProvider provider)
        {
            string str;
            FontStretches.FontStretchToString(this.RealStretch, out str);

            return str;
        }

        public bool Equals(FontStretch obj)
        {
            return this == obj;
        }

        public override bool Equals(object obj)
        {
            if (!(obj is FontStretch))
            {
                return false;
            }
            return this == (FontStretch)obj;
        }

        public static FontStretch FromOpenTypeStretch(int stretchValue)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, stretchValue, "stretchValue");

            return new FontStretch(stretchValue);
        }

        public override int GetHashCode()
        {
            return this.RealStretch;
        }

        public static bool operator ==(FontStretch left, FontStretch right)
        {
            return FontStretch.Compare(left, right) == 0;
        }

        public static bool operator >(FontStretch left, FontStretch right)
        {
            return FontStretch.Compare(left, right) > 0;
        }

        public static bool operator >=(FontStretch left, FontStretch right)
        {
            return FontStretch.Compare(left, right) >= 0;
        }

        public static bool operator !=(FontStretch left, FontStretch right)
        {
            return !(left == right);
        }

        public static bool operator <(FontStretch left, FontStretch right)
        {
            return FontStretch.Compare(left, right) < 0;
        }

        public static bool operator <=(FontStretch left, FontStretch right)
        {
            return FontStretch.Compare(left, right) <= 0;
        }

        string System.IFormattable.ToString(string format, IFormatProvider provider)
        {
            return this.ConvertToString(format, provider);
        }

        public int ToOpenTypeStretch()
        {
            return this.RealStretch;
        }

        public override string ToString()
        {
            return this.ConvertToString(null, null);
        }
    }
}
