﻿using System;
using System.Globalization;

namespace Telerik.Documents.Core.Fonts
{
    internal static class FontStretches
    {
        public static FontStretch Condensed
        {
            get
            {
                return new FontStretch(3);
            }
        }

        public static FontStretch Expanded
        {
            get
            {
                return new FontStretch(7);
            }
        }

        public static FontStretch ExtraCondensed
        {
            get
            {
                return new FontStretch(2);
            }
        }

        public static FontStretch ExtraExpanded
        {
            get
            {
                return new FontStretch(8);
            }
        }

        public static FontStretch Medium
        {
            get
            {
                return new FontStretch(5);
            }
        }

        public static FontStretch Normal
        {
            get
            {
                return new FontStretch(5);
            }
        }

        public static FontStretch SemiCondensed
        {
            get
            {
                return new FontStretch(4);
            }
        }

        public static FontStretch SemiExpanded
        {
            get
            {
                return new FontStretch(6);
            }
        }

        public static FontStretch UltraCondensed
        {
            get
            {
                return new FontStretch(1);
            }
        }

        public static FontStretch UltraExpanded
        {
            get
            {
                return new FontStretch(9);
            }
        }

        internal static bool FontStretchStringToKnownStretch(string s, IFormatProvider provider, ref FontStretch fontStretch)
        {
            int num;
            switch (s.Length)
            {
                case 6:
                    {
                        if (s.Equals("Normal", StringComparison.OrdinalIgnoreCase))
                        {
                            fontStretch = FontStretches.Normal;
                            return true;
                        }
                        if (!s.Equals("Medium", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.Medium;
                        return true;
                    }
                case 7:
                case 10:
                case 11:
                    {
                        if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                        {
                            return false;
                        }
                        fontStretch = FontStretch.FromOpenTypeStretch(num);
                        return true;
                    }
                case 8:
                    {
                        if (!s.Equals("Expanded", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.Expanded;
                        return true;
                    }
                case 9:
                    {
                        if (!s.Equals("Condensed", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.Condensed;
                        return true;
                    }
                case 12:
                    {
                        if (!s.Equals("SemiExpanded", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.SemiExpanded;
                        return true;
                    }
                case 13:
                    {
                        if (s.Equals("SemiCondensed", StringComparison.OrdinalIgnoreCase))
                        {
                            fontStretch = FontStretches.SemiCondensed;
                            return true;
                        }
                        if (s.Equals("ExtraExpanded", StringComparison.OrdinalIgnoreCase))
                        {
                            fontStretch = FontStretches.ExtraExpanded;
                            return true;
                        }
                        if (!s.Equals("UltraExpanded", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.UltraExpanded;
                        return true;
                    }
                case 14:
                    {
                        if (s.Equals("UltraCondensed", StringComparison.OrdinalIgnoreCase))
                        {
                            fontStretch = FontStretches.UltraCondensed;
                            return true;
                        }
                        if (!s.Equals("ExtraCondensed", StringComparison.OrdinalIgnoreCase))
                        {
                            if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                            {
                                return false;
                            }
                            fontStretch = FontStretch.FromOpenTypeStretch(num);
                            return true;
                        }
                        fontStretch = FontStretches.ExtraCondensed;
                        return true;
                    }
                default:
                    {
                        if (!int.TryParse(s, NumberStyles.Integer, provider, out num))
                        {
                            return false;
                        }
                        fontStretch = FontStretch.FromOpenTypeStretch(num);
                        return true;
                    }
            }
        }

        internal static bool FontStretchToString(int stretch, out string convertedValue)
        {
            switch (stretch)
            {
                case 1:
                    {
                        convertedValue = "UltraCondensed";
                        return true;
                    }
                case 2:
                    {
                        convertedValue = "ExtraCondensed";
                        return true;
                    }
                case 3:
                    {
                        convertedValue = "Condensed";
                        return true;
                    }
                case 4:
                    {
                        convertedValue = "SemiCondensed";
                        return true;
                    }
                case 5:
                    {
                        convertedValue = "Normal";
                        return true;
                    }
                case 6:
                    {
                        convertedValue = "SemiExpanded";
                        return true;
                    }
                case 7:
                    {
                        convertedValue = "Expanded";
                        return true;
                    }
                case 8:
                    {
                        convertedValue = "ExtraExpanded";
                        return true;
                    }
                case 9:
                    {
                        convertedValue = "UltraExpanded";
                        return true;
                    }
            }
            convertedValue = null;
            return false;
        }
    }
}
