﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Core.Imaging
{
    internal class BitmapImageSizeDecoder : ImageSizeDecoderBase
    {
        private const int BitmapImageSizeMinimumBytesCount = 26;

        private static readonly string[] supportedExtensions = new string[] { "bmp" };
        private static readonly byte[] signature = new byte[] { 0x42, 0x4D };

        protected override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return supportedExtensions;
            }
        }

        protected override int SignatureLength
        {
            get
            {
                return signature.Length;
            }
        }

        protected override int BytesToSkipAfterSignature
        {
            get
            {
                return 16;
            }
        }

        protected override bool IsValidImage(Stream stream)
        {
            return stream.Length > BitmapImageSizeMinimumBytesCount && IsSignatureValid(stream, signature);
        }

        protected override bool IsValidImage(byte[] bytes)
        {
            return bytes.Length > BitmapImageSizeMinimumBytesCount && IsSignatureValid(bytes, signature);
        }

        protected override Size DecodeSizeInternal(byte[] bytes)
        {
            int width = ReadInt32(bytes, this.GetSizeStartIndexInBytes());
            int height = ReadInt32(bytes, this.GetSizeStartIndexInBytes() + sizeof(int));

            return new Size(width, height);
        }

        protected override Size DecodeSizeInternal(Stream stream)
        {
            BinaryReader reader = new BinaryReader(stream);
            stream.Seek(0, SeekOrigin.Begin);
            reader.ReadBytes(this.GetSizeStartIndexInBytes());

            int width = reader.ReadInt32();
            int height = reader.ReadInt32();

            return new Size(width, height);
        }
    }
}