﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Core.Imaging
{
    internal class GifImageSizeDecoder : ImageSizeDecoderBase
    {
        private const int GifImageSizeMinimumBytesCount = 10;

        private static readonly string[] supportedExtensions = new string[] { "gif" };
        private static readonly byte[] signature1 = new byte[] { 0x47, 0x49, 0x46, 0x38, 0x37, 0x61 };
        private static readonly byte[] signature2 = new byte[] { 0x47, 0x49, 0x46, 0x38, 0x39, 0x61 };
        protected override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return supportedExtensions;
            }
        }

        protected override int SignatureLength
        {
            get
            {
                return signature1.Length;
            }
        }

        protected override bool IsValidImage(Stream stream)
        {
            return stream.Length > GifImageSizeMinimumBytesCount &&
                  (IsSignatureValid(stream, signature1) || IsSignatureValid(stream, signature2));
        }

        protected override bool IsValidImage(byte[] bytes)
        {
            return bytes.Length > GifImageSizeMinimumBytesCount &&
                   (IsSignatureValid(bytes, signature1) || IsSignatureValid(bytes, signature2));
        }

        protected override Size DecodeSizeInternal(byte[] bytes)
        {
            int width = ReadInt16(bytes, this.GetSizeStartIndexInBytes());
            int height = ReadInt16(bytes, this.GetSizeStartIndexInBytes() + sizeof(short));

            return new Size(width, height);
        }

        protected override Size DecodeSizeInternal(Stream stream)
        {
            BinaryReader reader = new BinaryReader(stream);
            int width = reader.ReadInt16();
            int height = reader.ReadInt16();

            return new Size(width, height);
        }
    }
}