﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging
{
    internal static class ImageSizeDecodersManager
    {
        private static readonly HashSet<IImageSizeDecoder> registeredDecoders = new HashSet<IImageSizeDecoder>();

        static ImageSizeDecodersManager()
        {
            InitializeDefaultDecoders();
        }

        public static Size GetSize(Stream stream, string imageTypeExtension)
        {
            IImageSizeDecoder decoder = registeredDecoders.Where(dec => dec.CanDecode(imageTypeExtension)).FirstOrDefault();
            if (decoder != null)
            {
                return decoder.DecodeSize(stream);
            }
            else
            {
                return ImageSizeDecoderBase.DefaultNotDecodedImageSize;
            }
        }

        public static Size GetSize(byte[] imageBytes, string imageTypeExtension)
        {
            if (imageBytes == null)
            {
                return Size.Empty;
            }

            IImageSizeDecoder decoder = registeredDecoders.Where(dec => dec.CanDecode(imageTypeExtension)).FirstOrDefault();
            if (decoder != null)
            {
                return decoder.DecodeSize(imageBytes);
            }
            else
            {
                return ImageSizeDecoderBase.DefaultNotDecodedImageSize;
            }
        }

        private static void InitializeDefaultDecoders()
        {
            registeredDecoders.Add(new PngImageSizeDecoder());
            registeredDecoders.Add(new JpegImageSizeDecoder());
            registeredDecoders.Add(new GifImageSizeDecoder());
            registeredDecoders.Add(new BitmapImageSizeDecoder());
#if WPF
            registeredDecoders.Add(new TifImageSizeDecoder());
            registeredDecoders.Add(new MetafileImageSizeDecoder());
#endif
        }
    }
}