﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Markers;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder
{
    internal class JpegByteArrayReader : ReaderBase, IJpegReader
    {
        private readonly JpegDataProcessor jpegProcessor;

        private byte bitsData;
        private int bitsCount;

        public JpegByteArrayReader(byte[] data)
            :base(data)
        {
            this.jpegProcessor = new JpegDataProcessor(this);
            this.Restart();
        }

        public int ReadBit()
        {
            int bit = 0;

            if (this.bitsCount > 0)
            {
                this.bitsCount--;
                bit = (this.bitsData >> this.bitsCount) & 1;
            }
            else
            {
                this.bitsData = this.Read();

                if (this.bitsData == 0xFF)
                {
                    var nextByte = this.Read();

                    if (nextByte != JpegMarker.EscapeFFByte)
                    {
                        throw new InvalidOperationException(string.Format("Unexpected marker {0:X}!", (this.bitsData << 8) | nextByte));
                    }
                    // unstuff 0
                }

                this.bitsCount = 7;
                bit = this.bitsData >> 7;
            }

            return bit;
        }

        public void Restart()
        {
            this.bitsCount = 0;
            this.bitsData = 0;
        }

        public JpegMarker ReadNextJpegMarker()
        {
            return this.jpegProcessor.ReadNextJpegMarker();
        }

        public IEnumerable<T> ReadJpegTables<T>()
            where T : JpegTable, new()
        {
            return this.jpegProcessor.ReadJpegTables<T>();
        }

        public byte Read4()
        {
            return this.jpegProcessor.Read4();
        }

        public byte Read8()
        {
            return this.jpegProcessor.Read8();
        }

        public ushort Read16()
        {
            return this.jpegProcessor.Read16();
        }

        public int ReceiveAndExtend(int length)
        {
            return this.jpegProcessor.ReceiveAndExtend(length);
        }

        public int Receive(int length)
        {
            return this.jpegProcessor.Receive(length);
        }

        public void FreeResources()
        {
        }
    }
}