﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Markers;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Utils;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder
{
    internal class JpegDataProcessor
    {
        private readonly BitsReader bitsReader;
        private readonly IJpegReader owner;

        public JpegDataProcessor(IJpegReader owner)
        {
            this.owner = owner;
            this.bitsReader = new BitsReader();
        }

        public byte Read4()
        {
            return (byte)this.ReadBit(4);
        }

        public byte Read8()
        {
            return this.owner.Read();
        }

        public ushort Read16()
        {
            byte[] bytes = new byte[2];
            this.ReadBE(bytes, 2);

            return BitConverter.ToUInt16(bytes, 0);
        }

        private int ReadBE(byte[] buffer, int count)
        {
            int read = 0;

            for (int i = count - 1; i >= 0; i--)
            {
                if (this.owner.Position >= this.owner.Length)
                {
                    return read;
                }

                buffer[i] = this.owner.Read();
                read++;
            }

            return read;
        }

        public int ReadBit()
        {
            return this.ReadBit(1);
        }

        public int Receive(int length)
        {
            int n = 0;

            while (length > 0)
            {
                int bit = this.owner.ReadBit();
                n = (n << 1) | bit;
                length--;
            }

            return n;
        }

        public int ReceiveAndExtend(int length)
        {
            int n = this.Receive(length);

            if (n >= (1 << (length - 1)))
            {
                return n;
            }

            return n + (-1 << length) + 1;
        }

        public IEnumerable<T> ReadJpegTables<T>()
            where T : JpegTable, new()
        {
            long position = this.owner.Position;
            ushort length = this.owner.Read16();

            while (position + length > this.owner.Position)
            {
                T table = new T();
                table.Read(this.owner);

                yield return table;
            }
        }

        public JpegMarker ReadNextJpegMarker()
        {
            this.owner.Restart();

            byte b;
            while (this.ReadJpegByte(out b))
            {
            }

            this.owner.Read();

            return JpegMarker.GetMarker(this.owner.Read());
        }

        private bool ReadJpegByte(out byte bits)
        {
            bits = 0;
            if (this.owner.Peek() == JpegMarker.MarkerIdentifier)
            {
                if (this.owner.Peek(1) == JpegMarker.EscapeFFByte)
                {
                    bits = this.owner.Read();
                    this.owner.Read();
                }
                else
                {
                    return false;
                }
            }
            else
            {
                bits = this.owner.Read();
            }

            return true;
        }

        private int ReadBit(int count)
        {
            if (this.bitsReader.IsEmpty)
            {
                this.bitsReader.Data = this.owner.Read();
            }

            return this.bitsReader.ReadBits(count);
        }

    }
}