﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder
{
    internal static class Logger
    {
        private static readonly Action<string> log;
        private static readonly Dictionary<string, string> markerNames;

        static Logger()
        {
            log = (s) => System.Diagnostics.Debug.WriteLine(s);

            markerNames = new Dictionary<string,string>();
            markerNames["ffd8"] = "SOI - start of image";
            markerNames["ffc0"] = "SOF0 - start of frame Baseline DCT";
            markerNames["ffc1"] = "SOF1 - start of frame Extended DCT";
            markerNames["ffc2"] = "SOF2 - start of frame Progressive DCT";
            markerNames["ffc4"] = "DHT - define huffman table";
            markerNames["ffdb"] = "DQT - define quantization table";
            markerNames["ffdd"] = "DRI - define restart interval";
            markerNames["ffda"] = "SOS - start of scan";
            for (int i = 0; i < 8; i += 1)
            {
                markerNames["ffd" + i] = "RST" + i + " - restart of scan";
            }
            for (int i = 0; i < 16; i += 1)
            {
                string hex = string.Format("{0:X}", i);
                markerNames["ffe" + hex] = "APP" + hex + " - application specific";
            }
            markerNames["fffe"] = "COM - comment";
            markerNames["ffd9"] = "EOI - end of image";
        }

        public static void Log(string text)
        {
            Logger.log(text);
        }

        public static void LogMarker(byte code)
        {
            string marker = string.Format("ff{0}", ToHex(code));

            string description;
            if(!markerNames.TryGetValue(marker, out description))
            {
                description = "unknown marker";
            }

            Logger.log(string.Format("marker: {0} ({1})", marker, description));
        }

        public static string ToHex(int b)
        {
            string hex = string.Format("{0:X}", b).ToLower(CultureInfo.InvariantCulture);

            if ((hex.Length & 1) == 1)
            {
                hex = "0" + hex;
            }

            return hex;
        }
    }
}
