﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Encoder
{
    internal abstract class ScanEncoder
    {
        public static ScanEncoder GetEncoder(JpegEncodingType encodingType)
        {
            ScanEncoder scanEncoder = null;

            switch (encodingType)
            {
                case JpegEncodingType.BaselineDct:
                    scanEncoder = new BaselineDCTEncoder();
                    break;

                case JpegEncodingType.NotSupported:
                default:
                    scanEncoder = new NotSupportedScanEncoder();
                    break;
            }

            return scanEncoder;
        }

        public abstract void EncodeBlock(JpegEncoder encoder, JpegComponent component, Block block);

        /// <summary>
        /// Prepare base quantization tables.
        /// </summary>
        /// <param name="encoder">Encoder to add table to.</param>
        public abstract void PrepareQuantizationTables(JpegEncoder encoder);

        /// <summary>
        /// Prepare Huffman tables.
        /// </summary>
        /// <param name="jpegEncoder">Encoder to add table to.</param>
        public abstract void PrepareHuffmanTables(JpegEncoder jpegEncoder);
        
        public abstract void ResetEncoder(JpegEncoder encoder);
    }
}
