﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Encoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Markers
{
    internal class DefineHuffmanTableMarker : JpegMarker
    {
        private ushort length = 0;

        /// <summary>
        /// Gets length of the marker segment.
        /// </summary>
        public override ushort Length
        {
            get
            {
                return this.length;
            }
        }

        public override JpegMarkerType MarkerType
        {
            get
            {
                return JpegMarkerType.DHT;
            }
        }

        public override void InterpretMarker(JpegDecoderBase decoder)
        {
            Guard.ThrowExceptionIfNull(decoder, "decoder");

            this.length = 2;

            foreach (HuffmanTable table in decoder.Reader.ReadJpegTables<HuffmanTable>())
            {
                decoder.AddHuffmanTable(table);
                this.length += table.Length;
            }
        }

        public override void WriteMarker(JpegEncoder encoder)
        {
            base.WriteMarker(encoder);

            encoder.Writer.WriteJpegTables<HuffmanTable>(encoder.HuffmanTables, 2);
        }
    }
}
