﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Encoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Markers
{
    internal class StartOfFrameMarker : JpegMarker
    {
        private ushort length = 0;

        #region Fields

        private readonly JpegEncodingType encodingType;
        private readonly ScanDecoder scanDecoder;
        private readonly ScanEncoder scanEncoder;

        #endregion


        #region Constructors

        public StartOfFrameMarker(JpegEncodingType encodingType)
        {
            this.encodingType = encodingType;
            this.scanDecoder = ScanDecoder.GetDecoder(encodingType);
            this.scanEncoder = ScanEncoder.GetEncoder(encodingType);
        }

        #endregion 
        

        #region Properties

        /// <summary>
        /// Gets encoding type.
        /// </summary>
        public JpegEncodingType EncodingType
        {
            get
            {
                return this.encodingType;
            }
        }

        /// <summary>
        /// Gets length of the marker segment.
        /// </summary>
        public override ushort Length
        {
            get
            {
                return this.length;
            }
        }

        public override JpegMarkerType MarkerType
        {
            get
            {
                return JpegMarkerType.SOF;
            }
        }

        public ScanDecoder ScanDecoder
        {
            get
            {
                return this.scanDecoder;
            }
        }

        public ScanEncoder ScanEncoder
        {
            get
            {
                return this.scanEncoder;
            }
        }

        #endregion

        public override void InterpretMarker(JpegDecoderBase decoder)
        {
            Guard.ThrowExceptionIfNull(decoder, "decoder");

            FrameHeader header = new FrameHeader();
            header.Read(decoder.Reader);
            decoder.FrameHeader = header;
            this.length = header.Length;
        }

        public override void WriteMarker(JpegEncoder encoder)
        {
            Guard.ThrowExceptionIfNull(encoder, "encoder");

            base.WriteMarker(encoder);

            encoder.FrameHeader.Write(encoder.Writer);
        }
    }
}
