﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Decoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Encoder;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Markers
{
    internal class StartOfScanMarker : JpegMarker
    {
        private ushort length = 0;

        /// <summary>
        /// Gets length of the marker segment.
        /// </summary>
        public override ushort Length
        {
            get
            {
                return this.length;
            }
        }

        public override JpegMarkerType MarkerType
        {
            get
            {
                return JpegMarkerType.SOS;
            }
        }

        public override void InterpretMarker(JpegDecoderBase decoder)
        {
            Guard.ThrowExceptionIfNull(decoder, "decoder");

            ScanHeader header = new ScanHeader();
            header.Read(decoder.Reader);
            decoder.ScanHeader = header;
            this.length = header.Length;
        }

        public override void WriteMarker(JpegEncoder encoder)
        {
            Guard.ThrowExceptionIfNull(encoder, "encoder");
            base.WriteMarker(encoder);
            encoder.ScanHeader.Write(encoder.Writer);
        }
    }
}
