﻿using System;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables
{
    internal class Block
    {
        #region Fields

        private readonly int[] data;

        #endregion



        #region Constructors

        public Block()
        {
            this.data = new int[Constants.BlockSize];
        }

        public Block(byte[] data)
        {
            this.data = new int[Constants.BlockSize];
            for (int i = 0; i < Constants.BlockSize; i++)
            {
                this.data[i] = data[i];
            }
        }

        public Block(byte[,] data)
        {
            this.data = new int[Constants.BlockSize];

            int len0 = data.GetLength(0);
            int len1 = data.GetLength(1);
            int index = 0;

            for (int i = 0; i < len0; i++)
            {
                for (int j = 0; j < len1; j++)
                {
                    this.data[index++] = data[i, j];
                }
            }
        }

        public Block(
            byte[,] raster,
            int blockStartX,
            int blockStartY,
            int horizontalFactor,
            int verticalFactor)
        {
            this.data = new int[Constants.BlockSize];

            int rasterHeight = raster.GetLength(0);
            int rasterWidth = raster.GetLength(1);

            int xblockoffset = horizontalFactor * Constants.BlockWidth;
            int yblockoffset = verticalFactor * Constants.BlockHeight;

            for (int row = 0; row < Constants.BlockHeight; row++)
            {
                // set Y value.  check bounds
                int y = blockStartY + yblockoffset + row;
                if (y >= rasterHeight)
                {
                    break;
                }

                for (int column = 0; column < Constants.BlockWidth; column++)
                {
                    int x = blockStartX + xblockoffset + column;
                    if (x >= rasterWidth)
                    {
                        break;
                    }

                    this[row, column] = raster[y, x];
                }
            }
        }

        #endregion

        #region Indexer

        public int this[int index]
        {
            get
            {
                return this.data[index];
            }
            set
            {
                this.data[index] = value;
            }
        }

        public int this[int row, int column]
        {
            get
            {
                return this.data[GetIndex(row, column)];
            }

            set
            {
                this.data[GetIndex(row, column)] = value;
            }
        }

        #endregion

        #region Methods

        public static int GetIndex(int row, int column)
        {
            return row * Constants.BlockWidth + column;
        }

        public float[] ToArray()
        {
            return this.data.Select(d => (float)d).ToArray();
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < this.data.Length; i++)
            {
                sb.Append(this.data[i] + " ");
            }

            return sb.ToString();
        }

        #endregion
    }
}
