﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging.Jpeg.Encoder;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Tables
{
    internal class JpegScan
    {
        #region Fields

        private readonly JpegComponent[] components;
        private readonly byte maxHSampleFactor;
        private readonly byte maxVSampleFactor;
        private int minHorizontalBlocksCount;
        private int minVerticalBlocksCount;

        #endregion


        #region Properties

        public JpegComponent[] Components
        {
            get
            {
                return this.components;
            }
        }

        public byte MaxVSampleFactor
        {
            get
            {
                return this.maxVSampleFactor;
            }
        }

        public byte MaxHSampleFactor
        {
            get
            {
                return this.maxHSampleFactor;
            }
        }

        public int MinHorizontalBlocksCount
        {
            get
            {
                return this.minHorizontalBlocksCount;
            }
        }

        public int MinVerticalBlocksCount
        {
            get
            {
                return this.minVerticalBlocksCount;
            }
        }

        #endregion


        #region Constructors

        public JpegScan(JpegComponent[] components)
        {
            this.components = components;

            this.maxHSampleFactor = this.components.Max(c => c.HSampleFactor);
            this.maxVSampleFactor = this.components.Max(c => c.VSampleFactor);
        }

        #endregion


        #region Methods

        public void InitializeEncoding(JpegEncoder encoder)
        {
            foreach (JpegComponent component in this.components)
            {
                component.InitializeEncoding(this, encoder);
            }

            this.minHorizontalBlocksCount = this.components.Min(c => c.HorizontalBlocksCount);
            this.minVerticalBlocksCount = this.components.Min(c => c.VerticalBlocksCount);
        }

        public bool TryGetComponentById(int id, out JpegComponent jpegComponent)
        {
            jpegComponent = this.components.FirstOrDefault((component) => component.Id == id);

            return jpegComponent != null;
        }

        #endregion
    }
}
