﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg.Utils
{
    internal class BitsWriter
    {
        private byte data;

        public BitsWriter()
        {
            this.Clear();
        }

        /// <summary>
        /// Gets number of the bits left in the buffer.
        /// </summary>
        public int BitsLeft
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets value which indicates that buffer is empty (no bits have been written).
        /// </summary>
        public bool IsEmpty
        {
            get
            {
                return this.BitsLeft == 8;
            }
        }

        /// <summary>
        /// Gets value which indicates whether all bits in the buffer have been written.
        /// </summary>
        public bool IsFull
        {
            get
            {
                return this.BitsLeft <= 0;
            }
        }

        /// <summary>
        /// Gets writer data.
        /// </summary>
        public byte Data
        {
            get
            {
                return this.data;
            }
        }

        /// <summary>
        /// Clear buffer.
        /// </summary>
        public void Clear()
        {
            this.data = 0;
            this.BitsLeft = 8;
        }

        /// <summary>
        /// Write bits into the internal buffer.
        /// </summary>
        /// <param name="value">Value to get bits from.</param>
        /// <param name="n">Number of bits.</param>
        public void WriteBits(byte value, int n)
        {
            if (n != 0)
            {
                if (this.BitsLeft >= n)
                {
                    int mask = 0;
                    for (int i = 0; i < n; i++)
                    {
                        mask = mask << 1;
                        mask |= 1;
                    }
                    int result = value & mask;
                    this.data = (byte)(this.data << n);
                    this.data = (byte)(this.data | result);
                    this.BitsLeft -= n;
                }
                else
                {
                    throw new NotSupportedException("Not enought bits.");
                }
            }
        }
    }
}
