﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Imaging.Jpeg2000.Decoder
{
    internal class Jpeg2000Reader : ReaderBase
    {
        private const int ShortBytesCount = 2;
        private const int IntegerBytesCount = 4;
        private const int LongBytesCount = 8;

        public Jpeg2000Reader(byte[] data)
            : base(data)
        {
        }

        public short ReadBigEndianShort()
        {
            return this.ReadBigEndianObject(ShortBytesCount, (bytes) => BitConverter.ToInt16(bytes, 0));
        }

        public int ReadBigEndianInt()
        {
            return this.ReadBigEndianObject(IntegerBytesCount, (bytes) => BitConverter.ToInt32(bytes, 0));
        }

        public ushort ReadBigEndianUShort()
        {
            return this.ReadBigEndianObject(ShortBytesCount, (bytes) => BitConverter.ToUInt16(bytes, 0));
        }

        public uint ReadBigEndianUInt()
        {
            return this.ReadBigEndianObject(IntegerBytesCount, (bytes) => BitConverter.ToUInt32(bytes, 0));
        }

        public ulong ReadBigEndianULong()
        {
            return this.ReadBigEndianObject(LongBytesCount, (bytes) => BitConverter.ToUInt64(bytes, 0));
        }

        public string ReadText(int bytesCount)
        {
            byte[] buffer = new byte[bytesCount];
            base.Read(buffer, bytesCount);
            string text = System.Text.Encoding.UTF8.GetString(buffer, 0, buffer.Length);

            return text;
        }

        public Box ReadBox()
        {
            long startPosition = this.Position;

            Box box = new Box();
            box.Length = this.ReadBigEndianUInt();
            box.Type = this.ReadText(4);

            if (box.Length == 1)
            {
                box.Length = this.ReadBigEndianULong();
            }

            if (box.Length == 0)
            {
                box.Length = (ulong)(this.Length - startPosition);
            }

            long byteEnd = startPosition + (long)box.Length;
            int contentLength = (int)(byteEnd - this.Position);
            byte[] content = new byte[contentLength];
            this.Read(content, contentLength);
            box.Content = content;

            return box;
        }

        private T ReadBigEndianObject<T>(int numberOfBytes, Func<byte[], T> convertBytesToObject)
        {
            byte[] buffer = new byte[numberOfBytes];
            base.ReadBE(buffer, numberOfBytes);
            T value = convertBytesToObject(buffer);

            return value;
        }
    }
}
