﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Core.Imaging
{
    internal class PngImageSizeDecoder : ImageSizeDecoderBase
    {
        private const int PngImageSizeMinimumBytesCount = 24;

        private static readonly string[] supportedExtensions = new string[] { "png" };
        private static readonly byte[] signature = new byte[] { 0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A };

        protected override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return supportedExtensions;
            }
        }

        protected override int SignatureLength
        {
            get
            {
                return signature.Length;
            }
        }

        protected override int BytesToSkipAfterSignature
        {
            get
            {
                return 8;
            }
        }

        protected override bool IsValidImage(Stream stream)
        {
            return stream.Length > PngImageSizeMinimumBytesCount && IsSignatureValid(stream, signature);
        }

        protected override bool IsValidImage(byte[] bytes)
        {
            return bytes.Length > PngImageSizeMinimumBytesCount && IsSignatureValid(bytes, signature);
        }

        protected override Size DecodeSizeInternal(byte[] bytes)
        {
            int width = ReadLittleEndianInt32(bytes, this.GetSizeStartIndexInBytes());
            int height = ReadLittleEndianInt32(bytes, this.GetSizeStartIndexInBytes() + sizeof(int));

            return new Size(width, height);
        }

        protected override Size DecodeSizeInternal(Stream stream)
        {
            BinaryReader reader = new BinaryReader(stream);
            stream.Seek(0, SeekOrigin.Begin);
            reader.ReadBytes(this.GetSizeStartIndexInBytes());

            int width = ReadLittleEndianInt32(reader);
            int height = ReadLittleEndianInt32(reader);

            return new Size(width, height);
        }
    }
}