﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.TextMeasurer
{
    internal partial class RadTextMeasurer
    {
        [ThreadStatic]
        private static TextBlock measureTextBlock;
        [ThreadStatic]
        private static TextBlock wrappingTextBlock;

        private static TextBlock MeasureTextBlock
        {
            get
            {
                if (measureTextBlock == null)
                {
                    measureTextBlock = new TextBlock();
                    measureTextBlock.FontSize = BaseFontSize;
                }

                return measureTextBlock;
            }
        }

        private static TextBlock WrappingTextBlock
        {
            get
            {
                if (wrappingTextBlock == null)
                {
                    wrappingTextBlock = new TextBlock();
                    wrappingTextBlock.TextWrapping = TextWrapping.Wrap;
                }

                return wrappingTextBlock;
            }
        }

        /// <summary>
        /// Measures the text with wrapping.
        /// </summary>
        /// <param name="textProperties">The text properties.</param>
        /// <param name="fontProperties">The font properties.</param>
        /// <param name="wrappingWidth">Width of the wrapping.</param>
        /// <returns></returns>
        public static TextMeasurementInfo MeasureTextWithWrapping(TextProperties textProperties, FontProperties fontProperties, double wrappingWidth)
        {
            TextBlock textBlock = CreateWrappingTextBlock(textProperties, fontProperties, textProperties.FlowDirection, wrappingWidth);

            TextMeasurementInfo info = new TextMeasurementInfo();
            info.Size = textBlock.GetSize();

            return info;
        }

        private static double MeasureWholeTextWidth(TextProperties textProperties, FontProperties fontProperties)
        {
            TextBlock measureTextBlock = CreateTextBlock(textProperties.Text, textProperties.Size, fontProperties, textProperties.FlowDirection);

            return measureTextBlock.GetSize().Width;
        }

        private static TextBlock CreateTextBlock(string text, double fontSize, FontProperties fontProperties, FlowDirection flowDirection)
        {
            TextBlock textBlock = MeasureTextBlock;
            SetTextBlockProperties(textBlock, text, fontSize, fontProperties, flowDirection);

            return textBlock;
        }

        private static TextBlock CreateWrappingTextBlock(TextProperties textProperties, FontProperties fontProperties, FlowDirection flowDirection, double wrappingWidth)
        {
            TextBlock textBlock = WrappingTextBlock;
            SetTextBlockProperties(textBlock, textProperties.Text, textProperties.Size, fontProperties, flowDirection);

            if (textBlock.Width != wrappingWidth)
            {
                textBlock.Width = wrappingWidth;
            }

            if (textBlock.MaxWidth != wrappingWidth)
            {
                textBlock.MaxWidth = wrappingWidth;
            }

            return textBlock;
        }

        private static void SetTextBlockProperties(TextBlock textBlock, string text, double fontSize, FontProperties fontProperties, FlowDirection flowDirection)
        {
            textBlock.Text = text;
            if (textBlock.FontSize != fontSize)
            {
                textBlock.FontSize = fontSize;
            }

            if (textBlock.FlowDirection != flowDirection)
            {
                textBlock.FlowDirection = flowDirection;
            }

            if (textBlock.FontFamily != fontProperties.FontFamily)
            {
                textBlock.FontFamily = fontProperties.FontFamily;
            }

            if (textBlock.FontStyle != fontProperties.FontStyle)
            {
                textBlock.FontStyle = fontProperties.FontStyle;
            }

            if (textBlock.FontWeight != fontProperties.FontWeight)
            {
                textBlock.FontWeight = fontProperties.FontWeight;
            }
        }

        private static Size MeasureTextSize(string text, FontProperties fontProperties)
        {
            TextBlock measureTextBlock = CreateTextBlock(text, BaseFontSize, fontProperties, FlowDirection.LeftToRight);

            return measureTextBlock.GetSize();
        }

        private static double GetBaseLineOffset(FontProperties fontProperties, FlowDirection flowDirection)
        {
            double baseLineOffset;
            FontProperties baselineFontProperties = new FontProperties(fontProperties.FontFamily);
            lock (fontBaseLineOffsetCache)
            {
                if (!fontBaseLineOffsetCache.TryGetValue(baselineFontProperties.FontFamilyName, out baseLineOffset))
                {
                    TextBlock textBlock = CreateTextBlock(BaselineString, BaseFontSize, baselineFontProperties, flowDirection);
                    baseLineOffset = textBlock.GetBaselineOffset() / textBlock.GetSize().Height;
                    fontBaseLineOffsetCache[baselineFontProperties.FontFamilyName] = baseLineOffset;
                }
            }

            return baseLineOffset;
        }
    }
}
