﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Fonts;

namespace Telerik.Windows.Documents.Core.TextMeasurer
{
    /// <summary>
    /// Represents text measurer that can be used in multi-threaded applications.
    /// </summary>
    internal partial class RadTextMeasurer
    {
        /// <summary>
        /// Measures the text with wrapping.
        /// </summary>
        /// <param name="textProperties">The text properties.</param>
        /// <param name="fontProperties">The font properties.</param>
        /// <param name="wrappingWidth">Width of the wrapping.</param>
        /// <returns></returns>
        public static TextMeasurementInfo MeasureTextWithWrapping(TextProperties textProperties, FontProperties fontProperties, double wrappingWidth)
        {
            FormattedText formattedText = CreateFormattedText(textProperties.Text.TrimEnd(), textProperties.Size, fontProperties, textProperties.FlowDirection);
            formattedText.MaxTextWidth = wrappingWidth;
            formattedText.Trimming = TextTrimming.None;

            TextMeasurementInfo result = new TextMeasurementInfo();
            result.Size = new Size(formattedText.Width, formattedText.Height);

            return result;
        }

        private static double MeasureWholeTextWidth(TextProperties textProperties, FontProperties fontProperties)
        {
            FormattedText ft = CreateFormattedText(textProperties.Text, textProperties.Size, fontProperties, textProperties.FlowDirection);

            return ft.Width;
        }

        private static FormattedText CreateFormattedText(string text, double fontSize, FontProperties fontProperties, FlowDirection flowDirection)
        {
            FormattedText ft = new FormattedText(text, CultureInfo.CurrentCulture, flowDirection, new Typeface(fontProperties.FontFamily, fontProperties.FontStyle, fontProperties.FontWeight, FontStretches.Normal), fontSize, Brushes.Black);

            return ft;
        }

        private static Size MeasureTextSize(string text, FontProperties fontProperties)
        {
            FormattedText ft = CreateFormattedText(text, BaseFontSize, fontProperties, FlowDirection.LeftToRight);

            return new Size(ft.Width, ft.Height);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", MessageId = "Telerik.Windows.Documents.Core.TextMeasurer.RadTextMeasurer.CreateFormattedText(System.String,System.Double,Telerik.Windows.Documents.Core.Fonts.FontProperties,System.Windows.FlowDirection)")]
        private static double GetBaseLineOffset(FontProperties fontProperties, FlowDirection flowDirection)
        {
            double baseLineOffset;
            FontProperties baselineFontProperties = new FontProperties(fontProperties.FontFamily);
            lock (fontBaseLineOffsetCache)
            {
                if (!fontBaseLineOffsetCache.TryGetValue(baselineFontProperties.FontFamilyName, out baseLineOffset))
                {
                    FormattedText ft = CreateFormattedText(BaselineString, BaseFontSize, baselineFontProperties, flowDirection);
                    baseLineOffset = ft.Baseline / ft.Height;
                    fontBaseLineOffsetCache[baselineFontProperties.FontFamilyName] = baseLineOffset;
                }
            }

            return baseLineOffset;
        }
    }
}
