﻿using System;
using System.Linq;
using System.Text.RegularExpressions;
#if NETSTANDARD
using Telerik.Documents.Core;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.TextMeasurer
{
    /// <summary>
    /// Represents text properties.
    /// </summary>
    public class TextProperties
    {
        private const string IsRtlLanguagePattern = @"[\p{IsArabic}\p{IsHebrew}\p{IsDevanagari}]";
        
        private static readonly Regex rtlRegex = new Regex(IsRtlLanguagePattern);

        private readonly string text;
        private readonly double size;
        private readonly SubStringPosition subStringPosition;
        private readonly FlowDirection flowDirection;

        /// <summary>
        /// Initializes a new instance of the <see cref="TextProperties" /> class.
        /// </summary>
        /// <param name="text">The text.</param>
        /// <param name="size">The size.</param>
        /// <param name="subStringPosition">The sub string position.</param>
        public TextProperties(string text, double size = RadTextMeasurer.BaseFontSize, SubStringPosition subStringPosition = SubStringPosition.None)
        {
            this.size = size;
            this.subStringPosition = subStringPosition;
            this.flowDirection = rtlRegex.IsMatch(text) ? FlowDirection.RightToLeft : FlowDirection.LeftToRight;

            if (flowDirection == FlowDirection.RightToLeft)
            {
                switch (subStringPosition)
                {
                    case SubStringPosition.Start:
                        text += DocumentsHelper.ZeroWidthSymbol;
                        break;

                    case SubStringPosition.Middle:
                        text = DocumentsHelper.ZeroWidthSymbol + text + DocumentsHelper.ZeroWidthSymbol;
                        break;

                    case SubStringPosition.End:
                        text = DocumentsHelper.ZeroWidthSymbol + text;
                        break;
                }
            }

            this.text = text;
        }

        /// <summary>
        /// Gets the sub string position.
        /// </summary>
        /// <value>The sub string position.</value>
        public SubStringPosition SubStringPosition
        {
            get
            {
                return this.subStringPosition;
            }
        }

        /// <summary>
        /// Gets the size.
        /// </summary>
        /// <value>The size.</value>
        public double Size
        {
            get
            {
                return this.size;
            }
        }

        /// <summary>
        /// Gets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text
        {
            get
            {
                return this.text;
            }
        }

        internal FlowDirection FlowDirection
        {
            get
            {
                return this.flowDirection;
            }
        }
    }
}
