﻿using System;
using System.Collections.Generic;
using System.Globalization;

namespace Telerik.Documents.Core.Media.Internal
{
    internal static class KnownColors
    {
        private static readonly Dictionary<string, KnownColor> s_knownArgbColors;

        static KnownColors()
        {
            KnownColors.s_knownArgbColors = new Dictionary<string, KnownColor>();
            foreach (KnownColor value in Enum.GetValues(typeof(KnownColor)))
            {
                string str = string.Format("#{0,8:X8}", (uint)value);
                KnownColors.s_knownArgbColors[str] = value;
            }
        }

        internal static KnownColor ColorStringToKnownColor(string colorString)
        {
            char chr;
            if (colorString != null)
            {
                string upper = colorString.ToUpper(CultureInfo.InvariantCulture);
                switch (upper.Length)
                {
                    case 3:
                        {
                            if (upper.Equals("RED"))
                            {
                                return KnownColor.Red;
                            }
                            if (!upper.Equals("TAN"))
                            {
                                break;
                            }
                            return KnownColor.Tan;
                        }
                    case 4:
                        {
                            chr = upper[0];
                            switch (chr)
                            {
                                case 'A':
                                    {
                                        if (!upper.Equals("AQUA"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Aqua;
                                    }
                                case 'B':
                                    {
                                        if (!upper.Equals("BLUE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Blue;
                                    }
                                case 'C':
                                    {
                                        if (!upper.Equals("CYAN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Aqua;
                                    }
                                case 'D':
                                case 'E':
                                case 'F':
                                    {
                                        break;
                                    }
                                case 'G':
                                    {
                                        if (upper.Equals("GOLD"))
                                        {
                                            return KnownColor.Gold;
                                        }
                                        if (!upper.Equals("GRAY"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Gray;
                                    }
                                default:
                                    {
                                        switch (chr)
                                        {
                                            case 'L':
                                                {
                                                    if (!upper.Equals("LIME"))
                                                    {
                                                        return KnownColor.UnknownColor;
                                                    }
                                                    return KnownColor.Lime;
                                                }
                                            case 'N':
                                                {
                                                    if (!upper.Equals("NAVY"))
                                                    {
                                                        return KnownColor.UnknownColor;
                                                    }
                                                    return KnownColor.Navy;
                                                }
                                            case 'P':
                                                {
                                                    if (upper.Equals("PERU"))
                                                    {
                                                        return KnownColor.Peru;
                                                    }
                                                    if (upper.Equals("PINK"))
                                                    {
                                                        return KnownColor.Pink;
                                                    }
                                                    if (!upper.Equals("PLUM"))
                                                    {
                                                        return KnownColor.UnknownColor;
                                                    }
                                                    return KnownColor.Plum;
                                                }
                                            case 'S':
                                                {
                                                    if (!upper.Equals("SNOW"))
                                                    {
                                                        return KnownColor.UnknownColor;
                                                    }
                                                    return KnownColor.Snow;
                                                }
                                            case 'T':
                                                {
                                                    if (!upper.Equals("TEAL"))
                                                    {
                                                        return KnownColor.UnknownColor;
                                                    }
                                                    return KnownColor.Teal;
                                                }
                                        }
                                        break;
                                    }
                            }
                            break;
                        }
                    case 5:
                        {
                            chr = upper[0];
                            switch (chr)
                            {
                                case 'A':
                                    {
                                        if (!upper.Equals("AZURE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Azure;
                                    }
                                case 'B':
                                    {
                                        if (upper.Equals("BEIGE"))
                                        {
                                            return KnownColor.Beige;
                                        }
                                        if (upper.Equals("BLACK"))
                                        {
                                            return KnownColor.Black;
                                        }
                                        if (!upper.Equals("BROWN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Brown;
                                    }
                                case 'C':
                                    {
                                        if (!upper.Equals("CORAL"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Coral;
                                    }
                                case 'D':
                                case 'E':
                                case 'F':
                                case 'H':
                                case 'J':
                                case 'M':
                                case 'N':
                                    {
                                        break;
                                    }
                                case 'G':
                                    {
                                        if (!upper.Equals("GREEN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Green;
                                    }
                                case 'I':
                                    {
                                        if (!upper.Equals("IVORY"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Ivory;
                                    }
                                case 'K':
                                    {
                                        if (!upper.Equals("KHAKI"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Khaki;
                                    }
                                case 'L':
                                    {
                                        if (!upper.Equals("LINEN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Linen;
                                    }
                                case 'O':
                                    {
                                        if (!upper.Equals("OLIVE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Olive;
                                    }
                                default:
                                    {
                                        if (chr == 'W')
                                        {
                                            if (upper.Equals("WHEAT"))
                                            {
                                                return KnownColor.Wheat;
                                            }
                                            if (!upper.Equals("WHITE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.White;
                                        }
                                        break;
                                    }
                            }
                            break;
                        }
                    case 6:
                        {
                            chr = upper[0];
                            if (chr == 'B')
                            {
                                if (!upper.Equals("BISQUE"))
                                {
                                    break;
                                }
                                return KnownColor.Bisque;
                            }
                            else if (chr == 'I')
                            {
                                if (!upper.Equals("INDIGO"))
                                {
                                    break;
                                }
                                return KnownColor.Indigo;
                            }
                            else
                            {
                                switch (chr)
                                {
                                    case 'M':
                                        {
                                            if (!upper.Equals("MAROON"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Maroon;
                                        }
                                    case 'O':
                                        {
                                            if (upper.Equals("ORANGE"))
                                            {
                                                return KnownColor.Orange;
                                            }
                                            if (!upper.Equals("ORCHID"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Orchid;
                                        }
                                    case 'P':
                                        {
                                            if (!upper.Equals("PURPLE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Purple;
                                        }
                                    case 'S':
                                        {
                                            if (upper.Equals("SALMON"))
                                            {
                                                return KnownColor.Salmon;
                                            }
                                            if (upper.Equals("SIENNA"))
                                            {
                                                return KnownColor.Sienna;
                                            }
                                            if (!upper.Equals("SILVER"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Silver;
                                        }
                                    case 'T':
                                        {
                                            if (!upper.Equals("TOMATO"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Tomato;
                                        }
                                    case 'V':
                                        {
                                            if (!upper.Equals("VIOLET"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Violet;
                                        }
                                    case 'Y':
                                        {
                                            if (!upper.Equals("YELLOW"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Yellow;
                                        }
                                }
                            }
                            break;
                        }
                    case 7:
                        {
                            chr = upper[0];
                            if (chr <= 'M')
                            {
                                switch (chr)
                                {
                                    case 'C':
                                        {
                                            if (!upper.Equals("CRIMSON"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Crimson;
                                        }
                                    case 'D':
                                        {
                                            if (upper.Equals("DARKRED"))
                                            {
                                                return KnownColor.DarkRed;
                                            }
                                            if (!upper.Equals("DIMGRAY"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.DimGray;
                                        }
                                    case 'E':
                                    case 'G':
                                        {
                                            break;
                                        }
                                    case 'F':
                                        {
                                            if (!upper.Equals("FUCHSIA"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Fuchsia;
                                        }
                                    case 'H':
                                        {
                                            if (!upper.Equals("HOTPINK"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.HotPink;
                                        }
                                    default:
                                        {
                                            if (chr == 'M')
                                            {
                                                if (!upper.Equals("MAGENTA"))
                                                {
                                                    return KnownColor.UnknownColor;
                                                }
                                                return KnownColor.Fuchsia;
                                            }
                                            break;
                                        }
                                }
                            }
                            else if (chr == 'O')
                            {
                                if (!upper.Equals("OLDLACE"))
                                {
                                    break;
                                }
                                return KnownColor.OldLace;
                            }
                            else if (chr == 'S')
                            {
                                if (!upper.Equals("SKYBLUE"))
                                {
                                    break;
                                }
                                return KnownColor.SkyBlue;
                            }
                            else if (chr == 'T')
                            {
                                if (!upper.Equals("THISTLE"))
                                {
                                    break;
                                }
                                return KnownColor.Thistle;
                            }
                            else
                            {
                                break;
                            }
                            break;
                        }
                    case 8:
                        {
                            chr = upper[0];
                            if (chr <= 'H')
                            {
                                if (chr == 'C')
                                {
                                    if (!upper.Equals("CORNSILK"))
                                    {
                                        break;
                                    }
                                    return KnownColor.Cornsilk;
                                }
                                else if (chr == 'D')
                                {
                                    if (upper.Equals("DARKBLUE"))
                                    {
                                        return KnownColor.DarkBlue;
                                    }
                                    if (upper.Equals("DARKCYAN"))
                                    {
                                        return KnownColor.DarkCyan;
                                    }
                                    if (upper.Equals("DARKGRAY"))
                                    {
                                        return KnownColor.DarkGray;
                                    }
                                    if (!upper.Equals("DEEPPINK"))
                                    {
                                        break;
                                    }
                                    return KnownColor.DeepPink;
                                }
                                else if (chr == 'H')
                                {
                                    if (!upper.Equals("HONEYDEW"))
                                    {
                                        break;
                                    }
                                    return KnownColor.Honeydew;
                                }
                                else
                                {
                                    break;
                                }
                            }
                            else if (chr == 'L')
                            {
                                if (!upper.Equals("LAVENDER"))
                                {
                                    break;
                                }
                                return KnownColor.Lavender;
                            }
                            else if (chr == 'M')
                            {
                                if (!upper.Equals("MOCCASIN"))
                                {
                                    break;
                                }
                                return KnownColor.Moccasin;
                            }
                            else if (chr == 'S')
                            {
                                if (upper.Equals("SEAGREEN"))
                                {
                                    return KnownColor.SeaGreen;
                                }
                                if (!upper.Equals("SEASHELL"))
                                {
                                    break;
                                }
                                return KnownColor.SeaShell;
                            }
                            else
                            {
                                break;
                            }
                        }
                    case 9:
                        {
                            chr = upper[0];
                            switch (chr)
                            {
                                case 'A':
                                    {
                                        if (!upper.Equals("ALICEBLUE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.AliceBlue;
                                    }
                                case 'B':
                                    {
                                        if (!upper.Equals("BURLYWOOD"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.BurlyWood;
                                    }
                                case 'C':
                                    {
                                        if (upper.Equals("CADETBLUE"))
                                        {
                                            return KnownColor.CadetBlue;
                                        }
                                        if (!upper.Equals("CHOCOLATE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Chocolate;
                                    }
                                case 'D':
                                    {
                                        if (upper.Equals("DARKGREEN"))
                                        {
                                            return KnownColor.DarkGreen;
                                        }
                                        if (!upper.Equals("DARKKHAKI"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.DarkKhaki;
                                    }
                                case 'F':
                                    {
                                        if (!upper.Equals("FIREBRICK"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Firebrick;
                                    }
                                case 'G':
                                    {
                                        if (upper.Equals("GAINSBORO"))
                                        {
                                            return KnownColor.Gainsboro;
                                        }
                                        if (!upper.Equals("GOLDENROD"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Goldenrod;
                                    }
                                case 'I':
                                    {
                                        if (!upper.Equals("INDIANRED"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.IndianRed;
                                    }
                                case 'L':
                                    {
                                        if (upper.Equals("LAWNGREEN"))
                                        {
                                            return KnownColor.LawnGreen;
                                        }
                                        if (upper.Equals("LIGHTBLUE"))
                                        {
                                            return KnownColor.LightBlue;
                                        }
                                        if (upper.Equals("LIGHTCYAN"))
                                        {
                                            return KnownColor.LightCyan;
                                        }
                                        if (upper.Equals("LIGHTGRAY"))
                                        {
                                            return KnownColor.LightGray;
                                        }
                                        if (upper.Equals("LIGHTPINK"))
                                        {
                                            return KnownColor.LightPink;
                                        }
                                        if (!upper.Equals("LIMEGREEN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.LimeGreen;
                                    }
                                case 'M':
                                    {
                                        if (upper.Equals("MINTCREAM"))
                                        {
                                            return KnownColor.MintCream;
                                        }
                                        if (!upper.Equals("MISTYROSE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.MistyRose;
                                    }
                                case 'O':
                                    {
                                        if (upper.Equals("OLIVEDRAB"))
                                        {
                                            return KnownColor.OliveDrab;
                                        }
                                        if (!upper.Equals("ORANGERED"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.OrangeRed;
                                    }
                                case 'P':
                                    {
                                        if (upper.Equals("PALEGREEN"))
                                        {
                                            return KnownColor.PaleGreen;
                                        }
                                        if (!upper.Equals("PEACHPUFF"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.PeachPuff;
                                    }
                                case 'R':
                                    {
                                        if (upper.Equals("ROSYBROWN"))
                                        {
                                            return KnownColor.RosyBrown;
                                        }
                                        if (!upper.Equals("ROYALBLUE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.RoyalBlue;
                                    }
                                case 'S':
                                    {
                                        if (upper.Equals("SLATEBLUE"))
                                        {
                                            return KnownColor.SlateBlue;
                                        }
                                        if (upper.Equals("SLATEGRAY"))
                                        {
                                            return KnownColor.SlateGray;
                                        }
                                        if (!upper.Equals("STEELBLUE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.SteelBlue;
                                    }
                                case 'T':
                                    {
                                        if (!upper.Equals("TURQUOISE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.Turquoise;
                                    }
                            }
                            break;
                        }
                    case 10:
                        {
                            chr = upper[0];
                            if (chr <= 'P')
                            {
                                switch (chr)
                                {
                                    case 'A':
                                        {
                                            if (!upper.Equals("AQUAMARINE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Aquamarine;
                                        }
                                    case 'B':
                                        {
                                            if (!upper.Equals("BLUEVIOLET"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.BlueViolet;
                                        }
                                    case 'C':
                                        {
                                            if (!upper.Equals("CHARTREUSE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.Chartreuse;
                                        }
                                    case 'D':
                                        {
                                            if (upper.Equals("DARKORANGE"))
                                            {
                                                return KnownColor.DarkOrange;
                                            }
                                            if (upper.Equals("DARKORCHID"))
                                            {
                                                return KnownColor.DarkOrchid;
                                            }
                                            if (upper.Equals("DARKSALMON"))
                                            {
                                                return KnownColor.DarkSalmon;
                                            }
                                            if (upper.Equals("DARKVIOLET"))
                                            {
                                                return KnownColor.DarkViolet;
                                            }
                                            if (!upper.Equals("DODGERBLUE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.DodgerBlue;
                                        }
                                    case 'E':
                                    case 'F':
                                    case 'H':
                                    case 'I':
                                    case 'J':
                                    case 'K':
                                        {
                                            break;
                                        }
                                    case 'G':
                                        {
                                            if (!upper.Equals("GHOSTWHITE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.GhostWhite;
                                        }
                                    case 'L':
                                        {
                                            if (upper.Equals("LIGHTCORAL"))
                                            {
                                                return KnownColor.LightCoral;
                                            }
                                            if (!upper.Equals("LIGHTGREEN"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.LightGreen;
                                        }
                                    case 'M':
                                        {
                                            if (!upper.Equals("MEDIUMBLUE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.MediumBlue;
                                        }
                                    default:
                                        {
                                            if (chr == 'P')
                                            {
                                                if (upper.Equals("PAPAYAWHIP"))
                                                {
                                                    return KnownColor.PapayaWhip;
                                                }
                                                if (!upper.Equals("POWDERBLUE"))
                                                {
                                                    return KnownColor.UnknownColor;
                                                }
                                                return KnownColor.PowderBlue;
                                            }
                                            break;
                                        }
                                }
                            }
                            else if (chr == 'S')
                            {
                                if (!upper.Equals("SANDYBROWN"))
                                {
                                    break;
                                }
                                return KnownColor.SandyBrown;
                            }
                            else if (chr == 'W')
                            {
                                if (!upper.Equals("WHITESMOKE"))
                                {
                                    break;
                                }
                                return KnownColor.WhiteSmoke;
                            }
                            else
                            {
                                break;
                            }
                            break;
                        }
                    case 11:
                        {
                            chr = upper[0];
                            if (chr <= 'N')
                            {
                                switch (chr)
                                {
                                    case 'D':
                                        {
                                            if (upper.Equals("DARKMAGENTA"))
                                            {
                                                return KnownColor.DarkMagenta;
                                            }
                                            if (!upper.Equals("DEEPSKYBLUE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.DeepSkyBlue;
                                        }
                                    case 'E':
                                        {
                                            break;
                                        }
                                    case 'F':
                                        {
                                            if (upper.Equals("FLORALWHITE"))
                                            {
                                                return KnownColor.FloralWhite;
                                            }
                                            if (!upper.Equals("FORESTGREEN"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.ForestGreen;
                                        }
                                    case 'G':
                                        {
                                            if (!upper.Equals("GREENYELLOW"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.GreenYellow;
                                        }
                                    default:
                                        {
                                            if (chr == 'L')
                                            {
                                                if (upper.Equals("LIGHTSALMON"))
                                                {
                                                    return KnownColor.LightSalmon;
                                                }
                                                if (!upper.Equals("LIGHTYELLOW"))
                                                {
                                                    return KnownColor.UnknownColor;
                                                }
                                                return KnownColor.LightYellow;
                                            }
                                            else if (chr == 'N')
                                            {
                                                if (!upper.Equals("NAVAJOWHITE"))
                                                {
                                                    return KnownColor.UnknownColor;
                                                }
                                                return KnownColor.NavajoWhite;
                                            }
                                            break;
                                        }
                                }
                            }
                            else if (chr == 'S')
                            {
                                if (upper.Equals("SADDLEBROWN"))
                                {
                                    return KnownColor.SaddleBrown;
                                }
                                if (!upper.Equals("SPRINGGREEN"))
                                {
                                    break;
                                }
                                return KnownColor.SpringGreen;
                            }
                            else if (chr == 'T')
                            {
                                if (!upper.Equals("TRANSPARENT"))
                                {
                                    break;
                                }
                                return KnownColor.Transparent;
                            }
                            else if (chr == 'Y')
                            {
                                if (!upper.Equals("YELLOWGREEN"))
                                {
                                    break;
                                }
                                return KnownColor.YellowGreen;
                            }
                            else
                            {
                                break;
                            }
                            break;
                        }
                    case 12:
                        {
                            chr = upper[0];
                            if (chr <= 'D')
                            {
                                if (chr == 'A')
                                {
                                    if (!upper.Equals("ANTIQUEWHITE"))
                                    {
                                        break;
                                    }
                                    return KnownColor.AntiqueWhite;
                                }
                                else if (chr == 'D')
                                {
                                    if (!upper.Equals("DARKSEAGREEN"))
                                    {
                                        break;
                                    }
                                    return KnownColor.DarkSeaGreen;
                                }
                                else
                                {
                                    break;
                                }
                            }
                            else if (chr == 'L')
                            {
                                if (upper.Equals("LIGHTSKYBLUE"))
                                {
                                    return KnownColor.LightSkyBlue;
                                }
                                if (!upper.Equals("LEMONCHIFFON"))
                                {
                                    break;
                                }
                                return KnownColor.LemonChiffon;
                            }
                            else if (chr == 'M')
                            {
                                if (upper.Equals("MEDIUMORCHID"))
                                {
                                    return KnownColor.MediumOrchid;
                                }
                                if (upper.Equals("MEDIUMPURPLE"))
                                {
                                    return KnownColor.MediumPurple;
                                }
                                if (!upper.Equals("MIDNIGHTBLUE"))
                                {
                                    break;
                                }
                                return KnownColor.MidnightBlue;
                            }
                            else
                            {
                                break;
                            }
                        }
                    case 13:
                        {
                            chr = upper[0];
                            if (chr == 'D')
                            {
                                if (upper.Equals("DARKSLATEBLUE"))
                                {
                                    return KnownColor.DarkSlateBlue;
                                }
                                if (upper.Equals("DARKSLATEGRAY"))
                                {
                                    return KnownColor.DarkSlateGray;
                                }
                                if (upper.Equals("DARKGOLDENROD"))
                                {
                                    return KnownColor.DarkGoldenrod;
                                }
                                if (!upper.Equals("DARKTURQUOISE"))
                                {
                                    break;
                                }
                                return KnownColor.DarkTurquoise;
                            }
                            else if (chr == 'L')
                            {
                                if (upper.Equals("LIGHTSEAGREEN"))
                                {
                                    return KnownColor.LightSeaGreen;
                                }
                                if (!upper.Equals("LAVENDERBLUSH"))
                                {
                                    break;
                                }
                                return KnownColor.LavenderBlush;
                            }
                            else if (chr == 'P')
                            {
                                if (upper.Equals("PALEGOLDENROD"))
                                {
                                    return KnownColor.PaleGoldenrod;
                                }
                                if (upper.Equals("PALETURQUOISE"))
                                {
                                    return KnownColor.PaleTurquoise;
                                }
                                if (!upper.Equals("PALEVIOLETRED"))
                                {
                                    break;
                                }
                                return KnownColor.PaleVioletRed;
                            }
                            else
                            {
                                break;
                            }
                        }
                    case 14:
                        {
                            chr = upper[0];
                            switch (chr)
                            {
                                case 'B':
                                    {
                                        if (!upper.Equals("BLANCHEDALMOND"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.BlanchedAlmond;
                                    }
                                case 'C':
                                    {
                                        if (!upper.Equals("CORNFLOWERBLUE"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.CornflowerBlue;
                                    }
                                case 'D':
                                    {
                                        if (!upper.Equals("DARKOLIVEGREEN"))
                                        {
                                            return KnownColor.UnknownColor;
                                        }
                                        return KnownColor.DarkOliveGreen;
                                    }
                                default:
                                    {
                                        if (chr == 'L')
                                        {
                                            if (upper.Equals("LIGHTSLATEGRAY"))
                                            {
                                                return KnownColor.LightSlateGray;
                                            }
                                            if (!upper.Equals("LIGHTSTEELBLUE"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.LightSteelBlue;
                                        }
                                        else if (chr == 'M')
                                        {
                                            if (!upper.Equals("MEDIUMSEAGREEN"))
                                            {
                                                return KnownColor.UnknownColor;
                                            }
                                            return KnownColor.MediumSeaGreen;
                                        }
                                        break;
                                    }
                            }
                            break;
                        }
                    case 15:
                        {
                            if (upper.Equals("MEDIUMSLATEBLUE"))
                            {
                                return KnownColor.MediumSlateBlue;
                            }
                            if (upper.Equals("MEDIUMTURQUOISE"))
                            {
                                return KnownColor.MediumTurquoise;
                            }
                            if (!upper.Equals("MEDIUMVIOLETRED"))
                            {
                                break;
                            }
                            return KnownColor.MediumVioletRed;
                        }
                    case 16:
                        {
                            if (!upper.Equals("MEDIUMAQUAMARINE"))
                            {
                                break;
                            }
                            return KnownColor.MediumAquamarine;
                        }
                    case 17:
                        {
                            if (!upper.Equals("MEDIUMSPRINGGREEN"))
                            {
                                break;
                            }
                            return KnownColor.MediumSpringGreen;
                        }
                    case 20:
                        {
                            if (!upper.Equals("LIGHTGOLDENRODYELLOW"))
                            {
                                break;
                            }
                            return KnownColor.LightGoldenrodYellow;
                        }
                }
            }
            return KnownColor.UnknownColor;
        }

        internal static string MatchColor(string colorString, out bool isKnownColor, out bool isNumericColor, out bool isContextColor, out bool isScRgbColor)
        {
            string str = colorString.Trim();
            if ((str.Length == 4 || str.Length == 5 || str.Length == 7 || str.Length == 9) && str[0] == '#')
            {
                isNumericColor = true;
                isScRgbColor = false;
                isKnownColor = false;
                isContextColor = false;
                return str;
            }
            isNumericColor = false;
            if (!str.StartsWith("sc#", StringComparison.Ordinal))
            {
                isScRgbColor = false;
            }
            else
            {
                isNumericColor = false;
                isScRgbColor = true;
                isKnownColor = false;
                isContextColor = false;
            }
            if (!str.StartsWith("ContextColor ", StringComparison.OrdinalIgnoreCase))
            {
                isContextColor = false;
                isKnownColor = true;
                return str;
            }
            isContextColor = true;
            isScRgbColor = false;
            isKnownColor = false;
            return str;
        }
    }
}
