﻿using System;
using System.ComponentModel;
using Telerik.Documents.Media;

namespace Telerik.Documents.Core.Media.Internal
{
    internal static class Parsers
    {
        internal static Color ParseColor(string color, IFormatProvider formatProvider)
        {
            return Parsers.ParseColor(color, formatProvider, null);
        }

        internal static Color ParseColor(string color, IFormatProvider formatProvider, ITypeDescriptorContext context)
        {
            bool flag;
            bool flag1;
            bool flag2;
            bool flag3;
            string str = KnownColors.MatchColor(color, out flag3, out flag2, out flag1, out flag);
            if (!flag3 && !flag2 && !flag && !flag1)
            {
                throw new FormatException("Parsers_IllegalToken");
            }
            if (flag2)
            {
                return Parsers.ParseHexColor(str);
            }
            KnownColor knownColor = KnownColors.ColorStringToKnownColor(str);
            if (knownColor == KnownColor.UnknownColor)
            {
                throw new FormatException("Parsers_IllegalToken");
            }
            return Color.FromUInt32((uint)knownColor);
        }

        private static int ParseHexChar(char c)
        {
            int num = c;
            if (num >= 48 && num <= 57)
            {
                return num - 48;
            }
            if (num >= 97 && num <= 102)
            {
                return num - 97 + 10;
            }
            if (num < 65 || num > 70)
            {
                throw new FormatException("Parsers_IllegalToken");
            }
            return num - 65 + 10;
        }

        private static Color ParseHexColor(string trimmedColor)
        {
            int num;
            int num1;
            int num2;
            int num3 = 255;
            if (trimmedColor.Length > 7)
            {
                num3 = Parsers.ParseHexChar(trimmedColor[1]) * 16 + Parsers.ParseHexChar(trimmedColor[2]);
                num2 = Parsers.ParseHexChar(trimmedColor[3]) * 16 + Parsers.ParseHexChar(trimmedColor[4]);
                num1 = Parsers.ParseHexChar(trimmedColor[5]) * 16 + Parsers.ParseHexChar(trimmedColor[6]);
                num = Parsers.ParseHexChar(trimmedColor[7]) * 16 + Parsers.ParseHexChar(trimmedColor[8]);
            }
            else if (trimmedColor.Length > 5)
            {
                num2 = Parsers.ParseHexChar(trimmedColor[1]) * 16 + Parsers.ParseHexChar(trimmedColor[2]);
                num1 = Parsers.ParseHexChar(trimmedColor[3]) * 16 + Parsers.ParseHexChar(trimmedColor[4]);
                num = Parsers.ParseHexChar(trimmedColor[5]) * 16 + Parsers.ParseHexChar(trimmedColor[6]);
            }
            else if (trimmedColor.Length <= 4)
            {
                num2 = Parsers.ParseHexChar(trimmedColor[1]);
                num2 = num2 + num2 * 16;
                num1 = Parsers.ParseHexChar(trimmedColor[2]);
                num1 = num1 + num1 * 16;
                num = Parsers.ParseHexChar(trimmedColor[3]);
                num = num + num * 16;
            }
            else
            {
                num3 = Parsers.ParseHexChar(trimmedColor[1]);
                num3 = num3 + num3 * 16;
                num2 = Parsers.ParseHexChar(trimmedColor[2]);
                num2 = num2 + num2 * 16;
                num1 = Parsers.ParseHexChar(trimmedColor[3]);
                num1 = num1 + num1 * 16;
                num = Parsers.ParseHexChar(trimmedColor[4]);
                num = num + num * 16;
            }
            return Color.FromArgb((byte)num3, (byte)num2, (byte)num1, (byte)num);
        }
    }
}
