﻿using System;
using System.Globalization;

namespace Telerik.Documents.Core.Media.Internal
{
    internal class TokenizerHelper
    {
        private char _quoteChar;

        private char _argSeparator;

        private string _str;

        private int _strLen;

        private int _charIndex;

        internal int _currentTokenIndex;

        internal int _currentTokenLength;

        private bool _foundSeparator;

        internal bool FoundSeparator
        {
            get
            {
                return this._foundSeparator;
            }
        }

        internal TokenizerHelper(string str, IFormatProvider formatProvider)
        {
            this.Initialize(str, '\'', TokenizerHelper.GetNumericListSeparator(formatProvider));
        }

        internal TokenizerHelper(string str, char quoteChar, char separator)
        {
            this.Initialize(str, quoteChar, separator);
        }

        internal string GetCurrentToken()
        {
            if (this._currentTokenIndex < 0)
            {
                return null;
            }
            return this._str.Substring(this._currentTokenIndex, this._currentTokenLength);
        }

        internal static char GetNumericListSeparator(IFormatProvider provider)
        {
            char chr = ',';
            NumberFormatInfo instance = NumberFormatInfo.GetInstance(provider);
            if (instance.NumberDecimalSeparator.Length > 0 && chr == instance.NumberDecimalSeparator[0])
            {
                chr = ';';
            }
            return chr;
        }

        private void Initialize(string str, char quoteChar, char separator)
        {
            this._str = str;
            this._strLen = (str == null ? 0 : str.Length);
            this._currentTokenIndex = -1;
            this._quoteChar = quoteChar;
            this._argSeparator = separator;
            while (this._charIndex < this._strLen && char.IsWhiteSpace(this._str, this._charIndex))
            {
                this._charIndex++;
            }
        }

        internal void LastTokenRequired()
        {
            if (this._charIndex != this._strLen)
            {
                throw new InvalidOperationException("TokenizerHelperExtraDataEncountered");
            }
        }

        internal bool NextToken()
        {
            return this.NextToken(false);
        }

        internal bool NextToken(bool allowQuotedToken)
        {
            return this.NextToken(allowQuotedToken, this._argSeparator);
        }

        internal bool NextToken(bool allowQuotedToken, char separator)
        {
            this._currentTokenIndex = -1;
            this._foundSeparator = false;
            if (this._charIndex >= this._strLen)
            {
                return false;
            }
            char chr = this._str[this._charIndex];
            int num = 0;
            if (allowQuotedToken && chr == this._quoteChar)
            {
                num++;
                this._charIndex++;
            }
            int num1 = this._charIndex;
            int num2 = 0;
            while (this._charIndex < this._strLen)
            {
                chr = this._str[this._charIndex];
                if (num > 0)
                {
                    if (chr == this._quoteChar)
                    {
                        num--;
                        if (num == 0)
                        {
                            this._charIndex++;
                            break;
                        }
                    }
                }
                else if (char.IsWhiteSpace(chr) || chr == separator)
                {
                    if (chr != separator)
                    {
                        break;
                    }
                    this._foundSeparator = true;
                    break;
                }
                this._charIndex++;
                num2++;
            }
            if (num > 0)
            {
                throw new InvalidOperationException("TokenizerHelperMissingEndQuote");
            }
            this.ScanToNextToken(separator);
            this._currentTokenIndex = num1;
            this._currentTokenLength = num2;
            if (this._currentTokenLength < 1)
            {
                throw new InvalidOperationException("TokenizerHelperEmptyToken");
            }
            return true;
        }

        internal string NextTokenRequired()
        {
            if (!this.NextToken(false))
            {
                throw new InvalidOperationException("TokenizerHelperPrematureStringTermination");
            }
            return this.GetCurrentToken();
        }

        internal string NextTokenRequired(bool allowQuotedToken)
        {
            if (!this.NextToken(allowQuotedToken))
            {
                throw new InvalidOperationException("TokenizerHelperPrematureStringTermination");
            }
            return this.GetCurrentToken();
        }

        private void ScanToNextToken(char separator)
        {
            if (this._charIndex < this._strLen)
            {
                char chr = this._str[this._charIndex];
                if (chr != separator && !char.IsWhiteSpace(chr))
                {
                    throw new InvalidOperationException("TokenizerHelperExtraDataEncountered");
                }
                int num = 0;
                while (this._charIndex < this._strLen)
                {
                    chr = this._str[this._charIndex];
                    if (chr != separator)
                    {
                        if (!char.IsWhiteSpace(chr))
                        {
                            break;
                        }
                        this._charIndex++;
                    }
                    else
                    {
                        this._foundSeparator = true;
                        num++;
                        this._charIndex++;
                        if (num <= 1)
                        {
                            continue;
                        }
                        throw new InvalidOperationException("TokenizerHelperEmptyToken");
                    }
                }
                if (num > 0 && this._charIndex >= this._strLen)
                {
                    throw new InvalidOperationException("TokenizerHelperEmptyToken");
                }
            }
        }
    }
}