﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Model
{
    /// <summary>
    /// Provides methods for converting from standard PaperTypes to Size
    /// </summary>
    public static class PaperTypeConverter
    {
        private static readonly Dictionary<PaperTypes, Size> paperTypeSizes = new Dictionary<PaperTypes, Size>();

        static PaperTypeConverter()
        {
            paperTypeSizes[PaperTypes.A0] = new Size(3173.0f, 4491.0f);
            paperTypeSizes[PaperTypes.A1] = new Size(2245.0f, 3173.0f);
            paperTypeSizes[PaperTypes.A2] = new Size(1587.0f, 2245.0f);
            paperTypeSizes[PaperTypes.A3] = new Size(1123.0f, 1587.0f);
            paperTypeSizes[PaperTypes.A4] = new Size(793.0f, 1123.0f);
            paperTypeSizes[PaperTypes.A5] = new Size(560.0f, 793.0f);
            paperTypeSizes[PaperTypes.RA0] = new Size(3251.0f, 4611.0f);
            paperTypeSizes[PaperTypes.RA1] = new Size(2305.0f, 3251.0f);
            paperTypeSizes[PaperTypes.RA2] = new Size(1625.0f, 2305.0f);
            paperTypeSizes[PaperTypes.RA3] = new Size(1153.0f, 1625.0f);
            paperTypeSizes[PaperTypes.RA4] = new Size(812.0f, 1153.0f);
            paperTypeSizes[PaperTypes.RA5] = new Size(457.0f, 812.0f);
            paperTypeSizes[PaperTypes.B0] = new Size(3780.0f, 5344.0f);
            paperTypeSizes[PaperTypes.B1] = new Size(2672.0f, 3780.0f);
            paperTypeSizes[PaperTypes.B2] = new Size(5669.0f, 1889.0f);
            paperTypeSizes[PaperTypes.B3] = new Size(1335.0f, 1889.0f);
            paperTypeSizes[PaperTypes.B4] = new Size(972.0f, 1376.0f);
            paperTypeSizes[PaperTypes.B5] = new Size(688.0f, 972.0f);
            paperTypeSizes[PaperTypes.Quarto] = new Size(768.0f, 960.0f);
            paperTypeSizes[PaperTypes.Foolscap] = new Size(768.0f, 1248.0f);
            paperTypeSizes[PaperTypes.Executive] = new Size(720.0f, 960.0f);
            paperTypeSizes[PaperTypes.GovernmentLetter] = new Size(1008.0f, 768.0f);
            paperTypeSizes[PaperTypes.Letter] = new Size(816.0f, 1056.0f);
            paperTypeSizes[PaperTypes.Legal] = new Size(816.0f, 1344.0f);
            paperTypeSizes[PaperTypes.Ledger] = new Size(1632.0f, 1056.0f);
            paperTypeSizes[PaperTypes.Tabloid] = new Size(1056.0f, 1632.0f);
            paperTypeSizes[PaperTypes.Post] = new Size(1501.0f, 1848.0f);
            paperTypeSizes[PaperTypes.Crown] = new Size(1920.0f, 1440.0f);
            paperTypeSizes[PaperTypes.LargePost] = new Size(1584.0f, 2016.0f);
            paperTypeSizes[PaperTypes.Demy] = new Size(1680.0f, 2112.0f);
            paperTypeSizes[PaperTypes.Medium] = new Size(1728.0f, 2208.0f);
            paperTypeSizes[PaperTypes.Royal] = new Size(1920.0f, 2400.0f);
            paperTypeSizes[PaperTypes.Elephant] = new Size(2087.0f, 2688.0f);
            paperTypeSizes[PaperTypes.DoubleDemy] = new Size(2256.0f, 3360.0f);
            paperTypeSizes[PaperTypes.QuadDemy] = new Size(3360.0f, 4320.0f);
            paperTypeSizes[PaperTypes.STMT] = new Size(528.0f, 816.0f);
            paperTypeSizes[PaperTypes.Folio] = new Size(816.0f, 1248.0f);
            paperTypeSizes[PaperTypes.Statement] = new Size(528.0f, 816.0f);
            paperTypeSizes[PaperTypes.Size10x14] = new Size(960.0f, 1344.0f);
        }

        /// <summary>
        /// Converts the specified PaperType enumeration to a pair of pixel values in Size.
        /// </summary>
        /// <param name="type">PaperType</param>
        /// <returns></returns>
        public static Size ToSize(PaperTypes type)
        {
            Size paperSize;
            if (paperTypeSizes.TryGetValue(type, out paperSize))
            {
                return paperSize;
            }

            throw new ArgumentException("Invalid PaperType.");
        }
    }
}
