﻿using System;

namespace Telerik.Windows.Documents.Model
{
    /// <summary>
    /// Provides enumeration for the most commonly used paper sizes.
    /// </summary>
    public enum PaperTypes
    {
        /// <summary>
        /// Identifies a paper sheet size of 33.1 inches x 46.8 inches.
        /// </summary>
        A0 = 1,

        /// <summary>
        /// Identifies a paper sheet size of 23.4 inches x 33.1 inches.
        /// </summary>
        A1 = 2,

        /// <summary>
        /// Identifies a paper sheet size of 16.5 inches x 23.4 inches.
        /// </summary>
        A2 = 3,

        /// <summary>
        /// Identifies a paper sheet size of 11.7 inches x 16.5 inches.
        /// </summary>
        A3 = 4,

        /// <summary>
        /// Identifies a paper sheet size of 8.3 inches x 11.7 inches.
        /// </summary>
        A4 = 5,

        /// <summary>
        /// Identifies a paper sheet size of 5.8 inches x 8.3 inches.
        /// </summary>
        A5 = 6,

        /// <summary>
        /// Identifies a paper sheet size of 33.9 inches x 48 inches.
        /// </summary>
        RA0 = 7,

        /// <summary>
        /// Identifies a paper sheet size of 24 inches x 33.9 inches.
        /// </summary>
        RA1 = 8,

        /// <summary>
        /// Identifies a paper sheet size of 16.9 inches x 24 inches.
        /// </summary>
        RA2 = 9,

        /// <summary>
        /// Identifies a paper sheet size of 12 inches x 16.9 inches.
        /// </summary>
        RA3 = 10,

        /// <summary>
        /// Identifies a paper sheet size of 8.5 inches x 12 inches.
        /// </summary>
        RA4 = 11,

        /// <summary>
        /// Identifies a paper sheet size of 4.8 inches x 8.5 inches.
        /// </summary>
        RA5 = 12,

        /// <summary>
        /// Identifies a paper sheet size of 39.4 inches x 55.7 inches.
        /// </summary>
        B0 = 13,

        /// <summary>
        /// Identifies a paper sheet size of 27.8 inches x 39.4 inches.
        /// </summary>
        B1 = 14,

        /// <summary>
        /// Identifies a paper sheet size of 59.1 inches x 19.7 inches.
        /// </summary>
        B2 = 15,

        /// <summary>
        /// Identifies a paper sheet size of 13.9 inches x 19.7 inches.
        /// </summary>
        B3 = 16,

        /// <summary>
        /// Identifies a paper sheet size of 10.1 inches x 14.3 inches.
        /// </summary>
        B4 = 17,

        /// <summary>
        /// Identifies a paper sheet size of 7.2 inches x 10.1 inches.
        /// </summary>
        B5 = 18,

        /// <summary>
        /// Identifies a paper sheet size of 8 inches x 10 inches.
        /// </summary>
        Quarto = 19,

        /// <summary>
        /// Identifies a paper sheet size of 8 inches x 13 inches.
        /// </summary>
        Foolscap = 20,

        /// <summary>
        /// Identifies a paper sheet size of 7.5 inches x 10 inches.
        /// </summary>
        Executive = 21,

        /// <summary>
        /// Identifies a paper sheet size of 10.5 inches x 8 inches.
        /// </summary>
        GovernmentLetter = 22,

        /// <summary>
        /// Identifies a paper sheet size of 8.5 inches x 11 inches.
        /// </summary>
        Letter = 23,

        /// <summary>
        /// Identifies a paper sheet size of 8.5 inches x 14 inches.
        /// </summary>
        Legal = 24,

        /// <summary>
        /// Identifies a paper sheet size of 17 inches x 11 inches.
        /// </summary>
        Ledger = 25,

        /// <summary>
        /// Identifies a paper sheet size of 11 inches x 17 inches.
        /// </summary>
        Tabloid = 26,

        /// <summary>
        /// Identifies a paper sheet size of 15.6 inches x 19.2 inches.
        /// </summary>
        Post = 27,

        /// <summary>
        /// Identifies a paper sheet size of 20 inches x 15 inches.
        /// </summary>
        Crown = 28,

        /// <summary>
        /// Identifies a paper sheet size of 16.5 inches x 21 inches.
        /// </summary>
        LargePost = 29,

        /// <summary>
        /// Identifies a paper sheet size of 17.5 inches x 22 inches.
        /// </summary>
        Demy = 30,

        /// <summary>
        /// Identifies a paper sheet size of 18 inches x 23 inches.
        /// </summary>
        Medium = 31,

        /// <summary>
        /// Identifies a paper sheet size of 20 inches x 25 inches.
        /// </summary>
        Royal = 32,

        /// <summary>
        /// Identifies a paper sheet size of 21.7 inches x 28 inches.
        /// </summary>
        Elephant = 33,

        /// <summary>
        /// Identifies a paper sheet size of 23.5 inches x 35 inches.
        /// </summary>
        DoubleDemy = 34,

        /// <summary>
        /// Identifies a paper sheet size of 35 inches x 45 inches.
        /// </summary>
        QuadDemy = 35,

        /// <summary>
        /// Identifies a paper sheet size of 5.5 inches x 8.5 inches.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "STMT")]
        STMT = 36,

        /// <summary>
        /// Identifies a paper sheet size of 8.5 inches x 13 inches.
        /// </summary>
        Folio = 37,

        /// <summary>
        /// Identifies a paper sheet size of 5.5 inches x 8.5 inches.
        /// </summary>
        Statement = 38,

        /// <summary>
        /// Identifies a paper sheet size of 10 inches x 14 inches.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "x")]
        Size10x14 = 39,
    }
}
