﻿namespace Telerik.Documents.Primitives
{
    /// <summary>
    /// Point structure in a 2D space, defined by two double values for the coordinates.
    /// </summary>
    public struct Point
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Point"/> structure.
        /// </summary>
        /// <param name="x">The X coordinate.</param>
        /// <param name="y">The Y coordinate.</param>
        public Point(double x, double y)
        {
            this.X = x;
            this.Y = y;
        }

        /// <summary>
        /// The X coordinate.  Default value is 0.
        /// </summary>
        public double X
        {
            get;
            set;
        }

        /// <summary>
        /// The Y coordinate.  Default value is 0.
        /// </summary>
        public double Y
        {
            get;
            set;
        }

        ///// <summary>
        ///// Offset - update the location by adding offsetX to X and offsetY to Y
        ///// </summary>
        ///// <param name="offsetX"> The offset in the x dimension </param>
        ///// <param name="offsetY"> The offset in the y dimension </param>
        //public void Offset(double offsetX, double offsetY)
        //{
        //    this.X += offsetX;
        //    this.Y += offsetY;
        //}

        /// <summary>
        /// Compares two <see cref="Point"/> instances for exact equality.
        /// </summary>
        /// <param name='point1'>The first <see cref="Point"/> to compare.</param>
        /// <param name='point2'>The second <see cref="Point"/> to compare.</param>
        /// <returns> true if the two <see cref="Point"/> instances are exactly equal; false otherwise. </returns>
        public static bool operator ==(Point point1, Point point2)
        {
            return point1.X == point2.X &&
                   point1.Y == point2.Y;
        }

        /// <summary>
        /// Compares two <see cref="Point"/> instances for exact inequality.
        /// </summary>
        /// <param name='point1'>The first <see cref="Point"/> to compare</param>
        /// <param name='point2'>The second <see cref="Point"/> to compare</param>
        /// <returns>true if the two <see cref="Point"/> instances are exactly unequal; false otherwise.</returns>
        public static bool operator !=(Point point1, Point point2)
        {
            return !(point1 == point2);
        }

        /// <summary>
        /// Returns a hash code for this instance.
        /// </summary>
        /// <returns>
        /// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
        /// </returns>
        public override int GetHashCode()
        {
            return this.X.GetHashCode() ^ this.Y.GetHashCode();
        }

        /// <summary>
        /// Determines whether the specified <see cref="System.Object" />, is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object" /> to compare with this instance.</param>
        /// <returns>
        ///   <c>true</c> if the specified <see cref="System.Object" /> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is Point))
            {
                return false;
            }

            return this.Equals((Point)obj);
        }

        private bool Equals(Point other)
        {
            return this.X.Equals(other.X) && this.Y.Equals(other.Y);
        }

        ///// <summary>
        ///// Operator Point * Matrix
        ///// </summary>
        //public static Point operator *(Point point, Matrix matrix)
        //{
        //    return matrix.Transform(point);
        //}

        ///// <summary>
        ///// Multiply: Point * Matrix
        ///// </summary>
        //public static Point Multiply(Point point, Matrix matrix)
        //{
        //    return matrix.Transform(point);
        //}

        ///// <summary>
        ///// Explicit conversion to Size.  Note that since Size cannot contain negative values,
        ///// the resulting size will contains the absolute values of X and Y
        ///// </summary>
        ///// <returns>
        ///// Size - A Size equal to this Point
        ///// </returns>
        ///// <param name="point"> Point - the Point to convert to a Size </param>
        //public static explicit operator Size(Point point)
        //{
        //    return new Size(Math.Abs(point.X), Math.Abs(point.X));
        //}        
    }
}
