﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Utilities
{
    internal static class CollectionsExtensions
    {
        public static void ForEach<T>(this IEnumerable<T> enumeration, Action<T> action)
        {
            foreach (T item in enumeration)
            {
                action(item);
            }
        }

        public static Dictionary<TKey, TValue> ToDictionary<TKey, TValue>(this IEnumerable<TKey> source, Func<TKey, TValue> converter)
        {
            Dictionary<TKey, TValue> result = new Dictionary<TKey, TValue>();
            foreach (TKey item in source)
            {
                result[item] = converter(item);
            }
            return result;
        }

        public static T1 FindElement<T1, T2>(IList<T1> collection, T2 index, CompareDelegate<T1, T2> comparer) where T1 : class
        {
            if (collection.Count == 0)
            {
                return null;
            }

            return FindElement(collection, 0, collection.Count - 1, index, comparer);
        }

        private static T1 FindElement<T1, T2>(IList<T1> collection, int lo, int hi, T2 element, CompareDelegate<T1, T2> comparer) where T1 : class
        {
            if (hi < lo)
            {
                int index = Math.Max(0, Math.Min(hi, collection.Count));
                return collection[index];
            }

            int mid = lo + (hi - lo) / 2;
            T1 line = collection[mid];
            int res = comparer(line, element);

            if (res == 0)
            {
                return line;
            }

            if (res > 0)
            {
                return FindElement(collection, mid + 1, hi, element, comparer);
            }

            return FindElement(collection, lo, mid - 1, element, comparer);
        }
    }
}
