﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents;
#else
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Utilities
{
    internal static class Extensions
    {
        private const int BufferSize = 1024 * 1024 * 10; 
       
        public static byte[] ReadAllBytes(this Stream reader)
        {
            if (!reader.CanRead)
            {
                return null;
            }

            if (reader.CanSeek)
            {
                reader.Seek(0, SeekOrigin.Begin);
            }

            using (MemoryStream ms = new MemoryStream())
            {
                reader.CopyTo(ms);

                byte[] result = null;

                if (ms.Length == ms.Capacity)
                {
                    result = ms.GetBuffer();
                }
                else
                {
                    result = ms.ToArray();
                }

                return result;
            }
        }

        public static bool IsNullEmptyOrWhiteSpace(string str)
        {
            if (string.IsNullOrEmpty(str))
            {
                return true;
            }
            return string.IsNullOrWhiteSpace(str);
        }

#if !NETSTANDARD
        public static Size GetSize(this TextBlock block)
        {
#if SILVERLIGHT
            return new Size(block.ActualWidth, block.ActualHeight);
#else
            block.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
            return block.DesiredSize;
#endif
        }

        public static double GetBaselineOffset(this TextBlock textBlock)
        {
#if WPF
            textBlock.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
#endif
            return textBlock.BaselineOffset;
        }

        internal static string GetFontFileName(this GlyphTypeface typeFace)
        {
#if WPF
            return Path.GetFileName(typeFace.FontUri.AbsolutePath);
#else
            return typeFace.FontFileName;
#endif
        }
#endif
    }
}
