﻿using System;
using System.Globalization;

namespace Telerik.Windows.Documents.Utilities
{
    internal class Interval
    {
        private readonly double start;
        private readonly double end;

        public Interval(double start, double end)
        {
            Guard.ThrowExceptionIfTrue(start > end, "start cannot be bigger than end!");

            this.start = start;
            this.end = end;
        }

        public double Start
        {
            get
            {
                return this.start;
            }
        }
        public double End
        {
            get
            {
                return this.end;
            }
        }

        public override bool Equals(object obj)
        {
            Interval other = obj as Interval;

            if (other == null)
            {
                return false;
            }

            return this.Start.Equals(other.Start) && this.End.Equals(other.End);
        }

        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.Start.GetHashCode(), this.End.GetHashCode());
        }

        public override string ToString()
        {
            return String.Format("[{0}{1}{2}]", this.Start, CultureInfo.CurrentCulture.TextInfo.ListSeparator, this.End);
        }
    }
}
