﻿using System;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Utilities
{
    internal static class TextHelper
    {
        private static readonly char LineFeed = '\n';
        private static readonly char CarrigeReturn = '\r';

        public static string SanitizeNewLines(string input)
        {
            StringBuilder result = new StringBuilder(input.Length);
            bool hasSeenCr = false;

            for (int i = 0, l = input.Length; i < l; ++i)
            {
                if (input[i] == CarrigeReturn)
                {
                    if (hasSeenCr)
                    {
                        result.Append(Environment.NewLine);
                    }

                    hasSeenCr = true;
                }
                else if (input[i] == LineFeed)
                {
                    result.Append(Environment.NewLine);
                    hasSeenCr = false;
                }
                else
                {
                    if (hasSeenCr)
                    {
                        result.Append(Environment.NewLine);
                        hasSeenCr = false;
                    }

                    result.Append(input[i]);
                }
            }

            if (hasSeenCr)
            {
                result.Append(Environment.NewLine);
            }

            return result.ToString();
        }

        public static string RemoveNewLines(string input)
        {
            StringBuilder result = new StringBuilder(input.Length);

            for (int i = 0, l = input.Length; i < l; ++i)
            {
                if (input[i] != CarrigeReturn && input[i] != LineFeed)
                {
                    result.Append(input[i]);
                }
            }

            return result.ToString();
        }

        public static string[] GetLines(string input)
        {
            input = TextHelper.SanitizeNewLines(input);
            string[] lines = input.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
            return lines;
        }

        public static string AddSlashes(string source)
        {
            if (source == null)
            {
                return string.Empty;
            }

            StringBuilder sb = new StringBuilder(source.Length);
            for (int i = 0, l = source.Length; i < l; ++i)
            {
                if (source[i] == '\\' || source[i] == '"')
                {
                    sb.Append('\\');
                }
                sb.Append(source[i]);
            }
            return sb.ToString();
        }
    }
}
