﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Contexts;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Relationships;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;

namespace Telerik.Windows.Documents.Core.Tests.Common.FormatProviders.OpenXml.Model
{
    [TestClass]
    public class OpenXmlPartsTests
    {
        [TestMethod]
        public void GetRelationshipRelativeTargetAlgorithmTest()
        {
            string workbookPartName = @"/xl/workbook.xml";
            string worksheetPartName = @"/xl/worksheets/sheet1.xml";
            string stylesPartName = @"/xl/styles.xml";
            string sharedStringsPartName = @"/xl/sharedStrings.xml";
            string themePartName = @"/xl/theme/theme1.xml";
            string drawingPartName = @"/xl/drawings/drawing1.xml";
            string resourcePartName = @"/xl/media/media1.png";


            string mainRelsPath = @"/_rels/.rels";

            AssertPath(mainRelsPath, workbookPartName, "xl/workbook.xml");
            AssertPath(mainRelsPath, worksheetPartName, "xl/worksheets/sheet1.xml");
            AssertPath(mainRelsPath, stylesPartName, "xl/styles.xml");
            AssertPath(mainRelsPath, sharedStringsPartName, "xl/sharedStrings.xml");
            AssertPath(mainRelsPath, themePartName, "xl/theme/theme1.xml");
            AssertPath(mainRelsPath, drawingPartName, "xl/drawings/drawing1.xml");
            AssertPath(mainRelsPath, resourcePartName, "xl/media/media1.png");


            string workbookRelsPath = @"\xl\_rels/workbook.xml.rels";

            AssertPath(workbookRelsPath, stylesPartName, "styles.xml");
            AssertPath(workbookRelsPath, sharedStringsPartName, "sharedStrings.xml");
            AssertPath(workbookRelsPath, worksheetPartName, "worksheets/sheet1.xml");
            AssertPath(workbookRelsPath, themePartName, "theme/theme1.xml");

            string drawingRelsPath = @"/xl/drawings/_rels/drawing1.xml.rels"; // alt directory separator.

            AssertPath(drawingRelsPath, resourcePartName, "../media/media1.png");

            string sheetRelsPath = @"\xl\worksheets\_rels\sheet1.xml.rels";

            AssertPath(sheetRelsPath, drawingPartName, "../drawings/drawing1.xml");

            AssertPath(sheetRelsPath, @"/xl/worktest/mywork.xml", "../worktest/mywork.xml");
        }

        [TestMethod]
        public void RelationshipRelativePath_ForExternalTarget_Hyperlink_Test()
        {
            RelationshipInfo relationshipInfo = new RelationshipInfo()
            {
                Id = "rId1",
                Type = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink",
                Target = "telerik.com",
                TargetMode = TargetModes.External
            };

            TestOpenXmlPartsManager partsManager = new TestOpenXmlPartsManager();

            RelationshipsPart relationshipsPart = new RelationshipsPart(partsManager, @"\xl\worksheets\_rels\sheet1.xml.rels");

            RelationshipElement relationshipElement = new RelationshipElement(partsManager);
            relationshipElement.Part = relationshipsPart;
            relationshipElement.SetRelationshipInfo(relationshipInfo);

            Assert.AreEqual(relationshipElement.Id, relationshipInfo.Id);
            Assert.AreEqual(relationshipElement.Type, relationshipInfo.Type);
            Assert.AreEqual(relationshipElement.Target, relationshipInfo.Target);
            Assert.AreEqual(relationshipElement.TargetMode, relationshipInfo.TargetMode);
        }

        private static void AssertPath(string mainRelsPath, string workbookPartName, string expectedResult)
        {
            string result = OpenXmlPartsManager.GetRelationshipRelativeTarget(mainRelsPath, workbookPartName);
            Assert.AreEqual(result, expectedResult);
        }
    }
}
