﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Common.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;

namespace Telerik.Windows.Documents.Core.Tests.Common.FormatProviders.OpenXml.Model.Types
{
    [TestClass]
    public class UniversalMeasureToDipConverterTests
    {
        [TestMethod]
        public void ConvertFromString_WithEmptyString_ReturnsZero()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("");

            Assert.AreEqual(0, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithInvalidNumber_ReturnsZero()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("auto");

            Assert.AreEqual(0, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumber_ReturnsZero()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("5");

            Assert.AreEqual(0, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInPicasWithPiSuffix_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("6pi");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInPicasWithPcSuffix_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("6pc");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInPoints_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("72pt");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInInches_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("1in");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInMm_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("25.4mm");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNumberInCm_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("2.54cm");

            Assert.AreEqual(96, actual);
        }

        [TestMethod]
        public void ConvertFromString_WithNegativeNumberWithSuffix_ReturnsTheValueInPixels()
        {
            IStringConverter<double> twipMeasureConverter = new UniversalMeasureToDipConverter();
            double actual = twipMeasureConverter.ConvertFromString("-2.54cm");

            Assert.AreEqual(-96, actual);
        }
    }
}
