﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Shapes
{
    [TestClass]
    public class ImageTests
    {
        [TestMethod]
        public void Clone_WhenImageSourceShouldBeCloned_CreatesCloning()
        {
            // Arrange
            Image image = new Image();
            image.Height = 42;
            image.ImageSource = new ImageSource(new byte[] { 1, 2, 3 }, "jpeg");
            image.IsHorizontallyFlipped = true;
            image.IsVerticallyFlipped = true;
            image.Name = "sample name";
            image.RotationAngle = 17;
            image.Width = 43;

            // Act
            Image clonedImage = image.Clone(true);

            // Assert
            Assert.AreNotSame(image, clonedImage);
            Assert.AreNotSame(image.ImageSource, clonedImage.ImageSource);

            Assert.IsTrue(clonedImage.LockAspectRatio);
            Assert.AreEqual(42, clonedImage.Height);
            ImageSourceAssert.AreEqual(new ImageSource(new byte[] { 1, 2, 3 }, "jpeg"), clonedImage.ImageSource);
            Assert.AreEqual(true, clonedImage.IsHorizontallyFlipped);
            Assert.AreEqual(true, clonedImage.IsVerticallyFlipped);
            Assert.AreEqual("sample name", clonedImage.Name);
            Assert.AreEqual(17, clonedImage.RotationAngle);
            Assert.AreEqual(43, clonedImage.Width);
        }

        [TestMethod]
        public void Clone_WhenImageSourceShouldNotBeCloned_CreatesCloning()
        {
            // Arrange
            Image image = new Image();
            image.Height = 42;
            image.ImageSource = new ImageSource(new byte[] { 1, 2, 3 }, "jpeg");
            image.IsHorizontallyFlipped = true;
            image.IsVerticallyFlipped = true;
            image.Name = "sample name";
            image.RotationAngle = 17;
            image.Width = 43;

            // Act
            Image clonedImage = image.Clone(false);

            // Assert
            Assert.AreNotSame(image, clonedImage);
            Assert.AreSame(image.ImageSource, clonedImage.ImageSource);

            Assert.IsTrue(clonedImage.LockAspectRatio);
            Assert.AreEqual(42, clonedImage.Height);
            Assert.AreEqual(true, clonedImage.IsHorizontallyFlipped);
            Assert.AreEqual(true, clonedImage.IsVerticallyFlipped);
            Assert.AreEqual("sample name", clonedImage.Name);
            Assert.AreEqual(17, clonedImage.RotationAngle);
            Assert.AreEqual(43, clonedImage.Width);
        }
    }
}
