﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Utilities;
using Matrix = System.Windows.Media.Matrix;

#if WPF
using RadMatrix = Telerik.Windows.Documents.Core.Data.Matrix;
#endif

namespace Telerik.Windows.Documents.Core.Tests.MatrixTests
{
    [TestClass]
    public class TestMatrix
    {
#if WPF
        private void AssertAreEquals(Matrix m, RadMatrix rm)
        {
            Assert.AreEqual(m.M11, rm.M11);
            Assert.AreEqual(m.M12, rm.M12);
            Assert.AreEqual(m.M21, rm.M21);
            Assert.AreEqual(m.M22, rm.M22);
            Assert.AreEqual(m.OffsetX, rm.OffsetX);
            Assert.AreEqual(m.OffsetY, rm.OffsetY);
        }
        
////#if DEBUG
////        [TestMethod]
////#endif
        public void TestMatrixScale()
        {
            Matrix m = Matrix.Identity;
            m.Scale(100, 100);

            RadMatrix rm = RadMatrix.Identity;
            rm.Scale(100, 100);

            AssertAreEquals(m, rm);

            m = new Matrix(0.001, 0, 0.000212557, 0.001, 0, 0);
            rm = new RadMatrix(0.001, 0, 0.000212557, 0.001, 0, 0);

            m.Scale(50, 50);
            rm.Scale(50, 50);

            AssertAreEquals(m, rm);

            m = new Matrix(1, 0, 1, 1, 0, 0);
            rm = new RadMatrix(1, 0, 1, 1, 0, 0);

            m.Scale(10, -10);
            rm.Scale(10, -10);

            AssertAreEquals(m, rm);
        }
#endif

        [TestMethod]
        public void ScaleAtMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.ScaleMatrixAt(10, 3, 5, 7);
            Matrix expected = new Matrix(10, 6, 30, 12, 5, 4);

            Assert.AreEqual(expected, actual);
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void ScaleMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.ScaleMatrix(10, 3);
            Matrix expected = new Matrix(10, 6, 30, 12, 50, 18);

            Assert.AreEqual(expected, actual);
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void RotateAtMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.RotateMatrixAt(32, 5, 7);
            Matrix expected = new Matrix(-0.211790432309984, 2.22601545654606, 0.424467231536458, 4.98195017732532, 5.52991926423321, 6.15195190384357);

            double tolerance = 1E-14;
            Assert.IsTrue(AreAlmostEqual(expected, actual, tolerance));
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void RotateMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.RotateMatrix(32);
            Matrix expected = new Matrix(-0.211790432309984, 2.22601545654606, 0.424467231536458, 4.98195017732532, 1.0607248953829, 7.73788489810458);

            double tolerance = 1E-14;
            Assert.IsTrue(AreAlmostEqual(expected, actual, tolerance));
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void TranslateMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.TranslateMatrix(4, 5);
            Matrix expected = new Matrix(1, 2, 3, 4, 9, 11);

            Assert.AreEqual(expected, actual);
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void MultiplyByMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.MultiplyBy(m);
            Matrix expected = new Matrix(7, 10, 15, 22, 28, 40);

            Assert.AreEqual(expected, actual);
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        [TestMethod]
        public void InverseMatrixTest()
        {
            Matrix m = new Matrix(1, 2, 3, 4, 5, 6);

            Matrix actual = m.InverseMatrix();
            Matrix expected = new Matrix(-2, 1, 1.5, -0.5, 1, -2);

            Assert.AreEqual(expected, actual);
            Assert.AreEqual(new Matrix(1, 2, 3, 4, 5, 6), m);
        }

        private static bool AreAlmostEqual(Matrix first, Matrix second, double epsilon)
        {
            return AreAlmostEqual(first.M11, second.M11, epsilon) &&
                AreAlmostEqual(first.M12, second.M12, epsilon) &&
                AreAlmostEqual(first.M21, second.M21, epsilon) &&
                AreAlmostEqual(first.M22, second.M22, epsilon) &&
                AreAlmostEqual(first.OffsetX, second.OffsetX, epsilon) &&
                AreAlmostEqual(first.OffsetY, second.OffsetY, epsilon);
        }

        private static bool AreAlmostEqual(double first, double second, double epsilon)
        {
            return System.Math.Abs(first - second) <= epsilon;
        }
    }
}
