﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using FontReaderTests.Utils;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;

namespace Telerik.Windows.Documents.Core.Tests
{
    [TestClass]
    public class TestTextMeasures
    {
        private readonly TextMeasurer textMeasurer = new TextMeasurer();

        private bool AreClose(double d1, double d2)
        {
            return Math.Abs(d1 - d2) < 0.6;
        }

        private bool AreClose(Size s1, Size s2)
        {
            return AreClose(s1.Width, s2.Width) && AreClose(s1.Height, s2.Height);
        }

        //[TestMethod]
        public void TestMeasureCorrectnessDifferentFontDescriptors()
        {
            double fontSize = 12;
            Helper.ExecuteActionWithDifferentFontDescriptors(new Action<FontDescriptor>(fd =>
            {
                string text = "sample";

                TextBlock tb = Helper.GetTextBlock(fd);
                tb.Text = text;
                tb.FontSize = fontSize;

                Size expectedSize = tb.GetSize();
                TextMeasurementInfo measurement = this.textMeasurer.MeasureText(fd, text, fontSize);

                Assert.IsTrue(AreClose(expectedSize, measurement.Size));
                Assert.IsTrue(AreClose(tb.BaselineOffset, tb.BaselineOffset));
            }), Helper.STANDARD_FONT_FAMILIES);
        }

        //[TestMethod]
        public void TestMeasureCorrectnessWithFallback()
        {
            double fontSize = 12;
            Helper.ExecuteActionWithDifferentFontDescriptors(new Action<FontDescriptor>(fd =>
            {
                Helper.ExecuteActionWithDifferentTexts(new Action<string>(text =>
                {
                    TextBlock tb = Helper.GetTextBlock(fd);
                    tb.Text = text;
                    tb.FontSize = fontSize;

                    Size expectedSize = tb.GetSize();
                    TextMeasurementInfo measurement = this.textMeasurer.MeasureText(fd, text, fontSize);

                    Assert.IsTrue(AreClose(expectedSize, measurement.Size));
                    Assert.IsTrue(AreClose(tb.BaselineOffset, tb.BaselineOffset));
                }));
            }), Helper.FALLBACK_FONT_FAMILIES);
        }

        //[TestMethod]
        public void TestMeasureCorrectnessWithUnknownFont()
        {
            double fontSize = 12;
            Helper.ExecuteActionWithDifferentFontStyles(new Action<FontDescriptor>(fd =>
            {
                Helper.ExecuteActionWithDifferentTexts(new Action<string>(text =>
                {
                    TextBlock tb = Helper.GetTextBlock(fd);
                    tb.Text = text;
                    tb.FontSize = fontSize;
                    Size expectedSize = tb.GetSize();
                    Size size = this.textMeasurer.MeasureText(fd, text, fontSize).Size;

                    Assert.IsTrue(AreClose(expectedSize, size));
                }));
            }), "SomeFontThatIsNotPresentForSure");
        }

        public void TestMeasureSpeedAndCorrectnessDifferentFontDescriptors()
        {
            double fontSize = 12;

            Helper.ExecuteActionWithDifferentFontDescriptors(new Action<FontDescriptor>(fd =>
            {
                SnippetPerformanceCounterRelease textBlockCounter = new SnippetPerformanceCounterRelease();
                SnippetPerformanceCounterRelease measurerCounter = new SnippetPerformanceCounterRelease();
                  
                TextBlock tb = Helper.GetTextBlock(fd);
                tb.FontSize = fontSize;
                foreach (string word in Helper.SAMPLE_WORDS)
                {
                    textBlockCounter.Start();
                    tb.Text = word;
                    Size expectedSize = tb.GetSize();
                    textBlockCounter.Stop();

                    measurerCounter.Start();
                    TextMeasurementInfo measurement = this.textMeasurer.MeasureText(fd, word, fontSize);
                    measurerCounter.Stop();

                    Assert.IsTrue(AreClose(expectedSize, measurement.Size));
                    Assert.IsTrue(AreClose(tb.BaselineOffset, measurement.BaselineOffset));
                }
                //MessageBox.Show(string.Format("TextBlock: {0} vs Measurer: {1}", textBlockCounter.TotalMilliseconds, measurerCounter.TotalMilliseconds));
            }), Helper.STANDARD_FONT_FAMILIES);
        }
    }
}
