﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Tests.Utils;
using M = System.Windows.Media;

namespace FontReaderTests.TrueTypeTests
{
    [TestClass]
    public class TestTrueTypeFontSource
    {
        [TestMethod]
        public void TestRequiredTables()
        {
            OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(new FontProperties(new FontFamily("Arial")));
            Assert.IsNotNull(src.Tables[Tags.CMAP_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HEAD_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HHEA_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HMTX_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.MAXP_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.NAME_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.OS_2_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.POST_TABLE]);
        }

        [TestMethod]
        public void TestCmapTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                M.GlyphTypeface gtf = TestHelper.GetGlyphTypeface(fd);
                OpenTypeFontSourceBase fs = TestHelper.GetFontSource(fd);
                foreach (char ch in TestHelper.TestString)
                {
                    ushort expectedGid;
                    gtf.CharacterToGlyphMap.TryGetValue(ch, out expectedGid);
                    ushort gid = fs.GetGlyphId(ch);
                    Assert.AreEqual(expectedGid, gid);
                }
            }));
        }

        [TestMethod]
        public void TestHeadTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                bool isBold = tf.Weight == FontWeights.Bold && tf.StyleSimulations != M.StyleSimulations.BoldSimulation && tf.StyleSimulations != M.StyleSimulations.BoldItalicSimulation;
                bool isItalic = tf.Style == FontStyles.Italic && tf.StyleSimulations != M.StyleSimulations.ItalicSimulation && tf.StyleSimulations != M.StyleSimulations.BoldItalicSimulation;
                Assert.AreEqual(isBold, src.IsBold);
                Assert.AreEqual(isItalic, src.IsItalic);
            }));
        }

        //[TestMethod]
        //public void TestHorizontalHeaderTable()
        //{
        //    Helper.ExecuteActionWithDifferentFontDescriptors(new Action<FontDescriptor>(fd =>
        //    {
        //        TrueTypeFontSource src = Helper.GetTrueTypeFontSource(fd);
        //        GlyphTypeface tf = Helper.GetGlyphTypeface(fd);
        //        double baseline = src.GetBaseline(1);tf.as
        //        Assert.AreEqual(tf.Baseline, baseline);
        //    }));
        //}

        // TODO: Fix this test (style simulations)
        //[TestMethod]
        public void TestHMetrixTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                for (ushort i = 0; i < tf.GlyphCount; i++)
                {
                    double lsb = src.Scaler.FUnitsToPixels(src.HMtx.GetLeftSideBearing(i), TestHelper.FontSize);
                    double expectedLsb = tf.LeftSideBearings[i];
                    Assert.AreEqual(expectedLsb, lsb);

                    double aw = src.Scaler.FUnitsToPixels(src.HMtx.GetAdvancedWidth(i), TestHelper.FontSize);
                    double expectedAw = tf.AdvanceWidths[i];
                    Assert.AreEqual(expectedAw, aw);
                }
            }));
        }

        [TestMethod]
        public void TestMaximumProfileTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                Assert.AreEqual(tf.GlyphCount, src.MaxP.NumGlyphs);
            }));
        }

        [TestMethod]
        public void TestNameTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                CultureInfo enUS = new CultureInfo("en-US");
                Assert.AreEqual(tf.Win32FamilyNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.FONT_FAMILY_ID));
                Assert.AreEqual(tf.ManufacturerNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.MANUFACTURER_NAME_ID));
                Assert.AreEqual(tf.DesignerNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.DESIGNER_ID));
                Assert.AreEqual(tf.SampleTexts[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.SAMPLE_TEXT_ID));
            }));
        }

        // TODO: Fix this test.
        //[TestMethod]
        public void TestTrueTypeMetricsAndOutlines()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                for (ushort i = 0; i < tf.GlyphCount; i++)
                {
                    double fontSize = 100;
                    Glyph g = src.GetGlyphMetrics(i, fontSize);
                    src.GetGlyphOutlines(g, fontSize);
                    M.Geometry gg = g.Render(M.Matrix.Identity, true, true);
                    M.Geometry go = tf.GetGlyphOutline(i, fontSize, fontSize);
                    if (gg.Bounds.IsEmpty || go.Bounds.IsEmpty)
                    {
                        continue;
                    }
                    M.Geometry intersect = M.Geometry.Combine(gg, go, M.GeometryCombineMode.Xor, null);
                    double area = intersect.GetArea();
                    if (area >= 10)
                    {
                        Window w = new Window();
                        Grid gr = new Grid();
                        
                        Path p1 = new Path();
                        p1.Fill = M.Brushes.Black;
                        p1.Data = go.GetFlattenedPathGeometry();
                        gr.Children.Add(p1);

                        Path p2 = new Path();
                        p2.Fill = M.Brushes.Yellow;
                        p2.Data = gg;
                        gr.Children.Add(p2);

                        gr.HorizontalAlignment = HorizontalAlignment.Center;
                        gr.VerticalAlignment = VerticalAlignment.Center;
                        w.Content = gr;
                        w.ShowDialog();
                    }
                }
            }));
        }
    }
}
