﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Tests.Utils;

namespace Telerik.Windows.Documents.Core.Tests.TrueTypeTests
{
    [TestClass]
    public class TestTrueTypeFontSourceMetafiles
    {
        [TestMethod]
        public void TestCmapTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource fs = TestHelper.GetTrueTypeFontSource(fd);
                OpenTypeFontSourceFromMetaFile mfs = TestHelper.GetTrueTypeFontSourceFromMetaFile(fs);
                 
                foreach (char ch in TestHelper.TestString)
                {
                    Assert.AreEqual(fs.GetGlyphId(ch), mfs.GetGlyphId(ch));
                }
            }));
        }

        [TestMethod]
        public void TestFontDescription()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource fs = TestHelper.GetTrueTypeFontSource(fd);
                OpenTypeFontSourceFromMetaFile mfs = TestHelper.GetTrueTypeFontSourceFromMetaFile(fs);
                Assert.AreEqual(fs.FontFamily, mfs.FontFamily);
                Assert.AreEqual(fs.IsBold, mfs.IsBold);
                Assert.AreEqual(fs.IsItalic, mfs.IsItalic);
                Assert.AreEqual(fs.NumberOfHorizontalMetrics, mfs.NumberOfHorizontalMetrics);
                Assert.AreEqual(fs.GlyphCount, mfs.GlyphCount);
            }));
        }

        [TestMethod]
        public void TestHorizontalHeaderTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource fs = TestHelper.GetTrueTypeFontSource(fd);
                OpenTypeFontSourceFromMetaFile mfs = TestHelper.GetTrueTypeFontSourceFromMetaFile(fs);
                Assert.AreEqual(fs.HHea.Ascender, mfs.HHea.Ascender);
                Assert.AreEqual(fs.HHea.Descender, mfs.HHea.Descender);
                Assert.AreEqual(fs.HHea.LineGap, mfs.HHea.LineGap);
                Assert.AreEqual(fs.HHea.NumberOfHMetrics, mfs.HHea.NumberOfHMetrics);
            }));
        }

        [TestMethod]
        public void TestHMetrixTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource fs = TestHelper.GetTrueTypeFontSource(fd);
                OpenTypeFontSourceFromMetaFile mfs = TestHelper.GetTrueTypeFontSourceFromMetaFile(fs);
                for (ushort i = 0; i < fs.GlyphCount; i++)
                {
                    double expectedAw = fs.Scaler.FUnitsToPixels(fs.HMtx.GetAdvancedWidth(i), TestHelper.FontSize);
                    double aw = mfs.Scaler.FUnitsToPixels(mfs.HMtx.GetAdvancedWidth(i), TestHelper.FontSize);
                    Assert.AreEqual(expectedAw, aw);
                }
            }));
        }
    }
}
