﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Shapes;
using M = System.Windows.Media;

namespace Telerik.Windows.Documents.Core.Tests.Utils
{
    internal static class Extensions
    {
        private static M.PathSegment CreatePathSegment(M.Matrix textRenderingMatrix, PathSegment segment, bool isStroked)
        {
            if (segment is LineSegment)
            {
                LineSegment s = (LineSegment)segment;
                M.LineSegment ls = new M.LineSegment();
                ls.Point = textRenderingMatrix.Transform(s.Point);
#if WPF
                ls.IsStroked = isStroked;
#endif
                return ls;
            }
            else if (segment is BezierSegment)
            {
                BezierSegment s = (BezierSegment)segment;
                M.BezierSegment bs = new M.BezierSegment();
                bs.Point1 = textRenderingMatrix.Transform(s.Point1);
                bs.Point2 = textRenderingMatrix.Transform(s.Point2);
                bs.Point3 = textRenderingMatrix.Transform(s.Point3);
#if WPF
                bs.IsStroked = isStroked;
#endif
                return bs;
            }
            else if (segment is QuadraticBezierSegment)
            {
                QuadraticBezierSegment s = (QuadraticBezierSegment)segment;
                M.QuadraticBezierSegment qbs = new M.QuadraticBezierSegment();
                qbs.Point1 = textRenderingMatrix.Transform(s.Point1);
                qbs.Point2 = textRenderingMatrix.Transform(s.Point2);
#if WPF
                qbs.IsStroked = isStroked;
#endif
                return qbs;
            }
            return null;
        }
        private static M.PathFigure CreatePathFigure(M.Matrix textRenderingMatrix, PathFigure figure, bool isFilled, bool isStroked)
        {
            M.PathFigure res = new M.PathFigure();
            foreach (PathSegment s in figure.Segments)
            {
                M.PathSegment ps = CreatePathSegment(textRenderingMatrix, s, isStroked);
                if (ps != null)
                {
                    res.Segments.Add(ps);
                }
            }
            res.StartPoint = textRenderingMatrix.Transform(figure.StartPoint);
            res.IsFilled = isFilled;
            return res;
        }



        public static M.Geometry Render(this Glyph glyph, M.Matrix textRenderingMatrix, bool isFilled, bool isStroked)
        {
            M.PathGeometry pg = new M.PathGeometry();
            foreach (PathFigure f in glyph.Outlines)
            {
                pg.Figures.Add(CreatePathFigure(textRenderingMatrix, f, isFilled, isStroked));
            }
            pg.FillRule = M.FillRule.Nonzero;
            return pg;
        }
    }
}
