﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported charset format exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedCharsetFormatException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Charset format is not supported.";
        private const string CharsetFormatNotSupportedMessage = "Charset format {0} is not supported.";

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedCharsetFormatException" /> class.
        /// </summary>
        public NotSupportedCharsetFormatException() : base(DefaultMessage)
        {
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedCharsetFormatException" /> class.
        /// </summary>
        /// <param name="charsetFormat">The charset format.</param>
        public NotSupportedCharsetFormatException(int charsetFormat) : this(charsetFormat, string.Format(CharsetFormatNotSupportedMessage, charsetFormat))
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedCharsetFormatException" /> class.
        /// </summary>
        /// <param name="charsetFormat">The charset format.</param>
        /// <param name="message">The message.</param>
        public NotSupportedCharsetFormatException(int charsetFormat, string message) : base(message)
        {
            this.CharsetFormat = charsetFormat;
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedCharsetFormatException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedCharsetFormatException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF 
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedCharsetFormatException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedCharsetFormatException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.CharsetFormat = info.GetInt32("CharsetFormat");
        }

#endif

        /// <summary>
        /// Gets or sets the charset format.
        /// </summary>
        /// <value>The charset format.</value>
        public int CharsetFormat { private set; get; }

#if WPF
        
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("CharsetFormat", this.CharsetFormat); 

            base.GetObjectData(info, context);
        }
#endif
    }
}
