﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported color space exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedColorSpaceException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Color space is not supported.";
        private const string ColorSpaceNotSupportedMessage = "{0} color space is not supported.";
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedColorSpaceException" /> class.
        /// </summary>
        public NotSupportedColorSpaceException() : base(DefaultMessage)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedColorSpaceException" /> class.
        /// </summary>
        /// <param name="colorSpace">The color space.</param>
        public NotSupportedColorSpaceException(string colorSpace) : this(colorSpace, string.Format(ColorSpaceNotSupportedMessage, colorSpace))
        {
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedColorSpaceException" /> class.
        /// </summary>
        /// <param name="colorSpace">The color space.</param>
        /// <param name="message">The message.</param>
        public NotSupportedColorSpaceException(string colorSpace, string message) : base(message)
        {
            this.ColorSpace = colorSpace;
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedColorSpaceException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedColorSpaceException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF  
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedColorSpaceException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedColorSpaceException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.ColorSpace = info.GetString("ColorSpace");
        }

#endif 
        /// <summary>
        /// Gets or sets the color space.
        /// </summary>
        /// <value>The color space.</value>
        public string ColorSpace { get; private set; }

#if WPF  
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("ColorSpace", this.ColorSpace);

            base.GetObjectData(info, context);
        }
#endif
    }
}
