﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported encryption exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedEncryptionException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "The encryption method is not supported.";
        private const string FilterTypeNotSupportedMessage = "The encryption method with code {0} is not supported.";
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedEncryptionException" /> class.
        /// </summary>
        public NotSupportedEncryptionException() : base(DefaultMessage)
        {
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedEncryptionException" /> class.
        /// </summary>
        /// <param name="encryptionCode">The encryption code.</param>
        public NotSupportedEncryptionException(int encryptionCode)
            : this(encryptionCode, string.Format(FilterTypeNotSupportedMessage, encryptionCode)) 
        { 
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedEncryptionException" /> class.
        /// </summary>
        /// <param name="encryptionCode">The encryption code.</param>
        /// <param name="message">The message.</param>
        public NotSupportedEncryptionException(int encryptionCode, string message) : base(message)
        {
            this.EncryptionCode = encryptionCode;
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedEncryptionException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedEncryptionException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF 
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedEncryptionException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedEncryptionException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.EncryptionCode = info.GetInt32("EncryptionCode");
        } 
#endif
         
        /// <summary>
        /// Gets or sets the encryption code.
        /// </summary>
        /// <value>The encryption code.</value>
        public int EncryptionCode { get; private set; }

#if WPF 
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("EncryptionCode", this.EncryptionCode);

            base.GetObjectData(info, context);
        }
#endif
    }
}
