﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using Telerik.Windows.Documents.Exceptions; 

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported filter exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedFilterException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Filter type not supported.";
        private const string FilterTypeNotSupportedMessage = "{0} is not supported.";

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFilterException" /> class.
        /// </summary>
        public NotSupportedFilterException()
            : base(DefaultMessage)
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFilterException" /> class.
        /// </summary>
        /// <param name="filterName">Name of the filter.</param>
        public NotSupportedFilterException(string filterName)
            : this(filterName, string.Format(FilterTypeNotSupportedMessage, filterName))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFilterException" /> class.
        /// </summary>
        /// <param name="filterName">Name of the filter.</param>
        /// <param name="message">The message.</param>
        public NotSupportedFilterException(string filterName, string message) : base(message)
        {
            this.FilterName = filterName;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFilterException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedFilterException(string message, Exception cause)
            : base(message, cause)
        {
        }

#if WPF
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFilterException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedFilterException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            this.FilterName = info.GetString("FilterName");
        }
#endif

        /// <summary>
        /// Gets the name of the filter.
        /// </summary>
        /// <value>The name of the filter.</value>
        public string FilterName { get; private set; }

#if WPF
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("FilterName", this.FilterName);

            base.GetObjectData(info, context);
        }
#endif
    }
}
