﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported font exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedFontException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Font type not supported.";
        private const string FontTypeNotSupportedMessage = "{0} font is not supported.";
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFontException" /> class.
        /// </summary>
        public NotSupportedFontException() : base(DefaultMessage)
        {
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFontException" /> class.
        /// </summary>
        /// <param name="fontType">Type of the font.</param>
        public NotSupportedFontException(string fontType) : this(fontType, string.Format(FontTypeNotSupportedMessage, fontType))
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFontException" /> class.
        /// </summary>
        /// <param name="fontType">Type of the font.</param>
        /// <param name="message">The message.</param>
        public NotSupportedFontException(string fontType, string message) : base(message)
        {
            this.FontType = fontType;
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFontException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedFontException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF 
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFontException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedFontException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.FontType = info.GetString("FontType");
        }

#endif
         
        /// <summary>
        /// Gets or sets the type of the font.
        /// </summary>
        /// <value>The type of the font.</value>
        public string FontType { get; private set; }

#if WPF 
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("FontType", this.FontType);

            base.GetObjectData(info, context);
        }
#endif
    }
}
