﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported function type exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedFunctionTypeException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Function type is not supported.";
        private const string FunctionTypeNotSupportedMessage = "Function type {0} is not supported.";

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        public NotSupportedFunctionTypeException() : base(DefaultMessage)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public NotSupportedFunctionTypeException(string message) : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        /// <param name="functionType">Type of the function.</param>
        public NotSupportedFunctionTypeException(int functionType) : this(functionType, string.Format(FunctionTypeNotSupportedMessage, functionType))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        /// <param name="functionType">Type of the function.</param>
        /// <param name="message">The message.</param>
        public NotSupportedFunctionTypeException(int functionType, string message) : base(message)
        {
            this.FunctionType = functionType;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedFunctionTypeException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedFunctionTypeException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedFunctionTypeException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.FunctionType = info.GetInt32("FunctionType");
        }

#endif 
        /// <summary>
        /// Gets or sets the type of the function.
        /// </summary>
        /// <value>The type of the function.</value>
        public int FunctionType { private set; get; }

#if WPF
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("FunctionType", this.FunctionType);

            base.GetObjectData(info, context);
        }
#endif
    }
}
