﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported paint type exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedPaintTypeException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Paint type is not supported.";
        private const string PaintTypeNotSupportedMessage = "Paint type {0} is not supported.";
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        public NotSupportedPaintTypeException() : base(DefaultMessage)
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public NotSupportedPaintTypeException(string message) : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        /// <param name="paintType">Type of the paint.</param>
        public NotSupportedPaintTypeException(int paintType) : this(paintType, string.Format(PaintTypeNotSupportedMessage, paintType))
        { 
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        /// <param name="paintType">Type of the paint.</param>
        /// <param name="message">The message.</param>
        public NotSupportedPaintTypeException(int paintType, string message) : base(message)
        {
            this.PaintType = paintType;
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedPaintTypeException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF 
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPaintTypeException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedPaintTypeException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.PaintType = info.GetInt32("PaintType");
        }

#endif
         
        /// <summary>
        /// Gets or sets the type of the paint.
        /// </summary>
        /// <value>The type of the paint.</value>
        public int PaintType { private set; get; }

#if WPF 
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("PaintType", this.PaintType); 

            base.GetObjectData(info, context);
        }
#endif
    }
}
