﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported predefined CMap exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedPredefinedCMapException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Predefined CMap not supported.";
        private const string PredefinedCMapNotSupportedMessage = "Predefined CMap {0} is not supported.";
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPredefinedCMapException" /> class.
        /// </summary>
        public NotSupportedPredefinedCMapException() : base(DefaultMessage)
        {
        }
          
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPredefinedCMapException" /> class.
        /// </summary>
        /// <param name="cmapName">Name of the cmap.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "cmap")]
        public NotSupportedPredefinedCMapException(string cmapName) : this(cmapName, string.Format(PredefinedCMapNotSupportedMessage, cmapName))
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPredefinedCMapException" /> class.
        /// </summary>
        /// <param name="cmapName">Name of the cmap.</param>
        /// <param name="message">The message.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "cmap")]
        public NotSupportedPredefinedCMapException(string cmapName, string message) : base(message)
        {
            this.CMapName = cmapName;
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPredefinedCMapException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedPredefinedCMapException(string message, Exception cause) : base(message, cause)
        {
        }

#if WPF 
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedPredefinedCMapException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedPredefinedCMapException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.CMapName = info.GetString("CMapName");
        }

#endif
         
        /// <summary>
        /// Gets or sets the name of the C map.
        /// </summary>
        /// <value>The name of the C map.</value>
        public string CMapName { get; private set; }

#if WPF 
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("CMapName", this.CMapName);

            base.GetObjectData(info, context);
        }
#endif
    }
}
