﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Telerik.Windows.Documents.Exceptions;

namespace Telerik.Windows.Documents.Fixed.Exceptions
{
    /// <summary>
    /// Represents not supported shading type exception.
    /// </summary>
#if WPF
    [Serializable]
#endif
    public class NotSupportedShadingTypeException : NotSupportedFeatureException
    {
        private const string DefaultMessage = "Shading type is not supported.";
        private const string ShadingTypeNotSupportedMessage = "Shading type {0} is not supported.";

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        public NotSupportedShadingTypeException() : base(DefaultMessage)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public NotSupportedShadingTypeException(string message) : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        /// <param name="shadingType">Type of the shading.</param>
        public NotSupportedShadingTypeException(int shadingType) : this(shadingType, string.Format(ShadingTypeNotSupportedMessage, shadingType))
        { 
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        /// <param name="shadingType">Type of the shading.</param>
        /// <param name="message">The message.</param>
        public NotSupportedShadingTypeException(int shadingType, string message) : base(message)
        {
            this.ShadingType = shadingType;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public NotSupportedShadingTypeException(string message, Exception cause) 
            : base(message, cause)
        {
        }

#if WPF
        /// <summary>
        /// Initializes a new instance of the <see cref="NotSupportedShadingTypeException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected NotSupportedShadingTypeException(SerializationInfo info, StreamingContext context) 
            : base(info, context)
        {
            this.ShadingType = info.GetInt32("ShadingType");
        }
#endif

        /// <summary>
        /// Gets or sets the type of the shading.
        /// </summary>
        /// <value>The type of the shading.</value>
        public int ShadingType { private set; get; }

#if WPF
        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [System.Security.SecurityCriticalAttribute()]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("ShadingType", this.ShadingType); 

            base.GetObjectData(info, context);
        }
#endif
    }
}
