﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType
{
    internal class OpenTypeFontReader : ReaderBase
    {
        #region Constructors

        public OpenTypeFontReader(byte[] data)
            : base(data)
        {
        }

        #endregion


        #region Fields
        #endregion


        #region Properties
        #endregion


        #region Methods

        public sbyte ReadChar()
        {
            return (sbyte)this.Read();
        }

        public ushort ReadUShort()
        {
            byte[] us = new byte[2];
            this.ReadBE(us, 2);
            return BitConverter.ToUInt16(us, 0);
        }

        public short ReadShort()
        {
            byte[] us = new byte[2];
            this.ReadBE(us, 2);
            return BitConverter.ToInt16(us, 0);
        }

        public uint ReadULong()
        {
            byte[] ul = new byte[4];
            this.ReadBE(ul, 4);
            return BitConverter.ToUInt32(ul, 0);
        }

        public int ReadLong()
        {
            byte[] ul = new byte[4];
            this.ReadBE(ul, 4);
            return BitConverter.ToInt32(ul, 0);
        }

        public long ReadLongDateTime()
        {
            byte[] dt = new byte[8];
            this.ReadBE(dt, 8);
            return BitConverter.ToInt64(dt, 0);
        }

        public float ReadFixed()
        {
            float res = this.ReadShort();
            res += (this.ReadUShort() / 65536.0f);
            return res;
        }

        public float Read2Dot14()
        {
            return (float)this.ReadShort() / (float)(1 << 14);
        }

        public string ReadString()
        {
            byte len = this.Read();
            byte[] bytes = new byte[len];
            for (int i = 0; i < len; i++)
            {
                bytes[i] = this.Read();
            }
            return System.Text.Encoding.UTF8.GetString(bytes, 0, bytes.Length);
        }

        #endregion


        #region Overrides

      
        #endregion
    }
}
