﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class CMapFormat4Table : CMapTable
    {
        #region Fields

        private Segment[] segments;
        private ushort firstCode;

        #endregion


        #region Properties

        //public ushort Length
        //{
        //    get;
        //    set;
        //}

        //public ushort Language
        //{
        //    get;
        //    set;
        //}

        //public ushort SegCountX2
        //{
        //    get;
        //    set;
        //}

        //public ushort SearchRange
        //{
        //    get;
        //    set;
        //}

        //public ushort EntrySelector
        //{
        //    get;
        //    set;
        //}

        //public ushort RangeShift
        //{
        //    get;
        //    set;
        //}

        //public ushort[] EndCount
        //{
        //    get;
        //    set;
        //}

        //public ushort ReservedPad
        //{
        //    get;
        //    set;
        //}

        //public ushort[] StartCount
        //{
        //    get;
        //    set;
        //}

        //public short[] IdDelta
        //{
        //    get;
        //    set;
        //}

        //public ushort[] IdRangeOffset
        //{
        //    get;
        //    set;
        //}

        //public ushort[] Length
        //{
        //    get;
        //    set;
        //}

        public override ushort FirstCode
        {
            get { return this.firstCode; }
        }
 
        #endregion


        #region Methods

        public override bool TryGetGlyphId(int unicode, out ushort glyphId)
        {
            glyphId = CMap.MISSING_GLYPH_ID;
            foreach (Segment s in this.segments)
            {
                if (s.IsInside(unicode))
                {
                    return s.TryGetGlyphId(unicode, out glyphId);
                }
            }

            return false;
        }

        public override bool TryGetCharId(ushort glyphId, out ushort charId)
        {
            charId = CMap.MISSING_GLYPH_ID;
            if (CMap.MISSING_GLYPH_ID == glyphId)
            {
                return false;
            }

            foreach (Segment s in this.segments)
            {
                if (s.Contains(glyphId))
                {
                    charId = s.GetCharId(glyphId);

                    return true;
                }
            }

            return false;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadUShort();
            reader.ReadUShort();
            ushort segCount = (ushort)(reader.ReadUShort() / 2);
            reader.ReadUShort();
            reader.ReadUShort();
            reader.ReadUShort();

            ushort[] endCodes = new ushort[segCount];
            ushort[] startCodes = new ushort[segCount];
            short[] idDelta = new short[segCount];
            ushort[] idRangeOffset = new ushort[segCount];

            this.segments = new Segment[segCount];
            
            this.firstCode = ushort.MaxValue;

            for (int i = 0; i < segCount; i++)
            {
                endCodes[i] = reader.ReadUShort();
            }
          
            reader.ReadUShort();
            for (int i = 0; i < segCount; i++)
            {
                startCodes[i] = reader.ReadUShort();
                if (this.firstCode > startCodes[i])
                {
                    this.firstCode = startCodes[i];
                }
            }
           
            for (int i = 0; i < segCount; i++)
            {
                idDelta[i] = reader.ReadShort();
            }
           
            for (int i = 0; i < segCount; i++)
            {
                long position = reader.Position;
                idRangeOffset[i] = reader.ReadUShort();
                if (idRangeOffset[i] <= 0)
                {
                    this.segments[i] = new Segment(startCodes[i], endCodes[i], idDelta[i]);
                }
                else
                {
                    position += idRangeOffset[i];
                    int size = endCodes[i] - startCodes[i] + 1;
                    ushort[] map = new ushort[size];

                    reader.BeginReadingBlock();
                    reader.Seek(position, System.IO.SeekOrigin.Begin);
                    
                    for (int c = 0; c < map.Length; c++)
                    {
                        map[c] = reader.ReadUShort();
                    }

                    this.segments[i] = new Segment(startCodes[i], endCodes[i], idDelta[i], map);
                    reader.EndReadingBlock();
                }
            }
        }

        public override void Write(FontWriter writer)
        {
            writer.WriteUShort(4);
            writer.WriteUShort(this.firstCode);
            writer.WriteUShort((ushort)this.segments.Length);
            for (int i = 0; i < this.segments.Length; i++)
            {
                this.segments[i].Write(writer);
            }
        }

        public override void Import(OpenTypeFontReader reader)
        {
            this.firstCode = reader.ReadUShort();
            ushort cnt = reader.ReadUShort();
            this.segments = new Segment[cnt];
            
            for (int i = 0; i < cnt; i++)
            {
                Segment s = new Segment();
                s.Import(reader);
                this.segments[i] = s;
            }
        }

        #endregion
    }
}
