﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal abstract class CMapTable
    {
        #region StaticMembers

        internal static CMapTable ReadCMapTable(OpenTypeFontReader reader)
        {
            CMapTable table = null;
            ushort format = reader.ReadUShort();
            switch (format)
            {
                case 0:
                    table = new CMapFormat0Table();
                    break;
                case 4:
                    table = new CMapFormat4Table();
                    break;
                case 6:
                    table = new CMapFormat6Table();
                    break;
                default:
                    return null;
            }
            table.Read(reader);
            return table;
        }


        internal static CMapTable ImportCMapTable(OpenTypeFontReader reader)
        {
            CMapTable table = null;
            ushort format = reader.ReadUShort();
            switch (format)
            {
                case 0:
                    table = new CMapFormat0Table();
                    break;
                case 4:
                    table = new CMapFormat4Table();
                    break;
                default:
                    return null;
            }
            table.Import(reader);
            return table;
        }

        #endregion


        #region Properties

        public abstract ushort FirstCode { get; }
        
        #endregion


        #region Methods

        public abstract bool TryGetGlyphId(int unicode, out ushort glyphId);

        public abstract bool TryGetCharId(ushort glyphId, out ushort charCode);

        public abstract void Read(OpenTypeFontReader reader);

        public abstract void Write(FontWriter writer);

        public abstract void Import(OpenTypeFontReader reader);

        #endregion
    }
}
