﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal abstract class Coverage : TableBase
    {
        #region StaticMembers

        private static Coverage CreateCoverageTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            Coverage c;
            long offset = reader.Position;
            ushort format = reader.ReadUShort();
            switch (format)
            {
                case 1:
                    c = new CoverageFormat1(fontSource);
                    break;
                case 2:
                    c = new CoverageFormat2(fontSource);
                    break;
                default:
                    return null;
            }
            c.Offset = offset;
            return c;
        }

        internal static Coverage ReadCoverageTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            Coverage c = CreateCoverageTable(fontSource, reader);
            if (c != null)
            {
                c.Read(reader);
            }
            return c;
        }

        internal static Coverage ImportCoverageTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            Coverage c = CreateCoverageTable(fontSource, reader);
            if (c != null)
            {
                c.Import(reader);
            }
            return c;
        }

        #endregion


        #region Constructors

        public Coverage(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Methods

        internal override void Import(OpenTypeFontReader reader)
        {
            this.Read(reader);
        }

        public abstract int GetCoverageIndex(ushort glyphIndex);

        #endregion
    }
}
