﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Features;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class Feature : TableBase
    {
        #region Constructors

        public Feature(OpenTypeFontSourceBase fontFile, FeatureInfo featureInfo)
            : base(fontFile)
        {
            this.FeatureInfo = featureInfo;
        }

        #endregion


        #region Fields

        private ushort[] lookupListIndices;

        #endregion


        #region Properties

        public FeatureInfo FeatureInfo
        {
            get;
            private set;
        }

        public ushort[] LookupsListIndices
        {
            get
            {
                return this.lookupListIndices;
            }
        }

        #endregion


        #region Methods

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadUShort();
            ushort lookupCount = reader.ReadUShort();
            this.lookupListIndices = new ushort[lookupCount];
            for (int i = 0; i < lookupCount; i++)
            {
                this.lookupListIndices[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteULong(this.FeatureInfo.Tag);
            writer.WriteUShort((ushort)this.lookupListIndices.Length);
            for (int i = 0; i < this.lookupListIndices.Length; i++)
            {
                writer.WriteUShort(this.lookupListIndices[i]);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            this.lookupListIndices = new ushort[cnt];
            for (int i = 0; i < cnt; i++)
            {
                this.lookupListIndices[i] = reader.ReadUShort();
            }
        }

        public override string ToString()
        {
            return this.FeatureInfo == null ? "Not supported" : Tags.GetStringFromTag(this.FeatureInfo.Tag);
        }

        #endregion
    }
}
