﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Features;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class FeatureList : TableBase
    {
        #region Construcors

        public FeatureList(OpenTypeFontSourceBase fontFile) 
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private FeatureRecord[] featureRecords;
        private Feature[] features;

        #endregion


        #region Methods

        private Feature ReadFeature(OpenTypeFontReader reader, FeatureRecord record)
        {
            FeatureInfo fi = FeatureInfo.CreateFeatureInfo(record.FeatureTag);
            if (fi == null)
            {
                return null;
            }
            reader.BeginReadingBlock();
            reader.Seek(this.Offset + record.FeatureOffset, System.IO.SeekOrigin.Begin);
            Feature f = new Feature(this.FontSource, fi);
            f.Read(reader);
            reader.EndReadingBlock();
            return f;
        }

        public Feature GetFeature(int index)
        {
            if (this.features[index] == null)
            {
                this.features[index] = this.ReadFeature(this.Reader, this.featureRecords[index]);
            }
            return this.features[index];
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort featureCount = reader.ReadUShort();
            this.featureRecords = new FeatureRecord[featureCount];
            this.features = new Feature[featureCount];
            for (int i = 0; i < featureCount; i++)
            {
                this.featureRecords[i] = new FeatureRecord();
                this.featureRecords[i].Read(reader);
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort((ushort)this.featureRecords.Length);
            for (int i = 0; i < this.featureRecords.Length; i++)
            {
                Feature f = this.GetFeature(i);
                if (f == null)
                {
                    writer.WriteULong(Tags.NULL_TAG);
                }
                else
                {
                    f.Write(writer);
                }
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            this.features = new Feature[cnt];
            for (int i = 0; i < cnt; i++)
            {
                uint tag = reader.ReadULong();
                if (tag != Tags.NULL_TAG)
                {
                    Feature f = new Feature(this.FontSource, FeatureInfo.CreateFeatureInfo(tag));
                    f.Import(reader);
                    this.features[i] = f;
                }
            }
        }

        #endregion
    }
}
