﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Features
{
    internal abstract class FeatureInfo
    {
        #region StaticMembers

        internal static FeatureInfo CreateFeatureInfo(uint featureTag)
        {
            if (featureTag == Tags.FEATURE_INITIAL_FORMS)
            {
                return new InitFeatureInfo();
            }
            if (featureTag == Tags.FEATURE_TERMINAL_FORMS)
            {
                return new FinalFeatureInfo();
            }
            if (featureTag == Tags.FEATURE_ISOLATED_FORMS)
            {
                return new IsolatedFeatureInfo();
            }
            if (featureTag == Tags.FEATURE_MEDIAL_FORMS)
            {
                return new MedialFeatureInfo();
            }
            if (featureTag == Tags.FEATURE_REQUIRED_LIGATURES)
            {
                return new RLigFeatureInfo();
            }
            if (featureTag == Tags.FEATURE_STANDARD_LIGATURES)
            {
                return new LigaFeatureInfo();
            }
            return null;
        }
       
        private static GlyphsSequence ApplyMultipleSubstitutionLookup(Lookup lookup, GlyphsSequence glyphIDs)
        {
            return lookup.Apply(glyphIDs);
        }

        #endregion


        #region Types

        internal enum FeatureType { SingleSubstitution, MultipleSubstitution, Unsupported } 

        #endregion


        #region Constructors
        #endregion


        #region Properties

        public abstract uint Tag { get; }

        public abstract FeatureType Type { get; }

        #endregion


        #region Methods

        public abstract bool ShouldApply(GlyphInfo glyphIndex);
        public GlyphsSequence ApplyLookup(Lookup lookup, GlyphsSequence glyphIDs)
        {
            if (lookup == null)
            {
                return glyphIDs;
            }
            switch (this.Type)
            {
                case Features.FeatureInfo.FeatureType.SingleSubstitution:
                    return this.ApplySingleGlyphSubstitutionLookup(lookup, glyphIDs);
                case Features.FeatureInfo.FeatureType.MultipleSubstitution:
                    return ApplyMultipleSubstitutionLookup(lookup, glyphIDs);
                default:
                    return glyphIDs;
            }
        }

        private GlyphsSequence ApplySingleGlyphSubstitutionLookup(Lookup lookup, GlyphsSequence glyphIDs)
        {
            GlyphsSequence res = new GlyphsSequence();
            foreach (GlyphInfo gi in glyphIDs)
            {
                if (this.ShouldApply(gi))
                {
                    GlyphsSequence gs = new GlyphsSequence(1);
                    gs.Add(gi);
                    res.AddRange(lookup.Apply(gs));
                }
                else
                {
                    res.Add(gi);
                }
            }
            return res;
        }

        #endregion
    }
}
