﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class Head : TrueTypeTableBase
    {
        private ushort macStyle;

        public Head(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        public ushort Flags
        {
            get;
            private set;
        }

        public short GlyphDataFormat
        {
            get;
            private set;
        }

        public ushort UnitsPerEm
        {
            get;
            private set;
        }

        public Rect BBox
        {
            get;
            private set;
        }

        public short IndexToLocFormat
        {
            get;
            private set;
        }

        public bool IsBold
        {
            get
            {
                return this.CheckMacStyle(0);
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.CheckMacStyle(1);
            }
        }

        internal override uint Tag
        {
            get { return Tags.HEAD_TABLE; }
        }

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadFixed();
            reader.ReadFixed();
            reader.ReadULong();
            reader.ReadULong();
            this.Flags = reader.ReadUShort();
            this.UnitsPerEm = reader.ReadUShort();
            reader.ReadLongDateTime();
            reader.ReadLongDateTime();
            this.BBox = new Rect(new Point(reader.ReadShort(), reader.ReadShort()), new Point(reader.ReadShort(), reader.ReadShort()));
            this.macStyle = reader.ReadUShort();
            reader.ReadUShort();
            reader.ReadShort();
            this.IndexToLocFormat = reader.ReadShort();
            reader.ReadShort();
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(this.UnitsPerEm);
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.UnitsPerEm = reader.ReadUShort();
        }

        private bool CheckMacStyle(byte bit)
        {
            return (this.macStyle & (1 << bit)) != 0;
        }
    }
}
