﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class HorizontalMetrics : TrueTypeTableBase
    {
        public HorizontalMetrics(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        private LongHorMetric[] hMetrics;

        internal override uint Tag
        {
            get { return Tags.HMTX_TABLE; }
        }

        public ushort GetAdvancedWidth(int glyphID)
        {
            if (glyphID < this.hMetrics.Length)
            {
                return this.hMetrics[glyphID].AdvanceWidth;
            }

            var element = this.hMetrics[this.hMetrics.Length - 1];
            return element.AdvanceWidth;
        }

        public override void Read(OpenTypeFontReader reader)
        {
            this.hMetrics = new LongHorMetric[this.FontSource.NumberOfHorizontalMetrics];
            for (int i = 0; i < this.hMetrics.Length; i++)
            {
                LongHorMetric m = new LongHorMetric();
                m.Read(reader);
                this.hMetrics[i] = m;
            }
        }

        internal override void Write(FontWriter writer)
        {
            for (int i = 0; i < this.hMetrics.Length; i++)
            {
                this.hMetrics[i].Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.Read(reader);
        }
    }
}
