﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Features;
using Telerik.Windows.Documents.Core.Fonts.Utils;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class LangSys : TableBase
    {
        #region Constructors

        public LangSys(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort reqFeatureIndex;
        private ushort[] featureIndices;

        private List<Tuple<FeatureInfo, Lookup>> lookups;

        #endregion


        #region Properties

        private IEnumerable<Tuple<FeatureInfo, Lookup>> Lookups
        {
            get
            {
                if (this.lookups == null)
                {
                    var indices = this.GetLookupIndices();
                    this.lookups = new List<Tuple<FeatureInfo, Lookup>>(Enumerable.Count(indices));
                    foreach (var li in indices)
                    {
                        this.lookups.Add(new Tuple<FeatureInfo, Lookup>(li.Item1, this.FontSource.GetLookup(li.Item2)));
                    }
                }
                return this.lookups;
            }
        }

        #endregion


        #region Methods

        private int Compare(Tuple<FeatureInfo, ushort> left, Tuple<FeatureInfo, ushort> right)
        {
            return left.Item2.CompareTo(right.Item2);
        }
        
        private IEnumerable<Feature> GetFeatures()
        {
            List<Feature> features = new List<Feature>(this.featureIndices.Length);
            for (int i = 0; i < this.featureIndices.Length; i++)
            {
                Feature f = this.FontSource.GetFeature(this.featureIndices[i]);
                if (f == null)
                {
                    continue;
                }
                features.Add(f);
            }
            if (this.reqFeatureIndex != TrueTypeConsts.REQUIRED_FEATURE_NULL)
            {
                features.Add(this.FontSource.GetFeature(this.reqFeatureIndex));
            }
            return features;
        }

        private IEnumerable<Tuple<FeatureInfo, ushort>> GetLookupIndices()
        {
            List<Tuple<FeatureInfo, ushort>> lookupListIndices = new List<Tuple<FeatureInfo, ushort>>();
            foreach (Feature f in this.GetFeatures())
            {
                foreach (ushort i in f.LookupsListIndices)
                {
                    lookupListIndices.Add(new Tuple<FeatureInfo, ushort>(f.FeatureInfo, i));
                }
            }
            lookupListIndices.Sort(this.Compare);
            return lookupListIndices;
        }



        public GlyphsSequence Apply(GlyphsSequence glyphIDs)
        {
            GlyphsSequence res = glyphIDs;
            foreach (var lookup in this.Lookups)
            {
                res = lookup.Item1.ApplyLookup(lookup.Item2, res);
            }
            return res;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadUShort();
            this.reqFeatureIndex = reader.ReadUShort();
            ushort featureCount = reader.ReadUShort();
            this.featureIndices = new ushort[featureCount];
            for (int i = 0; i < featureCount; i++)
            {
                this.featureIndices[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(this.reqFeatureIndex);
            writer.WriteUShort((ushort)this.featureIndices.Length);
            for (int i = 0; i < this.featureIndices.Length; i++)
            {
                writer.WriteUShort(this.featureIndices[i]);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.reqFeatureIndex = reader.ReadUShort();
            ushort featureCount = reader.ReadUShort();
            this.featureIndices = new ushort[featureCount];
            for (int i = 0; i < featureCount; i++)
            {
                this.featureIndices[i] = reader.ReadUShort();
            }
        }

        #endregion
    }
}
