﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class Ligature : TableBase
    {
        #region Constructors

        public Ligature(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort[] componentGlyphIds;

        #endregion


        #region Properties

        public ushort LigatureGlyphId
        {
            get;
            private set;
        }

        public int Length
        {
            get
            {
                return this.componentGlyphIds.Length;
            }
        }

        #endregion


        #region Fields

        public bool IsMatch(GlyphsSequence glyphIDs, int startIndex)
        {
            for (int i = 0; i < this.componentGlyphIds.Length; i++)
            {
                if (i + startIndex >= glyphIDs.Count || this.componentGlyphIds[i] != glyphIDs[i + startIndex].GlyphId)
                {
                    return false;
                }
            }
            return true;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            this.LigatureGlyphId = reader.ReadUShort();
            ushort components = (ushort)(reader.ReadUShort() - 1);
            this.componentGlyphIds = new ushort[components];
            for (int i = 0; i < components; i++)
            {
                this.componentGlyphIds[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(this.LigatureGlyphId);
            writer.WriteUShort((ushort)(this.componentGlyphIds.Length + 1));
            for (int i = 0; i < this.componentGlyphIds.Length; i++)
            {
                writer.WriteUShort(this.componentGlyphIds[i]);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.Read(reader);
        }

        #endregion
    }
}
