﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class LigatureSet : TableBase
    {
        #region Constructors

        public LigatureSet(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort[] ligatureOffsets;
        private Ligature[] ligatures;

        #endregion


        #region Properties

        public Ligature[] Ligatures
        {
            get
            {
                if (this.ligatures == null)
                {
                    this.ligatures = new Ligature[this.ligatureOffsets.Length];
                    for (int i = 0; i < this.ligatures.Length; i++)
                    {
                        this.ligatures[i] = this.ReadLigature(this.Reader, this.ligatureOffsets[i]);
                    }
                }
                return this.ligatures;
            }
        }

        #endregion


        #region Methods

        private Ligature ReadLigature(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            reader.Seek(this.Offset + offset, System.IO.SeekOrigin.Begin);
            Ligature l = new Ligature(this.FontSource);
            l.Read(reader);
            reader.EndReadingBlock();
            return l;
        }


        public Ligature FindLigature(GlyphsSequence glyphIDs, int startIndex)
        {
            foreach (Ligature l in this.Ligatures)
            {
                if (l.IsMatch(glyphIDs, startIndex + 1))
                {
                    return l;
                }
            }
            return null;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort ligatureCount = reader.ReadUShort();
            this.ligatureOffsets = new ushort[ligatureCount];
            for (int i = 0; i < ligatureCount; i++)
            {
                this.ligatureOffsets[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort((ushort)this.Ligatures.Length);
            for (int i = 0; i < this.Ligatures.Length; i++)
            {
                this.Ligatures[i].Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort ligatureCount = reader.ReadUShort();
            this.ligatures = new Ligature[ligatureCount];
            for (int i = 0; i < ligatureCount; i++)
            {
                Ligature l = new Ligature(this.FontSource);
                l.Import(reader);
                this.ligatures[i] = l;
            }
        }

        #endregion
    }
}
