﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class Lookup : TableBase
    {
        #region StaticMembers

        internal static bool IsSupported(ushort type)
        {
            switch ((LookupType)type)
            {
                case LookupType.Single:
                case LookupType.Multiple:
                case LookupType.Ligature:
                    return true;
                default:
                    return false;
            }
        }

        #endregion


        #region Constructors

        public Lookup(OpenTypeFontSourceBase fontFile, ushort type)
            : base(fontFile)
        {
            this.Type = type;
        }

        #endregion


        #region Fields

        private ushort[] subTableOffsets;
        private SubTable[] subTables;

        #endregion

        #region Properties

        public ushort Type
        {
            get;
            private set;
        }

        public ushort Flags
        {
            get;
            private set;
        }

        public IEnumerable<SubTable> SubTables
        {
            get
            {
                if (this.subTables == null)
                {
                    this.subTables = new SubTable[this.subTableOffsets.Length];
                    for (int i = 0; i < this.subTableOffsets.Length; i++)
                    {
                        this.subTables[i] = this.ReadSubtable(this.Reader, this.subTableOffsets[i]);
                    }
                }
                return this.subTables;
            }
        }

        #endregion


        #region Methods

        private SubTable ReadSubtable(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            long subtableOffset = this.Offset + offset;
            reader.Seek(subtableOffset, System.IO.SeekOrigin.Begin);
            SubTable st = SubTable.ReadSubTable(FontSource, reader, this.Type);
            st.Offset = subtableOffset;
            reader.EndReadingBlock();
            return st;
        }


        public GlyphsSequence Apply(GlyphsSequence glyphIDs)
        {
            GlyphsSequence res = glyphIDs;
            foreach (SubTable st in this.SubTables)
            {
                res = st.Apply(res);
            }
            return res;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            this.Flags = reader.ReadUShort();
            ushort subTableCount = reader.ReadUShort();
            this.subTableOffsets = new ushort[subTableCount];
            for (int i = 0; i < subTableCount; i++)
            {
                this.subTableOffsets[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(this.Type);
            writer.WriteUShort(this.Flags);
            writer.WriteUShort((ushort)this.subTableOffsets.Length);
            foreach (SubTable st in this.SubTables)
            {
                st.Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.Flags = reader.ReadUShort();
            ushort cnt = reader.ReadUShort();
            this.subTables = new SubTable[cnt];
            for (int i = 0; i < cnt; i++)
            {
                SubTable st = SubTable.ImportSubTable(this.FontSource, reader, this.Type);
                this.subTables[i] = st;
            }
        }

        #endregion
    }
}
