﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class LookupList : TableBase
    {
        #region Constructors

        public LookupList(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort[] lookupOffsets;
        private Lookup[] lookups;

        #endregion


        #region Methods

        private Lookup ReadLookup(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            long lookupOffset = this.Offset + offset;
            reader.Seek(lookupOffset, System.IO.SeekOrigin.Begin);
            ushort type = reader.ReadUShort();
            if (!Lookup.IsSupported(type))
            {
                return null;
            }
            Lookup l = new Lookup(this.FontSource, type);
            l.Offset = lookupOffset;
            l.Read(reader);
            reader.EndReadingBlock();
            return l;
        }

        public Lookup GetLookup(ushort index)
        {
            if (this.lookups[index] == null && this.lookupOffsets != null)
            {
                this.lookups[index] = this.ReadLookup(this.Reader, this.lookupOffsets[index]);
            }
            return this.lookups[index];
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort lookupCount = reader.ReadUShort();
            this.lookupOffsets = new ushort[lookupCount];
            this.lookups = new Lookup[lookupCount];
            for (int i = 0; i < lookupCount; i++)
            {
                this.lookupOffsets[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort((ushort)this.lookupOffsets.Length);
            for (ushort i = 0; i < this.lookupOffsets.Length; i++)
            {
                Lookup l = this.GetLookup(i);
                if (l == null)
                {
                    writer.WriteUShort(Tags.NULL_TYPE);
                }
                else
                {
                    l.Write(writer);
                }
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            this.lookups = new Lookup[cnt];
            for (int i = 0; i < cnt; i++)
            {
                ushort type = reader.ReadUShort();
                if (type != Tags.NULL_TYPE)
                {
                    Lookup l = new Lookup(this.FontSource, type);
                    l.Import(reader);
                }
            }
        }

        #endregion
    }
}
