﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class MultipleSubstitution : SubTable
    {
        #region Constructors

        public MultipleSubstitution(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort coverageOffset;
        private ushort[] sequenceOffsets;

        private Coverage coverage;
        private Sequence[] sequences;

        #endregion


        #region Properties

        protected Coverage Coverage
        {
            get
            {
                if (this.coverage == null)
                {
                    this.coverage = this.ReadCoverage(this.Reader, this.coverageOffset);
                }
                return this.coverage;
            }
        }

        protected Sequence[] Sequences
        {
            get
            {
                if (this.sequences == null)
                {
                    this.sequences = new Sequence[this.sequenceOffsets.Length];
                    for (int i = 0; i < this.sequences.Length; i++)
                    {
                        this.sequences[i] = this.ReadSequence(this.Reader, this.sequenceOffsets[i]);
                    }
                }
                return this.sequences;
            }
        }
                    

        #endregion


        #region Methods

        private Sequence ReadSequence(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            reader.Seek(this.Offset + offset, System.IO.SeekOrigin.Begin);
            Sequence s = new Sequence();
            s.Read(reader);
            reader.EndReadingBlock();
            return s;
        }

        #endregion


        #region Overrides

        public override GlyphsSequence Apply(GlyphsSequence glyphIDs)
        {
            GlyphsSequence res = new GlyphsSequence(glyphIDs.Count);
            for (int i = 0; i < glyphIDs.Count; i++)
            {
                int coverageIndex = this.Coverage.GetCoverageIndex(glyphIDs[i].GlyphId);
                if (coverageIndex < 0)
                {

                }
                else
                {
                    res.AddRange(this.sequences[coverageIndex].Subsitutes);
                }
            }
            return res;
        }

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadUShort();
            this.coverageOffset = reader.ReadUShort();
            ushort sequences = reader.ReadUShort();
            this.sequenceOffsets = new ushort[sequences];
            for (int i = 0; i < sequences; i++)
            {
                this.sequenceOffsets[i] = reader.ReadUShort();
            }

        }

        internal override void Write(FontWriter writer)
        {
            this.Coverage.Write(writer);
            writer.WriteUShort((ushort)this.Sequences.Length);
            for (int i = 0; i < this.Sequences.Length; i++)
            {
                this.Sequences[i].Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.coverage = Coverage.ImportCoverageTable(this.FontSource, reader);
            ushort sequences = reader.ReadUShort();
            this.sequences = new Sequence[sequences];
            for (int i = 0; i < sequences; i++)
            {
                Sequence s = new Sequence();
                s.Import(reader);
                this.sequences[i] = s;
            }
        }

        #endregion
    }
}
