﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class NameFormat0 : Name
    {
        #region Constructors

        public NameFormat0(OpenTypeFontSourceBase fontSource)
            : base(fontSource)
        {
        }

        #endregion


        #region Fields

        private ushort stringOffset;
        private NameRecord[] nameRecords;
        private Dictionary<NameRecord, string> strings;

        private string fontFamily;

        #endregion


        #region Properties

        public override string FontFamily
        {
            get 
            {
                if (fontFamily == null)
                {
                    this.fontFamily = this.ReadString(this.Reader, IDs.ENGLISH_US_ID, IDs.FONT_FAMILY_ID);
                }
                return this.fontFamily;
            }
        }

        #endregion


        #region Methods

        private string ReadString(OpenTypeFontReader reader, ushort languageId, ushort nameId)
        {
            IEnumerable<NameRecord> nameRecords = this.FindNameRecords(IDs.WINDOWS_PLATFORM_ID, languageId, nameId);
            foreach (NameRecord nr in nameRecords)
            {
                Encoding e = IDs.GetEncodingFromEncodingID(nr.EncodingID);
                if (e == null)
                {
                    continue;
                }
                return this.ReadString(reader, nr, e);
            }
            return null;
        }

        private string ReadString(OpenTypeFontReader reader, NameRecord record, Encoding encoding)
        {
            string str;
            if (!this.strings.TryGetValue(record, out str))
            {
                reader.BeginReadingBlock();
                long stringOffset = this.Offset + this.stringOffset + record.Offset;
                reader.Seek(stringOffset, System.IO.SeekOrigin.Begin);
                byte[] res = new byte[record.Length];
                reader.Read(res, record.Length);
                reader.EndReadingBlock();
                str = encoding.GetString(res, 0, res.Length);
                this.strings[record] = str;
            }
            return str;
        }

        private IEnumerable<NameRecord> FindNameRecords(int platformId, ushort languageId, ushort nameId)
        {
            // TODO: Faster search here! Try binary search.
            return Enumerable.Where(this.nameRecords, r => r.PlatformID == platformId && r.LanguageID == languageId && r.NameID == nameId);
        }

        #endregion


        #region Overrides

        internal override string ReadName(ushort languageID, ushort nameID)
        {
            return this.ReadString(this.Reader, languageID, nameID);
        }

        public override void Read(OpenTypeFontReader reader)
        {
            ushort count = reader.ReadUShort();
            this.stringOffset = reader.ReadUShort();
            this.nameRecords = new NameRecord[count];
            this.strings = new Dictionary<NameRecord, string>();
            for (int i = 0; i < count; i++)
            {
                this.nameRecords[i] = new NameRecord();
                this.nameRecords[i].Read(reader);
            }
        }

        #endregion
    }
}
