﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Core.PostScript.Data;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal abstract class Post : TrueTypeTableBase
    {
        protected static Dictionary<string, ushort> macintoshStandardOrderGlyphIds;
        protected static string[] macintoshStandardOrderNames;

        static Post()
        {
            Initialize();
        }

        public Post(OpenTypeFontSourceBase fontSource)
            : base(fontSource)
        {
        }

        internal override uint Tag
        {
            get { return Tags.POST_TABLE; }
        }

        public double ItalicAngle
        {
            get;
            private set;
        }

        public double UnderlinePosition
        {
            get;
            private set;
        }

        public double UnderlineThickness
        {
            get;
            private set;
        }

        public static Post ReadPostTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            float format = reader.ReadFixed();
            Post post = null;
            if (format == 1)
            {
                post = new PostFormat1(fontSource);
            }
            else if (format == 2)
            {
                post = new PostFormat2(fontSource);
            }
            else if (format == 3)
            {
                post = new PostFormat3(fontSource);
            }
            else
            {
                return null;
            }
            
            post.Read(reader);
         
            return post;
        }

        public abstract ushort GetGlyphId(string name);

        public override void Read(OpenTypeFontReader reader)
        {
            this.ItalicAngle = reader.ReadFixed();
            this.UnderlinePosition = reader.ReadShort();
            this.UnderlineThickness = reader.ReadShort();
            reader.ReadULong();
            reader.ReadULong();
            reader.ReadULong();
            reader.ReadULong();
            reader.ReadULong();
        }
     
        private static void Initialize()
        {
            macintoshStandardOrderGlyphIds = new Dictionary<string, ushort>();

            macintoshStandardOrderGlyphIds[PostScriptConstants.NotDef] = 0;
            macintoshStandardOrderGlyphIds[".null"] = 1;
            macintoshStandardOrderGlyphIds["nonmarkingreturn"] = 2;
            macintoshStandardOrderGlyphIds["space"] = 3;
            macintoshStandardOrderGlyphIds["exclam"] = 4;
            macintoshStandardOrderGlyphIds["quotedbl"] = 5;
            macintoshStandardOrderGlyphIds["numbersign"] = 6;
            macintoshStandardOrderGlyphIds["dollar"] = 7;
            macintoshStandardOrderGlyphIds["percent"] = 8;
            macintoshStandardOrderGlyphIds["ampersand"] = 9;
            macintoshStandardOrderGlyphIds["quotesingle"] = 10;
            macintoshStandardOrderGlyphIds["parenleft"] = 11;
            macintoshStandardOrderGlyphIds["parenright"] = 12;
            macintoshStandardOrderGlyphIds["asterisk"] = 13;
            macintoshStandardOrderGlyphIds["plus"] = 14;
            macintoshStandardOrderGlyphIds["comma"] = 15;
            macintoshStandardOrderGlyphIds["hyphen"] = 16;
            macintoshStandardOrderGlyphIds["period"] = 17;
            macintoshStandardOrderGlyphIds["slash"] = 18;
            macintoshStandardOrderGlyphIds["zero"] = 19;
            macintoshStandardOrderGlyphIds["one"] = 20;
            macintoshStandardOrderGlyphIds["two"] = 21;
            macintoshStandardOrderGlyphIds["three"] = 22;
            macintoshStandardOrderGlyphIds["four"] = 23;
            macintoshStandardOrderGlyphIds["five"] = 24;
            macintoshStandardOrderGlyphIds["six"] = 25;
            macintoshStandardOrderGlyphIds["seven"] = 26;
            macintoshStandardOrderGlyphIds["eight"] = 27;
            macintoshStandardOrderGlyphIds["nine"] = 28;
            macintoshStandardOrderGlyphIds["colon"] = 29;
            macintoshStandardOrderGlyphIds["semicolon"] = 30;
            macintoshStandardOrderGlyphIds["less"] = 31;
            macintoshStandardOrderGlyphIds["equal"] = 32;
            macintoshStandardOrderGlyphIds["greater"] = 33;
            macintoshStandardOrderGlyphIds["question"] = 34;
            macintoshStandardOrderGlyphIds["at"] = 35;
            macintoshStandardOrderGlyphIds["A"] = 36;
            macintoshStandardOrderGlyphIds["B"] = 37;
            macintoshStandardOrderGlyphIds["C"] = 38;
            macintoshStandardOrderGlyphIds["D"] = 39;
            macintoshStandardOrderGlyphIds["E"] = 40;
            macintoshStandardOrderGlyphIds["F"] = 41;
            macintoshStandardOrderGlyphIds["G"] = 42;
            macintoshStandardOrderGlyphIds["H"] = 43;
            macintoshStandardOrderGlyphIds["I"] = 44;
            macintoshStandardOrderGlyphIds["J"] = 45;
            macintoshStandardOrderGlyphIds["K"] = 46;
            macintoshStandardOrderGlyphIds["L"] = 47;
            macintoshStandardOrderGlyphIds["M"] = 48;
            macintoshStandardOrderGlyphIds["N"] = 49;
            macintoshStandardOrderGlyphIds["O"] = 50;
            macintoshStandardOrderGlyphIds["P"] = 51;
            macintoshStandardOrderGlyphIds["Q"] = 52;
            macintoshStandardOrderGlyphIds["R"] = 53;
            macintoshStandardOrderGlyphIds["S"] = 54;
            macintoshStandardOrderGlyphIds["T"] = 55;
            macintoshStandardOrderGlyphIds["U"] = 56;
            macintoshStandardOrderGlyphIds["V"] = 57;
            macintoshStandardOrderGlyphIds["W"] = 58;
            macintoshStandardOrderGlyphIds["X"] = 59;
            macintoshStandardOrderGlyphIds["Y"] = 60;
            macintoshStandardOrderGlyphIds["Z"] = 61;
            macintoshStandardOrderGlyphIds["bracketleft"] = 62;
            macintoshStandardOrderGlyphIds["backslash"] = 63;
            macintoshStandardOrderGlyphIds["bracketright"] = 64;
            macintoshStandardOrderGlyphIds["asciicircum"] = 65;
            macintoshStandardOrderGlyphIds["underscore"] = 66;
            macintoshStandardOrderGlyphIds["grave"] = 67;
            macintoshStandardOrderGlyphIds["a"] = 68;
            macintoshStandardOrderGlyphIds["b"] = 69;
            macintoshStandardOrderGlyphIds["c"] = 70;
            macintoshStandardOrderGlyphIds["d"] = 71;
            macintoshStandardOrderGlyphIds["e"] = 72;
            macintoshStandardOrderGlyphIds["f"] = 73;
            macintoshStandardOrderGlyphIds["g"] = 74;
            macintoshStandardOrderGlyphIds["h"] = 75;
            macintoshStandardOrderGlyphIds["i"] = 76;
            macintoshStandardOrderGlyphIds["j"] = 77;
            macintoshStandardOrderGlyphIds["k"] = 78;
            macintoshStandardOrderGlyphIds["l"] = 79;
            macintoshStandardOrderGlyphIds["m"] = 80;
            macintoshStandardOrderGlyphIds["n"] = 81;
            macintoshStandardOrderGlyphIds["o"] = 82;
            macintoshStandardOrderGlyphIds["p"] = 83;
            macintoshStandardOrderGlyphIds["q"] = 84;
            macintoshStandardOrderGlyphIds["r"] = 85;
            macintoshStandardOrderGlyphIds["s"] = 86;
            macintoshStandardOrderGlyphIds["t"] = 87;
            macintoshStandardOrderGlyphIds["u"] = 88;
            macintoshStandardOrderGlyphIds["v"] = 89;
            macintoshStandardOrderGlyphIds["w"] = 90;
            macintoshStandardOrderGlyphIds["x"] = 91;
            macintoshStandardOrderGlyphIds["y"] = 92;
            macintoshStandardOrderGlyphIds["z"] = 93;
            macintoshStandardOrderGlyphIds["braceleft"] = 94;
            macintoshStandardOrderGlyphIds["bar"] = 95;
            macintoshStandardOrderGlyphIds["braceright"] = 96;
            macintoshStandardOrderGlyphIds["asciitilde"] = 97;
            macintoshStandardOrderGlyphIds["Adieresis"] = 98;
            macintoshStandardOrderGlyphIds["Aring"] = 99;
            macintoshStandardOrderGlyphIds["Ccedilla"] = 100;
            macintoshStandardOrderGlyphIds["Eacute"] = 101;
            macintoshStandardOrderGlyphIds["Ntilde"] = 102;
            macintoshStandardOrderGlyphIds["Odieresis"] = 103;
            macintoshStandardOrderGlyphIds["Udieresis"] = 104;
            macintoshStandardOrderGlyphIds["aacute"] = 105;
            macintoshStandardOrderGlyphIds["agrave"] = 106;
            macintoshStandardOrderGlyphIds["acircumflex"] = 107;
            macintoshStandardOrderGlyphIds["adieresis"] = 108;
            macintoshStandardOrderGlyphIds["atilde"] = 109;
            macintoshStandardOrderGlyphIds["aring"] = 110;
            macintoshStandardOrderGlyphIds["ccedilla"] = 111;
            macintoshStandardOrderGlyphIds["eacute"] = 112;
            macintoshStandardOrderGlyphIds["egrave"] = 113;
            macintoshStandardOrderGlyphIds["ecircumflex"] = 114;
            macintoshStandardOrderGlyphIds["edieresis"] = 115;
            macintoshStandardOrderGlyphIds["iacute"] = 116;
            macintoshStandardOrderGlyphIds["igrave"] = 117;
            macintoshStandardOrderGlyphIds["icircumflex"] = 118;
            macintoshStandardOrderGlyphIds["idieresis"] = 119;
            macintoshStandardOrderGlyphIds["ntilde"] = 120;
            macintoshStandardOrderGlyphIds["oacute"] = 121;
            macintoshStandardOrderGlyphIds["ograve"] = 122;
            macintoshStandardOrderGlyphIds["ocircumflex"] = 123;
            macintoshStandardOrderGlyphIds["odieresis"] = 124;
            macintoshStandardOrderGlyphIds["otilde"] = 125;
            macintoshStandardOrderGlyphIds["uacute"] = 126;
            macintoshStandardOrderGlyphIds["ugrave"] = 127;
            macintoshStandardOrderGlyphIds["ucircumflex"] = 128;
            macintoshStandardOrderGlyphIds["udieresis"] = 129;
            macintoshStandardOrderGlyphIds["dagger"] = 130;
            macintoshStandardOrderGlyphIds["degree"] = 131;
            macintoshStandardOrderGlyphIds["cent"] = 132;
            macintoshStandardOrderGlyphIds["sterling"] = 133;
            macintoshStandardOrderGlyphIds["section"] = 134;
            macintoshStandardOrderGlyphIds["bullet"] = 135;
            macintoshStandardOrderGlyphIds["paragraph"] = 136;
            macintoshStandardOrderGlyphIds["germandbls"] = 137;
            macintoshStandardOrderGlyphIds["registered"] = 138;
            macintoshStandardOrderGlyphIds["copyright"] = 139;
            macintoshStandardOrderGlyphIds["trademark"] = 140;
            macintoshStandardOrderGlyphIds["acute"] = 141;
            macintoshStandardOrderGlyphIds["dieresis"] = 142;
            macintoshStandardOrderGlyphIds["notequal"] = 143;
            macintoshStandardOrderGlyphIds["AE"] = 144;
            macintoshStandardOrderGlyphIds["Oslash"] = 145;
            macintoshStandardOrderGlyphIds["infinity"] = 146;
            macintoshStandardOrderGlyphIds["plusminus"] = 147;
            macintoshStandardOrderGlyphIds["lessequal"] = 148;
            macintoshStandardOrderGlyphIds["greaterequal"] = 149;
            macintoshStandardOrderGlyphIds["yen"] = 150;
            macintoshStandardOrderGlyphIds["mu"] = 151;
            macintoshStandardOrderGlyphIds["partialdiff"] = 152;
            macintoshStandardOrderGlyphIds["summation"] = 153;
            macintoshStandardOrderGlyphIds["product"] = 154;
            macintoshStandardOrderGlyphIds["pi"] = 155;
            macintoshStandardOrderGlyphIds["integral"] = 156;
            macintoshStandardOrderGlyphIds["ordfeminine"] = 157;
            macintoshStandardOrderGlyphIds["ordmasculine"] = 158;
            macintoshStandardOrderGlyphIds["Omega"] = 159;
            macintoshStandardOrderGlyphIds["ae"] = 160;
            macintoshStandardOrderGlyphIds["oslash"] = 161;
            macintoshStandardOrderGlyphIds["questiondown"] = 162;
            macintoshStandardOrderGlyphIds["exclamdown"] = 163;
            macintoshStandardOrderGlyphIds["logicalnot"] = 164;
            macintoshStandardOrderGlyphIds["radical"] = 165;
            macintoshStandardOrderGlyphIds["florin"] = 166;
            macintoshStandardOrderGlyphIds["approxequal"] = 167;
            macintoshStandardOrderGlyphIds["Delta"] = 168;
            macintoshStandardOrderGlyphIds["guillemotleft"] = 169;
            macintoshStandardOrderGlyphIds["guillemotright"] = 170;
            macintoshStandardOrderGlyphIds["ellipsis"] = 171;
            macintoshStandardOrderGlyphIds["nonbreakingspace"] = 172;
            macintoshStandardOrderGlyphIds["Agrave"] = 173;
            macintoshStandardOrderGlyphIds["Atilde"] = 174;
            macintoshStandardOrderGlyphIds["Otilde"] = 175;
            macintoshStandardOrderGlyphIds["OE"] = 176;
            macintoshStandardOrderGlyphIds["oe"] = 177;
            macintoshStandardOrderGlyphIds["endash"] = 178;
            macintoshStandardOrderGlyphIds["emdash"] = 179;
            macintoshStandardOrderGlyphIds["quotedblleft"] = 180;
            macintoshStandardOrderGlyphIds["quotedblright"] = 181;
            macintoshStandardOrderGlyphIds["quoteleft"] = 182;
            macintoshStandardOrderGlyphIds["quoteright"] = 183;
            macintoshStandardOrderGlyphIds["divide"] = 184;
            macintoshStandardOrderGlyphIds["lozenge"] = 185;
            macintoshStandardOrderGlyphIds["ydieresis"] = 186;
            macintoshStandardOrderGlyphIds["Ydieresis"] = 187;
            macintoshStandardOrderGlyphIds["fraction"] = 188;
            macintoshStandardOrderGlyphIds["currency"] = 189;
            macintoshStandardOrderGlyphIds["guilsinglleft"] = 190;
            macintoshStandardOrderGlyphIds["guilsinglright"] = 191;
            macintoshStandardOrderGlyphIds["fi"] = 192;
            macintoshStandardOrderGlyphIds["fl"] = 193;
            macintoshStandardOrderGlyphIds["daggerdbl"] = 194;
            macintoshStandardOrderGlyphIds["periodcentered"] = 195;
            macintoshStandardOrderGlyphIds["quotesinglbase"] = 196;
            macintoshStandardOrderGlyphIds["quotedblbase"] = 197;
            macintoshStandardOrderGlyphIds["perthousand"] = 198;
            macintoshStandardOrderGlyphIds["Acircumflex"] = 199;
            macintoshStandardOrderGlyphIds["Ecircumflex"] = 200;
            macintoshStandardOrderGlyphIds["Aacute"] = 201;
            macintoshStandardOrderGlyphIds["Edieresis"] = 202;
            macintoshStandardOrderGlyphIds["Egrave"] = 203;
            macintoshStandardOrderGlyphIds["Iacute"] = 204;
            macintoshStandardOrderGlyphIds["Icircumflex"] = 205;
            macintoshStandardOrderGlyphIds["Idieresis"] = 206;
            macintoshStandardOrderGlyphIds["Igrave"] = 207;
            macintoshStandardOrderGlyphIds["Oacute"] = 208;
            macintoshStandardOrderGlyphIds["Ocircumflex"] = 209;
            macintoshStandardOrderGlyphIds["apple"] = 210;
            macintoshStandardOrderGlyphIds["Ograve"] = 211;
            macintoshStandardOrderGlyphIds["Uacute"] = 212;
            macintoshStandardOrderGlyphIds["Ucircumflex"] = 213;
            macintoshStandardOrderGlyphIds["Ugrave"] = 214;
            macintoshStandardOrderGlyphIds["dotlessi"] = 215;
            macintoshStandardOrderGlyphIds["circumflex"] = 216;
            macintoshStandardOrderGlyphIds["tilde"] = 217;
            macintoshStandardOrderGlyphIds["macron"] = 218;
            macintoshStandardOrderGlyphIds["breve"] = 219;
            macintoshStandardOrderGlyphIds["dotaccent"] = 220;
            macintoshStandardOrderGlyphIds["ring"] = 221;
            macintoshStandardOrderGlyphIds["cedilla"] = 222;
            macintoshStandardOrderGlyphIds["hungarumlaut"] = 223;
            macintoshStandardOrderGlyphIds["ogonek"] = 224;
            macintoshStandardOrderGlyphIds["caron"] = 225;
            macintoshStandardOrderGlyphIds["Lslash"] = 226;
            macintoshStandardOrderGlyphIds["lslash"] = 227;
            macintoshStandardOrderGlyphIds["Scaron"] = 228;
            macintoshStandardOrderGlyphIds["scaron"] = 229;
            macintoshStandardOrderGlyphIds["Zcaron"] = 230;
            macintoshStandardOrderGlyphIds["zcaron"] = 231;
            macintoshStandardOrderGlyphIds["brokenbar"] = 232;
            macintoshStandardOrderGlyphIds["Eth"] = 233;
            macintoshStandardOrderGlyphIds["eth"] = 234;
            macintoshStandardOrderGlyphIds["Yacute"] = 235;
            macintoshStandardOrderGlyphIds["yacute"] = 236;
            macintoshStandardOrderGlyphIds["Thorn"] = 237;
            macintoshStandardOrderGlyphIds["thorn"] = 238;
            macintoshStandardOrderGlyphIds["minus"] = 239;
            macintoshStandardOrderGlyphIds["multiply"] = 240;
            macintoshStandardOrderGlyphIds["onesuperior"] = 241;
            macintoshStandardOrderGlyphIds["twosuperior"] = 242;
            macintoshStandardOrderGlyphIds["threesuperior"] = 243;
            macintoshStandardOrderGlyphIds["onehalf"] = 244;
            macintoshStandardOrderGlyphIds["onequarter"] = 245;
            macintoshStandardOrderGlyphIds["threequarters"] = 246;
            macintoshStandardOrderGlyphIds["franc"] = 247;
            macintoshStandardOrderGlyphIds["Gbreve"] = 248;
            macintoshStandardOrderGlyphIds["gbreve"] = 249;
            macintoshStandardOrderGlyphIds["Idotaccent"] = 250;
            macintoshStandardOrderGlyphIds["Scedilla"] = 251;
            macintoshStandardOrderGlyphIds["scedilla"] = 252;
            macintoshStandardOrderGlyphIds["Cacute"] = 253;
            macintoshStandardOrderGlyphIds["cacute"] = 254;
            macintoshStandardOrderGlyphIds["Ccaron"] = 255;
            macintoshStandardOrderGlyphIds["ccaron"] = 256;
            macintoshStandardOrderGlyphIds["dcroat"] = 257;

            macintoshStandardOrderNames = new string[258];
            macintoshStandardOrderNames[0] = PostScriptConstants.NotDef;
            macintoshStandardOrderNames[1] = ".null";
            macintoshStandardOrderNames[2] = "nonmarkingreturn";
            macintoshStandardOrderNames[3] = "space";
            macintoshStandardOrderNames[4] = "exclam";
            macintoshStandardOrderNames[5] = "quotedbl";
            macintoshStandardOrderNames[6] = "numbersign";
            macintoshStandardOrderNames[7] = "dollar";
            macintoshStandardOrderNames[8] = "percent";
            macintoshStandardOrderNames[9] = "ampersand";
            macintoshStandardOrderNames[10] = "quotesingle";
            macintoshStandardOrderNames[11] = "parenleft";
            macintoshStandardOrderNames[12] = "parenright";
            macintoshStandardOrderNames[13] = "asterisk";
            macintoshStandardOrderNames[14] = "plus";
            macintoshStandardOrderNames[15] = "comma";
            macintoshStandardOrderNames[16] = "hyphen";
            macintoshStandardOrderNames[17] = "period";
            macintoshStandardOrderNames[18] = "slash";
            macintoshStandardOrderNames[19] = "zero";
            macintoshStandardOrderNames[20] = "one";
            macintoshStandardOrderNames[21] = "two";
            macintoshStandardOrderNames[22] = "three";
            macintoshStandardOrderNames[23] = "four";
            macintoshStandardOrderNames[24] = "five";
            macintoshStandardOrderNames[25] = "six";
            macintoshStandardOrderNames[26] = "seven";
            macintoshStandardOrderNames[27] = "eight";
            macintoshStandardOrderNames[28] = "nine";
            macintoshStandardOrderNames[29] = "colon";
            macintoshStandardOrderNames[30] = "semicolon";
            macintoshStandardOrderNames[31] = "less";
            macintoshStandardOrderNames[32] = "equal";
            macintoshStandardOrderNames[33] = "greater";
            macintoshStandardOrderNames[34] = "question";
            macintoshStandardOrderNames[35] = "at";
            macintoshStandardOrderNames[36] = "A";
            macintoshStandardOrderNames[37] = "B";
            macintoshStandardOrderNames[38] = "C";
            macintoshStandardOrderNames[39] = "D";
            macintoshStandardOrderNames[40] = "E";
            macintoshStandardOrderNames[41] = "F";
            macintoshStandardOrderNames[42] = "G";
            macintoshStandardOrderNames[43] = "H";
            macintoshStandardOrderNames[44] = "I";
            macintoshStandardOrderNames[45] = "J";
            macintoshStandardOrderNames[46] = "K";
            macintoshStandardOrderNames[47] = "L";
            macintoshStandardOrderNames[48] = "M";
            macintoshStandardOrderNames[49] = "N";
            macintoshStandardOrderNames[50] = "O";
            macintoshStandardOrderNames[51] = "P";
            macintoshStandardOrderNames[52] = "Q";
            macintoshStandardOrderNames[53] = "R";
            macintoshStandardOrderNames[54] = "S";
            macintoshStandardOrderNames[55] = "T";
            macintoshStandardOrderNames[56] = "U";
            macintoshStandardOrderNames[57] = "V";
            macintoshStandardOrderNames[58] = "W";
            macintoshStandardOrderNames[59] = "X";
            macintoshStandardOrderNames[60] = "Y";
            macintoshStandardOrderNames[61] = "Z";
            macintoshStandardOrderNames[62] = "bracketleft";
            macintoshStandardOrderNames[63] = "backslash";
            macintoshStandardOrderNames[64] = "bracketright";
            macintoshStandardOrderNames[65] = "asciicircum";
            macintoshStandardOrderNames[66] = "underscore";
            macintoshStandardOrderNames[67] = "grave";
            macintoshStandardOrderNames[68] = "a";
            macintoshStandardOrderNames[69] = "b";
            macintoshStandardOrderNames[70] = "c";
            macintoshStandardOrderNames[71] = "d";
            macintoshStandardOrderNames[72] = "e";
            macintoshStandardOrderNames[73] = "f";
            macintoshStandardOrderNames[74] = "g";
            macintoshStandardOrderNames[75] = "h";
            macintoshStandardOrderNames[76] = "i";
            macintoshStandardOrderNames[77] = "j";
            macintoshStandardOrderNames[78] = "k";
            macintoshStandardOrderNames[79] = "l";
            macintoshStandardOrderNames[80] = "m";
            macintoshStandardOrderNames[81] = "n";
            macintoshStandardOrderNames[82] = "o";
            macintoshStandardOrderNames[83] = "p";
            macintoshStandardOrderNames[84] = "q";
            macintoshStandardOrderNames[85] = "r";
            macintoshStandardOrderNames[86] = "s";
            macintoshStandardOrderNames[87] = "t";
            macintoshStandardOrderNames[88] = "u";
            macintoshStandardOrderNames[89] = "v";
            macintoshStandardOrderNames[90] = "w";
            macintoshStandardOrderNames[91] = "x";
            macintoshStandardOrderNames[92] = "y";
            macintoshStandardOrderNames[93] = "z";
            macintoshStandardOrderNames[94] = "braceleft";
            macintoshStandardOrderNames[95] = "bar";
            macintoshStandardOrderNames[96] = "braceright";
            macintoshStandardOrderNames[97] = "asciitilde";
            macintoshStandardOrderNames[98] = "Adieresis";
            macintoshStandardOrderNames[99] = "Aring";
            macintoshStandardOrderNames[100] = "Ccedilla";
            macintoshStandardOrderNames[101] = "Eacute";
            macintoshStandardOrderNames[102] = "Ntilde";
            macintoshStandardOrderNames[103] = "Odieresis";
            macintoshStandardOrderNames[104] = "Udieresis";
            macintoshStandardOrderNames[105] = "aacute";
            macintoshStandardOrderNames[106] = "agrave";
            macintoshStandardOrderNames[107] = "acircumflex";
            macintoshStandardOrderNames[108] = "adieresis";
            macintoshStandardOrderNames[109] = "atilde";
            macintoshStandardOrderNames[110] = "aring";
            macintoshStandardOrderNames[111] = "ccedilla";
            macintoshStandardOrderNames[112] = "eacute";
            macintoshStandardOrderNames[113] = "egrave";
            macintoshStandardOrderNames[114] = "ecircumflex";
            macintoshStandardOrderNames[115] = "edieresis";
            macintoshStandardOrderNames[116] = "iacute";
            macintoshStandardOrderNames[117] = "igrave";
            macintoshStandardOrderNames[118] = "icircumflex";
            macintoshStandardOrderNames[119] = "idieresis";
            macintoshStandardOrderNames[120] = "ntilde";
            macintoshStandardOrderNames[121] = "oacute";
            macintoshStandardOrderNames[122] = "ograve";
            macintoshStandardOrderNames[123] = "ocircumflex";
            macintoshStandardOrderNames[124] = "odieresis";
            macintoshStandardOrderNames[125] = "otilde";
            macintoshStandardOrderNames[126] = "uacute";
            macintoshStandardOrderNames[127] = "ugrave";
            macintoshStandardOrderNames[128] = "ucircumflex";
            macintoshStandardOrderNames[129] = "udieresis";
            macintoshStandardOrderNames[130] = "dagger";
            macintoshStandardOrderNames[131] = "degree";
            macintoshStandardOrderNames[132] = "cent";
            macintoshStandardOrderNames[133] = "sterling";
            macintoshStandardOrderNames[134] = "section";
            macintoshStandardOrderNames[135] = "bullet";
            macintoshStandardOrderNames[136] = "paragraph";
            macintoshStandardOrderNames[137] = "germandbls";
            macintoshStandardOrderNames[138] = "registered";
            macintoshStandardOrderNames[139] = "copyright";
            macintoshStandardOrderNames[140] = "trademark";
            macintoshStandardOrderNames[141] = "acute";
            macintoshStandardOrderNames[142] = "dieresis";
            macintoshStandardOrderNames[143] = "notequal";
            macintoshStandardOrderNames[144] = "AE";
            macintoshStandardOrderNames[145] = "Oslash";
            macintoshStandardOrderNames[146] = "infinity";
            macintoshStandardOrderNames[147] = "plusminus";
            macintoshStandardOrderNames[148] = "lessequal";
            macintoshStandardOrderNames[149] = "greaterequal";
            macintoshStandardOrderNames[150] = "yen";
            macintoshStandardOrderNames[151] = "mu";
            macintoshStandardOrderNames[152] = "partialdiff";
            macintoshStandardOrderNames[153] = "summation";
            macintoshStandardOrderNames[154] = "product";
            macintoshStandardOrderNames[155] = "pi";
            macintoshStandardOrderNames[156] = "integral";
            macintoshStandardOrderNames[157] = "ordfeminine";
            macintoshStandardOrderNames[158] = "ordmasculine";
            macintoshStandardOrderNames[159] = "Omega";
            macintoshStandardOrderNames[160] = "ae";
            macintoshStandardOrderNames[161] = "oslash";
            macintoshStandardOrderNames[162] = "questiondown";
            macintoshStandardOrderNames[163] = "exclamdown";
            macintoshStandardOrderNames[164] = "logicalnot";
            macintoshStandardOrderNames[165] = "radical";
            macintoshStandardOrderNames[166] = "florin";
            macintoshStandardOrderNames[167] = "approxequal";
            macintoshStandardOrderNames[168] = "Delta";
            macintoshStandardOrderNames[169] = "guillemotleft";
            macintoshStandardOrderNames[170] = "guillemotright";
            macintoshStandardOrderNames[171] = "ellipsis";
            macintoshStandardOrderNames[172] = "nonbreakingspace";
            macintoshStandardOrderNames[173] = "Agrave";
            macintoshStandardOrderNames[174] = "Atilde";
            macintoshStandardOrderNames[175] = "Otilde";
            macintoshStandardOrderNames[176] = "OE";
            macintoshStandardOrderNames[177] = "oe";
            macintoshStandardOrderNames[178] = "endash";
            macintoshStandardOrderNames[179] = "emdash";
            macintoshStandardOrderNames[180] = "quotedblleft";
            macintoshStandardOrderNames[181] = "quotedblright";
            macintoshStandardOrderNames[182] = "quoteleft";
            macintoshStandardOrderNames[183] = "quoteright";
            macintoshStandardOrderNames[184] = "divide";
            macintoshStandardOrderNames[185] = "lozenge";
            macintoshStandardOrderNames[186] = "ydieresis";
            macintoshStandardOrderNames[187] = "Ydieresis";
            macintoshStandardOrderNames[188] = "fraction";
            macintoshStandardOrderNames[189] = "currency";
            macintoshStandardOrderNames[190] = "guilsinglleft";
            macintoshStandardOrderNames[191] = "guilsinglright";
            macintoshStandardOrderNames[192] = "fi";
            macintoshStandardOrderNames[193] = "fl";
            macintoshStandardOrderNames[194] = "daggerdbl";
            macintoshStandardOrderNames[195] = "periodcentered";
            macintoshStandardOrderNames[196] = "quotesinglbase";
            macintoshStandardOrderNames[197] = "quotedblbase";
            macintoshStandardOrderNames[198] = "perthousand";
            macintoshStandardOrderNames[199] = "Acircumflex";
            macintoshStandardOrderNames[200] = "Ecircumflex";
            macintoshStandardOrderNames[201] = "Aacute";
            macintoshStandardOrderNames[202] = "Edieresis";
            macintoshStandardOrderNames[203] = "Egrave";
            macintoshStandardOrderNames[204] = "Iacute";
            macintoshStandardOrderNames[205] = "Icircumflex";
            macintoshStandardOrderNames[206] = "Idieresis";
            macintoshStandardOrderNames[207] = "Igrave";
            macintoshStandardOrderNames[208] = "Oacute";
            macintoshStandardOrderNames[209] = "Ocircumflex";
            macintoshStandardOrderNames[210] = "apple";
            macintoshStandardOrderNames[211] = "Ograve";
            macintoshStandardOrderNames[212] = "Uacute";
            macintoshStandardOrderNames[213] = "Ucircumflex";
            macintoshStandardOrderNames[214] = "Ugrave";
            macintoshStandardOrderNames[215] = "dotlessi";
            macintoshStandardOrderNames[216] = "circumflex";
            macintoshStandardOrderNames[217] = "tilde";
            macintoshStandardOrderNames[218] = "macron";
            macintoshStandardOrderNames[219] = "breve";
            macintoshStandardOrderNames[220] = "dotaccent";
            macintoshStandardOrderNames[221] = "ring";
            macintoshStandardOrderNames[222] = "cedilla";
            macintoshStandardOrderNames[223] = "hungarumlaut";
            macintoshStandardOrderNames[224] = "ogonek";
            macintoshStandardOrderNames[225] = "caron";
            macintoshStandardOrderNames[226] = "Lslash";
            macintoshStandardOrderNames[227] = "lslash";
            macintoshStandardOrderNames[228] = "Scaron";
            macintoshStandardOrderNames[229] = "scaron";
            macintoshStandardOrderNames[230] = "Zcaron";
            macintoshStandardOrderNames[231] = "zcaron";
            macintoshStandardOrderNames[232] = "brokenbar";
            macintoshStandardOrderNames[233] = "Eth";
            macintoshStandardOrderNames[234] = "eth";
            macintoshStandardOrderNames[235] = "Yacute";
            macintoshStandardOrderNames[236] = "yacute";
            macintoshStandardOrderNames[237] = "Thorn";
            macintoshStandardOrderNames[238] = "thorn";
            macintoshStandardOrderNames[239] = "minus";
            macintoshStandardOrderNames[240] = "multiply";
            macintoshStandardOrderNames[241] = "onesuperior";
            macintoshStandardOrderNames[242] = "twosuperior";
            macintoshStandardOrderNames[243] = "threesuperior";
            macintoshStandardOrderNames[244] = "onehalf";
            macintoshStandardOrderNames[245] = "onequarter";
            macintoshStandardOrderNames[246] = "threequarters";
            macintoshStandardOrderNames[247] = "franc";
            macintoshStandardOrderNames[248] = "Gbreve";
            macintoshStandardOrderNames[249] = "gbreve";
            macintoshStandardOrderNames[250] = "Idotaccent";
            macintoshStandardOrderNames[251] = "Scedilla";
            macintoshStandardOrderNames[252] = "scedilla";
            macintoshStandardOrderNames[253] = "Cacute";
            macintoshStandardOrderNames[254] = "cacute";
            macintoshStandardOrderNames[255] = "Ccaron";
            macintoshStandardOrderNames[256] = "ccaron";
            macintoshStandardOrderNames[257] = "dcroat";
        }
    }
}
