﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class PostFormat2 : Post
    {
        #region Constructors

        public PostFormat2(OpenTypeFontSourceBase fontSource)
            : base(fontSource)
        {
        }

        #endregion


        #region Fields

        private Dictionary<string, ushort> glyphNames;

        #endregion


        #region Methods

        private void CreateGlyphNamesMapping(ushort[] glyphNameIndex, string[] names)
        {
            this.glyphNames = new Dictionary<string, ushort>(glyphNameIndex.Length);
            for (int i = 0; i < glyphNameIndex.Length; i++)
            {
                ushort nameIndex = glyphNameIndex[i];
                if (nameIndex < macintoshStandardOrderNames.Length)
                {
                    this.glyphNames[macintoshStandardOrderNames[nameIndex]] = (ushort)i;
                }
                else
                {
                    nameIndex = (ushort)(nameIndex - macintoshStandardOrderNames.Length);
                    this.glyphNames[names[nameIndex]] = (ushort)i;
                }
            }
        }

        #endregion


        #region Overrides

        public override ushort GetGlyphId(string name)
        {
            ushort res;
            if (this.glyphNames.TryGetValue(name, out res))
            {
                return res;
            }
            return CMap.MISSING_GLYPH_ID;
        }

        public override void Read(OpenTypeFontReader reader)
        {
            base.Read(reader);
            ushort numberOfGlyphs = reader.ReadUShort();
            ushort[] glyphNameIndex = new ushort[numberOfGlyphs];
            int maxGlyphIndex = macintoshStandardOrderGlyphIds.Count - 1;
            for (int i = 0; i < numberOfGlyphs; i++)
            {
                glyphNameIndex[i] = reader.ReadUShort();
                if (glyphNameIndex[i] > maxGlyphIndex)
                {
                    maxGlyphIndex = glyphNameIndex[i];
                }
            }

            int newGlyphs = maxGlyphIndex - macintoshStandardOrderGlyphIds.Count + 1;
            string[] names = new string[newGlyphs];
            for (int i = 0; i < newGlyphs; i++)
            {
                names[i] = reader.ReadString();
            }

            this.CreateGlyphNamesMapping(glyphNameIndex, names);
        }

        #endregion
    }
}
