﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class Script : TableBase
    {
        #region Constructors

        public Script(OpenTypeFontSourceBase fontFile, uint scriptTag)
            : base(fontFile)
        {
            this.ScriptTag = scriptTag;
        }

        #endregion


        #region Fields

        private ushort defaultLangSysOffset;

        private LangSys defaultLangSys;

        #endregion


        #region Properties

        public uint ScriptTag
        {
            get;
            private set;
        }

        public LangSys DefaultLangSys
        {
            get
            {
                if (this.defaultLangSys == null && this.defaultLangSysOffset != TrueTypeConsts.NULL_OFFSET)
                {
                    this.defaultLangSys = this.ReadLangSys(this.Reader, this.defaultLangSysOffset);
                }
                return this.defaultLangSys;
            }
        }

        #endregion


        #region Methods

        private LangSys ReadLangSys(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            reader.Seek(this.Offset + offset, System.IO.SeekOrigin.Begin);
            LangSys ls = new LangSys(this.FontSource);
            ls.Read(reader);
            reader.EndReadingBlock();
            return ls;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            this.defaultLangSysOffset = reader.ReadUShort();
            //ushort langSysCount = reader.ReadUShort();
        }

        internal override void Write(FontWriter writer)
        {
            if (this.DefaultLangSys != null)
            {
                writer.WriteULong(this.ScriptTag);
                this.DefaultLangSys.Write(writer);
            }
            else
            {
                writer.WriteULong(Tags.NULL_TAG);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.defaultLangSys = new LangSys(this.FontSource);
            this.defaultLangSys.Import(reader);
        }

        public override string ToString()
        {
            return Tags.GetStringFromTag(this.ScriptTag);
        }

        #endregion
    }
}
