﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class ScriptList : TableBase
    {
        #region Constructors

        public ScriptList(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private Dictionary<uint, ScriptRecord> scriptRecords;
        private Dictionary<uint, Script> scripts;

        #endregion


        #region Properties
        #endregion


        #region Methods

        private Script ReadScript(OpenTypeFontReader reader, ScriptRecord record)
        {
            reader.BeginReadingBlock();
            long scriptOffset = this.Offset + record.ScriptOffset;
            reader.Seek(scriptOffset, System.IO.SeekOrigin.Begin);
            Script s = new Script(this.FontSource, record.ScriptTag);
            s.Offset = scriptOffset;
            s.Read(reader);
            reader.EndReadingBlock();
            return s;
        }



        public Script GetScript(uint tag)
        {
            Script s;
            if (!this.scripts.TryGetValue(tag, out s) && this.scriptRecords != null)
            {
                ScriptRecord r;
                if (this.scriptRecords.TryGetValue(tag, out r) || this.scriptRecords.TryGetValue(Tags.DEFAULT_TABLE_SCRIPT_TAG, out r))
                {
                    s = this.ReadScript(this.Reader, r);
                }
                else
                {
                    s = null;
                }
                this.scripts[tag] = s;
            }
            return s;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort count = reader.ReadUShort();
            this.scriptRecords = new Dictionary<uint, ScriptRecord>(count);
            this.scripts = new Dictionary<uint,Script>(count);
            for (int i = 0; i < count; i++)
            {
                ScriptRecord r = new ScriptRecord();
                r.Read(reader);
                this.scriptRecords[r.ScriptTag] = r;
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort((ushort)this.scriptRecords.Count);
            foreach (uint tag in this.scriptRecords.Keys)
            {
                Script s = this.ReadScript(this.Reader, this.scriptRecords[tag]);
                s.Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            this.scripts = new Dictionary<uint, Script>(cnt);
            for (int i = 0; i < cnt; i++)
            {
                uint tag = reader.ReadULong();
                if (tag != Tags.NULL_TAG)
                {
                    Script s = new Script(this.FontSource, tag);
                    s.Import(reader);
                    this.scripts[tag] = s;
                }
            }
        }

        #endregion
    }
}
