﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal abstract class SingleSubstitution : SubTable
    {
        #region StaticMembers

        internal static SingleSubstitution CreateSingleSubstitutionTable(OpenTypeFontSourceBase fontFile, ushort format)
        {
            switch (format)
            {
                case 1:
                    return new SingleSubstitutionFormat1(fontFile);
                case 2:
                    return new SingleSubstitutionFormat2(fontFile);
                default:
                    return null;
            }
        }

        #endregion

        #region Constructors

        public SingleSubstitution(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort coverageOffset;
        private Coverage coverage;

        #endregion


        #region Properties

        protected Coverage Coverage
        {
            get
            {
                if (this.coverage == null)
                {
                    this.coverage = this.ReadCoverage(this.Reader, this.coverageOffset);
                }
                return this.coverage;
            }
        }

        #endregion


        #region Methods

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            this.coverageOffset = reader.ReadUShort();
        }

        internal override void Write(FontWriter writer)
        {
            this.Coverage.Write(writer);
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            this.coverage = Coverage.ImportCoverageTable(this.FontSource, reader);
        }

        #endregion
    }
}
