﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class SingleSubstitutionFormat2 : SingleSubstitution
    {
        #region Constructors

        public SingleSubstitutionFormat2(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort[] substitutes;

        #endregion


        #region Overrides

        public override GlyphsSequence Apply(GlyphsSequence glyphIDs)
        {
            GlyphsSequence res = new GlyphsSequence(glyphIDs.Count);
            for (int i = 0; i < glyphIDs.Count; i++)
            {
                int coverageIndex = this.Coverage.GetCoverageIndex(glyphIDs[i].GlyphId);
                if (coverageIndex < 0)
                {
                    res.Add(glyphIDs[i]);
                }
                else
                {
                    res.Add(this.substitutes[coverageIndex]);
                }
            }
            return res;
        }

        public override void Read(OpenTypeFontReader reader)
        {
            base.Read(reader);
            ushort glyphCount = reader.ReadUShort();
            this.substitutes = new ushort[glyphCount];
            for (int i = 0; i < glyphCount; i++)
            {
                this.substitutes[i] = reader.ReadUShort();
            }
        }

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(2);
            base.Write(writer);
            writer.WriteUShort((ushort)this.substitutes.Length);
            for (int i = 0; i < this.substitutes.Length; i++)
            {
                writer.WriteUShort(this.substitutes[i]);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            base.Import(reader);
            ushort glyphCount = reader.ReadUShort();
            this.substitutes = new ushort[glyphCount];
            for (int i = 0; i < glyphCount; i++)
            {
                this.substitutes[i] = reader.ReadUShort();
            }
        }

        #endregion
    }
}
