﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class SubRuleSet : TableBase
    {
        #region Constructors

        public SubRuleSet(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private ushort[] subRuleOffsets;
        private SubRule[] subRules;

        #endregion


        #region Properties

        public SubRule[] SubRules
        {
            get
            {
                if (this.subRules == null)
                {
                    this.subRules = new SubRule[this.subRuleOffsets.Length];
                    for (int i = 0; i < this.subRules.Length; i++)
                    {
                        this.subRules[i] = this.ReadSubRule(this.Reader, this.subRuleOffsets[i]);
                    }
                }
                return this.subRules;
            }
        }

        #endregion


        #region Methods

        private SubRule ReadSubRule(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            long subRuleOffset = this.Offset + offset;
            reader.Seek(subRuleOffset, System.IO.SeekOrigin.Begin);
            SubRule sr = new SubRule(this.FontSource);
            sr.Read(reader);
            sr.Offset = subRuleOffset;
            reader.EndReadingBlock();
            return sr;
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort subRules = reader.ReadUShort();
            this.subRuleOffsets = new ushort[subRules];
            for (int i = 0; i < subRules; i++)
            {
                this.subRuleOffsets[i] = reader.ReadUShort();
            }
        }

        #endregion
    }
}
