﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal abstract class SubTable : TableBase
    {
        #region StaticMembers

        private static SubTable CreateSubTable(OpenTypeFontSourceBase fontSource, ushort type, OpenTypeFontReader reader)
        {
            SubTable st;
            long offset = reader.Position;
            switch ((LookupType)type)
            {
                case LookupType.Single:
                    ushort format = reader.ReadUShort();
                    st = SingleSubstitution.CreateSingleSubstitutionTable(fontSource, format);
                    break;
                case LookupType.Multiple:
                    st = new MultipleSubstitution(fontSource);
                    break;
                case LookupType.Ligature:
                    st = new LigatureSubstitution(fontSource);
                    break;
                default:
                    return null;
            }
            st.Offset = offset;
            return st;
        }

        internal static SubTable ReadSubTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader, ushort type)
        {
            SubTable st = CreateSubTable(fontSource, type, reader);
            if (st != null)
            {
                st.Read(reader);
            }
            return st;
        }

        internal static SubTable ImportSubTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader, ushort type)
        {
            SubTable st = CreateSubTable(fontSource, type, reader);
            if (st != null)
            {
                st.Import(reader);
            }
            return st;
        }

        #endregion


        #region Constructors

        public SubTable(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Methods

        protected Coverage ReadCoverage(OpenTypeFontReader reader, ushort offset)
        {
            reader.BeginReadingBlock();
            reader.Seek(this.Offset + offset, System.IO.SeekOrigin.Begin);
            Coverage c = Coverage.ReadCoverageTable(this.FontSource, reader);
            reader.EndReadingBlock();
            return c;
        }


        public abstract GlyphsSequence Apply(GlyphsSequence glyphIndices);

        #endregion
    }
}
