﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables
{
    internal class TCCHeader
    {
        #region StaticMembers

        private static OpenTypeFontSource ReadTrueTypeFontFile(OpenTypeFontReader reader, uint offset)
        {
            reader.BeginReadingBlock();
            reader.Seek(offset, System.IO.SeekOrigin.Begin);
            OpenTypeFontSource res = new OpenTypeFontSource(reader);
            //res.Offset = offset;
            reader.EndReadingBlock();
            return res;
        }

        #endregion


        #region Constructors

        public TCCHeader(TrueTypeCollection collection)
        {
            this.collection = collection;
        }

        #endregion


        #region Fields

        private readonly TrueTypeCollection collection;
        
        private uint[] offsetTable;
        private OpenTypeFontSourceBase[] fonts;

        #endregion


        #region Properties

        protected OpenTypeFontReader Reader
        {
            get
            {
                return this.collection.Reader;
            }
        }

        public OpenTypeFontSourceBase[] Fonts
        {
            get
            {
                if (this.fonts == null)
                {
                    this.fonts = new OpenTypeFontSourceBase[this.offsetTable.Length];
                    for (int i = 0; i < this.offsetTable.Length; i++)
                    {
                        this.fonts[i] = ReadTrueTypeFontFile(this.Reader, this.offsetTable[i]);
                    }
                }
                return this.fonts;
            }
        }

        #endregion


        #region Methods
        #endregion


        #region Overrides

        public void Read(OpenTypeFontReader reader)
        {
            reader.ReadULong();
            reader.ReadFixed();
            uint numFonts = reader.ReadULong();
            this.offsetTable = new uint[numFonts];
            for (int i = 0; i < numFonts; i++)
            {
                this.offsetTable[i] = reader.ReadULong();
            }
        }

        #endregion
    }
}
