﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;
using Telerik.Windows.Documents.Core.Data;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeOutlines
{
    internal class CompositeGlyph : GlyphData
    {
        #region StaticMembers
        
        private static OutlinePoint GetTransformedPoint(Matrix matrix, OutlinePoint point)
        {
            OutlinePoint res = new OutlinePoint(point.Flags);
            res.Point = matrix.Transform(point.Point);
            return res;
        }
        
        #endregion


        #region Constructors

        public CompositeGlyph(OpenTypeFontSourceBase fontFile, ushort glyphIndex)
            : base(fontFile, glyphIndex)
        {
        }

        #endregion


        #region Fields

        //private ushort metricsGlyphIndex;
        private List<OutlinePoint[]> contours;

        #endregion


        #region Properties

        internal override IEnumerable<OutlinePoint[]> Contours
        {
            get
            {
                return this.contours;
            }
        }

        #endregion


        #region Methods

        private void AddGlyph(GlyphDescription gd)
        {
            //if (gd.CheckFlag(GlyphDescription.USE_MY_METRICS))
            //{
            //    this.metricsGlyphIndex = gd.GlyphIndex;
            //}
            GlyphData glyph = this.FontSource.GetGlyphData(gd.GlyphIndex);
            
            if (glyph == null)
            {
                return;
            }
           
            foreach (OutlinePoint[] contour in glyph.Contours)
            {
                OutlinePoint[] transformedContour = new OutlinePoint[contour.Length];
                for (int i = 0; i < contour.Length; i++)
                {
                    transformedContour[i] = GetTransformedPoint(gd.Transform, contour[i]);
                }

                this.contours.Add(transformedContour);
            }
        }

        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            this.contours = new List<OutlinePoint[]>();
            //this.metricsGlyphIndex = this.GlyphIndex;
            GlyphDescription gd = null;
            
            do
            {
                gd = new GlyphDescription();
                gd.Read(reader);
                this.AddGlyph(gd);
            } while (gd.CheckFlag(GlyphDescription.MORE_COMPONENTS));
        }

        #endregion
    }
}
