﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeOutlines
{
    internal class GlyphData : TrueTypeTableBase
    {
        #region StaticMembers

        public static GlyphData ReadGlyf(OpenTypeFontSourceBase fontFile, ushort glyphIndex)
        {
            short contours = fontFile.Reader.ReadShort();
            GlyphData res;
            if (contours == 0)
            {
                res = new GlyphData(fontFile, glyphIndex);
            }
            else if (contours > 0)
            {
                res = new SimpleGlyph(fontFile, glyphIndex);
            }
            else
            {
                res = new CompositeGlyph(fontFile, glyphIndex);
            }

            res.NumberOfContours = contours;
            res.BoundingRect = new Rect(new Point(fontFile.Reader.ReadShort(), fontFile.Reader.ReadShort()), new Point(fontFile.Reader.ReadShort(), fontFile.Reader.ReadShort()));
            res.Read(fontFile.Reader);
            return res;
        }

        #endregion


        #region Constructors

        public GlyphData(OpenTypeFontSourceBase fontFile, ushort glyphIndex)
            : base(fontFile)
        {
            this.glyphIndex = glyphIndex;
            this.BoundingRect = Rect.Empty;
        }

        #endregion


        #region Fields

        private readonly ushort glyphIndex;

        #endregion


        #region Properties

        internal virtual IEnumerable<OutlinePoint[]> Contours
        {
            get
            {
                return Enumerable.Empty<OutlinePoint[]>();
            }
        }

        internal override uint Tag
        {
            get { return Tags.GLYF_TABLE; }
        }

        internal short NumberOfContours
        {
            get;
            set;
        }



        public ushort GlyphIndex
        {
            get
            {
                return this.glyphIndex;
            }
        }

        public Rect BoundingRect
        {
            get;
            set;
        }

        #endregion


        #region Methods

        override public void Read(OpenTypeFontReader reader)
        {
        }

        #endregion
    }
}
