﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.OpenType;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeOutlines
{
    internal class IndexToLocation : TrueTypeTableBase
    {
        #region Constructors

        public IndexToLocation(OpenTypeFontSource fontFile)
            : base(fontFile)
        {
        }

        #endregion


        #region Fields

        private uint[] offsets;

        #endregion


        #region Properties

        internal override uint Tag
        {
            get { return Tags.LOCA_TABLE; }
        }

        #endregion


        #region Methods

        public long GetOffset(ushort index)
        {
            if (this.offsets == null || index >= this.offsets.Length || (index < this.offsets.Length - 1 && this.offsets[index + 1] == this.offsets[index]))
            {
                return -1;
            }
            return this.offsets[index];
        }

        override public void Read(OpenTypeFontReader reader)
        {
            this.offsets = new uint[this.FontSource.GlyphCount+ 1];
            for (int i = 0; i < this.offsets.Length; i++)
            {
                if (this.FontSource.Head.IndexToLocFormat == 0)
                {
                    this.offsets[i] =(uint)(2 * reader.ReadUShort());
                }
                else
                {
                    this.offsets[i] = reader.ReadULong();
                }
            }
        }

        #endregion
    }
}
