﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;
using Telerik.Windows.Documents.Core.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeOutlines
{
    internal class SimpleGlyph : GlyphData
    {
        #region StaticMembers

        private static bool XIsByte(byte[] flags, int index)
        {
            return BitsHelper.GetBit(flags[index], 1);
        }
        private static bool YIsByte(byte[] flags, int index)
        {
            return BitsHelper.GetBit(flags[index], 2);
        }
        private static bool Repeat(byte[] flags, int index)
        {
            return BitsHelper.GetBit(flags[index], 3);
        }
        private static bool XIsSame(byte[] flags, int index)
        {
            return BitsHelper.GetBit(flags[index], 4);
        }
        private static bool YIsSame(byte[] flags, int index)
        {
            return BitsHelper.GetBit(flags[index], 5);
        }

        #endregion


        #region Constructors

        public SimpleGlyph(OpenTypeFontSourceBase fontFile, ushort glyphIndex)
            : base(fontFile, glyphIndex)
        {
        }

        #endregion
        

        #region Fields

        private List<OutlinePoint[]> contours;

        #endregion


        #region Properties

        internal override IEnumerable<OutlinePoint[]> Contours
        {
            get
            {
                return this.contours;
            }
        }


        #endregion


        #region Methods
        #endregion


        #region Overrides

        public override void Read(OpenTypeFontReader reader)
        {
            ushort[] endPtsOfContours = new ushort[this.NumberOfContours];
            for (int i = 0; i < this.NumberOfContours; i++)
            {
                endPtsOfContours[i] = reader.ReadUShort();
            }

            int numPoints = endPtsOfContours[this.NumberOfContours - 1] + 1;
            
            ushort numInstructions = reader.ReadUShort();
            byte[] instructions = new byte[numInstructions];
            for (int i = 0; i < numInstructions; i++)
            {
                instructions[i] = reader.Read();
            }

            byte[] flags = new byte[numPoints];
            for (int i = 0; i < numPoints; i++)
            {
                flags[i] = reader.Read();

                if (Repeat(flags, i))
                {
                    byte fl = flags[i];
                    byte n = reader.Read();
                    for (int c = 0; c < n; c++)
                    {
                        flags[++i] = fl;
                    }
                }
            }

            int[] xCoords = new int[numPoints];
            for (int i = 0; i < numPoints; i++)
            {
                if (i > 0)
                {
                    xCoords[i] = xCoords[i - 1];
                }
                if (XIsByte(flags, i))
                {
                    int val = reader.Read();
                    if (!XIsSame(flags, i))
                    {
                        val = -val;
                    }
                    xCoords[i] += val;
                }
                else if (!XIsSame(flags, i))
                {
                    xCoords[i] += reader.ReadShort();
                }
            }

            int[] yCoords = new int[numPoints];
            for (int i = 0; i < numPoints; i++)
            {
                if (i > 0)
                {
                    yCoords[i] = yCoords[i - 1];
                }
                if (YIsByte(flags, i))
                {
                    int val = reader.Read();
                    if (!YIsSame(flags, i))
                    {
                        val = -val;
                    }
                    yCoords[i] += val;
                }
                else if (!YIsSame(flags, i))
                {
                    yCoords[i] += reader.ReadShort();
                }
            }

            this.contours = new List<OutlinePoint[]>();
            int contourNo = 0;
            int contourPt = 0;
            OutlinePoint[] contour = new OutlinePoint[endPtsOfContours[0] + 1];
            this.contours.Add(contour);

            for (int i = 0; i < numPoints; i++)
            {
                contour[contourPt++] = new OutlinePoint(xCoords[i], yCoords[i], flags[i]);
                if (i == endPtsOfContours[contourNo])
                {
                    if (contourNo == endPtsOfContours.Length - 1)
                    {
                        break;
                    }
                    contourNo++;
                    contour = new OutlinePoint[endPtsOfContours[contourNo] - endPtsOfContours[contourNo - 1]];
                    this.contours.Add(contour);
                    contourPt = 0;
                }
            }
        }

        #endregion
    }
}
