﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeTables
{
    internal class Format0KerningSubTable : KerningSubTable
    {
        #region Constructors

        public Format0KerningSubTable(OpenTypeFontSourceBase fontSource)
            : base(fontSource)
        {
        }

        #endregion


        #region Fields

        private Dictionary<ushort, Dictionary<ushort, short>> values;

        #endregion


        #region Overrides

        public override short GetValue(ushort leftGlyphIndex, ushort rightGlyphIndex)
        {
            Dictionary<ushort, short> leftValues;
            if (!this.values.TryGetValue(leftGlyphIndex, out leftValues))
            {
                return 0;
            }
            short res;
            if (!leftValues.TryGetValue(rightGlyphIndex, out res))
            {
                return 0;
            }
            return res;
        }

        public override void Read(OpenTypeFontReader reader)
        {
            ushort pairs = reader.ReadUShort();
            this.values = new Dictionary<ushort, Dictionary<ushort, short>>(pairs);
            reader.ReadUShort();
            reader.ReadUShort();
            reader.ReadUShort();
            for (int i = 0; i < pairs; i++)
            {
                ushort left = reader.ReadUShort();
                ushort right = reader.ReadUShort();
                short value = reader.ReadShort();
                Dictionary<ushort, short> leftValues;
                if (!this.values.TryGetValue(left, out leftValues))
                {
                    leftValues = new Dictionary<ushort, short>();
                    this.values[left] = leftValues;
                }
                leftValues[right] = value;
            }
        }

        internal override void Write(FontWriter writer)
        {
            base.Write(writer);
            writer.WriteUShort((ushort)this.values.Count);
            foreach (ushort left in this.values.Keys)
            {
                var k = this.values[left];
                writer.WriteUShort(left);
                writer.WriteUShort((ushort)k.Count);
                foreach (ushort right in k.Keys)
                {
                    writer.WriteUShort(right);
                    writer.WriteShort(k[right]);
                }
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            this.values = new Dictionary<ushort, Dictionary<ushort, short>>(cnt);
            for (int i = 0; i < cnt; i++)
            {
                ushort left = reader.ReadUShort();
                ushort kCnt = reader.ReadUShort();
                Dictionary<ushort, short> k = new Dictionary<ushort, short>(kCnt);
                for (int j = 0; j < kCnt; j++)
                {
                    ushort right = reader.ReadUShort();
                    k[right] = reader.ReadShort();
                }
                this.values[left] = k;
            }
        }

        #endregion
    }
}
