﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeTables
{
    internal class Kerning : TrueTypeTableBase
    {
        #region Constructors

        public Kerning(OpenTypeFontSourceBase fontFile)
            : base(fontFile)
        {

        }

        #endregion


        #region Fields

        private List<KerningSubTable> subTables;

        #endregion


        #region Properties

        internal override uint Tag
        {
            get { return Tags.KERN_TABLE; }
        }

        #endregion


        #region Methods

        #endregion


        #region Overrides

        public KerningInfo GetKerning(ushort leftGlyphIndex, ushort rightGlyphIndex)
        {
            double horX = 0, verX = 0, horY = 0, verY = 0;
            short? horMinX = null, horMinY = null, verMinX = null, verMinY = null;
            short value;
            foreach (KerningSubTable st in this.subTables)
            {
                value = st.GetValue(leftGlyphIndex, rightGlyphIndex);
                if (st.IsHorizontal)
                {
                    if (st.HasMinimumValues)
                    {
                        if (st.IsCrossStream)
                        {
                            horMinY = value;
                        }
                        else
                        {
                            horMinX = value;
                        }
                    }
                    else
                    {
                        if (st.IsCrossStream)
                        {
                            if (st.Override)
                            {
                                horY = value;
                            }
                            else
                            {
                                horY += value;
                            }
                        }
                        else
                        {
                            if (st.Override)
                            {
                                horX += value;
                            }
                            else
                            {
                                horX = value;
                            }
                        }
                    }
                }
                else
                {
                    if (st.HasMinimumValues)
                    {
                        if (st.IsCrossStream)
                        {
                            verMinY = value;
                        }
                        else
                        {
                            verMinX = value;
                        }
                    }
                    else
                    {
                        if (st.IsCrossStream)
                        {
                            if (st.Override)
                            {
                                verY = value;
                            }
                            else
                            {
                                verY += value;
                            }
                        }
                        else
                        {
                            if (st.Override)
                            {
                                verX = value;
                            }
                            else
                            {
                                verX += value;
                            }
                        }
                    }
                }
            }
            if (horMinX.HasValue && horMinX > horX)
            {
                horX = horMinX.Value;
            }
            if (horMinY.HasValue && horMinY > horY)
            {
                horY = horMinY.Value;
            }
            if (verMinX.HasValue && verMinX > verX)
            {
                verX = verMinX.Value;
            }
            if (verMinY.HasValue && verMinY > verY)
            {
                verY = verMinY.Value;
            }
            return new KerningInfo { HorizontalKerning = new Point(horX, horY), VerticalKerning = new Point(verX, verY) };
        }

        public override void Read(OpenTypeFontReader reader)
        {
            reader.ReadUShort();
            ushort tables = reader.ReadUShort();
            this.subTables = new List<KerningSubTable>(tables);
            for (int i = 0; i < tables; i++)
            {
                KerningSubTable table = KerningSubTable.ReadSubTable(this.FontSource, reader);
                if (table != null)
                {
                    this.subTables.Add(table);
                }
            }
        }

        internal override void Write(FontWriter writer)
        {
            if (this.subTables == null)
            {
                writer.WriteUShort(0);
                return;
            }
            writer.WriteUShort((ushort)this.subTables.Count);
            for (int i = 0; i < this.subTables.Count; i++)
            {
                this.subTables[i].Write(writer);
            }
        }

        internal override void Import(OpenTypeFontReader reader)
        {
            ushort cnt = reader.ReadUShort();
            if (cnt > 0)
            {
                this.subTables = new List<KerningSubTable>(cnt);
                for (int i = 0; i < cnt; i++)
                {
                    this.subTables.Add(KerningSubTable.ImportSubTable(this.FontSource, reader));
                }
            }
        }

        #endregion
    }
}
