﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.TrueTypeTables
{
    internal abstract class KerningSubTable : TableBase
    {
        #region StaticMembers

        internal static KerningSubTable ReadSubTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            long position = reader.Position;
            reader.ReadUShort();
            ushort length = reader.ReadUShort();
            ushort coverage = reader.ReadUShort();
            byte format = BitConverter.GetBytes(coverage)[1];
            KerningSubTable res;
            switch (format)
            {
                case 0:
                    res = new Format0KerningSubTable(fontSource);
                    break;
                default:
                    reader.Seek(position + length, System.IO.SeekOrigin.Begin);
                    return null;
            }
            res.Coverage = coverage;
            res.Read(reader);
            return res;
        }

        internal static KerningSubTable ImportSubTable(OpenTypeFontSourceBase fontSource, OpenTypeFontReader reader)
        {
            ushort coverage = reader.ReadUShort();
            byte format = BitConverter.GetBytes(coverage)[1];
            KerningSubTable res;
            switch (format)
            {
                case 0:
                    res = new Format0KerningSubTable(fontSource);
                    break;
                default:
                    return null;
            }
            res.Coverage = coverage;
            res.Import(reader);
            return res;
        }

        #endregion


        #region Constructors

        public KerningSubTable(OpenTypeFontSourceBase fontSource)
            : base (fontSource)
        {
        }

        #endregion


        #region Fields
        #endregion


        #region Properties

        public ushort Coverage
        {
            get;
            private set;
        }

        public bool IsHorizontal
        {
            get
            {
                return this.GetBit(0);
            }
        }

        public bool HasMinimumValues
        {
            get
            {
                return this.GetBit(1);
            }
        }

        public bool IsCrossStream
        {
            get
            {
                return this.GetBit(2);
            }
        }

        public bool Override
        {
            get
            {
                return this.GetBit(3);
            }
        }

        #endregion


        #region Methods

        private bool GetBit(byte bit)
        {
            return BitsHelper.GetBit(this.Coverage, bit);
        }

        public abstract short GetValue(ushort leftGlyphIndex, ushort rightGlyphIndex);

        internal override void Write(FontWriter writer)
        {
            writer.WriteUShort(this.Coverage);
        }

        #endregion

        
    }
}
