﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Tables.Utils
{
    internal class Segment
    {
        #region Consts
        #endregion


        #region Constructors

        internal Segment()
        {
        }

        public Segment(ushort startCode, ushort endCode, short idDelta)
        {
            this.startCode = startCode;
            this.endCode = endCode;
            this.idDelta = idDelta;
        }

        public Segment(ushort startCode, ushort endCode, short idDelta, ushort[] map)
            : this(startCode, endCode, idDelta)
	    {
            this.map = new ValueMapper<ushort,ushort>();
            for (ushort i = 0; i < map.Length; i++)
            {
                this.map.AddPair(i, map[i]);
            }
	    }

        #endregion


        #region Fields

        private readonly ushort startCode;
        private readonly ushort endCode;
        private readonly short idDelta;
        private readonly ValueMapper<ushort, ushort> map;

        #endregion


        #region Properties

        #endregion


        #region Methods

        public bool IsInside(int unicode)
        {
            return this.startCode <= unicode && this.endCode >= unicode;
        }

        public bool Contains(ushort glyphId)
        {
            if (this.map != null)
            {
                return this.map.ContainsToValue(glyphId);
            }
            else
            {
                ushort first = (ushort)(this.startCode + this.idDelta);
                ushort last = (ushort)(this.endCode + this.idDelta);

                return first <= glyphId && glyphId <= last;
            }
        }

        internal ushort GetCharId(ushort glyphId)
        {
            if (this.map != null)
            {
                return (ushort)(this.startCode + this.map.GetFromValue((ushort)(glyphId - this.idDelta)));
            }
            else
            {
                return (ushort)(glyphId - this.idDelta);
            }
        }

        public bool TryGetGlyphId(int unicode, out ushort glyphId)
        {
            glyphId = CMap.MISSING_GLYPH_ID;
            if (this.map != null)
            {
                ushort index = (ushort)(unicode - startCode);

                if (index > this.map.Count || this.map.GetToValue(index) == CMap.MISSING_GLYPH_ID)
                {
                    return false;
                }

                glyphId = (ushort)(this.map.GetToValue(index) + this.idDelta);
            }
            else
            {
                glyphId = (ushort)(unicode + this.idDelta);
            }

            return true;
        }

        public void Write(FontWriter writer)
        {
        }

        public void Import(OpenTypeFontReader reader)
        {
        }

        #endregion


        #region Overrides

        public override int GetHashCode()
        {
            unchecked
            {
                int result = 17;
                result = result * 23 + this.startCode.GetHashCode();
                result = result * 23 + this.endCode.GetHashCode();
                return result;
            }
        }

        public override bool Equals(object obj)
        {
            Segment s = obj as Segment;
            if (s == null)
            {
                return false;
            }
            return this.startCode == s.startCode && this.endCode == s.endCode;
        }

        #endregion
    }
}
