﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Utils
{
    internal class FallbackRange
    {
        #region StaticMembers

        private static List<FallbackRange> fallbackRanges;

        private static void InitializeFallbackRanges()
        {
            fallbackRanges = new List<FallbackRange>();
            fallbackRanges.Add(new FallbackRange(new Fonts.Utils.Range[] { new Fonts.Utils.Range(0x0000, 0x024F), new Fonts.Utils.Range(0x0400, 0x052F) }, new string[] { "Times New Roman" })); 
        }

        internal static FallbackRange GetFallbackRange(char unicode)
        {
            foreach (FallbackRange fr in fallbackRanges)
            {
                if (fr.FallsInRange(unicode))
                {
                    return fr;
                }
            }
            return null;
        }

        #endregion


        #region Constructors

        static FallbackRange()
        {
            InitializeFallbackRanges();
        }

        public FallbackRange(Fonts.Utils.Range[] ranges, string[] fallbackFontFamilies)
        {
            this.Ranges = ranges;
            this.FallbackFontFamilies = fallbackFontFamilies;
        }

        #endregion


        #region Properties

        public Fonts.Utils.Range[] Ranges
        {
            get;
            private set;
        }

        public string[] FallbackFontFamilies
        {
            get;
            private set;
        }

        #endregion


        #region Methods

        public bool FallsInRange(char unicode)
        {
            foreach (Fonts.Utils.Range r in this.Ranges)
            {
                if (r.IsInRange(unicode))
                {
                    return true;
                }
            }
            return false;
        }

        #endregion
    }
}
