﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.OpenType.Utils
{
    internal class UnicodeRange
    {
        #region StaticMembers

        private static List<UnicodeRange> unicodeRanges;
        private static UnicodeRange defaultRange;

        private static int UnicodeRangeComparison(UnicodeRange range, char unicode)
        {
            if (unicode < range.Range.RangeStart)
            {
                return -1;
            }
            if (range.Range.RangeStart <= unicode && unicode <= range.Range.RangeEnd)
            {
                return 0;
            }
            return 1;
        }

        private static int UnicodeRangeComparison(UnicodeRange left, UnicodeRange range)
        {
            return left.Range.RangeStart.CompareTo(range.Range.RangeStart);
        }

        private static void InitializeUnicodeRanges()
        {
            unicodeRanges = new List<UnicodeRange>();
            unicodeRanges.Add(new UnicodeRange("arab", 0x0600, 0x06FF, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("armn", 0x0530, 0x058F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("avst", 0x10B00, 0x10B3F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("bali", 0x1B00, 0x1B7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("bamu", 0xA6A0, 0xA6FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("batk", 0x1BC0, 0x1BFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("beng", 0x0980, 0x09FF, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("bng2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("bopo", 0x3100, 0x31FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("brah", 0x11000, 0x1107F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("brai", 0x2800, 0x28FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("bugi", 0x1A00, 0x1A1F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("buhd", 0x1740, 0x175F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("byzm", 0x1D000, 0x1D0FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cans", 0x1400, 0x167F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cari", 0x102A0, 0x10DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cakm", 0x11100, 0x1114F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cham", 0xAA00, 0xAA5F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cher", 0x13A0, 0x13FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("hani", 0x4E00, 0x9FFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("copt", 0x2C80, 0x2CFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("cprt", 0x10800, 0x1083F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("cyrl", 0x0400, 0x04FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("dsrt", 0x10400, 0x1044F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("deva", 0x0900, 0x097F, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("dev2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("egyp", 0x13000, 0x1342F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("ethi", 0x1200, 0x137F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("geor", 0x10A0, 0x10FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("glag", 0x2C00, 0x2C5F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("goth", 0x10330, 0x1034F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("grek", 0x0370, 0x03FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("gujr", 0x0A80, 0x0AFF, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("gjr2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("guru", 0x0A00, 0x0A7F, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("gur2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("hang", 0xAC00, 0xD7AF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("jamo", 0x1100, 0x11FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("hano", 0x1720, 0x173F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("hebr", 0x0590, 0x05FF, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("kana", 0x3040, 0x309F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("armi", 0x10840, 0x1085F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("phli", 0x10B60, 0x10B7F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("prti", 0x10B40, 0x40B5F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("java", 0xA980, 0xA9DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("kthi", 0x11080, 0x110CF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("knda", 0x0C80, 0x0CFF, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("knd2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("kana", 0x30A0, 0x30FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("kali", 0xA900, 0xA92F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("khar", 0x10A00, 0x10A5F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("khmr", 0x1780, 0x17FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lao ", 0x0E80, 0x0EFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("latn", 0x0000, 0x007F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lepc", 0x1C00, 0x1C4F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("limb", 0x1900, 0x194F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("linb", 0x10000, 0x1007F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lisu", 0xA4D0, 0xA4FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lyci", 0x10280, 0x1029F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lydi", 0x10920, 0x1093F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("mlym", 0x0D00, 0x0D7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mlm2", 0x0D00, 0x0D7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mand", 0x0840, 0x085F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("math", 0x1D400, 0x1D7FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mtei", 0xABC0, 0xABFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("merc", 0x109A0, 0x109FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mero", 0x10980, 0x1099F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mong", 0x1800, 0x18AF, TextDirection.TopToBottom));
            unicodeRanges.Add(new UnicodeRange("musc", 0x1D100, 0x1D1FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("mymr", 0x1000, 0x109F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("talu", 0x1980, 0x19DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("nko ", 0x07C0, 0x07CF, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("orya", 0x0B00, 0x0B7F, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("ory2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("ogam", 0x1680, 0x169F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("olck", 0x1C50, 0x1C7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("ital", 0x10300, 0x1032F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("xpeo", 0x103A0, 0x103DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("sarb", 0x10A60, 0x10A7F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("orkh", 0x10C00, 0x10C4F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("osma", 0x10480, 0x104AF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("phag", 0xA840, 0xA87F, TextDirection.TopToBottom));
            unicodeRanges.Add(new UnicodeRange("phnx", 0x10900, 0x1091F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("rjng", 0xA930, 0xA95F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("runr", 0x16A0, 0x16FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("samr", 0x0800, 0x083F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("saur", 0xA880, 0xA8DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("shrd", 0x11180, 0x111DF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("shaw", 0x10450, 0x1047F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("sinh", 0x0D80, 0x0DFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("sora", 0x110D0, 0x110FF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("sund", 0x1B80, 0x1BBF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("sylo", 0xA800, 0xA82F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("syrc", 0x0700, 0x074F, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("tglg", 0x1700, 0x171F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("tagb", 0x1760, 0x177F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("tale", 0x1950, 0x197F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("lana", 0x1A20, 0x1AAF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("tavt", 0xAA80, 0xAADF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("takr", 0x11680, 0x116CF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("taml", 0x0B80, 0x0BFF, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("tml2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("telu", 0x0C00, 0x0C7F, TextDirection.LeftToRight));
            //unicodeRanges.Add(new UnicodeRange("tel2", , , TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("thaa", 0x0780, 0x07BF, TextDirection.RightToLeft));
            unicodeRanges.Add(new UnicodeRange("thai", 0x0E00, 0x0E7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("tibt", 0x0F00, 0x0FFF, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("tfng", 0x2D30, 0x2D7F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("ugar", 0x10380, 0x1039F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("vai ", 0xA500, 0xA63F, TextDirection.LeftToRight));
            unicodeRanges.Add(new UnicodeRange("yi  ", 0xA000, 0xA4CF, TextDirection.LeftToRight));
            unicodeRanges.Sort(UnicodeRangeComparison);
        }

        internal static UnicodeRange GetUnicodeRange(char unicode)
        {
            UnicodeRange ur = CollectionsExtensions.FindElement(unicodeRanges, unicode, UnicodeRangeComparison);
            if (ur.Range.RangeStart <= unicode && unicode <= ur.Range.RangeEnd)
            {
                return ur;
            }
            return defaultRange;
        }

        #endregion


        #region Constructors

        static UnicodeRange()
        {
            defaultRange = new UnicodeRange("DFLT");
            InitializeUnicodeRanges();
        }

        public UnicodeRange(string tag)
        {
            this.Tag = Tags.GetTagFromString(tag);
            this.Direction = TextDirection.LeftToRight;
            this.Range = new Fonts.Utils.Range();
        }

        public UnicodeRange(string tag, int rangeStart, int rangeEnd, TextDirection direction)
        {
            this.Tag = Tags.GetTagFromString(tag);
            this.Range = new Fonts.Utils.Range(rangeStart, rangeEnd);
            this.Direction = direction;
        }

        #endregion


        #region Properties

        public Fonts.Utils.Range Range
        {
            get;
            private set;
        }

        public uint Tag
        {
            get;
            private set;
        }

        public TextDirection Direction
        {
            get;
            private set;
        }

        #endregion


        #region Overrides

        public override string ToString()
        {
            return Tags.GetStringFromTag(this.Tag);
        }

        #endregion
    }
}
