﻿using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables;

namespace Telerik.Windows.Documents.Core.Fonts.Type1
{
    internal class CFFFontFile : ICFFFontFile
    {
        #region Constructors

        public CFFFontFile(byte[] data)
        {
            this.reader = new CFFFontReader(data);
            this.Initialize();
        }

        #endregion


        #region Fields

        private readonly CFFFontReader reader;
        private CFFFontSource fontSource;

        #endregion


        #region Properties

        public CFFFontSource FontSource
        {
            get
            {
                return this.fontSource;
            }
        }

        public Header Header
        {
            get;
            private set;
        }

        public NameIndex Name
        {
            get;
            private set;
        }

        public TopIndex TopIndex
        {
            get;
            private set;
        }

        public StringIndex StringIndex
        {
            get;
            private set;
        }

        public SubrsIndex GlobalSubrs
        {
            get;
            private set;
        }

        public CFFFontReader Reader
        {
            get
            {
                return this.reader;
            }
        }

        #endregion


        #region Methods

        public string ReadString(ushort sid)
        {
            return this.StringIndex[sid];
        }

        public void ReadTable(CFFTable table)
        {
            this.Reader.BeginReadingBlock();
            this.Reader.Seek(table.Offset, System.IO.SeekOrigin.Begin);
            table.Read(this.Reader);
            this.Reader.EndReadingBlock();
        }

        private void Initialize()
        {
            this.Header = new Header(this);
            this.ReadTable(this.Header);

            this.Name = new NameIndex(this, this.Header.HeaderSize);
            this.ReadTable(this.Name);

            this.TopIndex = new TopIndex(this, this.Name.SkipOffset);
            this.ReadTable(this.TopIndex);

            this.StringIndex = new StringIndex(this, this.TopIndex.SkipOffset);
            this.ReadTable(this.StringIndex);

            Top top = this.TopIndex[0];
            this.GlobalSubrs = new SubrsIndex(this, top.CharstringType, this.StringIndex.SkipOffset);
            this.ReadTable(this.GlobalSubrs);

            this.fontSource = new CFFFontSource(this, top);
        }

        #endregion
    }
}
