using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.Type1.CharStrings;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class CharString : Index
    {
        #region Constructors

        public CharString(Top top, long offset) 
            : base(top.File, offset)
        {
            Guard.ThrowExceptionIfNull(top, "top");

            this.top = top;
        }

        #endregion


        #region Fields

        private GlyphData[] glyphOutlines;

        #endregion


        #region Indexers

        public GlyphData this[ushort index]
        {
            get
            {
                if (this.glyphOutlines[index] == null)
                {
                    this.glyphOutlines[index] = this.ReadGlyphData(this.Reader, this.Offsets[index], this.GetDataLength(index));
                }

                return this.glyphOutlines[index];
            }
        }

        #endregion


        #region Fields

        private Top top;

        #endregion


        #region Methods

        public int GetAdvancedWidth(ushort glyphId, int defaultWidth, int nominalWidth)
        {
            GlyphData data = this[glyphId];
            return data.HasWidth ? data.AdvancedWidth + nominalWidth : defaultWidth;
        }

        public void GetGlyphOutlines(Glyph glyph, double fontSize)
        {
            GlyphData data = this[glyph.GlyphId];
            GlyphOutlinesCollection outlines = data.Oultlines.Clone();
            Matrix m = this.top.FontMatrix;
            m.ScaleAppend(fontSize, -fontSize);
            outlines.Transform(m);
            glyph.Outlines = outlines;
        }

        private GlyphData ReadGlyphData(CFFFontReader reader, uint offset, int length)
        {
            reader.BeginReadingBlock();
            reader.Seek(this.DataOffset + offset, System.IO.SeekOrigin.Begin);
            byte[] data = new byte[length];
            reader.Read(data, data.Length);
            BuildChar buildChar = new BuildChar(this.top);
            buildChar.Execute(data);
            reader.EndReadingBlock();
            return new GlyphData(buildChar.GlyphOutlines, (ushort?)buildChar.Width);
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            base.Read(reader);

            this.glyphOutlines = new GlyphData[this.Count];
        }

        #endregion
    }
}