﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Encoding;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class Dict : CFFTable
    {
        #region StaticMembers

        private static bool IsOperator(byte b)
        {
            return 0 <= b && b <= 21;
        }
        private static bool IsTwoByteOperator(byte b)
        {
            return b == OperatorDescriptor.TwoByteOperatorFirstByte;
        }

        #endregion


        #region Constructors

        public Dict(ICFFFontFile file, long offset, int length)
            : base(file, offset)
        {
            this.length = length;
        }

        #endregion


        #region Fields

        private readonly int length;

        #endregion


        #region Properties

        protected Dictionary<OperatorDescriptor, OperandsCollection> Data
        {
            get;
            private set;
        }

        public long SkipOffset
        {
            get
            {
                return this.Offset + this.length;
            }
        }

        #endregion


        #region Methods

        protected int GetInt(OperatorDescriptor op)
        {
            OperandsCollection oq;
            if (!this.Data.TryGetValue(op, out oq))
            {
                if (op.DefaultValue != null)
                {
                    return (int)op.DefaultValue;
                }
                throw new ArgumentException("Operator not found");
            }
            return oq.GetLastAsInt();
        }

        protected bool TryGetOperands(OperatorDescriptor op, out OperandsCollection operands)
        {
            return this.Data.TryGetValue(op, out operands);
        }

        protected double GetNumber(OperatorDescriptor op)
        {
            OperandsCollection oq;
            if (!this.Data.TryGetValue(op, out oq))
            {
                if (op.DefaultValue != null)
                {
                    return (double)op.DefaultValue;
                }
                throw new ArgumentException("Operator not found");
            }
            return oq.GetLastAsReal();
        }

        protected PostScriptArray GetArray(OperatorDescriptor op)
        {
            OperandsCollection oq;
            if (!this.Data.TryGetValue(op, out oq))
            {
                if (op.DefaultValue != null)
                {
                    return (PostScriptArray)op.DefaultValue;
                }
                throw new ArgumentException("Operator not found");
            }
            PostScriptArray arr = new PostScriptArray();
            while (oq.Count > 0)
            {
                arr.Add(oq.GetFirst());
            }
            return arr;
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            byte[] data = new byte[this.length];
            reader.Read(data, this.length);
            EncodedDataReader r = new EncodedDataReader(data, ByteEncoding.DictByteEncodings);
            this.Data = new Dictionary<OperatorDescriptor, OperandsCollection>();
            OperandsCollection operands = new OperandsCollection();
            while (!r.EndOfFile)
            {
                byte b0 = r.Peek();
                if (IsOperator(b0))
                {
                    OperatorDescriptor o;
                    if (IsTwoByteOperator(b0))
                    {
                        o = new OperatorDescriptor(Helper.CreateByteArray(r.Read(), r.Read()));
                    }
                    else
                    {
                        o = new OperatorDescriptor(r.Read());
                    }
                    this.Data[o] = operands;
                    operands = new OperandsCollection();
                }
                else
                {
                    operands.AddLast(r.ReadOperand());
                }
            }
        }

        #endregion
    }
}
