﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Core.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class Encoding : CFFTable, IEncoding
    {
        #region Constructors

        public Encoding(ICFFFontFile file, Charset charset, long offset)
            : base(file, offset)
        {
            this.charset = charset;
        }

        #endregion


        #region Fields

        private readonly Charset charset;

        private List<ushort> gids;

        #endregion


        #region Properties

        public SupplementalEncoding SupplementalEncoding
        {
            get;
            private set;
        }

        #endregion

        #region Methods

        private void ReadFormat0(CFFFontReader reader)
        {
            byte cnt = reader.ReadCard8();
            this.gids = new List<ushort>(cnt);
            for (int i = 0; i < cnt; i++)
            {
                this.gids.Add(reader.ReadCard8());
            }
        }
        private void ReadFormat1(CFFFontReader reader)
        {
            byte ranges = reader.ReadCard8();
            this.gids = new List<ushort>();
            byte first, left;
            for (int i = 0; i < ranges; i++)
            {
                first = reader.ReadCard8();
                left = reader.ReadCard8();
                this.gids.Add(first);
                for (int j = 0; j < left; j++)
                {
                    this.gids.Add((byte)(first + j + 1));
                }
            }
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            byte format = reader.ReadCard8();
            if (BitsHelper.GetBit(format, 0))
            {
                this.ReadFormat1(reader);
            }
            else
            {
                this.ReadFormat0(reader);
            }
            if (BitsHelper.GetBit(format, 7))
            {
                this.SupplementalEncoding = new SupplementalEncoding();
                this.SupplementalEncoding.Read(reader);
            }
        }

        public string GetGlyphName(ICFFFontFile fontFile, ushort index)
        {
            int glyphIndex = this.gids.IndexOf(index);
            if (glyphIndex < 0)
            {
                return PostScriptConstants.NotDef;
            }

            return this.charset[(ushort)glyphIndex];
        }

        #endregion
    }
}
