﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class Index : CFFTable
    {
        #region Constructors

        public Index(ICFFFontFile file, long offset)
            : base(file, offset)
        {
        }

        #endregion


        #region Properties

        public long SkipOffset
        {
            get
            {
                if (this.Offsets == null)
                {
                    return this.DataOffset;
                }

                return this.DataOffset + Enumerable.Last(this.Offsets);
            }
        }

        public ushort Count
        {
            get;
            private set;
        }


        protected uint[] Offsets
        {
            get;
            private set;
        }

        protected long DataOffset
        {
            get;
            private set;
        }

        #endregion


        #region Methods

        protected int GetDataLength(int index)
        {
            return (int)(this.Offsets[index + 1] - this.Offsets[index]);
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            this.Count = reader.ReadCard16();
            if (this.Count == 0)
            {
                this.DataOffset = reader.Position;
                return;
            }
           
            byte offSize = reader.ReadOffSize();
            ushort count = (ushort)(this.Count + 1);
            this.Offsets = new uint[count];
            
            for (int i = 0; i < count; i++)
            {
                this.Offsets[i] = reader.ReadOffset(offSize);
            }
            
            this.DataOffset = reader.Position - 1;
        }

        #endregion
    }
}
