﻿using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class Private : Dict
    {
        static Private()
        {
            SubrsOperator = new OperatorDescriptor(19);
            DefaultWidthXOperator = new OperatorDescriptor(20, 0);
            NominalWidthXOperator = new OperatorDescriptor(21, 0);
        }

        public Private(Top top, long offset, int length)
            : base(top.File, offset, length)
        {
            this.top = top;
        }

        private readonly Top top;
        private SubrsIndex subrs;
        private int? defaultWidthX;
        private int? nominalWidthX;

        public static OperatorDescriptor SubrsOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor DefaultWidthXOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor NominalWidthXOperator
        {
            get;
            private set;
        }

        public SubrsIndex Subrs
        {
            get
            {
                if (this.subrs == null)
                {
                    this.subrs = new SubrsIndex(this.File, this.top.CharstringType, this.Offset + this.GetInt(SubrsOperator));
                    this.File.ReadTable(this.subrs);
                }
                return this.subrs;
            }
        }

        public int DefaultWidthX
        {
            get
            {
                if (this.defaultWidthX == null)
                {
                    this.defaultWidthX = this.GetInt(DefaultWidthXOperator);
                }
                return this.defaultWidthX.Value;
            }
        }

        public int NominalWidthX
        {
            get
            {
                if (this.nominalWidthX == null)
                {
                    this.nominalWidthX = this.GetInt(NominalWidthXOperator);
                }
                return this.nominalWidthX.Value;
            }
        }
    }
}
