﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;
using Telerik.Windows.Documents.Core.PostScript.Data;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class StringIndex : Index
    {
        #region Constructors

        public StringIndex(ICFFFontFile file, long offset)
            : base(file, offset)
        {
        }

        #endregion


        #region Fields

        private string[] strings;

        #endregion


        #region Indexers

        public string this[ushort sid]
        {
            get
            {
                if (StandardStrings.IsStandardString(sid))
                {
                    return StandardStrings.GetStandardString(sid);
                }

                sid = (ushort)(sid - StandardStrings.StandardStringsCount);

                return this.GetString(sid);
            }
        }

        #endregion


        #region Methods

        private string ReadString(CFFFontReader reader, uint offset, int length)
        {
            reader.BeginReadingBlock();
            long topOffset = this.DataOffset + offset;
            reader.Seek(topOffset, System.IO.SeekOrigin.Begin);
            string str = reader.ReadString(length);
            reader.EndReadingBlock();
            return str;
        }

        private string GetString(int index)
        {
            if (this.strings.Length <= index)
            {
                return PostScriptConstants.NotDef;
            }

            if (this.strings[index] == null)
            {
                this.strings[index] = this.ReadString(this.Reader, this.Offsets[index], this.GetDataLength(index));
            }

            return this.strings[index];
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            base.Read(reader);
            if (this.Count == 0)
            {
                return;
            }
      
            this.strings = new string[this.Count];
        }

        #endregion
    }
}
