﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class SubrsIndex : Index
    {
        #region Constructors

        public SubrsIndex(ICFFFontFile fontFile, int charstringType, long offset)
            : base(fontFile, offset)
        {
            this.charstringType = charstringType;
        }

        #endregion


        #region Fields

        private readonly int charstringType;

        private byte[][] subrs;
        private ushort bias;

        #endregion


        #region Indexers

        public byte[] this[int index]
        {
            get
            {
                return this.GetSubr(index + this.bias);
            }
        }

        #endregion


        #region Methods

        private byte[] ReadSubr(CFFFontReader reader, uint offset, int length)
        {
            reader.BeginReadingBlock();
            long topOffset = this.DataOffset + offset;
            reader.Seek(topOffset, System.IO.SeekOrigin.Begin);
            byte[] res = new byte[length];
            reader.Read(res, length);
            reader.EndReadingBlock();
            return res;
        }

        private byte[] GetSubr(int index)
        {
            if (this.subrs[index] == null)
            {
                this.subrs[index] = this.ReadSubr(this.Reader, this.Offsets[index], this.GetDataLength(index));
            }
            return this.subrs[index];
        }

        #endregion


        #region Overrides

        public override void Read(CFFFontReader reader)
        {
            base.Read(reader);
            if (this.Count == 0)
            {
                return;
            }
            this.subrs = new byte[this.Count][];
            ushort nSubrs = this.Count;

            if (this.charstringType == 1)
            {
                this.bias = 0;
            }
            else if (nSubrs < 1240)
            {
                this.bias = 107;
            }
            else if (nSubrs < 33900)
            {
                this.bias = 1131;
            }
            else
            {
                this.bias = 32768;
            }
        }

        #endregion
    }
}
