﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;
using Telerik.Windows.Documents.Core.PostScript.Data;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables
{
    internal class Top : Dict, IBuildCharHolder
    {
        #region StaticMembers

        public static OperatorDescriptor FamilyNameOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor WeightOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor EncodingOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor CharStringsOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor ItalicAngleOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor UnderlinePositionOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor UnderlineThicknessOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor CharstringTypeOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor CharsetOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor FontMatrixOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor PrivateOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor ROSOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor FDArrayOperator
        {
            get;
            private set;
        }

        public static OperatorDescriptor FDSelectOperator
        {
            get;
            private set;
        }

        #endregion


        #region Constructors

        static Top()
        {
            FontMatrixOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 7), new PostScriptArray(new object[] { 0.001, 0, 0, 0.001, 0, 0 }));
            FamilyNameOperator = new OperatorDescriptor(3);
            WeightOperator = new OperatorDescriptor(4);
            ItalicAngleOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 2), 0);
            UnderlinePositionOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 3), -100);
            UnderlineThicknessOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 3), 50);
            CharstringTypeOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 6), 2);
            CharsetOperator = new OperatorDescriptor(15, 0);
            EncodingOperator = new OperatorDescriptor(16, 0);
            CharStringsOperator = new OperatorDescriptor(17);
            PrivateOperator = new OperatorDescriptor(18);

            // CID Operators
            ROSOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 30));
            FDArrayOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 36));
            FDSelectOperator = new OperatorDescriptor(Helper.CreateByteArray(12, 37));
        }

        public Top(ICFFFontFile file, long offset, int length)
            : base(file, offset, length)
        {
        }

        #endregion


        #region Fields

        private IEncoding encoding;
        private CharString charString;
        private Charset charset;
        private string familyName;
        private Matrix? fontMatrix;
        private Private priv;
        private int? charstringType;
        private bool? usesCIDFontOperators;
        private double? underlinePosition;
        private double? underlineThickness;

        #endregion


        #region Properties

        public double UnderlinePosition
        {
            get
            {
                if (!this.underlinePosition.HasValue)
                {
                    this.underlinePosition = this.GetNumber(UnderlinePositionOperator);
                }

                return this.underlinePosition.Value;
            }
        }

        public double UnderlineThickness
        {
            get
            {
                if (!this.underlineThickness.HasValue)
                {
                    this.underlineThickness = this.GetNumber(UnderlineThicknessOperator);
                }

                return this.underlineThickness.Value;
            }
        }

        public int CharstringType
        {
            get
            {
                if (!this.charstringType.HasValue)
                {
                    this.charstringType = this.GetInt(CharstringTypeOperator);
                }
                return this.charstringType.Value;
            }
        }

        public IEncoding Encoding
        {
            get
            {
                if (this.encoding == null)
                {
                    this.ReadEncoding();
                }
                return this.encoding;
            }
        }

        public Charset Charset
        {
            get
            {
                if (this.charset == null)
                {
                    this.ReadCharset();
                }
                return this.charset;
            }
        }

        public CharString CharString
        {
            get
            {
                if (this.charString == null)
                {
                    this.ReadCharString();
                }
                return this.charString;
            }
        }

        public string FamilyName
        {
            get
            {
                if (this.familyName == null)
                {
                    this.familyName = this.File.ReadString((ushort)this.GetInt(FamilyNameOperator));
                }
                return this.familyName;
            }
        }

        public Matrix FontMatrix
        {
            get
            {
                if (!this.fontMatrix.HasValue)
                {
                    this.fontMatrix = this.GetArray(FontMatrixOperator).ToMatrix();
                }

                return this.fontMatrix.Value;
            }
        }

        public Private Private
        {
            get
            {
                if (this.priv == null)
                {
                    this.ReadPrivate();
                }

                return this.priv;
            }
        }

        public int DefaultWidthX
        {
            get
            {
                if (this.Private != null)
                {
                    return this.Private.DefaultWidthX;
                }

                return 0;
            }
        }

        public int NominalWidthX
        {
            get
            {
                if (this.Private != null)
                {
                    return this.Private.NominalWidthX;
                }

                return 0;
            }
        }

        public bool UsesCIDFontOperators
        {
            get
            {
                if (!this.usesCIDFontOperators.HasValue)
                {
                    this.usesCIDFontOperators = this.Data.ContainsKey(ROSOperator);
                }

                return this.usesCIDFontOperators.Value;
            }
        }

        #endregion


        #region Methods

        public byte[] GetSubr(int index)
        {
            return this.Private.Subrs[index];
        }

        public byte[] GetGlobalSubr(int index)
        {
            return this.File.GlobalSubrs[index];
        }

        public GlyphData GetGlyphData(string name)
        {
            ushort glyphId;
            this.TryGetGlyphIdByCharacterName(name, out glyphId);
            return this.CharString[glyphId];
        }

        public bool TryGetGlyphIdByCharacterName(string name, out ushort glyphId)
        {
            return this.Charset.TryGetGlyphIdByCharacterName(name, out glyphId);
        }

        public bool TryGetGlyphIdByCharacterIdentifier(ushort cid, out ushort glyphId)
        {
            return this.Charset.TryGetGlyphIdByCharacterIdentifier(cid, out glyphId);
        }

        internal string GetGlyphName(ushort cid)
        {
            return this.Encoding.GetGlyphName(this.File, cid);
        }

        public ushort GetAdvancedWidth(ushort glyphId)
        {
            return (ushort)this.CharString.GetAdvancedWidth(glyphId, this.DefaultWidthX, this.NominalWidthX);
        }

        public void GetGlyphOutlines(Glyph glyph, double fontSize)
        {
            this.CharString.GetGlyphOutlines(glyph, fontSize);
        }

        private void ReadEncoding()
        {
            int encodingOffset = this.GetInt(EncodingOperator);
            if (PredefinedEncoding.IsPredefinedEncoding(encodingOffset))
            {
                this.encoding = PredefinedEncoding.GetPredefinedEncoding(encodingOffset);
            }
            else
            {
                Encoding e = new Encoding(this.File, this.Charset, encodingOffset);
                this.File.ReadTable(e);
                this.encoding = e;
            }
        }

        private void ReadPrivate()
        {
            OperandsCollection op;
            if (this.TryGetOperands(PrivateOperator, out op))
            {
                this.priv = new Private(this, op.GetLastAsInt(), op.GetLastAsInt());
                this.File.ReadTable(this.priv);
            }
        }

        private void ReadCharset()
        {
            int charsetOffset = this.GetInt(CharsetOperator);
            if (PredefinedCharset.IsPredefinedCharset(charsetOffset))
            {
                this.charset = new Charset(this.File, PredefinedCharset.GetPredefinedCodes(charsetOffset));
                return;
            }
            this.charset = new Charset(this.File, charsetOffset, this.CharString.Count);
            this.File.ReadTable(this.charset);
        }

        private void ReadCharString()
        {
            this.charString = new CharString(this, this.GetInt(CharStringsOperator));
            this.File.ReadTable(this.charString);
        }

        #endregion
    }
}
