﻿using System;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data
{
    internal class OperatorDescriptor
    {
        #region Constants

        internal const byte TwoByteOperatorFirstByte = 12;

        #endregion
        

        #region Constructors

        public OperatorDescriptor(byte b0)
        {
            this.value = new byte[1];
            this.value[0] = b0;

            this.CalculateHashCode();
        }

        public OperatorDescriptor(byte[] bytes)
        {
            this.value = bytes;

            this.CalculateHashCode();
        }

        public OperatorDescriptor(byte b0, object defaultValue)
            : this(b0)
        {
            this.defaultValue = defaultValue;
        }   
     
        public OperatorDescriptor(byte[] bytes, object defaultValue)
            : this(bytes)
        {
            this.defaultValue = defaultValue;
        }

        #endregion


        #region Fields

        private readonly byte[] value;
        private readonly object defaultValue;

        private int hashCode;

        #endregion


        #region Properties

        public object DefaultValue
        {
            get
            {
                return this.defaultValue;
            }
        }

        #endregion


        #region Methods

        private void CalculateHashCode()
        {
            unchecked
            {
                this.hashCode = 17;
                for (int i = 0; i < this.value.Length; i++)
                {
                    this.hashCode = this.hashCode * 23 + this.value[i];
                }
            }
        }

        #endregion


        #region Overrides

        public override bool Equals(object obj)
        {
            OperatorDescriptor o = obj as OperatorDescriptor;
            if (o == null || this.value.Length != o.value.Length)
            {
                return false;
            }
            for (int i = 0; i < this.value.Length; i++)
            {
                if (this.value[i] != o.value[i])
                {
                    return false;
                }
            }
            return true;
        }

        public override int GetHashCode()
        {
            return this.hashCode;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            foreach (int i in this.value)
            {
                sb.Append(i);
                sb.Append(" ");
            }
            return sb.ToString().Trim();
        }

        #endregion
    }
}
