﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
using Matrix = Telerik.Windows.Documents.Core.Data.Matrix;
#endif
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.Type1.CharStrings.Operators;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Encoding;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Core.Shapes;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;
using Telerik.Windows.Documents.Core.Data;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.CharStrings
{
    internal class BuildChar
    {
        #region StaticMembers
        
        private static readonly Dictionary<OperatorDescriptor, Operator> operators;
        
        private static OperatorDescriptor endChar;
        private static HintOperator vStem;

        internal static Point CalculatePoint(BuildChar interpreter, int dx, int dy)
        {
            interpreter.CurrentPoint = new Point(interpreter.CurrentPoint.X + dx, interpreter.CurrentPoint.Y + dy);
            return new Point(interpreter.CurrentPoint.X, -interpreter.CurrentPoint.Y);
        }

        private static void InitializePathConstructionOperators()
        {
            operators[new OperatorDescriptor(1)] = new HintOperator();
            vStem = new HintOperator();
            operators[new OperatorDescriptor(3)] = vStem;
            operators[new OperatorDescriptor(4)] = new VMoveTo();
            operators[new OperatorDescriptor(5)] = new RLineTo();
            operators[new OperatorDescriptor(6)] = new HLineTo();
            operators[new OperatorDescriptor(7)] = new VLineTo();
            operators[new OperatorDescriptor(8)] = new RRCurveTo();
            endChar = new OperatorDescriptor(14);
            operators[endChar] = new EndChar();
            operators[new OperatorDescriptor(18)] = new HintOperator();
            operators[new OperatorDescriptor(19)] = new HintMaskOperator();
            operators[new OperatorDescriptor(20)] = new HintMaskOperator();
            operators[new OperatorDescriptor(21)] = new RMoveTo();
            operators[new OperatorDescriptor(22)] = new HMoveTo();
            operators[new OperatorDescriptor(23)] = new HintOperator();
            operators[new OperatorDescriptor(24)] = new RCurveLine();
            operators[new OperatorDescriptor(25)] = new RLineCurve();
            operators[new OperatorDescriptor(26)] = new VVCurveTo();
            operators[new OperatorDescriptor(27)] = new HHCurveTo();
            operators[new OperatorDescriptor(30)] = new VHCurveTo();
            operators[new OperatorDescriptor(31)] = new HVCurveTo();
            operators[new OperatorDescriptor(11)] = new Return();
            operators[new OperatorDescriptor(10)] = new CallSubr();
            operators[new OperatorDescriptor(9)] = new ClosePath();
            operators[new OperatorDescriptor(13)] = new Hsbw();
            operators[new OperatorDescriptor(29)] = new CallGSubr();

            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 12))] = new Div();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 35))] = new Flex();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 37))] = new Flex1();

            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 34))] = new HFlex();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 36))] = new HFlex1();

            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 16))] = new CallOtherSubr();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 17))] = new Pop();

            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 6))] = new Seac();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 7))] = new Sbw();
            operators[new OperatorDescriptor(Helper.CreateByteArray(12, 33))] = new SetCurrentPoint();
        }

        private static bool IsOperator(byte b)
        {
            return b != 28 && 0 <= b && b <= 31;
        }

        private static bool IsTwoByteOperator(byte b)
        {
            return b == OperatorDescriptor.TwoByteOperatorFirstByte;
        }

        #endregion


        #region Constructors

        static BuildChar()
        {
            operators = new Dictionary<OperatorDescriptor, Operator>();
            InitializePathConstructionOperators();
        }

        public BuildChar(IBuildCharHolder subrsHodler)
        {
            this.buildCharHolder = subrsHodler;
        }

        #endregion


        #region Fields

        private readonly IBuildCharHolder buildCharHolder;


        private OperandsCollection postScriptStack;
        private OperandsCollection operands;
          
        #endregion


        #region Properties

        internal OperandsCollection Operands
        {
            get
            {
                return this.operands;
            }
        }
        internal OperandsCollection PostScriptStack
        {
            get
            {
                return this.postScriptStack;
            }
        }
        internal PathFigure CurrentPathFigure { get;set; }
        internal GlyphOutlinesCollection GlyphOutlines { get;private set; }
        internal Point CurrentPoint { get;set; }
        internal Point BottomLeft { get;set; }
        internal int? Width { get;set; }
        internal int Hints { get;set; }

        #endregion


        #region Methods

        public void ExecuteSubr(int index)
        {
            byte[] subr = this.buildCharHolder.GetSubr(index);
            this.ExecuteInternal(subr);
        }

        public void ExecuteGlobalSubr(int index)
        {
            byte[] subr = this.buildCharHolder.GetGlobalSubr(index);
            this.ExecuteInternal(subr);
        }

        public void CombineChars(string accentedChar, string baseChar, int dx, int dy)
        {
            GlyphData accentData = this.buildCharHolder.GetGlyphData(accentedChar);
            GlyphOutlinesCollection accentOutlines = accentData.Oultlines.Clone();

            GlyphData baseCharData = this.buildCharHolder.GetGlyphData(baseChar);
            GlyphOutlinesCollection baseCharOutlines = baseCharData.Oultlines.Clone();

            this.GlyphOutlines.AddRange(baseCharOutlines);
            accentOutlines.Transform(new Matrix(1, 0, 0, 1, dx, dy));
            this.GlyphOutlines.AddRange(accentOutlines);
        }

        public void Execute(byte[] data)
        {
            this.postScriptStack = new OperandsCollection();
            this.operands = new OperandsCollection();
            this.GlyphOutlines = new GlyphOutlinesCollection();
            this.CurrentPoint = new Point();
            this.Width = null;
            this.Hints = 0;

            this.ExecuteInternal(data);
        }

        private void ExecuteInternal(byte[] data)
        {
            EncodedDataReader r = new EncodedDataReader(data, ByteEncoding.CharStringByteEncodings);
            while (!r.EndOfFile)
            {
                byte b0 = r.Peek();
                if (IsOperator(b0))
                {
                    OperatorDescriptor o;
                    if (IsTwoByteOperator(b0))
                    {
                        o = new OperatorDescriptor(Helper.CreateByteArray(r.Read(), r.Read()));
                    }
                    else
                    {
                        o = new OperatorDescriptor(r.Read());
                    }
                    this.ExecuteOperator(o, r);
                   
                    if (o.Equals(endChar))
                    {
                        break;
                    }
                }
                else
                {
                    this.Operands.AddLast(r.ReadOperand());
                }
            }
        }

        private int GetMaskSize()
        {
            int size = this.Hints / 8;
            if (this.Hints % 8 != 0)
            {
                size++;
            }
            return size;
        }

        private void ExecuteOperator(OperatorDescriptor descr, EncodedDataReader reader)
        {
            Operator op;
            int count;
            if (operators.TryGetValue(descr, out op))
            {
                if (op is HintMaskOperator)
                {
                    vStem.Execute(this, out count);
                    this.Hints += count;

                    byte[] mask = new byte[this.GetMaskSize()];
                    reader.Read(mask, mask.Length);
                    ((HintMaskOperator)op).Execute(this, mask);
                }
                else if (op is HintOperator)
                {
                    ((HintOperator)op).Execute(this, out count);
                    this.Hints += count;
                }
                else
                {
                    op.Execute(this);
                }
            }
            else
            {
                this.Operands.Clear();
            }
        }
        
        #endregion

        
    }
}
