﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.Encoding
{
    internal abstract class ByteEncoding
    {
        #region StaticMembers
        public static ByteEncodingCollection DictByteEncodings { get;private set; }
        public static ByteEncodingCollection CharStringByteEncodings { get;private set; }

        private static void InitializeDictByteEncodings()
        {
            DictByteEncodings = new ByteEncodingCollection();
            DictByteEncodings.Add(new SingleByteIntegerEncoding());
            DictByteEncodings.Add(new TwoByteIntegerEncodingType1());
            DictByteEncodings.Add(new TwoByteIntegerEncodingType2());
            DictByteEncodings.Add(new ThreeByteIntegerEncoding());
            DictByteEncodings.Add(new FiveByteIntegerEncoding());
            DictByteEncodings.Add(new RealByteEncoding());
        }

        private static void InitializeCharStringByteEncodings()
        {
            CharStringByteEncodings = new ByteEncodingCollection();
            CharStringByteEncodings.Add(new ThreeByteIntegerEncoding());
            CharStringByteEncodings.Add(new SingleByteIntegerEncoding());
            CharStringByteEncodings.Add(new TwoByteIntegerEncodingType1());
            CharStringByteEncodings.Add(new TwoByteIntegerEncodingType2());
            CharStringByteEncodings.Add(new FiveByteFixedEncoding());
        }

        #endregion

        #region Constructors

        static ByteEncoding()
        {
            InitializeDictByteEncodings();
            InitializeCharStringByteEncodings();
        }

        public ByteEncoding(byte start, byte end)
        {
            this.range = new Fonts.Utils.Range(start, end);
        }

        #endregion

        #region Fields

        private readonly Fonts.Utils.Range range;

        #endregion

        #region Methods

        public abstract object Read(EncodedDataReader reader);

        public bool IsInRange(byte b0)
        {
            return this.range.IsInRange(b0);
        }
        #endregion
    }
}
