﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.Encoding
{
    internal class RealByteEncoding : ByteEncoding
    {
        #region StaticResources
        private static Dictionary<Nibble, string> nibbleMapping;

        private static Nibble endOfNumber;

        #endregion

        #region Constructors

        static RealByteEncoding()
        {
            nibbleMapping = new Dictionary<Nibble, string>();
            nibbleMapping[new Nibble(0x0)] = "0";
            nibbleMapping[new Nibble(0x1)] = "1";
            nibbleMapping[new Nibble(0x2)] = "2";
            nibbleMapping[new Nibble(0x3)] = "3";
            nibbleMapping[new Nibble(0x4)] = "4";
            nibbleMapping[new Nibble(0x5)] = "5";
            nibbleMapping[new Nibble(0x6)] = "6";
            nibbleMapping[new Nibble(0x7)] = "7";
            nibbleMapping[new Nibble(0x8)] = "8";
            nibbleMapping[new Nibble(0x9)] = "9";
            nibbleMapping[new Nibble(0xa)] = ".";
            nibbleMapping[new Nibble(0xb)] = "E";
            nibbleMapping[new Nibble(0xc)] = "E-";
            nibbleMapping[new Nibble(0xd)] = "";
            nibbleMapping[new Nibble(0xe)] = "-";
             
            endOfNumber = new Nibble(0xf);
        }

        public RealByteEncoding() : base(30, 30)
        {
        }

        #endregion

        #region Methods

        public override object Read(EncodedDataReader reader)
        {
            bool finish = false;
            reader.Read();
            StringBuilder num = new StringBuilder();
            do
            {
                byte b = reader.Read();
                Nibble[] nibbles = Nibble.GetNibbles(b);
                foreach (Nibble n in nibbles)
                {
                    if (n == endOfNumber)
                    {
                        finish = true;
                        break;
                    }
                    num.Append(nibbleMapping[n]);
                }
            }
            while (!finish);
            return double.Parse(num.ToString(), System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture.NumberFormat);
        }
        #endregion
    }
}
