﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format.Pfb
{
    internal class PfbHeader
    {
        #region Fields

        private readonly long offset;
        private readonly int headerLength;

        #endregion

        
        #region Properties

        public long Offset
        {
            get
            {
                return this.offset;
            }
        }

        public int HeaderLength
        {
            get
            {
                return this.headerLength;
            }
        }

        public uint NextHeaderOffset
        {
            get;
            private set;
        }

        #endregion


        #region Constructors

        public PfbHeader(long offset, int headerLength = 6)
        {
            Guard.ThrowExceptionIfLessThan(0, offset, "offset");

            this.offset = offset;
            this.headerLength = headerLength;
        }

        #endregion


        #region Methods

        public void Read(PfbFontReader reader)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");

            int b = reader.Read();
            int num = reader.Read();

            this.NextHeaderOffset = reader.ReadUInt();
        }

        public bool IsPositionInside(long position)
        {
            return this.Offset <= position && position < this.Offset + this.HeaderLength;
        }

        #endregion
    }
}
