﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format.Pfb;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format
{
    internal class PfbFontReader
    {
        #region Constants

        private const int HeadersCount = 4;
        private const int LastHeader = HeadersCount - 1;

        #endregion


        #region Fields

        private byte[] data;
        private PfbHeader[] headers;
        private long position;

        #endregion


        #region Constructors

        public PfbFontReader()
        {
        }

        #endregion


        #region Methods

        public void ReadHeaders()
        {
            long offset = 0;
            this.headers = new PfbHeader[HeadersCount];

            for (int i = 0; i < HeadersCount - 1; i++)
            {
                this.position = offset;

                PfbHeader header = new PfbHeader(offset);
                header.Read(this);
                
                this.headers[i] = header;
                offset += header.HeaderLength + header.NextHeaderOffset;
            }

            this.headers[LastHeader] = new PfbHeader(this.data.Length - 2, 2);
        }

        public uint ReadUInt()
        {
            byte[] i = new byte[4];
            this.ReadLE(i, i.Length);
            return BitConverter.ToUInt32(i, 0);
        }

        public byte Read()
        {
            return this.data[this.position++];
        }

        public byte[] ReadData(byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");
            this.data = data;

            this.ReadHeaders();
            List<byte> result = new List<byte>(data.Length);

            for (int i = 0; i < this.data.Length; i++)
            {
                if (this.IsPositionInHeader(i))
                {
                    continue;
                }

                result.Add(this.data[i]);
            }

            return result.ToArray();
        }

        private bool ReadLE(byte[] buffer, int count)
        {
            for (int i = 0; i < count; i++)
            {
                try
                {
                    buffer[i] = this.Read();
                }
                catch (ArgumentOutOfRangeException)
                {
                    return false;
                }
            }
            return true;
        }

        private bool IsPositionInHeader(long position)
        {
            return this.headers.Any(h => h.IsPositionInside(position));
        }

        #endregion
    }
}
