﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Utilities;

namespace Telerik.Windows.Documents.Core.Fonts.Utils
{
    internal class GlyphDescription
    {
        #region StaticMembers

        internal const int ARG_1_AND_2_ARE_WORDS = 0;
        internal const int ARGS_ARE_XY_VALUES = 1;
        internal const int ROUND_XY_TO_GRID = 2;
        internal const int WE_HAVE_A_SCALE = 3;
        internal const int MORE_COMPONENTS = 5;
        internal const int WE_HAVE_AN_X_AND_Y_SCALE = 6;
        internal const int WE_HAVE_A_TWO_BY_TWO = 7;
        internal const int WE_HAVE_INSTRUCTIONS = 8;
        internal const int USE_MY_METRICS = 9;
        internal const int OVERLAP_COMPOUND = 10;

        #endregion


        #region Fields
        #endregion


        #region Properties

        public ushort Flags
        {
            get;
            private set;
        }

        public ushort GlyphIndex
        {
            get;
            private set;
        }

        public Matrix Transform
        {
            get;
            private set;
        }


        #endregion


        #region Methods

        internal bool CheckFlag(byte bit)
        {
            return BitsHelper.GetBit(this.Flags, bit);
        }

        public void Read(OpenTypeFontReader reader)
        {
            this.Flags = reader.ReadUShort();
            this.GlyphIndex = reader.ReadUShort();
            int offsetX = 0, offsetY = 0;


            if (this.CheckFlag(ARG_1_AND_2_ARE_WORDS))
            {
                if (this.CheckFlag(ARGS_ARE_XY_VALUES))
                {
                    offsetX = reader.ReadShort();
                    offsetY = reader.ReadShort();
                }
                else
                {
                    ushort argument1 = reader.ReadUShort();
                    ushort argument2 = reader.ReadUShort();
                }
            }
            else
            {
                if (this.CheckFlag(ARGS_ARE_XY_VALUES))
                {
                    offsetX = reader.ReadChar();
                    offsetY = reader.ReadChar();
                }
                else
                {
                    sbyte argument1 = reader.ReadChar();
                    sbyte argument2 = reader.ReadChar();
                }
            }

            float a = 1, b = 0,c = 0, d = 1;

            if (this.CheckFlag(WE_HAVE_A_SCALE))
            {
                a = d = reader.Read2Dot14();

            }
            else if (this.CheckFlag(WE_HAVE_AN_X_AND_Y_SCALE))
            {
                a = reader.Read2Dot14();
                d = reader.Read2Dot14();
            }
            else if (this.CheckFlag(WE_HAVE_A_TWO_BY_TWO))
            {
                a = reader.Read2Dot14();
                b = reader.Read2Dot14();
                c = reader.Read2Dot14();
                d = reader.Read2Dot14();
            }
            
            this.Transform = new Matrix(a, b, c, d, offsetX, offsetY);
        }

        #endregion
    }
}
