﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Core.Fonts.Utils
{
    internal class GlyphsSequence : IEnumerable<GlyphInfo>
    {
        #region Constructors

        public GlyphsSequence()
        {
            this.store = new List<GlyphInfo>();
        }

        public GlyphsSequence(int capacity)
        {
            this.store = new List<GlyphInfo>(capacity);
        }

        #endregion


        #region Fields

        private readonly List<GlyphInfo> store;

        #endregion


        #region Properties

        public GlyphInfo this[int index]
        {
            get { return this.store[index]; }
        }

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        #endregion


        #region Methods

        public void Add(GlyphInfo glyphInfo)
        {
            this.store.Add(glyphInfo);
        }

        public void Add(ushort glyphID, GlyphForm form)
        {
            this.store.Add(new GlyphInfo(glyphID, form));
        }

        public void Add(ushort glyphId)
        {
            this.store.Add(new GlyphInfo(glyphId));
        }

        public void AddRange(IEnumerable<ushort> glyphIDs)
        {
            foreach (ushort gid in glyphIDs)
            {
                this.Add(gid);
            }
        }

        public void AddRange(IEnumerable<GlyphInfo> glyphIDs)
        {
            this.store.AddRange(glyphIDs);
        }

        public GlyphForm GetGlyphForm(int index)
        {
            return this.store[index].Form;
        }

        public IEnumerator<GlyphInfo> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        #endregion


        #region Overrides

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(this.store[0]);
            for (int i = 1; i < this.store.Count; i++)
            {
                sb.Append(" ");
                sb.Append(this.store[i]);
            }
            return sb.ToString();
        }


        #endregion
    }
}
