﻿using System;
using System.ComponentModel;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders
{
    /// <summary>
    /// Represents settings that <see cref="Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.PdfFormatProvider "/> uses to import PDF.
    /// </summary>
    [TypeConverter(typeof(FormatProviderSettingsTypeConverter))]
    public class FormatProviderSettings
    {
        /// <summary>
        /// Gets the default format provider settings.
        /// </summary>
        /// <value>The default.</value>
        public static FormatProviderSettings ReadAllAtOnce
        {
            get
            {
                return new FormatProviderSettings(ReadingMode.AllAtOnce);
            }
        }

        /// <summary>
        /// Gets the read on demand format provider settings.
        /// </summary>
        /// <value>The read on demand.</value>
        public static FormatProviderSettings ReadOnDemand
        {
            get
            {
                return new FormatProviderSettings(ReadingMode.OnDemand);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FormatProviderSettings" /> class.
        /// </summary>
        /// <param name="mode">The rendering mode.</param>
        public FormatProviderSettings(ReadingMode mode)
        {
            this.ReadingMode = mode;
        }

        /// <summary>
        /// Gets or sets the reading mode.
        /// </summary>
        /// <value>The reading mode.</value>
        public ReadingMode ReadingMode { get; set; }
    }
}
