﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Actions
{
    [PdfClass]
    internal class ActionOld : PdfObjectOld
    {
        private const string Uri = "URI";
        private const string GoTo = "GoTo";

        private readonly InstantLoadProperty<PdfNameOld> type;

        public ActionOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.type = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "S", IsRequired = true });
        }

        public PdfNameOld Type
        {
            get
            {
                return this.type.GetValue();
            }
            set
            {
                this.type.SetValue(value);
            }
        }

        public static ActionOld CreateAction(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dict, "dict");

            PdfNameOld type = dict.GetElement<PdfNameOld>(PdfNames.S);
            if (type != null)
            {
                ActionOld res;
                switch (type.ToString())
                {
                    case Uri:
                        res = new UriActionOld(contentManager);
                        res.Load(dict);

                        return res;

                    case GoTo:
                        res = new GoToActionOld(contentManager);
                        res.Load(dict);

                        return res;
                }
            }

            return null;
        }
    }
}
