﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Annotations;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Annot
{
    [PdfClass]
    internal abstract class AnnotationOld : PdfObjectOld
    {
        private readonly LoadOnDemandProperty<PdfArrayOld> rect;
        private readonly InstantLoadProperty<PdfIntOld> flags;
        private readonly LoadOnDemandProperty<AppearancesOld> appearances;
        private readonly InstantLoadProperty<PdfNameOld> appearanceState;

        public AnnotationOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.rect = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Rect", IsRequired = true });
            this.flags = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "F" }, new PdfIntOld(contentManager, 0));
            this.appearances = this.CreateLoadOnDemandProperty<AppearancesOld>(new PdfPropertyDescriptor { Name = "AP" }, Converters.PdfDictionaryToPdfObjectConverter);
            this.appearanceState = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "AS" });
        }

        public PdfArrayOld Rect
        {
            get
            {
                return this.rect.GetValue();
            }
            set
            {
                this.rect.SetValue(value);
            }
        }

        public PdfIntOld Flags
        {
            get
            {
                return this.flags.GetValue();
            }
            set
            {
                this.flags.SetValue(value);
            }
        }

        public AppearancesOld Appearances
        {
            get
            {
                return this.appearances.GetValue();
            }
            set
            {
                this.appearances.SetValue(value);
            }
        }

        public PdfNameOld AppearanceState 
        {
            get
            {
                return this.appearanceState.GetValue();
            }
            set
            {
                this.appearanceState.SetValue(value);
            }
        }

        public abstract AnnotationType Type
        {
            get;
        }

        public bool TryGetAppearanceContent(out AnnotationAppearancesOld appearances)
        {
            if (this.Appearances != null && this.Appearances.NormalAppearance != null)
            {
                if(this.AppearanceState == null)
                {
                    Container normalContent = this.ImportSingleStateAppearanceContent(this.Appearances.NormalAppearance);
                    Container mouseDownContent = this.ImportSingleStateAppearanceContent(this.Appearances.DownAppearance);
                    Container mouseOverContent = this.ImportSingleStateAppearanceContent(this.Appearances.RolloverAppearance);

                    appearances = new SingleStateAppearancesOld(normalContent, mouseDownContent, mouseOverContent);
                }
                else
                {
                    IEnumerable<KeyValuePair<string, SingleStateAppearancesOld>> statesToAppearances = this.ImportMultiStateAppearances();
                    appearances = new MultiStateAppearancesOld(this.AppearanceState.Value, statesToAppearances);
                }

                return true;
            }

            appearances = null;
            return false;
        }

        protected virtual PdfResourceOld GetDefaultAppearanceResource()
        {
            return new PdfResourceOld(this.ContentManager);
        }

        private IEnumerable<KeyValuePair<string, SingleStateAppearancesOld>> ImportMultiStateAppearances()
        {
            foreach(string state in this.Appearances.NormalAppearance.States)
            {
                Container normalContent = this.ImportSingleStateAppearanceContent(this.Appearances.NormalAppearance, state);
                Container mouseDownContent = this.ImportSingleStateAppearanceContent(this.Appearances.DownAppearance, state);
                Container mouseOverContent = this.ImportSingleStateAppearanceContent(this.Appearances.RolloverAppearance, state);

                yield return new KeyValuePair<string, SingleStateAppearancesOld>(state, new SingleStateAppearancesOld(normalContent, mouseDownContent, mouseOverContent));
            }
        }

        private Container ImportAppearanceContent(XForm appearanceObject)
        {
            if (appearanceObject == null)
            {
                return null;
            }

            PdfResourceOld resources = this.GetDefaultAppearanceResource();
            PdfContext context = new PdfContext(this.ContentManager, appearanceObject.BBox.ToRect());
            Container container = context.CreateXForm(resources, appearanceObject, false);
            container.BoundingRect = appearanceObject.BBox.ToRect();
            PdfElementToFixedElementTranslator.ToSLCoordinates(container);

            return container;
        }

        private Container ImportSingleStateAppearanceContent(AppearanceOld appearance)
        {
            if(appearance == null)
            {
                return null;
            }

            return this.ImportAppearanceContent(appearance.SingleStateAppearance);
        }

        private Container ImportSingleStateAppearanceContent(AppearanceOld appearance, string state)
        {
            Container content = null;

            XForm appearanceObject;
            if (appearance != null && appearance.TryGetStateAppearance(state, out appearanceObject))
            {
                content = this.ImportAppearanceContent(appearanceObject);
            }

            return content;
        }
    }
}
