﻿using System;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
    [PdfClass(TypeName = "CMap")]
	internal class CMapStream : PdfStreamOld
    {
        private readonly InstantLoadProperty<CMapStream> useCMap;

        private readonly bool isIdentityH;
        private readonly bool isIdentityV;

        public CMapStream(PdfContentManager contentManager)
            : this(contentManager, false, false)
        {
        }

        public CMapStream(PdfContentManager contentManager, bool isIdentityH = false, bool isIdentityV = false)
            : base(contentManager)
        {
            this.useCMap = this.CreateInstantLoadProperty<CMapStream>(new PdfPropertyDescriptor { Name = "UseCMap" }, Converters.CMapStreamConverter);

            this.isIdentityH = isIdentityH;
            this.isIdentityV = isIdentityV;
        }

        public bool IsIdentityV
        {
            get
            {
                return this.isIdentityV;
            }
        }

        public bool IsIdentityH
        {
            get
            {
                return this.isIdentityH;
            }
        }

        public CMapStream UseCMap
        {
            get
            {
                return this.useCMap.GetValue();
            }
            set
            {
                this.useCMap.SetValue(value);
            }
        }

        public static CMapStream CreateCMap(PdfContentManager contentManager, PdfNameOld name)
        {
            switch (name.Value)
            {
                case PdfNames.IdentityH:
                    return new CMapStream(contentManager, isIdentityH: true);
                case PdfNames.IdentityV:
                    return new CMapStream(contentManager, isIdentityV: true);
                default:
                    throw new NotSupportedPredefinedCMapException(name.Value); 
            }
        }
    }
}
