﻿using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Functions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces
{
    internal class DeviceNOld : CachedColorSpaceOld, IMulticomponentColorSpace, IColorSpace
    {
        private readonly DeviceNAttributes attributes;
        private PdfArrayOld names;

        public DeviceNOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.attributes = new DeviceNAttributes(contentManager);
        }

        public ColorSpaceOld AlternateColorSpace
        {
            get;
            private set;
        }

        public FunctionOld TintTransform
        {
            get;
            private set;
        }

        public override Brush DefaultBrush
        {
            get
            {
                return this.AlternateColorSpace.DefaultBrush;  
            }
        }

        public override ColorSpace Type
        {
            get { return this.AlternateColorSpace.Type; }
        }

        public override int ComponentCount
        {
            get
            {
                return this.names.Count;
            }
        }

        public void Init(PdfArrayOld array)
        {
            this.names = array.GetElement<PdfArrayOld>(1); 
            this.AlternateColorSpace = array.GetElement<ColorSpaceOld>(2, Converters.ColorSpaceConverter);
            this.TintTransform = array.GetElement<FunctionOld>(3, Converters.FunctionConverter);

            if (array.Count > 4)
            {
                PdfDictionaryOld pdfDictionary = array.GetElement<PdfDictionaryOld>(4);
                this.attributes.Load(pdfDictionary);
            }

            this.IsLoaded = true;
        } 
         
        protected override Color GetColorOverride(object[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            return this.AlternateColorSpace.GetColor(Utils.Extensions.ToParams(this.TintTransform.Execute(ToParams(pars))));
        }

        public override Color GetColor(byte[] bytes, int index)
        {
            Guard.ThrowExceptionIfNull(bytes, "bytes");

            object[] colorants = new object[this.ComponentCount];
            for (int i = 0; i < this.ComponentCount; i++)
            {
                colorants[i] = bytes[index + i];
            }

            return this.GetColor(colorants);
        }

        public override Brush GetBrush(PdfResourceOld resources, object[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            return this.AlternateColorSpace.GetBrush(resources, Utils.Extensions.ToParams(this.TintTransform.Execute(ToParams(pars))));
        }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return DeviceN.GetDefaultDecodeArray(this);
        }

        protected override PixelContainer GetPixelsOverride(IImageDescriptor image, bool applyMask)
        {
            PixelContainer result = DeviceN.GetDeviceNPixels(image, this, applyMask);

            if (this.AlternateColorSpace != null)
            {
                this.AlternateColorSpace.Clear();
            }

            return result;
        }

        private static double[] ToParams(object[] pars)
        {
            double[] res = new double[pars.Length];

            for (int i = 0; i < pars.Length; i++)
            {
                Helper.UnboxDouble(pars[i], out res[i]);
            }

            return res;
        }

        Color IMulticomponentColorSpace.GetColor(double[] components)
        {
            object[] parameters = Helper.BoxDoubleParameters(components);

            return this.GetColor(parameters);
        }
    }
}
