﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces
{
    internal abstract class PatternOld : PdfObjectOld
    {
        private readonly InstantLoadProperty<PdfArrayOld> matrix;

        public PatternOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.matrix = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Matrix"), PdfArrayOld.CreateMatrixIdentity(contentManager));
        }

        public PdfArrayOld Matrix
        {
            get
            {
                return this.matrix.GetValue();
            }
            set
            {
                this.matrix.SetValue(value);
            }
        }

        public ColorSpaceOld UnderlyingColorSpace { get; set; }

        public static PatternOld CreatePattern(PdfContentManager contentManager, PdfDictionaryOld pdfDictionary)
        {
            PatternOld res = CreatePatternInternal(contentManager, pdfDictionary);
            res.Load(pdfDictionary);
           
            return res;
        }

        public static PatternOld CreatePattern(PdfContentManager contentManager, PdfDataStream pdfDataStream)
        {
            PatternOld res = CreatePatternInternal(contentManager, pdfDataStream.Dictionary);
            res.Load(pdfDataStream);
            
            return res;
        }

        public virtual void Load(PdfDataStream stream)
        {
            this.Load(stream.Dictionary);
        }

        public Brush CreateBrush(object[] pars)
        {
            return this.CreateBrushOverride(this.Matrix.ToMatrix(), pars);
        }

        protected abstract PatternBrush CreateBrushOverride(Matrix matrix, object[] pars);
       
        private static PatternOld CreatePatternInternal(PdfContentManager contentManager, PdfDictionaryOld pdfDictionary)
        {
            int patternType;
            pdfDictionary.TryGetInt(PdfNames.PatternType, out patternType);

            PatternOld res;

            switch (patternType)
            {
                case 1:
                    res = new TilingPatternOld(contentManager);
                    break;
                case 2:
                    res = new ShadingPatternOld(contentManager);
                    break;
                default:
                    throw new InvalidOperationException("Pattern type is not supported.");
            }

            return res;
        }
    }
}
