﻿using System;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Functions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces
{
    internal class SeparationOld : CachedColorSpaceOld, IMulticomponentColorSpace, IColorSpace
    {
        public SeparationOld(PdfContentManager contentManager)
            : base(contentManager)
        {
        }

        public ColorSpaceOld AlternateColorSpace
        {
            get;
            private set;
        }

        public FunctionOld TintTransform
        {
            get;
            private set;
        }

        public override Brush DefaultBrush
        {
            get
            {
                return new SolidColorBrush(Color.Transparent);
            }
        }

        public override ColorSpace Type
        {
            get
            {
                return this.AlternateColorSpace.Type;
            }
        }

        public override int ComponentCount
        {
            get
            {
                return 1;
            }
        }

        public void Init(PdfArrayOld array)
        {
            this.AlternateColorSpace = array.GetElement<ColorSpaceOld>(2, Converters.ColorSpaceConverter);
            this.TintTransform = array.GetElement<FunctionOld>(3, Converters.FunctionConverter);
            
            this.IsLoaded = true;
        }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return Separation.GetDefaultDecodeArray();
        }

        protected override Color GetColorOverride(object[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            double[] functionParameters = ToParams(pars);
            double[] functionResult = this.TintTransform.Execute(functionParameters);
            object[] parameters = Utils.Extensions.ToParams(functionResult);
            Color color = this.AlternateColorSpace.GetColor(parameters);

            return color;
        }

        public override Color GetColor(byte[] bytes, int offset)
        {
            Guard.ThrowExceptionIfNull(bytes, "bytes");
             
            return this.GetColor(new object[] { bytes[offset] });
        }

        protected override PixelContainer GetPixelsOverride(IImageDescriptor image, bool applyMask)
        {
            PixelContainer result = Separation.GetSeparationPixels(image, this, applyMask);

            if (this.AlternateColorSpace != null)
            {
                this.AlternateColorSpace.Clear();
            }

            return result;
        }

        public override Brush GetBrush(PdfResourceOld resources, object[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            double[] unboxedParameters = ToParams(pars);
            double[] tintTransformedParameters = this.TintTransform.Execute(unboxedParameters);
            object[] boxedParameters = Utils.Extensions.ToParams(tintTransformedParameters);
            Brush result = this.AlternateColorSpace.GetBrush(resources, boxedParameters);

            return result;
        }

        private static double[] ToParams(object[] pars)
        {
            double[] res = new double[pars.Length];

            for (int i = 0; i < pars.Length; i++)
            {
                Helper.UnboxDouble(pars[i], out res[i]);
            }

            return res;
        }

        Color IMulticomponentColorSpace.GetColor(double[] components)
        {
            object[] parameters = Helper.BoxDoubleParameters(components);

            return this.GetColor(parameters);
        }
    }
}
