﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Exceptions;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces
{
    internal abstract class ShadingOld : PdfObjectOld
    {
        public const int AxialShadingType = 2;
        public const int RadialShadingType = 3;

        private readonly InstantLoadProperty<ColorSpaceOld> colorSpace;
        private readonly InstantLoadProperty<PdfArrayOld> background;
        private readonly InstantLoadProperty<PdfArrayOld> boundingBox;

        public ShadingOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.colorSpace = this.CreateInstantLoadProperty<ColorSpaceOld>(new PdfPropertyDescriptor("ColorSpace", true), Converters.ColorSpaceConverter);
            this.background = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Background"));
            this.boundingBox = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("BoundingBox"));
        }
        
        public ColorSpaceOld ColorSpace
        {
            get
            {
                return this.colorSpace.GetValue();
            }
            set
            {
                this.colorSpace.SetValue(value);
            }
        }

        public PdfArrayOld Background
        {
            get
            {
                return this.background.GetValue();
            }
            set
            {
                this.background.SetValue(value);
            }
        }

        public PdfArrayOld BoundingBox
        {
            get
            {
                return this.boundingBox.GetValue();
            }
            set
            {
                this.boundingBox.SetValue(value);
            }
        }

        public static ShadingOld CreateShading(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            ShadingOld shading = CreateShadingInternal(contentManager, dict);
            shading.Load(dict);

            return shading;
        }

        public static ShadingOld CreateShading(PdfContentManager contentManager, PdfDataStream pdfStream)
        {
            ShadingOld shading = CreateShadingInternal(contentManager, pdfStream.Dictionary);
            shading.Load(pdfStream);

            return shading;
        }

        public static ShadingOld CreateShadingInternal(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            int shadingType;
            dict.TryGetInt(PdfNames.ShadingType, out shadingType);

            ShadingOld shading;
            switch (shadingType)
            {
                case AxialShadingType:
                    shading = new AxialShadingOld(contentManager);
                    break;
                case RadialShadingType:
                    shading = new RadialShadingOld(contentManager);
                    break;
                default: 
                    throw new NotSupportedShadingTypeException(shadingType);
            }

            return shading;
        }

        public virtual void Load(PdfDataStream stream)
        {
            this.Load(stream.Dictionary);
        }

        public abstract GradientBrush CreateBrush(Matrix transform, object[] pars);

        protected Color? GetBackgroundColor()
        {
            if (this.Background == null)
            {
                return null;
            }

            return this.ColorSpace.GetColor(this.Background.ToArray());
        }
    }
}
