﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces
{
    [PdfClass(TypeName = "Pattern", SubtypeProperty = PdfNames.PatternType, SubtypeValue = "1")]
    internal class TilingPatternOld : PatternOld
    {
        private readonly InstantLoadProperty<PdfIntOld> paintType;
        private readonly InstantLoadProperty<PdfIntOld> tilingType;
        private readonly InstantLoadProperty<PdfArrayOld> boundingBox;
        private readonly InstantLoadProperty<PdfRealOld> xStep;
        private readonly InstantLoadProperty<PdfRealOld> yStep;
        private readonly InstantLoadProperty<PdfResourceOld> resources;

        public TilingPatternOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.paintType = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor("PaintType", true), Converters.PdfIntConverter);
            this.tilingType = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor("TilingType", true), Converters.PdfIntConverter);
            this.boundingBox = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("BBox", true));
            this.xStep = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor("XStep", true), Converters.PdfRealConverter);
            this.yStep = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor("YStep", true), Converters.PdfRealConverter);
            this.resources = this.CreateInstantLoadProperty<PdfResourceOld>(new PdfPropertyDescriptor("Resources", true), Converters.PdfDictionaryToPdfObjectConverter);
        }

        public PdfIntOld PaintType
        {
            get
            {
                return this.paintType.GetValue();
            }
            set
            {
                this.paintType.SetValue(value);
            }
        }

        public PdfIntOld TilingType
        {
            get
            {
                return this.tilingType.GetValue();
            }
            set
            {
                this.tilingType.SetValue(value);
            }
        }

        public PdfArrayOld BoundingBox
        {
            get
            {
                return this.boundingBox.GetValue();
            }
            set
            {
                this.boundingBox.SetValue(value);
            }
        }

        public PdfRealOld XStep
        {
            get
            {
                return this.xStep.GetValue();
            }
            set
            {
                this.xStep.SetValue(value);
            }
        }

        public PdfRealOld YStep
        {
            get
            {
                return this.yStep.GetValue();
            }
            set
            {
                this.yStep.SetValue(value);
            }
        }

        public PdfResourceOld Resources
        {
            get
            {
                return this.resources.GetValue();
            }
            set
            {
                this.resources.SetValue(value);
            }
        }

        public byte[] Data
        {
            get;
            private set;
        }

        public Rect Clip
        {
            get
            {
                return this.BoundingBox.ToRect();
            }
        }
            
        public override void Load(PdfDataStream stream)
        {
            this.Load(stream.Dictionary);
        }

        protected override PatternBrush CreateBrushOverride(Matrix transform, object[] pars)
        {
            ContentStreamParser parser;
            byte[] data = this.ContentManager.ReadData(this.Reference);
            switch (this.PaintType.Value)
            {
                case 1:
                    parser = new ContentStreamParser(this.ContentManager, data, this.Clip, this.Resources, this.Reference);
                    break;

                case 2:
                    parser = new ContentStreamParser(this.ContentManager, data, this.Clip, this.Resources, this.UnderlyingColorSpace.GetBrush(null, pars), this.Reference);
                    break;

                default:
                    throw new NotSupportedPaintTypeException(this.PaintType.Value);
            }

            Container tile = new Container();
            tile.Content.AddRange(parser.ParseContent());
            PdfElementToFixedElementTranslator.ToSLCoordinates(tile);

            return new TileBrush(tile, this.BoundingBox.ToRect(), transform);
        }
    }
}
