﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
    internal class ContentStreamOld : PdfObjectOld
    {
        public ContentStreamOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.Data = null;
        }

        public byte[] Data { get; private set; }

        public PdfArrayOld Array
        {
            get;
            set;
        }

        public void AppendData(PdfDataStream stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            byte[] dataToAppend = stream.ReadData(this.ContentManager);
            bool concatWithWhiteSpace = true;
            this.Data = PdfConcatenatableDataStream.Concat(this.Data, dataToAppend, concatWithWhiteSpace);

            this.IsLoaded = true;
        }

        public void Load(PdfArrayOld array)
        {
            Guard.ThrowExceptionIfNull(array, "array");

            IndirectObjectOld obj;
            foreach (object element in array)
            {
                IndirectReferenceOld indirectReference = element as IndirectReferenceOld;
                if (indirectReference != null)
                {
                    if (this.ContentManager.TryGetIndirectObject(indirectReference, out obj))
                    {
                        this.Load(obj);
                    }
                }
            }
        }

        public override void Load(IndirectObjectOld indirectObject)
        {
            Guard.ThrowExceptionIfNull(indirectObject, "indirectObject");

            PdfDataStream stream = indirectObject.Value as PdfDataStream;
            if (stream != null)
            {
                this.AppendData(stream);
            }
            else
            {
                PdfArrayOld array = indirectObject.Value as PdfArrayOld;
                if (array != null)
                {
                    this.Load(array);
                }
            }

            base.Load(indirectObject);
        }
    }
}
