﻿using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Encryption;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
	internal class CrossReferenceStreamOld : PdfStreamOld
    {
        private readonly LoadOnDemandProperty<PdfArrayOld> index;
        private readonly LoadOnDemandProperty<PdfArrayOld> w;
        private readonly InstantLoadProperty<PdfIntOld> size;
        private readonly InstantLoadProperty<PdfIntOld> prev;
        private readonly LoadOnDemandProperty<DocumentCatalogOld> root;
        private readonly LoadOnDemandProperty<PdfArrayOld> id;
        private readonly LoadOnDemandProperty<EncryptOld> encrypt;

        public CrossReferenceStreamOld(PdfContentManager contentManager)
            : base(contentManager)
		{
            this.index = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Index" });
            this.w = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "W" });
            this.size = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Size" });
            this.prev = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Prev" });
            this.root = this.CreateLoadOnDemandProperty<DocumentCatalogOld>(new PdfPropertyDescriptor { Name = "Root" });
            this.id = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "ID" });
            this.encrypt = this.CreateLoadOnDemandProperty<EncryptOld>(new PdfPropertyDescriptor { Name = "Encrypt" }, Converters.EncryptConverter);
        }
        
        public PdfArrayOld Index
        {
            get
            {
                return this.index.GetValue();
            }
            set
            {
                this.index.SetValue(value);
            }
        }

        public PdfArrayOld W
        {
            get
            {
                return this.w.GetValue();
            }
            set
            {
                this.w.SetValue(value);
            }
        }

        public PdfIntOld Size
        {
            get
            {
                return this.size.GetValue();
            }
            set
            {
                this.size.SetValue(value);
            }
        }

        public PdfIntOld Prev
        {
            get
            {
                return this.prev.GetValue();
            }
            set
            {
                this.prev.SetValue(value);
            }
        }

        public LoadOnDemandProperty<DocumentCatalogOld> Root
        {
            get
            {
                return this.root;
            }
        }

        public PdfArrayOld ID
        {
            get
            {
                return this.id.GetValue();
            }
            set
            {
                this.id.SetValue(value);
            }
        }

        public EncryptOld Encrypt
        {
            get
            {
                return this.encrypt.GetValue();
            }
            set
            {
                this.encrypt.SetValue(value);
            }
        }

        public void Append(CrossReferenceStreamOld stream, byte[] data)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            int begin, count;

            int w0;
            stream.W.TryGetInt(0, out w0);

            int w1;
            stream.W.TryGetInt(1, out w1);

            int w2;
            stream.W.TryGetInt(2, out w2);

            int len = w0 + w1 + w2;
            byte[] buffer = new byte[len];

            if (stream.Index == null)
            {
                begin = 0;
                count = stream.Size.Value;

                using (MemoryStream str = new MemoryStream(data))
                {
                    for (int i = 0; i < count; i++)
                    {
                        if (str.Read(buffer, 0, len) == len)
                        {
                            this.ContentManager.CrossReferences.AddCrossReferenceEntry(begin + i, stream.GetCrossReferenceEntry(buffer));
                        }
                    }
                }
            }
            else
            {
                using (MemoryStream str = new MemoryStream(data))
                {
                    for (int i = 0; i < stream.Index.Count; i += 2)
                    {
                        stream.Index.TryGetInt(i, out begin);
                        stream.Index.TryGetInt(i + 1, out count);

                        for (int j = 0; j < count; j++)
                        {
                            if (str.Read(buffer, 0, len) == len)
                            {
                                this.ContentManager.CrossReferences.AddCrossReferenceEntry(begin + j, stream.GetCrossReferenceEntry(buffer));
                            }
                        }
                    }
                }
            }
        }

        private CrossReferenceEntryOld GetCrossReferenceEntry(byte[] bytes)
        {
            int offset = 0;
            
            int w0;
            this.W.TryGetInt(0, out w0);

            int w1;
            this.W.TryGetInt(1, out w1);

            int w2;
            this.W.TryGetInt(2, out w2);

            CrossReferenceEntryOld entry = new CrossReferenceEntryOld();
            entry.Type = (CrossReferenceEntryTypeOld)BytesHelperOld.GetInt(bytes, offset, w0);
            offset += w0;
            
            entry.Field2 = BytesHelperOld.GetInt(bytes, offset, w1);
            offset += w1;
            
            entry.Field3 = BytesHelperOld.GetInt(bytes, offset, w2);
            
            return entry;
        }
    }
}
