﻿using System;
using System.Security.Cryptography.X509Certificates;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.DigitalSignatures;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.DigitalSignature
{
    [PdfClass(TypeName = "Sig")]
    internal class SignatureElementOld : PdfObjectOld
    {
        private readonly InstantLoadProperty<PdfNameOld> filter;
        private readonly InstantLoadProperty<PdfNameOld> subFilter;
        private readonly InstantLoadProperty<PdfStringOld> contents;
        private readonly InstantLoadProperty<PdfArrayOld> cert;
        private readonly InstantLoadProperty<PdfArrayOld> byteRange;
        private readonly InstantLoadProperty<PdfArrayOld> signatureReference;
        private readonly InstantLoadProperty<PdfArrayOld> changes;
        private readonly InstantLoadProperty<PdfStringOld> name;
        private readonly InstantLoadProperty<PdfStringOld> location;
        private readonly InstantLoadProperty<PdfStringOld> reason;
        private readonly InstantLoadProperty<PdfStringOld> contactInfo;
        private readonly InstantLoadProperty<PdfStringOld> timeOfSigning;
        private readonly InstantLoadProperty<PdfIntOld> signatureHandlerVersion;
        private readonly InstantLoadProperty<PdfIntOld> signatureDictionaryVersion;

        public SignatureElementOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.filter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "Filter", IsRequired = true });
            this.subFilter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "SubFilter" });
            this.contents = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("Contents"));
            this.cert = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Cert"), Converters.ArrayConverter);
            this.byteRange = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "ByteRange" });

            this.signatureReference = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Reference" });

            this.changes = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Changes" });
            this.name = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("Name"));
            this.timeOfSigning = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("M"));
            this.location = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("Location"));
            this.reason = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("Reason"));
            this.contactInfo = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor("ContactInfo"));
            this.signatureHandlerVersion = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor("R"));
            this.signatureDictionaryVersion = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "V" });
        }

        public PdfNameOld Filter
        {
            get
            {
                return this.filter.GetValue();
            }
            set
            {
                this.filter.SetValue(value);
            }
        }

        public PdfNameOld SubFilter
        {
            get
            {
                return this.subFilter.GetValue();
            }
            set
            {
                this.subFilter.SetValue(value);
            }
        }

        public PdfStringOld Contents
        {
            get
            {
                return this.contents.GetValue();
            }
            set
            {
                this.contents.SetValue(value);
            }
        }

        public PdfArrayOld Cert
        {
            get
            {
                return this.cert.GetValue();
            }
            set
            {
                this.cert.SetValue(value);
            }
        }

        public PdfArrayOld ByteRange
        {
            get
            {
                return this.byteRange.GetValue();
            }
            set
            {
                this.byteRange.SetValue(value);
            }
        }

        public PdfArrayOld SignatureReference
        {
            get
            {
                return this.signatureReference.GetValue();
            }
            set
            {
                this.signatureReference.SetValue(value);
            }
        }

        public PdfArrayOld Changes
        {
            get
            {
                return this.changes.GetValue();
            }
            set
            {
                this.changes.SetValue(value);
            }
        }

        public PdfStringOld Name
        {
            get
            {
                return this.name.GetValue();
            }
            set
            {
                this.name.SetValue(value);
            }
        }

        public PdfStringOld TimeOfSigning
        {
            get
            {
                return this.timeOfSigning.GetValue();
            }
            set
            {
                this.timeOfSigning.SetValue(value);
            }
        }

        public PdfStringOld Location
        {
            get
            {
                return this.location.GetValue();
            }
            set
            {
                this.location.SetValue(value);
            }
        }

        public PdfStringOld Reason
        {
            get
            {
                return this.reason.GetValue();
            }
            set
            {
                this.reason.SetValue(value);
            }
        }

        public PdfStringOld ContactInfo
        {
            get
            {
                return this.contactInfo.GetValue();
            }
            set
            {
                this.contactInfo.SetValue(value);
            }
        }

        public PdfIntOld SignatureHandlerVersion
        {
            get
            {
                return this.signatureHandlerVersion.GetValue();
            }
            set
            {
                this.signatureHandlerVersion.SetValue(value);
            }
        }

        public PdfIntOld SignatureDictionaryVersion
        {
            get
            {
                return this.signatureDictionaryVersion.GetValue();
            }
            set
            {
                this.signatureDictionaryVersion.SetValue(value);
            }
        }

        internal void CopyPropertiesTo(Signature signature)
        {
            Guard.ThrowExceptionIfNull(signature, "signature");

            SignatureDataPdfProperties properties = signature.Properties.PdfProperties;

            this.Filter.CopyToProperty(properties.Filter, (filter) => filter.Value);
            this.SubFilter.CopyToProperty(properties.SubFilter, (subFilter) => subFilter.Value);
            this.Contents.CopyToProperty(properties.Contents, (contents) => contents.Value);
            this.Cert.CopyToProperty(properties.Cert, (cert) => ToCertificateCollection(cert));

            this.ByteRange.CopyToProperty(properties.ByteRange, (byteRange) => byteRange.ToIntArray());

            this.Changes.CopyToProperty(properties.Changes, (changes) => changes.ToIntArray());
            this.Name.CopyToProperty(properties.Name, (name) => name.ToString());

            if (this.TimeOfSigning != null)
            {
                DateTime? dateTime = SignatureDateCodec.Decode(this.TimeOfSigning.ToString());
                if (dateTime != null)
                {
                    properties.TimeOfSigning.Value = dateTime.Value;
                }
            }

            this.Location.CopyToProperty(properties.Location, (location) => location.ToString());
            this.Reason.CopyToProperty(properties.Reason, (reason) => reason.ToString());
            this.ContactInfo.CopyToProperty(properties.ContactInfo, (contactInfo) => contactInfo.ToString());
            this.SignatureHandlerVersion.CopyToProperty(properties.SignatureHandlerVersion, (signatureHandlerVersion) => signatureHandlerVersion.Value);
            this.SignatureDictionaryVersion.CopyToProperty(properties.SignatureDictionaryVersion, (signatureDictionaryVersion) => signatureDictionaryVersion.Value);

            signature.Properties.SourceStream = this.ContentManager.GetSourceStream();
        }

        private X509Certificate2Collection ToCertificateCollection(PdfArrayOld array)
        {
            X509Certificate2Collection certificateCollection = new X509Certificate2Collection();
            for (int i = 0; i < array.Count; i++)
            {
                byte[] rawData = ((PdfStringOld)array[i]).Value;
                X509Certificate2 cert = new X509Certificate2(rawData);
                certificateCollection.Add(cert);
            }

            return certificateCollection;
        }
    }
}
