﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Navigation;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Trees;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
    [PdfClass(TypeName = "Catalog")]
	internal class DocumentCatalogOld : PdfObjectOld
    {
        private readonly LoadOnDemandProperty<PageTreeNodeOld> pages;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> dests;
        private readonly LoadOnDemandProperty<AcroFormOld> acroForm;
        private readonly InstantLoadProperty<Names> names;
        private readonly LoadOnDemandProperty<OutlinesOld> outlines;
        private readonly InstantLoadProperty<PdfNameOld> pageMode;

        public DocumentCatalogOld(PdfContentManager contentManager)
            : base(contentManager)
		{
            this.pages = this.CreateLoadOnDemandProperty<PageTreeNodeOld>(new PdfPropertyDescriptor { Name = "Pages", IsRequired = true, State = PdfPropertyState.MustBeIndirectReference });
            this.names = this.CreateInstantLoadProperty<Names>(new PdfPropertyDescriptor { Name = "Names" });
            this.dests = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "Dests", State = PdfPropertyState.MustBeIndirectReference });
            this.acroForm = this.CreateLoadOnDemandProperty<AcroFormOld>(new PdfPropertyDescriptor { Name = "AcroForm" });
            this.outlines = this.CreateLoadOnDemandProperty<OutlinesOld>(new PdfPropertyDescriptor { Name = "Outlines" });
            this.pageMode = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "PageMode" });
        }

        public PdfDictionaryOld Dests
        {
            get
            {
                return this.dests.GetValue();
            }
            set
            {
                this.dests.SetValue(value);
            }
        }
        
        public Names Names
        {
            get
            {
                return this.names.GetValue();
            }
            set
            {
                this.names.SetValue(value);
            }
        }

		public PageTreeNodeOld Pages
		{
            get
            {
                return this.pages.GetValue();
            }
            set
            {
                this.pages.SetValue(value);
            }
		}

        public AcroFormOld AcroForm
        {
            get
            {
                return this.acroForm.GetValue();
            }
            set
            {
                this.acroForm.SetValue(value);
            }
        }

        public DestinationOld GetDestination(PdfStringOld name)
        {
            if (this.Names != null && this.Names.Dests != null)
            {
                DestinationOld destination = this.Names.Dests.FindElement<DestinationOld>(name.ToString(), Converters.DestinationConverter);
                return destination;
            }
           
            return null;
        }

        public DestinationOld GetDestination(PdfNameOld name)
        {
            if (this.Dests != null)
            {
                return this.Dests.GetElement<DestinationOld>(name.Value, Converters.DestinationConverter);
            }

            return null;
        }

        public OutlinesOld Outlines
        {
            get
            {
                return this.outlines.GetValue();
            }
            set
            {
                this.outlines.SetValue(value);
            }
        }

        public PdfNameOld PageMode
        {
            get
            {
                return this.pageMode.GetValue();
            }
            set
            {
                this.pageMode.SetValue(value);
            }
        }
    }
}
