﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Encryption
{
    internal abstract class EncryptOld : PdfObjectOld
    {
        private const string Standard = "Standard";

        private readonly InstantLoadProperty<PdfNameOld> filter;
        private readonly InstantLoadProperty<PdfNameOld> subFilter;
        private readonly InstantLoadProperty<PdfRealOld> algorithm;
        private readonly InstantLoadProperty<PdfIntOld> length;
        private readonly LoadOnDemandProperty<CryptFiltersOld> cryptFilters;
        private readonly InstantLoadProperty<PdfNameOld> streamsFilter;
        private readonly InstantLoadProperty<PdfNameOld> stringsFilter;

        public EncryptOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.filter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "Filter", IsRequired = true });
            this.subFilter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "SubFilter" });
            this.algorithm = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "V" }, new PdfRealOld(contentManager, 0), Converters.PdfRealConverter);
            this.length = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Length" }, new PdfIntOld(contentManager, 40));
            this.cryptFilters = this.CreateLoadOnDemandProperty<CryptFiltersOld>(new PdfPropertyDescriptor { Name = "CF" });
            this.streamsFilter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "StmF" }, new PdfNameOld(contentManager, PdfNames.Identity));
            this.stringsFilter = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "StrF" }, new PdfNameOld(contentManager, PdfNames.Identity));
        }

        public PdfNameOld Filter
        {
            get
            {
                return this.filter.GetValue();
            }
            set
            {
                this.filter.SetValue(value);
            }
        }

        public PdfNameOld SubFilter
        {
            get
            {
                return this.subFilter.GetValue();
            }
            set
            {
                this.subFilter.SetValue(value);
            }
        }

        public PdfRealOld Algorithm
        {
            get
            {
                return this.algorithm.GetValue();
            }
            set
            {
                this.algorithm.SetValue(value);
            }
        }

        public PdfIntOld Length
        {
            get
            {
                return this.length.GetValue();
            }
            set
            {
                this.length.SetValue(value);
            }
        }

        public CryptFiltersOld CryptFilters
        {
            get
            {
                return this.cryptFilters.GetValue();
            }
            set
            {
                this.cryptFilters.SetValue(value);
            }
        }

        public PdfNameOld StreamsFilter
        {
            get
            {
                return this.streamsFilter.GetValue();
            }
            set
            {
                this.streamsFilter.SetValue(value);
            }
        }

        public PdfNameOld StringsFilter
        {
            get
            {
                return this.stringsFilter.GetValue();
            }
            set
            {
                this.stringsFilter.SetValue(value);
            }
        }

        public static EncryptOld CreateEncrypt(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            switch (dict.GetElement<PdfNameOld>(PdfNames.Filter).Value)
            {
                case Standard:
                    StandardEncryptionOld res = new StandardEncryptionOld(contentManager);
                    res.Load(dict);

                    return res;
            }

            return null;
        }

        public abstract void Initialize(PdfArrayOld id, string password);

        public abstract byte[] DecryptStream(int objectNo, int generationNo, byte[] data);
        
        public abstract byte[] DecryptString(int objectNo, int generationNo, byte[] data);
    }
}
