﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Encryption;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Encryption
{
    internal class StandardEncryptionOld : EncryptOld, IStandardCryptFilterMethodProvider
    {
        private readonly InstantLoadProperty<PdfRealOld> revision;
        private readonly InstantLoadProperty<PdfStringOld> owner;
        private readonly InstantLoadProperty<PdfStringOld> user;
        private readonly InstantLoadProperty<PdfIntOld> permissions;
        private byte[] encryptionKey;

        public StandardEncryptionOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.revision = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "R", IsRequired = true }, Converters.PdfRealConverter);
            this.owner = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "O", IsRequired = true });
            this.user = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "U", IsRequired = true });
            this.permissions = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "P", IsRequired = true });
        }

        public PdfRealOld Revision
        {
            get
            {
                return this.revision.GetValue();
            }
            set
            {
                this.revision.SetValue(value);
            }
        }

        public PdfStringOld Owner
        {
            get
            {
                return this.owner.GetValue();
            }
            set
            {
                this.owner.SetValue(value);
            }
        }

        public PdfStringOld User
        {
            get
            {
                return this.user.GetValue();
            }
            set
            {
                this.user.SetValue(value);
            }
        }

        public PdfIntOld Permissions
        {
            get
            {
                return this.permissions.GetValue();
            }
            set
            {
                this.permissions.SetValue(value);
            }
        }

        public override void Initialize(PdfArrayOld id, string password)
        {
            Guard.ThrowExceptionIfNull(id, "id");

            if (this.encryptionKey == null)
            {
                byte[] docId = id == null ? null : id.GetElement<PdfStringOld>(0).Value;
                this.encryptionKey = StandardEncrypt.CalculateEncryptionKey(password, docId, this.Length.Value, (int)this.Revision.Value, this.Owner.Value, this.Permissions.Value, false);
            }
        }

        public override byte[] DecryptStream(int objectNo, int generationNo, byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            string cryptFilter = this.Algorithm.Value == 4 ? this.StreamsFilter.Value : null;
            StandardEncryptionContext encryptionContext = StandardEncrypt.GetEncryptionContext(this, objectNo, generationNo, data, this.encryptionKey, cryptFilter);
            byte[] res = StandardEncrypt.Encrypt(encryptionContext);

            return res;
        }

        public override byte[] DecryptString(int objectNo, int generationNo, byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            string cryptFilter = this.Algorithm.Value == 4 ? this.StringsFilter.Value : null;
            StandardEncryptionContext encryptionContext = StandardEncrypt.GetEncryptionContext(this, objectNo, generationNo, data, this.encryptionKey, cryptFilter);
            byte[] res = StandardEncrypt.Encrypt(encryptionContext);

            return res;
        }

        string IStandardCryptFilterMethodProvider.GetStandardCryptFilterMethod()
        {
            Guard.ThrowExceptionIfNull(this.CryptFilters, "CryptFilters");
            Guard.ThrowExceptionIfNull(this.CryptFilters.StandardCryptFilter, "CryptFilters.StandardCryptFilter");
            string cfm = this.CryptFilters.StandardCryptFilter.CryptFilterMethod.Value;

            return cfm;
        }
    }
}
