﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using FontSource = Telerik.Windows.Documents.Core.Fonts.FontSource;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    internal abstract class CIDFontOld : PdfObjectOld
    {
        private readonly InstantLoadProperty<PdfNameOld> subtype;
        private readonly InstantLoadProperty<PdfNameOld> baseFont;
        private readonly InstantLoadProperty<PdfIntOld> defaultWidth;
        private readonly InstantLoadProperty<CIDFontDescriptorOld> fontDescriptor;
        private readonly InstantLoadProperty<PdfArrayOld> widths;

        private bool fontFamilyLoaded;

        private string fontFamily;
        private FontStyle fontStyle;
        private FontWeight fontWeight;

        public CIDFontOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.subtype = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "Subtype", IsRequired = true });
            this.baseFont = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "BaseFont", IsRequired = true }, Converters.PdfNameConverter);
            this.fontDescriptor = this.CreateInstantLoadProperty<CIDFontDescriptorOld>(new PdfPropertyDescriptor { Name = "FontDescriptor", IsRequired = true, State = PdfPropertyState.MustBeIndirectReference });
            this.defaultWidth = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "DW" });
            this.widths = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "W" });
        }

        public abstract FontSource FontSource { get; }

        public PdfNameOld Subtype
        {
            get
            {
                return this.subtype.GetValue();
            }
            set
            {
                this.subtype.SetValue(value);
            }
        }

        public PdfNameOld BaseFont
        {
            get
            {
                return this.baseFont.GetValue();
            }
            set
            {
                this.baseFont.SetValue(value);
            }
        }

        public CIDFontDescriptorOld FontDescriptor
        {
            get
            {
                return this.fontDescriptor.GetValue();
            }
            set
            {
                this.fontDescriptor.SetValue(value);
            }
        }

        public PdfIntOld DefaultWidth
        {
            get
            {
                return this.defaultWidth.GetValue();
            }
            set
            {
                this.defaultWidth.SetValue(value);
            }
        }

        public PdfArrayOld Widths
        {
            get
            {
                return this.widths.GetValue();
            }
            set
            {
                this.widths.SetValue(value);
            }
        }

        public static CIDFontOld CreateCIDFont(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            string subtype = dict.GetElement<PdfNameOld>(PdfNames.Subtype).Value;
            CIDFontOld res;

            switch (subtype)
            {
                case PdfNames.CIDFontType0:
                    res = new CIDFontType0Old(contentManager);
                    break;

                case PdfNames.CIDFontType2:
                    res = new CIDFontType2Old(contentManager);
                    break;

                default:
                    throw new NotSupportedException();
            }

            res.Load(dict);

            return res;
        }

        public abstract IEnumerable<GlyphOld> GetGlyphs(IEnumerable<CharCodeOld> charIds);

        public virtual void RenderGlyph(PdfContext context, GlyphOld glyph)
        {
            this.LoadFontFamily();

            glyph.FontFamily = this.fontFamily;
            glyph.FontStyle = this.fontStyle;
            glyph.FontWeight = this.fontWeight;
        }

        private void LoadFontFamily()
        {
            if (!this.fontFamilyLoaded)
            {
                if (this.FontDescriptor == null || !this.FontDescriptor.GetFontFamily(out this.fontFamily, out this.fontWeight, out this.fontStyle))
                {
                    FormatProviders.Pdf.Model.Elements.Fonts.FontsHelper.GetFontFamily(this.BaseFont.Value, out this.fontFamily, out this.fontWeight, out this.fontStyle);
                }

                this.fontFamilyLoaded = true;
            }
        }
    }
}
