﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    [PdfClass(TypeName = "Font", SubtypeProperty = "Subtype", SubtypeValue = "CIDFontType2")]
    internal class CIDFontType2Old : CIDFontOld
    {
        private readonly InstantLoadProperty<CIDToGIDMapOld> cidToGIDMap;

        private OpenTypeFontSource fontSource;

        public CIDFontType2Old(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.cidToGIDMap = this.CreateInstantLoadProperty<CIDToGIDMapOld>(new PdfPropertyDescriptor { Name = "CIDToGIDMap" }, new CIDToGIDMapOld(contentManager), Converters.CidToGidMapConverter);
        }
        
        public override FontSource FontSource
        {
            get
            {
                return this.OpenTypeFontSource;
            }
        }

        public CIDToGIDMapOld CIDToGIDMap
        {
            get
            {
                return this.cidToGIDMap.GetValue();
            }
            set
            {
                this.cidToGIDMap.SetValue(value);
            }
        }

        private OpenTypeFontSource OpenTypeFontSource
        {
            get
            {
                if (this.fontSource == null)
                {
                    if (this.fontSource == null && this.FontDescriptor != null)
                    {
                        this.fontSource = this.FontDescriptor.GetOpenTypeFontSource();
                    }

                    if (this.fontSource == null && this.BaseFont != null)
                    {
                        this.fontSource = this.ContentManager.FontsManager.GetTrueTypeFontSource(this.BaseFont.Value);
                    }
                }

                return this.fontSource;
            }
        }

        public override IEnumerable<GlyphOld> GetGlyphs(IEnumerable<CharCodeOld> charIds)
        {
            if (this.OpenTypeFontSource != null && this.CIDToGIDMap != null)
            {
                return EnumerateGlyphs(charIds, this.CIDToGIDMap);
            }
            
            return null;
        }

        private static IEnumerable<GlyphOld> EnumerateGlyphs(IEnumerable<CharCodeOld> charIds, CIDToGIDMapOld mapping)
        {
            foreach (CharCodeOld cid in charIds)
            {
                GlyphOld g = new GlyphOld();
                g.CharId = cid;
                CidType2Font.InitializeCoreGlyph(g, mapping, cid.IntValue);

                yield return g;
            }
        }
    }
}
