﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    internal class EncodingOld : PdfObjectOld, ISimpleEncoding, IEncoding
    {
        private readonly InstantLoadProperty<PdfNameOld> baseEncoding;
        private readonly LoadOnDemandProperty<PdfArrayOld> differences;
        
        private string namedEncoding;
        private string[] names;
        private bool isInitialized;

        public EncodingOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.isInitialized = false;

            this.baseEncoding = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "BaseEncoding" });
            this.differences = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Differences" });
        }
        
        public PdfNameOld BaseEncoding
        {
            get
            {
                return this.baseEncoding.GetValue();
            }
            set
            {
                this.baseEncoding.SetValue(value);
            }
        }

        public PdfArrayOld Differences
        {
            get
            {
                return this.differences.GetValue();
            }
            set
            {
                this.differences.SetValue(value);
            }
        }
        
        public string BaseEncodingName
        {
            get
            {
                if (this.BaseEncoding != null)
                {
                    return this.BaseEncoding.Value;
                }
                else
                {
                    return this.namedEncoding;
                }
            }
        }

        public bool IsNamedEncoding
        {
            get
            {
                return this.BaseEncodingName != null && this.Differences == null;
            }
        }

        public string GetName(byte b)
        {
            this.Initialize();
            return this.names[b];
        }

        public IEnumerable<Tuple<char, CharCodeOld>> Encode(byte[] bytes)
        {
            if (bytes == null)
            {
                yield break;
            }
            this.Initialize();
            foreach (byte b in bytes)
            {
                yield return new Tuple<char, CharCodeOld>(AdobeGlyphList.GetUnicode(this.names[b]), new CharCodeOld(b));
            }
        }

        public void Load(PdfNameOld name)
        {
            Guard.ThrowExceptionIfNull(name, "name");

            PredefinedSimpleEncoding encoding = PredefinedSimpleEncoding.GetPredefinedEncoding(name.Value);
            this.namedEncoding = encoding != null ? encoding.Name : null;

            this.IsLoaded = true;
        }

        public override void Load(IndirectObjectOld indirectObject)
        {
            Guard.ThrowExceptionIfNull(indirectObject, "indirectObject");

            PdfNameOld name = indirectObject.Value as PdfNameOld;

            if (name != null)
            {
                this.Load(name);
            }

            base.Load(indirectObject);
        }
                
        private void CalcDifferentsNames()
        {
            if (this.Differences == null)
            {
                return;
            }
            int code = 0;
            PdfNameOld name;
            object o;
            for (int i = 0; i < this.Differences.Count; i++)
            {
                o = this.Differences[i];
                if (o is PdfIntOld)
                {
                    code = (byte)((PdfIntOld)o).Value;
                }
                else if (o is PdfNameOld)
                {
                    if (code < 256)
                    {
                        name = (PdfNameOld)o;
                        this.names[code] = name.Value;
                    }
                    code++;
                }
            }
        }

        private void Initialize()
        {
            if (this.isInitialized)
            {
                return;
            }

            this.names = SimpleEncoding.GetNamesWithoutDifferences(this.BaseEncodingName);
            this.CalcDifferentsNames();

            this.isInitialized = true;
        }
    }
}
