﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Fonts.Type1;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
	internal class FontDescriptorOld : PdfObjectOld, IFontDescriptor
    {
        private readonly InstantLoadProperty<PdfNameOld> fontName;
        private readonly LoadOnDemandProperty<FontStreamOld> fontFile;
        private readonly LoadOnDemandProperty<FontStreamOld> fontFile2;
        private readonly LoadOnDemandProperty<FontStreamOld> fontFile3;
        private readonly InstantLoadProperty<PdfStringOld> fontFamily;
        private readonly InstantLoadProperty<PdfRealOld> fontWeight;
        private readonly InstantLoadProperty<PdfIntOld> flags;
        private readonly LoadOnDemandProperty<PdfArrayOld> fontBBox;
        private readonly InstantLoadProperty<PdfRealOld> italicAngle;
        private readonly InstantLoadProperty<PdfIntOld> missingWidth;
        private readonly InstantLoadProperty<PdfRealOld> ascent;
        private readonly InstantLoadProperty<PdfRealOld> descent;

        public FontDescriptorOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.fontName = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "FontName", IsRequired = true });
            this.fontFamily = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "FontFamily" }, Converters.PdfStringConverter);
            this.fontWeight = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "FontWeight" }, Converters.PdfRealConverter);
            this.flags = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Flags", IsRequired = true });
            this.fontBBox = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "FontBBox", IsRequired = true });
            this.italicAngle = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "ItalicAngle" }, Converters.PdfRealConverter);
            this.missingWidth = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "MissingWidth" }, new PdfIntOld(contentManager, 0));
            this.fontFile = this.CreateLoadOnDemandProperty<FontStreamOld>(new PdfPropertyDescriptor { Name = "FontFile" });
            this.fontFile2 = this.CreateLoadOnDemandProperty<FontStreamOld>(new PdfPropertyDescriptor { Name = "FontFile2" });
            this.fontFile3 = this.CreateLoadOnDemandProperty<FontStreamOld>(new PdfPropertyDescriptor { Name = "FontFile3" });
            this.ascent = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "Ascent", IsRequired = true }, Converters.PdfRealConverter);
            this.descent = this.CreateInstantLoadProperty<PdfRealOld>(new PdfPropertyDescriptor { Name = "Descent", IsRequired = true }, Converters.PdfRealConverter);
        }

        public bool IsSymbolic
        {
            get
            {
                return this.GetFlag(3);
            }
        }

        public bool IsNonSymbolic
        {
            get
            {
                return this.GetFlag(6);
            }
        }

        public PdfNameOld FontName
        {
            get
            {
                return this.fontName.GetValue();
            }
            set
            {
                this.fontName.SetValue(value);
            }
        }

        public PdfRealOld Ascent
        {
            get
            {
                return this.ascent.GetValue();
            }
            set
            {
                this.ascent.SetValue(value);
            }
        }

        public PdfRealOld Descent
        {
            get
            {
                return this.descent.GetValue();
            }
            set
            {
                this.descent.SetValue(value);
            }
        }

        public PdfStringOld FontFamily
        {
            get
            {
                return this.fontFamily.GetValue();
            }
            set
            {
                this.fontFamily.SetValue(value);
            }
        }

        public PdfRealOld FontWeight
        {
            get
            {
                return this.fontWeight.GetValue();
            }
            set
            {
                this.fontWeight.SetValue(value);
            }
        }

        public PdfIntOld Flags
        {
            get
            {
                return this.flags.GetValue();
            }
            set
            {
                this.flags.SetValue(value);
            }
        }

        public PdfArrayOld FontBBox
        {
            get
            {
                return this.fontBBox.GetValue();
            }
            set
            {
                this.fontBBox.SetValue(value);
            }
        }

        public PdfRealOld ItalicAngle
        {
            get
            {
                return this.italicAngle.GetValue();
            }
            set
            {
                this.italicAngle.SetValue(value);
            }
        }

        public PdfIntOld MissingWidth
        {
            get
            {
                return this.missingWidth.GetValue();
            }
            set
            {
                this.missingWidth.SetValue(value);
            }
        }

        public FontStreamOld FontFile
        {
            get
            {
                return this.fontFile.GetValue();
            }
            set
            {
                this.fontFile.SetValue(value);
            }
        }
      
        public FontStreamOld FontFile2
        {
            get
            {
                return this.fontFile2.GetValue();
            }
            set
            {
                this.fontFile2.SetValue(value);
            }
        }
      
        public FontStreamOld FontFile3
        {
            get
            {
                return this.fontFile3.GetValue();
            }
            set
            {
                this.fontFile3.SetValue(value);
            }
        }

        public bool GetFontFamily(out string fontFamily, out FontWeight fontWeight, out FontStyle fontStyle)
        {
            fontFamily = default(string);
            fontWeight = default(FontWeight);
            fontStyle = default(FontStyle);

            if (this.FontFamily == null || string.IsNullOrEmpty(this.FontFamily.ToString()))
            {
                return false;
            }

            FormatProviders.Pdf.Model.Elements.Fonts.FontsHelper.GetFontFamily(this.FontFamily.ToString(), out fontFamily, out fontWeight, out fontStyle);
           
            if (this.FontWeight != null)
            {
                fontWeight = FontWeights.Normal;
                int res;
                Helper.UnboxInt(this.FontWeight, out res);
                if (res >= PostScriptConstants.Bold)
                {
                    fontWeight = FontWeights.Bold;
                }
            }
            
            if (this.ItalicAngle != null)
            {
                fontStyle = FontStyles.Normal;
                if (this.ItalicAngle.Value != 0)
                {
                    fontStyle = FontStyles.Italic;
                }
            }

            return true;
        }

        public CFFFontSource GetCFFFontSource()
        {
            try
            {
                if (this.FontFile3 != null)
                {
                    byte[] data = this.ContentManager.ReadData(this.FontFile3.Reference);
                    if (this.FontFile3.Subtype.Value == PdfNames.CIDFontType0C || this.FontFile3.Subtype.Value == PdfNames.Type1C)
                    {
                        CFFFontFile cff = new CFFFontFile(data);
                        return cff.FontSource;
                    }
                   
                    if (this.FontFile3.Subtype.Value == PdfNames.OpenType)
                    {
                        OpenTypeFontSource openType = new OpenTypeFontSource(new OpenTypeFontReader(data));
                        if (openType.Outlines == Outlines.OpenType)
                        {
                            return openType.CFF;
                        }
                    }
                }
            }
            catch (Exception)
            {
            }

            return null;
        }

        public OpenTypeFontSource GetOpenTypeFontSource()
        {
            try
            {
                byte[] data;

                if (this.TryGetFontData(this.FontFile2, out data))
                {
                    return new OpenTypeFontSource(new OpenTypeFontReader(data));
                }
                else if (this.FontFile3 != null &&
                    this.FontFile3.Subtype.Value == PdfNames.OpenType &&
                    this.TryGetFontData(this.FontFile3, out data))
                {
                    return new OpenTypeFontSource(new OpenTypeFontReader(data));
                }
                else
                {
                    string fontFamily;
                    FontWeight fontWeight;
                    FontStyle fontStyle;

                    if (this.GetFontFamily(out fontFamily, out fontWeight, out fontStyle))
                    {
                        return this.ContentManager.FontsManager.GetTrueTypeFontSource(fontFamily, fontStyle, fontWeight);
                    }
                    else if (this.FontName != null && this.FontName.Value != null)
                    {
                        return this.ContentManager.FontsManager.GetTrueTypeFontSource(this.FontName.Value);
                    }
                    else
                    {
                        return null;
                    }
                }
            }
            catch (Exception)
            {
                return null;
            }
        }

        private bool TryGetFontData(FontStreamOld fontStream, out byte[] data)
        {
            if(fontStream == null)
            {
                data = null;
                return false;
            }

            data = this.ContentManager.ReadData(fontStream.Reference);
            return data.Length != 0;
        }

        private bool GetFlag(byte bit)
        {
            bit--;

            return Helper.GetBit(this.Flags.Value, bit);
        }
    }
}
