﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
	internal abstract class SimpleFontOld : FontBaseOld
    {
        private readonly InstantLoadProperty<PdfNameOld> baseFont;
        private readonly InstantLoadProperty<PdfIntOld> firstChar;
        private readonly InstantLoadProperty<PdfArrayOld> widths;
        private readonly InstantLoadProperty<FontDescriptorOld> fontDescriptor;
        private readonly InstantLoadProperty<EncodingOld> encoding;

        private bool fontFamilyLoaded;

        private string fontFamily;
        private FontStyle fontStyle;
        private FontWeight fontWeight;

        public SimpleFontOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.firstChar = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "FirstChar", IsRequired = true });
            this.widths = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Widths", IsRequired = true });
            this.fontDescriptor = this.CreateInstantLoadProperty<FontDescriptorOld>(new PdfPropertyDescriptor { Name = "FontDescriptor" });
            this.encoding = this.CreateInstantLoadProperty<EncodingOld>(new PdfPropertyDescriptor { Name = "Encoding" }, Converters.EncodingConverter);
            this.baseFont = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "BaseFont", IsRequired = true });
        }

        public PdfNameOld BaseFont
        {
            get
            {
                return this.baseFont.GetValue();
            }
            set
            {
                this.baseFont.SetValue(value);
            }
        }

        public PdfIntOld FirstChar
        {
            get
            {
                return this.firstChar.GetValue();
            }
            set
            {
                this.firstChar.SetValue(value);
            }
        }

        public PdfArrayOld Widths
        {
            get
            {
                return this.widths.GetValue();
            }
            set
            {
                this.widths.SetValue(value);
            }
        }

        public FontDescriptorOld FontDescriptor
        {
            get
            {
                return this.fontDescriptor.GetValue();
            }
            set
            {
                this.fontDescriptor.SetValue(value);
            }
        }

        public EncodingOld Encoding
        {
            get
            {
                return this.encoding.GetValue();
            }
            set
            {
                this.encoding.SetValue(value);
            }
        }

        public override double GetGlyphWidth(GlyphOld glyph)
        {
            if (this.Widths != null)
            {
                int ind = glyph.CharId.IntValue - this.FirstChar.Value;

                if (0 <= ind && ind < this.Widths.Count)
                {
                    double d;
                    this.Widths.TryGetReal(ind, out d);
                    d /= 1000.0;
                    return d;
                }

                if (this.FontDescriptor != null)
                {
                    return this.FontDescriptor.MissingWidth.Value / 1000.0;
                }
            }

            return base.GetGlyphWidth(glyph);
        }

        protected override void RenderGlyph(PdfContext context, GlyphOld glyph)
        {
            if (this.FontDescriptor != null)
            {
                glyph.Ascent = this.FontDescriptor.Ascent.Value;
                glyph.Descent = this.FontDescriptor.Descent.Value;
            }

            this.LoadFontFamily();
            glyph.FontFamily = this.fontFamily;
            glyph.FontStyle = this.fontStyle;
            glyph.FontWeight = this.fontWeight;

            base.RenderGlyph(context, glyph);
        }

        private void LoadFontFamily()
        {
            if (!this.fontFamilyLoaded)
            {
                if (this.FontDescriptor == null || !this.FontDescriptor.GetFontFamily(out this.fontFamily, out this.fontWeight, out this.fontStyle) && this.BaseFont.Value != null)
                {
                    FormatProviders.Pdf.Model.Elements.Fonts.FontsHelper.GetFontFamily(this.BaseFont.Value, out this.fontFamily, out this.fontWeight, out this.fontStyle);
                }

                this.fontFamilyLoaded = true;
            }
        }
    }
}
