﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    [PdfClass(TypeName = "Font", SubtypeProperty = "Subtype", SubtypeValue = "TrueType")]
    internal class TrueTypeFontOld : SimpleFontOld
    {
        private TrueTypeGlyphInitializationInfo initializationInfo;

        public TrueTypeFontOld(PdfContentManager contentManager) 
            : base(contentManager)
        {
        }

        protected override FontSource FontSource
        {
            get
            {
                this.EnsureGlyphInitializationInfo();

                return this.initializationInfo.FontSource;
            }
        }

        protected override IEnumerable<GlyphOld> GetGlyphs(PdfStringOld str)
        {
            this.EnsureGlyphInitializationInfo();

            if (this.initializationInfo.Platform == TrueTypeGlyphInitializationInfo.PlatformType.Unknown)
            {
                return base.GetGlyphs(str);
            }
            else
            {
                GlyphOld[] result = new GlyphOld[str.Value.Length];

                for (int i = 0; i < result.Length; i++)
                {
                    GlyphOld g = new GlyphOld();
                    byte b = str.Value[i];
                    g.CharId = new CharCodeOld(b);
                    TrueTypeFont.InitializeCoreGlyph(g, this.initializationInfo, b);
                    result[i] = g;
                }

                return result;
            }
        }

        private void EnsureGlyphInitializationInfo()
        {
            if (this.initializationInfo == null)
            {
                OpenTypeFontSource fontSource = null;

                if (this.FontDescriptor != null)
                {
                    fontSource = this.FontDescriptor.GetOpenTypeFontSource();
                }

                if (fontSource == null && this.BaseFont != null && this.BaseFont.Value != null)
                {
                    fontSource = this.ContentManager.FontsManager.GetTrueTypeFontSource(this.BaseFont.Value);
                }

                if (fontSource == null)
                {
                    fontSource = this.ContentManager.FontsManager.GetTrueTypeFontSource(FixedDocumentDefaults.TrueTypeFallbackFontName);
                }

                this.initializationInfo = TrueTypeFont.GetGlyphInitializationInfo(fontSource, this.FontDescriptor, this.Encoding);
            }
        }
    }
}
