﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    [PdfClass(TypeName = "Font", SubtypeProperty = "Subtype", SubtypeValue = "Type0")]
    internal class Type0FontOld : FontBaseOld
    {
        private static readonly double DefaultGlyphWidth = 1.0;

        private readonly InstantLoadProperty<PdfNameOld> baseFont;
        private readonly InstantLoadProperty<CMapStream> encoding;
        private readonly InstantLoadProperty<PdfArrayOld> descendantFonts;

        private Dictionary<CharCodeOld, double> widths;
        private double? defaultWidth;
        private CIDFontOld cidFont;

        public Type0FontOld(PdfContentManager contentManager) :
            base(contentManager)
        {
            this.encoding = this.CreateInstantLoadProperty<CMapStream>(new PdfPropertyDescriptor { Name = "Encoding", IsRequired = true }, Converters.CMapStreamConverter);
            this.descendantFonts = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "DescendantFonts", IsRequired = true });
            this.baseFont = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "BaseFont", IsRequired = true });
        }

        public PdfNameOld BaseFont
        {
            get
            {
                return this.baseFont.GetValue();
            }
            set
            {
                this.baseFont.SetValue(value);
            }
        }

        public CMapStream Encoding
        {
            get
            {
                return this.encoding.GetValue();
            }
            set
            {
                this.encoding.SetValue(value);
            }
        }

        public PdfArrayOld DescendantFonts
        {
            get
            {
                return this.descendantFonts.GetValue();
            }
            set
            {
                this.descendantFonts.SetValue(value);
            }
        }

        public CIDFontOld CIDFont
        {
            get
            {
                if (this.cidFont == null)
                {
                    this.cidFont = this.DescendantFonts.GetElement<CIDFontOld>(0, Converters.CIDFontConverter);
                }

                return this.cidFont;
            }
        }

        protected override Telerik.Windows.Documents.Core.Fonts.FontSource FontSource
        {
            get { return this.CIDFont.FontSource; }
        }

        public override double GetGlyphWidth(GlyphOld glyph)
        {
            this.LoadWidths();

            double glyphWidth;
            if (this.widths == null || !this.widths.TryGetValue(glyph.CharId, out glyphWidth))
            {
                if (this.defaultWidth.HasValue)
                {
                    glyphWidth = this.defaultWidth.Value;
                }
                else if (this.FontSource != null)
                {
                    glyphWidth = base.GetGlyphWidth(glyph);
                }
                else
                {
                    glyphWidth = DefaultGlyphWidth;
                }
            }

            return glyphWidth;
        }

        protected override IEnumerable<GlyphOld> GetGlyphs(PdfStringOld str)
        {
            // TODO: Handle predefined CMAPS.
            if (this.Encoding != null)
            {
                CMapOld encoding = this.FontsManager.GetType0FontEncoding(this);
                var cids = encoding.GetCharIds(str.Value);
                var res = this.CIDFont.GetGlyphs(cids);
                if (res != null)
                {
                    return res;
                }
            }

            return base.GetGlyphs(str);
        }

        protected override void RenderGlyph(PdfContext context, GlyphOld glyph)
        {
            this.CIDFont.RenderGlyph(context, glyph);

            if (this.CIDFont.FontDescriptor == null)
            {
                if (this.BaseFont == null)
                {
                    return;
                }
                if (Documents.Core.Fonts.FontsManager.IsStandardFontName(this.BaseFont.Value))
                {
                    Documents.Core.Fonts.StandardFontDescriptor sfd = Documents.Core.Fonts.FontsManager.GetStandardFontDescriptor(this.BaseFont.Value);
                    glyph.Ascent = sfd.Ascent;
                    glyph.Descent = sfd.Descent;
                }
                else if (this.FontSource != null)
                {
                    glyph.Ascent = this.FontSource.Ascent;
                    glyph.Descent = this.FontSource.Descent;
                }
            }
            else
            {
                glyph.Ascent = this.CIDFont.FontDescriptor.Ascent.Value;
                glyph.Descent = this.CIDFont.FontDescriptor.Descent.Value;
            }

            base.RenderGlyph(context, glyph);
        }

        private void LoadWidths()
        {
            if (this.CIDFont.DefaultWidth != null)
            {
                this.defaultWidth = this.CIDFont.DefaultWidth.Value / 1000.0;
            }

            if (this.widths == null && this.CIDFont.Widths != null)
            {
                this.widths = new Dictionary<CharCodeOld, double>();
                PdfArrayOld widths;
                int i = 0;
                int firstCharCode;
                int lastCharCode;
                double width;

                while (i + 1 < this.CIDFont.Widths.Count)
                {
                    widths = this.CIDFont.Widths[i + 1] as PdfArrayOld;
                    if (widths != null)
                    {
                        this.CIDFont.Widths.TryGetInt(i, out firstCharCode);

                        for (ushort j = 0; j < widths.Count; j++)
                        {
                            if (widths.TryGetReal(j, out width))
                            {
                                this.widths[new CharCodeOld((ushort)(firstCharCode + j))] = width / 1000;
                            }
                        }

                        i += 2;
                    }
                    else
                    {
                        this.CIDFont.Widths.TryGetInt(i, out firstCharCode);
                        this.CIDFont.Widths.TryGetInt(i + 1, out lastCharCode);
                        this.CIDFont.Widths.TryGetReal(i + 2, out width);
                        width /= 1000;

                        for (ushort j = (ushort)firstCharCode; j <= lastCharCode; j++)
                        {
                            this.widths[new CharCodeOld(j)] = width;
                        }

                        i += 3;
                    }
                }
            }
        }
    }
}
