﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;
using Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts
{
    [PdfClass(TypeName = "Font", SubtypeProperty = "Subtype", SubtypeValue = "Type1")]
    internal class Type1FontOld : SimpleFontOld
    {
        private Type1FontFileType fontFileType;
        private CFFFontSource ccfFontSource;
        private Type1FontSource type1FontSource;

        public Type1FontOld(PdfContentManager contentManager) 
            : base(contentManager)
        {
            this.fontFileType = Type1FontFileType.Undefined;
        }

        protected override FontSource FontSource
        {
            get
            {
                switch (this.fontFileType)
                {
                    case Type1FontFileType.CFF:
                        return this.CFFFontSource;
                    case Type1FontFileType.StandardFont:
                    case Type1FontFileType.Type1:
                    case Type1FontFileType.Missing:
                        return this.Type1FontSource;
                }
                return null;
            }
        }

        private Type1FontFileType FontFileType
        {
            get
            {
                if (this.fontFileType == Type1FontFileType.Undefined)
                {
                    if (this.FontDescriptor != null)
                    {
                        if (this.FontDescriptor.FontFile3 != null)
                        {
                            this.fontFileType = Type1FontFileType.CFF;
                        }
                        else if (this.FontDescriptor.FontFile != null)
                        {
                            this.fontFileType = Type1FontFileType.Type1;
                        }
                        else if (this.BaseFont != null && Documents.Core.Fonts.FontsManager.IsStandardFontName(this.BaseFont.Value))
                        {
                            this.fontFileType = Type1FontFileType.StandardFont;
                        }
                        else
                        {
                            this.fontFileType = Type1FontFileType.Missing;
                        }
                    }
                    else if (this.BaseFont != null && Documents.Core.Fonts.FontsManager.IsStandardFontName(this.BaseFont.Value))
                    {
                        this.fontFileType = Type1FontFileType.StandardFont;
                    }
                    else
                    {
                        this.fontFileType = Type1FontFileType.Missing;
                    }
                }
                return this.fontFileType;
            }
        }

        private CFFFontSource CFFFontSource
        {
            get
            {
                if (ccfFontSource == null && this.FontFileType == Type1FontFileType.CFF)
                {
                    this.ccfFontSource = this.FontDescriptor.GetCFFFontSource();
                }
                
                return this.ccfFontSource;
            }
        }

        private Type1FontSource Type1FontSource
        {
            get
            {
                if (this.type1FontSource == null)
                {
                    if (this.FontFileType == Type1FontFileType.Type1)
                    {
                        byte[] data = this.ContentManager.ReadData(this.FontDescriptor.FontFile.Reference);
                        this.type1FontSource = new Type1FontSource(data);
                    }
                    else if (this.FontFileType == Type1FontFileType.StandardFont)
                    {
                        this.type1FontSource = this.FontsManager.SystemFontsManager.GetStandardFontSource(this.BaseFont.Value);
                    }
                    else if (this.FontFileType == Type1FontFileType.Missing)
                    {
                        this.type1FontSource = this.FontsManager.SystemFontsManager.GetType1FallbackFontSource(this.BaseFont.Value);
                    }
                }
               
                return this.type1FontSource;
            }
        }

        protected override void RenderGlyph(PdfContext context, GlyphOld glyph)
        {
            if (this.FontDescriptor == null && this.BaseFont != null && Documents.Core.Fonts.FontsManager.IsStandardFontName(this.BaseFont.Value))
            {
                StandardFontDescriptor sfd = Documents.Core.Fonts.FontsManager.GetStandardFontDescriptor(this.BaseFont.Value);
                if (sfd != null)
                {
                    glyph.Ascent = sfd.Ascent;
                    glyph.Descent = sfd.Descent;
                }
            }

            base.RenderGlyph(context, glyph);
        }

        protected override IEnumerable<GlyphOld> GetGlyphs(PdfStringOld str)
        {
            switch (this.FontFileType)
            {
                case Type1FontFileType.Type1:
                case Type1FontFileType.StandardFont:
                case Type1FontFileType.Missing:
                    return this.GetGlyphsFromType1FontSource(str);
                case Type1FontFileType.CFF:
                    return this.GetGlyphsFromCFFFontSource(str);
            }

            return base.GetGlyphs(str);
        }

        private IEnumerable<GlyphOld> GetGlyphsFromCFFFontSource(PdfStringOld str)
        {
            foreach (byte b in str.Value)
            {                
                GlyphOld g = new GlyphOld();
                Type1Font.InitializeCoreGlyph(g, this.CFFFontSource, this.Encoding, b);
                g.CharId = new Model.Internal.Classes.CharCodeOld(b);
                
                yield return g;
            }
        }
      
        private IEnumerable<GlyphOld> GetGlyphsFromType1FontSource(PdfStringOld str)
        {
            foreach (byte b in str.Value)
            {
                GlyphOld g = new GlyphOld();
                Type1Font.InitializeCoreGlyph(g, this.Type1FontSource, this.Encoding, b);
                g.CharId = new Model.Internal.Classes.CharCodeOld(b);
               
                yield return g;
            }
        }
    }
}
