﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Forms
{
    [PdfClass]
    internal class FormFieldNodeOld : PdfObjectOld, ITreeNode<FormFieldNodeOld>
    {
        private readonly InstantLoadProperty<PdfNameOld> fieldType;
        private readonly LoadOnDemandProperty<FormFieldNodeOld> parent;
        private readonly LoadOnDemandProperty<PdfArrayOld> kids;
        private readonly InstantLoadProperty<PdfStringOld> partialName;
        private readonly InstantLoadProperty<PdfStringOld> userInterfaceName;
        private readonly InstantLoadProperty<PdfStringOld> mappingName;
        private readonly InstantLoadProperty<PdfIntOld> fieldFlags;
        private readonly InstantLoadProperty<PdfIntOld> maxLengthOfInputCharacters;
        private readonly LoadOnDemandProperty<PdfObjectOld> fieldValue;
        private readonly LoadOnDemandProperty<PdfObjectOld> defaultFieldValue;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> additionalActions;
        private readonly InstantLoadProperty<PdfStringOld> defaultAppearance;
        private readonly InstantLoadProperty<PdfIntOld> quadding;
        private readonly LoadOnDemandProperty<PdfArrayOld> options;
        private readonly InstantLoadProperty<PdfIntOld> topIndex;
        private readonly LoadOnDemandProperty<PdfArrayOld> sortedIndices;
        private readonly LoadOnDemandProperty<SignatureFieldLockOld> signatureFieldLock;
        private readonly LoadOnDemandProperty<SignatureFieldSeedOld> seedValue;

        public FormFieldNodeOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.fieldType = this.CreateInstantLoadProperty<PdfNameOld>(new PdfPropertyDescriptor { Name = "FT", IsInheritable = true });
            this.parent = this.CreateLoadOnDemandProperty<FormFieldNodeOld>(new PdfPropertyDescriptor { Name = "Parent" });
            this.kids = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Kids" });
            this.partialName = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "T" });
            this.userInterfaceName = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "TU" });
            this.mappingName = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "TM" });
            this.fieldFlags = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Ff", IsInheritable = true });
            this.maxLengthOfInputCharacters = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "MaxLen", IsInheritable = true });
            this.fieldValue = this.CreateLoadOnDemandProperty<PdfObjectOld>(new PdfPropertyDescriptor { Name = "V", IsInheritable = true });
            this.defaultFieldValue = this.CreateLoadOnDemandProperty<PdfObjectOld>(new PdfPropertyDescriptor { Name = "DV", IsInheritable = true });
            this.additionalActions = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "AA" });
            this.defaultAppearance = this.CreateInstantLoadProperty<PdfStringOld>(new PdfPropertyDescriptor { Name = "DA", IsInheritable = true });
            this.quadding = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Q", IsInheritable = true });
            this.options = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Opt", IsInheritable = true });
            this.topIndex = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "TI" }, new PdfIntOld(contentManager, 0));
            this.sortedIndices = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "I" });
            this.signatureFieldLock = this.CreateLoadOnDemandProperty<SignatureFieldLockOld>(new PdfPropertyDescriptor { Name = "Lock" }, Converters.PdfDictionaryToPdfObjectConverter);
            this.seedValue = this.CreateLoadOnDemandProperty<SignatureFieldSeedOld>(new PdfPropertyDescriptor { Name = "SV" }, Converters.PdfDictionaryToPdfObjectConverter);
        }

        public PdfNameOld FieldType
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.fieldType.GetValue());
            }
            set
            {
                this.fieldType.SetValue(value);
            }
        }

        public FormFieldNodeOld Parent
        {
            get
            {
                return this.parent.GetValue();
            }
            set
            {
                this.parent.SetValue(value);
            }
        }

        public PdfArrayOld Kids
        {
            get
            {
                return this.kids.GetValue();
            }
            set
            {
                this.kids.SetValue(value);
            }
        }

        public PdfStringOld PartialName
        {
            get
            {
                return this.partialName.GetValue();
            }
            set
            {
                this.partialName.SetValue(value);
            }
        }

        public PdfStringOld UserInterfaceName
        {
            get
            {
                return this.userInterfaceName.GetValue();
            }
            set
            {
                this.userInterfaceName.SetValue(value);
            }
        }

        public PdfStringOld MappingName
        {
            get
            {
                return this.mappingName.GetValue();
            }
            set
            {
                this.mappingName.SetValue(value);
            }
        }

        public PdfIntOld MaxLengthOfInputCharacters
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.maxLengthOfInputCharacters.GetValue());
            }
            set
            {
                this.maxLengthOfInputCharacters.SetValue(value);
            }
        }

        public PdfObjectOld FieldValue
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.fieldValue.GetValue());
            }
            set
            {
                this.fieldValue.SetValue(value);
            }
        }

        public PdfObjectOld DefaultFieldValue
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.defaultFieldValue.GetValue());
            }
            set
            {
                this.defaultFieldValue.SetValue(value);
            }
        }

        public PdfDictionaryOld AdditionalActions
        {
            get
            {
                return this.additionalActions.GetValue();
            }
            set
            {
                this.additionalActions.SetValue(value);
            }
        }

        public PdfStringOld DefaultAppearance
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.defaultAppearance.GetValue(), this.ContentManager.DocumentCatalog.AcroForm.DefaultAppearance);
            }
            set
            {
                this.defaultAppearance.SetValue(value);
            }
        }

        public PdfIntOld Quadding
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.quadding.GetValue(), this.ContentManager.DocumentCatalog.AcroForm.Quadding);
            }
            set
            {
                this.quadding.SetValue(value);
            }
        }

        public PdfArrayOld Options
        {
            get
            {
                return this.GetInheritableProperty((formFieldNode) => formFieldNode.options.GetValue());
            }
            set
            {
                this.options.SetValue(value);
            }
        }

        public PdfIntOld TopIndex
        {
            get
            {
                return this.topIndex.GetValue();
            }
            set
            {
                this.topIndex.SetValue(value);
            }
        }

        public PdfArrayOld SortedIndices
        {
            get
            {
                return this.sortedIndices.GetValue();
            }
            set
            {
                this.sortedIndices.SetValue(value);
            }
        }

        public SignatureFieldLockOld SignatureFieldLock
        {
            get
            {
                return this.signatureFieldLock.GetValue();
            }
            set
            {
                this.signatureFieldLock.SetValue(value);
            }
        }

        public SignatureFieldSeedOld SeedValue
        {
            get
            {
                return this.seedValue.GetValue();
            }
            set
            {
                this.seedValue.SetValue(value);
            }
        }

        public string FullName
        {
            get
            {
                string name = this.PartialName.ToString();

                FormFieldNodeOld parentNode = this.Parent;

                while(parentNode != null)
                {
                    if(parentNode.PartialName != null)
                    {
                        name = string.Format("{0}.{1}", parentNode.PartialName.ToString(), name);
                    }

                    parentNode = parentNode.Parent;
                }

                return name;
            }
        }

        FormFieldNodeOld ITreeNode<FormFieldNodeOld>.NodeValue
        {
            get
            {
                return this;
            }
        }

        ITreeNode<FormFieldNodeOld> ITreeNode<FormFieldNodeOld>.ParentNode
        {
            get
            {
                return this.Parent;
            }
        }
    }
}
