﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Functions
{
    internal abstract class FunctionOld : PdfObjectOld
    {
        private readonly InstantLoadProperty<PdfArrayOld> domain;
        private readonly InstantLoadProperty<PdfArrayOld> range;

        public FunctionOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.domain = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Domain", true));
            this.range = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Range"));
        }

        public PdfArrayOld Domain
        {
            get
            {
                return this.domain.GetValue();
            }
            set
            {
                this.domain.SetValue(value);
            }
        }

        public PdfArrayOld Range
        {
            get
            {
                return this.range.GetValue();
            }
            set
            {
                this.range.SetValue(value);
            }
        }

        internal FunctionBase FunctionModel { get; private set; }

        public static FunctionOld CreateFunction(PdfContentManager contentManager, PdfDataStream pdfStream)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(pdfStream, "pdfStream");

            FunctionOld function = CreateFunctionInternal(contentManager, pdfStream.Dictionary);
            byte[] data = pdfStream.ReadData(contentManager);
            function.FunctionModel = function.CreateFunctionModel(data);

            return function;
        }

        public static FunctionOld CreateFunction(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dict, "dict");

            FunctionOld function = CreateFunctionInternal(contentManager, dict);
            function.FunctionModel = function.CreateFunctionModel();

            return function;
        }

        public double[] Execute(double[] inputValues)
        {
            Guard.ThrowExceptionIfNull(this.FunctionModel, "FunctionModel");

            double[] outputValues = this.FunctionModel.Execute(inputValues);

            return outputValues;
        }

        protected abstract FunctionBase CreateFunctionModel();

        protected abstract FunctionBase CreateFunctionModel(byte[] data);

        protected double[] ReadDoubleArray(PdfArrayOld pdfArray)
        {
            if (pdfArray == null)
            {
                return null;
            }

            double[] array = new double[pdfArray.Count];

            for (int i = 0; i < array.Length; i++)
            {
                double number;
                if (pdfArray.TryGetReal(i, out number))
                {
                    array[i] = number;
                }
            }

            return array;
        }

        private static FunctionOld CreateFunctionInternal(PdfContentManager contentManager, PdfDictionaryOld dict)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dict, "dict");

            int functionType;
            dict.TryGetInt(PdfNames.FunctionType, out functionType);
            FunctionOld function;

            switch (functionType)
            {
                case 0:
                    function = new SampledFunctionOld(contentManager);
                    break;
                case 2:
                    function = new ExponentialInterpolationFunctionOld(contentManager);
                    break;
                case 3:
                    function = new StitchingFunctionOld(contentManager);
                    break;
                case 4:
                    function = new PostScriptCalculatorFunctionOld(contentManager);
                    break;
                default:
                    throw new NotSupportedFunctionTypeException(functionType);
            }

            function.Load(dict);

            return function;
        }
    }
}
