﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Functions
{
    [PdfClass(TypeName = "Function", SubtypeProperty = PdfNames.FunctionType, SubtypeValue = "0")]
    internal class SampledFunctionOld : FunctionOld
    {
        private readonly InstantLoadProperty<PdfArrayOld> size;
        private readonly InstantLoadProperty<PdfIntOld> bitsPerSample;
        private readonly InstantLoadProperty<PdfArrayOld> encode;
        private readonly InstantLoadProperty<PdfArrayOld> decode;

        public SampledFunctionOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.size = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Size", true));
            this.bitsPerSample = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor("BitsPerSample", true), Converters.PdfIntConverter);
            this.encode = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Encode"));
            this.decode = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor("Decode"));
        }
        
        public PdfArrayOld Size
        {
            get
            {
                return this.size.GetValue();
            }
            set
            {
                this.size.SetValue(value);
            }
        }

        public PdfIntOld BitsPerSample
        {
            get
            {
                return this.bitsPerSample.GetValue();
            }
            set
            {
                this.bitsPerSample.SetValue(value);
            }
        }

        public PdfArrayOld Encode
        {
            get
            {
                return this.encode.GetValue();
            }
            set
            {
                this.encode.SetValue(value);
            }
        }

        public PdfArrayOld Decode
        {
            get
            {
                return this.decode.GetValue();
            }
            set
            {
                this.decode.SetValue(value);
            }
        }

        protected override FunctionBase CreateFunctionModel()
        {
            return this.CreateFunctionModel(null);
        }

        protected override FunctionBase CreateFunctionModel(byte[] data)
        {
            int[]  size = new int[this.Size.Count];

            for (int i = 0; i < size.Length; i++)
            {
                int s;
                if (this.Size.TryGetInt(i, out s))
                {
                    size[i] = s;
                }
            }

            double[] domain = this.ReadDoubleArray(this.Domain);
            double[] range = this.ReadDoubleArray(this.Range);
            double[] encode = this.ReadDoubleArray(this.Encode);
            double[] decode = this.ReadDoubleArray(this.Decode);
            SampledFunction function = new SampledFunction(data, size, this.BitsPerSample.Value, domain, range, encode, decode);

            return function;
        }
    }
}
