﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Navigation
{
    internal class DestinationOld : PdfObjectOld
    {
        public DestinationOld(PdfContentManager contentManager)
            : base(contentManager)
        {
        }

        public PageOld Page
        {
            get;
            set;
        }

        public DestinationTypeOld Type
        {
            get;
            set;
        }

        public double? Left
        {
            get;
            set;
        }

        public double? Top
        {
            get;
            set;
        }

        public double? Right
        {
            get;
            set;
        }

        public double? Bottom
        {
            get;
            set;
        }

        public double? Zoom
        {
            get;
            set;
        }

        public void Load(PdfArrayOld array)
        {
            Guard.ThrowExceptionIfNull(array, "array");

            this.LoadPage(array);

            PdfNameOld typeName = array.GetElement<PdfNameOld>(1);
            if (typeName != null)
            {
                DestinationTypeOld type;
                Helper.EnumTryParse<DestinationTypeOld>(typeName.ToString(), out type);
                this.Type = type;

                switch (this.Type)
                {
                    case DestinationTypeOld.XYZ:
                        this.LoadXYZ(array);
                        break;
                 
                    case DestinationTypeOld.Fit:
                        this.LoadFit(array);
                        break;
                  
                    case DestinationTypeOld.FitH:
                        this.LoadFitH(array);
                        break;
                  
                    case DestinationTypeOld.FitV:
                        this.LoadFitV(array);
                        break;
                  
                    case DestinationTypeOld.FitR:
                        this.LoadFitR(array);
                        break;
                   
                    case DestinationTypeOld.FitB:
                        this.LoadFitB(array);
                        break;
                   
                    case DestinationTypeOld.FitBH:
                        this.LoadFitBH(array);
                        break;
                    
                    case DestinationTypeOld.FitBV:
                        this.LoadFitBV(array);
                        break;
                }
             
                this.IsLoaded = true;
            }
        }

        public override void Load(IndirectObjectOld indirectObject)
        {
            Guard.ThrowExceptionIfNull(indirectObject, "indirectObject");

            if (indirectObject.Value is PdfArrayOld)
            {
                this.Load((PdfArrayOld)indirectObject.Value);
            }
            
            base.Load(indirectObject);
        }

        private void LoadPage(PdfArrayOld array)
        {
            IndirectObjectOld obj;
            IndirectReferenceOld reference = array.First() as IndirectReferenceOld;
            if (reference != null && !this.ContentManager.TryGetIndirectObject(reference, out obj))
            {
                this.Page = null;
            }
            else
            {
                this.Page = array.GetElement<PageOld>(0);

                if (this.Page == null)
                {
                    var integer = array.GetElement<PdfSimpleTypeOld<int>>(0);

                    if (integer != null)
                    {
                        int pageIndex = integer.Value;

                        if (0 <= pageIndex && pageIndex < this.ContentManager.DocumentCatalog.Pages.Count.Value)
                        {
                            this.Page = this.ContentManager.GetPages().Skip(pageIndex).FirstOrDefault();
                        }
                    }
                }
            }
        }

        private void ClearValues()
        {
            this.Left = null;
            this.Right = null;
            this.Top = null;
            this.Bottom = null;
            this.Zoom = null;
        }

        private void LoadXYZ(PdfArrayOld array)
        {
            this.ClearValues();

            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Left = d;
            }
            if (array.TryGetReal(3, out d))
            {
                this.Top = d;
            }
            if (array.TryGetReal(4, out d))
            {
                this.Zoom = d;
            }
        }

        private void LoadFit(PdfArrayOld array)
        {
            this.ClearValues();
        }

        private void LoadFitH(PdfArrayOld array)
        {
            this.ClearValues();
            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Top = d;
            }
        }

        private void LoadFitV(PdfArrayOld array)
        {
            this.ClearValues();
            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Left = d;
            }
        }

        private void LoadFitR(PdfArrayOld array)
        {
            this.ClearValues();

            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Left = d;
            }
            if (array.TryGetReal(3, out d))
            {
                this.Bottom = d;
            }
            if (array.TryGetReal(4, out d))
            {
                this.Right = d;
            }
            if (array.TryGetReal(5, out d))
            {
                this.Top = d;
            }
        }

        private void LoadFitB(PdfArrayOld array)
        {
            this.ClearValues();
        }

        private void LoadFitBH(PdfArrayOld array)
        {
            this.ClearValues();
            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Top = d;
            }
        }

        private void LoadFitBV(PdfArrayOld array)
        {
            this.ClearValues();
            double d;
            if (array.TryGetReal(2, out d))
            {
                this.Left = d;
            }
        }
    }
}
