﻿using System.IO;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
    [PdfClass(TypeName = "ObjStm")]
    internal class ObjectStreamOld : PdfStreamOld
    {
        private readonly InstantLoadProperty<PdfIntOld> n;
        private readonly InstantLoadProperty<PdfIntOld> first;
        private int[] offsets;
        private ObjectStreamParser parser;

        public ObjectStreamOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.n = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "N", IsRequired = true });
            this.first = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "First", IsRequired = true });
        }

        public PdfIntOld N
        {
            get
            {
                return this.n.GetValue();
            }
            set
            {
                this.n.SetValue(value);
            }
        }

        public PdfIntOld First
        {
            get
            {
                return this.first.GetValue();
            }
            set
            {
                this.first.SetValue(value);
            }
        }

        public override void Load(Data.PdfDataStream stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            base.Load(stream);

            byte[] data = stream.ReadData(this.ContentManager);
            this.parser = new ObjectStreamParser(this.ContentManager, new MemoryStream(data));
            this.offsets = this.parser.ReadOffsets(this.N.Value, this.First.Value);
        }

        public IndirectObjectOld ReadIndirectObject(int fromIndex)
        {
            return new IndirectObjectOld { Value = this.parser.ReadPdfObject(this.offsets[fromIndex]) };
        }
    }
}
