﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
	internal class PdfResourceOld : PdfObjectOld
    {
        private readonly LoadOnDemandProperty<PdfDictionaryOld> fonts;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> extGState;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> colorSpaces;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> xObjects;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> patterns;
        private readonly LoadOnDemandProperty<PdfDictionaryOld> shadings;

        public PdfResourceOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.fonts = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "Font" });
            this.extGState = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "ExtGState" });
            this.colorSpaces = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "ColorSpace" });
            this.xObjects = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "XObject" });
            this.patterns = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "Pattern" });
            this.shadings = this.CreateLoadOnDemandProperty<PdfDictionaryOld>(new PdfPropertyDescriptor { Name = "Shading" });
        }

        public PdfDictionaryOld ExtGState
        {
            get
            {
                return this.extGState.GetValue();
            }
            set
            {
                this.extGState.SetValue(value);
            }
        }

        public PdfDictionaryOld ColorSpaces
        {
            get
            {
                return this.colorSpaces.GetValue();
            }
            set
            {
                this.colorSpaces.SetValue(value);
            }
        }

        public PdfDictionaryOld XObjects
        {
            get
            {
                return this.xObjects.GetValue();
            }
            set
            {
                this.xObjects.SetValue(value);
            }
        }

        public PdfDictionaryOld Fonts
        {
            get
            {
                return this.fonts.GetValue();
            }
            set
            {
                this.fonts.SetValue(value);
            }
        }

        public PdfDictionaryOld Patterns
        {
            get
            {
                return this.patterns.GetValue();
            }
            set
            {
                this.patterns.SetValue(value);
            }
        }

        public PdfDictionaryOld Shadings
        {
            get
            {
                return this.shadings.GetValue();
            }
            set
            {
                this.shadings.SetValue(value);
            }
        }

		public FontBaseOld GetFont(PdfNameOld key)
		{
            Guard.ThrowExceptionIfNull(key, "key");
           
            FontBaseOld font = this.Fonts.GetElement<FontBaseOld>(key.Value, Converters.FontConverter);
            return font;
		}

        public XObject GetXObject(PdfNameOld key)
        {
            Guard.ThrowExceptionIfNull(key, "key");
            
            XObject obj = this.XObjects.GetElement<XObject>(key.Value, Converters.XObjectConverter);
          
            return obj;
        }

        public ExtGStateOld GetExtGState(PdfNameOld key)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            return this.ExtGState.GetElement<ExtGStateOld>(key.Value);
        }

        public ColorSpaceOld GetColorSpace(PdfNameOld key)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            return this.ColorSpaces.GetElement<ColorSpaceOld>(key.Value, Converters.ColorSpaceConverter);
        }

        public PatternOld GetPattern(PdfNameOld key)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            return this.Patterns.GetElement<PatternOld>(key.Value, Converters.PatternConverter);
        }

        public ShadingOld GetShading(PdfNameOld key)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            return this.Shadings.GetElement<ShadingOld>(key.Value, Converters.ShadingConverter);
        }
    }
}
