﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Encryption;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core
{
	internal class TrailerOld : PdfObjectOld
    {
        private readonly InstantLoadProperty<PdfIntOld> size;
        private readonly InstantLoadProperty<PdfIntOld> prev;
        private readonly InstantLoadProperty<PdfIntOld> xRefStm;
        private readonly LoadOnDemandProperty<PdfArrayOld> id;
        private readonly LoadOnDemandProperty<EncryptOld> encrypt;
        private LoadOnDemandProperty<DocumentCatalogOld> root;

        public TrailerOld(PdfContentManager contentManager)
            : base(contentManager)
		{
            this.size = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Size", IsRequired = true, State = PdfPropertyState.MustBeDirectObject });
            this.prev = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Prev" });
            this.root = this.CreateLoadOnDemandProperty<DocumentCatalogOld>(new PdfPropertyDescriptor { Name = "Root", IsRequired = true, State = PdfPropertyState.MustBeIndirectReference });
            this.xRefStm = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "XRefStm" });
            this.id = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "ID" });
            this.encrypt = this.CreateLoadOnDemandProperty<EncryptOld>(new PdfPropertyDescriptor { Name = "Encrypt" }, PdfConverters.Converters.EncryptConverter);
		}

		public PdfIntOld Size
		{
            get
            {
                return this.size.GetValue();
            }
            set
            {
                this.size.SetValue(value);
            }
		}

		public PdfIntOld Prev
		{
            get
            {
                return this.prev.GetValue();
            }
            set
            {
                this.prev.SetValue(value);
            }
        }

        public DocumentCatalogOld Root
        {
            get
            {
                return this.root.GetValue();
            }
        }

        public PdfIntOld XRefStm
        {
            get
            {
                return this.xRefStm.GetValue();
            }
            set
            {
                this.xRefStm.SetValue(value);
            }
        }

        public PdfArrayOld ID
        {
            get
            {
                return this.id.GetValue();
            }
            set
            {
                this.id.SetValue(value);
            }
        }

        public EncryptOld Encrypt
        {
            get
            {
                return this.encrypt.GetValue();
            }
            set
            {
                this.encrypt.SetValue(value);
            }
        }

        internal void CopyPropertiesFrom(CrossReferenceStreamOld crossReferenceStream)
        {
            Guard.ThrowExceptionIfNull(crossReferenceStream, "crossReferenceStream");

            this.root = crossReferenceStream.Root;
            this.Encrypt = crossReferenceStream.Encrypt;
            this.ID = crossReferenceStream.ID;
            this.Prev = crossReferenceStream.Prev;
            this.Size = crossReferenceStream.Size;
        }
    }
}
