﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Trees
{
    internal class NameTreeNode : PdfObjectOld
    {
        private readonly LoadOnDemandProperty<PdfArrayOld> children;
        private readonly LoadOnDemandProperty<PdfArrayOld> names;
        private readonly LoadOnDemandProperty<PdfArrayOld> limits;

        public NameTreeNode(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.children = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Kids" });
            this.names = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Names" });
            this.limits = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Limits" });
        }

        public PdfArrayOld Names
        {
            get
            {
                return this.names.GetValue();
            }
            set
            {
                this.names.SetValue(value);
            }
        }
                
        public PdfArrayOld Children
        {
            get
            {
                return this.children.GetValue();
            }
            set
            {
                this.children.SetValue(value);
            }
        }

        public PdfArrayOld Limits
        {
            get
            {
                return this.limits.GetValue();
            }
            set
            {
                this.limits.SetValue(value);
            }
        }
        
        public T FindElement<T>(string name, IConverter converter = null) where T : PdfObjectOld
        {
            if (this.Limits != null && !this.IsInLimit(name))
            {
                return default(T); 
            }
            
            if (this.Names != null)
            {
                return this.FindElementInNames<T>(name, converter);
            }
            
            if (this.Children != null)
            {
                for (int i = 0; i < this.Children.Count; i++)
                {
                    NameTreeNode child = this.Children.GetElement<NameTreeNode>(i);
                    if (child.IsInLimit(name))
                    {
                        return child.FindElement<T>(name, converter);
                    }
                }
            }

            return default(T);
        }

        protected bool IsInLimit(string name)
        {
            if (this.Limits == null)
            {
                return false;
            }
            else
            {
                PdfStringOld first = this.Limits.GetElement<PdfStringOld>(0);
                PdfStringOld last = this.Limits.GetElement<PdfStringOld>(1);
                  
                return CompareNames(name, first.ToString()) >= 0 && CompareNames(name, last.ToString()) <= 0;
            }
        }

        private static int CompareNames(string x, string y)
        {
            int minLength = Math.Min(x.Length, y.Length);
            for (int i = 0; i < minLength; i++)
            {
                int compare = x[i].CompareTo(y[i]);
                if (compare != 0)
                {
                    return compare < 0 ? -1 : 1;
                }
            }

            if (x.Length == y.Length)
            {
                return 0;
            }
            else if (x.Length < y.Length)
            {
                return -1;
            }

            return 1;
        }

        private T FindElementInNames<T>(string name, IConverter converter = null) where T : PdfObjectOld
        {
            if (this.Children != null)
            {
                for (int i = 0; i < this.Children.Count; i++)
                {
                    NameTreeNode child = this.Children.GetElement<NameTreeNode>(i);
                    if (child.IsInLimit(name))
                    {
                        return child.FindElement<T>(name, converter);
                    }
                }
            }

            // TODO: Binary search here!
            if (this.Names != null)
            {
                for (int i = 0; i < this.Names.Count; i += 2)
                {
                    PdfStringOld str = this.Names.GetElement<PdfStringOld>(i);
                    if (str.ToString() == name)
                    {
                        T res = converter == null ? this.Names.GetElement<T>(i + 1) : this.Names.GetElement<T>(i + 1, converter);
                        return res;
                    }
                }
            }

            return default(T);
        }
    }
}
