﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Trees
{
    [PdfClass(TypeName = "Pages")]
	internal class PageTreeNodeOld : PdfObjectOld, ITreeNode<PageTreeNodeOld>
    {
        private readonly LoadOnDemandProperty<PageTreeNodeOld> parent;
        private readonly LoadOnDemandProperty<PdfArrayOld> children;
        private readonly InstantLoadProperty<PdfIntOld> count;
        private readonly LoadOnDemandProperty<PdfResourceOld> resources;
        private readonly LoadOnDemandProperty<PdfArrayOld> mediaBox;
        private readonly LoadOnDemandProperty<PdfArrayOld> cropBox;
        private readonly InstantLoadProperty<PdfIntOld> rotate;

        public PageTreeNodeOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.parent = this.CreateLoadOnDemandProperty<PageTreeNodeOld>(new PdfPropertyDescriptor { Name = "Parent", IsRequired = true, State = PdfPropertyState.MustBeIndirectReference });
            this.children = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Kids", IsRequired = true, State = PdfPropertyState.ContainsOnlyIndirectReferences });
            this.count = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Count", IsRequired = true });
            this.resources = this.CreateLoadOnDemandProperty<PdfResourceOld>(new PdfPropertyDescriptor { Name = "Resources", IsRequired = true, IsInheritable = true }, Converters.PdfDictionaryToPdfObjectConverter);
            this.mediaBox = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "MediaBox", IsRequired = true, IsInheritable = true });
            this.cropBox = this.CreateLoadOnDemandProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "CropBox", IsInheritable = true });
            this.rotate = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Rotate", IsInheritable = true });
        }

        public PageTreeNodeOld Parent
        {
            get
            {
                return this.parent.GetValue();
            }
            set
            {
                this.parent.SetValue(value);
            }
        }

        public PdfArrayOld Children
        {
            get
            {
                return this.children.GetValue();
            }
            set
            {
                this.children.SetValue(value);
            }
        }

        public PdfIntOld Count
        {
            get
            {
                return this.count.GetValue();
            }
            set
            {
                this.count.SetValue(value);
            }
        }

        public PdfResourceOld Resources
        {
            get
            {
                return this.GetInheritableProperty((pageTreeNode) => pageTreeNode.resources.GetValue());
            }
            set
            {
                this.resources.SetValue(value);
            }
        }

        public PdfIntOld Rotate
        {
            get
            {
                return this.GetInheritableProperty((pageTreeNode) => pageTreeNode.rotate.GetValue());
            }
            set
            {
                this.rotate.SetValue(value);
            }
        }

        public PdfArrayOld MediaBox
        {
            get
            {
                return this.GetInheritableProperty((pageTreeNode) => pageTreeNode.mediaBox.GetValue());
            }
            set
            {
                this.mediaBox.SetValue(value);
            }
        }

        public PdfArrayOld CropBox
        {
            get
            {
                return this.GetInheritableProperty((pageTreeNode) => pageTreeNode.cropBox.GetValue(), this.MediaBox);
            }
            set
            {
                this.cropBox.SetValue(value);
            }
        }

        PageTreeNodeOld ITreeNode<PageTreeNodeOld>.NodeValue
        {
            get
            {
                return this;
            }
        }

        ITreeNode<PageTreeNodeOld> ITreeNode<PageTreeNodeOld>.ParentNode
        {
            get
            {
                return this.Parent;
            }
        }
    }
}
