﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Primitives;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects
{
    internal class ImageMaskOld : XImage, IMask
    {
        private readonly byte[] decodedMaskData;
#if DEBUG
        private ImageBitReader bitReader;
        private int unmaskedColor;
#endif

        internal ImageMaskOld(PdfContentManager contentManager, byte[] decodedMaskData)
            : base(contentManager)
        {
            this.decodedMaskData = decodedMaskData;
        }

        public bool IsColorKeyMask
        {
            get
            {
                return false;
            }
        }

        public SizeI GetMaskedImageSize(IImageDescriptor image)
        {
            return Model.Resources.ImageMask.GetMaskedImageSize(this, image);
        }

        public PixelContainer MaskImage(IImageDescriptor image, PixelContainer pixels)
        {
#if DEBUG
            string filePath = PdfContentExtractor.GetFileName(this, PdfContentExtractor.MasksDir, "Mask");
            PdfContentExtractor.ExportToFile(new SizeI(this.Width.Value, this.Height.Value), filePath, (x, y) =>
            {
                bool isMasked = Model.Resources.ImageMask.IsMasked(this.bitReader, this.unmaskedColor, y, x);
                return isMasked ? Color.Black : Color.Transparent;
            });
#endif

            return Model.Resources.ImageMask.MaskImage(this, image, pixels);
        }

        public bool ShouldMaskColorComponents(int[] components)
        {
            return false;
        }

        public override void Load(PdfDictionaryOld dictionary)
        {
            base.Load(dictionary);
#if DEBUG
            this.bitReader = new ImageBitReader(this.decodedMaskData, 1, this.Width.Value);
            this.unmaskedColor = Model.Resources.ImageMask.GetUnmaskedColor(this);
#endif
        }

        protected override byte[] ReadData(bool shouldDecode)
        {
            if (shouldDecode)
            {
                return this.decodedMaskData;
            }
            else
            {
                return base.ReadData(shouldDecode);
            }
        }
    }
}