﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects
{
    internal class InlineImageInfo : IImageDescriptor
    {
        private readonly IImageDescriptor imageDescriptor;
        private readonly XImage xImage;
        private readonly byte[] rawData;
        private readonly long endOfImagePosition;
        private readonly PdfDictionaryOld dictionary;

        public InlineImageInfo(XImage xImage, PdfDictionaryOld dictionary, byte[] rawData, long endOfImagePosition)
        {
            this.imageDescriptor = xImage;
            this.xImage = xImage;
            this.dictionary = dictionary;
            this.rawData = rawData;
            this.endOfImagePosition = endOfImagePosition;
        }

        public XImage XImage
        {
            get
            {
                return this.xImage;
            }
        }

        public PdfDictionaryOld Dictionary
        {
            get
            {
                return this.dictionary;
            }
        }

        public byte[] RawData
        {
            get
            {
                return this.rawData;
            }
        }

        public long EndOfImagePosition
        {
            get
            {
                return this.endOfImagePosition;
            }
        }

        int IImageDescriptor.Width
        {
            get
            {
                return this.imageDescriptor.Width;
            }
        }

        int IImageDescriptor.Height
        {
            get
            {
                return this.imageDescriptor.Height;
            }
        }

        int IImageDescriptor.BitsPerComponent
        {
            get
            {
                return this.imageDescriptor.BitsPerComponent;
            }
        }

        double[] IImageDescriptor.Decode
        {
            get
            {
                return this.imageDescriptor.Decode;
            }
        }

        IColorSpace IImageDescriptor.ColorSpace
        {
            get
            {
                return this.imageDescriptor.ColorSpace;
            }
        }

        IMask IImageDescriptor.Mask
        {
            get
            {
                return this.imageDescriptor.Mask;
            }
        }

        IImageDescriptor IImageDescriptor.SMask
        {
            get
            {
                return this.imageDescriptor.SMask;
            }
        }

        string[] IImageDescriptor.Filters
        {
            get
            {
                string[] filters = FiltersManagerOld.GetFilterValues(this.XImage.ContentManager, this.Dictionary);

                return filters;
            }
        }

        byte[] IImageDescriptor.GetDecodedData()
        {
            byte[] data = this.RawData;

            if (this.Dictionary.ContainsKey(PdfNames.Filter))
            {
                PdfNameOld[] filters = FiltersManagerOld.GetFilters(this.XImage.ContentManager, this.Dictionary);
                DecodeParameters[] parms = FiltersManagerOld.GetDecodeParameters(this.XImage.ContentManager, this.Dictionary);
                data = FiltersManagerOld.Decode(dictionary, filters, data, parms);
            }

            return data;
        }

        byte[] IImageDescriptor.GetEncodedData()
        {
            return this.RawData;
        }
    }
}
