﻿using System;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.Model.Extensions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects
{
    [PdfClass(TypeName = "XObject", SubtypeProperty = "Subtype", SubtypeValue = "Image")]
    internal class XImage : XObject, IImageDescriptor
    {
        private readonly InstantLoadProperty<PdfIntOld> width;
        private readonly InstantLoadProperty<PdfIntOld> height;
        private readonly InstantLoadProperty<ColorSpaceOld> colorSpace;
        private readonly InstantLoadProperty<PdfIntOld> bitsPerComponent;
        private readonly InstantLoadProperty<PdfBoolOld> imageMask;
        private readonly InstantLoadProperty<XImage> softMask;
        private readonly InstantLoadProperty<PdfArrayOld> decode;
        private readonly InstantLoadProperty<PdfObjectOld> mask;

        public XImage(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.width = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Width", IsRequired = true }, Converters.PdfIntConverter);
            this.height = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "Height", IsRequired = true }, Converters.PdfIntConverter);
            this.colorSpace = this.CreateInstantLoadProperty<ColorSpaceOld>(new PdfPropertyDescriptor { Name = "ColorSpace" }, Converters.ColorSpaceConverter);
            this.bitsPerComponent = this.CreateInstantLoadProperty<PdfIntOld>(new PdfPropertyDescriptor { Name = "BitsPerComponent" }, Converters.PdfIntConverter);
            this.imageMask = this.CreateInstantLoadProperty<PdfBoolOld>(new PdfPropertyDescriptor { Name = "ImageMask" }, new PdfBoolOld(contentManager, false), Converters.PdfBoolConverter);
            this.softMask = this.CreateInstantLoadProperty<XImage>(new PdfPropertyDescriptor { Name = "SMask" });
            this.decode = this.CreateInstantLoadProperty<PdfArrayOld>(new PdfPropertyDescriptor { Name = "Decode" });
            this.mask = this.CreateInstantLoadProperty<PdfObjectOld>(new PdfPropertyDescriptor { Name = "Mask" }, Converters.MaskConverter);
        }

        public PdfIntOld Width
        {
            get
            {
                return this.width.GetValue();
            }
            set
            {
                this.width.SetValue(value);
            }
        }

        public PdfIntOld Height
        {
            get
            {
                return this.height.GetValue();
            }
            set
            {
                this.height.SetValue(value);
            }
        }

        public ColorSpaceOld ColorSpace
        {
            get
            {
                return this.colorSpace.GetValue();
            }
            set
            {
                this.colorSpace.SetValue(value);
            }
        }

        public PdfIntOld BitsPerComponent
        {
            get
            {
                return this.bitsPerComponent.GetValue();
            }
            set
            {
                this.bitsPerComponent.SetValue(value);
            }
        }

        public PdfBoolOld ImageMask
        {
            get
            {
                return this.imageMask.GetValue();
            }
            set
            {
                this.imageMask.SetValue(value);
            }
        }

        public PdfArrayOld Decode
        {
            get
            {
                return this.decode.GetValue();
            }
            set
            {
                this.decode.SetValue(value);
            }
        }

        public XImage SMask
        {
            get
            {
                return this.softMask.GetValue();
            }
            set
            {
                this.softMask.SetValue(value);
            }
        }

        public IMask Mask
        {
            get
            {
                return (IMask)this.mask.GetValue();
            }
        }

        int IImageDescriptor.Width
        {
            get
            {
                return this.Width.Value;
            }
        }

        int IImageDescriptor.Height
        {
            get
            {
                return this.Height.Value;
            }
        }

        int IImageDescriptor.BitsPerComponent
        {
            get
            {
                return this.BitsPerComponent.Value;
            }
        }

        double[] IImageDescriptor.Decode
        {
            get
            {
                if (this.Decode == null)
                {
                    return null;
                }

                double[] array = this.Decode.ToDoubleArray();

                return array;
            }
        }

        IColorSpace IImageDescriptor.ColorSpace
        {
            get
            {
                return this.ColorSpace;
            }
        }

        IImageDescriptor IImageDescriptor.SMask
        {
            get
            {
                return this.SMask;
            }
        }

        IMask IImageDescriptor.Mask
        {
            get
            {
                return this.Mask;
            }
        }

        string[] IImageDescriptor.Filters
        {
            get
            {
                string[] filters = null;

                IndirectObjectOld obj;
                if (this.Reference != null && this.ContentManager.TryReadIndirectObject(this.Reference, true, out obj))
                {
                    PdfDataStream dataStream = (PdfDataStream)obj.Value;
                    PdfDictionaryOld imageDictionary = dataStream.Dictionary;
                    filters = FiltersManagerOld.GetFilterValues(this.ContentManager, imageDictionary);
                }

                return filters;
            }
        }

        byte[] IImageDescriptor.GetDecodedData()
        {
            return this.ReadData(true);
        }

        byte[] IImageDescriptor.GetEncodedData()
        {
            return this.ReadData(false);
        }

        public override void Load(PdfDataStream stream)
        {
            this.Load(stream.Dictionary);
            this.IsLoaded = true;
        }

        public ImageDataSource CreateImageSource(IImageDescriptor image)
        {
            if (this.ImageMask.Value)
            {
                throw new NotSupportedException();
            }

            ImageDataSource result = image.ToImageDataSource();
#if DEBUG
            if (result.ImageDataType == ImageDataType.Decoded)
            {
                PixelContainer initialPixels = image.ColorSpace.GetPixels(image, false);
                string filePath = PdfContentExtractor.GetFileName(this, PdfContentExtractor.ImagesDir, "Image");
                PdfContentExtractor.ExportToFile(image, filePath, false);

                if (image.SMask != null)
                {
                    PixelContainer sMaskPixels = image.SMask.ColorSpace.GetPixels(image.SMask, false);
                    string maskFilePath = PdfContentExtractor.GetFileName(this.SMask, PdfContentExtractor.SMaskDir, "SMask");
                    PdfContentExtractor.ExportToFile(image.SMask, maskFilePath, false);
                }

                filePath = PdfContentExtractor.GetFileName(this, PdfContentExtractor.MaskResultImages, "MaskResultImage");
                PdfContentExtractor.ExportToFile((DecodedImageDataSource)result, filePath);
            }
            else
            {
                PdfContentExtractor.ExportJpegImage(((ImageDataSource<byte[]>)result).Data, "JpegImageDataSource");
            }
#endif
            return result;
        }

        public DecodedImageDataSource CreateImageSourceWithStencilColor(IImageDescriptor image, Color stencilColor)
        {
            if (!this.ImageMask.Value)
            {
                throw new NotSupportedException();
            }

            DecodedImageDataSource imageSource = image.ToDecodedImageDataSource(stencilColor);
#if DEBUG
            string filePath = PdfContentExtractor.GetFileName(this, PdfContentExtractor.ImagesDir, string.Format("StencilImage{0}", stencilColor));
            PdfContentExtractor.ExportToFile(imageSource, filePath);
#endif

            return imageSource;
        }

        protected virtual byte[] ReadData(bool shouldDecode)
        {
            byte[] data = null;

            if (this.Reference != null)
            {
                data = this.ContentManager.ReadData(this.Reference, shouldDecode);
            }

            return data;
        }
    }
}
