﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data
{
    internal class PdfArrayOld : PdfCollectionBaseOld<int>, IEnumerable<object>
    {
        private readonly List<object> store;

        public PdfArrayOld(PdfContentManager contentManager)
            : base(contentManager)
        {
            this.store = new List<object>();
        }

        public PdfArrayOld(PdfContentManager contentManager, IEnumerable<object> range)
            : base(contentManager)
        {
            this.store = new List<object>(range);

            this.IsLoaded = true;
        }

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public object this[int index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        public static PdfArrayOld CreateMatrixIdentity(PdfContentManager contentManager)
        {
            return new PdfArrayOld(contentManager, new object[] { 1, 0, 0, 1, 0, 0 });
        }

        public void Load(IEnumerable<object> content)
        {
            this.store.Clear();
            this.store.AddRange(content);

            this.IsLoaded = true;
        }

        public void Add(object obj)
        {
            this.store.Add(obj);
        }

        public IEnumerator<object> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        public override void Load(IndirectObjectOld indirectObject)
        {
            Guard.ThrowExceptionIfNull(indirectObject, "indirectObject");

            PdfArrayOld array = indirectObject.Value as PdfArrayOld;
            if (array != null)
            {
                this.Load(array.store);
            }

            base.Load(indirectObject);
        }

        public Rect ToRect()
        {
            double x1, y1, x2, y2;
            Helper.UnboxDouble(this.GetDirectValue(0), out x1);
            Helper.UnboxDouble(this.GetDirectValue(1), out y1);
            Helper.UnboxDouble(this.GetDirectValue(2), out x2);
            Helper.UnboxDouble(this.GetDirectValue(3), out y2);

            return new Rect(new Point(x1, y1), new Point(x2, y2));
        }

        public Matrix ToMatrix()
        {
            double m11, m12, m21, m22, offsetX, offsetY;
            Helper.UnboxDouble(this.store[0], out m11);
            Helper.UnboxDouble(this.store[1], out m12);
            Helper.UnboxDouble(this.store[2], out m21);
            Helper.UnboxDouble(this.store[3], out m22);
            Helper.UnboxDouble(this.store[4], out offsetX);
            Helper.UnboxDouble(this.store[5], out offsetY);

            return new Matrix(m11, m12, m21, m22, offsetX, offsetY);
        }

        public double[] ToDashPattern()
        {
            double[] pattern = new double[this.Count];
            for (int i = 0; i < this.Count; i++)
            {
                this.TryGetReal(i, out pattern[i]);
            }

            return pattern;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            foreach (object o in this.store)
            {
                sb.Append(" ");
                sb.Append(o);
            }

            sb.Remove(0, 1);
            sb.Append(Characters.ArrayEnd);
            sb.Insert(0, Characters.ArrayStart.ToString());
            return sb.ToString();
        }

        public object[] ToArray()
        {
            return this.store.ToArray();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        protected override object GetElementAtIndex(int index)
        {
            return this.store[index];
        }

        protected override bool ContainsElementAtIndex(int index)
        {
            return index >= 0 && index < this.store.Count;
        }

        private object GetDirectValue(int index)
        {
            IndirectReferenceOld indirReference = this.store[index] as IndirectReferenceOld;
            if (indirReference != null)
            {
                IndirectObjectOld indirectObjectResult;
                if (this.ContentManager.TryGetIndirectObject(indirReference, false, out indirectObjectResult))
                {
                    return indirectObjectResult.Value;
                }

                return null;
            }

            return this.store[index];
        }
    }
}
