﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data
{
    internal abstract class PdfCollectionBaseOld<TIndex> : PdfObjectOld
    {
        public PdfCollectionBaseOld(PdfContentManager contentManager)
            : base(contentManager)
        {
        }

        public T GetElement<T>(TIndex index, IConverter converter) where T : PdfObjectOld
        {
            object obj = this.GetElementAtIndex(index);

            if (obj == null)
            {
                return null;
            }

            T result = obj as T;
            if (result != null)
            {
                return result;
            }

            IndirectReferenceOld reference = obj as IndirectReferenceOld;
            if (reference != null && !converter.HandlesIndirectReference)
            {
                result = Converters.IndirectReferenceToPdfObjectConverter.Convert<T>(this.ContentManager, reference);
                result.Load();
            }
            else
            {
                result = (T)converter.Convert(typeof(T), this.ContentManager, obj);
            }

            return result;
        }

        public T GetElement<T>(TIndex index) where T : PdfObjectOld
        {
            object obj = this.GetElementAtIndex(index);

            if (obj == null)
            {
                return null;
            }

            T result = obj as T;
            if (result != null)
            {
                return result;
            }

            IndirectReferenceOld reference = obj as IndirectReferenceOld;
            if (reference != null)
            {
                result = Converters.IndirectReferenceToPdfObjectConverter.Convert<T>(this.ContentManager, reference);
                result.Load();
            }
            else
            {
                PdfDictionaryOld dictionary = obj as PdfDictionaryOld;
                if (dictionary != null)
                {
                    result = (T)Converters.PdfDictionaryToPdfObjectConverter.Convert(typeof(T), this.ContentManager, dictionary);
                }
                else
                {
                    result = obj as T;
                    if (result != null)
                    {
                        result.Load();
                    }
                }
            }

            return result;
        }

        public bool TryGetBool(TIndex index, out bool res)
        {
            res = default(bool);
            if (this.ContainsElementAtIndex(index))
            {
                PdfBoolOld b = this.GetElement<PdfBoolOld>(index, Converters.PdfBoolConverter);

                if (b != null)
                {
                    res = b.Value;

                    return true;
                }
            }

            return false;
        }

        public bool TryGetInt(TIndex index, out int res)
        {
            res = default(int);
            if (this.ContainsElementAtIndex(index))
            {
                PdfIntOld b = this.GetElement<PdfIntOld>(index, Converters.PdfIntConverter);

                if (b != null)
                {
                    res = b.Value;

                    return true;
                }
            }

            return false;

        }

        public bool TryGetReal(TIndex index, out double res)
        {
            res = default(double);
            if (this.ContainsElementAtIndex(index))
            {
                PdfRealOld b = this.GetElement<PdfRealOld>(index, Converters.PdfRealConverter);

                if (b != null)
                {
                    res = b.Value;

                    return true;
                }
            }

            return false;
        }

        protected abstract object GetElementAtIndex(TIndex index);

        protected abstract bool ContainsElementAtIndex(TIndex index);
    }
}
