﻿using System.IO;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data
{
	internal class PdfDataStream
	{
        private readonly IndirectObjectOld containingObject;
        private readonly PdfDictionaryOld dictionary;
        private readonly PdfReader.Readers.PdfReader reader;
       
        private long position;
        private int length;

        public PdfDataStream(PdfReader.Readers.PdfReader reader, PdfDictionaryOld dictionary)
            : this(reader, dictionary, null)
        {
        }

        public PdfDataStream(PdfReader.Readers.PdfReader reader, PdfDictionaryOld dictionary, IndirectObjectOld containingObject)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");

            this.reader = reader;
            this.dictionary = dictionary;
            this.containingObject = containingObject;
        }

		public PdfDictionaryOld Dictionary
		{
            get 
            { 
                return this.dictionary; 
            }
		}  

        public bool HasFilter
        {
            get
            {
                return this.Dictionary != null && this.Dictionary.ContainsKey(PdfNames.Filter);
            }
        }

        public void FixLengthAndPosition(long position)
        {
            lock (this.reader)
            {
                int length;
                this.reader.Seek(position, SeekOrigin.Begin);
                this.Dictionary.TryGetInt(PdfNames.Length, out length);
                this.reader.Seek(position, SeekOrigin.Begin);
                this.reader.GoToNextLine();

                this.position = reader.Position;
                this.length = this.ReadRawData(this.position, length).Length;

                reader.ReadToken();
            }
        }

        public byte[] ReadData(PdfContentManager contentManager)
        {
            byte[] data = this.ReadDataWithoutDecoding(contentManager);

            return this.DecodeData(contentManager, data);
        }

        public byte[] ReadDataWithoutDecoding(PdfContentManager contentManager)
        {
            byte[] data = this.ReadRawData(this.position, this.length);
            if (this.containingObject != null)
            {
                data = DecryptData(data, this.containingObject, contentManager);
            }

            return data;
        }

        private static byte[] DecryptData(byte[] data, IndirectObjectOld containingObject, PdfContentManager contentManager)
        {
            return contentManager.DecryptStream(containingObject, data);
        }

        private byte[] DecodeData(PdfContentManager contentManager, byte[] data)
        {
            if (!this.HasFilter)
            {
                return data;
            }
            else
            {
                PdfNameOld[] filters = FiltersManagerOld.GetFilters(contentManager, this.Dictionary);
                FormatProviders.Pdf.Filters.DecodeParameters[] parms = FiltersManagerOld.GetDecodeParameters(contentManager, this.Dictionary);
                return FiltersManagerOld.Decode(this.Dictionary, filters, data, parms);
            }
        }  

        private byte[] ReadRawData(long offset, int length)
        {
            StreamPart part = null;
            try
            {
                this.reader.Seek(offset + length, SeekOrigin.Begin);
                if (this.reader.ReadToken() == Token.StreamEnd)
                {
                    part = new StreamPart(this.reader.Stream, offset, length);
                }
                else
                {
                    part = new StreamPart(this.reader.Stream, offset, Keywords.StreamEnd);
                }

                return this.reader.ReadRawData(offset, part);
            }
            finally
            {
                if (part != null)
                {
                    part.Dispose();
                }
            }
        }
	}
}
