﻿using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using System;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data
{
    internal class PdfObjectOld : IPdfSharedObject
    {
        private readonly Dictionary<string, IPdfProperty> properties;
        private readonly PdfContentManager contentManager;

        public PdfObjectOld(PdfContentManager contentManager)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");

            this.contentManager = contentManager;
            this.properties = new Dictionary<string, IPdfProperty>();
        }

        public PdfContentManager ContentManager
        {
            get
            {
                return this.contentManager;
            }
        }

        public IndirectReferenceOld Reference
        {
            get;
            set;
        }

        public bool IsOldSchema
        {
            get
            {
                return true;
            }
        }

        public bool IsLoaded
        {
            get;
            protected set;
        }

        public void Load()
        {
            this.ContentManager.LoadIndirectObject(this);
        }

        public virtual void Load(PdfDictionaryOld dictionary)
        {
            IPdfProperty prop;
            foreach (var pair in dictionary)
            {
                if (this.properties.TryGetValue(pair.Key, out prop))
                {
                    prop.SetValue(pair.Value);
                }
            }

            this.IsLoaded = true;
        }

        public virtual void Load(Data.IndirectObjectOld indirectObject)
        {
            Guard.ThrowExceptionIfNull(indirectObject, "indirectObject");

            if (this.Reference == null)
            {
                this.Reference = new IndirectReferenceOld { GenerationNumber = indirectObject.GenerationNumber, ObjectNumber = indirectObject.ObjectNumber };
            }

            if (indirectObject.Value is PdfDictionaryOld)
            {
                this.Load(indirectObject.Value as PdfDictionaryOld);
            }
        }

        public PdfDictionaryOld ToPdfDictionary()
        {
            PdfDictionaryOld dict = new PdfDictionaryOld(this.ContentManager);
            PropertyInfo[] props = this.GetType().GetProperties();
            foreach (PropertyInfo prop in props)
            {
                // TODO: Handle when attribute is set more than once!
                PdfPropertyAttribute attr = Enumerable.FirstOrDefault(prop.GetCustomAttributes(typeof(PdfPropertyAttribute), false)) as PdfPropertyAttribute;
                if (attr == null)
                {
                    continue;
                }

                PdfObjectOld value = prop.GetValue(this, null) as PdfObjectOld;
                if (value == null)
                {
                    continue;
                }

                string name = string.IsNullOrEmpty(attr.Name) ? prop.Name : attr.Name;
                dict[name] = value;
            }

            return dict;
        }

        public override bool Equals(object obj)
        {
            PdfObjectOld other = obj as PdfObjectOld;

            if (other == null)
            {
                return false;
            }

            return this.Reference == other.Reference;
        }

        public override int GetHashCode()
        {
            if (this.Reference != null)
            {
                return this.Reference.GetHashCode();
            }

            return base.GetHashCode();
        }

        protected InstantLoadProperty<T> CreateInstantLoadProperty<T>(PdfPropertyDescriptor descriptor) where T : PdfObjectOld
        {
            InstantLoadProperty<T> res = new InstantLoadProperty<T>(this.ContentManager, descriptor);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected InstantLoadProperty<T> CreateInstantLoadProperty<T>(PdfPropertyDescriptor descriptor, T initialValue) where T : PdfObjectOld
        {
            InstantLoadProperty<T> res = new InstantLoadProperty<T>(this.ContentManager, descriptor, initialValue);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected InstantLoadProperty<T> CreateInstantLoadProperty<T>(PdfPropertyDescriptor descriptor, IConverter converter) where T : PdfObjectOld
        {
            InstantLoadProperty<T> res = new InstantLoadProperty<T>(this.ContentManager, descriptor, converter);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected InstantLoadProperty<T> CreateInstantLoadProperty<T>(PdfPropertyDescriptor descriptor, T initialValue, IConverter converter) where T : PdfObjectOld
        {
            InstantLoadProperty<T> res = new InstantLoadProperty<T>(this.ContentManager, descriptor, initialValue, converter);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected LoadOnDemandProperty<T> CreateLoadOnDemandProperty<T>(PdfPropertyDescriptor descriptor) where T : PdfObjectOld
        {
            LoadOnDemandProperty<T> res = new LoadOnDemandProperty<T>(this.ContentManager, descriptor);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected LoadOnDemandProperty<T> CreateLoadOnDemandProperty<T>(PdfPropertyDescriptor descriptor, T initialValue) where T : PdfObjectOld
        {
            LoadOnDemandProperty<T> res = new LoadOnDemandProperty<T>(this.ContentManager, descriptor, initialValue);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected LoadOnDemandProperty<T> CreateLoadOnDemandProperty<T>(PdfPropertyDescriptor descriptor, IConverter converter) where T : PdfObjectOld
        {
            LoadOnDemandProperty<T> res = new LoadOnDemandProperty<T>(this.ContentManager, descriptor, converter);
            this.RegisterPdfProperty(res);

            return res;
        }

        protected LoadOnDemandProperty<T> CreateLoadOnDemandProperty<T>(PdfPropertyDescriptor descriptor, T initialValue, IConverter converter) where T : PdfObjectOld
        {
            LoadOnDemandProperty<T> res = new LoadOnDemandProperty<T>(this.ContentManager, descriptor, initialValue, converter);
            this.RegisterPdfProperty(res);

            return res;
        }

        private void RegisterPdfProperty(string name, IPdfProperty property)
        {
            this.properties[name] = property;
        }

        private void RegisterPdfProperty(IPdfProperty property)
        {
            if (property.Descriptor.IsMultipleProperties)
            {
                string[] names = property.Descriptor.Name.Split('|');
                foreach (string name in names)
                {
                    this.RegisterPdfProperty(name, property);
                }
            }
            else
            {
                this.RegisterPdfProperty(property.Descriptor.Name, property);
            }
        }
    }
}
