﻿using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data
{
    internal class StreamPart : Stream
    {
        private const int BufferSize = 1024;

        private readonly Stream baseStream;
        private readonly long offset, length;
        private readonly byte[] buffer;

        public StreamPart(Stream baseStream, long offset, string endToken)
        {
            Guard.ThrowExceptionIfLessThan(0, offset, "offset");

            this.baseStream = baseStream;
            this.offset = offset;
            this.buffer = new byte[BufferSize];
            this.length = this.FindTokenOffset(endToken);
            this.baseStream.Seek(this.offset, SeekOrigin.Begin);
        }

        public StreamPart(Stream baseStream, long offset, int length)
        {
            if (offset < 0 || offset + length > baseStream.Length)
            {
                throw new ArgumentException();
            }
            this.baseStream = baseStream;
            this.offset = offset;
            this.length = length;
            this.baseStream.Seek(this.offset, SeekOrigin.Begin);
        }

        public override bool CanRead
        {
            get { return this.baseStream.CanRead; }
        }

        public override bool CanSeek
        {
            get { return this.baseStream.CanSeek; }
        }

        public override bool CanWrite
        {
            get { return this.baseStream.CanWrite; }
        }

        public override long Length
        {
            get { return this.length; }
        }

        public override long Position
        {
            get
            {
                return this.baseStream.Position - this.offset;
            }
            set
            {
                this.baseStream.Position = value + this.offset;
            }
        }

        public override void Flush()
        {
            this.baseStream.Flush();
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            int bytesToRead = (int)Math.Min(count, this.Length - this.Position);
            if (bytesToRead == 0)
            {
                return 0;
            }

            return this.baseStream.Read(buffer, offset, bytesToRead);
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            switch (origin)
            {
                case SeekOrigin.Begin:
                    Guard.ThrowExceptionIfOutOfRange(0, this.offset + this.length, offset, "offset");
            
                    return baseStream.Seek(this.offset + offset, SeekOrigin.Begin) - this.offset;

                default:
                    throw new NotSupportedException();
            }
        }

        public override void SetLength(long value)
        {
            throw new NotSupportedException();
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            throw new NotSupportedException();
        }

        private long FindTokenOffset(string token)
        {
            this.baseStream.Seek(this.offset, SeekOrigin.Begin);
            StringBuilder tmp = new StringBuilder();
            StringBuilder tok = new StringBuilder(token);
            int len;
            long pos = this.offset;
            bool hasMach = false;
            while (pos < this.baseStream.Length)
            {
                len = this.baseStream.Read(this.buffer, 0, BufferSize);
                for (int i = 0; i < len; i++)
                {
                    if (hasMach && Characters.IsDelimiter(this.buffer[i]))
                    {
                        return pos - offset - tmp.Length;
                    }

                    tmp.Append((char)this.buffer[i]);
                    pos++;
                    if (tok.Length < tmp.Length)
                    {
                        tmp.Remove(0, 1);
                    }
                  
                    hasMach = tmp.Equals(tok);
                }
            }

            return -1;
        }
    }
}
