﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Annot;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal class AnnotConverter : IndirectReferenceConverterBase
    {
        private readonly Dictionary<string, AnnotationType> nameToAnnotationType;

        public AnnotConverter()
        {
            this.nameToAnnotationType = new Dictionary<string, AnnotationType>();

            this.nameToAnnotationType[AnnotationTypes.Text] = AnnotationType.Text;
            this.nameToAnnotationType[AnnotationTypes.FreeText] = AnnotationType.FreeText; 
            this.nameToAnnotationType[AnnotationTypes.Line] = AnnotationType.Line; 
            this.nameToAnnotationType[AnnotationTypes.Square] = AnnotationType.Square; 
            this.nameToAnnotationType[AnnotationTypes.Circle] = AnnotationType.Circle; 
            this.nameToAnnotationType[AnnotationTypes.Polygon] = AnnotationType.Polygon; 
            this.nameToAnnotationType[AnnotationTypes.PolyLine] = AnnotationType.PolyLine; 
            this.nameToAnnotationType[AnnotationTypes.Highlight] = AnnotationType.Highlight; 
            this.nameToAnnotationType[AnnotationTypes.Underline] = AnnotationType.Underline; 
            this.nameToAnnotationType[AnnotationTypes.Squiggly] = AnnotationType.Squiggly; 
            this.nameToAnnotationType[AnnotationTypes.StrikeOut] = AnnotationType.StrikeOut; 
            this.nameToAnnotationType[AnnotationTypes.Stamp] = AnnotationType.Stamp; 
            this.nameToAnnotationType[AnnotationTypes.Caret] = AnnotationType.Caret; 
            this.nameToAnnotationType[AnnotationTypes.Ink] = AnnotationType.Ink; 
            this.nameToAnnotationType[AnnotationTypes.Popup] = AnnotationType.Popup; 
            this.nameToAnnotationType[AnnotationTypes.FileAttachment] = AnnotationType.FileAttachment; 
            this.nameToAnnotationType[AnnotationTypes.Sound] = AnnotationType.Sound; 
            this.nameToAnnotationType[AnnotationTypes.Movie] = AnnotationType.Movie; 
            this.nameToAnnotationType[AnnotationTypes.Screen] = AnnotationType.Screen; 
            this.nameToAnnotationType[AnnotationTypes.PrinterMark] = AnnotationType.PrinterMark; 
            this.nameToAnnotationType[AnnotationTypes.TrapNet] = AnnotationType.TrapNet; 
            this.nameToAnnotationType[AnnotationTypes.Watermark] = AnnotationType.Watermark; 
            this.nameToAnnotationType[AnnotationTypes.ThreeD] = AnnotationType.ThreeD; 
            this.nameToAnnotationType[AnnotationTypes.Redact] = AnnotationType.Redact; 
        }

        protected override object ConvertFromPdfDictionary(Type type, PdfReader.PdfContentManager contentManager, PdfDictionaryOld dictionary)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            AnnotationOld res = null;
            PdfNameOld annotType = dictionary.GetElement<PdfNameOld>(PdfNames.Subtype);

            if (annotType != null)
            {
                switch (annotType.Value)
                {
                    case AnnotationTypes.Link:
                        res = new LinkOld(contentManager);
                        break;
                    case AnnotationTypes.Widget:
                        res = new WidgetOld(contentManager);
                        break;
                    default:
                        AnnotationType annotationType;
                        if (this.nameToAnnotationType.TryGetValue(annotType.Value, out annotationType))
                        {
                            res = new UnsupportedAnnotationOld(contentManager, annotationType);
                        }
                        break;
                }
            }

            if (res != null)
            {
                res.Load(dictionary);
            }

            return res;
        }
    }
}
