﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal abstract class ConverterBase : IConverter
	{
        public virtual bool HandlesIndirectReference
        {
            get
            {
                return false;
            }
        }

        public virtual object Convert(Type type, PdfContentManager contentManager, object value)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");

            PdfDictionaryOld dictionary = value as PdfDictionaryOld;
            if (dictionary != null)
            {
                return this.ConvertFromPdfDictionary(type, contentManager, dictionary);
            }

            PdfNameOld name = value as PdfNameOld;
            if (name != null)
            {
                return this.ConvertFromPdfName(type, contentManager, name);
            }

            PdfDataStream stm = value as PdfDataStream;
            if (stm != null)
            {
                return this.ConvertFromPdfDataStream(type, contentManager, stm);
            }

            PdfArrayOld array = value as PdfArrayOld;
            if (array != null)
            {
                return this.ConvertFromPdfArray(type, contentManager, array);
            }

            PdfStringOld str = value as PdfStringOld;
            if (str != null)
            {
                return this.ConvertFromPdfString(type, contentManager, str);
            }

            return value;
        }

        protected virtual object ConvertFromPdfDictionary(Type type, PdfContentManager contentManager, PdfDictionaryOld dictionary)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            PdfObjectOld res = Activator.CreateInstance(type, contentManager) as PdfObjectOld;
            if (res != null)
            {
                res.Load(dictionary);
               
                return res;
            }

            return dictionary;
        }

        protected virtual object ConvertFromPdfDataStream(Type type, PdfContentManager contentManager, PdfDataStream stream)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(stream, "stream");

            PdfStreamOld res = Activator.CreateInstance(type, contentManager) as PdfStreamOld;
            if (res != null)
            {
                res.Load(stream);
                return res;
            }

            return stream;
        }

        protected virtual object ConvertFromPdfName(Type type, PdfContentManager contentManager, PdfNameOld name)
        {
            return name;
        }

        protected virtual object ConvertFromPdfString(Type type, PdfContentManager contentManager, PdfStringOld str)
        {
            return str;
        }

        protected virtual object ConvertFromPdfArray(Type type, PdfContentManager contentManager, PdfArrayOld array)
        {
            return array;
        }
	}
}
