﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal class DecodeParametersConverter : IndirectReferenceConverterBase
	{
        protected override object ConvertFromPdfDictionary(Type type, PdfContentManager contentManager, Data.PdfDictionaryOld dictionary)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            return new DecodeParameters[] { CreateDecodeParameters(contentManager, dictionary) };
        }

        protected override object ConvertFromPdfArray(Type type, PdfContentManager contentManager, Data.PdfArrayOld array)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(array, "array");

            DecodeParameters[] res = new DecodeParameters[array.Count];
            for (int i = 0; i < array.Count; i++)
            {
                PdfDictionaryOld dictionary = array.GetElement<Data.PdfDictionaryOld>(i);
                if (dictionary != null)
                {
                    res[i] = CreateDecodeParameters(contentManager, dictionary);
                }
            }

            return res;
        }

        private static DecodeParameters CreateDecodeParameters(PdfContentManager contentManager, Data.PdfDictionaryOld dictionary)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            DecodeParameters res = new DecodeParameters();

            foreach (var key in dictionary.Keys)
            {
                res[key] = StripPdfObject(contentManager, dictionary[key]);
            }

            return res;
        }

        private static object StripPdfObject(PdfContentManager contentManager, object obj)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");


            IndirectReferenceOld reference = obj as IndirectReferenceOld;
            if (reference != null)
            {
                IndirectObjectOld indirectObject;
                if (contentManager.TryGetIndirectObject(reference, out indirectObject))
                {
                    return StripPdfObject(contentManager, indirectObject.Value);
                }
            }

            Data.PdfObjectOld pdfObject = obj as Data.PdfObjectOld;
            if (pdfObject != null)
            {
                pdfObject.Load();

                Data.PdfSimpleTypeOld simpleType = pdfObject as Data.PdfSimpleTypeOld;
                if (simpleType != null)
                {
                    return simpleType.GetValue();
                }

                Data.PdfArrayOld array = pdfObject as Data.PdfArrayOld;
                if (array != null)
                {
                    return StripPdfArray(contentManager, array);
                }

                Data.PdfDictionaryOld dictionary = pdfObject as Data.PdfDictionaryOld;
                if (dictionary != null)
                {
                    return StripPdfDictionary(contentManager, dictionary);
                }
            }

            Data.PdfDataStream stream = obj as Data.PdfDataStream;
            if (stream != null)
            {
                return StripPdfDataStream(contentManager, stream);
            }

            return obj;
        }

        private static object[] StripPdfArray(PdfContentManager contentManager, Data.PdfArrayOld array)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(array, "array");

            object[] res = new object[array.Count];
            for (int i = 0; i < array.Count; i++)
            {
                res[i] = StripPdfObject(contentManager, array[i]);
            }

            return res;
        }

        private static Dictionary<string, object> StripPdfDictionary(PdfContentManager contentManager, Data.PdfDictionaryOld dictionary)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            Dictionary<string, object> res = new Dictionary<string, object>();
            foreach (var key in dictionary.Keys)
            {
                res[key] = StripPdfObject(contentManager, dictionary[key]);
            }

            return res;
        }

        private static byte[] StripPdfDataStream(PdfContentManager contentManager, Data.PdfDataStream stream)
        {
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(stream, "stream");

            return stream.ReadData(contentManager);
        }

    }
}
