﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Annot;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal class FieldsConverterOld : IndirectReferenceConverterBase
    {
        protected override object ConvertFromPdfArray(Type type, PdfContentManager contentManager, PdfArrayOld array)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(array, "array");

            FieldsConvertionContext convertionContext = new FieldsConvertionContext(new FormFieldsCollectionOld(contentManager), array, null);

            this.LoadFormFields(convertionContext);

            return convertionContext.FieldsTree;
        }

        private void LoadFormFields(FieldsConvertionContext context)
        {
            bool addParentAsTerminalNode = false;

            foreach (object child in context.Kids)
            {
                IndirectReferenceOld childReference = child as IndirectReferenceOld;

                if (childReference != null)
                {
                    IndirectObjectOld obj;
                    if (context.ContentManager.TryGetIndirectObject(childReference, out obj))
                    {
                        PdfDictionaryOld dictionary = obj.Value as PdfDictionaryOld;

                        if (dictionary != null)
                        {
                            if(FieldsConverterOld.ContainsWidgetProperties(dictionary))
                            {
                                WidgetOld widget = new WidgetOld(context.ContentManager);
                                widget.Load(obj);
                                context.ContentManager.RegisterIndirectReference(childReference, widget);

                                if(FieldsConverterOld.ContainsMergedFieldProperties(dictionary))
                                {
                                    FormFieldNodeOld node = new FormFieldNodeOld(context.ContentManager);
                                    node.Load(obj);
                                    context.FieldsTree.Add(node);
                                    context.ContentManager.AddWidgetParent(widget, node);
                                }
                                else
                                {
                                    context.ContentManager.AddWidgetParent(widget, context.Parent);
                                    addParentAsTerminalNode = true;
                                }
                            }
                            else
                            {
                                FormFieldNodeOld node = new FormFieldNodeOld(context.ContentManager);
                                node.Load(obj);
                                context.ContentManager.RegisterIndirectReference(childReference, node);

                                if (node.Kids == null || node.Kids.Count == 0)
                                {
                                    context.FieldsTree.Add(node);
                                }
                                else
                                {
                                    this.LoadFormFields(new FieldsConvertionContext(context.FieldsTree, node.Kids, node));
                                }
                            }
                        }
                    }
                }
            }

            if (addParentAsTerminalNode)
            {
                context.FieldsTree.Add(context.Parent);
            }
        }

        private static bool ContainsMergedFieldProperties(PdfDictionaryOld dictionary)
        {
            return dictionary.ContainsKey("FT") || dictionary.ContainsKey("T");
        }

        private static bool ContainsWidgetProperties(PdfDictionaryOld dictionary)
        {
            if(dictionary.ContainsKey("Subtype"))
            {
                PdfNameOld subtype = dictionary.GetElement<PdfNameOld>("Subtype");

                return subtype.Value == "Widget";
            }

            return false;
        }

        private class FieldsConvertionContext
        {
            private readonly FormFieldsCollectionOld fields;
            private readonly PdfArrayOld currentLevelKids;
            private readonly FormFieldNodeOld currentLevelParent;

            public FieldsConvertionContext(FormFieldsCollectionOld fields, PdfArrayOld currentLevelKids, FormFieldNodeOld currentLevelParent)
            {
                this.fields = fields;
                this.currentLevelKids = currentLevelKids;
                this.currentLevelParent = currentLevelParent;
            }

            public PdfArrayOld Kids
            {
                get
                {
                    return this.currentLevelKids;
                }
            }

            public FormFieldNodeOld Parent
            {
                get
                {
                    return this.currentLevelParent;
                }
            }

            public FormFieldsCollectionOld FieldsTree
            {
                get
                {
                    return this.fields;
                }
            }

            public PdfContentManager ContentManager
            {
                get
                {
                    return this.fields.ContentManager;
                }
            }
        }
    }
}
