﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal abstract class IndirectReferenceConverterBase : ConverterBase
	{
        public override bool HandlesIndirectReference
        {
            get
            {
                return true;
            }
        }

        public override object Convert(Type type, PdfContentManager contentManager, object value)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");

            IndirectReferenceOld reference = value as IndirectReferenceOld;
            if (reference != null)
            {
                return this.ConvertFromIndirectReference(type, contentManager, reference);
            }

            return base.Convert(type, contentManager, value);
        }

        protected virtual object ConvertFromIndirectReference(Type type, PdfContentManager contentManager, IndirectReferenceOld indirectReference)
        {
            object res;
            if (!contentManager.TryGetPdfObject(indirectReference, out res))
            {
                IndirectObjectOld indirectObject;
                if (contentManager.TryGetIndirectObject(indirectReference, out indirectObject))
                {
                    res = Convert(type, contentManager, indirectObject.Value);
                }

                PdfObjectOld pdfObject = res as PdfObjectOld;
                if (pdfObject != null)
                {
                    pdfObject.Reference = indirectReference;

                }

                contentManager.RegisterIndirectReference(indirectReference, res);
            }

            return res;
        }
	}
}