﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters
{
    internal class XObjectConverter : IndirectReferenceConverterBase
	{
        protected override object ConvertFromPdfDataStream(Type type, PdfReader.PdfContentManager contentManager, Data.PdfDataStream stream)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(contentManager, "contentManager");
            Guard.ThrowExceptionIfNull(stream, "stream");

            XObject res;

            if (stream.Dictionary.ContainsKey(PdfNames.Subtype))
            {
                PdfNameOld subtype = stream.Dictionary.GetElement<PdfNameOld>(PdfNames.Subtype);

                switch (subtype.Value)
                {
                    case XObjectFactory.ImageSubtype:
                        if (stream.Dictionary.ContainsKey(PdfNames.ImageMask) && stream.Dictionary.GetElement<PdfBoolOld>(PdfNames.ImageMask).Value)
                        {
                            res = new ImageMaskOld(contentManager, stream.ReadData(contentManager));
                        }
                        else
                        {
                            res = new XImage(contentManager);
                        }
                        break;

                    case XObjectFactory.FormSubtype:
                        res = new XForm(contentManager);
                        break;

                    default:
                        throw new NotSupportedXObjectTypeException(string.Format("Not supported subtype value: {0}!", subtype.Value));
                }
            }
            else if (stream.Dictionary.ContainsKey("BBox"))
            {
                res = new XForm(contentManager);
            }
            else
            {
                throw new NotSupportedXObjectTypeException("XObject has missing Required properties!");
            }

            res.Load(stream);

            return res;
        }
    }
}
