﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf
{
    internal static class OldToProcessingTranslator
    {
        /// <summary>
        /// Returns <see cref="ColorSpaceObject"/> from <see cref="ColorSpaceOld"/>. 
        /// Not implemented for all types! Throws not implemented exception for the non-implemented conversion.
        /// </summary>
        public static ColorSpaceObject GetColorSpaceObjectFromColorSpaceOld(ColorSpaceOld colorSpaceOld)
        {
            switch (colorSpaceOld.Type)
            {
                case FormatProviders.Pdf.Filters.ColorSpace.Gray:
                    return new DeviceGrayColorSpaceObject();

                case FormatProviders.Pdf.Filters.ColorSpace.RGB:
                    return new DeviceRgbColorSpaceObject();

                case FormatProviders.Pdf.Filters.ColorSpace.CMYK:
                    return new DeviceCmykColorSpaceObject();

                case FormatProviders.Pdf.Filters.ColorSpace.Indexed:
                    {
                        IndexedOld indexedOld = (IndexedOld)colorSpaceOld;

                        IndexedColorSpaceObject indexedColorSpace = new IndexedColorSpaceObject();

                        indexedColorSpace.Base = GetColorSpaceObjectFromColorSpaceOld(indexedOld.Base);
                        indexedColorSpace.HiVal = indexedOld.HiVal;
                        indexedColorSpace.Lookup = new IndexedLookupStream(indexedOld.Lookup.Data);

                        return indexedColorSpace;
                    }
                case FormatProviders.Pdf.Filters.ColorSpace.ICCBased:
                    {
                        ICCBasedOld iCCBasedOld = (ICCBasedOld)colorSpaceOld;

                        IccBasedColorSpaceObject iccBasedColorSpace = new IccBasedColorSpaceObject();

                        iccBasedColorSpace.IccProfile.N = new PdfInt(iCCBasedOld.Profile.N.Value);
                        iccBasedColorSpace.IccProfile.Alternate = GetColorSpaceObjectFromColorSpaceOld(iCCBasedOld.Profile.Alternate);

                        return iccBasedColorSpace;
                    }
                ////case FormatProviders.Pdf.Filters.ColorSpace.Pattern:
                ////    {
                ////        PatternColorSpaceOld patternOld = (PatternColorSpaceOld)colorSpaceOld;

                ////        PatternColorSpaceObject patternColorSpace = new PatternColorSpaceObject();
                ////        patternColorSpace.UnderlyingColorSpace = GetColorSpaceObjectFromColorSpaceOld(patternOld.UnderlyingColorSpace);

                ////        return patternColorSpace;
                ////    }
                case FormatProviders.Pdf.Filters.ColorSpace.CalRgb:
                    return new CalRgbColorSpaceObject();

                case FormatProviders.Pdf.Filters.ColorSpace.CalGray:
                    return new CalGrayColorSpaceObject();
                default:
                    throw new NotImplementedException(string.Format("Color space conversion is not supported for object of type: {0}", colorSpaceOld.GetType()));
            }
        }
    }
}