﻿using System;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Utilities;
using DM = Telerik.Windows.Documents.Fixed.Model;
using PdfCore = Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core;
using RadFixedPage = Telerik.Windows.Documents.Fixed.Model.RadFixedPage;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Annot;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Forms;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
#if WPF
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.DigitalSignature;
using Telerik.Windows.Documents.Fixed.Model.DigitalSignatures;
#endif
using Rect = System.Windows.Rect;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Model.Internal.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.Utilities.Rendering;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Navigation;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf
{
    internal static class PdfElementToFixedElementTranslator
    {
        private const string UseOutlines = "UseOutlines";

        public static double PointToDip = Unit.PointToDip(1);


        public static void ToSLCoordinates(ContentElement element)
        {
            Guard.ThrowExceptionIfNull(element, "element");

            Image image = element as Image;
            if (image != null)
            {
                ImageToSLCoordinates(image);
            }
            else
            {
                Container container = element as Container;
                if (container != null)
                {
                    ContainerToSLCoordinates(container);
                }
            }
        }

        public static RadFixedPageInternal CreateFixedPage(RadFixedDocumentInternal doc, PageOld source)
        {
            Guard.ThrowExceptionIfNull(doc, "doc");
            Guard.ThrowExceptionIfNull(source, "source");

            RadFixedPage publicPage = doc.PublicDocument.Pages.AddPage();
            RadFixedPageInternal res = new RadFixedPageInternal(doc, publicPage);
            publicPage.InternalRadFixedPage = res;
            res.CropBox = source.CropBox.ToRect();

            int rotate = source.Rotate == null ? 0 : source.Rotate.Value % 360;
            publicPage.Rotation = Page.GetRotation(rotate);
            publicPage.CropBox = new Rect(0, 0, res.CropBox.Width * PointToDip, res.CropBox.Height * PointToDip);
            publicPage.MediaBox = publicPage.CropBox;

            res.TransformMatrix = new Matrix(PageLayoutHelper.CalculatePointToDipPageTransformation(publicPage));
            res.BoundingRect = PageLayoutHelper.CalculatePageBoundingRectangle(publicPage);

            return res;
        }

        public static Annotation CreateAnnotation(AnnotationOld source, PageOld page, PdfFormatProvider provider)
        {
            Guard.ThrowExceptionIfNull(source, "source");
            Guard.ThrowExceptionIfNull(page, "page");
            Guard.ThrowExceptionIfNull(provider, "provider");

            Annotation res = null;
            bool importCommonProperties = true;

            switch (source.Type)
            {
                case AnnotationType.Link:
                    res = CreateLinkAnnotation((LinkOld)source, provider);
                    break;
                case AnnotationType.Widget:
                    Widget widget;
                    importCommonProperties = !provider.TryGetWidgetFromWidgetObject((WidgetOld)source, out widget);
                    res = widget;
                    break;
                default:
                    res = new DM.Annotations.UnsupportedAnnotation(source.Type);
                    break;
            }

            if (importCommonProperties && res != null)
            {
                ApplyCommonAnnotationProperties(source, page, res);
            }

            return res;
        }

        internal static void ApplyCommonAnnotationProperties(AnnotationOld source, PageOld page, Annotation res)
        {
            if (page != null)
            {
                res.Rect = CalculateAnnotationRect(source, page);
            }

            res.AnnotationOldRect = source.Rect.ToRect();
            AnnotationObject.ImportFlagsToAnnotation(res, source.Flags.Value);

            AnnotationAppearancesOld appearances;
            if (source.TryGetAppearanceContent(out appearances))
            {
                res.AppearanceOld = appearances;
            }
        }

        internal static BookmarkItem CreateBookmarkItemHierarchy(OutlineItemOld outline, PdfFormatProvider provider)
        {
            BookmarkItem bookmark = CreateBookmarkItem(outline, provider);
            OutlineItemOld childOutline = outline.First;
            while (childOutline != null)
            {
                BookmarkItem childBookmark = CreateBookmarkItemHierarchy(childOutline, provider);
                bookmark.Children.Add(childBookmark);

                childOutline = childOutline.Next;
            }

            return bookmark;
        }

        internal static PageMode CreatePageMode(PdfNameOld pageModeName)
        {
            string pageModeNameValue = pageModeName != null ? pageModeName.Value : string.Empty;
            PageMode pageMode;

            switch (pageModeNameValue)
            {
                case UseOutlines:
                    pageMode = PageMode.UseBookmarks;
                    break;
                default:
                    pageMode = PageMode.UseNone;
                    break;
            }

            return pageMode;
        }

        private static Rect CalculateAnnotationRect(AnnotationOld source, PageOld page)
        {
            Rect sourceRect = source.Rect.ToRect();
            Rect pageRect = page.MediaBox.ToRect();
            Rect annotationRectInPoints = AnnotationObject.ConvertRect(sourceRect, pageRect.Height);
            double dipX = Unit.PointToDip(annotationRectInPoints.X);
            double dipY = Unit.PointToDip(annotationRectInPoints.Y);
            double dipWidth = Unit.PointToDip(annotationRectInPoints.Width);
            double dipHeight = Unit.PointToDip(annotationRectInPoints.Height);

            return new Rect(dipX, dipY, dipWidth, dipHeight);
        }

        private static void ImageToSLCoordinates(Image img)
        {
            Guard.ThrowExceptionIfNull(img, "img");

            Matrix m = img.TransformMatrix;
            m.Scale(1, -1, 0, 1);
            img.TransformMatrix = m;
        }

        private static void ContainerToSLCoordinates(Container container)
        {
            Guard.ThrowExceptionIfNull(container, "container");

            foreach (ContentElement child in container.Content)
            {
                ToSLCoordinates(child);
            }
        }

        private static DM.Actions.UriAction CreateUriAction(PdfCore.Actions.UriActionOld source)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            DM.Actions.UriAction res = new DM.Actions.UriAction();

            try
            {
                res.Uri = new Uri(source.Uri.ToString(), UriKind.RelativeOrAbsolute);
                res.IncludeMouseCoordinates = source.IsMap != null && source.IsMap.Value;
            }
            catch
            {
            }

            return res;
        }

        private static DM.Actions.GoToAction CreateGoToAction(PdfCore.Actions.GoToActionOld source, PdfFormatProvider provider)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            Destination destination = CreateDestination(source.Destination, provider);
            DM.Actions.GoToAction res = new DM.Actions.GoToAction(destination);

            return res;
        }

        private static DM.Actions.Action CreateAction(PdfCore.Actions.ActionOld source, PdfFormatProvider provider)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            PdfCore.Actions.UriActionOld uriAction = source as PdfCore.Actions.UriActionOld;
            if (uriAction != null)
            {
                return CreateUriAction(uriAction);
            }

            PdfCore.Actions.GoToActionOld gotoAction = source as PdfCore.Actions.GoToActionOld;
            if (gotoAction != null)
            {
                return CreateGoToAction(gotoAction, provider);
            }

            return null;
        }

        private static DM.Navigation.Destination CreateDestination(PdfCore.Navigation.DestinationOld source, PdfFormatProvider provider)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            DestinationType destinationType = (DM.Navigation.DestinationType)source.Type;
            RadFixedPage page = null;

            double? left = null;
            double? right = null;
            double? top = null;
            double? bottom = null;
            double? zoom = null;

            if (source.Page != null)
            {
                page = provider.GetRadFixedPageFromPage(source.Page).PublicPage;

                left = source.Left.HasValue ? (double?)page.InternalRadFixedPage.GetXInDip(source.Left.Value) : null;
                right = source.Right.HasValue ? (double?)page.InternalRadFixedPage.GetXInDip(source.Right.Value) : null;
                top = source.Top.HasValue ? (double?)page.InternalRadFixedPage.GetYInDip(source.Top.Value) : null;
                bottom = source.Bottom.HasValue ? (double?)page.InternalRadFixedPage.GetYInDip(source.Bottom.Value) : null;
                zoom = (source.Zoom.HasValue && source.Zoom.Value > 0) ? (double?)source.Zoom.Value : null;
            }

            switch (destinationType)
            {
                case DM.Navigation.DestinationType.BoundingRectangleFit:
                    return new DM.Navigation.BoundingRectangleFit() { Page = page };

                case DM.Navigation.DestinationType.BoundingRectangleHorizontalFit:
                    return new DM.Navigation.BoundingRectangleHorizontalFit()
                    {
                        Page = page,
                        Top = top
                    };

                case DM.Navigation.DestinationType.BoundingRectangleVerticalFit:
                    return new DM.Navigation.BoundingRectangleVerticalFit()
                    {
                        Page = page,
                        Left = left
                    };

                case DM.Navigation.DestinationType.Location:
                    return new DM.Navigation.Location()
                    {
                        Page = page,
                        Left = left,
                        Top = top,
                        Zoom = zoom
                    };

                case DM.Navigation.DestinationType.PageFit:
                    return new DM.Navigation.PageFit()
                    {
                        Page = page
                    };

                case DM.Navigation.DestinationType.PageHorizontalFit:
                    return new DM.Navigation.PageHorizontalFit()
                    {
                        Page = page,
                        Top = top,
                    };

                case DM.Navigation.DestinationType.PageVerticalFit:
                    return new DM.Navigation.PageVerticalFit()
                    {
                        Page = page,
                        Left = left
                    };

                case DM.Navigation.DestinationType.RectangleFit:
                    return new DM.Navigation.RectangleFit()
                    {
                        Page = page,
                        Left = left,
                        Right = right,
                        Bottom = bottom,
                        Top = top
                    };

                default:
                    return DM.Navigation.Destination.CreateDestination(destinationType, page);
            }
        }

        private static Link CreateLinkAnnotation(LinkOld source, PdfFormatProvider provider)
        {
            Link res = new Link();
            if (source.Action != null)
            {
                res.Action = CreateAction(source.Action, provider);
            }
            else if (source.Destination != null)
            {
                res.Destination = CreateDestination(source.Destination, provider);
            }

            return res;
        }

        internal static void InitializeSignatureField(SignatureField field, PdfContentManager contentManager, FormFieldNodeOld terminalFormField)
        {
#if WPF
            IndirectObjectOld obj;
            if (terminalFormField.FieldValue != null && contentManager.TryGetIndirectObject(terminalFormField.FieldValue.Reference, out obj))
            {
                PdfDictionaryOld dictionary = obj.Value as PdfDictionaryOld;
                if (dictionary != null)
                {
                    SignatureElementOld signatureElement = new SignatureElementOld(contentManager);
                    signatureElement.Load(obj);

                    field.Signature = new Signature();
                    signatureElement.CopyPropertiesTo(field.Signature);
                }
            }
#endif
        }

        private static BookmarkItem CreateBookmarkItem(OutlineItemOld outline, PdfFormatProvider provider)
        {
            string title = outline.Title != null ? outline.Title.ToString() : string.Empty;
            BookmarkItem bookmark = null;

            if (outline.Destination != null)
            {
                Destination destination = CreateDestination(outline.Destination, provider);
                bookmark = new BookmarkItem(title, destination);
            }
            else if (outline.Action != null)
            {
                DM.Actions.Action action = CreateAction(outline.Action, provider);
                bookmark = new BookmarkItem(title, action);
            }
            else
            {
                bookmark = new BookmarkItem(title);
            }

            bookmark.IsExpanded = outline.Count != null && outline.Count.Value > 0;
            bookmark.TextStyle = outline.Flag != null ? (BookmarkItemStyles)outline.Flag.Value : 0;

            return bookmark;
        }
    }
}
