﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader
{
    internal class PagesAwareCache<Key, Value>
    {
        private readonly Dictionary<Key, ValueUsedByPages> cache;
        private readonly Dictionary<RadFixedPage, HashSet<Key>> pageToCachedValuesMapping;

        public PagesAwareCache()
        {
            this.cache = new Dictionary<Key, ValueUsedByPages>();
            this.pageToCachedValuesMapping = new Dictionary<RadFixedPage, HashSet<Key>>();
        }

        public bool TryGetFromCache(RadFixedPage forPage, Key key, out Value value)
        {
            ValueUsedByPages valueForPages;
            if(this.cache.TryGetValue(key, out valueForPages))
            {
                value = valueForPages.Value;

                if(valueForPages.AddPageForThisValue(forPage))
                {
                    this.GetPageKeys(forPage).Add(key);
                }

                return true;
            }

            value = default(Value);
            return false;
        }

        public void AddToCache(RadFixedPage forPage, Key key, Value value)
        {
            this.cache.Add(key, new ValueUsedByPages(value, forPage));
            this.GetPageKeys(forPage).Add(key);
        }

        public void ClearCache(RadFixedPage forPage)
        {
            HashSet<Key> pageKeys = this.GetPageKeys(forPage);

            foreach(Key key in pageKeys)
            {
                ValueUsedByPages valueForPages = this.cache[key];
                valueForPages.RemovePageForThisValue(forPage);

                if(valueForPages.PagesUsingThisValueCount == 0)
                {
                    this.cache.Remove(key);
                }
            }

            pageKeys.Clear();
        }

        public void ClearCache()
        {
            this.cache.Clear();
            this.pageToCachedValuesMapping.Clear();
        }

        private HashSet<Key> GetPageKeys(RadFixedPage page)
        {
            HashSet<Key> pageKeys;
            if (!this.pageToCachedValuesMapping.TryGetValue(page, out pageKeys))
            {
                pageKeys = new HashSet<Key>();
                this.pageToCachedValuesMapping.Add(page, pageKeys);
            }

            return pageKeys;
        }

        private class ValueUsedByPages
        {
            private readonly Value value;
            private readonly HashSet<RadFixedPage> pagesUsingThisValue;

            public ValueUsedByPages(Value value, RadFixedPage firstPage)
            {
                this.value = value;
                this.pagesUsingThisValue = new HashSet<RadFixedPage>();
                this.pagesUsingThisValue.Add(firstPage);
            }

            public Value Value
            {
                get
                {
                    return this.value;
                }
            }

            public int PagesUsingThisValueCount
            {
                get
                {
                    return this.pagesUsingThisValue.Count;
                }
            }

            public bool AddPageForThisValue(RadFixedPage page)
            {
                return this.pagesUsingThisValue.Add(page);
            }

            public void RemovePageForThisValue(RadFixedPage page)
            {
                this.pagesUsingThisValue.Remove(page);
            }
        }
    }
}
