﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers
{
	internal class PdfParserBase
    {
        #region Constructors

        public PdfParserBase(PdfContentManager contentManager, Stream stream)
		{
            this.Reader = new Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Readers.PdfReader(stream);
            this.ContentManager = contentManager;
		}

		#endregion


        #region Properties

        internal Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Readers.PdfReader Reader { get;private set; }

        protected PdfContentManager ContentManager
        {
            get;
            private set;
        }

		#endregion


		#region Methods


        public object ParseCurrentTokenToIndirectReference(int objectNo, int generationNo)
        {
            return new IndirectReferenceOld { GenerationNumber = generationNo, ObjectNumber = objectNo };
        }
		public PdfBoolOld ParseCurrentTokenToBool()
		{
			if (this.Reader.TokenType == Token.Boolean)
			{
				return new PdfBoolOld(this.ContentManager, this.Reader.Result == Keywords.True);
			}
			return null;
		}
		public PdfIntOld ParseCurrentTokenToInt()
		{
			if (this.Reader.TokenType == Token.Integer)
			{
				int i;
				int.TryParse(this.Reader.Result, out i);
                return new PdfIntOld(this.ContentManager, i);
			}
			return null;
		}
        public PdfRealOld ParseCurrentTokenToReal()
        {
			if (this.Reader.TokenType == Token.Real)
			{
				double r;
                double.TryParse(this.Reader.Result, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture.NumberFormat, out r);
                return new PdfRealOld(this.ContentManager, r);
			}
			return null;
		}
		public PdfNameOld ParseCurrentTokenToName()
		{
			if (this.Reader.TokenType == Token.Name)
			{
                return new PdfNameOld(this.ContentManager, this.Reader.Result);
			}
			return null;
		}
        public PdfStringOld ParseCurrentTokenToString()
        {
            if (this.Reader.TokenType == Token.String)
            {
                return new PdfStringOld(this.ContentManager, this.Reader.BytesToken);
            }
            return null;
        }
        public PdfArrayOld ParseCurrentTokenToArray()
        {
            if (this.Reader.TokenType == Token.ArrayStart)
            {
                PdfArrayOld arr = new PdfArrayOld(this.ContentManager);
                object[] content = this.ReadObjectsToToken(Token.ArrayEnd);
                arr.Load(content);
                return arr;
            }
            return null;
        }
        public PdfDictionaryOld ParseCurrentTokenToDictionary()
        {
            if (this.Reader.TokenType == Token.DictionaryStart)
            {
                PdfDictionaryOld dict = new PdfDictionaryOld(this.ContentManager);
                object[] content = this.ReadObjectsToToken(Token.DictionaryEnd);
                dict.Load(content);
                return dict;
            }
            return null;
        }

        protected object[] ReadObjectsToToken(Token endToken)
        {
            Stack<object> stack = new Stack<object>();

            Token tok;
            while ((tok = this.Reader.ReadToken()) != endToken)
            {
                switch (tok)
                {
                    case Token.ArrayStart:
                        stack.Push(this.ParseCurrentTokenToArray());
                        break;
                    case Token.Boolean:
                        stack.Push(this.ParseCurrentTokenToBool());
                        break;
                    case Token.Real:
                        stack.Push(this.ParseCurrentTokenToReal());
                        break;
                    case Token.DictionaryStart:
                        stack.Push(this.ParseCurrentTokenToDictionary());
                        break;
                    case Token.String:
                        stack.Push(this.ParseCurrentTokenToString());
                        break;
                    case Token.Integer:
                        stack.Push(this.ParseCurrentTokenToInt());
                        break;
                    case Token.Name:
                        stack.Push(this.ParseCurrentTokenToName());
                        break;
                    case Token.Null:
                        stack.Push(null);
                        break;
                    case Token.IndirectReference:
                        int generationNo = ((PdfIntOld)stack.Pop()).Value;
                        int objectNo = ((PdfIntOld)stack.Pop()).Value;
                        stack.Push(this.ParseCurrentTokenToIndirectReference(objectNo, generationNo));
                        break;
                    case Token.Unknown:
                        break;
                    default:
                        this.ParseCurrentTokenOverride(tok, stack);
                        break;
                }

                if (this.Reader.EndOfFile)
                {
                    break;
                }
            }

            int cnt = stack.Count - 1;
            object[] res = new object[cnt + 1];
            while (cnt >= 0)
            {
                res[cnt] = stack.Pop();
                cnt--;
            }

            return res;
        }

        protected int ReadInt()
        {
            if (this.Reader.ReadToken() == Token.Integer)
            {
                int res;
                int.TryParse(this.Reader.Result, out res);
                return res;
            }
            return -1;
        }
        protected virtual void ParseCurrentTokenOverride(Token token, Stack<object> stack)
        {
        }

		#endregion
	}
}
