﻿using System.IO;
using Telerik.Windows.Documents.Core.PostScript;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Readers
{
	internal class CMapStreamReader : PdfReaderBase
	{
		public CMapStreamReader(byte[] data)
			: base(new MemoryStream(data))
		{
		}

		public override Token ReadToken()
		{
            PostScriptReaderHelper.SkipUnusedCharacters(this);

            if (this.EndOfFile)
            {
                return this.TokenType = Token.EndOfFile;
            }

            switch ((char)this.Peek())
			{
				case Characters.LiteralStringStart:
					return this.ReadLiteralString();
                case Characters.Name:
					return this.TokenType = this.ReadName();
                case Characters.ArrayStart:
					this.Read();
					return this.TokenType = Token.ArrayStart;
                case Characters.ArrayEnd:
					this.Read();
                    return this.TokenType = Token.ArrayEnd;
                case Characters.HexadecimalStringStart:
                    if (this.Peek(1) == '<')
					{
						this.Read();
						this.Read();
                        return this.TokenType = Token.DictionaryStart;
					}
					return this.ReadHexadecimalString();
                case Characters.HexadecimalStringEnd:
                    if (this.Peek(1) == '>')
					{
						this.Read();
						this.Read();
                        return this.TokenType = Token.DictionaryEnd;
					}
                    break;
			}

            if (Characters.IsValidNumberChar(this))
			{
				return this.ReadNumber();
			}

            if (Characters.IsLetter(this.Peek()))
			{
				return this.ReadKeywordOrOperator();
			}

			this.Read();
            return this.TokenType = Token.Unknown;
		}

        private Token ReadKeywordOrOperator()
        {
            string token = PostScriptReaderHelper.ReadKeyword(this);
            this.SetToken(token);

            switch (token)
            {
                case Keywords.True:
                case Keywords.False:
                    return this.TokenType = Token.Boolean;
                case Keywords.Null:
                    return this.TokenType = Token.Null;
            }

            if (CMapOperators.IsOperator(token))
            {
                return this.TokenType = Token.Operator;
            }
            return this.TokenType = Token.Unknown;
        }
	}
}
