﻿using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Core.PostScript;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Readers
{
	internal class PdfReader : PdfReaderBase
	{
		#region Consts

		public static readonly int BufferSize = 1024;

		#endregion


		#region Constructors

		public PdfReader(Stream stream) :
			base(stream)
		{
		}

		#endregion


        #region Methods

        private Token ReadKeyword()
		{
            string token = PostScriptReaderHelper.ReadKeyword(this);
			this.SetToken(token);
			switch (token)
			{
				case Keywords.StartXRef:
                    return this.TokenType = Token.StartXRef;
				case Keywords.True:
				case Keywords.False:
                    return this.TokenType = Token.Boolean;
				case Keywords.XRef:
                    return this.TokenType = Token.XRef;
				case Keywords.StreamStart:
                    return this.TokenType = Token.StreamStart;
				case Keywords.StreamEnd:
                    return this.TokenType = Token.StreamEnd;
				case Keywords.Null:
                    return this.TokenType = Token.Null;
				case Keywords.IndirectObjectStart:
                    return this.TokenType = Token.IndirectObjectStart;
				case Keywords.IndirectObjectEnd:
                    return this.TokenType = Token.IndirectObjectEnd;
				case Keywords.IndirectReference:
                    return this.TokenType = Token.IndirectReference;
				case Keywords.Trailer:
                    return this.TokenType = Token.Trailer;
				default:
                    return this.TokenType = Token.Unknown;
			}
		}

		public bool SeekToStartXRef()
		{
			StringBuilder tmp = new StringBuilder();
			StringBuilder tok = new StringBuilder(Keywords.StartXRef);
            long seek = Math.Min(BufferSize, base.Stream.Length);
			base.Stream.Seek(-seek, SeekOrigin.End);
			long offset = -1;
            byte[] buffer = new byte[BufferSize];
            int len = this.Stream.Read(buffer, 0, BufferSize);
			for (int i = 0; i < len; i++)
			{
				tmp.Append((char)buffer[i]);
				if (tok.Length < tmp.Length)
				{
					tmp.Remove(0, 1);
				}
				if (tmp.Equals(tok))
				{
					offset = this.Stream.Position - (len - i - 1) - tmp.Length;
				}
			}
			if (offset == -1)
			{
				return false;
			}
			this.Seek(offset, SeekOrigin.Begin);
			return true;
		}

		public byte[] ReadStream(int length)
		{
			byte[] res = new byte[length];
			this.Stream.Seek(-2, SeekOrigin.Current);
			this.Stream.Read(res, 0, length);
			this.ReadBufferedChars();
			return res;
		}

		public override Token ReadToken()
		{
			this.SkipUnusedCharacters();

			switch ((char)this.Peek())
			{
                case Characters.LiteralStringStart:
					return this.ReadLiteralString();
                case Characters.Name:
					return this.TokenType = this.ReadName();
                case Characters.ArrayStart:
					this.Read();
                    return this.TokenType = Token.ArrayStart;
                case Characters.ArrayEnd:
					this.Read();
                    return this.TokenType = Token.ArrayEnd;
                case Characters.HexadecimalStringStart:
					if (this.Peek(1) == '<')
					{
						this.Read();
						this.Read();
                        return this.TokenType = Token.DictionaryStart;
					}
					return this.ReadHexadecimalString();
                case Characters.HexadecimalStringEnd:
					if (this.Peek(1) == '>')
					{
						this.Read();
						this.Read();
                        return this.TokenType = Token.DictionaryEnd;
					}
                    break;
			}

            if (Characters.IsValidNumberChar(this))
			{
				return this.ReadNumber();
			}

            if (Characters.IsLetter(this.Peek()))
			{
				return this.ReadKeyword();
			}

			this.Read();
            return this.TokenType = Token.Unknown;
		}

		#endregion

        
    }
}
