﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Core.PostScript;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Readers
{
	internal abstract class PdfReaderBase : IPostScriptReader
	{
        private int extraReadBytes;
        private string token;
        private byte[] strToken;

        private byte currentByte;
        private byte nextByte;

		public PdfReaderBase(Stream stream)
		{
			this.Stream = stream;
			this.ReadBufferedChars();
		}

		public Stream Stream
		{
			get;
			private set;
		}
		
        public Token TokenType
		{
			get;
			protected set;
		}
		
        public long Position
		{
			get
			{
				return this.Stream.Position - this.extraReadBytes;
			}
		}
	
        public string Result
		{
			get
			{
				return this.token;
			}
		}
       
        public byte[] BytesToken
        {
            get
            {
                return this.strToken;
            }
        }
      
        public bool EndOfFile
        {
            get { return this.Position >= this.Stream.Length; }
        }

        public byte Read(bool appendLineFeed = true)
        {
            byte res = this.currentByte;
            this.ReadInternal(appendLineFeed);
           
            return res;
        }
      
        public byte Peek(int skip = 0)
        {
            switch (skip)
            {
                case 0:
                    return this.currentByte;
                case 1:
                    return this.nextByte;
            }
          
            throw new NotSupportedException();
        }
    
        public string ReadLine()
        {
            StringBuilder sb = new StringBuilder();
            while (this.Peek() != Characters.LF)
            {
                sb.Append((char)this.Read());
            }
          
            this.Read();

            return sb.ToString();
        }
     
        public Token ReadName()
        {
            this.SetToken(PostScriptReaderHelper.ReadName(this));
           
            return Token.Name;
        }
      
        public Token ReadNumber()
        {
            string token = PostScriptReaderHelper.ReadNumber(this);
            bool period = Enumerable.Contains(token, Characters.DecimalPoint);
            this.SetToken(token.ToString());
         
            return this.TokenType = period ? Token.Real : Token.Integer;
        }
       
        public Token ReadHexadecimalString()
        {
            this.strToken = PostScriptReaderHelper.ReadHexadecimalString(this);
           
            return this.TokenType = Token.String;
        }
      
        public Token ReadLiteralString()
        {
            this.strToken = PostScriptReaderHelper.ReadLiteralString(this);
           
            return this.TokenType = Token.String;
        }
      
        public long Seek(long offset, SeekOrigin origin)
        {
            if (origin == SeekOrigin.Current)
            {
                if (offset < 0 && Math.Abs(offset) > this.Stream.Position)
                {
                    offset = 0;
                    origin = SeekOrigin.Begin;
                }
                else if (offset > 0 && this.Stream.Position + offset > this.Stream.Length)
                {
                    offset = 0;
                    origin = SeekOrigin.End;
                }
            }

            long res = this.Stream.Seek(offset, origin);
            this.ReadBufferedChars();
            
            return res;
        }
        
        public abstract Token ReadToken();

        public int ReadInternal(bool appendLineFeed = true)
		{
            if (this.EndOfFile)
            {
                Guard.ThrowPositionOutOfRangeException();
            }
			
            this.currentByte = this.nextByte;
            if (this.Stream.Position == this.Stream.Length)
            {
                this.extraReadBytes--;
            }
			
            this.nextByte = (byte)this.Stream.ReadByte();

            if (this.currentByte == Characters.CR)
            {
                if (this.nextByte == Characters.LF)
                {
                    this.ReadInternal();
                }
                else
                {
                    this.currentByte = (byte)Characters.LF;
                }
            }
			
            return this.currentByte;
		}

        public void SetToken(string tok)
		{
			this.token = tok;
		}

        public void SkipUnusedCharacters()
        {
            PostScriptReaderHelper.SkipUnusedCharacters(this);
        }

        public void GoToNextLine(bool appendLineFeed = true)
        {
            PostScriptReaderHelper.GoToNextLine(this, appendLineFeed);
        }

        public void SkipWhiteSpaces()
        {
            PostScriptReaderHelper.SkipWhiteSpaces(this);
        }

        public byte[] ReadRawData(long offset, StreamPart part)
        {
            part.Seek(0, SeekOrigin.Begin);
            byte[] data = Extensions.ReadAllBytes(part);
            this.Seek(offset + part.Length, SeekOrigin.Begin);

            return data;
        }

		protected void ReadBufferedChars()
		{
            if (!this.EndOfFile)
            {
                this.ReadInternal();

                if (!this.EndOfFile)
                {
                    this.ReadInternal();
                    this.extraReadBytes = 2;
                }
                else
                {
                    this.extraReadBytes = 1;
                }
            }
		}
    }
}
