﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader
{
    internal class ThreadSafePageContentManagerOld : IPageContentManager
    {
        private readonly PdfFormatProvider provider;
#if SILVERLIGHT
        private readonly object lockObject;
#endif

        public ThreadSafePageContentManagerOld(PdfFormatProvider provider)
        {
            Guard.ThrowExceptionIfNull(provider, "provider");

            this.provider = provider;
#if SILVERLIGHT
            this.lockObject = new object();
#endif
        }

        public void LoadPageContent(RadFixedPage page)
        {
            Guard.ThrowExceptionIfNull(page.InternalRadFixedPage, "page.InternalRadFixedPage");
            RadFixedPageInternal internalPage = page.InternalRadFixedPage;

            if (internalPage.Content == null)
            {
#if SILVERLIGHT
                lock (this.lockObject)
#else
                lock (this.provider)
#endif
                {
                    if (internalPage.Content == null)
                    {
                        internalPage.Content = this.provider.ParsePageContent(internalPage);
                        internalPage.Arrange();
                    }
                }
            }
        }

        public void UnloadPageContent(RadFixedPage page)
        {
            Guard.ThrowExceptionIfNull(page.InternalRadFixedPage, "page.InternalRadFixedPage");
            RadFixedPageInternal internalPage = page.InternalRadFixedPage;

            if (internalPage.Content != null)
            {
#if SILVERLIGHT
                lock (this.lockObject)
#else
                lock (this.provider)
#endif
                {
                    if (internalPage.Content != null)
                    {
                        internalPage.Content = null;
                    }
                }
            }
        }

        public bool IsPageContentEmpty(RadFixedPage page)
        {
            return page.Content.Count == 0;
        }
    }
}
