﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text
{
	internal class CMapOld
    {
        public static CMapOld Identity
        {
            get;
            private set;
        }

        static CMapOld()
        {
            CMapOld.Identity = new CMapOld(true);
        }

        public CMapOld()
        {
            this.codeRanges = new List<Tuple<CharCodeOld, CharCodeOld>>();
            this.cidToUnicodeMapping = new Dictionary<CharCodeOld, string>();
            this.ICCtoCIDMapping = new Dictionary<CharCodeOld, CharCodeOld>();
            this.notDef = new List<Tuple<CharCodeOld, Tuple<CharCodeOld, CharCodeOld>>>();
        }

        private CMapOld(bool identity)
        {
            this.isIdentity = identity;
        }

        private Dictionary<CharCodeOld, CharCodeOld> ICCtoCIDMapping;
        private Dictionary<CharCodeOld, string> cidToUnicodeMapping;
        private List<Tuple<CharCodeOld, Tuple<CharCodeOld, CharCodeOld>>> notDef;
        private List<Tuple<CharCodeOld, CharCodeOld>> codeRanges;
        bool isIdentity;

        public CMapOld UseCMap
        {
            get;
            set;
        }

        private CharCodeOld CreateCharCode(byte[] bytes, int offset, int count)
        {
            byte[] b = new byte[count];
            for (int i = 0; i < count; i++)
            {
                b[i] = bytes[offset + i];
            }
            return new CharCodeOld(b);
        }

        private bool IsInRange(CharCodeOld cc)
        {
            foreach (var p in this.codeRanges)
            {
                if (cc.BytesCount == p.Item1.BytesCount && p.Item1 <= cc && cc <= p.Item2)
                {
                    return true;
                }
            }
            return false;
        }

        private CharCodeOld GetNotDef(CharCodeOld cc)
        {
            foreach (var key in this.notDef)
            {
                if (key.Item1 >= cc && cc <= key.Item2.Item1)
                {
                    return key.Item2.Item2;
                }
            }
            return new CharCodeOld();
        }

        private Tuple<string, CharCodeOld> GetCIDtoUnicodeMapping(byte[] bytes, int offset, int count, bool checkRanges = true)
        {
            if (offset + count > bytes.Length)
            {
                return new Tuple<string, CharCodeOld>();
            }
            CharCodeOld cc = this.CreateCharCode(bytes, offset, count);
            string res;
            bool isInRange = checkRanges ? this.IsInRange(cc) : true;
            if (isInRange)
            {
                if (this.cidToUnicodeMapping.TryGetValue(cc, out res))
                {
                    return new Tuple<string, CharCodeOld>(res, cc);
                }
            }
            if (this.UseCMap != null && this.UseCMap.IsInRange(cc))
            {
                return this.UseCMap.GetCIDtoUnicodeMapping(bytes, offset, count, checkRanges);
            }
            return new Tuple<string, CharCodeOld>();
        }

        private CharCodeOld GetICCtoCIDMapping(byte[] bytes, int offset, int count, bool checkRanges = true)
        {
            if (offset + count > bytes.Length)
            {
                return new CharCodeOld();
            }

            CharCodeOld cc = this.CreateCharCode(bytes, offset, count);
            bool isInRange = checkRanges ? this.IsInRange(cc) : true;
            if (isInRange)
            {
                CharCodeOld result;
                if (this.ICCtoCIDMapping.TryGetValue(cc, out result))
                {
                    return result;
                }
            }

            if (this.UseCMap != null && this.UseCMap.IsInRange(cc))
            {
                return this.UseCMap.GetICCtoCIDMapping(bytes, offset, count, checkRanges);
            }

            return this.GetNotDef(cc);
        }

        private IEnumerable<CharCodeOld> GetIdentityCharIds(byte[] bytes)
        {
            List<CharCodeOld> res = new List<CharCodeOld>();
            for (int i = 0; i < bytes.Length; i+=2)
            {
                byte[] b = new byte[2];
                if (i + 1 >= bytes.Length)
                {
                    b[0] = 0;
                    b[1] = bytes[i];
                }
                else
                {
                    b[0] = bytes[i];
                    b[1] = bytes[i + 1];
                }
                CharCodeOld r = new CharCodeOld(b);
                res.Add(r);
            }
            return res.ToArray();
        }

        private IEnumerable<Tuple<string, CharCodeOld>> GetIdentityToUnicode(byte[] bytes)
        {
            string res = System.Text.Encoding.BigEndianUnicode.GetString(bytes, 0, bytes.Length);
            for (int i = 0; i < res.Length; i++)
            {
                yield return new Tuple<string, CharCodeOld>(res[i].ToString(), new CharCodeOld((ushort)res[i]));
            }
        }

        internal void AddCIDtoUnicodeMapping(CharCodeOld cid, string ch)
        {
            this.cidToUnicodeMapping[cid] = ch;
        }

        internal void AddICCtoCIDMapping(CharCodeOld icc, int cid)
        {
            this.ICCtoCIDMapping[icc] = new CharCodeOld(cid);
        }

        internal void AddNotDefRange(CharCodeOld begin, CharCodeOld end, int cid)
        {
            this.notDef.Add(new Tuple<CharCodeOld, Tuple<CharCodeOld, CharCodeOld>>(begin, new Tuple<CharCodeOld, CharCodeOld>(end, new CharCodeOld(cid))));
        }

        internal void AddCodeRange(CharCodeOld begin, CharCodeOld end)
        {
            this.codeRanges.Add(new Tuple<CharCodeOld, CharCodeOld>(begin, end));
        }

        private IEnumerable<Tuple<string, CharCodeOld>> GetToUnicode(byte[] bytes)
        {
            int i = 0;
            Tuple<string, CharCodeOld> res;
            bool found;
            while (i < bytes.Length)
            {
                found = false;
                for (int j = 1; j <= BytesHelperOld.CID_BYTES_COUNT; j++)
                {
                    res = this.GetCIDtoUnicodeMapping(bytes, i, j, false);
                    if (!res.IsEmpty())
                    {
                        yield return res;
                        i += j;
                        found = true;
                        break;
                    }
                }
                if (!found)
                {
                    i++;
                }
            }
        }

        public IEnumerable<CharCodeOld> GetCharIds(byte[] bytes)
        {
            if (this.isIdentity)
            {
                return this.GetIdentityCharIds(bytes);
            }
            int i = 0;
            List<CharCodeOld> charIDs = new List<CharCodeOld>();
            CharCodeOld res;
            bool found;
            while (i < bytes.Length)
            {
                found = false;
                for (int j = 1; j <= BytesHelperOld.CID_BYTES_COUNT; j++)
                {
                    res = this.GetICCtoCIDMapping(bytes, i, j, false);
                    if (!res.IsEmpty)
                    {
                        charIDs.Add(res);
                        i += j;
                        found = true;
                        break;
                    }
                }
                if (!found)
                {

                    i++;
                    //Debug.Assert(false, "Mapping not found!");
                }
            }
            return charIDs.ToArray();
        }

        public IEnumerable<Tuple<string, CharCodeOld>> ToUnicode(byte[] bytes)
        {
            if (this.isIdentity)
            {
                return this.GetIdentityToUnicode(bytes);
            }
            return this.GetToUnicode(bytes);
        }

        public bool TryGetToUnicode(CharCodeOld charCode, out string unicode)
        {
            unicode = null;

            if (this.isIdentity)
            {
                var res = Enumerable.FirstOrDefault(this.GetIdentityToUnicode(charCode.Bytes));
                if (res.IsEmpty())
                {
                    return false;
                }

                unicode = res.Item1;
                return true;
            }
            else
            {
                return this.cidToUnicodeMapping.TryGetValue(charCode, out unicode);
            }
        }
    }
}
