﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text
{
    internal partial class FontsManagerOld
    {
        private readonly Dictionary<FontBaseOld, EncodingOld> encodings;
        private readonly Dictionary<Type0FontOld, CMapOld> type0FontsEncodings;
        private readonly Dictionary<FontBaseOld, CMapOld> unicodes;
        private FontsManager systemFontsManager;

        public FontsManagerOld()
        {
            this.encodings = new Dictionary<FontBaseOld, EncodingOld>();
            this.type0FontsEncodings = new Dictionary<Type0FontOld, CMapOld>();
            this.unicodes = new Dictionary<FontBaseOld, CMapOld>();
        }

        public FontsManager SystemFontsManager
        {
            get
            {
                if (this.systemFontsManager == null)
                {
                    this.systemFontsManager = new FontsManager();
                }

                return this.systemFontsManager;
            }
        }

        public void SetEncoding(FontBaseOld font, EncodingOld encoding)
        {
            Guard.ThrowExceptionIfNull(font, "font");

            this.encodings[font] = encoding;
        }

        public CMapOld GetType0FontEncoding(Type0FontOld font)
        {
            Guard.ThrowExceptionIfNull(font, "font");

            CMapOld res;
            if (!this.type0FontsEncodings.TryGetValue(font, out res))
            {
                if (font.Encoding == null)
                {
                    res = null;
                }
                else if (font.Encoding.IsIdentityH || font.Encoding.IsIdentityV)
                {
                    res = CMapOld.Identity;
                }
                else
                {
                    CMapStreamParser parser = new CMapStreamParser(font.ContentManager, font.Encoding);
                    res = parser.ParseCMap();
                }

                this.type0FontsEncodings[font] = res;
            }

            return res;
        }

        public CMapOld GetFontToUnicode(FontBaseOld font)
        {
            Guard.ThrowExceptionIfNull(font, "font");

            CMapOld res = null;
            if (!this.unicodes.TryGetValue(font, out res))
            {
                if (font.ToUnicode != null)
                {
                    if (font.ToUnicode.IsIdentityH || font.ToUnicode.IsIdentityV)
                    {
                        res = CMapOld.Identity;
                    }
                    else
                    {
                        CMapStreamParser parser = new CMapStreamParser(font.ContentManager, font.ToUnicode);
                        res = parser.ParseCMap();
                    }
                }

                this.unicodes[font] = res;
            }

            return res;
        }
    }
}
