﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils
{
    internal static class BytesHelperOld
    {
        public const int CID_BYTES_COUNT = 4;

        public static char GetUnicodeChar(int i)
        {
            return (char)i;
        }

        public static byte GetDecimalFromHex(char hex)
        {
            if (hex >= '0' && hex <= '9')
            {
                return (byte)(hex - '0');
            }
            else if (hex >= 'a' && hex <= 'f')
            {
                return (byte)(hex - 'a' + 10);
            }
            else
            {
                return (byte)(hex - 'A' + 10);
            }
        }

        public static int GetInt(byte[] bytes, int offset, int count)
        {
            byte[] b = new byte[count];
            for (int i = 0; i < count; i++)
            {
                b[i] = bytes[offset + i];
            }
            return BytesHelperOld.GetInt(b);
        }

        public static int GetInt(byte[] bytes)
        {
            byte[] b = GetBytes(bytes);
            if (b == null)
            {
                return - 1;
            }
            int res = 0;
            int len = b.Length;
            for (int i = 0; i < len; i++)
            {
                res |= len <= i ? 0 & 0xFF : b[i] & 0xFF;
                if (i < len - 1)
                {
                    res <<= 8;
                }
            }
            return res;
        }

        private static byte[] GetBytes(byte[] bytes)
        {
            if (bytes == null || bytes.Length > 4)
            {
                return null;
            }
            if (bytes.Length == 0)
            {
                return bytes;
            }
            int len = bytes.Length;
            int bCount = (len / 2 + len % 2) * 2; // Get number of bytes;
            byte[] b = new byte[bCount];
            if (len == 1)
            {
                b[0] = 0;
                b[1] = bytes[0];
            }
            else if (bCount == 2)
            {
                b[0] = bytes[0];
                b[1] = bytes[1];
            }
            else if (len == 3)
            {
                b[0] = 0;
                b[1] = bytes[0];
                b[2] = bytes[1];
                b[3] = bytes[2];
            }
            else
            {
                b[0] = bytes[0];
                b[1] = bytes[1];
                b[2] = bytes[2];
                b[3] = bytes[3];
            }
            return b;
        }

        public static int ToInt16(byte[] bytes, int offset, int count)
        {
            if (bytes.Length <= offset)
            {
                return -1;
            }
            List<byte> b = new List<byte>();
            for (int i = 0; i < count; i++)
            {
                if (offset + i < bytes.Length)
                {
                    b.Add(bytes[offset + i]);
                }
                else
                {
                    b.Insert(0, 0);
                }
            }
            return BytesHelperOld.GetInt(b.ToArray());
        }

        public static int ToInt32(byte[] bytes, int offset, int count)
        {
            if (bytes.Length - 3 <= offset)
            {
                return -1;
            }
            List<byte> b = new List<byte>();
            for (int i = 0; i < count; i++)
            {
                if (offset + i < bytes.Length)
                {
                    b.Add(bytes[offset + i]);
                }
                else
                {
                    b.Insert(0, 0);
                }
            }
            return BytesHelperOld.GetInt(b.ToArray());
        }
        
        public static byte[] RemoveWhiteSpace(byte[] data)
        {
            int count = data.Length;
            int j = 0;
            for (int i = 0; i < count; i++, j++)
            {
                switch (data[i])
                {
                    case (byte)Characters.Null:  // 0 Null
                    case (byte)Characters.Tab:   // 9 Tab
                    case (byte)Characters.LF:   // 10 Line feed
                    case (byte)Characters.FF:   // 12 Form feed
                    case (byte)Characters.CR:   // 13 Carriage return
                    case (byte)Characters.Space:   // 32 Space
                        j--;
                        break;

                    default:
                        if (i != j)
                        {
                            data[j] = data[i];
                        }

                        break;
                }
                if (j < count)
                {
                    byte[] temp = data;
                    data = new byte[j];
                    for (int idx = 0; idx < j; idx++)
                    {
                        data[idx] = temp[idx];
                    }
                }

            }
            return data;
        }

        public static void GetBytes(int b, byte[] output)
        {
            byte[] res = new byte[4];

            res[0] = (byte)((b & 0xFF));
            res[1] = (byte)((b >> 8) & 0xFF);
            res[2] = (byte)((b >> 16) & 0xFF);
            res[3] = (byte)(b >> 24);

            for (int i = 0; i < output.Length; i++)
            {
                output[i] = res[i];
            }
        }
    }
}
