﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils
{
    internal static class Extensions
    {
        public static IEnumerable<T> EnumerateChildrenOfType<T>(IEnumerable<Model.Internal.Classes.IContentElement> collection) where T : Model.Internal.Classes.IContentElement
        {
            foreach (Telerik.Windows.Documents.Fixed.Model.Internal.Classes.IContentElement ce in collection)
            {
                if (ce is T)
                {
                    yield return (T)ce;
                }

                if (ce.HasChildren)
                {
                    foreach (Telerik.Windows.Documents.Fixed.Model.Internal.Classes.IContentElement el in Extensions.EnumerateChildrenOfType<T>(ce.Children))
                    {
                        yield return (T)el;
                    }
                }
            }
        }

        public static bool GetFlag(int flags, int flag)
        {
            return (flags & (1 << (flag - 1))) != 0;
        }

        public static object[] ToParams<T>(this T[] array)
        {
            object[] res = new object[array.Length];

            for (int i = 0; i < array.Length; i++)
            {
                res[i] = array[i];
            }

            return res;
        }

        public static T[] ToParams<T>(this object[] array)
        {
            T[] res = new T[array.Length];

            for (int i = 0; i < array.Length; i++)
            {
                res[i] = (T)array[i];
            }

            return res;
        }

        public static PdfArrayOld ToPdfArray(this string[] array, PdfContentManager contentManager)
        {
            PdfArrayOld pdfArray = new PdfArrayOld(contentManager);

            foreach (string text in array)
            {
                pdfArray.Add(text);
            }

            return pdfArray;
        }

        public static double[] ToDoubleArray(this PdfArrayOld array)
        {
            return array.ToArray((primitive) => primitive.ToReal());
        }

        public static int[] ToIntArray(this PdfArrayOld array)
        {
            return array.ToArray((primitive) => (int)(primitive.ToReal()));
        }

        public static string[] ToStringArray(this PdfArrayOld array)
        {
            return array.ToArray((primitive) => primitive.ToString());
        }

        private static double ToReal(this object value)
        {
            double real;
            Helper.UnboxDouble(value, out real);

            return real;
        }

        private static T[] ToArray<T>(this PdfArrayOld array, Func<object, T> primitiveConverter)
        {
            T[] result = new T[array.Count];

            for (int i = 0; i < array.Count; i++)
            {
                result[i] = primitiveConverter(array[i]);
            }

            return result;
        }
    }
}
