﻿using System;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utils
{
    /// <summary>
    /// Represents format provider settings converter.
    /// </summary>
    public class FormatProviderSettingsTypeConverter : TypeConverter
    {
        /// <summary>
        /// Returns whether the type converter can convert an object from the specified
        /// type to the type of this converter.
        /// </summary>
        /// <param name="context">An object that provides a format context.</param>
        /// <param name="sourceType">The type you want to convert from.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return sourceType == typeof(string) || sourceType == typeof(FormatProviderSettings);
        }

        /// <summary>
        /// Returns whether the type converter can convert an object to the specified
        /// type.
        /// </summary>
        /// <param name="context">An object that provides a format context.</param>
        /// <param name="destinationType">The type you want to convert to.</param>
        /// <returns>
        /// true if this converter can perform the conversion; otherwise, false.
        /// </returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return false;
        }

        /// <summary>
        /// Converts from the specified value to the intended conversion type of
        /// the converter.
        /// </summary>
        /// <param name="context">An object that provides a format context.</param>
        /// <param name="culture">The <see cref="T:System.Globalization.CultureInfo" /> to
        /// use as the current culture.</param>
        /// <param name="value">The value to convert to the type of this converter.</param>
        /// <exception cref="T:System.NotImplementedException">
        /// <see cref="M:System.ComponentModel.TypeConverter.ConvertFrom(System.ComponentModel.ITypeDescriptorContext,System.Globalization.CultureInfo,System.Object)" />
        /// not implemented in base <see cref="T:System.ComponentModel.TypeConverter" />.
        /// </exception>
        /// <returns>The converted value.</returns>
        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            string str = value as string;
            if (str != null)
            {
                switch (str)
                {
                    case "ReadAllAtOnce":
                        return FormatProviderSettings.ReadAllAtOnce;
                    case "ReadOnDemand":
                        return FormatProviderSettings.ReadOnDemand;
                    default:
                        return null;
                }
            }
            else if (value is FormatProviderSettings)
            {
                return value;
            }

            return null;
        }
    }
}
