﻿using System;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Threading;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.Model.Internal;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils
{
    internal static class Helper
    {
        public static Dispatcher Dispatcher
        {
            get
            {
#if SILVERLIGHT
                return Deployment.Current.Dispatcher;
#elif WPF
                if (Application.Current == null)
                {
                    return null;
                }

                return Application.Current.Dispatcher;
#endif
            }
        }

        public static Matrix CalculateTextMatrix(Matrix m, GlyphOld glyph)
        {
            double width = glyph.Width;
            double tx = ((width * glyph.FontSize) + glyph.CharSpacing + glyph.WordSpacing) * (glyph.HorizontalScaling / 100);
           
            return new Matrix(1, 0, 0, 1, tx, 0) * m;
        }

        public static object[] BoxDoubleParameters(double[] doubleParameters)
        {
            object[] parameters = new object[doubleParameters.Length];

            for (int i = 0; i < parameters.Length; i++)
            {
                parameters[i] = doubleParameters[i];
            }

            return parameters;
        }

        public static bool UnboxDouble(object obj, out double res)
        {
            res = 0;
            if (obj == null)
            {
                return false;
            }
            
            if (obj is byte)
            {
                res = (byte)obj;
                
                return true;
            }
            
            if (obj is int)
            {
                res = (int)obj;
                
                return true;
            }
          
            if (obj is double)
            {
                res = (double)obj;
                
                return true;
            }

            PdfIntOld i = obj as PdfIntOld;
            if (i != null)
            {
                i.Load();
                res = i.Value;

                return true;
            }

            PdfRealOld r = obj as PdfRealOld;
            if (r != null)
            {
                r.Load();
                res = r.Value;
                
                return true;
            }
            return false;
        }

        public static bool GetBit(int n, byte bit)
        {
            return (n & (1 << bit)) != 0;
        }

        public static bool UnboxBool(object obj, out bool res)
        {
            res = default(bool);
            if (obj == null)
            {
                return false;
            }
          
            if (obj is bool)
            {
                res = (bool)obj;
              
                return true;
            }

            PdfBoolOld b = obj as PdfBoolOld;
            if (b != null)
            {
                b.Load();
                res = b.Value;
                
                return true;
            }

            return false;
        }

        public static bool UnboxInt(object obj, out int res)
        {
            res = 0;
            if (obj == null)
            {
                return false;
            }

            if (obj is byte)
            {
                res = (byte)obj;
               
                return true;
            }

            if (obj is int)
            {
                res = (int)obj;

                return true;
            }

            if (obj is double)
            {
                res = (int)(double)obj;
             
                return true;
            }

            PdfIntOld i = obj as PdfIntOld;
            if (i != null)
            {
                i.Load();
                res = i.Value;
                
                return true;
            }

            PdfRealOld r = obj as PdfRealOld;
            if (r != null)
            {
                r.Load();
                res = (int)r.Value;
                
                return true;
            }
            
            return false;
        }

        public static bool EnumTryParse<TEnum>(string valueAsString, out TEnum value) where TEnum : struct
        {
            return EnumTryParse<TEnum>(valueAsString, out value, false);
        }

        public static bool EnumTryParse<TEnum>(string valueAsString, out TEnum value, bool ignoreCase) where TEnum : struct
        {
#if SILVERLIGHT
            return Enum.TryParse<TEnum>(valueAsString, ignoreCase, out value);
#elif WPF
            try
            {
                value = (TEnum)Enum.Parse(typeof(TEnum), valueAsString, ignoreCase);

                return true;
            }
            catch
            {
                value = default(TEnum);

                return false;
            }
#endif
        }
       
        public static Rect GetBoundingRect(Rect rect, Matrix matrix)
        {
            if (matrix.IsIdentity())
            {
                return rect;
            }

            Point[] points = new Point[4];
            points[0] = new Point(rect.Left, rect.Top);
            points[1] = new Point(rect.Right, rect.Top);
            points[2] = new Point(rect.Right, rect.Bottom);
            points[3] = new Point(rect.Left, rect.Bottom);

            TransformPoints(matrix, points);

            double left = Math.Min(Math.Min(points[0].X, points[1].X), Math.Min(points[2].X, points[3].X));
            double right = Math.Max(Math.Max(points[0].X, points[1].X), Math.Max(points[2].X, points[3].X));
            double top = Math.Min(Math.Min(points[0].Y, points[1].Y), Math.Min(points[2].Y, points[3].Y));
            double bottom = Math.Max(Math.Max(points[0].Y, points[1].Y), Math.Max(points[2].Y, points[3].Y));

            return new Rect(new Point(left, top), new Point(right, bottom));
        }

        private static void TransformPoints(Matrix matrix, Point[] points)
        {
            for (int i = 0; i < points.Length; i++)
            {
                Point res = matrix.Transform(points[i]);

                points[i].X = res.X;
                points[i].Y = res.Y;
            }
        }
    }
}
