﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.PdfConverters;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils
{
    internal abstract class PdfPropertyBase<T> : IPdfProperty where T : PdfObjectOld
    {
        #region Constructors

        public PdfPropertyBase(PdfContentManager contentManager, PdfPropertyDescriptor descriptor)
        {
            this.contentManager = contentManager;
            this.descriptor = descriptor;
        }

        public PdfPropertyBase(PdfContentManager contentManager, PdfPropertyDescriptor descriptor, IConverter converter)
            : this(contentManager, descriptor)
        {
            this.converter = converter;
        }

        #endregion


        #region Fields
        
        private readonly PdfContentManager contentManager;
        private readonly IConverter converter;
        private readonly PdfPropertyDescriptor descriptor;

        #endregion


        #region Properties

        protected PdfContentManager ContentManager
        {
            get
            {
                return this.contentManager;
            }
        }



        public PdfPropertyDescriptor Descriptor
        {
            get
            {
                return this.descriptor;
            }
        }

        #endregion


        #region Methods

        protected T CreateValue(object source)
        {
            IndirectReferenceOld ir = source as IndirectReferenceOld;
            if (ir != null && (this.converter == null || !this.converter.HandlesIndirectReference))
            {
                return Converters.IndirectReferenceToPdfObjectConverter.Convert<T>(this.contentManager, ir);
            }
            
            if (this.converter != null)
            {
                object convertedValue = this.converter.Convert(typeof(T), this.contentManager, source);
                return (T)convertedValue;
            }

            T s = source as T;
            if (s != null)
            {
                return s;
            }

            PdfDictionaryOld dictionary = source as PdfDictionaryOld;
            if (dictionary != null)
            {
                return (T)Converters.PdfDictionaryToPdfObjectConverter.Convert(typeof(T), this.contentManager, dictionary);
            }
            
            return default(T);
        }

        public abstract T GetValue();

        public abstract bool SetValue(object value);

        #endregion
    }
}
