﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class ClippingWriter : ContentElementWriter<Clipping>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, Clipping clipping)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(clipping, "element");

            using (ContentStreamOperators.PushGraphicsState(writer, context))
            {
                if (clipping.Clip != null)
                {
                    bool shouldExportTransform = !clipping.Transform.IsIdentity && clipping.Transform.HasInverseMatrix();
                    if (shouldExportTransform)
                    {
                        ContentStreamOperators.ConcatMatrixOperator.Write(writer, context, new MatrixPosition(clipping.Transform));
                    }

                    ContentElementWriters.ClippingGeometryWriter.Write(writer, context, clipping.Clip);

                    if (shouldExportTransform)
                    {
                        ContentStreamOperators.ConcatMatrixOperator.Write(writer, context, new MatrixPosition(clipping.Transform.InverseMatrix()));
                    }
                }

                IEnumerable<ContentElementBase> clippingChildren = context.GetClippingChildren(clipping);
                foreach (ContentElementBase contentElement in clippingChildren)
                {
                    ContentElementWriterBase.WriteElement(writer, context, contentElement);
                }
            }
        }
    }
}
