﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class ColorWriter : ColorWriterBase
    {
        protected override void WriteRgbColor(PdfWriter writer, IPdfContentExportContext context, RgbColor color)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "color");

            if (color.A != RgbColor.DefaultAlphaValue)
            {
                ExtGState extGState = new ExtGState()
                {
                    AlphaConstant = color.A / 255.0,
                    AlphaSource = false
                };

                Operators.ContentStreamOperators.SetGraphicsStateDictionaryOperator.Write(writer, context, context.GetResource(extGState).ResourceKey);
            }

            Operators.ContentStreamOperators.SetRgbColorOperator.Write(writer, context, color);
        }

        protected override void WritePatternColor(PdfWriter writer, IPdfContentExportContext context, PatternColor color)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "color");

            if (color.PatternType == PatternType.Tiling)
            {
                TilingBase tiling = (TilingBase)color;
                if (tiling.PaintType == PaintType.Uncolored)
                {
                    this.WriteUncoloredTiling(writer, context, (UncoloredTiling)tiling);
                    return;
                }
            }

            Operators.ContentStreamOperators.SetColorSpace.Write(writer, context, color.ColorSpace);
            Operators.ContentStreamOperators.SetColorN.Write(writer, context, context.GetResource(color).ResourceKey);
        }

        private void WriteUncoloredTiling(PdfWriter writer, IPdfContentExportContext context, UncoloredTiling uncoloredTiling)
        {
            Operators.ContentStreamOperators.SetColorSpace.Write(writer, context, context.GetResource(uncoloredTiling.ColorSpace, uncoloredTiling.Color.ColorSpace).ResourceKey);

            ColorSpaceObject colorSpaceObject = ColorSpaceManager.CreateColorSpaceObject(uncoloredTiling.Color.ColorSpace);
            ColorObjectBase color = colorSpaceObject.CreateColorObject(context, uncoloredTiling.Color);
            ResourceEntry entry = context.GetResource(uncoloredTiling);

            Operators.ContentStreamOperators.SetColorN.Write(writer, context, color, entry.ResourceKey);
        }
    }
}
