﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal static class ContentElementWriters
    {
        private readonly static Dictionary<Type, ContentElementWriterBase> elementTypeToElementWriter;

        internal readonly static PathWriter PathWriter;
        internal readonly static ColorWriterBase ColorWriter;
        internal readonly static ColorWriterBase StrokeColorWriter;
        internal readonly static SegmentWriter SegmentWriter;
        internal readonly static TextFragmentWriter TextFragmentWriter;
        internal readonly static ContentRootWriter ContentRootWriter;
        internal readonly static ImageWriter ImageWriter;
        internal readonly static FormWriter FormWriter;
        internal readonly static GeometryWriterBase GeometryWriter;
        internal readonly static ClippingGeometryWriter ClippingGeometryWriter;
        internal readonly static ClippingWriter ClippingWriter;
        internal readonly static ContainerWriter ContainerWriter;
        internal readonly static TextPropertiesWriter TextPropertiesWriter;
        internal readonly static GeometryPropertiesWriter GeometryPropertiesWriter;

        static ContentElementWriters()
        {
            elementTypeToElementWriter = new Dictionary<Type, ContentElementWriterBase>();
            PathWriter = new PathWriter();
            ColorWriter = new ColorWriter();
            SegmentWriter = new SegmentWriter();
            TextFragmentWriter = new TextFragmentWriter();
            ContentRootWriter = new ContentRootWriter();
            ImageWriter = new ImageWriter();
            FormWriter = new FormWriter();
            GeometryWriter = new GeometryWriterBase();
            ClippingGeometryWriter = new ClippingGeometryWriter();
            ClippingWriter = new ClippingWriter();
            StrokeColorWriter = new StrokeColorWriter();
            ContainerWriter = new ContainerWriter();
            TextPropertiesWriter = new TextPropertiesWriter();
            GeometryPropertiesWriter = new GeometryPropertiesWriter();

            RegisterDefaultElementWriters();
        }

        private static void RegisterDefaultElementWriters()
        {
            RegisterElementWriter(typeof(Path), PathWriter);
            RegisterElementWriter(typeof(RgbColor), ColorWriter);
            RegisterElementWriter(typeof(Clipping), ClippingWriter);
            RegisterElementWriter(typeof(PathGeometry), GeometryWriter);
            RegisterElementWriter(typeof(RectangleGeometry), GeometryWriter);
            RegisterElementWriter(typeof(BezierSegment), SegmentWriter);
            RegisterElementWriter(typeof(LineSegment), SegmentWriter);
            RegisterElementWriter(typeof(TextFragment), TextFragmentWriter);
            RegisterElementWriter(typeof(RadFixedPage), ContentRootWriter);
            RegisterElementWriter(typeof(FormSource), ContentRootWriter);
            RegisterElementWriter(typeof(Image), ImageWriter);
            RegisterElementWriter(typeof(Form), FormWriter);
            RegisterElementWriter(typeof(Tiling), ContainerWriter);
            RegisterElementWriter(typeof(UncoloredTiling), ContainerWriter);
            RegisterElementWriter(typeof(TextPropertiesOwner), TextPropertiesWriter);
            RegisterElementWriter(typeof(GeometryPropertiesOwner), GeometryPropertiesWriter);
        }

        public static bool TryGetWriter(object element, out ContentElementWriterBase writer)
        {
            return elementTypeToElementWriter.TryGetValue(element.GetType(), out writer);
        }

        private static void RegisterElementWriter(Type type, ContentElementWriterBase writer)
        {
            elementTypeToElementWriter.Add(type, writer);
        }
    }
}
