﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class ContentRootWriter : ContentElementWriter<IContentRootElement>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, IContentRootElement element)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");
            Guard.ThrowExceptionIfNotEqual(context.ContentRoot, element, "context.ContentRoot");

            ContentStreamOperators.ConcatMatrixOperator.Write(writer, context, context.RootDipToPointTransformation);

            foreach (ContentElementBase contentElement in context.GetRootElements())
            {
                Clipping clipping = contentElement as Clipping;

                if (clipping != null)
                {
                    ContentElementWriterBase.WriteElement(writer, context, clipping);
                }
                else
                {
                    ContentElementWriterBase.WriteElement(writer, context, contentElement);
                }
            }

            MarkedContentExportInfo exportInfo = context.GetCurrentContentMarkerExportInfo(null);

            if (exportInfo.ShouldExportMarkedContentEnd)
            {
                ContentStreamOperators.EndMarkedContentOperator.Write(writer, context.Owner);
            }
        }
    }
}
