﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class GeometryWriterBase : ContentElementWriter<GeometryBase>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, GeometryBase element)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            PathGeometry pathGeometry = element as PathGeometry;
            if (pathGeometry != null)
            {
                this.WritePathGeometry(writer, context, pathGeometry);
                return;
            }

            RectangleGeometry rectangleGeometry = element as RectangleGeometry;
            if (rectangleGeometry != null)
            {
                this.WriteRectangleGeometry(writer, context, rectangleGeometry);
                return;
            }
        }

        protected virtual void WritePathGeometry(PdfWriter writer, IPdfContentExportContext context, PathGeometry pathGeometry)
        {
            foreach (PathFigure pathFigure in pathGeometry.Figures)
            {
                this.WritePathFigure(writer, context, pathFigure);
            }
        }

        protected virtual void WriteRectangleGeometry(PdfWriter writer, IPdfContentExportContext context, RectangleGeometry rectangleGeometry)
        {
            Operators.ContentStreamOperators.AppendRectangleOperator.Write(writer, context, rectangleGeometry);
        }

        protected void WritePathFigure(PdfWriter writer, IPdfContentExportContext context, PathFigure pathFigure)
        {
            Operators.ContentStreamOperators.MoveToOperator.Write(writer, context, pathFigure.StartPoint);

            Point lastPoint = pathFigure.StartPoint;
            foreach (PathSegment segment in pathFigure.Segments)
            {
                ArcSegment arcSegment = segment as ArcSegment;
                QuadraticBezierSegment quadraticBezierSegment = segment as QuadraticBezierSegment;

                if (arcSegment != null)
                {
                    foreach (BezierSegment bezierSegment in arcSegment.ToBezierSegments(lastPoint))
                    {
                        ContentElementWriterBase.WriteElement(writer, context, bezierSegment);
                    }
                }
                else if (quadraticBezierSegment != null)
                {
                    BezierSegment bezierSegment = quadraticBezierSegment.ToBezierSegment(lastPoint);
                    ContentElementWriterBase.WriteElement(writer, context, bezierSegment);
                }
                else
                {
                    ContentElementWriterBase.WriteElement(writer, context, segment);
                }

                lastPoint = segment.LastPoint;
            }

            if (pathFigure.IsClosed)
            {
                Operators.ContentStreamOperators.ClosePathOperator.Write(writer, context);
            }
        }
    }
}
