﻿using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class PathWriter : MarkableContentElementWriter<Path>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, Path element)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            using (Operators.ContentStreamOperators.PushGraphicsState(writer, context))
            {
                Operators.ContentStreamOperators.ConcatMatrixOperator.Write(writer, context, element.Position);

                ContentElementWriters.GeometryPropertiesWriter.Write(writer, context, element.GeometryProperties);

                ContentElementWriterBase.WriteElement(writer, context, element.Geometry);
                PathGeometry pathGeometry = element.Geometry as PathGeometry;
                if (element.IsFilled && element.IsStroked)
                {
                    if (pathGeometry == null || pathGeometry.FillRule == FillRule.Nonzero)
                    {
                        Operators.ContentStreamOperators.FillAndStrokePathOperator.Write(writer, context);
                    }
                    else
                    {
                        Operators.ContentStreamOperators.FillAndStrokePathEvenOddOperator.Write(writer, context);
                    }
                }
                else if (element.IsFilled)
                {
                    if (pathGeometry == null || pathGeometry.FillRule == FillRule.Nonzero)
                    {
                        Operators.ContentStreamOperators.FillPathOperator.Write(writer, context);
                    }
                    else
                    {
                        Operators.ContentStreamOperators.FillPathEvenOddOperator.Write(writer, context);
                    }
                }
                else if (element.IsStroked)
                {
                    Operators.ContentStreamOperators.StrokePathOperator.Write(writer, context);
                }
                else
                {
                    Operators.ContentStreamOperators.EndPathOperator.Write(writer, context);
                }
            }
        }
    }
}
