﻿using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class SegmentWriter : ContentElementWriter<PathSegment>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, PathSegment element)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            BezierSegment bezierSegment = element as BezierSegment;
            if (bezierSegment != null)
            {
                this.WriteBezierSegment(writer, context, bezierSegment);

                return;
            }

            LineSegment lineSegment = element as LineSegment;
            if (lineSegment != null)
            {
                this.WriteLineSegment(writer, context, lineSegment);

                return;
            }
        }

        private void WriteLineSegment(PdfWriter writer, IPdfContentExportContext context, LineSegment lineSegment)
        {
            Operators.ContentStreamOperators.LineToOperator.Write(writer, context, lineSegment);
        }

        private void WriteBezierSegment(PdfWriter writer, IPdfContentExportContext context, BezierSegment bezierSegment)
        {
            Operators.ContentStreamOperators.CurveToOperator.Write(writer, context, bezierSegment);
        }
    }
}
