﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class StrokeColorWriter : ColorWriterBase
    {
        protected override void WriteRgbColor(PdfWriter writer, IPdfContentExportContext context, Fixed.Model.ColorSpaces.RgbColor rgbColor)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(rgbColor, "rgbColor");

            if (rgbColor.A != RgbColor.DefaultAlphaValue)
            {
                ExtGState extGState = new ExtGState()
                {
                    StrokeAlphaConstant = rgbColor.A / 255.0,
                    AlphaSource = false
                };

                Operators.ContentStreamOperators.SetGraphicsStateDictionaryOperator.Write(writer, context, context.GetResource(extGState).ResourceKey);
            }

            Operators.ContentStreamOperators.SetStrokeRgbColorOperator.Write(writer, context, rgbColor);
        }

        protected override void WritePatternColor(PdfWriter writer, IPdfContentExportContext context, PatternColor color)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "color");

            if (color.PatternType == PatternType.Tiling)
            {
                TilingBase tiling = color as TilingBase;
                if (tiling.PaintType == PaintType.Uncolored)
                {
                    this.WriteUncoloredTiling(writer, context, tiling as UncoloredTiling);
                    return;
                }
            }

            Operators.ContentStreamOperators.SetStrokeColorSpace.Write(writer, context, color.ColorSpace);
            Operators.ContentStreamOperators.SetStrokeColorN.Write(writer, context, context.GetResource(color).ResourceKey);
        }

        private void WriteUncoloredTiling(PdfWriter writer, IPdfContentExportContext context, UncoloredTiling uncoloredTiling)
        {
            ResourceEntry entry = context.GetResource(uncoloredTiling.ColorSpace, uncoloredTiling.Color.ColorSpace);
            Operators.ContentStreamOperators.SetStrokeColorSpace.Write(writer, context, entry.ResourceKey);

            ColorSpaceObject colorSpaceObject = ColorSpaceManager.CreateColorSpaceObject(uncoloredTiling.Color.ColorSpace);
            ColorObjectBase color = colorSpaceObject.CreateColorObject(context, uncoloredTiling.Color);
            ResourceEntry tilingEntry = context.GetResource(uncoloredTiling);
            Operators.ContentStreamOperators.SetStrokeColorN.Write(writer, context, color, tilingEntry.ResourceKey);
        }
    }
}
