﻿using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Text;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters
{
    internal class TextFragmentWriter : MarkableContentElementWriter<TextFragment>
    {
        protected override void WriteOverride(PdfWriter writer, IPdfContentExportContext context, TextFragment element)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            if (element.TextCollection.Characters.Any())
            {
                using (Operators.ContentStreamOperators.PushGraphicsState(writer, context))
                {
                    Matrix graphicMatrix = element.TextMatrix.InverseMatrix()
                        .MultiplyBy(ShowText.FlipTextTransformation)
                        .MultiplyBy(element.Position.Matrix);
                    Operators.ContentStreamOperators.ConcatMatrixOperator.Write(writer, context, graphicMatrix);

                    Operators.ContentStreamOperators.BeginTextOperator.Write(writer, context);

                    ContentElementWriters.TextPropertiesWriter.Write(writer, context, element.TextProperties);

                    Operators.ContentStreamOperators.ShowTextOperator.Write(writer, context, element.TextCollection);

                    context.SetUsedCharacters(element.Font, element.TextCollection);

                    Operators.ContentStreamOperators.EndTextOperator.Write(writer, context);
                }
            }
        }
    }
}
