﻿using System;
using System.Collections.Generic;
using System.Globalization;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export
{
    internal class FlagWriter<T>
        where T : struct, IConvertible
    {
        private int flags;

        public FlagWriter()
        {
            this.flags = 0;
        }

        public FlagWriter(IEnumerable<int> reservedSetFlagNumbers)
            : this()
        {
            foreach (int bitNumber in reservedSetFlagNumbers)
            {
                this.SetBit(bitNumber);
            }
        }

        public int ResultFlags
        {
            get
            {
                return this.flags;
            }
        }

        public void SetFlagOnCondition(T flag, bool shouldSetFlag)
        {
            if (shouldSetFlag)
            {
                this.SetFlag(flag);
            }
        }

        public void SetFlag(T flag)
        {
            int bitNumber = flag.ToInt32(CultureInfo.InvariantCulture);
            this.SetBit(bitNumber);
        }

        private void SetBit(int bitNumber)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 32, bitNumber, "bitNumber");

            int bitIndex = bitNumber - 1;
            this.flags |= (1 << bitIndex);
        }
    }
}
