﻿using System;
using System.ComponentModel;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export
{
    /// <summary>
    /// Represents PDF export settings.
    /// </summary>
    public class PdfExportSettings
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PdfExportSettings" /> class.
        /// </summary>
        public PdfExportSettings()
        {
            this.IsEncrypted = false;
            this.OwnerPassword = FixedDocumentDefaults.Password;
            this.ImageQuality = FixedDocumentDefaults.ImageQuality;
            this.ComplianceLevel = PdfComplianceLevel.None;
        }

        /// <summary>
        /// Specifies if the PDF document should be encrypted.
        /// </summary>
        /// <value>True if the document should be encrypted.</value>
        public bool IsEncrypted { get; set; }

        /// <summary>
        /// Gets or sets the user password.
        /// </summary>
        /// <value>The user password.</value>
        public string UserPassword { get; set; }

        /// <summary>
        /// Gets or sets the default image quality when exporting images to PDF. The default value is High.
        /// <para>
        /// The value of this property will be overriden when specifying the ImageQuality in ImageSource constructor or when creating ImageSource from EncodedImageData.
        /// The higher the quality, the bigger the document size.
        /// </para>
        /// </summary>
        /// <value>The image quality.</value>
        public ImageQuality ImageQuality { get; set; }

        /// <summary>
        /// Gets or sets the PDF compliance level.
        /// </summary>
        /// <value>The compliance level.</value>
        public PdfComplianceLevel ComplianceLevel { get; set; }

        /// <summary>
        /// Gets or sets the owner password.
        /// </summary>
        /// <value>The owner password.</value>
        internal string OwnerPassword { get; set; }
    }
}
