﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing; 
using Telerik.Windows.Documents.Licensing;
using Telerik.Windows.Documents.Utilities; 

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export
{
    internal class RadFixedDocumentLicenseCheck : IDisposable
    {
        private readonly RadFixedDocument document;
        private int additionalFragmentsCount = 0;
        private bool isDisposed;
        
        public RadFixedDocumentLicenseCheck(RadFixedDocument document)
        {
            Guard.ThrowExceptionIfNull(document, "document");

            this.document = document;

            string message;
            if (RadFixedDocumentLicenseCheck.TryGetLicenseMessage(out message))
            {
                RadFixedPage firstPage = this.document.Pages.FirstOrDefault();

                if (firstPage != null)
                {
                    int currentElementsCount = firstPage.Content.Count;
                    RadFixedDocumentLicenseCheck.AddLicenceMessageContent(firstPage, message);
                    this.additionalFragmentsCount = firstPage.Content.Count - currentElementsCount; 
                }
            }
        }

        public static bool TryGetLicenseMessage(out string message)
        {
            LicensingMode licensingMode = TelerikLicense.GetLicensingMode();

            switch (licensingMode)
            {
                case LicensingMode.Trial:
                    message = TrialMessages.DocumentProcessingTrialMessage;
                    break;
                case LicensingMode.Unlicensed:
                    message = TrialMessages.DocumentProcessingUnlicencedMessage;
                    break;
                default:
                    message = null;
                    break;
            }

            return !string.IsNullOrEmpty(message);
        }

        public static void AddLicenceMessageContent(RadFixedPage page, string message)
        {
            var editor = new FixedContentEditor(page);

            editor.TextProperties.FontSize = 8;

            editor.GraphicProperties.FillColor = new RgbColor() { R = 255 };
            editor.Position.Translate(2, 2);

            editor.DrawText(message);
        }

        public void Dispose()
        {
            if (!this.isDisposed)
            {
                if (this.additionalFragmentsCount > 0)
                { 
                    var firstPage = document.Pages.FirstOrDefault();
                    if (firstPage != null)
                    {
                        while (this.additionalFragmentsCount > 0)
                        {
                            var lastElement = firstPage.Content.LastOrDefault();
                            if (lastElement != null)
                            {
                                firstPage.Content.Remove(lastElement);
                                this.additionalFragmentsCount--;
                            }
                            else
                            {
                                break;
                            }
                        }
                    }
                }

                this.isDisposed = true;
            }
        }
    }
}
