﻿#region PDFsharp - A .NET library for processing PDF
//
// Authors:
//   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
//
// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters
{
    /// <summary>
    /// Implements the ASCIIHexDecode filter.
    /// </summary>
    internal class ASCIIHexDecode : IPdfFilter
    {
        private const byte Eod = (byte)'>';

        /// <summary>
        /// Encodes the specified data.
        /// </summary>
        public byte[] Encode(PdfObject encodedObject, byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            int count = data.Length;
            byte[] bytes = new byte[2 * count];
            for (int i = 0, j = 0; i < count; i++)
            {
                byte b = data[i];
                bytes[j++] = (byte)((b >> 4) + ((b >> 4) < 10 ? (byte)'0' : (byte)('A' - 10)));
                bytes[j++] = (byte)((b & 0xF) + ((b & 0xF) < 10 ? (byte)'0' : (byte)('A' - 10)));
            }
            return bytes;
        }

        /// <summary>
        /// Decodes the specified data.
        /// </summary>
        public byte[] Decode(PdfObject decodedObject, Stream stream, DecodeParameters parms)
        {
            throw new NotSupportedException();
        }

        public string Name
        {
            get { return PdfFilterNames.ASCIIHexDecode; }
        }

        public byte[] Decode(PdfObject decodedObject, byte[] inputData, DecodeParameters parms)
        {
            inputData = RemoveWhiteSpace(inputData);

            List<byte> result = new List<byte>(inputData.Length);
            byte currentByte;
            int index = 0;
            while (index < inputData.Length && (currentByte = inputData[index++]) != Eod)
            {
                byte nextByte = inputData[index++];
                if (nextByte == Eod)
                {
                    nextByte = (byte)'0';
                }

                byte hi = BytesHelper.GetDecimalFromHex((char)currentByte);
                byte lo = BytesHelper.GetDecimalFromHex((char)nextByte);

                byte byteValue = (byte)((hi << 4) | lo);
                result.Add(byteValue);
            }

            return result.ToArray();
        }

        private static byte[] RemoveWhiteSpace(byte[] data)
        {
            List<byte> result = new List<byte>(data.Length);
            for (int i = 0; i < data.Length; i++)
            {
                if (!char.IsWhiteSpace((char)data[i]))
                {
                    result.Add(data[i]);
                }
            }

            return result.ToArray();
        }
    }
}
