﻿using BitMiracle.LibTiff.Classic;
using System;
using System.IO;
using Telerik.Windows.Documents.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters
{
    internal class CCITTFaxDecode : IPdfFilter
    {
        private const int White = unchecked((int)0xFFFFFFFF);
        private const int Black = unchecked((int)0xFF000000);

        public string Name
        {
            get { return PdfFilterNames.CCITTFaxDecode; }
        }

        public byte[] Encode(PdfObject encodedObject, byte[] data)
        {
            throw new NotSupportedException();
        }

        public byte[] Decode(PdfObject decodedObject, byte[] inputData, DecodeParameters parms)
        {
            int columns = 1728;
            int rows = 0;
            int k = 0;
            bool blackIs1 = false;
            if (parms != null)
            {
                if (parms.ContainsKey("Columns"))
                {
                    columns = (int)parms["Columns"];
                }

                if (columns == 0)
                {
                    columns = decodedObject.Width;
                }

                if (parms.ContainsKey("Rows"))
                {
                    rows = (int)parms["Rows"];
                }

                if (rows == 0)
                {
                    rows = decodedObject.Height;
                }

                if (parms.ContainsKey("K"))
                {
                    k = (int)parms["K"];
                }

                if (parms.ContainsKey("BlackIs1"))
                {
                    blackIs1 = (bool)parms["BlackIs1"];
                }
            }

            Compression compression;
            if (k < 0)
            {
                compression = Compression.CCITTFAX4;
            }
            else if (k == 0)
            {
                compression = Compression.CCITTFAX3;
            }
            else
            {
                // Group3 2-D Not supported.
                throw new NotSupportedCCITTFaxDecodeFilterException("CCITTFaxDecode Group3 2-D filter is not supported.");
            }

            ColorSpaceObject colorSpaceObject = decodedObject.GetColorSpaceObject();
            if (colorSpaceObject == null)
            {
                colorSpaceObject = new DeviceGrayColorSpaceObject();
            }

            ColorSpaceBase colorSpaceBase = colorSpaceObject.ToColorSpace();

            TiffStream str = new TiffStream();
            using (MemoryStream memStream = new MemoryStream())
            {
                using (Tiff tiff = Tiff.ClientOpen("myTiff", "w", memStream, str))
                {
                    tiff.SetField(TiffTag.IMAGEWIDTH, columns);
                    tiff.SetField(TiffTag.IMAGELENGTH, rows);
                    tiff.SetField(TiffTag.COMPRESSION, compression);
                    tiff.SetField(TiffTag.BITSPERSAMPLE, 1);
                    tiff.SetField(TiffTag.SAMPLESPERPIXEL, 1);
                    tiff.WriteRawStrip(0, inputData, inputData.Length);
                    tiff.Flush();
                    tiff.FlushData();

                    memStream.Position = 0;
                    using (Tiff readTiff = Tiff.ClientOpen("myTiff", "r", memStream, str))
                    {
                        int[] raster = new int[columns * rows];
                        readTiff.ReadRGBAImage(columns, rows, raster);
                        
                        ColorBase color;

                        int stride = (columns + 7) / 8;
                        byte[] result = new byte[stride * rows];
                        int byteIndex = 0, bitIndex = 0;
                        for (int y = rows - 1; y >= 0; --y)
                        {
                            for (int x = 0; x < columns; ++x)
                            {
                                if (blackIs1)
                                {
                                    if (raster[(y * columns) + x] == Black)
                                    {
                                        color = colorSpaceBase.GetColor(new double[] { 1 });
                                        RgbColor rgbColor = color as RgbColor;

                                        if (rgbColor != null && rgbColor.R != 0)
                                        {
                                            result[byteIndex] |= (byte)(1 << (7 - bitIndex));
                                        }
                                    }
                                    else
                                    {
                                        color = colorSpaceBase.GetColor(new double[] { 0 });
                                        RgbColor rgbColor = color as RgbColor;

                                        if (rgbColor != null && rgbColor.R != 0)
                                        {
                                            result[byteIndex] |= (byte)(1 << (7 - bitIndex));
                                        }
                                    }
                                }
                                else
                                {
                                    if (raster[(y * columns) + x] == White)
                                    {
                                        color = colorSpaceBase.GetColor(new double[] { 1 });
                                        RgbColor rgbColor = color as RgbColor;
                                        if (rgbColor != null && rgbColor.R != 0)
                                        {
                                            result[byteIndex] |= (byte)(1 << (7 - bitIndex));
                                        }
                                    }
                                    else
                                    {
                                        color = colorSpaceBase.GetColor(new double[] { 0 });
                                        RgbColor rgbColor = color as RgbColor;
                                        if (rgbColor != null && rgbColor.R != 0)
                                        {
                                            result[byteIndex] |= (byte)(1 << (7 - bitIndex));
                                        }
                                    }
                                }

                                ++bitIndex;
                                if (bitIndex == 8)
                                {
                                    ++byteIndex;
                                    bitIndex = 0;
                                }
                            }

                            if (bitIndex != 0)
                            {
                                ++byteIndex;
                                bitIndex = 0;
                            }
                        }

                        decodedObject.ColorSpace = ColorSpace.Gray;
                        decodedObject.BitsPerComponent = 1;
                        
                        return result;
                    }
                }
            }
        }
    }
}
