﻿using System;
using System.IO;
using Telerik.Windows.Documents.Utilities;
#if NETSTANDARD
using Telerik.Zip;
#else
using Telerik.Windows.Zip;
#endif

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters
{
    /// <summary>
    /// Represents flate filter.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Flate")]
    public class FlateDecode : IPdfFilter
    {
        /// <summary>
        /// Gets the name of the PDF filter.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return PdfFilterNames.FlateDecode;
            }
        }

        /// <summary>
        /// Decodes the specified data.
        /// </summary>
        /// <param name="decodedObject">The decoded object.</param>
        /// <param name="data">The data.</param>
        /// <param name="parms">Decode parameters</param>
        /// <returns></returns>
        public byte[] Decode(PdfObject decodedObject, byte[] data, DecodeParameters parms)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            if (data.Length == 0)
            {
                return data;
            }

            using (MemoryStream stream = new MemoryStream(data))
            {
                byte[] result = null;

                using (CompressedStream compressedStream = new CompressedStream(stream, StreamOperationMode.Read, new DeflateSettings()))
                {
                    result = compressedStream.ReadAllBytes();
                }

                if (parms != null)
                {
                    result = Predictor.Decode(result, parms);
                }

                return result;
            }
        }

        /// <summary>
        /// Encodes the specified data.
        /// </summary>
        /// <param name="encodedObject">The encoded object.</param>
        /// <param name="data">The data</param>
        /// <returns>The encoded data.</returns>
        public byte[] Encode(PdfObject encodedObject, byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            using (MemoryStream stream = new MemoryStream())
            {
                using (CompressedStream compressedStream = new CompressedStream(stream, StreamOperationMode.Write, new DeflateSettings()))
                {
                    compressedStream.Write(data, 0, data.Length);
                }

                return stream.ReadAllBytes();
            }
        }
    }
}
