﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters
{
    internal static class Predictor
    {
        internal static byte[] Decode(byte[] data, DecodeParameters decodeParameters)
        {
            object tmp;
            int predictor = 1;
            if (decodeParameters.TryGetValue("Predictor", out tmp))
            {
                predictor = (int)tmp;
            }
            int colors = 1;
            if (decodeParameters.TryGetValue("Colors", out tmp))
            {
                colors = (int)tmp;
            }
            int bpc = 8;
            if (decodeParameters.TryGetValue("BitsPerComponent", out tmp))
            {
                bpc = (int)tmp;
            }
            int columns = 1;
            if (decodeParameters.TryGetValue("Columns", out tmp))
            {
                columns = (int)tmp;
            }
            switch (predictor)
            {
                case 1:
                    return data;
                case 2:
                    return TiffPredictor(data, colors, columns);
                case 10:
                case 11:
                case 12:
                case 13:
                case 14:
                case 15:
                    return PngPretictor(data, columns, colors, bpc);
            }
            return null;
        }

        private static byte[] TiffPredictor(byte[] data, int colors, int columns)
        {
            int rows = data.Length / (columns * colors);
            for (int i = 0; i < rows; i++)
            {
                int index = colors * ((i * columns) + 1);
                for (int j = colors; j < (columns * colors); j++)
                {
                    data[index] = (byte)(data[index] + data[index - colors]);
                    index++;
                }
            }
            return data;
        }

        private static byte[] PngPretictor(byte[] data, int columns, int colors, int bpc)
        {
            List<byte[]> rows = new List<byte[]>();
            int rowSize = columns * colors * bpc;
            rowSize = (int)Math.Ceiling(rowSize / 8.0);
            int sub = (int)Math.Ceiling((bpc * colors) / 8.0);
            byte[] prevLine = null;
            byte[] curLine = null;

            ByteReader reader = new ByteReader(data);

            while (reader.Remaining >= rowSize + 1)
            {
                int algorithm = (reader.ReadByte() & 0xff);

                curLine = new byte[rowSize];
                reader.Read(curLine);
                switch (algorithm)
                {
                    case 0:
                        break;
                    case 1:
                        Sub(curLine, sub);
                        break;
                    case 2:
                        Up(curLine, prevLine);
                        break;
                    case 3:
                        Average(curLine, prevLine, sub);
                        break;
                    case 4:
                        Paeth(curLine, prevLine, sub);
                        break;
                }

                rows.Add(curLine);
                prevLine = curLine;
            }

            List<byte> res = new List<byte>();
            foreach (var r in rows)
            {
                res.AddRange(r);
            }
            return res.ToArray();
        }

        private static void Sub(byte[] curLine, int sub)
        {
            for (int i = 0; i < curLine.Length; i++)
            {
                int prevIdx = i - sub;
                if (prevIdx >= 0)
                {
                    curLine[i] += curLine[prevIdx];
                }
            }
        }

        private static void Up(byte[] curLine, byte[] prevLine)
        {
            if (prevLine == null)
            {
                return;
            }
            for (int i = 0; i < curLine.Length; i++)
            {
                curLine[i] += prevLine[i];
            }
        }

        private static void Average(byte[] curLine, byte[] prevLine, int sub)
        {
            for (int i = 0; i < curLine.Length; i++)
            {
                int raw = 0;
                int prior = 0;
                int prevIdx = i - sub;
                if (prevIdx >= 0)
                {
                    raw = curLine[prevIdx] & 0xff;
                }
                if (prevLine != null)
                {
                    prior = prevLine[i] & 0xff;
                }
                curLine[i] += (byte)Math.Floor((raw + prior) / 2.0);
            }
        }

        private static void Paeth(byte[] curLine, byte[] prevLine, int sub)
        {
            for (int i = 0; i < curLine.Length; i++)
            {
                int left = 0;
                int up = 0;
                int upLeft = 0;
                int prevIdx = i - sub;
                if (prevIdx >= 0)
                {
                    left = curLine[prevIdx] & 0xff;
                }
                if (prevLine != null)
                {
                    up = prevLine[i] & 0xff;
                }
                if (prevIdx > 0 && prevLine != null)
                {
                    upLeft = prevLine[prevIdx] & 0xff;
                }
                curLine[i] += (byte)Paeth(left, up, upLeft);
            }
        }

        private static int Paeth(int left, int up, int upLeft)
        {
            int p = left + up - upLeft;
            int pa = Math.Abs(p - left);
            int pb = Math.Abs(p - up);
            int pc = Math.Abs(p - upLeft);
            if ((pa <= pb) && (pa <= pc))
            {
                return left;
            }
            else if (pb <= pc)
            {
                return up;
            }
            else
            {
                return upLeft;
            }
        }
    }
}