﻿using System;
using System.Collections.Generic;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class FieldsAndWidgetsPropertiesImportContext : RadFixedDocumentImportContext
    {
        private readonly List<FormField> fields;
        private readonly List<Widget> widgets;

        public FieldsAndWidgetsPropertiesImportContext(PdfImportSettings settings)
            : base(settings)
        {
            this.fields = new List<FormField>();
            this.widgets = new List<Widget>();
        }

        public FormField GetField(int index)
        {
            return this.fields[index];
        }

        public Widget GetWidget(int index)
        {
            return this.widgets[index];
        }

        protected override void BeginImportOverride()
        {
            DocumentCatalog catalog = this.Root.GetValue();

            if (catalog.AcroForm != null)
            {
                this.Document.AcroForm.ViewersShouldRecalculateWidgetAppearances = catalog.AcroForm.NeedAppearance.Value;

                foreach (FormFieldNode fieldNode in catalog.AcroForm.Fields)
                {
                    bool importFieldProperties = fieldNode.FieldType.Value != FormFieldTypes.Signature;
                    FormField field = fieldNode.ToFormField(this.Reader, this, importFieldProperties);
                    this.Document.AcroForm.FormFields.Add(field);
                    this.fields.Add(field);

                    foreach (WidgetObject widgetObject in this.GetChildWidgets(fieldNode))
                    {
                        double pageHeight = 0;
                        Page page = widgetObject.ParentPage;

                        if (page != null)
                        {
                            Rect pageBox = page.MediaBox.ToDipRect(this.Reader, this);
                            pageHeight = pageBox.Height;
                        }

                        Widget widget = (Widget)widgetObject.ToAnnotation(this.Reader, this, pageHeight);

                        this.widgets.Add(widget);
                    }
                }
            }
        }
    }
}
