﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class PagesContentCache : IPagesContentCache
    {
        private readonly Dictionary<RadFixedPage, Tuple<ContentElementCollection, AnnotationCollection>> pageToSavedPageContent;

        public PagesContentCache()
        {
            this.pageToSavedPageContent = new Dictionary<RadFixedPage, Tuple<ContentElementCollection, AnnotationCollection>>();
        }

        public void CachePageContent(RadFixedPage page)
        {
            ContentElementCollection coppiedPageContent = new ContentElementCollection(page);
            this.CopyPageContent(page.Content, coppiedPageContent);

            AnnotationCollection coppiedPageAnnotations = new AnnotationCollection(page);
            this.CopyPageAnnotations(page.Annotations, coppiedPageAnnotations);

            this.pageToSavedPageContent[page] = new Tuple<ContentElementCollection, AnnotationCollection>(coppiedPageContent, coppiedPageAnnotations);
        }

        public bool TryLoadCachedPageContent(RadFixedPage page)
        {
            Tuple<ContentElementCollection, AnnotationCollection> savedPageContent;
            bool isCached = this.pageToSavedPageContent.TryGetValue(page, out savedPageContent);

            if (isCached)
            {
                this.CopyPageContent(savedPageContent.Item1, page.Content);
                this.CopyPageAnnotations(savedPageContent.Item2, page.Annotations);
            }

            return isCached;
        }

        public void Clear()
        {
            this.pageToSavedPageContent.Clear();
        }

        private void CopyPageContent(ContentElementCollection source, ContentElementCollection destination)
        {
            foreach (PositionContentElement element in source)
            {
                destination.Add(element);
            }
        }

        private void CopyPageAnnotations(AnnotationCollection source, AnnotationCollection destination)
        {
            foreach (var annotation in source)
            {
                destination.Add(annotation);
            }
        }
    }
}
