﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.GraphicsState;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class PdfContentImportContext : IPdfContentImportContext
    {
        private readonly IRadFixedDocumentImportContext owner;
        private readonly IResourceHolder resourceHolder;
        private readonly IContentRootElement contentRoot;

        public PdfContentImportContext(IRadFixedDocumentImportContext owner, IResourceHolder resourceHolder, IContentRootElement contentRoot)
        {
            Guard.ThrowExceptionIfNull(owner, "owner");
            Guard.ThrowExceptionIfNull(resourceHolder, "resourceHolder");
            Guard.ThrowExceptionIfNull(contentRoot, "contentRoot");

            this.owner = owner;
            this.resourceHolder = resourceHolder;
            this.contentRoot = contentRoot;
        }

        public IRadFixedDocumentImportContext Owner
        {
            get
            {
                return this.owner;
            }
        }

        public IResourceHolder ResourceHolder
        {
            get
            {
                return this.resourceHolder;
            }
        }

        public IContentRootElement ContentRoot
        {
            get
            {
                return this.contentRoot;
            }
        }

        public Marker CurrentMarker { get; set; }

        public FontObject GetFont(PostScriptReader reader, PdfName key)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(key, "key");

            FontObject font;
            if (this.ResourceHolder.Resources != null && this.ResourceHolder.Resources.Fonts != null && this.ResourceHolder.Resources.Fonts.TryGetElement(reader, this.Owner, key.Value, out font))
            {
                return font;
            }
            else
            {
                return null;
            }
        }

        public XObjectBase GetXObject(PostScriptReader reader, PdfName key)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(key, "key");

            XObjectBase xObject;
            this.ResourceHolder.Resources.XObjects.TryGetElement(reader, this.Owner, key.Value, out xObject);

            return xObject;
        }

        public ExtGStateObject GetExtGState(PostScriptReader reader, PdfName key)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(key, "key");

            ExtGStateObject extGState;
            this.ResourceHolder.Resources.ExtGState.TryGetElement(reader, this.Owner, key.Value, out extGState);

            return extGState;
        }

        public PatternColorObject GetPatternColor(PostScriptReader reader, PdfName key)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(key, "key");

            PatternColorObject color;
            this.ResourceHolder.Resources.Patterns.TryGetElement(reader, this.Owner, key.Value, out color);

            return color;
        }

        public ColorSpaceObject GetColorSpace(PostScriptReader reader, PdfName key)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(key, "key");

            ColorSpaceObject colorSpace;
            this.ResourceHolder.Resources.ColorSpaces.TryGetElement(reader, this.Owner, key.Value, out colorSpace);

            return colorSpace;
        }
    }
}
