﻿using System;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    /// <summary>
    /// Represents PDF import settings.
    /// </summary>
    public class PdfImportSettings
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PdfImportSettings" /> class.
        /// </summary>
        public PdfImportSettings()
        {
            this.ReadingMode = ReadingMode.AllAtOnce;
            this.IsPdfViewer = false;
        }

        /// <summary>
        /// Occurs when the user password is needed.
        /// </summary>
        public event EventHandler<PasswordNeededEventArgs> UserPasswordNeeded;

        /// <summary>
        /// Gets or sets the mode for loading page content.
        /// </summary>
        internal ReadingMode ReadingMode { get; set; }

        internal bool IsPdfViewer { get; set; }

        internal string GetUserPassword()
        {
            PasswordNeededEventArgs args = new PasswordNeededEventArgs();
            this.OnUserPasswordNeeded(args);

            return args.Password;
        }

        private void OnUserPasswordNeeded(PasswordNeededEventArgs args)
        {
            if (this.UserPasswordNeeded != null)
            {
                this.UserPasswordNeeded(this, args);
            }
        }
    }
}
