﻿using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Fixed.Exceptions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class PdfImporter
    {
        public void Import(Stream input, IPdfImportContext context)
        {
            Guard.ThrowExceptionIfNull(input, "input");
            Guard.ThrowExceptionIfNull(context, "context");

            context.BeginImport(input);
        }

        internal static bool SeekToStartXRef(Stream input)
        {
            StringBuilder tmp = new StringBuilder();
            StringBuilder tok = new StringBuilder(PdfNames.StartXRef);
            int size = (int)Math.Min(input.Length, Constants.BufferSize);
            byte[] buffer = new byte[size];
            input.Seek(-size, SeekOrigin.End);

            long offset = -1;
            input.Read(buffer, 0, buffer.Length);
            for (int i = size - 1; i >= 0; i--)
            {
#if SILVERLIGHT
                tmp.Insert(0, ((char)buffer[i]).ToString());
#else
                tmp.Insert(0, (char)buffer[i]);
#endif
                if (tok.Length < tmp.Length)
                {
                    tmp.Remove(tmp.Length - 1, 1);
                }

                if (tmp.Equals(tok))
                {
                    offset = size - i;

                    break;
                }
            }

            if (offset == -1)
            {
                return false;
            }

            input.Seek(-offset, SeekOrigin.End);

            return true;
        }

        internal static void ValidateInputStreamCanReadAndSeek(Stream input)
        {
            if (!input.CanRead || !input.CanSeek)
            {
                throw new NotSupportedStreamTypeException(input.CanRead, input.CanSeek);
            }
        }
    }
}
