﻿using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class ThreadSafePageContentManager : IPageContentManager
    {
        private readonly IPageContentManager pageContentManager;
#if SILVERLIGHT
        private readonly object lockObject;
#endif

        public ThreadSafePageContentManager(IPageContentManager pageContentLoader)
        {
            this.pageContentManager = pageContentLoader;
#if SILVERLIGHT
            this.lockObject = new object();
#endif
        }

        public void LoadPageContent(RadFixedPage page)
        {
            if (this.pageContentManager.IsPageContentEmpty(page))
            {
#if SILVERLIGHT
                lock (this.lockObject)
#else
                lock (this.pageContentManager)
#endif
                {
                    if (this.pageContentManager.IsPageContentEmpty(page))
                    {
                        this.pageContentManager.LoadPageContent(page);
                    }
                }
            }
        }

        public void UnloadPageContent(RadFixedPage page)
        {
            if (!this.pageContentManager.IsPageContentEmpty(page))
            {
#if SILVERLIGHT
                lock (this.lockObject)
#else
                lock (this.pageContentManager)
#endif
                {
                    if (!this.pageContentManager.IsPageContentEmpty(page))
                    {
                        this.pageContentManager.UnloadPageContent(page);
                    }
                }
            }
        }

        public bool IsPageContentEmpty(RadFixedPage page)
        {
            return this.pageContentManager.IsPageContentEmpty(page);
        }
    }
}
