﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model
{
    internal static class Characters
    {
        public const char HexStringAndDictionaryStart = '<';
        public const char HexStringAndDictionaryEnd = '>';
        public const char StringEscapeCharacter = '\\';
        public const char LiteralStringStart = '(';
        public const char LiteralStringEnd = ')';
        public const char LineFeed = '\n';
        public const char CarriageReturn = '\r';
        public const char ArrayStart = '[';
        public const char ArrayEnd = ']';
        public const char ProcedureStart = '{';
        public const char ProcedureEnd = '}';
        public const char NameStart = '/';
        public const char Comment = '%';
        public const char NumberCharacter = '#';
        public const byte Zero = (byte)'0';
        public const byte Seven = (byte)'7';

        public static bool IsHexChar(byte b)
        {
            char ch = (char)b;
           
            return ('0' <= ch && ch <= '9') || ('A' <= ch && ch <= 'F') || ('a' <= ch && ch <= 'f');
        }

        public static bool IsOctalChar(byte b)
        {
            return Zero <= b && b <= Seven;
        }

        public static bool IsHexStringOrDictionaryStart(byte b)
        {
            return (char)b == HexStringAndDictionaryStart;
        }

        public static bool IsHexStringOrDictionaryEnd(byte b)
        {
            return (char)b == HexStringAndDictionaryEnd;
        }

        public static bool IsValidNameCharacter(byte b)
        {
            return !(Characters.IsDelimiter(b) || Characters.IsWhiteSpace(b));
        }

        public static bool IsDelimiter(byte b)
        {
            switch ((char)b)
            {
                case LiteralStringStart:
                case LiteralStringEnd:
                case HexStringAndDictionaryEnd:
                case HexStringAndDictionaryStart:
                case ArrayStart:
                case ArrayEnd:
                case ProcedureEnd:
                case ProcedureStart:
                case NameStart:
                case Comment:
                case LineFeed:
                case CarriageReturn:
                    return true;
            }

            return false;
        }

        public static bool IsLiteralStringStart(byte b)
        {
            return (char)b == LiteralStringStart;
        }

        public static bool IsLiteralStringEnd(byte b)
        {
            return (char)b == LiteralStringEnd;
        }

        public static bool IsArrayStart(byte b)
        {
            return (char)b == ArrayStart;
        }

        public static bool IsArrayEnd(byte b)
        {
            return (char)b == ArrayEnd;
        }

        public static bool IsNameStart(byte b)
        {
            return (char)b == NameStart;
        }

        public static bool IsNumberCharacter(byte b)
        {
            char ch = (char)b;

            return char.IsDigit(ch) || PdfHelper.CultureInfo.IsDecimalSeparator(ch);
        }

        public static bool IsCharacter(byte b)
        {
            return ('a' <= b && b <= 'z') || ('A' <= b && b <= 'Z');
        }

        public static bool IsKeywordCharacter(byte b)
        {
            return IsCharacter(b) || b == '*';
        }

        public static bool IsKeywordStartCharacter(byte b)
        {
            return IsCharacter(b) || b == '\'' || b == '"';
        }

        public static bool IsWhiteSpace(byte b)
        {
            return char.IsWhiteSpace((char)b);
        }

        public static bool IsCommentStart(byte b)
        {
            return (char)b == Comment;
        }

        public static bool IsLineFeed(byte b)
        {
            return (char)b == LineFeed;
        }

        public static bool IsCarriageReturn(byte b)
        {
            return (char)b == CarriageReturn;
        }

        public static bool IsSign(byte b)
        {
            char ch = (char)b;

            return '+' == ch || '-' == ch;
        }
    }
}
