﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    internal class AppearanceConverter : IConverter
    {
        public IPdfPrimitive Convert(Type type, PostScriptReader reader, IPdfImportContext context, IPdfPrimitive value)
        {
            if(value == null || value.Type == PdfElementType.Null)
            {
                return null;
            }

            switch(value.Type)
            {
                case PdfElementType.IndirectReference:
                    return this.ConvertFromIndirectReference(type, reader, context, (IndirectReference)value);
                case PdfElementType.IndirectObject:
                    return this.Convert(type, reader, context, ((IndirectObject)value).Content);
                case PdfElementType.Stream:
                    return this.ConvertFromStream(reader, context, (PdfStream)value);
                case PdfElementType.Dictionary:
                    return this.ConvertFromDictionary(reader, context, (PdfDictionary)value);
                default:
                    throw new NotSupportedException(string.Format("Not supported value type: {0}", value.Type));
            }
        }

        private Appearance ConvertFromIndirectReference(Type type, PostScriptReader reader, IPdfImportContext context, IndirectReference indirectReference)
        {
            IPdfPrimitive cachedResult;
            Appearance appearance;

            if (context.TryGetIndirectObject(indirectReference, out cachedResult))
            {
                FormXObject form = cachedResult as FormXObject;
                appearance = form == null ? ((Appearance)cachedResult) : new Appearance(form);
            }
            else
            {
                IPdfPrimitive primitive = context.ReadIndirectObject(indirectReference).Content;
                appearance = (Appearance)this.Convert(type, reader, context, primitive);

                if(appearance.SingleStateAppearance != null)
                {
                    context.RegisterIndirectObject(indirectReference, appearance.SingleStateAppearance);
                }
                else
                {
                    context.RegisterIndirectObject(indirectReference, appearance);
                }
            }

            return appearance;
        }

        private Appearance ConvertFromStream(PostScriptReader reader, IPdfImportContext context, PdfStream pdfStream)
        {
            IConverter xFormConverter = PdfObjectDescriptors.GetPdfObjectDescriptor<FormXObject>().Converter;
            FormXObject form = (FormXObject)xFormConverter.Convert(typeof(FormXObject), reader, context, pdfStream);

            return new Appearance(form);
        }

        private Appearance ConvertFromDictionary(PostScriptReader reader, IPdfImportContext context, PdfDictionary dictionary)
        {
            Dictionary<string, FormXObject> stateToAppearance = new Dictionary<string, FormXObject>();
            IConverter xFormConverter = PdfObjectDescriptors.GetPdfObjectDescriptor<FormXObject>().Converter;

            foreach (KeyValuePair<string, IPdfPrimitive> pair in dictionary)
            {
                FormXObject form = (FormXObject)xFormConverter.Convert(typeof(FormXObject), reader, context, pair.Value);
                stateToAppearance[pair.Key] = form;
            }

            return new Appearance(stateToAppearance);
        }
    }
}
