﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    internal class AppearancesConverter : Converter
    {
        private readonly AppearanceConverter appearanceConverter;

        public AppearancesConverter()
        {
            this.appearanceConverter = new AppearanceConverter();
        }

        protected override IPdfPrimitive ConvertFromDictionary(Type type, PostScriptReader reader, IPdfImportContext context, PdfDictionary dictionary)
        {
            Appearances appearances = new Appearances();

            Appearance normalAppearance;
            if(this.TryImportAppearance(Appearances.NormalAppearanceKey, reader, context, dictionary, out normalAppearance))
            {
                appearances.NormalAppearance = normalAppearance;
            }

            Appearance rolloverAppearance;
            if (this.TryImportAppearance(Appearances.RolloverAppearanceKey, reader, context, dictionary, out rolloverAppearance))
            {
                appearances.RolloverAppearance = rolloverAppearance;
            }

            Appearance downAppearance;
            if (this.TryImportAppearance(Appearances.DownAppearanceKey, reader, context, dictionary, out downAppearance))
            {
                appearances.DownAppearance = downAppearance;
            }

            return appearances;
        }

        private bool TryImportAppearance(string key, PostScriptReader reader, IPdfImportContext context, PdfDictionary appearancesDictionary, out Appearance appearance)
        {
            IPdfPrimitive appearancePrimitive;
            if (appearancesDictionary.TryGetElement(key, out appearancePrimitive))
            {
                appearance = (Appearance)this.appearanceConverter.Convert(typeof(Appearance), reader, context, appearancePrimitive);
                return true;
            }

            appearance = null;
            return false;
        }
    }
}
