﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    internal class DestinationConverter : Converter
    {
        protected override IPdfPrimitive ConvertFromArray(Type type, Import.Parser.PostScriptReader reader, Import.IPdfImportContext context, Types.PdfArray array)
        {
            IPdfPrimitive page;
            IPdfPrimitive pagePrimitive = array[0];
            IndirectReference pageReference = pagePrimitive as IndirectReference;

            if (pageReference == null)
            {
                int pageIndex = ((PdfInt)pagePrimitive).Value;
                page = context.Pages[pageIndex];
            }
            else
            {
                context.TryGetIndirectObject(pageReference, out page);
            }

            DestinationObject destination = new DestinationObject();
            destination.Page = (Page)page;
            destination.DestinationType = (PdfName)array[1];

            switch (destination.DestinationType.Value)
            {
                case DestinationTypes.Location:
                    destination.Left = array[2].GetNumeric().ToPdfReal();
                    destination.Top = array[3].GetNumeric().ToPdfReal();
                    destination.Zoom = array[4].GetNumeric().ToPdfReal();
                    break;

                case DestinationTypes.FitHorizontalToPage:
                    destination.Top = array[2].GetNumeric().ToPdfReal();
                    break;

                case DestinationTypes.FitVerticalToPage:
                    destination.Left = array[2].GetNumeric().ToPdfReal();
                    break;

                case DestinationTypes.FitToRect:
                    destination.Left = array[2].GetNumeric().ToPdfReal();
                    destination.Bottom = array[3].GetNumeric().ToPdfReal();
                    destination.Right = array[4].GetNumeric().ToPdfReal();
                    destination.Top = array[5].GetNumeric().ToPdfReal();
                    break;

                case DestinationTypes.FitHorizontalToBoundingRect:
                    destination.Top = array[2].GetNumeric().ToPdfReal();
                    break;

                case DestinationTypes.FitVerticalToBoundingRect:
                    destination.Left = array[2].GetNumeric().ToPdfReal();
                    break;
            }

            return destination;
        }
    }
}
