﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    internal class PageTypeAwareConverter : TypeAwareConverter<PdfName>
    {
        private const string Kids = "Kids";
        private const string MediaBox = "MediaBox";
        private const string CropBox = "CropBox";
        private const string Parent = "Parent";
        private const string Rotate = "Rotate";

        public PageTypeAwareConverter(Func<PdfName, object> factoryFunction, string typeProperty = "Type") : base(factoryFunction, typeProperty)
        {
        }

        protected override IPdfPrimitive ConvertFromIndirectReference(Type type, PostScriptReader reader, IPdfImportContext context, IndirectReference reference)
        {
            IndirectObject indirectObject = context.ReadIndirectObject(reference);
            IPdfPrimitive primitive = indirectObject.Content;
            bool isLoadOnDemand = context.ImportSettings.ReadingMode == ReadingMode.OnDemand;

            if (isLoadOnDemand)
            {
                PdfDictionary dictionary = primitive as PdfDictionary;
                PageTreeNode pageTreeNode = this.CreateInstance(type, reader, context, dictionary) as PageTreeNode;

                IPdfPrimitive value;
                IPdfProperty property;

                if (dictionary.TryGetElement(Parent, out value) && pageTreeNode.Properties.TryGetValue(Parent, out property))
                {
                    property.SetValue(reader, context, value);
                }

                if (dictionary.TryGetElement(Kids, out value) && pageTreeNode.Properties.TryGetValue(Kids, out property))
                {
                    property.SetValue(reader, context, value);
                }

                if (dictionary.TryGetElement(MediaBox, out value) && pageTreeNode.Properties.TryGetValue(MediaBox, out property))
                {
                    property.SetValue(reader, context, value);
                }

                if (dictionary.TryGetElement(CropBox, out value) && pageTreeNode.Properties.TryGetValue(CropBox, out property))
                {
                    property.SetValue(reader, context, value);
                }

                if (dictionary.TryGetElement(Rotate, out value) && pageTreeNode.Properties.TryGetValue(Rotate, out property))
                {
                    property.SetValue(reader, context, value);
                }

                context.MapPageToPdfDictionary(pageTreeNode, dictionary);

                return pageTreeNode;
            }
            else
            {
                return this.Convert(type, reader, context, primitive);
            }
        }
    }
}
