﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    internal class PatternColorConverter : Converter
    {
        private const int TilingPatternType = 1;

        protected override Types.IPdfPrimitive ConvertFromStream(Type type, Import.Parser.PostScriptReader reader, Import.IPdfImportContext context, Types.PdfStream stream)
        {
            PdfInt patternType;
            stream.Dictionary.TryGetElement(reader, context, PdfNames.PatternType, out patternType);

            if (patternType.Value == TilingPatternType)
            {
                var tiling = base.ConvertFromStream(typeof(Tiling), reader, context, stream) as Tiling;
                return new TilingPatternObject(tiling);
            }

            return base.ConvertFromDictionary(type, reader, context, stream.Dictionary);
        }

        protected override object CreateInstance(Type type, Import.Parser.PostScriptReader reader, Import.IPdfImportContext context, PdfDictionary dictionary)
        {
            PdfInt patternType;

            dictionary.TryGetElement(reader, context, PdfNames.PatternType, out patternType);

            if (patternType.Value != TilingPatternType)
            {
                return PatternColorObject.CreateInstance(patternType);
            }

            return base.CreateInstance(type, reader, context, dictionary);
        }

        protected override IPdfPrimitive ConvertFromDictionary(Type type, Import.Parser.PostScriptReader reader, Import.IPdfImportContext context, PdfDictionary dictionary)
        {
            return base.ConvertFromDictionary(type, reader, context, dictionary);
        }
    }
}
