﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    /// <summary>
    /// Converts a PDF dictionary to a concrete object instance based on some type property in the dictionary.
    /// </summary>
    /// <typeparam name="T">The type of the type property.</typeparam>
    internal class TypeAwareConverter<T> : Converter
        where T : class, IPdfPrimitive
    {
        private readonly Func<T, object> factoryFunction;
        private readonly string typeProperty;

        public TypeAwareConverter(Func<T, object> factoryFunction, string typeProperty = PdfNames.Type)
        {
            Guard.ThrowExceptionIfNull(factoryFunction, "factoryFunction");
            Guard.ThrowExceptionIfNullOrEmpty(typeProperty, "typeProperty");

            this.factoryFunction = factoryFunction;
            this.typeProperty = typeProperty;
        }

        protected override object CreateInstance(Type type, PostScriptReader reader, IPdfImportContext context, PdfDictionary dictionary)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            T typePropertyValue;
            if (dictionary.TryGetElement(reader, context, this.typeProperty, out typePropertyValue))
            {
                return this.factoryFunction(typePropertyValue);
            }

            return base.CreateInstance(type, reader, context, dictionary);
        }
    }
}
