﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters
{
    /// <summary>
    /// Converts a PDF dictionary to a concrete object instance based on some type property and some parameter property in the dictionary.
    /// </summary>
    /// <typeparam name="T">The type of the type property.</typeparam>
    /// <typeparam name="P">The type of the parameter property.</typeparam>
    internal class TypeAwareConverterWithParameter<T, P> : Converter
        where T : class, IPdfPrimitive
        where P : class, IPdfPrimitive
    {
        private readonly Func<T, P, object> factoryFunction;
        private readonly string typeProperty;
        private readonly string parameterProperty;

        public TypeAwareConverterWithParameter(Func<T, P, object> factoryFunction, string typeProperty, string parameterProperty)
        {
            Guard.ThrowExceptionIfNull(factoryFunction, "factoryFunction");
            Guard.ThrowExceptionIfNullOrEmpty(typeProperty, "typeProperty");
            Guard.ThrowExceptionIfNullOrEmpty(parameterProperty, "parameterProperty");

            this.factoryFunction = factoryFunction;
            this.typeProperty = typeProperty;
            this.parameterProperty = parameterProperty;
        }

        protected override object CreateInstance(Type type, PostScriptReader reader, IPdfImportContext context, PdfDictionary dictionary)
        {
            Guard.ThrowExceptionIfNull(type, "type");
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(dictionary, "dictionary");

            T typePropertyValue;
            if (dictionary.TryGetElement(reader, context, this.typeProperty, out typePropertyValue))
            {
                P parameterValue;
                dictionary.TryGetElement(reader, context, this.parameterProperty, out parameterValue);

                return this.factoryFunction(typePropertyValue, parameterValue);
            }

            return base.CreateInstance(type, reader, context, dictionary);
        }
    }
}
