﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data
{
    internal abstract class PdfPropertyBase<T> : IPdfProperty
        where T : IPdfPrimitive
    {
        private readonly PdfPropertyDescriptor descriptor;
        private readonly IPdfPrimitive defaultValue;

        public PdfPropertyBase(PdfPropertyDescriptor descriptor)
        {
            Guard.ThrowExceptionIfNull(descriptor, "descriptor");

            this.descriptor = descriptor;
        }

        public PdfPropertyBase(PdfPropertyDescriptor descriptor, IPdfPrimitive defaultValue)
            : this(descriptor)
        {
            this.defaultValue = defaultValue;
        }


        public PdfPropertyDescriptor Descriptor
        {
            get
            {
                return this.descriptor;
            }
        }

        public bool HasNonDefaultValue
        {
            get
            {
                if (this.defaultValue == null)
                {
                    return this.GetValue() != null;
                }

                return !this.defaultValue.Equals(this.GetValue());
            }
        }

        public bool HasValue
        {
            get
            {
                return this.GetValue() != null;
            }
        }

        public abstract T GetValue();

        public abstract void SetValue(IPdfPrimitive value);

        public abstract void SetValue(PostScriptReader reader, IPdfImportContext context, IPdfPrimitive value);

        IPdfPrimitive IPdfProperty.GetValue()
        {
            return this.GetValue();
        }

        protected T PrepareValue(PostScriptReader reader, IPdfImportContext context, IPdfPrimitive value)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfObjectDescriptor descriptor = PdfObjectDescriptors.GetPdfObjectDescriptor<T>();
            IPdfPrimitive result = descriptor.Converter.Convert(typeof(T), reader, context, value);

            return (T)result;
        }
    }
}
