﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data
{
    internal class ReferenceProperty<T> : PdfPropertyBase<T>
        where T : class, IPdfPrimitive
    {
        private ReferencePropertyValueSource source;
        private T value;

        public ReferenceProperty(PdfPropertyDescriptor descriptor)
            : base(descriptor)
        {
        }

        public ReferenceProperty(PdfPropertyDescriptor descriptor, T defaultValue)
            : base(descriptor, defaultValue)
        {
            this.value = defaultValue;
        }

        internal IndirectReference Reference { get; private set; }

        public override T GetValue()
        {
            this.EnsureValueSourceIsApplied();

            return this.value;
        }

        public override void SetValue(IPdfPrimitive newValue)
        {
            this.EnsureValueSourceIsApplied();

            if (this.value != newValue)
            {
                this.value = (T)newValue;
            }
        }

        public override void SetValue(PostScriptReader reader, IPdfImportContext context, IPdfPrimitive value)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            this.source = null;
            if (value == null)
            {
                this.value = null;
            }
            else if (value.Type == PdfElementType.IndirectReference)
            {
                this.Reference = (IndirectReference)value;
                this.source = new ReferencePropertyValueSource(reader, context, this.Reference);
            }
            else
            {
                this.SetValue(this.PrepareValue(reader, context, value));
            }
        }

        private void EnsureValueSourceIsApplied()
        {
            if (this.source != null)
            {
                this.value = this.PrepareValue(this.source.Reader, this.source.Context, this.source.Source);
                this.source = null;
            }
        }
    }
}