﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions
{
    internal abstract class ActionObject : PdfObject
    {
        private readonly DirectProperty<PdfName> name;

        public ActionObject()
        {
            this.name = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("S", true));
        }

        public ActionObject(string actionName)
            : this()
        { 
            this.Name = new PdfName(actionName);
        }

        public PdfName Name
        {
            get
            {
                return this.name.GetValue();
            }
            set
            {
                this.name.SetValue(value);
            }
        }

        public static ActionObject CreateInstance(PdfName type)
        {
            Guard.ThrowExceptionIfNull(type, "type");

            switch (type.Value)
            {
                case ActionNames.GoTo:
                    return new GoToActionObject();

                case ActionNames.Uri:
                    return new UriActionObject();

                default: 
                    throw new NotSupportedException("Action type is not supported.");
            }
        }

        public static ActionObject CreateAction(IPdfExportContext context, Fixed.Model.Actions.Action action)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(action, "action");

            switch (action.ActionType)
            {
                case Fixed.Model.Actions.ActionType.GoTo:
                    Fixed.Model.Actions.GoToAction goToAction = action as Fixed.Model.Actions.GoToAction;
                    GoToActionObject pdfGoToAction = new GoToActionObject();
                    pdfGoToAction.CopyPropertiesFrom(context, goToAction);

                    return pdfGoToAction;

                case Fixed.Model.Actions.ActionType.Uri:
                    Fixed.Model.Actions.UriAction uriAction = action as Fixed.Model.Actions.UriAction; 
                    UriActionObject pdfUriAction = new UriActionObject();
                    pdfUriAction.CopyPropertiesFrom(context, uriAction);

                    return pdfUriAction; 

                default:
                    throw new NotSupportedException("The specified action is not supported.");
            } 
        }

        public abstract Fixed.Model.Actions.Action ToAction(PostScriptReader reader, IRadFixedDocumentImportContext context);
    }
}
