﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions
{
    internal class GoToActionObject : ActionObject
    {
        private readonly DirectProperty<IDestinationObject> destination;

        public GoToActionObject()
            : base(ActionNames.GoTo)
        {
            this.destination = this.RegisterDirectProperty<IDestinationObject>(new PdfPropertyDescriptor("D", true));
        }

        public IDestinationObject Destination
        {
            get
            {
                return this.destination.GetValue();
            }
            set
            {
                this.destination.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, Fixed.Model.Actions.GoToAction action)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(action, "action");

            if (action.Destination != null)
            {
                DestinationObject destination = new DestinationObject();
                destination.CopyPropertiesFrom(context, action.Destination);

                this.Destination = destination;
            }
            else
            {
                NamedDestinationObject destination = new NamedDestinationObject();
                destination.CopyPropertiesFrom(context, action.NamedDestination);

                this.Destination = destination;
            }
        }

        public override Fixed.Model.Actions.Action ToAction(PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            Fixed.Model.Actions.GoToAction goToAction = null;
            if (this.Destination != null)
            {
                if (this.Destination.IsNamed)
                {
                    NamedDestinationObject namedDestinationObject = (NamedDestinationObject)this.Destination;
                    NamedDestination namedDestination = namedDestinationObject.ToNamedDestination(context);
                    goToAction = new Fixed.Model.Actions.GoToAction(namedDestination);
                }
                else
                {
                    DestinationObject destiantionObject = (DestinationObject)this.Destination;
                    Destination destination = destiantionObject.ToDestination(reader, context);
                    goToAction = new Fixed.Model.Actions.GoToAction(destination);
                }
            }

            return goToAction;
        }
    }
}
