﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions
{
    internal class UriActionObject : ActionObject
    {
        private readonly DirectProperty<PdfLiteralString> uri;
        private readonly DirectProperty<PdfBool> isMap;

        public UriActionObject()
            : base (ActionNames.Uri)
        {
            this.uri = this.RegisterDirectProperty<PdfLiteralString>(new PdfPropertyDescriptor("URI", true));
            this.isMap = this.RegisterDirectProperty<PdfBool>(new PdfPropertyDescriptor("IsMap"), false.ToPdfBool());
        }

        public PdfLiteralString Uri 
        {
            get
            {
                return this.uri.GetValue();
            }
            set
            {
                this.uri.SetValue(value);
            }
        }

        public PdfBool IsMap 
        {
            get
            {
                return this.isMap.GetValue();
            }
            set
            {
                this.isMap.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, Fixed.Model.Actions.UriAction uriAction)
        {
            this.Uri = uriAction.Uri.ToString().ToPdfLiteralString(StringType.ASCII);

            if (uriAction.IncludeMouseCoordinates.HasValue)
            {
                this.IsMap = uriAction.IncludeMouseCoordinates.Value.ToPdfBool();
            }
        }

        public override Fixed.Model.Actions.Action ToAction(PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            Fixed.Model.Actions.UriAction uriAction = new Fixed.Model.Actions.UriAction();
            if (this.IsMap != null)
            {
                uriAction.IncludeMouseCoordinates = this.IsMap.Value;
            }

            if (this.Uri != null)
            {
                string text = this.Uri.ToStringAs(StringType.ASCII);
                uriAction.Uri = new Uri(text, UriKind.RelativeOrAbsolute);
            }

            return uriAction;
        }
    }
}
