﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal static class AnnotationFactory
    {
        public static AnnotationObject CreateInstance(PdfName type)
        {
            Guard.ThrowExceptionIfNull(type, "type");

            switch (type.Value)
            {
                case AnnotationTypes.Link:
                    return new LinkObject();
                case AnnotationTypes.Widget:
                    return new WidgetObject();
                default:
                    return new UnsupportedAnnotation();
            }
        }

        public static AnnotationObject CreatePdfAnnotation(IPdfExportContext context, Annotation annotation, RadFixedPage fixedPage)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(annotation, "annotation");
            Guard.ThrowExceptionIfNull(fixedPage, "fixedPage");

            switch (annotation.Type)
            {
                case AnnotationType.Link:
                    LinkObject link = new LinkObject();
                    Link linkModel = (Link)annotation;
                    link.CopyPropertiesFrom(context, linkModel, fixedPage);

                    return link;
                case AnnotationType.Widget:
                    WidgetObject widget = new WidgetObject();
                    Widget widgetModel = (Widget)annotation;
                    widget.CopyPropertiesFrom(context, widgetModel, fixedPage);
                    context.MapWidgets(widgetModel, widget);

                    return widget;
                default:
                    throw new NotSupportedException("Annotation type is not supported");
            }
        }
    }
}
