﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal class Appearance : PdfObject
    {
        private readonly FormXObject singleStateAppearance;
        private readonly Dictionary<string, FormXObject> stateToAppearanceMapping;

        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.IndirectObject;
            }
        }

        public Appearance(FormXObject singleStateAppearance)
        {
            Guard.ThrowExceptionIfNull(singleStateAppearance, "singleStateAppearance");

            this.singleStateAppearance = singleStateAppearance;

            // in this case the Appearances class should be responsible for writing this single state appearance.
        }

        public Appearance(Dictionary<string, FormXObject> stateToAppearanceMapping)
        {
            Guard.ThrowExceptionIfNull(stateToAppearanceMapping, "stateToAppearanceMapping");

            this.stateToAppearanceMapping = stateToAppearanceMapping;
        }

        public FormXObject SingleStateAppearance
        {
            get
            {
                return this.singleStateAppearance;
            }
        }

        public IEnumerable<KeyValuePair<string, FormXObject>> StateAppearances
        {
            get
            {
                if(this.stateToAppearanceMapping == null)
                {
                    yield break;
                }

                foreach(KeyValuePair<string, FormXObject> stateAppearance in this.stateToAppearanceMapping)
                {
                    yield return stateAppearance;
                }
            }
        }

        public bool TryGetStateAppearance(string state, out FormXObject appearance)
        {
            appearance = null;

            return this.stateToAppearanceMapping != null && this.stateToAppearanceMapping.TryGetValue(state, out appearance);
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            if(this.SingleStateAppearance == null)
            {
                PdfDictionary dictionary = new PdfDictionary();

                foreach(KeyValuePair<string, FormXObject> stateToAppearance in this.StateAppearances)
                {
                    IndirectObject formObject = context.CreateIndirectObject(stateToAppearance.Value);
                    dictionary[stateToAppearance.Key] = formObject.Reference;
                }

                dictionary.Write(writer, context);
            }
            else
            {
                throw new InvalidOperationException("FormXObject should be reused instead of writing the single-state appearance as separate indirect object!");
            }
        }
    }
}
