﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal class AppearanceCharacteristics : PdfObject
    {
        private readonly DirectProperty<PdfInt> rotation;
        private readonly ReferenceProperty<PdfArray> borderColor;
        private readonly ReferenceProperty<PdfArray> background;
        private readonly DirectProperty<PdfString> buttonNormalCaption;
        private readonly DirectProperty<PdfString> pushButtonRolloverCaption;
        private readonly DirectProperty<PdfString> pushButtonDownCaption;
        private readonly ReferenceProperty<FormXObject> pushButtonNormalIcon;
        private readonly ReferenceProperty<FormXObject> pushButtonRolloverIcon;
        private readonly ReferenceProperty<FormXObject> pushButtonDownIcon;
        private readonly ReferenceProperty<IconFit> pushButtonIconFit;
        private readonly DirectProperty<PdfInt> pushButtonTextPosition;

        public AppearanceCharacteristics()
        {
            this.rotation = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("R"), new PdfInt(0));
            this.borderColor = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("BC"));
            this.background = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("BG"));
            this.buttonNormalCaption = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("CA"));
            this.pushButtonRolloverCaption = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("RC"));
            this.pushButtonDownCaption = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("AC"));
            this.pushButtonNormalIcon = this.RegisterReferenceProperty<FormXObject>(new PdfPropertyDescriptor("I", false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.pushButtonRolloverIcon = this.RegisterReferenceProperty<FormXObject>(new PdfPropertyDescriptor("RI", false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.pushButtonDownIcon = this.RegisterReferenceProperty<FormXObject>(new PdfPropertyDescriptor("IX", false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.pushButtonIconFit = this.RegisterReferenceProperty<IconFit>(new PdfPropertyDescriptor("IF"), new IconFit());
            this.pushButtonTextPosition = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("TP"), new PdfInt(0));
        }

        public PdfInt Rotation
        {
            get
            {
                return this.rotation.GetValue();
            }
            set
            {
                this.rotation.SetValue(value);
            }
        }

        public PdfArray BorderColor
        {
            get
            {
                return this.borderColor.GetValue();
            }
            set
            {
                this.borderColor.SetValue(value);
            }
        }

        public PdfArray Background
        {
            get
            {
                return this.background.GetValue();
            }
            set
            {
                this.background.SetValue(value);
            }
        }

        public PdfString ButtonNormalCaption
        {
            get
            {
                return this.buttonNormalCaption.GetValue();
            }
            set
            {
                this.buttonNormalCaption.SetValue(value);
            }
        }

        public PdfString RolloverCaption
        {
            get
            {
                return this.pushButtonRolloverCaption.GetValue();
            }
            set
            {
                this.pushButtonRolloverCaption.SetValue(value);
            }
        }

        public PdfString DownCaption
        {
            get
            {
                return this.pushButtonDownCaption.GetValue();
            }
            set
            {
                this.pushButtonDownCaption.SetValue(value);
            }
        }

        public FormXObject NormalIcon
        {
            get
            {
                return this.pushButtonNormalIcon.GetValue();
            }
            set
            {
                this.pushButtonNormalIcon.SetValue(value);
            }
        }

        public IconFit IconFit
        {
            get
            {
                return this.pushButtonIconFit.GetValue();
            }
            set
            {
                this.pushButtonIconFit.SetValue(value);
            }
        }

        public FormXObject RolloverIcon
        {
            get
            {
                return this.pushButtonRolloverIcon.GetValue();
            }
            set
            {
                this.pushButtonRolloverIcon.SetValue(value);
            }
        }

        public FormXObject DownIcon
        {
            get
            {
                return this.pushButtonDownIcon.GetValue();
            }
            set
            {
                this.pushButtonDownIcon.SetValue(value);
            }
        }

        public PdfInt PushButtonTextPosition
        {
            get
            {
                return this.pushButtonTextPosition.GetValue();
            }
            set
            {
                this.pushButtonTextPosition.SetValue(value);
            }
        }
    }
}
