﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal class BorderStyle : PdfObject
    {
        private readonly DirectProperty<PdfReal> width;
        private readonly DirectProperty<PdfName> style;
        private readonly DirectProperty<PdfArray> dashArray;

        public BorderStyle()
        {
            this.width = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("W"), 1.ToPdfReal());
            this.style = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("S"), "S".ToPdfName());
            this.dashArray = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("D"), new double[] { 3 }.ToPdfArray());
        }

        public PdfReal Width
        {
            get
            {
                return this.width.GetValue();
            }
            set
            {
                this.width.SetValue(value);
            }
        }

        public PdfName Style
        {
            get
            {
                return this.style.GetValue();
            }
            set
            {
                this.style.SetValue(value);
            }
        }

        public PdfArray DashArray
        {
            get
            {
                return this.dashArray.GetValue();
            }
            set
            {
                this.dashArray.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(Export.IPdfExportContext context, AnnotationBorder border)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(border, "border");

            this.Width = Unit.DipToPoint(border.Width).ToPdfReal();
            switch (border.Style)
            {
                case AnnotationBorderStyle.None:
                    this.Width = 0.ToPdfReal();
                    break;
                case AnnotationBorderStyle.Dashed:
                    this.Style = new PdfName(AnnotationBorderStyles.Dashed);
                    this.DashArray = GetDashArrayInPoints(border).ToPdfArray();
                    break;
                case AnnotationBorderStyle.Inset:
                    this.Style = new PdfName(AnnotationBorderStyles.Inset);
                    break;
                case AnnotationBorderStyle.Beveled:
                    this.Style = new PdfName(AnnotationBorderStyles.Beveled);
                    break;
                default:
                    this.Style = new PdfName(AnnotationBorderStyles.Solid);
                    break;
            }
        }

        public void CopyPropertiesTo(AnnotationBorder border)
        {
            border.Width = Unit.PointToDip(this.Width.Value);
            border.DashArray = this.ImportDashArray();
                        
            switch(this.Style.Value)
            {
                case AnnotationBorderStyles.Dashed:
                    border.Style = AnnotationBorderStyle.Dashed;
                    break;
                case AnnotationBorderStyles.Inset:
                    border.Style = AnnotationBorderStyle.Inset;
                    break;
                case AnnotationBorderStyles.Beveled:
                    border.Style = AnnotationBorderStyle.Beveled;
                    break;
                default:
                    border.Style = AnnotationBorderStyle.Solid;
                    break;
            }
        }

        private static IEnumerable<double> GetDashArrayInPoints(AnnotationBorder border)
        {
            foreach (double value in border.DashArray)
            {
                yield return Unit.DipToPoint(value);
            }
        }

        private double[] ImportDashArray()
        {
            double[] dashArray = this.DashArray.ToDoubleArray();

            for (int i = 0; i < dashArray.Length; i++)
            {
                dashArray[i] = Unit.PointToDip(dashArray[i]);
            }

            return dashArray;
        }
    }
}
