﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal class LinkObject : AnnotationObject
    {
        private readonly DirectProperty<IDestinationObject> destination;
        private readonly DirectProperty<ActionObject> action;

        public LinkObject()
        {
            this.destination = this.RegisterDirectProperty<IDestinationObject>(new PdfPropertyDescriptor("Dest"));
            this.action = this.RegisterDirectProperty<ActionObject>(new PdfPropertyDescriptor("A"));
        }

        public IDestinationObject Destination
        {
            get
            {
                return this.destination.GetValue();
            }
            set
            {
                this.destination.SetValue(value);
            }
        }

        public ActionObject Action
        {
            get
            {
                return this.action.GetValue();
            }
            set
            {
                this.action.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, Link annotation, RadFixedPage fixedPage)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(annotation, "annotation");
            Guard.ThrowExceptionIfNull(fixedPage, "fixedPage");

            base.CopyPropertiesFrom(context, annotation, fixedPage);

            if (annotation.Destination != null)
            {
                DestinationObject destination = new DestinationObject();
                destination.CopyPropertiesFrom(context, annotation.Destination);

                this.Destination = destination;
            }

            if (annotation.Action != null)
            {
                this.Action = ActionObject.CreateAction(context, annotation.Action);
            }

            if (annotation.NamedDestination != null)
            {
                NamedDestinationObject destination = new NamedDestinationObject();
                destination.CopyPropertiesFrom(context, annotation.NamedDestination);

                this.Destination = destination;
            }
        }

        public override Annotation ToAnnotationOverride(PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            Link link = new Link();
            if (this.Action != null)
            {
                link.Action = this.Action.ToAction(reader, context);
            }
            else if (this.Destination != null)
            {
                if (this.Destination.IsNamed)
                {
                    NamedDestinationObject namedDestinationObject = (NamedDestinationObject)this.Destination;
                    link.NamedDestination = namedDestinationObject.ToNamedDestination(context);
                }
                else
                {
                    DestinationObject destinationObject = (DestinationObject)this.Destination;
                    link.Destination = destinationObject.ToDestination(reader, context);
                }
            }

            return link;
        }
    }
}
