﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations
{
    internal class WidgetObject : AnnotationObject, IVariableTextPropertiesObject
    {
        private readonly DirectProperty<PdfName> highlightingMode;
        private readonly ReferenceProperty<AppearanceCharacteristics> dynamicAppearanceCharacteristics;
        private readonly ReferenceProperty<PdfString> defaultAppearance;
        private readonly DirectProperty<PdfInt> quadding;
        private readonly ReferenceProperty<PdfDictionary> actions;
        private readonly ReferenceProperty<PdfDictionary> additionalActions;
        private readonly ReferenceProperty<FormFieldNode> parentField;

        public WidgetObject()
        {
            this.parentField = this.RegisterReferenceProperty<FormFieldNode>(new PdfPropertyDescriptor("Parent", true, PdfPropertyRestrictions.MustBeIndirectReference));
            this.highlightingMode = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("H"), new PdfName("I"));
            this.dynamicAppearanceCharacteristics = this.RegisterReferenceProperty<AppearanceCharacteristics>(new PdfPropertyDescriptor("MK"));
            this.actions = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("A"));
            this.additionalActions = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("AA"));
            this.defaultAppearance = this.RegisterReferenceProperty<PdfString>(new PdfPropertyDescriptor("DA"));
            this.quadding = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Q"));
        }

        public FormFieldNode FormField
        {
            get
            {
                return this.parentField.GetValue();
            }
            set
            {
                this.parentField.SetValue(value);
            }
        }

        public PdfName HighlightingMode
        {
            get
            {
                return this.highlightingMode.GetValue();
            }
            set
            {
                this.highlightingMode.SetValue(value);
            }
        }

        public AppearanceCharacteristics DynamicAppearanceCharacteristics
        {
            get
            {
                return this.dynamicAppearanceCharacteristics.GetValue();
            }
            set
            {
                this.dynamicAppearanceCharacteristics.SetValue(value);
            }
        }

        public PdfDictionary Actions
        {
            get
            {
                return this.actions.GetValue();
            }
            set
            {
                this.actions.SetValue(value);
            }
        }

        public PdfDictionary AdditionalActions
        {
            get
            {
                return this.additionalActions.GetValue();
            }
            set
            {
                this.additionalActions.SetValue(value);
            }
        }

        public PdfString GetDefaultAppearance(AcroFormObject form)
        {
            PdfString result = this.defaultAppearance.GetValue();

            if (result == null)
            {
                result = this.FormField.GetDefaultAppearance(form);
            }

            return result;
        }

        public void SetDefaultAppearance(PdfString value)
        {
            this.defaultAppearance.SetValue(value);
        }

        public PdfInt GetQuadding(AcroFormObject form)
        {
            PdfInt result = this.quadding.GetValue();

            if(result == null)
            {
                result = this.FormField.GetQuadding(form);
            }

            return result;
        }

        public void SetQuadding(PdfInt value)
        {
            this.quadding.SetValue(value);
        }

        public void CopyPropertiesFrom(IPdfExportContext context, Widget annotation, RadFixedPage fixedPage)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(annotation, "annotation");
            Guard.ThrowExceptionIfNull(fixedPage, "fixedPage");

            base.CopyPropertiesFrom(context, annotation, fixedPage);
            
            this.FormField = context.CreateFormFieldObject(annotation.Field, false);
            WidgetPropertiesExporter widgetPropertiesExporter = new WidgetPropertiesExporter(this, context);
            widgetPropertiesExporter.ExportWidgetProperties(annotation);
        }

        public override Annotation ToAnnotationOverride(PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            return this.ToWidget(reader, context);
        }

        public Widget ToWidget(PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            Widget widget;
            if (!context.TryGetWidget(this, out widget))
            {
                if (this.FormField != null)
                {
                    FormField field = this.FormField.ToFormField(reader, context, true);
                    widget = field.AddWidget();
                    context.MapWidgets(this, widget);
                    WidgetPropertiesImporter widgetImporter = new WidgetPropertiesImporter(this, reader, context);
                    widgetImporter.ImportWidgetProperties(widget);
                }
            }

            return widget;
        }

        internal bool TryGetOnStateName(out string stateName)
        {
            if (this.Appearances != null)
            {
                Appearance appearance = this.Appearances.NormalAppearance;

                foreach (KeyValuePair<string, FormXObject> stateToAppearance in appearance.StateAppearances)
                {
                    if (stateToAppearance.Key != TwoStatesButtonWidget.OffState)
                    {
                        stateName = stateToAppearance.Key;
                        return true;
                    }
                }
            }

            stateName = null;
            return false;
        }
    }
}
