﻿using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.Writers;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.CMaps
{
    internal class CMapObject : PdfStreamObjectBase
    {
        private readonly CMapEncoding encoding;

        public CMapObject()
            : this(new CMapEncoding())
        {
        }

        public CMapObject(CMapEncoding encoding)
        {
            this.encoding = encoding;
        }

        public CMapEncoding Encoding
        {
            get
            {
                return this.encoding;
            }
        }

        protected override byte[] GetData(IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            using (Stream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);
                PdfElementWriters.CMapWriter.Write(writer, context, this.Encoding);

                return stream.ReadAllBytes();
            }
        }

        protected override void InterpretData(PostScriptReader reader, IPdfImportContext context, PdfStreamBase stream)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(stream, "stream");

            using (Stream localStream = new MemoryStream(stream.ReadDecodedPdfData()))
            {
                CMapEncodingInterpreter interpreter = new CMapEncodingInterpreter(localStream, context, this.Encoding);
                interpreter.Execute();
            }
        }
    }
}
