﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class CalGrayColorSpaceObject : PdfRealColorSpaceObject
    {
        private readonly ReferenceProperty<PdfArray> whitePoint;
        private readonly ReferenceProperty<PdfArray> blackPoint;
        private readonly DirectProperty<PdfReal> gamma;

        public CalGrayColorSpaceObject()
        {
            this.whitePoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("WhitePoint", true));
            this.blackPoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("BlackPoint"));
            this.gamma = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("Gamma"));
        }

        public override string Name
        {
            get
            {
                return ColorSpaceNames.CalGray;
            }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return new RgbColorObject(0, 0, 0);
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.CalGray;
            }
        }

        public PdfArray WhitePoint
        {
            get
            {
                return this.whitePoint.GetValue();
            }
            set
            {
                this.whitePoint.SetValue(value);
            }
        }

        public PdfArray BlackPoint
        {
            get
            {
                return this.blackPoint.GetValue();
            }
            set
            {
                this.blackPoint.SetValue(value);
            }
        }

        public PdfReal Gamma
        {
            get
            {
                return this.gamma.GetValue();
            }
            set
            {
                this.gamma.SetValue(value);
            }
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        protected override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfReal[] components)
        {
            PdfReal g = components[0];
            GrayColorObject grayColor = new GrayColorObject(g);

            return grayColor;
        }

        public override ColorSpaceBase ToColorSpace()
        {
            var colorSpace = new CalGray();

            colorSpace.WhitePoint = this.WhitePoint.ToDoubleArray();

            if (this.blackPoint.HasValue)
            {
                colorSpace.BlackPoint = this.BlackPoint.ToDoubleArray();
            }

            if (this.gamma.HasValue)
            {
                colorSpace.Gamma = this.Gamma.Value;
            }

            return colorSpace;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            var source = (CalGray)colorSpaceBase;

            this.WhitePoint = source.WhitePoint.ToPdfArray();
            this.BlackPoint = source.BlackPoint.ToPdfArray();
            this.Gamma = source.Gamma.ToPdfReal();
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            this.Load(reader, context, array[1]);
        }
    }
}