﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class CalRgbColorSpaceObject : PdfRealColorSpaceObject
    {
        private readonly ReferenceProperty<PdfArray> whitePoint;
        private readonly ReferenceProperty<PdfArray> blackPoint;
        private readonly ReferenceProperty<PdfArray> gamma;
        private readonly ReferenceProperty<PdfArray> matrix;

        public CalRgbColorSpaceObject()
        {
            this.whitePoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("WhitePoint", true));
            this.blackPoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("BlackPoint"));
            this.gamma = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("Gamma"));
            this.matrix = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("Matrix"));
        }

        public override string Name
        {
            get { return ColorSpaceNames.CalRgb; }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return new RgbColorObject(0, 0, 0);
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.CalRgb;
            }
        }

        public PdfArray WhitePoint
        {
            get
            {
                return this.whitePoint.GetValue();
            }
            set
            {
                this.whitePoint.SetValue(value);
            }
        }

        public PdfArray BlackPoint
        {
            get
            {
                return this.blackPoint.GetValue();
            }
            set
            {
                this.blackPoint.SetValue(value);
            }
        }

        public PdfArray Gamma
        {
            get
            {
                return this.gamma.GetValue();
            }
            set
            {
                this.gamma.SetValue(value);
            }
        }

        public PdfArray Matrix
        {
            get
            {
                return this.matrix.GetValue();
            }
            set
            {
                this.matrix.SetValue(value);
            }
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        protected override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfReal[] components)
        {
            PdfReal r = components[0];
            PdfReal g = components[1];
            PdfReal b = components[2];
            RgbColorObject rgbColorObject = new RgbColorObject(r, g, b);

            return rgbColorObject;
        }

        public override ColorSpaceBase ToColorSpace()
        {
            var colorSpace = new CalRgb();

            colorSpace.WhitePoint = this.WhitePoint.ToDoubleArray();

            if (this.blackPoint.HasValue)
            {
                colorSpace.BlackPoint = this.BlackPoint.ToDoubleArray();
            }

            if (this.gamma.HasValue)
            {
                colorSpace.Gamma = this.Gamma.ToDoubleArray();
            }

            if (this.matrix.HasValue)
            {
                colorSpace.Matrix = this.Matrix.ToDoubleArray();
            }

            return colorSpace;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            var source = (CalRgb)colorSpaceBase;

            this.WhitePoint = source.WhitePoint.ToPdfArray();
            this.BlackPoint = source.BlackPoint.ToPdfArray();
            this.Gamma = source.Gamma.ToPdfArray();
            this.Matrix = source.Matrix.ToPdfArray();
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            this.Load(reader, context, array[1]);
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.Write(PdfNames.PdfArrayStart);
            base.Write(writer, context);
            writer.Write(PdfNames.PdfArrayEnd);
        }
    }
}