﻿using System;
using System.Linq; 
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal abstract class ColorObjectBase : PdfObject
    {
        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.Color;
            }
        }

        public abstract ColorBase ToColor(PostScriptReader reader, IPdfContentImportContext context);
        
        public void CopyPropertiesFrom(IPdfContentExportContext context, ColorBase color)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "color");

            this.CopyPropertiesFromOverride(context, color);
        }

        public static byte ConvertToByte(PdfReal component)
        {
            return (byte)(Math.Round(255 * component.Value));
        }

        public static PdfReal ConvertFromByte(byte component)
        {
            return new PdfReal(component / 255.0);
        }

        protected abstract void CopyPropertiesFromOverride(IPdfContentExportContext context, ColorBase color); 
    }
}
