﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class DeviceGrayColorSpaceObject : PdfRealColorSpaceObject
    {
        private static readonly RgbColorObject defaultRgbColor;

        static DeviceGrayColorSpaceObject()
        {
            defaultRgbColor = new RgbColorObject(0, 0, 0);
        }

        public override string Name
        {
            get { return ColorSpaceNames.DeviceGray; }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return defaultRgbColor;
            }
        }

        public override Filters.ColorSpace Public
        {
            get
            {
                return Filters.ColorSpace.Gray;
            }
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            var rgbColor = new RgbColorObject();
            rgbColor.CopyPropertiesFrom(context, color as RgbColor);

            return rgbColor;
        }

        protected override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfReal[] components)
        {
            PdfReal g = components[0];
            GrayColorObject grayColor = new GrayColorObject(g);

            return grayColor;
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            // Nothing to import...
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            // Nothing to copy...
        }

        public override ColorSpaceBase ToColorSpace()
        {
            return new DeviceGray();
        }
    }
}
