﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class DeviceRgbColorSpaceObject : PdfRealColorSpaceObject
    {
        private static readonly RgbColorObject defaultRgbColor;

        static DeviceRgbColorSpaceObject()
        {
            defaultRgbColor = new RgbColorObject(0, 0, 0);
        }

        public override string Name
        {
            get { return ColorSpaceNames.DeviceRgb; }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return defaultRgbColor;
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.RGB;
            }
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            RgbColorObject rgbColor = new RgbColorObject();
            rgbColor.CopyPropertiesFrom(context, color as RgbColor);

            return rgbColor;
        }

        protected override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfReal[] components)
        {
            PdfReal r = components[0];
            PdfReal g = components[1];
            PdfReal b = components[2];
            RgbColorObject rgbColorObject = new RgbColorObject(r, g, b);

            return rgbColorObject;
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            // Nothing to import
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            // Nothing to copy...
        }

        public override ColorSpaceBase ToColorSpace()
        {
            return new DeviceRgb();
        }
    }
}
