﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class IccBasedColorSpaceObject : ColorSpaceObject
    {
        public IccBasedColorSpaceObject()
        {
            this.IccProfile = new IccProfileStreamObject();
        }

        public override string Name
        {
            get { return ColorSpaceNames.IccBased; }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return new RgbColorObject(0, 0, 0);
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.ICCBased;
            }
        }

        public IccProfileStreamObject IccProfile
        {
            get;
            set;
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        public override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfArray components)
        {
            ColorSpaceObject alternate;
            if (this.IccProfile.TryGetAlternateColorSpace(out alternate))
            {
                ColorObjectBase color = alternate.GetColor(context, reader, components);

                return color;
            }

            throw new NotSupportedException("Cannot get color without alternate colorspace.");
        }

        public override ColorSpaceBase ToColorSpace()
        {
            var colorSpace = new IccBased();

            colorSpace.Data = this.IccProfile.Data.ToArray();
            if (this.IccProfile.Alternate != null)
            {
                colorSpace.Alternate = this.IccProfile.Alternate.ToColorSpace();
            }

            colorSpace.N = this.IccProfile.N.Value;
            if (this.IccProfile.Range != null)
            {
                colorSpace.Range = this.IccProfile.Range.ToDoubleArray();
            }

            return colorSpace;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            var source = (IccBased)colorSpaceBase;

            var iccProfile = new IccProfileStreamObject(source.Data);
            if (source.Alternate != null)
            {
                iccProfile.Alternate = ColorSpaceManager.CreateColorSpaceObject(source.Alternate);
            }

            iccProfile.N = source.N.ToPdfInt();
            if (source.Range != null)
            {
                iccProfile.Range = source.Range.ToPdfArray();
            }

            this.IccProfile = iccProfile;
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            PdfObjectDescriptor iccProfileObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(IccProfileStreamObject));
            IccProfileStreamObject iccProfile = iccProfileObjectDescriptor.Converter.Convert(typeof(IccProfileStreamObject), reader, context, array[1]) as IccProfileStreamObject;

            this.IccProfile = iccProfile;
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            
            PdfArray array = new PdfArray(new PdfName(this.Name), this.IccProfile);
            array.Write(writer, context);
        }
    }
}