﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class IccProfileObject : PdfStreamObjectBase
    {
        private static readonly IccProfileObject sRgbProfile;

        private readonly DirectProperty<PdfInt> colorComponents;

        private readonly byte[] data;

        static IccProfileObject()
        {
#if NETSTANDARD
            string resourceName = "Telerik.Documents.Fixed.Resources.sRGBProfile.icc";
#else
            string resourceName = "Telerik.Windows.Documents.Fixed.Resources.sRGBProfile.icc";
#endif

            sRgbProfile = new IccProfileObject(3, ResourcesHelper.GetApplicationResourceStream(resourceName).ReadAllBytes());
        }

        public IccProfileObject(int colorComponents, byte[] data)
        {
            Guard.ThrowExceptionIfNull(data, "data");

            this.data = data;

            this.colorComponents = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("N"));
            this.colorComponents.SetValue(new PdfInt(colorComponents));
        }

        public static IccProfileObject SRgbProfile
        {
            get
            {
                return sRgbProfile;
            }
        }

        protected override byte[] GetData(Export.IPdfExportContext context)
        {
            return this.data;
        }
    }
}
