﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class IndexedColorSpaceObject : ColorSpaceObject
    {
        public override string Name
        {
            get { return ColorSpaceNames.Indexed; }
        }

        public ColorSpaceObject Base
        {
            get;
            set;
        }

        public int HiVal
        {
            get;
            set;
        }

        public IPdfPrimitive Lookup
        {
            get;
            set;
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return new RgbColorObject(0, 0, 0);
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.Indexed;
            }
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfHexString hexString = this.Lookup as PdfHexString;
            IPdfPrimitive lookUp = null;
            PdfArray array = null;
            if (hexString != null)
            {
                lookUp = hexString;
            }
            else
            {
                lookUp = this.Lookup;
            }

            array = new PdfArray(new PdfName(this.Name), this.Base, new PdfInt(this.HiVal), lookUp);
            IndirectObject indexedArrayIndirectObject = context.CreateIndirectObject(array);

            writer.WriteIndirectReference(indexedArrayIndirectObject.Reference);
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        public override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfArray components)
        {
            // TODO: Implement this for getting color from index...
            throw new NotImplementedException();
        }

        public override ColorSpaceBase ToColorSpace()
        {
            var colorSpace = new Indexed();

            colorSpace.Base = this.Base.ToColorSpace();
            colorSpace.HiVal = this.HiVal;

            PdfLiteralString literalStringLookup = this.Lookup as PdfLiteralString;
            if (literalStringLookup != null)
            {
                colorSpace.Lookup = literalStringLookup.Value;
            }
            else
            {
                IndexedLookupStream pdfArrayLookup = this.Lookup as IndexedLookupStream;
                if (pdfArrayLookup != null)
                {
                    colorSpace.Lookup = pdfArrayLookup.Data.ToArray();
                }
            }

            return colorSpace;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            var source = (Indexed)colorSpaceBase;

            this.Base = ColorSpaceManager.CreateColorSpaceObject(source.Base);
            this.HiVal = source.HiVal;
            this.Lookup = new IndexedLookupStream(source.Lookup);
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            PdfObjectDescriptor pdfNameObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(ColorSpaceObject));
            ColorSpaceObject baseColorSpace = pdfNameObjectDescriptor.Converter.Convert(typeof(ColorSpaceObject), reader, context, array[1]) as ColorSpaceObject;
            this.Base = baseColorSpace;

            PdfObjectDescriptor pdfIntObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(PdfInt));
            PdfInt highVal = pdfIntObjectDescriptor.Converter.Convert(typeof(PdfInt), reader, context, array[2]) as PdfInt;
            this.HiVal = highVal.Value;

            if (array[3].Type == PdfElementType.String)
            {
                PdfString str = (PdfString)array[3];
                this.Lookup = new IndexedLookupStream(str.Value);
            }
            else
            {
                PdfObjectDescriptor indirectReferenceObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(IndexedLookupStream));
                IndexedLookupStream stream = indirectReferenceObjectDescriptor.Converter.Convert(typeof(IndexedLookupStream), reader, context, array[3]) as IndexedLookupStream;
                this.Lookup = stream;
            }
        }
    }
}