﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class LabColorSpaceObject : ColorSpaceObject
    {
        private readonly ReferenceProperty<PdfArray> whitePoint;
        private readonly ReferenceProperty<PdfArray> blackPoint;
        private readonly ReferenceProperty<PdfArray> range;

        public LabColorSpaceObject()
        {
            this.whitePoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("WhitePoint", true));
            this.blackPoint = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("BlackPoint"));
            this.range = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("Range"));
        }

        public override string Name
        {
            get
            {
                return ColorSpaceNames.Lab;
            }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return new RgbColorObject(0, 0, 0);
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.Lab;
            }
        }

        public PdfArray WhitePoint
        {
            get
            {
                return this.whitePoint.GetValue();
            }
            set
            {
                this.whitePoint.SetValue(value);
            }
        }

        public PdfArray BlackPoint
        {
            get
            {
                return this.blackPoint.GetValue();
            }
            set
            {
                this.blackPoint.SetValue(value);
            }
        }

        public PdfArray Range
        {
            get
            {
                return this.range.GetValue();
            }
            set
            {
                this.range.SetValue(value);
            }
        }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            // TODO: Implement this method
            throw new NotImplementedException();
        }

        public override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfArray components)
        {
            // TODO: Implement this method
            throw new NotImplementedException();
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.Write(PdfNames.PdfArrayStart);
            base.Write(writer, context);
            writer.Write(PdfNames.PdfArrayEnd);
        }

        public override ColorSpaceBase ToColorSpace()
        {
            var colorSpace = new Lab();

            colorSpace.WhitePoint = this.WhitePoint.ToDoubleArray();

            if (this.blackPoint.HasValue)
            {
                colorSpace.BlackPoint = this.BlackPoint.ToDoubleArray();
            }

            if (this.range.HasValue)
            {
                colorSpace.Range = this.Range.ToDoubleArray();
            }

            return colorSpace;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            var source = (Lab)colorSpaceBase;

            this.WhitePoint = source.WhitePoint.ToPdfArray();

            if (source.BlackPoint != null)
            {
                this.BlackPoint = source.BlackPoint.ToPdfArray();
            }

            if (source.Range != null)
            {
                this.Range = source.Range.ToPdfArray();
            }
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            this.Load(reader, context, array[1]);
        }
    }
}
