﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class PatternColorSpaceObject : ColorSpaceObject
    {
        public override string Name
        {
            get
            {
                return ColorSpaceNames.Pattern;
            }
        }

        public override ColorObjectBase DefaultColor
        {
            get
            {
                return null;
            }
        }

        public override ColorSpace Public
        {
            get
            {
                return ColorSpace.Pattern;
            }
        }

        public ColorSpaceObject UnderlyingColorSpace { get; set; }

        public override ColorObjectBase CreateColorObject(Export.IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            Pattern source = (Pattern)colorSpaceBase;

            if (source.UnderlyingColorSpace != null)
            {
                ColorSpaceObject colorSpaceObject = ColorSpaceManager.CreateColorSpaceObject(source.UnderlyingColorSpace.Name);
                colorSpaceObject.CopyPropertiesFrom(source.UnderlyingColorSpace);

                this.UnderlyingColorSpace = colorSpaceObject;
            }
        }

        public override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfArray components)
        {
            PdfName key = (PdfName)components[components.Count - 1];
            PatternColorObject pattern = context.GetPatternColor(reader, key);

            if (this.UnderlyingColorSpace != null)
            {
                PdfArray underlyingOperands = new PdfArray(components.Take(components.Count - 1));
                ColorObjectBase underlyingColor = this.UnderlyingColorSpace.GetColor(context, reader, underlyingOperands);
                pattern.UnderlyingColor = underlyingColor;
            }

            return pattern;
        }

        public override ColorSpaceBase ToColorSpace()
        {
            Pattern resultColorSpace = new Pattern();
            if (this.UnderlyingColorSpace != null)
            {
                resultColorSpace.UnderlyingColorSpace = this.UnderlyingColorSpace.ToColorSpace();
            }

            return resultColorSpace;
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            ColorSpaceObject underlyingColorSpace;
            array.TryGetElement(reader, context, 1, out underlyingColorSpace);

            this.UnderlyingColorSpace = underlyingColorSpace;
        }
    }
}
