﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Functions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    internal class SeparationColorSpaceObject : PdfRealColorSpaceObject
    {
        public override string Name
        {
            get { return ColorSpaceNames.Separation; }
        }

        public override ColorObjectBase DefaultColor
        {
            get { return new RgbColorObject(0, 0, 0); }
        }

        public override ColorSpace Public
        {
            get { return ColorSpace.Separation; }
        }

        public PdfName ColorantName { get; set; }

        public ColorSpaceObject AlternateColorSpace { get; set; }

        public FunctionBase TintTransform { get; private set; }

        public override ColorObjectBase CreateColorObject(IPdfContentExportContext context, ColorBase color)
        {
            throw new NotImplementedException();
        }

        protected override ColorObjectBase GetColor(IPdfContentImportContext context, PostScriptReader reader, PdfReal[] components)
        {
            if (components.Length > 0)
            {
                double[] inputValues = new double[components.Length];
                for (int i = 0; i < components.Length; i++)
                {
                    inputValues[i] = components[i].Value;
                }

                double[] tintTransformedParameters = this.TintTransform.Execute(inputValues);

                PdfArray array = new PdfArray();
                foreach (var value in tintTransformedParameters)
                {
                    array.Add(new PdfReal(value));
                }

                ColorObjectBase result = this.AlternateColorSpace.GetColor(context, reader, array);

                return result;
            }

            return null;
        }

        public override ColorSpaceBase ToColorSpace()
        {
            Separation target = new Separation();

            target.ColorantName = this.ColorantName.ToString();
            target.AlternateColorSpace = this.AlternateColorSpace.ToColorSpace();
            target.TintTransform = this.TintTransform;

            return target;
        }

        public override void CopyPropertiesFrom(ColorSpaceBase colorSpaceBase)
        {
            Separation source = (Separation)colorSpaceBase;

            ColorSpaceObject colorSpaceObject = ColorSpaceManager.CreateColorSpaceObject(source.AlternateColorSpace.Name);
            colorSpaceObject.CopyPropertiesFrom(source.AlternateColorSpace);

            this.ColorantName = new PdfName(source.ColorantName);
            this.AlternateColorSpace = colorSpaceObject;
            this.TintTransform = source.TintTransform;
        }

        public override void ImportFromArray(PostScriptReader reader, IPdfImportContext context, PdfArray array)
        {
            this.ColorantName = array[1] as PdfName;

            PdfObjectDescriptor colorSpaceObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(ColorSpaceObject));
            this.AlternateColorSpace = colorSpaceObjectDescriptor.Converter.Convert(typeof(ColorSpaceObject), reader, context, array[2]) as ColorSpaceObject;

            PdfObjectDescriptor functionObjectDescriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(FunctionObject));
            FunctionObject functionObject = (FunctionObject)functionObjectDescriptor.Converter.Convert(typeof(FunctionObject), reader, context, array[3]);
            this.TintTransform = functionObject.ToFunction(reader, context);
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            PdfArray array = new PdfArray();

            array.Add(new PdfName(this.Name));
            array.Add(this.ColorantName);
            
            IndirectObject alternateObject = context.CreateIndirectObject(this.AlternateColorSpace);
            array.Add(alternateObject.Reference);

            FunctionObject function = FunctionObject.CreateFunction(new PdfInt((int)this.TintTransform.FunctionType));
            function.CopyPropertiesFrom(context, this.TintTransform);
            IndirectObject functionObject = context.CreateIndirectObject(function);
            array.Add(functionObject.Reference);

            array.Write(writer, context);
        }        
    }
}
