﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common
{
    internal abstract class PdfArrayObject : PdfObject
    {
        private readonly HashSet<IPdfProperty> propertiesToIgnore;
        private readonly Dictionary<IPdfProperty, int> propertyIndex;

        public PdfArrayObject()
        {
            this.propertiesToIgnore = new HashSet<IPdfProperty>();
            this.propertyIndex = new Dictionary<IPdfProperty, int>();
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.Write(PdfNames.PdfArrayStart);

            var orderedProperties = this.propertyIndex.Keys.OrderBy(prop => propertyIndex[prop]);

            bool isFirstProperty = true;
            foreach (IPdfProperty property in orderedProperties)
            {  
                if (!this.ShouldIgnoreProperty(property))
                {
                    if (!isFirstProperty)
                    {
                        writer.WriteSeparator();
                    }

                    isFirstProperty = false;
                    if (property.HasValue)
                    {
                        WritePropertyValue(writer, context, property);
                    }
                    else
                    {
                        writer.Write(PdfNames.PdfNull);
                    }
                }
            }

            writer.Write(PdfNames.PdfArrayEnd);
        } 

        protected override void RegisterPdfProperty(IPdfProperty property)
        {
            base.RegisterPdfProperty(property);

            this.propertyIndex[property] = this.Properties.Count - 1;
        }

        protected void IgnoreProperties(params IPdfProperty[] properties)
        {
            foreach (var property in properties)
            {
                this.IgnoreProperty(property);
            }
        }

        protected void IgnoreProperty(IPdfProperty property)
        {
            this.propertiesToIgnore.Add(property);
        }

        private bool ShouldIgnoreProperty(IPdfProperty property)
        {
            return this.propertiesToIgnore.Contains(property);
        }
    }
}
