﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common
{
    internal class PdfConcatenatableDataStream : PdfStreamObjectBase
    {
        private const byte WhiteSpace = 0x20;

        private byte[] data;
        private PdfStreamBase importedStream;

        public byte[] Data
        {
            get
            {
                if (this.data == null && this.importedStream != null)
                {
                    this.data = this.importedStream.ReadDecodedPdfData();
                }

                return this.data;
            }
            protected set
            {
                this.data = value;
            }
        }

        internal static byte[] Concat(byte[] data, byte[] dataToAppend, bool concatWithWhiteSpace)
        {
            int dataLength = data != null ? data.Length : 0;
            int dataToAppendLength = dataToAppend != null ? dataToAppend.Length : 0;
            int totalLength = dataLength + dataToAppendLength;

            bool addWhiteSpace = concatWithWhiteSpace && PdfConcatenatableDataStream.CheckWhiteSpaceNeeded(data, dataToAppend);
            if (addWhiteSpace)
            {
                totalLength++;
            }

            byte[] result = new byte[totalLength];
            int index = 0;
            if (data != null)
            {
                data.CopyTo(result, index);
            }

            if (dataToAppend != null)
            {
                index = dataLength;

                if (addWhiteSpace)
                {
                    result[index] = WhiteSpace;
                    index++;
                }

                dataToAppend.CopyTo(result, index);
            }

            return result;
        }

        protected bool HasImportedStream()
        {
            return this.importedStream != null;
        }

        internal virtual void Concat(PdfConcatenatableDataStream stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            bool concatWithWhiteSpace = false;
            this.Data = PdfConcatenatableDataStream.Concat(this.Data, stream.Data, concatWithWhiteSpace);
        }

        protected override void InterpretData(PostScriptReader reader, IPdfImportContext context, PdfStreamBase stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            this.importedStream = stream;
        }

        protected override byte[] GetData(IPdfExportContext context)
        {
            byte[] data = this.Data;
            this.Data = null;

            return data;
        }

        private static bool CheckWhiteSpaceNeeded(byte[] data, byte[] dataToAppend)
        {
            bool hasData = data != null && data.Length > 0;
            bool hasDataToAppend = dataToAppend != null && dataToAppend.Length > 0;

            if (hasData && hasDataToAppend)
            {
                bool isLastByteWhiteSpace = Characters.IsWhiteSpace(data[data.Length - 1]);
                bool isFirstByteToAppendWhiteSpace = Characters.IsWhiteSpace(dataToAppend[0]);

                return !isLastByteWhiteSpace && !isFirstByteToAppendWhiteSpace;
            }
            else
            {
                return false;
            }
        }
    }
}
