﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.Navigation;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations
{
    internal static class DestinationTypes
    {
        public const string Location = "XYZ";
        public const string FitToPage = "Fit";
        public const string FitHorizontalToPage = "FitH";
        public const string FitVerticalToPage = "FitV";
        public const string FitToRect = "FitR";
        public const string FitToBoundingRect = "FitB";
        public const string FitHorizontalToBoundingRect = "FitBH";
        public const string FitVerticalToBoundingRect = "FitBV";

        private readonly static Dictionary<string, DestinationType> stringToDestinationType;
        private readonly static Dictionary<DestinationType, string> destinationTypeToString;

        static DestinationTypes()
        {
            stringToDestinationType = new Dictionary<string, DestinationType>();
            destinationTypeToString = new Dictionary<DestinationType, string>();

            InitializeMappings();
        }  

        public static string GetDestinationType(DestinationType type)
        {
            return destinationTypeToString[type];
        }

        public static DestinationType GetDestinationType(string type)
        {
            return stringToDestinationType[type];
        }

        private static void InitializeMappings()
        {
            RegisterMapping(DestinationTypes.Location, DestinationType.Location);
            RegisterMapping(DestinationTypes.FitToPage, DestinationType.PageFit);
            RegisterMapping(DestinationTypes.FitHorizontalToPage, DestinationType.PageHorizontalFit);
            RegisterMapping(DestinationTypes.FitVerticalToPage, DestinationType.PageVerticalFit);
            RegisterMapping(DestinationTypes.FitToRect, DestinationType.RectangleFit);
            RegisterMapping(DestinationTypes.FitToBoundingRect, DestinationType.BoundingRectangleFit);
            RegisterMapping(DestinationTypes.FitHorizontalToBoundingRect, DestinationType.BoundingRectangleHorizontalFit);
            RegisterMapping(DestinationTypes.FitVerticalToBoundingRect, DestinationType.BoundingRectangleVerticalFit);
        }

        private static void RegisterMapping(string destinationTypeName, DestinationType destinationType)
        {
            stringToDestinationType[destinationTypeName] = destinationType;
            destinationTypeToString[destinationType] = destinationTypeName;
        }
    }
}
