﻿using System;
using System.Globalization;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    internal static class SignatureDateCodec
    {
        public static DateTime? Decode(string inputDate)
        {
            DateTimeStyles dateTimeStyle = DateTimeStyles.None;
            string format = "yyyyMMddHHmmsszzz";

            string date = inputDate;
            if (date.StartsWith("D:"))
            {
                date = date.Substring(2);
            }

            if (date.EndsWith("Z"))
            {
                date = date.Substring(0, date.Length - 1);
                format = "yyyyMMddHHmmss";
                dateTimeStyle = DateTimeStyles.AssumeUniversal;
            }
            else
            {
                date = date.Replace("'", ":");
                date = date.Substring(0, date.Length - 1);
            }

            DateTime result;
            if (!DateTime.TryParseExact(date, format, DateTimeFormatInfo.InvariantInfo, dateTimeStyle, out result))
            {
                return null;
            }

            DateTime localTime = result.ToLocalTime();

            return localTime;
        }

        internal static string Encode(DateTime date)
        {
            string format = "\\D\\:yyyyMMddHHmmss";
            
            string dateTime = date.ToString(format, DateTimeFormatInfo.InvariantInfo);

            if (date.Kind == DateTimeKind.Local)
            {
                string timezone = date.ToString("zzz", DateTimeFormatInfo.InvariantInfo);
                timezone = timezone.Replace(":", "'");
                dateTime += timezone + "'";
            }
            else
            {
                dateTime += "Z";
            }

            return dateTime;
        }
    }
}
