﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DigitalSignature
{
    internal class SignatureExportInfo
    {
        private Dictionary<string, SignaturePositions> signaturePositionsStore = new Dictionary<string, SignaturePositions>();

        private int documentStartPosition;
        private int documentEndPosition;
        private string currentSignatureFieldName;

        internal int DocumentStartPosition { get { return this.documentStartPosition; } }

        internal int DocumentEndPosition { get { return this.documentEndPosition; } }

        internal void SetSignatureFieldName(string signatureFieldName)
        {
            if (!string.IsNullOrEmpty(this.currentSignatureFieldName))
            {
                throw new ArgumentException("New signature export is triggered before the last one has finished.");
            }
            
            this.currentSignatureFieldName = signatureFieldName;
            this.signaturePositionsStore.Add(this.currentSignatureFieldName, new SignaturePositions());
        }

        internal void ClearSignatureFieldName()
        {
            this.currentSignatureFieldName = null;
        }

        internal void AddDocumentStartPosition(int position)
        {
            this.documentStartPosition = position;
        }

        internal void AddDocumentEndPosition(int position)
        {
            this.documentEndPosition = position;
        }

        internal void AddByteRangeStartPosition(int position)
        {
            this.signaturePositionsStore[this.currentSignatureFieldName].ByteRangeStartPosition = position;
        }

        internal void AddByteRangeEndPosition(int position)
        {
            this.signaturePositionsStore[this.currentSignatureFieldName].ByteRangeEndPosition = position;
        }

        internal void AddContentStartPosition(int position)
        {
            this.signaturePositionsStore[this.currentSignatureFieldName].ContentStartPosition = position;
        }

        internal void AddContentEndPosition(int position)
        {
            this.signaturePositionsStore[this.currentSignatureFieldName].ContentEndPosition = position;
        }

        internal int[] ComposeByteRangeArray(string signatureFieldName)
        {
            int[] array = new int[4];

            // document start position.
            array[0] = this.DocumentStartPosition;

            // lenght of the content just before the start of the signature content.
            array[1] = (this.signaturePositionsStore[signatureFieldName].ContentStartPosition) - this.DocumentStartPosition;

            // the signature is over the content of the document excluding the signature content itself.
            // This means that the start position of the second part of the byte range should be the position of the first symbol after the signature content end position.
            array[2] = this.signaturePositionsStore[signatureFieldName].ContentEndPosition + 1;

            // lenght of the content after the signature end.
            array[3] = this.DocumentEndPosition - this.signaturePositionsStore[signatureFieldName].ContentEndPosition;

            return array;
        }

        internal SignaturePositions GetSignaturePositions(string signatureFieldName)
        {
            SignaturePositions result = this.signaturePositionsStore[signatureFieldName];

            return result;
        }
    }
}
