﻿using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure
{
    internal class ContentStream : PdfConcatenatableDataStream
    {
        public ContentStream()
        {
        }

        internal ContentStream(byte[] data)
        {
            this.Data = data;
        }

        public static byte[] BuildContentData(IPdfExportContext context, IResourceHolder resourceHolder, IContentRootElement contentRootElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(resourceHolder, "resourceHolder");
            Guard.ThrowExceptionIfNull(contentRootElement, "contentRootElement");

            using (MemoryStream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);
                IPdfContentExportContext contentExportContext = context.CreateContentExportContext(resourceHolder, contentRootElement);
                ContentElementWriterBase.WriteElement(writer, contentExportContext, contentRootElement);
                byte[] data = new byte[stream.Length];
                stream.Seek(0, SeekOrigin.Begin);
                stream.Read(data, 0, data.Length);

                return data;
            }
        }

        public static void ParseContentData(byte[] data, IRadFixedDocumentImportContext context, IResourceHolder resourceHolder, IContentRootElement contentRoot)
        {
            Guard.ThrowExceptionIfNull(data, "data");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(resourceHolder, "resourceHolder");
            Guard.ThrowExceptionIfNull(contentRoot, "contentRoot");

            if (data.Length > 0)
            {
                using (Stream stream = new MemoryStream(data))
                {
                    ContentStreamInterpreter interpreter = new ContentStreamInterpreter(stream, new PdfContentImportContext(context, resourceHolder, contentRoot));

                    interpreter.Execute();
                }
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, IResourceHolder resourceHolder, IContentRootElement contentRootElement)
        {
            this.Data = ContentStream.BuildContentData(context, resourceHolder, contentRootElement);
        }

        public void CopyPropertiesTo(IRadFixedDocumentImportContext context, IResourceHolder resourceHolder, IContentRootElement contentRoot)
        {
            ContentStream.ParseContentData(this.Data, context, resourceHolder, contentRoot);

            if (this.HasImportedStream())
            {
                this.Data = null;
            }
        }

        internal override void Concat(PdfConcatenatableDataStream stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            bool concatWithWhiteSpace = true;
            this.Data = PdfConcatenatableDataStream.Concat(this.Data, stream.Data, concatWithWhiteSpace);
        }
    }
}
