﻿using System;
using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure
{
    internal class ObjectStream : PdfStreamObjectBase
    {
        private readonly DirectProperty<PdfInt> n;
        private readonly DirectProperty<PdfInt> first;
        private readonly ReferenceProperty<ObjectStream> extends;
        private PostScriptReader reader;
        private readonly List<KeyValuePair<int, int>> objectNumberToOffset;

        public ObjectStream()
        {
            this.n = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("N", true, PdfPropertyRestrictions.MustBeDirectObject));
            this.first = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("First", true, PdfPropertyRestrictions.MustBeDirectObject));
            this.extends = this.RegisterReferenceProperty<ObjectStream>(new PdfPropertyDescriptor("Extends", false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.objectNumberToOffset = new List<KeyValuePair<int, int>>();
        }
        
        public PdfInt N
        {
            get
            {
                return this.n.GetValue();
            }
            set
            {
                this.n.SetValue(value);
            }
        }

        public PdfInt First
        {
            get
            {
                return this.first.GetValue();
            }
            set
            {
                this.first.SetValue(value);
            }
        }
        
        public ObjectStream Extends
        {
            get
            {
                return this.extends.GetValue();
            }
            set
            {
                this.extends.SetValue(value);
            }
        }

        protected override void InterpretData(PostScriptReader reader, IPdfImportContext context, PdfStreamBase stream)
        {
            if (this.reader == null)
            {
                byte[] data = stream.ReadDecodedPdfData();
                this.reader = new PostScriptReader(new MemoryStream(data), new KeywordCollection());

                int numberOfEntries = this.N.Value;

                for (int i = 0; i < numberOfEntries; i++)
                {
                    int objectNumber = this.reader.Read<PdfInt>(context, PdfElementType.PdfInt).Value;
                    int offset = this.reader.Read<PdfInt>(context, PdfElementType.PdfInt).Value;
                    this.objectNumberToOffset.Add(new KeyValuePair<int,int>(objectNumber, offset));
                }
            }
        }

        internal IPdfPrimitive ReadObjectContent(CrossReferenceEntry entry, IPdfImportContext context)
        {
            int index = entry.Field2;
            int offset = this.GetOffset(index);

            this.reader.Reader.Seek(offset, SeekOrigin.Begin);
            bool isLastElement = index >= this.objectNumberToOffset.Count - 1;
            long endPosition = (isLastElement) ? this.reader.Reader.Length : this.GetOffset(index + 1);

            using (context.BeginImportOfStreamInnerContent())
            {
                IPdfPrimitive[] elements = this.reader.Read(context, endPosition);
                IPdfPrimitive content = elements[0];

                return content;
            }                
        }

        private int GetOffset(int objectIndex)
        {
            KeyValuePair<int, int> objectNumberToOffset = this.objectNumberToOffset[objectIndex];
            int offset = objectNumberToOffset.Value;
            offset += this.First.Value;

            return offset;
        }
    }
}
