﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure
{
    internal class PdfResource : PdfObject
    {
        private readonly ReferenceProperty<PdfDictionary> fonts;
        private readonly ReferenceProperty<PdfDictionary> xObjects;
        private readonly ReferenceProperty<PdfDictionary> patterns;
        private readonly ReferenceProperty<PdfDictionary> extGState;
        private readonly ReferenceProperty<PdfDictionary> colorSpaces;
        private readonly ReferenceProperty<PdfDictionary> shadings;
        private readonly ReferenceProperty<PdfDictionary> markedContentProperties;
        private readonly ReferenceProperty<PdfArray> procSets;

        public PdfResource()
        {
            this.fonts = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("Font"));
            this.xObjects = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("XObject"));
            this.patterns = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("Pattern"));
            this.extGState = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("ExtGState"));
            this.colorSpaces = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("ColorSpace"));
            this.shadings = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("Shading"));
            this.markedContentProperties = this.RegisterReferenceProperty<PdfDictionary>(new PdfPropertyDescriptor("Properties"));
            this.procSets = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("ProcSet"));
        }

        public PdfDictionary Fonts
        {
            get
            {
                return this.fonts.GetValue();
            }
            set
            {
                this.fonts.SetValue(value);
            }
        }

        public PdfDictionary XObjects
        {
            get
            {
                return this.xObjects.GetValue();
            }
            set
            {
                this.xObjects.SetValue(value);
            }
        }

        public PdfDictionary Patterns
        {
            get
            {
                return this.patterns.GetValue();
            }
            set
            {
                this.patterns.SetValue(value);
            }
        }

        public PdfDictionary ExtGState
        {
            get
            {
                return this.extGState.GetValue();
            }
            set
            {
                this.extGState.SetValue(value);
            }
        }

        public PdfDictionary ColorSpaces
        {
            get
            {
                return this.colorSpaces.GetValue();
            }
            set
            {
                this.colorSpaces.SetValue(value);
            }
        }

        public PdfDictionary Shadings
        {
            get
            {
                return this.shadings.GetValue();
            }
            set
            {
                this.shadings.SetValue(value);
            }
        }

        public PdfDictionary MarkedContentProperties
        {
            get
            {
                return this.markedContentProperties.GetValue();
            }
            set
            {
                this.markedContentProperties.SetValue(value);
            }
        }

        public PdfArray ProcSets
        {
            get
            {
                return this.procSets.GetValue();
            }
            set
            {
                this.procSets.SetValue(value);
            }
        }
    }
}
