﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Encryption;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure
{
    internal class Trailer : PdfObject
    {
        private readonly DirectProperty<PdfInt> size;
        private readonly DirectProperty<PdfInt> prev;
        private readonly ReferenceProperty<DocumentCatalog> root;
        private readonly DirectProperty<PdfArray> id;
        private readonly ReferenceProperty<Encrypt> encryption;
        private readonly ReferenceProperty<PdfInt> xRefStm;

        public Trailer()
            : base()
        {
            this.prev = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Prev", true, PdfPropertyRestrictions.MustBeDirectObject));
            this.size = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Size", true, PdfPropertyRestrictions.MustBeDirectObject));
            this.root = this.RegisterReferenceProperty<DocumentCatalog>(new PdfPropertyDescriptor("Root", true, PdfPropertyRestrictions.MustBeIndirectReference));
            this.id = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("ID"));
            this.encryption = this.RegisterReferenceProperty<Encrypt>(new PdfPropertyDescriptor("Encrypt", false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.xRefStm = this.RegisterReferenceProperty<PdfInt>(new PdfPropertyDescriptor("XRefStm", false, PdfPropertyRestrictions.MustBeDirectObject));
        }

        public PdfInt Prev
        {
            get
            {
                return this.prev.GetValue();
            }
            set
            {
                this.prev.SetValue(value);
            }
        }

        public PdfInt Size
        {
            get
            {
                return this.size.GetValue();
            }
            set
            {
                this.size.SetValue(value);
            }
        }

        public DocumentCatalog Root
        {
            get
            {
                return this.root.GetValue();
            }
            set
            {
                this.root.SetValue(value);
            }
        }

        public Encrypt Encryption
        {
            get
            {
                return this.encryption.GetValue();
            }
            set
            {
                this.encryption.SetValue(value);
            }
        }

        public PdfInt XRefStm
        {
            get
            {
                return this.xRefStm.GetValue();
            }
            set
            {
                this.xRefStm.SetValue(value);
            }
        }

        public PdfArray Id
        {
            get
            {
                return this.id.GetValue();
            }
            set
            {
                this.id.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Id = new PdfArray();
            this.Id.Add(new PdfHexString(context.DocumentId));
            this.Id.Add(new PdfHexString(context.DocumentId));
            this.Size = new PdfInt(context.CrossReferenceCollection.MaxObjectNumber + 1);
            this.Encryption = context.Encryption;
        }

        public void CopyPropertiesTo(PostScriptReader reader, IPdfImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            if (context.Encryption == null)
            {
                context.Encryption = this.Encryption;
            }

            if (context.DocumentId == null)
            {
                PdfString documentId;
                if (this.Id != null && this.Id.TryGetElement(reader, context, 0, out documentId))
                {
                    context.DocumentId = documentId.Value;
                }
                else
                {
                    context.DocumentId = new byte[0];
                }
            }

            if (context.Root == null)
            {
                context.Root = this.root;
            }
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            writer.WriteLine(PdfNames.TrailerMarker);
            base.Write(writer, context);
        }
    }
}
