﻿using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal abstract class CidFontObject : PdfObject
    {
        private const int WidthIntegersCount = 1;
        private const int WidthVerticalIntegersCount = 3;
        private const int StartEndRangeIntegersCount = 2;
        public const string CidFontType0 = "CIDFontType0";
        public const string CidFontType2 = "CIDFontType2";

        private readonly DirectProperty<PdfName> baseFont;
        private readonly DirectProperty<CidSystemInfo> cidSystemInfo;
        private readonly DirectProperty<PdfInt> defaultWidth;
        private readonly ReferenceProperty<FontDescriptor> fontDescriptor;
        private readonly ReferenceProperty<PdfArray> widths;
        private readonly ReferenceProperty<PdfArray> defaultWidthVertical;
        private readonly ReferenceProperty<PdfArray> widthsVerticle;
        private readonly ReferenceProperty<CharIdToGlyphIdMappingBase> charIdToGlyphIdMapping;

        public CidFontObject()
        {
            this.baseFont = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("BaseFont", true));
            this.cidSystemInfo = this.RegisterDirectProperty<CidSystemInfo>(new PdfPropertyDescriptor("CIDSystemInfo", true));
            this.defaultWidth = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("DW"));
            this.fontDescriptor = this.RegisterReferenceProperty<FontDescriptor>(new PdfPropertyDescriptor("FontDescriptor", true, PdfPropertyRestrictions.MustBeIndirectReference));
            this.widths = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("W"));
            this.defaultWidthVertical = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("DW2"));
            this.widthsVerticle = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("W2"));
            this.charIdToGlyphIdMapping = this.RegisterReferenceProperty<CharIdToGlyphIdMappingBase>(new PdfPropertyDescriptor("CIDToGIDMap"));
        }

        public PdfName BaseFont
        {
            get
            {
                return this.baseFont.GetValue();
            }
            set
            {
                this.baseFont.SetValue(value);
            }
        }

        public CidSystemInfo CidSystemInfo
        {
            get
            {
                return this.cidSystemInfo.GetValue();
            }
            set
            {
                this.cidSystemInfo.SetValue(value);
            }
        }

        public FontDescriptor FontDescriptor
        {
            get
            {
                return this.fontDescriptor.GetValue();
            }
            set
            {
                this.fontDescriptor.SetValue(value);
            }
        }

        public PdfArray Widths
        {
            get
            {
                return this.widths.GetValue();
            }
            set
            {
                this.widths.SetValue(value);
            }
        }

        public PdfInt DefaultWidth
        {
            get
            {
                return this.defaultWidth.GetValue();
            }
            set
            {
                this.defaultWidth.SetValue(value);
            }
        }

        public PdfArray WidthsVertical
        {
            get
            {
                return this.widthsVerticle.GetValue();
            }
            set
            {
                this.widthsVerticle.SetValue(value);
            }
        }

        public PdfArray DefaultWidthVertical
        {
            get
            {
                return this.defaultWidthVertical.GetValue();
            }
            set
            {
                this.defaultWidthVertical.SetValue(value);
            }
        }

        public CharIdToGlyphIdMappingBase CharIdToGlyphIdMapping
        {
            get
            {
                return this.charIdToGlyphIdMapping.GetValue();
            }
            set
            {
                this.charIdToGlyphIdMapping.SetValue(value);
            }
        }

        public static CidFontObject CreateInstance(PdfName typeName)
        {
            Guard.ThrowExceptionIfNull(typeName, "typeName");

            switch (typeName.Value)
            {
                case CidFontType0:
                    return new CidFontType0Object();
                case CidFontType2:
                    return new CidFontType2Object();
                default:
                    throw new NotSupportedException(string.Format("Not supported CidFont: {0}", typeName.Value));
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            this.CopyPropertiesFromOverride(context, font);
        }

        internal bool TryGetFontFamily(out FontFamily fontFamily)
        {
            fontFamily = null;

            if (this.FontDescriptor != null && this.FontDescriptor.FontFamily != null)
            {
                fontFamily = new FontFamily(this.FontDescriptor.FontFamily.ToString());
            }

            return fontFamily != null;
        }

        internal abstract bool TryCreateFontFromFontFile(out FontBase font);
        internal abstract FontBase CreateFontWithoutFontFile();
        internal abstract bool TryCreateFontFromFontProperties(FontProperties fontProperties, out FontBase font);

        internal virtual void CopyPropertiesToFont(CidFontBase font, PostScriptReader reader, IPdfImportContext context)
        {
            this.FontDescriptor.Lang.CopyToProperty(font.Lang, (language) => language.Value);
            this.FontDescriptor.CidSet.CopyToProperty(font.CidSet, (cidSet) => cidSet.Data);

            this.CidSystemInfo.CopyToProperty(font.CidSystemInfo, (info) => new Fixed.Model.Fonts.Encoding.CidSystemInfo(info.Registry.ToString(), info.Ordering.ToString(), info.Supplement.Value));
            this.Widths.CopyToProperty(font.Widths, (widthsArray) => CreateCidWidths(widthsArray, WidthIntegersCount, GetConstantWidth, GetWidthsArrayRange, context));
            this.WidthsVertical.CopyToProperty(font.WidthsVertical, (widthsArray) => CreateCidWidths(widthsArray, WidthVerticalIntegersCount, GetConstantWidthVertical, GetWidthsVerticalArrayRange, context));
            this.DefaultWidth.CopyToProperty(font.DefaultWidth, (width) => width.Value);
            this.DefaultWidthVertical.CopyToProperty(font.DefaultWidthsVertical, (array) => GetDefaultWidthVertical(array));
            this.CharIdToGlyphIdMapping.CopyToProperty(font.CharIdToGlyphIdMapping, (mapping) => CreateCidToGidMap(mapping));
        }

        protected virtual void CopyPropertiesFromOverride(IPdfExportContext context, FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            this.BaseFont = font.FontName.ToPdfName();
            this.FontDescriptor = new FontDescriptor();
            this.FontDescriptor.CopyPropertiesFrom(context, font);

            CidFontBase cidFont = (CidFontBase)font;
            this.CidSystemInfo = new CidSystemInfo();
            this.CidSystemInfo.CopyPropertiesFrom(context, cidFont);

            this.Widths = cidFont.Widths.ToPrimitive(CalculateWidths, () => CalculateWidths(context, font));
            this.DefaultWidth = cidFont.DefaultWidth.ToPrimitive((width) => new PdfInt(width));
            this.WidthsVertical = cidFont.WidthsVertical.ToPrimitive(CalculateWidths);
            this.DefaultWidthVertical = cidFont.DefaultWidthsVertical.ToPrimitive(CalculateDefaultWidth);
            this.CharIdToGlyphIdMapping = cidFont.CharIdToGlyphIdMapping.ToPrimitive(CalculateCidToGidMap, () => CharIdToGlyphIdMappingBase.Identity);
        }

        private static PdfArray CalculateWidths(IPdfExportContext context, FontBase font)
        {
            if (font.FontFileInfo.HasValue)
            {
                return null;
            }
            else
            {
                return CalculateUsedCharactersWidths(context, font);
            }
        }

        private static PdfArray CalculateUsedCharactersWidths(IPdfExportContext context, FontBase font)
        {
            PdfArray widths = new PdfArray();

            PdfArray localWidths = null;
            CharCode lastCharCode = null;
            CharInfo[] usedCharacters = context.GetUsedCharacters(font).OrderBy(g => g.CharCode.Code).ToArray();

            foreach (CharInfo glyph in usedCharacters)
            {
                if (lastCharCode == null || lastCharCode.Size != glyph.CharCode.Size || lastCharCode.Code != glyph.CharCode.Code - 1)
                {
                    widths.Add(new PdfInt(glyph.CharCode.Code));
                    localWidths = new PdfArray();
                    widths.Add(localWidths);
                    lastCharCode = glyph.CharCode;
                }

                localWidths.Add(new PdfInt((int)font.GetWidth(glyph.CharCode.Code)));
            }

            return widths;
        }

        private static CharIdToGlyphIdMappingBase CalculateCidToGidMap(CidToGidMap cidToGidMap)
        {
            if (cidToGidMap.IsIdentityMapping)
            {
                return CharIdToGlyphIdMappingBase.Identity;
            }
            else
            {
                return new CharIdToGlyphIdMapping() { Data = cidToGidMap.Data };
            }
        }

        private static CidToGidMap CreateCidToGidMap(CharIdToGlyphIdMappingBase mapping)
        {
            if (mapping == CharIdToGlyphIdMappingBase.Identity)
            {
                return new CidToGidMap();
            }
            else
            {
                return new CidToGidMap(mapping.Data);
            }
        }

        private static PdfArray CalculateWidths(CidWidths<int> cidWidths)
        {
            return CalculateWidths(cidWidths, GetWidthComponents);
        }

        private static PdfArray CalculateWidths(CidWidths<CidWidthVertical> cidWidths)
        {
            return CalculateWidths(cidWidths, GetWidthComponents);
        }

        private static PdfArray CalculateWidths<T>(CidWidths<T> cidWidths, Func<T, IEnumerable<int>> getWidthComponents)
        {
            PdfArray array = new PdfArray();

            foreach (CidWidthsRange<T> range in cidWidths)
            {
                array.Add(new PdfInt(range.StartCharCode));

                if (range.IsConstantWidthRange)
                {
                    array.Add(new PdfInt(range.EndCharCode));

                    foreach (int component in getWidthComponents(range.ConstantWidth))
                    {
                        array.Add(new PdfInt(component));
                    }
                }
                else
                {
                    PdfArray widthsArray = new PdfArray();

                    foreach (T width in range.Widths)
                    {
                        foreach (int component in getWidthComponents(width))
                        {
                            widthsArray.Add(new PdfInt(component));
                        }
                    }

                    array.Add(widthsArray);
                }
            }

            return array;
        }

        private static CidWidths<T> CreateCidWidths<T>(
            PdfArray widthsArray, 
            int widthRangeComponentsCount,
            Func<Queue<int>, T> getConstantWidth,
            Func<PdfArray, IEnumerable<T>> getArrayRange,
            IPdfImportContext context)
        {
            CidWidths<T> widths = new CidWidths<T>();
            Queue<int> numbers = new Queue<int>();
            
            foreach (IPdfPrimitive arrayPrimitive in widthsArray)
            {
                IPdfPrimitive primitive = CidFontObject.GetDirectPrimitive(arrayPrimitive, context);

                switch (primitive.Type)
                {
                    case PdfElementType.PdfInt:
                    case PdfElementType.PdfReal:
                        int number = CidFontObject.GetIntegerFromNumber(primitive);
                        numbers.Enqueue(number);

                        if (numbers.Count == widthRangeComponentsCount + StartEndRangeIntegersCount)
                        {
                            int startCode = numbers.Dequeue();
                            int endCode = numbers.Dequeue();
                            T width = getConstantWidth(numbers);
                            widths.Add(new CidWidthsRange<T>(startCode, endCode, width));
                        }
                        break;
                    default:
                        int firstCharCode = numbers.Dequeue();
                        PdfArray array = (PdfArray)primitive;
                        widths.Add(new CidWidthsRange<T>(firstCharCode, getArrayRange(array)));
                        break;
                }
            }

            return widths;
        }

        private static int GetIntegerFromNumber(IPdfPrimitive primitive)
        {
            int integer;

            if(primitive.Type == PdfElementType.PdfInt)
            {
                PdfInt pdfInteger = (PdfInt)primitive;
                integer = pdfInteger.Value;
            }
            else
            {
                PdfReal real = (PdfReal)primitive;
                integer = (int)real.Value;
            }

            return integer;
        }

        private static IPdfPrimitive GetDirectPrimitive(IPdfPrimitive primitive, IPdfImportContext context)
        {
            if (primitive.Type == PdfElementType.IndirectReference)
            {
                IndirectReference reference = (IndirectReference)primitive;

                IPdfPrimitive directPrimitive;
                if (!context.TryGetIndirectObject(reference, out directPrimitive))
                {
                    directPrimitive = context.ReadIndirectObject(reference).Content;
                    context.RegisterIndirectObject(reference, directPrimitive);
                }

                return directPrimitive;
            }

            return primitive;
        }

        private static CidWidthVertical GetConstantWidthVertical(Queue<int> numbers)
        {
            int wY = numbers.Dequeue();
            int vX = numbers.Dequeue();
            int vY = numbers.Dequeue();

            return new CidWidthVertical(wY, vX, vY);
        }

        private static IEnumerable<CidWidthVertical> GetWidthsVerticalArrayRange(PdfArray array)
        {
            Queue<int> numbers = new Queue<int>(array.ToIntArray());

            while (numbers.Count > 0)
            {
                yield return GetConstantWidthVertical(numbers);
            }
        }

        private static int GetConstantWidth(Queue<int> numbers)
        {
            return numbers.Dequeue();
        }

        private static IEnumerable<int> GetWidthsArrayRange(PdfArray array)
        {
            return array.ToIntArray();
        }

        private static PdfArray CalculateDefaultWidth(CidWidthVertical width)
        {
            PdfArray array = new PdfArray();
            array.Add(new PdfInt(width.DisplacementVectorY));
            array.Add(new PdfInt(width.PositionVectorY));

            return array;
        }

        private static CidWidthVertical GetDefaultWidthVertical(PdfArray array)
        {
            int[] integers = array.ToIntArray();

            return new CidWidthVertical(integers[0], 0, integers[1]);
        }

        private static IEnumerable<int> GetWidthComponents(CidWidthVertical width)
        {
            yield return width.DisplacementVectorY;
            yield return width.PositionVectorX;
            yield return width.PositionVectorY;
        }

        private static IEnumerable<int> GetWidthComponents(int width)
        {
            yield return width;
        }
    }
}
