﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class CidSystemInfo : PdfObject
    {
        private readonly DirectProperty<PdfString> registry;
        private readonly DirectProperty<PdfString> ordering;
        private readonly DirectProperty<PdfInt> supplement;

        public CidSystemInfo()
        {
            this.registry = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("Registry", true));
            this.ordering = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("Ordering", true));
            this.supplement = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Supplement", true));
        }

        public PdfString Registry
        {
            get
            {
                return this.registry.GetValue();
            }
            set
            {
                this.registry.SetValue(value);
            }
        }

        public PdfString Ordering
        {
            get
            {
                return this.ordering.GetValue();
            }
            set
            {
                this.ordering.SetValue(value);
            }
        }

        public PdfInt Supplement
        {
            get
            {
                return this.supplement.GetValue();
            }
            set
            {
                this.supplement.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(Export.IPdfExportContext context, Fixed.Model.Fonts.CidFontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            Fixed.Model.Fonts.Encoding.CidSystemInfo info = font.CidSystemInfo.Value;

            this.Registry = new PdfLiteralString(info.Registry, StringType.ASCII);
            this.Ordering = new PdfLiteralString(info.Ordering, StringType.ASCII);
            this.Supplement = new PdfInt(info.Supplement);
        }
    }
}
