﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts.Cid;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class FontDescriptor : PdfObject
    {
        private readonly DirectProperty<PdfName> fontName;
        private readonly DirectProperty<PdfString> fontFamily;
        private readonly DirectProperty<PdfInt> flags;
        private readonly DirectProperty<PdfArray> fontBBox;
        private readonly DirectProperty<PdfReal> italicAngle;
        private readonly DirectProperty<PdfReal> ascent;
        private readonly DirectProperty<PdfReal> descent;
        private readonly DirectProperty<PdfReal> capHeight;
        private readonly DirectProperty<PdfReal> stemV;
        private readonly DirectProperty<PdfName> fontStretch;
        private readonly DirectProperty<PdfReal> fontWeight;
        private readonly DirectProperty<PdfReal> leading;
        private readonly DirectProperty<PdfReal> xHeight;
        private readonly DirectProperty<PdfReal> stemH;
        private readonly DirectProperty<PdfReal> avgWidth;
        private readonly DirectProperty<PdfReal> maxWidth;
        private readonly DirectProperty<PdfReal> missingWidth;
        private readonly DirectProperty<PdfString> charSet;
        private readonly ReferenceProperty<Type1FontStream> fontFile;
        private readonly ReferenceProperty<Type2FontStream> fontFile2;
        private readonly ReferenceProperty<Type3FontStream> fontFile3;

        // Fields related to CidFontDescriptor
        private readonly ReferenceProperty<PdfName> lang;
        private readonly ReferenceProperty<CidSet> cidSet;

        public FontDescriptor()
        {
            this.fontName = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("FontName", true));
            this.flags = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Flags", true));
            this.fontBBox = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("FontBBox", true));
            this.italicAngle = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("ItalicAngle", true));
            this.ascent = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("Ascent", true));
            this.descent = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("Descent", true));
            this.capHeight = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("CapHeight", true));
            this.stemV = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("StemV", true));

            this.fontFamily = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("FontFamily"));
            this.charSet = this.RegisterDirectProperty<PdfString>(new PdfPropertyDescriptor("CharSet"));
            this.fontStretch = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("FontStretch"));
            this.fontWeight = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("FontWeight"));
            this.leading = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("Leading"));
            this.xHeight = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("XHeight"));
            this.stemH = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("StemH"));
            this.avgWidth = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("AvgWidth"));
            this.maxWidth = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("MaxWidth"));
            this.missingWidth = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("MissingWidth"));

            this.fontFile = this.RegisterReferenceProperty<Type1FontStream>(new PdfPropertyDescriptor("FontFile"));
            this.fontFile2 = this.RegisterReferenceProperty<Type2FontStream>(new PdfPropertyDescriptor("FontFile2"));
            this.fontFile3 = this.RegisterReferenceProperty<Type3FontStream>(new PdfPropertyDescriptor("FontFile3"));

            this.lang = this.RegisterReferenceProperty<PdfName>(new PdfPropertyDescriptor("Lang"));
            this.cidSet = this.RegisterReferenceProperty<CidSet>(new PdfPropertyDescriptor("CIDSet"));
        }

        public PdfName FontName
        {
            get
            {
                return this.fontName.GetValue();
            }
            set
            {
                this.fontName.SetValue(value);
            }
        }

        public PdfInt Flags
        {
            get
            {
                return this.flags.GetValue();
            }
            set
            {
                this.flags.SetValue(value);
            }
        }

        public PdfArray FontBBox
        {
            get
            {
                return this.fontBBox.GetValue();
            }
            set
            {
                this.fontBBox.SetValue(value);
            }
        }

        public PdfReal ItalicAngle
        {
            get
            {
                return this.italicAngle.GetValue();
            }
            set
            {
                this.italicAngle.SetValue(value);
            }
        }

        public PdfReal Ascent
        {
            get
            {
                return this.ascent.GetValue();
            }
            set
            {
                this.ascent.SetValue(value);
            }
        }

        public PdfReal Descent
        {
            get
            {
                return this.descent.GetValue();
            }
            set
            {
                this.descent.SetValue(value);
            }
        }

        public PdfReal CapHeight
        {
            get
            {
                return this.capHeight.GetValue();
            }
            set
            {
                this.capHeight.SetValue(value);
            }
        }

        public PdfReal StemV
        {
            get
            {
                return this.stemV.GetValue();
            }
            set
            {
                this.stemV.SetValue(value);
            }
        }

        public PdfString FontFamily
        {
            get
            {
                return this.fontFamily.GetValue();
            }
            set
            {
                this.fontFamily.SetValue(value);
            }
        }
        
        public PdfName FontStretch
        {
            get
            {
                return this.fontStretch.GetValue();
            }
            set
            {
                this.fontStretch.SetValue(value);
            }
        }

        public PdfReal FontWeight
        {
            get
            {
                return this.fontWeight.GetValue();
            }
            set
            {
                this.fontWeight.SetValue(value);
            }
        }

        public PdfReal Leading
        {
            get
            {
                return this.leading.GetValue();
            }
            set
            {
                this.leading.SetValue(value);
            }
        }

        public PdfReal XHeight
        {
            get
            {
                return this.xHeight.GetValue();
            }
            set
            {
                this.xHeight.SetValue(value);
            }
        }

        public PdfReal StemH
        {
            get
            {
                return this.stemH.GetValue();
            }
            set
            {
                this.stemH.SetValue(value);
            }
        }

        public PdfReal AvgWidth
        {
            get
            {
                return this.avgWidth.GetValue();
            }
            set
            {
                this.avgWidth.SetValue(value);
            }
        }

        public PdfReal MaxWidth
        {
            get
            {
                return this.maxWidth.GetValue();
            }
            set
            {
                this.maxWidth.SetValue(value);
            }
        }

        public PdfReal MissingWidth
        {
            get
            {
                return this.missingWidth.GetValue();
            }
            set
            {
                this.missingWidth.SetValue(value);
            }
        }

        public PdfString CharSet
        {
            get
            {
                return this.charSet.GetValue();
            }
            set
            {
                this.charSet.SetValue(value);
            }
        }

        public Type1FontStream FontFile
        {
            get
            {
                return this.fontFile.GetValue();
            }
            set
            {
                this.fontFile.SetValue(value);
            }
        }

        public Type2FontStream FontFile2
        {
            get
            {
                return this.fontFile2.GetValue();
            }
            set
            {
                this.fontFile2.SetValue(value);
            }
        }

        public Type3FontStream FontFile3
        {
            get
            {
                return this.fontFile3.GetValue();
            }
            set
            {
                this.fontFile3.SetValue(value);
            }
        }

        public PdfName Lang
        {
            get
            {
                return this.lang.GetValue();
            }
            set
            {
                this.lang.SetValue(value);
            }
        }

        public CidSet CidSet
        {
            get
            {
                return this.cidSet.GetValue();
            }
            set
            {
                this.cidSet.SetValue(value);
            }
        }

        public virtual void CopyPropertiesFrom(IPdfExportContext context, FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            this.CopyRequiredPropertiesFrom(font);

            this.FontFamily = font.FontFamily.ToPrimitive((fontFamily) => fontFamily.ToPdfLiteralString(StringType.ASCII));
            this.CharSet = font.CharSet.ToPrimitive((charSet) => charSet.ToPdfLiteralString(StringType.ASCII));
            this.FontStretch = font.FontStretch.ToPrimitive((fontStretch) => new PdfName(fontStretch));
            this.FontWeight = font.FontWeight.ToPrimitive((weight) => new PdfReal(weight));
            this.Leading = font.Leading.ToPrimitive((leading) => new PdfReal(leading));
            this.XHeight = font.XHeight.ToPrimitive((xHeight) => new PdfReal(xHeight));
            this.StemH = font.StemH.ToPrimitive((stemH) => new PdfReal(stemH));
            this.AvgWidth = font.AvgWidth.ToPrimitive((width) => new PdfReal(width));
            this.MaxWidth = font.MaxWidth.ToPrimitive((width) => new PdfReal(width));
            this.MissingWidth = font.MissingWidth.ToPrimitive((width) => new PdfReal(width));

            if (!this.TryExportImportedFontFile(font) && font.FontSource != Core.Fonts.EmptyFontSource.Instance)
            {
                this.CreateFontFile(context, font);
            }

            CidFontBase cidFont = font as CidFontBase;

            if (cidFont != null)
            {
                this.Lang = cidFont.Lang.ToPrimitive((language) => new PdfName(language));
                this.CidSet = cidFont.CidSet.ToPrimitive((cidSet) => new CidSet() { Data = cidSet }, () => this.CalculateCidSet(context, cidFont));
            }
        }

        private CidSet CalculateCidSet(IPdfExportContext context, CidFontBase font)
        {
            CidSet cidSet = null;

            switch (font.Type)
            {
                case Fixed.Model.Fonts.FontType.CidType2Font:
                    cidSet = new CidSet();
                    cidSet.CopyPropertiesFrom(context, font);
                    break;
            }

            return cidSet;
        }

        private bool TryExportImportedFontFile(FontBase font)
        {
            bool hasImportedFontFile = font.FontFileInfo.HasValue;

            if (hasImportedFontFile)
            {                
                switch(font.FontFileInfo.Value.FileType.Value)
                {
                    case FontFileType.FontFile:
                        this.FontFile = new Type1FontStream();
                        this.InitializeFontFile(font);
                        break;
                    case FontFileType.FontFile2:
                        this.FontFile2 = new Type2FontStream();
                        this.InitializeFontFile2(font);
                        break;
                    case FontFileType.FontFile3:
                        this.FontFile3 = new Type3FontStream();
                        this.InitializeFontFile3(font);
                        break;
                }
            }

            return hasImportedFontFile;
        }

        private void InitializeFontFile2(FontBase font)
        {
            FontFileInfo info = font.FontFileInfo.Value;
            this.FontFile2.Metadata = info.Metadata.ToPrimitive((meta) => new PdfStreamObject() { Data = meta });
            this.FontFile2.Length1 = info.Length1.ToPrimitive((length) => new PdfInt(length));
            this.FontFile2.Data = font.FontSource.Data;
        }

        private void InitializeFontFile(FontBase font)
        {
            FontFileInfo info = font.FontFileInfo.Value;
            this.FontFile.Metadata = info.Metadata.ToPrimitive((meta) => new PdfStreamObject() { Data = meta });
            this.FontFile.Length1 = info.Length1.ToPrimitive((length) => new PdfInt(length));
            this.FontFile.Length2 = info.Length2.ToPrimitive((length) => new PdfInt(length));
            this.FontFile.Length3 = info.Length3.ToPrimitive((length) => new PdfInt(length));
            this.FontFile.Data = font.FontSource.Data;
        }

        private void InitializeFontFile3(FontBase font)
        {
            FontFileInfo info = font.FontFileInfo.Value;
            this.FontFile3.Subtype = info.FileSubType.ToPrimitive((subtype) => new PdfName(subtype.ToString()));
            this.FontFile3.Metadata = info.Metadata.ToPrimitive((meta) => new PdfStreamObject() { Data = meta });
            this.FontFile3.Length1 = info.Length1.ToPrimitive((length) => new PdfInt(length));
            this.FontFile3.Length2 = info.Length2.ToPrimitive((length) => new PdfInt(length));
            this.FontFile3.Length3 = info.Length3.ToPrimitive((length) => new PdfInt(length));
            this.FontFile3.Data = font.FontSource.Data;
        }

        private void CreateFontFile(IPdfExportContext context, FontBase font)
        {
            switch (font.Type)
            {
                case FontType.TrueType:
                case FontType.CidType2Font:

                    this.FontFile2 = new Type2FontStream();
                    this.FontFile2.CopyPropertiesFrom(context, font);

                    break;
            }
        }

        private void CopyRequiredPropertiesFrom(FontBase font)
        {
            int flags = GetFlags(font);
            this.Flags = new PdfInt(flags);
            this.FontName = font.FontName.ToPdfName();
            this.FontBBox = font.BoundingBox.Value.ToPdfArray();
            this.ItalicAngle = font.ItalicAngle.Value.ToPdfReal();
            this.Ascent = (font.Ascent.Value * FontBase.FontRatio).ToPdfReal();
            this.Descent = (font.Descent.Value * FontBase.FontRatio).ToPdfReal();
            this.CapHeight = font.CapHeight.Value.ToPdfReal();
            this.StemV = font.StemV.Value.ToPdfReal();
        }

        private static int GetFlags(FontBase font)
        {
            FlagWriter<FontFlag> writer = new FlagWriter<FontFlag>();
            writer.SetFlagOnCondition(FontFlag.FixedPitch, font.IsFixedPitch);
            writer.SetFlagOnCondition(FontFlag.Serif, font.IsSerif);
            writer.SetFlagOnCondition(FontFlag.Symbolic, font.IsSymbolic);
            writer.SetFlagOnCondition(FontFlag.Script, font.IsScript);
            writer.SetFlagOnCondition(FontFlag.Nonsymbolic, font.IsNonSymbolic);
            writer.SetFlagOnCondition(FontFlag.Italic, font.IsItalic);
            writer.SetFlagOnCondition(FontFlag.AllCap, font.IsAllCap);
            writer.SetFlagOnCondition(FontFlag.SmallCap, font.IsSmallCap);
            writer.SetFlagOnCondition(FontFlag.ForceBold, font.IsForcingBold);

            return writer.ResultFlags;
        }
    }
}
