﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal abstract class FontStream : PdfStreamObject
    {
        private readonly ReferenceProperty<PdfStreamObject> metadata;

        public FontStream()
        {
            this.metadata = this.RegisterReferenceProperty<PdfStreamObject>(new PdfPropertyDescriptor("Metadata"));
        }

        public PdfStreamObject Metadata
        {
            get
            {
                return this.metadata.GetValue();
            }
            set
            {
                this.metadata.SetValue(value);
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context, Telerik.Windows.Documents.Fixed.Model.Fonts.FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            this.CopyPropertiesFromOverride(context, font);
        }
        
        protected virtual void CopyPropertiesFromOverride(IPdfExportContext context, Telerik.Windows.Documents.Fixed.Model.Fonts.FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            IEnumerable<CharInfo> usedCharacters = context.GetUsedCharacters(font);

            this.Data = font.ComputeSubset(usedCharacters);
            this.CopyPropertiesFromData();
        }

        public abstract FontFileInfo ToFileInfo();

        public abstract FontSource ToFontSource(FontType fontType);

        protected virtual void CopyPropertiesFromData()
        {
        }
    }
}
