﻿#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Core.Fonts;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class FontsHelper
    {
        internal static void GetFontFamily(string fontName, out string fontFamily, out FontWeight fontWeight, out FontStyle fontStyle)
        {
            Guard.ThrowExceptionIfNullOrEmpty(fontName, "fontName");

            if (!PredefinedFontFamilies.TryGetFontFamily(fontName, out fontFamily))
            {
                fontFamily = StripFontName(fontName);
            }

            string styles = GetFontStylesFromFontName(fontName).ToLower();

            fontWeight = FontWeights.Normal;
            if (IsBold(styles))
            {
                fontWeight = FontWeights.Bold;
            }
            else if (styles.Contains("black"))
            {
                fontWeight = FontWeights.Black;
            }

            fontStyle = FontStyles.Normal;
            if (IsItalic(styles))
            {
                fontStyle = FontStyles.Italic;
            }
        }

        internal static string StripFontName(string fontName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(fontName, "fontName");

            int index = fontName.IndexOf("+");
            if (index > 0)
            {
                return fontName.Substring(index + 1).Split(Core.PostScript.Data.Characters.FontFamilyDelimiters)[0];
            }
            else
            {
                return fontName.Split(Core.PostScript.Data.Characters.FontFamilyDelimiters)[0];
            }
        }

        internal static bool IsBold(string styles)
        {
            if (string.IsNullOrEmpty(styles))
            {
                return false;
            }

            return styles.Contains("bold") || styles.Contains("bd");
        }

        internal static bool IsItalic(string styles)
        {
            if (string.IsNullOrEmpty(styles))
            {
                return false;
            }

            return styles.Contains("it") || styles.Contains("obl");
        }

        private static string GetFontStylesFromFontName(string fontName)
        {
            string[] str = fontName.Split(Core.PostScript.Data.Characters.FontFamilyDelimiters);
            if (str.Length > 1)
            {
                return str[1];
            }

            return string.Empty;
        }
    }
}
