﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal static class PdfFontsFactory
    {
        public const string Type0Font = "Type0";
        public const string Type1Font = "Type1";
        public const string MMType1Font = "MMType1";
        public const string TrueTypeFont = "TrueType";

        private static readonly Dictionary<Fixed.Model.Fonts.FontType, Func<FontObject>> fontTypeToCreateFontFunctionMapping;

        static PdfFontsFactory()
        {
            fontTypeToCreateFontFunctionMapping = new Dictionary<Fixed.Model.Fonts.FontType, Func<FontObject>>();

            Register(Fixed.Model.Fonts.FontType.Standard, () => { return new Type1FontObject(); });
            Register(Fixed.Model.Fonts.FontType.Type1, () => { return new Type1FontObject(); });
            Register(Fixed.Model.Fonts.FontType.TrueType, () => { return new TrueTypeFontObject(); });
            Register(Fixed.Model.Fonts.FontType.CidType2Font, () => { return new Type0FontObject(); });
            Register(Fixed.Model.Fonts.FontType.CidType0Font, () => { return new Type0FontObject(); });
        }

        public static FontObject CreateFont(Fixed.Model.Fonts.FontType fontType)
        {
            return fontTypeToCreateFontFunctionMapping[fontType]();
        }

        public static FontObject CreateInstance(PdfName typeName)
        {
            Guard.ThrowExceptionIfNull(typeName, "typeName");

            switch (typeName.Value)
            {
                case Type0Font: return new Type0FontObject();

                case MMType1Font:
                case Type1Font: return new Type1FontObject();

                case TrueTypeFont: return new TrueTypeFontObject();

                default:

                    throw new NotSupportedException("Font is not supported.");
            }
        }

        private static void Register(Fixed.Model.Fonts.FontType fontType, Func<FontObject> font)
        {
            fontTypeToCreateFontFunctionMapping.Add(fontType, font);
        }
    }
}
