﻿using System.Windows;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class TrueTypeFontObject : SimpleFontObject
    {
        protected override bool TryCreateSimpleFontFromFontFile(out SimpleFont font)
        {
            FontStream fontStream = null;

            if (this.FontDescriptor != null)
            {
                fontStream = this.FontDescriptor.FontFile2 ?? this.FontDescriptor.FontFile3;
            }

            if (fontStream != null)
            {
                FontSource fontSource = fontStream.ToFontSource(FontType.TrueType);
                font = new TrueTypeFont(this.BaseFont.Value, fontSource);
                font.FontFileInfo.Value = fontStream.ToFileInfo();

                return true;
            }
            else
            {
                font = null;
                return false;
            }
        }

        protected override SimpleFont CreateSimpleFontWithoutFontFile()
        {
            return new TrueTypeFont(this.BaseFont.Value, EmptyFontSource.Instance);
        }

        protected override bool TryCreateFontFromFontProperties(FontProperties fontProperties, out FontBase font)
        {
            OpenTypeFontSource fontSource = FontsRepository.FontsManager.GetOpenTypeFontSource(fontProperties);
            font = fontSource != null ? new TrueTypeFont(this.BaseFont.Value, fontSource) : null;

            return font != null;
        }
    }
}
