﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Core.Fonts;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.CMaps;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class Type0FontObject : FontObject
    {
        private readonly ReferenceProperty<EncodingBaseObject> encoding;
        private readonly ReferenceProperty<PdfArray> descendantFonts;
        private CidFontObject cidFont;

        public Type0FontObject()
        {
            this.cidFont = null;
            this.encoding = this.RegisterReferenceProperty<EncodingBaseObject>(new PdfPropertyDescriptor("Encoding", true));
            this.descendantFonts = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor("DescendantFonts", true, PdfPropertyRestrictions.ContainsOnlyIndirectReferences));
        }

        public EncodingBaseObject Encoding
        {
            get
            {
                return this.encoding.GetValue();
            }
            set
            {
                this.encoding.SetValue(value);
            }
        }

        public PdfArray DescendantFonts
        {
            get
            {
                return this.descendantFonts.GetValue();
            }
            set
            {
                this.descendantFonts.SetValue(value);
            }
        }

        private CidFontObject CidFont
        {
            get
            {
                return this.cidFont;
            }
        }

        protected override IEnumerable<CharInfo> GetCharactersOverride(PdfString str)
        {
           if (this.Encoding != null && this.CidFont != null)
           {
               List<CharInfo> glyphs = new List<CharInfo>();
               foreach (CharCode charCode in this.Encoding.BuildCharCodes(str))
               {
                   glyphs.Add(new CharInfo(this.GetToUnicode(charCode), charCode));
               }

               return glyphs;
           }

            return base.GetCharactersOverride(str);
        }

        protected override void CopyPropertiesFromOverride(IPdfExportContext context, FontBase font)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(font, "font");

            base.CopyPropertiesFromOverride(context, font);

            this.DescendantFonts = new PdfArray();

            // TODO: Encoding should be imported and exported correctly.
            this.Encoding = EncodingBaseObject.IdentityH;

            switch (font.Type)
            {
                case FontType.CidType0Font:
                    CidFontType0Object cidFont0 = new CidFontType0Object();
                    cidFont0.CopyPropertiesFrom(context, font);
                    this.DescendantFonts.Add(cidFont0);
                    this.BaseFont = new PdfName(cidFont0.BaseFont.Value);
                    break;

                case FontType.CidType2Font:
                    CidFontType2Object cidFont2 = new CidFontType2Object();
                    cidFont2.CopyPropertiesFrom(context, font);
                    this.DescendantFonts.Add(cidFont2);
                    this.BaseFont = new PdfName(cidFont2.BaseFont.Value);
                    break;

                default:
                    throw new InvalidOperationException(string.Format("Cannot create CIDFont from font type: {0}", font.Type));
            }
        }

        protected override void CopyPropertiesToFont(FontBase font, PostScriptReader reader, IPdfImportContext context)
        {
            CidFontBase cidFont = font as CidFontBase;

            if (cidFont != null)
            {
                this.Encoding.CopyToProperty(cidFont.Encoding, (cMap) => cMap.Encoding);
                this.CidFont.CopyPropertiesToFont(cidFont, reader, context);
            }

            base.CopyFontDescriptorPropertiesToFont(this.CidFont.FontDescriptor, font);
            base.CopyPropertiesToFont(font, reader, context);
        }

        protected override bool TryGetFontFamily(out FontFamily fontFamily)
        {
            return this.CidFont.TryGetFontFamily(out fontFamily);
        }

        protected override bool TryCreateFontFromFontFile(out FontBase font)
        {
            return this.CidFont.TryCreateFontFromFontFile(out font);
        }

        protected override FontBase CreateFontWithoutFontFile()
        {
            return this.CidFont.CreateFontWithoutFontFile();
        }

        protected override void PrepareForImport(PostScriptReader reader, IPdfImportContext context)
        {
            CidFontObject descendant;
            if (this.DescendantFonts.TryGetElement(reader, context, 0, out descendant))
            {
                this.cidFont = descendant;
            }
            else
            {
                throw new InvalidOperationException("Cannot find descendant CidFont!");
            }

            base.PrepareForImport(reader, context);
        }

        protected override bool TryCreateFontFromFontProperties(FontProperties fontProperties, out FontBase font)
        {
            return this.CidFont.TryCreateFontFromFontProperties(fontProperties, out font);
        }
    }
}
