﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class Type1FontStream : Type2FontStream
    {
        private readonly DirectProperty<PdfInt> length2;
        private readonly DirectProperty<PdfInt> length3;

        public Type1FontStream()
        {
            this.length2 = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Length2", true));
            this.length3 = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Length3", true));
        }

        public PdfInt Length2
        {
            get
            {
                return this.length2.GetValue();
            }
            set
            {
                this.length2.SetValue(value);
            }
        }

        public PdfInt Length3
        {
            get
            {
                return this.length3.GetValue();
            }
            set
            {
                this.length3.SetValue(value);
            }
        }

        public override Fixed.Model.Fonts.FontFileInfo ToFileInfo()
        {
            var fileInfo = base.ToFileInfo();

            fileInfo.FileType.Value = Fixed.Model.Fonts.FontFileType.FontFile;

            if (this.Length2 != null)
            {
                fileInfo.Length2.Value = this.Length2.Value;
            }

            if (this.Length3 != null)
            {
                fileInfo.Length3.Value = this.Length3.Value;
            }

            return fileInfo;
        }

        public override FontSource ToFontSource(Fixed.Model.Fonts.FontType fontType)
        {
            switch (fontType)
            {
                case Fixed.Model.Fonts.FontType.Type1:
                case Fixed.Model.Fonts.FontType.Standard:
                    return new Type1FontSource(this.Data);
                default:
                    throw new InvalidOperationException(string.Format("Cannot create {0} font type from FontFile!", fontType));
            }
        }
    }
}
