﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class Type2FontStream : FontStream
    {
        private readonly DirectProperty<PdfInt> length1;

        public Type2FontStream()
        {
            this.length1 = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("Length1", true));
        }

        public PdfInt Length1
        {
            get
            {
                return this.length1.GetValue();
            }
            set
            {
                this.length1.SetValue(value);
            }
        }

        public override Fixed.Model.Fonts.FontFileInfo ToFileInfo()
        {
            var fileInfo = new Fixed.Model.Fonts.FontFileInfo();

            fileInfo.FileType.Value = Fixed.Model.Fonts.FontFileType.FontFile2;

            if (this.Length1 != null)
            {
                fileInfo.Length1.Value = this.Length1.Value;
            }

            if (this.Metadata != null)
            {
                fileInfo.Metadata.Value = this.Metadata.Data;
            }

            return fileInfo;
        }

        public override FontSource ToFontSource(Fixed.Model.Fonts.FontType fontType)
        {
            switch(fontType)
            {
                case Fixed.Model.Fonts.FontType.TrueType:
                case Fixed.Model.Fonts.FontType.CidType2Font:
                    return new OpenTypeFontSource(new OpenTypeFontReader(this.Data));
                default:
                    throw new InvalidOperationException(string.Format("Cannot create {0} font type from FontFile2!", fontType));
            }
        }

        protected override void CopyPropertiesFromData()
        {
            Guard.ThrowExceptionIfNull(this.Data, "Data");

            this.Length1 = new PdfInt(this.Data.Length);
        }
    }
}
