﻿using System;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Fonts.Type1;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts
{
    internal class Type3FontStream : Type1FontStream
    {
        private readonly DirectProperty<PdfName> subtype;

        public Type3FontStream()
        {
            this.subtype = this.RegisterDirectProperty<PdfName>(new PdfPropertyDescriptor("Subtype", true));
        }

        public PdfName Subtype
        {
            get
            {
                return this.subtype.GetValue();
            }
            set
            {
                this.subtype.SetValue(value);
            }
        }

        private Fixed.Model.Fonts.FontFileSubType FileSubType
        {
            get
            {
                Guard.ThrowExceptionIfNull(this.Subtype, "Subtype");

                object parseResult = Enum.Parse(typeof(Fixed.Model.Fonts.FontFileSubType), this.Subtype.Value, true);
                FontFileSubType value = (Fixed.Model.Fonts.FontFileSubType)parseResult;

                return value;
            }
        }

        public override Fixed.Model.Fonts.FontFileInfo ToFileInfo()
        {
            var fileInfo = base.ToFileInfo();

            fileInfo.FileType.Value = Fixed.Model.Fonts.FontFileType.FontFile3;

            if (this.Subtype != null)
            {
                fileInfo.FileSubType.Value = this.FileSubType;
            }

            return fileInfo;
        }

        public override FontSource ToFontSource(FontType fontType)
        {
            switch (fontType)
            {
                case FontType.Type1:
                case FontType.Standard:
                case FontType.CidType0Font:
                    CFFFontSource cff = this.CreateCffFontSource(fontType);
                    return cff ?? (FontSource)EmptyFontSource.Instance;
                case FontType.TrueType:
                case FontType.CidType2Font:
                    return this.CreateOpenTypeFontSource(fontType);
                default:
                    throw new NotSupportedException(string.Format("Not supported font type: {0}", fontType));
            }
        }

        private OpenTypeFontSource CreateOpenTypeFontSource(FontType fontType)
        {
            switch(this.FileSubType)
            {
                case FontFileSubType.OpenType:
                    return new OpenTypeFontSource(new OpenTypeFontReader(this.Data));
                default:
                    throw new InvalidOperationException(string.Format("Cannot create {0} font type from FontFile3 with {1} subtype!", fontType, this.FileSubType));
            }
        }

        private CFFFontSource CreateCffFontSource(FontType fontType)
        {
            CFFFontSource cffFontSource = null;

            switch(this.FileSubType)
            {
                case FontFileSubType.Type1C:
                case FontFileSubType.CIDFontType0C:
                    cffFontSource = this.GetFontSourceFromCffFontFile();
                    break;
                case FontFileSubType.OpenType:
                    cffFontSource = this.GetFontSourceFromOpenTypeFontSource();
                    break;
                default:
                    throw new NotSupportedException(string.Format("Not supported file subtype: {0}", this.FileSubType));
            }

            return cffFontSource;
        }

        private CFFFontSource GetFontSourceFromCffFontFile()
        {
            CFFFontFile cffFile = new CFFFontFile(this.Data);
            return cffFile.FontSource;
        }

        private CFFFontSource GetFontSourceFromOpenTypeFontSource()
        {
            OpenTypeFontSource openTypeSource = new OpenTypeFontSource(new OpenTypeFontReader(this.Data));

            if (openTypeSource.Outlines == Core.Fonts.OpenType.Outlines.OpenType)
            {
                return openTypeSource.CFF;
            }
            else
            {
                return null;
            }
        }
    }
}
