﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms
{
    internal class AcroFormObject : PdfObject, IResourceHolder
    {
        public const string FieldsName = "Fields";
        public const string NeedAppearancesName = "NeedAppearances";
        public const string SigFlagsName = "SigFlags";
        public const string COName = "CO";
        public const string DRName = "DR";
        public const string DAName = "DA";
        public const string QName = "Q";
        public const string XFAName = "XFA";

        private readonly ReferenceProperty<FormFieldsTree> fields;
        private readonly DirectProperty<PdfBool> needAppearance;
        private readonly DirectProperty<PdfInt> sigFlags;
        private readonly ReferenceProperty<PdfArray> calculationOrder;
        private readonly DirectProperty<PdfResource> defaultResources;
        private readonly ReferenceProperty<PdfString> defaultAppearance;
        private readonly DirectProperty<PdfInt> quadding;
        private readonly ReferenceProperty<XFAStream> xfa;
        
        public AcroFormObject()
        {
            this.fields = this.RegisterReferenceProperty<FormFieldsTree>(new PdfPropertyDescriptor(FieldsName, true, true));
            this.needAppearance = this.RegisterDirectProperty<PdfBool>(new PdfPropertyDescriptor(NeedAppearancesName), new PdfBool(false));
            this.sigFlags = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor(SigFlagsName), new PdfInt(0));
            this.calculationOrder = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor(COName));
            this.defaultResources = this.RegisterDirectProperty<PdfResource>(new PdfPropertyDescriptor(DRName));
            this.defaultAppearance = this.RegisterReferenceProperty<PdfString>(new PdfPropertyDescriptor(DAName));
            this.quadding = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor(QName), new PdfInt(0));
            this.xfa = this.RegisterReferenceProperty<XFAStream>(new PdfPropertyDescriptor(XFAName));
        }

        public FormFieldsTree Fields
        {
            get
            {
                return this.fields.GetValue();
            }
            set
            {
                this.fields.SetValue(value);
            }
        }

        public PdfBool NeedAppearance
        {
            get
            {
                return this.needAppearance.GetValue();
            }
            set
            {
                this.needAppearance.SetValue(value);
            }
        }

        public PdfInt SigFlags
        {
            get
            {
                return this.sigFlags.GetValue();
            }
            set
            {
                this.sigFlags.SetValue(value);
            }
        }

        public PdfArray CalculationOrder
        {
            get
            {
                return this.calculationOrder.GetValue();
            }
            set
            {
                this.calculationOrder.SetValue(value);
            }
        }

        public PdfResource Resources
        {
            get
            {
                return this.defaultResources.GetValue();
            }
            set
            {
                this.defaultResources.SetValue(value);
            }
        }

        public PdfString DefaultAppearance
        {
            get
            {
                return this.defaultAppearance.GetValue();
            }
            set
            {
                this.defaultAppearance.SetValue(value);
            }
        }

        public PdfInt Quadding
        {
            get
            {
                return this.quadding.GetValue();
            }
            set
            {
                this.quadding.SetValue(value);
            }
        }

        public XFAStream XFA
        {
            get
            {
                return this.xfa.GetValue();
            }
            set
            {
                this.xfa.SetValue(value);
            }
        }
#if WPF
        internal void SetSignatureFlags()
        {
            FlagWriter<SignatureFlags> flagWriter = new FlagWriter<SignatureFlags>();
            flagWriter.SetFlag(SignatureFlags.SignaturesExist);
            flagWriter.SetFlag(SignatureFlags.AppendOnly);

            this.SigFlags = new PdfInt(flagWriter.ResultFlags);
        }
#endif
    }
}
