﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms
{
    internal class FieldCreationContext
    {
        private readonly IPdfImportContext importContext;
        private readonly FormFieldNode node;
        private readonly FlagReader<FieldFlag> flagReader;

        public FieldCreationContext(IPdfImportContext importContext, FormFieldNode node)
        {
            this.importContext = importContext;
            this.node = node;
            this.flagReader = new FlagReader<FieldFlag>(this.node.FieldFlags.Value);
        }

        public string FullName
        {
            get
            {
                return this.node.FullName;
            }
        }

        public string UserInterfaceName
        {
            get
            {
                return this.node.UserInterfaceName == null ? null : this.node.UserInterfaceName.ToString();
            }
        }

        public string MappingName
        {
            get
            {
                return this.node.MappingName == null ? null : this.node.MappingName.ToString();
            }
        }

        public string FieldType
        {
            get
            {
                return this.node.FieldType.Value;
            }
        }

        public FlagReader<FieldFlag> FieldFlags
        {
            get
            {
                return this.flagReader;
            }
        }

        public int? MaxLen
        {
            get
            {
                return this.node.MaxLengthOfInputCharacters == null ? (int?)null : this.node.MaxLengthOfInputCharacters.Value;
            }
        }

        public bool HasWidgetsWithDifferentOnStateNames
        {
            get
            {
                string previousOnState = null;

                foreach (WidgetObject widgetObject in this.importContext.GetChildWidgets(this.node))
                {
                    string onStateName;
                    if (widgetObject.TryGetOnStateName(out onStateName))
                    {
                        if (previousOnState == null)
                        {
                            previousOnState = onStateName;
                        }
                        else if (previousOnState != onStateName)
                        {
                            return true;
                        }
                    }
                }

                return false;
            }
        }
    }
}
