﻿using System;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms
{
    internal class FieldsFactory
    {
        public static FormField CreateField(FieldCreationContext creationContext)
        {
            FormField field;

            switch (creationContext.FieldType)
            {
                case FormFieldTypes.Button:
                    field = CreateButtonField(creationContext);
                    break;
                case FormFieldTypes.Choice:
                    field = CreateChoiceField(creationContext);
                    break;
                case FormFieldTypes.Text:
                    field = CreateTextField(creationContext);
                    break;
                case FormFieldTypes.Signature:
                    field = new SignatureField(creationContext.FullName);
                    break;
                default:
                    throw new NotSupportedException(string.Format("Not supported field type: {0}", creationContext.FieldType));
            }

            field.IsReadOnly = creationContext.FieldFlags.IsSet(FieldFlag.ReadOnly);
            field.IsRequired = creationContext.FieldFlags.IsSet(FieldFlag.Required);
            field.ShouldBeSkipped = creationContext.FieldFlags.IsSet(FieldFlag.NoExport);
            field.UserInterfaceName = creationContext.UserInterfaceName;
            field.MappingName = creationContext.MappingName;

            return field;
        }

        private static FormField CreateButtonField(FieldCreationContext creationContext)
        {
            FormField button;

            if (creationContext.FieldFlags.IsSet(FieldFlag.PushButton))
            {
                button = new PushButtonField(creationContext.FullName);
            }
            else if (creationContext.FieldFlags.IsSet(FieldFlag.Radio) || creationContext.HasWidgetsWithDifferentOnStateNames)
            {
                button = new RadioButtonField(creationContext.FullName)
                {
                    AllowToggleOff = !creationContext.FieldFlags.IsSet(FieldFlag.NoToggleToOff),
                    ShouldUpdateRadiosInUnison = creationContext.FieldFlags.IsSet(FieldFlag.RadiosInUnison)
                };
            }
            else
            {
                button = new CheckBoxField(creationContext.FullName);
            }

            return button;
        }

        private static FormField CreateChoiceField(FieldCreationContext creationContext)
        {
            ChoiceField choiceField;

            if(creationContext.FieldFlags.IsSet(FieldFlag.Combo))
            {
                choiceField = new ComboBoxField(creationContext.FullName)
                {
                    HasEditableTextBox = creationContext.FieldFlags.IsSet(FieldFlag.Edit),
                    ShouldSpellCheck = !creationContext.FieldFlags.IsSet(FieldFlag.DoNotSpellCheck),
                };
            }
            else
            {
                choiceField = new ListBoxField(creationContext.FullName)
                {
                    AllowMultiSelection = creationContext.FieldFlags.IsSet(FieldFlag.MultiSelect)
                };
            }
            
            choiceField.ShouldCommitOnSelectionChange = creationContext.FieldFlags.IsSet(FieldFlag.CommitOnSelChange);

            return choiceField;
        }

        private static FormField CreateTextField(FieldCreationContext creationContext)
        {
            TextField textField;
            bool isCombSet = creationContext.FieldFlags.IsSet(FieldFlag.Comb);
            bool isMultiline = creationContext.FieldFlags.IsSet(FieldFlag.Multiline);
            bool isPassword = creationContext.FieldFlags.IsSet(FieldFlag.Password);
            bool isFileSelect = creationContext.FieldFlags.IsSet(FieldFlag.FileSelect);

            if (isCombSet && creationContext.MaxLen.HasValue && !isMultiline && !isPassword && !isFileSelect)
            {
                textField = new CombTextBoxField(creationContext.FullName)
                {
                    MaxLengthOfInputCharacters = creationContext.MaxLen.Value,
                };
            }
            else
            {
                textField = new TextBoxField(creationContext.FullName)
                {
                    MaxLengthOfInputCharacters = creationContext.MaxLen,
                    IsMultiline = isMultiline,
                    IsPassword = isPassword,
                    IsFileSelect = isFileSelect,
                    ShouldSpellCheck = !creationContext.FieldFlags.IsSet(FieldFlag.DoNotSpellCheck),
                    AllowScroll = !creationContext.FieldFlags.IsSet(FieldFlag.DoNotScroll),
                };
            }

            return textField;
        }
    }
}
