﻿using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.ContentElementWriters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms
{
    internal class VariableTextObjectPropertiesExporter<T>
        where T : IVariableTextPropertiesObject
    {
        private readonly T node;
        private readonly IPdfExportContext context;

        protected VariableTextObjectPropertiesExporter(T node, IPdfExportContext exportContext)
        {
            this.node = node;
            this.context = exportContext;
        }

        protected T Node
        {
            get
            {
                return this.node;
            }
        }

        protected IPdfExportContext Context
        {
            get
            {
                return this.context;
            }
        }

        protected void ExportVariableProperties(VariableTextProperties textProperties)
        {
            using (MemoryStream textStream = new MemoryStream())
            {
                PdfSingleLineWriter writer = new PdfSingleLineWriter(textStream);
                IPdfContentExportContext contentContext = this.Context.AcroFormContentExportContext;
                ContentElementWriters.TextPropertiesWriter.Write(writer, contentContext, textProperties.PropertiesOwner);
                PdfLiteralString pdfString = new PdfLiteralString(textStream.ToArray());

                this.Node.SetDefaultAppearance(pdfString);
            }

            this.ExportQuadding(textProperties.HorizontalAlignment);
        }

        private void ExportQuadding(HorizontalAlignment alignment)
        {
            PdfInt quadding;
            switch (alignment)
            {
                case HorizontalAlignment.Center:
                    quadding = new PdfInt(1);
                    break;
                case HorizontalAlignment.Right:
                    quadding = new PdfInt(2);
                    break;
                default:
                    quadding = new PdfInt(0);
                    break;
            }

            this.Node.SetQuadding(quadding);
        }
    }
}
