﻿using System;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms
{
    internal class VariableTextObjectPropertiesImporter<T>
        where T : IVariableTextPropertiesObject
    {
        private readonly T node;
        private readonly PostScriptReader reader;
        private readonly IRadFixedDocumentImportContext context;

        protected VariableTextObjectPropertiesImporter(T node, PostScriptReader reader, IRadFixedDocumentImportContext context)
        {
            this.node = node;
            this.reader = reader;
            this.context = context;
        }

        protected T Node
        {
            get
            {
                return this.node;
            }
        }

        protected PostScriptReader Reader
        {
            get
            {
                return this.reader;
            }
        }

        protected IRadFixedDocumentImportContext Context
        {
            get
            {
                return this.context;
            }
        }

        protected VariableTextProperties ReadTextProperties()
        {
            VariableTextProperties textProperties = new VariableTextProperties();
            AcroFormObject acroForm = this.context.Root.GetValue().AcroForm;
            PdfString appearance = this.node.GetDefaultAppearance(acroForm);

            if (appearance != null && appearance.Value.Length > 0)
            {
                byte[] data = appearance.Value;
                
                using (Stream stream = new MemoryStream(data))
                {
                    IPdfContentImportContext contentContext = this.Context.GetAcroFormContentImportContext();
                    ContentStreamInterpreter interpreter = new ContentStreamInterpreter(stream, contentContext);
                    interpreter.Execute();
                    interpreter.ApplyTextProperties(textProperties.PropertiesOwner);
                }
            }

            PdfInt quadding = this.node.GetQuadding(acroForm);

            switch (quadding.Value)
            {
                case 1:
                    textProperties.HorizontalAlignment = HorizontalAlignment.Center;
                    break;
                case 2:
                    textProperties.HorizontalAlignment = HorizontalAlignment.Right;
                    break;
                default:
                    textProperties.HorizontalAlignment = HorizontalAlignment.Left;
                    break;
            }

            return textProperties;
        }
    }
}
