﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Functions
{
    internal class ExponentialInterpolationFunctionObject : FunctionObject
    {
        private readonly DirectProperty<PdfArray> c0;
        private readonly DirectProperty<PdfArray> c1;
        private readonly DirectProperty<PdfReal> number;

        public ExponentialInterpolationFunctionObject()
        {
            this.c0 = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("C0"), new PdfArray(new PdfReal(0.0)));
            this.c1 = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("C1"), new PdfArray(new PdfReal(1.0)));
            this.number = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("N", true));
        }

        public PdfArray C0
        {
            get
            {
                return this.c0.GetValue();
            }
            set
            {
                this.c0.SetValue(value);
            }
        }

        public PdfArray C1
        {
            get
            {
                return this.c1.GetValue();
            }
            set
            {
                this.c1.SetValue(value);
            }
        }

        public PdfReal Number
        {
            get
            {
                return this.number.GetValue();
            }
            set
            {
                this.number.SetValue(value);
            }
        }

        protected override bool HasStreamData
        {
            get
            {
                return false;
            }
        }

        public override FunctionBase ToFunction(PostScriptReader reader, IPdfImportContext context)
        {
            double n = this.Number.Value;
            double[] c0 = this.C0.ToArray<PdfReal, double>(reader, context);
            double[] c1 = this.C1.ToArray<PdfReal, double>(reader, context);
            double[] domain = this.Domain.ToArray<PdfReal, double>(reader, context);
            double[] range = this.Range.ToArray<PdfReal, double>(reader, context);

            ExponentialInterpolationFunction function = new ExponentialInterpolationFunction(n, c0, c1, domain, range);

            return function;
        }

        protected override void CopyPropertiesFromOverride(IPdfExportContext context, FunctionBase function)
        {
            ExponentialInterpolationFunction exponentialFunction = (ExponentialInterpolationFunction)function;
            this.Number = new PdfReal(exponentialFunction.N);
            this.C0 = exponentialFunction.C0.ToPdfArray();
            this.C1 = exponentialFunction.C1.ToPdfArray();
        }
    }
}
