﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Functions
{
    internal class SampledFunctionObject : FunctionObject
    {
        private readonly DirectProperty<PdfArray> size;
        private readonly DirectProperty<PdfInt> bitsPerSample;
        private readonly DirectProperty<PdfArray> encode;
        private readonly DirectProperty<PdfArray> decode;

        public SampledFunctionObject()
        {
            this.size = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Size", true));
            this.bitsPerSample = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("BitsPerSample", true));
            this.encode = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Encode"));
            this.decode = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Decode"));
        }
        
        public PdfArray Size
        {
            get
            {
                return this.size.GetValue();
            }
            set
            {
                this.size.SetValue(value);
            }
        }

        public PdfInt BitsPerSample
        {
            get
            {
                return this.bitsPerSample.GetValue();
            }
            set
            {
                this.bitsPerSample.SetValue(value);
            }
        }

        public PdfArray Encode
        {
            get
            {             
                return this.encode.GetValue();
            }
            set
            {
                this.encode.SetValue(value);
            }
        }

        public PdfArray Decode
        {
            get
            {
                return this.decode.GetValue();
            }
            set
            {
                this.decode.SetValue(value);
            }
        }

        protected override bool HasStreamData
        {
            get
            {
                return true;
            }
        }

        public override FunctionBase ToFunction(PostScriptReader reader, IPdfImportContext context)
        {
            byte[] data = this.FunctionData;
            int bitsPerSample = this.BitsPerSample.Value;
            int[] size = this.Size.ToArray<PdfInt, int>(reader, context);
            double[] domain = this.Domain.ToArray<PdfReal, double>(reader, context);
            double[] range = this.Range.ToArray<PdfReal, double>(reader, context);
            double[] encode = this.Encode.ToArray<PdfReal, double>(reader, context);
            double[] decode = this.Decode.ToArray<PdfReal, double>(reader, context);

            SampledFunction function = new SampledFunction(data, size, bitsPerSample, domain, range, encode, decode);

            return function;
        }

        protected override void CopyPropertiesFromOverride(IPdfExportContext context, FunctionBase function)
        {
            SampledFunction sampledFunction = (SampledFunction)function;
            this.BitsPerSample = new PdfInt(sampledFunction.BitsPerSample);
            this.Size = sampledFunction.Size.ToPdfArray();
            this.Encode = sampledFunction.Encode.ToPdfArray();
            this.Decode = sampledFunction.Decode.ToPdfArray();
        }
    }
}
