﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Functions
{
    internal class StitchingFunctionObject : FunctionObject
    {
        private readonly DirectProperty<PdfArray> functions;
        private readonly DirectProperty<PdfArray> bounds;
        private readonly DirectProperty<PdfArray> encode;

        public StitchingFunctionObject()
        {
            this.functions = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Functions", true));
            this.bounds = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Bounds", true));
            this.encode = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Encode", true));
        }

        public override PdfElementType ExportAs
        {
            get
            {
                return PdfElementType.PdfObject;
            }
        }

        public PdfArray Functions
        {
            get
            {
                return this.functions.GetValue();
            }
            set
            {
                this.functions.SetValue(value);
            }
        }

        public PdfArray Bounds
        {
            get
            {
                return this.bounds.GetValue();
            }
            set
            {
                this.bounds.SetValue(value);
            }
        }

        public PdfArray Encode
        {
            get
            {
                return this.encode.GetValue();
            }
            set
            {
                this.encode.SetValue(value);
            }
        }

        protected override bool HasStreamData
        {
            get
            {
                return false;
            }
        }

        public override FunctionBase ToFunction(PostScriptReader reader, IPdfImportContext context)
        {
            FunctionBase[] functions = new FunctionBase[this.Functions.Count];

            for (int i = 0; i < functions.Length; i++)
            {
                FunctionObject functionObject;
                if (!this.Functions.TryGetElement(reader, context, i, out functionObject))
                {
                    throw new InvalidOperationException(string.Format("Cannot get function at index: {0}", i));
                }

                FunctionBase function = functionObject.ToFunction(reader, context);
                functions[i] = function;
            }

            double[] domain = this.Domain.ToArray<PdfReal, double>(reader, context);
            double[] range = this.Range.ToArray<PdfReal, double>(reader, context);
            double[] bounds = this.Bounds.ToArray<PdfReal, double>(reader, context);
            double[] encode = this.Encode.ToArray<PdfReal, double>(reader, context);

            StitchingFunction stitchingFunction = new StitchingFunction(functions, bounds, encode, domain, range);

            return stitchingFunction;
        }

        protected override void CopyPropertiesFromOverride(IPdfExportContext context, FunctionBase function)
        {
            StitchingFunction stitchingFunction = (StitchingFunction)function;

            this.Bounds = stitchingFunction.Bounds.ToPdfArray();
            this.Encode = stitchingFunction.Encode.ToPdfArray();

            this.Functions = new PdfArray();

            for (int i = 0; i < stitchingFunction.Functions.Length; i++)
            {
                FunctionBase innerFunction = stitchingFunction.Functions[i];
                FunctionObject functionObject = CreateFunction(new PdfInt((int)innerFunction.FunctionType));
                functionObject.CopyPropertiesFrom(context, innerFunction);
                IndirectObject indirectObject = context.CreateIndirectObject(functionObject);
                this.Functions.Add(indirectObject.Reference);
            }
        }
    }
}
