﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.GraphicsState
{
    internal class ExtGStateObject : PdfObject
    {
        private readonly DirectProperty<PdfReal> alphaConstant;
        private readonly DirectProperty<PdfReal> strokeAlphaConstant;
        private readonly DirectProperty<PdfBool> alphaSource;

        public ExtGStateObject()
        {
            this.alphaConstant = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("ca"));
            this.strokeAlphaConstant = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("CA"));
            this.alphaSource = this.RegisterDirectProperty<PdfBool>(new PdfPropertyDescriptor("AIS"));
        }

        public PdfReal AlphaConstant
        {
            get
            {
                return this.alphaConstant.GetValue();
            }
            set
            {
                this.alphaConstant.SetValue(value);
            }
        }

        public PdfReal StrokeAlphaConstant
        {
            get
            {
                return this.strokeAlphaConstant.GetValue();
            }
            set
            {
                this.strokeAlphaConstant.SetValue(value);
            }
        }

        public PdfBool AlphaSource
        {
            get
            {
                return this.alphaSource.GetValue();
            }
            set
            {
                this.alphaSource.SetValue(value);
            }
        }

        internal void CopyPropertiesFrom(Export.IPdfExportContext context,  Fixed.FormatProviders.Pdf.Data.ExtGState state)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(state, "resource");

            if (state.AlphaConstant.HasValue)
            {
                this.AlphaConstant = new PdfReal(state.AlphaConstant.Value);
            }

            if (state.StrokeAlphaConstant.HasValue)
            {
                this.StrokeAlphaConstant = new PdfReal(state.StrokeAlphaConstant.Value);
            }

            if (state.AlphaSource.HasValue)
            {
                this.AlphaSource = new PdfBool(state.AlphaSource.Value);
            }
        }

        internal Pdf.Data.ExtGState ToExtGState()
        {
            Pdf.Data.ExtGState state = new Pdf.Data.ExtGState();
            if (this.AlphaSource != null)
            {
                state.AlphaSource = this.AlphaSource.Value;
            }

            if (this.AlphaConstant != null)
            {
                state.AlphaConstant = this.AlphaConstant.Value;
            }

            if (this.StrokeAlphaConstant != null)
            {
                state.StrokeAlphaConstant = this.StrokeAlphaConstant.Value;
            }

            return state;
        }
    }
}
