﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata
{
    internal class DocumentInfo : MetadataStream
    {
        private IEnumerable<DescriptionElement> descriptions;

        public override IEnumerable<DescriptionElement> Descriptions
        {
            get
            {
                return this.descriptions;
            }
        }

        public void CopyPropertiesFrom(IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            List<DescriptionElement> descriptions = new List<DescriptionElement>();

            AddPdfComplianceLevelDescriptionElement(context, descriptions);
            AddPdfDocumentInfoDescriptionElement(context, descriptions);

            string createDate = DateTime.Now.ToXmpDate();
            descriptions.Add(new DescriptionElement(
                // CreateDate should be preserved in the model in case the document is imported.
                new XmpDataElement(XmpNamespaces.Xmp, "CreateDate", createDate),
                new XmpDataElement(XmpNamespaces.Xmp, "ModifyDate", createDate),
                new XmpDataElement(XmpNamespaces.Xmp, "CreatorTool", "Telerik PdfProcessing")));

            this.descriptions = descriptions;
        }

        private static void AddPdfComplianceLevelDescriptionElement(IPdfExportContext context, List<DescriptionElement> descriptions)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(descriptions, "descriptions");

            int part;
            string conformance;
            switch (context.Settings.ComplianceLevel)
            {
                case PdfComplianceLevel.PdfA1B:
                    part = 1;
                    conformance = "B";
                    break;

                case PdfComplianceLevel.PdfA2B:
                    part = 2;
                    conformance = "B";
                    break;

                case PdfComplianceLevel.PdfA2U:
                    part = 2;
                    conformance = "U";
                    break;

                case PdfComplianceLevel.PdfA3B:
                    part = 3;
                    conformance = "B";
                    break;

                case PdfComplianceLevel.PdfA3U:
                    part = 3;
                    conformance = "U";
                    break;

                default:
                    return;
            }

            descriptions.Add(new DescriptionElement(
                  new XmpDataElement(XmpNamespaces.PdfAId, "part", part),
                  new XmpDataElement(XmpNamespaces.PdfAId, "conformance", conformance)));
        }

        private static void AddPdfDocumentInfoDescriptionElement(IPdfExportContext context, List<DescriptionElement> descriptions)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(descriptions, "descriptions");

            List<XmpDataElement> elements = new List<XmpDataElement>();
            if (!string.IsNullOrEmpty(context.DocumentInfo.Title))
            {
                elements.Add(new XmpDataElement(XmpNamespaces.Dc, "title", new LanguageAlternativeElement(context.DocumentInfo.Title)));
            }

            if (!string.IsNullOrEmpty(context.DocumentInfo.Author))
            {
                elements.Add(new XmpDataElement(XmpNamespaces.Dc, "creator", new SequenceElement(context.DocumentInfo.Author)));
            }

            if (!string.IsNullOrEmpty(context.DocumentInfo.Description))
            {
                elements.Add(new XmpDataElement(XmpNamespaces.Dc, "description", new LanguageAlternativeElement(context.DocumentInfo.Description)));
            }

            if (elements.Count > 0)
            {
                descriptions.Add(new DescriptionElement(elements.ToArray()));
            }
        }
    }
}
