﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Structure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata
{
    internal abstract class MetadataStream : PdfStreamObjectBase
    {
        public abstract IEnumerable<DescriptionElement> Descriptions
        {
            get;
        }

        public override bool ShouldEncryptData
        {
            get
            {
                return false;
            }
        }

        protected override PdfArray GetExportFilters(IPdfExportContext context)
        {
            if(context.Settings.ComplianceLevel == PdfComplianceLevel.None)
            {
                return base.GetExportFilters(context);
            }
            else
            {
                return new PdfArray();
            }
        }

        protected override byte[] GetData(IPdfExportContext context)
        {
            using (Stream stream = new MemoryStream())
            {
                using (XmpWriter writer = new XmpWriter(stream))
                {
                    new HeaderElement().Write(writer);
                    new XmpMetaElement(this.Descriptions).Write(writer);
                    new PaddingElement().Write(writer);
                    new TrailerElement().Write(writer);
                }

                return stream.ReadAllBytes();
            }
        }
    }
}
