﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Namespaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Structure;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Data
{
    internal class XmpDataElement : XmpDataElementBase
    {
        private readonly string name;
        private readonly XmlNamespace ns;
        private readonly XmpDataElementBase child;

        public XmpDataElement(XmlNamespace ns, string name, object value)
            : this(ns, name, value.ToString())
        {
        }

        public XmpDataElement(XmlNamespace ns, string name, string value)
            : this(ns, name)
        {
            this.InnerText = value;
        }

        public XmpDataElement(XmlNamespace ns, string name)
        {
            Guard.ThrowExceptionIfNull(ns, "namespace");
            Guard.ThrowExceptionIfNull(name, "name");

            this.ns = ns;
            this.name = name;
        }

        public XmpDataElement(XmlNamespace ns, string name, XmpDataElementBase child)
            : this(ns, name)
        {
            Guard.ThrowExceptionIfNull(child, "child");

            this.child = child;
        }

        public override string Name
        {
            get
            {
                return this.name;
            }
        }

        public override XmlNamespace Namespace
        {
            get
            {
                return this.ns;
            }
        }

        protected override System.Collections.Generic.IEnumerable<Common.XmpElementBase> EnumerateInnerElements()
        {
            if (this.child != null)
            {
                yield return this.child;
            }
        }
    }
}
