﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Attributes;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Namespaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Model.Structure
{
    internal abstract class XmpDataElementBase : XmpElementBase
    {
        public override XmlNamespace Namespace
        {
            get
            {
                return XmpNamespaces.Rdf;
            }
        }

        public override void Write(IXmpWriter writer)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");

            this.WriteElementStart(writer);
            this.WriteAttributes(writer);

            if (!string.IsNullOrEmpty(this.InnerText))
            {
                writer.WriteValue(this.InnerText);
            }

            foreach (XmpElementBase innerElement in this.EnumerateInnerElements())
            {
                innerElement.Write(writer);
            }

            writer.WriteElementEnd();
        }

        protected virtual IEnumerable<XmpElementBase> EnumerateInnerElements()
        {
            return Enumerable.Empty<XmpElementBase>();
        }

        private void WriteAttributes(IXmpWriter writer)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");

            foreach (XmlAttribute attribute in this.Attributes)
            {
                if (attribute.ShouldExport())
                {
                    WriteAttribute(writer, attribute);
                }
            }
        }

    }
}
