﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.NamedTrees
{
    internal class DestinationsNamesTreeNode : PdfObject
    {
        public const string NamesName = "Names";
        public const string KidsName = "Kids";

        private readonly ReferenceProperty<NamedDestinationsArrayObject> dests;
        private readonly ReferenceProperty<PdfArray> kids;

        public DestinationsNamesTreeNode()
            : base()
        {
            this.dests = this.RegisterReferenceProperty<NamedDestinationsArrayObject>(new PdfPropertyDescriptor(NamesName, false, PdfPropertyRestrictions.MustBeIndirectReference));
            this.kids = this.RegisterReferenceProperty<PdfArray>(new PdfPropertyDescriptor(KidsName, false, PdfPropertyRestrictions.MustBeIndirectReference));
        }

        public NamedDestinationsArrayObject Destinations
        {
            get
            {
                return this.dests.GetValue();
            }
            set
            {
                this.dests.SetValue(value);
            }
        }

        public PdfArray Kids
        {
            get
            {
                return this.kids.GetValue();
            }
            set
            {
                this.kids.SetValue(value);
            }
        }

        internal IEnumerable<NamedDestinationObject> EnumerateChildDestinations(PostScriptReader reader, IPdfImportContext context)
        {
            if (this.Destinations != null)
            {
                return this.Destinations.Store;
            }
            else if (this.Kids != null)
            {
                return this.EnumerateKids(reader, context);
            }
            else
            {
                return Enumerable.Empty<NamedDestinationObject>();
            }
        }

        private IEnumerable<NamedDestinationObject> EnumerateKids(PostScriptReader reader, IPdfImportContext context)
        {
            for (int i = 0; i < this.Kids.Count; i++)
            {
                IPdfPrimitive kid = this.Kids[i];
                PdfObjectDescriptor descriptor = PdfObjectDescriptors.GetPdfObjectDescriptor(typeof(DestinationsNamesTreeNode));
                DestinationsNamesTreeNode node = (DestinationsNamesTreeNode)descriptor.Converter.Convert(typeof(DestinationsNamesTreeNode), reader, context, kid);

                foreach (NamedDestinationObject namedDestinationObject in node.EnumerateChildDestinations(reader, context))
                {
                    yield return namedDestinationObject;
                }
            }
        }
    }
}
