﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal class AxialShading : GradientShading
    {
        protected override void CopyPropertiesFromOverride(Export.IPdfContentExportContext context, Gradient gradient)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(gradient, "gradient");

            this.Coordinates = new PdfArray();
            this.Coordinates.Add(gradient.StartPoint.X.ToPdfReal());
            this.Coordinates.Add(gradient.StartPoint.Y.ToPdfReal());
            this.Coordinates.Add(gradient.EndPoint.X.ToPdfReal());
            this.Coordinates.Add(gradient.EndPoint.Y.ToPdfReal());

            base.CopyPropertiesFromOverride(context, gradient);
        }

        protected override Gradient ToColorOverride(Import.Parser.PostScriptReader reader, Import.IPdfContentImportContext context, Matrix matrix)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");
             
            var startPoint = new Point(this.Coordinates[0].ToReal(), this.Coordinates[1].ToReal());
            var endPoint = new Point(this.Coordinates[2].ToReal(), this.Coordinates[3].ToReal());
            var linearGradient = new LinearGradient(startPoint, endPoint);

#if SILVERLIGHT || NETSTANDARD
            double dist = endPoint.Minus(startPoint).Length();
#else
            double dist = (endPoint - startPoint).Length;
#endif
            this.InitializeGradientStops(reader, context, linearGradient, matrix, dist);
             
            return linearGradient;
        }
    }
}
