﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal abstract class PatternColorObject : ColorObjectBase
    {
        private readonly DirectProperty<PdfArray> matrix;

        public PatternColorObject()
        {
            this.matrix = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Matrix"), PdfArray.MatrixIdentity);
        }

        public PdfArray Matrix
        {
            get
            {
                return this.matrix.GetValue();
            }
            set
            {
                this.matrix.SetValue(value);
            }
        }

        public ColorObjectBase UnderlyingColor { get; set; }

        public static PatternColorObject CreateInstance(PdfInt type)
        {
            Guard.ThrowExceptionIfNull(type, "type");

            return CreateInstance(type.Value);
        }

        public static PatternColorObject CreateInstance(int type)
        {
            switch (type)
            {
                case 1: return new TilingPatternObject();

                case 2: return new ShadingPatternObject();

                default:

                    throw new NotSupportedException("Pattern type is not supported.");
            }
        }

        protected override void CopyPropertiesFromOverride(IPdfContentExportContext context, Fixed.Model.ColorSpaces.ColorBase color)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "color");

            RadFixedPage page = context.ContentRoot as RadFixedPage;

            Fixed.Model.ColorSpaces.PatternColor patternColor = (Fixed.Model.ColorSpaces.PatternColor)color;
            var transformMatrix = this.CreateInitialMatrix(page);
            this.Matrix = patternColor.Position.Matrix.MultiplyBy(transformMatrix).ToPdfArray();

            this.CopyPropertiesFromOverride(context, patternColor, transformMatrix);
        }

        protected Matrix CreateInitialMatrix(IContentRootElement element)
        {
            RadFixedPage page = element as RadFixedPage;
            Matrix matrix = new Matrix();
            if (page != null)
            {
#if !NETSTANDARD
                matrix = PdfExportContext.CalculateDipToPdfPointTransformation(page);
#else
                double dipToPoint = Media.Unit.DipToPoint(1);
                matrix = new Matrix(dipToPoint, 0, 0, -dipToPoint, 0, Media.Unit.DipToPoint(page.Size.Height));
#endif
            }

            return matrix;
        }

        protected abstract void CopyPropertiesFromOverride(IPdfContentExportContext context, Fixed.Model.ColorSpaces.PatternColor patternColor, Matrix transformMatrix);
    }
}
