﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal class RadialShading : GradientShading
    {
        protected override void CopyPropertiesFromOverride(Export.IPdfContentExportContext context, Fixed.Model.ColorSpaces.Gradient gradient)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(gradient, "gradient");

            RadialGradient radial = (RadialGradient)gradient;

            this.Coordinates = new PdfArray();
            this.Coordinates.Add(radial.StartPoint.X.ToPdfReal());
            this.Coordinates.Add(radial.StartPoint.Y.ToPdfReal());
            this.Coordinates.Add(radial.StartRadius.ToPdfReal());
            this.Coordinates.Add(radial.EndPoint.X.ToPdfReal());
            this.Coordinates.Add(radial.EndPoint.Y.ToPdfReal());
            this.Coordinates.Add(radial.EndRadius.ToPdfReal());

            base.CopyPropertiesFromOverride(context, gradient);
        }

        protected override Gradient ToColorOverride(Import.Parser.PostScriptReader reader, Import.IPdfContentImportContext context, Matrix matrix)
        {
            var startPoint = new Point(this.Coordinates[0].ToReal(), this.Coordinates[1].ToReal());
            var startRadius = this.Coordinates[2].ToReal();
            var endPoint = new Point(this.Coordinates[3].ToReal(), this.Coordinates[4].ToReal());
            var endRadius = this.Coordinates[5].ToReal();

#if SILVERLIGHT || NETSTANDARD
            double dist = endPoint.Minus(startPoint).Length();
#else
            double dist = (endPoint - startPoint).Length;
#endif
            double deltaRadius = Math.Abs(startRadius - endRadius);
            RadialGradient radialGradient = new RadialGradient(startPoint, endPoint, startRadius, endRadius);
            this.InitializeGradientStops(reader, context, radialGradient, matrix, deltaRadius + dist);

            return radialGradient;
        }
    }
}
