﻿using System;
using System.Linq;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal class ShadingPatternObject : PatternColorObject
    {
        private readonly ReferenceProperty<Shading> shading;

        public ShadingPatternObject()
        {
            this.shading = this.RegisterReferenceProperty<Shading>(new PdfPropertyDescriptor("Shading", true));

        }

        public Shading Shading
        {
            get
            {
                return this.shading.GetValue();
            }
            set
            {
                this.shading.SetValue(value);
            }
        }

        protected override void CopyPropertiesFromOverride(IPdfContentExportContext context, PatternColor patternColor, Matrix transformMatrix)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(patternColor, "patternColor");
            Guard.ThrowExceptionIfNull(transformMatrix, "transformMatrix");

            Gradient gradient = patternColor as Gradient;

            this.Shading = Shading.CreateInstance((int)gradient.GradientType);
            this.Shading.CopyPropertiesFrom(context, gradient);
        }

        public override ColorBase ToColor(PostScriptReader reader, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            return this.Shading.ToColor(reader, context, this.Matrix.ToMatrix(reader, context.Owner));
        }
    }
}
