﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal class Tiling : ContentStream, IResourceHolder
    {
        private readonly DirectProperty<PdfInt> paintType;
        private readonly DirectProperty<PdfInt> tilingType;
        private readonly DirectProperty<PdfArray> bbox;
        private readonly DirectProperty<PdfReal> xStep;
        private readonly DirectProperty<PdfReal> yStep;
        private readonly DirectProperty<PdfResource> resources;
        private readonly DirectProperty<PdfArray> matrix; 

        public Tiling()
        {
            this.matrix = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("Matrix"), PdfArray.MatrixIdentity);
            this.paintType = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("PaintType", true));
            this.tilingType = this.RegisterDirectProperty<PdfInt>(new PdfPropertyDescriptor("TilingType", true));
            this.bbox = this.RegisterDirectProperty<PdfArray>(new PdfPropertyDescriptor("BBox", true));
            this.xStep = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("XStep", true));
            this.yStep = this.RegisterDirectProperty<PdfReal>(new PdfPropertyDescriptor("YStep", true));
            this.resources = this.RegisterDirectProperty<PdfResource>(new PdfPropertyDescriptor("Resources", true)); 
        }

        public PdfArray Matrix
        {
            get
            {
                return this.matrix.GetValue();
            }
            set
            {
                this.matrix.SetValue(value);
            }
        }

        public PdfInt PaintType
        {
            get
            {
                return this.paintType.GetValue();
            }
            set
            {
                this.paintType.SetValue(value);
            }
        }

        public PdfInt TilingType
        {
            get
            {
                return this.tilingType.GetValue();
            }
            set
            {
                this.tilingType.SetValue(value);
            }
        }

        public PdfReal XStep
        {
            get
            {
                return this.xStep.GetValue();
            }
            set
            {
                this.xStep.SetValue(value);
            }
        }

        public PdfReal YStep
        {
            get
            {
                return this.yStep.GetValue();
            }
            set
            {
                this.yStep.SetValue(value);
            }
        }

        public PdfArray BBox
        {
            get
            {
                return this.bbox.GetValue();
            }
            set
            {
                this.bbox.SetValue(value);
            }
        }

        public PdfResource Resources
        {
            get
            {
                return this.resources.GetValue();
            }
            set
            {
                this.resources.SetValue(value);
            }
        }

        public bool IsColored
        {
            get
            {
                return this.PaintType.Value == 1;
            }
        } 

        public void CopyPropertiesFrom(IPdfExportContext context, TilingPatternObject tilingPattern, Fixed.Model.ColorSpaces.TilingBase tilingColor)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(tilingPattern, "tilingPattern");
            Guard.ThrowExceptionIfNull(tilingColor, "tilingColor");

            this.Resources = new PdfResource();
            var oldCanWriteColors = context.CanWriteColors;

            if (tilingColor.PaintType == Fixed.Model.ColorSpaces.PaintType.Uncolored)
            {
                context.CanWriteColors = false;
            }

            base.CopyPropertiesFrom(context, tilingPattern.Tiling, tilingColor);

            context.CanWriteColors = oldCanWriteColors;

            this.Matrix = tilingPattern.Matrix;
            this.BBox = tilingColor.BoundingBox.ToPdfArray();

            this.XStep = tilingColor.HorizontalSpacing.ToPdfReal();
            this.YStep = tilingColor.VerticalSpacing.ToPdfReal();
            this.TilingType = ((int)tilingColor.TilingType).ToPdfInt();
            this.PaintType = ((int)tilingColor.PaintType).ToPdfInt();
        }
    }
}
