﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns
{
    internal class TilingPatternObject : PatternColorObject
    {
        private readonly Tiling tiling;
        private Fixed.Model.ColorSpaces.Tiling tilingColor;

        public TilingPatternObject()
        {
            this.tiling = new Tiling();
        }

        public TilingPatternObject(Tiling tiling)
        {
            this.tiling = tiling;
            this.Matrix = tiling.Matrix;
        }

        public Tiling Tiling
        {
            get
            {
                return this.tiling;
            }
        }

        public override void Write(Export.PdfWriter writer, Export.IPdfExportContext context)
        {
            this.tiling.Write(writer, context);
        }

        protected override void CopyPropertiesFromOverride(Export.IPdfContentExportContext context, Fixed.Model.ColorSpaces.PatternColor patternColor, Matrix transformMatrix)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(patternColor, "patternColor");
            Guard.ThrowExceptionIfNull(transformMatrix, "transformMatrix");

            Fixed.Model.ColorSpaces.TilingBase tilingColor = patternColor as Fixed.Model.ColorSpaces.TilingBase;
            this.tiling.CopyPropertiesFrom(context.Owner, this, tilingColor);
        }

        public override Fixed.Model.ColorSpaces.ColorBase ToColor(PostScriptReader reader, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");
            if (this.tilingColor == null)
            {
                this.tilingColor = new Fixed.Model.ColorSpaces.Tiling();
                this.tilingColor.HorizontalSpacing = this.Tiling.XStep.Value;
                this.tilingColor.VerticalSpacing = this.Tiling.YStep.Value;
                this.tilingColor.TilingType = (TilingType)this.Tiling.TilingType.Value;

                var initialMatrix = this.CreateInitialMatrix(context.ContentRoot);
                this.tilingColor.Position = new MatrixPosition(this.Matrix.ToMatrix(reader, context.Owner).MultiplyBy(initialMatrix.InverseMatrix()));
                this.tilingColor.BoundingBox = this.Tiling.BBox.ToRect(reader, context.Owner);
                this.Tiling.CopyPropertiesTo(context.Owner, this.Tiling, this.tilingColor); 
            }

            if (this.Tiling.PaintType.Value == 2)
            {
                return new UncoloredTiling(this.tilingColor, this.UnderlyingColor.ToColor(reader, context) as SimpleColor);
            }
            else
            {
                return this.tilingColor;
            }
        }
    }
}
