﻿//
// Authors:
//   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
//
// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.

/* Portions of \code\PdfSharp\PdfSharp.Pdf.Security\PdfStandardSecurityHandler.cs file as modified by Progress EAD - Copyright © 2016 Progress EAD. All rights reserved.
*/

using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Encryption
{
    internal class RC4
    {
        private readonly byte[] state;

        public RC4()
        {
            this.state = new byte[256];
        }

        public void PrepareKey(byte[] key)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            this.PrepareKey(key, 0, key.Length);
        }

        public void PrepareKey(byte[] key, int offset, int length)
        {
            Guard.ThrowExceptionIfNull(key, "key");

            int idx1 = 0;
            int idx2 = 0;
            for (int idx = 0; idx < 256; idx++)
            {
                this.state[idx] = (byte)idx;
            }

            byte tmp;
            for (int idx = 0; idx < 256; idx++)
            {
                idx2 = (key[idx1 + offset] + this.state[idx] + idx2) & 255;
                tmp = this.state[idx];
                this.state[idx] = this.state[idx2];
                this.state[idx2] = tmp;
                idx1 = (idx1 + 1) % length;
            }
        }

        public void Encrypt(byte[] inputData, byte[] outputData)
        {
            Guard.ThrowExceptionIfNull(inputData, "inputData");
            Guard.ThrowExceptionIfNull(outputData, "outputData");

            int x = 0, y = 0;
            byte b;
            for (int idx = 0; idx < inputData.Length; idx++)
            {
                x = (x + 1) & 255;
                y = (this.state[x] + y) & 255;
                b = this.state[x];
                this.state[x] = this.state[y];
                this.state[y] = b;
                outputData[idx] = (byte)(inputData[idx] ^ this.state[(this.state[x] + this.state[y]) & 255]);
            }
        }
    }
}
