﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.CMaps;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
#if WPF
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DigitalSignature;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Functions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Encryption;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model
{
    internal static class PdfObjectDescriptors
    {
        private static readonly Dictionary<Type, PdfObjectDescriptor> pdfObjectDescriptors;

        static PdfObjectDescriptors()
        {
            pdfObjectDescriptors = new Dictionary<Type, PdfObjectDescriptor>();
            RegisterTypeDescriptor(typeof(DocumentCatalog), new PdfObjectDescriptor("Catalog"));
            RegisterTypeDescriptor(typeof(PageTreeNode), new PdfObjectDescriptor("Pages", new PageTypeAwareConverter(PageTreeNode.CreateInstance)));
            RegisterTypeDescriptor(typeof(Page), new PdfObjectDescriptor("Page", new PageTypeAwareConverter(PageTreeNode.CreateInstance)));
            RegisterTypeDescriptor(typeof(PdfReal), new PdfObjectDescriptor(new PdfRealConverter()));
            RegisterTypeDescriptor(typeof(ContentStream), new PdfObjectDescriptor(new ContentStreamConverter()));
            RegisterTypeDescriptor(typeof(XFAStream), new PdfObjectDescriptor(new XFAStreamConverter()));
            RegisterTypeDescriptor(typeof(SimpleFontEncoding), new PdfObjectDescriptor("Encoding", new SimpleFontEncodingConverter()));
            RegisterTypeDescriptor(typeof(FontObject), new PdfObjectDescriptor(new TypeAwareConverter<PdfName>(PdfFontsFactory.CreateInstance, PdfNames.SubType)));
            RegisterTypeDescriptor(typeof(Type1FontObject), new PdfObjectDescriptor("Font", new PdfName(PdfFontsFactory.Type1Font)));
            RegisterTypeDescriptor(typeof(TrueTypeFontObject), new PdfObjectDescriptor("Font", new PdfName(PdfFontsFactory.TrueTypeFont)));
            RegisterTypeDescriptor(typeof(Type0FontObject), new PdfObjectDescriptor("Font", new PdfName(PdfFontsFactory.Type0Font)));
            RegisterTypeDescriptor(typeof(CidFontObject), new PdfObjectDescriptor(new TypeAwareConverter<PdfName>(CidFontObject.CreateInstance, PdfNames.SubType)));
            RegisterTypeDescriptor(typeof(CidFontType0Object), new PdfObjectDescriptor("Font", new PdfName(CidFontObject.CidFontType0)));
            RegisterTypeDescriptor(typeof(CidFontType2Object), new PdfObjectDescriptor("Font", new PdfName(CidFontObject.CidFontType2)));
            RegisterTypeDescriptor(typeof(PrimitiveWrapper), new PdfObjectDescriptor(new PrimitiveWrapperConverter()));
            RegisterTypeDescriptor(typeof(ImageXObject), new PdfObjectDescriptor("XObject", new PdfName("Image")));
            RegisterTypeDescriptor(typeof(FormXObject), new PdfObjectDescriptor("XObject", new PdfName("Form")));
            RegisterTypeDescriptor(typeof(XObjectBase), new PdfObjectDescriptor(new TypeAwareConverterWithParameter<PdfName, PdfBool>(XObjectFactory.CreateInstance, PdfNames.SubType, ImageXObject.ImageMaskPropertyName)));
            RegisterTypeDescriptor(typeof(Mask), new PdfObjectDescriptor(new MaskConverter()));
            RegisterTypeDescriptor(typeof(AnnotationObject), new PdfObjectDescriptor(new TypeAwareConverter<PdfName>(AnnotationFactory.CreateInstance, PdfNames.SubType)));
            RegisterTypeDescriptor(typeof(ActionObject), new PdfObjectDescriptor(new TypeAwareConverter<PdfName>(ActionObject.CreateInstance, PdfNames.S)));
            RegisterTypeDescriptor(typeof(LinkObject), new PdfObjectDescriptor("Annot", new PdfName("Link")));
            RegisterTypeDescriptor(typeof(WidgetObject), new PdfObjectDescriptor("Annot", new PdfName("Widget")));
            RegisterTypeDescriptor(typeof(IDestinationObject), new PdfObjectDescriptor("Dest", new DestinationAndNamedDestinationConverter()));
            RegisterTypeDescriptor(typeof(DestinationObject), new PdfObjectDescriptor(new DestinationConverter()));
            RegisterTypeDescriptor(typeof(NamedDestinationsObject), new PdfObjectDescriptor(new NamedDestinationsConverter()));
            RegisterTypeDescriptor(typeof(NamedDestinationsArrayObject), new PdfObjectDescriptor(new NamedDestinationsConverter()));
            RegisterTypeDescriptor(typeof(GoToActionObject), new PdfObjectDescriptor("Action"));
            RegisterTypeDescriptor(typeof(UriActionObject), new PdfObjectDescriptor("Action"));
            RegisterTypeDescriptor(typeof(Encrypt), new PdfObjectDescriptor(new TypeAwareConverter<PdfName>(Encrypt.CreateInstance, PdfNames.Filters)));
            RegisterTypeDescriptor(typeof(EncodingBaseObject), new PdfObjectDescriptor(new EncodingBaseConverter()));
            RegisterTypeDescriptor(typeof(CharIdToGlyphIdMappingBase), new PdfObjectDescriptor(new CharCodeToGlyphIdMappingBaseConverter()));
            RegisterTypeDescriptor(typeof(ColorSpaceObject), new PdfObjectDescriptor(new ColorSpaceConverter()));
            RegisterTypeDescriptor(typeof(ShadingPatternObject), new PdfObjectDescriptor(new PdfInt(2), "PatternType"));
            RegisterTypeDescriptor(typeof(AxialShading), new PdfObjectDescriptor(new PdfInt(2), "ShadingType"));
            RegisterTypeDescriptor(typeof(RadialShading), new PdfObjectDescriptor(new PdfInt(3), "ShadingType"));
            RegisterTypeDescriptor(typeof(SampledFunctionObject), new PdfObjectDescriptor(new PdfInt(0), "FunctionType"));
            RegisterTypeDescriptor(typeof(StitchingFunctionObject), new PdfObjectDescriptor(new PdfInt(3), "FunctionType"));
            RegisterTypeDescriptor(typeof(Tiling), new PdfObjectDescriptor(new PdfInt(1), "PatternType"));
            RegisterTypeDescriptor(typeof(PatternColorObject), new PdfObjectDescriptor(new PatternColorConverter()));
            RegisterTypeDescriptor(typeof(Shading), new PdfObjectDescriptor(new TypeAwareConverter<PdfInt>(Shading.CreateInstance, PdfNames.ShadingType)));
            RegisterTypeDescriptor(typeof(FunctionObject), new PdfObjectDescriptor(new TypeAwareConverter<PdfInt>(FunctionObject.CreateFunction, PdfNames.FunctionType)));
            RegisterTypeDescriptor(typeof(PdfArray), new PdfObjectDescriptor(new PdfArrayConverter()));
            RegisterTypeDescriptor(typeof(FontDescriptor), new PdfObjectDescriptor("FontDescriptor"));
            RegisterTypeDescriptor(typeof(DocumentInfo), new PdfObjectDescriptor("Metadata", "XML".ToPdfName()));
            RegisterTypeDescriptor(typeof(Appearances), new PdfObjectDescriptor(new AppearancesConverter()));
            RegisterTypeDescriptor(typeof(FormFieldsTree), new PdfObjectDescriptor(new FieldsConverter()));
            RegisterTypeDescriptor(typeof(CrossReferenceStream), new PdfObjectDescriptor("XRef"));

#if WPF
            RegisterTypeDescriptor(typeof(SignatureReferenceObject), new PdfObjectDescriptor("SigRef"));
            RegisterTypeDescriptor(typeof(SignatureObject), new PdfObjectDescriptor("Sig"));
            RegisterTypeDescriptor(typeof(SignatureByteRange), new PdfObjectDescriptor(new SignatureByteRangeConverter()));
            RegisterTypeDescriptor(typeof(SignatureContents), new PdfObjectDescriptor(new SignatureContentsConverter()));
#endif
        }

        public static PdfObjectDescriptor GetPdfObjectDescriptor(IPdfPrimitive pdfObject)
        {
            return GetPdfObjectDescriptor(pdfObject.GetType());
        }

        public static PdfObjectDescriptor GetPdfObjectDescriptor<T>()
        {
            return GetPdfObjectDescriptor(typeof(T));
        }

        public static PdfObjectDescriptor GetPdfObjectDescriptor(Type type)
        {
            PdfObjectDescriptor res;
            if (pdfObjectDescriptors.TryGetValue(type, out res))
            {
                return res;
            }

            return CreateDefaultPdfObjectDescriptor(type);
        }

        private static PdfObjectDescriptor CreateDefaultPdfObjectDescriptor(Type type)
        {
            PdfObjectDescriptor res = new PdfObjectDescriptor();
            RegisterTypeDescriptor(type, res);

            return res;
        }

        private static void RegisterTypeDescriptor(Type type, PdfObjectDescriptor descriptor)
        {
            pdfObjectDescriptors[type] = descriptor;
        }
    }
}
