﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal class CrossReferenceEntry
    {
        public CrossReferenceEntry()
        {
        }

        public CrossReferenceEntry(long field1, int field2, CrossReferenceEntryType type)
        {
            this.Field1 = field1;
            this.Field2 = field2;
            this.Type = type;
        }

        public int Field2
        {
            get;
            set;
        }

        public long Field1
        {
            get;
            set;
        }

        public CrossReferenceEntryType Type
        {
            get;
            set;
        }

        public override string ToString()
        {
            string type = this.Type == CrossReferenceEntryType.Used ? "n" : (this.Type == CrossReferenceEntryType.Compressed ? "c" : "f");
            char separator = Telerik.Windows.Documents.Core.PostScript.Data.PostScriptConstants.SpaceSeparator;

            return string.Format("{0:0000000000}{3}{1:00000}{3}{2}", this.Field1, this.Field2, type, separator);
        }

        public string ToCrossReferenceTableString()
        {
            string type = this.Type == CrossReferenceEntryType.Free ? "f" : "n";
            char separator = Telerik.Windows.Documents.Core.PostScript.Data.PostScriptConstants.SpaceSeparator;

            return string.Format("{0:0000000000}{3}{1:00000}{3}{2}", this.Field1, this.Field2, type, separator);
        }

        public void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.WriteLine(this.ToCrossReferenceTableString());
        }

        public void Read(PostScriptReader reader)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");

            string[] fields;
            bool isEmptyLine;

            do
            {
                fields = reader.Reader.ReadSplitLine();
                isEmptyLine = fields.Length == 0;
            }
            while (isEmptyLine);

            long field1;
            int field2;

            long.TryParse(fields[0], out field1);
            int.TryParse(fields[1], out field2);

            this.Field1 = field1;
            this.Field2 = field2;
            this.Type = fields[2] == "f" ? CrossReferenceEntryType.Free : CrossReferenceEntryType.Used;
        }
    }
}
