﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal sealed class IndirectObject : PdfPrimitive
    {
        private readonly IndirectReference reference;
        private IPdfPrimitive content;
        private bool isContentReleased;

        public IndirectObject(IndirectReference reference, IPdfPrimitive content)
        {
            Guard.ThrowExceptionIfNull(reference, "reference");
            Guard.ThrowExceptionIfNull(content, "content");

            this.reference = reference;
            this.content = content;
        }

        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.IndirectObject;
            }
        }
        
        public IPdfPrimitive Content
        {
            get
            {
                Guard.ThrowExceptionIfTrue(this.isContentReleased, "isContentReleased");
                return this.content;
            }
        }

        public IndirectReference Reference
        {
            get
            {
                return this.reference;
            }
        }

        public T GetContent<T>()
            where T : IPdfPrimitive
        {
            return (T)this.Content;
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            context.BeginExportIndirectObject(this, writer.Position);
            writer.WriteLine("{0} {1} {2}", this.Reference.ObjectNumber, this.Reference.GenerationNumber, PdfNames.IndirectObjectStart);
            this.Content.Write(writer, context);
            writer.WriteLine();
            writer.WriteLine(PdfNames.IndirectObjectEnd);
            context.EndExportIndirectObject();
        }

        internal void ReleaseContent()
        {
            this.isContentReleased = true;
            this.content = null;
        }
    }
}
