﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal class IndirectReference : PdfPrimitive
    {
        private readonly int objectNumber;
        private readonly int generationNumber;

        public IndirectReference(int objectNumber, int generationNumber)
        {
            this.objectNumber = objectNumber;
            this.generationNumber = generationNumber;
        }

        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.IndirectReference;
            }
        }

        public int GenerationNumber
        {
            get
            {
                return this.generationNumber;
            }
        }

        public int ObjectNumber
        {
            get
            {
                return this.objectNumber;
            }
        }

        public static bool operator ==(IndirectReference left, IndirectReference right)
        {
            if (object.Equals(left, null))
            {
                return object.Equals(right, null);
            }

            if (object.Equals(right, null))
            {
                return false;
            }

            return left.Equals(right);
        }

        public static bool operator !=(IndirectReference left, IndirectReference right)
        {
            return !(left == right);
        }

        public override string ToString()
        {
            return string.Format("{0} {1} R", this.ObjectNumber, this.GenerationNumber);
        }

        public override bool Equals(object obj)
        {
            IndirectReference other = obj as IndirectReference;
            if (other == null)
            {
                return false;
            }

            return this.ObjectNumber == other.ObjectNumber && this.GenerationNumber == other.GenerationNumber;
        }

        public override int GetHashCode()
        {
            return this.ObjectNumber ^ this.GenerationNumber;
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.WriteIndirectReference(this);
        }
    }
}
