﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal class PdfArray : PdfCollectionBase<int>, IPdfCollection, IEnumerable<IPdfPrimitive>
    {
        private readonly List<IPdfPrimitive> store;

        public PdfArray(params IPdfPrimitive[] initialValues)
        {
            this.store = new List<IPdfPrimitive>();
            this.store.AddRange(initialValues);
        }

        public PdfArray(IEnumerable<IPdfPrimitive> initialValues)
            : this(initialValues.ToArray())
        {
        }

        public static PdfArray MatrixIdentity
        {
            get
            {
                return new PdfArray(new IPdfPrimitive[] { new PdfInt(1), new PdfInt(0), new PdfInt(0), new PdfInt(1), new PdfInt(0), new PdfInt(0) });
            }
        }

        public override int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.Array;
            }
        }

        public override IPdfPrimitive this[int index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        public void Add(IPdfPrimitive item)
        {
            this.store.Add(item);
        }

        public void AddRange(IEnumerable<IPdfPrimitive> items)
        {
            this.store.AddRange(items);
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            this.Write(writer, context, false);
        }

        public void Write(PdfWriter writer, IPdfExportContext context, bool writeItemsAsIndirectReferences)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.Write(PdfNames.PdfArrayStart);
            int lastIndex = this.store.Count - 1;
            for (int i = 0; i < this.store.Count; i++)
            {
                if (writeItemsAsIndirectReferences || store[i].ExportAs == PdfElementType.PdfStreamObject)
                {
                    WriteIndirectReference(writer, context, this.store[i]);
                }
                else
                {
                    this.store[i].Write(writer, context);
                }

                if (i != lastIndex)
                {
                    writer.WriteSeparator();
                }
            }

            writer.Write(PdfNames.PdfArrayEnd);
        }

        public IEnumerator<IPdfPrimitive> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        protected override bool TryGetElementOverride(int index, out IPdfPrimitive element)
        {
            if (index < 0 || index >= this.store.Count)
            {
                element = null;

                return false;
            }

            element = this.store[index];

            return true;
        }
    }
}
