﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal abstract class PdfCollectionBase<TIndex> : PdfPrimitive
    {
        public abstract int Count { get; }

        public abstract IPdfPrimitive this[TIndex index] { get; set; }

        public T GetElement<T>(PostScriptReader reader, IPdfImportContext context, TIndex index)
            where T : class, IPdfPrimitive
        {
            IPdfPrimitive result = this[index];
            T element = ConvertElementToType<T>(reader, context, result);

            return element;
        }

        public bool TryGetElement<T>(PostScriptReader reader, IPdfImportContext context, TIndex index, out T element) 
            where T : class, IPdfPrimitive
        {
            IPdfPrimitive result;
            if (!this.TryGetElementOverride(index, out result))
            {
                element = default(T);

                return false;
            }
            
            element = ConvertElementToType<T>(reader, context, result);
            return element != null;
        }

        public bool TryGetElement(TIndex index, out IPdfPrimitive element)
        {
            return this.TryGetElementOverride(index, out element);
        }

        protected abstract bool TryGetElementOverride(TIndex index, out IPdfPrimitive element);

        private static T ConvertElementToType<T>(PostScriptReader reader, IPdfImportContext context, IPdfPrimitive element)
            where T : class, IPdfPrimitive
        {
            PdfObjectDescriptor descriptor = PdfObjectDescriptors.GetPdfObjectDescriptor<T>();
            IPdfPrimitive convertedElement = descriptor.Converter.Convert(typeof(T), reader, context, element);

            return convertedElement as T;
        }
    }
}
