﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal class PdfDictionary : PdfCollectionBase<string>, IEnumerable<KeyValuePair<string, IPdfPrimitive>>
    {
        private readonly Dictionary<string, IPdfPrimitive> store;

        public PdfDictionary()
            : base()
        {
            this.store = new Dictionary<string, IPdfPrimitive>();
        }

        public override PdfElementType Type
        {
            get
            {
                return PdfElementType.Dictionary;
            }
        }

        public override int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public override IPdfPrimitive this[string index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        public IEnumerable<string> Keys
        {
            get
            {
                return this.store.Keys;
            }
        }

        public bool Remove(string key)
        {
            return this.store.Remove(key);
        }

        public override void Write(PdfWriter writer, IPdfExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.Write(PdfNames.PdfDictionaryStart);

            foreach (KeyValuePair<string, IPdfPrimitive> item in this.store)
            {
                writer.WritePdfName(item.Key);
                writer.WriteSeparator();

                IPdfPrimitive value = item.Value;
                if (value.Type == PdfElementType.PdfStreamObject)
                {
                    WriteIndirectReference(writer, context, value);
                }
                else
                {
                    value.Write(writer, context);
                }

                writer.WriteSeparator();
            }

            writer.Write(PdfNames.PdfDictionaryEnd);
        }

        public bool ContainsKey(string key)
        {
            Guard.ThrowExceptionIfNullOrEmpty(key, "key");

            return this.store.ContainsKey(key);
        }
        
        public IEnumerator<KeyValuePair<string, IPdfPrimitive>> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        protected override bool TryGetElementOverride(string index, out IPdfPrimitive element)
        {
            return this.store.TryGetValue(index, out element);
        }
    }
}
