﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types
{
    internal abstract class PdfString : PdfSimpleType<byte[]>
    {
        private static string DefaultCharacterName = "emdash";
        private static int PrintableAsciiLastCharacterIndex = 127;

        public PdfString()
        {
        }

        public PdfString(byte[] initialData)
            : base(initialData)
        {
        }

        public PdfString(string initialValue, StringType stringType) : this(PdfString.GetBytes(initialValue, stringType))
        {
        }

        public abstract StringObjectType StringObjectType
        {
            get;
        }

        public override PdfElementType Type
        {
            get { return PdfElementType.String; }
        }

        internal static string GetString(byte[] bytes)
        {
            if (bytes == null || bytes.Length == 0)
            {
                return string.Empty;
            }

            bool isUnicode = PdfString.IsUnicode(bytes);

            return isUnicode ? PdfString.GetUtfEncodedTextString(bytes) : PdfString.GetPdfEncodedTextString(bytes);
        }

        internal static bool IsUnicode(byte[] bytes)
        {
            return bytes != null && bytes.Length > 1 && bytes[0] == 254 && bytes[1] == 255;
        }

        internal static string GetUtfEncodedTextString(byte[] bytes)
        {
            if (bytes == null || bytes.Length == 0)
            {
                return string.Empty;
            }

            int start = 0;
            int lenght = bytes.Length;
            bool startsWithBigEndianByteOrderMask = false;

            if (bytes.Length > 2)
            {
                startsWithBigEndianByteOrderMask = IsUnicode(bytes);
            }

            if (startsWithBigEndianByteOrderMask)
            {
                start = 2;
                lenght -= 2;
            }

            return System.Text.Encoding.BigEndianUnicode.GetString(bytes, start, lenght);
        }

        internal static string GetPdfEncodedTextString(byte[] bytes)
        {
            if (bytes == null || bytes.Length == 0)
            {
                return string.Empty;
            }

            string[] names = PredefinedSimpleEncoding.PdfDocEncoding.GetNames();

            char[] characters = new char[bytes.Length];
            for (int i = 0; i < bytes.Length; i++)
            {
                byte b = bytes[i];

                char character;
                bool isPrintableAscii = b <= PrintableAsciiLastCharacterIndex;

                if (isPrintableAscii)
                {
                    character = (char)b;
                }
                else if (b < names.Length)
                {
                    string name = names[b];
                    character = AdobeGlyphList.GetUnicode(name);
                }
                else
                {
                    character = AdobeGlyphList.GetUnicode(DefaultCharacterName);
                }

                characters[i] = character;
            }

            return new string(characters);
        }

        private static byte[] GetBytes(string value, StringType stringType)
        {
            switch (stringType)
            {
                case StringType.Byte:
                    return BytesHelper.ToByteArray(value);

                case StringType.ASCII:
                    return System.Text.Encoding.UTF8.GetBytes(value);

                case StringType.UTFEncodedTextString:
                    return BytesHelper.MergeBytes(new byte[] { 255, 254 }, System.Text.Encoding.BigEndianUnicode.GetBytes(value));

                default:
                    throw new NotImplementedException(string.Format("{0} is not supported.", stringType.ToString()));
            }
        }
    }
}
