﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Color
{
    internal abstract class SetColorNBase : ContentStreamOperator
    {
        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "contentStreamInterpreter");
            Guard.ThrowExceptionIfNull(context, "context");
            
            IEnumerable<IPdfPrimitive> components = this.PopAllOperands(interpreter).Reverse();
            PdfArray componentsArray = new PdfArray(components);
            ColorSpaceObject colorSpace;

            if (!this.TryGetColorSpace(interpreter, out colorSpace))
            {
                switch (componentsArray.Count)
                {
                    case 1:
                        colorSpace = new DeviceGrayColorSpaceObject();
                        break;
                    case 3:
                        colorSpace = new DeviceRgbColorSpaceObject();
                        break;
                    case 4:
                        colorSpace = new DeviceCmykColorSpaceObject();
                        break;
                    default:
                        throw new InvalidOperationException("Components count must be 1, 3 or 4!");
                }
            }

            ColorObjectBase color = colorSpace.GetColor(context, interpreter.Reader, componentsArray);

            if (color != null)
            {
                this.SetColor(interpreter, color);
            }
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, string resourceName)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            this.WriteInternal(writer, context.Owner, new PdfName(resourceName));
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, int[] colorComponents, string resourceName)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            IPdfPrimitive[] operands = new IPdfPrimitive[colorComponents.Length + 1];
            for (int i = 0; i < colorComponents.Length; i++)
            {
                operands[i] = colorComponents[i].ToPdfInt();
            }

            operands[colorComponents.Length] = new PdfName(resourceName);

            this.WriteInternal(writer, context.Owner, operands);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, ColorObjectBase color, string resourceName)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(color, "color");
            Guard.ThrowExceptionIfNull(context, "context");

            this.WriteInternal(writer, context.Owner, color, new PdfName(resourceName));
        }

        protected abstract bool TryGetColorSpace(ContentStreamInterpreter interpreter, out ColorSpaceObject colorSpace);

        protected abstract void SetColor(ContentStreamInterpreter interpreter, ColorObjectBase color);

        private IEnumerable<IPdfPrimitive> PopAllOperands(ContentStreamInterpreter interpreter)
        {
            while (interpreter.Operands.Count > 0)
            {
                IPdfPrimitive primitive = interpreter.Operands.GetLast();
                yield return primitive;
            }
        }
    }
}
