﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Color
{
    internal abstract class SetColorSpaceBase : ContentStreamOperator
    {
        public override void Execute(ContentStreamInterpreter contentStreamInterpreter, IPdfContentImportContext context)
        {
            PdfName name = contentStreamInterpreter.Operands.GetLastAs<PdfName>();

            ColorSpaceObject colorSpace;
            if (ColorSpaceManager.IsColorSpaceName(name.Value))
            {
                colorSpace = ColorSpaceManager.CreateColorSpaceObject(name.Value);
            }
            else
            {
                colorSpace = context.GetColorSpace(contentStreamInterpreter.Reader, name);
            }

            this.SetColorSpaceInternal(contentStreamInterpreter.GraphicState, colorSpace);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, ColorSpaceBase colorSpace)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            ColorSpaceObject colorSpaceObject = ColorSpaceManager.CreateColorSpaceObject(colorSpace);

            this.WriteInternal(writer, context.Owner, colorSpaceObject);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, string resourceKey)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            this.WriteInternal(writer, context.Owner, new PdfName(resourceKey));
        }

        protected abstract void SetColorSpaceInternal(GraphicsState graphicsState, ColorSpaceObject colorSpace);
    }
}
