﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Color
{
    internal abstract class SetSimpleColorBase<TColor, TColorSpace> : ContentStreamOperator
        where TColor : ColorObjectBase
        where TColorSpace : ColorSpaceObject
    {
        protected abstract bool IsStrokeColorSetter { get; }

        public sealed override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            TColor color;
            TColorSpace colorspace;
            this.ReadColorComponents(interpreter, context, out color, out colorspace);

            if (this.IsStrokeColorSetter)
            {
                interpreter.GraphicState.SetStrokeColorSpace(colorspace);
                interpreter.GraphicState.StrokeColor = color;
            }
            else
            {
                interpreter.GraphicState.SetFillColorSpace(colorspace);
                interpreter.GraphicState.FillColor = color;
            }
        }

        protected abstract void ReadColorComponents(ContentStreamInterpreter interpreter, IPdfContentImportContext context, out TColor color, out TColorSpace colorspace);

        protected static PdfReal ReadRestrictedNumberComponentValue(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            PdfReal operandValue = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            double value = Core.Imaging.Color.RestrictComponentToDoubleLimits(operandValue.Value);
            PdfReal component = new PdfReal(value);

            return component;
        }
    }
}
