﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Color;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.InlineImage;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.MarkedContent;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.State;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators
{
    internal static class ContentStreamOperators
    {
        // Text
        public static readonly BeginText BeginTextOperator;
        public static readonly EndText EndTextOperator;
        public static readonly Font FontOperator;
        public static readonly CharSpace CharSpaceOperator;
        public static readonly RenderingMode RenderingModeOperator;
        public static readonly Rise RiseOperator;
        public static readonly HorizontalScale HorizontalScaleOperator;
        public static readonly WordSpace WordSpaceOperator;
        public static readonly ShowText ShowTextOperator;
        public static readonly ShowTextArray ShowTextArrayOperator;
        public static readonly MoveToNextLineAndShowText MoveToNextLineAndShowTextOperator;
        public static readonly SetSpacingMoveToNextLineAndShowText SetSpacingMoveToNextLineAndShowTextOperator;
        public static readonly MoveToNextLine MoveToNextLineOperator;
        public static readonly TextLeading TextLeadingOperator;
        public static readonly TextMatrix TextMatrixOperator;
        public static readonly TranslateText TranslateTextOperator;
        public static readonly TranslateTextAndSetLeading TranslateTextAndSetLeadingOperator;

        // Colors
        public static readonly SetRgbColor SetRgbColorOperator;
        public static readonly SetStrokeRgbColor SetStrokeRgbColorOperator;
        public static readonly SetGrayColor SetGrayColorOperator;
        public static readonly SetStrokeGrayColor SetStrokeGrayColorOperator;
        public static readonly SetFillColorN SetColorN;
        public static readonly SetFillColorNAlias SetColorNAlias;
        public static readonly SetStrokeColorN SetStrokeColorN;
        public static readonly SetStrokeColorNAlias SetStrokeColorNAlias;
        public static readonly SetFillColorSpace SetColorSpace;
        public static readonly SetStrokeColorSpace SetStrokeColorSpace;
        public static readonly SetCmykColor SetCmykColor;
        public static readonly SetStrokeCmykColor SetCmykStrokeColor;

        // State
        public static readonly ConcatMatrix ConcatMatrixOperator;
        public static readonly SetLineJoin SetLineJoinOperator;
        public static readonly SetLineCap SetLineCapOperator;
        public static readonly SetLineWidth SetLineWidthOperator;
        public static readonly SaveGraphicsState SaveGraphicsStateOperator;
        public static readonly RestoreGraphicsState RestoreGraphicsStateOperator;
        public static readonly SetGraphicsStateDictionary SetGraphicsStateDictionaryOperator;

        // Graphics
        public static readonly CurveTo CurveToOperator;
        public static readonly VCurveTo VCurveToOperator;
        public static readonly YCurveTo YCurveToOperator;
        public static readonly LineTo LineToOperator;
        public static readonly AppendRectangle AppendRectangleOperator;
        public static readonly MoveTo MoveToOperator;
        public static readonly ClosePath ClosePathOperator;
        public static readonly FillPath FillPathOperator;
        public static readonly FillPathAlias FillPathAliasOperator;
        public static readonly FillPathEvenOdd FillPathEvenOddOperator;
        public static readonly StrokePath StrokePathOperator;
        public static readonly CloseAndStrokePath CloseAndStrokePathOperator;
        public static readonly FillAndStrokePath FillAndStrokePathOperator;
        public static readonly FillAndStrokePathEvenOdd FillAndStrokePathEvenOddOperator;
        public static readonly ModifyClippingPathNonzero ModifyClippingPathNonzeroOperator;
        public static readonly ModifyClippingPathEvenOdd ModifyClippingPathEvenOddOperator;
        public static readonly EndPath EndPathOperator;
        public static readonly SetLineDashPattern SetLineDashPatternOperator;
        public static readonly SetMiterLimit SetMiterLimitOperator;
        public static readonly CloseFillAndStrokePath CloseFillAndStrokePathOperator;
        public static readonly CloseFillAndStrokePathEvenOdd CloseFillAndStrokePathEvenOddOperator;

        // Objects
        public static readonly Paint PaintOperator;

        // Marked content
        public static readonly BeginMarkedContent BeginMarkedContentOperator;
        public static readonly EndMarkedContent EndMarkedContentOperator;

        // Inline image
        public static readonly BeginInlineImage BeginInlineImageOperator;
        public static readonly EndInlineImage EndInlineImageOperator;

        static ContentStreamOperators()
        {
            // Text
            BeginTextOperator = new BeginText();
            EndTextOperator = new EndText();
            FontOperator = new Font();
            CharSpaceOperator = new CharSpace();
            RenderingModeOperator = new RenderingMode();
            RiseOperator = new Rise();
            HorizontalScaleOperator = new HorizontalScale();
            WordSpaceOperator = new WordSpace();
            ShowTextOperator = new ShowText();
            ShowTextArrayOperator = new ShowTextArray();
            MoveToNextLineAndShowTextOperator = new MoveToNextLineAndShowText();
            SetSpacingMoveToNextLineAndShowTextOperator = new SetSpacingMoveToNextLineAndShowText();
            MoveToNextLineOperator = new MoveToNextLine();
            TextLeadingOperator = new TextLeading();
            TextMatrixOperator = new TextMatrix();
            TranslateTextOperator = new TranslateText();
            TranslateTextAndSetLeadingOperator = new TranslateTextAndSetLeading();

            // Colors
            SetRgbColorOperator = new SetRgbColor();
            SetStrokeRgbColorOperator = new SetStrokeRgbColor();
            SetGrayColorOperator = new SetGrayColor();
            SetStrokeGrayColorOperator = new SetStrokeGrayColor();
            SetColorN = new SetFillColorN();
            SetColorNAlias = new SetFillColorNAlias();
            SetStrokeColorN = new SetStrokeColorN();
            SetStrokeColorNAlias = new SetStrokeColorNAlias();
            SetColorSpace = new SetFillColorSpace();
            SetStrokeColorSpace = new SetStrokeColorSpace();
            SetCmykColor = new SetCmykColor();
            SetCmykStrokeColor = new SetStrokeCmykColor();

            // State
            ConcatMatrixOperator = new ConcatMatrix();
            SetLineJoinOperator = new SetLineJoin();
            SetLineCapOperator = new SetLineCap();
            SetLineWidthOperator = new SetLineWidth();
            SaveGraphicsStateOperator = new SaveGraphicsState();
            RestoreGraphicsStateOperator = new RestoreGraphicsState();
            SetMiterLimitOperator = new SetMiterLimit();
            SetGraphicsStateDictionaryOperator = new SetGraphicsStateDictionary();

            // Graphics
            CurveToOperator = new CurveTo();
            VCurveToOperator = new VCurveTo();
            YCurveToOperator = new YCurveTo();
            LineToOperator = new LineTo();
            AppendRectangleOperator = new AppendRectangle();
            MoveToOperator = new MoveTo();
            ClosePathOperator = new ClosePath();
            FillPathOperator = new FillPath();
            FillPathAliasOperator = new FillPathAlias();
            FillPathEvenOddOperator = new FillPathEvenOdd();
            StrokePathOperator = new StrokePath();
            CloseAndStrokePathOperator = new CloseAndStrokePath();
            FillAndStrokePathOperator = new FillAndStrokePath();
            FillAndStrokePathEvenOddOperator = new FillAndStrokePathEvenOdd();
            ModifyClippingPathNonzeroOperator = new ModifyClippingPathNonzero();
            ModifyClippingPathEvenOddOperator = new ModifyClippingPathEvenOdd();
            EndPathOperator = new EndPath();
            SetLineDashPatternOperator = new SetLineDashPattern();
            CloseFillAndStrokePathOperator = new CloseFillAndStrokePath();
            CloseFillAndStrokePathEvenOddOperator = new CloseFillAndStrokePathEvenOdd();

            // Objects
            PaintOperator = new Paint();

            // Marked content
            BeginMarkedContentOperator = new BeginMarkedContent();
            EndMarkedContentOperator = new EndMarkedContent();

            // Inline image
            BeginInlineImageOperator = new BeginInlineImage();
            EndInlineImageOperator = new EndInlineImage();
        }

        public static GraphicsStatePusher PushGraphicsState(PdfWriter writer, IPdfContentExportContext context)
        {
            return new GraphicsStatePusher(writer, context);
        }
    }
}
