﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal class AppendRectangle : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.AppendRectangle; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfReal height = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal width = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal y = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);

            MoveTo.Execute(interpreter, x, y);
            LineTo.Execute(interpreter, new PdfReal(x.Value + width.Value), y);
            LineTo.Execute(interpreter, new PdfReal(x.Value + width.Value), new PdfReal(y.Value + height.Value));
            LineTo.Execute(interpreter, x, new PdfReal(y.Value + height.Value));
            ClosePath.Execute(interpreter);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, RectangleGeometry rectangleGeometry)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(rectangleGeometry, "rectangleGeometry");

            Point bottomLeft = new Point(rectangleGeometry.Rect.Left, rectangleGeometry.Rect.Bottom);
            double width = rectangleGeometry.Rect.Width;
            double height = rectangleGeometry.Rect.Height;
            bottomLeft.Y -= height;

            this.WriteInternal(writer, context.Owner,
                bottomLeft.X.ToPdfReal(), bottomLeft.Y.ToPdfReal(),
                width.ToPdfReal(), height.ToPdfReal());
        }
    }
}
