﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal class CurveTo : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.CurveTo; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfReal y3 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x3 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal y2 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x2 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal y1 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x1 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);

            CurveTo.Execute(interpreter, x1, y1, x2, y2, x3, y3);
        }

        public static void Execute(ContentStreamInterpreter interpreter, PdfReal x1, PdfReal y1, PdfReal x2, PdfReal y2, PdfReal x3, PdfReal y3)
        {
            interpreter.EnsurePathFigure();

            BezierSegment segment = interpreter.CurrentPathFigure.Segments.AddBezierSegment();
            segment.Point1 = new Point(x1.Value, y1.Value);
            segment.Point2 = new Point(x2.Value, y2.Value);
            segment.Point3 = new Point(x3.Value, y3.Value);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, BezierSegment segment)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(segment, "segment");

            Point p1 = segment.Point1;
            Point p2 = segment.Point2;
            Point p3 = segment.Point3;

            this.WriteInternal(writer, context.Owner,
                p1.X.ToPdfReal(), p1.Y.ToPdfReal(),
                p2.X.ToPdfReal(), p2.Y.ToPdfReal(),
                p3.X.ToPdfReal(), p3.Y.ToPdfReal());
        }
    }
}
