﻿using System;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal class LineTo : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.LineTo; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfReal y = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);

            LineTo.Execute(interpreter, x, y);
        }

        public static void Execute(ContentStreamInterpreter interpreter, PdfReal x, PdfReal y)
        {
            interpreter.EnsurePathFigure();

            var lineSegment = interpreter.CurrentPathFigure.Segments.AddLineSegment(); 
            lineSegment.Point = new Point(x.Value, y.Value);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, LineSegment line)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            Point p = line.Point;  

            this.WriteInternal(writer, context.Owner, p.X.ToPdfReal(), p.Y.ToPdfReal());
        }
    }
}
