﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal abstract class ModifyClippingPathBase : ContentStreamOperator
    {
        protected abstract FillRule FillRule { get; }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            Clipping clipping = new Clipping();
            clipping.Clip = interpreter.CalculateGeometry();
            clipping.Transform = context.ContentRoot.ToTopLeftCoordinateSystem(interpreter.GraphicState.Matrix);

            PathGeometry pathGeometry = clipping.Clip as PathGeometry;

            if (pathGeometry != null)
            {
                pathGeometry.FillRule = this.FillRule;
            }

            interpreter.GraphicState.IntersectClipping(clipping);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            this.WriteInternal(writer, context.Owner);
        }
    }
}
