﻿using System;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal class MoveTo : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.MoveTo; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfReal y = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);

            MoveTo.Execute(interpreter, x, y);
        }

        public static void Execute(ContentStreamInterpreter interpreter, PdfReal x, PdfReal y)
        {
            var lastPathFigure = interpreter.PathFigures.LastOrDefault();
            if (lastPathFigure != null && lastPathFigure.Segments.Count == 0)
            {
                interpreter.PathFigures.Remove(lastPathFigure);
            }

            interpreter.CurrentPathFigure = interpreter.PathFigures.AddPathFigure();
            interpreter.CurrentPathFigure.StartPoint = new Point(x.Value, y.Value);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, Point point)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(point, "point");
             
            this.WriteInternal(writer, context.Owner, point.X.ToPdfReal(), point.Y.ToPdfReal());
        }
    }
}
