﻿using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal abstract class PaintPathBase : ContentStreamOperator
    {
        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            Path path = context.ContentRoot.Content.AddPath();
            path.Geometry = interpreter.CalculateGeometry();
            interpreter.ClearGeometry();

            path.Position = new MatrixPosition(context.ContentRoot.ToTopLeftCoordinateSystem(interpreter.GraphicState.Matrix));
            path.Clipping = interpreter.GraphicState.Clipping;
            path.Marker = context.CurrentMarker;

            this.ExecuteOverride(interpreter, context, path);

            interpreter.ApplyGeometryProperties(path.GeometryProperties);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            this.WriteInternal(writer, context.Owner);
        }

        protected abstract void ExecuteOverride(ContentStreamInterpreter interpreter, IPdfContentImportContext context, Path path);
    }
}
