﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Graphics
{
    internal class VCurveTo : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.VCurveTo; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            PdfReal y3 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x3 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal y2 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfReal x2 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);

            interpreter.EnsurePathFigure();

            PathSegment lastSegment = interpreter.CurrentPathFigure.Segments.LastOrDefault();

            PdfReal x1;
            PdfReal y1;

            if (lastSegment != null)
            {
                x1 = new PdfReal(lastSegment.LastPoint.X);
                y1 = new PdfReal(lastSegment.LastPoint.Y);
            }
            else
            {
                x1 = new PdfReal(interpreter.CurrentPathFigure.StartPoint.X);
                y1 = new PdfReal(interpreter.CurrentPathFigure.StartPoint.Y);
            }

            CurveTo.Execute(interpreter, x1, y1, x2, y2, x3, y3);
        }
    }
}
