﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Objects
{
    internal class Paint : ContentStreamOperator
    {
        private const double ImageUnitSize = 1;
        private static readonly double DipToPoint = Unit.DipToPoint(1);

        public override string Name
        {
            get { return ContentStreamOperatorNames.Paint; }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            try
            {
                PdfName key = interpreter.Operands.GetLast<PdfName>(interpreter.Reader, context.Owner);
                XObjectBase xObject = context.GetXObject(interpreter.Reader, key);

                switch (xObject.XObjectType)
                {
                    case XObjectType.Image:
                        ImageXObject imageXObject = (ImageXObject)xObject;
                        PaintImage(interpreter, context, imageXObject);
                        break;
                    case XObjectType.Form:
                        FormXObject formXObject = (FormXObject)xObject;
                        var formSource = context.Owner.GetFormSource(interpreter.Reader, formXObject);
                        Fixed.Model.Objects.Form form = context.ContentRoot.Content.AddForm(formSource);
                        form.Position = CalculateFormPosition(interpreter, context, formSource);
                        form.Clipping = interpreter.GraphicState.Clipping;
                        form.Marker = context.CurrentMarker;
                        break;
                    default:
                        throw new NotSupportedException(string.Format("Not supported XObject type: {0}", xObject.XObjectType));
                }
            }
            //// This catching is done because we handle NotSupportedException and ArgumentNullException for image colorspaces and DecodeParms issues.
            catch
            {
#if DEBUG
                throw;
#endif
            }
        }

        internal static void PaintImage(ContentStreamInterpreter interpreter, IPdfContentImportContext context, ImageXObject imageXObject)
        {
            var imageSource = context.Owner.GetImageSource(interpreter.Reader, imageXObject);
            Fixed.Model.Objects.Image image = context.ContentRoot.Content.AddImage(imageSource);
            image.Position = CalculateImagePosition(interpreter, context, imageSource);
            image.Clipping = interpreter.GraphicState.Clipping;
            image.Marker = context.CurrentMarker;

            if (imageSource.ImageMask.Value)
            {
                image.StencilColor = interpreter.GraphicState.CalculateStencilColor(interpreter.Reader, context);
            }
        }

        private IPosition CalculateFormPosition(ContentStreamInterpreter interpreter, IPdfContentImportContext context, FormSource formSource)
        {
            Matrix scaleToPointSizeAndFlip = new Matrix(DipToPoint, 0, 0, -DipToPoint, 0, DipToPoint * formSource.Size.Height);
            MatrixPosition formPosition = new MatrixPosition(scaleToPointSizeAndFlip.MultiplyBy(interpreter.GraphicState.Matrix));
            formPosition = new MatrixPosition(context.ContentRoot.ToTopLeftCoordinateSystem(formPosition.Matrix));

            return formPosition;
        }

        private static MatrixPosition CalculateImagePosition(ContentStreamInterpreter interpreter, IPdfContentImportContext context, Fixed.Model.Resources.ImageSource imageSource)
        {
            double widthScale = imageSource.Width == 0 ? ImageUnitSize : ImageUnitSize / imageSource.Width;
            double heightScale = imageSource.Height == 0 ? ImageUnitSize : ImageUnitSize / imageSource.Height;
            Matrix scaleToUnitSizeAndFlip = new Matrix(widthScale, 0, 0, -heightScale, 0, ImageUnitSize);
            MatrixPosition imagePosition = new MatrixPosition(scaleToUnitSizeAndFlip.MultiplyBy(interpreter.GraphicState.Matrix));
            imagePosition = new MatrixPosition(context.ContentRoot.ToTopLeftCoordinateSystem(imagePosition.Matrix));

            return imagePosition;
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, string name)
        {
            this.WriteInternal(writer, context.Owner, new PdfName(name));
        }
    }
}
