﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.State
{
    internal abstract class MatrixOperatorBase : ContentStreamOperator
    {
        public sealed override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            double offsetY = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;
            double offsetX = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;
            double m22 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;
            double m21 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;
            double m12 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;
            double m11 = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner).Value;

            this.InterpretMatrixOverride(interpreter, new Matrix(m11, m12, m21, m22, offsetX, offsetY));
        }

        protected abstract void InterpretMatrixOverride(ContentStreamInterpreter interpreter, Matrix matrix);

        public void Write(PdfWriter writer, IPdfContentExportContext context, IPosition position)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(position, "position");

            this.Write(writer, context, position.Matrix);
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, Matrix matrix)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            this.Write(writer, context, matrix.M11, matrix.M12, matrix.M21, matrix.M22, matrix.OffsetX, matrix.OffsetY);
        }

        private void Write(PdfWriter writer, IPdfContentExportContext context, double m11, double m12, double m21, double m22, double offsetX, double offsetY)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            bool isIdentity = m11 == 1 && m12 == 0 && m21 == 0 && m22 == 1 && offsetX == 0 && offsetY == 0;

            if (!isIdentity)
            {
                this.WriteInternal(writer, context.Owner, m11.ToPdfReal(), m12.ToPdfReal(), m21.ToPdfReal(), m22.ToPdfReal(), offsetX.ToPdfReal(), offsetY.ToPdfReal());
            }
        }
    }
}
