﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.State
{
    internal class SetLineDashPattern : ContentStreamOperator
    {
        public override string Name
        {
            get { return ContentStreamOperatorNames.SetLineDashPattern; }
        }

        public override void Execute(Import.ContentStreamInterpreter interpreter, Import.IPdfContentImportContext context)
        {
            PdfReal dashOffset = interpreter.Operands.GetLast<PdfReal>(interpreter.Reader, context.Owner);
            PdfArray dashArray = interpreter.Operands.GetLast<PdfArray>(interpreter.Reader, context.Owner);

            interpreter.GraphicState.StrokeDashOffset = dashOffset.Value;

            List<double> graphicStateDashArray = new List<double>();
            for (int i = 0; i < dashArray.Count; i++)
            {
                PdfReal value;
                dashArray.TryGetElement<PdfReal>(interpreter.Reader, context.Owner, i, out value);
                graphicStateDashArray.Add(value.Value);
            }

            interpreter.GraphicState.StrokeDashArray = graphicStateDashArray;
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, IEnumerable<double> dashArray, double dashOffset)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(dashArray, "dashArray");

            this.WriteInternal(writer, context.Owner, dashArray.ToPdfArray(), dashOffset.ToPdfReal());
        }
    }
}
