﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Editing.Layout;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Text
{
    internal class ShowText : ContentStreamOperator
    {
        public static readonly Matrix FlipTextTransformation = new Matrix(1, 0, 0, -1, 0, 0);

        public override string Name
        {
            get
            {
                return ContentStreamOperatorNames.ShowText;
            }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            Guard.ThrowExceptionIfNull(interpreter, "interpreter");
            Guard.ThrowExceptionIfNull(context, "context");

            if (interpreter.Operands.Count > 0)
            {
                PdfString text = interpreter.Operands.GetLast<PdfString>(interpreter.Reader, context.Owner);
                ShowText.Execute(interpreter, context, text);
            }
        }

        public static void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context, PdfString text)
        {
            if (interpreter.TextState.Font != null)
            {
                TextFragment textFragment = context.ContentRoot.Content.AddTextFragment();
                interpreter.ApplyTextProperties(textFragment.TextProperties);
                textFragment.TextCollection.Characters = interpreter.TextState.Font.GetCharacters(text);
                Matrix matrix = FlipTextTransformation.MultiplyBy(interpreter.TextRenderingMatrix);
                textFragment.Position = new MatrixPosition(context.ContentRoot.ToTopLeftCoordinateSystem(matrix));
                textFragment.Clipping = interpreter.GraphicState.Clipping;
                textFragment.Marker = context.CurrentMarker;
                textFragment.TextTranslation = interpreter.TextState.TextTranslation;

                double width = TextFragmentLayoutElement.MeasureWidth(textFragment);
                TranslateText.MoveHorizontalPosition(interpreter.TextState, width);
            }
        }

        public void Write(PdfWriter writer, IPdfContentExportContext context, TextCollection glyphs)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            List<byte> bytes = new List<byte>();
            foreach (CharInfo glyph in glyphs.Characters)
            {
                bytes.AddRange(glyph.CharCode.ToBytes());
            }

            PdfString pdfString = new PdfHexString(bytes.ToArray());
            this.WriteInternal(writer, context.Owner, pdfString);
        }
    }
}
