﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Operators.Text
{
    internal class ShowTextArray : ContentStreamOperator
    {
        private IConverter realNumberConverter;

        public override string Name
        {
            get
            {
                return ContentStreamOperatorNames.ShowTextArray;
            }
        }

        private IConverter RealNumberConverter
        {
            get
            {
                if (this.realNumberConverter == null)
                {
                    this.realNumberConverter = PdfObjectDescriptors.GetPdfObjectDescriptor<PdfReal>().Converter;
                }

                return this.realNumberConverter;
            }
        }

        public override void Execute(ContentStreamInterpreter interpreter, IPdfContentImportContext context)
        {
            PdfArray array = interpreter.Operands.GetLast<PdfArray>(interpreter.Reader, context.Owner);

            foreach (IPdfPrimitive element in array)
            {
                PdfString text = element as PdfString;
                if (text == null)
                {
                    PdfReal number = (PdfReal)this.RealNumberConverter.Convert(typeof(PdfReal), interpreter.Reader, context.Owner, element);
                    double horizontalScale = interpreter.TextState.HorizontalScaling ?? 1;
                    double translation = (-number.Value / FontBase.FontRatio) * interpreter.TextState.FontSize * horizontalScale;
                    interpreter.TextState.TextTranslation = translation;

                    TranslateText.MoveHorizontalPosition(interpreter.TextState, translation);
                }
                else
                {
                    ShowText.Execute(interpreter, context, text);
                }
            }
        }
    }
}
