﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate.InteractiveForms
{
    internal class CheckBoxPropertiesEditor : TwoStateAppearanceFieldPropertiesEditor
    {
        public CheckBoxPropertiesEditor(PdfIncrementalStreamExportContext context, PdfFormFieldSource field) 
            : base(context, field)
        {
        }

        public void SetIsChecked(bool isChecked)
        {
            PdfName currentValue;
            PdfSourceImportContext importContext = this.Field.FileSource.Context;
            bool isCurrentlyChecked = this.Field.PdfDictionary.TryGetElement<PdfName>(importContext.Reader, importContext, FormFieldNode.FieldValueName, out currentValue) && currentValue.Value != TwoStatesButtonWidget.OffState;
            bool isAlreadySet = isChecked ? isCurrentlyChecked : (!isCurrentlyChecked);

            if (!isAlreadySet)
            {
                string stateName = isChecked ? this.GetOnStateName() : TwoStatesButtonWidget.OffState;

                PdfDictionary clonedField = this.GetClonedFieldDictionary();
                clonedField[FormFieldNode.FieldValueName] = new PdfName(stateName);

                for (int i = 0; i < this.Field.WidgetObjects.Length; i++)
                {
                    PdfDictionary clonedWidget = this.GetClonedWidgetDictionary(i);
                    clonedWidget[AnnotationObject.AppearanceStateName] = new PdfName(stateName);
                }
            }
        }

        private string GetOnStateName()
        {
            PdfSourceImportContext importContext = this.Field.FileSource.Context;

            PdfArray options;
            if (this.Field.PdfDictionary.TryGetElement<PdfArray>(importContext.Reader, importContext, FormFieldNode.OptionsName, out options) && options.Count > 0)
            {
                IPdfPrimitive option = options[0];
                return option.ToString();
            }

            PdfName value;
            if (this.Field.PdfDictionary.TryGetElement<PdfName>(importContext.Reader, importContext, FormFieldNode.FieldValueName, out value) && value.Value != TwoStatesButtonWidget.OffState)
            {
                return value.Value;
            }
            
            for (int index = 0; index < this.Field.WidgetObjects.Length; index++)
            {
                string onStateName;
                if (this.TryGetWidgetOnState(index, out onStateName))
                {
                    return onStateName;
                }
            }

            return CheckBoxField.DefaultExportValue;
        }
    }
}
