﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate.InteractiveForms
{
    internal class FormFieldPropertiesEditor : IDisposable
    {
        private readonly DisposeValidator disposeValidator;
        private readonly PdfIncrementalStreamExportContext context;
        private readonly PdfFormFieldSource field;
        private PdfDictionary[] clonedWidgetsDictionaries;
        private PdfDictionary clonedFieldDictionary;

        internal FormFieldPropertiesEditor(PdfIncrementalStreamExportContext context, PdfFormFieldSource field)
        {
            this.field = field;
            this.context = context;
            this.disposeValidator = new DisposeValidator();
        }

        protected PdfFormFieldSource Field
        {
            get
            {
                return this.field;
            }
        }

        protected PdfIncrementalStreamExportContext Context
        {
            get
            {
                return this.context;
            }
        }

        protected PdfDictionary GetClonedFieldDictionary()
        {
            this.EnsureClonedFieldDictionary();

            return this.clonedFieldDictionary;
        }

        protected PdfDictionary GetClonedWidgetDictionary(int widgetIndex)
        {
            if(this.Field.HasMergedWidgetDictionary)
            {
                Guard.ThrowExceptionIfNotEqual(0, widgetIndex, "widgetIndex");

                return this.GetClonedFieldDictionary();
            }
            else
            {
                this.EnsureClonedWidget(widgetIndex);

                return this.clonedWidgetsDictionaries[widgetIndex];
            }
        }

        public void Dispose()
        {
            this.disposeValidator.Dispose();

            this.WriteModifiedObjects();
        }

        private void WriteModifiedObjects()
        {
            if (this.clonedFieldDictionary != null)
            {
                this.context.RegisterIndirectReference(this.clonedFieldDictionary, this.Field.IndirectObject.Reference.ObjectNumber, true);
            }

            if (this.clonedWidgetsDictionaries != null)
            {                
                for (int index = 0; index < this.clonedWidgetsDictionaries.Length; index++)
                {
                    PdfDictionary widgetClonedDictionary = this.clonedWidgetsDictionaries[index];

                    if(widgetClonedDictionary != null)
                    {
                        IndirectObject widgetObject = this.Field.WidgetObjects[index];
                        this.context.RegisterIndirectReference(widgetClonedDictionary, widgetObject.Reference.ObjectNumber, true);
                    }
                }
            }

            PdfExporter.WritePendingIndirectObjects(this.context, this.context.Writer);
        }

        private void EnsureClonedFieldDictionary()
        {
            if (this.clonedFieldDictionary == null)
            {
                this.clonedFieldDictionary = ResourceRenamer.GetDictionaryWithRenamedResources(this.Field.PdfDictionary, this.Field.FileSource, context);
            }
        }

        private void EnsureClonedWidget(int widgetIndex)
        {
            if (this.clonedWidgetsDictionaries == null)
            {
                this.clonedWidgetsDictionaries = new PdfDictionary[this.Field.WidgetObjects.Length];
            }

            if(this.clonedWidgetsDictionaries[widgetIndex] == null)
            {
                PdfDictionary widgetDictionary = this.Field.WidgetObjects[widgetIndex].GetContent<PdfDictionary>();
                PdfDictionary clonedWidgetDictionary = ResourceRenamer.GetDictionaryWithRenamedResources(widgetDictionary, this.Field.FileSource, this.Context);
                this.clonedWidgetsDictionaries[widgetIndex] = clonedWidgetDictionary;
            }
        }
    }
}
