﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Annotations;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate.InteractiveForms
{
    internal class RadioButtonPropertiesEditor : TwoStateAppearanceFieldPropertiesEditor
    {
        public RadioButtonPropertiesEditor(PdfIncrementalStreamExportContext context, PdfFormFieldSource field) 
            : base(context, field)
        {
        }

        public void SetValue(int widgetIndex)
        {
            PdfName widgetState;
            PdfSourceImportContext importContext = this.Field.FileSource.Context;
            PdfDictionary widget = this.Field.WidgetObjects[widgetIndex].GetContent<PdfDictionary>();
            bool isAlreadySelected =
                widget.TryGetElement<PdfName>(importContext.Reader, importContext, AnnotationObject.AppearanceStateName, out widgetState) && widgetState.Value != TwoStatesButtonWidget.OffState;

            string onStateName;
            if (!isAlreadySelected && this.TryGetWidgetOnState(widgetIndex, out onStateName))
            {
                PdfDictionary field = this.GetClonedFieldDictionary();
                field[FormFieldNode.FieldValueName] = new PdfName(onStateName);

                for (int i = 0; i < this.Field.WidgetObjects.Length; i++)
                {
                    string currentWidgetOnState;
                    if (this.TryGetWidgetOnState(i, out currentWidgetOnState))
                    {
                        string stateToApply = currentWidgetOnState == onStateName ? onStateName : TwoStatesButtonWidget.OffState;
                        PdfDictionary clonedWidget = this.GetClonedWidgetDictionary(i);
                        clonedWidget[AnnotationObject.AppearanceStateName] = new PdfName(stateToApply);
                    }
                }
            }            
        }

        public void ClearValue()
        {
            bool isAlreadyCleared = !this.Field.PdfDictionary.ContainsKey(FormFieldNode.FieldValueName);

            if (!isAlreadyCleared)
            {
                PdfDictionary field = this.GetClonedFieldDictionary();
                field.Remove(FormFieldNode.FieldValueName);

                for (int i = 0; i < this.Field.WidgetObjects.Length; i++)
                {
                    PdfDictionary clonedWidget = this.GetClonedWidgetDictionary(i);
                    clonedWidget[AnnotationObject.AppearanceStateName] = new PdfName(TwoStatesButtonWidget.OffState);
                }
            }
        }
    }
}
