﻿using System;
using System.Security.Cryptography.X509Certificates;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DigitalSignature;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.DigitalSignatures;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate.InteractiveForms
{
    internal class SignaturePropertiesEditor : SingleStateAppearanceFieldPropertiesEditor
    {
        private SignatureObject signatureValue;

        public SignaturePropertiesEditor(PdfIncrementalStreamExportContext context, PdfFormFieldSource fieldSource)
            : base(context, fieldSource)
        {
        }

        public void SetValue(X509Certificate2 certificate)
        {
            Guard.ThrowExceptionIfNull(certificate, "certificate");

            Signature signature = new Signature(certificate);
            signature.Properties.FieldName = this.Field.FieldName;

            this.signatureValue = new SignatureObject();
            this.signatureValue.CopyPropertiesFrom(signature);
            this.Context.SignatureToUpdate = signature;
            
            if (this.Field.PdfDictionary.ContainsKey(FormFieldNode.FieldValueName))
            {
                IndirectReference signatureValueReference = (IndirectReference)this.Field.PdfDictionary[FormFieldNode.FieldValueName];
                this.Context.RegisterIndirectReference(this.signatureValue, signatureValueReference.ObjectNumber, true);
            }
            else
            {
                IndirectReference signatureValueReference = this.Context.CreateIndirectObject(this.signatureValue).Reference;
                PdfDictionary clonedDictionary = this.GetClonedFieldDictionary();
                clonedDictionary[FormFieldNode.FieldValueName] = signatureValueReference;
            }
        }
    }
}