﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate
{
    internal class PagePropertiesEditor : IDisposable
    {
        private readonly DisposeValidator disposeValidator;
        private readonly PdfIncrementalStreamExportContext context;
        private readonly PdfPageSource page;
        private readonly PdfDictionary originalPageDictionary;
        private PdfDictionary clonedPageDictionary;

        public PagePropertiesEditor(PdfIncrementalStreamExportContext context, PdfPageSource page)
        {
            this.disposeValidator = new DisposeValidator();
            this.context = context;
            this.page = page;

            IndirectObject originalPageObject = page.FileSource.Context.ReadIndirectObject(page.PageReference);
            this.originalPageDictionary = (PdfDictionary)originalPageObject.Content;
        }

        public void SetMediaBox(Rect mediaBox)
        {
            bool isAlreadySet = this.page.MediaBox == mediaBox;

            if (!isAlreadySet)
            {
                this.EnsureClonedPageDictionary();
                this.clonedPageDictionary[Page.MediaBoxName] = mediaBox.ToBottomLeftCoordinateSystem(mediaBox.Height);
            }
        }

        public void SetRotation(Rotation rotation)
        {
            bool isAlreadySet = this.page.Rotation == rotation;

            if (!isAlreadySet)
            {
                this.EnsureClonedPageDictionary();
                this.clonedPageDictionary[Page.RotateName] = Page.GetRotate(rotation);
            }
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing,
        /// or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            this.disposeValidator.Dispose();

            this.WritePageObject();
        }

        private void WritePageObject()
        {
            this.context.RegisterIndirectReference(this.clonedPageDictionary, this.page.PageReference.ObjectNumber, true);
            PdfExporter.WritePendingIndirectObjects(this.context, this.context.Writer);
        }

        private void EnsureClonedPageDictionary()
        {
            if (this.clonedPageDictionary == null)
            {
                this.clonedPageDictionary = ResourceRenamer.GetDictionaryWithRenamedResources(this.originalPageDictionary, page.FileSource, context);
            }
        }
    }
}
