﻿using System;
using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Utilities;
#if WPF
using Telerik.Windows.Documents.Fixed.Model.DigitalSignatures;
#endif

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate
{
    internal class PdfIncrementalStreamExportContext : PdfExportContext, IResourceRenamingExportContext
    {
        private readonly PdfWriter writer;
        private readonly int initialMaxObjectNumber;
        private readonly Dictionary<IndirectReference, IndirectReference> sourceToContextReferenceMapping;
#if WPF
        private Signature signatureToUpdate;
#endif
        public PdfIncrementalStreamExportContext(Stream stream, int initialMaxObjectNumber) 
            : base(null, new PdfExportSettings())
        {
            this.writer = new PdfWriter(stream);
            this.initialMaxObjectNumber = initialMaxObjectNumber;
            this.sourceToContextReferenceMapping = new Dictionary<IndirectReference, IndirectReference>();
        }

        public PdfWriter Writer
        {
            get
            {
                return this.writer;
            }
        }
#if WPF
        public Signature SignatureToUpdate
        {
            get
            {
                return this.signatureToUpdate;
            }
            set
            {
                Guard.ThrowExceptionIfNotNull(this.signatureToUpdate, "signatureToUpdate");

                this.signatureToUpdate = value;
            }
        }
#endif
        public void RegisterIndirectReference(IPdfPrimitive primitive, int objectNumber, bool writeObject)
        {
            this.CreateIndirectObject(primitive, objectNumber, writeObject, writeObject);
        }

        public void AddSourceToContextReferenceMapping(PdfFileSource source, IndirectReference sourceReference, IndirectReference contextReference)
        {
            this.sourceToContextReferenceMapping.Add(sourceReference, contextReference);
        }

        public bool TryGetContextIndirectReference(PdfFileSource source, IndirectReference sourceReference, out IndirectReference contextReference)
        {
            if(!this.sourceToContextReferenceMapping.TryGetValue(sourceReference, out contextReference))
            {
                contextReference = sourceReference;
            }

            return true;
        }

        protected override int GetNextObjectNumber()
        {
            int objectNumber = Math.Max(this.initialMaxObjectNumber + 1, base.GetNextObjectNumber());

            return objectNumber;
        }
    }
}
