﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Converters;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming
{
    internal class PdfFormFieldSource
    {
        private readonly string fieldName;
        private readonly PdfFileSource fileSource;
        private readonly IndirectObject sourceObject;
        private IndirectObject[] widgetObjects;

        internal PdfFormFieldSource(PdfFileSource pdfFileSource, IndirectObject sourceObject, string fieldName)
        {
            this.fileSource = pdfFileSource;
            this.sourceObject = sourceObject;
            this.fieldName = fieldName;
        }

        internal string FieldName
        {
            get
            {
                return this.fieldName;
            }
        }

        internal PdfFileSource FileSource
        {
            get
            {
                return this.fileSource;
            }
        }

        internal IndirectObject IndirectObject
        {
            get
            {
                return this.sourceObject;
            }
        }

        internal PdfDictionary PdfDictionary
        {
            get
            {
                return this.IndirectObject.GetContent<PdfDictionary>();
            }
        }

        internal IndirectObject[] WidgetObjects
        {
            get
            {
                if(this.widgetObjects == null)
                {
                    this.widgetObjects = this.ReadWidgetObjects().ToArray();
                }

                return this.widgetObjects;
            }
        }

        internal bool HasMergedWidgetDictionary
        {
            get
            {
                bool isMergedDictionary = FieldsConverter.IsWidget(this.FileSource.Context.Reader, this.FileSource.Context, this.PdfDictionary);

                return isMergedDictionary;
            }
        }

        public override string ToString()
        {
            return this.FieldName;
        }

        private IEnumerable<IndirectObject> ReadWidgetObjects()
        {
            if (this.HasMergedWidgetDictionary)
            {
                yield return this.IndirectObject;
            }
            else
            {
                if (this.PdfDictionary.ContainsKey(FormFieldNode.KidsName))
                {
                    PdfArray pdfArray = this.PdfDictionary[FormFieldNode.KidsName] as PdfArray;

                    if (pdfArray != null)
                    {
                        foreach (IPdfPrimitive item in pdfArray)
                        {
                            IndirectReference kidReference = (IndirectReference)item;
                            IndirectObject widgetObject = this.FileSource.Context.ReadIndirectObject(kidReference);

                            if (widgetObject.Content.Type == PdfElementType.Dictionary)
                            {
                                yield return widgetObject;
                            }
                        }
                    }
                }
            }
        }
    }
}
