﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming
{
    /// <summary>
    /// Represents PDF file page source.
    /// </summary>
    public class PdfPageSource : IFixedPage
    {
        private readonly PdfFileSource fileSource;
        private readonly IndirectReference reference;
        private readonly Page page;
        private readonly Rect mediaBox;
        private readonly Rect cropBox;
        private PdfDictionary pageDictionary;

        internal PdfPageSource(PdfFileSource fileSource, IndirectReference reference, Page page)
        {
            this.fileSource = fileSource;
            this.reference = reference;
            this.page = page;
            this.mediaBox = page.ConvertToTopLeftBox(page.MediaBox, fileSource.Context.Reader, fileSource.Context);
            this.cropBox = page.ConvertToTopLeftBox(page.CropBox, fileSource.Context.Reader, fileSource.Context);
        }

        /// <summary>
        /// Gets the rotation.
        /// </summary>
        /// <value>The rotation.</value>
        public Rotation Rotation
        {
            get
            {
                return this.page.Rotation;
            }
        }

        /// <summary>
        /// Gets the size of the physical medium 
        /// on which the page is intended to be displayed or printed.
        /// </summary>
        public Size Size
        {
            get
            {
                return new Size(this.MediaBox.Width, this.MediaBox.Height);
            }
        }

        /// <summary>
        /// Gets the rectangle defining the boundaries of the physical medium 
        /// on which the page is intended to be displayed or printed.
        /// </summary>
        public Rect MediaBox
        {
            get
            {
                return this.mediaBox;
            }
        }

        /// <summary>
        /// Gets the rectangle defining the boundaries of the visible region of the page. 
        /// When the page is displayed or printed, its contents are to be clipped (cropped) to this rectangle.
        /// </summary>
        public Rect CropBox
        {
            get
            {
                return this.cropBox;
            }
        }

        internal PdfFileSource FileSource
        {
            get
            {
                return this.fileSource;
            }
        }

        internal Page Page
        {
            get
            {
                return this.page;
            }
        }

        internal IndirectReference PageReference
        {
            get
            {
                return this.reference;
            }
        }

        internal PdfDictionary PageDictionary
        {
            get
            {
                if (this.pageDictionary == null)
                {
                    this.pageDictionary = this.ReadPageDictionary();
                }

                return this.pageDictionary;
            }
        }

        private PdfDictionary ReadPageDictionary()
        {
            IndirectObject pageObject = this.FileSource.Context.ReadIndirectObject(this.reference);
            PdfDictionary dictionary = (PdfDictionary)pageObject.Content;

            return dictionary;
        }
    }
}
