﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming
{
    /// <summary>
    /// Represents settings for the produced PDF file stream.
    /// </summary>
    public class PdfStreamWriterSettings
    {
        private readonly PdfExportSettings exportSettings;
        private readonly RadFixedDocumentInfo documentInfo;

        /// <summary>
        /// Initializes a new instance of the <see cref="PdfStreamWriterSettings" /> class.
        /// </summary>
        internal PdfStreamWriterSettings(PdfExportSettings exportSettings, RadFixedDocumentInfo documentInfo)
        {
            this.exportSettings = exportSettings;
            this.documentInfo = documentInfo;
            this.WriteAnnotations = true;
        }

        /// <summary>
        /// Gets the document information.
        /// </summary>
        public RadFixedDocumentInfo DocumentInfo
        {
            get
            {
                return this.documentInfo;
            }
        }

        /// <summary>
        /// Gets or sets the default image quality when exporting images to PDF. The default value is High.
        /// <para>
        /// The value of this property will be overriden when specifying the ImageQuality in ImageSource constructor or when creating ImageSource from EncodedImageData.
        /// The higher the quality, the bigger the document size.
        /// </para>
        /// </summary>
        /// <value>The image quality.</value>
        public ImageQuality ImageQuality
        {
            get
            {
                return this.exportSettings.ImageQuality;
            }
            set
            {
                this.exportSettings.ImageQuality = value;
            }
        }

        /// <summary>
        /// Gets or sets boolean value indicating whether to write annotations.
        /// </summary>
        /// <value>Value indication whether to write annotations.</value>
        public bool WriteAnnotations { get; set; }
    }
}
