﻿using System;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities
{
    internal class BitReader
    {
        private readonly int bits;

        protected byte[] Source { get; private set; }

        protected int SourcePointer { get; set; }

        protected int BitPointer { get; set; }

        public BitReader(byte[] source, int intSizeInBits)
        {
            if (intSizeInBits > 31)
            {
                throw new NotSupportedException("BitReader supports only integers with less than 31 bits.");
            }

            this.Source = source;
            this.bits = intSizeInBits;
        }

        // Returns a number greater than or equal to zero if not past the end, -1 otherwise.
        public virtual int Read()
        {
            if (this.SourcePointer >= this.Source.Length)
            {
                return -1;
            }

            int value;

            switch (this.bits)
            {
                case 1:
                    value = (this.Source[this.SourcePointer] >> (7 - this.BitPointer++)) & 1;
                    
                    if (this.BitPointer == 8)
                    {
                        this.SourcePointer++;
                        this.BitPointer = 0;
                    }
                    break;
                case 8:
                    value = this.Source[this.SourcePointer++];
                    break;
                case 16:
                    value = (this.Source[this.SourcePointer++] << 8);

                    if (this.SourcePointer < this.Source.Length)
                    {
                        value |= this.Source[this.SourcePointer++];
                    }
                    break;
                default:
                    value = this.ReadInGeneralScenario();
                    break;
            }

            return value;
        }

        private int ReadInGeneralScenario()
        {
            int value = 0;
            int remainingBits = this.bits;

            while (remainingBits > 0)
            {
                byte sourceByte = this.Source[this.SourcePointer];
                int numberOfBitsToRemoveFromLeft = this.BitPointer;
                int numberOfBitsToCopy = Math.Min(remainingBits, 8 - numberOfBitsToRemoveFromLeft);
                int numberOfBitsToRemoveFromRight = 8 - numberOfBitsToRemoveFromLeft - numberOfBitsToCopy;
                int mask = (1 << numberOfBitsToCopy) - 1;
                int bitsToCopy = (sourceByte >> numberOfBitsToRemoveFromRight) & mask;

                value |= (bitsToCopy << (remainingBits - numberOfBitsToCopy));

                remainingBits -= numberOfBitsToCopy;
                this.BitPointer += numberOfBitsToCopy;

                if (this.BitPointer == 8)
                {
                    this.SourcePointer++;
                    this.BitPointer = 0;
                }

                if (this.SourcePointer >= this.Source.Length)
                {
                    break;
                }
            }

            return value;
        }
    }
}
