﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities
{
    internal class BitWriter
    {
        private readonly byte[] target;
        private readonly int intSizeInBits;

        public BitWriter(int numberOfRecords, int intSizeInBits)
            : this(new byte[CalculateNumberOfBytes(numberOfRecords, intSizeInBits)], intSizeInBits)
        {
        }

        protected BitWriter(byte[] target, int intSizeInBits)
        {
            if (intSizeInBits > 31)
            {
                throw new NotSupportedException("BitReader supports only integers with less than 31 bits.");
            }

            this.target = target;
            this.intSizeInBits = intSizeInBits;
        }

        public byte[] ResultBits
        {
            get
            {
                return this.target;
            }
        }

        protected int IntSizeInBits
        {
            get
            {
                return this.intSizeInBits;
            }
        }

        protected int BytePointer { get; set; }

        protected int BitPointer { get; set; }

        public static int CalculateNumberOfBytes(int numberOfRecords, int bitsPerRecord)
        {
            int numberOfBits = numberOfRecords * bitsPerRecord;
            int numberOfBytes = (numberOfBits + 7) / 8;

            return numberOfBytes;
        }

        public virtual void Write(int bits)
        {
            for (int bitIndex = 0; bitIndex < this.IntSizeInBits; bitIndex++)
            {
                Guard.ThrowExceptionIfTrue(this.BytePointer == this.ResultBits.Length, "Cannot write after the last available byte!");

                this.WriteBit(bits, bitIndex, this.BytePointer, this.BitPointer);
                this.BitPointer++;

                if (this.BitPointer == 8)
                {
                    this.BytePointer++;
                    this.BitPointer = 0;
                }
            }
        }

        protected void WriteBit(int bitSource, int bitIndex, int bytePointer, int bitPointer)
        {
            int bitToSet = ((bitSource >> (this.IntSizeInBits - bitIndex - 1)) & 1);
            int mask = 1 << (7 - bitPointer);

            if (bitToSet == 1)
            {
                this.ResultBits[bytePointer] = (byte)(this.ResultBits[bytePointer] | mask);
            }
            else
            {
                mask = ~mask;
                this.ResultBits[bytePointer] = (byte)(this.ResultBits[bytePointer] & mask);
            }
        }
    }
}
