﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities
{
    internal static class BytesHelper
    {
        public static byte GetDecimalFromHex(char hex)
        {
            if (hex >= '0' && hex <= '9')
            {
                return (byte)(hex - '0');
            }
            else if (hex >= 'a' && hex <= 'f')
            {
                return (byte)(hex - 'a' + 10);
            }
            else
            {
                return (byte)(hex - 'A' + 10);
            }
        }

        public static string ToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 2);
            foreach (byte b in data)
            {
                sb.AppendFormat("{0:x2}", b);
            }
            
            return sb.ToString();
        }

        public static byte[] ToByteArray(string hex)
        {
            if (hex.Length % 2 == 1)
            {
                throw new Exception("The binary key cannot have an odd number of digits");
            }

            int size = hex.Length >> 1;
            byte[] arr = new byte[size];

            for (int i = 0; i < size; ++i)
            {
                arr[i] = (byte)((GetHexVal(hex[i << 1]) << 4) + GetHexVal(hex[(i << 1) + 1]));
            }

            return arr;
        }

        public static int GetHexVal(char hex)
        {
            return hex - (hex < 58 ? 48 : (hex < 97 ? 55 : 87));
        }

        public static int GetInt(byte[] bytes)
        {
            byte[] b = GetBytes(bytes);
            if (b == null)
            {
                return -1;
            }
            int res = 0;
            int len = b.Length;
            for (int i = 0; i < len; i++)
            {
                res |= len <= i ? 0 & 0xFF : b[i] & 0xFF;
                if (i < len - 1)
                {
                    res <<= 8;
                }
            }
            return res;
        }

        public static void GetBytesReverse(int value, byte[] result)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            for (int i = 0; i < bytes.Length; i++)
            {
                int index = result.Length - i - 1;
                if (index < 0 || index >= result.Length)
                {
                    break;
                }

                result[index] = bytes[i];
            }
        }

        public static byte[] MergeBytes(byte[] first, byte[] second)
        {
            LinkedList<byte> result = new LinkedList<byte>();
            foreach (var b in first)
            {
                result.AddFirst(b);
            }

            foreach (var b in second)
            {
                result.AddLast(b);
            }

            return result.ToArray();
        }

        public static byte[] GetBytes(Stream stream)
        {
            if (stream.Position > 0)
            {
                stream.Seek(0, SeekOrigin.Begin);
            }

            byte[] data = new byte[stream.Length];
            stream.Read(data, 0, data.Length);

            return data;
        }

        public static void GetBytes(int value, byte[] result)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            for (int i = 0; i < result.Length; i++)
            {
                result[i] = bytes[i];
            }
        }

        private static byte[] GetBytes(byte[] bytes)
        {
            Guard.ThrowExceptionIfNull(bytes, "bytes");
            Guard.ThrowExceptionIfLessThanOrEqual(0, bytes.Length, "bytes");
            Guard.ThrowExceptionIfGreaterThan(4, bytes.Length, "bytes");

            int len = bytes.Length;
            int bCount = (len / 2 + len % 2) * 2; // Get number of bytes;
            byte[] b = new byte[bCount];
            if (len == 1)
            {
                b[0] = 0;
                b[1] = bytes[0];
            }
            else if (bCount == 2)
            {
                b[0] = bytes[0];
                b[1] = bytes[1];
            }
            else if (len == 3)
            {
                b[0] = 0;
                b[1] = bytes[0];
                b[2] = bytes[1];
                b[3] = bytes[2];
            }
            else
            {
                b[0] = bytes[0];
                b[1] = bytes[1];
                b[2] = bytes[2];
                b[3] = bytes[3];
            }

            return b;
        }
    }
}
