﻿namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities
{
    internal class ImageBitReader : BitReader
    {
        private readonly int width;
        private int pixelIndexInRow = 0;

        public ImageBitReader(byte[] source, int bitsPerComponent, int width) 
            : base(source, bitsPerComponent)
        {
            this.width = width;
            this.pixelIndexInRow = 0;
        }

        public override int Read()
        {
            int readResult = base.Read();

            this.pixelIndexInRow++;
            if(this.pixelIndexInRow == this.width)
            {
                this.MoveToNextRow();
            }

            return readResult;
        }

        public int GetBitAt(int index)
        {
            int row = index / this.width;
            int column = index % this.width;
            return this.GetBitAt(row, column);
        }

        public int GetBitAt(int row, int column)
        {
            int bytesPerRow = 1 + (this.width - 1) / 8;
            int byteColumn = column / 8;

            int byteIndex = (row * bytesPerRow) + byteColumn;
            int bitIndex = column - (byteColumn * 8);
            int bit = (this.Source[byteIndex] >> (7 - bitIndex)) & 1;

            return bit;
        }

        private void MoveToNextRow()
        {
            if (BitPointer != 0)
            {
                this.SourcePointer++;
                BitPointer = 0;
            }

            this.pixelIndexInRow = 0;
        }
    }
}
