﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities
{
    internal class ImageBitWriter : BitWriter
    {
        private readonly int width;
        private int pixelIndexInRow;

        public ImageBitWriter(byte[] target, int intSizeInBits, int width)
            : base(target, intSizeInBits)
        {
            this.width = width;
        }

        public override void Write(int bits)
        {
            base.Write(bits);
            this.pixelIndexInRow++;

            if (this.pixelIndexInRow == this.width)
            {
                this.MoveToNextRow();
            }
        }

        public void WriteBitsToIndex(int bits, int index)
        {
            int bytePointer;
            int bitPointer;
            this.GetBitPositionFromIndex(index, out bytePointer, out bitPointer);

            for (int bitIndex = 0; bitIndex < this.IntSizeInBits; bitIndex++)
            {
                Guard.ThrowExceptionIfTrue(bytePointer == this.ResultBits.Length, "Cannot write after the last available byte!");

                this.WriteBit(bits, bitIndex, bytePointer, bitPointer);
                bitPointer++;

                if (bitPointer == 8)
                {
                    bytePointer++;
                    bitPointer = 0;
                }
            }
        }

        private void GetBitPositionFromIndex(int index, out int byteIndexInArray, out int bitIndexInByte)
        {
            int rowIndex = index / this.width;
            int bytesPerRow = CalculateNumberOfBytes(this.width, this.IntSizeInBits);
            int positionInRow = index % this.width;
            int bitPositionInRow = positionInRow * this.IntSizeInBits;   
            int byteIndexInRow = bitPositionInRow / 8;

            bitIndexInByte = bitPositionInRow % 8;
            byteIndexInArray = rowIndex * bytesPerRow + byteIndexInRow;
        }

        private void MoveToNextRow()
        {
            this.pixelIndexInRow = 0;

            if (this.BitPointer > 0)
            {
                this.BitPointer = 0;
                this.BytePointer++;
            }
        }
    }
}
