﻿using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Text
{
    /// <summary>
    /// Provides functionality to export RadFixedDocument to text.
    /// </summary>
    public class TextFormatProvider
    {
        /// <summary>
        /// Exports the specified document as text.
        /// </summary>
        /// <param name="document">The document to export.</param>
        /// <param name="settings">The settings.</param>
        /// <returns>The text, that is extracted from the RadFixedDocument.</returns>
        public string Export(RadFixedDocument document, TextFormatProviderSettings settings)
        {
            TextDocument textDocument;
            if (document.TextDocument == null)
            {
                // PdfProcessing
                textDocument = new TextDocument(document);
            }
            else
            {
                // PdfViewer
                textDocument = document.TextDocument;
            }

            return textDocument.ToString(settings.LinesSeparator, settings.PagesSeparator);
        }

        /// <summary>
        /// Exports the specified document as text.
        /// </summary>
        /// <param name="document">The document to export.</param>
        /// <returns>The text, that is extracted from the RadFixedDocument.</returns>
        public string Export(RadFixedDocument document)
        {
            return this.Export(document, TextFormatProviderSettings.Default);
        }
    }
}
