﻿using System;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Text
{
    /// <summary>
    /// Represents settigs that <see cref="TextFormatProvider"/> uses to export RadFixedDocument to text.
    /// </summary>
    public class TextFormatProviderSettings
    {
        internal static readonly string DefaultLinesSeparator = Line.LineSeparator;
        internal static readonly string DefaultPagesSeparator = "----------- Page{0} ------------";

        /// <summary>
        /// Gets the default settings.
        /// </summary>
        /// <value>The default settings.</value>
        public static TextFormatProviderSettings Default
        {
            get
            {
                return new TextFormatProviderSettings();
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextFormatProviderSettings" /> class.
        /// </summary>
        public TextFormatProviderSettings()
        {
            this.LinesSeparator = DefaultLinesSeparator;
            this.PagesSeparator = DefaultPagesSeparator;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextFormatProviderSettings" /> class.
        /// </summary>
        /// <param name="linesSeparator">The lines separator.</param>
        /// <param name="pagesSeparator">The pages separator.</param>
        public TextFormatProviderSettings(string linesSeparator, string pagesSeparator)
        {
            this.LinesSeparator = linesSeparator;
            this.PagesSeparator = pagesSeparator;
        }

        /// <summary>
        /// Gets or sets the lines separator.
        /// </summary>
        /// <value>The lines separator.</value>
        public string LinesSeparator { get;private set; }

        /// <summary>
        /// Gets or sets the pages separator.
        /// </summary>
        /// <value>The pages separator.</value>
        public string PagesSeparator { get;private set; }
    }
}
