﻿// The MIT License (MIT) 
// 
// Copyright (c) 2014 Daniel Glucksman 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 
using System;

namespace JBig2Decoder
{
    internal class GenericRegionSegment : RegionSegment
    {
    private GenericRegionFlags genericRegionFlags = new GenericRegionFlags();

    private bool inlineImage;
    private bool unknownLength = false;
    
    public GenericRegionSegment(JBIG2StreamDecoder streamDecoder, bool inlineImage):base(streamDecoder) {
        this.inlineImage = inlineImage;
    }

    public override void readSegment() {
    	
    	if(JBIG2StreamDecoder.debug)
    		Console.WriteLine("==== Reading Immediate Generic Region ====");
    	
    	base.readSegment();
        
        readGenericRegionFlags();

        bool useMMR = genericRegionFlags.getFlagValue(GenericRegionFlags.MMR) != 0;
        int template = genericRegionFlags.getFlagValue(GenericRegionFlags.GB_TEMPLATE);
        
        short[] genericBAdaptiveTemplateX = new short[4];
    	short[] genericBAdaptiveTemplateY = new short[4];
        
        if (!useMMR) {
        	if (template == 0) {
        		genericBAdaptiveTemplateX[0] = readATValue();
        		genericBAdaptiveTemplateY[0] = readATValue();
        		genericBAdaptiveTemplateX[1] = readATValue();
        		genericBAdaptiveTemplateY[1] = readATValue();
        		genericBAdaptiveTemplateX[2] = readATValue();
        		genericBAdaptiveTemplateY[2] = readATValue();
        		genericBAdaptiveTemplateX[3] = readATValue();
        		genericBAdaptiveTemplateY[3] = readATValue();
        	} else {
        		genericBAdaptiveTemplateX[0] = readATValue();
        		genericBAdaptiveTemplateY[0] = readATValue();
        	}
        	
        	arithmeticDecoder.resetGenericStats(template, null);
        	arithmeticDecoder.start();
        }
        
        bool typicalPredictionGenericDecodingOn = genericRegionFlags.getFlagValue(GenericRegionFlags.TPGDON) != 0;
        int length = segmentHeader.getSegmentDataLength();

        if(length == -1) { 
        	
        	
        	unknownLength = true;
        	
        	short match1;
        	short match2;
        	
        	if(useMMR) {
        		// look for 0x00 0x00 (0, 0)
        		
        		match1 = 0;
        		match2 = 0;
        	} else {
        		// look for 0xFF 0xAC (255, 172)
        		
        		match1 = 255;
        		match2 = 172;
        	}
        	
        	int bytesRead = 0;
    		while(true) {
    			short bite1 = decoder.readbyte();
    			bytesRead++;
    			
    			if(bite1 == match1){
    				short bite2 = decoder.readbyte();
    				bytesRead++;
    				
    				if(bite2 == match2){
    					length = bytesRead - 2;
    					break;
    				}
    			}
    		}
    		
    		decoder.movePointer(-bytesRead);
        }
        
        JBIG2Bitmap bitmap = new JBIG2Bitmap(regionBitmapWidth, regionBitmapHeight, arithmeticDecoder, huffmanDecoder, mmrDecoder);
        bitmap.clear(0);
        bitmap.readBitmap(useMMR, template, typicalPredictionGenericDecodingOn, false, null, genericBAdaptiveTemplateX, genericBAdaptiveTemplateY, useMMR ? 0 : length - 18);
        
        
        
        if (inlineImage) {
            PageInformationSegment pageSegment = decoder.findPageSegement(segmentHeader.getPageAssociation());
            JBIG2Bitmap pageBitmap = pageSegment.getPageBitmap();

            int extCombOp = regionFlags.getFlagValue(RegionFlags.EXTERNAL_COMBINATION_OPERATOR);
            
            if(pageSegment.getPageBitmapHeight() == -1 && regionBitmapYLocation + regionBitmapHeight > pageBitmap.getHeight()) {
            	pageBitmap.expand(regionBitmapYLocation + regionBitmapHeight, 
            			pageSegment.getPageInformationFlags().getFlagValue(PageInformationFlags.DEFAULT_PIXEL_VALUE));
            }
            
            pageBitmap.combine(bitmap, regionBitmapXLocation, regionBitmapYLocation, extCombOp);
        } else {
			bitmap.setBitmapNumber(getSegmentHeader().getSegmentNumber());
			decoder.appendBitmap(bitmap);
		}

        
        if(unknownLength) {
        	decoder.movePointer(4);
        }
        
    }

    private void readGenericRegionFlags() {
        short genericRegionFlagsField = decoder.readbyte();

        genericRegionFlags.setFlags(genericRegionFlagsField);
        
        if(JBIG2StreamDecoder.debug)
            Console.WriteLine("generic region Segment flags = " + genericRegionFlagsField);
    }

    public GenericRegionFlags getGenericRegionFlags() {
        return genericRegionFlags;
    }
}
}
