﻿// The MIT License (MIT) 
// 
// Copyright (c) 2014 Daniel Glucksman 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 
using System;

namespace JBig2Decoder
{
    internal class PageInformationSegment : Segment
    {

        private int pageBitmapHeight, pageBitmapWidth;
        private int yResolution, xResolution;

        PageInformationFlags pageInformationFlags = new PageInformationFlags();
        private int pageStriping;

        private JBIG2Bitmap pageBitmap;

        public PageInformationSegment(JBIG2StreamDecoder streamDecoder) : base(streamDecoder) { }

        public PageInformationFlags getPageInformationFlags()
        {
            return pageInformationFlags;
        }

        public JBIG2Bitmap getPageBitmap()
        {
            return pageBitmap;
        }

        public override void readSegment()  {

		if (JBIG2StreamDecoder.debug)
			Console.WriteLine("==== Reading Page Information Dictionary ====");

		    short[] buff = new short[4];
		    decoder.readbyte(buff);
		    pageBitmapWidth = BinaryOperation.getInt32(buff);

		    buff = new short[4];
		    decoder.readbyte(buff);
		    pageBitmapHeight = BinaryOperation.getInt32(buff);

		    if (JBIG2StreamDecoder.debug)
			    Console.WriteLine("Bitmap size = " + pageBitmapWidth + 'x' + pageBitmapHeight);

		    buff = new short[4];
		    decoder.readbyte(buff);
		    xResolution = BinaryOperation.getInt32(buff);

		    buff = new short[4];
		    decoder.readbyte(buff);
		    yResolution = BinaryOperation.getInt32(buff);

		    if (JBIG2StreamDecoder.debug)
			    Console.WriteLine("Resolution = " + xResolution + 'x' + yResolution);

		    short pageInformationFlagsField = decoder.readbyte();

		    pageInformationFlags.setFlags(pageInformationFlagsField);

		    if (JBIG2StreamDecoder.debug)
			    Console.WriteLine("symbolDictionaryFlags = " + pageInformationFlagsField);

		    buff = new short[2];
		    decoder.readbyte(buff);
		    pageStriping = BinaryOperation.getInt16(buff);

		    if (JBIG2StreamDecoder.debug)
                Console.WriteLine("Page Striping = " + pageStriping);

		    int defPix = pageInformationFlags.getFlagValue(PageInformationFlags.DEFAULT_PIXEL_VALUE);

		    int height;

		    if (pageBitmapHeight == -1) {
			    height = pageStriping & 0x7fff;
		    } else {
			    height = pageBitmapHeight;
		    }

		    pageBitmap = new JBIG2Bitmap(pageBitmapWidth, height, arithmeticDecoder, huffmanDecoder, mmrDecoder);
		    pageBitmap.clear(defPix);
	    }

        public int getPageBitmapHeight()
        {
            return pageBitmapHeight;
        }
    }
}
