﻿// The MIT License (MIT) 
// 
// Copyright (c) 2014 Daniel Glucksman 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 
using System;

namespace JBig2Decoder
{
    internal abstract class RegionSegment : Segment
    {
        protected int regionBitmapWidth, regionBitmapHeight;
        protected int regionBitmapXLocation, regionBitmapYLocation;

        protected RegionFlags regionFlags = new RegionFlags();

        public RegionSegment(JBIG2StreamDecoder streamDecoder): base(streamDecoder)  { }

        public override void readSegment()
        {
            short[] buff = new short[4];
            decoder.readbyte(buff);
            regionBitmapWidth = BinaryOperation.getInt32(buff);

            buff = new short[4];
            decoder.readbyte(buff);
            regionBitmapHeight = BinaryOperation.getInt32(buff);

            if (JBIG2StreamDecoder.debug)
                Console.WriteLine("Bitmap size = " + regionBitmapWidth + 'x' + regionBitmapHeight);

            buff = new short[4];
            decoder.readbyte(buff);
            regionBitmapXLocation = BinaryOperation.getInt32(buff);

            buff = new short[4];
            decoder.readbyte(buff);
            regionBitmapYLocation = BinaryOperation.getInt32(buff);

            if (JBIG2StreamDecoder.debug)
                Console.WriteLine("Bitmap location = " + regionBitmapXLocation + ',' + regionBitmapYLocation);

            short regionFlagsField = decoder.readbyte();

            regionFlags.setFlags(regionFlagsField);

            if (JBIG2StreamDecoder.debug)
                Console.WriteLine("region Segment flags = " + regionFlagsField);
        }
    }
}
