﻿// The MIT License (MIT) 
// 
// Copyright (c) 2014 Daniel Glucksman 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 

namespace JBig2Decoder
{
    internal class FastBitSet
    {
		public long[] w;
		public static int pot =  6 ;
        public static int mask = unchecked((int)(((ulong)(-1L)) >> (64 - pot)));
		public long length;
		
		
		public FastBitSet(long length) {
			this.length = length;
			long wcount = length / 64; 
			if (length % 64 != 0) wcount++;
			w = new long[wcount];
		}
		
		public long size() {
			return length;
		}

		public void setAll(bool value) {
			if (value)
				for (long i = 0; i<w.Length; i++){
					w[i] = -1L;
				}
			else
				for (long i = 0; i<w.Length; i++){
					w[i] = 0;
				}
				
		}
		
		public void set(long start, long end, bool value) {
			if (value) {
				for (long i=start; i<end; i++) {
					set(i);
				}
			}
			else {
				for (long i=start; i<end; i++) {
					clear(i);
				}				
			}
		}

        public void or(long startindex, FastBitSet set, long setStartIndex, long length)
        {           
            long shift = startindex - setStartIndex;
			long k = set.w[setStartIndex >> pot];            
			//Cyclic shift
            k = (k << (int)shift) | (long)((ulong)k >> (64 - (int)shift));            
			if ((setStartIndex & mask) + length <= 64) {
				setStartIndex += shift;
				for (long i=0; i<length; i++) {
                    w[(long)((ulong)startindex) >> pot] |= k & (1L << (int)setStartIndex);
					setStartIndex++;
					startindex++;
				}				
			}
			else{
				for (long i=0; i<length; i++) {                    
					if ((setStartIndex & mask) == 0){ 
						k = set.w[(setStartIndex) >> pot];
						k = (k << (int)shift) | (long)((ulong)k >> (64 - (int)shift));
					}          
                    w[(long)((ulong)startindex >> pot)] |= k & (1L << (int)(setStartIndex + shift));
					setStartIndex++;
					startindex++;
				}
			}
		}
		
		public void set(long index, bool value) {
			if (value) set(index);
			else clear(index);
		}
		
		public void set(long index) {
            long windex = (long)((ulong)index >> pot);
			w[windex] |= (1L<<(int)index);
		}
		
		public void clear(long index) {
            long windex = (long)((ulong)index >> pot);
			w[windex] &= ~(1L<<(int)index);
		}

		public bool get(long index) {
            long windex = (long)((ulong)index >> pot);
			return ((w[windex] & (1L<<(int)index)) != 0);
		}		
	}
}
