﻿// The MIT License (MIT) 
// 
// Copyright (c) 2014 Daniel Glucksman 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 

namespace JBig2Decoder
{
    internal class Big2StreamReader
    {
	    private byte[] data;

	    private int bitPointer = 7;

	    private int bytePointer = 0;

	    public Big2StreamReader(byte[] data) {
		    this.data = data;
	    }	    

	    public short readbyte() {
		    short bite = (short) (data[bytePointer++] & 255);

		    return bite;
	    }

	    public void readbyte(short[] buf) {
		    for (int i = 0; i < buf.Length; i++) {
			    buf[i] = (short) (data[bytePointer++] & 255);
		    }
	    }

	    public int readBit() {
		    short buf = readbyte();
		    short mask = (short) (1 << bitPointer);

		    int bit = (buf & mask) >> bitPointer;

		    bitPointer--;
		    if (bitPointer == -1) {
			    bitPointer = 7;
		    } else {
			    movePointer(-1);
		    }

		    return bit;
	    }

	    public int readBits(long num) {
		    int result = 0;

		    for (int i = 0; i < num; i++) {
			    result = (result << 1) | readBit();
		    }

		    return result;
	    }

	    public void movePointer(int ammount) {
		    bytePointer += ammount;
	    }

	    public void consumeRemainingBits() {
		    if (bitPointer != 7)
			    readBits(bitPointer + 1);
	    }

	    public bool isFinished() {
		    return bytePointer == data.Length;
	    }
}
}
