using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Media.Imaging;

namespace Telerik.Windows.Documents.Fixed.Imaging.Silverlight
{
	/// <summary>
	/// Encodes a collection of BitmapFrame objects to a stream.
	/// </summary>
    internal abstract class BitmapEncoder
	{
		private List<WriteableBitmap> frames;

		/// <summary>
		/// Initializes a new instance of the BitmapEncoder class.
		/// </summary>
        protected BitmapEncoder()
        {
			this.frames = new List<WriteableBitmap>();
        }

		/// <summary>
		/// Gets information that describes this codec.
		/// </summary>
		public virtual BitmapCodecInfo CodecInfo
        {
            get
            {
                return null;
            }
        }

		/// <summary>
		/// Gets or sets the individual frames within an image.
		/// </summary>
		public IList<WriteableBitmap> Frames
		{
			get
			{
				return this.frames;
			}
		}

		/// <summary>
		/// Disposes the encoder.
		/// </summary>
		public virtual void Dispose()
		{
		}

		/// <summary>
		/// Encodes a bitmap image to a specified System.IO.Stream.
		/// </summary>
		/// <param name="stream">Stream.</param>
        public virtual void Save(Stream stream)
        {
            if (this.Frames.Count == 0)
            {
                throw new System.NotSupportedException("Frames Count is 0.");
            }

            if (this.Frames[0] == null)
            {
                throw new System.NotSupportedException("The Frame is null.");
            }

			WriteableBitmap image = (this.Frames[0] as BitmapSource) as WriteableBitmap;
			if (image == null)
			{
				throw new System.NotSupportedException("The Frame must have the WriteableBitmap type.");
			}
		}
	}
}
