﻿using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Actions
{
    /// <summary>
    /// Represents go to action.
    /// </summary>
    public class GoToAction : Action
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GoToAction" /> class.
        /// </summary>
        public GoToAction()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GoToAction" /> class.
        /// </summary>
        /// <param name="destination">The destination for the action.</param>
        public GoToAction(Destination destination)
        {
            Guard.ThrowExceptionIfNull(destination, "destination");

            this.Destination = destination;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GoToAction" /> class.
        /// </summary>
        /// <param name="namedDestination">The named destination for the action.</param>
        public GoToAction(NamedDestination namedDestination)
        {
            Guard.ThrowExceptionIfNull(namedDestination, "namedDestination");

            this.NamedDestination = namedDestination;
        }

        /// <summary>
        /// Gets or sets the destination for the action.
        /// </summary>
        /// <value>The destination.</value>
        public Destination Destination { get; set; }

        /// <summary>
        /// Gets or sets the named destination for the action.
        /// </summary>
        /// <value>The named destination.</value>
        public NamedDestination NamedDestination { get; set; }

        internal override ActionType ActionType
        {
            get
            {
                return ActionType.GoTo;
            }
        }

        internal override Action Clone(RadFixedDocumentCloneContext cloneContext)
        {
            return new GoToAction()
            {
                Destination = cloneContext.GetClonedDestination(this.Destination),
                NamedDestination = cloneContext.GetClonedNamedDestination(this.NamedDestination),
            };
        }
    }
}
