﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Actions
{
    /// <summary>
    /// Represents Uri action.
    /// </summary>
    public class UriAction : Action
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="UriAction" /> class.
        /// </summary>
        public UriAction()  
        {
            this.IncludeMouseCoordinates = false;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UriAction" /> class.
        /// </summary>
        /// <param name="uri">The URI.</param>
        public UriAction(Uri uri) : this()
        {
            this.Uri = uri;
        }

        /// <summary>
        /// Gets or sets the URI for the action.
        /// </summary>
        /// <value>The URI.</value>
        public Uri Uri { get; set; }

        /// <summary>
        /// Gets or sets the value that indicates if the mouse coordinates should be included in the Uri.
        /// </summary>
        /// <value>If the mouse coordinates should be tracket.</value>
        public bool? IncludeMouseCoordinates { get; set; }
         
        internal override ActionType ActionType
        {
            get { return Actions.ActionType.Uri; }
        }

        internal override Action Clone(RadFixedDocumentCloneContext context)
        {
            return new UriAction()
            {
                Uri = this.Uri,
                IncludeMouseCoordinates = this.IncludeMouseCoordinates
            };
        }
    }
}
