﻿using System;
#if !NETSTANDARD
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Telerik.Windows.Documents.Fixed.Model.Internal.Annotations;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents base class for annotation object.
    /// </summary>
    public abstract class Annotation : FixedDocumentElementBase, IInstanceIdOwner, IContextClonable<Annotation>
    {
        private readonly int id;

        internal Annotation()
        {
            this.id = InstanceIdGenerator.GetNextId();
            this.Border = AnnotationBorder.DefaultAnnotationBorder;
            this.InitializeDefaultAnnotationFlags();
        }

        /// <summary>
        /// Gets or sets the position of the annotation.
        /// </summary>
        /// <value>The rect.</value>
        public Rect Rect { get; set; }

        /// <summary>
        /// Gets the type of the annotation.
        /// </summary>
        /// <value>The type of the annotation.</value>
        public abstract AnnotationType Type { get; }

        /// <summary>
        /// Gets or sets the annotation borders.
        /// </summary>
        /// <value>The border.</value>
        public AnnotationBorder Border { get; set; }

        /// <summary>
        /// Gets or sets boolean value indicating whether the annotation instance should be visualized when printing the document.
        /// </summary>
        public bool IsPrintable { get; set; }

        internal bool IsDisplayedWhenNotSupported { get; set; }

        internal bool IsHidden { get; set; }

        internal bool IsZoomingWithPage { get; set; }

        internal bool IsRotatedWithPage { get; set; }

        internal bool IsVisibleInViewerUI { get; set; }

        internal bool IsReadOnly { get; set; }

        internal bool IsLockedByPositionAndSize { get; set; }

        internal bool IsContentLocked { get; set; }

        internal bool IsTogglingVisibilityInViewerUI { get; set; }

        internal AnnotationAppearances Appearances { get; set; }

        //Legacy
        internal Rect? BoundingRect { get; set; }

#if !NETSTANDARD
        internal Rect AnnotationOldRect { get; set; }

        internal AnnotationAppearancesOld AppearanceOld { get; set; }
#endif
        //End Legacy

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.Annotation;
            }
        }

        int IInstanceIdOwner.InstanceId
        {
            get
            {
                return this.id;
            }
        }

#if !NETSTANDARD
        internal Rect Arrange(Core.Data.Matrix matrix)
        {
            this.BoundingRect = Helper.GetBoundingRect(this.AnnotationOldRect, matrix);

            return this.BoundingRect.Value;
        }
#endif

        internal virtual Annotation CreateClonedInstance(RadFixedDocumentCloneContext cloneContext)
        {
            // Implement in inheritors.
            throw new NotImplementedException("ClonedInstance should be created in Annotation class inheritors.");
        }

        internal virtual void DoOnAppearancesImport()
        {
            // TODO: Do nothing
        }

        private void InitializeDefaultAnnotationFlags()
        {
            this.IsDisplayedWhenNotSupported = true;
            this.IsHidden = false;
            this.IsPrintable = false;
            this.IsZoomingWithPage = true;
            this.IsRotatedWithPage = true;
            this.IsVisibleInViewerUI = true;
            this.IsReadOnly = false;
            this.IsLockedByPositionAndSize = false;
            this.IsTogglingVisibilityInViewerUI = false;
            this.IsContentLocked = false;
        }

        Annotation IContextClonable<Annotation>.Clone(RadFixedDocumentCloneContext cloneContext)
        {
            Annotation clonedAnnotation = this.CreateClonedInstance(cloneContext);

            if (this.Appearances != null)
            {
                clonedAnnotation.Appearances = this.Appearances.Clone(cloneContext);
            }

            clonedAnnotation.Border = this.Border;
            clonedAnnotation.BoundingRect = this.BoundingRect;
            clonedAnnotation.Rect = this.Rect;

            return clonedAnnotation;
        }
    }
}
