﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Describes the settings which are used to determine how the border will appear in the document.
    /// </summary>
    public class AnnotationBorder
    {
        internal static AnnotationBorder DefaultAnnotationBorder
        {
            get
            {
                return new AnnotationBorder(0, AnnotationBorderStyle.None, null);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AnnotationBorder" /> class.
        /// </summary>
        public AnnotationBorder()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AnnotationBorder" /> class.
        /// </summary>
        /// <param name="width">The width.</param>
        /// <param name="style">The style.</param>
        /// <param name="dashArray">The dash array.</param>
        public AnnotationBorder(double width, AnnotationBorderStyle style, double[] dashArray)
        {
            this.Width = width;
            this.Style = style;
            this.DashArray = dashArray;
        }

        /// <summary>
        /// Gets or sets the width.
        /// </summary>
        /// <value>The width.</value>
        public double Width { get; set; }

        /// <summary>
        /// Gets or sets the style.
        /// </summary>
        /// <value>The style.</value>
        public AnnotationBorderStyle Style { get; set; }

        /// <summary>
        /// Gets or sets the dash array.
        /// </summary>
        /// <value>The dash array.</value>
        public IEnumerable<double> DashArray { get; set; }
    }
}
