﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents the <see cref="FormSource"/> instances used for displaying the widget content.
    /// </summary>
    public class AnnotationContentSource
    {
        /// <summary>
        /// Gets or sets the normal content source displayed when the mouse is not interacting with the widget.
        /// </summary>
        public FormSource NormalContentSource { get; set; }

        /// <summary>
        /// Gets or sets the content source displayed when the mouse button is pressed on the widget.
        /// </summary>
        public FormSource MouseDownContentSource { get; set; }

        /// <summary>
        /// Gets or sets the content source displayed when the mouse is over the widget.
        /// </summary>
        public FormSource MouseOverContentSource { get; set; }

        internal void Initialize(SingleStateAppearances singleStateAppearances)
        {
            this.NormalContentSource = singleStateAppearances.NormalAppearance;
            this.MouseDownContentSource = singleStateAppearances.MouseDownAppearance;
            this.MouseOverContentSource = singleStateAppearances.MouseOverAppearance;
        }

        internal void Initialize(AnnotationContentSource other)
        {
            this.NormalContentSource = other.NormalContentSource;
            this.MouseDownContentSource = other.MouseDownContentSource;
            this.MouseOverContentSource = other.MouseOverContentSource;
        }
    }
}
