﻿using System;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents the characteristics of Widget annotation dynamic appearance.
    /// </summary>
    public class DynamicAppearanceCharacteristics
    {
        /// <summary>
        /// Initializes a new instance of <see cref="DynamicAppearanceCharacteristics"/> class.
        /// </summary>
        public DynamicAppearanceCharacteristics()
        {
            this.Rotation = Rotation.Rotate0;
        }

        /// <summary>
        /// Initializes a new instance of <see cref="DynamicAppearanceCharacteristics"/> class by copying its properties from other instance.
        /// </summary>
        /// <param name="other">Other <see cref="DynamicAppearanceCharacteristics"/> instance.</param>
        public DynamicAppearanceCharacteristics(DynamicAppearanceCharacteristics other)
        {
            this.Rotation = other.Rotation;

            if(other.BorderColor != null)
            {
                this.BorderColor = new RgbColor(other.BorderColor);                
            }

            if(other.Background != null)
            {
                this.Background = new RgbColor(other.Background);
            }
        }

        /// <summary>
        /// Gets or sets the rotation of the Widget appearance content within the annotation rectangle.
        /// </summary>
        public Rotation Rotation { get; set; }

        /// <summary>
        /// Gets or sets the border color. If null the border will be fully transparent.
        /// If not null, then only the R, G and B values are accounted and the alpha value is ignored.
        /// </summary>
        public RgbColor BorderColor { get; set; }

        /// <summary>
        /// Gets or sets the annotation rectangle background color. If null the background will be fully transparent.
        /// If not null, then only the R, G and B values are accounted and the alpha value is ignored.
        /// </summary>
        public RgbColor Background { get; set; }
    }
}
