﻿using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents link annotation.
    /// </summary>
    public class Link : Annotation
    {
        internal Link()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Link" /> class.
        /// </summary>
        /// <param name="destination">The destination.</param>
        public Link(Destination destination)
        {
            Guard.ThrowExceptionIfNull(destination, "destination");

            this.Destination = destination;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Link" /> class.
        /// </summary>
        /// <param name="action">The action.</param>
        public Link(Action action)
        {
            Guard.ThrowExceptionIfNull(action, "action");

            this.Action = action;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Link" /> class.
        /// </summary>
        /// <param name="namedDestination">The named destination.</param>
        public Link(NamedDestination namedDestination)
        {
            Guard.ThrowExceptionIfNull(namedDestination, "namedDestination");
            this.NamedDestination = namedDestination;
        }

        /// <summary>
        /// Gets or sets the destination for the link action.
        /// </summary>
        /// <value>The destination.</value>
        public Destination Destination { get; internal set; }

        /// <summary>
        /// Gets or sets the action for the link annotation.
        /// </summary>
        /// <value>The action.</value>
        public Action Action { get; internal set; }

        /// <summary>
        /// Gets or sets the named destination for the link annotation.
        /// </summary>
        /// <value>The named destination.</value>
        public NamedDestination NamedDestination { get; internal set; }

        /// <summary>
        /// Gets the type of the annotation.
        /// </summary>
        /// <value>The type of the annotation.</value>
        public override AnnotationType Type
        {
            get
            {
                return AnnotationType.Link;
            }
        }

        internal override Annotation CreateClonedInstance(RadFixedDocumentCloneContext cloneContext)
        {
            if (this.Action != null)
            {
                return new Link(this.Action.Clone(cloneContext));
            }
            else if (this.Destination != null)
            {
                Destination clonedDestination = cloneContext.GetClonedDestination(this.Destination);
                Link clonedAnnotation = new Link(clonedDestination);

                return clonedAnnotation;
            }
            else
            {
                NamedDestination clonedNamedDestination = cloneContext.GetClonedNamedDestination(this.NamedDestination);
                Link clonedAnnotation = new Link(clonedNamedDestination);

                return clonedAnnotation;
            }
        }
    }
}
