﻿using System;
using System.Collections;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    internal class MultiStateAppearances : AnnotationAppearances, IEnumerable, IEnumerable<KeyValuePair<string, SingleStateAppearances>>
    {
        private readonly Dictionary<string, SingleStateAppearances> stateToAppearances;

        public MultiStateAppearances()
        {
            this.stateToAppearances = new Dictionary<string, SingleStateAppearances>();
        }

        public sealed override AnnotationAppearancesType AppearancesType
        {
            get
            {
                return AnnotationAppearancesType.MultiStateAppearances;
            }
        }

        internal string CurrentState { get; set; }

        public SingleStateAppearances this[string stateName]
        {
            get
            {
                return this.stateToAppearances[stateName];
            }
            set
            {
                Guard.ThrowExceptionIfNull(stateName, "stateName");
                Guard.ThrowExceptionIfNull(value, "value");

                this.stateToAppearances[stateName] = value;
            }
        }

        public void AddAppearance(string stateName, SingleStateAppearances stateAppearances)
        {
            Guard.ThrowExceptionIfNull(stateName, "stateName");
            Guard.ThrowExceptionIfNull(stateAppearances, "stateAppearances");

            this.stateToAppearances.Add(stateName, stateAppearances);
        } 

        public bool TryGetAppearance(string stateName, out SingleStateAppearances stateAppearances)
        {
            return this.stateToAppearances.TryGetValue(stateName, out stateAppearances);
        }

        public override AnnotationAppearances Clone(RadFixedDocumentCloneContext cloneContext)
        {
            MultiStateAppearances result = new MultiStateAppearances();

            foreach(KeyValuePair<string, SingleStateAppearances> stateAppearance in this)
            {
                result.AddAppearance(stateAppearance.Key, (SingleStateAppearances)stateAppearance.Value.Clone(cloneContext));
            }

            result.CurrentState = this.CurrentState;

            return result;
        }

        public IEnumerator<KeyValuePair<string, SingleStateAppearances>> GetEnumerator()
        {
            return this.stateToAppearances.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.stateToAppearances.GetEnumerator();
        }
    }
}
