﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents the characteristics of а PushButton Widget annotation dynamic appearance.
    /// </summary>
    public class PushButtonAppearanceCharacteristics : ButtonAppearanceCharacteristics
    {
        private readonly IconFitOptions iconFitOptions;

        /// <summary>
        /// Initializes a new instance of <see cref="PushButtonAppearanceCharacteristics"/> class.
        /// </summary>
        public PushButtonAppearanceCharacteristics()
        {
            this.iconFitOptions = new IconFitOptions();
            this.IconAndCaptionPosition = IconAndCaptionPosition.NoIconCaptionOnly;
        }

        /// <summary>
        /// Initializes a new instance of <see cref="PushButtonAppearanceCharacteristics"/> class by copying its properties from other instance.
        /// </summary>
        /// <param name="other">Other <see cref="PushButtonAppearanceCharacteristics"/> instance.</param>
        public PushButtonAppearanceCharacteristics(PushButtonAppearanceCharacteristics other)
            : base(other)
        {
            this.MouseOverCaption = other.MouseOverCaption;
            this.MouseDownCaption = other.MouseDownCaption;
            this.NormalIconSource = other.NormalIconSource;
            this.MouseOverIconSource = other.MouseOverIconSource;
            this.MouseDownIconSource = other.MouseDownIconSource;
            this.iconFitOptions = new IconFitOptions(other.IconFitOptions);
            this.IconAndCaptionPosition = other.IconAndCaptionPosition;
        }

        /// <summary>
        /// Gets or sets the button caption displayed when the mouse is over the widget.
        /// </summary>
        public string MouseOverCaption { get; set; }

        /// <summary>
        /// Gets or sets the button caption displayed when the mouse button is pressed on the widget.
        /// </summary>
        public string MouseDownCaption { get; set; }

        /// <summary>
        /// Gets or sets the button normal icon displayed when the mouse is not interacting with the widget.
        /// </summary>
        public FormSource NormalIconSource { get; set; }

        /// <summary>
        /// Gets or sets the button icon displayed when the mouse is over the widget.
        /// </summary>
        public FormSource MouseOverIconSource { get; set; }

        /// <summary>
        /// Gets or sets the button icon displayed when the mouse button is pressed on the widget.
        /// </summary>
        public FormSource MouseDownIconSource { get; set; }

        /// <summary>
        /// Gets the options specifying how fit the icon inside the annotation rectangle.
        /// </summary>
        public IconFitOptions IconFitOptions
        {
            get
            {
                return this.iconFitOptions;
            }
        }

        /// <summary>
        /// Gets or sets the relative positioning of icon and caption in the button appearance.
        /// </summary>
        public IconAndCaptionPosition IconAndCaptionPosition { get; set; }
    }
}
