﻿using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Model.Annotations
{
    /// <summary>
    /// Represents widget used by the signature annotation.
    /// </summary>
    public class SignatureWidget : Widget<DynamicAppearanceCharacteristics>, IContentAnnotation
    {
        private readonly AnnotationContentSource content;

        internal SignatureWidget(FormField field)
            : base(field)
        {
            this.content = new AnnotationContentSource();
        }

        /// <summary>
        /// Gets the widget content type.
        /// </summary>
        public sealed override WidgetContentType WidgetContentType
        {
            get
            {
                return WidgetContentType.SignatureContent;
            }
        }

        /// <summary>
        /// Gets the source defining the visual content of the widget.
        /// This content is with bigger priority compared to Widget appearance characteristics and text properties and it is visualized by default when opening the exported document in some PDF viewer.
        /// </summary>
        public AnnotationContentSource Content
        {
            get
            {
                this.EnsureContentIsUpToDate();
                return this.content;
            }
        }

        /// <summary>
        /// Recalculates the Widget <see cref="AnnotationContentSource"/> content based on Widget text properties and dynamic appearance characteristics.
        /// </summary>
        public sealed override void RecalculateContent()
        {
            this.Content.NormalContentSource = this.RecalculateAppearance(this.DrawSignatureContent);
            this.Content.MouseDownContentSource = null;
            this.Content.MouseOverContentSource = null;
        }

        internal override void ConsumeImportedAppearances(AnnotationAppearances appearances)
        {
            if (appearances.AppearancesType == AnnotationAppearancesType.SingleStateAppearances)
            {
                SingleStateAppearances singeStateAppearances = (SingleStateAppearances)appearances;
                this.Content.Initialize(singeStateAppearances);
            }
        }

        internal void PrepareAppearancesForExport()
        {
            this.Appearances = new SingleStateAppearances(this.Content);
        }

        internal sealed override Widget CreateClonedWidgetInstance(RadFixedDocumentCloneContext cloneContext)
        {
            FormField field = cloneContext.GetClonedField(this.Field);
            SignatureWidget clonedWidget = new SignatureWidget(field);
            clonedWidget.Content.Initialize(this.Content);
            clonedWidget.AppearanceCharacteristics = new DynamicAppearanceCharacteristics(this.AppearanceCharacteristics);

            return clonedWidget;
        }

        private void DrawSignatureContent(FixedContentEditor editor)
        {
            // Do nothing
        }
    }
}
