﻿using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Collections
{
    /// <summary>
    /// Represents annotations collection.
    /// </summary>
    public sealed class AnnotationCollection : DocumentElementCollection<Annotation, RadFixedPage>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AnnotationCollection" /> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        public AnnotationCollection(RadFixedPage parent)
            : base(parent)
        {
        }

        /// <summary>
        /// Adds the link.
        /// </summary>
        /// <param name="destination">The destination.</param>
        /// <returns></returns>
        public Link AddLink(Destination destination)
        {
            Link link = new Link(destination);

            return this.CreateLink(link);
        }

        /// <summary>
        /// Adds the link.
        /// </summary>
        /// <param name="action">The action.</param>
        /// <returns></returns>
        public Link AddLink(Actions.Action action)
        {
            var link = new Link(action);

            return this.CreateLink(link);
        }

        /// <summary>
        /// Adds the link.
        /// </summary>
        /// <param name="namedDestination">The named destination.</param>
        /// <returns></returns>
        public Link AddLink(NamedDestination namedDestination)
        {
            var link = new Link(namedDestination);

            return this.CreateLink(link);
        }

        private Link CreateLink(Link link)
        {
            Guard.ThrowExceptionIfNull(link, "link");

            this.Add(link);
            return link;
        }
    }
}
