﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Collections
{
    /// <summary>
    /// Represents content elements collection.
    /// </summary>
    public sealed class ContentElementCollection : DocumentElementCollection<ContentElementBase, IContainerElement>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ContentElementCollection" /> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        public ContentElementCollection(IContainerElement parent)
            : base(parent)
        {
        }

        /// <summary>
        /// Creates new Path and add it to the collection.
        /// </summary>
        /// <returns>The Path.</returns>
        public Path AddPath()
        {
            Path path = new Path();
            this.Add(path);

            return path;
        }

        /// <summary>
        /// Creates new TextFragment and add it to the collection.
        /// </summary>
        /// <returns>The TextFragment.</returns>
        public TextFragment AddTextFragment()
        {
            TextFragment textFragment = new TextFragment();
            this.Add(textFragment);

            return textFragment;
        }

        /// <summary>
        /// Creates new TextFragment and add it to the collection.
        /// </summary>
        /// <param name="text">The text in the TextFragment.</param>
        /// <returns></returns>
        public TextFragment AddTextFragment(string text)
        {
            TextFragment textFragment = this.AddTextFragment();
            textFragment.Text = text;

            return textFragment;
        }

        /// <summary>
        /// Adds the image.
        /// </summary>
        /// <returns></returns>
        public Image AddImage()
        {
            Image image = new Image();
            this.Add(image);

            return image;
        }

        /// <summary>
        /// Adds the image.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <returns></returns>
        public Image AddImage(ImageSource source)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            var image = this.AddImage();
            image.ImageSource = source;

            return image;
        }

        /// <summary>
        /// Adds the form.
        /// </summary>
        /// <returns></returns>
        public Form AddForm()
        {
            Form form = new Form();
            this.Add(form);

            return form;
        }

        /// <summary>
        /// Adds the form.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <returns></returns>
        public Form AddForm(FormSource source)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            Form form = this.AddForm();
            form.FormSource = source;

            return form;
        }
    }
}
