﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Collections
{
    /// <summary>
    /// Represents base DocumentElementCollection.
    /// </summary>
    /// <typeparam name="T">The type of the T.</typeparam>
    /// <typeparam name="TOwner">The type of the T owner.</typeparam>
    public class DocumentElementCollection<T, TOwner> : DocumentElementCollectionBase<T, TOwner>
        where T : FixedDocumentElementBase
        where TOwner : IFixedDocumentElement
    {
        /// <summary>
        /// Initializes a new instance of the DocumentElementCollection /> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        public DocumentElementCollection(TOwner parent) : base(parent)
        {
        }

        /// <summary>
        /// Sets the parent of the document element.
        /// </summary>
        /// <param name="item">The document element.</param>
        /// <param name="parent">The parent which should be set to the document element.</param>
        protected override void SetParent(T item, TOwner parent)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            item.SetParent(parent);
        }

        /// <summary>
        /// Verifies the validity of the document element before it is inserted in the collection. 
        /// </summary>
        /// <param name="item">The item.</param>
        protected override void VerifyDocumentElementOnInsert(T item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            //if (item.Document != this.Owner.Document)
            //{
            //    throw new ArgumentException("The document element is associated with another document.", "item");
            //}

            if (item.Parent != null && !item.Parent.Equals(this.Owner))
            {
                throw new ArgumentException("The document element is associated with another parent.", "item");
            }
        }
    }
}
