﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Graphics;

namespace Telerik.Windows.Documents.Fixed.Model.Collections
{
    /// <summary>
    /// Represents path segment collection.
    /// </summary>
    public sealed class PathSegmentCollection : List<PathSegment>
    {
        /// <summary>
        /// Adds the line segment.
        /// </summary>
        /// <returns></returns>
        public LineSegment AddLineSegment()
        {
            LineSegment segment = new LineSegment();
            this.Add(segment);

            return segment;
        }

        /// <summary>
        /// Adds the line segment.
        /// </summary>
        /// <param name="point">The point.</param>
        /// <returns></returns>
        public LineSegment AddLineSegment(Point point)
        {
            LineSegment segment = this.AddLineSegment();
            segment.Point = point;

            return segment;
        }

        /// <summary>
        /// Adds the quadratic bezier segment.
        /// </summary>
        /// <returns></returns>
        public QuadraticBezierSegment AddQuadraticBezierSegment()
        {
            QuadraticBezierSegment segment = new QuadraticBezierSegment();
            this.Add(segment);

            return segment;
        }

        /// <summary>
        /// Adds the quadratic bezier segment.
        /// </summary>
        /// <param name="point1">The first point.</param>
        /// <param name="point2">The second point.</param>
        /// <returns></returns>
        public QuadraticBezierSegment AddQuadraticBezierSegment(Point point1, Point point2)
        {
            QuadraticBezierSegment segment = this.AddQuadraticBezierSegment();

            segment.Point1 = point1;
            segment.Point2 = point2;

            return segment;
        }

        /// <summary>
        /// Adds the bezier segment.
        /// </summary>
        /// <returns></returns>
        public BezierSegment AddBezierSegment()
        {
            BezierSegment segment = new BezierSegment();
            this.Add(segment);

            return segment;
        }

        /// <summary>
        /// Adds the bezier segment.
        /// </summary>
        /// <param name="point1">The first point.</param>
        /// <param name="point2">The second point.</param>
        /// <param name="point3">The third point.</param>
        /// <returns></returns>
        public BezierSegment AddBezierSegment(Point point1, Point point2, Point point3)
        {
            BezierSegment segment = this.AddBezierSegment();

            segment.Point1 = point1;
            segment.Point2 = point2;
            segment.Point3 = point3;

            return segment;
        }

        /// <summary>
        /// Adds the arc segment.
        /// </summary>
        /// <returns></returns>
        public ArcSegment AddArcSegment()
        {
            ArcSegment segment = new ArcSegment();
            this.Add(segment);

            return segment;
        }

        /// <summary>
        /// Adds the arc segment.
        /// </summary>
        /// <param name="point">The point.</param>
        /// <param name="radiusX">The radius X.</param>
        /// <param name="radiusY">The radius Y.</param>
        /// <returns></returns>
        public ArcSegment AddArcSegment(Point point, double radiusX, double radiusY)
        {
            ArcSegment segment = this.AddArcSegment();
            segment.Point = point;
            segment.RadiusX = radiusX;
            segment.RadiusY = radiusY;

            return segment;
        }
    }
}
