﻿using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Imaging;
using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class CalGray : ColorSpaceBase
    {
        public override int ComponentCount
        {
            get { return 1; }
        }

        public CalGray()
        {
            this.WhitePoint = new double[3];
            this.BlackPoint = new double[3];
            this.Gamma = 1;
        }

        public override string Name
        {
            get { return ColorSpaceNames.CalGray; }
        }

        public double[] WhitePoint { get; set; }

        public double[] BlackPoint { get; set; }

        public double Gamma { get; set; }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return DeviceGray.GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return DeviceGray.GetGrayPixels(image, applyMask);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            double g = pars[0];
            Color color = Color.FromGray(g);

            return new RgbColor(color);
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            Color color = Color.FromGray(lookupData[offset]);
            return new RgbColor(color);
        }
    }
}