﻿using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Imaging;
using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class CalRgb : ColorSpaceBase
    {
        public override int ComponentCount
        {
            get { return 3; }
        }

        public CalRgb()
        {
            this.WhitePoint = new double[3];
            this.BlackPoint = new double[3];
            this.Gamma = new double[] { 1, 1, 1 };
            this.Matrix = new double[] { 1, 0, 0, 0, 1, 0, 0, 0, 1 };
        }

        public override string Name
        {
            get { return ColorSpaceNames.CalRgb; }
        }

        public double[] WhitePoint { get; set; }

        public double[] BlackPoint { get; set; }

        public double[] Gamma { get; set; }

        public double[] Matrix { get; set; }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return DeviceRgb.GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return DeviceRgb.GetRgbPixels(image, applyMask);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            int len = pars.Length;
            byte r = Color.ConvertColorComponentToByte(pars[len - 3]);
            byte g = Color.ConvertColorComponentToByte(pars[len - 2]);
            byte b = Color.ConvertColorComponentToByte(pars[len - 1]);

            RgbColor rgbColor = new RgbColor(r, g, b);

            return rgbColor;
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            Color color = Color.FromArgb(255, lookupData[offset], lookupData[offset + 1], lookupData[offset + 2]);
            return new RgbColor(color);
        }
    }
}