﻿using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Imaging;
using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class DeviceCmyk : ColorSpaceBase
    {
        public override int ComponentCount
        {
            get
            {
                return 4;
            }
        }

        public override string Name
        {
            get { return ColorSpaceNames.DeviceCmyk; }
        }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return GetCmykPixels(image, applyMask);
        }

        internal static PixelContainer GetCmykPixels(IImageDescriptor image, bool applyMask)
        {
            int bitsPerComponent = image.BitsPerComponent;
            byte[] data = image.GetDecodedData();
            ImageBitReader reader = new ImageBitReader(data, bitsPerComponent, image.Width);
            PixelContainer pixels = PixelContainerFactory.GetCmykImagePixelContainer(image.Width, image.Height);
            double[] decode = GetDecodeArray(image);            
            double minC = GetDecodeArrayComponent(decode, 0);
            double maxC = GetDecodeArrayComponent(decode, 1);
            double minM = GetDecodeArrayComponent(decode, 2);
            double maxM = GetDecodeArrayComponent(decode, 3);
            double minY = GetDecodeArrayComponent(decode, 4);
            double maxY = GetDecodeArrayComponent(decode, 5);
            double minK = GetDecodeArrayComponent(decode, 6);
            double maxK = GetDecodeArrayComponent(decode, 7);

            double minX = 0;
            double maxX = Math.Pow(2, bitsPerComponent) - 1;
            IMask mask = image.Mask;
            bool shouldMaskColorComponents = ShouldMaskColorComponents(mask, applyMask);
            Func<int[], Color> getPixel = (componentsArray) =>
            {
                return Color.FromCmyk(
                        FunctionBase.Interpolate(componentsArray[0], minX, maxX, minC, maxC),
                        FunctionBase.Interpolate(componentsArray[1], minX, maxX, minM, maxM),
                        FunctionBase.Interpolate(componentsArray[2], minX, maxX, minY, maxY),
                        FunctionBase.Interpolate(componentsArray[3], minX, maxX, minK, maxK));
            };

            int[] components = new int[4];

            for (int i = 0; i < image.Height; i++)
            {
                for (int j = 0; j < image.Width; j++)
                {
                    components[0] = reader.Read();
                    components[1] = reader.Read();
                    components[2] = reader.Read();
                    components[3] = reader.Read();

                    pixels.Add(GetPixelFromComponents(components, getPixel, mask, shouldMaskColorComponents));
                }
            }

            return GetPixelArray(image, pixels, applyMask);
        }

        internal static double[] GetDefaultDecodeArray()
        {
            return GetDecodeArrayFromComponents(4);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            int len = pars.Length;

            double c, m, y, k;
            c = pars[len - 4];
            m = pars[len - 3];
            y = pars[len - 2];
            k = pars[len - 1];

            Color color = Color.FromCmyk(c, m, y, k);
            return new RgbColor(color);
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            Color color = Color.FromCmyk(lookupData[offset], lookupData[offset + 1], lookupData[offset + 2], lookupData[offset + 3]);
            return new RgbColor(color);
        }
    }
}