﻿using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using System;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class DeviceGray : ColorSpaceBase
    {
        public override int ComponentCount
        {
            get { return 1; }
        }

        public override string Name
        {
            get { return ColorSpaceNames.DeviceGray; }
        }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return GetGrayPixels(image, applyMask);
        }

        internal static PixelContainer GetGrayPixels(IImageDescriptor image, bool applyMask)
        {
            int bitsPerComponent = image.BitsPerComponent;
            byte[] data = image.GetDecodedData();
            ImageBitReader reader = new ImageBitReader(data, bitsPerComponent, image.Width);
            bool hasMask = HasMask(image);
            PixelContainer pixels = PixelContainerFactory.GetGrayImagePixelContainer(bitsPerComponent, hasMask, image.Width, image.Height);

            double[] decode = GetDecodeArray(image);
            double yMin = GetDecodeArrayComponent(decode, 0);
            double yMax = GetDecodeArrayComponent(decode, 1);

            double xMin = 0;
            double xMax = Math.Pow(2, bitsPerComponent) - 1;

            IMask mask = image.Mask;
            bool shouldMaskColorComponents = ShouldMaskColorComponents(mask, applyMask);
            Func<int[], Color> getPixel = (componentsArray) =>
            {
                return Color.FromGray(FunctionBase.Interpolate(componentsArray[0], xMin, xMax, yMin, yMax));
            };

            int[] components = new int[1];

            for (int i = 0; i < image.Height; i++)
            {
                for (int j = 0; j < image.Width; j++)
                {
                    components[0] = reader.Read();

                    pixels.Add(GetPixelFromComponents(components, getPixel, mask, shouldMaskColorComponents));
                }
            }

            return GetPixelArray(image, pixels, applyMask);
        }

        internal static double[] GetDefaultDecodeArray()
        {
            return GetDecodeArrayFromComponents(1);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");
            double g = pars[0];
            Color color = Color.FromGray(g);

            return new RgbColor(color);
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            Color color = Color.FromGray(lookupData[offset]);
            return new RgbColor(color);
        }
    }
}
