﻿using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using System;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class DeviceRgb : ColorSpaceBase
    {
        public override int ComponentCount
        {
            get { return 3; }
        }

        public override string Name
        {
            get { return ColorSpaceNames.DeviceRgb; }
        }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return GetRgbPixels(image, applyMask);
        }

        internal static PixelContainer GetRgbPixels(IImageDescriptor image, bool applyMask)
        {
            int bitsPerComponent = image.BitsPerComponent;
            byte[] data = image.GetDecodedData();
            ImageBitReader reader = new ImageBitReader(data, bitsPerComponent, image.Width);

            PixelContainer pixels = PixelContainerFactory.GetRgbImagePixelContainer(image.Width, image.Height);
            double[] decode = GetDecodeArray(image);
            double minR = GetDecodeArrayComponent(decode, 0);
            double maxR = GetDecodeArrayComponent(decode, 1);
            double minG = GetDecodeArrayComponent(decode, 2);
            double maxG = GetDecodeArrayComponent(decode, 3);
            double minB = GetDecodeArrayComponent(decode, 4);
            double maxB = GetDecodeArrayComponent(decode, 5);

            double minX = 0;
            double maxX = Math.Pow(2, bitsPerComponent) - 1;
            IMask mask = image.Mask;
            bool shouldMaskColorComponents = ShouldMaskColorComponents(mask, applyMask);
            Func<int[], Color> getPixel = (componentsArray) =>
            {
                return Color.FromArgb(1.0,
                    FunctionBase.Interpolate(componentsArray[0], minX, maxX, minR, maxR),
                    FunctionBase.Interpolate(componentsArray[1], minX, maxX, minG, maxG),
                    FunctionBase.Interpolate(componentsArray[2], minX, maxX, minB, maxB));
            };

            int[] components = new int[3];

            for (int i = 0; i < image.Height; i++)
            {
                for (int j = 0; j < image.Width; j++)
                {
                    components[0] = reader.Read();
                    components[1] = reader.Read();
                    components[2] = reader.Read();

                    pixels.Add(GetPixelFromComponents(components, getPixel, mask, shouldMaskColorComponents));
                }
            }

            return GetPixelArray(image, pixels, applyMask);
        }

        internal static double[] GetDefaultDecodeArray()
        {
            return GetDecodeArrayFromComponents(3);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            byte r = Color.ConvertColorComponentToByte(pars[0]);
            byte g = Color.ConvertColorComponentToByte(pars[1]);
            byte b = Color.ConvertColorComponentToByte(pars[2]);

            RgbColor rgbColor = new RgbColor(r, g, b);

            return rgbColor;
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            Color color = Color.FromArgb(255, lookupData[offset], lookupData[offset + 1], lookupData[offset + 2]);
            return new RgbColor(color);
        }
    }
}
