﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents gradient stop.
    /// </summary>
    public class GradientStop : IGradientStop
    {
        private readonly SimpleColor color;
        private readonly double offset;

        /// <summary>
        /// Initializes a new instance of the <see cref="GradientStop" /> class.
        /// </summary>
        /// <param name="color">The color.</param>
        /// <param name="offset">The offset. This value can be between 0 and 1.</param>
        public GradientStop(SimpleColor color, double offset)
        {
            Guard.ThrowExceptionIfNull(color, "color");
            Guard.ThrowExceptionIfOutOfRange(0, 1, offset, "offset");

            this.color = color;
            this.offset = offset;
        }

        /// <summary>
        /// Gets the color.
        /// </summary>
        /// <value>The color.</value>
        public SimpleColor Color
        {
            get
            {
                return this.color;
            }
        }

        /// <summary>
        /// Gets the offset. The value can be between 0 and 1.
        /// </summary>
        /// <value>The offset.</value>
        public double Offset
        {
            get
            {
                return this.offset;
            }
        }

        Color IGradientStop.Color
        {
            get
            {
                return this.Color.ToColor();
            }
        }

        double IGradientStop.Offset
        {
            get
            {
                return this.Offset;
            }
        }
    }
}
