﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents gradient stop collection.
    /// </summary>
    public class GradientStopCollection : IList<GradientStop>
    {
        private readonly List<GradientStop> store;

        /// <summary>
        /// Initializes a new instance of the <see cref="GradientStopCollection" /> class.
        /// </summary>
        public GradientStopCollection()
        {
            this.store = new List<GradientStop>();
        }

        /// <summary>
        /// Gets the number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1" />.
        /// </summary>
        /// <returns>The number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1" />.
        /// </returns>
        /// <value></value>
        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the <see cref="T:System.Collections.Generic.ICollection`1" />
        /// is read-only.
        /// </summary>
        /// <returns>true if the <see cref="T:System.Collections.Generic.ICollection`1" />
        /// is read-only; otherwise, false.</returns>
        /// <value></value>
        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="Telerik.Windows.Documents.Fixed.Model.ColorSpaces.GradientStop" /> at the specified index.
        /// </summary>
        /// <value></value>
        public GradientStop this[int index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        /// <summary>
        /// Indexes the of.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns></returns>
        public int IndexOf(GradientStop item)
        {
            return this.store.IndexOf(item);
        }

        /// <summary>
        /// Inserts the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="item">The item.</param>
        public void Insert(int index, GradientStop item)
        {
            this.store.Insert(index, item);
        }

        /// <summary>
        /// Removes the <see cref="T:System.Collections.Generic.IList`1" /> item
        /// at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index of the item to remove.</param>
        /// <exception cref="T:System.ArgumentOutOfRangeException">
        /// <paramref name="index" /> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1" />.
        /// </exception>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1" />
        /// is read-only.</exception>
        public void RemoveAt(int index)
        {
            this.store.RemoveAt(index);
        }

        /// <summary>
        /// Adds the specified item.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Add(GradientStop item)
        {
            this.store.Add(item);
        }

        /// <summary>
        /// Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1" />.
        /// </summary>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1" />
        /// is read-only. </exception>
        public void Clear()
        {
            this.store.Clear();
        }

        /// <summary>
        /// Determines whether [contains] [the specified item].
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns></returns>
        public bool Contains(GradientStop item)
        {
            return this.store.Contains(item);
        }

        /// <summary>
        /// Copies to.
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="arrayIndex">Index of the array.</param>
        public void CopyTo(GradientStop[] array, int arrayIndex)
        {
            this.store.CopyTo(array, arrayIndex);
        }

        /// <summary>
        /// Removes the specified item.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns></returns>
        public bool Remove(GradientStop item)
        {
            return this.store.Remove(item);
        }

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1" /> that can
        /// be used to iterate through the collection.
        /// </returns>
        public IEnumerator<GradientStop> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator" /> object that can be
        /// used to iterate through the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }
    }
}
