﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class IccBased : ColorSpaceBase
    {
        private ColorSpaceBase cachedAlternateColorSpace;

        public override int ComponentCount
        {
            get
            {
                return this.N;
            }
        }

        public string[] Filters { get; set; }

        public override string Name
        {
            get { return ColorSpaceNames.IccBased; }
        }

        public byte[] Data { get; set; }

        public ColorSpaceBase Alternate { get; set; }

        public int N { get; set; }

        public double[] Range { get; set; }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            ColorSpaceBase alternate = this.GetCachedAlternateColorSpace();

            return alternate.GetDefaultDecodeArray(bitsPerComponent);
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            ColorSpaceBase alternate = this.GetCachedAlternateColorSpace();

            return alternate.GetPixels(image, applyMask);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            ColorSpaceBase alternate = this.GetCachedAlternateColorSpace();

            return alternate.GetColor(pars);
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            ColorSpaceBase alternate = this.GetCachedAlternateColorSpace();

            return alternate.GetColorFromLookup(lookupData, offset);
        }


        internal ColorSpaceBase GetCachedAlternateColorSpace()
        {
            if (this.cachedAlternateColorSpace == null)
            {
                if (this.Alternate != null)
                {
                    this.cachedAlternateColorSpace = this.Alternate;
                }
                else
                {
                    this.cachedAlternateColorSpace = this.CalculateAlternateColorSpaceFromN();
                }
            }

            return this.cachedAlternateColorSpace;
        }
        
        private ColorSpaceBase CalculateAlternateColorSpaceFromN()
        {
            switch (this.N)
            {
                case 1:
                    return new DeviceGray();
                case 3:
                    return new DeviceRgb();
                case 4:
                    return new DeviceCmyk();
                default:
                    throw new NotSupportedException();
            }
        }
    }
}