﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents base class for Pattern color
    /// </summary>
    public abstract class PatternColor : ColorBase, IInstanceIdOwner, IPatternColor
    {
        private readonly int id;

        /// <summary>
        /// Creates new instance of <see cref="PatternColor"/> class. 
        /// </summary>
        protected PatternColor()
        {
            this.id = InstanceIdGenerator.GetNextId();
        }

        /// <summary>
        /// Gets or sets the position.
        /// </summary>
        /// <value>The position.</value>
        public abstract IPosition Position { get; set; } 

        internal override ColorSpaceBase ColorSpace
        {
            get
            {
                return new Pattern();
            }
        }

        internal override bool IsTransparent
        {
            get
            {
                return false;
            }
        }

        internal abstract PatternType PatternType
        {
            get;
        }

        int IInstanceIdOwner.InstanceId
        {
            get
            {
                return this.id;
            }
        }

        Matrix IPatternColor.Position
        {
            get
            {
                return this.Position.Matrix;
            }
        }

        internal override Core.Imaging.Color ToColor()
        {
            throw new NotSupportedException("PatternColor cannot be converted to simple Color.");
        }
    }
}
