﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents radial gradient.
    /// </summary>
    public class RadialGradient : Gradient, IRadialGradient
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="RadialGradient" /> class.
        /// </summary>
        /// <param name="startPoint">The start point.</param>
        /// <param name="endPoint">The end point.</param>
        /// <param name="startRadius">The start radius.</param>
        /// <param name="endRadius">The end radius.</param>
        public RadialGradient(Point startPoint, Point endPoint, double startRadius, double endRadius) : base(startPoint, endPoint)
        {
            this.StartRadius = startRadius;
            this.EndRadius = endRadius;
        }

        /// <summary>
        /// Gets or sets the start radius.
        /// </summary>
        /// <value>The start radius.</value>
        public double StartRadius { get; set; }

        /// <summary>
        /// Gets or sets the end radius.
        /// </summary>
        /// <value>The end radius.</value>
        public double EndRadius { get; set; }

        internal override GradientType GradientType
        {
            get
            {
                return GradientType.Radial;
            }
        }

        /// <summary>
        /// Equalses the specified other.
        /// </summary>
        /// <param name="other">The other.</param>
        /// <returns></returns>
        public override bool Equals(ColorBase other)
        {
            RadialGradient gradient = other as RadialGradient;
            if (gradient == null)
            {
                return false;
            }

            return this.StartRadius == gradient.StartRadius &&
                this.EndRadius == gradient.EndRadius &&
                base.Equals(gradient);
        }
    }
}
