﻿using System;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    internal class Separation : ColorSpaceBase, IMulticomponentColorSpace
    {
        public override int ComponentCount
        {
            get { return 1; }
        }

        public override string Name
        {
            get
            {
                return ColorSpaceNames.Separation;
            }
        }

        internal string ColorantName { get; set; }

        internal ColorSpaceBase AlternateColorSpace { get;  set; }

        internal FunctionBase TintTransform { get;  set; }

        public override double[] GetDefaultDecodeArray(int bitsPerComponent)
        {
            return GetDefaultDecodeArray();
        }

        public override PixelContainer GetPixels(IImageDescriptor image, bool applyMask)
        {
            return GetSeparationPixels(image, this, applyMask);
        }

        internal static PixelContainer GetSeparationPixels(IImageDescriptor image, IMulticomponentColorSpace cs, bool applyMask)
        {
            int bitsPerComponent = image.BitsPerComponent;
            byte[] data = image.GetDecodedData();
            ImageBitReader reader = new ImageBitReader(data, bitsPerComponent, image.Width);
            PixelContainer pixels = PixelContainerFactory.GetSeparationImagePixelContainer(image.Width, image.Height);

            double[] decode = GetDecodeArray(image);
            double yMin = GetDecodeArrayComponent(decode, 0);
            double yMax = GetDecodeArrayComponent(decode, 1);
            double xMin = 0;
            double xMax = Math.Pow(2, bitsPerComponent) - 1;

            IMask mask = image.Mask;
            bool shouldMaskColorComponents = ShouldMaskColorComponents(mask, applyMask);
            Func<int[], Color> getPixel = (componentsArray) =>
            {
                double colorComponent = FunctionBase.Interpolate(componentsArray[0], xMin, xMax, yMin, yMax);

                return cs.GetColor(new double[] { colorComponent });
            };

            int[] components = new int[1];
            for (int i = 0; i < image.Height; i++)
            {
                for (int j = 0; j < image.Width; j++)
                {
                    components[0] = reader.Read();

                    pixels.Add(GetPixelFromComponents(components, getPixel, mask, shouldMaskColorComponents));
                }
            }

            return GetPixelArray(image, pixels, applyMask);
        }

        internal static double[] GetDefaultDecodeArray()
        {
            return GetDecodeArrayFromComponents(1);
        }

        internal override ColorBase GetColor(double[] pars)
        {
            Guard.ThrowExceptionIfNull(pars, "pars");

            // TODO: try caching ColorBase result for specific pars to speed up this method. 
            double[] alternatePars = this.TintTransform.Execute(pars);

            return this.AlternateColorSpace.GetColor(alternatePars);
        }

        /// <summary>
        /// Called from <see cref="Indexed"/> to get color from lookup array.
        /// </summary>
        internal override ColorBase GetColorFromLookup(byte[] lookupData, int offset)
        {
            Guard.ThrowExceptionIfNull(lookupData, "lookupData");

            return this.GetColor(new double[] { lookupData[offset] });
        }

        Color IMulticomponentColorSpace.GetColor(double[] components)
        {
            ColorBase color = this.GetColor(components);

            return color.ToColor();
        }
    }
}
