﻿using System;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents tiling.
    /// </summary>
    public class Tiling : TilingBase
    { 
        private readonly ContentElementCollection content;

        private Rect boundingBox;
        private double? verticalSpacing;
        private double? horizontalSpacing;

        /// <summary>
        /// Initializes a new instance of the <see cref="Tiling" /> class.
        /// </summary>
        public Tiling()
            : this(FixedDocumentDefaults.TilingBoundingBox, SimplePosition.Default)
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="Tiling" /> class.
        /// </summary>
        /// <param name="boundingBox">The bounding box.</param>
        public Tiling(Rect boundingBox) : this(boundingBox, SimplePosition.Default)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Tiling" /> class.
        /// </summary>
        /// <param name="position">The position.</param>
        public Tiling(IPosition position)
            : this(FixedDocumentDefaults.TilingBoundingBox, position)
        {
        }
         
        /// <summary>
        /// Initializes a new instance of the <see cref="Tiling" /> class.
        /// </summary>
        /// <param name="boundingBox">The bounding box.</param>
        /// <param name="position">The position.</param>
        public Tiling(Rect boundingBox, IPosition position)
        { 
            this.TilingType = Model.ColorSpaces.TilingType.NoDistortion;
            this.content = new ContentElementCollection(this);
            this.Position = position;
            this.BoundingBox = boundingBox; 
        } 

        /// <summary>
        /// Gets the size.
        /// </summary>
        /// <value>The size.</value>
        public override Size Size
        {
            get
            {
                return new Size(this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        /// <summary>
        /// Gets the content for the element.
        /// </summary>
        /// <value>The content.</value>
        public override Collections.ContentElementCollection Content
        {
            get { return this.content; }
        } 

        /// <summary>
        /// Gets or sets the vertical spacing.
        /// </summary>
        /// <value>The vertical spacing.</value>
        public override double VerticalSpacing
        {
            get
            {
                if (this.verticalSpacing.HasValue)
                {
                    return this.verticalSpacing.Value;
                }

                return this.Size.Height;
            }
            set
            {
                this.verticalSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the horizontal spacing.
        /// </summary>
        /// <value>The horizontal spacing.</value>
        public override double HorizontalSpacing
        {
            get
            {
                if (this.horizontalSpacing.HasValue)
                {
                    return this.horizontalSpacing.Value;
                }

                return this.Size.Width;
            }
            set
            {
                this.horizontalSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the bounding box.
        /// </summary>
        /// <value>The bounding box.</value>
        public override Rect BoundingBox
        {
            get
            {
                return this.boundingBox;
            }
            set
            {
                this.boundingBox = value; 
            }
        }

        /// <summary>
        /// Gets or sets the position.
        /// </summary>
        /// <value>The position.</value>
        public override IPosition Position { get; set; }

        /// <summary>
        /// Gets or sets the type of the tiling.
        /// </summary>
        /// <value>The type of the tiling.</value>
        public override TilingType TilingType { get; set; }

        /// <summary>
        /// Equalses the specified other.
        /// </summary>
        /// <param name="other">The other.</param>
        /// <returns></returns>
        public override bool Equals(ColorBase other)
        {
            //// TODO: Think if content should be compared.
            return this == other;
        }

        internal override PaintType PaintType
        {
            get
            {
                return PaintType.Colored;
            }
        }
    }
}
