﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents tiling base.
    /// </summary>
    public abstract class TilingBase : PatternColor, IContentRootElement, ITiling
    {
        /// <summary>
        /// Gets or sets the type of the tiling.
        /// </summary>
        /// <value>The type of the tiling.</value>
        public abstract TilingType TilingType { get; set; }

        /// <summary>
        /// Gets or sets the vertical spacing.
        /// </summary>
        /// <value>The vertical spacing.</value>
        public abstract double VerticalSpacing { get; set; }

        /// <summary>
        /// Gets or sets the bounding box.
        /// </summary>
        /// <value>The bounding box.</value>
        public abstract Rect BoundingBox { get; set; }

        /// <summary>
        /// Gets or sets the horizontal spacing.
        /// </summary>
        /// <value>The horizontal spacing.</value>
        public abstract double HorizontalSpacing { get; set; }

        /// <summary>
        /// Gets the size.
        /// </summary>
        /// <value>The size.</value>
        public abstract Size Size { get; }

        /// <summary>
        /// Gets the content for the element.
        /// </summary>
        /// <value>The content.</value>
        public abstract Collections.ContentElementCollection Content { get; }

        /// <summary>
        /// Gets the parent.
        /// </summary>
        /// <value>The parent.</value>
        public IFixedDocumentElement Parent
        {
            get { return null; }
        }

        /// <summary>
        /// Gets if the element supports annotations.
        /// </summary>
        /// <value>True if the element supports annotations.</value>
        public bool SupportsAnnotations
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the annotations collection.
        /// </summary>
        /// <value>The annotations collection.</value>
        public AnnotationCollection Annotations
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        internal override PatternType PatternType
        {
            get { return Model.ColorSpaces.PatternType.Tiling; }
        }

        internal abstract PaintType PaintType
        {
            get;
        }
    }
}
