﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.ColorSpaces
{
    /// <summary>
    /// Represents uncolored tiling.
    /// </summary>
    public class UncoloredTiling : TilingBase
    {
        private readonly Tiling tiling;
        private readonly SimpleColor color;

        /// <summary>
        /// Initializes a new instance of the <see cref="UncoloredTiling" /> class.
        /// </summary>
        /// <param name="tiling">The tiling.</param>
        /// <param name="color">The color.</param>
        public UncoloredTiling(Tiling tiling, SimpleColor color)
        {
            Guard.ThrowExceptionIfNull(tiling, "tiling");
            Guard.ThrowExceptionIfNull(color, "color");

            this.tiling = tiling;
            this.color = color;
        }

        /// <summary>
        /// Gets the tiling.
        /// </summary>
        /// <value>The tiling.</value>
        public Tiling Tiling
        {
            get
            {
                return this.tiling;
            }
        }

        /// <summary>
        /// Gets the color.
        /// </summary>
        /// <value>The color.</value>
        public SimpleColor Color
        {
            get
            {
                return this.color;
            }
        }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <value>The content.</value>
        public override Collections.ContentElementCollection Content
        {
            get { return this.Tiling.Content; }
        }

        /// <summary>
        /// Gets the size.
        /// </summary>
        /// <value>The size.</value>
        public override Size Size
        {
            get { return this.Tiling.Size; }
        }

        /// <summary>
        /// Gets or sets the bounding box.
        /// </summary>
        /// <value>The bounding box.</value>
        public override Rect BoundingBox
        {
            get
            {
                return this.Tiling.BoundingBox;
            }
            set
            {
                this.Tiling.BoundingBox = value;
            }
        }

        /// <summary>
        /// Gets or sets the position.
        /// </summary>
        /// <value>The position.</value>
        public override Data.IPosition Position
        {
            get
            {
                return this.Tiling.Position;
            }
            set
            {
                this.Tiling.Position = value;
            }
        }

        /// <summary>
        /// Gets or sets the horizontal spacing.
        /// </summary>
        /// <value>The horizontal spacing.</value>
        public override double HorizontalSpacing
        {
            get
            {
                return this.Tiling.HorizontalSpacing;
            }
            set
            {
                this.Tiling.HorizontalSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the vertical spacing.
        /// </summary>
        /// <value>The vertical spacing.</value>
        public override double VerticalSpacing
        {
            get
            {
                return this.Tiling.VerticalSpacing;
            }
            set
            {
                this.Tiling.VerticalSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the type of the tiling.
        /// </summary>
        /// <value>The type of the tiling.</value>
        public override TilingType TilingType
        {
            get
            {
                return this.Tiling.TilingType;
            }
            set
            {
                this.Tiling.TilingType = value;
            }
        }

        internal override PaintType PaintType
        {
            get
            {
                return Model.ColorSpaces.PaintType.Uncolored;
            }
        }

        /// <summary>
        /// Equalses the specified other.
        /// </summary>
        /// <param name="other">The other.</param>
        /// <returns></returns>
        public override bool Equals(ColorBase other)
        {
            var uncolored = other as UncoloredTiling;
            if (uncolored == null)
            {
                return false;
            }

            return this.Color == uncolored.Color
                && this.Tiling == uncolored.Tiling;
        }
    }
}
