﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Common
{
    /// <summary>
    /// Represent base class for all fixed document elements.
    /// </summary>
    public abstract class FixedDocumentElementBase : IFixedDocumentElement
    {
        private IFixedDocumentElement parent;

        internal FixedDocumentElementBase() 
        {
        }

        internal FixedDocumentElementBase(IFixedDocumentElement parent)
        {
            this.SetParent(parent);
        }

        /// <summary>
        /// Gets the parent.
        /// </summary>
        /// <value>The parent.</value>
        public IFixedDocumentElement Parent
        {
            get
            {
                return this.parent;
            }
        }

        /// <summary>
        /// Sets the parent.
        /// </summary>
        /// <param name="newParent">The new parent.</param>
        internal void SetParent(IFixedDocumentElement newParent)
        {
            if (this.Parent == newParent)
            {
                return;
            }

            this.parent = newParent;
        }

        /// <summary>
        /// Gets the fixed document element type.
        /// </summary>
        /// <value>The fixed document element type.</value>
        internal abstract FixedDocumentElementType ElementType { get; }
    }
}
