﻿using System;

namespace Telerik.Windows.Documents.Fixed.Model.Common.Functions
{
    internal class ExponentialInterpolationFunction : FunctionBase
    {
        private readonly double n;
        private readonly double[] c0Array;
        private readonly double[] c1Array;

        public ExponentialInterpolationFunction(double n, double[] c0, double[] c1, double[] domain, double[] range) 
            : base(domain, range)
        {
            this.n = n;
            this.c0Array = c0 ?? new double[] { 0 };
            this.c1Array = c1 ?? new double[] { 1 };
        }

        public double N
        {
            get
            {
                return this.n;
            }
        }

        public double[] C0
        {
            get
            {
                return this.c0Array;
            }
        }

        public double[] C1
        {
            get
            {
                return this.c1Array;
            }
        }

        public override FunctionType FunctionType
        {
            get
            {
                return FunctionType.ExponentialInterpolation;
            }
        }

        protected override bool IsRangeRequired
        {
            get
            {
                return false;
            }
        }

        private int SingleResultValuesCount
        {
            get
            {
                return this.C0.Length;
            }
        }

        protected override double[] ExecuteOverride(double[] clippedInputValues)
        {
            double[] returnValues = new double[clippedInputValues.Length * this.SingleResultValuesCount];

            for (int i = 0; i < clippedInputValues.Length; i++)
            {
                double[] result = this.ExecuteSingle(clippedInputValues[i]);

                for (int j = 0; j < result.Length; j++)
                {
                    returnValues[(i * this.SingleResultValuesCount) + j] = result[j];
                }
            }

            return returnValues;
        }

        private double[] ExecuteSingle(double x)
        {
            double[] result = new double[this.SingleResultValuesCount];

            for (int i = 0; i < result.Length; i++)
            {
                double c0 = this.C0[i];
                double c1 = this.C1[i];
                double exponentialInterpolation = c0 + (Math.Pow(x, this.N) * (c1 - c0));
                result[i] = exponentialInterpolation;
            }

            return result;
        }
    }
}
