﻿using System;
using Telerik.Windows.Documents.Core.PostScript;
using Telerik.Windows.Documents.Core.PostScript.Data;

namespace Telerik.Windows.Documents.Fixed.Model.Common.Functions
{
    internal class PostScriptCalculatorFunction : FunctionBase
    {
        private readonly byte[] data;
        private readonly PostScriptArray procedure;

        public PostScriptCalculatorFunction(byte[] data, double[] domain, double[] range) 
            : base(domain, range)
        {
            this.data = data ?? new byte[0];

            Interpreter interpreter = new Interpreter();
            interpreter.Execute(data);
            this.procedure = interpreter.Operands.GetLastAs<PostScriptArray>();
        }

        public byte[] Data
        {
            get
            {
                return this.data;
            }
        }

        public override FunctionType FunctionType
        {
            get
            {
                return FunctionType.PostScriptCalculator;
            }
        }

        protected override bool IsRangeRequired
        {
            get
            {
                return true;
            }
        }

        public override byte[] GetFunctionData()
        {
            return this.Data;
        }

        protected override double[] ExecuteOverride(double[] clippedInputValues)
        {
            Interpreter interpreter = new Interpreter();
            interpreter.Operands = new OperandsCollection();
            foreach (double d in clippedInputValues)
            {
                interpreter.Operands.AddLast(d);
            }

            interpreter.ExecuteProcedure(this.procedure);

            int operands = this.Range.Length / 2;
            double[] result = new double[operands];
            for (int i = 0; i < operands; i++)
            {
                result[i] = interpreter.Operands.GetFirstAsReal();
            }

            return result;
        }
    }
}
