﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Common.Functions
{
    internal class StitchingFunction : FunctionBase
    {
        private readonly FunctionBase[] functions;
        private readonly double[] bounds;
        private readonly double[] encode;

        public StitchingFunction(FunctionBase[] functions, double[] bounds, double[] encode, double[] domain, double[] range) 
            : base(domain, range)
        {
            Guard.ThrowExceptionIfNull(functions, "functions");
            Guard.ThrowExceptionIfNull(bounds, "bounds");
            Guard.ThrowExceptionIfNull(encode, "encode");

            this.functions = functions;
            this.bounds = bounds;
            this.encode = encode;
        }

        public override FunctionType FunctionType
        {
            get
            {
                return FunctionType.Stitching;
            }
        }

        public double[] Encode
        {
            get
            {
                return this.encode;
            }
        }

        public double[] Bounds
        {
            get
            {
                return this.bounds;
            }
        }

        public FunctionBase[] Functions
        {
            get
            {
                return this.functions;
            }
        }

        protected override bool IsRangeRequired
        {
            get
            {
                return false;
            }
        }

        protected override double[] ExecuteOverride(double[] clippedInputValues)
        {
            double x = clippedInputValues[0];

            int index;
            FunctionBase function = this.FindFunction(x, out index);

            double minB;
            if (index > 0)
            {
                minB = this.Bounds[index - 1];
            }
            else
            {
                minB = this.Domain[0];
            }

            double maxB;
            if (index < this.Functions.Length - 1)
            {
                maxB = this.Bounds[index];
            }
            else
            {
                maxB = this.Domain[1];
            }

            double minE = this.Encode[2 * index];
            double maxE = this.Encode[(2 * index) + 1];
            double[] interpolatedInput = new double[] { Interpolate(x, minB, maxB, minE, maxE) };
            double[] result = function.Execute(interpolatedInput);

            return result;
        }

        private FunctionBase FindFunction(double x, out int index)
        {
            FunctionBase function;
            for (int i = 0; i < this.Bounds.Length; i++)
            {
                double bound = this.Bounds[i];

                if (x < bound)
                {
                    index = i;
                    function = this.Functions[i];

                    return function;
                }
            }

            index = this.Functions.Length - 1;
            function = this.Functions[index];

            return function;
        }
    }
}
