﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Common
{
    internal class PdfProperty<T>
    {
        private T value;
        private bool hasValue;
        private Func<T> getDefaultValue;

        public PdfProperty()
           : this(() => default(T))
        {
        }

        /// <summary>
        /// Initialize property with default value getter.
        /// </summary>
        /// <param name="getDefaultValue">Default value getter allowing to get the default value on demand only.</param>
        public PdfProperty(Func<T> getDefaultValue)
        {
            this.hasValue = false;
            this.getDefaultValue = getDefaultValue;
        }

        public T Value
        {
            get
            {
                if (this.hasValue)
                {
                    return this.value;
                }
                else
                {
                    return this.getDefaultValue();
                }
            }
            set
            {
                this.hasValue = true;
                this.value = value;
            }
        }

        public bool HasValue
        {
            get
            {
                return this.hasValue;
            }
        }

        public void SetDefaultValueFunc(Func<T> getDefautValue)
        {
            Guard.ThrowExceptionIfNull(getDefautValue, "getDefautValue");
            this.getDefaultValue = getDefautValue;
        }

        public void ClearValue()
        {
            this.hasValue = false;
            this.value = default(T);
        }

        public override string ToString()
        {
            if (this.HasValue)
            {
                return string.Format("{0}", this.Value);
            }
            else
            {
                return "<Empty property>";
            }
        }

        public override bool Equals(object obj)
        {
            PdfProperty<T> other = obj as PdfProperty<T>;

            if (other == null)
            {
                return false;
            }

            if (this.Value == null)
            {
                return other.Value == null;
            }

            return this.Value.Equals(other.Value);
        }

        public override int GetHashCode()
        {
            if (this.Value != null)
            {
                return this.Value.GetHashCode();
            }

            return 0;
        }
    }
}
