﻿using System;
using System.Linq;
#if !NETSTANDARD
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Common
{
    /// <summary>
    /// Represents a positionalbe content element.
    /// </summary>
    public abstract class PositionContentElement : ContentElementBase
    {
        private IPosition position;

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentElementBase" /> class.
        /// </summary>
        public PositionContentElement()
        {
            this.Position = new MatrixPosition();
        }
         
        /// <summary>
        /// Gets or sets the other of the element.
        /// </summary>
        /// <value>The position.</value>
        public IPosition Position
        {
            get
            {
                return this.position;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "Position");

                this.position = value;
            }
        }

        // Consider handling nested markers (nested BMC and EMC content stream operators)
        internal Marker Marker { get; set; }

        internal PositionContentElement Clone(RadFixedDocumentCloneContext cloneContext)
        {
            PositionContentElement clonedElement = this.CreateClonedInstance();

            clonedElement.Position = this.Position.Clone();
            clonedElement.Clipping = cloneContext.GetClonedClipping(this.Clipping);

            return clonedElement;
        }

        internal virtual PositionContentElement CreateClonedInstance()
        {
            // TODO: Should be implemented in inheritors!
            throw new NotImplementedException();
        }
    }
}
