﻿using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Data
{
    internal class CharCode : IComparable<CharCode>
    {
        private int size;
        private int code;
        private byte[] bytes = null;

        public CharCode(byte code)
            : this(code, 1)
        {
        }

        public CharCode(ushort code)
            : this(code, 2)
        {
        }

        public CharCode(int code, int size)
        {
            this.code = code;
            this.size = size;
        }

        public CharCode(byte[] code)
        {
            this.code = BytesHelper.GetInt(code);
            this.size = code.Length;
        }

        public int Code
        {
            get
            {
                return this.code;
            }
            set
            {
                this.code = value;
                this.bytes = null;
            }
        }

        public int Size
        {
            get
            {
                return this.size;
            }
            set
            {
                this.size = value;
                this.bytes = null;
            }
        }

        public byte[] ToBytes()
        {
            if (this.bytes == null)
            {
                this.bytes = new byte[this.Size];
                BytesHelper.GetBytesReverse(this.Code, this.bytes);
            }

            return this.bytes;
        }

        public CharCode Clone()
        {
            CharCode charCode = new CharCode(this.code, this.size)
            {
                bytes = this.bytes
            };

            return charCode;
        }

        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.Code, this.Size);
        }

        public override bool Equals(object obj)
        {
            CharCode other = obj as CharCode;
            if (other != null)
            {
                return this.Code == other.Code && this.Size == other.Size;
            }

            return false;
        }

        public override string ToString()
        {
            return BytesHelper.ToHexString(this.ToBytes());
        }

        public int CompareTo(CharCode other)
        {
            int result;

            if (this.Code > other.Code)
            {
                result = 1;
            }
            else if (this.Code == other.Code)
            {
                result = 0;
            }
            else
            {
                result = -1;
            }

            return result;
        }
    }
}
