﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Data
{
    /// <summary>
    /// Represents IPosition.
    /// </summary>
    public interface IPosition
    {
        /// <summary>
        /// Gets the matrix.
        /// </summary>
        /// <value>The matrix.</value>
        Matrix Matrix { get; }

        /// <summary>
        /// Appends the specified scale vector.
        /// </summary>
        /// <param name="scaleX">The scale X.</param>
        /// <param name="scaleY">The scale Y.</param>
        void Scale(double scaleX, double scaleY);

        /// <summary>
        /// Applies a scale by the specified amount about the specified point.
        /// </summary>
        /// <param name="scaleX">The scale X.</param>
        /// <param name="scaleY">The scale Y.</param>
        /// <param name="centerX">The center X.</param>
        /// <param name="centerY">The center Y.</param>
        void ScaleAt(double scaleX, double scaleY, double centerX, double centerY);

        /// <summary>
        /// Applies a rotation of the specified angle about the origin.
        /// </summary>
        /// <param name="angle">The angle.</param>
        void Rotate(double angle);

        /// <summary>
        /// Rotates this matrix about the specified point.
        /// </summary>
        /// <param name="angle">The angle.</param>
        /// <param name="centerX">The center X.</param>
        /// <param name="centerY">The center Y.</param>
        void RotateAt(double angle, double centerX, double centerY);

        /// <summary>
        /// Appends a translation of the specified offsets.
        /// </summary>
        /// <param name="offsetX">The offset X.</param>
        /// <param name="offsetY">The offset Y.</param>
        void Translate(double offsetX, double offsetY);

        /// <summary>
        /// Clears this instance.
        /// </summary>
        void Clear();

        /// <summary>
        /// Clones this instance.
        /// </summary>
        /// <returns></returns>
        IPosition Clone();
    }
}
