﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Data
{
    /// <summary>
    /// Represents position.
    /// </summary>
    public class MatrixPosition : IPosition
    {  
        /// <summary>
        /// Initializes a new instance of the <see cref="MatrixPosition" /> class.
        /// </summary>
        public MatrixPosition()
        {
            this.Matrix = new Matrix();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MatrixPosition" /> class.
        /// </summary>
        /// <param name="other">The other.</param>
        public MatrixPosition(IPosition other)
        {
            this.Matrix = other.Matrix;
        } 

        /// <summary>
        /// Initializes a new instance of the <see cref="MatrixPosition" /> class.
        /// </summary>
        /// <param name="matrix">The matrix.</param>
        public MatrixPosition(Matrix matrix)
        {
            this.Matrix = matrix;
        }

        /// <summary>
        /// Gets the default.
        /// </summary>
        /// <value>The default.</value>
        public static MatrixPosition Default
        {
            get
            {
                return new MatrixPosition();
            }
        }

        /// <summary>
        /// Gets or sets the matrix.
        /// </summary>
        /// <value>The matrix.</value>
        public Matrix Matrix { get; private set; }

        /// <summary>
        /// Appends the specified scale vector.
        /// </summary>
        /// <param name="scaleX">The scale X.</param>
        /// <param name="scaleY">The scale Y.</param>
        public void Scale(double scaleX, double scaleY)
        {
            this.Matrix = this.Matrix.ScaleMatrix(scaleX, scaleY);
        }
         
        /// <summary>
        /// Applies a scale by the specified amount about the specified point.
        /// </summary>
        /// <param name="scaleX">The scale X.</param>
        /// <param name="scaleY">The scale Y.</param>
        /// <param name="centerX">The center X.</param>
        /// <param name="centerY">The center Y.</param>
        public void ScaleAt(double scaleX, double scaleY, double centerX, double centerY)
        { 
            this.Matrix = this.Matrix.ScaleMatrixAt(scaleX, scaleY, centerX, centerY); 
        }

        /// <summary>
        /// Applies a rotation of the specified angle about the origin.
        /// </summary>
        /// <param name="angle">The angle.</param>
        public void Rotate(double angle)
        { 
            this.Matrix = this.Matrix.RotateMatrix(angle);
        }

        /// <summary>
        /// Rotates this matrix about the specified point.
        /// </summary>
        /// <param name="angle">The angle.</param>
        /// <param name="centerX">The center X.</param>
        /// <param name="centerY">The center Y.</param>
        public void RotateAt(double angle, double centerX, double centerY)
        {
            this.Matrix = this.Matrix.RotateMatrixAt(angle, centerX, centerY);
        }

        /// <summary>
        /// Appends a translation of the specified offsets.
        /// </summary>
        /// <param name="offsetX">The offset X.</param>
        /// <param name="offsetY">The offset Y.</param>
        public void Translate(double offsetX, double offsetY)
        { 
            this.Matrix = this.Matrix.TranslateMatrix(offsetX, offsetY);
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            this.Matrix = Matrix.Identity;
        }


        /// <summary>
        /// Clones this instance.
        /// </summary>
        /// <returns></returns>
        public IPosition Clone()
        {
            return new MatrixPosition(this.Matrix);
        }
    }
}
