﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Data
{
    internal class PositionStack
    {
        private readonly Stack<IPosition> positionsStack;
        private IPosition position;

        public PositionStack(IPosition initialPosition)
        {
            Guard.ThrowExceptionIfNull(initialPosition, "initialPosition");

            this.positionsStack = new Stack<IPosition>();
            this.Position = initialPosition.Clone();
        }

        /// <summary>
        /// Gets or sets the current position.
        /// </summary>
        /// <value>The position.</value>
        public IPosition Position
        {
            get
            {
                return this.position;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "Position");

                this.position = value;
            }
        }

        /// <summary>
        /// Saves the position.
        /// </summary>
        /// <returns></returns>
        public IDisposable SavePosition()
        {
            this.positionsStack.Push(this.Position.Clone());

            return new DisposableObject(this.RestorePosition);
        }

        /// <summary>
        /// Restores the position.
        /// </summary>
        public void RestorePosition()
        {
            this.Position = this.positionsStack.Pop();
        }
    }
}
