﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    /// Defines a class used for composing a byte range from a specified multiple <see cref="SourcePart"/> instances of a <see cref="ISourceStream"/>.
    /// </summary>
    internal class ByteRangeComposer
    {
        private ISourceStream source;
        private IEnumerable<SourcePart> sourceParts;
        private int? length;

        public ByteRangeComposer(ISourceStream source, IEnumerable<SourcePart> sourcePart)
        {
            this.source = source;
            this.sourceParts = sourcePart;
        }

        /// <summary>
        /// Gets the calculated length from all source parts.
        /// </summary>
        /// <value>The length.</value>
        public int Length
        {
            get
            {
                if (!this.length.HasValue)
                {
                    this.length = 0;

                    foreach (var sourceAccessPoint in this.sourceParts)
                    {
                        this.length += sourceAccessPoint.Length;
                    }
                }

                return this.length.Value;
            }
        }

        /// <summary>
        /// Composes the byte range.
        /// </summary>
        public byte[] Compose()
        {
            byte[] outputBuffer = new byte[this.Length];
            this.Read(outputBuffer);

            return outputBuffer;
        }

        private void Read(byte[] outputBuffer)
        {
            this.Read(0, outputBuffer);
        }

        private void Read(int outputBufferPosition, byte[] outputBuffer)
        {
            foreach (var sourcePart in this.sourceParts)
            {
                int countOfBytesRead = this.source.Read(outputBuffer, outputBufferPosition, sourcePart.Offset, sourcePart.Length);
                if (countOfBytesRead == -1)
                {
                    break;
                }

                outputBufferPosition += countOfBytesRead;
            }
        }
    }
}
