﻿using System;
using System.IO;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    /// Defines a class used for reading a document source in a <see cref="Stream" /> format.
    /// </summary>
    internal class FileSourceStream : ISourceStream
    {
        private Stream stream;

        /// <summary>
        /// Initializes a new instance of the <see cref="FileSourceStream" /> class.
        /// </summary>
        /// <param name="stream">The stream.</param>
        public FileSourceStream(Stream stream)
        {
            this.stream = stream;
        }

        public bool CanRead
        {
            get
            {
                return this.stream.CanRead;
            }
        }

        public bool CanSeek
        {
            get
            {
                return this.stream.CanSeek;
            }
        }

        /// <summary>
        ///  Reads a reads a sequence of bytes from the current stream and advances the position within the stream by the number of bytes read.
        /// </summary>
        /// <param name="outputBuffer">
        /// An array of bytes. When this method returns, the buffer contains the specified
        /// byte array with the values between offset and (streamOffset + length - 1) replaced by
        /// the bytes read from the current source.
        /// </param>
        /// <param name="outputBufferPosition">The zero-based byte offset in outputBuffer at which to begin storing the data read from the current stream.</param>
        /// <param name="streamOffset">The zero-based byte offset in buffer at which to begin read from the current stream.</param>
        /// <param name="length">The maximum number of bytes to be read from the current stream.</param>
        /// <returns>The total number of bytes read into the buffer.</returns>
        public int Read(byte[] outputBuffer, int outputBufferPosition, long streamOffset, int length)
        {
            long oldPosition = stream.Position;
            if (stream.Position != streamOffset)
            {
                stream.Seek(streamOffset, SeekOrigin.Begin);
            }

            int count = stream.Read(outputBuffer, outputBufferPosition, length);

            // returns the position to the original place.
            if (stream.Position != oldPosition)
            {
                stream.Seek(oldPosition, SeekOrigin.Begin);
            }

            return count == 0 ? -1 : count;
        }
    }
}
