﻿using System;
using System.Linq;
using System.Security.Cryptography.Pkcs;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    /// Defines a class used for handling the validation of digital signature in a PKCS#7 cryptographic message syntax.
    /// </summary>
    public class Pkcs7 : SignatureValidationHandlerBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Pkcs7" /> class.
        /// </summary>
        public Pkcs7()
        {
        }

        /// <summary>
        /// When overridden in a derived class, validates the digital signature.
        /// </summary>
        /// <param name="dataProperties"></param>
        /// <param name="validationProperties"></param>
        /// <returns>
        /// An instance of the <see cref="SignatureValidationResult" /> class describing the result from the validation.
        /// </returns>
        protected override SignatureValidationResult ValidateOverride(SignatureDataProperties dataProperties, SignatureValidationProperties validationProperties)
        {
            SignatureValidationResultBuilder validationResultBuilder = new SignatureValidationResultBuilder();

            SignedCms signedCms = new SignedCms();
            signedCms.Decode(dataProperties.Contents);

            this.ValidateCertificates(validationResultBuilder, signedCms.Certificates);
            validationResultBuilder.BuildCertificates(signedCms.Certificates);

            if (signedCms.SignerInfos.Count == 1)
            {
                SignerInfo signerInfo = signedCms.SignerInfos[0];

                this.SetResultHashAlgorithm(validationResultBuilder, signerInfo.DigestAlgorithm);

                bool isHashValid = this.ValidateHash(signedCms.ContentInfo.Content, signerInfo.DigestAlgorithm);
                validationResultBuilder.BuildIsDocumentModified(!isHashValid);
            }
            else
            {
                throw new NotSupportedException("There is more then one signer specified for a single signature.");
            }

            validationResultBuilder.BuildSignerInformation(dataProperties.Name);

            SignatureValidationResult validationResult = validationResultBuilder.GetResult();
            return validationResult;
        }
    }
}
