﻿using System;
using System.Collections.Generic;
using System.Security.Cryptography;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    /// Defines a predefined Oids used in the digital signature.
    /// </summary>
    internal static class SignatureOids
    {
        internal static Oid SHA1 = new Oid("1.3.14.3.2.26", "SHA1");
        internal static Oid SHA256 = new Oid("2.16.840.1.101.3.4.2.1", "SHA256");
        internal static Oid SHA512 = new Oid("2.16.840.1.101.3.4.2.3", "SHA512");

        internal static Oid SHA384 = new Oid("2.16.840.1.101.3.4.2.2", "SHA384");

        internal static Oid MD5 = new Oid("1.2.840.113549.2.5", "MD5");
        internal static Oid MD2 = new Oid("1.2.840.113549.2.2", "MD2");

        internal static Oid MD2RSA = new Oid("1.2.840.113549.1.1.2", "MD2RSA");
        internal static Oid MD5RSA = new Oid("1.2.840.113549.1.1.4", "MD5RSA");

        internal static Oid SHA1RSA = new Oid("1.2.840.113549.1.1.5", "SHA1RSA");
        internal static Oid SHA256RSA = new Oid("1.2.840.113549.1.1.11", "SHA256RSA");
        internal static Oid SHA384RSA = new Oid("1.2.840.113549.1.1.12", "SHA384RSA");
        internal static Oid SHA512RSA = new Oid("1.2.840.113549.1.1.13", "SHA512RSA");

        internal static Oid SHA1DSA = new Oid("1.2.840.10040.4.3", "SHA1DSA");

        internal static Oid RIPEMD160 = new Oid("1.3.36.3.2.1", "RIPEMD160");


        internal static Oid PKCS7_DATA = new Oid("1.2.840.113549.1.7.1");
        internal static Oid PKCS7_SIGNED_DATA = new Oid("1.2.840.113549.1.7.2");

        internal static Oid CONTENT_TYPE = new Oid("1.2.840.113549.1.9.3", "Content Type");
        internal static Oid MESSAGE_DIGEST = new Oid("1.2.840.113549.1.9.4", "Message Digest");
        internal static Oid SIGNING_TIME = new Oid("1.2.840.113549.1.9.5", "Signing Time");

        private static Dictionary<string, Oid> algorithmNames;

        static SignatureOids()
        {
            algorithmNames = new Dictionary<string, Oid>();

            algorithmNames[SHA1RSA.Value] = SHA1;
            algorithmNames[SHA256RSA.Value] = SHA256;
            algorithmNames[SHA384RSA.Value] = SHA384;
            algorithmNames[SHA512RSA.Value] = SHA512;

            algorithmNames[SHA1DSA.Value] = SHA1;

            algorithmNames["1.2.840.113549.2.5"] = MD5;
            algorithmNames["1.2.840.113549.2.2"] = MD2;

            algorithmNames["2.16.840.1.101.3.4.3.2"] = SHA256;
            algorithmNames["2.16.840.1.101.3.4.3.3"] = SHA384;
            algorithmNames["2.16.840.1.101.3.4.3.4"] = SHA512;

            algorithmNames["1.3.36.3.3.1.2"] = RIPEMD160;

            //algorithmNames["1.3.36.3.2.2"] = "RIPEMD128";
            //algorithmNames["1.3.36.3.2.3"] = "RIPEMD256";
            //algorithmNames["1.3.36.3.3.1.3"] = "RIPEMD128";
            //algorithmNames["1.3.36.3.3.1.4"] = "RIPEMD256";
        }

        /// <summary>
        /// Gets the main algorithm oid.
        /// </summary>
        /// <param name="oid">The oid.</param>
        /// <returns></returns>
        internal static Oid GetMainAlgorithmOid(Oid oid)
        {
            Oid mainAlgorithmOid;
            if (algorithmNames.TryGetValue(oid.Value, out mainAlgorithmOid))
            {

                return mainAlgorithmOid;
            }
            else
            {
                return oid;
            }
        }
    }
}
