﻿using System;
using System.Linq;
using System.Security.Cryptography.X509Certificates;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    ///  Defines a set of properties used to validate the certificates of the digital signature.
    /// </summary>
    public class SignatureValidationProperties
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SignatureValidationProperties" /> class.
        /// </summary>
        public SignatureValidationProperties()
        {
            this.Chain = new X509Chain();
            this.Chain.ChainPolicy.RevocationMode = X509RevocationMode.Online;

            this.ChainStatusFlags =
               X509ChainStatusFlags.CtlNotSignatureValid |
               X509ChainStatusFlags.Cyclic |
               X509ChainStatusFlags.InvalidBasicConstraints |
               X509ChainStatusFlags.NotSignatureValid |
               X509ChainStatusFlags.PartialChain |
               X509ChainStatusFlags.Revoked |
               X509ChainStatusFlags.UntrustedRoot;
        }

        /// <summary>
        /// Gets or sets the chain used to validate the certificate that signed the digital signature.
        /// </summary>
        /// <value>The chain.</value>
        public X509Chain Chain { get; set; }

        /// <summary>
        /// Gets or sets the chain status flags which describes the used signature certificate as invalid.
        /// </summary>
        /// <value>The chain status flags.</value>
        public X509ChainStatusFlags ChainStatusFlags { get; set; }
    }
}
