﻿using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace Telerik.Windows.Documents.Fixed.Model.DigitalSignatures
{
    /// <summary>
    /// Defines a set of properties describing the result from the digital signatire validation.
    /// </summary>
    public class SignatureValidationResult
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SignatureValidationResult" /> class.
        /// </summary>
        public SignatureValidationResult()
        {

        }

        /// <summary>
        /// Gets or sets the name of the signature form field associated with this validation result.
        /// </summary>
        /// <value>The name of the signature form field.</value>
        public string FieldName { get; internal set; }

        /// <summary>
        /// Gets or sets a value indicating whether the document was modified after it was signed.
        /// </summary>
        /// <value>
        /// <c>true</c> if the document was modified; otherwise, <c>false</c>.
        /// </value>
        public bool IsDocumentModified { get; internal set; }

        /// <summary>
        /// Gets or sets a value indicating whether the used certificate is valid. For additional details when the certificate is invalid use the CertificatesChainElements property.
        /// </summary>
        /// <value>
        /// <c>true</c> if the used certificate is valid; otherwise, <c>false</c>.
        /// </value>
        public bool IsCertificateValid { get; internal set; }

        /// <summary>
        /// Gets or sets the certificates used to sign the document.
        /// </summary>
        /// <value>The certificates used to sign the document.</value>
        public X509Certificate2Collection Certificates { get; internal set; }

        /// <summary>
        /// Gets or sets a collection of chain elements describing the errors from the certificate validation.
        /// </summary>
        /// <value>A collection of chain elements.</value>
        public X509ChainElementCollection CertificatesChainElements { get; internal set; }

        /// <summary>
        /// Gets or sets the signer information. The name of the person or authority signing the document.
        /// </summary>
        /// <value>The signer information.</value>
        public string SignerInformation { get; internal set; }

        /// <summary>
        /// Gets or sets the hash algorithm Oid used for calculating the message digest.
        /// </summary>
        /// <value>The hash algorithm Oid.</value>
        public Oid HashAlgorithm { get; internal set; }
    }
}