﻿using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Model.Editing
{
    /// <summary>
    /// Represents a border element.
    /// </summary>
    public class Border
    {
        private readonly double thickness;
        private readonly ColorBase color;
        private readonly BorderStyle borderStyle;

        /// <summary>
        /// Initializes a new instance of the <see cref="Border" /> class.
        /// </summary>
        public Border()
            : this(FixedDocumentDefaults.StrokeThickness, FixedDocumentDefaults.DefaultBorderStyle, FixedDocumentDefaults.Color)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Border" /> class.
        /// </summary>
        /// <param name="borderStyle">The border style.</param>
        public Border(BorderStyle borderStyle)
            : this(FixedDocumentDefaults.StrokeThickness, borderStyle, FixedDocumentDefaults.Color)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Border" /> class.
        /// </summary>
        /// <param name="thickness">The thickness.</param>
        /// <param name="color">The color.</param>
        public Border(double thickness, ColorBase color)
            : this(thickness, FixedDocumentDefaults.DefaultBorderStyle, color)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Border" /> class.
        /// </summary>
        /// <param name="thickness">The thickness.</param>
        /// <param name="borderStyle">The border style.</param>
        /// <param name="color">The color.</param>
        public Border(double thickness, BorderStyle borderStyle, ColorBase color)
        {
            this.borderStyle = borderStyle;
            this.color = color;
            this.thickness = thickness;
        }

        /// <summary>
        /// Gets the border style.
        /// </summary>
        /// <value>The border style.</value>
        public BorderStyle BorderStyle
        {
            get
            {
                return this.borderStyle;
            }
        }

        /// <summary>
        /// Gets the border thickness.
        /// </summary>
        /// <value>The thickness.</value>
        public double Thickness
        {
            get
            {
                return this.thickness;
            }
        }

        /// <summary>
        /// Gets the border color.
        /// </summary>
        /// <value>The color.</value>
        public ColorBase Color
        {
            get
            {
                return this.color;
            }
        }

        internal static double GetThickness(Border border)
        {
            if (border == null || border.BorderStyle == BorderStyle.None)
            {
                return 0;
            }

            return border.Thickness;
        }
    }
}
