﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Editing.Lists;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Collections
{
    /// <summary>
    /// Represents the collection of lists in RadFixedDocument.
    /// </summary>
    public class ListCollection
    {
        private readonly IdGenerator listIdGenerator;
        private readonly Dictionary<int, List> idToList;

        internal ListCollection()
        {
            this.listIdGenerator = new IdGenerator();
            this.idToList = new Dictionary<int, List>();
        }

        internal List this[int id]
        {
            get
            {
                return this.idToList[id];
            }
        }

        internal bool TryGetList(int id, out List list)
        {
            return this.idToList.TryGetValue(id, out list);
        }

        /// <summary>
        /// Adds the list.
        /// </summary>
        /// <returns>The list.</returns>
        public List AddList()
        {
            return this.AddList(new List());
        }

        /// <summary>
        /// Adds the list by some list template.
        /// </summary>
        /// <param name="listTemplateType">Type of the list template.</param>
        /// <returns>The list</returns>
        public List AddList(ListTemplateType listTemplateType)
        {
            return this.AddList(new List(listTemplateType));
        }

        private List AddList(List list)
        {
            list.Id = this.listIdGenerator.GetNext();
            this.idToList.Add(list.Id, list);

            return list;
        }
    }
}
