﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Flow
{
    /// <summary>
    /// Represents a block element.
    /// </summary>
    public interface IBlockElement
    {
        /// <summary>
        /// Gets if there is any pending content after the last measure.
        /// </summary>
        /// <value>Boolean value indicating whether there is any pending content after the last measure.</value>
        bool HasPendingContent { get; }

        /// <summary>
        /// Gets the desired size.
        /// </summary>
        /// <value>The desired size.</value>
        Size DesiredSize { get; }

        /// <summary>
        /// Measures the block element with specified available size.
        /// </summary>
        /// <param name="availableSize">The available size.</param>
        /// <returns>The result size.</returns>
        Size Measure(Size availableSize);

        /// <summary>
        /// Draws the block element with the specified editor.
        /// </summary>
        /// <param name="editor">The editor.</param>
        /// <param name="boundingRect">The bounding rect.</param>
        void Draw(FixedContentEditor editor, Rect boundingRect);

        /// <summary>
        /// Splits this instance.
        /// </summary>
        /// <returns>The remaining block element.</returns>
        IBlockElement Split();
    }
}
