﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
using Padding = Telerik.Documents.Primitives.Thickness;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Flow
{
    /// <summary>
    /// Defines a set of properties which will be applied when a page is created.
    /// </summary>
    public class SectionProperties
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SectionProperties" /> class.
        /// </summary>
        public SectionProperties()
        {
            this.PageSize = FixedDocumentDefaults.PageSize;
            this.PageMargins = FixedDocumentDefaults.PageMargins;
            this.PageRotation = FixedDocumentDefaults.PageRotation;
        }

        /// <summary>
        /// Gets or sets the size of the page.
        /// </summary>
        /// <value>The size of the page.</value>
        public Size PageSize { get; set; }

        /// <summary>
        /// Gets or sets the page margins.
        /// </summary>
        /// <value>The page margins.</value>
        public Padding PageMargins { get; set; }

        /// <summary>
        /// Gets or sets the page rotation.
        /// </summary>
        /// <value>The page rotation.</value>
        public Rotation PageRotation { get; set; }

        /// <summary>
        /// Copies the properties from.
        /// </summary>
        /// <param name="fromProperties">From properties.</param>
        public void CopyPropertiesFrom(SectionProperties fromProperties)
        {
            this.PageSize = fromProperties.PageSize;
            this.PageMargins = fromProperties.PageMargins;
            this.PageRotation = fromProperties.PageRotation;
        }

        internal void CopyTo(RadFixedPage page)
        {
            Guard.ThrowExceptionIfNull(page, "page");

            page.Size = this.PageSize;
            page.Rotation = this.PageRotation;
        }
    }
}
