﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Flow
{
    internal class SpacingsCalculator
    {
        private readonly SectionInfo section;
        private readonly ParagraphInfo previousParagraph;

        private bool isFirstBlock;

        public SpacingsCalculator(SectionInfo section)
        {
            Guard.ThrowExceptionIfNull(section, "section");

            this.section = section;
            this.previousParagraph = new ParagraphInfo();

            this.Clear();
            this.isFirstBlock = false;
        }

        public void SetPreviousBlock(IBlockElement blockElement)
        {
            this.previousParagraph.Copy(blockElement);
            this.isFirstBlock = false;
        }

        public double GetSpacingBefore(Block block)
        {
            Guard.ThrowExceptionIfNull(block, "block");

            if (this.ShouldApplySpacingBefore(block))
            {
                double previousBoxActualSpacingAfter = this.GetPreviousBlockSpacingAfter();
                double spacingBefore = block.SpacingBefore;

                double spacing = Math.Max(previousBoxActualSpacingAfter, spacingBefore);

                return spacing - previousBoxActualSpacingAfter;
            }
            else
            {
                return 0;
            }
        }

        private bool ShouldApplySpacingBefore(Block block)
        {
            Guard.ThrowExceptionIfNull(block, "block");

            return !this.isFirstBlock || this.section.IsFirstPage();
        }

        private double GetPreviousBlockSpacingAfter()
        {
            if (!this.previousParagraph.IsEmpty)
            {
                return this.previousParagraph.SpacingAfter;
            }

            return 0;
        }

        public bool ShouldExpandBackground(Block block)
        {
            return block.HasBackground &&
                !this.previousParagraph.IsEmpty &&
                this.previousParagraph.HasBackground &&
                block.LeftIndent.IsEqualTo(this.previousParagraph.LeftIndent) &&
                block.RightIndent.IsEqualTo(this.previousParagraph.RightIndent);
        }

        public Path GetExpandedBackground(double offsetX, double offsetY)
        {
            return this.previousParagraph.CreateAdditionalBackgroundPath(offsetX, offsetY);
        }

        public void Clear()
        {
            this.previousParagraph.Clear();
        }

        public void BeginExportContent()
        {
            this.Clear();
            this.isFirstBlock = true;
        }
    }
}
