﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Flow
{
    internal class TextWatermarkSettings : WatermarkSettingsBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TextWatermarkSettings" /> class.
        /// </summary>
        public TextWatermarkSettings()
        {
            this.Font = FixedDocumentDefaults.Font;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text { get; set; }

        /// <summary>
        /// Gets or sets the font family.
        /// </summary>
        /// <value>The font family.</value>
        public FontBase Font { get; set; }

        /// <summary>
        /// Gets or sets the foreground color of the text.
        /// </summary>
        /// <value>The the foreground color.</value>
        public RgbColor ForegroundColor { get; set; }

        /// <summary>
        /// Tries to set the current font.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <returns>True if succeeded.</returns>
        public bool TrySetFont(FontFamily fontFamily)
        {
            FontBase font;
            bool result = FontsRepository.TryCreateFont(fontFamily, FontStyles.Normal, FontWeights.Normal, out font);
            if (result)
            {
                this.Font = font;
            }

            return result;
        }
    }
}