﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing
{
    internal class GraphicsEditor : FixedContentElementEditorBase
    {
        private static readonly double controlPointRatio = (Math.Sqrt(2) - 1) * 4 / 3;

        public GraphicsEditor(FixedContentEditorBase editor)
            : base(editor)
        {
        }

        public void DrawLine(Point point1, Point point2)
        {
            PathGeometry pathGeometry = new PathGeometry();
            PathFigure figure = pathGeometry.Figures.AddPathFigure();
            figure.StartPoint = point1;
            figure.Segments.AddLineSegment().Point = point2;

            this.DrawPathInternal(pathGeometry);
        }

        public void DrawRectangle(Rect rectangle)
        {
            RectangleGeometry rectangleGeometry = new RectangleGeometry();
            rectangleGeometry.Rect = rectangle;

            this.DrawPathInternal(rectangleGeometry);
        }

        public void DrawEllipse(Point center, double radiusX, double radiusY)
        {
            double x0 = center.X - radiusX;
            double x1 = center.X - radiusX * controlPointRatio;
            double x2 = center.X;
            double x3 = center.X + radiusX * controlPointRatio;
            double x4 = center.X + radiusX;

            double y0 = center.Y - radiusY;
            double y1 = center.Y - radiusY * controlPointRatio;
            double y2 = center.Y;
            double y3 = center.Y + radiusY * controlPointRatio;
            double y4 = center.Y + radiusY;

            PathGeometry pathGeometry = new PathGeometry();
            PathFigure pathFigure = pathGeometry.Figures.AddPathFigure();
            pathFigure.StartPoint = new Point(x2, y0);
            pathFigure.IsClosed = true;

            pathFigure.Segments.AddBezierSegment(new Point(x3, y0), new Point(x4, y1), new Point(x4, y2));
            pathFigure.Segments.AddBezierSegment(new Point(x4, y3), new Point(x3, y4), new Point(x2, y4));
            pathFigure.Segments.AddBezierSegment(new Point(x1, y4), new Point(x0, y3), new Point(x0, y2));
            pathFigure.Segments.AddBezierSegment(new Point(x0, y1), new Point(x1, y0), new Point(x2, y0));

            this.DrawPathInternal(pathGeometry);
        }

        public void DrawPath(GeometryBase geometry)
        {
            Guard.ThrowExceptionIfNull(geometry, "geometry");

            this.DrawPathInternal(geometry);
        }

        private void DrawPathInternal(GeometryBase geometry)
        {
            Guard.ThrowExceptionIfNull(geometry, "geometry");

            Path path = new Path();
            path.Geometry = geometry;
            path.Fill = this.Editor.GraphicProperties.FillColor;
            path.Stroke = this.Editor.GraphicProperties.StrokeColor;
            path.StrokeThickness = this.Editor.GraphicProperties.StrokeThickness;
            path.MiterLimit = this.Editor.GraphicProperties.MiterLimit;
            path.StrokeDashArray = this.Editor.GraphicProperties.StrokeDashArray;
            path.StrokeDashOffset = this.Editor.GraphicProperties.StrokeDashOffset;
            path.StrokeLineCap = this.Editor.GraphicProperties.StrokeLineCap;
            path.StrokeLineJoin = this.Editor.GraphicProperties.StrokeLineJoin;
            path.IsFilled = this.Editor.GraphicProperties.IsFilled;
            path.IsStroked = this.Editor.GraphicProperties.IsStroked;

            this.Editor.Append(path);
        }
    }
}
