﻿using System.IO;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing
{
    internal class ImageEditor : FixedContentElementEditorBase
    {
        public ImageEditor(FixedContentEditorBase editor)
            : base(editor)
        {
        }

        public void DrawImage(Stream stream)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            this.DrawImage(stream, null, null);
        }

        public void DrawImage(Stream stream, double? width, double? height)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");

            ImageSource source = new ImageSource(stream);
            this.DrawImageInternal(source, width, height);
        }

        public void DrawImage(ImageSource source)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            this.DrawImageInternal(source, null, null);
        }

        public void DrawImage(ImageSource source, double width, double height)
        {
            Guard.ThrowExceptionIfNull(source, "source");

            this.DrawImageInternal(source, width, height);
        }

        private void DrawImageInternal(ImageSource source, double? width, double? height)
        {
            Image image = new Image();
            image.ImageSource = source;

            if (width.HasValue)
            {
                image.Width = width.Value;
            }

            if (height.HasValue)
            {
                image.Height = height.Value;
            }

            this.Editor.Append(image);
        }
    }
}
