﻿using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class BulletLayoutElement : LayoutElementBase
    {
        private readonly IEnumerable<LayoutElementBase> bulletElements;

        public BulletLayoutElement(LineInfo bulletElementsLine, double indentAfterBullet, TextProperties textProperties)
            : base(bulletElementsLine.Width + indentAfterBullet, bulletElementsLine.TrimmedWidth, bulletElementsLine.BaselineOffset,
            textProperties.Font, textProperties.FontSize)
        {
            this.bulletElements = bulletElementsLine.Elements;
        }

        internal override LineInfo CompleteLine(double maxLineWidth, double currentLineOffsetX, List<LayoutElementBase> elementsInCurrentLine, out IEnumerable<LayoutElementBase> pendingLayoutElements)
        {
            LineInfo line = new LineInfo();
            line.AddRange(elementsInCurrentLine);
            line.Add(this);
            pendingLayoutElements = Enumerable.Empty<LayoutElementBase>();

            return line;
        }

        internal override void Draw(DrawLayoutElementContext context)
        {
            Block.Draw(this.bulletElements, context);
        }

        internal override bool CanFit(double maxTotalWidth, double currentLineOffsetX)
        {
            return true;
        }

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return this.bulletElements.ToString();
        }
    }
}
