﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class DrawLayoutElementContext
    {
        private readonly Block block;
        private readonly FixedContentEditor editor;
        private readonly LineInfo line;
        private readonly double offsetX;
        private readonly double offsetY;
        private readonly Rect boundingRectangle;

        public DrawLayoutElementContext(FixedContentEditor editor, Block block, LineInfo line, double offsetX, double offsetY, Rect boundingRectangle)
        {
            Guard.ThrowExceptionIfNull(editor, "editor");
            Guard.ThrowExceptionIfNull(block, "block");
            Guard.ThrowExceptionIfNull(line, "line");

            this.block = block;
            this.editor = editor;
            this.line = line;
            this.offsetX = offsetX;
            this.offsetY = offsetY;
            this.boundingRectangle = boundingRectangle;
        }

        public Rect BoundingRectangle
        {
            get
            {
                return this.boundingRectangle;
            }
        }

        public double OffsetY
        {
            get
            {
                return this.offsetY;
            }
        }

        public double OffsetX
        {
            get
            {
                return this.offsetX;
            }
        }

        public LineInfo Line
        {
            get
            {
                return this.line;
            }
        }

        public FixedContentEditor Editor
        {
            get
            {
                return this.editor;
            }
        }

        public Block Block
        {
            get
            {
                return this.block;
            }
        }

        public double GetLineActualBaselineOffset()
        {
            switch (this.Block.LineSpacingType)
            {
                case HeightType.AtLeast:
                    return this.Block.GetActualLineHeight(this.Line) - this.Line.Descent;

                case HeightType.Exact:
                    double k = this.Block.LineSpacing / (this.Line.BaselineOffset + this.Line.LineSpacingDescent);

                    return this.Block.LineSpacing - this.Line.LineSpacingDescent * k;

                default:
                    return this.Line.BaselineOffset;
            }
        }
    }
}
