﻿using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class EndMarkerLayoutElement : MarkerLayoutElementBase
    {
        public EndMarkerLayoutElement(FontBase font, double fontSize)
            : base(font, fontSize)
        {
        }
    }

    internal class EndMarkerLayoutElement<TStart> : EndMarkerLayoutElement
        where TStart : StartMarkerLayoutElement
    {
        private readonly TStart start;

        public EndMarkerLayoutElement(TStart start, FontBase font, double fontSize)
            : base(font, fontSize)
        {
            Guard.ThrowExceptionIfNull(start, "start");

            this.start = start;
        }

        public TStart Start
        {
            get
            {
                return this.start;
            }
        }

        internal override bool ShouldSplitToBlocks(Block block)
        {
            Guard.ThrowExceptionIfNull(block, "block");

            return !block.IsPending(this.Start);
        }

        internal override void SplitToBlocks(Block block, out LayoutElementBase element, out System.Collections.Generic.IEnumerable<LayoutElementBase> elementsToInclude, out System.Collections.Generic.IEnumerable<LayoutElementBase> pendingElements)
        {
            Guard.ThrowExceptionIfNull(block, "block");

            StartMarkerLayoutElement newStart = this.Start.Clone();
            element = newStart.CreateEnd();
            elementsToInclude = new LayoutElementBase[] { this };
            pendingElements = new LayoutElementBase[] { newStart };
        }
    }
}
