﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class GraphicBasedLayoutElementBase<T> : ContentElementLayoutElementBase<T>
        where T : PositionContentElement
    {
        internal GraphicBasedLayoutElementBase(T element, Size size, TextProperties properties)
            : base(element,
                    size.Width,
                    size.Width,
                    size.Height,
                    properties.Font,
                    properties.FontSize,
                    properties.RenderingMode,
                    properties.UnderlineColor,
                    properties.UnderlinePattern,
                    properties.HighlightColor)
        {
        }

        internal override double Descent
        {
            get
            {
                if (this.UnderlinePattern != UnderlinePattern.None)
                {
                    return base.Descent;
                }

                return 0;
            }
        }

        internal virtual Point LayoutBoundsTopLeft
        {
            get
            {
                return new Point();
            }
        }

        protected override Matrix Transform(DrawLayoutElementContext context, Matrix transform)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            Point topLeft = this.LayoutBoundsTopLeft;
            double horizotalOffset = -topLeft.X;
            double verticalOffset = context.GetLineActualBaselineOffset() - this.BaselineOffset - topLeft.Y;

            return base.Transform(context, transform)
                .TranslateMatrix(horizotalOffset, verticalOffset);
        }
    }
}
