﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class HyperlinkEndLayoutElement : EndMarkerLayoutElement<HyperlinkStartLayoutElement>
    {
        public HyperlinkEndLayoutElement(HyperlinkStartLayoutElement start, FontBase font, double fontSize)
            : base(start, font, fontSize)
        {
        }

        internal override void Draw(DrawLayoutElementContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNotEqual(this.Start.Block, context.Block, "block");

            if (context.Editor.SupportsAnnotations)
            {
                Block block = context.Block;
                LineInfo first = this.Start.Line;
                LineInfo last = context.Line;
                HyperlinkStartLayoutElement hyperlinkStart = this.Start;

                foreach (LineInfo line in block.GetLinesBetween(first, last))
                {
                    context.Editor.AddAnnotation(CreateAnnotation(line, hyperlinkStart));
                }

                if (first == last)
                {
                    context.Editor.AddAnnotation(CreateAnnotation(first, hyperlinkStart, this.Start.OffsetX, context.OffsetX));
                }
                else
                {
                    context.Editor.AddAnnotation(CreateAnnotation(first, hyperlinkStart, this.Start.OffsetX, first.BoundingRect.X + first.BoundingRect.Width));
                    context.Editor.AddAnnotation(CreateAnnotation(last, hyperlinkStart, 0, context.OffsetX));
                }
            }
        }

        private static Annotation CreateAnnotation(LineInfo line, HyperlinkStartLayoutElement hyperlinkStart)
        {
            Guard.ThrowExceptionIfNull(line, "line");
            Guard.ThrowExceptionIfNull(hyperlinkStart, "hyperlinkStart");

            return CreateAnnotation(line.BoundingRect, hyperlinkStart);
        }

        private static Annotation CreateAnnotation(LineInfo line, HyperlinkStartLayoutElement hyperlinkStart, double x1, double x2)
        {
            Guard.ThrowExceptionIfNull(line, "line");
            Guard.ThrowExceptionIfNull(hyperlinkStart, "hyperlinkStart");

            double width = Math.Max(0, x2 - x1);
            return CreateAnnotation(new Rect(x1, line.BoundingRect.Y, width, line.BoundingRect.Height), hyperlinkStart);
        }

        private static Annotation CreateAnnotation(Rect rect, HyperlinkStartLayoutElement hyperlinkStart)
        {
            Guard.ThrowExceptionIfNull(hyperlinkStart, "hyperlinkStart");

            Annotation annotation = hyperlinkStart.CreateAnnotation();
            annotation.Rect = rect;

            return annotation;
        }
    }
}
