﻿using System;
using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.Utilities;
#if NETSTANDARD
using Rect = Telerik.Documents.Primitives.Rect;
#else
using Rect = System.Windows.Rect;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class LineInfo
    {
        private readonly List<LayoutElementBase> elements;

        public LineInfo()
        {
            this.elements = new List<LayoutElementBase>();
        }

        public IEnumerable<LayoutElementBase> Elements
        {
            get
            {
                return this.elements;
            }
        }

        public double Width
        {
            get;
            private set;
        }

        public double TrimmedWidth
        {
            get;
            private set;
        }

        public Rect BoundingRect { get; set; }

        public double BaselineOffset
        {
            get;
            private set;
        }

        public double Descent
        {
            get;
            private set;
        }

        public double LineSpacingBaselineOffset
        {
            get;
            private set;
        }

        public double LineSpacingDescent
        {
            get;
            private set;
        }

        public double UnderlineThickness
        {
            get;
            private set;
        }

        public double UnderlinePosition
        {
            get;
            private set;
        }

        public bool IsEndingWithLineBreak
        {
            get
            {
                return this.elements.Count > 0 && this.elements[this.elements.Count - 1].IsLineBreak;
            }
        }

        public void Add(LayoutElementBase element)
        {
            Guard.ThrowExceptionIfNull(element, "element");

            if (element.TrimmedWidth != 0)
            {
                this.TrimmedWidth = this.Width + element.TrimmedWidth;
            }

            this.Width += element.Width;

            this.UnderlinePosition = Math.Max(this.UnderlinePosition, element.UnderlinePosition);
            this.UnderlineThickness = Math.Max(this.UnderlineThickness, element.UnderlineThickness);
            this.BaselineOffset = Math.Max(this.BaselineOffset, element.BaselineOffset);
            this.Descent = Math.Max(this.Descent, element.Descent);
            this.LineSpacingBaselineOffset = Math.Max(this.LineSpacingBaselineOffset, element.LineSpacingBaselineOffset);
            this.LineSpacingDescent = Math.Max(this.LineSpacingDescent, element.LineSpacingDescent);

            this.elements.Add(element);
        }

        public void AddRange(IEnumerable<LayoutElementBase> elements)
        {
            Guard.ThrowExceptionIfNull(elements, "elements");

            foreach (LayoutElementBase element in elements)
            {
                this.Add(element);
            }
        }

        public override string ToString()
        {
            StringBuilder builder = new StringBuilder();

            foreach (LayoutElementBase element in this.elements)
            {
                builder.Append(element.ToString());
            }

            return builder.ToString();
        }
    }
}