﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Layout
{
    internal class PathLayoutElement : GraphicBasedLayoutElementBase<Path>
    {
        public PathLayoutElement(Path path, TextProperties textProperties)
            : base(path, GetSize(path), textProperties)
        {
        }

        internal override Point LayoutBoundsTopLeft
        {
            get
            {
                Rect bounds = this.Element.Geometry.Bounds;

                double x = bounds.X;
                double y = bounds.Y;

                if (this.Element.IsStroked)
                {
                    double strokeThickness = this.Element.StrokeThickness / 2;
                    x -= strokeThickness;
                    y -= strokeThickness;
                }

                return new Point(x, y);
            }
        }

        private static Size GetSize(Path path)
        {
            Guard.ThrowExceptionIfNull(path, "path");

            Rect bounds = path.Geometry.Bounds;

            double width = bounds.Width;
            double height = bounds.Height;

            if (path.IsStroked)
            {
                width += path.StrokeThickness;
                height += path.StrokeThickness;
            }

            return new Size(width, height);
        }
    }
}
