﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Lists
{
    internal static class BulletNumberingFormats
    {
        public const string Whitespace = " ";
        public static readonly IBulletNumberingFormat EmptyNumbering;
        
        static BulletNumberingFormats()
        {
            EmptyNumbering = new TextBulletNumberingFormat((i) => { return null; });
        }

        public static string GetNumber(int number)
        {
            return number.ToString();
        }

        public static string GetLowerLetter(int number)
        {
            if (number <= 0)
            {
                return Whitespace;
            }

            int index = number - 1;

            int a = 'a';
            int z = 'z';
            int symbolsCount = (z - a + 1);

            int digitsCount = 1 + index / symbolsCount;
            int symbolIndex = index % symbolsCount;
            char symbol = (char)(a + symbolIndex);

            return new String(symbol, digitsCount);
        }

        public static string GetLowerRomanNumber(int number)
        {
            if (number <= 0)
            {
                return Whitespace;
            }

            return MathUtilities.IntegerToRomanString(number);
        }

        public static string GetUpperRomanNumber(int number)
        {
            return GetLowerRomanNumber(number).ToUpper();
        }

        public static string GetUpperLetter(int number)
        {
            return GetLowerLetter(number).ToUpper();
        }
    }
}
