﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Editing.Collections;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Lists
{
    /// <summary>
    /// Represent list element.
    /// </summary>
    public class List
    {
        private readonly ListLevelCollection levels;
        private readonly ListLevelsIndexer indexer;
        private int? id;

        /// <summary>
        /// Initializes a new instance of the <see cref="List" /> class.
        /// </summary>
        public List()
        {
            this.levels = new ListLevelCollection();
            this.indexer = new ListLevelsIndexer(this.levels);
            this.id = null;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="List" /> class.
        /// </summary>
        /// <param name="listTemplateType">Type of the list template.</param>
        public List(ListTemplateType listTemplateType)
            : this()
        {
            ListFactory.InitializeList(this, listTemplateType);
        }

        /// <summary>
        /// Gets the id.
        /// </summary>
        /// <value>The id.</value>
        public int Id
        {
            get
            {
                return this.id.HasValue ? this.id.Value : -1;
            }
            internal set
            {
                if (this.id.HasValue)
                {
                    throw new InvalidOperationException("Cannot add same List instance to ListCollection multiple times!");
                }

                this.id = value;
            }
        }

        /// <summary>
        /// Gets the list levels.
        /// </summary>
        /// <value>The list levels.</value>
        public ListLevelCollection Levels
        {
            get
            {
                return this.levels;
            }
        }

        internal ListLevelsIndexer Indexer
        {
            get
            {
                return this.indexer;
            }
        }
    }
}
