﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Lists
{
    /// <summary>
    /// Represents the text bullet numbering format.
    /// </summary>
    public class TextBulletNumberingFormat : IBulletNumberingFormat
    {
        private readonly Func<IListLevelsIndexer, string> getTextBullet;

        /// <summary>
        /// Initializes a new instance of the <see cref="TextBulletNumberingFormat" /> class.
        /// </summary>
        /// <param name="getTextBullet">The method for getting text bullet representation from number.</param>
        public TextBulletNumberingFormat(Func<IListLevelsIndexer, string> getTextBullet)
        {
            Guard.ThrowExceptionIfNull(getTextBullet, "getTextBullet");

            this.getTextBullet = getTextBullet;
        }

        /// <summary>
        /// Gets the bullet numbering element.
        /// </summary>
        /// <param name="listLevelsIndexer">The list levels indexer.</param>
        /// <returns>The bullet numbering element.</returns>
        public PositionContentElement GetBulletNumberingElement(IListLevelsIndexer listLevelsIndexer)
        {
            string text = this.getTextBullet(listLevelsIndexer);
            TextFragment textFragment = string.IsNullOrEmpty(text) ? null : new TextFragment(text);

            return textFragment;
        }
    }
}
