﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Editing
{
    /// <summary>
    /// Represents base Properties class.
    /// </summary>
    public abstract class PropertiesBase<T> where T : PropertiesBase<T>, new()
    {
        private readonly Stack<T> propertiesStack;

        internal PropertiesBase()
        {
            this.propertiesStack = new Stack<T>();
        }

        /// <summary>
        /// Saves the text properties.
        /// </summary>
        public IDisposable Save()
        {
            T textProperties = new T();
            textProperties.CopyFrom((T)this);
            this.propertiesStack.Push(textProperties);

            return new DisposableObject(this.Restore);
        }

        /// <summary>
        /// Restores the text properties.
        /// </summary>
        public void Restore()
        {
            this.CopyFrom(this.propertiesStack.Pop());
        }

        /// <summary>
        /// Copies the text properties from other instance.
        /// </summary>
        /// <param name="other">The other properties.</param>
        public abstract void CopyFrom(T other);
    }
}
