﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Editing
{
    /// <summary>
    /// Represent the borders for a table.
    /// </summary>
    public class TableBorders
    {
        private readonly Border left;
        private readonly Border top;
        private readonly Border right;
        private readonly Border bottom;

        /// <summary>
        /// Initializes a new instance of the <see cref="TableBorders" /> class.
        /// </summary>
        public TableBorders()
            : this(null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TableBorders" /> class.
        /// </summary>
        /// <param name="all">All.</param>
        public TableBorders(Border all)
            : this(all, all, all, all)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TableBorders" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="top">The top.</param>
        /// <param name="right">The right.</param>
        /// <param name="bottom">The bottom.</param>
        public TableBorders(Border left, Border top, Border right, Border bottom)
        {
            this.left = left;
            this.top = top;
            this.right = right;
            this.bottom = bottom;
        }

        /// <summary>
        /// Gets the left.
        /// </summary>
        /// <value>The left.</value>
        public Border Left
        {
            get
            {
                return this.left;
            }
        }

        /// <summary>
        /// Gets the top.
        /// </summary>
        /// <value>The top.</value>
        public Border Top
        {
            get
            {
                return this.top;
            }
        }

        /// <summary>
        /// Gets the right.
        /// </summary>
        /// <value>The right.</value>
        public Border Right
        {
            get
            {
                return this.right;
            }
        }

        /// <summary>
        /// Gets the bottom.
        /// </summary>
        /// <value>The bottom.</value>
        public Border Bottom
        {
            get
            {
                return this.bottom;
            }
        }

        internal Thickness Thickness
        {
            get
            {
                return new Thickness(Border.GetThickness(this.Left), Border.GetThickness(this.Top), Border.GetThickness(this.Right), Border.GetThickness(this.Bottom));
            }
        }

        internal virtual void Draw(FixedContentEditor editor, Rect middleLinesRect)
        {
            DrawBorders(editor, middleLinesRect, this.Left, this.Top, this.Right, this.Bottom);
        }

        internal static void DrawBorders(FixedContentEditor editor, Rect middleLinesRect, Border leftBorder, Border topBorder, Border rightBorder, Border bottomBorder)
        {
            Thickness thickness = new Thickness(Border.GetThickness(leftBorder), Border.GetThickness(topBorder), Border.GetThickness(rightBorder), Border.GetThickness(bottomBorder));

            double left = middleLinesRect.Left;
            double top = middleLinesRect.Top;
            double right = left + middleLinesRect.Width;
            double bottom = top + middleLinesRect.Height;

            editor.GraphicProperties.IsFilled = false;
            editor.GraphicProperties.IsStroked = true;

            DrawBorder(editor, leftBorder, new Point(left, top - thickness.Top / 2), new Point(left, bottom + thickness.Bottom / 2));
            DrawBorder(editor, topBorder, new Point(left - thickness.Left / 2, top), new Point(right + thickness.Right / 2, top));
            DrawBorder(editor, rightBorder, new Point(right, top - thickness.Top / 2), new Point(right, bottom + thickness.Bottom / 2));
            DrawBorder(editor, bottomBorder, new Point(left - thickness.Left / 2, bottom), new Point(right + thickness.Right / 2, bottom));
        }

        internal static void DrawBorder(FixedContentEditor editor, Border border, Point start, Point end)
        {
            if (border != null && border.BorderStyle != BorderStyle.None)
            {
                editor.GraphicProperties.StrokeThickness = border.Thickness;
                editor.GraphicProperties.StrokeColor = border.Color;
                editor.DrawLine(start, end);
            }
        }

        ///// <summary>
        ///// Gets or sets the top left radiuses.
        ///// </summary>
        ///// <value>The top left radiuses.</value>
        //internal Size TopLeftRadiuses { get; set; }

        ///// <summary>
        ///// Gets or sets the top right radiuses.
        ///// </summary>
        ///// <value>The top right radiuses.</value>
        //internal Size TopRightRadiuses { get; set; }

        ///// <summary>
        ///// Gets or sets the bottom left radiuses.
        ///// </summary>
        ///// <value>The bottom left radiuses.</value>
        //internal Size BottomLeftRadiuses { get; set; }

        ///// <summary>
        ///// Gets or sets the bottom right radiuses.
        ///// </summary>
        ///// <value>The bottom right radiuses.</value>
        //internal Size BottomRightRadiuses { get; set; }
    }
}
