﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Tables
{
    /// <summary>
    /// Represents the borders of a table cell.
    /// </summary>
    public class TableCellBorders
    {
        private readonly Border left;
        private readonly Border top;
        private readonly Border right;
        private readonly Border bottom;
        private readonly Border diagonalUp;
        private readonly Border diagonalDown;

        /// <summary>
        /// Initializes a new instance of the <see cref="TableCellBorders" /> class.
        /// </summary>
        public TableCellBorders()
            : this(null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TableCellBorders" /> class.
        /// </summary>
        /// <param name="all">All.</param>
        public TableCellBorders(Border all)
            : this(all, all, all, all, all, all)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TableCellBorders" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="top">The top.</param>
        /// <param name="right">The right.</param>
        /// <param name="bottom">The bottom.</param>
        public TableCellBorders(Border left, Border top, Border right, Border bottom)
            : this(left, top, right, bottom, null, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TableCellBorders" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="top">The top.</param>
        /// <param name="right">The right.</param>
        /// <param name="bottom">The bottom.</param>
        /// <param name="diagonalUp">The diagonal up.</param>
        /// <param name="diagonalDown">The diagonal down.</param>
        public TableCellBorders(Border left, Border top, Border right, Border bottom, Border diagonalUp, Border diagonalDown)
        {
            this.left = left;
            this.top = top;
            this.right = right;
            this.bottom = bottom;
            this.diagonalUp = diagonalUp;
            this.diagonalDown = diagonalDown;
        }

        /// <summary>
        /// Gets the left.
        /// </summary>
        /// <value>The left.</value>
        public Border Left
        {
            get
            {
                return this.left;
            }
        }

        /// <summary>
        /// Gets the top.
        /// </summary>
        /// <value>The top.</value>
        public Border Top
        {
            get
            {
                return this.top;
            }
        }

        /// <summary>
        /// Gets the right.
        /// </summary>
        /// <value>The right.</value>
        public Border Right
        {
            get
            {
                return this.right;
            }
        }

        /// <summary>
        /// Gets the bottom.
        /// </summary>
        /// <value>The bottom.</value>
        public Border Bottom
        {
            get
            {
                return this.bottom;
            }
        }

        /// <summary>
        /// Gets or sets the diagonal up.
        /// </summary>
        /// <value>The diagonal up.</value>
        public Border DiagonalUp
        {
            get
            {
                return this.diagonalUp;
            }
        }

        /// <summary>
        /// Gets or sets the diagonal down.
        /// </summary>
        /// <value>The diagonal down.</value>
        public Border DiagonalDown
        {
            get
            {
                return this.diagonalDown;
            }
        }

        internal Thickness Thickness
        {
            get
            {
                return new Thickness(Border.GetThickness(this.Left), Border.GetThickness(this.Top), Border.GetThickness(this.Right), Border.GetThickness(this.Bottom));
            }
        }

        internal void Draw(FixedContentEditor editor, Rect middleLinesRect)
        {
            Thickness thickness = this.Thickness;

            double left = middleLinesRect.Left + thickness.Left / 2;
            double top = middleLinesRect.Top + thickness.Top / 2;
            double right = middleLinesRect.Left + middleLinesRect.Width - thickness.Right / 2;
            double bottom = middleLinesRect.Top + middleLinesRect.Height - thickness.Bottom / 2;

            editor.GraphicProperties.IsFilled = false;
            editor.GraphicProperties.IsStroked = true;

            TableBorders.DrawBorder(editor, this.DiagonalUp, new Point(left, bottom), new Point(right, top));
            TableBorders.DrawBorder(editor, this.DiagonalDown, new Point(left, top), new Point(right, bottom));
            TableBorders.DrawBorders(editor, middleLinesRect, this.Left, this.Top, this.Right, this.Bottom);
        }
    }
}
