﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Editing.Collections;

namespace Telerik.Windows.Documents.Fixed.Model.Editing.Tables
{
    /// <summary>
    /// Represents a row in a table.
    /// </summary>
    public class TableRow
    {
        private readonly TableCellCollection cells;
        private readonly List<TableCell> allIntersectingCells;

        internal TableRow()
        {
            this.cells = new TableCellCollection();
            this.allIntersectingCells = new List<TableCell>();
        }

        /// <summary>
        /// Gets the cells for the row.
        /// </summary>
        /// <value>The cells.</value>
        public TableCellCollection Cells
        {
            get
            {
                return this.cells;
            }
        }

        internal double Height
        {
            get;
            set;
        }

        internal double Top
        {
            get;
            set;
        }

        internal double Bottom
        {
            get
            {
                return this.Top + this.Height;
            }
        }

        internal List<TableCell> AllIntersectingCells
        {
            get
            {
                return this.allIntersectingCells;
            }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return String.Format("Row Top:{0}, Height:{1}", this.Top, this.Height);
        }

        internal TableRow Copy()
        {
            TableRow row = new TableRow();
            row.Cells.AddRange(this.Cells);
            row.AllIntersectingCells.AddRange(this.AllIntersectingCells);

            return row;
        }
    }
}
