﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    internal partial class CidType2Font : CidFontBase
    {
        private GlyphTypeface glyphTypeface;
        private string prefix;

        internal override string FontName
        {
            get
            {
                return this.prefix + base.FontName;
            }
        }

        internal GlyphTypeface GlyphTypeface
        {
            get
            {
                return this.glyphTypeface;
            }
            set
            {
                if (this.glyphTypeface != value)
                {
                    this.glyphTypeface = value;
                    this.prefix = this.glyphTypeface == null ? string.Empty : CreateEmbeddedFontSubsetName();
                }
            }
        }

        internal override byte[] ComputeSubset(IEnumerable<CharInfo> usedCharacters)
        {
            if (!usedCharacters.Any())
            {
                return new byte[0];
            }

            byte[] subset = null;
            if (this.GlyphTypeface != null)
            {
                try
                {
                    subset = this.GlyphTypeface.ComputeSubset(usedCharacters.Select(g => (ushort)g.CharCode.Code).ToArray());
                }
                catch (System.IO.FileFormatException)
                {
                   // The GlyphTypeface class can throw an exception for specific fonts. Invoke the base class in such case.
                }
            }

            if (subset == null)
            {
                subset = base.ComputeSubset(usedCharacters);
            }

            return subset;
        }

        private string CreateEmbeddedFontSubsetName()
        {
            System.Text.StringBuilder stringBuilder = new System.Text.StringBuilder(64);
            byte[] bytes = Guid.NewGuid().ToByteArray();
            for (int index = 0; index < 6; index++)
            {
                stringBuilder.Append((char)('A' + bytes[index] % 26));
            }

            stringBuilder.Append('+');

            return stringBuilder.ToString();
        }
    }
}