﻿using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    internal class CidWidthsRange<T>
    {
        private readonly int startCharCode;
        private readonly int endCharCode;
        private readonly T constantWidth;
        private readonly List<T> widths;

        public CidWidthsRange(int startCharCode, int endCharCode, T constantWidth)
        {
            this.startCharCode = startCharCode;
            this.endCharCode = endCharCode;
            this.constantWidth = constantWidth;
            this.widths = null;
        }

        public CidWidthsRange(int startCharCode, IEnumerable<T> widths)
        {
            this.startCharCode = startCharCode;
            this.widths = new List<T>(widths);
            this.endCharCode = this.startCharCode + this.widths.Count - 1;
        }

        public bool IsConstantWidthRange
        {
            get
            {
                return this.widths == null;
            }
        }

        public int StartCharCode
        {
            get
            {
                return this.startCharCode;
            }
        }

        public int EndCharCode
        {
            get
            {
                return this.endCharCode;
            }
        }

        public T ConstantWidth
        {
            get
            {
                return this.constantWidth;
            }
        }

        public IEnumerable<T> Widths
        {
            get
            {
                if (this.IsConstantWidthRange)
                {
                    int numberOfCharCodes = this.EndCharCode - this.StartCharCode + 1;
                    return Enumerable.Repeat(this.ConstantWidth, numberOfCharCodes);
                }
                else
                {
                    return this.widths;
                }
            }
        }

        public bool TryGetWidth(int charCode, out T width)
        {
            width = default(T);
            bool isInRange = this.StartCharCode <= charCode && charCode <= this.EndCharCode;

            if (isInRange)
            {
                if (this.IsConstantWidthRange)
                {
                    width = this.ConstantWidth;
                }
                else
                {
                    int index = charCode - this.StartCharCode;
                    width = this.widths[index];
                }
            }

            return isInRange;
        }
    }
}
