﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding
{
    internal class SimpleEncoding : ISimpleEncoding
    {
        private readonly PdfProperty<string> predefinedEncoding;
        private readonly PdfProperty<DifferencesRanges> differences;
        private string[] names;

        public SimpleEncoding()
        {
            this.predefinedEncoding = new PdfProperty<string>();
            this.differences = new PdfProperty<DifferencesRanges>();
        }

        public PdfProperty<string> PredefinedEncoding
        {
            get
            {
                return this.predefinedEncoding;
            }
        }

        public PdfProperty<DifferencesRanges> Differences
        {
            get
            {
                return this.differences;
            }
        }

        public bool IsNamedEncoding
        {
            get
            {
                return this.PredefinedEncoding.HasValue && !this.Differences.HasValue;
            }
        }

        public string BaseEncodingName
        {
            get
            {
                return this.PredefinedEncoding.Value;
            }
        }

        public string GetName(byte b)
        {
            this.EnsureNamesAreInitialized();
            string name = this.names[b];

            return name;
        }
        
        internal static string[] GetNamesWithoutDifferences(string baseEncoding)
        {
            PredefinedSimpleEncoding encoding = PredefinedSimpleEncoding.GetPredefinedEncoding(baseEncoding);
            encoding = encoding ?? PredefinedSimpleEncoding.StandardEncoding;

            return encoding.GetNames();
        }

        private void EnsureNamesAreInitialized()
        {
            if (this.names == null)
            {
                this.names = GetNamesWithoutDifferences(this.PredefinedEncoding.Value);
                this.CalculateDifferences();
            }
        }

        private void CalculateDifferences()
        {
            if (this.Differences.HasValue)
            {
                foreach(DifferencesRange range in this.Differences.Value)
                {
                    int charCode = range.StartCharCode;

                    foreach (string name in range.Differences)
                    {
                        if (charCode >= this.names.Length)
                        {
                            break;
                        }

                        this.names[charCode] = name;
                        charCode++;
                    }
                }
            }
        }
    }
}
