﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows;
using System.Windows.Media;
#else
using Telerik.Documents.Core.Fonts;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Utils;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    internal class FallbackRange
    {
        private const string DefaultFallbackFont = "Arial";

        private readonly object lockObject = new object();

        private readonly Dictionary<FontProperties, FontBase> fonts;
        private readonly string[] fallbackFontFamilies;

        private static List<FallbackRange> fallbackRanges;
        public static FallbackRange DefaultRange
        {
            get
            {
                return new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x2000, 0x202E),
                    new Core.Fonts.Utils.Range(0x2030, 0x20CF),
                    new Core.Fonts.Utils.Range(0x2100, 0x23FF),
                    new Core.Fonts.Utils.Range(0x2460, 0x27BF),
                },
                new string[]
                {
                    "Verdana",
                    "Segoe UI Symbol",
                    "SimSun",
                    "MS Gothic",
                    "MingLiU",
                    "Tahoma",
                    "Lucida Sans",
                    "Unicode", 
                    "Arial", 
                    "Arial Unicode MS"
                });
            }
        }

        private static void InitializeFallbackRanges()
        {
            fallbackRanges = new List<FallbackRange>();

            // Language = "ug"
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x0600, 0x06FF),
                    new Core.Fonts.Utils.Range(0x0750, 0x077F),
                    new Core.Fonts.Utils.Range(0xFB50, 0xFDCF),
                    new Core.Fonts.Utils.Range(0xFDF0, 0xFDFF),
                    new Core.Fonts.Utils.Range(0xFE70, 0xFEFE)
                },
                new string[]
                {
                    "Microsoft Uighur",
                    "Segoe UI", "Tahoma",
                    "Simplified Arabic",
                    "Traditional Arabic",
                    "Arial",
                    "Arial Unicode MS"
                }));

            // Other (include CHT, JA and KO)
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x0000, 0x052F),
                    new Core.Fonts.Utils.Range(0x0590, 0x06FF),
                    new Core.Fonts.Utils.Range(0x0750, 0x077F),
                    new Core.Fonts.Utils.Range(0x1D00, 0x1FFF),
                    new Core.Fonts.Utils.Range(0x2C60, 0x2C7F),
                    new Core.Fonts.Utils.Range(0xFB00, 0xFB0F),
                    new Core.Fonts.Utils.Range(0xFB1D, 0xFBFF),
                },
                new string[]
                {
                    "Segoe UI",
                    "Tahoma",
                    "Arial",
                    "Arial Unicode MS",
                    "Microsoft Sans Serif",
                    "Lucida Sans Unicode"
                }));

            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0xFC00, 0xFDCF),
                    new Core.Fonts.Utils.Range(0xFDF0, 0xFDFF),
                    new Core.Fonts.Utils.Range(0xFE20, 0xFE2F),
                    new Core.Fonts.Utils.Range(0xFE70, 0xFEFE)
                },
                new string[]
                {
                    "Tahoma",
                    "Simplified Arabic",
                    "Traditional Arabic",
                    "Arial Unicode MS"
                }));

            // Armenian    
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x0530, 0x058F),
                    new Core.Fonts.Utils.Range(0x10A0, 0x10CF),
                    new Core.Fonts.Utils.Range(0x10D0, 0x10FF),
                    new Core.Fonts.Utils.Range(0xFB10, 0xFB1C)
                },
                new string[]
                {
                    "Sylfaen",
                    "Arial Unicode MS"
                }));

            // Hangul Jamo, Enc. CJK, Hangul Syllables
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x1100, 0x11FF),
                    new Core.Fonts.Utils.Range(0x3130, 0x318F),
                    new Core.Fonts.Utils.Range(0x3200, 0x321F),
                    new Core.Fonts.Utils.Range(0x3260, 0x327F),
                    new Core.Fonts.Utils.Range(0xAC00, 0xD7AF)
                },
                new string[]
                {
                    "Malgun Gothic",
                    "Gulim"
                }));

            // General punctuation and Currency symbols, Letterlike symbols...
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x2000, 0x202E),
                    new Core.Fonts.Utils.Range(0x2030, 0x20CF),
                    new Core.Fonts.Utils.Range(0x2100, 0x23FF),
                    new Core.Fonts.Utils.Range(0x2460, 0x27BF),
                },
                new string[]
                {
                    "Microsoft YaHei",
                    "Meiryo", 
                    "Segoe UI",
                    "Segoe UI Symbol",
                    "SimSun",
                    "MS Gothic",
                    "MingLiU",
                    "Tahoma",
                    "Lucida Sans",
                    "Unicode", 
                    "Arial", 
                    "Arial Unicode MS"
                }));

            // Mathematical Symbols
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x27C0, 0x297F),
                    new Core.Fonts.Utils.Range(0x2A00, 0x2BFF),
                },
                new string[]
                {
                    "Segoe UI Symbol",
                    "Cambria Math"
                }));

            // CHS
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x2E80, 0x2EFF),
                },
                new string[]
                {
                    "Microsoft YaHei", 
                    "SimSun", 
                    "Meiryo",
                    "MingLiu"
                }));

            // Other (include JA)
            fallbackRanges.Add(new FallbackRange(
                new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x3000, 0x30FF),
                    new Core.Fonts.Utils.Range(0x31F0, 0x31FF),
                },
                new string[]
                {
                    "Meiryo",
                    "Segoe UI",
                    "Microsoft JhengHei",
                    "MingLiu",
                    "MS Gothic",
                    "Arial Unicode MS"
                }));


            fallbackRanges.Add(new FallbackRange(
               new Core.Fonts.Utils.Range[]
                {
                    new Core.Fonts.Utils.Range(0x3400, 0x4DBF),
                    new Core.Fonts.Utils.Range(0x4E00, 0x9FFF),
                },
               new string[]
                {
                   "Meiryo",
                   "Segoe UI",
                   "Microsoft YaHei",
                   "SimSun",
                   "SimSun-18030",
                   "SimSun-ExtB", 
                   "MingLiU"
                }));
        }

        internal static FallbackRange GetFallbackRange(char unicode)
        {
            foreach (FallbackRange fr in fallbackRanges)
            {
                if (fr.FallsInRange(unicode))
                {
                    return fr;
                }
            }
            return null;
        }

        static FallbackRange()
        {
            InitializeFallbackRanges();
        }

        public FallbackRange(Core.Fonts.Utils.Range[] ranges, string[] fallbackFontFamilies)
        {
            this.Ranges = ranges;
            this.fonts = new Dictionary<FontProperties, FontBase>();
            this.fallbackFontFamilies = fallbackFontFamilies;
        }

        public Core.Fonts.Utils.Range[] Ranges { get; private set; }

        private FontBase GetFont(string fontFamily, FontStyle fontStyle, FontWeight fontWeight)
        {
            FontProperties fontProperties = new FontProperties(new FontFamily(fontFamily), fontStyle, fontWeight);

            FontBase font;
            if (!this.fonts.TryGetValue(fontProperties, out font))
            {
                lock (lockObject)
                {
                    if (!this.fonts.TryGetValue(fontProperties, out font))
                    {
                        if (FontsRepository.TryCreateFont(new FontFamily(fontFamily), fontStyle, fontWeight, out font))
                        {
                            this.fonts[fontProperties] = font;
                        }
                    }
                }
            }

            return font;
        }

        public FontBase GetFallbackFont(FontBase original, char ch)
        {
            FontStyle fontStyle = original.IsItalic ? FontStyles.Italic : FontStyles.Normal;
            FontWeight fontWeight = original.IsBold ? FontWeights.Bold : FontWeights.Normal;
            foreach (string fontFamily in this.fallbackFontFamilies)
            {
                FontBase font = this.GetFont(fontFamily, fontStyle, fontWeight);
                if (font == null)
                {
                    continue;
                }

                ushort value;
                if (font.TryGetGlyphId(ch, out value))
                {
                    return font;
                }
            }

            return this.GetFont(DefaultFallbackFont, fontStyle, fontWeight);
        }

        private bool FallsInRange(char unicode)
        {
            foreach (Core.Fonts.Utils.Range r in this.Ranges)
            {
                if (r.IsInRange(unicode))
                {
                    return true;
                }
            }
            return false;
        }
    }
}
