﻿using System;
using System.IO;
using System.Linq;
using System.Windows;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Core.Fonts;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    /// <summary>
    /// Represents the fonts repository. It can be used to create new font object.
    /// </summary>
    public static partial class FontsRepository
    {
        /// <summary>
        /// Tries to create a new font object based on the given font properties.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <param name="fontStyle">The font style.</param>
        /// <param name="fontWeight">The font weight.</param>
        /// <param name="font">The font.</param>
        /// <returns>True if the font is created succesfully.</returns>
        public static bool TryCreateFont(FontFamily fontFamily, FontStyle fontStyle, FontWeight fontWeight, out FontBase font)
        {
            Guard.ThrowExceptionIfNull(fontFamily, "fontFamily");
            Guard.ThrowExceptionIfNull(fontStyle, "fontStyle");
            Guard.ThrowExceptionIfNull(fontWeight, "fontWeight");

            FontProperties fontProperties = new FontProperties(fontFamily, fontStyle, fontWeight);

            return fontPropertiesToFont.TryGetValue(fontProperties, out font);
        }
       
        /// <summary>
        /// Tries to create a new font object based on the given font properties.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <param name="font">The font.</param>
        /// <returns>True if the font is created succesfully.</returns>
        public static bool TryCreateFont(FontFamily fontFamily, out FontBase font)
        {
            Guard.ThrowExceptionIfNull(fontFamily, "fontFamily");

            return TryCreateFont(fontFamily, FontStyles.Normal, FontWeights.Normal, out font);
        }
    }
}
