﻿using System;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Fonts;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    /// <summary>
    /// Represents the fonts repository. It can be used to create new font object.
    /// </summary>
    public static partial class FontsRepository
    {
        private static readonly object lockObject = new object();

        /// <summary>
        /// Tries to create a new font object based on the given font properties.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <param name="fontStyle">The font style.</param>
        /// <param name="fontWeight">The font weight.</param>
        /// <param name="font">The font.</param>
        /// <returns>True if the font is created succesfully.</returns>
        public static bool TryCreateFont(FontFamily fontFamily, FontStyle fontStyle, FontWeight fontWeight, out FontBase font)
        {
            FontProperties fontProperties = new FontProperties(fontFamily, fontStyle, fontWeight);

            if (!fontPropertiesToFont.TryGetValue(fontProperties, out font))
            {
                lock (lockObject)
                {
                    if (!fontPropertiesToFont.TryGetValue(fontProperties, out font))
                    {
                        Typeface typeface = new Typeface(fontProperties.FontFamily, fontProperties.FontStyle, fontProperties.FontWeight, FontStretches.Normal);
                        GlyphTypeface glyphTypeface;
                        if (typeface.TryGetGlyphTypeface(out glyphTypeface))
                        {
                            Stream fontStream = glyphTypeface.GetFontStream();
                            font = CreateFont(fontProperties, Telerik.Windows.Documents.Utilities.Extensions.ReadAllBytes(fontStream));
                        }

                        if (font != null)
                        {
                            if (font.Type == FontType.CidType2Font)
                            {
                                ((CidType2Font)font).GlyphTypeface = glyphTypeface;
                            }

                            RegisterFont(fontProperties, font);
                        }

                        return font != null;
                    }
                }
            }

            return true;
        }
       
        /// <summary>
        /// Tries to create a new font object based on the given font properties.
        /// </summary>
        /// <param name="fontFamily">The font family.</param>
        /// <param name="font">The font.</param>
        /// <returns>True if the font is created succesfully.</returns>
        public static bool TryCreateFont(FontFamily fontFamily, out FontBase font)
        {
            return TryCreateFont(fontFamily, FontStyles.Normal, FontWeights.Normal, out font);
        }
    }
}
