﻿using System;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    internal abstract class SimpleFont : FontBase
    {
        private readonly PdfProperty<int> firstChar;
        private readonly PdfProperty<int> lastChar;
        private readonly PdfProperty<int[]> widths;
        private readonly PdfProperty<SimpleEncoding> encoding;

        internal SimpleFont(string name, FontSource fontSource)
            : base(name, fontSource)
        {
            this.firstChar = new PdfProperty<int>();
            this.lastChar = new PdfProperty<int>();
            this.widths = new PdfProperty<int[]>();
            this.encoding = new PdfProperty<SimpleEncoding>();
        }

        public PdfProperty<int> FirstChar
        {
            get
            {
                return this.firstChar;
            }
        }

        public PdfProperty<int> LastChar
        {
            get
            {
                return this.lastChar;
            }
        }

        public PdfProperty<int[]> Widths
        {
            get
            {
                return this.widths;
            }
        }

        public PdfProperty<SimpleEncoding> Encoding
        {
            get
            {
                return this.encoding;
            }
        }

        internal override double GetWidth(int charCode)
        {
            if (this.FirstChar.HasValue && this.Widths.HasValue)
            {
                int index = charCode - this.FirstChar.Value;
                
                if (0 <= index && index < this.Widths.Value.Length)
                {
                    return this.Widths.Value[index];
                }
            }

            return base.GetWidth(charCode);
        }
    }
}
