﻿using System;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;
using Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Model.Fonts
{
    internal class Type1Font : SimpleFont
    {
        private Type1FontSource fallbackFontSource;

        public Type1Font(string name, FontSource fontSource)
            : base(name, fontSource)
        {
        }

        internal override FontType Type
        {
            get
            {
                return FontType.Type1;
            }
        }

        internal override FontSource ActualFontSource
        {
            get
            {
                if (this.FontSource == EmptyFontSource.Instance)
                {
                    this.EnsureFallbackFontSource();
                    return this.fallbackFontSource;
                }
                else
                {
                    return this.FontSource;
                }
            }
        }

        internal override void InitializeCoreGlyph(Glyph glyph, CharCode charcode)
        {
            byte b = (byte)charcode.Code;

            FontSource actualFont = this.ActualFontSource;
            CFFFontSource cffFontSource = actualFont as CFFFontSource;

            if (cffFontSource != null)
            {
                InitializeCoreGlyph(glyph, cffFontSource, this.Encoding.Value, b);
            }
            else
            {
                Type1FontSource type1FontSource = (Type1FontSource)actualFont;
                InitializeCoreGlyph(glyph, type1FontSource, this.Encoding.Value, b);
            }
        }

        internal static void InitializeCoreGlyph(Glyph glyph, CFFFontSource fontSource, ISimpleEncoding encoding, byte b)
        {
            string glyphName = encoding == null ? fontSource.GetGlyphName(b) : encoding.GetName(b);
            glyph.Name = glyphName;

            ushort glyphId;
            fontSource.TryGetGlyphId(glyphName, out glyphId);
            glyph.GlyphId = glyphId;
        }

        internal static void InitializeCoreGlyph(Glyph glyph, Type1FontSource fontSource, ISimpleEncoding encoding, byte b)
        {
            string glyphName = encoding == null ? fontSource.GetGlyphName(b) : encoding.GetName(b);
            glyph.Name = glyphName;
            glyph.GlyphId = b;
        }

        private void EnsureFallbackFontSource()
        {
            if (this.fallbackFontSource == null)
            {
                string fontName = this.Name;

                if (FontsManagerBase.IsStandardFontName(fontName))
                {
                    this.fallbackFontSource = FontsRepository.FontsManager.GetStandardFontSource(fontName);
                }
                else
                {
                    this.fallbackFontSource = FontsRepository.FontsManager.GetType1FallbackFontSource(fontName);
                }
            }
        }
    }
}