﻿using System;
using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents arc segment.
    /// </summary>
    public class ArcSegment : PathSegment
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ArcSegment" /> class.
        /// </summary>
        public ArcSegment()
        {
            this.IsLargeArc = false;
            this.SweepDirection = SweepDirection.Clockwise;
        }

        /// <summary>
        /// Gets or sets the point.
        /// </summary>
        /// <value>The point.</value>
        public Point Point { get; set; }

        /// <summary>
        /// Gets or sets the radius X.
        /// </summary>
        /// <value>The radius X.</value>
        public double RadiusX { get; set; }

        /// <summary>
        /// Gets or sets the radius Y.
        /// </summary>
        /// <value>The radius Y.</value>
        public double RadiusY { get; set; }

        /// <summary>
        /// Gets or sets the is large arc.
        /// </summary>
        /// <value>The is large arc.</value>
        public bool IsLargeArc { get; set; }

        /// <summary>
        /// Gets or sets the sweep direction.
        /// </summary>
        /// <value>The sweep direction.</value>
        public SweepDirection SweepDirection { get; set; }

        /// <summary>
        /// Gets or sets the x-axis rotation angle.
        /// </summary>
        /// <value>The x-axis rotation angle.</value>
        public double RotationAngle { get; set; }

        internal override Point LastPoint
        {
            get
            {
                return this.Point;
            }
        }

        internal IEnumerable<BezierSegment> ToBezierSegments(Point previousPoint)
        {
            if (previousPoint.Minus(this.LastPoint).IsZero())
            {
                yield break;
            }

#if NETSTANDARD
            SweepDirection sweepDirection = this.SweepDirection;
#else
            System.Windows.Media.SweepDirection sweepDirection = this.SweepDirection ==
                Graphics.SweepDirection.Clockwise ? System.Windows.Media.SweepDirection.Clockwise : System.Windows.Media.SweepDirection.Counterclockwise;
#endif

            var bezierSegments = ArcHelper.GetEllipticArcCubicBezierApproximation(
                previousPoint, this.LastPoint, new Size(this.RadiusX, this.RadiusY), this.IsLargeArc, sweepDirection, this.RotationAngle);

            foreach (var bezierSegment in bezierSegments)
            {
                yield return new BezierSegment()
                {
                    Point1 = bezierSegment.Item2,
                    Point2 = bezierSegment.Item3,
                    Point3 = bezierSegment.Item4
                };
            }
        }

        internal override Rect GetBounds(Point lastPoint)
        {
            Rect chordBounds = new Rect(lastPoint, this.Point);

            double minX = chordBounds.X;
            double maxX = chordBounds.X + chordBounds.Width;
            double minY = chordBounds.Y;
            double maxY = chordBounds.Y + chordBounds.Height;

            Point p = lastPoint;
            foreach (BezierSegment segment in this.ToBezierSegments(lastPoint))
            {
                PathGeometry.AppendBounds(segment.GetBounds(p), ref minX, ref maxX, ref minY, ref maxY);
                p = segment.LastPoint;
            }

            return new Rect(new Point(minX, minY), new Point(maxX, maxY));
        }
    }
}