﻿using System;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents bezier segment.
    /// </summary>
    public class BezierSegment : PathSegment
    {
        private static readonly Interval ParametricInterval;

        static BezierSegment()
        {
            ParametricInterval = new Interval(0, 1);
        }

        /// <summary>
        /// Gets or sets the first point.
        /// </summary>
        /// <value>The first point.</value>
        public Point Point1 { get; set; }

        /// <summary>
        /// Gets or sets the second point.
        /// </summary>
        /// <value>The second point.</value>
        public Point Point2 { get; set; }

        /// <summary>
        /// Gets or sets the third point.
        /// </summary>
        /// <value>The third point.</value>
        public Point Point3 { get; set; }

        internal override Point LastPoint
        {
            get
            {
                return this.Point3;
            }
        }

        internal override Rect GetBounds(Point lastPoint)
        {
            Point a = (lastPoint.MultiplyBy(-1))
                .Plus(this.Point1.MultiplyBy(3))
                .Plus(this.Point2.MultiplyBy(-3))
                .Plus(this.Point3);

            Point b = (lastPoint.MultiplyBy(3))
                .Plus(this.Point1.MultiplyBy(-6))
                .Plus(this.Point2.MultiplyBy(3));

            Point c = (lastPoint.MultiplyBy(-3))
                .Plus(this.Point1.MultiplyBy(3));

            Point d = lastPoint;

            Interval horizontalBound = MathUtilities.FindLocalMinimumAndMaximum(ParametricInterval, a.X, b.X, c.X, d.X);
            Interval verticalBound = MathUtilities.FindLocalMinimumAndMaximum(ParametricInterval, a.Y, b.Y, c.Y, d.Y);

            return new Rect(new Point(horizontalBound.Start, verticalBound.Start), new Point(horizontalBound.End, verticalBound.End));
        }
    }
}