﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents path.
    /// </summary>
    public class Path : PositionContentElement
    {
        private readonly GeometryPropertiesOwner geometryProperties;

        /// <summary>
        /// Initializes a new instance of the <see cref="Path" /> class.
        /// </summary>
        public Path()  
        {
            this.geometryProperties = new GeometryPropertiesOwner();
        }

        private Path(Path other)
        {
            this.geometryProperties = new GeometryPropertiesOwner(other.geometryProperties);
            this.Geometry = other.Geometry;
        }

        /// <summary>
        /// Gets or sets the geometry.
        /// </summary>
        /// <value>The geometry.</value>
        public GeometryBase Geometry { get; set; }

        /// <summary>
        /// Gets or sets the fill color.
        /// </summary>
        /// <value>The fill color.</value>
        public ColorBase Fill
        {
            get
            {
                return this.GeometryProperties.Fill;
            }
            set
            {
                this.GeometryProperties.Fill = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke color.
        /// </summary>
        /// <value>The stroke color.</value>
        public ColorBase Stroke
        {
            get
            {
                return this.GeometryProperties.Stroke;
            }
            set
            {
                this.GeometryProperties.Stroke = value;
            }
        }

        /// <summary>
        /// Gets or sets if the path figure is filled.
        /// </summary>
        /// <value>If the path figure is filled.</value>
        public bool IsFilled
        {
            get
            {
                return this.GeometryProperties.IsFilled;
            }
            set
            {
                this.GeometryProperties.IsFilled = value;
            }
        }

        /// <summary>
        /// Gets or sets the is stroked.
        /// </summary>
        /// <value>The is stroked.</value>
        public bool IsStroked
        {
            get
            {
                return this.GeometryProperties.IsStroked;
            }
            set
            {
                this.GeometryProperties.IsStroked = value;
            }
        }

        /// <summary>
        /// Gets or sets the width of the stroke line.
        /// </summary>
        /// <value>The width of the stroke line.</value>
        public double StrokeThickness
        {
            get
            {
                return this.GeometryProperties.StrokeThickness;
            }
            set
            {
                this.GeometryProperties.StrokeThickness = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke line cap.
        /// </summary>
        /// <value>The stroke line cap.</value>
        public LineCap StrokeLineCap
        {
            get
            {
                return this.GeometryProperties.StrokeLineCap;
            }
            set
            {
                this.GeometryProperties.StrokeLineCap = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke line join.
        /// </summary>
        /// <value>The stroke line join.</value>
        public LineJoin StrokeLineJoin
        {
            get
            {
                return this.GeometryProperties.StrokeLineJoin;
            }
            set
            {
                this.GeometryProperties.StrokeLineJoin = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke dash array.
        /// </summary>
        /// <value>The stroke dash array.</value>
        public IEnumerable<double> StrokeDashArray
        {
            get
            {
                return this.GeometryProperties.StrokeDashArray;
            }
            set
            {
                this.GeometryProperties.StrokeDashArray = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke dash offset.
        /// </summary>
        /// <value>The stroke dash offset.</value>
        public double StrokeDashOffset
        {
            get
            {
                return this.GeometryProperties.StrokeDashOffset;
            }
            set
            {
                this.GeometryProperties.StrokeDashOffset = value;
            }
        }

        /// <summary>
        /// Gets or sets the miter limit.
        /// </summary>
        /// <value>The miter limit.</value>
        public double? MiterLimit
        {
            get
            {
                return this.GeometryProperties.MiterLimit;
            }
            set
            {
                this.GeometryProperties.MiterLimit = value;
            }
        }

        internal GeometryPropertiesOwner GeometryProperties
        {
            get
            {
                return this.geometryProperties;
            }
        }

        internal override FixedDocumentElementType ElementType
        {
            get { return FixedDocumentElementType.Path; }
        }

        internal override PositionContentElement CreateClonedInstance()
        {
            return new Path(this);
        }
    }
}
