﻿using System;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Collections;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents path figure.
    /// </summary>
    public class PathFigure  
    {
        private readonly PathSegmentCollection segments;

        /// <summary>
        /// Initializes a new instance of the <see cref="PathFigure" /> class.
        /// </summary>
        public PathFigure()
        {
            this.segments = new PathSegmentCollection(); 
        }

        /// <summary>
        /// Gets the path segments collection.
        /// </summary>
        /// <value>The path segments collection.</value>
        public PathSegmentCollection Segments
        {
            get
            {
                return this.segments;
            }
        }

        /// <summary>
        /// Gets or sets the start point.
        /// </summary>
        /// <value>The start point.</value>
        public Point StartPoint { get; set; }

        /// <summary>
        /// Gets or sets if the path figure is closed.
        /// </summary>
        /// <value>If the path figure is closed.</value>
        public bool IsClosed { get; set; }

        internal Rect GetBounds()
        {
            double minX = this.StartPoint.X;
            double maxX = this.StartPoint.X;
            double minY = this.StartPoint.Y;
            double maxY = this.StartPoint.Y;

            Point lastPoint = this.StartPoint;
            foreach (PathSegment segment in this.Segments)
            {
                PathGeometry.AppendBounds(segment.GetBounds(lastPoint), ref minX, ref maxX, ref minY, ref maxY);
                lastPoint = segment.LastPoint;
            }

            return new Rect(new Point(minX, minY), new Point(maxX, maxY));
        }
    }
}
