﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Collections;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents path geometry.
    /// </summary>
    public class PathGeometry : GeometryBase
    {
        private readonly PathFigureCollection figures;

        /// <summary>
        /// Initializes a new instance of the <see cref="PathGeometry" /> class.
        /// </summary>
        public PathGeometry()
        {
            this.figures = new PathFigureCollection();
            this.FillRule = FixedDocumentDefaults.FillRule;
        }

        /// <summary>
        /// Gets the path figures collection.
        /// </summary>
        /// <value>The path figures collection.</value>
        public PathFigureCollection Figures
        {
            get
            {
                return this.figures;
            }
        }

        /// <summary>
        /// Gets or sets the fill rule.
        /// </summary>
        /// <value>The fill rule.</value>
        public FillRule FillRule { get; set; }

        internal static void AppendBounds(Rect rect, ref double minX, ref double maxX, ref double minY, ref double maxY)
        {
            minX = Math.Min(minX, rect.X);
            maxX = Math.Max(maxX, rect.X + rect.Width);
            minY = Math.Min(minY, rect.Y);
            maxY = Math.Max(maxY, rect.Y + rect.Height);
        }

        internal static void AppendBounds(Point point, ref double minX, ref double maxX, ref double minY, ref double maxY)
        {
            minX = Math.Min(minX, point.X);
            maxX = Math.Max(maxX, point.X);
            minY = Math.Min(minY, point.Y);
            maxY = Math.Max(maxY, point.Y);
        }

        /// <summary>
        /// Gets the geometry bounds.
        /// </summary>
        /// <returns></returns>
        protected override Rect GetBounds()
        {
            double minX = double.MaxValue;
            double maxX = double.MinValue;
            double minY = double.MaxValue;
            double maxY = double.MinValue;

            foreach (PathFigure figure in this.Figures)
            {
                AppendBounds(figure.GetBounds(), ref minX, ref maxX, ref minY, ref maxY);
            }

            return new Rect(new Point(minX, minY), new Point(maxX, maxY));
        }
    }
}
