﻿using System;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Graphics
{
    /// <summary>
    /// Represents bezier segment.
    /// </summary>
    public class QuadraticBezierSegment : PathSegment
    {
        private static readonly Interval ParametricInterval;

        static QuadraticBezierSegment()
        {
            ParametricInterval = new Interval(0, 1);
        }
        /// <summary>
        /// Gets or sets the first point.
        /// </summary>
        /// <value>The first point.</value>
        public Point Point1 { get; set; }

        /// <summary>
        /// Gets or sets the second point.
        /// </summary>
        /// <value>The second point.</value>
        public Point Point2 { get; set; }

        internal override Point LastPoint
        {
            get
            {
                return this.Point2;
            }
        }

        internal BezierSegment ToBezierSegment(Point previousPoint)
        {
            Point point1 = previousPoint.Plus(this.Point1.MultiplyBy(2)).MultiplyBy(1.0 / 3);
            Point point2 = this.LastPoint.Plus(this.Point1.MultiplyBy(2)).MultiplyBy(1.0 / 3);

            return new BezierSegment()
            {
                Point1 = point1,
                Point2 = point2,
                Point3 = this.LastPoint
            };
        }

        internal override Rect GetBounds(Point lastPoint)
        {
            Point a = lastPoint.Plus(this.Point1.MultiplyBy(-2)).Plus(this.Point2);
            Point b = lastPoint.MultiplyBy(-2).Plus(this.Point1.MultiplyBy(2));
            Point c = lastPoint;

            Interval horizontalBound = MathUtilities.FindLocalMinimumAndMaximum(ParametricInterval, a.X, b.X, c.X);
            Interval verticalBound = MathUtilities.FindLocalMinimumAndMaximum(ParametricInterval, a.Y, b.Y, c.Y);

            return new Rect(new Point(horizontalBound.Start, verticalBound.Start), new Point(horizontalBound.End, verticalBound.End));
        }
    }
}