﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a choice field.
    /// </summary>
    public abstract class ChoiceField : FormField<VariableContentWidget>
    {
        private readonly ChoiceOptionCollection options;

        /// <summary>
        /// Initializes a new instance of <see cref="ChoiceField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        internal ChoiceField(string fieldName)
            : base(fieldName)
        {
            this.options = new ChoiceOptionCollection(this);
        }

        /// <summary>
        /// Gets the collection of options to choose.
        /// </summary>
        public ChoiceOptionCollection Options
        {
            get
            {
                return this.options;
            }
        }

        /// <summary>
        /// Gets or sets boolean value indicating whether to submit the newly selected value on selection change.
        /// </summary>
        public bool ShouldCommitOnSelectionChange { get; set; }
        
        internal sealed override VariableContentWidget CreateEmptyWidget()
        {
            VariableContentWidget widget = new VariableContentWidget(this);

            return widget;
        }
        internal sealed override void PrepareWidgetAppearancesForExport()
        {
            foreach(VariableContentWidget widget in this.Widgets)
            {
                widget.PrepareAppearancesForExport();
            }
        }

        internal sealed override FormField GetClonedInstanceWithoutWidgetsOverride(RadFixedDocumentCloneContext cloneContext)
        {
            ChoiceField field = this.GetClonedChoiceField(cloneContext);
            field.ShouldCommitOnSelectionChange = this.ShouldCommitOnSelectionChange;

            foreach(ChoiceOption option in this.Options)
            {
                field.Options.Add(cloneContext.GetClonedOption(option));
            }

            return field;
        }

        internal sealed override void InitializeWidgetAppearanceProperties(VariableContentWidget widget)
        {
            widget.AppearanceCharacteristics.Background = RgbColors.White;
        }

        internal abstract ChoiceField GetClonedChoiceField(RadFixedDocumentCloneContext cloneContext);
    }
}
