﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents an option in a choice field.
    /// </summary>
    public sealed class ChoiceOption : FieldOptionBase, IContextClonable<ChoiceOption>
    {
        private string value;
        private string userInterfaceValue;

        /// <summary>
        /// Initializes a new instance of <see cref="ChoiceOption"/> class.
        /// </summary>
        /// <param name="value"></param>
        public ChoiceOption(string value) 
            : base(value)
        {
        }

        /// <summary>
        /// Gets or sets the option value.
        /// </summary>
        public override string Value
        {
            get
            {
                return this.value;
            }
            set
            {
                Guard.ThrowExceptionIfNullOrEmpty(value, "value");

                if (this.value != value)
                {
                    this.value = value;

                    if(this.UserInterfaceValue == null)
                    {
                        this.OnUserInterfaceValueChanged();
                    }
                }
            }
        }

        /// <summary>
        /// Gets or set the value that will be displayed in the UI.
        /// If this property is null then the Value property will be displayed in the UI instead.
        /// </summary>
        public string UserInterfaceValue
        {
            get
            {
                return this.userInterfaceValue;
            }
            set
            {
                if (this.userInterfaceValue != value)
                {
                    this.userInterfaceValue = value;
                    this.OnUserInterfaceValueChanged();
                }
            }
        }
        

        internal event EventHandler UserInterfaceValueChanged;

        private void OnUserInterfaceValueChanged()
        {
            if (this.UserInterfaceValueChanged != null)
            {
                this.UserInterfaceValueChanged(this, EventArgs.Empty);
            }
        }

        ChoiceOption IContextClonable<ChoiceOption>.Clone(RadFixedDocumentCloneContext cloneContext)
        {
            ChoiceOption option = new ChoiceOption(this.Value);
            option.UserInterfaceValue = this.UserInterfaceValue;

            return option;
        }
    }
}
