﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a collection of <see cref="ChoiceOption"/> instances.
    /// </summary>
    public sealed class ChoiceOptionCollection : FieldOptionCollection<ChoiceOption>
    {
        private readonly ChoiceField parent;

        internal ChoiceOptionCollection(ChoiceField choiceField)
            : base()
        {
            this.parent = choiceField;
        }

        internal override void OnBeforeAddOption(ChoiceOption option)
        {
            this.parent.InvalidateWidgetAppearances();

            option.UserInterfaceValueChanged += this.OptionUserInterfaceValueChanged;

            base.OnBeforeAddOption(option);
        }

        internal override void OnBeforeRemoveOptions(IEnumerable<ChoiceOption> options)
        {
            this.parent.InvalidateWidgetAppearances();

            foreach(ChoiceOption option in options)
            {
                option.UserInterfaceValueChanged -= this.OptionUserInterfaceValueChanged;
            }

            base.OnBeforeRemoveOptions(options);
        }

        private void OptionUserInterfaceValueChanged(object sender, EventArgs e)
        {
            this.parent.InvalidateWidgetAppearances();
        }
    }
}
