﻿using System;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a text field which restricts its text input to equally spaced character positions.
    /// </summary>
    public sealed class CombTextBoxField : TextField
    {
        private int maxLengthOfInputCharacters;

        /// <summary>
        /// Initializes a new instance of <see cref="CombTextBoxField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        public CombTextBoxField(string fieldName) 
            : base(fieldName)
        {
        }

        /// <summary>
        /// Gets the field type.
        /// </summary>
        public override FormFieldType FieldType
        {
            get
            {
                return FormFieldType.CombTextBox;
            }
        }

        /// <summary>
        /// Gets or sets the max length of the input characters.
        /// </summary>
        public int MaxLengthOfInputCharacters
        {
            get
            {
                return this.maxLengthOfInputCharacters;
            }
            set
            {
                if(this.maxLengthOfInputCharacters != value)
                {
                    this.InvalidateWidgetAppearances();
                    this.maxLengthOfInputCharacters = value;
                }
            }
        }

        internal sealed override TextField GetClonedTextField(RadFixedDocumentCloneContext cloneContext)
        {
            CombTextBoxField field = new CombTextBoxField(this.Name);
            field.MaxLengthOfInputCharacters = this.MaxLengthOfInputCharacters;

            return field;
        }
    }
}
