﻿using System;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a combobox field.
    /// </summary>
    public sealed class ComboBoxField : ChoiceField
    {
        private ChoiceOption value;

        /// <summary>
        /// Initializes a new instance of <see cref="ComboBoxField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        public ComboBoxField(string fieldName)
            : base(fieldName)
        {
        }

        /// <summary>
        /// Gets the field type.
        /// </summary>
        public override FormFieldType FieldType
        {
            get
            {
                return FormFieldType.ComboBox;
            }
        }

        /// <summary>
        /// Gets or sets boolean value indicating whether combo box should have an additional text box for choice input.
        /// </summary>
        public bool HasEditableTextBox { get; set; }
        
        /// <summary>
        /// Gets or sets boolean value indicating whether the inputted text should be spell checked.
        /// </summary>
        public bool ShouldSpellCheck { get; set; }

        /// <summary>
        /// Gets or set the field value. 
        /// This value may differ from all choice values when it is inputed from the editable  text box.
        /// </summary>
        public ChoiceOption Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value != value)
                {

                    if(this.value != null)
                    {
                        this.value.UserInterfaceValueChanged -= this.ValueUserInterfaceValueChanged;
                    }

                    this.value = value;
                    this.InvalidateWidgetAppearances();

                    if (this.value != null)
                    {
                        this.value.UserInterfaceValueChanged += this.ValueUserInterfaceValueChanged;
                    }
                }
            }
        }

        /// <summary>
        /// Gets or set the default value used when the form is reset to its default values.
        /// </summary>
        public ChoiceOption DefaultValue { get; set; }

        internal sealed override ChoiceField GetClonedChoiceField(RadFixedDocumentCloneContext cloneContext)
        {
            ComboBoxField field = new ComboBoxField(this.Name);
            field.HasEditableTextBox = this.HasEditableTextBox;
            field.ShouldSpellCheck = this.ShouldSpellCheck;
            field.Value = cloneContext.GetClonedOption(this.Value);
            field.DefaultValue = cloneContext.GetClonedOption(this.DefaultValue);

            return field;
        }

        private void ValueUserInterfaceValueChanged(object sender, EventArgs e)
        {
            this.InvalidateWidgetAppearances();
        }
    }
}
