﻿using System;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents the type of a <see cref="FormField"/>.
    /// </summary>
    public enum FormFieldType
    {
        /// <summary>
        /// Represents the PushButton button type, which is used by simple button that may be clicked with the mouse in a PDF viewer.
        /// </summary>
        PushButton,

        /// <summary>
        /// Represents the CheckBox button type, which is used by buttons that may be checked or unchecked.
        /// </summary>
        CheckBox,

        /// <summary>
        /// Represents the RadioButton button type, which is usually used by a group of buttons only one of which may be selected at a time.
        /// </summary>
        RadioButton,

        /// <summary>
        /// Represents the Combs type which restricts the text input to equally spaced character positions.
        /// </summary>
        CombTextBox,

        /// <summary>
        /// Representes the TextBox type which provides a box for the text input.
        /// </summary>
        TextBox,
        
        /// <summary>
        /// Represents the ComboBox choice type.
        /// </summary>
        ComboBox,

        /// <summary>
        /// Represents the ListBox choice type.
        /// </summary>
        ListBox,

        /// <summary>
        /// Represents the Signature form field type.
        /// </summary>
        Signature
    }
}
