﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a button that may be pushed.
    /// </summary>
    public sealed class PushButtonField : FormField<PushButtonWidget>
    {

        /// <summary>
        /// Initializes a new instance of <see cref="PushButtonField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        public PushButtonField(string fieldName)
            : base(fieldName)
        {
        }

        /// <summary>
        /// Gets the field type.
        /// </summary>
        public override FormFieldType FieldType
        {
            get
            {
                return FormFieldType.PushButton;
            }
        }

        internal override PushButtonWidget CreateEmptyWidget()
        {
            return new PushButtonWidget(this);
        }

        internal override void PrepareWidgetAppearancesForExport()
        {
            foreach(PushButtonWidget widget in this.Widgets)
            {
                widget.PrepareAppearancesForExport();
            }
        }

        internal override FormField GetClonedInstanceWithoutWidgetsOverride(RadFixedDocumentCloneContext cloneContext)
        {
            PushButtonField field = new PushButtonField(this.Name);

            return field;
        }

        internal sealed override void InitializeWidgetAppearanceProperties(PushButtonWidget widget)
        {
            widget.AppearanceCharacteristics.Background = FixedDocumentDefaults.PushButtonBackground;
            widget.AppearanceCharacteristics.BorderColor = RgbColors.Black;
            widget.Border = new AnnotationBorder(FixedDocumentDefaults.ButtonWidgetBorderThickness, AnnotationBorderStyle.Beveled, null);
        }
    }
}
