﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a Radio button option.
    /// </summary>
    public class RadioOption : FieldOptionBase, IContextClonable<RadioOption>
    {
        private string value;

        /// <summary>
        /// Initializes a new instance of <see cref="RadioOption"/> class.
        /// </summary>
        /// <param name="value">The option value.</param>
        public RadioOption(string value) 
            : base(value)
        {
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        public override string Value
        {
            get
            {
                return this.value;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");

                this.value = value;
            }
        }

        RadioOption IContextClonable<RadioOption>.Clone(RadFixedDocumentCloneContext cloneContext)
        {
            return new RadioOption(this.Value);
        }

        internal bool Equals(RadioOption other, bool shouldUpdateRadiosInUnison)
        {
            bool result;

            if (shouldUpdateRadiosInUnison)
            {
                result = other != null && other.Value.Equals(this.Value);
            }
            else
            {
                result = other != null && other.Equals(this);
            }

            return result;
        }
    }
}
