﻿using System;

using Telerik.Windows.Documents.Fixed.Model.Annotations;
#if WPF
using Telerik.Windows.Documents.Fixed.Model.DigitalSignatures;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a signature field.
    /// </summary>
    public sealed class SignatureField : FormField<SignatureWidget>
    {
#if WPF
        private Signature signature;
#endif
        /// <summary>
        /// Initializes a new instance of <see cref="SignatureField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        public SignatureField(string fieldName)
            : base(fieldName)
        {
        }

#if WPF
        /// <summary>
        /// Gets the signature.
        /// </summary>
        /// <value>The signature.</value>
        public Signature Signature
        {
            get
            {
                return this.signature;
            }
            set
            {
                if (value != null)
                {
                    if (!string.IsNullOrEmpty(value.Properties.FieldName))
                    {
                        throw new ArgumentException("The new signature value is already associated with another signature field named \"" + value.Properties.FieldName + "\".");
                    }

                    this.signature = value;
                    this.signature.Properties.FieldName = this.Name;
                }
                else
                {
                    this.signature = null;
                }
            }
        }
#endif

        /// <summary>
        /// Gets the field type.
        /// </summary>
        public sealed override FormFieldType FieldType
        {
            get
            {
                return FormFieldType.Signature;
            }
        }

        internal sealed override SignatureWidget CreateEmptyWidget()
        {
            return new SignatureWidget(this);
        }

        internal sealed override void PrepareWidgetAppearancesForExport()
        {
            foreach (SignatureWidget widget in this.Widgets)
            {
                widget.PrepareAppearancesForExport();
            }
        }

        internal override FormField GetClonedInstanceWithoutWidgetsOverride(RadFixedDocumentCloneContext cloneContext)
        {
            SignatureField field = new SignatureField(this.Name);
#if WPF
            field.Signature = this.Signature;
#endif

            return field;
        }

        internal sealed override void InitializeWidgetAppearanceProperties(SignatureWidget widget)
        {
            // Do nothing.
        }
    }
}
