﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a text field.
    /// </summary>
    public abstract class TextField : FormField<VariableContentWidget>
    {
        private string value;

        /// <summary>
        /// Initializes a new instance of <see cref="TextField"/> class. 
        /// </summary>
        /// <param name="fieldName">The field name.</param>
        public TextField(string fieldName)
            : base(fieldName)
        {
        }

        /// <summary>
        /// Gets or sets the field value.
        /// </summary>
        public string Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value != value)
                {
                    this.value = value;
                    this.InvalidateWidgetAppearances();
                }
            }
        }

        /// <summary>
        /// Gets or sets the field default value used when the form is reset to its default values.
        /// </summary>
        public string DefaultValue { get; set; }

        internal sealed override VariableContentWidget CreateEmptyWidget()
        {
            VariableContentWidget widget = new VariableContentWidget(this);
            
            return widget;
        }

        internal sealed override void PrepareWidgetAppearancesForExport()
        {
            foreach(VariableContentWidget widget in this.Widgets)
            {
                widget.PrepareAppearancesForExport();
            }
        }

        internal sealed override FormField GetClonedInstanceWithoutWidgetsOverride(RadFixedDocumentCloneContext cloneContext)
        {
            TextField field = this.GetClonedTextField(cloneContext);
            field.Value = this.Value;
            field.DefaultValue = this.DefaultValue;

            return field;
        }

        internal sealed override void InitializeWidgetAppearanceProperties(VariableContentWidget widget)
        {
            widget.AppearanceCharacteristics.Background = RgbColors.White;
        }

        internal abstract TextField GetClonedTextField(RadFixedDocumentCloneContext cloneContext);
    }
}
