﻿using System;
using System.Collections;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Annotations;

namespace Telerik.Windows.Documents.Fixed.Model.InteractiveForms
{
    /// <summary>
    /// Represents a collection of widgets from specific type.
    /// </summary>
    /// <typeparam name="T">The widget type.</typeparam>
    public class WidgetCollectionBase<T> : IEnumerable<T>
        where T : Widget
    {
        private IWidgetCreator<T> creator;
        private readonly List<T> widgets;

        internal WidgetCollectionBase(IWidgetCreator<T> creator)
        {
            this.widgets = new List<T>();
            this.creator = creator;
        }

        /// <summary>
        /// Gets the count of the widgets in the collection.
        /// </summary>
        public int Count
        {
            get
            {
                return this.widgets.Count;
            }
        }

        /// <summary>
        /// Removes widget from the collection
        /// </summary>
        /// <param name="widget">The widget to remove.</param>
        public void Remove(T widget)
        {
            this.widgets.Remove(widget);
        }

        /// <summary>
        /// Iterates the options in the collection.
        /// </summary>
        /// <returns>The options.</returns>
        public IEnumerator<T> GetEnumerator()
        {
            return this.widgets.GetEnumerator();
        }

        internal T AddEmptyWidget()
        {
            T widget = this.creator.CreateWidget();
            this.widgets.Add(widget);

            return widget;
        }

        internal void AddClonedWidget(Widget clonedWidget)
        {
            T widget = (T)clonedWidget;
            this.widgets.Add(widget);
        }

        /// <summary>
        /// Iterates the options in the collection.
        /// </summary>
        /// <returns>The options.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.widgets.GetEnumerator();
        }
    }
}
