﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Internal.Annotations
{
    internal class MultiStateAppearancesOld : AnnotationAppearancesOld
    {
        private readonly Dictionary<string, SingleStateAppearancesOld> statesToAppearance;
        private string currentState;

        public MultiStateAppearancesOld(string currentState, IEnumerable<KeyValuePair<string, SingleStateAppearancesOld>> statesToAppearances)
        {
            this.statesToAppearance = new Dictionary<string, SingleStateAppearancesOld>();
            
            foreach(KeyValuePair<string, SingleStateAppearancesOld> stateToAppearance in statesToAppearances)
            {
                this.statesToAppearance.Add(stateToAppearance.Key, stateToAppearance.Value);
            }

            this.CurrentState = currentState;
        }

        public string CurrentState
        {
            get
            {
                return this.currentState;
            }
            set
            {
                Guard.ThrowExceptionIfNullOrEmpty(value, "value");

                if(this.currentState != value)
                {
                    this.currentState = value;
                    this.DoOnCurrentStateChanged();
                }
            }
        }

        public IEnumerable<string> States
        {
            get
            {
                return this.statesToAppearance.Keys;
            }
        }

        private SingleStateAppearancesOld CurrentAppearances { get; set; }

        public override Container GetNormalAppearance()
        {
            return this.CurrentAppearances != null ? this.CurrentAppearances.GetNormalAppearance() : null;
        }

        public override Container GetMouseDownAppearance()
        {
            return this.CurrentAppearances != null ? this.CurrentAppearances.GetMouseDownAppearance() : null;
        }

        public override Container GetMouseOverAppearance()
        {
            return this.CurrentAppearances != null ? this.CurrentAppearances.GetMouseOverAppearance() : null;
        }

        private void DoOnCurrentStateChanged()
        {
            SingleStateAppearancesOld current;
            if(this.statesToAppearance.TryGetValue(this.CurrentState, out current))
            {
                this.CurrentAppearances = current;
            }
            else
            {
                this.CurrentAppearances = null;
            }
        }
    }
}
