﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Imaging;
using M = System.Windows.Media;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    internal static class BrushFactory
    {
        private static readonly object lockObject = new object();
        private static readonly Dictionary<Color, SolidColorBrush> solidColorBrushesStore = new Dictionary<Color, SolidColorBrush>();

        internal static SolidColorBrush GetSolidColorBrush(M.Color color)
        {
            Color resultColor = new Color(color.A, color.R, color.G, color.B);

            return BrushFactory.GetSolidColorBrush(resultColor);
        }

        internal static Brush GetBrushWithDifferentAlpha(Brush brush, byte alpha)
        {
            Brush newBrush = brush.Clone();
            newBrush.AlphaConstant = alpha;

            SolidColorBrush solidColorBrush = newBrush as SolidColorBrush;
            if (solidColorBrush != null)
            {
                Color colorKey = Color.FromColor(alpha, solidColorBrush.Color);
                newBrush = BrushFactory.GetSolidColorBrush(colorKey);
            }

            return newBrush;
        }

        internal static SolidColorBrush GetSolidColorBrush(Color color)
        {
            SolidColorBrush result;
            lock (lockObject)
            {
                if (!BrushFactory.solidColorBrushesStore.TryGetValue(color, out result))
                {
                    result = new SolidColorBrush(color);
                    result.AlphaConstant = color.A;

                    BrushFactory.solidColorBrushesStore.Add(result.Color, result);
                }
            }

            return result;
        }
    }
}
