﻿using System;
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Internal.Classes
{
    internal struct CharCodeOld
    {
        public CharCodeOld(byte[] bytes)
        {
            this.bytes = bytes;
            this.intValue = BytesHelper.GetInt(this.bytes);
        }

        public CharCodeOld(byte b)
        {
            this.bytes = new byte[1];
            this.bytes[0] = b;
            this.intValue = b;
        }

        public CharCodeOld(ushort us)
        {
            byte[] tmp = BitConverter.GetBytes(us);
            this.bytes = new byte[tmp.Length];
            int cnt = tmp.Length - 1;
            for (int i = 0; i < tmp.Length; i++)
            {
                this.bytes[cnt - i] = tmp[i];
            }
            this.intValue = us;
        }

        public CharCodeOld(int ii)
        {
            byte[] tmp = BitConverter.GetBytes(ii);
            this.bytes = new byte[tmp.Length];
            int cnt = tmp.Length - 1;
            for (int i = 0; i < tmp.Length; i++)
            {
                this.bytes[cnt - i] = tmp[i];
            }
            this.intValue = ii;
        }

        private readonly byte[] bytes;
        private readonly int intValue;

        public int BytesCount
        {
            get
            {
                if (this.bytes == null)
                {
                    return 0;
                }
                return this.bytes.Length;
            }
        }

        public byte[] Bytes
        {
            get
            {
                return this.bytes;
            }
        }

        public int IntValue
        {
            get
            {
                return this.intValue;
            }
        }

        public bool IsEmpty
        {
            get
            {
                return this.bytes == null;
            }
        }

        private static void InsureCharCodes(CharCodeOld left, CharCodeOld right)
        {
            if (left.bytes == null || right.bytes == null)
            {
                throw new ArgumentException("Bytes cannot be null.");
            }
            if (left.bytes.Length != right.bytes.Length)
            {
                throw new InvalidOperationException("Cannot compare CharCodes with different length.");
            }
        }

        public static CharCodeOld operator ++(CharCodeOld cc)
        {
            byte[] res = (byte[])cc.bytes.Clone();
            int ost = 1;
            int tmp = 0;
            
            for (int i = res.Length - 1; i >= 0; i--)
            {
                tmp = res[i] + ost;
                ost = tmp / 256;
                res[i] = (byte)(tmp % 256);
                if (ost == 0)
                {
                    break;
                }
            }

            if (ost > 0)
            {
                throw new OverflowException();
            }
            
            return new CharCodeOld(res);
        }

        public static bool operator <(CharCodeOld left, CharCodeOld right)
        {
            InsureCharCodes(left, right);
            for (int i = 0; i < left.bytes.Length; i++)
            {
                if (left.bytes[i] > right.bytes[i])
                {
                    return false;
                }
                else if (left.bytes[i] == right.bytes[i] && left.IntValue == right.IntValue)
                {
                    return false;
                }
            }
            return true;
        }

        public static bool operator ==(CharCodeOld left, CharCodeOld right)
        {
            InsureCharCodes(left, right);
            for (int i = 0; i < left.bytes.Length; i++)
            {
                if (left.bytes[i] != right.bytes[i])
                {
                    return false;
                }
            }
            return true;
        }

        public static bool operator >(CharCodeOld left, CharCodeOld right)
        {
            InsureCharCodes(left, right);
            for (int i = 0; i < left.bytes.Length; i++)
            {
                if (left.bytes[i] <= right.bytes[i])
                {
                    return false;
                }
            }
            return true;
        }

        public static bool operator !=(CharCodeOld left, CharCodeOld right)
        {
            return !(left == right);
        }

        public static bool operator <=(CharCodeOld left, CharCodeOld right)
        {
            return left < right || left == right;
        }

        public static bool operator >=(CharCodeOld left, CharCodeOld right)
        {
            return left > right || left == right;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("<");
            for (int i = 0; i < this.bytes.Length; i++)
            {
                sb.AppendFormat("{0:X2}", this.bytes[i]);
            }
            sb.Append("> ");
            sb.Append(this.GetHashCode());
            return sb.ToString();
        }

        public override bool Equals(object obj)
        {
            if (this.bytes == null)
            {
                return false;
            }
            if (obj is CharCodeOld)
            {
                CharCodeOld cc = (CharCodeOld)obj;
                if (this.BytesCount != cc.BytesCount)
                {
                    return false;
                }
                return this == cc;
            }
            return base.Equals(obj);
        }

        public override int GetHashCode()
        {
            return this.IntValue;
        }
    }
}
