﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;

namespace Telerik.Windows.Documents.Fixed.Model.Internal.Collections
{
	internal class GlyphsCollection : IList<GlyphOld>
    {
        #region Constructors

        public GlyphsCollection()
        {
            this.store = new List<GlyphOld>();
        }

        #endregion


        #region Fields

        private readonly List<GlyphOld> store;

        #endregion


        #region Properties

        public GlyphOld this[int index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        #endregion


        #region Metods

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }



        private void Compare(Rect rect, ref double minX, ref double maxX, ref double minY, ref double maxY)
        {
            minX = Math.Min(rect.Left, minX);
            maxX = Math.Max(rect.Right, maxX);
            minY = Math.Min(rect.Top, minY);
            maxY = Math.Max(rect.Bottom, maxY);
        }



        internal Rect GetBoundingRect()
        {
            double minX = double.PositiveInfinity, minY = double.PositiveInfinity, maxX = double.NegativeInfinity, maxY = double.NegativeInfinity;
            foreach (GlyphOld g in this)
            {
                this.Compare(g.BoundingRect, ref minX, ref maxX, ref minY, ref maxY);
            }
            return new Rect(new Point(minX, minY), new Point(maxX, maxY));
        }



        public int IndexOf(GlyphOld item)
        {
            return this.store.IndexOf(item);
        }

        public void Insert(int index, GlyphOld item)
        {
            this.store.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            this.store.RemoveAt(index);
        }

        public void Add(GlyphOld item)
        {
            this.store.Add(item);
        }

        public void Clear()
        {
            this.store.Clear();
        }

        public bool Contains(GlyphOld item)
        {
            return this.store.Contains(item);
        }

        public void CopyTo(GlyphOld[] array, int arrayIndex)
        {
            this.store.CopyTo(array, arrayIndex);
        }

        public bool Remove(GlyphOld item)
        {
            return this.store.Remove(item);
        }

        public IEnumerator<GlyphOld> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        public void AddRange(IEnumerable<GlyphOld> glyphs)
        {
            foreach (GlyphOld g in glyphs)
            {
                this.store.Add(g);
            }
        }

        #endregion
    }
}