﻿using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Fixed.Model.Internal.Collections
{
    /// <summary>
    /// Represents the pages collection.
    /// </summary>
    internal sealed class PagesCollection : IEnumerable<RadFixedPageInternal>
    {
        private readonly List<RadFixedPageInternal> pages;

        internal PagesCollection(IEnumerable<RadFixedPageInternal> pages)
        {
            this.pages = new List<RadFixedPageInternal>(pages);
        }

        public RadFixedPageInternal this[int index]
        {
            get
            {
                return this.pages[index];
            }
        }

        /// <summary>
        /// Gets the pages count.
        /// </summary>
        /// <value>The pages count.</value>
        public int Count
        {
            get
            {
                return this.pages.Count;
            }
        }

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1" /> that can
        /// be used to iterate through the collection.
        /// </returns>
        public IEnumerator<RadFixedPageInternal> GetEnumerator()
        {
            return this.pages.GetEnumerator();
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator" /> object that can be
        /// used to iterate through the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.pages.GetEnumerator();
        }
    }
}
