﻿using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.Internal.Collections;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using Rect = System.Windows.Rect;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    internal class Container : ContentElement
    {
        #region Constructors

        public Container()
        {
            this.Content = new ContentCollection();
        }

        #endregion


        #region Fields


        #endregion


        #region Properties

        public override ContentElementTypeOld Type
        {
            get
            {
                return ContentElementTypeOld.Container;
            }
        }

        public override bool HasChildren
        {
            get
            {
                return true;
            }
        }

        public ContentCollection Content
        {
            get;
            set;
        }

        #endregion


        #region Methods

        private Rect CalculateContentsBoundingRect()
        {
            if (this.Content.Count == 0)
            {
                return Rect.Empty;
            }
            Rect res = this.Content[0].BoundingRect;
            for (int i = 1; i < this.Content.Count; i++)
            {
                res.Union(this.Content[i].BoundingRect);
            }
            return res;
        }

        #endregion


        #region Overrides

        public override IContentElement Clone()
        {
            Container c = new Container();
            c.TransformMatrix = this.TransformMatrix;
            c.ZIndex = this.ZIndex;
            c.Size = this.Size;
            foreach (ContentElement ce in this.Content)
            {
                c.Content.AddChild(ce.Clone());
            }
            if (this.Clip != null)
            {
                c.Clip = this.Clip.Clone();
            }
            return c;
        }

        public override Rect Arrange(Matrix transformMatrix)
        {
            foreach (ContentElement child in this.Content)
            {
                child.Arrange(this.TransformMatrix * transformMatrix);
            }

            this.BoundingRect = this.Clip == null ? this.CalculateContentsBoundingRect() : Helper.GetBoundingRect(this.Clip.GetBoundingRect(), 
               this.TransformMatrix * this.Clip.TransformMatrix * transformMatrix);
            return this.BoundingRect;
        }

        public override System.Collections.Generic.IEnumerable<IContentElement> Children
        {
            get
            {
                return this.Content;
            }
        }

        #endregion
    }
}
