﻿using System;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Utilities;
using FontStyle = System.Windows.FontStyle;
using FontWeight = System.Windows.FontWeight;
using Point = System.Windows.Point;
using Rect = System.Windows.Rect;
using Size = System.Windows.Size;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    internal class GlyphInfo
    {
        private double ascent;
        private double descent;
        private bool isFrozen;
        private ResourceKey key;
        private CharCodeOld charId;
        private string toUnicode;
        private string fontFamily;
        private FontStyle fontStyle;
        private FontWeight fontWeight;
        private bool isBold;
        private bool isItalic;
        private double fontSize;
        private double rise;
        private double charSpacing;
        private double horizontalScaling;
        private double width;
        private Brush fill;
        private Brush stroke;
        private bool isFilled;
        private bool isStroked;
        private Size size;
        private double strokeThickness;
        private double wordSpacing;
        private double textTranslation;

        public GlyphInfo()
        {
            this.Ascent = 1000;
            this.Descent = 0;
        }

        public ResourceKey Key
        {
            get { return this.key; }
            set
            {
                this.ValidateValueChange();

                this.key = value;
            }
        }

        public CharCodeOld CharId
        {
            get { return charId; }
            set
            {
                this.ValidateValueChange();

                charId = value;
            }
        }

        public string ToUnicode
        {
            get { return toUnicode; }
            set
            {
                this.ValidateValueChange();

                toUnicode = value;
            }
        }

        public string FontFamily
        {
            get { return fontFamily; }
            set
            {
                this.ValidateValueChange();

                fontFamily = value;
            }
        }

        public FontStyle FontStyle
        {
            get { return fontStyle; }
            set
            {
                this.ValidateValueChange();

                fontStyle = value;
            }
        }

        public FontWeight FontWeight
        {
            get { return fontWeight; }
            set
            {
                this.ValidateValueChange();

                fontWeight = value;
            }
        }

        public bool IsBold
        {
            get { return isBold; }
            set
            {
                this.ValidateValueChange();

                isBold = value;
            }
        }

        public bool IsItalic
        {
            get { return isItalic; }
            set
            {
                this.ValidateValueChange();

                isItalic = value;
            }
        }

        public double FontSize
        {
            get { return fontSize; }
            set
            {
                this.ValidateValueChange();

                fontSize = value;
            }
        }

        public double Rise
        {
            get { return rise; }
            set
            {
                this.ValidateValueChange();

                rise = value;
            }
        }

        public double CharSpacing
        {
            get { return charSpacing; }
            set
            {

                this.ValidateValueChange();

                charSpacing = value;
            }
        }

        public double WordSpacing
        {
            get { return wordSpacing; }
            set
            {

                this.ValidateValueChange();

                wordSpacing = value;
            }
        }

        public double HorizontalScaling
        {
            get { return horizontalScaling; }
            set
            {
                this.ValidateValueChange();

                horizontalScaling = value;
            }
        }

        public double Width
        {
            get { return width; }
            set
            {

                this.ValidateValueChange();

                width = value;
            }
        }

        public double Ascent
        {
            get { return ascent; }
            set
            {
                this.ValidateValueChange();

                ascent = value;
            }
        }

        public double Descent
        {
            get
            {
                return this.descent;
            }
            set
            {
                this.ValidateValueChange();

                if (value > 0)
                {
                    this.descent = -value;
                }
                else
                {
                    this.descent = value;
                }
            }
        }

        public Brush Fill
        {
            get { return fill; }
            set
            {
                this.ValidateValueChange();
                fill = value;
            }
        }

        public Brush Stroke
        {
            get { return stroke; }
            set
            {
                this.ValidateValueChange();
                stroke = value;
            }
        }

        public bool IsFilled
        {
            get { return isFilled; }
            set
            {
                this.ValidateValueChange();
                isFilled = value;
            }
        }

        public bool IsStroked
        {
            get { return isStroked; }
            set
            {
                this.ValidateValueChange();
                isStroked = value;
            }
        }

        public Size Size
        {
            get { return size; }
            set
            {
                this.ValidateValueChange();
                size = value;
            }
        }

        public bool IsSpace
        {
            get
            {
                return this.CharId.BytesCount == 1 && this.CharId.Bytes[0] == Characters.SingleByteSpace;
            }
        }

        public double StrokeThickness
        {
            get { return strokeThickness; }
            set
            {
                this.ValidateValueChange();
                strokeThickness = value;
            }
        }

        public double ScaledStrokeThickness
        {
            get
            {
                double fontSizeMultiplier = this.FontSize == 0 ? 1 : Math.Abs(FontBaseOld.FontSize / this.FontSize);

                return this.StrokeThickness * fontSizeMultiplier;
            }
        }

        public bool IsFrozen
        {
            get
            {
                return this.isFrozen;
            }
        }

        public double TextTranslation
        {
            get
            {
                return this.textTranslation;
            }
            set
            {
                this.ValidateValueChange();
                this.textTranslation = value;
            }
        }

        public void Freeze()
        {
            this.isFrozen = true;
        }

        public GlyphInfo Clone()
        {
            GlyphInfo glyphInfo = new GlyphInfo();

            glyphInfo.Ascent = this.Ascent;
            glyphInfo.CharSpacing = this.CharSpacing;
            glyphInfo.Fill = this.Fill;
            glyphInfo.Stroke = this.Stroke;
            glyphInfo.Descent = this.Descent;
            glyphInfo.FontFamily = this.FontFamily;
            glyphInfo.FontWeight = this.FontWeight;
            glyphInfo.FontStyle = this.FontStyle;
            glyphInfo.FontSize = this.FontSize;
            glyphInfo.HorizontalScaling = this.HorizontalScaling;
            glyphInfo.Rise = this.Rise;
            glyphInfo.WordSpacing = this.WordSpacing;
            glyphInfo.Width = this.Width;
            glyphInfo.Size = this.Size;
            glyphInfo.ToUnicode = this.ToUnicode;
            glyphInfo.IsFilled = this.IsFilled;
            glyphInfo.IsStroked = this.IsStroked;
            glyphInfo.Key = this.Key;

            return glyphInfo;
        }

        public override string ToString()
        {
            return this.ToUnicode;
        }

        public override bool Equals(object obj)
        {
            bool referenceEquals = object.ReferenceEquals(this, obj);
            if (referenceEquals)
            {
                return referenceEquals;
            }

            GlyphInfo other = obj as GlyphInfo;
            if (other == null)
            {
                return false;
            }

            bool isEqual =
                this.ascent.Equals(other.ascent) &&
                this.descent.Equals(other.descent) &&
                this.isBold.Equals(other.isBold) &&
                this.isItalic.Equals(other.isItalic) &&
                this.fontSize.Equals(other.fontSize) &&
                this.rise.Equals(other.rise) &&
                this.charSpacing.Equals(other.charSpacing) &&
                this.horizontalScaling.Equals(other.horizontalScaling) &&
                this.width.Equals(other.width) &&
                this.isFilled.Equals(other.isFilled) &&
                this.isStroked.Equals(other.isStroked) &&
                this.strokeThickness.Equals(other.strokeThickness) &&
                this.wordSpacing.Equals(other.wordSpacing) &&

                IsObjectEqual(this.key, other.key) &&
                IsObjectEqual(this.charId, other.charId) &&
                IsObjectEqual(this.toUnicode, other.toUnicode) &&
                IsObjectEqual(this.fontFamily, other.fontFamily) &&
                IsObjectEqual(this.fontStyle, other.fontStyle) &&
                IsObjectEqual(this.fontWeight, other.fontWeight) &&
                IsObjectEqual(this.fill, other.fill) &&
                IsObjectEqual(this.stroke, other.stroke) &&
                IsObjectEqual(this.size, other.size);
            return isEqual;
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int hashCode = ObjectExtensions.CombineHashCodes(
                    ascent.GetHashCode(),
                    descent.GetHashCode(),
                    (key != null) ? key.GetHashCode() : 0,
                    charId.GetHashCode(),
                    toUnicode.GetHashCode(),
                    (fontFamily != null) ? fontFamily.GetHashCode() : 0,
                    (fontStyle != null) ? fontStyle.GetHashCode() : 0,
                    (fontWeight != null) ? fontWeight.GetHashCode() : 0,
                    fontSize.GetHashCode(),
                    isBold.GetHashCode(),
                    isItalic.GetHashCode(),
                    rise.GetHashCode(),
                    charSpacing.GetHashCode(),
                    horizontalScaling.GetHashCode(),
                    width.GetHashCode(),
                    (fill != null) ? fill.GetHashCode() : 0,
                    (stroke != null) ? stroke.GetHashCode() : 0,
                    isFilled.GetHashCode(),
                    isStroked.GetHashCode(),
                    (size != null) ? size.GetHashCode() : 0,
                    strokeThickness.GetHashCode(),
                    wordSpacing.GetHashCode());

                return hashCode;
            }
        }

        private void ValidateValueChange()
        {
            if (this.IsFrozen)
            {
                throw new InvalidOperationException("The object is frozen");
            }
        }

        private bool IsObjectEqual<T>(T left, T right)
        {
            bool sameReference = object.ReferenceEquals(left, right);
            if (sameReference)
            {
                return true;
            }

            if (object.ReferenceEquals(left, null))
            {
                return false;
            }

            return left.Equals(right);
        }
    }
}
