﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.Internal.Collections;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Utils;
using System.Collections.Generic;
using Rect = System.Windows.Rect;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    internal class GlyphsLayoutBox : ContentElement
    {
        #region Constructors

        public GlyphsLayoutBox()
        {
            this.Glyphs = new GlyphsCollection();
        }

        #endregion


        #region Properties

        public GlyphsCollection Glyphs
        {
            get;
            set;
        }

        public override bool HasChildren
        {
            get
            {
                return true;
            }
        }

        public override IEnumerable<IContentElement> Children
        {
            get
            {
                return this.Glyphs;
            }
        }

        #endregion


        #region Overrides

        public override IContentElement Clone()
        {
            GlyphsLayoutBox gb = new GlyphsLayoutBox();
            gb.Size = this.Size;
            gb.TransformMatrix = this.TransformMatrix;
            gb.ZIndex = this.ZIndex;
            foreach (GlyphOld g in this.Glyphs)
            {
                gb.Glyphs.Add((GlyphOld)g.Clone());
            }
            return gb;
        }

        public override Rect Arrange(Matrix transformMatrix)
        {
            foreach (GlyphOld g in this.Glyphs)
            {
                g.Arrange(transformMatrix);
            }
            this.BoundingRect = Helper.GetBoundingRect(this.Glyphs.GetBoundingRect(), Matrix.Identity);
            return this.BoundingRect;
        }


        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            foreach (GlyphOld g in this.Glyphs)
            {
                sb.Append(g.ToUnicode);
            }
            return sb.ToString();
        }

        #endregion

        public override ContentElementTypeOld Type
        {
            get { return ContentElementTypeOld.GlyphLayoutBox; }
        }
    }
}
