﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    internal abstract class GradientBrush : PatternBrush, IGradient
    {
        private readonly Point startPoint;
        private readonly Point endPoint;
        private readonly Color startColor;
        private readonly Color endColor;
        private readonly bool extendBefore;
        private readonly bool extendAfter;
        private readonly Color? background;
        private readonly Rect? boundingBox;
        
        public GradientBrush(Color startColor, Color endColor, Point startPoint, Point endPoint, bool extendBefore, bool extendAfter, Color? background, Matrix transform, Rect? boundingBox)
            : base(transform)
        {
            this.startColor = startColor;
            this.endColor = endColor;
            this.startPoint = startPoint;
            this.endPoint = endPoint;
            this.extendBefore = extendBefore;
            this.extendAfter = extendAfter;
            this.background = background;
            this.boundingBox = boundingBox;
        }

        public Rect? BoundingBox
        {
            get
            {
                return this.boundingBox;
            }
        }

        public Point EndPoint
        {
            get
            {
                return this.endPoint;
            }
        }

        public Point StartPoint
        {
            get
            {
                return this.startPoint;
            }
        }

        public GradientStop[] GradientStops
        {
            get;
            set;
        }

        public Color? Background
        {
            get
            {
                return this.background;
            }
        }

        public bool ExtendAfter
        {
            get
            {
                return this.extendAfter;
            }
        }

        public bool ExtendBefore
        {
            get
            {
                return this.extendBefore;
            }
        }

        public Color EndColor
        {
            get
            {
                return this.endColor;
            }
        }

        public Color StartColor
        {
            get
            {
                return this.startColor;
            }
        }

        Color IGradient.StartColor
        {
            get
            {
                return new Color(this.AlphaConstant, this.StartColor.R, this.StartColor.G, this.StartColor.B);
            }
        }

        Color IGradient.EndColor
        {
            get
            {
                return new Color(this.AlphaConstant, this.EndColor.R, this.EndColor.G, this.EndColor.B);
            }
        }

        IEnumerable<IGradientStop> IGradient.GradientStops
        {
            get
            {
                if (this.GradientStops != null)
                {
                    foreach (GradientStop stop in this.GradientStops)
                    {
                        GradientStop stopWithAlpha = new GradientStop(new Color(this.AlphaConstant, stop.Color.R, stop.Color.G, stop.Color.B), stop.Offset);
                        yield return stopWithAlpha;
                    }
                }
            }
        }

        public override Brush Clone()
        {
            GradientBrush brush = this.CloneOverride();
            brush.GradientStops = (GradientStop[])this.GradientStops.Clone();

            return brush;
        }

        protected abstract GradientBrush CloneOverride();
    }
}
