﻿using System;
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utils;
using Telerik.Windows.Documents.Utilities;

#if WINFORMS
using Telerik.Fixed.Legacy;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents a destination on a RadFixedPage.
    /// </summary>
    public class Destination
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Destination" /> class.
        /// </summary>
        /// <param name="page">The page.</param>
        /// <param name="horizontalOffset">The horizontal offset.</param>
        /// <param name="verticalOffset">The vertical offset.</param>
        public Destination(RadFixedPage page, int horizontalOffset, int verticalOffset)
        {
            Guard.ThrowExceptionIfNull(page, "page");

            this.Type = DestinationType.Location;
            this.Page = page;
            this.Left = horizontalOffset / PdfSettings.PpiToDpi;
            this.Top = (page.ActualHeight - verticalOffset) / PdfSettings.PpiToDpi;
        }

        internal Destination()
        {
        }

        /// <summary>
        /// Gets the page.
        /// </summary>
        /// <value>The page.</value>
        public RadFixedPage Page { get; internal set; }

        internal DestinationType Type
        {
            get;
            set;
        }

        internal double? Left
        {
            get;
            set;
        }

        internal double? Right
        {
            get;
            set;
        }

        internal double? Top
        {
            get;
            set;
        }

        internal double? Bottom
        {
            get;
            set;
        }

        internal double? Zoom
        {
            get;
            set;
        }

        internal double GetHorizontalOffset()
        {
            switch (this.Type)
            {
                case DestinationType.Location:
                    if (this.Left.HasValue)
                    {
                        Point point = new Point(this.Left.Value, 0);
                        point = this.Page.TransformMatrix.Transform(point);
                       
                        return point.X;
                    }
                    
                    break;
            }
         
            return -1;
        }

        internal double GetVerticalOffset()
        {
            switch (this.Type)
            {
                case DestinationType.Location:
                    if (this.Top.HasValue)
                    {
                        Point point = new Point(0, this.Top.Value);
                        point = this.Page.TransformMatrix.Transform(point);
                        
                        return point.Y;
                    }
                    
                    break;
            }
         
            return -1;
        }
    }
}
