﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;
using Telerik.Windows.Documents.Fixed.Model.Internal.Collections;
using Rect = System.Windows.Rect;
using Size = System.Windows.Size;

namespace Telerik.Windows.Documents.Fixed.Model.Internal
{
    /// <summary>
    /// Represents a fixed page.
    /// </summary>
    internal class RadFixedPageInternal
    {
        private readonly RadFixedDocumentInternal document;

        internal RadFixedPageInternal(RadFixedDocumentInternal document, RadFixedPage publicPage)
        {
            this.document = document;
            this.PublicPage = publicPage;
        }

        public RadFixedPage PublicPage { get; private set; }

        /// <summary>
        /// Gets a value indicating if the page has content.
        /// </summary>
        /// <value>A value indicating if the page has content.</value>
        public bool HasContent
        {
            get
            {
                return this.Content != null;
            }
        }

        /// <summary>
        /// Gets the document that contains the page.
        /// </summary>
        /// <value>The document.</value>
        public RadFixedDocumentInternal Document
        {
            get
            {
                return this.document;
            }
        }

        internal ContentCollection Content
        {
            get;
            set;
        }

        internal AnnotationCollection Annotations
        {
            get
            {
                return this.PublicPage.Annotations;
            }
        }

        internal Rect CropBox
        {
            get;
            set;
        }

        internal Matrix TransformMatrix
        {
            get;
            set;
        }

        internal Rect BoundingRect
        {
            get;
            set;
        }

        internal void Arrange()
        {
            if (this.Content != null)
            {
                foreach (ContentElement ce in this.Content)
                {
                    ce.Arrange(this.TransformMatrix);
                }
            }
            if (this.Annotations != null)
            {
                foreach (Annotation annot in this.Annotations)
                {
                    annot.Arrange(this.TransformMatrix);
                }
            }
        }

        internal double GetXInDip(double pdfXCoordinate)
        {
            return this.TransformMatrix.Transform(new System.Windows.Point(pdfXCoordinate, 0)).X;
        }

        internal double GetYInDip(double pdfYCoordinate)
        {
            return this.TransformMatrix.Transform(new System.Windows.Point(0, pdfYCoordinate)).Y;
        }
    }
}
