﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents destination.
    /// </summary>
    public abstract class Destination : FixedDocumentElementBase, IContextClonable<Destination>
    {
        /// <summary>
        /// Gets or sets the page.
        /// </summary>
        /// <value>The page.</value>
        public RadFixedPage Page { get; set; }

        /// <summary>
        /// Gets or sets the type of the destination.
        /// </summary>
        /// <value>The type of the destination.</value>
        internal abstract DestinationType DestinationType { get; }

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.Destination;
            }
        }

        internal static Size GetScale(Size actualSize, Size sizeToFitIn)
        {
            double horizontalScale = Destination.GetScale(actualSize.Width, sizeToFitIn.Width);
            double verticalScale = Destination.GetScale(actualSize.Height, sizeToFitIn.Height);

            return new Size(horizontalScale, verticalScale);
        }

        internal static double GetScale(double actualSize, double sizeToFitIn)
        {
            return sizeToFitIn / actualSize;
        }

        internal static Destination CreateDestination(DestinationType type, RadFixedPage page)
        {
            switch (type)
            {
                case DestinationType.BoundingRectangleFit:
                    return new BoundingRectangleFit()
                    {
                        Page = page
                    };

                case DestinationType.BoundingRectangleHorizontalFit:
                    return new BoundingRectangleHorizontalFit()
                    {
                        Page = page
                    };

                case DestinationType.BoundingRectangleVerticalFit:
                    return new BoundingRectangleVerticalFit()
                    {
                        Page = page
                    };

                case DestinationType.Location:
                    return new Location()
                    {
                        Page = page
                    };

                case DestinationType.PageFit:
                    return new PageFit()
                    {
                        Page = page
                    };

                case DestinationType.PageHorizontalFit:
                    return new PageHorizontalFit()
                    {
                        Page = page
                    };

                case DestinationType.PageVerticalFit:
                    return new PageVerticalFit()
                    {
                        Page = page
                    };

                case DestinationType.RectangleFit:
                    return new RectangleFit()
                    {
                        Page = page
                    };

                default:
                    throw new NotSupportedException(string.Format("{0} is not supported destination type.", type.ToString()));
            }
        }

        internal virtual bool TryGetHorizontalOffsetOnPage(Size viewportSize, out double offset)
        {
            offset = 0;
            return false;
        }

        internal virtual bool TryGetVerticalOffsetOnPage(Size viewportSize, out double offset)
        {
            offset = 0;
            return false;
        }

        internal virtual bool TryGetScaleFactor(Size viewportSize, out double scaleFactor)
        {
            scaleFactor = 0;
            return false;
        }

        Destination IContextClonable<Destination>.Clone(RadFixedDocumentCloneContext cloneContext)
        {
            Destination clonedDestination = this.CreateClonedInstance();
            clonedDestination.Page = cloneContext.GetClonedPage(this.Page);

            return clonedDestination;
        }

        internal abstract Destination CreateClonedInstance();
    }
}
