﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents location.
    /// </summary>
    public class Location : Destination
    {
        /// <summary>
        /// Gets or sets the type of the destination.
        /// </summary>
        /// <value>The type of the destination.</value>
        internal override DestinationType DestinationType
        {
            get { return Navigation.DestinationType.Location; }
        }

        /// <summary>
        /// Gets or sets the left.
        /// </summary>
        /// <value>The left.</value>
        public double? Left { get; set; }

        /// <summary>
        /// Gets or sets the top.
        /// </summary>
        /// <value>The top.</value>
        public double? Top { get; set; }

        /// <summary>
        /// Gets or sets the zoom.
        /// </summary>
        /// <value>The zoom.</value>
        public double? Zoom { get; set; }

        internal override bool TryGetScaleFactor(Size viewportSize, out double scaleFactor)
        {
            if (this.Zoom.HasValue && this.Zoom.Value > 0)
            {
                scaleFactor = this.Zoom.Value;
                return true;
            }

            return base.TryGetScaleFactor(viewportSize, out scaleFactor);
        }

        internal override bool TryGetHorizontalOffsetOnPage(Size viewportSize, out double offset)
        {
            if (this.Left.HasValue)
            {
                offset = this.Left.Value;
                return true;
            }

            return base.TryGetHorizontalOffsetOnPage(viewportSize, out offset);
        }

        internal override bool TryGetVerticalOffsetOnPage(Size viewportSize, out double offset)
        {
            if (this.Top.HasValue)
            {
                offset = this.Top.Value;
                return true;
            }

            return base.TryGetVerticalOffsetOnPage(viewportSize, out offset);
        }

        internal override Destination CreateClonedInstance()
        {
            return new Location()
            {
                Left = this.Left,
                Top = this.Top,
                Zoom = this.Zoom
            };
        }
    }
}
