﻿using System;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.EventArguments;

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents a named destination. Named destinations are destinations in the document which can be referred to indirectly by means of a name.
    /// </summary>
    public class NamedDestination : FixedDocumentElementBase, IContextClonable<NamedDestination>
    {
        private string name;

        internal NamedDestination()
        {
        }

        internal event EventHandler<NamedDestinationNameChangedEventArgs> NameChanged;

        /// <summary>
        /// Gets or sets the destination name.
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                if (this.name != value)
                {
                    string oldName = this.name;
                    this.name = value;
                    this.OnNameChanged(this.name, oldName);
                }
            }
        }

        /// <summary>
        /// Gets or sets the Destination of the named destination.
        /// </summary>
        public Destination Destination { get; set; }

        /// <summary>
        /// Gets or sets the GoToAction of the named destination.
        /// </summary>
        public GoToAction GoToAction { get; set; }

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.NamedDestination;
            }
        }

        NamedDestination IContextClonable<NamedDestination>.Clone(RadFixedDocumentCloneContext cloneContext)
        {
            NamedDestination clonedNamedDestination = new NamedDestination();

            clonedNamedDestination.Name = this.Name;
            if (this.Destination != null)
            {
                clonedNamedDestination.Destination = cloneContext.GetClonedDestination(this.Destination);
            }
            if (this.GoToAction != null)
            {
                clonedNamedDestination.GoToAction = (GoToAction)this.GoToAction.Clone(cloneContext);
            }

            return clonedNamedDestination;
        }

        private void OnNameChanged(string newValue, string oldValue)
        {
            if (this.NameChanged != null)
            {
                this.NameChanged(this, new NamedDestinationNameChangedEventArgs(newValue, oldValue));
            }
        }
    }
}
