﻿using System;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{ 
    /// <summary>
    /// Represents page horizontal fit.
    /// </summary>
    public class PageHorizontalFit : Destination
    { 
        /// <summary>
        /// Gets or sets the type of the destination.
        /// </summary>
        /// <value>The type of the destination.</value>
        internal override DestinationType DestinationType
        {
            get { return Navigation.DestinationType.PageHorizontalFit; }
        }
         
        /// <summary>
        /// Gets or sets the top.
        /// </summary>
        /// <value>The top.</value>
        public double? Top { get;set; }

        internal override bool TryGetScaleFactor(Size viewportSize, out double scaleFactor)
        {
            return PageHorizontalFit.TryGetScaleFactor(this.Page, viewportSize, out scaleFactor);
        }

        internal override bool TryGetHorizontalOffsetOnPage(Size viewportSize, out double offset)
        {
            return PageHorizontalFit.TryGetHorizontalOffsetOnPage(out offset);
        }

        internal override bool TryGetVerticalOffsetOnPage(Size viewportSize, out double offset)
        {
            return PageHorizontalFit.TryGetVerticalOffsetOnPage(this.Top, out offset);
        }

        internal static bool TryGetScaleFactor(RadFixedPage page, Size viewportSize, out double scaleFactor)
        {
            scaleFactor = Destination.GetScale(page.Size.Width, viewportSize.Width);
            return true;
        }

        internal static bool TryGetHorizontalOffsetOnPage(out double offset)
        {
            offset = 0;
            return true;
        }

        internal static bool TryGetVerticalOffsetOnPage(double? top, out double offset)
        {
            if (top.HasValue)
            {
                offset = top.Value;
                return true;
            }

            offset = 0;
            return false;
        }

        internal override Destination CreateClonedInstance()
        {
            return new PageHorizontalFit() { Top = this.Top };
        }
    }
}
