﻿using System;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents vertical page fit.
    /// </summary>
    public class PageVerticalFit : Destination
    {  
        /// <summary>
        /// Gets or sets the type of the destination.
        /// </summary>
        /// <value>The type of the destination.</value>
        internal override DestinationType DestinationType
        {
            get { return Navigation.DestinationType.PageVerticalFit; }
        }
         
        /// <summary>
        /// Gets or sets the left.
        /// </summary>
        /// <value>The left.</value>
        public double? Left { get;set; }

        internal override bool TryGetScaleFactor(Size viewportSize, out double scaleFactor)
        {
            return PageVerticalFit.TryGetScaleFactor(this.Page, viewportSize, out scaleFactor);
        }

        internal override bool TryGetHorizontalOffsetOnPage(Size viewportSize, out double offset)
        {
            return PageVerticalFit.TryGetHorizontalOffsetOnPage(this.Left, out offset);
        }

        internal override bool TryGetVerticalOffsetOnPage(Size viewportSize, out double offset)
        {
            return PageVerticalFit.TryGetVerticalOffsetOnPage(out offset);
        }

        internal static bool TryGetHorizontalOffsetOnPage(double? left, out double offset)
        {
            if (left.HasValue)
            {
                offset = left.Value;
                return true;
            }

            offset = 0;
            return false;
        }

        internal static bool TryGetVerticalOffsetOnPage(out double offset)
        {
            offset = 0;
            return true;
        }

        internal static bool TryGetScaleFactor(RadFixedPage page, Size viewportSize, out double scaleFactor)
        {
            scaleFactor = Destination.GetScale(page.Size.Height, viewportSize.Height);
            return true;
        }

        internal override Destination CreateClonedInstance()
        {
            return new PageVerticalFit() { Left = this.Left };
        }
    }
}
