﻿using System;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Navigation
{
    /// <summary>
    /// Represents rectangle fit.
    /// </summary>
    public class RectangleFit : Destination
    { 
        /// <summary>
        /// Gets or sets the type of the destination.
        /// </summary>
        /// <value>The type of the destination.</value>
        internal override DestinationType DestinationType
        {
            get { return Navigation.DestinationType.RectangleFit; }
        }
         
        /// <summary>
        /// Gets or sets the left.
        /// </summary>
        /// <value>The left.</value>
        public double? Left { get;set; }
         
        /// <summary>
        /// Gets or sets the bottom.
        /// </summary>
        /// <value>The bottom.</value>
        public double? Bottom { get; set; }

        /// <summary>
        /// Gets or sets the top.
        /// </summary>
        /// <value>The top.</value>
        public double? Top { get; set; }

        /// <summary>
        /// Gets or sets the right.
        /// </summary>
        /// <value>The right.</value>
        public double? Right { get; set; }

        private bool IsValid
        {
            get
            {
                return this.Left.HasValue && this.Bottom.HasValue && this.Top.HasValue && this.Right.HasValue;
            }
        }

        private Rect RectangleToFit
        {
            get
            {
                Guard.ThrowExceptionIfFalse(this.IsValid, "IsValid");

                double left = Math.Min(this.Left.Value, this.Right.Value);
                double top = Math.Min(this.Top.Value, this.Bottom.Value);
                double width = Math.Abs(this.Left.Value - this.Right.Value);
                double height = Math.Abs(this.Top.Value - this.Bottom.Value);

                return new Rect(left, top, width, height);
            }
        }

        internal override bool TryGetScaleFactor(Size viewportSize, out double scaleFactor)
        {
            if (this.IsValid)
            {
                Rect rect = this.RectangleToFit;
                Size scaleSize = Destination.GetScale(new Size(rect.Width, rect.Height), viewportSize);
                scaleFactor = Math.Min(scaleSize.Width, scaleSize.Height);

                return true;
            }

            return base.TryGetScaleFactor(viewportSize, out scaleFactor);
        }

        internal override bool TryGetHorizontalOffsetOnPage(Size viewportSize, out double offset)
        {
            if (this.IsValid)
            {
                offset = this.RectangleToFit.Left;

                return true;
            }

            return base.TryGetHorizontalOffsetOnPage(viewportSize, out offset);
        }

        internal override bool TryGetVerticalOffsetOnPage(Size viewportSize, out double offset)
        {
            if (this.IsValid)
            {
                offset = this.RectangleToFit.Top;

                return true;
            }

            return base.TryGetVerticalOffsetOnPage(viewportSize, out offset);
        }

        internal override Destination CreateClonedInstance()
        {
            return new RectangleFit()
            {
                Left = this.Left,
                Top = this.Top,
                Right = this.Right,
                Bottom = this.Bottom
            };
        }
    }
}
