﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Model.Objects
{
    /// <summary>
    /// Represents form element which allows to reuse PDF content by using <see cref="Resources.FormSource"/>.
    /// </summary>
    public class Form : PositionContentElement
    {
        private double? width;
        private double? height;

        /// <summary>
        /// Gets or sets the form source.
        /// </summary>
        /// <value>The form source.</value>
        public FormSource FormSource { get; set; }

        /// <summary>
        /// Gets or sets the width.
        /// </summary>
        /// <value>The width.</value>
        public double Width
        {
            get
            {
                if (this.width.HasValue)
                {
                    return this.width.Value;
                }

                if (this.FormSource != null)
                {
                    return this.FormSource.Size.Width;
                }

                return 0;
            }
            set
            {
                this.width = value;
            }
        }

        /// <summary>
        /// Gets or sets the height.
        /// </summary>
        /// <value>The height.</value>
        public double Height
        {
            get
            {
                if (this.height.HasValue)
                {
                    return this.height.Value;
                }

                if (this.FormSource != null)
                {
                    return this.FormSource.Size.Height;
                }

                return 0;
            }
            set
            {
                this.height = value;
            }
        }

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.Form;
            }
        }

        internal override PositionContentElement CreateClonedInstance()
        {
            Form clonedForm = new Form();
            clonedForm.width = this.width;
            clonedForm.height = this.height;
            clonedForm.FormSource = this.FormSource;

            return clonedForm;
        }
    }
}
