﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
#if !NETSTANDARD
using System.Windows.Media.Imaging;
#endif
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Objects
{
    /// <summary>
    /// Represents image element.
    /// </summary>
    public class Image : PositionContentElement
    {
        private double? width;
        private double? height;

        /// <summary>
        /// Initializes a new instance of the <see cref="Image" /> class.
        /// </summary>
        public Image()
        {
        }

        /// <summary>
        /// Gets or sets the image source.
        /// </summary>
        /// <value>The image source.</value>
        public ImageSource ImageSource
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the width.
        /// </summary>
        /// <value>The width.</value>
        public double Width
        {
            get
            {
                if (this.width.HasValue)
                {
                    return this.width.Value;
                }

                if (this.ImageSource != null)
                {
                    return this.ImageSource.Width;
                }

                return 0;
            }
            set
            {
                this.width = value;
            }
        }

        /// <summary>
        /// Gets or sets the height.
        /// </summary>
        /// <value>The height.</value>
        public double Height
        {
            get
            {
                if (this.height.HasValue)
                {
                    return this.height.Value;
                }

                if (this.ImageSource != null)
                {
                    return this.ImageSource.Height;
                }

                return 0;
            }
            set
            {
                this.height = value;
            }
        }

        internal ColorBase StencilColor { get; set; }

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.Image;
            }
        }

#if !NETSTANDARD
        /// <summary>
        /// Creates <see cref="BitmapSource"/> from the image element.
        /// </summary>
        /// <returns>The <see cref="BitmapSource"/> instance.</returns>
        public BitmapSource GetBitmapSource()
        {
            BitmapSource result = null;

            if (this.ImageSource != null)
            {
                if (this.StencilColor == null)
                {
                    result = this.ImageSource.GetBitmapSource();
                }
                else
                {
                    result = this.ImageSource.GetBitmapSource(this.StencilColor);
                }
            }

            return result;
        }
#endif

        internal override PositionContentElement CreateClonedInstance()
        {
            Image clonedImage = new Image();
            clonedImage.width = this.width;
            clonedImage.height = this.height;
            clonedImage.ImageSource = this.ImageSource;

            return clonedImage;
        }
    }
}
