﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using Telerik.Windows.Documents.Fixed.Model.Navigation;

namespace Telerik.Windows.Documents.Fixed.Model
{
    internal class RadFixedDocumentCloneContext
    {
        private readonly Dictionary<RadFixedPage, RadFixedPage> pageToClonedPage;
        private readonly Dictionary<Clipping, Clipping> clippingToClonedClipping;
        private readonly Dictionary<Destination, Destination> destinationToClonedDestination;
        private readonly Dictionary<RadioOption, RadioOption> radioOptionToClonedRadioOption;
        private readonly Dictionary<ChoiceOption, ChoiceOption> choiceOptionToClonedChoiceOption;
        private readonly Dictionary<Annotation, Annotation> annotationToClonedAnnotation;
        private readonly Dictionary<FormField, FormField> formFieldToClonedFormField;
        private readonly Dictionary<BookmarkItem, BookmarkItem> bookmarkToClonedBookmark;
        private readonly Dictionary<NamedDestination, NamedDestination> namedDestinationToClonedNamedDestination;

        public RadFixedDocumentCloneContext()
        {
            this.pageToClonedPage = new Dictionary<RadFixedPage, RadFixedPage>();
            this.clippingToClonedClipping = new Dictionary<Clipping, Clipping>();
            this.destinationToClonedDestination = new Dictionary<Destination, Destination>();
            this.radioOptionToClonedRadioOption = new Dictionary<RadioOption, RadioOption>();
            this.choiceOptionToClonedChoiceOption = new Dictionary<ChoiceOption, ChoiceOption>();
            this.annotationToClonedAnnotation = new Dictionary<Annotation, Annotation>();
            this.formFieldToClonedFormField = new Dictionary<FormField, FormField>();
            this.bookmarkToClonedBookmark = new Dictionary<BookmarkItem, BookmarkItem>();
            this.namedDestinationToClonedNamedDestination = new Dictionary<NamedDestination, NamedDestination>();
        }

        public RadFixedPage GetClonedPage(RadFixedPage originalPage)
        {
            return this.GetClonedObject(originalPage, this.pageToClonedPage);
        }

        public Clipping GetClonedClipping(Clipping originalClipping)
        {
            return this.GetClonedObject(originalClipping, this.clippingToClonedClipping);
        }

        public Destination GetClonedDestination(Destination originalDestination)
        {
            return this.GetClonedObject(originalDestination, this.destinationToClonedDestination);
        }

        internal NamedDestination GetClonedNamedDestination(NamedDestination originalDestination)
        {
            return this.GetClonedObject(originalDestination, this.namedDestinationToClonedNamedDestination);
        }

        internal Annotation GetClonedAnnotation(Annotation originalAnnotation)
        {
            return this.GetClonedObject(originalAnnotation, this.annotationToClonedAnnotation);
        }

        internal ChoiceOption GetClonedOption(ChoiceOption option)
        {
            return this.GetClonedObject(option, this.choiceOptionToClonedChoiceOption);
        }

        internal RadioOption GetClonedOption(RadioOption option)
        {
            return this.GetClonedObject(option, this.radioOptionToClonedRadioOption);
        }

        internal BookmarkItem GetClonedBookmark(BookmarkItem bookmark)
        {
            return this.GetClonedObject(bookmark, this.bookmarkToClonedBookmark);
        }

        internal FormField GetClonedField(FormField field)
        {
            FormField clonedField;
            if (!this.formFieldToClonedFormField.TryGetValue(field, out clonedField))
            {
                clonedField = field.GetClonedInstanceWithoutWidgets(this);
                this.formFieldToClonedFormField.Add(field, clonedField);
            }

            return clonedField;
        }

        private T GetClonedObject<T>(T originalObject, Dictionary<T, T> objectToClonedObject)
            where T : class, IContextClonable<T>
        {
            if (originalObject == null)
            {
                return null;
            }

            T clonedObject;

            if (!objectToClonedObject.TryGetValue(originalObject, out clonedObject))
            {
                clonedObject = originalObject.Clone(this);
                objectToClonedObject.Add(originalObject, clonedObject);
            }

            return clonedObject;
        }
    }
}
