﻿using Telerik.Windows.Documents.Core.Imaging;

namespace Telerik.Windows.Documents.Fixed.Model.Resources
{
    internal class BgraPixelContainer : PixelContainer
    {
        private const int BgraComponentsPerColor = 4;

        private int index;

        public override int PixelCount
        {
            get
            {
                return this.Array.Length / BgraComponentsPerColor;
            }
        }

        public override ImageDecodedDataType DataType
        {
            get
            {
                return ImageDecodedDataType.Bgra;
            }
        }

        public BgraPixelContainer(int pixelCount)
            : base(pixelCount * BgraComponentsPerColor)
        {
            this.index = 0;
        }

        public override void SetColorToIndex(Color color, int index)
        {
            this.Array[index * BgraComponentsPerColor] = color.B;
            this.Array[(index * BgraComponentsPerColor) + 1] = color.G;
            this.Array[(index * BgraComponentsPerColor) + 2] = color.R;
            this.Array[(index * BgraComponentsPerColor) + 3] = color.A;
        }

        public override Color GetColorFromIndex(int index)
        {
            return new Color(
                this.Array[(index * BgraComponentsPerColor) + 3],
                this.Array[(index * BgraComponentsPerColor) + 2],
                this.Array[(index * BgraComponentsPerColor) + 1],
                this.Array[index * BgraComponentsPerColor]);
        }

        public override void Add(Color color)
        {
            this.Add(color.B);
            this.Add(color.G);
            this.Add(color.R);
            this.Add(color.A);
        }

        private void Add(byte item)
        {
            this.Array[this.index] = item;
            this.index++;
        }
    }
}
