﻿using System;
using System.Windows;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Resources
{
    /// <summary>
    /// Represents a PDF form source. This class owns a <see cref="ContentElementCollection"/> and
    /// allows reuse of PDF content by using the same <see cref="FormSource"/> on different <see cref="Form"/> class instances.
    /// </summary>
    public class FormSource : IContentRootElement, IInstanceIdOwner
    {
        private readonly int id;
        private readonly ContentElementCollection content;

        /// <summary>
        /// Creates new instance of FormSource.
        /// </summary>
        public FormSource()
        {
            this.id = InstanceIdGenerator.GetNextId();
            this.content = new ContentElementCollection(this);
            this.Matrix = Matrix.Identity;
            this.BoundingBox = new Rect();
        }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <value>The content.</value>
        public ContentElementCollection Content
        {
            get
            {
                return this.content;
            }
        }

        /// <summary>
        /// Gets or sets the size of the <see cref="FormSource"/>.
        /// </summary>
        public Size Size
        {
            get
            {
                return new Size(this.BoundingBox.Width, this.BoundingBox.Height);
            }
            set
            {
                this.BoundingBox = new Rect(this.BoundingBox.X, this.BoundingBox.Y, value.Width, value.Height);
            }
        }

        /// <summary>
        /// Indicates whether the IContentRootElement supports annotations.
        /// </summary>
        public bool SupportsAnnotations
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the annotation collection of the <see cref="IContentRootElement"/>.
        /// </summary>
        public AnnotationCollection Annotations
        {
            get
            {
                throw new NotSupportedException();
            }
        }

        /// <summary>
        /// Gets the parent of the <see cref="IContentRootElement"/>.
        /// </summary>
        public IFixedDocumentElement Parent
        {
            get
            {
                return null;
            }
        }

        /// <summary>
        /// Gets or sets the matrix which maps the form space into user space.
        /// </summary>
        internal Matrix Matrix { get; set; }

        internal Rect BoundingBox { get; set; }

        int IInstanceIdOwner.InstanceId
        {
            get
            {
                return this.id;
            }
        }
    }
}
