﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging;

namespace Telerik.Windows.Documents.Fixed.Model.Resources
{
    internal class IndexedPixelContainer : PixelContainer
    {
        private readonly Palette palette;
        private int arrayIndex;

        public override ImageDecodedDataType DataType
        {
            get
            {
                return ImageDecodedDataType.Indexed;
            }
        }

        public override int PixelCount
        {
            get
            {
                return this.Array.Length;
            }
        }

        public IEnumerable<Color> Palette
        {
            get
            {
                return this.palette.Colors;
            }
        }

        public IndexedPixelContainer(int pixelCount, Color[] palette) 
            : base(pixelCount)
        {
            this.palette = new Palette(palette);
            this.arrayIndex = 0;
        }

        public override void Add(Color color)
        {
            byte index = this.GetIndexOfColorInPalette(color);
            this.Array[this.arrayIndex] = index;
            this.arrayIndex++;
        }

        public override Color GetColorFromIndex(int index)
        {
            return this.GetColorFromPalette(this.Array[index]);
        }

        public override void SetColorToIndex(Color color, int index)
        {
            this.Array[index] = this.GetIndexOfColorInPalette(color);
        }

        private byte GetIndexOfColorInPalette(Color color)
        {
            return (byte)this.palette.GetIndexOfColor(color);
        }

        private Color GetColorFromPalette(byte index)
        {
            return this.palette.GetColorAtIndex(index);
        }
    }
}
