﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.Model.Resources
{
    internal class MonochromePixelContainer : PixelContainer
    {
        public static readonly Color DefaultFirstColor = Color.Black;
        public static readonly Color DefaultSecondColor = Color.White;

        private readonly Color firstColor;
        private readonly Color secondColor;

        private readonly int width;
        private readonly int height;

        private readonly ImageBitWriter writer;
        private readonly ImageBitReader reader;

        public override int PixelCount
        {
            get
            {
                return this.width * height;
            }
        }

        public IEnumerable<Color> Palette
        {
            get
            {
                yield return firstColor;
                yield return secondColor;
            }
        }

        public override ImageDecodedDataType DataType
        {
            get
            {
                return ImageDecodedDataType.Monochrome;
            }
        }

        public MonochromePixelContainer(int width, int height, Color firstColor, Color secondColor) 
            : base(BitWriter.CalculateNumberOfBytes(width, 1) * height)
        {
            this.width = width;
            this.height = height;
            this.firstColor = firstColor;
            this.secondColor = secondColor;

            this.reader = new ImageBitReader(this.Array, 1, width);
            this.writer = new ImageBitWriter(this.Array, 1, width);
        }

        public override Color GetColorFromIndex(int index)
        {
            int bit = this.reader.GetBitAt(index);
            Color result = bit == 0 ? this.firstColor : this.secondColor;

            return result;
        }

        public override void SetColorToIndex(Color color, int index)
        {
            int colorValue = this.GetMonochromeColorValue(color);
            this.writer.WriteBitsToIndex(colorValue, index);
        }

        public override void Add(Color color)
        {
            int colorValue = this.GetMonochromeColorValue(color);
            this.writer.Write(colorValue);
        }

        private int GetMonochromeColorValue(Color color)
        {
            return color == this.firstColor ? 0 : 1;
        }
    }
}
