﻿using Telerik.Windows.Documents.Core.Imaging;

namespace Telerik.Windows.Documents.Fixed.Model.Resources
{
    internal static class PixelContainerFactory
    {
#if !SILVERLIGHT
        public static IPixelContainersCreator PixelContainersCreator = new DefaultPixelContainersCreator();
#endif

        public static PixelContainer GetRgbImagePixelContainer(int width, int height)
        {
            return GetBgraPixelContainer(width, height);
        }

        public static PixelContainer GetCmykImagePixelContainer(int width, int height)
        {
            return GetBgraPixelContainer(width, height);
        }

        public static PixelContainer GetDeviceNImagePixelContainer(int width, int height)
        {
            return GetBgraPixelContainer(width, height);
        }

        public static PixelContainer GetSeparationImagePixelContainer(int width, int height)
        {
            return GetBgraPixelContainer(width, height);
        }

        public static PixelContainer GetMaskedImagePixelContainer(int width, int height)
        {
            return GetBgraPixelContainer(width, height);
        }

        public static PixelContainer GetStencilImagePixelContainer(int width, int height, Color firstColor, Color secondColor)
        {
#if !SILVERLIGHT
            return PixelContainersCreator.CreateMonochromeContainer(width, height, firstColor, secondColor);
#else
            return new PixelContainer(height * width);
#endif
        }

        public static PixelContainer GetGrayImagePixelContainer(int bitsPerComponent, bool hasMask, int width, int height)
        {
            PixelContainer result;

#if !SILVERLIGHT
            if (hasMask)
            {
                result = PixelContainersCreator.CreateBgraContainer(width, height);
            }
            else if (bitsPerComponent == 1)
            {
                result = PixelContainersCreator.CreateMonochromeContainer(width, height, MonochromePixelContainer.DefaultFirstColor, MonochromePixelContainer.DefaultSecondColor);
            }
            else
            {
                result = PixelContainersCreator.CreateGrayContainer(width, height);
            }

#else
            result = new PixelContainer(height * width);
#endif

            return result;
        }

        public static PixelContainer GetIndexedImagePixelContainer(bool hasMask, int width, int height, Color[] palette)
        {
            PixelContainer result;

#if !SILVERLIGHT
            if (hasMask)
            {
                result = PixelContainersCreator.CreateBgraContainer(width, height);
            }
            else
            {
                result = PixelContainersCreator.CreateIndexedContainer(width, height, palette);
            }
#else
            result = new PixelContainer(height * width);
#endif

            return result;
        }

        private static PixelContainer GetBgraPixelContainer(int width, int height)
        {
#if !SILVERLIGHT
            return PixelContainersCreator.CreateBgraContainer(width, height);
#else
            return new PixelContainer(height * width);
#endif
        }
    }
}
