﻿using System;
using System.Collections.Generic;
using System.Linq;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Graphics;

namespace Telerik.Windows.Documents.Fixed.Model.Text
{
    /// <summary>
    /// Represents text segment.
    /// </summary>
    public class TextFragment : PositionContentElement
    {
        private readonly TextPropertiesOwner textPropertiesOwner;
        
        /// <summary>
        /// Initializes a new instance of the <see cref="TextFragment" /> class.
        /// </summary>
        public TextFragment()
        {
            this.textPropertiesOwner = new TextPropertiesOwner();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TextFragment" /> class.
        /// </summary>
        /// <param name="text">The text.</param>
        public TextFragment(string text)
            : this()
        {
            this.Text = text;
        }

        internal TextFragment(TextFragment other, string text)
            : this(other)
        {
            this.Text = text;
        }

        internal TextFragment(TextFragment other)
            : this(other.TextProperties)
        {
            this.Clipping = other.Clipping;
            this.Position = other.Position;
        }

        internal TextFragment(TextPropertiesOwner otherProperties)
        {
            this.textPropertiesOwner = new TextPropertiesOwner(otherProperties);
        }

        /// <summary>
        /// Gets or sets the character spacing.
        /// </summary>
        /// <value>The character spacing.</value>
        public double? CharacterSpacing
        {
            get
            {
                return this.textPropertiesOwner.CharacterSpacing;
            }
            set
            {
                this.textPropertiesOwner.CharacterSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the word spacing.
        /// </summary>
        /// <value>The word spacing.</value>
        public double? WordSpacing
        {
            get
            {
                return this.textPropertiesOwner.WordSpacing;
            }
            set
            {
                this.textPropertiesOwner.WordSpacing = value;
            }
        }

        /// <summary>
        /// Gets or sets the horizontal scaling.
        /// </summary>
        /// <value>The horizontal scaling.</value>
        public double? HorizontalScaling
        {
            get
            {
                return this.textPropertiesOwner.HorizontalScaling;
            }
            set
            {
                this.textPropertiesOwner.HorizontalScaling = value;
            }
        }

        /// <summary>
        /// Gets or sets the font.
        /// </summary>
        /// <value>The font.</value>
        public FontBase Font
        {
            get
            {
                return this.textPropertiesOwner.Font;
            }
            set
            {
                this.textPropertiesOwner.Font = value;
            }
        }

        /// <summary>
        /// Gets or sets the size of the font.
        /// </summary>
        /// <value>The size of the font.</value>
        public double FontSize
        {
            get
            {
                return this.textPropertiesOwner.FontSize;
            }
            set
            {
                this.textPropertiesOwner.FontSize = value;
            }
        }

        /// <summary>
        /// Gets or sets the text rise.
        /// </summary>
        /// <value>The text rise.</value>
        public double? TextRise
        {
            get
            {
                return this.textPropertiesOwner.TextRise;
            }
            set
            {
                this.textPropertiesOwner.TextRise = value;
            }
        }

        /// <summary>
        /// Gets or sets the rendering mode.
        /// </summary>
        /// <value>The rendering mode.</value>
        public RenderingMode RenderingMode
        {
            get
            {
                return this.textPropertiesOwner.RenderingMode;
            }
            set
            {
                this.textPropertiesOwner.RenderingMode = value;               
            }
        }

        /// <summary>
        /// Gets or sets the fill color.
        /// </summary>
        /// <value>The fill color.</value>
        public ColorBase Fill
        {
            get
            {
                return this.textPropertiesOwner.Fill;
            }
            set
            {
                this.textPropertiesOwner.Fill = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke color.
        /// </summary>
        /// <value>The stroke color.</value>
        public ColorBase Stroke
        {
            get
            {
                return this.textPropertiesOwner.Stroke;
            }
            set
            {
                this.textPropertiesOwner.Stroke = value;
            }
        }

        /// <summary>
        /// Gets or sets the width of the stroke line.
        /// </summary>
        /// <value>The width of the stroke line.</value>
        public double StrokeThickness
        {
            get
            {
                return this.textPropertiesOwner.StrokeThickness;
            }
            set
            {
                this.textPropertiesOwner.StrokeThickness = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke line cap.
        /// </summary>
        /// <value>The stroke line cap.</value>
        public LineCap StrokeLineCap
        {
            get
            {
                return this.textPropertiesOwner.StrokeLineCap;
            }
            set
            {
                this.textPropertiesOwner.StrokeLineCap = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke line join.
        /// </summary>
        /// <value>The stroke line join.</value>
        public LineJoin StrokeLineJoin
        {
            get
            {
                return this.textPropertiesOwner.StrokeLineJoin;
            }
            set
            {
                this.textPropertiesOwner.StrokeLineJoin = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke dash array.
        /// </summary>
        /// <value>The stroke dash array.</value>
        public IEnumerable<double> StrokeDashArray
        {
            get
            {
                return this.textPropertiesOwner.StrokeDashArray;
            }
            set
            {
                this.textPropertiesOwner.StrokeDashArray = value;
            }
        }

        /// <summary>
        /// Gets or sets the stroke dash offset.
        /// </summary>
        /// <value>The stroke dash offset.</value>
        public double StrokeDashOffset
        {
            get
            {
                return this.textPropertiesOwner.StrokeDashOffset;
            }
            set
            {
                this.textPropertiesOwner.StrokeDashOffset = value;
            }
        }

        /// <summary>
        /// Gets or sets the miter limit.
        /// </summary>
        /// <value>The miter limit.</value>
        public double? MiterLimit
        {
            get
            {
                return this.textPropertiesOwner.MiterLimit;
            }
            set
            {
                this.textPropertiesOwner.MiterLimit = value;
            }
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text
        {
            get
            {
                return this.TextCollection.Text;
            }
            set
            {
                this.TextCollection.Text = value;
            }
        }

        internal TextCollection TextCollection
        {
            get
            {
                return this.textPropertiesOwner.TextCollection;
            }
        }

        internal Matrix TextMatrix
        {
            get
            {
                return this.textPropertiesOwner.TextMatrix;
            }
            set
            {
                this.textPropertiesOwner.TextMatrix = value;
            }
        }

        internal double? TextTranslation
        {
            get
            {
                return this.textPropertiesOwner.TextTranslation;
            }
            set
            {
                this.textPropertiesOwner.TextTranslation = value;
            }
        }

        internal TextPropertiesOwner TextProperties
        {
            get
            {
                return this.textPropertiesOwner;
            }
        }

        internal override FixedDocumentElementType ElementType
        {
            get
            {
                return FixedDocumentElementType.TextFragment;
            }
        }

        /// <summary>
        /// Returns a string that represents the current object.
        /// </summary>
        /// <returns>A string that represents the current object.</returns>
        public override string ToString()
        {
            return this.Text;
        }

        internal override PositionContentElement CreateClonedInstance()
        {
            return new TextFragment(this);
        }
    }
}
