﻿using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;
using Telerik.Windows.Documents.Fixed.PostScript.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.PostScript.Parser.Operators;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class CMapEncodingInterpreter : IOperandsInterpreter, ICMapEncodingInterpreter
    {
        private readonly CMapEncoding encoding;
        private readonly OperandsCollection operands;
        private readonly PostScriptReader reader;
        private readonly IPdfImportContext context;

        public CMapEncodingInterpreter(Stream stream, IPdfImportContext context, CMapEncoding encoding)
        {
            Guard.ThrowExceptionIfNull(stream, "stream");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(encoding, "encoding");

            this.context = context;
            this.encoding = encoding;
            this.operands = new OperandsCollection();
            this.reader = new PostScriptReader(stream, new PostScriptKeywordCollection());
        }

        public CMapEncoding Encoding
        {
            get
            {
                return this.encoding;
            }
        }

        public OperandsCollection Operands
        {
            get
            {
                return this.operands;
            }
        }

        public PostScriptReader Reader
        {
            get
            {
                return this.reader;
            }
        }

        public void Execute()
        { 
            this.Reader.Reader.Seek(0, SeekOrigin.Begin);

            IPdfPrimitive[] primitives = reader.Read(this.context);
            foreach (var primitive in primitives)
            {
                if (primitive != null)
                {
                    if (primitive.Type == PdfElementType.Operator)
                    {
                        CMapEncodingOperator oper = (CMapEncodingOperator)primitive;
                        oper.Execute(this, context);
                    }
                    else
                    {
                        this.Operands.AddLast(primitive);
                    }
                }
            }
        }
    }
}