﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;

namespace Telerik.Windows.Documents.Core.PostScript.Data
{
    internal static class Characters
    {
        public const char NotDef = '\0';
        public const char Null = '\0';
        public const byte SingleByteSpace = 32;
        //public const char NameEscape = '#';
        public const char Ap = '\'';
        public const char Qu = '"';
        public const char Space = ' ';
        public const char Tab = '\t';
        public const char LF = '\n';
        public const char FF = (char)12;
        public const char CR = '\r';
        public const char DecimalPoint = '.';
        public const char LiteralStringStart = '(';
        public const char LiteralStringEnd = ')';
        public const char HexadecimalStringStart = '<';
        public const char HexadecimalStringEnd = '>';
        public const char Comment = '%';
        public const char Name = '/';
        public const char ArrayStart = '[';
        public const char ArrayEnd = ']';
        public const char ExecuteableArrayStart = '{';
        public const char ExecuteableArrayEnd = '}';
        public const char StringEscapeCharacter = '\\';
        public static readonly char[] FontFamilyDelimiters = new char[] { ',', '-' };
        public const char Minus = '-';
        public const char Plus = '+';
        public const char VerticalBar = '|';
          
        public static bool IsWhiteSpace(int b)
        {
            char ch = (char)b;
            return char.IsWhiteSpace(ch);
        }

        public static bool IsOctalChar(int b)
        {
            char ch = (char)b;
            return '0' <= ch && ch <= '7';
        }

        public static bool IsHexChar(int b)
        {
            char ch = (char)b;
            return ('0' <= ch && ch <= '9') || ('A' <= ch && ch <= 'F') || ('a' <= ch && ch <= 'f');
        }

        public static bool IsDelimiter(int b)
        {
            char ch = (char)b;
            if (char.IsWhiteSpace(ch))
            {
                return true;
            }

            switch (ch)
            {
                case Characters.Null:
                case Characters.ArrayStart:
                case Characters.ArrayEnd:
                case Characters.HexadecimalStringStart:
                case Characters.HexadecimalStringEnd:
                case Characters.LiteralStringStart:
                case Characters.LiteralStringEnd:
                case Characters.Name:
                case Characters.Comment:
                case Characters.ExecuteableArrayStart:
                case Characters.ExecuteableArrayEnd:
                    return true;
            }
            return false;
        }

        public static bool IsLetter(int b)
        {
            char ch = (char)b;
            return char.IsLetter(ch);
        }

        public static bool IsValidNumberChar(IPostScriptReader reader)
        {
            char ch = (char)reader.Peek();
            if (ch == Plus || ch == Minus)
            {
                char next = (char)reader.Peek(1);
                return IsDigitOrDecimalPoint(next);
            }
            return IsDigitOrDecimalPoint(ch);
        }

        public static bool IsValidHexCharacter(int ch)
        {
            return 80 <= ch && ch <= 128;
        }

        private static bool IsDigitOrDecimalPoint(char ch)
        {
            return char.IsDigit(ch) || ch == Characters.DecimalPoint;
        }
    }
}
