﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;
using Matrix = Telerik.Windows.Documents.Core.Data.Matrix;

namespace Telerik.Windows.Documents.Core.PostScript.Data
{
    internal class PostScriptArray : PostScriptObject, IList<object>
    {
        #region StaticMembers

        public static PostScriptArray MatrixIdentity 
        {
            get
            {
                return new PostScriptArray(new object[] { 1, 0, 0, 1, 0, 0 });
            }
        }

        #endregion

        #region Constructors

        public PostScriptArray()
        {
            this.store = new List<object>();
        }
         
        public PostScriptArray(int capacity)
        {
            this.store = new List<object>(capacity);
            for (int i = 0; i < capacity; i++)
            {
                this.store.Add(null);
            }
        }

        public PostScriptArray(object[] initialValue)
        {
            this.store = new List<object>(initialValue);
        }

        #endregion

        #region Fields

        private readonly List<object> store;

        #endregion

        #region Operators

        public object this[int index]
        {
            get
            {
                return this.store[index];
            }
            set
            {
                this.store[index] = value;
            }
        }

        #endregion

        #region Methods

        public Matrix ToMatrix()
        {
            double m11, m12, m21, m22, offsetX, offsetY;
            Helper.UnboxReal(this.store[0], out m11);
            Helper.UnboxReal(this.store[1], out m12);
            Helper.UnboxReal(this.store[2], out m21);
            Helper.UnboxReal(this.store[3], out m22);
            Helper.UnboxReal(this.store[4], out offsetX);
            Helper.UnboxReal(this.store[5], out offsetY);
           
            return new Matrix(m11, m12, m21, m22, offsetX, offsetY);
        }

        public Rect ToRect()
        {
            double x1, y1, x2, y2;
            Helper.UnboxReal(this.store[0], out x1);
            Helper.UnboxReal(this.store[1], out y1);
            Helper.UnboxReal(this.store[2], out x2);
            Helper.UnboxReal(this.store[3], out y2);

            return new Rect(new Point(x1, y1), new Point(x2, y2));
        }

        public int IndexOf(object item)
        {
            return this.store.IndexOf(item);
        }

        public void Insert(int index, object item)
        {
            this.store.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            this.store.RemoveAt(index);
        }
        
        public void Add(object item)
        {
            this.store.Add(item);
        }

        public void Clear()
        {
            this.store.Clear();
        }

        public bool Contains(object item)
        {
            return this.store.Contains(item);
        }

        public void CopyTo(object[] array, int arrayIndex)
        {
            this.store.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }
        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }
        public bool Remove(object item)
        {
            return this.store.Remove(item);
        }

        public IEnumerator<object> GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.store.GetEnumerator();
        }

        public void Load(object[] content)
        {
            foreach (object obj in content)
            {
                this.store.Add(obj);
            }
        }

        public T GetElementAs<T>(int index)
        {
            return (T)this.store[index];
        }

        #endregion

        #region Overrides

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            foreach (object o in this.store)
            {
                sb.Append(" ");
                sb.Append(o);
            }
            sb.Remove(0, 1);
            sb.Append("]");
            sb.Insert(0, "[");
            return sb.ToString();
        }
        #endregion
    }
}
