﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts.Type1.Type1Format.Converters;
using Telerik.Windows.Documents.Core.Fonts.Type1.Utils;

namespace Telerik.Windows.Documents.Core.PostScript.Data
{
    internal abstract class PostScriptObject
    {
        private readonly Dictionary<string, IProperty> properties;

        public PostScriptObject()
        {
            this.properties = new Dictionary<string, IProperty>();

        }

        public void Load(PostScriptDictionary fromDict)
        {
            IProperty prop;
            foreach (var pair in fromDict)
            {
                if (this.properties.TryGetValue(pair.Key, out prop))
                {
                    prop.SetValue(pair.Value);
                }
            }
        }

        protected Property<T> CreateProperty<T>(PropertyDescriptor descriptor)
        {
            Property<T> res = new Property<T>(descriptor);
            this.RegisterProperty(res);
            return res;
        }

        protected Property<T> CreateProperty<T>(PropertyDescriptor descriptor, IConverter converter)
        {
            Property<T> res = new Property<T>(descriptor, converter);
            this.RegisterProperty(res);
            return res;
        }

        protected Property<T> CreateProperty<T>(PropertyDescriptor descriptor, T defaultValue)
        {
            Property<T> res = new Property<T>(descriptor, defaultValue);
            this.RegisterProperty(res);
            return res;
        }

        protected Property<T> CreateProperty<T>(PropertyDescriptor descriptor, IConverter converter, T defaultValue)
        {
            Property<T> res = new Property<T>(descriptor, converter, defaultValue);
            this.RegisterProperty(res);
            return res;
        }

        private void RegisterProperty(PropertyDescriptor descriptor, IProperty property)
        {
            if (descriptor.Name != null)
            {
                this.properties[descriptor.Name] = property;
            }
        }

        private void RegisterProperty(IProperty property)
        {
            this.RegisterProperty(property.Descriptor, property);
        }
    }
}
