﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import
{
    internal class OperandsCollection
    { 
        private readonly LinkedList<IPdfPrimitive> store;

        public OperandsCollection()
        {
            this.store = new LinkedList<IPdfPrimitive>();
        } 

        public int Count
        {
            get
            {
                return this.store.Count;
            }
        }

        public IPdfPrimitive First
        {
            get
            {
                return Enumerable.First(this.store);
            }
        }

        public IPdfPrimitive Last
        {
            get
            {
                return Enumerable.Last(this.store);
            }
        }

        public IPdfPrimitive GetElementAt(Origin origin, int index)
        {
            if (origin == Origin.Begin)
            {
                LinkedListNode<IPdfPrimitive> item = this.store.First;
                for (int i = 0; i < index; i++)
                {
                    item = item.Next;
                }

                return item.Value;
            }
            else
            {
                LinkedListNode<IPdfPrimitive> item = this.store.Last;
                for (int i = 0; i < index; i++)
                {
                    item = item.Previous;
                }

                return item.Value;
            }
        }

        public void AddLast(IPdfPrimitive primitive)
        {
            this.store.AddLast(primitive);
        }

        public void AddFirst(IPdfPrimitive primitive)
        {
            this.store.AddFirst(primitive);
        }

        public IPdfPrimitive GetLast()
        {
            IPdfPrimitive last = this.store.Last.Value;
            this.store.RemoveLast();

            return last;
        } 
         
        public IPdfPrimitive GetFirst()
        {
            IPdfPrimitive first = this.store.First.Value;
            this.store.RemoveFirst();

            return first;
        }

        public T GetLast<T>(PostScriptReader reader, IPdfImportContext context)
            where T : IPdfPrimitive
        {
            IPdfPrimitive result = this.GetLast();

            PdfObjectDescriptor descriptor = PdfObjectDescriptors.GetPdfObjectDescriptor<T>();
            T element = (T)descriptor.Converter.Convert(typeof(T), reader, context, result);

            return element;
        }

        public T GetLastAs<T>()
            where T : IPdfPrimitive
        {
            return (T)this.GetLast();
        }

        public void Clear()
        {
            this.store.Clear();
        }
    }
}
