using System.Collections.Generic;

namespace Telerik.Windows.Documents.Core.PostScript.Operators
{
    internal abstract class Operator
    {
        #region Consts

        internal const string Array = "array";
        internal const string Begin = "begin";
        internal const string ClearToMark = "cleartomark";
        internal const string CloseFile = "closefile";
        internal const string CurrentDict = "currentdict";
        internal const string CurrentFile = "currentfile";
        internal const string Def = "def";
        internal const string DefineFont = "definefont";
        internal const string Dict = "dict";
        internal const string Dup = "dup";
        internal const string EExec = "eexec";
        internal const string End = "end";
        internal const string Exch = "exch";
        internal const string For = "for";
        internal const string Get = "get";
        internal const string Index = "index";
        internal const string Put = "put";
        internal const string RD = "RD";
        internal const string RDAlternate = "-|";
        internal const string Mark = "mark";
        internal const string ND = "ND";
        internal const string NDAlternate = "|-";
        internal const string NoAccess = "noaccess";
        internal const string NP = "NP";
        internal const string NPAlternate = "|";
        internal const string ReadString = "readstring";
        internal const string String = "string";
        internal const string Pop = "pop";
        internal const string Copy = "copy";
        internal const string SystemDict = "systemdict";
        internal const string Known = "known";
        internal const string If = "if";
        internal const string IfElse = "ifelse";
        internal const string FontDirectory = "FontDirectory";
        internal const string Add = "add";
        internal const string Sub = "sub";
        internal const string Mul = "mul";
        internal const string Div = "div";
        internal const string IDiv = "idiv";
        internal const string Mod = "mod";
        internal const string Neg = "neg";
        internal const string Abs = "abs";
        internal const string Ceiling = "ceiling";
        internal const string Floor = "floor";
        internal const string Round = "round";
        internal const string Truncate = "truncate";
        internal const string Sqrt = "sqrt";
        internal const string Sin = "sin";
        internal const string Cos = "cos";
        internal const string Atan = "atan";
        internal const string Exp = "exp";
        internal const string Ln = "ln";
        internal const string Log = "log";
        internal const string Cvi = "cvi";
        internal const string Cvr = "cvr";
        internal const string Eq = "eq";
        internal const string Ne = "ne";
        internal const string Gt = "gt";
        internal const string Ge = "ge";
        internal const string Lt = "lt";
        internal const string Le = "le";
        internal const string And = "and";
        internal const string Or = "or";
        internal const string Xor = "xor";
        internal const string Not = "not";
        internal const string BitShift = "bitshift";
        internal const string True = "true";
        internal const string False = "false";
        internal const string Roll = "roll";
        internal const string UserDict = "userdict";

        #endregion

        #region StaticMembers

        private static Dictionary<string, Operator> operators;

        public static bool IsOperator(string str)
        {
            return operators.ContainsKey(str);
        }

        public static Operator FindOperator(string op)
        {
            Operator res;
            if (!operators.TryGetValue(op, out res))
            {
                return null;
            }
            return res;
        }

        private static void InitializeOperators()
        {
            operators = new Dictionary<string, Operator>();

            operators[Array] = new Array();
            operators[Begin] = new Begin();
            operators[ClearToMark] = new ClearToMark();
            operators[CloseFile] = new CloseFile();
            operators[CurrentDict] = new CurrentDict();
            operators[CurrentFile] = new CurrentFile();
            operators[Def] = new Def();
            operators[DefineFont] = new DefineFont();
            operators[Dict] = new Dict();
            operators[Dup] = new Dup();
            operators[EExec] = new EExec();
            operators[End] = new End();
            operators[Exch] = new Exch();
            operators[For] = new For();
            operators[Get] = new Get();
            operators[Index] = new Index();
            operators[Mark] = new Mark();
            operators[ND] = new ND();
            operators[NDAlternate] = new ND();
            operators[NoAccess] = new NoAccess();
            operators[NP] = new NP();
            operators[NPAlternate] = new NP();
            operators[Pop] = new Pop();
            operators[Put] = new Put();
            operators[RD] = new RD();
            operators[RDAlternate] = new RD();
            operators[ReadString] = new ReadString();
            operators[String] = new Operators.String();
            operators[Copy] = new Copy();
            operators[SystemDict] = new SystemDict();
            operators[Known] = new Known();
            operators[If] = new If();
            operators[IfElse] = new IfElse();
            operators[FontDirectory] = new FontDirectory();
            operators[Add] = new Add();
            operators[Sub] = new Sub();
            operators[Mul] = new Mul();
            operators[Div] = new Div();
            operators[IDiv] = new IDiv();
            operators[Mod] = new Mod();
            operators[Atan] = new Atan();
            operators[Exp] = new Exp();
            operators[Eq] = new Eq();
            operators[Ne] = new Ne();
            operators[Gt] = new Gt();
            operators[Ge] = new Ge();
            operators[Lt] = new Lt();
            operators[Le] = new Le();
            operators[And] = new And();
            operators[Or] = new Or();
            operators[Xor] = new Xor();
            operators[BitShift] = new BitShift();
            operators[Neg] = new Neg();
            operators[Abs] = new Abs();
            operators[Ceiling] = new Ceiling();
            operators[Floor] = new Floor();
            operators[Round] = new Round();
            operators[Truncate] = new Truncate();
            operators[Sqrt] = new Sqrt();
            operators[Sin] = new Sin();
            operators[Cos] = new Cos();
            operators[Ln] = new Ln();
            operators[Log] = new Log();
            operators[Cvi] = new Cvi();
            operators[Cvr] = new Cvr();
            operators[True] = new True();
            operators[False] = new False();
            operators[Roll] = new Roll();
            operators[UserDict] = new UserDict();
        }

        #endregion

        #region Constructors

        static Operator()
        {
            InitializeOperators();
        }

        #endregion

        #region Methods

        public abstract void Execute(Interpreter interpreter);

        #endregion
    }
}