﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.PostScript.Parser.Operators.Utils
{
    internal static class ToUnicodeRangeBuilder
    {
        internal static IDictionary<CharCode, string> BuildToUnicodeMapping(IOperandsInterpreter interpreter)
        {
#if !SILVERLIGHT
            SortedDictionary<CharCode, string> toUnicodeMapping = new SortedDictionary<CharCode, string>();
#else
            Dictionary<CharCode, string> toUnicodeMapping = new Dictionary<CharCode, string>();
#endif
            for (int i = 0; i < interpreter.Operands.Count; i += 3)
            {
                PdfString begin = (PdfString)interpreter.Operands.GetElementAt(Origin.Begin, i);
                CharCode beginCharCode = new CharCode(begin.Value);

                PdfString end = (PdfString)interpreter.Operands.GetElementAt(Origin.Begin, i + 1);
                CharCode endCharCode = new CharCode(end.Value);

                IPdfPrimitive unicodeCharCodesRange = interpreter.Operands.GetElementAt(Origin.Begin, i + 2);

                CharCode currentCharCode = beginCharCode.Clone();
                int unicodeCharsCount = endCharCode.Code - beginCharCode.Code;

                PdfString unicodeStringStart = unicodeCharCodesRange as PdfString;
                CharCode startUnicodeChar = null;

                if (unicodeStringStart != null)
                {
                    startUnicodeChar = new CharCode(((PdfString)unicodeCharCodesRange).Value);
                }

                for (int index = 0; index <= unicodeCharsCount; index++)
                {
                    PdfString unicode = GetUnicodeAt(index, startUnicodeChar, unicodeCharCodesRange);
                    string unicodeString = PdfString.GetUtfEncodedTextString(unicode.Value).ToString();

                    CharCode charCodeToMap = currentCharCode.Clone();
                    //// TODO: Handle different glyphs with equal CharCodes.
                    toUnicodeMapping[charCodeToMap] = unicodeString;

                    currentCharCode.Code++;
                }
            }

            return toUnicodeMapping;
        }

        private static PdfString GetUnicodeAt(int index, CharCode startUnicodeChar, IPdfPrimitive unicodeCharCodesRange)
        {
            PdfString result;
            switch (unicodeCharCodesRange.Type)
            {
                case PdfElementType.String:
                    int startUnicode = startUnicodeChar.Code;
                    startUnicodeChar.Code += index;
                    result = new PdfHexString(startUnicodeChar.ToBytes());

                    // Set the initial value to reuse the same instance for each method call.
                    startUnicodeChar.Code = startUnicode;

                    break;

                case PdfElementType.Array:
                    PdfArray unicodeCharCodes = unicodeCharCodesRange as PdfArray;
                    result = unicodeCharCodes[index] as PdfString;
                    break;

                default:
                    throw new NotSupportedException("The ToUnicode range definition doesn't support range of type " + unicodeCharCodesRange.Type);
            }

            return result;
        }
    }
}
