﻿using System;
using System.Collections.Generic;
#if SILVERLIGHT
using System.Linq;
#endif
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model.Data;

namespace Telerik.Windows.Documents.Fixed.PostScript.Parser.Operators.Utils
{
    internal static class ToUnicodeRangeParser
    {
        internal static void ParseToUnicodeMapping(ICMapEncodingInterpreter interpreter, IDictionary<CharCode, string> toUnicodeMapping)
        {
            List<string> unicodes = new List<string>();

            CharCode previousCharCode = null;
            string previousUnicode = null;
            
            List<CharCode> range = new List<CharCode>();

#if !SILVERLIGHT
            foreach (KeyValuePair<CharCode, string> charCodeToUnicode in toUnicodeMapping)
            {
#else
            var keys = toUnicodeMapping.Keys.ToArray();
            Array.Sort(keys);

            for (int i = 0; i < keys.Length; i++)
            {
                CharCode key = keys[i];
                KeyValuePair<CharCode, string> charCodeToUnicode = new KeyValuePair<CharCode, string>(key, toUnicodeMapping[key]);
#endif
                CharCode currentCharCode = charCodeToUnicode.Key;
                string currentUnicode = charCodeToUnicode.Value;

                if (previousCharCode == null)
                {
                    previousCharCode = currentCharCode;
                    range.Add(currentCharCode);
                    previousUnicode = currentUnicode;
                    unicodes.Add(currentUnicode);

                    continue;
                }

                bool areSequentialCharCodes = ToUnicodeRangeParser.CheckAreSequentialCharCodes(previousCharCode, currentCharCode);

                if (areSequentialCharCodes)
                {
                    range.Add(currentCharCode);
                    unicodes.Add(currentUnicode);
                }
                else
                {
                    ToUnicodeRangeParser.HandleToUnicodeMapping(interpreter, range, unicodes);

                    unicodes.Clear();
                    unicodes.Add(currentUnicode);
                    range.Clear();
                    range.Add(currentCharCode);
                }

                previousCharCode = currentCharCode;
            }

            ToUnicodeRangeParser.HandleToUnicodeMapping(interpreter, range, unicodes);
        }

        private static void HandleToUnicodeMapping(ICMapEncodingInterpreter interpreter, List<CharCode> range, List<string> unicodes)
        {
            if (unicodes.Count > 1)
            {
                ToUnicodeRangeParser.AddТоUnicodeRanges(interpreter, range, unicodes);
            }
            else if (unicodes.Count == 1)
            {
                interpreter.Encoding.AddUnicodeMapping(range[0], unicodes[0]);
            }
        }

        private static void AddТоUnicodeRanges(ICMapEncodingInterpreter interpreter, List<CharCode> charCodes, List<string> unicodes)
        {
            CharCode firstCode = charCodes[0];
            CharCode beginCharCode = firstCode;
            CharCode endCharCode = firstCode;
            CharCode previousCharCode = firstCode.Clone();
            CharCode currentCharCode = charCodes[1].Clone();

            string previousUnicode = unicodes[0];

            List<string> sequentialUnicodes = new List<string>();
            List<string> nonSequentialUnicodes = new List<string>();

            for (int i = 1; i < unicodes.Count; i += 1)
            {
                string currentUnicode = unicodes[i];
                bool areSequentialUnicodes = ToUnicodeRangeParser.CheckAreSequentialUnicodeStrings(previousUnicode, currentUnicode);

                if (areSequentialUnicodes)
                {
                    ToUnicodeRangeParser.HandleUnicodeRange(
                        sequentialUnicodes,
                        nonSequentialUnicodes,
                        previousCharCode,
                        currentCharCode,
                        previousUnicode,
                        currentUnicode,
                        i,
                        interpreter.Encoding.AddUnicodeRangeArray,
                        ref beginCharCode,
                        ref endCharCode);
                }
                else
                {
                    ToUnicodeRangeParser.HandleUnicodeRange(
                        nonSequentialUnicodes,
                        sequentialUnicodes,
                        previousCharCode,
                        currentCharCode,
                        previousUnicode,
                        currentUnicode,
                        i,
                        interpreter.Encoding.AddUnicodeRange,
                        ref beginCharCode,
                        ref endCharCode);
                }

                previousCharCode.Code++;
                previousUnicode = currentUnicode;
                currentCharCode.Code++;
            }

            ToUnicodeRangeParser.TryAddRange(sequentialUnicodes, beginCharCode, endCharCode, interpreter.Encoding.AddUnicodeRange);
            ToUnicodeRangeParser.TryAddRange(nonSequentialUnicodes, beginCharCode, endCharCode, interpreter.Encoding.AddUnicodeRangeArray);
        }

        private static void HandleUnicodeRange(
            List<string> newUnicodesRange,
            List<string> oldUnicodesRange,
            CharCode previousCharCode,
            CharCode currentCharCode,
            string previousUnicode,
            string currentUnicode,
            int index,
            Action<CharCode, CharCode, List<string>> addOldRangeAction,
            ref CharCode beginCharCode,
            ref CharCode endCharCode)
        {
            bool isAdded = ToUnicodeRangeParser.TryAddRange(oldUnicodesRange, beginCharCode, endCharCode, addOldRangeAction);
            if (isAdded)
            {
                oldUnicodesRange.Clear();
            }

            if (index == 1)
            {
                newUnicodesRange.Add(previousUnicode);
                beginCharCode = previousCharCode.Clone();
            }

            if (newUnicodesRange.Count == 0)
            {
                beginCharCode = currentCharCode.Clone();
            }

            newUnicodesRange.Add(currentUnicode);
            endCharCode = currentCharCode.Clone();
        }

        private static bool TryAddRange(
            List<string> unicodes,
            CharCode beginCharCode,
            CharCode endCharCode,
            Action<CharCode, CharCode, List<string>> addRangeAction)
        {
            bool shouldAdd = unicodes.Count > 0;

            if (shouldAdd)
            {
                List<string> unicodesToAdd = new List<string>(unicodes);
                addRangeAction(beginCharCode, endCharCode, unicodesToAdd);
            }

            return shouldAdd;
        }

        private static bool CheckAreSequentialCharCodes(CharCode previousCharCode, CharCode currentCharCode)
        {
            byte previousCharCodeLastByte = previousCharCode.ToBytes()[previousCharCode.Size - 1];
            byte currentCharcodeLastByte = currentCharCode.ToBytes()[currentCharCode.Size - 1];

            bool areSequentialCharCodes = currentCharcodeLastByte == (previousCharCodeLastByte + 1);

            return areSequentialCharCodes;
        }

        private static bool CheckAreSequentialUnicodeStrings(string first, string second)
        {
            byte[] firstUnicodeBytes = Encoding.BigEndianUnicode.GetBytes(first);
            byte firstUnicodeLastByte = firstUnicodeBytes[firstUnicodeBytes.Length - 1];

            byte[] secondUnicodeBytes = Encoding.BigEndianUnicode.GetBytes(second);
            byte secondUnicodeLastByte = secondUnicodeBytes[secondUnicodeBytes.Length - 1];

            bool areSequentialUnicodes = secondUnicodeLastByte == (firstUnicodeLastByte + 1);

            return areSequentialUnicodes;
        }
    }
}
