﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.PrimitiveParsers
{
    internal class NumberParser
    {
        private readonly PostScriptReader parser;
        private readonly StringBuilder token;
        
        private bool isDouble;

        public NumberParser(PostScriptReader parser)
        {
            Guard.ThrowExceptionIfNull(parser, "parser");

            this.parser = parser;
            this.token = new StringBuilder();
            this.isDouble = false;
        }

        public void Start(char sign)
        {
            this.token.Append(sign);
        }

        public virtual void Append(PostScriptReaderArgs args)
        {
            char ch = (char)args.Byte;
            
            this.token.Append(ch);
            this.isDouble |= PdfHelper.CultureInfo.IsDecimalSeparator(ch);
        }

        public void Complete()
        {
            if (this.isDouble)
            {
                double value;
                PdfHelper.CultureInfo.TryParseDouble(this.token.ToString(), out value);

                this.parser.PushToken(new PdfReal(value));
            }
            else
            {
                int value;
                int.TryParse(this.token.ToString(), out value);

                this.parser.PushToken(new PdfInt(value));
            }

            this.token.Clear();
            this.isDouble = false;
        }

        public bool IsNumber(PostScriptReaderArgs args)
        {
            return char.IsDigit((char)args.Byte) || (!this.isDouble && PdfHelper.CultureInfo.IsDecimalSeparator((char)args.Byte)); 
        }
    }
}
