﻿using Telerik.Windows.Documents.Fixed.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Search
{
    /// <summary>
    /// Represent search result.
    /// </summary>
    public class SearchResult
    {
        private static readonly SearchResult notFound;

        private readonly TextRange range;
        private readonly string value;

        static SearchResult()
        {
            notFound = new SearchResult();
        }

        internal SearchResult(TextPosition startPosition, TextPosition endPosition)
        {
            Guard.ThrowExceptionIfNull(startPosition, "startPosition");
            Guard.ThrowExceptionIfNull(endPosition, "endPosition");

            this.range = new TextRange(startPosition, endPosition);
            this.value = TextDocument.GetText(startPosition, endPosition, " ");
        }

        private SearchResult()
        {
            this.range = TextRange.Empty;
        }

        /// <summary>
        /// Gets or sets the not found.
        /// </summary>
        /// <value>The not found.</value>
        public static SearchResult NotFound
        {
            get
            {
                return notFound;
            }
        }

        /// <summary>
        /// Gets the text range.
        /// </summary>
        /// <value>The range.</value>
        public TextRange Range
        {
            get
            {
                return this.range;
            }
        }

        /// <summary>
        /// Gets the result as text.
        /// </summary>
        /// <value>The result.</value>
        public string Result
        {
            get
            {
                return this.value;
            }
        }

        /// <summary>
        /// Returns a string that represents the current object.
        /// </summary>
        /// <returns>A string that represents the current object.</returns>
        public override string ToString()
        {
            return this.Result;
        }
    }
}
