﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#endif
using Matrix = Telerik.Windows.Documents.Core.Data.Matrix;

namespace Telerik.Windows.Documents.Core.Shapes
{
    internal class PathGeometry
    {
        internal static PathGeometry CreateRectangle(Rect rect)
        {
            PathGeometry pg = new PathGeometry();
            PathFigure f = new PathFigure();
            f.StartPoint = new Point(rect.Left, rect.Top);
            f.IsClosed = true;
            f.IsFilled = true;
            f.Segments.Add(new LineSegment { Point = new Point(rect.Right, rect.Top) });
            f.Segments.Add(new LineSegment { Point = new Point(rect.Right, rect.Bottom) });
            f.Segments.Add(new LineSegment { Point = new Point(rect.Left, rect.Bottom) });
            pg.Figures.Add(f);
            return pg;
        }

        private static void Compare(Point point, ref double minX, ref double maxX, ref double minY, ref double maxY)
        {
            minX = Math.Min(point.X, minX);
            maxX = Math.Max(point.X, maxX);
            minY = Math.Min(point.Y, minY);
            maxY = Math.Max(point.Y, maxY);
        }

        public PathGeometry()
        {
            this.Figures = new List<PathFigure>();
            this.TransformMatrix = Matrix.Identity;
        }

        public List<PathFigure> Figures { get;set; }

        public FillRule FillRule { get;set; }

        public PathGeometry Clone()
        {
            PathGeometry pg = new PathGeometry();
            pg.FillRule = this.FillRule;
            pg.TransformMatrix = this.TransformMatrix;
            foreach (PathFigure pf in this.Figures)
            {
                pg.Figures.Add(pf.Clone());
            }
            return pg;
        }

        public Matrix TransformMatrix { get;set; }

        public bool IsEmpty
        {
            get
            {
                return this.Figures == null || this.Figures.Count == 0;
            }
        }

        public Rect GetBoundingRect()
        {
            double minX = double.PositiveInfinity, minY = double.PositiveInfinity, maxX = double.NegativeInfinity, maxY = double.NegativeInfinity;
            foreach (PathFigure figure in this.Figures)
            {
                Compare(figure.StartPoint, ref minX, ref maxX, ref minY, ref maxY);
                foreach (PathSegment segment in figure.Segments)
                {
                    if (segment is LineSegment)
                    {
                        Compare(((LineSegment)segment).Point, ref minX, ref maxX, ref minY, ref maxY);
                    }
                    else if (segment is BezierSegment)
                    {
                        BezierSegment bs = (BezierSegment)segment;
                        Compare(bs.Point1, ref minX, ref maxX, ref minY, ref maxY);
                        Compare(bs.Point2, ref minX, ref maxX, ref minY, ref maxY);
                        Compare(bs.Point3, ref minX, ref maxX, ref minY, ref maxY);
                    }
                }
            }
            return new Rect(new Point(minX, minY), new Point(maxX, maxY));
        }
    }
}
