﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Text
{
    /// <summary>
    /// Represents text range.
    /// </summary>
    public class TextRange
    {
        private readonly TextPosition startPosition;
        private readonly TextPosition endPosition;

        internal TextRange(TextPosition startPosition, TextPosition endPosition)
        {
            Guard.ThrowExceptionIfNull(startPosition, "startPosition");
            Guard.ThrowExceptionIfNull(endPosition, "endPosition");

            this.startPosition = new TextPosition(startPosition);
            this.endPosition = new TextPosition(endPosition);
        }

        private TextRange()
        {
            this.startPosition = null;
            this.endPosition = null;
        }

        /// <summary>
        /// Gets empty text range.
        /// </summary>
        /// <value>The range.</value>
        public static TextRange Empty
        {
            get
            {
                return new TextRange();
            }
        }

        /// <summary>
        /// Gets the start position.
        /// </summary>
        /// <value>The start position.</value>
        public TextPosition StartPosition
        {
            get
            {
                return this.startPosition;
            }
        }

        /// <summary>
        /// Gets the end position.
        /// </summary>
        /// <value>The end position.</value>
        public TextPosition EndPosition
        {
            get
            {
                return this.endPosition;
            }
        }

        /// <summary>
        /// Check if text range is empty.
        /// </summary>
        /// <value>Is empty.</value>
        public bool IsEmpty
        {
            get
            {
                return this.startPosition == null && this.endPosition == null;
            }
        }
    }
}
