﻿using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Utilities.Rendering.Annotations
{
    internal class WidgetAppearanceProvider : BaseAppearanceProvider
    {
        internal WidgetAppearanceProvider(BaseAppearanceProvider nextProvider) : base(nextProvider)
        {
        }

        protected override bool CanProvide(Annotation annotation, AnnotationAppearanceMode annotationAppearanceMode)
        {
            return (annotation as Widget) != null;
        }

        protected override FormSource ProvideAppearance(Annotation annotation, AnnotationAppearanceMode annotationAppearanceMode)
        {
            IContentAnnotation contentWidget = annotation as IContentAnnotation;

            if (contentWidget != null)
            {
                if (contentWidget.Content != null)
                {
                    SingleStateAppearances singleStateAppearances = new SingleStateAppearances(contentWidget.Content);
                    return this.GetAppearanceFormSource(singleStateAppearances, annotationAppearanceMode);
                }
            }

            RadioButtonWidget radioButtonWidget = annotation as RadioButtonWidget;

            if (radioButtonWidget != null)
            {
                SingleStateAppearances singleStateAppearances = this.GetRadioButtonSingleStateAppearances(radioButtonWidget);

                if (singleStateAppearances != null)
                {
                    return this.GetAppearanceFormSource(singleStateAppearances, annotationAppearanceMode);
                }
            }

            TwoStatesButtonWidget twoStatesButtonWidget = annotation as TwoStatesButtonWidget;

            if (twoStatesButtonWidget != null)
            {
                CheckBoxField checkBoxField = twoStatesButtonWidget.Field as CheckBoxField;
                SingleStateAppearances singleStateAppearances = this.GetCheckBoxSingleStateAppearances(twoStatesButtonWidget, checkBoxField);

                if (singleStateAppearances != null)
                {
                    return this.GetAppearanceFormSource(singleStateAppearances, annotationAppearanceMode);
                }
            }

            return null;
        }

        private SingleStateAppearances GetRadioButtonSingleStateAppearances(RadioButtonWidget radioButtonWidget)
        {
            SingleStateAppearances result = null;

            if (radioButtonWidget.IsSelected && radioButtonWidget.OnStateContent != null)
            {
                result = new SingleStateAppearances(radioButtonWidget.OnStateContent);
            }
            else if (!radioButtonWidget.IsSelected && radioButtonWidget.OffStateContent != null)
            {
                result = new SingleStateAppearances(radioButtonWidget.OffStateContent);
            }

            return result;
        }

        private SingleStateAppearances GetCheckBoxSingleStateAppearances(TwoStatesButtonWidget twoStatesButtonWidget, CheckBoxField checkBoxField)
        {
            SingleStateAppearances result = null;

            if (checkBoxField.IsChecked && twoStatesButtonWidget.OnStateContent != null)
            {
                result = new SingleStateAppearances(twoStatesButtonWidget.OnStateContent);
            }
            else if (!checkBoxField.IsChecked && twoStatesButtonWidget.OffStateContent != null)
            {
                result = new SingleStateAppearances(twoStatesButtonWidget.OffStateContent);
            }

            return result;
        }
    }
}
