﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.Utilities.Rendering
{
    internal class GlyphRenderingContext : ContentRenderingContext
    {
        private readonly GlyphOutlinesCollection outlines;
        private readonly Matrix outlinesTransformation;
        private readonly GeometryPropertiesOwner properties;
        
        internal GlyphRenderingContext(TextFragment textFragment, GlyphOutlinesCollection outlines, Matrix outlinesTransformation, Rect currentContainerBounds)
            : base(textFragment, currentContainerBounds)
        {
            this.outlines = outlines;
            this.outlinesTransformation = outlinesTransformation;
            this.properties = textFragment.TextProperties.GeometryProperties;
        }

        /// <summary>
        /// The outlines.
        /// </summary>
        public GlyphOutlinesCollection Outlines
        {
            get
            {
                return this.outlines;
            }
        }

        /// <summary>
        /// The transformation that should be applied to the original outlines collection.
        /// </summary>
        public Matrix OutlinesTransformation
        {
            get
            {
                return this.outlinesTransformation;
            }
        }

        /// <summary>
        /// The geometry properties that should be applied to the glyphs geometry.
        /// </summary>
        public GeometryPropertiesOwner Properties
        {
            get
            {
                return this.properties;
            }
        }
    }
}
