﻿using System;
#if !NETSTANDARD
using System.Windows.Media;
#else
using Telerik.Documents.Primitives;
#endif
using Telerik.Windows.Documents.Fixed.Model.Graphics;

namespace Telerik.Windows.Documents.Fixed.Utilities.Rendering
{
    internal interface IFixedContentRenderer
    {
        /// <summary>
        /// Adds new transformation to the current rendering state.
        /// </summary>
        /// <param name="matrix">The matrix describing the transformation.</param>
        /// <returns>Disposable object which removes the given transformation once its Dispose method is called.</returns>
        IDisposable PushTransform(Matrix matrix);

        /// <summary>
        /// Adds new clipping to the current rendering state.
        /// </summary>
        /// <param name="geometry">The clipping geometry.</param>
        /// <returns>Disposable object which removes the given clipping once its Dispose method is called.</returns>
        IDisposable PushClipping(GeometryBase geometry);

        /// <summary>
        /// Renders text from font properties when the font source is not embedded and cannot be provided from the system font files.
        /// </summary>
        /// <param name="renderingContext">The <see cref="TextRenderingContext"/> context.</param>        
        void RenderText(TextRenderingContext renderingContext);

        /// <summary>
        /// Renders glyph oulines geometry.
        /// </summary>
        /// <param name="renderingContext">The <see cref="GlyphRenderingContext"/> context.</param>        
        void RenderGlyph(GlyphRenderingContext renderingContext);

        /// <summary>
        /// Renders geometry.
        /// </summary>
        /// <param name="renderingContext">The <see cref="GeometryRenderingContext"/> context.</param>
        void RenderGeometry(GeometryRenderingContext renderingContext);

        /// <summary>
        /// Renders image.
        /// </summary>
        /// <param name="renderingContext">The <see cref="ImageRenderingContext"/> context.</param>
        void RenderImage(ImageRenderingContext renderingContext);
    }
}
