﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Utilities.Rendering
{
    internal class ImageRenderingContext : ContentRenderingContext
    {
        private ImageSource imageSource;
        private double renderingWidth;
        private double renderingHeight;
        private ColorBase stencilColor;
        
        internal ImageRenderingContext(Image image,Rect currentContainerBounds)
            : base(image, currentContainerBounds)
        {
            this.imageSource = image.ImageSource;
            this.renderingWidth = image.Width;
            this.renderingHeight = image.Height;
            this.stencilColor = image.StencilColor;
        }

        /// <summary>
        /// The image source.
        /// </summary>
        public ImageSource ImageSource
        {
            get
            {
                return this.imageSource;
            }
        }

        /// <summary>
        /// The width of the rendered image.
        /// </summary>
        public double RenderingWidth
        {
            get
            {
                return this.renderingWidth;
            }
        }

        /// <summary>
        /// The height of the rendered image.
        /// </summary>
        public double RenderingHeight
        {
            get
            {
                return this.renderingHeight;
            }
        }

        /// <summary>
        /// If not null this is the color that should be used when drawing monochrome image source.
        /// </summary>
        public ColorBase StencilColor
        {
            get
            {
                return this.stencilColor;
            }
        }
    }
}
