﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.Utilities.Rendering
{
    internal class TextRenderingContext : ContentRenderingContext
    {
        private readonly string text;
        private readonly double fontSize;
        private readonly FontProperties fontProperties;
        private readonly GeometryPropertiesOwner geometryProperties;

        internal TextRenderingContext(TextFragment textFragment, Rect currentContainerBounds)
            : base(textFragment, currentContainerBounds)
        {
            FontStyle fontStyle;
            FontWeight fontWeight;
            string fontFamily;
            FontsHelper.GetFontFamily(textFragment.Font.FontName, out fontFamily, out fontWeight, out fontStyle);

            this.fontProperties = new FontProperties(new FontFamily(fontFamily), fontStyle, fontWeight);
            this.text = textFragment.Text;
            this.fontSize = textFragment.FontSize;
            this.geometryProperties = textFragment.TextProperties.GeometryProperties;
        }

        /// <summary>
        /// The text.
        /// </summary>
        public string Text
        {
            get
            {
                return this.text;
            }
        }

        /// <summary>
        /// The font size.
        /// </summary>
        public double FontSize
        {
            get
            {
                return this.fontSize;
            }
        }

        /// <summary>
        /// The <see cref="FontProperties"/> instance describing how to style the text.
        /// </summary>
        public FontProperties FontProperties
        {
            get
            {
                return this.fontProperties;
            }
        }

        /// <summary>
        /// The geometry properties that should be applied to the text geometry.
        /// </summary>
        public GeometryPropertiesOwner GeometryProperties
        {
            get
            {
                return this.geometryProperties;
            }
        }
    }
}
