﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts.Type1.Data;
using Telerik.Windows.Documents.Core.Fonts.Type1.Encoding;
#if !NETSTANDARD
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts;
#endif

namespace FontReaderTests.CFFTests
{
    [TestClass]
    public class TestEncodings
    {
        [TestMethod]
        public void TestIntegerEncodings()
        {
            byte[] data = new byte[] { 0x8b, 0xef, 0x27, 0xfa, 0x7c, 0xfe, 0x7c, 0x1c, 0x27, 0x10, 0x1c, 0xd8, 0xf0, 0x1d, 0x00, 0x01, 0x86, 0xa0, 0x1d, 0xff, 0xfe, 0x79, 0x60 };
            EncodedDataReader reader = new EncodedDataReader(data, ByteEncoding.DictByteEncodings);

            Assert.AreEqual((sbyte)0, reader.ReadOperand());
            Assert.AreEqual((sbyte)100, reader.ReadOperand());
            Assert.AreEqual((sbyte)-100, reader.ReadOperand());
            Assert.AreEqual((short)1000, reader.ReadOperand());
            Assert.AreEqual((short)-1000, reader.ReadOperand());
            Assert.AreEqual((short)10000, reader.ReadOperand());
            Assert.AreEqual((short)-10000, reader.ReadOperand());
            Assert.AreEqual(100000, reader.ReadOperand());
            Assert.AreEqual(-100000, reader.ReadOperand());
        }

        [TestMethod]
        public void TestRealEncodings()
        {
            byte[] data = new byte[] { 0x1e, 0xe2, 0xa2, 0x5f, 0x1e, 0x0a, 0x14, 0x05, 0x41, 0xc3, 0xff };
            EncodedDataReader reader = new EncodedDataReader(data, ByteEncoding.DictByteEncodings);
            Assert.AreEqual(-2.25, reader.ReadOperand());
            Assert.AreEqual(0.140541E-3, reader.ReadOperand());
        }

#if !NETSTANDARD
        [TestMethod]
        public void TestEncodingInitializedWithNullName()
        {           
            IndirectObjectOld indirectObject = new IndirectObjectOld();
            indirectObject.Value = new PdfNameOld(TestUtils.ContentManager, "NULL");

            EncodingOld encoding = new EncodingOld(TestUtils.ContentManager);
            encoding.Load(indirectObject);

            string nameFromStandardEncoding = encoding.GetName(163);
            Assert.AreEqual("sterling", nameFromStandardEncoding);
        }
#endif
    }
}
