﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.PostScript;
using Telerik.Windows.Documents.Core.PostScript.Encryption;

namespace Telerik.Windows.Documents.Core.Tests.CCFTests
{
    [TestClass]
    public class TestEncryption
    {
        private readonly byte[] data = PostScriptReaderHelper.GetBytesFromHexString("10BF31704FAB5B1F03F9B68B1F39A66521B1841F1481697F8E12B7F7DDD6E3D7248D965B1CD45E2114");
        private readonly byte[] expected = PostScriptReaderHelper.GetBytesFromHexString("BDF9B40D8BEF038BEF01F8ECEF018B16F95006EF07FCEC06F88807F8EC06EF07FD5006090E");

        [TestMethod]
        public void TestCharStringDecryption()
        {
            PostScriptReader reader = new PostScriptReader(this.data);
            CharStringEncryption encryption = new CharStringEncryption(reader);
            reader.PushEncryption(encryption);
            encryption.Initialize();

            for (int i = 0; i < 30; i++)
            {
                byte b = reader.Read();
                Assert.AreEqual(this.expected[i], b);
            }
        }

        [TestMethod]
        public void TestCharStringReadAndPeek()
        {
            PostScriptReader reader = new PostScriptReader(this.data);
            CharStringEncryption encryption = new CharStringEncryption(reader);
            reader.PushEncryption(encryption);
            encryption.Initialize();
            Assert.AreEqual(this.expected[3], reader.Peek(3));
            Assert.AreEqual(this.expected[0], reader.Peek());
            Assert.AreEqual(this.expected[10], reader.Peek(10));
            Assert.AreEqual(this.expected[3], reader.Peek(3));

            for (int i = 0; i < 30; i++)
            {
                byte b = reader.Read();
                Assert.AreEqual(this.expected[i], b);
            }
        }
    }
}
