﻿using System;
using System.IO;
using System.Security.Cryptography.X509Certificates;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming.IncrementalUpdate.InteractiveForms;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.DigitalSignature
{
    [TestClass]
    public class IncrementalSignTests : SignatureTests
    {
        [TestMethod]
        public void ImportAndSign_OneSignatureFieldTwoWidgetsOnTwoPages_UnSigned()
        {
            this.SignAndValidateDocument("OneSignatureFieldTwoWidgetsOnTwoPages_UnSigned.pdf", "FirstSignature");
        }

        [TestMethod]
        public void ImportAndSign_OneSignatureFieldTwoWidgetsOnTwoPages_Signed()
        {
            this.SignAndValidateDocument("OneSignatureFieldTwoWidgetsOnTwoPages_Signed.pdf", "FirstSignature");
        }

        [TestMethod]
        public void ImportAndSign_TwoSignatureFields_OneSigned()
        {
            this.SignAndValidateDocument("TwoSignatureFields_OneSigned.pdf", "SecondSignature");
        }

        private void SignAndValidateDocument(string documentName, string signatureFieldName)
        {
            X509Certificate2 certificate = GetCertificate(SignatureTests.ChildCertificate);

            using (Stream stream = TestUtils.GetApplicationResourceStream(SignatureTests.DocumentFolder + documentName))
            {
                using (Stream outputStream = new MemoryStream())
                {
                    using (PdfFileSource source = new PdfFileSource(stream))
                    {
                        using (PdfIncrementalStreamWriter writer = new PdfIncrementalStreamWriter(source, outputStream, true))
                        {
                            using (SignaturePropertiesEditor signatureEditor = writer.EditSignatureProperties(signatureFieldName))
                            {
                                signatureEditor.SetValue(certificate);

                                FormSource formSouce = GetFormSource();
                                signatureEditor.SetAppearance(Enumerable.Repeat(formSouce, 1));
                            }
                        }
                    }

                    RadFixedDocument document = this.Import(outputStream);
                    this.ValidateDocument(document);
                }
            }
        }

        private FormSource GetFormSource(string signatureName = "MySignature")
        {
            FormSource formSource = new FormSource();
            formSource.Size = new Size(120, 120);

            FixedContentEditor fixedContentEditor = new FixedContentEditor(formSource);
            fixedContentEditor.GraphicProperties.FillColor = new RgbColor(255, 0, 0);
            fixedContentEditor.DrawCircle(new Point(50, 50), 20);
            fixedContentEditor.DrawText(signatureName);

            return formSource;
        }
    }
}
