﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.Editing
{
    [TestClass]
    public class MultithreadedTests
    {
        private readonly Dictionary<int, bool> dictionary = new Dictionary<int, bool>();

#if WPF45 || NETSTANDARD
        [TestMethod]
        public void GenerateDocumentPerformanceTest_InMultithreadedScenario()
        {
            FontBase myFont;
            var font = FontsRepository.TryCreateFont(new FontFamily("Arial"), FontStyles.Normal, FontWeights.Normal, out myFont);

            int iCount = 2000;

            int reportId = 1;

            DateTime start = DateTime.UtcNow;

            for (int i = 0; i < iCount; i++)
            {
                var tuple = new Tuple<int, FontBase>(reportId, myFont);

                dictionary[reportId] = false;

                Task task = Task.Factory.StartNew(this.CreateReport, tuple);
                task.ContinueWith(TaskCompleteCallback, reportId);

                reportId++;
            }

            while (dictionary.Values.ToArray().Any((isComplete)=> isComplete == false))
            {
                double totalSeconds = (DateTime.UtcNow - start).TotalSeconds;
                if (totalSeconds > 25)
                {
                    throw new TimeoutException("This test takes too long to finish");
                }
            }
        }

        private void CreateReport(object obj)
        {
            Tuple<int, FontBase> tuple = obj as Tuple<int, FontBase>;
            var report = new TestReport();
            var stream = report.CreateReport(tuple.Item1, tuple.Item2);
            stream.Flush();
        }

        private void TaskCompleteCallback(Task task, object obj)
        {
            int id = (int)obj;

            dictionary[id] = true;
        }
#endif
    }
}
