﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;

namespace Telerik.Windows.Documents.Fixed.Tests.Filters
{
    [TestClass]
    public class TestASCIIHexDecode : TestFilterBase
    {
        [TestMethod]
        public void WhenTheNumberOfInputBytesIsOdd()
        {
            ASCIIHexDecode decoder = new ASCIIHexDecode();
            byte[] inputData = new byte[] { ToByte('0'), ToByte('1'), ToByte('A'), ToByte('1'), ToByte('7'), ToByte('>') };
            byte[] expected = new byte[] { 1, 161, 112 };
            byte[] received = decoder.Decode(null, inputData, null);

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheNumberOfInputBytesIsEven()
        {
            ASCIIHexDecode decoder = new ASCIIHexDecode();
            byte[] inputData = new byte[] { ToByte('9'), ToByte('F'), ToByte('8'), ToByte('E'), ToByte('7'), ToByte('D'), ToByte('>') };
            byte[] expected = new byte[] { 159, 142, 125 };
            byte[] received = decoder.Decode(null, inputData, null);

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheInputContainsWhiteSpaces()
        {
            ASCIIHexDecode decoder = new ASCIIHexDecode();
            byte[] inputData = new byte[]
            {
                ToByte(' '), ToByte(' '), ToByte('9'), ToByte('F'), ToByte(' '), ToByte('8'), ToByte(' '),
                ToByte(' '), ToByte('E'), ToByte('7'), ToByte(' '), ToByte('D'), ToByte('>'), ToByte(' '),
            };

            byte[] expected = new byte[] { 159, 142, 125 };
            byte[] received = decoder.Decode(null, inputData, null);

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheInputBytesAreAllWhiteSpacesEndingWithEod()
        {
            ASCIIHexDecode decoder = new ASCIIHexDecode();
            byte[] inputData = new byte[]
            {
                ToByte(' '), ToByte('\n'), ToByte('\r'), ToByte(' '), ToByte(' '), ToByte('>')
            };

            byte[] received = decoder.Decode(null, inputData, null);

            Assert.AreEqual(0, received.Length);
        }
    }
}
