﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;

namespace Telerik.Windows.Documents.Fixed.Tests.Filters
{
    [TestClass]
    public class TestRunLengthDecode : TestFilterBase
    {
        [TestMethod]
        public void WhenTheInputBytesContainOnlyLengthBytesBetween0And127()
        {
            RunLengthDecode decoder = new RunLengthDecode();
            byte[] inputData = new byte[] { 3, ToByte('T'), ToByte('E'), ToByte('S'), ToByte('T'), 0, ToByte('!'), 128 };
            byte[] received = decoder.Decode(null, inputData, null);
            byte[] expected = ToByteArray("TEST!");

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheInputBytesContainOnlyLengthBytesBetween129And255()
        {
            RunLengthDecode decoder = new RunLengthDecode();
            byte[] inputData = new byte[] { 255, ToByte('T'), 253, ToByte('E'), 252, ToByte('S'), 255, ToByte('T'), 254, ToByte('!'),  128};
            byte[] received = decoder.Decode(null, inputData, null);
            byte[] expected = ToByteArray("TTEEEESSSSSTT!!!");

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheInputBytesContainLengthBytesOfAllRanges()
        {
            RunLengthDecode decoder = new RunLengthDecode();
            byte[] inputData = new byte[] { 1, ToByte('T'), ToByte('E'), 254, ToByte('S'), 255, ToByte('T'), 0, ToByte('!'), 128 };
            byte[] received = decoder.Decode(null, inputData, null);
            byte[] expected = ToByteArray("TESSSTT!");

            Assert.AreEqual(expected.Length, received.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], received[i]);
            }
        }

        [TestMethod]
        public void WhenTheFirstByteIsEod()
        {
            RunLengthDecode decoder = new RunLengthDecode();
            byte[] inputData = new byte[] { 128, ToByte('T'), ToByte('E'), 254, ToByte('S'), 255, ToByte('T'), 0, ToByte('!'), 128 };
            byte[] received = decoder.Decode(null, inputData, null); 

            Assert.AreEqual(0, received.Length); 
        }
    }
}
