﻿using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using FontFamily = System.Windows.Media.FontFamily;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Editing.Layout;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Editing.Tables;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using System.IO;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class BlockTests : FixedContentEditorTestBase
    {
#if !NETSTANDARD
        [TestMethod]
        public void Test_MeasureBlock_Bug()
        {
            Block block = new Block();
            block.TextProperties.TrySetFont(new FontFamily("Calibri"), FontStyles.Italic, FontWeights.Normal);
            block.TextProperties.FontSize = 14.666666666666666;
            block.InsertText("one of the “lucky” associates. She’s offered an opportunity to work at a legal aid clinic for one year witho");
            block.InsertText("ut pay, after which there would be a slim chance that she’d get her old job back…");

            block.Measure(new Size(624, 294.252083333333));

            Assert.AreEqual(2, block.Lines.Count());
            Assert.AreEqual("one of the “lucky” associates. She’s offered an opportunity to work at a legal aid clinic for one year ", block.Lines.ElementAt(0).ToString());
            Assert.AreEqual("without pay, after which there would be a slim chance that she’d get her old job back…", block.Lines.ElementAt(1).ToString());
        }
#endif

        [TestMethod]
        [Timeout(1000)]
        public void Test_MeasureBlock_0Size()
        {
            Block block = new Block();
            block.InsertText("te xt");
            block.Measure(new Size(0, 0));
        }

        [TestMethod]
        public void Test_LineSplitMergeBack()
        {
            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("t ", properties);
            TextFragmentLayoutElement second = CreateTextFragmentLayoutElement("ala", properties);
            TextFragmentLayoutElement third = CreateTextFragmentLayoutElement("bala", properties);

            LineInfo line = new LineInfo();
            line.Add(first);
            line.Add(second);

            IEnumerable<LayoutElementBase> pending;
            double width = CreateTextFragmentLayoutElement(first.Element.Text + second.Element.Text, properties).Width;
            line = third.CompleteLine(width, line.Width, new List<LayoutElementBase>(line.Elements), out pending);
            Assert.AreEqual(1, line.Elements.Count());
            Assert.AreEqual(first.Element.Text, line.ToString());
            Assert.AreEqual(2, pending.Count());
        }

        [TestMethod]
        public void Test_LineSplitMergeBackWhiteSpace()
        {
            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("t  ", properties);
            TextFragmentLayoutElement second = CreateTextFragmentLayoutElement("  ala", properties);

            LineInfo line = new LineInfo();
            line.Add(first);

            IEnumerable<LayoutElementBase> pending;
            double width = first.TrimmedWidth;
            line = second.CompleteLine(width, line.Width, new List<LayoutElementBase>(line.Elements), out pending);
            Assert.AreEqual(2, line.Elements.Count());
            Assert.AreEqual("t    ", line.ToString());
            Assert.AreEqual(1, pending.Count());
        }

        [TestMethod]
        public void Test_LineSplitMergeBackMultipleWhiteSpace()
        {
            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("test  ", properties);
            TextFragmentLayoutElement second = CreateTextFragmentLayoutElement("   ", properties);
            TextFragmentLayoutElement third = CreateTextFragmentLayoutElement("      ", properties);
            string forthWhite = "  ";
            string forthNonWhite = "ala";
            TextFragmentLayoutElement forth = CreateTextFragmentLayoutElement(String.Format("{0}{1}", forthWhite, forthNonWhite), properties);

            LineInfo line = new LineInfo();
            line.Add(first);

            IEnumerable<LayoutElementBase> pending;
            double width = first.TrimmedWidth;

            line = second.CompleteLine(width, line.Width, new List<LayoutElementBase>(line.Elements), out pending);
            Assert.AreEqual(2, line.Elements.Count());
            Assert.AreEqual(first.Element.Text + second.Element.Text, line.ToString());
            Assert.AreEqual(0, pending.Count());

            line = third.CompleteLine(width, line.Width, new List<LayoutElementBase>(line.Elements), out pending);
            Assert.AreEqual(3, line.Elements.Count());
            Assert.AreEqual(first.Element.Text + second.Element.Text + third.Element.Text, line.ToString());
            Assert.AreEqual(0, pending.Count());

            line = forth.CompleteLine(width, line.Width, new List<LayoutElementBase>(line.Elements), out pending);
            Assert.AreEqual(4, line.Elements.Count());
            Assert.AreEqual(first.Element.Text + second.Element.Text + third.Element.Text + forthWhite, line.ToString());
            Assert.AreEqual(1, pending.Count());
            string pendingText = pending.First().ToString();
            Assert.AreEqual(pendingText, forthNonWhite);
        }

        [TestMethod]
        public void Test_BlockSplitLinesMergeBackMultipleWhiteSpace()
        {
            string[] firstRow =
            {
                "t  ", "   ", "      "
            };

            string lastWordStart = "  ";
            string lastWordEnd = "ala";

            Block block = new Block();
            block.InsertText(firstRow[0]);
            block.InsertText(firstRow[1]);
            block.InsertText(firstRow[2]);
            block.InsertText(string.Format("{0}{1}", lastWordStart, lastWordEnd));

            AssertMeasureInMinimalSize(block);

            Assert.AreEqual(block.Lines.Count(), 1);
            LineInfo line = block.Lines.First();
            Assert.AreEqual(line.Elements.Count(), 4);
            Assert.AreEqual(string.Format("{0}{1}{2}{3}", firstRow[0], firstRow[1], firstRow[2], lastWordStart), line.ToString());
            Assert.AreEqual(block.PendingElements.Count(), 1);
            LayoutElementBase pending = block.PendingElements.First();
            Assert.AreEqual(lastWordEnd, pending.ToString());
        }

        [TestMethod]
        public void Test_BlockLineBreakDoNotMergeBackWhiteSpaceFromTheBeginingOfNextLine()
        {
            string firstLineText = "First line ";
            string secondLineText = "      this line starts with space";

            Block block = new Block();
            block.InsertText(firstLineText);
            block.InsertLineBreak();
            block.InsertText(secondLineText);

            block.Measure();

            LineInfo[] lines = block.Lines.ToArray();

            Assert.AreEqual(2, lines.Length);

            LineInfo firstLine = lines[0];
            LayoutElementBase[] firstElements = firstLine.Elements.ToArray();
            Assert.AreEqual(2, firstElements.Length);

            TextFragmentLayoutElement firstTextElement = (TextFragmentLayoutElement)firstElements[0];
            Assert.AreEqual(firstTextElement.Element.Text, firstLineText);
            Assert.IsTrue(firstElements[1] is LineBreakLayoutElement);

            LineInfo secondLine = lines[1];
            LayoutElementBase[] secondElements = secondLine.Elements.ToArray();
            Assert.AreEqual(1, secondElements.Length);
            TextFragmentLayoutElement secondTextElement = (TextFragmentLayoutElement)secondElements[0];
            Assert.AreEqual(secondTextElement.Element.Text, secondLineText);
        }

        [TestMethod]
        public void Test_CompleteLineBySplittingCurrentLineElements()
        {
            string firstWord = "Calculated ";
            string secondWord = "Deviation";

            Block measuringBlock = new Block();
            measuringBlock.InsertText(firstWord + "De");
            measuringBlock.Measure(new Size(double.MaxValue, double.MaxValue));

            string text = firstWord + secondWord;
            Block block = new Block();
            for (int i = 0; i < text.Length; i++)
            {
                block.InsertText(text[i].ToString());
            }

            block.Measure(new Size(measuringBlock.DesiredSize.Width, double.MaxValue));

            LineInfo[] lines = block.Lines.ToArray();
            Assert.AreEqual(2, lines.Length);
            Assert.AreEqual(firstWord, lines[0].ToString());
            Assert.AreEqual(secondWord, lines[1].ToString());
        }

        [TestMethod]
        public void TestBlockZeroWidthMeasureSplitToLines()
        {
            Block block = new Block();
            block.InsertText("test11 ab cd   ef ghi  ");
            block.Measure(new Size(0, double.MaxValue));

            string[] expectedLines = new string[]
            {
                "t", "e", "s", "t", "1", "1 ", "a", "b ", "c", "d   ", "e", "f ",  "g", "h", "i  ",
            };

            AssertBlockExpectedLines(block, expectedLines);
        }

        [TestMethod]
        public void TestBlockMergeBackAfterZeroWidthMeasureSplitToLines()
        {
            Block block = new Block();
            block.InsertText("test11 ab cd   ef ghi  ");
            block.Measure(new Size(0, double.PositiveInfinity));

            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("test11 ab cd", properties);
            double width = first.Width;

            block.Measure(new Size(width, double.PositiveInfinity));

            string[] expectedLines = new string[]
            {
                "test11 ab cd   ", "ef ghi  ",
            };

            AssertBlockExpectedLines(block, expectedLines);
        }

        [TestMethod]
        public void TestBlockHasSingleLineAfterMergeBackAndMeasureInZeroHeight()
        {
            Block block = new Block();
            block.InsertText("test11 ab cd   ef ghi  ");
            block.Measure(new Size(0, double.PositiveInfinity));

            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("test11 ab cd", properties);
            double width = first.Width;

            block.Measure(new Size(width, 0));

            string[] expectedLines = new string[]
            {
                "test11 ab cd   ",
            };

            AssertBlockExpectedLines(block, expectedLines);
        }

        [TestMethod]
        public void TestBlockWrapFirstSpaceTest()
        {
            Block block = new Block();
            block.InsertText("        test");

            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("       ", properties);
            double width = first.Width;

            block.Measure(new Size(width, double.PositiveInfinity));

            string[] expectedLines = new string[]
            {
                "        ", "test",
            };

            AssertBlockExpectedLines(block, expectedLines);
        }

        [TestMethod]
        public void TestBlockMeasureZeroSizeWhenStartWithWhitespaceTest()
        {
            Block block = new Block();
            block.InsertText("        test  ds  ");

            block.Measure(new Size(0, 0));

            string[] expectedLines = new string[]
            {
                "        ",
            };

            AssertMeasureInMinimalSize(block);
            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);
            string pendingString = block.PendingElements.First().ToString();
            Assert.AreEqual("test  ds  ", pendingString);
        }

        [TestMethod]
        public void TestBlockMeasureZeroSizeWhenStartWithSingleNonWhitespaceTest()
        {
            Block block = new Block();
            block.InsertText("a        test  ds  ");

            string[] expectedLines = new string[]
            {
                "a        ",
            };

            AssertMeasureInMinimalSize(block);
            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);
            string pendingString = block.PendingElements.First().ToString();
            Assert.AreEqual("test  ds  ", pendingString);
        }

        [TestMethod]
        public void TestBlockMeasureZeroSizeWhenStartWithTwoNonWhitespaceTest()
        {
            Block block = new Block();
            block.InsertText("ab        test  ds  ");

            string[] expectedLines = new string[]
            {
                "a",
            };

            AssertMeasureInMinimalSize(block);
            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);
            string pendingString = block.PendingElements.First().ToString();
            Assert.AreEqual("b        test  ds  ", pendingString);
        }

        [TestMethod]
        public void TestBlockMeasureStartingWithWhiteSpaceTest()
        {
            Block block = new Block();
            block.InsertText("        test  ab   c  de ");

            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("        test  ab   c", properties);
            double width = first.Width;

            block.Measure(new Size(width, 0));

            string[] expectedLines = new string[]
            {
                "        test  ab   c  ",
            };

            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);
            string pendingString = block.PendingElements.First().ToString();
            Assert.AreEqual("de ", pendingString);
        }

        [TestMethod]
        public void TestMeasureBlockWithSpacesAndDashes()
        {
            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("test43 - test54 lo", properties);
            double width = first.Width;

            Block block = new Block();
            block.InsertText("test43 - test54 long");
            block.Measure(new Size(width, double.PositiveInfinity));

            string[] expectedLines = new string[]
            {
                "test43 - test54 ", "long",
            };

            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(0, pendingCount);
        }

        [TestMethod]
        public void TestMeasureBlockWithMultipleTextInsert()
        {
            TextProperties properties = new TextProperties();
            TextFragmentLayoutElement first = CreateTextFragmentLayoutElement("test43 - test54 lo", properties);
            double width = first.Width;

            Block block = new Block();
            block.InsertText("test43 ");
            block.InsertText("-");
            block.InsertText(" test54 long");
            block.Measure(new Size(width, double.PositiveInfinity));

            string[] expectedLines = new string[]
            {
                "test43 - test54 ", "long",
            };

            AssertBlockExpectedLines(block, expectedLines);
            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(0, pendingCount);
        }

        [TestMethod]
        public void Test_Block_SpacingBefore()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();
            block.GraphicProperties.IsStroked = false;
            block.InsertRectangle(new Rect(0, 0, 10, 10));
            block.InsertRectangle(new Rect(0, 0, 50, 50));
            block.SpacingBefore = 50;
            editor.DrawBlock(block, new Size(50, 130));

            Assert.AreEqual(2, document.Pages[0].Content.Count);
        }

        [TestMethod]
        public void Test_Block_SpacingAfter()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();
            block.GraphicProperties.IsStroked = false;
            block.InsertRectangle(new Rect(0, 0, 10, 10));
            block.InsertRectangle(new Rect(0, 0, 50, 50));
            block.SpacingAfter = 50;
            editor.DrawBlock(block, new Size(50, 70));

            Assert.AreEqual(2, document.Pages[0].Content.Count);
        }

        [TestMethod]
        public void Test_Fallback_SameHeight()
        {
            Block block = new Block();
            block.InsertText("H");

            Block fallback = new Block();
            fallback.InsertText("Х");

            Assert.AreEqual(block.Measure().Height, fallback.Measure().Height);
        }

        [TestMethod]
        public void Test_Fallback_RespectFontStyle()
        {
            RadFixedPage page1 = new RadFixedPage();
            FixedContentEditor editor1 = new FixedContentEditor(page1);
            editor1.TextProperties.Font = FontsRepository.HelveticaOblique;
            editor1.DrawText("H");

            RadFixedPage page2 = new RadFixedPage();
            FixedContentEditor editor2 = new FixedContentEditor(page2);
            editor2.TextProperties.Font = FontsRepository.HelveticaOblique;
            editor2.DrawText("Х");

            TextFragment original = page1.Content[0] as TextFragment;
            TextFragment fallback = page2.Content[0] as TextFragment;

            Assert.AreEqual(original.Font.IsItalic, fallback.Font.IsItalic);
        }

        [TestMethod]
        public void Test_Fallback_RespectFontWeight()
        {
            RadFixedPage page1 = new RadFixedPage();
            FixedContentEditor editor1 = new FixedContentEditor(page1);
            editor1.TextProperties.Font = FontsRepository.HelveticaBold;
            editor1.DrawText("H");

            RadFixedPage page2 = new RadFixedPage();
            FixedContentEditor editor2 = new FixedContentEditor(page2);
            editor2.TextProperties.Font = FontsRepository.HelveticaBold;
            editor2.DrawText("Х");

            TextFragment original = page1.Content[0] as TextFragment;
            TextFragment fallback = page2.Content[0] as TextFragment;

            Assert.AreEqual(original.Font.IsBold, fallback.Font.IsBold);
        }

        [TestMethod]
        public void Test_Fallback_RespectFontWeightAndFontStyle()
        {
            RadFixedPage page1 = new RadFixedPage();
            FixedContentEditor editor1 = new FixedContentEditor(page1);
            editor1.TextProperties.Font = FontsRepository.HelveticaBoldOblique;
            editor1.DrawText("H");

            RadFixedPage page2 = new RadFixedPage();
            FixedContentEditor editor2 = new FixedContentEditor(page2);
            editor2.TextProperties.Font = FontsRepository.HelveticaBoldOblique;
            editor2.DrawText("Х");

            TextFragment original = page1.Content[0] as TextFragment;
            TextFragment fallback = page2.Content[0] as TextFragment;

            Assert.AreEqual(original.Font.IsBold, fallback.Font.IsBold);
            Assert.AreEqual(original.Font.IsItalic, fallback.Font.IsItalic);
        }

        [TestMethod]
        public void Test_Fallback_LinesHeight()
        {
            Block block = new Block();
            block.InsertText("-b");
            block.InsertLineBreak();
            block.InsertText("a");

            block.Measure();

            LineInfo firstLine = block.Lines.First();
            LineInfo secondLine = block.Lines.Last();

            Assert.AreEqual(firstLine.BaselineOffset, secondLine.BaselineOffset);
            Assert.AreEqual(firstLine.Descent, secondLine.Descent);
            Assert.AreEqual(firstLine.UnderlinePosition, secondLine.UnderlinePosition);
            Assert.AreEqual(firstLine.UnderlineThickness, secondLine.UnderlineThickness);
        }

        [TestMethod]
        public void TestWordBreakWhenNotEnoughWidthOneLetterPending()
        {
            Block block = new Block();
            block.InsertText("t");
            block.InsertText("est1");
            Size smallSize = block.Measure();
            block.InsertText("2");
            double bigWidth = block.Measure().Width;

            double middleWidth = (bigWidth + smallSize.Width) / 2;
            double middleHeight = smallSize.Height / 2;

            block.Measure(new Size(middleWidth, middleHeight));

            int linesCount = block.Lines.Count();
            Assert.AreEqual(1, linesCount);

            string firstLine = block.Lines.First().ToString();
            Assert.AreEqual("test1", firstLine);

            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);

            string firstPending = block.PendingElements.First().ToString();
            Assert.AreEqual("2", firstPending);
        }

        [TestMethod]
        public void TestWordBreakWhenNotEnoughWidthTwoLettersPending()
        {
            Block block = new Block();
            block.InsertText("t");
            block.InsertText("est");
            Size smallSize = block.Measure();
            block.InsertText("42");

            Block longBlock = new Block();
            longBlock.InsertText("test4");
            double bigWidth = longBlock.Measure().Width;

            double middleWidth = (bigWidth + smallSize.Width) / 2;
            double middleHeight = smallSize.Height / 2;

            block.Measure(new Size(middleWidth, middleHeight));

            int linesCount = block.Lines.Count();
            Assert.AreEqual(1, linesCount);

            string firstLine = block.Lines.First().ToString();
            Assert.AreEqual("test", firstLine);

            int pendingCount = block.PendingElements.Count();
            Assert.AreEqual(1, pendingCount);

            string firstPending = block.PendingElements.First().ToString();
            Assert.AreEqual("42", firstPending);
        }

#if !NETSTANDARD
        [TestMethod]
        public void Test_Wrap_UsingFallbackFont()
        {
            Block block = new Block();
            block.InsertText("trтп");

            Block block2 = new Block();
            block2.InsertText("trт");
            double width = block2.Measure().Width;

            block.Measure(new Size(width, double.PositiveInfinity));

            LineInfo line1 = block.Lines.First();
            LineInfo line2 = block.Lines.Last();

            Assert.AreEqual(2, line1.Elements.Count());
            Assert.AreEqual(1, line2.Elements.Count());
            Assert.AreEqual("tr", line1.Elements.First().ToString());
            Assert.AreEqual("т", line1.Elements.Last().ToString());
            Assert.AreEqual("п", line2.Elements.First().ToString());
        }
#endif

        [TestMethod]
        public void SplitEmptyBlockTest()
        {
            Block block = new Block();
            Assert.IsTrue(block.IsEmpty);

            block = block.Split();
            Assert.IsTrue(block.IsEmpty);
        }

        [TestMethod]
        public void Measure_WithNegativeLeftIndent_DecreaseWidthWithTheIndent()
        {
            // Arrange
            Block block = new Block();
            block.InsertText("AAA");
            Size originalsDesiredSize = block.Measure();

            block.LeftIndent = -20;

            // Act
            Size desiredSize = block.Measure();

            // Assert
            double widthDifference = originalsDesiredSize.Width - desiredSize.Width;
            Assert.AreEqual(20, widthDifference, 0.001);
        }

        [TestMethod]
        public void Measure_WithNegativeLeftIndentLargerThanContentWidth_ReturnsZeroWidth()
        {
            // Arrange
            Block block = new Block();
            block.InsertText("A");
            block.LeftIndent = -20;

            // Act
            Size desiredSize = block.Measure();

            // Assert
            Assert.AreEqual(0, desiredSize.Width);
        }

        [TestMethod]
        public void Measure_WithPositiveRightIndent_IncreaseWidthWithTheIndent()
        {
            // Arrange
            Block block = new Block();
            block.InsertText("AAA");
            Size originalsDesiredSize = block.Measure();

            block.RightIndent = 20;

            // Act
            Size desiredSize = block.Measure();

            // Assert
            double widthDifference = desiredSize.Width - originalsDesiredSize.Width;
            Assert.AreEqual(20, widthDifference, 0.001);
        }

        [TestMethod]
        public void Measure_WithNegativeRightIndent_RightIndentIsNotRespected()
        {
            // Arrange
            Block block = new Block();
            block.InsertText("A");
            Size originalDesiredSize = block.Measure();

            block.RightIndent = -20;

            // Act
            Size desiredSize = block.Measure();

            // Assert
            double widthDifference = originalDesiredSize.Width - desiredSize.Width;
            Assert.AreEqual(0, widthDifference, 0.001);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("id:155223 tfs: 234660")]
        public void ApplyMixedIndentsOfBlockPdfProcessingTest()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Add block with negative right indent
                AddBlock(editor, 0, -30);

                // Add block with negative left indent
                AddBlock(editor, -30, 0);

                // Add block with negative left and right indent
                AddBlock(editor, -20, -20);

                // Add block with positive left and negative right indent
                AddBlock(editor, 20, -20);

                // Add block with negative left and positive right indent
                AddBlock(editor, -20, 20);
            }
#if VISUAL
            this.AssertVisual(document);
#endif
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("id:155223 tfs: 234660")]
        public void ApplyMixedIndentsOfBlockInTablePdfProcessingTest()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine("AutoFitTable");

                Table tableAutoFit = new Table();
                CreateTableRow(tableAutoFit, new Block[] { CreateBlock(0, -20), CreateBlock(-20, 0), CreateBlock(-20, -20) });
                CreateTableRow(tableAutoFit, new Block[] { CreateBlock(-20, -20), CreateBlock(20, -10), CreateBlock(-20, 10) });
                SetTableProperties(tableAutoFit, TableLayoutType.AutoFit);

                editor.InsertTable(tableAutoFit);
                editor.InsertLine("FixedWidthTable");

                Table tableFixedWidth = new Table();
                // tableFixedWidth.Margin = new Thickness(-40, 0, 0, 0);
                CreateTableRow(tableFixedWidth, new Block[] { CreateBlock(0, -20), CreateBlock(-20, 0), CreateBlock(-20, -20) });
                CreateTableRow(tableFixedWidth, new Block[] { CreateBlock(-20, -20), CreateBlock(20, -10), CreateBlock(-20, 10) });
                SetTableProperties(tableFixedWidth, TableLayoutType.FixedWidth);

                editor.InsertTable(tableFixedWidth);
            }
#if VISUAL
            this.AssertAreVisualyEqual(document);
#endif
        }

        [TestMethod]
        public void CreateTextFragment_CopiesAllPropertiesFromTheBlockTextPropertiesIntoTheTextFragmentTextProperties()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();

            block.TextProperties.CharacterSpacing = 2.3;

            block.TextProperties.TrySetFont(new FontFamily("Calibri"), FontStyles.Italic, FontWeights.Normal);

            block.TextProperties.FontSize = 66.3;
            block.TextProperties.HighlightColor = new LinearGradient(new Point(0.4, 0.4), new Point(1.6, 1.6));
            block.TextProperties.HorizontalScaling = 1.3;
            block.TextProperties.RenderingMode = RenderingMode.StrokeAndAddToClippingPath;
            block.TextProperties.UnderlineColor = new LinearGradient(new Point(2.4, 2.4), new Point(3.9, 7.3));
            block.TextProperties.UnderlinePattern = UnderlinePattern.Single;
            block.TextProperties.WordSpacing = 18.7;

            block.InsertText("You can easily insert list items with RadFixedDocumentEditor. The first thing you have to do is add a List in editor’s ListCollection by using the Lists property. Then, each time you want to add list item you have to set the appropriate ListId");

            editor.DrawBlock(block, new Size(50, 70));

            foreach (var textFragment in document.Pages[0].Content.OfType<TextFragment>())
            {
                Assert.AreEqual(block.TextProperties.CharacterSpacing, textFragment.TextProperties.CharacterSpacing);

                Assert.AreEqual(block.TextProperties.Font, textFragment.TextProperties.Font);
                Assert.AreEqual(block.TextProperties.FontSize, textFragment.TextProperties.FontSize);

                Assert.AreEqual(block.TextProperties.HorizontalScaling, textFragment.TextProperties.HorizontalScaling);
                Assert.AreEqual(block.TextProperties.RenderingMode, textFragment.TextProperties.RenderingMode);

                Assert.AreEqual(block.TextProperties.WordSpacing, textFragment.TextProperties.WordSpacing);
            }
        }

        [TestMethod]
        public void CreateTextFragment_CopiesAllPropertiesFromTheBlockGraphicPropertiesIntoTheTextFragmentTextPropertiesGeometryProperties()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();

            // Test with clear rendering mode, because it overrides the values of IsStroked and IsFilled.
            block.TextProperties.RenderingMode = RenderingMode.None;

            block.GraphicProperties.FillColor = new LinearGradient(new Point(7.4, 7.4), new Point(5.6, 5.6));
            block.GraphicProperties.IsFilled = false;
            block.GraphicProperties.IsStroked = false;

            block.GraphicProperties.MiterLimit = 5.2;
            block.GraphicProperties.StrokeColor = new LinearGradient(new Point(0.4, 0.4), new Point(1.6, 1.6));
            block.GraphicProperties.StrokeDashArray = new[] { 1.2, 3.4, 5.6 };
            block.GraphicProperties.StrokeDashOffset = 2;
            block.GraphicProperties.StrokeLineCap = LineCap.Round;
            block.GraphicProperties.StrokeLineJoin = LineJoin.Round;

            block.GraphicProperties.StrokeThickness = 7;

            block.InsertText("You can easily insert list items with RadFixedDocumentEditor. The first thing you have to do is add a List in editor’s ListCollection by using the Lists property. Then, each time you want to add list item you have to set the appropriate ListId");

            editor.DrawBlock(block, new Size(50, 70));

            foreach (var textFragment in document.Pages[0].Content.OfType<TextFragment>())
            {
                Assert.AreEqual(block.GraphicProperties.FillColor, textFragment.TextProperties.GeometryProperties.Fill);
                Assert.AreEqual(block.GraphicProperties.IsFilled, textFragment.TextProperties.GeometryProperties.IsFilled);
                Assert.AreEqual(block.GraphicProperties.IsStroked, textFragment.TextProperties.GeometryProperties.IsStroked);
                Assert.AreEqual(block.GraphicProperties.MiterLimit, textFragment.TextProperties.GeometryProperties.MiterLimit);
                Assert.AreEqual(block.GraphicProperties.StrokeColor, textFragment.TextProperties.GeometryProperties.Stroke);
                Assert.AreEqual(block.GraphicProperties.StrokeDashArray, textFragment.TextProperties.GeometryProperties.StrokeDashArray);
                Assert.AreEqual(block.GraphicProperties.StrokeDashOffset, textFragment.TextProperties.GeometryProperties.StrokeDashOffset);
                Assert.AreEqual(block.GraphicProperties.StrokeLineCap, textFragment.TextProperties.GeometryProperties.StrokeLineCap);
                Assert.AreEqual(block.GraphicProperties.StrokeLineJoin, textFragment.TextProperties.GeometryProperties.StrokeLineJoin);
                Assert.AreEqual(block.GraphicProperties.StrokeThickness, textFragment.TextProperties.GeometryProperties.StrokeThickness);
            }
        }

        [TestMethod]
        public void TextFragment_GeometryPropertiesIsFilled_DependsOnTheTextPropertiesRenderingMode()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();

            block.TextProperties.RenderingMode = RenderingMode.Fill;
            block.GraphicProperties.IsFilled = false;

            block.InsertText("one");
            editor.DrawBlock(block, new Size(50, 70));

            foreach (var textFragment in document.Pages[0].Content.OfType<TextFragment>())
            {
                Assert.AreEqual(true, textFragment.TextProperties.GeometryProperties.IsFilled);
            }
        }

        [TestMethod]
        public void TextFragment_GeometryPropertiesIsStroked_DependsOnTheTextPropertiesRenderingMode()
        {
            RadFixedDocument document = new RadFixedDocument();
            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());

            Block block = new Block();

            block.TextProperties.RenderingMode = RenderingMode.Stroke;
            block.GraphicProperties.IsStroked = false;

            block.InsertText("one");
            editor.DrawBlock(block, new Size(50, 70));

            foreach (var textFragment in document.Pages[0].Content.OfType<TextFragment>())
            {
                Assert.AreEqual(true, textFragment.TextProperties.GeometryProperties.IsStroked);
            }
        }

#if VISUAL
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("TFS ID: 288175 - PdfProcessing: Editing API does not apply some graphics properties to text fragments")]
        public void ApplyGraphicPropertiesToTextContentTest()
        {
            string resultFile = GetResultDocumentPath(TestContext.TestName + ".pdf");
            RadFixedDocument document = new RadFixedDocument();

            Block block = new Block();
            block.TextProperties.FontSize = 30;
            block.TextProperties.RenderingMode = RenderingMode.Stroke;
            block.GraphicProperties.StrokeThickness = 2.5;
            block.GraphicProperties.StrokeColor = new RgbColor(0, 0, 255);
            block.InsertText("Sample stroked text.");

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(block);
            }

            PdfFormatProvider provider = new PdfFormatProvider();
            byte[] data;

            using (Stream output = File.Create(resultFile))
            {
                data = provider.Export(document);

            }

            AssertAreVisualyEqual(document);
        }
#endif

        private void SetTableProperties(Table table, TableLayoutType layoutType, int rowCount = 2, int columnCount = 3)
        {
            table.LayoutType = layoutType;
            table.DefaultCellProperties.Padding = new Thickness(5, 0, 5, 0);
            Border cellBorders = new Border(1, BorderStyle.Single, TestHelper.Blue);

            for (int rows = 0; rows < rowCount; rows++)
            {
                for (int columns = 0; columns < columnCount; columns++)
                {
                    table.Rows[rows].Cells[columns].Borders = new TableCellBorders(left: cellBorders, top: cellBorders, right: cellBorders, bottom: cellBorders);
                }
            }
        }

        private static void CreateTableRow(Table table, Block[] blocks)
        {
            TableRow tableRows = table.Rows.AddTableRow();

            for (int j = 0; j < blocks.Length; j++)
            {
                TableCell tableCell = tableRows.Cells.AddTableCell();
                tableCell.Blocks.Add(blocks[j]);
            }
        }

        private static Block CreateBlock(int leftIndent, int rightIndent)
        {
            Block block = new Block();

            block.InsertText(String.Format("Block with left indent: {0} / right indent: {1}", leftIndent, rightIndent));
            block.RightIndent = rightIndent;
            block.LeftIndent = leftIndent;

            return block;
        }

        private static void AddBlock(RadFixedDocumentEditor editor, int leftIndent, int rightIndent)
        {
            editor.InsertBlock(CreateBlock(leftIndent, rightIndent));
        }

        private static void AssertBlockExpectedLines(Block block, string[] expectedLines)
        {
            LineInfo[] actualLines = block.Lines.ToArray();

            Assert.AreEqual(expectedLines.Length, actualLines.Length);

            for (int i = 0; i < expectedLines.Length; i++)
            {
                string expected = expectedLines[i];
                string actual = actualLines[i].ToString();

                Assert.AreEqual(expected, actual);
            }
        }

        private static void AssertMeasureInMinimalSize(Block block)
        {
            Size measureZeroSize = block.Measure(RadFixedDocumentEditor.MinimalMeasureSize);
            Size measureSizeToWrapElements = block.Measure(new Size(0.001, 0.001));

            Assert.AreEqual(measureSizeToWrapElements, measureZeroSize);
        }

        private static TextFragmentLayoutElement CreateTextFragmentLayoutElement(string text, TextProperties properties)
        {
            return new TextFragmentLayoutElement(new TextFragment(text), properties, true);
        }
    }
}
