﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.Fonts
{
    [TestClass]
    public class FontSourceTests
    {
        [TestMethod]
        public void TestMeiryo()
        {
            this.MatchGlyphs("Meiryo");
        }

        [TestMethod]
        public void TestSegoeUI()
        {
            this.MatchGlyphs("Segoe UI");
        }

        [TestMethod]
        public void TestMicrosoftYahei()
        {
            this.MatchGlyphs("Microsoft Yahei");
        }

        [TestMethod]
        public void TestSimSun()
        {
            this.MatchGlyphs("SimSun");
        }

        [TestMethod]
        public void TestSimSun18030()
        {
            this.MatchGlyphs("SimSun-18030");
        }

        [TestMethod]
        public void TestSimSunExtB()
        {
            this.MatchGlyphs("SimSun-ExtB");
        }

        [TestMethod]
        public void TestMingliU()
        {
            this.MatchGlyphs("MingLiU");
        }

        [TestMethod]
        public void TestMicrosoftJhengHei()
        {
            this.MatchGlyphs("Microsoft JhengHei");
        }

        [TestMethod]
        public void TestMSGothic()
        {
            this.MatchGlyphs("MS Gothic");
        }

        [TestMethod]
        public void TestArialUnicodeMS()
        {
            this.MatchGlyphs("Arial Unicode MS");
        }

        public void TestCambriaMath()
        {
            this.MatchGlyphs("Cambria Math");
        }

        [TestMethod]
        public void TestSegoeUISymbol()
        {
            this.MatchGlyphs("Segoe UI Symbol");
        }

        [TestMethod]
        public void TestArial()
        {
            this.MatchGlyphs("Arial");
        }

        [TestMethod]
        public void TestUnicode()
        {
            this.MatchGlyphs("Unicode");
        }

        [TestMethod]
        public void TestLucidaSans()
        {
            this.MatchGlyphs("Lucida Sans");
        }

        [TestMethod]
        public void TestTahoma()
        {
            this.MatchGlyphs("Tahoma");
        } 

        [TestMethod]
        public void TestGulim()
        {
            this.MatchGlyphs("Gulim");
        }

        [TestMethod]
        public void TestMalgunGothic()
        {
            this.MatchGlyphs("Malgun Gothic");
        }

        [TestMethod]
        public void TestSylfaen()
        {
            this.MatchGlyphs("Sylfaen");
        }

        [TestMethod]
        public void TestSimplifiedArabic()
        {
            this.MatchGlyphs("Simplified Arabic");
        }

        [TestMethod]
        public void TestTraditionalArabic()
        {
            this.MatchGlyphs("Traditional Arabic");
        }

        [TestMethod]
        public void TestLucidaSansUnicode()
        {
            this.MatchGlyphs("Lucida Sans Unicode");
        }

        [TestMethod]
        public void TestMicrosoftSansSerif()
        {
            this.MatchGlyphs("Microsoft Sans Serif");
        }

        [TestMethod]
        public void TestMicrosoftUighur()
        {
            this.MatchGlyphs("Microsoft Uighur");
        }

        [TestMethod]
        public void TestVerdana()
        {
            this.MatchGlyphs("Verdana");
        }

        private GlyphTypeface CreateTypeface(string name, FontStyle fontStyle, FontWeight fontWeight, FontStretch fontStretch)
        {
            GlyphTypeface gtf;
            Typeface typeface = new Typeface(new FontFamily(name), fontStyle, fontWeight, fontStretch);
            typeface.TryGetGlyphTypeface(out gtf);

            return gtf;
        }

        private GlyphTypeface CreateTypeface(string name)
        {
            return this.CreateTypeface(name, FontStyles.Normal, FontWeights.Normal, FontStretches.Normal);
        }

        private FontBase CreateFont(string name, FontStyle fontStyle, FontWeight fontWeight)
        {
            FontBase font;
            Telerik.Windows.Documents.Fixed.Model.Fonts.FontsRepository.TryCreateFont(new FontFamily(name), fontStyle, fontWeight, out font);
            return font;
        }

        private FontBase CreateFont(string name)
        {
            return this.CreateFont(name, FontStyles.Normal, FontWeights.Normal);
        }

        private void MatchGlyphs(string name)
        {
            FontBase font = this.CreateFont(name);
            if (font != null)
            {
                GlyphTypeface typeface = this.CreateTypeface(name);
                foreach (var map in typeface.CharacterToGlyphMap)
                {
                    if (map.Key <= ushort.MaxValue)
                    {
                        ushort glyph;
                        font.TryGetGlyphId((ushort)map.Key, out glyph);

                        Assert.AreEqual(glyph, map.Value);
                    }
                }
            }
        }
    }
}