﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Model.Internal.Classes;

namespace Telerik.Windows.Documents.Fixed.Tests.Fonts
{
    [TestClass]
    public class Type0FontOldTests
    {
        [TestMethod]
        public void GetGlyphWidth_AndFontSource_WithDefinedDefaultWidth_DoesNotExtractGlyphWidthsFromFontSource()
        {
            var emptyPdfContentManager = new PdfContentManager();

            var widths = new object[] { };
            var descendantFont = new TestCidFontOld(emptyPdfContentManager)
            {
                Widths = new PdfArrayOld(emptyPdfContentManager, widths),
                DefaultWidth = new PdfIntOld(emptyPdfContentManager, 5)
            };

            var type0Font = new Type0FontOld(emptyPdfContentManager);

            type0Font.DescendantFonts = new PdfArrayOld(emptyPdfContentManager, new[] { descendantFont });

            try
            {
                var glyph = new GlyphOld();

                double width = type0Font.GetGlyphWidth(glyph);

                Assert.AreEqual(0.005, width);
            }
            catch (NotImplementedException)
            {
                Assert.Fail("FontSource is used when extracting glyph widths!");
            }
        }

        [TestMethod]
        public void GetGlyphWidth_AndFontSource_WithNotDefinedDefaultWidth_ExtractsGlyphWidthsFromFontSource()
        {
            var emptyPdfContentManager = new PdfContentManager();

            var widths = new object[] { };
            var descendantFont = new TestCidFontOld(emptyPdfContentManager)
            {
                Widths = new PdfArrayOld(emptyPdfContentManager, widths)
            };

            var type0Font = new Type0FontOld(emptyPdfContentManager);

            type0Font.DescendantFonts = new PdfArrayOld(emptyPdfContentManager, new[] { descendantFont });

            try
            {
                var glyph = new GlyphOld();

                double width = type0Font.GetGlyphWidth(glyph);

                Assert.AreEqual(glyph.AdvancedWidth, width);
            }
            catch (NotImplementedException)
            {
                Assert.Fail("FontSource is used when extracting glyph widths!");
            }
        }

        [TestMethod]
        public void GetGlyphWidth_ReturnsDefaultWidthWhenGlypIsNotPresentInWidths()
        {
            var emptyPdfContentManager = new PdfContentManager();
            var widths = new object[] { 1, new PdfArrayOld(emptyPdfContentManager, new object[] { 224 }) };
            var descendantFont = new TestCidFontOld(emptyPdfContentManager)
            {
                Widths = new PdfArrayOld(emptyPdfContentManager, widths),
                DefaultWidth = new PdfIntOld(emptyPdfContentManager, 1000)
            };

            var type0Font = new Type0FontOld(emptyPdfContentManager);
            type0Font.DescendantFonts = new PdfArrayOld(emptyPdfContentManager, new[] { descendantFont });

            var glyph = new GlyphOld() { CharId = new CharCodeOld((ushort)2) };

            var width = type0Font.GetGlyphWidth(glyph);

            Assert.AreEqual(1, width);
        }

        [TestMethod]
        public void GetGlyphWidth_ReturnsCorrectWidthBasedOnTheWidthsArrayOfItsDescendantFont()
        {
            var emptyPdfContentManager = new PdfContentManager();

            var widths = new object[]
            {
                1, new PdfArrayOld(emptyPdfContentManager, new object[] { 224, 315, 555 }),
                17, 19, 562
            };

            var descendantFont = new TestCidFontOld(emptyPdfContentManager)
            {
                Widths = new PdfArrayOld(emptyPdfContentManager, widths),
                DefaultWidth = new PdfIntOld(emptyPdfContentManager, 1000)
            };

            var type0Font = new Type0FontOld(emptyPdfContentManager);
            type0Font.DescendantFonts = new PdfArrayOld(emptyPdfContentManager, new[] { descendantFont });

            Dictionary<ushort, double> expectedGlyphWidths = new Dictionary<ushort, double>();
            expectedGlyphWidths.Add(1, 0.224);
            expectedGlyphWidths.Add(2, 0.315);
            expectedGlyphWidths.Add(3, 0.555);

            // not present in widths
            expectedGlyphWidths.Add(4, 1);

            expectedGlyphWidths.Add(17, 0.562);
            expectedGlyphWidths.Add(18, 0.562);
            expectedGlyphWidths.Add(19, 0.562);

            foreach (var glyphWidthPair in expectedGlyphWidths)
            {
                var glyph = new GlyphOld() { CharId = new CharCodeOld(glyphWidthPair.Key) };

                var width = type0Font.GetGlyphWidth(glyph);

                Assert.AreEqual(glyphWidthPair.Value, width);
            }
        }
    }
}
