﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat.CFFTables;

namespace Telerik.Windows.Documents.Fixed.Tests.Fonts.Type1.CFFFormat.CFFTables
{
    [TestClass]
    public class CharsetTests
    {
        [TestMethod]
        public void TryGetGlyphIdByCharacterName_WhenFound_ShouldReturnCorrectGlyphId()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, ushort.MaxValue };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], "third" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterName("first", out firstGlyphId);

            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterName("second", out secondGlyphId);

            ushort thirdGlyphId;
            charset.TryGetGlyphIdByCharacterName("third", out thirdGlyphId);

            // Assert
            Assert.AreEqual(1, firstGlyphId);
            Assert.AreEqual(2, secondGlyphId);
            Assert.AreEqual(3, thirdGlyphId);
        }

        [TestMethod]
        public void TryGetGlyphIdByCharacterName_WhenContainsNotDefGlyph_ShouldReturnCorrectGlyphId()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, 0 };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], ".notdef" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterName("first", out firstGlyphId);

            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterName("second", out secondGlyphId);

            ushort thirdGlyphId;
            charset.TryGetGlyphIdByCharacterName(".notdef", out thirdGlyphId);

            // Assert
            Assert.AreEqual(1, firstGlyphId);
            Assert.AreEqual(2, secondGlyphId);
            Assert.AreEqual(3, thirdGlyphId);
        }

        [TestMethod]
        public void TryGetGlyphIdByCharacterName_WhenNotFound_ShouldReturnZero()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, ushort.MaxValue };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], "third" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterName("notfound", out firstGlyphId);
            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterName(".notdef", out secondGlyphId);

            // Assert
            Assert.AreEqual(0, firstGlyphId);
            Assert.AreEqual(0, secondGlyphId);
        }

        [TestMethod]
        public void TryGetGlyphIdByCharacterIdentifier_WhenFound_ShouldReturnCorrectGlyphId()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, ushort.MaxValue };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], "third" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(5, out firstGlyphId);

            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(1000, out secondGlyphId);

            ushort thirdGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(ushort.MaxValue, out thirdGlyphId);

            // Assert
            Assert.AreEqual(1, firstGlyphId);
            Assert.AreEqual(2, secondGlyphId);
            Assert.AreEqual(3, thirdGlyphId);
        }

        [TestMethod]
        public void TryGetGlyphIdByCharacterIdentifier_WhenNotFound_ShouldReturnZero()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, ushort.MaxValue };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], "third" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(0, out firstGlyphId);
            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(15, out secondGlyphId);

            // Assert
            Assert.AreEqual(0, firstGlyphId);
            Assert.AreEqual(0, secondGlyphId);

            // Assert
            Assert.AreEqual(0, firstGlyphId);
            Assert.AreEqual(0, secondGlyphId);
        }

        [TestMethod]
        public void TryGetGlyphIdByCharacterIdentifier_WhenContainsNotDefGlyph_ShouldReturnCorrectGlyphId()
        {
            // Arrange
            ushort[] glyphs = new ushort[] { 5, 1000, 0 };
            Dictionary<int, string> charEncodingToName = new Dictionary<int, string>
            {
                { glyphs[0], "first" },
                { glyphs[1], "second" },
                { glyphs[2], ".notdef" }
            };

            TestCFFFontFile cFFFontFile = new TestCFFFontFile(charEncodingToName);
            Charset charset = new Charset(cFFFontFile, glyphs);

            // Act
            ushort firstGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(5, out firstGlyphId);

            ushort secondGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(1000, out secondGlyphId);

            ushort thirdGlyphId;
            charset.TryGetGlyphIdByCharacterIdentifier(0, out thirdGlyphId);

            // Assert
            Assert.AreEqual(1, firstGlyphId);
            Assert.AreEqual(2, secondGlyphId);
            Assert.AreEqual(3, thirdGlyphId);
        }
    }
}
