﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Windows;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Old.Pdf.PdfReader;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Old.Pdf.DocumentModel.Data
{
    [TestClass]
    public class PdfObjectOldTests
    {
        [TestMethod]
        public void ToRect_ForIndirectReference_WithoutIndirectObjectValue_ReturnsZeroRectangle()
        {
            IndirectReferenceOld referenceX1 = new IndirectReferenceOld() { ObjectNumber = 1 };
            IndirectReferenceOld referenceY1 = new IndirectReferenceOld() { ObjectNumber = 2 };
            IndirectReferenceOld referenceX2 = new IndirectReferenceOld() { ObjectNumber = 3 };
            IndirectReferenceOld referenceY2 = new IndirectReferenceOld() { ObjectNumber = 4 };

            PdfContentManagerMockForIndirectReferences contentManager = new PdfContentManagerMockForIndirectReferences();
            // Do Not regiester anything

            PdfArrayOld pdfArray = new PdfArrayOld(contentManager, new object[] { referenceX1, referenceY1, referenceX2, referenceY2 });
            Rect rect = pdfArray.ToRect();
            Rect expectedRect = new Rect();

            Assert.AreEqual(expectedRect, rect);
        }

        [TestMethod]
        public void ToRect_WhenReadRectangleValuesAreNOTIndirectReferences_ReturnsCorrectRectangle()
        {
            PdfContentManagerMockForIndirectReferences contentManager = new PdfContentManagerMockForIndirectReferences();

            PdfArrayOld pdfArray = new PdfArrayOld(contentManager, new object[] { 1, 2.2, 3.1, 6.4 });
            Rect rect = pdfArray.ToRect();
            Rect expectedRect = new Rect(new Point(1.0, 2.2), new Point(3.1, 6.4));

            Assert.AreEqual(expectedRect, rect);
        }

        [TestMethod]
        public void ToRect_WhenReadRectangleValuesAreIndirectReferences_ReturnsCorrectRectangle()
        {
            IndirectReferenceOld referenceX1 = new IndirectReferenceOld() { ObjectNumber = 1 };
            IndirectReferenceOld referenceY1 = new IndirectReferenceOld() { ObjectNumber = 2 };
            IndirectReferenceOld referenceX2 = new IndirectReferenceOld() { ObjectNumber = 3 };
            IndirectReferenceOld referenceY2 = new IndirectReferenceOld() { ObjectNumber = 4 };

            PdfContentManagerMockForIndirectReferences contentManager = new PdfContentManagerMockForIndirectReferences();
            contentManager.RegisterMockIndirectObject(referenceX1, 1);
            contentManager.RegisterMockIndirectObject(referenceY1, 2.2);
            contentManager.RegisterMockIndirectObject(referenceX2, 3.1);
            contentManager.RegisterMockIndirectObject(referenceY2, 6.4);

            PdfArrayOld pdfArray = new PdfArrayOld(contentManager, new object[] { referenceX1, referenceY1, referenceX2, referenceY2 });

            Rect rect = pdfArray.ToRect();
            Rect expectedRect = new Rect(new Point(1.0, 2.2), new Point(3.1, 6.4));

            Assert.AreEqual(expectedRect, rect);
        }
    }
}
