﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export
{
    [TestClass]
    public class PdfLiteralStringTests
    {
        private const byte LeftBracketByte = (byte)'(';
        private const byte RightBracketByte = (byte)')';
        private const byte BackslashByte = (byte)'\\';

        [TestMethod]
        public void Write_WhenValueContainsBackslash_BackslashIsEscapedWithBackslash()
        {
            // Arrange
            string value = @"\";
            PdfLiteralString literalString = new PdfLiteralString(value, StringType.ASCII);

            using (MemoryStream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);

                // Act
                literalString.Write(writer, new TestPdfExportContext());

                // Assert
                CollectionAssert.AreEqual(new byte[] { LeftBracketByte, BackslashByte, BackslashByte, RightBracketByte }, stream.ToArray());
            }
        }

        [TestMethod]
        public void Write_WhenValueContainsLeftBracket_LeftBracketIsEscapedWithBackslash()
        {
            // Arrange
            string value = @"(";
            PdfLiteralString literalString = new PdfLiteralString(value, StringType.ASCII);

            using (MemoryStream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);

                // Act
                literalString.Write(writer, new TestPdfExportContext());

                // Assert
                CollectionAssert.AreEqual(new byte[] { LeftBracketByte, BackslashByte, LeftBracketByte, RightBracketByte }, stream.ToArray());
            }
        }

        [TestMethod]
        public void Write_WhenValueContainsRightBracket_RightBracketIsEscapedWithBackslash()
        {
            // Arrange
            string value = @")";
            PdfLiteralString literalString = new PdfLiteralString(value, StringType.ASCII);

            using (MemoryStream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);

                // Act
                literalString.Write(writer, new TestPdfExportContext());

                // Assert
                CollectionAssert.AreEqual(new byte[] { LeftBracketByte, BackslashByte, RightBracketByte, RightBracketByte }, stream.ToArray());
            }
        }

        [TestMethod]
        public void Write_WhenValueContainsMultipleSpecialSymbols_TheyAreEscapedWithBackslashes()
        {
            // Arrange
            string value = @"a)b(a\\";
            PdfLiteralString literalString = new PdfLiteralString(value, StringType.ASCII);

            using (MemoryStream stream = new MemoryStream())
            {
                PdfWriter writer = new PdfWriter(stream);

                // Act
                literalString.Write(writer, new TestPdfExportContext());

                // Assert
                CollectionAssert.AreEqual(
                    new byte[] {
                        LeftBracketByte,
                        (byte)'a',
                        BackslashByte,
                        RightBracketByte,
                        (byte)'b',
                        BackslashByte,
                        LeftBracketByte,
                        (byte)'a',
                        BackslashByte,
                        BackslashByte,
                        BackslashByte,
                        BackslashByte,
                        RightBracketByte
                    },
                    stream.ToArray());
            }
        }
    }
}
