﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export
{
    [TestClass]
    public class PdfWirterTests
    {
        [TestMethod]
        public void Write_WithALetter_WritesTheLetter()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.Write("a");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("a"), output.ToArray());
            }
        }

        [TestMethod]
        public void Write_WithSpecialSymbol_WritesTheSymbol()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.Write("{0}");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("{0}"), output.ToArray());
            }
        }

        [TestMethod]
        public void Write_WithFormattedTextAndArguments_WritesTheFormattedText()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.Write("{0}", "a");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("a"), output.ToArray());
            }
        }

        [TestMethod]
        public void WriteLine_WithALetter_WritesTheLetter()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.WriteLine("a");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("a\r\n"), output.ToArray());
            }
        }

        [TestMethod]
        public void WriteLine_WithSpecialSymbol_WritesTheSymbol()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.WriteLine("{0}");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("{0}\r\n"), output.ToArray());
            }
        }

        [TestMethod]
        public void WriteLine_WithFormattedTextAndArguments_WritesTheFormattedText()
        {
            using (MemoryStream output = new MemoryStream())
            {
                PdfWriter pdfWriter = new PdfWriter(output);
                pdfWriter.WriteLine("{0}", "a");

                CollectionAssert.AreEqual(Encoding.ASCII.GetBytes("a\r\n"), output.ToArray());
            }
        }
    }
}
