﻿using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export.Writers;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export.Writers
{
    [TestClass]
    public class CMapWriterTests
    {
        private const string CMapTemplate = @"/CIDInit /ProcSet findresource begin
12 dict begin
begincmap
/CIDSystemInfo << /Registry (Adobe)/Ordering (Identity)/Supplement 0>> def
/CMapName /Adobe-Identity-UCS def
/CMapType 2 def
1 begincodespacerange
<0000><ffff>
endcodespacerange
{0}
endcmap CMapName currentdict /CMap defineresource pop end end
";

        [TestMethod]
        public void Write_WhenSequentialUnicodeRanges_ShouldWriteCmapWithCorrectRanges()
        {
            // Arrange
            CMapWriter cMapWriter = new CMapWriter();
            MemoryStream output = new MemoryStream();
            PdfWriter writer = new PdfWriter(output);
            TestPdfExportContext context = new TestPdfExportContext();
            CMapEncoding encoding = new CMapEncoding();
            encoding.AddUnicodeRange(new CharCode(100, 2), new CharCode(101, 2), new List<string>() { "A", "B" });
            encoding.AddUnicodeRange(new CharCode(103, 2), new CharCode(104, 2), new List<string>() { "C", "D" });

            string expectedRange = @"2 beginbfrange
<0064><0065> <0041>
<0067><0068> <0043>
endbfrange";

            // Act
            cMapWriter.Write(writer, context, encoding);

            // Assert
            string result = Encoding.UTF8.GetString(output.ToArray());

            Assert.AreEqual(string.Format(CMapTemplate, expectedRange), result);
        }

        [TestMethod]
        public void Write_WhenNonSequentialUnicodeRanges_ShouldWriteCmapWithCorrectRanges()
        {
            // Arrange
            CMapWriter cMapWriter = new CMapWriter();
            MemoryStream output = new MemoryStream();
            PdfWriter writer = new PdfWriter(output);
            TestPdfExportContext context = new TestPdfExportContext();
            CMapEncoding encoding = new CMapEncoding();
            encoding.AddUnicodeRangeArray(new CharCode(100, 2), new CharCode(101, 2), new List<string>() { "A", "D" });
            encoding.AddUnicodeRangeArray(new CharCode(103, 2), new CharCode(104, 2), new List<string>() { "X", "Z" });

            string expectedRange = @"2 beginbfrange
<0064><0065> [<0041> <0044>]
<0067><0068> [<0058> <005a>]
endbfrange";

            // Act
            cMapWriter.Write(writer, context, encoding);

            // Assert
            string result = Encoding.UTF8.GetString(output.ToArray());

            Assert.AreEqual(string.Format(CMapTemplate, expectedRange), result);
        }

        [TestMethod]
        public void Write_WhenManyRanges_ShouldWriteCmapWithCorrectRanges()
        {
            // Arrange
            CMapWriter cMapWriter = new CMapWriter();
            MemoryStream output = new MemoryStream();
            PdfWriter writer = new PdfWriter(output);
            TestPdfExportContext context = new TestPdfExportContext();
            CMapEncoding encoding = new CMapEncoding();
            encoding.AddUnicodeRangeArray(new CharCode(100, 2), new CharCode(101, 2), new List<string>() { "A", "D" });
            encoding.AddUnicodeRange(new CharCode(102, 2), new CharCode(103, 2), new List<string>() { "F", "G" });
            encoding.AddUnicodeRangeArray(new CharCode(104, 2), new CharCode(105, 2), new List<string>() { "X", "Z" });
            encoding.AddUnicodeRange(new CharCode(106, 2), new CharCode(107, 2), new List<string>() { "H", "Y" });

            string expectedRange = @"4 beginbfrange
<0064><0065> [<0041> <0044>]
<0066><0067> <0046>
<0068><0069> [<0058> <005a>]
<006a><006b> <0048>
endbfrange";

            // Act
            cMapWriter.Write(writer, context, encoding);

            // Assert
            string result = Encoding.UTF8.GetString(output.ToArray());

            Assert.AreEqual(string.Format(CMapTemplate, expectedRange), result);
        }
    }
}
