﻿using System.IO;
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Tests.Import;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Import
{
    [TestClass]
    public class ContentStreamInterpreterTests
    {
        const string StrokeOperatorWithColorSpace = @"/CS0 cs 0 0 0 sc";
        const string RectangleWithStrokeOperatorAndColorSpace = @"/CS0 cs 0 0 0 sc
0 0 10 10 re";

#if DEBUG
        [TestMethod]
        [ExpectedException(typeof(System.NotImplementedException))]
        public void Execute_WhenLabColorSpaceAndDebug_ShouldThrow()
        {
            // Arrange
            TestPdfContentImportContext context = new TestPdfContentImportContext();
            context.NameToColorSpace.Add("CS0", new LabColorSpaceObject());

            ContentStreamInterpreter contentStreamInterpreter = new ContentStreamInterpreter(new MemoryStream(Encoding.UTF8.GetBytes(StrokeOperatorWithColorSpace)), context);

            // Act && Assert
            contentStreamInterpreter.Execute();
        }
#else
        [TestMethod]
        public void Execute_WhenLabColorSpaceAndNotDebug_ShouldNotThrow()
        {
            // Arrange
            TestPdfContentImportContext context = new TestPdfContentImportContext();
            context.NameToColorSpace.Add("CS0", new LabColorSpaceObject());

            ContentStreamInterpreter contentStreamInterpreter = new ContentStreamInterpreter(new MemoryStream(Encoding.UTF8.GetBytes(StrokeOperatorWithColorSpace)), context);

            // Act && Assert
            contentStreamInterpreter.Execute();
        }

        [TestMethod]
        public void Execute_WhenPathFigureWithLabColorSpaceAndNotDebug_ShouldSetPathFigure()
        {
            // Arrange
            TestPdfContentImportContext context = new TestPdfContentImportContext();
            context.NameToColorSpace.Add("CS0", new LabColorSpaceObject());

            ContentStreamInterpreter contentStreamInterpreter = new ContentStreamInterpreter(new MemoryStream(Encoding.UTF8.GetBytes(RectangleWithStrokeOperatorAndColorSpace)), context);

            // Act
            contentStreamInterpreter.Execute();

            // Assert
            Assert.IsNotNull(contentStreamInterpreter.CurrentPathFigure);
        }
#endif
    }
}
