﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Import
{
    [TestClass]
    public class PagesContentCacheTests
    {
        [TestMethod]
        public void TryLoadCachedPageContent_WhenPageNotCached_ShouldReturnFalse()
        {
            // Arrange
            PagesContentCache pagesContentCache = new PagesContentCache();
            RadFixedPage page = new RadFixedPage();

            // Act
            bool isCached = pagesContentCache.TryLoadCachedPageContent(page);

            // Assert
            Assert.IsFalse(isCached);

            Assert.AreEqual(0, page.Content.Count);
            Assert.AreEqual(0, page.Annotations.Count);
        }

        [TestMethod]
        public void TryLoadCachedPageContent_WhenPageContentIsCached_ShouldLoadCachedPageContent()
        {
            // Arrange
            PagesContentCache pagesContentCache = new PagesContentCache();
            RadFixedPage page = new RadFixedPage();
            TextFragment textFragment = page.Content.AddTextFragment("Text");
            pagesContentCache.CachePageContent(page);
            page.Content.Clear();

            // Act
            bool isCached = pagesContentCache.TryLoadCachedPageContent(page);

            // Assert
            Assert.IsTrue(isCached);

            Assert.AreEqual(1, page.Content.Count);
            TextFragment cachedTextFragment = page.Content[0] as TextFragment;
            Assert.IsNotNull(textFragment);
            Assert.AreEqual(textFragment, cachedTextFragment);
        }

        [TestMethod]
        public void TryLoadCachedPageContent_WhenPageAnnotationsAreCached_ShouldLoadCachedPageAnnotations()
        {
            // Arrange
            PagesContentCache pagesContentCache = new PagesContentCache();
            RadFixedPage page = new RadFixedPage();
            TextFragment textFragment = page.Content.AddTextFragment("Text");
            Link link = page.Annotations.AddLink(new GoToAction());
            pagesContentCache.CachePageContent(page);
            page.Annotations.Clear();

            // Act
            bool isCached = pagesContentCache.TryLoadCachedPageContent(page);

            // Assert
            Assert.IsTrue(isCached);

            Assert.AreEqual(1, page.Annotations.Count);
            Annotation cachedLink = page.Annotations[0] as Link;
            Assert.IsNotNull(cachedLink);
            Assert.AreEqual(link, cachedLink);
        }

        [TestMethod]
        public void Clear_ShouldClearCachedPageContent()
        {
            // Arrange
            PagesContentCache pagesContentCache = new PagesContentCache();
            RadFixedPage page = new RadFixedPage();
            page.Content.AddTextFragment("Text");
            page.Annotations.AddLink(new GoToAction());

            pagesContentCache.CachePageContent(page);

            page.Content.Clear();
            page.Annotations.Clear();

            // Act
            pagesContentCache.Clear();

            // Assert
            bool isCached = pagesContentCache.TryLoadCachedPageContent(page);
            Assert.IsFalse(isCached);

            Assert.AreEqual(0, page.Content.Count);
            Assert.AreEqual(0, page.Annotations.Count);
        }
    }
}
