﻿using System.IO;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Import.Parser
{
    [TestClass]
    public class PostScriptReaderTests
    {
        [TestMethod]
        public void Read_WhenReachesEndOfFileBeforeStateTransition_ShouldReturnCorrectTokenResult()
        {
            // Arrange
            MemoryStream stream1 = new MemoryStream(Encoding.ASCII.GetBytes("123456"));
            MemoryStream stream2 = new MemoryStream(Encoding.ASCII.GetBytes("/Name"));
            MemoryStream stream3 = new MemoryStream(Encoding.ASCII.GetBytes("true"));
            MemoryStream stream4 = new MemoryStream(Encoding.ASCII.GetBytes("1.00"));

            PostScriptReader postScriptReader1 = new PostScriptReader(stream1, new KeywordCollection());
            PostScriptReader postScriptReader2 = new PostScriptReader(stream2, new KeywordCollection());
            PostScriptReader postScriptReader3 = new PostScriptReader(stream3, new KeywordCollection());
            PostScriptReader postScriptReader4 = new PostScriptReader(stream4, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            // Act
            PdfInt pdfIntResult = postScriptReader1.Read<PdfInt>(context, PdfElementType.PdfInt);
            PdfName pdfNameResult = postScriptReader2.Read<PdfName>(context, PdfElementType.PdfName);
            PdfBool pdfBoolResult = postScriptReader3.Read<PdfBool>(context, PdfElementType.PdfBool);
            PdfReal pdfRealResult = postScriptReader4.Read<PdfReal>(context, PdfElementType.PdfBool);

            // Assert
            Assert.AreEqual(123456, pdfIntResult.Value);
            Assert.AreEqual("Name", pdfNameResult.Value);
            Assert.AreEqual(true, pdfBoolResult.Value);
            Assert.AreEqual(1, pdfRealResult.Value);
        }

        [TestMethod]
        public void Read_WhenPdfDictionaryWithConcatenatedEmptyNames_ShouldReturnEmptyNameTokens()
        {
            // Arrange
            MemoryStream stream = new MemoryStream(Encoding.ASCII.GetBytes("<< // >>"));

            PostScriptReader postScriptReader = new PostScriptReader(stream, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            // Act
            PdfDictionary pdfDictionary = postScriptReader.Read<PdfDictionary>(context, PdfElementType.Dictionary);

            // Assert
            Assert.AreEqual(1, pdfDictionary.Count);
            string key = pdfDictionary.Keys.First();
            Assert.AreEqual(string.Empty, key);

            PdfName value = pdfDictionary[pdfDictionary.Keys.First()] as PdfName;
            Assert.IsNotNull(value);
            Assert.AreEqual(string.Empty, value.Value);
        }

        [TestMethod]
        public void Read_WhenPdfDictionaryWithConcatenatedEmptyNameAndArrayStart_ShouldReturnCorrectTokens()
        {
            // Arrange
            MemoryStream stream = new MemoryStream(Encoding.ASCII.GetBytes("<< /[] >>"));

            PostScriptReader postScriptReader = new PostScriptReader(stream, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            // Act
            PdfDictionary pdfDictionary = postScriptReader.Read<PdfDictionary>(context, PdfElementType.Dictionary);

            // Assert
            Assert.AreEqual(1, pdfDictionary.Count);
            string key = pdfDictionary.Keys.First();
            Assert.AreEqual(string.Empty, key);

            PdfArray value = pdfDictionary[pdfDictionary.Keys.First()] as PdfArray;
            Assert.IsNotNull(value);
            Assert.AreEqual(0, value.Count);
        }

        [TestMethod]
        public void Read_WhenPdfDictionaryWithConcatenatedEmptyNameAndArrayEnd_ShouldReturnCorrectTokens()
        {
            // Arrange
            MemoryStream stream = new MemoryStream(Encoding.ASCII.GetBytes("[/]"));

            PostScriptReader postScriptReader = new PostScriptReader(stream, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            // Act
            PdfArray pdfArray = postScriptReader.Read<PdfArray>(context, PdfElementType.Dictionary);

            // Assert
            Assert.AreEqual(1, pdfArray.Count);
            PdfName name = pdfArray[0] as PdfName;
            Assert.AreEqual(string.Empty, name.Value);
        }

        [TestMethod]
        public void Read_WhenPdfDictionaryWithConcatenatedEmptyNameAndDictionaryEnd_ShouldReturnCorrectTokens()
        {
            // Arrange
            MemoryStream stream = new MemoryStream(Encoding.ASCII.GetBytes("<<//>>"));

            PostScriptReader postScriptReader = new PostScriptReader(stream, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            // Act
            PdfDictionary pdfDictionary = postScriptReader.Read<PdfDictionary>(context, PdfElementType.Dictionary);

            // Assert
            Assert.AreEqual(1, pdfDictionary.Count);
            string key = pdfDictionary.Keys.First();
            Assert.AreEqual(string.Empty, key);

            PdfName value = pdfDictionary[pdfDictionary.Keys.First()] as PdfName;
            Assert.IsNotNull(value);
            Assert.AreEqual(string.Empty, value.Value);
        }
    }
}
