﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Import
{
    public class TestPageContentManager : IPageContentManager
    {
        private readonly Dictionary<RadFixedPage, int> pageToLoadedContentCount;
        private readonly Dictionary<RadFixedPage, int> pageToUnloadedContentCount;
        private readonly bool throwExceptionOnLoad;

        public TestPageContentManager(bool throwExceptionOnLoad = false)
        {
            this.pageToLoadedContentCount = new Dictionary<RadFixedPage, int>();
            this.pageToUnloadedContentCount = new Dictionary<RadFixedPage, int>();
            this.IsContentEmpty = true;
            this.throwExceptionOnLoad = throwExceptionOnLoad;
        }

        public Dictionary<RadFixedPage, int> PageToLoadedContentCount
        {
            get
            {
                return this.pageToLoadedContentCount;
            }
        }

        public Dictionary<RadFixedPage, int> PageToUnloadedContentCount
        {
            get
            {
                return this.pageToUnloadedContentCount;
            }
        }

        public bool IsContentEmpty { get; set; }

        public void LoadPageContent(RadFixedPage page)
        {
            int loadedContentCount;
            if (!this.PageToLoadedContentCount.TryGetValue(page, out loadedContentCount))
            {
                loadedContentCount = 1;
                this.PageToLoadedContentCount.Add(page, loadedContentCount);
            }
            else
            {
                this.PageToLoadedContentCount[page] = ++loadedContentCount;
            }

            if (this.throwExceptionOnLoad)
            {
                throw new NullReferenceException();
            }
        }

        public void UnloadPageContent(RadFixedPage page)
        {
            int unloadedContentCount;
            if (!this.PageToUnloadedContentCount.TryGetValue(page, out unloadedContentCount))
            {
                unloadedContentCount = 1;
                this.PageToUnloadedContentCount.Add(page, unloadedContentCount);
            }
            else
            {
                this.PageToUnloadedContentCount[page] = ++unloadedContentCount;
            }
        }

        public bool IsPageContentEmpty(RadFixedPage page)
        {
            return this.IsContentEmpty;
        }
    }
}
