﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Import
{
    [TestClass]
    public class ThreadSafePageContentManagerTests
    {
        [TestMethod]
        public void LoadPageContent_WhenPageContentIsEmpty_ShouldLoadPageContent()
        {
            // Arrange
            TestPageContentManager pageContentManager = new TestPageContentManager();
            ThreadSafePageContentManager threadSafePageContentManager = new ThreadSafePageContentManager(pageContentManager);
            RadFixedPage page = new RadFixedPage();

            // Act
            threadSafePageContentManager.LoadPageContent(page);

            // Assert
            int count;
            bool found =pageContentManager.PageToLoadedContentCount.TryGetValue(page, out count);

            Assert.IsTrue(found);
            Assert.AreEqual(1, count);
        }

        [TestMethod]
        public void LoadPageContent_WhenPageContentIsNotEmpty_ShouldNotLoadPageContent()
        {
            // Arrange
            TestPageContentManager pageContentManager = new TestPageContentManager();
            pageContentManager.IsContentEmpty = false;
            ThreadSafePageContentManager threadSafePageContentManager = new ThreadSafePageContentManager(pageContentManager);
            RadFixedPage page = new RadFixedPage();

            // Act
            threadSafePageContentManager.LoadPageContent(page);

            // Assert
            int count;
            bool found = pageContentManager.PageToLoadedContentCount.TryGetValue(page, out count);

            Assert.IsFalse(found);
        }

        [TestMethod]
        public void UnloadPageContent_WhenPageContentIsEmpty_ShouldNotUnloadPageContent()
        {
            // Arrange
            TestPageContentManager pageContentManager = new TestPageContentManager();
            ThreadSafePageContentManager threadSafePageContentManager = new ThreadSafePageContentManager(pageContentManager);
            RadFixedPage page = new RadFixedPage();

            // Act
            threadSafePageContentManager.UnloadPageContent(page);

            // Assert
            int count;
            bool found = pageContentManager.PageToUnloadedContentCount.TryGetValue(page, out count);

            Assert.IsFalse(found);
        }

        [TestMethod]
        public void UnloadPageContent_WhenPageContentIsNotEmpty_ShouldUnloadPageContent()
        {
            // Arrange
            TestPageContentManager pageContentManager = new TestPageContentManager();
            pageContentManager.IsContentEmpty = false;
            ThreadSafePageContentManager threadSafePageContentManager = new ThreadSafePageContentManager(pageContentManager);
            RadFixedPage page = new RadFixedPage();

            // Act
            threadSafePageContentManager.UnloadPageContent(page);

            // Assert
            int count;
            bool found = pageContentManager.PageToUnloadedContentCount.TryGetValue(page, out count);

            Assert.IsTrue(found);
            Assert.AreEqual(1, count);
        }

        [TestMethod]
        public void IsPageContentEmpty_ShouldReturnCorrectResult()
        {
            // Arrange
            TestPageContentManager pageContentManager = new TestPageContentManager();
            ThreadSafePageContentManager threadSafePageContentManager = new ThreadSafePageContentManager(pageContentManager);
            RadFixedPage page = new RadFixedPage();

            // Act
            pageContentManager.IsContentEmpty = false;
            bool first = threadSafePageContentManager.IsPageContentEmpty(page);
            pageContentManager.IsContentEmpty = true;
            bool second = threadSafePageContentManager.IsPageContentEmpty(page);

            // Assert
            Assert.IsFalse(first);
            Assert.IsTrue(second);
        }
    }
}
