﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Annotations
{
    [TestClass]
    public class AnnotationObjectTests
    {
        [TestMethod]
        public void ToAnnotation_WhenAppearancesAreNull_ShouldNotSetAnnotationApperances()
        {
            // Arrange
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            int pageHeightInDip = 1000;

            TestWidgetAnnotationObject annotationObject = new TestWidgetAnnotationObject();
            annotationObject.Rect = new PdfArray(new PdfReal(0), new PdfReal(0), new PdfReal(20), new PdfReal(20));
            annotationObject.Flags = new PdfInt(0);

            // Act
            Annotation result = annotationObject.ToAnnotation(reader, context, pageHeightInDip);

            // Assert
            Assert.IsNull(result.Appearances);
        }

        [TestMethod]
        public void ToAnnotation_WhenMultiStateAppearancesWithEmptyNormaApperance_ShouldReturnAnnotationWithEmptyAppearances()
        {
            // Arrange
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            int pageHeightInDip = 1000;

            TestWidgetAnnotationObject annotationObject = new TestWidgetAnnotationObject();
            annotationObject.Rect = new PdfArray(new PdfReal(0), new PdfReal(0), new PdfReal(20), new PdfReal(20));
            annotationObject.Flags = new PdfInt(0);

            annotationObject.AppearanceState = new PdfName("Off");
            annotationObject.Appearances = new Appearances();
            Appearance normaAppearance = new Appearance(new Dictionary<string, FormXObject>());
            annotationObject.Appearances.NormalAppearance = normaAppearance;

            // Act
            Annotation result = annotationObject.ToAnnotation(reader, context, pageHeightInDip);

            // Assert
            MultiStateAppearances apperances = result.Appearances as MultiStateAppearances;
            Assert.IsNotNull(apperances);
            Assert.AreEqual(0, apperances.Count());
        }

        [TestMethod]
        public void ToAnnotation_WhenMultiStateAppearancesWithNullNormaApperance_ShouldReturnAnnotationWithEmptyAppearances()
        {
            // Arrange
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            int pageHeightInDip = 1000;

            TestWidgetAnnotationObject annotationObject = new TestWidgetAnnotationObject();
            annotationObject.Rect = new PdfArray(new PdfReal(0), new PdfReal(0), new PdfReal(20), new PdfReal(20));
            annotationObject.Flags = new PdfInt(0);

            annotationObject.AppearanceState = new PdfName("Off");
            annotationObject.Appearances = new Appearances();
            annotationObject.Appearances.NormalAppearance = null;

            // Act
            Annotation result = annotationObject.ToAnnotation(reader, context, pageHeightInDip);

            // Assert
            MultiStateAppearances apperances = result.Appearances as MultiStateAppearances;
            Assert.IsNotNull(apperances);
            Assert.AreEqual(0, apperances.Count());
        }
    }
}
