﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Functions;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.ColorSpaces
{
    [TestClass]
    public class DeviceNColorSpaceObjectTests
    {
        [TestMethod]
        public void GetColor_WhenComponentsAreIntegers_ShouldReturnCorrectColorInstance()
        {
            // Arrange
            TestPdfContentImportContext testPdfContentImportContext = new TestPdfContentImportContext();
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            PdfInt firstElement = new PdfInt(1);
            PdfInt secondElement = new PdfInt(0);
            PdfInt thirdElement = new PdfInt(1);
            PdfInt fourthElement = new PdfInt(0);
            PdfArray pdfArray = new PdfArray(firstElement, secondElement, thirdElement, fourthElement);

            DeviceNColorSpaceObject deviceNColorSpace = new DeviceNColorSpaceObject();
            this.Import(deviceNColorSpace);

            // Act
            CmykColorObject color = (CmykColorObject)deviceNColorSpace.GetColor(testPdfContentImportContext, reader, pdfArray);

            // Assert
            Assert.IsNotNull(color);
            Assert.AreEqual(firstElement.Value, color.C.Value);
            Assert.AreEqual(secondElement.Value, color.M.Value);
            Assert.AreEqual(thirdElement.Value, color.Y.Value);
            Assert.AreEqual(fourthElement.Value, color.K.Value);
        }

        private void Import(DeviceNColorSpaceObject deviceNColorSpace)
        {
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());

            PdfName deviceNPdfName = new PdfName("DeviceN");
            PdfArray colorSpacePdfArray = new PdfArray();
            PdfName cmykPdfName = new PdfName("DeviceCMYK");
            IndirectReference indirectReference = new IndirectReference(1, 0);
            PdfArray array = new PdfArray(deviceNPdfName, colorSpacePdfArray, cmykPdfName, indirectReference);

            TestPdfImportContext pdfImportContext = new TestPdfImportContext();
            TestPostScriptCalculatorFunctionObject postScriptCalculatorFunctionObject = new TestPostScriptCalculatorFunctionObject
            {
                Domain = new PdfArray(new IPdfPrimitive[] { new PdfReal(0), new PdfReal(1), new PdfReal(0), new PdfReal(1),
                new PdfReal(0), new PdfReal(1), new PdfReal(0), new PdfReal(1) }),
                Range = new PdfArray(new IPdfPrimitive[] { new PdfReal(0), new PdfReal(1), new PdfReal(0), new PdfReal(1),
                new PdfReal(0), new PdfReal(1), new PdfReal(0), new PdfReal(1) })
            };

            pdfImportContext.RegisterIndirectObject(indirectReference, postScriptCalculatorFunctionObject);

            deviceNColorSpace.ImportFromArray(reader, pdfImportContext, array);
        }
    }
}
