﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Common;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Common
{
    [TestClass]
    public class PdfConcatenatableDataStreamTests
    {
        private const string FirstStreamElement = @"q
            /F1 21 Tf
            1 0 0 1 51.42 730.86 Tm
            0.384 0.392 0.388 rg
            0.384 0.392 0.388 RG
            Q";

        private const string SecondStreamElement = @"q
            /F1 21 Tf
            1 0 0 1 568.92 769.56 Tm
            0 g
            0 G
            Q";

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Concat_WhenStreamToAppendIsNull_ShouldThrow()
        {
            // Arrange
            PdfConcatenatableDataStream pdfConcatenatableDataStream = new PdfConcatenatableDataStream();

            // Act && Assert
            pdfConcatenatableDataStream.Concat(null);
        }

        [TestMethod]
        public void Concat_WhenLocalDataIsNull_ShouldAddStreamData()
        {
            // Arrange
            PdfConcatenatableDataStream pdfConcatenatableDataStream = new PdfConcatenatableDataStream();

            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            Stream dataToAppend = new MemoryStream(Encoding.UTF8.GetBytes(SecondStreamElement));
            TestPdfStream testPdfStreamToAppend = new TestPdfStream(dataToAppend);

            PdfConcatenatableDataStream streamToAppend = new PdfConcatenatableDataStream();
            streamToAppend.Load(reader, context, testPdfStreamToAppend);

            // Act
            pdfConcatenatableDataStream.Concat(streamToAppend);

            // Assert
            string result = Encoding.UTF8.GetString(pdfConcatenatableDataStream.Data);
            Assert.AreEqual(SecondStreamElement, result);
        }

        [TestMethod]
        public void Concat_WhenStreamDataToAppendIsNull_ShouldNotThrow()
        {
            // Arrange
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            Stream data = new MemoryStream(Encoding.UTF8.GetBytes(FirstStreamElement));
            TestPdfStream testPdfStream = new TestPdfStream(data);

            PdfConcatenatableDataStream pdfConcatenatableDataStream = new PdfConcatenatableDataStream();
            pdfConcatenatableDataStream.Load(reader, context, testPdfStream);

            PdfConcatenatableDataStream streamToAppend = new PdfConcatenatableDataStream();

            // Act
            pdfConcatenatableDataStream.Concat(streamToAppend);

            // Assert
            string result = Encoding.UTF8.GetString(pdfConcatenatableDataStream.Data);
            Assert.AreEqual(FirstStreamElement, result);
        }

        [TestMethod]
        public void Concat_WhenAllDataIsNull_ShouldSetEmptyData()
        {
            // Arrange
            PdfConcatenatableDataStream pdfConcatenatableDataStream = new PdfConcatenatableDataStream();
            PdfConcatenatableDataStream streamToAppend = new PdfConcatenatableDataStream();

            // Act
            pdfConcatenatableDataStream.Concat(streamToAppend);

            // Assert
            Assert.IsNotNull(pdfConcatenatableDataStream.Data);
            Assert.AreEqual(0, pdfConcatenatableDataStream.Data.Length);
        }

        [TestMethod]
        public void Concat_WhenValidData_ShouldConcatWithoutWhiteSpace()
        {
            // Arrange
            PostScriptReader reader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            Stream data = new MemoryStream(Encoding.UTF8.GetBytes(FirstStreamElement));
            TestPdfStream testPdfStream = new TestPdfStream(data);

            PdfConcatenatableDataStream pdfConcatenatableDataStream = new PdfConcatenatableDataStream();
            pdfConcatenatableDataStream.Load(reader, context, testPdfStream);

            Stream dataToAppend = new MemoryStream(Encoding.UTF8.GetBytes(SecondStreamElement));
            TestPdfStream testPdfStreamToAppend = new TestPdfStream(dataToAppend);
            PdfConcatenatableDataStream streamToAppend = new PdfConcatenatableDataStream();
            streamToAppend.Load(reader, context, testPdfStreamToAppend);

            string expected = FirstStreamElement + SecondStreamElement;

            // Act
            pdfConcatenatableDataStream.Concat(streamToAppend);

            // Assert
            string result = Encoding.UTF8.GetString(pdfConcatenatableDataStream.Data);
            Assert.AreEqual(expected, result);
        }
    }
}
