﻿using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Destinations
{
    [TestClass]
    public class DestinationObjectTests
    {
        [TestMethod]
        public void CopyPropertiesFrom_WhenPageIsNull_ShouldNotCallTryGetPage()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            Location destination = new Location();

            DestinationObject destinationObject = new DestinationObject();

            // Act
            destinationObject.CopyPropertiesFrom(context, destination);

            // Assert
            Assert.IsNull(destinationObject.Page);
            Assert.AreEqual(0, context.TryGetPageMethodCalledCounter);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenPageIsNull_ShouldNotCallGetDipToPdfPointTransformation()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            Location destination = new Location();

            DestinationObject destinationObject = new DestinationObject();

            // Act
            destinationObject.CopyPropertiesFrom(context, destination);

            // Assert
            Assert.AreEqual(0, context.GetDipToPdfPointTransformationMethodCalledCounter);
        }

        [TestMethod]
        public void ToDestination_WhenPageIsNull_ShouldNotCallGetFixedPage()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.DestinationType = new PdfName("XYZ");

            // Act
            Destination destination = destinationObject.ToDestination(postScriptReader, context);

            // Assert
            Assert.IsNull(destination.Page);
            Assert.AreEqual(0, context.TryGetFixedPageMethodCalledCounter);
        }

        [TestMethod]
        public void ToDestination_WhenPageIsNull_ShouldNotCallGetPdfPointToDipTransformation()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.DestinationType = new PdfName("XYZ");

            // Act
            Destination destination = destinationObject.ToDestination(postScriptReader, context);

            // Assert
            Assert.AreEqual(0, context.GetPdfPointToDipTransformationMethodCalledCounter);
        }
    }
}
