﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Tests.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Fonts
{
    [TestClass]
    public class Type1FontObjectTests
    {
        [TestMethod]
        public void CopyPropertiesFrom_WHenTypeFontWithEmptyUsedCharacters_ShouldNotCreateToUnicodeCMap()
        {
            // Arrange
            Type1FontObject trueTypeFontObject = new Type1FontObject();
            TestPdfExportContext context = new TestPdfExportContext();

            TestFontSource fontSource = new TestFontSource();
            Type1Font font = new Type1Font("test", fontSource);

            // Act
            trueTypeFontObject.CopyPropertiesFrom(context, font);

            // Assert
            Assert.IsNull(trueTypeFontObject.ToUnicode);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WHenTypeFontWithNonEmptyUsedCharacters_ShouldCreateToUnicodeCMap()
        {
            // Arrange
            Type1FontObject trueTypeFontObject = new Type1FontObject();
            TestPdfExportContext context = new TestPdfExportContext();
            TestFontSource fontSource = new TestFontSource();

            Type1Font font = new Type1Font("test", fontSource);
            List<CharInfo> charInfos = new List<CharInfo>() { new CharInfo("", new CharCode(0)) };
            context.FontToUsedCharacters.Add(font, charInfos);

            // Act
            trueTypeFontObject.CopyPropertiesFrom(context, font);

            // Assert
            Assert.IsNotNull(trueTypeFontObject.ToUnicode);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WHenTypeFontWithEmptyUsedCharactersAndEncoding_ShouldNotCreateToUnicodeCMap()
        {
            // Arrange
            Type1FontObject trueTypeFontObject = new Type1FontObject();
            TestPdfExportContext context = new TestPdfExportContext();
            TestFontSource fontSource = new TestFontSource();

            Type1Font font = new Type1Font("test", fontSource);
            List<CharInfo> charInfos = new List<CharInfo>() { new CharInfo("", new CharCode(0)) };
            font.Encoding.Value = new SimpleEncoding();

            // Act
            trueTypeFontObject.CopyPropertiesFrom(context, font);

            // Assert
            Assert.IsNull(trueTypeFontObject.ToUnicode);
        }
    }
}
