﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model.Common.Functions;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Tests.Functions;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Functions
{
    [TestClass]
    public class FunctionObjectTests
    {
        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectDomain()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            double[] domain = new double[] { 0, 1, 0, 1 };
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(new byte[0], domain, new double[0]);

            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject();

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.AreEqual(domain.Length, functionObject.Domain.Count);
            Assert.AreEqual(domain[0], ((PdfReal)functionObject.Domain[0]).Value);
            Assert.AreEqual(domain[1], ((PdfReal)functionObject.Domain[1]).Value);
            Assert.AreEqual(domain[2], ((PdfReal)functionObject.Domain[2]).Value);
            Assert.AreEqual(domain[3], ((PdfReal)functionObject.Domain[3]).Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectRange()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            double[] range = new double[] { 1, 0, 1, 0, 1, 0, 1, 0 };
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(new byte[0], new double[0], range);

            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject();

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.AreEqual(range.Length, functionObject.Range.Count);
            Assert.AreEqual(range[0], ((PdfReal)functionObject.Range[0]).Value);
            Assert.AreEqual(range[1], ((PdfReal)functionObject.Range[1]).Value);
            Assert.AreEqual(range[2], ((PdfReal)functionObject.Range[2]).Value);
            Assert.AreEqual(range[3], ((PdfReal)functionObject.Range[3]).Value);
            Assert.AreEqual(range[4], ((PdfReal)functionObject.Range[4]).Value);
            Assert.AreEqual(range[5], ((PdfReal)functionObject.Range[5]).Value);
            Assert.AreEqual(range[6], ((PdfReal)functionObject.Range[6]).Value);
            Assert.AreEqual(range[7], ((PdfReal)functionObject.Range[7]).Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectFunctionType()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(new byte[0], new double[0], new double[0]);

            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject();

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.AreEqual(new PdfInt((int)FunctionType.PostScriptCalculator), functionObject.FunctionType);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldCallCopyPropertiesFromOverrideWithCorrectParameters()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(new byte[0], new double[0], new double[0]);

            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject();

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.AreEqual(true, functionObject.IsCopyPropertiesFromOverrideInvoked);
            Assert.AreEqual(1, functionObject.CopyPropertiesFromOverrideInvocationCount);
            Assert.AreEqual(context, functionObject.CopyPropertiesFromOverrideArgs[0]);
            Assert.AreEqual(function, functionObject.CopyPropertiesFromOverrideArgs[1]);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenHasDataIsFalse_ShouldNotSetData()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            byte[] data = new byte[] { 0, 1 };
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(data, new double[0], new double[0]);

            bool hasData = false;
            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject(hasData);

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.IsNull(functionObject.GetFunctionData());
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenHasDataIsTrue_ShouldSetCorrectData()
        {
            // Arrange
            TestPdfExportContext context = new TestPdfExportContext();
            byte[] data = new byte[] { 0, 1 };
            TestPostScriptCalculatorFunction function = new TestPostScriptCalculatorFunction(data, new double[0], new double[0]);

            bool hasData = true;
            TestPostScriptCalculatorFunctionObject functionObject = new TestPostScriptCalculatorFunctionObject(hasData);

            // Act
            functionObject.CopyPropertiesFrom(context, function);

            // Assert
            Assert.AreEqual(data.Length, functionObject.GetFunctionData().Length);
            Assert.AreEqual(data[0], functionObject.GetFunctionData()[0]);
            Assert.AreEqual(data[1], functionObject.GetFunctionData()[1]);
        }
    }
}
