﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Globalization;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Utilities;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Metadata.Xmp.Utilities
{
    [TestClass]
    public class ExtensionsTests
    {
        [TestMethod]
        public void ToXmpDate_WhenDateTimeIsUtc_ReturnsDateInXmpFormat()
        {
            // Arrange
            DateTime dateTime = new DateTime(2000, 8, 9, 1, 2, 3, 123, DateTimeKind.Utc);

            // Act
            string xmpDate = dateTime.ToXmpDate();

            // Assert
            Assert.AreEqual("2000-08-09T01:02:03.1230000Z", xmpDate);
        }

        [TestMethod]
        public void ToXmpDate_WhenDateTimeIsUnspecified_ReturnsDateInXmpFormat()
        {
            // Arrange
            DateTime dateTime = new DateTime(2000, 8, 9, 1, 2, 3, 123);

            // Act
            string xmpDate = dateTime.ToXmpDate();

            // Assert
            Assert.AreEqual("2000-08-09T01:02:03.1230000", xmpDate);
        }

        [TestMethod]
        public void ToXmpDate_WhenDateTimeIsLocal_ReturnsDateInXmpFormat()
        {
            // Arrange
            DateTime dateTime = new DateTime(2000, 8, 9, 1, 2, 3, 123, DateTimeKind.Local);

            // Act
            string xmpDate = dateTime.ToXmpDate();

            // Assert
            DateTime parsedDateTime = DateTime.ParseExact(xmpDate, "yyyy-MM-ddTHH:mm:ss.fffffffK", CultureInfo.InvariantCulture, DateTimeStyles.None);

            // Assert
            Assert.AreEqual(new DateTime(2000, 8, 9, 1, 2, 3, 123, DateTimeKind.Local), parsedDateTime);
        }
    }
}
