﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Objects
{
    [TestClass]
    public class ImageXObjectTests
    {
        [TestMethod]
        public void ImageXObject_CopyPropertiesFrom_WhenMaskHasValueWhichIsNull()
        {
            ImageSource image = new ImageSource(new byte[] { 4, 8, 15, 16, 23, 42 }, 8, 10, 20, new DeviceRgb(), null, null, false);
            image.Mask.Value = null;

            ExportAndAssertImageWithoutFiltersAndMasks(image);
        }

        [TestMethod]
        public void ImageXObject_CopyPropertiesFrom_WhenSMaskHasValueWhichIsNull()
        {
            ImageSource image = new ImageSource(new byte[] { 4, 8, 15, 16, 23, 42 }, 4, 11, 22, new DeviceCmyk(), null, null, false);
            image.SMask.Value = null;

            ExportAndAssertImageWithoutFiltersAndMasks(image);
        }

        private static void ExportAndAssertImageWithoutFiltersAndMasks(ImageSource image)
        {
            EncodedImageData data = image.GetEncodedImageData();
            IPdfExportContext context = new TestPdfExportContext();
            PdfWriter writer = new PdfWriter(new MemoryStream());
            ImageXObject imageXObject = new ImageXObject();
            imageXObject.CopyPropertiesFrom(context, image);
            imageXObject.Write(writer, context);

            Assert.AreEqual(data.BitsPerComponent, imageXObject.BitsPerComponent.Value);
            Assert.AreEqual(data.Width, imageXObject.Width.Value);
            Assert.AreEqual(data.Height, imageXObject.Height.Value);
            Assert.AreEqual(data.ColorSpace, imageXObject.ColorSpace.Name);
            Assert.AreEqual(0, imageXObject.Filters.Count);
            Assert.AreEqual(data.Data.Length, imageXObject.Length.Value);
            Assert.IsFalse(imageXObject.ImageMask.Value);
            Assert.IsNull(imageXObject.Decode);
            Assert.IsNull(imageXObject.Mask);
            Assert.IsNull(imageXObject.SMask);
        }
    }
}
