﻿using System.IO;
using System.Linq;
using System.Text;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Actions;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Outlines;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Tests.Import;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Elements.Outlines
{
    [TestClass]
    public class OutlineItemObjectTests
    {
        private const double Delta = 0.01;
        private const string FirstChildTitle = "First Child";
        private const string SecondChildTitle = "Second Child";

        [TestMethod]
        public void ToBookmarkItem_WhenTitleIsNull_ShouldCreateBookmarkItemWithEmptyTitle()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Title = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(string.Empty, bookmarkItem.Title);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenTitleIsNotNull_ShouldCreateBookmarkItemWithCorrectTitle()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            string title = "Title";
            outlineItemObject.Title = new PdfLiteralString(Encoding.UTF8.GetBytes(title));

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(title, bookmarkItem.Title);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenCountIsNull_ShouldSetIsExpandedToFalse()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Count = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(false, bookmarkItem.IsExpanded);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenCountIsZero_ShouldSetIsExpandedToFalse()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Count = new PdfInt(0);

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(false, bookmarkItem.IsExpanded);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenCountIsNegatgive_ShouldSetIsExpandedToFalse()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Count = new PdfInt(-1);

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(false, bookmarkItem.IsExpanded);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenCountIsPositive_ShouldSetIsExpandedToTrue()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Count = new PdfInt(1);

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(true, bookmarkItem.IsExpanded);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenFlagIsNull_ShouldSetTextStyleToNormal()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Flag = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(BookmarkItemStyles.Normal, bookmarkItem.TextStyle);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenFlagIsNotNull_ShouldSetCorrectTextStyle()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject0 = new OutlineItemObject();
            outlineItemObject0.Flag = new PdfInt(0);

            OutlineItemObject outlineItemObject1 = new OutlineItemObject();
            outlineItemObject1.Flag = new PdfInt(1);

            OutlineItemObject outlineItemObject2 = new OutlineItemObject();
            outlineItemObject2.Flag = new PdfInt(2);

            OutlineItemObject outlineItemObject3 = new OutlineItemObject();
            outlineItemObject3.Flag = new PdfInt(3);

            // Act
            BookmarkItem bookmarkItem0 = outlineItemObject0.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem1 = outlineItemObject1.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem2 = outlineItemObject2.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem3 = outlineItemObject3.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(BookmarkItemStyles.Normal, bookmarkItem0.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Italic, bookmarkItem1.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Bold, bookmarkItem2.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Italic | BookmarkItemStyles.Bold, bookmarkItem3.TextStyle);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNull_ShouldDoNothing()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Destination = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNull(bookmarkItem.Destination);
        }

        [TestMethod]
        public void ToBookmarkItem_WheNamedDestinationIsNotNull_ShouldSetNamedDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedDocument fixedDocument = context.Document;
            RadFixedPage fixedPage = fixedDocument.Pages.AddPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Top = new PdfReal(100);
            destinationObject.Zoom = new PdfReal(10);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("XYZ");

            string namedDestinationName = "Test Dest";

            fixedDocument.NamedDestinations.Add(namedDestinationName, destinationObject.ToDestination(postScriptReader, context));
            outlineItemObject.Destination = new NamedDestinationObject() { Name = namedDestinationName, Destination = destinationObject };

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNull(bookmarkItem.Destination);
            Assert.IsNull(bookmarkItem.Action);
            Assert.IsNotNull(bookmarkItem.NamedDestination);
            Assert.AreEqual(namedDestinationName, bookmarkItem.NamedDestination.Name);

            Assert.IsNull(bookmarkItem.NamedDestination.GoToAction);
            Assert.IsNotNull(bookmarkItem.NamedDestination.Destination);

            Destination destination = bookmarkItem.NamedDestination.Destination;

            Assert.AreEqual(DestinationType.Location, destination.DestinationType);

            Location location = destination as Location;
            Assert.IsNotNull(location);
            Assert.AreEqual(50, location.Left);
            Assert.AreEqual(100, location.Top);
            Assert.AreEqual(10, location.Zoom);
            Assert.IsNotNull(location.Page);
            Assert.AreEqual(fixedPage, location.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsLocation_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Top = new PdfReal(100);
            destinationObject.Zoom = new PdfReal(10);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("XYZ");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.Location, bookmarkItem.Destination.DestinationType);

            Location destination = bookmarkItem.Destination as Location;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.AreEqual(100, destination.Top);
            Assert.AreEqual(10, destination.Zoom);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsPageHorizontalFit_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Top = new PdfReal(100);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitH");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.PageHorizontalFit, bookmarkItem.Destination.DestinationType);

            PageHorizontalFit destination = bookmarkItem.Destination as PageHorizontalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(100, destination.Top);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsPageVerticalFit_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitV");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.PageVerticalFit, bookmarkItem.Destination.DestinationType);

            PageVerticalFit destination = bookmarkItem.Destination as PageVerticalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsRectangleFit_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Right = new PdfReal(150);
            destinationObject.Top = new PdfReal(100);
            destinationObject.Bottom = new PdfReal(200);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitR");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.RectangleFit, bookmarkItem.Destination.DestinationType);

            RectangleFit destination = bookmarkItem.Destination as RectangleFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.AreEqual(150, destination.Right);
            Assert.AreEqual(100, destination.Top);
            Assert.AreEqual(200, destination.Bottom);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsBoundingRectangleHorizontalFit_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Top = new PdfReal(100);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitBH");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.BoundingRectangleHorizontalFit, bookmarkItem.Destination.DestinationType);

            BoundingRectangleHorizontalFit destination = bookmarkItem.Destination as BoundingRectangleHorizontalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(100, destination.Top);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenDestinationIsNotNullAndIsBoundingRectangleVerticalFit_ShouldSetCorrectDestination()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitBV");
            outlineItemObject.Destination = destinationObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Destination);
            Assert.AreEqual(DestinationType.BoundingRectangleVerticalFit, bookmarkItem.Destination.DestinationType);

            BoundingRectangleVerticalFit destination = bookmarkItem.Destination as BoundingRectangleVerticalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenActionIsNull_ShouldDoNothing()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Action = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNull(bookmarkItem.Action);
            Assert.IsNull(bookmarkItem.Destination);
            Assert.IsNull(bookmarkItem.NamedDestination);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenActionIsNotNull_ShouldSetCorrentAction()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitBV");
            outlineItemObject.Destination = destinationObject;

            GoToActionObject goToActionObject = new GoToActionObject();
            goToActionObject.Destination = destinationObject;

            outlineItemObject.Action = goToActionObject;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNotNull(bookmarkItem.Action);
            Assert.AreEqual(ActionType.GoTo, bookmarkItem.Action.ActionType);

            GoToAction goToAction = bookmarkItem.Action as GoToAction;
            Assert.IsNotNull(goToAction);

            Assert.IsNotNull(goToAction.Destination);
            Assert.AreEqual(DestinationType.BoundingRectangleVerticalFit, goToAction.Destination.DestinationType);

            BoundingRectangleVerticalFit destination = goToAction.Destination as BoundingRectangleVerticalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenNamedDestinationActionIsNotNull_ShouldSetCorrentAction()
        {
            // Arrange
            string namedDestinationName = "Test Destination";
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            RadFixedDocument fixedDocument = context.Document;
            RadFixedPage fixedPage = fixedDocument.Pages.AddPage();
            Page page = new Page();
            context.PageToFixedPage.Add(page, fixedPage);
            context.PageToMatrix.Add(page, new Matrix());

            DestinationObject destinationObject = new DestinationObject();
            destinationObject.Left = new PdfReal(50);
            destinationObject.Page = page;
            destinationObject.DestinationType = new PdfName("FitBV");
            outlineItemObject.Destination = destinationObject;

            GoToActionObject goToActionObject = new GoToActionObject();
            goToActionObject.Destination = destinationObject;

            fixedDocument.NamedDestinations.Add(namedDestinationName, (GoToAction)goToActionObject.ToAction(postScriptReader, context));
            outlineItemObject.Destination = new NamedDestinationObject() { Name = namedDestinationName, GoToAction = goToActionObject };

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.IsNull(bookmarkItem.Action);
            Assert.IsNull(bookmarkItem.Destination);
            Assert.IsNotNull(bookmarkItem.NamedDestination);

            NamedDestination namedDestination = bookmarkItem.NamedDestination;
            Assert.AreEqual(namedDestinationName, namedDestination.Name);

            GoToAction goToAction = namedDestination.GoToAction;
            Assert.AreEqual(ActionType.GoTo, goToAction.ActionType);

            Assert.IsNotNull(goToAction);

            Assert.IsNotNull(goToAction.Destination);
            Assert.AreEqual(DestinationType.BoundingRectangleVerticalFit, goToAction.Destination.DestinationType);

            BoundingRectangleVerticalFit destination = goToAction.Destination as BoundingRectangleVerticalFit;
            Assert.IsNotNull(destination);
            Assert.AreEqual(50, destination.Left);
            Assert.IsNotNull(destination.Page);
            Assert.AreEqual(fixedPage, destination.Page);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenColorIsNull_ShouldSetTextColorToBlack()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Color = null;

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(RgbColors.Black, bookmarkItem.TextColor);
        }

        [TestMethod]
        public void ToBookmarkItem_WhenColorIsNotNull_ShouldSetCorrectTextColor()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            outlineItemObject.Color = new PdfArray(new PdfReal(0.5), new PdfReal(1), new PdfReal(0));

            // Act
            BookmarkItem bookmarkItem = outlineItemObject.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(new RgbColor(128, 255, 0), bookmarkItem.TextColor);
        }

        [TestMethod]
        public void ToBookmarkItem_ShouldSetCorrectTextStyle()
        {
            // Arrange
            PostScriptReader postScriptReader = new PostScriptReader(new MemoryStream(), new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();

            OutlineItemObject outlineItemObject0 = new OutlineItemObject();
            outlineItemObject0.Flag = new PdfInt(0);

            OutlineItemObject outlineItemObject1 = new OutlineItemObject();
            outlineItemObject1.Flag = new PdfInt(1);

            OutlineItemObject outlineItemObject2 = new OutlineItemObject();
            outlineItemObject2.Flag = new PdfInt(2);

            OutlineItemObject outlineItemObject3 = new OutlineItemObject();
            outlineItemObject3.Flag = new PdfInt(3);

            // Act
            BookmarkItem bookmarkItem0 = outlineItemObject0.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem1 = outlineItemObject1.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem2 = outlineItemObject2.ToBookmarkItem(postScriptReader, context);
            BookmarkItem bookmarkItem3 = outlineItemObject3.ToBookmarkItem(postScriptReader, context);

            // Assert
            Assert.AreEqual(BookmarkItemStyles.Normal, bookmarkItem0.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Italic, bookmarkItem1.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Bold, bookmarkItem2.TextStyle);
            Assert.AreEqual(BookmarkItemStyles.Italic | BookmarkItemStyles.Bold, bookmarkItem3.TextStyle);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectTitle()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            string title = "Title";
            BookmarkItem bookmarkItem = new BookmarkItem(title);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            PdfHexString actualTitle = outlineItemObject.Title as PdfHexString;
            Assert.IsNotNull(actualTitle);

            byte[] expectedTitle = BytesHelper.MergeBytes(new byte[] { 255, 254 }, Encoding.BigEndianUnicode.GetBytes(title));
            Assert.IsTrue(expectedTitle.SequenceEqual(actualTitle.Value));
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasNoChildren_ShouldSetCountToZero()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();
            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);
            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.AreEqual(0, outlineItemObject.Count.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildrenAndIsExpandedIsFalse_ShouldSetCountToCorrectNegativeNumber()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);
            bookmarkItem.IsExpanded = false;
            bookmarkItem.Children.Add(new BookmarkItem(string.Empty));

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.AreEqual(-1, outlineItemObject.Count.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildrenAndIsExpandedIsTrue_ShouldSetCountToCorrectPositiveNumber()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);
            bookmarkItem.IsExpanded = true;
            bookmarkItem.Children.Add(new BookmarkItem(string.Empty));

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.AreEqual(1, outlineItemObject.Count.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsNull_ShouldDoNothing()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNull(outlineItemObject.Destination);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsLocation_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            Location destination = new Location
            {
                Left = 50,
                Top = 100,
                Zoom = 10,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);
            Assert.IsFalse(outlineItemObject.Destination.IsNamed);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;
            Assert.AreEqual("XYZ", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.AreEqual(100, destinationObject.Top.Value);
            Assert.AreEqual(10, destinationObject.Zoom.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenNamedDestinationDestinationIsLocation_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            RadFixedDocument fixedDocument = new RadFixedDocument();
            RadFixedPage fixedPage = fixedDocument.Pages.AddPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            Location destination = new Location
            {
                Left = 50,
                Top = 100,
                Zoom = 10,
                Page = fixedPage
            };

            string namedDestinationName = "Test Destination";

            NamedDestination namedDestination = fixedDocument.NamedDestinations.Add(namedDestinationName, destination);
            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, namedDestination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);
            Assert.IsTrue(outlineItemObject.Destination.IsNamed);

            NamedDestinationObject namedDestinationObject = (NamedDestinationObject)outlineItemObject.Destination;
            Assert.AreEqual(namedDestinationName, namedDestinationObject.Name);
            DestinationObject destinationObject = namedDestinationObject.Destination;

            Assert.IsNotNull(destination);

            Assert.AreEqual("XYZ", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.AreEqual(100, destinationObject.Top.Value);
            Assert.AreEqual(10, destinationObject.Zoom.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsPageHorizontalFit_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();


            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            PageHorizontalFit destination = new PageHorizontalFit
            {
                Top = 100,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;

            Assert.AreEqual("FitH", destinationObject.DestinationType.Value);
            Assert.AreEqual(100, destinationObject.Top.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsPageVerticalFit_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();


            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            PageVerticalFit destination = new PageVerticalFit
            {
                Left = 50,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;

            Assert.AreEqual("FitV", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsRectangleFit_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            RectangleFit destination = new RectangleFit
            {
                Left = 50,
                Right = 150,
                Top = 100,
                Bottom = 200,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;

            Assert.AreEqual("FitR", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.AreEqual(150, destinationObject.Right.Value);
            Assert.AreEqual(100, destinationObject.Top.Value);
            Assert.AreEqual(200, destinationObject.Bottom.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsBoundingRectangleHorizontalFit_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            BoundingRectangleHorizontalFit destination = new BoundingRectangleHorizontalFit
            {
                Top = 100,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;

            Assert.AreEqual("FitBH", destinationObject.DestinationType.Value);
            Assert.AreEqual(100, destinationObject.Top.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenDestinationIsBoundingRectangleVerticalFit_ShouldSetCorrectDestiantion()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();


            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            BoundingRectangleVerticalFit destination = new BoundingRectangleVerticalFit
            {
                Left = 50,
                Page = fixedPage
            };

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, destination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Destination);

            DestinationObject destinationObject = (DestinationObject)outlineItemObject.Destination;

            Assert.AreEqual("FitBV", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenActionDestinationOrNamedDestinationIsNull_ShouldDoNothing()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNull(outlineItemObject.Action);
            Assert.IsNull(outlineItemObject.Destination);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenActionIsNotNull_ShouldSetCorrectAction()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();


            RadFixedPage fixedPage = new RadFixedPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            BoundingRectangleVerticalFit destination = new BoundingRectangleVerticalFit
            {
                Left = 50,
                Page = fixedPage
            };

            GoToAction goToAction = new GoToAction(destination);

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, goToAction);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Action);

            GoToActionObject goToActionObject = outlineItemObject.Action as GoToActionObject;
            Assert.IsNotNull(goToActionObject.Destination);

            DestinationObject destinationObject = (DestinationObject)goToActionObject.Destination;

            Assert.AreEqual("FitBV", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenNamedDestinationActionIsNotNull_ShouldSetCorrectAction()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();


            RadFixedDocument fixedDocument = new RadFixedDocument();
            RadFixedPage fixedPage = fixedDocument.Pages.AddPage();
            Page page = new Page();
            context.FixedPageToPage.Add(fixedPage, page);
            context.FixedPageToMatrix.Add(fixedPage, new Matrix());

            BoundingRectangleVerticalFit destination = new BoundingRectangleVerticalFit
            {
                Left = 50,
                Page = fixedPage
            };

            GoToAction goToAction = new GoToAction(destination);

            string namedDestinationName = "Test name";
            NamedDestination namedDestination = fixedDocument.NamedDestinations.Add(namedDestinationName, goToAction);

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty, namedDestination);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNull(outlineItemObject.Action);
            Assert.IsNotNull(outlineItemObject.Destination);
            Assert.IsTrue(outlineItemObject.Destination.IsNamed);

            var namedDestinationObject = (NamedDestinationObject)outlineItemObject.Destination;

            GoToActionObject goToActionObject = namedDestinationObject.GoToAction as GoToActionObject;
            Assert.IsNotNull(goToActionObject);

            Assert.IsNotNull(goToActionObject.Destination);

            DestinationObject destinationObject = (DestinationObject)goToActionObject.Destination;

            Assert.AreEqual("FitBV", destinationObject.DestinationType.Value);
            Assert.AreEqual(50, destinationObject.Left.Value);
            Assert.IsNotNull(destinationObject.Page);
            Assert.AreEqual(page, destinationObject.Page);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectColor()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);
            bookmarkItem.TextColor = new RgbColor(128, 255, 0);

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            PdfArray pdfArray = new PdfArray(new PdfReal(0.5), new PdfReal(1), new PdfReal(0));

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Color);

            PdfReal redComponent = outlineItemObject.Color[0] as PdfReal;
            Assert.IsNotNull(redComponent);
            Assert.AreEqual(0.5, redComponent.Value, Delta);

            PdfReal greenComponent = outlineItemObject.Color[1] as PdfReal;
            Assert.IsNotNull(greenComponent);
            Assert.AreEqual(1, greenComponent.Value);

            PdfReal blueComponent = outlineItemObject.Color[2] as PdfReal;
            Assert.IsNotNull(blueComponent);
            Assert.AreEqual(0, blueComponent.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenTextColorIsBlack_ShouldSetCorrectColor()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);
            bookmarkItem.TextColor = new RgbColor(0, 0, 0);

            OutlineItemObject outlineItemObject = new OutlineItemObject();
            PdfArray pdfArray = new PdfArray(new PdfReal(0.5), new PdfReal(1), new PdfReal(0));

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.IsNotNull(outlineItemObject.Color);

            PdfReal redComponent = outlineItemObject.Color[0] as PdfReal;
            Assert.IsNotNull(redComponent);
            Assert.AreEqual(0, redComponent.Value, Delta);

            PdfReal greenComponent = outlineItemObject.Color[1] as PdfReal;
            Assert.IsNotNull(greenComponent);
            Assert.AreEqual(0, greenComponent.Value);

            PdfReal blueComponent = outlineItemObject.Color[2] as PdfReal;
            Assert.IsNotNull(blueComponent);
            Assert.AreEqual(0, blueComponent.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_ShouldSetCorrectFlag()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem0 = new BookmarkItem(string.Empty);
            bookmarkItem0.TextStyle = BookmarkItemStyles.Normal;

            BookmarkItem bookmarkItem1 = new BookmarkItem(string.Empty);
            bookmarkItem1.TextStyle = BookmarkItemStyles.Italic;

            BookmarkItem bookmarkItem2 = new BookmarkItem(string.Empty);
            bookmarkItem2.TextStyle = BookmarkItemStyles.Bold;

            BookmarkItem bookmarkItem3 = new BookmarkItem(string.Empty);
            bookmarkItem3.TextStyle = BookmarkItemStyles.Italic | BookmarkItemStyles.Bold;

            OutlineItemObject outlineItemObject0 = new OutlineItemObject();
            OutlineItemObject outlineItemObject1 = new OutlineItemObject();
            OutlineItemObject outlineItemObject2 = new OutlineItemObject();
            OutlineItemObject outlineItemObject3 = new OutlineItemObject();

            // Act
            outlineItemObject0.CopyPropertiesFrom(context, bookmarkItem0);
            outlineItemObject1.CopyPropertiesFrom(context, bookmarkItem1);
            outlineItemObject2.CopyPropertiesFrom(context, bookmarkItem2);
            outlineItemObject3.CopyPropertiesFrom(context, bookmarkItem3);

            // Assert
            Assert.AreEqual(0, outlineItemObject0.Flag.Value);
            Assert.AreEqual(1, outlineItemObject1.Flag.Value);
            Assert.AreEqual(2, outlineItemObject2.Flag.Value);
            Assert.AreEqual(3, outlineItemObject3.Flag.Value);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildren_ShouldSetCorrectFirst()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            BookmarkItem firstChild = new BookmarkItem(FirstChildTitle);
            bookmarkItem.Children.Add(firstChild);

            BookmarkItem secondChild = new BookmarkItem(SecondChildTitle);
            bookmarkItem.Children.Add(secondChild);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            PdfHexString actualTitle = outlineItemObject.First.Title as PdfHexString;
            Assert.IsNotNull(actualTitle);

            byte[] expectedTitle = BytesHelper.MergeBytes(new byte[] { 255, 254 }, Encoding.BigEndianUnicode.GetBytes(FirstChildTitle));
            Assert.IsTrue(expectedTitle.SequenceEqual(actualTitle.Value));
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildren_ShouldSetCorrectLast()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            BookmarkItem firstChild = new BookmarkItem(FirstChildTitle);
            bookmarkItem.Children.Add(firstChild);

            BookmarkItem secondChild = new BookmarkItem(SecondChildTitle);
            bookmarkItem.Children.Add(secondChild);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            PdfHexString actualTitle = outlineItemObject.Last.Title as PdfHexString;
            Assert.IsNotNull(actualTitle);

            byte[] expectedTitle = BytesHelper.MergeBytes(new byte[] { 255, 254 }, Encoding.BigEndianUnicode.GetBytes(SecondChildTitle));
            Assert.IsTrue(expectedTitle.SequenceEqual(actualTitle.Value));
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildren_ShouldSetCorrectParent()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            BookmarkItem firstChild = new BookmarkItem(FirstChildTitle);
            bookmarkItem.Children.Add(firstChild);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            Assert.AreEqual(outlineItemObject, outlineItemObject.First.Parent);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildren_ShouldSetCorrectPrevAndNext()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            BookmarkItem firstChild = new BookmarkItem(FirstChildTitle);
            bookmarkItem.Children.Add(firstChild);

            BookmarkItem secondChild = new BookmarkItem(SecondChildTitle);
            bookmarkItem.Children.Add(secondChild);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            OutlineItemObject firstChildOutlineItem = outlineItemObject.First;
            OutlineItemObject secondChildOutlineItem = outlineItemObject.Last;

            Assert.IsNull(firstChildOutlineItem.Prev);
            Assert.AreEqual(secondChildOutlineItem, firstChildOutlineItem.Next);

            Assert.AreEqual(firstChildOutlineItem, secondChildOutlineItem.Prev);
            Assert.IsNull(secondChildOutlineItem.Next);
        }

        [TestMethod]
        public void CopyPropertiesFrom_WhenBookmarkItemHasChildren_ShouldSetCorrectChildProperties()
        {
            // Arrange
            TestRadFixedDocumentExportContext context = new TestRadFixedDocumentExportContext();

            BookmarkItem bookmarkItem = new BookmarkItem(string.Empty);

            BookmarkItem firstChild = new BookmarkItem(FirstChildTitle);
            firstChild.IsExpanded = false;
            firstChild.TextColor = new RgbColor(128, 255, 0);
            firstChild.TextStyle = BookmarkItemStyles.Bold;

            bookmarkItem.Children.Add(firstChild);

            OutlineItemObject outlineItemObject = new OutlineItemObject();

            // Act
            outlineItemObject.CopyPropertiesFrom(context, bookmarkItem);

            // Assert
            OutlineItemObject childOutlineItem = outlineItemObject.First;
            Assert.IsNotNull(childOutlineItem);

            PdfHexString actualTitle = childOutlineItem.Title as PdfHexString;
            Assert.IsNotNull(actualTitle);

            byte[] expectedTitle = BytesHelper.MergeBytes(new byte[] { 255, 254 }, Encoding.BigEndianUnicode.GetBytes(FirstChildTitle));
            Assert.IsTrue(expectedTitle.SequenceEqual(actualTitle.Value));

            Assert.AreEqual(0, childOutlineItem.Count.Value);
            Assert.AreEqual(2, childOutlineItem.Flag.Value);

            PdfReal redComponent = childOutlineItem.Color[0] as PdfReal;
            Assert.IsNotNull(redComponent);
            Assert.AreEqual(0.5, redComponent.Value, Delta);

            PdfReal greenComponent = childOutlineItem.Color[1] as PdfReal;
            Assert.IsNotNull(greenComponent);
            Assert.AreEqual(1, greenComponent.Value);

            PdfReal blueComponent = childOutlineItem.Color[2] as PdfReal;
            Assert.IsNotNull(blueComponent);
            Assert.AreEqual(0, blueComponent.Value);
        }
    }
}
