﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Text;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Pdf.Model.Types
{
    [TestClass]
    public class PdfHexStringTests
    {
        private const string FamilyName = "FamilyName";
        private readonly byte[] FamilyNameBytes = new byte[] { 254, 255, 0, 70, 0, 97, 0, 109, 0, 105, 0, 108, 0, 121, 0, 78, 0, 97, 0, 109, 0, 101 };

        [TestMethod]
        public void Constructor_WhenUtfEncodedStringType_ShouldSetCorrectValue()
        {
            // Arrange & Act
            PdfHexString pdfHexString = new PdfHexString(FamilyName, StringType.UTFEncodedTextString);

            // Assert
            Assert.IsTrue(FamilyNameBytes.SequenceEqual(pdfHexString.Value));
        }

        [TestMethod]
        public void ToString_ShouldReturnEmptyString()
        {
            PdfHexString hexString = new PdfHexString();
            hexString.Value = Encoding.ASCII.GetBytes("");

            string actual = hexString.ToString();
            string expected = string.Empty;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void AssertToStringConvertsPdfEncodedTextCorrectly()
        {
            byte[] value = new byte[] { 82, 233, 112, 111, 110, 115, 101, 95, 51, 48 };
            PdfHexString hexString = new PdfHexString(value);

            string actual = hexString.ToString();
            string expected = "Réponse_30";

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void AssertToStringConvertsUtfEncodedTextCorrectly()
        {
            byte[] value = FamilyNameBytes;
            PdfHexString hexString = new PdfHexString(value);

            string actual = hexString.ToString();
            string expected = FamilyName;

            Assert.AreEqual(expected, actual);
        }
    }
}
