﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Fixed.Tests.FormatProviders.Utilities
{
    [TestClass]
    public class PdfObjectsExtensionsTests
    {
        [TestMethod]
        public void ToTopLeftCoordinateSystem_WhenContentRootElement_ShouldReturPositionTranformedCorrectly()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(800, 1000);

            Matrix matrix = new Matrix();
            matrix.Translate(100, 200);

            double pointToDipConversion = Unit.PointToDip(1);
            Matrix expected = matrix * new Matrix(pointToDipConversion, 0, 0, -pointToDipConversion, 0, 1000);

            // Act
            Matrix result = page.ToTopLeftCoordinateSystem(matrix);

            // Assert
            Assert.AreEqual(expected, result);
        }

        [TestMethod]
        public void ToTopLeftCoordinateSystem_WhenRect_ShouldReturPositionTranformedCorrectly()
        {
            // Arrange
            Rect rect = new Rect(100, 200, 300, 400);

            double pointToDipConversion = Unit.PointToDip(1);
            Rect expected = this.Transform(new Matrix(pointToDipConversion, 0, 0, -pointToDipConversion, 0, Unit.PointToDip(800)), rect);

            // Act
            Rect result = rect.ToTopLeftCoordinateSystem(800);

            // Assert
            Assert.AreEqual(expected, result);
        }

        [TestMethod]
        public void ToBottomLeftCoordinateSystem_WhenContentRootElement_ShouldReturPositionTranformedCorrectly()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(800, 1000);

            Rect rect = new Rect(100, 200, 300, 400);

            double dipToPointConversion = Unit.DipToPoint(1);
            Rect expected = this.Transform(new Matrix(dipToPointConversion, 0, 0, -dipToPointConversion, 0, Unit.DipToPoint(1000)), rect);

            // Act
            PdfArray result = page.ToBottomLeftCoordinateSystem(rect);

            // Assert
            Assert.AreEqual(expected.X, ((PdfReal)(result[0])).Value);
            Assert.AreEqual(expected.Y, ((PdfReal)(result[1])).Value);
            Assert.AreEqual(expected.X + expected.Width, ((PdfReal)(result[2])).Value);
            Assert.AreEqual(expected.Y + expected.Height, ((PdfReal)(result[3])).Value);
        }

        [TestMethod]
        public void ToBottomLeftCoordinateSystem_WhenRect_ShouldReturPositionTranformedCorrectly()
        {
            // Arrange
            Rect rect = new Rect(100, 200, 300, 400);

            double dipToPointConversion = Unit.DipToPoint(1);
            Rect expected = this.Transform(new Matrix(dipToPointConversion, 0, 0, -dipToPointConversion, 0, Unit.DipToPoint(1000)), rect);

            // Act
            PdfArray result = rect.ToBottomLeftCoordinateSystem(1000);

            // Assert
            Assert.AreEqual(expected.X, ((PdfReal)(result[0])).Value);
            Assert.AreEqual(expected.Y, ((PdfReal)(result[1])).Value);
            Assert.AreEqual(expected.X + expected.Width, ((PdfReal)(result[2])).Value);
            Assert.AreEqual(expected.Y + expected.Height, ((PdfReal)(result[3])).Value);
        }

        private Rect Transform(Matrix matrix, Rect rect)
        {
            return new Rect(matrix.Transform(new Point(rect.Left, rect.Top)), matrix.Transform(new Point(rect.Right, rect.Bottom)));
        }
    }
}
