﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Graphics;

namespace Telerik.Windows.Documents.Fixed.Tests.Geometry
{
    [TestClass]
    public class BoundingRectangleTests
    {
        private const double Epsilon = 1E-2;

        [TestMethod]
        public void QuadraticBezierGeometryBoundsTest()
        {
            PathGeometry geometry = new PathGeometry();
            PathFigure figure = geometry.Figures.AddPathFigure();
            figure.IsClosed = false;

            figure.StartPoint = new Point(150, 150);
            figure.Segments.AddQuadraticBezierSegment(new Point(300, 200), new Point(200, 50));

            Rect actualBounds = geometry.Bounds;
            Rect expectedBounds = new Rect(150, 50, 90, 112.5);
            bool areEqual = TestHelper.AreEqual(expectedBounds, actualBounds, Epsilon);

            Assert.IsTrue(areEqual);
        }

        [TestMethod]
        public void CubicBezierGeometryBoundsTest()
        {
            PathGeometry geometry = new PathGeometry();
            PathFigure figure = geometry.Figures.AddPathFigure();
            figure.IsClosed = false;

            figure.StartPoint = new Point(150, 150);
            figure.Segments.AddBezierSegment(new Point(300, 50), new Point(100, 700), new Point(500, 200));

            Rect actualBounds = geometry.Bounds;
            Rect expectedBounds = new Rect(150, 139.351, 350, 251.908);
            bool areEqual = TestHelper.AreEqual(expectedBounds, actualBounds, Epsilon);

            Assert.IsTrue(areEqual);
        }

        [TestMethod]
        public void ArcGeometryBoundsSimpleTest()
        {
            PathGeometry geometry = new PathGeometry();
            PathFigure figure = geometry.Figures.AddPathFigure();
            figure.IsClosed = false;

            figure.StartPoint = new Point(150, 150);
            var arcSegment = figure.Segments.AddArcSegment(new Point(500, 300), 200, 100);
            arcSegment.IsLargeArc = true;
            arcSegment.RotationAngle = 30;

            Rect actualBounds = geometry.Bounds;
            Rect expectedBounds = new Rect(146.715, 67.372, 360.602, 232.628);
            bool areEqual = TestHelper.AreEqual(expectedBounds, actualBounds, Epsilon);

            Assert.IsTrue(areEqual);
        }

        [TestMethod]
        public void ArcGeometryBoundsComplexTest()
        {
            PathGeometry geometry = new PathGeometry();
            PathFigure figure = geometry.Figures.AddPathFigure();
            figure.IsClosed = false;

            figure.StartPoint = new Point(50, 50);
            figure.Segments.AddArcSegment(new Point(200, 200), 30, 30);
            figure.Segments.AddArcSegment(new Point(200, 200), 20, 30);
            var arcSegment = figure.Segments.AddArcSegment(new Point(300, 300), 140, 100);
            arcSegment.IsLargeArc = true;
            arcSegment.SweepDirection = SweepDirection.Counterclockwise;
            figure.Segments.AddArcSegment(new Point(400, 300), 200, 100);
            arcSegment = figure.Segments.AddArcSegment(new Point(500, 300), 200, 100);
            arcSegment.IsLargeArc = true;
            arcSegment.RotationAngle = 30;

            Rect actualBounds = geometry.Bounds;
            Rect expectedBounds = new Rect(20.120, 18.934, 517.532, 376.923);
            bool areEqual = TestHelper.AreEqual(expectedBounds, actualBounds, Epsilon);

            Assert.IsTrue(areEqual);
        }
    }
}
