﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.Type1;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.Import.Fonts
{
    [TestClass]
    public class TestCidType0Font
    {
        private static byte[] FontFile3;

        [ClassInitialize]
        public static void Initialize(TestContext context)
        {
            TestCidType0Font.FontFile3 = ResourceHelper.GetResourceBytes("Resources/Fonts/FontFile3.ocf");
        }

        [ClassCleanup]
        public static void Cleanup()
        {
            TestCidType0Font.FontFile3 = new byte[0];
        }

        [TestMethod]
        public void TestCidType0Font_InitializeCoreGlyph_WhenFontFileWithCidOperators()
        {
            // Arrange
            CFFFontFile fontFile = new CFFFontFile(FontFile3);
            CidType0Font font = new CidType0Font("test", fontFile.FontSource);

            // Act
            Glyph firstGlyph = new Glyph();
            font.InitializeCoreGlyph(firstGlyph, new CharCode(1));

            Glyph secondGlyph = new Glyph();
            font.InitializeCoreGlyph(secondGlyph, new CharCode(66));

            Glyph thirdGlyph = new Glyph();
            font.InitializeCoreGlyph(thirdGlyph, new CharCode(690));

            Glyph fourthGlyph = new Glyph();
            font.InitializeCoreGlyph(fourthGlyph, new CharCode(2742));

            // Assert
            Assert.IsNull(firstGlyph.Name);
            Assert.AreEqual(1, firstGlyph.GlyphId);
            Assert.AreEqual(2, secondGlyph.GlyphId);
            Assert.AreEqual(3, thirdGlyph.GlyphId);
            Assert.AreEqual(4, fourthGlyph.GlyphId);
        }

        [TestMethod]
        public void TestCidType0Font_InitializeCoreGlyph_WhenFontFileAndNotFoundCid_ShouldReturnZeroGlyphId()
        {
            // Arrange
            CFFFontFile fontFile = new CFFFontFile(FontFile3);
            CidType0Font font = new CidType0Font("test", fontFile.FontSource);

            // Act
            Glyph glyph = new Glyph();
            font.InitializeCoreGlyph(glyph, new CharCode(0));

            // Assert
            Assert.IsNull(glyph.Name);
            Assert.AreEqual(0, glyph.GlyphId);
        }
    }
}
