﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser.Keywords;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    [TestClass]
    public class TestCrossReferences
    {
        private class ExpectedCrossReferences
        {
            public int StartXRefKeywordByteOffset { get; set; }
            public int StartXRefByteOffset { get; set; }
            public IndirectReference RootReference { get; set; }
            public KeyValuePair<int, CrossReferenceEntry>[] Entries { get; set; } 
        }

        [TestMethod]
        public void TestCrossReferenceTableSimple()
        {
            ExpectedCrossReferences expectedReferences = new ExpectedCrossReferences()
            {
                StartXRefKeywordByteOffset = 1292,
                StartXRefByteOffset = 901,
                RootReference = new IndirectReference(10, 0),
                Entries = new KeyValuePair<int, CrossReferenceEntry>[]
                {
                    new KeyValuePair<int, CrossReferenceEntry>(0, new CrossReferenceEntry(0, 65535, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(1, new CrossReferenceEntry(108006, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(2, new CrossReferenceEntry(108158, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(3, new CrossReferenceEntry(17, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(4, new CrossReferenceEntry(21273, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(5, new CrossReferenceEntry(112563, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(6, new CrossReferenceEntry(112797, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(7, new CrossReferenceEntry(186156, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(8, new CrossReferenceEntry(275718, 0, CrossReferenceEntryType.Used)),
                }
            };

            TestCrossRefereces("Resources/CrossReferences/CrossReferenceTableSimple.txt", expectedReferences);
        }

        [TestMethod]
        public void TestCrossReferenceTableWithSections()
        {
            ExpectedCrossReferences expectedReferences = new ExpectedCrossReferences()
            {
                StartXRefKeywordByteOffset = 1369,
                StartXRefByteOffset = 838,
                RootReference = new IndirectReference(7, 0),
                Entries = new KeyValuePair<int, CrossReferenceEntry>[]
                {
                    new KeyValuePair<int, CrossReferenceEntry>(0, new CrossReferenceEntry(0, 65535, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(1, new CrossReferenceEntry(108006, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(2, new CrossReferenceEntry(108158, 100, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(3, new CrossReferenceEntry(17, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(4, new CrossReferenceEntry(21273, 99, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(5, new CrossReferenceEntry(112563, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(6, new CrossReferenceEntry(112797, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(7, new CrossReferenceEntry(186156, 49, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(8, new CrossReferenceEntry(275718, 0, CrossReferenceEntryType.Free)),

                    new KeyValuePair<int, CrossReferenceEntry>(9, new CrossReferenceEntry(17, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(10, new CrossReferenceEntry(21273, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(11, new CrossReferenceEntry(112563, 60, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(12, new CrossReferenceEntry(112797, 0, CrossReferenceEntryType.Used)),
                                        
                    new KeyValuePair<int, CrossReferenceEntry>(13, new CrossReferenceEntry(186156, 24, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(14, new CrossReferenceEntry(275718, 1, CrossReferenceEntryType.Used)),
                }
            };

            TestCrossRefereces("Resources/CrossReferences/CrossReferenceTableWithSections.txt", expectedReferences);
        }

        [TestMethod]
        public void TestCrossReferenceTableWhenMultipleCrossReferenceTables()
        {
            ExpectedCrossReferences expectedReferences = new ExpectedCrossReferences()
            {
                StartXRefKeywordByteOffset = 3706,
                StartXRefByteOffset = 2027,
                RootReference = new IndirectReference(6, 0),
                Entries = new KeyValuePair<int, CrossReferenceEntry>[]
                {
                    new KeyValuePair<int, CrossReferenceEntry>(0, new CrossReferenceEntry(0, 65535, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(1, new CrossReferenceEntry(108006, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(2, new CrossReferenceEntry(108158, 100, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(3, new CrossReferenceEntry(17, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(4, new CrossReferenceEntry(21273, 99, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(5, new CrossReferenceEntry(112563, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(6, new CrossReferenceEntry(112797, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(7, new CrossReferenceEntry(186156, 49, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(8, new CrossReferenceEntry(275718, 0, CrossReferenceEntryType.Free)),

                    new KeyValuePair<int, CrossReferenceEntry>(9, new CrossReferenceEntry(17, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(10, new CrossReferenceEntry(21273, 0, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(11, new CrossReferenceEntry(112563, 60, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(12, new CrossReferenceEntry(112797, 0, CrossReferenceEntryType.Used)),
                                        
                    new KeyValuePair<int, CrossReferenceEntry>(13, new CrossReferenceEntry(186156, 24, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(14, new CrossReferenceEntry(275718, 1, CrossReferenceEntryType.Used)),

                    new KeyValuePair<int, CrossReferenceEntry>(15, new CrossReferenceEntry(2034, 46, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(16, new CrossReferenceEntry(1034, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(17, new CrossReferenceEntry(89234, 0, CrossReferenceEntryType.Used)),
                }
            };

            TestCrossRefereces("Resources/CrossReferences/MultipleCrossReferenceTables.txt", expectedReferences);
        }

        [TestMethod]
        public void TestCrossReferenceTableWhenTrailerIsOnASingleRow()
        {
            ExpectedCrossReferences expectedReferences = new ExpectedCrossReferences()
            {
                StartXRefKeywordByteOffset = 602,
                StartXRefByteOffset = 390,
                RootReference = new IndirectReference(2, 0),
                Entries = new KeyValuePair<int, CrossReferenceEntry>[]
                {
                    new KeyValuePair<int, CrossReferenceEntry>(0, new CrossReferenceEntry(0, 65535, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(1, new CrossReferenceEntry(10, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(2, new CrossReferenceEntry(362, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(3, new CrossReferenceEntry(416, 0, CrossReferenceEntryType.Used)),
                }
            };

            TestCrossRefereces("Resources/CrossReferences/CrossReferenceTableWhenTrailerOnASingleRow.txt", expectedReferences);
        }

        [TestMethod]
        public void TestCrossReferenceTableWithMissingEof()
        {
            ExpectedCrossReferences expectedReferences = new ExpectedCrossReferences()
            {
                StartXRefKeywordByteOffset = 1114,
                StartXRefByteOffset = 901,
                RootReference = new IndirectReference(1, 0),
                Entries = new KeyValuePair<int, CrossReferenceEntry>[]
                {
                    new KeyValuePair<int, CrossReferenceEntry>(0, new CrossReferenceEntry(0, 65535, CrossReferenceEntryType.Free)),
                    new KeyValuePair<int, CrossReferenceEntry>(1, new CrossReferenceEntry(15, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(2, new CrossReferenceEntry(61, 0, CrossReferenceEntryType.Used)),
                    new KeyValuePair<int, CrossReferenceEntry>(3, new CrossReferenceEntry(146, 0, CrossReferenceEntryType.Used))
                }
            };

            TestCrossRefereces("Resources/CrossReferences/CrossReferenceTableWithMissingEof.txt", expectedReferences);
        }

        private void TestCrossRefereces(string resourceFile, ExpectedCrossReferences expectedReferences)
        {
            Stream stream = ResourceHelper.GetResourceMemoryStream(resourceFile);
            PdfImporter.SeekToStartXRef(stream);

            Assert.AreEqual(expectedReferences.StartXRefKeywordByteOffset, stream.Position, "Incorrect position of startxref keyword!");

            PostScriptReader reader = new PostScriptReader(stream, new KeywordCollection());
            TestPdfImportContext context = new TestPdfImportContext();
            context.RegisterIndirectObject(expectedReferences.RootReference, new DocumentCatalog());
            int offset = RadFixedDocumentImportContext.ReadStartXRefOffset(reader, context);

            Assert.AreEqual(expectedReferences.StartXRefByteOffset, offset);

            stream.Seek(offset, SeekOrigin.Begin);
            CrossReferenceCollection crossReferences = new CrossReferenceCollection();
            crossReferences.Read(reader, context);

            this.AssertCrossReferenceCollection(crossReferences, expectedReferences.Entries);
        }

        private void AssertCrossReferenceCollection(CrossReferenceCollection crossReferences, KeyValuePair<int, CrossReferenceEntry>[] expectedEntries)
        {
            Dictionary<int, CrossReferenceEntry> actualEntries = new Dictionary<int, CrossReferenceEntry>();
            foreach (KeyValuePair<int, CrossReferenceEntry> entry in crossReferences.Entries)
            {
                actualEntries.Add(entry.Key, entry.Value);
            }

            Assert.AreEqual(expectedEntries.Length, actualEntries.Count);
            
            for (int i = 0; i < expectedEntries.Length; i++)
            {
                int objectNumber = expectedEntries[i].Key;
                CrossReferenceEntry actualEntry;

                Assert.IsTrue(actualEntries.TryGetValue(objectNumber, out actualEntry));

                CrossReferenceEntry expectedEntry = expectedEntries[i].Value;

                Assert.AreEqual(expectedEntry.Field1, actualEntry.Field1);
                Assert.AreEqual(expectedEntry.Field2, actualEntry.Field2);
                Assert.AreEqual(expectedEntry.Type, actualEntry.Type);
            }
        }
    }
}
