﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    [TestClass]
    public class TestDestinations
    {
        [TestMethod]
        public void Destination_WhenPageIsNotDefinedAsIndirrectReferenceButAsPageIndex_IsImported()
        {
            using (Stream stream = ResourceHelper.GetResourceMemoryStream("Documents/Destinations/DestinationAsPageNumber.pdf"))
            {
                PdfFormatProvider provider = new PdfFormatProvider();

                RadFixedDocument pdfDocument = provider.Import(stream);
                RadFixedPage page = pdfDocument.Pages.First();

                Assert.AreEqual(2, page.Annotations.Count);

                Location destination = ((Link)page.Annotations[0]).Destination as Location;

                Assert.IsNotNull(destination);
                Assert.AreEqual(destination.Page, page);
                Assert.AreEqual(destination.Left.Value, 0);
                Assert.AreEqual(destination.Top.Value, 500);
                Assert.AreEqual(destination.Zoom.Value, 4);

                UriAction action = ((Link)page.Annotations[1]).Action as UriAction;
                Assert.IsNotNull(destination);
                Assert.AreEqual(destination.Page, page);
                Assert.AreEqual(action.Uri, new Uri(@"http://www.telerik.com"));
            }
        }

        [TestMethod]
        public void UriExceptionTest_WhenUriIsNotFull_Import()
        {
            using (Stream stream = ResourceHelper.GetResourceMemoryStream("Documents/Destinations/DestinationAsPageNumber_NotFullURI.pdf"))
            {
                PdfFormatProvider provider = new PdfFormatProvider();

                RadFixedDocument pdfDocument = provider.Import(stream);
                RadFixedPage page = pdfDocument.Pages.First();

                Assert.AreEqual(2, page.Annotations.Count);
                
                UriAction action = ((Link)page.Annotations[1]).Action as UriAction;
                Assert.IsNotNull(action);
                Assert.AreEqual(action.Uri, new Uri(@"www.telerik.com/", UriKind.RelativeOrAbsolute));
            }
        }
    }
}