﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    [TestClass]
    public class TestPageTree
    {
        [TestMethod]
        public void TestCreatePage()
        {
            PageTreeNode instance = PageTreeNode.CreateInstance(new PdfName("Page"));

            Assert.AreEqual(PageTreeNodeType.Page, instance.PageTreeNodeType);
        }

        [TestMethod]
        public void TestCreatePageTreeNode()
        {
            PageTreeNode instance = PageTreeNode.CreateInstance(new PdfName("Pages"));

            Assert.AreEqual(PageTreeNodeType.PageTreeNode, instance.PageTreeNodeType);
        }

        [TestMethod]
        public void TestPropertiesInheritanceImport()
        {
            RadFixedDocument document = new PdfFormatProvider().Import(ResourceHelper.GetResourceBytes("Resources/Centaur-mediabox inherited.pdf"));

            Assert.AreEqual(4, document.Pages.Count);
            AssertRadFixedPageInheritance(document.Pages[0], new Size(612, 792), Rotation.Rotate0);
            AssertRadFixedPageInheritance(document.Pages[1], new Size(700, 1400), Rotation.Rotate180);
            AssertRadFixedPageInheritance(document.Pages[2], new Size(612, 792), Rotation.Rotate90);
            AssertRadFixedPageInheritance(document.Pages[3], new Size(800, 800), Rotation.Rotate270);
        }

        private static void AssertRadFixedPageInheritance(RadFixedPage actualPage, Size expectedSize, Rotation expectedRotation)
        {
            Size actualSize = new Size(Unit.DipToPoint(actualPage.Size.Width), Unit.DipToPoint(actualPage.Size.Height));
            Assert.AreEqual(expectedSize.Width, actualSize.Width, 0.00001);
            Assert.AreEqual(expectedSize.Height, actualSize.Height, 0.00001);
            Assert.AreEqual(expectedRotation, actualPage.Rotation);
        }
    }
}
