﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using Matrix = System.Windows.Media.Matrix;
using Point = System.Windows.Point;
#endif

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    [TestClass]
    public class TestPathContentStreamOperators : TestContentStreamOperatorsBase
    {
        [TestMethod]
        public void TestPathImport_SetDashOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 0.00000 792.00000 cm
[ 2 1 ] 0 d
1.00000 1.00000 m
100.00000 100.00000 l
B
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                StrokeDashArray = new double[] { 2, 1 },
                StrokeDashOffset = 0,
                IsStroked = true,
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 0.00000, 792.00000),
                IsFilled = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                            }
                        }
                    },
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_WidthOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 0.00000 792.00000 cm
10 w
1.00000 1.00000 m
100.00000 100.00000 l
B
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 0.00000, 792.00000),
                StrokeThickness = 10,
                IsFilled = true,
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                            },
                        }
                    },
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_LineCapStyleOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
10 w
1 J
1.00000 1.00000 m
100.00000 100.00000 l
B
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsFilled = true,
                StrokeThickness = 10,
                IsStroked = true,
                StrokeLineCap = LineCap.Round,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                    },
                        }
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_LineJoinStyleOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
10 w
2 j
1.00000 1.00000 m
100.00000 100.00000 l
100.00000 200.00000 l
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                StrokeThickness = 10,
                IsStroked = true,
                StrokeLineJoin = LineJoin.Bevel,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                                new ExpectedLineSegment() { Point = new Point(100, 200) },
                    },
                        }
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_MiterLimitOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
10 w
1.5 M
1.00000 1.00000 m
100.00000 100.00000 l
1.00000 50.00000 l
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                StrokeThickness = 10,
                IsStroked = true,
                StrokeLineJoin = LineJoin.Miter,
                MiterLimit = 1.5,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                                new ExpectedLineSegment() { Point = new Point(1, 50) },
                    },
                        }
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_GraphicStateOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
/State1 gs
1.00000 1.00000 m
100.00000 100.00000 l
1.00000 100.00000 l
100.00000 1.00000 l
B
50 50 200 100 re
B
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                Fill = new RgbColor(127, 0, 0, 0),
                IsFilled = true,
                Stroke = new RgbColor(127, 0, 0, 0),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                                new ExpectedLineSegment() { Point = new Point(1, 100) },
                                new ExpectedLineSegment() { Point = new Point(100, 1) },
                            },
                    },
                    }
                },
            };

            ExpectedPath expectedRectangle = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                Fill = new RgbColor(127, 0, 0, 0),
                IsFilled = true,
                Stroke = new RgbColor(127, 0, 0, 0),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(50, 50),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(250, 50)},
                                new ExpectedLineSegment() {Point = new Point(250, 150)},
                                new ExpectedLineSegment() {Point = new Point(50, 150)},
                            }
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath, expectedRectangle });
        }

        [TestMethod]
        public void TestPathColorsImport_WithAndWithoutTransparencyAndAlphaSource()
        {
            RadFixedPage page = this.GeneratePageContent(@"
q
/State1 gs
1 0 0 rg
0 1 0 RG
50 50 200 100 re
B
Q
0.4 0 0 rg
0 0 1 RG
60 70 200 100 re
B
/StateWithAlphaWithoutAlphaSource gs
70 90 200 100 re
B
");
            ExpectedPath expectedRectangle1 = new ExpectedPath()
            {
                Fill = new RgbColor(127, 255, 0, 0),
                IsFilled = true,
                Stroke = new RgbColor(127, 0, 255, 0),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(50, 50),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(250, 50)},
                                new ExpectedLineSegment() {Point = new Point(250, 150)},
                                new ExpectedLineSegment() {Point = new Point(50, 150)},
                            }
                        }
                    }
                }
            };

            ExpectedPath expectedRectangle2 = new ExpectedPath()
            {
                Fill = new RgbColor(255, 102, 0, 0),
                IsFilled = true,
                Stroke = new RgbColor(255, 0, 0, 255),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(60, 70),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(260, 70)},
                                new ExpectedLineSegment() {Point = new Point(260, 170)},
                                new ExpectedLineSegment() {Point = new Point(60, 170)},
                            }
                        }
                    }
                }
            };

            ExpectedPath expectedRectangle3 = new ExpectedPath()
            {
                Fill = new RgbColor(51, 102, 0, 0),
                IsFilled = true,
                Stroke = new RgbColor(255, 0, 0, 255),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(70, 90),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(270, 90)},
                                new ExpectedLineSegment() {Point = new Point(270, 190)},
                                new ExpectedLineSegment() {Point = new Point(70, 190)},
                            }
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedRectangle1, expectedRectangle2, expectedRectangle3 });
        }

        [TestMethod]
        public void TestPathImport_ScOperatorsWithoutSpecifyingColorSpaceBefore()
        {
            RadFixedPage page = this.GeneratePageContent(@"
0.2 SC
0.7 0.3 0.1 0.5 scn
50 300 200 100 re
B");

            byte expectedStrokeGray = (byte)(0.2 * 255);
            Color expectedFillColor = Color.FromCmyk(0.7, 0.3, 0.1, 0.5);

            ExpectedPath expectedRectangle = new ExpectedPath()
            {
                Position = new Matrix(),
                Fill = new RgbColor(expectedFillColor.R, expectedFillColor.G, expectedFillColor.B),
                IsFilled = true,
                Stroke = new RgbColor(expectedStrokeGray, expectedStrokeGray, expectedStrokeGray),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(50, 300),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(250, 300)},
                                new ExpectedLineSegment() {Point = new Point(250, 400)},
                                new ExpectedLineSegment() {Point = new Point(50, 400)},
                            }
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedRectangle });
        }

        [TestMethod]
        public void TestPathImport_GrayColorOperators()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
0.3 g
0.7 G
2 w
1.00000 1.00000 m
100.00000 100.00000 l
1.00000 100.00000 l
100.00000 1.00000 l
B
50 300 200 100 re
B
Q");
            byte expectedFillGray = (byte)(0.3 * 255);
            byte expectedStrokeGray = (byte)(0.7 * 255);

            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                Fill = new RgbColor(expectedFillGray, expectedFillGray, expectedFillGray),
                IsFilled = true,
                Stroke = new RgbColor(expectedStrokeGray, expectedStrokeGray, expectedStrokeGray),
                IsStroked = true,
                StrokeThickness = 2,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(1, 1),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(100, 100) },
                                new ExpectedLineSegment() { Point = new Point(1, 100) },
                                new ExpectedLineSegment() { Point = new Point(100, 1) },
                            },
                    },
                    }
                },
            };

            ExpectedPath expectedRectangle = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                Fill = new RgbColor(expectedFillGray, expectedFillGray, expectedFillGray),
                IsFilled = true,
                Stroke = new RgbColor(expectedStrokeGray, expectedStrokeGray, expectedStrokeGray),
                IsStroked = true,
                StrokeThickness = 2,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(50, 300),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(250, 300)},
                                new ExpectedLineSegment() {Point = new Point(250, 400)},
                                new ExpectedLineSegment() {Point = new Point(50, 400)},
                            }
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath, expectedRectangle });
        }

        [TestMethod]
        public void TestPathImport_BezierOperatorC()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
5.00000 5.00000 m
205.00000 5.00000 l
105 50 130 105 100.00000 200.00000 c
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(5, 5),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(205, 5) },
                        new ExpectedBezierSegment()
                        {
                                    Point1 = new Point(105, 50),
                                    Point2 = new Point(130, 105),
                                    Point3 = new Point(100, 200)
                        },
                    },
                        }
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_BezierOperatorV()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
205.00000 5.00000 m
105 50 100.00000 200.00000 v
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(205, 5),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                        new ExpectedBezierSegment()
                        {
                                    Point1 = new Point(205, 5),
                                    Point2 = new Point(105, 50),
                                    Point3 = new Point(100, 200)
                        },
                    },
                }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_BezierOperatorY()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
5.00000 5.00000 m
205.00000 5.00000 l
105 50 100.00000 200.00000 y
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(5, 5),
                    ExpectedSegments = new List<ExpectedSegment>()
                    {
                                new ExpectedLineSegment() { Point = new Point(205, 5) },
                        new ExpectedBezierSegment()
                        {
                                    Point1 = new Point(105, 50),
                                    Point2 = new Point(100, 200),
                                    Point3 = new Point(100, 200)
                        },
                    },
                }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
5.00000 5.00000 m
205.00000 5.00000 l
100.00000 200.00000 l
h
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(5, 5),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(205, 5) },
                                new ExpectedLineSegment() { Point = new Point(100, 200) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseOperatorMovesBeginingOfNextSubpathToBeginingOfCurrent()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
50.00000 50.00000 m
100.00000 100.00000 l
100.00000 200.00000 l
h
1.00000 100.00000 l
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(50, 50),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(100, 100)},
                                new ExpectedLineSegment() {Point = new Point(100, 200)},
                            },
                        },
                        new ExpectedFigure()
                        {
                            //The start point coincides with the start point of the previous subpath.
                            StartPoint = new Point(50, 50),
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() {Point = new Point(1, 100)},
                            },
                        }
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseAndStrokeOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
5.00000 5.00000 m
205.00000 5.00000 l
100.00000 200.00000 l
s
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(5, 5),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(205, 5) },
                                new ExpectedLineSegment() { Point = new Point(100, 200) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseFillAndStrokeOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
25.00000 35.00000 m
215.00000 15.00000 l
200.00000 300.00000 l
b
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsFilled = true,
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(25, 35),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(215, 15) },
                                new ExpectedLineSegment() { Point = new Point(200, 300) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseFillAndStrokeOperatorWithComplexPathGeometry()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
q
100.00000 100.00000 m
50.00000 0.00000 l
0.00000 100.00000 l
100.00000 100.00000 l
100.00000 50.00000 m
0.00000 50.00000 l
50.00000 150.00000 l
100.00000 50.00000 l
b
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsFilled = true,
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(100, 100),
                            IsClosed = false,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(50, 0) },
                                new ExpectedLineSegment() { Point = new Point(0, 100) },
                                new ExpectedLineSegment() { Point = new Point(100, 100) }
                            },
                        },
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(100, 50),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(0, 50) },
                                new ExpectedLineSegment() { Point = new Point(50, 150) },
                                new ExpectedLineSegment() { Point = new Point(100, 50) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseFillAndStrokeEvenOddOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
125.00000 135.00000 m
315.00000 115.00000 l
300.00000 400.00000 l
b*
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsFilled = true,
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    FillRule = FillRule.EvenOdd,
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(125, 135),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(315, 115) },
                                new ExpectedLineSegment() { Point = new Point(300, 400) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_CloseFillAndStrokeEvenOddOperatorWithComplexPathGeometry()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
q
100.00000 100.00000 m
50.00000 0.00000 l
0.00000 100.00000 l
100.00000 100.00000 l
100.00000 50.00000 m
0.00000 50.00000 l
50.00000 150.00000 l
100.00000 50.00000 l
b*
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsFilled = true,
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    FillRule = FillRule.EvenOdd,
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(100, 100),
                            IsClosed = false,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(50, 0) },
                                new ExpectedLineSegment() { Point = new Point(0, 100) },
                                new ExpectedLineSegment() { Point = new Point(100, 100) }
                            },
                        },
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(100, 50),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(0, 50) },
                                new ExpectedLineSegment() { Point = new Point(50, 150) },
                                new ExpectedLineSegment() { Point = new Point(100, 50) }
                            },
                        }
                    }
                }
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_RectangleOperatorInterruptingPath()
        {
            RadFixedPage page = this.GeneratePageContent(@"0.75000 0.00000 0.00000 -0.75000 100.00000 600.00000 cm
5.00000 5.00000 m
205.00000 5.00000 l
100.00000 200.00000 10 20 re
5.00000 100.00000 l
S
Q");
            ExpectedPath expectedPath = new ExpectedPath()
            {
                Position = new Matrix(0.75000, 0.00000, 0.00000, -0.75000, 100.00000, 600.00000),
                IsStroked = true,
                ExpectedGeometry = new ExpectedPathGeometry()
                {
                    ExpectedFigures = new List<ExpectedFigure>()
                    {
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(5, 5),
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(205, 5) },
                            },
                        },
                        new ExpectedFigure()
                        {
                            StartPoint = new Point(100, 200),
                            IsClosed = true,
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(110, 200) },
                                new ExpectedLineSegment() { Point = new Point(110, 220) },
                                new ExpectedLineSegment() { Point = new Point(100, 220) },
                            },
                        },
                        new ExpectedFigure()
                         {
                            StartPoint = new Point(100, 200),
                            ExpectedSegments = new List<ExpectedSegment>()
                            {
                                new ExpectedLineSegment() { Point = new Point(5, 100) },
                            },
                         },
                    }
                },
            };

            this.AssertPaths(page, new ExpectedPath[] { expectedPath });
        }

        [TestMethod]
        public void TestPathImport_ClippingsTest()
        {
            RadFixedPage page = this.GeneratePageContent(@"2 0 0 3 10 40 cm
100 200 10 20 re S
q
105 205 -5 15 re W n
q
110 210 15 -25 re W n
115 215 10 -20 re S
Q
95 195 5 10 re S
Q
60 70 5 15 re S
");
            Matrix expectedPosition = new Matrix(2, 0, 0, 3, 10, 40);

            ExpectedPath[] expectedPaths = new ExpectedPath[]
            {
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(100, 200, 10, 20),
                    IsStroked = true
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(115, 215, 10, -20),
                    IsStroked = true,
                    Clipping = new ExpectedClipping()
                    {
                        Clip = this.GetExpectedRectangle(110, 210, 15, -25),
                        Transform = expectedPosition
                    }
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(95, 195, 5, 10),
                    IsStroked = true,
                    Clipping = new ExpectedClipping()
                    {
                        Clip = this.GetExpectedRectangle(105, 205, -5, 15),
                        Transform = expectedPosition
                    }
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(60, 70, 5, 15),
                    IsStroked = true
                },
            };

            this.AssertPaths(page, expectedPaths);
        }

        [TestMethod]
        public void TestPathImport_ClippingsEvenOddTest()
        {
            RadFixedPage page = this.GeneratePageContent(@"2 0 0 3 10 40 cm
100 200 10 20 re S
q
105 205 -5 15 re W* n
q
110 210 15 -25 re W* n
115 215 10 -20 re S
Q
95 195 5 10 re S
Q
60 70 5 15 re S
");
            Matrix expectedPosition = new Matrix(2, 0, 0, 3, 10, 40);

            ExpectedPath[] expectedPaths = new ExpectedPath[]
            {
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(100, 200, 10, 20),
                    IsStroked = true
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(115, 215, 10, -20),
                    IsStroked = true,
                    Clipping = new ExpectedClipping()
                    {
                        Clip = this.GetExpectedRectangle(110, 210, 15, -25, FillRule.EvenOdd),
                        Transform = expectedPosition
                    }
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(95, 195, 5, 10),
                    IsStroked = true,
                    Clipping = new ExpectedClipping()
                    {
                        Clip = this.GetExpectedRectangle(105, 205, -5, 15, FillRule.EvenOdd),
                        Transform = expectedPosition
                    }
                },
                new ExpectedPath()
                {
                    Position = expectedPosition,
                    ExpectedGeometry = this.GetExpectedRectangle(60, 70, 5, 15),
                    IsStroked = true
                },
            };

            this.AssertPaths(page, expectedPaths);
        }

        private void AssertPaths(RadFixedPage page, ExpectedPath[] expectedPaths)
        {
            Assert.AreEqual(expectedPaths.Length, page.Content.Count);

            for (int index = 0; index < expectedPaths.Length; index++)
            {
                Path path = (Path)page.Content[index];
                ExpectedPath expectedPath = expectedPaths[index];

                this.AssertPathProperties(expectedPath, path);
                this.AssertContentElementPosition(page, path, expectedPath.Position);
                base.AssertContentElementClipping(page, path, expectedPath.Clipping);
                base.AssertGeometry(path.Geometry, expectedPath.ExpectedGeometry);
            }
        }

        private void AssertPathProperties(ExpectedPath expectedGeometry, Path path)
        {
            if (expectedGeometry.StrokeDashArray != null)
            {
                Assert.IsNotNull(path.StrokeDashArray);
                Assert.AreEqual(expectedGeometry.StrokeDashArray.Count(), path.StrokeDashArray.Count());

                for (int i = 0; i < expectedGeometry.StrokeDashArray.Count(); i++)
                {
                    Assert.AreEqual(expectedGeometry.StrokeDashArray.ElementAt(i), path.StrokeDashArray.ElementAt(i));
                }
            }
            else
            {
                Assert.IsNull(path.StrokeDashArray);
            }

            Assert.AreEqual(expectedGeometry.StrokeDashOffset, path.StrokeDashOffset);
            Assert.AreEqual(expectedGeometry.StrokeThickness, path.StrokeThickness);
            Assert.AreEqual(expectedGeometry.StrokeLineCap, path.StrokeLineCap);
            Assert.AreEqual(expectedGeometry.StrokeLineJoin, path.StrokeLineJoin);
            Assert.AreEqual(expectedGeometry.MiterLimit, path.MiterLimit);
            Assert.AreEqual(expectedGeometry.Stroke, path.Stroke);
            Assert.AreEqual(expectedGeometry.IsStroked, path.IsStroked);
            Assert.AreEqual(expectedGeometry.Fill, path.Fill);
            Assert.AreEqual(expectedGeometry.IsFilled, path.IsFilled);
        }

        protected override Matrix GetActualMatrix(RadFixedPage page, Matrix positionMatrix)
        {
            return GetGeometryActualMatrix(page, positionMatrix);
        }
    }
}
