﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.GraphicsState;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Patterns;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    internal class TestPdfContentImportContext : IPdfContentImportContext
    {
        private readonly TestPdfImportContext importContext;
        private readonly Dictionary<string, FontObject> fonts;
        private readonly Dictionary<string, ExtGStateObject> graphicStates;
        private readonly RadFixedPage page;
        private readonly Dictionary<string, ColorSpaceObject> nameToColorSpace;

        public TestPdfContentImportContext()
            : this(new TestPdfImportContext(), new Dictionary<string, FontObject>(), new Dictionary<string, ExtGStateObject>(), new RadFixedPage())
        {
        }

        public TestPdfContentImportContext(TestPdfImportContext importContext, Dictionary<string, FontObject> fonts,
            Dictionary<string, ExtGStateObject> graphicStates, RadFixedPage page)
        {
            this.nameToColorSpace = new Dictionary<string, ColorSpaceObject>();
            this.importContext = importContext;
            this.fonts = fonts;
            this.graphicStates = graphicStates;
            this.page = page;
        }

        public IRadFixedDocumentImportContext Owner
        {
            get
            {
                return this.importContext;
            }
        }

        public IResourceHolder ResourceHolder
        {
            get { throw new NotImplementedException(); }
        }

        public IContentRootElement ContentRoot
        {
            get
            {
                return this.page;
            }
        }

        public Marker CurrentMarker { get; set; }

        internal Dictionary<string, ColorSpaceObject> NameToColorSpace
        {
            get
            {
                return this.nameToColorSpace;
            }
        }

        public FontObject GetFont(PostScriptReader reader, PdfName key)
        {
            return this.fonts[key.Value];
        }

        public XObjectBase GetXObject(PostScriptReader reader, PdfName key)
        {
            throw new NotImplementedException();
        }

        public ExtGStateObject GetExtGState(PostScriptReader reader, PdfName key)
        {
            return this.graphicStates[key.Value];
        }

        public PatternColorObject GetPatternColor(PostScriptReader reader, PdfName key)
        {
            throw new NotImplementedException();
        }

        public ColorSpaceObject GetColorSpace(PostScriptReader reader, PdfName key)
        {
            return this.NameToColorSpace[key.Value];
        }
    }
}
