﻿using System;
using System.Collections.Generic;
using System.IO;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import.Parser;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Annotations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.DocumentStructure;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Fonts;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Forms;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Objects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Encryption;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Types;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Utilities;
using ImageSource = Telerik.Windows.Documents.Fixed.Model.Resources.ImageSource;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    internal class TestPdfImportContext : IRadFixedDocumentImportContext
    {
        private readonly Dictionary<IndirectReference, IndirectObject> indirectObjects;
        private readonly Dictionary<FormFieldNode, List<WidgetObject>> parentToWidgets;
        private readonly BeginEndCounter innerStreamCounter;
        private readonly RadFixedDocument document;
        private readonly List<Page> pages;
        private readonly PdfImportSettings importSettings;
        private readonly Dictionary<Page, RadFixedPage> pageToFixedPage;
        private readonly Dictionary<RadFixedPage, Page> fixedPageToPage;
        private readonly Dictionary<Page, Matrix> pageToMatrix;
        private readonly Dictionary<PageTreeNode, PdfDictionary> pagesToPdfDictionaries;

        public TestPdfImportContext()
        {
            this.TryGetFixedPageMethodCalledCounter = 0;
            this.GetPdfPointToDipTransformationMethodCalledCounter = 0;

            this.indirectObjects = new Dictionary<IndirectReference, IndirectObject>();
            this.parentToWidgets = new Dictionary<FormFieldNode, List<WidgetObject>>();
            this.innerStreamCounter = new BeginEndCounter();
            this.document = new RadFixedDocument();
            this.pages = new List<Page>();
            this.importSettings = new PdfImportSettings();
            this.fixedPageToPage = new Dictionary<RadFixedPage, Page>();
            this.pageToMatrix = new Dictionary<Page, Matrix>();
            this.pageToFixedPage = new Dictionary<Page, RadFixedPage>();
            this.pagesToPdfDictionaries = new Dictionary<PageTreeNode, PdfDictionary>();
            this.IndirectObjectsToRead = new Dictionary<IndirectReference, IndirectObject>();
            this.RegisteredIndirectObjects = new Dictionary<IndirectReference, IndirectObject>();
        }

        public byte[] DocumentId
        {
            get;
            set;
        }

        public Encrypt Encryption
        {
            get;
            set;
        }

        public ReferenceProperty<DocumentCatalog> Root
        {
            get;
            set;
        }

        public Dictionary<FormFieldNode, List<WidgetObject>> ParentToWidgets
        {
            get
            {
                return this.parentToWidgets;
            }
        }

        public List<Page> Pages
        {
            get
            {
                return this.pages;
            }
        }

        public RadFixedDocument Document
        {
            get { return this.document; }
        }

        public CrossReferenceCollection CrossReferences
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        public IndirectReference CurrentIndirectReference
        {
            get
            {
                return null;
            }
        }

        public PdfImportSettings ImportSettings
        {
            get
            {
                return this.importSettings;
            }
        }

        public PostScriptReader Reader { get; set; }

        public Dictionary<IndirectReference, IndirectObject> IndirectObjectsToRead { get; set; }

        public Dictionary<IndirectReference, IndirectObject> RegisteredIndirectObjects { get; set; }

        public Dictionary<Page, RadFixedPage> PageToFixedPage
        {
            get
            {
                return this.pageToFixedPage;
            }
        }

        public Dictionary<Page, Matrix> PageToMatrix
        {
            get
            {
                return this.pageToMatrix;
            }
        }

        public int TryGetFixedPageMethodCalledCounter { get; set; }

        public int GetPdfPointToDipTransformationMethodCalledCounter { get; set; }

        public void BeginImport(Stream stream)
        {
            throw new NotImplementedException();
        }

        public IndirectObject ReadIndirectObject(IndirectReference reference)
        {
            return this.IndirectObjectsToRead[reference];
        }

        public FontBase GetFont(PostScriptReader reader, FontObject font)
        {
            return font.ToFont(reader, this);
        }

        public void RegisterIndirectObject(IndirectReference reference, IPdfPrimitive primitive)
        {
            if (!this.indirectObjects.ContainsKey(reference))
            {
                IndirectObject indirectObject = new IndirectObject(reference, primitive);
                this.indirectObjects.Add(reference, indirectObject);
                this.RegisteredIndirectObjects.Add(reference, indirectObject);
            }
        }

        public bool TryGetIndirectObject(IndirectReference reference, out IPdfPrimitive primitive)
        {
            IndirectObject indirectObject;
            bool result = this.indirectObjects.TryGetValue(reference, out indirectObject);
            if (result)
            {
                primitive = indirectObject.Content;
            }
            else
            {
                primitive = null;
            }

            return result;
        }

        public IDisposable BeginImportOfStreamInnerContent()
        {
            return this.innerStreamCounter.Begin();
        }

        public ImageSource GetImageSource(PostScriptReader reader, ImageXObject image)
        {
            throw new NotImplementedException();
        }

        public FormSource GetFormSource(PostScriptReader reader, FormXObject form)
        {
            throw new NotImplementedException();
        }

        public byte[] DecryptStream(IndirectReference reference, byte[] data)
        {
            return data;
        }

        public byte[] DecryptString(byte[] data)
        {
            return data;
        }

        public void MapPages(Page page, RadFixedPage fixedPage)
        {
            this.fixedPageToPage[fixedPage] = page;
        }

        public RadFixedPage GetFixedPage(Page page)
        {
            this.TryGetFixedPageMethodCalledCounter++;

            return this.pageToFixedPage[page];
        }

        public Page GetPage(RadFixedPage fixedPage)
        {
            return this.fixedPageToPage[fixedPage];
        }

        public ExtGState GetExtGState(PostScriptReader reader, Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.GraphicsState.ExtGStateObject extGState)
        {
            return extGState.ToExtGState();
        }

        public Matrix GetPdfPointToDipTransformation(Page page)
        {
            this.GetPdfPointToDipTransformationMethodCalledCounter++;

            return this.PageToMatrix[page];
        }

        public void MapFields(FormFieldNode node, FormField field)
        {
            throw new NotImplementedException();
        }

        public bool TryGetField(FormFieldNode node, out FormField field)
        {
            throw new NotImplementedException();
        }

        public void MapWidgets(WidgetObject widgetObject, Widget widget)
        {
            throw new NotImplementedException();
        }

        public bool TryGetWidget(WidgetObject widgetObject, out Widget widget)
        {
            throw new NotImplementedException();
        }

        public void AddWidgetParent(WidgetObject widgetObject, FormFieldNode parent)
        {
            List<WidgetObject> childWidgets;
            if (!this.ParentToWidgets.TryGetValue(parent, out childWidgets))
            {
                childWidgets = new List<WidgetObject>();
                this.ParentToWidgets.Add(parent, childWidgets);
            }

            childWidgets.Add(widgetObject);
        }

        public IEnumerable<WidgetObject> GetChildWidgets(FormFieldNode parent)
        {
            throw new NotImplementedException();
        }

        public IPdfContentImportContext GetAcroFormContentImportContext()
        {
            throw new NotImplementedException();
        }

        public void MapPageToPdfDictionary(PageTreeNode page, PdfDictionary dictionary)
        {
            this.pagesToPdfDictionaries[page] = dictionary;
        }

        public void UnmapPdfDictionary(PageTreeNode page)
        {
            this.pagesToPdfDictionaries.Remove(page);
        }

        public bool TryGetPdfDictionary(PageTreeNode page, out PdfDictionary dictionary)
        {
            return this.pagesToPdfDictionaries.TryGetValue(page, out dictionary);
        }

        public void MapNameToNamedDestination(string name, NamedDestinationObject destination)
        {
            throw new NotImplementedException();
        }

        public bool TryGetNamedDestination(string name, out NamedDestinationObject destination)
        {
            throw new NotImplementedException();
        }
    }
}
