﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Fixed.Tests.Import
{
    [TestClass]
    public class TestTextContentStreamOperators : TestContentStreamOperatorsBase
    {
        private const double AbcTextWidthFontSize12 = 24.672;
        private readonly Matrix flipTextMatrix;

        public TestTextContentStreamOperators()
        {
            this.flipTextMatrix = new Matrix(1, 0, 0, -1, 0, 0);
        }

        [TestMethod]
        public void TestTextFragmentImport_OperatorTm()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
9 0 0 10 282 712 Tm
(ABC) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(9, 0, 0, 10, 282, 712)),
            });
        }


        [TestMethod]
        public void TestTextFragmentImport_OperatorTj_WhenEmpty()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
9 0 0 10 282 712 Tm
 Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[0]);
        }

        [TestMethod]
        public void TestTextFragmentImport_OperatorTd()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
288 720 Td
(ABC) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 288, 720)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_MultipleOperatorTd()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
20 50 Td
(ABC) Tj
20 50 Td
(Def) Tj
ET");

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 20, 50)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 40, 100)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_MultipleTjOperators()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
20 50 Td
(ABC) Tj
(Def) Tj
ET");

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 20, 50)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 20 + AbcTextWidthFontSize12, 50)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_TextLeadAndTAsteristSymbols()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
1 0 0 1 0 40 cm
20 TL
(ABC) Tj
T*
(Def) Tj
T*
(gH) Tj
ET");

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 0, 40)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 0, 20)),
                new ExpectedTextFragment("gH", new Matrix(1, 0, 0, 1, 0, 0)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_TDOperatorWithTAsteriskOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
1 0 0 1 0 150 cm
(First) Tj
10 -20 TD
(ABC) Tj
(Def) Tj
T*
(gH) Tj
T*
T*
5 -30 TD
(ij) Tj
T*
(The end) Tj
ET");

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("First", new Matrix(1, 0, 0, 1, 0, 150)),
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 10, 130)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 10 + AbcTextWidthFontSize12, 130)),
                new ExpectedTextFragment("gH", new Matrix(1, 0, 0, 1, 10, 110)),
                new ExpectedTextFragment("ij", new Matrix(1, 0, 0, 1, 15, 40)),
                new ExpectedTextFragment("The end", new Matrix(1, 0, 0, 1, 15, 10)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_TextShowingOperators()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
1 0 0 1 15 150 cm
20 TL
(ABC) Tj
(Def) Tj
(single quote) '
5 10 (double quote) ""
T*
(is there character and word spacing here) Tj
ET");

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 15, 150)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 15 + AbcTextWidthFontSize12, 150)),
                new ExpectedTextFragment("single quote", new Matrix(1, 0, 0, 1, 15, 130)),
                new ExpectedTextFragment("double quote", new Matrix(1, 0, 0, 1, 15, 110), 10, 5),
                new ExpectedTextFragment("is there character and word spacing here", new Matrix(1, 0, 0, 1, 15, 90), 10, 5),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_ShowTextArrayOperator()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
1 0 0 1 15 150 cm
20 TL
[(ABC)(Def)] TJ
T*
[-4550(ABC)-2350(Def)] TJ
ET");

            double[] displacements = 
            {
                ConvertTextArrayDisplacementToPdfUnit(-4550, 12, 100),
                ConvertTextArrayDisplacementToPdfUnit(-2350, 12, 100),
            };

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 15, 150)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 15 + AbcTextWidthFontSize12, 150)),
                new ExpectedTextFragment("ABC", new Matrix(1, 0, 0, 1, 15 - displacements[0], 130)),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 15 + AbcTextWidthFontSize12 - displacements[0] - displacements[1], 130)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_ShowTextArrayOperatorWithAllRelevantTransformations()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 5 Tf
20 TL
150 Tz
1.5 0.5 1 2 -60 100 cm
2 1 1 4 100 200 Tm
[(ABC)(Def)] TJ
T*
[-4550(ABC)6350(Def)] TJ
ET");

            
            double аbcTextWidth = 15.42;
            double[] displacements = 
            {
                ConvertTextArrayDisplacementToPdfUnit(-4550, 5, 150),
                ConvertTextArrayDisplacementToPdfUnit(6350, 5, 150),
            };

            Matrix initialTransform = new Matrix(2, 1, 1, 4, 100, 200).MultiplyBy(new Matrix(1.5, 0.5, 1, 2, -60, 100));
            Func<double, double, Matrix> getTranslation = (deltaX, deltaY) => { return new Matrix(1, 0, 0, 1, deltaX, deltaY); };
            List<Matrix> expectedTransforms = new List<Matrix>();
            expectedTransforms.Add(initialTransform);
            expectedTransforms.Add(getTranslation(аbcTextWidth, 0).MultiplyBy(expectedTransforms.Last()));
            expectedTransforms.Add(getTranslation(-displacements[0], -20).MultiplyBy(initialTransform));
            expectedTransforms.Add(getTranslation(аbcTextWidth - displacements[1], 0).MultiplyBy(expectedTransforms.Last()));

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", expectedTransforms[0]),
                new ExpectedTextFragment("Def", expectedTransforms[1]),
                new ExpectedTextFragment("ABC", expectedTransforms[2]),
                new ExpectedTextFragment("Def", expectedTransforms[3]),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_TestWordSpacingAndCharacterSpacingCalculations()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
10 Tc
3 Tw
1 0 0 1 100 200 cm
[( ABC )(Def)] TJ
ET");
            double characterSpacing = 10;
            double wordSpacing = 3;
            double spaceCharacterWidth = 3.336;
            double expectedFragmentWidth = AbcTextWidthFontSize12 + 2 * spaceCharacterWidth + 5 * characterSpacing + 2 * wordSpacing;

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment(" ABC ", new Matrix(1, 0, 0, 1, 100, 200), characterSpacing, wordSpacing),
                new ExpectedTextFragment("Def", new Matrix(1, 0, 0, 1, 100 + expectedFragmentWidth, 200), characterSpacing, wordSpacing),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_Clipping()
        {
            RadFixedPage page = this.GeneratePageContent(@"2 0 0 3 15 150 cm
BT /F1 12 Tf 20 TL (ABC) Tj T* ET
q
95 205 -5 15 re W n
q
105 125 5 25 re W n
BT /F1 12 Tf 20 TL (gh) Tj T* ET
Q
BT /F1 12 Tf 20 TL (Def) Tj T* ET
Q
BT /F1 12 Tf 20 TL (ijK) Tj T* ET
");

            Matrix expectedPosition = new Matrix(2, 0, 0, 3, 15, 150);

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("ABC", expectedPosition),
                new ExpectedTextFragment("gh", expectedPosition)
                {
                    Clipping = new ExpectedClipping()
                    {
                        Transform = expectedPosition,
                        Clip = base.GetExpectedRectangle(105, 125, 5, 25)
                    }
                },
                new ExpectedTextFragment("Def", expectedPosition)
                {
                    Clipping = new ExpectedClipping()
                    {                        
                        Transform = expectedPosition,
                        Clip = base.GetExpectedRectangle(95, 205, -5, 15)
                    }
                },
                new ExpectedTextFragment("ijK", expectedPosition),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_SingleLineWrittenOnMultipleLines()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
288 720 Td
(This\
 is\
 single line\
 text) Tj
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is single line text", new Matrix(1, 0, 0, 1, 288, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 298, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_SingleLineWrittenOnMultipleLinesWithDifferentLineEndings()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
288 720 Td
" + 
"(This\\\r" +
" is\\\n" +
" single line\\\r\n" +
@" text) Tj
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is single line text", new Matrix(1, 0, 0, 1, 288, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 298, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_EndOfLineCharacters()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
28 720 Td
" +
"(This\\\r" +
" is\\\n" +
" single line\\\r\n" +
" text\\\n\r" +
"with\\\r\r" +
@" new
 line\r\n" +
"characters\n" +
"different\r" +
"combination\n\r" +
"scenarios\\r\n" +
"...?\r\n?" +
@") Tj
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is single line text\nwith\n new\n line\r\ncharacters\ndifferent\ncombination\n\nscenarios\r\n...?\n?", new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 38, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_SingleEscapingSymbolWithBalancedBrackets()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
28 720 Td
(This example contains escaped characters in balanced brackets:(\r;\n;\t;\b;\f;\\;\\\;\);\(;\wrong)) Tj
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This example contains escaped characters in balanced brackets:(\r;\n;\t;\b;\f;\\;\\;);(;wrong)", new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 38, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_Brackets()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
28 720 Td
(This is example containing brackets (2+(2 - 3) * 4\))-:\(-:) Tj
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is example containing brackets (2+(2 - 3) * 4))-:(-:", new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 38, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_BalancedBracketDuringEscapeMode()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
30 TL
28 720 Td
(This is example with balanced bracket after octal: (\57) end.) Tj
10 -20 Td
" +
"(This is example with balanced bracket after escaping cariage return: (\\\r) end.) Tj" +
@"
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is example with balanced bracket after octal: (/) end.", new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("This is example with balanced bracket after escaping cariage return: () end.", new Matrix(1, 0, 0, 1, 38, 700)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 48, 680)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_OctalNumbers()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
20 TL
28 720 Td
(This example contains octals: \61\062\634\65
\66 \1237\57a \
123 \461 \400\572 \48 \8\0end) Tj
10 -20 Td
(some other text) Tj
ET");
            string expectedText = string.Format("This example contains octals: 12{0}5{1}6 S7/a 123 1 {2}z {3}8 8{2}end", (char)156, '\n', (char)0, (char)4);

            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment(expectedText, new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 38, 700)),
            });
        }

        [TestMethod]
        public void TestTextFragmentImport_LiteralStringEscapings_NewLinesAtTheEnd()
        {
            RadFixedPage page = this.GeneratePageContent(@"BT
/F1 12 Tf
20 TL
28 720 Td
(This is new line
with new line at the end
) Tj
10 -20 Td
(This is new line
with new lines at the end


) Tj
10 -20 Td
" +
"(This is new line\rwith cariage return at the end\r) Tj" +
@"
10 -20 Td
" +
"(This is new line\rwith cariage returns at the end\r\r\r) Tj" +
@"
10 -20 Td
" +
"(This is new line\nwith end of line marker at the end\n) Tj" +
@"
10 -20 Td
" +
"(This is new line\nwith end of line markers at the end\n\n\n) Tj" +
@"
10 -20 Td
" +
"(This is new line\\rwith escaped cariage return at the end\\r) Tj" +
@"
10 -20 Td
" +
"(This is new line\\rwith escaped cariage returns at the end\\r\\r\\r) Tj" +
@"
10 -20 Td
" +
"(This is new line\\nwith escaped end of line marker at the end\\n) Tj" +
@"
10 -20 Td
" +
"(This is new line\\nwith escaped end of line markers at the end\\n\\n\\n) Tj" +
@"
10 -20 Td
" +
"(This is new line\\r\\nwith escaped cariage return and end of line marker at the end\\r\\n) Tj" +
@"
10 -20 Td
" +
"(This is new line\\r\\nwith escaped cariage return and end of line markers at the end\\r\\n\\r\\n\\r\\n) Tj" +
@"
10 -20 Td
(some other text) Tj
ET");
            this.AssertTextFragments(page, new ExpectedTextFragment[]
            {
                new ExpectedTextFragment("This is new line\nwith new line at the end\n", new Matrix(1, 0, 0, 1, 28, 720)),
                new ExpectedTextFragment("This is new line\nwith new lines at the end\n\n\n", new Matrix(1, 0, 0, 1, 38, 700)),
                new ExpectedTextFragment("This is new line\nwith cariage return at the end\n", new Matrix(1, 0, 0, 1, 48, 680)),
                new ExpectedTextFragment("This is new line\nwith cariage returns at the end\n\n\n", new Matrix(1, 0, 0, 1, 58, 660)),
                new ExpectedTextFragment("This is new line\nwith end of line marker at the end\n", new Matrix(1, 0, 0, 1, 68, 640)),
                new ExpectedTextFragment("This is new line\nwith end of line markers at the end\n\n\n", new Matrix(1, 0, 0, 1, 78, 620)),
                new ExpectedTextFragment("This is new line\rwith escaped cariage return at the end\r", new Matrix(1, 0, 0, 1, 88, 600)),
                new ExpectedTextFragment("This is new line\rwith escaped cariage returns at the end\r\r\r", new Matrix(1, 0, 0, 1, 98, 580)),
                new ExpectedTextFragment("This is new line\nwith escaped end of line marker at the end\n", new Matrix(1, 0, 0, 1, 108, 560)),
                new ExpectedTextFragment("This is new line\nwith escaped end of line markers at the end\n\n\n", new Matrix(1, 0, 0, 1, 118, 540)),
                new ExpectedTextFragment("This is new line\r\nwith escaped cariage return and end of line marker at the end\r\n", new Matrix(1, 0, 0, 1, 128, 520)),
                new ExpectedTextFragment("This is new line\r\nwith escaped cariage return and end of line markers at the end\r\n\r\n\r\n", new Matrix(1, 0, 0, 1, 138, 500)),
                new ExpectedTextFragment("some other text", new Matrix(1, 0, 0, 1, 148, 480)),
            });
        }

        protected override Matrix GetActualMatrix(RadFixedPage page, Matrix positionMatrix)
        {
            Matrix conversionMatrix = GetCoordinatesConversionMatrix(page);
            Matrix actualMatrix = this.flipTextMatrix
                .MultiplyBy(positionMatrix)
                .MultiplyBy(conversionMatrix);

            return actualMatrix;
        }

        private void AssertTextFragments(RadFixedPage page, ExpectedTextFragment[] expectedTextFragments)
        {
            Assert.AreEqual(expectedTextFragments.Length, page.Content.Count);

            for (int index = 0; index < expectedTextFragments.Length; index++)
            {
                TextFragment text = (TextFragment)page.Content[index];
                ExpectedTextFragment expectedValues = expectedTextFragments[index];
                Assert.AreEqual(expectedValues.Text, text.Text);
                Assert.AreEqual(expectedValues.WordSpacing, text.WordSpacing ?? 0);
                Assert.AreEqual(expectedValues.CharacterSpacing, text.CharacterSpacing ?? 0);
                base.AssertContentElementPosition(page, text, expectedValues.Position);
                base.AssertContentElementClipping(page, text, expectedValues.Clipping);
            }
        }

        private static double ConvertTextArrayDisplacementToPdfUnit(double displacement, double fontSize, double horizontalScalingPercent)
        {
            return (displacement / 1000) * fontSize * (horizontalScalingPercent / 100);
        }
    }
}
