﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Model.Actions;

#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using Matrix = System.Windows.Media.Matrix;
#endif

namespace Telerik.Windows.Documents.FixedDocumentViewers.Tests.Integration
{
    internal static class AssertHelper
    {
        private const double Epsilon = 1E-1;

        public static void AssertDocument(RadFixedDocument expected, RadFixedDocument document, bool isOldFormatProviderImport)
        {
            Assert.AreEqual(expected.Pages.Count, document.Pages.Count);
            Assert.AreEqual(expected.NamedDestinations.Count, document.NamedDestinations.Count);
            AssertAcroForm(expected.AcroForm, document.AcroForm);
            AssertAnnotations(expected.Annotations, document.Annotations);
            AssertNamedDestinations(expected.NamedDestinations, document.NamedDestinations);

            for (int i = 0; i < expected.Pages.Count; i++)
            {
                AssertPage(expected.Pages[i], document.Pages[i], isOldFormatProviderImport);
            }
        }

        private static void AssertNamedDestinations(NamedDestinationsCollection expected, NamedDestinationsCollection namedDestinations)
        {
            if (expected == null && namedDestinations == null)
            {
                return;
            }

            if (expected == null || namedDestinations == null)
            {
                Assert.Fail();
            }

            Assert.AreEqual(expected.Count, namedDestinations.Count);
            List<NamedDestination> expectedList = expected.ToList();
            List<NamedDestination> namedDestinationsList = namedDestinations.ToList();

            for (int i = 0; i < expected.Count; i++)
            {
                AssertNamedDestination(expectedList[i], namedDestinationsList[i]);
            }
        }

        private static void AssertAnnotations(IEnumerable<Annotation> expected, IEnumerable<Annotation> annoations)
        {
            List<Annotation> expectedList = expected.ToList();
            List<Annotation> annotationsList = annoations.ToList();

            Assert.AreEqual(expectedList.Count, annotationsList.Count);

            for (int i = 0; i < expectedList.Count; i++)
            {
                AssertAnnotation(expectedList[i], annotationsList[i]);
            }
        }

        private static void AssertAnnotation(Annotation expected, Annotation annotation)
        {
            Link expectedLink = expected as Link;
            Link annotationLink = annotation as Link;

            AssertLinkAnnotation(expectedLink, annotationLink);
        }

        private static void AssertLinkAnnotation(Link expectedLink, Link annotationLink)
        {
            if (expectedLink == null && annotationLink == null)
            {
                return;
            }

            if (expectedLink == null || annotationLink == null)
            {
                Assert.Fail();
            }

            AssertRect(expectedLink.Rect, annotationLink.Rect);
            Assert.AreEqual(expectedLink.Type, annotationLink.Type);
            AssertNamedDestination(expectedLink.NamedDestination, annotationLink.NamedDestination);
            Assert.AreEqual(expectedLink.IsPrintable, annotationLink.IsPrintable);
            AssertDestination(expectedLink.Destination, annotationLink.Destination);
            AssertAnnotationBorder(expectedLink.Border, annotationLink.Border);
            AssertAction(expectedLink.Action, annotationLink.Action);
        }

        private static void AssertRect(Rect expected, Rect rect)
        {
            double epsilon = 0.00001;

            Assert.IsTrue(expected.Left.IsEqualTo(rect.Left, epsilon));
            Assert.IsTrue(expected.Top.IsEqualTo(rect.Top, epsilon));
            Assert.IsTrue(expected.Right.IsEqualTo(rect.Right, epsilon));
            Assert.IsTrue(expected.Bottom.IsEqualTo(rect.Bottom, epsilon));
        }

        private static void AssertAction(Fixed.Model.Actions.Action expected, Fixed.Model.Actions.Action action)
        {
            if (expected == null && action == null)
            {
                return;
            }

            if (expected == null || action == null)
            {
                Assert.Fail();
            }

            Assert.AreEqual(expected.ActionType, action.ActionType);

            if (expected.ActionType == ActionType.GoTo)
            {
                AssertGoToAction(expected as GoToAction, action as GoToAction);
            }
            else
            {
                AssertUriAction(expected as UriAction, action as UriAction);
            }
        }

        private static void AssertUriAction(UriAction expected, UriAction action)
        {
            if (expected == null && action == null)
            {
                return;
            }

            if (expected == null || action == null)
            {
                Assert.Fail();
            }

            Assert.AreEqual(expected.IncludeMouseCoordinates, action.IncludeMouseCoordinates);
            Assert.AreEqual(expected.Uri, action.Uri);
        }

        private static void AssertGoToAction(GoToAction expected, GoToAction action)
        {
            if (expected == null && action == null)
            {
                return;
            }

            if (expected == null || action == null)
            {
                Assert.Fail();
            }

            AssertDestination(expected.Destination, action.Destination);
            AssertNamedDestination(expected.NamedDestination, action.NamedDestination);
        }

        private static void AssertNamedDestination(NamedDestination expected, NamedDestination namedDestination)
        {
            if (expected == null && namedDestination == null)
            {
                return;
            }

            if (expected == null || namedDestination == null)
            {
                Assert.Fail();
            }

            Assert.AreEqual(expected.Name, namedDestination.Name);
            AssertDestination(expected.Destination, namedDestination.Destination);
            AssertGoToAction(expected.GoToAction, namedDestination.GoToAction);
        }

        private static void AssertAnnotationBorder(AnnotationBorder expected, AnnotationBorder border)
        {
            // TODO: Failing tests if uncomment
            //if (expected == null && border == null)
            //{
            //    return;
            //}

            //if (expected == null || border == null)
            //{
            //    Assert.Fail();
            //}

            //AssertDashArray(expected.DashArray, border.DashArray);
            //Assert.AreEqual(expected.Style, border.Style);
            //Assert.AreEqual(expected.Width, border.Width);
        }

        //private static void AssertDashArray(IEnumerable<double> expected, IEnumerable<double> dashArray)
        //{
        //    // TODO: Failing tests if uncomment
        //    if (expected == null && dashArray == null)
        //    {
        //        return;
        //    }

        //    if (expected == null || dashArray == null)
        //    {
        //        Assert.Fail();
        //    }

        //    Assert.IsTrue(expected.SequenceEqual(dashArray));
        //}

        private static void AssertDestination(Destination expected, Destination destination)
        {
            if (expected == null && destination == null)
            {
                return;
            }

            if (expected == null || destination == null)
            {
                Assert.Fail();
            }

            Assert.AreEqual(expected.DestinationType, destination.DestinationType);
            // TODO: implement assertions for all kind of destniations
        }

        private static void AssertAcroForm(AcroForm expected, AcroForm acroForm)
        {
            Assert.AreEqual(expected.ViewersShouldRecalculateWidgetAppearances, acroForm.ViewersShouldRecalculateWidgetAppearances);
            Assert.AreEqual(expected.FormFields.Count, acroForm.FormFields.Count);

            foreach (FormField field in acroForm.FormFields)
            {
                Assert.IsTrue(expected.FormFields.Contains(field.Name));
                FormField expectedField = expected.FormFields[field.Name];
                AssertField(expectedField, field);
            }
        }

        private static void AssertField(FormField expected, FormField field)
        {
            Assert.AreEqual(expected.FieldType, field.FieldType);
            Assert.AreEqual(expected.Name, field.Name);
            Assert.AreEqual(expected.UserInterfaceName, field.UserInterfaceName);
            Assert.AreEqual(expected.MappingName, field.MappingName);
            Assert.AreEqual(expected.IsReadOnly, field.IsReadOnly);
            Assert.AreEqual(expected.IsRequired, field.IsRequired);
            Assert.AreEqual(expected.ShouldBeSkipped, field.ShouldBeSkipped);
            AssertTextProperties(expected.TextProperties, field.TextProperties);

            switch (field.FieldType)
            {
                case FormFieldType.CheckBox:
                    AssertCheckBoxField((CheckBoxField)expected, (CheckBoxField)field);
                    break;
                case FormFieldType.ComboBox:
                    AssertComboBoxField((ComboBoxField)expected, (ComboBoxField)field);
                    break;
                case FormFieldType.CombTextBox:
                    AssertCombTextBoxField((CombTextBoxField)expected, (CombTextBoxField)field);
                    break;
                case FormFieldType.ListBox:
                    AssertListBoxField((ListBoxField)expected, (ListBoxField)field);
                    break;
                case FormFieldType.PushButton:
                    AssertPushButtonField((PushButtonField)expected, (PushButtonField)field);
                    break;
                case FormFieldType.RadioButton:
                    AssertRadioButtonField((RadioButtonField)expected, (RadioButtonField)field);
                    break;
                case FormFieldType.Signature:
                    AssertSignatureField((SignatureField)expected, (SignatureField)field);
                    break;
                case FormFieldType.TextBox:
                    AssertTextBoxField((TextBoxField)expected, (TextBoxField)field);
                    break;
            }
        }

        private static void AssertCheckBoxField(CheckBoxField expected, CheckBoxField field)
        {
            Assert.AreEqual(expected.IsChecked, field.IsChecked);
            Assert.AreEqual(expected.IsCheckedByDefault, field.IsCheckedByDefault);
            Assert.AreEqual(expected.ExportValue, field.ExportValue);
        }

        private static void AssertComboBoxField(ComboBoxField expected, ComboBoxField field)
        {
            AssertChoiceField(expected, field);
            Assert.AreEqual(expected.HasEditableTextBox, field.HasEditableTextBox);
            Assert.AreEqual(expected.ShouldSpellCheck, field.ShouldSpellCheck);
            AssertChoiceOption(expected.Value, field.Value);
            AssertChoiceOption(expected.DefaultValue, field.DefaultValue);
        }

        private static void AssertCombTextBoxField(CombTextBoxField expected, CombTextBoxField field)
        {
            AssertTextField(expected, field);
            Assert.AreEqual(expected.MaxLengthOfInputCharacters, field.MaxLengthOfInputCharacters);
        }

        private static void AssertListBoxField(ListBoxField expected, ListBoxField field)
        {
            AssertChoiceField(expected, field);
            Assert.AreEqual(expected.AllowMultiSelection, field.AllowMultiSelection);
            Assert.AreEqual(expected.TopIndex, field.TopIndex);
            AssertChoiceOptions(expected.Value, field.Value);
            AssertChoiceOptions(expected.DefaultValue, field.DefaultValue);
        }

        private static void AssertPushButtonField(PushButtonField expected, PushButtonField field)
        {
            // Nothing to assert
        }

        private static void AssertRadioButtonField(RadioButtonField expected, RadioButtonField field)
        {
            AssertRadioOption(expected.Value, field.Value);
            AssertRadioOption(expected.DefaultValue, field.DefaultValue);
            Assert.AreEqual(expected.AllowToggleOff, field.AllowToggleOff);
            Assert.AreEqual(expected.ShouldUpdateRadiosInUnison, field.ShouldUpdateRadiosInUnison);

            RadioButtonWidget[] expectedWidgets = expected.Widgets.ToArray();
            RadioButtonWidget[] fieldWidgets = field.Widgets.ToArray();
            Assert.AreEqual(expectedWidgets.Length, fieldWidgets.Length);

            for (int i = 0; i < expectedWidgets.Length; i++)
            {
                RadioButtonWidget expectedWidget = expectedWidgets[i];
                RadioButtonWidget fieldWidget = fieldWidgets[i];
                Assert.IsNotNull(expectedWidget);
                Assert.IsNotNull(fieldWidget);
                AssertRadioOption(expectedWidget.Option, fieldWidget.Option);
            }

            HashSet<string> expectedOptions = new HashSet<string>();
            HashSet<string> fieldOptions = new HashSet<string>();

            foreach (RadioOption option in expected.Options)
            {
                Assert.IsNotNull(option);
                expectedOptions.Add(option.Value);
            }

            foreach (RadioOption option in field.Options)
            {
                Assert.IsNotNull(option);
                fieldOptions.Add(option.Value);
                bool isExpected = expectedOptions.Contains(option.Value);
                Assert.IsTrue(isExpected);
            }

            Assert.AreEqual(expectedOptions.Count, fieldOptions.Count);
        }

        private static void AssertSignatureField(SignatureField expected, SignatureField field)
        {
#if !NETSTANDARD
#if WPF
            if (expected.Signature == null)
            {
                Assert.IsNull(field.Signature);
            }
            else
            {
                Assert.IsNotNull(field.Signature);
            }
#endif
#endif
        }

        private static void AssertTextBoxField(TextBoxField expected, TextBoxField field)
        {
            AssertTextField(expected, field);
            Assert.AreEqual(expected.MaxLengthOfInputCharacters, field.MaxLengthOfInputCharacters);
            Assert.AreEqual(expected.IsMultiline, field.IsMultiline);
            Assert.AreEqual(expected.IsPassword, field.IsPassword);
            Assert.AreEqual(expected.IsFileSelect, field.IsFileSelect);
            Assert.AreEqual(expected.ShouldSpellCheck, field.ShouldSpellCheck);
            Assert.AreEqual(expected.AllowScroll, field.AllowScroll);
        }

        private static void AssertChoiceField(ChoiceField expected, ChoiceField field)
        {
            Assert.AreEqual(expected.ShouldCommitOnSelectionChange, field.ShouldCommitOnSelectionChange);
            Assert.AreEqual(expected.Options.Count, field.Options.Count);

            for (int i = 0; i < expected.Options.Count; i++)
            {
                ChoiceOption expectedChoice = expected.Options[i];
                ChoiceOption fieldChoice = field.Options[i];
                Assert.IsNotNull(expectedChoice);
                Assert.IsNotNull(fieldChoice);
                AssertChoiceOption(expectedChoice, fieldChoice);
            }
        }

        private static void AssertTextField(TextField expected, TextField field)
        {
            Assert.AreEqual(expected.Value, field.Value);
            Assert.AreEqual(expected.DefaultValue, field.DefaultValue);
        }

        private static void AssertChoiceOptions(ChoiceOption[] expected, ChoiceOption[] options)
        {
            if (expected == null)
            {
                Assert.IsNull(options);
            }
            else
            {
                Assert.AreEqual(expected.Length, options.Length);

                for (int i = 0; i < expected.Length; i++)
                {
                    ChoiceOption expectedChoice = expected[i];
                    ChoiceOption optionsChoice = options[i];
                    AssertChoiceOption(expectedChoice, optionsChoice);
                }
            }
        }

        private static void AssertChoiceOption(ChoiceOption expected, ChoiceOption option)
        {
            if (expected == null)
            {
                Assert.IsNull(option);
            }
            else
            {
                Assert.AreEqual(expected.Value, option.Value);
                Assert.AreEqual(expected.UserInterfaceValue, option.UserInterfaceValue);
            }
        }

        private static void AssertRadioOption(RadioOption expected, RadioOption option)
        {
            if (expected == null)
            {
                Assert.IsNull(option);
            }
            else
            {
                Assert.IsNotNull(option);
                Assert.AreEqual(expected.Value, option.Value);
            }
        }

        public static void AssertPage(RadFixedPage expected, RadFixedPage page, bool isOldFormatProviderImport)
        {
            AssertAreEqual(expected.MediaBox, page.MediaBox);
            AssertAreEqual(expected.CropBox, page.CropBox);

            AssertContentElementCollection(expected.Content, page.Content);
            AssertAnnotations(expected.Annotations, page.Annotations, isOldFormatProviderImport);
        }

        public static void AssertContentElementCollection(ContentElementCollection expected, ContentElementCollection content)
        {
            int notVisiblePathsCount = CalculateNotVisiblePathsCount(expected);
            Assert.AreEqual(expected.Count - notVisiblePathsCount, content.Count);

            int contentElementIndex = 0;
            for (int i = 0; i < expected.Count; i++)
            {
                if (!IsInvisiblePath(expected[i]))
                {
                    AssertContentElement(expected[i], content[contentElementIndex]);
                    contentElementIndex++;
                }
            }
        }

        public static void AssertAnnotations(AnnotationCollection expected, AnnotationCollection annotations, bool isOldFormatProviderImport)
        {
            Assert.AreEqual(expected.Count, annotations.Count);
            Action<Annotation, Annotation> assertAnnotations = GetAssertAnnotationsMethod(isOldFormatProviderImport);

            for (int i = 0; i < expected.Count; i++)
            {
                assertAnnotations(expected[i], annotations[i]);
            }
        }

        public static void AssertContentElement(ContentElementBase expected, ContentElementBase contentElement)
        {
            Assert.AreEqual(expected.ElementType, contentElement.ElementType);

            AssertClipping(expected.Clipping, contentElement.Clipping);

            switch (expected.ElementType)
            {
                case FixedDocumentElementType.TextFragment:
                    AssertTextFragment((TextFragment)expected, (TextFragment)contentElement);
                    break;

                case FixedDocumentElementType.Path:
                    AssertPath((Path)expected, (Path)contentElement);
                    break;

                case FixedDocumentElementType.Image:
                    AssertImage((Image)expected, (Image)contentElement);
                    break;
                case FixedDocumentElementType.Form:
                    AssertForm((Form)expected, (Form)contentElement);
                    break;
                default:
                    throw new NotSupportedException(string.Format("Not supported element type: {0}", expected.ElementType));
            }
        }

        public static void AssertForm(Form expected, Form form)
        {
            if (expected.FormSource == null)
            {
                Assert.IsNull(form.FormSource);
            }
            else
            {
                Matrix expectedMatrix = new Matrix();
                double widthScale = expected.Width / expected.FormSource.Size.Width;
                double heightScale = expected.Height / expected.FormSource.Size.Height;
                expectedMatrix = expectedMatrix.ScaleMatrix(widthScale, heightScale);
                expectedMatrix = expectedMatrix.MultiplyBy(expected.Position.Matrix);
                AssertAreEqual(expectedMatrix, form.Position.Matrix);
                AssertMarker(expected.Marker, form.Marker);

                AssertFormSource(expected.FormSource, form.FormSource);
            }
        }

        public static void AssertImage(Image expected, Image image)
        {
            if (expected.ImageSource == null)
            {
                Assert.IsNull(image.ImageSource);
            }
            else
            {
                Matrix expectedMatrix = new Matrix();
                double widthScale = expected.Width / expected.ImageSource.Width;
                double heightScale = expected.Height / expected.ImageSource.Height;
                expectedMatrix = expectedMatrix.ScaleMatrix(widthScale, heightScale);
                expectedMatrix = expectedMatrix.MultiplyBy(expected.Position.Matrix);
                AssertAreEqual(expectedMatrix, image.Position.Matrix);
                AssertMarker(expected.Marker, image.Marker);
                Assert.AreEqual(expected.StencilColor, image.StencilColor);

                AssertImageSource(expected.ImageSource, image.ImageSource);
            }
        }

        public static void AssertFormSource(FormSource expected, FormSource formSource)
        {
            if (expected == null)
            {
                Assert.IsNull(formSource);
            }
            else
            {
                AssertAreEqual(expected.Size, formSource.Size);
                AssertAreEqual(expected.BoundingBox, formSource.BoundingBox);
                AssertContentElementCollection(expected.Content, formSource.Content);
            }
        }

        public static void AssertImageSource(ImageSource expected, ImageSource imageSource)
        {
            Assert.IsNotNull(imageSource);
            AssertColorspace(expected.ColorSpace.Value, imageSource.ColorSpace.Value);
            Assert.AreEqual(expected.BitsPerComponent, imageSource.BitsPerComponent);
            Assert.AreEqual(expected.Width, imageSource.Width);
            Assert.AreEqual(expected.Height, imageSource.Height);

            byte[] expectedData = expected.GetEncodedImageData(FixedDocumentDefaults.ImageQuality);
            byte[] actualData = imageSource.GetEncodedImageData(FixedDocumentDefaults.ImageQuality);
            Assert.AreEqual(expectedData.Length, actualData.Length);

            if (expected.DecodeArray == null)
            {
                Assert.IsNull(imageSource.DecodeArray);
            }
            else
            {
                Assert.IsNotNull(imageSource.DecodeArray);
                Assert.AreEqual(expected.DecodeArray.Length, imageSource.DecodeArray.Length);
                for (int i = 0; i < expected.DecodeArray.Length; i++)
                {
                    Assert.AreEqual(expected.DecodeArray[i], imageSource.DecodeArray[i]);
                }
            }
        }

        public static void AssertPosition(IPosition expected, IPosition position)
        {
            Assert.IsTrue(TestUtils.EqualMatrices(expected.Matrix, position.Matrix));
        }

        public static void AssertAreEqual(Point point1, Point point2)
        {
            AssertAreEqual(point1.X, point2.X);
            AssertAreEqual(point1.Y, point2.Y);
        }

        public static void AssertAreEqual(Size size1, Size size2)
        {
            AssertAreEqual(size1.Width, size2.Width);
            AssertAreEqual(size1.Height, size2.Height);
        }

        public static void AssertAreEqual(Rect rect1, Rect rect2)
        {
            AssertAreEqual(rect1.X, rect2.X);
            AssertAreEqual(rect1.Y, rect2.Y);
            AssertAreEqual(rect1.Width, rect2.Width);
            AssertAreEqual(rect1.Height, rect2.Height);
        }

        public static void AssertClipping(Clipping expected, Clipping clipping)
        {
            if (expected != null)
            {
                Assert.IsNotNull(clipping);

                AssertGeometry(expected.Clip, clipping.Clip);
                AssertClipping(expected.Clipping, clipping.Clipping);
            }
            else
            {
                Assert.IsNull(clipping);
            }
        }

        public static void AssertGeometry(GeometryBase expected, GeometryBase geometry)
        {
            RectangleGeometry expectedRectangleGeometry = expected as RectangleGeometry;
            if (expectedRectangleGeometry != null)
            {
                if (geometry is RectangleGeometry)
                {
                    AssertRectangleGeometry(expectedRectangleGeometry, geometry as RectangleGeometry);
                }
                else if (geometry is PathGeometry)
                {
                    PathGeometry resultPathGeometry = CreatePathGeometryFromRectangleGeometry(expectedRectangleGeometry);
                    AssertPathGeometry(resultPathGeometry, geometry as PathGeometry);
                }
                else
                {
                    Assert.Fail("A rectangle can be represented only by rectangle or path geometry.");
                }

                return;
            }

            PathGeometry expectedPathGeometry = expected as PathGeometry;
            if (expectedPathGeometry != null)
            {
                AssertPathGeometry(expectedPathGeometry, geometry as PathGeometry);
                return;
            }
        }

        public static void AssertTextFragment(TextFragment expected, TextFragment textFragment)
        {
            Assert.AreEqual(expected.Text, textFragment.Text);
            AssertPosition(expected.Position, textFragment.Position);
            AssertMarker(expected.Marker, textFragment.Marker);
            AssertTextProperties(expected.TextProperties, textFragment.TextProperties);
        }

        public static void AssertTextProperties(VariableTextProperties expected, VariableTextProperties textProperties)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(textProperties);
            Assert.AreEqual(expected.HorizontalAlignment, textProperties.HorizontalAlignment);
            AssertTextProperties(expected.PropertiesOwner, textProperties.PropertiesOwner);
        }

        public static void AssertTextProperties(TextPropertiesOwner expected, TextPropertiesOwner textProperties)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(textProperties);
            Assert.AreEqual(expected.CharacterSpacing, textProperties.CharacterSpacing);
            Assert.AreEqual(expected.Fill, textProperties.Fill);
            Assert.AreEqual(expected.HorizontalScaling, textProperties.HorizontalScaling);
            Assert.AreEqual(expected.RenderingMode, textProperties.RenderingMode);
            Assert.AreEqual(expected.Stroke, textProperties.Stroke);
            Assert.AreEqual(expected.TextRise, textProperties.TextRise);
            Assert.AreEqual(expected.WordSpacing, textProperties.WordSpacing);
            AssertAreEqual(expected.FontSize, textProperties.FontSize);
            AssertFont(expected.Font, textProperties.Font);
        }

        public static void AssertFont(FontBase expected, FontBase font)
        {
            bool expectedIsCidFont = expected is CidType2Font;
            bool fontIsCidFont = font is CidType2Font;
            Assert.AreEqual(expectedIsCidFont, fontIsCidFont);

            if (expectedIsCidFont)
            {
                int expectedPrefixEndIndex = expected.FontName.IndexOf('+');
                int prefixEndIndex = font.FontName.IndexOf('+');
                bool expectedHasPrefix = expectedPrefixEndIndex > -1;
                bool hasPrefix = prefixEndIndex > -1;
                Assert.AreEqual(expectedHasPrefix, hasPrefix);

                string expectedNameWithoutPrefix = expectedHasPrefix ? expected.FontName.Substring(expectedPrefixEndIndex + 1) : expected.FontName;
                string nameWithoutPrefix = hasPrefix ? font.FontName.Substring(prefixEndIndex + 1) : font.FontName;
                Assert.AreEqual(expectedNameWithoutPrefix, nameWithoutPrefix);
            }
            else
            {
                Assert.AreEqual(expected.FontName, font.FontName);
            }

            Assert.AreEqual(expected.Height, font.Height);
            Assert.AreEqual(expected.Type, font.Type);
        }

        public static void AssertAreEqual(double a, double b)
        {
            Assert.IsTrue(a.IsEqualTo(b, Epsilon));
        }

        public static void AssertPath(Path expected, Path path)
        {
            AssertPosition(expected.Position, path.Position);
            AssertMarker(expected.Marker, path.Marker);
            Assert.AreEqual(expected.IsFilled, path.IsFilled);
            Assert.AreEqual(expected.IsStroked, path.IsStroked);

            if (expected.IsFilled)
            {
                AssertColor(expected.Fill, path.Fill);
            }

            if (expected.IsStroked)
            {
                AssertColor(expected.Stroke, path.Stroke);
                Assert.AreEqual(expected.StrokeLineCap, path.StrokeLineCap);
                Assert.AreEqual(expected.StrokeLineJoin, path.StrokeLineJoin);
                AssertAreEqual(expected.StrokeThickness, path.StrokeThickness);
                Assert.AreEqual(expected.StrokeDashOffset, path.StrokeDashOffset);
                Assert.IsTrue(TestUtils.EqualValues(expected.MiterLimit, path.MiterLimit, 0.01));

                if (expected.StrokeDashArray != null)
                {
                    Assert.IsNotNull(path.StrokeDashArray);

                    Assert.AreEqual(expected.StrokeDashArray.Count(), path.StrokeDashArray.Count());
                    List<double> expectedDashList = expected.StrokeDashArray.ToList();
                    List<double> pathDashList = path.StrokeDashArray.ToList();

                    for (int i = 0; i < expectedDashList.Count; i++)
                    {
                        Assert.AreEqual(expectedDashList[i], pathDashList[i]);
                    }
                }
                else
                {
                    Assert.IsNull(path.StrokeDashArray);
                }
            }


            AssertGeometry(expected.Geometry, path.Geometry);
        }

        public static bool AssertNotNull<T>(T a, T b)
        {
            if (a != null)
            {
                Assert.IsNotNull(b);
                return true;
            }
            else
            {
                Assert.IsNull(b);
                return false;
            }
        }

        private static void AssertMarker(Marker expected, Marker marker)
        {
            if (expected == null)
            {
                Assert.IsNull(marker);
            }
            else
            {
                Assert.IsNotNull(marker);
                Assert.AreEqual(expected.Name, marker.Name);
            }
        }

        private static PathGeometry CreatePathGeometryFromRectangleGeometry(RectangleGeometry oldGeometry)
        {
            PathFigure newFigure = new PathFigure()
            {
                IsClosed = true,
                StartPoint = oldGeometry.Rect.TopLeft
            };
            newFigure.Segments.Add(new LineSegment() { Point = oldGeometry.Rect.TopRight });
            newFigure.Segments.Add(new LineSegment() { Point = oldGeometry.Rect.BottomRight });
            newFigure.Segments.Add(new LineSegment() { Point = oldGeometry.Rect.BottomLeft });

            PathGeometry newGeometry = new PathGeometry();
            newGeometry.Figures.Add(newFigure);

            return newGeometry;
        }

        private static void AssertAreEqual(Matrix matrix1, Matrix matrix2)
        {
            AssertAreEqual(matrix1.M11, matrix2.M11);
            AssertAreEqual(matrix1.M12, matrix2.M12);
            AssertAreEqual(matrix1.M21, matrix2.M21);
            AssertAreEqual(matrix1.M22, matrix2.M22);
            AssertAreEqual(matrix1.OffsetX, matrix2.OffsetX);
            AssertAreEqual(matrix1.OffsetY, matrix2.OffsetY);
        }

        private static void AssertColor(ColorBase expected, ColorBase color)
        {
            if (AssertNotNull(expected, color))
            {
                AssertColorspace(expected.ColorSpace, color.ColorSpace);

                if (expected.ColorSpace.Name == ColorSpaceNames.DeviceRgb)
                {
                    AssertDeviceRgbColor(expected as RgbColor, color as RgbColor);
                }

                else if (expected.ColorSpace.Name == ColorSpaceNames.Pattern)
                {
                    AssertPatternColor(expected as PatternColor, color as PatternColor);
                }
            }
        }

        private static void AssertColorspace(ColorSpaceBase expected, ColorSpaceBase colorspace)
        {
            if (expected == null)
            {
                Assert.IsNull(colorspace);
            }
            else
            {
                Assert.IsNotNull(colorspace);
                Assert.AreEqual(expected.Name, colorspace.Name);
            }
        }

        private static void AssertPatternColor(PatternColor expected, PatternColor color)
        {
            if (AssertNotNull(expected, color))
            {
                Assert.AreEqual(expected.PatternType, color.PatternType);

                switch (expected.PatternType)
                {
                    case PatternType.Gradient:
                        AssertGradientColor(expected as Gradient, color as Gradient);
                        break;
                }
            }
        }

        private static void AssertGradientColor(Gradient expected, Gradient color)
        {
            if (AssertNotNull(expected, color))
            {
                Assert.AreEqual(expected.GradientType, color.GradientType);
                AssertColor(expected.Background, color.Background);
                Assert.AreEqual(expected.StartPoint, color.StartPoint);
                Assert.AreEqual(expected.EndPoint, color.EndPoint);
                Assert.AreEqual(expected.ExtendAfter, color.ExtendAfter);
                Assert.AreEqual(expected.ExtendBefore, color.ExtendBefore);


                switch (expected.GradientType)
                {
                    case GradientType.Linear:
                        AssertLinearGradientColor(expected as LinearGradient, color as LinearGradient);
                        break;

                    case GradientType.Radial:
                        AssertRadialGradientColor(expected as RadialGradient, color as RadialGradient);
                        break;
                }
            }
        }

        private static void AssertRadialGradientColor(RadialGradient expected, RadialGradient color)
        {
            if (AssertNotNull(expected, color))
            {
                Assert.AreEqual(expected.StartRadius, color.StartRadius);
                Assert.AreEqual(expected.EndRadius, color.EndRadius);
            }
        }

        private static void AssertLinearGradientColor(LinearGradient expected, LinearGradient color)
        {
            AssertNotNull(expected, color);
        }

        private static void AssertDeviceRgbColor(RgbColor expected, RgbColor color)
        {
            if (AssertNotNull(expected, color))
            {
                Assert.IsTrue(TestUtils.EqualColorComponents(expected.A, color.A));
                Assert.IsTrue(TestUtils.EqualColorComponents(expected.R, color.R));
                Assert.IsTrue(TestUtils.EqualColorComponents(expected.B, color.B));
                Assert.IsTrue(TestUtils.EqualColorComponents(expected.G, color.G));
            }
        }

        private static void AssertPathGeometry(PathGeometry expected, PathGeometry pathGeometry)
        {
            if (expected != null)
            {
                Assert.IsNotNull(pathGeometry);
                Assert.AreEqual(expected.Figures.Count, pathGeometry.Figures.Count);
                Assert.AreEqual(expected.FillRule, pathGeometry.FillRule);

                int pathFiguresCount = expected.Figures.Count;
                for (int i = 0; i < pathFiguresCount; i++)
                {
                    AssertPathFigure(expected.Figures[i], pathGeometry.Figures[i]);
                }
            }
            else
            {
                Assert.IsNull(pathGeometry);
            }
        }

        private static void AssertPathFigure(PathFigure expected, PathFigure pathFigure)
        {
            Assert.IsNotNull(pathFigure);
            AssertAreEqual(expected.StartPoint, pathFigure.StartPoint);
            Assert.AreEqual(expected.IsClosed, pathFigure.IsClosed);
            Assert.AreEqual(expected.Segments.Count, pathFigure.Segments.Count);

            int segmentsCount = expected.Segments.Count;
            for (int i = 0; i < segmentsCount; i++)
            {
                AssertSegment(expected.Segments[i], pathFigure.Segments[i]);
            }
        }

        private static void AssertSegment(PathSegment expected, PathSegment pathSegment)
        {
            LineSegment expectedLineSegment = expected as LineSegment;
            if (expectedLineSegment != null)
            {
                AssertLineSegment(expectedLineSegment, pathSegment as LineSegment);
                return;
            }

            BezierSegment expectedBezierSegment = expected as BezierSegment;
            if (expectedBezierSegment != null)
            {
                AssertBezierSegment(expectedBezierSegment, pathSegment as BezierSegment);
                return;
            }
        }

        private static void AssertBezierSegment(BezierSegment expected, BezierSegment bezierSegment)
        {
            Assert.IsNotNull(bezierSegment);

            AssertAreEqual(expected.Point1, bezierSegment.Point1);
            AssertAreEqual(expected.Point2, bezierSegment.Point2);
            AssertAreEqual(expected.Point3, bezierSegment.Point3);
        }

        private static void AssertLineSegment(LineSegment expected, LineSegment lineSegment)
        {
            if (expected != null)
            {
                Assert.IsNotNull(lineSegment);

                AssertAreEqual(expected.Point, lineSegment.Point);
            }
            else
            {
                Assert.IsNull(lineSegment);
            }
        }

        private static void AssertRectangleGeometry(RectangleGeometry expected, RectangleGeometry rectangleGeometry)
        {
            if (expected != null)
            {
                Assert.IsNotNull(rectangleGeometry);

                AssertAreEqual(expected.Rect, rectangleGeometry.Rect);

            }
            else
            {
                Assert.IsNull(rectangleGeometry);
            }
        }

        private static bool IsVisiblePath(Path path)
        {
            return path.IsFilled || path.IsStroked;
        }

        private static bool IsInvisiblePath(ContentElementBase element)
        {
            Path path = element as Path;
            if (path == null)
            {
                return false;
            }

            return !IsVisiblePath(path);
        }

        private static int CalculateNotVisiblePathsCount(ContentElementCollection elements)
        {
            int notVisiblePathsCount = 0;
            foreach (ContentElementBase element in elements)
            {
                Path path = element as Path;
                if (path != null)
                {
                    if (!IsVisiblePath(path))
                    {
                        notVisiblePathsCount++;
                    }
                }
            }

            return notVisiblePathsCount;
        }

        private static Action<Annotation, Annotation> GetAssertAnnotationsMethod(bool isOldFormatProviderImport)
        {
#if NETSTANDARD
            return Fixed.Tests.Integration.IntegrationTestClassBase.AssertAnnotationInternal;
#else
            if (isOldFormatProviderImport)
            {
                return IntegrationOldTestClassBase.AssertAnnotation;
            }
            else
            {
                return Fixed.Tests.Integration.IntegrationTestClassBase.AssertAnnotationInternal;
            }
#endif
        }
    }
}
