﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Annotations
{
    [TestClass]
    public class TestLink : IntegrationTestClassBase
    {
        private void AssertDocumentWithDestination(Destination destination)
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page1 = document.Pages.AddPage();
            RadFixedPage page2 = document.Pages.AddPage();

            destination.Page = page2;
            Fixed.Model.Annotations.Link link = page1.Annotations.AddLink(destination);
            link.Rect = new Rect(30, 30, 60, 80);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithLocationDestination()
        {
            Location destination = new Location()
            {
                Left = 10,
                Top = 35,
                Zoom = 34
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithPageHorizontalFitDestination()
        {
            PageHorizontalFit destination = new PageHorizontalFit()
            {
                Top = 345
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithPageVErticalFitDestination()
        {
            PageVerticalFit destination = new PageVerticalFit()
            {
                Left = 3
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithRectangleFitDestination()
        {
            RectangleFit destination = new RectangleFit()
            {
                Bottom = 43,
                Left = 23,
                Top = 99,
                Right = 3
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithBoundingRectangleFitDestination()
        {
            BoundingRectangleFit destination = new BoundingRectangleFit();

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithBoundingRectangleHorizontalFitDestination()
        {
            BoundingRectangleHorizontalFit destination = new BoundingRectangleHorizontalFit()
            {
                Top = 43
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithBoundingRectangleVerticalFitDestination()
        {
            BoundingRectangleVerticalFit destination = new BoundingRectangleVerticalFit()
            {
                Left = 21
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithUriActionWithoutMouseCoordinates()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            Fixed.Model.Actions.UriAction uriAction = new Fixed.Model.Actions.UriAction()
            {
                Uri = new Uri(@"http://www.telerik.com/")
            };

            Fixed.Model.Annotations.Link annot = page.Annotations.AddLink(uriAction);
            annot.Rect = new Rect(30, 30, 60, 80);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Link_ImportExport_WithUriActionWithMouseCoordinates()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            Fixed.Model.Actions.UriAction uriAction = new Fixed.Model.Actions.UriAction()
            {
                Uri = new Uri(@"http://www.telerik.com/"),
                IncludeMouseCoordinates = true
            };

            Fixed.Model.Annotations.Link annot = page.Annotations.AddLink(uriAction);
            annot.Rect = new Rect(65, 9, 366, 344);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_LinkImportExport_WithGoToAction()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            Fixed.Model.Actions.GoToAction goToAction = new Fixed.Model.Actions.GoToAction()
            {
                Destination = new Location()
                {
                    Page = page,
                    Zoom = 43,
                    Left = 4,
                }
            };

            Fixed.Model.Annotations.Link annot = page.Annotations.AddLink(goToAction);
            annot.Rect = new Rect(0, 100, 5, 5);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_LinkImportExport_WithNamedGoToAction()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            Fixed.Model.Actions.GoToAction goToAction = new Fixed.Model.Actions.GoToAction()
            {
                Destination = new Location()
                {
                    Page = page,
                    Zoom = 43,
                    Left = 4,
                }
            };

            NamedDestination namedDestination = document.NamedDestinations.Add("Test destination", goToAction);

            Fixed.Model.Annotations.Link annot = page.Annotations.AddLink(namedDestination);
            annot.Rect = new Rect(0, 100, 5, 5);

            ExportImportAndAssert(document);
        }
    }
}
