﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Navigation;

namespace Telerik.Windows.Documents.FixedDocumentViewers.Tests.Integration.ImportExport.Annotations
{
    [TestClass]
    public class TestLinkOld : IntegrationOldTestClassBase
    {
        private void AssertDocumentWithDestination(Destination destination)
        {
            RadFixedDocument document = new RadFixedDocument();
            var page1 = document.Pages.AddPage();
            var page2 = document.Pages.AddPage();

            destination.Page = page2;
            var link = page1.Annotations.AddLink(destination);
            link.Rect = new Rect(30, 30, 60, 80);

            ExportImportAndAssert(document);
        }


        [TestMethod]
        public void Test_Link_OldImportExport_WithLocationDestination()
        {
            var destination = new Location()
            {
                Left = 10,
                Top = 35,
                Zoom = 34
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithPageHorizontalFitDestination()
        {
            var destination = new PageHorizontalFit()
            {
                Top = 345
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithPageVErticalFitDestination()
        {
            var destination = new PageVerticalFit()
            {
                Left = 3
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithRectangleFitDestination()
        {
            var destination = new RectangleFit()
            {
                Bottom = 43,
                Left = 23,
                Top = 99,
                Right = 3
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithBoundingRectangleFitDestination()
        {
            var destination = new BoundingRectangleFit();

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithBoundingRectangleHorizontalFitDestination()
        {
            var destination = new BoundingRectangleHorizontalFit()
            {
                Top = 43
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithBoundingRectangleVerticalFitDestination()
        {
            var destination = new BoundingRectangleVerticalFit()
            {
                Left = 21
            };

            this.AssertDocumentWithDestination(destination);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithUriActionWithoutMouseCoordinates()
        {
            RadFixedDocument document = new RadFixedDocument();
            var page = document.Pages.AddPage();

            var uriAction = new Fixed.Model.Actions.UriAction()
            {
                Uri = new Uri(@"http://www.telerik.com/")
            };

            var annot = page.Annotations.AddLink(uriAction);
            annot.Rect = new Rect(30, 30, 60, 80);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithUriActionWithMouseCoordinates()
        {
            RadFixedDocument document = new RadFixedDocument();
            var page = document.Pages.AddPage();

            var uriAction = new Fixed.Model.Actions.UriAction()
            {
                Uri = new Uri(@"http://www.telerik.com/"),
                IncludeMouseCoordinates = true
            };

            var annot = page.Annotations.AddLink(uriAction);
            annot.Rect = new Rect(65, 9, 366, 344);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Link_OldImportExport_WithGoToAction()
        {
            RadFixedDocument document = new RadFixedDocument();
            var page = document.Pages.AddPage();

            var goToAction = new Fixed.Model.Actions.GoToAction()
            {
                Destination = new Location()
                {
                    Page = page,
                    Zoom = 43,
                    Left = 4,
                }
            };

            var annot = page.Annotations.AddLink(goToAction);
            annot.Rect = new Rect(0, 100, 5, 5);

            ExportImportAndAssert(document);
        }
    }
}
