﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
#if NETSTANDARD
using Point = Telerik.Documents.Primitives.Point;
using Rect = Telerik.Documents.Primitives.Rect;
using Size = Telerik.Documents.Primitives.Size;
#else
using Point = System.Windows.Point;
using Rect = System.Windows.Rect;
using Size = System.Windows.Size;
#endif
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class FixedContentEditorTests : FixedContentEditorTestBase
    {
        // Can not use this test because the Calibri font can not be used
#if !NETSTANDARD
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_DrawText()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);

            double offsetX = 10;
            double offsetY = 30;
            editor.Position.Translate(offsetX, offsetY);
            editor.DrawText("Hello");

            RadFixedDocument expected = new RadFixedDocument();
            TextFragment textFragment = expected.Pages.AddPage().Content.AddTextFragment("Hello");
            textFragment.Font = editor.TextProperties.Font;
            textFragment.FontSize = editor.TextProperties.FontSize;
            textFragment.Position.Translate(offsetX, offsetY + (textFragment.Font.Ascent.Value + textFragment.Font.LineGap) * textFragment.FontSize);

            this.ExportAndAssert(expected, document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_DrawImage()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);

            ImageSource imageSource = CreateImageSource();
            Size size = new Size(100, 80);
            editor.Position.Translate(10, 30);
            editor.DrawImage(imageSource, size);

            RadFixedDocument expected = new RadFixedDocument();
            Image image = expected.Pages.AddPage().Content.AddImage(imageSource);
            image.Width = size.Width;
            image.Height = size.Height;
            image.Position.Translate(10, 30);

            this.ExportAndAssert(expected, document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_DrawGeometry()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);

            Point point1 = new Point(10, 80);
            Point point2 = new Point(300, 204);
            editor.Position.Translate(10, 30);
            editor.DrawLine(point1, point2);

            RadFixedDocument expected = new RadFixedDocument();
            Path path = expected.Pages.AddPage().Content.AddPath();
            path.Geometry = CreateLine(point1, point2);
            path.Position.Translate(10, 30);
            path.IsStroked = editor.GraphicProperties.IsStroked;
            path.StrokeThickness = editor.GraphicProperties.StrokeThickness;
            path.Stroke = editor.GraphicProperties.StrokeColor;

            this.ExportAndAssert(expected, document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);

            TestTextFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);

            TestImageFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);

            TestGeometryFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_WithRotationTranslationAndScale()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            ApplyTransformations(editor);
            Block block = CreateBlock(editor);

            TestTextFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_WithRotationTranslationAndScale()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            ApplyTransformations(editor);
            Block block = CreateBlock(editor);

            TestImageFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_WithRotationTranslationAndScale()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            ApplyTransformations(editor);
            Block block = CreateBlock(editor);

            TestGeometryFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_WithRotationTranslationAndScale()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            ApplyTransformations(editor);
            Block block = CreateBlock(editor);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_Centered()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Center);

            TestTextFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_Centered()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Center);

            TestImageFlowsTwoLines(document, editor, block, 250);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_Centered()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Center);

            TestGeometryFlowsTwoLines(document, editor, block, 250);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_Center()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Center);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_Right()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Right);

            TestTextFlowsTwoLines(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_Right()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Right);

            TestImageFlowsTwoLines(document, editor, block, 250);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_Right()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Right);

            TestGeometryFlowsTwoLines(document, editor, block, 250);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_Right()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyAlignment(block, HorizontalAlignment.Right);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_Spacing()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplySpacings(block, top: 0);
            block.SpacingBefore = 10;

            TestTextFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImages_TwoLines_Spacing()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplySpacings(block);

            TestImageFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextGeometry_TwoLines_Spacing()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplySpacings(block);

            TestGeometryFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_Spacing()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplySpacings(block);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_Underline()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyUnderline(block);

            TestTextFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_Underline()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyUnderline(block);

            TestImageFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_Underline()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyUnderline(block);

            TestGeometryFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_Underline()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyUnderline(block);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_Highlights()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyHighlights(block);

            TestTextFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_Highlights()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyHighlights(block);

            TestImageFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_Highlights()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyHighlights(block);

            TestGeometryFlowsTwoLines(document, editor, block, 215);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_Highlights()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyHighlights(block);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_VertCenter()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Center);

            TestTextFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_VertCenter()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Center);

            TestImageFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_VertCenter()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Center);

            TestGeometryFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_VertCenter()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Center);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextFlows_TwoLines_VertBottom()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Bottom);

            TestTextFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ImageFlows_TwoLines_VertBottom()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Bottom);

            TestImageFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_GeometryFlows_TwoLines_VertBottom()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Bottom);

            TestGeometryFlowsTwoLines(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextImageAndGeometry_OnSameLine_VertBottom()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            ApplyVerticalAlignment(block, VerticalAlignment.Bottom);

            TestTextImageAndGeometryOnSameLine(document, editor, block, 600, 400);
        }
#endif

        [TestMethod]
        public void Test_MarkedContent_SimpleScenario()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = new FixedContentEditor(page);
            
            editor.CurrentMarker = new Marker("CustomMarker");
            editor.DrawText("Marked with first marker.");

            Assert.AreEqual(1, page.Content.Count);
            TextFragment text = (TextFragment)page.Content[0];
            Assert.IsNotNull(text.Marker);
            Assert.AreEqual("CustomMarker", text.Marker.Name);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_MarkedContent_MarkedAndUnmarkedContent()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = new FixedContentEditor(page);

            editor.DrawText("Not marked.");

            editor.CurrentMarker = new Marker("FirstMarker");
            editor.DrawText("Marked with first marker.");
            editor.CurrentMarker = null;

            editor.DrawText("Not marked after first marker.");

            Assert.AreEqual(3, page.Content.Count);

            foreach (TextFragment element in page.Content)
            {
                bool shouldBeMarked = (element.Text == "Marked with first marker.");
                
                if(shouldBeMarked)
                {
                    Assert.IsNotNull(element.Marker);
                    Assert.AreEqual("FirstMarker", element.Marker.Name);
                }
                else
                {
                    Assert.IsNull(element.Marker);
                }
            }

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_MarkedContent_AllContentTypes()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = new FixedContentEditor(page);
            
            editor.CurrentMarker = new Marker("TextMarker");
            editor.DrawText("First text.");
            editor.DrawText("Second text.");
            editor.DrawText("ThirdText.");

            editor.CurrentMarker = new Marker("GeometryMarker");
            editor.DrawRectangle(new Rect(10, 20, 30, 40));
            editor.DrawCircle(new Point(100, 100), 50);
            editor.DrawEllipse(new Point(200, 300), 100, 60);

            editor.CurrentMarker = new Marker("ImageMarker");
            ImageSource imageSource = CreateImageSource();
            editor.DrawImage(imageSource, new Size(100, 200));
            editor.Position.Translate(400, 500);
            editor.DrawImage(imageSource, new Size(50, 50));

            editor.CurrentMarker = new Marker("FormMarker");
            FormSource formSource = CreateFormSource();
            editor.DrawForm(formSource, new Size(100, 200));
            editor.Position.Translate(400, 500);
            editor.DrawForm(formSource, new Size(50, 50));

            editor.Position.Translate(500, 200);
            editor.CurrentMarker = new Marker("AllElementsMarker");
            editor.DrawText("Text element.");
            editor.DrawRectangle(new Rect(10, 20, 30, 40));
            editor.DrawImage(imageSource, new Size(78, 78));
            editor.DrawForm(formSource, new Size(108, 108));

            Assert.AreEqual(14, page.Content.Count);
            int index = 0;

            foreach (PositionContentElement element in page.Content)
            {
                string expectedMarkerName;

                if(index < 3)
                {
                    expectedMarkerName = "TextMarker";
                }
                else if(index < 6)
                {
                    expectedMarkerName = "GeometryMarker";
                }
                else if (index < 8)
                {
                    expectedMarkerName = "ImageMarker";
                }
                else if (index < 10)
                {
                    expectedMarkerName = "FormMarker";
                }
                else
                {
                    expectedMarkerName = "AllElementsMarker";
                }

                Assert.IsNotNull(element.Marker);
                Assert.AreEqual(expectedMarkerName, element.Marker.Name);
                index++;
            }

            ExportImportAndAssert(document);
        }

        private void TestTextImageAndGeometryOnSameLine(RadFixedDocument document, FixedContentEditor editor, Block block, double blockWidth = 200, double blockHeight = double.MaxValue)
        {
            ImageSource imageSource = CreateImageSource();
            Size textSize = MeasureText(editor, "Text");
            double imageWidth = 100;
            double imageHeight = 50;
            double strokeThickness = editor.GraphicProperties.IsStroked ? editor.GraphicProperties.StrokeThickness : 0;
            Rect rect = new Rect(0, 0, 50, 80);

            block.InsertText("Text");
            block.InsertImage(imageSource, imageWidth, imageHeight);
            block.InsertRectangle(rect);

            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double descent = block.TextProperties.UnderlinePattern == UnderlinePattern.None ? Math.Abs(block.TextProperties.Font.Descent.Value) * block.TextProperties.FontSize : 0;
            double lineHeight = Math.Max(GetLineHeight(block), GetLineHeight(block, Math.Max(rect.Height + strokeThickness, imageHeight), false));
            lineHeight += descent;
            double offsetX = CalculateLeftOffset(block, blockWidth, textSize.Width + imageWidth + rect.Width + strokeThickness, 0);
            double offsetY = CalculateTopOffset(block, lineHeight, blockHeight);

            Rect rect1 = new Rect(offsetX, offsetY, MeasureText(editor, "Text").Width, lineHeight);
            Rect rect2 = new Rect(offsetX + rect1.Width, offsetY, imageWidth, lineHeight);
            Rect rect3 = new Rect(offsetX + rect1.Width + rect2.Width, offsetY, rect.Width + strokeThickness, lineHeight);


            RadFixedDocument expected = CreateExpectedDocument(
                CreateElement(editor, block, CreateTextFragment(editor, block, "Text", rect1.X, rect1.Y + lineHeight - textSize.Height), rect1),
                CreateElement(editor, block, CreateImage(editor, block, imageSource, imageWidth, imageHeight, offsetX + textSize.Width, rect2.Y + lineHeight - GetHeight(block, imageHeight, true)), rect2),
                CreateElement(editor, block, CreatePath(editor, block, new RectangleGeometry(rect), offsetX + textSize.Width + imageWidth, rect3.Y + lineHeight - GetHeight(block, rect.Height + strokeThickness, true)), rect3));

            ExportAndAssert(expected, document);
        }

        private void TestGeometryFlowsTwoLines(RadFixedDocument document, FixedContentEditor editor, Block block, double blockWidth = 200, double blockHeight = double.MaxValue)
        {
            Point linePoint1 = new Point(-100, -10);
            Point linePoint2 = new Point(50, 40);
            Rect rect = new Rect(0, 0, 30, 80);
            Point center = new Point(-30, -30);
            double radius = 30;
            double strokeThickness = editor.GraphicProperties.IsStroked ? editor.GraphicProperties.StrokeThickness : 0;

            block.InsertRectangle(rect);
            block.InsertLine(linePoint1, linePoint2);
            block.InsertCircle(center, radius);
            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double lineWidth = Math.Abs(linePoint1.X - linePoint2.X);
            double lineHeight = Math.Abs(linePoint1.Y - linePoint2.Y);
            double line1Height = GetLineHeight(block, Math.Max(lineHeight, rect.Height) + strokeThickness);
            double line2Height = GetLineHeight(block, 2 * radius + strokeThickness);
            double line1OffsetX = CalculateLeftOffset(block, blockWidth, lineWidth + rect.Width + 2 * strokeThickness, 0);
            double line2OffsetX = CalculateLeftOffset(block, blockWidth, radius * 2 + strokeThickness, 1);
            double offsetY = CalculateTopOffset(block, line1Height + line2Height, blockHeight);

            Rect rect1 = new Rect(line1OffsetX, offsetY, rect.Width + strokeThickness, line1Height);
            Rect rect2 = new Rect(rect1.Width + line1OffsetX, offsetY, lineWidth + strokeThickness, line1Height);
            Rect rect3 = new Rect(line2OffsetX, offsetY + line1Height, radius * 2 + strokeThickness, line2Height);

            RadFixedDocument expected = CreateExpectedDocument(
                CreateElement(editor, block, CreatePath(editor, block, new RectangleGeometry(rect), rect1.X, rect1.Y + line1Height - GetHeight(block, rect.Height + strokeThickness)), rect1),
                CreateElement(editor, block, CreatePath(editor, block, CreateLine(linePoint1, linePoint2), rect2.X, rect2.Y + line1Height - GetHeight(block, lineHeight + strokeThickness)), rect2),
                CreateElement(editor, block, CreatePath(editor, block, CreateEllipse(center, radius, radius), line2OffsetX, rect3.Y + line2Height - GetHeight(block, 2 * radius + strokeThickness)), rect3));

            ExportAndAssert(expected, document);
        }

        private void TestImageFlowsTwoLines(RadFixedDocument document, FixedContentEditor editor, Block block, double blockWidth = 200, double blockHeight = double.MaxValue)
        {
            ImageSource imageSource = CreateImageSource();
            double width1 = 150;
            double height1 = 30;
            double width2 = 50;
            double height2 = 50;
            double width3 = 250;
            double height3 = 50;

            block.InsertImage(imageSource, width1, height1);
            block.InsertImage(imageSource, width2, height2);
            block.InsertImage(imageSource, width3, height3);

            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double line1Height = GetLineHeight(block, Math.Max(height1, height2));
            double line2Height = GetLineHeight(block, height2);
            double line1OffsetX = CalculateLeftOffset(block, blockWidth, width1 + width2, 0);
            double line2OffsetX = CalculateLeftOffset(block, blockWidth, width3, 1);
            double offsetY = CalculateTopOffset(block, line1Height + line2Height, blockHeight);

            Rect rect1 = new Rect(line1OffsetX, offsetY, width1, line1Height);
            Rect rect2 = new Rect(rect1.Width + line1OffsetX, offsetY, width2, line1Height);
            Rect rect3 = new Rect(line2OffsetX, offsetY + line1Height, width3, line2Height);

            RadFixedDocument expected = CreateExpectedDocument(
                CreateElement(editor, block, CreateImage(editor, block, imageSource, width1, height1, rect1.X, rect1.Y + rect1.Height - GetHeight(block, height1)), rect1),
                CreateElement(editor, block, CreateImage(editor, block, imageSource, width2, height2, rect2.X, rect2.Y + rect2.Height - GetHeight(block, height2)), rect2),
                CreateElement(editor, block, CreateImage(editor, block, imageSource, width3, height3, rect3.X, rect3.Y + rect3.Height - GetHeight(block, height3)), rect3));

            ExportAndAssert(expected, document);
        }

        private void TestTextFlowsTwoLines(RadFixedDocument document, FixedContentEditor editor, Block block, double blockWidth = 200, double blockHeight = double.PositiveInfinity)
        {
            block.InsertText("Video provides ");
            block.InsertText("a powerful way to help you prove your point.");

            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double lineHeight = GetLineHeight(block);
            double line1OffsetX = CalculateLeftOffset(block, blockWidth, MeasureText(editor, "Video provides a powerful way to").Width, 0);
            double line2OffsetX = CalculateLeftOffset(block, blockWidth, MeasureText(editor, "help you prove your point.").Width, 1);
            double offsetY = CalculateTopOffset(block, 2 * lineHeight, blockHeight);

            Rect rect1 = new Rect(line1OffsetX, offsetY, MeasureText(editor, "Video provides ").Width, lineHeight);
            Rect rect2 = new Rect(rect1.Width + line1OffsetX, offsetY, MeasureText(editor, "a powerful way to").Width, lineHeight);
            Rect rect3 = new Rect(line2OffsetX, offsetY + lineHeight, MeasureText(editor, "help you prove your point.").Width, lineHeight);

            RadFixedDocument expected = CreateExpectedDocument(
                CreateElement(editor, block, CreateTextFragment(editor, block, "Video provides ", rect1.X, rect1.Y), rect1),
                CreateElement(editor, block, CreateTextFragment(editor, block, "a powerful way to ", rect2.X, rect2.Y), rect2),
                CreateElement(editor, block, CreateTextFragment(editor, block, "help you prove your point.", rect3.X, rect3.Y), rect3));

            this.ExportAndAssert(expected, document);
        }

        protected void ApplyVerticalAlignment(Block block, VerticalAlignment alignment)
        {
            block.VerticalAlignment = alignment;
        }

        protected static void ApplyHighlights(Block editor)
        {
            editor.TextProperties.HighlightColor = new RgbColor(0, 255, 255);
        }

        protected static void ApplyUnderline(Block editor)
        {
            editor.TextProperties.UnderlinePattern = UnderlinePattern.Single;
        }

        protected static void ApplySpacings(Block block, double left = 5, double right = 10, double top = 15, double bottom = 20, double lineSpacing = 1, HeightType lineSpacingType = HeightType.Auto)
        {
            block.SpacingBefore = top;
            block.LeftIndent = left;
            block.RightIndent = right;
            block.SpacingAfter = bottom;
            block.LineSpacing = lineSpacing;
            block.LineSpacingType = lineSpacingType;
        }

        protected static void ApplyAlignment(Block block, HorizontalAlignment alignment)
        {
            block.HorizontalAlignment = alignment;
        }

        protected static void ApplyTransformations(FixedContentEditor editor)
        {
            editor.Position = SimplePosition.Default;
            editor.Position.Translate(300, 300);
            editor.Position.Scale(2, 2);
            editor.Position.Rotate(50);
        }
    }
}
