﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.IO;
using System.Windows;
//using System.Windows.Documents;
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class FixedParagraphPropertiesTest : FixedContentEditorTestBase
    {
        private static RadFixedDocument document;

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_AddNewParagraphWithDefaultSettings()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingBeforeAppliedOnFirstParagraphOnly()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Set SpacingBefore
                editor.ParagraphProperties.SpacingBefore = 50;
                editor.InsertLine(TestHelper.SampleParagraph);

                // Set SpacingBefore to default value
                editor.ParagraphProperties.SpacingBefore = FixedDocumentDefaults.SpacingBefore;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingBeforeAppliedOnSecondParagraphOnly()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleParagraph);

                // Set SpacingBefore
                editor.ParagraphProperties.SpacingBefore = 50;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingBeforeIsAppliedToEachParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.SpacingBefore = 60;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.SpacingBefore = FixedDocumentDefaults.SpacingBefore;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingAfterAppliedOnFirstParagrapOnly()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.SpacingAfter = 50;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.SpacingAfter = FixedDocumentDefaults.SpacingAfter;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingAfterAppliedOnSecondParagrapOnly()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.SpacingAfter = 50;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.SpacingAfter = FixedDocumentDefaults.SpacingAfter;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingAfterAppliedToEachParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.SpacingAfter = 50;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLineSpacingOfTypeAutoAppliedToAParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LineSpacingType = HeightType.Auto;
                editor.ParagraphProperties.LineSpacing = 2;
                editor.CharacterProperties.FontSize = 8;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.CharacterProperties.FontSize = 20;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLineSpacingOfTypeAtLeastAppliedToAParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LineSpacingType = HeightType.AtLeast;
                editor.ParagraphProperties.LineSpacing = 30;
                editor.CharacterProperties.FontSize = 8;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.CharacterProperties.FontSize = 18;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLineSpacingOfTypeExactAppliedToAParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LineSpacingType = HeightType.Exact;
                editor.ParagraphProperties.LineSpacing = 12;
                editor.CharacterProperties.FontSize = 10;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.CharacterProperties.FontSize = 20;
                editor.ParagraphProperties.LineSpacing = 28;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndent()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LeftIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndentOnlyToFirstParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LeftIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.LeftIndent = FixedDocumentDefaults.LeftIndent;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndentOnlyToSecondParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.LeftIndent = 80;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndentOnlyToAllParagraphsUntilValueIsChanged()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LeftIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleString.Text);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.LeftIndent = FixedDocumentDefaults.LeftIndent;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestRightIndent()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.RightIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestRightIndentOnlyToFirstParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.RightIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.RightIndent = FixedDocumentDefaults.RightIndent;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestRightIndentOnlyToSecondParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.RightIndent = 80;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestRightIndentOnlyToAllParagraphsUntilValueIsChanged()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.RightIndent = 80;
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.ParagraphProperties.RightIndent = FixedDocumentDefaults.RightIndent;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleString.Text);
                editor.InsertLineBreak();
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndentAndRightIndentSetToAllParagraphs()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LeftIndent = 50;
                editor.ParagraphProperties.RightIndent = 50;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphLeftAlignment()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Left;
                editor.InsertLine(TestHelper.SampleString.Text);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphRightAlignment()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Right;
                editor.InsertLine(TestHelper.TestString);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphCenteredAlignment()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Center;
                editor.InsertLine(TestHelper.TestString);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphsWithDifferentHorizontalAlignments()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Left;
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Right;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Center;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFirstLineIndentAppliedToFirstParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.FirstLineIndent = 20;
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFirstLineIndentAppliedToAllParagraphs()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.FirstLineIndent = 20;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestAllParagraphPropertiesAppliedToAParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.BackgroundColor = TestHelper.Yellow;
                editor.ParagraphProperties.SpacingBefore = 20;
                editor.ParagraphProperties.SpacingAfter = 20;
                editor.ParagraphProperties.LeftIndent = 50;
                editor.ParagraphProperties.RightIndent = 50;
                editor.ParagraphProperties.FirstLineIndent = 40;
                editor.ParagraphProperties.LineSpacing = 2;
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Center;
                editor.CharacterProperties.FontSize = Unit.DipToPoint(12);
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.ParagraphProperties.BackgroundColor = TestHelper.Turqouise;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);

                editor.ParagraphProperties.BackgroundColor = TestHelper.Pink;
                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }

            this.AssertAreVisualyEqual(document);
        }

        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        public void TestAddingChineseTextToAParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.FontSize = Unit.DipToPoint(12);
                editor.InsertLine(TestHelper.SampleLongStringCh);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFontSizeResetToDefaultValue()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.FontSize = Unit.DipToPoint(40);
                editor.InsertLine("FontSize 40");
                editor.InsertParagraph();
                editor.CharacterProperties.FontSize = FixedDocumentDefaults.FontSize;
                editor.InsertLine("Default font size");
            }
            this.AssertAreVisualyEqual(document);
        }

        //TODO: Decide how to check all content of all pages
        // Bug ID: 142883
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphSplitWhenParagraphIsLongerThanOnePage()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine(TestHelper.SampleLongString);
            }
            this.AssertAreVisualyEqual(document);
        }

        // SpacingAfter, LineSpacing and FirstLineIndent applied combined
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestSpacingAfterLineSpacingAndFirstLineIndentAppliedCombined()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.SpacingAfter = 40;
                editor.ParagraphProperties.FirstLineIndent = 30;
                editor.ParagraphProperties.LineSpacing = 2;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        // BackgroundColor, SpacingBefore, LeftIndent and RightIndent applied combined
        // [TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        [Description("Bug: 151145")]
        public void TestBackgroundColorSpacingBeforeLeftIndentAndRightIndentAppliedCombined()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.BackgroundColor = TestHelper.Green;
                editor.ParagraphProperties.SpacingBefore = 40;
                editor.ParagraphProperties.LeftIndent = 30;
                editor.ParagraphProperties.RightIndent = 30;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.TestString);
            }
            this.AssertAreVisualyEqual(document);
        }

        // HorizontalAlignment and LeftIndent applied
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestCenterAlignmentAndLeftIndentApplied()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Center;
                editor.ParagraphProperties.LeftIndent = 60;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        // LeftIndent, RightIndent and LineSpacing applied combined
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestLeftIndentRightIndentAndLineSpacingAppliedCombined()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.ParagraphProperties.LeftIndent = 80;
                editor.ParagraphProperties.RightIndent = 80;
                editor.ParagraphProperties.LineSpacing = 2;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFontSizeSetChangedForFirstWordInParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.FontSize = Unit.PointToDip(24);
                editor.InsertRun("Text");
                editor.CharacterProperties.FontSize = Unit.PointToDip(10);
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFontSizeChangedForWholeParagraph()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.FontSize = Unit.PointToDip(18);
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestFontChangedToFirstWordInParagraph()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertRun("Text in CourierOblique: ");
                editor.CharacterProperties.Font = FontsRepository.CourierOblique;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestDifferentFontsForThreeParagraphs()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.Font = FontsRepository.CourierOblique;
                editor.InsertRun("Text in " + editor.CharacterProperties.Font.FontName + ":");
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.CharacterProperties.Font = FontsRepository.HelveticaBold;
                editor.InsertParagraph();
                editor.InsertRun("Text in " + editor.CharacterProperties.Font.FontName + ":");
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();

                editor.CharacterProperties.Font = FixedDocumentDefaults.Font;
                editor.InsertParagraph();
                editor.InsertRun("Text in " + editor.CharacterProperties.Font.FontName + ":");
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphWithTextWithForegroundColor()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.ForegroundColor = TestHelper.Blue;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphWithTextWithHighlight()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.HighlightColor = TestHelper.Yellow;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.CharacterProperties.HighlightColor = FixedDocumentDefaults.HighlightColor;
                editor.InsertLine(TestHelper.SampleParagraph);

                editor.InsertParagraph();
                editor.CharacterProperties.HighlightColor = TestHelper.Turqouise;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphWithBaselineAlignmentSet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.UnderlineColor = TestHelper.Black;
                editor.CharacterProperties.Font = FontsRepository.HelveticaBoldOblique;
                editor.CharacterProperties.UnderlinePattern = Model.Editing.Flow.UnderlinePattern.Single;
                editor.InsertRun("Normal Text");
                editor.CharacterProperties.BaselineAlignment = Model.Editing.Flow.BaselineAlignment.Superscript;
                editor.InsertRun("Superscript Text");
                editor.InsertRun(" ");

                editor.CharacterProperties.BaselineAlignment = Model.Editing.Flow.BaselineAlignment.Baseline;
                editor.InsertRun("Normal Text");
                editor.CharacterProperties.BaselineAlignment = Model.Editing.Flow.BaselineAlignment.Subscript;
                editor.InsertRun("Subscript Text");
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphWithAllCharacterProperiesSet()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.Font = FontsRepository.HelveticaOblique;
                editor.CharacterProperties.FontSize = Unit.PointToDip(14);
                editor.CharacterProperties.ForegroundColor = new RgbColor(0, 0, 0);
                editor.CharacterProperties.HighlightColor = TestHelper.Yellow;
                editor.CharacterProperties.UnderlinePattern = Model.Editing.Flow.UnderlinePattern.Single;
                editor.CharacterProperties.UnderlineColor = TestHelper.Red;
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphWithBoldAndItalicText()
        {
            document = new RadFixedDocument();
            string[] standardFontFamilies = new string[] { "Arial", "Arial Black", "Calibri", "Comic Sans MS", "Courier New", "Georgia", "Lucida Sans Unicode", "Times New Roman", "Trebuchet MS", "Verdana" };

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                for (int i = 0; i < standardFontFamilies.Length; i++)
                {
                    editor.ParagraphProperties.LeftIndent = 10;
                    editor.CharacterProperties.FontSize = Unit.DipToPoint(12);
                    editor.CharacterProperties.Font = FixedDocumentDefaults.Font;
                    editor.InsertLine("Text in " + standardFontFamilies[i] + ":");
                    editor.CharacterProperties.TrySetFont(new FontFamily(standardFontFamilies[i]), FontStyles.Italic, FontWeights.Bold);
                    editor.InsertLine(TestHelper.TestString);

                    //editor.CharacterProperties.TrySetFont(new FontFamily(standardFontFamilies[i]));
                    //editor.InsertLine(TestHelper.TestString);

                    editor.InsertLineBreak();
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void TestParagraphInPageRotatedTo180()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.SectionProperties.PageRotation = Model.Data.Rotation.Rotate90;
                editor.ParagraphProperties.LineSpacing = 5;
                editor.ParagraphProperties.HorizontalAlignment = Model.Editing.Flow.HorizontalAlignment.Center;
                editor.InsertRun(new FontFamily("Comic Sans MS"), FontStyles.Italic, FontWeights.UltraBold, "Paragraph in a rotated page");
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_AddNewParagraphWithFontFallback()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.CharacterProperties.Font = FontsRepository.HelveticaBoldOblique;
                editor.InsertLineBreak();
                editor.InsertLine(TestHelper.SampleString.Text);
                editor.InsertLineBreak();
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();
                editor.InsertLine(TestHelper.TestString);
            }

            this.AssertAreVisualyEqual(document);
        }

        // [TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        [Description("Bug: 151145")]
        public void Test_InsertEmptyparagrphWithBackgroundColorSet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertLine("Test Text");
                editor.ParagraphProperties.BackgroundColor = TestHelper.Turqouise;
                editor.ParagraphProperties.SpacingAfter = 50;
                editor.InsertParagraph();

                editor.ParagraphProperties.BackgroundColor = TestHelper.Yellow;
                editor.InsertParagraph();
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_AddNewParagraphWithDefaultSettingsTest()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertRun("First line text");
                editor.InsertLineBreak();
                editor.InsertRun("This line is placed at second line");
            }

            this.AssertAreVisualyEqual(document);
        }

        // Team Pulse bug ID: 140587 (issue is fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_AddTextUsingInsertTextMethods()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Block block = new Block();

                // Insert text with default settings
                block.InsertText("This is text with default settings");
                block.InsertLineBreak();

                // Insert text using the given font family
                var fontFamily = new System.Windows.Media.FontFamily("Comic Sans MS");
                block.InsertText(fontFamily, "This text should be Comic Sans MS!");
                block.InsertLineBreak();

                // Insert text using the given font family, font style and font weight
                var italic = System.Windows.FontStyles.Italic;
                var bold = System.Windows.FontWeights.Bold;
                fontFamily = new System.Windows.Media.FontFamily("Arial");
                block.InsertText(fontFamily, italic, bold, "This text should be Arial, Bold and Italic!");
                editor.InsertBlock(block);
            }

            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 147101
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that a document containging a value close to zero could be opened after export.")]
        public void Test_PdfRealValueIsNoteExportedInExponensialFormat()
        {
            document = new RadFixedDocument();
            double smallAngleValue = 2E-8;

            FixedContentEditor editor = new FixedContentEditor(document.Pages.AddPage());
            {
                Block block = new Block();
                block.BackgroundColor = TestHelper.Turqouise;
                block.InsertLine(new Point(0, 0), new Point(smallAngleValue, smallAngleValue));
                editor.DrawBlock(block);
            }

            this.AssertAreVisualyEqual(document);
        }

        // Team Pulse story ID: 141827
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that block element circle could be inserted in RadFixedDocumentEditor through InsertBlock()")]
        public void Test_InsertCircleInEditorThroughBlock()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Block block = new Block();
                block.InsertCircle(new Point(100, 100), 100);
                editor.InsertBlock(block);
            }

            this.AssertAreVisualyEqual(document);
        }
    }
}
