﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Editing.Tables;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Media;
using Edit = Telerik.Windows.Documents.Fixed.Model.Editing;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class FixedTablePropertiesTests : FixedContentEditorTestBase
    {
        RadFixedDocument document;
        readonly Border invisibleBorder = new Border(BorderStyle.None);
        readonly Border singleBlackBorder = new Border(1, BorderStyle.Single, TestHelper.Black);
        readonly Border redBorder = new Border(2, new RgbColor(255, 0, 0));
        readonly Border blueBorder = new Border(2, new RgbColor(0, 0, 255));
        readonly Border blackBorder = new Border(1, new RgbColor(0, 0, 0));
        readonly Border greenBorder = new Border(4, new RgbColor(0, 255, 0));

        readonly string[,] content =
        {
            { "test11 ds ds ds dsd ", "test12", "test13", "test14" },
            { "test41", "test42", "test43 - test54 long", "test45" },
        };

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithNoBorders()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "No borders");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);
                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithLeftBorderOnlySet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Left border set");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);
                table.BorderCollapse = BorderCollapse.Separate;
                Border visibleBorder = new Border(3, BorderStyle.Single, TestHelper.Pink);
                table.Borders = new TableBorders(left: visibleBorder, right: invisibleBorder, top: invisibleBorder, bottom: invisibleBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithRightBorderOnlySet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Right border set");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);

                Border visibleBorder = new Border(3, BorderStyle.Single, TestHelper.Blue);
                Border invisibleBorder = new Border(BorderStyle.None);
                table.Borders = new TableBorders(left: invisibleBorder, right: visibleBorder, top: invisibleBorder, bottom: invisibleBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithTopBorderOnlySet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Top border set");

                Border visibleBorder = new Border(3, BorderStyle.Single, TestHelper.Red);
                Border invisibleBorder = new Border(BorderStyle.None);
                table.Borders = new TableBorders(left: invisibleBorder, right: invisibleBorder, top: visibleBorder, bottom: invisibleBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithBottomBorderSet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Bottom border set");

                Border visibleBorder = new Border(3, BorderStyle.Single, TestHelper.Green);
                Border invisibleBorder = new Border(BorderStyle.None);
                table.Borders = new TableBorders(left: invisibleBorder, right: invisibleBorder, top: invisibleBorder, bottom: visibleBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithOutsideBordersOnly()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Outside borders set");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);

                Border visibleBorder = new Border(3, BorderStyle.Single, TestHelper.Yellow);
                table.Borders = new TableBorders(left: visibleBorder, right: visibleBorder, top: visibleBorder, bottom: visibleBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TablewithAllBordersSet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                int rowCount = 3;
                int columnCount = 4;

                Border cellBorders = new Border(1, BorderStyle.Single, TestHelper.Blue);

                Table table = GenerateTable(rowCount, columnCount, "All borders set");
                table.LayoutType = Edit.Flow.TableLayoutType.AutoFit;
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);

                for (int rows = 0; rows < rowCount; rows++)
                {
                    for (int columns = 0; columns < columnCount; columns++)
                    {
                        table.Rows[rows].Cells[columns].Borders = new TableCellBorders(left: cellBorders, top: cellBorders, right: cellBorders, bottom: cellBorders);
                    }
                }

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        // TODO: Test is not yet implemented
        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        //public void Test_TableWithInsideBordersOnlySet()
        //{
        //    document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        int rowCount = 3;
        //        int columnCount = 4;
        //        Border visibleBorder = new Border(1, BorderStyle.Single, TestHelper.Black);

        //        Table table = GenerateTable(rowCount, columnCount, "Inside Borders");
        //        table.DefaultCellProperties.Borders = new TableCellBorders(left: visibleBorder, top: visibleBorder, right: visibleBorder, bottom: visibleBorder);

        //        for (int i = 0; i < rowCount; i++)
        //        {
        //            for (int j = 0; j < columnCount; j++)
        //            {
        //                if (i == 0)
        //                { 

        //                }

        //            }
        //        }

        //        editor.InsertTable(table);
        //    }

        //     this.AssertAreVisualyEqual(document);
        //}

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_InsideHorizontalBorders()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 4, "Inside horizontal borders");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(8, 0, 8, 0);
                table.BorderCollapse = BorderCollapse.Separate;

                for (int i = 0; i < 3; i++)
                {
                    for (int j = 0; j < 4; j++)
                    {
                        table.Rows[1].Cells[j].Borders = new TableCellBorders(left: invisibleBorder, top: singleBlackBorder, right: invisibleBorder, bottom: singleBlackBorder);
                    }
                }
                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_InsideVerticalBorders()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 4, "Inside vertical borders");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(8, 0, 8, 0);
                table.BorderCollapse = BorderCollapse.Separate;

                for (int i = 0; i < 3; i++)
                {
                    for (int j = 0; j < 3; j++)
                    {
                        table.Rows[i].Cells[j].Borders = new TableCellBorders(left: invisibleBorder, top: invisibleBorder, right: singleBlackBorder, bottom: invisibleBorder);
                    }
                }
                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithBoldBordersSet()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 3, "Bold outside borders");
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);
                Border doubleThickBorder = new Border(3.0, BorderStyle.Single, TestHelper.Black);
                Border visibleBorder = new Border(1, BorderStyle.Single, TestHelper.Black);
                table.DefaultCellProperties.Borders = new TableCellBorders(left: visibleBorder, top: visibleBorder, right: visibleBorder, bottom: visibleBorder);
                table.Borders = new TableBorders(doubleThickBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithCustomBorders()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(6, 3, "All borders custom");
                table.BorderCollapse = BorderCollapse.Separate;
                table.BorderSpacing = 0;
                Border blueBorder = new Border(3, BorderStyle.Single, TestHelper.Blue);
                Border yellowBorder = new Border(3, BorderStyle.Single, TestHelper.Yellow);
                Border redBorder = new Border(3, BorderStyle.Single, TestHelper.Red);
                Border greenBorder = new Border(3, BorderStyle.Single, TestHelper.Green);
                Border topBottomBorder = new Border(1, BorderStyle.Single, TestHelper.Turqouise);
                Border leftRightBorder = new Border(1, BorderStyle.Single, TestHelper.Pink);


                table.Borders = new TableBorders(left: blueBorder, top: yellowBorder, right: redBorder, bottom: greenBorder);
                table.DefaultCellProperties.Borders = new TableCellBorders(left: leftRightBorder, top: topBottomBorder, right: leftRightBorder, bottom: topBottomBorder);

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableBetweenTwoParagraphs()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Border border = new Border(1, BorderStyle.Single, TestHelper.Pink);
                editor.ParagraphProperties.FirstLineIndent = 30;
                editor.InsertLine(TestHelper.SampleParagraph);
                editor.InsertLineBreak();
                Table table = GenerateTable(4, 3, "Table Between Paragraphs");
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;
                table.DefaultCellProperties.Borders = new TableCellBorders(left: border, top: border, right: border, bottom: border);
                table.Borders = new TableBorders(border);
                editor.InsertTable(table);
                editor.InsertLineBreak();

                editor.InsertParagraph();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithSpacingBeforeAndSpacingAfterSet()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = new Table();

                for (int i = 0; i < 3; i++)
                {
                    TableRow tableRows = table.Rows.AddTableRow();

                    for (int j = 0; j < 4; j++)
                    {
                        TableCell tableCell = tableRows.Cells.AddTableCell();
                        Block block = tableCell.Blocks.AddBlock();
                        block.SpacingBefore = 30;
                        block.SpacingAfter = 30;
                        block.InsertText("sdsds");
                    }
                }

                table.Borders = new TableBorders(singleBlackBorder);
                table.DefaultCellProperties.Borders = new TableCellBorders(left: singleBlackBorder, top: singleBlackBorder, right: singleBlackBorder, bottom: singleBlackBorder);
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithLineSpacingSet()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = new Table();

                for (int i = 0; i < 3; i++)
                {
                    TableRow tableRows = table.Rows.AddTableRow();

                    for (int j = 0; j < 7; j++)
                    {
                        TableCell tableCell = tableRows.Cells.AddTableCell();
                        Block block = tableCell.Blocks.AddBlock();
                        block.LineSpacing = 4;
                        block.InsertText("Line spacing set to 20");
                    }
                }

                table.Borders = new TableBorders(singleBlackBorder);
                table.DefaultCellProperties.Borders = new TableCellBorders(left: singleBlackBorder, top: singleBlackBorder, right: singleBlackBorder, bottom: singleBlackBorder);
                table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_InsertTableRowToTableEnd()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 7, "Test");
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;

                Border border = new Border(3, BorderStyle.Single, TestHelper.Blue);

                table.DefaultCellProperties.Borders = new TableCellBorders(left: border, right: border, top: border, bottom: border);
                table.Borders = new TableBorders(border);
                TableRow tableRow = table.Rows.AddTableRow();
                for (int i = 0; i < 7; i++)
                {
                    TableCell tableCell = tableRow.Cells.AddTableCell();
                    Block block = tableCell.Blocks.AddBlock();
                    block.InsertText("Row added after table generation");
                }

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_InsertTableColumnToTableEnd()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(3, 4, "Test");
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;

                Border border = new Border(3, BorderStyle.Single, TestHelper.Blue);

                table.DefaultCellProperties.Borders = new TableCellBorders(left: border, right: border, top: border, bottom: border);
                table.Borders = new TableBorders(border);

                for (int i = 0; i < 3; i++)
                {
                    TableCell tableCell = table.Rows[i].Cells.AddTableCell();
                    Block block = tableCell.Blocks.AddBlock();
                    block.InsertText("Column added after table generation");
                }

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_SplitLongTableWithNoBorders()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                int rowCount = 70;
                int columnCount = 4;

                editor.InsertLine("Some text before the table");
                Border cellBorders = new Border(1, BorderStyle.Single, TestHelper.Blue);

                Table table = GenerateTable(rowCount, columnCount, "First block text");
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;
                //table.DefaultCellProperties.Padding = new System.Windows.Thickness(5, 0, 5, 0);

                table.BorderCollapse = BorderCollapse.Separate;

                for (int rows = 0; rows < rowCount; rows++)
                {
                    for (int columns = 0; columns < columnCount; columns++)
                    {
                        table.Rows[rows].Cells[columns].Borders = new TableCellBorders(left: cellBorders, top: cellBorders, right: cellBorders, bottom: cellBorders);
                    }
                }

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown());
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_ColspanSetForFirstRow()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = new Table();

                TableRow firstRow = table.Rows.AddTableRow();
                TableCell firstRowCell = firstRow.Cells.AddTableCell();
                firstRowCell.ColumnSpan = 4;
                firstRowCell.Background = TestHelper.Yellow;
                Block block = firstRowCell.Blocks.AddBlock();
                block.InsertText("First row text");
                block.HorizontalAlignment = Edit.Flow.HorizontalAlignment.Center;

                for (int i = 1; i < 3; i++)
                {
                    TableRow row = table.Rows.AddTableRow();

                    for (int j = 0; j < 4; j++)
                    {
                        TableCell cell = row.Cells.AddTableCell();
                        Block bl = cell.Blocks.AddBlock();
                        bl.InsertText("This is some test text");
                    }
                }

                table.DefaultCellProperties.Padding = new System.Windows.Thickness(10, 3, 10, 3);
                table.DefaultCellProperties.Borders = new TableCellBorders(left: singleBlackBorder, top: singleBlackBorder, right: singleBlackBorder, bottom: singleBlackBorder);
                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 140813 (fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_NestedTableTextWrapping()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.SectionProperties.PageMargins = new Telerik.Windows.Documents.Primitives.Padding(0);
                editor.SectionProperties.PageSize = new Size(600, 500);
                var table = new Table();
                table.BorderCollapse = BorderCollapse.Separate;
                table.Borders = new TableBorders(blackBorder);
                table.DefaultCellProperties.Borders = new TableCellBorders(redBorder, redBorder, redBorder, redBorder);

                var row = table.Rows.AddTableRow();
                var cell = row.Cells.AddTableCell();

                var nestedTable = cell.Blocks.AddTable();
                nestedTable.BorderCollapse = BorderCollapse.Separate;
                nestedTable.Borders = new TableBorders(greenBorder);
                nestedTable.DefaultCellProperties.Borders = new TableCellBorders(blueBorder, blueBorder, blueBorder, blueBorder);

                row = nestedTable.Rows.AddTableRow();
                cell = row.Cells.AddTableCell();
                var block = cell.Blocks.AddBlock();
                block.InsertText("Test");

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 144471
        // When the bug is fixed uncomment TestMethod attribute
        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        public void Test_AddNestedTableToSecondCellFirstRowOfParentTable()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(2, 2, "Test");
                table.LayoutType = Telerik.Windows.Documents.Fixed.Model.Editing.Flow.TableLayoutType.FixedWidth;

                Table table2 = new Table();

                for (int i = 0; i < content.GetLength(0); i++)
                {
                    var row = table2.Rows.AddTableRow();

                    for (int j = 0; j < content.GetLength(1); j++)
                    {
                        row.Cells.AddTableCell().Blocks.AddBlock().InsertText(content[i, j]);
                    }
                }

                table.Rows[0].Cells[1].Blocks.Add(table2);
                table.DefaultCellProperties.Borders = new TableCellBorders(redBorder, redBorder, redBorder, redBorder);
                table2.DefaultCellProperties.Borders = new TableCellBorders(blueBorder, blueBorder, blueBorder, blueBorder);

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 144471
        // When the bug is fixed uncomment TestMethod attribute
        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        public void Test_AddNestedTableToAllParentTableCells()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table = GenerateTable(2, 2, "Test");
                table.LayoutType = Telerik.Windows.Documents.Fixed.Model.Editing.Flow.TableLayoutType.FixedWidth;

                Table table2 = new Table();

                for (int i = 0; i < content.GetLength(0); i++)
                {
                    var row = table2.Rows.AddTableRow();

                    for (int j = 0; j < content.GetLength(1); j++)
                    {
                        row.Cells.AddTableCell().Blocks.AddBlock().InsertText(content[i, j]);
                    }
                }

                for (int i = 0; i < 2; i++)
                {
                    for (int j = 0; j < 2; j++)
                    {
                        table.Rows[i].Cells[j].Blocks.Add(table2);
                    }
                }

                table.DefaultCellProperties.Borders = new TableCellBorders(redBorder, redBorder, redBorder, redBorder);
                table2.DefaultCellProperties.Borders = new TableCellBorders(blueBorder, blueBorder, blueBorder, blueBorder);

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextBeforeAndAfterANestedTable()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Generate table with 3 rows and 4 columns
                Table table = GenerateTable(3, 4, "Test");
                table.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;

                // Generate second table which will be used as nested table
                Table table2 = new Table();

                for (int i = 0; i < 4; i++)
                {
                    var row = table2.Rows.AddTableRow();

                    for (int j = 0; j < 4; j++)
                    {
                        row.Cells.AddTableCell().Blocks.AddBlock().InsertText("Test " + i + " " + j);
                    }
                }

                table.Rows[1].Cells[1].Blocks.AddBlock().InsertText("Before table: This is some longer text before the nested table. It should be transferred to the next line");
                table.Rows[1].Cells[1].Blocks.Add(table2);
                table.Rows[1].Cells[1].Blocks.AddBlock().InsertText("After table: Some text after the nested table!");

                table.DefaultCellProperties.Borders = new TableCellBorders(redBorder, redBorder, redBorder, redBorder);
                table2.DefaultCellProperties.Borders = new TableCellBorders(blueBorder, blueBorder, blueBorder, blueBorder);

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 144171 (fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextWrappingInTablesWithFixedWidthSize300()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.SectionProperties.PageMargins = new Telerik.Windows.Documents.Primitives.Padding(0);
                editor.SectionProperties.PageSize = new Size(300, 300);

                var table = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.Table();
                table.LayoutType = Telerik.Windows.Documents.Fixed.Model.Editing.Flow.TableLayoutType.FixedWidth;
                table.Margin = new Thickness(10);
                var redBorder = new Border(5, new RgbColor(50, 255, 0, 0));
                table.DefaultCellProperties.Borders = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.TableCellBorders(redBorder, redBorder, redBorder, redBorder);

                for (int i = 0; i < content.GetLength(0); i++)
                {
                    var row = table.Rows.AddTableRow();

                    for (int j = 0; j < content.GetLength(1); j++)
                    {
                        row.Cells.AddTableCell().Blocks.AddBlock().InsertText(content[i, j]);
                    }
                }

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 144171 (fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TextWrappingInTablesWithFixedWidthSize270()
        {
            document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.SectionProperties.PageMargins = new Telerik.Windows.Documents.Primitives.Padding(0);
                editor.SectionProperties.PageSize = new Size(270, 270);

                var table = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.Table();
                table.LayoutType = Telerik.Windows.Documents.Fixed.Model.Editing.Flow.TableLayoutType.FixedWidth;

                table.Margin = new Thickness(10);
                var redBorder = new Border(5, new RgbColor(50, 255, 0, 0));
                table.DefaultCellProperties.Borders = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.TableCellBorders(redBorder, redBorder, redBorder, redBorder);

                for (int i = 0; i < content.GetLength(0); i++)
                {
                    var row = table.Rows.AddTableRow();

                    for (int j = 0; j < content.GetLength(1); j++)
                    {
                        row.Cells.AddTableCell().Blocks.AddBlock().InsertText(content[i, j]);
                    }
                }

                editor.InsertTable(table);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithPreferredWidthSetToOneCell()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table1 = GenerateTable(3, 4, "First table", singleBlackBorder);
                table1.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;
                table1.Rows[2].Cells[2].PreferredWidth = 250;

                Table table4 = GenerateTable(3, 4, "First table", singleBlackBorder);
                table1.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;
                table1.Rows[2].Cells[2].PreferredWidth = null;

                Table table2 = GenerateTable(3, 4, "Second table", singleBlackBorder);
                table2.LayoutType = Edit.Flow.TableLayoutType.AutoFit;
                table2.Rows[2].Cells[2].PreferredWidth = 250;

                Table table3 = GenerateTable(3, 4, LongTableString, singleBlackBorder);
                table3.LayoutType = Edit.Flow.TableLayoutType.AutoFit;
                table3.Rows[2].Cells[2].PreferredWidth = 300.5;

                editor.InsertTable(table1);
                editor.InsertLineBreak();
                editor.InsertTable(table2);
                editor.InsertLineBreak();
                editor.InsertTable(table3);
                editor.InsertLineBreak();
                editor.InsertTable(table4);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithPreferredWidthSetToTwoCellsInOneColumn()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                Table table1 = GenerateTable(3, 4, "First table", singleBlackBorder);
                table1.LayoutType = Edit.Flow.TableLayoutType.FixedWidth;
                table1.Rows[2].Cells[2].PreferredWidth = 30;
                table1.Rows[1].Cells[2].PreferredWidth = 50;

                editor.InsertTable(table1);
                editor.InsertLineBreak();
                editor.InsertLine(TestHelper.SampleParagraph);
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableWithPrefferedwidthSetToMergedCellInFirstRow()
        {
            document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Generate a table
                Table table1 = GenerateTable(3, 5, "Table text", singleBlackBorder);
                // Set preferred width for first cell in the first row
                table1.Rows[0].Cells[0].PreferredWidth = 100;
                // Set preferred width to the third cell in the second row
                table1.Rows[1].Cells[2].PreferredWidth = 150;

                // Add row with column span set to 3 for the first cell
                TableRow row = table1.Rows.AddTableRow();
                TableCell cell = row.Cells.AddTableCell();
                cell.ColumnSpan = 3;
                cell.PreferredWidth = 400;
                row.Cells.AddTableCell();
                row.Cells.AddTableCell();
                Block block = cell.Blocks.AddBlock();
                block.InsertText("Text");

                table1.DefaultCellProperties.Borders = new TableCellBorders(left: singleBlackBorder, top: singleBlackBorder, right: singleBlackBorder, bottom: singleBlackBorder);

                editor.InsertTable(table1);
            }
            this.AssertAreVisualyEqual(document);
        }

        // TeamPulse bug ID: 149963 (fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableMergedCellSplittingToTwoPages()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                editor.SectionProperties.PageSize = new Size(300, 200);
                editor.SectionProperties.PageMargins = new Telerik.Windows.Documents.Primitives.Padding(25);

                byte alpha = 50;
                var table = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.Table();
                table.Borders = new TableBorders(new Border(5, new RgbColor(alpha, 0, 0, 0)));
                Border red = new Border(2, new RgbColor(alpha, 255, 0, 0));
                table.DefaultCellProperties.Borders = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.TableCellBorders(red);
                table.Margin = new Thickness(10);
                table.DefaultCellProperties.Background = new RgbColor(alpha, 255, 200, 50);
                table.Background = new RgbColor(alpha, 200, 200, 200);

                string[][] content = {
                                new string[] {"test11", "test12", },
                                new string[] {"test21",  "test22", },
                                new string[] {"test31", "test32 - test43 long", },
                                new string[] {"test41", },
                            };

                for (int i = 0; i < content.Length; i++)
                {
                    var row = table.Rows.AddTableRow();

                    for (int j = 0; j < content[i].Length; j++)
                    {
                        var cell = row.Cells.AddTableCell();

                        if (i == 2 && j == 1)
                        {
                            cell.RowSpan = 2;
                            cell.ColumnSpan = 2;
                            Border green = new Border(6, new RgbColor(alpha, 0, 255, 0));
                            Border invisible = new Border(BorderStyle.None);
                            cell.Borders = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.TableCellBorders(green, green, green, green, invisible, null);
                        }
                        else if ((i + j) % 2 == 0)
                        {
                            Border blue = new Border(4, new RgbColor(alpha, 0, 0, 255));
                            cell.Borders = new Telerik.Windows.Documents.Fixed.Model.Editing.Tables.TableCellBorders(blue, blue, blue, blue);
                        }

                        var block = new Telerik.Windows.Documents.Fixed.Model.Editing.Block();
                        block.InsertText(content[i][j]);
                        cell.Blocks.Add(block);
                    }
                }

                table.Rows[2].Cells[0].PreferredWidth = 30;
                table.Rows[2].Cells[1].PreferredWidth = 60;

                editor.InsertTable(table);
            }
            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown());
        }

        // TeamPulse bug ID: 142644 (fixed)
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_TableSplitToNewPageWhenParagraphsContainOneLineTextOnly()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                for (int i = 0; i < 100; i++)
                {
                    editor.InsertParagraph();
                    editor.InsertRun("test " + i);
                }
            }
            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown());
        }

        private Table GenerateTable(int rows, int columns, string cellText)
        {
            Table table = new Table();

            for (int i = 0; i < rows; i++)
            {
                TableRow tableRows = table.Rows.AddTableRow();

                for (int j = 0; j < columns; j++)
                {
                    TableCell tableCell = tableRows.Cells.AddTableCell();
                    Block block = tableCell.Blocks.AddBlock();
                    block.InsertText(cellText);
                }
            }

            return table;
        }

        private Table GenerateTable(int rows, int columns, string cellText, Border border)
        {
            Table table = new Table();

            for (int i = 0; i < rows; i++)
            {
                TableRow tableRows = table.Rows.AddTableRow();

                for (int j = 0; j < columns; j++)
                {
                    TableCell tableCell = tableRows.Cells.AddTableCell();
                    Block block = tableCell.Blocks.AddBlock();
                    block.InsertText(cellText);
                    table.Rows[i].Cells[j].Borders = new TableCellBorders(left: border, top: border, right: border, bottom: border);
                }
            }

            return table;
        }

        public string LongTableString = "Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add.";
    }
}
