﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Rect = System.Windows.Rect;
using Size = System.Windows.Size;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
//using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Model.Elements.Destinations;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Navigation;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
#if VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utils;
#endif
namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class HyperlinkTests : FixedContentEditorTestBase
    {
        private static readonly Uri uri = new Uri(@"http://www.google.com");

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_Hyperlink()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);

            TestHyperlink(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_Hyperlink_Center()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            block.HorizontalAlignment = HorizontalAlignment.Center;

            TestHyperlink(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_Hyperlink_Right()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            block.HorizontalAlignment = HorizontalAlignment.Right;

            TestHyperlink(document, editor, block);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_Hyperlink_CenterVert()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            block.VerticalAlignment = VerticalAlignment.Center;

            TestHyperlink(document, editor, block, 200, 400);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_Hyperlink_Bottom()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);
            Block block = CreateBlock(editor);
            block.VerticalAlignment = VerticalAlignment.Bottom;

            TestHyperlink(document, editor, block, 200, 400);
        }

#if VISUAL

        // Location
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type Location.")]
        public void Test_AddHyperlinkWithDestinationOfTypeLocation()
        {
            RadFixedDocument document = new RadFixedDocument();

            // The destination of the hyperlink is of type Locationp
            Location location = new Location();

            using (var editor = new RadFixedDocumentEditor(document))
            {
                // Create a document with two pages. The created hyperlink is a circle in the first page
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(location));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));

                // The second page is set as the destination page when the hyperlink is clicked
                location.Page = document.Pages[1];
            }

            int annotationIndex = this.CalculateAnnotationIndex(1);
            Rect annotationRectangle = this.GetAnnotationRect(document, annotationIndex);

            //var point = ConvertPointToAdobeScreenPoint(new System.Windows.Point((rect.TopLeft.X + rect.BottomRight.X) / 2, (rect.TopLeft.Y + rect.BottomRight.Y) / 2));
            var hyperlinkToClick = ConvertPointToAdobeScreenPoint(annotationRectangle);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkToClick.X, hyperlinkToClick.Y));
        }

        // PageFit
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type PageFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypePageFit()
        {
            RadFixedDocument document = new RadFixedDocument();

            List<Destination> destinations = new List<Destination>();
            int maxPageNumber = 10;

            using (var editor = new RadFixedDocumentEditor(document))
            {

                // Define the destination of the hyperlinks of type PageFit
                for (int page = 1; page <= maxPageNumber; page++)
                {
                    destinations.Add(new PageFit());
                }

                // Create document with 10 pages. Each page contains links each of the other pages.
                for (int page = 1; page <= maxPageNumber; page++)
                {
                    Block block = new Block();
                    block.InsertText(string.Format("This is page {0}!", page));
                    block.InsertLineBreak();
                    block.InsertLineBreak();

                    for (int linkPage = 1; linkPage <= maxPageNumber; linkPage++)
                    {
                        block.InsertHyperlinkStart(destinations[linkPage - 1]);
                        block.InsertText(string.Format("Click to fit page {0}!", linkPage));
                        block.InsertLineBreak();
                        block.InsertLineBreak();
                        block.InsertHyperlinkEnd();
                    }

                    editor.InsertBlock(block);
                    editor.InsertPageBreak();
                }

                // When "Click to fit page X!" link is clicked, the respective X-th page will be displayed 
                for (int pageIndex = 0; pageIndex < maxPageNumber; pageIndex++)
                {
                    destinations[pageIndex].Page = document.Pages[pageIndex];
                }
            }

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = 3;
            Rect annotationRectangle = document.Pages[0].Annotations[1 + annotationIndex * 3].Rect;

            // Calculate the exact position of the hyperlink
            var hyperlinkToClick = ConvertPointToAdobeScreenPoint(annotationRectangle);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkToClick.X, hyperlinkToClick.Y));
        }

        // PageHorizontalFit
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type PageHorizontalFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypePageHorizontalFit()
        {
            RadFixedDocument document = new RadFixedDocument();

            // Define the destination of the hyperlink of type PageHorizontalFit
            PageHorizontalFit pageHorizontalFit = new PageHorizontalFit();
            pageHorizontalFit.Top = 100;

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(pageHorizontalFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));

                pageHorizontalFit.Page = document.Pages[1];
            }

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = this.CalculateAnnotationIndex(1);
            Rect annotationRect = this.GetAnnotationRect(document, annotationIndex);

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        // PageVerticalFit
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type PageVerticalFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypePageVerticalFit()
        {
            RadFixedDocument document = new RadFixedDocument();
            PageVerticalFit pageVerticalFit = new PageVerticalFit();

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(pageVerticalFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));

                pageVerticalFit.Page = document.Pages[1];
            }

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = this.CalculateAnnotationIndex(1);
            Rect annotationRect = this.GetAnnotationRect(document, annotationIndex);

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        // RectangleFit
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type RectangleFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypeRectangleFit()
        {
            RadFixedDocument document = new RadFixedDocument();

            int rectX = 0;
            int rectY = 0;
            int rectWidth = 400;
            int rectHeight = 500;

            RectangleFit rectangleFit = new RectangleFit();
            rectangleFit.Left = rectX;
            rectangleFit.Top = rectY;
            rectangleFit.Right = rectWidth;
            rectangleFit.Bottom = rectHeight;

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(rectangleFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages.AddPage());
            Block block = new Block();
            block.InsertRectangle(new Rect(rectX, rectY, rectWidth, rectHeight));
            fixedEditor.DrawBlock(block);

            rectangleFit.Page = document.Pages[2];

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = this.CalculateAnnotationIndex(1);
            Rect annotationRect = this.GetAnnotationRect(document, annotationIndex);

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        // BoundingRectangleFit
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Creates a hyperlink with destination of type BoundingRectangleFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypeBoundingRectangleFit()
        {
            RadFixedDocument document = new RadFixedDocument();
            BoundingRectangleFit boundingRectangleFit = new BoundingRectangleFit();

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(boundingRectangleFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));
                editor.InsertPageBreak();
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));

                //boundingRectangleFit.Page = document.Pages[1];
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages.AddPage());
            Block block = new Block();
            fixedEditor.DrawBlock(block, new Size(300, 400));
            fixedEditor.GraphicProperties.IsFilled = false;
            fixedEditor.GraphicProperties.IsStroked = true;
            block.InsertText("Bounding rectangle fit");
            fixedEditor.DrawBlock(block);

            boundingRectangleFit.Page = document.Pages[2];

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = this.CalculateAnnotationIndex(1);
            Rect annotationRect = this.GetAnnotationRect(document, annotationIndex);

            Assert.AreEqual<double>(96, annotationRect.X);
            Assert.AreEqual(172, annotationRect.Y, 1);
            Assert.AreEqual<double>(61, annotationRect.Width);
            Assert.AreEqual(63, annotationRect.Height, 1);

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        // BoundingRectangleHorizontalFit
        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        [Description("Creates a hyperlink with destination of type BoundingRectangleHorizontalFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypeBoundingRectangleHorizontalFit()
        {
            RadFixedDocument document = new RadFixedDocument();

            BoundingRectangleHorizontalFit boundingRectangleHorizontalFit = new BoundingRectangleHorizontalFit();
            boundingRectangleHorizontalFit.Top = 200;

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(boundingRectangleHorizontalFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));
                editor.InsertPageBreak();
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages.AddPage());
            Block block = new Block();
            fixedEditor.DrawBlock(block, new Size(300, 400));
            fixedEditor.GraphicProperties.IsFilled = false;
            fixedEditor.GraphicProperties.IsStroked = true;
            block.InsertText("Bounding rectangle horizontal fit");
            fixedEditor.DrawBlock(block);

            boundingRectangleHorizontalFit.Page = document.Pages[2];

            FixedContentEditor ed = new FixedContentEditor(document.Pages[0]);
            ed.DrawBlock(new Block(), new Size(300, 400));
            ed.GraphicProperties.IsFilled = false;
            ed.GraphicProperties.IsStroked = true;
            int count = 0;

            foreach (var annot in document.Pages[0].Annotations)
            {
                if (count == 0) ed.GraphicProperties.StrokeColor = new RgbColor(255, 0, 0);
                else if (count == 1) ed.GraphicProperties.StrokeColor = new RgbColor(0, 255, 0);
                else ed.GraphicProperties.StrokeColor = new RgbColor(0, 0, 255);
                count = (count + 1) % 3;

                ed.DrawRectangle(annot.Rect);
                ed.DrawLine(annot.Rect.TopLeft, annot.Rect.BottomRight);
                ed.DrawLine(annot.Rect.BottomLeft, annot.Rect.TopRight);
            }

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = 0;
            Rect annotationRect = document.Pages[0].Annotations[2 + annotationIndex * 3].Rect;

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        // BoundingRectangleVerticalFit
        //[TestMethod][TestCategory(TestCategory.PdfViewerVisualTests)]
        [Description("Creates a hyperlink with destination of type BoundingRectangleVerticalFit.")]
        public void Test_AddHyperlinkWithDestinationOfTypeBoundingRectangleVerticalFit()
        {
            RadFixedDocument document = new RadFixedDocument();

            BoundingRectangleVerticalFit boundingRectangleVerticalFit = new BoundingRectangleVerticalFit();
            boundingRectangleVerticalFit.Left = 200;

            using (var editor = new RadFixedDocumentEditor(document))
            {
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
                editor.InsertBlock(CreateBlockWithHyperlink(boundingRectangleVerticalFit));
                editor.InsertBlock(CreateBlockWithText(TestHelper.SampleLongString));
                //editor.InsertSectionBreak();
                //editor.InsertBlock(CreateBlockWithText(TestHelper.SampleParagraph));
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages.AddPage());
            Block block = new Block();
            fixedEditor.DrawBlock(block, new Size(300, 400));
            fixedEditor.GraphicProperties.IsFilled = false;
            fixedEditor.GraphicProperties.IsStroked = true;
            block.InsertText("Bounding rectangle vertical fit");
            fixedEditor.DrawBlock(block);

            boundingRectangleVerticalFit.Page = document.Pages[2];

            FixedContentEditor ed = new FixedContentEditor(document.Pages[0]);
            ed.DrawBlock(new Block(), new Size(300, 400));
            ed.GraphicProperties.IsFilled = false;
            ed.GraphicProperties.IsStroked = true;
            int count = 0;

            foreach (var annot in document.Pages[0].Annotations)
            {
                if (count == 0) ed.GraphicProperties.StrokeColor = new RgbColor(255, 0, 0);
                else if (count == 1) ed.GraphicProperties.StrokeColor = new RgbColor(0, 255, 0);
                else ed.GraphicProperties.StrokeColor = new RgbColor(0, 0, 255);
                count = (count + 1) % 3;

                ed.DrawRectangle(annot.Rect);
                ed.DrawLine(annot.Rect.TopLeft, annot.Rect.BottomRight);
                ed.DrawLine(annot.Rect.BottomLeft, annot.Rect.TopRight);
            }

            // Get the coordinates of the hyperlink so it could be clicked 
            int annotationIndex = 0;
            Rect annotationRect = document.Pages[0].Annotations[2 + annotationIndex * 3].Rect;

            // Calculate the exact position of the hyperlink
            var hyperlinkCoordinates = ConvertPointToAdobeScreenPoint(annotationRect);

            this.AssertAreVisualyEqual(document, UIInteractionHelper.LeftMouseClick(hyperlinkCoordinates.X, hyperlinkCoordinates.Y));
        }

        public static System.Drawing.Point ConvertPointToAdobeScreenPoint(Rect rect)
        {
            int xAdobe = 559;
            int yAdobe = 85;
            double scale = 1;
            double customResolutionCoeficient = 1;

            double pdfViewerPointX = (rect.TopLeft.X + rect.BottomRight.X) / 2;
            double pdfViewerPointY = (rect.TopLeft.Y + rect.BottomRight.Y) / 2;

            int xPoint = xAdobe + (int)(scale * customResolutionCoeficient * pdfViewerPointX);
            int yPoint = yAdobe + (int)(scale * customResolutionCoeficient * pdfViewerPointY);

            return new System.Drawing.Point(xPoint, yPoint);
        }

        public static System.Drawing.Point ConvertPointToAdobeScreenPoint(System.Windows.Point pdfViewerPoint)
        {
            int xAdobe = 46;
            int yAdobe = 85;
            double scale = 2.27;
            double customResolutionCoeficient = 1;

            int xPoint = xAdobe + (int)(scale * customResolutionCoeficient * pdfViewerPoint.X);
            int yPoint = yAdobe + (int)(scale * customResolutionCoeficient * pdfViewerPoint.Y);

            return new System.Drawing.Point(xPoint, yPoint);
        }

#endif

        private void TestHyperlink(RadFixedDocument document, FixedContentEditor editor, Block block, double blockWidth = 200, double blockHeight = double.PositiveInfinity)
        {
            block.InsertHyperlinkStart(uri);
            block.InsertText("Link");
            block.InsertHyperlinkEnd();

            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double lineHeight = GetLineHeight(block);
            double leftIndent = block.LeftIndent;
            double rightIndent = block.RightIndent;
            double offsetX = CalculateLeftOffset(block, blockWidth, MeasureText(editor, "Link").Width + leftIndent + rightIndent, 0);
            double offsetY = CalculateTopOffset(block, lineHeight, blockHeight);

            Rect rect = new Rect(offsetX, offsetY, MeasureText(editor, "Link").Width, lineHeight);

            RadFixedDocument expected = CreateExpectedDocument(CreateElement(editor, block, CreateTextFragment(editor, block, "Link", rect.X, rect.Y), rect));
            Link link = expected.Pages[0].Annotations.AddLink(new UriAction(uri));
            link.Rect = rect;

            ExportAndAssert(expected, document);
        }

        private void TestHyperlink(RadFixedDocument document, FixedContentEditor editor, Block block, System.Action action, int repeatActionTime, double blockWidth = 200, double blockHeight = double.PositiveInfinity)
        {
            block.InsertHyperlinkStart(uri);
            block.InsertText("Link");
            block.InsertHyperlinkEnd();

            editor.DrawBlock(block, new Size(blockWidth, blockHeight));

            double lineHeight = GetLineHeight(block);
            double leftIndent = block.LeftIndent;
            double rightIndent = block.RightIndent;
            double offsetX = CalculateLeftOffset(block, blockWidth, MeasureText(editor, "Link").Width + leftIndent + rightIndent, 0);
            double offsetY = CalculateTopOffset(block, lineHeight, blockHeight);

            Rect rect = new Rect(offsetX, offsetY, MeasureText(editor, "Link").Width, lineHeight);

            RadFixedDocument expected = CreateExpectedDocument(CreateElement(editor, block, CreateTextFragment(editor, block, "Link", rect.X, rect.Y), rect));
            Link link = expected.Pages[0].Annotations.AddLink(new UriAction(uri));
            link.Rect = rect;

            this.ExportAndAssert(expected, document, action, repeatActionTime);
        }

        private Block CreateBlockWithText(string text)
        {
            Block block = new Block();
            block.InsertText(text);
            block.InsertLineBreak();

            return block;
        }

        private Block CreateBlockWithHyperlink(Destination destination)
        {
            Block block = new Block();
            block.InsertHyperlinkStart(destination);
            block.InsertCircle(new System.Windows.Point(200, 300), 30);
            block.InsertLineBreak();
            block.InsertLineBreak();
            block.InsertHyperlinkEnd();

            return block;
        }

        private int CalculateAnnotationIndex(int annotationIndex)
        {
            //  Rect annotationRectangle = document.Pages[0].Annotations[2 + annotationIndex * 3].Rect;
            return annotationIndex + 0 * 3;
        }

        private System.Windows.Rect GetAnnotationRect(RadFixedDocument document, int annotationIndex)
        {
            Rect annotationRect = document.Pages[0].Annotations[annotationIndex].Rect;
            return annotationRect;
        }
    }
}
