﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Actions;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Editing.Flow;
using Rect = System.Windows.Rect;
using Size = System.Windows.Size;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Editing.Lists;
using Telerik.Windows.Documents.Core.Tests.Utils;
using System.Collections.Generic;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class ListsTests : FixedContentEditorTestBase
    {
        //private string secondListText = "First list level ";
        private string twoLinesText = "This line should be extended to two lines so the behaviour of the lists could be observed when the list level text is two or more lines long.";

        readonly List<Tuple<int, string>> listItems;

        public ListsTests()
        {
            this.listItems = new List<Tuple<int, string>>();
            this.listItems.Add(new Tuple<int, string>(0, "Video provides a powerful way to help you prove your point"));
            this.listItems.Add(new Tuple<int, string>(0, "When you click Online"));
            this.listItems.Add(new Tuple<int, string>(1, "Click Insert and then choose the elements you want from the different galleries"));
            this.listItems.Add(new Tuple<int, string>(2, "Foreground of the selected ToolBoxItem becomes white when"));
            this.listItems.Add(new Tuple<int, string>(3, "Header Button visual states and Foreground improvements in Windows8 theme.Cannot paste using context menu after pressing clear button"));
            this.listItems.Add(new Tuple<int, string>(2, "Avoid something is not BY DESIGN"));
            this.listItems.Add(new Tuple<int, string>(1, "Manipulation adorner"));
            this.listItems.Add(new Tuple<int, string>(0, "Cannot paste using context menu after pressing clear button."));
            this.listItems.Add(new Tuple<int, string>(1, "Header Button visual states and Foreground improvements in Windows8 theme.Cannot paste using context menu after pressing clear button"));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies bulleted list creation")]
        public void Test_CreateBulletedList()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create four bulleted lists
                List list1 = editor.Lists.AddList(ListTemplateType.BulletDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.BulletDefault);
                List list3 = editor.Lists.AddList(ListTemplateType.BulletDefault);
                List list4 = editor.Lists.AddList(ListTemplateType.BulletDefault);

                InsertNewParagraphWithoutList(editor, "Paragraph before the first list");

                // Populate first list items
                for (int i = 0; i < 9; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("First list, level: " + i);
                }

                InsertNewParagraphWithoutList(editor, "Paragraph between first and second paragraph");

                // Populate second list items
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine("Second list item text");
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate third list items
                for (int i = 0; i < 3; i++)
                {
                    editor.ParagraphProperties.ListId = list3.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate forth list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list4.Id, listItem.Item1, listItem.Item2);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies numbered list creation")]
        public void Test_CreateNumberedList()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create four numbered lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list3 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list4 = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                InsertNewParagraphWithoutList(editor, "Paragraph before the first list");

                // Populate first list items
                for (int i = 0; i < 9; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("First list, level: " + i);
                }

                InsertNewParagraphWithoutList(editor, "Paragraph between first and second paragraph");

                // Populate second list items
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine("Second list item text");
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate third list items
                for (int i = 0; i < 3; i++)
                {
                    editor.ParagraphProperties.ListId = list3.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate forth list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list4.Id, listItem.Item1, listItem.Item2);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies paranthesis list creation")]
        public void Test_CreateParanthesisList()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create four paranthesis lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);
                List list3 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);
                List list4 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);

                InsertNewParagraphWithoutList(editor, "Paragraph before the first list");

                // Populate first list items
                for (int i = 0; i < 9; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("First list, level: " + i);
                }

                InsertNewParagraphWithoutList(editor, "Paragraph between first and second paragraph");

                // Populate second list items
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine("Second list item text");
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate third list items
                for (int i = 0; i < 3; i++)
                {
                    editor.ParagraphProperties.ListId = list3.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate forth list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list4.Id, listItem.Item1, listItem.Item2);
                }
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies hierarchical list creation")]
        public void Test_CreateHierarchicalList()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create four hierrarchical lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedHierarchical);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedHierarchical);
                List list3 = editor.Lists.AddList(ListTemplateType.NumberedHierarchical);

                InsertNewParagraphWithoutList(editor, "Paragraph before the first list");

                // Populate first list items
                for (int i = 0; i < 9; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("First list, level: " + i);
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate third list items
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                InsertNewParagraphWithoutList(editor, TestHelper.SampleParagraph);

                // Populate forth list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list3.Id, listItem.Item1, listItem.Item2);
                }
            }
            this.AssertAreVisualyEqual(document);
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that the different character properties could be applied to different lists")]
        public void Test_ApplyDifferentCharacterPropertiesToDifferentLists()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Define three lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.BulletDefault);
                List list3 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);

                // Modify the default character properties values
                editor.CharacterProperties.Font = FontsRepository.CourierOblique;
                editor.CharacterProperties.FontSize = 10;
                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                editor.CharacterProperties.HighlightColor = TestHelper.Yellow;
                editor.CharacterProperties.UnderlineColor = TestHelper.Blue;
                editor.CharacterProperties.UnderlinePattern = UnderlinePattern.Single;

                // Insert a paragraph after the first list
                InsertNewParagraphWithoutList(editor, "List with custom character properties for all list items");

                // Populate first list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list1.Id, listItem.Item1, listItem.Item2);
                }

                // Reset the character properties to the default values and insert a new paragraph
                TestHelper.ResetCharacterProperties(editor);
                InsertNewParagraphWithoutList(editor, "List with different character properties for all list items");

                // Populate second list items where each list level item has different values
                editor.CharacterProperties.Font = FontsRepository.CourierBold;
                editor.CharacterProperties.ForegroundColor = TestHelper.Blue;
                InsertListItemText(editor, list2.Id, 0, "This is blue text");

                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                InsertListItemText(editor, list2.Id, 1, "This is red text");

                editor.CharacterProperties.ForegroundColor = TestHelper.Green;
                InsertListItemText(editor, list2.Id, 0, "This is green text");

                editor.CharacterProperties.ForegroundColor = TestHelper.Pink;
                InsertListItemText(editor, list2.Id, 1, "This is pink text");
                editor.InsertLineBreak();

                // Insert a new paragraph
                InsertNewParagraphWithoutList(editor, "List with character properties inherited from the previos list");

                // Character properties of the thirs list are inherited from the last item of the second list
                for (int i = 0; i < 2; i++)
                {
                    editor.ParagraphProperties.ListId = list3.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("Third list, level: " + i);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that the different paragraph properties could be applied to different lists")]
        public void Test_ApplyDifferentParagraphPropertiesToDifferentLists()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Define lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);

                // Insert a paragraph before the first list
                InsertNewParagraphWithoutList(editor, "List with custom paragraph properties for all list items");

                // Modify the default paragraph properties values
                editor.ParagraphProperties.BackgroundColor = TestHelper.Turqouise;
                editor.ParagraphProperties.FirstLineIndent = 80;
                editor.ParagraphProperties.LineSpacing = 3.0;

                // Populate list items
                for (int i = 0; i < 3; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine("First list, level: " + i);
                }

                // Reset the paragraph properties to the default values and insert a new paragraph
                editor.ParagraphProperties.LineSpacing = FixedDocumentDefaults.LineSpacing;
                editor.ParagraphProperties.FirstLineIndent = FixedDocumentDefaults.FirstLineIndent;
                editor.ParagraphProperties.BackgroundColor = null;
                InsertNewParagraphWithoutList(editor, "List with default paragraph properties for all list items");

                // Populate list items
                foreach (var listItem in listItems)
                {
                    InsertListItemText(editor, list1.Id, listItem.Item1, listItem.Item2);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that a list starts from a predefined start index")]
        public void Test_CreateListWithStartIndexDifferentThanTheDefaultOne()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                list1.Levels[0].StartIndex = 6;

                for (int i = 0; i < 9; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine("Start index 6");
                }
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that a list level restarts at a predefined level")]
        public void Test_RestartAfterLevel()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                int[] listLevel = new int[] { 0, 1, 2, 1, 2, 1, 0, 1, 2 };
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                InsertNewParagraphWithoutList(editor, "List with RestartAfterLevel=0 for the third level");

                // Predefine RestartAfterLevel for Level 2
                list1.Levels[2].RestartAfterLevel = 0;

                // Create a list 
                for (int i = 0; i < listLevel.Length; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = listLevel[i];
                    editor.InsertParagraph();
                    editor.InsertLine("Level " + listLevel[i]);
                }

                InsertNewParagraphWithoutList(editor, "List with default RestartAfterLevel value for all list levels");

                // Create a list with default RestartAfterLevel values for all levels
                for (int i = 0; i < listLevel.Length; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = listLevel[i];
                    editor.InsertParagraph();
                    editor.InsertLine("Level " + listLevel[i]);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that a list level restarts at a predefined level")]
        public void Test_IndentAfterBulletNumbering()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                int[] listLevel = new int[] { 0, 1, 2, 1, 2, 1, 0, 1, 2 };
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                InsertNewParagraphWithoutList(editor, "List with default IndentAfterBullet value for all list levels");

                // Create a list with default IndentAfterBulletNumbering values for all levels
                for (int i = 0; i < listLevel.Length; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = listLevel[i];
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText + listLevel[i]);
                }

                InsertNewParagraphWithoutList(editor, "List with custom value for IndentAfterBullet value for all list levels");

                // Set a custom value fot the indent after bullet 
                for (int i = 0; i < list1.Levels.Count; i++)
                {
                    list1.Levels[i].ParagraphProperties.FirstLineIndent = -15;
                    list1.Levels[i].IndentAfterBullet = 350;
                }

                // Create a list 
                for (int i = 0; i < listLevel.Length; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = listLevel[i];
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText + listLevel[i]);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that list's FirstLineIndent could be predefined")]
        public void Test_OverideDefaultFirstLineIndentValue()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create two lists
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                // Insert new paragraph
                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                editor.InsertLine("This is a list with default FirstLineIdent");
                editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;
                editor.InsertParagraph();

                // Create a list with default FirtLineIndent value
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                // Insert new paragraph
                editor.ParagraphProperties.ListId = null;
                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                editor.InsertLineBreak();
                editor.InsertParagraph();
                editor.InsertLine("This is a list with overridden FirstLineIdent");
                editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;

                // Override the list's FirstLineIndent value
                foreach (var level in list2.Levels)
                {
                    level.ParagraphProperties.FirstLineIndent = 40;
                }

                // Create a list with overridden FirstLineIndent value
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list2.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that list's LeftIndent could be predefined")]
        public void Test_OverideDefaultLeftIndentValue()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create two lists
                List list = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list1 = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                // Insert a new paragraph
                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                editor.InsertLine("This is a list with default LeftIndent");
                editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;
                editor.InsertParagraph();

                // Create a list with default LeftIndent value
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }

                // Insert a new paragraph
                editor.ParagraphProperties.ListId = null;
                editor.CharacterProperties.ForegroundColor = TestHelper.Red;
                editor.InsertLineBreak();
                editor.InsertParagraph();
                editor.InsertLine("This is a list with overridden LeftIndent");
                editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;

                // Override list's LeftIndent value
                foreach (var level in list1.Levels)
                {
                    level.ParagraphProperties.LeftIndent = 100;
                }

                // Create a list with overridden LeftIndent value
                for (int i = 0; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list1.Id;
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that default bullet style could be set")]
        public void Test_SetCustomBulletFormat()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create a list
                List list = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                editor.CharacterProperties.ForegroundColor = TestHelper.Pink;

                // Define new bullet type for all list levels
                foreach (var level in list.Levels)
                {
                    level.CharacterProperties.Font = FontsRepository.TimesRoman;
                    level.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
                    list.Levels[0].BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
                    {
                        return string.Format("¤", indexer.GetCurrentIndex(0));
                    });
                }

                // Create the list
                for (int i = 1; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list.Id;
                    editor.InsertParagraph();
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertLine("Custom bullet style" + i);
                }
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that negative value for FirstLineIndent is applied as Hanging indent")]
        public void Test_NegativeFirstLineIndentAppliesAsHangingIndent()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                List list = editor.Lists.AddList(ListTemplateType.NumberedDefault);

                foreach (var level in list.Levels)
                {
                    level.ParagraphProperties.FirstLineIndent = -80;
                }

                for (int i = 1; i < 4; i++)
                {
                    editor.ParagraphProperties.ListId = list.Id;
                    editor.InsertParagraph();
                    editor.ParagraphProperties.ListLevel = 0;
                    editor.InsertLine(TestHelper.SampleParagraph);
                }
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("Verifies that custom defined list levels could be applied")]
        public void Test_SetSeparateCharacterPropertiesForEachListLevel()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                List list = editor.Lists.AddList();

                ListLevel listLevel = new ListLevel();
                listLevel.StartIndex = 2;
                listLevel.RestartAfterLevel = 0;
                listLevel.CharacterProperties.Font = FontsRepository.CourierBold;
                listLevel.CharacterProperties.FontSize = 20;
                listLevel.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
                listLevel.IndentAfterBullet = 30;
                listLevel.ParagraphProperties.FirstLineIndent = 10;
                listLevel.ParagraphProperties.LeftIndent = 20;
                listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
                {
                    return string.Format("-{0}->", indexer.GetCurrentIndex(0));
                });

                ListLevel listLevel1 = new ListLevel();
                listLevel1.StartIndex = 4;
                listLevel1.RestartAfterLevel = 0;
                listLevel1.CharacterProperties.Font = FontsRepository.CourierBoldOblique;
                listLevel1.CharacterProperties.FontSize = 22;
                listLevel1.CharacterProperties.ForegroundColor = TestHelper.Red;
                listLevel1.IndentAfterBullet = 30;
                listLevel1.ParagraphProperties.FirstLineIndent = 10;
                listLevel1.ParagraphProperties.LeftIndent = 30;
                listLevel1.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
                {
                    return string.Format("-{0}->", indexer.GetCurrentIndex(1));
                });

                ListLevel listLevel2 = new ListLevel();
                listLevel2.StartIndex = 6;
                listLevel2.RestartAfterLevel = 0;
                listLevel2.CharacterProperties.Font = FontsRepository.CourierBold;
                listLevel2.CharacterProperties.FontSize = 24;
                listLevel2.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
                listLevel2.IndentAfterBullet = 30;
                listLevel2.ParagraphProperties.FirstLineIndent = 10;
                listLevel2.ParagraphProperties.LeftIndent = 40;
                listLevel2.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
                {
                    return string.Format("-{0}->", indexer.GetCurrentIndex(2));
                });

                list.Levels.Add(listLevel);
                list.Levels.Add(listLevel1);
                list.Levels.Add(listLevel2);

                for (int i = 0; i < 3; i++)
                {
                    editor.ParagraphProperties.ListId = list.Id;
                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);

                    editor.ParagraphProperties.ListLevel = i;
                    editor.InsertParagraph();
                    editor.InsertLine(twoLinesText);
                }
            }
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_CreateADocumentWithAllListTemplateTypes()
        {
            RadFixedDocument document = new RadFixedDocument();

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
                // Create four bulleted lists
                List list1 = editor.Lists.AddList(ListTemplateType.BulletDefault);
                List list2 = editor.Lists.AddList(ListTemplateType.NumberedDefault);
                List list3 = editor.Lists.AddList(ListTemplateType.NumberedHierarchical);
                List list4 = editor.Lists.AddList(ListTemplateType.NumberedParentheses);

                List<List> lists = new List<List>();
                lists.Add(list1);
                lists.Add(list2);
                lists.Add(list3);
                lists.Add(list4);

                foreach (var list in lists)
                {
                    InsertNewParagraphWithoutList(editor, "Paragraph separator");

                    // Populate list items
                    for (int i = 0; i < 4; i++)
                    {
                        editor.ParagraphProperties.ListId = list.Id;
                        editor.ParagraphProperties.ListLevel = i;
                        editor.InsertParagraph();
                        editor.InsertLine("List level: " + i);
                    }
                }
            }

            this.AssertAreVisualyEqual(document);
        }

        private ListLevel CreateListLevel(int currentIndex)
        {
            ListLevel listLevel = new ListLevel();
            listLevel.CharacterProperties.Font = FontsRepository.CourierBold;
            listLevel.CharacterProperties.FontSize = 20;
            listLevel.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
            listLevel.StartIndex = 0;
            listLevel.IndentAfterBullet = 30;
            listLevel.RestartAfterLevel = 0;
            listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
            {
                return string.Format("-{0}->", indexer.GetCurrentIndex(currentIndex));
            });
            return listLevel;
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_InsertListThroughBlock()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = CreateFixedContentEditor(page);

            List list = new List();

            // Create first list level
            ListLevel listLevel = new ListLevel();
            listLevel.StartIndex = 2;
            listLevel.ParagraphProperties.LeftIndent = 20;
            listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
            {
                return string.Format("-{0}->", indexer.GetCurrentIndex(0));
            });

            // Create second list level
            ListLevel listLevel1 = new ListLevel();
            listLevel1.StartIndex = 4;
            listLevel1.ParagraphProperties.LeftIndent = 30;
            listLevel1.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
            {
                return string.Format("-{0}->", indexer.GetCurrentIndex(1));
            });

            // Add list levels to the list
            list.Levels.Add(listLevel);
            list.Levels.Add(listLevel1);

            Block block1 = new Block();
            block1.SetBullet(list, 0);
            block1.InsertText("one");
            var size1 = block1.Measure();
            editor.DrawBlock(block1);
            editor.Position.Translate(0, size1.Height);

            Block block2 = new Block();
            block2.SetBullet(list, 0);
            block2.InsertText("one");
            var size2 = block2.Measure();
            editor.DrawBlock(block2);
            editor.Position.Translate(0, size1.Height + size2.Height);

            Block block3 = new Block();
            block3.SetBullet(list, 1);
            block3.InsertText("two");
            editor.DrawBlock(block3);
            var size3 = block3.Measure();
            editor.DrawBlock(block3);
            editor.Position.Translate(0, size1.Height + size2.Height + size3.Height);

            Block block4 = new Block();
            block4.SetBullet(list, 1);
            block4.InsertText("two");
            editor.DrawBlock(block4);

            this.AssertAreVisualyEqual(document);
        }

        private void InsertListItemText(RadFixedDocumentEditor editor, int listId, int listLevel, string text)
        {
            editor.ParagraphProperties.ListId = listId;
            editor.ParagraphProperties.ListLevel = listLevel;
            editor.InsertParagraph();
            editor.InsertRun(text);
        }

        private void InsertNewParagraphWithoutList(RadFixedDocumentEditor editor, string text)
        {
            editor.ParagraphProperties.ListId = null;
            editor.InsertLineBreak();
            editor.InsertParagraph();
            editor.InsertLine(text);
            editor.InsertLineBreak();
        }
    }
}
