﻿using System;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Media;
using Rect = System.Windows.Rect;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Editing
{
    [TestClass]
    public class SpacingTests : FixedContentEditorTestBase
    {
        private static readonly RgbColor blue = new RgbColor(0, 0, 255);

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_Text()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateTextDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_TextAndImage()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateTextAndImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_TextAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateTextAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_TextImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateTextImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_Image()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_Path()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreatePathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAuto_ImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAutoLineSpacing(editor);
                CreateImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_Text()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateTextDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_TextAndImage()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateTextAndImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_TextAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateTextAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_TextImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateTextImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_Image()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_Path()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreatePathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingAtLeast_ImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyAtLeastLineSpacing(editor);
                CreateImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_Text()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateTextDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_TextAndImage()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateTextAndImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_TextAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateTextAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_TextImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateTextImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_Image()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateImageDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_Path()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreatePathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_LineSpacingExact_ImageAndPath()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplyExactLineSpacing(editor);
                CreateImageAndPathDocument(editor);
            }

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void Test_SpacingBefore_SecondPage()
        {
            RadFixedDocument document = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = CreateRadDocumentEditor(document))
            {
                ApplySpacingBefore(editor);
                CreateTwoPagesDocument(editor);
            }

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown());
        }

        private static RadFixedDocumentEditor CreateRadDocumentEditor(RadFixedDocument document)
        {
            RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document);
            editor.CharacterProperties.TrySetFont(new System.Windows.Media.FontFamily("Calibri"));
            editor.CharacterProperties.FontSize = Unit.PointToDip(11);

            return editor;
        }

        private static void CreateTextDocument(RadFixedDocumentEditor editor)
        {
            editor.CharacterProperties.HighlightColor = blue;
            editor.InsertLine("Line 1");
            editor.CharacterProperties.HighlightColor = null;
            editor.InsertImageInline(CreateImageSource(), new Size(500, 30));
        }

        private static void CreateImageDocument(RadFixedDocumentEditor editor)
        {
            editor.CharacterProperties.HighlightColor = blue;
            editor.InsertImageInline(CreateImageSource(), new Size(500, 30));
            editor.CharacterProperties.HighlightColor = null;
            editor.InsertImageInline(CreateImageSource(), new Size(500, 30));
        }

        private static void CreatePathDocument(RadFixedDocumentEditor editor)
        {
            Block block = CreateBlock(editor);
            block.TextProperties.HighlightColor = blue;
            block.InsertRectangle(new Rect(0, 0, 500, 30));
            block.TextProperties.HighlightColor = null;
            block.InsertImage(CreateImageSource(), new Size(500, 30));

            editor.InsertBlock(block);
        }

        private static void CreateImageAndPathDocument(RadFixedDocumentEditor editor)
        {
            Block block = CreateBlock(editor);
            block.TextProperties.HighlightColor = blue;
            block.InsertImage(CreateImageSource(), new Size(500, 30));
            block.TextProperties.HighlightColor = null;
            block.InsertRectangle(new Rect(0, 0, 40, 30));
            block.InsertImage(CreateImageSource(), new Size(500, 30));

            editor.InsertBlock(block);
        }

        private static void CreateTextAndImageDocument(RadFixedDocumentEditor editor)
        {
            editor.CharacterProperties.HighlightColor = blue;
            editor.InsertRun("Line 1");
            editor.CharacterProperties.HighlightColor = null;
            editor.InsertImageInline(CreateImageSource(), new Size(500, 30));
            editor.InsertImageInline(CreateImageSource(), new Size(500, 30));
        }

        private static void CreateTextAndPathDocument(RadFixedDocumentEditor editor)
        {
            Block block = CreateBlock(editor);
            block.TextProperties.HighlightColor = blue;
            block.InsertText("Line 1");
            block.TextProperties.HighlightColor = null;
            block.InsertRectangle(new Rect(0, 0, 300, 30));
            block.InsertImage(CreateImageSource(), new Size(500, 30));

            editor.InsertBlock(block);
        }

        private static void CreateTextImageAndPathDocument(RadFixedDocumentEditor editor)
        {
            Block block = CreateBlock(editor);
            block.TextProperties.HighlightColor = blue;
            block.InsertText("Line 1");
            block.TextProperties.HighlightColor = null;
            block.InsertImage(CreateImageSource(), new Size(500, 30));
            block.InsertRectangle(new Rect(0, 0, 50, 30));
            block.InsertImage(CreateImageSource(), new Size(500, 30));

            editor.InsertBlock(block);
        }

        private static void CreateTwoPagesDocument(RadFixedDocumentEditor editor)
        {
            editor.InsertLine("Paragraph1");
            editor.InsertPageBreak();
            editor.InsertLine("Paragraph2");
        }

        private static void ApplyAutoLineSpacing(RadFixedDocumentEditor editor)
        {
            editor.ParagraphProperties.LineSpacing = 2;
            editor.ParagraphProperties.LineSpacingType = HeightType.Auto;
        }

        private static void ApplyAtLeastLineSpacing(RadFixedDocumentEditor editor)
        {
            editor.ParagraphProperties.LineSpacing = 50;
            editor.ParagraphProperties.LineSpacingType = HeightType.AtLeast;
        }

        private static void ApplyExactLineSpacing(RadFixedDocumentEditor editor)
        {
            editor.ParagraphProperties.LineSpacing = 90;
            editor.ParagraphProperties.LineSpacingType = HeightType.Exact;
        }

        private static void ApplySpacingBefore(RadFixedDocumentEditor editor)
        {
            editor.ParagraphProperties.SpacingBefore = 200;
        }

        private static Block CreateBlock(RadFixedDocumentEditor editor)
        {
            Block block = new Block();
            editor.ParagraphProperties.CopyTo(block);
            editor.CharacterProperties.CopyTo(block);

            return block;
        }
    }
}
