﻿using System;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.FixedDocumentViewers.Tests.Integration;

namespace Telerik.Windows.Documents.Fixed.Tests.Editor
{
    [TestClass]
    public class TestTextWrap
    {
        private const double FontSize = 14;

        [TestMethod]
        public void Test_TextWrap()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FixedContentEditor editor = new FixedContentEditor(page);
            editor.TextProperties.FontSize = FontSize;
            editor.TextProperties.BlockWidth = 100;
            editor.DrawText("test text wrapping ala bala");

            Assert.AreEqual(3, page.Content.Count);
            Assert.AreEqual("test text", ((TextFragment)page.Content[0]).Text);
            Assert.AreEqual("wrapping ala", ((TextFragment)page.Content[1]).Text);
            Assert.AreEqual("bala", ((TextFragment)page.Content[2]).Text);
        }

        [TestMethod]
        public void Test_TextWrap_FirstLongWord()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FixedContentEditor editor = new FixedContentEditor(page);
            editor.TextProperties.FontSize = FontSize;
            editor.TextProperties.BlockWidth = 100;
            editor.DrawText("wrappingdsadsadsadasdasd");

            Assert.AreEqual(2, page.Content.Count);
            Assert.AreEqual("wrappingdsads", ((TextFragment)page.Content[0]).Text);
            Assert.AreEqual("adsadasdasd", ((TextFragment)page.Content[1]).Text);
        }

        [TestMethod]
        public void Test_TextWrap_LongWord()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FixedContentEditor editor = new FixedContentEditor(page);
            editor.TextProperties.FontSize = FontSize;
            editor.TextProperties.BlockWidth = 100;
            editor.DrawText("ala bala wrappingdsadsadsadasdasd");

            Assert.AreEqual(3, page.Content.Count);
            Assert.AreEqual("ala bala", ((TextFragment)page.Content[0]).Text);
            Assert.AreEqual("wrappingdsads", ((TextFragment)page.Content[1]).Text);
            Assert.AreEqual("adsadasdasd", ((TextFragment)page.Content[2]).Text);
        }

        [TestMethod]
        public void Test_TextWrap_MeasureText()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FixedContentEditor editor = new FixedContentEditor(page);
            editor.TextProperties.TrySetFont(new System.Windows.Media.FontFamily("Calibri"));
            editor.TextProperties.FontSize = FontSize;
            editor.TextProperties.BlockWidth = 280;
            string text = "CPOE for Medication Orders: Use computerized provider order entry\n(CPOE) for medication orders directly entered by any licensed\nhealthcare professional who can enter orders into the medical record\nper state, local and professional guidelines.";

            Size size = editor.MeasureText(text);

            AssertHelper.AssertAreEqual(size.Width, 270.73046875);
            AssertHelper.AssertAreEqual(size.Height, 102.5390625);
        }

        [TestMethod]
        public void Test_TextWrap_MeasureText_SingleLine()
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FixedContentEditor editor = new FixedContentEditor(page);
            editor.TextProperties.TrySetFont(new System.Windows.Media.FontFamily("Calibri"));
            editor.TextProperties.FontSize = FontSize;
            editor.TextProperties.BlockWidth = 280;
            string text = "ala";

            Size size = editor.MeasureText(text);

            AssertHelper.AssertAreEqual(size.Width, 16.625);
            AssertHelper.AssertAreEqual(size.Height, 17.08984375);
        }
    }
}
