﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Fonts
{
    public class FontTestBase : IntegrationTestClassBase
    {
        protected static RadFixedDocument CreateDocumentWithTextFragment(FontFamily fontFamily, FontStyle fontStyle, FontWeight fontWeight, string text = "Alabala")
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            FontBase font;
            if (!FontsRepository.TryCreateFont(fontFamily, fontStyle, fontWeight, out font))
            {
                Assert.Fail("Cannot create font.");
            }

            CreateTextFragment(page, font, text);

            return document;
        }

        protected static RadFixedDocument CreateDocumentWithTextFragment(string fontName)
        {
            TextFragment textFragment;

            return CreateDocumentWithTextFragment(fontName, out textFragment);
        }

        protected static RadFixedDocument CreateDocumentWithTextFragment(string fontName, out TextFragment textFragment, string text = "Alabala")
        {
            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();

            StandardFont font;
            if (!FontsRepository.TryCreateStandardFont(fontName, out font))
            {
                Assert.Fail("Cannot create font.");
            }

            textFragment = CreateTextFragment(page, font, text);

            return document;
        }

        private static TextFragment CreateTextFragment(RadFixedPage page, FontBase font, string text)
        {
            //FixedContentEditor editor = new FixedContentEditor(page);
            //editor.TextState.Font = font;
            //editor.DrawText(text);

            TextFragment textFragment = page.Content.AddTextFragment();
            textFragment.Position.Translate(30, 30);
            textFragment.Text = text;
            textFragment.Font = font;
            textFragment.Fill = new RgbColor() { R = 123 };
            textFragment.Stroke = new RgbColor() { B = 255 };
            textFragment.RenderingMode = RenderingMode.FillAndStroke;
            textFragment.HorizontalScaling = 2;
            textFragment.WordSpacing = 5;
            textFragment.CharacterSpacing = 3;
            textFragment.TextRise = 23;

            return textFragment;
        }
    }
}
