﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Graphics;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Graphics
{
    public class GraphicsTestBase : IntegrationTestClassBase
    {
        protected static RadFixedDocument CreateDocumentWithSamplePath(out Path path)
        {
            RadFixedDocument document = CreateDocumentWithPath(out path);

            path.Geometry = CreatePathGeometry(
                    new Point(3, 5),
                    true,
                    new LineSegment()
                    {
                        Point = new Point(34, 54)
                    },
                    new BezierSegment()
                    {
                        Point1 = new Point(455, 34),
                        Point2 = new Point(34, 444),
                        Point3 = new Point(58, 500)
                    });


            return document;
        }

        protected static RadFixedDocument CreateDocumentWithRectangleGeometries(params Rect[] rectangles)
        {
            RadFixedDocument document = new RadFixedDocument();
            GraphicsTestBase.CreatePageWithRectangleGeometries(document, rectangles);
             
            return document;
        }

        protected static void CreatePageWithRectangleGeometries(RadFixedDocument doc, params Rect[] rectangles)
        {
            RadFixedPage page = doc.Pages.AddPage();
            foreach (var rect in rectangles)
            {
                var path = page.Content.AddPath();
                path.Geometry = new RectangleGeometry()
                {
                    Rect = rect
                };

                path.Fill = new RgbColor(255, 0, 0); 
            } 
        }

        protected static RadFixedDocument CreateDocumentWithSegments(FillRule fillRule, out Path path, params PathSegment[] segments)
        {
            RadFixedDocument document = new RadFixedDocument();
            var page = document.Pages.AddPage();
            path = page.Content.AddPath();
            var pathGeometry = new PathGeometry();
            pathGeometry.FillRule = fillRule;
            var figure = pathGeometry.Figures.AddPathFigure();

            foreach (var segment in segments)
            {
                figure.Segments.Add(segment);
            }

            figure.StartPoint = new Point(12, 72);

            path.Geometry = pathGeometry;
            return document;
        }

        protected static RadFixedDocument CreateDocumentWithSegments(out Path path, params PathSegment[] segments)
        {
            return CreateDocumentWithSegments(FillRule.Nonzero, out path, segments);
        }

        protected static RadFixedDocument CreateDocumentWithPath(out Path path)
        {
            RadFixedDocument document = new RadFixedDocument();
            var page = document.Pages.AddPage();
            path = page.Content.AddPath();

            return document;
        }

        protected static RadFixedDocument CreateDocumentWithPage(out RadFixedPage page)
        {
            RadFixedDocument document = new RadFixedDocument();
            page = document.Pages.AddPage();

            return document;
        }

        protected static PathGeometry CreatePathGeometry(Point startPoint, bool isClosed, params PathSegment[] segments)
        {
            var pathGeometry = new PathGeometry();
            var pathFigure = pathGeometry.Figures.AddPathFigure();
            pathFigure.IsClosed = isClosed;
            pathFigure.StartPoint = startPoint;
            foreach (var segment in segments)
            {
                pathFigure.Segments.Add(segment);
            }

            return pathGeometry;
        } 
    }
}
