﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Objects;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Images
{
    [TestClass]
    public class TestExportImportImages : TestImageBase
    {
        private const int ImagesCount = 3; 

        [TestMethod]
        public void Test_Image_ExportImport_WithoutTransformations()
        {
            for (int i = 0; i < ImagesCount; i++)
            {
                TestImportExport(CreateImageName(i));
            }
        }

        [TestMethod]
        public void Test_Image_ExportImport_ScaledPosition()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.Width = 100;
            image.Height = 50;
            image.Position.Scale(4.6, 0.3);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Image_ExportImport_RotatedPosition()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.Position.Rotate(-43); 

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Image_ExportImport_TranslatedPosition()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.Position.Translate(47, 3.44);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_Image_ExportImport_CombinedTransformations()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.Position.Scale(8.3, 2.3);
            image.Position.Translate(47, 3.44);
            image.Position.Rotate(84.3);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_ExportImport_Png()
        {
            Image image;
            var document = CreateDocumentWithImage(out image, "1.png");

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_ExportImport_Jpg()
        {
            Image image;
            var document = CreateDocumentWithImage(out image, "1.jpg");

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Test_ExportImport_Bmp()
        {
            Image image;
            var document = CreateDocumentWithImage(out image, "1.bmp");

            ExportImportAndAssert(document);
        }

        private void TestImportExport(string name)
        {
            Image image;
            var document = CreateDocumentWithImage(out image, name);

            ExportImportAndAssert(document);
        }
    }
}
