﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using System.Windows;
using System.Windows.Media.Imaging;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Extensions;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Primitives;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Images
{
    [TestClass]
    public class TestImageMasks : TestImageBase
    {
        [TestMethod]
        public void TestImageMaskWhenMaskIsBigger()
        {
            TestImageMasks.TestImageMask(3, 5, 42, 9);
        }

        [TestMethod]
        public void TestImageMaskWhenMaskIsSmaller()
        {
            TestImageMasks.TestImageMask(42, 9, 3, 5);
        }

        [TestMethod]
        public void TestImageMaskWhenMaskWidthIsBiggerAndMaskHeightIsSmaller()
        {
            TestImageMasks.TestImageMask(3, 9, 42, 5);
        }

        [TestMethod]
        public void TestImageMaskWhenMaskHeightIsBiggerAndMaskWidthIsSmaller()
        {
            TestImageMasks.TestImageMask(42, 5, 3, 9);
        }

        [TestMethod]
        public void TestImageMaskWhenWidthsAreEqualAndMaskHeightIsBigger()
        {
            TestImageMasks.TestImageMask(42, 5, 42, 9);
        }

        [TestMethod]
        public void TestImageMaskWhenWidthsAreEqualAndMaskHeightIsSmaller()
        {
            TestImageMasks.TestImageMask(42, 9, 42, 5);
        }

        [TestMethod]
        public void TestImageMaskWhenHeightsAreEqualAndMaskWidthIsBigger()
        {
            TestImageMasks.TestImageMask(3, 9, 42, 9);
        }

        [TestMethod]
        public void TestImageMaskWhenHeightsAreEqualAndMaskWidthIsSmaller()
        {
            TestImageMasks.TestImageMask(42, 9, 3, 9);
        }

        [TestMethod]
        public void TestImageMaskBothWidthAndHeightAreEqual()
        {
            TestImageMasks.TestImageMask(42, 9, 42, 9);
        }

        [TestMethod]
        public void TestReadingImageMaskBitsWhenWidthIsMultipleOfEight()
        {
            TestImageMasks.TestReadingImageMaskBits(true);
        }

        [TestMethod]
        public void TestReadingImageMaskBitsWhenWidthIsNotMultipleOfEight()
        {
            TestImageMasks.TestReadingImageMaskBits(false);
        }

        [TestMethod]
        public void TestColorKeyMaskRgbColorSpace()
        {
            byte[] colorRanges = new byte[]
            {
                10, 170,
                50, 200,
                80, 240
            };

            this.TestColorKeyMask(colorRanges, GetRgbComponents);
        }

        [TestMethod]
        public void TestColorKeyMaskIndexedColorSpace()
        {
            byte[] colorRanges = new byte[]
            {
                50, 200,
            };

            this.TestColorKeyMask(colorRanges, GetIndexedComponents);
        }

        [TestMethod]
        public void TestImportExportImageMask_WhenImageMaskIsRenderedAfterStencilColorRendering()
        {
            RadFixedDocument document = Import(TestDocumentsPath + "ImageMask after Stencil color images.pdf");
            AssertDocumentWithImageMaskRenderedAfterStencilColorRendering(document);

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void TestImportExportImageMask_WhenImageMaskIsRenderedBeforeStencilColorRendering()
        {
            RadFixedDocument document = Import(TestDocumentsPath + "ImageMask before Stencil color images.pdf");
            AssertDocumentWithImageMaskRenderedBeforeStencilColorRendering(document);

            ExportImportAndAssert(document);
        }

        private static void AssertDocumentWithImageMaskRenderedAfterStencilColorRendering(RadFixedDocument document)
        {
            Assert.AreEqual(1, document.Pages.Count);

            RadFixedPage page = document.Pages[0];
            Assert.AreEqual(8, page.Content.Count);

            AssertStencilMaskedImage(2, 655, 983, new RgbColor(255, 0, 0), page);
            AssertStencilMaskedImage(3, 655, 983, new RgbColor(0, 0, 0), page);
            AssertStencilMaskedImage(4, 655, 983, new RgbColor(0, 128, 0), page);
            AssertStencilMaskedImage(5, 655, 983, new RgbColor(0, 0, 255), page);
            AssertThreeColoredImageMaskedFlag(6, 655, 983, new Color[] { new Color(255, 255, 255), new Color(0, 128, 0), new Color(255, 0, 0) }, page);
            AssertThreeColoredImageMaskedFlag(7, 655, 983, new Color[] { new Color(0, 0, 0), new Color(208, 0, 0), new Color(255, 206, 0) }, page);
        }

        private static void AssertDocumentWithImageMaskRenderedBeforeStencilColorRendering(RadFixedDocument document)
        {
            Assert.AreEqual(1, document.Pages.Count);

            RadFixedPage page = document.Pages[0];
            Assert.AreEqual(8, page.Content.Count);

            AssertThreeColoredImageMaskedFlag(2, 655, 983, new Color[] { new Color(255, 255, 255), new Color(0, 128, 0), new Color(255, 0, 0) }, page);
            AssertStencilMaskedImage(3, 655, 983, new RgbColor(255, 0, 0), page);
            AssertStencilMaskedImage(4, 655, 983, new RgbColor(0, 0, 0), page);
            AssertStencilMaskedImage(5, 655, 983, new RgbColor(0, 128, 0), page);
            AssertStencilMaskedImage(6, 655, 983, new RgbColor(0, 0, 255), page);
            AssertThreeColoredImageMaskedFlag(7, 655, 983, new Color[] { new Color(0, 0, 0), new Color(208, 0, 0), new Color(255, 206, 0) }, page);
        }

        private static void AssertStencilMaskedImage(int elementIndex, int width, int height, ColorBase stencilColor, RadFixedPage page)
        {
            ContentElementBase element = page.Content[elementIndex];

            Assert.AreEqual(FixedDocumentElementType.Image, element.ElementType);

            Fixed.Model.Objects.Image image = (Fixed.Model.Objects.Image)element;
            Assert.IsNotNull(image.ImageSource);
            Assert.IsTrue(image.ImageSource.ImageMask.HasValue);
            Assert.IsTrue(image.ImageSource.ImageMask.Value);
            Assert.AreEqual(width, image.Width);
            Assert.AreEqual(height, image.Height);
            Assert.AreEqual(stencilColor, image.StencilColor);
        }

        private static void AssertThreeColoredImageMaskedFlag(int elementIndex, int maskWidth, int maskHeight, Color[] colors, RadFixedPage page)
        {
            ContentElementBase element = page.Content[elementIndex];

            Assert.AreEqual(FixedDocumentElementType.Image, element.ElementType);

            Fixed.Model.Objects.Image image = (Fixed.Model.Objects.Image)element;
            Assert.IsNotNull(image.ImageSource);
            Assert.IsTrue(image.ImageSource.ImageMask.HasValue);
            Assert.IsFalse(image.ImageSource.ImageMask.Value);
            Assert.AreEqual(1, image.Width);
            Assert.AreEqual(3, image.Height);
            Assert.IsTrue(image.ImageSource.Mask.HasValue);
            Assert.IsFalse(image.ImageSource.Mask.Value.IsColorKeyMask);
            Assert.IsTrue(image.ImageSource.Mask.Value.Image.HasValue);

            ImageSource mask = image.ImageSource.Mask.Value.Image.Value;
            Assert.IsTrue(mask.ImageMask.HasValue);
            Assert.IsTrue(mask.ImageMask.Value);
            Assert.AreEqual(maskWidth, mask.Width);
            Assert.AreEqual(maskHeight, mask.Height);

            BitmapSource maskSource = mask.GetBitmapSource();
            Assert.AreEqual(maskWidth, maskSource.PixelWidth);
            Assert.AreEqual(maskHeight, maskSource.PixelHeight);

            BitmapSource resultSource = image.GetBitmapSource();
            Assert.AreEqual(maskWidth, resultSource.PixelWidth);
            Assert.AreEqual(maskHeight, resultSource.PixelHeight);

            BitmapSource originalFlagSource = image.ImageSource.GetBitmapSource();
            Assert.AreEqual(maskWidth, originalFlagSource.PixelWidth);
            Assert.AreEqual(maskHeight, originalFlagSource.PixelHeight);

            ImageMask maskValue = image.ImageSource.Mask.Value;
            ImageSource unmaskedOriginalSource = image.ImageSource;
            unmaskedOriginalSource.Mask.Value = null;

            BitmapSource unmaskedFlagSource = unmaskedOriginalSource.GetBitmapSource();
            Assert.AreEqual(1, unmaskedFlagSource.Width);
            Assert.AreEqual(3, unmaskedFlagSource.Height);

            Color[] actualColors = ImageSourceExtensions.GetPixels(unmaskedFlagSource).Select(p => Color.FromPixel(p)).ToArray();
            Assert.AreEqual(colors.Length, actualColors.Length);

            for (int i = 0; i < colors.Length; i++)
            {
                Color expected = colors[i];
                Color actual = actualColors[i];
                Assert.AreEqual(expected, actual);
            }

            image.ImageSource.Mask.Value = maskValue;
        }

        private static void TestImageMask(int imageWidth, int imageHeight, int maskWidth, int maskHeight)
        {
            PdfContentManager manager = TestUtils.ContentManager;
            byte alternatingByte = 1 + 4 + 16 + 64;
            byte inverseAlternatingByte = (byte)(~alternatingByte);

            PixelContainer pixels = TestImageMasks.CreateInterpolatedImagePixels(imageWidth, imageHeight);
            XImage xImage = new XImage(manager);
            xImage.Width = new PdfIntOld(manager, imageWidth);
            xImage.Height = new PdfIntOld(manager, imageHeight);

            int maskBytesPerRow = 1 + (maskWidth - 1) / 8;
            byte[] maskBytes = new byte[maskHeight * maskBytesPerRow];

            for (int row = 0; row < maskHeight; row++)
            {
                byte rowByte = row % 2 == 0 ? alternatingByte : inverseAlternatingByte;

                for (int byteColumn = 0; byteColumn < maskBytesPerRow; byteColumn++)
                {
                    maskBytes[row * maskBytesPerRow + byteColumn] = rowByte;
                }
            }

            ImageMaskOld mask = new ImageMaskOld(manager, maskBytes);
            PdfDictionaryOld maskDictionary = new PdfDictionaryOld(manager);
            maskDictionary.Add("Width", new PdfIntOld(manager, maskWidth));
            maskDictionary.Add("Height", new PdfIntOld(manager, maskHeight));
            mask.Load(maskDictionary);

            Size expectedSize = new Size(Math.Max(imageWidth, maskWidth), Math.Max(imageHeight, maskHeight));
            Size imageScale = new Size(imageWidth / expectedSize.Width, imageHeight / expectedSize.Height);
            Size maskScale = new Size(maskWidth / expectedSize.Width, maskHeight / expectedSize.Height);
            PixelContainer maskedPixels = mask.MaskImage(xImage, pixels);
            SizeI maskedImageSize = mask.GetMaskedImageSize(xImage);

            Assert.AreEqual(expectedSize.Width, maskedImageSize.Width);
            Assert.AreEqual(expectedSize.Height, maskedImageSize.Height);
            Assert.AreEqual(maskedPixels.PixelCount, maskedImageSize.Width * maskedImageSize.Height);

            for (int row = 0; row < maskHeight; row++)
            {
                for (int column = 0; column < maskWidth; column++)
                {
                    int maskRow = (int)(row * maskScale.Height);
                    int maskColumn = (int)(column * maskScale.Width);
                    int imageRow = (int)(row * imageScale.Height);
                    int imageColumn = (int)(column * imageScale.Width);

                    bool isMasked = (maskRow + maskColumn) % 2 == 1;
                    Color expectedColor = isMasked ? Color.Transparent : GetInterpolatedColor(imageRow, imageColumn, imageWidth, imageHeight);
                    int maskedPixelIndex = row * maskedImageSize.Width + column;
                    Color actualColor = maskedPixels.GetColorFromIndex(maskedPixelIndex);

                    Assert.AreEqual(expectedColor, actualColor);
                }
            }
        }

        private void TestColorKeyMask(byte[] colorRanges, Func<Color, int[]> getComponentsFromColor)
        {
            PdfContentManager manager = TestUtils.ContentManager;

            int imageWidth = 800;
            int imageHeight = 600;
            XImage xImage = new XImage(manager);
            xImage.Width = new PdfIntOld(manager, imageWidth);
            xImage.Height = new PdfIntOld(manager, imageHeight);

            object[] colorRangesObjects = new object[colorRanges.Length];

            for (int i = 0; i < colorRangesObjects.Length; i++)
            {
                colorRangesObjects[i] = colorRanges[i];
            }

            PixelContainer pixels = TestImageMasks.CreateInterpolatedImagePixels(imageWidth, imageHeight);
            ColorKeyMaskOld colorMask = new ColorKeyMaskOld(manager, new PdfArrayOld(manager, colorRangesObjects));
            PixelContainer maskedPixels = GetMaskedPixels(colorMask, xImage, pixels, getComponentsFromColor);

            for (int row = 0; row < imageHeight; row++)
            {
                for (int column = 0; column < imageWidth; column++)
                {
                    Color color = TestImageMasks.GetInterpolatedColor(row, column, imageWidth, imageHeight);
                    bool isMasked = TestImageMasks.ShouldMaskColor(color, colorRanges, getComponentsFromColor);

                    Color expectedColor = isMasked ? Color.Transparent : color;
                    int pixelIndex = row * imageWidth + column;
                    Color actualColor = maskedPixels.GetColorFromIndex(pixelIndex);

                    Assert.AreEqual(expectedColor, actualColor);
                }
            }
        }

        private static bool ShouldMaskColor(Color color, byte[] colorRanges, Func<Color, int[]> getComponentsFromColor)
        {
            int[] components = getComponentsFromColor(color);
            int count = colorRanges.Length / 2;

            for (int i = 0; i < count; i++)
            {
                int component = components[i];

                if (component < colorRanges[i * 2] || colorRanges[i * 2 + 1] < component)
                {
                    return false;
                }
            }

            return true;
        }

        private static PixelContainer CreateInterpolatedImagePixels(int width, int height)
        {
            PixelContainer pixels = PixelContainerFactory.GetMaskedImagePixelContainer(width, height);

            for (int row = 0; row < height; row++)
            {
                for (int column = 0; column < width; column++)
                {
                    int pixelIndex = row * width + column;
                    Color interpolatedColor = TestImageMasks.GetInterpolatedColor(row, column, width, height);
                    pixels.SetColorToIndex(interpolatedColor, pixelIndex);
                }
            }

            return pixels;
        }

        private static Color GetInterpolatedColor(int row, int column, int width, int height)
        {
            Point red = new Point(0, 0);
            Point green = new Point(width, 0);
            Point blue = new Point(width / 2.0, height);

            var coordinates = GetBarycentricCoordinates(new Point(column, row), red, green, blue);

            Color color = new Color(
                GetColorFromBarycentricCoordinate(coordinates.Item1),
                GetColorFromBarycentricCoordinate(coordinates.Item2),
                GetColorFromBarycentricCoordinate(coordinates.Item3));

            return color;
        }

        private static byte GetColorFromBarycentricCoordinate(double coordinate)
        {
            return (byte)(Math.Max(0, Math.Min(1, coordinate)) * 255);
        }

        private static Tuple<double, double, double> GetBarycentricCoordinates(Point point, Point a, Point b, Point c)
        {
            double area = Vector.CrossProduct(b - a, c - a);

            Tuple<double, double, double> coordinates = new Tuple<double, double, double>(
                Vector.CrossProduct(b - a, point - a) / area,
                Vector.CrossProduct(c - b, point - b) / area,
                Vector.CrossProduct(a - c, point - c) / area);

            return coordinates;
        }

        private static void TestReadingImageMaskBits(bool isWidthMultipleOfEigth)
        {
            byte nonZeroByte = 1 + 4 + 16 + 64;
            byte[] data = 
            {
                0, 0,
                0, nonZeroByte,
                0, 0 
            };
            int imageWidth = isWidthMultipleOfEigth ? 16 : 13;
            int imageHeight = 2;
            int nonZeroRow = 1;
            int nonZeroByteColumnStart = 8;
            int nonZeroByteColumnEnd = 15;
            ImageBitReader bitReader = new ImageBitReader(data, 1, imageWidth);

            Action<bool> assertBits = (areOddBitsNonZero) =>
            {
                for (int row = 0; row < imageHeight; row++)
                {
                    for (int column = 0; column < imageWidth; column++)
                    {
                        int bit = bitReader.GetBitAt(row, column);

                        bool isNonZeroRow = row == nonZeroRow;
                        bool isNonZeroByte = nonZeroByteColumnStart <= column && column <= nonZeroByteColumnEnd;
                        bool isOddBit = (column % 8) % 2 == 1;
                        bool isNonZeroBit = !(areOddBitsNonZero ^ isOddBit);
                        int expectedBit = isNonZeroRow && isNonZeroByte && isNonZeroBit ? 1 : 0;

                        Assert.AreEqual(bit, expectedBit);
                    }
                }
            };

            assertBits(true);
            data[3] = (byte)(~nonZeroByte);
            assertBits(false);
        }

        private static PixelContainer GetMaskedPixels(ColorKeyMaskOld mask, XImage xImage, PixelContainer pixels, Func<Color, int[]> getComponentsFromColor)
        {
            SizeI maskedSize = mask.GetMaskedImageSize(xImage);
            int pixelsCount = maskedSize.Width * maskedSize.Height;
            PixelContainer maskedPixels = PixelContainerFactory.GetMaskedImagePixelContainer(maskedSize.Width, maskedSize.Height);

            Assert.AreEqual(pixels.PixelCount, pixelsCount);
            Assert.AreEqual(xImage.Width.Value, maskedSize.Width);
            Assert.AreEqual(xImage.Height.Value, maskedSize.Height);

            for (int i = 0; i < pixelsCount; i++)
            {
                Color color = pixels.GetColorFromIndex(i);
                Color maskedColor = mask.ShouldMaskColorComponents(getComponentsFromColor(color)) ? Color.Transparent : color;
                maskedPixels.SetColorToIndex(maskedColor, i); 
            }

            return maskedPixels;
        }

        private static int[] GetIndexedComponents(Color color)
        {
            return new int[] { color.G };
        }

        private static int[] GetRgbComponents(Color color)
        {
            return new int[] { color.R, color.G, color.B };
        }
    }
}
