﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.Images
{
    [TestClass]
    public class TestImageSource : TestImageBase
    {
        [TestMethod]
        public void Decode_ExportImport()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.ImageSource.DecodeArray = new double[] { 0, 1, 0, 1, 0, 1 };

            ExportImportAndAssert(document);
        }

        [TestMethod]
        public void Decode_Import()
        {
            double[] expectedDecode = new double[] { 0, 1, 0, 1, 0, 1 };

            RadFixedDocument imported = Import(TestDocumentsPath + "ImageSourceDecodeNormal.pdf");
            Image image = (Image)imported.Pages.First().Content.Where(c => c.ElementType == FixedDocumentElementType.Image).FirstOrDefault();

            AssertDecodeArrays(expectedDecode, image.ImageSource.DecodeArray);
        }

        [TestMethod]
        public void Decode_WhenIsInvered_Import()
        {
            double[] expectedDecode = new double[] { 1, 0, 1, 0, 1, 0 };

            RadFixedDocument imported = Import(TestDocumentsPath + "ImageSourceDecodeInverted.pdf");
            Image image = (Image)imported.Pages.First().Content.Where(c => c.ElementType == FixedDocumentElementType.Image).FirstOrDefault();

            AssertDecodeArrays(expectedDecode, image.ImageSource.DecodeArray);
        }

        [TestMethod]
        public void Decode_WhenHasRandomValues_Import()
        {
            double[] expectedDecode = new double[] { 0, 0.5, 0.5, 2, 1, 2 };

            RadFixedDocument imported = Import(TestDocumentsPath + "ImageSourceDecodeRandom.pdf");
            Image image = (Image)imported.Pages.First().Content.Where(c => c.ElementType == FixedDocumentElementType.Image).FirstOrDefault();

            AssertDecodeArrays(expectedDecode, image.ImageSource.DecodeArray);
        }

        [TestMethod]
        public void Decode_WhenHasIncorrectElementsCount_ElementsArePreserved_ExportImport()
        {
            Image image;
            var document = CreateDocumentWithImage(out image);
            image.ImageSource.DecodeArray = new double[] { 0 };

            ExportImportAndAssert(document);
        }

        private static void AssertDecodeArrays(double[] expected, double[] actual)
        {
            Assert.AreEqual(expected.Length, actual.Length);
            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual(expected[i], actual[i]);
            }
        }
    }
}