﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    [TestClass]
    public class TestComboBoxField : TestTwoOppositeFieldValuesBase<ComboBoxField>
    {
        protected override string NamePrefix
        {
            get
            {
                return "combo";
            }
        }

        [TestMethod]
        public void TestComboBoxFieldProperties()
        {
            this.GenerateAssertExportAndImportSinglePageDocument();
        }

        protected override ComboBoxField CreateField(string name, bool isFirstCase)
        {
            ComboBoxField field = new ComboBoxField(name);
            field.Options.Add(new ChoiceOption("Option 1") { UserInterfaceValue = "UI Option 1" });
            field.Options.Add(new ChoiceOption("Option 2") { UserInterfaceValue = "UI Option 2" });
            field.Options.Add(new ChoiceOption("Option 3") { UserInterfaceValue = "UI Option 3" });

            field.HasEditableTextBox = isFirstCase;
            field.ShouldSpellCheck = !isFirstCase;

            if (isFirstCase)
            {
                field.Value = field.Options[1];
            }
            else
            {
                field.DefaultValue = field.Options[2];
            }

            return field;
        }

        protected override void AssertComboBox(ComboBoxField field)
        {
            bool isFirstCase = this.AssertFieldNameAndGetIfFirstCase(field);
            Assert.AreEqual(3, field.Options.Count);

            for (int i = 0; i < field.Options.Count; i++)
            {
                ChoiceOption option = field.Options[i];
                string expectedOptionValue = string.Format("Option {0}", i + 1);
                string expectedOptionUIValue = string.Format("UI {0}", expectedOptionValue);
                Assert.AreEqual(expectedOptionValue, option.Value);
                Assert.AreEqual(expectedOptionUIValue, option.UserInterfaceValue);
            }

            Assert.AreEqual(isFirstCase, field.HasEditableTextBox);
            Assert.AreEqual(!isFirstCase, field.ShouldSpellCheck);

            ChoiceOption expectedValue = isFirstCase ? field.Options[1] : null;
            ChoiceOption expectedDefaultValue = isFirstCase ?  null : field.Options[2];

            Assert.AreEqual(expectedValue, field.Value);
            Assert.AreEqual(expectedDefaultValue, field.DefaultValue);
        }
    }
}
