﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    [TestClass]
    public class TestExportImportFieldCollection : TestAcroFormBase
    {
        private const string AllFormFieldsTestDocumentPath = "Documents/InteractiveForms/AllFormFields.pdf";
        private const string AllFormFieldsMultiplePagesTestDocumentPath = "Documents/InteractiveForms/AllFormFieldsMultiplePages.pdf";

        [TestMethod]
        public void TestGenerationExportAndImportOfAllFormFields()
        {
            RadFixedDocument document = this.GenerateDocumentWithAllFields(1);
            this.AssertFieldsWidgetsExportAndImport(document);
        }

        [TestMethod]
        public void TestGenerationExportAndImportOfAllFormFieldsInMultiplePages()
        {
            RadFixedDocument document = this.GenerateDocumentWithAllFields(3);
            this.AssertFieldsWidgetsExportAndImport(document);
        }

        [TestMethod]
        public void TestImportAndGenerationOfAllFormFields()
        {
            RadFixedDocument importedDocument = Import(AllFormFieldsTestDocumentPath);
            RadFixedDocument generatedDocument = this.GenerateDocumentWithAllFields(1);
            AssertDocument(generatedDocument, importedDocument);
        }

        [TestMethod]
        public void TestImportAndGenerationOfAllFormFieldsMultiplePages()
        {
            RadFixedDocument importedDocument = Import(AllFormFieldsMultiplePagesTestDocumentPath);
            RadFixedDocument generatedDocument = this.GenerateDocumentWithAllFields(3);
            AssertDocument(generatedDocument, importedDocument);
        }

        private RadFixedDocument GenerateDocumentWithAllFields(int numberOfPages)
        {
            RadFixedDocument document = new RadFixedDocument();
            this.CreateFields(document);

            for(int i = 0; i < numberOfPages; i++)
            {
                DrawPageWithWidgets(document.Pages.AddPage());
            }

            return document;
        }
    }
}
