﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    [TestClass]
    public class TestListBoxField : TestTwoOppositeFieldValuesBase<ListBoxField>
    {
        protected override string NamePrefix
        {
            get
            {
                return "list";
            }
        }

        [TestMethod]
        public void TestListBoxFieldProperties()
        {
            this.GenerateAssertExportAndImportSinglePageDocument();
        }

        protected override ListBoxField CreateField(string name, bool isFirstCase)
        {
            ListBoxField field = new ListBoxField(name);
            field.Options.Add(new ChoiceOption("Option 1") { UserInterfaceValue = "UI Option 1" });
            field.Options.Add(new ChoiceOption("Option 2") { UserInterfaceValue = "UI Option 2" });
            field.Options.Add(new ChoiceOption("Option 3") { UserInterfaceValue = "UI Option 3" });
            field.Options.Add(new ChoiceOption("Option 4") { UserInterfaceValue = "UI Option 4" });

            field.AllowMultiSelection = isFirstCase;

            if (isFirstCase)
            {
                field.Value = new ChoiceOption[] { field.Options[1], field.Options[2] };
            }
            else
            {
                field.DefaultValue = new ChoiceOption[] { field.Options[3] };
            }

            return field;
        }

        protected override void AssertListBox(ListBoxField field)
        {
            bool isFirstCase = this.AssertFieldNameAndGetIfFirstCase(field);
            Assert.AreEqual(4, field.Options.Count);

            for (int i = 0; i < field.Options.Count; i++)
            {
                ChoiceOption option = field.Options[i];
                string expectedOptionValue = string.Format("Option {0}", i + 1);
                string expectedOptionUIValue = string.Format("UI {0}", expectedOptionValue);
                Assert.AreEqual(expectedOptionValue, option.Value);
                Assert.AreEqual(expectedOptionUIValue, option.UserInterfaceValue);
            }

            ChoiceOption[] expectedValue = isFirstCase ? new ChoiceOption[] { field.Options[1], field.Options[2] } : null;
            ChoiceOption[] expectedDefaultValue = isFirstCase ? null : new ChoiceOption[] { field.Options[3] };

            AssertValue(expectedValue, field.Value);
            AssertValue(expectedDefaultValue, field.DefaultValue);
        }

        private void AssertValue(ChoiceOption[] expected, ChoiceOption[] value)
        {
            if (expected == null)
            {
                Assert.IsNull(value);
            }
            else
            {
                Assert.IsNotNull(value);
                Assert.AreEqual(expected.Length, value.Length);

                for(int i = 0; i < expected.Length; i++)
                {
                    ChoiceOption expectedOption = expected[i];
                    ChoiceOption valueOption = value[i];
                    Assert.AreEqual(expectedOption, valueOption);
                }
            }
        }
    }
}
