﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    [TestClass]
    public class TestRadioButtonField : TestTwoOppositeFieldValuesBase<RadioButtonField>
    {
        protected override string NamePrefix
        {
            get
            {
                return "radio";
            }
        }

        [TestMethod]
        public void TestRadioButtonFieldProperties()
        {
            this.GenerateAssertExportAndImportSinglePageDocument();
        }

        protected override RadioButtonField CreateField(string name, bool isFirstCase)
        {
            RadioButtonField field = new RadioButtonField(name);
            field.Options.Add(new RadioOption("Option 1"));
            field.Options.Add(new RadioOption("Option 2"));
            field.Options.Add(new RadioOption("Option 1"));
            field.Options.Add(new RadioOption("Option 2"));
            field.Options.Add(new RadioOption("Option 3"));

            field.AllowToggleOff = isFirstCase;
            field.ShouldUpdateRadiosInUnison = !isFirstCase;

            if(isFirstCase)
            {
                field.Value = field.Options[1];
            }
            else
            {
                field.DefaultValue = field.Options[0];
            }

            return field;
        }

        protected override void AssertRadioButton(RadioButtonField field)
        {
            bool isFirstCase = this.AssertFieldNameAndGetIfFirstCase(field);

            Assert.AreEqual(5, field.Options.Count);
            RadioOption[] options = field.Options.ToArray();
            Assert.AreEqual("Option 1", options[0].Value);
            Assert.AreEqual("Option 2", options[1].Value);
            Assert.AreEqual("Option 1", options[2].Value);
            Assert.AreEqual("Option 2", options[3].Value);
            Assert.AreEqual("Option 3", options[4].Value);

            Assert.AreEqual(isFirstCase, field.AllowToggleOff);
            Assert.AreEqual(!isFirstCase, field.ShouldUpdateRadiosInUnison);

            RadioOption expectedValue = isFirstCase ? options[1] : null;
            RadioOption expectedDefaultValue = isFirstCase ? null : options[0];

            Assert.AreEqual(expectedValue, field.Value);
            Assert.AreEqual(expectedDefaultValue, field.DefaultValue);
        }
    }
}
