﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    [TestClass]
    public class TestTextBoxField : TestTwoOppositeFieldValuesBase<TextBoxField>
    {
        protected override string NamePrefix
        {
            get
            {
                return "text";
            }
        }

        [TestMethod]
        public void TestTextBoxFieldProperties()
        {
            this.GenerateAssertExportAndImportSinglePageDocument();
        }

        protected override TextBoxField CreateField(string name, bool isFirstCase)
        {
            TextBoxField field = new TextBoxField(name);
            field.MaxLengthOfInputCharacters = isFirstCase ? 4 : (int?)null;
            field.IsMultiline = isFirstCase;
            field.IsFileSelect = !isFirstCase;
            field.IsPassword = !isFirstCase;
            field.IsReadOnly = isFirstCase;
            field.IsRequired = !isFirstCase;
            field.ShouldBeSkipped = isFirstCase;
            field.ShouldSpellCheck = !isFirstCase;
            field.UserInterfaceName = isFirstCase ? "Text UI" : null;
            field.MappingName = isFirstCase ? null : "custom mapping";
            field.AllowScroll = !isFirstCase;

            if(isFirstCase)
            {
                field.Value = "1234";
            }
            else
            {
                field.DefaultValue = "default";
            }

            return field;
        }

        protected override void AssertTextBox(TextBoxField field)
        {
            bool isFirstCase = this.AssertFieldNameAndGetIfFirstCase(field);

            Assert.AreEqual(isFirstCase ? 4 : (int?)null, field.MaxLengthOfInputCharacters);
            Assert.AreEqual(isFirstCase, field.IsMultiline);
            Assert.AreEqual(!isFirstCase, field.IsFileSelect);
            Assert.AreEqual(!isFirstCase, field.IsPassword);
            Assert.AreEqual(isFirstCase, field.IsReadOnly);
            Assert.AreEqual(!isFirstCase, field.IsRequired);
            Assert.AreEqual(isFirstCase, field.ShouldBeSkipped);
            Assert.AreEqual(!isFirstCase, field.ShouldSpellCheck);
            Assert.AreEqual(isFirstCase ? "Text UI" : null, field.UserInterfaceName);
            Assert.AreEqual(isFirstCase ? null : "custom mapping", field.MappingName);
            Assert.AreEqual(!isFirstCase, field.AllowScroll);

            string expectedValue = isFirstCase ? "1234" : null;
            string expectedDefaultValue = isFirstCase ? null : "default";

            Assert.AreEqual(expectedValue, field.Value);
            Assert.AreEqual(expectedDefaultValue, field.DefaultValue);
        }
    }
}
