﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.InteractiveForms;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration.ImportExport.InteractiveForms
{
    public abstract class TestTwoOppositeFieldValuesBase<T> : TestAcroFormBase
        where T : FormField
    {
        public void GenerateAssertExportAndImportSinglePageDocument()
        {
            RadFixedDocument document = new RadFixedDocument();
            this.CreateFields(document);
            DrawPageWithWidgets(document.Pages.AddPage());

            this.AssertFieldsWidgetsExportAndImport(document);
        }

        protected sealed override int ExpectedFieldsCount
        {
            get
            {
                return 2;
            }
        }

        protected abstract string NamePrefix { get; }

        protected sealed override void CreateFields(RadFixedDocument document)
        {
            for (int i = 0; i < this.ExpectedFieldsCount; i++)
            {
                bool isFirstCase = i == 0;
                string name = this.GetFieldNameByValuesCaseIndex(i);
                T field = this.CreateField(name, isFirstCase);
                document.AcroForm.FormFields.Add(field);
            }
        }

        protected bool AssertFieldNameAndGetIfFirstCase(T field)
        {
            int index = -1;

            for (int i = 0; i < this.ExpectedFieldsCount; i++)
            {
                string expectedName = GetFieldNameByValuesCaseIndex(i);

                if (field.Name.Equals(expectedName))
                {
                    index = i;
                    break;
                }
            }

            Assert.IsTrue(index > -1);
            bool isFirstCase = index == 0;

            return isFirstCase;
        }

        protected abstract T CreateField(string name, bool isFirstCase);

        private string GetFieldNameByValuesCaseIndex(int index)
        {
            return this.NamePrefix + (index + 1);
        }
    }
}
