﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Tests.Integration;

namespace Telerik.Windows.Documents.FixedDocumentViewers.Tests.Integration
{
    public class IntegrationOldTestClassBase
    {
        private const string LocalTestFolder = @"..\..\..\TestApplications\DocumentProcessingTestApplication\bin\WPF\Output";
#if !NETSTANDARD
        protected static RadFixedDocument ExportAndImport(RadFixedDocument expected, bool exportToFile = false, PdfImportSettings importSettings = null, PdfExportSettings exportSettings = null)
        {
            PdfFormatProvider provider = new PdfFormatProvider();

            if (importSettings != null)
            {
                provider.ImportSettings = importSettings;
            }

            if (exportSettings != null)
            {
                provider.ExportSettings = exportSettings;
            }

            if (exportToFile)
            {
                ExportToFile(expected);
            }

            using (MemoryStream stream = new MemoryStream())
            {
                provider.Export(expected, stream);

                PdfFormatProvider oldProvider = new PdfFormatProvider(stream, FormatProviderSettings.ReadOnDemand);

                return oldProvider.Import();
            }
        }

        protected static void ExportImportAndAssert(RadFixedDocument expected, bool exportToFile = false, PdfImportSettings importSettings = null, PdfExportSettings exportSettings = null)
        {
            RadFixedDocument document = ExportAndImport(expected, exportToFile, importSettings, exportSettings);
            AssertDocument(expected, document);
        }
#endif
        protected static void AssertDocument(RadFixedDocument expected, RadFixedDocument document)
        {
            AssertHelper.AssertDocument(expected, document, true);
        }

        internal static void AssertAnnotation(Annotation expected, Annotation annotation)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(annotation);
            Assert.AreEqual(expected.Type, annotation.Type);
            Assert.AreEqual(expected.Rect, annotation.BoundingRect);
            IntegrationTestClassBase.AssertAnnotationFlags(expected, annotation);

            switch (expected.Type)
            {
                case AnnotationType.Link:
                    IntegrationTestClassBase.AssertLinkInternal((Link)expected, (Link)annotation);
                    break;
                case AnnotationType.Widget:
                    // TODO:
                    break;
            }
        }

        private static void ExportToFile(RadFixedDocument document)
        {
            string fileName = DateTime.Now.ToString("yyyy.MM.dd hh-mm-ss.fff") + ".pdf";
            using (Stream stream = File.OpenWrite(System.IO.Path.Combine(LocalTestFolder, fileName)))
            {
                PdfFormatProvider provider = new PdfFormatProvider();
                provider.Export(document, stream);
            }
        }
    }
}
