﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.ColorSpaces
{
    [TestClass]
    public class DeviceCmykTests
    {
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetColor_WhenParsIsNull_ThrowsException()
        {
            DeviceCmyk deviceCmykColorSpace = new DeviceCmyk();
            ColorBase result = deviceCmykColorSpace.GetColor(null);
        }

        [TestMethod]
        public void GetColor_WhenParsIsNotNullAndNotEmpty_ReturnsCorrectRgbColor()
        {
            // Arrange
            double[] pars = new double[] { 1, 0, 1, 0, 0, 0 };
            Color c = Color.FromCmyk(pars[2], pars[3], pars[4], pars[5]);
            RgbColor expectedColorBase = new RgbColor(c);

            DeviceCmyk deviceCmykColorSpace = new DeviceCmyk();

            // Act
            ColorBase result = deviceCmykColorSpace.GetColor(pars);
            RgbColor rgbColor = result as RgbColor;

            // Assert
            Assert.IsNotNull(rgbColor);

            // the last four doubles from the pars are taken; this is a 4-component color space

            Assert.AreEqual(expectedColorBase, rgbColor);
        }
    }
}
