﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Core.Imaging;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.ColorSpaces
{
    [TestClass]
    public class IccBasedTests
    {
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetColor_WhenParsIsNull_ThrowsException()
        {
            IccBased iccBasedColorSpace = new IccBased();
            ColorBase result = iccBasedColorSpace.GetColor(null);
        }

        [TestMethod]
        public void GetColor_WhenParsIsNotNullAndNotEmpty_ReturnsCorrectRgbColor()
        {
            // Arrange
            double[] pars = new double[] { 1, 0 };
            RgbColor expectedColorBase = new RgbColor(255, 255, 255);

            IccBased iccBasedColorSpace = new IccBased()
            {
                Alternate = new DeviceGray()
            };

            // Act
            ColorBase result = iccBasedColorSpace.GetColor(pars);
            RgbColor rgbColor = result as RgbColor;

            // Assert
            Assert.IsNotNull(rgbColor);
            Assert.AreEqual(expectedColorBase, rgbColor);
        }

        [TestMethod]
        public void GetColor_WhenNoAlternateColorSpace_ReturnsCorrectRgbColor()
        {
            // Arrange
            double[] pars = new double[] { 0.2, 0.3, 0.4, 0.5 };
            Color color = Color.FromCmyk(0.2, 0.3, 0.4, 0.5);
            RgbColor expectedColorBase = new RgbColor(color);

            IccBased iccBasedColorSpace = new IccBased()
            {
                N = 4
            };

            // Act
            ColorBase result = iccBasedColorSpace.GetColor(pars);
            RgbColor rgbColor = result as RgbColor;

            // Assert
            Assert.IsNotNull(rgbColor);
            Assert.AreEqual(expectedColorBase, rgbColor);
        }
    }
}
