﻿using System;
#if !NETSTANDARD
using System.Windows;
#else
using Telerik.Documents.Primitives;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Collections;
using Telerik.Windows.Documents.Fixed.Model.Extensions;
using Telerik.Windows.Documents.Fixed.Tests.Model.Annotations;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Extensions
{
    [TestClass]
    public class AnnotationExtentionsTests
    {
        [TestMethod]
        public void TryGetAnnotationFromLocation_WhenAnnotationsContainLocation_ShouldReturnTrue()
        {
            // Arrange
            AnnotationCollection annotations = new AnnotationCollection(new RadFixedPage());

            Annotation first = new TestWidgetAnnotation();
            first.BoundingRect = new Rect(50, 50, 50, 50);
            annotations.Add(first);

            Annotation second = new TestWidgetAnnotation();
            second.BoundingRect = new Rect(200, 200, 50, 50);

            Point location = new Point(50, 50);
            Annotation annotation;

            // Act
            bool isFound = annotations.TryGetAnnotationFromLocation(location, out annotation);

            // Assert
            Assert.IsTrue(isFound);
            Assert.AreEqual(first, annotation);
        }

        [TestMethod]
        public void TryGetAnnotationFromLocation_WhenAnnotationsDoesNotContainLocation_ShouldReturnFalse()
        {
            // Arrange
            AnnotationCollection annotations = new AnnotationCollection(new RadFixedPage());

            Annotation first = new TestWidgetAnnotation();
            first.BoundingRect = new Rect(50, 50, 50, 50);
            annotations.Add(first);

            Annotation second = new TestWidgetAnnotation();
            second.BoundingRect = new Rect(200, 200, 50, 50);

            Point location = new Point();
            Annotation annotation;

            // Act
            bool isFound = annotations.TryGetAnnotationFromLocation(location, out annotation);

            // Assert
            Assert.IsFalse(isFound);
            Assert.IsNull(annotation);
        }

        [TestMethod]
        public void TryGetAnnotationFromLocation_WhenEmptyCollection_ShouldReturnFalse()
        {
            // Arrange
            AnnotationCollection annotations = new AnnotationCollection(new RadFixedPage());
            Point location = new Point();
            Annotation annotation;

            // Act
            bool isFound = annotations.TryGetAnnotationFromLocation(location, out annotation);

            // Assert
            Assert.IsFalse(isFound);
            Assert.IsNull(annotation);
        }
    }
}
