﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{

    [TestClass]
    public class CidType0FontTests
    {
        [TestMethod]
        public void TestCidType0Font_ActualFontSourceIsInitialFontSourceEvenWhenUnknownSourceType()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType0Font font = new CidType0Font("test", fontSource);
            FontSource actualFontSource = font.ActualFontSource;

            Assert.AreEqual(fontSource, actualFontSource);
        }

        [TestMethod]
        public void TestCidType0Font_ActualFontSourceIsInitialFontSourceEvenWhenEmptyFontSource()
        {
            CidType0Font font = new CidType0Font("test", EmptyFontSource.Instance);
            FontSource actualFontSource = font.ActualFontSource;

            Assert.AreEqual(EmptyFontSource.Instance, actualFontSource);
        }

        [TestMethod]
        public void TestCidType0Font_InitializeCoreGlyph_WhenUsesCIDFontOperators()
        {
            ushort bigCid = ushort.MaxValue - 5;
            TestCFFFontSource fontSource = new TestCFFFontSource(true);
            fontSource.AddGlyphIdMapping(bigCid, 123);
            CidType0Font font = new CidType0Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(bigCid, 2));

            Assert.IsNull(g.Name);
            Assert.AreEqual(123, g.GlyphId);
        }

        [TestMethod]
        public void TestCidType0Font_InitializeCoreGlyph_WithoutCIDFontOperators()
        {
            ushort bigCid = ushort.MaxValue - 5;
            TestCFFFontSource fontSource = new TestCFFFontSource(false);
            fontSource.AddGlyphIdMapping(bigCid, 123);
            CidType0Font font = new CidType0Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(bigCid, 2));

            Assert.IsNull(g.Name);
            Assert.AreEqual(bigCid, g.GlyphId);
        }

        [TestMethod]
        public void TestCidType0Font_InitializeCoreGlyph_GlyphIsNotInitializedWhenFontSourceInNotCFF()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType0Font font = new CidType0Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(149));

            Assert.IsNull(g.Name);
            Assert.AreEqual(0, g.GlyphId);
        }
    }
}
