﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;
using Telerik.Windows.Documents.Fixed.Model.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{
    [TestClass]
    public class CidType2FontTests
    {
        [TestMethod]
        public void TestCidType2Font_ActualFontSourceIsInitialFontSourceEvenWhenUnknownSourceType()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType2Font font = new CidType2Font("test", fontSource);
            FontSource actualFontSource = font.ActualFontSource;

            Assert.AreEqual(fontSource, actualFontSource);
        }

        [TestMethod]
        public void TestCidType2Font_ActualFontSourceIsInitialFontSourceEvenWhenEmptyFontSource()
        {
            CidType2Font font = new CidType2Font("test", EmptyFontSource.Instance);
            FontSource actualFontSource = font.ActualFontSource;

            Assert.AreEqual(EmptyFontSource.Instance, actualFontSource);
        }

        [TestMethod]
        public void TestCidType2Font_InitializeCoreGlyph_WithoutMapping()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType2Font font = new CidType2Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(134));

            Assert.IsNull(g.Name);
            Assert.AreEqual(134, g.GlyphId);
        }

        [TestMethod]
        public void TestCidType2Font_InitializeCoreGlyph_WithMapping()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType2Font font = new CidType2Font("test", fontSource);
            byte[] data = new byte[10];
            data[6] = 15;
            data[7] = 10;
            font.CharIdToGlyphIdMapping.Value = new CidToGidMap(data);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(3));

            Assert.IsNull(g.Name);
            Assert.AreEqual(2575, g.GlyphId);
        }

#if !NETSTANDARD
        [TestMethod]
        public void TestCidType2Font_ComputeSubset_ReturnsEmptyArrayWhenThereAreNoUsedCharacters()
        {
            TestFontSource fontSource = new TestFontSource();
            CidType2Font font = new CidType2Font("test", fontSource);
            byte[] subset = font.ComputeSubset(Enumerable.Empty<CharInfo>());

            Assert.IsNotNull(subset);
            Assert.AreEqual(0, subset.Length);
        }
#endif
    }
}
