﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts.Encoding
{
    [TestClass]
    public class CMapEncodingTests
    {
        [TestMethod]
        public void AddUnicodeRange_ShouldAddSequentialRanges()
        {
            // Arrange
            CMapEncoding cMapEncoding = new CMapEncoding();
            CharCode start = new CharCode(1);
            CharCode end = new CharCode(2);
            List<string> unicodeStrings = new List<string>() { "a", "b" };

            // Act
            cMapEncoding.AddUnicodeRange(start, end, unicodeStrings);

            // Assert
            List<CharCodeRange> charCodeRange = cMapEncoding.UnicodeRanges;
            Assert.AreEqual(1, charCodeRange.Count);

            CharCodeRange range = charCodeRange.First();
            Assert.IsTrue(range.IsSequential);
            Assert.AreEqual(start, range.Start);
            Assert.AreEqual(end, range.End);

            List<string> unicodes = charCodeRange.First().Unicodes;
            Assert.AreEqual(2, unicodes.Count);
            Assert.AreEqual("a", unicodes[0]);
            Assert.AreEqual("b", unicodes[1]);
        }

        [TestMethod]
        public void AddUnicodeRangeArray_ShouldAddNonSequentialRanges()
        {
            // Arrange
            CMapEncoding cMapEncoding = new CMapEncoding();
            CharCode start = new CharCode(1);
            CharCode end = new CharCode(2);
            List<string> unicodeStrings = new List<string>() { "a", "b" };

            // Act
            cMapEncoding.AddUnicodeRangeArray(start, end, unicodeStrings);

            // Assert
            Assert.AreEqual(1, cMapEncoding.UnicodeRanges.Count);

            CharCodeRange charCodeRange = cMapEncoding.UnicodeRanges.First();
            CharCodeRange range = charCodeRange;
            Assert.IsFalse(range.IsSequential);
            Assert.AreEqual(start, range.Start);
            Assert.AreEqual(end, range.End);

            List<string> unicodes = charCodeRange.Unicodes;
            Assert.AreEqual(2, unicodes.Count);
            Assert.AreEqual("a", unicodes[0]);
            Assert.AreEqual("b", unicodes[1]);
        }

        [TestMethod]
        public void TryGetToUnicode_WhenRangeAndValueIsInRange_ShouldReturnTrue()
        {
            // Arrange
            CMapEncoding cMapEncoding = new CMapEncoding();
            CharCode start = new CharCode(2);
            CharCode end = new CharCode(3);
            List<string> unicodeStrings = new List<string>() { "b", "c" };
            cMapEncoding.AddUnicodeRange(start, end, unicodeStrings);

            CharCode charCodeToFind1 = new CharCode(2);
            CharCode charCodeToFind2 = new CharCode(3);
            string unicode1, unicode2;

            // Act
            bool result1 = cMapEncoding.TryGetToUnicode(charCodeToFind1, out unicode1);
            bool result2 = cMapEncoding.TryGetToUnicode(charCodeToFind2, out unicode2);

            // Assert
            Assert.AreEqual(1, cMapEncoding.UnicodeRanges.Count);

            Assert.IsTrue(result1);
            Assert.AreEqual("b", unicode1);

            Assert.IsTrue(result2);
            Assert.AreEqual("c", unicode2);
        }

        [TestMethod]
        public void TryGetToUnicode_WhenRangeAndValueIsNotInRange_ShouldReturnFalse()
        {
            // Arrange
            CMapEncoding cMapEncoding = new CMapEncoding();
            CharCode start = new CharCode(2);
            CharCode end = new CharCode(3);
            List<string> unicodeStrings = new List<string>() { "b", "c" };
            cMapEncoding.AddUnicodeRange(start, end, unicodeStrings);

            CharCode charCodeToFind1 = new CharCode(1);
            CharCode charCodeToFind2 = new CharCode(4);
            string unicode1, unicode2;

            // Act
            bool result1 = cMapEncoding.TryGetToUnicode(charCodeToFind1, out unicode1);
            bool result2 = cMapEncoding.TryGetToUnicode(charCodeToFind2, out unicode2);

            // Assert
            Assert.AreEqual(1, cMapEncoding.UnicodeRanges.Count);

            Assert.IsFalse(result1);
            Assert.IsNull(unicode1);

            Assert.IsFalse(result2);
            Assert.IsNull(unicode2);
        }
    }
}
