﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts.Encoding
{
    [TestClass]
    public class CharCodeRangeTests
    {
        [TestMethod]
        public void Constructor_ShouldSetCorrectValues()
        {
            // Arrange
            CharCode start = new CharCode(1);
            CharCode end = new CharCode(2);
            List<string> unicodes = new List<string>();
            bool isSequential = true;

            // Act
            CharCodeRange range = new CharCodeRange(start, end, unicodes, isSequential);

            // Assert
            Assert.AreEqual(start, range.Start);
            Assert.AreEqual(end, range.End);
            Assert.AreEqual(unicodes, range.Unicodes);
            Assert.IsTrue(isSequential);
        }

        [TestMethod]
        public void CompareTo_WhenGreaterThan_ShouldReturnOne()
        {
            // Arrange
            CharCodeRange range1 = new CharCodeRange(new CharCode(3), new CharCode(4));
            CharCodeRange range2 = new CharCodeRange(new CharCode(1), new CharCode(2));

            // Act
            int result = range1.CompareTo(range2);

            // Assert
            Assert.AreEqual(1, result);
        }

        [TestMethod]
        public void CompareTo_WhenLessThan_ShouldReturnMinusOne()
        {
            // Arrange
            CharCodeRange range1 = new CharCodeRange(new CharCode(3), new CharCode(4));
            CharCodeRange range2 = new CharCodeRange(new CharCode(5), new CharCode(6));

            // Act
            int result = range1.CompareTo(range2);

            // Assert
            Assert.AreEqual(-1, result);
        }

        [TestMethod]
        public void CompareTo_WhenEqualRanges_ShouldReturnMinusOne()
        {
            // Arrange
            CharCodeRange range1 = new CharCodeRange(new CharCode(3), new CharCode(4));
            CharCodeRange range2 = new CharCodeRange(new CharCode(3), new CharCode(4));

            // Act
            int result = range1.CompareTo(range2);

            // Assert
            Assert.AreEqual(0, result);
        }

        [TestMethod]
        public void CompareTo_WhenIntercetingRanges_ShouldReturnZero()
        {
            // Arrange
            CharCodeRange range1 = new CharCodeRange(new CharCode(3), new CharCode(4));
            CharCodeRange range2 = new CharCodeRange(new CharCode(2), new CharCode(3));
            CharCodeRange range3 = new CharCodeRange(new CharCode(4), new CharCode(5));
            CharCodeRange range4 = new CharCodeRange(new CharCode(1), new CharCode(5));

            // Act
            int result1 = range1.CompareTo(range2);
            int result2 = range1.CompareTo(range3);
            int result3 = range1.CompareTo(range4);

            // Assert
            Assert.AreEqual(0, result1);
            Assert.AreEqual(0, result2);
            Assert.AreEqual(0, result3);
        }
    }
}
