﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Fonts.Type1.CFFFormat;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{
    internal class TestCFFFontSource : CFFFontSource
    {
        private readonly bool usesCIDFontOperators;
        private readonly Dictionary<int, ushort> unicodeToGlyphIdMapping;
        private readonly Dictionary<string, ushort> nameToGlyphIdMapping;
        private readonly Dictionary<ushort, string> cidMapping;

        public TestCFFFontSource()
            : this(false)
        {
        }

        public TestCFFFontSource(bool usesCIDFontOperators)
            : base(null, null)
        {
            this.usesCIDFontOperators = usesCIDFontOperators;
            this.unicodeToGlyphIdMapping = new Dictionary<int, ushort>();
            this.nameToGlyphIdMapping = new Dictionary<string, ushort>();
            this.cidMapping = new Dictionary<ushort, string>();
        }

        public override bool UsesCIDFontOperators
        {
            get
            {
                return this.usesCIDFontOperators;
            }
        }

        public override string GetGlyphName(ushort cid)
        {
            return this.cidMapping[cid];
        }

        public override bool TryGetGlyphId(string name, out ushort glyphId)
        {
            return this.nameToGlyphIdMapping.TryGetValue(name, out glyphId);
        }

        public override bool TryGetGlyphId(int unicode, out ushort glyphId)
        {
            return this.unicodeToGlyphIdMapping.TryGetValue(unicode, out glyphId);
        }

        public void AddGlyphIdMapping(int unicode, ushort glyphId)
        {
            this.unicodeToGlyphIdMapping.Add(unicode, glyphId);
        }

        public void AddGlyphIdMapping(string name, ushort glyphId)
        {
            this.nameToGlyphIdMapping.Add(name, glyphId);
        }

        public void AddCidIdMapping(ushort cid, string name)
        {
            this.cidMapping.Add(cid, name);
        }
    }
}
