﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{
    internal class TestFont : FontBase
    {
        private readonly FontSource actualFontSource;
        private readonly Dictionary<CharCode, int> glyphInitializationCounter;

        public TestFont(string name, FontSource fontSource, FontSource actualFontSource) 
            : base(name, fontSource)
        {
            this.actualFontSource = actualFontSource;
            this.glyphInitializationCounter = new Dictionary<CharCode, int>();
        }

        internal override FontType Type
        {
            get
            {
                throw new NotSupportedException();
            }
        }

        internal override FontSource ActualFontSource
        {
            get
            {
                this.ActualFontSourceCounter++;
                return this.actualFontSource;
            }
        }

        internal int ActualFontSourceCounter
        {
            get;
            private set;
        }

        internal int GetGlyphInitializationCount(CharCode charcode)
        {
            int count;
            if (!this.glyphInitializationCounter.TryGetValue(charcode, out count))
            {
                count = 0;
            }

            return count;
        }

        internal override void InitializeCoreGlyph(Glyph glyphInfo, CharCode charcode)
        {
            int count = this.GetGlyphInitializationCount(charcode);
            this.glyphInitializationCounter[charcode] = count + 1;

            glyphInfo.GlyphId = (ushort)charcode.Code;
        }
    }
}
