﻿using System;
using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{
    internal class TestFontSource : FontSource
    {
        private readonly bool isItalic;
        private readonly Dictionary<ushort, int> glyphIdToOutlinesCount;
        private readonly Dictionary<ushort, GlyphOutlinesCollection> registeredGlyphOutlines;

        public TestFontSource()
            : this(false)
        {
        }

        public TestFontSource(bool isItalic)
        {
            this.isItalic = isItalic;
            this.glyphIdToOutlinesCount = new Dictionary<ushort, int>();
            this.registeredGlyphOutlines = new Dictionary<ushort, GlyphOutlinesCollection>();
        }

        internal int TotalOutlinesCount { get; private set; }

        public override double GetAdvancedWidthOverride(int charCode)
        {
            return 15.5;
        }

        public override void GetAdvancedWidthOverride(Glyph glyph)
        {
            glyph.AdvancedWidth = 15.5;
        }

        public override double GetAscent()
        {
            return 0;
        }

        public override Rect GetBoundingBox()
        {
            return new Rect();
        }

        public override double GetCapHeight()
        {
            return 0;
        }

        public override byte[] GetData()
        {
            throw new NotImplementedException();
        }

        public override double GetDescent()
        {
            return 0;
        }
        
        public override string GetFontFamily()
        {
            throw new NotImplementedException();
        }

        public override void InitializeGlyphOutlinesOverride(Glyph glyph, double fontSize)
        {
            int count = this.GetOutlinesCount(glyph.GlyphId);
            this.glyphIdToOutlinesCount[glyph.GlyphId] = count + 1;
            this.TotalOutlinesCount++;

            GlyphOutlinesCollection outlines;
            if (!this.registeredGlyphOutlines.TryGetValue(glyph.GlyphId, out outlines))
            {
                outlines = new GlyphOutlinesCollection();
            }

            glyph.Outlines = outlines;
        }

        public override bool GetIsBold()
        {
            throw new NotImplementedException();
        }

        public override bool GetIsItalic()
        {
            return this.isItalic;
        }

        public override double GetItalicAngle()
        {
            return this.isItalic ? 30 : 0;
        }

        public override double GetLineGap()
        {
            throw new NotImplementedException();
        }

        public override double GetStemV()
        {
            return 0;
        }

        public override double GetUnderlinePosition()
        {
            throw new NotImplementedException();
        }

        public override double GetUnderlineThickness()
        {
            throw new NotImplementedException();
        }

        public override bool TryGetCharCode(int unicode, out int charCode)
        {
            charCode = unicode;
            return true;
        }

        public override bool TryGetGlyphId(int unicode, out ushort glyphId)
        {
            throw new NotImplementedException();
        }

        internal int GetOutlinesCount(ushort glyphId)
        {
            int count;
            if (!glyphIdToOutlinesCount.TryGetValue(glyphId, out count))
            {
                count = 0;
            }

            return count;
        }

        internal void RegisterGlyphOuntlines(ushort glyphId, GlyphOutlinesCollection outlines)
        {
            this.registeredGlyphOutlines.Add(glyphId, outlines);
        }
    }
}
