﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Fonts.Encoding;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Fonts
{
    [TestClass]
    public class Type1FontTests
    {
        [TestMethod]
        public void TestType1Font_ActualFontSourceRemainsTheSameWhenType1FontSource()
        {
            TestType1FontSource fontSource = new TestType1FontSource();
            Type1Font font = new Type1Font("test", fontSource);
            FontSource actualFontSource = font.ActualFontSource;

            Assert.AreEqual(fontSource, actualFontSource);
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardFontWhenTheNameIsStandard()
        {
            Type1Font font = new Type1Font("Times-Bold", EmptyFontSource.Instance);
            FontSource actualFontSource = font.ActualFontSource;
            FontSource expectedFontSource = FontsRepository.TimesBold.FontSource;       
            
            Assert.AreEqual(expectedFontSource, actualFontSource);
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardFontWhenTheNameIsAlternative()
        {
            Type1Font font = new Type1Font("Courier New,BoldItalic", EmptyFontSource.Instance);
            FontSource actualFontSource = font.ActualFontSource;
            FontSource expectedFontSource = FontsRepository.CourierBoldOblique.FontSource;

            Assert.AreEqual(expectedFontSource, actualFontSource);
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardFontWhenTheNameIsUnknown()
        {
            Type1Font font = new Type1Font("test", EmptyFontSource.Instance);
            FontSource actualFontSource = font.ActualFontSource;
            FontSource expectedFontSource = FontsRepository.Helvetica.FontSource;

            Assert.AreEqual(expectedFontSource, actualFontSource);
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardRespectingBoldWhenTheNameIsUnknown()
        {
            string[] sampleNames = { "testBold", "testbd", "boldtest", "tebd", "test,bold"};
            FontSource expectedFontSource = FontsRepository.HelveticaBold.FontSource;

            foreach (string name in sampleNames)
            {
                Type1Font font = new Type1Font(name, EmptyFontSource.Instance);
                FontSource actualFontSource = font.ActualFontSource;

                Assert.AreEqual(expectedFontSource, actualFontSource);
            }
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardRespectingItalicWhenTheNameIsUnknown()
        {
            string[] sampleNames = { "testItilic", "testit", "oblitest", "tobl", "test,italic" };
            FontSource expectedFontSource = FontsRepository.HelveticaOblique.FontSource;

            foreach (string name in sampleNames)
            {
                Type1Font font = new Type1Font(name, EmptyFontSource.Instance);
                FontSource actualFontSource = font.ActualFontSource;

                Assert.AreEqual(expectedFontSource, actualFontSource);
            }
        }

        [TestMethod]
        public void TestType1Font_EmptyFontSourceFallbacksToStandardRespectingBoldAndItalicWhenTheNameIsUnknown()
        {
            string[] sampleNames = { "testItilicbOlD", "testbdit", "oblitestbold", "toblbD", "test,bolditalic" };
            FontSource expectedFontSource = FontsRepository.HelveticaBoldOblique.FontSource;

            foreach (string name in sampleNames)
            {
                Type1Font font = new Type1Font(name, EmptyFontSource.Instance);
                FontSource actualFontSource = font.ActualFontSource;

                Assert.AreEqual(expectedFontSource, actualFontSource);
            }
        }

        [TestMethod]
        public void TestType1Font_InitializeCoreGlyph_CffFontSourceWithoutEncoding()
        {
            TestCFFFontSource fontSource = new TestCFFFontSource();
            fontSource.AddGlyphIdMapping("testName", 456);
            fontSource.AddCidIdMapping(131, "testName");
            Type1Font font = new Type1Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(131));

            Assert.AreEqual("testName", g.Name);
            Assert.AreEqual(456, g.GlyphId);
        }

        [TestMethod]
        public void TestType1Font_InitializeCoreGlyph_CffFontSourceWithEncoding()
        {
            SimpleEncoding encoding = new SimpleEncoding();
            encoding.PredefinedEncoding.Value = PredefinedSimpleEncoding.PdfDocEncodingName;
            encoding.Differences.Value = new DifferencesRanges();
            DifferencesRange differences = encoding.Differences.Value.AddDifferencesRange(150);
            differences.AddNextDifference("testDifference");

            TestCFFFontSource fontSource = new TestCFFFontSource();
            fontSource.AddGlyphIdMapping("Euro", 567);
            fontSource.AddGlyphIdMapping("testDifference", 123);
            Type1Font font = new Type1Font("test", fontSource);
            font.Encoding.Value = encoding;

            Glyph first = new Glyph();
            Glyph second = new Glyph();
            font.InitializeCoreGlyph(first, new CharCode(160));
            font.InitializeCoreGlyph(second, new CharCode(150));

            Assert.AreEqual("Euro", first.Name);
            Assert.AreEqual("testDifference", second.Name);
            Assert.AreEqual(567, first.GlyphId);
            Assert.AreEqual(123, second.GlyphId);
        }

        [TestMethod]
        public void TestType1Font_InitializeCoreGlyph_Type1FontSourceWithoutEncoding()
        {
            TestType1FontSource fontSource = new TestType1FontSource();
            fontSource.AddCidMapping(123, "testName");
            Type1Font font = new Type1Font("test", fontSource);

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(123));

            Assert.AreEqual("testName", g.Name);
            Assert.AreEqual(123, g.GlyphId);
        }

        [TestMethod]
        public void TestType1Font_InitializeCoreGlyph_Type1FontSourceWithEncoding()
        {
            SimpleEncoding encoding = new SimpleEncoding();
            encoding.PredefinedEncoding.Value = PredefinedSimpleEncoding.PdfDocEncodingName;
            encoding.Differences.Value = new DifferencesRanges();
            DifferencesRange differences = encoding.Differences.Value.AddDifferencesRange(150);
            differences.AddNextDifference("testDifference");

            TestType1FontSource fontSource = new TestType1FontSource();
            Type1Font font = new Type1Font("test", fontSource);
            font.Encoding.Value = encoding;

            Glyph first = new Glyph();
            Glyph second = new Glyph();
            font.InitializeCoreGlyph(first, new CharCode(160));
            font.InitializeCoreGlyph(second, new CharCode(150));

            Assert.AreEqual("Euro", first.Name);
            Assert.AreEqual("testDifference", second.Name);
            Assert.AreEqual(160, first.GlyphId);
            Assert.AreEqual(150, second.GlyphId);
        }

        [TestMethod]
        public void TestType1Font_WithSimpleEncoding_InitializedWithNullEncodingName()
        {
            SimpleEncoding encoding = new SimpleEncoding();
            encoding.PredefinedEncoding.Value = "NULL";

            TestType1FontSource fontSource = new TestType1FontSource();
            Type1Font font = new Type1Font("test", fontSource);
            font.Encoding.Value = encoding;

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(163));

            Assert.AreEqual("sterling", g.Name);
            Assert.AreEqual(163, g.GlyphId);
        }

        [TestMethod]
        public void TestType1Font_WithSimpleEncoding_InitializedWithMissingPredefinedEncoding()
        {
            SimpleEncoding encoding = new SimpleEncoding();

            TestType1FontSource fontSource = new TestType1FontSource();
            Type1Font font = new Type1Font("test", fontSource);
            font.Encoding.Value = encoding;

            Glyph g = new Glyph();
            font.InitializeCoreGlyph(g, new CharCode(163));

            Assert.AreEqual("sterling", g.Name);
            Assert.AreEqual(163, g.GlyphId);
        }
    }
}
