﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Imaging;
#if !NETSTANDARD
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Core.XObjects;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
#endif
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Filters;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Extensions;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Tests.Model.ColorSpaces;

namespace Telerik.Windows.Documents.Fixed.Tests.Model.Resources
{
    [TestClass]
    public class ImageDescriptorTests
    {
        [TestMethod]
        public void ToImageDataSourceConversion_CreatesJpegImageDataSource_WhenSimpleRgbJpeg()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 3 };   
            
            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Jpeg, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            JpegImageDataSource jpegData = (JpegImageDataSource)dataSource;
            Assert.AreEqual(encodedData, jpegData.Data);
            Assert.AreNotEqual(decodedData, jpegData.Data);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CreatesJpegImageDataSource_WhenSimpleGrayJpeg()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 1 };

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Jpeg, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            JpegImageDataSource jpegData = (JpegImageDataSource)dataSource;
            Assert.AreEqual(encodedData, jpegData.Data);
            Assert.AreNotEqual(decodedData, jpegData.Data);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CreatesNonJpegImageDataSource_WhenSimpleCmykJpeg()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 4 };

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Decoded, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            DecodedImageDataSource decodedSource = (DecodedImageDataSource)dataSource;
            Assert.AreEqual(ImageDecodedDataType.Bgra, decodedSource.Data.DataType);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CreatesJpegImageDataSource_WhenRgbWithDefaultDecode()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 3 };
            image.Decode = ColorSpaceBase.GetDecodeArrayFromComponents(3);

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Jpeg, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            JpegImageDataSource jpegData = (JpegImageDataSource)dataSource;
            Assert.AreEqual(encodedData, jpegData.Data);
            Assert.AreNotEqual(decodedData, jpegData.Data);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CreatesNonJpegImageDataSource_WhenRgbWithNonDefaultDecode()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 3 };
            image.Decode = ColorSpaceBase.GetDecodeArrayFromComponents(3);
            image.Decode[2] = 0.5;

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Decoded, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            DecodedImageDataSource decodedSource = (DecodedImageDataSource)dataSource;
            Assert.AreEqual(ImageDecodedDataType.Bgra, decodedSource.Data.DataType);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CreatesNonJpegImageDataSource_WhenGrayWithNonDefaultDecode()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new TestColorSpace() { ComponentCount = 1 };
            image.Decode = new double[] { 1, 0 };

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Decoded, dataSource.ImageDataType);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            DecodedImageDataSource decodedSource = (DecodedImageDataSource)dataSource;
            Assert.AreEqual(ImageDecodedDataType.Bgra, decodedSource.Data.DataType);
        }

        [TestMethod]
        public void ToDecodedDataSourceConversion_CreatesNonJpegImageDataSource_WhenGrayWithDefaultDecode()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.Width = 10;
            image.Height = 20;
            image.Filters = new string[] { PdfFilterNames.DCTDecode };
            image.ColorSpace = new DeviceGray();
            image.Decode = new double[] { 0, 1 };

            DecodedImageDataSource dataSource = image.ToDecodedImageDataSource();
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);
            Assert.AreEqual(ImageDecodedDataType.Gray, dataSource.Data.DataType);
        }

        [TestMethod]
        public void ToDecodedDataSourceConversion_CreatesBgraPixelContainer_WhenIndexedColorSpaceWithMask()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.ColorSpace = new Indexed();
            image.Width = 10;
            image.Height = 20;
            image.Mask = new ImageMask(new int[] { 2, 3 });

            DecodedImageDataSource dataSource = image.ToDecodedImageDataSource();
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);
            Assert.AreEqual(ImageDecodedDataType.Bgra, dataSource.Data.DataType);
        }

        [TestMethod]
        public void ToDecodedDataSourceConversion_CreatesIndexedPixelContainer_WhenIndexedColorSpaceWithoutMask()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.ColorSpace = new Indexed();
            image.Width = 10;
            image.Height = 20;

            DecodedImageDataSource dataSource = image.ToDecodedImageDataSource();
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);
            Assert.AreEqual(ImageDecodedDataType.Indexed, dataSource.Data.DataType);
        }

        [TestMethod]
        public void ToDecodedDataSourceConversion_CreatesMonochromePixelContainer_WhenGettingStencilMaskedPixels()
        {
            byte[] encodedData = new byte[1];
            byte[] decodedData = new byte[2];
            TestImageDescriptor image = new TestImageDescriptor(encodedData, decodedData);
            image.ColorSpace = new TestColorSpace() { ComponentCount = 1 };
            image.Width = 10;
            image.Height = 20;

            Color stencilColor = Color.FromArgb(0.5, 1, 0, 0);
            DecodedImageDataSource dataSource = image.ToDecodedImageDataSource(stencilColor);
            Assert.AreEqual(10, dataSource.Width);
            Assert.AreEqual(20, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);
            Assert.AreEqual(ImageDecodedDataType.Monochrome, dataSource.Data.DataType);
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CalculatesAlphaMaskedPixels_WhenMaskAndImageHaveSameSize()
        {
            Indexed indexedColorSpace = new Indexed();
            indexedColorSpace.Base = new DeviceRgb();
            indexedColorSpace.HiVal = 4;
            indexedColorSpace.Lookup = new byte[]
            {
                10, 100, 50,
                255, 240, 0,
                130, 0, 108,
                4, 8, 15,
                16, 23, 42,
            };

            byte[] imageData = new byte[] { (4 << 4) | 3, 2 << 4, 1 << 4, 3 << 4 };
            byte[] sMaskData = new byte[] { 0, 51, 102, 153, 204, 255 };

            TestImageDescriptor sMask = new TestImageDescriptor(sMaskData, sMaskData);
            sMask.ColorSpace = new DeviceGray();
            sMask.Width = 3;
            sMask.Height = 2;
            sMask.BitsPerComponent = 8;
            TestImageDescriptor image = new TestImageDescriptor(imageData, imageData);
            image.ColorSpace = indexedColorSpace;
            image.SMask = sMask;
            image.Width = 3;
            image.Height = 2;
            image.BitsPerComponent = 4;

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Decoded, dataSource.ImageDataType);
            Assert.AreEqual(3, dataSource.Width);
            Assert.AreEqual(2, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            DecodedImageDataSource decodedSource = (DecodedImageDataSource)dataSource;
            Assert.AreEqual(ImageDecodedDataType.Bgra, decodedSource.Data.DataType);

            PixelContainer pixelContainer = decodedSource.Data;
            Assert.AreEqual(6, pixelContainer.PixelCount);

            Color[] expectedColors = new Color[]
            {
                Color.FromArgb(0, 16, 23, 42),
                Color.FromArgb(51, 4, 8, 15),
                Color.FromArgb(102, 130, 0, 108),
                Color.FromArgb(153, 255, 240, 0),
                Color.FromArgb(204, 10, 100, 50),
                Color.FromArgb(255, 4, 8, 15),
            };

            for (int i = 0; i < expectedColors.Length; i++)
            {
                Color actual = pixelContainer.GetColorFromIndex(i);
                Color expected = expectedColors[i];
                Assert.AreEqual(expected, actual);
            }
        }

        [TestMethod]
        public void ToImageDataSourceConversion_CalculatesAlphaMaskedPixels_WhenMaskAndImageHaveDifferentSize()
        {
            byte[] imageData = new byte[]
            {
                16, 23, 42,
                4, 8, 15,
                130, 0, 108,
                255, 240, 0,
                10, 100, 50,
                0, 80, 155,
            };

            byte[] sMaskData = new byte[] { 0, 36, 73, 109, 146, 182, 219, 255 };

            TestImageDescriptor sMask = new TestImageDescriptor(sMaskData, sMaskData);
            sMask.ColorSpace = new DeviceGray();
            sMask.Width = 2;
            sMask.Height = 4;
            sMask.BitsPerComponent = 8;
            TestImageDescriptor image = new TestImageDescriptor(imageData, imageData);
            image.ColorSpace = new DeviceRgb();
            image.SMask = sMask;
            image.Width = 3;
            image.Height = 2;
            image.BitsPerComponent = 8;

            ImageDataSource dataSource = image.ToImageDataSource();
            Assert.AreEqual(ImageDataType.Decoded, dataSource.ImageDataType);
            Assert.AreEqual(3, dataSource.Width);
            Assert.AreEqual(4, dataSource.Height);
            Assert.IsFalse(dataSource.IsEmpty);

            DecodedImageDataSource decodedSource = (DecodedImageDataSource)dataSource;
            Assert.AreEqual(ImageDecodedDataType.Bgra, decodedSource.Data.DataType);

            PixelContainer pixelContainer = decodedSource.Data;
            Assert.AreEqual(12, pixelContainer.PixelCount);

            Color[] expectedColors = new Color[]
            {
                Color.FromArgb(0, 16, 23, 42), Color.FromArgb(0, 4, 8, 15), Color.FromArgb(36, 130, 0, 108),
                Color.FromArgb(73, 16, 23, 42), Color.FromArgb(73, 4, 8, 15), Color.FromArgb(109, 130, 0, 108),
                Color.FromArgb(146, 255, 240, 0), Color.FromArgb(146, 10, 100, 50), Color.FromArgb(182, 0, 80, 155),
                Color.FromArgb(219, 255, 240, 0), Color.FromArgb(219, 10, 100, 50), Color.FromArgb(255, 0, 80, 155),
            };

            for (int i = 0; i < expectedColors.Length; i++)
            {
                Color actual = pixelContainer.GetColorFromIndex(i);
                Color expected = expectedColors[i];
                Assert.AreEqual(expected, actual);
            }
        }

        [TestMethod]
        public void TestImageSourceColorSpace_WhenImageIsMaskAndHasNoSpecifiedColorSpace()
        {
            ImageSource image = new ImageSource(new byte[5], -1, 10, 20, null, null, null, true);

            Assert.IsFalse(image.ColorSpace.HasValue);
            Assert.IsNull(image.ColorSpace.Value);
            Assert.IsTrue(image.BitsPerComponent.HasValue);
            Assert.AreEqual(1, image.BitsPerComponent.Value);

            IImageDescriptor imageDescriptor = image;
            Assert.IsNotNull(imageDescriptor.ColorSpace);
            Assert.AreEqual(1, imageDescriptor.ColorSpace.ComponentCount);
            Assert.AreEqual(ColorSpaceNames.DeviceGray, imageDescriptor.ColorSpace.ToString());
            Assert.AreEqual(1, imageDescriptor.BitsPerComponent);
        }

#if !NETSTANDARD
        [TestMethod]
        public void TestXImage_DecodeArrayHasIntegerValues()
        {
            XImage image = new XImage(TestUtils.ContentManager);
            image.Decode = new PdfArrayOld(TestUtils.ContentManager, new object[] { 1, 0 });

            IImageDescriptor descriptor = image;
            double[] actualDecode = descriptor.Decode;

            Assert.IsNotNull(actualDecode);
            Assert.AreEqual(2, actualDecode.Length);
            Assert.AreEqual(1, actualDecode[0]);
            Assert.AreEqual(0, actualDecode[1]);
        }
#endif
    }
}
