﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.Tests.PdfReaderTests
{
    [TestClass]
    public class TestBitReader
    {
        private readonly byte[] sampleSource = { 0x23, 0xff, 0x5d, 0xab, 0x42};

        [TestMethod]
        public void TestBitReaderWithOneBitIntegers()
        {
            AssertBitReaderWithSampleSource(1, new int[] { 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithTwoBitIntegers()
        {
            AssertBitReaderWithSampleSource(2, new int[] { 0, 2, 0, 3, 3, 3, 3, 3, 1, 1, 3, 1, 2, 2, 2, 3, 1, 0, 0, 2, -1, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithThreeBitIntegers()
        {
            AssertBitReaderWithSampleSource(3, new int[] { 1, 0, 7, 7, 7, 5, 3, 5, 5, 2, 6, 4, 1, 0, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithFourBitIntegers()
        {
            AssertBitReaderWithSampleSource(4, new int[] { 2, 3, 15, 15, 5, 13, 10, 11, 4, 2, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithEightBitIntegers()
        {
            AssertBitReaderWithSampleSource(8, new int[] { 35, 255, 93, 171, 66, -1, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithTwelveBitIntegers()
        {
            AssertBitReaderWithSampleSource(12, new int[] { 575, 3933, 2740, 512, -1, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithSixteenBitIntegers()
        {
            AssertBitReaderWithSampleSource(16, new int[] { 9215, 23979, 16896, -1, -1, -1 });
        }

        [TestMethod]
        public void TestBitReaderWithTwentyFourBitIntegers()
        {
            AssertBitReaderWithSampleSource(24, new int[] { 2359133, 11223552, -1, -1, -1 });
        }

        private void AssertBitReaderWithSampleSource(int intSizeInBits, int[] expectedValues)
        {
            BitReader bitReader = new BitReader(this.sampleSource, intSizeInBits);

            for (int i = 0; i < expectedValues.Length; i++)
            {
                int actual = bitReader.Read();
                int expected = expectedValues[i];
                Assert.AreEqual(expected, actual);
            }
        }
    }
}
