﻿using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.PostScript.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.DocumentModel.Data;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader.Parsers;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.Text;
using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.PdfReaderTests
{
    [TestClass]
    public class TestPdfParser
    {
        private Stream CreateStreamFromString(string text)
        {
            MemoryStream ms = new MemoryStream(System.Text.Encoding.UTF8.GetBytes(text));
            return ms;
        }

        private string GetString(byte[] bytes)
        {
            return System.Text.Encoding.UTF8.GetString(bytes, 0, bytes.Length);
        }

        [TestMethod]
        public void TestParseBool()
        {
            string text = "true false";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.Boolean, parser.Reader.ReadToken());
                Assert.AreEqual(true, parser.ParseCurrentTokenToBool().Value);
                Assert.AreEqual(Token.Boolean, parser.Reader.ReadToken());
                Assert.AreEqual(false, parser.ParseCurrentTokenToBool().Value);
            }
        }

        [TestMethod]
        public void TestParseInt()
        {
            string text = "123 43445 +17 -98 0 0 0 0 0 0 35";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(123, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(43445, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(17, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(-98, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(0, parser.ParseCurrentTokenToInt().Value);
                Assert.AreEqual(Token.Integer, parser.Reader.ReadToken());
                Assert.AreEqual(35, parser.ParseCurrentTokenToInt().Value);
            }
        }

        [TestMethod]
        public void TestParseReal()
        {
            string text = "34.5 -3.62 +123.6 4. -.002 0.0";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(34.5, parser.ParseCurrentTokenToReal().Value);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(-3.62, parser.ParseCurrentTokenToReal().Value);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(123.6, parser.ParseCurrentTokenToReal().Value);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(4.0, parser.ParseCurrentTokenToReal().Value);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(-.002, parser.ParseCurrentTokenToReal().Value);
                Assert.AreEqual(Token.Real, parser.Reader.ReadToken());
                Assert.AreEqual(0.0, parser.ParseCurrentTokenToReal().Value);
            }
        }

        [TestMethod]
        public void TestParseLiteralString()
        {
            string text =
@"(This is a string)
(Strings may contain newlines\r\nand such.)
(Strings may contain balanced parentheses () and special characters (*!&}^% and so on).)
()";

            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.String, parser.Reader.ReadToken());
                Assert.AreEqual("This is a string", this.GetString(parser.ParseCurrentTokenToString().Value));
                Assert.AreEqual(Token.String, parser.Reader.ReadToken());
                Assert.AreEqual("Strings may contain newlines\r\nand such.", this.GetString(parser.ParseCurrentTokenToString().Value));
                Assert.AreEqual(Token.String, parser.Reader.ReadToken());
                Assert.AreEqual("Strings may contain balanced parentheses () and special characters (*!&}^% and so on).", this.GetString(parser.ParseCurrentTokenToString().Value));
                Assert.AreEqual(Token.String, parser.Reader.ReadToken());
                Assert.AreEqual("", this.GetString(parser.ParseCurrentTokenToString().Value));
            }
        }

        [TestMethod]
        public void TestParsePdfName()
        {
            string text = @"
/Name1
/ASomewhatLongerName
/A;Name_With-Various***Characters?
/1.2
/$$
/@pattern
/.notdef
/Adobe#20Green
/PANTONE#205757#20CV
/paired#28#29parentheses
/The_Key_of_F#23_Minor
/A#42
";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("Name1", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("ASomewhatLongerName", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("A;Name_With-Various***Characters?", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("1.2", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("$$", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("@pattern", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual(".notdef", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("Adobe Green", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("PANTONE 5757 CV", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("paired()parentheses", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("The_Key_of_F#_Minor", parser.ParseCurrentTokenToName().Value);
                Assert.AreEqual(Token.Name, parser.Reader.ReadToken());
                Assert.AreEqual("AB", parser.ParseCurrentTokenToName().Value);
            }
        }

        [TestMethod]
        public void TestParsePdfArray()
        {
            string text = @"[549 3.14 false (Ralph) /SomeName]";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.ArrayStart, parser.Reader.ReadToken());
                PdfArrayOld arr = parser.ParseCurrentTokenToArray();
                
                Assert.AreEqual(549, arr.GetElement<PdfIntOld>(0).Value);
                Assert.AreEqual(3.14, arr.GetElement<PdfRealOld>(1).Value);
                Assert.AreEqual(false, arr.GetElement<PdfBoolOld>(2).Value);
                Assert.AreEqual("Ralph", this.GetString(arr.GetElement<PdfStringOld>(3).Value));
                Assert.AreEqual("SomeName", arr.GetElement<PdfNameOld>(4).Value);
            }
        }

        [TestMethod]
        public void TestParsePdfDictionary()
        {
            string text = @"
<< 
/Type /Example
/Subtype /DictionaryExample
/SubDictionary 
<< 
/Item /Item
>>
>>
";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParserBase parser = new PdfParserBase(TestUtils.ContentManager, s);
                Assert.AreEqual(Token.DictionaryStart, parser.Reader.ReadToken());
                PdfDictionaryOld dict = parser.ParseCurrentTokenToDictionary();
                Assert.AreEqual("Example", ((PdfNameOld)dict["Type"]).Value);
                Assert.AreEqual("DictionaryExample", ((PdfNameOld)dict["Subtype"]).Value);
                PdfDictionaryOld inner = (PdfDictionaryOld)dict["SubDictionary"];
                Assert.AreEqual("Item", ((PdfNameOld)inner["Item"]).Value);

            }
        }

        [TestMethod]
        public void TestParseIndirectObject()
        {
            string text = @"
12 0 obj
1234
endobj
";
            using (Stream s = this.CreateStreamFromString(text))
            {
                PdfParser parser = new PdfParser(TestUtils.ContentManager, s);
                IndirectObjectOld io;
                parser.TryReadIndirectObject(0, false, out io);
                Assert.AreEqual(12, io.ObjectNumber);
                Assert.AreEqual(0, io.GenerationNumber);
                Assert.AreEqual(1234, ((PdfIntOld)io.Value).Value);
            }
        }
    }
}
