﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Utilities;

namespace Telerik.Windows.Documents.Fixed.Tests.PdfWriterTests
{
    [TestClass]
    public class TestBitWriter
    {
        private readonly byte[] expectedResult = { 0x23, 0xff, 0x5d, 0xab, 0x42 };
        private readonly byte[] extendedExpectedResult = { 0x23, 0xff, 0x5d, 0xab, 0x42, 0 };

    [TestMethod]
        public void TestBitWriterWithOneBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(1, new int[] { 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0 });
        }

        [TestMethod]
        public void TestBitWriterWithTwoBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(2, new int[] { 0, 2, 0, 3, 3, 3, 3, 3, 1, 1, 3, 1, 2, 2, 2, 3, 1, 0, 0, 2 });
        }

        [TestMethod]
        public void TestBitWriterWithThreeBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(3, new int[] { 1, 0, 7, 7, 7, 5, 3, 5, 5, 2, 6, 4, 1, 0 }, extendedExpectedResult);
        }

        [TestMethod]
        public void TestBitWriterWithFourBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(4, new int[] { 2, 3, 15, 15, 5, 13, 10, 11, 4, 2 });
        }

        [TestMethod]
        public void TestBitWriterWithEightBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(8, new int[] { 35, 255, 93, 171, 66 });
        }

        [TestMethod]
        public void TestBitWriterWithTwelveBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(12, new int[] { 575, 3933, 2740, 512 }, extendedExpectedResult);
        }

        [TestMethod]
        public void TestBitWriterWithSixteenBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(16, new int[] { 9215, 23979, 16896 }, extendedExpectedResult);
        }

        [TestMethod]
        public void TestBitWriterWithTwentyFourBitIntegers()
        {
            this.AssertBitWriterWithSampleInput(24, new int[] { 2359133, 11223552 }, extendedExpectedResult);
        }

        [TestMethod]
        public void TestImageBitWriterOneBitPerInteger()
        {
            ImageBitWriter imageBitWriter = new ImageBitWriter(new byte[8], 1, 12);
            int[] input = 
            {
                0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1,
                1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1,
                1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0,
                0, 0, 1, 0
            };

            byte[] exected = { 0x23, 0xf0, 0xf5, 0xd0, 0xab, 0x40, 0x20, 0x00 };

            this.AssertBitWriterWithSampleInput(imageBitWriter, input, exected);
        }

        [TestMethod]
        public void TestImageBitWriterWriteAtPositionWithOneBitPerInteger()
        {
            ImageBitWriter imageBitWriter = new ImageBitWriter(new byte[8], 1, 12);

            imageBitWriter.WriteBitsToIndex(1, 6);
            imageBitWriter.WriteBitsToIndex(1, 5);
            imageBitWriter.WriteBitsToIndex(1, 9);
            imageBitWriter.WriteBitsToIndex(1, 12);
            imageBitWriter.WriteBitsToIndex(1, 13);

            byte[] expected = { 0x06, 0x40, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00 };

            this.AssertBitWriterResult(imageBitWriter, expected);

            imageBitWriter.WriteBitsToIndex(0, 6);
            imageBitWriter.WriteBitsToIndex(0, 13);
            imageBitWriter.WriteBitsToIndex(0, 16);

            expected[0] = 0x04;
            expected[2] = 0x80;

            this.AssertBitWriterResult(imageBitWriter, expected);
        }

        [TestMethod]
        public void TestImageBitWriterOverwriteBitsWithTwelveBitsPerInteger()
        {
            ImageBitWriter imageBitWriter = new ImageBitWriter(new byte[10], 12, 3);

            imageBitWriter.WriteBitsToIndex(575, 1);
            imageBitWriter.WriteBitsToIndex(3933, 4);
            imageBitWriter.WriteBitsToIndex(2740, 5);

            byte[] expected = { 0x00, 0x02, 0x3f, 0x00, 0x00, 0x00, 0x0f, 0x5d, 0xab, 0x40 };

            this.AssertBitWriterResult(imageBitWriter, expected);

            imageBitWriter.WriteBitsToIndex(1372, 0);
            imageBitWriter.WriteBitsToIndex(1372, 1);
            imageBitWriter.WriteBitsToIndex(1372, 5);

            expected[0] = 0x55;
            expected[1] = 0xc5;
            expected[2] = 0x5c;
            expected[8] = 0x55;
            expected[9] = 0xc0;
            
            this.AssertBitWriterResult(imageBitWriter, expected);
        }

        private void AssertBitWriterWithSampleInput(int intSizeInBits, int[] input)
        {
            this.AssertBitWriterWithSampleInput(intSizeInBits, input, expectedResult);
        }

        private void AssertBitWriterWithSampleInput(int intSizeInBits, int[] input, byte[] expectedResult)
        {
            BitWriter bitWriter = new BitWriter(input.Length, intSizeInBits);

            this.AssertBitWriterWithSampleInput(bitWriter, input, expectedResult);
        }

        private void AssertBitWriterWithSampleInput(BitWriter bitWriter, int[] input, byte[] expectedResult)
        {
            for (int i = 0; i < input.Length; i++)
            {
                int bits = input[i];
                bitWriter.Write(bits);
            }

            this.AssertBitWriterResult(bitWriter, expectedResult);
        }

        private void AssertBitWriterResult(BitWriter bitWriter, byte[] expectedResult)
        {
            byte[] actualResult = bitWriter.ResultBits;
            Assert.AreEqual(expectedResult.Length, actualResult.Length);

            for (int i = 0; i < expectedResult.Length; i++)
            {
                byte actual = actualResult[i];
                byte expected = expectedResult[i];
                Assert.AreEqual(expected, actual);
            }
        }
    }
}
