﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Search;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.Search
{
    [TestClass]
    public class TestTextSearchRegex
    {
        [TestMethod]
        public void TestFindSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(3, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(3, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<nl>lAlz<nl>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(3, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(3, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("a.*", new TextSearchOptions(false, true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(10, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultipleLinesCaseInsensitive2()
        {
            TextDocument doc = TestUtils.CreateTextDocument("Ala<nl>Bala<nl>Nica");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("a.*", new TextSearchOptions(false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(13, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<np>lAlz<np>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(false, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<nl>lAlz<nl>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(7, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<np>lAlz<np>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l[a-z]", new TextSearchOptions(true, true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz l2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(3, res[1].Range.StartPosition.Index);
            Assert.AreEqual(5, res[1].Range.EndPosition.Index);

            Assert.AreEqual(5, res[2].Range.StartPosition.Index);
            Assert.AreEqual(7, res[2].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(4, res[0].Range.EndPosition.Index);

            Assert.AreEqual(5, res[1].Range.StartPosition.Index);
            Assert.AreEqual(9, res[1].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] lasdsd", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<nl>lAlz<nl>l2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(3, res[1].Range.StartPosition.Index);
            Assert.AreEqual(5, res[1].Range.EndPosition.Index);

            Assert.AreEqual(5, res[2].Range.StartPosition.Index);
            Assert.AreEqual(7, res[2].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(4, res[0].Range.EndPosition.Index);

            Assert.AreEqual(5, res[1].Range.StartPosition.Index);
            Assert.AreEqual(9, res[1].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] lasdsd", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<np>lAlz<np>l2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(false, true)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(0, res[1].Range.StartPosition.Index);
            Assert.AreEqual(2, res[1].Range.EndPosition.Index);

            Assert.AreEqual(2, res[2].Range.StartPosition.Index);
            Assert.AreEqual(4, res[2].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(false, true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz L2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(5, res[1].Range.StartPosition.Index);
            Assert.AreEqual(7, res[1].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(1, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(4, res[0].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] lasdsd", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<nl>lAlz<nl>L2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(5, res[1].Range.StartPosition.Index);
            Assert.AreEqual(7, res[1].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(1, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(4, res[0].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] lasdsd", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<np>lAlz<np>L2");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("l[a-z]", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(0, res[0].Range.StartPosition.Index);
            Assert.AreEqual(2, res[0].Range.EndPosition.Index);

            Assert.AreEqual(2, res[1].Range.StartPosition.Index);
            Assert.AreEqual(4, res[1].Range.EndPosition.Index);

            res = search.FindAll("l[a-z] l", new TextSearchOptions(true, true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }
    }
}
