﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Search;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.Search
{
    [TestClass]
    public class TestTextSearchWholeWordsOnly
    {
        [TestMethod]
        public void TestFindSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("lA", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("lz", new TextSearchOptions(false, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l2", new TextSearchOptions(false, false, true));
            Assert.AreEqual(8, res.Range.StartPosition.Index);
            Assert.AreEqual(10, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la lAlz l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(true, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("lA", new TextSearchOptions(true, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("lz", new TextSearchOptions(true, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("l2", new TextSearchOptions(true, false, true));
            Assert.AreEqual(8, res.Range.StartPosition.Index);
            Assert.AreEqual(10, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultipleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("la<nl>lAlz<nl>lz");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("lz", new TextSearchOptions(false, false, true));
            Assert.AreEqual(8, res.Range.StartPosition.Index);
            Assert.AreEqual(10, res.Range.EndPosition.Index);

            res = search.Find("lz", new TextSearchOptions(false, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("lz", new TextSearchOptions(false, false, true));
            Assert.AreEqual(8, res.Range.StartPosition.Index);
            Assert.AreEqual(10, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("l.a<np>lA?lz l.a<np>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l", new TextSearchOptions(true, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(1, res.Range.EndPosition.Index);

            res = search.Find("L", new TextSearchOptions(true, false, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("a", new TextSearchOptions(true, false, true));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("lA", new TextSearchOptions(true, false, false));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("lz", new TextSearchOptions(true, false, true));
            Assert.AreEqual(3, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            res = search.Find("l2", new TextSearchOptions(true, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("L.A<np>lA?lz<np>l2");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("l", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(1, res.Range.EndPosition.Index);

            res = search.Find("l.a", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("La", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("l2", new TextSearchOptions(false, false, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word1 wOrd drow drow2word,word worD.word1word word");
            TextSearch search = new TextSearch(doc);
            SearchResult[] res = search.FindAll("word", new TextSearchOptions(false, false, true)).ToArray();

            Assert.AreEqual(4, res.Length);
            Assert.AreEqual(6, res[0].Range.StartPosition.Index);
            Assert.AreEqual(10, res[0].Range.EndPosition.Index);

            Assert.AreEqual(26, res[1].Range.StartPosition.Index);
            Assert.AreEqual(30, res[1].Range.EndPosition.Index);

            Assert.AreEqual(31, res[2].Range.StartPosition.Index);
            Assert.AreEqual(35, res[2].Range.EndPosition.Index);

            Assert.AreEqual(46, res[3].Range.StartPosition.Index);
            Assert.AreEqual(50, res[3].Range.EndPosition.Index);

            res = search.FindAll("words", new TextSearchOptions(false, false, true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word1 wOrD word drow2word,wOrd woRD.word1word woRD");
            TextSearch search = new TextSearch(doc);
            SearchResult[] res = search.FindAll("woRD", new TextSearchOptions(true, false, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(31, res[0].Range.StartPosition.Index);
            Assert.AreEqual(35, res[0].Range.EndPosition.Index);

            Assert.AreEqual(46, res[1].Range.StartPosition.Index);
            Assert.AreEqual(50, res[1].Range.EndPosition.Index);

            res = search.FindAll("woRDs", new TextSearchOptions(true, false, true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word1<nl> wOrD word drow2word,wOrd<nl>woRD.word1word woRD");
            TextSearch search = new TextSearch(doc);
            SearchResult[] res = search.FindAll("word", new TextSearchOptions(false, false, true)).ToArray();

            Assert.AreEqual(5, res.Length);

            Assert.AreEqual(7, res[0].Range.StartPosition.Index);
            Assert.AreEqual(11, res[0].Range.EndPosition.Index);

            Assert.AreEqual(12, res[1].Range.StartPosition.Index);
            Assert.AreEqual(16, res[1].Range.EndPosition.Index);

            Assert.AreEqual(27, res[2].Range.StartPosition.Index);
            Assert.AreEqual(31, res[2].Range.EndPosition.Index);

            Assert.AreEqual(32, res[3].Range.StartPosition.Index);
            Assert.AreEqual(36, res[3].Range.EndPosition.Index);

            Assert.AreEqual(47, res[4].Range.StartPosition.Index);
            Assert.AreEqual(51, res[4].Range.EndPosition.Index);

            res = search.FindAll("words", new TextSearchOptions(false, false, true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word1<nl> wOrD word drow2word,wOrd<nl>woRD.word1word woRD");
            TextSearch search = new TextSearch(doc);
            SearchResult[] res = search.FindAll("woRD", new TextSearchOptions(true, false, true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(32, res[0].Range.StartPosition.Index);
            Assert.AreEqual(36, res[0].Range.EndPosition.Index);

            Assert.AreEqual(47, res[1].Range.StartPosition.Index);
            Assert.AreEqual(51, res[1].Range.EndPosition.Index);

            res = search.FindAll("Word", new TextSearchOptions(true, false, true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word1<np> wOrD word drow2word,wOrd<np>woRD.word1word woRD");
            TextSearch search = new TextSearch(doc);
            SearchResult[] res = search.FindAll("word", new TextSearchOptions(false, false, true)).ToArray();

            Assert.AreEqual(5, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(5, res[0].Range.EndPosition.Index);

            Assert.AreEqual(6, res[1].Range.StartPosition.Index);
            Assert.AreEqual(10, res[1].Range.EndPosition.Index);

            Assert.AreEqual(21, res[2].Range.StartPosition.Index);
            Assert.AreEqual(25, res[2].Range.EndPosition.Index);

            Assert.AreEqual(0, res[3].Range.StartPosition.Index);
            Assert.AreEqual(4, res[3].Range.EndPosition.Index);

            Assert.AreEqual(15, res[4].Range.StartPosition.Index);
            Assert.AreEqual(19, res[4].Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindSingleLineCaseInsensitiveRegex()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word.word22 wOrdw. Word");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("wo[a-z]d", new TextSearchOptions(false, true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("word[0-9][0-9]", new TextSearchOptions(false, true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("word22", new TextSearchOptions(false, true, true));
            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("wordw.", new TextSearchOptions(false, true, true));
            Assert.AreEqual(12, res.Range.StartPosition.Index);
            Assert.AreEqual(18, res.Range.EndPosition.Index);

            res = search.Find("word\\.", new TextSearchOptions(false, true, true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindSingleLineCaseSensitiveRegex()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word.word22 wOrdw. Word Word");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("WOR[a-z]", new TextSearchOptions(true, true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("w[a-z]rd", new TextSearchOptions(true, true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("wordw.", new TextSearchOptions(true, true, true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("wOrdw.", new TextSearchOptions(true, true, true));
            Assert.AreEqual(12, res.Range.StartPosition.Index);
            Assert.AreEqual(18, res.Range.EndPosition.Index);

            res = search.Find("Word", new TextSearchOptions(true, true, true));
            Assert.AreEqual(19, res.Range.StartPosition.Index);
            Assert.AreEqual(23, res.Range.EndPosition.Index);

            res = search.Find("Word", new TextSearchOptions(true, true, true));
            Assert.AreEqual(24, res.Range.StartPosition.Index);
            Assert.AreEqual(28, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindWholeWordsOnlyRegex()
        {
            TextDocument doc = TestUtils.CreateTextDocument("word");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("word", new TextSearchOptions(false, true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            doc = TestUtils.CreateTextDocument("word ");
            search = new TextSearch(doc);
            res = search.Find("word", new TextSearchOptions(false, true, true));
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            doc = TestUtils.CreateTextDocument(" word");
            search = new TextSearch(doc);
            res = search.Find("word", new TextSearchOptions(false, true, true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);

            doc = TestUtils.CreateTextDocument(" word ");
            search = new TextSearch(doc);
            res = search.Find("word", new TextSearchOptions(false, true, true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(5, res.Range.EndPosition.Index);
        }
    }
}
