﻿
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;

namespace DPL_Visual
{
    public class TestHelper 
    {
        private string FolderPath = @"D:\Images\DPL\PdfViewer\";
        public static string BuildResults = @"\\x5testpc\TeamBuildResults\";
        public static string ImageVerificationResultFolder = @"D:\ImageVerificationResults\";

        private static int counter = 1;

        public bool CompareImages(string expectedImagePath, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            System.Windows.Forms.Cursor.Hide();
            Bitmap exp = GetSavedImageAsBitmap(expectedImagePath);

            Bitmap container;
            Bitmap act;
            bool equal;

            equal = ImageComparer(similarPixelArgbTolerance, differentPixelsPercentTolerance, similarPixelsPercentTolerance, exp, out container, out act);

            if (equal == false)
            {
                Thread.Sleep(1000);
                equal = ImageComparer(similarPixelArgbTolerance, differentPixelsPercentTolerance, similarPixelsPercentTolerance, exp, out container, out act);
            }

            if (equal == false)
            {
                var image = System.IO.Path.GetFileName(expectedImagePath);

                string imageName = System.IO.Path.GetFileNameWithoutExtension(image);
                var curruntTestPath = CreateTestResultFolders(expectedImagePath);

                exp.Save(curruntTestPath + "\\" + imageName + "_expected.png", System.Drawing.Imaging.ImageFormat.Png);
                container.Save(curruntTestPath + "\\" + imageName + "_diff.png", System.Drawing.Imaging.ImageFormat.Png);
                act.Save(curruntTestPath + "\\" + imageName + "_actual.png", System.Drawing.Imaging.ImageFormat.Png);
            }

            return equal;
        }

        private Bitmap GetSavedImageAsBitmap(string expectedImagePath)
        {
            Bitmap exp;
            try
            {
                if (FolderPath != string.Empty)
                {
                    exp = new Bitmap(this.FolderPath + expectedImagePath);
                }
                else
                {
                    exp = new Bitmap(expectedImagePath);
                }
            }
            catch
            {
                if (FolderPath != string.Empty)
                {
                    CaptureAndSave(this.FolderPath + expectedImagePath);
                    exp = new Bitmap(this.FolderPath + expectedImagePath);
                }
                else
                {
                    CaptureAndSave(expectedImagePath);
                    exp = new Bitmap(expectedImagePath);
                }
            }

            return exp;
        }


        // new compare iamges method
        private bool ImageComparer(double similarPixelArgbTolerance, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, Bitmap exp, out Bitmap container, out Bitmap act)
        {
            Thread.Sleep(1000);

            bool equal = true;

            var rect = Capture(GetForegroundWindow());

            int w = Math.Min(exp.Width, rect.Width);
            int h = Math.Min(exp.Height, rect.Height);

            int diferentPixels = 0;
            int similarPixels = 0;
            int totalPixels = w * h;

            container = new Bitmap(w, h);
            for (int i = 0; i < w; ++i)
            {
                for (int j = 0; j < h; ++j)
                {
                    System.Drawing.Color secondColor = rect.GetPixel(i, j);
                    System.Drawing.Color firstColor = exp.GetPixel(i, j);

                    if (firstColor != secondColor)
                    {
                        if ((Math.Abs(firstColor.A - secondColor.A) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.B - secondColor.B) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.G - secondColor.G) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.R - secondColor.R) < similarPixelArgbTolerance))
                        {
                            similarPixels++;
                            container.SetPixel(i, j, System.Drawing.Color.Green);
                        }
                        else
                        {
                            diferentPixels++;
                            container.SetPixel(i, j, System.Drawing.Color.Red);
                        }
                    }
                    else
                    {
                        container.SetPixel(i, j, firstColor);
                    }
                }
            }

            double diffPercent = ((double)diferentPixels / (double)totalPixels) * 100;
            double similarPercent = ((double)similarPixels / (double)totalPixels) * 100;

            if ((diffPercent > differentPixelsPercentTolerance) ||
                (similarPercent > similarPixelsPercentTolerance))
            {
                equal = false;
            }

            act = rect;

            return equal;
        }

        private string CreateTestResultFolders(string folder)
        {
            string directory = "";

            if (this.FolderPath == string.Empty)
            {
                directory = System.IO.Path.GetDirectoryName(folder);
            }
            else
            {
                directory = System.IO.Path.GetDirectoryName(this.FolderPath + folder);
            }

            string control = directory.Split(new char[] { System.IO.Path.DirectorySeparatorChar }, StringSplitOptions.RemoveEmptyEntries).Last();

            //var folderPath = CurrentBuild + "\\" + control;
            var folderPath = ImageVerificationResultFolder + GetLatestBuildFolder() + "\\" + control;

            if (!System.IO.Directory.Exists(folderPath))
            {
                System.IO.Directory.CreateDirectory(folderPath);
            }

            string curruntTestPath = folderPath + "\\" + counter;
            counter++;

            if (!System.IO.Directory.Exists(curruntTestPath))
            {
                System.IO.Directory.CreateDirectory(curruntTestPath);
            }

            return curruntTestPath;
        }

        private string GetLatestBuildFolder()
        {
            var mainDirectory = new DirectoryInfo(BuildResults);
            var subdir = mainDirectory.GetDirectories().OrderBy(t => t.LastWriteTime).LastOrDefault();

            var buildFolder = subdir.GetDirectories().OrderBy(t => t.LastWriteTime).LastOrDefault();


            var dirs = buildFolder.Name.Split(new char[] { '_' }, StringSplitOptions.RemoveEmptyEntries);

            var folder = dirs[dirs.Length - 2] + "_" + dirs.LastOrDefault();
            return folder;
        }

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        private static extern IntPtr GetWindowRect(IntPtr hWnd, ref Rect rect);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        private static extern IntPtr GetForegroundWindow();

        /// <summary> Position of the cursor relative to the start of the capture </summary>
        public static System.Drawing.Point CursorPosition;

        /// <summary> Capture the active window (default) or the desktop and return it as a bitmap </summary>
        /// <param name="mode">Optional. The default value is CaptureMode.Window.</param>
        public static Bitmap Capture(IntPtr handle)
        {
            System.Drawing.Rectangle bounds;
            var rect = new Rect();
            GetWindowRect(handle, ref rect);
            bounds = new System.Drawing.Rectangle(rect.Left, rect.Top, rect.Right - rect.Left, rect.Bottom - rect.Top);
            CursorPosition = new System.Drawing.Point(System.Windows.Forms.Cursor.Position.X - rect.Left, System.Windows.Forms.Cursor.Position.Y - rect.Top);

            var result = new Bitmap(bounds.Width, bounds.Height);
            using (var g = Graphics.FromImage(result))
                g.CopyFromScreen(new System.Drawing.Point(bounds.Left, bounds.Top), System.Drawing.Point.Empty, bounds.Size);

            return result;
        }
        /// <summary> Capture the active window (default) or the desktop and save it as a bitmap </summary>
        /// <param name="mode">Optional. The default value is CaptureMode.Window.</param>
        public static void CaptureAndSave(string filename)
        {
            ImageSave(filename, System.Drawing.Imaging.ImageFormat.Png, Capture(GetForegroundWindow()));
        }

        /// <summary> Save an image to a specific file </summary>
        /// <param name="format">Optional file save mode.  Default is PNG</param>
        /// <param name="image">Image to save.  Usually a BitMap, but can be any
        /// Image.</param>
        static void ImageSave(string filename, System.Drawing.Imaging.ImageFormat format, Bitmap image)
        {
            //if (!filename.Contains("."))
            //   filename = filename.Trim() + "." + format.ToString().ToLower();

            //if (!filename.Contains(@"\"))
            //filename = Path.Combine(Environment.GetEnvironmentVariable("TEMP") ?? @"C:\Temp", filename);
            //filename = @"C:\Images\Images\" + filename;
            //filename = filename.Replace("%NOW%", DateTime.Now.ToString("yyyy-MM-dd@hh.mm.ss"));

            if (!Directory.Exists(FolderPath))
            {
                Directory.CreateDirectory(FolderPath);
            }

            image.Save(filename, format);
            System.Threading.Thread.Sleep(1000);
        }

    }

    [System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Sequential)]
    public struct Rect
    {
        public int Left;
        public int Top;
        public int Right;
        public int Bottom;
    }
}
