﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.TextTests.Old
{
    [TestClass]
    public class TestTextPositionOld
    {
        [TestMethod]
        public void TestTextPositionConstructors()
        {
            TextDocument textDocument = TestUtils.CreateTextDocumentOld("ala<np>bala");

            TextPosition pos = new TextPosition(textDocument);
            Assert.AreEqual(pos.TextPage, textDocument.FirstPage);
            Assert.AreEqual(pos.Index, 0);

            pos = new TextPosition(textDocument.FirstPage);
            Assert.AreEqual(pos.TextPage, textDocument.FirstPage);
            Assert.AreEqual(pos.Index, 0);

            pos = new TextPosition(textDocument.LastPage);
            Assert.AreEqual(pos.TextPage, textDocument.LastPage);
            Assert.AreEqual(pos.Index, 0);

            pos = new TextPosition(textDocument.LastPage, 4);
            Assert.AreEqual(pos.TextPage, textDocument.LastPage);
            Assert.AreEqual(pos.Index, 4);

            TextPosition pos2 = new TextPosition(pos);
            Assert.AreEqual(pos2.TextPage, textDocument.LastPage);
            Assert.AreEqual(pos2.Index, 4);
            
            RadFixedDocument fixedDocument = new RadFixedDocument();
            fixedDocument.TextDocument = textDocument;
            RadFixedPage fixedPage = fixedDocument.Pages.AddPage();
            fixedPage.PageNumber = 1;

            TextPosition pos3 = new TextPosition(fixedPage);
            Assert.AreEqual(pos3.TextPage, textDocument.FirstPage);
            Assert.AreEqual(pos3.Index, 0);

            TextPosition pos4 = new TextPosition(fixedPage, 3);
            Assert.AreEqual(pos4.TextPage, textDocument.FirstPage);
            Assert.AreEqual(pos4.Index, 3);
        }
        
        [ExpectedException(typeof(ArgumentNullException))]
        [TestMethod]
        public void TestTextPositionConstructor_WhenDocumentNull_ShouldThrow()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();

            // Act && Assert
            TextPosition position = new TextPosition(page);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestMethod]
        public void TestTextPositionConstructorWithIndex_WhenDocumentNull_ShouldThrow()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();

            // Act && Assert
            TextPosition position = new TextPosition(page, 0);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestTextPositionArgumentNullException()
        {
            TextDocument doc1 = new TextDocument(0);
            TextPosition position = new TextPosition(doc1);
        }

        [TestMethod]
        public void TestMoveToIndex()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);

            position.MoveToIndex(doc.FirstPage, 3);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 3);

            position.MoveToIndex(doc.FirstPage, 8);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 8);

            position.MoveToIndex(doc.FirstPage, 22);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 8);

            doc = TestUtils.CreateTextDocumentOld("a b<nl>c d e<np>e f<nl>g");

            position = new TextPosition(doc);
            position.MoveToIndex(doc.FirstPage, 3);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 3);

            position.MoveToIndex(doc.FirstPage, 4);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 4);

            position.MoveToIndex(doc.LastPage, 3);
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 3);

            position.MoveToIndex(doc.LastPage, 5);
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 5);

            position.MoveToIndex(doc.LastPage, 20);
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 5);
        }

        [TestMethod]
        public void TestMoveToPosition()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.FirstPage, 4);
            TextPosition pos2 = new TextPosition(doc);
            pos2.MoveToPosition(pos1);
            Assert.AreEqual(pos2.TextPage, doc.FirstPage);
            Assert.AreEqual(pos2.Index, 4);

            pos1 = new TextPosition(doc.LastPage, 2);
            pos2 = new TextPosition(doc);
            pos2.MoveToPosition(pos1);
            Assert.AreEqual(pos2.TextPage, doc.LastPage);
            Assert.AreEqual(pos2.Index, 2);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMoveToPositionArgumentNullException()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("ala");
            TextPosition position = new TextPosition(doc);

            position.MoveToPosition(null);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestMoveToPositionArgumentException()
        {
            TextDocument doc1 = TestUtils.CreateTextDocumentOld("ala");
            TextDocument doc2 = TestUtils.CreateTextDocumentOld("bala");
            TextPosition pos1 = new TextPosition(doc1);
            TextPosition pos2 = new TextPosition(doc2);

            pos2.MoveToPosition(pos1);
        }

        [TestMethod]
        public void TestMoveToStartOfDocument()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.GetTextPage(0), 6);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToStartOfDocumentEmptyFirstPage()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("<np>hello world");
            TextPosition position = new TextPosition(doc);
            position.MoveToStartOfDocument();

            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.LastPage, 6);
            position.MoveToStartOfDocument();
            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToStartOfDocumentEmptyPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("<np>");
            TextPosition position = new TextPosition(doc);

            Assert.IsTrue(position.MoveToStartOfDocument());
            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(0, position.Index);
        }

        [TestMethod]
        public void TestMoveToStartOfDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.FirstPage, 8);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToStartOfDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world<np>a<np>b<np>c");
            TextPosition position = new TextPosition(doc);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.GetTextPage(1), 5);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.GetTextPage(2), 1);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.GetTextPage(3), 1);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 0);

            position.MoveToIndex(doc.GetTextPage(4), 1);
            position.MoveToStartOfDocument();
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToEndOfDocument()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11);

            position.MoveToIndex(doc.GetTextPage(0), 6);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11);
        }

        [TestMethod]
        public void TestMoveToEndOfDocumentEmptyPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("<np>");
            TextPosition position = new TextPosition(doc);

            Assert.IsTrue(position.MoveToEndOfDocument());
            Assert.AreEqual(doc.LastPage, position.TextPage);
            Assert.AreEqual(0, position.Index);
        }

        [TestMethod]
        public void TestMoveToEndOfDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11);

            position.MoveToIndex(doc.GetTextPage(0), 5);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11);
        }

        [TestMethod]
        public void TestMoveToEndOfDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world<np>a<np>b<np>c");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 1);

            position.MoveToIndex(doc.GetTextPage(1), 5);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 1);

            position.MoveToIndex(doc.GetTextPage(2), 1);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 1);

            position.MoveToIndex(doc.GetTextPage(3), 1);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 1);

            position.MoveToIndex(doc.GetTextPage(4), 1);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.TextPage, doc.LastPage);
            Assert.AreEqual(position.Index, 1);
        }

        [TestMethod]
        public void TestMoveToNextChar()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            Assert.AreEqual(position.Index, 0); // h

            bool hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 1); // e

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 2); // l

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 3); // l

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 4); // o

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 5); // end of word

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 6); // w

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 7); // o

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 8); // r

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 9); // l

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 10); // d

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsTrue(hasPositionMoved);
            Assert.AreEqual(position.Index, 11); // end of word

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsFalse(hasPositionMoved);
            Assert.AreEqual(position.Index, 11); // end of word

            hasPositionMoved = position.MoveToNextPosition();
            Assert.IsFalse(hasPositionMoved);
            Assert.AreEqual(position.Index, 11); // end of word
        }

        [TestMethod]
        public void TestMoveToNextCharDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            Assert.AreEqual(position.Index, 0); // h
            bool res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 1); // e
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2); // l
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4); // o
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6); // w
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 7); // o
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8); // r
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 9); // l
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10); // d
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 11); // end of word
            res = position.MoveToNextPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 11); // end of word
            res = position.MoveToNextPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 11); // end of word
        }

        [TestMethod]
        public void TestMoveToNextCharDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");
            TextPosition position = new TextPosition(doc);
            Assert.AreEqual(position.Index, 0); // h
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            bool res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 1); // e
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2); // l
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 3); // l
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4); // o
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0); // w
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 1); // o
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 2); // r
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 4); // d
            res = position.MoveToNextPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToNextPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToNextPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 5); // end of word
        }

        [TestMethod]
        public void TestMoveToNextCharEmptyPage()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np><np>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToLineEnd();

            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(5, position.Index);

            Assert.IsTrue(position.MoveToNextPosition());
            Assert.AreEqual(doc.GetTextPage(1), position.TextPage);
            Assert.AreEqual(0, position.Index);

            Assert.IsTrue(position.MoveToNextPosition());
            Assert.AreEqual(doc.LastPage, position.TextPage);
            Assert.AreEqual(0, position.Index);
        }

        [TestMethod]
        public void TestMoveToNextCharEmptyLastPage()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>");
            TextPosition position = new TextPosition(doc);
            position.MoveToLineEnd();

            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(5, position.Index);

            Assert.IsTrue(position.MoveToNextPosition());
            Assert.AreEqual(doc.LastPage, position.TextPage);
            Assert.AreEqual(0, position.Index);
        }

        [TestMethod]
        public void TestMoveToPrevChar()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11); // end of word
            bool res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10); // d
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 9); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8); // r
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 7); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6); // w
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 1); // e
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0); // h
        }

        [TestMethod]
        public void TestMoveToPrevCharDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 11); // end of word
            bool res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10); // d
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 9); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8); // r
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 7); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6); // w
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 1); // e
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0); // h
        }

        [TestMethod]
        public void TestMoveToPrevCharDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            Assert.AreEqual(position.Index, 5); // end of word
            bool res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 4); // d
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 2); // r
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 1); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0); // w
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 5); // end of word
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 4); // o
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 3); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 2); // l
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 1); // e
            res = position.MoveToPreviousPosition();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0); // h
            res = position.MoveToPreviousPosition();
            Assert.IsFalse(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0); // h
        }

        [TestMethod]
        public void TestMoveToPrevCharEmptyPage()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np><np>world");
            TextPosition position = new TextPosition(doc.LastPage);

            Assert.IsTrue(position.MoveToPreviousPosition());
            Assert.AreEqual(doc.GetTextPage(1), position.TextPage);
            Assert.AreEqual(0, position.Index);

            Assert.IsTrue(position.MoveToPreviousPosition());
            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(5, position.Index);
        }

        [TestMethod]
        public void TestMoveToPrevCharEmptyFirstPage()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("<np>world");
            TextPosition position = new TextPosition(doc.LastPage);

            Assert.IsTrue(position.MoveToPreviousPosition());
            Assert.AreEqual(doc.FirstPage, position.TextPage);
            Assert.AreEqual(0, position.Index);
        }

        [TestMethod]
        public void TestMoveToNextWord()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            bool res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            position = new TextPosition(doc.FirstPage, 3);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);

            doc = TestUtils.CreateTextDocumentOld("a b c d e f");
            position = new TextPosition(doc);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 10);
        }

        [TestMethod]
        public void TestMoveToNextWordDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            bool res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            position = new TextPosition(doc.FirstPage, 3);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);

            doc = TestUtils.CreateTextDocumentOld("a b<nl>c d e f g<nl>e<nl>f");
            position = new TextPosition(doc);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 12);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 14);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 16);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 16);
        }

        [TestMethod]
        public void TestMoveToNextWordDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");
            TextPosition position = new TextPosition(doc);
            bool res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            position = new TextPosition(doc.FirstPage, 3);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0);

            doc = TestUtils.CreateTextDocumentOld("a<np>b<nl>c<np>d e<np>f");
            position = new TextPosition(doc);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToNextWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(3));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToNextWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToPrevWord()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();

            bool res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);

            position = new TextPosition(doc.FirstPage, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);

            doc = TestUtils.CreateTextDocumentOld("a b c d e f");
            position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToPrevWordDifferentLines()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<nl>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();

            bool res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);

            position = new TextPosition(doc.FirstPage, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);

            doc = TestUtils.CreateTextDocumentOld("a b<nl>c d e f g<nl>e<nl>f");
            position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 16);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 14);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 12);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 10);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 8);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 6);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 4);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToPrevWordDifferentPages()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");
            TextPosition position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            bool res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0);

            position = new TextPosition(doc.FirstPage, 3);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0);

            doc = TestUtils.CreateTextDocumentOld("a<np>b<nl>c<np>d e<np>f");
            position = new TextPosition(doc);
            position.MoveToEndOfDocument();
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(3));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 2);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsTrue(res);
            Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
            Assert.AreEqual(position.Index, 0);
            res = position.MoveToPreviousWord();
            Assert.IsFalse(res);
            Assert.AreEqual(position.Index, 0);
        }

        [TestMethod]
        public void TestMoveToCurrentWordStart()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);

            position.MoveToCurrentWordStart();
            Assert.AreEqual(position.Index, 0);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToIndex(3);
            position.MoveToCurrentWordStart();
            Assert.AreEqual(position.Index, 0);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            position.MoveToCurrentWordStart();
            Assert.AreEqual(position.Index, 0);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToIndex(9);
            position.MoveToCurrentWordStart();
            Assert.AreEqual(position.Index, 6);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
            position.MoveToCurrentWordStart();
            Assert.AreEqual(position.Index, 6);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
        }

        [TestMethod]
        public void TestMoveToCurrentWordEnd()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello world");
            TextPosition position = new TextPosition(doc);

            position.MoveToCurrentWordEnd();
            Assert.AreEqual(position.Index, 5);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToIndex(3);
            position.MoveToCurrentWordEnd();
            Assert.AreEqual(position.Index, 5);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToCurrentWordEnd();
            Assert.AreEqual(position.Index, 5);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToIndex(9);
            position.MoveToCurrentWordEnd();
            Assert.AreEqual(position.Index, 11);
            Assert.AreEqual(position.TextPage, doc.FirstPage);

            position.MoveToCurrentWordEnd();
            Assert.AreEqual(position.Index, 11);
            Assert.AreEqual(position.TextPage, doc.FirstPage);
        }

        //[TestMethod]
        //public void TestMoveLineUp()
        //{
        //    RadFixedDocument document = TestUtils.CreateTextDocumentOld("000000<nl>1111<nl>222<nl>3333<nl>444444", 12, 0, 0);
        //    TextDocument doc = document.TextDocument;
        //    TextPosition position = new TextPosition(doc);

        //    position.MoveToIndex(7);
        //    bool res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 0);

        //    position.MoveToIndex(9);
        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 2);

        //    position.MoveToEndOfDocument();
        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 20);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 15);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 11);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 6);

        //    res = position.MoveLineUp();
        //    Assert.IsFalse(res);
        //    Assert.AreEqual(position.Index, 6);
        //}

        //[TestMethod]
        //public void TestMoveLineUpDifferentPages()
        //{
        //    RadFixedDocument document = TestUtils.CreateTextDocumentOld("000000<np>1111<np>222<np>3333<np>444444", 12, 0, 0);
        //    TextDocument doc = document.TextDocument;
        //    TextPosition position = new TextPosition(doc);

        //    position.MoveToIndex(doc.GetTextPage(1), 0);
        //    bool res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
        //    Assert.AreEqual(position.Index, 0);

        //    position.MoveToIndex(doc.GetTextPage(1), 2);
        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
        //    Assert.AreEqual(position.Index, 2);

        //    position.MoveToEndOfDocument();

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(3));
        //    Assert.AreEqual(position.Index, 4);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
        //    Assert.AreEqual(position.Index, 3);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
        //    Assert.AreEqual(position.Index, 4);

        //    res = position.MoveLineUp();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
        //    Assert.AreEqual(position.Index, 6);

        //    res = position.MoveLineUp();
        //    Assert.IsFalse(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(0));
        //    Assert.AreEqual(position.Index, 6);
        //}

        //[TestMethod]
        //public void TestMoveLineDown()
        //{
        //    RadFixedDocument document = TestUtils.CreateTextDocumentOld("000000<nl>1111<nl>222<nl>3333<nl>444444", 12, 0, 0);
        //    TextDocument doc = document.TextDocument;
        //    TextPosition position = new TextPosition(doc);
        //    position.MoveToIndex(2);
        //    bool res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 9);

        //    position.MoveToIndex(6);
        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 11);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 15);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 20);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 27);

        //    res = position.MoveLineDown();
        //    Assert.IsFalse(res);
        //    Assert.AreEqual(position.Index, 27);
        //}

        //[TestMethod]
        //public void TestMoveLineDownDifferentPages()
        //{
        //    RadFixedDocument document = TestUtils.CreateTextDocumentOld("000000<np>1111<np>222<np>3333<np>444444", 12, 0, 0);
        //    TextDocument doc = document.TextDocument;
        //    TextPosition position = new TextPosition(doc);
        //    position.MoveToIndex(2);

        //    bool res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.Index, 2);

        //    position.MoveToIndex(doc.GetTextPage(0), 6);
        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(1));
        //    Assert.AreEqual(position.Index, 4);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(2));
        //    Assert.AreEqual(position.Index, 3);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(3));
        //    Assert.AreEqual(position.Index, 4);

        //    res = position.MoveLineDown();
        //    Assert.IsTrue(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(4));
        //    Assert.AreEqual(position.Index, 6);

        //    res = position.MoveLineDown();
        //    Assert.IsFalse(res);
        //    Assert.AreEqual(position.TextPage, doc.GetTextPage(4));
        //    Assert.AreEqual(position.Index, 6);
        //}

        [TestMethod]
        public void TestTextPositionInEmptyDocument()
        {
            TextDocument doc = new TextDocument(1);
            doc.AddPage(0, TestUtils.CreateTextPageOld(doc, string.Empty, 0));
            TextPosition pos = new TextPosition(doc);
            bool res = pos.MoveToNextPosition();
            Assert.AreEqual(false, res);

            res = pos.MoveToPreviousPosition();
            Assert.AreEqual(false, res);

            res = pos.MoveToNextWord();
            Assert.AreEqual(false, res);

            res = pos.MoveToPreviousWord();
            Assert.AreEqual(false, res);

            res = pos.MoveLineUp();
            Assert.AreEqual(false, res);

            res = pos.MoveLineDown();
            Assert.AreEqual(false, res);

            res = pos.MoveToCurrentWordEnd();
            Assert.AreEqual(false, res);

            res = pos.MoveToCurrentWordStart();
            Assert.AreEqual(false, res);
        }

        [TestMethod]
        public void TestTextPositionOperatorEqual()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(pos1);

            Assert.IsTrue(pos1 == pos2);
        }

        [TestMethod]
        public void TestTextPositionOperatorNotEqual()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(doc.GetTextPage(1), 3);

            Assert.IsTrue(pos1 != pos2);
        }

        [TestMethod]
        public void TestTextPositionOperatorLessThan()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(doc.GetTextPage(1), 3);

            Assert.IsTrue(pos1 < pos2);
        }

        [TestMethod]
        public void TestTextPositionOperatorGreaterThan()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(doc.GetTextPage(1), 3);

            Assert.IsTrue(pos2 > pos1);
        }

        [TestMethod]
        public void TestTextPositionOperatorLessThanOrEqual()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(doc.GetTextPage(1), 3);

            Assert.IsTrue(pos1 <= pos2);

            pos2 = new TextPosition(pos1);

            Assert.IsTrue(pos1 <= pos2);
        }

        [TestMethod]
        public void TestTextPositionOperatorGreaterThanOrEqual()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("hello<np>world");

            TextPosition pos1 = new TextPosition(doc.GetTextPage(0), 3);
            TextPosition pos2 = new TextPosition(doc.GetTextPage(1), 3);

            Assert.IsTrue(pos2 >= pos1);

            pos2 = new TextPosition(pos1);

            Assert.IsTrue(pos1 >= pos2);
        }
    }
}
