﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Search;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Text;

namespace Telerik.Windows.Documents.Fixed.Tests.TextTests.Old
{
    [TestClass]
    public class TestTextSearchSimpleTextOld
    {
        [TestMethod]
        public void TestFindSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));

            Assert.AreEqual(4, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));

            Assert.AreEqual(4, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(doc.FirstPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("ala", new TextSearchOptions(false));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));

            Assert.AreEqual(doc.FirstPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(true));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(true));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("ala", new TextSearchOptions(true));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));

            Assert.AreEqual(doc.FirstPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(false));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(false));

            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));

            Assert.AreEqual(4, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(false));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(false));

            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));

            Assert.AreEqual(4, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(false));


            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(false));


            Assert.AreEqual(doc.FirstPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(true));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(true));

            Assert.AreEqual(5, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>Bala");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("ala", new TextSearchOptions(true));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("ala", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));

            Assert.AreEqual(doc.LastPage, res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.FindPrevious("ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a balA ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("a balA ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindMultipleWordsMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("a bala ba", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(6, res.Range.EndPosition.Index);

            res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindPrevMultipleWordsMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>baba");
            TextSearch search = new TextSearch(doc);

            SearchResult res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("a balA ba", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);
        }

        [TestMethod]
        public void TestFindCombinationsSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(6, res.Range.StartPosition.Index);
            Assert.AreEqual(8, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(1), res.Range.StartPosition.TextPage);
            Assert.AreEqual(2, res.Range.StartPosition.Index);
            Assert.AreEqual(4, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(false));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(9, res.Range.StartPosition.Index);
            Assert.AreEqual(11, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindCombinationsMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>laba");
            TextSearch search = new TextSearch(doc);
            SearchResult res;

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(2), res.Range.StartPosition.TextPage);
            Assert.AreEqual(0, res.Range.StartPosition.Index);
            Assert.AreEqual(2, res.Range.EndPosition.Index);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.FindPrevious("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(SearchResult.NotFound, res);

            res = search.Find("la", new TextSearchOptions(true));
            Assert.AreEqual(doc.GetTextPage(0), res.Range.StartPosition.TextPage);
            Assert.AreEqual(1, res.Range.StartPosition.Index);
            Assert.AreEqual(3, res.Range.EndPosition.Index);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(6, res[1].Range.StartPosition.Index);
            Assert.AreEqual(8, res[1].Range.EndPosition.Index);

            Assert.AreEqual(11, res[2].Range.StartPosition.Index);
            Assert.AreEqual(13, res[2].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(6, res[0].Range.EndPosition.Index);

            Assert.AreEqual(6, res[1].Range.StartPosition.Index);
            Assert.AreEqual(11, res[1].Range.EndPosition.Index);

            res = search.FindAll("latata", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(6, res[1].Range.StartPosition.Index);
            Assert.AreEqual(8, res[1].Range.EndPosition.Index);

            Assert.AreEqual(11, res[2].Range.StartPosition.Index);
            Assert.AreEqual(13, res[2].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(6, res[0].Range.EndPosition.Index);

            Assert.AreEqual(6, res[1].Range.StartPosition.Index);
            Assert.AreEqual(11, res[1].Range.EndPosition.Index);

            res = search.FindAll("latata", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultiplePagesCaseInsensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(false)).ToArray();

            Assert.AreEqual(3, res.Length);

            Assert.AreEqual(doc.GetTextPage(0), res[0].Range.StartPosition.TextPage);
            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(doc.GetTextPage(1), res[1].Range.StartPosition.TextPage);
            Assert.AreEqual(2, res[1].Range.StartPosition.Index);
            Assert.AreEqual(4, res[1].Range.EndPosition.Index);

            Assert.AreEqual(doc.GetTextPage(2), res[2].Range.StartPosition.TextPage);
            Assert.AreEqual(2, res[2].Range.StartPosition.Index);
            Assert.AreEqual(4, res[2].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(false)).ToArray();
            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllSingleLineCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala balA balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(11, res[1].Range.StartPosition.Index);
            Assert.AreEqual(13, res[1].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(1, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(6, res[0].Range.EndPosition.Index);

            res = search.FindAll("latata", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultipleLinesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<nl>balA<nl>balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(11, res[1].Range.StartPosition.Index);
            Assert.AreEqual(13, res[1].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(1, res.Length);

            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(6, res[0].Range.EndPosition.Index);

            res = search.FindAll("latata", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(0, res.Length);
        }

        [TestMethod]
        public void TestFindAllMultiplePagesCaseSensitive()
        {
            TextDocument doc = TestUtils.CreateTextDocumentOld("Ala<np>balA<np>balaba");
            TextSearch search = new TextSearch(doc);

            SearchResult[] res = search.FindAll("la", new TextSearchOptions(true)).ToArray();

            Assert.AreEqual(2, res.Length);

            Assert.AreEqual(doc.GetTextPage(0), res[0].Range.StartPosition.TextPage);
            Assert.AreEqual(1, res[0].Range.StartPosition.Index);
            Assert.AreEqual(3, res[0].Range.EndPosition.Index);

            Assert.AreEqual(doc.GetTextPage(2), res[1].Range.StartPosition.TextPage);
            Assert.AreEqual(2, res[1].Range.StartPosition.Index);
            Assert.AreEqual(4, res[1].Range.EndPosition.Index);

            res = search.FindAll("la ba", new TextSearchOptions(true)).ToArray();
            Assert.AreEqual(0, res.Length);
        }
    }
}
