﻿using System;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.Tests.Utils;
using M = System.Windows.Media;

namespace FontReaderTests.TrueTypeTests
{
    [TestClass]
    public class TestTrueTypeFontSource
    {
        [TestMethod]
        public void TestRequiredTables()
        {
            OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(new FontProperties(new FontFamily("Arial")));
            Assert.IsNotNull(src.Tables[Tags.CMAP_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HEAD_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HHEA_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.HMTX_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.MAXP_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.NAME_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.OS_2_TABLE]);
            Assert.IsNotNull(src.Tables[Tags.POST_TABLE]);
        }

        [TestMethod]
        public void TestCmapTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                M.GlyphTypeface gtf = TestHelper.GetGlyphTypeface(fd);
                OpenTypeFontSourceBase fs = TestHelper.GetFontSource(fd);
                foreach (char ch in TestHelper.TestString)
                {
                    ushort expectedGid;
                    gtf.CharacterToGlyphMap.TryGetValue(ch, out expectedGid);
                    ushort gid;
                    fs.TryGetGlyphId(ch, out gid);
                    Assert.AreEqual(expectedGid, gid);
                }
            }));
        }

        [TestMethod]
        public void TestHeadTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                bool isBold = tf.Weight == FontWeights.Bold && tf.StyleSimulations != M.StyleSimulations.BoldSimulation && tf.StyleSimulations != M.StyleSimulations.BoldItalicSimulation;
                bool isItalic = tf.Style == FontStyles.Italic && tf.StyleSimulations != M.StyleSimulations.ItalicSimulation && tf.StyleSimulations != M.StyleSimulations.BoldItalicSimulation;
                Assert.AreEqual(isBold, src.IsBold);
                Assert.AreEqual(isItalic, src.IsItalic);
            }));
        }

        //[TestMethod]
        //public void TestHorizontalHeaderTable()
        //{
        //    Helper.ExecuteActionWithDifferentFontDescriptors(new Action<FontDescriptor>(fd =>
        //    {
        //        TrueTypeFontSource src = Helper.GetTrueTypeFontSource(fd);
        //        GlyphTypeface tf = Helper.GetGlyphTypeface(fd);
        //        double baseline = src.GetBaseline(1);tf.as
        //        Assert.AreEqual(tf.Baseline, baseline);
        //    }));
        //}

        [TestMethod]
        public void TestMaximumProfileTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                Assert.AreEqual(tf.GlyphCount, src.MaxP.NumGlyphs);
            }));
        }

        [TestMethod]
        public void TestNameTable()
        {
            TestHelper.ExecuteActionWithDifferentFontProperties(new Action<FontProperties>(fd =>
            {
                OpenTypeFontSource src = TestHelper.GetTrueTypeFontSource(fd);
                Assert.IsNotNull(src, "src is null for the '" + fd.FontFamilyName + "' font.");

                M.GlyphTypeface tf = TestHelper.GetGlyphTypeface(fd);
                Assert.IsNotNull(tf, "tf is null");

                CultureInfo enUS = new CultureInfo("en-US");
                Assert.AreEqual(tf.Win32FamilyNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.FONT_FAMILY_ID));
                Assert.AreEqual(tf.ManufacturerNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.MANUFACTURER_NAME_ID));
                Assert.AreEqual(tf.DesignerNames[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.DESIGNER_ID));
                Assert.AreEqual(tf.SampleTexts[enUS], src.Name.ReadName(IDs.ENGLISH_US_ID, IDs.SAMPLE_TEXT_ID));
            }));
        }
    }
}
