﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Utilities.Rendering.Annotations;

namespace Telerik.Windows.Documents.Fixed.Tests.Utilities.Rendering.Annotations
{
    [TestClass]
    public class DefaultAppearanceProviderTests
    {
        private const string CurrentState = "CurrentState";

        [TestMethod]
        public void TryProvideAppearance_WhenAnnotationIsNull_ShouldReturnFalse()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(null, AnnotationAppearanceMode.Normal, out formSource);

            // Assert
            Assert.IsFalse(found);
            Assert.IsNull(formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenAppearancesAreNull_ShouldReturnFalse()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Normal, out formSource);

            // Assert
            Assert.IsFalse(found);
            Assert.IsNull(formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenSingleStateWithNormalAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.NormalAppearance = expectedAppearance;
            annotation.Appearances = singleStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Normal, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenSingleStateWithDownAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.MouseDownAppearance = expectedAppearance;
            annotation.Appearances = singleStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Down, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenSingleStateWithRolloverAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.MouseOverAppearance = expectedAppearance;
            annotation.Appearances = singleStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Rollover, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenMultiStateWithNormalAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            MultiStateAppearances multiStateAppearances = new MultiStateAppearances();
            multiStateAppearances.CurrentState = CurrentState;
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            multiStateAppearances.AddAppearance(CurrentState, singleStateAppearances);
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.NormalAppearance = expectedAppearance;
            annotation.Appearances = multiStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Normal, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenMultiStateWithDownAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            MultiStateAppearances multiStateAppearances = new MultiStateAppearances();
            multiStateAppearances.CurrentState = CurrentState;
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            multiStateAppearances.AddAppearance(CurrentState, singleStateAppearances);
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.MouseDownAppearance = expectedAppearance;
            annotation.Appearances = multiStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Down, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }

        [TestMethod]
        public void TryProvideAppearance_WhenMultiStateWithRolloverAppearance_ShouldProvideAppearance()
        {
            // Arrange
            DefaultAppearanceProvider provider = new DefaultAppearanceProvider(null);
            Link annotation = new Link();
            MultiStateAppearances multiStateAppearances = new MultiStateAppearances();
            multiStateAppearances.CurrentState = CurrentState;
            SingleStateAppearances singleStateAppearances = new SingleStateAppearances();
            multiStateAppearances.AddAppearance(CurrentState, singleStateAppearances);
            FormSource expectedAppearance = new FormSource();
            singleStateAppearances.MouseOverAppearance = expectedAppearance;
            annotation.Appearances = multiStateAppearances;
            FormSource formSource;

            // Act
            bool found = provider.TryProvideAppearance(annotation, AnnotationAppearanceMode.Rollover, out formSource);

            // Assert
            Assert.IsTrue(found);
            Assert.AreEqual(expectedAppearance, formSource);
        }
    }
}
