﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Utilities.Rendering;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Resources = Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Tests.Model.Fonts;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Tests.Utilities.Rendering
{
    [TestClass]
    public class ContentRootRenderingHelperTests
    {
        [TestMethod]
        public void RenderContentElements_WhenCorrectPath_ShouldPushCorrectPathTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            Fixed.Model.Graphics.PathGeometry geometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = geometry;

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(200, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectPathWithRotate_ShouldPushCorrectPathTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            path.Position.Rotate(90);
            Fixed.Model.Graphics.PathGeometry geometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = geometry;

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(200, 200);
            expectedTransformation.Rotate(90);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectPathWithScale_ShouldPushCorrectPathTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            path.Position.Scale(1.2, 1.5);
            Fixed.Model.Graphics.PathGeometry geometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = geometry;

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(200, 200);
            expectedTransformation.Scale(1.2, 1.5);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectPath_ShouldRenderPathGeometry()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            Fixed.Model.Graphics.RectangleGeometry geometry = new Fixed.Model.Graphics.RectangleGeometry();
            geometry.Rect = new Rect(new Size(225, 350));
            path.Geometry = geometry;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.RenderedGeometries.Count);

            GeometryBase pushedGeometry = renderer.RenderedGeometries[0].Item1;
            Assert.AreEqual(geometry, pushedGeometry);

            GeometryPropertiesOwner pushedGeometryProperties = renderer.RenderedGeometries[0].Item2;
            Assert.AreEqual(path.GeometryProperties, pushedGeometryProperties);
        }


        [TestMethod]
        public void RenderContentElements_WhenCorrectPath_ShouldPushPathTransformationAndRenderGeometryInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            Fixed.Model.Graphics.RectangleGeometry geometry = new Fixed.Model.Graphics.RectangleGeometry();
            geometry.Rect = new Rect(new Size(225, 350));
            path.Geometry = geometry;

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(200, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);

            Matrix pushedTransformation = (Matrix)renderer.GraphicElements[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);

            GeometryBase pushedGeometry = (GeometryBase)renderer.GraphicElements[1];
            Assert.AreEqual(geometry, pushedGeometry);
        }

        [TestMethod]
        public void RenderContentElements_WhenPathWithNullGeometry_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Path path = page.Content.AddPath();
            path.Position.Translate(200, 200);
            path.Geometry = null;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedGeometries.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectImage_ShouldPushCorrectImageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 100;
            image.Position.Translate(300, 250);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(300, 250);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectImageWithRotate_ShouldPushCorrectImageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 100;
            image.Position.Translate(300, 250);
            image.Position.Rotate(270);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(300, 250);
            expectedTransformation.Rotate(270);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectImageWithScale_ShouldPushCorrectImageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 100;
            image.Position.Translate(300, 250);
            image.Position.Scale(2.2, 3.2);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(300, 250);
            expectedTransformation.Scale(2.2, 3.2);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectImage_ShouldRenderImageSource()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 100;
            image.Position.Translate(300, 250);
            image.StencilColor = new RgbColor(122, 122, 122);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.RenderedImages.Count);

            Resources.ImageSource pushedImageSource = renderer.RenderedImages[0].Item1;
            Assert.AreEqual(imageSource, pushedImageSource);

            double width = renderer.RenderedImages[0].Item2;
            Assert.AreEqual(50, width);

            double height = renderer.RenderedImages[0].Item3;
            Assert.AreEqual(100, height);

            ColorBase stencilColor = renderer.RenderedImages[0].Item4;
            Assert.AreEqual(image.StencilColor, stencilColor);
        }

        [TestMethod]
        public void RenderContentElements_WhenCorrectImage_ShouldPushImageTransformationAndRenderImageSourceInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 100;
            image.Position.Translate(300, 250);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(300, 250);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);

            Matrix pushedTransformation = (Matrix)renderer.GraphicElements[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);

            Resources.ImageSource pushedImageSource = (Resources.ImageSource)renderer.GraphicElements[1];
            Assert.AreEqual(imageSource, pushedImageSource);
        }

        [TestMethod]
        public void RenderContentElements_WhenImageWithNullImageSource_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Image image = page.Content.AddImage();
            image.Width = 50;
            image.Height = 100;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenImageWithWidthLessThanOrEqualToZero_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 0;
            image.Height = 100;

            Resources.ImageSource imageSource1 = this.CreateEmptyImageSource();
            Image image1 = page.Content.AddImage(imageSource1);
            image1.Width = -1;
            image1.Height = 100;

            Resources.ImageSource imageSource2 = this.CreateEmptyImageSource();
            Image image2 = page.Content.AddImage(imageSource2);
            image2.Width = double.MinValue;
            image2.Height = 100;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenImageWithHeightLessThanOrEqualToZero_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 0;

            Resources.ImageSource imageSource1 = this.CreateEmptyImageSource();
            Image image1 = page.Content.AddImage(imageSource1);
            image1.Width = 50;
            image1.Height = -1;

            Resources.ImageSource imageSource2 = this.CreateEmptyImageSource();
            Image image2 = page.Content.AddImage(imageSource2);
            image2.Width = 50;
            image2.Height = double.MinValue;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragment_ShouldPushCorrectTextFragmentTranformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TextFragment textFragment = page.Content.AddTextFragment("Test");
            textFragment.Position.Translate(100, 150);

            Matrix expectedMatrix = new Matrix(1, 0, 0, -1, 100, 150);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragmentWithRotate_ShouldPushCorrectTextFragmentTranformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TextFragment textFragment = page.Content.AddTextFragment("Test");
            textFragment.Position.Translate(100, 150);
            textFragment.Position.Rotate(120);

            Matrix expectedMatrix = new Matrix(1, 0, 0, -1, 100, 150);
            expectedMatrix.Rotate(120);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragmentWithScale_ShouldPushCorrectTextFragmentTranformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TextFragment textFragment = page.Content.AddTextFragment("Test");
            textFragment.Position.Translate(100, 150);
            textFragment.Position.Scale(0.5, 2.7);

            Matrix expectedMatrix = new Matrix(1, 0, 0, -1, 100, 150);
            expectedMatrix.Scale(0.5, 2.7);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragment_ShouldRenderGlyphs()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TextFragment textFragment = page.Content.AddTextFragment("Test");
            textFragment.FontSize = 14;
            textFragment.Position.Translate(100, 150);

            Matrix[] expectedOutlinesTransformations = new Matrix[]
            {
                new Matrix(0.14, 0, 0, -0.14, 0, 0),
                new Matrix(0.14, 0, 0, -0.14, 8.554, 0),
                new Matrix(0.14, 0, 0, -0.14, 16.338, 0),
                new Matrix(0.14, 0, 0, -0.14, 23.338, 0),
            };

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            this.AssertGlyphs(textFragment, expectedOutlinesTransformations, renderer);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragment_ShouldRenderGlyphsWithoutOutlines()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TestFontSource fs = new TestFontSource();
            fs.RegisterGlyphOuntlines('b', null);

            TextFragment textFragment = page.Content.AddTextFragment("abc");
            textFragment.Font = new TestFont("testFont", fs, fs);
            textFragment.FontSize = 14;
            textFragment.Position.Translate(100, 150);

            Matrix[] expectedOutlinesTransformations = new Matrix[]
            {
                new Matrix(0.14, 0, 0, -0.14, 0, 0),
                new Matrix(0.14, 0, 0, -0.14, 0.434, 0),
            };

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            this.AssertGlyphs(textFragment, expectedOutlinesTransformations, renderer, "ac");
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragment_HasEmptyFontSource_RendersTextInsteadOfGlyphs()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TestFontSource fs = new TestFontSource();

            TextFragment textFragment = page.Content.AddTextFragment("ac");
            textFragment.Font = new TestFont("testFont", fs, EmptyFontSource.Instance);
            textFragment.FontSize = 14;
            textFragment.Position.Translate(100, 150);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.RenderedGlyphs.Count);

            Assert.AreEqual(1, renderer.RenderedTexts.Count);
            TextRenderingContext c = renderer.RenderedTexts[0];
            Assert.AreEqual(textFragment.Text, c.Text);
            Assert.AreEqual(textFragment.FontSize, c.FontSize);
            Assert.AreEqual(textFragment.TextProperties.GeometryProperties, c.GeometryProperties);
            Assert.AreEqual(textFragment.Font.Name, c.FontProperties.FontFamilyName);
            Assert.AreEqual(FontWeights.Normal, c.FontProperties.FontWeight);
            Assert.AreEqual(FontStyles.Normal, c.FontProperties.FontStyle);

            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix expectedTextPosition = renderer.PoppedTransformations[0];
            Assert.AreEqual(new Matrix(1, 0, 0, 1, 100, 150), expectedTextPosition);
        }

        [TestMethod]
        public void RenderContentElements_WhenTextFragment_ShouldPushTextTransformationAndRenderGlyphsInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            TextFragment textFragment = page.Content.AddTextFragment("a");
            textFragment.FontSize = 14;
            textFragment.Position.Translate(100, 150);

            Matrix expectedMatrix = new Matrix(1, 0, 0, -1, 100, 150);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);

            Matrix pushedTransformation = (Matrix)renderer.GraphicElements[0];
            Assert.AreEqual(expectedMatrix, pushedTransformation);

            GlyphOutlinesCollection outlines = (GlyphOutlinesCollection)renderer.GraphicElements[1];
            GlyphOutlinesCollection expectedOutlines = textFragment.Font.GetCachedOutlines(new Fixed.Model.Data.CharCode('a'));
            this.AssertOutlinesAreEqual(expectedOutlines, outlines);
        }

        [TestMethod]
        public void RenderContentElements_WhenForm_ShouldPushCorrectFormTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            formSource.BoundingBox = new Rect(0, 0, 200, 300);
            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithRotate_ShouldPushCorrectFormTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);
            form.Position.Rotate(90);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(150, 300);
            expectedTransformation.Rotate(90);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithScale_ShouldPushCorrectFormTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);
            form.Position.Scale(0.8, 3);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(150, 300);
            expectedTransformation.Scale(0.8, 3);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormIsLargerThanFormSource_ShouldPushScaledFormTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            Form form = page.Content.AddForm(formSource);
            formSource.BoundingBox = new Rect(0, 0, 100, 200);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Matrix expectedTransformation = new Matrix();
            double widthScale = form.Width / formSource.Size.Width;
            double heightScale = form.Height / formSource.Size.Height;
            expectedTransformation.Scale(widthScale, heightScale);
            expectedTransformation.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormIsSmallerThanFormSource_ShouldPushScaledFormTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            Form form = page.Content.AddForm(formSource);
            formSource.BoundingBox = new Rect(0, 0, 300, 500);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Matrix expectedTransformation = new Matrix();
            double widthScale = form.Width / formSource.Size.Width;
            double heightScale = form.Height / formSource.Size.Height;
            expectedTransformation.Scale(widthScale, heightScale);
            expectedTransformation.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithFormSourceMatrix_ShouldPushCorrectFormSourceTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            formSource.BoundingBox = new Rect(0, 0, 200, 300);
            Matrix expectedFormSourceMatrix = new Matrix(2, 0, 0, 3, 300, 450);
            formSource.Matrix = expectedFormSourceMatrix;
            Form form = page.Content.AddForm(formSource);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(2, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedFormSourceMatrix, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenForm_ShouldPushFormBoundingClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            formSource.BoundingBox = new Rect(20, 30, 100, 200);
            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Rect expectedClipping = new Rect(20, 30, 100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pushedClipping = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedClipping, pushedClipping.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenForm_ShouldPushFormTransformationFormSourceTransformationAndFormBoundingClippingInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();
            Matrix expectedFormSourceMatrix = new Matrix(2, 0, 0, 3, 300, 450);
            formSource.Matrix = expectedFormSourceMatrix;

            formSource.BoundingBox = new Rect(50, 60, 200, 100);
            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Rect expectedClipping = new Rect(50, 60, 200, 100);

            Matrix expectedTransformation = new Matrix();
            double widthScale = form.Width / formSource.Size.Width;
            double heightScale = form.Height / formSource.Size.Height;
            expectedTransformation.Scale(widthScale, heightScale);
            expectedTransformation.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(3, renderer.GraphicElements.Count);

            Matrix pushedFormTransformation = (Matrix)renderer.GraphicElements[0];
            Assert.AreEqual(expectedTransformation, pushedFormTransformation);

            Matrix pushedFormSourceTransformation = (Matrix)renderer.GraphicElements[1];
            Assert.AreEqual(expectedFormSourceMatrix, pushedFormSourceTransformation);

            GeometryBase pushedClipping = (GeometryBase)renderer.GraphicElements[2];
            Assert.AreEqual(expectedClipping, pushedClipping.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithNullFormSource_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Form form = page.Content.AddForm();
            form.Width = 200;
            form.Height = 300;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithWidthLessThanOrEqualToZero_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource1 = new FormSource();
            Form form1 = page.Content.AddForm(formSource1);
            form1.Width = 0;
            form1.Height = 300;

            FormSource formSource2 = new FormSource();
            Form form2 = page.Content.AddForm(formSource2);
            form2.Width = -1;
            form2.Height = 300;

            FormSource formSource3 = new FormSource();
            Form form3 = page.Content.AddForm(formSource3);
            form3.Width = double.MinValue;
            form3.Height = 300;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithHeightLessThanOrEqualToZero_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource1 = new FormSource();
            Form form1 = page.Content.AddForm(formSource1);
            form1.Width = 300;
            form1.Height = 0;

            FormSource formSource2 = new FormSource();
            Form form2 = page.Content.AddForm(formSource2);
            form2.Width = 300;
            form2.Height = -1;

            FormSource formSource3 = new FormSource();
            Form form3 = page.Content.AddForm(formSource3);
            form3.Width = 300;
            form3.Height = double.MinValue;

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
            Assert.AreEqual(0, renderer.PoppedClippings.Count);
            Assert.AreEqual(0, renderer.RenderedImages.Count);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithPath_ShouldPushCorrectPathTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();

            Path path = formSource.Content.AddPath();
            path.Position.Translate(100, 200);
            Fixed.Model.Graphics.PathGeometry pathGeometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = pathGeometry;

            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(3, renderer.PoppedTransformations.Count);

            Matrix pushedTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedTransformation, pushedTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithPath_ShouldRenderPathGeometry()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();

            Path path = formSource.Content.AddPath();
            path.Position.Translate(100, 200);
            Fixed.Model.Graphics.RectangleGeometry pathGeometry = new Fixed.Model.Graphics.RectangleGeometry();
            pathGeometry.Rect = new Rect(new Size(20, 30));
            path.Geometry = pathGeometry;

            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.RenderedGeometries.Count);

            GeometryBase pushedGeometry = renderer.RenderedGeometries[0].Item1;
            Assert.AreEqual(pathGeometry, pushedGeometry);

            GeometryPropertiesOwner pushedGeometryProperties = renderer.RenderedGeometries[0].Item2;
            Assert.AreEqual(path.GeometryProperties, pushedGeometryProperties);
        }

        [TestMethod]
        public void RenderContentElements_WhenFormWithPath_ShouldPushPathTransformationAndRenderGeometryInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource formSource = new FormSource();

            Path path = formSource.Content.AddPath();
            path.Position.Translate(100, 200);
            Fixed.Model.Graphics.RectangleGeometry pathGeometry = new Fixed.Model.Graphics.RectangleGeometry();
            pathGeometry.Rect = new Rect(new Size(20, 30));
            path.Geometry = pathGeometry;

            Form form = page.Content.AddForm(formSource);
            form.Width = 200;
            form.Height = 300;
            form.Position.Translate(150, 300);

            Matrix expectedTransformation = new Matrix();
            expectedTransformation.Translate(100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(5, renderer.GraphicElements.Count);

            Matrix pushedTransformation = (Matrix)renderer.GraphicElements[3];
            Assert.AreEqual(expectedTransformation, pushedTransformation);

            GeometryBase pushedGeometry = (GeometryBase)renderer.GraphicElements[4];
            Assert.AreEqual(pathGeometry, pushedGeometry);
        }

        [TestMethod]
        public void RenderContentElements_WhenNestedFormsWithPath_ShouldPushCorrectPathGeometryTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource parentFormSource = new FormSource();

            Form parentForm = page.Content.AddForm(parentFormSource);
            parentForm.Width = 300;
            parentForm.Height = 400;
            parentForm.Position.Translate(100, 200);

            FormSource childFormSource = new FormSource();

            Path path = childFormSource.Content.AddPath();
            path.Position.Translate(100, 200);
            Fixed.Model.Graphics.PathGeometry pathGeometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = pathGeometry;

            Form childForm = parentFormSource.Content.AddForm(childFormSource);
            childForm.Width = 200;
            childForm.Height = 300;
            childForm.Position.Translate(150, 300);

            Matrix expectedGeometryTransformation = new Matrix();
            expectedGeometryTransformation.Translate(100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(5, renderer.PoppedTransformations.Count);

            Matrix pushedGeometryTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedGeometryTransformation, pushedGeometryTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenNestedFormsWithPath_ShouldRenderPathGeometry()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource parentFormSource = new FormSource();

            Form parentForm = page.Content.AddForm(parentFormSource);
            parentForm.Width = 300;
            parentForm.Height = 400;
            parentForm.Position.Translate(100, 200);

            FormSource childFormSource = new FormSource();

            Path path = childFormSource.Content.AddPath();
            path.Position.Translate(100, 200);
            Fixed.Model.Graphics.PathGeometry pathGeometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = pathGeometry;

            Form childForm = parentFormSource.Content.AddForm(childFormSource);
            childForm.Width = 200;
            childForm.Height = 300;
            childForm.Position.Translate(150, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(1, renderer.RenderedGeometries.Count);

            GeometryBase pushedGeometry = renderer.RenderedGeometries[0].Item1;
            Assert.AreEqual(pathGeometry, pushedGeometry);

            GeometryPropertiesOwner pushedGeometryProperties = renderer.RenderedGeometries[0].Item2;
            Assert.AreEqual(path.GeometryProperties, pushedGeometryProperties);
        }

        [TestMethod]
        public void RenderContentElements_WhenNestedForms_ShouldPushGraphicElementsInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            FormSource parentFormSource = new FormSource();

            Form parentForm = page.Content.AddForm(parentFormSource);
            parentForm.Width = 300;
            parentForm.Height = 400;
            parentForm.Position.Translate(100, 200);

            FormSource childFormSource = new FormSource();
            childFormSource.BoundingBox = new Rect(0, 0, 200, 300);

            Path path = childFormSource.Content.AddPath();
            path.Position.Translate(500, 600);
            Fixed.Model.Graphics.PathGeometry pathGeometry = new Fixed.Model.Graphics.PathGeometry();
            path.Geometry = pathGeometry;

            Form childForm = parentFormSource.Content.AddForm(childFormSource);
            childForm.Width = 200;
            childForm.Height = 300;
            childForm.Position.Translate(150, 300);

            Matrix expectedChildFormTransformation = new Matrix();
            expectedChildFormTransformation.Translate(150, 300);

            Rect expectedChildFormClipping = new Rect(0, 0, 200, 300);

            Matrix expectedGeometryTransformation = new Matrix();
            expectedGeometryTransformation.Translate(500, 600);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();
            Rect viewport = new Rect();
            Rect containerBounds = new Rect(page.Size);

            ContentRootRenderingHelper helper = new ContentRootRenderingHelper();

            // Act
            helper.RenderContentElements(renderer, viewport, page, containerBounds);

            // Assert
            Assert.AreEqual(8, renderer.GraphicElements.Count);

            Matrix childFormTransformation = (Matrix)renderer.GraphicElements[3];
            Assert.AreEqual(expectedChildFormTransformation, childFormTransformation);

            Fixed.Model.Graphics.RectangleGeometry childFormClipping = (Fixed.Model.Graphics.RectangleGeometry)renderer.GraphicElements[5];
            Assert.AreEqual(expectedChildFormClipping, childFormClipping.Bounds);

            Matrix pushedGeometryTransformation = (Matrix)renderer.GraphicElements[6];
            Assert.AreEqual(expectedGeometryTransformation, pushedGeometryTransformation);

            GeometryBase pushedGeometry = (GeometryBase)renderer.GraphicElements[7];
            Assert.AreEqual(pathGeometry, pushedGeometry);
        }

        private void AssertGlyphs(TextFragment textFragment, Matrix[] expectedOutlinesTransformations, TestFixedContentRenderer renderer)
        {
            this.AssertGlyphs(textFragment, expectedOutlinesTransformations, renderer, textFragment.Text);
        }

        private void AssertGlyphs(TextFragment textFragment, Matrix[] expectedOutlinesTransformations, TestFixedContentRenderer renderer, string expectedRenderedText)
        {
            Assert.AreEqual(expectedRenderedText.Length, renderer.RenderedGlyphs.Count);

            for (int i = 0; i < expectedRenderedText.Length; i++)
            {
                char character = expectedRenderedText[i];

                GlyphOutlinesCollection expectedOutlines = textFragment.Font.GetCachedOutlines(new Fixed.Model.Data.CharCode(character));
                GlyphOutlinesCollection outlines = renderer.RenderedGlyphs[i].Item1;
                this.AssertOutlinesAreEqual(expectedOutlines, outlines);

                Matrix expectedOutlinesTransformation = expectedOutlinesTransformations[i];
                Matrix outlinesTransformation = renderer.RenderedGlyphs[i].Item2;
                Assert.AreEqual(expectedOutlinesTransformation, outlinesTransformation);

                GeometryPropertiesOwner geometryPropertiesOwner = renderer.RenderedGlyphs[i].Item3;
                Assert.AreEqual(textFragment.TextProperties.GeometryProperties, geometryPropertiesOwner);
            }
        }

        private void AssertOutlinesAreEqual(GlyphOutlinesCollection expected, GlyphOutlinesCollection actual)
        {
            Assert.AreEqual(expected.Count, actual.Count);

            for (int i = 0; i < expected.Count; i++)
            {
                Core.Shapes.PathFigure expectedPathFigure = expected[i];
                Core.Shapes.PathFigure actualPathFigure = expected[i];

                Assert.AreEqual(expectedPathFigure.IsClosed, actualPathFigure.IsClosed);
                Assert.AreEqual(expectedPathFigure.IsFilled, actualPathFigure.IsFilled);

                Assert.AreEqual(expectedPathFigure.StartPoint.X, actualPathFigure.StartPoint.X, 0.1);
                Assert.AreEqual(expectedPathFigure.StartPoint.Y, actualPathFigure.StartPoint.Y, 0.1);

                Assert.AreEqual(expectedPathFigure.Segments.Count, actualPathFigure.Segments.Count);

                for (int y = 0; y < expectedPathFigure.Segments.Count; y++)
                {
                    Core.Shapes.PathSegment expectedPathSegment = expectedPathFigure.Segments[y];
                    Core.Shapes.PathSegment actualPathSegment = expectedPathFigure.Segments[y];

                    Assert.AreEqual(expectedPathSegment, actualPathSegment);
                }
            }
        }

        private Resources.ImageSource CreateEmptyImageSource()
        {
            byte[] encodedData = new byte[0];
            int width = 0;
            int height = 0;
            int bitsPerComponent = 0;
            ColorSpaceBase colorSpace = null;
            string[] filters = new string[0];
            double[] decodeArray = null;
            bool imageMask = false;

            Resources.ImageSource imageSource = new Resources.ImageSource(
                encodedData, bitsPerComponent, width, height, colorSpace, filters, decodeArray, imageMask);

            return imageSource;
        }
    }
}
