﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Model.Objects;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.Tests.Model.Annotations;
using Telerik.Windows.Documents.Fixed.Utilities.Rendering;
using Telerik.Windows.Documents.Utilities;
using Telerik.Windows.Documents.Core.Tests.Utils;
using Resources = Telerik.Windows.Documents.Fixed.Model.Resources;

namespace Telerik.Windows.Documents.Fixed.Tests.Utilities.Rendering
{
    [TestClass]
    public class FixedPageContentRenderingHelperTests
    {
        [TestMethod]
        public void RenderContentElements_ShouldPushPageBoundingClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            Rect expectedBoundingRect = new Rect(0, 0, 500, 800);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageBoundingRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedBoundingRect, pageBoundingRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated90_ShouldPushPageBoundingClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            Rect expectedBoundingRect = new Rect(0, 0, 500, 800);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageBoundingRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedBoundingRect, pageBoundingRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated180_ShouldPushPageBoundingClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate180;
            Rect expectedBoundingRect = new Rect(0, 0, 500, 800);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageBoundingRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedBoundingRect, pageBoundingRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated270_ShouldPushPageBoundingClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate270;
            Rect expectedBoundingRect = new Rect(0, 0, 500, 800);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageBoundingRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedBoundingRect, pageBoundingRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenCropBox_ShouldPushCropBoxClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(-10, 20, 100, 200);
            Rect expectedCropBoxRect = new Rect(0, 20, 90, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageCropBoxRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedCropBoxRect, pageCropBoxRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenCropBoxIsEmpty_ShouldCropToZeroSizedRectangle()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = Rect.Empty;
            Rect expectedCropBoxRect = new Rect(new Size(0, 0));

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);

            GeometryBase pageCropBoxRect = renderer.PoppedClippings[0];
            Assert.AreEqual(expectedCropBoxRect, pageCropBoxRect.Bounds);
        }

#if !NETSTANDARD
        [TestMethod]
        public void RenderContentElements_WhenPageIsNotRotated_ShouldPushCorrectPageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            Matrix matrix = new Matrix();
            matrix.RotateAt(0, 500, 800);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pageRotation = renderer.PoppedTransformations[0];
            Assert.AreEqual(matrix, pageRotation);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated90_ShouldPushCorrectPageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;

            Matrix matrix = new Matrix();
            matrix.RotateAt(90, 400, 400);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pageRotation = renderer.PoppedTransformations[0];
            Assert.AreEqual(matrix, pageRotation);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated180_ShouldPushCorrectPageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate180;

            Matrix matrix = new Matrix();
            matrix.RotateAt(180, 250, 400);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pageRotation = renderer.PoppedTransformations[0];
            Assert.AreEqual(matrix, pageRotation);
        }

        [TestMethod]
        public void RenderContentElements_WhenPageRotated270_ShouldPushCorrectPageTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate270;

            Matrix matrix = new Matrix();
            matrix.RotateAt(270, 250, 250);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix pageRotation = renderer.PoppedTransformations[0];
            Assert.AreEqual(matrix, pageRotation);
        }
#endif

        [TestMethod]
        public void RenderContentElements_WhenCropBoxIntersectsMediaBoxInTopRight_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(50, -30, 150, 250);

            Matrix expectedCropBoxOffsetTransformation = new Matrix();
            expectedCropBoxOffsetTransformation.Translate(-50, 0);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCropBoxIntersectsMediaBoxInBottomLeft_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.MediaBox = new Rect(50, 30, 500, 800);
            page.CropBox = new Rect(-200, 400, 400, 250);

            Matrix expectedCropBoxOffsetTransformation = new Matrix();
            expectedCropBoxOffsetTransformation.Translate(-50, -400);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCropBoxFitsInMediaBox_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(100, 100, 400, 250);

            Matrix expectedCropBoxOffsetTransformation = new Matrix();
            expectedCropBoxOffsetTransformation.Translate(-100, -100);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }

#if !NETSTANDARD
        [TestMethod]
        public void RenderContentElements_WhenPageRotatedAndCropBoxAndMediaBoxIntersects_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(-50, 100, 400, 250);

            Matrix expectedCropBoxOffsetTransformation = new Matrix();
            expectedCropBoxOffsetTransformation.Translate(0, -100);
            expectedCropBoxOffsetTransformation.RotateAt(90, 125, 125);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }
#endif

        [TestMethod]
        public void RenderContentElements_WhenCropBoxAndMediaBoxHaveNegativeOffsets_ShouldPushCorrectCropBoxTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.MediaBox = new Rect(-2000, -2000, 500, 800);
            page.CropBox = new Rect(-1800, -1600, 400, 250);

            Matrix expectedCropBoxOffsetTransformation = new Matrix();
            expectedCropBoxOffsetTransformation.Translate(1800, 1600);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }

        [TestMethod]
        public void RenderContentElements_WhenCropBoxIsEmpty_ShouldPushNoTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = Rect.Empty;

            Matrix expectedCropBoxOffsetTransformation = new Matrix();

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);

            Matrix cropBoxOffsetTransformation = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedCropBoxOffsetTransformation, cropBoxOffsetTransformation);
        }

        [TestMethod]
        public void RenderContentElements_ShouldPushPageLevelGraphicElementsInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(-200, 200, 400, 250);

            Matrix expectedPageRotation = new Matrix();
            expectedPageRotation.Translate(0, -200);
            Rect expectedBoundingRect = new Rect(0, 200, 200, 250);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);

            Matrix contentTransformation = (Matrix)renderer.GraphicElements[0];
            Assert.AreEqual(expectedPageRotation, contentTransformation);

            GeometryBase contentBoundingRect = (GeometryBase)renderer.GraphicElements[1];
            Assert.AreEqual(expectedBoundingRect, contentBoundingRect.Bounds);
        }

        [TestMethod]
        public void RenderContentElements_WhenClipping_ShouldPushCorrectClippings()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.CropBox = new Rect(100, 100, 300, 300);

            this.CreateContentElementsGraph(page);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(5, renderer.PoppedClippings.Count);

            GeometryBase childClip = renderer.PoppedClippings[0];
            GeometryBase childFormClip = renderer.PoppedClippings[1];
            GeometryBase parentFormClip = renderer.PoppedClippings[2];
            GeometryBase parentClip = renderer.PoppedClippings[3];
            GeometryBase pageClip = renderer.PoppedClippings[4];

            this.AssertContentElementsGraphClippings(childClip, childFormClip, parentFormClip, parentClip, pageClip);
        }
#if !NETSTANDARD
        [TestMethod]
        public void RenderContentElements_WhenClipping_ShouldPushCorrectTransforamtions()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            page.Rotation = Fixed.Model.Data.Rotation.Rotate180;
            page.CropBox = new Rect(50, 100, 300, 400);

            Clipping clipping = new Clipping();
            Matrix clipMatrix = new Matrix();
            clipMatrix.Translate(125, 400);
            clipping.Transform = clipMatrix;

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 20;
            image.Height = 30;
            image.Position.Translate(200, 300);
            image.Clipping = clipping;

            Matrix expectedPageTransform = new Matrix();
            expectedPageTransform.Translate(-50, -100);
            expectedPageTransform.RotateAt(180, 150, 200);

            Matrix expectedClippingTransform = new Matrix();
            expectedClippingTransform.Translate(125, 400);

            Matrix expectedInversedClippingTransform = new Matrix();
            expectedInversedClippingTransform.Translate(-125, -400);

            Matrix expectedImageTransform = new Matrix();
            expectedImageTransform.Translate(200, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(4, renderer.PoppedTransformations.Count);

            Matrix imageTransform = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedImageTransform, imageTransform);

            Matrix inversedClippingTransform = renderer.PoppedTransformations[1];
            Assert.AreEqual(expectedInversedClippingTransform, inversedClippingTransform);

            Matrix clippingTransform = renderer.PoppedTransformations[2];
            Assert.AreEqual(expectedClippingTransform, clippingTransform);

            Matrix pageTransform = renderer.PoppedTransformations[3];
            Assert.AreEqual(expectedPageTransform, pageTransform);
        }
#endif
        [TestMethod]
        public void RenderContentElements_WhenClipping_ShouldRenderElementsInCorrectOrder()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Clipping clipping = new Clipping();
            Matrix clipMatrix = new Matrix();
            clipMatrix.Translate(125, 400);
            clipping.Transform = clipMatrix;
            clipping.Clip = new Fixed.Model.Graphics.RectangleGeometry(new Rect(new Size(45, 85)));

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = page.Content.AddImage(imageSource);
            image.Width = 20;
            image.Height = 30;
            image.Position.Translate(200, 300);
            image.Clipping = clipping;

            Matrix expectedClippingTransform = new Matrix();
            expectedClippingTransform.Translate(125, 400);

            Rect expectedClippingGeometry = new Rect(new Size(45, 85));

            Matrix expectedInversedClippingTransform = new Matrix();
            expectedInversedClippingTransform.Translate(-125, -400);

            Matrix expectedImageTransform = new Matrix();
            expectedImageTransform.Translate(200, 300);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderContentElements(renderer);

            // Assert
            Assert.AreEqual(7, renderer.GraphicElements.Count);

            Matrix clippingTransform = (Matrix)renderer.GraphicElements[2];
            Assert.AreEqual(expectedClippingTransform, clippingTransform);

            Fixed.Model.Graphics.RectangleGeometry clippingGeometry = (Fixed.Model.Graphics.RectangleGeometry)renderer.GraphicElements[3];
            Assert.AreEqual(expectedClippingGeometry, clippingGeometry.Bounds);

            Matrix inversedClippingTransform = (Matrix)renderer.GraphicElements[4];
            Assert.AreEqual(expectedInversedClippingTransform, inversedClippingTransform);

            Matrix imageTransform = (Matrix)renderer.GraphicElements[5];
            Assert.AreEqual(expectedImageTransform, imageTransform);

            Resources.ImageSource pushedImageSource = (Resources.ImageSource)renderer.GraphicElements[6];
            Assert.AreEqual(imageSource, pushedImageSource);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenNoNormalAppearance_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenNormalAppearance_ShouldPushGraphicElements()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            widgetContent.NormalContentSource = new FormSource();
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenScaledAndTranslated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.Rect = new Rect(100, 200, 4, 2);
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            appearance.Size = new Size(1, 1);
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            Matrix expectedMatrix = new Matrix(4, 0, 0, 2, 100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenPageNotRotated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(0, 500, 800);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenPageRotated90_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(90, 400, 400);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenPageRotated180_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate180;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(180, 250, 400);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenPageRotated270_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate270;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(270, 250, 250);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenTranslatedScaledAndRotated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            page.Size = new Size(500, 800);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.Rect = new Rect(100, 200, 4, 2);
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Matrix appearanceMatrix = new Matrix(0.5, 0, 0, 0.5, 0, 0);
            appearance.Matrix = appearanceMatrix;
            appearance.Size = new Size(1, 1);
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            Matrix scaleMatrix = new Matrix(4, 0, 0, 2, 100, 200);
            scaleMatrix = appearanceMatrix.MultiplyBy(scaleMatrix);

            Matrix pageMatrix = new Matrix();
            pageMatrix = pageMatrix.RotateAtCenter(90, 400, 400);

            Matrix expectedMatrix = scaleMatrix.MultiplyBy(pageMatrix);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_ShouldPushCorrectClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Rect expectedClipping = new Rect(50, 100, 25, 30);
            appearance.BoundingBox = expectedClipping;
            widgetContent.NormalContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);
            Rect clipping = renderer.PoppedClippings[0].Bounds;
            Assert.AreEqual(expectedClipping, clipping);
        }

        [TestMethod]
        public void RenderNormalAnnotationAppearance_WhenClipping_ShouldPushCorrectClippings()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Rect appearanceClipping = new Rect(100, 100, 300, 300);
            appearance.BoundingBox = appearanceClipping;
            widgetContent.NormalContentSource = appearance;

            this.CreateContentElementsGraph(appearance);

            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderNormalAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(5, renderer.PoppedClippings.Count);

            GeometryBase childClip = renderer.PoppedClippings[0];
            GeometryBase childFormClip = renderer.PoppedClippings[1];
            GeometryBase parentFormClip = renderer.PoppedClippings[2];
            GeometryBase parentClip = renderer.PoppedClippings[3];
            GeometryBase appearanceClip = renderer.PoppedClippings[4];

            this.AssertContentElementsGraphClippings(childClip, childFormClip, parentFormClip, parentClip, appearanceClip);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenNoDownAndNoNormalAppearance_ShouldDoNothing()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(0, renderer.GraphicElements.Count);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenNormalAppearance_ShouldPushGraphicElements()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            widgetContent.NormalContentSource = new FormSource();
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(2, renderer.GraphicElements.Count);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenScaledAndTranslated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.Rect = new Rect(100, 200, 4, 2);
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            appearance.Size = new Size(1, 1);
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            Matrix expectedMatrix = new Matrix(4, 0, 0, 2, 100, 200);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenPageNotRotated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix.RotateAtCenter(0, 500, 800);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenPageRotated90_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(90, 400, 400);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenPageRotated180_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate180;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(180, 250, 400);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenPageRotated270_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate270;
            page.Size = new Size(500, 800);

            Matrix expectedMatrix = new Matrix();
            expectedMatrix = expectedMatrix.RotateAtCenter(270, 250, 250);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenTranslatedScaledAndRotated_ShouldPushCorrectTransformation()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            page.Rotation = Fixed.Model.Data.Rotation.Rotate90;
            page.Size = new Size(500, 800);

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.Rect = new Rect(100, 200, 4, 2);
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Matrix appearanceMatrix = new Matrix(0.5, 0, 0, 0.5, 0, 0);
            appearance.Matrix = appearanceMatrix;
            appearance.Size = new Size(1, 1);
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            Matrix scaleMatrix = new Matrix(4, 0, 0, 2, 100, 200);
            scaleMatrix = appearanceMatrix.MultiplyBy(scaleMatrix);

            Matrix pageMatrix = new Matrix();
            pageMatrix = pageMatrix.RotateAtCenter(90, 400, 400);

            Matrix expectedMatrix = scaleMatrix.MultiplyBy(pageMatrix);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedTransformations.Count);
            Matrix matrix = renderer.PoppedTransformations[0];
            Assert.AreEqual(expectedMatrix, matrix);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_ShouldPushCorrectClipping()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();
            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Rect expectedClipping = new Rect(50, 100, 25, 30);
            appearance.BoundingBox = expectedClipping;
            widgetContent.MouseDownContentSource = appearance;
            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(1, renderer.PoppedClippings.Count);
            Rect clipping = renderer.PoppedClippings[0].Bounds;
            Assert.AreEqual(expectedClipping, clipping);
        }

        [TestMethod]
        public void RenderDownAnnotationAppearance_WhenClipping_ShouldPushCorrectClippings()
        {
            // Arrange
            RadFixedPage page = new RadFixedPage();

            TestWidgetAnnotation annotation = new TestWidgetAnnotation();
            annotation.SetParent(page);
            AnnotationContentSource widgetContent = new AnnotationContentSource();
            FormSource appearance = new FormSource();
            Rect appearanceClipping = new Rect(100, 100, 300, 300);
            appearance.BoundingBox = appearanceClipping;
            widgetContent.MouseDownContentSource = appearance;

            this.CreateContentElementsGraph(appearance);

            annotation.Appearances = new SingleStateAppearances(widgetContent);

            TestFixedContentRenderer renderer = new TestFixedContentRenderer();

            FixedPageContentRenderingHelper helper = new FixedPageContentRenderingHelper(page);

            // Act
            helper.RenderDownAnnotationAppearance(renderer, annotation);

            // Assert
            Assert.AreEqual(5, renderer.PoppedClippings.Count);

            GeometryBase childClip = renderer.PoppedClippings[0];
            GeometryBase childFormClip = renderer.PoppedClippings[1];
            GeometryBase parentFormClip = renderer.PoppedClippings[2];
            GeometryBase parentClip = renderer.PoppedClippings[3];
            GeometryBase appearanceClip = renderer.PoppedClippings[4];

            this.AssertContentElementsGraphClippings(childClip, childFormClip, parentFormClip, parentClip, appearanceClip);
        }

        private void CreateContentElementsGraph(IContentRootElement contentRoot)
        {
            Clipping parentClipping = new Clipping();
            Fixed.Model.Graphics.RectangleGeometry parentClip = new Fixed.Model.Graphics.RectangleGeometry();
            parentClip.Rect = new Rect(new Size(200, 250));
            parentClipping.Clip = parentClip;

            Clipping childClipping = new Clipping();
            childClipping.Clipping = parentClipping;

            Fixed.Model.Graphics.RectangleGeometry childClip = new Fixed.Model.Graphics.RectangleGeometry();
            childClip.Rect = new Rect(new Size(50, 50));
            childClipping.Clip = childClip;

            Resources.ImageSource imageSource = this.CreateEmptyImageSource();
            Image image = contentRoot.Content.AddImage(imageSource);
            image.Width = 50;
            image.Height = 40;
            image.Clipping = childClipping;

            FormSource parentFormSource = new FormSource();
            parentFormSource.BoundingBox = new Rect(5, 5, 150, 250);
            Form parentForm = contentRoot.Content.AddForm(parentFormSource);
            parentForm.Width = 200;
            parentForm.Height = 300;
            parentForm.Clipping = parentClipping;

            FormSource childFormSource = new FormSource();
            childFormSource.BoundingBox = new Rect(10, 10, 100, 120);
            Form childForm = parentFormSource.Content.AddForm(childFormSource);
            childForm.Width = 100;
            childForm.Height = 120;

            TextFragment textFragment = childFormSource.Content.AddTextFragment("Text");
        }

        private void AssertContentElementsGraphClippings(GeometryBase childClip, GeometryBase childFormClip,
            GeometryBase parentFormClip, GeometryBase parentClip, GeometryBase pageClip)
        {
            Rect expectedChildClip = new Rect(new Size(50, 50));
            Assert.AreEqual(expectedChildClip, childClip.Bounds);

            Rect expectedChildFormClip = new Rect(10, 10, 100, 120);
            Assert.AreEqual(expectedChildFormClip, childFormClip.Bounds);

            Rect expectedParentFormClip = new Rect(5, 5, 150, 250);
            Assert.AreEqual(expectedParentFormClip, parentFormClip.Bounds);

            Rect expectedParentClip = new Rect(new Size(200, 250));
            Assert.AreEqual(expectedParentClip, parentClip.Bounds);

            Rect expectedPageClip = new Rect(100, 100, 300, 300);
            Assert.AreEqual(expectedPageClip, pageClip.Bounds);
        }

        private Resources.ImageSource CreateEmptyImageSource()
        {
            byte[] encodedData = new byte[0];
            int width = 0;
            int height = 0;
            int bitsPerComponent = 0;
            ColorSpaceBase colorSpace = null;
            string[] filters = new string[0];
            double[] decodeArray = null;
            bool imageMask = false;

            Resources.ImageSource imageSource = new Resources.ImageSource(
                encodedData, bitsPerComponent, width, height, colorSpace, filters, decodeArray, imageMask);

            return imageSource;
        }
    }
}
