﻿using System;
using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Graphics;
using Telerik.Windows.Documents.Fixed.Utilities.Rendering;

namespace Telerik.Windows.Documents.Fixed.Tests.Utilities.Rendering
{
    internal class TestFixedContentRenderer : IFixedContentRenderer
    {
        public TestFixedContentRenderer()
        {
            this.PoppedClippings = new List<GeometryBase>();
            this.PoppedTransformations = new List<Matrix>();
            this.GraphicElements = new List<object>();
            this.RenderedGeometries = new List<Tuple<GeometryBase, GeometryPropertiesOwner>>();
            this.RenderedImages = new List<Tuple<Fixed.Model.Resources.ImageSource, double, double, ColorBase>>();
            this.RenderedGlyphs = new List<Tuple<GlyphOutlinesCollection, Matrix, GeometryPropertiesOwner>>();
            this.RenderedTexts = new List<TextRenderingContext>();
        }

        public List<GeometryBase> PoppedClippings { get; set; }

        public List<Matrix> PoppedTransformations { get; set; }

        public List<object> GraphicElements { get; set; }

        public List<Tuple<GeometryBase, GeometryPropertiesOwner>> RenderedGeometries { get; set; }

        public List<Tuple<Fixed.Model.Resources.ImageSource, double, double, ColorBase>> RenderedImages { get; set; }

        public List<Tuple<GlyphOutlinesCollection, Matrix, GeometryPropertiesOwner>> RenderedGlyphs { get; set; }

        public List<TextRenderingContext> RenderedTexts { get; private set; }

        public IDisposable PushClipping(GeometryBase geometry)
        {
            this.GraphicElements.Add(geometry);

            return new TestDisposableObject<GeometryBase>(this.PoppedClippings.Add, geometry);
        }

        public IDisposable PushTransform(Matrix matrix)
        {
            this.GraphicElements.Add(matrix);

            return new TestDisposableObject<Matrix>(this.PoppedTransformations.Add, matrix);
        }

        public void RenderGeometry(GeometryRenderingContext c)
        {
            this.GraphicElements.Add(c.Geometry);

            this.RenderedGeometries.Add(new Tuple<GeometryBase, GeometryPropertiesOwner>(c.Geometry, c.Properties));
        }

        public void RenderGlyph(GlyphRenderingContext c)
        {
            this.GraphicElements.Add(c.Outlines);

            this.RenderedGlyphs.Add(new Tuple<GlyphOutlinesCollection, Matrix, GeometryPropertiesOwner>(c.Outlines, c.OutlinesTransformation, c.Properties));
        }

        public void RenderImage(ImageRenderingContext c)
        {
            this.GraphicElements.Add(c.ImageSource);

            this.RenderedImages.Add(new Tuple<Fixed.Model.Resources.ImageSource, double, double, ColorBase>(c.ImageSource, c.RenderingWidth, c.RenderingHeight, c.StencilColor));
        }

        public void RenderText(TextRenderingContext c)
        {
            this.RenderedTexts.Add(c);
        }

        private class TestDisposableObject<T> : IDisposable
        {
            private readonly Action<T> action;
            private readonly T funcParam;

            public TestDisposableObject(Action<T> action, T funcParam)
            {
                this.action = action;
                this.funcParam = funcParam;
            }

            public void Dispose()
            {
                this.action(this.funcParam);
            }
        }
    }
}
