﻿using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Fixed.Tests.Utils
{
    public class DocumentFixedVisualBaseTest : IntegrationTestClassBase
    {
        protected virtual void AssertVisual(RadFixedDocument document, Action action = null, int repeat = 0)
        {
            this.AssertAreVisualyEqual(document, action, repeat);
        }

        public void AssertAreVisualyEqual(RadFixedDocument document, PdfExportSettings settings = null)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            if (settings != null)
            {
                provider.ExportSettings = settings;
            }

            byte[] data = provider.Export(document);
            this.VisualTestHelper.Assert(data, PDFFILEEXTENSION, null, 0);
#endif
        }

        public void AssertAreVisualyEqual(RadFixedDocument document, Action action, int actionExecuteCount = 1)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            byte[] data = provider.Export(document);
            this.VisualTestHelper.Assert(data, PDFFILEEXTENSION, action, actionExecuteCount);
#endif
        }

        public RadFixedDocument LoadVisualTestDocument(string documentName)
        {
            var source = VisualTestHelper.LoadVisualTestDocumentFromByteArray(documentName);
            RadFixedDocument document = new PdfFormatProvider().Import(source);

            return document;
        }

        protected static ImageSource GetImageAsImageSource(string name)
        {
            var byteArray = File.ReadAllBytes(name);
            Stream stream = new MemoryStream(byteArray);
            var imageSource = new ImageSource(stream);

            return imageSource;
        }

        protected string GetInitialDocumentPath(string documentName)
        {
            return FolderSettings.VisualTestsDocuments + documentName;
        }

        protected string GetResultDocumentPath(string documentName)
        {
            return Path.Combine(FolderSettings.ResultDirName, documentName);
        }
    }
}

