﻿using System;
using Telerik.Windows.Documents.Utilities;
#if NETSTANDARD
using Telerik.Documents.Primitives;
using Telerik.Windows.Documents.Core.Shapes;
#else
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Fonts.Glyphs;
#endif

namespace Telerik.Windows.Documents.Core.Tests.Utils
{
    internal static class Extensions
    {
        public const double Epsilon = 1E-8;

        public static bool IsEqualTo(this double a, double b, double epsilon = Epsilon)
        {
            return Math.Abs(a - b) < epsilon;
        }

        public static bool IsGreaterThanOrEqualTo(this double a, double b, double epsilon = Epsilon)
        {
            return a > b || a.IsEqualTo(b, epsilon);
        }

        public static bool IsLessThanOrEqualTo(this double a, double b, double epsilon = Epsilon)
        {
            return a < b || a.IsEqualTo(b, epsilon);
        }

        public static Matrix RotateAtCenter(this Matrix matrix, double rotationAngle, double centerX, double centerY)
        {
#if SILVERLIGHT || NETSTANDARD
            return matrix.RotateMatrixAt(rotationAngle, centerX, centerY);
#else
            matrix.RotateAt(rotationAngle, centerX, centerY);

            return matrix;
#endif
        }

        private static PathSegment CreatePathSegment(Matrix textRenderingMatrix, Shapes.PathSegment segment, bool isStroked)
        {
            if (segment is Shapes.LineSegment)
            {
                Shapes.LineSegment s = (Shapes.LineSegment)segment;
                LineSegment ls = new LineSegment();
                ls.Point = textRenderingMatrix.Transform(s.Point);
#if !NETSTANDARD
#if WPF
                ls.IsStroked = isStroked;
#endif
#endif
                return ls;
            }
            else if (segment is Shapes.BezierSegment)
            {
                Shapes.BezierSegment s = (Shapes.BezierSegment)segment;
                BezierSegment bs = new BezierSegment();
                bs.Point1 = textRenderingMatrix.Transform(s.Point1);
                bs.Point2 = textRenderingMatrix.Transform(s.Point2);
                bs.Point3 = textRenderingMatrix.Transform(s.Point3);
#if !NETSTANDARD
#if WPF
                bs.IsStroked = isStroked;
#endif
#endif
                return bs;
            }
            else if (segment is Shapes.QuadraticBezierSegment)
            {
                Shapes.QuadraticBezierSegment s = (Shapes.QuadraticBezierSegment)segment;
                QuadraticBezierSegment qbs = new QuadraticBezierSegment();
                qbs.Point1 = textRenderingMatrix.Transform(s.Point1);
                qbs.Point2 = textRenderingMatrix.Transform(s.Point2);
#if !NETSTANDARD
#if WPF
                qbs.IsStroked = isStroked;
#endif
#endif
                return qbs;
            }
            return null;
        }
        private static PathFigure CreatePathFigure(Matrix textRenderingMatrix, Shapes.PathFigure figure, bool isFilled, bool isStroked)
        {
            PathFigure res = new PathFigure();
            foreach (var s in figure.Segments)
            {
                var ps = CreatePathSegment(textRenderingMatrix, s, isStroked);
                if (ps != null)
                {
                    res.Segments.Add(ps);
                }
            }
            res.StartPoint = textRenderingMatrix.Transform(figure.StartPoint);
            res.IsFilled = isFilled;
            return res;
        }

#if !NETSTANDARD
        public static Geometry Render(this Glyph glyph, Matrix textRenderingMatrix, bool isFilled, bool isStroked)
        {
            PathGeometry pg = new PathGeometry();
            foreach (var f in glyph.Outlines)
            {
                pg.Figures.Add(CreatePathFigure(textRenderingMatrix, f, isFilled, isStroked));
            }
            pg.FillRule = FillRule.Nonzero;
            return pg;
        }
#endif
    }
}
