﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

using Telerik.Windows.Documents.Fixed.Tests.Utils;

namespace DPL_Visual
{
    public class ImageComparerHelper : Telerik.Windows.Documents.Fixed.Tests.Integration.IntegrationTestClassBase
    {
        private static int counter = 1;

        public static bool CompareImages(string expectedImagePath, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            Bitmap exp = GetSavedImageAsBitmap(expectedImagePath);

            Bitmap container;
            Bitmap act;
            bool equal;

            equal = ImageComparer(similarPixelArgbTolerance, differentPixelsPercentTolerance, similarPixelsPercentTolerance, exp, out container, out act);

            if (equal == false)
            {
                Thread.Sleep(1000);
                equal = ImageComparer(similarPixelArgbTolerance, differentPixelsPercentTolerance, similarPixelsPercentTolerance, exp, out container, out act);
            }

            if (equal == false)
            {
                var image = System.IO.Path.GetFileName(expectedImagePath);

                string imageName = System.IO.Path.GetFileNameWithoutExtension(image);
                var curruntTestPath = CreateTestResultFolders(expectedImagePath);

                exp.Save(curruntTestPath + "\\" + imageName + "_expected.png", System.Drawing.Imaging.ImageFormat.Png);
                container.Save(curruntTestPath + "\\" + imageName + "_diff.png", System.Drawing.Imaging.ImageFormat.Png);
                act.Save(curruntTestPath + "\\" + imageName + "_actual.png", System.Drawing.Imaging.ImageFormat.Png);
            }

            return equal;
        }

        private static Bitmap GetSavedImageAsBitmap(string expectedImagePath)
        {
            Bitmap exp;
            try
            {
                exp = new Bitmap(PdfProcessingFolder + expectedImagePath);
            }
            catch
            {
                CaptureAndSave(PdfProcessingFolder + expectedImagePath);
                exp = new Bitmap(PdfProcessingFolder + expectedImagePath);
            }

            return exp;
        }

        // Image comparer method
        private static bool ImageComparer(double similarPixelArgbTolerance, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, Bitmap exp, out Bitmap container, out Bitmap act)
        {
            Thread.Sleep(2000);
            bool equal = true;
            var rect = CapturePartOfDesktop(Extern.GetForegroundWindow());

            int w = Math.Min(exp.Width, rect.Width);
            int h = Math.Min(exp.Height, rect.Height);

            int diferentPixels = 0;
            int similarPixels = 0;
            int totalPixels = w * h;

            container = new Bitmap(w, h);
            for (int i = 0; i < w; ++i)
            {
                for (int j = 0; j < h; ++j)
                {
                    System.Drawing.Color secondColor = rect.GetPixel(i, j);
                    System.Drawing.Color firstColor = exp.GetPixel(i, j);

                    if (firstColor != secondColor)
                    {
                        if ((Math.Abs(firstColor.A - secondColor.A) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.B - secondColor.B) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.G - secondColor.G) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.R - secondColor.R) < similarPixelArgbTolerance))
                        {
                            similarPixels++;
                            container.SetPixel(i, j, System.Drawing.Color.Green);
                        }
                        else
                        {
                            diferentPixels++;
                            container.SetPixel(i, j, System.Drawing.Color.Red);
                        }
                    }
                    else
                    {
                        container.SetPixel(i, j, firstColor);
                    }
                }
            }

            double diffPercent = ((double)diferentPixels / (double)totalPixels) * 100;
            double similarPercent = ((double)similarPixels / (double)totalPixels) * 100;

            if ((diffPercent > differentPixelsPercentTolerance) ||
                (similarPercent > similarPixelsPercentTolerance))
            {
                equal = false;
            }

            act = rect;

            return equal;
        }

        private static string CreateTestResultFolders(string testName)
        {
            var latestBuildFolder = GetLatestBuildFolder();

            var mainFolder = FolderSettings.ImageVerificationResultFolder + latestBuildFolder;

            if (!Directory.Exists(mainFolder))
            {
                Directory.CreateDirectory(mainFolder);
            }

            var dpl = mainFolder + @"/" + PdfProcessingControl;

            if (!System.IO.Directory.Exists(dpl))
            {
                System.IO.Directory.CreateDirectory(dpl);
            }

            return dpl;
        }

        private static string GetLatestBuildFolder()
        {
            try
            {
                var mainDirectory = new DirectoryInfo(FolderSettings.BuildResults)
                                        .GetDirectories("Centaur*.*", SearchOption.AllDirectories)
                                        .OrderBy(x => x.CreationTime).ToList();

                var buildFolder = mainDirectory.LastOrDefault();

                return buildFolder.Name;
            }
            catch
            {
                string time = DateTime.Now.ToString();
                time = time.Replace(@"/", "");
                time = time.Replace(" ", "");
                time = time.Replace(":", "");
                return @"\Test_" + time;
            }
        }

        /// <summary> Capture the part of active window (default) or the desktop and return it as a bitmap </summary>
        /// <param name="mode">Optional. The default value is CaptureMode.Window.</param>
        private static Bitmap CapturePartOfDesktop(IntPtr handle)
        {
            Point p = new Point(0, 0);
            System.Windows.Forms.Cursor.Position = p;
            Thread.Sleep(50);
            System.Drawing.Rectangle bounds;
            var rect = new Rect();
            Extern.GetWindowRect(handle, ref rect);
            bounds = new System.Drawing.Rectangle(0, 0, rect.Right - rect.Left, rect.Bottom - 62);

            var result = new Bitmap(bounds.Width, bounds.Height);

            using (var g = Graphics.FromImage(result))
            {
                g.CopyFromScreen(new System.Drawing.Point(bounds.Left, bounds.Top), new System.Drawing.Point(bounds.Left, bounds.Top), bounds.Size);
            }

            return result;
        }

        /// <summary> Capture the active window (default) or the desktop and return it as a bitmap </summary>
        /// <param name="mode">Optional. The default value is CaptureMode.Window.</param>
        private static Bitmap CaptureDesktop(IntPtr handle)
        {
            Point p = new Point(0, 0);
            System.Windows.Forms.Cursor.Position = p;
            Thread.Sleep(50);
            System.Drawing.Rectangle bounds;
            var rect = new Rect();
            Extern.GetWindowRect(handle, ref rect);
            bounds = new System.Drawing.Rectangle(rect.Left, rect.Top, rect.Right - rect.Left, rect.Bottom - rect.Top);

            var result = new Bitmap(bounds.Width, bounds.Height);

            using (var g = Graphics.FromImage(result))
            {
                g.CopyFromScreen(new System.Drawing.Point(bounds.Left, bounds.Top), System.Drawing.Point.Empty, bounds.Size);
            }

            return result;
        }

        /// <summary> Capture the active window (default) or the desktop and save it as a bitmap </summary>
        /// <param name="mode">Optional. The default value is CaptureMode.Window.</param>
        private static void CaptureAndSave(string filename)
        {
            ImageSave(filename, System.Drawing.Imaging.ImageFormat.Png, CapturePartOfDesktop(Extern.GetForegroundWindow()));
        }

        /// <summary> Save an image to a specific file </summary>
        /// <param name="format">Optional file save mode.  Default is PNG</param>
        /// <param name="image">Image to save.  Usually a BitMap, but can be any
        /// Image.</param>
        private static void ImageSave(string filename, System.Drawing.Imaging.ImageFormat format, Bitmap image)
        {
            //if (!filename.Contains("."))
            //   filename = filename.Trim() + "." + format.ToString().ToLower();

            //if (!filename.Contains(@"\"))
            //filename = Path.Combine(Environment.GetEnvironmentVariable("TEMP") ?? @"C:\Temp", filename);
            //filename = @"C:\Images\Images\" + filename;
            //filename = filename.Replace("%NOW%", DateTime.Now.ToString("yyyy-MM-dd@hh.mm.ss"));

            if (!Directory.Exists(PdfProcessingFolder))
            {
                Directory.CreateDirectory(PdfProcessingFolder);
            }

            image.Save(filename, format);
            System.Threading.Thread.Sleep(1000);
        }
    }
}