﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Fonts;
using Telerik.Windows.Documents.Core.Fonts.OpenType;
using Telerik.Windows.Documents.Core.TextMeasurer;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model;

namespace Telerik.Windows.Documents.Core.Tests.Utils
{
    internal class TestHelper
    {
        
        public const string TestString = "1234567890’_-–/|:[]().,!~'\"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZабвгдежзийклмнопрстуфхцчшщъьюяАБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЬЮЯ中國哲學書電子化計劃";
        
        public const string TestStringWithArabicSymbols = "1234567890’_-–/|:[]().,!~'\"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZабвгдежзийклмнопрстуфхцчшщъьюяАБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЬЮЯ中國哲學書電子化計劃لوحة المفاتيح العربية";
        
        public static readonly TextProperties SampleString = new TextProperties("123456 7890’_- –/|:[]().,!~'\"abcde fghijklmnopq rstuvwx yzABCDEFGHIJKLMNO PQRSTUVWXYZабвгде жзийклмноп рстуфхцчшщъь юяАБВГДЕЖ ЗИЙКЛМНО ПРСТ УФХЦЧШЩ ЪЬЮЯ", 12);
        
        private static readonly string[] standardFontFamilies = new string[] { "Arial", "Arial Black", "Calibri", "Comic Sans MS", "Courier New", "Georgia", "Lucida Sans Unicode", "Times New Roman", "Trebuchet MS", "Verdana" };
        private static readonly string[] fallbackFontFamilies = new string[] {
                                                                                "Arial", // Arabic, Hebrew
                                                                                "Mangal", // Hindi
                                                                                "MS Mincho", "MS Gothic", // Japanese
                                                                                "SimSun", "NSimSun", // Chinese (Proportional, Monospaced)
                                                                                "Malgun Gothic" // Korean
                                                                             };
        private static readonly string[] fontFamilies = standardFontFamilies.Union(fallbackFontFamilies).ToArray();
        private static readonly string[] sampleWords = SampleLongString.Replace(Environment.NewLine, " ").Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
        private static readonly string[] sampleWordsDifferentLanguages = (SampleLongString + " " + SampleLongStringRtl + " " + SampleLongStringCh).Replace(Environment.NewLine, " ").Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

        public static readonly string SampleParagraph = @"Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar";

        public const string SampleLongStringRtl = @"أعلن رئيس المجلس العسكري الأعلى للجيش السوري الحر اللواء سليم إدريس اليوم أنه فقد الأمل بوصول مساعدات عسكرية غربية إلى الثوار، في حين أكد مصدر من المعارضة المسلحة الإعداد لهجوم شامل هدفه إحكام السيطرة على مدينة حلب وريفها في شمال سوريا بدعم من المملكة العربية السعودية.
وتعليقاً على تصريحات صادرة من واشنطن والاتحاد الأوروبي وفرنسا أكّدت فيها للمعارضة أنها سترسل إليها مساعدات عسكرية، قال إدريس لوكالة أنباء الأناضول التركية فقدنا أملنا بالغرب، لكن ليس بالله، مضيفا لا نفهم موقف الغرب، وعدونا في البداية برفع الحظر عن بيع الأسلحة إلى سوريا، ثم عدلوا عن ذلك.
كما وجّه إدريس انتقادات إلى رئيس الوزراء البريطاني ديفد كاميرون، قائلا إنه يعذّبه، مشيرا إلى أن سبب عدم وصول مساعدات عسكرية من الاتحاد الأوروبي أو من الولايات المتحدة اعتبارهم تطميناتنا حول عدم وقوع الأسلحة في أيدي جماعات متطرفة غير مناسبة.
وأعرب مسؤولون أميركيون أمس عن تشاؤمهم حيال تمرير خطط تسليح المعارضة السورية سريعا، معتبرين أن خطط تسليح الثوار تخطت عقبة في الكونغرس لكنها قد تواجه المزيد من العقبات عندما يجري تنفيذ عملية التمويل خلال شهرين، مما يزيد من تأخر تدفق الأسلحة
";
        public const string SampleLongStringCh = @"按一般规律，从一名普通科员提升为一名正科级干部，需要6年时间，对于一名22岁的本科毕业生来说，正科任职年龄得满28岁。5月初，31位平均年龄只有26.2岁的怀柔区基层年轻干部通过公选方式走上了正科级实职岗位。
据了解，此次公开选拔，怀柔区一次性拿出31个单位团（工）委书记的职位，将报名年龄限定在1985年7月1日以后出生。报名人员可以是副科级干部，也可以是普通科员。
根据党政领导干部选拔任用相关工作条例，特别优秀的年轻干部或者工作特殊需要的，可以破格提拔。破格提拔时，可适当放宽对任职资格要求的限制。怀柔区此次共拿出31个正科级实职岗位，破格的规模和力度前所未有。
这次公选的初衷，一方面是为了改善当地干部的专业和年龄结构，通过公选的平台发现人才；另一方面是为了充实后备人才库，为今后领导班子建设储备干部。怀柔区委组织部副处级组织员翟文岩说。
“参加工作才四年多一点，这些年轻干部的资历、履历、业绩能够服众么？”有人提出这样的疑问。翟文岩表示，公选对干部的资格认定采用统一标准，对于资历履历相对单一的年轻干部而言，统一的选拔标准为他们提供了公平的竞争环境。公选不仅仅看重干部干过些什么，更要考察他能干什么，是否符合公选岗位的要求。也就是更关注他们的发展潜质。因此，选拔的各个环节设计就会更为严格。
根据要求，所有报考者不能直接填报31个单位的某一具体职位，而要先参加笔试，笔试成绩优异者，优先选报岗位。避免了因报考本单位而带来的“近水楼台”、“人选内定”等非客观因素。
此外，怀柔区组织部门在面试评委的组织方式上进行了创新，实行“大评委制”。评委既有来自组织部、人力社保局、团区委等单位的7个专业考官，也有10名区级“两代表一委员”组成的群众评委团队。众多报考者认为，这是一次“多数人选多数人”的选拔。
“相对于众多专业性岗位而言，团委书记的工作涉及众多领域，对报考者的综合素质要求更高，是一个年轻人的展示舞台。同时，这一岗位的灵活性很强，干得好的人可以转岗担任综合性领导职务，能力欠缺的人也可被调整至其他岗位上，而不影响中心工作，降低了用人风险。”翟文岩说。
据了解，新上任的团委书记们将经历军事化训练、岗前教育等多个培训环节。组织部门将对他们的工作表现进行长期跟踪，并有意识地让他们参与一些专项工作，提升工作能力。（记者 吴迪 通讯员 温来生）
";
        public const string SampleLongString =
@"2.Project Tesseract – Motivation and History
Tesseract is an open-source OCR engine that was developed at HP between 1984 and 1994. Like a supernova, it appeared from nowhere for the 1995 UNLV Annual Test of OCR Accuracy, shone brightly with its results, and then vanished back under the same cloak of secrecy under which it had been developed. Now for the first time, details of the architecture and algorithms can be revealed.
Tesseract began as a PhD research project in HP Labs, Bristol, and gained momentum as a possible software and/or hardware add-on for HP’s line of flatbed scanners. Motivation was provided by the fact that the commercial OCR engines of the day were in their infancy, and failed miserably on anything but the best quality print. After a joint project between HP Labs Bristol, and HP’s scanner division in Colorado, Tesseract had a significant lead in accuracy over the commercial engines, but did not become a product. The next stage of its development was back in HP Labs Bristol as an investigation of OCR for compression. Work concentrated more on improving rejection efficiency than on base-level accuracy. At the end of this project, at the end of 1994, development ceased entirely. The engine was sent to UNLV for the 1995 Annual Test of OCR Accuracy, where it proved its worth against the commercial engines of the time. In late 2005, HP released Tesseract for open source. It is now available at a. 
3.Architecture
Since HP had independently-developed page layout analysis technology that was used in products, (and therefore not released for open-source) Tesseract never needed its own page layout analysis. Tesseract therefore assumes that its input is a binary image with optional polygonal text regions defined.
 
Processing follows a traditional step-by-step pipeline, but some of the stages were unusual in their day, and possibly remain so even now. The first step is a connected component analysis in which outlines of the components are stored. This was a computationally expensive design decision at the time, but had a significant advantage: by inspection of the nesting of outlines, and the number of child and grandchild outlines, it is simple to detect inverse text and recognize it as easily as black-on-white text. Tesseract was probably the first OCR engine able to handle white-on-black text so trivially. At this stage, outlines are gathered together, purely by nesting, into Blobs.
Blobs are organized into text lines, and the lines and regions are analyzed for fixed pitch or proportional text. Text lines are broken into words differently according to the kind of character spacing. Fixed pitch text is chopped immediately by character cells. Proportional text is broken into words using definite spaces and fuzzy spaces.
Recognition then proceeds as a two-pass process. In the first pass, an attempt is made to recognize each word in turn. Each word that is satisfactory is passed to an adaptive classifier as training data. The adaptive classifier then gets a chance to more accurately recognize text lower down the page. 
Since the adaptive classifier may have learned something useful too late to make a contribution near the top of the page, a second pass is run over the page, in which words that were not recognized well enough are recognized again. 
A final phase resolves fuzzy spaces, and checks alternative hypotheses for the x-height to locate smallcap text.
4.Adaptive Tresholding
Thresholding is the simplest method of image segmentation. From a grayscale image, thresholding can be used to create binary images. 
During the thresholding process, individual pixels in an image are marked as object pixels if their value is greater than some threshold value (assuming an object to be brighter than the background) and as background pixels otherwise. This convention is known as threshold above. Variants include threshold below, which is opposite of threshold above; threshold inside, where a pixel is labeled object if its value is between two thresholds; and threshold outside, which is the opposite of threshold inside (Shapiro, et al. 2001:83). Typically, an object pixel is given a value of 1 while a background pixel is given a value of 0. Finally, a binary image is created by coloring each pixel white or black, depending on a pixel's label.
Thresholding is called adaptive thresholding when a different threshold is used for different regions in the image. This may also be known as local or dynamic thresholding. 
The key parameter in the thresholding process is the choice of the threshold value (or values, as mentioned earlier). Several different methods for choosing a threshold exist; users can manually choose a threshold value, or a thresholding algorithm can compute a value automatically, which is known as automatic thresholding. A simple method would be to choose the mean or median value, the rationale being that if the object pixels are brighter than the background, they should also be brighter than the average. In a noiseless image with uniform background and object values, the mean or median will work well as the threshold, however, this will generally not be the case. A more sophisticated approach might be to create a histogram of the image pixel intensities and use the valley point as the threshold. The histogram approach assumes that there is some average value for the background and object pixels, but that the actual pixel values have some variation around these average values. However, this may be computationally expensive, and image histograms may not have clearly defined valley points, often making the selection of an accurate threshold difficult. One method that is relatively simple, does not require much specific knowledge of the image, and is robust against image noise, is the following iterative method:2.Project Tesseract – Motivation and History
Tesseract is an open-source OCR engine that was developed at HP between 1984 and 1994. Like a supernova, it appeared from nowhere for the 1995 UNLV Annual Test of OCR Accuracy, shone brightly with its results, and then vanished back under the same cloak of secrecy under which it had been developed. Now for the first time, details of the architecture and algorithms can be revealed.
Tesseract began as a PhD research project in HP Labs, Bristol, and gained momentum as a possible software and/or hardware add-on for HP’s line of flatbed scanners. Motivation was provided by the fact that the commercial OCR engines of the day were in their infancy, and failed miserably on anything but the best quality print. After a joint project between HP Labs Bristol, and HP’s scanner division in Colorado, Tesseract had a significant lead in accuracy over the commercial engines, but did not become a product. The next stage of its development was back in HP Labs Bristol as an investigation of OCR for compression. Work concentrated more on improving rejection efficiency than on base-level accuracy. At the end of this project, at the end of 1994, development ceased entirely. The engine was sent to UNLV for the 1995 Annual Test of OCR Accuracy, where it proved its worth against the commercial engines of the time. In late 2005, HP released Tesseract for open source. It is now available at a.";

        public const double FontSize = 1;
        public const double Epsylon = .01;
        private const string MetadataFolder = @"";

        private static readonly string[] differentText = new string[] { 
                                                                        "abcdefghijklmnopqr", // English
                                                                        "дежзийклмнопр", // Bulgarian
                                                                        "المفاتيح", // Arabic
                                                                        "שדגכזסבה/'קר", // Hebrew
                                                                        "यह एक परीक्षण है", // Hindi
                                                                        "中國哲學書電子化計劃", // Chinese
                                                                        "あうぇふぁうぇふぁうぇふぁ", // Japanese
                                                                        "ㅁㄴㅇㄹㅋㅌㅊㅍㅂㅈㄷㄱ" // Korean
                                                                      };
        
        private static readonly FontWeight[] fontWeights = new FontWeight[] { FontWeights.Normal, FontWeights.Bold };
        private static readonly FontStyle[] fontStyles = new FontStyle[] { FontStyles.Normal, FontStyles.Italic };

        public static readonly RgbColor Black = new RgbColor(0, 0, 0);
        public static readonly RgbColor Blue = new RgbColor(0, 128, 255);
        public static readonly RgbColor Green = new RgbColor(0, 255, 0);
        public static readonly RgbColor Red = new RgbColor(255, 0, 0);
        public static readonly RgbColor Yellow = new RgbColor(255, 255, 0);
        public static readonly RgbColor Pink = new RgbColor(255, 102, 178);
        public static readonly RgbColor Turqouise = new RgbColor(0, 255, 255);

#if !NETSTANDARD
        public static TextBlock CreateTextBlock(FontProperties fontProperties)
        {
            TextBlock textBlock = new TextBlock();
            textBlock.FontFamily = fontProperties.FontFamily;
            textBlock.FontStyle = fontProperties.FontStyle;
            textBlock.FontWeight = fontProperties.FontWeight;

            return textBlock;
        }

        private static readonly FontsManager manager = new FontsManager();

        public static GlyphTypeface GetGlyphTypeface(FontProperties descr)
        {
            Typeface tf = new Typeface(descr.FontFamily, descr.FontStyle, descr.FontWeight, FontStretches.Normal);
            GlyphTypeface gtf;
            if(!tf.TryGetGlyphTypeface(out gtf))
            {
                throw new InvalidOperationException(string.Format("The font {0} is missing on the machine!", descr.FontFamily));
            }

            return gtf;
        }

        public static OpenTypeFontSourceBase GetFontSource(FontProperties descr)
        {
            return manager.GetOpenTypeFontSource(descr);
        }

        public static OpenTypeFontSource GetTrueTypeFontSource(FontProperties descr)
        {
            return (OpenTypeFontSource)TestHelper.GetFontSource(descr);
        }
#endif

        public static bool AreClose(double f1, double f2)
        {
            return Math.Abs(f1 - f2) < 0.5;
        }

        public static bool AreEqual(double a, double b, double epsilon = Extensions.Epsilon)
        {
            return a.IsEqualTo(b, epsilon);
        }

        public static bool AreEqual(Size a, Size b, double epsilon = Extensions.Epsilon)
        {
            return AreEqual(a.Width, b.Width, epsilon) && AreEqual(a.Height, b.Height, epsilon);
        }

        public static bool AreEqual(Rect a, Rect b, double epsilon = Extensions.Epsilon)
        {
            return AreEqual(a.X, b.X, epsilon) 
                && AreEqual(a.Y, b.Y, epsilon) 
                && AreEqual(a.Width, b.Width, epsilon) 
                && AreEqual(a.Height, b.Height, epsilon);
        }

        public static void ExecuteActionWithDifferentFontProperties(Action<FontProperties> action)
        {
            for (int i = 0; i < fontFamilies.Length; i++)
            {
                for (int j = 0; j < fontStyles.Length; j++)
                {
                    for (int k = 0; k < fontWeights.Length; k++)
                    {
                        action(new FontProperties(new FontFamily(fontFamilies[i]), fontStyles[j], fontWeights[k]));
                    }
                }
            }
        }

        public static void ExecuteActionWithDifferentTexts(Action<TextProperties> action)
        {
            for (int i = 0; i < differentText.Length; i++)
            {
                action(new TextProperties(differentText[i], 12));
            }
        }

        public static void ExecuteActionWithDifferentWords(Action<TextProperties> action)
        {
            for (int i = 0; i < sampleWords.Length; i++)
            {
                action(new TextProperties(sampleWords[i], 12));
            }
        }

        public static void ExecuteActionWithDifferentWordsDifferentLanguages(Action<TextProperties> action)
        {
            for (int i = 0; i < sampleWords.Length; i++)
            {
                action(new TextProperties(sampleWordsDifferentLanguages[i], 12));
            }
        }

        public static void ExecuteActionWithDifferentFontFamilies(Action<FontFamily> action)
        {
            for (int i = 0; i < fontFamilies.Length; i++)
            {
                action(new FontFamily(fontFamilies[i]));
            }
        }

        public static void ExecuteActionWithDifferentFontStyles(Action<FontProperties> action, string fontFamily)
        {
            for (int j = 0; j < fontStyles.Length; j++)
            {
                for (int k = 0; k < fontWeights.Length; k++)
                {
                    action(new FontProperties(new FontFamily(fontFamily), fontStyles[j], fontWeights[k]));
                }
            }
        }

        public static void AssertExceptionIsThrown<TException>(Action action)
           where TException : Exception
        {
            try
            {
                action();
                Assert.Fail(string.Format("Expected exception type:{0} was not thrown.",
                    typeof(TException).Name));
            }
            catch (TException)
            {
            }
            catch (Exception ex)
            {
                Assert.Fail(string.Format(
                    "Expected exception type: '{0}', Actual: '{1}'",
                    typeof(TException).Name,
                    ex.GetType().Name));
            }
        }

        public static void ResetCharacterProperties(Telerik.Windows.Documents.Fixed.Model.Editing.RadFixedDocumentEditor editor)
        {
            editor.CharacterProperties.BaselineAlignment = FixedDocumentDefaults.BaselineAlignment;
            editor.CharacterProperties.Font = FixedDocumentDefaults.Font;
            editor.CharacterProperties.FontSize = FixedDocumentDefaults.FontSize;
            editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;
            editor.CharacterProperties.HighlightColor = FixedDocumentDefaults.HighlightColor;
            editor.CharacterProperties.UnderlinePattern = FixedDocumentDefaults.UnderlinePattern;
        }
    }
}
