﻿using System;
using System.IO;
#if NETSTANDARD
using System.Linq;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.FormatProviders;
using Telerik.Windows.Documents.Fixed.FormatProviders.Old.Pdf.PdfReader;
using Telerik.Windows.Documents.Fixed.Model.Internal;
using Telerik.Windows.Documents.Fixed.Text.Internal;
#endif
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using System.Reflection;

namespace Telerik.Windows.Documents.Fixed.Tests.Utils
{
    internal class TestUtils
    {
        //private static RadFixedPage CreatePage(RadFixedDocument document,  Size pageSize, string text, double fontSize, double charSpacing, double wordSpacing, int pageIndex)
        //{
        //    RadFixedPage page = new RadFixedPage(document);
        //    page.Size = pageSize;
        //    string[] lines = text.Split(new string[] { "<nl>" }, StringSplitOptions.None);
        //    double offsetX = 50, offsetY = 50;
        //    foreach (string line in lines)
        //    {
        //        page.DrawString(line, Fonts.Helvetica, offsetX, offsetY, fontSize, charSpacing, wordSpacing);
        //        offsetY += fontSize + 5;
        //    }
        //    page.BoundingRect = Helper.GetBoundingRect(new System.Windows.Rect(0, 0, page.Size.Width, page.Size.Height), Matrix.Identity);
        //    page.PageNo = pageIndex + 1;
        //    page.Arrange();
        //    return page;
        //}

        public const double ColorEps = 1;
        public const double MatrixEps = 0.00001;
        public const double DefaultEps = 1;

#if !NETSTANDARD
        private static readonly PdfContentManager contentManager = new PdfContentManager();

        public static PdfContentManager ContentManager
        {
            get
            {
                return contentManager;
            }
        }
#endif

        public static void ExecuteMultipleTimes<T>(int times, Func<T> action)
        {
            for (int i = 0; i < times; i++)
            {
                action();
            }
        }

        public static void ExecuteMultipleTimes(int times, Action action)
        {
            for (int i = 0; i < times; i++)
            {
                action();
            }
        }

        public static TextDocument GetTextDocument(string documentName)
        {
            Stream stm = GetDocumentApplicationResourceStream(documentName);
            PdfFormatProvider formatProvider = new PdfFormatProvider();
            formatProvider.ImportSettings.ReadingMode = Fixed.FormatProviders.ReadingMode.AllAtOnce;
            formatProvider.ImportSettings.IsPdfViewer = true;
            RadFixedDocument document = formatProvider.Import(stm);

            return document.TextDocument;
        }

#if !NETSTANDARD
        public static TextDocument GetTextDocumentOld(string documentName)
        {
            Stream stm = GetDocumentApplicationResourceStream(documentName);

            PdfFormatProvider formatProvider = new PdfFormatProvider(stm, FormatProviderSettings.ReadOnDemand);
            RadFixedDocument document = formatProvider.Import();

            return document.TextDocument;
        }
#endif

        public static bool EqualColorComponents(int firstComponent, int secondComponent)
        {
            return Math.Abs(firstComponent - secondComponent) <= ColorEps;
        }

        private static bool EqualMatrixComponents(double firstComponent, double secondComponent)
        {
            return Math.Round(firstComponent, 1) == Math.Round(secondComponent, 1);
        }

        public static bool EqualValuesWithEps(double first, double second)
        {
            return EqualValues(first, second, DefaultEps);
        }

        public static bool EqualValues(double first, double second, double possibleDiff)
        {
            return Math.Abs(first - second) < possibleDiff;
        }

        public static bool EqualValues(double? first, double? second, double possibleDiff)
        {
            if (first.HasValue != second.HasValue)
            {
                return false;
            }

            if (first.HasValue)
            {
                return Math.Abs(first.Value - second.Value) < possibleDiff;
            }

            return true;
        }

        public static bool EqualColors(ColorBase firstColor, ColorBase secondColor)
        {
            if (firstColor.ColorSpace != secondColor.ColorSpace)
            {
                return false;
            }

            if (firstColor.ColorSpace.Name == ColorSpaceNames.DeviceRgb)
            {
                return EqualRgbColors((RgbColor)firstColor, (RgbColor)secondColor);
            }

            return false;
        }

        public static bool EqualMatrices(Matrix firstMatrix, Matrix secondMatrix)
        {
            return EqualMatrixComponents(firstMatrix.M11, secondMatrix.M11)
                && EqualMatrixComponents(firstMatrix.M12, secondMatrix.M12)
                && EqualMatrixComponents(firstMatrix.M21, secondMatrix.M21)
                && EqualMatrixComponents(firstMatrix.M22, secondMatrix.M22)
                && EqualMatrixComponents(firstMatrix.OffsetX, secondMatrix.OffsetX)
                && EqualMatrixComponents(firstMatrix.OffsetY, secondMatrix.OffsetY);
        }

        internal static TextPage CreateTextPage(TextDocument document, string text, int pageZeroBasedIndex)
        {
            RadFixedPage fixedPage = new RadFixedPage();

            return CreateTextPage(document, text, pageZeroBasedIndex, fixedPage, TestUtils.AddCharacter);
        }

#if !NETSTANDARD
        internal static TextPage CreateTextPageOld(TextDocument document, string text, int pageZeroBasedIndex)
        {
            RadFixedPage fixedPage = new RadFixedPage();
            fixedPage.InternalRadFixedPage = new RadFixedPageInternal(null, fixedPage);

            return CreateTextPage(document, text, pageZeroBasedIndex, fixedPage, TestUtils.AddCharacterOld);
        }
#endif

        internal static byte[] GetResourceBytes(string resource)
        {
            Stream stream = GetApplicationResourceStream(resource);

            byte[] result = new byte[stream.Length];

            stream.Read(result, 0, result.Length);

            return result;
        }

#if NETSTANDARD
        private static Assembly GetCurrentAssembly()
        {
            AssemblyName assemblyName = typeof(TestUtils).GetTypeInfo().Assembly.GetName();
            return Assembly.Load(new AssemblyName(assemblyName.Name));
        }

        private static Stream GetResourceStream(string name)
        {
            name = name.Replace('/', '.');
            name = name.Replace('\\', '.');
            Assembly assembly = GetCurrentAssembly();
            string[] resourceNames = assembly.GetManifestResourceNames();
            string resourceName = resourceNames.Where(p => p.Contains(name)).FirstOrDefault();

            if (string.IsNullOrEmpty(resourceName))
            {
                throw new ArgumentException("Missing resource with given name.", nameof(name));
            }

            return assembly.GetManifestResourceStream(resourceName);
        }

        internal static Stream GetApplicationResourceStream(string path)
        {
            Stream stm = GetResourceStream(path);

            return stm;
        }

        private static Stream GetDocumentApplicationResourceStream(string documentName)
        {
            string uri = string.Format(@"Documents/{0}.pdf", documentName);
            Stream stm = GetResourceStream(uri);
            return stm;
        }
#else
        internal static Stream GetApplicationResourceStream(string path)
        {
            AssemblyName assemblyName = new AssemblyName(typeof(TestUtils).Assembly.FullName);
            string uri = string.Format(@"/{0};component/{1}", assemblyName.Name, path);
            Stream stm = Application.GetResourceStream(new Uri(uri, UriKind.Relative)).Stream;
            return stm;
        }

        private static Stream GetDocumentApplicationResourceStream(string documentName)
        {
            string uri = string.Format(@"/{0};component/Documents/{1}.pdf", typeof(TestUtils).Assembly.FullName, documentName);
            Stream stm = Application.GetResourceStream(new Uri(uri, UriKind.Relative)).Stream;
            return stm;
        }
#endif

        internal static TextDocument CreateTextDocument(string text)
        {
            return CreateTextDocument(text, CreateTextPage);
        }

#if !NETSTANDARD
        internal static TextDocument CreateTextDocumentOld(string text)
        {
            return CreateTextDocument(text, CreateTextPageOld);
        }
#endif

        internal static RadFixedDocument CreateFixedDocument(string documentName, bool withPdfProcessing = false)
        {
            Stream stream = GetDocumentApplicationResourceStream(documentName);
            RadFixedDocument document = null;
            PdfFormatProvider formatProvider = null;
            if (withPdfProcessing)
            {
                formatProvider = new PdfFormatProvider();
                document = formatProvider.Import(stream);
                stream.Close();
            }
#if !NETSTANDARD
            else
            {
                // OLD API
                formatProvider = new PdfFormatProvider(stream, FormatProviderSettings.ReadOnDemand);
                document = formatProvider.Import();
            }
#endif

            return document;
        }

        //internal static RadFixedDocument CreateFixedDocument(string text, int fontSize, int charSpacing, int wordSpacing, Size pageSize, bool initializeDocument = true)
        //{
        //    string[] contents = text.Split(new string[] { "<np>" }, StringSplitOptions.None);
        //    RadFixedDocument doc = new RadFixedDocument(null);
        //    RadFixedPage[] pages = new RadFixedPage[contents.Length];
        //    for (int i = 0; i < pages.Length; i++)
        //    {
        //        pages[i] = CreatePage(doc, pageSize, contents[i], fontSize, charSpacing, wordSpacing, i);
        //    }
        //    doc.Pages = new Telerik.Windows.Documents.Fixed.Model.Collections.PagesCollection(pages);
        //    doc.Initialize();
        //    return doc;
        //}

        //internal static RadFixedDocument CreateFixedDocument(string text, int fontSize, int charSpacing, int wordSpacing, bool initializeDocument = true)
        //{
        //    return CreateFixedDocument(text, fontSize, charSpacing, wordSpacing, new Size(800, 1200), initializeDocument);
        //}

        private static bool EqualRgbColors(RgbColor firstColor, RgbColor secondColor)
        {
            return EqualColorComponents(firstColor.A, secondColor.A)
                && EqualColorComponents(firstColor.R, secondColor.R)
                && EqualColorComponents(firstColor.B, secondColor.B)
                && EqualColorComponents(firstColor.G, secondColor.G);
        }

        private static TextDocument CreateTextDocument(string text, Func<TextDocument, string, int, TextPage> createTextPageAction)
        {
            string[] pages = text.Split(new string[] { "<np>" }, StringSplitOptions.None);
            TextDocument doc = new TextDocument(pages.Length);

            for (int i = 0; i < pages.Length; i++)
            {
                TextPage page = createTextPageAction(doc, pages[i], i);
                doc.AddPage(i, page);
            }

            return doc;
        }

        private static TextPage CreateTextPage(TextDocument document, string text, int pageZeroBasedIndex, RadFixedPage fixedPage, Action<Word, char> addCharacterAction)
        {
            fixedPage.PageNumber = pageZeroBasedIndex + 1;

            TextPage page = new TextPage(document, fixedPage);
            int index = 0;
            string[] lines = text.Split(new string[] { "<nl>" }, StringSplitOptions.None);

            foreach (string line in lines)
            {
                if (string.IsNullOrEmpty(line))
                {
                    continue;
                }

                page.AddLine(CreateLine(line, addCharacterAction, ref index));
            }

            page.Finish();

            return page;
        }

        private static Line CreateLine(string line, Action<Word, char> addCharacterAction, ref int index)
        {
            string[] words = line.Split(' ');
            Line res = new Line();
            int last = words.Length - 1;

            for (int i = 0; i <= last; i++)
            {
                res.AddWord(CreateWord(words[i], ref index, i != last, addCharacterAction));
            }

            res.Finish();

            return res;
        }

        private static Word CreateWord(string str, ref int index, bool endsWithSpace, Action<Word, char> addCharacterAction)
        {
            Word word = new Word();

            foreach (char ch in str)
            {
                word.AddPosition(index++);
                addCharacterAction(word, ch);
            }

            word.AddPosition(index++);
            word.Finish(endsWithSpace);

            return word;
        }

        private static void AddCharacter(Word word, char ch)
        {
            word.AddCharacter(
                   new Character(
                   boundingRect: Rect.Empty,
                   toUnicode: ch.ToString(),
                   charSpacing: 0, 
                   textPosition: new Matrix(), 
                   fontSize: 14,
                   textTranslation: 0));
        }

#if !NETSTANDARD
        private static void AddCharacterOld(Word word, char ch)
        {
            GlyphOld glyph = new GlyphOld { ToUnicode = ch.ToString() };
            glyph.FreezeGlyphInfo();
            word.AddCharacter(new Character(glyph.BoundingRect, ch.ToString(), glyph.CharSpacing, new Matrix(), 14, 0));
        }
#endif
    }
}
