﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
#if !NETSTANDARD
using Telerik.Windows.Documents.Fixed.FormatProviders.Text;
#endif
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Model.Fonts;
using System.Windows.Media;
using System.Windows;

namespace Telerik.Windows.Documents.Fixed.Tests.VisualTests
{
    [TestClass]
    public class VisualTestsImportExport : DocumentFixedVisualBaseTest
    {
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        public void DocumentWithSingleAndDoubleQuotes()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("text showing.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 161907 tfs: 238681")]
        public void DocumentWithTextArray()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("show text array.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 161639 tfs: 238463")]
        public void DocumentWithStrokeLineThickness()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("TestFile.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:151286 tfs: 232225")]
        public void DocumentWithCrossReferenceTable()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Telerik Reporting Issue.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:154517 tfs: 234260")]
        public void DocumentWithCrossReferenceTable2()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("PDFfromScan.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:166759 tfs: 241612")]
        public void DocumentWithLineEndingWithSingleCarriageReturn()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("21130M500.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:161457 tfs: 238364")]
        public void ImportExportOfDocumentWithMultipleCrossReferenceTable()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("frontcover-image.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:147336 tfs: 229323")]
        public void ImportExportOfDocumentWithCrossReferenceStream()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Reference Stream.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:147336 tfs: 229323")]
        public void ImportExportOfDocumentWithCrossReferenceStream2()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("hck forms.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168024 tfs: 242148")]
        public void ImportExportOfDocumentContainingVBezierGeometryOperator()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("VOperator.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168024 tfs: 242148")]
        public void ImportExportOfDocumentContainingYBezierGeometryOperator()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("YOperator.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:160867 tfs: 237999")]
        public void ImportExportOfDocumentImageSourceEncodingImageData1()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("convert-jpg-to-pdf.net.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168033 tfs: 242158")]
        public void PathIsVisualisedWhenItIsInterrupedByInsertionOfRectangle()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("PathInterruptedByRect.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168033 tfs: 242158")]
        public void PathIsVisualisedWhenItIsInterrupedByInsertionOfRectangle_2()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("closePathChangesTest.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("Add option to invert colors when creating ImageSource using EncodedImageData class: id:160633 tfs: 237860")]
        public void ImportExportDocumentWithImageSourceUsingEncodedImageData()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("test.pdf");
            var img = document.Pages.First().Content.First() as Model.Objects.Image;
            img.ImageSource.DecodeArray = new double[] { 1, 0, 1, 0, 1, 0, 1, 0 };
            this.AssertAreVisualyEqual(document);
            img.ImageSource.DecodeArray = null;
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168160 tfs: 242226")]
        public void CloseAndStrokePathPathPaingingOpetratorImportExport()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("ICS 214a - Individual Log.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168160 tfs: 242226")]
        public void CloseAndStrokePathPathPaingingOpetratorImportExport_2()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("ICS 214a - Individual Log-modified.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168160 tfs: 242226")]
        public void CloseAndStrokePathPathPaingingOpetratorImportExport_3()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("closeAndStrokeOperator.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:169459 tfs: 169459")]
        public void ExceptionNotThrownWhenMediaBoxPropertyIsRequiredToProcessPageContent()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Centaur-mediabox inherited.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168227 tfs: 242242")]
        public void FontThatIsNotEmebeddedAndHasNoFontFamilyCouldBeImportedTest()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Cross reference stream.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:154511 tfs: 234259")]
        public void ImportExportPbs7()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("PBS7.pdf");

            this.AssertAreVisualyEqual(document, () => UIInteractionHelper.PageDown(2));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:164050 tfs: 239953")]
        public void ImportExportWillDistort()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Will Distort.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245")]
        public void ImportExporPDFfromWordNative()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("PDFfromWordNative - 913463.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245")]
        public void ImportExporSimpleCrossReferenceStream()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("Simple cross reference stream.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245")]
        public void ImportExporTestWordClientFile()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("test-word-client-file.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245 Text is not visible in the tables where the background is blue;")]
        public void ImportExporTestSig()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("TestSig - 958323.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(3));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245 Missing page 23 and maby more. On the first page the color of the title should be black.")]
        public void ImportExporTicket947262()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("40348 - ticket 947262.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(3));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245 The text in the footer has diff color and part of the text is not readable")]
        public void ImportExporBackcover()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("backcover.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:168243 tfs: 242245")]
        public void ImportExporFrontCoverFrontOfHouse()
        {
            RadFixedDocument document = this.LoadVisualTestDocument("FrontCover-FrontOfHouse.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id:174227 tfs: 244226")]
        public void ImportExpor21130M500()
        {
            RadFixedDocument document = this.LoadVisualTestDocument(@"21130M500.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        public void ImportExportDoctExportedImageMask()
        {
            RadFixedDocument document = this.LoadVisualTestDocument(@"test-exported-imagemask.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 176786 tfs: 244952")]
        public void ImportExportPdfStreamPointNotToEndStream()
        {
            RadFixedDocument document = this.LoadVisualTestDocument(@"test color key mask.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 141095 tfs: 225402")]
        public void ImportExportImagesWithSeparateAlphaChannel()
        {
            var document = this.CreateDocument();

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169912 tfs: 242804")]
        public void ImportExportImagesJPEGDecoder()
        {
            var document = this.CreateDocumentWithJPEG();

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 176013 tfs: 244747")]
        public void ImportExportTicketpro()
        {
            var document = this.LoadVisualTestDocument("Ticketpro.pdf");

            this.AssertAreVisualyEqual(document);
        }

        //[TestMethod]
        //[TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        //[Description("id: 169549 tfs: 242673")]
        //public void ImportExportDocWithLowAndMedumimageQaulity()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    document.Pages.AddPage();
        //    FixedContentEditor contentEditor = new FixedContentEditor(document.Pages.FirstOrDefault());
        //    contentEditor.ImageEditor.DrawImage(new MemoryStream(File.ReadAllBytes(FolderSettings.VisualTestsDocuments + "DemoDocumentFirstPage153Zoom.png")), 500, 500);

        //    PdfExportSettings settings = new PdfExportSettings();
        //    settings.ImageQuality = ImageQuality.Low;

        //    this.AssertAreVisualyEqual(document, settings);

        //    settings.ImageQuality = ImageQuality.Medium;
        //    this.AssertAreVisualyEqual(document, settings);
        //}

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 159686 tfs: 237250")]
        public void ImportExportPdfWithPng()
        {
            var document = this.LoadVisualTestDocument("PdfDocument.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 159686 tfs: 237250")]
        public void ImportExportV00()
        {
            var document = this.LoadVisualTestDocument("3630_9221_83DI12930_V00.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169703 tfs: 242723 Still not working")]
        public void ImportExportCalRgb()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\CalRgbTestDocument.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242726")]
        public void ImportExportCalLab1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ArabicCIDTrueType.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\cmykjpeg.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\issue215.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab4()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\issue1453.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab5()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\issue5549.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab6()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\issue5567.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 169707 tfs: 242723")]
        public void ImportExportCalLab7()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\LabTestDocument.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("")]
        public void ImportExportIccBased()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\IccBasedTestDocument.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("")]
        public void ImportExportDeviceCmyk()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\DeviceCmykTestDocument.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 177063 tfs: 244998")]
        public void ImportDocumentWithUnsupportedAnnotationsDoesNotThrowException()
        {
            var document = this.LoadVisualTestDocument(@"\UnsupportedAnnotationTypeDocument\1.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 177063 tfs: 244998")]
        public void ImportDocumentWithUnsupportedAnnotationsDoesNotThrowException2()
        {
            var document = this.LoadVisualTestDocument(@"\UnsupportedAnnotationTypeDocument\2.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 177063 tfs: 244998")]
        public void ImportDocumentWithUnsupportedAnnotationsDoesNotThrowException3()
        {
            var document = this.LoadVisualTestDocument(@"\UnsupportedAnnotationTypeDocument\Document1.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 177063 tfs: 244998")]
        public void ImportDocumentWithUnsupportedAnnotationsDoesNotThrowException4()
        {
            var document = this.LoadVisualTestDocument(@"\UnsupportedAnnotationTypeDocument\Export2016Q1Rtb.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 176540 tfs: 244892")]
        public void TextFragmentRespectsGenralGraphicContentStreamOperatorsTest()
        {
            var document = this.LoadVisualTestDocument(@"\TextFragment\FromDistiller10.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 176540 tfs: 244892")]
        public void TextFragmentRespectsGenralGraphicContentStreamOperatorsTest2()
        {
            var document = this.LoadVisualTestDocument(@"\TextFragment\Test Graphic Properties.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 176540 tfs: 244892")]
        public void TextFragmentRespectsGenralGraphicContentStreamOperatorsTest3()
        {
            var document = this.LoadVisualTestDocument(@"\TextFragment\Manual.pdf");

            this.AssertAreVisualyEqual(document);
        }

        //[TestMethod]
        //[TestCategory(PdfProcessingCategory.VisualTest)]
        //[Description("id: 191042 tfs: 250288")]
        //public void EmptyPdfNameObjectIsParsedAsEmptyName()
        //{
        //    var document = this.LoadVisualTestDocument("EmptyPfdName.pdf");

        //    this.AssertAreVisualyEqual(document);
        //}

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 185779 tfs: 247316")]
        public void ImageMaskPropertyIsExportedEvenWhenColorSpaceIsNotSpecified()
        {
            var document = this.LoadVisualTestDocument("List.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 185803 tfs: 247322")]
        public void CidFontsWidthsIsCorrectlyImportedExported()
        {
            var document = this.LoadVisualTestDocument("Electronic POS A.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 165443 tfs: 240806")]
        public void WhenImageWithoutColorspaveIsImportedNullReferenceExceptionIsNotThrown()
        {
            string[] docs = new string[] { "pdf import error.pdf", "ProblemDocument2.pdf" };
            foreach (var doc in docs)
            {
                var document = this.LoadVisualTestDocument(doc);
                this.AssertAreVisualyEqual(document);
            }
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 174640 tfs: 244397")]
        public void ContentStreamOperatorGrayscaleColorTest()
        {
            var document = this.LoadVisualTestDocument("Original.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 206386 tfs: 255471")]
        public void InvalidOperationExceptionEndStreamKeyworkTest()
        {
            var document = this.LoadVisualTestDocument("InvalidOperationDoc.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 206386 tfs: 255471")]
        public void InvalidOperationExceptionEndStreamKeyworkTest1()
        {
            var document = this.LoadVisualTestDocument("InvalidOperationSampleDoc.pdf");

            this.AssertAreVisualyEqual(document);
        }

        //[TestMethod]
        //[TestCategory(PdfProcessingCategory.VisualTest)]
        //[Description("id: 170812 tfs: 243007")]
        //public void ImportExportDocWithEmbeddedFont()
        //{
        //    RadFixedDocument document = this.LoadVisualTestDocument(@"with_embedded_font.pdf");

        //    this.AssertAreVisualyEqual(document);
        //}

        private RadFixedDocument CreateDocument()
        {
            RadFixedDocument document = new RadFixedDocument();
            document.DocumentInfo.Author = "Telerik";
            document.DocumentInfo.Title = "Cool PDF Processing";
            document.DocumentInfo.Description = "Alabala";

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
#if !NETSTANDARD
                editor.CharacterProperties.TrySetFont(new System.Windows.Media.FontFamily("Calibri"));
#else
                editor.CharacterProperties.TrySetFont(new Telerik.Documents.Core.Fonts.FontFamily("Calibri"));
#endif
                editor.InsertRun("Hello PDF/A");
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"\AlphaChannel\Image.png"));
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages[0]);
            fixedEditor.DrawImage(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"\AlphaChannel\Transparant.png"));

            return document;
        }

        private RadFixedDocument CreateDocumentWithJPEG()
        {
            RadFixedDocument document = new RadFixedDocument();
            document.DocumentInfo.Author = "Kamen Velikov";
            document.DocumentInfo.Title = "Cool PDF Processing";
            document.DocumentInfo.Description = "Alabala";

            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
            {
#if !NETSTANDARD
                editor.CharacterProperties.TrySetFont(new System.Windows.Media.FontFamily("Calibri"));
#else
                editor.CharacterProperties.TrySetFont(new Telerik.Documents.Core.Fonts.FontFamily("Calibri"));
#endif
                editor.InsertRun("Hello PDF/A");
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/Image.png"));
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/cmyk.jpg"));
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/GRAYSCALE.jp2"));
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/GRAYSCALE.jpg"));
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/rgb.jp2"));
                editor.InsertImageInline(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/rgb.jpg"));
            }

            FixedContentEditor fixedEditor = new FixedContentEditor(document.Pages[0]);
            fixedEditor.DrawImage(GetImageAsImageSource(FolderSettings.VisualTestsDocuments + @"/JPEG/Transparant.png"));

            return document;
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 180203 tfs: 246043 - Implement import of Form XObjects")]
        public void XFormTest()
        {
            var document = this.LoadVisualTestDocument("XForm.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 180203 tfs: 246043 - Implement import of Form XObjects")]
        public void XFormWithTextGeometriesAndTransformationsTest()
        {
            var document = this.LoadVisualTestDocument("XFormWithTextGeometriesandTransformations.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 180203 tfs: 246043 - Implement import of Form XObjects")]
        public void XFormSimpleTransformationsTest()
        {
            var document = this.LoadVisualTestDocument("XFormSimpleTransformations.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("id: 180203 tfs: 246043 - Implement import of Form XObjects")]
        public void DifferentRotationsXFormTest()
        {
            var document = this.LoadVisualTestDocument("DifferentRotationsXForm.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(5));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 285454 - PdfProcessing: Button field is imported as CheckBoxField instead of RadioButtonField")]
        public void RadioButtonFieldImportTest()
        {
            var document = this.LoadVisualTestDocument("LP1F make and register your LPA web 0715-checked Jointly and severally.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(5));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 254476 - PdfProcessing: InvalidCastException when importing inline images")]
        public void ImportInlineImagesTest1()
        {
            var document = this.LoadVisualTestDocument("InlineImage.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 254476 - PdfProcessing: InvalidCastException when importing inline images")]
        public void ImportInlineImagesTest2()
        {
            //InlineImages with indexed color space is not supported at this time
            // Test will fail when this is supported
            var document = this.LoadVisualTestDocument("TAMReview.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242519 - PdfProcessing: InvalidCastException is thrown when parsing dictionary end without having dictionary start")]
        public void ParseDictionaryEndWithoutHavingDictionaryStartTest()
        {
            var document = this.LoadVisualTestDocument("BrokenTestQuote.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 237603 - PdfProcessing: Importing document with GIF image and Indexed color space causes exception")]
        public void ImportDocumentWithGIFImageAndIndexedColorTest()
        {
            var document = this.LoadVisualTestDocument("6dhmm-ee7pc.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243749 - PdfProcessing: Indexed color space lookup stream is not imported and exported correctly")]
        public void IndexedColorSpaceLookUpStreamTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\10835585.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243749 - PdfProcessing: Indexed color space lookup stream is not imported and exported correctly")]
        public void IndexedColorSpaceLookUpStreamTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\LeagueTable 10-02-2012 15-04.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243749 - PdfProcessing: Indexed color space lookup stream is not imported and exported correctly")]
        public void IndexedColorSpaceLookUpStreamTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\IndexedColorSpace.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243749 - PdfProcessing: Indexed color space lookup stream is not imported and exported correctly")]
        public void IndexedColorSpaceLookUpStreamTest4()
        {
            // THERE ARE FOUR SQUARES AT THE LAST PAGE WITH MISSING COLORS
            // IF TEST STARTS TO FAIL CHECK WHETHER A BUG HAS BEEN FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\IndexedColorSpace2.pdf");
            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(2));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 283011 - PdfProcessing: Implement all Function types")]
        public void FunctionType0AndFunctionType3Test()
        {
            var document = this.LoadVisualTestDocument(@"Func0AndFunc3Doc.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 283011 - PdfProcessing: Implement all Function types")]
        public void FunctionType2AndFunctionType4Test()
        {
            // DUE TO MISSING COLOR SPACE FUNCTIONALITY THE EXPECTED RESULT DIFFERES FROM THE ONE IN ADOBE
            // TEST WILL START TO FAIL ONCE THIS IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"Func2AndFun4Doc.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 281818 - PdfProcessing: Implement import  for 'k' and 'K' color operators")]
        public void DocumentWith_k_And_K_ColorOperatorsTest1()
        {
            // THERE IS KNOWN GEOMETRIES CLIPPING ISSUE - TFS ID 286991
            // CHECK WHETHER IT IS FIXED IF THE TEST STARTS TO FAIL
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ABC Key Features and Benefits 10 - Cross Option 2 Solutions.pdf");
            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 281818 - PdfProcessing: Implement import  for 'k' and 'K' color operators")]
        public void DocumentWith_k_And_K_ColorOperatorsTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\fmbl-2009-01.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 281818 - PdfProcessing: Implement import  for 'k' and 'K' color operators")]
        public void DocumentWith_k_And_K_ColorOperatorsTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\21130M500.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(4), 2);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243767 - PdfProcessing: Implement content stream operators that allow setting color in CMYK colorspace")]
        public void SettingColorInCMYKColorSpaceTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\CW - LA Trust Software Notes 2015.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243767 - PdfProcessing: Implement content stream operators that allow setting color in CMYK colorspace")]
        public void SettingColorInCMYKColorSpaceTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\VS-KB-Brochure-CPP-A4-ProblemWhenUsingToUnicodeForType0Fonts.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243767 - PdfProcessing: Implement import and export of DeviceN color space")]
        public void ImportExportOfDeviceNColorSpaceTest1()
        {
            // THERE IS MISSING GRADIENT DUE TO USING 'SH' OPERATOR WHICH IS NOT YET SUPPORTED
            // TEST WILL START FAILING WHEN SUPPORT IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\cmp_device_n_gradient_base.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 243767 - PdfProcessing: Implement import and export of DeviceN color space")]
        public void ImportExportOfDeviceNColorSpaceTest2()
        {
            // THERE IS MISSING GRADIENT DUE TO USING 'SH' OPERATOR WHICH IS NOT YET SUPPORTED
            // TEST WILL START FAILING WHEN SUPPORT IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\cmp_device_n_gradient_CmykRedRgbBlue.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorStencilColorTest1()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\braun_feudel page 2.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorStencilColorTest2()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\braun_feudel.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(5));
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorStencilColorTest3()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\meister_dziourkevich.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(2));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorCropBoxTest1()
        {
            // SUPPORT FOR CROP BOX IS NOT YET IMPLEMENTED : 255511
            // TEST WILL START FAILING WHEN IT IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Funktionale_Varietaeten.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\pardes_10_a4.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\vjs_05_Art05.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest3()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\TestPDF2.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1), 2);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest4()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\allmbl-2009-01.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest5()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\PBS7.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest6()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\48-8905r4.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242727 - PdfProcessing: Implement import and export of Separation color space")]
        public void ImportExportSeparationColorTest7()
        {
            // THERE IS KNOWN STENCIL COLOR ISSUE : 286944
            // TEST WILL START FAILING WHEN THE ISSUE IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\warrantyAgreement.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1), 1);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Bad Pdf.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\new-osha300form1-1-04-FormsOnly.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Electronic POS A.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(6), 1);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest4()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\certificate_sb3.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest5()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\HSE_Guide_-_Employee_DSE_Workstation_Checklist_FORM.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(7));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest6()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\template1.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest7()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\List.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest8()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\adding_PDF_comments.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(4));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest9()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Template Highlevel Overview.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest10()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\FromDistiller10.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest11()
        {
            // SH SHADING OPERATOR IS NOT YET IMPLEMENTED : 283398
            // THERE IS A MISSING STAMP IN THE DOCUMENT SAYING 'SAMPLE'
            // TEST WILL START FAILING WHEN SH SHADING OPERATOR IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\fm082.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest12()
        {
            // SH SHADING OPERATOR IS NOT YET IMPLEMENTED : 283398
            // TEST WILL START FAILING WHEN SH SHADING OPERATOR IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Comparison S-F5X vs Global 5000 September 2014.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest13()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Bulk Emailer Manual 45.05.47.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242682 - PdfProcessing: Implement import of 'scn' content stream operator for all colorspaces")]
        public void ImportOfScnContentStreamOperatorTest14()
        {
            // W CLIPPING OPERATOR IS NOT YET IMPLEMENTED : 236733
            // TEST WILL START FAILING WHEN W CLIPPING OPERATOR IS IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Lines_wrong_colour.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 251327 - PdfProcessing: NotImplementedException occurs when importing pattern which uses unsupported colorspace")]
        public void ImportPatternWithUnsupportedColorSpaceTest()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Cst10Lay.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 251327 - PdfProcessing: NotImplementedException occurs when importing pattern which uses unsupported colorspace")]
        public void ImportPatternWithUnsupportedPatternPositioningTest()
        {
            // THERE IS KNOWN ISSUE WITH PATTERN POSITIONING : 220248
            // TEST WILL START FAILING WHEN IT IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Iatric Analytics on Demand Core Hardware Specifications 03-2016.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest1()
        {
            var document = this.LoadVisualTestDocument(@"\Sample (3).pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest2()
        {
            var document = this.LoadVisualTestDocument(@"\POS TEST MAGNUS SINGLE - Client Disclaimer.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest3()
        {
            var document = this.LoadVisualTestDocument(@"\Verkoopfactuur-VF-275977.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest4()
        {
            var document = this.LoadVisualTestDocument(@"\ltpdftest.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest5()
        {
            var document = this.LoadVisualTestDocument(@"\rx1.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288087 - PdfProcessing: Handle invalid PDF documents which content streams contain concatenated 'q' and 'Q' graphic state operators.")]
        public void ContentStreamWith_q_And_Q_GraphicStateOperatorsTest6()
        {
            var document = this.LoadVisualTestDocument(@"\TEST ADDRESS LPA - LPA Health  Welfare.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288049 - PdfProcessing: Corrupted document after export because of missing FunctionType PDF property")]
        public void DocumentWithFunctionTypeTest()
        {
            var document = this.LoadVisualTestDocument(@"\AB. CW - STEP SOT Sign Notes.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288232 - PdfProcessing: InvalidCastException when importing DeviceN color with integer components")]
        public void ImportDeviceNColorWithIntegerComponents()
        {
            var document = this.LoadVisualTestDocument(@"\cerfa_11389_05.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288458 - PdfProcessing: IndexOutOfRangeException is thrown when importing cross reference entries which are not on consequent rows")]
        public void ImportCrossReferenceEntriesOnInconsiquentRows()
        {
            var document = this.LoadVisualTestDocument(@"\ScannedPdfVer2.PDF");

            this.AssertAreVisualyEqual(document);
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247289 - PdfProcessing: Implement import  for 'sc' colorspace operator")]
        public void ImportDocumentWithScColorSpaceOperatorTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\WIDEROE_AntibacHånddes85våtserviett.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247289 - PdfProcessing: Implement import  for 'sc' colorspace operator")]
        public void ImportDocumentWithScColorSpaceOperatorTest2()
        {
            // GEOMETRY OPERATOR NOT YET IMPLEMENTED
            // TEST WILL START TO FAIL WHEN IMPLEMENTED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\neofoam_p.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247289 - PdfProcessing: Implement import  for 'sc' colorspace operator")]
        public void ImportDocumentWithScColorSpaceOperatorTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ADDAFLYTST¥P.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247289 - PdfProcessing: Implement import  for 'sc' colorspace operator")]
        public void ImportDocumentWithScColorSpaceOperatorTest4()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\in.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(4));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247289 - PdfProcessing: Implement import  for 'sc' colorspace operator")]
        public void ImportDocumentWithScColorSpaceOperatorTes5()
        {
            // THERE IS AN ISSUE WITH THIS DOCUMENT
            // CHECK BEHAVIOR IN ADOBE FOR PROPER VISUALIZATION AND UPDATE IMATES ONCE IT IS FIXED
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\OriginalFile.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(2));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 287650 - PdfProcessing: Transparency is lost on import when the graphics state does not specify the alpha source")]
        public void AlphaSourceNotSpecifiedByGraphicStateTest()
        {
            var document = this.LoadVisualTestDocument(@"\AlphaSourceNotSpecifiedByGraphicState.pdf");

            this.AssertAreVisualyEqual(document);
        }
#if !NETSTANDARD
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289347 - PdfProcessing: FileFormatException is thrown while exporting document with specific font ")]
        public void TestExportDocumentWithCustomFontDoesNotThrowException()
        {
            var testFontPath = FolderSettings.VisualTestsDocuments + @"\Resources\sisan03o.ttf";

            byte[] fontData = File.ReadAllBytes(testFontPath);
            FontsRepository.RegisterFont(new FontFamily("Siemens Sans OSF"), FontStyles.Normal, FontWeights.Normal, fontData);

            FontBase font;
            FontsRepository.TryCreateFont(new FontFamily("Siemens Sans OSF"), FontStyles.Normal, FontWeights.Normal, out font);

            //RadFixedDocument document = this.CreateSampleDocumentWithFont(font);

            RadFixedDocument document = new RadFixedDocument();
            RadFixedPage page = document.Pages.AddPage();
            FixedContentEditor editor = new FixedContentEditor(page);

            Block block = new Block();
            block.TextProperties.Font = font;
            block.InsertText("Sample text.");

            editor.DrawBlock(block, new Size(100, double.PositiveInfinity));

            this.AssertAreVisualyEqual(document);
        }
#endif
        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 322498 - PdfProcessing: InvalidCastException is thrown while importing font descriptor used by different types of fonts")]
        public void ImportFontDescriptorUsedByDifferentTypesOfFontsTest()
        {
            var document = this.LoadVisualTestDocument(@"\064078.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 236733 - PdfProcessing: Implement import and export for 'W * ' clipping operator")]
        public void ClippingWithWAndWStarOperatorsTest()
        {
            var document = this.LoadVisualTestDocument(@"\ClippingWithWAndWStarOperators.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 247288 - PdfProcessing: Implement import for 'b' geometry operator")]
        public void PathsWithBAndBStarOperatorsTest()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\PathsWithBAndBStarOperators.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 322734 - PdfProcessing: NullReferenceException is thrown when importing document with XFA fields")]
        public void ImportDocumentWithXFAFieldsTest1()
        {
            var document = this.LoadVisualTestDocument(@"\I-905 - (Instructions).pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 322734 - PdfProcessing: NullReferenceException is thrown when importing document with XFA fields")]
        public void ImportDocumentWithXFAFieldsTest2()
        {
            var document = this.LoadVisualTestDocument(@"\N-644 - (Instructions).pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 322734 - PdfProcessing: NullReferenceException is thrown when importing document with XFA fields")]
        public void ImportDocumentWithXFAFieldsTest3()
        {
            var document = this.LoadVisualTestDocument(@"\N-648 - (Instructions).pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest1()
        {
            var document = this.LoadVisualTestDocument(@"\Test_BW_Inverted.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(1));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest2()
        {
            var document = this.LoadVisualTestDocument(@"\test-compressed.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest3()
        {
            var document = this.LoadVisualTestDocument(@"\PDF not loading properly.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest4()
        {
            var document = this.LoadVisualTestDocument(@"\essai-CCITTFaxDecode-issue.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest5()
        {
            var document = this.LoadVisualTestDocument(@"\SkewedSig_4-TextPosition.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(4));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest6()
        {
            var document = this.LoadVisualTestDocument(@"\Capps Manufacturing Invoice 2012-05-25.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 289424 - PdfProcessing: Make CCITTFaxDecode to work for PdfProcessing")]
        public void CheckThatCCITTFaxDecodeWorksTest7()
        {
            var document = this.LoadVisualTestDocument(@"\40-83621-11.pdf");

            this.AssertAreVisualyEqual(document, UIInteractionHelper.PageDown(2));
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 322765 - PdfProcessing: NullReferenceException is thrown while importing tj operator without operands")]
        public void ImportTJOperatorWithoutOperandsTest()
        {
            var document = this.LoadVisualTestDocument(@"\I-129 - Petition for a Nonimmigrant Worker.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 286944 - PdfProcessing: Stencil color is not preserved when importing and exporting images")]
        public void StencilColorCheckWhenImportExportImagesTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ImagesWithStencilColor.pdf");

            this.AssertAreVisualyEqual(document);
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 286944 - PdfProcessing: Stencil color is not preserved when importing and exporting images")]
        public void StencilColorCheckWhenImportExportImagesTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ImageMask after Stencil color images.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 286944 - PdfProcessing: Stencil color is not preserved when importing and exporting images")]
        public void StencilColorCheckWhenImportExportImagesTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\ImageMask before Stencil color images.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242909 - PdfProcessing: Implement ImageSource.GetBitmapSource method to work with all color spaces and filters")]
        public void CheckThatGetBitmapSourceWorksWithAllColorSpacesTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\FrontFace for Public Displays Produkt-Präsentation.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242909 - PdfProcessing: Implement ImageSource.GetBitmapSource method to work with all color spaces and filters")]
        public void CheckThatGetBitmapSourceWorksWithAllColorSpacesTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\NSISDP_0008_20140729155355545_946_96fd645d.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242909 - PdfProcessing: Implement ImageSource.GetBitmapSource method to work with all color spaces and filters")]
        public void CheckThatGetBitmapSourceWorksWithAllColorSpacesTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Simplified_neofoam_p.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242909 - PdfProcessing: Implement ImageSource.GetBitmapSource method to work with all color spaces and filters")]
        public void CheckThatGetBitmapSourceWorksWithAllColorSpacesTest4()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\Simplified_tmp44E.tmp - modified row 2067 one page.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 242909 - PdfProcessing: Implement ImageSource.GetBitmapSource method to work with all color spaces and filters")]
        public void CheckThatGetBitmapSourceWorksWithAllColorSpacesTest5()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\TestNew.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupport2ndDocTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\2nd Doc.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportClippingInCropBoxBoundsTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\ClippingInCropBoxBounds.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportClippingInCropBoxBoundsRotate90Test()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\ClippingInCropBoxBounds-Rotate90.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportClippingInCropBoxBoundsRotate180Test()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\ClippingInCropBoxBounds-Rotate180.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportClippingInCropBoxBoundsRotate270Test()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\ClippingInCropBoxBounds-Rotate270.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportCropBoxAndMediaBoxIntersectionTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\CropBoxAndMediaBoxIntersection.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportCropBoxAndMediaBoxIntersectionBottomLeftTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\CropBoxAndMediaBoxIntersection-BottomLeft.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportCropBoxAndMediaBoxIntersectionTopRightTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\CropBoxAndMediaBoxIntersection-TopRight.pdf");

            this.AssertAreVisualyEqual(document);
        }

        //TODO: ***CHECK IF THIS TEST IS STABLE AFTER TEST MACHINES ARE VIRTUALIZED***
        //[TestMethod]
        //[TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        //[Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        //public void CropBoxSupportCropBoxAndMediaBoxNoIntersectionTest()
        //{
        //    var document = this.LoadVisualTestDocument(@"\CropBox\CropBoxAndMediaBoxNoIntersection.pdf");

        //    this.AssertAreVisualyEqual(document);
        //}

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportCropBoxAndMediaBoxWithNegativeOffsetsTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\CropBoxAndMediaBoxWithNegativeOffsets.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportFormXObjectBoundingBoxTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\FormXObjectBoundingBox.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportFunktionaleVarietaetenTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\Funktionale_Varietaeten.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupporTextContentWithoutPopupAnnotationTest()
        {
            // THERE IS ANNNOTATION THAT CURRENTLY IS NOT IMPORTED
            // NOTE WHETHER THERE IS RECENT ANNOTATION FIX IF THE TEST STARTS FAILING
            // IT MIGHT INDICATE THAT THE ISSUE WITH THE ANNOTATION MISSING IS FIXED
            var document = this.LoadVisualTestDocument(@"\CropBox\generated.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\input.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportTranslatedCropboxTopToBottomTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\TranslatedCropbox_TopToBottom.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 255511 - PdfProcessing: Add support for CropBox element in PDF page")]
        public void CropBoxSupportTranslatedCropboxLeftToRightTest()
        {
            var document = this.LoadVisualTestDocument(@"\CropBox\TranslatedCropboxLeftToRight.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 213359 - PdfProcessing: Support for document outline (bookmarks)")]
        public void SupportForBookmarksTest()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\Bookmarks.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 213359 - PdfProcessing: Support for document outline (bookmarks)")]
        public void SupportForBookmarksWithStylesTest()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\Bookmarks-with-styles.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 213359 - PdfProcessing: Support for document outline (bookmarks)")]
        public void SupportForBookmarksWithTextColorTest()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\Bookmarks-with-TextColor.pdf");

            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 288059 - PdfProcessing: NotImplementedException is thrown when importing color with Lab colorspace")]
        public void ImportColorWithLabColorSpaceTest()
        {
            var document = this.LoadVisualTestDocument(@"\ColorSpaces\50-144.pdf");

            this.AssertAreVisualyEqual(document);
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 258986 - PdfProcessing: Add support for Named Destinations")]
        public void NamedDestinationsSupportTest1()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\IV_UI_Coping.pdf");

            this.AssertAreVisualyEqual(document);
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 258986 - PdfProcessing: Add support for Named Destinations")]
        public void NamedDestinationsSupportTest2()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\Lorem ipsum - Wikipedia.pdf");

            this.AssertAreVisualyEqual(document);
        }


        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 258986 - PdfProcessing: Add support for Named Destinations")]
        public void NamedDestinationsSupportTest3()
        {
            var document = this.LoadVisualTestDocument(@"\Bookmarks\sample-link_1.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest1()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\12.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest2()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\Document.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest3()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\Report2.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest4()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\sample_pdf.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest5()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\TestPDF.pdf");
            this.AssertAreVisualyEqual(document);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTestImportExport)]
        [Description("TFS ID: 244367 - PdfProcessing: Import and export of ToUnicode CMap")]
        public void ImportExportToUnicodeCMapTest6()
        {
            var document = this.LoadVisualTestDocument(@"\ToUnicodeCMap\TestPDF2.pdf");
            this.AssertAreVisualyEqual(document);
        }
    }
}
