﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Fixed.Tests.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Fixed.Model.Resources;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;
#if !NETSTANDARD
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Streaming;
using Telerik.Windows.Documents.Fixed.FormatProviders.Text;
#endif
using System.Diagnostics;
using System.Collections.Generic;
using System.Windows;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Import;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;

namespace Telerik.Windows.Documents.Fixed.Tests.VisualTests
{
    [TestClass]
    public class VisualTestsPDFStreamWriter : DocumentFixedVisualBaseTest
    {
        public VisualTestsPDFStreamWriter()
        {
            CreateResultsFolder();
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void ReversePagesTest()
        {
            string initialDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string resultDocument = GetResultDocumentPath("ReversedPages.pdf");

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultDocument)))
            {
                using (PdfFileSource fileSource = new PdfFileSource(File.OpenRead(initialDocument)))
                {
                    for (int i = fileSource.Pages.Length - 1; i >= 0; i--)
                    {
                        PdfPageStreamWriter resultPage = null;
                        PdfPageSource sourcePage = fileSource.Pages[i];
                        resultPage = fileWriter.BeginPage(fileSource.Pages[i].Size, fileSource.Pages[i].Rotation);
                        resultPage.WriteContent(sourcePage);
                        resultPage.Dispose();
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultDocument, PDFFILEEXTENSION, ClickEnd, 1, 3000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void OddAndEvenPagesTest()
        {
            string initialDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string oddPagesDocument = GetResultDocumentPath("OddPages.pdf");
            string evenPagesDocument = GetResultDocumentPath("EvenPages.pdf");

            var streamWriters = new List<PdfStreamWriter>();
            streamWriters.Add(new PdfStreamWriter(File.OpenWrite(evenPagesDocument)));
            streamWriters.Add(new PdfStreamWriter(File.OpenWrite(oddPagesDocument)));

            using (PdfFileSource fileSource = new PdfFileSource(File.OpenRead(initialDocument)))
            {
                for (int i = 0; i < fileSource.Pages.Length; i++)
                {
                    PdfPageStreamWriter resultPage = null;
                    PdfPageSource sourcePage = fileSource.Pages[i];

                    if (i % 2 == 0)
                    {
                        resultPage = streamWriters[0].BeginPage(fileSource.Pages[i].Size, fileSource.Pages[i].Rotation);
                        resultPage.WriteContent(sourcePage);
                    }
                    else
                    {
                        resultPage = streamWriters[1].BeginPage(fileSource.Pages[i].Size, fileSource.Pages[i].Rotation);
                        resultPage.WriteContent(sourcePage);
                    }


                    resultPage.Dispose();
                }
            }

            streamWriters[0].Dispose();
            streamWriters[1].Dispose();

            this.VisualTestHelper.AssertFromProcess(oddPagesDocument, PDFFILEEXTENSION, ClickPageDown, 1);
            System.Threading.Thread.Sleep(350);
            this.VisualTestHelper.AssertFromProcess(evenPagesDocument, PDFFILEEXTENSION, ClickPageDown, 1);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void MakeTemplateDocumentUsingRadFixedDocumentTest()
        {
            string templateDocument = GetInitialDocumentPath("ProgressTemplate.pdf");
            string resultFile = GetResultDocumentPath("TemplateDocument.pdf");

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultFile)))
            {
                using (PdfFileSource fileSource = new PdfFileSource(File.OpenRead(templateDocument)))
                {
                    var size = fileSource.Pages[0].Size;
                    var rotation = fileSource.Pages[0].Rotation;
                    RadFixedDocument document = GenerateDocument(size, rotation);

                    foreach (var page in document.Pages)
                    {
                        PdfPageStreamWriter resultPage = null;
                        resultPage = fileWriter.BeginPage(fileSource.Pages[0].Size, fileSource.Pages[0].Rotation);


                        resultPage.WriteContent(fileSource.Pages[0]);

                        double xCenteringTranslation = (page.Size.Width / 3);
                        double yCenteringTranslation = (page.Size.Height / 3);
                        resultPage.ContentPosition.Translate(xCenteringTranslation, yCenteringTranslation);
                        resultPage.ContentPosition.Scale(0.6, 0.6);
                        resultPage.WriteContent(page);

                        resultPage.Dispose();
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultFile, PDFFILEEXTENSION, ClickPageDown, 1, 6000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void MakeTemplateDocumentTest()
        {
            string templateDocument = GetInitialDocumentPath("ProgressTemplate.pdf");
            string multiplePagesDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string resultFile = GetResultDocumentPath("TemplateDocument2.pdf");

            var fileSources = new List<PdfFileSource>();
            fileSources.Add(new PdfFileSource(File.OpenRead(templateDocument)));
            fileSources.Add(new PdfFileSource(File.OpenRead(multiplePagesDocument)));


            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultFile)))
            {
                var size = fileSources[0].Pages[0].Size;
                var rotation = fileSources[0].Pages[0].Rotation;

                foreach (var page in fileSources[1].Pages)
                {
                    PdfPageStreamWriter resultPage = null;
                    resultPage = fileWriter.BeginPage(size, rotation);


                    resultPage.WriteContent(fileSources[0].Pages[0]);

                    double xCenteringTranslation = (page.Size.Width / 1.5);
                    double yCenteringTranslation = (page.Size.Height / 12);
                    resultPage.ContentPosition.Translate(xCenteringTranslation, yCenteringTranslation);
                    resultPage.ContentPosition.Scale(0.6, 0.6);
                    resultPage.WriteContent(page);

                    resultPage.Dispose();
                }
            }

            fileSources[0].Dispose();
            fileSources[1].Dispose();

            this.VisualTestHelper.AssertFromProcess(resultFile, PDFFILEEXTENSION, ClickPageDown, 1, 3000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void MergeDocumentsTest()
        {
            string initialDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string resultDocument = GetResultDocumentPath("MergedDocuments.pdf");

            string[] documentsToMerge =
            {
                GetInitialDocumentPath("BarChart.pdf"),
                GetInitialDocumentPath("MultipageDocument1.pdf")
            };

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultDocument)))
            {
                foreach (string documentName in documentsToMerge)
                {
                    using (PdfFileSource fileToMerge = new PdfFileSource(File.OpenRead(documentName)))
                    {
                        foreach (PdfPageSource pageToMerge in fileToMerge.Pages)
                        {
                            fileWriter.WritePage(pageToMerge);
                        }
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultDocument, PDFFILEEXTENSION, ClickPageDown, 2, 3000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void SplitDocumentPagesTest()
        {
            string documentToSplit = GetInitialDocumentPath("MultipageDocument1.pdf");

            using (PdfFileSource fileToSplit = new PdfFileSource(File.OpenRead(documentToSplit)))
            {
                for (int i = 0; i < fileToSplit.Pages.Length; i++)
                {
                    PdfPageSource page = fileToSplit.Pages[i];
                    string splitDocumentName = GetResultDocumentPath(string.Format("SplitPage{0}.pdf", i + 1));

                    using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(splitDocumentName)))
                    {
                        fileWriter.WritePage(page);
                    }
                }
            }

            string resultDocument = GetResultDocumentPath("SplitPage1.pdf");
            string resultDocument2 = GetResultDocumentPath("SplitPage2.pdf");
            this.VisualTestHelper.AssertFromProcess(resultDocument, PDFFILEEXTENSION, null, 0, 1000);
            System.Threading.Thread.Sleep(350);
            this.VisualTestHelper.AssertFromProcess(resultDocument2, PDFFILEEXTENSION, null, 0, 1000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void FitAndPositionMultiplePagesOnSinglePageTest()
        {
            string initialDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string resultDocument = GetResultDocumentPath("FitAndPositionMultiplePagesOnSinglePage.pdf");

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultDocument)))
            {
                using (PdfFileSource fileSource = new PdfFileSource(File.OpenRead(initialDocument)))
                {
                    double halfWidth = fileSource.Pages[0].Size.Width / 2;
                    double halfHeight = fileSource.Pages[0].Size.Height / 2;
                    Vector[] translateDirections = new Vector[] { new Vector(0, 0), new Vector(1, 0), new Vector(0, 1), new Vector(1, 1) };
                    PdfPageStreamWriter resultPage = null;

                    for (int i = 0; i < fileSource.Pages.Length; i++)
                    {
                        if (i % 4 == 0)
                        {
                            if (i > 0)
                            {
                                resultPage.Dispose();
                            }

                            resultPage = fileWriter.BeginPage(fileSource.Pages[0].Size, fileSource.Pages[0].Rotation);
                            resultPage.ContentPosition.Scale(0.5, 0.5);
                        }

                        Vector direction = translateDirections[i % 4];
                        resultPage.ContentPosition.Translate(direction.X * halfWidth, direction.Y * halfHeight);
                        PdfPageSource sourcePage = fileSource.Pages[i];
                        resultPage.WriteContent(sourcePage);
                    }

                    resultPage.Dispose();
                }
            }
            this.VisualTestHelper.AssertFromProcess(resultDocument, PDFFILEEXTENSION, null, 0);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        public void PrependAndAppendPageContentTest()
        {
            string initialDocument = GetInitialDocumentPath("MultipageDocument1.pdf");
            string resultDocument = GetResultDocumentPath("PrependAndAppendPageContent.pdf");
            string folder = FolderSettings.VisualTestsDocuments + "/" + "JPEG";
            string backgroundImage = Path.Combine(folder, "GRAYSCALE.jpg");

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultDocument)))
            {
                using (PdfFileSource fileSource = new PdfFileSource(File.OpenRead(initialDocument)))
                {
                    RadFixedPage backgroundContentOwner = GenerateBackgroundImageContent(backgroundImage);
                    RadFixedPage foregroundContentOwner = GenerateForegroundTextContent("WATERMARK");

                    foreach (PdfPageSource pageSource in fileSource.Pages)
                    {
                        using (PdfPageStreamWriter pageWriter = fileWriter.BeginPage(pageSource.Size, pageSource.Rotation))
                        {
                            using (pageWriter.SaveContentPosition())
                            {
                                double xCenteringTranslation = (pageSource.Size.Width - backgroundContentOwner.Size.Width) / 2;
                                double yCenteringTranslation = (pageSource.Size.Height - backgroundContentOwner.Size.Height) / 2;
                                pageWriter.ContentPosition.Translate(xCenteringTranslation, yCenteringTranslation);
                                pageWriter.WriteContent(backgroundContentOwner);
                            }

                            pageWriter.WriteContent(pageSource);

                            using (pageWriter.SaveContentPosition())
                            {
                                double xCenteringTranslation = (pageSource.Size.Width - foregroundContentOwner.Size.Width) / 2;
                                double yCenteringTranslation = (pageSource.Size.Height - foregroundContentOwner.Size.Height) / 2;
                                pageWriter.ContentPosition.Translate(xCenteringTranslation, yCenteringTranslation);
                                pageWriter.WriteContent(foregroundContentOwner);
                            }
                        }
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultDocument, PDFFILEEXTENSION, null, 0, 3000);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("TFS ID: 285189 - PdfProcessing: Support for encrypted documents when using PdfFileSource class")]
        public void EncryptedDocumentWithPasswordTest()
        {
            var initialDocumentPath = GetInitialDocumentPath(@"\ColorSpaces\EncryptedWithPass.pdf");
            string resultFile = GetResultDocumentPath(TestContext.TestName + ".pdf");
            // Specify import settings in order to properly import encrypted document
            PdfImportSettings importSettings = new PdfImportSettings();
            importSettings.UserPasswordNeeded += (s, e) => { e.Password = "pass"; };

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultFile)))
            {
                using (PdfFileSource passwordEncryptedDocument =
                        new PdfFileSource(File.OpenRead(initialDocumentPath), importSettings, false))
                {
                    foreach (var page in passwordEncryptedDocument.Pages)
                    {
                        fileWriter.WritePage(page);
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultFile, PDFFILEEXTENSION, null, 0);
        }

        [TestMethod]
        [TestCategory(PdfProcessingCategory.VisualTest)]
        [Description("TFS ID: 285189 - PdfProcessing: Support for encrypted documents when using PdfFileSource class")]
        public void EncryptedDocumentWithoutPasswordTest()
        {
            var initialDocumentPath = GetInitialDocumentPath(@"\ColorSpaces\EncryptedWithoutPassword.pdf");
            string resultFile = GetResultDocumentPath(TestContext.TestName + ".pdf");
            // Specify import settings in order to properly import encrypted document
            PdfImportSettings importSettings = new PdfImportSettings();

            using (PdfStreamWriter fileWriter = new PdfStreamWriter(File.OpenWrite(resultFile)))
            {
                using (PdfFileSource passwordEncryptedDocument =
                        new PdfFileSource(File.OpenRead(initialDocumentPath), importSettings, false))
                {
                    foreach (var page in passwordEncryptedDocument.Pages)
                    {
                        fileWriter.WritePage(page);
                    }
                }
            }

            this.VisualTestHelper.AssertFromProcess(resultFile, PDFFILEEXTENSION, null, 0);
        }

#region VisualTestsPDFStreamWriter HelperMethods
        private void ClickPageDown()
        {
            this.Manager.Desktop.KeyBoard.KeyPress(System.Windows.Forms.Keys.PageDown);
        }

        private void ClickEnd()
        {
            this.Manager.Desktop.KeyBoard.KeyPress(System.Windows.Forms.Keys.End);
        }

        private void ClickHome()
        {
            this.Manager.Desktop.KeyBoard.KeyPress(System.Windows.Forms.Keys.Home);
        }

        private void CreateResultsFolder()
        {
            System.IO.Directory.CreateDirectory(FolderSettings.ResultDirName);
        }

#if !NETSTANDARD
        private static RadFixedDocument GenerateDocument(System.Windows.Size size, Telerik.Windows.Documents.Fixed.Model.Data.Rotation rotation)
#else
        private static RadFixedDocument GenerateDocument(Telerik.Documents.Primitives.Size size, Telerik.Windows.Documents.Fixed.Model.Data.Rotation rotation)
#endif
        {
            RadFixedDocument radFixedDocument = new RadFixedDocument();
            using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(radFixedDocument))
            {

                editor.SectionProperties.PageSize = size;
                editor.SectionProperties.PageRotation = rotation;

                for (int i = 0; i < 10; i++)
                {
                    editor.InsertRun("Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like Aldus PageMaker including versions of Lorem Ipsum.");
                    editor.InsertParagraph();
                    editor.InsertLine("Line text");

                    editor.InsertRun("Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like Aldus PageMaker including versions of Lorem Ipsum.");
                    editor.InsertLine("Line text");
                }
            }

            return radFixedDocument;
        }

        private static RadFixedPage GenerateForegroundTextContent(string text)
        {
            Block block = new Block();
            block.BackgroundColor = new RgbColor(50, 0, 0, 0);
            block.GraphicProperties.FillColor = new RgbColor(255, 0, 0);
            block.TextProperties.FontSize = 120;
            block.InsertText(text);
            Size horizontalTextSize = block.Measure();
            double rotatedTextSquareSize = (horizontalTextSize.Width + horizontalTextSize.Height) / Math.Sqrt(2);

            RadFixedPage foregroundContentOwner = new RadFixedPage();
            foregroundContentOwner.Size = new Size(rotatedTextSquareSize, rotatedTextSquareSize);
            FixedContentEditor foregroundEditor = new FixedContentEditor(foregroundContentOwner);
            foregroundEditor.Position.Translate(horizontalTextSize.Height / Math.Sqrt(2), 0);
            foregroundEditor.Position.Rotate(45);
            foregroundEditor.DrawBlock(block);

            return foregroundContentOwner;
        }

        private static RadFixedPage GenerateBackgroundImageContent(string watermarkImage)
        {
            using (Stream imageStream = File.OpenRead(watermarkImage))
            {
                ImageSource image = new ImageSource(imageStream);
                RadFixedPage backgroundContentOwner = new RadFixedPage();
                backgroundContentOwner.Size = new Size(image.Width, image.Height);
                FixedContentEditor imagePageEditor = new FixedContentEditor(backgroundContentOwner);
                imagePageEditor.DrawImage(image);

                return backgroundContentOwner;
            }
        }
#endregion
    }
}
