﻿using System;
using System.IO;
using System.Text;
using System.Xml;
using Microsoft.VisualStudio.DebuggerVisualizers;

namespace Telerik.Windows.Documents.Flow.DebuggerVisualizers
{
    public class DocumentElementDebuggerVisualizer : DialogDebuggerVisualizer
    {
        protected override void Show(IDialogVisualizerService windowService, IVisualizerObjectProvider objectProvider)
        {
            using (StreamReader reader = new StreamReader(objectProvider.GetData()))
            {
                string objectString = FormatXml(reader.ReadToEnd());

                TextBoxWindow textBoxWindow = new TextBoxWindow()
                {
                    TextContent = objectString
                };

                windowService.ShowDialog(textBoxWindow);
            }
        }

        private string FormatXml(string xml)
        {
            XmlDocument doc = new XmlDocument();

            try
            {
                doc.LoadXml(xml);
            }
            catch (XmlException)
            {
                return xml;
            }

            XmlWriterSettings settings = new XmlWriterSettings()
            {
                OmitXmlDeclaration = true, 
                Indent = true
            };

            StringBuilder stringBuilder = new StringBuilder();

            using (XmlWriter writer = XmlTextWriter.Create(stringBuilder, settings))
            {
                doc.Save(writer);

                return stringBuilder.ToString();
            }
        }
    }
}
