﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Licensing;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Common
{
    internal class RadFlowDocumentLicenseCheck : IDisposable
    {
        private readonly RadFlowDocument document;
        private readonly bool hasAddedSection;
        private readonly bool hasAddedParagraph;
        private bool isDisposed;

        public RadFlowDocumentLicenseCheck(RadFlowDocument document)
        {
            this.document = document;

            LicensingMode licensingMode = TelerikLicense.GetLicensingMode();
            string message;
            switch (licensingMode)
            {
                case LicensingMode.Trial:
                    message = TrialMessages.DocumentProcessingTrialMessage;
                    break;
                case LicensingMode.Unlicensed:
                    message = TrialMessages.DocumentProcessingUnlicencedMessage;
                    break;
                default:
                    message = null;
                    break;
            }

            if (message != null)
            {
                if (!this.document.Sections.Any())
                {
                    this.document.Sections.AddSection();
                    this.hasAddedSection = true;
                }

                Paragraph paragraph = new Paragraph(this.document);
                paragraph.Inlines.AddRun(message).ForegroundColor = new ThemableColor(Colors.Red);
                this.document.Sections[0].Blocks.Insert(0, paragraph);
                this.hasAddedParagraph = true;
            }
        }

        public void Dispose()
        {
            if (!this.isDisposed)
            {
                if (this.hasAddedSection)
                {
                    this.document.Sections.RemoveAt(0);
                }
                else if (this.hasAddedParagraph)
                {
                    this.document.Sections[0].Blocks.RemoveAt(0);
                }
                this.isDisposed = true;
            }
        }
    }
}