﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Contexts
{
    internal abstract class AnnotationContextBase<T>
    {
        private readonly RadFlowDocument document;
        private readonly Dictionary<int, T> idToAnnotationImportDictionary;
        private readonly Dictionary<T, int> annotationToIdExportDictionary;

        public AnnotationContextBase(RadFlowDocument document)
        {
            this.document = document;
            this.idToAnnotationImportDictionary = new Dictionary<int, T>();
            this.annotationToIdExportDictionary = new Dictionary<T, int>();
        }

        public RadFlowDocument Document
        {
            get
            {
                return this.document;
            }
        }

        public void RegisterAnnotationToImport(int id, T annotation)
        {
            this.idToAnnotationImportDictionary.Add(id, annotation);
        }

        public T PopRegisteredAnnotationById(int id)
        {
            T annotation = this.idToAnnotationImportDictionary[id];
            this.idToAnnotationImportDictionary.Remove(id);

            return annotation;
        }

        public void RegisterAnnotationToExport(T annotation, int id)
        {
            this.annotationToIdExportDictionary.Add(annotation, id);
        }

        public int PopIdByRegisteredAnnotation(T annotation)
        {
            int id;
            if (this.annotationToIdExportDictionary.TryGetValue(annotation, out id))
            {
                this.annotationToIdExportDictionary.Remove(annotation);
            }
            else
            {
                id = AnnotationIdGenerator.GetNext();
            }

            return id;
        }

        public int GetIdByRegisteredAnnotation(T annotation)
        {
            return this.annotationToIdExportDictionary[annotation];
        }
    }
}