﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Contexts
{
    internal class BookmarkContext : AnnotationContextBase<Bookmark>
    {
        private readonly List<BookmarkRangeStart> hangingBookmarkStarts;
        private readonly List<int> hangingBookmarkEndIds;

        public BookmarkContext(RadFlowDocument document)
            : base(document)
        {
            this.hangingBookmarkStarts = new List<BookmarkRangeStart>();
            this.hangingBookmarkEndIds = new List<int>();
        }

        public static Bookmark CreateBookmark(RadFlowDocument document, string name, int colFirst, int colLast)
        {
            Bookmark bookmark;
            if (colFirst != colLast)
            {
                bookmark = new Bookmark(document, name, colFirst, colLast);
            }
            else
            {
                bookmark = new Bookmark(document, name);
            }

            return bookmark;
        }

        public void AddHangingBookmarkStart(BookmarkRangeStart annotationStart)
        {
            this.hangingBookmarkStarts.Add(annotationStart);
        }

        public void AddHangingBookmarkEndId(int id)
        {
            this.hangingBookmarkEndIds.Add(id);
        }

        public List<BookmarkRangeStart> GetHangingBookmarkStarts()
        {
            return this.hangingBookmarkStarts;
        }

        public List<int> GetHangingBookmarkEndIds()
        {
            return this.hangingBookmarkEndIds;
        }

        public void ClearHangingBookmarkStarts()
        {
            this.hangingBookmarkStarts.Clear();
        }

        public void ClearHangingBookmarkEnds()
        {
            this.hangingBookmarkEndIds.Clear();
        }
    }
}