﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Contexts
{
    internal class CommentContext : AnnotationContextBase<Comment>
    {
        private readonly Queue<int> importedCommentIds;
        private readonly Dictionary<int, Comment> importedIdToCommentPairs;

        public CommentContext(RadFlowDocument document)
            : base(document)
        {
            this.importedCommentIds = new Queue<int>();
            this.importedIdToCommentPairs = new Dictionary<int, Comment>();
        }

        public bool HasCommentsToExport
        {
            get
            {
                return this.Document.Comments
                    .Where(c => c.CommentRangeStart.Paragraph != null || c.CommentRangeEnd.Paragraph != null).Any();
            }
        }

        public static Comment CreateComment(RadFlowDocument document)
        {
            return document.Comments.AddComment();
        }

        public void AddImportedComment(int id, Comment comment)
        {
            this.importedIdToCommentPairs.Add(id, comment);
        }

        public Comment GetImportedCommentById(int id)
        {
            Comment comment;
            this.importedIdToCommentPairs.TryGetValue(id, out comment);

            return comment;
        }

        public Comment PopImportedComment()
        {
            int commentId = this.importedCommentIds.Dequeue();
            Comment comment = this.importedIdToCommentPairs[commentId];
            this.importedIdToCommentPairs.Remove(commentId);

            return comment;
        }

        public void AddIdToCommentsPartImport(int id)
        {
            this.importedCommentIds.Enqueue(id);
        }
    }
}