﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Protection;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Contexts
{
    internal class PermissionContext : AnnotationContextBase<PermissionRange>
    {
        private readonly List<PermissionRangeStart> hangingPermissionStarts;
        private readonly List<int> hangingPermissionEndIds;

        internal PermissionContext(RadFlowDocument document)
            : base(document)
        {
            this.hangingPermissionStarts = new List<PermissionRangeStart>();
            this.hangingPermissionEndIds = new List<int>();
        }

        internal static PermissionRange CreatePermission(RadFlowDocument document, PermissionStartElement permissionStartElement)
        {
            PermissionRange permission;
            PermissionRangeCredentials permissionCredentials;

            if (!string.IsNullOrEmpty(permissionStartElement.Editor))
            {
                permissionCredentials = new PermissionRangeCredentials(permissionStartElement.Editor);
            }
            else if (permissionStartElement.EditingGroup.HasValue)
            {
                permissionCredentials = new PermissionRangeCredentials(permissionStartElement.EditingGroup.Value);
            }
            else
            {
                // individual and group name are optional
                return null;
            }

            if (permissionStartElement.ColFirst != permissionStartElement.ColLast)
            {
                permission = new PermissionRange(document, permissionCredentials, permissionStartElement.ColFirst, permissionStartElement.ColLast);
            }
            else
            {
                permission = new PermissionRange(document, permissionCredentials);
            }

            return permission;
        }

        internal void AddHangingPermissionStart(PermissionRangeStart annotationStart)
        {
            this.hangingPermissionStarts.Add(annotationStart);
        }

        internal void AddHangingPermissionEndId(int id)
        {
            this.hangingPermissionEndIds.Add(id);
        }

        internal List<PermissionRangeStart> GetHangingPermissionStarts()
        {
            return this.hangingPermissionStarts;
        }

        internal List<int> GetHangingPermissionEndIds()
        {
            return this.hangingPermissionEndIds;
        }

        internal void ClearHangingPermissionStarts()
        {
            this.hangingPermissionStarts.Clear();
        }

        internal void ClearHangingPermissionEnds()
        {
            this.hangingPermissionEndIds.Clear();
        }
    }
}
