﻿using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Parts;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export
{
    internal class DocxExporter : OpenXmlExporter<DocxPartsManager, IDocxExportContext>
    {
        protected override DocxPartsManager CreatePartsManager()
        {
            return new DocxPartsManager();
        }

        protected override void InitializeParts(DocxPartsManager partsManager, IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNull(context, "context");

            base.InitializeParts(partsManager, context);

            ThemePart themePart = new ThemePart(partsManager, DocxPartNames.ThemePartName);
            partsManager.RegisterPart(themePart);
            partsManager.CreateDocumentRelationship(themePart.Name, OpenXmlRelationshipTypes.ThemeRelationshipType);

            DocumentPart documentPart = new DocumentPart(partsManager);

            RegisterHeaderParts(partsManager, context);
            RegisterFooterParts(partsManager, context);

            partsManager.RegisterPart(new DocumentSettingsPart(partsManager));
            partsManager.RegisterPart(new StylesPart(partsManager));
            if (context.CommentContext.HasCommentsToExport)
            {
                partsManager.RegisterPart(new CommentsPart(partsManager));
            }

            RegisterListsPart(partsManager, context);

            partsManager.RegisterPart(documentPart);
        }

        protected override string CreateResourceName(IResource resource)
        {
            return DocxHelper.CreateResourceName(resource);
        }

        private static void RegisterHeaderParts(DocxPartsManager partsManager, IDocxExportContext context)
        {
            foreach (Header header in context.GetHeaders())
            {
                RegisterHeaderPart(partsManager, context, header);
            }
        }

        private static void RegisterFooterParts(DocxPartsManager partsManager, IDocxExportContext context)
        {
            foreach (Footer footer in context.GetFooters())
            {
                RegisterFooterPart(partsManager, context, footer);
            }
        }

        private static void RegisterHeaderPart(DocxPartsManager partsManager, IDocxExportContext context, Header header)
        {
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(header, "header");

            HeaderPart headerPart = new HeaderPart(partsManager, header, context.GetHeaderFooterPartNumberByHeaderFooter(header));
            context.RegisterHeaderFooter(headerPart.RelationshipId, header);
            partsManager.RegisterPart(headerPart);
        }

        private static void RegisterFooterPart(DocxPartsManager partsManager, IDocxExportContext context, Footer footer)
        {
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(footer, "footer");

            FooterPart footerPart = new FooterPart(partsManager, footer, context.GetHeaderFooterPartNumberByHeaderFooter(footer));
            context.RegisterHeaderFooter(footerPart.RelationshipId, footer);
            partsManager.RegisterPart(footerPart);
        }

        private static void RegisterListsPart(DocxPartsManager partsManager, IDocxExportContext context)
        {
            if (context.Document.Lists.Count > 0)
            {
                partsManager.RegisterPart(new ListsPart(partsManager));
            }
        }
    }
}
