﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Comments;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Settings;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.DefaultStyles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.ParagraphPropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.RunPropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.StylePropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TableCellPropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TablePropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TableRowPropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Vector;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Utilities;
using F = Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model
{
    internal static class DocxElementsFactory
    {
        private static readonly Dictionary<string, Func<DocxPartsManager, OpenXmlElementBase>> elementNameToFactoryMethod;

        static DocxElementsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<DocxPartsManager, OpenXmlElementBase>>();

            RegisterFactoryMethod(DocxElementNames.Body, (pm) => new BodyElement(pm));
            RegisterFactoryMethod(DocxElementNames.Paragraph, (pm) => new F.ParagraphElement(pm));
            RegisterFactoryMethod(DocxElementNames.Run, (pm) => new RunElement(pm));
            RegisterFactoryMethod(DocxElementNames.Text, (pm) => new F.TextElement(pm));
            RegisterFactoryMethod(DocxElementNames.InstructionText, (pm) => new InstructionTextElement(pm));
            RegisterFactoryMethod(DocxElementNames.Tab, (pm) => new TabElement(pm));
            RegisterFactoryMethod(DocxElementNames.Table, (pm) => new TableElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableRow, (pm) => new TableRowElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableCell, (pm) => new TableCellElement(pm));
            RegisterFactoryMethod(DocxElementNames.FieldCharacter, (pm) => new FieldCharacterElement(pm));
            RegisterFactoryMethod(DocxElementNames.Hyperlink, (pm) => new HyperlinkElement(pm));
            RegisterFactoryMethod(DocxElementNames.SimpleField, (pm) => new SimpleFieldElement(pm));
            RegisterFactoryMethod(DocxElementNames.BookmarkStart, (pm) => new BookmarkStartElement(pm));
            RegisterFactoryMethod(DocxElementNames.BookmarkEnd, (pm) => new BookmarkEndElement(pm));
            RegisterFactoryMethod(DocxElementNames.Break, (pm) => new BreakElement(pm));
            RegisterFactoryMethod(DocxElementNames.CommentRangeStart, (pm) => new CommentRangeStartElement(pm));
            RegisterFactoryMethod(DocxElementNames.CommentRangeEnd, (pm) => new CommentRangeEndElement(pm));
            RegisterFactoryMethod(DocxElementNames.CommentReference, (pm) => new CommentReferenceElement(pm));
            RegisterFactoryMethod(DocxElementNames.PermissionStart, (pm) => new PermissionStartElement(pm));
            RegisterFactoryMethod(DocxElementNames.PermissionEnd, (pm) => new PermissionEndElement(pm));

            // comments
            RegisterFactoryMethod(DocxElementNames.Comment, (pm) => new CommentElement(pm));

            // lists
            RegisterFactoryMethod(DocxElementNames.AbstractList, (pm) => new AbstractListElement(pm));
            RegisterFactoryMethod(DocxElementNames.AbstractListId, (pm) => new AbstractListIdElement(pm));
            RegisterFactoryMethod(DocxElementNames.NumberedList, (pm) => new NumberedListElement(pm));
            RegisterFactoryMethod(DocxElementNames.ListLevel, (pm) => new ListLevelElement(pm));
            RegisterFactoryMethod(DocxElementNames.NumberTextFormat, (pm) => new NumberTextFormatElement(pm));
            RegisterFactoryMethod(DocxElementNames.MultiLevelType, (pm) => new MultiLevelTypeElement(pm));
            RegisterFactoryMethod(DocxElementNames.NumberingStyle, (pm) => new NumberingStyleElement(pm));
            RegisterFactoryMethod(DocxElementNames.NumStyleLink, (pm) => new NumStyleLinkElement(pm));
            RegisterFactoryMethod(DocxElementNames.StyleLink, (pm) => new StyleLinkElement(pm));
            RegisterFactoryMethod(DocxElementNames.StartIndex, (pm) => new StartIndexElement(pm));
            RegisterFactoryMethod(DocxElementNames.RestartAfterLevel, (pm) => new RestartAfterLevelElement(pm));
            RegisterFactoryMethod(DocxElementNames.IsLegal, (pm) => new IsLegalElement(pm));
            RegisterFactoryMethod(DocxElementNames.ListLevelAlignment, (pm) => new AlignmentElement(pm, DocxElementNames.ListLevelAlignment));
            RegisterFactoryMethod(DocxElementNames.StartIndexOverride, (pm) => new StartIndexOverrideElement(pm));
            RegisterFactoryMethod(DocxElementNames.ListLevelOverride, (pm) => new ListLevelOverrideElement(pm));

            // styles
            RegisterFactoryMethod(DocxElementNames.DocumentDefaultStyles, (pm) => new DocumentDefaultStylesElement(pm));
            RegisterFactoryMethod(DocxElementNames.ParagraphDefaultProperties, (pm) => new ParagraphDefaultPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.RunDefaultProperties, (pm) => new RunDefaultPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.Style, (pm) => new StyleElement(pm));
            RegisterFactoryMethod(DocxElementNames.StyleName, (pm) => new StyleNameElement(pm));
            RegisterFactoryMethod(DocxElementNames.IsPrimary, (pm) => new IsPrimaryElement(pm));
            RegisterFactoryMethod(DocxElementNames.NextStyleId, (pm) => new NextStyleIdElement(pm));
            RegisterFactoryMethod(DocxElementNames.LinkedStyleId, (pm) => new LinkedStyleIdElement(pm));
            RegisterFactoryMethod(DocxElementNames.UIPriority, (pm) => new UIPriorityElement(pm));
            RegisterFactoryMethod(DocxElementNames.BasedOnStyleId, (pm) => new BasedOnStyleIdElement(pm));

            // property containers
            RegisterFactoryMethod(DocxElementNames.RunProperties, (pm) => new RunPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.ParagraphProperties, (pm) => new ParagraphPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableRowProperties, (pm) => new TableRowPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableProperties, (pm) => new TablePropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableCellProperties, (pm) => new TableCellPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.SectionProperties, (pm) => new SectionPropertiesElement(pm));

            // common properties
            RegisterFactoryMethod(DocxElementNames.Shading, (pm) => new ShadingElement(pm));
            RegisterFactoryMethod(DocxElementNames.Alignment, (pm) => new AlignmentElement(pm, DocxElementNames.Alignment));
            RegisterFactoryMethod(DocxElementNames.Left, (pm) => new BorderElement(pm, DocxElementNames.Left));
            RegisterFactoryMethod(DocxElementNames.Top, (pm) => new BorderElement(pm, DocxElementNames.Top));
            RegisterFactoryMethod(DocxElementNames.Right, (pm) => new BorderElement(pm, DocxElementNames.Right));
            RegisterFactoryMethod(DocxElementNames.Bottom, (pm) => new BorderElement(pm, DocxElementNames.Bottom));
            RegisterFactoryMethod(DocxElementNames.BorderBetween, (pm) => new BorderElement(pm, DocxElementNames.BorderBetween));
            RegisterFactoryMethod(DocxElementNames.BorderInsideHorizontal, (pm) => new BorderElement(pm, DocxElementNames.BorderInsideHorizontal));
            RegisterFactoryMethod(DocxElementNames.BorderInsideVertical, (pm) => new BorderElement(pm, DocxElementNames.BorderInsideVertical));
            RegisterFactoryMethod(DocxElementNames.BorderDiagonalDown, (pm) => new BorderElement(pm, DocxElementNames.BorderDiagonalDown));
            RegisterFactoryMethod(DocxElementNames.BorderDiagonalUp, (pm) => new BorderElement(pm, DocxElementNames.BorderDiagonalUp));
            RegisterFactoryMethod(DocxElementNames.VerticalAlignment, (pm) => new Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document.VerticalAlignmentElement(pm));

            // run properties
            RegisterFactoryMethod(DocxElementNames.Bold, (pm) => new BoldElement(pm));
            RegisterFactoryMethod(DocxElementNames.BaselineAlignment, (pm) => new BaselineAlignmentElement(pm));
            RegisterFactoryMethod(DocxElementNames.FontSize, (pm) => new FontSizeElement(pm));
            RegisterFactoryMethod(DocxElementNames.Italic, (pm) => new ItalicElement(pm));
            RegisterFactoryMethod(DocxElementNames.Font, (pm) => new FontElement(pm));
            RegisterFactoryMethod(DocxElementNames.ForegroundColor, (pm) => new ForegroundColorElement(pm));
            RegisterFactoryMethod(DocxElementNames.Highlight, (pm) => new HighlightElement(pm));
            RegisterFactoryMethod(DocxElementNames.Strikethrough, (pm) => new StrikethroughElement(pm));
            RegisterFactoryMethod(DocxElementNames.Underline, (pm) => new UnderlineElement(pm));
            RegisterFactoryMethod(DocxElementNames.RunFlowDirection, (pm) => new FlowDirectionElement(pm, DocxElementNames.RunFlowDirection));
            RegisterFactoryMethod(DocxElementNames.RunStyleId, (pm) => new StyleIdElement(pm, DocxElementNames.RunStyleId));

            // paragraph properties
            RegisterFactoryMethod(DocxElementNames.FlowDirection, (pm) => new FlowDirectionElement(pm, DocxElementNames.FlowDirection));
            RegisterFactoryMethod(DocxElementNames.KeepOnOnePage, (pm) => new KeepOnOnePageElement(pm));
            RegisterFactoryMethod(DocxElementNames.KeepWithNextParagraph, (pm) => new KeepWithNextParagraphElement(pm));
            RegisterFactoryMethod(DocxElementNames.ApplyEastAsianLineBreakingRules, (pm) => new ApplyEastAsianLineBreakingRulesElement(pm));
            RegisterFactoryMethod(DocxElementNames.PageBreakBefore, (pm) => new PageBreakBeforeElement(pm));
            RegisterFactoryMethod(DocxElementNames.ContextualSpacing, (pm) => new ContextualSpacingElement(pm));
            RegisterFactoryMethod(DocxElementNames.MirrorIndents, (pm) => new MirrorIndentsElement(pm));
            RegisterFactoryMethod(DocxElementNames.Indentation, (pm) => new IndentationElement(pm));
            RegisterFactoryMethod(DocxElementNames.AllowOverflowPunctuation, (pm) => new AllowOverflowPunctuationElement(pm));
            RegisterFactoryMethod(DocxElementNames.NumberingProperties, (pm) => new NumberingPropertiesElement(pm));
            RegisterFactoryMethod(DocxElementNames.ListId, (pm) => new ListIdElement(pm));
            RegisterFactoryMethod(DocxElementNames.ParagraphListLevel, (pm) => new ParagraphListLevel(pm));
            RegisterFactoryMethod(DocxElementNames.Spacing, (pm) => new SpacingElement(pm));
            RegisterFactoryMethod(DocxElementNames.OutlineLevel, (pm) => new OutlineLevelElement(pm));
            RegisterFactoryMethod(DocxElementNames.ParagraphBorders, (pm) => new ParagraphBordersElement(pm));
            RegisterFactoryMethod(DocxElementNames.ParagraphStyleId, (pm) => new StyleIdElement(pm, DocxElementNames.ParagraphStyleId));
            RegisterFactoryMethod(DocxElementNames.TabStops, (pm) => new TabStopsElement(pm));

            // table properties
            RegisterFactoryMethod(DocxElementNames.TableBorders, (pm) => new TableBordersElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableTableCellSpacing, (pm) => new TableWidthElement(pm, DocxElementNames.TableTableCellSpacing));
            RegisterFactoryMethod(DocxElementNames.TableIndent, (pm) => new TableWidthElement(pm, DocxElementNames.TableIndent));
            RegisterFactoryMethod(DocxElementNames.TableTableCellPadding, (pm) => new TableCellPaddingElement(pm, DocxElementNames.TableTableCellPadding));
            RegisterFactoryMethod(DocxElementNames.RowBanding, (pm) => new RowBandingElement(pm));
            RegisterFactoryMethod(DocxElementNames.ColumnBanding, (pm) => new ColumnBandingElement(pm));
            RegisterFactoryMethod(DocxElementNames.TablePreferredWidth, (pm) => new TableWidthElement(pm, DocxElementNames.TablePreferredWidth));
            RegisterFactoryMethod(DocxElementNames.TableFlowDirection, (pm) => new FlowDirectionElement(pm, DocxElementNames.TableFlowDirection));
            RegisterFactoryMethod(DocxElementNames.TableLayoutType, (pm) => new TableLayoutTypeElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableLooks, (pm) => new TableLooksElement(pm));
            RegisterFactoryMethod(DocxElementNames.Overlap, (pm) => new OverlapElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableStyleId, (pm) => new StyleIdElement(pm, DocxElementNames.TableStyleId));

            // section properties
            RegisterFactoryMethod(DocxElementNames.HeaderReference, (pm) => new HeaderReferenceElement(pm));
            RegisterFactoryMethod(DocxElementNames.FooterReference, (pm) => new FooterReferenceElement(pm));
            RegisterFactoryMethod(DocxElementNames.TitlePage, (pm) => new TitlePageElement(pm));
            RegisterFactoryMethod(DocxElementNames.SectionType, (pm) => new SectionTypeElement(pm));
            RegisterFactoryMethod(DocxElementNames.PageMargins, (pm) => new PageMarginsElement(pm));
            RegisterFactoryMethod(DocxElementNames.PageSize, (pm) => new PageSizeElement(pm));
            RegisterFactoryMethod(DocxElementNames.PageNumberingSettings, (pm) => new PageNumberingSettingsElement(pm));

            // table row properties
            RegisterFactoryMethod(DocxElementNames.RepeatOnEveryPage, (pm) => new RepeatOnEveryPageElement(pm));
            RegisterFactoryMethod(DocxElementNames.CanSplit, (pm) => new CanSplitElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableRowHeight, (pm) => new TableRowHeightElement(pm));

            // table cell properties
            RegisterFactoryMethod(DocxElementNames.GridSpan, (pm) => new GridSpanElement(pm));
            RegisterFactoryMethod(DocxElementNames.VerticalMerge, (pm) => new VerticalMergeElement(pm));
            RegisterFactoryMethod(DocxElementNames.HorizontalMerge, (pm) => new HorizontalMergeElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableCellBorders, (pm) => new TableCellBordersElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableCellPadding, (pm) => new TableCellPaddingElement(pm, DocxElementNames.TableCellPadding));
            RegisterFactoryMethod(DocxElementNames.IgnoreCellMarkerInRowHeightCalculation, (pm) => new IgnoreCellMarkerInRowHeightCalculationElement(pm));
            RegisterFactoryMethod(DocxElementNames.CanWrapContent, (pm) => new CanWrapContentElement(pm));
            RegisterFactoryMethod(DocxElementNames.TableCellPreferredWidth, (pm) => new TableWidthElement(pm, DocxElementNames.TableCellPreferredWidth));
            RegisterFactoryMethod(DocxElementNames.TextDirection, (pm) => new TextDirectionElement(pm));

            // drawings
            RegisterFactoryMethod(DocxElementNames.Extent, (pm) => new SizeElement(pm, DocxElementNames.Extent, OpenXmlNamespaces.WordprocessingDrawingMLNamespace));
            RegisterFactoryMethod(DocxElementNames.Drawing, (pm) => new DrawingElement(pm));
            RegisterFactoryMethod(DocxElementNames.Inline, (pm) => new InlineElement(pm));
            RegisterFactoryMethod(DocxElementNames.Anchor, (pm) => new AnchorElement(pm));
            RegisterFactoryMethod(DocxElementNames.DrawingObjectNonVisualProperties, (pm) => new NonVisualDrawingPropertiesElement(pm, DocxElementNames.DrawingObjectNonVisualProperties, OpenXmlNamespaces.WordprocessingDrawingMLNamespace));
            RegisterFactoryMethod(DocxElementNames.SimplePositioning, (pm) => new OffsetElement(pm, DocxElementNames.SimplePositioning, OpenXmlNamespaces.WordprocessingDrawingMLNamespace));
            RegisterFactoryMethod(DocxElementNames.HorizontalPosition, (pm) => new HorizontalPositionElement(pm));
            RegisterFactoryMethod(DocxElementNames.VerticalPosition, (pm) => new VerticalPositionElement(pm));
            RegisterFactoryMethod(DocxElementNames.Offset, (pm) => new OneDimentionOffsetElement(pm, DocxElementNames.Offset, OpenXmlNamespaces.WordprocessingDrawingMLNamespace));
            RegisterFactoryMethod(DocxElementNames.WrapNone, (pm) => new WrapNoneElement(pm));
            RegisterFactoryMethod(DocxElementNames.WrapSquare, (pm) => new WrapSquareElement(pm));
            RegisterFactoryMethod(DocxElementNames.WrapTopAndBottom, (pm) => new WrapTopAndBottomElement(pm));
            RegisterFactoryMethod(OpenXmlElementNames.PictureFillElementName, (pm) => new PictureFillElement(pm));
            RegisterFactoryMethod(DocxElementNames.EffectExtent, (pm) => new EffectExtentElement(pm));

            // vector
            RegisterFactoryMethod(DocxElementNames.VmlContainer, (pm) => new VmlContainerElement(pm));
            RegisterFactoryMethod(DocxElementNames.Shape, (pm) => new ShapeElement(pm));
            RegisterFactoryMethod(DocxElementNames.ImageData, (pm) => new ImageDataElement(pm));
            RegisterFactoryMethod(DocxElementNames.Fill, (pm) => new FillElement(pm));
            RegisterFactoryMethod(DocxElementNames.Textpath, (pm) => new TextpathElement(pm));

            // document settings
            RegisterFactoryMethod(DocxElementNames.EvenAndOddHeadersFooters, (pm) => new EvenAndOddHeadersFootersElement(pm));
            RegisterFactoryMethod(DocxElementNames.UpdateFields, (pm) => new UpdateFieldsElement(pm));
            RegisterFactoryMethod(DocxElementNames.Compatibility, (pm) => new CompatibilityElement(pm));
            RegisterFactoryMethod(DocxElementNames.CompatibilitySetting, (pm) => new CompatibilitySettingElement(pm));
            RegisterFactoryMethod(DocxElementNames.DocumentVariableCollection, (pm) => new DocumentVariableCollectionElement(pm));
            RegisterFactoryMethod(DocxElementNames.DocumentVariable, (pm) => new DocumentVariableElement(pm));
            RegisterFactoryMethod(DocxElementNames.DocumentProtection, (pm) => new DocumentProtectionElement(pm));
            RegisterFactoryMethod(DocxElementNames.DefaultTabStopWidth, (pm) => new DefaultTabStopWidthElement(pm));

            RegisterFactoryMethod(DocxElementNames.StructuredDocumentTag, (pm) => new StructuredDocumentTagElement(pm));
            RegisterFactoryMethod(DocxElementNames.StructuredDocumentTagContent, (pm) => new StructuredDocumentTagContentElement(pm));
        }

        public static OpenXmlElementBase CreateInstance(string elementName, DocxPartsManager partsManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");

            return elementNameToFactoryMethod[elementName](partsManager);
        }

        public static bool CanCreateInstance(string elementName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");

            return elementNameToFactoryMethod.ContainsKey(elementName);
        }

        private static void RegisterFactoryMethod(string elementName, Func<DocxPartsManager, OpenXmlElementBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(elementName, factoryMethod);
        }
    }
}
