﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model
{
    internal class DocxPartsFactory
    {
        private static readonly Dictionary<string, Func<DocxPartsManager, string, DocxPartBase>> elementNameToFactoryMethod;

        static DocxPartsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<DocxPartsManager, string, DocxPartBase>>();

            RegisterFactoryMethod(DocxContentTypeNames.DocumentContentType, (pm, pn) => new DocumentPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.HeaderContentType, (pm, pn) => new HeaderPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.FooterContentType, (pm, pn) => new FooterPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.DocumentSettingsContentType, (pm, pn) => new DocumentSettingsPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.StylesContentType, (pm, pn) => new StylesPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.CommentsContentType, (pm, pn) => new CommentsPart(pm, pn));
            RegisterFactoryMethod(DocxContentTypeNames.ListsContentType, (pm, pn) => new ListsPart(pm, pn));
        }

        public static DocxPartBase CreateInstance(string partType, DocxPartsManager partsManager, string partName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNullOrEmpty(partName, "partName");

            return elementNameToFactoryMethod[partType](partsManager, partName);
        }

        public static bool CanCreateInstance(string partType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");

            return elementNameToFactoryMethod.ContainsKey(partType);
        }

        private static void RegisterFactoryMethod(string partType, Func<DocxPartsManager, string, DocxPartBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(partType, factoryMethod);
        }
    }
}
