﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Comments
{
    internal class CommentElement : BlockLevelElementsContainerElementBase<BlockContainerBase>
    {
        private readonly IntOpenXmlAttribute idAttribute;
        private readonly OpenXmlAttribute<string> authorAttribute;
        private readonly OpenXmlAttribute<string> initialsAttribute;
        private readonly OpenXmlAttribute<string> dateAttribute;

        public CommentElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.idAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("id", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.authorAttribute = this.RegisterAttribute<string>("author", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.initialsAttribute = this.RegisterAttribute<string>("initials", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.dateAttribute = this.RegisterAttribute<string>("date", OpenXmlNamespaces.WordprocessingMLNamespace);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Comment;
            }
        }

        public int Id
        {
            get
            {
                return this.idAttribute.Value;
            }
            set
            {
                this.idAttribute.Value = value;
            }
        }

        public string Author
        {
            get
            {
                return this.authorAttribute.Value;
            }
            set
            {
                this.authorAttribute.Value = value;
            }
        }

        public string Initials
        {
            get
            {
                return this.initialsAttribute.Value;
            }
            set
            {
                this.initialsAttribute.Value = value;
            }
        }

        public string Date
        {
            get
            {
                return this.dateAttribute.Value;
            }
            set
            {
                this.dateAttribute.Value = value;
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Comment comment = (Comment)this.BlockContainer;
            comment.Author = this.Author;
            comment.Initials = this.Initials;
            DateTime commentDate;
            if (DateTime.TryParse(this.Date, out commentDate))
            {
                comment.Date = commentDate;
            }
        }
    }
}