﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal abstract class BlockLevelElementsContainerElementBase<T> : DocumentElementBase
        where T : BlockContainerBase
    {
        private T blockContainer;

        public BlockLevelElementsContainerElementBase(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        protected T BlockContainer
        {
            get
            {
                return this.blockContainer;
            }
        }

        public void SetAssociatedFlowModelElement(T blockContainer)
        {
            Guard.ThrowExceptionIfNull(blockContainer, "blockContainer");

            this.blockContainer = blockContainer;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return this.blockContainer.Blocks.Any() || base.ShouldExport(context);
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (BlockBase block in this.blockContainer.Blocks)
            {
                switch (block.Type)
                {
                    case DocumentElementType.Paragraph:
                        ParagraphElement paragraphElement = this.CreateElement<ParagraphElement>(DocxElementNames.Paragraph);
                        paragraphElement.SetAssociatedFlowModelElement((Paragraph)block);

                        yield return paragraphElement;

                        break;

                    case DocumentElementType.Table:
                        TableElement tableElement = this.CreateElement<TableElement>(DocxElementNames.Table);
                        tableElement.SetAssociatedFlowModelElement((Table)block);

                        yield return tableElement;

                        break;
                }
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            switch (childElement.ElementName)
            {
                case DocxElementNames.Paragraph:
                    ((ParagraphElement)childElement).SetAssociatedFlowModelElement(this.blockContainer.Blocks.AddParagraph());
                    break;
                case DocxElementNames.Table:
                    ((TableElement)childElement).SetAssociatedFlowModelElement(this.blockContainer.Blocks.AddTable());
                    break;
            }
        }
    }
}
