﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class BodyElement : DocxElementBase
    {
        private bool hasSectionPropertiesChild;

        public BodyElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.hasSectionPropertiesChild = false;
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Body;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return context.GetSections().Any() || base.ShouldExport(context);
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            List<Section> sections = context.GetSections().ToList();

            foreach (Section section in sections)
            {
                foreach (BlockBase block in section.Blocks)
                {
                    switch (block.Type)
                    {
                        case DocumentElementType.Paragraph:
                            ParagraphElement paragraphElement = this.CreateElement<ParagraphElement>(DocxElementNames.Paragraph);
                            paragraphElement.SetAssociatedFlowModelElement((Paragraph)block, section);

                            yield return paragraphElement;
                            break;

                        case DocumentElementType.Table:
                            TableElement tableElement = this.CreateElement<TableElement>(DocxElementNames.Table);
                            tableElement.SetAssociatedFlowModelElement((Table)block);

                            yield return tableElement;
                            break;
                    }
                }
            }

            SectionPropertiesElement sectionPropertiesElement = this.CreateElement<SectionPropertiesElement>(DocxElementNames.SectionProperties);
            sectionPropertiesElement.SetAssociatedFlowModelElement(sections.Last());

            yield return sectionPropertiesElement;
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            switch (childElement.ElementName)
            {
                case DocxElementNames.SectionProperties:

                    ((SectionPropertiesElement)childElement).SetAssociatedFlowModelElement(context.GetCurrentSection());
                    this.hasSectionPropertiesChild = true;
                    break;

                case DocxElementNames.Paragraph:
                    ((ParagraphElement)childElement).SetAssociatedFlowModelElement(context.GetCurrentSection().Blocks.AddParagraph());
                    break;

                case DocxElementNames.Table:
                    ((TableElement)childElement).SetAssociatedFlowModelElement(context.GetCurrentSection().Blocks.AddTable());
                    break;
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            DocxHelper.AddHangingAnnotation(context, childElement);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (!this.hasSectionPropertiesChild)
            {
                context.GetCurrentSection();
                context.CloseCurrentSection();
            }
        }

        protected override void ClearOverride()
        {
            this.hasSectionPropertiesChild = false;
        }
    }
}
