﻿using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class BookmarkStartElement : AnnotationStartEndElementBase
    {
        private readonly OpenXmlAttribute<string> nameAttribute;
        private readonly IntOpenXmlAttribute colFirstAttribute;
        private readonly IntOpenXmlAttribute colLastAttribute;

        public BookmarkStartElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.nameAttribute = this.RegisterAttribute<string>("name", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.colFirstAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("colFirst", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.colLastAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("colLast", OpenXmlNamespaces.WordprocessingMLNamespace));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.BookmarkStart;
            }
        }

        public string Name
        {
            get
            {
                return this.nameAttribute.Value;
            }
            set
            {
                this.nameAttribute.Value = value;
            }
        }

        public int ColFirst
        {
            get
            {
                return this.colFirstAttribute.Value;
            }
            set
            {
                this.colFirstAttribute.Value = value;
            }
        }

        public int ColLast
        {
            get
            {
                return this.colLastAttribute.Value;
            }
            set
            {
                this.colLastAttribute.Value = value;
            }
        }

        internal void CopyPropertiesFrom(Bookmark bookmark)
        {
            this.Id = AnnotationIdGenerator.GetNext();
            this.Name = bookmark.Name;

            if (bookmark.FromColumn != bookmark.ToColumn)
            {
                this.ColFirst = (int)bookmark.FromColumn;
                this.ColLast = (int)bookmark.ToColumn;
            }
        }
    }
}