﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class BreakElement : DocumentElementBase
    {
        private readonly MappedOpenXmlAttribute<BreakType> typeAttribute;
        private readonly MappedOpenXmlAttribute<TextWrappingRestartLocation> clearAttribute;

        public BreakElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.typeAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<BreakType>("type", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.BreakTypeMapper));
            this.clearAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<TextWrappingRestartLocation>("clear", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.RestartLocationMapper));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Break;
            }
        }

        public BreakType Type
        {
            get
            {
                return this.typeAttribute.Value;
            }
            set
            {
                this.typeAttribute.Value = value;
            }
        }

        public TextWrappingRestartLocation TextWrappingRestartLocation
        {
            get
            {
                return this.clearAttribute.Value;
            }
            set
            {
                this.clearAttribute.Value = value;
            }
        }

        internal void CopyPropertiesFrom(IDocxExportContext context, Break br)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (br.BreakType == BreakType.LineBreak &&
                br.TextWrappingRestartLocation == TextWrappingRestartLocation.NextLine)
            {
                return;
            }

            this.Type = br.BreakType;

            if (br.BreakType == BreakType.LineBreak)
            {
                this.TextWrappingRestartLocation = br.TextWrappingRestartLocation;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }
    }
}