﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class DrawingElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<InlineElement> inline;
        private readonly OpenXmlChildElement<AnchorElement> anchor;

        public DrawingElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.inline = this.RegisterChildElement<InlineElement>(DocxElementNames.Inline);
            this.anchor = this.RegisterChildElement<AnchorElement>(DocxElementNames.Anchor);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Drawing;
            }
        }

        public InlineElement InlineElement
        {
            get
            {
                return this.inline.Element;
            }
        }

        public AnchorElement AnchorElement
        {
            get
            {
                return this.anchor.Element;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeInlineBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            this.CreateElement(this.inline);
            this.InlineElement.CopyPropertiesFrom(context, shape);
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeAnchorBase anchor)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(anchor, "anchor");

            this.CreateElement(this.anchor);
            this.AnchorElement.CopyPropertiesFrom(context, anchor);
        }

        public InlineBase CreateInline(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            InlineBase result = null;
            if (this.InlineElement != null)
            {
                result = this.InlineElement.CreateShapeInline(context);
                this.ReleaseElement(this.inline);
            }

            if (this.AnchorElement != null)
            {
                result = this.AnchorElement.CreateShapeInline(context);
                this.ReleaseElement(this.anchor);
            }

            return result;
        }
    }
}
