﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class FieldCharacterElement : DocumentElementBase
    {
        private readonly MappedOpenXmlAttribute<FieldCharacterType> filedCharTypeAttribute;
        private readonly BoolOpenXmlAttribute isLocked;
        private readonly BoolOpenXmlAttribute isDirty;

        public FieldCharacterElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.filedCharTypeAttribute = new MappedOpenXmlAttribute<FieldCharacterType>(
                "fldCharType", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.FileldCharTypeMapper, true);
            this.RegisterAttribute(this.filedCharTypeAttribute);

            this.isLocked = this.RegisterAttribute(new BoolOpenXmlAttribute("fldLock", OpenXmlNamespaces.WordprocessingMLNamespace, false));
            this.isDirty = this.RegisterAttribute(new BoolOpenXmlAttribute("dirty", OpenXmlNamespaces.WordprocessingMLNamespace, false));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.FieldCharacter;
            }
        }

        public FieldCharacterType FieldCharacterType
        {
            get
            {
                return this.filedCharTypeAttribute.Value;
            }
            set
            {
                this.filedCharTypeAttribute.Value = value;
            }
        }

        public bool IsLocked
        {
            get
            {
                return this.isLocked.Value;
            }
            set
            {
                this.isLocked.Value = value;
            }
        }

        public bool IsDirty
        {
            get
            {
                return this.isDirty.Value;
            }
            set
            {
                this.isDirty.Value = value;
            }
        }
    }
}
