﻿using System.Diagnostics;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class HyperlinkElement : ParagraphContentElementBase
    {
        private readonly OpenXmlAttribute<string> anchorAttribute;
        private readonly OpenXmlAttribute<string> tooltipAttribute;
        private readonly OpenXmlAttribute<string> relationshipIdAttribute;

        public HyperlinkElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.anchorAttribute = this.RegisterAttribute<string>("anchor", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.tooltipAttribute = this.RegisterAttribute<string>("tooltip", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.relationshipIdAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("id", OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Hyperlink;
            }
        }

        public string RelationshipId
        {
            get
            {
                return this.relationshipIdAttribute.Value;
            }
            set
            {
                this.relationshipIdAttribute.Value = value;
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Hyperlink hyperlink = new Hyperlink(context.Document);

            if (this.anchorAttribute.HasValue)
            {
                hyperlink.Uri = this.anchorAttribute.Value;
                hyperlink.IsAnchor = true;
            }
            else if (this.relationshipIdAttribute.HasValue)
            {
                hyperlink.Uri = this.PartsManager.GetRelationshipTarget(this.Part.Name, this.RelationshipId);
            }
            else
            {
                Debug.Assert(false, "No bookmark or URI found in hyperlink element");
            }
            if (this.tooltipAttribute.HasValue)
            {
                hyperlink.ToolTip = this.tooltipAttribute.Value;
            }

            FieldInfo hyperlinkField = new FieldInfo(context.Document, hyperlink);
            this.Paragraph.Inlines.Add(hyperlinkField.Start);
            this.Paragraph.Inlines.AddRun().Text = hyperlink.CreateHyperlinkCode();
            this.Paragraph.Inlines.Add(hyperlinkField.Separator);
            this.MoveInlinesToParagraph();
            this.Paragraph.Inlines.Add(hyperlinkField.End);
        }
    }
}