﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class PageMarginsElement : DocxElementBase
    {
        private readonly ConvertedOpenXmlAttribute<double> topAttribute;
        private readonly ConvertedOpenXmlAttribute<double> rightAttribute;
        private readonly ConvertedOpenXmlAttribute<double> bottomAttribute;
        private readonly ConvertedOpenXmlAttribute<double> leftAttribute;
        private readonly ConvertedOpenXmlAttribute<double> gutterAttribute;
        private readonly ConvertedOpenXmlAttribute<double> headerAttribute;
        private readonly ConvertedOpenXmlAttribute<double> footerAttribute;

        public PageMarginsElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.topAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("top", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, true));
            this.rightAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("right", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, true));
            this.bottomAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("bottom", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, true));
            this.leftAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("left", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, true));
            this.headerAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("header", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, DocumentDefaultStyleSettings.SectionHeaderTopMargin, true));
            this.footerAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("footer", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, DocumentDefaultStyleSettings.SectionFooterBottomMargin, true));
            this.gutterAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("gutter", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter, true));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.PageMargins;
            }
        }

        public double Top
        {
            get
            {
                return this.topAttribute.Value;
            }
            set
            {
                this.topAttribute.Value = value;
            }
        }

        public double Right
        {
            get
            {
                return this.rightAttribute.Value;
            }
            set
            {
                this.rightAttribute.Value = value;
            }
        }

        public double Bottom
        {
            get
            {
                return this.bottomAttribute.Value;
            }
            set
            {
                this.bottomAttribute.Value = value;
            }
        }

        public double Left
        {
            get
            {
                return this.leftAttribute.Value;
            }
            set
            {
                this.leftAttribute.Value = value;
            }
        }

        public double Header
        {
            get
            {
                return this.headerAttribute.Value;
            }
            set
            {
                this.headerAttribute.Value = value;
            }
        }

        public double Footer
        {
            get
            {
                return this.footerAttribute.Value;
            }
            set
            {
                this.footerAttribute.Value = value;
            }
        }

        public double Gutter
        {
            get
            {
                return this.gutterAttribute.Value;
            }
            set
            {
                this.gutterAttribute.Value = value;
            }
        }
    }
}
