﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class PageNumberingSettingsElement : DocumentElementBase
    {
        private readonly MappedOpenXmlAttribute<ChapterSeparatorType> chapterSeparatorCharacterAttribute;
        private readonly OpenXmlAttribute<int> chapterHeadingStyleIndexAttribute;
        private readonly MappedOpenXmlAttribute<NumberingStyle> pageNumberFormatAttribute;
        private readonly OpenXmlAttribute<int> startingPageNumberAttribute;

        public PageNumberingSettingsElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.chapterSeparatorCharacterAttribute =
                this.RegisterAttribute(new MappedOpenXmlAttribute<ChapterSeparatorType>("chapSep", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ChapterSeparatorTypeMapper));

            this.chapterHeadingStyleIndexAttribute =
                this.RegisterAttribute<int>("chapStyle", OpenXmlNamespaces.WordprocessingMLNamespace);

            this.pageNumberFormatAttribute =
                this.RegisterAttribute(new MappedOpenXmlAttribute<NumberingStyle>("fmt", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.NumberingStyleMapper));

            this.startingPageNumberAttribute =
                this.RegisterAttribute<int>("start", OpenXmlNamespaces.WordprocessingMLNamespace);
        }

        public override string ElementName
        {
            get
            {
                {
                    return DocxElementNames.PageNumberingSettings;
                }
            }
        }

        internal void CopyPropertiesFrom(PageNumberingSettings pageNumberingSettings)
        {
            if (pageNumberingSettings.ChapterSeparatorCharacter.HasValue)
            {
                this.chapterSeparatorCharacterAttribute.Value = pageNumberingSettings.ChapterSeparatorCharacter.Value;
            }

            if (pageNumberingSettings.ChapterHeadingStyleIndex.HasValue)
            {
                this.chapterHeadingStyleIndexAttribute.Value = pageNumberingSettings.ChapterHeadingStyleIndex.Value;
            }

            if (pageNumberingSettings.PageNumberFormat.HasValue)
            {
                this.pageNumberFormatAttribute.Value = pageNumberingSettings.PageNumberFormat.Value;
            }

            if (pageNumberingSettings.StartingPageNumber.HasValue)
            {
                this.startingPageNumberAttribute.Value = pageNumberingSettings.StartingPageNumber.Value;
            }
        }

        internal void CopyPropertiesTo(PageNumberingSettings pageNumberingSettings)
        {
            if (this.chapterSeparatorCharacterAttribute.HasValue)
            {
                pageNumberingSettings.ChapterSeparatorCharacter = this.chapterSeparatorCharacterAttribute.Value;
            }

            if (this.chapterHeadingStyleIndexAttribute.HasValue)
            {
                pageNumberingSettings.ChapterHeadingStyleIndex = this.chapterHeadingStyleIndexAttribute.Value;
            }

            if (this.pageNumberFormatAttribute.HasValue)
            {
                pageNumberingSettings.PageNumberFormat = this.pageNumberFormatAttribute.Value;
            }

            if (this.startingPageNumberAttribute.HasValue)
            {
                pageNumberingSettings.StartingPageNumber = this.startingPageNumberAttribute.Value;
            }
        }
    }
}
