﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class PageSizeElement : DocxElementBase
    {
        private readonly ConvertedOpenXmlAttribute<double> heightAttribute;
        private readonly ConvertedOpenXmlAttribute<double> widthAttribute;
        private readonly MappedOpenXmlAttribute<PageOrientation> orientationAttribute;

        public PageSizeElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.heightAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("h", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter));
            this.widthAttribute = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("w", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter));
            this.orientationAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<PageOrientation>("orient", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.PageOrientationMapper));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.PageSize;
            }
        }

        public double Width
        {
            get
            {
                return this.widthAttribute.Value;
            }
            set
            {
                this.widthAttribute.Value = value;
            }
        }

        public double Height
        {
            get
            {
                return this.heightAttribute.Value;
            }
            set
            {
                this.heightAttribute.Value = value;
            }
        }

        public PageOrientation PageOrientation
        {
            get
            {
                return this.orientationAttribute.Value;
            }

            set
            {
                this.orientationAttribute.Value = value;
            }
        }
    }
}
