using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal abstract class ParagraphContentElementBase : DocumentElementBase
    {
        // This is just a temp paragraph to store the content while the element is imported
        private Section tempSection;
        private Paragraph tempParagraph;

        public ParagraphContentElementBase(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        protected Paragraph Paragraph { get; private set; }

        public void SetAssociatedFlowModelElementParent(Paragraph paragraph)
        {
            Guard.ThrowExceptionIfNull(paragraph, "paragraph");

            this.Paragraph = paragraph;
        }

        protected override void OnBeforeReadInnerElements(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            this.tempSection = new Section(context.Document);
            this.tempParagraph = this.tempSection.Blocks.AddParagraph();
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "element");

            switch (childElement.ElementName)
            {
                case DocxElementNames.Run:
                    ((RunElement)childElement).SetAssociatedFlowModelElementParent(this.tempParagraph);
                    break;
            }
        }

        protected void MoveInlinesToParagraph()
        {
            // The paragraph could be null when the element doesn't have any child elements.
            if (this.tempParagraph == null)
            {
                return;
            }

            while (this.tempParagraph.Inlines.Count > 0)
            {
                InlineBase inline = this.tempParagraph.Inlines[0];
                this.tempParagraph.Inlines.RemoveAt(0);
                this.Paragraph.Inlines.Add(inline);
            }
        }

        protected override void ClearOverride()
        {
            base.ClearOverride();
            this.tempSection = null;
            this.tempParagraph = null;
        }
    }
}