﻿using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Protection;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class PermissionStartElement : AnnotationStartEndElementBase
    {
        private readonly OpenXmlAttribute<string> editorCredentialsAttribute;
        private readonly MappedOpenXmlAttribute<EditingGroup?> editingGroupAttribute;
        private readonly IntOpenXmlAttribute colFirstAttribute;
        private readonly IntOpenXmlAttribute colLastAttribute;

        public PermissionStartElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.editorCredentialsAttribute = this.RegisterAttribute<string>("ed", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.editingGroupAttribute = this.RegisterAttribute(
               new MappedOpenXmlAttribute<EditingGroup?>("edGrp", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.EditingGroupTypeMapper));
            this.colFirstAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("colFirst", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.colLastAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("colLast", OpenXmlNamespaces.WordprocessingMLNamespace));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.PermissionStart;
            }
        }

        public string Editor
        {
            get
            {
                return this.editorCredentialsAttribute.Value;
            }
            set
            {
                this.editorCredentialsAttribute.Value = value;
            }
        }

        public EditingGroup? EditingGroup
        {
            get
            {
                return this.editingGroupAttribute.Value;
            }
            set
            {
                this.editingGroupAttribute.Value = value;
            }
        }

        public int ColFirst
        {
            get
            {
                return this.colFirstAttribute.Value;
            }
            set
            {
                this.colFirstAttribute.Value = value;
            }
        }

        public int ColLast
        {
            get
            {
                return this.colLastAttribute.Value;
            }
            set
            {
                this.colLastAttribute.Value = value;
            }
        }

        internal void CopyPropertiesFrom(PermissionRange permission)
        {
            this.Id = AnnotationIdGenerator.GetNext();

            if (permission.Credentials.EditingGroup.HasValue)
            {
                this.EditingGroup = permission.Credentials.EditingGroup.Value;
            }

            if (!string.IsNullOrEmpty(permission.Credentials.Editor))
            {
                this.Editor = permission.Credentials.Editor;
            }

            if (permission.FromColumn != permission.ToColumn)
            {
                this.ColFirst = (int)permission.FromColumn;
                this.ColLast = (int)permission.ToColumn;
            }
        }
    }
}
