﻿using System;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.RunPropertiesElements;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;
using FlowDirection = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class RunPropertiesElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<BoldElement> boldChildElement;
        private readonly OpenXmlChildElement<ItalicElement> italicChildElement;
        private readonly OpenXmlChildElement<StrikethroughElement> strikethroughChildElement;
        private readonly OpenXmlChildElement<BaselineAlignmentElement> baselineAlignmentChildElement;
        private readonly OpenXmlChildElement<ShadingElement> shadingChildElement;
        private readonly OpenXmlChildElement<FontSizeElement> fontSizeChildElement;
        private readonly OpenXmlChildElement<FontElement> fontChildElement;
        private readonly OpenXmlChildElement<ForegroundColorElement> foregroundColorChildElement;
        private readonly OpenXmlChildElement<HighlightElement> highlightColorChildElement = null;
        private readonly OpenXmlChildElement<UnderlineElement> underlineElementChildElement;
        private readonly OpenXmlChildElement<FlowDirectionElement> flowDirectionElement;
        private readonly OpenXmlChildElement<StyleIdElement> styleIdElement;

        private CharacterProperties characterProperties;

        public RunPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.styleIdElement = this.RegisterChildElement<StyleIdElement>(DocxElementNames.RunStyleId);
            this.fontChildElement = this.RegisterChildElement<FontElement>(DocxElementNames.Font);
            this.boldChildElement = this.RegisterChildElement<BoldElement>(DocxElementNames.Bold);
            this.italicChildElement = this.RegisterChildElement<ItalicElement>(DocxElementNames.Italic);
            this.strikethroughChildElement = this.RegisterChildElement<StrikethroughElement>(DocxElementNames.Strikethrough);
            this.foregroundColorChildElement = this.RegisterChildElement<ForegroundColorElement>(DocxElementNames.ForegroundColor);
            this.fontSizeChildElement = this.RegisterChildElement<FontSizeElement>(DocxElementNames.FontSize);
            this.highlightColorChildElement = this.RegisterChildElement<HighlightElement>(DocxElementNames.Highlight);
            this.underlineElementChildElement = this.RegisterChildElement<UnderlineElement>(DocxElementNames.Underline);
            this.shadingChildElement = this.RegisterChildElement<ShadingElement>(DocxElementNames.Shading);
            this.baselineAlignmentChildElement = this.RegisterChildElement<BaselineAlignmentElement>(DocxElementNames.BaselineAlignment);
            this.flowDirectionElement = this.RegisterChildElement<FlowDirectionElement>(DocxElementNames.RunFlowDirection);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.RunProperties;
            }
        }

        public CharacterProperties CharacterProperties
        {
            get
            {
                return this.characterProperties;
            }
        }

        public void SetAssociatedFlowModelElement(CharacterProperties characterProperties)
        {
            Guard.ThrowExceptionIfNull(characterProperties, "characterProperties");

            this.characterProperties = characterProperties;
        }

        public void CopyPropertiesTo(CharacterProperties characterProperties)
        {
            Guard.ThrowExceptionIfNull(characterProperties, "characterProperties");

            if (this.boldChildElement.Element != null)
            {
                characterProperties.FontWeight.LocalValue = this.boldChildElement.Element.Value ? FontWeights.Bold : FontWeights.Normal;
                this.ReleaseElement(this.boldChildElement);
            }

            if (this.italicChildElement.Element != null)
            {
                characterProperties.FontStyle.LocalValue = this.italicChildElement.Element.Value ? FontStyles.Italic : FontStyles.Normal;
                this.ReleaseElement(this.italicChildElement);
            }

            if (this.strikethroughChildElement.Element != null)
            {
                characterProperties.Strikethrough.LocalValue = this.strikethroughChildElement.Element.Value;
                this.ReleaseElement(this.strikethroughChildElement);
            }

            if (this.baselineAlignmentChildElement.Element != null)
            {
                characterProperties.BaselineAlignment.LocalValue = this.baselineAlignmentChildElement.Element.Value;
                this.ReleaseElement(this.baselineAlignmentChildElement);
            }

            if (this.shadingChildElement.Element != null)
            {
                this.shadingChildElement.Element.ReadAttributes(characterProperties);
                this.ReleaseElement(this.shadingChildElement);
            }

            if (this.fontSizeChildElement.Element != null)
            {
                double size = Unit.PointToDip(this.fontSizeChildElement.Element.Value / 2.0);
                characterProperties.FontSize.LocalValue = size;
                this.ReleaseElement(this.fontSizeChildElement);
            }

            if (this.fontChildElement.Element != null)
            {
                ThemableFontFamily fontFamily = this.fontChildElement.Element.GetFontFamily();
                if (fontFamily != null)
                {
                    characterProperties.FontFamily.LocalValue = fontFamily;
                }

                this.ReleaseElement(this.fontChildElement);
            }

            if (this.foregroundColorChildElement.Element != null)
            {
                ThemableColor foreColor = this.foregroundColorChildElement.Element.GetThemableColor();
                if (foreColor != null)
                {
                    characterProperties.ForegroundColor.LocalValue = foreColor;
                }

                this.ReleaseElement(this.foregroundColorChildElement);
            }

            if (this.highlightColorChildElement.Element != null)
            {
                characterProperties.HighlightColor.LocalValue = this.highlightColorChildElement.Element.Value;
                this.ReleaseElement(this.highlightColorChildElement);
            }

            if (this.underlineElementChildElement.Element != null)
            {
                this.underlineElementChildElement.Element.ReadAttributes(characterProperties);
                this.ReleaseElement(this.underlineElementChildElement);
            }

            if (this.flowDirectionElement.Element != null)
            {
                characterProperties.FlowDirection.LocalValue = this.flowDirectionElement.Element.Value ? FlowDirection.RightToLeft : FlowDirection.LeftToRight;
                this.ReleaseElement(this.flowDirectionElement);
            }

            if (this.styleIdElement.Element != null)
            {
                characterProperties.StyleId = this.styleIdElement.Element.Value;
                this.ReleaseElement(this.styleIdElement);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            DocumentTheme theme = context.Document.Theme;

            if (this.CharacterProperties.FontWeight.HasLocalValue)
            {
                this.CreateElement(this.boldChildElement);
                FontWeight boldLocalValue = this.CharacterProperties.FontWeight.LocalValue.Value;
                this.boldChildElement.Element.Value = boldLocalValue == FontWeights.Bold;
            }

            if (this.CharacterProperties.FontStyle.HasLocalValue)
            {
                this.CreateElement(this.italicChildElement);
                FontStyle italicLocalValue = this.CharacterProperties.FontStyle.LocalValue.Value;
                this.italicChildElement.Element.Value = italicLocalValue == FontStyles.Italic;
            }

            if (this.CharacterProperties.Strikethrough.HasLocalValue)
            {
                this.CreateElement(this.strikethroughChildElement);
                this.strikethroughChildElement.Element.Value = this.CharacterProperties.Strikethrough.LocalValue.Value;
            }

            if (this.CharacterProperties.BaselineAlignment.HasLocalValue)
            {
                this.CreateElement(this.baselineAlignmentChildElement);
                this.baselineAlignmentChildElement.Element.Value = this.CharacterProperties.BaselineAlignment.LocalValue.Value;
            }

            if (this.CharacterProperties.ShadingPattern.HasLocalValue ||
                this.CharacterProperties.ShadingPatternColor.HasLocalValue ||
                this.CharacterProperties.BackgroundColor.HasLocalValue)
            {
                this.CreateElement(this.shadingChildElement);
                this.shadingChildElement.Element.FillAttributes(this.CharacterProperties);
            }

            if (this.CharacterProperties.FontSize.HasLocalValue)
            {
                this.CreateElement(this.fontSizeChildElement);
                double size = this.CharacterProperties.FontSize.LocalValue.Value;
                this.fontSizeChildElement.Element.Value = (int)Math.Round(2 * Unit.DipToPoint(size));
            }

            if (this.CharacterProperties.FontFamily.HasLocalValue)
            {
                this.CreateElement(this.fontChildElement);
                ThemableFontFamily fontFamily = this.CharacterProperties.FontFamily.LocalValue;
                this.fontChildElement.Element.FillAttributes(fontFamily, theme);
            }

            if (this.CharacterProperties.ForegroundColor.HasLocalValue)
            {
                this.CreateElement(this.foregroundColorChildElement);
                ThemableColor color = this.CharacterProperties.ForegroundColor.LocalValue;
                this.foregroundColorChildElement.Element.FillAttributes(color, theme);
            }

            if (this.CharacterProperties.HighlightColor.HasLocalValue)
            {
                this.CreateElement(this.highlightColorChildElement);
                Color color = this.characterProperties.HighlightColor.LocalValue.Value;
                this.highlightColorChildElement.Element.Value = color;
            }

            if (this.CharacterProperties.UnderlineColor.HasLocalValue ||
                this.CharacterProperties.UnderlinePattern.HasLocalValue)
            {
                this.CreateElement(this.underlineElementChildElement);
                this.underlineElementChildElement.Element.FillAttributes(this.characterProperties, theme);
            }

            if (this.CharacterProperties.FlowDirection.HasLocalValue)
            {
                this.CreateElement(this.flowDirectionElement);
                FlowDirection flowDirectionValue = this.CharacterProperties.FlowDirection.LocalValue.Value;
                this.flowDirectionElement.Element.Value = flowDirectionValue == FlowDirection.RightToLeft ? true : false;
            }

            if (!string.IsNullOrEmpty(this.CharacterProperties.StyleId))
            {
                this.CreateElement(this.styleIdElement);
                this.styleIdElement.Element.Value = this.CharacterProperties.StyleId;
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.characterProperties != null)
            {
                this.CopyPropertiesTo(this.characterProperties);
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return this.characterProperties != null || base.ShouldExport(context);
        }

        protected override void ClearOverride()
        {
            this.characterProperties = null;
        }
    }
}