﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class SimpleFieldElement : ParagraphContentElementBase
    {
        private readonly OpenXmlAttribute<string> codeAttribute;
        private readonly BoolOpenXmlAttribute isLocked;
        private readonly BoolOpenXmlAttribute isDirty;

        public SimpleFieldElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.codeAttribute = this.RegisterAttribute<string>("instr", OpenXmlNamespaces.WordprocessingMLNamespace, true);
            this.isLocked = this.RegisterAttribute(new BoolOpenXmlAttribute("fldLock", OpenXmlNamespaces.WordprocessingMLNamespace, false));
            this.isDirty = this.RegisterAttribute(new BoolOpenXmlAttribute("dirty", OpenXmlNamespaces.WordprocessingMLNamespace, false));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.SimpleField;
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string code = string.Empty;
            if (this.codeAttribute.HasValue)
            {
                code = this.codeAttribute.Value;
            }

            FieldInfo field = new FieldInfo(context.Document);
            field.IsLocked = this.isLocked.Value;
            field.IsDirty = this.isDirty.Value;

            this.Paragraph.Inlines.Add(field.Start);
            this.Paragraph.Inlines.AddRun(code);
            this.Paragraph.Inlines.Add(field.Separator);
            this.MoveInlinesToParagraph();
            this.Paragraph.Inlines.Add(field.End);

            field.LoadFieldFromCode();
        }
    }
}