﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Watermarks;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class StructuredDocumentTagContentElement : DocxElementBase
    {
        private Watermark watermark;
        private Paragraph watermarkOwnerDummyParagraph;

        public StructuredDocumentTagContentElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.StructuredDocumentTagContent;
            }
        }

        public Watermark Watermark
        {
            get
            {
                return this.watermark;
            }
        }

        protected override bool ShouldImport(IDocxImportContext context)
        {
            return true;
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            if (childElement.ElementName == DocxElementNames.Paragraph)
            {
                // At the moment the structured document tag is used only for the import of watermarks. The paragraph held in the SDT
                // holds a watermark and is therefore not to be added to the imported document.
                this.watermarkOwnerDummyParagraph = new Paragraph(context.Document);

                ((ParagraphElement)childElement).SetAssociatedFlowModelElement(this.watermarkOwnerDummyParagraph);
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            if (childElement.ElementName == DocxElementNames.Paragraph)
            {
                ParagraphElement paragraphElement = childElement as ParagraphElement;
                if (paragraphElement.Watermark != null)
                {
                    this.watermark = paragraphElement.Watermark;
                }
            }
        }

        protected override void ClearOverride()
        {
            this.watermark = null;
            this.watermarkOwnerDummyParagraph = null;
        }
    }
}
