﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    /// <summary>
    /// Represents an element which is used as a tab stop in a paragraph's properties and as a tab in a run content.
    /// </summary>
    internal class TabElement : DocumentElementBase
    {
        private readonly OpenXmlAttribute<double> positionAttribute;
        private readonly MappedOpenXmlAttribute<TabStopType> valAttribute;
        private readonly MappedOpenXmlAttribute<TabStopLeader> leaderAttribute;

        public TabElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.positionAttribute = this.RegisterAttribute<double>("pos", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.valAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<TabStopType>("val", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.TabStopTypeMapper));
            this.leaderAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<TabStopLeader>("leader", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.TabStopLeaderMapper));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Tab;
            }
        }

        public double Position
        {
            get
            {
                return this.positionAttribute.Value;
            }
            set
            {
                this.positionAttribute.Value = value;
            }
        }

        public TabStopType Type
        {
            get
            {
                return this.valAttribute.Value;
            }
            set
            {
                this.valAttribute.Value = value;
            }
        }

        public TabStopLeader Leader
        {
            get
            {
                return this.leaderAttribute.Value;
            }
            set
            {
                this.leaderAttribute.Value = value;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }
    }
}