﻿using Telerik.Windows.Documents.Flow.FormatProviders.Common.Tables;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class TableCellElement : BlockLevelElementsContainerElementBase<TableCell>
    {
        private readonly OpenXmlChildElement<TableCellPropertiesElement> tableCellPropertiesChildElement;

        public TableCellElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.tableCellPropertiesChildElement = this.RegisterChildElement<TableCellPropertiesElement>(DocxElementNames.TableCellProperties);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableCell;
            }
        }

        public TableCellData TableCellData { get; set; }

        private TableCellPropertiesElement TableCellPropertiesElement
        {
            get
            {
                return this.tableCellPropertiesChildElement.Element;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.BlockContainer.Properties.HasLocalValues())
            {
                this.CreateElement(this.tableCellPropertiesChildElement);
                this.TableCellPropertiesElement.SetAssociatedFlowModelElement(this.BlockContainer.Properties);
                this.TableCellPropertiesElement.TableCellType = this.TableCellData.Type;
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            TableCellType tableCellType = TableCellType.Cell;

            if (this.TableCellPropertiesElement != null)
            {
                tableCellType = this.TableCellPropertiesElement.TableCellType;
                this.ReleaseElement(this.tableCellPropertiesChildElement);
            }

            this.TableCellData = new TableCellData(tableCellType, this.BlockContainer);
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            base.OnBeforeReadChildElement(context, childElement);

            switch (childElement.ElementName)
            {
                case DocxElementNames.TableCellProperties:
                    this.TableCellPropertiesElement.SetAssociatedFlowModelElement(this.BlockContainer.Properties);
                    break;
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            DocxHelper.AddHangingAnnotation(context, childElement);
        }
    }
}