﻿using Telerik.Windows.Documents.Flow.FormatProviders.Common.Tables;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TableCellPropertiesElements;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class TableCellPropertiesElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<GridSpanElement> gridSpanChildElement;
        private readonly OpenXmlChildElement<VerticalMergeElement> verticalMergeChildElement;
        private readonly OpenXmlChildElement<HorizontalMergeElement> horizontalMergeChildElement;
        private readonly OpenXmlChildElement<ShadingElement> shadingChildElement;
        private readonly OpenXmlChildElement<TableCellBordersElement> bordersChildElement;
        private readonly OpenXmlChildElement<TableCellPaddingElement> paddingElement;
        private readonly OpenXmlChildElement<IgnoreCellMarkerInRowHeightCalculationElement> ignoreCellMarkerInRowHeightCalculationElement;
        private readonly OpenXmlChildElement<CanWrapContentElement> canWrapContentElement;
        private readonly OpenXmlChildElement<TableWidthElement> preferredWidthElement;
        private readonly OpenXmlChildElement<VerticalAlignmentElement> verticalAlignmentElement;
        private readonly OpenXmlChildElement<TextDirectionElement> textDirectionElement;

        private TableCellProperties tableCellProperties;

        public TableCellPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.preferredWidthElement = this.RegisterChildElement<TableWidthElement>(DocxElementNames.TableCellPreferredWidth);
            this.gridSpanChildElement = this.RegisterChildElement<GridSpanElement>(DocxElementNames.GridSpan);
            this.verticalMergeChildElement = this.RegisterChildElement<VerticalMergeElement>(DocxElementNames.VerticalMerge);
            this.shadingChildElement = this.RegisterChildElement<ShadingElement>(DocxElementNames.Shading);
            this.bordersChildElement = this.RegisterChildElement<TableCellBordersElement>(DocxElementNames.TableCellBorders);
            this.canWrapContentElement = this.RegisterChildElement<CanWrapContentElement>(DocxElementNames.CanWrapContent);
            this.paddingElement = this.RegisterChildElement<TableCellPaddingElement>(DocxElementNames.TableCellPadding);
            this.textDirectionElement = this.RegisterChildElement<TextDirectionElement>(DocxElementNames.TextDirection);
            this.horizontalMergeChildElement = this.RegisterChildElement<HorizontalMergeElement>(DocxElementNames.HorizontalMerge);
            this.verticalAlignmentElement = this.RegisterChildElement<VerticalAlignmentElement>(DocxElementNames.VerticalAlignment);
            this.ignoreCellMarkerInRowHeightCalculationElement =
                this.RegisterChildElement<IgnoreCellMarkerInRowHeightCalculationElement>(DocxElementNames.IgnoreCellMarkerInRowHeightCalculation);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableCellProperties;
            }
        }

        public TableCellType TableCellType { get; set; }

        public TableCellBordersElement TableCellBordersElement
        {
            get
            {
                return this.bordersChildElement.Element;
            }
        }

        public TableCellPaddingElement TableCellPaddingElement
        {
            get
            {
                return this.paddingElement.Element;
            }
        }

        public TableCellProperties TableCellProperties
        {
            get
            {
                return this.tableCellProperties;
            }
        }

        private GridSpanElement GridSpanElement
        {
            get
            {
                return this.gridSpanChildElement.Element;
            }
        }

        private VerticalMergeElement VerticalMergeElement
        {
            get
            {
                return this.verticalMergeChildElement.Element;
            }
        }

        private HorizontalMergeElement HorizontalMergeElement
        {
            get
            {
                return this.horizontalMergeChildElement.Element;
            }
        }

        public void SetAssociatedFlowModelElement(TableCellProperties tableCellProperties)
        {
            Guard.ThrowExceptionIfNull(tableCellProperties, "tableCellProperties");

            this.tableCellProperties = tableCellProperties;
            this.TableCellType = TableCellType.Cell;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.GridSpanElement != null)
            {
                this.tableCellProperties.ColumnSpan.LocalValue = this.GridSpanElement.Value;
                this.ReleaseElement(this.gridSpanChildElement);
            }

            if (this.VerticalMergeElement != null)
            {
                if (this.VerticalMergeElement.TableCellMergeType == TableCellMergeType.Continue)
                {
                    this.TableCellType = TableCellType.VerticallyMerged;
                }

                this.ReleaseElement(this.verticalMergeChildElement);
            }

            if (this.HorizontalMergeElement != null)
            {
                if (this.HorizontalMergeElement.TableCellMergeType == TableCellMergeType.Continue)
                {
                    this.TableCellType = TableCellType.HorizontalyMerged;
                }

                this.ReleaseElement(this.horizontalMergeChildElement);
            }

            if (this.shadingChildElement.Element != null)
            {
                this.shadingChildElement.Element.ReadAttributes(this.tableCellProperties);
                this.ReleaseElement(this.shadingChildElement);
            }

            if (this.TableCellBordersElement != null)
            {
                this.ReleaseElement(this.bordersChildElement);
            }

            if (this.TableCellPaddingElement != null)
            {
                this.ReleaseElement(this.paddingElement);
            }

            if (this.ignoreCellMarkerInRowHeightCalculationElement.Element != null)
            {
                this.tableCellProperties.IgnoreCellMarkerInRowHeightCalculation.LocalValue = this.ignoreCellMarkerInRowHeightCalculationElement.Element.Value;
                this.ReleaseElement(this.ignoreCellMarkerInRowHeightCalculationElement);
            }

            if (this.canWrapContentElement.Element != null)
            {
                this.tableCellProperties.CanWrapContent.LocalValue = !this.canWrapContentElement.Element.Value;
                this.ReleaseElement(this.canWrapContentElement);
            }

            if (this.preferredWidthElement.Element != null)
            {
                this.tableCellProperties.PreferredWidth.LocalValue = this.preferredWidthElement.Element.Value;
                this.ReleaseElement(this.preferredWidthElement);
            }

            if (this.verticalAlignmentElement.Element != null)
            {
                this.tableCellProperties.VerticalAlignment.LocalValue = this.verticalAlignmentElement.Element.Value;
                this.ReleaseElement(this.verticalAlignmentElement);
            }

            if (this.textDirectionElement.Element != null)
            {
                this.tableCellProperties.TextDirection.LocalValue = this.textDirectionElement.Element.Value;
                this.ReleaseElement(this.textDirectionElement);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.tableCellProperties.ColumnSpan.LocalValue > DocumentDefaultStyleSettings.ColumnSpan)
            {
                this.CreateElement(this.gridSpanChildElement);
                this.GridSpanElement.Value = this.tableCellProperties.ColumnSpan.LocalValue.Value;
            }

            if (this.TableCellType == TableCellType.VerticallyMerged)
            {
                this.CreateElement(this.verticalMergeChildElement);
                this.VerticalMergeElement.TableCellMergeType = TableCellMergeType.Continue;
            }
            else
            {
                Flow.Model.Styles.Core.IStyleProperty<int?> rowSpanProperty = this.tableCellProperties.RowSpan;
                if (rowSpanProperty.LocalValue.Value > DocumentDefaultStyleSettings.RowSpan)
                {
                    this.CreateElement(this.verticalMergeChildElement);
                    this.VerticalMergeElement.TableCellMergeType = TableCellMergeType.Restart;
                }
            }

            if (this.tableCellProperties.ShadingPattern.HasLocalValue ||
                this.tableCellProperties.ShadingPatternColor.HasLocalValue ||
                this.tableCellProperties.BackgroundColor.HasLocalValue)
            {
                this.CreateElement(this.shadingChildElement);
                this.shadingChildElement.Element.FillAttributes(this.tableCellProperties);
            }

            if (this.tableCellProperties.Borders.HasLocalValue)
            {
                this.CreateElement(this.bordersChildElement);
                this.TableCellBordersElement.SetAssociatedFlowModelElement(this.tableCellProperties);
            }

            if (this.tableCellProperties.Padding.HasLocalValue)
            {
                this.CreateElement(this.paddingElement);
                this.TableCellPaddingElement.SetAssociatedFlowModelElement(this.tableCellProperties);
            }

            if (!this.tableCellProperties.IgnoreCellMarkerInRowHeightCalculation.LocalValue.Value.Equals(DocumentDefaultStyleSettings.IgnoreCellMarkerInRowHeightCalculation))
            {
                this.CreateElement(this.ignoreCellMarkerInRowHeightCalculationElement);
                this.ignoreCellMarkerInRowHeightCalculationElement.Element.Value = this.tableCellProperties.IgnoreCellMarkerInRowHeightCalculation.LocalValue.Value;
            }

            if (!this.tableCellProperties.CanWrapContent.LocalValue.Value.Equals(DocumentDefaultStyleSettings.CanWrapContent))
            {
                this.CreateElement(this.canWrapContentElement);
                this.canWrapContentElement.Element.Value = !this.tableCellProperties.CanWrapContent.LocalValue.Value;
            }

            if (!this.tableCellProperties.PreferredWidth.LocalValue.Equals(DocumentDefaultStyleSettings.TableCellPreferredWidth))
            {
                this.CreateElement(this.preferredWidthElement);
                TableWidthUnit localValue = this.tableCellProperties.PreferredWidth.LocalValue;
                this.preferredWidthElement.Element.Value = new TableWidthUnit(localValue.Type, localValue.Value);
            }

            if (!this.tableCellProperties.VerticalAlignment.LocalValue.Value.Equals(DocumentDefaultStyleSettings.TableCellVerticalAlignment))
            {
                this.CreateElement(this.verticalAlignmentElement);
                this.verticalAlignmentElement.Element.Value = this.tableCellProperties.VerticalAlignment.LocalValue.Value;
            }

            if (!this.tableCellProperties.TextDirection.LocalValue.Equals(DocumentDefaultStyleSettings.TableCellTextDirection))
            {
                this.CreateElement(this.textDirectionElement);
                this.textDirectionElement.Element.Value = this.tableCellProperties.TextDirection.LocalValue;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return this.tableCellProperties != null || base.ShouldExport(context);
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            switch (childElement.ElementName)
            {
                case DocxElementNames.TableCellBorders:
                    this.TableCellBordersElement.SetAssociatedFlowModelElement(this.tableCellProperties);
                    break;
                case DocxElementNames.TableCellPadding:
                    this.TableCellPaddingElement.SetAssociatedFlowModelElement(this.tableCellProperties);
                    break;
            }
        }

        protected override void ClearOverride()
        {
            base.ClearOverride();

            this.tableCellProperties = null;
        }
    }
}
