﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TablePropertiesElements;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class TablePropertiesElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<AlignmentElement> alignmentElement;
        private readonly OpenXmlChildElement<ShadingElement> shadingChildElement;
        private readonly OpenXmlChildElement<TableBordersElement> bordersElement;
        private readonly OpenXmlChildElement<TableWidthElement> tableCellSpacingElement;
        private readonly OpenXmlChildElement<TableWidthElement> tableIndentElement;
        private readonly OpenXmlChildElement<RowBandingElement> rowBandingElement;
        private readonly OpenXmlChildElement<ColumnBandingElement> columnBandingElement;
        private readonly OpenXmlChildElement<TableCellPaddingElement> tableCellPaddingElement;
        private readonly OpenXmlChildElement<TableWidthElement> preferredWidthElement;
        private readonly OpenXmlChildElement<FlowDirectionElement> flowDirectionElement;
        private readonly OpenXmlChildElement<TableLayoutTypeElement> layoutTypeElement;
        private readonly OpenXmlChildElement<TableLooksElement> looksElement;
        private readonly OpenXmlChildElement<OverlapElement> overlapElement;
        private readonly OpenXmlChildElement<StyleIdElement> styleIdElement;

        private TableProperties tableProperties;

        public TablePropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.styleIdElement = this.RegisterChildElement<StyleIdElement>(DocxElementNames.TableStyleId);
            this.overlapElement = this.RegisterChildElement<OverlapElement>(DocxElementNames.Overlap);
            this.flowDirectionElement = this.RegisterChildElement<FlowDirectionElement>(DocxElementNames.TableFlowDirection);
            this.rowBandingElement = this.RegisterChildElement<RowBandingElement>(DocxElementNames.RowBanding);
            this.columnBandingElement = this.RegisterChildElement<ColumnBandingElement>(DocxElementNames.ColumnBanding);
            this.preferredWidthElement = this.RegisterChildElement<TableWidthElement>(DocxElementNames.TablePreferredWidth);
            this.alignmentElement = this.RegisterChildElement<AlignmentElement>(DocxElementNames.Alignment);
            this.tableCellSpacingElement = this.RegisterChildElement<TableWidthElement>(DocxElementNames.TableTableCellSpacing);
            this.tableIndentElement = this.RegisterChildElement<TableWidthElement>(DocxElementNames.TableIndent);
            this.bordersElement = this.RegisterChildElement<TableBordersElement>(DocxElementNames.TableBorders);
            this.shadingChildElement = this.RegisterChildElement<ShadingElement>(DocxElementNames.Shading);
            this.layoutTypeElement = this.RegisterChildElement<TableLayoutTypeElement>(DocxElementNames.TableLayoutType);
            this.tableCellPaddingElement = this.RegisterChildElement<TableCellPaddingElement>(DocxElementNames.TableTableCellPadding);
            this.looksElement = this.RegisterChildElement<TableLooksElement>(DocxElementNames.TableLooks);
        }

        /// <summary>
        /// Gets the name of the element.
        /// </summary>
        /// <value>The name of the element.</value>
        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableProperties;
            }
        }

        public TableBordersElement TableBordersElement
        {
            get
            {
                return this.bordersElement.Element;
            }
        }

        public TableCellPaddingElement TableCellPaddingElement
        {
            get
            {
                return this.tableCellPaddingElement.Element;
            }
        }

        public TableProperties TableProperties
        {
            get
            {
                return this.tableProperties;
            }
        }

        public void SetAssociatedFlowModelElement(TableProperties tableProperties)
        {
            Guard.ThrowExceptionIfNull(tableProperties, "tableProperties");

            this.tableProperties = tableProperties;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return this.tableProperties != null || base.ShouldExport(context);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (this.alignmentElement.Element != null)
            {
                this.tableProperties.Alignment.LocalValue = this.alignmentElement.Element.Value;
                this.ReleaseElement(this.alignmentElement);
            }

            if (this.shadingChildElement.Element != null)
            {
                this.shadingChildElement.Element.ReadAttributes(this.tableProperties);
                this.ReleaseElement(this.shadingChildElement);
            }

            if (this.TableBordersElement != null)
            {
                this.ReleaseElement(this.bordersElement);
            }

            if (this.tableCellSpacingElement.Element != null)
            {
                this.tableProperties.TableCellSpacing.LocalValue = this.tableCellSpacingElement.Element.ToDouble();
                this.ReleaseElement(this.tableCellSpacingElement);
            }

            if (this.tableIndentElement.Element != null)
            {
                this.tableProperties.Indent.LocalValue = this.tableIndentElement.Element.ToDouble();
                this.ReleaseElement(this.tableIndentElement);
            }

            if (this.columnBandingElement.Element != null)
            {
                this.tableProperties.ColumnBanding.LocalValue = this.columnBandingElement.Element.Value;
                this.ReleaseElement(this.columnBandingElement);
            }

            if (this.rowBandingElement.Element != null)
            {
                this.tableProperties.RowBanding.LocalValue = this.rowBandingElement.Element.Value;
                this.ReleaseElement(this.rowBandingElement);
            }

            if (this.TableCellPaddingElement != null)
            {
                this.ReleaseElement(this.tableCellPaddingElement);
            }

            if (this.preferredWidthElement.Element != null)
            {
                this.tableProperties.PreferredWidth.LocalValue = this.preferredWidthElement.Element.Value;
                this.ReleaseElement(this.preferredWidthElement);
            }

            if (this.flowDirectionElement.Element != null)
            {
                this.tableProperties.FlowDirection.LocalValue = this.flowDirectionElement.Element.Value ? FlowDirection.RightToLeft : FlowDirection.LeftToRight;
                this.ReleaseElement(this.flowDirectionElement);
            }

            if (this.layoutTypeElement.Element != null)
            {
                this.layoutTypeElement.Element.ReadAttributes(this.tableProperties);
                this.ReleaseElement(this.layoutTypeElement);
            }

            if (this.looksElement.Element != null)
            {
                this.looksElement.Element.ReadAttributes(this.tableProperties);
                this.ReleaseElement(this.looksElement);
            }

            if (this.overlapElement.Element != null)
            {
                this.tableProperties.Overlap.LocalValue = this.overlapElement.Element.Value;
                this.ReleaseElement(this.overlapElement);
            }

            if (this.styleIdElement.Element != null)
            {
                this.TableProperties.StyleId = this.styleIdElement.Element.Value;
                this.ReleaseElement(this.styleIdElement);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (this.tableProperties.Alignment.HasLocalValue)
            {
                this.CreateElement(this.alignmentElement);
                this.alignmentElement.Element.Value = this.tableProperties.Alignment.LocalValue.Value;
            }

            if (this.tableProperties.ShadingPattern.HasLocalValue ||
                this.tableProperties.ShadingPatternColor.HasLocalValue ||
                this.tableProperties.BackgroundColor.HasLocalValue)
            {
                this.CreateElement(this.shadingChildElement);
                this.shadingChildElement.Element.FillAttributes(this.tableProperties);
            }

            if (this.tableProperties.Borders.HasLocalValue)
            {
                this.CreateElement(this.bordersElement);
                this.TableBordersElement.SetAssociatedFlowModelElement(this.tableProperties);
            }

            if (this.tableProperties.TableCellSpacing.HasLocalValue)
            {
                this.CreateElement(this.tableCellSpacingElement);
                this.tableCellSpacingElement.Element.Value = new TableWidthUnit(TableWidthUnitType.Fixed, this.tableProperties.TableCellSpacing.LocalValue.Value);
            }

            if (this.tableProperties.Indent.HasLocalValue)
            {
                this.CreateElement(this.tableIndentElement);
                this.tableIndentElement.Element.Value = new TableWidthUnit(TableWidthUnitType.Fixed, this.tableProperties.Indent.LocalValue.Value);
            }

            if (this.tableProperties.ColumnBanding.HasLocalValue)
            {
                this.CreateElement(this.columnBandingElement);
                this.columnBandingElement.Element.Value = this.tableProperties.ColumnBanding.LocalValue.Value;
            }

            if (this.tableProperties.RowBanding.HasLocalValue)
            {
                this.CreateElement(this.rowBandingElement);
                this.rowBandingElement.Element.Value = this.tableProperties.RowBanding.LocalValue.Value;
            }

            if (this.tableProperties.TableCellPadding.HasLocalValue)
            {
                this.CreateElement(this.tableCellPaddingElement);
                this.TableCellPaddingElement.SetAssociatedFlowModelElement(this.tableProperties);
            }

            if (!this.tableProperties.PreferredWidth.LocalValue.Equals(DocumentDefaultStyleSettings.TablePreferredWidth))
            {
                this.CreateElement(this.preferredWidthElement);
                TableWidthUnit localValue = this.tableProperties.PreferredWidth.LocalValue;
                this.preferredWidthElement.Element.Value = new TableWidthUnit(localValue.Type, localValue.Value);
            }

            if (!this.tableProperties.FlowDirection.LocalValue.Value.Equals(DocumentDefaultStyleSettings.TableFlowDirection))
            {
                this.CreateElement(this.flowDirectionElement);
                FlowDirection flowDirectionValue = this.tableProperties.FlowDirection.LocalValue.Value;
                this.flowDirectionElement.Element.Value = flowDirectionValue == FlowDirection.RightToLeft ? true : false;
            }

            if (!this.tableProperties.LayoutType.LocalValue.Equals(DocumentDefaultStyleSettings.TableLayoutType))
            {
                this.CreateElement(this.layoutTypeElement);
                this.layoutTypeElement.Element.FillAttributes(this.tableProperties);
            }

            if (!this.tableProperties.Looks.LocalValue.Equals(DocumentDefaultStyleSettings.TableLooks))
            {
                this.CreateElement(this.looksElement);
                this.looksElement.Element.FillAttributes(this.tableProperties);
            }

            if (!this.tableProperties.Overlap.LocalValue.Value.Equals(DocumentDefaultStyleSettings.TableOverlap))
            {
                this.CreateElement(this.overlapElement);
                this.overlapElement.Element.Value = this.tableProperties.Overlap.LocalValue.Value;
            }

            if (!string.IsNullOrEmpty(this.TableProperties.StyleId))
            {
                this.CreateElement(this.styleIdElement);
                this.styleIdElement.Element.Value = this.TableProperties.StyleId;
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase element)
        {
            switch (element.ElementName)
            {
                case DocxElementNames.TableBorders:
                    this.TableBordersElement.SetAssociatedFlowModelElement(this.tableProperties);
                    break;
                case DocxElementNames.TableTableCellPadding:
                    this.TableCellPaddingElement.SetAssociatedFlowModelElement(this.tableProperties);
                    break;
            }
        }
    }
}
