﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Common.Tables;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class TableRowElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<TableRowPropertiesElement> tableRowPropertiesChildElement;

        private TableRow tableRow;

        public TableRowElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.tableRowPropertiesChildElement = this.RegisterChildElement<TableRowPropertiesElement>(DocxElementNames.TableRowProperties);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableRow;
            }
        }

        public List<TableCellData> TableCellDatas { get; private set; }

        public TableRow TableRow
        {
            get
            {
                return this.tableRow;
            }
        }

        public TableRowPropertiesElement TableRowPropertiesElement
        {
            get
            {
                return this.tableRowPropertiesChildElement.Element;
            }
        }

        public void SetAssociatedFlowModelElement(TableRow tableRow, List<TableCellData> tableCellDatas)
        {
            Guard.ThrowExceptionIfNull(tableRow, "tableRow");
            Guard.ThrowExceptionIfNull(tableCellDatas, "tableCellDatas");

            this.tableRow = tableRow;
            this.TableCellDatas = tableCellDatas;
        }

        protected override void ClearOverride()
        {
            this.TableCellDatas = null;
            this.tableRow = null;

            base.ClearOverride();
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return this.tableRow.Cells.Any() || base.ShouldExport(context);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (this.TableRowPropertiesElement != null)
            {
                this.ReleaseElement(this.tableRowPropertiesChildElement);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (this.tableRow.Properties.HasLocalValues())
            {
                this.CreateElement(this.tableRowPropertiesChildElement);
                this.TableRowPropertiesElement.SetAssociatedFlowModelElement(this.tableRow.Properties);
            }
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (TableCellData tableCellData in this.TableCellDatas)
            {
                if (tableCellData.Type == TableCellType.HorizontalyMerged)
                {
                    continue;
                }

                TableCellElement tableCellElement = this.CreateElement<TableCellElement>(DocxElementNames.TableCell);
                tableCellElement.SetAssociatedFlowModelElement(tableCellData.Cell);
                tableCellElement.TableCellData = tableCellData;

                yield return tableCellElement;
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            switch (childElement.ElementName)
            {
                case DocxElementNames.TableCell:
                    TableCellElement tableCellElement = (TableCellElement)childElement;
                    tableCellElement.SetAssociatedFlowModelElement(new TableCell(this.tableRow.Document));
                    break;
                case DocxElementNames.TableRowProperties:
                    this.TableRowPropertiesElement.SetAssociatedFlowModelElement(this.tableRow.Properties);
                    break;
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            DocxHelper.AddHangingAnnotation(context, childElement);

            switch (childElement.ElementName)
            {
                case DocxElementNames.TableCell:
                    TableCellElement tableCellElement = (TableCellElement)childElement;
                    this.TableCellDatas.Add(tableCellElement.TableCellData);
                    break;
            }
        }
    }
}