﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TableRowPropertiesElements;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class TableRowPropertiesElement : DocumentElementBase
    {
        private readonly OpenXmlChildElement<TableWidthElement> tableCellSpacingElement;
        private readonly OpenXmlChildElement<RepeatOnEveryPageElement> repeatOnEveryPageElement;
        private readonly OpenXmlChildElement<CanSplitElement> canSplitElement;
        private readonly OpenXmlChildElement<TableRowHeightElement> tableRowHeightElement;

        private TableRowProperties tableRowProperties;

        public TableRowPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.tableCellSpacingElement = this.RegisterChildElement<TableWidthElement>(DocxElementNames.TableTableCellSpacing);
            this.repeatOnEveryPageElement = this.RegisterChildElement<RepeatOnEveryPageElement>(DocxElementNames.RepeatOnEveryPage);
            this.repeatOnEveryPageElement = this.RegisterChildElement<RepeatOnEveryPageElement>(DocxElementNames.RepeatOnEveryPage);
            this.canSplitElement = this.RegisterChildElement<CanSplitElement>(DocxElementNames.CanSplit);
            this.tableRowHeightElement = this.RegisterChildElement<TableRowHeightElement>(DocxElementNames.TableRowHeight);
        }

        /// <summary>
        /// Gets the name of the element.
        /// </summary>
        /// <value>The name of the element.</value>
        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableRowProperties;
            }
        }

        public TableRowProperties TableRowProperties
        {
            get
            {
                return this.tableRowProperties;
            }
        }

        public void SetAssociatedFlowModelElement(TableRowProperties tableRowProperties)
        {
            Guard.ThrowExceptionIfNull(tableRowProperties, "tableRowProperties");

            this.tableRowProperties = tableRowProperties;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return this.tableRowProperties != null || base.ShouldExport(context);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (this.tableCellSpacingElement.Element != null)
            {
                this.tableRowProperties.TableCellSpacing.LocalValue = this.tableCellSpacingElement.Element.ToDouble();
                this.ReleaseElement(this.tableCellSpacingElement);
            }

            if (this.repeatOnEveryPageElement.Element != null)
            {
                this.tableRowProperties.RepeatOnEveryPage.LocalValue = this.repeatOnEveryPageElement.Element.Value;
                this.ReleaseElement(this.repeatOnEveryPageElement);
            }

            if (this.canSplitElement.Element != null)
            {
                this.tableRowProperties.CanSplit.LocalValue = !this.canSplitElement.Element.Value;
                this.ReleaseElement(this.canSplitElement);
            }

            if (this.tableRowHeightElement.Element != null)
            {
                this.tableRowProperties.Height.LocalValue = this.tableRowHeightElement.Element.Value;
                this.ReleaseElement(this.tableRowHeightElement);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (this.tableRowProperties.TableCellSpacing.HasLocalValue)
            {
                this.CreateElement(this.tableCellSpacingElement);
                this.tableCellSpacingElement.Element.Value = new TableWidthUnit(TableWidthUnitType.Fixed, this.tableRowProperties.TableCellSpacing.LocalValue.Value);
            }

            if (!this.tableRowProperties.RepeatOnEveryPage.LocalValue.Value.Equals(DocumentDefaultStyleSettings.TableRowRepeatOnEveryPage))
            {
                this.CreateElement(this.repeatOnEveryPageElement);
                this.repeatOnEveryPageElement.Element.Value = this.tableRowProperties.RepeatOnEveryPage.LocalValue.Value;
            }

            if (!this.tableRowProperties.CanSplit.LocalValue.Value.Equals(DocumentDefaultStyleSettings.TableRowCanSplit))
            {
                this.CreateElement(this.canSplitElement);
                this.canSplitElement.Element.Value = !this.tableRowProperties.CanSplit.LocalValue.Value;
            }

            if (!this.tableRowProperties.Height.LocalValue.Equals(DocumentDefaultStyleSettings.TableRowHeight))
            {
                this.CreateElement(this.tableRowHeightElement);
                this.tableRowHeightElement.Element.Value = this.tableRowProperties.Height.LocalValue;
            }
        }
    }
}
