﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Vector;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Model.Watermarks;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document
{
    internal class VmlContainerElement : DocxElementBase
    {
        private Watermark watermark;
        private ImageInline image;

        public VmlContainerElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.VmlContainer;
            }
        }

        public Watermark Watermark
        {
            get
            {
                return this.watermark;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");
                this.watermark = value;
            }
        }

        public ImageInline Image
        {
            get
            {
                return this.image;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");
                this.image = value;
            }
        }

        protected override bool ShouldImport(IDocxImportContext context)
        {
            return true;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return this.EnumerateChildElements(context).Any();
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            if (this.watermark != null)
            {
                ShapeElement shapeElement = this.CreateElement<ShapeElement>(DocxElementNames.Shape);
                shapeElement.SetAssociatedWatermarkElement(this.watermark);
                yield return shapeElement;
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            if (childElement.ElementName == DocxElementNames.Shape)
            {
                ShapeElement shapeElement = (ShapeElement)childElement;
                if (shapeElement != null)
                {
                    switch (shapeElement.ShapeType)
                    {
                        case ShapeTypes.Watermark:
                            this.watermark = shapeElement.CreateWatermark();
                            break;
                        case ShapeTypes.Image:
                            this.image = new ImageInline(context.Document, shapeElement.CreateImage());
                            break;
                        case ShapeTypes.Unknown:
                        default:
                            break;
                    }
                }
            }
        }

        protected override void ClearOverride()
        {
            this.watermark = null;
            this.image = null;
        }
    }
}
