﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal class AnchorElement : DrawingElementBase
    {
        private readonly OpenXmlChildElement<OffsetElement> simplePos;
        private readonly OpenXmlChildElement<HorizontalPositionElement> positionH;
        private readonly OpenXmlChildElement<VerticalPositionElement> positionV;
        private readonly OpenXmlChildElement<SizeElement> extent;
        private readonly OpenXmlChildElement<WrapNoneElement> wrapNone;
        private readonly OpenXmlChildElement<WrapSquareElement> wrapSquare;
        private readonly OpenXmlChildElement<WrapTopAndBottomElement> wrapTopAndBottom;
        private readonly OpenXmlChildElement<NonVisualDrawingPropertiesElement> drawingObjectNonVisualProperties;
        private readonly OpenXmlChildElement<GraphicElement> graphics;
        private readonly OpenXmlChildElement<EffectExtentElement> effectExtent;

        private readonly BoolOpenXmlAttribute allowOverlap;
        private readonly BoolOpenXmlAttribute isBehindDoc;
        private readonly BoolOpenXmlAttribute isSimplePos;
        private readonly BoolOpenXmlAttribute isLocked;
        private readonly BoolOpenXmlAttribute layoutInCell;
        private readonly IntOpenXmlAttribute zindex;
        private readonly ConvertedOpenXmlAttribute<double> leftMargin;
        private readonly ConvertedOpenXmlAttribute<double> rightMargin;
        private readonly ConvertedOpenXmlAttribute<double> bottomMargin;
        private readonly ConvertedOpenXmlAttribute<double> topMargin;

        public AnchorElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.topMargin = new ConvertedOpenXmlAttribute<double>("distT", Converters.EmuOrUniversalMeasureToDipConverter);
            this.RegisterAttribute(this.topMargin);
            this.bottomMargin = new ConvertedOpenXmlAttribute<double>("distB", Converters.EmuOrUniversalMeasureToDipConverter);
            this.RegisterAttribute(this.bottomMargin);
            this.leftMargin = new ConvertedOpenXmlAttribute<double>("distL", Converters.EmuOrUniversalMeasureToDipConverter);
            this.RegisterAttribute(this.leftMargin);
            this.rightMargin = new ConvertedOpenXmlAttribute<double>("distR", Converters.EmuOrUniversalMeasureToDipConverter);
            this.RegisterAttribute(this.rightMargin);
            this.isSimplePos = this.RegisterAttribute(new BoolOpenXmlAttribute("simplePos"));
            this.zindex = this.RegisterAttribute(new IntOpenXmlAttribute("relativeHeight", true));
            this.isBehindDoc = this.RegisterAttribute(new BoolOpenXmlAttribute("behindDoc", false, true));
            this.isLocked = this.RegisterAttribute(new BoolOpenXmlAttribute("locked", false, true));
            this.layoutInCell = this.RegisterAttribute(new BoolOpenXmlAttribute("layoutInCell", false, true));
            this.allowOverlap = this.RegisterAttribute(new BoolOpenXmlAttribute("allowOverlap", false, true));

            this.simplePos = this.RegisterChildElement<OffsetElement>(DocxElementNames.SimplePositioning);
            this.positionH = this.RegisterChildElement<HorizontalPositionElement>(DocxElementNames.HorizontalPosition);
            this.positionV = this.RegisterChildElement<VerticalPositionElement>(DocxElementNames.VerticalPosition);
            this.extent = this.RegisterChildElement<SizeElement>(DocxElementNames.Extent);
            this.effectExtent = this.RegisterChildElement<EffectExtentElement>(DocxElementNames.EffectExtent);
            this.wrapNone = this.RegisterChildElement<WrapNoneElement>(DocxElementNames.WrapNone);
            this.wrapSquare = this.RegisterChildElement<WrapSquareElement>(DocxElementNames.WrapSquare);
            this.wrapTopAndBottom = this.RegisterChildElement<WrapTopAndBottomElement>(DocxElementNames.WrapTopAndBottom);
            this.drawingObjectNonVisualProperties = this.RegisterChildElement<NonVisualDrawingPropertiesElement>(DocxElementNames.DrawingObjectNonVisualProperties);
            this.graphics = this.RegisterChildElement<GraphicElement>(OpenXmlElementNames.GraphicElementName);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Anchor;
            }
        }

        public bool AllowOverlap
        {
            get
            {
                return this.allowOverlap.Value;
            }
            set
            {
                this.allowOverlap.Value = value;
            }
        }

        public bool IsBehindDoc
        {
            get
            {
                return this.isBehindDoc.Value;
            }
            set
            {
                this.isBehindDoc.Value = value;
            }
        }

        public bool IsSimplePos
        {
            get
            {
                return this.isSimplePos.Value;
            }
            set
            {
                this.isSimplePos.Value = value;
            }
        }

        public bool IsLocked
        {
            get
            {
                return this.isLocked.Value;
            }
            set
            {
                this.isLocked.Value = value;
            }
        }

        public bool LayoutInCell
        {
            get
            {
                return this.layoutInCell.Value;
            }
            set
            {
                this.layoutInCell.Value = value;
            }
        }

        public int ZIndex
        {
            get
            {
                return this.zindex.Value;
            }
            set
            {
                this.zindex.Value = value;
            }
        }

        public double LeftMargin
        {
            get
            {
                return this.leftMargin.Value;
            }
            set
            {
                this.leftMargin.Value = value;
            }
        }

        public double RightMargin
        {
            get
            {
                return this.rightMargin.Value;
            }
            set
            {
                this.rightMargin.Value = value;
            }
        }

        public double TopMargin
        {
            get
            {
                return this.topMargin.Value;
            }
            set
            {
                this.topMargin.Value = value;
            }
        }

        public double BottomMargin
        {
            get
            {
                return this.bottomMargin.Value;
            }
            set
            {
                this.bottomMargin.Value = value;
            }
        }

        public HorizontalPositionElement HorizontalPositionElement
        {
            get
            {
                return this.positionH.Element;
            }
        }

        public VerticalPositionElement VerticalPositionElement
        {
            get
            {
                return this.positionV.Element;
            }
        }

        public NonVisualDrawingPropertiesElement DrawingObjectNonVisualProperties
        {
            get
            {
                return this.drawingObjectNonVisualProperties.Element;
            }
        }

        public GraphicElement GraphicElement
        {
            get
            {
                return this.graphics.Element;
            }
        }

        public SizeElement ExtentElement
        {
            get
            {
                return this.extent.Element;
            }
        }

        public WrapNoneElement WrapNoneElement
        {
            get
            {
                return this.wrapNone.Element;
            }
        }

        public WrapSquareElement WrapSquareElement
        {
            get
            {
                return this.wrapSquare.Element;
            }
        }

        public WrapTopAndBottomElement WrapTopAndBottomElement
        {
            get
            {
                return this.wrapTopAndBottom.Element;
            }
        }

        public OffsetElement SimplePositionElement
        {
            get
            {
                return this.simplePos.Element;
            }
        }

        public EffectExtentElement EffectExtentElement
        {
            get
            {
                return this.effectExtent.Element;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeAnchorBase anchor)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(anchor, "anchor");

            this.AllowOverlap = anchor.AllowOverlap;
            this.IsBehindDoc = anchor.IsBehindDocument;
            this.TopMargin = anchor.Margin.Top;
            this.BottomMargin = anchor.Margin.Bottom;
            this.LeftMargin = anchor.Margin.Left;
            this.RightMargin = anchor.Margin.Right;
            this.LayoutInCell = anchor.LayoutInCell;
            this.ZIndex = anchor.ZIndex;
            this.IsLocked = anchor.IsLocked;
            this.IsSimplePos = false;

            this.CreateElement(this.simplePos);

            this.CreateElement(this.positionH);
            this.HorizontalPositionElement.CopyPropertiesFrom(context, anchor);

            this.CreateElement(this.positionV);
            this.VerticalPositionElement.CopyPropertiesFrom(context, anchor);

            this.CreateElement(this.drawingObjectNonVisualProperties);
            this.DrawingObjectNonVisualProperties.Id = anchor.Shape.Id;
            this.DrawingObjectNonVisualProperties.Name = anchor.Shape.Name;

            this.CreateElement(this.graphics);
            this.GraphicElement.CopyPropertiesFrom(context, anchor.Shape);

            switch (anchor.Wrapping.WrappingType)
            {
                case ShapeWrappingType.Square:
                    this.CreateElement(this.wrapSquare);
                    this.WrapSquareElement.CopyPropertiesFrom(context, anchor.Wrapping);
                    break;

                case ShapeWrappingType.TopAndBottom:
                    this.CreateElement(this.wrapTopAndBottom);
                    break;

                default:
                    this.CreateElement(this.wrapNone);
                    break;
            }

            this.CreateElement(this.extent);
            this.ExtentElement.ExtentWidth = anchor.Shape.Width;
            this.ExtentElement.ExtentHeight = anchor.Shape.Height;

            this.CreateElement(this.effectExtent);
            this.EffectExtentElement.CopyPropertiesFrom(context, anchor);
        }

        public ShapeAnchorBase CreateShapeInline(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ShapeBase shape = null;
            if (this.GraphicElement != null)
            {
                shape = this.GraphicElement.CreateShapeAndCopyPropertiesTo(context);
                this.ReleaseElement(this.graphics);
            }

            if (shape != null)
            {
                if (this.ExtentElement != null)
                {
                    shape.Width = this.ExtentElement.ExtentWidth;
                    shape.Height = this.ExtentElement.ExtentHeight;
                    this.ReleaseElement(this.extent);
                }

                if (this.DrawingObjectNonVisualProperties != null)
                {
                    shape.Name = this.DrawingObjectNonVisualProperties.Name;
                    shape.Id = this.DrawingObjectNonVisualProperties.Id;

                    this.ReleaseElement(this.drawingObjectNonVisualProperties);
                }
            }

            ShapeAnchorBase anchor = CreateShapeInlineFromShape(context, shape);
            if (anchor != null)
            {
                anchor.AllowOverlap = this.AllowOverlap;
                anchor.IsBehindDocument = this.IsBehindDoc;
                anchor.Margin = new Padding(this.LeftMargin, this.TopMargin, this.RightMargin, this.BottomMargin);
                anchor.LayoutInCell = this.LayoutInCell;
                anchor.ZIndex = this.ZIndex;
                anchor.IsLocked = this.IsLocked;

                if (this.HorizontalPositionElement != null)
                {
                    this.HorizontalPositionElement.CopyPropertiesТо(context, anchor);
                    this.ReleaseElement(this.positionH);
                }

                if (this.VerticalPositionElement != null)
                {
                    this.VerticalPositionElement.CopyPropertiesТо(context, anchor);
                    this.ReleaseElement(this.positionV);
                }

                if (this.WrapNoneElement != null)
                {
                    anchor.Wrapping.WrappingType = ShapeWrappingType.None;
                    this.ReleaseElement(this.wrapNone);
                }

                if (this.WrapSquareElement != null)
                {
                    anchor.Wrapping.WrappingType = ShapeWrappingType.Square;
                    this.WrapSquareElement.CopyPropertiesTo(context, anchor.Wrapping);
                    this.ReleaseElement(this.wrapSquare);
                }

                if (this.WrapTopAndBottomElement != null)
                {
                    anchor.Wrapping.WrappingType = ShapeWrappingType.TopAndBottom;
                    this.ReleaseElement(this.wrapTopAndBottom);
                }
            }

            if (this.EffectExtentElement != null)
            {
                this.ReleaseElement(this.effectExtent);
            }

            return anchor;
        }

        private static ShapeAnchorBase CreateShapeInlineFromShape(IDocxImportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            Image image = shape as Image;
            if (image != null)
            {
                return new FloatingImage(context.Document, image);
            }

            return null;
        }
    }
}
