﻿using System;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal class EffectExtentElement : DrawingElementBase
    {
        private readonly ConvertedOpenXmlAttribute<double> left;
        private readonly ConvertedOpenXmlAttribute<double> right;
        private readonly ConvertedOpenXmlAttribute<double> top;
        private readonly ConvertedOpenXmlAttribute<double> bottom;

        public EffectExtentElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.left = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("l", Converters.EmuToDipConverter));
            this.right = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("r", Converters.EmuToDipConverter));
            this.top = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("t", Converters.EmuToDipConverter));
            this.bottom = this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("b", Converters.EmuToDipConverter));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.EffectExtent;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public double Left
        {
            get
            {
                return this.left.Value;
            }
            set
            {
                this.left.Value = value;
            }
        }

        public double Right
        {
            get
            {
                return this.right.Value;
            }
            set
            {
                this.right.Value = value;
            }
        }

        public double Top
        {
            get
            {
                return this.top.Value;
            }
            set
            {
                this.top.Value = value;
            }
        }

        public double Bottom
        {
            get
            {
                return this.bottom.Value;
            }
            set
            {
                this.bottom.Value = value;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeAnchorBase anchor)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(anchor, "anchor");

            this.CopyPropertiesFrom(context, anchor.Shape);
            this.Left = 0;
            this.Right = 0;
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeInlineBase inline)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(inline, "inline");

            this.CopyPropertiesFrom(context, inline.Shape);

            this.Right -= this.Left;
            this.Left = 0;
        }

        private static double Min(params double[] args)
        {
            double min = args[0];
            for (int i = 1; i < args.Length; i++)
            {
                if (min > args[i])
                {
                    min = args[i];
                }
            }

            return min;
        }

        private static double Max(params double[] args)
        {
            double max = args[0];
            for (int i = 1; i < args.Length; i++)
            {
                if (max < args[i])
                {
                    max = args[i];
                }
            }

            return max;
        }

        private void CopyPropertiesFrom(IDocxExportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            Rect box = new Rect(0, 0, shape.Width, shape.Height);

            Matrix matrix = new Matrix();
            matrix.RotateAt(shape.RotationAngle, box.Left + (box.Width / 2), box.Top + (box.Height / 2));

            Point topLeft = matrix.Transform(box.TopLeft);
            Point topRight = matrix.Transform(box.TopRight);
            Point bottomLeft = matrix.Transform(box.BottomLeft);
            Point bottomRight = matrix.Transform(box.BottomRight);

            Rect effectBBox = new Rect(
                new Point(Min(topLeft.X, topRight.X, bottomLeft.X, bottomRight.X), Min(topLeft.Y, topRight.Y, bottomLeft.Y, bottomRight.Y)),
                new Point(Max(topLeft.X, topRight.X, bottomLeft.X, bottomRight.X), Max(topLeft.Y, topRight.Y, bottomLeft.Y, bottomRight.Y)));

            this.Left = Math.Abs(box.Left - effectBBox.Left);
            this.Right = Math.Abs(box.Right - effectBBox.Right);
            this.Top = Math.Abs(box.Top - effectBBox.Top);
            this.Bottom = Math.Abs(box.Bottom - effectBBox.Bottom);
        }
    }
}
