﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal class HorizontalPositionElement : DrawingElementBase
    {
        private readonly OpenXmlChildElement<HorizontalAlignmentElement> align;
        private readonly OpenXmlChildElement<OneDimentionOffsetElement> offset;

        private readonly MappedOpenXmlAttribute<HorizontalRelativeFrom> relativeFrom;

        public HorizontalPositionElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.align = this.RegisterChildElement<HorizontalAlignmentElement>(DocxElementNames.DrawingAlignment);
            this.offset = this.RegisterChildElement<OneDimentionOffsetElement>(DocxElementNames.Offset);

            this.relativeFrom = new MappedOpenXmlAttribute<HorizontalRelativeFrom>("relativeFrom", null, TypeMappers.HorizontalRelativeFromMapper, true);
            this.RegisterAttribute(this.relativeFrom);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.HorizontalPosition;
            }
        }

        public OneDimentionOffsetElement OffsetElement
        {
            get
            {
                return this.offset.Element;
            }
        }

        public HorizontalAlignmentElement HorizontalAlignmentElement
        {
            get
            {
                return this.align.Element;
            }
        }

        public HorizontalRelativeFrom RelativeFrom
        {
            get
            {
                return this.relativeFrom.Value;
            }
            set
            {
                this.relativeFrom.Value = value;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeAnchorBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            this.RelativeFrom = shape.HorizontalPosition.RelativeFrom;
            switch (shape.HorizontalPosition.ValueType)
            {
                case PositionValueType.Alignment:

                    this.CreateElement(this.align);
                    this.HorizontalAlignmentElement.InnerText = TypeMappers.RelativeHorizontalAlignmentMapper.GetFromValue(shape.HorizontalPosition.Alignment);

                    break;

                case PositionValueType.Offset:

                    this.CreateElement(this.offset);
                    this.OffsetElement.InnerText = Converters.EmuToDipConverter.ConvertToString(shape.HorizontalPosition.Offset);

                    break;
            }
        }

        public void CopyPropertiesТо(IDocxImportContext context, ShapeAnchorBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            shape.HorizontalPosition.RelativeFrom = this.RelativeFrom;
            if (this.HorizontalAlignmentElement != null)
            {
                RelativeHorizontalAlignment align = TypeMappers.RelativeHorizontalAlignmentMapper.GetToValue(this.HorizontalAlignmentElement.InnerText);
                shape.HorizontalPosition = new HorizontalPosition(this.RelativeFrom, align);
                this.ReleaseElement(this.align);
            }

            if (this.OffsetElement != null)
            {
                shape.HorizontalPosition = new HorizontalPosition(this.RelativeFrom, Converters.EmuToDipConverter.ConvertFromString(this.OffsetElement.InnerText));
                this.ReleaseElement(this.offset);
            }
        }

        protected override OpenXmlElementBase CreateElement(string elementName)
        {
            if (elementName == DocxElementNames.DrawingAlignment)
            {
                HorizontalAlignmentElement element = new HorizontalAlignmentElement(this.PartsManager);
                element.Part = this.Part;

                return element;
            }

            return base.CreateElement(elementName);
        }

        protected override void ReleaseElementOverride(OpenXmlElementBase element)
        {
            if (element.ElementName != DocxElementNames.DrawingAlignment)
            {
                base.ReleaseElementOverride(element);
            }
        }
    }
}
