﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal class InlineElement : DrawingElementBase
    {
        private readonly OpenXmlChildElement<NonVisualDrawingPropertiesElement> drawingObjectNonVisualProperties;
        private readonly OpenXmlChildElement<SizeElement> extent;
        private readonly OpenXmlChildElement<GraphicElement> graphics;
        private readonly OpenXmlChildElement<EffectExtentElement> effectExtent;

        public InlineElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.extent = this.RegisterChildElement<SizeElement>(DocxElementNames.Extent);
            this.effectExtent = this.RegisterChildElement<EffectExtentElement>(DocxElementNames.EffectExtent);
            this.drawingObjectNonVisualProperties = this.RegisterChildElement<NonVisualDrawingPropertiesElement>(DocxElementNames.DrawingObjectNonVisualProperties);
            this.graphics = this.RegisterChildElement<GraphicElement>(OpenXmlElementNames.GraphicElementName);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Inline;
            }
        }

        public NonVisualDrawingPropertiesElement DrawingObjectNonVisualProperties
        {
            get
            {
                return this.drawingObjectNonVisualProperties.Element;
            }
        }

        public GraphicElement GraphicElement
        {
            get
            {
                return this.graphics.Element;
            }
        }

        public SizeElement ExtentElement
        {
            get
            {
                return this.extent.Element;
            }
        }

        public EffectExtentElement EffectExtentElement
        {
            get
            {
                return this.effectExtent.Element;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeInlineBase shape)
        {
            this.CreateElement(this.drawingObjectNonVisualProperties);
            this.DrawingObjectNonVisualProperties.Id = shape.Shape.Id;
            this.DrawingObjectNonVisualProperties.Name = shape.Shape.Name;

            this.CreateElement(this.graphics);
            this.GraphicElement.CopyPropertiesFrom(context, shape.Shape);

            this.CreateElement(this.extent);
            this.ExtentElement.ExtentWidth = (int)shape.Shape.Width;
            this.ExtentElement.ExtentHeight = (int)shape.Shape.Height;

            this.CreateElement(this.effectExtent);
            this.EffectExtentElement.CopyPropertiesFrom(context, shape);
        }

        public ShapeInlineBase CreateShapeInline(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ShapeBase shape = null;
            if (this.GraphicElement != null)
            {
                shape = this.GraphicElement.CreateShapeAndCopyPropertiesTo(context);
                this.ReleaseElement(this.graphics);
            }

            if (this.ExtentElement != null)
            {
                if (shape != null)
                {
                    shape.Width = this.ExtentElement.ExtentWidth;
                    shape.Height = this.ExtentElement.ExtentHeight;
                }

                this.ReleaseElement(this.extent);
            }

            if (this.EffectExtentElement != null)
            {
                this.ReleaseElement(this.effectExtent);
            }

            return CreateShapeInlineFromShape(context, shape);
        }

        private static ShapeInlineBase CreateShapeInlineFromShape(IDocxImportContext context, ShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            Image image = shape as Image;
            if (image != null)
            {
                return new ImageInline(context.Document, image);
            }

            return null;
        }
    }
}
