﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal class VerticalPositionElement : DrawingElementBase
    {
        private readonly OpenXmlChildElement<VerticalAlignmentElement> align;
        private readonly OpenXmlChildElement<OneDimentionOffsetElement> offset;

        private readonly MappedOpenXmlAttribute<VerticalRelativeFrom> relativeFrom;

        public VerticalPositionElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.align = this.RegisterChildElement<VerticalAlignmentElement>(DocxElementNames.DrawingAlignment);
            this.offset = this.RegisterChildElement<OneDimentionOffsetElement>(DocxElementNames.Offset);

            this.relativeFrom = new MappedOpenXmlAttribute<VerticalRelativeFrom>("relativeFrom", null, TypeMappers.VerticalRelativeFromMapper, true);
            this.RegisterAttribute(this.relativeFrom);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.VerticalPosition;
            }
        }

        public OneDimentionOffsetElement OffsetElement
        {
            get
            {
                return this.offset.Element;
            }
        }

        public VerticalAlignmentElement VerticalAlignmentElement
        {
            get
            {
                return this.align.Element;
            }
        }

        public VerticalRelativeFrom RelativeFrom
        {
            get
            {
                return this.relativeFrom.Value;
            }
            set
            {
                this.relativeFrom.Value = value;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeAnchorBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            this.RelativeFrom = shape.VerticalPosition.RelativeFrom;
            switch (shape.VerticalPosition.ValueType)
            {
                case PositionValueType.Alignment:

                    this.CreateElement(this.align);
                    this.VerticalAlignmentElement.InnerText = TypeMappers.RelativeVerticalAlignmentMapper.GetFromValue(shape.VerticalPosition.Alignment);

                    break;

                case PositionValueType.Offset:

                    this.CreateElement(this.offset);
                    this.OffsetElement.InnerText = Converters.EmuToDipConverter.ConvertToString(shape.VerticalPosition.Offset);

                    break;
            }
        }

        public void CopyPropertiesТо(IDocxImportContext context, ShapeAnchorBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            shape.VerticalPosition.RelativeFrom = this.RelativeFrom;
            if (this.VerticalAlignmentElement != null)
            {
                RelativeVerticalAlignment align = TypeMappers.RelativeVerticalAlignmentMapper.GetToValue(this.VerticalAlignmentElement.InnerText);
                shape.VerticalPosition = new VerticalPosition(this.RelativeFrom, align);
                this.ReleaseElement(this.align);
            }

            if (this.OffsetElement != null)
            {
                shape.VerticalPosition = new VerticalPosition(this.RelativeFrom, Converters.EmuToDipConverter.ConvertFromString(this.OffsetElement.InnerText));
                this.ReleaseElement(this.offset);
            }
        }

        protected override OpenXmlElementBase CreateElement(string elementName)
        {
            if (elementName == DocxElementNames.DrawingAlignment)
            {
                VerticalAlignmentElement element = new VerticalAlignmentElement(this.PartsManager);
                element.Part = this.Part;

                return element;
            }

            return base.CreateElement(elementName);
        }

        protected override void ReleaseElementOverride(OpenXmlElementBase element)
        {
            if (element.ElementName != DocxElementNames.DrawingAlignment)
            {
                base.ReleaseElementOverride(element);
            }
        }
    }
}
