﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Drawing
{
    internal abstract class WrapTextElementBase : DrawingElementBase
    {
        private readonly MappedOpenXmlAttribute<TextWrap> wrapText;

        public WrapTextElementBase(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.wrapText = new MappedOpenXmlAttribute<TextWrap>("wrapText", null, TypeMappers.TextWrapMapper, true);
            this.RegisterAttribute(this.wrapText);
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public TextWrap WrapText
        {
            get
            {
                return this.wrapText.Value;
            }
            set
            {
                this.wrapText.Value = value;
            }
        }

        public void CopyPropertiesFrom(IDocxExportContext context, ShapeWrapping shapeWrapping)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shapeWrapping, "shapeWrapping");

            this.WrapText = shapeWrapping.TextWrap;
        }

        public void CopyPropertiesTo(IDocxImportContext context, ShapeWrapping shapeWrapping)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shapeWrapping, "shapeWrapping");

            shapeWrapping.TextWrap = this.WrapText;
        }
    }
}
