﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Watermarks;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.HeaderFooter
{
    internal class HeaderElement : HeaderFooterElementBase<Header>
    {
        public HeaderElement(DocxPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Header;
            }
        }

        public override IEnumerable<OpenXmlNamespace> Namespaces
        {
            get
            {
                yield return OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace;
            }
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            IEnumerable<OpenXmlElementBase> baseResults = base.EnumerateChildElements(context);

            foreach (OpenXmlElementBase result in baseResults)
            {
                yield return result;
            }

            foreach (Watermark watermark in this.BlockContainer.Watermarks)
            {
                yield return this.CreateWatermarkContainerParagraphElement(watermark);
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            base.OnAfterReadChildElement(context, childElement);

            if (childElement.ElementName == DocxElementNames.Paragraph)
            {
                ParagraphElement paragraphElement = (ParagraphElement)childElement;
                if (paragraphElement.Watermark != null)
                {
                    this.BlockContainer.Blocks.Remove(paragraphElement.Paragraph);
                    this.BlockContainer.Watermarks.Add(paragraphElement.Watermark);
                }
            }

            if (childElement.ElementName == DocxElementNames.StructuredDocumentTag)
            {
                StructuredDocumentTagElement sdtElement = (StructuredDocumentTagElement)childElement;
                if (sdtElement.Watermark != null)
                {
                    this.BlockContainer.Watermarks.Add(sdtElement.Watermark);
                }
            }
        }

        private ParagraphElement CreateWatermarkContainerParagraphElement(Watermark watermark)
        {
            RadFlowDocument dummy = new RadFlowDocument();
            Paragraph paragraph = new Paragraph(dummy);
            paragraph.Inlines.Add(new Run(dummy));

            ParagraphElement paragraphElement = this.CreateElement<ParagraphElement>(DocxElementNames.Paragraph);
            paragraphElement.SetAssociatedFlowModelElement(paragraph);
            paragraphElement.Watermark = watermark;

            return paragraphElement;
        }
    }
}
