﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists.ListsInfo;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists
{
    internal class ListLevelOverrideElement : DocxElementBase
    {
        private readonly IntOpenXmlAttribute listLevelIdAttribute;
        private readonly OpenXmlChildElement<StartIndexOverrideElement> startIndexOverride;
        private ListLevelInfo listLevelInfo;

        public ListLevelOverrideElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.listLevelIdAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("ilvl", OpenXmlNamespaces.WordprocessingMLNamespace, true));

            this.startIndexOverride = this.RegisterChildElement<StartIndexOverrideElement>(DocxElementNames.StartIndexOverride);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.ListLevelOverride;
            }
        }

        public void SetAssociatedElementInfo(ListLevelInfo listLevelInfo)
        {
            Guard.ThrowExceptionIfNull(listLevelInfo, "listLevelInfo");

            this.listLevelInfo = listLevelInfo;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            this.listLevelInfo.ListLevelId = this.listLevelIdAttribute.Value;

            if (this.startIndexOverride.Element != null && !this.listLevelInfo.StartIndex.HasValue)
            {
                this.listLevelInfo.StartIndex = this.startIndexOverride.Element.Value;
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "element");

            switch (childElement.ElementName)
            {
                case DocxElementNames.ListLevel:
                    ((ListLevelElement)childElement).SetAssociatedElementInfo(this.listLevelInfo);
                    break;
            }
        }
    }
}
