﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists
{
    internal class ListsExportContext
    {
        private readonly RadFlowDocument document;
        private readonly IdGenerator numberedListIdGenerator = new IdGenerator(1);
        private readonly IdGenerator abstractListIdGenerator = new IdGenerator(0);

        private readonly Dictionary<int, DocxListIds> listAbstraction = new Dictionary<int, DocxListIds>();

        public ListsExportContext(RadFlowDocument document)
        {
            this.document = document;

            this.InitializeCollections();
        }

        public int GetNumberedListIdForParagraphOrStyle(ParagraphProperties paragraphProperties)
        {
            if (paragraphProperties.OwnerDocumentElement != null)
            {
                if (this.listAbstraction[paragraphProperties.ListId.LocalValue.Value].NumberedListIdForStyle.HasValue)
                {
                    return this.listAbstraction[paragraphProperties.ListId.LocalValue.Value].NumberedListIdForStyle.Value;
                }
                else
                {
                    return this.listAbstraction[paragraphProperties.ListId.LocalValue.Value].NumberedListIdForParagraph;
                }
            }
            else
            {
                return this.listAbstraction[paragraphProperties.ListId.LocalValue.Value].NumberedListIdForParagraph;
            }
        }

        public DocxListIds GetDocxListIds(int listId)
        {
            return this.listAbstraction[listId];
        }

        private void InitializeCollections()
        {
            if (this.document.Lists.Count <= 0)
            {
                return;
            }

            foreach (List list in this.document.Lists)
            {
                if (!string.IsNullOrEmpty(list.StyleId))
                {
                    this.ExtractComplexListIds(list);
                }
                else
                {
                    this.ExtractSimpleListIds(list);
                }
            }
        }

        private void ExtractComplexListIds(List list)
        {
            //// paragraph -> paragraphProperties -> numbered list -> style -> paragraphProperties -> numbered list -> abstract list.

            int numberedListIdForParagraph = this.numberedListIdGenerator.GetNext();
            int numberedListIdForStyle = this.numberedListIdGenerator.GetNext();
            int abstractListId = this.abstractListIdGenerator.GetNext();
            int abstractListIdForStyle = this.abstractListIdGenerator.GetNext();

            DocxListIds listPair = new DocxListIds();
            listPair.NumberedListIdForParagraph = numberedListIdForParagraph;
            listPair.NumberedListIdForStyle = numberedListIdForStyle;
            listPair.AbstractListId = abstractListId;
            listPair.AbstractListIdForStyle = abstractListIdForStyle;

            this.listAbstraction.Add(list.Id, listPair);
        }

        private void ExtractSimpleListIds(List list)
        {
            //// paragraph -> paragraphProperties -> numbered list -> abstract list.

            int numberedListId = this.numberedListIdGenerator.GetNext();
            int abstractListId = this.abstractListIdGenerator.GetNext();

            DocxListIds listPair = new DocxListIds();
            listPair.AbstractListId = abstractListId;
            listPair.NumberedListIdForParagraph = numberedListId;

            this.listAbstraction.Add(list.Id, listPair);
        }
    }
}
