﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists.ListsInfo;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists
{
    internal class ListsImportContext
    {
        private readonly Dictionary<int, AbstractListInfo> abstractListInfoCollection = new Dictionary<int, AbstractListInfo>();
        private readonly Dictionary<int, NumberingListInfo> numberingListInfoCollection = new Dictionary<int, NumberingListInfo>();
        private readonly Dictionary<int, int> abstractListToListMapper = new Dictionary<int, int>();

        public Dictionary<int, NumberingListInfo> NumberingListInfoCollection
        {
            get
            {
                return this.numberingListInfoCollection;
            }
        }

        public Dictionary<int, AbstractListInfo> AbstractListInfoCollection
        {
            get
            {
                return this.abstractListInfoCollection;
            }
        }

        public void AddAbstractListToListMapping(int abstractListInfoId, int listId, out int nextAbstractListInfoId)
        {
            // common list.
            if (!this.abstractListToListMapper.ContainsKey(abstractListInfoId))
            {
                this.abstractListToListMapper.Add(abstractListInfoId, listId);
                nextAbstractListInfoId = abstractListInfoId;
            }
            else
            {
                // overriden list.
                int next = abstractListInfoId;
                while (this.abstractListToListMapper.ContainsKey(next))
                {
                    next++;
                }
                nextAbstractListInfoId = next;
                this.abstractListToListMapper.Add(nextAbstractListInfoId, listId);
            }
        }

        public int? GetListIdByNumberedListId(int numberedListId)
        {
            if (!this.numberingListInfoCollection.ContainsKey(numberedListId))
            {
                return null;
            }

            int abstractListId = this.numberingListInfoCollection[numberedListId].AbstractListId;

            // simple list
            int? listId = this.GetListIdByAbstractListId(abstractListId);
            if (listId.HasValue)
            {
                return listId;
            }

            // complex list
            if (this.AbstractListInfoCollection.ContainsKey(abstractListId))
            {
                string numStyleLink = this.AbstractListInfoCollection[abstractListId].NumStyleLink;
                foreach (KeyValuePair<int, AbstractListInfo> pair in this.AbstractListInfoCollection)
                {
                    if (pair.Value.StyleLink == numStyleLink)
                    {
                        listId = this.GetListIdByAbstractListId(pair.Value.Id);
                        if (listId.HasValue)
                        {
                            return listId;
                        }
                        break;
                    }
                }
            }

            return null;
        }

        private int? GetListIdByAbstractListId(int abstractListId)
        {
            if (this.abstractListToListMapper.ContainsKey(abstractListId))
            {
                int listId = this.abstractListToListMapper[abstractListId];
                return listId;
            }

            return null;
        }
    }
}
