﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Lists;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists.ListsInfo
{
    internal class AbstractListInfo
    {
        public AbstractListInfo()
        {
            this.Levels = new List<ListLevelInfo>();
        }

        public AbstractListInfo(List list, bool isAbstractListForStyle)
            : this()
        {
            this.IsAbstractListForStyle = isAbstractListForStyle;

            this.Id = list.Id;
            this.MultilevelType = list.MultilevelType;
            this.StyleLink = list.StyleId;
            this.Name = list.Name;

            if (this.IsAbstractListForStyle)
            {
                this.NumStyleLink = list.StyleId;
            }

            if (list.MultilevelType == Flow.Model.Lists.MultilevelType.SingleLevel)
            {
                this.Levels.Add(new ListLevelInfo(list.Levels[0]));
            }
            else
            {
                foreach (ListLevel level in list.Levels)
                {
                    this.Levels.Add(new ListLevelInfo(level));
                }
            }
        }

        public AbstractListInfo(AbstractListInfo abstractListInfo)
            : this()
        {
            this.Id = abstractListInfo.Id;
            this.MultilevelType = abstractListInfo.MultilevelType;
            this.StyleLink = abstractListInfo.StyleLink;
            this.NumStyleLink = abstractListInfo.NumStyleLink;
            this.Name = abstractListInfo.Name;

            foreach (ListLevelInfo level in abstractListInfo.Levels)
            {
                ListLevelInfo levelInfo = new ListLevelInfo(level);
                this.Levels.Add(levelInfo);
            }
        }

        public bool IsAbstractListForStyle { get; set; }

        public int Id { get; set; }

        public MultilevelType? MultilevelType { get; set; }

        public string Name { get; set; }

        public string NumStyleLink { get; set; }

        public string StyleLink { get; set; }

        public List<ListLevelInfo> Levels { get; set; }

        public List GetList()
        {
            List list = new List();
            list.MultilevelType = this.MultilevelType ?? Telerik.Windows.Documents.Flow.Model.Lists.MultilevelType.HybridMultilevel;
            list.Name = this.Name;
            list.StyleId = this.StyleLink;

            if (list.MultilevelType == Flow.Model.Lists.MultilevelType.SingleLevel)
            {
                if (this.Levels.Count > 0)
                {
                    ListLevel listLevel = this.Levels[0].GetListLevel();
                    listLevel.OwnerList = list;
                    list.Levels[this.Levels[0].ListLevelId] = listLevel;
                }
            }
            else
            {
                foreach (ListLevelInfo listLevelInfo in this.Levels)
                {
                    ListLevel listLevel = listLevelInfo.GetListLevel();
                    listLevel.OwnerList = list;
                    list.Levels[listLevelInfo.ListLevelId] = listLevel;
                }
            }

            return list;
        }
    }
}
