﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists.ListsInfo;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Lists
{
    internal class NumberedListElement : DocxElementBase
    {
        private readonly IntOpenXmlAttribute listIdAttribute;
        private readonly OpenXmlChildElement<AbstractListIdElement> abstractListId;
        private NumberingListInfo numberingListInfo;

        public NumberedListElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.listIdAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("numId", OpenXmlNamespaces.WordprocessingMLNamespace, true));

            this.abstractListId = this.RegisterChildElement<AbstractListIdElement>(DocxElementNames.AbstractListId);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.NumberedList;
            }
        }

        public void SetAssociatedElementInfo(NumberingListInfo numberingListInfo)
        {
            Guard.ThrowExceptionIfNull(numberingListInfo, "numberingListInfo");

            this.numberingListInfo = numberingListInfo;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            this.listIdAttribute.Value = this.numberingListInfo.Id;

            this.CreateElement(this.abstractListId);
            this.abstractListId.Element.Value = this.numberingListInfo.AbstractListId;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (this.listIdAttribute.HasValue)
            {
                this.numberingListInfo.Id = this.listIdAttribute.Value;
            }

            if (this.abstractListId.Element != null)
            {
                this.numberingListInfo.AbstractListId = this.abstractListId.Element.Value;
                this.ReleaseElement(this.abstractListId);
            }

            context.ListsImportContext.NumberingListInfoCollection.Add(this.numberingListInfo.Id, this.numberingListInfo);
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "element");

            switch (childElement.ElementName)
            {
                case DocxElementNames.ListLevelOverride:
                    ListLevelInfo listLevelInfo = new ListLevelInfo();
                    this.numberingListInfo.ListLevelOverrideCollection.Add(listLevelInfo);
                    ((ListLevelOverrideElement)childElement).SetAssociatedElementInfo(listLevelInfo);
                    break;
            }
        }
    }
}
