﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Settings
{
    internal class CompatibilityElement : DocxElementBase
    {
        public CompatibilityElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Compatibility;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return context.ExportSettings.CompatibilityVersion.HasValue;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (context.ExportSettings.CompatibilityVersion.HasValue)
            {
                yield return this.CreateCompatibilitySettingElement("compatibilityMode", "http://schemas.microsoft.com/office/word", context.ExportSettings.CompatibilityVersion.Value.ToString());
            }
        }

        private CompatibilitySettingElement CreateCompatibilitySettingElement(string name, string uri, string value)
        {
            CompatibilitySettingElement compatibilitySetting = this.CreateElement<CompatibilitySettingElement>(DocxElementNames.CompatibilitySetting);
            compatibilitySetting.Name = name;
            compatibilitySetting.Uri = uri;
            compatibilitySetting.Value = value;

            return compatibilitySetting;
        }
    }
}
