﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Protection;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Settings
{
    internal class DocumentProtectionElement : DocxElementBase
    {
        private readonly MappedOpenXmlAttribute<ProtectionMode> protectionModeAttribute;
        private readonly BoolOpenXmlAttribute enforcementAttribute;
        private readonly OpenXmlAttribute<string> hashValueAttribute;
        private readonly OpenXmlAttribute<string> saltValueAttribute;
        private readonly IntOpenXmlAttribute spinCountAttribute;
        private readonly MappedOpenXmlAttribute<string> algorithmTypeAttribute;

        public DocumentProtectionElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.enforcementAttribute = this.RegisterAttribute(new BoolOpenXmlAttribute("enforcement", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.hashValueAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("hash", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.saltValueAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("salt", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.spinCountAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("cryptSpinCount", OpenXmlNamespaces.WordprocessingMLNamespace));

            this.protectionModeAttribute = this.RegisterAttribute(
                new MappedOpenXmlAttribute<ProtectionMode>("edit", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ProtectionModeTypeMapper));

            this.algorithmTypeAttribute = this.RegisterAttribute(
                new MappedOpenXmlAttribute<string>("cryptAlgorithmSid", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ProtectionAlgorithmTypeMapper));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.DocumentProtection;
            }
        }

        internal ProtectionMode ProtectionMode
        {
            get
            {
                return this.protectionModeAttribute.Value;
            }
            set
            {
                this.protectionModeAttribute.Value = value;
            }
        }

        internal bool Enforcement
        {
            get
            {
                return this.enforcementAttribute.Value;
            }
            set
            {
                this.enforcementAttribute.Value = value;
            }
        }

        internal string AlgorithmName
        {
            get
            {
                return this.algorithmTypeAttribute.Value;
            }
            set
            {
                this.algorithmTypeAttribute.Value = value;
            }
        }

        internal string Salt
        {
            get
            {
                return this.saltValueAttribute.Value;
            }
            set
            {
                this.saltValueAttribute.Value = value;
            }
        }

        internal string Hash
        {
            get
            {
                return this.hashValueAttribute.Value;
            }
            set
            {
                this.hashValueAttribute.Value = value;
            }
        }

        internal int SpinCount
        {
            get
            {
                return this.spinCountAttribute.Value;
            }
            set
            {
                this.spinCountAttribute.Value = value;
            }
        }

        // import
        internal void CopyPropertiesTo(ProtectionSettings protectionSettings)
        {
            protectionSettings.Enforced = this.Enforcement;
            protectionSettings.ProtectionMode = this.ProtectionMode;
            protectionSettings.AlgorithmName = this.AlgorithmName;
            protectionSettings.Salt = this.Salt;
            protectionSettings.Hash = this.Hash;
            protectionSettings.SpinCount = this.SpinCount;
        }

        // export
        internal void CopyPropertiesFrom(ProtectionSettings protectionSettings)
        {
            this.Enforcement = protectionSettings.Enforced;
            this.ProtectionMode = protectionSettings.ProtectionMode;
            this.AlgorithmName = protectionSettings.AlgorithmName;
            this.Salt = protectionSettings.Salt;
            this.Hash = protectionSettings.Hash;
            this.SpinCount = protectionSettings.SpinCount;
        }
    }
}
