﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Settings
{
    internal class DocumentVariableCollectionElement : DocxElementBase
    {
        public DocumentVariableCollectionElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.DocumentVariableCollection;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return context.Document.DocumentVariables.Count > 0;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (KeyValuePair<string, string> docVariable in context.Document.DocumentVariables)
            {
                yield return this.CreateDocumentVariable(docVariable.Key, docVariable.Value);
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            DocumentVariableElement docVariable = (DocumentVariableElement)childElement;
            context.Document.DocumentVariables.Add(docVariable.Name, docVariable.Value);
        }

        private DocumentVariableElement CreateDocumentVariable(string name, string value)
        {
            DocumentVariableElement docVariable = this.CreateElement<DocumentVariableElement>(DocxElementNames.DocumentVariable);
            docVariable.Name = name;
            docVariable.Value = value;

            return docVariable;
        }
    }
}