﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Settings
{
    internal class SettingsElement : DocxPartRootElementBase
    {
        private readonly OpenXmlChildElement<EvenAndOddHeadersFootersElement> evenAndOddHeadersFootersChildElement;
        private readonly OpenXmlChildElement<UpdateFieldsElement> updateFieldsElementChildElement;
        private readonly OpenXmlChildElement<CompatibilityElement> compatibilityElement;
        private readonly OpenXmlChildElement<DocumentVariableCollectionElement> documentVariableCollectionElement;
        private readonly OpenXmlChildElement<DocumentProtectionElement> documentProtection;
        private readonly OpenXmlChildElement<DefaultTabStopWidthElement> defaultTabStopWidthChildElement;

        public SettingsElement(DocxPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
            this.evenAndOddHeadersFootersChildElement = this.RegisterChildElement<EvenAndOddHeadersFootersElement>(DocxElementNames.EvenAndOddHeadersFooters);
            this.updateFieldsElementChildElement = this.RegisterChildElement<UpdateFieldsElement>(DocxElementNames.UpdateFields);
            this.documentProtection = this.RegisterChildElement<DocumentProtectionElement>(DocxElementNames.DocumentProtection);
            this.defaultTabStopWidthChildElement = this.RegisterChildElement<DefaultTabStopWidthElement>(DocxElementNames.DefaultTabStopWidth);
            this.compatibilityElement = this.RegisterChildElement<CompatibilityElement>(DocxElementNames.Compatibility);
            this.documentVariableCollectionElement = this.RegisterChildElement<DocumentVariableCollectionElement>(DocxElementNames.DocumentVariableCollection);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.DocumentSettings;
            }
        }

        private EvenAndOddHeadersFootersElement EvenAndOddHeadersFootersElement
        {
            get
            {
                return this.evenAndOddHeadersFootersChildElement.Element;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            bool hasDifferentEvenOddPageHeadersFootersLocalValue = context.Document.Properties.HasDifferentEvenOddPageHeadersFooters.LocalValue.Value;
            if (!hasDifferentEvenOddPageHeadersFootersLocalValue.Equals(DocumentDefaultStyleSettings.HasDifferentEvenOddPageHeadersFooters))
            {
                this.CreateElement(this.evenAndOddHeadersFootersChildElement);
                this.EvenAndOddHeadersFootersElement.Value = hasDifferentEvenOddPageHeadersFootersLocalValue;
            }

            if (context.ExportSettings.AutoUpdateFields)
            {
                this.CreateElement(this.updateFieldsElementChildElement);
                this.updateFieldsElementChildElement.Element.Value = true;
            }

            this.CreateElement(this.compatibilityElement);
            this.CreateElement(this.documentVariableCollectionElement);

            if (context.Document.ProtectionSettings.Enforced || !string.IsNullOrEmpty(context.Document.ProtectionSettings.Hash))
            {
                this.CreateElement(this.documentProtection);
                this.documentProtection.Element.CopyPropertiesFrom(context.Document.ProtectionSettings);
            }
            this.CreateElement(this.defaultTabStopWidthChildElement);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.EvenAndOddHeadersFootersElement != null)
            {
                context.Document.Properties.HasDifferentEvenOddPageHeadersFooters.LocalValue = this.EvenAndOddHeadersFootersElement.Value;
                this.ReleaseElement(this.evenAndOddHeadersFootersChildElement);
            }

            if (this.documentProtection.Element != null)
            {
                this.documentProtection.Element.CopyPropertiesTo(context.Document.ProtectionSettings);
            }
        }
    }
}
