﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal class BorderElement : DocxElementBase
    {
        private readonly ThemableColorAttributes colorAttributes;
        private readonly BoolOpenXmlAttribute frameAttribute;
        private readonly BoolOpenXmlAttribute shadowAttribute;
        private readonly OpenXmlAttribute<double> thicnessAttribute;
        private readonly OpenXmlAttribute<double> spacingAttribute;
        private readonly MappedOpenXmlAttribute<BorderStyle> styleAttribute;
        private readonly string elementName;

        private DocumentElementPropertiesBase properties;

        public BorderElement(DocxPartsManager partsManager, string elementName)
            : base(partsManager)
        {
            this.elementName = elementName;
            this.colorAttributes = new ThemableColorAttributes(
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<AutoColor>("color", OpenXmlNamespaces.WordprocessingMLNamespace, DocxConverters.AutoColorConverter)),
                this.RegisterAttribute(new MappedOpenXmlAttribute<ThemeColorType>("themeColor", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ThemeColorTypeMapper)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeTint", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeShade", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                Colors.Black);

            this.frameAttribute = this.RegisterAttribute(new BoolOpenXmlAttribute("frame", OpenXmlNamespaces.WordprocessingMLNamespace, false, false));
            this.shadowAttribute = this.RegisterAttribute(new BoolOpenXmlAttribute("shadow", OpenXmlNamespaces.WordprocessingMLNamespace, false, false));
            this.thicnessAttribute = this.RegisterAttribute(new OpenXmlAttribute<double>("sz", OpenXmlNamespaces.WordprocessingMLNamespace, true));
            this.spacingAttribute = this.RegisterAttribute(new OpenXmlAttribute<double>("space", OpenXmlNamespaces.WordprocessingMLNamespace, true));
            this.styleAttribute = this.RegisterAttribute(new MappedOpenXmlAttribute<BorderStyle>("val", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.BorderStyleMapper, true));
        }

        public override string ElementName
        {
            get
            {
                return this.elementName;
            }
        }

        public Border Value
        {
            get
            {
                return new Border(Unit.PointToDip(this.thicnessAttribute.Value / 8.0),
                    this.styleAttribute.Value,
                    this.colorAttributes.GetThemableColor(),
                    this.shadowAttribute.Value,
                    this.frameAttribute.Value,
                    this.spacingAttribute.Value);
            }
            set
            {
                this.thicnessAttribute.Value = Unit.DipToPoint(value.Thickness * 8);
                this.styleAttribute.Value = value.Style;
                if (value.Color != null)
                {
                    this.colorAttributes.FillAttributes(value.Color, this.Theme);
                }
                this.shadowAttribute.Value = value.Shadow;
                this.frameAttribute.Value = value.Frame;
                this.spacingAttribute.Value = value.Spacing;
            }
        }

        public DocumentTheme Theme
        {
            get
            {
                if (this.properties.OwnerDocumentElement != null)
                {
                    return this.properties.OwnerDocumentElement.Document.Theme;
                }
                else
                {
                    return this.properties.OwnerStyle.Document.Theme;
                }
            }
        }

        public void SetAssociatedFlowModelElement(DocumentElementPropertiesBase properties)
        {
            Guard.ThrowExceptionIfNull(properties, "properties");

            this.properties = properties;
        }
    }
}
