﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal abstract class BordersElement : DocxElementBase
    {
        public BordersElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
        }

        public void SetBorderElementProperties(OpenXmlChildElement<BorderElement> borderElement, Border border, DocumentElementPropertiesBase properties, BorderStyle borderStyleToCompare)
        {
            Guard.ThrowExceptionIfNull(borderElement, "borderElement");
            Guard.ThrowExceptionIfNull(border, "border");
            Guard.ThrowExceptionIfNull(properties, "properties");

            if (border.Style != borderStyleToCompare)
            {
                this.CreateElement(borderElement);
                borderElement.Element.SetAssociatedFlowModelElement(properties);
                borderElement.Element.Value = border;
            }
        }

        public void SetBorderValue(OpenXmlChildElement<BorderElement> borderElement, Func<Border, ParagraphBorders> setBorderFunc, ref ParagraphBorders borders, ref bool hasLocalBorderValue)
        {
            Guard.ThrowExceptionIfNull(borderElement, "borderElement");
            Guard.ThrowExceptionIfNull(setBorderFunc, "setBorderFunc");

            if (borderElement.Element != null)
            {
                hasLocalBorderValue = true;
                borders = setBorderFunc(borderElement.Element.Value);
            }
        }

        public void SetBorderValue(OpenXmlChildElement<BorderElement> borderElement, Func<Border, TableBorders> setBorderFunc, ref TableBorders borders, ref bool hasLocalBorderValue)
        {
            Guard.ThrowExceptionIfNull(borderElement, "borderElement");
            Guard.ThrowExceptionIfNull(setBorderFunc, "setBorderFunc");

            if (borderElement.Element != null)
            {
                hasLocalBorderValue = true;
                borders = setBorderFunc(borderElement.Element.Value);
            }
        }

        public void SetBorderValue(OpenXmlChildElement<BorderElement> borderElement, Func<Border, TableCellBorders> setBorderFunc, ref TableCellBorders borders, ref bool hasLocalBorderValue)
        {
            Guard.ThrowExceptionIfNull(borderElement, "borderElement");
            Guard.ThrowExceptionIfNull(setBorderFunc, "setBorderFunc");

            if (borderElement.Element != null)
            {
                hasLocalBorderValue = true;
                borders = setBorderFunc(borderElement.Element.Value);
            }
        }
    }
}
