﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.DefaultStyles
{
    internal class DocumentDefaultStylesElement : DocxElementBase
    {
        private readonly OpenXmlChildElement<RunDefaultPropertiesElement> runDefaultPropertiesChildElement;
        private readonly OpenXmlChildElement<ParagraphDefaultPropertiesElement> paragraphDefaultPropertiesChildElement;

        public DocumentDefaultStylesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.runDefaultPropertiesChildElement = this.RegisterChildElement<RunDefaultPropertiesElement>(DocxElementNames.RunDefaultProperties);
            this.paragraphDefaultPropertiesChildElement = this.RegisterChildElement<ParagraphDefaultPropertiesElement>(DocxElementNames.ParagraphDefaultProperties);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.DocumentDefaultStyles;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (context.Document.DefaultStyle.CharacterProperties.HasLocalValues())
            {
                this.CreateElement(this.runDefaultPropertiesChildElement);
            }

            if (context.Document.DefaultStyle.ParagraphProperties.HasLocalValues())
            {
                this.CreateElement(this.paragraphDefaultPropertiesChildElement);
            }
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.runDefaultPropertiesChildElement);
            this.ReleaseElement(this.paragraphDefaultPropertiesChildElement);
        }
    }
}
