﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.DefaultStyles
{
    internal class ParagraphDefaultPropertiesElement : DocxElementBase
    {
        private readonly OpenXmlChildElement<ParagraphPropertiesElement> paragraphPropertiesChildElement;

        public ParagraphDefaultPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.paragraphPropertiesChildElement = this.RegisterChildElement<ParagraphPropertiesElement>(DocxElementNames.ParagraphProperties);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.ParagraphDefaultProperties;
            }
        }
        private ParagraphPropertiesElement ParagraphPropertiesElement
        {
            get
            {
                return this.paragraphPropertiesChildElement.Element;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ParagraphProperties parProps = context.Document.DefaultStyle.ParagraphProperties;
            if (parProps.HasLocalValues())
            {
                this.CreateElement(this.paragraphPropertiesChildElement);
                this.ParagraphPropertiesElement.SetAssociatedFlowModelElement(parProps);
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase element)
        {
            switch (element.ElementName)
            {
                case DocxElementNames.ParagraphProperties:
                    this.ParagraphPropertiesElement.SetAssociatedFlowModelElement(context.Document.DefaultStyle.ParagraphProperties);
                    break;
            }
        }
    }
}
