﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.DefaultStyles
{
    internal class RunDefaultPropertiesElement : DocxElementBase
    {
        private readonly OpenXmlChildElement<RunPropertiesElement> runPropertiesChildElement;

        public RunDefaultPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.runPropertiesChildElement = this.RegisterChildElement<RunPropertiesElement>(DocxElementNames.RunProperties);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.RunDefaultProperties;
            }
        }

        private RunPropertiesElement RunPropertiesElement
        {
            get
            {
                return this.runPropertiesChildElement.Element;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CharacterProperties charProps = context.Document.DefaultStyle.CharacterProperties;
            if (charProps.HasLocalValues())
            {
                this.CreateElement(this.runPropertiesChildElement);
                this.RunPropertiesElement.SetAssociatedFlowModelElement(charProps);
            }
        }

        protected override void OnBeforeReadChildElement(IDocxImportContext context, OpenXmlElementBase element)
        {
            switch (element.ElementName)
            {
                case DocxElementNames.RunProperties:
                    this.RunPropertiesElement.SetAssociatedFlowModelElement(context.Document.DefaultStyle.CharacterProperties);
                    break;
            }
        }
    }
}
