﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.ParagraphPropertiesElements
{
    internal class IndentationElement : DocxElementBase
    {
        private readonly ConvertedOpenXmlAttribute<double> firstLineIndentAttribute;
        private readonly ConvertedOpenXmlAttribute<double> hangingIndentIndentAttribute;
        private readonly ConvertedOpenXmlAttribute<double> leftIndentIndentAttribute;
        private readonly ConvertedOpenXmlAttribute<double> rightIndentIndentAttribute;

        public IndentationElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.firstLineIndentAttribute = new ConvertedOpenXmlAttribute<double>("firstLine", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.firstLineIndentAttribute);

            this.hangingIndentIndentAttribute = new ConvertedOpenXmlAttribute<double>("hanging", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.hangingIndentIndentAttribute);

            // 'left' and 'right' are named 'start' and 'end' in the 2012 spec, but Word up to 2013 exports them with the old names.
            this.leftIndentIndentAttribute = new ConvertedOpenXmlAttribute<double>("left", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.leftIndentIndentAttribute);

            this.rightIndentIndentAttribute = new ConvertedOpenXmlAttribute<double>("right", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.rightIndentIndentAttribute);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Indentation;
            }
        }

        public void FillAttributes(ParagraphProperties paragraphProperties)
        {
            if (paragraphProperties.FirstLineIndent.HasLocalValue)
            {
                double value = paragraphProperties.FirstLineIndent.LocalValue.Value;
                this.firstLineIndentAttribute.Value = value;
            }

            if (paragraphProperties.HangingIndent.HasLocalValue)
            {
                double value = paragraphProperties.HangingIndent.LocalValue.Value;
                this.hangingIndentIndentAttribute.Value = value;
            }

            if (paragraphProperties.LeftIndent.HasLocalValue)
            {
                double value = paragraphProperties.LeftIndent.LocalValue.Value;
                this.leftIndentIndentAttribute.Value = value;
            }

            if (paragraphProperties.RightIndent.HasLocalValue)
            {
                double value = paragraphProperties.RightIndent.LocalValue.Value;
                this.rightIndentIndentAttribute.Value = value;
            }
        }

        public void ReadAttributes(ParagraphProperties paragraphProperties)
        {
            if (this.firstLineIndentAttribute.HasValue)
            {
                double value = this.firstLineIndentAttribute.Value;
                paragraphProperties.FirstLineIndent.LocalValue = value;
            }

            if (this.hangingIndentIndentAttribute.HasValue)
            {
                double value = this.hangingIndentIndentAttribute.Value;
                paragraphProperties.HangingIndent.LocalValue = value;
            }

            if (this.leftIndentIndentAttribute.HasValue)
            {
                double value = this.leftIndentIndentAttribute.Value;
                paragraphProperties.LeftIndent.LocalValue = value;
            }

            if (this.rightIndentIndentAttribute.HasValue)
            {
                double value = this.rightIndentIndentAttribute.Value;
                paragraphProperties.RightIndent.LocalValue = value;
            }
        }
    }
}
