﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.ParagraphPropertiesElements
{
    internal class NumberingPropertiesElement : DocxElementBase
    {
        private readonly OpenXmlChildElement<ListIdElement> listId;
        private readonly OpenXmlChildElement<ParagraphListLevel> listLevel;

        private ParagraphProperties paragraphProperties;

        public NumberingPropertiesElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.listLevel = this.RegisterChildElement<ParagraphListLevel>(DocxElementNames.ParagraphListLevel);
            this.listId = this.RegisterChildElement<ListIdElement>(DocxElementNames.ListId);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.NumberingProperties;
            }
        }

        public void SetAssociatedFlowModelElement(ParagraphProperties paragraphProperties)
        {
            Guard.ThrowExceptionIfNull(paragraphProperties, "paragraphProperties");

            this.paragraphProperties = paragraphProperties;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return (this.paragraphProperties != null && this.paragraphProperties.ListId.HasLocalValue) || base.ShouldExport(context);
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            if (this.listLevel.Element != null)
            {
                this.paragraphProperties.ListLevel.LocalValue = this.listLevel.Element.Value;
                this.ReleaseElement(this.listLevel);
            }

            if (this.listId.Element != null)
            {
                int? listId = context.ListsImportContext.GetListIdByNumberedListId(this.listId.Element.Value);
                if (listId.HasValue)
                {
                    this.paragraphProperties.ListId.LocalValue = listId;
                }

                this.ReleaseElement(this.listId);
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (this.paragraphProperties.ListId.HasLocalValue)
            {
                this.CreateElement(this.listId);
                this.listId.Element.Value = context.ListsExportContext.GetNumberedListIdForParagraphOrStyle(this.paragraphProperties);
            }

            if (this.paragraphProperties.ListLevel.HasLocalValue)
            {
                this.CreateElement(this.listLevel);
                this.listLevel.Element.Value = this.paragraphProperties.ListLevel.LocalValue.Value;
            }
        }
    }
}
