﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.ParagraphPropertiesElements
{
    internal class ParagraphBordersElement : BordersElement
    {
        private readonly OpenXmlChildElement<BorderElement> leftBorder;
        private readonly OpenXmlChildElement<BorderElement> topBorder;
        private readonly OpenXmlChildElement<BorderElement> rightBorder;
        private readonly OpenXmlChildElement<BorderElement> bottomBorder;
        private readonly OpenXmlChildElement<BorderElement> betweenBorder;

        private ParagraphProperties paragraphProperties;

        public ParagraphBordersElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.topBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Top);
            this.leftBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Left);
            this.bottomBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Bottom);
            this.rightBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Right);
            this.betweenBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderBetween);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.ParagraphBorders;
            }
        }

        public void SetAssociatedFlowModelElement(ParagraphProperties paragraphProperties)
        {
            Guard.ThrowExceptionIfNull(paragraphProperties, "paragraphProperties");

            this.paragraphProperties = paragraphProperties;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            bool hasLocalBorderValue = false;
            ParagraphBorders borders = new ParagraphBorders();

            this.SetBorderValue(this.leftBorder, borders.SetLeft, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.topBorder, borders.SetTop, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.rightBorder, borders.SetRight, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.bottomBorder, borders.SetBottom, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.betweenBorder, borders.SetBetween, ref borders, ref hasLocalBorderValue);

            if (hasLocalBorderValue)
            {
                this.paragraphProperties.Borders.LocalValue = borders;
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (!this.paragraphProperties.Borders.HasLocalValue)
            {
                return;
            }

            ParagraphBorders borders = this.paragraphProperties.Borders.LocalValue;

            this.SetBorderElementProperties(this.leftBorder, borders.Left, this.paragraphProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.topBorder, borders.Top, this.paragraphProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.rightBorder, borders.Right, this.paragraphProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.bottomBorder, borders.Bottom, this.paragraphProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.betweenBorder, borders.Between, this.paragraphProperties, Border.DefaultBorder.Style);
        }
    }
}
