﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.ParagraphPropertiesElements
{
    internal class SpacingElement : DocxElementBase
    {
        private readonly ConvertedOpenXmlAttribute<double> afterAttribute;
        private readonly ConvertedOpenXmlAttribute<double> beforeAttribute;
        private readonly BoolOpenXmlAttribute automaticSpacingAfterAttribute;
        private readonly BoolOpenXmlAttribute automaticSpacingBeforeAttribute;
        private readonly OpenXmlAttribute<double> lineSpacingAttribute;
        private readonly MappedOpenXmlAttribute<HeightType> lineSpacingTypeAttribute;

        public SpacingElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.afterAttribute = new ConvertedOpenXmlAttribute<double>("after", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.afterAttribute);

            this.beforeAttribute = new ConvertedOpenXmlAttribute<double>("before", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.SignedTwipsMeasureToDipConverter);
            this.RegisterAttribute(this.beforeAttribute);

            this.automaticSpacingAfterAttribute = new BoolOpenXmlAttribute("afterAutospacing", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.RegisterAttribute(this.automaticSpacingAfterAttribute);

            this.automaticSpacingBeforeAttribute = new BoolOpenXmlAttribute("beforeAutospacing", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.RegisterAttribute(this.automaticSpacingBeforeAttribute);

            this.lineSpacingAttribute = new OpenXmlAttribute<double>("line", OpenXmlNamespaces.WordprocessingMLNamespace);
            this.RegisterAttribute(this.lineSpacingAttribute);

            this.lineSpacingTypeAttribute = new MappedOpenXmlAttribute<HeightType>("lineRule", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.HeightTypeMapper);
            this.RegisterAttribute(this.lineSpacingTypeAttribute);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Spacing;
            }
        }

        public void FillAttributes(ParagraphProperties paragraphProperties)
        {
            if (paragraphProperties.SpacingAfter.HasLocalValue)
            {
                double value = paragraphProperties.SpacingAfter.LocalValue.Value;
                this.afterAttribute.Value = value;
            }

            if (paragraphProperties.AutomaticSpacingAfter.HasLocalValue)
            {
                this.automaticSpacingAfterAttribute.Value = paragraphProperties.AutomaticSpacingAfter.LocalValue.Value;
            }

            if (paragraphProperties.SpacingBefore.HasLocalValue)
            {
                double value = paragraphProperties.SpacingBefore.LocalValue.Value;
                this.beforeAttribute.Value = value;
            }

            if (paragraphProperties.AutomaticSpacingBefore.HasLocalValue)
            {
                this.automaticSpacingBeforeAttribute.Value = paragraphProperties.AutomaticSpacingBefore.LocalValue.Value;
            }

            if (paragraphProperties.LineSpacing.HasLocalValue)
            {
                double value = paragraphProperties.LineSpacing.LocalValue.Value;
                if (paragraphProperties.LineSpacingType.GetActualValue().Value == HeightType.Auto)
                {
                    value = value * 240;
                }
                else
                {
                    value = Unit.DipToTwip(value);
                }

                this.lineSpacingAttribute.Value = value;
            }

            if (paragraphProperties.LineSpacingType.HasLocalValue)
            {
                this.lineSpacingTypeAttribute.Value = paragraphProperties.LineSpacingType.LocalValue.Value;
            }
        }

        public void ReadAttributes(ParagraphProperties paragraphProperties)
        {
            if (this.afterAttribute.HasValue)
            {
                double value = this.afterAttribute.Value;
                paragraphProperties.SpacingAfter.LocalValue = value;
            }

            if (this.automaticSpacingAfterAttribute.HasValue)
            {
                paragraphProperties.AutomaticSpacingAfter.LocalValue = this.automaticSpacingAfterAttribute.Value;
            }

            if (this.beforeAttribute.HasValue)
            {
                double value = this.beforeAttribute.Value;
                paragraphProperties.SpacingBefore.LocalValue = value;
            }

            if (this.automaticSpacingBeforeAttribute.HasValue)
            {
                paragraphProperties.AutomaticSpacingBefore.LocalValue = this.automaticSpacingBeforeAttribute.Value;
            }

            if (this.lineSpacingAttribute.HasValue)
            {
                double value = this.lineSpacingAttribute.Value;
                if (!this.lineSpacingTypeAttribute.HasValue ||
                    (this.lineSpacingTypeAttribute.HasValue && this.lineSpacingTypeAttribute.Value == HeightType.Auto))
                {
                    value = value / 240;
                }
                else
                {
                    value = Unit.TwipToDip(value);
                }
                paragraphProperties.LineSpacing.LocalValue = value;
            }

            if (this.lineSpacingTypeAttribute.HasValue)
            {
                paragraphProperties.LineSpacingType.LocalValue = this.lineSpacingTypeAttribute.Value;
            }
        }
    }
}
