﻿using System;
using System.Globalization;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.RunPropertiesElements
{
    internal class FontElement : DocxElementBase
    {
        private readonly OpenXmlAttribute<string> asciiAttribute;
        private readonly OpenXmlAttribute<string> highAnsiAttribute;
        private readonly OpenXmlAttribute<string> complexScriptAttribute;

        private readonly OpenXmlAttribute<string> asciiThemeAttribute;
        private readonly OpenXmlAttribute<string> highAnsiThemeAttribute;

        public FontElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.asciiAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("ascii", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.highAnsiAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("hAnsi", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.complexScriptAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("cs", OpenXmlNamespaces.WordprocessingMLNamespace));

            this.asciiThemeAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("asciiTheme", OpenXmlNamespaces.WordprocessingMLNamespace));
            this.highAnsiThemeAttribute = this.RegisterAttribute(new OpenXmlAttribute<string>("hAnsiTheme", OpenXmlNamespaces.WordprocessingMLNamespace));
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Font;
            }
        }

        public void FillAttributes(ThemableFontFamily fontFamily, DocumentTheme theme)
        {
            FontFamily actulaFontFamily = fontFamily.GetActualValue(theme);

            this.asciiAttribute.Value = actulaFontFamily.Source;
            this.highAnsiAttribute.Value = actulaFontFamily.Source;
            this.complexScriptAttribute.Value = actulaFontFamily.Source;

            if (fontFamily.IsFromTheme)
            {
                string themeStr = fontFamily.ThemeFontType.ToString().ToLower(CultureInfo.InvariantCulture) + "HAnsi";
                this.asciiThemeAttribute.Value = themeStr;
                this.highAnsiThemeAttribute.Value = themeStr;
            }
        }

        public ThemableFontFamily GetFontFamily()
        {
            ThemableFontFamily result = null;
            if (this.asciiThemeAttribute.HasValue)
            {
                if (this.asciiThemeAttribute.Value.StartsWith("minor", StringComparison.OrdinalIgnoreCase))
                {
                    result = new ThemableFontFamily(ThemeFontType.Minor);
                }
                else
                {
                    result = new ThemableFontFamily(ThemeFontType.Major);
                }
            }
            else if (this.asciiAttribute.HasValue)
            {
                result = new ThemableFontFamily(this.asciiAttribute.Value);
            }

            return result;
        }
    }
}