﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.RunPropertiesElements
{
    internal class UnderlineElement : DocxElementBase
    {
        private readonly ThemableColorAttributes colorAttributes;
        private readonly MappedOpenXmlAttribute<UnderlinePattern> patternAttribute;

        public UnderlineElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.colorAttributes = new ThemableColorAttributes(
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<AutoColor>("color", OpenXmlNamespaces.WordprocessingMLNamespace, DocxConverters.AutoColorConverter)),
                this.RegisterAttribute(new MappedOpenXmlAttribute<ThemeColorType>("themeColor", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ThemeColorTypeMapper)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeTint", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeShade", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                Colors.Black);

            this.patternAttribute = new MappedOpenXmlAttribute<UnderlinePattern>("val", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.UnderlinePatternMapper, true);
            this.RegisterAttribute(this.patternAttribute);
        }

        /// <summary>
        /// Gets the name of the element.
        /// </summary>
        /// <value>The name of the element.</value>
        public override string ElementName
        {
            get
            {
                return DocxElementNames.Underline;
            }
        }

        public void FillAttributes(CharacterProperties characterProperties, DocumentTheme theme)
        {
            if (characterProperties.UnderlineColor.HasLocalValue)
            {
                ThemableColor color = characterProperties.UnderlineColor.LocalValue;
                this.colorAttributes.FillAttributes(color, theme);
            }

            if (characterProperties.UnderlinePattern.HasLocalValue)
            {
                this.patternAttribute.Value = characterProperties.UnderlinePattern.LocalValue.Value;
            }
        }

        public void ReadAttributes(CharacterProperties characterProperties)
        {
            ThemableColor color = this.colorAttributes.GetThemableColor();
            if (color != null)
            {
                characterProperties.UnderlineColor.LocalValue = color;
            }

            if (this.patternAttribute.HasValue)
            {
                characterProperties.UnderlinePattern.LocalValue = this.patternAttribute.Value;
            }
        }
    }
}