﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal class ShadingElement : DocxElementBase
    {
        private readonly ThemableColorAttributes backgroundColorAttributes;
        private readonly ThemableColorAttributes patternColorAttributes;
        private readonly MappedOpenXmlAttribute<ShadingPattern> shadingPatternAttribute;

        public ShadingElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.backgroundColorAttributes = new ThemableColorAttributes(
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<AutoColor>("fill", OpenXmlNamespaces.WordprocessingMLNamespace, DocxConverters.AutoColorConverter)),
                this.RegisterAttribute(new MappedOpenXmlAttribute<ThemeColorType>("themeFill", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ThemeColorTypeMapper)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeFillTint", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeFillShade", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                Colors.Transparent);

            this.patternColorAttributes = new ThemableColorAttributes(
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<AutoColor>("color", OpenXmlNamespaces.WordprocessingMLNamespace, DocxConverters.AutoColorConverter)),
                this.RegisterAttribute(new MappedOpenXmlAttribute<ThemeColorType>("themeColor", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ThemeColorTypeMapper)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeTint", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                this.RegisterAttribute(new ConvertedOpenXmlAttribute<double>("themeShade", OpenXmlNamespaces.WordprocessingMLNamespace, Converters.HexToTintConverter)),
                Colors.Transparent);

            this.shadingPatternAttribute = new MappedOpenXmlAttribute<ShadingPattern>("val", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.ShadingPatternMapper, true);
            this.RegisterAttribute(this.shadingPatternAttribute);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Shading;
            }
        }

        public void FillAttributes(IPropertiesWithShading properties)
        {
            DocumentTheme theme = properties.Document.Theme;

            if (properties.BackgroundColor.HasLocalValue)
            {
                ThemableColor color = properties.BackgroundColor.LocalValue;
                this.backgroundColorAttributes.FillAttributes(color, theme);
            }

            if (properties.ShadingPatternColor.HasLocalValue)
            {
                ThemableColor color = properties.ShadingPatternColor.LocalValue;
                this.patternColorAttributes.FillAttributes(color, theme);
            }

            // Shading pattern is required.
            ShadingPattern pattern = TypeMappers.ShadingPatternMapper.DefaultToValue.Value;
            if (properties.ShadingPattern.HasLocalValue)
            {
                pattern = properties.ShadingPattern.LocalValue.Value;
            }
            this.shadingPatternAttribute.Value = pattern;
        }

        public void ReadAttributes(IPropertiesWithShading properties)
        {
            ThemableColor backgroundColor = this.backgroundColorAttributes.GetThemableColor();
            if (backgroundColor != null)
            {
                properties.BackgroundColor.LocalValue = backgroundColor;
            }

            ThemableColor patternColor = this.patternColorAttributes.GetThemableColor();
            if (patternColor != null)
            {
                properties.ShadingPatternColor.LocalValue = patternColor;
            }

            properties.ShadingPattern.LocalValue = this.shadingPatternAttribute.Value;
        }
    }
}