﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.DefaultStyles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal class StylesElement : DocxPartRootElementBase
    {
        private readonly OpenXmlChildElement<DocumentDefaultStylesElement> docDefaultsChildElement;

        public StylesElement(DocxPartsManager partsManager, OpenXmlPartBase part)
            : base(partsManager, part)
        {
            this.docDefaultsChildElement = this.RegisterChildElement<DocumentDefaultStylesElement>(DocxElementNames.DocumentDefaultStyles);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.Styles;
            }
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.docDefaultsChildElement);
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IDocxExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (Style style in context.Document.StyleRepository.Styles)
            {
                StyleElement styleElement = this.CreateElement<StyleElement>(DocxElementNames.Style);
                styleElement.Style = style;

                yield return styleElement;
            }
        }

        protected override void OnAfterReadChildElement(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            switch (childElement.ElementName)
            {
                case DocxElementNames.Style:
                    context.Document.StyleRepository.Add((childElement as StyleElement).Style);
                    break;
            }
        }
    }
}
