﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TableCellPropertiesElements
{
    internal class TableCellBordersElement : BordersElement
    {
        private readonly OpenXmlChildElement<BorderElement> leftBorder;
        private readonly OpenXmlChildElement<BorderElement> topBorder;
        private readonly OpenXmlChildElement<BorderElement> rightBorder;
        private readonly OpenXmlChildElement<BorderElement> bottomBorder;
        private readonly OpenXmlChildElement<BorderElement> insideHorizontalBorder;
        private readonly OpenXmlChildElement<BorderElement> insideVerticalBorder;
        private readonly OpenXmlChildElement<BorderElement> diagonalDownBorder;
        private readonly OpenXmlChildElement<BorderElement> diagonalUpBorder;

        private TableCellProperties tableCellProperties;

        public TableCellBordersElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.topBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Top);
            this.leftBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Left);
            this.bottomBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Bottom);
            this.rightBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Right);
            this.insideHorizontalBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderInsideHorizontal);
            this.insideVerticalBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderInsideVertical);
            this.diagonalDownBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderDiagonalDown);
            this.diagonalUpBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderDiagonalUp);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableCellBorders;
            }
        }

        public void SetAssociatedFlowModelElement(TableCellProperties tableCellProperties)
        {
            Guard.ThrowExceptionIfNull(tableCellProperties, "tableCellProperties");

            this.tableCellProperties = tableCellProperties;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            bool hasLocalBorderValue = false;
            TableCellBorders borders = new TableCellBorders();

            this.SetBorderValue(this.leftBorder, (Border b) => { return new TableCellBorders(borders, leftBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.topBorder, (Border b) => { return new TableCellBorders(borders, topBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.rightBorder, (Border b) => { return new TableCellBorders(borders, rightBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.bottomBorder, (Border b) => { return new TableCellBorders(borders, bottomBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.insideHorizontalBorder, (Border b) => { return new TableCellBorders(borders, insideHorizontalBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.insideVerticalBorder, (Border b) => { return new TableCellBorders(borders, insideVerticalBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.diagonalDownBorder, (Border b) => { return new TableCellBorders(borders, diagonalDownBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.diagonalUpBorder, (Border b) => { return new TableCellBorders(borders, diagonalUpBorder: b); }, ref borders, ref hasLocalBorderValue);

            if (hasLocalBorderValue)
            {
                this.tableCellProperties.Borders.LocalValue = borders;
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (!this.tableCellProperties.Borders.HasLocalValue)
            {
                return;
            }

            TableCellBorders borders = this.tableCellProperties.Borders.LocalValue;

            this.SetBorderElementProperties(this.leftBorder, borders.Left, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.topBorder, borders.Top, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.rightBorder, borders.Right, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.bottomBorder, borders.Bottom, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.insideHorizontalBorder, borders.InsideHorizontal, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.insideVerticalBorder, borders.InsideVertical, this.tableCellProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.diagonalDownBorder, borders.DiagonalDown, this.tableCellProperties, Border.EmptyBorder.Style);
            this.SetBorderElementProperties(this.diagonalUpBorder, borders.DiagonalUp, this.tableCellProperties, Border.EmptyBorder.Style);
        }
    }
}
