﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles.TablePropertiesElements
{
    internal class TableBordersElement : BordersElement
    {
        private readonly OpenXmlChildElement<BorderElement> leftBorder;
        private readonly OpenXmlChildElement<BorderElement> topBorder;
        private readonly OpenXmlChildElement<BorderElement> rightBorder;
        private readonly OpenXmlChildElement<BorderElement> bottomBorder;
        private readonly OpenXmlChildElement<BorderElement> insideHorizontalBorder;
        private readonly OpenXmlChildElement<BorderElement> insideVerticalBorder;

        private TableProperties tableProperties;

        public TableBordersElement(DocxPartsManager partsManager)
            : base(partsManager)
        {
            this.topBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Top);
            this.leftBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Left);
            this.bottomBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Bottom);
            this.rightBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.Right);
            this.insideHorizontalBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderInsideHorizontal);
            this.insideVerticalBorder = this.RegisterChildElement<BorderElement>(DocxElementNames.BorderInsideVertical);
        }

        public override string ElementName
        {
            get
            {
                return DocxElementNames.TableBorders;
            }
        }

        public void SetAssociatedFlowModelElement(TableProperties tableProperties)
        {
            Guard.ThrowExceptionIfNull(tableProperties, "tableProperties");

            this.tableProperties = tableProperties;
        }

        protected override void OnAfterRead(IDocxImportContext context)
        {
            bool hasLocalBorderValue = false;
            TableBorders borders = new TableBorders();

            this.SetBorderValue(this.leftBorder, (Border b) => { return new TableBorders(borders, leftBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.topBorder, (Border b) => { return new TableBorders(borders, topBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.rightBorder, (Border b) => { return new TableBorders(borders, rightBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.bottomBorder, (Border b) => { return new TableBorders(borders, bottomBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.insideHorizontalBorder, (Border b) => { return new TableBorders(borders, insideHorizontalBorder: b); }, ref borders, ref hasLocalBorderValue);
            this.SetBorderValue(this.insideVerticalBorder, (Border b) => { return new TableBorders(borders, insideVerticalBorder: b); }, ref borders, ref hasLocalBorderValue);

            if (hasLocalBorderValue)
            {
                this.tableProperties.Borders.LocalValue = borders;
            }
        }

        protected override void OnBeforeWrite(IDocxExportContext context)
        {
            if (!this.tableProperties.Borders.HasLocalValue)
            {
                return;
            }

            TableBorders borders = this.tableProperties.Borders.LocalValue;

            this.SetBorderElementProperties(this.leftBorder, borders.Left, this.tableProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.topBorder, borders.Top, this.tableProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.rightBorder, borders.Right, this.tableProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.bottomBorder, borders.Bottom, this.tableProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.insideHorizontalBorder, borders.InsideHorizontal, this.tableProperties, Border.DefaultBorder.Style);
            this.SetBorderElementProperties(this.insideVerticalBorder, borders.InsideVertical, this.tableProperties, Border.DefaultBorder.Style);
        }
    }
}
