﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal class TableWidthElement : DocumentElementBase
    {
        private readonly string elementName;
        private readonly IntOpenXmlAttribute widthAttribute;
        private readonly MappedOpenXmlAttribute<TableWidthUnitType> widthTypeAttribute;

        public TableWidthElement(DocxPartsManager partsManager, string elementName)
            : base(partsManager)
        {
            this.elementName = elementName;

            this.widthAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("w", OpenXmlNamespaces.WordprocessingMLNamespace, true));
            this.widthTypeAttribute =
                this.RegisterAttribute(new MappedOpenXmlAttribute<TableWidthUnitType>("type", OpenXmlNamespaces.WordprocessingMLNamespace, TypeMappers.TableWidthTypeMapper, true));
        }

        public override string ElementName
        {
            get
            {
                return this.elementName;
            }
        }

        public TableWidthUnit Value
        {
            get
            {
                double widthValue = Unit.TwipToDip(this.widthAttribute.Value);
                if (this.widthTypeAttribute.Value == TableWidthUnitType.Nil)
                {
                    widthValue = 0;
                }
                else if (this.widthTypeAttribute.Value == TableWidthUnitType.Percent)
                {
                    widthValue = this.widthAttribute.Value / 50;
                }
                else if (this.widthTypeAttribute.Value == TableWidthUnitType.Auto && widthValue == 0)
                {
                    return new TableWidthUnit(TableWidthUnitType.Auto);
                }

                return new TableWidthUnit(this.widthTypeAttribute.Value, widthValue);
            }
            set
            {
                this.widthTypeAttribute.Value = value.Type;
                if (value.Type == TableWidthUnitType.Fixed)
                {
                    this.widthAttribute.Value = (int)Unit.DipToTwip(value.Value);
                }
                else if (value.Type == TableWidthUnitType.Percent)
                {
                    this.widthAttribute.Value = (int)(value.Value * 50);
                }
                else
                {
                    this.widthAttribute.Value = 0;
                }
            }
        }

        public double ToDouble()
        {
            // for indent, cell spacing and padding any units with type percent or auto shall be ignored.
            if (this.Value.Type == TableWidthUnitType.Percent || this.Value.Type == TableWidthUnitType.Auto)
            {
                return 0;
            }

            return this.Value.Value;
        }

        protected override bool ShouldExport(IDocxExportContext context)
        {
            return true;
        }
    }
}
