#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Styles
{
    internal class ThemableColorAttributes
    {
        private readonly OpenXmlAttributeBase<AutoColor> colorAttribute;

        private readonly OpenXmlAttributeBase<ThemeColorType> themeColorAttribute;

        private readonly OpenXmlAttributeBase<double> tintAttribute;

        private readonly OpenXmlAttributeBase<double> shadeAttribute;

        private readonly Color autoColorImportValue;

        public ThemableColorAttributes(
            OpenXmlAttributeBase<AutoColor> colorAttribute,
            OpenXmlAttributeBase<ThemeColorType> themeColorAttribute,
            OpenXmlAttributeBase<double> tintAttribute,
            OpenXmlAttributeBase<double> shadeAttribute,
            Color autoColorImportValue)
        {
            this.colorAttribute = colorAttribute;
            this.themeColorAttribute = themeColorAttribute;
            this.tintAttribute = tintAttribute;
            this.shadeAttribute = shadeAttribute;
            this.autoColorImportValue = autoColorImportValue;
        }

        public void FillAttributes(ThemableColor color, DocumentTheme theme)
        {
            if (color.IsFromTheme)
            {
                this.colorAttribute.Value = new AutoColor(color.GetActualValue(theme), color.IsAutomatic);
                this.themeColorAttribute.Value = color.ThemeColorType;

                if (color.TintAndShade.HasValue && color.TintAndShade.Value != 0)
                {
                    double value = color.TintAndShade.Value;
                    if (value > 0)
                    {
                        this.tintAttribute.Value = value;
                    }
                    else
                    {
                        this.shadeAttribute.Value = -value;
                    }
                }
            }
            else
            {
                this.colorAttribute.Value = new AutoColor(color.LocalValue, color.IsAutomatic);
            }
        }

        public ThemableColor GetThemableColor()
        {
            ThemableColor color = null;
            if (this.themeColorAttribute.HasValue)
            {
                ThemeColorType type = this.themeColorAttribute.Value;
                double? tintAndShade = null;

                if (this.tintAttribute.HasValue)
                {
                    tintAndShade = this.tintAttribute.Value;
                }
                else if (this.shadeAttribute.HasValue)
                {
                    tintAndShade = -this.shadeAttribute.Value;
                }

                if (tintAndShade.HasValue)
                {
                    color = new ThemableColor(type, tintAndShade.Value);
                }
                else
                {
                    color = new ThemableColor(type);
                }
            }
            else if (this.colorAttribute.HasValue)
            {
                if (this.colorAttribute.Value.IsAutomatic)
                {
                    color = new ThemableColor(this.autoColorImportValue, true);
                }
                else
                {
                    color = new ThemableColor(this.colorAttribute.Value.Color, false);
                }
            }
            return color;
        }
    }
}