﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.HeaderFooter;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Parts
{
    internal class FooterPart : HeaderFooterPartBase
    {
        private FooterElement footerElement;

        public FooterPart(DocxPartsManager partsManager, Footer footer, int partNumber)
            : base(partsManager, DocxPartNames.FooterPartName, partNumber)
        {
            this.RelationshipId = this.PartsManager.CreateDocumentRelationship(this.Name, DocxRelationshipTypes.FooterRelationshipType);
            this.footerElement = new FooterElement(this.PartsManager, this);
            this.footerElement.SetAssociatedFlowModelElement(footer);
        }

        public FooterPart(DocxPartsManager partsManager, string name)
            : base(partsManager, name)
        {
        }

        public override OpenXmlElementBase RootElement
        {
            get
            {
                return this.footerElement;
            }
        }

        public override string ContentType
        {
            get
            {
                return DocxContentTypeNames.FooterContentType;
            }
        }

        public override void Import(IOpenXmlReader reader, IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            this.footerElement = new FooterElement(this.PartsManager, this);
            this.RelationshipId = this.PartsManager.GetDocumentRelationshipId(this.Name);
            this.footerElement.SetAssociatedFlowModelElement(((IDocxImportContext)context).GetFooterByRelationshipId(this.RelationshipId));

            base.Import(reader, context);
        }
    }
}