﻿using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.HeaderFooter;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Import;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Parts
{
    internal class HeaderPart : HeaderFooterPartBase
    {
        private HeaderElement headerElement;

        public HeaderPart(DocxPartsManager partsManager, Header header, int headerNumber)
            : base(partsManager, DocxPartNames.HeaderPartName, headerNumber)
        {
            this.RelationshipId = this.PartsManager.CreateDocumentRelationship(this.Name, DocxRelationshipTypes.HeaderRelationshipType);
            this.headerElement = new HeaderElement(this.PartsManager, this);
            this.headerElement.SetAssociatedFlowModelElement(header);
        }

        public HeaderPart(DocxPartsManager partsManager, string name)
            : base(partsManager, name)
        {
        }

        public override OpenXmlElementBase RootElement
        {
            get
            {
                return this.headerElement;
            }
        }

        public override string ContentType
        {
            get
            {
                return DocxContentTypeNames.HeaderContentType;
            }
        }

        public override void Import(IOpenXmlReader reader, IOpenXmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            this.headerElement = new HeaderElement(this.PartsManager, this);
            this.RelationshipId = this.PartsManager.GetDocumentRelationshipId(this.Name);
            this.headerElement.SetAssociatedFlowModelElement(((IDocxImportContext)context).GetHeaderByRelationshipId(this.RelationshipId));

            base.Import(reader, context);
        }
    }
}
