﻿using System;
using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Core.Data;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types
{
    internal static class HighlightColors
    {
        static HighlightColors()
        {
            HighlightColors.Mapper = new ValueMapper<string, Color>("none", Colors.Transparent);
            HighlightColors.AvailableColors = new List<Color>();

            HighlightColors.AddColor(Colors.Transparent, "none");
            HighlightColors.AddColor(Colors.Black, "black");
            HighlightColors.AddColor(Colors.Blue, "blue");
            HighlightColors.AddColor(Colors.Cyan, "cyan");
            HighlightColors.AddColor(Color.FromArgb(255, 0, 0, 128), "darkBlue");
            HighlightColors.AddColor(Color.FromArgb(255, 0, 128, 128), "darkCyan");
            HighlightColors.AddColor(Color.FromArgb(255, 128, 128, 128), "darkGray");
            HighlightColors.AddColor(Color.FromArgb(255, 0, 128, 0), "darkGreen");
            HighlightColors.AddColor(Color.FromArgb(255, 128, 0, 128), "darkMagenta");
            HighlightColors.AddColor(Color.FromArgb(255, 128, 0, 0), "darkRed");
            HighlightColors.AddColor(Color.FromArgb(255, 128, 128, 0), "darkYellow");
            HighlightColors.AddColor(Color.FromArgb(255, 0, 255, 0), "green");
            HighlightColors.AddColor(Color.FromArgb(255, 192, 192, 192), "lightGray");
            HighlightColors.AddColor(Color.FromArgb(255, 255, 0, 255), "magenta");
            HighlightColors.AddColor(Colors.Red, "red");
            HighlightColors.AddColor(Colors.White, "white");
            HighlightColors.AddColor(Colors.Yellow, "yellow");
        }

        public static List<Color> AvailableColors { get; private set; }

        public static ValueMapper<string, Color> Mapper { get; private set; }

        public static Color GetNearestHighlightingColor(Color color)
        {
            if (color == Colors.Transparent)
            {
                return color;
            }

            double shortestDistance = double.MaxValue;
            Color nearestColor = Colors.Transparent;
            foreach (Color standardColor in HighlightColors.AvailableColors)
            {
                if (standardColor != Colors.Transparent)
                {
                    double currentDistance = ColorDistance(color, standardColor);
                    if (shortestDistance > currentDistance)
                    {
                        shortestDistance = currentDistance;
                        nearestColor = standardColor;
                    }
                }
            }

            return nearestColor;
        }

        private static void AddColor(Color color, string str)
        {
            HighlightColors.Mapper.AddPair(str, color);
            HighlightColors.AvailableColors.Add(color);
        }

        private static double ColorDistance(Color color1, Color color2)
        {
            return Math.Sqrt(
                Math.Pow(color1.R - color2.R, 2) +
                Math.Pow(color1.G - color2.G, 2) +
                Math.Pow(color1.B - color2.B, 2));
        }
    }
}