﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Common.Tables;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types
{
    internal class TableImporter
    {
        private int currentRowIndex; 
        private int currentColumnIndex;
        private TableCellDataGrid tableCellDataGrid;

        public TableImporter()
        {
        }

        public Table FillTableFromTableCellsData(Table tableToFill, List<TableRowData> tableRowDatasList)
        {
            Guard.ThrowExceptionIfNull(tableRowDatasList, "tableCellDatasList");

            this.tableCellDataGrid = new TableCellDataGrid();
            this.tableCellDataGrid.IsInitializing = true;

            this.currentRowIndex = 0;

            foreach (TableRowData rowData in tableRowDatasList)
            {
                this.currentColumnIndex = 0;

                TableRow currentRow = rowData.Row;
                tableToFill.Rows.Add(currentRow);

                foreach (TableCellData tableCellData in rowData.TableCellDatas)
                {
                    bool skipCell = false;

                    while (this.tableCellDataGrid[this.currentRowIndex, this.currentColumnIndex].Type != TableCellType.Empty)
                    {
                        this.currentColumnIndex++;
                    }

                    int columnSpan = tableCellData.Cell.ColumnSpan;

                    if (tableCellData.Type == TableCellType.VerticallyMerged && this.currentRowIndex > 0 && this.tableCellDataGrid[this.currentRowIndex - 1, this.currentColumnIndex].Type != TableCellType.Empty)
                    {
                        TableCellData tableCellAbove = this.tableCellDataGrid[this.currentRowIndex - 1, this.currentColumnIndex];
                        this.AddCellsToGrid(columnSpan, tableCellAbove);

                        int maxRowSpanForCurrentCell = this.GetMaxRowSpanForCell(tableCellAbove, tableRowDatasList.Count);
                        if (tableCellAbove.Cell.RowSpan < maxRowSpanForCurrentCell)
                        {
                            tableCellAbove.Cell.RowSpan++;
                        }
                        skipCell = true;
                    }
                    else if (columnSpan > 1)
                    {
                        this.AddCellsToGrid(columnSpan, tableCellData);
                    }
                    else if (tableCellData.Type == TableCellType.HorizontalyMerged && this.currentColumnIndex > 0 && this.tableCellDataGrid[this.currentRowIndex, this.currentColumnIndex - 1].Type != TableCellType.Empty)
                    {
                        TableCellData cellBefore = this.tableCellDataGrid[this.currentRowIndex, this.currentColumnIndex - 1];
                        this.AddCellsToGrid(columnSpan, cellBefore);
                        cellBefore.Cell.ColumnSpan++;
                        skipCell = true;
                    }
                    else
                    {
                        this.AddCellsToGrid(columnSpan, tableCellData);
                    }

                    if (!skipCell)
                    {
                        currentRow.Cells.Add(tableCellData.Cell);
                    }
                }

                this.currentRowIndex++;
            }

            return tableToFill;
        }

        private int GetMaxRowSpanForCell(TableCellData tableCellData, int rowsCount)
        {
            bool indexFound = false;
            int startIndex = 0;
            
            for (int i = 0; i < rowsCount; i++)
            {
                if (this.tableCellDataGrid[i, this.currentColumnIndex - tableCellData.Cell.ColumnSpan] == tableCellData)
                {
                    indexFound = true;
                    startIndex = i;
                    break;
                }
            }

            Guard.ThrowExceptionIfFalse(indexFound, "The start index of the cell could not miss.");

            int maxRowSpanForCell = rowsCount - startIndex;

            return maxRowSpanForCell;
        }

        private void AddCellsToGrid(int gridSpan, TableCellData tableCellData)
        {
            for (int limit = this.currentColumnIndex + gridSpan; this.currentColumnIndex < limit; this.currentColumnIndex++)
            {
                this.tableCellDataGrid[this.currentRowIndex, this.currentColumnIndex] = tableCellData;
            }
        }
    }
}
