﻿using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Protection;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Model;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Types
{
    internal static class TypeMappers
    {
        static TypeMappers()
        {
            InitBaselineAlignmentMapper();
            InitThemeColorTypeMapper();
            InitShadingPatternMapper();
            InitUnderlinePatternMapper();
            InitTableCellMergeTypeMapper();
            InitAlignmentMapper();
            InitHeightTypeMapper();
            InitOutlineLevelMapper();
            InitBorderStyleMapper();
            InitHeaderFooterTypeMapper();
            InitTableWidthTypeMapper();
            InitTableLayoutTypeMapper();
            InitVerticalAlignmentMapper();
            InitStyleTypeMapper();
            InitSectionTypeMapper();
            InitPageOrientationMapper();
            InitFileldCharTypeMapper();
            InitRelativeHorizontalAlignmentMapper();
            InitRelativeVerticalAlignmentMapper();
            InitHorizontalRelativeFromMapper();
            InitVerticalRelativeFromMapper();
            InitTextWrapMapper();
            InitBreakTypeMapper();
            InitRestartLocationMapper();
            InitMultilevelTypeMapper();
            InitNumberingStyleMapper();
            InitChapterSeparatorTypeMapper();
            InitProtectionModeTypeMapper();
            InitProtectionAlgorithmsTypeMapper();
            InitEditingGroupTypeMapper();
            InitTabStopTypeMapper();
            InitTabStopLeaderMapper();
            InitTextDirectionMapper();
        }

        public static ValueMapper<string, BaselineAlignment> BaselineAlignmentMapper { get; private set; }

        public static ValueMapper<string, ThemeColorType> ThemeColorTypeMapper { get; private set; }

        public static ValueMapper<string, ShadingPattern> ShadingPatternMapper { get; private set; }

        public static ValueMapper<string, TableCellMergeType> TableCellMergeTypeMapper { get; private set; }

        public static ValueMapper<string, UnderlinePattern> UnderlinePatternMapper { get; private set; }

        public static ValueMapper<string, Alignment> AlignmentMapper { get; private set; }

        public static ValueMapper<string, HeightType> HeightTypeMapper { get; private set; }

        public static ValueMapper<string, OutlineLevel> OutlineLevelMapper { get; private set; }

        public static ValueMapper<string, BorderStyle> BorderStyleMapper { get; private set; }

        public static ValueMapper<string, TableWidthUnitType> TableWidthTypeMapper { get; private set; }

        public static ValueMapper<string, TableLayoutType> TableLayoutTypeMapper { get; private set; }

        public static ValueMapper<string, HeaderFooterType> HeaderFooterTypeMapper { get; private set; }

        public static ValueMapper<string, VerticalAlignment> VerticalAlignmentMapper { get; private set; }

        public static ValueMapper<string, StyleType> StyleTypeMapper { get; private set; }

        public static ValueMapper<string, SectionType> SectionTypeMapper { get; private set; }

        public static ValueMapper<string, PageOrientation> PageOrientationMapper { get; private set; }

        public static ValueMapper<string, FieldCharacterType> FileldCharTypeMapper { get; private set; }

        public static ValueMapper<string, RelativeHorizontalAlignment> RelativeHorizontalAlignmentMapper { get; private set; }

        public static ValueMapper<string, RelativeVerticalAlignment> RelativeVerticalAlignmentMapper { get; private set; }

        public static ValueMapper<string, HorizontalRelativeFrom> HorizontalRelativeFromMapper { get; private set; }

        public static ValueMapper<string, VerticalRelativeFrom> VerticalRelativeFromMapper { get; private set; }

        public static ValueMapper<string, TextWrap> TextWrapMapper { get; private set; }

        public static ValueMapper<string, BreakType> BreakTypeMapper { get; private set; }

        public static ValueMapper<string, TextWrappingRestartLocation> RestartLocationMapper { get; private set; }

        public static ValueMapper<string, MultilevelType> MultilevelTypeMapper { get; private set; }

        public static ValueMapper<string, NumberingStyle> NumberingStyleMapper { get; private set; }

        public static ValueMapper<string, ChapterSeparatorType> ChapterSeparatorTypeMapper { get; private set; }

        public static ValueMapper<string, ProtectionMode> ProtectionModeTypeMapper { get; private set; }

        public static ValueMapper<string, string> ProtectionAlgorithmTypeMapper { get; private set; }

        public static ValueMapper<string, EditingGroup?> EditingGroupTypeMapper { get; private set; }

        public static ValueMapper<string, TabStopType> TabStopTypeMapper { get; private set; }

        public static ValueMapper<string, TabStopLeader> TabStopLeaderMapper { get; private set; }

        public static ValueMapper<string, TextDirection> TextDirectionMapper { get; set; }

        private static void InitShadingPatternMapper()
        {
            ShadingPatternMapper = new ValueMapper<string, ShadingPattern>("clear", ShadingPattern.Clear);
            ShadingPatternMapper.AddPair("nil", ShadingPattern.None);
            ShadingPatternMapper.AddPair("clear", ShadingPattern.Clear);
            ShadingPatternMapper.AddPair("solid", ShadingPattern.Solid);
            ShadingPatternMapper.AddPair("horzStripe", ShadingPattern.HorizontalStripe);
            ShadingPatternMapper.AddPair("vertStripe", ShadingPattern.VerticalStripe);
            ShadingPatternMapper.AddPair("reverseDiagStripe", ShadingPattern.ReverseDiagonalStripe);
            ShadingPatternMapper.AddPair("diagStripe", ShadingPattern.DiagonalStripe);
            ShadingPatternMapper.AddPair("horzCross", ShadingPattern.HorizontalCross);
            ShadingPatternMapper.AddPair("diagCross", ShadingPattern.DiagonalCross);
            ShadingPatternMapper.AddPair("thinHorzStripe", ShadingPattern.ThinHorizontalStripe);
            ShadingPatternMapper.AddPair("thinVertStripe", ShadingPattern.ThinVerticalStripe);
            ShadingPatternMapper.AddPair("thinReverseDiagStripe", ShadingPattern.ThinReverseDiagonalStripe);
            ShadingPatternMapper.AddPair("thinDiagStripe", ShadingPattern.ThinDiagonalStripe);
            ShadingPatternMapper.AddPair("thinHorzCross", ShadingPattern.ThinHorizontalCross);
            ShadingPatternMapper.AddPair("thinDiagCross", ShadingPattern.ThinDiagonalCross);
            ShadingPatternMapper.AddPair("pct5", ShadingPattern.Percent5);
            ShadingPatternMapper.AddPair("pct10", ShadingPattern.Percent10);
            ShadingPatternMapper.AddPair("pct12", ShadingPattern.Percent12);
            ShadingPatternMapper.AddPair("pct15", ShadingPattern.Percent15);
            ShadingPatternMapper.AddPair("pct20", ShadingPattern.Percent20);
            ShadingPatternMapper.AddPair("pct25", ShadingPattern.Percent25);
            ShadingPatternMapper.AddPair("pct30", ShadingPattern.Percent30);
            ShadingPatternMapper.AddPair("pct35", ShadingPattern.Percent35);
            ShadingPatternMapper.AddPair("pct37", ShadingPattern.Percent37);
            ShadingPatternMapper.AddPair("pct40", ShadingPattern.Percent40);
            ShadingPatternMapper.AddPair("pct45", ShadingPattern.Percent45);
            ShadingPatternMapper.AddPair("pct50", ShadingPattern.Percent50);
            ShadingPatternMapper.AddPair("pct55", ShadingPattern.Percent55);
            ShadingPatternMapper.AddPair("pct60", ShadingPattern.Percent60);
            ShadingPatternMapper.AddPair("pct62", ShadingPattern.Percent62);
            ShadingPatternMapper.AddPair("pct65", ShadingPattern.Percent65);
            ShadingPatternMapper.AddPair("pct70", ShadingPattern.Percent70);
            ShadingPatternMapper.AddPair("pct75", ShadingPattern.Percent75);
            ShadingPatternMapper.AddPair("pct80", ShadingPattern.Percent80);
            ShadingPatternMapper.AddPair("pct85", ShadingPattern.Percent85);
            ShadingPatternMapper.AddPair("pct87", ShadingPattern.Percent87);
            ShadingPatternMapper.AddPair("pct90", ShadingPattern.Percent90);
            ShadingPatternMapper.AddPair("pct95", ShadingPattern.Percent95);
        }

        private static void InitThemeColorTypeMapper()
        {
            ThemeColorTypeMapper = new ValueMapper<string, ThemeColorType>();
            ThemeColorTypeMapper.AddPair("accent1", ThemeColorType.Accent1);
            ThemeColorTypeMapper.AddPair("accent2", ThemeColorType.Accent2);
            ThemeColorTypeMapper.AddPair("accent3", ThemeColorType.Accent3);
            ThemeColorTypeMapper.AddPair("accent4", ThemeColorType.Accent4);
            ThemeColorTypeMapper.AddPair("accent5", ThemeColorType.Accent5);
            ThemeColorTypeMapper.AddPair("accent6", ThemeColorType.Accent6);
            ThemeColorTypeMapper.AddPair("background1", ThemeColorType.Background1);
            ThemeColorTypeMapper.AddPair("background2", ThemeColorType.Background2);
            ThemeColorTypeMapper.AddPair("followedHyperlink", ThemeColorType.FollowedHyperlink);
            ThemeColorTypeMapper.AddPair("hyperlink", ThemeColorType.Hyperlink);
            ThemeColorTypeMapper.AddPair("text1", ThemeColorType.Text1);
            ThemeColorTypeMapper.AddPair("text2", ThemeColorType.Text2);

            // TODO: ThemeColorTypes mismatch! Check specification about Background and Text colors (page 2837, ECMA-376, 4th Edition)
            ThemeColorTypeMapper.AddPair("light1", ThemeColorType.Background1);
            ThemeColorTypeMapper.AddPair("light2", ThemeColorType.Background2);
            ThemeColorTypeMapper.AddPair("dark1", ThemeColorType.Text1);
            ThemeColorTypeMapper.AddPair("dark2", ThemeColorType.Text2);
        }

        private static void InitBaselineAlignmentMapper()
        {
            BaselineAlignmentMapper = new ValueMapper<string, BaselineAlignment>("baseline", BaselineAlignment.Baseline);
            BaselineAlignmentMapper.AddPair("baseline", BaselineAlignment.Baseline);
            BaselineAlignmentMapper.AddPair("superscript", BaselineAlignment.Superscript);
            BaselineAlignmentMapper.AddPair("subscript", BaselineAlignment.Subscript);
        }

        private static void InitTableCellMergeTypeMapper()
        {
            TableCellMergeTypeMapper = new ValueMapper<string, TableCellMergeType>("continue", TableCellMergeType.Continue);
            TableCellMergeTypeMapper.AddPair("continue", TableCellMergeType.Continue);
            TableCellMergeTypeMapper.AddPair("restart", TableCellMergeType.Restart);
        }

        private static void InitUnderlinePatternMapper()
        {
            UnderlinePatternMapper = new ValueMapper<string, UnderlinePattern>("none", UnderlinePattern.None);
            UnderlinePatternMapper.AddPair("none", UnderlinePattern.None);
            UnderlinePatternMapper.AddPair("single", UnderlinePattern.Single);
            UnderlinePatternMapper.AddPair("words", UnderlinePattern.Words);
            UnderlinePatternMapper.AddPair("double", UnderlinePattern.Double);
            UnderlinePatternMapper.AddPair("thick", UnderlinePattern.Thick);
            UnderlinePatternMapper.AddPair("dotted", UnderlinePattern.Dotted);
            UnderlinePatternMapper.AddPair("dottedHeavy", UnderlinePattern.DottedHeavy);
            UnderlinePatternMapper.AddPair("dash", UnderlinePattern.Dash);
            UnderlinePatternMapper.AddPair("dashedHeavy", UnderlinePattern.DashedHeavy);
            UnderlinePatternMapper.AddPair("dashLong", UnderlinePattern.DashLong);
            UnderlinePatternMapper.AddPair("dashLongHeavy", UnderlinePattern.DashLongHeavy);
            UnderlinePatternMapper.AddPair("dotDash", UnderlinePattern.DotDash);
            UnderlinePatternMapper.AddPair("dashDotHeavy", UnderlinePattern.DashDotHeavy);
            UnderlinePatternMapper.AddPair("dotDotDash", UnderlinePattern.DotDotDash);
            UnderlinePatternMapper.AddPair("dashDotDotHeavy", UnderlinePattern.DashDotDotHeavy);
            UnderlinePatternMapper.AddPair("wave", UnderlinePattern.Wave);
            UnderlinePatternMapper.AddPair("wavyHeavy", UnderlinePattern.WavyHeavy);
            UnderlinePatternMapper.AddPair("wavyDouble", UnderlinePattern.WavyDouble);
        }

        private static void InitAlignmentMapper()
        {
            AlignmentMapper = new ValueMapper<string, Alignment>();
            AlignmentMapper.AddPair("center", Alignment.Center);
            AlignmentMapper.AddPair("distribute", Alignment.Distribute);
            AlignmentMapper.AddPair("highKashida", Alignment.HighKashida);
            AlignmentMapper.AddPair("both", Alignment.Justified);
            AlignmentMapper.AddPair("start", Alignment.Left);
            AlignmentMapper.AddPair("left", Alignment.Left);
            AlignmentMapper.AddPair("lowKashida", Alignment.LowKashida);
            AlignmentMapper.AddPair("mediumKashida", Alignment.MediumKashida);
            AlignmentMapper.AddPair("end", Alignment.Right);
            AlignmentMapper.AddPair("right", Alignment.Right);
            AlignmentMapper.AddPair("thaiDistribute", Alignment.ThaiDistribute);
        }

        private static void InitHeightTypeMapper()
        {
            HeightTypeMapper = new ValueMapper<string, HeightType>("auto", HeightType.Auto);
            HeightTypeMapper.AddPair("atLeast", HeightType.AtLeast);
            HeightTypeMapper.AddPair("auto", HeightType.Auto);
            HeightTypeMapper.AddPair("exact", HeightType.Exact);
        }

        private static void InitOutlineLevelMapper()
        {
            OutlineLevelMapper = new ValueMapper<string, OutlineLevel>("9", OutlineLevel.None);
            OutlineLevelMapper.AddPair("9", OutlineLevel.None);
            OutlineLevelMapper.AddPair("0", OutlineLevel.Level1);
            OutlineLevelMapper.AddPair("1", OutlineLevel.Level2);
            OutlineLevelMapper.AddPair("2", OutlineLevel.Level3);
            OutlineLevelMapper.AddPair("3", OutlineLevel.Level4);
            OutlineLevelMapper.AddPair("4", OutlineLevel.Level5);
            OutlineLevelMapper.AddPair("5", OutlineLevel.Level6);
            OutlineLevelMapper.AddPair("6", OutlineLevel.Level7);
            OutlineLevelMapper.AddPair("7", OutlineLevel.Level8);
            OutlineLevelMapper.AddPair("8", OutlineLevel.Level9);
        }

        private static void InitBorderStyleMapper()
        {
            BorderStyleMapper = new ValueMapper<string, BorderStyle>();
            BorderStyleMapper.AddPair("none", BorderStyle.None);
            BorderStyleMapper.AddPair("nil", BorderStyle.None);
            BorderStyleMapper.AddPair("single", BorderStyle.Single);
            BorderStyleMapper.AddPair("dotted", BorderStyle.Dotted);
            BorderStyleMapper.AddPair("dashed", BorderStyle.Dashed);
            BorderStyleMapper.AddPair("dashSmallGap", BorderStyle.DashSmallGap);
            BorderStyleMapper.AddPair("dotDash", BorderStyle.DotDash);
            BorderStyleMapper.AddPair("dotDotDash", BorderStyle.DotDotDash);
            BorderStyleMapper.AddPair("double", BorderStyle.Double);
            BorderStyleMapper.AddPair("triple", BorderStyle.Triple);
            BorderStyleMapper.AddPair("thick", BorderStyle.Thick);
            BorderStyleMapper.AddPair("thickThinSmallGap", BorderStyle.ThickThinSmallGap);
            BorderStyleMapper.AddPair("thickThinMediumGap", BorderStyle.ThickThinMediumGap);
            BorderStyleMapper.AddPair("thickThinLargeGap", BorderStyle.ThickThinLargeGap);
            BorderStyleMapper.AddPair("thinThickSmallGap", BorderStyle.ThinThickSmallGap);
            BorderStyleMapper.AddPair("thinThickMediumGap", BorderStyle.ThinThickMediumGap);
            BorderStyleMapper.AddPair("thinThickLargeGap", BorderStyle.ThinThickLargeGap);
            BorderStyleMapper.AddPair("thinThickThinSmallGap", BorderStyle.ThinThickThinSmallGap);
            BorderStyleMapper.AddPair("thinThickThinMediumGap", BorderStyle.ThinThickThinMediumGap);
            BorderStyleMapper.AddPair("thinThickThinLargeGap", BorderStyle.ThinThickThinLargeGap);
            BorderStyleMapper.AddPair("wave", BorderStyle.Wave);
            BorderStyleMapper.AddPair("doubleWave", BorderStyle.DoubleWave);
            BorderStyleMapper.AddPair("dashDotStroked", BorderStyle.DashDotStroked);
            BorderStyleMapper.AddPair("threeDEmboss", BorderStyle.ThreeDEmboss);
            BorderStyleMapper.AddPair("threeDEngrave", BorderStyle.ThreeDEngrave);
            BorderStyleMapper.AddPair("outset", BorderStyle.Outset);
            BorderStyleMapper.AddPair("inset", BorderStyle.Inset);
        }

        private static void InitTableWidthTypeMapper()
        {
            TableWidthTypeMapper = new ValueMapper<string, TableWidthUnitType>();
            TableWidthTypeMapper.AddPair("nil", TableWidthUnitType.Nil);
            TableWidthTypeMapper.AddPair("pct", TableWidthUnitType.Percent);
            TableWidthTypeMapper.AddPair("dxa", TableWidthUnitType.Fixed);
            TableWidthTypeMapper.AddPair("auto", TableWidthUnitType.Auto);
        }

        private static void InitTableLayoutTypeMapper()
        {
            TableLayoutTypeMapper = new ValueMapper<string, TableLayoutType>("auto", TableLayoutType.AutoFit);
            TableLayoutTypeMapper.AddPair("auto", TableLayoutType.AutoFit);
            TableLayoutTypeMapper.AddPair("fixed", TableLayoutType.FixedWidth);
        }

        private static void InitHeaderFooterTypeMapper()
        {
            HeaderFooterTypeMapper = new ValueMapper<string, HeaderFooterType>();
            HeaderFooterTypeMapper.AddPair("default", HeaderFooterType.Default);
            HeaderFooterTypeMapper.AddPair("even", HeaderFooterType.Even);
            HeaderFooterTypeMapper.AddPair("first", HeaderFooterType.First);
        }

        private static void InitVerticalAlignmentMapper()
        {
            VerticalAlignmentMapper = new ValueMapper<string, VerticalAlignment>("top", VerticalAlignment.Top);
            VerticalAlignmentMapper.AddPair("top", VerticalAlignment.Top);
            VerticalAlignmentMapper.AddPair("center", VerticalAlignment.Center);
            VerticalAlignmentMapper.AddPair("bottom", VerticalAlignment.Bottom);
            VerticalAlignmentMapper.AddPair("both", VerticalAlignment.Justified);
        }

        private static void InitStyleTypeMapper()
        {
            StyleTypeMapper = new ValueMapper<string, StyleType>();
            StyleTypeMapper.AddPair("character", StyleType.Character);
            StyleTypeMapper.AddPair("paragraph", StyleType.Paragraph);
            StyleTypeMapper.AddPair("table", StyleType.Table);
            StyleTypeMapper.AddPair("numbering", StyleType.Numbering);
        }

        private static void InitSectionTypeMapper()
        {
            SectionTypeMapper = new ValueMapper<string, SectionType>("nextPage", SectionType.NextPage);
            SectionTypeMapper.AddPair("nextPage", SectionType.NextPage);
            SectionTypeMapper.AddPair("nextColumn", SectionType.NextColumn);
            SectionTypeMapper.AddPair("continuous", SectionType.Continuous);
            SectionTypeMapper.AddPair("evenPage", SectionType.EvenPage);
            SectionTypeMapper.AddPair("oddPage", SectionType.OddPage);
        }

        private static void InitPageOrientationMapper()
        {
            PageOrientationMapper = new ValueMapper<string, PageOrientation>();
            PageOrientationMapper.AddPair("portrait", PageOrientation.Portrait);
            PageOrientationMapper.AddPair("landscape", PageOrientation.Landscape);
        }

        private static void InitRelativeHorizontalAlignmentMapper()
        {
            RelativeHorizontalAlignmentMapper = new ValueMapper<string, RelativeHorizontalAlignment>();
            RelativeHorizontalAlignmentMapper.AddPair("left", RelativeHorizontalAlignment.Left);
            RelativeHorizontalAlignmentMapper.AddPair("right", RelativeHorizontalAlignment.Right);
            RelativeHorizontalAlignmentMapper.AddPair("center", RelativeHorizontalAlignment.Center);
            RelativeHorizontalAlignmentMapper.AddPair("inside", RelativeHorizontalAlignment.Inside);
            RelativeHorizontalAlignmentMapper.AddPair("outside", RelativeHorizontalAlignment.Outside);
        }

        private static void InitRelativeVerticalAlignmentMapper()
        {
            RelativeVerticalAlignmentMapper = new ValueMapper<string, RelativeVerticalAlignment>();
            RelativeVerticalAlignmentMapper.AddPair("top", RelativeVerticalAlignment.Top);
            RelativeVerticalAlignmentMapper.AddPair("bottom", RelativeVerticalAlignment.Bottom);
            RelativeVerticalAlignmentMapper.AddPair("center", RelativeVerticalAlignment.Center);
            RelativeVerticalAlignmentMapper.AddPair("inside", RelativeVerticalAlignment.Inside);
            RelativeVerticalAlignmentMapper.AddPair("outside", RelativeVerticalAlignment.Outside);
        }

        private static void InitHorizontalRelativeFromMapper()
        {
            HorizontalRelativeFromMapper = new ValueMapper<string, HorizontalRelativeFrom>();
            HorizontalRelativeFromMapper.AddPair("character", HorizontalRelativeFrom.Character);
            HorizontalRelativeFromMapper.AddPair("column", HorizontalRelativeFrom.Column);
            HorizontalRelativeFromMapper.AddPair("insideMargin", HorizontalRelativeFrom.InsideMargin);
            HorizontalRelativeFromMapper.AddPair("leftMargin", HorizontalRelativeFrom.LeftMargin);
            HorizontalRelativeFromMapper.AddPair("margin", HorizontalRelativeFrom.Margin);
            HorizontalRelativeFromMapper.AddPair("outsideMargin", HorizontalRelativeFrom.OutsideMargin);
            HorizontalRelativeFromMapper.AddPair("page", HorizontalRelativeFrom.Page);
            HorizontalRelativeFromMapper.AddPair("rightMargin", HorizontalRelativeFrom.RightMargin);
        }

        private static void InitVerticalRelativeFromMapper()
        {
            VerticalRelativeFromMapper = new ValueMapper<string, VerticalRelativeFrom>();
            VerticalRelativeFromMapper.AddPair("bottomMargin", VerticalRelativeFrom.BottomMargin);
            VerticalRelativeFromMapper.AddPair("insideMargin", VerticalRelativeFrom.InsideMargin);
            VerticalRelativeFromMapper.AddPair("line", VerticalRelativeFrom.Line);
            VerticalRelativeFromMapper.AddPair("margin", VerticalRelativeFrom.Margin);
            VerticalRelativeFromMapper.AddPair("outsideMargin", VerticalRelativeFrom.OutsideMargin);
            VerticalRelativeFromMapper.AddPair("page", VerticalRelativeFrom.Page);
            VerticalRelativeFromMapper.AddPair("paragraph", VerticalRelativeFrom.Paragraph);
            VerticalRelativeFromMapper.AddPair("topMargin", VerticalRelativeFrom.TopMargin);
        }

        private static void InitFileldCharTypeMapper()
        {
            FileldCharTypeMapper = new ValueMapper<string, FieldCharacterType>();
            FileldCharTypeMapper.AddPair("begin", FieldCharacterType.Start);
            FileldCharTypeMapper.AddPair("separate", FieldCharacterType.Separator);
            FileldCharTypeMapper.AddPair("end", FieldCharacterType.End);
        }

        private static void InitTextWrapMapper()
        {
            TextWrapMapper = new ValueMapper<string, TextWrap>();
            TextWrapMapper.AddPair("bothSides", TextWrap.BothSides);
            TextWrapMapper.AddPair("right", TextWrap.RightOnly);
            TextWrapMapper.AddPair("left", TextWrap.LeftOnly);
            TextWrapMapper.AddPair("largest", TextWrap.Largest);
        }

        private static void InitBreakTypeMapper()
        {
            BreakTypeMapper = new ValueMapper<string, BreakType>("textWrapping", BreakType.LineBreak);
            BreakTypeMapper.AddPair("page", BreakType.PageBreak);
            BreakTypeMapper.AddPair("column", BreakType.ColumnBreak);
        }

        private static void InitRestartLocationMapper()
        {
            RestartLocationMapper = new ValueMapper<string, TextWrappingRestartLocation>("none", TextWrappingRestartLocation.NextLine);
            RestartLocationMapper.AddPair("all", TextWrappingRestartLocation.NextFullLine);
            RestartLocationMapper.AddPair("left", TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft);
            RestartLocationMapper.AddPair("right", TextWrappingRestartLocation.NextTextRegionUnblockedOnRight);
        }

        private static void InitMultilevelTypeMapper()
        {
            MultilevelTypeMapper = new ValueMapper<string, MultilevelType>("hybridMultilevel", MultilevelType.HybridMultilevel);
            MultilevelTypeMapper.AddPair("multilevel", MultilevelType.Multilevel);
            MultilevelTypeMapper.AddPair("singleLevel", MultilevelType.SingleLevel);
        }

        private static void InitNumberingStyleMapper()
        {
            NumberingStyleMapper = new ValueMapper<string, NumberingStyle>();
            NumberingStyleMapper.AddPair("decimal", NumberingStyle.Decimal);
            NumberingStyleMapper.AddPair("upperRoman", NumberingStyle.UpperRoman);
            NumberingStyleMapper.AddPair("lowerRoman", NumberingStyle.LowerRoman);
            NumberingStyleMapper.AddPair("upperLetter", NumberingStyle.UpperLetter);
            NumberingStyleMapper.AddPair("lowerLetter", NumberingStyle.LowerLetter);
            NumberingStyleMapper.AddPair("ordinal", NumberingStyle.Ordinal);
            NumberingStyleMapper.AddPair("cardinalText", NumberingStyle.CardinalText);
            NumberingStyleMapper.AddPair("ordinalText", NumberingStyle.OrdinalText);
            NumberingStyleMapper.AddPair("hex", NumberingStyle.Hex);
            NumberingStyleMapper.AddPair("chicago", NumberingStyle.Chicago);
            NumberingStyleMapper.AddPair("ideographDigital", NumberingStyle.IdeographDigital);
            NumberingStyleMapper.AddPair("japaneseCounting", NumberingStyle.JapaneseCounting);
            NumberingStyleMapper.AddPair("aiueo", NumberingStyle.Aiueo);
            NumberingStyleMapper.AddPair("iroha", NumberingStyle.Iroha);
            NumberingStyleMapper.AddPair("decimalFullWidth", NumberingStyle.DecimalFullWidth);
            NumberingStyleMapper.AddPair("decimalHalfWidth", NumberingStyle.DecimalHalfWidth);
            NumberingStyleMapper.AddPair("japaneseLegal", NumberingStyle.JapaneseLegal);
            NumberingStyleMapper.AddPair("japaneseDigitalTenThousand", NumberingStyle.JapaneseDigitalTenThousand);
            NumberingStyleMapper.AddPair("decimalEnclosedCircle", NumberingStyle.DecimalEnclosedCircle);
            NumberingStyleMapper.AddPair("decimalFullWidth2", NumberingStyle.DecimalFullWidth2);
            NumberingStyleMapper.AddPair("aiueoFullWidth", NumberingStyle.AiueoFullWidth);
            NumberingStyleMapper.AddPair("irohaFullWidth", NumberingStyle.IrohaFullWidth);
            NumberingStyleMapper.AddPair("decimalZero", NumberingStyle.DecimalZero);
            NumberingStyleMapper.AddPair("bullet", NumberingStyle.Bullet);
            NumberingStyleMapper.AddPair("ganada", NumberingStyle.Ganada);
            NumberingStyleMapper.AddPair("chosung", NumberingStyle.Chosung);
            NumberingStyleMapper.AddPair("decimalEnclosedFullstop", NumberingStyle.DecimalEnclosedFullStop);
            NumberingStyleMapper.AddPair("decimalEnclosedParen", NumberingStyle.DecimalEnclosedParent);
            NumberingStyleMapper.AddPair("decimalEnclosedCircleChinese", NumberingStyle.DecimalEnclosedCircleChinese);
            NumberingStyleMapper.AddPair("ideographEnclosedCircle", NumberingStyle.IdeographEnclosedCircle);
            NumberingStyleMapper.AddPair("ideographTraditional", NumberingStyle.IdeographTraditional);
            NumberingStyleMapper.AddPair("ideographZodiac", NumberingStyle.IdeographZodiac);
            NumberingStyleMapper.AddPair("ideographZodiacTraditional", NumberingStyle.IdeographZodiacTraditional);
            NumberingStyleMapper.AddPair("taiwaneseCounting", NumberingStyle.TaiwaneseCounting);
            NumberingStyleMapper.AddPair("ideographLegalTraditional", NumberingStyle.IdeographLegalTraditional);
            NumberingStyleMapper.AddPair("taiwaneseCountingThousand", NumberingStyle.TaiwaneseCountingThousand);
            NumberingStyleMapper.AddPair("taiwaneseDigital", NumberingStyle.TaiwaneseDigital);
            NumberingStyleMapper.AddPair("chineseCounting", NumberingStyle.ChineseCounting);
            NumberingStyleMapper.AddPair("chineseLegalSimplified", NumberingStyle.ChineseLegalSimplified);
            NumberingStyleMapper.AddPair("chineseCountingThousand", NumberingStyle.ChineseCountingThousand);
            NumberingStyleMapper.AddPair("koreanDigital", NumberingStyle.KoreanDigital);
            NumberingStyleMapper.AddPair("koreanCounting", NumberingStyle.KoreanCounting);
            NumberingStyleMapper.AddPair("koreanLegal", NumberingStyle.KoreanLegal);
            NumberingStyleMapper.AddPair("koreanDigital2", NumberingStyle.KoreanDigital2);
            NumberingStyleMapper.AddPair("vietnameseCounting", NumberingStyle.VietnameseCounting);
            NumberingStyleMapper.AddPair("russianLower", NumberingStyle.RussianLower);
            NumberingStyleMapper.AddPair("russianUpper", NumberingStyle.RussianUpper);
            NumberingStyleMapper.AddPair("none", NumberingStyle.None);
            NumberingStyleMapper.AddPair("numberInDash", NumberingStyle.NumberInDash);
            NumberingStyleMapper.AddPair("hebrew1", NumberingStyle.Hebrew1);
            NumberingStyleMapper.AddPair("hebrew2", NumberingStyle.Hebrew2);
            NumberingStyleMapper.AddPair("arabicAlpha", NumberingStyle.ArabicAlpha);
            NumberingStyleMapper.AddPair("arabicAbjad", NumberingStyle.ArabicAbjad);
            NumberingStyleMapper.AddPair("hindiVowels", NumberingStyle.HindiVowels);
            NumberingStyleMapper.AddPair("hindiConsonants", NumberingStyle.HindiConsonants);
            NumberingStyleMapper.AddPair("hindiNumbers", NumberingStyle.HindiNumbers);
            NumberingStyleMapper.AddPair("hindiCounting", NumberingStyle.HindiCounting);
            NumberingStyleMapper.AddPair("thaiLetters", NumberingStyle.ThaiLetters);
            NumberingStyleMapper.AddPair("thaiNumbers", NumberingStyle.ThaiNumbers);
            NumberingStyleMapper.AddPair("thaiCounting", NumberingStyle.ThaiCounting);
            NumberingStyleMapper.AddPair("bahtText", NumberingStyle.BahtText);
            NumberingStyleMapper.AddPair("dollarText", NumberingStyle.DollarText);
            NumberingStyleMapper.AddPair("custom", NumberingStyle.Custom);
        }

        private static void InitChapterSeparatorTypeMapper()
        {
            ChapterSeparatorTypeMapper = new ValueMapper<string, ChapterSeparatorType>();
            ChapterSeparatorTypeMapper.AddPair("colon", ChapterSeparatorType.Colon);
            ChapterSeparatorTypeMapper.AddPair("hyphen", ChapterSeparatorType.Hyphen);
            ChapterSeparatorTypeMapper.AddPair("emDash", ChapterSeparatorType.EmDash);
            ChapterSeparatorTypeMapper.AddPair("enDash", ChapterSeparatorType.EnDash);
            ChapterSeparatorTypeMapper.AddPair("period", ChapterSeparatorType.Period);
        }

        private static void InitProtectionModeTypeMapper()
        {
            ProtectionModeTypeMapper = new ValueMapper<string, ProtectionMode>("none", ProtectionMode.NoProtection);
            ProtectionModeTypeMapper.AddPair("none", ProtectionMode.NoProtection);
            ProtectionModeTypeMapper.AddPair("comments", ProtectionMode.AllowComments);
            ProtectionModeTypeMapper.AddPair("readOnly", ProtectionMode.ReadOnly);
        }

        private static void InitProtectionAlgorithmsTypeMapper()
        {
            ProtectionAlgorithmTypeMapper = new ValueMapper<string, string>("4", ProtectionAlgorithmNames.SHA1);
            ProtectionAlgorithmTypeMapper.AddPair("4", ProtectionAlgorithmNames.SHA1);
            ProtectionAlgorithmTypeMapper.AddPair("12", ProtectionAlgorithmNames.SHA256);
            ProtectionAlgorithmTypeMapper.AddPair("13", ProtectionAlgorithmNames.SHA384);
            ProtectionAlgorithmTypeMapper.AddPair("14", ProtectionAlgorithmNames.SHA512);
        }

        private static void InitEditingGroupTypeMapper()
        {
            EditingGroupTypeMapper = new ValueMapper<string, EditingGroup?>();
            EditingGroupTypeMapper.AddPair("none", EditingGroup.None);
            EditingGroupTypeMapper.AddPair("everyone", EditingGroup.Everyone);
            EditingGroupTypeMapper.AddPair("administrators", EditingGroup.Administrators);
            EditingGroupTypeMapper.AddPair("contributors", EditingGroup.Contributors);
            EditingGroupTypeMapper.AddPair("editors", EditingGroup.Editors);
            EditingGroupTypeMapper.AddPair("owners", EditingGroup.Owners);
            EditingGroupTypeMapper.AddPair("current", EditingGroup.Current);
        }

        private static void InitTabStopLeaderMapper()
        {
            TabStopLeaderMapper = new ValueMapper<string, TabStopLeader>();
            TabStopLeaderMapper.AddPair("none", TabStopLeader.None);
            TabStopLeaderMapper.AddPair("dot", TabStopLeader.Dot);
            TabStopLeaderMapper.AddPair("hyphen", TabStopLeader.Hyphen);
            TabStopLeaderMapper.AddPair("middleDot", TabStopLeader.MiddleDot);
            TabStopLeaderMapper.AddPair("underscore", TabStopLeader.Underscore);
            TabStopLeaderMapper.AddPair("heavy", TabStopLeader.Underscore);
        }

        private static void InitTabStopTypeMapper()
        {
            TabStopTypeMapper = new ValueMapper<string, TabStopType>();
            TabStopTypeMapper.AddPair("bar", TabStopType.Bar);
            TabStopTypeMapper.AddPair("center", TabStopType.Center);
            TabStopTypeMapper.AddPair("clear", TabStopType.Clear);
            TabStopTypeMapper.AddPair("decimal", TabStopType.Decimal);
            TabStopTypeMapper.AddPair("start", TabStopType.Left);
            TabStopTypeMapper.AddPair("num", TabStopType.Left);
            TabStopTypeMapper.AddPair("left", TabStopType.Left);
            TabStopTypeMapper.AddPair("end", TabStopType.Right);
            TabStopTypeMapper.AddPair("right", TabStopType.Right);
        }

        private static void InitTextDirectionMapper()
        {
            TextDirectionMapper = new ValueMapper<string, TextDirection>();
            TextDirectionMapper.AddPair("lrTb", TextDirection.LeftToRightTopToBottom);
            TextDirectionMapper.AddPair("tbRl", TextDirection.TopToBottomRightToLeft);
            TextDirectionMapper.AddPair("btLr", TextDirection.BottomToTopLeftToRight);
            TextDirectionMapper.AddPair("lrTbV", TextDirection.LeftToRightTopToBottomRotated);
            TextDirectionMapper.AddPair("tbRlV", TextDirection.TopToBottomRightToLeftRotated);
            TextDirectionMapper.AddPair("tbLrV", TextDirection.TopToBottomLeftToRightRotated);
        }
    }
}