﻿using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Contexts;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Model.Elements.Document;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Protection;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Utilities
{
    internal static class DocxHelper
    {
        public static string CreateResourceName(IResource resource)
        {
            return string.Format(DocxPartNames.ResourcePartName, resource.Name);
        }

        public static void AddHangingAnnotation(IDocxImportContext context, OpenXmlElementBase childElement)
        {
            switch (childElement.ElementName)
            {
                case DocxElementNames.BookmarkStart:
                    BookmarkStartElement startElement = (BookmarkStartElement)childElement;
                    Bookmark bookmark = BookmarkContext.CreateBookmark(context.Document, startElement.Name, startElement.ColFirst, startElement.ColLast);
                    context.BookmarkContext.AddHangingBookmarkStart(bookmark.BookmarkRangeStart);
                    context.BookmarkContext.RegisterAnnotationToImport(startElement.Id, bookmark);
                    break;

                case DocxElementNames.BookmarkEnd:
                    BookmarkEndElement endElement = (BookmarkEndElement)childElement;
                    context.BookmarkContext.AddHangingBookmarkEndId(endElement.Id);
                    break;

                case DocxElementNames.PermissionStart:
                    PermissionStartElement permissionStartElement = (PermissionStartElement)childElement;
                    PermissionRange permission = PermissionContext.CreatePermission(context.Document, permissionStartElement);
                    context.PermissionContext.AddHangingPermissionStart(permission.Start);
                    context.PermissionContext.RegisterAnnotationToImport(permissionStartElement.Id, permission);
                    break;

                case DocxElementNames.PermissionEnd:
                    PermissionEndElement permissionEndElement = (PermissionEndElement)childElement;
                    context.PermissionContext.AddHangingPermissionEndId(permissionEndElement.Id);
                    break;
            }
        }
    }
}