﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Security;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation
{
    /// <summary>
    /// Represents invalid document exception.
    /// </summary>
    [Serializable]
    public class InvalidDocumentException : Exception
    {
        private readonly IEnumerable<ValidationError> validationErrors;

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidDocumentException" /> class.
        /// </summary>
        public InvalidDocumentException() : this(string.Empty)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidDocumentException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public InvalidDocumentException(string message) : this(message, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidDocumentException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="innerException">The inner exception.</param>
        public InvalidDocumentException(string message, Exception innerException) : base(message, innerException)
        {
            this.validationErrors = Enumerable.Empty<ValidationError>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidDocumentException" /> class.
        /// </summary>
        /// <param name="validationErrors">The validation errors.</param>
        public InvalidDocumentException(IEnumerable<ValidationError> validationErrors)
        {
            Guard.ThrowExceptionIfNull(validationErrors, "validationErrors");

            this.validationErrors = validationErrors;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidDocumentException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected InvalidDocumentException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.validationErrors = Enumerable.Empty<ValidationError>();
        }

        /// <summary>
        /// Gets the validation errors.
        /// </summary>
        /// <value>The validation errors.</value>
        public IEnumerable<ValidationError> ValidationErrors
        {
            get
            {
                return this.validationErrors;
            }
        }

        /// <summary>
        /// Provides COM objects with version-independent access to the <see cref="M:System.Exception.GetObjectData(System.Runtime.Serialization.SerializationInfo,System.Runtime.Serialization.StreamingContext)" />
        /// method.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// object that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext" />
        /// structure that contains contextual information about the source or destination.</param>
        [SecurityCritical]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
        }
    }
}
