﻿using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation
{
    /// <summary>
    /// Represents validation error class.
    /// </summary>
    public class ValidationError
    {
        private readonly DocumentElementBase documentElement;
        private readonly string message;

        internal ValidationError(DocumentElementBase documentElement, string message)
        {
            Guard.ThrowExceptionIfNull(documentElement, "documentElement");
            Guard.ThrowExceptionIfNullOrEmpty(message, "message");

            this.documentElement = documentElement;
            this.message = message;
        }

        /// <summary>
        /// Gets the error message.
        /// </summary>
        /// <value>The error message.</value>
        public string Message
        {
            get
            {
                return this.message;
            }
        }

        /// <summary>
        /// Gets the document element.
        /// </summary>
        /// <value>The document element.</value>
        public DocumentElementBase DocumentElement
        {
            get
            {
                return this.documentElement;
            }
        }
    }
}
