﻿using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation
{
    /// <summary>
    /// Represents validation result.
    /// </summary>
    public class ValidationResult
    {
        private readonly ValidationResultType resultType;
        private readonly IEnumerable<ValidationError> validationErrors;

        internal ValidationResult(IEnumerable<ValidationError> validationErrors)
            : this(ValidationResultType.Error, validationErrors)
        {
        }

        internal ValidationResult()
            : this(ValidationResultType.Success, Enumerable.Empty<ValidationError>())
        {
        }

        private ValidationResult(ValidationResultType resultType, IEnumerable<ValidationError> validationErrors)
        {
            this.resultType = resultType;
            this.validationErrors = validationErrors;
        }

        /// <summary>
        /// Gets the result type.
        /// </summary>
        /// <value>The result type.</value>
        public ValidationResultType ResultType
        {
            get
            {
                return this.resultType;
            }
        }

        /// <summary>
        /// Gets the validation errors.
        /// </summary>
        /// <value>The validation errors.</value>
        public IEnumerable<ValidationError> ValidationErrors
        {
            get
            {
                return this.validationErrors;
            }
        }
    }
}
