﻿using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation
{
    internal class ValidationRule<T> : IValidationRule
        where T : DocumentElementBase
    {
        private readonly Func<T, bool> predicate;
        private readonly string errorMessage;
        private readonly bool isDocumentChanging;

        public ValidationRule(Func<T, bool> predicate, string errorMessage, bool isDocumentStructureChanging)
        {
            Guard.ThrowExceptionIfNull(predicate, "predicate");
            Guard.ThrowExceptionIfNullOrEmpty(errorMessage, "errorMessage");

            this.predicate = predicate;
            this.errorMessage = errorMessage;
            this.isDocumentChanging = isDocumentStructureChanging;
        }

        public string ErrorMessage
        {
            get
            {
                return this.errorMessage;
            }
        }

        public bool IsDocumentChanging
        {
            get
            {
                return this.isDocumentChanging;
            }
        }

        public bool IsCompliant(DocumentElementBase documentElement)
        {
            Guard.ThrowExceptionIfNull(documentElement, "documentElement");

            return this.predicate((T)documentElement);
        }
    }
}
