﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Export
{
    internal class CssStyleWriter : CssWriter
    {
        private const string CssStyleSelectorNameFormat = ".{0}";
        private const string CssStyleSelectorForElementNameFormat = "{0}.{1}";
        private const string CssElementSelectorNameFormat = "{0}";
        private const string CssClassStartFormat = "{0} {{";
        private const string CssClassFromatEndFormat = "}";

        public CssStyleWriter()
        {
            this.Writer.AppendLine();
        }

        public void WriteStyleStart(string name, SelectorType type, string forElementType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            this.Writer.AppendLine(string.Format(CssClassStartFormat, BuildSelectorName(type, name, forElementType)));
        }

        public void WriteStyleEnd()
        {
            this.Writer.AppendLine(CssClassFromatEndFormat);
        }

        protected override void WriteStylePropertyOverride(string name, string value)
        {
            this.Writer.AppendLine();
        }

        private static string BuildSelectorName(SelectorType type, string name, string forElementType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            string result;
            switch (type)
            {
                case SelectorType.Elements:
                    result = string.Format(CssElementSelectorNameFormat, name);
                    break;

                case SelectorType.Style:
                    result = string.Format(CssStyleSelectorNameFormat, name);
                    break;

                case SelectorType.StyleForElements:
                    result = string.Format(CssStyleSelectorForElementNameFormat, forElementType, name);
                    break;

                default:
                    throw new NotSupportedException("Selector type is not supported.");
            }

            return result;
        }
    }
}
