﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Export
{
    internal class HtmlExporter
    {
        private readonly HtmlContentManager contentManager;

        public HtmlExporter()
        {
            this.contentManager = new HtmlContentManager();
        }

        public void Export(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            context.BeginExport();

            switch (context.Settings.DocumentExportLevel)
            {
                case DocumentExportLevel.Document:

                    this.ExportDocument(writer, context);

                    break;

                case DocumentExportLevel.Fragment:

                    this.ExportDocumentFragment(writer, context);

                    break;
            }

            context.EndExport();
        }

        private void ExportDocument(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.WriteDocumentStart();

            HtmlElement htmlElement = this.contentManager.CreateElement<HtmlElement>(HtmlElementNames.Html);
            htmlElement.Write(writer, context);
        }

        private void ExportDocumentFragment(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            BodyElement bodyElement = this.contentManager.CreateElement<BodyElement>(HtmlElementNames.Body);
            bodyElement.Write(writer, context);
        }
    }
}
