using System;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html
{
    /// <summary>
    /// Provides data for <see cref="HtmlExportSettings.ExternalStylesExporting" /> event.
    /// </summary>
    public class ExternalStylesExportingEventArgs : EventArgs
    {
        private readonly string css;

        internal ExternalStylesExportingEventArgs(string css)
        {
            this.css = css;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the event is handled.
        /// </summary>
        /// <value>
        ///   <c>true</c> if handled; otherwise, <c>false</c>.
        /// </value>
        public bool Handled { get; set; }

        /// <summary>
        /// Gets or sets the value which will be set to the 'href' attribute of the 'link' tag pointing to the external style file.
        /// </summary>
        /// <value>The reference.</value>
        public string Reference { get; set; }

        /// <summary>
        /// Gets the CSS that will be exported.
        /// </summary>
        /// <value>
        /// The CSS string.
        /// </value>
        public string Css
        {
            get
            {
                return this.css;
            }
        }
    }
}