﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html
{
    /// <summary>
    /// Contains settings for import with <see cref="HtmlFormatProvider"/>.
    /// </summary>
    public class HtmlImportSettings
    {
        private readonly GenericHtmlFonts genericFonts;

        /// <summary>
        /// Initializes a new instance of the <see cref="HtmlImportSettings" /> class.
        /// </summary>
        public HtmlImportSettings()
        {
            this.genericFonts = new GenericHtmlFonts();
            this.DefaultStyleSheet = HtmlConstants.DefaultStyleSheet;
        }

        /// <summary>
        /// Occurs when loading from URL.
        /// </summary>
        public event EventHandler<LoadFromUriEventArgs> LoadFromUri;

        /// <summary>
        /// Gets the generic fonts collection.
        /// </summary>
        /// <value>The generic fonts collection.</value>
        public GenericHtmlFonts GenericFonts
        {
            get
            {
                return this.genericFonts;
            }
        }

        /// <summary>
        /// Gets or sets whether non breaking space characters in strings should be replaced with whitespaces.
        /// <para>The default value is false.</para>
        /// </summary>
        /// <value>Indicates whether non breaking spaces should be replaced.</value>
        public bool ReplaceNonBreakingSpaces { get; set; }

        /// <summary>
        /// Gets or sets the default style sheet that will be used during the import.
        /// </summary>
        /// <value>The default style sheet that will be used during the import.</value>
        public string DefaultStyleSheet { get; set; }

        internal byte[] GetDataFromUri(string uri)
        {
            Guard.ThrowExceptionIfNullOrEmpty(uri, "uri");

            LoadFromUriEventArgs args = new LoadFromUriEventArgs(uri);
            this.OnLoadFromUri(args);

            byte[] data = args.GetData();

            return data;
        }

        private void OnLoadFromUri(LoadFromUriEventArgs args)
        {
            if (this.LoadFromUri != null)
            {
                this.LoadFromUri(this, args);
            }
        }
    }
}