using System;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html
{
    /// <summary>
    /// Provides data for <see cref="HtmlExportSettings.ImageExporting"/> event.
    /// </summary>
    public class ImageExportingEventArgs : EventArgs
    {
        private readonly Image image;

        internal ImageExportingEventArgs(Image imageInline)
        {
            Guard.ThrowExceptionIfNull(imageInline, "imageInline");

            this.image = imageInline;
            this.ExportSize = true;
        }

        /// <summary>
        /// Gets the image that will be exported.
        /// </summary>
        /// <value>The image.</value>
        public Image Image
        {
            get
            {
                return this.image;
            }
        }

        /// <summary>
        /// Gets or sets the value which indicates whether the 'width' and 'height' attributes of an 'image' element should be exported.
        /// <para>The default value is true.</para>
        /// </summary>
        /// <value>The value indicating whether the image size should be exported.</value>
        public bool ExportSize { get; set; }

        /// <summary>
        /// Gets or sets the value which will be set to the 'src' attribute of the 'image' element.
        /// </summary>
        /// <value>The source.</value>
        public string Source { get; set; }

        /// <summary>
        /// Gets or sets the value which will be set to the 'alt' attribute of the 'image' element.
        /// </summary>
        /// <value>The alternative text.</value>
        public string AlternativeText { get; set; }

        /// <summary>
        /// Gets or sets the 'title' attribute of the 'image' element.
        /// </summary>
        /// <value>The title.</value>
        public string Title { get; set; }

        /// <summary>
        /// Indicates if the event is handled.
        /// </summary>
        /// <value>True if the event is handled. False otherwise.</value>
        public bool Handled { get; set; }
    }
}