﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Import
{
    internal class HtmlImportContext : IHtmlImportContext
    {
        private readonly HtmlImportSettings settings;
        private readonly RadFlowDocumentImportEditor editor;
        private readonly ListImportContext listImportContext;

        public HtmlImportContext(HtmlImportSettings settings)
        {
            Guard.ThrowExceptionIfNull(settings, "settings");

            this.settings = settings;

            RadFlowDocument document = HtmlImportDocumentFactory.CreateDocumentForImport();
            this.editor = new RadFlowDocumentImportEditor(document, this.settings);
            this.listImportContext = new ListImportContext();
        }

        public RadFlowDocument Document
        {
            get
            {
                return this.editor.Document;
            }
        }

        public HtmlImportSettings Settings
        {
            get
            {
                return this.settings;
            }
        }

        public HtmlStyleRepository HtmlStyleRepository
        {
            get;
            set;
        }

        public StyleNamesInfo StyleNamesInfo
        {
            get;
            set;
        }

        public Paragraph CurrentParagraph
        {
            get
            {
                return this.editor.CurrentParagraph;
            }
        }

        public ListImportContext ListImportContext
        {
            get
            {
                return this.listImportContext;
            }
        }

        public bool ShouldImportNormalWeb
        {
            get;
            set;
        }

        public void BeginParagraph(Paragraph paragraph)
        {
            Guard.ThrowExceptionIfNull(paragraph, "paragraph");

            this.editor.BeginParagraph(paragraph);
        }

        public void EndParagraph()
        {
            this.editor.EndParagraph();
        }

        public void BeginHyperlink()
        {
            this.editor.BeginHyperlink();
        }

        public void EndHyperlink()
        {
            this.editor.EndHyperlink();
        }

        public void PushTable(Table table)
        {
            Guard.ThrowExceptionIfNull(table, "table");

            this.editor.PushTable(table);
        }

        public void PopTable()
        {
            this.editor.PopTable();
        }

        public void PushTableRow(TableRow row, RowType rowType)
        {
            Guard.ThrowExceptionIfNull(row, "row");

            this.editor.PushTableRow(row, rowType);
        }

        public void PopTableRow()
        {
            this.editor.PopTableRow();
        }

        public void InsertTableCell(TableCell cell, CellType cellType)
        {
            Guard.ThrowExceptionIfNull(cell, "cell");

            this.editor.InsertTableCell(cell, cellType);
        }

        public void InsertRun(Run run)
        {
            Guard.ThrowExceptionIfNull(run, "run");

            this.editor.InsertRun(run);
        }

        public void InsertInline(InlineBase inline)
        {
            Guard.ThrowExceptionIfNull(inline, "inline");

            this.editor.InsertInline(inline);
        }

        public void PopBlockContainer()
        {
            this.editor.PopBlockContainer();
        }

        public void BeginImport()
        {
        }

        public void EndImport()
        {
            StylesImporter.Import(this);

            this.editor.FinishDocument();
        }

        public bool HasCurrentParagraph()
        {
            return this.editor.HasCurrentParagraph();
        }
    }
}
