﻿using CsQuery;
using PreMailer.Net;
using System.IO;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Import
{
    internal class HtmlImporter
    {
        private const string StyleElementsQuery = "style, link";

        private readonly HtmlContentManager contentManager;

        public HtmlImporter()
        {
            this.contentManager = new HtmlContentManager();
        }

        public RadFlowDocument Import(Stream input, IHtmlImportContext context)
        {
            context.BeginImport();

            CQ cq = CQ.Create(input);
            this.ImportStyles(cq, context);
            this.ImportContent(cq, context);

            context.EndImport();

            return context.Document;
        }

        private void ImportStyles(CQ cq, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CssParser cssParser = new CssParser();
            cssParser.AddStyleSheet(context.Settings.DefaultStyleSheet);

            foreach (IDomElement element in cq.Select(StyleElementsQuery))
            {
                HtmlReader reader = new HtmlReader(element);
                string elementName = reader.GetCurrentChildElementName();
                HtmlElementBase styleElement = this.contentManager.CreateElement(elementName, false);
                styleElement.Read(reader, context);
                if (!string.IsNullOrEmpty(styleElement.InnerText))
                {
                    cssParser.AddStyleSheet(styleElement.InnerText);
                }
            }

            StyleNamesResolver styleNamesResolver = new StyleNamesResolver(cq);
            context.StyleNamesInfo = styleNamesResolver.Resolve();

            StyleEvaluator styleEvaluator = new StyleEvaluator(cq, cssParser.Styles, context.StyleNamesInfo);
            context.HtmlStyleRepository = styleEvaluator.Evaluate();
        }

        private void ImportContent(CQ cq, IHtmlImportContext context)
        {
            cq.Select(StyleElementsQuery).Remove();
            HtmlReader reader = new HtmlReader(cq);
            if (reader.HasChildElements())
            {
                do
                {
                    string elementName = reader.GetCurrentChildElementName();
                    HtmlElementBase root = this.contentManager.CreateElement(elementName, true);
                    if (root != null)
                    {
                        root.Read(reader, context);
                    }
                }
                while (reader.MoveToNextChildElement());
            }
        }
    }
}
