﻿using CsQuery;
using System.Collections.Generic;
using System.Diagnostics;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Import
{
    internal class HtmlReader : IHtmlReader
    {
        private readonly Stack<IDomObject> domObjectsStack;

        private bool isInSpan;
        private IDomObject domObject;

        private IEnumerator<KeyValuePair<string, string>> attribute;

        public HtmlReader(IDomObject domObject)
        {
            this.domObject = domObject;

            this.domObjectsStack = new Stack<IDomObject>();
        }

        public HtmlReader(CQ cq)
            : this(GetFirstDomObject(cq))
        {
        }

        private IDomObject CurrentObject
        {
            get
            {
                return this.domObjectsStack.Peek();
            }
        }

        public bool BeginReadElement()
        {
            if (this.domObject != null)
            {
                if (this.domObject.NodeName.ToLowerInvariant() == HtmlElementNames.Span)
                {
                    this.isInSpan = !this.isInSpan;
                }

                this.SetCurrentDomObject(this.domObject);

                return true;
            }

            return false;
        }

        public bool MoveToNextAttribute()
        {
            if (this.attribute == null)
            {
                return false;
            }

            return this.attribute.MoveNext();
        }

        public bool IsInsideSpanElement()
        {
            return this.isInSpan;
        }

        public bool HasChildElements()
        {
            return this.domObject != null;
        }

        public HtmlElementType GetCurrentChildElementType()
        {
            if (IsTextElement(this.domObject))
            {
                return HtmlElementType.Text;
            }

            return HtmlElementType.Element;
        }

        public string GetCurrentChildElementName()
        {
            return this.domObject.NodeName.ToLowerInvariant();
        }

        public string GetAttributeName()
        {
            return this.attribute.Current.Key.ToLowerInvariant();
        }

        public string GetAttributeValue()
        {
            return this.attribute.Current.Value;
        }

        public string GetInnerText()
        {
            if (IsTextElement(this.CurrentObject))
            {
                return this.CurrentObject.NodeValue;
            }

            return string.Empty;
        }

        public bool MoveToNextChildElement()
        {
            if (this.domObject != null)
            {
                this.domObject = this.domObject.NextSibling;
            }

            return this.domObject != null;
        }

        public string GetElementName()
        {
            return this.CurrentObject.NodeName.ToLowerInvariant();
        }

        public void EndReadElement()
        {
            this.domObject = this.domObjectsStack.Pop();

            if (this.domObject.NodeName.ToLowerInvariant() == HtmlElementNames.Span)
            {
                Debug.Assert(this.isInSpan, "We get into a span at the end of the element's read.");

                this.isInSpan = !this.isInSpan;
            }
        }

        private static bool IsTextElement(IDomObject element)
        {
            return element != null && element.NodeType == NodeType.TEXT_NODE;
        }

        private static IDomObject GetFirstDomObject(CQ dom)
        {
            IDomObject first = dom.Document.FirstChild;
            while (first != null && first.NodeType == NodeType.DOCUMENT_TYPE_NODE)
            {
                first = first.NextSibling;
            }

            return first;
        }

        private void SetCurrentDomObject(IDomObject node)
        {
            Guard.ThrowExceptionIfNull(node, "node");

            this.domObjectsStack.Push(node);
            if (this.domObject.HasAttributes)
            {
                this.attribute = this.domObject.Attributes.GetEnumerator();
            }
            else
            {
                this.attribute = null;
            }

            this.domObject = this.domObject.FirstChild;
        }
    }
}