﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        /// <summary>
        /// Given a table header or cell, returns all members of the same column in the table. This will
        /// most likely not work as you would expect if there are colspan cells.
        /// </summary>
        ///
        /// <returns>
        /// A new CQ object containing all the th and td cells in the specified column.
        /// </returns>

        public CQ GetTableColumn()
        {
            var els = this.Filter("th,td");
            CQ result = NewCqInDomain();
            foreach (var el in els)
            {
                var elCq = el.Cq();
                int colIndex = elCq.Index();
                result.AddSelection(elCq.Closest("table").GetTableColumn(colIndex));
            }
            return result;
        }

        /// <summary>
        /// Selects then zero-based nth cells  (th and td) from all rows in any matched tables. This will
        /// most likely no do what you expect if the table has colspan cells.
        /// </summary>
        ///
        /// <param name="column">
        /// The zero-based index of the column to target.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object containing all the th and td cells in the specified column.
        /// </returns>

        public CQ GetTableColumn(int column)
        {
            CQ result = NewCqInDomain();
            foreach (var el in FilterElements(this, "table"))
            {

                result.AddSelection(el.Cq().Find(String.Format("tr>th:eq({0}), tr>td:eq({0})", column)));

            }
            return result;
        }
    }
}
