﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Dynamic;
using System.IO;

using CsQuery.ExtensionMethods.Internal;
using CsQuery.Utility;
using CsQuery.Engine;
using CsQuery.Promises;
using CsQuery.HtmlParser;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        #region CsQuery global options

        /// <summary>
        /// DEPRECATED. Please use CsQuery.Config.DomRenderingOptions. 
        /// </summary>
        
        [Obsolete]
        public static DomRenderingOptions DefaultDomRenderingOptions 
        {
            get {
                return Config.DomRenderingOptions;
            }
            set {
                Config.DomRenderingOptions = value;
            }
        }

        /// <summary>
        /// DEPRECATED. Please use CsQuery.Config.DocType
        /// </summary>
        
        [Obsolete]
        public static DocType DefaultDocType {
            get
            {
                return Config.DocType;
            }
            set
            {
                Config.DocType = value;
            }
        }

        #endregion 

        #region private properties
        
        #endregion

        #region static utility methods

        /// <summary>
        /// Convert a dictionary to a dynamic object. Use to get another expando object from a sub-
        /// object of an expando object, e.g. as returned from JSON data.
        /// </summary>
        ///
        /// <param name="obj">
        /// The object.
        /// </param>
        ///
        /// <returns>
        /// obj as a JsObject.
        /// </returns>

        public static JsObject ToExpando(object obj)
        {
            JsObject result;


            if (obj is IDictionary<string, object>)
            {
                result = Objects.Dict2Dynamic<JsObject>((IDictionary<string, object>)obj);
            }
            else
            {
                return Objects.ToExpando(obj);
            }
            return result;
        }

        /// <summary>
        /// Converts an object to a dynamic object of type T.
        /// </summary>
        ///
        /// <typeparam name="T">
        /// The type of object to create. This must be an IDynamicMetaObjectProvider that also implements
        /// IDictionary&lt;string,object&gt;
        /// </typeparam>
        /// <param name="obj">
        /// The object.
        /// </param>
        ///
        /// <returns>
        /// A new object of type T.
        /// </returns>

        public static T ToDynamic<T>(object obj) where T : IDynamicMetaObjectProvider, IDictionary<string,object>,new()
        {
            if (obj is IDictionary<string, object>)
            {
                return Objects.Dict2Dynamic<T>((IDictionary<string, object>)obj);
            }
            else
            {
                return Objects.ToExpando<T>(obj);
            }
        }
        
        #endregion

    }
}
