﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        #region public methods

        /// <summary>
        /// Get the descendants of each element in the current set of matched elements, filtered by a
        /// selector.
        /// </summary>
        ///
        /// <param name="selector">
        /// A string containing a selector expression to match elements against.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/find/
        /// </url>

        public CQ Find(string selector)
        {
            return FindImpl(new Selector(selector));
        }

        /// <summary>
        /// Get the descendants of each element in the current set of matched elements, filtered by a
        /// sequence of elements or CQ object.
        /// </summary>
        ///
        /// <param name="elements">
        /// The elements to match against.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/find/
        /// </url>

        public CQ Find(IEnumerable<IDomObject> elements)
        {
            return FindImpl(new Selector(elements));
        }

        /// <summary>
        /// Get a single element, if it is a descendant of the current selection set.
        /// </summary>
        ///
        /// <param name="element">
        /// The element to matc.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/find/
        /// </url>

        public CQ Find(IDomObject element)
        {
            return FindImpl(new Selector(element));
        }

        #endregion

        #region private methods

        private CQ FindImpl(Selector selector)
        {
            CQ csq = NewCqInDomain();
            var selection = selector.ToContextSelector().Select(Document, this);

            csq.AddSelection(selection);
            csq.Selector = selector;
            return csq;
        }

        #endregion

    }
}
