﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        /// <summary>
        /// Reduce the set of matched elements to those that have a descendant that matches the selector
        /// or DOM element.
        /// </summary>
        ///
        /// <param name="selector">
        /// A valid CSS/jQuery selector.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/has/
        /// </url>

        public CQ Has(string selector)
        {
            var csq = NewCqInDomain();

            foreach (IDomObject obj in SelectionSet)
            {
                if (Select(obj).Find(selector).Length > 0)
                {
                    csq.SelectionSet.Add(obj);
                }
            }
            return csq;
        }

        /// <summary>
        /// Reduce the set of matched elements to those that have the element passed as a descendant.
        /// </summary>
        ///
        /// <param name="element">
        /// The element to match.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/has/
        /// </url>

        public CQ Has(IDomObject element)
        {
            return Has(Objects.Enumerate(element));
        }

        /// <summary>
        /// Reduce the set of matched elements to those that have each of the elements passed as a descendant.
        /// </summary>
        ///
        /// <param name="elements">
        /// The elements to be excluded.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/has/
        /// </url>

        public CQ Has(IEnumerable<IDomObject> elements)
        {
            var csq = NewCqInDomain();
            foreach (IDomObject obj in SelectionSet)
            {
                if (obj.Cq().Find(elements).Length > 0)
                {
                    csq.SelectionSet.Add(obj);
                }
            }
            return csq;
        }

    }
}
