﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        /// <summary>
        /// Map each element of the result set to a new form. If a value is returned from the function,
        /// the element will be excluded.
        /// </summary>
        ///
        /// <typeparam name="T">
        /// .
        /// </typeparam>
        /// <param name="elements">
        /// .
        /// </param>
        /// <param name="function">
        /// .
        /// </param>
        ///
        /// <returns>
        /// An enumerator that allows foreach to be used to process map&lt; t&gt; in this collection.
        /// </returns>

        public static IEnumerable<T> Map<T>(IEnumerable<IDomObject> elements, Func<IDomObject, T> function)
        {
            foreach (var element in elements)
            {
                T result = function(element);
                if (result != null)
                {
                    yield return result;
                }
            }
        }

        /// <summary>
        /// Map each element of the result set to a new form. If a value is returned from the function,
        /// the element will be excluded.
        /// </summary>
        ///
        /// <typeparam name="T">
        /// Generic type parameter.
        /// </typeparam>
        /// <param name="function">
        /// .
        /// </param>
        ///
        /// <returns>
        /// An enumerator that allows foreach to be used to process map&lt; t&gt; in this collection.
        /// </returns>

        public IEnumerable<T> Map<T>(Func<IDomObject, T> function)
        {
            return CQ.Map(this, function);
        }

    }
}
