﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;

namespace CsQuery
{
    internal partial class CQ
    {

        /// <summary>
        /// Get the ancestors of each element in the current set of matched elements, optionally filtered
        /// by a selector.
        /// </summary>
        ///
        /// <param name="filter">
        /// (optional) a selector which limits the elements returned.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/parents/
        /// </url>

        public CQ Parents(string filter = null)
        {
            return ParentsUntil((string)null, filter);
        }

        private static IEnumerable<IDomElement> ParentsImpl(IEnumerable<IDomObject> source, HashSet<IDomElement> until)
        {

            HashSet<IDomElement> alreadyAdded = new HashSet<IDomElement>();

            foreach (var item in source)
            {
                IDomElement parent = item.ParentNode as IDomElement;
                while (parent != null && !until.Contains(parent))
                {
                    if (alreadyAdded.Add(parent))
                    {
                        yield return parent;
                    }
                    else
                    {
                        break;
                    }

                    parent = parent.ParentNode as IDomElement;
                }
            }
        }
    }
}
