﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Implementation;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;

namespace CsQuery
{
    internal partial class CQ
    {

        /// <summary>
        /// Get the ancestors of each element in the current set of matched elements, up to but not
        /// including any element matched by the selector, optionally filtered by another selector.
        /// </summary>
        ///
        /// <param name="selector">
        /// A string containing a selector expression to match elements against.
        /// </param>
        /// <param name="filter">
        /// (optional) a selector which limits the elements returned.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/parentsUntil/
        /// </url>

        public CQ ParentsUntil(string selector = null, string filter = null)
        {
            return ParentsUntil(Select(selector).Elements,filter);
        }

        /// <summary>
        /// Get the ancestors of each element in the current set of matched elements, up to but not
        /// including the element matched by the selector.
        /// </summary>
        ///
        /// <param name="element">
        /// The element.
        /// </param>
        /// <param name="filter">
        /// (optional) a selector which limits the elements returned.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/parentsUntil/
        /// </url>
        /// 
        public CQ ParentsUntil(IDomElement element, string filter = null)
        {
            return ParentsUntil(Objects.Enumerate(element),filter);
        }

        /// <summary>
        /// Get the ancestors of each element in the current set of matched elements, up to but not
        /// including any element matched by the selector, optionally filtered by another selector.
        /// </summary>
        ///
        /// <param name="elements">
        /// The elements.
        /// </param>
        /// <param name="filter">
        /// (optional) a selector which limits the elements returned.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>

        public CQ ParentsUntil(IEnumerable<IDomElement> elements, string filter = null)
        {

            CQ output = NewCqInDomain();
            HashSet<IDomElement> targets = new HashSet<IDomElement>(elements);

            var filtered = FilterElementsIgnoreNull(ParentsImpl(Selection, targets), filter);
            
            output.SetSelection(filtered,
                SelectionSetOrder.OrderAdded,
                SelectionSetOrder.Descending);

            return output;
        }

       
    }
}
