﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {


        /// <summary>
        /// Insert every element in the set of matched elements to the beginning of the target.
        /// </summary>
        ///
        /// <param name="target">
        /// One or more HTML strings that will be targeted.
        /// </param>
        ///
        /// <returns>
        /// A CQ object containing all the elements added
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/prependTo/
        /// </url>

        public CQ PrependTo(params string[] target)
        {

            CQ output;
            NewInstance(MergeSelections(target)).Prepend(SelectionSet, out output);

            return output;
        }

        /// <summary>
        /// Insert every element in the set of matched elements to the beginning of the target.
        /// </summary>
        ///
        /// <param name="targets">
        /// The targets to which the current selection will be appended.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object representing the target elements.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/prependTo/
        /// </url>

        public CQ PrependTo(IEnumerable<IDomObject> targets)
        {
            CQ output;
            EnsureCsQuery(targets).Prepend(SelectionSet, out output);
            return output;
        }
        

    }
}
