﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Utility;
using CsQuery.ExtensionMethods;
using CsQuery.ExtensionMethods.Internal;
using CsQuery.Engine;
using CsQuery.Implementation;

namespace CsQuery
{
    internal partial class CQ
    {
        /// <summary>
        /// Reduce the set of matched elements to a subset beginning with the 0-based index provided.
        /// </summary>
        ///
        /// <param name="start">
        /// The 0-based index at which to begin selecting.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/slice/
        /// </url>

        public CQ Slice(int start)
        {
            return Slice(start, SelectionSet.Count);
        }

        /// <summary>
        /// Reduce the set of matched elements to a subset specified by a range of indices.
        /// </summary>
        ///
        /// <param name="start">
        /// The 0-based index at which to begin selecting.
        /// </param>
        /// <param name="end">
        /// The 0-based index of the element at which to stop selecting. The actual element at this
        /// position is not included in the result.
        /// </param>
        ///
        /// <returns>
        /// A new CQ object.
        /// </returns>
        ///
        /// <url>
        /// http://api.jquery.com/slice/
        /// </url>

        public CQ Slice(int start, int end)
        {
            if (start < 0)
            {
                start = SelectionSet.Count + start;
                if (start < 0) { start = 0; }
            }
            if (end < 0)
            {
                end = SelectionSet.Count + end;
                if (end < 0) { end = 0; }
            }
            if (end >= SelectionSet.Count)
            {
                end = SelectionSet.Count;
            }

            CQ output = NewCqInDomain();

            for (int i = start; i < end; i++)
            {
                output.SelectionSet.Add(SelectionSet[i]);
            }

            return output;
        }

        

    }
}
