﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery
{
    /// <summary>
    /// Flags specifying how the document should be rendered
    /// </summary>

    [Flags]
    internal enum DomRenderingOptions
    {
        /// <summary>
        /// No option flags. This is not the same as "default", but rather explicitly uses "false" values for all flags.
        /// </summary>
        None=0,

        /// <summary>
        /// Render with default options as determined by CsQuery.Config.DomRenderingOptions
        /// </summary>
        Default = 32,

        /// <summary>
        /// This option only appies to the old HTML parser. It is obsolete, has no effect, and will be
        /// removed in a future version of CsQuery.
        /// </summary>
        
        [Obsolete]
        RemoveMismatchedCloseTags = 1,
        
        /// <summary>
        /// Remove comments from the output
        /// </summary>
        
        RemoveComments = 2,
        
        /// <summary>
        /// Add quotes around each attribute value, whether or not they are needed. The alternative is to only 
        /// use quotes when they are necesssary to delimit the value (e.g. because it includes spaces or other quote characters)
        /// </summary>
        
        QuoteAllAttributes = 4



    }
 
}
