﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery
{   
    /// <summary>
    /// A FORM element.
    /// </summary>
    ///
    /// <url>
    /// http://dev.w3.org/html5/spec/single-page.html#the-form-element
    /// </url>

    internal interface IHTMLFormElement : IDomElement, INodeList<IDomElement>
    {
        /// <summary>
        /// The accept-charset content attribute.
        /// </summary>

        string AcceptCharset {get;set;}

        /// <summary>
        /// The action attribute
        /// </summary>

        string Action {get;set;}

        /// <summary>
        /// The automcomplete attribute
        /// </summary>

        string Autocomplete {get;set;}

        /// <summary>
        /// Gets or sets the enctype.
        /// </summary>

        string Enctype {get;set;}

        /// <summary>
        /// Gets or sets the encoding.
        /// </summary>

        string Encoding {get;set;}

        /// <summary>
        /// Gets or sets the method attribute.
        /// </summary>

        string Method {get;set;}

        /// <summary>
        /// Gets or sets a value indicating whether the no validate.
        /// </summary>

        bool NoValidate {get;set;}

        /// <summary>
        /// Gets or sets the target attribute
        /// </summary>

        string Target {get;set;}

        /// <summary>
        /// An INodeList containing the form elements.
        /// </summary>

        INodeList<IDomElement> Elements {get;}
        
        // could be implemented?
        // 
        //bool CheckValiditry()
        //void Reset();
        
        // no
        // 
        //void Submit();
    }
}
