﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery
{   
    /// <summary>
    /// An PROGRESS element
    /// </summary>
    ///
    /// <url>
    /// http://dev.w3.org/html5/spec/the-meter-element.html#the-meter-element
    /// </url>

    internal interface IHTMLMeterElement : IDomElement
    {
        /// <summary>
        /// The current value
        /// </summary>

        new int Value {get;set;}

        /// <summary>
        /// The maximum value
        /// </summary>

        double Min{ get; set; }

        /// <summary>
        /// The maximum value
        /// </summary>

        double Max {get;set;}

        /// <summary>
        /// The low value
        /// </summary>

        double Low { get; set; }

        /// <summary>
        /// The high value
        /// </summary>

        double High { get; set; }

        /// <summary>
        /// The optimum value
        /// </summary>

        double Optimum { get; set; }

        /// <summary>
        ///  A NodeList of all LABEL elements within this Progress element
        /// </summary>

        INodeList<IDomElement> Labels {get;}
    }
}
