﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery
{   
    /// <summary>
    /// A SELECT element
    /// </summary>

    internal interface IHTMLSelectElement : IDomElement, IFormSubmittableElement, IFormReassociateableElement
    {
        /// <summary>
        /// A collection of HTML option elements (in document order)
        /// </summary>
        /// <url>https://developer.mozilla.org/en/DOM/HTMLOptionsCollection</url>

        IHTMLOptionsCollection Options { get; }

        /// <summary>
        /// Returns the index of the currently selected item. You may select an item by assigning its
        /// index to this property. By assigning -1 to this property, all items will be deselected.
        /// Returns -1 if no items are selected.
        /// </summary>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/XUL/Property/selectedIndex.
        /// </url>

        int SelectedIndex { get; set; }

        /// <summary>
        /// Holds the currently selected item. If no item is currently selected, this value will be null.
        /// You can select an item by setting this value. A select event will be sent to the container
        /// (i.e. the listbox, richlistbox, etc., not the list item that was selected) when it is changed
        /// either via this property, the selectedIndex property, or changed by the user.
        /// </summary>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/XUL/Property/selectedItem
        /// </url>

        IDomElement SelectedItem { get; set; }

        /// <summary>
        /// This Boolean attribute indicates that multiple options can be selected in the list. If it is
        /// not specified, then only one option can be selected at a time.
        /// </summary>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/HTML/Element/select
        /// </url>

        bool Multiple { get; set; }

        /// <summary>
        /// Gets the number of options in the select
        /// </summary>

        int Length { get; }
    }
}
