﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.CompilerServices;

namespace CsQuery
{
    /// <summary>
    /// Interface to a collection of HTML options.
    /// </summary>
    ///
    /// <url>
    /// https://developer.mozilla.org/en/DOM/HTMLOptionsCollection
    /// </url>

    internal interface IHTMLOptionsCollection : IEnumerable<IDomObject>
    {
        /// <summary>
        /// Returns the specific node at the given zero-based index (gives null if out of range)
        /// </summary>
        ///
        /// <param name="index">
        /// The zero-based index of the option element.
        /// </param>
        ///
        /// <returns>
        /// An HTML Option element.
        /// </returns>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/DOM/HTMLOptionsCollection
        /// </url>

        IDomElement Item(int index);

        /// <summary>
        /// Returns the specific node at the given zero-based index (gives null if out of range)
        /// </summary>
        ///
        /// <param name="index">
        /// The zero-based index of the option element.
        /// </param>
        ///
        /// <returns>
        /// An HTML Option element.
        /// </returns>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/DOM/HTMLOptionsCollection
        /// </url>

        [IndexerName("Indexer")]
        IDomElement this[int index] { get; }

        /// <summary>
        /// Returns the specific node with the given DOMString (i.e., string) id. Returns null if no such named node exists.
        /// </summary>
        ///
        /// <param name="name">
        /// The zero-based index of the option element.
        /// </param>
        ///
        /// <returns>
        /// An HTML Option element.
        /// </returns>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/DOM/HTMLOptionsCollection
        /// </url>
        
        IDomElement NamedItem(string name);

        
        /// <summary>
        /// Returns the specific node with the given DOMString (i.e., string) id. Returns null if no such named node exists.
        /// </summary>
        ///
        /// <param name="name">
        /// The zero-based index of the option element.
        /// </param>
        ///
        /// <returns>
        /// An HTML Option element.
        /// </returns>
        ///
        /// <url>
        /// https://developer.mozilla.org/en/DOM/HTMLOptionsCollection
        /// </url>

        [IndexerName("Indexer")]
        IDomElement this[string name] {get;}
        
    }
}
