﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery
{
    /// <summary>
    /// Interface for methods to access the attributes on a DOM element.
    /// </summary>

    internal interface IAttributeCollection : IEnumerable<KeyValuePair<string, string>>
    {
        /// <summary>
        /// Get the value of a named attribute
        /// </summary>
        /// <param name="name">The attribute name</param>
        /// <returns>The attribute value</returns>
        string GetAttribute(string name);

        /// <summary>
        /// Set the value of a named attribute
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        void SetAttribute(string name, string value);

        /// <summary>
        /// Get or set the value of a named attribute
        /// </summary>
        /// <param name="attributeName">The attribute name</param>
        /// <returns>The attribute value</returns>
        /// <returntype>string</returntype>
        string this[string attributeName] { get; set; }

        /// <summary>
        /// The number of attributes in this attribute collection. This includes special attributes such as
        /// "class", "id", and "style"
        /// </summary>
        /// <returntype>int</returntype>
        int Length { get; }
    }
}
