﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.Implementation
{

    /// <summary>
    /// Interface for icss rule.
    /// </summary>
    ///
    /// <url>
    /// http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule
    /// </url>

    internal abstract class CSSRule : ICSSRule
    {
        /// <summary>
        /// Constructor for a CSS rule.
        /// </summary>
        ///
        /// <param name="parentStyleSheet">
        /// The parent style sheet.
        /// </param>
        /// <param name="parentRule">
        /// The parent rule.
        /// </param>

        public CSSRule(ICSSStyleSheet parentStyleSheet, ICSSRule parentRule)
        {
            ParentStyleSheet = parentStyleSheet;
            ParentRule = parentRule;
        }
        /// <summary>
        /// Gets the type of rule.
        /// </summary>

        public CSSRuleType Type
        {
            get;
            set;
        }

        /// <summary>
        /// The parsable textual representation of the rule. This reflects the current state of the rule
        /// and not its initial value.
        /// </summary>

        public abstract string CssText { get; set; }

        /// <summary>
        /// The style sheet that contains this rule.
        /// </summary>
        ///
        /// <value>
        /// The parent style sheet.
        /// </value>

        public ICSSStyleSheet ParentStyleSheet
        {
            get;
            protected set;
        }

        /// <summary>
        /// If this rule is contained inside another rule (e.g. a style rule inside an @media block),
        /// this is the containing rule. If this rule is not nested inside any other rules, this returns
        /// null.
        /// </summary>
        ///
        /// <value>
        /// The parent rule.
        /// </value>

        public ICSSRule ParentRule
        {
            get;
            protected set;
        }
    }
}
