﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.IO;
using CsQuery.HtmlParser;
using CsQuery.Output;

namespace CsQuery.Implementation
{
    /// <summary>
    /// Used for literal text (not part of a tag)
    /// </summary>
    internal class DomText : DomObject<DomText>, IDomText
    {
        /// <summary>
        /// Create a new empty Text node
        /// </summary>

        public DomText()
            : base()
        {
            
        }

        /// <summary>
        /// Create a new Text node containing the text passed
        /// </summary>
        ///
        /// <param name="nodeValue">
        /// The text value of this Text node.
        /// </param>

        public DomText(string nodeValue)
            : base()
        {
            NodeValue = nodeValue;
        }


        /// <summary>
        /// The inner node value; the text.
        /// </summary>

        protected string _NodeValue;
       

        /// <summary>
        /// The node (tag) name, in upper case. For Text nodes, this is always "#text".
        /// </summary>

        public override string NodeName
        {
            get
            {
                return "#text";
            }
        }

        /// <summary>
        /// Gets the type of the node. For Text nodes, this is always NodeType.TEXT_NODE
        /// </summary>

        public override NodeType NodeType
        {
            get { return NodeType.TEXT_NODE; }
        }



        /// <summary>
        /// Gets or sets the text value of this Text node. Null values will be converted to an empty string.
        /// </summary>

        public override string NodeValue
        {
            get
            {
                return _NodeValue ?? "";
            }
            set
            {
                _NodeValue=value;
            }
        }

        /// <summary>
        /// Makes a clone of this TextNode
        /// </summary>
        ///
        /// <returns>
        /// A copy of this object.
        /// </returns>

        public override DomText Clone()
        {
            return new DomText(NodeValue);
        }

        /// <summary>
        /// Gets a value indicating whether HTML is allowed as a child of this element. For Text nodes,
        /// this is always false.
        /// </summary>

        public override bool InnerHtmlAllowed
        {
            get { return false; }
        }

        /// <summary>
        /// For Text nodes, this is always false
        /// </summary>

        public override bool HasChildren
        {
            get { return false; }
        }

        /// <summary>
        /// Return the value of this text node
        /// </summary>
        ///
        /// <returns>
        /// This object as a string.
        /// </returns>

        public override string ToString()
        {
            return NodeValue;
        }

    }
}
