﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.HtmlParser;
using CsQuery.Implementation;

namespace CsQuery.Implementation
{
    /// <summary>
    /// An implementation of properties shared by all form associated elements.
    /// </summary>
    /// 
    /// <url>
    /// http://www.w3.org/html/wg/drafts/html/master/forms.html#form-associated-element
    /// </url>
    internal abstract class FormAssociatedElement : DomElement, IFormAssociatedElement
    {
        /// <summary>
        /// Constructor to specify the element's token ID.
        /// </summary>
        /// <param name="tokenId">The token ID of the element.</param>
        protected FormAssociatedElement(ushort tokenId)
            : base(tokenId)
        {
        }

        /// <summary>
        /// The value of form element with which to associate the element.
        /// </summary>
        ///
        /// <remarks>
        /// The HTML5 spec says "The value of the id attribute on the form with which to associate the
        /// element." This is not what browsers currently return; they return the actual element. We'll
        /// keep that for now.
        /// </remarks>

        public IHTMLFormElement Form
        {
            get
            {
                return Closest(HtmlData.tagFORM) as IHTMLFormElement;
            }
        }
    }
}
