﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.HtmlParser;
using CsQuery.Utility;

namespace CsQuery.Implementation
{
    /// <summary>
    /// An Anchor (A) element.
    /// </summary>
    ///
    /// <url>
    /// http://dev.w3.org/html5/spec/single-page.html#the-a-element
    /// </url>
    internal class HtmlAnchorElement : DomElement, IHTMLAnchorElement
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HtmlAnchorElement()
            : base(HtmlData.tagA)
        {
        }

        /// <summary>
        /// A name or keyword giving a browsing context for UAs to use when following the hyperlink.
        /// </summary>

        public string Target
        {
            get
            {
                return GetAttribute("target");
            }
            set
            {
                SetAttribute("target", value);
            }
        }

        /// <summary>
        /// A URL that provides the destination of the hyperlink. If the href attribute is not specified,
        /// the element represents a placeholder hyperlink.
        /// </summary>

        public string Href
        {
            get
            {
                return GetAttribute("href");
            }
            set
            {
                SetAttribute("href",value);
            }
        }

        /// <summary>
        /// The rel attribute on a and area elements controls what kinds of links the elements create.
        /// The attribue's value must be a set of space-separated tokens.
        /// </summary>

        public RelAnchor Rel
        {
            get
            {
                return Support.AttributeToEnum<RelAnchor>(GetAttribute("rel"));
            }
            set
            {
                SetAttribute("rel", Support.EnumToAttribute(value));
            }
        }

        /// <summary>
        /// A list of tokens that specify the relationship between the document containing the hyperlink
        /// and the destination indicated by the hyperlink.
        /// </summary>

        public string Hreflang
        {
            get
            {
                return GetAttribute("hreflang");
            }
            set
            {
                SetAttribute("hreflang", value);
            }
        }

        /// <summary>
        /// The media for which the destination of the hyperlink was designed.
        /// </summary>

        public string Media
        {
            get
            {
                return GetAttribute("media");
            }
            set
            {
                SetAttribute("media", value);
            }
        }
    }
}
