﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using CsQuery.HtmlParser;

namespace CsQuery.Implementation
{   
    /// <summary>
    /// A LABEL element.
    /// </summary>
    ///
    /// <url>
    /// http://dev.w3.org/html5/spec/single-page.html#the-label-element
    /// </url>

    internal class HTMLLabelElement : FormAssociatedElement, IHTMLLabelElement
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HTMLLabelElement()
            : base(HtmlData.tagLABEL)
        {
        }

        /// <summary>
        /// Gets or sets the for attribute
        /// </summary>

        public string HtmlFor 
        {
            get
            {
                return GetAttribute("for");   
            }
            set{
                SetAttribute("for",value);
            }
        }

        /// <summary>
        /// The control bound to this label. If the "for" attribute is set, this is the control with that
        /// ID. If not, the first input control that is a child of the label will be returned.
        /// </summary>

        public IDomElement Control {
            get
            {
                var id = HtmlFor;
                if (!String.IsNullOrEmpty(id))
                {
                    return Document.GetElementById(id);
                }
                else
                {
                    foreach (var el in DescendantElements())
                    {
                        if (HtmlData.IsFormInputControl(el.NodeNameID))
                        {
                            return el;
                        }
                    }
                    return null;
                }
            }
        }
    }
}
