﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.HtmlParser;
using CsQuery.Utility;

namespace CsQuery.Implementation
{
    /// <summary>
    /// An HTML progress element.
    /// </summary>

    internal class HTMLMeterElement : DomElement, IHTMLMeterElement
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HTMLMeterElement()
            : base(HtmlData.tagMETER)
        {
        }

        /// <summary>
        /// The value of the meter
        /// </summary>

        public new int Value
        {
            get
            {
                return Support.IntOrZero(GetAttribute(HtmlData.ValueAttrId));
            }
            set
            {
                SetAttribute(HtmlData.ValueAttrId, value.ToString());
            }
        }

        /// <summary>
        /// The maximum value.
        /// </summary>

        public double Max
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("max"));
            }
            set
            {
                SetAttribute("max", value.ToString());
            }
        }

        /// <summary>
        /// The minimum value.
        /// </summary>

        public double Min
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("min"));
            }
            set
            {
                SetAttribute("min", value.ToString());
            }
        }
        /// <summary>
        /// The low value.
        /// </summary>

        public double Low
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("low"));
            }
            set
            {
                SetAttribute("low", value.ToString());
            }
        }

        /// <summary>
        /// The high value.
        /// </summary>

        public double High
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("high"));
            }
            set
            {
                SetAttribute("high", value.ToString());
            }
        }

        /// <summary>
        /// The optimum value.
        /// </summary>

        public double Optimum
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("optimum"));
            }
            set
            {
                SetAttribute("optimum", value.ToString());
            }
        }
       

        /// <summary>
        /// A NodeList of all LABEL elements within this Progress element
        /// </summary>

        public INodeList<IDomElement> Labels
        {
            get {
                return new NodeList<IDomElement>(ChildElementsOfTag<IDomElement>(HtmlData.tagLABEL));
            }
        }


     
    }
}
