﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.HtmlParser;
using CsQuery.Utility;

namespace CsQuery.Implementation
{
    /// <summary>
    /// An HTML progress element.
    /// </summary>

    internal class HTMLProgressElement : DomElement, IHTMLProgressElement
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HTMLProgressElement()
            : base(HtmlData.tagPROGRESS)
        {
        }

        /// <summary>
        /// For Progress elements, returns the value of the "value" attribute, or zero.
        /// </summary>

        public new int Value
        {
            get
            {
                return Support.IntOrZero(GetAttribute(HtmlData.ValueAttrId));
            }
            set
            {
                SetAttribute(HtmlData.ValueAttrId, value.ToString());
            }
        }

        /// <summary>
        /// The maximum value allowed for this Progress bar.
        /// </summary>

        public double Max
        {
            get
            {
                return Support.DoubleOrZero(GetAttribute("max"));
            }
            set
            {
                SetAttribute("max", value.ToString());
            }
        }

        ///  <summary>
        /// If the progress bar is an indeterminate progress bar, then the position IDL attribute must
        /// return −1. Otherwise, it must return the result of dividing the current value by the maximum
        /// value.
        /// </summary>

        public double Position
        {
            get
            {
                if (!HasAttribute("value"))
                {
                    return -1;
                }
                else
                {
                    return Value / Max;

                }
            }
          
        }

        /// <summary>
        /// A NodeList of all LABEL elements within this Progress element
        /// </summary>

        public INodeList<IHTMLLabelElement> Labels
        {
            get {
                return new NodeList<IHTMLLabelElement>(ChildElementsOfTag<IHTMLLabelElement>(HtmlData.tagLABEL));
            }
        }
    }
}
