﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System.IO;
using CsQuery.HtmlParser;

namespace CsQuery.Implementation
{
    /// <summary>
    /// An HTML text area element.
    /// </summary>

    internal class HTMLTextAreaElement : FormSubmittableElement, IHTMLTextAreaElement
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HTMLTextAreaElement(): base(HtmlData.tagTEXTAREA)
        {

        }

        /// <summary>
        /// The value of the HTMLRawInnerTextElementBase's contents
        /// </summary>

        public override string Value
        {
            get
            {
                var formatter = new Output.FormatDefault();
                StringWriter sw = new StringWriter();

                formatter.RenderChildren(this, sw);
                return sw.ToString();
            }
            set
            {
                ChildNodes.Clear();
                ChildNodes.Add(Document.CreateTextNode(value));
            }
        }


        /// <summary>
        /// The string "textarea", per the HTML5 spec.
        /// </summary>
        /// 
        /// <url>
        /// http://www.w3.org/html/wg/drafts/html/master/forms.html#dom-textarea-type
        /// </url>
        public override string Type
        {
            get { return "textarea"; }
        }


        /// <summary>
        /// For HTMLRawInnerTextElementBase elements, InnerText doesn't actually do anything, whereas Value is the InnerText.
        /// </summary>

        public new string InnerText
        {
            get
            {
                return "";
            }
            set
            {
                return;
            }
        }

    }
}
