﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.Engine
{
    /// <summary>
    /// Base class for an Child-type pseudoselector.
    /// </summary>

    internal abstract class PseudoSelectorChild : PseudoSelector, IPseudoSelectorChild
    {
        /// <summary>
        /// Test whether an element matches this selector.
        /// </summary>
        ///
        /// <param name="element">
        /// The element to test.
        /// </param>
        ///
        /// <returns>
        /// true if it matches, false if not.
        /// </returns>

        public abstract bool Matches(IDomObject element);

        /// <summary>
        /// Basic implementation of ChildMatches, runs the Matches method against each child. This should
        /// be overridden with something more efficient if possible. For example, selectors that inspect
        /// the element's index could get their results more easily by picking the correct results from
        /// the list of children rather than testing each one.
        /// 
        /// Also note that the default iterator for ChildMatches only passed element (e.g. non-text node)
        /// children. If you wanted to design a filter that worked on other node types, you should
        /// override this to access all children instead of just the elements.
        /// </summary>
        ///
        /// <param name="element">
        /// The parent element.
        /// </param>
        ///
        /// <returns>
        /// A sequence of children that match.
        /// </returns>

        public virtual IEnumerable<IDomObject> ChildMatches(IDomContainer element)
        {
            return element.ChildElements.Where(item => Matches(item));
        }
    }

}
