﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.Engine.PseudoClassSelectors
{
    /// <summary>
    /// Base class for jQuery filters that test whether an element appears at the specified position with the list.
    /// </summary>

    internal abstract class Indexed : PseudoSelector, IPseudoSelectorFilter
    {
        private int _Index;
        private bool IndexParsed;

        /// <summary>
        /// The zero-based index for which to test.
        /// </summary>

        protected int Index
        {
            get
            {
                if (!IndexParsed)
                {
                    if (!int.TryParse(Parameters[0], out _Index))
                    {
                        throw new ArgumentException(String.Format("The {0} selector requires a single integer parameter.", Name));
                    }
                    IndexParsed = true;
                }
                return _Index;
            }
        }

        /// <summary>
        /// The maximum number of parameters that this selector can accept (1)
        /// </summary>
        ///
        /// <value>
        /// An integer.
        /// </value>

        public override int  MaximumParameterCount
        {
	        get 
	        { 
		         return 1;
	        }
        }

        /// <summary>
        /// The minimum number of parameters that this selector requires (1)
        /// </summary>
        ///
        /// <value>
        /// An integer.
        /// </value>

        public override int  MinimumParameterCount
        {
	        get 
	        { 
		         return 1;
	        }
        }

        /// <summary>
        /// Abstract implementation of the Filter method for the Index filter.
        /// </summary>
        ///
        /// <param name="selection">
        /// The sequence of elements prior to this filter being applied.
        /// </param>
        ///
        /// <returns>
        /// A sequence of matching elements.
        /// </returns>

        public abstract IEnumerable<IDomObject> Filter(IEnumerable<IDomObject> selection);
    }
}
