﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

// TODO this should be fully commented; however it's not part of the main public API

#pragma warning disable 1591
#pragma warning disable 1570


namespace CsQuery.EquationParser.Implementation
{
    internal class Literal : Operand, ILiteral
    {
        #region constructors
        public Literal(): base()
        {

        }
        public Literal(IConvertible value)
            : base()
        {
            Set(value);
        }

        //public static implicit operator Literal(IConvertible value)
        //{
        //    return new Literal(value);
        //}
        #endregion

        #region private properties
        protected IConvertible _Value;
        #endregion
     
        #region public methods
        public new ILiteral Clone()
        {
            return (ILiteral)base.Clone();
        }

        protected override IOperand GetNewInstance()
        {
            return new Literal();
        }
        protected override IOperand CopyTo(IOperand operand)
        {
            ((Literal)operand).Set(GetValue());
            return operand;
        }
        protected override IConvertible GetValue()
        {
            return _Value;
        }
        public override string ToString()
        {
            return Value.ToString();
        }
        #endregion

        #region private methods

        public virtual void Set(IConvertible value)
        {
            _Value = value;
        }

        #endregion

        #region interface members
        //void ILiteral.Set(IConvertible value)
        //{
        //    Set((T)Convert.ChangeType(value, typeof(T)));
        //}
        #endregion
    }

    internal class Literal<T> : Literal, ILiteral<T> where T : IConvertible
    {
        #region constructors
        public Literal(): base()
        {

        }
        public Literal(IConvertible value)
            : base()
        {
            SetConvert(value);
        }

        public static implicit operator Literal<T>(int value)
        {
            return new Literal<T>(value);
        }
        public static implicit operator Literal<T>(double value)
        {
            return new Literal<T>(value);
        }
        public static implicit operator Literal<T>(string value)
        {
            return new Literal<T>(value);
        }
        #endregion

        #region private properties
        
        #endregion

        #region public properties
        public new T Value {
            get
            {
                return (T)_Value;
            }
        }

        public new ILiteral<T> Clone()
        {
            return (ILiteral<T>)CopyTo(GetNewInstance());
        }
        public void Set(T value)
        {
            _Value = value;
        }
        #endregion

        #region private methods
        protected override IOperand GetNewInstance()
        {
            return new Literal<T>(Value);
        }

        /// <summary>
        /// This is static so it can be used by the constructors -- sets the value of the strongly typed
        /// instance.
        /// </summary>
        ///
        /// <param name="value">
        /// The value to set
        /// </param>

        private void SetConvert(IConvertible value)
        {

            Set((T)Convert.ChangeType(value, typeof(T)));
        }

        #endregion

        #region interface members
        IOperand<T> IOperand<T>.Clone()
        {
            return Clone();
        }
        void ILiteral.Set(IConvertible value)
        {
            Set((T)Convert.ChangeType(value,typeof(T)));
        }
        #endregion
    }
}
