﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace CsQuery.ExtensionMethods.Xml
{
    /// <summary>
    /// An adapter to map an INodeList to an XmlNodeList. NOT IMPLEMENTED.
    /// </summary>

    internal class CqXmlNodeList : XmlNodeList
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        ///
        /// <param name="xmlDocument">
        /// The XML document.
        /// </param>
        /// <param name="nodeList">
        /// List of nodes.
        /// </param>

        public CqXmlNodeList(XmlDocument xmlDocument, INodeList nodeList)
        {
            NodeList = nodeList;
            XmlDocument = xmlDocument;
        }

        private INodeList NodeList;
        private XmlDocument XmlDocument;

        /// <summary>
        /// The number of nodes in this list
        /// </summary>

        public override int Count
        {
            get { return NodeList.Count; }
        }

        /// <summary>
        /// An enumerator for the node list.
        /// </summary>
        ///
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator" />.
        /// </returns>

        public override System.Collections.IEnumerator GetEnumerator()
        {

            return Nodes().GetEnumerator();
        }

        private IEnumerable<XmlNode> Nodes()
        {
            foreach (var node in NodeList)
            {
                yield return node.ToXml(XmlDocument);
            }
        }

        /// <summary>
        /// Retrieves a node at the given index.
        /// </summary>
        ///
        /// <param name="index">
        /// Zero-based index into the list of nodes.
        /// </param>
        ///
        /// <returns>
        /// The <see cref="T:System.Xml.XmlNode" /> in the collection. If <paramref name="index" /> is
        /// greater than or equal to the number of nodes in the list, this returns null.
        /// </returns>

        public override XmlNode Item(int index)
        {
            return NodeList[index].ToXml();
        }
    }
}
