﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.HtmlParser
{
    /// <summary>
    /// Simplify access to character set encodings for this system.
    /// </summary>

    internal static class HtmlEncoding
    {
        private static Dictionary<string, EncodingInfo> _Encodings;

        /// <summary>
        /// A dictionary of all encodings available on this system
        /// </summary>

        private static Dictionary<string, EncodingInfo> Encodings
        {
            get
            {
                if (_Encodings == null)
                {
                    _Encodings = new Dictionary<string, EncodingInfo>(StringComparer.CurrentCultureIgnoreCase);
                    foreach (var encoding in Encoding.GetEncodings())
                    {
                        _Encodings[encoding.Name] = encoding;
                    }
                }
                return _Encodings;
            }
        }

        /// <summary>
        /// Try to get a character set encoding from its web name.
        /// </summary>
        ///
        /// <param name="encodingName">
        /// Name of the encoding.
        /// </param>
        /// <param name="encoding">
        /// [out] The encoding.
        /// </param>
        ///
        /// <returns>
        /// true if it succeeds, false if it fails.
        /// </returns>

        public static bool TryGetEncoding(string encodingName, out Encoding encoding)
        {
            EncodingInfo info;
            if (Encodings.TryGetValue(encodingName, out info)) {
                encoding = info.GetEncoding();
                return true;
            } else {
                encoding =null;
                return false;
            }
        }

        /// <summary>
        /// Gets an encoding.
        /// </summary>
        ///
        /// <param name="encodingName">
        /// Name of the encoding.
        /// </param>
        ///
        /// <returns>
        /// The encoding.
        /// </returns>

        public static Encoding GetEncoding(string encodingName)
        {
            Encoding encoding;
            if (TryGetEncoding(encodingName, out encoding)) {
                return encoding;
            } else {
                return null;
            }
        }
    }
}
