﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.HtmlParser
{
    /// <summary>
    /// Bitfield of flags for specifying properties that may be tested on HTML tokens using a bitmap.
    /// </summary>

    [Flags]
    internal enum TokenProperties : ushort
    {
        /// <summary>
        /// the element is an HTML block-level element
        /// </summary>
        
        BlockElement=1,

        /// <summary>
        /// the attribute is a boolean property e.g. 'checked'
        /// </summary>
        
        BooleanProperty=2,
        
        /// <summary>
        /// the tag is automatically closing, e.g. 'p'.
        /// </summary>
        
        AutoOpenOrClose=4,
        
        /// <summary>
        /// the tag may not have children
        /// </summary>
        
        ChildrenNotAllowed=8,

        /// <summary>
        /// the tag may not have HTML children (but could possibly have children)
        /// </summary>
        
        HtmlChildrenNotAllowed=16,

        /// <summary>
        /// this tag causes an open p tag to close
        /// </summary>
        
        ParagraphCloser=32,

        /// <summary>
        /// The tag may appear in HEAD
        /// </summary>
        
        MetaDataTags = 64,
       
        /// <summary>
        /// election of attribute values is not case sensitive
        /// </summary>
        
        CaseInsensitiveValues = 128,

        /// <summary>
        /// Has a VALUE property
        /// </summary>
        
        HasValue = 256,

        /// <summary>
        /// Element is a form input control
        /// </summary>
        
        FormInputControl = 512
    }
}
