﻿/*
 * Copyright 2004-2010 Apple Computer, Inc., Mozilla Foundation, and Opera 
 * Software ASA.
 * 
 * You are granted a license to use, reproduce and create derivative works of 
 * this document.
 */

#pragma warning disable 1591 // Missing XML comment
#pragma warning disable 1570 // XML comment on 'construct' has badly formed XML — 'reason'
#pragma warning disable 1587 // XML comment is not placed on a valid element

namespace HtmlParserSharp.Core
{
    internal sealed class NamedCharacters
    {

	internal static readonly string[] NAMES = { "lig", "lig;",
			"P", "P;", "cute", "cute;", "reve;", "irc", "irc;", "y;", "r;",
			"rave", "rave;", "pha;", "acr;", "d;", "gon;", "pf;",
			"plyFunction;", "ing", "ing;", "cr;", "sign;", "ilde", "ilde;",
			"ml", "ml;", "ckslash;", "rv;", "rwed;", "y;", "cause;",
			"rnoullis;", "ta;", "r;", "pf;", "eve;", "cr;", "mpeq;", "cy;",
			"PY", "PY;", "cute;", "p;", "pitalDifferentialD;", "yleys;",
			"aron;", "edil", "edil;", "irc;", "onint;", "ot;", "dilla;",
			"nterDot;", "r;", "i;", "rcleDot;", "rcleMinus;", "rclePlus;",
			"rcleTimes;", "ockwiseContourIntegral;", "oseCurlyDoubleQuote;",
			"oseCurlyQuote;", "lon;", "lone;", "ngruent;", "nint;",
			"ntourIntegral;", "pf;", "product;",
			"unterClockwiseContourIntegral;", "oss;", "cr;", "p;", "pCap;",
			";", "otrahd;", "cy;", "cy;", "cy;", "gger;", "rr;", "shv;",
			"aron;", "y;", "l;", "lta;", "r;", "acriticalAcute;",
			"acriticalDot;", "acriticalDoubleAcute;", "acriticalGrave;",
			"acriticalTilde;", "amond;", "fferentialD;", "pf;", "t;", "tDot;",
			"tEqual;", "ubleContourIntegral;", "ubleDot;", "ubleDownArrow;",
			"ubleLeftArrow;", "ubleLeftRightArrow;", "ubleLeftTee;",
			"ubleLongLeftArrow;", "ubleLongLeftRightArrow;",
			"ubleLongRightArrow;", "ubleRightArrow;", "ubleRightTee;",
			"ubleUpArrow;", "ubleUpDownArrow;", "ubleVerticalBar;", "wnArrow;",
			"wnArrowBar;", "wnArrowUpArrow;", "wnBreve;", "wnLeftRightVector;",
			"wnLeftTeeVector;", "wnLeftVector;", "wnLeftVectorBar;",
			"wnRightTeeVector;", "wnRightVector;", "wnRightVectorBar;",
			"wnTee;", "wnTeeArrow;", "wnarrow;", "cr;", "trok;", "G;", "H",
			"H;", "cute", "cute;", "aron;", "irc", "irc;", "y;", "ot;", "r;",
			"rave", "rave;", "ement;", "acr;", "ptySmallSquare;",
			"ptyVerySmallSquare;", "gon;", "pf;", "silon;", "ual;",
			"ualTilde;", "uilibrium;", "cr;", "im;", "a;", "ml", "ml;",
			"ists;", "ponentialE;", "y;", "r;", "lledSmallSquare;",
			"lledVerySmallSquare;", "pf;", "rAll;", "uriertrf;", "cr;", "cy;",
			"", ";", "mma;", "mmad;", "reve;", "edil;", "irc;", "y;", "ot;",
			"r;", ";", "pf;", "eaterEqual;", "eaterEqualLess;",
			"eaterFullEqual;", "eaterGreater;", "eaterLess;",
			"eaterSlantEqual;", "eaterTilde;", "cr;", ";", "RDcy;", "cek;",
			"t;", "irc;", "r;", "lbertSpace;", "pf;", "rizontalLine;", "cr;",
			"trok;", "mpDownHump;", "mpEqual;", "cy;", "lig;", "cy;", "cute",
			"cute;", "irc", "irc;", "y;", "ot;", "r;", "rave", "rave;", ";",
			"acr;", "aginaryI;", "plies;", "t;", "tegral;", "tersection;",
			"visibleComma;", "visibleTimes;", "gon;", "pf;", "ta;", "cr;",
			"ilde;", "kcy;", "ml", "ml;", "irc;", "y;", "r;", "pf;", "cr;",
			"ercy;", "kcy;", "cy;", "cy;", "ppa;", "edil;", "y;", "r;", "pf;",
			"cr;", "cy;", "", ";", "cute;", "mbda;", "ng;", "placetrf;", "rr;",
			"aron;", "edil;", "y;", "ftAngleBracket;", "ftArrow;",
			"ftArrowBar;", "ftArrowRightArrow;", "ftCeiling;",
			"ftDoubleBracket;", "ftDownTeeVector;", "ftDownVector;",
			"ftDownVectorBar;", "ftFloor;", "ftRightArrow;", "ftRightVector;",
			"ftTee;", "ftTeeArrow;", "ftTeeVector;", "ftTriangle;",
			"ftTriangleBar;", "ftTriangleEqual;", "ftUpDownVector;",
			"ftUpTeeVector;", "ftUpVector;", "ftUpVectorBar;", "ftVector;",
			"ftVectorBar;", "ftarrow;", "ftrightarrow;", "ssEqualGreater;",
			"ssFullEqual;", "ssGreater;", "ssLess;", "ssSlantEqual;",
			"ssTilde;", "r;", ";", "eftarrow;", "idot;", "ngLeftArrow;",
			"ngLeftRightArrow;", "ngRightArrow;", "ngleftarrow;",
			"ngleftrightarrow;", "ngrightarrow;", "pf;", "werLeftArrow;",
			"werRightArrow;", "cr;", "h;", "trok;", ";", "p;", "y;",
			"diumSpace;", "llintrf;", "r;", "nusPlus;", "pf;", "cr;", ";",
			"cy;", "cute;", "aron;", "edil;", "y;", "gativeMediumSpace;",
			"gativeThickSpace;", "gativeThinSpace;", "gativeVeryThinSpace;",
			"stedGreaterGreater;", "stedLessLess;", "wLine;", "r;", "Break;",
			"nBreakingSpace;", "pf;", "t;", "tCongruent;", "tCupCap;",
			"tDoubleVerticalBar;", "tElement;", "tEqual;", "tEqualTilde;",
			"tExists;", "tGreater;", "tGreaterEqual;", "tGreaterFullEqual;",
			"tGreaterGreater;", "tGreaterLess;", "tGreaterSlantEqual;",
			"tGreaterTilde;", "tHumpDownHump;", "tHumpEqual;",
			"tLeftTriangle;", "tLeftTriangleBar;", "tLeftTriangleEqual;",
			"tLess;", "tLessEqual;", "tLessGreater;", "tLessLess;",
			"tLessSlantEqual;", "tLessTilde;", "tNestedGreaterGreater;",
			"tNestedLessLess;", "tPrecedes;", "tPrecedesEqual;",
			"tPrecedesSlantEqual;", "tReverseElement;", "tRightTriangle;",
			"tRightTriangleBar;", "tRightTriangleEqual;", "tSquareSubset;",
			"tSquareSubsetEqual;", "tSquareSuperset;", "tSquareSupersetEqual;",
			"tSubset;", "tSubsetEqual;", "tSucceeds;", "tSucceedsEqual;",
			"tSucceedsSlantEqual;", "tSucceedsTilde;", "tSuperset;",
			"tSupersetEqual;", "tTilde;", "tTildeEqual;", "tTildeFullEqual;",
			"tTildeTilde;", "tVerticalBar;", "cr;", "ilde", "ilde;", ";",
			"lig;", "cute", "cute;", "irc", "irc;", "y;", "blac;", "r;",
			"rave", "rave;", "acr;", "ega;", "icron;", "pf;",
			"enCurlyDoubleQuote;", "enCurlyQuote;", ";", "cr;", "lash",
			"lash;", "ilde", "ilde;", "imes;", "ml", "ml;", "erBar;",
			"erBrace;", "erBracket;", "erParenthesis;", "rtialD;", "y;", "r;",
			"i;", ";", "usMinus;", "incareplane;", "pf;", ";", "ecedes;",
			"ecedesEqual;", "ecedesSlantEqual;", "ecedesTilde;", "ime;",
			"oduct;", "oportion;", "oportional;", "cr;", "i;", "OT", "OT;",
			"r;", "pf;", "cr;", "arr;", "G", "G;", "cute;", "ng;", "rr;",
			"rrtl;", "aron;", "edil;", "y;", ";", "verseElement;",
			"verseEquilibrium;", "verseUpEquilibrium;", "r;", "o;",
			"ghtAngleBracket;", "ghtArrow;", "ghtArrowBar;",
			"ghtArrowLeftArrow;", "ghtCeiling;", "ghtDoubleBracket;",
			"ghtDownTeeVector;", "ghtDownVector;", "ghtDownVectorBar;",
			"ghtFloor;", "ghtTee;", "ghtTeeArrow;", "ghtTeeVector;",
			"ghtTriangle;", "ghtTriangleBar;", "ghtTriangleEqual;",
			"ghtUpDownVector;", "ghtUpTeeVector;", "ghtUpVector;",
			"ghtUpVectorBar;", "ghtVector;", "ghtVectorBar;", "ghtarrow;",
			"pf;", "undImplies;", "ightarrow;", "cr;", "h;", "leDelayed;",
			"CHcy;", "cy;", "FTcy;", "cute;", ";", "aron;", "edil;", "irc;",
			"y;", "r;", "ortDownArrow;", "ortLeftArrow;", "ortRightArrow;",
			"ortUpArrow;", "gma;", "allCircle;", "pf;", "rt;", "uare;",
			"uareIntersection;", "uareSubset;", "uareSubsetEqual;",
			"uareSuperset;", "uareSupersetEqual;", "uareUnion;", "cr;", "ar;",
			"b;", "bset;", "bsetEqual;", "cceeds;", "cceedsEqual;",
			"cceedsSlantEqual;", "cceedsTilde;", "chThat;", "m;", "p;",
			"perset;", "persetEqual;", "pset;", "ORN", "ORN;", "ADE;", "Hcy;",
			"cy;", "b;", "u;", "aron;", "edil;", "y;", "r;", "erefore;",
			"eta;", "ickSpace;", "inSpace;", "lde;", "ldeEqual;",
			"ldeFullEqual;", "ldeTilde;", "pf;", "ipleDot;", "cr;", "trok;",
			"cute", "cute;", "rr;", "rrocir;", "rcy;", "reve;", "irc", "irc;",
			"y;", "blac;", "r;", "rave", "rave;", "acr;", "derBar;",
			"derBrace;", "derBracket;", "derParenthesis;", "ion;", "ionPlus;",
			"gon;", "pf;", "Arrow;", "ArrowBar;", "ArrowDownArrow;",
			"DownArrow;", "Equilibrium;", "Tee;", "TeeArrow;", "arrow;",
			"downarrow;", "perLeftArrow;", "perRightArrow;", "si;", "silon;",
			"ing;", "cr;", "ilde;", "ml", "ml;", "ash;", "ar;", "y;", "ash;",
			"ashl;", "e;", "rbar;", "rt;", "rticalBar;", "rticalLine;",
			"rticalSeparator;", "rticalTilde;", "ryThinSpace;", "r;", "pf;",
			"cr;", "dash;", "irc;", "dge;", "r;", "pf;", "cr;", "r;", ";",
			"pf;", "cr;", "cy;", "cy;", "cy;", "cute", "cute;", "irc;", "y;",
			"r;", "pf;", "cr;", "ml;", "cy;", "cute;", "aron;", "y;", "ot;",
			"roWidthSpace;", "ta;", "r;", "pf;", "cr;", "cute", "cute;",
			"reve;", ";", "E;", "d;", "irc", "irc;", "ute", "ute;", "y;",
			"lig", "lig;", ";", "r;", "rave", "rave;", "efsym;", "eph;",
			"pha;", "acr;", "alg;", "p", "p;", "d;", "dand;", "dd;", "dslope;",
			"dv;", "g;", "ge;", "gle;", "gmsd;", "gmsdaa;", "gmsdab;",
			"gmsdac;", "gmsdad;", "gmsdae;", "gmsdaf;", "gmsdag;", "gmsdah;",
			"grt;", "grtvb;", "grtvbd;", "gsph;", "gst;", "gzarr;", "gon;",
			"pf;", ";", "E;", "acir;", "e;", "id;", "os;", "prox;", "proxeq;",
			"ing", "ing;", "cr;", "t;", "ymp;", "ympeq;", "ilde", "ilde;",
			"ml", "ml;", "conint;", "int;", "ot;", "ckcong;", "ckepsilon;",
			"ckprime;", "cksim;", "cksimeq;", "rvee;", "rwed;", "rwedge;",
			"rk;", "rktbrk;", "ong;", "y;", "quo;", "caus;", "cause;",
			"mptyv;", "psi;", "rnou;", "ta;", "th;", "tween;", "r;", "gcap;",
			"gcirc;", "gcup;", "godot;", "goplus;", "gotimes;", "gsqcup;",
			"gstar;", "gtriangledown;", "gtriangleup;", "guplus;", "gvee;",
			"gwedge;", "arow;", "acklozenge;", "acksquare;", "acktriangle;",
			"acktriangledown;", "acktriangleleft;", "acktriangleright;",
			"ank;", "k12;", "k14;", "k34;", "ock;", "e;", "equiv;", "ot;",
			"pf;", "t;", "ttom;", "wtie;", "xDL;", "xDR;", "xDl;", "xDr;",
			"xH;", "xHD;", "xHU;", "xHd;", "xHu;", "xUL;", "xUR;", "xUl;",
			"xUr;", "xV;", "xVH;", "xVL;", "xVR;", "xVh;", "xVl;", "xVr;",
			"xbox;", "xdL;", "xdR;", "xdl;", "xdr;", "xh;", "xhD;", "xhU;",
			"xhd;", "xhu;", "xminus;", "xplus;", "xtimes;", "xuL;", "xuR;",
			"xul;", "xur;", "xv;", "xvH;", "xvL;", "xvR;", "xvh;", "xvl;",
			"xvr;", "rime;", "eve;", "vbar", "vbar;", "cr;", "emi;", "im;",
			"ime;", "ol;", "olb;", "olhsub;", "ll;", "llet;", "mp;", "mpE;",
			"mpe;", "mpeq;", "cute;", "p;", "pand;", "pbrcup;", "pcap;",
			"pcup;", "pdot;", "ps;", "ret;", "ron;", "aps;", "aron;", "edil",
			"edil;", "irc;", "ups;", "upssm;", "ot;", "dil", "dil;", "mptyv;",
			"nt", "nt;", "nterdot;", "r;", "cy;", "eck;", "eckmark;", "i;",
			"r;", "rE;", "rc;", "rceq;", "rclearrowleft;", "rclearrowright;",
			"rcledR;", "rcledS;", "rcledast;", "rcledcirc;", "rcleddash;",
			"re;", "rfnint;", "rmid;", "rscir;", "ubs;", "ubsuit;", "lon;",
			"lone;", "loneq;", "mma;", "mmat;", "mp;", "mpfn;", "mplement;",
			"mplexes;", "ng;", "ngdot;", "nint;", "pf;", "prod;", "py", "py;",
			"pysr;", "arr;", "oss;", "cr;", "ub;", "ube;", "up;", "upe;",
			"dot;", "darrl;", "darrr;", "epr;", "esc;", "larr;", "larrp;",
			"p;", "pbrcap;", "pcap;", "pcup;", "pdot;", "por;", "ps;", "rarr;",
			"rarrm;", "rlyeqprec;", "rlyeqsucc;", "rlyvee;", "rlywedge;",
			"rren", "rren;", "rvearrowleft;", "rvearrowright;", "vee;", "wed;",
			"conint;", "int;", "lcty;", "rr;", "ar;", "gger;", "leth;", "rr;",
			"sh;", "shv;", "karow;", "lac;", "aron;", "y;", ";", "agger;",
			"arr;", "otseq;", "g", "g;", "lta;", "mptyv;", "isht;", "r;",
			"arl;", "arr;", "am;", "amond;", "amondsuit;", "ams;", "e;",
			"gamma;", "sin;", "v;", "vide", "vide;", "videontimes;", "vonx;",
			"cy;", "corn;", "crop;", "llar;", "pf;", "t;", "teq;", "teqdot;",
			"tminus;", "tplus;", "tsquare;", "ublebarwedge;", "wnarrow;",
			"wndownarrows;", "wnharpoonleft;", "wnharpoonright;", "bkarow;",
			"corn;", "crop;", "cr;", "cy;", "ol;", "trok;", "dot;", "ri;",
			"rif;", "arr;", "har;", "angle;", "cy;", "igrarr;", "Dot;", "ot;",
			"cute", "cute;", "ster;", "aron;", "ir;", "irc", "irc;", "olon;",
			"y;", "ot;", ";", "Dot;", "r;", ";", "rave", "rave;", "s;",
			"sdot;", ";", "inters;", "l;", "s;", "sdot;", "acr;", "pty;",
			"ptyset;", "ptyv;", "sp13;", "sp14;", "sp;", "g;", "sp;", "gon;",
			"pf;", "ar;", "arsl;", "lus;", "si;", "silon;", "siv;", "circ;",
			"colon;", "sim;", "slantgtr;", "slantless;", "uals;", "uest;",
			"uiv;", "uivDD;", "vparsl;", "Dot;", "arr;", "cr;", "dot;", "im;",
			"a;", "h", "h;", "ml", "ml;", "ro;", "cl;", "ist;", "pectation;",
			"ponentiale;", "llingdotseq;", "y;", "male;", "ilig;", "lig;",
			"llig;", "r;", "lig;", "lig;", "at;", "lig;", "tns;", "of;", "pf;",
			"rall;", "rk;", "rkv;", "artint;", "ac12", "ac12;", "ac13;",
			"ac14", "ac14;", "ac15;", "ac16;", "ac18;", "ac23;", "ac25;",
			"ac34", "ac34;", "ac35;", "ac38;", "ac45;", "ac56;", "ac58;",
			"ac78;", "asl;", "own;", "cr;", ";", "l;", "cute;", "mma;",
			"mmad;", "p;", "reve;", "irc;", "y;", "ot;", ";", "l;", "q;",
			"qq;", "qslant;", "s;", "scc;", "sdot;", "sdoto;", "sdotol;",
			"sl;", "sles;", "r;", ";", "g;", "mel;", "cy;", ";", "E;", "a;",
			"j;", "E;", "ap;", "approx;", "e;", "eq;", "eqq;", "sim;", "pf;",
			"ave;", "cr;", "im;", "ime;", "iml;", "", ";", "cc;", "cir;",
			"dot;", "lPar;", "quest;", "rapprox;", "rarr;", "rdot;",
			"reqless;", "reqqless;", "rless;", "rsim;", "ertneqq;", "nE;",
			"rr;", "irsp;", "lf;", "milt;", "rdcy;", "rr;", "rrcir;", "rrw;",
			"ar;", "irc;", "arts;", "artsuit;", "llip;", "rcon;", "r;",
			"searow;", "swarow;", "arr;", "mtht;", "okleftarrow;",
			"okrightarrow;", "pf;", "rbar;", "cr;", "lash;", "trok;", "bull;",
			"phen;", "cute", "cute;", ";", "irc", "irc;", "y;", "cy;", "xcl",
			"xcl;", "f;", "r;", "rave", "rave;", ";", "iint;", "int;", "nfin;",
			"ota;", "lig;", "acr;", "age;", "agline;", "agpart;", "ath;",
			"of;", "ped;", ";", "care;", "fin;", "fintie;", "odot;", "t;",
			"tcal;", "tegers;", "tercal;", "tlarhk;", "tprod;", "cy;", "gon;",
			"pf;", "ta;", "rod;", "uest", "uest;", "cr;", "in;", "inE;",
			"indot;", "ins;", "insv;", "inv;", ";", "ilde;", "kcy;", "ml",
			"ml;", "irc;", "y;", "r;", "ath;", "pf;", "cr;", "ercy;", "kcy;",
			"ppa;", "ppav;", "edil;", "y;", "r;", "reen;", "cy;", "cy;", "pf;",
			"cr;", "arr;", "rr;", "tail;", "arr;", ";", "g;", "ar;", "cute;",
			"emptyv;", "gran;", "mbda;", "ng;", "ngd;", "ngle;", "p;", "quo",
			"quo;", "rr;", "rrb;", "rrbfs;", "rrfs;", "rrhk;", "rrlp;",
			"rrpl;", "rrsim;", "rrtl;", "t;", "tail;", "te;", "tes;", "arr;",
			"brk;", "race;", "rack;", "rke;", "rksld;", "rkslu;", "aron;",
			"edil;", "eil;", "ub;", "y;", "ca;", "quo;", "quor;", "rdhar;",
			"rushar;", "sh;", ";", "ftarrow;", "ftarrowtail;",
			"ftharpoondown;", "ftharpoonup;", "ftleftarrows;", "ftrightarrow;",
			"ftrightarrows;", "ftrightharpoons;", "ftrightsquigarrow;",
			"ftthreetimes;", "g;", "q;", "qq;", "qslant;", "s;", "scc;",
			"sdot;", "sdoto;", "sdotor;", "sg;", "sges;", "ssapprox;",
			"ssdot;", "sseqgtr;", "sseqqgtr;", "ssgtr;", "sssim;", "isht;",
			"loor;", "r;", ";", "E;", "ard;", "aru;", "arul;", "blk;", "cy;",
			";", "arr;", "corner;", "hard;", "tri;", "idot;", "oust;",
			"oustache;", "E;", "ap;", "approx;", "e;", "eq;", "eqq;", "sim;",
			"ang;", "arr;", "brk;", "ngleftarrow;", "ngleftrightarrow;",
			"ngmapsto;", "ngrightarrow;", "oparrowleft;", "oparrowright;",
			"par;", "pf;", "plus;", "times;", "wast;", "wbar;", "z;", "zenge;",
			"zf;", "ar;", "arlt;", "arr;", "corner;", "har;", "hard;", "m;",
			"tri;", "aquo;", "cr;", "h;", "im;", "ime;", "img;", "qb;", "quo;",
			"quor;", "trok;", "", ";", "cc;", "cir;", "dot;", "hree;", "imes;",
			"larr;", "quest;", "rPar;", "ri;", "rie;", "rif;", "rdshar;",
			"ruhar;", "ertneqq;", "nE;", "Dot;", "cr", "cr;", "le;", "lt;",
			"ltese;", "p;", "psto;", "pstodown;", "pstoleft;", "pstoup;",
			"rker;", "omma;", "y;", "ash;", "asuredangle;", "r;", "o;", "cro",
			"cro;", "d;", "dast;", "dcir;", "ddot", "ddot;", "nus;", "nusb;",
			"nusd;", "nusdu;", "cp;", "dr;", "plus;", "dels;", "pf;", ";",
			"cr;", "tpos;", ";", "ltimap;", "map;", "g;", "t;", "tv;",
			"eftarrow;", "eftrightarrow;", "l;", "t;", "tv;", "ightarrow;",
			"Dash;", "dash;", "bla;", "cute;", "ng;", "p;", "pE;", "pid;",
			"pos;", "pprox;", "tur;", "tural;", "turals;", "sp", "sp;", "ump;",
			"umpe;", "ap;", "aron;", "edil;", "ong;", "ongdot;", "up;", "y;",
			"ash;", ";", "Arr;", "arhk;", "arr;", "arrow;", "dot;", "quiv;",
			"sear;", "sim;", "xist;", "xists;", "r;", "E;", "e;", "eq;",
			"eqq;", "eqslant;", "es;", "sim;", "t;", "tr;", "Arr;", "arr;",
			"par;", ";", "s;", "sd;", "v;", "cy;", "Arr;", "E;", "arr;", "dr;",
			"e;", "eftarrow;", "eftrightarrow;", "eq;", "eqq;", "eqslant;",
			"es;", "ess;", "sim;", "t;", "tri;", "trie;", "id;", "pf;", "t",
			"t;", "tin;", "tinE;", "tindot;", "tinva;", "tinvb;", "tinvc;",
			"tni;", "tniva;", "tnivb;", "tnivc;", "ar;", "arallel;", "arsl;",
			"art;", "olint;", "r;", "rcue;", "re;", "rec;", "receq;", "Arr;",
			"arr;", "arrc;", "arrw;", "ightarrow;", "tri;", "trie;", "c;",
			"ccue;", "ce;", "cr;", "hortmid;", "hortparallel;", "im;", "ime;",
			"imeq;", "mid;", "par;", "qsube;", "qsupe;", "ub;", "ubE;", "ube;",
			"ubset;", "ubseteq;", "ubseteqq;", "ucc;", "ucceq;", "up;", "upE;",
			"upe;", "upset;", "upseteq;", "upseteqq;", "gl;", "ilde", "ilde;",
			"lg;", "riangleleft;", "rianglelefteq;", "riangleright;",
			"rianglerighteq;", ";", "m;", "mero;", "msp;", "Dash;", "Harr;",
			"ap;", "dash;", "ge;", "gt;", "infin;", "lArr;", "le;", "lt;",
			"ltrie;", "rArr;", "rtrie;", "sim;", "Arr;", "arhk;", "arr;",
			"arrow;", "near;", ";", "cute", "cute;", "st;", "ir;", "irc",
			"irc;", "y;", "ash;", "blac;", "iv;", "ot;", "sold;", "lig;",
			"cir;", "r;", "on;", "rave", "rave;", "t;", "bar;", "m;", "nt;",
			"arr;", "cir;", "cross;", "ine;", "t;", "acr;", "ega;", "icron;",
			"id;", "inus;", "pf;", "ar;", "erp;", "lus;", ";", "arr;", "d;",
			"der;", "derof;", "df", "df;", "dm", "dm;", "igof;", "or;",
			"slope;", "v;", "cr;", "lash", "lash;", "ol;", "ilde", "ilde;",
			"imes;", "imesas;", "ml", "ml;", "bar;", "r;", "ra", "ra;",
			"rallel;", "rsim;", "rsl;", "rt;", "y;", "rcnt;", "riod;", "rmil;",
			"rp;", "rtenk;", "r;", "i;", "iv;", "mmat;", "one;", ";",
			"tchfork;", "v;", "anck;", "anckh;", "ankv;", "us;", "usacir;",
			"usb;", "uscir;", "usdo;", "usdu;", "use;", "usmn", "usmn;",
			"ussim;", "ustwo;", ";", "intint;", "pf;", "und", "und;", ";",
			"E;", "ap;", "cue;", "e;", "ec;", "ecapprox;", "eccurlyeq;",
			"eceq;", "ecnapprox;", "ecneqq;", "ecnsim;", "ecsim;", "ime;",
			"imes;", "nE;", "nap;", "nsim;", "od;", "ofalar;", "ofline;",
			"ofsurf;", "op;", "opto;", "sim;", "urel;", "cr;", "i;", "ncsp;",
			"r;", "nt;", "pf;", "rime;", "cr;", "aternions;", "atint;", "est;",
			"esteq;", "ot", "ot;", "arr;", "rr;", "tail;", "arr;", "ar;",
			"ce;", "cute;", "dic;", "emptyv;", "ng;", "ngd;", "nge;", "ngle;",
			"quo", "quo;", "rr;", "rrap;", "rrb;", "rrbfs;", "rrc;", "rrfs;",
			"rrhk;", "rrlp;", "rrpl;", "rrsim;", "rrtl;", "rrw;", "tail;",
			"tio;", "tionals;", "arr;", "brk;", "race;", "rack;", "rke;",
			"rksld;", "rkslu;", "aron;", "edil;", "eil;", "ub;", "y;", "ca;",
			"ldhar;", "quo;", "quor;", "sh;", "al;", "aline;", "alpart;",
			"als;", "ct;", "g", "g;", "isht;", "loor;", "r;", "ard;", "aru;",
			"arul;", "o;", "ov;", "ghtarrow;", "ghtarrowtail;",
			"ghtharpoondown;", "ghtharpoonup;", "ghtleftarrows;",
			"ghtleftharpoons;", "ghtrightarrows;", "ghtsquigarrow;",
			"ghtthreetimes;", "ng;", "singdotseq;", "arr;", "har;", "m;",
			"oust;", "oustache;", "mid;", "ang;", "arr;", "brk;", "par;",
			"pf;", "plus;", "times;", "ar;", "argt;", "polint;", "arr;",
			"aquo;", "cr;", "h;", "qb;", "quo;", "quor;", "hree;", "imes;",
			"ri;", "rie;", "rif;", "riltri;", "luhar;", ";", "cute;", "quo;",
			";", "E;", "ap;", "aron;", "cue;", "e;", "edil;", "irc;", "nE;",
			"nap;", "nsim;", "polint;", "sim;", "y;", "ot;", "otb;", "ote;",
			"Arr;", "arhk;", "arr;", "arrow;", "ct", "ct;", "mi;", "swar;",
			"tminus;", "tmn;", "xt;", "r;", "rown;", "arp;", "chcy;", "cy;",
			"ortmid;", "ortparallel;", "y", "y;", "gma;", "gmaf;", "gmav;",
			"m;", "mdot;", "me;", "meq;", "mg;", "mgE;", "ml;", "mlE;", "mne;",
			"mplus;", "mrarr;", "arr;", "allsetminus;", "ashp;", "eparsl;",
			"id;", "ile;", "t;", "te;", "tes;", "ftcy;", "l;", "lb;", "lbar;",
			"pf;", "ades;", "adesuit;", "ar;", "cap;", "caps;", "cup;",
			"cups;", "sub;", "sube;", "subset;", "subseteq;", "sup;", "supe;",
			"supset;", "supseteq;", "u;", "uare;", "uarf;", "uf;", "arr;",
			"cr;", "etmn;", "mile;", "tarf;", "ar;", "arf;", "raightepsilon;",
			"raightphi;", "rns;", "b;", "bE;", "bdot;", "be;", "bedot;",
			"bmult;", "bnE;", "bne;", "bplus;", "brarr;", "bset;", "bseteq;",
			"bseteqq;", "bsetneq;", "bsetneqq;", "bsim;", "bsub;", "bsup;",
			"cc;", "ccapprox;", "cccurlyeq;", "cceq;", "ccnapprox;", "ccneqq;",
			"ccnsim;", "ccsim;", "m;", "ng;", "p1", "p1;", "p2", "p2;", "p3",
			"p3;", "p;", "pE;", "pdot;", "pdsub;", "pe;", "pedot;", "phsol;",
			"phsub;", "plarr;", "pmult;", "pnE;", "pne;", "pplus;", "pset;",
			"pseteq;", "pseteqq;", "psetneq;", "psetneqq;", "psim;", "psub;",
			"psup;", "Arr;", "arhk;", "arr;", "arrow;", "nwar;", "lig", "lig;",
			"rget;", "u;", "rk;", "aron;", "edil;", "y;", "ot;", "lrec;", "r;",
			"ere4;", "erefore;", "eta;", "etasym;", "etav;", "ickapprox;",
			"icksim;", "insp;", "kap;", "ksim;", "orn", "orn;", "lde;", "mes",
			"mes;", "mesb;", "mesbar;", "mesd;", "nt;", "ea;", "p;", "pbot;",
			"pcir;", "pf;", "pfork;", "sa;", "rime;", "ade;", "iangle;",
			"iangledown;", "iangleleft;", "ianglelefteq;", "iangleq;",
			"iangleright;", "ianglerighteq;", "idot;", "ie;", "iminus;",
			"iplus;", "isb;", "itime;", "pezium;", "cr;", "cy;", "hcy;",
			"trok;", "ixt;", "oheadleftarrow;", "oheadrightarrow;", "rr;",
			"ar;", "cute", "cute;", "rr;", "rcy;", "reve;", "irc", "irc;",
			"y;", "arr;", "blac;", "har;", "isht;", "r;", "rave", "rave;",
			"arl;", "arr;", "blk;", "corn;", "corner;", "crop;", "tri;",
			"acr;", "l", "l;", "gon;", "pf;", "arrow;", "downarrow;",
			"harpoonleft;", "harpoonright;", "lus;", "si;", "sih;", "silon;",
			"uparrows;", "corn;", "corner;", "crop;", "ing;", "tri;", "cr;",
			"dot;", "ilde;", "ri;", "rif;", "arr;", "ml", "ml;", "angle;",
			"rr;", "ar;", "arv;", "ash;", "ngrt;", "repsilon;", "rkappa;",
			"rnothing;", "rphi;", "rpi;", "rpropto;", "rr;", "rrho;",
			"rsigma;", "rsubsetneq;", "rsubsetneqq;", "rsupsetneq;",
			"rsupsetneqq;", "rtheta;", "rtriangleleft;", "rtriangleright;",
			"y;", "ash;", "e;", "ebar;", "eeq;", "llip;", "rbar;", "rt;", "r;",
			"tri;", "sub;", "sup;", "pf;", "rop;", "tri;", "cr;", "ubnE;",
			"ubne;", "upnE;", "upne;", "igzag;", "irc;", "dbar;", "dge;",
			"dgeq;", "ierp;", "r;", "pf;", ";", ";", "eath;", "cr;", "ap;",
			"irc;", "up;", "tri;", "r;", "Arr;", "arr;", ";", "Arr;", "arr;",
			"ap;", "is;", "dot;", "pf;", "plus;", "time;", "Arr;", "arr;",
			"cr;", "qcup;", "plus;", "tri;", "ee;", "edge;", "cute", "cute;",
			"cy;", "irc;", "y;", "n", "n;", "r;", "cy;", "pf;", "cr;", "cy;",
			"ml", "ml;", "cute;", "aron;", "y;", "ot;", "etrf;", "ta;", "r;",
			"cy;", "grarr;", "pf;", "cr;", "j;", "nj;", };

	internal static readonly char[][] VALUES = new char[][] {new char[] { '\u00c6' }, new char[] { '\u00c6' },
			new char[] { '\u0026' }, new char[] { '\u0026' }, new char[] { '\u00c1' }, new char[] { '\u00c1' },
			new char[] { '\u0102' }, new char[] { '\u00c2' }, new char[] { '\u00c2' }, new char[] { '\u0410' },
			new char[] { '\ud835', '\udd04' }, new char[] { '\u00c0' }, new char[] { '\u00c0' }, new char[] { '\u0391' },
			new char[] { '\u0100' }, new char[] { '\u2a53' }, new char[] { '\u0104' }, new char[] { '\ud835', '\udd38' },
			new char[] { '\u2061' }, new char[] { '\u00c5' }, new char[] { '\u00c5' }, new char[] { '\ud835', '\udc9c' },
			new char[] { '\u2254' }, new char[] { '\u00c3' }, new char[] { '\u00c3' }, new char[] { '\u00c4' },
			new char[] { '\u00c4' }, new char[] { '\u2216' }, new char[] { '\u2ae7' }, new char[] { '\u2306' },
			new char[] { '\u0411' }, new char[] { '\u2235' }, new char[] { '\u212c' }, new char[] { '\u0392' },
			new char[] { '\ud835', '\udd05' }, new char[] { '\ud835', '\udd39' }, new char[] { '\u02d8' },
			new char[] { '\u212c' }, new char[] { '\u224e' }, new char[] { '\u0427' }, new char[] { '\u00a9' },
			new char[] { '\u00a9' }, new char[] { '\u0106' }, new char[] { '\u22d2' }, new char[] { '\u2145' },
			new char[] { '\u212d' }, new char[] { '\u010c' }, new char[] { '\u00c7' }, new char[] { '\u00c7' },
			new char[] { '\u0108' }, new char[] { '\u2230' }, new char[] { '\u010a' }, new char[] { '\u00b8' },
			new char[] { '\u00b7' }, new char[] { '\u212d' }, new char[] { '\u03a7' }, new char[] { '\u2299' },
			new char[] { '\u2296' }, new char[] { '\u2295' }, new char[] { '\u2297' }, new char[] { '\u2232' },
			new char[] { '\u201d' }, new char[] { '\u2019' }, new char[] { '\u2237' }, new char[] { '\u2a74' },
			new char[] { '\u2261' }, new char[] { '\u222f' }, new char[] { '\u222e' }, new char[] { '\u2102' },
			new char[] { '\u2210' }, new char[] { '\u2233' }, new char[] { '\u2a2f' }, new char[] { '\ud835', '\udc9e' },
			new char[] { '\u22d3' }, new char[] { '\u224d' }, new char[] { '\u2145' }, new char[] { '\u2911' },
			new char[] { '\u0402' }, new char[] { '\u0405' }, new char[] { '\u040f' }, new char[] { '\u2021' },
			new char[] { '\u21a1' }, new char[] { '\u2ae4' }, new char[] { '\u010e' }, new char[] { '\u0414' },
			new char[] { '\u2207' }, new char[] { '\u0394' }, new char[] { '\ud835', '\udd07' }, new char[] { '\u00b4' },
			new char[] { '\u02d9' }, new char[] { '\u02dd' }, new char[] { '\u0060' }, new char[] { '\u02dc' },
			new char[] { '\u22c4' }, new char[] { '\u2146' }, new char[] { '\ud835', '\udd3b' }, new char[] { '\u00a8' },
			new char[] { '\u20dc' }, new char[] { '\u2250' }, new char[] { '\u222f' }, new char[] { '\u00a8' },
			new char[] { '\u21d3' }, new char[] { '\u21d0' }, new char[] { '\u21d4' }, new char[] { '\u2ae4' },
			new char[] { '\u27f8' }, new char[] { '\u27fa' }, new char[] { '\u27f9' }, new char[] { '\u21d2' },
			new char[] { '\u22a8' }, new char[] { '\u21d1' }, new char[] { '\u21d5' }, new char[] { '\u2225' },
			new char[] { '\u2193' }, new char[] { '\u2913' }, new char[] { '\u21f5' }, new char[] { '\u0311' },
			new char[] { '\u2950' }, new char[] { '\u295e' }, new char[] { '\u21bd' }, new char[] { '\u2956' },
			new char[] { '\u295f' }, new char[] { '\u21c1' }, new char[] { '\u2957' }, new char[] { '\u22a4' },
			new char[] { '\u21a7' }, new char[] { '\u21d3' }, new char[] { '\ud835', '\udc9f' }, new char[] { '\u0110' },
			new char[] { '\u014a' }, new char[] { '\u00d0' }, new char[] { '\u00d0' }, new char[] { '\u00c9' },
			new char[] { '\u00c9' }, new char[] { '\u011a' }, new char[] { '\u00ca' }, new char[] { '\u00ca' },
			new char[] { '\u042d' }, new char[] { '\u0116' }, new char[] { '\ud835', '\udd08' }, new char[] { '\u00c8' },
			new char[] { '\u00c8' }, new char[] { '\u2208' }, new char[] { '\u0112' }, new char[] { '\u25fb' },
			new char[] { '\u25ab' }, new char[] { '\u0118' }, new char[] { '\ud835', '\udd3c' }, new char[] { '\u0395' },
			new char[] { '\u2a75' }, new char[] { '\u2242' }, new char[] { '\u21cc' }, new char[] { '\u2130' },
			new char[] { '\u2a73' }, new char[] { '\u0397' }, new char[] { '\u00cb' }, new char[] { '\u00cb' },
			new char[] { '\u2203' }, new char[] { '\u2147' }, new char[] { '\u0424' }, new char[] { '\ud835', '\udd09' },
			new char[] { '\u25fc' }, new char[] { '\u25aa' }, new char[] { '\ud835', '\udd3d' }, new char[] { '\u2200' },
			new char[] { '\u2131' }, new char[] { '\u2131' }, new char[] { '\u0403' }, new char[] { '\u003e' },
			new char[] { '\u003e' }, new char[] { '\u0393' }, new char[] { '\u03dc' }, new char[] { '\u011e' },
			new char[] { '\u0122' }, new char[] { '\u011c' }, new char[] { '\u0413' }, new char[] { '\u0120' },
			new char[] { '\ud835', '\udd0a' }, new char[] { '\u22d9' }, new char[] { '\ud835', '\udd3e' },
			new char[] { '\u2265' }, new char[] { '\u22db' }, new char[] { '\u2267' }, new char[] { '\u2aa2' },
			new char[] { '\u2277' }, new char[] { '\u2a7e' }, new char[] { '\u2273' }, new char[] { '\ud835', '\udca2' },
			new char[] { '\u226b' }, new char[] { '\u042a' }, new char[] { '\u02c7' }, new char[] { '\u005e' },
			new char[] { '\u0124' }, new char[] { '\u210c' }, new char[] { '\u210b' }, new char[] { '\u210d' },
			new char[] { '\u2500' }, new char[] { '\u210b' }, new char[] { '\u0126' }, new char[] { '\u224e' },
			new char[] { '\u224f' }, new char[] { '\u0415' }, new char[] { '\u0132' }, new char[] { '\u0401' },
			new char[] { '\u00cd' }, new char[] { '\u00cd' }, new char[] { '\u00ce' }, new char[] { '\u00ce' },
			new char[] { '\u0418' }, new char[] { '\u0130' }, new char[] { '\u2111' }, new char[] { '\u00cc' },
			new char[] { '\u00cc' }, new char[] { '\u2111' }, new char[] { '\u012a' }, new char[] { '\u2148' },
			new char[] { '\u21d2' }, new char[] { '\u222c' }, new char[] { '\u222b' }, new char[] { '\u22c2' },
			new char[] { '\u2063' }, new char[] { '\u2062' }, new char[] { '\u012e' }, new char[] { '\ud835', '\udd40' },
			new char[] { '\u0399' }, new char[] { '\u2110' }, new char[] { '\u0128' }, new char[] { '\u0406' },
			new char[] { '\u00cf' }, new char[] { '\u00cf' }, new char[] { '\u0134' }, new char[] { '\u0419' },
			new char[] { '\ud835', '\udd0d' }, new char[] { '\ud835', '\udd41' },
			new char[] { '\ud835', '\udca5' }, new char[] { '\u0408' }, new char[] { '\u0404' }, new char[] { '\u0425' },
			new char[] { '\u040c' }, new char[] { '\u039a' }, new char[] { '\u0136' }, new char[] { '\u041a' },
			new char[] { '\ud835', '\udd0e' }, new char[] { '\ud835', '\udd42' },
			new char[] { '\ud835', '\udca6' }, new char[] { '\u0409' }, new char[] { '\u003c' }, new char[] { '\u003c' },
			new char[] { '\u0139' }, new char[] { '\u039b' }, new char[] { '\u27ea' }, new char[] { '\u2112' },
			new char[] { '\u219e' }, new char[] { '\u013d' }, new char[] { '\u013b' }, new char[] { '\u041b' },
			new char[] { '\u27e8' }, new char[] { '\u2190' }, new char[] { '\u21e4' }, new char[] { '\u21c6' },
			new char[] { '\u2308' }, new char[] { '\u27e6' }, new char[] { '\u2961' }, new char[] { '\u21c3' },
			new char[] { '\u2959' }, new char[] { '\u230a' }, new char[] { '\u2194' }, new char[] { '\u294e' },
			new char[] { '\u22a3' }, new char[] { '\u21a4' }, new char[] { '\u295a' }, new char[] { '\u22b2' },
			new char[] { '\u29cf' }, new char[] { '\u22b4' }, new char[] { '\u2951' }, new char[] { '\u2960' },
			new char[] { '\u21bf' }, new char[] { '\u2958' }, new char[] { '\u21bc' }, new char[] { '\u2952' },
			new char[] { '\u21d0' }, new char[] { '\u21d4' }, new char[] { '\u22da' }, new char[] { '\u2266' },
			new char[] { '\u2276' }, new char[] { '\u2aa1' }, new char[] { '\u2a7d' }, new char[] { '\u2272' },
			new char[] { '\ud835', '\udd0f' }, new char[] { '\u22d8' }, new char[] { '\u21da' }, new char[] { '\u013f' },
			new char[] { '\u27f5' }, new char[] { '\u27f7' }, new char[] { '\u27f6' }, new char[] { '\u27f8' },
			new char[] { '\u27fa' }, new char[] { '\u27f9' }, new char[] { '\ud835', '\udd43' }, new char[] { '\u2199' },
			new char[] { '\u2198' }, new char[] { '\u2112' }, new char[] { '\u21b0' }, new char[] { '\u0141' },
			new char[] { '\u226a' }, new char[] { '\u2905' }, new char[] { '\u041c' }, new char[] { '\u205f' },
			new char[] { '\u2133' }, new char[] { '\ud835', '\udd10' }, new char[] { '\u2213' },
			new char[] { '\ud835', '\udd44' }, new char[] { '\u2133' }, new char[] { '\u039c' }, new char[] { '\u040a' },
			new char[] { '\u0143' }, new char[] { '\u0147' }, new char[] { '\u0145' }, new char[] { '\u041d' },
			new char[] { '\u200b' }, new char[] { '\u200b' }, new char[] { '\u200b' }, new char[] { '\u200b' },
			new char[] { '\u226b' }, new char[] { '\u226a' }, new char[] { '\n' }, new char[] { '\ud835', '\udd11' },
			new char[] { '\u2060' }, new char[] { '\u00a0' }, new char[] { '\u2115' }, new char[] { '\u2aec' },
			new char[] { '\u2262' }, new char[] { '\u226d' }, new char[] { '\u2226' }, new char[] { '\u2209' },
			new char[] { '\u2260' }, new char[] { '\u2242', '\u0338' }, new char[] { '\u2204' }, new char[] { '\u226f' },
			new char[] { '\u2271' }, new char[] { '\u2267', '\u0338' }, new char[] { '\u226b', '\u0338' },
			new char[] { '\u2279' }, new char[] { '\u2a7e', '\u0338' }, new char[] { '\u2275' },
			new char[] { '\u224e', '\u0338' }, new char[] { '\u224f', '\u0338' }, new char[] { '\u22ea' },
			new char[] { '\u29cf', '\u0338' }, new char[] { '\u22ec' }, new char[] { '\u226e' }, new char[] { '\u2270' },
			new char[] { '\u2278' }, new char[] { '\u226a', '\u0338' }, new char[] { '\u2a7d', '\u0338' },
			new char[] { '\u2274' }, new char[] { '\u2aa2', '\u0338' }, new char[] { '\u2aa1', '\u0338' },
			new char[] { '\u2280' }, new char[] { '\u2aaf', '\u0338' }, new char[] { '\u22e0' }, new char[] { '\u220c' },
			new char[] { '\u22eb' }, new char[] { '\u29d0', '\u0338' }, new char[] { '\u22ed' },
			new char[] { '\u228f', '\u0338' }, new char[] { '\u22e2' }, new char[] { '\u2290', '\u0338' },
			new char[] { '\u22e3' }, new char[] { '\u2282', '\u20d2' }, new char[] { '\u2288' }, new char[] { '\u2281' },
			new char[] { '\u2ab0', '\u0338' }, new char[] { '\u22e1' }, new char[] { '\u227f', '\u0338' },
			new char[] { '\u2283', '\u20d2' }, new char[] { '\u2289' }, new char[] { '\u2241' }, new char[] { '\u2244' },
			new char[] { '\u2247' }, new char[] { '\u2249' }, new char[] { '\u2224' }, new char[] { '\ud835', '\udca9' },
			new char[] { '\u00d1' }, new char[] { '\u00d1' }, new char[] { '\u039d' }, new char[] { '\u0152' },
			new char[] { '\u00d3' }, new char[] { '\u00d3' }, new char[] { '\u00d4' }, new char[] { '\u00d4' },
			new char[] { '\u041e' }, new char[] { '\u0150' }, new char[] { '\ud835', '\udd12' }, new char[] { '\u00d2' },
			new char[] { '\u00d2' }, new char[] { '\u014c' }, new char[] { '\u03a9' }, new char[] { '\u039f' },
			new char[] { '\ud835', '\udd46' }, new char[] { '\u201c' }, new char[] { '\u2018' }, new char[] { '\u2a54' },
			new char[] { '\ud835', '\udcaa' }, new char[] { '\u00d8' }, new char[] { '\u00d8' }, new char[] { '\u00d5' },
			new char[] { '\u00d5' }, new char[] { '\u2a37' }, new char[] { '\u00d6' }, new char[] { '\u00d6' },
			new char[] { '\u203e' }, new char[] { '\u23de' }, new char[] { '\u23b4' }, new char[] { '\u23dc' },
			new char[] { '\u2202' }, new char[] { '\u041f' }, new char[] { '\ud835', '\udd13' }, new char[] { '\u03a6' },
			new char[] { '\u03a0' }, new char[] { '\u00b1' }, new char[] { '\u210c' }, new char[] { '\u2119' },
			new char[] { '\u2abb' }, new char[] { '\u227a' }, new char[] { '\u2aaf' }, new char[] { '\u227c' },
			new char[] { '\u227e' }, new char[] { '\u2033' }, new char[] { '\u220f' }, new char[] { '\u2237' },
			new char[] { '\u221d' }, new char[] { '\ud835', '\udcab' }, new char[] { '\u03a8' }, new char[] { '\u0022' },
			new char[] { '\u0022' }, new char[] { '\ud835', '\udd14' }, new char[] { '\u211a' },
			new char[] { '\ud835', '\udcac' }, new char[] { '\u2910' }, new char[] { '\u00ae' }, new char[] { '\u00ae' },
			new char[] { '\u0154' }, new char[] { '\u27eb' }, new char[] { '\u21a0' }, new char[] { '\u2916' },
			new char[] { '\u0158' }, new char[] { '\u0156' }, new char[] { '\u0420' }, new char[] { '\u211c' },
			new char[] { '\u220b' }, new char[] { '\u21cb' }, new char[] { '\u296f' }, new char[] { '\u211c' },
			new char[] { '\u03a1' }, new char[] { '\u27e9' }, new char[] { '\u2192' }, new char[] { '\u21e5' },
			new char[] { '\u21c4' }, new char[] { '\u2309' }, new char[] { '\u27e7' }, new char[] { '\u295d' },
			new char[] { '\u21c2' }, new char[] { '\u2955' }, new char[] { '\u230b' }, new char[] { '\u22a2' },
			new char[] { '\u21a6' }, new char[] { '\u295b' }, new char[] { '\u22b3' }, new char[] { '\u29d0' },
			new char[] { '\u22b5' }, new char[] { '\u294f' }, new char[] { '\u295c' }, new char[] { '\u21be' },
			new char[] { '\u2954' }, new char[] { '\u21c0' }, new char[] { '\u2953' }, new char[] { '\u21d2' },
			new char[] { '\u211d' }, new char[] { '\u2970' }, new char[] { '\u21db' }, new char[] { '\u211b' },
			new char[] { '\u21b1' }, new char[] { '\u29f4' }, new char[] { '\u0429' }, new char[] { '\u0428' },
			new char[] { '\u042c' }, new char[] { '\u015a' }, new char[] { '\u2abc' }, new char[] { '\u0160' },
			new char[] { '\u015e' }, new char[] { '\u015c' }, new char[] { '\u0421' }, new char[] { '\ud835', '\udd16' },
			new char[] { '\u2193' }, new char[] { '\u2190' }, new char[] { '\u2192' }, new char[] { '\u2191' },
			new char[] { '\u03a3' }, new char[] { '\u2218' }, new char[] { '\ud835', '\udd4a' }, new char[] { '\u221a' },
			new char[] { '\u25a1' }, new char[] { '\u2293' }, new char[] { '\u228f' }, new char[] { '\u2291' },
			new char[] { '\u2290' }, new char[] { '\u2292' }, new char[] { '\u2294' }, new char[] { '\ud835', '\udcae' },
			new char[] { '\u22c6' }, new char[] { '\u22d0' }, new char[] { '\u22d0' }, new char[] { '\u2286' },
			new char[] { '\u227b' }, new char[] { '\u2ab0' }, new char[] { '\u227d' }, new char[] { '\u227f' },
			new char[] { '\u220b' }, new char[] { '\u2211' }, new char[] { '\u22d1' }, new char[] { '\u2283' },
			new char[] { '\u2287' }, new char[] { '\u22d1' }, new char[] { '\u00de' }, new char[] { '\u00de' },
			new char[] { '\u2122' }, new char[] { '\u040b' }, new char[] { '\u0426' }, new char[] { '\u0009' },
			new char[] { '\u03a4' }, new char[] { '\u0164' }, new char[] { '\u0162' }, new char[] { '\u0422' },
			new char[] { '\ud835', '\udd17' }, new char[] { '\u2234' }, new char[] { '\u0398' },
			new char[] { '\u205f', '\u200a' }, new char[] { '\u2009' }, new char[] { '\u223c' }, new char[] { '\u2243' },
			new char[] { '\u2245' }, new char[] { '\u2248' }, new char[] { '\ud835', '\udd4b' }, new char[] { '\u20db' },
			new char[] { '\ud835', '\udcaf' }, new char[] { '\u0166' }, new char[] { '\u00da' }, new char[] { '\u00da' },
			new char[] { '\u219f' }, new char[] { '\u2949' }, new char[] { '\u040e' }, new char[] { '\u016c' },
			new char[] { '\u00db' }, new char[] { '\u00db' }, new char[] { '\u0423' }, new char[] { '\u0170' },
			new char[] { '\ud835', '\udd18' }, new char[] { '\u00d9' }, new char[] { '\u00d9' }, new char[] { '\u016a' },
			new char[] { '\u005f' }, new char[] { '\u23df' }, new char[] { '\u23b5' }, new char[] { '\u23dd' },
			new char[] { '\u22c3' }, new char[] { '\u228e' }, new char[] { '\u0172' }, new char[] { '\ud835', '\udd4c' },
			new char[] { '\u2191' }, new char[] { '\u2912' }, new char[] { '\u21c5' }, new char[] { '\u2195' },
			new char[] { '\u296e' }, new char[] { '\u22a5' }, new char[] { '\u21a5' }, new char[] { '\u21d1' },
			new char[] { '\u21d5' }, new char[] { '\u2196' }, new char[] { '\u2197' }, new char[] { '\u03d2' },
			new char[] { '\u03a5' }, new char[] { '\u016e' }, new char[] { '\ud835', '\udcb0' }, new char[] { '\u0168' },
			new char[] { '\u00dc' }, new char[] { '\u00dc' }, new char[] { '\u22ab' }, new char[] { '\u2aeb' },
			new char[] { '\u0412' }, new char[] { '\u22a9' }, new char[] { '\u2ae6' }, new char[] { '\u22c1' },
			new char[] { '\u2016' }, new char[] { '\u2016' }, new char[] { '\u2223' }, new char[] { '\u007c' },
			new char[] { '\u2758' }, new char[] { '\u2240' }, new char[] { '\u200a' }, new char[] { '\ud835', '\udd19' },
			new char[] { '\ud835', '\udd4d' }, new char[] { '\ud835', '\udcb1' }, new char[] { '\u22aa' },
			new char[] { '\u0174' }, new char[] { '\u22c0' }, new char[] { '\ud835', '\udd1a' },
			new char[] { '\ud835', '\udd4e' }, new char[] { '\ud835', '\udcb2' },
			new char[] { '\ud835', '\udd1b' }, new char[] { '\u039e' }, new char[] { '\ud835', '\udd4f' },
			new char[] { '\ud835', '\udcb3' }, new char[] { '\u042f' }, new char[] { '\u0407' }, new char[] { '\u042e' },
			new char[] { '\u00dd' }, new char[] { '\u00dd' }, new char[] { '\u0176' }, new char[] { '\u042b' },
			new char[] { '\ud835', '\udd1c' }, new char[] { '\ud835', '\udd50' },
			new char[] { '\ud835', '\udcb4' }, new char[] { '\u0178' }, new char[] { '\u0416' }, new char[] { '\u0179' },
			new char[] { '\u017d' }, new char[] { '\u0417' }, new char[] { '\u017b' }, new char[] { '\u200b' },
			new char[] { '\u0396' }, new char[] { '\u2128' }, new char[] { '\u2124' }, new char[] { '\ud835', '\udcb5' },
			new char[] { '\u00e1' }, new char[] { '\u00e1' }, new char[] { '\u0103' }, new char[] { '\u223e' },
			new char[] { '\u223e', '\u0333' }, new char[] { '\u223f' }, new char[] { '\u00e2' }, new char[] { '\u00e2' },
			new char[] { '\u00b4' }, new char[] { '\u00b4' }, new char[] { '\u0430' }, new char[] { '\u00e6' },
			new char[] { '\u00e6' }, new char[] { '\u2061' }, new char[] { '\ud835', '\udd1e' }, new char[] { '\u00e0' },
			new char[] { '\u00e0' }, new char[] { '\u2135' }, new char[] { '\u2135' }, new char[] { '\u03b1' },
			new char[] { '\u0101' }, new char[] { '\u2a3f' }, new char[] { '\u0026' }, new char[] { '\u0026' },
			new char[] { '\u2227' }, new char[] { '\u2a55' }, new char[] { '\u2a5c' }, new char[] { '\u2a58' },
			new char[] { '\u2a5a' }, new char[] { '\u2220' }, new char[] { '\u29a4' }, new char[] { '\u2220' },
			new char[] { '\u2221' }, new char[] { '\u29a8' }, new char[] { '\u29a9' }, new char[] { '\u29aa' },
			new char[] { '\u29ab' }, new char[] { '\u29ac' }, new char[] { '\u29ad' }, new char[] { '\u29ae' },
			new char[] { '\u29af' }, new char[] { '\u221f' }, new char[] { '\u22be' }, new char[] { '\u299d' },
			new char[] { '\u2222' }, new char[] { '\u00c5' }, new char[] { '\u237c' }, new char[] { '\u0105' },
			new char[] { '\ud835', '\udd52' }, new char[] { '\u2248' }, new char[] { '\u2a70' }, new char[] { '\u2a6f' },
			new char[] { '\u224a' }, new char[] { '\u224b' }, new char[] { '\'' }, new char[] { '\u2248' }, new char[] { '\u224a' },
			new char[] { '\u00e5' }, new char[] { '\u00e5' }, new char[] { '\ud835', '\udcb6' }, new char[] { '\u002a' },
			new char[] { '\u2248' }, new char[] { '\u224d' }, new char[] { '\u00e3' }, new char[] { '\u00e3' },
			new char[] { '\u00e4' }, new char[] { '\u00e4' }, new char[] { '\u2233' }, new char[] { '\u2a11' },
			new char[] { '\u2aed' }, new char[] { '\u224c' }, new char[] { '\u03f6' }, new char[] { '\u2035' },
			new char[] { '\u223d' }, new char[] { '\u22cd' }, new char[] { '\u22bd' }, new char[] { '\u2305' },
			new char[] { '\u2305' }, new char[] { '\u23b5' }, new char[] { '\u23b6' }, new char[] { '\u224c' },
			new char[] { '\u0431' }, new char[] { '\u201e' }, new char[] { '\u2235' }, new char[] { '\u2235' },
			new char[] { '\u29b0' }, new char[] { '\u03f6' }, new char[] { '\u212c' }, new char[] { '\u03b2' },
			new char[] { '\u2136' }, new char[] { '\u226c' }, new char[] { '\ud835', '\udd1f' }, new char[] { '\u22c2' },
			new char[] { '\u25ef' }, new char[] { '\u22c3' }, new char[] { '\u2a00' }, new char[] { '\u2a01' },
			new char[] { '\u2a02' }, new char[] { '\u2a06' }, new char[] { '\u2605' }, new char[] { '\u25bd' },
			new char[] { '\u25b3' }, new char[] { '\u2a04' }, new char[] { '\u22c1' }, new char[] { '\u22c0' },
			new char[] { '\u290d' }, new char[] { '\u29eb' }, new char[] { '\u25aa' }, new char[] { '\u25b4' },
			new char[] { '\u25be' }, new char[] { '\u25c2' }, new char[] { '\u25b8' }, new char[] { '\u2423' },
			new char[] { '\u2592' }, new char[] { '\u2591' }, new char[] { '\u2593' }, new char[] { '\u2588' },
			new char[] { '\u003d', '\u20e5' }, new char[] { '\u2261', '\u20e5' }, new char[] { '\u2310' },
			new char[] { '\ud835', '\udd53' }, new char[] { '\u22a5' }, new char[] { '\u22a5' }, new char[] { '\u22c8' },
			new char[] { '\u2557' }, new char[] { '\u2554' }, new char[] { '\u2556' }, new char[] { '\u2553' },
			new char[] { '\u2550' }, new char[] { '\u2566' }, new char[] { '\u2569' }, new char[] { '\u2564' },
			new char[] { '\u2567' }, new char[] { '\u255d' }, new char[] { '\u255a' }, new char[] { '\u255c' },
			new char[] { '\u2559' }, new char[] { '\u2551' }, new char[] { '\u256c' }, new char[] { '\u2563' },
			new char[] { '\u2560' }, new char[] { '\u256b' }, new char[] { '\u2562' }, new char[] { '\u255f' },
			new char[] { '\u29c9' }, new char[] { '\u2555' }, new char[] { '\u2552' }, new char[] { '\u2510' },
			new char[] { '\u250c' }, new char[] { '\u2500' }, new char[] { '\u2565' }, new char[] { '\u2568' },
			new char[] { '\u252c' }, new char[] { '\u2534' }, new char[] { '\u229f' }, new char[] { '\u229e' },
			new char[] { '\u22a0' }, new char[] { '\u255b' }, new char[] { '\u2558' }, new char[] { '\u2518' },
			new char[] { '\u2514' }, new char[] { '\u2502' }, new char[] { '\u256a' }, new char[] { '\u2561' },
			new char[] { '\u255e' }, new char[] { '\u253c' }, new char[] { '\u2524' }, new char[] { '\u251c' },
			new char[] { '\u2035' }, new char[] { '\u02d8' }, new char[] { '\u00a6' }, new char[] { '\u00a6' },
			new char[] { '\ud835', '\udcb7' }, new char[] { '\u204f' }, new char[] { '\u223d' }, new char[] { '\u22cd' },
			new char[] { '\\' }, new char[] { '\u29c5' }, new char[] { '\u27c8' }, new char[] { '\u2022' }, new char[] { '\u2022' },
			new char[] { '\u224e' }, new char[] { '\u2aae' }, new char[] { '\u224f' }, new char[] { '\u224f' },
			new char[] { '\u0107' }, new char[] { '\u2229' }, new char[] { '\u2a44' }, new char[] { '\u2a49' },
			new char[] { '\u2a4b' }, new char[] { '\u2a47' }, new char[] { '\u2a40' }, new char[] { '\u2229', '\ufe00' },
			new char[] { '\u2041' }, new char[] { '\u02c7' }, new char[] { '\u2a4d' }, new char[] { '\u010d' },
			new char[] { '\u00e7' }, new char[] { '\u00e7' }, new char[] { '\u0109' }, new char[] { '\u2a4c' },
			new char[] { '\u2a50' }, new char[] { '\u010b' }, new char[] { '\u00b8' }, new char[] { '\u00b8' },
			new char[] { '\u29b2' }, new char[] { '\u00a2' }, new char[] { '\u00a2' }, new char[] { '\u00b7' },
			new char[] { '\ud835', '\udd20' }, new char[] { '\u0447' }, new char[] { '\u2713' }, new char[] { '\u2713' },
			new char[] { '\u03c7' }, new char[] { '\u25cb' }, new char[] { '\u29c3' }, new char[] { '\u02c6' },
			new char[] { '\u2257' }, new char[] { '\u21ba' }, new char[] { '\u21bb' }, new char[] { '\u00ae' },
			new char[] { '\u24c8' }, new char[] { '\u229b' }, new char[] { '\u229a' }, new char[] { '\u229d' },
			new char[] { '\u2257' }, new char[] { '\u2a10' }, new char[] { '\u2aef' }, new char[] { '\u29c2' },
			new char[] { '\u2663' }, new char[] { '\u2663' }, new char[] { '\u003a' }, new char[] { '\u2254' },
			new char[] { '\u2254' }, new char[] { '\u002c' }, new char[] { '\u0040' }, new char[] { '\u2201' },
			new char[] { '\u2218' }, new char[] { '\u2201' }, new char[] { '\u2102' }, new char[] { '\u2245' },
			new char[] { '\u2a6d' }, new char[] { '\u222e' }, new char[] { '\ud835', '\udd54' }, new char[] { '\u2210' },
			new char[] { '\u00a9' }, new char[] { '\u00a9' }, new char[] { '\u2117' }, new char[] { '\u21b5' },
			new char[] { '\u2717' }, new char[] { '\ud835', '\udcb8' }, new char[] { '\u2acf' }, new char[] { '\u2ad1' },
			new char[] { '\u2ad0' }, new char[] { '\u2ad2' }, new char[] { '\u22ef' }, new char[] { '\u2938' },
			new char[] { '\u2935' }, new char[] { '\u22de' }, new char[] { '\u22df' }, new char[] { '\u21b6' },
			new char[] { '\u293d' }, new char[] { '\u222a' }, new char[] { '\u2a48' }, new char[] { '\u2a46' },
			new char[] { '\u2a4a' }, new char[] { '\u228d' }, new char[] { '\u2a45' }, new char[] { '\u222a', '\ufe00' },
			new char[] { '\u21b7' }, new char[] { '\u293c' }, new char[] { '\u22de' }, new char[] { '\u22df' },
			new char[] { '\u22ce' }, new char[] { '\u22cf' }, new char[] { '\u00a4' }, new char[] { '\u00a4' },
			new char[] { '\u21b6' }, new char[] { '\u21b7' }, new char[] { '\u22ce' }, new char[] { '\u22cf' },
			new char[] { '\u2232' }, new char[] { '\u2231' }, new char[] { '\u232d' }, new char[] { '\u21d3' },
			new char[] { '\u2965' }, new char[] { '\u2020' }, new char[] { '\u2138' }, new char[] { '\u2193' },
			new char[] { '\u2010' }, new char[] { '\u22a3' }, new char[] { '\u290f' }, new char[] { '\u02dd' },
			new char[] { '\u010f' }, new char[] { '\u0434' }, new char[] { '\u2146' }, new char[] { '\u2021' },
			new char[] { '\u21ca' }, new char[] { '\u2a77' }, new char[] { '\u00b0' }, new char[] { '\u00b0' },
			new char[] { '\u03b4' }, new char[] { '\u29b1' }, new char[] { '\u297f' }, new char[] { '\ud835', '\udd21' },
			new char[] { '\u21c3' }, new char[] { '\u21c2' }, new char[] { '\u22c4' }, new char[] { '\u22c4' },
			new char[] { '\u2666' }, new char[] { '\u2666' }, new char[] { '\u00a8' }, new char[] { '\u03dd' },
			new char[] { '\u22f2' }, new char[] { '\u00f7' }, new char[] { '\u00f7' }, new char[] { '\u00f7' },
			new char[] { '\u22c7' }, new char[] { '\u22c7' }, new char[] { '\u0452' }, new char[] { '\u231e' },
			new char[] { '\u230d' }, new char[] { '\u0024' }, new char[] { '\ud835', '\udd55' }, new char[] { '\u02d9' },
			new char[] { '\u2250' }, new char[] { '\u2251' }, new char[] { '\u2238' }, new char[] { '\u2214' },
			new char[] { '\u22a1' }, new char[] { '\u2306' }, new char[] { '\u2193' }, new char[] { '\u21ca' },
			new char[] { '\u21c3' }, new char[] { '\u21c2' }, new char[] { '\u2910' }, new char[] { '\u231f' },
			new char[] { '\u230c' }, new char[] { '\ud835', '\udcb9' }, new char[] { '\u0455' }, new char[] { '\u29f6' },
			new char[] { '\u0111' }, new char[] { '\u22f1' }, new char[] { '\u25bf' }, new char[] { '\u25be' },
			new char[] { '\u21f5' }, new char[] { '\u296f' }, new char[] { '\u29a6' }, new char[] { '\u045f' },
			new char[] { '\u27ff' }, new char[] { '\u2a77' }, new char[] { '\u2251' }, new char[] { '\u00e9' },
			new char[] { '\u00e9' }, new char[] { '\u2a6e' }, new char[] { '\u011b' }, new char[] { '\u2256' },
			new char[] { '\u00ea' }, new char[] { '\u00ea' }, new char[] { '\u2255' }, new char[] { '\u044d' },
			new char[] { '\u0117' }, new char[] { '\u2147' }, new char[] { '\u2252' }, new char[] { '\ud835', '\udd22' },
			new char[] { '\u2a9a' }, new char[] { '\u00e8' }, new char[] { '\u00e8' }, new char[] { '\u2a96' },
			new char[] { '\u2a98' }, new char[] { '\u2a99' }, new char[] { '\u23e7' }, new char[] { '\u2113' },
			new char[] { '\u2a95' }, new char[] { '\u2a97' }, new char[] { '\u0113' }, new char[] { '\u2205' },
			new char[] { '\u2205' }, new char[] { '\u2205' }, new char[] { '\u2004' }, new char[] { '\u2005' },
			new char[] { '\u2003' }, new char[] { '\u014b' }, new char[] { '\u2002' }, new char[] { '\u0119' },
			new char[] { '\ud835', '\udd56' }, new char[] { '\u22d5' }, new char[] { '\u29e3' }, new char[] { '\u2a71' },
			new char[] { '\u03b5' }, new char[] { '\u03b5' }, new char[] { '\u03f5' }, new char[] { '\u2256' },
			new char[] { '\u2255' }, new char[] { '\u2242' }, new char[] { '\u2a96' }, new char[] { '\u2a95' },
			new char[] { '\u003d' }, new char[] { '\u225f' }, new char[] { '\u2261' }, new char[] { '\u2a78' },
			new char[] { '\u29e5' }, new char[] { '\u2253' }, new char[] { '\u2971' }, new char[] { '\u212f' },
			new char[] { '\u2250' }, new char[] { '\u2242' }, new char[] { '\u03b7' }, new char[] { '\u00f0' },
			new char[] { '\u00f0' }, new char[] { '\u00eb' }, new char[] { '\u00eb' }, new char[] { '\u20ac' },
			new char[] { '\u0021' }, new char[] { '\u2203' }, new char[] { '\u2130' }, new char[] { '\u2147' },
			new char[] { '\u2252' }, new char[] { '\u0444' }, new char[] { '\u2640' }, new char[] { '\ufb03' },
			new char[] { '\ufb00' }, new char[] { '\ufb04' }, new char[] { '\ud835', '\udd23' }, new char[] { '\ufb01' },
			new char[] { '\u0066', '\u006a' }, new char[] { '\u266d' }, new char[] { '\ufb02' }, new char[] { '\u25b1' },
			new char[] { '\u0192' }, new char[] { '\ud835', '\udd57' }, new char[] { '\u2200' }, new char[] { '\u22d4' },
			new char[] { '\u2ad9' }, new char[] { '\u2a0d' }, new char[] { '\u00bd' }, new char[] { '\u00bd' },
			new char[] { '\u2153' }, new char[] { '\u00bc' }, new char[] { '\u00bc' }, new char[] { '\u2155' },
			new char[] { '\u2159' }, new char[] { '\u215b' }, new char[] { '\u2154' }, new char[] { '\u2156' },
			new char[] { '\u00be' }, new char[] { '\u00be' }, new char[] { '\u2157' }, new char[] { '\u215c' },
			new char[] { '\u2158' }, new char[] { '\u215a' }, new char[] { '\u215d' }, new char[] { '\u215e' },
			new char[] { '\u2044' }, new char[] { '\u2322' }, new char[] { '\ud835', '\udcbb' }, new char[] { '\u2267' },
			new char[] { '\u2a8c' }, new char[] { '\u01f5' }, new char[] { '\u03b3' }, new char[] { '\u03dd' },
			new char[] { '\u2a86' }, new char[] { '\u011f' }, new char[] { '\u011d' }, new char[] { '\u0433' },
			new char[] { '\u0121' }, new char[] { '\u2265' }, new char[] { '\u22db' }, new char[] { '\u2265' },
			new char[] { '\u2267' }, new char[] { '\u2a7e' }, new char[] { '\u2a7e' }, new char[] { '\u2aa9' },
			new char[] { '\u2a80' }, new char[] { '\u2a82' }, new char[] { '\u2a84' }, new char[] { '\u22db', '\ufe00' },
			new char[] { '\u2a94' }, new char[] { '\ud835', '\udd24' }, new char[] { '\u226b' }, new char[] { '\u22d9' },
			new char[] { '\u2137' }, new char[] { '\u0453' }, new char[] { '\u2277' }, new char[] { '\u2a92' },
			new char[] { '\u2aa5' }, new char[] { '\u2aa4' }, new char[] { '\u2269' }, new char[] { '\u2a8a' },
			new char[] { '\u2a8a' }, new char[] { '\u2a88' }, new char[] { '\u2a88' }, new char[] { '\u2269' },
			new char[] { '\u22e7' }, new char[] { '\ud835', '\udd58' }, new char[] { '\u0060' }, new char[] { '\u210a' },
			new char[] { '\u2273' }, new char[] { '\u2a8e' }, new char[] { '\u2a90' }, new char[] { '\u003e' },
			new char[] { '\u003e' }, new char[] { '\u2aa7' }, new char[] { '\u2a7a' }, new char[] { '\u22d7' },
			new char[] { '\u2995' }, new char[] { '\u2a7c' }, new char[] { '\u2a86' }, new char[] { '\u2978' },
			new char[] { '\u22d7' }, new char[] { '\u22db' }, new char[] { '\u2a8c' }, new char[] { '\u2277' },
			new char[] { '\u2273' }, new char[] { '\u2269', '\ufe00' }, new char[] { '\u2269', '\ufe00' },
			new char[] { '\u21d4' }, new char[] { '\u200a' }, new char[] { '\u00bd' }, new char[] { '\u210b' },
			new char[] { '\u044a' }, new char[] { '\u2194' }, new char[] { '\u2948' }, new char[] { '\u21ad' },
			new char[] { '\u210f' }, new char[] { '\u0125' }, new char[] { '\u2665' }, new char[] { '\u2665' },
			new char[] { '\u2026' }, new char[] { '\u22b9' }, new char[] { '\ud835', '\udd25' }, new char[] { '\u2925' },
			new char[] { '\u2926' }, new char[] { '\u21ff' }, new char[] { '\u223b' }, new char[] { '\u21a9' },
			new char[] { '\u21aa' }, new char[] { '\ud835', '\udd59' }, new char[] { '\u2015' },
			new char[] { '\ud835', '\udcbd' }, new char[] { '\u210f' }, new char[] { '\u0127' }, new char[] { '\u2043' },
			new char[] { '\u2010' }, new char[] { '\u00ed' }, new char[] { '\u00ed' }, new char[] { '\u2063' },
			new char[] { '\u00ee' }, new char[] { '\u00ee' }, new char[] { '\u0438' }, new char[] { '\u0435' },
			new char[] { '\u00a1' }, new char[] { '\u00a1' }, new char[] { '\u21d4' }, new char[] { '\ud835', '\udd26' },
			new char[] { '\u00ec' }, new char[] { '\u00ec' }, new char[] { '\u2148' }, new char[] { '\u2a0c' },
			new char[] { '\u222d' }, new char[] { '\u29dc' }, new char[] { '\u2129' }, new char[] { '\u0133' },
			new char[] { '\u012b' }, new char[] { '\u2111' }, new char[] { '\u2110' }, new char[] { '\u2111' },
			new char[] { '\u0131' }, new char[] { '\u22b7' }, new char[] { '\u01b5' }, new char[] { '\u2208' },
			new char[] { '\u2105' }, new char[] { '\u221e' }, new char[] { '\u29dd' }, new char[] { '\u0131' },
			new char[] { '\u222b' }, new char[] { '\u22ba' }, new char[] { '\u2124' }, new char[] { '\u22ba' },
			new char[] { '\u2a17' }, new char[] { '\u2a3c' }, new char[] { '\u0451' }, new char[] { '\u012f' },
			new char[] { '\ud835', '\udd5a' }, new char[] { '\u03b9' }, new char[] { '\u2a3c' }, new char[] { '\u00bf' },
			new char[] { '\u00bf' }, new char[] { '\ud835', '\udcbe' }, new char[] { '\u2208' }, new char[] { '\u22f9' },
			new char[] { '\u22f5' }, new char[] { '\u22f4' }, new char[] { '\u22f3' }, new char[] { '\u2208' },
			new char[] { '\u2062' }, new char[] { '\u0129' }, new char[] { '\u0456' }, new char[] { '\u00ef' },
			new char[] { '\u00ef' }, new char[] { '\u0135' }, new char[] { '\u0439' }, new char[] { '\ud835', '\udd27' },
			new char[] { '\u0237' }, new char[] { '\ud835', '\udd5b' }, new char[] { '\ud835', '\udcbf' },
			new char[] { '\u0458' }, new char[] { '\u0454' }, new char[] { '\u03ba' }, new char[] { '\u03f0' },
			new char[] { '\u0137' }, new char[] { '\u043a' }, new char[] { '\ud835', '\udd28' }, new char[] { '\u0138' },
			new char[] { '\u0445' }, new char[] { '\u045c' }, new char[] { '\ud835', '\udd5c' },
			new char[] { '\ud835', '\udcc0' }, new char[] { '\u21da' }, new char[] { '\u21d0' }, new char[] { '\u291b' },
			new char[] { '\u290e' }, new char[] { '\u2266' }, new char[] { '\u2a8b' }, new char[] { '\u2962' },
			new char[] { '\u013a' }, new char[] { '\u29b4' }, new char[] { '\u2112' }, new char[] { '\u03bb' },
			new char[] { '\u27e8' }, new char[] { '\u2991' }, new char[] { '\u27e8' }, new char[] { '\u2a85' },
			new char[] { '\u00ab' }, new char[] { '\u00ab' }, new char[] { '\u2190' }, new char[] { '\u21e4' },
			new char[] { '\u291f' }, new char[] { '\u291d' }, new char[] { '\u21a9' }, new char[] { '\u21ab' },
			new char[] { '\u2939' }, new char[] { '\u2973' }, new char[] { '\u21a2' }, new char[] { '\u2aab' },
			new char[] { '\u2919' }, new char[] { '\u2aad' }, new char[] { '\u2aad', '\ufe00' }, new char[] { '\u290c' },
			new char[] { '\u2772' }, new char[] { '\u007b' }, new char[] { '\u005b' }, new char[] { '\u298b' },
			new char[] { '\u298f' }, new char[] { '\u298d' }, new char[] { '\u013e' }, new char[] { '\u013c' },
			new char[] { '\u2308' }, new char[] { '\u007b' }, new char[] { '\u043b' }, new char[] { '\u2936' },
			new char[] { '\u201c' }, new char[] { '\u201e' }, new char[] { '\u2967' }, new char[] { '\u294b' },
			new char[] { '\u21b2' }, new char[] { '\u2264' }, new char[] { '\u2190' }, new char[] { '\u21a2' },
			new char[] { '\u21bd' }, new char[] { '\u21bc' }, new char[] { '\u21c7' }, new char[] { '\u2194' },
			new char[] { '\u21c6' }, new char[] { '\u21cb' }, new char[] { '\u21ad' }, new char[] { '\u22cb' },
			new char[] { '\u22da' }, new char[] { '\u2264' }, new char[] { '\u2266' }, new char[] { '\u2a7d' },
			new char[] { '\u2a7d' }, new char[] { '\u2aa8' }, new char[] { '\u2a7f' }, new char[] { '\u2a81' },
			new char[] { '\u2a83' }, new char[] { '\u22da', '\ufe00' }, new char[] { '\u2a93' }, new char[] { '\u2a85' },
			new char[] { '\u22d6' }, new char[] { '\u22da' }, new char[] { '\u2a8b' }, new char[] { '\u2276' },
			new char[] { '\u2272' }, new char[] { '\u297c' }, new char[] { '\u230a' }, new char[] { '\ud835', '\udd29' },
			new char[] { '\u2276' }, new char[] { '\u2a91' }, new char[] { '\u21bd' }, new char[] { '\u21bc' },
			new char[] { '\u296a' }, new char[] { '\u2584' }, new char[] { '\u0459' }, new char[] { '\u226a' },
			new char[] { '\u21c7' }, new char[] { '\u231e' }, new char[] { '\u296b' }, new char[] { '\u25fa' },
			new char[] { '\u0140' }, new char[] { '\u23b0' }, new char[] { '\u23b0' }, new char[] { '\u2268' },
			new char[] { '\u2a89' }, new char[] { '\u2a89' }, new char[] { '\u2a87' }, new char[] { '\u2a87' },
			new char[] { '\u2268' }, new char[] { '\u22e6' }, new char[] { '\u27ec' }, new char[] { '\u21fd' },
			new char[] { '\u27e6' }, new char[] { '\u27f5' }, new char[] { '\u27f7' }, new char[] { '\u27fc' },
			new char[] { '\u27f6' }, new char[] { '\u21ab' }, new char[] { '\u21ac' }, new char[] { '\u2985' },
			new char[] { '\ud835', '\udd5d' }, new char[] { '\u2a2d' }, new char[] { '\u2a34' }, new char[] { '\u2217' },
			new char[] { '\u005f' }, new char[] { '\u25ca' }, new char[] { '\u25ca' }, new char[] { '\u29eb' },
			new char[] { '\u0028' }, new char[] { '\u2993' }, new char[] { '\u21c6' }, new char[] { '\u231f' },
			new char[] { '\u21cb' }, new char[] { '\u296d' }, new char[] { '\u200e' }, new char[] { '\u22bf' },
			new char[] { '\u2039' }, new char[] { '\ud835', '\udcc1' }, new char[] { '\u21b0' }, new char[] { '\u2272' },
			new char[] { '\u2a8d' }, new char[] { '\u2a8f' }, new char[] { '\u005b' }, new char[] { '\u2018' },
			new char[] { '\u201a' }, new char[] { '\u0142' }, new char[] { '\u003c' }, new char[] { '\u003c' },
			new char[] { '\u2aa6' }, new char[] { '\u2a79' }, new char[] { '\u22d6' }, new char[] { '\u22cb' },
			new char[] { '\u22c9' }, new char[] { '\u2976' }, new char[] { '\u2a7b' }, new char[] { '\u2996' },
			new char[] { '\u25c3' }, new char[] { '\u22b4' }, new char[] { '\u25c2' }, new char[] { '\u294a' },
			new char[] { '\u2966' }, new char[] { '\u2268', '\ufe00' }, new char[] { '\u2268', '\ufe00' },
			new char[] { '\u223a' }, new char[] { '\u00af' }, new char[] { '\u00af' }, new char[] { '\u2642' },
			new char[] { '\u2720' }, new char[] { '\u2720' }, new char[] { '\u21a6' }, new char[] { '\u21a6' },
			new char[] { '\u21a7' }, new char[] { '\u21a4' }, new char[] { '\u21a5' }, new char[] { '\u25ae' },
			new char[] { '\u2a29' }, new char[] { '\u043c' }, new char[] { '\u2014' }, new char[] { '\u2221' },
			new char[] { '\ud835', '\udd2a' }, new char[] { '\u2127' }, new char[] { '\u00b5' }, new char[] { '\u00b5' },
			new char[] { '\u2223' }, new char[] { '\u002a' }, new char[] { '\u2af0' }, new char[] { '\u00b7' },
			new char[] { '\u00b7' }, new char[] { '\u2212' }, new char[] { '\u229f' }, new char[] { '\u2238' },
			new char[] { '\u2a2a' }, new char[] { '\u2adb' }, new char[] { '\u2026' }, new char[] { '\u2213' },
			new char[] { '\u22a7' }, new char[] { '\ud835', '\udd5e' }, new char[] { '\u2213' },
			new char[] { '\ud835', '\udcc2' }, new char[] { '\u223e' }, new char[] { '\u03bc' }, new char[] { '\u22b8' },
			new char[] { '\u22b8' }, new char[] { '\u22d9', '\u0338' }, new char[] { '\u226b', '\u20d2' },
			new char[] { '\u226b', '\u0338' }, new char[] { '\u21cd' }, new char[] { '\u21ce' },
			new char[] { '\u22d8', '\u0338' }, new char[] { '\u226a', '\u20d2' },
			new char[] { '\u226a', '\u0338' }, new char[] { '\u21cf' }, new char[] { '\u22af' }, new char[] { '\u22ae' },
			new char[] { '\u2207' }, new char[] { '\u0144' }, new char[] { '\u2220', '\u20d2' }, new char[] { '\u2249' },
			new char[] { '\u2a70', '\u0338' }, new char[] { '\u224b', '\u0338' }, new char[] { '\u0149' },
			new char[] { '\u2249' }, new char[] { '\u266e' }, new char[] { '\u266e' }, new char[] { '\u2115' },
			new char[] { '\u00a0' }, new char[] { '\u00a0' }, new char[] { '\u224e', '\u0338' },
			new char[] { '\u224f', '\u0338' }, new char[] { '\u2a43' }, new char[] { '\u0148' }, new char[] { '\u0146' },
			new char[] { '\u2247' }, new char[] { '\u2a6d', '\u0338' }, new char[] { '\u2a42' }, new char[] { '\u043d' },
			new char[] { '\u2013' }, new char[] { '\u2260' }, new char[] { '\u21d7' }, new char[] { '\u2924' },
			new char[] { '\u2197' }, new char[] { '\u2197' }, new char[] { '\u2250', '\u0338' }, new char[] { '\u2262' },
			new char[] { '\u2928' }, new char[] { '\u2242', '\u0338' }, new char[] { '\u2204' }, new char[] { '\u2204' },
			new char[] { '\ud835', '\udd2b' }, new char[] { '\u2267', '\u0338' }, new char[] { '\u2271' },
			new char[] { '\u2271' }, new char[] { '\u2267', '\u0338' }, new char[] { '\u2a7e', '\u0338' },
			new char[] { '\u2a7e', '\u0338' }, new char[] { '\u2275' }, new char[] { '\u226f' }, new char[] { '\u226f' },
			new char[] { '\u21ce' }, new char[] { '\u21ae' }, new char[] { '\u2af2' }, new char[] { '\u220b' },
			new char[] { '\u22fc' }, new char[] { '\u22fa' }, new char[] { '\u220b' }, new char[] { '\u045a' },
			new char[] { '\u21cd' }, new char[] { '\u2266', '\u0338' }, new char[] { '\u219a' }, new char[] { '\u2025' },
			new char[] { '\u2270' }, new char[] { '\u219a' }, new char[] { '\u21ae' }, new char[] { '\u2270' },
			new char[] { '\u2266', '\u0338' }, new char[] { '\u2a7d', '\u0338' },
			new char[] { '\u2a7d', '\u0338' }, new char[] { '\u226e' }, new char[] { '\u2274' }, new char[] { '\u226e' },
			new char[] { '\u22ea' }, new char[] { '\u22ec' }, new char[] { '\u2224' }, new char[] { '\ud835', '\udd5f' },
			new char[] { '\u00ac' }, new char[] { '\u00ac' }, new char[] { '\u2209' }, new char[] { '\u22f9', '\u0338' },
			new char[] { '\u22f5', '\u0338' }, new char[] { '\u2209' }, new char[] { '\u22f7' }, new char[] { '\u22f6' },
			new char[] { '\u220c' }, new char[] { '\u220c' }, new char[] { '\u22fe' }, new char[] { '\u22fd' },
			new char[] { '\u2226' }, new char[] { '\u2226' }, new char[] { '\u2afd', '\u20e5' },
			new char[] { '\u2202', '\u0338' }, new char[] { '\u2a14' }, new char[] { '\u2280' }, new char[] { '\u22e0' },
			new char[] { '\u2aaf', '\u0338' }, new char[] { '\u2280' }, new char[] { '\u2aaf', '\u0338' },
			new char[] { '\u21cf' }, new char[] { '\u219b' }, new char[] { '\u2933', '\u0338' },
			new char[] { '\u219d', '\u0338' }, new char[] { '\u219b' }, new char[] { '\u22eb' }, new char[] { '\u22ed' },
			new char[] { '\u2281' }, new char[] { '\u22e1' }, new char[] { '\u2ab0', '\u0338' },
			new char[] { '\ud835', '\udcc3' }, new char[] { '\u2224' }, new char[] { '\u2226' }, new char[] { '\u2241' },
			new char[] { '\u2244' }, new char[] { '\u2244' }, new char[] { '\u2224' }, new char[] { '\u2226' },
			new char[] { '\u22e2' }, new char[] { '\u22e3' }, new char[] { '\u2284' }, new char[] { '\u2ac5', '\u0338' },
			new char[] { '\u2288' }, new char[] { '\u2282', '\u20d2' }, new char[] { '\u2288' },
			new char[] { '\u2ac5', '\u0338' }, new char[] { '\u2281' }, new char[] { '\u2ab0', '\u0338' },
			new char[] { '\u2285' }, new char[] { '\u2ac6', '\u0338' }, new char[] { '\u2289' },
			new char[] { '\u2283', '\u20d2' }, new char[] { '\u2289' }, new char[] { '\u2ac6', '\u0338' },
			new char[] { '\u2279' }, new char[] { '\u00f1' }, new char[] { '\u00f1' }, new char[] { '\u2278' },
			new char[] { '\u22ea' }, new char[] { '\u22ec' }, new char[] { '\u22eb' }, new char[] { '\u22ed' },
			new char[] { '\u03bd' }, new char[] { '\u0023' }, new char[] { '\u2116' }, new char[] { '\u2007' },
			new char[] { '\u22ad' }, new char[] { '\u2904' }, new char[] { '\u224d', '\u20d2' }, new char[] { '\u22ac' },
			new char[] { '\u2265', '\u20d2' }, new char[] { '\u003e', '\u20d2' }, new char[] { '\u29de' },
			new char[] { '\u2902' }, new char[] { '\u2264', '\u20d2' }, new char[] { '\u003c', '\u20d2' },
			new char[] { '\u22b4', '\u20d2' }, new char[] { '\u2903' }, new char[] { '\u22b5', '\u20d2' },
			new char[] { '\u223c', '\u20d2' }, new char[] { '\u21d6' }, new char[] { '\u2923' }, new char[] { '\u2196' },
			new char[] { '\u2196' }, new char[] { '\u2927' }, new char[] { '\u24c8' }, new char[] { '\u00f3' },
			new char[] { '\u00f3' }, new char[] { '\u229b' }, new char[] { '\u229a' }, new char[] { '\u00f4' },
			new char[] { '\u00f4' }, new char[] { '\u043e' }, new char[] { '\u229d' }, new char[] { '\u0151' },
			new char[] { '\u2a38' }, new char[] { '\u2299' }, new char[] { '\u29bc' }, new char[] { '\u0153' },
			new char[] { '\u29bf' }, new char[] { '\ud835', '\udd2c' }, new char[] { '\u02db' }, new char[] { '\u00f2' },
			new char[] { '\u00f2' }, new char[] { '\u29c1' }, new char[] { '\u29b5' }, new char[] { '\u03a9' },
			new char[] { '\u222e' }, new char[] { '\u21ba' }, new char[] { '\u29be' }, new char[] { '\u29bb' },
			new char[] { '\u203e' }, new char[] { '\u29c0' }, new char[] { '\u014d' }, new char[] { '\u03c9' },
			new char[] { '\u03bf' }, new char[] { '\u29b6' }, new char[] { '\u2296' }, new char[] { '\ud835', '\udd60' },
			new char[] { '\u29b7' }, new char[] { '\u29b9' }, new char[] { '\u2295' }, new char[] { '\u2228' },
			new char[] { '\u21bb' }, new char[] { '\u2a5d' }, new char[] { '\u2134' }, new char[] { '\u2134' },
			new char[] { '\u00aa' }, new char[] { '\u00aa' }, new char[] { '\u00ba' }, new char[] { '\u00ba' },
			new char[] { '\u22b6' }, new char[] { '\u2a56' }, new char[] { '\u2a57' }, new char[] { '\u2a5b' },
			new char[] { '\u2134' }, new char[] { '\u00f8' }, new char[] { '\u00f8' }, new char[] { '\u2298' },
			new char[] { '\u00f5' }, new char[] { '\u00f5' }, new char[] { '\u2297' }, new char[] { '\u2a36' },
			new char[] { '\u00f6' }, new char[] { '\u00f6' }, new char[] { '\u233d' }, new char[] { '\u2225' },
			new char[] { '\u00b6' }, new char[] { '\u00b6' }, new char[] { '\u2225' }, new char[] { '\u2af3' },
			new char[] { '\u2afd' }, new char[] { '\u2202' }, new char[] { '\u043f' }, new char[] { '\u0025' },
			new char[] { '\u002e' }, new char[] { '\u2030' }, new char[] { '\u22a5' }, new char[] { '\u2031' },
			new char[] { '\ud835', '\udd2d' }, new char[] { '\u03c6' }, new char[] { '\u03d5' }, new char[] { '\u2133' },
			new char[] { '\u260e' }, new char[] { '\u03c0' }, new char[] { '\u22d4' }, new char[] { '\u03d6' },
			new char[] { '\u210f' }, new char[] { '\u210e' }, new char[] { '\u210f' }, new char[] { '\u002b' },
			new char[] { '\u2a23' }, new char[] { '\u229e' }, new char[] { '\u2a22' }, new char[] { '\u2214' },
			new char[] { '\u2a25' }, new char[] { '\u2a72' }, new char[] { '\u00b1' }, new char[] { '\u00b1' },
			new char[] { '\u2a26' }, new char[] { '\u2a27' }, new char[] { '\u00b1' }, new char[] { '\u2a15' },
			new char[] { '\ud835', '\udd61' }, new char[] { '\u00a3' }, new char[] { '\u00a3' }, new char[] { '\u227a' },
			new char[] { '\u2ab3' }, new char[] { '\u2ab7' }, new char[] { '\u227c' }, new char[] { '\u2aaf' },
			new char[] { '\u227a' }, new char[] { '\u2ab7' }, new char[] { '\u227c' }, new char[] { '\u2aaf' },
			new char[] { '\u2ab9' }, new char[] { '\u2ab5' }, new char[] { '\u22e8' }, new char[] { '\u227e' },
			new char[] { '\u2032' }, new char[] { '\u2119' }, new char[] { '\u2ab5' }, new char[] { '\u2ab9' },
			new char[] { '\u22e8' }, new char[] { '\u220f' }, new char[] { '\u232e' }, new char[] { '\u2312' },
			new char[] { '\u2313' }, new char[] { '\u221d' }, new char[] { '\u221d' }, new char[] { '\u227e' },
			new char[] { '\u22b0' }, new char[] { '\ud835', '\udcc5' }, new char[] { '\u03c8' }, new char[] { '\u2008' },
			new char[] { '\ud835', '\udd2e' }, new char[] { '\u2a0c' }, new char[] { '\ud835', '\udd62' },
			new char[] { '\u2057' }, new char[] { '\ud835', '\udcc6' }, new char[] { '\u210d' }, new char[] { '\u2a16' },
			new char[] { '\u003f' }, new char[] { '\u225f' }, new char[] { '\u0022' }, new char[] { '\u0022' },
			new char[] { '\u21db' }, new char[] { '\u21d2' }, new char[] { '\u291c' }, new char[] { '\u290f' },
			new char[] { '\u2964' }, new char[] { '\u223d', '\u0331' }, new char[] { '\u0155' }, new char[] { '\u221a' },
			new char[] { '\u29b3' }, new char[] { '\u27e9' }, new char[] { '\u2992' }, new char[] { '\u29a5' },
			new char[] { '\u27e9' }, new char[] { '\u00bb' }, new char[] { '\u00bb' }, new char[] { '\u2192' },
			new char[] { '\u2975' }, new char[] { '\u21e5' }, new char[] { '\u2920' }, new char[] { '\u2933' },
			new char[] { '\u291e' }, new char[] { '\u21aa' }, new char[] { '\u21ac' }, new char[] { '\u2945' },
			new char[] { '\u2974' }, new char[] { '\u21a3' }, new char[] { '\u219d' }, new char[] { '\u291a' },
			new char[] { '\u2236' }, new char[] { '\u211a' }, new char[] { '\u290d' }, new char[] { '\u2773' },
			new char[] { '\u007d' }, new char[] { '\u005d' }, new char[] { '\u298c' }, new char[] { '\u298e' },
			new char[] { '\u2990' }, new char[] { '\u0159' }, new char[] { '\u0157' }, new char[] { '\u2309' },
			new char[] { '\u007d' }, new char[] { '\u0440' }, new char[] { '\u2937' }, new char[] { '\u2969' },
			new char[] { '\u201d' }, new char[] { '\u201d' }, new char[] { '\u21b3' }, new char[] { '\u211c' },
			new char[] { '\u211b' }, new char[] { '\u211c' }, new char[] { '\u211d' }, new char[] { '\u25ad' },
			new char[] { '\u00ae' }, new char[] { '\u00ae' }, new char[] { '\u297d' }, new char[] { '\u230b' },
			new char[] { '\ud835', '\udd2f' }, new char[] { '\u21c1' }, new char[] { '\u21c0' }, new char[] { '\u296c' },
			new char[] { '\u03c1' }, new char[] { '\u03f1' }, new char[] { '\u2192' }, new char[] { '\u21a3' },
			new char[] { '\u21c1' }, new char[] { '\u21c0' }, new char[] { '\u21c4' }, new char[] { '\u21cc' },
			new char[] { '\u21c9' }, new char[] { '\u219d' }, new char[] { '\u22cc' }, new char[] { '\u02da' },
			new char[] { '\u2253' }, new char[] { '\u21c4' }, new char[] { '\u21cc' }, new char[] { '\u200f' },
			new char[] { '\u23b1' }, new char[] { '\u23b1' }, new char[] { '\u2aee' }, new char[] { '\u27ed' },
			new char[] { '\u21fe' }, new char[] { '\u27e7' }, new char[] { '\u2986' }, new char[] { '\ud835', '\udd63' },
			new char[] { '\u2a2e' }, new char[] { '\u2a35' }, new char[] { '\u0029' }, new char[] { '\u2994' },
			new char[] { '\u2a12' }, new char[] { '\u21c9' }, new char[] { '\u203a' }, new char[] { '\ud835', '\udcc7' },
			new char[] { '\u21b1' }, new char[] { '\u005d' }, new char[] { '\u2019' }, new char[] { '\u2019' },
			new char[] { '\u22cc' }, new char[] { '\u22ca' }, new char[] { '\u25b9' }, new char[] { '\u22b5' },
			new char[] { '\u25b8' }, new char[] { '\u29ce' }, new char[] { '\u2968' }, new char[] { '\u211e' },
			new char[] { '\u015b' }, new char[] { '\u201a' }, new char[] { '\u227b' }, new char[] { '\u2ab4' },
			new char[] { '\u2ab8' }, new char[] { '\u0161' }, new char[] { '\u227d' }, new char[] { '\u2ab0' },
			new char[] { '\u015f' }, new char[] { '\u015d' }, new char[] { '\u2ab6' }, new char[] { '\u2aba' },
			new char[] { '\u22e9' }, new char[] { '\u2a13' }, new char[] { '\u227f' }, new char[] { '\u0441' },
			new char[] { '\u22c5' }, new char[] { '\u22a1' }, new char[] { '\u2a66' }, new char[] { '\u21d8' },
			new char[] { '\u2925' }, new char[] { '\u2198' }, new char[] { '\u2198' }, new char[] { '\u00a7' },
			new char[] { '\u00a7' }, new char[] { '\u003b' }, new char[] { '\u2929' }, new char[] { '\u2216' },
			new char[] { '\u2216' }, new char[] { '\u2736' }, new char[] { '\ud835', '\udd30' }, new char[] { '\u2322' },
			new char[] { '\u266f' }, new char[] { '\u0449' }, new char[] { '\u0448' }, new char[] { '\u2223' },
			new char[] { '\u2225' }, new char[] { '\u00ad' }, new char[] { '\u00ad' }, new char[] { '\u03c3' },
			new char[] { '\u03c2' }, new char[] { '\u03c2' }, new char[] { '\u223c' }, new char[] { '\u2a6a' },
			new char[] { '\u2243' }, new char[] { '\u2243' }, new char[] { '\u2a9e' }, new char[] { '\u2aa0' },
			new char[] { '\u2a9d' }, new char[] { '\u2a9f' }, new char[] { '\u2246' }, new char[] { '\u2a24' },
			new char[] { '\u2972' }, new char[] { '\u2190' }, new char[] { '\u2216' }, new char[] { '\u2a33' },
			new char[] { '\u29e4' }, new char[] { '\u2223' }, new char[] { '\u2323' }, new char[] { '\u2aaa' },
			new char[] { '\u2aac' }, new char[] { '\u2aac', '\ufe00' }, new char[] { '\u044c' }, new char[] { '\u002f' },
			new char[] { '\u29c4' }, new char[] { '\u233f' }, new char[] { '\ud835', '\udd64' }, new char[] { '\u2660' },
			new char[] { '\u2660' }, new char[] { '\u2225' }, new char[] { '\u2293' }, new char[] { '\u2293', '\ufe00' },
			new char[] { '\u2294' }, new char[] { '\u2294', '\ufe00' }, new char[] { '\u228f' }, new char[] { '\u2291' },
			new char[] { '\u228f' }, new char[] { '\u2291' }, new char[] { '\u2290' }, new char[] { '\u2292' },
			new char[] { '\u2290' }, new char[] { '\u2292' }, new char[] { '\u25a1' }, new char[] { '\u25a1' },
			new char[] { '\u25aa' }, new char[] { '\u25aa' }, new char[] { '\u2192' }, new char[] { '\ud835', '\udcc8' },
			new char[] { '\u2216' }, new char[] { '\u2323' }, new char[] { '\u22c6' }, new char[] { '\u2606' },
			new char[] { '\u2605' }, new char[] { '\u03f5' }, new char[] { '\u03d5' }, new char[] { '\u00af' },
			new char[] { '\u2282' }, new char[] { '\u2ac5' }, new char[] { '\u2abd' }, new char[] { '\u2286' },
			new char[] { '\u2ac3' }, new char[] { '\u2ac1' }, new char[] { '\u2acb' }, new char[] { '\u228a' },
			new char[] { '\u2abf' }, new char[] { '\u2979' }, new char[] { '\u2282' }, new char[] { '\u2286' },
			new char[] { '\u2ac5' }, new char[] { '\u228a' }, new char[] { '\u2acb' }, new char[] { '\u2ac7' },
			new char[] { '\u2ad5' }, new char[] { '\u2ad3' }, new char[] { '\u227b' }, new char[] { '\u2ab8' },
			new char[] { '\u227d' }, new char[] { '\u2ab0' }, new char[] { '\u2aba' }, new char[] { '\u2ab6' },
			new char[] { '\u22e9' }, new char[] { '\u227f' }, new char[] { '\u2211' }, new char[] { '\u266a' },
			new char[] { '\u00b9' }, new char[] { '\u00b9' }, new char[] { '\u00b2' }, new char[] { '\u00b2' },
			new char[] { '\u00b3' }, new char[] { '\u00b3' }, new char[] { '\u2283' }, new char[] { '\u2ac6' },
			new char[] { '\u2abe' }, new char[] { '\u2ad8' }, new char[] { '\u2287' }, new char[] { '\u2ac4' },
			new char[] { '\u27c9' }, new char[] { '\u2ad7' }, new char[] { '\u297b' }, new char[] { '\u2ac2' },
			new char[] { '\u2acc' }, new char[] { '\u228b' }, new char[] { '\u2ac0' }, new char[] { '\u2283' },
			new char[] { '\u2287' }, new char[] { '\u2ac6' }, new char[] { '\u228b' }, new char[] { '\u2acc' },
			new char[] { '\u2ac8' }, new char[] { '\u2ad4' }, new char[] { '\u2ad6' }, new char[] { '\u21d9' },
			new char[] { '\u2926' }, new char[] { '\u2199' }, new char[] { '\u2199' }, new char[] { '\u292a' },
			new char[] { '\u00df' }, new char[] { '\u00df' }, new char[] { '\u2316' }, new char[] { '\u03c4' },
			new char[] { '\u23b4' }, new char[] { '\u0165' }, new char[] { '\u0163' }, new char[] { '\u0442' },
			new char[] { '\u20db' }, new char[] { '\u2315' }, new char[] { '\ud835', '\udd31' }, new char[] { '\u2234' },
			new char[] { '\u2234' }, new char[] { '\u03b8' }, new char[] { '\u03d1' }, new char[] { '\u03d1' },
			new char[] { '\u2248' }, new char[] { '\u223c' }, new char[] { '\u2009' }, new char[] { '\u2248' },
			new char[] { '\u223c' }, new char[] { '\u00fe' }, new char[] { '\u00fe' }, new char[] { '\u02dc' },
			new char[] { '\u00d7' }, new char[] { '\u00d7' }, new char[] { '\u22a0' }, new char[] { '\u2a31' },
			new char[] { '\u2a30' }, new char[] { '\u222d' }, new char[] { '\u2928' }, new char[] { '\u22a4' },
			new char[] { '\u2336' }, new char[] { '\u2af1' }, new char[] { '\ud835', '\udd65' }, new char[] { '\u2ada' },
			new char[] { '\u2929' }, new char[] { '\u2034' }, new char[] { '\u2122' }, new char[] { '\u25b5' },
			new char[] { '\u25bf' }, new char[] { '\u25c3' }, new char[] { '\u22b4' }, new char[] { '\u225c' },
			new char[] { '\u25b9' }, new char[] { '\u22b5' }, new char[] { '\u25ec' }, new char[] { '\u225c' },
			new char[] { '\u2a3a' }, new char[] { '\u2a39' }, new char[] { '\u29cd' }, new char[] { '\u2a3b' },
			new char[] { '\u23e2' }, new char[] { '\ud835', '\udcc9' }, new char[] { '\u0446' }, new char[] { '\u045b' },
			new char[] { '\u0167' }, new char[] { '\u226c' }, new char[] { '\u219e' }, new char[] { '\u21a0' },
			new char[] { '\u21d1' }, new char[] { '\u2963' }, new char[] { '\u00fa' }, new char[] { '\u00fa' },
			new char[] { '\u2191' }, new char[] { '\u045e' }, new char[] { '\u016d' }, new char[] { '\u00fb' },
			new char[] { '\u00fb' }, new char[] { '\u0443' }, new char[] { '\u21c5' }, new char[] { '\u0171' },
			new char[] { '\u296e' }, new char[] { '\u297e' }, new char[] { '\ud835', '\udd32' }, new char[] { '\u00f9' },
			new char[] { '\u00f9' }, new char[] { '\u21bf' }, new char[] { '\u21be' }, new char[] { '\u2580' },
			new char[] { '\u231c' }, new char[] { '\u231c' }, new char[] { '\u230f' }, new char[] { '\u25f8' },
			new char[] { '\u016b' }, new char[] { '\u00a8' }, new char[] { '\u00a8' }, new char[] { '\u0173' },
			new char[] { '\ud835', '\udd66' }, new char[] { '\u2191' }, new char[] { '\u2195' }, new char[] { '\u21bf' },
			new char[] { '\u21be' }, new char[] { '\u228e' }, new char[] { '\u03c5' }, new char[] { '\u03d2' },
			new char[] { '\u03c5' }, new char[] { '\u21c8' }, new char[] { '\u231d' }, new char[] { '\u231d' },
			new char[] { '\u230e' }, new char[] { '\u016f' }, new char[] { '\u25f9' }, new char[] { '\ud835', '\udcca' },
			new char[] { '\u22f0' }, new char[] { '\u0169' }, new char[] { '\u25b5' }, new char[] { '\u25b4' },
			new char[] { '\u21c8' }, new char[] { '\u00fc' }, new char[] { '\u00fc' }, new char[] { '\u29a7' },
			new char[] { '\u21d5' }, new char[] { '\u2ae8' }, new char[] { '\u2ae9' }, new char[] { '\u22a8' },
			new char[] { '\u299c' }, new char[] { '\u03f5' }, new char[] { '\u03f0' }, new char[] { '\u2205' },
			new char[] { '\u03d5' }, new char[] { '\u03d6' }, new char[] { '\u221d' }, new char[] { '\u2195' },
			new char[] { '\u03f1' }, new char[] { '\u03c2' }, new char[] { '\u228a', '\ufe00' },
			new char[] { '\u2acb', '\ufe00' }, new char[] { '\u228b', '\ufe00' },
			new char[] { '\u2acc', '\ufe00' }, new char[] { '\u03d1' }, new char[] { '\u22b2' }, new char[] { '\u22b3' },
			new char[] { '\u0432' }, new char[] { '\u22a2' }, new char[] { '\u2228' }, new char[] { '\u22bb' },
			new char[] { '\u225a' }, new char[] { '\u22ee' }, new char[] { '\u007c' }, new char[] { '\u007c' },
			new char[] { '\ud835', '\udd33' }, new char[] { '\u22b2' }, new char[] { '\u2282', '\u20d2' },
			new char[] { '\u2283', '\u20d2' }, new char[] { '\ud835', '\udd67' }, new char[] { '\u221d' },
			new char[] { '\u22b3' }, new char[] { '\ud835', '\udccb' }, new char[] { '\u2acb', '\ufe00' },
			new char[] { '\u228a', '\ufe00' }, new char[] { '\u2acc', '\ufe00' },
			new char[] { '\u228b', '\ufe00' }, new char[] { '\u299a' }, new char[] { '\u0175' }, new char[] { '\u2a5f' },
			new char[] { '\u2227' }, new char[] { '\u2259' }, new char[] { '\u2118' }, new char[] { '\ud835', '\udd34' },
			new char[] { '\ud835', '\udd68' }, new char[] { '\u2118' }, new char[] { '\u2240' }, new char[] { '\u2240' },
			new char[] { '\ud835', '\udccc' }, new char[] { '\u22c2' }, new char[] { '\u25ef' }, new char[] { '\u22c3' },
			new char[] { '\u25bd' }, new char[] { '\ud835', '\udd35' }, new char[] { '\u27fa' }, new char[] { '\u27f7' },
			new char[] { '\u03be' }, new char[] { '\u27f8' }, new char[] { '\u27f5' }, new char[] { '\u27fc' },
			new char[] { '\u22fb' }, new char[] { '\u2a00' }, new char[] { '\ud835', '\udd69' }, new char[] { '\u2a01' },
			new char[] { '\u2a02' }, new char[] { '\u27f9' }, new char[] { '\u27f6' }, new char[] { '\ud835', '\udccd' },
			new char[] { '\u2a06' }, new char[] { '\u2a04' }, new char[] { '\u25b3' }, new char[] { '\u22c1' },
			new char[] { '\u22c0' }, new char[] { '\u00fd' }, new char[] { '\u00fd' }, new char[] { '\u044f' },
			new char[] { '\u0177' }, new char[] { '\u044b' }, new char[] { '\u00a5' }, new char[] { '\u00a5' },
			new char[] { '\ud835', '\udd36' }, new char[] { '\u0457' }, new char[] { '\ud835', '\udd6a' },
			new char[] { '\ud835', '\udcce' }, new char[] { '\u044e' }, new char[] { '\u00ff' }, new char[] { '\u00ff' },
			new char[] { '\u017a' }, new char[] { '\u017e' }, new char[] { '\u0437' }, new char[] { '\u017c' },
			new char[] { '\u2128' }, new char[] { '\u03b6' }, new char[] { '\ud835', '\udd37' }, new char[] { '\u0436' },
			new char[] { '\u21dd' }, new char[] { '\ud835', '\udd6b' }, new char[] { '\ud835', '\udccf' },
			new char[] { '\u200d' }, new char[] { '\u200c' }, };

	internal static readonly char[][] WINDOWS_1252 = { new char[] { '\u20AC' }, new char[] { '\u0081' },
			new char[] { '\u201A' }, new char[] { '\u0192' }, new char[] { '\u201E' }, new char[] { '\u2026' },
			new char[] { '\u2020' }, new char[] { '\u2021' }, new char[] { '\u02C6' }, new char[] { '\u2030' },
			new char[] { '\u0160' }, new char[] { '\u2039' }, new char[] { '\u0152' }, new char[] { '\u008D' },
			new char[] { '\u017D' }, new char[] { '\u008F' }, new char[] { '\u0090' }, new char[] { '\u2018' },
			new char[] { '\u2019' }, new char[] { '\u201C' }, new char[] { '\u201D' }, new char[] { '\u2022' },
			new char[] { '\u2013' }, new char[] { '\u2014' }, new char[] { '\u02DC' }, new char[] { '\u2122' },
			new char[] { '\u0161' }, new char[] { '\u203A' }, new char[] { '\u0153' }, new char[] { '\u009D' },
			new char[] { '\u017E' }, new char[] { '\u0178' } };

}

}
