// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Net;
using System.Text;
using HttpWebAdapters.Adapters;

namespace HttpWebAdapters {
    /// <summary>
    /// Creates a web request that does basic auth
    /// </summary>
    internal class BasicAuthHttpWebRequestFactory : IHttpWebRequestFactory
    {
        private readonly string username;
        private readonly string password;

        /// <summary>
        /// Creates a web request that does basic auth.
        /// </summary>
        ///
        /// <param name="username">
        /// HTTP username.
        /// </param>
        /// <param name="password">
        /// HTTP password.
        /// </param>

        public BasicAuthHttpWebRequestFactory(string username, string password) {
            this.username = username;
            this.password = password;
        }

        /// <summary>
        /// Creates a web request for a url
        /// </summary>
        ///
        /// <param name="url">
        /// URL of the document.
        /// </param>
        ///
        /// <returns>
        /// A new IHttpWebRequest
        /// </returns>

        public IHttpWebRequest Create(string url) {
            return Create(new Uri(url));
        }

        /// <summary>
        /// Creates a concrete HttpWebRequest for a URI
        /// </summary>
        ///
        /// <param name="url">
        /// URI of the document.
        /// </param>
        ///
        /// <returns>
        /// A new IHttpWebRequest.
        /// </returns>

        public IHttpWebRequest Create(Uri url) {
            var req = (HttpWebRequest) WebRequest.Create(url);
            var credentials = Convert.ToBase64String(Encoding.ASCII.GetBytes(username + ":" + password));
            req.Headers.Add("Authorization", "Basic " + credentials);
            return new HttpWebRequestAdapter(req);
        }
    }
}