﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

namespace HttpWebAdapters {

    /// <summary>
    /// An HTTP web request method (GET or POST).
    /// </summary>

    internal class HttpWebRequestMethod
    {
		private string method;
		private static readonly string SGET = "GET";
		private static readonly string SPOST = "POST";

        /// <summary>
        /// The GET method
        /// </summary>

		public static readonly HttpWebRequestMethod GET = new HttpWebRequestMethod(SGET);

        /// <summary>
        /// The POST method
        /// </summary>

		public static readonly HttpWebRequestMethod POST = new HttpWebRequestMethod(SPOST);

		private HttpWebRequestMethod(string m) {
			method = m;
		}

        /// <summary>
        /// Returns a string that represents the current object.
        /// </summary>
        ///
        /// <returns>
        /// A string representation of the web method.
        /// </returns>

		public override string ToString() {
			return method;
		}

        /// <summary>
        /// Creates a new HttpWebRequestMethod from a string
        /// </summary>
        ///
        /// <param name="s">
        /// The string.
        /// </param>
        ///
        /// <returns>
        /// A new HttpWebRequestMethod
        /// </returns>

		public static HttpWebRequestMethod Parse(string s) {
			if (s == SGET)
				return GET;
			return POST;
		}
	}
}