﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.Implementation
{
    /// <summary>
    /// A comparer to ensure that items are returned from a selection set in DOM order, e.g. by comparing their
    /// internal paths.
    /// </summary>
    internal class SelectionSetComparer : IComparer<IDomObject>
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        ///
        /// <exception cref="InvalidOperationException">
        /// Thrown when the requested operation is invalid.
        /// </exception>
        ///
        /// <param name="order">
        /// The order used to compare two items. This must be Ascending or Descending
        /// </param>

        public SelectionSetComparer(SelectionSetOrder order)
        {
            if (order != SelectionSetOrder.Ascending && order != SelectionSetOrder.Descending)
            {
                throw new InvalidOperationException("This comparer can only be used to sort.");
            }
            Order = order;
        }

        private SelectionSetOrder Order;

        /// <summary>
        /// Compares two IDomObject objects to determine their relative ordering.
        /// </summary>
        ///
        /// <param name="x">
        /// I dom object to be compared.
        /// </param>
        /// <param name="y">
        /// I dom object to be compared.
        /// </param>
        ///
        /// <returns>
        /// Negative if 'x' is less than 'y', 0 if they are equal, or positive if it is greater.
        /// </returns>

        public int Compare(IDomObject x, IDomObject y)
        {
            return Order == SelectionSetOrder.Ascending ?
                PathKeyComparer.Comparer.Compare(x.NodePath, y.NodePath) :
                PathKeyComparer.Comparer.Compare(y.NodePath, x.NodePath);
        }
    }
}
