﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Output;

namespace CsQuery
{
    /// <summary>
    /// Factory for HTML encoders included with CsQuery
    /// </summary>

    internal static class HtmlEncoders
    {
        /// <summary>
        /// The default HTML encoder
        /// </summary>

        public static IHtmlEncoder Default
        {
            get
            {
                return Config.HtmlEncoder;
            }
        }

        /// <summary>
        /// The standard HTML encoder; encodes most entities, and any characters that are above ascii 160.
        /// </summary>

        public static IHtmlEncoder Basic = new HtmlEncoderBasic();


        /// <summary>
        /// The minimum HTML encoder; only encodes left-caret, right-caret, and ampersand. All other
        /// characters are passed through.
        /// </summary>

        public static IHtmlEncoder Minimum = new HtmlEncoderMinimum();

        /// <summary>
        /// The same as the minimum HTML encoder, but also encodes nonbreaking space (ascii 160 becomes
        /// &amp;nbsp;).
        /// </summary>

        public static IHtmlEncoder MinimumNbsp = new HtmlEncoderMinimumNbsp();

        /// <summary>
        /// No HTML encoding -- all characters are passed through. Will likely produce invalid HTML.
        /// </summary>

        public static IHtmlEncoder None = new HtmlEncoderNone();

        /// <summary>
        /// Full HTML encoding -- all entities mapped to their named (not numeric) entities when
        /// available.
        /// </summary>

        public static IHtmlEncoder Full = new HtmlEncoderFull();
    }
}
