﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.Promises
{
    /// <summary>
    /// A strongly-typed deferred object
    /// </summary>
    ///
    /// <typeparam name="T">
    /// Generic type parameter.
    /// </typeparam>

    internal class Deferred<T> : Deferred, IPromise<T>
    {
        /// <summary>
        /// Bind delegates to the success or failure of a promise
        /// </summary>
        ///
        /// <param name="success">
        /// The success delegate
        /// </param>
        /// <param name="failure">
        /// (optional) the failure delegate
        /// </param>
        ///
        /// <returns>
        /// A new promise that resolves when the current promise resolves.
        /// </returns>

        public IPromise Then(PromiseAction<T> success, PromiseAction<T> failure = null)
        {
            return base.Then(success, failure);
        }

        /// <summary>
        /// Bind delegates to the success or failure of a promise
        /// </summary>
        ///
        /// <param name="success">
        /// The success delegate
        /// </param>
        /// <param name="failure">
        /// (optional) the failure delegate
        /// </param>
        ///
        /// <returns>
        /// A new promise that resolves when the current promise resolves.
        /// </returns>

        public IPromise Then(PromiseFunction<T> success, PromiseFunction<T> failure = null)
        {
            return base.Then(success, failure);
        }

     
    }
   
}
