﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Timers;

namespace CsQuery.Promises
{
    /// <summary>
    /// A promise that resolves or fails after a certain amount of time
    /// </summary>

    internal class Timeout : Timeout<object>, IPromise
    {
        /// <summary>
        /// Create a new Timeout that rejects after the specified time.
        /// </summary>
        ///
        /// <param name="timeoutMilliseconds">
        /// The timeout in milliseconds.
        /// </param>

        public Timeout(int timeoutMilliseconds): base(timeoutMilliseconds)
        {

        }

        /// <summary>
        /// Create a new Timeout that rejects with the provided parameter value after the specified time.
        /// </summary>
        ///
        /// <param name="timeoutMilliseconds">
        /// The timeout in milliseconds.
        /// </param>
        /// <param name="parameterValue">
        /// The parameter value.
        /// </param>

        public Timeout(int timeoutMilliseconds, object parameterValue):
            base(timeoutMilliseconds,parameterValue)
        {
        }

        /// <summary>
        /// Create a new Timeout that resolves or rejects with the provided parameter value after the specified time.
        /// </summary>
        ///
        /// <param name="timeoutMilliseconds">
        /// The timeout in milliseconds.
        /// </param>
        /// <param name="succeedOnTimeout">
        /// true to resolve the promise on the timeout, false to reject it.
        /// </param>

        public Timeout(int timeoutMilliseconds, bool succeedOnTimeout):
            base(timeoutMilliseconds,succeedOnTimeout)
        {

        }

        /// <summary>
        /// Create a new Timeout that resolves or rejects with the provided parameter value after the specified time.
        /// </summary>
        ///
        /// <param name="timeoutMilliseconds">
        /// The timeout in milliseconds.
        /// </param>
        /// <param name="parameterValue">
        /// The parameter value.
        /// </param>
        /// <param name="succeedOnTimeout">
        /// true to resolve the promise on the timeout, false to reject it.
        /// </param>

        public Timeout(int timeoutMilliseconds, object parameterValue, bool succeedOnTimeout) :
            base(timeoutMilliseconds,parameterValue,succeedOnTimeout)
        {
        }
    }
}
