﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.StringScanner
{
    /// <summary>
    /// An interface that describes characterstics of a single character.
    /// </summary>

    internal interface ICharacterInfo : IValueInfo<char>
    {
        /// <summary>
        /// The character is an opening or closing parenthesis.
        /// </summary>

        bool Parenthesis { get; }

        /// <summary>
        /// The character is an enclosing type such as a parenthesis or curly brace (anything which has a
        /// matching close that's not the same as the opening; this specifically excludes single and
        /// double-quote characters).
        /// </summary>

        bool Enclosing { get; }

        /// <summary>
        /// Gets a value indicating whether the character is any bounding type (includes all Enclosing types, plus quotes).
        /// </summary>

        bool Bound { get; }

        /// <summary>
        /// Gets a value indicating whether the character is a quote.
        /// </summary>

        bool Quote { get; }

        /// <summary>
        /// Gets a value indicating whether the character is a separator (a space, or pipe)
        /// </summary>

        bool Separator { get; }
    }
}
