﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CsQuery.StringScanner
{
    /// <summary>
    /// Interface for characterstics of a value, either a single character or a string.
    /// </summary>

    internal interface IValueInfo
    {
        /// <summary>
        /// The value is alphabetic
        /// </summary>

        bool Alpha { get; }

        /// <summary>
        /// The value is numeric.
        /// </summary>

        bool Numeric { get; }

        /// <summary>
        /// The value is numeric, or characters that can be parts of numbers (+,-,.)
        /// </summary>

        bool NumericExtended { get; }

        /// <summary>
        ///The value is all lowercase
        /// </summary>

        bool Lower { get; }

        /// <summary>
        /// Gets a value indicating whether the cvale upper.
        /// </summary>

        bool Upper { get; }

        /// <summary>
        /// The value is whitespace.
        /// </summary>

        bool Whitespace { get; }

        /// <summary>
        /// The value is alphanumeric.
        /// </summary>

        bool Alphanumeric { get; }

        /// <summary>
        /// The value is a math operator
        /// </summary>

        bool Operator { get; }

        /// <summary>
        /// Indicates that a character is alphabetic-like character defined as a-z, A-Z, hyphen,
        /// underscore, and ISO 10646 code U+00A1 and higher. (per characters allowed in CSS identifiers)
        /// </summary>

        bool AlphaISO10646 { get; }

        /// <summary>
        /// The bound character or string for this instance. This is the character against which all
        /// tests are performed.
        /// </summary>

        IConvertible Target { get; set; }
    }



}
