﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.StringScanner.Implementation;

namespace CsQuery.StringScanner.Patterns
{
    /// <summary>
    /// ID can contain any character other than a space; however, a selector is bounded by other terminators.
    /// </summary>
    internal class HtmlIDSelector : EscapedString
    {
        /// <summary>
        /// Default constructor.
        /// </summary>

        public HtmlIDSelector() : 
            base(IsValidIDSelector)
        {

        }

        /// <summary>
        /// Match a pattern for a valid HTML ID.
        /// </summary>
        ///
        /// <param name="index">
        /// The index to match
        /// </param>
        /// <param name="character">
        /// The character to match
        /// </param>
        ///
        /// <returns>
        /// true if valid identifier, false if not.
        /// </returns>

        private static bool IsValidIDSelector(int index, char character)
        {

            return !CharacterData.IsType(character, CharacterType.SelectorTerminator) &&
                !CharacterData.IsType(character, CharacterType.HtmlSpace);
        }
    }
}
