﻿// Copyright (c) 2012 James Treworgy
// 
// LICENSE (MIT License)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
// of the Software, and to permit persons to whom the Software is furnished to do 
// so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
// PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CsQuery.Implementation;
using CsQuery.Engine;

namespace CsQuery.Utility
{
    /// <summary>
    /// Class to cache selectors on a DOM. NOT YET IMPLEMENTED.
    /// </summary>

    internal class SelectorCache
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        ///
        /// <param name="cqSource">
        /// The cq source.
        /// </param>

        public SelectorCache(CQ cqSource)
        {
            CqSource = cqSource;
        }


        private CQ  CqSource;

        private IDictionary<Selector, IList<IDomObject>> _SelectionCache;

        /// <summary>
        /// Gets the selection cache.
        /// </summary>

        protected IDictionary<Selector, IList<IDomObject>> SelectionCache
        {
            get
            {
                if (_SelectionCache == null)
                {
                    _SelectionCache = new Dictionary<Selector, IList<IDomObject>>();
                }
                return _SelectionCache;

            }
        }

        /// <summary>
        /// Run the selector.
        /// </summary>
        ///
        /// <param name="selector">
        /// The selector.
        /// </param>
        ///
        /// <returns>
        /// .
        /// </returns>

        public CQ Select(string selector)
        {
            IList<IDomObject> selection;

            var sel = new Selector(selector);
            if (SelectionCache.TryGetValue(sel, out selection)) {
                return new CQ(selection);
            } else {
                var result = CqSource.Select(sel);
                SelectionCache.Add(sel, result.Selection.ToList());
                return result;
            }   


        }
    }
}
