﻿using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Import.Parser
{
    internal static class HtmlUnitParser
    {
        private static readonly Regex invalidValueRegex = new Regex("\\s+", RegexOptions.Compiled);
        private static readonly Regex unitRegex = new Regex("[a-z%]{1,2}", RegexOptions.Compiled);
        private static readonly Regex valueRegex = new Regex("^[0-9]*$", RegexOptions.Compiled);

        internal static bool Parse(string value, out string parsedValue, out UnitType? unitType)
        {
            parsedValue = string.Empty;
            unitType = null;

            bool isInvalidValue = invalidValueRegex.IsMatch(value);
            if (isInvalidValue)
            {
                return false;
            }

            Match match = unitRegex.Match(value);
            if (match.Length > 0)
            {
                parsedValue = value.Substring(0, value.Length - match.Length);
            }

            switch (match.Value)
            {
                case "px":
                    unitType = UnitType.Dip;
                    break;

                case "pt":
                    unitType = UnitType.Point;
                    break;

                case "em":
                    unitType = UnitType.Em;
                    break;

                case "%":
                    unitType = UnitType.Percent;
                    break;

                case "cm":
                    unitType = UnitType.Cm;
                    break;

                case "mm":
                    unitType = UnitType.Mm;
                    break;

                case "in":
                    unitType = UnitType.Inch;
                    break;
            }

            if (unitType == null)
            {
                if (valueRegex.IsMatch(value))
                {
                    parsedValue = value;
                }
            }

            if (unitType == null || string.IsNullOrEmpty(parsedValue))
            {
                return false;
            }

            return true;
        }
    }
}
