﻿// Copyright (c) 2013 Milkshake Software (http://milkshakesoftware.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

namespace PreMailer.Net
{
    internal class CssSpecificity
	{
		public int Ids { get; protected set; }

		/// <summary>
		/// Classes, attributes and pseudo-classes.
		/// </summary>
		public int ClassesAttributesPseudoElements { get; protected set; }

		/// <summary>
		/// Elements and pseudo-elements.
		/// </summary>
		public int Elements { get; protected set; }

		public static CssSpecificity None
		{
			get
			{
				return new CssSpecificity(0, 0, 0);
			}
		}

        internal int Classes { get; set; }

        public CssSpecificity(int ids, int classesAttributesPseudoElements, int elements)
		{
			Ids = ids;
            ClassesAttributesPseudoElements = classesAttributesPseudoElements;
			Elements = elements;
		}

		public int ToInt()
		{
			var s = ToString();
			var result = 0;
			int.TryParse(s, out result);
			return result;
		}

		public override string ToString()
		{
			return string.Format("{0}{1}{2}", Ids, ClassesAttributesPseudoElements, Elements);
		}

        public static CssSpecificity operator +(CssSpecificity first, CssSpecificity second)
        {
            var cssSpecificity = new CssSpecificity(
                first.Ids + second.Ids,
                first.ClassesAttributesPseudoElements + second.ClassesAttributesPseudoElements,
                first.Elements + second.Elements);

            cssSpecificity.Classes = first.Classes + second.Classes;

            return cssSpecificity;
        }
	}
}