﻿// Copyright (c) 2013 Milkshake Software (http://milkshakesoftware.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System.Collections.Generic;
using System.Text;

namespace PreMailer.Net {
    internal class StyleClass
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="StyleClass"/> class.
        /// </summary>
        public StyleClass() {
            Attributes = new SortedList<string, string>();
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the attributes.
        /// </summary>
        /// <value>The attributes.</value>
        public SortedList<string, string> Attributes { get; set; }

        /// <summary>
        /// Merges the specified style class, with this instance. Styles on this instance are not overriden by duplicates in the specified styleClass.
        /// </summary>
        /// <param name="styleClass">The style class.</param>
        /// <param name="canOverwrite">if set to <c>true</c> [can overwrite].</param>
        public void Merge(StyleClass styleClass, bool canOverwrite) {
            foreach (var item in styleClass.Attributes) {
                if (!Attributes.ContainsKey(item.Key))
                    Attributes.Add(item.Key, item.Value);
                else if (canOverwrite)
                    Attributes[item.Key] = item.Value;
            }
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString() {
            var sb = new StringBuilder();

            foreach (var item in this.Attributes)
                sb.AppendFormat("{0}: {1};", item.Key, item.Value);

            return sb.ToString();
        }
    }
}