using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html
{
    /// <summary>
    /// Represents load from URI event arguments.
    /// </summary>
    public class LoadFromUriEventArgs : EventArgs
    {
        private readonly string uri;
        private byte[] data;

        internal LoadFromUriEventArgs(string uri)
        {
            Guard.ThrowExceptionIfNullOrEmpty(uri, "uri");

            this.uri = uri;
        }

        /// <summary>
        /// Gets the URI.
        /// </summary>
        /// <value>The URI.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings")]
        public string Uri
        {
            get
            {
                return this.uri;
            }
        }

        /// <summary>
        /// Sets the data.
        /// </summary>
        /// <param name="data">The data.</param>
        public void SetData(byte[] data)
        {
            this.data = data;
        }

        internal byte[] GetData()
        {
            return this.data;
        }
    }
}