﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes
{
    internal class ClassAttribute : HtmlAttribute<ClassNamesCollection>
    {
        private const string ClassSeparator = " ";

        public ClassAttribute()
            : base("class")
        {
            this.Value = new ClassNamesCollection();
        }

        public override bool HasValue
        {
            get
            {
                return this.Value.Count > 0;
            }
        }

        public override string GetValue()
        {
            return string.Join(ClassSeparator, this.Value.ToEnumerable());
        }

        public override void SetValue(string value)
        {
            Guard.ThrowExceptionIfNullOrEmpty(value, "value");

            this.Value.AddRange(value.Split(new string[] { ClassSeparator }, StringSplitOptions.RemoveEmptyEntries));
        }

        public override void ResetValue()
        {
            this.Value.Clear();
        }

        public override bool ShouldExport(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            return base.ShouldExport(writer, context) &&
                (context.Settings.StylesExportMode == StylesExportMode.Embedded ||
                context.Settings.StylesExportMode == StylesExportMode.External);
        }
    }
}
