﻿using System;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Attributes;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Namespaces;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes
{
    internal abstract class HtmlAttribute : XmlAttribute
    {
        public HtmlAttribute(string name, XmlNamespace ns, bool isRequired)
            : base(name, ns, isRequired)
        {
        }

        public virtual bool ShouldExport(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            return this.ShouldExport();
        }
    }

    internal class HtmlAttribute<T> : HtmlAttribute
    {
        private T defaultValue;
        private T value;
        private bool isSet;

        public HtmlAttribute(string name, XmlNamespace ns, bool isRequired = false)
            : this(name, ns, default(T), isRequired)
        {
        }

        public HtmlAttribute(string name, bool isRequired = false)
            : this(name, null, default(T), isRequired)
        {
        }

        public HtmlAttribute(string name, XmlNamespace ns, T defaultValue, bool isRequired = false)
            : base(name, ns, isRequired)
        {
            this.defaultValue = defaultValue;
            this.value = defaultValue;
        }

        public T DefaultValue
        {
            get
            {
                return this.defaultValue;
            }
            protected set
            {
                this.defaultValue = value;
                if (!this.isSet)
                {
                    this.ResetValue();
                }
            }
        }

        public T Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if (!ObjectExtensions.EqualsOfT(this.value, value))
                {
                    this.value = value;
                }

                this.isSet = true;
            }
        }

        public override bool HasValue
        {
            get
            {
                return this.isSet || !ObjectExtensions.EqualsOfT(this.value, this.defaultValue);
            }
        }

        public override string GetValue()
        {
            if (object.ReferenceEquals(this.Value, null))
            {
                return string.Empty;
            }

            return this.Value.ToString();
        }

        public override void SetValue(string value)
        {
            Guard.ThrowExceptionIfNull(value, "value");
            try
            {
                this.Value = (T)Convert.ChangeType(value, typeof(T), System.Globalization.CultureInfo.InvariantCulture);

                this.isSet = true;
            }
            catch (FormatException)
            {
            }
        }

        public override bool ShouldExport()
        {
            return this.IsRequired || this.HasValue;
        }

        public override void ResetValue()
        {
            this.value = this.defaultValue;
            this.isSet = false;
        }
    }
}
