﻿using System;
using Telerik.Windows.Documents.Common.FormatProviders.XmlBased.Model.Namespaces;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes
{
    internal class StyleValueAttribute : HtmlAttribute
    {
        private readonly StyleAttribute styleAttribute;
        private readonly string stylePropertyName;
        private readonly ILegacyConverter converter;

        public StyleValueAttribute(string name, string stylePropertyName, StyleAttribute styleAttribute, XmlNamespace ns = null, bool isRequired = false, ILegacyConverter converter = null)
            : base(name, ns, isRequired)
        {
            Guard.ThrowExceptionIfNull(styleAttribute, "styleAttribute");
            Guard.ThrowExceptionIfNull(stylePropertyName, "stylePropertyName");

            this.styleAttribute = styleAttribute;
            this.stylePropertyName = stylePropertyName;
            this.converter = converter;
        }

        public StyleValueAttribute(string name, StyleAttribute styleAttribute, XmlNamespace ns = null, bool isRequired = false, ILegacyConverter converter = null)
            : this(name, name, styleAttribute, ns, isRequired, converter)
        {
        }

        public override bool HasValue
        {
            get
            {
                return false;
            }
        }

        public override string GetValue()
        {
            throw new NotSupportedException();
        }

        public override void ResetValue()
        {
        }

        public override void SetValue(string value)
        {
            HtmlStyleProperty property;
            if (!this.styleAttribute.Value.TryGetProperty(this.stylePropertyName, out property))
            {
                string convertedLegacyValue = string.Empty;
                if (this.converter != null && this.converter.TryGetConvertedValue(value, out convertedLegacyValue))
                {
                    this.styleAttribute.Value.RegisterProperty(this.stylePropertyName, convertedLegacyValue);
                }
                else
                {
                    this.styleAttribute.Value.RegisterProperty(this.stylePropertyName, value);
                }
            }
        }

        public override bool ShouldExport()
        {
            return false;
        }
    }
}
