﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.BubblingRules
{
    internal class TableRowHeightBubblingRule : HtmlBubblingPropertyRule
    {
        public TableRowHeightBubblingRule(IStylePropertyDefinition propertyDefinition)
            : base(propertyDefinition)
        {
        }

        public override bool ShouldRegisterProperty(DocumentElementPropertiesBase documentElementProperties)
        {
            TableCellProperties tableCellProperties = documentElementProperties as TableCellProperties;
            if (tableCellProperties != null)
            {
                return tableCellProperties.RowSpan.GetActualValue().Value == 1;
            }

            TableRowProperties tableRowProperties = documentElementProperties as TableRowProperties;
            if (tableRowProperties != null)
            {
                return true;
            }

            return false;
        }

        protected override bool IsRuleSatisfied(object value, object candidateValue, out object result)
        {
            Guard.ThrowExceptionIfNull(value, "value");
            Guard.ThrowExceptionIfNull(candidateValue, "candidateValue");

            TableRowHeight valueInType = (TableRowHeight)value;
            TableRowHeight candidateValueInType = (TableRowHeight)candidateValue;

            if (candidateValueInType.Value > valueInType.Value)
            {
                result = candidateValue;
                return true;
            }

            result = null;
            return false;
        }
    }
}
