﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class BaselineAlignmentConverter : StringConverterBase<BaselineAlignment>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out BaselineAlignment result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            switch (property.Values[0].Value)
            {
                case "sub":
                    result = BaselineAlignment.Subscript;
                    return true;
                case "super":
                    result = BaselineAlignment.Superscript;
                    return true;
                case "baseline":
                    result = BaselineAlignment.Baseline;
                    return true;
            }

            result = BaselineAlignment.Baseline;
            return false;
        }

        public override bool ConvertBack(IHtmlExportContext context, BaselineAlignment value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            switch (value)
            {
                case BaselineAlignment.Subscript:
                    result = "sub";
                    return true;
                case BaselineAlignment.Superscript:
                    result = "super";
                    return true;
                case BaselineAlignment.Baseline:
                    result = "baseline";
                    return true;
            }

            result = "baseline";
            return false;
        }
    }
}
