﻿using System;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class BorderConverter : StringConverterBase<Border>
    {
        private static readonly ThemableColor DefaultBorderColor = new ThemableColor(Colors.Black);

        public override bool Convert(Import.IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out Border result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(property, "property");

            int parsedValues = 0;

            result = null;
            if (property.Values.Count == 0)
            {
                return false;
            }

            // Parse thickness.
            double thickness = 1;
            bool parsedThickness = false;
            for (int i = 0; i < property.Values.Count; i++)
            {
                HtmlStylePropertyValue borderWidth = property.Values[i];
                if (ParseThickness(borderWidth, out thickness))
                {
                    parsedThickness = true;
                    break;
                }
            }

            if (parsedThickness)
            {
                parsedValues++;
            }
            else
            {
                thickness = 1;
            }

            // Parse style
            BorderStyle style = BorderStyle.Single;
            bool parsedStyle = false;
            for (int i = 0; i < property.Values.Count; i++)
            {
                HtmlStylePropertyValue borderStyle = property.Values[i];

                if (ParseStyle(borderStyle, out style))
                {
                    parsedStyle = true;
                    break;
                }
            }

            if (parsedStyle)
            {
                parsedValues++;
            }
            else
            {
                style = BorderStyle.Single;
            }

            // Parse color
            ThemableColor color = BorderConverter.DefaultBorderColor;
            if (parsedValues == property.Values.Count - 1)
            {
                bool parsedColor = false;
                for (int i = 0; i < property.Values.Count; i++)
                {
                    HtmlStylePropertyValue borderColor = property.Values[i];

                    if (ParseColor(borderColor, context, properties, out color))
                    {
                        parsedColor = true;
                        break;
                    }
                }

                if (!parsedColor)
                {
                    result = null;
                    return false;
                }
            }

            result = new Border(thickness, style, color);
            return true;
        }

        public override bool ConvertBack(Export.IHtmlExportContext context, Border value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(value, "property");

            // TODO: Handle all border types.
            result = null;
            if (value.Style == BorderStyle.Inherit)
            {
                // Cannot export Inherit border;
                return false;
            }

            // thickness
            double thickness = Math.Max(value.Thickness, context.Settings.BordersMinimalThickness);
            string borderWidth = CssValueTypesHelper.ConvertToPixels(context, thickness);

            // style
            string borderStyle;
            if (!HtmlValueMappers.BorderStyleValueMapper.TryGetFromValue(value.Style, out borderStyle))
            {
                borderStyle = HtmlValueMappers.BorderStyleValueMapper.DefaultFromValue.Value;
            }

            // color
            string borderColor;
            if (!HtmlConverters.ThemableColorConverter.ConvertBack(context, value.Color, properties, out borderColor))
            {
                return false;
            }

            result = borderWidth + " " + borderStyle + " " + borderColor;
            return true;
        }

        private static bool ParseThickness(HtmlStylePropertyValue borderWidth, out double thickness)
        {
            return CssValueTypesHelper.TryConvertNonRelativeLengthValue(borderWidth, out thickness);
        }

        private static bool ParseColor(HtmlStylePropertyValue borderColor, Import.IHtmlImportContext context, DocumentElementPropertiesBase properties, out ThemableColor color)
        {
            HtmlStyleProperty htmlStyleProperty = new HtmlStyleProperty("color", borderColor.Value);

            return HtmlConverters.ThemableColorConverter.Convert(context, htmlStyleProperty, properties, out color);
        }

        private static bool ParseStyle(HtmlStylePropertyValue borderStyle, out BorderStyle style)
        {
            return HtmlValueMappers.BorderStyleValueMapper.TryGetToValue(borderStyle.Value, out style);
        }
    }
}
