using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    // TODO: Think of better name.
    internal static class CssValueTypesHelper
    {
        public static bool IsPercentValue(HtmlStylePropertyValue val)
        {
            return val.UnitType.HasValue && val.UnitType.Value == UnitType.Percent;
        }

        public static bool IsLengthValue(HtmlStylePropertyValue val)
        {
            if (!val.UnitType.HasValue)
            {
                return false;
            }

            return val.UnitType.Value != UnitType.Percent;
        }

        public static bool IsNumberValue(HtmlStylePropertyValue val)
        {
            return !val.UnitType.HasValue && !double.IsNaN(val.ValueAsDouble);
        }

        public static bool TryConvertNonRelativeLengthValue(HtmlStylePropertyValue val, out double result)
        {
            // When a value is defined in CSS style, it should always have a unit type specified. 
            // For reference: http://www.w3.org/TR/css3-values/#lengths
            if (val.UnitType.HasValue &&
                CssValueTypesHelper.IsLengthValue(val) &&
                !Unit.IsRelativeUnitType(val.UnitType.Value))
            {
                result = Unit.UnitToDip(val.ValueAsDouble, val.UnitType.Value);
                return true;
            }
            else
            {
                result = double.NaN;
                return false;
            }
        }

        public static string ConvertToPixels(IHtmlExportContext context, double value)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return string.Format("{0}px", ToString(context, value));
        }

        public static string ConvertToPercents(IHtmlExportContext context, double value)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return string.Format("{0}%", ToString(context, value));
        }

        public static string ToString(IHtmlExportContext context, double value)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return value.ToString(context.Culture);
        }

        public static string ToLower(IHtmlExportContext context, string value)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return value.ToLower(context.Culture);
        }
    }
}