﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class FontSizeConverter : StringConverterBase<double>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out double result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            result = double.NaN;
            if (property.Values.Count > 1)
            {
                return false;
            }

            HtmlStylePropertyValue value = property.Values[0];
            IStyleProperty<double?> styleProperty = properties.GetStyleProperty(Run.FontSizePropertyDefinition) as IStyleProperty<double?>;

            if (value.UnitType.HasValue && Unit.IsRelativeUnitType(value.UnitType.Value))
            {
                // Value depends on base property;
                double basePixelSize;
                if (property.BaseProperty != null)
                {
                    HtmlConverters.FontSizeConverter.Convert(context, property.BaseProperty, properties, out basePixelSize);
                }
                else
                {
                    basePixelSize = (double)styleProperty.GetActualValueAsObject();
                }

                result = Unit.UnitToPixel(basePixelSize, value.ValueAsDouble, value.UnitType.Value);
            }
            else
            {
                result = Unit.UnitToDip(value.ValueAsDouble, value.UnitType.HasValue ? value.UnitType.Value : UnitType.Dip);
            }

            return !double.IsNaN(result) && result > 0.1;
        }

        public override bool ConvertBack(IHtmlExportContext context, double value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            result = CssValueTypesHelper.ConvertToPixels(context, value);

            return true;
        }
    }
}
