﻿using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using FontWeight = System.Windows.FontWeight;
using FontWeights = System.Windows.FontWeights;
using FontStyle = System.Windows.FontStyle;
using FontStyles = System.Windows.FontStyles;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal static class HtmlValueMappers
    {
        public static readonly ValueMapper<string, TableLayoutType> TableLayoutTypeMapper;
        public static readonly ValueMapper<string, BaselineAlignment> BaselineAlignmentMapper;
        public static readonly ValueMapper<string, VerticalAlignment> VerticalAlignmentMapper;
        public static readonly ValueMapper<string, NumberingStyle> NumberingStyleTypeMapper;
        public static readonly ValueMapper<string, NumberingStyle> NumberingStyleMapper;
        public static readonly ValueMapper<string, Alignment> AlignmentValueMapper;
        public static readonly ValueMapper<string, BorderStyle> BorderStyleValueMapper;
        public static readonly ValueMapper<string, FlowDirection> FlowDirectionValueMapper;
        public static readonly ValueMapper<string, FontWeight> FontWeightValueMapper;
        public static readonly ValueMapper<string, FontStyle> FontStyleValueMapper;
        public static readonly ValueMapper<string, bool> StrikethroughValueMapper;

        static HtmlValueMappers()
        {
            TableLayoutTypeMapper = InitializeTableLayoutTypeMapper();
            BaselineAlignmentMapper = InitializeBaselineAlignmentMapper();
            VerticalAlignmentMapper = InitializeVerticalAlignmentMapper();
            NumberingStyleTypeMapper = InitializeNumberingStyleTypeMapper();
            NumberingStyleMapper = InitializeNumberingStyleMapper();
            AlignmentValueMapper = InitializeAlignmentMapper();
            BorderStyleValueMapper = InitializeBorderStyleMapper();
            FlowDirectionValueMapper = InitializeFlowDirectionMapper();
            FontWeightValueMapper = InitializeFontWeightMapper();
            FontStyleValueMapper = InitializeFontStyleMapper();
            StrikethroughValueMapper = InitializeStrikethroughMapper();
        }

        private static ValueMapper<string, TableLayoutType> InitializeTableLayoutTypeMapper()
        {
            ValueMapper<string, TableLayoutType> mapper = new ValueMapper<string, TableLayoutType>();
            mapper.AddPair("auto", TableLayoutType.AutoFit);
            mapper.AddPair("fixed", TableLayoutType.FixedWidth);

            return mapper;
        }

        private static ValueMapper<string, BaselineAlignment> InitializeBaselineAlignmentMapper()
        {
            ValueMapper<string, BaselineAlignment> mapper = new ValueMapper<string, BaselineAlignment>();
            mapper.AddPair("sub", BaselineAlignment.Subscript);
            mapper.AddPair("super", BaselineAlignment.Superscript);
            mapper.AddPair("baseline", BaselineAlignment.Baseline);

            return mapper;
        }

        private static ValueMapper<string, VerticalAlignment> InitializeVerticalAlignmentMapper()
        {
            ValueMapper<string, VerticalAlignment> mapper = new ValueMapper<string, VerticalAlignment>();
            mapper.AddPair("top", VerticalAlignment.Top);
            mapper.AddPair("middle", VerticalAlignment.Center);
            mapper.AddPair("bottom", VerticalAlignment.Bottom);

            return mapper;
        }

        private static ValueMapper<string, NumberingStyle> InitializeNumberingStyleTypeMapper()
        {
            ValueMapper<string, NumberingStyle> mapper = new ValueMapper<string, NumberingStyle>("decimal", NumberingStyle.Decimal);

            mapper.AddPair("circle", NumberingStyle.Bullet);
            mapper.AddPair("square", NumberingStyle.Bullet);
            mapper.AddPair("disc", NumberingStyle.Bullet);
            mapper.AddPair("lower-greek", NumberingStyle.Decimal);
            mapper.AddPair("lower-alpha", NumberingStyle.LowerLetter);
            mapper.AddPair("upper-alpha", NumberingStyle.UpperLetter);
            mapper.AddPair("lower-roman", NumberingStyle.LowerRoman);
            mapper.AddPair("upper-roman", NumberingStyle.UpperRoman);
            mapper.AddPair("lower-latin", NumberingStyle.LowerLetter);
            mapper.AddPair("upper-latin", NumberingStyle.UpperLetter);
            mapper.AddPair("decimal", NumberingStyle.Decimal);

            return mapper;
        }

        private static ValueMapper<string, NumberingStyle> InitializeNumberingStyleMapper()
        {
            ValueMapper<string, NumberingStyle> mapper = new ValueMapper<string, NumberingStyle>("1", NumberingStyle.Decimal);

            mapper.AddPair("a", NumberingStyle.LowerLetter);
            mapper.AddPair("A", NumberingStyle.UpperLetter);
            mapper.AddPair("i", NumberingStyle.LowerRoman);
            mapper.AddPair("I", NumberingStyle.UpperRoman);

            mapper.AddPair("disc", NumberingStyle.Bullet);
            mapper.AddPair("circle", NumberingStyle.Bullet);
            mapper.AddPair("square", NumberingStyle.Bullet);

            return mapper;
        }

        private static ValueMapper<string, Alignment> InitializeAlignmentMapper()
        {
            ValueMapper<string, Alignment> mapper = new ValueMapper<string, Alignment>();
            mapper.AddPair("left", Alignment.Left);
            mapper.AddPair("center", Alignment.Center);
            mapper.AddPair("right", Alignment.HighKashida);
            mapper.AddPair("right", Alignment.MediumKashida);
            mapper.AddPair("right", Alignment.LowKashida);
            mapper.AddPair("right", Alignment.Right);
            mapper.AddPair("justify", Alignment.ThaiDistribute);
            mapper.AddPair("justify", Alignment.Distribute);
            mapper.AddPair("justify", Alignment.Justified);

            return mapper;
        }

        private static ValueMapper<string, BorderStyle> InitializeBorderStyleMapper()
        {
            ValueMapper<string, BorderStyle> mapper = new ValueMapper<string, BorderStyle>("solid", BorderStyle.Inherit);

            mapper.AddPair("hidden", BorderStyle.None);
            mapper.AddPair("none", BorderStyle.None);
            mapper.AddPair("groove", BorderStyle.Single);
            mapper.AddPair("ridge", BorderStyle.Single);
            mapper.AddPair("solid", BorderStyle.Single);
            mapper.AddPair("dotted", BorderStyle.Dotted);
            mapper.AddPair("dashed", BorderStyle.Dashed);
            mapper.AddPair("double", BorderStyle.Double);
            mapper.AddPair("inset", BorderStyle.Inset);
            mapper.AddPair("outset", BorderStyle.Outset);

            return mapper;
        }

        private static ValueMapper<string, FlowDirection> InitializeFlowDirectionMapper()
        {
            ValueMapper<string, FlowDirection> mapper = new ValueMapper<string, FlowDirection>();

            mapper.AddPair("rtl", FlowDirection.RightToLeft);
            mapper.AddPair("ltr", FlowDirection.LeftToRight);

            return mapper;
        }

        private static ValueMapper<string, FontWeight> InitializeFontWeightMapper()
        {
            ValueMapper<string, FontWeight> mapper = new ValueMapper<string, FontWeight>();

            mapper.AddPair("bold", FontWeights.Bold);
            mapper.AddPair("normal", FontWeights.Normal);

            return mapper;
        }

        private static ValueMapper<string, FontStyle> InitializeFontStyleMapper()
        {
            ValueMapper<string, FontStyle> mapper = new ValueMapper<string, FontStyle>();

            mapper.AddPair("italic", FontStyles.Italic);
            mapper.AddPair("normal", FontStyles.Normal);

            return mapper;
        }

        private static ValueMapper<string, bool> InitializeStrikethroughMapper()
        {
            ValueMapper<string, bool> mapper = new ValueMapper<string, bool>();

            mapper.AddPair("line-through", true);
            mapper.AddPair(string.Empty, false);

            return mapper;
        }
    }
}
