using System.Diagnostics;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class LineSpacingConverter : StringConverterBase<double>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out double result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            HtmlStylePropertyValue firstValue = property.Values[0];
            result = double.NaN;
            if (firstValue.UnitType.HasValue)
            {
                if (CssValueTypesHelper.IsPercentValue(firstValue))
                {
                    result = firstValue.ValueAsDouble / 100;
                }
                else if (CssValueTypesHelper.IsLengthValue(firstValue) &&
                    !Unit.IsRelativeUnitType(firstValue.UnitType.Value))
                {
                    result = Unit.UnitToDip(firstValue.ValueAsDouble, firstValue.UnitType.Value);
                }
            }
            else if (CssValueTypesHelper.IsNumberValue(firstValue))
            {
                result = firstValue.ValueAsDouble;
            }

            return !double.IsNaN(result);
        }

        public override bool ConvertBack(IHtmlExportContext context, double value, DocumentElementPropertiesBase properties, out string result)
        {
            Debug.Fail("ConvertBack should not be called - line-height property is set in LineSpacingTypeConverter.");
            result = null;
            return false;
        }
    }
}