﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class LineSpacingTypeConverter : IStringConverter
    {
        public bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out object result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            HtmlStylePropertyValue firstValue = property.Values[0];
            result = HeightType.Auto;
            if (firstValue.UnitType.HasValue)
            {
                switch (firstValue.UnitType.Value)
                {
                    case UnitType.Dip:
                    case UnitType.Em:
                    case UnitType.Point:
                        result = HeightType.Exact;
                        break;
                    default:
                        break;
                }
            }
            return true;
        }

        public bool ConvertBack(IHtmlExportContext context, object value, HtmlStylePropertyDescriptor descriptor, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            HeightType lineSpacingType = (HeightType)value;

            IStyleProperty<double?> lineSpacingStyleProperty = properties.GetStyleProperty(Paragraph.LineSpacingPropertyDefinition) as IStyleProperty<double?>;
            double? lineSpacingValue = lineSpacingStyleProperty.GetActualValue();

            if (lineSpacingType == HeightType.Auto)
            {
                result = CssValueTypesHelper.ConvertToPercents(context, lineSpacingValue.Value * 100);
            }
            else
            {
                result = CssValueTypesHelper.ConvertToPixels(context, lineSpacingValue.Value);
            }

            return true;
        }
    }
}
