﻿using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class MappedConverter<T> : StringConverterBase<T>
    {
        private readonly ValueMapper<string, T> mapper;

        public MappedConverter(ValueMapper<string, T> mapper)
        {
            Guard.ThrowExceptionIfNull(mapper, "mapper");

            this.mapper = mapper;
        }

        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out T result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            return this.mapper.TryGetToValue(property.Values[0].Value.ToLowerInvariant(), out result);
        }

        public override bool ConvertBack(Export.IHtmlExportContext context, T value, Flow.Model.Styles.DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            return this.mapper.TryGetFromValue(value, out result);
        }
    }
}
