﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class PaddingConverter : StringConverterBase<Padding>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out Padding result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            result = null;

            Flow.Model.Styles.Core.IStyleProperty paddingProperty = properties.GetStyleProperty(TableCell.PaddingPropertyDefinition) ?? properties.GetStyleProperty(Table.TableCellPaddingPropertyDefinition);
            Padding padding = paddingProperty.GetActualValueAsObject() as Padding;

            if (property.Name == HtmlStylePropertyDescriptors.PaddingPropertyDescriptor.Name)
            {
                double left, right, top, bottom;
                TryGetPaddingValues(property.Values, out left, out right, out top, out bottom);
                result = new Padding(left, top, right, bottom);

                return true;
            }
            else
            {
                double pixelValue;
                HtmlStylePropertyValue propValue = property.Values[0];
                if (CssValueTypesHelper.TryConvertNonRelativeLengthValue(propValue, out pixelValue))
                {
                    if (padding == null)
                    {
                        padding = new Padding(0);
                    }

                    if (property.Name == HtmlStylePropertyDescriptors.PaddingLeftPropertyDescriptor.Name)
                    {
                        result = SetLeft(padding, pixelValue);
                    }
                    else if (property.Name == HtmlStylePropertyDescriptors.PaddingRightPropertyDescriptor.Name)
                    {
                        result = SetRight(padding, pixelValue);
                    }
                    else if (property.Name == HtmlStylePropertyDescriptors.PaddingTopPropertyDescriptor.Name)
                    {
                        result = SetTop(padding, pixelValue);
                    }
                    else if (property.Name == HtmlStylePropertyDescriptors.PaddingBottomPropertyDescriptor.Name)
                    {
                        result = SetBottom(padding, pixelValue);
                    }
                    else
                    {
                        result = new Padding(pixelValue);
                    }

                    return true;
                }
            }

            return false;
        }

        public override bool ConvertBack(IHtmlExportContext context, Padding value, DocumentElementPropertiesBase properties, out string result)
        {
            result = null;

            return false;
        }

        private static void TryGetPaddingValues(HtmlStylePropertyValues values, out double left, out double right, out double top, out double bottom)
        {
            Guard.ThrowExceptionIfNull(values, "values");

            double[] pixelValues = new double[values.Count];
            for (int i = 0; i < values.Count; i++)
            {
                double result;
                if (CssValueTypesHelper.TryConvertNonRelativeLengthValue(values[i], out result))
                {
                    pixelValues[i] = result;
                }
                else
                {
                    pixelValues[i] = 0;
                }
            }

            switch (pixelValues.Length)
            {
                case 1:
                    left = right = top = bottom = pixelValues[0];

                    break;
                case 2:
                    top = bottom = pixelValues[0];
                    left = right = pixelValues[1];

                    break;
                case 3:
                    top = pixelValues[0];
                    left = right = pixelValues[1];
                    bottom = pixelValues[2];

                    break;
                case 4:
                    top = pixelValues[0];
                    right = pixelValues[1];
                    bottom = pixelValues[2];
                    left = pixelValues[3];

                    break;
                default:
                    left = right = top = bottom = 0;

                    break;
            }
        }

        private static Padding SetLeft(Padding padding, double left)
        {
            Guard.ThrowExceptionIfNull(padding, "padding");

            return new Padding(left, padding.Top, padding.Right, padding.Bottom);
        }

        private static Padding SetRight(Padding padding, double right)
        {
            Guard.ThrowExceptionIfNull(padding, "padding");

            return new Padding(padding.Left, padding.Top, right, padding.Bottom);
        }

        private static Padding SetTop(Padding padding, double top)
        {
            Guard.ThrowExceptionIfNull(padding, "padding");

            return new Padding(padding.Left, top, padding.Right, padding.Bottom);
        }

        private static Padding SetBottom(Padding padding, double bottom)
        {
            Guard.ThrowExceptionIfNull(padding, "padding");

            return new Padding(padding.Left, padding.Top, padding.Right, bottom);
        }
    }
}
