﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class PaddingShortHandConverter : StringConverterBase<Padding>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out Padding result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            double left, right, top, bottom;
            GetPaddingValues(property.Values, out left, out right, out top, out bottom);
            result = new Padding(left, top, right, bottom);

            return true;
        }

        public override bool ConvertBack(IHtmlExportContext context, Padding value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            bool hasValue = false;

            Padding padding = value;
            Flow.Model.Styles.Core.IStyleProperty property = properties.GetStyleProperty(TableCell.PaddingPropertyDefinition);

            if (property != null && !property.HasLocalValue)
            {
                TableCell cell = properties.OwnerDocumentElement as TableCell;
                if (cell != null)
                {
                    if (cell.Row.Table.Properties.TableCellPadding.HasLocalValue)
                    {
                        padding = cell.Row.Table.TableCellPadding;
                    }

                    hasValue = true;
                }
            }
            else
            {
                hasValue = true;
            }

            result = string.Format("{0} {1} {2} {3}",
                CssValueTypesHelper.ConvertToPixels(context, padding.Top),
                CssValueTypesHelper.ConvertToPixels(context, padding.Right),
                CssValueTypesHelper.ConvertToPixels(context, padding.Bottom),
                CssValueTypesHelper.ConvertToPixels(context, padding.Left));

            return hasValue;
        }

        private static void GetPaddingValues(HtmlStylePropertyValues values, out double left, out double right, out double top, out double bottom)
        {
            Guard.ThrowExceptionIfNull(values, "values");

            double[] pixelValues = GetPixelValues(values);
            switch (pixelValues.Length)
            {
                case 1:
                    left = right = top = bottom = pixelValues[0];

                    break;
                case 2:
                    top = bottom = pixelValues[0];
                    left = right = pixelValues[1];

                    break;
                case 3:
                    top = pixelValues[0];
                    left = right = pixelValues[1];
                    bottom = pixelValues[2];

                    break;
                case 4:
                    top = pixelValues[0];
                    right = pixelValues[1];
                    bottom = pixelValues[2];
                    left = pixelValues[3];

                    break;
                default:
                    left = right = top = bottom = 0;

                    break;
            }
        }

        private static double[] GetPixelValues(HtmlStylePropertyValues values)
        {
            Guard.ThrowExceptionIfNull(values, "values");

            double[] pixelValues = new double[values.Count];
            for (int i = 0; i < values.Count; i++)
            {
                double result;
                if (CssValueTypesHelper.TryConvertNonRelativeLengthValue(values[i], out result))
                {
                    pixelValues[i] = result;
                }
                else
                {
                    pixelValues[i] = 0;
                }
            }

            return pixelValues;
        }
    }
}
