﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class PreferredWidthConverter : StringConverterBase<TableWidthUnit>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out TableWidthUnit result)
        {
            HtmlStylePropertyValue propValue = property.Values[0];

            result = null;

            if (propValue.UnitType.HasValue)
            {
                if (propValue.UnitType.Value == UnitType.Percent)
                {
                    result = new TableWidthUnit(TableWidthUnitType.Percent, propValue.ValueAsDouble);
                }
                else
                {
                    double pixelWidth;
                    if (CssValueTypesHelper.TryConvertNonRelativeLengthValue(propValue, out pixelWidth))
                    {
                        result = new TableWidthUnit(TableWidthUnitType.Fixed, pixelWidth);
                    }
                }
            }
            else
            {
                if (propValue.Value.ToLowerInvariant() == "auto")
                {
                    result = new TableWidthUnit(TableWidthUnitType.Auto);
                }
            }

            return result != null;
        }

        public override bool ConvertBack(IHtmlExportContext context, TableWidthUnit value, DocumentElementPropertiesBase properties, out string result)
        {
            switch (value.Type)
            {
                case TableWidthUnitType.Fixed:
                    result = CssValueTypesHelper.ConvertToPixels(context, value.Value);
                    break;
                case TableWidthUnitType.Percent:
                    result = CssValueTypesHelper.ConvertToPercents(context, value.Value);
                    break;
                case TableWidthUnitType.Auto:
                case TableWidthUnitType.Nil:
                default:
                    result = string.Empty;
                    break;
            }

            return !string.IsNullOrEmpty(result);
        }
    }
}
