﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class TableBordersColorConverter : IStringConverter
    {
        public bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out object result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(property, "property");

            result = null;
            TableBorders borders = new TableBorders();

            Flow.Model.Styles.Core.IStyleProperty styleProperty = properties.GetStyleProperty(Table.TableBordersPropertyDefinition);
            borders = styleProperty.GetActualValueAsObject() as TableBorders;

            // Parse color
            ThemableColor color = new ThemableColor(Colors.Black);
            HtmlStylePropertyValue borderColor = property.Values[0];
            if (borderColor != null)
            {
                HtmlStyleProperty htmlStyleProperty = new HtmlStyleProperty("color", borderColor.Value);
                if (HtmlConverters.ThemableColorConverter.Convert(context, htmlStyleProperty, properties, out color))
                {
                    result = GetTableBorders(borders, color);
                    return true;
                }
            }

            return false;
        }

        public bool ConvertBack(IHtmlExportContext context, object value, HtmlStylePropertyDescriptor descriptor, DocumentElementPropertiesBase properties, out string result)
        {
            // Table border styles will be exported through 'border' style property. 
            // When support for a single border-color (e.g. 'border-bottom-color' attribute is added, change this.
            result = null;
            return false;
        }

        private static TableBorders GetTableBorders(TableBorders borders, ThemableColor color)
        {
            Border currentBorder = borders.Left;
            Border leftBorder = new Border(currentBorder.Thickness,
                                           currentBorder.Style,
                                           color ?? currentBorder.Color,
                                           currentBorder.Shadow,
                                           currentBorder.Frame,
                                           currentBorder.Spacing);

            currentBorder = borders.Top;
            Border topBorder = new Border(currentBorder.Thickness,
                                          currentBorder.Style,
                                          color ?? currentBorder.Color,
                                          currentBorder.Shadow,
                                          currentBorder.Frame,
                                          currentBorder.Spacing);

            currentBorder = borders.Right;
            Border rightBorder = new Border(currentBorder.Thickness,
                                            currentBorder.Style,
                                            color ?? currentBorder.Color,
                                            currentBorder.Shadow,
                                            currentBorder.Frame,
                                            currentBorder.Spacing);

            currentBorder = borders.Bottom;
            Border bottomBorder = new Border(currentBorder.Thickness,
                                             currentBorder.Style,
                                             color ?? currentBorder.Color,
                                             currentBorder.Shadow,
                                             currentBorder.Frame,
                                             currentBorder.Spacing);

            return new TableBorders(leftBorder, topBorder, rightBorder, bottomBorder);
        }
    }
}