﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class TableBordersConverter : IStringConverter
    {
        public bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out object result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(property, "property");

            result = null;
            TableBorders borders = new TableBorders();

            Flow.Model.Styles.Core.IStyleProperty styleProperty = properties.GetStyleProperty(Table.TableBordersPropertyDefinition);
            borders = styleProperty.GetActualValueAsObject() as TableBorders;

            Border border;
            if (!HtmlConverters.BorderConverter.Convert(context, property, properties, out border))
            {
                return false;
            }

            if (property.Name == HtmlStylePropertyDescriptors.BorderLeftPropertyDescriptor.Name)
            {
                borders = new TableBorders(borders, leftBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderTopPropertyDescriptor.Name)
            {
                borders = new TableBorders(borders, topBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderRightPropertyDescriptor.Name)
            {
                borders = new TableBorders(borders, rightBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderBottomPropertyDescriptor.Name)
            {
                borders = new TableBorders(borders, bottomBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderPropertyDescriptor.Name)
            {
                borders = new TableBorders(border, border, border, border);
            }

            result = borders;
            return true;
        }

        public bool ConvertBack(IHtmlExportContext context, object value, HtmlStylePropertyDescriptor descriptor, DocumentElementPropertiesBase properties, out string result)
        {
            TableBorders borders = value as TableBorders;
            result = null;

            bool areAllbordersSame = borders.Left.Equals(borders.Top) &&
                borders.Left.Equals(borders.Right) &&
                borders.Left.Equals(borders.Bottom);
            if (areAllbordersSame)
            {
                if (descriptor.Name == HtmlStylePropertyDescriptors.BorderPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, borders.Left, properties, out result);
                }
            }
            else
            {
                if (descriptor.Name == HtmlStylePropertyDescriptors.BorderLeftPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, borders.Left, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderTopPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, borders.Top, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderRightPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, borders.Right, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderBottomPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, borders.Bottom, properties, out result);
                }
            }

            return false;
        }
    }
}
