using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.BorderEvaluation;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class TableCellBordersConverter : IStringConverter
    {
        public bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out object result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(property, "property");

            result = null;
            TableCellBorders borders = new TableCellBorders();

            Flow.Model.Styles.Core.IStyleProperty styleProperty = properties.GetStyleProperty(TableCell.BordersPropertyDefinition);
            borders = styleProperty.GetActualValueAsObject() as TableCellBorders;

            Border border;
            if (!HtmlConverters.BorderConverter.Convert(context, property, properties, out border))
            {
                return false;
            }

            if (property.Name == HtmlStylePropertyDescriptors.BorderLeftPropertyDescriptor.Name)
            {
                borders = new TableCellBorders(borders, leftBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderTopPropertyDescriptor.Name)
            {
                borders = new TableCellBorders(borders, topBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderRightPropertyDescriptor.Name)
            {
                borders = new TableCellBorders(borders, rightBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderBottomPropertyDescriptor.Name)
            {
                borders = new TableCellBorders(borders, bottomBorder: border);
            }
            else if (property.Name == HtmlStylePropertyDescriptors.BorderPropertyDescriptor.Name)
            {
                borders = new TableCellBorders(border);
            }

            result = borders;
            return true;
        }

        public bool ConvertBack(IHtmlExportContext context, object value, HtmlStylePropertyDescriptor descriptor, DocumentElementPropertiesBase properties, out string result)
        {
            TableCellBorders borders = value as TableCellBorders;
            result = null;

            Border left = borders.Left;
            Border right = borders.Right;
            Border top = borders.Top;
            Border bottom = borders.Bottom;

            TableCell cell = properties.OwnerDocumentElement as TableCell;
            if (cell != null)
            {
                Table table = cell.Row.Table;
                if (table.HasCellSpacing)
                {
                    if (left.Style == BorderStyle.Inherit)
                    {
                        left = table.Borders.InsideVertical;
                    }
                    if (right.Style == BorderStyle.Inherit)
                    {
                        right = table.Borders.InsideVertical;
                    }
                    if (top.Style == BorderStyle.Inherit)
                    {
                        top = table.Borders.InsideHorizontal;
                    }
                    if (bottom.Style == BorderStyle.Inherit)
                    {
                        bottom = table.Borders.InsideHorizontal;
                    }
                }
                else
                {
                    TableBorderGrid borderGrid = context.GetBorderGrid(table);

                    top = borderGrid.GetTopHorizontalBorder(cell.GridRowIndex, cell.GridColumnIndex).Border;
                    bottom = borderGrid.GetTopHorizontalBorder(cell.GridRowIndex + cell.RowSpan, cell.GridColumnIndex).Border;
                    left = borderGrid.GetLeftVerticalBorder(cell.GridRowIndex, cell.GridColumnIndex).Border;
                    right = borderGrid.GetLeftVerticalBorder(cell.GridRowIndex, cell.GridColumnIndex + cell.ColumnSpan).Border;
                }
            }

            bool areAllbordersSame = left.Equals(top) && left.Equals(right) && left.Equals(bottom);
            if (areAllbordersSame)
            {
                if (descriptor.Name == HtmlStylePropertyDescriptors.BorderPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, left, properties, out result);
                }
            }
            else
            {
                if (descriptor.Name == HtmlStylePropertyDescriptors.BorderLeftPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, left, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderTopPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, top, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderRightPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, right, properties, out result);
                }
                else if (descriptor.Name == HtmlStylePropertyDescriptors.BorderBottomPropertyDescriptor.Name)
                {
                    return HtmlConverters.BorderConverter.ConvertBack(context, bottom, properties, out result);
                }
            }

            return false;
        }
    }
}