﻿using System.Globalization;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class ThemableFontFamilyConverter : StringConverterBase<ThemableFontFamily>
    {
        public override bool Convert(IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out ThemableFontFamily result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            string value = property.UnparsedValue;
            System.Collections.Generic.IEnumerable<string> results = value.Split(',').Select(s => s.Trim(new char[] { '\'', '"' }));

            string fontFamilyString = results.First();

            // Try parse generic font.
            GenericHtmlFonts genericFonts = context.Settings.GenericFonts;
            switch (fontFamilyString)
            {
                case "cursive":
                    result = genericFonts.Cursive;
                    break;

                case "fantasy":
                    result = genericFonts.Fantasy;
                    break;

                case "monospace":
                    result = genericFonts.Monospace;
                    break;

                case "sans-serif":
                    result = genericFonts.SansSerif;
                    break;

                case "serif":
                    result = genericFonts.Serif;
                    break;

                default:
                    fontFamilyString = CultureInfo.InvariantCulture.TextInfo.ToTitleCase(fontFamilyString);
                    result = new ThemableFontFamily(fontFamilyString);
                    break;
            }

            return true;
        }

        public override bool ConvertBack(IHtmlExportContext context, ThemableFontFamily value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            result = value.GetActualValue(context.Document.Theme).ToString();
            return true;
        }
    }
}
