﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters
{
    internal class UnderlinePatternConverter : StringConverterBase<UnderlinePattern>
    {
        public override bool Convert(Import.IHtmlImportContext context, HtmlStyleProperty property, DocumentElementPropertiesBase properties, out UnderlinePattern result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(property, "property");

            if (property.Values[0].Value == "underline")
            {
                result = UnderlinePattern.Single;
                return true;
            }

            result = UnderlinePattern.None;
            return false;
        }

        public override bool ConvertBack(Export.IHtmlExportContext context, UnderlinePattern value, DocumentElementPropertiesBase properties, out string result)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(value, "value");

            if (value != UnderlinePattern.None)
            {
                StyleProperty<ThemableColor> colorProp = properties.GetStyleProperty(Run.UnderlineColorPropertyDefinition) as StyleProperty<ThemableColor>;
                if (colorProp != null)
                {
                    ThemableColor themableColor = colorProp.GetActualValue();
                    if (themableColor.IsAutomatic || themableColor.GetActualValue(context.Document.Theme).A > 0)
                    {
                        result = "underline";
                        return true;
                    }
                }
            }

            result = null;
            return false;
        }
    }
}
