﻿using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements
{
    internal class LinkElement : HtmlElementBase
    {
        private static readonly Encoding cssEncoding = Encoding.UTF8;

        private readonly HtmlAttribute<string> referenceAttribute;
        private readonly HtmlAttribute<string> relationAttribute;
        private readonly HtmlAttribute<string> typeAttribute;

        public LinkElement(HtmlContentManager contentManger)
            : base(contentManger)
        {
            this.referenceAttribute = this.RegisterAttribute<string>("href");
            this.relationAttribute = this.RegisterAttribute<string>("rel");
            this.typeAttribute = this.RegisterAttribute<string>("type");
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.Link;
            }
        }

        public string Reference
        {
            get
            {
                return this.referenceAttribute.Value;
            }
            set
            {
                this.referenceAttribute.Value = value;
            }
        }

        public string Relation
        {
            get
            {
                return this.relationAttribute.Value;
            }
            set
            {
                this.relationAttribute.Value = value;
            }
        }

        public string Type
        {
            get
            {
                return this.typeAttribute.Value;
            }
            set
            {
                this.typeAttribute.Value = value;
            }
        }

        protected override void OnAfterRead(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.Relation == HtmlConstants.StyleSheet && (this.Type == HtmlConstants.TextCss || this.Type == null) && !string.IsNullOrEmpty(this.Reference))
            {
                byte[] data = context.Settings.GetDataFromUri(this.Reference);
                Guard.ThrowExceptionIfNull(data, string.Format("Can not get the data from {0}. Make sure the external content is loaded in HtmlImportSettings.LoadFromUri.", this.Reference));
                this.InnerText = cssEncoding.GetString(data);
            }
        }

        protected override void OnBeforeWrite(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            string css = StyleExporter.Export(context);

            ExternalStylesExportingEventArgs args = new ExternalStylesExportingEventArgs(css);
            context.Settings.OnExternalStylesExporting(args);

            if (args.Handled)
            {
                Guard.ThrowExceptionIfNullOrEmpty(args.Reference, "Reference");
                this.Reference = args.Reference;
            }
            else
            {
                if (context.Settings.StylesFilePath == null)
                {
                    throw new NotSupportedException("When exporting styles to external file, HtmlExportSettings.StylesFileName should be set, or HtmlExportSettings.ImageExport should be marked as handled.");
                }

                byte[] data = cssEncoding.GetBytes(args.Css);
                File.WriteAllBytes(Path.Combine(context.Settings.StylesFilePath), data);

                this.Reference = context.Settings.StylesSourcePath;
            }

            this.Type = HtmlConstants.TextCss;
            this.Relation = HtmlConstants.StyleSheet;
        }
    }
}
