﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Lists
{
    internal class HtmlExportListManager : HtmlListManager
    {
        private readonly List<IStyleProperty> exportedCharacterProperties = new List<IStyleProperty>();
        private int isInListCounter = 0;

        public List<IStyleProperty> ExportedCharacterProperties
        {
            get
            {
                return this.exportedCharacterProperties;
            }
        }

        public bool IsInList
        {
            get
            {
                return this.isInListCounter > 0;
            }
        }

        internal void BeginList()
        {
            this.isInListCounter++;
        }

        internal void EndList()
        {
            this.isInListCounter--;
        }

        internal bool ShouldRestartLevel(List list, int listLevelIndex)
        {
            Guard.ThrowExceptionIfNull(list, "list");

            int restartAfterLevel = list.Levels[listLevelIndex].RestartAfterLevel;
            if (restartAfterLevel != DocumentDefaultStyleSettings.RestartAfterLevel)
            {
                int restartAfterLevelSerialIndex = this.ListIndexes.TotalListItemsPerLevel[restartAfterLevel].VerticalDepthSerialIndex;
                if (this.ListIndexes.TotalListItemsPerLevel.ContainsKey(listLevelIndex))
                {
                    int levelSerialIndex = this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].VerticalDepthSerialIndex;
                    if (restartAfterLevelSerialIndex > levelSerialIndex)
                    {
                        return true;
                    }
                }
            }
            else
            {
                // Needed for continue list feature.
                if (listLevelIndex == 0 && this.ListIndexes.TotalListItemsPerLevel.ContainsKey(listLevelIndex) &&
                    this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].VerticalDepthSerialIndex > 0)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }

            return false;
        }

        internal void UpdateListIndexes(List list, int listLevelIndex, bool shouldRestartLevel)
        {
            Guard.ThrowExceptionIfNull(list, "list");
            Guard.ThrowExceptionIfLessThan<int>(DocumentDefaultStyleSettings.ListLevel, listLevelIndex, "listLevelIndex");

            this.ListIndexes.TotalNumberOfItems++;

            this.ListIndexes.AddLevelIndex(listLevelIndex);

            if (shouldRestartLevel)
            {
                this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].BulletSerialIndex = list.Levels[listLevelIndex].StartIndex;
            }
            else
            {
                this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].BulletSerialIndex =
                    this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].BulletSerialIndex + 1;
            }

            this.ListIndexes.TotalListItemsPerLevel[listLevelIndex].VerticalDepthSerialIndex = this.ListIndexes.TotalNumberOfItems;
        }
    }
}