﻿using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Lists
{
    internal class HtmlImportListManager : HtmlListManager
    {
        internal List List { get; set; }

        internal int? LevelIndex { get; set; }

        internal int LevelBulletSerialIndex
        {
            get
            {
                return this.ListIndexes.TotalListItemsPerLevel[this.LevelIndex.Value].BulletSerialIndex;
            }
            set
            {
                this.ListIndexes.TotalListItemsPerLevel[this.LevelIndex.Value].BulletSerialIndex = value;
            }
        }

        internal ListLevel BeginLevel(RadFlowDocument document)
        {
            Guard.ThrowExceptionIfNull(document, "document");

            if (this.List == null)
            {
                this.List = document.Lists.Add(ListTemplateType.NumberedDefault);

                for (int i = 0; i < this.List.Levels.Count; i++)
                {
                    this.List.Levels[i].NumberingStyle = NumberingStyle.Decimal;
                }

                this.ListIndexes = new ListIndexes(this.List.Id);
            }

            if (!this.LevelIndex.HasValue)
            {
                this.LevelIndex = 0;
            }
            else
            {
                if (this.LevelIndex.Value < 8)
                {
                    this.LevelIndex++;
                }
            }

            return this.List.Levels[this.LevelIndex.Value];
        }

        internal ListLevel GetLevel()
        {
            return this.List.Levels[this.LevelIndex.Value];
        }

        internal void EndLevel()
        {
            if (this.LevelIndex.Value >= 0)
            {
                this.LevelIndex--;
            }

            if (this.LevelIndex.Value == -1)
            {
                this.List = null;
            }
        }

        internal void ResertAllSubsequentLevels()
        {
            for (int i = this.LevelIndex.Value + 1; i < this.List.Levels.Count; i++)
            {
                if (this.ListIndexes.TotalListItemsPerLevel.ContainsKey(i))
                {
                    this.ListIndexes.TotalListItemsPerLevel[i].BulletSerialIndex = 0;
                }
            }
        }

        internal void ClearListLevel()
        {
            // Set ListLevel's properties to default as in the BeginLevel() method.
            ListLevel listLevel = this.GetLevel();

            listLevel.NumberingStyle = NumberingStyle.Decimal;
            listLevel.NumberTextFormat = "%" + ((this.LevelIndex + 1) ?? 1) + ".";
            listLevel.CharacterProperties.FontFamily.ClearValue();
        }
    }
}
