﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters;
using Telerik.Windows.Documents.Flow.Model.Lists;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Lists
{
    internal class OrderedListElement : ListElement
    {
        private readonly HtmlAttribute<string> numberingStyleTypeAttribute;

        public OrderedListElement(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.numberingStyleTypeAttribute = this.RegisterAttribute<string>("type");
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.OrderedList;
            }
        }

        protected override void ApplyAttributesOnListLevelAfterRead(IHtmlImportContext context, ListLevel listLevel)
        {
            if (this.numberingStyleTypeAttribute.HasValue)
            {
                NumberingStyle convertedNumberingStyle;
                if (HtmlValueMappers.NumberingStyleMapper.TryGetToValue(this.numberingStyleTypeAttribute.Value, out convertedNumberingStyle))
                {
                    if (convertedNumberingStyle != NumberingStyle.Bullet)
                    {
                        listLevel.NumberingStyle = convertedNumberingStyle;
                    }
                    else
                    {
                        // The element is <ol>, but in type attribute is specified value for <ul> element.
                        BulletListInfo bulletListInfo = this.ContentManager.ImportListManager.GetBulletListInfoByHtmlName(this.numberingStyleTypeAttribute.Value);
                        if (bulletListInfo != null)
                        {
                            listLevel.NumberingStyle = NumberingStyle.Bullet;
                            listLevel.NumberTextFormat = bulletListInfo.BulletSymbol;
                            listLevel.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(bulletListInfo.SymbolFontFamily);
                        }
                    }
                }
            }
        }
    }
}