﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Converters;
using Telerik.Windows.Documents.Flow.Model.Lists;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Lists
{
    internal class UnorderedListElement : ListElement
    {
        private readonly HtmlAttribute<string> numberingStyleTypeAttribute;

        public UnorderedListElement(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.numberingStyleTypeAttribute = this.RegisterAttribute<string>("type");
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.UnorderedList;
            }
        }

        protected override void ApplyAttributesOnListLevelAfterRead(IHtmlImportContext context, ListLevel listLevel)
        {
            listLevel.NumberingStyle = NumberingStyle.Bullet;
            listLevel.NumberTextFormat = ListFactory.BulletSymbols[0];
            listLevel.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ListFactory.BulletsFontFamily[0]);

            if (this.numberingStyleTypeAttribute.HasValue)
            {
                BulletListInfo bulletListInfo = this.ContentManager.ImportListManager.GetBulletListInfoByHtmlName(this.numberingStyleTypeAttribute.Value);
                if (bulletListInfo != null)
                {
                    listLevel.NumberTextFormat = bulletListInfo.BulletSymbol;
                    listLevel.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(bulletListInfo.SymbolFontFamily);
                }
                else
                {
                    // The element is <ul>, but in type attribute is specified value for <ol> element.
                    this.ContentManager.ImportListManager.ClearListLevel();

                    NumberingStyle convertedNumberingStyle;
                    if (HtmlValueMappers.NumberingStyleMapper.TryGetToValue(this.numberingStyleTypeAttribute.Value, out convertedNumberingStyle))
                    {
                        if (convertedNumberingStyle != NumberingStyle.Bullet)
                        {
                            listLevel.NumberingStyle = convertedNumberingStyle;
                        }
                    }
                }
            }
        }
    }
}