﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements
{
    internal class StyleElement : HtmlElementBase
    {
        private readonly HtmlAttribute<string> typeAttribute;

        public StyleElement(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.typeAttribute = this.RegisterAttribute<string>("type");
        }

        public string Type
        {
            get
            {
                return this.typeAttribute.Value;
            }
            set
            {
                this.typeAttribute.Value = value;
            }
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.Style;
            }
        }

        protected override void OnBeforeWrite(IHtmlWriter writer, IHtmlExportContext context)
        {
            this.Type = HtmlConstants.TextCss;

            this.InnerText = StyleExporter.Export(context);
        }

        protected override void OnBeforeRead(IHtmlReader reader, IHtmlImportContext context)
        {
            base.OnBeforeRead(reader, context);

            this.InnerText = string.Empty;
        }

        protected override void OnAfterReadChildElement(IHtmlReader reader, IHtmlImportContext context, HtmlElementBase innerElement)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(innerElement, "innerElement");

            if (innerElement is TextElement)
            {
                this.InnerText += innerElement.InnerText;
            }
        }
    }
}
