﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Tables
{
    internal abstract class TableCellElementBase : BlockContainerElementBase
    {
        private readonly HtmlAttribute<int> columnSpan;
        private readonly HtmlAttribute<int> rowSpan;

        public TableCellElementBase(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.columnSpan = this.RegisterAttribute<int>("colspan");
            this.rowSpan = this.RegisterAttribute<int>("rowspan");
            this.RegisterAttribute(new StyleValueAttribute("bgcolor", "background-color", this.Style));
            this.RegisterAttribute(new StyleValueAttribute("height", this.Style));
            this.RegisterAttribute(new StyleValueAttribute("width", this.Style));
            this.RegisterAttribute(new StyleValueAttribute("valign", "vertical-align", this.Style));
        }

        protected TableCell AssociatedTableCell
        {
            get
            {
                return this.AssociatedBlockContainer as TableCell;
            }
        }

        protected abstract CellType CellType { get; }

        protected override void OnAfterReadAttributes(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            TableCell cell = new TableCell(context.Document);
            this.SetAssociatedFlowElement(cell);
            this.CopyLocalPropertiesTo(context, cell);

            if (this.columnSpan.HasValue)
            {
                cell.ColumnSpan = this.columnSpan.Value;
            }

            if (this.rowSpan.HasValue)
            {
                cell.RowSpan = this.rowSpan.Value;
            }

            context.InsertTableCell(cell, this.CellType);
        }

        protected override void EvaluatePropertiesAfterReadAttributes(IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.RuleOutBubblingProperties(context, this.AssociatedTableCell);
        }

        protected override void OnAfterRead(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            context.PopBlockContainer();
        }

        protected override void OnBeforeWrite(IHtmlWriter writer, Export.IHtmlExportContext context)
        {
            TableCell cell = this.AssociatedTableCell;
            Guard.ThrowExceptionIfFalse(cell != null, "No associated table cell.");

            if (context.Settings.StylesExportMode == StylesExportMode.External || context.Settings.StylesExportMode == StylesExportMode.Embedded)
            {
                if (cell.Parent != null && cell.Parent.Parent != null)
                {
                    string styleId = (cell.Parent.Parent as IElementWithStyle).StyleId;
                    if (!string.IsNullOrEmpty(styleId))
                    {
                        string convertedStyleId = StyleNamesConverter.ConvertStyleNameOnExport(styleId);
                        if (context.HtmlStyleRepository.ContainsStyle(convertedStyleId, HtmlElementNames.TableCell))
                        {
                            this.CopyStylePropertiesFrom(context, styleId);
                        }
                    }
                }
            }

            this.CopyLocalPropertiesFrom(context, cell);

            this.RemoveLocalPropertiesAlreadyInAppliedStyles(context, HtmlElementNames.TableCell);

            if (cell.ColumnSpan != 1)
            {
                this.columnSpan.Value = cell.ColumnSpan;
            }
            if (cell.RowSpan != 1)
            {
                this.rowSpan.Value = cell.RowSpan;
            }
        }
    }
}
