﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Tables
{
    internal class TableRowElement : BodyElementBase
    {
        private TableRow row;

        private RowType rowType;

        public TableRowElement(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.rowType = RowType.Body;
            this.RegisterAttribute(new StyleValueAttribute("height", this.Style));
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.TableRow;
            }
        }

        public void SetAssociatedFlowElement(TableRow row)
        {
            Guard.ThrowExceptionIfNull(row, "row");

            this.row = row;
        }

        public void SetRowType(RowType rowType)
        {
            this.rowType = rowType;
        }

        protected override void OnAfterReadAttributes(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            TableRow row = new TableRow(context.Document);
            this.SetAssociatedFlowElement(row);
            this.CopyLocalPropertiesTo(context, row);
            context.PushTableRow(row, this.rowType);
        }

        protected override IEnumerable<HtmlElementBase> OnEnumerateInnerElements(IHtmlWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (TableCell cell in this.row.Cells)
            {
                TableCellElement cellElement = this.CreateElement<TableCellElement>(HtmlElementNames.TableCell);
                cellElement.SetAssociatedFlowElement(cell);

                yield return cellElement;
            }
        }

        protected override void OnAfterRead(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            this.BubblingProperties.CopyTo(context, this.row);

            this.BubblingProperties.RemoveProperty(HtmlStylePropertyDescriptors.RowHeightPropertyDescriptor.Name);

            context.PopTableRow();
        }

        protected override void OnBeforeWrite(IHtmlWriter writer, IHtmlExportContext context)
        {
            base.OnBeforeWrite(writer, context);

            this.CopyLocalPropertiesFrom(context, this.row);
        }

        protected override void ClearOverride()
        {
            base.ClearOverride();

            this.rowType = RowType.Body;
            this.row = null;
        }
    }
}