﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Text
{
    internal class ParagraphElement : ParagraphElementBase
    {
        public ParagraphElement(HtmlContentManager contentManager)
            : base(contentManager)
        {
            this.RegisterAttribute(new StyleValueAttribute("dir", "direction", this.Style));
        }

        public override string Name
        {
            get
            {
                return HtmlElementNames.Paragraph;
            }
        }

        protected override void OnAfterReadAttributes(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            if (!context.ListImportContext.IsInListLevel ||
                context.ListImportContext.CurrentListLevelContentImporter.ShouldCreateParagraph)
            {
                Paragraph paragraph = new Paragraph(context.Document);
                this.SetAssociatedFlowElement(paragraph);

                this.ApplyStyle(context, paragraph);
                this.CopyLocalPropertiesTo(context, paragraph);
                this.CopyLocalPropertiesTo(context, paragraph.Properties.ParagraphMarkerProperties);

                context.BeginParagraph(paragraph);
            }
        }

        protected override void OnAfterRead(IHtmlReader reader, IHtmlImportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            this.ApplyListLevelIndentationOnCurrentParagraphIfNecessary(context);

            context.EndParagraph();
        }
    }
}