﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Attributes;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Elements.Text
{
    internal class TextBasedElement : BodyElementBase, IPhrasingElement
    {
        private readonly string name;

        private Run run;

        public TextBasedElement(HtmlContentManager contentManager, string name)
            : base(contentManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            this.name = name;

            this.RegisterAttribute(new StyleValueAttribute("dir", "direction", this.Style));
        }

        public override string Name
        {
            get
            {
                return this.name;
            }
        }

        protected override bool CanHaveStyle
        {
            get
            {
                return true;
            }
        }

        protected override bool PreserveWhiteSpaces
        {
            get
            {
                return true;
            }
        }

        public void SetAssociatedFlowElement(Run run)
        {
            Guard.ThrowExceptionIfNull(run, "run");

            this.run = run;
        }

        protected override void PrepareRunOverride(IHtmlImportContext context, Run run)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(run, "run");

            this.ApplyStyle(context, run);
            this.CopyLocalPropertiesTo(context, run);
        }

        protected override void OnBeforeWrite(IHtmlWriter reader, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(reader, "reader");
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.ContentManager.ExportListManager.IsInList)
            {
                foreach (IStyleProperty exportedProperty in this.ContentManager.ExportListManager.ExportedCharacterProperties)
                {
                    IStyleProperty localProperty = this.run.Properties.GetStyleProperty(exportedProperty.PropertyDefinition);
                    if (localProperty.GetActualValueAsObject() != exportedProperty.GetActualValueAsObject())
                    {
                        this.LocalProperties.CopyFrom(context, localProperty, this.run.Properties, false);
                    }
                }
            }

            this.CopyStyleFrom(context, this.run);
            this.CopyLocalPropertiesFrom(context, this.run);

            this.InnerText = this.run.Text;
        }

        protected override void ClearOverride()
        {
            base.ClearOverride();

            this.run = null;
        }
    }
}
