﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal class HtmlBubblingProperties
    {
        private readonly Dictionary<string, List<object>> bubblingProperties;

        public HtmlBubblingProperties()
        {
            this.bubblingProperties = new Dictionary<string, List<object>>();
        }

        public void RegisterProperty(DocumentElementPropertiesBase properties, HtmlStyleProperty htmlStyleProperty, object value)
        {
            this.RegisterProperty(properties, htmlStyleProperty.Name, value);
        }

        public void RegisterProperty(DocumentElementPropertiesBase properties, string propertyName, object value)
        {
            if (!HtmlBubblingProperties.ShouldPropertyBeRegistered(properties, propertyName))
            {
                return;
            }

            this.RegisterProperty(propertyName, value);
        }

        public void RemoveProperty(string name)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            this.bubblingProperties.Remove(name);
        }

        public void CopyFrom(HtmlBubblingProperties bubblingProperties)
        {
            foreach (KeyValuePair<string, List<object>> pair in bubblingProperties.bubblingProperties)
            {
                foreach (object value in pair.Value)
                {
                    this.RegisterProperty(pair.Key, value);
                }
            }
        }

        public void CopyTo(IHtmlImportContext context, IElementWithProperties element)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(element, "element");

            this.CopyTo(context, element.Properties);
        }

        public void CopyTo(IHtmlImportContext context, DocumentElementPropertiesBase properties)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");

            foreach (KeyValuePair<string, List<object>> pair in this.bubblingProperties)
            {
                foreach (HtmlBubblingPropertyRule rule in HtmlBubblingPropertyRules.TryGetPropertyRule(pair.Key))
                {
                    Flow.Model.Styles.Core.IStyleProperty styleProperty = properties.GetStyleProperty(rule.PropertyDefinition);
                    if (styleProperty != null)
                    {
                        rule.SetMostSuitableCandidateValue(properties, pair.Value);
                    }
                }
            }
        }

        public void Clear()
        {
            this.bubblingProperties.Clear();
        }

        public void RuleOutProperties(IHtmlImportContext context, DocumentElementPropertiesBase properties)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(properties, "properties");

            foreach (string propertyName in this.bubblingProperties.Keys.ToArray())
            {
                if (!HtmlBubblingProperties.ShouldPropertyBeRegistered(properties, propertyName))
                {
                    this.RemoveProperty(propertyName);
                }
            }
        }

        private static bool ShouldPropertyBeRegistered(DocumentElementPropertiesBase properties, string propertyName)
        {
            foreach (HtmlBubblingPropertyRule rule in HtmlBubblingPropertyRules.TryGetPropertyRule(propertyName))
            {
                if (!rule.ShouldRegisterProperty(properties))
                {
                    return false;
                }
            }

            return true;
        }

        private void RegisterProperty(string propertyName, object value)
        {
            if (!this.bubblingProperties.ContainsKey(propertyName))
            {
                this.bubblingProperties.Add(propertyName, new List<object>());
            }

            this.bubblingProperties[propertyName].Add(value);
        }
    }
}
