﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal abstract class HtmlBubblingPropertyRule
    {
        private readonly IStylePropertyDefinition propertyDefinition;

        public HtmlBubblingPropertyRule(IStylePropertyDefinition propertyDefinition)
        {
            Guard.ThrowExceptionIfNull(propertyDefinition, "propertyDefinition");

            this.propertyDefinition = propertyDefinition;
        }

        public IStylePropertyDefinition PropertyDefinition
        {
            get
            {
                return this.propertyDefinition;
            }
        }

        public void SetMostSuitableCandidateValue(DocumentElementPropertiesBase properties, IEnumerable<object> candidateValues)
        {
            Guard.ThrowExceptionIfNull(properties, "properties");
            Guard.ThrowExceptionIfNull(candidateValues, "candidateValues");

            IStyleProperty styleProperty = properties.GetStyleProperty(this.PropertyDefinition);
            if (styleProperty != null)
            {
                object value = styleProperty.GetActualValueAsObject();

                foreach (object candidateValue in candidateValues)
                {
                    object result;
                    if (this.IsRuleSatisfied(value, candidateValue, out result))
                    {
                        value = result;
                    }
                }

                styleProperty.SetValueAsObject(value);
            }
        }

        public virtual bool ShouldRegisterProperty(DocumentElementPropertiesBase properties)
        {
            return true;
        }

        protected abstract bool IsRuleSatisfied(object value, object candidateValue, out object result);
    }
}
