﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.BubblingRules;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal static class HtmlBubblingPropertyRules
    {
        private static readonly Dictionary<string, List<HtmlBubblingPropertyRule>> htmlPropertiesToBubblingRules = new Dictionary<string, List<HtmlBubblingPropertyRule>>();

        static HtmlBubblingPropertyRules()
        {
            InitializeRules();
        }

        internal static IEnumerable<HtmlBubblingPropertyRule> TryGetPropertyRule(string propertyName)
        {
            if (htmlPropertiesToBubblingRules.ContainsKey(propertyName))
            {
                foreach (HtmlBubblingPropertyRule rule in htmlPropertiesToBubblingRules[propertyName])
                {
                    yield return rule;
                }
            }

            yield break;
        }

        private static void InitializeRules()
        {
            AddRule(HtmlStylePropertyDescriptors.RowHeightPropertyDescriptor.Name, new TableRowHeightBubblingRule(TableRow.HeightPropertyDefinition));
        }

        private static void AddRule(string propertyName, TableRowHeightBubblingRule bubblingRule)
        {
            if (!htmlPropertiesToBubblingRules.ContainsKey(propertyName))
            {
                htmlPropertiesToBubblingRules.Add(propertyName, new List<HtmlBubblingPropertyRule>());
            }

            htmlPropertiesToBubblingRules[propertyName].Add(bubblingRule);
        }
    }
}
