﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal static class HtmlStylePropertyDescriptors
    {
        public static readonly HtmlStylePropertyDescriptor ColorPropertyDescriptor = new HtmlStylePropertyDescriptor("color", true);
        public static readonly HtmlStylePropertyDescriptor FontSizePropertyDescriptor = new HtmlStylePropertyDescriptor("font-size", true, applyAsLocalValueIfRelative: true);
        public static readonly HtmlStylePropertyDescriptor FontFamilyPropertyDescriptor = new HtmlStylePropertyDescriptor("font-family", true);
        public static readonly HtmlStylePropertyDescriptor TextAlignPropertyDescriptor = new HtmlStylePropertyDescriptor("text-align", true);
        public static readonly HtmlStylePropertyDescriptor FontWeightPropertyDescriptor = new HtmlStylePropertyDescriptor("font-weight", true);
        public static readonly HtmlStylePropertyDescriptor FontStylePropertyDescriptor = new HtmlStylePropertyDescriptor("font-style", true);
        public static readonly HtmlStylePropertyDescriptor TextDecorationPropertyDescriptor = new HtmlStylePropertyDescriptor("text-decoration", true);
        public static readonly HtmlStylePropertyDescriptor BackgroundColorPropertyDescriptor = new HtmlStylePropertyDescriptor("background-color", false);
        public static readonly HtmlStylePropertyDescriptor VerticalAlignmentPropertyDescriptor = new HtmlStylePropertyDescriptor("vertical-align", true);

        public static readonly HtmlStylePropertyDescriptor MarginLeftPropertyDescriptor = new HtmlStylePropertyDescriptor("margin-left", false);
        public static readonly HtmlStylePropertyDescriptor MarginTopPropertyDescriptor = new HtmlStylePropertyDescriptor("margin-top", false);
        public static readonly HtmlStylePropertyDescriptor MarginRightPropertyDescriptor = new HtmlStylePropertyDescriptor("margin-right", false);
        public static readonly HtmlStylePropertyDescriptor MarginBottomPropertyDescriptor = new HtmlStylePropertyDescriptor("margin-bottom", false);
        public static readonly HtmlStylePropertyDescriptor TextIndentPropertyDescriptor = new HtmlStylePropertyDescriptor("text-indent", true);
        public static readonly HtmlStylePropertyDescriptor LineHeightPropertyDescriptor = new HtmlStylePropertyDescriptor("line-height", true);
        public static readonly HtmlStylePropertyDescriptor DirectionPropertyDescriptor = new HtmlStylePropertyDescriptor("direction", true);

        public static readonly HtmlStylePropertyDescriptor WidthPropertyDescriptor = new HtmlStylePropertyDescriptor("width", false);
        public static readonly HtmlStylePropertyDescriptor BorderSpacingPropertyDescriptor = new HtmlStylePropertyDescriptor("border-spacing", true);
        public static readonly HtmlStylePropertyDescriptor BorderCollapsePropertyDescriptor = new HtmlStylePropertyDescriptor("border-collapse", true);
        public static readonly HtmlStylePropertyDescriptor BorderColorPropertyDescriptor = new HtmlStylePropertyDescriptor("border-color", false);
        public static readonly HtmlStylePropertyDescriptor BorderPropertyDescriptor = new HtmlStylePropertyDescriptor("border", false);
        public static readonly HtmlStylePropertyDescriptor BorderLeftPropertyDescriptor = new HtmlStylePropertyDescriptor("border-left", false);
        public static readonly HtmlStylePropertyDescriptor BorderTopPropertyDescriptor = new HtmlStylePropertyDescriptor("border-top", false);
        public static readonly HtmlStylePropertyDescriptor BorderRightPropertyDescriptor = new HtmlStylePropertyDescriptor("border-right", false);
        public static readonly HtmlStylePropertyDescriptor BorderBottomPropertyDescriptor = new HtmlStylePropertyDescriptor("border-bottom", false);
        public static readonly HtmlStylePropertyDescriptor TableLayoutPropertyDescriptor = new HtmlStylePropertyDescriptor("table-layout", false);

        public static readonly HtmlStylePropertyDescriptor RowHeightPropertyDescriptor = new HtmlStylePropertyDescriptor("height", false, isBubblingInheritable: true);

        public static readonly HtmlStylePropertyDescriptor PaddingPropertyDescriptor = new HtmlStylePropertyDescriptor("padding", false);
        public static readonly HtmlStylePropertyDescriptor PaddingLeftPropertyDescriptor = new HtmlStylePropertyDescriptor("padding-left", false);
        public static readonly HtmlStylePropertyDescriptor PaddingRightPropertyDescriptor = new HtmlStylePropertyDescriptor("padding-right", false);
        public static readonly HtmlStylePropertyDescriptor PaddingTopPropertyDescriptor = new HtmlStylePropertyDescriptor("padding-top", false);
        public static readonly HtmlStylePropertyDescriptor PaddingBottomPropertyDescriptor = new HtmlStylePropertyDescriptor("padding-bottom", false);

        public static readonly HtmlStylePropertyDescriptor ListStyleTypeDescriptor = new HtmlStylePropertyDescriptor("list-style-type", true);

        private static readonly Dictionary<string, HtmlStylePropertyDescriptor> propertyDescriptors;

        static HtmlStylePropertyDescriptors()
        {
            propertyDescriptors = new Dictionary<string, HtmlStylePropertyDescriptor>();

            RegisterPropertyDescriptor(ColorPropertyDescriptor);
            RegisterPropertyDescriptor(FontSizePropertyDescriptor);
            RegisterPropertyDescriptor(FontFamilyPropertyDescriptor);
            RegisterPropertyDescriptor(TextAlignPropertyDescriptor);
            RegisterPropertyDescriptor(FontWeightPropertyDescriptor);
            RegisterPropertyDescriptor(FontStylePropertyDescriptor);
            RegisterPropertyDescriptor(TextDecorationPropertyDescriptor);
            RegisterPropertyDescriptor(BackgroundColorPropertyDescriptor);
            RegisterPropertyDescriptor(VerticalAlignmentPropertyDescriptor);

            RegisterPropertyDescriptor(MarginLeftPropertyDescriptor);
            RegisterPropertyDescriptor(MarginTopPropertyDescriptor);
            RegisterPropertyDescriptor(MarginRightPropertyDescriptor);
            RegisterPropertyDescriptor(MarginBottomPropertyDescriptor);
            RegisterPropertyDescriptor(DirectionPropertyDescriptor);
            RegisterPropertyDescriptor(TextIndentPropertyDescriptor);
            RegisterPropertyDescriptor(LineHeightPropertyDescriptor);

            RegisterPropertyDescriptor(WidthPropertyDescriptor);
            RegisterPropertyDescriptor(BorderSpacingPropertyDescriptor);
            RegisterPropertyDescriptor(BorderCollapsePropertyDescriptor);
            RegisterPropertyDescriptor(BorderPropertyDescriptor);
            RegisterPropertyDescriptor(BorderLeftPropertyDescriptor);
            RegisterPropertyDescriptor(BorderTopPropertyDescriptor);
            RegisterPropertyDescriptor(BorderRightPropertyDescriptor);
            RegisterPropertyDescriptor(BorderBottomPropertyDescriptor);
            RegisterPropertyDescriptor(BorderColorPropertyDescriptor);
            RegisterPropertyDescriptor(TableLayoutPropertyDescriptor);

            RegisterPropertyDescriptor(RowHeightPropertyDescriptor);

            RegisterPropertyDescriptor(PaddingPropertyDescriptor);
            RegisterPropertyDescriptor(PaddingLeftPropertyDescriptor);
            RegisterPropertyDescriptor(PaddingRightPropertyDescriptor);
            RegisterPropertyDescriptor(PaddingTopPropertyDescriptor);
            RegisterPropertyDescriptor(PaddingBottomPropertyDescriptor);

            RegisterPropertyDescriptor(ListStyleTypeDescriptor);
        }

        public static bool TryGetPropertyDescriptor(string propertyName, out HtmlStylePropertyDescriptor result)
        {
            return propertyDescriptors.TryGetValue(propertyName, out result);
        }

        private static void RegisterPropertyDescriptor(HtmlStylePropertyDescriptor descriptor)
        {
            Guard.ThrowExceptionIfNull(descriptor, "descriptor");

            propertyDescriptors[descriptor.Name] = descriptor;
        }
    }
}
