﻿using System;
using System.Globalization;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import.Parser;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    // Immutable!
    internal class HtmlStylePropertyValue
    {
        private readonly string value;

        private bool isParsed;
        private double valueAsDouble;
        private UnitType? unitType;

        public HtmlStylePropertyValue(string value)
        {
            this.value = value;
        }

        public string Value
        {
            get
            {
                return this.value;
            }
        }

        public double ValueAsDouble
        {
            get
            {
                this.AssureValuesAreParsed();
                return this.valueAsDouble;
            }
        }

        public int ValueAsInt
        {
            get
            {
                this.AssureValuesAreParsed();
                return (int)Math.Round(this.valueAsDouble);
            }
        }

        public UnitType? UnitType
        {
            get
            {
                this.AssureValuesAreParsed();
                return this.unitType;
            }
        }

        private void AssureValuesAreParsed()
        {
            if (!this.isParsed)
            {
                string parsedValue;
                double parsedValueAsDouble = 0.0;
                UnitType? parsedUnitType = null;

                HtmlUnitParser.Parse(this.Value, out parsedValue, out parsedUnitType);

                if (double.TryParse(parsedValue, NumberStyles.Number, CultureInfo.InvariantCulture, out parsedValueAsDouble))
                {
                    this.valueAsDouble = parsedValueAsDouble;
                }
                else
                {
                    this.valueAsDouble = double.NaN;
                }

                this.unitType = parsedUnitType;
                this.isParsed = true;
            }
        }
    }
}
