using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal class HtmlStylePropertyValues
    {
        private readonly string unparesedValue;
        private readonly List<HtmlStylePropertyValue> values;
        private readonly bool hasRelativeValues;

        public HtmlStylePropertyValues(string value)
        {
            this.unparesedValue = value;
            this.hasRelativeValues = false;
            string[] splitValues = value.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

            this.values = new List<HtmlStylePropertyValue>(splitValues.Length);
            for (int i = 0; i < splitValues.Length; i++)
            {
                HtmlStylePropertyValue propertyValue = new HtmlStylePropertyValue(splitValues[i]);
                if (propertyValue.UnitType.HasValue && Unit.IsRelativeUnitType(propertyValue.UnitType.Value))
                {
                    this.hasRelativeValues = true;
                }

                this.values.Add(propertyValue);
            }

            if (this.values.Count == 0)
            {
                // Guarantee at least one value;
                this.values.Add(new HtmlStylePropertyValue(string.Empty));
            }
        }

        public bool HasRelativeValues
        {
            get
            {
                return this.hasRelativeValues;
            }
        }

        public string UnparsedValue
        {
            get
            {
                return this.unparesedValue;
            }
        }

        public int Count
        {
            get
            {
                return this.values.Count;
            }
        }

        public HtmlStylePropertyValue this[int index]
        {
            get
            {
                return this.values[index];
            }
        }
    }
}