﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal class HtmlStyleRepository
    {
        private readonly Dictionary<string, List<Selector>> styles;

        public HtmlStyleRepository()
        {
            this.styles = new Dictionary<string, List<Selector>>();
        }

        public Selector DefaultParagraphStyle
        {
            get;
            set;
        }

        public Selector DefaultCharacterStyle
        {
            get;
            set;
        }

        public void RegisterStyle(string styleId, Selector style)
        {
            Guard.ThrowExceptionIfNullOrEmpty(styleId, "styleId");
            Guard.ThrowExceptionIfNull(style, "style");

            if (!this.styles.ContainsKey(styleId))
            {
                this.styles.Add(styleId, new List<Selector>());
            }

            this.styles[styleId].Add(style);
        }

        public bool ContainsStyle(string styleId, string elementType)
        {
            if (!this.styles.ContainsKey(styleId))
            {
                return false;
            }

            return this.styles[styleId].Where(s => s.ForElementType == elementType).FirstOrDefault() != null;
        }

        public bool ContainsStyle(string styleId)
        {
            if (!this.styles.ContainsKey(styleId))
            {
                return false;
            }

            return this.styles[styleId].Any();
        }

        public Selector GetStyle(string styleId)
        {
            Guard.ThrowExceptionIfNullOrEmpty(styleId, "styleId");

            return this.styles[styleId].FirstOrDefault();
        }

        public Selector GetStyle(string styleId, string elementType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(styleId, "styleId");

            return this.styles[styleId].Where(s => s.ForElementType == elementType).FirstOrDefault();
        }

        public bool TryGetStyle(string styleId, out Selector selector)
        {
            Guard.ThrowExceptionIfNullOrEmpty(styleId, "styleId");

            if (this.styles.ContainsKey(styleId))
            {
                selector = this.styles[styleId].FirstOrDefault();
                return true;
            }

            selector = null;
            return false;
        }

        public IEnumerable<Selector> GetStyles()
        {
            foreach (List<Selector> list in this.styles.Values)
            {
                foreach (Selector style in list)
                {
                    yield return style;
                }
            }
        }
    }
}
