﻿using Telerik.Windows.Documents.Flow.FormatProviders.Html.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html.Import;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html.Model.Styles
{
    internal class Selector : HtmlStyleProperties
    {
        private readonly string name;
        private readonly string forElementType;
        private readonly SelectorType type;

        public Selector(string name, SelectorType type)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            this.name = name;
            this.type = type;
        }

        public Selector(string name, SelectorType type, string forElementType)
            : this(name, type)
        {
            Guard.ThrowExceptionIfNullOrEmpty(forElementType, "forElementsofType");

            this.forElementType = forElementType;
        }

        public SelectorType Type
        {
            get
            {
                return this.type;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public string ForElementType
        {
            get
            {
                return this.forElementType;
            }
        }

        public void CopyFrom(IHtmlExportContext context, Style style)
        {
            if (string.IsNullOrEmpty(style.BasedOnStyleId))
            {
                this.CopyFromIncludingNonHtmlDefaultActualValues(context, style);
            }
            else
            {
                this.CopyFromInternal(context, style);
            }
        }

        public void CopyTo(IHtmlImportContext context, Style style)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(style, "style");

            this.CopyTo(context, style.CharacterProperties);
            this.CopyTo(context, style.ParagraphProperties);
            this.CopyTo(context, style.TableCellProperties);
            this.CopyTo(context, style.TableProperties);
            this.CopyTo(context, style.TableRowProperties);
        }

        public void Write(CssStyleWriter writer, IHtmlExportContext context)
        {
            Guard.ThrowExceptionIfNull(writer, "writer");
            Guard.ThrowExceptionIfNull(context, "context");

            writer.WriteStyleStart(this.Name, this.Type, this.ForElementType);
            this.Write(writer);
            writer.WriteStyleEnd();
        }

        private void CopyFromInternal(IHtmlExportContext context, Style style)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(style, "style");

            this.CopyFrom(context, style.CharacterProperties);
            this.CopyFrom(context, style.ParagraphProperties);
            this.CopyFrom(context, style.TableCellProperties);
            this.CopyFrom(context, style.TableProperties);
            this.CopyFrom(context, style.TableRowProperties);
        }

        private void CopyFromIncludingNonHtmlDefaultActualValues(IHtmlExportContext context, Style style)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(style, "style");

            this.CopyFromIncludingNonHtmlDefaultActualValues(context, context.Document.DefaultStyle.CharacterProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, context.Document.DefaultStyle.ParagraphProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, style.CharacterProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, style.ParagraphProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, style.TableCellProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, style.TableProperties);
            this.CopyFromIncludingNonHtmlDefaultActualValues(context, style.TableRowProperties);
        }
    }
}
