﻿using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Html
{
    internal static class StyleNamesConverter
    {
        public static readonly string PrefixedNormalStyleId = ConvertStyleNameOnExport(BuiltInStyleNames.NormalStyleId);
        public static readonly string PrefixedNormalTableStyleId = ConvertStyleNameOnExport(BuiltInStyleNames.NormalTableStyleId);
        public static readonly string PrefixedNormalWebStyleId = ConvertStyleNameOnExport(BuiltInStyleNames.NormalWebStyleId);

        private const string BuiltInStyleNamesPrefix = "Telerik";

        public static string ConvertStyleNameOnExport(string styleId)
        {
            if (BuiltInStyles.IsBuiltInStyle(styleId))
            {
                return BuiltInStyleNamesPrefix + styleId;
            }

            return styleId;
        }

        public static bool TryGetTelerikExportedBuiltInStyle(string styleId, out string originalStyleId)
        {
            Guard.ThrowExceptionIfNullOrEmpty(styleId, "styleId");

            if (styleId.StartsWith(BuiltInStyleNamesPrefix))
            {
                originalStyleId = styleId.Substring(BuiltInStyleNamesPrefix.Length);
                if (BuiltInStyles.IsBuiltInStyle(originalStyleId))
                {
                    return true;
                }
            }

            originalStyleId = null;
            return false;
        }

        public static bool IsTelerikExportedDefaultBuiltInStyle(string styleId)
        {
            string originalStyleId;
            if (TryGetTelerikExportedBuiltInStyle(styleId, out originalStyleId))
            {
                if (originalStyleId == BuiltInStyleNames.NormalStyleId ||
                    originalStyleId == BuiltInStyleNames.NormalTableStyleId)
                {
                    return true;
                }
            }

            return false;
        }
    }
}