﻿using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Utils;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export
{
    internal class PdfExportContext
    {
        private readonly DefaultValues defaultValues;
        private readonly RadFlowDocument document;
        private readonly PdfExportSettings settings;
        private readonly Dictionary<ImageSource, Fixed.Model.Resources.ImageSource> imageSourceMapping;

        public PdfExportContext(RadFlowDocument document, PdfExportSettings settings)
        {
            Guard.ThrowExceptionIfNull(document, "document");
            Guard.ThrowExceptionIfNull(settings, "settings");

            this.document = document;
            this.settings = settings;
            this.defaultValues = new DefaultValues(document);

            this.imageSourceMapping = new Dictionary<ImageSource, Fixed.Model.Resources.ImageSource>();
        }

        public PdfExportSettings Settings
        {
            get
            {
                return this.settings;
            }
        }

        public RadFlowDocument Document
        {
            get
            {
                return this.document;
            }
        }

        public DefaultValues DefaultValues
        {
            get
            {
                return this.defaultValues;
            }
        }

        public Fixed.Model.Resources.ImageSource GetImageSource(ImageSource imageSource)
        {
            Guard.ThrowExceptionIfNull(imageSource, "imageSource");

            Fixed.Model.Resources.ImageSource result;
            if (!this.imageSourceMapping.TryGetValue(imageSource, out result))
            {
                using (MemoryStream memoryStream = new MemoryStream(imageSource.Data))
                {
                    result = new Fixed.Model.Resources.ImageSource(memoryStream);
                    this.imageSourceMapping[imageSource] = result;
                }
            }

            return result;
        }
    }
}
