﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Pdf
{
    /// <summary>
    /// Represents PDF format provider.
    /// </summary>
    public class PdfFormatProvider : BinaryFormatProviderBase<RadFlowDocument>
    {
        private static readonly IEnumerable<string> supportedExtensions = new string[] { ".pdf" };

        /// <summary>
        /// Initializes a new instance of the <see cref="PdfFormatProvider" /> class.
        /// </summary>
        public PdfFormatProvider()
        {
            this.ExportSettings = new PdfExportSettings();
        }

        /// <summary>
        /// Gets or sets the export settings.
        /// </summary>
        /// <value>The export settings.</value>
        public PdfExportSettings ExportSettings { get; set; }

        /// <summary>
        /// Gets a value indicating whether format provider can export.
        /// </summary>
        /// <value>The value indicating whether can export.</value>
        public override bool CanExport
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Gets a value indicating whether format provider can import.
        /// </summary>
        /// <value>The value indicating whether can import.</value>
        public override bool CanImport
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the supported extensions.
        /// </summary>
        /// <value>The supported extensions.</value>
        public override System.Collections.Generic.IEnumerable<string> SupportedExtensions
        {
            get
            {
                return supportedExtensions;
            }
        }

        /// <summary>
        /// Exports to fixed document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns>The fixed document.</returns>
        public RadFixedDocument ExportToFixedDocument(RadFlowDocument document)
        {
            RadFixedDocument fixedDocument = new PdfExporter(new PdfExportContext(document, this.ExportSettings)).Export();
            return fixedDocument;
        }

        /// <summary>
        /// Exports the specified document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <param name="output">The output.</param>
        protected override void ExportOverride(RadFlowDocument document, System.IO.Stream output)
        {
            RadFixedDocument fixedDocument = new PdfExporter(new PdfExportContext(document, this.ExportSettings)).Export();
            Fixed.FormatProviders.Pdf.PdfFormatProvider provider = new Fixed.FormatProviders.Pdf.PdfFormatProvider();
            provider.ExportSettings = this.ExportSettings;
            provider.Export(fixedDocument, output);
        }
    }
}
