﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Fixed.Model.Editing.Lists;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Utils
{
    internal class ListHelper
    {
        private static readonly INumberingStyleConverter emptyConverter;
        private readonly Dictionary<Model.Lists.NumberingStyle, INumberingStyleConverter> styleToConverter;

        static ListHelper()
        {
            ListHelper.emptyConverter = new NumberingStyleConverter((number) => { return string.Empty; });
        }

        public ListHelper()
        {
            this.styleToConverter = new Dictionary<Model.Lists.NumberingStyle, INumberingStyleConverter>();

            this.RegisterNumberingStyle(Model.Lists.NumberingStyle.Decimal, new NumberingStyleConverter(BulletNumberingFormats.GetNumber));
            this.RegisterNumberingStyle(Model.Lists.NumberingStyle.LowerLetter, new NumberingStyleConverter(BulletNumberingFormats.GetLowerLetter));
            this.RegisterNumberingStyle(Model.Lists.NumberingStyle.LowerRoman, new NumberingStyleConverter(BulletNumberingFormats.GetLowerRomanNumber));
            this.RegisterNumberingStyle(Model.Lists.NumberingStyle.UpperRoman, new NumberingStyleConverter(BulletNumberingFormats.GetUpperRomanNumber));
            this.RegisterNumberingStyle(Model.Lists.NumberingStyle.UpperLetter, new NumberingStyleConverter(BulletNumberingFormats.GetUpperLetter));
        }

        public void RegisterNumberingStyle(Model.Lists.NumberingStyle numberingStyle, INumberingStyleConverter converter)
        {
            Guard.ThrowExceptionIfNull(converter, "converter");

            this.styleToConverter[numberingStyle] = converter;
        }

        public IBulletNumberingFormat CreateBulletFormat(Model.Lists.ListLevel listLevel)
        {
            Model.Lists.NumberingStyle[] numberingStyles = GetNumberingStyles(listLevel.OwnerList).ToArray();

            return new TextFormattedBullet(this, numberingStyles, listLevel.NumberTextFormat ?? string.Empty);
        }

        public string GetNumberText(Model.Lists.NumberingStyle numberingStyle, int number)
        {
            INumberingStyleConverter converter;
            if (!this.styleToConverter.TryGetValue(numberingStyle, out converter))
            {
                converter = ListHelper.emptyConverter;
            }

            return converter.ConvertNumberToText(number);
        }

        private static IEnumerable<Model.Lists.NumberingStyle> GetNumberingStyles(Model.Lists.List list)
        {
            foreach (Model.Lists.ListLevel level in list.Levels)
            {
                yield return level.NumberingStyle;
            }
        }
    }
}
