﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Common;
using Telerik.Windows.Documents.Fixed.Model.Editing.Lists;
using Telerik.Windows.Documents.Fixed.Model.Text;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Utils
{
    internal class TextFormattedBullet : IBulletNumberingFormat
    {
        private const char Percent = '%';
        private const char LeftBracket = '{';
        private const char RightBracket = '}';
        private static readonly int leftBracketIndex = levelsCount;
        private static readonly int rightBracketIndex = levelsCount + 1;
        private static readonly int levelsCount = FixedDocumentDefaults.DefaultListLevelsCount;
        private readonly string formattedString;
        private readonly ListHelper listHelper;
        private readonly NumberingStyle[] levelsNumberingStyles;

        public TextFormattedBullet(ListHelper listHelper, NumberingStyle[] levelsNumberingStyles, string numberFormatString)
        {
            Guard.ThrowExceptionIfNull(listHelper, "listHelper");
            Guard.ThrowExceptionIfOutOfRange(levelsCount, levelsCount, levelsNumberingStyles.Length, "levelsNumberingStyles.Length");
            Guard.ThrowExceptionIfNull(numberFormatString, "numberFormatString");

            this.listHelper = listHelper;
            this.formattedString = ConvertNumberFormatStringToFormattedString(numberFormatString);
            this.levelsNumberingStyles = levelsNumberingStyles;
        }

        public PositionContentElement GetBulletNumberingElement(IListLevelsIndexer listLevelsIndexer)
        {
            IEnumerable<string> arguments = this.GetFormattedStringArguments(listLevelsIndexer);
            string text = string.Format(this.formattedString, arguments.ToArray());
            TextFragment textFragment = string.IsNullOrEmpty(text) ? null : new TextFragment(text);

            return textFragment;
        }

        private static string ConvertNumberFormatStringToFormattedString(string numberFormatString)
        {
            StringBuilder formatBuilder = new StringBuilder();

            int textLength = numberFormatString.Length;

            for (int i = 0; i < textLength; i++)
            {
                char symbol = numberFormatString[i];
                int nextIndex = i + 1;

                if (symbol == LeftBracket)
                {
                    formatBuilder.AppendFormat("{0}{1}{2}", LeftBracket, leftBracketIndex, RightBracket);
                }
                else if (symbol == RightBracket)
                {
                    formatBuilder.AppendFormat("{0}{1}{2}", LeftBracket, rightBracketIndex, RightBracket);
                }
                else if (symbol == Percent && nextIndex < textLength && char.IsNumber(numberFormatString[nextIndex]))
                {
                    int number = int.Parse(numberFormatString[nextIndex].ToString());
                    i++;

                    if (IsValidLevelNumber(number))
                    {
                        formatBuilder.AppendFormat("{0}{1}{2}", LeftBracket, number - 1, RightBracket);
                    }
                    else
                    {
                        return string.Empty;
                    }
                }
                else
                {
                    formatBuilder.Append(symbol);
                }
            }

            return formatBuilder.ToString();
        }

        private static bool IsValidLevelNumber(int number)
        {
            return number >= 1 && number <= levelsCount;
        }

        private IEnumerable<string> GetFormattedStringArguments(IListLevelsIndexer listLevelsIndexer)
        {
            for (int i = 0; i < levelsCount; i++)
            {
                int index = listLevelsIndexer.GetCurrentIndex(i);
                NumberingStyle numberingStyle = this.levelsNumberingStyles[i];
                yield return this.listHelper.GetNumberText(numberingStyle, index);
            }

            yield return LeftBracket.ToString();
            yield return RightBracket.ToString();
        }
    }
}
