﻿using System;
using System.Runtime.Serialization;
using System.Security;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Exceptions
{
    /// <summary>
    /// Represents errors caused by unexpected element when importing RTF document.
    /// </summary>
    [Serializable]
    public class RtfUnexpectedElementException : RtfException
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        public RtfUnexpectedElementException()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public RtfUnexpectedElementException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="cause">The cause.</param>
        public RtfUnexpectedElementException(string message, Exception cause)
            : base(message, cause)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        /// <param name="expected">The expected.</param>
        /// <param name="actual">The actual.</param>
        public RtfUnexpectedElementException(string expected, string actual)
            : this(expected, actual, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        /// <param name="expected">The expected.</param>
        /// <param name="actual">The actual.</param>
        /// <param name="message">The message.</param>
        public RtfUnexpectedElementException(string expected, string actual, string message)
            : base(message)
        {
            this.Expected = expected;
            this.Actual = actual;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfUnexpectedElementException" /> class.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <param name="context">The context.</param>
        protected RtfUnexpectedElementException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            this.Expected = info.GetString("Expected");
            this.Actual = info.GetString("Actual");
        }

        /// <summary>
        /// Gets the expected element.
        /// </summary>
        /// <value>The expected.</value>
        public string Expected { get; private set; }

        /// <summary>
        /// Gets the actual element.
        /// </summary>
        /// <value>The actual.</value>
        public string Actual { get; private set; }

        /// <summary>
        /// Populates a <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// with the data needed to serialize the target object.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo" />
        /// to populate with data.</param>
        /// <param name="context">The destination (see <see cref="T:System.Runtime.Serialization.StreamingContext" />)
        /// for this serialization.</param>
        /// <exception cref="T:System.Security.SecurityException">The caller does not have
        /// the required permission. </exception>
        [SecurityCritical]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Expected", this.Expected);
            info.AddValue("Actual", this.Actual);

            base.GetObjectData(info, context);
        }
    }
}