﻿using System;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.BorderEvaluation;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Export
{
    internal static class BorderExporter
    {
        public static void ExportParagraphBorders(ParagraphBorders borders, ExportContext context, RtfWriter rtfWriter)
        {
            ExportBorder(RtfTags.TagParagraphBorderTop, borders.Top, context, rtfWriter);
            ExportBorder(RtfTags.TagParagraphBorderBottom, borders.Bottom, context, rtfWriter);
            ExportBorder(RtfTags.TagParagraphBorderLeft, borders.Left, context, rtfWriter);
            ExportBorder(RtfTags.TagParagraphBorderRight, borders.Right, context, rtfWriter);
            ExportBorder(RtfTags.TagParagraphBorderBetween, borders.Between, context, rtfWriter);
        }

        public static void ExportTableBorders(TableBorders borders, ExportContext context, RtfWriter rtfWriter)
        {
            if (borders == null || borders == DocumentDefaultStyleSettings.TableBorders)
            {
                return;
            }

            ExportBorder(RtfTags.TagRowBordersTop, borders.Top, context, rtfWriter);
            ExportBorder(RtfTags.TagRowBordersBottom, borders.Bottom, context, rtfWriter);
            ExportBorder(RtfTags.TagRowBordersLeft, borders.Left, context, rtfWriter);
            ExportBorder(RtfTags.TagRowBordersRight, borders.Right, context, rtfWriter);
            ExportBorder(RtfTags.TagRowBordersInnerHorizontal, borders.InsideHorizontal, context, rtfWriter);
            ExportBorder(RtfTags.TagRowBordersInnerVertical, borders.InsideVertical, context, rtfWriter);
        }

        public static void ExportCellBorders(Table table, TableCell cell, TableBorderGrid borderGrid, ExportContext context, RtfWriter rtfWriter)
        {
            if (table.HasCellSpacing)
            {
                Border top = (cell.Borders.Top.Style == BorderStyle.Inherit) ? table.Borders.InsideHorizontal : cell.Borders.Top;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderTop, top, context, rtfWriter);

                Border bottom = (cell.Borders.Bottom.Style == BorderStyle.Inherit) ? table.Borders.InsideHorizontal : cell.Borders.Bottom;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderBottom, bottom, context, rtfWriter);

                Border left = (cell.Borders.Left.Style == BorderStyle.Inherit) ? table.Borders.InsideVertical : cell.Borders.Left;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderLeft, left, context, rtfWriter);

                Border right = (cell.Borders.Right.Style == BorderStyle.Inherit) ? table.Borders.InsideVertical : cell.Borders.Right;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderRight, right, context, rtfWriter);
            }
            else
            {
                // Use the table grid
                Border top = borderGrid.GetTopHorizontalBorder(cell.GridRowIndex, cell.GridColumnIndex).Border;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderTop, top, context, rtfWriter);

                Border bottom = borderGrid.GetTopHorizontalBorder(cell.GridRowIndex + cell.RowSpan, cell.GridColumnIndex).Border;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderBottom, bottom, context, rtfWriter);

                Border left = borderGrid.GetLeftVerticalBorder(cell.GridRowIndex, cell.GridColumnIndex).Border;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderLeft, left, context, rtfWriter);

                Border right = borderGrid.GetLeftVerticalBorder(cell.GridRowIndex, cell.GridColumnIndex + cell.ColumnSpan).Border;
                BorderExporter.ExportBorder(RtfTags.TagCellBorderRight, right, context, rtfWriter);
            }

            // Diagonal borders are exported no matter the cell spacing
            if (cell.Borders.DiagonalDown.Style != BorderStyle.Inherit && cell.Borders.DiagonalDown.Style != BorderStyle.None)
            {
                BorderExporter.ExportBorder(RtfTags.TagCellBorderDiagonalUpperLeft, cell.Borders.DiagonalDown, context, rtfWriter);
            }

            if (cell.Borders.DiagonalUp.Style != BorderStyle.Inherit && cell.Borders.DiagonalUp.Style != BorderStyle.None)
            {
                BorderExporter.ExportBorder(RtfTags.TagCellBorderDiagonalLowerLeft, cell.Borders.DiagonalUp, context, rtfWriter);
            }
        }

        public static void ExportCellBordersFromStyle(TableCellBorders borders, ExportContext context, RtfWriter rtfWriter)
        {
            if (borders == null || borders == DocumentDefaultStyleSettings.TableCellBorders)
            {
                return;
            }

            ExportBorder(RtfTags.TagCellBorderTop, borders.Top, context, rtfWriter);
            ExportBorder(RtfTags.TagCellBorderBottom, borders.Bottom, context, rtfWriter);
            ExportBorder(RtfTags.TagCellBorderLeft, borders.Left, context, rtfWriter);
            ExportBorder(RtfTags.TagCellBorderRight, borders.Right, context, rtfWriter);
            ExportBorder(RtfTags.TagCellBorderDiagonalUpperLeft, borders.DiagonalDown, context, rtfWriter);
            ExportBorder(RtfTags.TagCellBorderDiagonalLowerLeft, borders.DiagonalUp, context, rtfWriter);
        }

        private static void ExportBorder(string tag, Border border, ExportContext context, RtfWriter rtfWriter)
        {
            rtfWriter.DebugWriteNewLine();
            rtfWriter.WriteTag(tag);
            if (border.Thickness != 0)
            {
                rtfWriter.WriteTag(RtfHelper.BorderMapper.GetFromValue(border.Style));
                rtfWriter.WriteTag(RtfTags.TagBorderThickness, Math.Min(255, Unit.DipToTwipI(border.Thickness)));

                Color borderColor = border.Color.GetActualValue(context.Document.Theme);
                if (!RtfHelper.IsTransparentColor(borderColor))
                {
                    rtfWriter.WriteTag(RtfTags.TagBorderColor, context.ColorTable[borderColor]);
                }
            }
            else
            {
                rtfWriter.WriteTag(RtfTags.TagBorderTypeNoBorder);
            }
        }
    }
}
