﻿using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Export
{
    internal class CommentExporter
    {
        private readonly RtfDocumentExporter documentExporter;
        private readonly RtfWriter writer;

        public CommentExporter(RtfDocumentExporter exporter)
        {
            this.documentExporter = exporter;
            this.writer = exporter.Writer;
        }

        private ExportContext Context
        {
            get
            {
                return this.documentExporter.Context;
            }
        }

        public void ExportComment(CommentRangeEnd commentRangeEnd)
        {
            // From Rtf Spec :<annotid> <atnauthor> \chatn <atnicn>? <annotdef> 
            // <annotid>
            using (this.writer.WriteGroup(RtfTags.CommentID, true))
            {
                this.writer.WriteText(commentRangeEnd.Comment.Initials);
            }

            // <atnauthor>
            using (this.writer.WriteGroup(RtfTags.CommentAuthor, true))
            {
                this.writer.WriteText(commentRangeEnd.Comment.Author);
            }

            // \chatn
            this.writer.WriteTag(RtfTags.CommentMarkerTag);

            this.ExportCommentDefinition(commentRangeEnd);
        }

        private void ExportCommentDefinition(CommentRangeEnd commentRangeEnd)
        {
            // <annotdef> 
            using (this.writer.WriteGroup(RtfTags.CommentDefinition, true))
            {
                // <atnref> group first
                using (this.writer.WriteGroup(RtfTags.CommentRef, true))
                {
                    this.writer.WriteText(this.Context.Comments[commentRangeEnd.Comment].ToString());
                }

                using (this.writer.WriteGroup(RtfTags.CommentDate, true))
                {
                    this.writer.WriteText(RtfHelper.ConvertDateTimeToRtfInt(commentRangeEnd.Comment.Date).ToString());
                }

                // export the actual comment document
                foreach (BlockBase blockBase in commentRangeEnd.Comment.Blocks)
                {
                    this.documentExporter.ExportBlock(blockBase);
                }
            }
        }
    }
}