﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Export
{
    internal class ExportContext
    {
        public ExportContext(RadFlowDocument document)
        {
            this.Document = document;
            this.TableStylesStack = new Stack<int?>();
            this.TableStylesStack.Push(null);
            this.Comments = new Dictionary<Comment, int>();
            this.ListNumberingStyleNamesToListIds = new Dictionary<string, int>();
            this.StyleNumberingListIdsToActualListIds = new Dictionary<int, int>();
        }

        public RadFlowDocument Document { get; private set; }

        public Dictionary<Color, int> ColorTable { get; private set; }

        public Dictionary<FontFamily, int> FontTable { get; private set; }

        public Dictionary<string, int> StyleTable { get; private set; }

        public Stack<int?> TableStylesStack { get; private set; }

        public Dictionary<Comment, int> Comments { get; private set; }

        public Dictionary<string, int> ListNumberingStyleNamesToListIds { get; private set; }
        public Dictionary<int, int> StyleNumberingListIdsToActualListIds { get; private set; }

        public void InitializeContext()
        {
            ////this.Document.PopulateListSerializationData();
            this.ConstructColorAndFontTables();
            this.ConstructStyleTable();
        }

        public int GetIdForComment(Comment comment)
        {
            int id;
            if (this.Comments.ContainsKey(comment))
            {
                id = this.Comments[comment];
            }
            else
            {
                id = AnnotationIdGenerator.GetNext();
                this.Comments.Add(comment, id);
            }

            return id;
        }

        private static void AddCharacterProperties(CharacterProperties props, DocumentTheme theme, InitTablesContext initContext)
        {
            if (props.FontFamily.HasLocalValue)
            {
                initContext.Fonts.Add(props.FontFamily.LocalValue.GetActualValue(theme));
            }

            if (props.ForegroundColor.HasLocalValue)
            {
                initContext.Colors.Add(props.ForegroundColor.LocalValue.GetActualValue(theme));
            }

            if (props.HighlightColor.HasLocalValue)
            {
                Color? col = props.HighlightColor.LocalValue;
                if (col.HasValue)
                {
                    initContext.Colors.Add(col.Value);
                }
            }

            if (props.UnderlineColor.HasLocalValue)
            {
                initContext.Colors.Add(props.UnderlineColor.LocalValue.GetActualValue(theme));
            }

            ExportContext.AddShadingColors(props, theme, initContext);
        }

        private static void AddParagraphProperties(ParagraphProperties props, DocumentTheme theme, InitTablesContext initContext)
        {
            ExportContext.AddShadingColors(props, theme, initContext);
            if (props.ParagraphMarkerProperties != null)
            {
                ExportContext.AddCharacterProperties(props.ParagraphMarkerProperties, theme, initContext);
            }

            ParagraphBorders borders = props.Borders.GetActualValue();
            if (borders != null)
            {
                ExportContext.AddParagraphBorderColors(borders, theme, initContext);
            }
        }

        private static void AddTablePropsFromStyle(Style style, DocumentTheme theme, InitTablesContext initContext)
        {
            ExportContext.AddShadingColors(style.TableProperties, theme, initContext);
            ExportContext.AddShadingColors(style.TableCellProperties, theme, initContext);

            ParagraphBorders paragraphBorders = style.ParagraphProperties.Borders.GetActualValue();
            if (paragraphBorders != null)
            {
                ExportContext.AddParagraphBorderColors(paragraphBorders, theme, initContext);
            }

            TableCellBorders cellBorders = style.TableCellProperties.Borders.GetActualValue();
            if (cellBorders != null)
            {
                ExportContext.AddTableCellBorderColors(cellBorders, theme, initContext);
            }

            TableBorders tableBorders = style.TableProperties.Borders.GetActualValue();
            if (tableBorders != null)
            {
                ExportContext.AddTableBorderColors(tableBorders, theme, initContext);
            }
        }

        private static void AddShadingColors(IPropertiesWithShading props, DocumentTheme theme, InitTablesContext initContext)
        {
            if (props.BackgroundColor.HasLocalValue)
            {
                initContext.Colors.Add(props.BackgroundColor.LocalValue.GetActualValue(theme));
            }

            if (props.ShadingPatternColor.HasLocalValue)
            {
                initContext.Colors.Add(props.ShadingPatternColor.LocalValue.GetActualValue(theme));
            }
        }

        private static void AddTableCellBorderColors(TableCellBorders cellBorders, DocumentTheme theme, InitTablesContext initContext)
        {
            initContext.Colors.Add(cellBorders.Left.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.Top.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.Right.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.Bottom.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.InsideHorizontal.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.InsideVertical.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.DiagonalDown.Color.GetActualValue(theme));
            initContext.Colors.Add(cellBorders.DiagonalUp.Color.GetActualValue(theme));
        }

        private static void AddTableBorderColors(TableBorders tableBorders, DocumentTheme theme, InitTablesContext initContext)
        {
            initContext.Colors.Add(tableBorders.Left.Color.GetActualValue(theme));
            initContext.Colors.Add(tableBorders.Top.Color.GetActualValue(theme));
            initContext.Colors.Add(tableBorders.Right.Color.GetActualValue(theme));
            initContext.Colors.Add(tableBorders.Bottom.Color.GetActualValue(theme));
            initContext.Colors.Add(tableBorders.InsideHorizontal.Color.GetActualValue(theme));
            initContext.Colors.Add(tableBorders.InsideVertical.Color.GetActualValue(theme));
        }

        private static void AddParagraphBorderColors(ParagraphBorders parBorders, DocumentTheme theme, InitTablesContext initContext)
        {
            initContext.Colors.Add(parBorders.Left.Color.GetActualValue(theme));
            initContext.Colors.Add(parBorders.Top.Color.GetActualValue(theme));
            initContext.Colors.Add(parBorders.Right.Color.GetActualValue(theme));
            initContext.Colors.Add(parBorders.Bottom.Color.GetActualValue(theme));
            initContext.Colors.Add(parBorders.Between.Color.GetActualValue(theme));
        }

        private void ConstructStyleTable()
        {
            this.StyleTable = new Dictionary<string, int>();
            int styleId = 1;
            foreach (Style style in this.Document.StyleRepository.Styles)
            {
                this.StyleTable[style.Id] = (style.StyleType == StyleType.Paragraph && style.IsDefault) ? 0 : styleId++;
                if (style.LinkedStyleId != null && this.Document.StyleRepository.Contains(style.LinkedStyleId) && !this.StyleTable.ContainsKey(style.LinkedStyleId))
                {
                    this.StyleTable[style.LinkedStyleId] = styleId++;
                }
            }
        }

        private void ConstructColorAndFontTables()
        {
            // TODO: Change color table to work with themable colors in order to know when to export omitted (automatic) color.
            InitTablesContext initContext = new InitTablesContext();

            // First Font should be the the document default font
            initContext.Fonts.Add(this.Document.DefaultStyle.CharacterProperties.FontFamily.GetActualValue().GetActualValue(this.Document.Theme));
            initContext.Fonts.Add(Run.FontFamilyPropertyDefinition.DefaultValue.GetActualValue(this.Document.Theme));
            initContext.Colors.Add(Run.ForegroundColorPropertyDefinition.DefaultValue.GetActualValue(this.Document.Theme));

            this.InitTablesForDocument(initContext);

            // Finally fill dictionaries
            initContext.Colors.Remove(Colors.Transparent); // Remove the transparent color, since it is not supported by RTF.
            int index = 0;
            this.ColorTable = initContext.Colors.ToDictionary(c => index++);

            index = 0;
            this.FontTable = initContext.Fonts.ToDictionary(f => index++);
        }

        private void InitTablesForDocument(InitTablesContext initContext)
        {
            if (this.Document == null)
            {
                return;
            }

            DocumentTheme theme = this.Document.Theme;

            foreach (DocumentElementBase element in this.Document.EnumerateChildrenOfType<DocumentElementBase>())
            {
                this.InitTablesForElement(element, theme, initContext);
            }

            // Default Properties
            AddCharacterProperties(this.Document.DefaultStyle.CharacterProperties, theme, initContext);
            AddParagraphProperties(this.Document.DefaultStyle.ParagraphProperties, theme, initContext);

            foreach (Style style in this.Document.StyleRepository.Styles)
            {
                switch (style.StyleType)
                {
                    case StyleType.Character:
                        ExportContext.AddCharacterProperties(style.CharacterProperties, theme, initContext);
                        break;
                    case StyleType.Paragraph:
                        ExportContext.AddCharacterProperties(style.CharacterProperties, theme, initContext);
                        ExportContext.AddParagraphProperties(style.ParagraphProperties, theme, initContext);
                        break;
                    case StyleType.Table:
                        ExportContext.AddCharacterProperties(style.CharacterProperties, theme, initContext);
                        ExportContext.AddParagraphProperties(style.ParagraphProperties, theme, initContext);
                        ExportContext.AddTablePropsFromStyle(style, theme, initContext);
                        break;
                    case StyleType.Numbering:
                    default:
                        break;
                }
            }

            foreach (List list in this.Document.Lists)
            {
                foreach (ListLevel listLevel in list.Levels)
                {
                    ExportContext.AddCharacterProperties(listLevel.CharacterProperties, theme, initContext);
                }
            }
        }

        private void InitTablesForElement(DocumentElementBase element, DocumentTheme theme, InitTablesContext initContext)
        {
            if (element == null)
            {
                return;
            }

            Run run = element as Run;
            if (run != null)
            {
                ExportContext.AddCharacterProperties(run.Properties, theme, initContext);
            }
            else if (element is Paragraph)
            {
                Paragraph par = (Paragraph)element;
                ExportContext.AddParagraphProperties(par.Properties, theme, initContext);

                // TODO: Lists
                ////if (par.IsInList)
                ////{
                ////    GetDocumentListForParagraph(par).Style.Levels.ForEach(levelStyle =>
                ////    {
                ////        if (levelStyle.FontFamily != null)
                ////        {
                ////            initContext.Fonts.Add(levelStyle.FontFamily);
                ////        }
                ////    });
                ////}
            }
            else if (element is TableCell)
            {
                TableCell cell = (TableCell)element;
                ExportContext.AddShadingColors(cell.Properties, theme, initContext);
                ExportContext.AddTableCellBorderColors(cell.Borders, theme, initContext);
            }
            else if (element is Table)
            {
                Table table = (Table)element;
                ExportContext.AddShadingColors(table.Properties, theme, initContext);
                ExportContext.AddTableBorderColors(table.Borders, theme, initContext);
            }
            else if (element is Section)
            {
                Section section = (Section)element;
                this.InitTablesForHeaderFooter(section.Headers.First, theme, initContext);
                this.InitTablesForHeaderFooter(section.Headers.Default, theme, initContext);
                this.InitTablesForHeaderFooter(section.Headers.Even, theme, initContext);
                this.InitTablesForHeaderFooter(section.Footers.First, theme, initContext);
                this.InitTablesForHeaderFooter(section.Footers.Default, theme, initContext);
                this.InitTablesForHeaderFooter(section.Footers.Even, theme, initContext);
            }

            // TODO: other types
            ////else if (element is CommentRangeEnd)
            ////{
            ////    InitTablesForDocument(((CommentRangeEnd)element).Comment.Body, initContext);
            ////}
        }

        private void InitTablesForHeaderFooter(HeaderFooterBase headerFooter, DocumentTheme theme, InitTablesContext initContext)
        {
            if (headerFooter == null)
            {
                return;
            }
            foreach (DocumentElementBase element in headerFooter.EnumerateChildrenOfType<DocumentElementBase>())
            {
                this.InitTablesForElement(element, theme, initContext);
            }
        }

        private class InitTablesContext
        {
            public InitTablesContext()
            {
                this.Colors = new HashSet<Color>();
                this.Fonts = new HashSet<FontFamily>();
            }

            public HashSet<Color> Colors { get; set; }

            public HashSet<FontFamily> Fonts { get; set; }
        }
    }
}