﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Common.Tables;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.BorderEvaluation;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Primitives;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Export
{
    internal class TableExporter
    {
        private readonly RtfDocumentExporter exporter;
        private readonly RtfWriter writer;
        private readonly Table table;
        private readonly bool isNestedTable;
        private readonly TableBorderGrid borderGrid;

        ////private int[] columnRightBoundaries;

        public TableExporter(RtfDocumentExporter exporter, Table table)
        {
            this.exporter = exporter;
            this.writer = exporter.Writer;
            this.table = table;
            this.isNestedTable = !(table.Parent is Section);
            ////this.InitColumnRightBoundaries();

            // Recalculate border grid - we will need it when exporting cell borders
            if (!this.table.HasCellSpacing)
            {
                this.borderGrid = new TableBorderGrid(this.table);
                this.borderGrid.AssureValid();
            }
        }

        public static void ExportTableStyle(Style style, ExportContext context, RtfWriter rtfWriter)
        {
            // Table Properties
            BorderExporter.ExportTableBorders(style.TableProperties.Borders.GetActualValue(), context, rtfWriter);

            double? indent = style.TableProperties.Indent.GetActualValue();
            if (indent.HasValue)
            {
                TableExporter.ExportTableIndent(indent.Value, rtfWriter);
            }

            Alignment? align = style.TableProperties.Alignment.GetActualValue();
            if (align.HasValue)
            {
                FlowDirection? direction = style.TableProperties.FlowDirection.GetActualValue();
                FlowDirection directionValue = direction.HasValue ? direction.Value : FlowDirection.LeftToRight;
                TableExporter.ExportTableHorizontalAlignment(align.Value, directionValue, rtfWriter);
            }

            int? rowBands = style.TableProperties.RowBanding.GetActualValue();
            if (rowBands.HasValue)
            {
                rtfWriter.WriteTag(RtfTags.TagTableRowBanding, rowBands.Value);
            }

            int? colBands = style.TableProperties.ColumnBanding.GetActualValue();
            if (colBands.HasValue)
            {
                rtfWriter.WriteTag(RtfTags.TagTableColumnBanding, colBands.Value);
            }

            double? tableCellSpacing = style.TableProperties.TableCellSpacing.GetActualValue();
            if (tableCellSpacing.HasValue)
            {
                TableExporter.ExportCellSpacing(tableCellSpacing.Value, rtfWriter);
            }

            Padding tableCellPaddingInTableProps = style.TableProperties.TableCellPadding.GetActualValue();
            if (tableCellPaddingInTableProps != null)
            {
                TableExporter.ExportCellPaddingForTable(tableCellPaddingInTableProps, rtfWriter);
            }

            TableExporter.ExportTableShading(style.TableProperties, context, rtfWriter);

            // Table Cell Properties
            VerticalAlignment? cellVertAlign = style.TableCellProperties.VerticalAlignment.GetActualValue();
            if (cellVertAlign.HasValue)
            {
                TableExporter.ExportCellVerticalAlignment(cellVertAlign.Value, rtfWriter, true);
            }

            BorderExporter.ExportCellBordersFromStyle(style.TableCellProperties.Borders.GetActualValue(), context, rtfWriter);

            Padding cellPaddingInCellProps = style.TableCellProperties.Padding.GetActualValue();
            if (cellPaddingInCellProps != null)
            {
                TableExporter.ExportCellPaddingForTableCell(cellPaddingInCellProps, rtfWriter);
            }

            TableExporter.ExportCellShading(style.TableCellProperties, context, rtfWriter, true);
        }

        public static void ExportCellVerticalAlignment(VerticalAlignment align, RtfWriter rtfWriter, bool isInStyle)
        {
            switch (align)
            {
                case VerticalAlignment.Top:
                    rtfWriter.WriteTag(isInStyle ? RtfTags.TagCellContentAlignmentTopInStyle : RtfTags.TagCellContentAlignmentTop);
                    break;

                case VerticalAlignment.Bottom:
                    rtfWriter.WriteTag(isInStyle ? RtfTags.TagCellContentAlignmentBottomInStyle : RtfTags.TagCellContentAlignmentBottom);
                    break;

                // Justified is not supported in RTF
                case VerticalAlignment.Justified:
                case VerticalAlignment.Center:
                    rtfWriter.WriteTag(isInStyle ? RtfTags.TagCellContentAlignmentCenterInStyle : RtfTags.TagCellContentAlignmentCenter);
                    break;
            }
        }

        public static void ExportPreferredWidth(TableCell cell, RtfWriter rtfWriter)
        {
            if (cell.PreferredWidth != null)
            {
                switch (cell.PreferredWidth.Type)
                {
                    case TableWidthUnitType.Fixed:
                        rtfWriter.WriteTag(RtfTags.TagCellPreferredWidthUnit, 3);
                        rtfWriter.WriteTag(RtfTags.TagCellPreferredWidth, Unit.DipToTwipI(cell.PreferredWidth.Value));
                        break;
                    case TableWidthUnitType.Percent:
                        rtfWriter.WriteTag(RtfTags.TagCellPreferredWidthUnit, 2);
                        rtfWriter.WriteTag(RtfTags.TagCellPreferredWidth, (int)(cell.PreferredWidth.Value * 50));
                        break;
                }
            }
        }

        public static void ExportCellPaddingForTableCell(Padding cellPadding, RtfWriter rtfWriter)
        {
            rtfWriter.WriteTag(RtfTags.TagCellPaddingUnitLeft, 3);
            rtfWriter.WriteTag(RtfTags.TagCellPaddingLeft, Unit.DipToTwipI(cellPadding.Left));
            rtfWriter.WriteTag(RtfTags.TagCellPaddingUnitRight, 3);
            rtfWriter.WriteTag(RtfTags.TagCellPaddingRight, Unit.DipToTwipI(cellPadding.Right));
            rtfWriter.WriteTag(RtfTags.TagCellPaddingUnitTop, 3);
            rtfWriter.WriteTag(RtfTags.TagCellPaddingTop, Unit.DipToTwipI(cellPadding.Top));
            rtfWriter.WriteTag(RtfTags.TagCellPaddingUnitBottom, 3);
            rtfWriter.WriteTag(RtfTags.TagCellPaddingBottom, Unit.DipToTwipI(cellPadding.Bottom));
        }

        public static void ExportCellPaddingForTable(Padding cellPadding, RtfWriter rtfWriter)
        {
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingUnitLeft, 3);
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingLeft, Unit.DipToTwipI(cellPadding.Left));
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingUnitRight, 3);
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingRight, Unit.DipToTwipI(cellPadding.Right));
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingUnitTop, 3);
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingTop, Unit.DipToTwipI(cellPadding.Top));
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingUnitBottom, 3);
            rtfWriter.WriteTag(RtfTags.TagRowDefaultCellPaddingBottom, Unit.DipToTwipI(cellPadding.Bottom));
        }

        public static void ExportCellShading(TableCellProperties cellProps, ExportContext context, RtfWriter writer, bool isInStyle)
        {
            DocumentTheme docTheme = cellProps.Document.Theme;
            Color shadingBack = cellProps.BackgroundColor.GetActualValue().GetActualValue(docTheme);
            Color shadingFore = cellProps.ShadingPatternColor.GetActualValue().GetActualValue(docTheme);
            if (!RtfHelper.IsTransparentColor(shadingBack) || !RtfHelper.IsTransparentColor(shadingFore))
            {
                if (!RtfHelper.IsTransparentColor(shadingBack))
                {
                    string backTag = isInStyle ? RtfTags.TagCellBackgroundInStyle : RtfTags.TagCellBackground;
                    writer.WriteTag(backTag, context.ColorTable[shadingBack]);
                }

                if (!RtfHelper.IsTransparentColor(shadingFore))
                {
                    string foreTag = isInStyle ? RtfTags.TagCellForegroundInStyle : RtfTags.TagCellForeground;
                    writer.WriteTag(foreTag, context.ColorTable[shadingFore]);
                }

                string patternTag = isInStyle ? RtfTags.TagCellShadingPercentInStyles : RtfTags.TagCellShadingPercent;
                writer.WriteTag(patternTag, RtfHelper.ShadingPatternToRtfTag(cellProps.ShadingPattern.GetActualValue().Value));
            }
        }

        public static void ExportTableShading(TableProperties tableProps, ExportContext context, RtfWriter writer)
        {
            DocumentTheme docTheme = tableProps.Document.Theme;
            Color shadingBack = tableProps.BackgroundColor.GetActualValue().GetActualValue(docTheme);
            Color shadingFore = tableProps.ShadingPatternColor.GetActualValue().GetActualValue(docTheme);
            if (!RtfHelper.IsTransparentColor(shadingBack) || !RtfHelper.IsTransparentColor(shadingFore))
            {
                if (!RtfHelper.IsTransparentColor(shadingBack))
                {
                    writer.WriteTag(RtfTags.TagRowShadingBackground, context.ColorTable[shadingBack]);
                }

                if (!RtfHelper.IsTransparentColor(shadingFore))
                {
                    writer.WriteTag(RtfTags.TagRowShadingForeground, context.ColorTable[shadingFore]);
                }

                writer.WriteTag(RtfTags.TagRowShadingPercent, RtfHelper.ShadingPatternToRtfTag(tableProps.ShadingPattern.GetActualValue().Value));
            }
        }

        public static void ExportTableIndent(double tableIndent, RtfWriter rtfWriter)
        {
            if (tableIndent != 0.0)
            {
                rtfWriter.WriteTag(RtfTags.TagTableIndent, Unit.DipToTwipI(tableIndent));
                rtfWriter.WriteTag(RtfTags.TagTableIndentType, 3);
            }
        }

        public static void ExportTableHorizontalAlignment(Alignment alignment, FlowDirection flowDirection, RtfWriter rtfWriter)
        {
            Alignment alignmentToExport = alignment;
            if (flowDirection == FlowDirection.RightToLeft)
            {
                // Reverse alignments if right-to-left.
                if (alignment == Alignment.Left)
                {
                    alignmentToExport = Alignment.Right;
                }
                else if (alignment == Alignment.Right)
                {
                    alignmentToExport = Alignment.Left;
                }
            }
            if (alignmentToExport != Alignment.Left || flowDirection == FlowDirection.RightToLeft)
            {
                rtfWriter.WriteTag(RtfHelper.TableHorzontalAlignmentToRtfTag(alignmentToExport));
            }
        }

        public static void ExportCellSpacing(double cellSpacing, RtfWriter rtfwriter)
        {
            rtfwriter.WriteTag(RtfTags.TagSpaceBetweenCells, Unit.DipToTwipI(cellSpacing));
            if (cellSpacing > 0)
            {
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingLeft, Unit.DipToTwipI(cellSpacing));
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingUnitLeft, 3);
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingRight, Unit.DipToTwipI(cellSpacing));
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingUnitRight, 3);
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingTop, Unit.DipToTwipI(cellSpacing));
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingUnitTop, 3);
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingBottom, Unit.DipToTwipI(cellSpacing));
                rtfwriter.WriteTag(RtfTags.TagRowCellSpacingUnitBottom, 3);
            }
        }

        public void ExportTable()
        {
            using (this.writer.WriteGroup())
            {
                int styleId;
                if (!string.IsNullOrEmpty(this.table.StyleId) && this.exporter.Context.StyleTable.TryGetValue(this.table.StyleId, out styleId))
                {
                    this.exporter.Context.TableStylesStack.Push(styleId);
                }
                else
                {
                    this.exporter.Context.TableStylesStack.Push(null);
                }

                this.writer.DebugWriteNewLine();

                TableCellDataGrid grid = TableCellDataGrid.CreateFromTable(this.table);

                foreach (TableRowData rowData in grid.Data)
                {
                    foreach (TableCellData cellData in rowData.TableCellDatas)
                    {
                        if (cellData.Type == TableCellType.Empty || cellData.Type == TableCellType.HorizontalyMerged)
                        {
                            continue;
                        }
                        else if (cellData.Type == TableCellType.VerticallyMerged)
                        {
                            this.WriteCellEnd();
                        }
                        else if (cellData.Type == TableCellType.Cell)
                        {
                            if (cellData.Cell.Blocks.Count > 0)
                            {
                                foreach (BlockBase block in cellData.Cell.Blocks)
                                {
                                    this.exporter.ExportBlock(block);
                                }
                            }
                            else
                            {
                                this.ExportEmptyCell(cellData.Cell);
                            }
                            this.WriteCellEnd();
                        }
                    }

                    this.WriteRowProperties(rowData);
                    this.writer.DebugWriteNewLine();
                }
                this.exporter.Context.TableStylesStack.Pop();
            }
        }

        private static void ExportTableLook(TableLooks tableValues, RtfWriter writer)
        {
            ExportTableLook(tableValues, TableLooks.FirstRow, true, writer);
            ExportTableLook(tableValues, TableLooks.LastRow, true, writer);
            ExportTableLook(tableValues, TableLooks.FirstColumn, true, writer);
            ExportTableLook(tableValues, TableLooks.LastColumn, true, writer);
            ExportTableLook(tableValues, TableLooks.BandedRows, false, writer);
            ExportTableLook(tableValues, TableLooks.BandedColumns, false, writer);
        }

        private static void ExportTableLook(TableLooks tableValues, TableLooks valueToExport, bool exportIfFlagIsSet, RtfWriter writer)
        {
            if (exportIfFlagIsSet)
            {
                if (tableValues.HasFlag(valueToExport))
                {
                    writer.WriteTag(RtfHelper.TableLookMapper.GetFromValue(valueToExport));
                }
            }
            else
            {
                // Banded cols/rows are exported if the fals is NOT set
                if (!tableValues.HasFlag(valueToExport))
                {
                    writer.WriteTag(RtfHelper.TableLookMapper.GetFromValue(valueToExport));
                }
            }
        }

        private void ExportEmptyCell(TableCell cell)
        {
            this.writer.WriteTag(RtfTags.TagParagraphDefaults);
            this.writer.WriteTag(RtfTags.TagParagraphInTable);

            int nestingLevel = RtfHelper.GetNestingLevel(cell);
            nestingLevel = (nestingLevel - 1) / 3;
            this.writer.WriteTag(RtfTags.TagParagraphNestingLevel, nestingLevel);
        }

        private void WriteCellEnd()
        {
            if (this.isNestedTable)
            {
                this.writer.WriteTag(RtfTags.TagNestedCellEnd);
            }
            else
            {
                this.writer.WriteTag(RtfTags.TagCellEnd);
            }
        }

        private void WriteRowProperties(TableRowData rowData)
        {
            this.writer.DebugWriteNewLine();
            if (this.isNestedTable)
            {
                using (this.writer.WriteGroup(RtfTags.TagNestedTablePropertiesDestenation, true))
                {
                    this.ExportRowDefinition(rowData);
                    this.writer.WriteTag(RtfTags.TagNestedRowEnd);
                }
            }
            else
            {
                this.ExportRowDefinition(rowData);
                this.writer.WriteTag(RtfTags.TagRowEnd);
            }
        }

        private void ExportRowDefinition(TableRowData rowData)
        {
            TableRow row = rowData.Row;
            int gridRowIndex = row.GridRowIndex;

            this.writer.WriteTag(RtfTags.TagRowDefaults);
            this.writer.WriteTag(RtfTags.TagRowIndex, gridRowIndex);
            this.writer.WriteTag(RtfTags.TagRowIndexHeaderAdjusted, gridRowIndex);

            int styleId;
            if (!string.IsNullOrEmpty(this.table.StyleId) && this.exporter.Context.StyleTable.TryGetValue(this.table.StyleId, out styleId))
            {
                this.writer.WriteTag(RtfTags.TagStyleTable, styleId);
            }

            if (row.Table.Properties.FlowDirection.GetActualValue().Value == FlowDirection.LeftToRight)
            {
                this.writer.WriteTag(RtfTags.TagRowLtr);
            }
            else
            {
                this.writer.WriteTag(RtfTags.TagRowRtl);
            }

            if (gridRowIndex == row.Table.Rows.Count - 1)
            {
                this.writer.WriteTag(RtfTags.TagLastRow);
            }

            TableExporter.ExportCellSpacing(this.table.TableCellSpacing, this.writer);
            BorderExporter.ExportTableBorders(this.table.Borders, this.exporter.Context, this.writer);

            TableRowHeight heightValue = row.Properties.Height.GetActualValue();
            if (heightValue.Type == HeightType.Exact && heightValue.Value != 0)
            {
                this.writer.WriteTag(RtfTags.TagRowHeight, -Unit.DipToTwipI(row.Properties.Height.GetActualValue().Value));
            }
            else if (heightValue.Type == HeightType.AtLeast && heightValue.Value != 0)
            {
                this.writer.WriteTag(RtfTags.TagRowHeight, Unit.DipToTwipI(row.Properties.Height.GetActualValue().Value));
            }

            if (!row.CanSplit)
            {
                this.writer.WriteTag(RtfTags.TagRowNoSplit);
            }

            if (row.RepeatOnEveryPage)
            {
                this.writer.WriteTag(RtfTags.TagRowRepeatOnEveryPage);
            }

            // TableWidthUnitType.Auto is not exported as it tis the default
            if (this.table.PreferredWidth.Type == TableWidthUnitType.Fixed)
            {
                this.writer.WriteTag(RtfTags.TagRowPreferredWidth, Unit.DipToTwipI(this.table.PreferredWidth.Value));
                this.writer.WriteTag(RtfTags.TagRowPreferredWidthUnit, 3);
            }
            else if (this.table.PreferredWidth.Type == TableWidthUnitType.Percent)
            {
                this.writer.WriteTag(RtfTags.TagRowPreferredWidth, (int)(this.table.PreferredWidth.Value * 50));
                this.writer.WriteTag(RtfTags.TagRowPreferredWidthUnit, 2);
            }

            this.writer.WriteTag(RtfTags.TagRowAutoFit, row.Table.LayoutType == TableLayoutType.FixedWidth ? 0 : 1);

            TableExporter.ExportTableIndent(this.table.Indent, this.writer);
            TableExporter.ExportCellPaddingForTable(this.table.TableCellPadding, this.writer);
            TableExporter.ExportTableHorizontalAlignment(this.table.Alignment, this.table.FlowDirection, this.writer);
            TableExporter.ExportTableShading(this.table.Properties, this.exporter.Context, this.writer);
            TableExporter.ExportTableLook(this.table.Looks, this.writer);

            this.writer.DebugWriteNewLine();
            foreach (TableCellData cellData in rowData.TableCellDatas)
            {
                if (cellData.Type == TableCellType.Empty || cellData.Type == TableCellType.HorizontalyMerged)
                {
                    continue;
                }

                if (cellData.Type == TableCellType.Cell && cellData.Cell.RowSpan > 1)
                {
                    // First cell of vertical span.
                    this.writer.WriteTag(RtfTags.TagFirstRowSpannedCell);
                }
                else if (cellData.Type == TableCellType.VerticallyMerged)
                {
                    // Cell is in vertical span.
                    this.writer.WriteTag(RtfTags.TagRowSpannedCell);
                }

                this.ExportCellProperties(cellData.Cell);
                this.writer.DebugWriteNewLine();
            }
        }

        private void ExportCellProperties(TableCell cell)
        {
            TableExporter.ExportCellVerticalAlignment(cell.VerticalAlignment, this.writer, false);

            BorderExporter.ExportCellBorders(this.table, cell, this.borderGrid, this.exporter.Context, this.writer);

            TableExporter.ExportCellShading(cell.Properties, this.exporter.Context, this.writer, false);

            TableExporter.ExportPreferredWidth(cell, this.writer);

            TableExporter.ExportCellPaddingForTableCell(cell.Padding, this.writer);

            string textDirection;
            if (RtfHelper.TableCellTextDirectionMapper.TryGetFromValue(cell.TextDirection, out textDirection))
            {
                this.writer.WriteTag(textDirection);
            }

            if (!cell.CanWrapContent)
            {
                this.writer.WriteTag(RtfTags.TagCellNoWrapText);
            }

            if (cell.IgnoreCellMarkerInRowHeightCalculation)
            {
                this.writer.WriteTag(RtfTags.TagCellIgnoreCellMarkerInRowHeight);
            }

            // TODO: This is just temporary calculation of table boundaries
            int cellLastCol = cell.GridColumnIndex + cell.ColumnSpan - 1;
            double width = (cellLastCol + 1) * 200;
            this.writer.WriteTag(RtfTags.TagCellRightBoundry, Unit.DipToTwipI(width));
        }

#if false
        private void InitColumnRightBoundaries()
        {
            this.columnRightBoundaries = new int[this.table.Grid.Columns.Count];
            this.columnRightBoundaries[0] = Unit.DipToTwipI(this.table.Grid.Columns[0].Width);
            for (int i = 1; i < this.table.Grid.Columns.Count; i++)
            {
                this.columnRightBoundaries[i] = Unit.DipToTwipI(this.table.Grid.Columns[i].Width) + this.columnRightBoundaries[i - 1];
            }
        }  
#endif
    }
}