﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal class CommentImporter
    {
        private const string SingleCommentReferenceString = "singleRef";
        private readonly RtfImportContext context;

        public CommentImporter(RtfImportContext context)
        {
            this.context = context;
        }

        public Comment Comment { get; set; }

        public string RefID { get; set; }

        public string Author { get; set; }

        public string Initials { get; set; }

        public DateTime Date { get; set; }

        public void ImportCommentId(RtfGroup group)
        {
            this.Initials = RtfHelper.GetGroupText(group);
        }

        public void ImportAuthor(RtfGroup group)
        {
            this.Author = RtfHelper.GetGroupText(group);
        }

        public void ImportDate(RtfGroup group)
        {
            int dateTimeRtf;
            if (int.TryParse(RtfHelper.GetGroupText(group), out dateTimeRtf))
            {
                this.Date = RtfHelper.ConvertRtfIntToDateTime(dateTimeRtf);
            }
        }

        public void ImportCommentDefinition(RtfGroup group)
        {
            // Find Ref group
            RtfGroup refGroup = group.Elements.FirstOrDefault(e =>
                {
                    RtfGroup g = e as RtfGroup;
                    return g != null && g.Destination == RtfTags.CommentRef;
                }) as RtfGroup;

            // Read RefID
            if (refGroup != null)
            {
                this.RefID = RtfHelper.GetGroupText(refGroup);
            }
            else
            {
                // If there is none - generate one
                this.RefID = CommentImporter.SingleCommentReferenceString + AnnotationIdGenerator.GetNext().ToString();
            }

            // If there are no preceding start and end annotations - add them
            if (!this.context.Comments.ContainsKey(this.RefID))
            {
                this.Comment = this.context.Document.Comments.AddComment();
                this.context.Comments[this.RefID] = this.Comment;

                this.context.AddInline(this.Comment.CommentRangeStart);
                this.context.AddInline(this.Comment.CommentRangeEnd);
            }

            this.Comment = this.context.Comments[this.RefID];

            RtfImportContext localContext = new RtfImportContext(this.context, this.Comment);
            localContext.CurrentCommentHandler = this;

            RtfDocumentImporter importer = new RtfDocumentImporter(localContext);
            importer.ImportRoot(group);

            this.Comment.Author = this.Author;
            this.Comment.Initials = this.Initials;
            this.Comment.Date = this.Date;
        }
    }
}