﻿using System;
using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model
{
    internal sealed class RtfGroup : RtfElement
    {
        private readonly List<RtfElement> elements = new List<RtfElement>();

        public override RtfElementType Type
        {
            get { return RtfElementType.Group; }
        }

        public List<RtfElement> Elements
        {
            get { return this.elements; }
        }

        public bool IsExtensionDestination
        {
            get
            {
                if (this.elements.Count > 0)
                {
                    RtfElement firstChild = this.elements[0];
                    if (firstChild.Type == RtfElementType.Tag)
                    {
                        RtfTag firstTag = (RtfTag)firstChild;
                        if (RtfTags.TagExtensionDestination.Equals(firstTag.Name))
                        {
                            return true;
                        }
                    }
                }

                return false;
            }
        }

        public string Destination
        {
            get
            {
                if (this.elements.Count > 0)
                {
                    RtfElement firstChild = this.elements[0];
                    if (firstChild.Type == RtfElementType.Tag)
                    {
                        RtfTag firstTag = (RtfTag)firstChild;
                        if (RtfTags.TagExtensionDestination.Equals(firstTag.Name))
                        {
                            if (this.elements.Count > 1)
                            {
                                RtfElement secondChild = this.elements[1];
                                if (secondChild.Type == RtfElementType.Tag)
                                {
                                    RtfTag secondTag = (RtfTag)secondChild;
                                    return secondTag.Name;
                                }
                            }
                        }

                        return firstTag.Name;
                    }
                }
                return null;
            }
        }

        public RtfGroup SelectChildGroupWithDestination(string destination)
        {
            if (destination == null)
            {
                throw new ArgumentNullException("destination");
            }

            foreach (RtfElement child in this.elements)
            {
                if (child.Type == RtfElementType.Group)
                {
                    RtfGroup group = (RtfGroup)child;
                    if (destination.Equals(group.Destination))
                    {
                        return group;
                    }
                }
            }
            return null;
        }

        public override string ToString()
        {
            StringBuilder buf = new StringBuilder("{");
            int count = this.elements.Count;
            buf.Append(count);
            buf.Append(" items");
            if (count > 0)
            {
                // visualize the first two child elements for convenience during debugging
                buf.Append(": [");
                buf.Append(this.elements[0]);
                if (count > 1)
                {
                    buf.Append(", ");
                    buf.Append(this.elements[1]);
                    if (count > 2)
                    {
                        buf.Append(", ");
                        if (count > 3)
                        {
                            buf.Append("..., ");
                        }
                        buf.Append(this.elements[count - 1]);
                    }
                }
                buf.Append("]");
            }
            buf.Append("}");
            return buf.ToString();
        }

        protected override bool IsEqual(object obj)
        {
            RtfGroup compare = obj as RtfGroup; // guaranteed to be non-null
            return compare != null && base.IsEqual(obj) &&
                this.elements.Equals(compare.elements);
        }

        protected override int ComputeHashCode()
        {
            return HashTool.AddHashCode(base.ComputeHashCode(), this.elements);
        }
    }
}