﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Parser
{
    internal class RtfCodePageDecoder
    {
        private const string MetafilesRelativePath = "CodePages";
        private const string MetafilesExtension = ".bin";

        // Unsupported codepages: 709, 710, 711, 819
        private static readonly int[] CodePages = new int[] { 437, 708, 720, 850, 852, 860, 862, 863, 864, 865, 866, 874, 932, 936, 949, 950, 1250, 1251, 1252, 1253, 1254, 1255, 1256, 1257, 1258, 1361, 10000, 10001, 10004, 10005, 10006, 10007, 10029, 10081, 57002, 57003, 57004, 57005, 57006, 57007, 57008, 57009, 57010, 57011 };
        private static readonly Dictionary<int, string> CodePagesData;

        private string codePageData;
        private int currentCodePage;

        static RtfCodePageDecoder()
        {
            RtfCodePageDecoder.CodePagesData = new Dictionary<int, string>();
            foreach (int codePage in RtfCodePageDecoder.CodePages)
            {
                RtfCodePageDecoder.CodePagesData[codePage] = ReadCodePage(codePage);
            }
        }

        public RtfCodePageDecoder(int codePage)
        {
            this.CurrentCodePage = codePage;
        }

        public int CurrentCodePage
        {
            get
            {
                return this.currentCodePage;
            }
            set
            {
                this.currentCodePage = value;

                if (!RtfCodePageDecoder.CodePagesData.TryGetValue(this.currentCodePage, out this.codePageData))
                {
                    this.codePageData = CodePagesData[CharSetHelper.AnsiCodePage];
                }
            }
        }

        public char Convert(int b)
        {
            if (this.codePageData.Length <= b)
            {
                return '?';
            }
            return this.codePageData[b];
        }

        private static string ReadCodePage(int codePage)
        {
            Assembly currentAssembly = Assembly.GetExecutingAssembly();
            string resourcePathFormat = "{0}.{1}{2}";

            string resourcePath = string.Format(resourcePathFormat, MetafilesRelativePath, codePage, MetafilesExtension);

            string[] names = currentAssembly.GetManifestResourceNames();

            string resourceName = names.Where(p => p.Contains(resourcePath)).Single();

            using (Stream stream = currentAssembly.GetManifestResourceStream(resourceName))
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }
    }
}