using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Exceptions;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Parser
{
    internal sealed class RtfParserListenerStructureBuilder : RtfParserListenerBase
    {
        private readonly Stack<RtfGroup> openGroupStack = new Stack<RtfGroup>();
        private RtfGroup curGroup;
        private RtfGroup structureRoot;

        public RtfGroup StructureRoot
        {
            get { return this.structureRoot; }
        }

        protected override void DoParseBegin()
        {
            this.openGroupStack.Clear();
            this.curGroup = null;
            this.structureRoot = null;
        }

        protected override void DoGroupBegin()
        {
            RtfGroup newGroup = new RtfGroup();
            if (this.curGroup != null)
            {
                this.openGroupStack.Push(this.curGroup);
                this.curGroup.Elements.Add(newGroup);
            }
            this.curGroup = newGroup;
        }

        protected override void DoTagFound(RtfTag tag)
        {
            if (this.curGroup == null)
            {
                throw new RtfStructureException("MissingGroupForNewTag ");
            }
            this.curGroup.Elements.Add(tag);
        }

        protected override void DoTextFound(RtfText text)
        {
            if (this.curGroup == null)
            {
                throw new RtfStructureException("MissingGroupForNewText");
            }
            this.curGroup.Elements.Add(text);
        }

        protected override void DoBinaryFound(RtfBinary bin)
        {
            if (this.curGroup == null)
            {
                throw new RtfStructureException("MissingGroupForNewText");
            }
            this.curGroup.Elements.Add(bin);
        }

        protected override void DoGroupEnd()
        {
            if (this.openGroupStack.Count > 0)
            {
                this.curGroup = this.openGroupStack.Pop();
            }
            else
            {
                if (this.structureRoot != null)
                {
                    throw new RtfStructureException("MultipleRootLevelGroups ");
                }
                this.structureRoot = this.curGroup;
                this.curGroup = null;
            }
        }

        protected override void DoParseEnd()
        {
            if (this.openGroupStack.Count > 0)
            {
                throw new RtfBraceNestingException("UnclosedGroups");
            }
        }
    }
}