﻿using System;
using System.IO;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Parser
{
    internal class RtfReader : IDisposable
    {
        private readonly Stream stream;

        private int buffer;
        private bool usingBuffer = false;

        public RtfReader(Stream stream)
        {
            this.stream = stream;
            this.CodePageDecoder = new RtfCodePageDecoder(CharSetHelper.AnsiCodePage);
        }

        public RtfCodePageDecoder CodePageDecoder { get; private set; }

        public int Peek()
        {
            if (this.usingBuffer)
            {
                return this.buffer;
            }
            else
            {
                this.buffer = this.Read();
                this.usingBuffer = true;
                return this.buffer;
            }
        }

        public int Read()
        {
            if (this.usingBuffer)
            {
                this.usingBuffer = false;
                return this.buffer;
            }
            else
            {
                int b = this.stream.ReadByte();
                if (b == -1)
                {
                    return -1;
                }

                return this.CodePageDecoder.Convert(b);
            }
        }

        public byte[] ReadBinary(int count)
        {
            byte[] result = new byte[count];

            int readResult = 0;
            if (this.usingBuffer)
            {
                result[0] = (byte)this.buffer;
                this.usingBuffer = false;

                --count;
                readResult = this.stream.Read(result, 1, count);
            }
            else
            {
                readResult = this.stream.Read(result, 0, count);
            }

            if (readResult < count)
            {
                throw new EndOfStreamException("Unexpected end of stream.");
            }

            return result;
        }

        public void Close()
        {
            this.Dispose();
        }

        public void Dispose()
        {
            this.stream.Dispose();
        }
    }
}