﻿using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal static class RedundantPropertiesProcessor
    {
        public static void ClearRedundantLocalProperties(RadFlowDocument document)
        {
            ClearRedundantLocalPropertiesRecursive(document);

            foreach (List list in document.Lists)
            {
                foreach (ListLevel listLevel in list.Levels)
                {
                    ClearRedundantLocalProperties(listLevel.CharacterProperties);
                    ClearRedundantLocalProperties(listLevel.ParagraphProperties);
                }
            }
        }

        private static void ClearRedundantLocalPropertiesRecursive(DocumentElementBase element)
        {
            foreach (DocumentElementBase child in element.Children)
            {
                ClearRedundantLocalPropertiesRecursive(child);
            }

            Run run = element as Run;
            if (run != null)
            {
                ClearRedundantLocalProperties(run.Properties);
                return;
            }

            Paragraph par = element as Paragraph;
            if (par != null)
            {
                ClearRedundantLocalProperties(par.Properties);
                return;
            }

            TableCell cell = element as TableCell;
            if (cell != null)
            {
                ClearRedundantLocalProperties(cell.Properties);
                return;
            }

            TableRow row = element as TableRow;
            if (row != null)
            {
                ClearRedundantLocalProperties(row.Properties);
                return;
            }

            Table table = element as Table;
            if (table != null)
            {
                ClearRedundantLocalProperties(table.Properties);
                return;
            }

            Section section = element as Section;
            if (section != null)
            {
                HeaderFooterBase headerFooter;

                headerFooter = section.Headers.First;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }

                headerFooter = section.Headers.Default;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }

                headerFooter = section.Headers.Even;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }

                headerFooter = section.Footers.First;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }

                headerFooter = section.Footers.Default;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }

                headerFooter = section.Footers.Even;
                if (headerFooter != null)
                {
                    ClearRedundantLocalPropertiesRecursive(headerFooter);
                }
            }
        }

        private static void ClearRedundantLocalProperties(DocumentElementPropertiesBase documentElementProperties)
        {
            foreach (Flow.Model.Styles.Core.IStyleProperty prop in documentElementProperties.StyleProperties)
            {
                if (prop.HasLocalValue)
                {
                    object localValue = prop.GetActualValueAsObject();
                    if (localValue != null)
                    {
                        prop.ClearValue();
                        object nonLocalValue = prop.GetActualValueAsObject();
                        if (!localValue.Equals(nonLocalValue))
                        {
                            prop.SetValueAsObject(localValue);
                        }
                    }
                }
            }
        }

        ////private void ClearRedundantTableCellBorders(TableCell cell)
        ////{
        ////    Table table = cell.Row.Table;

        ////    var bordersValueSource = cell.GetPropertyValueSource(TableCell.BordersProperty);
        ////    var tableAllBorder = table.Borders.All;
        ////    if (bordersValueSource == RadValueSource.Local && tableAllBorder != null)
        ////    {
        ////        Border cellAllBorder = null;
        ////        if (cell.Borders.Top != null && cell.Borders.Top.Equals(cell.Borders.Bottom) &&
        ////            cell.Borders.Top.Equals(cell.Borders.Left) &&
        ////            cell.Borders.Top.Equals(cell.Borders.Right))
        ////        {
        ////            cellAllBorder = cell.Borders.Top;
        ////        }

        ////        if (cellAllBorder != null)
        ////        {
        ////            //fix for all borders at once
        ////            if (tableAllBorder == cellAllBorder)
        ////            {
        ////                cell.ClearValue(TableCell.BordersProperty);
        ////            }
        ////        }
        ////        else
        ////        {
        ////            //fix for individual borders
        ////            if (cell.Borders.Left == tableAllBorder)
        ////            {
        ////                cell.Borders = cell.Borders.SetLeft(TableCellBorders.DefaultBorder);
        ////            }
        ////            if (cell.Borders.Right == tableAllBorder)
        ////            {
        ////                cell.Borders = cell.Borders.SetRight(TableCellBorders.DefaultBorder);
        ////            }
        ////            if (cell.Borders.Top == tableAllBorder)
        ////            {
        ////                cell.Borders = cell.Borders.SetTop(TableCellBorders.DefaultBorder);
        ////            }
        ////            if (cell.Borders.Bottom == tableAllBorder)
        ////            {
        ////                cell.Borders = cell.Borders.SetBottom(TableCellBorders.DefaultBorder);
        ////            }
        ////        }
        ////    }
        ////}

        ////private void ClearRedundantLocalParagraphIndents(Paragraph paragraph)
        ////{
        ////    if (!paragraph.IsInList)
        ////    {
        ////        return;
        ////    }

        ////    DocumentList currentList = this.Document.ListManager.GetDocumentListById(paragraph.ListId);
        ////    if (currentList == null)
        ////    {
        ////        return;
        ////    }
        ////    ListStyle listStyle = currentList.ActualStyle;
        ////    if (listStyle == null)
        ////    {
        ////        return;
        ////    }

        ////    if (listStyle.Levels.Count <= paragraph.ListLevel)
        ////    {
        ////        return;
        ////    }

        ////    // The left indent and hanging indent can be evaluated trough style
        ////    // We should clear the local property values in this case
        ////    ListLevelStyle lvlStyle = listStyle.Levels[paragraph.ListLevel];
        ////    if (lvlStyle.Indent == paragraph.LeftIndent)
        ////    {
        ////        paragraph.ClearValue(Paragraph.LeftIndentProperty);
        ////    }

        ////    if (lvlStyle.HangingIndent == paragraph.HangingIndent)
        ////    {
        ////        paragraph.ClearValue(Paragraph.FirstLineIndentProperty);
        ////    }
        ////}
    }
}
