﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal static class ReplaceStyleNameHandler
    {
        private static Dictionary<string, BuiltInStyleInfo> styleNameToStyleId;

        private static Dictionary<string, BuiltInStyleInfo> StyleNameToStyleId
        {
            get
            {
                if (styleNameToStyleId == null)
                {
                    InitializeStyleNameToStyleIdDictionary();
                }

                return styleNameToStyleId;
            }
        }

        public static void Replace(ReplaceStyleNameContext context)
        {
            BuiltInStyleInfo defaultStyleInfo;
            if (TryGetStyleIdByStyleName(context.InputStyleName, out defaultStyleInfo))
            {
                context.StyleDefinition.Id = defaultStyleInfo.Id;
                context.StyleDefinition.Name = defaultStyleInfo.Name;
            }
            else
            {
                context.StyleDefinition.Id = context.InputStyleName.Replace(" ", string.Empty);
                context.StyleDefinition.Name = context.InputStyleName;
            }
        }

        private static bool TryGetStyleIdByStyleName(string name, out BuiltInStyleInfo defaultStyleInfo)
        {
            return StyleNameToStyleId.TryGetValue(name, out defaultStyleInfo);
        }

        private static void InitializeStyleNameToStyleIdDictionary()
        {
            styleNameToStyleId = new Dictionary<string, BuiltInStyleInfo>(StringComparer.OrdinalIgnoreCase);

            Add(BuiltInStyleNames.NormalTableStyleName, BuiltInStyleNames.NormalTableStyleId);
            Add(BuiltInStyleNames.TableGridStyleName, BuiltInStyleNames.TableGridStyleId);
            Add(BuiltInStyleNames.NormalStyleName, BuiltInStyleNames.NormalStyleId);
            Add(BuiltInStyleNames.NormalWebStyleName, BuiltInStyleNames.NormalWebStyleId);
            Add(BuiltInStyleNames.HyperlinkStyleName, BuiltInStyleNames.HyperlinkStyleId);
            Add(BuiltInStyleNames.CaptionStyleName, BuiltInStyleNames.CaptionStyleId);
            Add(BuiltInStyleNames.TofStyleName, BuiltInStyleNames.TofStyleId);
            Add(BuiltInStyleNames.FootnoteReferenceStyleName, BuiltInStyleNames.FootnoteReferenceStyleId);
            Add(BuiltInStyleNames.FootnoteTextStyleName, BuiltInStyleNames.FootnoteTextStyleId);
            Add(BuiltInStyleNames.FootnoteTextCharStyleName, BuiltInStyleNames.FootnoteTextCharStyleId);
            Add(BuiltInStyleNames.EndnoteReferenceStyleName, BuiltInStyleNames.EndnoteReferenceStyleId);
            Add(BuiltInStyleNames.EndnoteTextStyleName, BuiltInStyleNames.EndnoteTextStyleId);
            Add(BuiltInStyleNames.EndnoteTextCharStyleName, BuiltInStyleNames.EndnoteTextCharStyleId);

            for (int i = 1; i <= 9; i++)
            {
                Add(BuiltInStyleNames.GetHeadingStyleNameByIndex(i), BuiltInStyleNames.GetHeadingStyleIdByIndex(i));
                Add(BuiltInStyleNames.GetHeadingCharStyleNameByIndex(i), BuiltInStyleNames.GetHeadingCharStyleIdByIndex(i));
                Add(BuiltInStyleNames.GetTocStyleNameByIndex(i), BuiltInStyleNames.GetTocStyleIdByIndex(i));
            }
        }

        private static void Add(string name, string id)
        {
            styleNameToStyleId.Add(name, new BuiltInStyleInfo(id, name));
        }

        private class BuiltInStyleInfo
        {
            private readonly string id;
            private readonly string name;

            public BuiltInStyleInfo(string id, string name)
            {
                this.id = id;
                this.name = name;
            }

            public string Id
            {
                get
                {
                    return this.id;
                }
            }

            public string Name
            {
                get
                {
                    return this.name;
                }
            }
        }
    }
}