﻿using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Parser;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal class RtfDocumentImporter : RtfElementIteratorBase
    {
        private static readonly Dictionary<string, ControlTagHandler> TagHandlers;
        private static readonly Dictionary<string, ControlGroupHandler> GroupHandlers;

        private RtfImportContext context;

        static RtfDocumentImporter()
        {
            RtfDocumentImporter.TagHandlers = new Dictionary<string, ControlTagHandler>();
            RtfTagHandlers.InitializeAllTagHandlers(RtfDocumentImporter.TagHandlers);

            RtfDocumentImporter.GroupHandlers = new Dictionary<string, ControlGroupHandler>();
            GroupTagHandlers.InitializeGroupHandlers(RtfDocumentImporter.GroupHandlers);
        }

        public RtfDocumentImporter()
        {
        }

        public RtfDocumentImporter(RtfImportContext context)
        {
            this.context = context;
        }

        public RtfImportContext Context
        {
            get
            {
                return this.context;
            }
        }

        public RadFlowDocument Import(Stream input)
        {
            RtfSource source = new RtfSource(input);
            RtfParserListenerStructureBuilder structBuilder = new RtfParserListenerStructureBuilder();

            using (RtfParser parser = new RtfParser(structBuilder/*, new RtfDebugListener()*/))
            {
                parser.Parse(source);

                this.context = new RtfImportContext();

                // This will ensure that DocumentContentChanged event is not fired during the import
                ////this.context.Document.BeginInitialization();

                Util.EnsureGroupDestination(structBuilder.StructureRoot, RtfTags.TagRtf);
                this.ImportRoot(structBuilder.StructureRoot);

                ////this.context.UpdateRegisteredFields();
                ////this.context.Document.EndInitialization();

                this.context.ClearUnclosedBookmarkRanges();

                return this.context.Document;
            }
        }

        public void ImportRoot(RtfGroup rtfGroup)
        {
            this.VisitGroupChildren(rtfGroup, false);

            this.context.FlushSection();
            RedundantPropertiesProcessor.ClearRedundantLocalProperties(this.context.Document);
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            ControlGroupHandler handler = null;
            if (group.Destination != null)
            {
                RtfDocumentImporter.GroupHandlers.TryGetValue(group.Destination, out handler);
            }

            if (handler != null)
            {
                this.context.EnterGroup(group);

                bool shoudVisitChildren = handler(group, this.context);
                if (shoudVisitChildren)
                {
                    this.VisitGroupChildren(group, false);
                }

                this.context.LeaveGroup(group);
            }
            else
            {
                if (!group.IsExtensionDestination)
                {
                    this.context.EnterGroup(group);

                    this.VisitGroupChildren(group, false);

                    this.context.LeaveGroup(group);
                }
                else
                {
                    Debug.WriteLine(string.Format("Unrecognized extension group: [{0}] which will be ignored", group.Destination));
                }
            }
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            ControlTagHandler handler;
            RtfDocumentImporter.TagHandlers.TryGetValue(tag.Name, out handler);

            if (handler != null)
            {
                handler(tag, this.context);
            }
            ////else
            ////{
            ////    Debug.WriteLine(string.Format("Unrecognized tag: [{0}] which will be ignored", tag.Name));
            ////}
        }

        protected override void DoVisitText(RtfText text)
        {
            this.context.AddText(text.Text);
        }
    }
}