﻿using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal abstract class RtfElementIteratorBase
    {
        protected void VisitGroupChildren(RtfGroup group, bool recursive)
        {
            foreach (RtfElement element in group.Elements)
            {
                this.VisitElement(element, recursive);
            }
        }

        protected void VisitElement(RtfElement element, bool recursive)
        {
            switch (element.Type)
            {
                case RtfElementType.Tag:
                    this.DoVisitTag((RtfTag)element);
                    break;

                case RtfElementType.Group:
                    RtfGroup group = (RtfGroup)element;
                    this.DoVisitGroup(group);
                    if (recursive)
                    {
                        this.VisitGroupChildren(group, recursive);
                    }
                    break;

                case RtfElementType.Text:
                    this.DoVisitText((RtfText)element);
                    break;

                case RtfElementType.Binary:
                    this.DoVisitBinary((RtfBinary)element);
                    break;

                default:
                    break;
            }
        }

        protected virtual void DoVisitGroup(RtfGroup group)
        {
            // Do nothing.
        }

        protected virtual void DoVisitTag(RtfTag tag)
        {
            // Do nothing.
        }

        protected virtual void DoVisitText(RtfText text)
        {
            // Do nothing.
        }

        protected virtual void DoVisitBinary(RtfBinary bin)
        {
            // Do nothing.
        }
    }
}