using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal class RtfStyleImporter : RtfElementIteratorBase
    {
        private static readonly Dictionary<string, ControlTagHandler> TagHandlers;

        private readonly RtfImportContext context;
        private readonly Style style;

        static RtfStyleImporter()
        {
            RtfStyleImporter.TagHandlers = new Dictionary<string, ControlTagHandler>();

            SpanStyleHandlers.InitializeSpanStyleHandlers(RtfStyleImporter.TagHandlers);
            ParagraphStyleHandlers.InitializeParagraphStyleHandlers(RtfStyleImporter.TagHandlers);
            StyleSheetHandlers.InitializeStyleDefinitionHandlers(RtfStyleImporter.TagHandlers);
            TableStyleHandlers.InitializeTableStyleHandlers(RtfStyleImporter.TagHandlers);
        }

        public RtfStyleImporter(RtfImportContext context, StyleType type)
        {
            this.context = context;

            // TODO: fix this
            this.style = new Style("TempName", type);
        }

        public RtfStyleDefinitionInfo ImportStyleGroup(RtfGroup group)
        {
            this.VisitGroupChildren(group, true);

            RtfStyleDefinitionInfo styleInfo = this.context.CurrentStyle.StyleDefinitionInfo;
            styleInfo.ImportedSpanProperties = new Run(this.context.Document);
            this.context.CurrentStyle.ApplyCharacterStyle(styleInfo.ImportedSpanProperties);

            styleInfo.ImportedParagraphProperties = new Paragraph(this.context.Document);
            this.context.CurrentStyle.ApplyParagraphStyle(styleInfo.ImportedParagraphProperties);

            if (this.context.CurrentRowStyle != null)
            {
                // Table
                Table importedTableStyle = new Table(this.context.Document);
                this.context.CurrentRowStyle.ApplyTableStyle(importedTableStyle);
                styleInfo.ImportedTableProperties = importedTableStyle;

                // Row
                TableRow importedRowStyle = new TableRow(this.context.Document);
                this.context.CurrentRowStyle.ApplyRowStyle(importedRowStyle);
                styleInfo.ImportedTableRowProperties = importedRowStyle;

                // Cell
                TableCell importedCellStyle = new TableCell(this.context.Document);
                if (this.context.CurrentRowStyle.CurrentCellStyle != null)
                {
                    this.context.CurrentRowStyle.CurrentCellStyle.ApplyCellStyle(importedCellStyle);
                }
                styleInfo.ImportedTableCellProperties = importedCellStyle;
            }

            this.style.IsPrimary = styleInfo.IsPrimary;

            if (styleInfo.UIPriority.HasValue)
            {
                this.style.UIPriority = styleInfo.UIPriority.Value;
            }

            styleInfo.CurrentStyle = this.style;

            return styleInfo;
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            ControlTagHandler handler;
            RtfStyleImporter.TagHandlers.TryGetValue(tag.Name, out handler);

            if (handler != null)
            {
                handler(tag, this.context);
            }
        }

        protected override void DoVisitText(RtfText text)
        {
            string dispalyName = text.Text.Trim().TrimEnd(';');
            ReplaceStyleNameContext context = new ReplaceStyleNameContext() { InputStyleName = dispalyName, StyleDefinition = this.style };
            ReplaceStyleNameHandler.Replace(context);
        }
    }
}