﻿using System;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Borders
{
    internal class RtfBorder
    {
        private float thickness;
        private BorderStyle borderStyle;
        private Color color;
        private float thicknessModifier;

        public RtfBorder(bool isNone = false)
        {
            if (isNone)
            {
                this.thickness = 0;
                this.thicknessModifier = 1f;
                this.borderStyle = Border.EmptyBorder.Style;
                this.color = Border.EmptyBorder.Color.LocalValue;
            }
            else
            {
                this.thickness = 0;
                this.thicknessModifier = 1f;
                this.borderStyle = BorderStyle.Inherit;
                this.color = Border.DefaultBorder.Color.LocalValue;
            }
        }

        public bool HasValue { get; set; }

        public void HandleTag(RtfTag tag, RtfImportContext context)
        {
            switch (tag.Name)
            {
                case RtfTags.TagBorderThickness:
                    this.thickness = Math.Max(Unit.TwipToDipF(tag.ValueAsNumber), 1f);
                    break;
                case RtfTags.TagBorderTypeDoubleThickness:
                    this.thicknessModifier = 2f;
                    this.borderStyle = BorderStyle.Single;
                    break;
                case RtfTags.TagBorderColor:
                    RtfColor rtfColor;
                    if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
                    {
                        this.color = rtfColor.Color;
                    }
                    break;
                default:
                    this.borderStyle = RtfHelper.BorderMapper.GetToValue(tag.Name);
                    break;
            }
        }

        public Border CreateBorder()
        {
            return new Border(
                this.thickness * this.thicknessModifier,
                this.borderStyle,
                new ThemableColor(this.color));
        }

        public RtfBorder CreateCopy()
        {
            RtfBorder result = new RtfBorder();
            if (this.HasValue)
            {
                result.HasValue = true;
                result.thickness = this.thickness;
                result.borderStyle = this.borderStyle;
                result.color = this.color;
                result.thicknessModifier = this.thicknessModifier;
            }
            return result;
        }
    }
}