﻿using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Fonts
{
    internal sealed class RtfFontTable : RtfElementIteratorBase
    {
        private readonly Dictionary<string, RtfFont> fontByIdMap = new Dictionary<string, RtfFont>();

        public bool ContainsFontWithId(string fontId)
        {
            return this.fontByIdMap.ContainsKey(fontId);
        }

        public FontFamily GetFontFamily(string fontId)
        {
            RtfFont font = this.fontByIdMap[fontId];
            string fontName = font.Name;

            if (string.IsNullOrEmpty(fontName))
            {
                fontName = font.NameAlt;
            }

            FontFamily result = new FontFamily(fontName);

            return result;
        }

        public void FillFontTable(RtfGroup group)
        {
            Util.EnsureGroupDestination(group, RtfTags.TagFontTable);

            if (group.Elements.Count > 1)
            {
                if (group.Elements.Any(e => e.Type == RtfElementType.Group))
                {
                    // the 'new' style where each font resides in a group of its own
                    this.VisitGroupChildren(group, false);
                }
                else
                {
                    // the 'old' style where individual fonts are 'terminated' by their
                    // respective name content text (which ends with ';')
                    // -> need to manually iterate from here
                    int childCount = group.Elements.Count;
                    RtfGroup tmpGroup = new RtfGroup();

                    // skip over the initial \fonttbl tag
                    for (int i = 1; i < childCount; i++)
                    {
                        RtfElement element = group.Elements[i];

                        if (element.Type == RtfElementType.Text &&
                            ((RtfText)element).Text == RtfTags.TagDelimiter)
                        {
                            this.BuildFontFromGroup(tmpGroup);
                        }

                        tmpGroup = new RtfGroup();
                    }
                }
            }
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            switch (group.Destination)
            {
                case RtfTags.TagFont:
                case RtfTags.TagThemeFontLoMajor:
                case RtfTags.TagThemeFontHiMajor:
                case RtfTags.TagThemeFontDbMajor:
                case RtfTags.TagThemeFontBiMajor:
                case RtfTags.TagThemeFontLoMinor:
                case RtfTags.TagThemeFontHiMinor:
                case RtfTags.TagThemeFontDbMinor:
                case RtfTags.TagThemeFontBiMinor:
                    this.BuildFontFromGroup(group);
                    break;
            }
        }

        private void BuildFontFromGroup(RtfGroup group)
        {
            RtfFont font = new RtfFont();
            font.FillFont(group);
            this.Add(font);
        }

        private void Add(RtfFont font)
        {
            if (font == null)
            {
                throw new ArgumentNullException("font");
            }

            this.fontByIdMap[font.Id] = font;
        }
    }
}