﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Images
{
    internal abstract class RtfImageBase
    {
        public RtfImageBase()
        {
            this.Properties = new Dictionary<string, RtfGroup>();
        }

        public Dictionary<string, RtfGroup> Properties { get; private set; }

        public void ReadInstructionGroup(RtfGroup group)
        {
            RtfGroup nameGroup = group.Elements.FirstOrDefault(
                e =>
                    e.Type == RtfElementType.Group &&
                    ((RtfGroup)e).Destination == RtfTags.TagShapePopertyNameDestenation) as RtfGroup;

            RtfGroup valueGroup = group.Elements.FirstOrDefault(
                e =>
                    e.Type == RtfElementType.Group &&
                    ((RtfGroup)e).Destination == RtfTags.TagShapePopertyValueDestenation) as RtfGroup;

            if (nameGroup == null || valueGroup == null)
            {
                return;
            }

            RtfText name = nameGroup.Elements.FirstOrDefault(e => e.Type == RtfElementType.Text) as RtfText;
            if (name == null)
            {
                return;
            }

            this.Properties[name.Text] = valueGroup;
        }

        public string GetPropertyStringValue(string propertyName)
        {
            return this.GetPropertyStringValue(propertyName, string.Empty);
        }

        public string GetPropertyStringValue(string propertyName, string defalutValue)
        {
            string result = defalutValue;
            RtfGroup group = null;
            this.Properties.TryGetValue(propertyName, out group);

            if (group != null && group.Elements.Count > 1 && group.Elements[1] is RtfText)
            {
                result = ((RtfText)group.Elements[1]).Text;
            }

            return result;
        }

        public int GetPropertyIntValue(string propertyName, int defaultValue)
        {
            int result;

            string strResult = this.GetPropertyStringValue(propertyName, string.Empty);
            if (!int.TryParse(strResult, out result))
            {
                result = defaultValue;
            }

            return result;
        }

        public bool GetPropertyBoolValue(string propertyName, bool defalutValue)
        {
            bool result = defalutValue;

            string strResult = this.GetPropertyStringValue(propertyName, string.Empty);
            if (!string.IsNullOrEmpty(strResult))
            {
                result = strResult == "1";
            }

            return result;
        }
    }
}