﻿using System.Collections.Generic;
using System.Diagnostics;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Lists
{
    internal class RtfList : RtfElementIteratorBase
    {
        private readonly Dictionary<int, string> listIdToStyleIds;
        private List list;
        private RtfImportContext context;

        // TODO: ListIdToStyles
        public RtfList()
        {
            this.ListLevelInfos = new List<ListLevelInfo>();
            this.listIdToStyleIds = new Dictionary<int, string>();
        }

        public int ListId { get; private set; }
        public int? ListTemplateId { get; private set; }
        public bool IsSingleLevel { get; private set; }
        public bool ListHybrid { get; private set; }
        public int? ListStyleId { get; private set; }
        public string ListStyleName { get; private set; }
        public List<ListLevelInfo> ListLevelInfos { get; private set; }

        public List List
        {
            get
            {
                return this.list;
            }
        }

        public void ReadList(RtfGroup group, RtfImportContext context)
        {
            Util.EnsureGroupDestination(group, RtfTags.TagList);

            this.context = context;

            this.VisitGroupChildren(group, false);

            this.CreateList(context);
        }

        public List GetCreatedStyledList(RtfImportContext context)
        {
            Style styleDef = null;
            if (!string.IsNullOrEmpty(this.list.StyleId))
            {
                styleDef = context.Document.StyleRepository.GetStyle(this.list.StyleId);
                if (styleDef == null)
                {
                    styleDef = new Style(this.list.StyleId, StyleType.Numbering);
                    context.Document.StyleRepository.Add(styleDef);
                }
                else if (styleDef.StyleType != StyleType.Numbering)
                {
                    Debug.Assert(false, "Style name for numbering style already taken.");
                    this.list.StyleId = null;
                    styleDef = null;
                }
            }

            if (styleDef != null)
            {
                // Connect the StyleDefinition to the document list;
                styleDef.ParagraphProperties.ListId.LocalValue = this.list.Id;
            }

            return this.list;
        }

        public void CreateList(RtfImportContext context)
        {
            this.list = context.Document.Lists.Add(ListTemplateType.NumberedDefault);

            this.list.MultilevelType = this.GetMultilevelType();

            context.CurrentStyle.CurrentList = this.list;

            if (this.ListLevelInfos.Count == 0)
            {
                return;
            }

            if (this.ListStyleName != null)
            {
                this.listIdToStyleIds.Add(this.ListId, this.ListStyleName);
                context.Document.Lists.Remove(this.list);
            }

            if (this.list.MultilevelType == MultilevelType.SingleLevel)
            {
                this.list.Levels[0] = this.ListLevelInfos[0].CreateListLevel(context, 0);
            }
            else
            {
                for (int i = 0; i < this.list.Levels.Count && i < this.ListLevelInfos.Count; i++)
                {
                    this.list.Levels[i] = this.ListLevelInfos[i].CreateListLevel(context, i);
                }
            }
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            switch (tag.Name)
            {
                case RtfTags.TagListId:
                    this.ListId = tag.ValueAsNumber;
                    break;

                case RtfTags.TagListTemplateId:
                    if (tag.ValueAsNumber != -1)
                    {
                        this.ListTemplateId = tag.ValueAsNumber;
                    }
                    break;

                case RtfTags.TagListSingleLevel:
                    this.IsSingleLevel = tag.ValueAsNumber != 0 ? true : false;
                    break;

                case RtfTags.TagListIsHybrid:
                    this.ListHybrid = true;
                    break;

                case RtfTags.TagListStyleId:
                    this.ListStyleId = tag.ValueAsNumber;
                    break;
            }
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            if (group.Destination == RtfTags.TagListLevel)
            {
                ListLevelInfo info = new ListLevelInfo();
                info.ReadListLevelInfo(group, this.context);
                this.ListLevelInfos.Add(info);
            }
            if (group.Destination == RtfTags.TagListStyleName)
            {
                this.ListStyleName = RtfHelper.GetGroupText(group).Trim().TrimEnd(';');
            }
        }

        private MultilevelType GetMultilevelType()
        {
            if (this.IsSingleLevel)
            {
                return MultilevelType.SingleLevel;
            }
            else if (this.ListHybrid)
            {
                return MultilevelType.HybridMultilevel;
            }

            return MultilevelType.Multilevel;
        }
    }
}