﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Lists;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Lists
{
    internal class RtfListOverride : RtfElementIteratorBase
    {
        private bool shouldCreateOverridenList;
        private List<ListLevelInfo> listLevelInfos;

        public RtfListOverride()
        {
        }

        public RtfImportContext Context { get; set; }

        public int ListOverrideId { get; set; }

        public int ListId { get; set; }

        public int ListLevelIndexToOverrideCount { get; set; }

        public int? ListOverrideStartAtIndex { get; set; }

        public int? ListOverrideFormat { get; set; }

        public void ReadListOverride(RtfGroup group)
        {
            Util.EnsureGroupDestination(group, RtfTags.TagListOverride);

            this.listLevelInfos = new List<ListLevelInfo>();

            this.VisitGroupChildren(group, false);
        }

        public List CreateListOverride(RtfImportContext context)
        {
            RtfList rtfList = context.ListTable.GetRtfList(this.ListId);

            List result;
            if (this.shouldCreateOverridenList)
            {
                rtfList.CreateList(context);
            }
            result = rtfList.GetCreatedStyledList(context);

            if (this.ListOverrideStartAtIndex != null && this.ListOverrideStartAtIndex != 0)
            {
                for (int i = 0; i < this.ListLevelIndexToOverrideCount; i++)
                {
                    result.Levels[i].StartIndex = this.ListOverrideStartAtIndex.Value;
                }
            }

            if (this.ListLevelIndexToOverrideCount <= this.listLevelInfos.Count)
            {
                ListLevel listLevel;
                for (int i = 0; i < this.ListLevelIndexToOverrideCount; i++)
                {
                    listLevel = result.Levels[i];
                    listLevel.Alignment = this.listLevelInfos[i].Alignment;
                    listLevel.NumberingStyle = this.listLevelInfos[i].NumberingStyle;
                    listLevel.NumberTextFormat = this.listLevelInfos[i].LevelText;
                    listLevel.StartIndex = this.listLevelInfos[i].LevelStartIndex;
                    listLevel.IsLegal = this.listLevelInfos[i].IsLegal;

                    if (this.listLevelInfos[i].StyleId != null)
                    {
                        listLevel.StyleId = this.Context.StylesTable.GetStyleById(this.listLevelInfos[i].StyleId.Value).Id;
                    }
                }
            }

            this.shouldCreateOverridenList = false;
            return result;
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            switch (tag.Name)
            {
                case RtfTags.TagListId:
                    this.ListId = tag.ValueAsNumber;
                    break;

                case RtfTags.TagListOverrideId:
                    this.ListOverrideId = tag.ValueAsNumber;
                    break;

                case RtfTags.TagListOverrideCount:
                    this.ListLevelIndexToOverrideCount = tag.ValueAsNumber;
                    break;

                case RtfTags.TagListLevelStartIndex:
                    this.ListOverrideStartAtIndex = tag.ValueAsNumber;
                    break;

                case RtfTags.TagListOverrideFormat:
                    this.ListOverrideFormat = tag.ValueAsNumber;
                    break;
            }
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            switch (group.Destination)
            {
                case RtfTags.TagListOverrideLevelDestination:
                    this.shouldCreateOverridenList = true;
                    this.VisitGroupChildren(group, false);
                    break;

                case RtfTags.TagListLevel:
                    ListLevelInfo info = new ListLevelInfo();
                    info.ReadListLevelInfo(group, this.Context);
                    this.listLevelInfos.Add(info);
                    break;
            }
        }
    }
}