﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Lists
{
    internal class RtfListTable : RtfElementIteratorBase
    {
        private Dictionary<int, RtfList> listDictionary;
        private RtfImportContext context;

        public RtfListTable()
        {
        }

        public bool IsLoaded { get; set; }

        public void ReadTable(RtfGroup group, RtfImportContext context)
        {
            if (!this.IsLoaded)
            {
                this.context = context;

                Util.EnsureGroupDestination(group, RtfTags.TagListTable);

                this.listDictionary = new Dictionary<int, RtfList>();
                this.VisitGroupChildren(group, false);

                this.LinkNumberingStyles();

                this.IsLoaded = true;
                this.context = null;
            }
        }

        public RtfList GetRtfList(int id)
        {
            RtfList result = null;

            this.listDictionary.TryGetValue(id, out result);

            return result;
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            if (group.Destination == RtfTags.TagList)
            {
                RtfList list = new RtfList();
                list.ReadList(group, this.context);

                this.listDictionary[list.ListId] = list;
            }
        }

        private void LinkNumberingStyles()
        {
            foreach (RtfList rtfList in this.listDictionary.Values)
            {
                if (!string.IsNullOrEmpty(rtfList.ListStyleName))
                {
                    rtfList.List.StyleId = rtfList.ListStyleName;
                }
                else if (rtfList.ListStyleId.HasValue)
                {
                    RtfList actualRtfList;
                    if (this.listDictionary.TryGetValue(rtfList.ListStyleId.Value, out actualRtfList))
                    {
                        rtfList.List.StyleId = actualRtfList.ListStyleName;
                    }
                }
            }
        }
    }
}