﻿using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Exceptions;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes
{
    internal sealed class RtfColorTable : RtfElementIteratorBase
    {
        private readonly List<RtfColor> innerList = new List<RtfColor>();

        private int currentRed = 0;
        private int currentGreen = 0;
        private int currentBlue = 0;

        private bool isColorDefined = false;

        public int Count
        {
            get
            {
                return this.innerList.Count;
            }
        }

        public RtfColor this[int index]
        {
            get
            {
                return this.innerList[index];
            }
        }

        public bool TryGetColor(int index, out RtfColor rtfColor)
        {
            rtfColor = null;
            if (this.innerList.Count > index && index >= 0)
            {
                rtfColor = this.innerList[index];
                return true;
            }

            return false;
        }

        public void FillColorTable(RtfGroup group)
        {
            Util.EnsureGroupDestination(group, RtfTags.TagColorTable);

            this.ResetState();

            this.VisitGroupChildren(group, false);
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            switch (tag.Name)
            {
                case RtfTags.TagColorRed:
                    this.currentRed = tag.ValueAsNumber;
                    this.isColorDefined = true;
                    break;
                case RtfTags.TagColorGreen:
                    this.currentGreen = tag.ValueAsNumber;
                    this.isColorDefined = true;
                    break;
                case RtfTags.TagColorBlue:
                    this.currentBlue = tag.ValueAsNumber;
                    this.isColorDefined = true;
                    break;
            }
        }

        protected override void DoVisitText(RtfText text)
        {
            if (text.Text.StartsWith(RtfTags.TagDelimiter))
            {
                int count = text.Text.Count(c => c == RtfTags.TagDelimiterChar);
                for (int i = 0; i < count; i++)
                {
                    this.Add(Color.FromArgb(255, (byte)this.currentRed, (byte)this.currentGreen, (byte)this.currentBlue));
                    this.ResetState();
                }
            }
            else
            {
                throw new RtfColorTableFormatException("ColorTableUnsupportedText: " + text.Text);
            }
        }

        private void Add(Color item)
        {
            Guard.ThrowExceptionIfNull(item, "item");

            this.innerList.Add(new RtfColor(item, !this.isColorDefined));
        }

        private void ResetState()
        {
            this.currentRed = 0;
            this.currentGreen = 0;
            this.currentBlue = 0;
            this.isColorDefined = false;
        }
    }
}