﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes
{
    internal sealed class RtfDocumentInfo : RtfElementIteratorBase
    {
        private readonly RtfTimestampBuilder timestampBuilder = new RtfTimestampBuilder();

        public int? Id { get; set; }

        public int? Version { get; set; }

        public int? Revision { get; set; }

        public string Title { get; set; }

        public string Subject { get; set; }

        public string Author { get; set; }

        public string Manager { get; set; }

        public string Company { get; set; }

        public string Operator { get; set; }

        public string Category { get; set; }

        public string Keywords { get; set; }

        public string Comment { get; set; }

        public string DocumentComment { get; set; }

        public string HyperLinkbase { get; set; }

        public DateTime? CreationTime { get; set; }

        public DateTime? RevisionTime { get; set; }

        public DateTime? PrintTime { get; set; }

        public DateTime? BackupTime { get; set; }

        public int? NumberOfPages { get; set; }

        public int? NumberOfWords { get; set; }

        public int? NumberOfCharacters { get; set; }

        public int? EditingTimeInMinutes { get; set; }

        public void FillDocumentInfo(RtfGroup group)
        {
            Util.EnsureGroupDestination(group, RtfTags.TagInfo);

            this.VisitGroupChildren(group, false);
        }

        protected override void DoVisitGroup(RtfGroup group)
        {
            switch (group.Destination)
            {
                case RtfTags.TagInfoTitle:
                    this.Title = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoSubject:
                    this.Subject = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoAuthor:
                    this.Author = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoManager:
                    this.Manager = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoCompany:
                    this.Company = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoOperator:
                    this.Operator = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoCategory:
                    this.Category = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoKeywords:
                    this.Keywords = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoComment:
                    this.Comment = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoDocumentComment:
                    this.DocumentComment = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoHyperLinkBase:
                    this.HyperLinkbase = RtfHelper.GetGroupText(group);
                    break;
                case RtfTags.TagInfoCreationTime:
                    this.CreationTime = this.ExtractTimestamp(group);
                    break;
                case RtfTags.TagInfoRevisionTime:
                    this.RevisionTime = this.ExtractTimestamp(group);
                    break;
                case RtfTags.TagInfoPrintTime:
                    this.PrintTime = this.ExtractTimestamp(group);
                    break;
                case RtfTags.TagInfoBackupTime:
                    this.BackupTime = this.ExtractTimestamp(group);
                    break;
                default:
                    this.VisitGroupChildren(group, false);
                    break;
            }
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            switch (tag.Name)
            {
                case RtfTags.TagInfoVersion:
                    this.Version = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoRevision:
                    this.Revision = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoNumberOfPages:
                    this.NumberOfPages = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoNumberOfWords:
                    this.NumberOfWords = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoNumberOfChars:
                    this.NumberOfCharacters = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoId:
                    this.Id = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoEditingTimeMinutes:
                    this.EditingTimeInMinutes = tag.ValueAsNumber;
                    break;
            }
        }

        private DateTime ExtractTimestamp(RtfGroup group)
        {
            return this.timestampBuilder.CreateTimestamp(group);
        }
    }
}