﻿using System;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes
{
    internal sealed class RtfTimestampBuilder : RtfElementIteratorBase
    {
        private int year;
        private int month;
        private int day;
        private int hour;
        private int minutes;
        private int seconds;

        public DateTime CreateTimestamp(RtfGroup group)
        {
            this.Reset();

            this.VisitGroupChildren(group, false);

            DateTime result;
            try
            {
                result = new DateTime(this.year, this.month, this.day, this.hour, this.minutes, this.seconds);
            }
            catch (ArgumentException)
            {
                result = DateTime.MinValue;
            }

            return result;
        }

        public void Reset()
        {
            this.year = 0;
            this.month = 1;
            this.day = 1;
            this.hour = 0;
            this.minutes = 0;
            this.seconds = 0;
        }

        protected override void DoVisitTag(RtfTag tag)
        {
            switch (tag.Name)
            {
                case RtfTags.TagInfoYear:
                    this.year = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoMonth:
                    this.month = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoDay:
                    this.day = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoHour:
                    this.hour = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoMinute:
                    this.minutes = tag.ValueAsNumber;
                    break;
                case RtfTags.TagInfoSecond:
                    this.seconds = tag.ValueAsNumber;
                    break;
            }
        }
    }
}