﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Borders;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Primitives;

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal class TableCellStyle
    {
        private readonly TableRowStyle rowStyle;

        public TableCellStyle(TableRowStyle rowStyle)
        {
            this.rowStyle = rowStyle;

            this.ResetDefaults();
        }

        public bool CanWrapContent { get; set; }

        public bool IgnoreCellMarkerInRowHeight { get; set; }

        public VerticalAlignment VerticalAlignment { get; set; }

        public Alignment TextAlignment { get; set; }

        public RtfPadding Padding { get; set; }

        public Color? PatternColor { get; set; }

        public Color? BackgroundColor { get; set; }

        public ShadingPattern? Pattern { get; set; }

        public int CellXBoundry { get; set; }

        public bool IsFirstInHorizontalRange { get; set; }

        public bool IsInHorizontalRange { get; set; }

        public bool IsFirstInVerticalRange { get; set; }

        public bool IsInVerticalRange { get; set; }

        public bool IsLocked { get; set; }

        public RtfCellBorders Borders { get; set; }

        public int PreferredWidthValue { get; set; }

        public int PreferredWidthUnitType { get; set; }

        public TextDirection TextDirection { get; set; }

        public void ResetDefaults()
        {
            this.Padding = new RtfPadding();

            this.TextAlignment = Alignment.Left;
            this.VerticalAlignment = VerticalAlignment.Top;
            this.TextDirection = DocumentDefaultStyleSettings.TableCellTextDirection;
            this.PatternColor = null;
            this.BackgroundColor = null;
            this.Pattern = null;
            this.CellXBoundry = 0;
            this.Borders = new RtfCellBorders();
            this.CanWrapContent = true;
            this.IgnoreCellMarkerInRowHeight = false;
        }

        // TODO: Move this somewhere else
        public void ApplyCellStyle(TableCell cell)
        {
            cell.VerticalAlignment = this.VerticalAlignment;
            cell.Padding = this.CalculatePadding();
            if (this.PreferredWidthUnitType != 0)
            {
                cell.PreferredWidth = RtfHelper.ConvertTableWidthUnitType(this.PreferredWidthUnitType, this.PreferredWidthValue);
            }

            if (this.BackgroundColor.HasValue)
            {
                cell.Shading.BackgroundColor = new ThemableColor(this.BackgroundColor.Value);
            }

            if (this.PatternColor.HasValue)
            {
                cell.Shading.PatternColor = new ThemableColor(this.PatternColor.Value);
            }

            if (this.Pattern.HasValue)
            {
                cell.Shading.Pattern = this.Pattern.Value;
            }

            cell.TextDirection = this.TextDirection;

            cell.CanWrapContent = this.CanWrapContent;
            cell.IgnoreCellMarkerInRowHeightCalculation = this.IgnoreCellMarkerInRowHeight;

            TableCellBorders borders = this.Borders.CreateBorders();
            if (!DocumentDefaultStyleSettings.TableCellBorders.Equals(borders))
            {
                cell.Borders = borders;
            }
        }

        private static double GetPaddingValue(RtfPaddingValue cellPaddingValue, RtfPaddingValue rowPaddingValue, double rowDefault, double cellDefault)
        {
            if (cellPaddingValue.HasValue)
            {
                return (cellPaddingValue.UnitType == 0) ? rowDefault : Unit.TwipToDip(cellPaddingValue.Value);
            }
            else if (rowPaddingValue.HasValue)
            {
                return (rowPaddingValue.UnitType == 0) ? rowDefault : Unit.TwipToDip(rowPaddingValue.Value);
            }
            else
            {
                return cellDefault;
            }
        }

        private Padding CalculatePadding()
        {
            int rowDefault = (int)this.rowStyle.DefaultCellPadding;
            RtfPadding rowPadding = this.rowStyle.CellPadding;

            double left = GetPaddingValue(this.Padding.Left, rowPadding.Left, rowDefault, DocumentDefaultStyleSettings.TableCellPadding.Left);
            double top = GetPaddingValue(this.Padding.Top, rowPadding.Top, rowDefault, DocumentDefaultStyleSettings.TableCellPadding.Top);
            double right = GetPaddingValue(this.Padding.Right, rowPadding.Right, rowDefault, DocumentDefaultStyleSettings.TableCellPadding.Right);
            double bottom = GetPaddingValue(this.Padding.Bottom, rowPadding.Bottom, rowDefault, DocumentDefaultStyleSettings.TableCellPadding.Bottom);

            Padding result = new Padding(left, top, right, bottom);
            return result;
        }
    }
}