﻿using System;
using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Borders;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Primitives;

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import
{
    internal class TableRowStyle
    {
        public TableRowStyle()
        {
            this.CellStyles = new List<TableCellStyle>();
            this.Borders = new RtfRowBorders();
            this.CellPadding = new RtfPadding();
            this.DefaultCellSpacing = new RtfCellSpacing();
            this.AutoFit = TableLayoutType.FixedWidth;
            this.TableLooks = DocumentDefaultStyleSettings.TableLooks;
            this.CanSplit = true;
            this.RepeatOnEveryPage = false;
        }

        public TableRowHeight RowHeight { get; set; }

        public TableLooks TableLooks { get; set; }

        public int? ColumnBanding { get; set; }
        public int? RowBanding { get; set; }

        public int PreferredWidthValue { get; set; }

        public int PreferredWidthUnitType { get; set; }

        public double DefaultCellPadding { get; set; }

        public RtfPadding CellPadding { get; set; }

        public RtfCellSpacing DefaultCellSpacing { get; set; }

        public List<TableCellStyle> CellStyles { get; set; }

        public RtfRowBorders Borders { get; set; }

        public TableLayoutType AutoFit { get; set; }

        public FlowDirection FlowDirection { get; set; }

        public int TableIndentValue { get; set; }

        public int TableIndentType { get; set; }

        public int TableRowPosition { get; set; }

        public Alignment TableHorizontalAlignment { get; set; }

        public string TableStyleId { get; set; }

        public Color? PatternColor { get; set; }

        public Color? BackgroundColor { get; set; }

        public ShadingPattern? Pattern { get; set; }

        public bool CanSplit { get; set; }

        public bool RepeatOnEveryPage { get; set; }

        public TableCellStyle CurrentCellStyle
        {
            get
            {
                if (this.CellStyles.Count > 0)
                {
                    return this.CellStyles[this.CellStyles.Count - 1];
                }
                else
                {
                    return null;
                }
            }
        }

        public void EnsureCurrentCellStyle()
        {
            if (this.CellStyles.Count == 0 || this.CellStyles[this.CellStyles.Count - 1].IsLocked)
            {
                TableCellStyle cellStyle = new TableCellStyle(this);
                this.CellStyles.Add(cellStyle);
            }
        }

        // TODO: Move this somewhere else
        public void ApplyRowStyle(TableRow row)
        {
            if (this.RowHeight != null)
            {
                row.Height = this.RowHeight;
            }

            row.CanSplit = this.CanSplit;
            row.RepeatOnEveryPage = this.RepeatOnEveryPage;
        }

        // TODO: Move this somewhere else
        public void ApplyTableStyle(Table table)
        {
            if (!string.IsNullOrEmpty(this.TableStyleId))
            {
                table.StyleId = this.TableStyleId;
            }

            table.Alignment = TableRowStyle.GetHorizontalAlignment(this.TableHorizontalAlignment, this.FlowDirection);

            table.Borders = this.Borders.CreateBorders();

            table.TableCellSpacing = Math.Round(this.DefaultCellSpacing.Left.ValueInDips);

            table.TableCellPadding = CalculatePadding(this.CellPadding, this.DefaultCellPadding);

            if (this.PreferredWidthUnitType != 0)
            {
                table.PreferredWidth = RtfHelper.ConvertTableWidthUnitType(this.PreferredWidthUnitType, this.PreferredWidthValue);
            }

            table.LayoutType = this.AutoFit;
            table.FlowDirection = this.FlowDirection;
            table.Indent = RtfHelper.GetTableIndentFormRtf(this.TableIndentValue, this.TableIndentType);
            table.Looks = this.TableLooks;

            // Apply TableRowPosition property only if there is no TableIndent set.
            if (this.TableIndentValue == 0 && this.TableIndentType == 0)
            {
                table.Indent = Unit.TwipToDip(this.TableRowPosition);
            }

            if (this.BackgroundColor.HasValue)
            {
                table.Shading.BackgroundColor = new ThemableColor(this.BackgroundColor.Value);
            }

            if (this.PatternColor.HasValue)
            {
                table.Shading.PatternColor = new ThemableColor(this.PatternColor.Value);
            }

            if (this.Pattern.HasValue)
            {
                table.Shading.Pattern = this.Pattern.Value;
            }

            if (this.RowBanding.HasValue)
            {
                table.Properties.RowBanding.LocalValue = this.RowBanding;
            }

            if (this.ColumnBanding.HasValue)
            {
                table.Properties.ColumnBanding.LocalValue = this.ColumnBanding;
            }
        }

        private static Alignment GetHorizontalAlignment(Alignment alignment, FlowDirection flowDirection)
        {
            Alignment result = alignment;
            if (flowDirection == FlowDirection.RightToLeft)
            {
                // Reverse alignments if right-to-left.
                if (alignment == Alignment.Left)
                {
                    result = Alignment.Right;
                }
                else if (alignment == Alignment.Right)
                {
                    result = Alignment.Left;
                }
            }

            return result;
        }

        private static Padding CalculatePadding(RtfPadding rowPadding, double rowDefault)
        {
            double left = GetPaddingValue(rowPadding.Left, rowDefault, DocumentDefaultStyleSettings.TableTableCellPadding.Left);
            double top = GetPaddingValue(rowPadding.Top, rowDefault, DocumentDefaultStyleSettings.TableTableCellPadding.Top);
            double right = GetPaddingValue(rowPadding.Right, rowDefault, DocumentDefaultStyleSettings.TableTableCellPadding.Right);
            double bottom = GetPaddingValue(rowPadding.Bottom, rowDefault, DocumentDefaultStyleSettings.TableTableCellPadding.Bottom);

            Padding result = new Padding(left, top, right, bottom);
            return result;
        }

        private static double GetPaddingValue(RtfPaddingValue rowPaddingValue, double rowDefault, double modelDefault)
        {
            if (rowPaddingValue.HasValue)
            {
                return (rowPaddingValue.UnitType == 0) ? rowDefault : Unit.TwipToDip(rowPaddingValue.Value);
            }
            else
            {
                return modelDefault;
            }
        }
    }
}