﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class DocumentStyleHandlers
    {
        public static void InitializeDocumentStyleHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            tagHandlers[RtfTags.TagLayoutMode] = DocumentStyleHandlers.DocumentLayoutModeHandler;
            tagHandlers[RtfTags.TagDocumentPageWidth] = DocumentStyleHandlers.DocumentPageSizeHandler;
            tagHandlers[RtfTags.TagDocumentPageHeight] = DocumentStyleHandlers.DocumentPageSizeHandler;
            tagHandlers[RtfTags.TagDocumentPageMarginLeft] = DocumentStyleHandlers.DocumentPageMarginsHandler;
            tagHandlers[RtfTags.TagDocumentPageMarginRight] = DocumentStyleHandlers.DocumentPageMarginsHandler;
            tagHandlers[RtfTags.TagDocumentPageMarginTop] = DocumentStyleHandlers.DocumentPageMarginsHandler;
            tagHandlers[RtfTags.TagDocumentPageMarginBottom] = DocumentStyleHandlers.DocumentPageMarginsHandler;
            tagHandlers[RtfTags.TagDocumentLandscapeMode] = DocumentStyleHandlers.DocumentLadscapeModeHandler;
            tagHandlers[RtfTags.TagDifferentOddEvenHeaderFooter] = DocumentStyleHandlers.DocumentDifferentOddEvenHeaderFooterHandler;

            tagHandlers[RtfTags.TagDocumentTabDefaultWidth] = DocumentStyleHandlers.DocumentTabDefaultWidthHandler;
        }

        private static void DocumentLayoutModeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagLayoutMode);

            context.Document.ViewType = RtfHelper.DocumentViewTypeMapper.GetToValue(tag.ValueAsNumber);
        }
        private static void DocumentPageSizeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDocumentPageWidth, RtfTags.TagDocumentPageHeight);

            switch (tag.Name)
            {
                case RtfTags.TagDocumentPageWidth:
                    context.CurrentStyle.DocumentSettings.PageWidth = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagDocumentPageHeight:
                    context.CurrentStyle.DocumentSettings.PageHeight = Unit.TwipToDipF(tag.ValueAsNumber);
                    break;
            }
        }

        private static void DocumentPageMarginsHandler(RtfTag tag, RtfImportContext context)
        {
            switch (tag.Name)
            {
                case RtfTags.TagDocumentPageMarginLeft:
                    context.CurrentStyle.DocumentSettings.LeftPageMargin = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagDocumentPageMarginRight:
                    context.CurrentStyle.DocumentSettings.RightPageMargin = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagDocumentPageMarginTop:
                    context.CurrentStyle.DocumentSettings.TopPageMargin = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagDocumentPageMarginBottom:
                    context.CurrentStyle.DocumentSettings.BottomPageMargin = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
            }
        }

        private static void DocumentLadscapeModeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDocumentLandscapeMode);

            context.CurrentStyle.DocumentSettings.PageOrientation = PageOrientation.Landscape;
        }

        private static void DocumentDifferentOddEvenHeaderFooterHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDifferentOddEvenHeaderFooter);

            context.Document.HasDifferentEvenOddPageHeadersFooters = true;
        }

        private static void DocumentTabDefaultWidthHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDocumentTabDefaultWidth);

            context.Document.DefaultTabStopWidth = Unit.TwipToDipI(tag.ValueAsNumber);
        }
    }
}
