﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Exceptions;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class ParagraphStyleHandlers
    {
        public static void InitializeParagraphStyleHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            // Paragraph
            tagHandlers[RtfTags.TagParagraphDefaults] = ParagraphStyleHandlers.ResetParagraphStyleHandler;

            // Alignment
            tagHandlers[RtfTags.TagAlignCenter] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignLeft] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignRight] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignJustify] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignDistribute] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignKashida] = ParagraphStyleHandlers.ParagraphAlignementHandler;
            tagHandlers[RtfTags.TagAlignThai] = ParagraphStyleHandlers.ParagraphAlignementHandler;

            // Indent
            tagHandlers[RtfTags.TagParagraphFirstLineIndent] = ParagraphStyleHandlers.ParagraphFirstLineIndentHandler;
            tagHandlers[RtfTags.TagParagraphFistLineIndentInCharUnits] = ParagraphStyleHandlers.ParagraphFirstLineIndentInCharUnitsHandler;
            tagHandlers[RtfTags.TagParagraphLeftIndent] = ParagraphStyleHandlers.ParagraphLeftIndentHandler;
            tagHandlers[RtfTags.TagParagraphLeftIndentBiDirectional] = ParagraphStyleHandlers.ParagraphLeftIndentBiDirectionalHandler;
            tagHandlers[RtfTags.TagParagraphLeftIndentInCharUnits] = ParagraphStyleHandlers.ParagraphLeftIndentInCharUnitsHandler;
            tagHandlers[RtfTags.TagParagraphRightIndent] = ParagraphStyleHandlers.ParagraphRightIndentHandler;
            tagHandlers[RtfTags.TagParagraphRightIndentBiDirectional] = ParagraphStyleHandlers.ParagraphRightIndentBiDirectionalHandler;
            tagHandlers[RtfTags.TagParagraphRightIndentInCharUnits] = ParagraphStyleHandlers.ParagraphRightIndentInCharUnitsHandler;

            // Spacing
            tagHandlers[RtfTags.TagParagraphSpacingBefore] = ParagraphStyleHandlers.ParagraphSpacingBeforeHandler;
            tagHandlers[RtfTags.TagParagraphAutomaticSpacingBefore] = ParagraphStyleHandlers.ParagraphAutomaticSpacingBeforeHandler;
            tagHandlers[RtfTags.TagParagraphSpacingAfter] = ParagraphStyleHandlers.ParagraphSpacingAfterHandler;
            tagHandlers[RtfTags.TagParagraphAutomaticSpacingAfter] = ParagraphStyleHandlers.ParagraphAutomaticSpacingAfterHandler;
            tagHandlers[RtfTags.TagParagraphSpacingBeforeInCharUnits] = ParagraphStyleHandlers.ParagraphSpacingBeforeInCharUnitsHandler;
            tagHandlers[RtfTags.TagParagraphSpacingAfterInCharUnits] = ParagraphStyleHandlers.ParagraphSpacingAfterInCharUnitsHandler;
            tagHandlers[RtfTags.TagParagraphLineSpacing] = ParagraphStyleHandlers.ParagraphLineSpacingHandler;
            tagHandlers[RtfTags.TagParagraphLineSpacingType] = ParagraphStyleHandlers.ParagraphLineSpacingTypeHandler;

            // Shading
            tagHandlers[RtfTags.TagParagraphShadingBackground] = ParagraphStyleHandlers.ParagraphShadingBackroundColorHandler;
            tagHandlers[RtfTags.TagParagraphShadingForeground] = ParagraphStyleHandlers.ParagraphShadingForegroundColorHandler;
            tagHandlers[RtfTags.TagParagraphShadingPercent] = ParagraphStyleHandlers.ParagraphShadingPercentHandler;

            // Borders
            tagHandlers[RtfTags.TagParagraphBorderLeft] = ParagraphStyleHandlers.ParagraphBordersHandler;
            tagHandlers[RtfTags.TagParagraphBorderTop] = ParagraphStyleHandlers.ParagraphBordersHandler;
            tagHandlers[RtfTags.TagParagraphBorderRight] = ParagraphStyleHandlers.ParagraphBordersHandler;
            tagHandlers[RtfTags.TagParagraphBorderBottom] = ParagraphStyleHandlers.ParagraphBordersHandler;
            tagHandlers[RtfTags.TagParagraphBorderBetween] = ParagraphStyleHandlers.ParagraphBordersHandler;

            tagHandlers[RtfTags.TagListOverrideId] = ParagraphStyleHandlers.ParagraphListIdHandler;
            tagHandlers[RtfTags.TagParagraphListLevel] = ParagraphStyleHandlers.ParagraphListLevelHandler;

            tagHandlers[RtfTags.TagTabStopPosition] = ParagraphStyleHandlers.TabStopPositionHandler;
            tagHandlers[RtfTags.TagTabStopRight] = ParagraphStyleHandlers.TabStopTypeHandler;
            tagHandlers[RtfTags.TagTabStopCenter] = ParagraphStyleHandlers.TabStopTypeHandler;
            tagHandlers[RtfTags.TagTabStopDecimal] = ParagraphStyleHandlers.TabStopTypeHandler;

            tagHandlers[RtfTags.TagTabStopBarPosition] = ParagraphStyleHandlers.TabStopBarPositionHandler;

            tagHandlers[RtfTags.TagTabStopLeaderDots] = ParagraphStyleHandlers.TabStopLeaderHandler;
            tagHandlers[RtfTags.TagTabStopLeaderHyphens] = ParagraphStyleHandlers.TabStopLeaderHandler;
            tagHandlers[RtfTags.TagTabStopLeaderMiddleDots] = ParagraphStyleHandlers.TabStopLeaderHandler;
            tagHandlers[RtfTags.TagTabStopLeaderThickLine] = ParagraphStyleHandlers.TabStopLeaderHandler;
            tagHandlers[RtfTags.TagTabStopLeaderUnderline] = ParagraphStyleHandlers.TabStopLeaderHandler;

            // FlowDirection
            tagHandlers[RtfTags.TagParagraphLtr] = ParagraphStyleHandlers.ParagraphLtrHandler;
            tagHandlers[RtfTags.TagParagraphRtl] = ParagraphStyleHandlers.ParagraphRtlHandler;

            // Borders generic handler
            BorderHandlers.InitializeBorderHandlers(tagHandlers);
        }

        private static void ResetParagraphStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphDefaults);

            context.CurrentStyle.CurrentBorder = null;
            context.CurrentStyle.CurrentBorderedElementType = BorderedElementType.Paragraph;

            context.CurrentStyle.ResetParagraphStyle();
        }

        private static void ParagraphAlignementHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentStyle.ParagraphStyle.TextAlignment = RtfHelper.RtfTagToAlignment(tag);
        }

        #region Spacing

        private static void ParagraphSpacingBeforeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphSpacingBefore);
            context.CurrentStyle.ParagraphStyle.Spacing.SpacingBefore = Unit.TwipToDip(tag.ValueAsNumber);
        }

        private static void ParagraphAutomaticSpacingBeforeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphAutomaticSpacingBefore);
            if (tag.ValueAsText == "1")
            {
                context.CurrentStyle.ParagraphStyle.Spacing.AutomaticSpacingBefore = true;
            }
        }

        private static void ParagraphSpacingAfterHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphSpacingAfter);
            context.CurrentStyle.ParagraphStyle.Spacing.SpacingAfter = Unit.TwipToDip(tag.ValueAsNumber);
        }

        private static void ParagraphAutomaticSpacingAfterHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphAutomaticSpacingAfter);
            if (tag.ValueAsText == "1")
            {
                context.CurrentStyle.ParagraphStyle.Spacing.AutomaticSpacingAfter = true;
            }
        }

        private static void ParagraphSpacingBeforeInCharUnitsHandler(RtfTag tag, RtfImportContext context)
        {
            // TODO: Units are hundredths of a character unit
        }

        private static void ParagraphSpacingAfterInCharUnitsHandler(RtfTag tag, RtfImportContext context)
        {
            // TODO: Units are hundredths of a character unit
        }

        private static void ParagraphLineSpacingHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphLineSpacing);

            if (tag.HasValue && tag.ValueAsNumber != 0)
            {
                if (tag.ValueAsNumber < 0)
                {
                    context.CurrentStyle.ParagraphStyle.Spacing.LineSpacingType = HeightType.Exact;
                    context.CurrentStyle.ParagraphStyle.Spacing.LineSpacing = Unit.TwipToDip(-tag.ValueAsNumber);
                }
                else
                {
                    context.CurrentStyle.ParagraphStyle.Spacing.LineSpacingType = HeightType.AtLeast;
                    context.CurrentStyle.ParagraphStyle.Spacing.LineSpacing = Unit.TwipToDip(tag.ValueAsNumber);
                }
            }
        }

        private static void ParagraphLineSpacingTypeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphLineSpacingType);

            if (tag.ValueAsNumber == 1)
            {
                // We should try to convert the exact value for line spacing to a relative one using the DEFAULT_LINE_SPACING
                double value = Unit.DipToTwip(context.CurrentStyle.ParagraphStyle.Spacing.LineSpacing);
                value = Math.Round(value / RtfHelper.DefaultLineSpacing, 2);

                context.CurrentStyle.ParagraphStyle.Spacing.LineSpacingType = HeightType.Auto;
                context.CurrentStyle.ParagraphStyle.Spacing.LineSpacing = value;
            }
        }

        #endregion

        #region Indentation

        private static void ParagraphFirstLineIndentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphFirstLineIndent);

            double value = Unit.TwipToDip(tag.ValueAsNumber);

            if (value > 0)
            {
                context.CurrentStyle.ParagraphStyle.Indentation.FirstLineIndent = value;
            }
            else
            {
                context.CurrentStyle.ParagraphStyle.Indentation.HangingIndent = -value;
            }
        }

        private static void ParagraphFirstLineIndentInCharUnitsHandler(RtfTag tag, RtfImportContext context)
        {
            // TODO: First indent in hundredths of a character unit
        }

        private static void ParagraphLeftIndentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphLeftIndent);

            double value = Unit.TwipToDip(tag.ValueAsNumber);

            if (context.CurrentStyle.ParagraphStyle.FlowDirection == FlowDirection.LeftToRight)
            {
                context.CurrentStyle.ParagraphStyle.Indentation.LeftIndent = value;
            }
            else
            {
                context.CurrentStyle.ParagraphStyle.Indentation.RightIndent = value;
            }
        }

        private static void ParagraphLeftIndentBiDirectionalHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphLeftIndentBiDirectional);

            double value = Unit.TwipToDip(tag.ValueAsNumber);

            context.CurrentStyle.ParagraphStyle.Indentation.LeftIndent = value;
        }

        private static void ParagraphLeftIndentInCharUnitsHandler(RtfTag tag, RtfImportContext context)
        {
            // TODO: Left indent (space before) in hundredths of a character unit
        }

        private static void ParagraphRightIndentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphRightIndent);
            double value = Unit.TwipToDip(tag.ValueAsNumber);
            if (context.CurrentStyle.ParagraphStyle.FlowDirection == FlowDirection.LeftToRight)
            {
                context.CurrentStyle.ParagraphStyle.Indentation.RightIndent = value;
            }
            else
            {
                context.CurrentStyle.ParagraphStyle.Indentation.LeftIndent = value;
            }
        }

        private static void ParagraphRightIndentBiDirectionalHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphRightIndentBiDirectional);
            context.CurrentStyle.ParagraphStyle.Indentation.RightIndent = Unit.TwipToDip(tag.ValueAsNumber);
        }

        private static void ParagraphRightIndentInCharUnitsHandler(RtfTag tag, RtfImportContext context)
        {
            // TODO: Right indent (space before) in hundredths of a character unit
        }

        #endregion

        #region Shading

        private static void ParagraphShadingBackroundColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphShadingBackground);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.ParagraphStyle.Shading.BackgroundColor = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void ParagraphShadingForegroundColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphShadingForeground);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.ParagraphStyle.Shading.PatternColor = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void ParagraphShadingPercentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphShadingPercent);

            context.CurrentStyle.ParagraphStyle.Shading.Pattern = RtfHelper.RtfTagToShadingPattern(tag);
        }

        #endregion

        private static void ParagraphBordersHandler(RtfTag tag, RtfImportContext context)
        {
            switch (tag.Name)
            {
                case RtfTags.TagParagraphBorderTop:
                    context.CurrentStyle.CurrentBorder = context.CurrentStyle.ParagraphBorders.Top;
                    break;
                case RtfTags.TagParagraphBorderLeft:
                    context.CurrentStyle.CurrentBorder = context.CurrentStyle.ParagraphBorders.Left;
                    break;
                case RtfTags.TagParagraphBorderBottom:
                    context.CurrentStyle.CurrentBorder = context.CurrentStyle.ParagraphBorders.Bottom;
                    break;
                case RtfTags.TagParagraphBorderRight:
                    context.CurrentStyle.CurrentBorder = context.CurrentStyle.ParagraphBorders.Right;
                    break;
                case RtfTags.TagParagraphBorderBetween:
                    context.CurrentStyle.CurrentBorder = context.CurrentStyle.ParagraphBorders.Between;
                    break;

                default:
                    throw new RtfUnexpectedElementException("Paragraph border control word", tag.Name);
            }
            context.CurrentStyle.ParagraphBorders.HasValue = true;
            context.CurrentStyle.CurrentBorder.HasValue = true;
            context.CurrentStyle.CurrentBorderedElementType = BorderedElementType.Paragraph;
        }

        private static void ParagraphLtrHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphLtr);
            context.CurrentStyle.ParagraphStyle.FlowDirection = FlowDirection.LeftToRight;
        }

        private static void ParagraphRtlHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphRtl);
            context.CurrentStyle.ParagraphStyle.FlowDirection = FlowDirection.RightToLeft;
        }

        private static void ParagraphListIdHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagListOverrideId);

            context.CurrentStyle.CurrentList = context.ListOverrideTable.GetList(tag.ValueAsNumber);
            if (context.CurrentStyle.CurrentList != null)
            {
                context.CurrentStyle.ParagraphStyle.ListId = context.CurrentStyle.CurrentList.Id;
            }
        }

        private static void ParagraphListLevelHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphListLevel);

            context.CurrentStyle.CurrentListLevel = tag.ValueAsNumber;
            context.CurrentStyle.ParagraphStyle.ListLevel = context.CurrentStyle.CurrentListLevel;
        }

        private static void TabStopLeaderHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentStyle.CurrentTabStop.Leader = RtfHelper.TabStopLeaderMapper.GetToValue(tag.Name);
        }

        private static void TabStopTypeHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentStyle.CurrentTabStop.Type = RtfHelper.TabStopTypeMapper.GetToValue(tag.Name);
        }

        private static void TabStopPositionHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTabStopPosition);

            context.CurrentStyle.CurrentTabStop.Position = Unit.TwipToDipI(tag.ValueAsNumber);
            context.CurrentStyle.ParagraphStyle.TabStops =
                context.CurrentStyle.ParagraphStyle.Properties.TabStops.LocalValue.Insert(context.CurrentStyle.CurrentTabStop.GetTabStop());
            context.CurrentStyle.ResetTabStop();
        }

        private static void TabStopBarPositionHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTabStopBarPosition);

            context.CurrentStyle.CurrentTabStop.Type = TabStopType.Bar;
            context.CurrentStyle.CurrentTabStop.Position = Unit.TwipToDipI(tag.ValueAsNumber);
            context.CurrentStyle.ParagraphStyle.TabStops =
                context.CurrentStyle.ParagraphStyle.Properties.TabStops.LocalValue.Insert(context.CurrentStyle.CurrentTabStop.GetTabStop());

            context.CurrentStyle.ResetTabStop();
        }
    }
}
