﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal delegate void ControlTagHandler(RtfTag tag, RtfImportContext context);

    // TODO: think of a better name
    internal static class RtfTagHandlers
    {
        public static void InitializeAllTagHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            ////// RTF Header tags
            tagHandlers[RtfTags.TagDefaultFont] = RtfTagHandlers.DefaultFontHandler;
            tagHandlers[RtfTags.TagDefaultLanguage] = RtfTagHandlers.DefaultLanguageHandler;
            tagHandlers[RtfTags.TagDefaultBiDiFont] = RtfTagHandlers.DefaultLanguageHandler;

            DocumentStyleHandlers.InitializeDocumentStyleHandlers(tagHandlers);

            SectionStyleHandlers.InitializeSectionStyleHandlers(tagHandlers);

            SpanStyleHandlers.InitializeSpanStyleHandlers(tagHandlers);

            ParagraphStyleHandlers.InitializeParagraphStyleHandlers(tagHandlers);

            TableStyleHandlers.InitializeTableStyleHandlers(tagHandlers);

            SpecialSymbolsHandlers.InitializeSymbolsHandlers(tagHandlers);

            ////// Styles
            tagHandlers[RtfTags.TagStyleCharacter] = RtfTagHandlers.CharacterStyleHandler;
            tagHandlers[RtfTags.TagStyleParagraph] = RtfTagHandlers.ParagraphStyleHandler;
            tagHandlers[RtfTags.TagStyleTable] = RtfTagHandlers.TableStyleHandler;

            ////// Structure
            tagHandlers[RtfTags.TagSectionEnd] = RtfTagHandlers.SectionEndHandler;
            tagHandlers[RtfTags.TagParagraphEnd] = RtfTagHandlers.ParagraphEndHandler;
            tagHandlers[RtfTags.TagCellEnd] = RtfTagHandlers.CellEndHandler;
            tagHandlers[RtfTags.TagNestedCellEnd] = RtfTagHandlers.CellEndHandler;
            tagHandlers[RtfTags.TagRowEnd] = RtfTagHandlers.RowEndHandler;
            tagHandlers[RtfTags.TagNestedRowEnd] = RtfTagHandlers.RowEndHandler;
            tagHandlers[RtfTags.TagParagraphInTable] = RtfTagHandlers.ParagraphInTableHandler;
            tagHandlers[RtfTags.TagParagraphNestingLevel] = RtfTagHandlers.ParagraphNestingLevelHandler;
        }

        private static void DefaultFontHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDefaultFont);

            context.DefaultFontId = RtfTags.TagFont + tag.ValueAsNumber.ToString();
        }

        private static void DefaultLanguageHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDefaultLanguage, RtfTags.TagDefaultBiDiFont);
        }

        private static void SectionEndHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionEnd);

            context.HandleSectionEnd();
        }

        private static void ParagraphInTableHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphInTable);

            context.CurrentStyle.IsInTable = true;
            context.CurrentStyle.CurrentNestingLevel = 1;
        }

        private static void ParagraphNestingLevelHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphNestingLevel);

            context.CurrentStyle.CurrentNestingLevel = tag.ValueAsNumber;
        }

        private static void ParagraphEndHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagParagraphEnd);
            context.HandleParagraphEnd();
        }

        private static void RowEndHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowEnd, RtfTags.TagNestedRowEnd);

            context.HandleRowEnd();
        }

        private static void CellEndHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagCellEnd, RtfTags.TagNestedCellEnd);

            context.HandleCellEnd();
        }

        private static void CharacterStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleCharacter);
            Style style = context.StylesTable.GetStyleById(tag.ValueAsNumber);
            if (style != null && style.StyleType == StyleType.Character)
            {
                context.CurrentStyle.CharacterStyle.StyleId = style.Id;
            }
        }

        private static void ParagraphStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleParagraph);
            Style style = context.StylesTable.GetStyleById(tag.ValueAsNumber);
            if (style != null && style.StyleType == StyleType.Paragraph)
            {
                context.CurrentStyle.ParagraphStyle.StyleId = style.Id;
            }
        }

        private static void TableStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleTable);
            Style style = context.StylesTable.GetStyleById(tag.ValueAsNumber);
            if (style != null && style.StyleType == StyleType.Table)
            {
                context.CurrentStyle.RowStyle.TableStyleId = style.Id;
            }
        }
    }
}
