﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class SectionStyleHandlers
    {
        public static void InitializeSectionStyleHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            tagHandlers[RtfTags.TagSectionDefaults] = SectionStyleHandlers.ResetSectionStyleHandler;
            tagHandlers[RtfTags.TagSectionLandscapeMode] = SectionStyleHandlers.SectionLadscapeModeHandler;

            tagHandlers[RtfTags.TagSectionWidth] = SectionStyleHandlers.SectionPageSizeHandler;
            tagHandlers[RtfTags.TagSectionHeight] = SectionStyleHandlers.SectionPageSizeHandler;

            tagHandlers[RtfTags.TagSectionMarginLeft] = SectionStyleHandlers.SectionMarginsHandler;
            tagHandlers[RtfTags.TagSectionMarginRight] = SectionStyleHandlers.SectionMarginsHandler;
            tagHandlers[RtfTags.TagSectionMarginTop] = SectionStyleHandlers.SectionMarginsHandler;
            tagHandlers[RtfTags.TagSectionMarginBottom] = SectionStyleHandlers.SectionMarginsHandler;

            tagHandlers[RtfTags.TagDifferentFirstHeaderFooter] = SectionStyleHandlers.SectionDifferentFirstPageHeaderFooterHandler;
            tagHandlers[RtfTags.TagHeaderTopMargin] = SectionStyleHandlers.SectionHeaderTopMarginHandler;
            tagHandlers[RtfTags.TagFooterBottomMargin] = SectionStyleHandlers.SectionFooterBottomMarginHandler;

            tagHandlers[RtfTags.TagSectionBreakNewPage] = SectionStyleHandlers.SectionTypePageHandler;
            tagHandlers[RtfTags.TagSectionBreakEvenPage] = SectionStyleHandlers.SectionTypePageHandler;
            tagHandlers[RtfTags.TagSectionBreakOddPage] = SectionStyleHandlers.SectionTypePageHandler;
            tagHandlers[RtfTags.TagSectionBreakColumn] = SectionStyleHandlers.SectionTypePageHandler;
            tagHandlers[RtfTags.TagSectionBreakNone] = SectionStyleHandlers.SectionTypePageHandler;

            tagHandlers[RtfTags.TagSectionVerticalAlignmentTop] = SectionStyleHandlers.SectionVerticalAlignmentHandler;
            tagHandlers[RtfTags.TagSectionVerticalAlignmentCenter] = SectionStyleHandlers.SectionVerticalAlignmentHandler;
            tagHandlers[RtfTags.TagSectionVerticalAlignmentJustified] = SectionStyleHandlers.SectionVerticalAlignmentHandler;
            tagHandlers[RtfTags.TagSectionVerticalAlignmentBottom] = SectionStyleHandlers.SectionVerticalAlignmentHandler;
            tagHandlers[RtfTags.TagSectionVerticalAlignmentBottomAlias] = SectionStyleHandlers.SectionVerticalAlignmentHandler;

            // Page numbering settings handlers
            tagHandlers[RtfTags.TagSectionStartingPageNumber] = SectionStyleHandlers.SectionStartingPageNumberHandler;
            tagHandlers[RtfTags.TagSectionChapterHeadingStyle] = SectionStyleHandlers.SectionChapterHeadingStyleHandler;
            tagHandlers[RtfTags.TagSectionChapterSeparatorCharacterHyphen] = SectionStyleHandlers.SectionChapterSeparatorCharacterHandler;
            tagHandlers[RtfTags.TagSectionChapterSeparatorCharacterPeriod] = SectionStyleHandlers.SectionChapterSeparatorCharacterHandler;
            tagHandlers[RtfTags.TagSectionChapterSeparatorCharacterColon] = SectionStyleHandlers.SectionChapterSeparatorCharacterHandler;
            tagHandlers[RtfTags.TagSectionChapterSeparatorCharacterEmDash] = SectionStyleHandlers.SectionChapterSeparatorCharacterHandler;
            tagHandlers[RtfTags.TagSectionChapterSeparatorCharacterEnDash] = SectionStyleHandlers.SectionChapterSeparatorCharacterHandler;

            tagHandlers[RtfTags.TagSectionPageNumberFormatArabicAbjad] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatCircleNumbering] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatDecimal] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatHindiConsonants] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatHindiCounting] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatHindiNumbers] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatHindiVowel] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatKoreanNumbering] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatKoreanNumbering2] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatLowercaseLetter] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatLowerRomanNumeral] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatThaiCounting] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatThaiLetters] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatThaiNumbers] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatUppercaseLetter] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatUppercaseRomanNumeral] = SectionStyleHandlers.SectionPageNumberFormatHandler;
            tagHandlers[RtfTags.TagSectionPageNumberFormatVietnameseCounting] = SectionStyleHandlers.SectionPageNumberFormatHandler;
        }

        private static void ResetSectionStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionDefaults);

            context.CurrentStyle.ResetSectionStyle();
        }

        private static void SectionTypePageHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentStyle.SectionStyle.SectionType = RtfHelper.SectionTypeMapper.GetToValue(tag.Name);
        }

        private static void SectionPageSizeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionWidth, RtfTags.TagSectionHeight);

            float valueInPixels = Unit.TwipToDipF(tag.ValueAsNumber);
            Size newSize = context.CurrentStyle.SectionStyle.PageSize;
            switch (tag.Name)
            {
                case RtfTags.TagSectionWidth:
                    newSize.Width = valueInPixels;
                    break;
                case RtfTags.TagSectionHeight:
                    newSize.Height = valueInPixels;
                    break;
            }
            context.CurrentStyle.SectionStyle.PageSize = newSize;
        }

        private static void SectionLadscapeModeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionLandscapeMode);
            context.CurrentStyle.SectionStyle.PageOrientation = PageOrientation.Landscape;
        }

        private static void SectionMarginsHandler(RtfTag tag, RtfImportContext context)
        {
            Padding margins = context.CurrentStyle.SectionStyle.PageMargins;
            double left = margins.Left;
            double right = margins.Right;
            double top = margins.Top;
            double bottom = margins.Bottom;
            switch (tag.Name)
            {
                case RtfTags.TagSectionMarginLeft:
                    left = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagSectionMarginRight:
                    right = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagSectionMarginTop:
                    top = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
                case RtfTags.TagSectionMarginBottom:
                    bottom = Unit.TwipToDip(tag.ValueAsNumber);
                    break;
            }
            context.CurrentStyle.SectionStyle.PageMargins = new Padding(left, top, right, bottom);
        }

        private static void SectionDifferentFirstPageHeaderFooterHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagDifferentFirstHeaderFooter);

            context.CurrentStyle.SectionStyle.HasDifferentFirstPageHeaderFooter = true;
        }

        private static void SectionHeaderTopMarginHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagHeaderTopMargin);

            context.CurrentStyle.SectionStyle.HeaderTopMargin = Unit.TwipToDipI(tag.ValueAsNumber);
        }

        private static void SectionFooterBottomMarginHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagFooterBottomMargin);

            context.CurrentStyle.SectionStyle.FooterBottomMargin = Unit.TwipToDipI(tag.ValueAsNumber);
        }

        private static void SectionVerticalAlignmentHandler(RtfTag tag, RtfImportContext context)
        {
            Flow.Model.Styles.VerticalAlignment? verticalAlignment = null;
            switch (tag.Name)
            {
                case RtfTags.TagSectionVerticalAlignmentTop:
                    verticalAlignment = Flow.Model.Styles.VerticalAlignment.Top;
                    break;
                case RtfTags.TagSectionVerticalAlignmentCenter:
                    verticalAlignment = Flow.Model.Styles.VerticalAlignment.Center;
                    break;
                case RtfTags.TagSectionVerticalAlignmentJustified:
                    verticalAlignment = Flow.Model.Styles.VerticalAlignment.Justified;
                    break;
                case RtfTags.TagSectionVerticalAlignmentBottom:
                case RtfTags.TagSectionVerticalAlignmentBottomAlias:
                    verticalAlignment = Flow.Model.Styles.VerticalAlignment.Bottom;
                    break;
            }

            if (verticalAlignment != null)
            {
                context.CurrentStyle.SectionStyle.VerticalAlignment = verticalAlignment.Value;
            }
        }

        private static void SectionStartingPageNumberHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionStartingPageNumber);

            context.CurrentStyle.SectionStyle.PageNumberingSettings.StartingPageNumber = tag.ValueAsNumber;
        }

        private static void SectionPageNumberFormatHandler(RtfTag tag, RtfImportContext context)
        {
            NumberingStyle numberFormat;
            if (RtfHelper.NumberingStyleMapper.TryGetToValue(tag.Name, out numberFormat))
            {
                context.CurrentStyle.SectionStyle.PageNumberingSettings.PageNumberFormat = numberFormat;
            }
        }

        private static void SectionChapterSeparatorCharacterHandler(RtfTag tag, RtfImportContext context)
        {
            ChapterSeparatorType chapterSeparatorType;
            if (RtfHelper.ChapterSeparatorTypeMapper.TryGetToValue(tag.Name, out chapterSeparatorType))
            {
                context.CurrentStyle.SectionStyle.PageNumberingSettings.ChapterSeparatorCharacter = chapterSeparatorType;
            }
        }

        private static void SectionChapterHeadingStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSectionChapterHeadingStyle);

            // 0 is the default value in RTF.
            if (tag.ValueAsNumber != 0)
            {
                context.CurrentStyle.SectionStyle.PageNumberingSettings.ChapterHeadingStyleIndex = tag.ValueAsNumber;
            }
        }
    }
}