﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using BaselineAlignment = Telerik.Windows.Documents.Flow.Model.Styles.BaselineAlignment;
using FlowDirection = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class SpanStyleHandlers
    {
        public static void InitializeSpanStyleHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            tagHandlers[RtfTags.TagResetFormatting] = SpanStyleHandlers.SpanResetFormattingHandler;
            tagHandlers[RtfTags.TagSubscript] = SpanStyleHandlers.SpanBaselineAlignmentHandler;
            tagHandlers[RtfTags.TagSuperscript] = SpanStyleHandlers.SpanBaselineAlignmentHandler;
            tagHandlers[RtfTags.TagResetBaselineAlignement] = SpanStyleHandlers.SpanBaselineAlignmentHandler;

            tagHandlers[RtfTags.TagBold] = SpanStyleHandlers.SpanBoldHandler;
            tagHandlers[RtfTags.TagItalic] = SpanStyleHandlers.SpanItalicHandler;
            tagHandlers[RtfTags.TagStrikethrough] = SpanStyleHandlers.SpanStrikethroughHandler;
            tagHandlers[RtfTags.TagDoubleStrikethrough] = SpanStyleHandlers.SpanStrikethroughHandler;
            tagHandlers[RtfTags.TagFontSize] = SpanStyleHandlers.SpanFontSizeHandler;
            tagHandlers[RtfTags.TagFont] = SpanStyleHandlers.SpanFontHandler;

            tagHandlers[RtfTags.TagSpanForecolor] = SpanStyleHandlers.SpanForecolorHandler;
            tagHandlers[RtfTags.TagSpanHighlight] = SpanStyleHandlers.SpanHighlightColorHandler;
            tagHandlers[RtfTags.TagSpanHighlightWord] = SpanStyleHandlers.SpanHighlightColorHandler;

            tagHandlers[RtfTags.TagUnderlineColor] = SpanStyleHandlers.SpanUnderlineColorHandler;
            tagHandlers[RtfTags.TagUnderlineNone] = SpanStyleHandlers.SpanClearUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineLine] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDashed] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDashDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDashDotDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDouble] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineHeavyWave] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineLongDashed] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThick] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThickDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThickDashed] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThickDashDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThickDashDotDotted] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineThickLongDashed] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineDoubleWave] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineWord] = SpanStyleHandlers.SpanUnderlineHandler;
            tagHandlers[RtfTags.TagUnderlineWave] = SpanStyleHandlers.SpanUnderlineHandler;

            tagHandlers[RtfTags.TagSpanShadingBackground] = SpanStyleHandlers.SpanShadingBackroundColorHandler;
            tagHandlers[RtfTags.TagSpanShadingForeground] = SpanStyleHandlers.SpanShadingForegroundColorHandler;
            tagHandlers[RtfTags.TagSpanShadingPercent] = SpanStyleHandlers.SpanShadingPercentHandler;
            tagHandlers[RtfTags.TagLineBreakTextWrapping] = SpanStyleHandlers.TextWrappingHandler;

            tagHandlers[RtfTags.TagSpanLtr] = SpanStyleHandlers.SpanLTRHandler;
            tagHandlers[RtfTags.TagSpanRtl] = SpanStyleHandlers.SpanRTLHandler;
            ////tagHandlers[RtfTags.TagHidden] = SpanStyleHandlers.SpanHiddenHandler;

            tagHandlers[RtfTags.TagFieldIsDirty] = SpanStyleHandlers.FieldIsDirtyHandler;
            tagHandlers[RtfTags.TagFieldIsLocked] = SpanStyleHandlers.FieldIsLockedHandler;
        }

        private static void SpanResetFormattingHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagResetFormatting);

            context.CurrentStyle.ResetSpanStyle();
        }

        private static void SpanFontSizeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagFontSize);

            double size = tag.ValueAsNumber / 2.0;
            size = Unit.PointToDip(size);

            context.CurrentStyle.CharacterStyle.FontSize = size;
        }

        private static void SpanBaselineAlignmentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSubscript, RtfTags.TagSuperscript, RtfTags.TagResetBaselineAlignement);

            BaselineAlignment alignement = BaselineAlignment.Baseline;

            switch (tag.Name)
            {
                case RtfTags.TagSubscript:
                    alignement = BaselineAlignment.Subscript;
                    break;

                case RtfTags.TagSuperscript:
                    alignement = BaselineAlignment.Superscript;
                    break;

                case RtfTags.TagResetBaselineAlignement:
                    alignement = BaselineAlignment.Baseline;
                    break;
            }

            context.CurrentStyle.CharacterStyle.BaselineAlignment = alignement;
        }

        private static void SpanBoldHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagBold);
            context.CurrentStyle.CharacterStyle.FontWeight = tag.HasValue && tag.ValueAsNumber == 0 ? FontWeights.Normal : FontWeights.Bold;
        }

        private static void SpanItalicHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagItalic);
            context.CurrentStyle.CharacterStyle.FontStyle = tag.HasValue && tag.ValueAsNumber == 0 ? FontStyles.Normal : FontStyles.Italic;
        }

        private static void SpanStrikethroughHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStrikethrough, RtfTags.TagDoubleStrikethrough);

            context.CurrentStyle.CharacterStyle.Strikethrough = !(tag.HasValue && tag.ValueAsNumber == 0);
        }

        private static void SpanFontHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagFont);

            context.CurrentStyle.CharacterStyle.FontFamily = new ThemableFontFamily(context.FontTable.GetFontFamily(tag.FullName));
        }

        private static void SpanClearUnderlineHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagUnderlineNone);

            context.CurrentStyle.CharacterStyle.Underline.Pattern = UnderlinePattern.None;
        }

        private static void SpanUnderlineColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagUnderlineColor);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.CharacterStyle.Underline.Color = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void SpanUnderlineHandler(RtfTag tag, RtfImportContext context)
        {
            UnderlinePattern decoration;

            if (tag.HasValue && tag.ValueAsNumber == 0)
            {
                decoration = UnderlinePattern.None;
            }
            else
            {
                decoration = RtfHelper.RtfTagToUndelineDecoration(tag.Name);
            }

            context.CurrentStyle.CharacterStyle.Underline.Pattern = decoration;
        }

        private static void SpanForecolorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanForecolor);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.CharacterStyle.ForegroundColor = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void SpanHighlightColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanHighlight, RtfTags.TagSpanHighlightWord);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.CharacterStyle.HighlightColor = rtfColor.IsAutomatic ? Run.HighlightColorPropertyDefinition.DefaultValue.Value : rtfColor.Color;
            }
        }

        private static void SpanShadingBackroundColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanShadingBackground);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.CharacterStyle.Shading.BackgroundColor = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void SpanShadingForegroundColorHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanShadingForeground);

            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentStyle.CharacterStyle.Shading.PatternColor = new ThemableColor(rtfColor.Color, rtfColor.IsAutomatic);
            }
        }

        private static void SpanShadingPercentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanShadingPercent);

            context.CurrentStyle.CharacterStyle.Shading.Pattern = RtfHelper.RtfTagToShadingPattern(tag);
        }

        private static void TextWrappingHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagLineBreakTextWrapping);

            context.CurrentStyle.CurrentTextWrapping = RtfHelper.LineBreakTextWrappingMapper.GetToValue(tag.ValueAsNumber);
        }

        private static void FieldIsLockedHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagFieldIsLocked);

            context.FieldContext.SetIsLocked(true);
        }

        private static void FieldIsDirtyHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagFieldIsDirty);

            context.FieldContext.SetIsDirty(true);
        }

        private static void SpanLTRHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanLtr);
            context.CurrentStyle.CharacterStyle.FlowDirection = FlowDirection.LeftToRight;
        }

        private static void SpanRTLHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagSpanRtl);
            context.CurrentStyle.CharacterStyle.FlowDirection = FlowDirection.RightToLeft;
        }

#if flase
        private static void SpanHiddenHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagHidden);
            context.CurrentStyle.IsHidden = (!tag.HasValue || tag.ValueAsNumber != 0);
        }
#endif
    }
}
