﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class SpecialSymbolsHandlers
    {
        public static void InitializeSymbolsHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            tagHandlers[RtfTags.TagTabulator] = SpecialSymbolsHandlers.TabHandler;
            tagHandlers[RtfTags.TagNonBreakingHyphen] = SpecialSymbolsHandlers.NonBreakingHyphenHandler;
            tagHandlers[RtfTags.TagTilde] = SpecialSymbolsHandlers.NonBreakingSpaceHandler;
            tagHandlers[RtfTags.TagRightSingleQuote] = SpecialSymbolsHandlers.RightSingleQuoteHandler;
            tagHandlers[RtfTags.TagLeftSingleQuote] = SpecialSymbolsHandlers.LeftSingleQuoteHandler;
            tagHandlers[RtfTags.TagRightDoubleQuote] = SpecialSymbolsHandlers.RightDoubleQuoteHandler;
            tagHandlers[RtfTags.TagLeftDoubleQuote] = SpecialSymbolsHandlers.LeftDoubleQuoteHandler;
            tagHandlers[RtfTags.TagEnDash] = SpecialSymbolsHandlers.EnDashHandler;
            tagHandlers[RtfTags.TagEmDash] = SpecialSymbolsHandlers.EmDashHandler;
            tagHandlers[RtfTags.TagEnSpace] = SpecialSymbolsHandlers.EnSpaceHandler;
            tagHandlers[RtfTags.TagEmSpace] = SpecialSymbolsHandlers.EmSpaceHandler;
            tagHandlers[RtfTags.TagBulltet] = SpecialSymbolsHandlers.BulletCharHandler;
            tagHandlers[RtfTags.TagPageBreak] = SpecialSymbolsHandlers.BreakHandler;
            tagHandlers[RtfTags.TagLineBreak] = SpecialSymbolsHandlers.BreakHandler;
            tagHandlers[RtfTags.TagColumnBreak] = SpecialSymbolsHandlers.BreakHandler;
        }

        private static void TabHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("\t");
        }

        private static void EnDashHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("–");
        }

        private static void EmDashHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("—");
        }

        private static void EnSpaceHandler(RtfTag tag, RtfImportContext context)
        {
            // This is an en space
            context.AddText(" ");
        }

        private static void EmSpaceHandler(RtfTag tag, RtfImportContext context)
        {
            // This is an em space
            context.AddText(" ");
        }

        private static void NonBreakingHyphenHandler(RtfTag tag, RtfImportContext context)
        {
            // This is an en-dash
            context.AddText("–");
        }

        private static void RightSingleQuoteHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("’");
        }

        private static void LeftSingleQuoteHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("‘");
        }

        private static void RightDoubleQuoteHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("”");
        }

        private static void LeftDoubleQuoteHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("“");
        }

        private static void BulletCharHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("•");
        }

        private static void NonBreakingSpaceHandler(RtfTag tag, RtfImportContext context)
        {
            context.AddText("\u00A0");
        }

        private static void BreakHandler(RtfTag tag, RtfImportContext context)
        {
            Break br = new Break(context.Document);
            switch (tag.Name)
            {
                case RtfTags.TagPageBreak:
                    br.BreakType = BreakType.PageBreak;
                    break;

                case RtfTags.TagLineBreak:
                    br.BreakType = BreakType.LineBreak;
                    br.TextWrappingRestartLocation = context.CurrentStyle.CurrentTextWrapping;
                    break;

                case RtfTags.TagColumnBreak:
                    br.BreakType = BreakType.ColumnBreak;
                    break;
            }

            context.AddInline(br);
        }
    }
}