﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class StyleSheetHandlers
    {
        public static void InitializeStyleDefinitionHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            tagHandlers[RtfTags.TagStyleCharacter] = StyleSheetHandlers.StyleDefinitionIdHandler;
            tagHandlers[RtfTags.TagStyleParagraph] = StyleSheetHandlers.StyleDefinitionIdHandler;
            tagHandlers[RtfTags.TagStyleTable] = StyleSheetHandlers.StyleDefinitionIdHandler;

            tagHandlers[RtfTags.TagStyleBasedOn] = StyleSheetHandlers.BasedOnStyleHandler;
            tagHandlers[RtfTags.TagStyleLinked] = StyleSheetHandlers.LinkedStyleHandler;
            tagHandlers[RtfTags.TagStyleNextParagraphStyle] = StyleSheetHandlers.NextParagraphStyleHandler;
            tagHandlers[RtfTags.TagStyleIsPrimary] = StyleSheetHandlers.IsPrimaryStyleHandler;
            tagHandlers[RtfTags.TagStyleUIPriority] = StyleSheetHandlers.UIPriorityStyleHandler;
        }

        private static void StyleDefinitionIdHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleCharacter, RtfTags.TagStyleParagraph, RtfTags.TagStyleTable);
            context.CurrentStyle.StyleDefinitionInfo.StyleId = tag.ValueAsNumber;
        }

        private static void BasedOnStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleBasedOn);
            context.CurrentStyle.StyleDefinitionInfo.BasedOnStyleId = tag.ValueAsNumber;
        }

        private static void LinkedStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleLinked);
            context.CurrentStyle.StyleDefinitionInfo.LinkedStyleId = tag.ValueAsNumber;
        }

        private static void NextParagraphStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleNextParagraphStyle);
            context.CurrentStyle.StyleDefinitionInfo.NextStyleId = tag.ValueAsNumber;
        }

        private static void IsPrimaryStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleIsPrimary);
            context.CurrentStyle.StyleDefinitionInfo.IsPrimary = true;
        }

        private static void UIPriorityStyleHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagStyleUIPriority);
            context.CurrentStyle.StyleDefinitionInfo.UIPriority = tag.ValueAsNumber;
        }
    }
}
