﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Exceptions;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.RtfTypes.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.TagHandlers
{
    internal static class TableStyleHandlers
    {
        public static void InitializeTableStyleHandlers(Dictionary<string, ControlTagHandler> tagHandlers)
        {
            // Table properties
            tagHandlers[RtfTags.TagRowDefaults] = TableStyleHandlers.ResetRowSettingsHandler;
            tagHandlers[RtfTags.TagStyleRowDefaults] = TableStyleHandlers.ResetRowSettingsHandler;
            tagHandlers[RtfTags.TagCellDefaults] = TableStyleHandlers.ResetCellSettingsHandler;

            tagHandlers[RtfTags.TagTableIndent] = TableStyleHandlers.TableIndentHandler;
            tagHandlers[RtfTags.TagTableIndentType] = TableStyleHandlers.TableIndentTypeHandler;
            tagHandlers[RtfTags.TagRowPosition] = TableStyleHandlers.TableRowPositionHandler;

            tagHandlers[RtfTags.TagRowAlignemntLeft] = TableStyleHandlers.TableAlignemntHandler;
            tagHandlers[RtfTags.TagRowAlignemntCenter] = TableStyleHandlers.TableAlignemntHandler;
            tagHandlers[RtfTags.TagRowAlignemntRight] = TableStyleHandlers.TableAlignemntHandler;

            tagHandlers[RtfTags.TagTableLookFirstRow] = TableStyleHandlers.TableLooksHandler;
            tagHandlers[RtfTags.TagTableLookLastRow] = TableStyleHandlers.TableLooksHandler;
            tagHandlers[RtfTags.TagTableLookFirstColumn] = TableStyleHandlers.TableLooksHandler;
            tagHandlers[RtfTags.TagTableLookLastColumn] = TableStyleHandlers.TableLooksHandler;
            tagHandlers[RtfTags.TagTableLookNoBandedRows] = TableStyleHandlers.BandedTableLooksHandler;
            tagHandlers[RtfTags.TagTableLookNoBandedColumns] = TableStyleHandlers.BandedTableLooksHandler;

            tagHandlers[RtfTags.TagRowShadingBackground] = TableStyleHandlers.TableRowShadingBackgroundHandler;
            tagHandlers[RtfTags.TagRowShadingForeground] = TableStyleHandlers.TableRowShadingForegroundHandler;
            tagHandlers[RtfTags.TagRowShadingPercent] = TableStyleHandlers.TableRowShadingPercentHandler;

            tagHandlers[RtfTags.TagRowNoSplit] = TableStyleHandlers.TableRowNoSplitHandler;
            tagHandlers[RtfTags.TagRowRepeatOnEveryPage] = TableStyleHandlers.TableRowRepeatOnEveryPageHandler;

            tagHandlers[RtfTags.TagTableColumnBanding] = TableStyleHandlers.TagTableColumnBandingHandler;
            tagHandlers[RtfTags.TagTableRowBanding] = TableStyleHandlers.TagTableRowBandingHandler;

            // Row settings
            tagHandlers[RtfTags.TagRowHeight] = TableStyleHandlers.RowSettingsHandler;
            tagHandlers[RtfTags.TagSpaceBetweenCells] = TableStyleHandlers.RowSettingsHandler;
            tagHandlers[RtfTags.TagRowAutoFit] = TableStyleHandlers.RowSettingsHandler;
            tagHandlers[RtfTags.TagRowPreferredWidth] = TableStyleHandlers.RowSettingsHandler;
            tagHandlers[RtfTags.TagRowPreferredWidthUnit] = TableStyleHandlers.RowSettingsHandler;

            tagHandlers[RtfTags.TagRowLtr] = TableStyleHandlers.RowLtrHandler;
            tagHandlers[RtfTags.TagRowRtl] = TableStyleHandlers.RowRtlHandler;
            tagHandlers[RtfTags.TagTableRtl] = TableStyleHandlers.TableRtlHandler;

            // Row borders
            tagHandlers[RtfTags.TagRowBordersTop] = TableStyleHandlers.RowBordersHandler;
            tagHandlers[RtfTags.TagRowBordersLeft] = TableStyleHandlers.RowBordersHandler;
            tagHandlers[RtfTags.TagRowBordersBottom] = TableStyleHandlers.RowBordersHandler;
            tagHandlers[RtfTags.TagRowBordersRight] = TableStyleHandlers.RowBordersHandler;
            tagHandlers[RtfTags.TagRowBordersInnerHorizontal] = TableStyleHandlers.RowBordersHandler;
            tagHandlers[RtfTags.TagRowBordersInnerVertical] = TableStyleHandlers.RowBordersHandler;

            // Cell borders
            tagHandlers[RtfTags.TagCellBorderTop] = TableStyleHandlers.CellBordersHandler;
            tagHandlers[RtfTags.TagCellBorderLeft] = TableStyleHandlers.CellBordersHandler;
            tagHandlers[RtfTags.TagCellBorderBottom] = TableStyleHandlers.CellBordersHandler;
            tagHandlers[RtfTags.TagCellBorderRight] = TableStyleHandlers.CellBordersHandler;
            tagHandlers[RtfTags.TagCellBorderDiagonalUpperLeft] = TableStyleHandlers.CellBordersHandler;
            tagHandlers[RtfTags.TagCellBorderDiagonalLowerLeft] = TableStyleHandlers.CellBordersHandler;

            // Cell settings
            tagHandlers[RtfTags.TagCellContentAlignmentTop] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellContentAlignmentTopInStyle] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellContentAlignmentCenter] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellContentAlignmentCenterInStyle] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellContentAlignmentBottom] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellContentAlignmentBottomInStyle] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellBackground] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellBackgroundWithStyles] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellForeground] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellForegroundWithStyles] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellForegroundInStyle] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellShadingPercent] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellShadingPercentWithStyles] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellShadingPercentInStyles] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellBackgroundInStyle] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellRightBoundry] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellPreferredWidth] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellPreferredWidthUnit] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellTextFlowLeftRightTopBottom] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellTextFlowRightLeftTopBottom] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellTextFlowLeftRightBottomTop] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellTextFlowLeftRightTopBottomVertical] = TableStyleHandlers.CellSettingsHandler;
            tagHandlers[RtfTags.TagCellTextFlowRightLeftTopBottomVertical] = TableStyleHandlers.CellSettingsHandler;

            tagHandlers[RtfTags.TagCellFirstInHorizontalRange] = TableStyleHandlers.CellMergeSettingsHandler;
            tagHandlers[RtfTags.TagCellInHorizontalRange] = TableStyleHandlers.CellMergeSettingsHandler;
            tagHandlers[RtfTags.TagCellFirstInVerticalRange] = TableStyleHandlers.CellMergeSettingsHandler;
            tagHandlers[RtfTags.TagCellInVerticalRange] = TableStyleHandlers.CellMergeSettingsHandler;

            tagHandlers[RtfTags.TagCellNoWrapText] = TableStyleHandlers.CellNoWrapTextHandler;
            tagHandlers[RtfTags.TagCellIgnoreCellMarkerInRowHeight] = TableStyleHandlers.CellIgnoreCellMarkerInRowHeightHandler;

            // Cell padding
            tagHandlers[RtfTags.TagCellPaddingLeft] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingTop] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingRight] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingBottom] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingUnitLeft] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingUnitTop] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingUnitRight] = TableStyleHandlers.CellPaddingHandler;
            tagHandlers[RtfTags.TagCellPaddingUnitBottom] = TableStyleHandlers.CellPaddingHandler;

            tagHandlers[RtfTags.TagRowDefaultCellPaddingLeft] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingTop] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingRight] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingBottom] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingUnitLeft] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingUnitTop] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingUnitRight] = TableStyleHandlers.RowCellPaddingHandler;
            tagHandlers[RtfTags.TagRowDefaultCellPaddingUnitBottom] = TableStyleHandlers.RowCellPaddingHandler;

            // Cell spacing
            tagHandlers[RtfTags.TagRowCellSpacingLeft] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingTop] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingRight] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingBottom] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingUnitLeft] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingUnitTop] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingUnitRight] = TableStyleHandlers.RowCellSpacingHandler;
            tagHandlers[RtfTags.TagRowCellSpacingUnitBottom] = TableStyleHandlers.RowCellSpacingHandler;

            // Borders generic handler
            BorderHandlers.InitializeBorderHandlers(tagHandlers);
        }

        private static void ResetRowSettingsHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowDefaults, RtfTags.TagStyleRowDefaults);

            context.CurrentStyle.ResetRowStyle();
        }

        private static void ResetCellSettingsHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagCellDefaults);

            context.CurrentRowStyle.EnsureCurrentCellStyle();
            context.CurrentRowStyle.CurrentCellStyle.ResetDefaults();
        }

        private static void RowBordersHandler(RtfTag tag, RtfImportContext context)
        {
            switch (tag.Name)
            {
                case RtfTags.TagRowBordersTop:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.Top;
                    break;
                case RtfTags.TagRowBordersLeft:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.Left;
                    break;
                case RtfTags.TagRowBordersBottom:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.Bottom;
                    break;
                case RtfTags.TagRowBordersRight:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.Right;
                    break;
                case RtfTags.TagRowBordersInnerHorizontal:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.InnerHorizontal;
                    break;
                case RtfTags.TagRowBordersInnerVertical:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.Borders.InnerVertical;
                    break;
                default:
                    throw new RtfUnexpectedElementException("RowBorders control word", tag.Name);
            }
            context.CurrentStyle.CurrentBorder.HasValue = true;
            context.CurrentStyle.CurrentBorderedElementType = BorderedElementType.TableRow;
        }

        private static void CellBordersHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();
            switch (tag.Name)
            {
                case RtfTags.TagCellBorderTop:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.Top;
                    break;
                case RtfTags.TagCellBorderLeft:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.Left;
                    break;
                case RtfTags.TagCellBorderBottom:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.Bottom;
                    break;
                case RtfTags.TagCellBorderRight:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.Right;
                    break;
                case RtfTags.TagCellBorderDiagonalLowerLeft:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.DiagonalLowerLeft;
                    break;
                case RtfTags.TagCellBorderDiagonalUpperLeft:
                    context.CurrentStyle.CurrentBorder = context.CurrentRowStyle.CurrentCellStyle.Borders.DiagonalUpperLeft;
                    break;
                default:
                    throw new RtfUnexpectedElementException("CellBorders control word", tag.Name);
            }
            context.CurrentStyle.CurrentBorder.HasValue = true;
            context.CurrentStyle.CurrentBorderedElementType = BorderedElementType.TableCell;
        }

        private static void CellSettingsHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();

            RtfColor rtfColor;
            switch (tag.Name)
            {
                case RtfTags.TagCellContentAlignmentTop:
                case RtfTags.TagCellContentAlignmentTopInStyle:
                    context.CurrentRowStyle.CurrentCellStyle.VerticalAlignment = VerticalAlignment.Top;
                    break;

                case RtfTags.TagCellContentAlignmentCenter:
                case RtfTags.TagCellContentAlignmentCenterInStyle:
                    context.CurrentRowStyle.CurrentCellStyle.VerticalAlignment = VerticalAlignment.Center;
                    break;

                case RtfTags.TagCellContentAlignmentBottom:
                case RtfTags.TagCellContentAlignmentBottomInStyle:
                    context.CurrentRowStyle.CurrentCellStyle.VerticalAlignment = VerticalAlignment.Bottom;
                    break;

                case RtfTags.TagCellBackground:
                case RtfTags.TagCellBackgroundWithStyles:
                case RtfTags.TagCellBackgroundInStyle:
                    if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
                    {
                        context.CurrentRowStyle.CurrentCellStyle.BackgroundColor = rtfColor.Color;
                    }
                    break;

                case RtfTags.TagCellForeground:
                case RtfTags.TagCellForegroundWithStyles:
                case RtfTags.TagCellForegroundInStyle:
                    if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
                    {
                        context.CurrentRowStyle.CurrentCellStyle.PatternColor = rtfColor.Color;
                    }
                    break;

                case RtfTags.TagCellShadingPercent:
                case RtfTags.TagCellShadingPercentWithStyles:
                case RtfTags.TagCellShadingPercentInStyles:
                    context.CurrentRowStyle.CurrentCellStyle.Pattern = RtfHelper.RtfTagToShadingPattern(tag);
                    break;

                case RtfTags.TagCellRightBoundry:
                    context.CurrentRowStyle.CurrentCellStyle.CellXBoundry = tag.ValueAsNumber;
                    context.CurrentRowStyle.CurrentCellStyle.IsLocked = true;
                    break;

                case RtfTags.TagCellPreferredWidth:
                    context.CurrentRowStyle.CurrentCellStyle.PreferredWidthValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPreferredWidthUnit:
                    context.CurrentRowStyle.CurrentCellStyle.PreferredWidthUnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellTextFlowLeftRightTopBottom:
                case RtfTags.TagCellTextFlowRightLeftTopBottom:
                case RtfTags.TagCellTextFlowLeftRightBottomTop:
                case RtfTags.TagCellTextFlowLeftRightTopBottomVertical:
                case RtfTags.TagCellTextFlowRightLeftTopBottomVertical:
                    TextDirection textDirection;
                    if (RtfHelper.TableCellTextDirectionMapper.TryGetToValue(tag.Name, out textDirection))
                    {
                        context.CurrentRowStyle.CurrentCellStyle.TextDirection = textDirection;
                    }
                    break;

                default:
                    throw new RtfUnexpectedElementException("CellSettings control word", tag.Name);
            }
        }

        private static void CellMergeSettingsHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();

            switch (tag.Name)
            {
                case RtfTags.TagCellFirstInHorizontalRange:
                    context.CurrentRowStyle.CurrentCellStyle.IsFirstInHorizontalRange = true;
                    break;

                case RtfTags.TagCellInHorizontalRange:
                    context.CurrentRowStyle.CurrentCellStyle.IsInHorizontalRange = true;
                    break;

                case RtfTags.TagCellFirstInVerticalRange:
                    context.CurrentRowStyle.CurrentCellStyle.IsFirstInVerticalRange = true;
                    break;

                case RtfTags.TagCellInVerticalRange:
                    context.CurrentRowStyle.CurrentCellStyle.IsInVerticalRange = true;
                    break;

                default:
                    throw new RtfUnexpectedElementException("CellMergeSettings control word", tag.Name);
            }
        }

        private static void CellPaddingHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();

            switch (tag.Name)
            {
                case RtfTags.TagCellPaddingLeft:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Left.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingTop:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Top.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingRight:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Right.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingBottom:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Bottom.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingUnitLeft:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Left.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingUnitTop:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Top.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingUnitRight:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Right.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagCellPaddingUnitBottom:
                    context.CurrentRowStyle.CurrentCellStyle.Padding.Bottom.UnitType = tag.ValueAsNumber;
                    break;
                default:
                    throw new RtfUnexpectedElementException("cell padding control word", tag.Name);
            }
        }

        private static void CellNoWrapTextHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();
            context.CurrentRowStyle.CurrentCellStyle.CanWrapContent = false;
        }

        private static void CellIgnoreCellMarkerInRowHeightHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();
            context.CurrentRowStyle.CurrentCellStyle.IgnoreCellMarkerInRowHeight = true;
        }

        private static void RowCellSpacingHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();

            switch (tag.Name)
            {
                case RtfTags.TagRowCellSpacingLeft:
                    context.CurrentRowStyle.DefaultCellSpacing.Left.RtfValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingTop:
                    context.CurrentRowStyle.DefaultCellSpacing.Top.RtfValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingRight:
                    context.CurrentRowStyle.DefaultCellSpacing.Right.RtfValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingBottom:
                    context.CurrentRowStyle.DefaultCellSpacing.Bottom.RtfValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingUnitLeft:
                    context.CurrentRowStyle.DefaultCellSpacing.Left.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingUnitTop:
                    context.CurrentRowStyle.DefaultCellSpacing.Top.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingUnitRight:
                    context.CurrentRowStyle.DefaultCellSpacing.Right.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowCellSpacingUnitBottom:
                    context.CurrentRowStyle.DefaultCellSpacing.Bottom.UnitType = tag.ValueAsNumber;
                    break;
                default:
                    throw new RtfUnexpectedElementException("cell spacing control word", tag.Name);
            }
        }

        private static void RowCellPaddingHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.EnsureCurrentCellStyle();

            switch (tag.Name)
            {
                case RtfTags.TagRowDefaultCellPaddingLeft:
                    context.CurrentRowStyle.CellPadding.Left.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingTop:
                    context.CurrentRowStyle.CellPadding.Top.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingRight:
                    context.CurrentRowStyle.CellPadding.Right.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingBottom:
                    context.CurrentRowStyle.CellPadding.Bottom.Value = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingUnitLeft:
                    context.CurrentRowStyle.CellPadding.Left.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingUnitTop:
                    context.CurrentRowStyle.CellPadding.Top.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingUnitRight:
                    context.CurrentRowStyle.CellPadding.Right.UnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowDefaultCellPaddingUnitBottom:
                    context.CurrentRowStyle.CellPadding.Bottom.UnitType = tag.ValueAsNumber;
                    break;
                default:
                    throw new RtfUnexpectedElementException("cell spacing control word", tag.Name);
            }
        }

        private static void RowSettingsHandler(RtfTag tag, RtfImportContext context)
        {
            switch (tag.Name)
            {
                case RtfTags.TagRowHeight:
                    if (tag.HasValue && tag.ValueAsNumber != 0)
                    {
                        HeightType type = tag.ValueAsNumber > 0 ? HeightType.AtLeast : HeightType.Exact;
                        double value = Unit.TwipToDip(Math.Abs(tag.ValueAsNumber));
                        context.CurrentRowStyle.RowHeight = new TableRowHeight(type, value);
                    }
                    break;

                case RtfTags.TagSpaceBetweenCells:
                    context.CurrentRowStyle.DefaultCellPadding = Unit.TwipToDip(tag.ValueAsNumber);
                    break;

                case RtfTags.TagRowPreferredWidth:
                    context.CurrentRowStyle.PreferredWidthValue = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowPreferredWidthUnit:
                    context.CurrentRowStyle.PreferredWidthUnitType = tag.ValueAsNumber;
                    break;

                case RtfTags.TagRowAutoFit:
                    context.CurrentRowStyle.AutoFit = (tag.ValueAsNumber == 1) ? TableLayoutType.AutoFit : TableLayoutType.FixedWidth;
                    break;

                default:
                    throw new RtfUnexpectedElementException("row settings control word", tag.Name);
            }
        }

        private static void RowLtrHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowLtr);
            context.CurrentRowStyle.FlowDirection = FlowDirection.LeftToRight;
        }

        private static void RowRtlHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowRtl);
            context.CurrentRowStyle.FlowDirection = FlowDirection.RightToLeft;
        }

        private static void TableRtlHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTableRtl);
            context.CurrentRowStyle.FlowDirection = FlowDirection.RightToLeft;
        }

        private static void TableIndentHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTableIndent);

            context.CurrentRowStyle.TableIndentValue = tag.ValueAsNumber;
        }

        private static void TableIndentTypeHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTableIndentType);

            context.CurrentRowStyle.TableIndentType = tag.ValueAsNumber;
        }

        private static void TagTableColumnBandingHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTableColumnBanding);

            context.CurrentRowStyle.ColumnBanding = tag.ValueAsNumber;
        }

        private static void TagTableRowBandingHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagTableRowBanding);

            context.CurrentRowStyle.RowBanding = tag.ValueAsNumber;
        }

        private static void TableRowPositionHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowPosition);

            context.CurrentRowStyle.TableRowPosition = tag.ValueAsNumber;
        }

        private static void TableAlignemntHandler(RtfTag tag, RtfImportContext context)
        {
            Util.EnsureTagName(tag, RtfTags.TagRowAlignemntLeft, RtfTags.TagRowAlignemntCenter, RtfTags.TagRowAlignemntRight);

            context.CurrentRowStyle.TableHorizontalAlignment = RtfHelper.GetTableHorzontalAlignmentFormRtf(tag.Name);
        }

        private static void TableLooksHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.TableLooks |= RtfHelper.TableLookMapper.GetToValue(tag.Name);
        }

        private static void BandedTableLooksHandler(RtfTag tag, RtfImportContext context)
        {
            // Control words for banded rows/cols should unset the flag
            context.CurrentRowStyle.TableLooks &= ~RtfHelper.TableLookMapper.GetToValue(tag.Name);
        }

        private static void TableRowShadingBackgroundHandler(RtfTag tag, RtfImportContext context)
        {
            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentRowStyle.BackgroundColor = rtfColor.Color;
            }
        }

        private static void TableRowShadingForegroundHandler(RtfTag tag, RtfImportContext context)
        {
            RtfColor rtfColor;
            if (context.ColorTable.TryGetColor(tag.ValueAsNumber, out rtfColor))
            {
                context.CurrentRowStyle.PatternColor = rtfColor.Color;
            }
        }

        private static void TableRowShadingPercentHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.Pattern = RtfHelper.RtfTagToShadingPattern(tag);
        }

        private static void TableRowNoSplitHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.CanSplit = false;
        }

        private static void TableRowRepeatOnEveryPageHandler(RtfTag tag, RtfImportContext context)
        {
            context.CurrentRowStyle.RepeatOnEveryPage = true;
        }
    }
}
