﻿using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf
{
    /// <summary>
    /// Format provider for importing and exporting documents to RTF format.
    /// </summary>
    public class RtfFormatProvider : TextBasedFormatProviderBase<RadFlowDocument>
    {
        private static readonly IEnumerable<string> supportedExtensionsList = new string[] { ".rtf" };

        /// <summary>
        /// Initializes a new instance of the <see cref="RtfFormatProvider" /> class.
        /// </summary>
        public RtfFormatProvider()
        {
            this.ExportSettings = new RtfExportSettings();
        }

        /// <summary>
        /// Gets the supported extensions.
        /// </summary>
        /// <value>The supported extensions.</value>
        public override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return RtfFormatProvider.supportedExtensionsList;
            }
        }

        /// <summary>
        /// Gets a value indicating whether format provider can import.
        /// </summary>
        /// <value>The value indicating whether can import.</value>
        public override bool CanImport
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Gets a value indicating whether format provider can export.
        /// </summary>
        /// <value>The value indicating whether can export.</value>
        public override bool CanExport
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Gets or sets the export settings.
        /// </summary>
        /// <value>The export settings.</value>
        public RtfExportSettings ExportSettings { get; set; }

        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The imported document.</returns>
        protected override RadFlowDocument ImportOverride(Stream input)
        {
            Guard.ThrowExceptionIfNull(input, "input");

            RtfDocumentImporter importer = new RtfDocumentImporter();

            return importer.Import(input);
        }

        /// <summary>
        /// Exports the specified document.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <param name="output">The output.</param>
        protected override void ExportOverride(RadFlowDocument document, Stream output)
        {
            Guard.ThrowExceptionIfNull(document, "document");
            Guard.ThrowExceptionIfNull(output, "output");

            using (new RadFlowDocumentLicenseCheck(document))
            {
                RtfDocumentExporter exporter = new RtfDocumentExporter(document, output, this.ExportSettings);
                exporter.Export();
            }
        }
    }
}
