﻿using System;
using System.Collections.Generic;
using System.Text;
#if NETSTANDARD
using Telerik.Documents.Media;
using Telerik.Documents.Primitives;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Import.Model;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Rtf.Utilities
{
    internal static class RtfHelper
    {
        public static readonly double DefaultLineSpacing = 240.0;
        public static readonly int DefaultFloatingBlockHorizontalMargin = 114305;
        public static readonly double RtfAngleFactor = 65536.0;
        public static readonly int ImageShapeType = 75;
        public static readonly int TextShapeType = 136;

        public static readonly Core.Data.ValueMapper<string, TableLooks> TableLookMapper;
        public static readonly Core.Data.ValueMapper<string, BorderStyle> BorderMapper;
        public static readonly Core.Data.ValueMapper<string, SectionType> SectionTypeMapper;
        public static readonly Core.Data.ValueMapper<int, DocumentViewType> DocumentViewTypeMapper;
        public static readonly Core.Data.ValueMapper<int, ShapeWrappingType> WrappingStyleMapper;
        public static readonly Core.Data.ValueMapper<int, TextWrap> TextWrapMapper;
        public static readonly Core.Data.ValueMapper<int, VerticalRelativeFrom> VerticalRelativeFromMapper;
        public static readonly Core.Data.ValueMapper<int, HorizontalRelativeFrom> HorizontalRelativeFromMapper;
        public static readonly Core.Data.ValueMapper<int, RelativeVerticalAlignment> RelativeVerticalAlignmentMapper;
        public static readonly Core.Data.ValueMapper<int, RelativeHorizontalAlignment> RelativeHorizontalAlignmentMapper;
        public static readonly Core.Data.ValueMapper<int, TextWrappingRestartLocation> LineBreakTextWrappingMapper;
        public static readonly Core.Data.ValueMapper<int, Alignment> ListLevelAlignmentMapper;
        public static readonly Core.Data.ValueMapper<string, NumberingStyle> NumberingStyleMapper;
        public static readonly Core.Data.ValueMapper<string, ChapterSeparatorType> ChapterSeparatorTypeMapper;
        public static readonly Core.Data.ValueMapper<string, TabStopType> TabStopTypeMapper;
        public static readonly Core.Data.ValueMapper<string, TabStopLeader> TabStopLeaderMapper;
        public static readonly Core.Data.ValueMapper<string, TextDirection> TableCellTextDirectionMapper;

        private static readonly Dictionary<Alignment, string> alignmentToRtfTag = new Dictionary<Alignment, string>()
        {
            { Alignment.Center, RtfTags.TagAlignCenter },
            { Alignment.Justified, RtfTags.TagAlignJustify },
            { Alignment.Left, RtfTags.TagAlignLeft },
            { Alignment.Right, RtfTags.TagAlignRight },
            { Alignment.Distribute, RtfTags.TagAlignDistribute },
            { Alignment.ThaiDistribute, RtfTags.TagAlignThai },
            { Alignment.LowKashida, RtfTags.TagAlignKashida },
            { Alignment.MediumKashida, RtfTags.TagAlignKashida },
            { Alignment.HighKashida, RtfTags.TagAlignKashida },
        };

        private static readonly Dictionary<UnderlinePattern, string> undelineDecorationToRtfTag = new Dictionary<UnderlinePattern, string>()
        {
            { UnderlinePattern.None, RtfTags.TagUnderlineNone },
            { UnderlinePattern.Single, RtfTags.TagUnderlineLine },
            { UnderlinePattern.Double, RtfTags.TagUnderlineDouble },
            { UnderlinePattern.Thick, RtfTags.TagUnderlineThick },
            { UnderlinePattern.Wave, RtfTags.TagUnderlineWave },
            { UnderlinePattern.Dotted, RtfTags.TagUnderlineDotted },
            { UnderlinePattern.Dash, RtfTags.TagUnderlineDashed },
            { UnderlinePattern.DotDash, RtfTags.TagUnderlineDashDotted },
            { UnderlinePattern.DotDotDash, RtfTags.TagUnderlineDashDotDotted },
        };

        private static readonly Dictionary<string, UnderlinePattern> rtfTagToUndelineDecoration = new Dictionary<string, UnderlinePattern>()
        {
            { RtfTags.TagUnderlineNone, UnderlinePattern.None },
            { RtfTags.TagUnderlineLine, UnderlinePattern.Single },
            { RtfTags.TagUnderlineDotted, UnderlinePattern.Dotted },
            { RtfTags.TagUnderlineThickDotted, UnderlinePattern.DottedHeavy },
            { RtfTags.TagUnderlineDashed, UnderlinePattern.Dash },
            { RtfTags.TagUnderlineThickDashed, UnderlinePattern.DashedHeavy },
            { RtfTags.TagUnderlineLongDashed, UnderlinePattern.DashLong },
            { RtfTags.TagUnderlineThickLongDashed, UnderlinePattern.DashLongHeavy },
            { RtfTags.TagUnderlineDashDotted, UnderlinePattern.DotDash },
            { RtfTags.TagUnderlineThickDashDotted, UnderlinePattern.DashDotHeavy },
            { RtfTags.TagUnderlineDashDotDotted, UnderlinePattern.DotDotDash },
            { RtfTags.TagUnderlineThickDashDotDotted, UnderlinePattern.DashDotDotHeavy },
            { RtfTags.TagUnderlineDouble, UnderlinePattern.Double },
            { RtfTags.TagUnderlineThick, UnderlinePattern.Thick },
            { RtfTags.TagUnderlineWord, UnderlinePattern.Words },
            { RtfTags.TagUnderlineWave, UnderlinePattern.Wave },
            { RtfTags.TagUnderlineHeavyWave, UnderlinePattern.WavyHeavy },
            { RtfTags.TagUnderlineDoubleWave, UnderlinePattern.WavyDouble },
        };

        // TODO: Implement tab stops.
        ////private static readonly Dictionary<string, TabStopType> rtfTagToTabStopType = new Dictionary<string, TabStopType>()
        ////{
        ////    { RtfTags.TagTabStopRight, TabStopType.Right },
        ////    { RtfTags.TagTabStopCenter, TabStopType.Center },
        ////    { RtfTags.TagTabStopDeciaml, TabStopType.Decimal },
        ////};

        ////private static readonly Dictionary<TabStopType, string> tabStopTypeToRtfType = new Dictionary<TabStopType, string>()
        ////{
        ////    ////{TabStopType.Left, null }, this is the default value
        ////    { TabStopType.Right, RtfTags.TagTabStopRight },
        ////    { TabStopType.Center, RtfTags.TagTabStopCenter },
        ////    { TabStopType.Decimal, RtfTags.TagTabStopDeciaml },
        ////};

        private static readonly Dictionary<ShadingPattern, double> PatternToShadingPrecent = new Dictionary<ShadingPattern, double>()
        {
            { ShadingPattern.Percent5, 5.0 },
            { ShadingPattern.Percent10, 10.0 },
            { ShadingPattern.Percent12, 12.5 },
            { ShadingPattern.Percent15, 15.0 },
            { ShadingPattern.Percent20, 20.0 },
            { ShadingPattern.Percent25, 25.0 },
            { ShadingPattern.Percent30, 30.0 },
            { ShadingPattern.Percent35, 35.0 },
            { ShadingPattern.Percent37, 37.5 },
            { ShadingPattern.Percent40, 40.0 },
            { ShadingPattern.Percent45, 45.0 },
            { ShadingPattern.Percent50, 50.0 },
            { ShadingPattern.Percent55, 55.0 },
            { ShadingPattern.Percent60, 60.0 },
            { ShadingPattern.Percent62, 62.5 },
            { ShadingPattern.Percent65, 65.0 },
            { ShadingPattern.Percent70, 70.0 },
            { ShadingPattern.Percent75, 75.0 },
            { ShadingPattern.Percent80, 80.0 },
            { ShadingPattern.Percent85, 85.0 },
            { ShadingPattern.Percent87, 87.5 },
            { ShadingPattern.Percent90, 90.0 },
            { ShadingPattern.Percent95, 95.0 },
        };

        private static readonly List<Tuple<double, ShadingPattern>> ShadingPrecentToPattern = new List<Tuple<double, ShadingPattern>>()
        {
            new Tuple<double, ShadingPattern>(0, ShadingPattern.Clear),
            new Tuple<double, ShadingPattern>(5.0, ShadingPattern.Percent5),
            new Tuple<double, ShadingPattern>(10.0, ShadingPattern.Percent10),
            new Tuple<double, ShadingPattern>(12.5, ShadingPattern.Percent12),
            new Tuple<double, ShadingPattern>(15.0, ShadingPattern.Percent15),
            new Tuple<double, ShadingPattern>(20.0, ShadingPattern.Percent20),
            new Tuple<double, ShadingPattern>(25.0, ShadingPattern.Percent25),
            new Tuple<double, ShadingPattern>(30.0, ShadingPattern.Percent30),
            new Tuple<double, ShadingPattern>(35.0, ShadingPattern.Percent35),
            new Tuple<double, ShadingPattern>(37.5, ShadingPattern.Percent37),
            new Tuple<double, ShadingPattern>(40.0, ShadingPattern.Percent40),
            new Tuple<double, ShadingPattern>(45.0, ShadingPattern.Percent45),
            new Tuple<double, ShadingPattern>(50.0, ShadingPattern.Percent50),
            new Tuple<double, ShadingPattern>(55.0, ShadingPattern.Percent55),
            new Tuple<double, ShadingPattern>(60.0, ShadingPattern.Percent60),
            new Tuple<double, ShadingPattern>(62.5, ShadingPattern.Percent62),
            new Tuple<double, ShadingPattern>(65.0, ShadingPattern.Percent65),
            new Tuple<double, ShadingPattern>(70.0, ShadingPattern.Percent70),
            new Tuple<double, ShadingPattern>(75.0, ShadingPattern.Percent75),
            new Tuple<double, ShadingPattern>(80.0, ShadingPattern.Percent80),
            new Tuple<double, ShadingPattern>(85.0, ShadingPattern.Percent85),
            new Tuple<double, ShadingPattern>(87.5, ShadingPattern.Percent87),
            new Tuple<double, ShadingPattern>(90.0, ShadingPattern.Percent90),
            new Tuple<double, ShadingPattern>(95.0, ShadingPattern.Percent95),
        };

        private static int[] hexCharValues;

        static RtfHelper()
        {
            TableLookMapper = new Documents.Core.Data.ValueMapper<string, TableLooks>();
            TableLookMapper.AddPair(RtfTags.TagTableLookFirstRow, TableLooks.FirstRow);
            TableLookMapper.AddPair(RtfTags.TagTableLookLastRow, TableLooks.LastRow);
            TableLookMapper.AddPair(RtfTags.TagTableLookFirstColumn, TableLooks.FirstColumn);
            TableLookMapper.AddPair(RtfTags.TagTableLookLastColumn, TableLooks.LastColumn);
            TableLookMapper.AddPair(RtfTags.TagTableLookNoBandedRows, TableLooks.BandedRows);
            TableLookMapper.AddPair(RtfTags.TagTableLookNoBandedColumns, TableLooks.BandedColumns);

            BorderMapper = new Core.Data.ValueMapper<string, BorderStyle>(RtfTags.TagBorderTypeSingleThickness, BorderStyle.Single);
            BorderMapper.AddPair(RtfTags.TagBorderTypeNoBorder, BorderStyle.None);
            BorderMapper.AddPair(RtfTags.TagBorderTypeShadowed, BorderStyle.Single);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDouble, BorderStyle.Double);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDotted, BorderStyle.Dotted);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDashed, BorderStyle.Dashed);
            BorderMapper.AddPair(RtfTags.TagBorderTypeHairline, BorderStyle.Single);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDashedSmall, BorderStyle.DashSmallGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDotDashed, BorderStyle.DotDash);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDotDotDashed, BorderStyle.DotDotDash);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDotDashedAlt, BorderStyle.DotDash);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDotDotDashedAlt, BorderStyle.DotDotDash);
            BorderMapper.AddPair(RtfTags.TagBorderTypeInset, BorderStyle.Inset);
            BorderMapper.AddPair(RtfTags.TagBorderTypeOutset, BorderStyle.Outset);
            BorderMapper.AddPair(RtfTags.TagBorderTypeTriple, BorderStyle.Triple);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThickThinS, BorderStyle.ThickThinSmallGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickS, BorderStyle.ThinThickSmallGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickThinS, BorderStyle.ThinThickThinSmallGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThickThinM, BorderStyle.ThickThinMediumGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickM, BorderStyle.ThinThickMediumGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickThinM, BorderStyle.ThinThickThinMediumGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThickThinL, BorderStyle.ThickThinLargeGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickL, BorderStyle.ThinThickLargeGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeThinThickThinL, BorderStyle.ThinThickThinLargeGap);
            BorderMapper.AddPair(RtfTags.TagBorderTypeWavy, BorderStyle.Wave);
            BorderMapper.AddPair(RtfTags.TagBorderTypeDoubleWavy, BorderStyle.DoubleWave);
            BorderMapper.AddPair(RtfTags.TagBorderTypeStriped, BorderStyle.DashDotStroked);
            BorderMapper.AddPair(RtfTags.TagBorderTypeEmbossed, BorderStyle.ThreeDEmboss);
            BorderMapper.AddPair(RtfTags.TagBorderTypeEngraved, BorderStyle.ThreeDEngrave);

            // Single value should be defined last
            BorderMapper.AddPair(RtfTags.TagBorderTypeNoBorder, BorderStyle.Inherit);
            BorderMapper.AddPair(RtfTags.TagBorderTypeSingleThickness, BorderStyle.Single);

            SectionTypeMapper = new Core.Data.ValueMapper<string, SectionType>(RtfTags.TagSectionBreakNewPage, SectionType.NextPage);
            SectionTypeMapper.AddPair(RtfTags.TagSectionBreakNewPage, SectionType.NextPage);
            SectionTypeMapper.AddPair(RtfTags.TagSectionBreakOddPage, SectionType.OddPage);
            SectionTypeMapper.AddPair(RtfTags.TagSectionBreakEvenPage, SectionType.EvenPage);
            SectionTypeMapper.AddPair(RtfTags.TagSectionBreakNone, SectionType.Continuous);
            SectionTypeMapper.AddPair(RtfTags.TagSectionBreakColumn, SectionType.NextColumn);

            DocumentViewTypeMapper = new Core.Data.ValueMapper<int, DocumentViewType>();
            DocumentViewTypeMapper.AddPair(0, DocumentViewType.None);
            DocumentViewTypeMapper.AddPair(1, DocumentViewType.PrintLayout);
            DocumentViewTypeMapper.AddPair(2, DocumentViewType.Outline);
            DocumentViewTypeMapper.AddPair(3, DocumentViewType.MasterPages);
            DocumentViewTypeMapper.AddPair(4, DocumentViewType.Normal);
            DocumentViewTypeMapper.AddPair(5, DocumentViewType.WebLayout);

            WrappingStyleMapper = new Core.Data.ValueMapper<int, ShapeWrappingType>(2, ShapeWrappingType.Square);
            WrappingStyleMapper.AddPair(1, ShapeWrappingType.TopAndBottom);
            WrappingStyleMapper.AddPair(2, ShapeWrappingType.Square);
            WrappingStyleMapper.AddPair(3, ShapeWrappingType.None);

            TextWrapMapper = new Core.Data.ValueMapper<int, TextWrap>(0, TextWrap.BothSides);
            TextWrapMapper.AddPair(0, TextWrap.BothSides);
            TextWrapMapper.AddPair(1, TextWrap.LeftOnly);
            TextWrapMapper.AddPair(2, TextWrap.RightOnly);
            TextWrapMapper.AddPair(3, TextWrap.Largest);

            VerticalRelativeFromMapper = new Core.Data.ValueMapper<int, VerticalRelativeFrom>(0, VerticalRelativeFrom.Margin);
            VerticalRelativeFromMapper.AddPair(0, VerticalRelativeFrom.Margin);
            VerticalRelativeFromMapper.AddPair(1, VerticalRelativeFrom.Page);
            VerticalRelativeFromMapper.AddPair(2, VerticalRelativeFrom.Paragraph);
            VerticalRelativeFromMapper.AddPair(3, VerticalRelativeFrom.Line);
            VerticalRelativeFromMapper.AddPair(4, VerticalRelativeFrom.TopMargin);
            VerticalRelativeFromMapper.AddPair(5, VerticalRelativeFrom.BottomMargin);
            VerticalRelativeFromMapper.AddPair(6, VerticalRelativeFrom.InsideMargin);
            VerticalRelativeFromMapper.AddPair(7, VerticalRelativeFrom.OutsideMargin);

            HorizontalRelativeFromMapper = new Core.Data.ValueMapper<int, HorizontalRelativeFrom>(0, HorizontalRelativeFrom.Margin);
            HorizontalRelativeFromMapper.AddPair(0, HorizontalRelativeFrom.Margin);
            HorizontalRelativeFromMapper.AddPair(1, HorizontalRelativeFrom.Page);
            HorizontalRelativeFromMapper.AddPair(2, HorizontalRelativeFrom.Column);
            HorizontalRelativeFromMapper.AddPair(3, HorizontalRelativeFrom.Character);
            HorizontalRelativeFromMapper.AddPair(4, HorizontalRelativeFrom.LeftMargin);
            HorizontalRelativeFromMapper.AddPair(5, HorizontalRelativeFrom.RightMargin);
            HorizontalRelativeFromMapper.AddPair(6, HorizontalRelativeFrom.InsideMargin);
            HorizontalRelativeFromMapper.AddPair(7, HorizontalRelativeFrom.OutsideMargin);

            RelativeVerticalAlignmentMapper = new Core.Data.ValueMapper<int, RelativeVerticalAlignment>(1, RelativeVerticalAlignment.Top);
            RelativeVerticalAlignmentMapper.AddPair(1, RelativeVerticalAlignment.Top);
            RelativeVerticalAlignmentMapper.AddPair(2, RelativeVerticalAlignment.Center);
            RelativeVerticalAlignmentMapper.AddPair(3, RelativeVerticalAlignment.Bottom);
            RelativeVerticalAlignmentMapper.AddPair(4, RelativeVerticalAlignment.Inside);
            RelativeVerticalAlignmentMapper.AddPair(5, RelativeVerticalAlignment.Outside);

            RelativeHorizontalAlignmentMapper = new Core.Data.ValueMapper<int, RelativeHorizontalAlignment>(1, RelativeHorizontalAlignment.Left);
            RelativeHorizontalAlignmentMapper.AddPair(1, RelativeHorizontalAlignment.Left);
            RelativeHorizontalAlignmentMapper.AddPair(2, RelativeHorizontalAlignment.Center);
            RelativeHorizontalAlignmentMapper.AddPair(3, RelativeHorizontalAlignment.Right);
            RelativeHorizontalAlignmentMapper.AddPair(4, RelativeHorizontalAlignment.Inside);
            RelativeHorizontalAlignmentMapper.AddPair(5, RelativeHorizontalAlignment.Outside);

            LineBreakTextWrappingMapper = new Core.Data.ValueMapper<int, TextWrappingRestartLocation>(0, TextWrappingRestartLocation.NextLine);
            LineBreakTextWrappingMapper.AddPair(1, TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft);
            LineBreakTextWrappingMapper.AddPair(2, TextWrappingRestartLocation.NextTextRegionUnblockedOnRight);
            LineBreakTextWrappingMapper.AddPair(3, TextWrappingRestartLocation.NextFullLine);

            ListLevelAlignmentMapper = new Core.Data.ValueMapper<int, Alignment>(0, Alignment.Left);
            ListLevelAlignmentMapper.AddPair(1, Alignment.Center);
            ListLevelAlignmentMapper.AddPair(2, Alignment.Right);

            NumberingStyleMapper = new Core.Data.ValueMapper<string, NumberingStyle>();
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatDecimal, NumberingStyle.Decimal);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatUppercaseRomanNumeral, NumberingStyle.UpperRoman);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatLowerRomanNumeral, NumberingStyle.LowerRoman);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatUppercaseLetter, NumberingStyle.UpperLetter);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatLowercaseLetter, NumberingStyle.LowerLetter);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatKoreanNumbering, NumberingStyle.Chosung);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatKoreanNumbering2, NumberingStyle.KoreanDigital2);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatCircleNumbering, NumberingStyle.DecimalEnclosedCircle);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatArabicAbjad, NumberingStyle.ArabicAbjad);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatHindiVowel, NumberingStyle.HindiVowels);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatHindiConsonants, NumberingStyle.HindiConsonants);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatHindiNumbers, NumberingStyle.HindiNumbers);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatHindiCounting, NumberingStyle.HindiCounting);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatThaiLetters, NumberingStyle.ThaiLetters);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatThaiNumbers, NumberingStyle.ThaiNumbers);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatThaiCounting, NumberingStyle.ThaiCounting);
            NumberingStyleMapper.AddPair(RtfTags.TagSectionPageNumberFormatVietnameseCounting, NumberingStyle.VietnameseCounting);

            ChapterSeparatorTypeMapper = new Core.Data.ValueMapper<string, ChapterSeparatorType>();
            ChapterSeparatorTypeMapper.AddPair(RtfTags.TagSectionChapterSeparatorCharacterColon, ChapterSeparatorType.Colon);
            ChapterSeparatorTypeMapper.AddPair(RtfTags.TagSectionChapterSeparatorCharacterEmDash, ChapterSeparatorType.EmDash);
            ChapterSeparatorTypeMapper.AddPair(RtfTags.TagSectionChapterSeparatorCharacterEnDash, ChapterSeparatorType.EnDash);
            ChapterSeparatorTypeMapper.AddPair(RtfTags.TagSectionChapterSeparatorCharacterHyphen, ChapterSeparatorType.Hyphen);
            ChapterSeparatorTypeMapper.AddPair(RtfTags.TagSectionChapterSeparatorCharacterPeriod, ChapterSeparatorType.Period);

            TabStopTypeMapper = new Core.Data.ValueMapper<string, TabStopType>();
            TabStopTypeMapper.AddPair(string.Empty, TabStopType.Left);
            TabStopTypeMapper.AddPair(string.Empty, TabStopType.Clear);
            TabStopTypeMapper.AddPair(RtfTags.TagTabStopCenter, TabStopType.Center);
            TabStopTypeMapper.AddPair(RtfTags.TagTabStopDecimal, TabStopType.Decimal);
            TabStopTypeMapper.AddPair(RtfTags.TagTabStopRight, TabStopType.Right);

            TabStopLeaderMapper = new Core.Data.ValueMapper<string, TabStopLeader>();
            TabStopLeaderMapper.AddPair(string.Empty, TabStopLeader.None);
            TabStopLeaderMapper.AddPair(RtfTags.TagTabStopLeaderDots, TabStopLeader.Dot);
            TabStopLeaderMapper.AddPair(RtfTags.TagTabStopLeaderMiddleDots, TabStopLeader.MiddleDot);
            TabStopLeaderMapper.AddPair(RtfTags.TagTabStopLeaderHyphens, TabStopLeader.Hyphen);
            TabStopLeaderMapper.AddPair(RtfTags.TagTabStopLeaderUnderline, TabStopLeader.Underscore);
            TabStopLeaderMapper.AddPair(RtfTags.TagTabStopLeaderThickLine, TabStopLeader.Underscore);

            TableCellTextDirectionMapper = new Core.Data.ValueMapper<string, TextDirection>();
            TableCellTextDirectionMapper.AddPair(RtfTags.TagCellTextFlowLeftRightTopBottom, TextDirection.LeftToRightTopToBottom);
            TableCellTextDirectionMapper.AddPair(RtfTags.TagCellTextFlowLeftRightBottomTop, TextDirection.BottomToTopLeftToRight);
            TableCellTextDirectionMapper.AddPair(RtfTags.TagCellTextFlowLeftRightTopBottomVertical, TextDirection.LeftToRightTopToBottomRotated);
            TableCellTextDirectionMapper.AddPair(RtfTags.TagCellTextFlowRightLeftTopBottom, TextDirection.TopToBottomRightToLeft);
            TableCellTextDirectionMapper.AddPair(RtfTags.TagCellTextFlowRightLeftTopBottomVertical, TextDirection.TopToBottomRightToLeftRotated);
        }

        ////public static DocumentLayoutMode RtfLayoutModeToDocumentLayoutMode(int rtfMode)
        ////{
        ////    if (rtfMode == 1)
        ////    {
        ////        return DocumentLayoutMode.Paged;
        ////    }
        ////    else
        ////    {
        ////        return DocumentLayoutMode.Flow;
        ////    }
        ////}
        ////public static int DocumentLayoutModeToRtfLayoutMode(DocumentLayoutMode docMode)
        ////{
        ////    //0     None
        ////    //1     Page Layout view
        ////    //2     Outline view
        ////    //3     Master Document view
        ////    //4     Draft view
        ////    //5     Online Layout view
        ////    if (docMode == DocumentLayoutMode.Paged)
        ////    {
        ////        return 1;
        ////    }
        ////    else
        ////    {
        ////        return 4;
        ////    }
        ////}

        public static string AlignmentToRtfTag(Alignment align, bool isRtl)
        {
            if (isRtl)
            {
                // Reverse paragraph alignment if right to left.
                if (align == Alignment.Left)
                {
                    align = Alignment.Right;
                }
                else if (align == Alignment.Right)
                {
                    align = Alignment.Left;
                }
            }

            return alignmentToRtfTag[align];
        }

        public static Alignment RtfTagToAlignment(RtfTag tag)
        {
            Alignment result = Alignment.Left;
            switch (tag.Name)
            {
                case RtfTags.TagAlignLeft:
                    result = Alignment.Left;
                    break;
                case RtfTags.TagAlignCenter:
                    result = Alignment.Center;
                    break;
                case RtfTags.TagAlignRight:
                    result = Alignment.Right;
                    break;
                case RtfTags.TagAlignJustify:
                    result = Alignment.Justified;
                    break;
                case RtfTags.TagAlignDistribute:
                    result = Alignment.Distribute;
                    break;
                case RtfTags.TagAlignThai:
                    result = Alignment.ThaiDistribute;
                    break;
                case RtfTags.TagAlignKashida:
                    if (tag.ValueAsNumber <= 0)
                    {
                        result = Alignment.LowKashida;
                    }
                    else if (tag.ValueAsNumber <= 10)
                    {
                        result = Alignment.MediumKashida;
                    }
                    else
                    {
                        result = Alignment.HighKashida;
                    }
                    break;
            }

            return result;
        }

        public static UnderlinePattern RtfTagToUndelineDecoration(string tag)
        {
            UnderlinePattern result = UnderlinePattern.Single;

            rtfTagToUndelineDecoration.TryGetValue(tag, out result);

            return result;
        }

        public static string UndelineDecorationToRtfTag(UnderlinePattern decoration)
        {
            string result = RtfTags.TagUnderlineLine;

            undelineDecorationToRtfTag.TryGetValue(decoration, out result);

            return result;
        }

        public static VerticalRelativeFrom RtfValueToVerticalRelativeFrom(int rtfValue, VerticalRelativeFrom defaultValue)
        {
            if (VerticalRelativeFromMapper.ContainsFromValue(rtfValue))
            {
                return VerticalRelativeFromMapper.GetToValue(rtfValue);
            }
            return defaultValue;
        }
        public static HorizontalRelativeFrom RtfValueToHorizontalRelativeFrom(int rtfValue, HorizontalRelativeFrom defaultValue)
        {
            if (HorizontalRelativeFromMapper.ContainsFromValue(rtfValue))
            {
                return HorizontalRelativeFromMapper.GetToValue(rtfValue);
            }
            return defaultValue;
        }

        ////public static int VerticalRelativeFromToRtfValue(VerticalRelativeFrom value)
        ////{
        ////    int result = 2;
        ////    verticalRelativeFromToRtfValue.TryGetValue(value, out result);
        ////    return result;
        ////}
        ////public static int HorizontalRelativeFromToRtfValue(HorizontalRelativeFrom value)
        ////{
        ////    int result = 2;
        ////    horizontalRelativeFromToRtfValue.TryGetValue(value, out result);
        ////    return result;
        ////}

        // TODO: Implement tab stops.
        ////public static TabStopType RtfTagToTabStopType(string tag)
        ////{
        ////    TabStopType result = TabStopType.Left;

        ////    rtfTagToTabStopType.TryGetValue(tag, out result);

        ////    return result;
        ////}

        ////public static string TabStopTypeToRtfTag(TabStopType type)
        ////{
        ////    string result = null;

        ////    tabStopTypeToRtfType.TryGetValue(type, out result);

        ////    return result;
        ////}

        public static string GetGroupText(RtfGroup group, bool recoursive = true)
        {
            StringBuilder sb = new StringBuilder();
            GetGroupText(group, sb, recoursive);
            return sb.ToString();
        }

        public static byte[] GetBytesFromHexString(string hexString)
        {
            if (string.IsNullOrEmpty(hexString))
            {
                return new byte[0];
            }

            if (hexCharValues == null)
            {
                hexCharValues = new int[128];
                for (int i = 0; i < 10; ++i)
                {
                    hexCharValues['0' + i] = i;
                }
                for (int i = 0; i < 6; ++i)
                {
                    hexCharValues['a' + i] = 0xA + i;
                    hexCharValues['A' + i] = 0xA + i;
                }
            }

            byte[] result = new byte[hexString.Length >> 1];
            for (int i = 0, l = hexString.Length; i < l; i += 2)
            {
                result[i >> 1] = (byte)((hexCharValues[hexString[i]] << 4) | hexCharValues[hexString[i + 1]]);
            }
            return result;
        }

        public static TableWidthUnit ConvertTableWidthUnitType(int unitType, int parameter)
        {
            TableWidthUnit result;
            if (unitType == 2)
            {
                result = new TableWidthUnit(TableWidthUnitType.Percent, ((float)parameter) / 50.0);
            }
            else if (unitType == 3)
            {
                result = new TableWidthUnit(TableWidthUnitType.Fixed, Unit.TwipToDipF(parameter));
            }
            else
            {
                result = new TableWidthUnit(TableWidthUnitType.Auto);
            }

            return result;
        }

        public static double GetTableIndentFormRtf(int value, int type)
        {
            double result = 0;

            switch (type)
            {
                case 0:
                case 2:
                    result = 0.0;
                    break;
                case 1:
                case 3:
                    // Table indent cannot be defined with percents in our model, so use twips instead.
                    result = Unit.TwipToDip(value);
                    break;
            }

            return result;
        }

        public static Alignment GetTableHorzontalAlignmentFormRtf(string tag)
        {
            Alignment result = Alignment.Left;

            switch (tag)
            {
                case RtfTags.TagRowAlignemntCenter:
                    result = Alignment.Center;
                    break;
                case RtfTags.TagRowAlignemntRight:
                    result = Alignment.Right;
                    break;
            }

            return result;
        }

        public static string TableHorzontalAlignmentToRtfTag(Alignment align)
        {
            string result = RtfTags.TagRowAlignemntLeft;

            // Only Left, Right and Center alignments are supported.
            switch (align)
            {
                case Alignment.Center:
                    result = RtfTags.TagRowAlignemntCenter;
                    break;
                case Alignment.Right:
                    result = RtfTags.TagRowAlignemntRight;
                    break;
            }

            return result;
        }

        public static Size CalculateImageSizeForRotateAngle(Size orginalSize, double rotateAngle)
        {
            double normalizedAngle = Math.Abs(rotateAngle) % 180d;
            if (normalizedAngle > 45 && normalizedAngle < 135d)
            {
                return new Size(orginalSize.Height, orginalSize.Width);
            }
            else
            {
                return orginalSize;
            }
        }

        public static int ConvertDateTimeToRtfInt(DateTime date)
        {
            if (date.Year < 1900)
            {
                return 0;
            }

            int dateBitNumbers = 0 | date.Minute;
            dateBitNumbers = dateBitNumbers | date.Hour << 6;
            dateBitNumbers = dateBitNumbers | date.Day << 11;
            dateBitNumbers = dateBitNumbers | date.Month << 16;
            dateBitNumbers = dateBitNumbers | (date.Year - 1900) << 20;

            return dateBitNumbers;
        }

        public static DateTime ConvertRtfIntToDateTime(int numeric)
        {
            if (numeric == 0)
            {
                return DateTime.MinValue;
            }

            int minute = numeric & 63;
            int hour = (numeric & 1984) >> 6;
            int day = (numeric & 63488) >> 11;
            int month = (numeric & 983040) >> 16;
            int year = ((numeric & 535822336) >> 20) + 1900;

            DateTime dateTime;
            try
            {
                dateTime = new DateTime(year, month, day, hour, minute, 0, 0);
            }
            catch (ArgumentOutOfRangeException)
            {
                dateTime = DateTime.MinValue;
            }

            return dateTime;
        }

        public static string ConvertNumberedListFormatToRtfLevelText(string text)
        {
            for (int i = 0; i < 9; ++i)
            {
                text = text.Replace("%" + (i + 1), "{" + i + "}");
            }
            return text;
        }

        internal static double RtfValueToAngle(int rtfAngle)
        {
            return rtfAngle / RtfAngleFactor;
        }

        internal static int AngleToRtfValue(double angle)
        {
            return (int)Math.Round(angle * RtfAngleFactor);
        }

        internal static double RtfFixedPointSizeToDouble(int value)
        {
            return value / RtfAngleFactor;
        }

        internal static double DoubleToRtfFixedPointSize(double value)
        {
            return Math.Round(value * RtfAngleFactor);
        }

        internal static Color IntToColor(int value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Color result = Color.FromArgb(255, bytes[0], bytes[1], bytes[2]);

            return result;
        }

        internal static int ColorToInt(Color color)
        {
            int result = 0 << 24 | color.B << 16 | color.G << 8 | color.R;
            return result;
        }

        internal static ShadingPattern RtfTagToShadingPattern(RtfTag tag)
        {
            if (tag.Name == RtfTags.TagSpanShadingPercent ||
                tag.Name == RtfTags.TagParagraphShadingPercent ||
                tag.Name == RtfTags.TagCellShadingPercent ||
                tag.Name == RtfTags.TagCellShadingPercentWithStyles ||
                tag.Name == RtfTags.TagCellShadingPercentInStyles ||
                tag.Name == RtfTags.TagRowShadingPercent)
            {
                double value = tag.ValueAsNumber / 100.0;
                int index = 0;
                while (index < ShadingPrecentToPattern.Count - 1 && ShadingPrecentToPattern[index].Item1 < value)
                {
                    index++;
                }
                return ShadingPrecentToPattern[index].Item2;
            }
            else
            {
                return ShadingPattern.Solid;
            }
        }

        internal static int ShadingPatternToRtfTag(ShadingPattern pattern)
        {
            // TODO: Shading patterns
            double result = 100;
            PatternToShadingPrecent.TryGetValue(pattern, out result);

            return (int)(result * 100);
        }

        // TODO: Remove this method when it is no longer needed 
        internal static int GetNestingLevel(DocumentElementBase element)
        {
            DocumentElementBase parent = element.Parent;
            int level = 0;
            while (parent != null)
            {
                level++;
                parent = parent.Parent;
            }
            return level;
        }

        internal static Size EnsureShapeSize(Size shapeSize)
        {
            return !shapeSize.IsEmpty ? shapeSize : ShapeBase.DefaultSize;
        }

        internal static bool IsTransparentColor(Color color)
        {
            return color.A == 0;
        }

        private static void GetGroupText(RtfGroup group, StringBuilder sb, bool recoursive)
        {
            foreach (RtfElement element in group.Elements)
            {
                RtfGroup gr = element as RtfGroup;
                if (recoursive && gr != null && !gr.IsExtensionDestination)
                {
                    GetGroupText(gr, sb, recoursive);
                }

                RtfText text = element as RtfText;
                if (text != null)
                {
                    sb.Append(text.Text);
                }
            }
        }
    }
}