﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Common;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Txt
{
    /// <summary>
    /// Format provider for importing and exporting documents to TXT format.
    /// </summary>
    public class TxtFormatProvider : TextBasedFormatProviderBase<RadFlowDocument>
    {
        private static readonly IEnumerable<string> SupportedExtensionsList = new string[] { ".txt" };

        /// <summary>
        /// Initializes a new instance of the <see cref="TxtFormatProvider" /> class.
        /// </summary>
        public TxtFormatProvider()
        {
        }

        /// <inheritdoc />
        public override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return TxtFormatProvider.SupportedExtensionsList;
            }
        }

        /// <inheritdoc />
        public override bool CanImport
        {
            get
            {
                return true;
            }
        }

        /// <inheritdoc />
        public override bool CanExport
        {
            get
            {
                return true;
            }
        }

        /// <inheritdoc />
        protected override RadFlowDocument ImportOverride(Stream input)
        {
            Guard.ThrowExceptionIfNull(input, "input");

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            using (StreamReader reader = new StreamReader(input))
            {
                editor.InsertText(reader.ReadToEnd());
            }
            return editor.Document;
        }

        /// <inheritdoc />
        protected override void ExportOverride(RadFlowDocument document, Stream output)
        {
            Guard.ThrowExceptionIfNull(document, "document");
            Guard.ThrowExceptionIfNull(output, "output");

            using (new RadFlowDocumentLicenseCheck(document))
            {
                StreamWriter writer = new StreamWriter(output);
                bool isFirst = true;
                foreach (Paragraph par in document.EnumerateChildrenOfType<Paragraph>())
                {
                    if (isFirst)
                    {
                        isFirst = false;
                    }
                    else
                    {
                        writer.WriteLine();
                    }

                    foreach (Run run in par.Inlines.Where(inline => inline is Run))
                    {
                        writer.Write(run.Text);
                    }
                }
                writer.Flush();
            }
        }
    }
}
