﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines bookmark end annotation.
    /// </summary>
    public class BookmarkRangeEnd : AnnotationRangeEndBase
    {
        private readonly Bookmark bookmark;

        internal BookmarkRangeEnd(RadFlowDocument document, Bookmark bookmark)
            : base(document)
        {
            this.bookmark = bookmark;
        }

        /// <summary>
        /// Gets the associated bookmark.
        /// </summary>
        /// <value>The bookmark.</value>
        public Bookmark Bookmark
        {
            get
            {
                return this.bookmark;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.BookmarkRangeEnd;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            var start = cloneContext.BookmarkContext.PopHangingAnnotationStart();
            if (start != null)
            {
                return start.Bookmark.BookmarkRangeEnd;
            }

            return null;
        }
    }
}
