﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines bookmark start annotation.
    /// </summary>
    public class BookmarkRangeStart : AnnotationRangeStartBase
    {
        private readonly Bookmark bookmark;

        internal BookmarkRangeStart(RadFlowDocument document, Bookmark bookmark)
            : base(document)
        {
            this.bookmark = bookmark;
        }

        /// <summary>
        /// Gets the associated bookmark.
        /// </summary>
        /// <value>The bookmark.</value>
        public Bookmark Bookmark
        {
            get
            {
                return this.bookmark;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.BookmarkRangeStart;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            BookmarkRangeStart start = new Bookmark(
                   cloneContext.Document,
                   this.Bookmark.Name,
                   this.Bookmark.FromColumn,
                   this.Bookmark.ToColumn).BookmarkRangeStart;

            cloneContext.BookmarkContext.AddHangingAnnotationStart(start);

            return start;
        }
    }
}