﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines comment range end annotation.
    /// </summary>
    public class CommentRangeEnd : AnnotationRangeEndBase
    {
        private readonly Comment comment;

        internal CommentRangeEnd(RadFlowDocument document, Comment comment)
            : base(document)
        {
            this.comment = comment;
        }

        /// <summary>
        /// Gets the associated comment.
        /// </summary>
        /// <value>The comment.</value>
        public Comment Comment
        {
            get
            {
                return this.comment;
            }
        }

        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.CommentRangeEnd;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            CommentRangeStart start = cloneContext.CommentContext.PopHangingAnnotationStart();
            if (start != null)
            {
                return start.Comment.CommentRangeEnd;
            }

            return null;
        }
    }
}