﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines comment range start annotation.
    /// </summary>
    public class CommentRangeStart : AnnotationRangeStartBase
    {
        private readonly Comment comment;

        internal CommentRangeStart(RadFlowDocument document, Comment comment)
            : base(document)
        {
            this.comment = comment;
        }

        /// <summary>
        /// Gets the associated comment.
        /// </summary>
        /// <value>The comment.</value>
        public Comment Comment
        {
            get
            {
                return this.comment;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.CommentRangeStart;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            CommentRangeStart start = ((Comment)this.Comment.CloneCore(cloneContext)).CommentRangeStart;
            cloneContext.CommentContext.AddHangingAnnotationStart(start);

            return start;
        }
    }
}