﻿using Telerik.Windows.Documents.Flow.Model.Cloning;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Represents a break.
    /// </summary>
    public class Break : InlineBase
    {
        private BreakType breakType;
        private TextWrappingRestartLocation textWrappingRestartLocation;

        /// <summary>
        /// Initializes a new instance of the <see cref="Break" /> class.
        /// </summary>
        /// <param name="document">The document.</param>
        public Break(RadFlowDocument document) : base(document)
        {
        }

        /// <summary>
        /// Gets or sets the type of the break.
        /// </summary>
        /// <value>The type of the break.</value>
        public BreakType BreakType
        {
            get
            {
                return this.breakType;
            }
            set
            {
                this.breakType = value;
            }
        }

        /// <summary>
        /// Gets or sets the text wrapping restart location.
        /// </summary>
        /// <value>The text wrapping restart location.</value>
        public TextWrappingRestartLocation TextWrappingRestartLocation
        {
            get
            {
                return this.textWrappingRestartLocation;
            }
            set
            {
                this.textWrappingRestartLocation = value;
            }
        }

        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.Break;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Break br = new Break(cloneContext.Document);
            br.BreakType = this.BreakType;
            br.TextWrappingRestartLocation = this.TextWrappingRestartLocation;

            return br;
        }
    }
}
