﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Model.Cloning
{
    internal class StyleMergeEqualityComparer : IEqualityComparer<Style>
    {
        public bool Equals(Style first, Style second)
        {
            if (ReferenceEquals(null, second))
            {
                return false;
            }

            if (ReferenceEquals(this, second))
            {
                return true;
            }

            return
                first.StyleType == second.StyleType &&
                first.Id == second.Id &&
                first.BasedOnStyleId == second.BasedOnStyleId &&
                first.LinkedStyleId == second.LinkedStyleId &&
                first.NextStyleId == second.NextStyleId &&
                first.Name == second.Name &&
                first.IsCustom == second.IsCustom &&
                first.IsDefault == second.IsDefault &&
                first.IsPrimary == second.IsPrimary &&
                first.UIPriority == second.UIPriority &&
                PropertiesAreEqual(first.CharacterProperties, second.CharacterProperties) &&
                PropertiesAreEqual(first.ParagraphProperties, second.ParagraphProperties) &&
                PropertiesAreEqual(first.TableProperties, second.TableProperties) &&
                PropertiesAreEqual(first.TableRowProperties, second.TableRowProperties) &&
                PropertiesAreEqual(first.TableCellProperties, second.TableCellProperties);
        }

        public int GetHashCode(Style obj)
        {
            throw new NotImplementedException();
        }

        internal static bool PropertiesAreEqual(DocumentElementPropertiesBase firstProperties, DocumentElementPropertiesBase secondProperties)
        {
            if ((firstProperties == null && secondProperties == null) ||
                firstProperties == secondProperties)
            {
                return true;
            }

            if ((firstProperties != null && secondProperties == null) ||
                (firstProperties == null && secondProperties != null))
            {
                return false;
            }

            if (firstProperties.StyleId != secondProperties.StyleId)
            {
                return false;
            }

            foreach (Styles.Core.IStyleProperty firstProperty in firstProperties.StyleProperties)
            {
                Styles.Core.IStyleProperty secondProperty = secondProperties.GetStyleProperty(firstProperty.PropertyDefinition);

                if (firstProperty.HasLocalValue != secondProperty.HasLocalValue)
                {
                    return false;
                }

                if (firstProperty.PropertyDefinition == Paragraph.ListIdPropertyDefinition && firstProperty.HasLocalValue && secondProperty.HasLocalValue)
                {
                    if (firstProperties.GetStyleProperty(Paragraph.ListLevelPropertyDefinition).HasLocalValue ||
                        secondProperties.GetStyleProperty(Paragraph.ListLevelPropertyDefinition).HasLocalValue)
                    {
                        continue;
                    }

                    return false;
                }

                if (firstProperty.HasLocalValue || secondProperty.HasLocalValue)
                {
                    object firstValue = firstProperty.GetActualValueAsObject();
                    object secondValue = secondProperty.GetActualValueAsObject();

                    if (!object.Equals(firstValue, secondValue))
                    {
                        return false;
                    }
                }
            }

            return true;
        }
    }
}
