﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Collections
{
    // TODO: Refactor this
    internal sealed class AnnotationCollection : ICollection<Bookmark>
    {
        private readonly RadFlowDocument document;
        private readonly List<Bookmark> innerCollection;

        public AnnotationCollection(RadFlowDocument document)
        {
            this.document = document;
            this.innerCollection = new List<Bookmark>();
        }

        public int Count
        {
            get
            {
                return this.innerCollection.Count;
            }
        }

        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        public void Add(Bookmark item)
        {
            Guard.ThrowExceptionIfFalse(item.Document == this.document, "Bookmark is from another document.");

            this.innerCollection.Add(item);
        }

        public bool Remove(Bookmark item)
        {
            Guard.ThrowExceptionIfFalse(item.Document == this.document, "Bookmark is from another document.");

            AnnotationRangeStartBase start = item.BookmarkRangeStart;
            AnnotationRangeEndBase end = item.BookmarkRangeEnd;

            if (start.Paragraph == null || end.Paragraph == null)
            {
                throw new InvalidOperationException("Bookmark is not part of the document.");
            }

            start.Paragraph.Inlines.Remove(start);
            end.Paragraph.Inlines.Remove(end);

            return this.innerCollection.Remove(item);
        }

        public void Clear()
        {
            this.innerCollection.Clear();
        }

        public bool Contains(Bookmark item)
        {
            Guard.ThrowExceptionIfFalse(item.Document == this.document, "Bookmark is from another document.");

            return this.innerCollection.Contains(item);
        }

        public void CopyTo(Bookmark[] array, int arrayIndex)
        {
            this.innerCollection.CopyTo(array, arrayIndex);
        }

        public IEnumerator<Bookmark> GetEnumerator()
        {
            return this.innerCollection.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.innerCollection.GetEnumerator();
        }
    }
}
