﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Collections
{
    /// <summary>
    /// Defines a collection of comments.
    /// </summary>
    public sealed class CommentCollection : DocumentElementCollection<Comment, RadFlowDocument>
    {
        internal CommentCollection(RadFlowDocument document)
            : base(document)
        {
        }

        /// <summary>
        /// Adds a comment.
        /// </summary>
        /// <returns>The comment.</returns>
        public Comment AddComment()
        {
            Comment comment = new Comment(this.Owner.Document);
            this.Add(comment);

            return comment;
        }

        /// <summary>
        /// Called when the comment is removed.
        /// </summary>
        /// <param name="comment">The comment.</param>
        protected override void OnAfterRemove(Comment comment)
        {
            Guard.ThrowExceptionIfNull(comment, "comment");

            CommentRangeStart start = comment.CommentRangeStart;
            if (start.Paragraph != null)
            {
                start.Paragraph.Inlines.Remove(start);
            }

            CommentRangeEnd end = comment.CommentRangeEnd;
            if (end.Paragraph != null)
            {
                end.Paragraph.Inlines.Remove(end);
            }
        }
    }
}