﻿using Telerik.Windows.Documents.Flow.Model.Shapes;

namespace Telerik.Windows.Documents.Flow.Model.Collections
{
    /// <summary>
    /// Defines a collection of inline elements.
    /// </summary>
    public sealed class InlineCollection : DocumentElementCollection<InlineBase, Paragraph>
    {
        internal InlineCollection(Paragraph parent)
            : base(parent)
        {
        }

        /// <summary>
        /// Adds a new <see cref="Run"/> to the collection.
        /// </summary>
        /// <returns>The added run.</returns>
        public Run AddRun()
        {
            Run run = new Run(this.Owner.Document);
            this.Add(run);

            return run;
        }

        /// <summary>
        /// Adds a new <see cref="Run"/> to the collection.
        /// </summary>
        /// <returns>The added run.</returns>
        public Run AddRun(string text)
        {
            Run run = new Run(this.Owner.Document);
            run.Text = text;
            this.Add(run);

            return run;
        }

        /// <summary>
        /// Adds a new <see cref="ImageInline"/> to the collection.
        /// </summary>
        /// <returns>The added image inline.</returns>
        public ImageInline AddImageInline()
        {
            ImageInline imageInline = new ImageInline(this.Owner.Document);
            this.Add(imageInline);

            return imageInline;
        }

        /// <summary>
        /// Adds a new <see cref="FloatingImage"/> to the collection.
        /// </summary>
        /// <returns>The added floating image.</returns>
        public FloatingImage AddFloatingImage()
        {
            FloatingImage floatingImage = new FloatingImage(this.Owner.Document);
            this.Add(floatingImage);

            return floatingImage;
        }
    }
}
