﻿using System;
using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Represents a comment.
    /// </summary>
    public class Comment : BlockContainerBase
    {
        private readonly CommentRangeStart commentRangeStart;
        private readonly CommentRangeEnd commentRangeEnd;

        private string author;
        private string initials;
        private DateTime date;

        internal Comment(RadFlowDocument document)
            : base(document)
        {
            this.commentRangeStart = new CommentRangeStart(document, this);
            this.commentRangeEnd = new CommentRangeEnd(document, this);
        }

        /// <summary>
        /// Gets or sets the author.
        /// </summary>
        /// <value>The author.</value>
        public string Author
        {
            get
            {
                return this.author;
            }
            set
            {
                this.author = value;
            }
        }

        /// <summary>
        /// Gets or sets the author's initials.
        /// </summary>
        /// <value>The initials.</value>
        public string Initials
        {
            get
            {
                return this.initials;
            }
            set
            {
                this.initials = value;
            }
        }

        /// <summary>
        /// Gets or sets the date and time.
        /// </summary>
        /// <value>The date and time.</value>
        public DateTime Date
        {
            get
            {
                return this.date;
            }
            set
            {
                this.date = value;
            }
        }

        /// <summary>
        /// Gets the comment range start.
        /// </summary>
        /// <value>The comment range start.</value>
        public CommentRangeStart CommentRangeStart
        {
            get
            {
                return this.commentRangeStart;
            }
        }

        /// <summary>
        /// Gets the comment range end.
        /// </summary>
        /// <value>The comment range end.</value>
        public CommentRangeEnd CommentRangeEnd
        {
            get
            {
                return this.commentRangeEnd;
            }
        }

        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.Comment;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            Comment comment = cloneContext.Document.Comments.AddComment();
            comment.Blocks.AddClonedChildrenFrom(this.Blocks, cloneContext);
            comment.Author = this.Author;
            comment.Initials = this.Initials;
            comment.Date = this.Date;

            return comment;
        }
    }
}