﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Flow.Model
{
    public abstract partial class DocumentElementBase
    {
        /// <summary>
        /// This API supports the debugging infrastructure and is not intended to be used directly from your code.
        /// </summary>
        internal string GetDebuggerDisplayText()
        {
            string childrenText = this.GetChildrenDebuggerDisplayText();
            string propertiesText = this.GetPropertiesDebuggerDisplayText();
            if (!string.IsNullOrEmpty(propertiesText))
            {
                propertiesText = " " + propertiesText;
            }

            string typeName = DebugVisualizer.EscapeXmlContent(this.Type);

            if (childrenText == null)
            {
                return string.Format("<{0}{1}/>", typeName, propertiesText);
            }

            return string.Format("<{0}{1}>{2}</{0}>", typeName, propertiesText, childrenText);
        }

        /// <summary>
        /// This API supports the internal infrastructure and is not intended to be used directly from your code.
        /// </summary>
        internal virtual string GetChildrenDebuggerDisplayText()
        {
            if (!this.Children.Any())
            {
                return null;
            }

            return string.Join(string.Empty, this.Children.Select(element => element.GetDebuggerDisplayText()));
        }

        /// <summary>
        /// This API supports the internal infrastructure and is not intended to be used directly from your code.
        /// </summary>
        internal virtual string GetPropertiesDebuggerDisplayText()
        {
            Styles.IElementWithProperties elementWithProperties = this as Styles.IElementWithProperties;

            if (elementWithProperties == null)
            {
                return null;
            }

            System.Text.StringBuilder propertiesString = new System.Text.StringBuilder();

            foreach (Styles.Core.IStyleProperty styleProperty in elementWithProperties.Properties.StyleProperties.Where(property => property.HasLocalValue))
            {
                propertiesString.AppendFormat(@"{0}=""{1}"" ", styleProperty.PropertyDefinition.PropertyName, DebugVisualizer.EscapeXmlContent(styleProperty.GetLocalValueAsObject()));
            }

            return propertiesString.ToString();
        }

        /// <summary>
        /// This API supports the debugging infrastructure and is not intended to be used directly from your code.
        /// </summary>
        protected static class DebugVisualizer
        {
            /// <summary>
            /// This API supports the debugging infrastructure and is not intended to be used directly from your code.
            /// </summary>
            public static string EscapeXmlContent(object stringContent)
            {
                if (stringContent == null)
                {
                    return null;
                }

                return stringContent
                    .ToString()
                    .Replace("&", "&amp;")
                    .Replace("<", "&lt;")
                    .Replace(">", "&gt;")
                    .Replace("\"", "&quot;")
                    .Replace("'", "&apos;")
                    .ToString();
            }
        }
    }
}
