﻿namespace Telerik.Windows.Documents.Flow.Model.Editing
{
    /// <summary>
    /// Holds different options for controlling the insertion of a source <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument"/> 
    /// to a target <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument"/>.
    /// </summary>
    public class InsertDocumentOptions
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="InsertDocumentOptions" /> class.
        /// </summary>
        public InsertDocumentOptions()
        {
            this.ConflictingStylesResolutionMode = ConflictingStylesResolutionMode.RenameSourceStyle;
            this.InsertLastParagraphMarker = true;
        }

        /// <summary>
        /// Gets or sets the mode used for resolving conflicts between styles with same IDs.
        /// <para>The default value is <see cref="Telerik.Windows.Documents.Flow.Model.ConflictingStylesResolutionMode.RenameSourceStyle"/>.</para>
        /// </summary>
        /// <value>The conflicting styles resolution mode.</value>
        public ConflictingStylesResolutionMode ConflictingStylesResolutionMode { get; set; }

        /// <summary>
        /// Gets or sets whether the formatting from the last paragraph in the source <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument"/> should be obtained.
        /// <para>If the value is <code>true</code>, then a new paragraph with same formatting will be inserted. 
        /// Otherwise, only the inlines from that paragraph will be inserted.</para>
        /// <para>The default value is <code>true</code>.</para>
        /// </summary>
        /// <value>The insert last paragraph marker.</value>
        public bool InsertLastParagraphMarker { get; set; }
    }
}