﻿using System.Collections.Generic;
using System.Diagnostics;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    internal class CloneFieldContext : FieldContext
    {
        private readonly List<FieldCharacter> unpairedFieldCharacters;
        private bool strictMode;

        public CloneFieldContext() : base(true)
        {
            this.unpairedFieldCharacters = new List<FieldCharacter>();
            this.strictMode = true;
        }

        public IEnumerable<FieldCharacter> GetUnpairedFieldCharacters()
        {
            List<FieldCharacter> allUnpairedChracters = new List<FieldCharacter>(this.unpairedFieldCharacters);
            allUnpairedChracters.AddRange(this.CurrentFieldCharacters);

            return allUnpairedChracters;
        }

        [Conditional("DEBUG")]
        public void DisableStrictMode()
        {
            this.strictMode = false;
        }

        protected override void OnInvalidSeparatorFieldCharacter(FieldCharacter fieldCharacter)
        {
            this.unpairedFieldCharacters.Add(fieldCharacter);

            if (this.strictMode)
            {
                base.OnInvalidSeparatorFieldCharacter(fieldCharacter);
            }
        }

        protected override void OnInvalidEndFieldCharacter(FieldCharacter fieldCharacter)
        {
            this.unpairedFieldCharacters.Add(fieldCharacter);

            if (this.strictMode)
            {
                base.OnInvalidEndFieldCharacter(fieldCharacter);
            }
        }
    }
}
