﻿using Telerik.Windows.Documents.Flow.Model.Fields.FieldCode;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    /// <summary>
    /// Represents a DOCVARIABLE field.
    /// </summary>
    public class DocumentVariableField : Field
    {
        private static readonly string nameNotDefinedError = "Error! Document Variable not defined.";
        private static readonly string missingDocumentVariableError = "Error! No document variable supplied.";

        internal DocumentVariableField(RadFlowDocument document)
            : base(document)
        {
        }

        /// <summary>
        /// Gets the name of the document variable.
        /// </summary>
        /// <value>The name.</value>
        public string Name { get; private set; }

        internal override void LoadParametersOverride(FieldParameters parameters)
        {
            if (parameters.FirstArgument != null)
            {
                this.Name = parameters.FirstArgument.Value;
            }
        }

        internal override FieldResult GetFieldResult(FieldUpdateContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string result;
            bool hasError;

            if (string.IsNullOrEmpty(this.Name))
            {
                result = nameNotDefinedError;
                hasError = true;
            }
            else if (context.Document.DocumentVariables.TryGetValue(this.Name, out result))
            {
                hasError = false;
            }
            else
            {
                result = missingDocumentVariableError;
                hasError = true;
            }

            return new FieldResult(result, hasError);
        }
    }
}