using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions
{
    internal class BookmarkExpression : Expression
    {
        private readonly RadFlowDocument document;
        private readonly string bookmarkName;

        public BookmarkExpression(string bookmark, RadFlowDocument document)
        {
            Guard.ThrowExceptionIfNullOrEmpty(bookmark, "bookmark");
            Guard.ThrowExceptionIfNull(document, "document");

            this.bookmarkName = bookmark;
            this.document = document;
        }

        public string BookmarkName
        {
            get
            {
                return this.bookmarkName;
            }
        }

        public RadFlowDocument Document
        {
            get
            {
                return this.document;
            }
        }

        public override ExpressionResult GetResult()
        {
            Bookmark bmk = this.document.GetBookmarkByName(this.BookmarkName);
            if (bmk == null)
            {
                return ErrorExpressions.GetExpressionResult(ExpressionErrorType.UndefinedBookmark);
            }

            string bookmarkString = GetBookmarkContent(bmk);
            Expression bookmarkExpression = ExpressionParser.Parse(bookmarkString, this.Document);
            return bookmarkExpression.GetResult();
        }

        private static string GetBookmarkContent(Bookmark bmk)
        {
            string bmkContent = InlineRangeEditor.GetTextInRange(bmk.BookmarkRangeStart, bmk.BookmarkRangeEnd, true, false);
            return bmkContent;
        }
    }
}