using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions.Functions
{
    internal static class FunctionExpressionFactory
    {
        private static readonly Dictionary<string, FunctionInfo> Functions;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1502:AvoidExcessiveComplexity")]
        static FunctionExpressionFactory()
        {
            FunctionExpressionFactory.Functions = new Dictionary<string, FunctionInfo>();
            AddFunctionInfo(new FunctionInfo("ABS", 1, (args) => new AbsFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("AND", 2, (args) => new AndFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("AVERAGE", 0, (args) => new AverageFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("COUNT", 0, (args) => new CountFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("DEFINED", 1, (args) => new DefinedFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("INT", 1, (args) => new IntFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("MAX", 0, (args) => new MaxFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("MIN", 0, (args) => new MinFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("MOD", 2, (args) => new ModFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("NOT", 1, (args) => new NotFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("OR", 2, (args) => new OrFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("PRODUCT", 0, (args) => new ProductFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("ROUND", 2, (args) => new RoundFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("SIGN", 1, (args) => new SignFunctionExpression(args)));
            AddFunctionInfo(new FunctionInfo("SUM", 0, (args) => new SumFunctionExpression(args)));
        }

        public static FunctionExpression CreateFunctionExpression(string funcName, Expression[] arguments)
        {
            funcName = funcName.Trim().ToUpperInvariant();

            FunctionInfo info = null;
            if (Functions.TryGetValue(funcName, out info))
            {
                if (info.Arguments > 0 && info.Arguments != arguments.Length)
                {
                    throw new ExpressionException("Unexpected number of function arguments.");
                }

                return info.Constructor(arguments);
            }

            throw new ExpressionException("Unrecognized function: " + funcName);
        }

        private static void AddFunctionInfo(FunctionInfo info)
        {
            FunctionExpressionFactory.Functions.Add(info.Name, info);
        }

        private class FunctionInfo
        {
            private readonly int arguments;
            private readonly string name;
            private readonly Func<Expression[], FunctionExpression> constructor;

            public FunctionInfo(string name, int arguments, Func<Expression[], FunctionExpression> constructor)
            {
                this.name = name;
                this.arguments = arguments;
                this.constructor = constructor;
            }

            public int Arguments
            {
                get
                {
                    return this.arguments;
                }
            }

            public string Name
            {
                get
                {
                    return this.name;
                }
            }

            public Func<Expression[], FunctionExpression> Constructor
            {
                get
                {
                    return this.constructor;
                }
            }
        }
    }
}