using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions.Operators
{
    internal abstract class BinaryOperatorExpression : OperatorExpression
    {
        private readonly Expression left;
        private readonly Expression right;

        /// <summary>
        /// Initializes a new instance of the <see cref="BinaryOperatorExpression" /> class.
        /// </summary>
        /// <param name="left">The left operand.</param>
        /// <param name="right">The right operand.</param>
        protected BinaryOperatorExpression(Expression left, Expression right)
        {
            Guard.ThrowExceptionIfNull(left, "left");
            Guard.ThrowExceptionIfNull(right, "right");

            this.left = left;
            this.right = right;
        }

        /// <summary>
        /// Gets the left operand.
        /// </summary>
        /// <value>The left operand.</value>
        /// <returns>The left operand as Expression.</returns>
        public Expression Left
        {
            get
            {
                return this.left;
            }
        }

        /// <summary>
        /// Gets the right operand.
        /// </summary>
        /// <value>The right operand.</value>
        /// <returns>The right operand as Expression.</returns>
        public Expression Right
        {
            get
            {
                return this.right;
            }
        }
    }
}