﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions.Operators
{
    internal static class BinaryOperatorExpressionFactory
    {
        private static readonly Dictionary<ExpressionTokenType, Func<Expression, Expression, BinaryOperatorExpression>> operators;

        static BinaryOperatorExpressionFactory()
        {
            operators = new Dictionary<ExpressionTokenType, Func<Expression, Expression, BinaryOperatorExpression>>();
            operators.Add(ExpressionTokenType.Plus, (arg1, arg2) => new AdditionExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.Minus, (arg1, arg2) => new SubtractionExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.Multiply, (arg1, arg2) => new MultiplicationExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.Divide, (arg1, arg2) => new DivisionExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.Power, (arg1, arg2) => new PowerExpression(arg1, arg2));

            operators.Add(ExpressionTokenType.Equal, (arg1, arg2) => new EqualExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.NotEqual, (arg1, arg2) => new NotEqualExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.LessThan, (arg1, arg2) => new LessThanExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.LessThanOrEqualTo, (arg1, arg2) => new LessThanOrEqualToExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.GreaterThan, (arg1, arg2) => new GreaterThanExpression(arg1, arg2));
            operators.Add(ExpressionTokenType.GreaterThanOrEqualTo, (arg1, arg2) => new GreaterThanOrEqualToExpression(arg1, arg2));
        }

        public static bool TryCreateExpression(ExpressionTokenType type, Func<Expression[]> getArgsFunc, out BinaryOperatorExpression result)
        {
            result = null;

            Func<Expression, Expression, BinaryOperatorExpression> constructor = null;
            if (operators.TryGetValue(type, out constructor))
            {
                Expression[] operands = getArgsFunc();
                result = constructor(operands[0], operands[1]);
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}