using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions.Operators
{
    /// <summary>
    /// Provides properties describing and operator.
    /// </summary>
    internal class OperatorInfo
    {
        private readonly string symbol;
        private readonly int precedence;
        private readonly OperatorAssociativity associativity;

        /// <summary>
        /// Initializes a new instance of the <see cref="OperatorInfo" /> class.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <param name="precedence">The precedence.</param>
        /// <param name="associativity">The associativity.</param>
        public OperatorInfo(string symbol, int precedence, OperatorAssociativity associativity)
        {
            Guard.ThrowExceptionIfNullOrEmpty(symbol, "symbol");

            this.symbol = symbol;
            this.associativity = associativity;
            this.precedence = precedence;
        }

        /// <summary>
        /// Gets the symbol.
        /// </summary>
        /// <value>The symbol.</value>
        /// <returns>The symbol as String.</returns>
        public string Symbol
        {
            get
            {
                return this.symbol;
            }
        }

        /// <summary>
        /// Gets the precedence.
        /// </summary>
        /// <value>The precedence.</value>
        /// <returns>The precedence as Int32.</returns>
        public int Precedence
        {
            get
            {
                return this.precedence;
            }
        }

        /// <summary>
        /// Gets the associativity.
        /// </summary>
        /// <value>The associativity.</value>
        /// <returns>The associativity as OperatorAssociativity.</returns>
        public OperatorAssociativity Associativity
        {
            get
            {
                return this.associativity;
            }
        }
    }
}