using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields.Expressions.Operators
{
    /// <summary>
    /// Provides a set of OperatorInfo instances in RadSpreadsheet.
    /// </summary>
    internal static class OperatorInfos
    {
        public static readonly OperatorInfo Range;
        public static readonly OperatorInfo Intersection;
        public static readonly OperatorInfo UnaryPlus;
        public static readonly OperatorInfo UnaryMinus;
        public static readonly OperatorInfo Percent;
        public static readonly OperatorInfo Power;
        public static readonly OperatorInfo Multiply;
        public static readonly OperatorInfo Divide;
        public static readonly OperatorInfo Plus;
        public static readonly OperatorInfo Minus;
        public static readonly OperatorInfo Ampersand;
        public static readonly OperatorInfo Equal;
        public static readonly OperatorInfo LessThan;
        public static readonly OperatorInfo GreaterThan;
        public static readonly OperatorInfo LessThanOrEqualTo;
        public static readonly OperatorInfo GreaterThanOrEqualTo;
        public static readonly OperatorInfo NotEqual;

        private static readonly Dictionary<ExpressionTokenType, OperatorInfo> TokenTypeToOperatorInfo;

        static OperatorInfos()
        {
            OperatorInfos.TokenTypeToOperatorInfo = new Dictionary<ExpressionTokenType, OperatorInfo>();

            UnaryMinus = RegisterOperatorInfo(ExpressionTokenType.UnaryMinus, new OperatorInfo(ExpressionConstants.UnaryMinus, 5, OperatorAssociativity.Right));
            Percent = RegisterOperatorInfo(ExpressionTokenType.Percent, new OperatorInfo(ExpressionConstants.Percent, 7, OperatorAssociativity.Left));
            Power = RegisterOperatorInfo(ExpressionTokenType.Power, new OperatorInfo(ExpressionConstants.Power, 10, OperatorAssociativity.Left));
            Multiply = RegisterOperatorInfo(ExpressionTokenType.Multiply, new OperatorInfo(ExpressionConstants.Multiplication, 15, OperatorAssociativity.Left));
            Divide = RegisterOperatorInfo(ExpressionTokenType.Divide, new OperatorInfo(ExpressionConstants.Division, 15, OperatorAssociativity.Left));
            Plus = RegisterOperatorInfo(ExpressionTokenType.Plus, new OperatorInfo(ExpressionConstants.Plus, 20, OperatorAssociativity.Left));
            Minus = RegisterOperatorInfo(ExpressionTokenType.Minus, new OperatorInfo(ExpressionConstants.Minus, 20, OperatorAssociativity.Left));
            Equal = RegisterOperatorInfo(ExpressionTokenType.Equal, new OperatorInfo(ExpressionConstants.Equal, 25, OperatorAssociativity.Left));
            LessThan = RegisterOperatorInfo(ExpressionTokenType.LessThan, new OperatorInfo(ExpressionConstants.LessThan, 25, OperatorAssociativity.Left));
            GreaterThan = RegisterOperatorInfo(ExpressionTokenType.GreaterThan, new OperatorInfo(ExpressionConstants.GreaterThan, 25, OperatorAssociativity.Left));
            LessThanOrEqualTo = RegisterOperatorInfo(ExpressionTokenType.LessThanOrEqualTo, new OperatorInfo(ExpressionConstants.LessThanOrEqualTo, 25, OperatorAssociativity.Left));
            GreaterThanOrEqualTo = RegisterOperatorInfo(ExpressionTokenType.GreaterThanOrEqualTo, new OperatorInfo(ExpressionConstants.GreaterThanOrEqualTo, 25, OperatorAssociativity.Left));
            NotEqual = RegisterOperatorInfo(ExpressionTokenType.NotEqual, new OperatorInfo(ExpressionConstants.NotEqual, 25, OperatorAssociativity.Left));
        }

        internal static OperatorInfo GetOperatorInfo(ExpressionTokenType tokenType)
        {
            OperatorInfo operatorInfo;
            TokenTypeToOperatorInfo.TryGetValue(tokenType, out operatorInfo);

            return operatorInfo;
        }

        internal static bool IsOperator(ExpressionTokenType tokenType)
        {
            return GetOperatorInfo(tokenType) != null;
        }

        internal static OperatorAssociativity GetAssociativity(ExpressionTokenType tokenType)
        {
            return GetOperatorInfo(tokenType).Associativity;
        }

        internal static int ComparePrecedence(ExpressionTokenType tokenType1, ExpressionTokenType tokenType2)
        {
            return ComparePrecedence(GetOperatorInfo(tokenType1).Precedence, GetOperatorInfo(tokenType2).Precedence);
        }

        internal static int ComparePrecedence(int precedence1, int precedence2)
        {
            return precedence1 - precedence2;
        }

        private static OperatorInfo RegisterOperatorInfo(ExpressionTokenType tokenType, OperatorInfo operatorInfo)
        {
            Guard.ThrowExceptionIfNull(operatorInfo, "operatorInfo");

            TokenTypeToOperatorInfo.Add(tokenType, operatorInfo);
            return operatorInfo;
        }
    }
}