﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    /// <summary>
    /// Field character is a special character which delimits the start and end of a field or 
    /// separates its field codes from its current field result.
    /// </summary>
    public sealed class FieldCharacter : AnnotationMarkerBase
    {
        private readonly FieldCharacterType fieldCharacterType;

        internal FieldCharacter(RadFlowDocument document, FieldCharacterType type)
            : base(document)
        {
            this.fieldCharacterType = type;
        }

        /// <summary>
        /// Gets the type of the field character.
        /// </summary>
        /// <value>The type of the field character.</value>
        public FieldCharacterType FieldCharacterType
        {
            get
            {
                return this.fieldCharacterType;
            }
        }

        /// <summary>
        /// Gets the field info.
        /// </summary>
        /// <value>The field info.</value>
        public FieldInfo FieldInfo { get; internal set; }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.FieldCharacter;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            FieldCharacter fieldCharacter = new FieldCharacter(cloneContext.Document, this.FieldCharacterType);

            return fieldCharacter;
        }

        internal override void OnAfterCloneCore(CloneContext cloneContext, DocumentElementBase clonedElement)
        {
            cloneContext.FieldContext.OnFieldCharacter((FieldCharacter)clonedElement);

            if (this.fieldCharacterType == Fields.FieldCharacterType.Start && this.FieldInfo != null)
            {
                cloneContext.FieldContext.SetIsDirty(this.FieldInfo.IsDirty);
                cloneContext.FieldContext.SetIsLocked(this.FieldInfo.IsLocked);
            }
        }

#if !NETSTANDARD
        internal override string GetPropertiesDebuggerDisplayText()
        {
            return string.Format(@"{0} FieldCharacterType=""{1}""", base.GetPropertiesDebuggerDisplayText(), this.FieldCharacterType);
        }
#endif
    }
}