﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields.FieldCode
{
    internal static class FieldFactory
    {
        private static readonly Dictionary<string, Func<RadFlowDocument, Field>> RegisterdFields;

        static FieldFactory()
        {
            RegisterdFields = new Dictionary<string, Func<RadFlowDocument, Field>>();
            RegisterFieldType("HYPERLINK", (doc) => new Hyperlink(doc));
            RegisterFieldType("DATE", (doc) => new DateField(doc));
            RegisterFieldType("TIME", (doc) => new TimeField(doc));
            RegisterFieldType("IF", (doc) => new IfField(doc));
            RegisterFieldType("COMPARE", (doc) => new CompareField(doc));
            RegisterFieldType("DOCVARIABLE", (doc) => new DocumentVariableField(doc));
            RegisterFieldType(MergeField.Type, (doc) => new MergeField(doc));
        }

        public static void RegisterFieldType(string fieldType, Func<RadFlowDocument, Field> createFieldFunc)
        {
            RegisterdFields[fieldType.ToUpperInvariant()] = createFieldFunc;
        }

        internal static Field CreateField(string fieldType, RadFlowDocument document)
        {
            Guard.ThrowExceptionIfNull(document, "document");
            Guard.ThrowExceptionIfNullOrEmpty(fieldType, "fieldType");

            Field result = null;
            Func<RadFlowDocument, Field> creator = null;
            if (RegisterdFields.TryGetValue(fieldType.ToUpperInvariant(), out creator))
            {
                result = creator(document);
            }
            else
            {
                result = new CustomCodeField(document);
            }

            return result;
        }
    }
}
