﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    /// <summary>
    /// Represents the field's result fragment.
    /// </summary>
    internal class FieldResult
    {
        private readonly string result;
        private readonly bool isError;

        /// <summary>
        /// Initializes a new instance of the <see cref="FieldResult" /> class.
        /// </summary>
        /// <param name="result">The result.</param>
        public FieldResult(string result)
            : this(result, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FieldResult" /> class.
        /// </summary>
        /// <param name="result">The result.</param>
        /// <param name="isError">Is the result an error.</param>
        public FieldResult(string result, bool isError)
        {
            Guard.ThrowExceptionIfNull(result, "result");

            this.isError = isError;
            this.result = result;
        }

        /// <summary>
        /// Gets the result.
        /// </summary>
        /// <value>The result.</value>
        public string Result
        {
            get
            {
                return this.result;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the result is an error.
        /// </summary>
        /// <value>True if the result is an error, otherwise false.</value>
        public bool IsError
        {
            get
            {
                return this.isError;
            }
        }
    }
}