﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    internal static class FieldUpdateScheduler
    {
        internal static List<FieldCharacter> GetOrderedListOfFieldToUpdate(IEnumerable<FieldCharacter> fields)
        {
            List<FieldCharacter> fieldUpdateStack = new List<FieldCharacter>();
            Stack<FieldCharacter> currentFieldsStack = new Stack<FieldCharacter>();

            foreach (FieldCharacter fieldChar in fields)
            {
                switch (fieldChar.FieldCharacterType)
                {
                    case FieldCharacterType.Start:
                        if (ShouldAddToUpdateStack(currentFieldsStack))
                        {
                            fieldUpdateStack.Add(fieldChar);
                        }

                        currentFieldsStack.Push(fieldChar);
                        break;

                    case FieldCharacterType.Separator:
                        if (currentFieldsStack.Count == 0 || currentFieldsStack.Peek().FieldCharacterType == FieldCharacterType.Separator)
                        {
                            Debug.Fail("No currently opened fields.");
                        }

                        currentFieldsStack.Push(fieldChar);
                        break;

                    case FieldCharacterType.End:
                        if (currentFieldsStack.Count == 0)
                        {
                            Debug.Fail("No currently opened fields.");
                        }

                        OnFieldEnd(currentFieldsStack);
                        break;
                }
            }

            fieldUpdateStack.Reverse();
            return fieldUpdateStack;
        }

        private static void OnFieldEnd(Stack<FieldCharacter> stcurrentFieldsStackack)
        {
            FieldCharacter start;
            FieldCharacter separate = stcurrentFieldsStackack.Pop();

            if (separate.FieldCharacterType == FieldCharacterType.Separator)
            {
                start = stcurrentFieldsStackack.Pop();
            }
            else
            {
                start = separate;
                separate = null;
            }

            Debug.Assert(start.FieldCharacterType == FieldCharacterType.Start, "Could not find start character");
        }

        private static bool ShouldAddToUpdateStack(Stack<FieldCharacter> currentFieldsStack)
        {
            // Do not schedule update when inside result fragment.
            return !currentFieldsStack.Any(fc => fc.FieldCharacterType == FieldCharacterType.Separator);
        }
    }
}
