using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Fields
{
    internal class TokenStringReader
    {
        private readonly string input;
        private int position;

        public TokenStringReader(FieldCharacter start, FieldCharacter end)
        {
            Guard.ThrowExceptionIfNull(start, "start");
            Guard.ThrowExceptionIfNull(end, "end");

            this.input = InlineRangeEditor.GetTextInRange(start, end, true, false);
        }

        public TokenStringReader(string input)
        {
            Guard.ThrowExceptionIfNull(input, "input");
            this.input = input;
        }

        public bool EndOfFile
        {
            get
            {
                return this.position >= this.input.Length;
            }
        }

        public int Peek()
        {
            if (this.EndOfFile)
            {
                return -1;
            }

            return this.input[this.position];
        }

        public int Read()
        {
            if (this.EndOfFile)
            {
                return -1;
            }

            return this.input[this.position++];
        }

        public void MoveToEnd()
        {
            this.position = this.input.Length;
        }
    }
}