﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Flow.Model.Watermarks;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Represents header for pages in a section.
    /// </summary>
    public sealed class Header : HeaderFooterBase
    {
        private readonly WatermarkCollection watermarkCollection;

        /// <summary>
        /// Initializes a new instance of the <see cref="Header" /> class.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <param name="section">The section.</param>
        internal Header(RadFlowDocument document, Section section)
            : base(document, section)
        {
            this.watermarkCollection = new WatermarkCollection(document);
        }

        /// <summary>
        /// Gets the watermarks for this header.
        /// </summary>
        /// <value>The watermarks.</value>
        public WatermarkCollection Watermarks
        {
            get
            {
                return this.watermarkCollection;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.Header;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            Header header = new Header(cloneContext.Document, cloneContext.CurrentSection);
            header.Blocks.AddClonedChildrenFrom(this.Blocks, cloneContext);
            header.Watermarks.AddClonedChildrenFrom(this.Watermarks);

            return header;
        }
    }
}
