﻿using System.Text;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Model.Lists
{
    internal static class ListFactory
    {
        private static readonly string[] bullets = new string[] { "", "o", "" };
        private static readonly FontFamily[] bulletsFontFamily = new FontFamily[] { new FontFamily("Symbol"), new FontFamily("Courier New"), new FontFamily("Wingdings") };

        private static readonly NumberingStyle[] numberedListLevels = new NumberingStyle[]
        {
            NumberingStyle.Decimal,
            NumberingStyle.LowerLetter,
            NumberingStyle.LowerRoman
        };

        internal static string[] BulletSymbols
        {
            get
            {
                return bullets;
            }
        }

        internal static FontFamily[] BulletsFontFamily
        {
            get
            {
                return bulletsFontFamily;
            }
        }

        internal static List GetList(ListTemplateType listTemplate)
        {
            switch (listTemplate)
            {
                case ListTemplateType.BulletDefault:
                    return ListFactory.CreateBulletedDefaultList();
                case ListTemplateType.NumberedDefault:
                    return ListFactory.CreateNumberedDefault();
                case ListTemplateType.NumberedParentheses:
                    return ListFactory.CreateNumberedParenthesesList();
                case ListTemplateType.NumberedHierarchical:
                    return ListFactory.CreateNumberedHierarchicalList();
            }

            return null;
        }

        internal static List CreateBulletedDefaultList()
        {
            List list = new List();

            for (int i = 0; i < list.Levels.Count; ++i)
            {
                list.Levels[i].StartIndex = 1;
                list.Levels[i].NumberingStyle = NumberingStyle.Bullet;
                list.Levels[i].NumberTextFormat = bullets[i % bullets.Length];
                list.Levels[i].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(bulletsFontFamily[i % bulletsFontFamily.Length]);
                ListFactory.SetIndentation(list.Levels[i].ParagraphProperties, i);
            }

            return list;
        }

        internal static List CreateNumberedDefault()
        {
            List list = new List();

            for (int i = 0; i < list.Levels.Count; ++i)
            {
                list.Levels[i].StartIndex = 1;
                list.Levels[i].NumberingStyle = numberedListLevels[i % numberedListLevels.Length];
                list.Levels[i].NumberTextFormat = "%" + (i + 1) + ".";
                ListFactory.SetIndentation(list.Levels[i].ParagraphProperties, i);
            }

            return list;
        }

        internal static List CreateNumberedParenthesesList()
        {
            List list = new List();

            for (int i = 0; i < list.Levels.Count; ++i)
            {
                list.Levels[i].StartIndex = 1;
                list.Levels[i].NumberingStyle = numberedListLevels[i % numberedListLevels.Length];
                list.Levels[i].NumberTextFormat += "%" + (i + 1) + ")";
                ListFactory.SetIndentation(list.Levels[i].ParagraphProperties, i);
            }

            return list;
        }

        internal static List CreateNumberedHierarchicalList()
        {
            List list = new List();

            for (int i = 0; i < list.Levels.Count; ++i)
            {
                StringBuilder levelText = new StringBuilder();
                for (int j = 0; j < i + 1; ++j)
                {
                    levelText.Append("%" + (j + 1) + ".");
                }

                list.Levels[i].StartIndex = 1;
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
                list.Levels[i].NumberTextFormat = levelText.ToString();
                ListFactory.SetIndentation(list.Levels[i].ParagraphProperties, i);
            }

            return list;
        }

        private static void SetIndentation(ParagraphProperties paragraphProperties, int listLevelIndex)
        {
            paragraphProperties.LeftIndent.LocalValue = 48 + (listLevelIndex * 24);
            paragraphProperties.HangingIndent.LocalValue = 24;
        }
    }
}
