﻿namespace Telerik.Windows.Documents.Flow.Model.Lists
{
    /// <summary>
    /// Describes the numbering style of a list level.
    /// </summary>
    public enum NumberingStyle
    {
        /// <summary>
        /// Decimal numbering style.
        /// </summary>
        Decimal = 0,

        /// <summary>
        /// UpperRoman numbering style.
        /// </summary>
        UpperRoman = 1,

        /// <summary>
        /// LowerRoman numbering style.
        /// </summary>
        LowerRoman = 2,

        /// <summary>
        /// UpperLetter numbering style.
        /// </summary>
        UpperLetter = 3,

        /// <summary>
        /// LowerLetter numbering style.
        /// </summary>
        LowerLetter = 4,

        /// <summary>
        /// Ordinal numbering style.
        /// </summary>
        Ordinal = 5,

        /// <summary>
        /// CardinalText numbering style.
        /// </summary>
        CardinalText = 6,

        /// <summary>
        /// OrdinalText numbering style.
        /// </summary>
        OrdinalText = 7,

        /// <summary>
        /// Hex numbering style.
        /// </summary>
        Hex = 8,

        /// <summary>
        /// Chicago numbering style.
        /// </summary>
        Chicago = 9,

        /// <summary>
        /// IdeographDigital numbering style.
        /// </summary>
        IdeographDigital = 10,

        /// <summary>
        /// JapaneseCounting numbering style.
        /// </summary>
        JapaneseCounting = 11,

        /// <summary>
        /// Aiueo numbering style.
        /// </summary>
        Aiueo = 12,

        /// <summary>
        /// Iroha numbering style.
        /// </summary>
        Iroha = 13,

        /// <summary>
        /// DecimalFullWidth numbering style.
        /// </summary>
        DecimalFullWidth = 14,

        /// <summary>
        /// DecimalHalfWidth numbering style.
        /// </summary>
        DecimalHalfWidth = 15,

        /// <summary>
        /// JapaneseLegal numbering style.
        /// </summary>
        JapaneseLegal = 16,

        /// <summary>
        /// JapaneseDigitalTenThousand numbering style.
        /// </summary>
        JapaneseDigitalTenThousand = 17,

        /// <summary>
        /// DecimalEnclosedCircle numbering style.
        /// </summary>
        DecimalEnclosedCircle = 18,

        /// <summary>
        /// DecimalFullWidth2 numbering style.
        /// </summary>
        DecimalFullWidth2 = 19,

        /// <summary>
        /// AiueoFullWidth numbering style.
        /// </summary>
        AiueoFullWidth = 20,

        /// <summary>
        /// IrohaFullWidth numbering style.
        /// </summary>
        IrohaFullWidth = 21,

        /// <summary>
        /// DecimalZero numbering style.
        /// </summary>
        DecimalZero = 22,

        /// <summary>
        /// Bullet numbering style.
        /// </summary>
        Bullet = 23,

        /// <summary>
        /// Ganada numbering style.
        /// </summary>
        Ganada = 24,

        /// <summary>
        /// Chosung numbering style.
        /// </summary>
        Chosung = 25,

        /// <summary>
        /// DecimalEnclosedFullStop numbering style.
        /// </summary>
        DecimalEnclosedFullStop = 26,

        /// <summary>
        /// DecimalEnclosedParent numbering style.
        /// </summary>
        DecimalEnclosedParent = 27,

        /// <summary>
        /// DecimalEnclosedCircleChinese numbering style.
        /// </summary>
        DecimalEnclosedCircleChinese = 28,

        /// <summary>
        /// IdeographEnclosedCircle numbering style.
        /// </summary>
        IdeographEnclosedCircle = 29,

        /// <summary>
        /// IdeographTraditional numbering style.
        /// </summary>
        IdeographTraditional = 30,

        /// <summary>
        /// IdeographZodiac numbering style.
        /// </summary>
        IdeographZodiac = 31,

        /// <summary>
        /// IdeographZodiacTraditional numbering style.
        /// </summary>
        IdeographZodiacTraditional = 32,

        /// <summary>
        /// TaiwaneseCounting numbering style.
        /// </summary>
        TaiwaneseCounting = 33,

        /// <summary>
        /// IdeographLegalTraditional numbering style.
        /// </summary>
        IdeographLegalTraditional = 34,

        /// <summary>
        /// TaiwaneseCountingThousand numbering style.
        /// </summary>
        TaiwaneseCountingThousand = 35,

        /// <summary>
        /// TaiwaneseDigital numbering style.
        /// </summary>
        TaiwaneseDigital = 36,

        /// <summary>
        /// ChineseCounting numbering style.
        /// </summary>
        ChineseCounting = 37,

        /// <summary>
        /// ChineseLegalSimplified numbering style.
        /// </summary>
        ChineseLegalSimplified = 38,

        /// <summary>
        /// ChineseCountingThousand numbering style.
        /// </summary>
        ChineseCountingThousand = 39,

        /// <summary>
        /// KoreanDigital numbering style.
        /// </summary>
        KoreanDigital = 41,

        /// <summary>
        /// KoreanCounting numbering style.
        /// </summary>
        KoreanCounting = 42,

        /// <summary>
        /// KoreanLegal numbering style.
        /// </summary>
        KoreanLegal = 43,

        /// <summary>
        /// KoreanDigital2 numbering style.
        /// </summary>
        KoreanDigital2 = 44,

        /// <summary>
        /// VietnameseCounting numbering style.
        /// </summary>
        VietnameseCounting = 56,

        /// <summary>
        /// RussianLower numbering style.
        /// </summary>
        RussianLower = 58,

        /// <summary>
        /// RussianUpper numbering style.
        /// </summary>
        RussianUpper = 59,

        /// <summary>
        /// None numbering style.
        /// </summary>
        None = 255,

        /// <summary>
        /// NumberInDash numbering style.
        /// </summary>
        NumberInDash = 57,

        /// <summary>
        /// Hebrew1 numbering style.
        /// </summary>
        Hebrew1 = 45,

        /// <summary>
        /// Hebrew2 numbering style.
        /// </summary>
        Hebrew2 = 47,

        /// <summary>
        /// ArabicAlpha numbering style.
        /// </summary>
        ArabicAlpha = 46,

        /// <summary>
        /// ArabicAbjad numbering style.
        /// </summary>
        ArabicAbjad = 48,

        /// <summary>
        /// HindiVowels numbering style.
        /// </summary>
        HindiVowels = 49,

        /// <summary>
        /// HindiConsonants numbering style.
        /// </summary>
        HindiConsonants = 50,

        /// <summary>
        /// HindiNumbers numbering style.
        /// </summary>
        HindiNumbers = 51,

        /// <summary>
        /// HindiCounting numbering style.
        /// </summary>
        HindiCounting = 52,

        /// <summary>
        /// ThaiLetters numbering style.
        /// </summary>
        ThaiLetters = 53,

        /// <summary>
        /// ThaiNumbers numbering style.
        /// </summary>
        ThaiNumbers = 54,

        /// <summary>
        /// ThaiCounting numbering style.
        /// </summary>
        ThaiCounting = 55,

        /// <summary>
        /// BahtText numbering style.
        /// </summary>
        BahtText = 0,

        /// <summary>
        /// DollarText numbering style.
        /// </summary>
        DollarText = 0,

        /// <summary>
        /// Custom numbering style.
        /// </summary>
        Custom = 0
    }
}
