﻿using System;
using System.Collections;
using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.Flow.Model.Fields.MailMerge;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model
{
    internal static class MailMergeProcessor
    {
        public static RadFlowDocument Execute(RadFlowDocument document, IEnumerable collection)
        {
            Guard.ThrowExceptionIfNull(collection, "collection");
            Guard.ThrowExceptionIfNull(document, "document");

            RadFlowDocument result = null;

            IEnumerator enumerator = collection.GetEnumerator();
            while (enumerator.MoveNext())
            {
                RadFlowDocument template = document.Clone();
                ExecuteMailMerge(template, enumerator.Current);

                if (result == null)
                {
                    result = template;
                }
                else
                {
                    result.Merge(template);
                }
            }

            if (result == null)
            {
                throw new ArgumentException("Collection could not be empty.", "collection");
            }

            return result;
        }

        private static void ExecuteMailMerge(RadFlowDocument document, object record)
        {
            IEnumerable<FieldCharacter> allFields = document.EnumerateChildrenOfType<FieldCharacter>();
            List<FieldCharacter> fieldsToUpdate = FieldUpdateScheduler.GetOrderedListOfFieldToUpdate(allFields);

            foreach (FieldCharacter fieldCharacter in fieldsToUpdate)
            {
                FieldInfo fieldInfo = fieldCharacter.FieldInfo;

                string code = fieldInfo.GetCode();
                if (!code.Contains(MergeField.Type))
                {
                    fieldInfo.UpdateField();

                    continue;
                }

                ParagraphProperties startParagraphProperties = fieldInfo.Start.Paragraph.Properties;
                ParagraphProperties endParagraphProperties = fieldInfo.End.Paragraph.Properties;

                MergeFieldUpdateContext mergeFieldUpdateContex = new MergeFieldUpdateContext(fieldInfo, record);
                fieldInfo.UpdateFieldInternal(mergeFieldUpdateContex);

                string result = fieldInfo.GetResult();

                CharacterProperties resultCharacterFormatting = InlineRangeEditor.GetCharacterFormattingFromFirstRunInRange(fieldInfo.Start, fieldInfo.CodeEnd);

                InlineRangeEditor.DeleteContent(fieldInfo.Start, fieldInfo.End);

                Paragraph fieldStartParagraph = fieldInfo.Start.Paragraph;
                fieldStartParagraph.Properties.CopyPropertiesFrom(startParagraphProperties);

                int index = fieldInfo.Start.Paragraph.Inlines.IndexOf(fieldInfo.Start);
                fieldInfo.Start.Paragraph.Inlines.Remove(fieldInfo.Start);
                fieldInfo.End.Paragraph.Inlines.Remove(fieldInfo.End);

                RadFlowDocumentEditor editor = new RadFlowDocumentEditor(fieldStartParagraph.Document);
                if (index + 1 <= fieldStartParagraph.Inlines.Count)
                {
                    editor.MoveToInlineStart(fieldStartParagraph.Inlines[index]);
                }
                else if (index > 0)
                {
                    editor.MoveToInlineEnd(fieldStartParagraph.Inlines[index - 1]);
                }
                else
                {
                    editor.MoveToParagraphStart(fieldStartParagraph);
                }

                editor.CharacterFormatting.CopyPropertiesFrom(resultCharacterFormatting);
                editor.ParagraphFormatting.CopyPropertiesFrom(endParagraphProperties);
                editor.InsertText(result);
            }
        }
    }
}
