﻿using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines a set of style property definitions which are for initialization for all style properties in <see cref="ParagraphProperties"/> element.
    /// </summary>
    public partial class Paragraph
    {
        /// <summary>
        /// Gets the text alignment property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<Alignment?> TextAlignmentPropertyDefinition =
            new StylePropertyDefinition<Alignment?>("TextAlignment", DocumentDefaultStyleSettings.TextAlignment, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the spacing after property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> SpacingAfterPropertyDefinition =
            new StylePropertyDefinition<double?>("SpacingAfter", DocumentDefaultStyleSettings.SpacingAfter, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the spacing before property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> SpacingBeforePropertyDefinition =
            new StylePropertyDefinition<double?>("SpacingBefore", DocumentDefaultStyleSettings.SpacingBefore, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the allow overflow punctuation property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> AllowOverflowPunctuationPropertyDefinition =
            new StylePropertyDefinition<bool?>("AllowOverflowPunctuation", DocumentDefaultStyleSettings.AllowOverflowPunctuation, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the paragraph borders property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ParagraphBorders> BordersPropertyDefinition =
            new StylePropertyDefinition<ParagraphBorders>("Borders", DocumentDefaultStyleSettings.ParagraphBorders, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the flow direction property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<FlowDirection?> FlowDirectionPropertyDefinition =
            new StylePropertyDefinition<FlowDirection?>("FlowDirection", DocumentDefaultStyleSettings.FlowDirection, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the line spacing property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> LineSpacingPropertyDefinition =
            new StylePropertyDefinition<double?>("LineSpacing", DocumentDefaultStyleSettings.LineSpacing, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the line spacing type property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<HeightType?> LineSpacingTypePropertyDefinition =
            new StylePropertyDefinition<HeightType?>("LineSpacingType", DocumentDefaultStyleSettings.LineSpacingType, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the keep on one page property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> KeepOnOnePagePropertyDefinition =
            new StylePropertyDefinition<bool?>("KeepOnOnePage", DocumentDefaultStyleSettings.KeepOnOnePage, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the keep with next paragraph property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> KeepWithNextParagraphPropertyDefinition =
            new StylePropertyDefinition<bool?>("KeepWithNextParagraph", DocumentDefaultStyleSettings.KeepWithNextParagraph, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the list id property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<int?> ListIdPropertyDefinition =
            new StylePropertyDefinition<int?>("ListId", DocumentDefaultStyleSettings.ListId, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the list level property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<int?> ListLevelPropertyDefinition =
            new StylePropertyDefinition<int?>("ListLevel", DocumentDefaultStyleSettings.ListLevel, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the outline level property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<OutlineLevel?> OutlineLevelPropertyDefinition =
            new StylePropertyDefinition<OutlineLevel?>("OutlineLevel", DocumentDefaultStyleSettings.OutlineLevel, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the apply east asian line breaking rules property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> ApplyEastAsianLineBreakingRulesPropertyDefinition =
            new StylePropertyDefinition<bool?>("ApplyEastAsianLineBreakingRules", DocumentDefaultStyleSettings.ApplyEastAsianLineBreakingRules, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the page break before property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> PageBreakBeforePropertyDefinition =
            new StylePropertyDefinition<bool?>("PageBreakBefore", DocumentDefaultStyleSettings.PageBreakBefore, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the contextual spacing property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> ContextualSpacingPropertyDefinition =
            new StylePropertyDefinition<bool?>("ContextualSpacing", DocumentDefaultStyleSettings.ContextualSpacing, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the mirror indents property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> MirrorIndentsPropertyDefinition =
            new StylePropertyDefinition<bool?>("MirrorIndents", DocumentDefaultStyleSettings.MirrorIndents, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the automatic spacing before property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> AutomaticSpacingBeforePropertyDefinition =
            new StylePropertyDefinition<bool?>("AutomaticSpacingBefore", DocumentDefaultStyleSettings.AutomaticSpacingBefore, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the automatic spacing after property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> AutomaticSpacingAfterPropertyDefinition =
            new StylePropertyDefinition<bool?>("AutomaticSpacingAfter", DocumentDefaultStyleSettings.AutomaticSpacingAfter, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the first line indent property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> FirstLineIndentPropertyDefinition =
            new StylePropertyDefinition<double?>("FirstLineIndent", DocumentDefaultStyleSettings.FirstLineIndent, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the hanging indent property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> HangingIndentPropertyDefinition =
            new StylePropertyDefinition<double?>("HangingIndent", DocumentDefaultStyleSettings.HangingIndent, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the left indent property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> LeftIndentPropertyDefinition =
            new StylePropertyDefinition<double?>("LeftIndent", DocumentDefaultStyleSettings.LeftIndent, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the right indent property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> RightIndentPropertyDefinition =
            new StylePropertyDefinition<double?>("RightIndent", DocumentDefaultStyleSettings.RightIndent, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the tab stops property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<TabStopCollection> TabStopsPropertyDefinition =
            new StylePropertyDefinition<TabStopCollection>("TabStops", DocumentDefaultStyleSettings.TabStops, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the background color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> BackgroundColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("BackgroundColor", DocumentDefaultStyleSettings.ParagraphBackgroundColor, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the shading pattern color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> ShadingPatternColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("ShadingPatternColor", DocumentDefaultStyleSettings.ParagraphShadingPatternColor, StylePropertyType.Paragraph);

        /// <summary>
        /// Gets the shading pattern property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ShadingPattern?> ShadingPatternPropertyDefinition =
            new StylePropertyDefinition<ShadingPattern?>("ShadingPattern", DocumentDefaultStyleSettings.ParagraphShadingPattern, StylePropertyType.Paragraph);

        static Paragraph()
        {
            FirstLineIndentPropertyDefinition.Validation.AddRule(new ValidationRule<double>(value => value >= 0));
            HangingIndentPropertyDefinition.Validation.AddRule(new ValidationRule<double>(value => value >= 0));
        }
    }
}