﻿namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    /// <summary>
    /// Describes possible types of permission for user groups.
    /// </summary>
    public enum EditingGroup
    {
        /// <summary> 
        /// Specifies that none of the users that open the document shall be allowed to edit range permissions using this editing group when document protection is being enforced.
        /// </summary> 
        None,

        /// <summary> 
        /// Specifies that all users that open the document shall be allowed to edit range permissions using this editing group when document protection is being enforced.
        /// </summary> 
        Everyone,

        /// <summary> 
        /// Specifies that only user(s) who the viewing application associates with the Administrators group shall be allowed to edit the contents 
        /// between the start and end permission ranges when document protection is being enforced.
        /// </summary> 
        Administrators,

        /// <summary> 
        /// Specifies that only user(s) who the viewing application associates with the Contributors group shall be allowed to edit the contents 
        /// between the start and end permission ranges when document protection is being enforced.
        /// </summary> 
        Contributors,

        /// <summary> 
        /// Specifies that only user(s) who the viewing application associates with the Editors group shall be allowed to edit the contents 
        /// between the start and end permission ranges when document protection is being enforced.
        /// </summary> 
        Editors,

        /// <summary> 
        /// Specifies that only user(s) who the viewing application associates with the Owners group shall be allowed to edit the contents 
        /// between the start and end permission ranges when document protection is being enforced.
        /// </summary> 
        Owners,

        /// <summary> 
        /// Specifies that only user(s) who the viewing application associates with the Current group shall be allowed to edit the contents 
        /// between the start and end permission ranges when document protection is being enforced.
        /// </summary> 
        Current
    }
}
