﻿using System;
using Telerik.Windows.Documents.Common.Model.Protection;

namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    internal class FlowProtectionHelper : ProtectionHelperBase
    {
        private static readonly int[] initialCodeArray = new int[] { 0, 0xE1F0, 0x1D0F, 0xCC9C, 0x84C0, 0x110C, 0x0E10, 0xF1CE, 0x313E, 0x1872, 0xE139, 0xD40F, 0x84F9, 0x280C, 0xA96A, 0x4EC3 };
        private static readonly int[][] cryptoMatrix = new int[][]
        {
           new int[7] { 0xAEFC, 0x4DD9, 0x9BB2, 0x2745, 0x4E8A, 0x9D14, 0x2A09 },
           new int[7] { 0x7B61, 0xF6C2, 0xFDA5, 0xEB6B, 0xC6F7, 0x9DCF, 0x2BBF },
           new int[7] { 0x4563, 0x8AC6, 0x05AD, 0x0B5A, 0x16B4, 0x2D68, 0x5AD0 },
           new int[7] { 0x0375, 0x06EA, 0x0DD4, 0x1BA8, 0x3750, 0x6EA0, 0xDD40 },
           new int[7] { 0xD849, 0xA0B3, 0x5147, 0xA28E, 0x553D, 0xAA7A, 0x44D5 },
           new int[7] { 0x6F45, 0xDE8A, 0xAD35, 0x4A4B, 0x9496, 0x390D, 0x721A },
           new int[7] { 0xEB23, 0xC667, 0x9CEF, 0x29FF, 0x53FE, 0xA7FC, 0x5FD9 },
           new int[7] { 0x47D3, 0x8FA6, 0x0F6D, 0x1EDA, 0x3DB4, 0x7B68, 0xF6D0 },
           new int[7] { 0xB861, 0x60E3, 0xC1C6, 0x93AD, 0x377B, 0x6EF6, 0xDDEC },
           new int[7] { 0x45A0, 0x8B40, 0x06A1, 0x0D42, 0x1A84, 0x3508, 0x6A10 },
           new int[7] { 0xAA51, 0x4483, 0x8906, 0x022D, 0x045A, 0x08B4, 0x1168 },
           new int[7] { 0x76B4, 0xED68, 0xCAF1, 0x85C3, 0x1BA7, 0x374E, 0x6E9C },
           new int[7] { 0x3730, 0x6E60, 0xDCC0, 0xA9A1, 0x4363, 0x86C6, 0x1DAD },
           new int[7] { 0x3331, 0x6662, 0xCCC4, 0x89A9, 0x0373, 0x06E6, 0x0DCC },
           new int[7] { 0x1021, 0x2042, 0x4084, 0x8108, 0x1231, 0x2462, 0x48C4 },
        };

        private static readonly FlowProtectionHelper instance;

        static FlowProtectionHelper()
        {
            instance = new FlowProtectionHelper();
        }

        internal static FlowProtectionHelper Instance
        {
            get
            {
                return instance;
            }
        }

        protected override string PreprocessPassword(string password)
        {
            int legacyHash = FlowProtectionHelper.CalculateLegacyHash(password);

            string legacyHashHexadecimal = legacyHash.ToString("X");
            return legacyHashHexadecimal;
        }

        private static int CalculateLegacyHash(string password)
        {
            if (string.IsNullOrEmpty(password))
            {
                return 0;
            }

            byte[] buffer = FlowProtectionHelper.Pretransform(password);

            int highWord = initialCodeArray[buffer.Length];
            for (int i = 0; i < buffer.Length; ++i)
            {
                for (int j = 0; j < 7; ++j)
                {
                    if (((buffer[i] >> j) & 1) != 0)
                    {
                        highWord ^= cryptoMatrix[15 - buffer.Length + i][j];
                    }
                }
            }

            int lowWord = 0;
            for (int i = buffer.Length - 1; i >= 0; --i)
            {
                lowWord = (((lowWord >> 14) & 0x0001) | (lowWord << 1) & 0x7FFF) ^ buffer[i];
            }
            lowWord = (((lowWord >> 14) & 0x0001) | (lowWord << 1) & 0x7FFF) ^ password.Length ^ 0xCE4B;

            int b0 = lowWord & 0xFF;
            int b1 = (lowWord >> 8) & 0xFF;
            int b2 = highWord & 0xFF;
            int b3 = (highWord >> 8) & 0xFF;
            return (b0 << 24) | (b1 << 16) | (b2 << 8) | b3;
        }

        private static byte[] Pretransform(string password)
        {
            password = password.Substring(0, Math.Min(password.Length, 15));
            byte[] result = new byte[password.Length];
            for (int i = 0; i < password.Length; ++i)
            {
                char c = password[i];
                if ((c & 0xFF) == 0)
                {
                    result[i] = (byte)(c >> 8);
                }
                else
                {
                    result[i] = (byte)(c & 0xFF);
                }
            }
            return result;
        }
    }
}
