﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    /// <summary>
    /// Describes information for <see cref="PermissionRange" /> credentials.
    /// </summary>
    public class PermissionRangeCredentials
    {
        private string editor;
        private EditingGroup? editingGroup;

        /// <summary>
        /// Initializes a new instance of the <see cref="PermissionRangeCredentials" /> class.
        /// </summary>
        /// <param name="editor">The editor. Could be defined as DOMAIN\name, name or email.</param>
        public PermissionRangeCredentials(string editor)
        {
            Guard.ThrowExceptionIfNullOrEmpty(editor, "editor");
            Guard.ThrowExceptionIfContainsWhitespace(editor, "editor");

            this.editor = editor;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PermissionRangeCredentials" /> class.
        /// </summary>
        /// <param name="editingGroup">The editing group.</param>
        public PermissionRangeCredentials(EditingGroup editingGroup)
        {
            Guard.ThrowExceptionIfNull(editingGroup, "editingGroup");

            this.editingGroup = editingGroup;
        }

        private PermissionRangeCredentials()
        {
        }

        /// <summary>
        /// Gets or sets the editor.
        /// </summary>
        /// <value>The editor.</value>
        public string Editor
        {
            get
            {
                return this.editor;
            }
            set
            {
                this.editor = value;
            }
        }

        /// <summary>
        /// Gets or sets the editing group.
        /// </summary>
        /// <value>The editing group.</value>
        public EditingGroup? EditingGroup
        {
            get
            {
                return this.editingGroup;
            }
            set
            {
                this.editingGroup = value;
            }
        }

        internal PermissionRangeCredentials Clone()
        {
            PermissionRangeCredentials permissionCredentials = new PermissionRangeCredentials();
            permissionCredentials.Editor = this.Editor;
            permissionCredentials.EditingGroup = this.EditingGroup;

            return permissionCredentials;
        }
    }
}
