﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    /// <summary>
    /// Defines permission range end annotation.
    /// </summary>
    public class PermissionRangeEnd : AnnotationRangeEndBase
    {
        private readonly PermissionRange permission;

        internal PermissionRangeEnd(RadFlowDocument document, PermissionRange permission)
            : base(document)
        {
            this.permission = permission;
        }

        /// <summary>
        /// Gets the associated permission.
        /// </summary>
        /// <value>The permission.</value>
        public PermissionRange Permission
        {
            get
            {
                return this.permission;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.PermissionRangeEnd;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            PermissionRangeStart start = cloneContext.PermissionContext.PopHangingAnnotationStart();
            if (start != null)
            {
                return start.Permission.End;
            }

            return null;
        }
    }
}
