﻿using Telerik.Windows.Documents.Flow.Model.Cloning;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    /// <summary>
    /// Defines permission range start annotation.
    /// </summary>
    public class PermissionRangeStart : AnnotationRangeStartBase
    {
        private readonly PermissionRange permission;

        internal PermissionRangeStart(RadFlowDocument document, PermissionRange permission)
            : base(document)
        {
            this.permission = permission;
        }

        /// <summary>
        /// Gets the associated permission.
        /// </summary>
        /// <value>The permission.</value>
        public PermissionRange Permission
        {
            get
            {
                return this.permission;
            }
        }

        /// <summary>
        /// Gets the type of the element.
        /// </summary>
        /// <value>The type.</value>
        internal override DocumentElementType Type
        {
            get
            {
                return DocumentElementType.PermissionRangeStart;
            }
        }

        internal override DocumentElementBase CloneCore(CloneContext cloneContext)
        {
            Guard.ThrowExceptionIfNull(cloneContext, "cloneContext");

            PermissionRangeStart start = new PermissionRange(
                cloneContext.Document,
                this.Permission.Credentials.Clone(),
                this.Permission.FromColumn,
                this.Permission.ToColumn).Start;

            cloneContext.PermissionContext.AddHangingAnnotationStart(start);

            return start;
        }
    }
}
