﻿namespace Telerik.Windows.Documents.Flow.Model.Protection
{
    /// <summary>
    /// Contains all protection settings.
    /// </summary>
    public class ProtectionSettings
    {
        private static readonly ProtectionMode DefaultProtectionMode = ProtectionMode.NoProtection;
        private static readonly bool DefaultEnforced = false;
        private static readonly string DefaultAlgorithmName = ProtectionAlgorithmNames.SHA512;
        private static readonly string DefaultHash = string.Empty;
        private static readonly string DefaultSalt = string.Empty;
        private static readonly int DefaultSpinCount = 100000;

        internal ProtectionSettings()
        {
            // setting default values.
            this.ProtectionMode = ProtectionSettings.DefaultProtectionMode;
            this.Enforced = ProtectionSettings.DefaultEnforced;
            this.AlgorithmName = ProtectionSettings.DefaultAlgorithmName;
            this.Hash = ProtectionSettings.DefaultHash;
            this.Salt = ProtectionSettings.DefaultSalt;
            this.SpinCount = ProtectionSettings.DefaultSpinCount;
        }

        /// <summary>
        /// Gets or sets the protection mode.
        /// </summary>
        /// <value>The protection mode.</value>
        public ProtectionMode ProtectionMode { get; set; }

        /// <summary>
        /// Gets or sets the enforcement of document protection.
        /// </summary>
        /// <value>The enforced.</value>
        public bool Enforced { get; set; }

        /// <summary>
        /// Gets or sets the algorithm name. Use predefined values from <see cref="ProtectionAlgorithmNames" />.
        /// </summary>
        /// <value>The name of the algorithm. The protection algorithm.</value>
        public string AlgorithmName { get; set; }

        /// <summary>
        /// Gets or sets the salt.
        /// </summary>
        /// <value>The salt.</value>
        public string Salt { get; set; }

        /// <summary>
        /// Gets or sets the password hash.
        /// </summary>
        /// <value>The hash.</value>
        public string Hash { get; set; }

        /// <summary>
        /// Gets or sets the spin count.
        /// </summary>
        /// <value>The spin count.</value>
        public int SpinCount { get; set; }

        internal ProtectionSettings Clone()
        {
            ProtectionSettings protectionSettings = new ProtectionSettings();
            protectionSettings.ProtectionMode = this.ProtectionMode;
            protectionSettings.Enforced = this.Enforced;
            protectionSettings.AlgorithmName = this.AlgorithmName;
            protectionSettings.Salt = this.Salt;
            protectionSettings.Hash = this.Hash;
            protectionSettings.SpinCount = this.SpinCount;

            return protectionSettings;
        }
    }
}
