﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using BaselineAlignment = Telerik.Windows.Documents.Flow.Model.Styles.BaselineAlignment;
using FlowDirection = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection;

namespace Telerik.Windows.Documents.Flow.Model
{
    /// <summary>
    /// Defines a set of style property definitions which are used for initialization of all style properties in <see cref="CharacterProperties"/> element.
    /// </summary>
    public partial class Run
    {
        /// <summary>
        /// Gets the font size property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<double?> FontSizePropertyDefinition =
            new StylePropertyDefinition<double?>("FontSize", DocumentDefaultStyleSettings.FontSize, StylePropertyType.Character);

        /// <summary>
        /// Gets the font family property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableFontFamily> FontFamilyPropertyDefinition =
            new StylePropertyDefinition<ThemableFontFamily>("FontFamily", DocumentDefaultStyleSettings.FontFamily, StylePropertyType.Character);

        /// <summary>
        /// Gets the font style property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<FontStyle?> FontStylePropertyDefinition =
            new StylePropertyDefinition<FontStyle?>("FontStyle", DocumentDefaultStyleSettings.FontStyle, StylePropertyType.Character);

        /// <summary>
        /// Gets the font weight property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<FontWeight?> FontWeightPropertyDefinition =
            new StylePropertyDefinition<FontWeight?>("FontWeight", DocumentDefaultStyleSettings.FontWeight, StylePropertyType.Character);

        /// <summary>
        /// Gets the foreground color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> ForegroundColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("ForegroundColor", DocumentDefaultStyleSettings.ForegroundColor, StylePropertyType.Character);

        /// <summary>
        /// Gets the highlight color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<Color?> HighlightColorPropertyDefinition =
            new StylePropertyDefinition<Color?>("HighlightColor", DocumentDefaultStyleSettings.HighlightColor, StylePropertyType.Character);

        /// <summary>
        /// Gets the baseline alignment property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<BaselineAlignment?> BaselineAlignmentPropertyDefinition =
            new StylePropertyDefinition<BaselineAlignment?>("BaselineAlignment", DocumentDefaultStyleSettings.BaselineAlignment, StylePropertyType.Character);

        /// <summary>
        /// Gets the strikethrough property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> StrikethroughPropertyDefinition =
            new StylePropertyDefinition<bool?>("Strikethrough", DocumentDefaultStyleSettings.Strikethrough, StylePropertyType.Character);

        /// <summary>
        /// Gets the background color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> BackgroundColorPropertyDefinition
            = new StylePropertyDefinition<ThemableColor>("BackgroundColor", DocumentDefaultStyleSettings.BackgroundColor, StylePropertyType.Character);

        /// <summary>
        /// Gets the shading pattern color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> ShadingPatternColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("ShadingPatternColor", DocumentDefaultStyleSettings.ShadingPatternColor, StylePropertyType.Character);

        /// <summary>
        /// Gets the shading pattern property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ShadingPattern?> ShadingPatternPropertyDefinition =
            new StylePropertyDefinition<ShadingPattern?>("ShadingPattern", DocumentDefaultStyleSettings.ShadingPattern, StylePropertyType.Character);

        /// <summary>
        /// Gets the underline color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> UnderlineColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("UnderlineColor", DocumentDefaultStyleSettings.UnderlineColor, StylePropertyType.Character);

        /// <summary>
        /// Gets the underline pattern property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<UnderlinePattern?> UnderlinePatternPropertyDefinition =
            new StylePropertyDefinition<UnderlinePattern?>("UnderlinePattern", DocumentDefaultStyleSettings.UnderlinePattern, StylePropertyType.Character);

        /// <summary>
        /// Gets the flow direction property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<FlowDirection?> FlowDirectionPropertyDefinition =
            new StylePropertyDefinition<FlowDirection?>("FlowDirection", DocumentDefaultStyleSettings.FlowDirection, StylePropertyType.Character);

        static Run()
        {
            FontSizePropertyDefinition.Validation.AddRule(new ValidationRule<double>(value => value > 0));
            FontWeightPropertyDefinition.Validation.AddRule(new ValidationRule<FontWeight>(value => value == FontWeights.Bold || value == FontWeights.Normal));
            FontStylePropertyDefinition.Validation.AddRule(new ValidationRule<FontStyle>(value => value == FontStyles.Italic || value == FontStyles.Normal));
        }
    }
}
