namespace Telerik.Windows.Documents.Flow.Model.Shapes
{
    /// <summary>
    /// Specifies the possible values for the base from which the relative horizontal positioning of a shape shall be calculated.
    /// </summary>
    public enum HorizontalRelativeFrom
    {
        /// <summary>
        /// Specifies that the horizontal positioning shall be relative to the position of the anchor within its run content.
        /// </summary>
        Character,

        /// <summary>
        /// Relative to the extents of the column which contains its anchor.
        /// </summary>
        Column,

        /// <summary>
        /// Specifies that the horizontal positioning shall be relative to the edge of the page.
        /// </summary>
        Page,

        /// <summary>
        ///  Specifies that the horizontal positioning shall be relative to the page margins.
        /// </summary>
        Margin,

        /// <summary>
        /// Specifies that the horizontal positioning shall be relative to the left margin of the page.
        /// </summary>
        LeftMargin,

        /// <summary>
        /// Specifies that the horizontal positioning shall be relative to the right margin of the page.
        /// </summary>
        RightMargin,

        /// <summary>
        ///  Specifies that the horizontal positioning shall be relative to the inside margin of the current page (the left margin on odd pages, right on even pages).
        /// </summary>
        InsideMargin,

        /// <summary>
        /// Specifies that the horizontal positioning shall be relative to the outside margin of the current page (the right margin on odd pages, left on even pages).
        /// </summary>
        OutsideMargin
    }
}