﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Shapes
{
    /// <summary>
    /// Base class for shape wrapping.
    /// </summary>
    public class ShapeWrapping
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ShapeWrapping" /> class.
        /// </summary>
        /// <param name="wrappingType">Type of the wrapping.</param>
        /// <param name="textWrap">The text wrap.</param>
        public ShapeWrapping(ShapeWrappingType wrappingType, TextWrap textWrap)
        {
            this.WrappingType = wrappingType;
            this.TextWrap = textWrap;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ShapeWrapping" /> class.
        /// </summary>
        public ShapeWrapping()
            : this(ShapeWrappingType.Square, TextWrap.BothSides)
        {
        }

        /// <summary>
        /// Gets or sets the type of the wrapping.
        /// </summary>
        /// <value>The type of the wrapping.</value>
        public ShapeWrappingType WrappingType { get; set; }

        /// <summary>
        /// Gets or sets the value indicating how the text should be wrapped around the shape.
        /// </summary>
        /// <value>The text wrap.</value>
        public TextWrap TextWrap { get; set; }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            ShapeWrapping other = obj as ShapeWrapping;
            if (other == null)
            {
                return false;
            }

            return this.WrappingType == other.WrappingType &&
                   this.TextWrap == other.TextWrap;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(
                this.WrappingType.GetHashCode(),
                this.TextWrap.GetHashCode());
        }

        internal ShapeWrapping Clone()
        {
            ShapeWrapping shapeWrapping = new ShapeWrapping(this.WrappingType, this.TextWrap);

            return shapeWrapping;
        }
    }
}