namespace Telerik.Windows.Documents.Flow.Model.Shapes
{
    /// <summary>
    /// Specifies the possible values for the base from which the relative vertical positioning of a shape shall be calculated.
    /// </summary>
    public enum VerticalRelativeFrom
    {
        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the line containing the anchor character.
        /// </summary>
        Line,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the paragraph which contains the drawing anchor.
        /// </summary>
        Paragraph,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the edge of the page.
        /// </summary>
        Page,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the page margins.
        /// </summary>
        Margin,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the top margin of the current page.
        /// </summary>
        TopMargin,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the bottom margin of the current page.
        /// </summary>
        BottomMargin,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the outside margin of the current page.
        /// </summary>
        OutsideMargin,

        /// <summary>
        /// Specifies that the vertical positioning shall be relative to the inside margin of the current page.
        /// </summary>
        InsideMargin
    }
}