﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Represents static holder of built in style names.
    /// </summary>
    public static class BuiltInStyleNames
    {
        /// <summary>
        /// Gets the id of the default table style.
        /// </summary>
        /// <value>Style id.</value>
        public const string NormalTableStyleId = "TableNormal";

        /// <summary>
        /// Gets the name of the default table style.
        /// </summary>
        /// <value>Style name.</value>
        public const string NormalTableStyleName = "Table Normal";

        /// <summary>
        /// Gets the id of the table grid style.
        /// </summary>
        /// <value>Style id.</value>
        public const string TableGridStyleId = "TableGrid";

        /// <summary>
        /// Gets the name of the table grid style.
        /// </summary>
        /// <value>Style name.</value>
        public const string TableGridStyleName = "Table Grid";

        /// <summary>
        /// Gets the id of the normal style.
        /// </summary>
        /// <value>Style id.</value>
        public const string NormalStyleId = "Normal";

        /// <summary>
        /// Gets the name of the normal style.
        /// </summary>
        /// <value>Style name.</value>
        public const string NormalStyleName = "Normal";

        /// <summary>
        /// Gets the id of the normal web style.
        /// </summary>
        /// <value>Style id.</value>
        public const string NormalWebStyleId = "NormalWeb";

        /// <summary>
        /// Gets the name of the normal web style.
        /// </summary>
        /// <value>Style name.</value>
        public const string NormalWebStyleName = "Normal (Web)";

        /// <summary>
        /// Gets the id of the hyperlink style.
        /// </summary>
        /// <value>Style id.</value>
        public const string HyperlinkStyleId = "Hyperlink";

        /// <summary>
        /// Gets the name of the hyperlink style.
        /// </summary>
        /// <value>Style name.</value>
        public const string HyperlinkStyleName = "Hyperlink";

        /// <summary>
        /// Gets the id of the caption style.
        /// </summary>
        /// <value>Style id.</value>
        public const string CaptionStyleId = "Caption";

        /// <summary>
        /// Gets the name of the caption style.
        /// </summary>
        /// <value>Style name.</value>
        public const string CaptionStyleName = "Caption";

        /// <summary>
        /// Gets the id of the table of figures style.
        /// </summary>
        /// <value>Style id.</value>
        public const string TofStyleId = "TableofFigures";

        /// <summary>
        /// Gets the name of the table of figures style.
        /// </summary>
        /// <value>Style name.</value>
        public const string TofStyleName = "table of figures";

        /// <summary>
        /// Gets the id of the footnote reference style.
        /// </summary>
        /// <value>Style id.</value>
        public const string FootnoteReferenceStyleId = "FootnoteReference";

        /// <summary>
        /// Gets the name of the footnote reference style.
        /// </summary>
        /// <value>Style name.</value>
        public const string FootnoteReferenceStyleName = "footnote reference";

        /// <summary>
        /// Gets the id of the footnote text style.
        /// </summary>
        /// <value>Style id.</value>
        public const string FootnoteTextStyleId = "FootnoteText";

        /// <summary>
        /// Gets the name of the footnote text style.
        /// </summary>
        /// <value>Style name.</value>
        public const string FootnoteTextStyleName = "footnote text";

        /// <summary>
        /// Gets the id of the footnote text character style.
        /// </summary>
        /// <value>Style id.</value>
        public const string FootnoteTextCharStyleId = "FootnoteTextChar";

        /// <summary>
        /// Gets the name of the footnote text character style.
        /// </summary>
        /// <value>Style name.</value>
        public const string FootnoteTextCharStyleName = "Footnote Text Char";

        /// <summary>
        /// Gets the id of the endnote reference style.
        /// </summary>
        /// <value>Style id.</value>
        public const string EndnoteReferenceStyleId = "EndnoteReference";

        /// <summary>
        /// Gets the name of the endnote reference style.
        /// </summary>
        /// <value>Style name.</value>
        public const string EndnoteReferenceStyleName = "endnote reference";

        /// <summary>
        /// Gets the name of the endnote text style.
        /// </summary>
        /// <value>Style name.</value>
        public const string EndnoteTextStyleId = "EndnoteText";

        /// <summary>
        /// Gets the name of the endnote text style.
        /// </summary>
        /// <value>Style name.</value>
        public const string EndnoteTextStyleName = "endnote text";

        /// <summary>
        /// Gets the id of the endnote text character style.
        /// </summary>
        /// <value>Style id.</value>
        public const string EndnoteTextCharStyleId = "EndnoteTextChar";

        /// <summary>
        /// Gets the name of the endnote text character style.
        /// </summary>
        /// <value>Style name.</value>
        public const string EndnoteTextCharStyleName = "Endnote Text Char";

        internal const string HeadingStyleIdTemplate = "Heading{0}";
        internal const string HeadingStyleNameTemplate = "Heading {0}";
        internal const string HeadingCharStyleIdTemplate = "Heading{0}Char";
        internal const string HeadingCharStyleNameTemplate = "Heading {0} Char";
        internal const string TocStyleIdTemplate = "TOC{0}";
        internal const string TocStyleNameTemplate = "toc {0}";

        /// <summary>
        /// Gets the heading style ID by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style ID.</returns>
        public static string GetHeadingStyleIdByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.HeadingStyleIdTemplate, index);
        }

        /// <summary>
        /// Gets the heading style name by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style name.</returns>
        public static string GetHeadingStyleNameByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.HeadingStyleNameTemplate, index);
        }

        /// <summary>
        /// Gets the heading character style id by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style id.</returns>
        public static string GetHeadingCharStyleIdByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.HeadingCharStyleIdTemplate, index);
        }

        /// <summary>
        /// Gets the heading character style name by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style name.</returns>
        public static string GetHeadingCharStyleNameByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.HeadingCharStyleNameTemplate, index);
        }

        /// <summary>
        /// Gets the table of content style ID by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style id.</returns>
        public static string GetTocStyleIdByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.TocStyleIdTemplate, index);
        }

        /// <summary>
        /// Gets the table of content style name by index in the range 1 - 9.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns>Style name.</returns>
        public static string GetTocStyleNameByIndex(int index)
        {
            Guard.ThrowExceptionIfOutOfRange(1, 9, index, "index");

            return string.Format(BuiltInStyleNames.TocStyleNameTemplate, index);
        }
    }
}
