﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif

#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    internal static class BuiltInStylesGenerator
    {
        public static Style NormalStyle(StyleType type)
        {
            Style normalStyle = new Style(BuiltInStyleNames.NormalStyleId, type)
            {
                Name = BuiltInStyleNames.NormalStyleName,
                IsDefault = true,
                IsCustom = false,
            };

            return normalStyle;
        }

        public static Style NormalWebStyle(StyleType type)
        {
            Style normalWebStyle = new Style(BuiltInStyleNames.NormalWebStyleId, type)
            {
                Name = BuiltInStyleNames.NormalWebStyleName,
                IsDefault = false,
                IsCustom = false,
                BasedOnStyleId = BuiltInStyleNames.NormalStyleId,
                UIPriority = 99,
            };

            normalWebStyle.CharacterProperties.FontSize.LocalValue = 16;
            normalWebStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Times New Roman"));

            return normalWebStyle;
        }

        public static Style Heading1Style(StyleType type)
        {
            Style heading1Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(1), type);
            heading1Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(1);
            heading1Style.IsCustom = false;
            heading1Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading1Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading1Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading1Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading1Style.ParagraphProperties.SpacingBefore.LocalValue = 32;
            heading1Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level1;
            heading1Style.CharacterProperties.FontSize.LocalValue = 18.666666030883789d;
            heading1Style.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading1Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading1Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading1CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(1), StyleType.Character);
            heading1CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(1);
            heading1CharStyle.IsCustom = false;
            heading1CharStyle.IsPrimary = false;
            heading1CharStyle.CharacterProperties.FontSize.LocalValue = 18.666666030883789d;
            heading1CharStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading1CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading1CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading1Style, heading1CharStyle);

            return heading1Style;
        }

        public static Style Heading2Style(StyleType type)
        {
            Style heading2Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(2), type);
            heading2Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading2Style.IsCustom = false;
            heading2Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(2);
            heading2Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading2Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading2Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading2Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading2Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level2;
            heading2Style.CharacterProperties.FontSize.LocalValue = 17.333333969116211d;
            heading2Style.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading2Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading2Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading2CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(2), StyleType.Character);
            heading2CharStyle.IsCustom = false;
            heading2CharStyle.IsPrimary = false;
            heading2CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(2);
            heading2CharStyle.CharacterProperties.FontSize.LocalValue = 17.333333969116211;
            heading2CharStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading2CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading2CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading2Style, heading2CharStyle);

            return heading2Style;
        }

        public static Style Heading3Style(StyleType type)
        {
            Style heading3Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(3), type);
            heading3Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading3Style.IsCustom = false;
            heading3Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(3);
            heading3Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading3Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading3Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading3Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading3Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level3;
            heading3Style.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading3Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading3Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading3CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(3), StyleType.Character);
            heading3CharStyle.IsCustom = false;
            heading3CharStyle.IsPrimary = false;
            heading3CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(3);
            heading3CharStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading3CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading3CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading3Style, heading3CharStyle);

            return heading3Style;
        }

        public static Style Heading4Style(StyleType type)
        {
            Style heading4Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(4), type);
            heading4Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading4Style.IsCustom = false;
            heading4Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(4);
            heading4Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading4Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading4Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading4Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading4Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level4;
            heading4Style.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading4Style.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading4Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading4Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading4CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(4), StyleType.Character);
            heading4CharStyle.IsCustom = false;
            heading4CharStyle.IsPrimary = false;
            heading4CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(4);
            heading4CharStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading4CharStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            heading4CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading4CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading4Style, heading4CharStyle);

            return heading4Style;
        }

        public static Style Heading5Style(StyleType type)
        {
            Style heading5Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(5), type);
            heading5Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading5Style.IsCustom = false;
            heading5Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(5);
            heading5Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading5Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading5Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading5Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading5Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level5;
            heading5Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading5Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading5CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(5), StyleType.Character);
            heading5CharStyle.IsCustom = false;
            heading5CharStyle.IsPrimary = false;
            heading5CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(5);
            heading5CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading5CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading5Style, heading5CharStyle);

            return heading5Style;
        }

        public static Style Heading6Style(StyleType type)
        {
            Style heading6Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(6), type);
            heading6Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading6Style.IsCustom = false;
            heading6Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(6);
            heading6Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading6Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading6Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading6Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading6Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level6;
            heading6Style.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading6Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading6Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            Style heading6CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(6), StyleType.Character);
            heading6CharStyle.IsCustom = false;
            heading6CharStyle.IsPrimary = false;
            heading6CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(6);
            heading6CharStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading6CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading6CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            BuiltInStylesGenerator.LinkStyles(heading6Style, heading6CharStyle);

            return heading6Style;
        }

        public static Style Heading7Style(StyleType type)
        {
            Style heading7Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(7), type);
            heading7Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading7Style.IsCustom = false;
            heading7Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(7);
            heading7Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading7Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading7Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading7Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading7Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level7;
            heading7Style.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading7Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading7Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            Style heading7CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(7), StyleType.Character);
            heading7CharStyle.IsCustom = false;
            heading7CharStyle.IsPrimary = false;
            heading7CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(7);
            heading7CharStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading7CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading7CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            BuiltInStylesGenerator.LinkStyles(heading7Style, heading7CharStyle);

            return heading7Style;
        }

        public static Style Heading8Style(StyleType type)
        {
            Style heading8Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(8), type);
            heading8Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading8Style.IsCustom = false;
            heading8Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(8);
            heading8Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading8Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading8Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading8Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading8Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level8;
            heading8Style.CharacterProperties.FontSize.LocalValue = 13.333333015441895;
            heading8Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading8Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            Style heading8CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(8), StyleType.Character);
            heading8CharStyle.IsCustom = false;
            heading8CharStyle.IsPrimary = false;
            heading8CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(8);
            heading8CharStyle.CharacterProperties.FontSize.LocalValue = 13.333333015441895;
            heading8CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading8CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            BuiltInStylesGenerator.LinkStyles(heading8Style, heading8CharStyle);

            return heading8Style;
        }

        public static Style Heading9Style(StyleType type)
        {
            Style heading9Style = new Style(BuiltInStyleNames.GetHeadingStyleIdByIndex(9), type);
            heading9Style.ParagraphProperties.KeepOnOnePage.LocalValue = true;
            heading9Style.IsCustom = false;
            heading9Style.Name = BuiltInStyleNames.GetHeadingStyleNameByIndex(9);
            heading9Style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            heading9Style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            heading9Style.ParagraphProperties.SpacingAfter.LocalValue = 0;
            heading9Style.ParagraphProperties.SpacingBefore.LocalValue = 13.333333015441895;
            heading9Style.ParagraphProperties.OutlineLevel.LocalValue = OutlineLevel.Level9;
            heading9Style.CharacterProperties.FontSize.LocalValue = 13.333333015441895;
            heading9Style.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading9Style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading9Style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            Style heading9CharStyle = new Style(BuiltInStyleNames.GetHeadingCharStyleIdByIndex(9), StyleType.Character);
            heading9CharStyle.IsCustom = false;
            heading9CharStyle.IsPrimary = false;
            heading9CharStyle.Id = BuiltInStyleNames.GetHeadingCharStyleIdByIndex(9);
            heading9CharStyle.Name = BuiltInStyleNames.GetHeadingCharStyleNameByIndex(9);
            heading9CharStyle.CharacterProperties.FontSize.LocalValue = 13.333333015441895;
            heading9CharStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            heading9CharStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Major);
            heading9CharStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Text1);

            BuiltInStylesGenerator.LinkStyles(heading9Style, heading9CharStyle);

            return heading9Style;
        }

        public static Style CaptionStyle(StyleType type)
        {
            Style captionStyle = new Style(BuiltInStyleNames.CaptionStyleId, type);
            captionStyle.IsCustom = false;
            captionStyle.Name = BuiltInStyleNames.CaptionStyleName;
            captionStyle.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            captionStyle.NextStyleId = BuiltInStyleNames.NormalStyleId;
            captionStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            captionStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            captionStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            captionStyle.CharacterProperties.FontSize.LocalValue = 12;
            captionStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            captionStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent1);

            ////by default in Word the FontFamily is not set explicitly for Caption style, here we set it because our documentDefaultValue for
            ////FontFamily is Verdana and theirs is Calibri
            captionStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Minor);

            return captionStyle;
        }

        public static Style HyperlinkStyle(StyleType type)
        {
            Style hyperlinkStyle = new Style(BuiltInStyleNames.HyperlinkStyleName, type);
            hyperlinkStyle.IsCustom = false;
            hyperlinkStyle.Name = BuiltInStyleNames.HyperlinkStyleName;
            hyperlinkStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Blue);
            hyperlinkStyle.CharacterProperties.UnderlineColor.LocalValue = new ThemableColor(Colors.Blue);
            hyperlinkStyle.CharacterProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;

            return hyperlinkStyle;
        }

        public static Style FootnoteReferenceStyle(StyleType type)
        {
            Style footnoteRefStyle = BuiltInStylesGenerator.CreateNoteStyle(
                BuiltInStyleNames.FootnoteReferenceStyleId,
                BuiltInStyleNames.FootnoteReferenceStyleName,
                type);

            footnoteRefStyle.CharacterProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;

            return footnoteRefStyle;
        }

        public static Style FootnoteTextStyle(StyleType type)
        {
            Style footnoteTextStyle = BuiltInStylesGenerator.CreateNoteStyle(
                BuiltInStyleNames.FootnoteTextStyleId,
                BuiltInStyleNames.FootnoteTextStyleName,
                type);

            footnoteTextStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            footnoteTextStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            footnoteTextStyle.ParagraphProperties.SpacingAfter.LocalValue = 0;
            footnoteTextStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(10);

            Style footnoteTextStyleChar = BuiltInStylesGenerator.CreateNoteStyle(
                BuiltInStyleNames.FootnoteTextCharStyleId,
                BuiltInStyleNames.FootnoteTextCharStyleName,
                StyleType.Character);

            footnoteTextStyleChar.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(10);

            BuiltInStylesGenerator.LinkStyles(footnoteTextStyleChar, footnoteTextStyle);

            return footnoteTextStyle;
        }

        public static Style EndnoteReferenceStyle(StyleType type)
        {
            Style endnoteRefStyle = BuiltInStylesGenerator.CreateNoteStyle(
                BuiltInStyleNames.EndnoteReferenceStyleId,
                BuiltInStyleNames.EndnoteReferenceStyleName,
                type);

            endnoteRefStyle.CharacterProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;

            return endnoteRefStyle;
        }

        public static Style EndnoteTextStyle(StyleType type)
        {
            Style endnoteTextStyle = BuiltInStylesGenerator.CreateNoteStyle(BuiltInStyleNames.EndnoteTextStyleId, BuiltInStyleNames.EndnoteTextStyleName, type);
            endnoteTextStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            endnoteTextStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            endnoteTextStyle.ParagraphProperties.SpacingAfter.LocalValue = 0;
            endnoteTextStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(10);

            Style endnoteTextStyleChar = BuiltInStylesGenerator.CreateNoteStyle(BuiltInStyleNames.EndnoteTextCharStyleId, BuiltInStyleNames.EndnoteTextCharStyleName, StyleType.Character);
            endnoteTextStyleChar.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(10);

            BuiltInStylesGenerator.LinkStyles(endnoteTextStyleChar, endnoteTextStyle);

            return endnoteTextStyle;
        }

        public static Style TableNormalStyle(StyleType type)
        {
            Style normalTableStyle = new Style(BuiltInStyleNames.NormalTableStyleId, type);
            normalTableStyle.Name = BuiltInStyleNames.NormalTableStyleName;
            normalTableStyle.IsDefault = true;
            normalTableStyle.IsCustom = false;
            normalTableStyle.TableProperties.TableCellPadding.LocalValue = new Padding(7.2, 0, 7.2, 0);

            return normalTableStyle;
        }

        public static Style TableGridStyle(StyleType type)
        {
            Style tableGridStyle = new Style(BuiltInStyleNames.TableGridStyleId, StyleType.Table);
            tableGridStyle.Name = BuiltInStyleNames.TableGridStyleName;
            tableGridStyle.BasedOnStyleId = BuiltInStyleNames.NormalTableStyleId;
            tableGridStyle.ParagraphProperties.SpacingAfter.LocalValue = 0;
            tableGridStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            tableGridStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            tableGridStyle.IsCustom = false;
            tableGridStyle.TableProperties.Borders.LocalValue = new TableBorders(new Border(1f, BorderStyle.Single, new ThemableColor(Colors.Black)));

            return tableGridStyle;
        }

        public static Style TofStyle(StyleType type)
        {
            Style tofStyle = new Style(BuiltInStyleNames.TofStyleId, type);
            tofStyle.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            tofStyle.IsCustom = false;
            tofStyle.IsDefault = false;
            tofStyle.IsPrimary = false;
            tofStyle.NextStyleId = BuiltInStyleNames.NormalStyleId;
            tofStyle.Name = BuiltInStyleNames.TofStyleName;
            tofStyle.ParagraphProperties.SpacingAfter.LocalValue = 6.6666665077209473d;
            tofStyle.ParagraphProperties.SpacingAfter.LocalValue = 0d;

            return tofStyle;
        }

        public static Style GetToc1Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(1, type);
        }

        public static Style GetToc2Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(2, type);
        }

        public static Style GetToc3Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(3, type);
        }

        public static Style GetToc4Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(4, type);
        }

        public static Style GetToc5Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(5, type);
        }

        public static Style GetToc6Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(6, type);
        }

        public static Style GetToc7Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(7, type);
        }

        public static Style GetToc8Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(8, type);
        }

        public static Style GetToc9Style(StyleType type)
        {
            return BuiltInStylesGenerator.GetTocStyle(9, type);
        }

        public static Style GetTocStyle(int index, StyleType type)
        {
            Style tocStyle = new Style(string.Format(BuiltInStyleNames.GetTocStyleIdByIndex(index)), type);
            tocStyle.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            tocStyle.IsCustom = false;
            tocStyle.IsDefault = false;
            tocStyle.NextStyleId = BuiltInStyleNames.NormalStyleId;
            tocStyle.Name = BuiltInStyleNames.GetTocStyleNameByIndex(index);
            tocStyle.ParagraphProperties.SpacingAfter.LocalValue = 6.6666665077209473d;
            if (index > 1)
            {
                tocStyle.ParagraphProperties.LeftIndent.LocalValue = 14.666666984558105d * (index - 1);
            }

            return tocStyle;
        }

        private static Style CreateNoteStyle(string name, string dispalyName, StyleType type)
        {
            Style noteStyle = new Style(name, type);
            noteStyle.Name = dispalyName;
            noteStyle.IsCustom = false;
            noteStyle.IsPrimary = false;
            return noteStyle;
        }

        private static void LinkStyles(Style paragraphStyle, Style charStyle)
        {
            paragraphStyle.LinkedStyleId = charStyle.Id;
            charStyle.LinkedStyleId = paragraphStyle.Id;
        }
    }
}
