﻿namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    /// <summary>
    /// Establishes common functionality for generic style properties.
    /// </summary>
    public interface IStyleProperty<TValue> : IStyleProperty
    {
        /// <summary>
        /// Gets or sets the local value.
        /// </summary>
        /// <param name="value">The value.</param>
        TValue LocalValue { get; set; }

        /// <summary>
        /// Gets the actual value.
        /// </summary>
        /// <returns></returns>
        TValue GetActualValue();
    }

    /// <summary>
    /// Establishes common properties and functionality for style properties.
    /// </summary>
    public interface IStyleProperty
    {
        /// <summary>
        /// Gets the property definition.
        /// </summary>
        /// <value>The property definition.</value>
        IStylePropertyDefinition PropertyDefinition { get; }

        /// <summary>
        /// Gets a value indicating whether the style property has local value.
        /// </summary>
        /// <value>The has local value.</value>
        bool HasLocalValue { get; }

        /// <summary>
        /// Gets the local value as object.
        /// </summary>
        /// <returns></returns>
        object GetLocalValueAsObject();

        /// <summary>
        /// Gets the actual value as object.
        /// </summary>
        /// <returns></returns>
        object GetActualValueAsObject();

        /// <summary>
        /// Sets the local value as object.
        /// </summary>
        /// <param name="value">The value.</param>
        void SetValueAsObject(object value);

        /// <summary>
        /// Clears the local value.
        /// </summary>
        void ClearValue();
    }
}
