﻿using System;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    /// <summary>
    /// Establishes common properties for style property definition.
    /// </summary>
    public interface IStylePropertyDefinition
    {
        /// <summary>
        /// Gets the global property index.
        /// </summary>
        /// <value>The global property index.</value>
        [EditorBrowsable(EditorBrowsableState.Never)]
        int GlobalPropertyIndex { get; }

        /// <summary>
        /// Gets the type of the property.
        /// </summary>
        /// <value>The type of the property.</value>
        [EditorBrowsable(EditorBrowsableState.Never)]
        Type PropertyType { get; }

        /// <summary>
        /// Gets the name of the property.
        /// </summary>
        /// <value>The name of the property.</value>
        string PropertyName { get; }

        /// <summary>
        /// Gets the type of the style property.
        /// </summary>
        /// <value>The type of the style property.</value>
        StylePropertyType StylePropertyType { get; }

        /// <summary>
        /// Gets the validation object for the current style property definition.
        /// </summary>
        /// <value>The validation.</value>
        StylePropertyValidation Validation { get; }

        /// <summary>
        /// Gets the actual value as object.
        /// </summary>
        /// <returns></returns>
        object GetDefaultValueAsObject();
    }
}
