﻿namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    internal class LineSpacingTypeStyleProperty : StyleProperty<HeightType?>
    {
        public LineSpacingTypeStyleProperty(StylePropertyDefinition<HeightType?> propertyDefinition, DocumentElementPropertiesBase propertyContainer)
            : base(propertyDefinition, propertyContainer)
        {
        }

        private ParagraphProperties OwnerProperties
        {
            get
            {
                return (ParagraphProperties)this.PropertyContainer;
            }
        }

        private StyleRepository StyleRepository
        {
            get
            {
                return this.PropertyContainer.Document.StyleRepository;
            }
        }

        public override HeightType? GetActualValue()
        {
            if (this.PropertyContainer.Document != null)
            {
                return this.EvaluateActualValue(this.OwnerProperties);
            }
            else if (this.PropertyContainer.OwnerStyle != null)
            {
                return base.GetActualValue();
            }

            return this.DefaultValue;
        }

        private static string GetStyleId(ParagraphProperties properties)
        {
            string styleId = properties.StyleId;
            if (properties.OwnerStyle != null)
            {
                styleId = properties.OwnerStyle.Id;
            }

            return styleId;
        }

        private static HeightType GetValue(ParagraphProperties properties)
        {
            return properties.LineSpacingType.GetLocalValueAsObject() as HeightType? ?? HeightType.Auto;
        }

        private HeightType? EvaluateActualValue(ParagraphProperties properties)
        {
            IStyleProperty<double?> lineSpacing = properties.LineSpacing;
            if (lineSpacing.HasLocalValue)
            {
                return GetValue(properties);
            }
            else
            {
                string styleId = GetStyleId(properties);
                if (!string.IsNullOrEmpty(styleId))
                {
                    Style style = this.StyleRepository.GetStyle(styleId);
                    if (style != null)
                    {
                        if (style.ParagraphProperties.LineSpacing.HasLocalValue)
                        {
                            return GetValue(style.ParagraphProperties);
                        }
                        else
                        {
                            string basedOnStyleId = style.BasedOnStyleId;
                            if (!string.IsNullOrEmpty(basedOnStyleId))
                            {
                                Style basedOnStyle = this.StyleRepository.GetStyle(basedOnStyleId);
                                if (basedOnStyle != null)
                                {
                                    return this.EvaluateActualValue(basedOnStyle.ParagraphProperties);
                                }
                            }
                        }
                    }
                }
                else
                {
                    Style defaultParagraphStyle = this.StyleRepository.GetDefaultStyle(StyleType.Paragraph);
                    if (defaultParagraphStyle != null)
                    {
                        return this.EvaluateActualValue(defaultParagraphStyle.ParagraphProperties);
                    }
                }
            }

            double? documentLineSpacingDefault =
                ((StyleProperty<double?>)this.OwnerProperties.LineSpacing).GetDocumentDefaultValue(this.PropertyContainer.Document);

            if (documentLineSpacingDefault.HasValue)
            {
                return this.GetDocumentDefaultValue(this.PropertyContainer.Document) ?? this.DefaultValue;
            }

            return this.DefaultValue;
        }
    }
}
