﻿using System;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    /// <summary>
    /// A generic content element that defines a row within a <see cref="Table"/>.
    /// </summary>
    public class StylePropertyDefinition<TValue> : IStylePropertyDefinition
    {
        private readonly Type propertyType;

        internal StylePropertyDefinition(string propertyName, TValue defaultValue, StylePropertyType stylePropertyType)
        {
            this.GlobalPropertyIndex = GlobalPropertyIndexGenerator.GetNext(stylePropertyType);
            this.StylePropertyType = stylePropertyType;
            this.PropertyName = propertyName;
            this.DefaultValue = defaultValue;
            this.Validation = new StylePropertyValidation();

            this.propertyType = typeof(TValue);
        }

        /// <summary>
        /// Gets the index of the global property.
        /// </summary>
        /// <value>
        /// The index of the global property.
        /// </value>
        [EditorBrowsable(EditorBrowsableState.Never)]
        public int GlobalPropertyIndex { get; private set; }

        /// <summary>
        /// Gets the type of the property.
        /// </summary>
        /// <value>The type of the property.</value>
        [EditorBrowsable(EditorBrowsableState.Never)]
        public Type PropertyType
        {
            get
            {
                return this.propertyType;
            }
        }

        /// <summary>
        /// Gets the name of the property.
        /// </summary>
        /// <value>The name of the property.</value>
        public string PropertyName { get; private set; }

        /// <summary>
        /// Gets the default value of the current style property definition.
        /// </summary>
        /// <value>The default value.</value>
        public TValue DefaultValue { get; private set; }

        /// <summary>
        /// Gets the type of the style property.
        /// </summary>
        /// <value>The type of the style property.</value>
        public StylePropertyType StylePropertyType { get; private set; }

        /// <summary>
        /// Gets the validation object for the current style property definition.
        /// </summary>
        /// <value>The validation.</value>
        public StylePropertyValidation Validation { get; private set; }

        /// <summary>
        /// Gets the actual value as object.
        /// </summary>
        /// <returns></returns>
        public object GetDefaultValueAsObject()
        {
            return this.DefaultValue;
        }
    }
}
