﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    /// <summary>
    /// Providers validation functionality for the new value of specific style property.
    /// </summary>
    public class StylePropertyValidation
    {
        private readonly List<IValidationRule> rules;

        /// <summary>
        /// Initializes a new instance of the <see cref="StylePropertyValidation" /> class.
        /// </summary>
        internal StylePropertyValidation()
        {
            this.rules = new List<IValidationRule>();
        }

        /// <summary>
        /// Gets the validation rules.
        /// </summary>
        /// <value>The validation rules.</value>
        internal IEnumerable<IValidationRule> Rules
        {
            get
            {
                return this.rules;
            }
        }

        /// <summary>
        /// Determines whether the specified value is valid.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public bool IsValid(object value)
        {
            foreach (IValidationRule rule in this.Rules)
            {
                if (!rule.IsValid(value))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Adds the validation rule.
        /// </summary>
        /// <param name="rule">The validation rule.</param>
        internal void AddRule(IValidationRule rule)
        {
            this.rules.Add(rule);
        }

        /// <summary>
        /// Removes validation rule.
        /// </summary>
        /// <param name="rule">The validation rule.</param>
        internal void RemoveRule(IValidationRule rule)
        {
            this.rules.Remove(rule);
        }

        /// <summary>
        /// Clears all validation rules.
        /// </summary>
        internal void ClearRules()
        {
            this.rules.Clear();
        }
    }
}
