﻿using System;

namespace Telerik.Windows.Documents.Flow.Model.Styles.Core
{
    /// <summary>
    /// Providers single rule validation functionality for the new value of specific style properties.
    /// </summary>
    internal class ValidationRule<TValue> : IValidationRule
    {
        private readonly Func<TValue, bool> rule;

        /// <summary>
        /// Initializes a new instance of the <see cref="ValidationRule{TValue}" /> class.
        /// </summary>
        /// <param name="rule">The rule.</param>
        public ValidationRule(Func<TValue, bool> rule)
        {
            this.rule = rule;
        }

        /// <summary>
        /// Determines whether the specified value is valid.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public bool IsValid(TValue value)
        {
            return this.rule(value);
        }

        /// <summary>
        /// Determines whether the specified value is valid.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public bool IsValid(object value)
        {
            return this.IsValid((TValue)value);
        }
    }
}
