﻿namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Defines a set of default styling properties which are applied to the characters or paragraphs in a <see cref="RadFlowDocument"/> element.
    /// </summary>
    public sealed class DocumentDefaultStyle
    {
        internal DocumentDefaultStyle(RadFlowDocument document)
        {
            this.CharacterProperties = new CharacterProperties(document, true);
            this.ParagraphProperties = new ParagraphProperties(document, true);
        }

        /// <summary>
        /// Gets the default character styling properties of <see cref="RadFlowDocument"/> element.
        /// </summary>
        /// <value>The character properties.</value>
        public CharacterProperties CharacterProperties { get; private set; }

        /// <summary>
        /// Gets the styling properties of <see cref="RadFlowDocument"/> element.
        /// </summary>
        /// <value>The paragraph properties.</value>
        public ParagraphProperties ParagraphProperties { get; private set; }
    }
}
