﻿namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Describes the alignment of a rendered text.
    /// </summary>
    public enum Alignment
    {
        /// <summary>
        /// Specifies that the text should aligns with the left of the screen.
        /// </summary>
        Left,

        /// <summary>
        /// Specifies that the text should aligns with the center of the screen.
        /// </summary>
        Center,

        /// <summary>
        /// Specifies that the text should aligns with the right of the screen.
        /// </summary>
        Right,

        /// <summary>
        /// Specifies that the text should be justified between the left and right of the screen. 
        /// This option affects only the spacing between each word.
        /// </summary>
        Justified,

        /// <summary>
        /// Specifies that the text should be justified between the left and right of the screen. 
        /// This option affects the spacing between the words and the characters.
        /// </summary>
        Distribute,

        /// <summary>
        /// Specifies that the kashida length for the text should be extended to its widest possible value.
        /// This option affect only kashidas, which are special characters used to extend the joiner between two Arabic characters.
        /// </summary>
        HighKashida,

        /// <summary>
        /// Specifies that the kashida length for the text should be extended to medium value.
        /// This option affect only kashidas, which are special characters used to extend the joiner between two Arabic characters.
        /// </summary>
        MediumKashida,

        /// <summary>
        /// Specifies that the kashida length for the text should be extended to longer value.
        /// This option affect only kashidas, which are special characters used to extend the joiner between two Arabic characters.
        /// </summary>
        LowKashida,

        /// <summary>
        /// Specifies that the text should be justified with optimizations for Thai.
        /// </summary>
        ThaiDistribute
    }
}
