﻿namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Describes the possible types for the style of the border.
    /// </summary>
    public enum BorderStyle
    {
        /// <summary>
        /// Specifies that no border should be applied.
        /// If set to a table cell this value indicates that the table border should be calculated based on the table style.
        /// </summary>
        Inherit,

        /// <summary>
        /// Specifies that no border should be applied on this element.
        /// Table cell borders will not inherit their value from the containing table.
        /// </summary>
        None,

        /// <summary>
        /// Specifies that the border should be from one line.
        /// </summary>
        Single,

        /// <summary>
        /// Specifies that the border should be from one doted line.
        /// </summary>
        Dotted,

        /// <summary>
        /// Specifies that the border should be from one dashed line.
        /// </summary>
        Dashed,

        /// <summary>
        /// Specifies that the border should be from one dashed line with small gaps.
        /// </summary>
        DashSmallGap,

        /// <summary>
        /// Specifies that the border should be from alternating dotted and dashed line.
        /// </summary>
        DotDash,

        /// <summary>
        /// Specifies that the border should be from alternating dotted, dotted, dashed line.
        /// </summary>
        DotDotDash,

        /// <summary>
        /// Specifies that the border should be from double line.
        /// </summary>
        Double,

        /// <summary>
        /// Specifies that the border should be from triple line.
        /// </summary>
        Triple,

        /// <summary>
        /// Specifies that the border should be from one heavy line.
        /// </summary>
        Thick,

        /// <summary>
        /// Specifies that the border should consist thick line followed by thin line with small gap between them.
        /// </summary>
        ThickThinSmallGap,

        /// <summary>
        /// Specifies that the border should consist thick line followed by thin line with medium gap between them.
        /// </summary>
        ThickThinMediumGap,

        /// <summary>
        /// Specifies that the border should consist thick line followed by thin line with large gap between them.
        /// </summary>
        ThickThinLargeGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line with small gap between them.
        /// </summary>
        ThinThickSmallGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line with medium gap between them.
        /// </summary>
        ThinThickMediumGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line with large gap between them.
        /// </summary>
        ThinThickLargeGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line followed by thin line with small gap between them.
        /// </summary>
        ThinThickThinSmallGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line followed by thin line with medium gap between them.
        /// </summary>
        ThinThickThinMediumGap,

        /// <summary>
        /// Specifies that the border should consist thin line followed by thick line followed by thin line with large gap between them.
        /// </summary>
        ThinThickThinLargeGap,

        /// <summary>
        /// Specifies that the border should be from one wavy line.
        /// </summary>
        Wave,

        /// <summary>
        /// Specifies that the border should be from double wavy line.
        /// </summary>
        DoubleWave,

        /// <summary>
        /// Specifies that the border should consist line with a series of alternating thin and thick strokes.
        /// </summary>
        DashDotStroked,

        /// <summary>
        /// Specifies that the border should consist three staged gradient lines, which are getting darker toward the content.
        /// </summary>
        ThreeDEmboss,

        /// <summary>
        /// Specifies that the border should consist three staged gradient lines, which are getting darker away from the content.
        /// </summary>
        ThreeDEngrave,

        /// <summary>
        /// Specifies that the border should consist outset set of lines.
        /// </summary>
        Outset,

        /// <summary>
        /// Specifies that the border should consist inset set of lines.
        /// </summary>
        Inset
    }
}
