﻿namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Defines set of indentations which can be applied to paragraph element.
    /// </summary>
    public class ParagraphIndentation
    {
        private readonly ParagraphProperties paragraphProperties;

        /// <summary>
        /// Initializes a new instance of the <see cref="ParagraphIndentation" /> class.
        /// </summary>
        /// <param name="paragraphProperties">The paragraph properties.</param>
        public ParagraphIndentation(ParagraphProperties paragraphProperties)
        {
            this.paragraphProperties = paragraphProperties;
        }

        /// <summary>
        /// Gets or sets a value indicating the additional indentation which shall be applied to the first line of the paragraph.
        /// The value is in device independent pixels (1/96 inch).
        /// </summary>
        /// <value>The first line indent.</value>
        public double FirstLineIndent
        {
            get
            {
                return this.paragraphProperties.FirstLineIndent.GetActualValue().Value;
            }
            set
            {
                this.paragraphProperties.FirstLineIndent.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value which shall be removed from the indentation of the first line of the paragraph, 
        /// by moving the indentation on the first line back towards the beginning of the direction of text flow.
        /// The value is in device independent pixels (1/96 inch).
        /// </summary>
        /// <value>The hanging indent.</value>
        public double HangingIndent
        {
            get
            {
                return this.paragraphProperties.HangingIndent.GetActualValue().Value;
            }
            set
            {
                this.paragraphProperties.HangingIndent.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the indentation which shall be applied to the left side of the whole paragraph.
        /// The value is in device independent pixels (1/96 inch).
        /// </summary>
        /// <value>The left indent.</value>
        public double LeftIndent
        {
            get
            {
                return this.paragraphProperties.LeftIndent.GetActualValue().Value;
            }
            set
            {
                this.paragraphProperties.LeftIndent.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the indentation which shall be applied to the right side of the whole paragraph.
        /// The value is in device independent pixels (1/96 inch).
        /// </summary>
        /// <value>The left indent.</value>
        public double RightIndent
        {
            get
            {
                return this.paragraphProperties.RightIndent.GetActualValue().Value;
            }
            set
            {
                this.paragraphProperties.RightIndent.LocalValue = value;
            }
        }
    }
}
