﻿#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Specifies the shading applied to a document element.
    /// </summary>
    public class Shading
    {
        private readonly IPropertiesWithShading properties;

        /// <summary>
        /// Initializes a new instance of the <see cref="Shading" /> class.
        /// </summary>
        /// <param name="properties">Properties implementing IPropertiesWithShading interface.</param>
        public Shading(IPropertiesWithShading properties)
        {
            this.properties = properties;
        }

        /// <summary>
        /// Gets or sets the color of the background.
        /// </summary>
        /// <value>The color of the background.</value>
        public ThemableColor BackgroundColor
        {
            get
            {
                return this.properties.BackgroundColor.GetActualValue();
            }
            set
            {
                this.properties.BackgroundColor.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the color for any foreground pattern used in this shading.
        /// </summary>
        /// <value>The color of the pattern.</value>
        public ThemableColor PatternColor
        {
            get
            {
                return this.properties.ShadingPatternColor.GetActualValue();
            }
            set
            {
                this.properties.ShadingPatternColor.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the pattern which shall be used to lay the pattern color over the background color for this shading.
        /// </summary>
        /// <returns>The default value is <c>Clear</c></returns>
        public ShadingPattern Pattern
        {
            get
            {
                return this.properties.ShadingPattern.GetActualValue().Value;
            }
            set
            {
                this.properties.ShadingPattern.LocalValue = value;
            }
        }
    }
}
