﻿using System;

namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// <para> Specifies the components of conditional table formatting. </para>
    /// <para> This enumeration has a <see cref="System.FlagsAttribute">Flags</see> attribute 
    /// that allows a bitwise combination of its member values.</para>
    /// </summary>
    [Flags]
    public enum TableLooks
    {
        /// <summary>
        /// No conditional formatting.
        /// </summary>
        None = 0,

        /// <summary>
        /// First row conditional formatting.
        /// </summary>
        FirstRow = 1,

        /// <summary>
        /// Last row conditional formatting.
        /// </summary>
        LastRow = 2,

        /// <summary>
        /// First column conditional formatting.
        /// </summary>
        FirstColumn = 4,

        /// <summary>
        /// Last column conditional formatting.
        /// </summary>
        LastColumn = 8,

        /// <summary>
        /// Row banding conditional formatting.
        /// </summary>
        BandedRows = 16,

        /// <summary>
        /// Column banding conditional formatting.
        /// </summary>
        BandedColumns = 32
    }
}
