﻿#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Model.Styles
{
    /// <summary>
    /// Defines an underline settings. 
    /// </summary>
    public class Underline
    {
        private readonly CharacterProperties characterProperties;

        /// <summary>
        /// Initializes a new instance of the <see cref="Underline" /> class.
        /// </summary>
        /// <param name="characterProperties">The character properties.</param>
        public Underline(CharacterProperties characterProperties)
        {
            this.characterProperties = characterProperties;
        }

        /// <summary>
        /// Gets or sets the color for the underlining.
        /// </summary>
        /// <value>The color.</value>
        public ThemableColor Color
        {
            get
            {
                return this.characterProperties.UnderlineColor.GetActualValue();
            }
            set
            {
                this.characterProperties.UnderlineColor.LocalValue = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the pattern which shall be used to create the underlining applied beneath the text.
        /// </summary>
        /// <returns>The default value is <c>None</c></returns>
        public UnderlinePattern Pattern
        {
            get
            {
                return this.characterProperties.UnderlinePattern.GetActualValue().Value;
            }
            set
            {
                this.characterProperties.UnderlinePattern.LocalValue = value;
            }
        }
    }
}
