﻿using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Primitives;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Model
{
    public partial class TableCell
    {
        /// <summary>
        /// Gets the Left border property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<TableCellBorders> BordersPropertyDefinition =
            new StylePropertyDefinition<TableCellBorders>("Borders", DocumentDefaultStyleSettings.TableCellBorders, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the background color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> BackgroundColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("BackgroundColor", DocumentDefaultStyleSettings.TableCellBackgroundColor, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the shading pattern color property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ThemableColor> ShadingPatternColorPropertyDefinition =
            new StylePropertyDefinition<ThemableColor>("ShadingPatternColor", DocumentDefaultStyleSettings.TableCellShadingPatternColor, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the shading pattern property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<ShadingPattern?> ShadingPatternPropertyDefinition =
            new StylePropertyDefinition<ShadingPattern?>("ShadingPattern", DocumentDefaultStyleSettings.TableCellShadingPattern, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the padding property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<Padding> PaddingPropertyDefinition =
            new StylePropertyDefinition<Padding>("Padding", DocumentDefaultStyleSettings.TableCellPadding, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the columns span property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<int?> ColumnSpanPropertyDefinition =
            new StylePropertyDefinition<int?>("ColumnSpan", DocumentDefaultStyleSettings.ColumnSpan, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the row span property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<int?> RowSpanPropertyDefinition =
            new StylePropertyDefinition<int?>("RowSpan", DocumentDefaultStyleSettings.RowSpan, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the ignore cell marker in row height calculation property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> IgnoreCellMarkerInRowHeightCalculationPropertyDefinition =
            new StylePropertyDefinition<bool?>(
                "IgnoreCellMarkerInRowHeightCalculation", DocumentDefaultStyleSettings.IgnoreCellMarkerInRowHeightCalculation, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the can wrap content property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<bool?> CanWrapContentPropertyDefinition =
            new StylePropertyDefinition<bool?>("CanWrapContent", DocumentDefaultStyleSettings.CanWrapContent, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the preferred width property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<TableWidthUnit> PreferredWidthPropertyDefinition =
            new StylePropertyDefinition<TableWidthUnit>(
                "PreferredWidth", DocumentDefaultStyleSettings.TableCellPreferredWidth, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the vertical alignment property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<VerticalAlignment?> VerticalAlignmentPropertyDefinition =
            new StylePropertyDefinition<VerticalAlignment?>(
                "VerticalAlignment", DocumentDefaultStyleSettings.TableCellVerticalAlignment, StylePropertyType.TableCell);

        /// <summary>
        /// Gets the text direction property definition.
        /// </summary>
        public static readonly StylePropertyDefinition<TextDirection> TextDirectionPropertyDefinition =
          new StylePropertyDefinition<TextDirection>(
              "TextDirection", DocumentDefaultStyleSettings.TableCellTextDirection, StylePropertyType.TableCell);

        static TableCell()
        {
            ColumnSpanPropertyDefinition.Validation.AddRule(new ValidationRule<int>(value => value > 0));
            RowSpanPropertyDefinition.Validation.AddRule(new ValidationRule<int>(value => value > 0));
        }
    }
}
