﻿using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Model.Watermarks
{
    /// <summary>
    /// Specifies image settings for the <see cref="Watermark"/> class.
    /// </summary>
    public class ImageWatermarkSettings : WatermarkSettingsBase
    {
        private ImageSource imageSource;

        /// <summary>
        /// Gets or sets the source of the image.
        /// </summary>
        /// <value>The source of the image.</value>
        public ImageSource ImageSource
        {
            get
            {
                return this.imageSource;
            }
            set
            {
                if (this.imageSource != value)
                {
                    this.ReleaseImageSourceFromDocument();

                    this.imageSource = value;

                    this.RegisterImageSourceInDocument();
                }
            }
        }

        /// <summary>
        /// Creates a deep copy of this element.
        /// </summary>
        /// <returns>A clone of the settings.</returns>
        public ImageWatermarkSettings Clone()
        {
            ImageWatermarkSettings clonedSettings = new ImageWatermarkSettings();
            clonedSettings.Angle = this.Angle;
            clonedSettings.Width = this.Width;
            clonedSettings.Height = this.Height;
            clonedSettings.ImageSource = this.ImageSource.Clone();

            return clonedSettings;
        }

        internal void ReleaseImageSourceFromDocument()
        {
            if (this.imageSource != null && this.Watermark != null && this.Watermark.Document != null)
            {
                this.Watermark.Document.Resources.ReleaseResource(this.imageSource);
            }
        }

        internal void RegisterImageSourceInDocument()
        {
            if (this.imageSource != null && this.Watermark != null && this.Watermark.Document != null)
            {
                this.Watermark.Document.Resources.RegisterResource(this.imageSource);
            }
        }
    }
}