﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Watermarks
{
    /// <summary>
    /// Specifies text settings for the <see cref="Watermark"/> class.
    /// </summary>
    public class TextWatermarkSettings : WatermarkSettingsBase
    {
        private const int DefaultOpacity = 1;

        private double opacity;
        private FontFamily fontFamily;

        /// <summary>
        /// Initializes a new instance of the <see cref="TextWatermarkSettings" /> class.
        /// </summary>
        public TextWatermarkSettings()
        {
            this.Opacity = DefaultOpacity;
            this.FontFamily = DocumentDefaultStyleSettings.FontFamily.LocalValue;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text { get; set; }

        /// <summary>
        /// Gets or sets the font family.
        /// </summary>
        /// <value>The font family.</value>
        public FontFamily FontFamily
        {
            get
            {
                return this.fontFamily;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");
                this.fontFamily = value;
            }
        }

        /// <summary>
        /// Gets or sets the foreground color of the text.
        /// </summary>
        /// <value>The the foreground color.</value>
        public Color ForegroundColor { get; set; }

        /// <summary>
        /// Gets or sets the opacity of the text.
        /// </summary>
        /// <value>The opacity is a double number between 0 and 1.</value>
        public double Opacity
        {
            get
            {
                return this.opacity;
            }
            set
            {
                Guard.ThrowExceptionIfOutOfRange(0, DefaultOpacity, value, "value");
                this.opacity = value;
            }
        }

        /// <summary>
        /// Creates a deep copy of this element.
        /// </summary>
        /// <returns>A clone of the settings.</returns>
        public TextWatermarkSettings Clone()
        {
            TextWatermarkSettings clonedSettings = new TextWatermarkSettings();
            clonedSettings.Text = this.Text;
            clonedSettings.FontFamily = this.FontFamily;
            clonedSettings.ForegroundColor = this.ForegroundColor;
            clonedSettings.Opacity = this.Opacity;
            clonedSettings.Angle = this.Angle;
            clonedSettings.Width = this.Width;
            clonedSettings.Height = this.Height;

            return clonedSettings;
        }
    }
}