﻿using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.Model.Watermarks
{
    /// <summary>
    /// Represents a base class for the watermark settings classes.
    /// </summary>
    public class WatermarkSettingsBase
    {
        private Watermark watermark;

        private double width;
        private double height;
        private double angle;

        /// <summary>
        /// Gets or sets the watermark associated with the settings.
        /// </summary>
        /// <value>The watermark.</value>
        public Watermark Watermark
        {
            get
            {
                return this.watermark;
            }
            internal set
            {
                Guard.ThrowExceptionIfNotNull(this.watermark, "watermark");
                this.watermark = value;
            }
        }

        /// <summary>
        /// Gets or sets the width of the watermark element.
        /// </summary>
        /// <value>The width.</value>
        public double Width
        {
            get
            {
                return this.width;
            }
            set
            {
                Guard.ThrowExceptionIfLessThan(0, value, "value");

                this.width = value;
            }
        }

        /// <summary>
        /// Gets or sets the height of the watermark element.
        /// </summary>
        /// <value>The height.</value>
        public double Height
        {
            get
            {
                return this.height;
            }
            set
            {
                Guard.ThrowExceptionIfLessThan(0, value, "value");

                this.height = value;
            }
        }

        /// <summary>
        /// Gets or sets the angle of the watermark element towards the horizontal direction.
        /// </summary>
        /// <value>The angle.</value>
        public double Angle
        {
            get
            {
                return this.angle;
            }
            set
            {
                this.angle = value;
            }
        }
    }
}
