﻿using System;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.TextSearch.Core
{
    [System.Diagnostics.DebuggerDisplay("[{Start} - {End}]")]
    internal class Interval<T> : IComparable<T>
        where T : IComparable<T>
    {
        private readonly T start;
        private readonly T end;

        public Interval(T start, T end)
        {
            Guard.ThrowExceptionIfTrue(start.CompareTo(end) == 1, "Interval's end cannot be less than its start.");

            this.start = start;
            this.end = end;
        }

        public T Start
        {
            get
            {
                return this.start;
            }
        }

        public T End
        {
            get
            {
                return this.end;
            }
        }

        public bool Contains(T value)
        {
            // Note: If this object should be used in other structure, expose options whether the start and end should be included/exluded when comparing.
            return this.Start.CompareTo(value) <= 0 && this.End.CompareTo(value) == 1;
        }

        public int CompareTo(T other)
        {
            return this.Start.CompareTo(other);
        }
    }
}