﻿using System.Text;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.TextSearch.Core;

namespace Telerik.Windows.Documents.Flow.TextSearch
{
    internal class DocumentTextInfo
    {
        private const char UnitSeparatorChar = '\u001f';

        private readonly RadFlowDocument document;
        private readonly IntervalMap<int, Run> textIntervalToRunMap;
        private readonly string documentAsText;

        public DocumentTextInfo(RadFlowDocument document)
        {
            this.document = document;

            this.textIntervalToRunMap = new IntervalMap<int, Run>();
            this.documentAsText = this.BuildSearchableMappedText();
        }

        public string Text
        {
            get
            {
                return this.documentAsText;
            }
        }

        public bool TryGetRun(int textPositionIndex, out Run run)
        {
            // The run is null is in case match index is matching with the paragraph in the document text.
            run = this.textIntervalToRunMap.GetValueFromIntervalPoint(textPositionIndex);
            return run != null;
        }

        public Interval<int> GetLastRunInterval()
        {
            return this.textIntervalToRunMap.LastIntervalCached;
        }

        private string BuildSearchableMappedText()
        {
            StringBuilder builder = new StringBuilder();

            Paragraph lastParentParagraph = null;
            int runStartIndexInDocumentText = 0;
            int runEndIndexInDocumentText = 0;
            foreach (InlineBase inline in this.document.EnumerateChildrenOfType<InlineBase>())
            {
                Run run = inline as Run;
                if (run != null)
                {
                    if (lastParentParagraph != null && run.Paragraph != lastParentParagraph)
                    {
                        builder.Append(UnitSeparatorChar);
                        runStartIndexInDocumentText++;
                    }

                    builder.Append(run.Text);
                    runEndIndexInDocumentText = runStartIndexInDocumentText + run.Text.Length;

                    this.textIntervalToRunMap.Add(runStartIndexInDocumentText, runEndIndexInDocumentText, run);
                }
                else
                {
                    builder.Append(UnitSeparatorChar);
                    runEndIndexInDocumentText++;
                }

                runStartIndexInDocumentText = runEndIndexInDocumentText;
                lastParentParagraph = inline.Paragraph;
            }

            return builder.ToString();
        }
    }
}