﻿using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.TextSearch
{
    /// <summary>
    /// Represents an abstraction of a replace handler.
    /// </summary>
    internal abstract class ReplaceHandlerBase
    {
        /// <summary>
        /// Gets or sets the regex by which the results are matched.
        /// </summary>
        /// <value>The regex.</value>
        protected Regex Regex { get; private set; }

        /// <summary>
        /// Called before the replacement of all matches in a <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument" /> begins.
        /// </summary>
        /// <param name="regex">The regex by which the match is found.</param>
        /// <param name="replaceText">The replace text.</param>
        internal void BeginReplace(Regex regex, string replaceText)
        {
            Guard.ThrowExceptionIfNull(regex, "regex");

            this.Regex = regex;

            this.BeginReplaceCore(replaceText);
        }

        /// <summary>
        /// Called after the replacement of all matches in a <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument" /> ends.
        /// </summary>
        internal void EndReplace()
        {
            this.EndReplaceCore();
        }

        /// <summary>
        /// Replaces matched text in a run.
        /// </summary>
        /// <param name="runTextMatch">The match info in a text of a run.</param>
        protected internal abstract void ReplaceMatch(RunTextMatch runTextMatch);

        /// <summary>
        /// Called before the replacement of all matches in a <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument" /> begins.
        /// </summary>
        /// <param name="replaceText">The replace text.</param>
        protected virtual void BeginReplaceCore(string replaceText)
        {
        }

        /// <summary>
        /// Called after the replacement of all matches in a <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument" /> ends.
        /// </summary>
        protected virtual void EndReplaceCore()
        {
        }
    }
}