﻿using System;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.TextSearch
{
    internal static class RunMatchHelper
    {
        /// <summary>
        /// Gets the matched index in run.
        /// </summary>
        /// <param name="matchedRun">The matched run.</param>
        /// <param name="searchedText">The searched text.</param>
        /// <param name="startIndex">The index where the matching index searching shall begin.</param>
        /// <returns>The index of the run text where the searched text starts matching.</returns>
        public static int GetMatchedIndexInRun(Run matchedRun, string searchedText, int startIndex = 0)
        {
            string searchedTextPartial = searchedText;
            startIndex = Math.Max(0, startIndex);
            int result = matchedRun.Text.IndexOf(searchedTextPartial, startIndex);

            if (result == -1)
            {
                searchedTextPartial = searchedTextPartial.Substring(0, searchedTextPartial.Length - 1);
                while (result == -1 && !string.IsNullOrEmpty(searchedTextPartial))
                {
                    result = matchedRun.Text.IndexOf(searchedTextPartial, startIndex);
                    searchedTextPartial = searchedTextPartial.Substring(0, searchedTextPartial.Length - 1);
                }
            }

            if (result == -1)
            {
                return GetMatchedIndexInRun(matchedRun, searchedText);
            }

            return result;
        }

        /// <summary>
        /// Gets the matched text length in run.
        /// </summary>
        /// <param name="matchedRun">The matched run.</param>
        /// <param name="searchedText">The searched text.</param>
        /// <param name="matchIndex">Start index of the match of the searched text in the run.</param>
        /// <returns>The length of matched text in the run.</returns>
        public static int GetMatchedTextLengthInRun(Run matchedRun, string searchedText, int matchIndex)
        {
            return Math.Min(searchedText.Length, matchedRun.Text.Length - matchIndex);
        }
    }
}