﻿using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.TextSearch
{
    /// <summary>
    /// Represents a class containing information which matches given criteria.
    /// </summary>
    [System.Diagnostics.DebuggerDisplay("Text = {Run.Text}, StartIndex = {StartIndex}, Length = {Length}, FullMatchText = {FullMatchText}")]
    internal class RunTextMatch
    {
        private readonly Run run;
        private readonly int startIndex;
        private readonly int length;
        private readonly string fullMatchText;

        internal RunTextMatch(Run run, int startIndex, int length, string fullMatchText)
        {
            Guard.ThrowExceptionIfNull(run, "run");
            Guard.ThrowExceptionIfLessThan(0, startIndex, "startIndex");

            this.run = run;
            this.startIndex = startIndex;
            this.length = length;
            this.fullMatchText = fullMatchText;
        }

        /// <summary>
        /// Gets the run containing part or full matched text.
        /// </summary>
        /// <value>The run.</value>
        public Run Run
        {
            get
            {
                return this.run;
            }
        }

        /// <summary>
        /// Gets the index where the matched text in the run starts.
        /// </summary>
        /// <value>The start index of the match.</value>
        public int StartIndex
        {
            get
            {
                return this.startIndex;
            }
        }

        /// <summary>
        /// Gets the length of the matched text in the run.
        /// </summary>
        /// <value>The length of the match.</value>
        public int Length
        {
            get
            {
                return this.length;
            }
        }

        /// <summary>
        /// Gets the text which is matched during the search.
        /// </summary>
        /// <value>The full match text.</value>
        public string FullMatchText
        {
            get
            {
                return this.fullMatchText;
            }
        }
    }
}