﻿using System.Collections.Generic;
using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Flow.TextSearch
{
    /// <summary>
    /// Represents a service for replacing a text in found matches in a <see cref="Telerik.Windows.Documents.Flow.Model.RadFlowDocument"/>.
    /// </summary>
    internal class ReplaceTextService
    {
        private readonly ReplaceHandlerBase replaceHandler;

        public ReplaceTextService(ReplaceHandlerBase replaceHandler)
        {
            Guard.ThrowExceptionIfNull(replaceHandler, "replaceHandler");

            this.replaceHandler = replaceHandler;
        }

        public void Replace(IEnumerable<RunTextMatch> matches, Regex regex, string replaceText)
        {
            Guard.ThrowExceptionIfNull(matches, "matches");

            this.replaceHandler.BeginReplace(regex, replaceText);

            foreach (RunTextMatch match in matches)
            {
                this.replaceHandler.ReplaceMatch(match);
            }

            this.replaceHandler.EndReplace();
        }
    }
}