﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration
{
    [TestClass]
    public class DocumentElementPropertiesTests
    {
        [TestMethod]
        public void CharacterProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            Style characterStyle = new Style("test", StyleType.Character);
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(CharacterProperties)).ToList();
            CharacterProperties properties = characterStyle.CharacterProperties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void ParagraphProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            Style paragraphStyle = new Style("test", StyleType.Paragraph);
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(ParagraphProperties)).ToList();
            ParagraphProperties properties = paragraphStyle.ParagraphProperties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void TableProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            Style tableStyle = new Style("test", StyleType.Table);
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(TableProperties)).ToList();
            TableProperties properties = tableStyle.TableProperties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void TableRowProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            Style tableStyle = new Style("test", StyleType.Table);
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(TableRowProperties)).ToList();
            TableRowProperties properties = tableStyle.TableRowProperties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void TableCellProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            Style tableStyle = new Style("test", StyleType.Table);
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(TableCellProperties)).ToList();
            TableCellProperties properties = tableStyle.TableCellProperties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void SectionProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(SectionProperties)).ToList();
            SectionProperties properties = new Section(new RadFlowDocument()).Properties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        [TestMethod]
        public void DocumentProperties_StyleProperties_ReturnsAllStyleProperties()
        {
            // Arrange
            List<string> stylePropertiesExpected = GetStylePropetiesNames(typeof(DocumentProperties)).ToList();
            DocumentProperties properties = new RadFlowDocument().Properties;

            // Act
            List<string> stylePropertiesActual = properties.StyleProperties.Select(property => property.PropertyDefinition.PropertyName).ToList();

            // Assert
            CollectionAssert.AreEquivalent(stylePropertiesExpected, stylePropertiesActual);
        }

        private static IEnumerable<string> GetStylePropetiesNames(Type type)
        {
            return type.GetProperties()
                .Where(property => IsStyleProperty(property))
                .Select(property => property.Name)
                .Concat(Enumerable.Repeat("StyleID", 1));
        }

        private static bool IsStyleProperty(PropertyInfo propertyInfo)
        {
            return
                propertyInfo.PropertyType.IsGenericType &&
                propertyInfo.PropertyType.GetGenericTypeDefinition() == typeof(IStyleProperty<>);
        }
    }
}
