﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class FieldsTests : DocumentFlowVisualBaseTest
    {
        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_InvalidHyperlink_Docx()
        {
            RadFlowDocument document = CreateInvalidUriHyperlinkDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_InvalidHyperlink_Pdf()
        {
            RadFlowDocument document = CreateInvalidUriHyperlinkDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_Hyperlink_Docx()
        {
            RadFlowDocument document = CreateValidUriHyperlinkDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_Hyperlink_Pdf()
        {
            RadFlowDocument document = CreateValidUriHyperlinkDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_Field_Docx()
        {
            RadFlowDocument document = CreateFieldDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_Field_Pdf()
        {
            RadFlowDocument document = CreateFieldDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_DocumentVariableField_Docx()
        {
            RadFlowDocument document = CreateDocumentVariableFieldDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_DocumentVariableField_Pdf()
        {
            RadFlowDocument document = CreateDocumentVariableFieldDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFields_Docx()
        {
            RadFlowDocument document = CreatePageFieldsDocument();
            document.UpdateFields();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFields_Pdf()
        {
            RadFlowDocument document = CreatePageFieldsDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFieldsStartNumber_Docx()
        {
            RadFlowDocument document = CreatePageFieldsPageNumberingSettingsDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFieldsPageNumberingSettings_Pdf()
        {
            RadFlowDocument document = CreatePageFieldsPageNumberingSettingsDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFieldsPageNumberingSettings()
        {
            RadFlowDocument document = CreatePageFieldsPageNumberingSettingsDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void Test_PageFieldsPageNumberingSettings_All()
        {
            RadFlowDocument document = CreatePageFieldsInSectionDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        private static RadFlowDocument CreateInvalidUriHyperlinkDocument()
        {
            return CreateHyperlinkDocument("telerik.com");
        }

        private static RadFlowDocument CreateValidUriHyperlinkDocument()
        {
            return CreateHyperlinkDocument("http://www.telerik.com");
        }

        private static RadFlowDocument CreateHyperlinkDocument(string uri)
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("telerik", uri, false, "Telerik Site");
            editor.InsertText(" after");

            return editor.Document;
        }

        private static RadFlowDocument CreateFieldDocument()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            return editor.Document;
        }

        private static RadFlowDocument CreateDocumentVariableFieldDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "Kamen Velikov");

            editor.InsertField(" docvariable name ", "t").UpdateField();

            return document;
        }

        private static RadFlowDocument CreatePageFieldsDocument()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            var field = editor.InsertField(" PAGE", "test");
            field.IsDirty = true;

            editor.InsertBreak(BreakType.PageBreak);

            field = editor.InsertField(" PAGE", "test");
            field.IsDirty = true;

            return editor.Document;
        }

        private static RadFlowDocument CreatePageFieldsInSectionDocument()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            var field = editor.InsertField(" PAGE", "test");
            field.IsDirty = true;

            editor.InsertBreak(BreakType.PageBreak);

            field = editor.InsertField(" PAGE", "test");
            field.IsDirty = true;

            return editor.Document;
        }

        private static RadFlowDocument CreatePageFieldsPageNumberingSettingsDocument()
        {
            RadFlowDocument document = CreatePageFieldsDocument();
            document.Sections.First().PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperRoman;
            document.Sections.First().PageNumberingSettings.StartingPageNumber = 3;

            return document;
        }

        protected override void AssertVisual(RadFlowDocument document, int pageDown = 1)
        {
#if VISUAL
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION, PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), pageDown);
            //this.AssertAreVisualyEqualDocx(document, UIInteractionHelper.PageDown(), 1, UIInteractionHelper.KeyPress("Left"), UIInteractionHelper.KeyPress("Enter"));
            //this.AssertAreVisualyEqualRtf(document, UIInteractionHelper.PageDown(), 1, UIInteractionHelper.KeyPress("Left"), UIInteractionHelper.KeyPress("Enter"));
            //this.AssertAreVisualyEqualPdf(document, UIInteractionHelper.PageDown());
            //this.AssertAreVisualyEqualHtml(document);
#endif
        }
    }
}
