﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Spreadsheet.Model;
using System.Windows.Media;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
//using System.Windows;
//using System.Windows.Documents;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class FlowParagraphProperties : DocumentFlowVisualBaseTest
    {

        protected const double FontSize8 = 10.67;
        protected const double FontSize12 = 16;
        protected const double FontSize18 = 24;

        // Default paragraph settings
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_AddNewParagraphWithDefaultSettings_Docx()
        {
            RadFlowDocument document = AddNewParagraphWithDefaultSettingsTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_AddNewParagraphWithDefaultSettings_Pdf()
        {
            RadFlowDocument document = AddNewParagraphWithDefaultSettingsTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // SpacingBefore property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeAppliedToFirstParagraphOnly_Docx()
        {
            RadFlowDocument document = SpacingBeforeAppliedToFirstParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeAppliedToFirstParagraphOnly_Pdf()
        {
            RadFlowDocument document = SpacingBeforeAppliedToFirstParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeAppliedToSecondParagraphOnly_Docx()
        {
            RadFlowDocument document = SpacingBeforeAppliedOnSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeAppliedToSecondParagraphOnly_Pdf()
        {
            RadFlowDocument document = SpacingBeforeAppliedOnSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeIsAppliedToEachParagraph_Docx()
        {
            RadFlowDocument document = SpacingBeforeIsAppliedToEachParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingBeforeIsAppliedToEachParagraph_Pdf()
        {
            RadFlowDocument document = SpacingBeforeIsAppliedToEachParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // SpacingAfter property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToFirstParagrapOnly_Docx()
        {
            RadFlowDocument document = SpacingAfterAppliedOnFirstParagrapOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToFirstParagrapOnly_Pdf()
        {
            RadFlowDocument document = SpacingAfterAppliedOnFirstParagrapOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToSecondParagrapOnly_Docx()
        {
            RadFlowDocument document = SpacingAfterAppliedOnSecondParagrapOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToSecondParagrapOnly_Pdf()
        {
            RadFlowDocument document = SpacingAfterAppliedOnSecondParagrapOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToEachParagraph_Docx()
        {
            RadFlowDocument document = SpacingAfterAppliedToEachParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterAppliedToEachParagraph_Pdf()
        {
            RadFlowDocument document = SpacingAfterAppliedToEachParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // LineSpacing property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToAutoAppliedToAParagraph_Docx()
        {
            RadFlowDocument document = LineSpacingSetToAutoTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToAutoAppliedToAParagraph_Pdf()
        {
            RadFlowDocument document = LineSpacingSetToAutoTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToAtLeastAppliedToAParagraph_Docx()
        {
            RadFlowDocument document = LineSpacingSetToAtLeastDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToAtLeastAppliedToAParagraph_Pdf()
        {
            RadFlowDocument document = LineSpacingSetToAtLeastDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToExactAppliedToAParagraph_Docx()
        {
            RadFlowDocument document = LineSpacingSetToExactDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LineSpacingSetToExactAppliedToAParagraph_Pdf()
        {
            RadFlowDocument document = LineSpacingSetToExactDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // LeftIndent property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToFirstParagraph_Docx()
        {
            RadFlowDocument document = LeftIndentAppliedToFirstParagraphOnlyTextDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToFirstParagraph_Pdf()
        {
            RadFlowDocument document = LeftIndentAppliedToFirstParagraphOnlyTextDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToSecondParagraph_Docx()
        {
            RadFlowDocument document = LeftIndentAppliedToSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToSecondParagraph_Pdf()
        {
            RadFlowDocument document = LeftIndentAppliedToSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToAllParagraphsUntilValueIsChanged_Docx()
        {
            RadFlowDocument document = LeftIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentOnlyToAllParagraphsUntilValueIsChanged_Pdf()
        {
            RadFlowDocument document = LeftIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // RightIndent property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToFirstParagraph_Docx()
        {
            RadFlowDocument document = RightIndentAppliedToFirstParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToFirstParagraph_Pdf()
        {
            RadFlowDocument document = RightIndentAppliedToFirstParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToSecondParagraph_Docx()
        {
            RadFlowDocument document = RightIndentAppliedToSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToSecondParagraph_Pdf()
        {
            RadFlowDocument document = RightIndentAppliedToSecondParagraphOnlyTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToAllParagraphsUntilValueIsChanged_Docx()
        {
            RadFlowDocument document = RightIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_RightIndentOnlyToAllParagraphsUntilValueIsChanged_Pdf()
        {
            RadFlowDocument document = RightIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // LeftIndent and RightIndent properties combined
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentAndRightIndentSetToAllParagraphs_Docx()
        {
            RadFlowDocument document = LeftIndentAndRightIndentSetToAllParagraphsTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_LeftIndentAndRightIndentSetToAllParagraphs_Pdf()
        {
            RadFlowDocument document = LeftIndentAndRightIndentSetToAllParagraphsTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // TextAlignment property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ParagraphsWithDifferentHorizontalAlignments_Docx()
        {
            RadFlowDocument document = ParagraphsWithDifferentHorizontalAlignmentsTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ParagraphsWithDifferentHorizontalAlignments_Pdf()
        {
            RadFlowDocument document = ParagraphsWithDifferentHorizontalAlignmentsTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // FirstLineIndent property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FirstLineIndentAppliedToFirstParagraph_Docx()
        {
            RadFlowDocument document = FirstLineIndentAppliedToFirstParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FirstLineIndentAppliedToFirstParagraph_Pdf()
        {
            RadFlowDocument document = FirstLineIndentAppliedToFirstParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FirstLineIndentAppliedToSecondParagraph_Docx()
        {
            RadFlowDocument document = FirstLineIndentAppliedToSecondParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FirstLineIndentAppliedToSecondParagraph_Pdf()
        {
            RadFlowDocument document = FirstLineIndentAppliedToSecondParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // HangingIndent property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_HangingIndentAppliedToFirstParagraph_Docx()
        {
            RadFlowDocument document = HangingIndentAppliedToFirstParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_HangingIndentAppliedToFirstParagraph_Pdf()
        {
            RadFlowDocument document = HangingIndentAppliedToFirstParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_HangingIndentAppliedToSecondParagraph_Docx()
        {
            RadFlowDocument document = HangingIndentAppliedToSecondParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        [Description("BUG 155223  TFS 234660")]
        public void Test_HangingIndentAppliedToSecondParagraph_Pdf()
        {
            RadFlowDocument document = HangingIndentAppliedToSecondParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // FlowDirection property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_AddingRtlTextToAParagraph_Docx()
        {
            RadFlowDocument document = RtlTextTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_AddingRtlTextToAParagraph_Pdf()
        {
            RadFlowDocument document = RtlTextTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // BackgroundColor property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_BackgroundColorApplied_Docx()
        {
            RadFlowDocument document = BackgroundColorAppliedDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_BackgroundColorApplied_Pdf()
        {
            RadFlowDocument document = BackgroundColorAppliedDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // ShadingPattern and ShadingPatternColor properties
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyShadingPatternToAParagraph_Docx()
        {
            RadFlowDocument document = ShadingPatternAppliedDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_ApplyShadingPatternToAParagraph_Pdf()
        {
            RadFlowDocument document = ShadingPatternAppliedDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // PageBreakBefore property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_PageBreakBeforeParagraph_Docx()
        {
            RadFlowDocument document = ApplyPageBreakBeforeOnParagraph();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_PageBreakBeforeParagraph_Pdf()
        {
            RadFlowDocument document = ApplyPageBreakBeforeOnParagraph();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        // Border property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SetBorderToAParagraph_Docx()
        {
            RadFlowDocument document = ParagraphWithABorderDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_SetBorderToAParagraph_Pdf()
        {
            RadFlowDocument document = ParagraphWithABorderDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_SetBorderToAParagraphThroughEditor_Docx()
        {
            RadFlowDocument document = ParagraphWithABorderThroughEditorDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // FontSize property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontSizeOnAParagraph_Docx()
        {
            RadFlowDocument document = DifferentFontSizeAppliedDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontSizeOnAParagraph_Pdf()
        {
            RadFlowDocument document = DifferentFontSizeAppliedDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // FontFamily property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontFamilyOnAParagraph_Docx()
        {
            RadFlowDocument document = DifferentFontFamiliesAppliedDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontFamilyOnAParagraph_Pdf()
        {
            RadFlowDocument document = DifferentFontFamiliesAppliedDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // FontWeight property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontWeightOnAParagraph_Docx()
        {
            RadFlowDocument document = DifferentFontWeightAppliedDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyFontWeightOnAParagraph_Pdf()
        {
            RadFlowDocument document = DifferentFontWeightAppliedDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        //ForegroundColor and HighlightColor properties
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyForegroundPropertyToAParagraph_Docx()
        {
            RadFlowDocument document = ForegroundColorAndHighlightColorTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyForegroundPropertyToAParagraph_Pdf()
        {
            RadFlowDocument document = ForegroundColorAndHighlightColorTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // UnderlinePattern and UnderlineColor properties
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyUnderlinePatternAndUnderlineColor_Docx()
        {
            RadFlowDocument document = UnderlinePatterAndUnderlineColorDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyUnderlinePatternAndUnderlineColor_Pdf()
        {
            RadFlowDocument document = UnderlinePatterAndUnderlineColorDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // ShadingPattern and ShadingColor properties
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyShadingPatternAndShadingColor_Docx()
        {
            RadFlowDocument document = ShadingPatternAndShadingColorDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_ApplyShadingPatternAndShadingColor_Pdf()
        {
            RadFlowDocument document = ShadingPatternAndShadingColorDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // StrikeThrough property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ApplyStrikeThrough_Docx()
        {
            RadFlowDocument document = StrikeThroughDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_ApplyStrikeThrough_Pdf()
        {
            RadFlowDocument document = StrikeThroughDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // BackgroundColor property
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_HighlightColor_Docx()
        {
            RadFlowDocument document = HighlightColorDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_HighlightColor_Pdf()
        {
            RadFlowDocument document = HighlightColorDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // BaselineAlignment
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_BaseLinePropery_Docx()
        {
            RadFlowDocument document = BaselineAlignmentDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_BaseLinePropery_Pdf()
        {
            RadFlowDocument document = BaselineAlignmentDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // All paragraph properties applied
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_AllParagraphFormattingAppliedToAParagraph_Docx()
        {
            RadFlowDocument document = AllParagraphFormattingAppliedTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_AllParagraphFormattingAppliedToAParagraph_Pdf()
        {
            RadFlowDocument document = AllParagraphFormattingAppliedTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_AddingChineseTextToAParagraph_Docx()
        {
            RadFlowDocument document = ChineseTextAddedToAParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        //[TestMethod]
        public void Test_AddingChineseTextToAParagraph_Pdf()
        {
            RadFlowDocument document = ChineseTextAddedToAParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // SpacingAfter, LineSpacing and FirstLineIndent applied combined
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterLineSpacingAndFirstLineIndentAppliedCombined_Docx()
        {
            RadFlowDocument document = SpacingAfterLineSpacingAndFirstLineIndentAppliedCombinedTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_SpacingAfterLineSpacingAndFirstLineIndentAppliedCombined_Pdf()
        {
            RadFlowDocument document = SpacingAfterLineSpacingAndFirstLineIndentAppliedCombinedTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FontSizeSetChangedForFirstWordInParagraph_Docx()
        {
            RadFlowDocument document = FontSizeSetChangedForFirstWordInParagraphTestDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_FontSizeSetChangedForFirstWordInParagraph_Pdf()
        {
            RadFlowDocument document = FontSizeSetChangedForFirstWordInParagraphTestDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ParagraphWithBoldAndItalicText()
        {
            RadFlowDocument document = new RadFlowDocument();
            string[] standardFontFamilies = new string[] { "Arial", "Arial Black", "Calibri", "Comic Sans MS", "Courier New", "Georgia", "Lucida Sans Unicode", "Times New Roman", "Trebuchet MS", "Verdana" };

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            for (int i = 0; i < standardFontFamilies.Length; i++)
            {
                editor.CharacterFormatting.FontWeight.LocalValue = System.Windows.FontWeights.Bold;
                editor.CharacterFormatting.FontStyle.LocalValue = System.Windows.FontStyles.Italic;
                editor.CharacterFormatting.FontFamily.LocalValue = new ThemableFontFamily(standardFontFamilies[i]);
                editor.ParagraphFormatting.LeftIndent.LocalValue = 60;
                editor.InsertLine("Text in: " + standardFontFamilies[i]);
            }

            // var keysCombination = KeyBoard.KeysFromString("Ctrl + OemMinus");
            //this. AssertAreVisualyEqual(document, (()=> this.Desktop.KeyBoard.KeyPress(keysCombination, 50,5)));
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Test_ParagraphWithBoldAndItalicText_Pdf()
        {
            RadFlowDocument document = new RadFlowDocument();
            string[] standardFontFamilies = new string[] { "Arial", "Arial Black", "Calibri", "Comic Sans MS", "Courier New", "Georgia", "Lucida Sans Unicode", "Times New Roman", "Trebuchet MS", "Verdana" };

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            for (int i = 0; i < standardFontFamilies.Length; i++)
            {
                editor.CharacterFormatting.FontWeight.LocalValue = System.Windows.FontWeights.Bold;
                editor.CharacterFormatting.FontStyle.LocalValue = System.Windows.FontStyles.Italic;
                editor.CharacterFormatting.FontFamily.LocalValue = new ThemableFontFamily(standardFontFamilies[i]);
                editor.ParagraphFormatting.LeftIndent.LocalValue = 60;
                editor.InsertLine("Text in: " + standardFontFamilies[i]);
            }

            // var keysCombination = KeyBoard.KeysFromString("Ctrl + OemMinus");
            //this. AssertAreVisualyEqual(document, (()=> this.Desktop.KeyBoard.KeyPress(keysCombination, 50,5)));
            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        private static RadFlowDocument AddNewParagraphWithDefaultSettingsTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingBeforeAppliedToFirstParagraphOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph p1 = section.Blocks.AddParagraph();
            p1.Properties.SpacingBefore.LocalValue = 200;
            p1.Inlines.AddRun(SampleParagraphText);

            Paragraph p2 = section.Blocks.AddParagraph();
            p2.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingBeforeAppliedOnSecondParagraphOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertLine(SampleParagraphText);

            // Set SpacingBefore
            editor.ParagraphFormatting.SpacingBefore.LocalValue = 200;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingBeforeIsAppliedToEachParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.SpacingBefore.LocalValue = 60;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.SpacingBefore.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingAfterAppliedOnFirstParagrapOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.SpacingAfter.LocalValue = 50;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.SpacingAfter.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingAfterAppliedOnSecondParagrapOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.SpacingAfter.LocalValue = 50;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.SpacingAfter.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument SpacingAfterAppliedToEachParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.SpacingAfter.LocalValue = 50;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LineSpacingSetToAutoTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacingType.LocalValue = HeightType.Auto;
            editor.ParagraphFormatting.LineSpacing.LocalValue = 6;
            editor.InsertParagraph();
            editor.CharacterFormatting.HighlightColor.LocalValue = Colors.Blue;
            editor.InsertText("Test ");
            editor.CharacterFormatting.HighlightColor.ClearValue();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacing.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LineSpacingSetToAtLeastDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacingType.LocalValue = HeightType.AtLeast;
            editor.ParagraphFormatting.LineSpacing.LocalValue = 30;
            editor.InsertParagraph();
            editor.CharacterFormatting.HighlightColor.LocalValue = Colors.Blue;
            editor.InsertText("Test ");
            editor.CharacterFormatting.HighlightColor.ClearValue();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacing.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LineSpacingSetToExactDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacingType.LocalValue = HeightType.Exact;
            editor.ParagraphFormatting.LineSpacing.LocalValue = 40;
            editor.InsertParagraph();
            editor.CharacterFormatting.HighlightColor.LocalValue = Colors.Blue;
            editor.InsertText("Test ");
            editor.CharacterFormatting.HighlightColor.ClearValue();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LineSpacing.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LeftIndentAppliedToFirstParagraphOnlyTextDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.LeftIndent.LocalValue = 80;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LeftIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LeftIndentAppliedToSecondParagraphOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LeftIndent.LocalValue = 60;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LeftIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.LeftIndent.LocalValue = 70;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.LeftIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument RightIndentAppliedToFirstParagraphOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.RightIndent.LocalValue = 80;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.RightIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument RightIndentAppliedToSecondParagraphOnlyTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.RightIndent.LocalValue = 90;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument RightIndentAppliedToAllParagraphsUntilValueIsChangedTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.RightIndent.LocalValue = 80;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.RightIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument LeftIndentAndRightIndentSetToAllParagraphsTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.LeftIndent.LocalValue = 50;
            editor.ParagraphFormatting.RightIndent.LocalValue = 50;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument ParagraphsWithDifferentHorizontalAlignmentsTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertParagraph();
            editor.ParagraphFormatting.TextAlignment.LocalValue = Alignment.Justified;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.ParagraphFormatting.TextAlignment.LocalValue = Alignment.Right;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.ParagraphFormatting.TextAlignment.LocalValue = Alignment.Center;
            editor.InsertText(SampleParagraphText);

            editor.InsertParagraph();
            editor.ParagraphFormatting.TextAlignment.LocalValue = Alignment.Left;
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument FirstLineIndentAppliedToFirstParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.FirstLineIndent.LocalValue = 40;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.FirstLineIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument FirstLineIndentAppliedToSecondParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.FirstLineIndent.LocalValue = 40;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument HangingIndentAppliedToFirstParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.HangingIndent.LocalValue = 40;
            editor.ParagraphFormatting.LeftIndent.LocalValue = 40;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.HangingIndent.ClearValue();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument HangingIndentAppliedToSecondParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.HangingIndent.LocalValue = 40;
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument AllParagraphFormattingAppliedTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.BackgroundColor.LocalValue = new ThemableColor(Colors.Aquamarine);
            editor.ParagraphFormatting.SpacingBefore.LocalValue = 50;
            editor.ParagraphFormatting.SpacingAfter.LocalValue = 50;
            editor.ParagraphFormatting.LeftIndent.LocalValue = 50;
            editor.ParagraphFormatting.RightIndent.LocalValue = 50;
            editor.ParagraphFormatting.TextAlignment.LocalValue = Alignment.Center;
            editor.ParagraphFormatting.FirstLineIndent.LocalValue = 40;
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.BackgroundColor.LocalValue = new ThemableColor(Colors.Fuchsia);
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            editor.ParagraphFormatting.ClearLocalValues();
            editor.InsertParagraph();
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument RtlTextTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.FlowDirection.LocalValue = FlowDirection.RightToLeft;
            editor.InsertText(SampleLongStringRtl);

            return document;
        }

        private static RadFlowDocument ChineseTextAddedToAParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertLine(SampleLongStringCh);

            return document;
        }

        private static RadFlowDocument SpacingAfterLineSpacingAndFirstLineIndentAppliedCombinedTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.SpacingAfter.LocalValue = 40;
            editor.ParagraphFormatting.LineSpacing.LocalValue = 3;
            editor.ParagraphFormatting.FirstLineIndent.LocalValue = 30;
            editor.InsertLine(SampleParagraphText);

            editor.InsertParagraph();
            editor.InsertLine(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument BackgroundColorAppliedDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ParagraphFormatting.BackgroundColor.LocalValue = new ThemableColor(Colors.LightBlue);
            editor.InsertText(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument ParagraphWithABorderDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            p.Inlines.AddRun(SampleParagraphText);

            Border border = new Border(5, BorderStyle.Thick, new ThemableColor(Colors.Black));
            p.Properties.Borders.LocalValue = new ParagraphBorders(border);

            return document;
        }

        private static RadFlowDocument ParagraphWithABorderThroughEditorDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Border border = new Border(5, BorderStyle.Thick, new ThemableColor(Colors.Black));

            editor.ParagraphFormatting.Borders.LocalValue = new ParagraphBorders(border);
            editor.InsertLine(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument FontSizeSetChangedForFirstWordInParagraphTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.CharacterFormatting.FontSize.LocalValue = 32;
            editor.InsertText("Test text");
            editor.CharacterFormatting.FontSize.LocalValue = 13;
            editor.InsertLine(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument ShadingPatternAppliedDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph paragraph = section.Blocks.AddParagraph();
            paragraph.Properties.ShadingPattern.LocalValue = ShadingPattern.HorizontalStripe;
            paragraph.Properties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Bisque);
            paragraph.Inlines.AddRun(SampleParagraphText);

            Paragraph paragraph2 = section.Blocks.AddParagraph();
            paragraph2.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument ApplyPageBreakBeforeOnParagraph()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Blocks.AddParagraph();
            p1.Inlines.AddRun("First page");

            Paragraph p2 = section.Blocks.AddParagraph();
            p2.Properties.PageBreakBefore.LocalValue = true;
            p2.Inlines.AddRun("Second page");

            return document;
        }

        private static RadFlowDocument DifferentFontSizeAppliedDocument()
        {
            double[] fontSizes = new double[] { FontSize8, FontSize12, FontSize18 };

            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            // Set FontSize property through the run
            for (int i = 0; i < fontSizes.Length; i++)
            {
                Paragraph p = section.Blocks.AddParagraph();
                Run run = p.Inlines.AddRun(SampleParagraphText);
                run.FontSize = fontSizes[i];
            }

            // Set FontSize property through ParagraphMarkerProperties
            Paragraph p4 = section.Blocks.AddParagraph();
            p4.Properties.ParagraphMarkerProperties.FontSize.LocalValue = fontSizes[2];
            p4.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument DifferentFontFamiliesAppliedDocument()
        {
            string[] fonts = new string[] { "Verdana", "Courier New", "Comic Sans MS" };

            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            // Set FontFamily property through the run
            for (int i = 0; i < fonts.Length; i++)
            {
                Paragraph p = section.Blocks.AddParagraph();
                Run run = p.Inlines.AddRun(SampleParagraphText);
                run.FontFamily = new ThemableFontFamily(fonts[i]);
            }

            // Set FontFamily property through ParagraphMarkerProperties
            Paragraph p4 = section.Blocks.AddParagraph();
            p4.Properties.ParagraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily(fonts[0]);
            p4.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument DifferentFontWeightAppliedDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();

            // Set FontWeight property through the run
            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            run1.FontWeight = System.Windows.FontWeights.Bold;

            Run run2 = p2.Inlines.AddRun(SampleParagraphText);
            run2.FontWeight = System.Windows.FontWeights.Normal;

            // Set FontWeight property through ParagraphMarkerProperties
            p3.Inlines.AddRun(SampleParagraphText);
            p3.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = System.Windows.FontWeights.Bold;

            return document;
        }

        private static RadFlowDocument ForegroundColorAndHighlightColorTestDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();
            Paragraph p4 = section.Blocks.AddParagraph();

            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            Run run2 = p2.Inlines.AddRun(SampleParagraphText);

            // Set ForegroundColor and highlightColor through the run
            run1.ForegroundColor = new ThemableColor(Colors.BlueViolet);
            run2.HighlightColor = Colors.Yellow;

            // Set ForegroundColor and highlightColor through ParagraphMarkerProperties
            p3.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.HotPink);
            p3.Inlines.AddRun(SampleParagraphText);

            p4.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Orange);
            p4.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument UnderlinePatterAndUnderlineColorDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();
            Paragraph p4 = section.Blocks.AddParagraph();

            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            Run run2 = p2.Inlines.AddRun(SampleParagraphText);
            Run run3 = p3.Inlines.AddRun(SampleParagraphText);

            // Set UnderlinePattern and UnderlineColor properties through the run
            run1.Underline.Pattern = UnderlinePattern.Single;
            run1.Underline.Color = new ThemableColor(Colors.Orange);
            run2.Underline.Pattern = UnderlinePattern.Double;
            run2.Underline.Color = new ThemableColor(Colors.BlueViolet);
            run3.Underline.Pattern = UnderlinePattern.None;

            // Set UnderlinePattern and UnderlineColor properties through ParagraphMarkerProperties
            p4.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            p4.Properties.ParagraphMarkerProperties.UnderlineColor.LocalValue = new ThemableColor(Colors.Blue);
            p4.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument ShadingPatternAndShadingColorDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();
            Paragraph p4 = section.Blocks.AddParagraph();

            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            Run run2 = p2.Inlines.AddRun(SampleParagraphText);
            Run run3 = p3.Inlines.AddRun(SampleParagraphText);

            // Set ShadingPattern and ShadingColor properties through the run
            run1.Shading.Pattern = ShadingPattern.HorizontalStripe;
            run2.Shading.Pattern = ShadingPattern.VerticalStripe;
            run3.Shading.Pattern = ShadingPattern.None;

            run1.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            run2.Shading.PatternColor = new ThemableColor(Colors.Red);

            // Set ShadingPattern and ShadingColor properties through ParagraphMarkerProperties
            p4.Properties.ParagraphMarkerProperties.ShadingPattern.LocalValue = ShadingPattern.HorizontalStripe;
            p4.Properties.ParagraphMarkerProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Yellow);

            return document;
        }

        private static RadFlowDocument StrikeThroughDocument()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();

            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            Run run2 = p2.Inlines.AddRun(SampleParagraphText);

            // Set StrikeThrough property through the run
            run1.Strikethrough = true;
            run2.Strikethrough = false;

            // Set StrikeThrough property through ParagraphMarkerProperties
            p3.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p3.Inlines.AddRun(SampleParagraphText);

            return document;
        }

        private static RadFlowDocument HighlightColorDocument()
        {
            RadFlowDocument document = new RadFlowDocument();

            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();

            // Set BackgroundColor property through the run
            Run run1 = p1.Inlines.AddRun(SampleParagraphText);
            run1.HighlightColor = Colors.Red;

            // Set backgroundColot property through ParagraphMarkerProperties
            p2.Inlines.AddRun(SampleParagraphText);
            p1.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);

            return document;
        }

        private static RadFlowDocument BaselineAlignmentDocument()
        {
            RadFlowDocument document = new RadFlowDocument();

            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Blocks.AddParagraph();
            Paragraph p2 = section.Blocks.AddParagraph();
            Paragraph p3 = section.Blocks.AddParagraph();
            Paragraph p4 = section.Blocks.AddParagraph();

            // Set BaselineAlignment property through the run
            p1.Inlines.AddRun(SampleParagraphText);
            Run run1 = p1.Inlines.AddRun("Superscript");
            run1.BaselineAlignment = BaselineAlignment.Subscript;

            p2.Inlines.AddRun(SampleParagraphText);
            Run run2 = p2.Inlines.AddRun("Subscript");
            run2.BaselineAlignment = BaselineAlignment.Superscript;

            // Set BaselineAlignment property through ParagraphMarkerProperties
            p3.Inlines.AddRun("Superscript");
            p3.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;

            p4.Inlines.AddRun("Subscript");
            p4.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;

            return document;
        }

        private static readonly string SampleParagraphText = "Video provides a powerful way to help you prove your point. " +
                                              "When you click Online Video, you can paste in the embed code for the video you want to add. " +
                                              "You can also type a keyword to search online for the video that best fits your document. " +
                                              "To make your document look professionally produced, Word provides header, footer, cover page, " +
                                              "and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar";

        private const string SampleLongStringRtl = @"أعلن رئيس المجلس العسكري الأعلى للجيش السوري الحر اللواء سليم إدريس اليوم أنه فقد الأمل بوصول مساعدات عسكرية غربية إلى الثوار، في حين أكد مصدر من المعارضة المسلحة الإعداد لهجوم شامل هدفه إحكام السيطرة على مدينة حلب وريفها في شمال سوريا بدعم من المملكة العربية السعودية.
وتعليقاً على تصريحات صادرة من واشنطن والاتحاد الأوروبي وفرنسا أكّدت فيها للمعارضة أنها سترسل إليها مساعدات عسكرية، قال إدريس لوكالة أنباء الأناضول التركية فقدنا أملنا بالغرب، لكن ليس بالله، مضيفا لا نفهم موقف الغرب، وعدونا في البداية برفع الحظر عن بيع الأسلحة إلى سوريا، ثم عدلوا عن ذلك.
كما وجّه إدريس انتقادات إلى رئيس الوزراء البريطاني ديفد كاميرون، قائلا إنه يعذّبه، مشيرا إلى أن سبب عدم وصول مساعدات عسكرية من الاتحاد الأوروبي أو من الولايات المتحدة اعتبارهم تطميناتنا حول عدم وقوع الأسلحة في أيدي جماعات متطرفة غير مناسبة.
وأعرب مسؤولون أميركيون أمس عن تشاؤمهم حيال تمرير خطط تسليح المعارضة السورية سريعا، معتبرين أن خطط تسليح الثوار تخطت عقبة في الكونغرس لكنها قد تواجه المزيد من العقبات عندما يجري تنفيذ عملية التمويل خلال شهرين، مما يزيد من تأخر تدفق الأسلحة
";
        private const string SampleLongStringCh = @"按一般规律，从一名普通科员提升为一名正科级干部，需要6年时间，对于一名22岁的本科毕业生来说，正科任职年龄得满28岁。5月初，31位平均年龄只有26.2岁的怀柔区基层年轻干部通过公选方式走上了正科级实职岗位。
据了解，此次公开选拔，怀柔区一次性拿出31个单位团（工）委书记的职位，将报名年龄限定在1985年7月1日以后出生。报名人员可以是副科级干部，也可以是普通科员。
根据党政领导干部选拔任用相关工作条例，特别优秀的年轻干部或者工作特殊需要的，可以破格提拔。破格提拔时，可适当放宽对任职资格要求的限制。怀柔区此次共拿出31个正科级实职岗位，破格的规模和力度前所未有。
这次公选的初衷，一方面是为了改善当地干部的专业和年龄结构，通过公选的平台发现人才；另一方面是为了充实后备人才库，为今后领导班子建设储备干部。怀柔区委组织部副处级组织员翟文岩说。
“参加工作才四年多一点，这些年轻干部的资历、履历、业绩能够服众么？”有人提出这样的疑问。翟文岩表示，公选对干部的资格认定采用统一标准，对于资历履历相对单一的年轻干部而言，统一的选拔标准为他们提供了公平的竞争环境。公选不仅仅看重干部干过些什么，更要考察他能干什么，是否符合公选岗位的要求。也就是更关注他们的发展潜质。因此，选拔的各个环节设计就会更为严格。
根据要求，所有报考者不能直接填报31个单位的某一具体职位，而要先参加笔试，笔试成绩优异者，优先选报岗位。避免了因报考本单位而带来的“近水楼台”、“人选内定”等非客观因素。
此外，怀柔区组织部门在面试评委的组织方式上进行了创新，实行“大评委制”。评委既有来自组织部、人力社保局、团区委等单位的7个专业考官，也有10名区级“两代表一委员”组成的群众评委团队。众多报考者认为，这是一次“多数人选多数人”的选拔。
“相对于众多专业性岗位而言，团委书记的工作涉及众多领域，对报考者的综合素质要求更高，是一个年轻人的展示舞台。同时，这一岗位的灵活性很强，干得好的人可以转岗担任综合性领导职务，能力欠缺的人也可被调整至其他岗位上，而不影响中心工作，降低了用人风险。”翟文岩说。
据了解，新上任的团委书记们将经历军事化训练、岗前教育等多个培训环节。组织部门将对他们的工作表现进行长期跟踪，并有意识地让他们参与一些专项工作，提升工作能力。（记者 吴迪 通讯员 温来生）
";
    }
}
