﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class FlowTableProperties : DocumentFlowVisualBaseTest
    {
        private const string CellContentTemplate = "Cell [{0}, {1}]";

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void Test_DefaultTableWithBlackBorders()
        {
            RadFlowDocument document = CreateDefaultTableWithBlackBorders();

            AssertVisual(document);
        }

        private static RadFlowDocument CreateDefaultTableWithBlackBorders()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Table table = editor.InsertTable(6, 4);
            document.StyleRepository.AddBuiltInStyle(Telerik.Windows.Documents.Flow.Model.Styles.BuiltInStyleNames.TableGridStyleId);
            table.StyleId = Telerik.Windows.Documents.Flow.Model.Styles.BuiltInStyleNames.TableGridStyleId;

            for (int i = 0; i < 6; i++)
            {
                for (int j = 0; j < 3; j++)
                {
                    table.Rows[i].Cells[j].Blocks.AddParagraph().Inlines.AddRun("Test");
                }
            }

            return document;
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void Test_TableCellTextDirection()
        {
            string[] textDirections = new string[] { "Bottom to top left to right" ,
                                                     "Left to right top to bottom",
                                                     "Top to bottom right to left",
                                                     "Left to right top to bottom rotated to 270 degrees",
                                                     "Top to bottom left to right rotated to 90 degrees",
                                                     "Top to bottom right to left rotated to 90 degrees"};

            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Table table = editor.InsertTable(6, 1);

            table.Rows[0].Cells[0].TextDirection = TextDirection.BottomToTopLeftToRight;
            table.Rows[1].Cells[0].TextDirection = TextDirection.LeftToRightTopToBottom;
            table.Rows[2].Cells[0].TextDirection = TextDirection.TopToBottomRightToLeft;
            table.Rows[3].Cells[0].TextDirection = TextDirection.LeftToRightTopToBottomRotated;
            table.Rows[4].Cells[0].TextDirection = TextDirection.TopToBottomLeftToRightRotated;
            table.Rows[5].Cells[0].TextDirection = TextDirection.TopToBottomRightToLeftRotated;

            for (int i = 0; i <= 5; i++)
            {
                table.Rows[i].Cells[0].PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 150);
                table.Rows[i].Height = new TableRowHeight(HeightType.AtLeast, 50);
                table.Rows[i].Cells[0].Blocks.AddParagraph().Inlines.AddRun(textDirections[i]);
            }

            document.StyleRepository.AddBuiltInStyle(Telerik.Windows.Documents.Flow.Model.Styles.BuiltInStyleNames.TableGridStyleId);
            table.StyleId = Telerik.Windows.Documents.Flow.Model.Styles.BuiltInStyleNames.TableGridStyleId;

            this.AssertVisual(document);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void Test_ExportTableWithMergedCells()
        {
            RadFlowDocument document = this.CreateDocumentWithMergedTableCell();

            this.AssertVisual(document);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void Test_ImportExoportTableWithMergedCells()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("TableWithMergedCell.docx", new DocxFormatProvider());

            this.AssertVisual(document);
        }

        private void AddCell(TableRow tableRow, string content, int colspan, int rowspan)
        {
            TableCell cell = tableRow.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun().Text = content;
            cell.ColumnSpan = colspan;
            cell.RowSpan = rowspan;
        }

        public RadFlowDocument CreateDocumentWithMergedTableCell()
        {
            RadFlowDocument document = new RadFlowDocument();
            var section = new Section(document);
            document.Sections.Add(section);

            var table = section.Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            this.AddCell(row, string.Format(CellContentTemplate, 1, 0), 1, 1);
            this.AddCell(row, string.Format(CellContentTemplate, 1, 1), 1, 2);
            this.AddCell(row, string.Format(CellContentTemplate, 1, 2), 1, 1);

            row = table.Rows.AddTableRow();
            this.AddCell(row, string.Format(CellContentTemplate, 2, 0), 1, 1);
            this.AddCell(row, string.Format(CellContentTemplate, 2, 2), 1, 1);

            return document;
        }
    }
}