﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class HeaderAndFooterTests : DocumentFlowVisualBaseTest
    {
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DefaultHeader_Docx()
        {
            RadFlowDocument document = CreateDefaultHeaderDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });

        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DefaultHeader_Pdf()
        {
            RadFlowDocument document = CreateDefaultHeaderDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DifferentFirstPageHeader_Docx()
        {
            RadFlowDocument document = CreateDifferentFirstPageHeaderDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DifferentFirstPageHeader_Pdf()
        {
            RadFlowDocument document = CreateDifferentFirstPageHeaderDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_EvenOddPageHeader_Docx()
        {
            RadFlowDocument document = CreateEvenOddPageHeaderDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_EvenOddPageHeader_Pdf()
        {
            RadFlowDocument document = CreateEvenOddPageHeaderDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_LongPageHeader_Docx()
        {
            RadFlowDocument document = CreateLongPageHeaderDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_LongPageHeader_Pdf()
        {
            RadFlowDocument document = CreateLongPageHeaderDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DefaultFooter_Docx()
        {
            RadFlowDocument document = CreateDefaultFooterDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DefaultFooter_Pdf()
        {
            RadFlowDocument document = CreateDefaultFooterDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(2), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DifferentFirstPageFooter_Docx()
        {
            RadFlowDocument document = CreateDifferentFirstPageFooterDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_DifferentFirstPageFooter_Pdf()
        {
            RadFlowDocument document = CreateDifferentFirstPageFooterDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_EvenOddPageFooter_Docx()
        {
            RadFlowDocument document = CreateEvenOddPageFooterDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_EvenOddPageFooter_Pdf()
        {
            RadFlowDocument document = CreateEvenOddPageFooterDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_LongPageFooter_Docx()
        {
            RadFlowDocument document = CreateLongPageFooterDocument();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
        public void Test_LongPageFooter_Pdf()
        {
            RadFlowDocument document = CreateLongPageFooterDocument();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 2);
        }

        private static RadFlowDocument CreateLongPageHeaderDocument()
        {
            return CreateDefaultHeaderDocument("Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar. Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated. When you click Design and choose a new Theme, the pictures, charts, and SmartArt graphics change to match your new theme. When you apply styles, your headings change to match the new theme. Save time in Word with new buttons that show up where you need them. To change the way a picture fits in your document, click it and a button for layout options appears next to it. When you work on a table, click where you want to add a row or a column, and then click the plus sign. Reading is easier, too, in the new Reading view. You can collapse parts of the document and focus on the text you want. If you need to stop reading before you reach the end, Word remembers where you left off - even on another device. Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar.");
        }

        private static RadFlowDocument CreateLongPageFooterDocument()
        {
            return CreateDefaultFooterDocument("Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar. Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated. When you click Design and choose a new Theme, the pictures, charts, and SmartArt graphics change to match your new theme. When you apply styles, your headings change to match the new theme. Save time in Word with new buttons that show up where you need them. To change the way a picture fits in your document, click it and a button for layout options appears next to it. When you work on a table, click where you want to add a row or a column, and then click the plus sign. Reading is easier, too, in the new Reading view. You can collapse parts of the document and focus on the text you want. If you need to stop reading before you reach the end, Word remembers where you left off - even on another device. Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar.",
                "Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar. Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated. When you click Design and choose a new Theme, the pictures, charts, and SmartArt graphics change to match your new theme. When you apply styles, your headings change to match the new theme. Save time in Word with new buttons that show up where you need them. To change the way a picture fits in your document, click it and a button for layout options appears next to it. When you work on a table, click where you want to add a row or a column, and then click the plus sign. Reading is easier, too, in the new Reading view. You can collapse parts of the document and focus on the text you want. If you need to stop reading before you reach the end, Word remembers where you left off - even on another device. Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar. Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated. When you click Design and choose a new Theme, the pictures, charts, and SmartArt graphics change to match your new theme. When you apply styles, your headings change to match the new theme. Save time in Word with new buttons that show up where you need them. To change the way a picture fits in your document, click it and a button for layout options appears next to it. When you work on a table, click where you want to add a row or a column, and then click the plus sign. Reading is easier, too, in the new Reading view. You can collapse parts of the document and focus on the text you want. If you need to stop reading before you reach the end, Word remembers where you left off - even on another device. Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar. Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated. When you click Design and choose a new Theme, the pictures, charts, and SmartArt graphics change to match your new theme. When you apply styles, your headings change to match the new theme. Save time in Word with new buttons that show up where you need them.");
        }

        private static RadFlowDocument CreateEvenOddPageHeaderDocument()
        {
            RadFlowDocument document = CreateDefaultHeaderDocument("First");
            document.HasDifferentEvenOddPageHeadersFooters = true;
            Section section = document.Sections.First();
            section.Headers.Add(HeaderFooterType.Even).Blocks.AddParagraph().Inlines.AddRun("Second");
            Paragraph paragraph = (section.Blocks.First() as Paragraph);
            paragraph.Inlines.Add(new Break(document) { BreakType = BreakType.PageBreak });
            paragraph.Inlines.AddRun("Page 2");

            return document;
        }

        private static RadFlowDocument CreateEvenOddPageFooterDocument()
        {
            RadFlowDocument document = CreateDefaultFooterDocument("First");
            document.HasDifferentEvenOddPageHeadersFooters = true;
            Section section = document.Sections.First();
            section.Footers.Add(HeaderFooterType.Even).Blocks.AddParagraph().Inlines.AddRun("Second");
            Paragraph paragraph = (section.Blocks.First() as Paragraph);
            paragraph.Inlines.Add(new Break(document) { BreakType = BreakType.PageBreak });
            paragraph.Inlines.AddRun("Page 2");

            return document;
        }

        private static RadFlowDocument CreateDifferentFirstPageFooterDocument()
        {
            RadFlowDocument document = CreateDefaultFooterDocument();
            Section section = document.Sections.First();
            section.HasDifferentFirstPageHeaderFooter = true;
            section.Footers.Add(HeaderFooterType.First).Blocks.AddParagraph().Inlines.AddRun("First");
            Paragraph paragraph = (section.Blocks.First() as Paragraph);
            paragraph.Inlines.Add(new Break(document) { BreakType = BreakType.PageBreak });
            paragraph.Inlines.AddRun("Page 2");

            return document;
        }

        private static RadFlowDocument CreateDifferentFirstPageHeaderDocument()
        {
            RadFlowDocument document = CreateDefaultHeaderDocument();
            Section section = document.Sections.First();
            section.HasDifferentFirstPageHeaderFooter = true;
            section.Headers.Add(HeaderFooterType.First).Blocks.AddParagraph().Inlines.AddRun("First");
            Paragraph paragraph = (section.Blocks.First() as Paragraph);
            paragraph.Inlines.Add(new Break(document) { BreakType = BreakType.PageBreak });
            paragraph.Inlines.AddRun("Page 2");

            return document;
        }

        private static RadFlowDocument CreateDefaultHeaderDocument(string text = "Default")
        {
            RadFlowDocument document = CreateDefaulfFlowDocument();
            document.Sections.First()
                .Headers.Add(HeaderFooterType.Default)
                .Blocks.AddParagraph().Inlines.AddRun(text);

            return document;
        }

        private static RadFlowDocument CreateDefaultFooterDocument(string text = "Default", string body = "Page 1")
        {
            RadFlowDocument document = CreateDefaulfFlowDocument(body);
            document.Sections.First()
                .Footers.Add(HeaderFooterType.Default)
                .Blocks.AddParagraph().Inlines.AddRun(text);


            return document;
        }

        private static RadFlowDocument CreateDefaulfFlowDocument(string text = "Page 1")
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph().Inlines.AddRun(text);

            return document;
        }
    }
}
