﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Utils;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class Lists : DocumentFlowVisualBaseTest, INumberingStyleConverter
    {
        readonly List<Tuple<int, string>> listItems;
        private string twoLinesText = "This line should be extended to two lines so the behaviour of the lists could be observed when the list level text is two or more lines long.";

        public Lists()
        {
            this.listItems = new List<Tuple<int, string>>();
            this.listItems.Add(new Tuple<int, string>(0, "Video provides a powerful way to help you prove your point"));
            this.listItems.Add(new Tuple<int, string>(0, "When you click Online"));
            this.listItems.Add(new Tuple<int, string>(1, "Click Insert and then choose the elements you want from the different galleries"));
            this.listItems.Add(new Tuple<int, string>(2, "Foreground of the selected ToolBoxItem becomes white when"));
            this.listItems.Add(new Tuple<int, string>(3, "Header Button visual states and Foreground improvements in Windows8 theme.Cannot paste using context menu after pressing clear button"));
            this.listItems.Add(new Tuple<int, string>(2, "Avoid something is not BY DESIGN"));
            this.listItems.Add(new Tuple<int, string>(1, "Manipulation adorner"));
            this.listItems.Add(new Tuple<int, string>(0, "Cannot paste using context menu after pressing clear button."));
            this.listItems.Add(new Tuple<int, string>(1, "Header Button visual states and Foreground improvements in Windows8 theme.Cannot paste using context menu after pressing clear button"));
        }

        public string ConvertNumberToText(int number)
        {
            return ("s");
        }

        // OK
        // "Dont add space between paragrahs of the same style" checkbox is not chekced as in MS Word)
        // First list level is indented at 0.25", instead of being indented at 0"
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        [Description("Verifies bulleted list creation")]
        public void BulletedListExport_Docx()
        {
            RadFlowDocument document = CreateBulletedList();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        [Description("Verifies bulleted list creation")]
        public void BulletedListExport_Pdf()
        {
            RadFlowDocument document = CreateBulletedList();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        // "Dont add space between paragrahs of the same style" checkbox is not chekced as in MS Word)
        // First list level is indented at 0.25", instead of being indented at 0"
        [TestMethod]
        [Description("Verifies numbered list creation")]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void NumberedListExport_Docx()
        {
            RadFlowDocument document = CreateNumberedList();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [Description("Verifies numbered list creation")]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void NumberedListExport_Pdf()
        {
            RadFlowDocument document = CreateNumberedList();
           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        // "Dont add space between paragrahs of the same style" checkbox is not chekced as in MS Word)
        // First list level is indented at 0.25", instead of being indented at 0"
        [TestMethod]
        [Description("Verifies numbered parenthesis list creation")]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void NumberedParenthesesListExport_Docx()
        {
            RadFlowDocument document = CreateNumberedParenthesisList();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        [Description("Verifies numbered parenthisis list creation")]
        public void NumberedParenthesesListExport_Pdf()
        {
            RadFlowDocument document = CreateNumberedParenthesisList();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK 
        // Indentation does not look OK in WordsProcessing. 
        // "Dont add space between paragrahs of the same style" checkbox is not chekced as in MS Word)
        // First list level is indented at 0.25", instead of being indented at 0"
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        [Description("Verifies hierarchical numbered list creation")]
        public void NumberedHierarchcalListExport_Docx()
        {
            RadFlowDocument document = CreateNumberedHierarchicalList();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        [Description("Verifies hierarchical numbered list creation")]
        public void NumberedHierarchcalListExport_Pdf()
        {
            RadFlowDocument document = CreateNumberedHierarchicalList();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithPredefinedStartIndexExport_Docx()
        {
            RadFlowDocument document = CreateListWithPredefinedStartIndex();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithPredefinedStartIndexExport_Pdf()
        {
            RadFlowDocument document = CreateListWithPredefinedStartIndex();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithPredefinedRestartAfterLevelExport_Docx()
        {
            RadFlowDocument document = CreateListWithPredefinedRestartAfterLevel();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithPredefinedRestartAfterLevelExport_Pdf()
        {
            RadFlowDocument document = CreateListWithPredefinedRestartAfterLevel();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListDevidedToTwoByTwoEmptyParagraphsExport_Docx()
        {
            RadFlowDocument document = CreateListDevidedToTwoByTwoEmptyParagraphs();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListDevidedToTwoByTwoEmptyParagraphsExport_Pdf()
        {
            RadFlowDocument document = CreateListDevidedToTwoByTwoEmptyParagraphs();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListDevidedToTwoByTwoSimpleParagraphsExport_Docx()
        {
            RadFlowDocument document = CreateListDevidedToTwoByTwoParagraphs();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListDevidedToTwoByTwoSimpleParagraphsExport_Pdf()
        {
            RadFlowDocument document = CreateListDevidedToTwoByTwoParagraphs();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithIncreasingAndDecreasingLevelsExport_Docx()
        {
            RadFlowDocument document = CreateListWithIncreasingAndDecreasingLevels();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithIncreasingAndDecreasingLevelsExport_Pdf()
        {
            RadFlowDocument document = CreateListWithIncreasingAndDecreasingLevels();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithDecimalNumberingStyleExport_Docx()
        {
            RadFlowDocument document = CreateListWithDecimalNumberingStyle();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithDecimalNumberingStyleExport_Pdf()
        {
            RadFlowDocument document = CreateListWithDecimalNumberingStyle();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithLowerLetterNumberingStyleExport_Docx()
        {
            RadFlowDocument document = CreateListWithLowerLetterNumberingStyle();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithLowerLetterNumberingStyleExport_Pdf()
        {
            RadFlowDocument document = CreateListWithLowerLetterNumberingStyle();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithIdeographZodiacNumberingStyleExport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.IdeographZodiac;

                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // Check
        //[TestMethod]
        //public void ListWithIdeographZodiacNumberingStyleExport_Pdf()
        //{
        //    RadFlowDocument document = new RadFlowDocument();
        //    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
        //    PdfExportSettings setting = new PdfExportSettings();
        //    setting.ExtensibilityManager.RegisterNumberingStyleConverter(NumberingStyle.IdeographZodiac, ConvertNumberToText(0));

        //    List list = document.Lists.Add(ListTemplateType.NumberedDefault);
        //    Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

        //    for (int i = 0; i < 9; i++)
        //    {
        //        list.Levels[i].NumberingStyle = NumberingStyle.IdeographZodiac;

        //        p = editor.InsertParagraph();
        //        p.ListId = list.Id;
        //        p.ListLevel = i;
        //        editor.InsertText("Hello " + i);
        //    }

        //    AssertAreVisualyEqualPdf(document);
        //}

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithCustomBulletSymbolExport_Docx()
        {
            RadFlowDocument document = CreateListWithCustomBulletSymbol();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithCustomBulletSymbolExport_Pdf()
        {
            RadFlowDocument document = CreateListWithCustomBulletSymbol();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithAppliedCustomStyleExport_Docx()
        {
            RadFlowDocument document = CreateListWithAppliedCustomStyle();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithAppliedCustomStyleExport_Pdf()
        {
            RadFlowDocument document = CreateListWithAppliedCustomStyle();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithNoBulletAndNoTextExport_Docx()
        {
            RadFlowDocument document = CreateListWithNoBUlletAndNoTextLevelItem();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithNoBulletAndNoTextExport_Pdf()
        {
            RadFlowDocument document = CreateListWithNoBUlletAndNoTextLevelItem();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithBuiltInStyleAppliedToTheParagraphExport_Docx()
        {
            RadFlowDocument document = CreateListWithBuiltInStyleAppliedToTheParagraph();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // Style settings are not applied on the bullet -  DISCUSS
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithBuiltInStyleAppliedToTheParagraphExport_Pdf()
        {
            RadFlowDocument document = CreateListWithBuiltInStyleAppliedToTheParagraph();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithBuiltInStyleAppliedToTheListLevelExport_Docx()
        {
            RadFlowDocument document = CreateListWithBuiltInStyleAppliedToTheListLevel();
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithBuiltInStyleAppliedToTheListLevelExport_Pdf()
        {
            RadFlowDocument document = CreateListWithBuiltInStyleAppliedToTheListLevel();
           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithOverrdenCharacterPropertiesForListLevelsExport_Docx()
        {
            RadFlowDocument document = OverrideCharacterPropertiesForListLevels();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithOverrdenCharacterPropertiesForListLevelsExport_Pdf()
        {
            RadFlowDocument document = OverrideCharacterPropertiesForListLevels();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void DifferentCharacterPropertiesAppliedToDifferentListsExport_Docx()
        {
            RadFlowDocument document = CreateListWithDifferentCharacterPropertiesApplied();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void DifferentCharacterPropertiesAppliedToDifferentListsExport_Pdf()
        {
            RadFlowDocument document = CreateListWithDifferentCharacterPropertiesApplied();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithParagraphPropertiesModifiedForTheEditorExport_Docx()
        {
            RadFlowDocument document = CreateListWithParagraphPropertiesModifiedThroughTheEditor();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // OK
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        public void ListWithParagraphPropertiesModifiedForTheEditorExport_Pdf()
        {
            RadFlowDocument document = CreateListWithParagraphPropertiesModifiedThroughTheEditor();

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.List)]
        // FIRST LINE INDENT ISSUE!!!!!!!!!!!!!!!!!!!
        public void FirstLineIndentAfterBullet()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("List with default FirstLineIndent value for all list levels");

            // Create a list with default IndentAfterBulletNumbering values for all levels
            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun(twoLinesText);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("List with custom value for FirstLineIndent value for all list levels");

            //// Set a custom value for the indent after bullet 
            //foreach (var level in list2.Levels)
            //{
            //    level.ParagraphProperties.FirstLineIndent.LocalValue = 80;
            //}

            list2.Levels[1].ParagraphProperties.HangingIndent.LocalValue = 40;
            list2.Levels[1].ParagraphProperties.LeftIndent.LocalValue = 0;

            // Create a list 
            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                //p.Properties.FirstLineIndent.LocalValue = 40;
                p.ListId = list2.Id;
                p.ListLevel = i;
                p.Inlines.AddRun(twoLinesText);
            }

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        public void AddParagraphWithListToDocument(Section section, List list, int index)
        {
            Paragraph p = null;
            for (int i = index; i < index + 1; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("List item " + i);
            }
        }

        public void AddParagraphsToDocument(RadFlowDocumentEditor editor, List list, int index)
        {
            Paragraph p = null;
            for (int i = index; i < index + 1; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }
        }

        public void AddParagraphsToDocument(RadFlowDocumentEditor editor, List list, int index, int serialIndexOfListLevelElement = 1)
        {
            Paragraph p = null;
            for (int i = index; i < index + 1; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                p.StyleId = BuiltInStyleNames.NormalWebStyleId;
                editor.InsertText(i + "-" + serialIndexOfListLevelElement);
            }
        }

        private RadFlowDocument CreateBulletedList()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list1 = document.Lists.Add(ListTemplateType.BulletDefault);
            List list2 = document.Lists.Add(ListTemplateType.BulletDefault);
            List list3 = document.Lists.Add(ListTemplateType.BulletDefault);
            List list4 = document.Lists.Add(ListTemplateType.BulletDefault);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph before the list");

            for (int i = 0; i < 9; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Hello " + i);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Paragraph between first and second paragraph");

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun(SampleParagraph);

            for (int i = 0; i < 3; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list3.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun(twoLinesText);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Sample text");

            foreach (var item in listItems)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list4.Id;
                p.ListLevel = item.Item1;
                p.Inlines.AddRun(item.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateNumberedList()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list3 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list4 = document.Lists.Add(ListTemplateType.NumberedDefault);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph before the list");

            for (int i = 0; i < 9; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Hello " + i);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Paragraph between first and second paragraph");

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun(SampleParagraph);

            for (int i = 0; i < 3; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list3.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun(twoLinesText);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Sample text");

            foreach (var item in listItems)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list4.Id;
                p.ListLevel = item.Item1;
                p.Inlines.AddRun(item.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateNumberedParenthesisList()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list1 = document.Lists.Add(ListTemplateType.NumberedParentheses);
            List list2 = document.Lists.Add(ListTemplateType.NumberedParentheses);
            List list3 = document.Lists.Add(ListTemplateType.NumberedParentheses);
            List list4 = document.Lists.Add(ListTemplateType.NumberedParentheses);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph before the list");

            for (int i = 0; i < 9; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Hello " + i);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Paragraph between first and second paragraph");

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun(SampleParagraph);

            for (int i = 0; i < 3; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list3.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun(twoLinesText);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Sample text");

            foreach (var item in listItems)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list4.Id;
                p.ListLevel = item.Item1;
                p.Inlines.AddRun(item.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateNumberedHierarchicalList()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list1 = document.Lists.Add(ListTemplateType.NumberedHierarchical);
            List list2 = document.Lists.Add(ListTemplateType.NumberedHierarchical);
            List list3 = document.Lists.Add(ListTemplateType.NumberedHierarchical);
            List list4 = document.Lists.Add(ListTemplateType.NumberedHierarchical);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph before the list");

            for (int i = 0; i < 9; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Hello " + i);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Paragraph between first and second paragraph");

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun(SampleParagraph);

            for (int i = 0; i < 3; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list3.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun(twoLinesText);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Sample text");

            foreach (var item in listItems)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list4.Id;
                p.ListLevel = item.Item1;
                p.Inlines.AddRun(item.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateListWithPredefinedStartIndex()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            list1.Levels[0].StartIndex = 3;

            foreach (var listitem in listItems)
            {
                Paragraph p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = listitem.Item1;
                p.Inlines.AddRun(listitem.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateListWithPredefinedRestartAfterLevel()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);
            int[] listLevel = new int[] { 0, 1, 2, 1, 2, 1, 0, 1, 2 };

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("List with RestartAfterLevel=0 for the third level");

            // Predefine RestartAfterLevel for Level 2
            list1.Levels[2].RestartAfterLevel = 0;

            // Create a list 
            for (int i = 0; i < listLevel.Length; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = listLevel[i];
                p.Inlines.AddRun("Level " + listLevel[i]);
            }

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("List with default RestartAfterLevel value for all list levels");

            for (int i = 0; i < listLevel.Length; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = listLevel[i];
                p.Inlines.AddRun("Level " + listLevel[i]);
            }
            return document;
        }

        private RadFlowDocument CreateListDevidedToTwoByTwoEmptyParagraphs()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph");

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            this.AddParagraphWithListToDocument(section, list, 0);
            this.AddParagraphWithListToDocument(section, list, 0);
            this.AddParagraphWithListToDocument(section, list, 0);

            section.Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            this.AddParagraphWithListToDocument(section, list, 0);
            this.AddParagraphWithListToDocument(section, list, 0);
            this.AddParagraphWithListToDocument(section, list, 0);
            return document;
        }

        private RadFlowDocument CreateListDevidedToTwoByTwoParagraphs()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            editor.InsertText("Simple paragraph");

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            this.AddParagraphsToDocument(editor, list, 0);
            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 2);

            editor.InsertParagraph();
            editor.InsertText(SampleParagraph);
            editor.InsertParagraph();
            editor.InsertText(SampleParagraph);

            this.AddParagraphsToDocument(editor, list, 2);
            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 0);
            return document;
        }

        private RadFlowDocument CreateListWithIncreasingAndDecreasingLevels()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            for (int i = 7; i >= 0; i--)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }
            return document;
        }

        private RadFlowDocument CreateListWithDecimalNumberingStyle()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;

                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("List level with Decimal numbering style " + i);
            }
            return document;
        }

        private RadFlowDocument CreateListWithLowerLetterNumberingStyle()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.LowerLetter;

                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }
            return document;
        }

        private RadFlowDocument CreateListWithCustomBulletSymbol()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph");

            List list1 = document.Lists.Add(ListTemplateType.BulletDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 4; i++)
            {
                list1.Levels[i].NumberingStyle = NumberingStyle.Bullet;
                list1.Levels[i].NumberTextFormat = "¤";
                list1.Levels[i].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Times New Roman"));

                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Test text " + i);
            }

            list2.Levels[0].StartIndex = 6;
            list2.Levels[0].NumberTextFormat = "__%1)__";

            p = section.Blocks.AddParagraph();

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list2.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }
            return document;
        }

        private RadFlowDocument CreateListWithAppliedCustomStyle()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Style style = new Style("PStyle", StyleType.Paragraph);
            style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            document.StyleRepository.Add(style);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 3; i++)
            {
                p = editor.InsertParagraph();
                editor.InsertText("Text");
                p.ListId = list.Id;
                p.ListLevel = i;
                p.StyleId = style.Id;
            }
            return document;
        }

        private RadFlowDocument CreateListWithNoBUlletAndNoTextLevelItem()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("List item 0");
            p.ListId = list.Id;
            p.ListLevel = 0;

            p = section.Blocks.AddParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;

            this.AddParagraphWithListToDocument(section, list, 1);
            this.AddParagraphWithListToDocument(section, list, 1);
            return document;
        }

        private RadFlowDocument CreateListWithBuiltInStyleAppliedToTheParagraph()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1));
            }

            for (int i = 0; i < 4; i++)
            {
                Paragraph p = section.Blocks.AddParagraph();
                p.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
                p.ListId = list.Id;
                p.ListLevel = i;
                p.Inlines.AddRun("Heading style " + i);
            }
            return document;
        }

        private RadFlowDocument CreateListWithBuiltInStyleAppliedToTheListLevel()
        {
            RadFlowDocument document = new RadFlowDocument();

            for (int i = 0; i < 9; i++)
            {
                document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1));
            }

            Section section = document.Sections.AddSection();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                Paragraph p = section.Blocks.AddParagraph();
                p.ListId = list.Id;
                list.Levels[i].StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
                p.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
                p.Inlines.AddRun("Heading style " + (i + 1));
            }
            return document;
        }

        private RadFlowDocument OverrideCharacterPropertiesForListLevels()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            // Define a list
            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list1.Levels[i].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
                list1.Levels[i].CharacterProperties.FontSize.LocalValue = 20;
                list1.Levels[i].CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            }

            // Populate list items
            foreach (var listItem in listItems)
            {
                p = section.Blocks.AddParagraph();
                p.ListId = list1.Id;
                p.ListLevel = listItem.Item1;
                p.Inlines.AddRun(listItem.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateListWithDifferentCharacterPropertiesApplied()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            // Define three lists
            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);

            // Modify the default character properties values            
            editor.CharacterFormatting.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            editor.CharacterFormatting.FontSize.LocalValue = 10;
            editor.CharacterFormatting.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            editor.CharacterFormatting.HighlightColor.LocalValue = Colors.Yellow;

            // Insert a paragraph after the first list
            editor.InsertParagraph();
            editor.InsertText("List with custom character properties for all list items");

            // Populate first list items
            foreach (var listItem in listItems)
            {
                Paragraph p = editor.InsertParagraph();
                p.ListId = list1.Id;
                p.ListLevel = listItem.Item1;
                editor.InsertText(listItem.Item2);
            }
            return document;
        }

        private RadFlowDocument CreateListWithParagraphPropertiesModifiedThroughTheEditor()
        {
            RadFlowDocument document = new RadFlowDocument();

            document.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.Yellow);
            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            p.ListId = list.Id;
            p.Inlines.AddRun("test");

            p = section.Blocks.AddParagraph();
            p.ListId = list.Id;
            p.Inlines.AddRun("test");
            return document;
        }

        private static readonly string SampleParagraph = @"Video provides a powerful way to help you prove your point. When you click Online Video, you can paste in the embed code for the video you want to add. You can also type a keyword to search online for the video that best fits your document. To make your document look professionally produced, Word provides header, footer, cover page, and text box designs that complement each other. For example, you can add a matching cover page, header, and sidebar";

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void CreateDocumentWithEmptyParagraphs()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertParagraph();
            editor.InsertText("First");

            editor.InsertParagraph();
            editor.InsertParagraph().Inlines.AddRun("Run text");

            editor.InsertParagraph();
            editor.InsertText("Second");

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that the different paragraph properties could be applied to different lists")]
        //public void Test_ApplyDifferentParagraphPropertiesToDifferentLists()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        // Define lists
        //        List list1 = document.Lists.AddList(ListTemplateType.NumberedDefault);
        //        List list2 = document.Lists.AddList(ListTemplateType.NumberedParentheses);

        //        // Insert a paragraph before the first list
        //        InsertNewParagraphWithoutList(editor, "List with custom paragraph properties for all list items");

        //        // Modify the default paragraph properties values
        //        editor.ParagraphProperties.BackgroundColor = TestHelper.Turqouise;
        //        editor.ParagraphProperties.FirstLineIndent = 80;
        //        editor.ParagraphProperties.LineSpacing = 10.0;

        //        // Populate list items
        //        for (int i = 0; i < 3; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list2.Id;
        //            editor.ParagraphProperties.ListLevel = i;
        //            editor.InsertParagraph();
        //            editor.InsertLine("First list, level: " + i);
        //        }

        //        // Reset the paragraph properties to the default values and insert a new paragraph
        //        editor.ParagraphProperties.LineSpacing = FixedDocumentDefaults.LineSpacing;
        //        editor.ParagraphProperties.FirstLineIndent = FixedDocumentDefaults.FirstLineIndent;
        //        editor.ParagraphProperties.BackgroundColor = null;
        //        InsertNewParagraphWithoutList(editor, "List with default paragraph properties for all list items");

        //        // Populate list items
        //        foreach (var listItem in listItems)
        //        {
        //            InsertListItemText(editor, list1.Id, listItem.Item1, listItem.Item2);
        //        }
        //    }

        //    this.AssertAreVisualyEqual(document);
        //}



        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that list's FirstLineIndent could be predefined")]
        //public void Test_OverideDefaultFirstLineIndentValue()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        // Create two lists
        //        List list1 = document.Lists.AddList(ListTemplateType.NumberedDefault);
        //        List list2 = document.Lists.AddList(ListTemplateType.NumberedDefault);

        //        // Insert new paragraph
        //        editor.CharacterProperties.ForegroundColor = TestHelper.Red;
        //        editor.InsertLine("This is a list with default FirstLineIdent");
        //        editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;
        //        editor.InsertParagraph();

        //        // Create a list with default FirtLineIndent value
        //        for (int i = 0; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list1.Id;
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);
        //        }

        //        // Insert new paragraph
        //        editor.ParagraphProperties.ListId = null;
        //        editor.CharacterProperties.ForegroundColor = TestHelper.Red;
        //        editor.InsertLineBreak();
        //        editor.InsertParagraph();
        //        editor.InsertLine("This is a list with overridden FirstLineIdent");
        //        editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;

        //        // Override the list's FirstLineIndent value
        //        foreach (var level in list2.Levels)
        //        {
        //            level.ParagraphProperties.FirstLineIndent = 40;
        //        }

        //        // Create a list with overridden FirstLineIndent value
        //        for (int i = 0; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list2.Id;
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);
        //        }
        //    }

        //    this.AssertAreVisualyEqual(document);
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that list's LeftIndent could be predefined")]
        //public void Test_OverideDefaultLeftIndentValue()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        // Create two lists
        //        List list = document.Lists.AddList(ListTemplateType.NumberedDefault);
        //        List list1 = document.Lists.AddList(ListTemplateType.NumberedDefault);

        //        // Insert a new paragraph
        //        editor.CharacterProperties.ForegroundColor = TestHelper.Red;
        //        editor.InsertLine("This is a list with default LeftIndent");
        //        editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;
        //        editor.InsertParagraph();

        //        // Create a list with default LeftIndent value
        //        for (int i = 0; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list.Id;
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);
        //        }

        //        // Insert a new paragraph
        //        editor.ParagraphProperties.ListId = null;
        //        editor.CharacterProperties.ForegroundColor = TestHelper.Red;
        //        editor.InsertLineBreak();
        //        editor.InsertParagraph();
        //        editor.InsertLine("This is a list with overridden LeftIndent");
        //        editor.CharacterProperties.ForegroundColor = FixedDocumentDefaults.ForegroundColor;

        //        // Override list's LeftIndent value
        //        foreach (var level in list1.Levels)
        //        {
        //            level.ParagraphProperties.LeftIndent = 100;
        //        }

        //        // Create a list with overridden LeftIndent value
        //        for (int i = 0; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list1.Id;
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);
        //        }
        //    }

        //    this.AssertAreVisualyEqual(document);
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that default bullet style could be set")]
        //public void Test_SetCustomBulletFormat()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        // Create a list
        //        List list = document.Lists.AddList(ListTemplateType.NumberedDefault);

        //        editor.CharacterProperties.ForegroundColor = TestHelper.Pink;

        //        // Define new bullet type for all list levels
        //        foreach (var level in list.Levels)
        //        {
        //            level.CharacterProperties.Font = FontsRepository.TimesRoman;
        //            level.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
        //            list.Levels[0].BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //            {
        //                return string.Format("¤", indexer.GetCurrentIndex(0));
        //            });
        //        }

        //        // Create the list
        //        for (int i = 1; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list.Id;
        //            editor.InsertParagraph();
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertLine("Custom bullet style" + i);
        //        }
        //    }
        //    this.AssertAreVisualyEqual(document);
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that negative value for FirstLineIndent is applied as Hanging indent")]
        //public void Test_NegativeFirstLineIndentAppliesAsHangingIndent()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        List list = document.Lists.AddList(ListTemplateType.NumberedDefault);

        //        foreach (var level in list.Levels)
        //        {
        //            level.ParagraphProperties.FirstLineIndent = -80;
        //        }

        //        for (int i = 1; i < 4; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list.Id;
        //            editor.InsertParagraph();
        //            editor.ParagraphProperties.ListLevel = 0;
        //            editor.InsertLine(TestHelper.SampleParagraph);
        //        }
        //    }
        //    this.AssertAreVisualyEqual(document);
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //[Description("Verifies that custom defined list levels could be applied")]
        //public void Test_SetSeparateCharacterPropertiesForEachListLevel()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        List list = document.Lists.AddList();

        //        ListLevel listLevel = new ListLevel();
        //        listLevel.StartIndex = 2;
        //        listLevel.RestartAfterLevel = 0;
        //        listLevel.CharacterProperties.Font = FontsRepository.CourierBold;
        //        listLevel.CharacterProperties.FontSize = 20;
        //        listLevel.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
        //        listLevel.IndentAfterBullet = 30;
        //        listLevel.ParagraphProperties.FirstLineIndent = 10;
        //        listLevel.ParagraphProperties.LeftIndent = 20;
        //        listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //        {
        //            return string.Format("-{0}->", indexer.GetCurrentIndex(0));
        //        });

        //        ListLevel listLevel1 = new ListLevel();
        //        listLevel1.StartIndex = 4;
        //        listLevel1.RestartAfterLevel = 0;
        //        listLevel1.CharacterProperties.Font = FontsRepository.CourierBoldOblique;
        //        listLevel1.CharacterProperties.FontSize = 22;
        //        listLevel1.CharacterProperties.ForegroundColor = TestHelper.Red;
        //        listLevel1.IndentAfterBullet = 30;
        //        listLevel1.ParagraphProperties.FirstLineIndent = 10;
        //        listLevel1.ParagraphProperties.LeftIndent = 30;
        //        listLevel1.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //        {
        //            return string.Format("-{0}->", indexer.GetCurrentIndex(1));
        //        });

        //        ListLevel listLevel2 = new ListLevel();
        //        listLevel2.StartIndex = 6;
        //        listLevel2.RestartAfterLevel = 0;
        //        listLevel2.CharacterProperties.Font = FontsRepository.CourierBold;
        //        listLevel2.CharacterProperties.FontSize = 24;
        //        listLevel2.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
        //        listLevel2.IndentAfterBullet = 30;
        //        listLevel2.ParagraphProperties.FirstLineIndent = 10;
        //        listLevel2.ParagraphProperties.LeftIndent = 40;
        //        listLevel2.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //        {
        //            return string.Format("-{0}->", indexer.GetCurrentIndex(2));
        //        });

        //        list.Levels.Add(listLevel);
        //        list.Levels.Add(listLevel1);
        //        list.Levels.Add(listLevel2);

        //        for (int i = 0; i < 3; i++)
        //        {
        //            editor.ParagraphProperties.ListId = list.Id;
        //            editor.ParagraphProperties.ListLevel = i;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);

        //            editor.ParagraphProperties.ListLevel = i;
        //            editor.InsertParagraph();
        //            editor.InsertLine(twoLinesText);
        //        }
        //    }
        //    this.AssertAreVisualyEqual(document);
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //public void Test_CreateADocumentWithAllListTemplateTypes()
        //{
        //    RadFixedDocument document = new RadFixedDocument();

        //    using (RadFixedDocumentEditor editor = new RadFixedDocumentEditor(document))
        //    {
        //        // Create four bulleted lists
        //        List list1 = document.Lists.AddList(ListTemplateType.BulletDefault);
        //        List list2 = document.Lists.AddList(ListTemplateType.NumberedDefault);
        //        List list3 = document.Lists.AddList(ListTemplateType.NumberedHierarchical);
        //        List list4 = document.Lists.AddList(ListTemplateType.NumberedParentheses);

        //        List<List> lists = new List<List>();
        //        lists.Add(list1);
        //        lists.Add(list2);
        //        lists.Add(list3);
        //        lists.Add(list4);

        //        foreach (var list in lists)
        //        {
        //            InsertNewParagraphWithoutList(editor, "Paragraph separator");

        //            // Populate list items
        //            for (int i = 0; i < 4; i++)
        //            {
        //                editor.ParagraphProperties.ListId = list.Id;
        //                editor.ParagraphProperties.ListLevel = i;
        //                editor.InsertParagraph();
        //                editor.InsertLine("List level: " + i);
        //            }
        //        }
        //    }

        //    this.AssertAreVisualyEqual(document);
        //}

        //private ListLevel CreateListLevel(int currentIndex)
        //{
        //    ListLevel listLevel = new ListLevel();
        //    listLevel.CharacterProperties.Font = FontsRepository.CourierBold;
        //    listLevel.CharacterProperties.FontSize = 20;
        //    listLevel.CharacterProperties.ForegroundColor = TestHelper.Turqouise;
        //    listLevel.StartIndex = 0;
        //    listLevel.IndentAfterBullet = 30;
        //    listLevel.RestartAfterLevel = 0;
        //    listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //    {
        //        return string.Format("-{0}->", indexer.GetCurrentIndex(currentIndex));
        //    });
        //    return listLevel;
        //}

        // [TestMethod]  [TestCategory(WordsProcessingCategory.VisualTest)][TestCategory(WordsProcessingCategory.List)]
        //public void Test_InsertListThroughBlock()
        //{
        //    RadFixedDocument document = new RadFixedDocument();
        //    RadFixedPage page = document.Pages.AddPage();
        //    FixedContentEditor editor = CreateFixedContentEditor(page);

        //    List list = new List();

        //    // Create first list level
        //    ListLevel listLevel = new ListLevel();
        //    listLevel.StartIndex = 2;
        //    listLevel.ParagraphProperties.LeftIndent = 20;
        //    listLevel.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //    {
        //        return string.Format("-{0}->", indexer.GetCurrentIndex(0));
        //    });

        //    // Create second list level
        //    ListLevel listLevel1 = new ListLevel();
        //    listLevel1.StartIndex = 4;
        //    listLevel1.ParagraphProperties.LeftIndent = 30;
        //    listLevel1.BulletNumberingFormat = new TextBulletNumberingFormat((indexer) =>
        //    {
        //        return string.Format("-{0}->", indexer.GetCurrentIndex(1));
        //    });

        //    // Add list levels to the list
        //    list.Levels.Add(listLevel);
        //    list.Levels.Add(listLevel1);

        //    Block block1 = new Block();
        //    block1.SetBullet(list, 0);
        //    block1.InsertText("one");
        //    var size1 = block1.Measure();
        //    editor.DrawBlock(block1);
        //    editor.Position.Translate(0, size1.Height);

        //    Block block2 = new Block();
        //    block2.SetBullet(list, 0);
        //    block2.InsertText("one");
        //    var size2 = block2.Measure();
        //    editor.DrawBlock(block2);
        //    editor.Position.Translate(0, size1.Height + size2.Height);

        //    Block block3 = new Block();
        //    block3.SetBullet(list, 1);
        //    block3.InsertText("two");
        //    editor.DrawBlock(block3);
        //    var size3 = block3.Measure();
        //    editor.DrawBlock(block3);
        //    editor.Position.Translate(0, size1.Height + size2.Height + size3.Height);

        //    Block block4 = new Block();
        //    block4.SetBullet(list, 1);
        //    block4.InsertText("two");
        //    editor.DrawBlock(block4);

        //    this.AssertAreVisualyEqual(document);
        //}

        //private void InsertListItemText(RadFixedDocumentEditor editor, int listId, int listLevel, string text)
        //{
        //    editor.ParagraphProperties.ListId = listId;
        //    editor.ParagraphProperties.ListLevel = listLevel;
        //    editor.InsertParagraph();
        //    editor.InsertRun(text);
        //}

        //private void InsertNewParagraphWithoutList(RadFixedDocumentEditor editor, string text)
        //{
        //    editor.ParagraphProperties.ListId = null;
        //    editor.InsertLineBreak();
        //    editor.InsertParagraph();
        //    editor.InsertLine(text);
        //    editor.InsertLineBreak();
        //}
    }
}
