﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Editing
{
    [TestClass]
    public class TextSearch : DocumentFlowVisualBaseTest
    {
        public static string TestFilesFolder = "TextSearch/";

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextInCommentAndBookmarkTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "DocumentWithCommentAndBookmark.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ReplaceText("Video", "Presentation", false, true);
            editor.ReplaceText("Word", "Excel", false, true);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextInHyperlinkTest()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText("test ");
            editor.InsertHyperlink("http://www.telerik.com", "http://www.telerik.com", false);
            editor.InsertText(" replace");

            editor.ReplaceText("test http", "http");
            editor.ReplaceText("com replace", "com");
            editor.ReplaceText("telerik", "progress");

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextInEmailTest()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText("This is a test email: ");
            editor.InsertHyperlink("test@test.com", "test@test.com", false);
            editor.InsertText(" - belongs to Test User");

            editor.ReplaceText(new Regex(@"\w+@\w+.([\w+]{2,4})"), "test_user@testinguser.com");
            editor.ReplaceStyling(new Regex(@"\w+@\w+.([\w+]{2,4})"), (props) =>
            {
                props.HighlightColor.LocalValue = System.Windows.Media.Colors.Yellow;
            });

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceNumbersTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "JohnGrisham.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Regex regex = new Regex("1984");
            editor.ReplaceText(regex, "2000");
            editor.ReplaceText("1989", "2015");
            editor.ReplaceText("275 million", "275 000 000");

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextAndVerifyCharacterPropertiesDefinedInDocumentArePersistedTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "DocumentWithShadingPatternAndShading.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Regex regex = new Regex(@"([A-Z])\w+");
            editor.ReplaceStyling(regex, (props) =>
            {
                props.HighlightColor.LocalValue = Colors.Yellow;
            });

            editor.ReplaceText("Video", "Presentation");

            editor.ReplaceStyling("Presentation", (props) =>
            {
                props.HighlightColor.LocalValue = Colors.Gray;
            });

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextAndThenReplaceStylingForSmallLettersUsingRegexTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "JohnGrisham.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Regex regex = new Regex("[a-z]");
            editor.ReplaceText("John", "William", true, true);
            editor.ReplaceText("sources", "SOURCES", true, true);
            editor.ReplaceStyling(regex, GetReplacePropertiesHandler());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(1), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceStylingFirstThenReplaceTextInADocumentWithStylesTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "SampleDocument.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Regex regex = new Regex(".");
            editor.ReplaceStyling(regex, GetReplacePropertiesHandler());

            editor.ReplaceText("RadWordsProcessing", "Replaced_RadWordsProcessing", false, true);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceStylingInADocumentUsingRegexTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "JohnGrisham.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Regex regex = new Regex(@"[0-9]");
            editor.ReplaceText("Book", "Video game", true, false);
            editor.ReplaceText("book", "video game", true, false);
            editor.ReplaceStyling(regex, GetReplacePropertiesHandler());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, UIInteractionHelper.PageDown(1), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceTextInProtectedDocumentTest()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile("VisualTestsDocuments/" + "DocumentWithProtection.docx", new DocxFormatProvider());
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.ReplaceText("here", "THERE", true, true);
            editor.ReplaceText("there", "THERE", true, true);
            editor.ReplaceText("Here", "THERE", true, true);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.FindAndReplace)]
        public void ReplaceStylingWhichAppliesNewCharacterPropertiesTest()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText("Highlightcolor test");

            editor.ReplaceStyling("highlightcolor", false, true, (props) =>
                {
                    props.HighlightColor.LocalValue = Colors.Yellow;
                    props.FontSize.LocalValue = 72;
                    props.ForegroundColor.LocalValue = new ThemableColor(Colors.Blue);
                    props.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
                    props.UnderlineColor.LocalValue = new ThemableColor(Colors.Red);
                    props.UnderlinePattern.LocalValue = UnderlinePattern.Double;
                });

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        private static Action<CharacterProperties> GetReplacePropertiesHandler()
        {
            return (props) =>
            {
                props.HighlightColor.LocalValue = Colors.Yellow;
            };
        }
    }
}
