﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !WPF && !SILVERLIGHT
using Telerik.Documents.Common.Model;
#else
using Telerik.Windows.Documents.Spreadsheet.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport
{
    public abstract class HtmlTestBase : TestsBase
    {
        private const string TestFilesFolder = "Html";
        private readonly StylePropertiesAsserterInfo asserterInfo;

        public HtmlTestBase()
        {
            this.asserterInfo = new StylePropertiesAsserterInfo();

            this.asserterInfo.RegisterPropertiesThatShouldNotAssertHasLocalValue(new string[]
            {
                "TableCellPadding",
            });
        }

        protected StylePropertiesAsserterInfo AsserterInfo
        {
            get
            {
                return this.asserterInfo;
            }
        }

        protected abstract string FolderName { get; }

        protected RadFlowDocument GetExpectedDocumentForImport(bool addNormalWebStyle = true)
        {
            RadFlowDocument expected = new RadFlowDocument();
            ModifyBuiltInStyles(expected);

            if (addNormalWebStyle)
            {
                Style normalWebStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.NormalWebStyleId);
                normalWebStyle.CharacterProperties.FontFamily.ClearValue();
                normalWebStyle.CharacterProperties.FontSize.ClearValue();
                normalWebStyle.ParagraphProperties.SpacingAfter.LocalValue = 6.66;
                normalWebStyle.ParagraphProperties.AutomaticSpacingAfter.LocalValue = true;
                normalWebStyle.ParagraphProperties.SpacingBefore.LocalValue = 6.66;
                normalWebStyle.ParagraphProperties.AutomaticSpacingBefore.LocalValue = true;
            }

            return expected;
        }

        protected RadFlowDocument GetExpectedDocumentForExportImport(bool addNormalWebStyle = true)
        {
            RadFlowDocument expected = new RadFlowDocument();
            ModifyBuiltInStyles(expected);

            if (addNormalWebStyle)
            {
                expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.NormalWebStyleId);
            }

            return expected;
        }

        protected static void ModifyBuiltInStyles(RadFlowDocument document)
        {
            document.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Times New Roman");
            document.DefaultStyle.CharacterProperties.FontSize.ClearValue();
            document.DefaultStyle.ParagraphProperties.SpacingAfter.ClearValue();
            document.DefaultStyle.ParagraphProperties.LineSpacing.ClearValue();
            document.DefaultStyle.ParagraphProperties.LineSpacingType.ClearValue();

            document.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId).CharacterProperties.FontSize.LocalValue = 16;
        }

        protected RadFlowDocument ImportHtml(string fileName)
        {
            return TestHelper.GetDocumentFromFile(string.Format("{0}/{1}/{2}", TestFilesFolder, this.FolderName, fileName), new HtmlFormatProvider());
        }

        protected RadFlowDocument ImportHtmlFromString(string html)
        {
            return new HtmlFormatProvider().Import(html);
        }

        protected override RadFlowDocument ExportImportDocument(RadFlowDocument original, IFormatProvider<RadFlowDocument> provider = null)
        {
            if (provider == null)
            {
                provider = new HtmlFormatProvider();
            }

            return base.ExportImportDocument(original, provider);
        }

        protected void AssertDocuments(RadFlowDocument expected, RadFlowDocument actual, bool assertOnlyActualValuesOnBaseStyles = false)
        {
            StructureAsserter.AssertDocumentsAreEqual(expected, actual);

            StylePropertyAsserter.AssertStylesAreEqual(expected, actual, assertOnlyActualValuesOnBaseStyles);
            StylePropertyAsserter.AssertPropertiesAreEqual(expected.DefaultStyle.CharacterProperties, actual.DefaultStyle.CharacterProperties);
            StylePropertyAsserter.AssertPropertiesAreEqual(expected.DefaultStyle.ParagraphProperties, actual.DefaultStyle.ParagraphProperties);

            StylePropertyAsserter.AssertListsAreEqual(expected, actual);

            List<IElementWithProperties> expectedElements = expected.EnumerateChildrenOfType<DocumentElementBase>().Where(e => e is IElementWithProperties).Select(e => (IElementWithProperties)e).ToList();
            List<IElementWithProperties> actualElements = actual.EnumerateChildrenOfType<DocumentElementBase>().Where(e => e is IElementWithProperties).Select(e => (IElementWithProperties)e).ToList();


            for (int i = 0; i < expectedElements.Count; i++)
            {
                //StylePropertyAsserter.AssertPropertiesAreEqual(expectedElements[i], actualElements[i], this.AsserterInfo);
                StylePropertyAsserter.AssertPropertiesAreEqual(expectedElements[i], actualElements[i], null);
            }
        }

        protected void AssertDocumentsStructureOnly(RadFlowDocument expected, RadFlowDocument actual)
        {
            StructureAsserter.AssertDocumentsAreEqual(expected, actual);
        }

        protected void AssertDocumentStructureAndProperties(RadFlowDocument expected, RadFlowDocument actual)
        {
            //StylePropertyAsserter.AssertStylesAreEqual(expected, actual);
            StylePropertyAsserter.AssertPropertiesAreEqual(expected.DefaultStyle.CharacterProperties, actual.DefaultStyle.CharacterProperties, this.AsserterInfo);
            StylePropertyAsserter.AssertPropertiesAreEqual(expected.DefaultStyle.ParagraphProperties, actual.DefaultStyle.ParagraphProperties, this.AsserterInfo);

            List<IElementWithProperties> originalElements = expected.EnumerateChildrenOfType<DocumentElementBase>().Where(e => e is IElementWithProperties).Select(e => (IElementWithProperties)e).ToList();
            List<IElementWithProperties> exportedElements = actual.EnumerateChildrenOfType<DocumentElementBase>().Where(e => e is IElementWithProperties).Select(e => (IElementWithProperties)e).ToList();

            for (int i = 0; i < originalElements.Count; i++)
            {
                StylePropertyAsserter.AssertPropertiesAreEqual(originalElements[i], exportedElements[i], this.AsserterInfo);
            }
        }
    }
}
