﻿using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
#if VISUAL
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
#endif
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Lists
{
    [TestClass]
    public class ExportImportTests : ListTestsBase
    {
        [TestMethod]
        public void ListWithNineIncreadinglyNestedLevelsEachLevelWithTwoItemsFollowedByNineDecreasinglyNestedLevelsEachWithTwoItems_IsCorrectlyExportedAndImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            this.AddParagraphsToDocument(editor, list, 0, 2);

            for (int i = 1; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i, 1);
                this.AddParagraphsToDocument(editor, list, i, 2);
            }

            for (int i = 8; i >= 0; i--)
            {
                this.AddParagraphsToDocument(editor, list, i, 1);
                this.AddParagraphsToDocument(editor, list, i, 2);
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void ListWithNineIncreasinglyNestedLevelesFollowedByNineIncreasinglyNestedLevels_IsCorrectlyExportedAndImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            editor.InsertText("Simple paragraph");

            for (int i = 0; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            for (int i = 0; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void ListWithNestedLevelsWithSkeepingTheSecondAndThirdLevels_IsCorrectlyExportedAndImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            this.AddParagraphsToDocument(editor, list, 0);

            for (int i = 3; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void ListWithNestedLevelsWithSkeepingTheFirstLevel_IsCorrectlyExportedAndImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            for (int i = 0; i < 9; i++)
            {
                editor.InsertParagraph();
                editor.InsertText("Separator");

                for (int j = i; j < 9; j++)
                {
                    this.AddParagraphsToDocument(editor, list, j, 0);
                }
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);

            List<Paragraph> expectedParagraphsList = expected.EnumerateChildrenOfType<Paragraph>().ToList();
            List<Paragraph> importedParagraphsList = imported.EnumerateChildrenOfType<Paragraph>().ToList();

            for (int i = 0; i < expectedParagraphsList.Count; i++)
            {
                Assert.AreEqual(expectedParagraphsList[i].Properties.ListLevel.LocalValue, importedParagraphsList[i].Properties.ListLevel.LocalValue);
            }
        }

        [TestMethod]
        public void ListWithSingleLevel_WhenParagraphMarkerPropertiesAreDefined_ListLevelGetMarkerProperties_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 36;
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.ListId = list.Id;
            p.ListLevel = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithSingleLevel_WhenParagraphMarkerPropertiesAreDefinedAndListLevelHasLocalCharProperties_LocalCharacterAndMarkerPropertiesAreApplied_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            ListLevel listLevel = list.Levels[0];
            listLevel.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 36;
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.ListId = list.Id;
            p.ListLevel = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // We import the list level style properties as ParagraphMarkerProperties for the corresponding paragraph
            p.Properties.ParagraphMarkerProperties.CopyPropertiesFrom(listLevel.CharacterProperties);
            foreach (IStyleProperty property in listLevel.CharacterProperties.StyleProperties)
            {
                property.ClearValue();
            }

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithSingleLevel_WhenListLevelHasLocalCharProperties_PropertiesAreImportedAsMarkerPropertiesToTheAssociatedParagraph_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            ListLevel listLevel = list.Levels[0];
            listLevel.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);
            listLevel.CharacterProperties.FontSize.LocalValue = 36;
            listLevel.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            listLevel.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.ListId = list.Id;
            p.ListLevel = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // We import the list level style properties as ParagraphMarkerProperties for the corresponding paragraph
            p.Properties.ParagraphMarkerProperties.CopyPropertiesFrom(listLevel.CharacterProperties);
            foreach (IStyleProperty property in listLevel.CharacterProperties.StyleProperties)
            {
                property.ClearValue();
            }

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithMultipleLevels_WhenDifferentAssociatedParagraphsHaveDifferentMarkerProperties_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertParagraph();
            editor.InsertText("0-1");
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 36;
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.ListId = list.Id;
            p.ListLevel = 0;

            p = editor.InsertParagraph();
            editor.InsertText("1-1");
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.ListId = list.Id;
            p.ListLevel = 0;

            p = editor.InsertParagraph();
            editor.InsertText("2-1");
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.ListId = list.Id;
            p.ListLevel = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithNestedLevels_WhenDifferentAssociatedParagraphsHaveDifferentMarkerProperties_ExportImport()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            List<Action<CharacterProperties>> paragraphMarkerPropertyActions = new List<Action<CharacterProperties>>()
            {
                new Action<CharacterProperties>((props)=>
                    {
                        props.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
                        props.FontSize.LocalValue = 30;
                        props.FontFamily.LocalValue = new ThemableFontFamily("Comic Sans MS");
                    }),
                new Action<CharacterProperties>((props)=>
                    {
                        props.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);
                        props.FontSize.LocalValue = 15;
                        props.FontStyle.LocalValue = FontStyles.Italic;
                    }),
                new Action<CharacterProperties>((props)=>
                    {
                        props.ForegroundColor.LocalValue = new ThemableColor(Colors.Blue);
                        props.FontFamily.LocalValue = new ThemableFontFamily("Arial");
                    }),
                new Action<CharacterProperties>((props)=>
                    {
                        props.ForegroundColor.LocalValue = new ThemableColor(Colors.Violet);
                        props.FontSize.LocalValue = 36;
                        props.BackgroundColor.LocalValue = new ThemableColor(Colors.Yellow);
                        props.FontWeight.LocalValue = FontWeights.Bold;
                    }),
            };

            for (int i = 0; i < 9; i++)
            {
                editor.InsertParagraph();
                editor.InsertText("Separator");

                for (int j = i; j < 9; j++)
                {
                    AddParagraphsWithMarkerPropertiesToDocument(editor, list, j, paragraphMarkerPropertyActions);
                }
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);

            List<Paragraph> expectedParagraphsList = expected.EnumerateChildrenOfType<Paragraph>().ToList();
            List<Paragraph> importedParagraphsList = imported.EnumerateChildrenOfType<Paragraph>().ToList();

            for (int i = 0; i < expectedParagraphsList.Count; i++)
            {
                Assert.AreEqual(expectedParagraphsList[i].Properties.ListLevel.LocalValue, importedParagraphsList[i].Properties.ListLevel.LocalValue);

                StructureAsserter.AssertParagraphsAreEqual(expectedParagraphsList[i], importedParagraphsList[i]);

                StylePropertyAsserter.AssertPropertiesAreEqual(expectedParagraphsList[i].Properties.ParagraphMarkerProperties,
                                                               expectedParagraphsList[i].Properties.ParagraphMarkerProperties);
            }
        }

        [TestMethod]
        public void ListWithNestedLevels_WhenStyleSetToFirstLevel_ExportImport()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            ThemableColor color = new ThemableColor(Colors.Red);

            list.Levels[0].CharacterProperties.ForegroundColor.LocalValue = color;

            this.AddParagraphsToDocument(editor, list, 0);
            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 2);
            this.AddParagraphsToDocument(editor, list, 2);
            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 0);
            this.AddParagraphsToDocument(editor, list, 0);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // Assert
            for (int level = 0; level < 3; level++)
            {
                foreach (Paragraph paragraph in imported.EnumerateChildrenOfType<Paragraph>())
                {
                    if (paragraph.ListLevel == level)
                    {
                        StylePropertyAsserter.AssertPropertiesAreEqual(list.Levels[level].CharacterProperties, paragraph.Properties.ParagraphMarkerProperties);
                    }
                }
            }
        }

        [TestMethod]
        public void ListWithMoreThanEightParagraphs_ExportImport()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            for (int i = 0; i < 20; i++)
            {
                this.AddParagraphsToDocument(editor, list, 0);
            }

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            // Assert
            this.AssertDocuments(expected, imported);
        }

#if VISUAL
        // TeamPulse bug ID: 143210
        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void VerifyThatListItemsAreExportedWithCorrectStylingToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Section section = document.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("Simple paragraph before the list");

            for (int i = 0; i < 4; i++)
            {
                p = section.Blocks.AddParagraph();
                p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 30;
                p.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
                p.ListId = list.Id;
                p.ListLevel = 0;
                p.Inlines.AddRun("Hello " + 0);
            }

            AssertVisual(document, new string[] { DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void Verify()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile("VisualTestsDocuments/ListItemsWithStylingExport.html", new HtmlFormatProvider());

            AssertVisual(document, new string[] { HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse ID: 154309, TFS ID: 234099")]
        public void ListLevelContainingEmptyLineBeforeParagraphShouldImportOnlyOneParagraph()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile("VisualTestsDocuments/list.html", new HtmlFormatProvider());

            AssertVisual(document, new string[] { DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse ID: 154312, TFS ID: 234103")]
        public void AllListLevelChildParapgraphsHaveSameIndentation()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile("VisualTestsDocuments/list2.html", new HtmlFormatProvider());

            AssertVisual(document, new string[] { DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }
        
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse ID: 190891, TFS ID: 250198")]
        public void WhenHtmlIsNotWellFormattedExtraEmptyParagraphIsNotAddedTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("WrongIndentDueToMissingSpace.html", new HtmlFormatProvider());

            AssertVisual(document, new string[] { DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse ID: 154312, TFS ID: 234103")]
        public void VerifyListItemIsWithCorrectIndentation()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("ListImport.html", new HtmlFormatProvider());

            AssertVisual(document, new string[] { DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }

#endif

        private static void AddParagraphsWithMarkerPropertiesToDocument(RadFlowDocumentEditor editor, List list, int index, List<Action<CharacterProperties>> propertyActions)
        {
            Paragraph p = null;
            for (int i = index; i < index + 1; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                p.StyleId = BuiltInStyleNames.NormalWebStyleId;
                editor.InsertText(i + "-" + 0);

                int propertyActionIndex = (index + propertyActions.Count) % propertyActions.Count;
                propertyActions[propertyActionIndex](p.Properties.ParagraphMarkerProperties);
            }
        }
    }
}
