﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Lists
{
    [TestClass]
    public class ListStructureImportTests : ListTestsBase
    {
        [TestMethod]
        public void SimpleList_IsImportedWithThreeElementsInTheFirstLevel()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("0-2");

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("0-3");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><ol><li>0-1</li><li>0-2</li><li>0-3</li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWihtoutOpeningListElement_IsImportedAsRegularListWithThreeElementsInTheFirstLevel()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("0-2");

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("0-3");

            p = editor.InsertParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("text");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<li>0-1</li><li>0-2</li><li>0-3</li><p>text</p>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithNineNestedLevels_AreListLevelsImportedInTheCorrectOrder()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            for (int i = 1; i < list.Levels.Count; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            // Act
            RadFlowDocument imported = this.ImportHtml("NineNestedLevels.html");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithTenNestedLevels_AreElementsImportedAsNineLevelsAndTwoElementsInTheLastLevel()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            for (int i = 1; i < list.Levels.Count; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            this.AddParagraphsToDocument(editor, list, 8, 2);

            // Act
            RadFlowDocument imported = this.ImportHtml("TenNestedLevels.html");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithNineLevelsFollowedByAnotherNineNestedLevels_AreListLevelsImportedInTheCorrectOrder()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            for (int i = 1; i < list.Levels.Count; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            for (int i = 0; i < list.Levels.Count; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            // Act
            RadFlowDocument imported = this.ImportHtml("NineLevelsFollowedByAnotherNineNestedLevels.html");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithTwoItemsForEachLevelToTheThirdLevelAndThenTwoItemsToTheFirstLevel_AreListLevelsImportedInTheCorrectOrder()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            this.AddParagraphsToDocument(editor, list, 0, 2);

            this.AddParagraphsToDocument(editor, list, 1, 1);
            this.AddParagraphsToDocument(editor, list, 1, 2);

            this.AddParagraphsToDocument(editor, list, 2, 1);
            this.AddParagraphsToDocument(editor, list, 2, 2);

            this.AddParagraphsToDocument(editor, list, 1, 3);
            this.AddParagraphsToDocument(editor, list, 1, 4);

            this.AddParagraphsToDocument(editor, list, 0, 3);
            this.AddParagraphsToDocument(editor, list, 0, 4);

            // Act
            RadFlowDocument imported = this.ImportHtml("ListWithTwoItemsForEachLevelToTheThirdLevelAndThenTwoItemsToTheFirstLevel.html");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithNoItemsForTheFirstAndSecondListLevelIndexesFollowedByOnItemForEachOfTheOtherLevels_AreListLevelsImportedInTheCorrectOrder()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            for (int i = 3; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i, 1);
            }

            // Act
            RadFlowDocument imported = this.ImportHtml("ListWithNoItemsForTheFirstAndSecondListLevelIndexesFollowedByOnItemForEachOfTheOtherLevels.html");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListLevelWithNoBulletAndNoTextInIt_IsImportedAsListLevelAsBulletAndEmptyParagraph()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            this.AddParagraphsToDocument(editor, list, 1, 1);
            this.AddParagraphsToDocument(editor, list, 1, 2);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><ol><li>0-1</li><li style=\"list-style-type: none;\"><ol><li>1-1</li><li>1-2</li></ol></li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListLevelWithNoBulletAndTextInIt_IsImportedAsListLevelAsBulletAndParagraphWithText()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("0-2");

            this.AddParagraphsToDocument(editor, list, 1, 1);
            this.AddParagraphsToDocument(editor, list, 1, 2);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><ol><li>0-1</li><li style=\"list-style-type: none;\">0-2<ol><li>1-1</li><li>1-2</li></ol></li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListLevelWithNoBulletTheTextInItIsAfterTheNestedLevel_IsImportedAsListLevelWithBulleteEmptyParagraphAndAdditionalParagraphWithTheText()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            this.AddParagraphsToDocument(editor, list, 1, 1);
            this.AddParagraphsToDocument(editor, list, 1, 2);

            p = editor.InsertParagraph();
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            editor.InsertText("ParagraphWithNoList");

            this.AddParagraphsToDocument(editor, list, 0, 3);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><ol><li>0-1</li><li style=\"list-style-type: none;\"><ol><li>1-1</li><li>1-2</li></ol>ParagraphWithNoList</li><li>0-3</li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithSingleListLevel_WhenContainsSingleParagraphAsListLevelContent_IsImportedContainingParagraph()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("paragraph text").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act 
            RadFlowDocument imported = this.ImportHtmlFromString(@"<ol>
        <li>
            <p>paragraph text</p>
        </li>
    </ol>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void List_WhenContainsTextAndParagraphsAsListLevelContent_HasProperlyImportedListLevelsAndParagraphsWithIndentationAfterThat()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("test").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            Run r = p.Inlines.AddRun("paragraph text");
            editor.MoveToInlineEnd(r);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("paragraph text");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("another test");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("yet another test");
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act 
            RadFlowDocument imported = this.ImportHtmlFromString(@"<ol>
        <li>test
            <p>paragraph text</p>
            <p>paragraph text</p>
            another test
            <p>yet another test</p>
        </li>
    </ol>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListWithTwoItems_WhenContainsTextAndParagraphsAsListLevelContent_HasProperlyImportedListLevelsWithCorrectNumberingAndParagraphsAfterThat()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("test").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("first p");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("second p");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("another");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("yet another");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("level p");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("third p");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = 48;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act 
            RadFlowDocument imported = this.ImportHtmlFromString(@"<html>
<body>
    <ol>
		<li>
            test
            <p>first p</p>
			<p>second p</p>
			another
			<p>yet another</p>
        </li>
        <li>
            <p>level p</p>
			<p>third p</p>
        </li>
    </ol>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void NestedLists_WhenContainMultipleParagraphsAsListLevelContent_HasProperLeftIndentationAppliedOnTheNestedParagraphs()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("par p").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("test");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.ListId = list.Id;
            p.ListLevel = 1;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("nested p");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.Indentation.LeftIndent = list.Levels[1].ParagraphProperties.LeftIndent.GetActualValue().Value;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act 
            RadFlowDocument imported = this.ImportHtmlFromString(@"<html>
<body>
    <ol>
		<li>
			<p>par p</p>
        </li>
        <ol>
		    <li>
                test
			    <p>nested p</p>
            </li>
        </ol>
    </ol>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void List_WhenContainsParagraphElementAndHtmlHasNoSpacesBeforeItInHtmlString_IsProperlyImportedOnlySingleParagraph()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("test").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"<ol><li><p>test</p></li></ol>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void List_WhenContainsTextElementDirectlyAndHtmlHasNoSpacesBeforeItInHtmlString_IsProperlyImportedOnlySingleParagraph()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("test").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"<ol><li>test</li></ol>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void ListLevel_WhenIsInNestedList_IsProperlyImportedWithoutExtraEmptyParagraph()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            Paragraph p = editor.InsertText("test").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("another test");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.ListId = list.Id;
            p.ListLevel = 1;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p = editor.InsertParagraph();
            p.Inlines.AddRun("last paragraph");
            editor.MoveToInlineEnd(p.Inlines[0]);
            p.ListId = list.Id;
            p.ListLevel = 1;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@" <ol>
                    <li>
                        <p>
                           test
                        </p>
                        <ol>
                            <li>
                                <p>
                                    another

                                    test
                                </p>
                            </li>
                            <li>
                                <p>last paragraph</p>
                            </li>
                        </ol>
                    </li>
            </ol>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        // Uncomment when item TP: #199805 TFS: #253565 is fixed.
        //[TestMethod]
        public void Import_ListWithEmptyListLevel_IsImportedAsEmptyParagraphInAList()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            List list = this.GetList(expected);

            Paragraph firstParagraph = expected.Sections.AddSection().Blocks.AddParagraph();
            firstParagraph.ListId = list.Id;
            firstParagraph.ListLevel = 0;
            firstParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(
@"<html>
	<ol type=""1"">
        <li></li>
    </ol>
</html>
");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Import_ListWithListLevelWithMultipleChildrenFollowedByAParagraph_IsImportedAsParagraphInListFollowedByParagraphNotInAList()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            List list = this.GetList(expected);

            Section firstSection = expected.Sections.AddSection();
            Paragraph firstParagraph = firstSection.Blocks.AddParagraph();
            firstParagraph.ListId = list.Id;
            firstParagraph.ListLevel = 0;
            firstParagraph.Inlines.AddRun("text ");
            firstParagraph.Inlines.AddRun("bold").FontWeight = FontWeights.Bold;
            firstParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            firstSection.Blocks.AddParagraph().Inlines.AddRun("After").Paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(
@"<html>
	<ol type=""1"">
        <li>text <b>bold</b></li>
    </ol>
    <p>After</p>
</html> ");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Import_ParagraphInListFollowingNestedList_IsImportedAsParagraphInTheOuterListWithProperIdentation()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            List list = this.GetList(expected);
            list.Levels[1].NumberingStyle = NumberingStyle.LowerLetter;

            Section firstSection = expected.Sections.AddSection();
            Paragraph firstParagraph = firstSection.Blocks.AddParagraph();
            firstParagraph.ListId = list.Id;
            firstParagraph.ListLevel = 0;
            firstParagraph.Inlines.AddRun("Before");
            firstParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Paragraph secondParagraph = firstSection.Blocks.AddParagraph();
            secondParagraph.ListId = list.Id;
            secondParagraph.ListLevel = 1;
            secondParagraph.Inlines.AddRun("Nested list item");
            secondParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Paragraph thirdParagraph = firstSection.Blocks.AddParagraph();
            thirdParagraph.Indentation.LeftIndent = list.Levels[0].ParagraphProperties.LeftIndent.GetActualValue().Value;
            thirdParagraph.Inlines.AddRun("After");
            thirdParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Paragraph fourthParagraph = firstSection.Blocks.AddParagraph();
            fourthParagraph.Inlines.AddRun("test");
            fourthParagraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(
@"<html>
	<ol type=""1"">
		<li>
			Before
			<ol type=""a"">
				<li>Nested list item</li>
			</ol>
			After
		</li>
	</ol>
<p>test</p>
</html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}