﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Lists
{
    public class ListTestsBase : HtmlTestBase
    {
        private readonly Dictionary<string, NumberingStyle> numberingStyles = new Dictionary<string, NumberingStyle>();

        public ListTestsBase()
        {
            this.numberingStyles.Add("circle", NumberingStyle.Bullet);
            this.numberingStyles.Add("square", NumberingStyle.Bullet);
            this.numberingStyles.Add("disc", NumberingStyle.Bullet);
            this.numberingStyles.Add("lower-greek", NumberingStyle.Decimal);
            this.numberingStyles.Add("lower-alpha", NumberingStyle.LowerLetter);
            this.numberingStyles.Add("upper-alpha", NumberingStyle.UpperLetter);
            this.numberingStyles.Add("lower-roman", NumberingStyle.LowerRoman);
            this.numberingStyles.Add("upper-roman", NumberingStyle.UpperRoman);
            this.numberingStyles.Add("lower-latin", NumberingStyle.LowerLetter);
            this.numberingStyles.Add("upper-latin", NumberingStyle.UpperLetter);
            this.numberingStyles.Add("decimal", NumberingStyle.Decimal);
        }

        protected virtual Dictionary<string, NumberingStyle> NumberingStyles
        {
            get
            {
                return this.numberingStyles;
            }
        }

        protected override string FolderName
        {
            get
            {
                return "Lists";
            }
        }

        protected void AddParagraphsToDocument(RadFlowDocumentEditor editor, List list, int index, int serialIndexOfListLevelElement = 1)
        {
            Paragraph p = null;
            for (int i = index; i < index + 1; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                p.StyleId = BuiltInStyleNames.NormalWebStyleId;
                editor.InsertText(i + "-" + serialIndexOfListLevelElement);
            }
        }

        protected List GetList(RadFlowDocument expected)
        {
            List list = expected.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            return list;
        }
    }
}
