﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Lists
{
    [TestClass]
    public class NumberingStyleTests : ListTestsBase
    {
        private void ArrangeActAndAssertOnCorrectImportOfNumberingStyleProperty(string htmlNumberingStyle, NumberingStyle numberingStyle)
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = numberingStyle;
            if (numberingStyle == NumberingStyle.Bullet)
            {
                list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
                if (htmlNumberingStyle == "circle")
                {
                    list.Levels[0].NumberTextFormat = "o";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Courier New"));
                }
                else if (htmlNumberingStyle == "square")
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Wingdings"));
                }
                else
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Symbol"));
                }
            }

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><ol><li style=\"list-style-type: " + htmlNumberingStyle + ";\">first item</li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);

            // Act
            imported = this.ImportHtmlFromString("<html><body><ol style=\"list-style-type: " + htmlNumberingStyle + ";\"><li>first item</li></ol></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleList_IsFirstLevelWithCorrectNumberingStyle()
        {
            foreach (KeyValuePair<string, NumberingStyle> pair in this.NumberingStyles)
            {
                this.ArrangeActAndAssertOnCorrectImportOfNumberingStyleProperty(pair.Key, pair.Value);
            }
        }

        private void ArrangeActAndAssertOnImportOnStyledHtmlListElement(string listElementName, string htmlNumberingStyle, NumberingStyle numberingStyle)
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = numberingStyle;
            if (numberingStyle == NumberingStyle.Bullet)
            {
                list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
                if (htmlNumberingStyle == "circle")
                {
                    list.Levels[0].NumberTextFormat = "o";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Courier New"));
                }
                else if (htmlNumberingStyle == "square")
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Wingdings"));
                }
                else
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Symbol"));
                }
            }

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><style type=\"text/css\">" + listElementName + " { list-style-type: " + htmlNumberingStyle + "; }</style><body><" + listElementName + "><li>first item</li></" + listElementName + "></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleListWithStyledUlElement_IsImportedWithCorrectNumberedStyle()
        {
            foreach (KeyValuePair<string, NumberingStyle> pair in this.NumberingStyles)
            {
                this.ArrangeActAndAssertOnImportOnStyledHtmlListElement("ul", pair.Key, pair.Value);
            }
        }

        [TestMethod]
        public void SimpleListWithStyledOlElement_IsImportedWithCorrectNumberedStyle()
        {
            foreach (KeyValuePair<string, NumberingStyle> pair in this.NumberingStyles)
            {
                this.ArrangeActAndAssertOnImportOnStyledHtmlListElement("ol", pair.Key, pair.Value);
            }
        }

        [TestMethod]
        public void BulletedListWithThreeNestedLevels_IsImportedWithCorrectNumberedStyle()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            for (int i = 0; i < 3; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Bullet;
                list.Levels[i].NumberTextFormat = "";
                list.Levels[i].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Symbol"));
            }

            Paragraph p = editor.InsertText("0-1").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            this.AddParagraphsToDocument(editor, list, 1);
            this.AddParagraphsToDocument(editor, list, 2);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<ul><li>0-1</li><ul><li>1-1</li><ul><li>2-1</li></ul></ul></ul>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleOrderedList_WhenHasNeitherAttributesOrStyle_IsImportedWithCorrectDecimalNumberingStyle()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><body><ol><li>first item</li></ol></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleUnorderedList_WhenHasNeitherAttributesOrStyle_IsImportedWithCorrectBulletNumberingStyle()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
            list.Levels[0].NumberTextFormat = "";
            list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Symbol"));

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><body><ul><li>first item</li></ul></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleOrderedList_WhenHasTypeAttributesAndListStyleType_IsImportedWithListStyleTypeNumbering()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = NumberingStyle.UpperRoman;

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = @"<html><body><ol style=""list-style-type:upper-roman;"" type=""1""><li>first item</li></ol></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleUnorderedList_WhenHasTypeAttributesAndListStyleType_IsImportedWithListStyleTypeNumbering()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
            list.Levels[0].NumberTextFormat = "";
            list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Wingdings"));

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = @"<html><body><ul style=""list-style-type:square;"" type=""circle""><li>first item</li></ul></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void OrderedList_WhenHasListStyleTypeValueForUnorderedList_IsImportedWithBulletNumberingStyle()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
            list.Levels[0].NumberTextFormat = "";
            list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Wingdings"));

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = @"<html><body><ol style=""list-style-type:square;"" type=""circle""><li>first item</li></ol></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void UnorderedList_WhenHasListStyleTypeValueForOrderedList_IsImportedWithNumberingStyleFromListStyleType()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = NumberingStyle.UpperRoman;

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = @"<html><body><ul style=""list-style-type:upper-roman;"" type=""1""><li>first item</li></ul></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
