﻿using System.Collections.Generic;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Lists
{
    [TestClass]
    public class TypeAttributeTests : ListTestsBase
    {
        private readonly Dictionary<string, NumberingStyle> typeNumberingStyles = new Dictionary<string, NumberingStyle>();

        public TypeAttributeTests()
        {
            this.typeNumberingStyles.Add("circle", NumberingStyle.Bullet);
            this.typeNumberingStyles.Add("square", NumberingStyle.Bullet);
            this.typeNumberingStyles.Add("disc", NumberingStyle.Bullet);
            this.typeNumberingStyles.Add("1", NumberingStyle.Decimal);
            this.typeNumberingStyles.Add("a", NumberingStyle.LowerLetter);
            this.typeNumberingStyles.Add("A", NumberingStyle.UpperLetter);
            this.typeNumberingStyles.Add("i", NumberingStyle.LowerRoman);
            this.typeNumberingStyles.Add("I", NumberingStyle.UpperRoman);
        }

        protected override Dictionary<string, NumberingStyle> NumberingStyles
        {
            get
            {
                return this.typeNumberingStyles;
            }
        }

        private void ArrangeActAndAssertOnImportHtmlListElementWithTypeAttribute(string listElementName, string htmlNumberingStyle, NumberingStyle numberingStyle)
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            List list = this.GetList(expected);
            list.Levels[0].NumberingStyle = numberingStyle;
            if (numberingStyle == NumberingStyle.Bullet)
            {
                list.Levels[0].NumberingStyle = NumberingStyle.Bullet;
                if (htmlNumberingStyle == "circle")
                {
                    list.Levels[0].NumberTextFormat = "o";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Courier New"));
                }
                else if (htmlNumberingStyle == "square")
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Wingdings"));
                }
                else
                {
                    list.Levels[0].NumberTextFormat = "";
                    list.Levels[0].CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Symbol"));
                }
            }

            Paragraph p = editor.InsertText("first item").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><body><" + listElementName + " type=" + htmlNumberingStyle + "><li>first item</li></" + listElementName + "></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void SimpleOrderedList_WhenHasTypeAttribute_IsImportedWithCorrectNumberingStyle()
        {
            foreach (KeyValuePair<string, NumberingStyle> pair in this.NumberingStyles)
            {
                this.ArrangeActAndAssertOnImportHtmlListElementWithTypeAttribute("ol", pair.Key, pair.Value);
            }
        }

        [TestMethod]
        public void SimpleUnorderedList_WhenHasTypeAttribute_IsImportedWithCorrectNumberingStyle()
        {
            foreach (KeyValuePair<string, NumberingStyle> pair in this.NumberingStyles)
            {
                this.ArrangeActAndAssertOnImportHtmlListElementWithTypeAttribute("ul", pair.Key, pair.Value);
            }
        }
    }
}
