﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public abstract class BoldStyledTagTestsBase : ParagraphTestsBase
    {
        public BoldStyledTagTestsBase()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Run.FontWeightPropertyDefinition
            });
        }

        protected abstract string Tag { get; }

        [TestMethod]
        public void Test_BoldStyledTag_AsChild_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test").FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><" + this.Tag + ">test</" + this.Tag + "></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_BoldStyledTag_AsParent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section section = expected.Sections.AddSection();

            Paragraph paragraph = section.Blocks.AddParagraph();
            paragraph.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraph.Inlines.AddRun("first").FontWeight = FontWeights.Bold;

            paragraph = section.Blocks.AddParagraph();
            paragraph.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraph.Inlines.AddRun("second").FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><" + this.Tag + "><p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_BoldStyledTag_AsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").FontWeight = FontWeights.Bold;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Inlines.AddRun("first").FontWeight = FontWeights.Bold;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Inlines.AddRun("second").FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_BoldStyledTag_AsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").FontWeight = FontWeights.Bold;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Inlines.AddRun("first").FontWeight = FontWeights.Bold;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            p.Inlines.AddRun("second").FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }

    [TestClass]
    public class BoldTagTests : BoldStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "b";
            }
        }
    }

    [TestClass]
    public class StrongTagTests : BoldStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "strong";
            }
        }
    }
}
