﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class CenterTagTests : ParagraphTestsBase
    {
        public CenterTagTests()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Paragraph.TextAlignmentPropertyDefinition
            });
        }

        [TestMethod]
        public void Test_CenterAsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("first");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><center>first</center></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_CenterAsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("first");

            p = section.Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("second");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><center><p>first</p><p>second</p></center></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_CenterAsParentElementAndAdditionalText_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("first");

            p = section.Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("second");

            p = section.Blocks.AddParagraph();
            p.TextAlignment = Alignment.Center;
            p.Inlines.AddRun("third");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><center>first<p>second</p><p>third</p></center></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_CenterAsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("first");

            p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("second");
            p.TextAlignment = Alignment.Center;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>first<center>second</center></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_CenterWithParagraphBefore_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("first");

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("second");
            p.TextAlignment = Alignment.Center;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>first<center>second</center></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }
}
