﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class HyperlinkTests : ParagraphTestsBase
    {
        [TestMethod]
        public void Test_Hyperlink_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(true);
            this.AddDefaultHtmlHyperlinkStyle(expected);

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            editor.InsertHyperlink("hyperlink", "telerik.com", false, "Telerik Site");
            editor.Document.EnumerateChildrenOfType<Paragraph>().First().StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><body><p><a href=\"telerik.com\" title=\"Telerik Site\">hyperlink</a></p></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_Hyperlink_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            this.AddDefaultHtmlHyperlinkStyle(expected);

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            editor.InsertHyperlink("hyperlink", "telerik.com", false, "Telerik Site");

            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new HtmlFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void Test_Hyperlink_WithNoTitle_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(true);
            this.AddDefaultHtmlHyperlinkStyle(expected);

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            editor.InsertHyperlink("hyperlink", "telerik.com", false);
            editor.Document.EnumerateChildrenOfType<Paragraph>().First().StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            string html = "<html><body><p><a href=\"telerik.com\">hyperlink</a></p></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_Hyperlink_WithNoTitle_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            editor.InsertHyperlink("hyperlink", "telerik.com", false);

            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new HtmlFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void Test_MultiLineHyperlink_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(true);
            this.AddDefaultHtmlHyperlinkStyle(expected);

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);

            editor.InsertText("before");
            editor.InsertHyperlink("hyper", "telerik.com", false, "Telerik Site");
            editor.InsertParagraph();
            editor.InsertHyperlink("link", "telerik.com", false, "Telerik Site");
            editor.InsertText("after");

            editor.Document.EnumerateChildrenOfType<Paragraph>().ToList().ForEach(p => p.StyleId = BuiltInStyleNames.NormalWebStyleId);

            // Act
            string html = "<html><body><p>before<a href=\"telerik.com\" title=\"Telerik Site\">hyper</p><p>link</a>after</p></body></html>";
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_MultiLineHyperlink_ExportImport()
        {
            // Arrange
            RadFlowDocument export = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(export);

            editor.InsertText("before");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "link", "telerik.com", false, "Telerik Site");
            editor.InsertText("after");

            string html = new HtmlFormatProvider().Export(export);

            RadFlowDocument expected = new RadFlowDocument();
            editor = new RadFlowDocumentEditor(expected);

            editor.InsertText("before");
            editor.InsertHyperlink("hyper", "telerik.com", false, "Telerik Site");
            editor.InsertParagraph();
            editor.InsertHyperlink("link", "telerik.com", false, "Telerik Site");
            editor.InsertText("after");

            //Act
            RadFlowDocument imported = this.ImportHtmlFromString(html);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        private void AddDefaultHtmlHyperlinkStyle(RadFlowDocument expected)
        {
            Style anchorStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.HyperlinkStyleId);
            anchorStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Color.FromRgb(0, 102, 204));
        }
    }
}
