﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Model.Shapes;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class ImagesTests : ParagraphTestsBase
    {
        [TestMethod]
        public void Test_InlineImage_ExportImport_Png()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngImage(expected));

            RadFlowDocument actual = this.ExportImportDocument(expected, new HtmlFormatProvider());

            this.AssertDocuments(expected, actual, true);
        }

        [TestMethod]
        public void Test_InlineImage_ExportImport_Bmp()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateBmpImage(expected));

            RadFlowDocument actual = this.ExportImportDocument(expected, new HtmlFormatProvider());

            this.AssertDocuments(expected, actual, true);
        }

        [TestMethod]
        public void Test_InlineImage_ExportImport_Jpg()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(expected));

            RadFlowDocument actual = this.ExportImportDocument(expected, new HtmlFormatProvider());

            this.AssertDocuments(expected, actual, true);
        }

        [TestMethod]
        public void Test_InlineImage_ExportImport_External()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(expected));

            byte[] data = null;

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.ImageExporting += (s, e) =>
            {
                e.Source = "test.jpg";
                
                data = e.Image.ImageSource.Data;

                e.Handled = true;
            };

            provider.ImportSettings.LoadFromUri += (s, e) =>
            {
                if (e.Uri == "test.jpg")
                {
                    e.SetData(data);
                }
            };

            RadFlowDocument actual = this.ExportImportDocument(expected, provider);

            this.AssertDocuments(expected, actual, true);
        }

        [TestMethod]
        public void InlineImage_ExportExternalImagesWhenImagesFolderIsNotSet_ThrowsException()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(expected));

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.ImagesExportMode = ImagesExportMode.External;

            try
            {
                string html = provider.Export(expected);
                Assert.Fail("Should throw exception.");
            }
            catch (NotSupportedException)
            {
            }
        }

        [TestMethod]
        public void InlineImage_ExportExternalStyleWhenStylesFileIsNotSet_ThrowsException()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport(false);
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("text");

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.StylesExportMode = StylesExportMode.External;

            try
            {
                string html = provider.Export(expected);
                Assert.Fail("Should throw exception.");
            }
            catch (NotSupportedException)
            {
            }
        }

        [TestMethod]
        public void InlineImage_WhenImageHasNotSetSize_ExportImport()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            ImageInline image = new ImageInline(document);
            image.Image.ImageSource = new Media.ImageSource(new byte[] { 1, 2, 3, 4 }, "png");
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.Add(image);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(document);

            // Assert
            ImageInline importedImage = imported.EnumerateChildrenOfType<ImageInline>().FirstOrDefault();
            Assert.IsTrue(importedImage.Image.Size.IsEmpty);
            Assert.AreEqual(double.NegativeInfinity, importedImage.Image.Height);
            Assert.AreEqual(double.NegativeInfinity, importedImage.Image.Width);
        }
    }
}
