#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public abstract class ItalicStyledTagTestsBase : ParagraphTestsBase
    {
        public ItalicStyledTagTestsBase()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
                Run.FontStylePropertyDefinition
            });
        }

        protected abstract string Tag { get; }

        [TestMethod]
        public void Test_ItalicStyledTag_AsChild_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.FontStyle = FontStyles.Italic;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><" + this.Tag + ">test</cite></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_ItalicStyledTag_AsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("first").FontStyle = FontStyles.Italic;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("second").FontStyle = FontStyles.Italic;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><" + this.Tag + "><p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_ItalicStyledTag_AsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").FontStyle = FontStyles.Italic;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("first").FontStyle = FontStyles.Italic;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("second").FontStyle = FontStyles.Italic;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_ItalicStyledTag_AsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").FontStyle = FontStyles.Italic;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("first").FontStyle = FontStyles.Italic;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            p.Inlines.AddRun("second").FontStyle = FontStyles.Italic;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

    }

    [TestClass]
    public class CiteTagTests : ItalicStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "cite";
            }
        }
    }

    [TestClass]
    public class VarTagTests : ItalicStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "var";
            }
        }
    }

    [TestClass]
    public class DfnTagTests : ItalicStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "dfn";
            }
        }
    }

    [TestClass]
    public class EmTagTests : ItalicStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "em";
            }
        }
    }

    [TestClass]
    public class ItalicTagTests : ItalicStyledTagTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "i";
            }
        }
    }
}