﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class ParagraphTests : ParagraphTestsBase
    {
        [TestMethod]
        public void Test_EmptyParagraph_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            expected.Sections.AddSection();

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_Paragraph_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p>test</p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_ParagraphWithSpan_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Test_ParagraphWithBr_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;
            paragraph.Inlines.AddRun("line1");
            paragraph.Inlines.Add(new Break(expected) { BreakType = Flow.Model.BreakType.LineBreak });
            paragraph.Inlines.AddRun("line2");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p>line1<br>line2</p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
