﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class QuoteTagTests : ParagraphTestsBase
    {
        [TestMethod]
        public void Test_QuoteAsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><q>test</q></p></body></html>");

            //Assert
            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_QuoteAsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("first");

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("second");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><q><p>first</p><p>second</p></q></body></html>");

            //Assert
            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_QuoteAsParentElementWithStartAndEndMarks_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("startMark");
            p.Inlines.AddRun("before");

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("first");

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("second");

            p = section.Blocks.AddParagraph();
            p.Inlines.AddRun("endMark");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>startMark<q>before<p>first</p><p>second</p></q>endMark</body></html>");

            //Assert
            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_QuoteAsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("before");

            p.Inlines.AddRun("after");

            p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("first");

            p = cell.Blocks.AddParagraph();
            p.Inlines.AddRun("second");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<q>after<p>first</p><p>second</p></q></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentsStructureOnly(expected, imported);
        }
    }
}
