﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public abstract class StrikethroughStyledTagsTestsBase : ParagraphTestsBase
    {
        public StrikethroughStyledTagsTestsBase()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Run.StrikethroughPropertyDefinition
            });
        }

        protected abstract string Tag { get; }

        [TestMethod]
        public void Test_StrikethroughStyledTag_AsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.Strikethrough = true;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><" + this.Tag + ">test</" + this.Tag + "></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_StrikethroughStyledTag_AsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            Paragraph p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("first").Strikethrough = true;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("second").Strikethrough = true;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><" + this.Tag + "><p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_StrikethroughStyledTag_AsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").Strikethrough = true;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("first").Strikethrough = true;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("second").Strikethrough = true;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_StrikethroughStyledTag_AsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").Strikethrough = true;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("first").Strikethrough = true;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.Strikethrough.LocalValue = true;
            p.Inlines.AddRun("second").Strikethrough = true;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }

    [TestClass]
    public class StrikeTagTestsBase : StrikethroughStyledTagsTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "strike";
            }
        }
    }

    [TestClass]
    public class DelTagTestsBase : StrikethroughStyledTagsTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "del";
            }
        }
    }

    [TestClass]
    public class StriketroughTagTestsBase : StrikethroughStyledTagsTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "s";
            }
        }
    }
}
