﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class SubscriptTagTests : ParagraphTestsBase
    {
        public SubscriptTagTests()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Run.BaselineAlignmentPropertyDefinition
            });
        }

        [TestMethod]
        public void Test_SubscriptAsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.BaselineAlignment = Flow.Model.Styles.BaselineAlignment.Subscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><sub>test</sub></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SubscriptAsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Subscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Subscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><sub><p>first</p><p>second</p></sub></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SubscriptAsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").BaselineAlignment = BaselineAlignment.Subscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Subscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Subscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<sub>after<p>first</p><p>second</p></sub></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SubscriptAsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").BaselineAlignment = BaselineAlignment.Subscript;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Subscript;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Subscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Subscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<sub>after<p>first</p><p>second</p></sub></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }
}
