﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public class SuperscriptTagTests : ParagraphTestsBase
    {
        public SuperscriptTagTests()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Run.BaselineAlignmentPropertyDefinition
            });
        }

        [TestMethod]
        public void Test_SuperscriptAsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.BaselineAlignment = BaselineAlignment.Superscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><sup>test</sup></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SuperscriptAsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Superscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Superscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><sup><p>first</p><p>second</p></sup></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SuperscriptAsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").BaselineAlignment = BaselineAlignment.Superscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Superscript;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Superscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<sup>after<p>first</p><p>second</p></sup></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_SuperscriptAsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();

            p.Inlines.AddRun("before");
            p.Inlines.AddRun("after").BaselineAlignment = BaselineAlignment.Superscript;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("first").BaselineAlignment = BaselineAlignment.Superscript;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.BaselineAlignment.LocalValue = BaselineAlignment.Superscript;
            p.Inlines.AddRun("second").BaselineAlignment = BaselineAlignment.Superscript;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<sup>after<p>first</p><p>second</p></sup></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }
}
