﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Paragraphs
{
    [TestClass]
    public abstract class UnderlineStyledTagsTestsBase : ParagraphTestsBase
    {
        public UnderlineStyledTagsTestsBase()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Run.UnderlineColorPropertyDefinition,
               Run.UnderlinePatternPropertyDefinition
            });
        }

        protected abstract string Tag { get; }

        [TestMethod]
        public void Test_UnderlineStyledTag_AsChildElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><" + this.Tag + ">test</" + this.Tag + "></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_UnderlineStyledTag_AsChildElementWithSetColor_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Run run = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><u style='color:red'>test</" + this.Tag + "></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_UnderlineStyledTag_AsParentElement_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            Run run = p.Inlines.AddRun("first");
            run.Underline.Pattern = UnderlinePattern.Single;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            run = p.Inlines.AddRun("second");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><" + this.Tag + "><p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_UnderlineStyledTag_AsParentElementSurroundedByText_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            Paragraph p = section.Blocks.AddParagraph();

            Run run = p.Inlines.AddRun("before");
            run = p.Inlines.AddRun("after");
            run.Underline.Pattern = UnderlinePattern.Single;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            run = p.Inlines.AddRun("first");
            run.Underline.Pattern = UnderlinePattern.Single;

            p = section.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            run = p.Inlines.AddRun("second");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void Test_UnderlineStyledTag_AsChildElementInTableCell_Import()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();

            Paragraph p = cell.Blocks.AddParagraph();

            Run run = p.Inlines.AddRun("before");
            run = p.Inlines.AddRun("after");
            run.Underline.Pattern = UnderlinePattern.Single;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            run = p.Inlines.AddRun("first");
            run.Underline.Pattern = UnderlinePattern.Single;

            p = cell.Blocks.AddParagraph();
            p.Properties.ParagraphMarkerProperties.UnderlinePattern.LocalValue = UnderlinePattern.Single;
            run = p.Inlines.AddRun("second");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td>before<" + this.Tag + ">after<p>first</p><p>second</p></" + this.Tag + "></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }
    }

    [TestClass]
    public class UnderlineTagTests : UnderlineStyledTagsTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "u";
            }
        }
    }


    [TestClass]
    public class InsertTagTests : UnderlineStyledTagsTestsBase
    {
        protected override string Tag
        {
            get
            {
                return "ins";
            }
        }
    }
}
