﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.PartialHtml
{
    [TestClass]
    public class ImageTests : HtmlTestBase
    {
        private const string DataPrefix = "data:image/";
        private const string Base64Prefix = "base64,";
        private const string Delimiter = ";";

        protected override string FolderName
        {
            get { throw new NotImplementedException(); }
        }

        [TestMethod]
        public void Import_WhenImageSourceWithBase64Data_ShouldImportCorrectly()
        {
            RadFlowDocument expected = new RadFlowDocument();
            ImageInline image = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            image.Image.Width = 218;
            image.Image.Height = 92;
            image.Image.ImageSource = new ImageSource(Convert.FromBase64String("/9j/4AAQSkZJRgABAgAAZABkAAD/"), "png");

            RadFlowDocument imported = this.ImportHtmlFromString(string.Format("<img src='{0}' width='218' height='92' />", "data:image/png;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/"));

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Import_WhenImageSourceWithLeadingAndTrailingWhiteSpaces_ShouldImportCorrectly()
        {
            RadFlowDocument expected = new RadFlowDocument();
            ImageInline image = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            image.Image.Width = 218;
            image.Image.Height = 92;
            image.Image.ImageSource = new ImageSource(Convert.FromBase64String("/9j/4AAQSkZJRgABAgAAZABkAAD/"), "png");

            RadFlowDocument imported = this.ImportHtmlFromString(string.Format("<img src='{0}' width='218' height='92' />", "      data:image/png;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/     "));

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Import_WhenImageSourceWithLeadingAndTrailingCarrageReturn_ShouldImportCorrectly()
        {
            RadFlowDocument expected = new RadFlowDocument();
            ImageInline image = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            image.Image.Width = 218;
            image.Image.Height = 92;
            image.Image.ImageSource = new ImageSource(Convert.FromBase64String("/9j/4AAQSkZJRgABAgAAZABkAAD/"), "png");

            RadFlowDocument imported = this.ImportHtmlFromString(string.Format("<img src='{0}' width='218' height='92' />", "\r\r\r\rdata:image/png;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/\r\r\r\r"));

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Import_WhenImageSourceWithLeadingAndTrailingLineFeed_ShouldImportCorrectly()
        {
            RadFlowDocument expected = new RadFlowDocument();
            ImageInline image = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            image.Image.Width = 218;
            image.Image.Height = 92;
            image.Image.ImageSource = new ImageSource(Convert.FromBase64String("/9j/4AAQSkZJRgABAgAAZABkAAD/"), "png");

            RadFlowDocument imported = this.ImportHtmlFromString(string.Format("<img src='{0}' width='218' height='92' />", "\n\n\n\ndata:image/png;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/\n\n\n\n"));

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Import_WhenImageSourceIsEmpty_ShouldNotThrow()
        {
            RadFlowDocument expected = new RadFlowDocument();
            ImageInline image = expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            image.Image.Width = 218;
            image.Image.Height = 92;

            RadFlowDocument imported = this.ImportHtmlFromString(string.Format("<img src='{0}' width='218' height='92' />", ""));

            this.AssertDocumentsStructureOnly(expected, imported);
        }
    }
}
