﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.PartialHtml
{
    [TestClass]
    public class ParagraphTests : HtmlTestBase
    {
        protected override string FolderName
        {
            get { throw new NotImplementedException(); }
        }

        [TestMethod]
        public void Test_EmptyString_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection();

            RadFlowDocument imported = this.ImportHtmlFromString("");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_PainText_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("Plain Text");

            RadFlowDocument imported = this.ImportHtmlFromString("Plain Text");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_NoBody_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection();

            RadFlowDocument imported = this.ImportHtmlFromString("<html><head><title>title</title></head></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_ParagraphOnly_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("Single paragraph");

            RadFlowDocument imported = this.ImportHtmlFromString("<p>Single paragraph</p>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_ParagraphInPlainText_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            section.Blocks.AddParagraph().Inlines.AddRun("first");
            section.Blocks.AddParagraph().Inlines.AddRun("second");
            section.Blocks.AddParagraph().Inlines.AddRun("third");

            RadFlowDocument imported = this.ImportHtmlFromString("first<p>second</p>third");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_SpanOnly_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            section.Blocks.AddParagraph().Inlines.AddRun("Only span");

            RadFlowDocument imported = this.ImportHtmlFromString("<span>Only span</span>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_SpanInPlainText_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();

            paragraph.Inlines.AddRun("first");
            paragraph.Inlines.AddRun("second");
            paragraph.Inlines.AddRun("third");

            RadFlowDocument imported = this.ImportHtmlFromString("first<span>second</span>third");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_HtmlInHtml_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            section.Blocks.AddParagraph().Inlines.AddRun("p1");
            section.Blocks.AddParagraph().Inlines.AddRun("p2");

            RadFlowDocument imported = this.ImportHtmlFromString("<html><head></head><body><html><head></head><body><p>p1</p></body></html><p>p2</p></body></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_HtmlInTable_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();

            section.Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("p1");
            section.Blocks.AddParagraph().Inlines.AddRun("p2");

            RadFlowDocument imported = this.ImportHtmlFromString("<html><head></head><body><table><tr><td><html><head></head><body><p>p1</p></body></html></td></tr></table><p>p2</p></body></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }
    }
}
