﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.PartialHtml
{
    [TestClass]
    public class TableTests : HtmlTestBase
    {
        protected override string FolderName
        {
            get
            {
                return "Tables";
            }
        }

        [TestMethod]
        public void Test_TableRowOnly_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();

            paragraph.Inlines.AddRun("Row");

            RadFlowDocument imported = this.ImportHtmlFromString("<tr>Row</tr>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_TableRowInPlainText_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();

            paragraph.Inlines.AddRun("first");
            paragraph.Inlines.AddRun("secondthird");

            RadFlowDocument imported = this.ImportHtmlFromString("first<tr>second</tr>third");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_TableOnly_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("Table");

            RadFlowDocument imported = this.ImportHtmlFromString("<table><tr><td>Table</td></tr></table>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_TableInPlainText_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph().Inlines.AddRun("first");
            section.Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("second");
            section.Blocks.AddParagraph().Inlines.AddRun("third");

            RadFlowDocument imported = this.ImportHtmlFromString("first<table><tr><td>second</td></tr></table>third");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_NestedHtmlInTable_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell1");
            row.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell2");

            RadFlowDocument imported = this.ImportHtml("NestedTables2.html");

            this.AssertDocumentsStructureOnly(expected, imported);
        }
    }
}
