﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles
{
    [TestClass]
    public class BuiltInStylesPropertiesValuesTests : StyleTestsBase
    {
        [TestMethod]
        public void NormalStyle_WhenDocumentIsEmpty_Import()
        {
            // Arrange
            RadFlowDocument expected = CreateDocument();
            Style expectedNormalStyle = expected.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);

            // Act
            RadFlowDocument actual = this.ImportHtmlFromString("<html><body>test</body></html>");

            //Assert
            Style actualNormalStyle = actual.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.CharacterProperties, actualNormalStyle.CharacterProperties);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.ParagraphProperties, actualNormalStyle.ParagraphProperties);
        }

        [TestMethod]
        public void NormalStyle_WhenDocumentIsEmpty_ExportImport()
        {
            // Arrange
            RadFlowDocument actual = this.GetExpectedDocumentForExportImport(false);
            Style actualNormalStyle = actual.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);

            // Act
            RadFlowDocument expected = CreateDocument();

            //Assert
            Style expectedNormalStyle = expected.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.CharacterProperties, actualNormalStyle.CharacterProperties);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.ParagraphProperties, actualNormalStyle.ParagraphProperties);
        }

        [TestMethod]
        public void NormalStyle_WhenDocumentIsEmptyAndNormalStyleIsEdited_ExportImport()
        {
            // Arrange
            RadFlowDocument actual = new RadFlowDocument();
            Style actualNormalStyle = actual.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);
            actualNormalStyle.CharacterProperties.FontSize.LocalValue = 36;
            actualNormalStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Arial"));
            actualNormalStyle.ParagraphProperties.AutomaticSpacingAfter.LocalValue = false;
            actualNormalStyle.ParagraphProperties.AutomaticSpacingBefore.LocalValue = false;
            actualNormalStyle.ParagraphProperties.SpacingAfter.LocalValue = 50;
            actualNormalStyle.ParagraphProperties.SpacingBefore.LocalValue = 75;
            actualNormalStyle.ParagraphProperties.LineSpacing.LocalValue = 10;
            actualNormalStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Exact;

            // Act
            RadFlowDocument expected = this.ExportImportDocument(actual);

            // Assert
            Style expectedNormalStyle = expected.StyleRepository.GetStyle(BuiltInStyleNames.NormalStyleId);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.CharacterProperties, actualNormalStyle.CharacterProperties);
            StylePropertyAsserter.AssertPropertiesAreEqual(expectedNormalStyle.ParagraphProperties, actualNormalStyle.ParagraphProperties);
        }
    }
}