﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class FlowDirectionTests : StyleTestsBase
    {
        [TestMethod]
        public void FlowDirection_Attribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = paragraph.Inlines.AddRun("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span dir=\"RTL\">שדג:3</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_Style_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = paragraph.Inlines.AddRun("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"direction: RTL;\">שדג:3</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_DefinedAsAttributeAndStyle_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = paragraph.Inlines.AddRun("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span dir=\"LTR\" style=\"direction: RTL;\">שדג:3</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void FlowDirection_Invalid_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;
            Run run = paragraph.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"direction: invalidDirection;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
