﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class FontFamilyTests : StyleTestsBase
    {
        [TestMethod]
        public void FontFamily_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontFamily = new ThemableFontFamily("Times New Roman");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-family: Times New Roman;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontFamily_WithGenericFontFamilies_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Section s = expected.Sections.AddSection();
            Paragraph p = s.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("cursive").FontFamily = new ThemableFontFamily("Comic Sans MS");

            p = s.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("fantasy").FontFamily = new ThemableFontFamily("Algerian");

            p = s.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("monospace").FontFamily = new ThemableFontFamily("Courier New");

            p = s.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("sansserif").FontFamily = new ThemableFontFamily("Arial");

            p = s.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("serif").FontFamily = new ThemableFontFamily("Times New Roman");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"<html>
    <head>
    </head>
    <body>
        <p><span style='font-family:cursive'>cursive</span></p>
        <p><span style='font-family:fantasy'>fantasy</span></p>
        <p><span style='font-family:monospace'>monospace</span></p>
        <p><span style='font-family:sans-serif'>sansserif</span></p>
        <p><span style='font-family:serif'>serif</span></p>
    </body>
</html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }




        [TestMethod]
        public void FontFamily_MultiDefined_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontFamily = new ThemableFontFamily("Times New Roman");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-family: Times New Roman, Cambria;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontFamily_MultiDefinedWithQuotes_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontFamily = new ThemableFontFamily("Verdana");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-family: 'Verdana', sans-serif;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontFamily_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontFamily = new ThemableFontFamily("Times New Roman");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
    }
}
