﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class FontSizeTests : StyleTestsBase
    {
        [TestMethod]
        public void FontSize_UnitPixel_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 44px;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitPixel_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void FontSize_UnitUndefined_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 44;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_InvalidValue_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 44 px;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_ValueWhithTrailingSpaces_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 44   ;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitEm_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 44;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 22;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body style='font-size: 44px;'><p><span style='font-size: 0.5em;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitPercent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 44;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 22;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body style='font-size: 44;'><p><span style='font-size: 50%;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitEmBig_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 22;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body style='font-size: 22;'><p><span style='font-size: 2em;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitEmRelativeToDefaultFontSize_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 32;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 2em;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitPercentBig_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 22;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 44;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body style='font-size: 22;'><p><span style='font-size: 200%;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontSize_UnitPercentRelativeToDefaultFontSize_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontSize = 32;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='font-size: 200%;'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
