﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class FontWeightTests : StyleTestsBase
    {
        [TestMethod]
        public void FontWeight_Bold_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-weight: bold;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontWeight_Bold_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontWeight = FontWeights.Bold;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void FontWeight_Normal_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontWeight = FontWeights.Normal;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-weight: normal;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FontWeight_Normal_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontWeight = FontWeights.Normal;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void FontWeight_Digits_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.FontWeight = FontWeights.Normal;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"font-weight: 900;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
