﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif

#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Html;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class ForegroundTests : StyleTestsBase
    {
        [TestMethod]
        public void Foreground_HexColor_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"color: #FF0000;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Foreground_TextColor_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"color: red;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        // TODO: add convertion for rgb defined color
        //[TestMethod]
        public void Foreground_RgbColor_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.ForegroundColor = new ThemableColor(Colors.Red);
            run.Underline.Color = run.ForegroundColor;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"color: rgb(255,0,0);\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Foreground_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
    }
}
