﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.CharacterProperties
{
    [TestClass]
    public class UnderlineTests : StyleTestsBase
    {
        [TestMethod]
        public void Underline_NoColor_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"text-decoration: underline;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Underline_WithColor_ColorDefinedSecond_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"text-decoration: underline; color: #FF0000;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Underline_WithColor_ColorDefinedFirst_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style=\"color: #FF0000; text-decoration: underline;\">test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Underline_WithColor_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void Underline_NoColor_UndelrineIsNotExported_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");
            run.Underline.Pattern = UnderlinePattern.Single;
            run.Underline.Color = new ThemableColor(Colors.Transparent);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            // Assert
            // UnderlinePattern is not exported when there is no color.
            run.Properties.UnderlinePattern.ClearValue();
            run.Properties.UnderlineColor.ClearValue();
            this.AssertDocuments(expected, imported, true);
        }
    }
}
