﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Flow.Model.Editing;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles
{
    [TestClass]
    public class ElementSelectorsTests : StyleTestsBase
    {
        [TestMethod]
        public void Heading1Style_IsImportedAsHeading1BuiltInStyle()
        {
            // Arrange
            RadFlowDocument expected = CreateDocument();
            Style headingStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));
            headingStyle.CharacterProperties.FontSize.LocalValue = 32;
            headingStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;

            headingStyle.ParagraphProperties.SpacingBefore.LocalValue = Unit.PointToDip(14);
            headingStyle.ParagraphProperties.AutomaticSpacingBefore.LocalValue = false;

            headingStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.PointToDip(14);
            headingStyle.ParagraphProperties.AutomaticSpacingAfter.LocalValue = false;


            Paragraph paragraph1 = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph1.Inlines.AddRun("text");
            paragraph1.StyleId = headingStyle.Id;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<head>
    <title></title>
    <style type='text/css'>
        h1 {
            text-align: center;
        }
    </style>
</head>
<body>
    <h1>text</h1>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Heading2Style_IsImportedAsHeading2BuiltInStyle()
        {
            // Arrange
            RadFlowDocument expected = CreateDocument();
            Style headingStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(2));
            headingStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(18);
            headingStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;

            headingStyle.ParagraphProperties.SpacingBefore.LocalValue = Unit.PointToDip(14);
            headingStyle.ParagraphProperties.AutomaticSpacingBefore.LocalValue = false;

            headingStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.PointToDip(14);
            headingStyle.ParagraphProperties.AutomaticSpacingAfter.LocalValue = false;


            Paragraph paragraph1 = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph1.Inlines.AddRun("text");
            paragraph1.StyleId = headingStyle.Id;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<head>
    <title></title>
    <style type='text/css'>
        h2 {
            text-align: center;
        }
    </style>
</head>
<body>
    <h2>text</h2>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void AnchorStyle_IsImportedAsHyperlinkBuiltInStyle()
        {
            // Arrange
            RadFlowDocument expected = CreateDocument();
            Style anchorStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.HyperlinkStyleId);
            anchorStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);
            editor.InsertHyperlink("text", "www.telerik.com", false);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<head>
    <title></title>
    <style type='text/css'>
        a {
            color: red;
        }
    </style>
</head>
<body>
    <a href='www.telerik.com'>text</a>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void AnchorWithSpan_IsImportedAsHyperlinkWithBuiltInStyle()
        {
            // Arrange
            RadFlowDocument expected = CreateDocument();
            Style anchorStyle = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.HyperlinkStyleId);
            anchorStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Color.FromRgb(0, 102, 204));

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);
            editor.InsertHyperlink("text", "www.telerik.com", false);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<head>
</head>
<body>
    <a href='www.telerik.com'><span>text</span></a>
</body>
</html>");

            // Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
