﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.ParagraphProperties
{
    [TestClass]
    public class FlowDirectionTests : StyleTestsBase
    {
        public FlowDirectionTests()
        {
            base.AsserterInfo.RegisterPropertiesToCheck(new IStylePropertyDefinition[]
            {
               Paragraph.FlowDirectionPropertyDefinition
            });
        }

        [TestMethod]
        public void FlowDirection_LeftToRight_UsingStyling_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.LeftToRight;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"direction: ltr;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_LeftToRight_UsingAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.LeftToRight;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p dir=\"ltr\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_LeftToRight_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.LeftToRight;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void FlowDirection_RightToLeft_UsingStyling_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"direction: rtl;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_RightToLeft_UsingAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p dir=\"rtl\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_RightToLeft_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.FlowDirection = FlowDirection.RightToLeft;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocumentStructureAndProperties(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_Invalid_UsingStyling_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"direction: invalidDirection;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void FlowDirection_Invalid_UsingAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p dir=\"invalidDirection\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
