﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.ParagraphProperties
{
    [TestClass]
    public class LineSpacingTests : StyleTestsBase
    {
        [TestMethod]
        public void LineSpacing_AutoValue_AsPercent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 2;
            p.Spacing.LineSpacingType = HeightType.Auto;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"line-height: 200%;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LineSpacing_AutoValue_AsNumber_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 2;
            p.Spacing.LineSpacingType = HeightType.Auto;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"line-height: 2;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LineSpacing_AutoValue_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 2;
            p.Spacing.LineSpacingType = HeightType.Auto;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void LineSpacing_ExactValue_AsPixel_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 30;
            p.Spacing.LineSpacingType = HeightType.Exact;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"line-height: 30px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LineSpacing_ExactValue_AsPoint_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 32;
            p.Spacing.LineSpacingType = HeightType.Exact;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"line-height: 24pt;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LineSpacing_ExactValue_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 30;
            p.Spacing.LineSpacingType = HeightType.Exact;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void LineSpacing_AtLeastValue_IsExportedAsExact_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 30;
            p.Spacing.LineSpacingType = HeightType.AtLeast;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            p.Spacing.LineSpacingType = HeightType.Exact;
            this.AssertDocuments(expected, imported, true);
        }
    }
}
