﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.ParagraphProperties
{
    [TestClass]
    public class MarginTests : StyleTestsBase
    {
        [TestMethod]
        public void MarginLeft_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.LeftIndent = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"margin-left: 20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void MarginLeft_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.LeftIndent = 20;

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void MarginRight_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.RightIndent = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"margin-right: 20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void MarginRight_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.RightIndent = 20;

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void MarginTop_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.AutomaticSpacingBefore = false;
            p.Spacing.SpacingBefore = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"margin-top: 20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void MarginTop_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.AutomaticSpacingBefore = false;
            p.Spacing.SpacingBefore = 20;

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void MarginBottom_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.AutomaticSpacingAfter = false;
            p.Spacing.SpacingAfter = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"margin-bottom: 20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void MarginBottom_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.AutomaticSpacingAfter = false;
            p.Spacing.SpacingAfter = 20;

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
    }
}
