﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.ParagraphProperties
{
    [TestClass]
    public class TextAlignmentTests : StyleTestsBase
    {
        [TestMethod]
        public void TextAlignment_Left_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Left;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-align: left;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextAlignment_Left_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Left;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TextAlignment_Right_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Right;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-align: right;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextAlignment_Right_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Right;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TextAlignment_Center_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Center;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-align: center;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextAlignment_Center_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Center;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TextAlignment_Justified_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Justified;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-align: justify;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextAlignment_Justified_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.Justified;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TextAlignment_ThaiDistribute_Export()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.TextAlignment = Alignment.ThaiDistribute;

            // Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("style=\"text-align: justify;\""));
        }

        [TestMethod]
        public void TextAlignment_Invalid_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-align: invalidAlignment;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
