﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.ParagraphProperties
{
    [TestClass]
    public class TextIndentTests : StyleTestsBase
    {
        [TestMethod]
        public void TextIndent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.FirstLineIndent = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-indent: 20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextIndent_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.FirstLineIndent = 20;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TextHangingIndent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.HangingIndent = 20;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style=\"text-indent: -20px;\"><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TextHangingIndent_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun("test");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Indentation.HangingIndent = 20;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
    }
}
