﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles
{
    [TestClass]
    public class PropertyInheritanceTests : StyleTestsBase
    {
        [TestMethod]
        public void Run_InheritsProperties_FromTable()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections
                .AddSection().Blocks
                .AddTable().Rows
                .AddTableRow().Cells
                .AddTableCell().Blocks
                .AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p.Properties.ParagraphMarkerProperties.FontSize.LocalValue = 32;
            p.Inlines.AddRun("my test table").FontSize = 32;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style> body {font-size: 24pt;}</style><body><table><tr><td><p>my test table</p></td></tr></table></body>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Paragraph_InheritsProperties_FromTable()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections
                .AddSection().Blocks
                .AddTable().Rows
                .AddTableRow().Cells
                .AddTableCell().Blocks
                .AddParagraph();
            p.Inlines.AddRun("my test table");

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Spacing.LineSpacing = 30;
            p.Spacing.LineSpacingType = HeightType.Exact;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style> body {line-height: 30px;}</style><body><table><tr><td><p>my test table</p></td></tr></table></body>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }

}
