﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles
{
    [TestClass]
    public class StyleExportTests : StyleTestsBase
    {
        [TestMethod]
        public void DocumentElementWithMissingStyle_ExportDoNotThrowException()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Hello world!");
            Table table = editor.InsertTable(3, 3);
            table.StyleId = "TableGrid";

            HtmlFormatProvider provider = new HtmlFormatProvider();
            string html = provider.Export(editor.Document);
        }

        [TestMethod]
        public void StylesWithExportModeNone_IsNotExported()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(CreateDocument());
            editor.InsertText("Hello world!");

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.StylesExportMode = StylesExportMode.None;

            string html = provider.Export(editor.Document);
            Assert.IsFalse(html.Contains("class="));
        }

        [TestMethod]
        public void StylesWithExportModeInline_AreExportedAsInlined()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(CreateDocument());
            editor.InsertText("Hello world!");

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.StylesExportMode = StylesExportMode.Inline;

            string html = provider.Export(editor.Document);
            Assert.IsFalse(html.Contains("class="));
            Assert.IsTrue(html.Contains("style="));
        }

        [TestMethod]
        public void TableStylesWithExportModeInline_AreExportedAsInlined()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(CreateDocument());
            Table table = editor.InsertTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.ExportSettings.StylesExportMode = StylesExportMode.Inline;

            string html = provider.Export(editor.Document);
            Assert.IsFalse(html.Contains("class="));
            Assert.IsTrue(html.Contains("style="));
        }
    }
}
