﻿using System;
using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableCellProperties
{
    [TestClass]
    public class TableCellBordersTests : StyleTestsBase
    {
        [TestMethod]
        public void TableCellBorders_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border: 1px solid black;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellBorders_NoColor_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border: 1px solid;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void TableCellBorders_AllEqualBorders_AreExprtedAsShorthandProperty()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border:"));
            Assert.IsFalse(html.Contains("border-left:"));
            Assert.IsFalse(html.Contains("border-right:"));
            Assert.IsFalse(html.Contains("border-top:"));
            Assert.IsFalse(html.Contains("border-bottom:"));

            AssertDocumentVisual(expected);
        }

        [TestMethod]
        public void TableCellBorders_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            //Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableCellBorders_Left_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, leftBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border-left: 2px solid red;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellBorders_Left_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, leftBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            //Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableCellBorders_Top_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, topBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border-top: 2px solid red;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellBorders_Top_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, topBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            //Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableCellBorders_Right_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, rightBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border-right: 2px solid red;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellBorders_Right_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, rightBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            //Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableCellBorders_Bottom_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, bottomBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='border-bottom: 2px solid red;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellBorders_Bottom_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Borders = new TableCellBorders(cell.Borders, bottomBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            //Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void InheritedCellBorders_AreExportedAsLocalValues()
        {
            RadFlowDocument original = this.GetExpectedDocumentForImport();
            Table table = original.Sections.AddSection().Blocks.AddTable();
            for (int i = 0; i < 3; i++)
            {
                TableRow row = table.Rows.AddTableRow();
                for (int j = 0; j < 3; j++)
                {
                    TableCell cell = row.Cells.AddTableCell();
                    Paragraph p = cell.Blocks.AddParagraph();
                    p.StyleId = BuiltInStyleNames.NormalWebStyleId;
                    p.Inlines.AddRun("test");
                }
            }

            table.Borders = new TableBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Blue)));

            // Act
            RadFlowDocument result = this.ExportImportDocument(original);

            // Assert
            this.AssertDocumentsStructureOnly(original, result);
            TableCellBorders expectedBorder = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Blue)));
            foreach (var cell in result.EnumerateChildrenOfType<TableCell>().ToList())
            {
                Assert.AreEqual(expectedBorder, cell.Borders);
            }

            AssertDocumentVisual(result);
        }
    }
}
