﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Primitives;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableCellProperties
{
    [TestClass]
    public class TableCellPaddingTests : StyleTestsBase
    {
        [TestMethod]
        public void LocalPadding_OneValue_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(1);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='padding: 1px'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LocalPadding_TwoValues_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(2, 1, 2, 1);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='padding: 1px 2px'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void LocalPadding_ThreeValues_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(2, 1, 2, 3);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='padding: 1px 2px 3px'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void LocalPadding_FourValues_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(4, 1, 2, 3);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='padding: 1px 2px 3px 4px'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void LocalPadding_LeftRightTopBottom_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(1, 3, 2, 4);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='padding-left: 1px; padding-right: 2px; padding-top: 3px; padding-bottom: 4px'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void LocalPadding_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.Padding = new Padding(1, 3, 2, 4);

            // Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void Table_CellPadding_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p2 = cell.Blocks.AddParagraph();

            p2.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p2.Inlines.AddRun("test");

            cell.Padding = new Padding(10);

            RadFlowDocument document = this.GetExpectedDocumentForExportImport();
            Table table = document.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();

            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.TableCellPadding = new Padding(10);

            // Act
            string html = new HtmlFormatProvider().Export(document);
            RadFlowDocument imported = this.ExportImportDocument(document, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }
    }
}
