﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableCellProperties
{
    [TestClass]
    public class TableCellPreferredWidthTests : StyleTestsBase
    {
        [TestMethod]
        public void TableCellPreferredWidth_Auto_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void TableCellPreferredWidth_Auto_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableCellPreferredWidth_Fixed_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='width: 100px;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableCellPreferredWidth_Fixed_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableCellPreferredWidth_Percent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 100);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='width: 100%;'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableCellPreferredWidth_Percent_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableCell cell = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }
    }
}
