﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableCellProperties
{
    [TestClass]
    public class TableCellVerticalAlignmentTests : StyleTestsBase
    {
        [TestMethod]
        public void TableCellVerticalAlignment_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            this.AddTableCellToRow(row, true);
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Top;
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Center;
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Bottom;

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"<html><body><table><tr>
<td><p><span>test</span></p><p><span>test</span></p><p><span>test</span></p></td>
<td style='vertical-align:top;'><p><span>test</span></p></td>
<td style='vertical-align:middle;'><p><span>test</span></p></td>
<td style='vertical-align:bottom;'><p><span>test</span></p></td>
</tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }


        [TestMethod]
        public void TableCellVerticalAlignment_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            this.AddTableCellToRow(row, true);
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Top;
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Center;
            this.AddTableCellToRow(row).VerticalAlignment = VerticalAlignment.Bottom;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableCellVerticalAlignment_ExportNotThrowingException()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(expected);
            Table table = editor.InsertTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("Hello");

            Style style = new Style("aa", StyleType.Table);
            style.TableCellProperties.VerticalAlignment.LocalValue = VerticalAlignment.Center;

            expected.StyleRepository.Add(style);

            table.StyleId = style.Id;

            HtmlFormatProvider provider = new HtmlFormatProvider();
            provider.Export(expected);
            this.AssertDocumentVisual(expected);
        }

        private TableCell AddTableCellToRow(TableRow row, bool exportLargeCell = false)
        {
            TableCell cell = row.Cells.AddTableCell();

            int parCount = exportLargeCell ? 3 : 1;

            for (int i = 0; i < parCount; i++)
            {
                Paragraph p = cell.Blocks.AddParagraph();
                p.StyleId = BuiltInStyleNames.NormalWebStyleId;
                p.Inlines.AddRun("test");
            }

            return cell;
        }
    }
}
