﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class CellSpacingTests : StyleTestsBase
    {
        [TestMethod]
        public void CellSpacing_OneValidValue_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='border-spacing:10px;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void CellSpacing_OneInvalidValue_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='border-spacing:10;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void CellSpacing_TwoValidValues_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='border-spacing:10px 20px;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void CellSpacing_TwoInValidValues_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='border-spacing:10 20;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void CellSpacing_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void CellSpacing_Exports_BorderCollapsed_WhenNoSpacing()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 0;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border-collapse: collapse"));
        }

        [TestMethod]
        public void CellSpacing_Exports_BorderCollapsed_ThereIsNoLocalValue()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border-collapse: collapse"));
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void CellSpacing_Exports_BorderSeparate_WhenPositiveSpacing()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border-collapse: separate"));
            this.AssertDocumentVisual(expected);
        }

        [TestMethod]
        public void CellSpacing_WhenAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table cellspacing=\"10px\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void CellSpacing_WhenIsDeclaredInAttributeAndUnitTypeIsNotDefined_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table cellspacing=\"10\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void CellSpacing_WhenIsDeclaredInAttributeAsMultipleNumericsAndUnitTypeIsNotDefined_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table cellspacing=\"10 20\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void CellSpacing_WhenIsDeclaredInAttributeAndStyle_StylePropertyValueIsImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.TableCellSpacing = 10;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table cellspacing=\"40\" style='border-spacing:10px 20px;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
