﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class IndentTests : StyleTestsBase
    {
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void Indent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Indent = 20;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"margin-left: 20px;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void Indent_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Indent = 20;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
    }
}
