﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class TableAlignmentTests : StyleTestsBase
    {
        [TestMethod]
        public void TableAlignmentAttribute_WhenIsLeft_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Left;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table align=\"left\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableAlignmentAttribute_WhenIsLeft_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Left;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableAlignmentAttribute_WhenIsRight_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Right;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table align=\"right\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableAlignmentAttribute_WhenIsRight_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Right;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableAlignmentAttribute_WhenIsCenter_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Center;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table align=\"center\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableAlignmentAttribute_WhenIsCenter_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Alignment = Alignment.Center;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableAlignmentAttribute_WhenIsUnknown_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table align=\"justified\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocumentVisual(imported);
            this.AssertDocuments(expected, imported);
        }
    }
}