﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class TableBackgroundTests : StyleTestsBase
    {
        [TestMethod]
        public void TableBackground_Color_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Shading.BackgroundColor = new ThemableColor(Colors.Red);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"background-color: #FF0000;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableBackground_Color_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Shading.BackgroundColor = new ThemableColor(Colors.Red);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
            this.AssertDocumentVisual(imported);
        }
    }
}
