﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class TableBordersTests : StyleTestsBase
    {
        [TestMethod]
        public void TableBorders_AllEqualBorders_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border: 1px solid black;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableBorders_AllEqualBorders_NoColorInCss_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border: 1px solid;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableBorders_AllEqualBorders_AreExprtedAsShorthandProperty()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border:"));
            Assert.IsFalse(html.Contains("border-left:"));
            Assert.IsFalse(html.Contains("border-right:"));
            Assert.IsFalse(html.Contains("border-top:"));
            Assert.IsFalse(html.Contains("border-bottom:"));
            this.AssertDocumentVisual(expected);
        }

        [TestMethod]
        public void TableBorders_AllEqualBorders_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            // Cell borders are exported as local values.
            cell.Borders = new TableCellBorders(new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableBorders_Left_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders();
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border-left: 1px solid black;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        //[TestMethod]
        public void TableBorders_Left_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableBorders_Top_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders();
            table.Borders = new TableBorders(table.Borders, topBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border-top: 2px solid red;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        //[TestMethod]
        public void TableBorders_Top_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders(
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableBorders_Right_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders();
            table.Borders = new TableBorders(table.Borders, rightBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border-right: 2px solid red;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        //[TestMethod]
        public void TableBorders_Right_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders();
            table.Borders = new TableBorders(
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TableBorders_Bottom_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders();
            table.Borders = new TableBorders(table.Borders, bottomBorder: new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border-bottom: 2px solid red;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        //[TestMethod]
        public void TableBorders_Bottom_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Borders = new TableBorders(
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(0, BorderStyle.None, new ThemableColor(Colors.Black)),
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            //Act
            string html = new HtmlFormatProvider().Export(expected);
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableBorders_WhenAreDefinedThroughAttributes_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(10, BorderStyle.Single, new ThemableColor(Colors.Red));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table border=\"10px\" bordercolor=\"#FF0000\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableBorders_WhenBorderIsInAttributeWithoutUnitType_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(10, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table border=\"10\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        public void TableBorders_WhenAreDefinedThroughAttributesAndStyle_StylePropertiesAreImported()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(2, BorderStyle.Single, new ThemableColor(Colors.Red));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table border=\"10\" bordercolor=\"#000000\" style=\"border: 2px solid #FFFF00; border-color:#ff0000;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        public void TableBorders_BorderStyleAttributeValuesAreInNonStandardOrder_ImportExport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border: solid 1px black;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableBorders_BorderStyleAttributeOnlyStyleValue_ImportExport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border: solid;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableBorders_BorderStyleAttributeInvalidColor_ImportExport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style=\"border: solid 1px qwerty;\"><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableBorders_BorderStyleAttributeExport_UnsupportedBorderStyle()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Border border = new Border(1, BorderStyle.ThreeDEmboss, new ThemableColor(Colors.Black));
            table.Borders = new TableBorders(border, border, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Run run = p.Inlines.AddRun("test");

            // Act
            string html = new HtmlFormatProvider().Export(expected);

            //Assert
            Assert.IsTrue(html.Contains("border: 1px solid #000000"));
        }
    }
}
