﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableProperties
{
    [TestClass]
    public class TablePreferredWidthTests : StyleTestsBase
    {
        [TestMethod]
        public void TablePreferredWidth_Auto_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TablePreferredWidth_Auto_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TablePreferredWidth_Fixed_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='width: 100px;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TablePreferredWidth_Fixed_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void TablePreferredWidth_Percent_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 100);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='width: 100%;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TablePreferredWidth_Percent_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            Paragraph p = table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new HtmlFormatProvider());

            //Assert
            this.AssertDocuments(expected, imported, true);
        }
#if !VISUAL
        [TestMethod]
        public void TablePreferredWidth_AttributeAndStyleConcurrency_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(addNormalWebStyle: false);
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell");
            table.PreferredWidth = new Flow.Model.Styles.TableWidthUnit(Flow.Model.Styles.TableWidthUnitType.Percent, 100);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><head><body><table style='width: 100%' width='200px' ><tr><td>cell</td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
#endif

        [TestMethod]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void TablePreferredWidth_WhenUnitTypeIsNotDefined_IsImportedAsAuto()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(false);
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("cell");
            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><head><body><table style='width:200' ><tr><td>cell</td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
#if VISUAL
            imported.EnumerateChildrenOfType<TableCell>().FirstOrDefault().Borders = new TableCellBorders(new Border(1, BorderStyle.Thick, new ThemableColor(Colors.Red)));
            this.AssertDocumentVisual(imported);
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.TableLayout)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void TablePreferredWidth_WhenSizeValueIsNotDefinedAndWidthIsAsProperty_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(false);
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell");
            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 200);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><head><body><table width='200' ><tr><td>cell</td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
            imported.EnumerateChildrenOfType<TableCell>().FirstOrDefault().Borders = new TableCellBorders(new Border(1, BorderStyle.Thick, new ThemableColor(Colors.Red)));
            this.AssertDocumentVisual(imported);
        }
    }
}