﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles.TableRowProperties
{
    [TestClass]
    public class HeightTests : StyleTestsBase
    {
        /***************************** Height Comming From Table Row ********************************/
        [TestMethod]
        public void Height_AtLeast_AsStyleProperty_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr style='height: 100px;'><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_AtLeast_AsAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr height='100px'><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_AtLeast_InClass_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Style heightStyle = StyleTestsBase.CreateWebBasedParagraphStyle("height");
            expected.StyleRepository.Add(heightStyle);

            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style>.height{height: 100px;}</style><body><table><tr class='height'><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_AtLeast_AsAttributeOnRowAndCells_GreaterValueForRowHeight_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr height='100px'><td height='40px'><p><span>test 1</span></p></td><td height='60px'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_AtLeast_AsAttributeOnRowAndCells_GreaterValueForCellHeight_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr height='60px'><td height='40px'><p><span>test 1</span></p></td><td height='100px'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_AtLeast_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void Height_Auto_AsStyleProperty_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.Auto);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr style='height: auto;'><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Height_Auto_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.Auto);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            this.AssertDocuments(expected, imported, true);
        }

        [TestMethod]
        public void Height_Exact_IsExportedAs_AtLeast_ExportImport()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForExportImport();
            TableRow row = expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.Exact, 100);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            this.AssertDocuments(expected, imported, true);
        }

        /***************************** Height Comming From Table ********************************/
        [TestMethod]
        public void TableHeight_Auto_AsAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.Auto);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table height='100px'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableHeight_Auto_AsStyle_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            row.Height = new TableRowHeight(HeightType.Auto);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table style='height: 100px;'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableHeight_Auto_InClass_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Style heightStyle = StyleTestsBase.CreateWebBasedParagraphStyle("height");
            expected.StyleRepository.Add(heightStyle);

            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            table.StyleId = heightStyle.Id;

            row.Height = new TableRowHeight(HeightType.Auto);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style>.height{height: 100px;}</style><body><table class='height'><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        /***************************** Height Comming From Table Cell ********************************/
        [TestMethod]
        public void TableCellHeight_AtLeast_AsAttribute_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 200);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td height='100px'><p><span>test 1</span></p></td><td height='200px'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_AtLeast_AsAttribute_ForTwoRows_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");
            row.Height = new TableRowHeight(HeightType.AtLeast, 200);

            row = table.Rows.AddTableRow();
            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td height='200px'><p><span>test 1</span></p></td></tr><tr><td height='100px'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_AtLeast_AsStyle_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 200);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='height: 100px;'><p><span>test 1</span></p></td><td style='height: 200px;'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_AtLeast_AsStyleAndAttributeConcurrency_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);

            row = table.Rows.AddTableRow();
            p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 100);
            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td height='40px' style='height: 100px'><p><span>test 1</span></p></td></tr><tr><td style='height: 100px' height='40px'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_InClass_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Style heightStyle = StyleTestsBase.CreateWebBasedParagraphStyle("height");
            expected.StyleRepository.Add(heightStyle);

            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            Paragraph p = row.Cells.AddTableCell().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //table.StyleId = heightStyle.Id;
            row.Height = new TableRowHeight(HeightType.AtLeast, 100);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style>.height{height: 100px;}</style><body><table><tr><td class='height'><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        //[TestMethod]
        public void TableCellHeight_InTableCellElementClass_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Style heightStyle = StyleTestsBase.CreateWebBasedParagraphStyle("height");
            expected.StyleRepository.Add(heightStyle);

            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            //table.StyleId = heightStyle.Id;
            row.Height = new TableRowHeight(HeightType.AtLeast, 100);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style>td{height: 100px;}</style><body><table><tr><td><p><span>test</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_InClassAndStylePropertyConcurrency_Import()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Style heightStyle = StyleTestsBase.CreateWebBasedParagraphStyle("height");
            expected.StyleRepository.Add(heightStyle);

            Table table = expected.Sections.AddSection().Blocks.AddTable();
            //table.StyleId = heightStyle.Id;

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            row.Height = new TableRowHeight(HeightType.AtLeast, 40);

            row = table.Rows.AddTableRow();
            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 60);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><style>.height{height: 100px;}</style><body><table><tr><td class='height' style='height: 40px;'><p><span>test 1</span></p></td></tr><tr><td style='height: 60px;' class='height'><p><span>test 2</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_InStyleProperty_WhenNestedTableIsAdded_Import()
        {
            // Arange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            row.Height = new TableRowHeight(HeightType.AtLeast, 40);

            TableCell cell = row.Cells.AddTableCell();

            Table nestedTable = cell.Blocks.AddTable();
            row = nestedTable.Rows.AddTableRow();
            cell = row.Cells.AddTableCell();
            Paragraph p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 1");

            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 2");

            row.Height = new TableRowHeight(HeightType.AtLeast, 200);

            row = table.Rows.AddTableRow();
            cell = row.Cells.AddTableCell();
            p = cell.Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test 3");

            row.Height = new TableRowHeight(HeightType.AtLeast, 60);

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><table><tr><td style='height: 40px;'><table><tr><td style='height: 200px;'><p><span>test 1</span></p></td><td style='height: 60px;' class='height'><p><span>test 2</span></p></td></tr></table></td></tr><tr><td style='height: 60px;' class='height'><p><span>test 3</span></p></td></tr></table></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void TableCellHeight_WithRowSpan_Import()
        {
            RadFlowDocument expected = this.GetExpectedDocumentForImport(false);
            Table table = expected.Sections.AddSection().Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("td 1");
            cell.RowSpan = 2;

            cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("td 2");

            row = table.Rows.AddTableRow();
            cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("td 2");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<table><tr><td rowspan='2' style='height: 100px;'>td 1</td><td>td 2</td></tr><tr><td>td 2</td></tr></table>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
