﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Styles
{
    [TestClass]
    public class UnknownCssPropertyTests : StyleTestsBase
    {
        [TestMethod]
        public void UnknownSpanProperty_IsIgnored()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p><span style='some-property: value'>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void UnknownParagraphProperty_IsIgnored()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><p style='some-property: value'><span>test</span></p></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void UnknownTableProperty_IsIgnored()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Paragraph p = expected.
                Sections.AddSection().
                Blocks.AddTable().
                Rows.AddTableRow().
                Cells.AddTableCell().
                Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<body>
	<table style='some-property: value'>
		<tr>
			<td><p>test</p></td>
		</tr>
	</table>
</body>
<html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void UnknownTableRowProperty_IsIgnored()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Paragraph p = expected.
                Sections.AddSection().
                Blocks.AddTable().
                Rows.AddTableRow().
                Cells.AddTableCell().
                Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<body>
	<table>
		<tr style='some-property: value'>
			<td><p>test</p></td>
		</tr>
	</table>
</body>
<html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void UnknownTableCellProperty_IsIgnored()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport();

            Paragraph p = expected.
                Sections.AddSection().
                Blocks.AddTable().
                Rows.AddTableRow().
                Cells.AddTableCell().
                Blocks.AddParagraph();
            p.StyleId = BuiltInStyleNames.NormalWebStyleId;
            p.Inlines.AddRun("test");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString(@"
<html>
<body>
	<table>
		<tr>
			<td style='some-property: value'><p>test</p></td>
		</tr>
	</table>
</body>
<html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }
    }
}
