﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Tables;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Tables
{
    [TestClass]
    public class MergedCellsTests : HtmlTestBase
    {
        protected override string FolderName
        {
            get
            {
                return "MergedCells";
            }
        }

        [TestMethod]
        public void TableCell_WitHorizontalMergedCell_Import()
        {
            // Act
            string html = @"<html>
<body>
    <table>
        <tr>
            <td>Cell [0, 0]</td>
            <td>Cell [0, 1]</td>
            <td>Cell [0, 2]</td>
            <td>Cell [0, 3]</td>
        </tr>
        <tr>
            <td>Cell [1, 0]</td>
            <td colspan='2'>Cell [1, 1]</td>
            <td>Cell [1, 3]</td>
        </tr>
        <tr>
            <td>Cell [2, 0]</td>
            <td>Cell [2, 1]</td>
            <td>Cell [2, 2]</td>
            <td>Cell [2, 3]</td>
        </tr>
    </table>
</body>
</html>";
            RadFlowDocument exported = this.ImportHtmlFromString(html);

            //Assert
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithHorizontalMerge(table);
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WitVerticalMergedCell_Import()
        {
            // Act
            string html = @"<html>
<body>
    <table>
        <tr>
            <td>Cell [0, 0]</td>
            <td>Cell [0, 1]</td>
            <td>Cell [0, 2]</td>
        </tr>
        <tr>
            <td>Cell [1, 0]</td>
            <td rowspan='2'>Cell [1, 1]</td>
            <td>Cell [1, 2]</td>
        </tr>
        <tr>
            <td>Cell [2, 0]</td>
            <td>Cell [2, 2]</td>
        </tr>
        <tr>
            <td>Cell [3, 0]</td>
            <td>Cell [3, 1]</td>
            <td>Cell [3, 2]</td>
        </tr>
    </table>
</body>
</html>";
            RadFlowDocument exported = this.ImportHtmlFromString(html);

            //Assert
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithVerticalMerge(table);
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WitVerticalAndHorizontalMergedCell_Import()
        {
            // Act
            string html = @"<html>
<body>
    <table>
        <tr>
            <td>Cell [0, 0]</td>
            <td>Cell [0, 1]</td>
            <td>Cell [0, 2]</td>
            <td>Cell [0, 3]</td>
        </tr>
        <tr>
            <td>Cell [1, 0]</td>
            <td colspan='2' rowspan='2'>Cell [1, 1]</td>
            <td>Cell [1, 3]</td>
        </tr>
        <tr>
            <td>Cell [2, 0]</td>
            <td>Cell [2, 3]</td>
        </tr>
        <tr>
            <td>Cell [3, 0]</td>
            <td>Cell [3, 1]</td>
            <td>Cell [3, 2]</td>
            <td>Cell [3, 3]</td>
        </tr>
    </table>
</body>
</html>";
            RadFlowDocument exported = this.ImportHtmlFromString(html);

            //Assert
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithVerticalAndHorizontalMerge(table);
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WitHorizontalMergedCell_ExportImport()
        {
            // Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithHorizontalMerge(table);

            // Act
            RadFlowDocument exported = this.ExportImportDocument(originalDoc);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WithVerticalMergedCell_ExportImport()
        {
            // Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithVerticalMerge(table);

            // Act
            RadFlowDocument exported = this.ExportImportDocument(originalDoc);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WithVerticalAndHorizontalMergedCell_ExportImport()
        {
            // Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithVerticalAndHorizontalMerge(table);

            // Act
            RadFlowDocument exported = this.ExportImportDocument(originalDoc);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WithTwoConsecutiveHorizontallyMerged_ExportImport()
        {
            // Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithTwoConsecutiveHorizontallyMergedGroups(table);

            // Act
            RadFlowDocument exported = this.ExportImportDocument(originalDoc);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }

        [TestMethod]
        public void TableCell_WithTwoConsecutiveVerticallyMergedGroups_ExportImport()
        {
            // Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();
            TableHelper.FillTableWithTwoConsecutiveVerticallyMergedGroups(table);

            // Act
            RadFlowDocument exported = this.ExportImportDocument(originalDoc);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, exported);
        }
    }
}
