﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport.Tables
{
    [TestClass]
    public class TableTests : HtmlTestBase
    {
        protected override string FolderName
        {
            get
            {
                return "Tables";
            }
        }

        [TestMethod]
        public void Table_WhenTableIsEmpty_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddTable();

            // Act
            RadFlowDocument imported = this.ImportHtml("Table_Empty.html");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_TwoTables_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = this.GetExpectedDocumentForImport(addNormalWebStyle: false);
            Section section = expected.Sections.AddSection();
            section.Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("table1");
            section.Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("table2");

            // Act
            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><div><table><tr><td>table1</td></tr></table></div><div><table><tr><td>table2</td></tr></table></div></body></html>");

            //Assert
            this.AssertDocuments(expected, imported);
        }

        [TestMethod]
        public void Table_WhenTableHasTableBody_WithTwoRowsAndFourCells_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            cellOne.Blocks.AddParagraph().Inlines.AddRun("1");

            TableCell cellTwo = row.Cells.AddTableCell();
            cellTwo.Blocks.AddParagraph().Inlines.AddRun("2");

            row = table.Rows.AddTableRow();

            TableCell cellThree = row.Cells.AddTableCell();
            cellThree.Blocks.AddParagraph().Inlines.AddRun("3");

            TableCell cellFour = row.Cells.AddTableCell();
            cellFour.Blocks.AddParagraph().Inlines.AddRun("4");

            // Act
            RadFlowDocument imported = this.ImportHtml("TableHasTableBody_WithTwoRowsAndFourCells.html");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_WhenTableWithTwoRowsAndFourCells_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            cellOne.Blocks.AddParagraph().Inlines.AddRun("1");

            TableCell cellTwo = row.Cells.AddTableCell();
            cellTwo.Blocks.AddParagraph().Inlines.AddRun("2");

            row = table.Rows.AddTableRow();

            TableCell cellThree = row.Cells.AddTableCell();
            cellThree.Blocks.AddParagraph().Inlines.AddRun("3");

            TableCell cellFour = row.Cells.AddTableCell();
            cellFour.Blocks.AddParagraph().Inlines.AddRun("4");

            // Act
            RadFlowDocument imported = this.ImportHtml("Table_WithTwoRowsAndFourCells.html");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_WhenTableHasNestedTable_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            table = cellOne.Blocks.AddTable();

            row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("NestedTables");

            // Act
            RadFlowDocument imported = this.ImportHtml("NestedTables.html");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_WithTableCellHeaders_Import_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            cellOne.Blocks.AddParagraph().Inlines.AddRun("1");

            TableCell cellTwo = row.Cells.AddTableCell();
            cellTwo.Blocks.AddParagraph().Inlines.AddRun("2");

            // Act
            RadFlowDocument imported = this.ImportHtml("TableWithTableCellHeaders.html");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_WhenTableHasTwoRowsAndFourCellsWithContent_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            cellOne.Blocks.AddParagraph().Inlines.AddRun("1");

            TableCell cellTwo = row.Cells.AddTableCell();
            cellTwo.Blocks.AddParagraph().Inlines.AddRun("2");

            row = table.Rows.AddTableRow();

            TableCell cellThree = row.Cells.AddTableCell();
            cellThree.Blocks.AddParagraph().Inlines.AddRun("3");

            TableCell cellFour = row.Cells.AddTableCell();
            cellFour.Blocks.AddParagraph().Inlines.AddRun("4");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void Table_WhenTable_HasNestedTable_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Table table = expected.Sections.AddSection().Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cellOne = row.Cells.AddTableCell();
            table = cellOne.Blocks.AddTable();

            row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph().Inlines.AddRun("NestedTables");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }
    }
}
