﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.HtmlImportExport
{
    [TestClass]
    public class UnknownTagTests : HtmlTestBase
    {
        protected override string FolderName
        {
            get { throw new NotImplementedException(); }
        }

        [TestMethod]
        public void Test_UnknownTagInHeader_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection();

            RadFlowDocument imported = this.ImportHtmlFromString("<html><head><script>Invisible Content</script></head></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_UnknownTagInBody_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("Content");

            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><mytag>Content</mytag></body></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_TwoParagraphsInUnknownTag_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph().Inlines.AddRun("first");
            section.Blocks.AddParagraph().Inlines.AddRun("first");

            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><mytag><p>first</p><p>first</p></mytag></body></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }

        [TestMethod]
        public void Test_TableInUnknownTag_Import()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("Table");

            RadFlowDocument imported = this.ImportHtmlFromString("<html><body><mytag><table><tr><td>Table</td></tr></table></body></html>");

            this.AssertDocumentsStructureOnly(expected, imported);
        }
    }
}
