﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Breaks
{
    [TestClass]
    public class BreaksTests : PropertiesTestsBase
    {
        public const string TestFilesFolder = "Breaks/";

        [TestMethod]
        public void LineBreak_WhenDocumentIsEmpty_Import_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}EmptyDocumentOnlyLineBreak.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void LineBreak_WhenDocumentIsEmpty_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}EmptyDocumentOnlyLineBreak.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void LineBreak_WhenDocumentIsEmpty_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.LineBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LineBreak_WhenDocumentIsEmpty_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.LineBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LineBreak_WhenDocumentIsEmpty_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertBreak(BreakType.LineBreak);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentIsEmpty_Import()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}EmptyDocumentOnlyPageBreak.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentIsEmpty_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}EmptyDocumentOnlyPageBreak.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentIsEmpty_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentIsEmpty_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentIsEmpty_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertBreak(BreakType.PageBreak);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentContainsSingleInlineBeforeBreak_Import_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("inline");
            editor.InsertParagraph();
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleInlinePageBreak.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentContainsSingleInlineBeforeBreak_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("inline");
            editor.InsertParagraph();
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleInlinePageBreak.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentContainsSingleInlineBeforeBreak_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("inline");
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentContainsSingleInlineBeforeBreak_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("inline");
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenDocumentContainsSingleInlineBeforeBreak_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("inline");
            editor.InsertBreak(BreakType.PageBreak);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void LineBreak_WhenIsInsertedBetweenTwoInlines_Import()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("first ");
            editor.InsertText("second");
            editor.MoveToInlineEnd(run);
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LineBreakBetweenTwoInlines.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void LineBreak_WhenIsInsertedBetweenTwoInlines_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("first ");
            editor.InsertText("second");
            editor.MoveToInlineEnd(run);
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LineBreakBetweenTwoInlines.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void LineBreak_WhenIsInsertedBetweenTwoInlines_ExportImport()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("first ");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("second");

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LineBreak_WhenIsInsertedBetweenTwoInlines_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("first ");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("second");

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LineBreak_WhenIsInsertedBetweenTwoInlines_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first ");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("second");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void PageBreak_WhenIsInsertedBetweenTwoInlines_Import_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("some");
            editor.InsertText("text");
            editor.MoveToInlineEnd(run);
            editor.InsertParagraph();
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}PageBreakBetweenTwoInlines.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenIsInsertedBetweenTwoInlines_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("some");
            editor.InsertText("text");
            editor.MoveToInlineEnd(run);
            editor.InsertParagraph();
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}PageBreakBetweenTwoInlines.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void PageBreak_WhenIsInsertedBetweenTwoInlines_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("first ");
            editor.InsertText("second");
            editor.MoveToInlineEnd(run);
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenIsInsertedBetweenTwoInlines_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run run = editor.InsertText("first ");
            editor.InsertText("second");
            editor.MoveToInlineEnd(run);
            editor.InsertBreak(BreakType.PageBreak);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void PageBreak_WhenIsInsertedBetweenTwoInlines_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("first ");
            editor.InsertText("second");
            editor.MoveToInlineEnd(run);
            editor.InsertBreak(BreakType.PageBreak);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_Import_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            Run run = new Run(editor.Document); // <w:t xml:space="preserve" />
            run.Text = string.Empty;
            editor.InsertInline(run);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LineBreakDifferentClearValues.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LineBreakDifferentClearValues.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText("just text");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_AndTextWrappingIsAppliedOnMultipleLineBreaksInSameGroup_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("clear left again ");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("linebreak after lbr3 and plain ");
            editor.InsertBreak(BreakType.LineBreak);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText(" clear diff group ");
            editor.InsertBreak(BreakType.LineBreak);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LineBreakDifferentClearValuesAndSameLineBreaksInGroup.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleLineBreaks_WhenDifferentClearAttributeValuesAreSet_AndTextWrappingIsAppliedOnMultipleLineBreaksInSameGroup_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Clear none.");
            editor.InsertBreak(BreakType.LineBreak);
            editor.InsertText("Clear left.");
            Break br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("clear left again ");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnLeft;
            editor.InsertInline(br);

            editor.InsertText("Clear all");
            editor.InsertText(".");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextFullLine;
            editor.InsertInline(br);

            editor.InsertText("linebreak after lbr3 and plain ");
            editor.InsertBreak(BreakType.LineBreak);

            editor.InsertText("Clear right.");
            br = new Break(editor.Document);
            br.TextWrappingRestartLocation = TextWrappingRestartLocation.NextTextRegionUnblockedOnRight;
            editor.InsertInline(br);

            editor.InsertText(" clear diff group ");
            editor.InsertBreak(BreakType.LineBreak);

            editor.InsertText("just text");

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void ColumnBreak_WhenIsInsertedAfterSingleInline_Import_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("text");
            editor.InsertBreak(BreakType.ColumnBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleInlineColumnBreak.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void ColumnBreak_WhenIsInsertedAfterSingleInline_Import_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("text");
            editor.InsertBreak(BreakType.ColumnBreak);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleInlineColumnBreak.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void ColumnBreak_WhenIsInsertedAfterSingleInline_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("text");
            editor.InsertBreak(BreakType.ColumnBreak);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void ColumnBreak_WhenIsInsertedAfterSingleInline_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("text");
            editor.InsertBreak(BreakType.ColumnBreak);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void ColumnBreak_WhenIsInsertedAfterSingleInline_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("text");
            editor.InsertBreak(BreakType.ColumnBreak);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }
    }
}