using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using BaselineAlignment = Telerik.Windows.Documents.Flow.Model.Styles.BaselineAlignment;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class BaselineAlignmentTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void BaselineAlignment_Superscript_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Superscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void BaselineAlignment_Superscript_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Superscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void BaselineAlignment_Superscript_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Superscript;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void BaselineAlignment_Subscript_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Subscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void BaselineAlignment_Subscript_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Subscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void BaselineAlignment_Subscript_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.BaselineAlignment = BaselineAlignment.Subscript;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void BaselineAlignment_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph paragraph = doc.Sections.AddSection().Blocks.AddParagraph();
            Run run1 = paragraph.Inlines.AddRun();
            run1.Text = "super";
            run1.BaselineAlignment = BaselineAlignment.Superscript;

            Run run2 = paragraph.Inlines.AddRun();
            run2.Text = "sub";
            run2.BaselineAlignment = BaselineAlignment.Subscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void BaselineAlignment_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph paragraph = doc.Sections.AddSection().Blocks.AddParagraph();
            Run run1 = paragraph.Inlines.AddRun();
            run1.Text = "super";
            run1.BaselineAlignment = BaselineAlignment.Superscript;

            Run run2 = paragraph.Inlines.AddRun();
            run2.Text = "sub";
            run2.BaselineAlignment = BaselineAlignment.Subscript;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }
        
        [TestMethod]
        public void BaselineAlignment_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph paragraph = doc.Sections.AddSection().Blocks.AddParagraph();
            Run run1 = paragraph.Inlines.AddRun();
            run1.Text = "super";
            run1.BaselineAlignment = BaselineAlignment.Superscript;

            Run run2 = paragraph.Inlines.AddRun();
            run2.Text = "sub";
            run2.BaselineAlignment = BaselineAlignment.Subscript;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void BaselineAlignment_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run1 = paragraph.Inlines.AddRun();
            run1.Text = "super";
            run1.BaselineAlignment = BaselineAlignment.Superscript;

            Run run2 = paragraph.Inlines.AddRun();
            run2.Text = "sub";
            run2.BaselineAlignment = BaselineAlignment.Subscript;

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}BaselineAlignment.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void BaselineAlignment_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run1 = paragraph.Inlines.AddRun();
            run1.Text = "super";
            run1.BaselineAlignment = BaselineAlignment.Superscript;

            Run run2 = paragraph.Inlines.AddRun();
            run2.Text = "sub";
            run2.BaselineAlignment = BaselineAlignment.Subscript;

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}BaselineAlignment.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }
    }
}