#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class CharacterShadingTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void CharacterShading_LocalColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            run.Shading.PatternColor = new ThemableColor(Colors.Blue);
            run.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void CharacterShading_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            run.Shading.PatternColor = new ThemableColor(Colors.Blue);
            run.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void CharacterShading_LocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            run.Shading.PatternColor = new ThemableColor(Colors.Blue);
            run.Shading.Pattern = ShadingPattern.Percent40;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void CharacterShading_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            run.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            run.Shading.Pattern = ShadingPattern.DiagonalCross;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void CharacterShading_ThemeColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            run.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            run.Shading.Pattern = ShadingPattern.DiagonalCross;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void CharacterShading_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "CharacterShading";
            run.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            run.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            run.Shading.Pattern = ShadingPattern.HorizontalStripe;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterShadingLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void CharacterShading_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "CharacterShading";
            run.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Accent6);
            run.Shading.PatternColor = new ThemableColor(ThemeColorType.Accent5, 0.8);
            run.Shading.Pattern = ShadingPattern.HorizontalStripe;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterShadingThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }
    }
}