﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class FlowDirectionTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void FlowDirection_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Run run = editor.InsertText("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(document, new RtfFormatProvider());
        }

        [TestMethod]
        public void FlowDirection_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Run run = editor.InsertText("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(document, new DocxFormatProvider());
        }

        [TestMethod]
        public void FlowDirection_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(original);
            Run run = editor.InsertText("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FlowDirection.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FlowDirection_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(original);
            Run run = editor.InsertText("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;
            run.FontFamily = new ThemableFontFamily("Arial (Hebrew)");

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FlowDirection.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FlowDirection_Validate_Xml()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Run run = editor.InsertText("שדג:3");
            run.FlowDirection = FlowDirection.RightToLeft;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
