using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class FontFamilyTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void FontFamily_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontFamily = new ThemableFontFamily("Courier New");

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FontFamily_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontFamily = new ThemableFontFamily("Courier New");

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FontFamily_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontFamily = new ThemableFontFamily("Courier New");

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void FontFamily_WithTheme_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run1 = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run1.Text = "major";
            run1.FontFamily = new ThemableFontFamily(ThemeFontType.Major);

            Run run2 = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run2.Text = "minor";
            run2.FontFamily = new ThemableFontFamily(ThemeFontType.Minor);

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FontFamily_WithTheme_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run1 = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run1.Text = "major";
            run1.FontFamily = new ThemableFontFamily(ThemeFontType.Major);

            Run run2 = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run2.Text = "minor";
            run2.FontFamily = new ThemableFontFamily(ThemeFontType.Minor);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}