using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class FontSizeTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void FontSize_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 36;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FontSize_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 36;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FontSize_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 36;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void FontSize_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 36;

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FontSize.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FontSize_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 36;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FontSize.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FontSize_WhenValueIsDouble_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun("Test10.03");
            run.FontSize = Unit.PointToDip(10.03 / 2);

            Run run2 = paragraph.Inlines.AddRun("Test20.5");
            run2.FontSize = Unit.PointToDip(20.5 / 2);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FontSizeDoubleValue.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertCharcterProperties(original, imported);
        }
    }
}