#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class FontStyleTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void FontStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontStyle = FontStyles.Italic;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FontStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontStyle = FontStyles.Italic;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FontStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "FontStyle";
            run.FontStyle = FontStyles.Italic;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FontStyle.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FontStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "FontStyle";
            run.FontStyle = FontStyles.Italic;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}FontStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void FontStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontStyle = FontStyles.Italic;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}