using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class ForegroundColorTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void ForegroundColor_LocalColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Green);

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ForegroundColor_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Green);

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ForegroundColor_LocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Green);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ForegroundColor_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Black, true);

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ForegroundColor_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Black, true);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ForegroundColor_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(ThemeColorType.Text1, -0.30);

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ForegroundColor_ThemeColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(ThemeColorType.Text1, -0.30);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ForegroundColor_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "ForegroundColor";
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ForegroundColorLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void ForegroundColor_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "ForegroundColor";
            run.ForegroundColor = new ThemableColor(ThemeColorType.Accent6);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ForegroundColorThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void ForegroundColor_AutomaticColor_Import_Rtf()
        {
            // Arrange
            string rtfString = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fnil\fcharset0 Arial;}}
{\colortbl ;\red255\green0\blue0;}
\viewkind4\uc1\pard\f0\fs24\highlight1 RedHighlight \highlight0 TransparentHighlight
\par\f0\cf0 BlackForeground \par
}";

            // Act
            RadFlowDocument imported = new RtfFormatProvider().Import(rtfString);

            Run foregroundRun = imported.EnumerateChildrenOfType<Run>().Where(s => s.Text.Contains("BlackForeground")).FirstOrDefault();

            // Assert
            Assert.AreEqual(new ThemableColor(Colors.Black, true), foregroundRun.ForegroundColor);
        }

        [TestMethod]
        public void ForegroundColor_WhenIsTransparent_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = new Paragraph(document);
            Run run = new Run(document);
            run.Text = "test";
            run.ForegroundColor = new ThemableColor(Colors.Transparent);
            paragraph.Inlines.Add(run);
            document.Sections.AddSection().Blocks.Add(paragraph);

            // Act
            RadFlowDocument imported = new RtfFormatProvider().Import(new RtfFormatProvider().Export(document));

            // Assert
            Run importedRun = imported.EnumerateChildrenOfType<Run>().FirstOrDefault();
            Assert.AreEqual(new ThemableColor(Colors.Black, false), importedRun.ForegroundColor);
        }

        [TestMethod]
        public void ForegroundColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "ForegroundColor";
            run.ForegroundColor = new ThemableColor(Colors.Red);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ForegroundColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }
    }
}