using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.CharacterProperties
{
    [TestClass]
    public class HighlightColorTests : CharacterPropertiesTestsBase
    {
        [TestMethod]
        public void HighlightColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.HighlightColor = Colors.Yellow;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void HighlightColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.HighlightColor = Colors.Yellow;

            // Act and Assert
            this.CharacterPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void HighlightColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "HighlightColor";
            run.HighlightColor = Colors.Red;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}HighlightColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void HighlightColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Run run = original.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "HighlightColor";
            run.HighlightColor = Colors.Red;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}HighlightColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertCharcterProperties(original, imported);
        }

        [TestMethod]
        public void HighlightColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.HighlightColor = Colors.Yellow;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void HighlightColor_AutomaticColor_Import_Rtf()
        {
            // Arrange
            string rtfString = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fnil\fcharset0 Arial;}}
{\colortbl ;\red255\green0\blue0;}
\viewkind4\uc1\pard\f0\fs24\highlight1 RedHighlight \highlight0 TransparentHighlight
\par\f0\cf0 BlackForeground \par
}";

            // Act
            RadFlowDocument imported = new RtfFormatProvider().Import(rtfString);

            Run highlightRun = imported.EnumerateChildrenOfType<Run>().Where(s => s.Text.Contains("TransparentHighlight")).FirstOrDefault();

            // Assert
            Assert.AreEqual(Run.HighlightColorPropertyDefinition.DefaultValue.Value, highlightRun.HighlightColor);
        }

        [TestMethod]
        public void HighlightColor_WhenIsTransparent_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = new Paragraph(document);
            Run run = new Run(document);
            run.Text = "test";
            run.HighlightColor = Colors.Transparent;
            paragraph.Inlines.Add(run);
            document.Sections.AddSection().Blocks.Add(paragraph);

            // Act
            RadFlowDocument imported = new RtfFormatProvider().Import(new RtfFormatProvider().Export(document));

            // Assert
            Run importedRun = imported.EnumerateChildrenOfType<Run>().FirstOrDefault();
            Assert.AreEqual(Run.HighlightColorPropertyDefinition.DefaultValue.Value, importedRun.HighlightColor);
        }
    }
}