﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Tables;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Comments
{
    [TestClass]
    public class CommentsTests : PropertiesTestsBase
    {
        public const string TestFilesFolder = "Comments/";

        private void AssignDocumentCommentInfo(Comment comment)
        {
            comment.Author = "Todor Mitev";
            comment.Initials = "TM";
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundSingleInline_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");
            Comment comment = editor.InsertComment("Single Comment around.", run, run);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleCommentAroundInline.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundSingleInline_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");
            Comment comment = editor.InsertComment("Single Comment around.", run, run);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleCommentAroundInline.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundSingleInline_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");
            Comment comment = editor.InsertComment("Single Comment around.", run, run);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundSingleInline_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");
            Comment comment = editor.InsertComment("Single Comment around.", run, run);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundSingleInline_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");
            Comment comment = editor.InsertComment("Single Comment around.", run, run);
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void CommentStartEnd_WhenDocumentIsEmpty_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("Comment reference only.");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}OnlyCommentReference.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void CommentStartEnd_WhenDocumentIsEmpty_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("Comment reference only.");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}OnlyCommentReference.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void CommentStartEnd_WhenDocumentIsEmpty_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("Comment start and end only.");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void CommentStartEnd_WhenDocumentIsEmpty_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("Comment start and end only.");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void CommentStartEnd_WhenDocumentIsEmpty_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("Comment start and end only.");
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundInlinesOnTwoParagraphs_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first");
            editor.InsertText(" ");
            Run run1 = editor.InsertText("second");
            editor.InsertParagraph();
            Run run2 = editor.InsertText("third");
            editor.InsertText(" fourth");
            Comment comment = editor.InsertComment("Single Comment 2 paragraphs.", run1, run2);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleCommentAroundInlinesOnTwoParagraphs.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundInlinesOnTwoParagraphs_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first ");
            Run run1 = editor.InsertText("second");
            editor.InsertParagraph();
            Run run2 = editor.InsertText("third");
            editor.InsertText(" fourth");
            Comment comment = editor.InsertComment("Single Comment 2 paragraphs.", run1, run2);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}SingleCommentAroundInlinesOnTwoParagraphs.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundInlinesOnTwoParagraphs_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first");
            Run run1 = editor.InsertText("second");
            editor.InsertParagraph();
            Run run2 = editor.InsertText("third");
            editor.InsertText(" fourth");
            Comment comment = editor.InsertComment("Single Comment 2 paragraphs.", run1, run2);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundInlinesOnTwoParagraphs_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first");
            Run run1 = editor.InsertText("second");
            editor.InsertParagraph();
            Run run2 = editor.InsertText("third");
            editor.InsertText(" fourth");
            Comment comment = editor.InsertComment("Single Comment 2 paragraphs.", run1, run2);
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void SingleComment_WhenIsAroundInlinesOnTwoParagraphs_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("first");
            Run run1 = editor.InsertText("second");
            editor.InsertParagraph();
            Run run2 = editor.InsertText("third");
            editor.InsertText(" fourth");
            Comment comment = editor.InsertComment("Single Comment 2 paragraphs.", run1, run2);
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundDifferentInlines_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run1 = editor.InsertText("first");
            Comment comment1 = editor.InsertComment("First comment", run1, run1);
            this.AssignDocumentCommentInfo(comment1);
            editor.InsertText(" second");
            Paragraph paragraph = editor.InsertParagraph();
            editor.InsertText("third");
            editor.InsertText(" ");

            Comment comment2 = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment2);
            Paragraph commentPar = comment2.Blocks.AddParagraph();
            commentPar.Inlines.AddRun("Second ");
            commentPar.Inlines.AddRun("comment");
            paragraph.Inlines.Add(comment2.CommentRangeStart);
            paragraph.Inlines.AddRun("fourth");
            paragraph.Inlines.Add(comment2.CommentRangeEnd);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentsAroundDifferentInlines.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundDifferentInlines_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run1 = editor.InsertText("first");
            Comment comment1 = editor.InsertComment("First comment", run1, run1);
            this.AssignDocumentCommentInfo(comment1);
            editor.InsertText(" second");
            Paragraph paragraph = editor.InsertParagraph();
            editor.InsertText("third ");

            Comment comment2 = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment2);
            Paragraph commentPar = comment2.Blocks.AddParagraph();
            commentPar.Inlines.AddRun("Second comment");
            paragraph.Inlines.Add(comment2.CommentRangeStart);
            paragraph.Inlines.AddRun("fourth");
            paragraph.Inlines.Add(comment2.CommentRangeEnd);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentsAroundDifferentInlines.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundDifferentInlines_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run1 = editor.InsertText("first");
            Comment comment1 = editor.InsertComment("First comment", run1, run1);
            this.AssignDocumentCommentInfo(comment1);

            editor.InsertText(" second");
            editor.InsertParagraph();
            editor.InsertText("third ");

            Comment comment2 = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment2);
            comment2.Blocks.AddParagraph().Inlines.AddRun("Second comment");
            Run run2 = editor.InsertText("fourth");
            editor.InsertComment(comment2, run2, run2);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundDifferentInlines_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run1 = editor.InsertText("first");
            Comment comment1 = editor.InsertComment("First comment", run1, run1);
            this.AssignDocumentCommentInfo(comment1);

            editor.InsertText(" second");
            editor.InsertParagraph();
            editor.InsertText("third ");

            Comment comment2 = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment2);
            comment2.Blocks.AddParagraph().Inlines.AddRun("Second comment");
            Run run2 = editor.InsertText("fourth");
            editor.InsertComment(comment2, run2, run2);

            // Act
            RadFlowDocument importedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundDifferentInlines_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run1 = editor.InsertText("first");
            Comment comment1 = editor.InsertComment("First comment", run1, run1);
            this.AssignDocumentCommentInfo(comment1);

            editor.InsertText(" second");
            editor.InsertParagraph();
            editor.InsertText("third ");

            Comment comment2 = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment2);
            comment2.Blocks.AddParagraph().Inlines.AddRun("Second comment");
            Run run2 = editor.InsertText("fourth");
            editor.InsertComment(comment2, run2, run2);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundSameInline_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("first");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("second");
            this.AssignDocumentCommentInfo(c2);

            Comment c3 = editor.Document.Comments.AddComment();
            c3.Blocks.AddParagraph().Inlines.AddRun("third");
            this.AssignDocumentCommentInfo(c3);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertInline(c3.CommentRangeStart);
            editor.InsertText("around");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);
            editor.InsertInline(c3.CommentRangeEnd);

            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentsAroundSameInline.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundSameInline_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("first");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("second");
            this.AssignDocumentCommentInfo(c2);

            Comment c3 = editor.Document.Comments.AddComment();
            c3.Blocks.AddParagraph().Inlines.AddRun("third");
            this.AssignDocumentCommentInfo(c3);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c3.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("around");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c3.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);


            // Act
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentsAroundSameInline.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, importedDocument);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundSameInline_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");

            Comment c1 = editor.InsertComment("first", run, run);
            Comment c2 = editor.InsertComment("second", run, run);
            Comment c3 = editor.InsertComment("third", run, run);

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            Assert.AreEqual(3, expectedDoc.Comments.Count);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundSameInline_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");

            Comment c1 = editor.InsertComment("first", run, run);
            Comment c2 = editor.InsertComment("second", run, run);
            Comment c3 = editor.InsertComment("third", run, run);

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            Assert.AreEqual(3, expectedDoc.Comments.Count);
        }

        [TestMethod]
        public void MultipleComments_WhenAreAroundSameInline_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("around");

            Comment c1 = editor.InsertComment("first", run, run);
            Comment c2 = editor.InsertComment("second", run, run);
            Comment c3 = editor.InsertComment("third", run, run);

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleCommentReferences_InEmptyDocument_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Com ref 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("Com ref 2");
            this.AssignDocumentCommentInfo(c2);

            Comment c3 = editor.Document.Comments.AddComment();
            c3.Blocks.AddParagraph().Inlines.AddRun("Com ref 3");
            this.AssignDocumentCommentInfo(c3);

            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertInline(c3.CommentRangeStart);
            editor.InsertComment(c1);
            editor.InsertInline(c2.CommentRangeEnd);
            editor.InsertInline(c3.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentReferences.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_InEmptyDocument_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Com ref 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("Com ref 2");
            this.AssignDocumentCommentInfo(c2);

            Comment c3 = editor.Document.Comments.AddComment();
            c3.Blocks.AddParagraph().Inlines.AddRun("Com ref 3");
            this.AssignDocumentCommentInfo(c3);

            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertInline(c3.CommentRangeStart);
            editor.InsertComment(c1);
            editor.InsertInline(c2.CommentRangeEnd);
            editor.InsertInline(c3.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleCommentReferences.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_InEmptyDocument_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("Com ref 1");
            Comment c2 = editor.InsertComment("Com ref 2");
            Comment c3 = editor.InsertComment("Com ref 3");

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_InEmptyDocument_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("Com ref 1");
            Comment c2 = editor.InsertComment("Com ref 2");
            Comment c3 = editor.InsertComment("Com ref 3");

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_InEmptyDocument_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("Com ref 1");
            Comment c2 = editor.InsertComment("Com ref 2");
            Comment c3 = editor.InsertComment("Com ref 3");

            this.AssignDocumentCommentInfo(c1);
            this.AssignDocumentCommentInfo(c2);
            this.AssignDocumentCommentInfo(c3);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultipleCommentReferences_WhenAreSpreaded_InEmptyDocument_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("REFERENCE");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();

            Comment c2 = editor.InsertComment("REFERENCE 2");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertText("\t");

            Comment c3 = editor.InsertComment("REFERENCE3");
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleSpreadedCommentReferences.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_WhenAreSpreaded_InEmptyDocument_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("REFERENCE");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();

            Comment c2 = editor.InsertComment("REFERENCE 2");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertText("\t");

            Comment c3 = editor.InsertComment("REFERENCE3");
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleSpreadedCommentReferences.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_WhenAreSpreaded_InEmptyDocument_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("REFERENCE");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();

            Comment c2 = editor.InsertComment("REFERENCE 2");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertText("\t");

            Comment c3 = editor.InsertComment("REFERENCE3");
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_WhenAreSpreaded_InEmptyDocument_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("REFERENCE");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();

            Comment c2 = editor.InsertComment("REFERENCE 2");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertText("\t");

            Comment c3 = editor.InsertComment("REFERENCE3");
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void MultipleCommentReferences_WhenAreSpreaded_InEmptyDocument_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("REFERENCE");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();

            Comment c2 = editor.InsertComment("REFERENCE 2");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertText("\t");

            Comment c3 = editor.InsertComment("REFERENCE3");
            this.AssignDocumentCommentInfo(c3);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void CommentAndCommentReference_WhenCommentIsAroundInline_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("ref");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();
            Run run = editor.InsertText("inline");
            Comment c2 = editor.InsertComment("around", run, run);
            this.AssignDocumentCommentInfo(c2);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentAroundInlineAndCommentReference.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentAndCommentReference_WhenCommentIsAroundInline_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("ref");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();
            Run run = editor.InsertText("inline");
            Comment c2 = editor.InsertComment("around", run, run);
            this.AssignDocumentCommentInfo(c2);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentAroundInlineAndCommentReference.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentAndCommentReference_WhenCommentIsAroundInline_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("ref");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();
            Run run = editor.InsertText("inline");
            Comment c2 = editor.InsertComment("around", run, run);
            this.AssignDocumentCommentInfo(c2);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentAndCommentReference_WhenCommentIsAroundInline_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.InsertComment("ref");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();
            Run run = editor.InsertText("inline");
            Comment c2 = editor.InsertComment("around", run, run);
            this.AssignDocumentCommentInfo(c2);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentAndCommentReference_WhenCommentIsAroundInline_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.InsertComment("ref");
            this.AssignDocumentCommentInfo(c1);
            editor.InsertParagraph();
            Run run = editor.InsertText("inline");
            Comment c2 = editor.InsertComment("around", run, run);
            this.AssignDocumentCommentInfo(c2);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void CommentsAroundInline_WhenAreNested_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Comment 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("nested");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("inline");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}NestedCommentsAroundInline.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsAroundInline_WhenAreNested_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Comment 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("nested");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("inline");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}NestedCommentsAroundInline.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsAroundInline_WhenAreNested_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Comment 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("nested");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("inline");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsAroundInline_WhenAreNested_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Comment 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("nested");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("inline");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsAroundInline_WhenAreNested_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment c1 = editor.Document.Comments.AddComment();
            c1.Blocks.AddParagraph().Inlines.AddRun("Comment 1");
            this.AssignDocumentCommentInfo(c1);

            Comment c2 = editor.Document.Comments.AddComment();
            c2.Blocks.AddParagraph().Inlines.AddRun("nested");
            this.AssignDocumentCommentInfo(c2);

            editor.InsertInline(c1.CommentRangeStart);
            editor.InsertInline(c2.CommentRangeStart);
            editor.InsertText("inline");
            editor.InsertInline(c1.CommentRangeEnd);
            editor.InsertInline(c2.CommentRangeEnd);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void CommentsInInline_WhenAreNestedFromInToOut_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run outerRun1 = editor.InsertText("i");
            Run middleRun1 = editor.InsertText("n");
            Run innerRun = editor.InsertText("li");
            Run middleRun2 = editor.InsertText("n");
            Run outerRun2 = editor.InsertText("e");

            Comment c1 = editor.InsertComment("inner", innerRun, innerRun);
            this.AssignDocumentCommentInfo(c1);
            Comment c2 = editor.InsertComment("middle", middleRun1, middleRun2);
            this.AssignDocumentCommentInfo(c2);
            Comment c3 = editor.InsertComment("outer", outerRun1, outerRun2);
            this.AssignDocumentCommentInfo(c3);


            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentsFromInsideToOutside.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsInInline_WhenAreNestedFromInToOut_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run outerRun1 = editor.InsertText("i");
            Run middleRun1 = editor.InsertText("n");
            Run innerRun = editor.InsertText("li");
            Run middleRun2 = editor.InsertText("n");
            Run outerRun2 = editor.InsertText("e");

            Comment c1 = editor.InsertComment("inner", innerRun, innerRun);
            this.AssignDocumentCommentInfo(c1);
            Comment c2 = editor.InsertComment("middle", middleRun1, middleRun2);
            this.AssignDocumentCommentInfo(c2);
            Comment c3 = editor.InsertComment("outer", outerRun1, outerRun2);
            this.AssignDocumentCommentInfo(c3);


            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentsFromInsideToOutside.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsInInline_WhenAreNestedFromInToOut_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run outerRun1 = editor.InsertText("i");
            Run middleRun1 = editor.InsertText("n");
            Run innerRun = editor.InsertText("li");
            Run middleRun2 = editor.InsertText("n");
            Run outerRun2 = editor.InsertText("e");

            Comment c1 = editor.InsertComment("inner", innerRun, innerRun);
            this.AssignDocumentCommentInfo(c1);
            Comment c2 = editor.InsertComment("middle", middleRun1, middleRun2);
            this.AssignDocumentCommentInfo(c2);
            Comment c3 = editor.InsertComment("outer", outerRun1, outerRun2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsInInline_WhenAreNestedFromInToOut_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run outerRun1 = editor.InsertText("i");
            Run middleRun1 = editor.InsertText("n");
            Run innerRun = editor.InsertText("li");
            Run middleRun2 = editor.InsertText("n");
            Run outerRun2 = editor.InsertText("e");

            Comment c1 = editor.InsertComment("inner", innerRun, innerRun);
            this.AssignDocumentCommentInfo(c1);
            Comment c2 = editor.InsertComment("middle", middleRun1, middleRun2);
            this.AssignDocumentCommentInfo(c2);
            Comment c3 = editor.InsertComment("outer", outerRun1, outerRun2);
            this.AssignDocumentCommentInfo(c3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void CommentsInInline_WhenAreNestedFromInToOut_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run outerRun1 = editor.InsertText("i");
            Run middleRun1 = editor.InsertText("n");
            Run innerRun = editor.InsertText("li");
            Run middleRun2 = editor.InsertText("n");
            Run outerRun2 = editor.InsertText("e");

            Comment c1 = editor.InsertComment("inner", innerRun, innerRun);
            this.AssignDocumentCommentInfo(c1);
            Comment c2 = editor.InsertComment("middle", middleRun1, middleRun2);
            this.AssignDocumentCommentInfo(c2);
            Comment c3 = editor.InsertComment("outer", outerRun1, outerRun2);
            this.AssignDocumentCommentInfo(c3);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenHoldsTable_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("inline");
            Comment comment = editor.InsertComment("Table:", run, run);
            this.AssignDocumentCommentInfo(comment);

            Table table = comment.Blocks.AddTable();
            TableRow row;
            for (int i = 0; i < 3; i++)
            {
                row = table.Rows.AddTableRow();
                for (int j = 0; j < 3; j++)
                {
                    row.Cells.AddTableCell().Blocks.AddParagraph();
                }
            }
            comment.Blocks.AddParagraph();

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}TableComment.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenHoldsTable_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("inline");
            Comment comment = editor.InsertComment("Table:", run, run);
            this.AssignDocumentCommentInfo(comment);

            Table table = comment.Blocks.AddTable();
            TableRow row;
            for (int i = 0; i < 3; i++)
            {
                row = table.Rows.AddTableRow();
                for (int j = 0; j < 3; j++)
                {
                    row.Cells.AddTableCell();
                }
            }

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}TableComment.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenHoldsTable_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("inline");
            Comment comment = editor.InsertComment("Table:", run, run);
            this.AssignDocumentCommentInfo(comment);

            Table table = comment.Blocks.AddTable();
            TableHelper.FillTable(table, 3, 3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenHoldsTable_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("inline");
            Comment comment = editor.InsertComment("Table:", run, run);
            this.AssignDocumentCommentInfo(comment);

            Table table = comment.Blocks.AddTable();
            TableHelper.FillTable(table, 3, 3);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenHoldsTable_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("inline");
            Comment comment = editor.InsertComment("Table:", run, run);
            this.AssignDocumentCommentInfo(comment);

            Table table = comment.Blocks.AddTable();
            TableHelper.FillTable(table, 3, 3);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenIsInTable_AndCommentRangeEndIsBetweenTableCells_Import_Docx()
        {
            // Arrange && Act
            RadFlowDocument importedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentInTable.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            Assert.AreEqual(1, importedDoc.Comments.Count);

            Assert.IsTrue(importedDoc.EnumerateChildrenOfType<TableCell>()
                .Where(c => c.GridRowIndex == 0 && c.GridColumnIndex == 0).FirstOrDefault()
                .Blocks.First().EnumerateChildrenOfType<CommentRangeStart>().Any());

            Assert.IsTrue(importedDoc.EnumerateChildrenOfType<TableCell>()
                .Where(c => c.GridRowIndex == 0 && c.GridColumnIndex == 2).FirstOrDefault()
                .Blocks.First().EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }

        [TestMethod]
        public void Comment_WhenIsInTable_AndCommentRangeEndIsBetweenTableCells_Import_Rtf()
        {
            // Arrange && Act
            RadFlowDocument importedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentInTable.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            Assert.AreEqual(1, importedDoc.Comments.Count);

            Assert.IsTrue(importedDoc.EnumerateChildrenOfType<TableCell>()
                .Where(c => c.GridRowIndex == 0 && c.GridColumnIndex == 0).FirstOrDefault()
                .Blocks.First().EnumerateChildrenOfType<CommentRangeStart>().Any());

            Assert.IsTrue(importedDoc.EnumerateChildrenOfType<TableCell>()
                .Where(c => c.GridRowIndex == 0 && c.GridColumnIndex == 2).FirstOrDefault()
                .Blocks.First().EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }

        [TestMethod]
        public void Comment_WhenIsInTable_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("First row");
            this.AssignDocumentCommentInfo(comment);

            Table table = editor.InsertTable(2, 4);
            table.Rows.First().Cells[0].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeStart);
            table.Rows.First().Cells[3].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenIsInTable_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("First row");
            this.AssignDocumentCommentInfo(comment);

            Table table = editor.InsertTable(2, 4);
            table.Rows.First().Cells[0].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeStart);
            table.Rows.First().Cells[3].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenIsInTable_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("First row");
            this.AssignDocumentCommentInfo(comment);

            Table table = editor.InsertTable(2, 4);
            table.Rows.First().Cells[0].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeStart);
            table.Rows.First().Cells[3].Blocks.AddParagraph().Inlines.Add(comment.CommentRangeEnd);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenIsInHeader_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Header");
            this.AssignDocumentCommentInfo(comment);

            Header header = editor.Document.Sections.First().Headers.Add();
            Paragraph headerParagraph = header.Blocks.AddParagraph();
            headerParagraph.Inlines.Add(comment.CommentRangeStart);
            headerParagraph.Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenIsInHeader_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Header");
            this.AssignDocumentCommentInfo(comment);

            Header header = editor.Document.Sections.First().Headers.Add();
            Paragraph headerParagraph = header.Blocks.AddParagraph();
            headerParagraph.Inlines.Add(comment.CommentRangeStart);
            headerParagraph.Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        //[TestMethod]
        public void Comment_WhenIsInHeader_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Header");
            this.AssignDocumentCommentInfo(comment);

            Header header = editor.Document.Sections.First().Headers.Add();
            Paragraph headerParagraph = header.Blocks.AddParagraph();
            headerParagraph.Inlines.Add(comment.CommentRangeStart);
            headerParagraph.Inlines.Add(comment.CommentRangeEnd);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenIsInFooter_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Footer");
            this.AssignDocumentCommentInfo(comment);

            Footer footer = editor.Document.Sections.First().Footers.Add();
            Paragraph footerParagraph = footer.Blocks.AddParagraph();
            footerParagraph.Inlines.Add(comment.CommentRangeStart);
            footerParagraph.Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenIsInFooter_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Footer");
            this.AssignDocumentCommentInfo(comment);

            Footer footer = editor.Document.Sections.First().Footers.Add();
            Paragraph footerParagraph = footer.Blocks.AddParagraph();
            footerParagraph.Inlines.Add(comment.CommentRangeStart);
            footerParagraph.Inlines.Add(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        //[TestMethod]
        public void Comment_WhenIsInFooter_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("Footer");
            this.AssignDocumentCommentInfo(comment);

            Footer footer = editor.Document.Sections.First().Footers.Add();
            Paragraph footerParagraph = footer.Blocks.AddParagraph();
            footerParagraph.Inlines.Add(comment.CommentRangeStart);
            footerParagraph.Inlines.Add(comment.CommentRangeEnd);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenContainsImage_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("text");
            Comment comment = editor.Document.Comments.AddComment();
            editor.InsertComment(comment, run, run);
            comment.Blocks.AddParagraph().Inlines.Add(TestHelper.CreatePngImage(editor.Document));
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}ImageComment.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenContainsImage_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("text");
            Comment comment = editor.Document.Comments.AddComment();
            editor.InsertComment(comment, run, run);
            comment.Blocks.AddParagraph().Inlines.Add(TestHelper.CreatePngImage(editor.Document));
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}ImageComment.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenContainsImage_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("text");
            Comment comment = editor.Document.Comments.AddComment();
            editor.InsertComment(comment, run, run);
            comment.Blocks.AddParagraph().Inlines.Add(TestHelper.CreatePngImage(editor.Document));
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenContainsImage_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("text");
            Comment comment = editor.Document.Comments.AddComment();
            editor.InsertComment(comment, run, run);
            comment.Blocks.AddParagraph().Inlines.Add(TestHelper.CreatePngImage(editor.Document));
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenContainsImage_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("text");
            Comment comment = editor.Document.Comments.AddComment();
            editor.InsertComment(comment, run, run);
            comment.Blocks.AddParagraph().Inlines.Add(TestHelper.CreatePngImage(editor.Document));
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenCommentReferenceIsBeforeStartEndRanges_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("f");
            editor.InsertText("irst");
            editor.InsertText(" ");
            Run run = editor.InsertText("second");
            Comment comment = editor.InsertComment("comment.", run, run);
            editor.InsertText(" third");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentReferenceBeforeStartEndRanges.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenCommentReferenceIsBeforeStartEndRanges_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("f");
            editor.InsertText("irst");
            editor.InsertText(" ");
            Run run = editor.InsertText("second");
            Comment comment = editor.InsertComment("comment.", run, run);
            editor.InsertText(" third");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentReferenceBeforeStartEndRanges.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenCommentReferenceIsBetweenStartEndRanges_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("f");
            editor.InsertText("irst");
            editor.InsertText(" ");
            Run run = editor.InsertText("second");
            Comment comment = editor.InsertComment("comment.", run, run);
            editor.InsertText(" third");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentReferenceBetweenStartEndRanges.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenCommentReferenceIsBetweenStartEndRanges_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("f");
            editor.InsertText("irst");
            editor.InsertText(" ");
            Run run = editor.InsertText("second");
            Comment comment = editor.InsertComment("comment.", run, run);
            editor.InsertText(" third");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDoc = TestHelper.GetDocumentFromFile(string.Format("{0}CommentReferenceBetweenStartEndRanges.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
        }

        [TestMethod]
        public void Comment_WhenCommentReferenceIsBetweenStartEndRanges_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("f");
            editor.InsertText("irst");
            editor.InsertText(" ");
            Run run = editor.InsertText("second");
            Comment comment = editor.InsertComment("comment.", run, run);
            editor.InsertText(" third");
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenOnlyCommentRangeEndExists_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            editor.InsertInline(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void Comment_WhenOnlyCommentRangeEndExists_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            editor.InsertInline(comment.CommentRangeEnd);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
        }

        [TestMethod]
        public void Comment_WhenOnlyCommentRangeEndExists_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            editor.InsertInline(comment.CommentRangeEnd);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Comment_WhenExistsOnlyInCommentsCollection_IsNotExported_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);
            Assert.IsFalse(expectedDocument.Document.EnumerateChildrenOfType<CommentRangeStart>().Any());
            Assert.IsFalse(expectedDocument.Document.EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }

        [TestMethod]
        public void Comment_WhenExistsOnlyInCommentsCollection_IsNotExported_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            // Act
            RadFlowDocument expectedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDocument);

            Assert.IsFalse(expectedDocument.Document.EnumerateChildrenOfType<CommentRangeStart>().Any());
            Assert.IsFalse(expectedDocument.Document.EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }

        [TestMethod]
        public void Comment_WhenExistsOnlyInCommentsCollection_IsNotExported_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Comment comment = editor.Document.Comments.AddComment();
            comment.Blocks.AddParagraph().Inlines.AddRun("comment");
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        // DateTime
        [TestMethod]
        public void Comment_DateTimeTest_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("test");
            Comment comment = editor.InsertComment("test comment", run, run);
            comment.Date = DateTime.Now;
            this.AssignDocumentCommentInfo(comment);

            // Act 
            RadFlowDocument exportedDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            DateTime exportedDate = exportedDocument.Comments.First().Date;
            DateTime expectedDate = comment.Date;

            Assert.AreEqual(expectedDate.Year, exportedDate.Year);
            Assert.AreEqual(expectedDate.Month, exportedDate.Month);
            Assert.AreEqual(expectedDate.Day, exportedDate.Day);
            Assert.AreEqual(expectedDate.Hour, exportedDate.Hour);
            Assert.AreEqual(expectedDate.Minute, exportedDate.Minute);
        }

        [TestMethod]
        public void Comment_DateTimeTest_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("test");
            Comment comment = editor.InsertComment("test comment", run, run);
            comment.Date = DateTime.Now;
            this.AssignDocumentCommentInfo(comment);

            // Act 
            RadFlowDocument exportedDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            DateTime exportedDate = exportedDocument.Comments.First().Date;
            DateTime expectedDate = comment.Date;

            Assert.AreEqual(expectedDate.Year, exportedDate.Year);
            Assert.AreEqual(expectedDate.Month, exportedDate.Month);
            Assert.AreEqual(expectedDate.Day, exportedDate.Day);
            Assert.AreEqual(expectedDate.Hour, exportedDate.Hour);
            Assert.AreEqual(expectedDate.Minute, exportedDate.Minute);
        }

        [TestMethod]
        public void Comment_DateTimeTest_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Run run = editor.InsertText("test");
            Comment comment = editor.InsertComment("test comment", run, run);
            comment.Date = DateTime.Now;
            this.AssignDocumentCommentInfo(comment);

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        // DocxValidator
        [TestMethod]
        public void Comment_WhenOnlyHasCommentStarts_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.Document.Comments.AddComment();
            this.AssignDocumentCommentInfo(comment);
            editor.InsertText("before ");
            editor.InsertInline(comment.CommentRangeStart);
            editor.InsertText("after");

            // Act
            RadFlowDocument expectedDoc = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, expectedDoc);
            Assert.IsFalse(expectedDoc.EnumerateChildrenOfType<CommentRangeStart>().Any());
            Assert.IsTrue(expectedDoc.EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }
    }
}