﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.DocumentProperties
{
    [TestClass]
    public class DefaultTabStopWidthTests : DocumentPropertiesTestBase
    {
        [TestMethod]
        public void DefaultTabStop_WhenIsSetOnDocument_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            expected.DefaultTabStopWidth = Unit.InchToDip(3);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}DefaultTabSopWidth.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertDocumentProperties(expected, imported);
        }

        [TestMethod]
        public void DefaultTabStop_WhenIsSetOnDocument_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            expected.DefaultTabStopWidth = Unit.InchToDip(3);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            this.AssertDocumentProperties(expected, imported);
        }

        [TestMethod]
        public void DefaultTabStop_WhenIsSetOnDocument_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            expected.DefaultTabStopWidth = Unit.InchToDip(3);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}DefaultTabSopWidth.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertDocumentProperties(expected, imported);
        }

        [TestMethod]
        public void DefaultTabStop_WhenIsSetOnDocument_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            expected.DefaultTabStopWidth = Unit.InchToDip(3);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            this.AssertDocumentProperties(expected, imported);
        }
    }
}
