﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.DocumentProperties
{
    [TestClass]
    public class HasDifferentEvenOddPageHeadersFootersTests : DocumentPropertiesTestBase
    {
        private const string TestFileNameWithoutExtension = TestFilesFolder + "HasDifferentEvenOddPageHeadersFooters";

        [TestMethod]
        public void HasDifferentEvenOddPageHeadersFooters_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = CreateDocumentWithDifferentEvenOddPageHeadersFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertDocumentProperties(original, imported);
        }

        [TestMethod]
        public void HasDifferentEvenOddPageHeadersFooters_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithDifferentEvenOddPageHeadersFooters();

            // Act and Assert
            this.DocumentPropertiesExportImportTemplate(document, new DocxFormatProvider());
        }

        [TestMethod]
        public void HasDifferentEvenOddPageHeadersFooters_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = CreateDocumentWithDifferentEvenOddPageHeadersFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertDocumentProperties(original, imported);
        }

        [TestMethod]
        public void HasDifferentEvenOddPageHeadersFooters_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithDifferentEvenOddPageHeadersFooters();

            // Act and Assert
            this.DocumentPropertiesExportImportTemplate(document, new RtfFormatProvider());
        }

        [TestMethod]
        public void HasDifferentEvenOddPageHeadersFooters_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithDifferentEvenOddPageHeadersFooters();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private static RadFlowDocument CreateDocumentWithDifferentEvenOddPageHeadersFooters()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.HasDifferentEvenOddPageHeadersFooters = true;

            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph();

            return expected;
        }
    }
}
