﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ExportSettings
{
    [TestClass]
    public class TestDocxExportSettings
    {
        [TestMethod]
        public void Test_InvalidDocument_None()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection();
            document.Sections.AddSection();

            DocxFormatProvider provider = new DocxFormatProvider();
            provider.ExportSettings.InvalidDocumentAction = InvalidDocumentAction.None;

            RadFlowDocument newDocument = this.ExportImport(document, provider);

            Assert.AreEqual(1, newDocument.Sections.Count);
        }

        [TestMethod]
        public void Test_InvalidDocument_Repair()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection();
            document.Sections.AddSection();

            DocxFormatProvider provider = new DocxFormatProvider();
            provider.ExportSettings.InvalidDocumentAction = InvalidDocumentAction.Repair;

            this.ExportImportAndAssert(document, provider);
        }

        [TestMethod]
        public void Test_InvalidDocument_ThrowAnException()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection();
            document.Sections.AddSection();

            DocxFormatProvider provider = new DocxFormatProvider();
            provider.ExportSettings.InvalidDocumentAction = InvalidDocumentAction.ThrowException;

            TestHelper.AssertExceptionIsThrown<InvalidDocumentException>(() => this.ExportImportAndAssert(document, provider));
        }

        private void ExportImportAndAssert(RadFlowDocument originalDoc, IFormatProvider<RadFlowDocument> provider)
        {
            //Act
            RadFlowDocument testDocument = this.ExportImport(originalDoc, provider);

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private RadFlowDocument ExportImport(RadFlowDocument originalDoc, IFormatProvider<RadFlowDocument> provider)
        {
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);

                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            return testDocument;
        }
    }
}