﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Fields
{
    [TestClass]
    public class CodeFieldsTest : TestsBase
    {
        public const string TestFilesFolder = "Fields/CodeFields/";

        [TestMethod]
        public void AuthorField_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorField_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("Before ");
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            editor.InsertText(" after");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void AuthorField_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            editor.InsertText(" after");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}AuthorField.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorFieldSimple_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorFieldSimple_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void AuthorFieldSimple_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}AuthorFieldSimple.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorField_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorField_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            editor.InsertText(" after");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}AuthorField.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorFieldSimple_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void AuthorFieldSimple_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}AuthorFieldSimple.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LockedField_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsLocked = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LockedField.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LockedField_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsLocked = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LockedField.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LockedSimpleField_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            field.IsLocked = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LockedFieldSimple.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LockedField_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsLocked = true;
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void LockedField_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsLocked = true;
            editor.InsertText(" after");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void LockedField_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsLocked = true;
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void DirtyField_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            field.IsDirty = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}DirtyField.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void DirtyField_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            field.IsDirty = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}DirtyField.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void DirtySimpleField_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            field.IsDirty = true;

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}DirtyFieldSimple.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void DirtySimpleField_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField(" AUTHOR   \\* MERGEFORMAT ", "Alexander Vakrilov");
            field.IsDirty = true;

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void DirtyField_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsDirty = true;
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void DirtyField_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsDirty = true;
            editor.InsertText(" after");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void DirtyField_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            FieldInfo field = editor.InsertField(" DATE \\@ \"hh:mm:ss\" ", "02:53:23");
            field.IsDirty = true;
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void Field_WithMissingSeparatorCharacter_Import_Docx()
        {
            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}FieldWithMIssingSeparatorCharacter.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertField_WithMissingSeparatorCharacter_Import(testDocument, "field", false);
        }

        [TestMethod]
        public void Field_WithMissingSeparatorCharacter_Import_Rtf()
        {
            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}FieldWithMIssingSeparatorCharacter.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            Paragraph paragraph = testDocument.Sections.First().Blocks.First() as Paragraph;
            Assert.AreEqual(5, paragraph.Inlines.Count);

            FieldCharacter fieldCharacter = paragraph.Inlines[0] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.Start, fieldCharacter.FieldCharacterType);

            Run run = paragraph.Inlines[2] as Run;
            Assert.AreEqual("field", run.Text.Trim());

            fieldCharacter = paragraph.Inlines[3] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.Separator, fieldCharacter.FieldCharacterType);

            fieldCharacter = paragraph.Inlines[paragraph.Inlines.Count - 2] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.Separator, fieldCharacter.FieldCharacterType);

            fieldCharacter = paragraph.Inlines[paragraph.Inlines.Count - 1] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.End, fieldCharacter.FieldCharacterType);
        }

        [TestMethod]
        public void Field_WithMissingSeparatorCharacter_ExportImport_Docx()
        {
            this.Field_WithMissingSeparatorCharacter_ExportImport(new DocxFormatProvider(), false);
        }

        [TestMethod]
        public void Field_WithMissingSeparatorCharacter_ExportImport_Rtf()
        {
            this.Field_WithMissingSeparatorCharacter_ExportImport(new RtfFormatProvider(), true);
        }

        private void Field_WithMissingSeparatorCharacter_ExportImport(IFormatProvider<RadFlowDocument> provider, bool isRtf)
        {
            // Arrange
            string fieldCode = "field";

            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            FieldInfo field = new FieldInfo(document);

            paragraph.Inlines.Add(field.Start);

            paragraph.Inlines.AddRun(fieldCode);

            paragraph.Inlines.Add(field.End);

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(document, provider);

            // Assert
            this.AssertField_WithMissingSeparatorCharacter_Import(testDocument, fieldCode, isRtf);
        }

        private void AssertField_WithMissingSeparatorCharacter_Import(RadFlowDocument testDocument, string fieldCode, bool isRtf)
        {
            Paragraph paragraph = testDocument.Sections.First().Blocks.First() as Paragraph;
            Assert.AreEqual((isRtf) ? 4 : 3, paragraph.Inlines.Count);

            FieldCharacter fieldCharacter = paragraph.Inlines[0] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.Start, fieldCharacter.FieldCharacterType);

            Run run = paragraph.Inlines[1] as Run;
            Assert.AreEqual(fieldCode, run.Text.Trim());

            if (isRtf)
            {
                fieldCharacter = paragraph.Inlines[2] as FieldCharacter;
                Assert.AreEqual(FieldCharacterType.Separator, fieldCharacter.FieldCharacterType);
            }

            fieldCharacter = paragraph.Inlines[paragraph.Inlines.Count - 1] as FieldCharacter;
            Assert.AreEqual(FieldCharacterType.End, fieldCharacter.FieldCharacterType);
        }
    }
}
