﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Fields
{
    [TestClass]
    public class DocumentVariableFieldTests : TestsBase
    {
        public const string TestFilesFolder = "Fields/DocumentVariableField/";

        [TestMethod]
        public void DocumentVariableField_Import_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test name");
            document.DocumentVariables.Add("user name", "test user");

            editor.InsertField(" docvariable name ", "test name");
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"user name\" ", "test user");
            editor.InsertParagraph();
            editor.InsertField(" docvariable  ", "result").UpdateField();
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"missing name\" ", "result").UpdateField();

            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}DocumentVariableField.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(document, testDocument);

        }

        [TestMethod]
        public void DocumentVariableField_Import_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test name");
            document.DocumentVariables.Add("user name", "test user");

            editor.InsertField(" docvariable name ", "test name");
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"user name\" ", "test user");
            editor.InsertParagraph();
            editor.InsertField(" docvariable  ", "result").UpdateField();
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"missing name\" ", "result").UpdateField();

            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}DocumentVariableField.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(document, testDocument);
        }

        [TestMethod]
        public void DocumentVariableField_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test name");
            document.DocumentVariables.Add("user name", "test user");

            editor.InsertField("docvariable name", "test name");
            editor.InsertField("docvariable \"user name\"", "test user");
            editor.InsertField("docvariable ", "result").UpdateField();
            editor.InsertField("docvariable \"missing name\"", "result").UpdateField();

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(document, testDocument);
        }

        [TestMethod]
        public void DocumentVariableField_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test name");
            document.DocumentVariables.Add("user name", "test user");

            editor.InsertField("docvariable name", "test name");
            editor.InsertField("docvariable \"user name\"", "test user");
            editor.InsertField("docvariable ", "result").UpdateField();
            editor.InsertField("docvariable \"missing name\"", "result").UpdateField();

            // Act
            RadFlowDocument testDocument = this.ExportImportDocument(document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(document, testDocument);
        }

        [TestMethod]
        public void DocumentVariableField_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test name");
            document.DocumentVariables.Add("user name", "test user");

            editor.InsertField(" docvariable name ", "test name");
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"user name\" ", "test user");
            editor.InsertParagraph();
            editor.InsertField(" docvariable  ", "result").UpdateField();
            editor.InsertParagraph();
            editor.InsertField(" docvariable \"missing name\" ", "result").UpdateField();

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
