﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Fields
{
    [TestClass]
    public class HyperlinkTests : PropertiesTestsBase
    {
        public const string TestFilesFolder = "Fields/Hyperlinks/";

        [TestMethod]
        public void Hyperlink_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("telerik", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void Hyperlink_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("telerik", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void MultiLineHyperlink_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "link", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void MultiLineHyperlink_Validate_Xml()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "link", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            XmlValidator.ValidateOpenXmlDocument(editor.Document);
        }

        [TestMethod]
        public void Hyperlink_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("hyperlink", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Hyperlink.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void MultilineHyperlink_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "Link", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultilineHyperlink.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void HyperlinkWithoutRuns_Import_Docx()
        {
            // Act
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}EmptyHyperlink.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            Assert.AreEqual(3, testDocument.EnumerateChildrenOfType<FieldCharacter>().Count());
            Assert.IsTrue(testDocument.EnumerateChildrenOfType<Run>().First().Text.Contains("HYPERLINK"));
        }

        [TestMethod]
        public void Hyperlink_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("telerik", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void MultiLineHyperlink_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "link", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            //Act
            RadFlowDocument testDocument = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void Hyperlink_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("before ");
            editor.InsertHyperlink("hyperlink", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Hyperlink.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }

        [TestMethod]
        public void MultilineHyperlink_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            editor.InsertText("Before ");
            editor.InsertHyperlink("hyper" + Environment.NewLine + "Link", "telerik.com", false, "Telerik Site");
            editor.InsertText(" after");

            // Act and Assert
            RadFlowDocument testDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultilineHyperlink.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, testDocument);
        }
    }
}
