﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.HeadersFooters
{
    [TestClass]
    public class DefaultEvenFirstHeadersFootersTests : TestsBase
    {
        private const string TestFilesFolder = "HeadersFooters/";

        [TestMethod]
        public void AllHeaderFooterTypes_Import_Rtf()
        {
            RadFlowDocument expected = CreateDocumentWithAllHeadersAndFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "AllHeadersFooters.rtf", new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void AllHeaderFooterTypes_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithAllHeadersAndFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "AllHeadersFooters.docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void AllHeaderFooterTypes_ExportImport_Rtf()
        {
            this.AllHeaderFooterTypesExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.HeadersAndFooters)]
#if !VISUAL
        public void AllHeaderFooterTypes_ExportImport_Docx()
#else
        public void AllHeaderFooterTypes_ExportImport()
#endif
        {
            this.AllHeaderFooterTypesExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void AllHeaderFooterTypes_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithAllHeadersAndFooters();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void AllHeaderFooterTypesExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithAllHeadersAndFooters();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);

#if VISUAL
            this.AssertVisual(originalDoc);
#endif
        }

        private static RadFlowDocument CreateDocumentWithAllHeadersAndFooters()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph();

            DocumentHelper.CreateHeaderFooterWithText("Hello odd header!", section.Headers, HeaderFooterType.Default);
            DocumentHelper.CreateHeaderFooterWithText("Hello even header!", section.Headers, HeaderFooterType.Even);
            DocumentHelper.CreateHeaderFooterWithText("Hello first header!", section.Headers, HeaderFooterType.First);

            DocumentHelper.CreateHeaderFooterWithText("Hello odd footer!", section.Footers, HeaderFooterType.Default);
            DocumentHelper.CreateHeaderFooterWithText("Hello even footer!", section.Footers, HeaderFooterType.Even);
            DocumentHelper.CreateHeaderFooterWithText("Hello first footer!", section.Footers, HeaderFooterType.First);

            return expected;
        }
    }
}
