﻿using System;
using System.IO;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.HeadersFooters
{
    [TestClass]
    public class DefaultHeadersFootersTests
    {
        private const string TestFilesFolder = "HeadersFooters/";

        [TestMethod]
        public void DefaultHeader_Import_Rtf()
        {
            RadFlowDocument expected = CreateDocumentWithDefaultHeader();

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "DefaultHeader.rtf", new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void DefaultHeader_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithDefaultHeader();

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "DefaultHeader.docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void DefaultFooter_Import_Rtf()
        {
            RadFlowDocument expected = CreateDocumentWithDefaultFooter();

            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "DefaultFooter.rtf", new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void DefaultFooter_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithDefaultFooter();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "DefaultFooter.docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void DefaultHeader_ExportImport_Rtf()
        {
            this.DefaultHeaderExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void DefaultHeader_ExportImport_Docx()
        {
            this.DefaultHeaderExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void DefaultHeader_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithDefaultHeader();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void DefaultFooter_ExportImport_Rtf()
        {
            this.DefaultFooterExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void DefaultFooter_ExportImport_Docx()
        {
            this.DefaultFooterExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void DefaultFooter_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithDefaultFooter();
            XmlValidator.ValidateOpenXmlDocument(document);
        }


        private void DefaultHeaderExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithDefaultHeader();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private void DefaultFooterExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithDefaultFooter();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private static RadFlowDocument CreateDocumentWithDefaultHeader()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph();
            DocumentHelper.CreateHeaderFooterWithText("Hello header!", section.Headers, HeaderFooterType.Default);
            return expected;
        }

        private static RadFlowDocument CreateDocumentWithDefaultFooter()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph();
            DocumentHelper.CreateHeaderFooterWithText("Hello footer!", section.Footers, HeaderFooterType.Default);
            return expected;
        }

        [TestMethod]
        public void DefaultHeader_WithFontAndColors_ExportImport_Rtf_Bug()
        {
            //Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Section section = originalDoc.Sections.AddSection();
            section.Blocks.AddParagraph();
            Header header = section.Headers.Add(HeaderFooterType.Default);

            Run run = header.Blocks.AddParagraph()
                  .Inlines.AddRun("Hello footer!");
            run.FontFamily = new ThemableFontFamily(new FontFamily("Courier New"));
            run.ForegroundColor = new ThemableColor(Colors.BlueViolet);

            //Act
            RadFlowDocument testDocument = null;
            RtfFormatProvider provider = new RtfFormatProvider();
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }
    }
}
