﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.HeadersFooters
{
    [TestClass]
    public class MultipleSectionsTests
    {
        private const string TestFilesFolder = "HeadersFooters/";

        [TestMethod]
        public void AllHeaderFooterTypes_Import_Rtf()
        {
            RadFlowDocument expected = CreateDocumentWithTwoSectionsWithOddHeadersAndFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleSectionsDifferentHeaders.rtf", TestFilesFolder), new RtfFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void AllHeaderFooterTypes_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithTwoSectionsWithOddHeadersAndFooters();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}MultipleSectionsDifferentHeaders.docx", TestFilesFolder), new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void AllHeaderFooterTypes_ExportImport_Rtf()
        {
            this.MultipleSectionsExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void AllHeaderFooterTypes_ExportImport_Docx()
        {
            this.MultipleSectionsExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void AllHeaderFooterTypes_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithTwoSectionsWithOddHeadersAndFooters();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void MultipleSectionsExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithTwoSectionsWithOddHeadersAndFooters();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string debugStr = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private static RadFlowDocument CreateDocumentWithTwoSectionsWithOddHeadersAndFooters()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph()
                .Inlines.AddRun().Text = "body";
            DocumentHelper.CreateHeaderFooterWithText("First section header!", section.Headers, HeaderFooterType.Default);
            DocumentHelper.CreateHeaderFooterWithText("First section footer!", section.Footers, HeaderFooterType.Default);

            Section section2 = expected.Sections.AddSection();
            section2.Blocks.AddParagraph();
            DocumentHelper.CreateHeaderFooterWithText("Second section header!", section2.Headers, HeaderFooterType.Default);
            DocumentHelper.CreateHeaderFooterWithText("Second section footer!", section2.Footers, HeaderFooterType.Default);

            return expected;
        }
    }
}
