﻿using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Images
{
    [TestClass]
    public class FloatingImagesTests : ImagesTestsBase
    {
        [TestMethod]
        public void FloatingImage_Png_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_HeaderFooter_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithRotation_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);

            image.Image.RotationAngle = 160;

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithMargin_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);

            image.Margin = new Primitives.Padding(12, 24, 36, 48);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithAllowOverlap_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.AllowOverlap = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.AllowOverlap = false;

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithLayoutInCell_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.LayoutInCell = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.LayoutInCell = false;

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithIsBehindDocument_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.IsBehindDocument = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.IsBehindDocument = false;

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithHorizontalPosition_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Character, 10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Column, 20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Page, 40);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.LeftMargin, -30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.RightMargin, -40);

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Left);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Center);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Right);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Inside);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Outside);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithVerticalPosition_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Line, 10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Paragraph, 20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Page, 40);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.TopMargin, -30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.BottomMargin, -40);

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Top);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Center);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Bottom);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Inside);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Outside);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithWrapping_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.None, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.TopAndBottom, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.LeftOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.RightOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.Largest);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithZIndex_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.ZIndex = 0;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.ZIndex = 10;

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_Png_Import_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            image.HorizontalPosition = new HorizontalPosition(HorizontalRelativeFrom.Column, 16);
            image.VerticalPosition = new VerticalPosition(VerticalRelativeFrom.Paragraph, 16);
            image.LayoutInCell = true;
            image.ZIndex = 251658240;

            doc.Sections.AddSection().Blocks.AddParagraph()
                .Inlines.Add(image);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(
                string.Format("{0}FloatingImagePng.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            StructureAsserter.AssertDocumentsAreEqual(doc, imported);
        }

        [TestMethod]
        public void FloatingImage_Png_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_Png_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngFloatingImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_HeaderFooter_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngFloatingImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_HeaderFooter_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngFloatingImage(document));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngFloatingImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithRotation_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);

            image.Image.RotationAngle = 160;

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithRotation_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);

            image.Image.RotationAngle = 160;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithMargin_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);

            image.Margin = new Primitives.Padding(12, 24, 36, 48);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithMargin_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);

            image.Margin = new Primitives.Padding(12, 24, 36, 48);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithAllowOverlap_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.AllowOverlap = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.AllowOverlap = false;

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithAllowOverlap_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);
            image.AllowOverlap = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image2);
            image2.AllowOverlap = false;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithLayoutInCell_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.LayoutInCell = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.LayoutInCell = false;

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithLayoutInCell_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);
            image.LayoutInCell = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image2);
            image2.LayoutInCell = false;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithIsBehindDocument_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.IsBehindDocument = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.IsBehindDocument = false;

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithIsBehindDocument_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);
            image.IsBehindDocument = true;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image2);
            image2.IsBehindDocument = false;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithHorizontalPosition_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Character, 10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Column, 20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Page, 40);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.LeftMargin, -30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.RightMargin, -40);

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Left);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Center);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Right);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Inside);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Outside);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithHorizontalPosition_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Character, 10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Column, 20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Page, 40);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.LeftMargin, -30);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.RightMargin, -40);

            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Left);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Center);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Right);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Inside);
            this.AddFloatingImageWithHorizontalPosition(p, HorizontalRelativeFrom.Margin, RelativeHorizontalAlignment.Outside);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithVerticalPosition_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Line, 10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Paragraph, 20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Page, 40);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.TopMargin, -30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.BottomMargin, -40);

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Top);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Center);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Bottom);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Inside);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Outside);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithVerticalPosition_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Line, 10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Paragraph, 20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, 30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Page, 40);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.InsideMargin, -10);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.OutsideMargin, -20);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.TopMargin, -30);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.BottomMargin, -40);

            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Top);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Center);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Bottom);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Inside);
            this.AddFloatingImageWithVerticalPosition(p, VerticalRelativeFrom.Margin, RelativeVerticalAlignment.Outside);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithWrapping_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.None, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.TopAndBottom, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.LeftOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.RightOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.Largest);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithWrapping_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.None, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.TopAndBottom, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.BothSides);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.LeftOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.RightOnly);
            this.AddFloatingImageWithWrapping(p, ShapeWrappingType.Square, TextWrap.Largest);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_WithZIndex_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image);
            image.ZIndex = 0;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(doc);
            p.Inlines.Add(image2);
            image2.ZIndex = 10;

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FloatingImage_WithZIndex_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();

            FloatingImage image = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image);
            image.ZIndex = 0;

            FloatingImage image2 = TestHelper.CreatePngFloatingImage(document);
            p.Inlines.Add(image2);
            image2.ZIndex = 10;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FloatingImage_Png_Import_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            image.HorizontalPosition = new HorizontalPosition(HorizontalRelativeFrom.Column, 16);
            image.VerticalPosition = new VerticalPosition(VerticalRelativeFrom.Paragraph, 16);
            image.LayoutInCell = true;
            image.ZIndex = 251658240;

            doc.Sections.AddSection().Blocks.AddParagraph()
                .Inlines.Add(image);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(
                string.Format("{0}FloatingImagePng.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            StructureAsserter.AssertDocumentsAreEqual(doc, imported);
        }

        [TestMethod]
        public void FloatingImage_WhenSizeIsEmpty_IsExportedWithShapeDefaultSize_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            image.Image.Size = Size.Empty;
            p.Inlines.Add(image);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(doc, new DocxFormatProvider());

            // Assert
            FloatingImage importedImage = imported.EnumerateChildrenOfType<FloatingImage>().FirstOrDefault();
            Assert.AreEqual(ShapeBaseDefaultSize, importedImage.Image.Size);
        }

        [TestMethod]
        public void FloatingImage_WhenSizeIsEmpty_IsExportedWithShapeDefaultSize_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            FloatingImage image = TestHelper.CreatePngFloatingImage(doc);
            image.Image.Size = Size.Empty;
            p.Inlines.Add(image);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(doc, new RtfFormatProvider());

            // Assert
            FloatingImage importedImage = imported.EnumerateChildrenOfType<FloatingImage>().FirstOrDefault();
            Assert.AreEqual(ShapeBaseDefaultSize, importedImage.Image.Size);
        }

        private void AddFloatingImageWithHorizontalPosition(Paragraph p, HorizontalRelativeFrom rel, double offset)
        {
            FloatingImage image;
            image = TestHelper.CreatePngFloatingImage(p.Document);
            p.Inlines.Add(image);
            image.HorizontalPosition = new HorizontalPosition(rel, offset);
        }

        private void AddFloatingImageWithHorizontalPosition(Paragraph p, HorizontalRelativeFrom rel, RelativeHorizontalAlignment align)
        {
            FloatingImage image;
            image = TestHelper.CreatePngFloatingImage(p.Document);
            p.Inlines.Add(image);
            image.HorizontalPosition = new HorizontalPosition(rel, align);
        }

        private void AddFloatingImageWithVerticalPosition(Paragraph p, VerticalRelativeFrom rel, double offset)
        {
            FloatingImage image;
            image = TestHelper.CreatePngFloatingImage(p.Document);
            p.Inlines.Add(image);
            image.VerticalPosition = new VerticalPosition(rel, offset);
        }

        private void AddFloatingImageWithVerticalPosition(Paragraph p, VerticalRelativeFrom rel, RelativeVerticalAlignment align)
        {
            FloatingImage image;
            image = TestHelper.CreatePngFloatingImage(p.Document);
            p.Inlines.Add(image);
            image.VerticalPosition = new VerticalPosition(rel, align);
        }

        private void AddFloatingImageWithWrapping(Paragraph p, ShapeWrappingType wrapType, TextWrap textWrap)
        {
            FloatingImage image;
            image = TestHelper.CreatePngFloatingImage(p.Document);
            p.Inlines.Add(image);

            image.Wrapping = new ShapeWrapping(wrapType, textWrap);
        }
    }
}