﻿using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
using Telerik.Windows.Documents.Model.Drawing.Shapes;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Images
{
    [TestClass]
    public class InlineImagesTests : ImagesTestsBase
    {
        [TestMethod]
        public void InlineImage_Png_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_HeaderFooter_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngImage(doc));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Jpg_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        // [TestMethod]
        // This test will not work as we export bmp images as png, otherwise MS Word does not open them.
        public void InlineImage_Bmp_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateBmpImage(doc));

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithRotation_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.RotationAngle = 148;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithHorizontalFlip_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.IsHorizontallyFlipped = true;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithVerticalFlip_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.IsVerticallyFlipped = true;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Png_Import_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();

            doc.Sections.AddSection().Blocks.AddParagraph()
                .Inlines.Add(TestHelper.CreatePngImage(doc));

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(
                string.Format("{0}InlineImagePng.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            StructureAsserter.AssertDocumentsAreEqual(doc, imported);
        }

        [TestMethod]
        public void InlineImage_Png_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Png_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreatePngImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_HeaderFooter_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngImage(doc));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_HeaderFooter_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph p1 = section.Headers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p1.Inlines.Add(TestHelper.CreatePngImage(document));

            Paragraph p2 = section.Footers.Add(HeaderFooterType.Default).Blocks.AddParagraph();
            p2.Inlines.Add(TestHelper.CreatePngImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_Jpg_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Jpg_ExportImport_Docx_DoesNotRespectTheCasing()
        {
            RadFlowDocument doc = new RadFlowDocument();
            ImageInline image;

            using (System.IO.Stream stream = TestHelper.GetImageStream("test.jpg"))
            {
                image = new ImageInline(doc);
                image.Image.ImageSource = new Media.ImageSource(stream, "JPG");
            }

            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Jpg_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateJpgImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_Bmp_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateBmpImage(doc));

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_Bmp_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(TestHelper.CreateBmpImage(document));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_WithRotation_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.RotationAngle = 148;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithRotation_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(document);
            image.Image.RotationAngle = 148;
            p.Inlines.Add(image);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_WithHorizontalFlip_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.IsHorizontallyFlipped = true;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithHorizontalFlip_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(document);
            image.Image.IsHorizontallyFlipped = true;
            p.Inlines.Add(image);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_WithVerticalFlip_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.IsVerticallyFlipped = true;
            p.Inlines.Add(image);

            this.ExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void InlineImage_WithVerticalFlip_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(document);
            image.Image.IsVerticallyFlipped = true;
            p.Inlines.Add(image);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void InlineImage_Png_Import_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();

            doc.Sections.AddSection().Blocks.AddParagraph()
                .Inlines.Add(TestHelper.CreatePngImage(doc));

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(
                string.Format("{0}InlineImagePng.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            StructureAsserter.AssertDocumentsAreEqual(doc, imported);
        }

        [TestMethod]
        public void InlineImage_WhenSizeIsEmpty_IsExportedWithShapeDefaultSize_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.Size = Size.Empty;
            p.Inlines.Add(image);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(doc, new DocxFormatProvider());

            // Assert
            ImageInline importedImage = imported.EnumerateChildrenOfType<ImageInline>().FirstOrDefault();
            Assert.AreEqual(ShapeBaseDefaultSize, importedImage.Image.Size);
        }

        [TestMethod]
        public void InlineImage_WhenSizeIsEmpty_IsExportedWithShapeDefaultSize_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph p = doc.Sections.AddSection().Blocks.AddParagraph();
            ImageInline image = TestHelper.CreatePngImage(doc);
            image.Image.Size = Size.Empty;
            p.Inlines.Add(image);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(doc, new RtfFormatProvider());

            // Assert
            ImageInline importedImage = imported.EnumerateChildrenOfType<ImageInline>().FirstOrDefault();
            Assert.AreEqual(ShapeBaseDefaultSize, importedImage.Image.Size);
        }

        [TestMethod]
        public void InlineImage_WhenIsDefinedInPictElement_Import_Docx()
        {
            // Arrange and Act
            string documentPath = string.Format("{0}ImageDeclaredInPictElement.docx", TestFilesFolder);
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(documentPath, new DocxFormatProvider());

            List<ImageInline> images = imported.EnumerateChildrenOfType<ImageInline>().ToList();

            // Assert
            Assert.AreEqual(2, images.Count);

            Image image = images[0].Image;
            Assert.AreEqual(108, image.Width);
            Assert.AreEqual(36, image.Height);
            Assert.AreEqual(0, image.RotationAngle);
            Assert.AreEqual("png", image.ImageSource.Extension);
            Assert.AreEqual(1457, image.ImageSource.Data.Length);


            image = images[1].Image;
            Assert.AreEqual(585, image.Width);
            Assert.AreEqual(174, image.Height);
            Assert.AreEqual(0, image.RotationAngle);
            Assert.AreEqual("png", image.ImageSource.Extension);
            Assert.AreEqual(20195, image.ImageSource.Data.Length);
        }
    }
}
