﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class AlignmentTests : ListsTestBase
    {
        [TestMethod]
        public void Alignment_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.Alignment = Flow.Model.Styles.Alignment.Right;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void Alignment_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.Alignment = Flow.Model.Styles.Alignment.Right;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void Alignment_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].Alignment = Flow.Model.Styles.Alignment.Right;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Alignment.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].Alignment == importedList.Levels[0].Alignment);
        }

        [TestMethod]
        public void Alignment_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].Alignment = Flow.Model.Styles.Alignment.Right;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Alignment.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].Alignment == importedList.Levels[0].Alignment);
        }

        [TestMethod]
        public void Alignment_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.Alignment = Flow.Model.Styles.Alignment.Right;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
