﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class CharacterPropertiesTests : ListsTestBase
    {
        [TestMethod]
        public void CharacterProperties_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
                level.CharacterProperties.FontSize.LocalValue = 44;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void CharacterProperties_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
                level.CharacterProperties.FontSize.LocalValue = 44;
                level.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void CharacterProperties_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            list.Levels[0].CharacterProperties.FontSize.LocalValue = 44;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterProperties.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].CharacterProperties.ForegroundColor.LocalValue == importedList.Levels[0].CharacterProperties.ForegroundColor.LocalValue);
            Assert.IsTrue(list.Levels[0].CharacterProperties.FontSize.LocalValue == importedList.Levels[0].CharacterProperties.FontSize.LocalValue);
        }

        [TestMethod]
        public void CharacterProperties_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            list.Levels[0].CharacterProperties.FontSize.LocalValue = 44;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterProperties.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].CharacterProperties.ForegroundColor.LocalValue == importedList.Levels[0].CharacterProperties.ForegroundColor.LocalValue);
            Assert.IsTrue(list.Levels[0].CharacterProperties.FontSize.LocalValue == importedList.Levels[0].CharacterProperties.FontSize.LocalValue);
        }

        [TestMethod]
        public void CharacterProperties_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
                level.CharacterProperties.FontSize.LocalValue = 44;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
