﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class ListLevelOverriderTests : ListsTestBase
    {
        [TestMethod]
        public void StartIndexOverride_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 5;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}StartIndexOverride.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(1);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
        }

        [TestMethod]
        public void StartIndexOverride_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 5;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}StartIndexOverride.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(1);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
        }

        [TestMethod]
        public void ListLevelOverride_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 6;
            list.Levels[0].Alignment = Flow.Model.Styles.Alignment.Left;
            list.Levels[0].NumberTextFormat = "__%1)__";

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LevelOverride.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(1);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
            Assert.IsTrue(list.Levels[0].Alignment == importedList.Levels[0].Alignment);
            Assert.IsTrue(list.Levels[0].NumberTextFormat == importedList.Levels[0].NumberTextFormat);
        }

        [TestMethod]
        public void ListLevelOverride_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 6;
            list.Levels[0].Alignment = Flow.Model.Styles.Alignment.Left;
            list.Levels[0].NumberTextFormat = "__%1)__";

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}LevelOverride.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(1);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
            Assert.IsTrue(list.Levels[0].Alignment == importedList.Levels[0].Alignment);
            Assert.IsTrue(list.Levels[0].NumberTextFormat == importedList.Levels[0].NumberTextFormat);
        }
    }
}
