﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class ListToParagraphTests : ListsTestBase
    {
        [TestMethod]
        public void ListIdInParagraph_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(0);

            Paragraph importedParagraph = importedDocument.Sections.First().Blocks.First() as Paragraph;

            Assert.IsTrue(importedParagraph.ListId == importedList.Id);
            Assert.IsTrue(importedParagraph.ListLevel == paragraph.ListLevel);
        }

        [TestMethod]
        public void ListIdInParagraph_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(0);

            Paragraph importedParagraph = importedDocument.Sections.First().Blocks.First() as Paragraph;

            Assert.IsTrue(importedParagraph.ListId == importedList.Id);
            Assert.IsTrue(importedParagraph.ListLevel == paragraph.ListLevel);
        }

        [TestMethod]
        public void ListIdInParagraph_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void ListIdInParagraph_WithStyle_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Style style = new Style("numberingStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            list.StyleId = style.Id;

            //style.ParagraphProperties.ListId.LocalValue = list.Id;

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            // Act and assert
            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(0);
            Paragraph importedParagraph = importedDocument.Sections.First().Blocks.First() as Paragraph;

            // Assert list properties
            Assert.IsTrue(importedParagraph.ListId == importedList.Id);
            Assert.IsTrue(importedParagraph.ListLevel == paragraph.ListLevel);

            // Assert style properties
            Style importedStyle = importedDocument.StyleRepository.GetStyle(style.Id);
            Assert.IsTrue(importedStyle.ParagraphProperties.ListId.LocalValue == importedList.Id);
        }

        [TestMethod]
        public void ListIdInParagraph_WithStyle_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Style style = new Style("numberingStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            list.StyleId = style.Id;

            //style.ParagraphProperties.ListId.LocalValue = list.Id;

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            // Act and assert
            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(1);
            Paragraph importedParagraph = importedDocument.Sections.First().Blocks.First() as Paragraph;

            // Assert list properties
            Assert.IsTrue(importedParagraph.ListId == importedList.Id);
            Assert.IsTrue(importedParagraph.ListLevel == paragraph.ListLevel);

            // Assert style properties
            Style importedStyle = importedDocument.StyleRepository.GetStyle(style.Id);
            Assert.IsTrue(importedStyle.ParagraphProperties.ListId.LocalValue == importedList.Id);
        }

        [TestMethod]
        public void ListIdInParagraph_WithStyle_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Style style = new Style("numberingStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            list.StyleId = style.Id;

            //style.ParagraphProperties.ListId.LocalValue = list.Id;

            paragraph.ListId = list.Id;
            paragraph.ListLevel = 2;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
