﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class MultiLevelTypeTests : ListsTestBase
    {
        [TestMethod]
        public void MultiLevelType_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.MultilevelType = MultilevelType.Multilevel;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void MultiLevelType_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.MultilevelType = MultilevelType.Multilevel;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void MultiLevelType_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.MultilevelType = MultilevelType.SingleLevel;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultilevelType.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.MultilevelType == importedList.MultilevelType);
        }

#if VISUAL
        [TestMethod]
        [TestCategory("Visual Tests")]
        [TestCategory("MultiLevelType")]
        public void MultiLevelTypeWithoutExplicitlySpecifiedMultilevelType_Import_Docx()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "BarrierstoAccessArchivalCopy.docx", new DocxFormatProvider());

            AssertVisual(document, 5);
        }
#endif

        [TestMethod]
        public void MultiLevelType_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.MultilevelType = MultilevelType.SingleLevel;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}MultilevelType.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.MultilevelType == importedList.MultilevelType);
        }

        [TestMethod]
        public void MultiLevelType_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();

            document.Lists.Add(ListTemplateType.NumberedDefault);
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void MultiLevelType_WhenIsSingleLevel_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.MultilevelType = MultilevelType.SingleLevel;

            for (int level = 0; level < list.Levels.Count; ++level)
            {
                list.Levels[level].NumberingStyle = NumberingStyle.Decimal;
                list.Levels[level].NumberTextFormat = "%" + (level + 1) + "test.";
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            StylePropertyAsserter.AssertListLevelsAreEqual(list.Levels[0], importedList.Levels[0]);

            List defaultList = new List();
            for (int i = 1; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.AssertListLevelsAreEqual(defaultList.Levels[i], importedList.Levels[i]);
            }
        }

        [TestMethod]
        public void List_WhenMultiLevelTypeMissingInAbstractList_IsImportedAsHybridMultilevel()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            List list = expected.Lists.Add(ListTemplateType.BulletDefault);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}AbstractListWithoutMultilevelType.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = imported.Lists.GetList(0);

            // Assert
            Assert.AreEqual(list.MultilevelType, importedList.MultilevelType);
            Assert.AreEqual(MultilevelType.HybridMultilevel, importedList.MultilevelType);
        }
    }
}
