﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class NumberTextFormatTests : ListsTestBase
    {
        [TestMethod]
        public void NumberTextFormat_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberTextFormat = "__%__";
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void NumberTextFormat_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberTextFormat = "__%__";
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void NumberTextFormat_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].NumberTextFormat = "__%__";

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}NumberTextFormat.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].NumberTextFormat == importedList.Levels[0].NumberTextFormat);
        }

        [TestMethod]
        public void NumberTextFormat_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].NumberTextFormat = "__%__";

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}NumberTextFormat.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].NumberTextFormat == importedList.Levels[0].NumberTextFormat);
        }

        [TestMethod]
        public void NumberTextFormat_WhenIsNull_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            list.Levels[0].NumberTextFormat = null;

            // Act
            RadFlowDocument imported = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = imported.Lists.GetList(0);

            // Assert
            Assert.IsNull(importedList.Levels[0].NumberTextFormat);
        }

        [TestMethod]
        public void NumberTextFormat_WhenIsNull_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            list.Levels[0].NumberTextFormat = null;

            // Act
            RadFlowDocument imported = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = imported.Lists.GetList(0);

            // Assert
            Assert.IsNull(importedList.Levels[0].NumberTextFormat);
        }

        [TestMethod]
        public void NumberTextFormat_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberTextFormat = "__%__";
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
