﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class NumberingStyleTests : ListsTestBase
    {
        [TestMethod]
        public void NumberingStyle_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberingStyle = NumberingStyle.Hebrew1;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void NumberingStyle_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberingStyle = NumberingStyle.Hebrew1;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void NumberingStyle_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].NumberingStyle = NumberingStyle.Hebrew1;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}NumberingStyle.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].NumberingStyle == importedList.Levels[0].NumberingStyle);
        }

        [TestMethod]
        public void NumberingStyle_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].NumberingStyle = NumberingStyle.Hebrew1;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}NumberingStyle.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].NumberingStyle == importedList.Levels[0].NumberingStyle);
        }

        [TestMethod]
        public void NumberingStyle_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.NumberingStyle = NumberingStyle.Hebrew1;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
