﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class ParagraphPropertiesTests : ListsTestBase
    {
        [TestMethod]
        public void ParagraphProperties_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.ParagraphProperties.LeftIndent.LocalValue = 200;
                level.ParagraphProperties.SpacingBefore.LocalValue = 43;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void ParagraphProperties_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.ParagraphProperties.LeftIndent.LocalValue = 200;
                level.ParagraphProperties.SpacingBefore.LocalValue = 43;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void ParagraphProperties_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].ParagraphProperties.LeftIndent.LocalValue = 96;
            list.Levels[0].ParagraphProperties.HangingIndent.LocalValue = 96;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphProperties.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].ParagraphProperties.LeftIndent.LocalValue == importedList.Levels[0].ParagraphProperties.LeftIndent.LocalValue);
            Assert.IsTrue(list.Levels[0].ParagraphProperties.HangingIndent.LocalValue == importedList.Levels[0].ParagraphProperties.HangingIndent.LocalValue);
        }

        [TestMethod]
        public void ParagraphProperties_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].ParagraphProperties.LeftIndent.LocalValue = 96;
            list.Levels[0].ParagraphProperties.HangingIndent.LocalValue = 96;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphProperties.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].ParagraphProperties.LeftIndent.LocalValue == importedList.Levels[0].ParagraphProperties.LeftIndent.LocalValue);
            Assert.IsTrue(list.Levels[0].ParagraphProperties.HangingIndent.LocalValue == importedList.Levels[0].ParagraphProperties.HangingIndent.LocalValue);
        }

        [TestMethod]
        public void ParagraphProperties_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.ParagraphProperties.LeftIndent.LocalValue = 200;
                level.ParagraphProperties.SpacingBefore.LocalValue = 43;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
