﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class RestartAfterLevelTests : ListsTestBase
    {
        [TestMethod]
        public void RestartAfterLevelWithGreaterValue_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.RestartAfterLevel = 2;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void RestartAfterLevelWithZeroValue_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.RestartAfterLevel = 0;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void RestartAfterLevel_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.RestartAfterLevel = 0;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void RestartAfterLevel_WhenSingleLevelHasRestartAfterLevel_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[2].RestartAfterLevel = 0;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void RestartAfterLevel_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[3].RestartAfterLevel = 0;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}RestartAfterLevel.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[3].RestartAfterLevel == importedList.Levels[3].RestartAfterLevel);
        }

        [TestMethod]
        public void RestartAfterLevel_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[3].RestartAfterLevel = 0;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}RestartAfterLevel.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[3].RestartAfterLevel == importedList.Levels[3].RestartAfterLevel);
        }

        [TestMethod]
        public void RestartAfterLevel_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.RestartAfterLevel = 2;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
