﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class StartIndexTests : ListsTestBase
    {
        [TestMethod]
        public void StartIndex_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.StartIndex = 4;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void StartIndex_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.StartIndex = 4;
            }

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);
        }

        [TestMethod]
        public void StartIndex_Import_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 5;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Start.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
        }

        [TestMethod]
        public void StartIndex_Import_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.Levels[0].StartIndex = 5;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}Start.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(list.Levels[0].StartIndex == importedList.Levels[0].StartIndex);
        }

        [TestMethod]
        public void StartIndex_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            foreach (ListLevel level in list.Levels)
            {
                level.StartIndex = 4;
            }

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
