﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Lists
{
    [TestClass]
    public class StyleIdTests : ListsTestBase
    {
        [TestMethod]
        public void StyleId_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();
            Style style = new Style("MyStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.StyleId = style.Id;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new DocxFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(0);

            this.AssertListProperties(list, importedList);

            Style importedNumberedStyle = importedDocument.StyleRepository.GetStyle(style.Id);
            Assert.IsNotNull(importedNumberedStyle);
            Assert.IsTrue(importedNumberedStyle.StyleType == StyleType.Numbering);
        }

        [TestMethod]
        public void StyleId_ExportImport_Rtf()
        {
            RadFlowDocument document = new RadFlowDocument();
            Style style = new Style("MyStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.StyleId = style.Id;

            RadFlowDocument importedDocument = this.StyleExportImportTemplate(document, new RtfFormatProvider());
            Assert.IsTrue(importedDocument.Lists.Count == 1);

            List importedList = importedDocument.Lists.GetList(1);

            this.AssertListProperties(list, importedList);

            Style importedNumberedStyle = importedDocument.StyleRepository.GetStyle(style.Id);
            Assert.IsNotNull(importedNumberedStyle);
            Assert.IsTrue(importedNumberedStyle.StyleType == StyleType.Numbering);
        }

        [TestMethod]
        public void StyleId_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Style style = new Style("MyStyle", StyleType.Numbering);

            document.StyleRepository.Add(style);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            list.StyleId = style.Id;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void StyleId_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Style style = new Style("CustomListStyle", StyleType.Numbering);
            original.StyleRepository.Add(style);

            List list = original.Lists.Add(ListTemplateType.NumberedParentheses);
            list.StyleId = style.Id;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}CustomListStyle.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(importedDocument.Lists.Count == 1);
            Assert.IsTrue(importedList.StyleId == style.Id);
        }

        [TestMethod]
        public void StyleId_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Style style = new Style("CustomListStyle", StyleType.Numbering);
            original.StyleRepository.Add(style);

            List list = original.Lists.Add(ListTemplateType.NumberedParentheses);
            list.StyleId = style.Id;

            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}CustomListStyle.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            Assert.IsTrue(importedDocument.Lists.Count == 1);
            Assert.IsTrue(importedList.StyleId == style.Id);
        }

        [TestMethod]
        public void ParagraphStyleOnEachLevel_Import_Docx()
        {
            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}HeadingList.docx", TestFilesFolder), new DocxFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            for (int i = 0; i < importedList.Levels.Count; i++)
            {
                Assert.IsTrue(importedList.Levels[i].StyleId == BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1));
            }
        }

        [TestMethod]
        public void ParagraphStyleOnEachLevel_Import_Rtf()
        {
            // Act and Assert
            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}HeadingList.rtf", TestFilesFolder), new RtfFormatProvider());
            List importedList = importedDocument.Lists.GetList(0);

            for (int i = 0; i < importedList.Levels.Count; i++)
            {
                Assert.IsTrue(importedList.Levels[i].StyleId == BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1));
            }
        }
    }
}
