﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class AllowOverflowPunctuationTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void AllowOverflowPunctuation_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().AllowOverflowPunctuation = false;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void AllowOverflowPunctuation_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().AllowOverflowPunctuation = false;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void AllowOverflowPunctuation_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().AllowOverflowPunctuation = false;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void AllowOverflowPunctuation_ValueIsZero_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.AllowOverflowPunctuation = false;
            paragraph.Inlines.AddRun().Text = "AllowOverflowPunctuation";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}AllowOverflowPunctuationValueIsZero.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void AllowOverflowPunctuation_ValueIsOff_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.AllowOverflowPunctuation = false;
            paragraph.Inlines.AddRun().Text = "AllowOverflowPunctuation";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}AllowOverflowPunctuationValueIsOff.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void AllowOverflowPunctuation_ValueIsFalse_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.AllowOverflowPunctuation = false;
            paragraph.Inlines.AddRun().Text = "AllowOverflowPunctuation";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}AllowOverflowPunctuationValueIsFalse.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void AllowOverflowPunctuation_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.AllowOverflowPunctuation = false;
            paragraph.Inlines.AddRun().Text = "AllowOverflowPunctuation";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}AllowOverflowPunctuation.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }
    }
}
