﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ApplyEastAsianLineBreakingRulesTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void ApplyEastAsianLineBreakingRules_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ApplyEastAsianLineBreakingRules = false;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ApplyEastAsianLineBreakingRules_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ApplyEastAsianLineBreakingRules = false;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ApplyEastAsianLineBreakingRules_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.ApplyEastAsianLineBreakingRules = false;
            paragraph.Inlines.AddRun().Text = "ApplyEastAsianLineBreakingRules";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ApplyEastAsianLineBreakingRules.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void ApplyEastAsianLineBreakingRules_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.ApplyEastAsianLineBreakingRules = false;
            paragraph.Inlines.AddRun().Text = "ApplyEastAsianLineBreakingRules";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ApplyEastAsianLineBreakingRules.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void ApplyEastAsianLineBreakingRules_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ApplyEastAsianLineBreakingRules = false;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}
