﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ContextualSpacingTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void ContextualSpacing_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ContextualSpacing = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ContextualSpacing_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ContextualSpacing = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ContextualSpacing_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.ContextualSpacing = true;
            paragraph.Inlines.AddRun().Text = "ContextualSpacing";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ContextualSpacing.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void ContextualSpacing_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.ContextualSpacing = true;
            paragraph.Inlines.AddRun().Text = "ContextualSpacing";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ContextualSpacing.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

#if !VISUAL
        [TestMethod]
        public void ContextualSpacing_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ContextualSpacing = true;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
#endif

#if VISUAL
        [TestMethod]
        public void ContextualSpacing_Export_Pdf()
        { 
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFilesFolder + "ContextualSpacingProperty.docx", new DocxFormatProvider());

            AssertVisual(doc);
        }
#endif

    }
}
