﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class IndentationTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void Indentation_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Indentation.FirstLineIndent = 10;
            section.Blocks.AddParagraph().Indentation.HangingIndent = 20;
            section.Blocks.AddParagraph().Indentation.LeftIndent = 30;
            section.Blocks.AddParagraph().Indentation.RightIndent = 40;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Indentation_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Indentation.FirstLineIndent = 10;
            section.Blocks.AddParagraph().Indentation.HangingIndent = 20;
            section.Blocks.AddParagraph().Indentation.LeftIndent = 30;
            section.Blocks.AddParagraph().Indentation.RightIndent = 40;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void FlowDirection_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Indentation.FirstLineIndent = 10;
            section.Blocks.AddParagraph().Indentation.HangingIndent = 20;
            section.Blocks.AddParagraph().Indentation.LeftIndent = 30;
            section.Blocks.AddParagraph().Indentation.RightIndent = 40;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void Indentation_WithFirstLine_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun().Text = "Indentation";
            paragraph.Indentation.LeftIndent = 10;
            paragraph.Indentation.RightIndent = 20;
            paragraph.Indentation.FirstLineIndent = 30;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IndentationWithFirstLine.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void Indentation_WithHanging_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun().Text = "Indentation";
            paragraph.Indentation.LeftIndent = 10;
            paragraph.Indentation.RightIndent = 20;
            paragraph.Indentation.HangingIndent = 30;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IndentationWithHanging.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void Indentation_WithFirstLine_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun().Text = "Indentation";
            paragraph.Indentation.LeftIndent = 10;
            paragraph.Indentation.RightIndent = 20;
            paragraph.Indentation.FirstLineIndent = 30;

            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IndentationWithFirstLine.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void Indentation_WithHanging_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun().Text = "Indentation";
            paragraph.Indentation.LeftIndent = 10;
            paragraph.Indentation.RightIndent = 20;
            paragraph.Indentation.HangingIndent = 30;

            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IndentationWithHanging.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }
    }
}
