﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class KeepOnOnePageTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void KeepOnOnePage_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().KeepOnOnePage = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void KeepOnOnePage_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().KeepOnOnePage = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void KeepOnOnePage_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.KeepOnOnePage = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "KeepOnOnePage";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}KeepOnOnePage.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void KeepOnOnePage_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.KeepOnOnePage = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "KeepOnOnePage";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}KeepOnOnePage.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }
#if !VISUAL
        [TestMethod]
        public void KeepOnOnePage_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().KeepOnOnePage = true;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
#endif

#if VISUAL
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        public void KeepOnOnePageAndKeepWithNextParagraph_Export_Pdf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFilesFolder + "KeepOnOnePageProperty.docx", new DocxFormatProvider());

            AssertVisual(doc, 1);
        }
#endif

    }
}
