﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ListPropertiesTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void ListLevel_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ListLevel = 3;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ListLevel_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ListLevel = 3;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ListLevel_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().ListLevel = 3;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ListLevel_WhenInParagraphStyle_Import_Docx()
        {
            // Act and Assert
            string styleName = "Style1";

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}ListIdInStyle.docx", TestFilesFolder), new DocxFormatProvider());
            Paragraph paragraph = importedDocument.EnumerateChildrenOfType<Paragraph>().FirstOrDefault();
            paragraph.StyleId = styleName;

            Style style = importedDocument.StyleRepository.GetStyle(styleName);
            Assert.AreEqual(style.ParagraphProperties.ListId.LocalValue, 0);
        }

        [TestMethod]
        public void ListLevel_WhenInParagraphStyle_AndExplicitlySetToNoListApplied_Import_Docx()
        {
            // Act and Assert
            string styleName = "Style1";

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(string.Format("{0}ListIdInStyleExplicitlySetToNoListApplied.docx", TestFilesFolder), new DocxFormatProvider());
            Paragraph paragraph = importedDocument.EnumerateChildrenOfType<Paragraph>().FirstOrDefault();
            paragraph.StyleId = styleName;

            Style style = importedDocument.StyleRepository.GetStyle(styleName);
            Assert.AreEqual(style.ParagraphProperties.ListId.LocalValue, null);
        }
    }
}
