﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class MirrorIndentsTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void MirrorIndents_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().MirrorIndents = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void MirrorIndents_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().MirrorIndents = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void MirrorIndents_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.MirrorIndents = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "MirrorIndents";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}MirrorIndents.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void MirrorIndents_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.MirrorIndents = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "MirrorIndents";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}MirrorIndents.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

#if !VISUAL
        [TestMethod]
        public void MirrorIndents_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().MirrorIndents = true;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
#endif

#if VISUAL
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void MirrorIndents_Export_Pdf()
        { 
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFilesFolder + "MirrorIndentsProperty.docx", new DocxFormatProvider());

            AssertVisual(doc,1);
        }
#endif
    }
}
