﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class PageBreakBeforeTests : ParagraphPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void PageBreakBefore_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().PageBreakBefore = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void PageBreakBefore_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().PageBreakBefore = true;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void PageBreakBefore_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.PageBreakBefore = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "PageBreakBefore";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PageBreakBefore.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        // TODO: [TestMethod]
        public void PageBreakBefore_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.PageBreakBefore = true;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "PageBreakBefore";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PageBreakBefore.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void OutlineLevel_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().PageBreakBefore = true;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}
