﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ParagraphBordersTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void ParagraphBorders_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            ParagraphBorders borders = new ParagraphBorders();

            section.Blocks.AddParagraph().Borders = borders.SetLeft(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            section.Blocks.AddParagraph().Borders = borders.SetTop(
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            section.Blocks.AddParagraph().Borders = borders.SetRight(
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            section.Blocks.AddParagraph().Borders = borders.SetBottom(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            section.Blocks.AddParagraph().Borders = borders.SetBetween(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Cyan)));

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ParagraphBorders_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            ParagraphBorders borders = new ParagraphBorders();

            section.Blocks.AddParagraph().Borders = borders.SetLeft(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            section.Blocks.AddParagraph().Borders = borders.SetTop(
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            section.Blocks.AddParagraph().Borders = borders.SetRight(
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            section.Blocks.AddParagraph().Borders = borders.SetBottom(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            section.Blocks.AddParagraph().Borders = borders.SetBetween(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Cyan)));

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphBorders_LocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            ParagraphBorders borders = new ParagraphBorders();

            section.Blocks.AddParagraph().Borders = borders.SetLeft(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            section.Blocks.AddParagraph().Borders = borders.SetTop(
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            section.Blocks.AddParagraph().Borders = borders.SetRight(
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            section.Blocks.AddParagraph().Borders = borders.SetBottom(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            section.Blocks.AddParagraph().Borders = borders.SetBetween(
                new Border(3, BorderStyle.Dashed, new ThemableColor(Colors.Cyan)));

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphBorders_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            section.Blocks.AddParagraph().Borders = new ParagraphBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphBorders_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            section.Blocks.AddParagraph().Borders = new ParagraphBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphBorders_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Border border = new Border(1, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent6));
            ParagraphBorders borders = new ParagraphBorders(border, border, border, border, border);
            paragraph.Borders = borders;
            paragraph.Inlines.AddRun().Text = "Borders";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}BordersThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void ParagraphBorders_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            ParagraphBorders borders = new ParagraphBorders(border, border, border, border, border);
            paragraph.Borders = borders;
            paragraph.Inlines.AddRun().Text = "Borders";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}BordersLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void ParagraphBorders_LocalColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            ParagraphBorders borders = new ParagraphBorders(border, border, border, border, border);
            paragraph.Borders = borders;
            paragraph.Inlines.AddRun().Text = "Borders";

            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}BordersLocalColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }
    }
}
