﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ParagraphMarkerPropertiesTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void ParagraphMarkerProperties_WhenDocumentIsEmpty_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            this.AddParagraphWithParagraphMarkerProperties(document);

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(document, new DocxFormatProvider());
#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void ParagraphMarkerProperties_WhenDocumentIsEmpty_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            this.AddParagraphWithParagraphMarkerProperties(expected);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphMarkerPropertiesEmptyDocument.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
#if VISUAL
            AssertVisual(imported, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentIsEmpty_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            this.AddParagraphWithParagraphMarkerProperties(document);

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(document, new RtfFormatProvider());

#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentIsEmpty_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            this.AddParagraphWithParagraphMarkerProperties(expected);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphMarkerPropertiesEmptyDocument.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
#if VISUAL
            AssertVisual(imported, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsText_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            this.AddParagraphWithParagraphMarkerProperties(document);

            // Assert
            this.ParagraphPropertiesExportImportTemplate(document, new DocxFormatProvider());
#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsText_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            this.AddParagraphWithParagraphMarkerProperties(document);

            // Assert
            this.ParagraphPropertiesExportImportTemplate(document, new RtfFormatProvider());

#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsTextAndPropertiesAreAppliedOnTheParagraphWithText_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = this.AddParagraphWithParagraphMarkerProperties(document);
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            // Assert
            this.ParagraphPropertiesExportImportTemplate(document, new DocxFormatProvider());
#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsTextAndPropertiesAreAppliedOnTheParagraphWithText_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = this.AddParagraphWithParagraphMarkerProperties(expected);
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphMarkerProperties.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
#if VISUAL
            AssertVisual(imported, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsTextAndPropertiesAreAppliedOnTheParagraphWithText_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = this.AddParagraphWithParagraphMarkerProperties(document);
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            // Assert
            this.ParagraphPropertiesExportImportTemplate(document, new RtfFormatProvider());

#if VISUAL
            AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Section)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void ParagraphMarkerProperties_WhenDocumentContainsTextAndPropertiesAreAppliedOnTheParagraphWithText_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = this.AddParagraphWithParagraphMarkerProperties(expected);
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphMarkerProperties.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
#if VISUAL
            AssertVisual(imported, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
#endif
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphIsWithModifiedHeadingStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphMarkerPropertiesFromStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphIsWithModifiedHeadingStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily("Arial");
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphIsWithModifiedHeadingStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily("Arial");
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphHasHeadingStyleAndMarkerPropertiesHasLocalProperties_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraph.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphParagraphMarkerPropertiesFromStyleAndLocalValues.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphHasHeadingStyleAndMarkerPropertiesHasLocalProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraph.Properties.ParagraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily("Arial");
            paragraph.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        [TestMethod]
        public void ParagraphMarkerProperties_WhenParagraphHasHeadingStyleAndMarkerPropertiesHasLocalProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "Test";

            paragraph.StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(1);
            expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));

            paragraph.Properties.ParagraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(16);
            paragraph.Properties.ParagraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraph.Properties.ParagraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily("Arial");
            paragraph.Properties.ParagraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            paragraph.Properties.ParagraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
            this.AssertRunProperties(expected, imported);
        }

        private Paragraph AddParagraphWithParagraphMarkerProperties(RadFlowDocument document)
        {
            Section section = document.Sections.Count == 0 ? document.Sections.AddSection() : document.Sections.First();
            Paragraph paragraph = section.Blocks.AddParagraph();

            var paragraphMarkerProperties = paragraph.Properties.ParagraphMarkerProperties;
            paragraphMarkerProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            paragraphMarkerProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Comic Sans MS"));
            paragraphMarkerProperties.FontSize.LocalValue = Unit.PointToDip(20);
            paragraphMarkerProperties.FontStyle.LocalValue = FontStyles.Italic;
            paragraphMarkerProperties.FontWeight.LocalValue = FontWeights.Bold;

            return paragraph;
        }
    }
}
