﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class ParagraphShadingTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void ParagraphShading_LocalColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            par.Shading.PatternColor = new ThemableColor(Colors.Blue);
            par.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ParagraphShading_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            par.Shading.PatternColor = new ThemableColor(Colors.Blue);
            par.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphBorders_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            par.Shading.PatternColor = new ThemableColor(Colors.Blue);
            par.Shading.Pattern = ShadingPattern.Percent40;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphShading_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();

            par.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            par.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            par.Shading.Pattern = ShadingPattern.DiagonalCross;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphShading_ThemeColor_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();

            par.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            par.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            par.Shading.Pattern = ShadingPattern.DiagonalCross;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphShading_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            paragraph.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            paragraph.Shading.Pattern = ShadingPattern.HorizontalStripe;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "ParagraphShading";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphShadingLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void ParagraphShading_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Accent6);
            paragraph.Shading.PatternColor = new ThemableColor(ThemeColorType.Accent5, 0.8);
            paragraph.Shading.Pattern = ShadingPattern.HorizontalStripe;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "ParagraphShading";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphShadingThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void ParagraphShading_LocalColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            paragraph.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            paragraph.Shading.Pattern = ShadingPattern.Percent30;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "ParagraphShading";

            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphShadingLocalColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }
    }
}
