﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#endif
namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class SpacingTests : ParagraphPropertiesTestsBase
    {

        [TestMethod]
        public void Spacing_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Spacing.SpacingAfter = 10;
            section.Blocks.AddParagraph().Spacing.SpacingBefore = 20;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingAfter = true;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingBefore = true;

            Paragraph par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacing = 2;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.AtLeast;
            par.Spacing.LineSpacing = 3;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.Exact;
            par.Spacing.LineSpacing = 4;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Spacing_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Spacing.SpacingAfter = 10;
            section.Blocks.AddParagraph().Spacing.SpacingBefore = 20;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingAfter = true;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingBefore = true;

            Paragraph par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacing = 2;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.AtLeast;
            par.Spacing.LineSpacing = 3;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.Exact;
            par.Spacing.LineSpacing = 4;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Spacing_Import_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.SpacingAfter = 16;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.SpacingBefore = 28;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.LineSpacingType = HeightType.AtLeast;
            paragraph.Spacing.LineSpacing = 4;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.LineSpacingType = HeightType.Exact;
            paragraph.Spacing.LineSpacing = 8;
            paragraph.Inlines.AddRun().Text = "Spacing";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Spacing.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(document, imported);
        }

        [TestMethod]
        public void Spacing_Import_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.SpacingAfter = 16;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.SpacingBefore = 28;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.LineSpacingType = HeightType.AtLeast;
            paragraph.Spacing.LineSpacing = 4;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.LineSpacingType = HeightType.Exact;
            paragraph.Spacing.LineSpacing = 8;
            paragraph.Inlines.AddRun().Text = "Spacing";

            paragraph = section.Blocks.AddParagraph();
            paragraph.Spacing.LineSpacingType = HeightType.Auto;
            paragraph.Spacing.LineSpacing = 1;
            paragraph.Inlines.AddRun().Text = "Spacing";

            foreach (Paragraph p in document.EnumerateChildrenOfType<Paragraph>())
            {
                DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(p);
            }

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Spacing.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(document, imported);
        }
#if !VISUAL        
        [TestMethod]
        public void Spacing_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Spacing.SpacingAfter = 10;
            section.Blocks.AddParagraph().Spacing.SpacingBefore = 20;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingAfter = true;
            section.Blocks.AddParagraph().Spacing.AutomaticSpacingBefore = true;

            Paragraph par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacing = 2;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.AtLeast;
            par.Spacing.LineSpacing = 3;

            par = section.Blocks.AddParagraph();
            par.Spacing.LineSpacingType = HeightType.Exact;
            par.Spacing.LineSpacing = 4;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
#endif

#if VISUAL
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void AutomaticSpacing_Export_Pdf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFilesFolder + "AutomaticSpacing.docx", new DocxFormatProvider());

            AssertVisual(doc);
        }
#endif
    }
}
