﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class TabStopsTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfDifferentType_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center, TabStopLeader.Dot))
                                                   .Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right, TabStopLeader.Hyphen))
                                                   .Insert(new TabStop(Unit.InchToDip(4), TabStopType.Decimal, TabStopLeader.Underscore))
                                                   .Insert(new TabStop(Unit.InchToDip(5.5), TabStopType.Bar));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopMultipleTabStopsInSingleParagraph.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfDifferentType_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center, TabStopLeader.Dot))
                                                   .Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right, TabStopLeader.Hyphen))
                                                   .Insert(new TabStop(Unit.InchToDip(4), TabStopType.Decimal, TabStopLeader.Underscore))
                                                   .Insert(new TabStop(Unit.InchToDip(5.5), TabStopType.Bar));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            Assert.AreEqual(5, imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops.Count);
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfDifferentType_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center, TabStopLeader.Dot))
                                                   .Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right, TabStopLeader.Hyphen))
                                                   .Insert(new TabStop(Unit.InchToDip(4), TabStopType.Decimal, TabStopLeader.Underscore))
                                                   .Insert(new TabStop(Unit.InchToDip(5.5), TabStopType.Bar));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopMultipleTabStopsInSingleParagraph.rtf", TestFilesFolder), new RtfFormatProvider());
            imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().Properties.ParagraphMarkerProperties.ClearLocalValues();

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfDifferentType_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center, TabStopLeader.Dot))
                                                   .Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right, TabStopLeader.Hyphen))
                                                   .Insert(new TabStop(Unit.InchToDip(4), TabStopType.Decimal, TabStopLeader.Underscore))
                                                   .Insert(new TabStop(Unit.InchToDip(5.5), TabStopType.Bar));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            Assert.AreEqual(5, imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops.Count);
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfSameTypeButOnDifferentPosition_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopMultipleTabStopsFromSameTypeInSingleParagraph.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfSameTypeButOnDifferentPosition_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfSameTypeButOnDifferentPosition_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopMultipleTabStopsFromSameTypeInSingleParagraph.rtf", TestFilesFolder), new RtfFormatProvider());
            imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().Properties.ParagraphMarkerProperties.ClearLocalValues();

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphHasMultipleTabStopsOfSameTypeButOnDifferentPosition_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(1), TabStopType.Left));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenMultipleParagraphHaveDifferentTabStops_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();

            Paragraph par1 = expected.Sections.AddSection().Blocks.AddParagraph();
            par1.Inlines.AddRun("Par1");
            par1.TabStops = par1.TabStops.Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center));

            Paragraph par2 = expected.Sections.First().Blocks.AddParagraph();
            par2.Inlines.AddRun("Par2");
            par2.TabStops = par2.TabStops.Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right));

            Paragraph par3 = expected.Sections.First().Blocks.AddParagraph();
            par3.Inlines.AddRun("Par3");

            Paragraph par4 = expected.Sections.First().Blocks.AddParagraph();
            par4.Inlines.AddRun("Par4");
            par4.TabStops = par4.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Bar));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopDifferentTabStopsInMultipleParagraphs.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenMultipleParagraphHaveDifferentTabStops_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();

            Paragraph par1 = expected.Sections.AddSection().Blocks.AddParagraph();
            par1.Inlines.AddRun("Par1");
            par1.TabStops = par1.TabStops.Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center));

            Paragraph par2 = expected.Sections.First().Blocks.AddParagraph();
            par2.Inlines.AddRun("Par2");
            par2.TabStops = par2.TabStops.Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right));

            Paragraph par3 = expected.Sections.First().Blocks.AddParagraph();
            par3.Inlines.AddRun("Par3");

            Paragraph par4 = expected.Sections.First().Blocks.AddParagraph();
            par4.Inlines.AddRun("Par4");
            par4.TabStops = par4.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Bar));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenMultipleParagraphHaveDifferentTabStops_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();

            Paragraph par1 = expected.Sections.AddSection().Blocks.AddParagraph();
            par1.Inlines.AddRun("Par1");
            par1.TabStops = par1.TabStops.Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center));

            Paragraph par2 = expected.Sections.First().Blocks.AddParagraph();
            par2.Inlines.AddRun("Par2");
            par2.TabStops = par2.TabStops.Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right));

            Paragraph par3 = expected.Sections.First().Blocks.AddParagraph();
            par3.Inlines.AddRun("Par3");

            Paragraph par4 = expected.Sections.First().Blocks.AddParagraph();
            par4.Inlines.AddRun("Par4");
            par4.TabStops = par4.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Bar));

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TabStopDifferentTabStopsInMultipleParagraphs.rtf", TestFilesFolder), new RtfFormatProvider());
            foreach (Paragraph p in imported.EnumerateChildrenOfType<Paragraph>())
            {
                p.Properties.ParagraphMarkerProperties.ClearLocalValues();
            }

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenMultipleParagraphHaveDifferentTabStops_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();

            Paragraph par1 = expected.Sections.AddSection().Blocks.AddParagraph();
            par1.Inlines.AddRun("Par1");
            par1.TabStops = par1.TabStops.Insert(new TabStop(Unit.InchToDip(2), TabStopType.Center));

            Paragraph par2 = expected.Sections.First().Blocks.AddParagraph();
            par2.Inlines.AddRun("Par2");
            par2.TabStops = par2.TabStops.Insert(new TabStop(Unit.InchToDip(3), TabStopType.Right));

            Paragraph par3 = expected.Sections.First().Blocks.AddParagraph();
            par3.Inlines.AddRun("Par3");

            Paragraph par4 = expected.Sections.First().Blocks.AddParagraph();
            par4.Inlines.AddRun("Par4");
            par4.TabStops = par4.TabStops.Insert(new TabStop(Unit.InchToDip(1), TabStopType.Bar));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphContainsSameTabStopMultipleTimesItIsPreserved_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            TabStop tabStop = new TabStop(Unit.InchToDip(0.5), TabStopType.Left);
            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                   .Insert(tabStop)
                                                   .Insert(tabStop);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            Assert.AreEqual(4, imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops.Count);
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStop_WhenParagraphContainsSameTabStopMultipleTimesItIsPreserved_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            TabStop tabStop = new TabStop(Unit.InchToDip(0.5), TabStopType.Left);
            paragraph.TabStops = paragraph.TabStops.Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                 .Insert(new TabStop(Unit.InchToDip(0.5), TabStopType.Left))
                                                 .Insert(tabStop)
                                                 .Insert(tabStop);

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            Assert.AreEqual(4, imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops.Count);
            this.AssertParagraphProperties(expected, imported);
        }

        [TestMethod]
        public void TabStopCollectionActualValue_WhenParagraphHasTabStopsFromLocalValueAndStyleAndBasedStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Style baseStyle = new Style("base", StyleType.Paragraph);
            baseStyle.ParagraphProperties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(100, TabStopType.Right));
            document.StyleRepository.Add(baseStyle);

            Style style = new Style("test", StyleType.Paragraph);
            style.BasedOnStyleId = baseStyle.Id;
            style.ParagraphProperties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(100, TabStopType.Left, TabStopLeader.Dot));
            document.StyleRepository.Add(style);

            Paragraph paragraph = document.Sections.AddSection().Blocks
                                                   .AddParagraph().Inlines
                                                   .AddRun("text")
                                                   .Paragraph;
            paragraph.StyleId = style.Id;
            paragraph.Properties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(200, TabStopType.Decimal, TabStopLeader.Dot));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(document, new DocxFormatProvider());
            TabStopCollection tabStops = imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops;

            // Assert
            Assert.AreEqual(3, tabStops.Count);

            Assert.AreEqual(new TabStop(100, TabStopType.Right), tabStops.FirstOrDefault());
            Assert.AreEqual(new TabStop(100, TabStopType.Left, TabStopLeader.Dot), tabStops.ElementAt(1));
            Assert.AreEqual(new TabStop(200, TabStopType.Decimal, TabStopLeader.Dot), tabStops.LastOrDefault());

            this.AssertParagraphProperties(document, imported);
        }

        [TestMethod]
        public void TabStopCollectionActualValue_WhenParagraphHasTabStopsFromLocalValueAndStyleAndBasedStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Style baseStyle = new Style("base", StyleType.Paragraph);
            baseStyle.ParagraphProperties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(100, TabStopType.Right));
            document.StyleRepository.Add(baseStyle);

            Style style = new Style("test", StyleType.Paragraph);
            style.BasedOnStyleId = baseStyle.Id;
            style.ParagraphProperties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(100, TabStopType.Left, TabStopLeader.Dot));
            document.StyleRepository.Add(style);

            Paragraph paragraph = document.Sections.AddSection().Blocks
                                                   .AddParagraph().Inlines
                                                   .AddRun("text")
                                                   .Paragraph;
            paragraph.StyleId = style.Id;
            paragraph.Properties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(200, TabStopType.Decimal, TabStopLeader.Dot));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(document, new RtfFormatProvider());
            TabStopCollection tabStops = imported.EnumerateChildrenOfType<Paragraph>().FirstOrDefault().TabStops;

            // Assert
            Assert.AreEqual(3, tabStops.Count);

            Assert.AreEqual(new TabStop(100, TabStopType.Right), tabStops.FirstOrDefault());
            Assert.AreEqual(new TabStop(100, TabStopType.Left, TabStopLeader.Dot), tabStops.ElementAt(1));
            Assert.AreEqual(new TabStop(200, TabStopType.Decimal, TabStopLeader.Dot), tabStops.LastOrDefault());

            this.AssertParagraphProperties(document, imported);
        }
    }
}