﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.ParagraphProperties
{
    [TestClass]
    public class TextAlignmentTests : ParagraphPropertiesTestsBase
    {
        [TestMethod]
        public void TextAlignment_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().TextAlignment = Alignment.Center;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Right;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Distribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Justified;
            section.Blocks.AddParagraph().TextAlignment = Alignment.ThaiDistribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.LowKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.MediumKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.HighKashida;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TextAlignment_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().TextAlignment = Alignment.Center;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Right;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Distribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Justified;
            section.Blocks.AddParagraph().TextAlignment = Alignment.ThaiDistribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.LowKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.MediumKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.HighKashida;

            // Act and Assert
            this.ParagraphPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TextAlignment_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.TextAlignment = Alignment.Right;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "TextAlignment";

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextAlignment.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void TextAlignment_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            paragraph.TextAlignment = Alignment.Right;

            Run run = paragraph.Inlines.AddRun();
            run.Text = "TextAlignment";

            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextAlignment.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertParagraphProperties(original, imported);
        }

        [TestMethod]
        public void TextAlignment_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().TextAlignment = Alignment.Center;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Right;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Distribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Justified;
            section.Blocks.AddParagraph().TextAlignment = Alignment.ThaiDistribute;
            section.Blocks.AddParagraph().TextAlignment = Alignment.LowKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.MediumKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.HighKashida;
            section.Blocks.AddParagraph().TextAlignment = Alignment.Left;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}
