﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.PlainText
{
    [TestClass]
    public class OneParagraphTests
    {
        private const string TestFileNameWithoutExtension = "PlainText/OneParagraph";

        [TestMethod]
        public void OneParagraph_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertOneParagraphDocument(doc);
        }

        [TestMethod]
        public void OneParagraph_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertOneParagraphDocument(doc);
        }

        [TestMethod]
        public void OneParagraph_ExportImport_Docx()
        {
            this.OneParagraphExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void OneParagraph_ExportImport_Rtf()
        {
            this.OneParagraphExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void OneParagraph_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithOnParagraph();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void OneParagraphExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithOnParagraph();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertOneParagraphDocument(testDocument);
        }

        private void AssertOneParagraphDocument(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Section s = doc.Sections[0];

            Assert.AreEqual(1, s.Blocks.Count);
            DocumentAsserter.AssertParagraphWithText(s.Blocks[0], "Hello World!");
        }

        private static RadFlowDocument CreateDocumentWithOnParagraph()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection()
                       .Blocks.AddParagraph()
                       .Inlines.AddRun()
                       .Text = "Hello World!";

            return document;
        }
    }
}
