﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.PlainText
{
    [TestClass]
    public class PreserveWhitespacesTests
    {
        [TestMethod]
        public void PreserveWhitespaces_ImportExport_Docx()
        {
            RadFlowDocument document = CreateTestDocumentWithWhitespaces();

            PreserveWhitespacesExportImportTemplate(document, new DocxFormatProvider());
        }

        [TestMethod]
        public void PreserveWhitespaces_Validate_Xml()
        {
            RadFlowDocument document = CreateTestDocumentWithWhitespaces();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private static RadFlowDocument CreateTestDocumentWithWhitespaces()
        {
            //Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            originalDoc.Sections.AddSection()
                       .Blocks.AddParagraph()
                       .Inlines.AddRun()
                       .Text = "Spaces\t ";

            return originalDoc;
        }

        private static void PreserveWhitespacesExportImportTemplate(RadFlowDocument document, IFormatProvider<RadFlowDocument> provider)
        {
            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(document, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(document, testDocument);
        }
    }
}
