﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.PlainText
{
    [TestClass]
    public class SpecialSymbolsTests
    {
        private const string TestFileNameWithoutExtension = "PlainText/Symbols";

        [TestMethod]
        public void SpecialSymbols_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertSpecialSymbolsDocument(doc);
        }

        [TestMethod]
        public void SpecialSymbols_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertSpecialSymbolsDocument(doc);
        }

        [TestMethod]
        public void SpecialSymbols_ExportImport_Docx()
        {
            this.SpecialSymbolsExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void SpecialSymbols_ExportImport_Rtf()
        {
            this.SpecialSymbolsExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void SpecialSymbols_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithSpecialSymbols();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void SpecialSymbolsExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithSpecialSymbols();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertSpecialSymbolsDocument(testDocument);
        }

        private void AssertSpecialSymbolsDocument(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Section s = doc.Sections[0];

            Assert.AreEqual(11, s.Blocks.Count);
            DocumentAsserter.AssertParagraphWithText(s.Blocks[0], "\t");
            DocumentAsserter.AssertParagraphWithText(s.Blocks[1], "–"); // en-dash
            DocumentAsserter.AssertParagraphWithText(s.Blocks[2], "—"); // em-dash
            DocumentAsserter.AssertParagraphWithText(s.Blocks[3], " "); // en space
            DocumentAsserter.AssertParagraphWithText(s.Blocks[4], " "); // em space
            DocumentAsserter.AssertParagraphWithText(s.Blocks[5], "‘"); // left quote
            DocumentAsserter.AssertParagraphWithText(s.Blocks[6], "’"); // right quote
            DocumentAsserter.AssertParagraphWithText(s.Blocks[7], "“");
            DocumentAsserter.AssertParagraphWithText(s.Blocks[8], "”");
            DocumentAsserter.AssertParagraphWithText(s.Blocks[9], "•");
            DocumentAsserter.AssertParagraphWithText(s.Blocks[10], "\u00A0"); // non-breaking space
        }

        private static RadFlowDocument CreateDocumentWithSpecialSymbols()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            section.Blocks.AddParagraph().Inlines.AddRun().Text = "\t";
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "–"; // en-dash
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "—"; // em-dash
            section.Blocks.AddParagraph().Inlines.AddRun().Text = " "; // en space
            section.Blocks.AddParagraph().Inlines.AddRun().Text = " "; // em space
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "‘"; // left 
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "’"; // right
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "“";
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "”";
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "•";
            section.Blocks.AddParagraph().Inlines.AddRun().Text = "\u00A0"; // non-breaking space

            return document;
        }
    }
}
