﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.PlainText
{
    [TestClass]
    public class TwoParagraphsTests
    {
        private const string TestFileNameWithoutExtension = "PlainText/TwoParagraphs";

        [TestMethod]
        public void TwoParagraphs_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertTwoPragraphsDocument(doc);
        }

        [TestMethod]
        public void TwoParagraphs_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertTwoPragraphsDocument(doc);
        }

        [TestMethod]
        public void TwoParagraphs_ExportImport_Docx()
        {
            this.TwoParagraphsExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void TwoParagraphs_ExportImport_Rtf()
        {
            this.TwoParagraphsExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void TwoParagraphs_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithTwoParagraphs();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void TwoParagraphsExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithTwoParagraphs();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);
                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertTwoPragraphsDocument(testDocument);
        }

        private void AssertTwoPragraphsDocument(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Section s = doc.Sections[0];

            Assert.AreEqual(2, s.Blocks.Count);
            DocumentAsserter.AssertParagraphWithText(s.Blocks[0], "First paragraph");
            DocumentAsserter.AssertParagraphWithText(s.Blocks[1], "Second paragraph");
        }

        private static RadFlowDocument CreateDocumentWithTwoParagraphs()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph()
                .Inlines.AddRun()
                .Text = "First paragraph";

            section.Blocks.AddParagraph()
                .Inlines.AddRun()
                .Text = "Second paragraph";

            return document;
        }
    }
}