﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class HasDifferentFirstPageHeaderFooterTests : SectionPropertiesTestBase
    {
        private const string TestFileNameWithoutExtension = TestFilesFolder + "HasDifferentFirstPageHeaderFooter";

        [TestMethod]
        public void HasDifferentFirstPageHeaderFooter_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithSectionWithDifferentFirstPageHeaderFooter();

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(document, new DocxFormatProvider());
        }

        [TestMethod]
        public void HasDifferentFirstPageHeaderFooter_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().AllowOverflowPunctuation = false;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void HasDifferentFirstPageHeaderFooter_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = CreateDocumentWithSectionWithDifferentFirstPageHeaderFooter();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void HasDifferentFirstPageHeaderFooter_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = CreateDocumentWithSectionWithDifferentFirstPageHeaderFooter();
            original.Sections.First().HeaderTopMargin = 47;
            original.Sections.First().FooterBottomMargin = 47;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void HasDifferentFirstPageHeaderFooter_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithSectionWithDifferentFirstPageHeaderFooter();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private static RadFlowDocument CreateDocumentWithSectionWithDifferentFirstPageHeaderFooter()
        {
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            section.HasDifferentFirstPageHeaderFooter = true;

            section.Blocks.AddParagraph();

            return original;
        }
    }
}
