﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class HeaderFooterMarginsTests : SectionPropertiesTestBase
    {
        [TestMethod]
        public void HeaderTopMargin_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.HeaderTopMargin = 36;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void HeaderTopMargin_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.HeaderTopMargin = 36;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void HeaderTopMargin_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.HeaderTopMargin = 36;

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void FooterBottomMargin_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.FooterBottomMargin = 42;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void FooterBottomMargin_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.FooterBottomMargin = 42;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void FooterBottomMargin_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.FooterBottomMargin = 42;

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void HeaderTopMargin_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.HeaderTopMargin = 80;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "HeaderTopMargin.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void HeaderTopMargin_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.HeaderTopMargin = 80;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "HeaderTopMargin.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void FooterBottomMargin_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.FooterBottomMargin = 120;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "FooterBottomMargin.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void FooterBottomMargin_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.FooterBottomMargin = 120;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "FooterBottomMargin.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }
    }
}
