﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class PageMarginsTests : SectionPropertiesTestBase
    {
        [TestMethod]
        public void PageMargins_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void PageMarginsAndOrientation_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);
            section.PageOrientation = PageOrientation.Landscape;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void PageMargins_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(32, 48, 64, 80);

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageMargins.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageMarginsAndOrientation_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(48, 64, 80, 32);
            section.PageSize = new Size(1056, 816);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageMarginsAndOrientation.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageMargins_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void PageMarginsAndOrientation_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);
            section.PageOrientation = PageOrientation.Landscape;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void PageMargins_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(32, 48, 64, 80);

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageMargins.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageMarginsAndOrientation_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(48, 64, 80, 32);
            section.PageSize = new Size(1056, 816);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageMarginsAndOrientation.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageMargins_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void PageMarginsAndOrientation_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.PageMargins = new Primitives.Padding(10, 20, 30, 40);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
