﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utils;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class PageNumeringSettingsTests : SectionPropertiesTestBase
    {
        [TestMethod]
        public void PageNumberingSettings_WhenAllPropertiesAreSet_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.StartingPageNumber = 0;
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.Decimal;
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Colon;
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenAllPropertiesAreSet_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.StartingPageNumber = 2;
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.LowerLetter;
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsAll.docx", new DocxFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            AssertPageNumberingSettings(section, importedSection);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenAllPropertiesAreSet_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.StartingPageNumber = 0;
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.Decimal;
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Colon;
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenAllPropertiesAreSet_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.StartingPageNumber = 2;
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.LowerLetter;
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsAll.rtf", new RtfFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            AssertPageNumberingSettings(section, importedSection);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenStartingPageNumberIsSet_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.StartingPageNumber = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenStartingPageNumberIsSet_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.StartingPageNumber = 5;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsStartingPageNumber.docx", new DocxFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenStartingPageNumberIsSet_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.StartingPageNumber = 0;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenStartingPageNumberIsSet_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.StartingPageNumber = 5;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsStartingPageNumber.rtf", new RtfFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenPageNumberFormatIsSet_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperRoman;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenPageNumberFormatIsSet_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperRoman;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsPageNumberFormat.docx", new DocxFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.PageNumberSettings)]
        public void PageNumberingSettings_WhenPageNumberFormatIsSet_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperRoman;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);

#if VISUAL
            AssertVisual(editor.Document);
#endif
        }

        [TestMethod]
        public void PageNumberingSettings_WhenPageNumberFormatIsSet_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperRoman;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsPageNumberFormat.rtf", new RtfFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterHeadingStyleIndexIsSet_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterHeadingStyleIndexIsSet_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsChapterHeadingStyle.docx", new DocxFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterHeadingStyleIndexIsSet_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterHeadingStyleIndexIsSet_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.ChapterHeadingStyleIndex = 1;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsChapterHeadingStyle.rtf", new RtfFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterSeparatorCharacterIsSet_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.EmDash;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterSeparatorCharacterIsSet_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsChapterSeparatorCharacter.docx", new DocxFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterSeparatorCharacterIsSet_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("test");
            Section section = editor.Document.Sections.First();
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.EmDash;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenChapterSeparatorCharacterIsSet_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsChapterSeparatorCharacter.rtf", new RtfFormatProvider());

            // Assert
            this.AssertSectionProperties(section.Document, imported);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.PageNumberSettings)]
        public void PageNumberingSettings_WhenAreDifferentForMultipleSections_ExportImport_Docx()
        {
            RadFlowDocument document = new RadFlowDocument();

            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            editor.InsertText("section1");
            Section firstSection = editor.Document.Sections.First();
            firstSection.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperLetter;
            firstSection.PageNumberingSettings.StartingPageNumber = 2;

            Section secondSection = editor.InsertSection();
            secondSection.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            secondSection.PageNumberingSettings.ChapterHeadingStyleIndex = 2;
            editor.InsertText("section2");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);

            Assert.AreEqual(NumberingStyle.UpperLetter, firstSection.PageNumberingSettings.PageNumberFormat);
            Assert.AreEqual(2, firstSection.PageNumberingSettings.StartingPageNumber);
            Assert.IsNull(firstSection.PageNumberingSettings.ChapterSeparatorCharacter);
            Assert.IsNull(firstSection.PageNumberingSettings.ChapterHeadingStyleIndex);

            Assert.AreEqual(ChapterSeparatorType.Period, secondSection.PageNumberingSettings.ChapterSeparatorCharacter);
            Assert.AreEqual(2, secondSection.PageNumberingSettings.ChapterHeadingStyleIndex);
            Assert.IsNull(secondSection.PageNumberingSettings.PageNumberFormat);
            Assert.IsNull(secondSection.PageNumberingSettings.StartingPageNumber);

#if VISUAL
            AssertVisual(document, 1);
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.PageNumberSettings)]
        public void PageNumberingSettings_WhenAreDifferentForMultipleSections_Import_Docx()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("section1");
            Section firstSection = editor.Document.Sections.First();
            firstSection.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperLetter;
            firstSection.PageNumberingSettings.StartingPageNumber = 2;

            Section secondSection = editor.InsertSection();
            secondSection.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            secondSection.PageNumberingSettings.ChapterHeadingStyleIndex = 2;
            editor.InsertText("section2");

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsMultipleSections.docx", new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);

#if VISUAL
            AssertVisual(imported, 1);
#endif
        }

        [TestMethod]
        public void PageNumberingSettings_WhenAreDifferentForMultipleSections_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("section1");
            Section firstSection = editor.Document.Sections.First();
            firstSection.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperLetter;
            firstSection.PageNumberingSettings.StartingPageNumber = 2;

            Section secondSection = editor.InsertSection();
            secondSection.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            secondSection.PageNumberingSettings.ChapterHeadingStyleIndex = 2;
            editor.InsertText("section2");

            // Act
            RadFlowDocument imported = this.ExportImportDocument(editor.Document, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);

            Assert.AreEqual(NumberingStyle.UpperLetter, firstSection.PageNumberingSettings.PageNumberFormat);
            Assert.AreEqual(2, firstSection.PageNumberingSettings.StartingPageNumber);
            Assert.IsNull(firstSection.PageNumberingSettings.ChapterSeparatorCharacter);
            Assert.IsNull(firstSection.PageNumberingSettings.ChapterHeadingStyleIndex);

            Assert.AreEqual(ChapterSeparatorType.Period, secondSection.PageNumberingSettings.ChapterSeparatorCharacter);
            Assert.AreEqual(2, secondSection.PageNumberingSettings.ChapterHeadingStyleIndex);
            Assert.IsNull(secondSection.PageNumberingSettings.PageNumberFormat);
            Assert.IsNull(secondSection.PageNumberingSettings.StartingPageNumber);
        }

        [TestMethod]
        public void PageNumberingSettings_WhenAreDifferentForMultipleSections_Import_Rtf()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.InsertText("section1");
            Section firstSection = editor.Document.Sections.First();
            firstSection.PageNumberingSettings.PageNumberFormat = NumberingStyle.UpperLetter;
            firstSection.PageNumberingSettings.StartingPageNumber = 2;

            Section secondSection = editor.InsertSection();
            secondSection.PageNumberingSettings.ChapterSeparatorCharacter = ChapterSeparatorType.Period;
            secondSection.PageNumberingSettings.ChapterHeadingStyleIndex = 2;
            editor.InsertText("section2");

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageNumberingSettingsMultipleSections.rtf", new RtfFormatProvider());


            // Assert
            StructureAsserter.AssertDocumentsAreEqual(editor.Document, imported);
        }

        private static void AssertPageNumberingSettings(Section expected, Section imported)
        {
            Assert.AreEqual(expected.PageNumberingSettings.ChapterHeadingStyleIndex, imported.PageNumberingSettings.ChapterHeadingStyleIndex);
            Assert.AreEqual(expected.PageNumberingSettings.ChapterSeparatorCharacter, imported.PageNumberingSettings.ChapterSeparatorCharacter);
            Assert.AreEqual(expected.PageNumberingSettings.PageNumberFormat, imported.PageNumberingSettings.PageNumberFormat);
            Assert.AreEqual(expected.PageNumberingSettings.StartingPageNumber, imported.PageNumberingSettings.StartingPageNumber);
        }
    }
}