﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class PageSizeTests : SectionPropertiesTestBase
    {
        [TestMethod]
        public void PageSize_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(300, 400);

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void PageSizeAndOrientation_ExportImport_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(300, 400);
            section.PageOrientation = PageOrientation.Landscape;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void PageSize_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(400, 800);

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageSize.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageSizeAndOrientation_Import_Rtf()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(800, 400);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageSizeAndOrientation.rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageSize_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(300, 400);

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void PageSizeAndOrientation_ExportImport_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(300, 400);
            section.PageOrientation = PageOrientation.Landscape;

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(section.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void PageSize_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(400, 800);

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageSize.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageSizeAndOrientation_Import_Docx()
        {
            // Arrange
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(800, 400);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument original = section.Document;
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageSizeAndOrientation.docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(original, imported);
        }

        [TestMethod]
        public void PageSize_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(300, 400);

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void PageSizeAndOrientation_Validate_Xml()
        {
            Section section = this.CreateDocumentWithSection();
            section.PageSize = new Size(800, 400);
            section.PageOrientation = PageOrientation.Landscape;

            RadFlowDocument document = section.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
