﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if !VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#endif
namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class SectionTypeTests : SectionPropertiesTestBase
    {
        private const string TestFileNameWithoutExtension = TestFilesFolder + "SectionTypes";

        [TestMethod]
        public void SectionType_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = CreateDocumentWithAllTypesOfSections();

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void SectionType_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = CreateDocumentWithAllTypesOfSections();

            // Act and Assert
            this.SectionPropertiesExportImportTemplate(doc, new DocxFormatProvider());
#if VISUAL
            AssertVisual(doc, 4);
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void SectionType_Import_Docx()
        {
            // Arrange
            RadFlowDocument expected = CreateDocumentWithAllTypesOfSections();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(expected, imported);

#if VISUAL
            AssertVisual(imported, 4);
#endif
        }

        [TestMethod]
        public void SectionType_Import_Rtf()
        {
            // Arrange
            RadFlowDocument expected = CreateDocumentWithAllTypesOfSections();
            foreach (Section s in expected.Sections)
            {
                s.HeaderTopMargin = 47;
                s.FooterBottomMargin = 47;
            }
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            // Act and Assert
            this.AssertSectionProperties(expected, imported);
        }
#if !VISUAL
        [TestMethod]
        public void SectionType_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithAllTypesOfSections();
            XmlValidator.ValidateOpenXmlDocument(document);
        }
#endif

        private static RadFlowDocument CreateDocumentWithAllTypesOfSections()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section1 = doc.Sections.AddSection();
            section1.Blocks.AddParagraph().Inlines.AddRun().Text = "NextPage";

            Section section2 = doc.Sections.AddSection();
            section2.Blocks.AddParagraph().Inlines.AddRun().Text = "OddPage";
            section2.SectionType = SectionType.OddPage;

            Section section3 = doc.Sections.AddSection();
            section3.Blocks.AddParagraph().Inlines.AddRun().Text = "EvenPage";
            section3.SectionType = SectionType.EvenPage;

            Section section4 = doc.Sections.AddSection();
            section4.Blocks.AddParagraph().Inlines.AddRun().Text = "NextColumn";
            section4.SectionType = SectionType.NextColumn;

            Section section5 = doc.Sections.AddSection();
            section5.Blocks.AddParagraph().Inlines.AddRun().Text = "Continuous";
            section5.SectionType = SectionType.Continuous;

            return doc;
        }
    }
}
