﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if VISUAL
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Flow.Model.Editing;
#endif
namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.SectionProperties
{
    [TestClass]
    public class VerticalAlignmentTests : SectionPropertiesTestBase
    {
        [TestMethod]
        public void VerticalAlignment_WhenIsTop_Import_Docx()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentTop.docx", new DocxFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Top, importedSection.VerticalAlignment);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void VerticalAlignment_WhenIsTop_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Top;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
#if VISUAL
            AssertVisual(expected);
#endif
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsTop_Import_Rtf()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentTop.rtf", new RtfFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Top, importedSection.VerticalAlignment);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsTop_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Top;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsBottom_Import_Docx()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentBottom.docx", new DocxFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Bottom, importedSection.VerticalAlignment);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void VerticalAlignment_WhenIsBottom_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Bottom;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);

#if VISUAL
            AssertVisual(expected);
#endif
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsBottom_Import_Rtf()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentBottom.rtf", new RtfFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Bottom, importedSection.VerticalAlignment);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsBottom_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Bottom;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsCenter_Import_Docx()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentCenter.docx", new DocxFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Center, importedSection.VerticalAlignment);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void VerticalAlignment_WhenIsCenter_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Center;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
#if VISUAL
            AssertVisual(expected);
#endif
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsCenter_Import_Rtf()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentCenter.rtf", new RtfFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Center, importedSection.VerticalAlignment);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsCenter_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Center;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsJustified_Import_Docx()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentJustified.docx", new DocxFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Justified, importedSection.VerticalAlignment);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Section)]
        public void VerticalAlignment_WhenIsJustified_ExportImport_Docx()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Justified;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new DocxFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
#if VISUAL
            AssertVisual(expected, 1);
#endif
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsJustified_Import_Rtf()
        {
            // Arrange & Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesFolder + "SectionVerticalAlignmentJustified.rtf", new RtfFormatProvider());
            Section importedSection = imported.Sections.First();

            // Assert
            Assert.AreEqual(VerticalAlignment.Justified, importedSection.VerticalAlignment);
        }

        [TestMethod]
        public void VerticalAlignment_WhenIsJustified_ExportImport_Rtf()
        {
            // Arrange 
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.VerticalAlignment = VerticalAlignment.Justified;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(expected, new RtfFormatProvider());

            // Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

#if VISUAL
        protected override void AssertVisual(RadFlowDocument document, int pageDown = 0)
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText("Paragraph 1");
            editor.InsertParagraph();
            editor.InsertText("Paragraph 2");
            editor.InsertParagraph();
            editor.InsertText("Paragraph 3");
            editor.InsertParagraph();

            base.AssertVisual(document, pageDown);
        }
#endif
    }
}
