﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Sections
{
    [TestClass]
    public class OneSectionTests
    {
        private const string TestFilesWithoutExtension = "Sections/OneSection";

        [TestMethod]
        public void OneSection_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithOneSection();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void OneSection_ExportImport_Docx()
        {
            this.OneSectionExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void OneSection_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithOneSection();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void OneSectionExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithOneSection();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string debugStr = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private static RadFlowDocument CreateDocumentWithOneSection()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks.AddParagraph();

            return expected;
        }
    }
}
