﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Sections
{
    [TestClass]
    public class SingleEmptySectionTests
    {
        private const string TestFilesWithoutExtension = "Sections/SingleEmptySection";

        [TestMethod]
        public void SingleEmptySection_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithSingleEmptySection();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void SingleEmptySection_ExportImport_Docx()
        {
            this.SingleEmptySectionExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void SingleEmptySection_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithSingleEmptySection();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void SingleEmptySectionExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithSingleEmptySection();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string debugStr = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private static RadFlowDocument CreateDocumentWithSingleEmptySection()
        {
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection();

            return expected;
        }
    }
}
