﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Sections
{
    [TestClass]
    public class TwoSectionsTests
    {
        private const string TestFilesWithoutExtension = "Sections/TwoSections";

        [TestMethod]
        public void TwoSections_Import_Docx()
        {
            RadFlowDocument expected = CreateDocumentWithTwoSections();
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(TestFilesWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void TwoSections_ExportImport_Docx()
        {
            this.TwoSectionsExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void TwoSections_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithTwoSections();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void TwoSectionsExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = CreateDocumentWithTwoSections();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string debugStr = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(originalDoc, testDocument);
        }

        private static RadFlowDocument CreateDocumentWithTwoSections()
        {
            RadFlowDocument expected = new RadFlowDocument();
            Section section = expected.Sections.AddSection();
            section.Blocks
                .AddParagraph().Inlines
                .AddRun()
                .Text = "First";


            Section section2 = expected.Sections.AddSection();
            section2.Blocks
                .AddParagraph().Inlines
                .AddRun()
                .Text = "Second";

            return expected;
        }
    }
}
