﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Watermarks;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.StructuredDocumentTags
{
    [TestClass]
    public class StructuredDocumentTagTests
    {
        private const string TestFilesFolder = "StructuredDocumentTags/";

        [TestMethod]
        public void SdtContent_WhenIsAtInlineLevel_IsNotImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun().Text = "test";

            // Act
            RadFlowDocument imported = GetDocument("InlineSdt.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void SdtContent_WhenIsAtParagraphLevel_IsNotImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun().Text = "test";

            // Act
            RadFlowDocument imported = GetDocument("ParagraphSdt.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void SdtContent_WhenIsAtTableLevel_IsNotImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun().Text = "Test";
            expected.Sections.First().Blocks.AddParagraph();

            // Act
            RadFlowDocument imported = GetDocument("TableSdt.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void SdtContent_WhenIsAtTableRowLevel_IsNotImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun().Text = "Test";
            expected.Sections.First().Blocks.Add(new Table(expected, 1, 2));
            expected.Sections.First().Blocks.AddParagraph();

            foreach (TableCell cell in expected.EnumerateChildrenOfType<TableCell>())
            {
                cell.Blocks.AddParagraph();
            }

            // Act
            RadFlowDocument imported = GetDocument("TableRowSdt.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void SdtContent_WhenIsAtTableCellLevel_IsNotImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun().Text = "Test";
            expected.Sections.First().Blocks.Add(new Table(expected, 2, 2));
            expected.Sections.First().Blocks.AddParagraph();

            foreach (TableCell cell in expected.EnumerateChildrenOfType<TableCell>())
            {
                cell.Blocks.AddParagraph();
            }

            expected.EnumerateChildrenOfType<Table>().First().Rows.First().Cells.RemoveAt(0);

            // Act
            RadFlowDocument imported = GetDocument("TableCellSdt.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        [TestMethod]
        public void StdContent_WhenContainsWatermark_WatermarkIsImported_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            expected.Sections.AddSection();
            expected.Sections.First().Headers.Add(HeaderFooterType.Even).Blocks.AddParagraph();
            expected.Sections.First().Headers.Add(HeaderFooterType.First).Blocks.AddParagraph();
            expected.Sections.First().Footers.Add().Blocks.AddParagraph();
            expected.Sections.First().Footers.Add(HeaderFooterType.Even).Blocks.AddParagraph();
            expected.Sections.First().Footers.Add(HeaderFooterType.First).Blocks.AddParagraph();

            TextWatermarkTests.SetTextWatermarkToHeader(expected.Sections.First().Headers.Add(), 762.4, 117.2666, 315, 0.5, Colors.Red, "CONFIDENTIAL", new FontFamily("Arial"));
            expected.Sections.First().Blocks.AddParagraph();

            // Act
            RadFlowDocument imported = GetDocument("Watermark.docx");

            //Assert
            StructureAsserter.AssertDocumentsAreEqual(expected, imported);
        }

        private static RadFlowDocument GetDocument(string fileName)
        {
            return TestHelper.GetDocumentFromFile(TestFilesFolder + fileName, new DocxFormatProvider());
        }
    }
}