﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class BasedOnTests : StyleTestBase
    {
        [TestMethod]
        public void BasedOnParagraphStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = CreateDocumentWithBasedOnParagraphStyle();

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void BasedOnParagraphStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = CreateDocumentWithBasedOnParagraphStyle();

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void BasedOnParagraphStyle_Validate_Xml()
        {
            RadFlowDocument document = CreateDocumentWithBasedOnParagraphStyle();
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private static RadFlowDocument CreateDocumentWithBasedOnParagraphStyle()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style baseParStyle = new Style("BaseStyle", StyleType.Paragraph);
            baseParStyle.Name = "Base Style";

            Style derivedParStyle = new Style("DerivedStyle", StyleType.Paragraph);
            derivedParStyle.Name = "Derived Style";
            derivedParStyle.BasedOnStyleId = baseParStyle.Id;

            doc.StyleRepository.Add(baseParStyle);
            doc.StyleRepository.Add(derivedParStyle);
            return doc;
        }
    }
}
