﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using FontWeights = System.Windows.FontWeights;
using FontStyles = System.Windows.FontStyles;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class CharacterStyleTests : StyleTestBase
    {
        [TestMethod]
        public void CharacterStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            charStyle.UIPriority = 66;
            charStyle.Name = "Test Character Style";

            doc.StyleRepository.Add(charStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void CharacterStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            charStyle.UIPriority = 66;
            charStyle.Name = "Test Character Style";

            doc.StyleRepository.Add(charStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void CharacterStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            charStyle.UIPriority = 66;
            charStyle.Name = "Test Character Style";

            doc.StyleRepository.Add(charStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void CharacterStyleWithProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);

            this.FillPropertiesWithValues(charStyle.CharacterProperties);

            doc.StyleRepository.Add(charStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void CharacterStyleWithProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);

            this.FillPropertiesWithValues(charStyle.CharacterProperties);

            doc.StyleRepository.Add(charStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void CharacterStyleWithProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);

            this.FillPropertiesWithValues(charStyle.CharacterProperties);

            doc.StyleRepository.Add(charStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void RunWithStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 48;

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            this.FillPropertiesWithValues(charStyle.CharacterProperties);
            doc.StyleRepository.Add(charStyle);
            run.StyleId = charStyle.Id;

            // Act and Assert
            this.StyleAndElementsExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void RunWithStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 48;

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            this.FillPropertiesWithValues(charStyle.CharacterProperties);
            doc.StyleRepository.Add(charStyle);
            run.StyleId = charStyle.Id;

            // Act and Assert
            this.StyleAndElementsExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void RunWithStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";
            run.FontSize = 48;

            Style charStyle = new Style("TestCharacterStyle", StyleType.Character);
            this.FillPropertiesWithValues(charStyle.CharacterProperties);
            doc.StyleRepository.Add(charStyle);
            run.StyleId = charStyle.Id;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void RunStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            run.StyleId = "CharacterStyle";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.BasedOnStyleId = "DefaultParagraphFont";
            characterStyle.IsCustom = true;
            characterStyle.IsPrimary = true;
            characterStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            characterStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            characterStyle.UIPriority = 1;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterStyle.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(characterStyle, imported.StyleRepository.GetStyle(characterStyle.Id));

            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Run>(original, imported);
        }

        [TestMethod]
        public void RunStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            run.StyleId = "CharacterStyle";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.BasedOnStyleId = "DefaultParagraphFont";
            characterStyle.IsCustom = false;
            characterStyle.IsPrimary = true;
            characterStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            characterStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            characterStyle.UIPriority = 1;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CharacterStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(characterStyle, imported.StyleRepository.GetStyle(characterStyle.Id));

            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Run>(original, imported);
        }
    }
}
