using System.IO;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Media;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class DefaultStylesTests
    {
        public const string TestFilesFolder = "DefaultStyles/";

        [TestMethod]
        public void DefaultProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Calibri");

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Custom_DefaultCharacterProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Calibri");
            doc.DefaultStyle.CharacterProperties.FontSize.LocalValue = 36;
            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            doc.DefaultStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Custom_DefaultParagraphProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();

            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Calibri");

            doc.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.GreenYellow);
            doc.DefaultStyle.ParagraphProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Azure);
            doc.DefaultStyle.ParagraphProperties.ShadingPattern.LocalValue = ShadingPattern.Percent45;
            doc.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = 36;
            doc.DefaultStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Exact;
            doc.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 12;

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void DefaultProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.DefaultStyle.CharacterProperties.ClearLocalValues();
            doc.DefaultStyle.ParagraphProperties.ClearLocalValues();

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void DefaultProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void Custom_DefaultCharacterProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();

            doc.DefaultStyle.CharacterProperties.FontSize.LocalValue = 36;
            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            doc.DefaultStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Custom_DefaultCharacterProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();

            doc.DefaultStyle.CharacterProperties.FontSize.LocalValue = 36;
            doc.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            doc.DefaultStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Green);

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void Custom_DefaultParagraphProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();

            doc.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.GreenYellow);
            doc.DefaultStyle.ParagraphProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Azure);
            doc.DefaultStyle.ParagraphProperties.ShadingPattern.LocalValue = ShadingPattern.Percent45;
            doc.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = 36;
            doc.DefaultStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Exact;
            doc.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 12;

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Custom_DefaultParagraphProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();

            doc.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.GreenYellow);
            doc.DefaultStyle.ParagraphProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Azure);
            doc.DefaultStyle.ParagraphProperties.ShadingPattern.LocalValue = ShadingPattern.Percent45;
            doc.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = 36;
            doc.DefaultStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Exact;
            doc.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 12;

            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void Word_DefaultStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            original.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(ThemeFontType.Minor);
            original.DefaultStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(11);

            original.DefaultStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            original.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.TwipToDip(160);
            original.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 259.0 / 240.0;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}WordDefaultStyles.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertDocumentDefaultStyles(original, imported);
        }

        [TestMethod]
        public void Word_DefaultStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            original.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Calibri"));

            original.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.TwipToDip(160);
            original.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 1.08;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}WordDefaultStyles.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertDocumentDefaultStyles(original, imported);
        }


        [TestMethod]
        public void Custom_DefaultStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            original.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            original.DefaultStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(24);

            original.DefaultStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Justified;
            original.DefaultStyle.ParagraphProperties.LeftIndent.LocalValue = Unit.TwipToDip(1440);
            original.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.TwipToDip(480);
            original.DefaultStyle.ParagraphProperties.SpacingBefore.LocalValue = Unit.TwipToDip(720);
            original.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 259.0 / 240.0;
            original.DefaultStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CustomDefaultStyle.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertDocumentDefaultStyles(original, imported);
        }

        [TestMethod]
        public void Custom_DefaultStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            original.DefaultStyle.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            original.DefaultStyle.CharacterProperties.FontSize.LocalValue = Unit.PointToDip(24);

            original.DefaultStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Justified;
            original.DefaultStyle.ParagraphProperties.LeftIndent.LocalValue = Unit.TwipToDip(1440);
            original.DefaultStyle.ParagraphProperties.SpacingAfter.LocalValue = Unit.TwipToDip(480);
            original.DefaultStyle.ParagraphProperties.SpacingBefore.LocalValue = Unit.TwipToDip(720);
            original.DefaultStyle.ParagraphProperties.LineSpacing.LocalValue = 1.08;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CustomDefaultStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertDocumentDefaultStyles(original, imported);
        }


        private void DefaultPropertiesExportImportTemplate(
          RadFlowDocument original,
          IFormatProvider<RadFlowDocument> provider)
        {
            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(original, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string rtfString = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertDocumentDefaultStyles(original, testDocument);
        }

        private void AssertDocumentDefaultStyles(RadFlowDocument original, RadFlowDocument testDocument)
        {
            //Assert
            StylePropertyAsserter.AssertPropertiesAreEqual(
                original.DefaultStyle.CharacterProperties,
                testDocument.DefaultStyle.CharacterProperties);

            StylePropertyAsserter.AssertPropertiesAreEqual(
                original.DefaultStyle.ParagraphProperties,
                testDocument.DefaultStyle.ParagraphProperties);
        }

        [TestMethod]
        public void InitialProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            this.DefaultPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }
    }
}