﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class LinkedStyleTests : StyleTestBase
    {
        [TestMethod]
        public void LinkedStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("CharacterStyle", StyleType.Character);
            charStyle.Name = "Character Style";

            Style parStyle = new Style("ParagraphStyle", StyleType.Paragraph);
            parStyle.Name = "Paragraph Style";

            parStyle.LinkedStyleId = charStyle.Id;

            doc.StyleRepository.Add(charStyle);
            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void LinkedStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("CharacterStyle", StyleType.Character);
            charStyle.Name = "Character Style";

            Style parStyle = new Style("ParagraphStyle", StyleType.Paragraph);
            parStyle.Name = "Paragraph Style";

            parStyle.LinkedStyleId = charStyle.Id;

            doc.StyleRepository.Add(charStyle);
            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void LinkedStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style charStyle = new Style("CharacterStyle", StyleType.Character);
            charStyle.Name = "Character Style";

            Style parStyle = new Style("ParagraphStyle", StyleType.Paragraph);
            parStyle.Name = "Paragraph Style";

            parStyle.LinkedStyleId = charStyle.Id;

            doc.StyleRepository.Add(charStyle);
            doc.StyleRepository.Add(parStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}
