﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class NextStyleIdTests : StyleTestBase
    {
        [TestMethod]
        public void NextParagraphStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style firstStyle = new Style("FirstStyle", StyleType.Paragraph);
            firstStyle.Name = "First Style";

            Style nextStyle = new Style("NextStyle", StyleType.Paragraph);
            nextStyle.Name = "Next Style";
            firstStyle.NextStyleId = nextStyle.Id;

            doc.StyleRepository.Add(firstStyle);
            doc.StyleRepository.Add(nextStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void NextParagraphStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style firstStyle = new Style("FirstStyle", StyleType.Paragraph);
            firstStyle.Name = "First Style";

            Style nextStyle = new Style("NextStyle", StyleType.Paragraph);
            nextStyle.Name = "Next Style";
            firstStyle.NextStyleId = nextStyle.Id;

            doc.StyleRepository.Add(firstStyle);
            doc.StyleRepository.Add(nextStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void NextParagraphStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style firstStyle = new Style("FirstStyle", StyleType.Paragraph);
            firstStyle.Name = "First Style";

            Style nextStyle = new Style("NextStyle", StyleType.Paragraph);
            nextStyle.Name = "Next Style";
            firstStyle.NextStyleId = nextStyle.Id;

            doc.StyleRepository.Add(firstStyle);
            doc.StyleRepository.Add(nextStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }
    }
}
