﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using FontWeights = System.Windows.FontWeights;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class ParagraphStyleTests : StyleTestBase
    {
        [TestMethod]
        public void ParagraphStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            parStyle.UIPriority = 66;
            parStyle.Name = "Test Paragraph Style";

            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ParagraphStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            parStyle.UIPriority = 66;
            parStyle.Name = "Test Paragraph Style";

            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            parStyle.UIPriority = 66;
            parStyle.Name = "Test Paragraph Style";

            doc.StyleRepository.Add(parStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphStyleWithProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);

            this.FillPropertiesWithValues(parStyle.CharacterProperties);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);

            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ParagraphStyleWithProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);

            this.FillPropertiesWithValues(parStyle.CharacterProperties);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);

            doc.StyleRepository.Add(parStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphStyleWithProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);

            this.FillPropertiesWithValues(parStyle.CharacterProperties);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);

            doc.StyleRepository.Add(parStyle);


            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphWithStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Inlines.AddRun().Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);
            doc.StyleRepository.Add(parStyle);
            par.StyleId = parStyle.Id;
            par.Spacing.SpacingAfter = 100;

            // Act and Assert
            this.StyleAndElementsExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void ParagraphWithStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Inlines.AddRun().Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);
            doc.StyleRepository.Add(parStyle);
            par.StyleId = parStyle.Id;
            par.Spacing.SpacingAfter = 100;

            // Act and Assert
            this.StyleAndElementsExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void ParagraphWithStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph par = doc.Sections.AddSection().Blocks.AddParagraph();
            par.Inlines.AddRun().Text = "test";

            Style parStyle = new Style("TestParagraphStyle", StyleType.Paragraph);
            this.FillPropertiesWithValues(parStyle.ParagraphProperties);
            doc.StyleRepository.Add(parStyle);
            par.StyleId = parStyle.Id;
            par.Spacing.SpacingAfter = 100;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void ParagraphStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            paragraph.StyleId = "ParagraphStyle";

            Style paragraphStyle = new Style("ParagraphStyle", StyleType.Paragraph);
            paragraphStyle.BasedOnStyleId = "Normal";
            paragraphStyle.IsCustom = true;
            paragraphStyle.IsPrimary = true;
            paragraphStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraphStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphStyle.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(paragraphStyle, imported.StyleRepository.GetStyle(paragraphStyle.Id));
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Paragraph>(original, imported);
        }

        [TestMethod]
        public void ParagraphStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Paragraph paragraph = original.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            paragraph.StyleId = "ParagraphStyle";

            Style paragraphStyle = new Style("ParagraphStyle", StyleType.Paragraph);
            paragraphStyle.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            paragraphStyle.IsCustom = false;
            paragraphStyle.IsPrimary = true;
            paragraphStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraphStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;
            paragraphStyle.NextStyleId = paragraphStyle.Id;
            paragraphStyle.UIPriority = 0;
            original.StyleRepository.Add(paragraphStyle);

            paragraph.Properties.ParagraphMarkerProperties.CopyPropertiesFrom(paragraphStyle.CharacterProperties);
            DocumentHelper.SetRtfDefaultDocumentParagraphMarkerProperties(paragraph);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}ParagraphStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(paragraphStyle, imported.StyleRepository.GetStyle(paragraphStyle.Id));
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Paragraph>(original, imported);
        }

        [TestMethod]
        public void NormalWeb_ImportAsBuiltInStyle_Docx()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Style expectedNormalWeb = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.NormalWebStyleId);

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}NormalWeb.docx", TestFilesFolder), new DocxFormatProvider());
            Style actualNormalWeb = imported.StyleRepository.GetStyle(BuiltInStyleNames.NormalWebStyleId);

            // Assert
            Assert.IsNotNull(actualNormalWeb);
            StylePropertyAsserter.AssertStylesAreEqual(expectedNormalWeb, actualNormalWeb);
        }

        [TestMethod]
        public void NormalWeb_ImportAsBuiltInStyle_Rtf()
        {
            // Arrange
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph paragraph = expected.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "text";
            paragraph.StyleId = BuiltInStyleNames.NormalWebStyleId;

            Style expectedNormalWeb = expected.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.NormalWebStyleId);
            expectedNormalWeb.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily("Comic Sans MS");
            expectedNormalWeb.UIPriority = int.MaxValue;
            expectedNormalWeb.NextStyleId = expectedNormalWeb.Id;
            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}NormalWeb.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            Style actualNormalWeb = imported.StyleRepository.GetStyle(BuiltInStyleNames.NormalWebStyleId);

            Assert.IsNotNull(actualNormalWeb);
            StylePropertyAsserter.AssertStylesAreEqual(expectedNormalWeb, actualNormalWeb);
        }
    }
}
