﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class StyleCommonPropertiesTests : StyleTestBase
    {
        [TestMethod]
        public void IsDefault_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsDefault = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsDefault = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void IsDefault_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsDefault = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsDefault = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void IsCustom_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsCustom = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsCustom = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void IsCustom_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsCustom = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsCustom = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void IsPrimary_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsPrimary = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsPrimary = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void IsPrimary_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsPrimary = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsPrimary = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void IsPrimary_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.IsPrimary = true;

            Style characterStyle2 = new Style("CharacterStyle2", StyleType.Character);
            characterStyle2.IsPrimary = false;

            doc.StyleRepository.Add(characterStyle);
            doc.StyleRepository.Add(characterStyle2);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void StyleName_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.Name = "CharacterStyleName";

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void StyleName_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.Name = "CharacterStyleName";

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void StyleName_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.Name = "CharacterStyleName";

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void UIPriority_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.UIPriority = 43;

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void UIPriority_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.UIPriority = 43;

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void UIPriority_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style characterStyle = new Style("CharacterStyle", StyleType.Character);
            characterStyle.UIPriority = 43;

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }
    }
}
