﻿using System.IO;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    public class StyleTestBase
    {
        public const string TestFilesFolder = "Styles/";

        protected void StyleExportImportTemplate(
          RadFlowDocument original,
          IFormatProvider<RadFlowDocument> provider)
        {
            // Act
            RadFlowDocument exported = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(original, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string rtfString = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                exported = provider.Import(stream);
            }

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(original, exported);
        }

        protected void StyleAndElementsExportImportTemplate(
            RadFlowDocument original,
            IFormatProvider<RadFlowDocument> provider)
        {
            // Act
            RadFlowDocument exported = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(original, stream);
                stream.Seek(0, SeekOrigin.Begin);

                string rtfString = new StreamReader(stream).ReadToEnd();
                stream.Seek(0, SeekOrigin.Begin);

                exported = provider.Import(stream);
            }

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(original, exported);
            this.AssertElementsProperties(original, exported);
        }

        protected void AssertElementsProperties(RadFlowDocument original, RadFlowDocument exported)
        {
            StructureAsserter.AssertDocumentsAreEqual(original, exported);
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Run>(original, exported);
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Paragraph>(original, exported);
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Table>(original, exported);
        }

        protected void FillPropertiesWithValues(Documents.Flow.Model.Styles.CharacterProperties props)
        {
            props.BackgroundColor.LocalValue = new ThemableColor(Colors.Red);
            props.BaselineAlignment.LocalValue = Telerik.Windows.Documents.Flow.Model.Styles.BaselineAlignment.Subscript;
            props.FontFamily.LocalValue = new ThemableFontFamily("Courier New");
            props.FontSize.LocalValue = 36;
            props.FontStyle.LocalValue = FontStyles.Italic;
            props.FontWeight.LocalValue = FontWeights.Bold;
            props.ForegroundColor.LocalValue = new ThemableColor(Colors.Blue);
            props.HighlightColor.LocalValue = Colors.Green;
            props.ShadingPattern.LocalValue = ShadingPattern.Percent15;
            props.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Yellow);
            props.Strikethrough.LocalValue = true;
            props.UnderlineColor.LocalValue = new ThemableColor(Colors.Gray);
            props.UnderlinePattern.LocalValue = UnderlinePattern.DotDash;
        }

        protected void FillPropertiesWithValues(Documents.Flow.Model.Styles.ParagraphProperties props)
        {
            props.BackgroundColor.LocalValue = new ThemableColor(Colors.Red);
            props.ShadingPattern.LocalValue = ShadingPattern.Percent15;
            props.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Yellow);
            props.FirstLineIndent.LocalValue = 12;
            props.LeftIndent.LocalValue = 18;
            props.RightIndent.LocalValue = 24;
            props.SpacingAfter.LocalValue = 30;
            props.SpacingBefore.LocalValue = 36;
            props.TextAlignment.LocalValue = Alignment.Center;

            props.Borders.LocalValue = new ParagraphBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)),
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)),
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)),
                new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)),
                new Border(5, BorderStyle.Wave, new ThemableColor(Colors.Cyan)));
        }

        protected void FillPropertiesWithValues(Documents.Flow.Model.Styles.TableProperties tableProperties)
        {
            tableProperties.Alignment.LocalValue = Alignment.Right;
            tableProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.Red);
            tableProperties.ShadingPattern.LocalValue = ShadingPattern.Percent15;
            tableProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Yellow);
            tableProperties.TableCellPadding.LocalValue = new Primitives.Padding(3, 4, 5, 6);
            tableProperties.TableCellSpacing.LocalValue = 10;
            tableProperties.Borders.LocalValue = new TableBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)),
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)),
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)),
                new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)),
                new Border(5, BorderStyle.Wave, new ThemableColor(Colors.Cyan)),
                new Border(6, BorderStyle.DashDotStroked, new ThemableColor(Colors.Aquamarine)));

            tableProperties.ColumnBanding.LocalValue = 2;
            tableProperties.RowBanding.LocalValue = 3;
        }

        protected void FillPropertiesWithValues(Documents.Flow.Model.Styles.TableCellProperties tableCellProperties)
        {
            tableCellProperties.VerticalAlignment.LocalValue = Flow.Model.Styles.VerticalAlignment.Center;
            tableCellProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.BlanchedAlmond);
            tableCellProperties.ShadingPattern.LocalValue = ShadingPattern.Percent12;
            tableCellProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.BlueViolet);
            tableCellProperties.Padding.LocalValue = new Primitives.Padding(9, 8, 7, 6);
            tableCellProperties.Borders.LocalValue = new TableCellBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)),
                new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)),
                new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)),
                new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)),
                null,
                null,
                new Border(5, BorderStyle.Wave, new ThemableColor(Colors.Cyan)),
                new Border(6, BorderStyle.DashDotStroked, new ThemableColor(Colors.Aquamarine)));
        }
    }
}
