﻿using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using FontWeights = System.Windows.FontWeights;
using FontStyles = System.Windows.FontStyles;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class TableStyleTests : StyleTestBase
    {
        [TestMethod]
        public void TableStyle_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);
            tableStyle.UIPriority = 66;
            tableStyle.Name = "Test Table Style";

            doc.StyleRepository.Add(tableStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableStyle_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);
            tableStyle.UIPriority = 66;
            tableStyle.Name = "Test Table Style";

            doc.StyleRepository.Add(tableStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }
        
        [TestMethod]
        public void TableStyle_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);
            tableStyle.UIPriority = 66;
            tableStyle.Name = "Test Table Style";

            doc.StyleRepository.Add(tableStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void TableStyleWithProperties_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);

            this.FillPropertiesWithValues(tableStyle.CharacterProperties);
            this.FillPropertiesWithValues(tableStyle.ParagraphProperties);
            this.FillPropertiesWithValues(tableStyle.TableProperties);
            this.FillPropertiesWithValues(tableStyle.TableCellProperties);

            doc.StyleRepository.Add(tableStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableStyleWithProperties_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);

            this.FillPropertiesWithValues(tableStyle.CharacterProperties);
            this.FillPropertiesWithValues(tableStyle.ParagraphProperties);
            this.FillPropertiesWithValues(tableStyle.TableProperties);
            this.FillPropertiesWithValues(tableStyle.TableCellProperties);

            doc.StyleRepository.Add(tableStyle);

            // Act and Assert
            this.StyleExportImportTemplate(doc, new DocxFormatProvider());
        }
        
        [TestMethod]
        public void TableStyleWithProperties_Validate_Xml()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();
            run.Text = "test";

            Style tableStyle = new Style("TestTableStyle", StyleType.Table);

            this.FillPropertiesWithValues(tableStyle.CharacterProperties);
            this.FillPropertiesWithValues(tableStyle.ParagraphProperties);
            this.FillPropertiesWithValues(tableStyle.TableProperties);
            this.FillPropertiesWithValues(tableStyle.TableCellProperties);

            doc.StyleRepository.Add(tableStyle);

            XmlValidator.ValidateOpenXmlDocument(doc);
        }

        [TestMethod]
        public void TableStyle_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.StyleId = "TableStyle";
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            
            var row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            
            section.Blocks.AddParagraph();

            Style tableStyle = new Style("TableStyle", StyleType.Table);
            tableStyle.BasedOnStyleId = "TableNormal";
            tableStyle.IsCustom = true;
            tableStyle.UIPriority = 99;
            tableStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            tableStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            tableStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;
            tableStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            tableStyle.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Auto;
            tableStyle.ParagraphProperties.SpacingAfter.LocalValue = 0;
            tableStyle.TableProperties.Borders.LocalValue = new TableBorders(new Border(0.66666666666666663, BorderStyle.Single, new ThemableColor(Colors.Black, true)));
            tableStyle.TableProperties.TableCellPadding.LocalValue = new Primitives.Padding(7.2, 0, 7.2, 0);
            tableStyle.TableProperties.Indent.LocalValue = 0;
            tableStyle.TableCellProperties.ShadingPattern.LocalValue = ShadingPattern.Clear;
            tableStyle.TableCellProperties.VerticalAlignment.LocalValue = VerticalAlignment.Center;

            tableStyle.TableCellProperties.BackgroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent6);
            tableStyle.TableCellProperties.ShadingPatternColor.LocalValue = new ThemableColor(Colors.Transparent, true);

            original.StyleRepository.Add(tableStyle);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableStyle.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(tableStyle, imported.StyleRepository.GetStyle(tableStyle.Id));

            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Table>(original, imported);
        }

        [TestMethod]
        public void TableStyle_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.StyleId = "TableStyle";
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            
            var row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();
            
            section.Blocks.AddParagraph();

            Style tableStyle = new Style("TableStyle", StyleType.Table);
            tableStyle.BasedOnStyleId = "NormalTable";
            tableStyle.IsCustom = false;
            tableStyle.NextStyleId = tableStyle.Id;
            
            tableStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            tableStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            
            tableStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;
            tableStyle.ParagraphProperties.LineSpacing.LocalValue = 1;
            tableStyle.ParagraphProperties.SpacingAfter.LocalValue = 0;
            
            tableStyle.TableCellProperties.VerticalAlignment.LocalValue = VerticalAlignment.Center;
            tableStyle.TableCellProperties.BackgroundColor.LocalValue = new ThemableColor(original.Theme.ColorScheme[ThemeColorType.Accent6].Color);
            
            var border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black));
            tableStyle.TableProperties.Borders.LocalValue = new TableBorders(border);

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableStyle.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(tableStyle, imported.StyleRepository.GetStyle(tableStyle.Id));

            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Table>(original, imported);
        }
    }
}
