using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Styles
{
    [TestClass]
    public class UnexistingStyleIdExportTests : StyleTestBase
    {
        [TestMethod]
        public void Unexisting_RunStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingRunStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_RunStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingRunStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_RunStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingRunStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentWithUnexisitingRunStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Run run = doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");
            run.StyleId = "FakeStyle";
            return doc;
        }

        [TestMethod]
        public void Unexisting_ParagraphStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingParagraphStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_ParagraphStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingParagraphStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_ParagraphStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingParagraphStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentWithUnexisitingParagraphStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Paragraph paragraph = doc.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("test");

            paragraph.StyleId = "FakeStyle";
            return doc;
        }

        [TestMethod]
        public void Unexisting_TableStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingTableStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_TableStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingTableStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_TableStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingTableStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentWithUnexisitingTableStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph().Inlines.AddRun("test");
            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("test");
            section.Blocks.AddParagraph().Inlines.AddRun("test");

            table.StyleId = "FakeStyle";
            return doc;
        }

        [TestMethod]
        public void Unexisting_BasedOnStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentUnexisitingWithBasedOnStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_BasedOnStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentUnexisitingWithBasedOnStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_BasedOnStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentUnexisitingWithBasedOnStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentUnexisitingWithBasedOnStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            Style style = new Style("TestStyle", StyleType.Character);
            style.BasedOnStyleId = "FakeStyle";

            doc.StyleRepository.Add(style);
            return doc;
        }

        [TestMethod]
        public void Unexisting_LinkedStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingLinkedStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_LinkedStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingLinkedStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_LinkedStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingLinkedStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentWithUnexisitingLinkedStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            Style style = new Style("TestStyle", StyleType.Paragraph);
            style.LinkedStyleId = "FakeStyle";

            doc.StyleRepository.Add(style);
            return doc;
        }

        [TestMethod]
        public void Unexisting_NextStyleId_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingNextStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Unexisting_NextStyleId_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingNextStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Unexisting_NextStyleId_ExportImport_Html()
        {
            // Arrange
            RadFlowDocument doc = this.GetDocumentWithUnexisitingNextStyleId();

            // Act and Assert
            this.AssertExportWithNoException(doc, new HtmlFormatProvider());
        }

        private RadFlowDocument GetDocumentWithUnexisitingNextStyleId()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test");

            Style style = new Style("TestStyle", StyleType.Paragraph);
            style.NextStyleId = "FakeStyle";

            doc.StyleRepository.Add(style);
            return doc;
        }

        private void AssertExportWithNoException(RadFlowDocument original, IFormatProvider<RadFlowDocument> provider)
        {
            // Act
            using (Stream stream = new MemoryStream())
            {
                provider.Export(original, stream);
            }
        }
    }
}