using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class CanWrapContentTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void CanWrapContent_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.CanWrapContent = false;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void CanWrapContent_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.CanWrapContent = false;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void CanWrapContent_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;
            cell.Blocks.AddParagraph();

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;
            cell.Blocks.AddParagraph();

            row = table.Rows.AddTableRow();

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;
            cell.Blocks.AddParagraph();

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;
            cell.Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CanWrapText.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void CanWrapContent_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            row = table.Rows.AddTableRow();

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CanWrapText.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();

            Section section = document.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            TableRow row = table.Rows.AddTableRow();

            TableCell cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            row = table.Rows.AddTableRow();

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            cell = row.Cells.AddTableCell();
            cell.CanWrapContent = false;

            section.Blocks.AddParagraph();

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}