using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class IgnoreCellMarkerInRowHeightCalculationTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = true;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = true;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = false;
            cell.Blocks.AddParagraph();

            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = true;
            cell.Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IgnoreCellMarkerInRowHeightCalculation.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = false;

            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = true;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}IgnoreCellMarkerInRowHeightCalculation.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void IgnoreCellMarkerInRowHeightCalculation_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();

            Section section = document.Sections.AddSection();

            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = false;

            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.IgnoreCellMarkerInRowHeightCalculation = true;

            section.Blocks.AddParagraph();

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}