#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class TableCellBordersTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void TableCellBorders_NoCellSapcing_ExportImport_Rtf()
        {
            // Create new table for each border, as in RTF calculated borders are exported and if 
            // all cells are in one table they interfere with each other

            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            TableCell tableCell = this.AddTableAndParagraph(section);
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            TableCell tableCell2 = this.AddTableAndParagraph(section);
            tableCell2.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            TableCell table3 = this.AddTableAndParagraph(section);
            table3.Borders = new TableCellBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            TableCell tableCell4 = this.AddTableAndParagraph(section);
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            // There are no vertical and horizontal borders in RTF

            TableCell tableCell7 = this.AddTableAndParagraph(section);
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(Colors.DarkGreen)));

            TableCell tableCell8 = this.AddTableAndParagraph(section);
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(Colors.Gray)));

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellBorders_WithCellSapcing_ExportImport_Rtf()
        {
            // Create new table for each border, as in RTF calculated borders are exported and if 
            // all cells are in one table they interfere with each other

            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            TableCell tableCell = this.AddTableAndParagraph(section, true);
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            TableCell tableCell2 = this.AddTableAndParagraph(section, true);
            tableCell2.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            TableCell table3 = this.AddTableAndParagraph(section, true);
            table3.Borders = new TableCellBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            TableCell tableCell4 = this.AddTableAndParagraph(section, true);
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            // There are no vertical and horizontal borders in RTF

            TableCell tableCell7 = this.AddTableAndParagraph(section, true);
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(Colors.DarkGreen)));

            TableCell tableCell8 = this.AddTableAndParagraph(section, true);
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(Colors.Gray)));

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellBorders_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            TableCell tableCell2 = row.Cells.AddTableCell();
            tableCell2.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            TableCell table3 = row.Cells.AddTableCell();
            table3.Borders = new TableCellBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            TableCell tableCell4 = row.Cells.AddTableCell();
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            TableCell tableCell5 = row.Cells.AddTableCell();
            tableCell5.Borders = new TableCellBorders(tableCell5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(Colors.Cyan)));

            TableCell tableCell6 = row.Cells.AddTableCell();
            tableCell6.Borders = new TableCellBorders(tableCell6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(Colors.DarkCyan)));

            TableCell tableCell7 = row.Cells.AddTableCell();
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(Colors.DarkGreen)));

            TableCell tableCell8 = row.Cells.AddTableCell();
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(Colors.Gray)));

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellBorders_LocalColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            TableCell tableCell2 = row.Cells.AddTableCell();
            tableCell2.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            TableCell table3 = row.Cells.AddTableCell();
            table3.Borders = new TableCellBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            TableCell tableCell4 = row.Cells.AddTableCell();
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            TableCell tableCell5 = row.Cells.AddTableCell();
            tableCell5.Borders = new TableCellBorders(tableCell5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(Colors.Cyan)));

            TableCell tableCell6 = row.Cells.AddTableCell();
            tableCell6.Borders = new TableCellBorders(tableCell6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(Colors.DarkCyan)));

            TableCell tableCell7 = row.Cells.AddTableCell();
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(Colors.DarkGreen)));

            TableCell tableCell8 = row.Cells.AddTableCell();
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(Colors.Gray)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellBorders_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellBorders_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellBorders_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(ThemeColorType.Accent1)));

            TableCell tableCell2 = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent2)));

            TableCell tableCell3 = row.Cells.AddTableCell();
            tableCell3.Borders = new TableCellBorders(tableCell3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent3)));

            TableCell tableCell4 = row.Cells.AddTableCell();
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent4)));

            TableCell tableCell5 = row.Cells.AddTableCell();
            tableCell5.Borders = new TableCellBorders(tableCell5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(ThemeColorType.Accent5)));

            TableCell tableCell6 = row.Cells.AddTableCell();
            tableCell6.Borders = new TableCellBorders(tableCell6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(ThemeColorType.Accent6)));

            TableCell tableCell7 = row.Cells.AddTableCell();
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(ThemeColorType.Background1)));

            TableCell tableCell8 = row.Cells.AddTableCell();
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(ThemeColorType.Background2)));

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellBorders_ThemeColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            TableCell tableCell = row.Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(tableCell.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(ThemeColorType.Accent1)));

            TableCell tableCell2 = row.Cells.AddTableCell();
            tableCell2.Borders = new TableCellBorders(tableCell2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent2)));

            TableCell tableCell3 = row.Cells.AddTableCell();
            tableCell3.Borders = new TableCellBorders(tableCell3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent3)));

            TableCell tableCell4 = row.Cells.AddTableCell();
            tableCell4.Borders = new TableCellBorders(tableCell4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent4)));

            TableCell tableCell5 = row.Cells.AddTableCell();
            tableCell5.Borders = new TableCellBorders(tableCell5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(ThemeColorType.Accent5)));

            TableCell tableCell6 = row.Cells.AddTableCell();
            tableCell6.Borders = new TableCellBorders(tableCell6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(ThemeColorType.Accent6)));

            TableCell tableCell7 = row.Cells.AddTableCell();
            tableCell7.Borders = new TableCellBorders(tableCell7.Borders, diagonalDownBorder: new Border(7, BorderStyle.Wave, new ThemableColor(ThemeColorType.Background1)));

            TableCell tableCell8 = row.Cells.AddTableCell();
            tableCell8.Borders = new TableCellBorders(tableCell8.Borders, diagonalUpBorder: new Border(8, BorderStyle.Wave, new ThemableColor(ThemeColorType.Background2)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellBorders_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            TableCellBorders borders = new TableCellBorders(border, border, border, border, null, null, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();

            cell.Borders = borders;

            section.Blocks.AddParagraph();


            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellBordersLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellBorders_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent6));
            TableCellBorders borders = new TableCellBorders(border, border, border, border, null, null, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();

            cell.Borders = borders;

            section.Blocks.AddParagraph();


            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellBordersThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellBorders_LocalColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            TableCellBorders borders = new TableCellBorders(border, border, border, border, null, null, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Borders = borders;

            section.Blocks.AddParagraph();

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellBordersLocalColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellBorders_AutomaticColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Black));
            TableCellBorders borders = new TableCellBorders(border, border, border, border, null, null, border, border);

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Borders = borders;

            section.Blocks.AddParagraph();

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellBordersAutomaticColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        private TableCell AddTableAndParagraph(Section section, bool addCellSpacing = false)
        {
            Table table = section.Blocks.AddTable();
            if (addCellSpacing)
            {
                table.TableCellSpacing = 10;
            }

            TableCell cell = table
                .Rows.AddTableRow()
                .Cells.AddTableCell();

            section.Blocks.AddParagraph();

            return cell;
        }
    }
}