using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;
using Telerik.Windows.Documents.Primitives;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class TableCellPaddingTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void TableCellPadding_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.Padding = new Padding(10, 20, 30, 40);

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellPadding_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.Padding = new Padding(10, 20, 30, 40);

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellPadding_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();

            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Padding = new Padding(30);

            cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Padding = new Padding(30);

            row = table.Rows.AddTableRow();

            cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Padding = new Padding(30);

            cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Padding = new Padding(30);

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Padding.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellPadding_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();

            TableCell cell = row.Cells.AddTableCell();
            cell.Padding = new Padding(30);

            cell = row.Cells.AddTableCell();
            cell.Padding = new Padding(30);

            row = table.Rows.AddTableRow();

            cell = row.Cells.AddTableCell();
            cell.Padding = new Padding(30);

            cell = row.Cells.AddTableCell();
            cell.Padding = new Padding(30);

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Padding.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellPadding_Validate_Xml()
        {
            TableCell cell = this.CreateDocumentWithCell();
            cell.Padding = new Padding(10, 20, 30, 40);

            RadFlowDocument document = cell.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}