using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class TableCellPreferredWidthTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void TableCellPreferredWidth_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // TableWidthUnitType.Auto is the default value and will be cleared during the RtfImport
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 200);
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellPreferredWidth_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // TableWidthUnitType.Auto is the default value and will be cleared during the RtfImport
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 200);
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(doc.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellPreferredWidth_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            table = section.Blocks.AddTable();
            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);

            section.Blocks.AddParagraph();

            table = section.Blocks.AddTable();
            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 30);

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PreferredWidth.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellPreferredWidth_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();

            section.Blocks.AddParagraph();

            table = section.Blocks.AddTable();
            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);

            section.Blocks.AddParagraph();

            table = section.Blocks.AddTable();
            cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 30);

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PreferredWidth.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellPreferredWidth_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // TableWidthUnitType.Auto is the default value and will be cleared during the RtfImport
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 200);
            row.Cells.AddTableCell().PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}