#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class TableCellShadingTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void TableCellShading_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            cell.Shading.PatternColor = new ThemableColor(Colors.Blue);
            cell.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellShading_LocalColor_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            cell.Shading.PatternColor = new ThemableColor(Colors.Blue);
            cell.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellShading_LocalColor_Validate_Xml()
        {
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            cell.Shading.PatternColor = new ThemableColor(Colors.Blue);
            cell.Shading.Pattern = ShadingPattern.Percent40;

            RadFlowDocument document = cell.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellShading_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.PatternColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.Pattern = ShadingPattern.Clear;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellShading_AutoLocalColor_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.PatternColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.Pattern = ShadingPattern.Clear;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellShading_AutoLocalColor_Validate_Xml()
        {
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.PatternColor = new ThemableColor(Colors.Transparent, true);
            cell.Shading.Pattern = ShadingPattern.Clear;

            RadFlowDocument document = cell.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellShading_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            cell.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            cell.Shading.Pattern = ShadingPattern.DiagonalCross;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(cell.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellShading_ThemeColor_Validate_Xml()
        {
            TableCell cell = this.CreateDocumentWithCell();

            cell.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            cell.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            cell.Shading.Pattern = ShadingPattern.DiagonalCross;

            RadFlowDocument document = cell.Document;
            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableCellShading_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            cell.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            cell.Shading.Pattern = ShadingPattern.Percent40;


            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellShadingLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellShading_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            cell.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Accent6);
            cell.Shading.PatternColor = new ThemableColor(ThemeColorType.Accent5, 0.8);
            cell.Shading.Pattern = ShadingPattern.Percent25;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellShadingThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void TableCellShading_LocalColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            cell.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            cell.Shading.Pattern = ShadingPattern.Percent40;


            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellShadingLocalColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        // Uncomment the test when full support of automatic (omitted values in color table) colors is introduced. #166212
        //[TestMethod]
        public void TableCellShading_AutomaticColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell cell = table.Rows.AddTableRow().Cells.AddTableCell();
            cell.Shading.BackgroundColor = new ThemableColor(Colors.Transparent);
            cell.Shading.PatternColor = new ThemableColor(Colors.Black);
            cell.Shading.Pattern = ShadingPattern.Percent40;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableCellShadingAutomaticColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }
    }
}