﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class TextDirectionTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void TextDirection_WhenIsBottomToTopLeftToRight_Import_Docx()
        {
            // Arrange
            TableCell cell = this.GetTableCell();

            cell.TextDirection = Flow.Model.Styles.TextDirection.BottomToTopLeftToRight;

            // Act
            RadFlowDocument imported = this.ImportDocumentForAssert("TextDirectionBottomTopLeftRight.docx", new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsBottomToTopLeftToRight_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.BottomToTopLeftToRight;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsBottomToTopLeftToRight_Import_Rtf()
        {
            // Arrange
            TableCell cell = this.GetTableCell(false);

            cell.TextDirection = Flow.Model.Styles.TextDirection.BottomToTopLeftToRight;

            // Act
            RadFlowDocument imported = this.ImportDocumentForAssert("TextDirectionBottomTopLeftRight.rtf", new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsBottomToTopLeftToRight_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.BottomToTopLeftToRight;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsLeftToRightTopToBottomRotated_Import_Docx()
        {
            // Arrange
            TableCell cell = this.GetTableCellWithContent();
            cell.TextDirection = TextDirection.LeftToRightTopToBottomRotated;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextDirectionLeftToRightTopToBottomRotated.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsLeftToRightTopToBottomRotated_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.LeftToRightTopToBottomRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsLeftToRightTopToBottomRotated_Import_Rtf()
        {
            // Arrange
            TableCell cell = this.GetTableCellWithContent();
            cell.TextDirection = TextDirection.LeftToRightTopToBottomRotated;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextDirectionLeftToRightTopToBottomRotated.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsLeftToRightTopToBottomRotated_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.LeftToRightTopToBottomRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomLeftToRightRotated_Import_Docx()
        {
            // Arrange
            TableCell cell = this.GetTableCellWithContent();
            cell.TextDirection = TextDirection.TopToBottomLeftToRightRotated;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextDirectionTopToBottomLeftToRightRotated.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomLeftToRightRotated_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomLeftToRightRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomLeftToRightRotated_IsImportedAsLeftToRightTopToBottom_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomLeftToRightRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new RtfFormatProvider());

            // Assert
            TableCell importedCell = imported.EnumerateChildrenOfType<TableCell>().FirstOrDefault();
            Assert.AreEqual(TextDirection.LeftToRightTopToBottom, importedCell.TextDirection);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeft_Import_Docx()
        {
            // Arrange
            TableCell cell = this.GetTableCell();

            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeft;

            // Act
            RadFlowDocument imported = this.ImportDocumentForAssert("TextDirectionTopBottomRightLeft.docx", new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeft_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeft;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeft_Import_Rtf()
        {
            // Arrange
            TableCell cell = this.GetTableCell(false);

            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeft;

            // Act
            RadFlowDocument imported = this.ImportDocumentForAssert("TextDirectionTopBottomRightLeft.rtf", new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeft_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeft;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeftRotated_Import_Docx()
        {
            // Arrange
            TableCell cell = this.GetTableCellWithContent();
            cell.TextDirection = TextDirection.TopToBottomRightToLeftRotated;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextDirectionTopToBottomRightToLeftRotated.docx", TestFilesFolder), new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeftRotated_ExportImport_Docx()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeftRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new DocxFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeftRotated_Import_Rtf()
        {
            // Arrange
            TableCell cell = this.GetTableCellWithContent();
            cell.TextDirection = TextDirection.TopToBottomRightToLeftRotated;

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TextDirectionTopToBottomRightToLeftRotated.rtf", TestFilesFolder), new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        [TestMethod]
        public void TextDirection_WhenIsTopToBottomRightToLeftRotated_ExportImport_Rtf()
        {
            // Arrange
            TableCell cell = this.CreateDocumentWithCell();
            cell.TextDirection = Flow.Model.Styles.TextDirection.TopToBottomRightToLeftRotated;

            // Act
            RadFlowDocument imported = this.ExportImportDocument(cell.Document, new RtfFormatProvider());

            // Assert
            this.AssertTableCellProperties(cell.Document, imported);
        }

        private TableCell GetTableCell(bool addParagraphInCell = true)
        {
            TableCell cell = this.CreateDocumentWithCell(false);
            cell.PreferredWidth = new Flow.Model.Styles.TableWidthUnit(Unit.InchToDip(5));
            if (addParagraphInCell)
            {
                cell.Blocks.AddParagraph();
            }

            return cell;
        }

        private RadFlowDocument ImportDocumentForAssert(string documentName, IFormatProvider<RadFlowDocument> formatProvider)
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(string.Format("{0}{1}", TestFilesFolder, documentName), formatProvider);
            TableCell importedCell = document.EnumerateChildrenOfType<TableCell>().FirstOrDefault();
            importedCell.Borders = new Flow.Model.Styles.TableCellBorders();

            return document;
        }

        private TableCell GetTableCellWithContent()
        {
            TableCell cell = this.CreateDocumentWithCell(false);
            // We remove the lastly added paragraph from the base class' method.
            cell.Document.Sections.First().Blocks.Remove(cell.Document.EnumerateChildrenOfType<Paragraph>().LastOrDefault());
            cell.Blocks.AddParagraph().Inlines.AddRun("test");
            return cell;
        }
    }
}