using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableCellProperties
{
    [TestClass]
    public class VerticalAlignmentTests : TableCellPropertiesTestBase
    {
        [TestMethod]
        public void VerticalAlignment_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // VerticalAlignment.Top is the default value and will be cleared during the RtfImport
            // VerticalAlignment.Justified is not supported in Rtf
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Center;
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Bottom;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(row.Document, new RtfFormatProvider());
        }

        [TestMethod]
        public void VerticalAlignment_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // VerticalAlignment.Top is the default value
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Center;
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Bottom;
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Justified;

            // Act and Assert
            this.TableCellPropertiesExportImportTemplate(row.Document, new DocxFormatProvider());
        }

        [TestMethod]
        public void VerticalAlignment_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell tableCell = table.Rows.AddTableRow().Cells.AddTableCell();
            tableCell.Blocks.AddParagraph();

            tableCell = table.Rows.AddTableRow().Cells.AddTableCell();
            tableCell.Blocks.AddParagraph();
            tableCell.VerticalAlignment = VerticalAlignment.Center;

            tableCell = table.Rows.AddTableRow().Cells.AddTableCell();
            tableCell.Blocks.AddParagraph();
            tableCell.VerticalAlignment = VerticalAlignment.Bottom;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}VerticalAlignment.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void VerticalAlignment_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableCell tableCell = table.Rows.AddTableRow().Cells.AddTableCell();

            tableCell = table.Rows.AddTableRow().Cells.AddTableCell();
            tableCell.VerticalAlignment = VerticalAlignment.Center;

            tableCell = table.Rows.AddTableRow().Cells.AddTableCell();
            tableCell.VerticalAlignment = VerticalAlignment.Bottom;

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}VerticalAlignment.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableCellProperties(original, imported);
        }

        [TestMethod]
        public void VerticalAlignment_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            section.Blocks.AddParagraph();
            TableRow row = section.Blocks.AddTable().Rows.AddTableRow();

            // VerticalAlignment.Top is the default value
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Center;
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Bottom;
            row.Cells.AddTableCell().VerticalAlignment = VerticalAlignment.Justified;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}