﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class AlignmentTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void Alignment_ExportImport_Rtf()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            // Only Left, Center and Right alignments are supported in RTF. Left is default
            this.AddTableWithAlignment(section, Alignment.Center);
            section.Blocks.AddParagraph();
            this.AddTableWithAlignment(section, Alignment.Right);

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void Alignment_ExportImport_Docx()
        {
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            this.AddTableWithAlignment(section, Alignment.Center);
            this.AddTableWithAlignment(section, Alignment.Right);
            this.AddTableWithAlignment(section, Alignment.Distribute);
            this.AddTableWithAlignment(section, Alignment.Justified);
            this.AddTableWithAlignment(section, Alignment.ThaiDistribute);
            this.AddTableWithAlignment(section, Alignment.LowKashida);
            this.AddTableWithAlignment(section, Alignment.MediumKashida);
            this.AddTableWithAlignment(section, Alignment.HighKashida);

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void Alignment_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();

            table.Alignment = Alignment.Right;
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Alignment.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void Alignment_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Alignment = Alignment.Right;
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Alignment.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void Alignment_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            this.AddTableWithAlignment(section, Alignment.Center);
            this.AddTableWithAlignment(section, Alignment.Right);
            this.AddTableWithAlignment(section, Alignment.Distribute);
            this.AddTableWithAlignment(section, Alignment.Justified);
            this.AddTableWithAlignment(section, Alignment.ThaiDistribute);
            this.AddTableWithAlignment(section, Alignment.LowKashida);
            this.AddTableWithAlignment(section, Alignment.MediumKashida);
            this.AddTableWithAlignment(section, Alignment.HighKashida);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void AddTableWithAlignment(Section section, Alignment alignment)
        {
            Table table = section.Blocks.AddTable();
            table.Alignment = alignment;
            table.Rows.AddTableRow().Cells.AddTableCell();
        }
    }
}
