﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TableBordersTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TableBorders_LocalColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            Table table = this.AddTableAndParagraph(section);
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            Table table2 = this.AddTableAndParagraph(section);
            table2.Borders = new TableBorders(table2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            Table table3 = this.AddTableAndParagraph(section);
            table3.Borders = new TableBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            Table table4 = this.AddTableAndParagraph(section);
            table4.Borders = new TableBorders(table4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            Table table5 = this.AddTableAndParagraph(section);
            table5.Borders = new TableBorders(table5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(Colors.Cyan)));

            Table table6 = this.AddTableAndParagraph(section);
            table6.Borders = new TableBorders(table6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(Colors.DarkCyan)));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableBorders_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            Table table2 = this.AddTable(section);
            table2.Borders = new TableBorders(table2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            Table table3 = this.AddTable(section);
            table3.Borders = new TableBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            Table table4 = this.AddTable(section);
            table4.Borders = new TableBorders(table4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            Table table5 = this.AddTable(section);
            table5.Borders = new TableBorders(table5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(Colors.Cyan)));

            Table table6 = this.AddTable(section);
            table6.Borders = new TableBorders(table6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(Colors.DarkCyan)));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableBorders_LocalColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black)));

            Table table2 = this.AddTable(section);
            table2.Borders = new TableBorders(table2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(Colors.Blue)));

            Table table3 = this.AddTable(section);
            table3.Borders = new TableBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(Colors.Orange)));

            Table table4 = this.AddTable(section);
            table4.Borders = new TableBorders(table4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(Colors.Red)));

            Table table5 = this.AddTable(section);
            table5.Borders = new TableBorders(table5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(Colors.Cyan)));

            Table table6 = this.AddTable(section);
            table6.Borders = new TableBorders(table6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(Colors.DarkCyan)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableBorders_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableBorders_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(
                new Border(1, BorderStyle.Single, new ThemableColor(Colors.Black, true)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableBorders_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(ThemeColorType.Accent1)));

            Table table2 = this.AddTable(section);
            table2.Borders = new TableBorders(table2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent2)));

            Table table3 = this.AddTable(section);
            table3.Borders = new TableBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent3)));

            Table table4 = this.AddTable(section);
            table4.Borders = new TableBorders(table4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent4)));

            Table table5 = this.AddTable(section);
            table5.Borders = new TableBorders(table5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(ThemeColorType.Accent5)));

            Table table6 = this.AddTable(section);
            table6.Borders = new TableBorders(table6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(ThemeColorType.Accent6)));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableBorders_ThemeColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            Table table = this.AddTable(section);
            table.Borders = new TableBorders(table.Borders, leftBorder: new Border(1, BorderStyle.Single, new ThemableColor(ThemeColorType.Accent1)));

            Table table2 = this.AddTable(section);

            table2.Borders = new TableBorders(table2.Borders, topBorder: new Border(2, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent2)));

            Table table3 = this.AddTable(section);
            table3.Borders = new TableBorders(table3.Borders, rightBorder: new Border(3, BorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent3)));

            Table table4 = this.AddTable(section);
            table4.Borders = new TableBorders(table4.Borders, bottomBorder: new Border(4, BorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent4)));

            Table table5 = this.AddTable(section);
            table5.Borders = new TableBorders(table5.Borders, insideHorizontalBorder: new Border(5, BorderStyle.DoubleWave, new ThemableColor(ThemeColorType.Accent5)));

            Table table6 = this.AddTable(section);
            table6.Borders = new TableBorders(table6.Borders, insideVerticalBorder: new Border(6, BorderStyle.Wave, new ThemableColor(ThemeColorType.Accent6)));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableBorders_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(ThemeColorType.Accent6));
            TableBorders borders = new TableBorders(border, border, border, border, border, border);
            table.Borders = borders;

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableBordersThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableBorders_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            TableBorders borders = new TableBorders(border, border, border, border, border, border);
            table.Borders = borders;

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();


            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableBordersLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableBorders_LocalColor_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            Border border = new Border(1, BorderStyle.Double, new ThemableColor(Colors.Red));
            TableBorders borders = new TableBorders(border, border, border, border, border, border);
            table.Borders = borders;

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            section.Blocks.AddParagraph();


            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableBordersLocalColor.rtf", TestFilesFolder), new RtfFormatProvider());

            // Act and Assert
            this.AssertTableProperties(original, imported);
        }

        private Table AddTableAndParagraph(Section section)
        {
            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            section.Blocks.AddParagraph();
            return table;
        }

        private Table AddTable(Section section)
        {
            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            return table;
        }

        [TestMethod]
        public void TableBorder_BorderStyleNone_WithOneBorderWithStyleSingle_ImportExportImport_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            table.Borders = new TableBorders(new Border(0, BorderStyle.None, new ThemableColor(Colors.Black, true)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            cell.Borders = new TableCellBorders(cell.Borders, bottomBorder: new Border(6, BorderStyle.Single, new ThemableColor(Colors.Black)));

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            DocxFormatProvider formatProvider = new DocxFormatProvider();

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Table_BorderStyleNone_WithOneBorderSingle.docx", TestFilesFolder), formatProvider);

            // Assert
            this.AssertTableProperties(original, imported);

            // Act
            byte[] exportedDocument = formatProvider.Export(imported);
            imported = formatProvider.Import(exportedDocument);

            // Assert
            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableBorder_BorderStyleNil_WithOneBorderWithStyleSingle_ImportExportImport_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();

            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            table.Borders = new TableBorders(new Border(0, BorderStyle.None, new ThemableColor(Colors.Black, true)));

            TableRow row = table.Rows.AddTableRow();
            TableCell cell = row.Cells.AddTableCell();
            cell.Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            cell.Borders = new TableCellBorders(cell.Borders, bottomBorder: new Border(6, BorderStyle.Single, new ThemableColor(Colors.Black)));

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            DocxFormatProvider formatProvider = new DocxFormatProvider();

            // Act
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}Table_BorderStyleNil_WithOneBorderSingle.docx", TestFilesFolder), formatProvider);

            // Assert
            this.AssertTableProperties(original, imported);

            // Act
            byte[] exportedDocument = formatProvider.Export(imported);
            imported = formatProvider.Import(exportedDocument);

            // Assert
            this.AssertTableProperties(original, imported);
        }
    }
}
