﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TableIndentTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TableIndent_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Indent = 20;
            table.Rows.AddTableRow().Cells.AddTableCell();

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableIndent_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddTable().Indent = 20;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableIndent_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table1 = section.Blocks.AddTable();
            table1.Indent = 0;
            TableRow row = table1.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            Table table2 = section.Blocks.AddTable();
            table2.Indent = 30;
            row = table2.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            Table table3 = section.Blocks.AddTable();
            table3.Indent = 0;
            row = table3.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableIndent.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableIndent_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            section.Blocks.AddParagraph();

            Table table2 = section.Blocks.AddTable();
            table2.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table2.StyleId = "NormalTable";
            table2.Indent = 30;
            TableRow row = table2.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();
            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableIndent.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableIndent_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddTable().Indent = 20;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
