﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TableLooksTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TableLooks_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            table.Looks = TableLooks.FirstRow | TableLooks.LastColumn;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableLooks_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            table.Looks = TableLooks.FirstRow | TableLooks.LastColumn;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableLooks_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();
            table.StyleId = "TableGrid";

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            table.Looks =
                TableLooks.BandedColumns |
                TableLooks.BandedRows |
                TableLooks.FirstColumn |
                TableLooks.FirstRow |
                TableLooks.LastColumn |
                TableLooks.LastRow;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableLooks.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableLooks_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            section.Blocks.AddParagraph();

            table.StyleId = "TableGrid";
            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto, double.NaN);

            table.Looks =
                TableLooks.BandedColumns |
                TableLooks.BandedRows |
                TableLooks.FirstColumn |
                TableLooks.FirstRow |
                TableLooks.LastColumn |
                TableLooks.LastRow;

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableLooks.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableLooks_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            table.Looks = TableLooks.FirstRow | TableLooks.LastColumn;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
