﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TablePreferredWidthTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TablePreferredWidth_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Auto));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Fixed, 340));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Percent, 80));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TablePreferredWidth_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Section section = doc.Sections.AddSection();

            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Auto));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Fixed, 340));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Percent, 80));

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TablePreferredWidth_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table1 = section.Blocks.AddTable();
            table1.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table1.StyleId = "TableGrid";
            table1.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            Table table2 = section.Blocks.AddTable();
            table2.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);
            table2.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table2.StyleId = "TableGrid";
            table2.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            Table table3 = section.Blocks.AddTable();
            table3.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table3.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 30);
            table3.StyleId = "TableGrid";
            table3.Rows.AddTableRow().Cells.AddTableCell().Blocks.AddParagraph();
            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PreferredWidth.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TablePreferredWidth_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table1 = section.Blocks.AddTable();
            table1.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table1.StyleId = "TableGrid";
            table1.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Auto, double.NaN);
            table1.Rows.AddTableRow().Cells.AddTableCell();
            section.Blocks.AddParagraph();

            Table table2 = section.Blocks.AddTable();
            table2.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table2.StyleId = "TableGrid";
            table2.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Percent, 30);
            table2.Rows.AddTableRow().Cells.AddTableCell();
            section.Blocks.AddParagraph();

            Table table3 = section.Blocks.AddTable();
            table3.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table3.StyleId = "TableGrid";
            table3.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 30);
            table3.Rows.AddTableRow().Cells.AddTableCell();
            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}PreferredWidth.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TablePreferredWidth_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Auto));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Fixed, 340));
            section.Blocks.AddParagraph();
            this.AddTableWithPreferredWidth(section, new TableWidthUnit(TableWidthUnitType.Percent, 80));

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void AddTableWithPreferredWidth(Section section, TableWidthUnit tableWidthUnit)
        {
            Table table = section.Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            table.PreferredWidth = tableWidthUnit;
        }
    }
}
