﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TableShadingTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TableShading_LocalColor_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            table.Shading.PatternColor = new ThemableColor(Colors.Blue);
            table.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableShading_LocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(Colors.Green);
            table.Shading.PatternColor = new ThemableColor(Colors.Blue);
            table.Shading.Pattern = ShadingPattern.Percent40;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableShading_AutoLocalColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(Colors.Transparent, true);
            table.Shading.PatternColor = new ThemableColor(Colors.Transparent, true);
            table.Shading.Pattern = ShadingPattern.Clear;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableShading_AutoLocalColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(Colors.Transparent, true);
            table.Shading.PatternColor = new ThemableColor(Colors.Transparent, true);
            table.Shading.Pattern = ShadingPattern.Clear;

            XmlValidator.ValidateOpenXmlDocument(document);
        }


        [TestMethod]
        public void TableShading_ThemeColor_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            table.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            table.Shading.Pattern = ShadingPattern.DiagonalCross;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableShading_LocalColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Shading.BackgroundColor = new ThemableColor(Colors.Red);
            table.Shading.PatternColor = new ThemableColor(Colors.Yellow);
            table.Shading.Pattern = ShadingPattern.Percent25;
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun().Text = "test";
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableShadingLocalColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableShading_ThemeColor_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Accent6);
            table.Shading.PatternColor = new ThemableColor(ThemeColorType.Accent5, 0.8);
            table.Shading.Pattern = ShadingPattern.Percent25;
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.StyleId = "TableGrid";

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun().Text = "test";
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableShadingThemeColor.docx", TestFilesFolder), new DocxFormatProvider());

            // Act and Assert
            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableShading_ThemeColor_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();

            table.Shading.BackgroundColor = new ThemableColor(ThemeColorType.Background1, -0.30);
            table.Shading.PatternColor = new ThemableColor(ThemeColorType.Text2, 0.40);
            table.Shading.Pattern = ShadingPattern.DiagonalCross;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
