﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableProperties
{
    [TestClass]
    public class TableTableCellSpacingTests : TablePropertiesTestsBase
    {
        [TestMethod]
        public void TableCellSpacing_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();
            table.Rows.AddTableRow().Cells.AddTableCell();
            table.TableCellSpacing = 20;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableCellSpacing_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            doc.Sections.AddSection().Blocks.AddTable().TableCellSpacing = 20;

            // Act and Assert
            this.TablePropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableCellSpacing_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.TableCellSpacing = 30;
            table.StyleId = "TableGrid";

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell().Blocks.AddParagraph();
            row.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableTableCellSpacing.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableCellSpacing_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            Table table = section.Blocks.AddTable();
            table.Looks = TableLooks.FirstColumn | TableLooks.FirstRow | TableLooks.BandedRows;
            table.TableCellSpacing = 30;
            table.StyleId = "TableGrid";

            TableRow row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            row = table.Rows.AddTableRow();
            row.Cells.AddTableCell();
            row.Cells.AddTableCell();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableTableCellSpacing.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableProperties(original, imported);
        }

        [TestMethod]
        public void TableCellSpacing_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddTable().TableCellSpacing = 20;

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
