﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableRowProperties
{
    [TestClass]
    public class CanSplitTests : TableRowPropertiesTestsBase
    {
        [TestMethod]
        public void CanSplit_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.CanSplit = true;
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.CanSplit = false;
            row2.Cells.AddTableCell();


            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void CanSplit_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            Table table = doc.Sections.AddSection().Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.CanSplit = true;
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.CanSplit = false;
            row2.Cells.AddTableCell();

            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void CanSplit_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.CanSplit = false;
            row1.Cells.AddTableCell().Blocks.AddParagraph();
            row1.Cells.AddTableCell().Blocks.AddParagraph();
            row1.Cells.AddTableCell().Blocks.AddParagraph();

            TableRow row2 = table.Rows.AddTableRow();
            row2.CanSplit = true;
            row2.Cells.AddTableCell().Blocks.AddParagraph();
            row2.Cells.AddTableCell().Blocks.AddParagraph();
            row2.Cells.AddTableCell().Blocks.AddParagraph();

            TableRow row3 = table.Rows.AddTableRow();
            row3.CanSplit = false;
            row3.Cells.AddTableCell().Blocks.AddParagraph();
            row3.Cells.AddTableCell().Blocks.AddParagraph();
            row3.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CanSplit.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }

        [TestMethod]
        public void CanSplit_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.CanSplit = false;
            row1.Cells.AddTableCell();
            row1.Cells.AddTableCell();
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.CanSplit = true;
            row2.Cells.AddTableCell();
            row2.Cells.AddTableCell();
            row2.Cells.AddTableCell();

            TableRow row3 = table.Rows.AddTableRow();
            row3.CanSplit = false;
            row3.Cells.AddTableCell();
            row3.Cells.AddTableCell();
            row3.Cells.AddTableCell();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}CanSplit.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }

        [TestMethod]
        public void CanSplit_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.CanSplit = true;
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.CanSplit = false;
            row2.Cells.AddTableCell();

            XmlValidator.ValidateOpenXmlDocument(document);
        }
    }
}
