﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableRowProperties
{
    [TestClass]
    public class TableRowHeightTests : TableRowPropertiesTestsBase
    {
        [TestMethod]
        public void TableRowHeight_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            TableRow row = doc.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.Height = new TableRowHeight(HeightType.Exact, 30);

            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableRowHeight_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            TableRow row = doc.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.Height = new TableRowHeight(HeightType.Exact, 30);

            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableRowHeight_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.Height = new TableRowHeight(HeightType.AtLeast, 30);
            row1.Cells.AddTableCell().Blocks.AddParagraph();
            row1.Cells.AddTableCell().Blocks.AddParagraph();

            TableRow row2 = table.Rows.AddTableRow();
            row2.Height = new TableRowHeight(HeightType.Exact, 30);
            row2.Cells.AddTableCell().Blocks.AddParagraph();
            row2.Cells.AddTableCell().Blocks.AddParagraph();

            TableRow row3 = table.Rows.AddTableRow();
            row3.Height = new TableRowHeight(HeightType.Auto);
            row3.Cells.AddTableCell().Blocks.AddParagraph();
            row3.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableRowHeight.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }

        [TestMethod]
        public void TableRowHeight_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.Height = new TableRowHeight(HeightType.AtLeast, 30);
            row1.Cells.AddTableCell();
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.Height = new TableRowHeight(HeightType.Exact, 30);
            row2.Cells.AddTableCell();
            row2.Cells.AddTableCell();

            TableRow row3 = table.Rows.AddTableRow();
            row3.Height = new TableRowHeight(HeightType.Auto);
            row3.Cells.AddTableCell();
            row3.Cells.AddTableCell();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableRowHeight.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }

        [TestMethod]
        public void TableRowHeight_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            TableRow row = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.Height = new TableRowHeight(HeightType.Exact, 30);

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        [TestMethod]
        public void TableRowHeight_WithInheritedRowProperties_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();

            section.Blocks.AddParagraph();

            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.Height = new TableRowHeight(HeightType.AtLeast, 200);
            row1.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 11");
            row1.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 12");
            row1.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 13");

            TableRow row2 = table.Rows.AddTableRow();
            row2.Height = new TableRowHeight(HeightType.AtLeast, 200);
            row2.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 21");
            row2.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 22");
            row2.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 23");

            TableRow row3 = table.Rows.AddTableRow();
            row3.Height = new TableRowHeight(HeightType.AtLeast, 200);
            row3.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 31");
            row3.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 32");
            row3.Cells.AddTableCell().Blocks.AddParagraph().Inlines.AddRun("cell 33");


            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}InheritedTableRowHeight.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }
    }
}
