﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.TableRowProperties
{
    [TestClass]
    public class TableRowTableCellSpacingTests : TableRowPropertiesTestsBase
    {
        // TODO: [TestMethod]
        public void TableRowCellSpacing_ExportImport_Rtf()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            TableRow row = doc.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.TableCellSpacing = 40;

            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new RtfFormatProvider());
        }

        [TestMethod]
        public void TableRowCellSpacing_ExportImport_Docx()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            TableRow row = doc.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.TableCellSpacing = 40;

            // Act and Assert
            this.TableRowPropertiesExportImportTemplate(doc, new DocxFormatProvider());
        }

        [TestMethod]
        public void TableRowCellSpacing_Import_Docx()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.TableCellSpacing = 30;
            row1.Cells.AddTableCell().Blocks.AddParagraph();
            row1.Cells.AddTableCell().Blocks.AddParagraph();

            TableRow row2 = table.Rows.AddTableRow();
            row2.TableCellSpacing = 30;
            row2.Cells.AddTableCell().Blocks.AddParagraph();
            row2.Cells.AddTableCell().Blocks.AddParagraph();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableRowCellSpacing.docx", TestFilesFolder), new DocxFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }

        [TestMethod]
        public void TableRowCellSpacing_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            TableRow row = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();
            row.Cells.AddTableCell();

            row.TableCellSpacing = 40;

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        // TODO: [TestMethod]
        public void TableRowCellSpacing_Import_Rtf()
        {
            // Arrange
            RadFlowDocument original = new RadFlowDocument();
            Section section = original.Sections.AddSection();
            Table table = section.Blocks.AddTable();

            TableRow row1 = table.Rows.AddTableRow();
            row1.TableCellSpacing = 30;
            row1.Cells.AddTableCell();
            row1.Cells.AddTableCell();

            TableRow row2 = table.Rows.AddTableRow();
            row2.TableCellSpacing = 30;
            row2.Cells.AddTableCell();
            row2.Cells.AddTableCell();

            section.Blocks.AddParagraph();

            // Act and Assert
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(string.Format("{0}TableRowCellSpacing.rtf", TestFilesFolder), new RtfFormatProvider());

            this.AssertTableRowProperties(original, imported);
        }
    }
}
