﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Tables
{
    [TestClass]
    public class OneCellTableTests
    {
        private const string TestFileNameWithoutExtension = "Tables/OneCellTable";

        [TestMethod]
        public void OneCellTable_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertOneCellTableDocument(doc);
        }

        [TestMethod]
        public void OneCellTable_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertOneCellTableDocument(doc);
        }

        [TestMethod]
        public void OneCellTable_ExportImport_Docx()
        {
            this.OneCellTableExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void OneCellTable_ExportImport_Rtf()
        {
            this.OneCellTableExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void OneCellTable_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection()
                       .Blocks.AddTable()
                       .Rows.AddTableRow()
                       .Cells.AddTableCell()
                       .Blocks.AddParagraph()
                       .Inlines.AddRun()
                       .Text = "Hello tables!";
            document.Sections[0].Blocks.AddParagraph();

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void OneCellTableExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            originalDoc.Sections.AddSection()
                       .Blocks.AddTable()
                       .Rows.AddTableRow()
                       .Cells.AddTableCell()
                       .Blocks.AddParagraph()
                       .Inlines.AddRun()
                       .Text = "Hello tables!";
            originalDoc.Sections[0].Blocks.AddParagraph();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                ////StreamReader r = new StreamReader(stream);
                ////var docString = r.ReadToEnd();
                ////stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertOneCellTableDocument(testDocument);
        }

        private void AssertOneCellTableDocument(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Assert.AreEqual(2, doc.Sections[0].Blocks.Count);
            Table table = doc.Sections[0].Blocks[0] as Table;

            Assert.AreEqual(1, table.Rows.Count);
            Assert.AreEqual(1, table.Rows[0].Cells.Count);
            Assert.AreEqual(1, table.Rows[0].Cells[0].Blocks.Count);
            DocumentAsserter.AssertParagraphWithText(table.Rows[0].Cells[0].Blocks[0], "Hello tables!");

            DocumentAsserter.AssertEmptyParagraph(doc.Sections[0].Blocks[1]);
        }
    }
}
