﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Tables
{
    [TestClass]
    public class Table3by3Tests
    {
        private const string TestFileNameWithoutExtension = "Tables/Table3by3";

        [TestMethod]
        public void Table3by3_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertTable3X3Document(doc);
        }

        [TestMethod]
        public void Table3by3_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertTable3X3Document(doc);
        }

        [TestMethod]
        public void Table3by3_ExportImport_Rtf()
        {
            this.Table3by3ExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void Table3by3_ExportImport_Docx()
        {
            this.Table3by3ExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void Table3by3_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection()
                       .Blocks.AddTable();

            TableHelper.FillTable(table, 3, 3);
            document.Sections[0].Blocks.AddParagraph();

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void Table3by3ExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();

            TableHelper.FillTable(table, 3, 3);
            originalDoc.Sections[0].Blocks.AddParagraph();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                //StreamReader r = new StreamReader(stream);
                //var docString = r.ReadToEnd();
                //stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertTable3X3Document(testDocument);
        }

        private void AssertTable3X3Document(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Assert.AreEqual(2, doc.Sections[0].Blocks.Count);
            Table table = doc.Sections[0].Blocks[0] as Table;

            TableHelper.AssertTable(table, 3, 3);

            DocumentAsserter.AssertEmptyParagraph(doc.Sections[0].Blocks[1]);
        }
    }
}
