﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Tables
{
    [TestClass]
    public class VerticallyAndHorizontallyMergedCellsTests
    {
        private const string TestFileNameWithoutExtension = "Tables/VerticallyAndHorizontallyMergedCells";

        [TestMethod]
        public void VerticallyAndHorizontallyMergedCells_Import_Rtf()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".rtf", new RtfFormatProvider());

            //Assert
            this.AssertTableWithVerticalAndHorizontalMerge(doc);
        }

        [TestMethod]
        public void VerticallyAndHorizontallyMergedCells_ExportImport_Rtf()
        {
            this.VerticallyAndHorizontallyMergedCells_ExportImportTemplate(new RtfFormatProvider());
        }

        [TestMethod]
        public void VerticallyAndHorizontallyMergedCells_Import_Docx()
        {
            RadFlowDocument doc = TestHelper.GetDocumentFromFile(TestFileNameWithoutExtension + ".docx", new DocxFormatProvider());

            //Assert
            this.AssertTableWithVerticalAndHorizontalMerge(doc);
        }

        [TestMethod]
        public void VerticallyAndHorizontallyMergedCells_ExportImport_Docx()
        {
            this.VerticallyAndHorizontallyMergedCells_ExportImportTemplate(new DocxFormatProvider());
        }

        [TestMethod]
        public void VerticallyAndHorizontallyMergedCells_Validate_Xml()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection()
                       .Blocks.AddTable();

            TableHelper.FillTableWithVerticalAndHorizontalMerge(table);
            document.Sections[0].Blocks.AddParagraph();

            XmlValidator.ValidateOpenXmlDocument(document);
        }

        private void VerticallyAndHorizontallyMergedCells_ExportImportTemplate(IFormatProvider<RadFlowDocument> provider)
        {
            //Arrange
            RadFlowDocument originalDoc = new RadFlowDocument();
            Table table = originalDoc.Sections.AddSection()
                       .Blocks.AddTable();

            TableHelper.FillTableWithVerticalAndHorizontalMerge(table);
            originalDoc.Sections[0].Blocks.AddParagraph();

            //Act
            RadFlowDocument testDocument = null;
            using (Stream stream = new MemoryStream())
            {
                provider.Export(originalDoc, stream);
                stream.Seek(0, SeekOrigin.Begin);

                //StreamReader r = new StreamReader(stream);
                //var docString = r.ReadToEnd();
                //stream.Seek(0, SeekOrigin.Begin);

                testDocument = provider.Import(stream);
            }

            //Assert
            this.AssertTableWithVerticalAndHorizontalMerge(testDocument);
        }

        private void AssertTableWithVerticalAndHorizontalMerge(RadFlowDocument doc)
        {
            //Assert
            Assert.AreEqual(1, doc.Sections.Count);
            Assert.AreEqual(2, doc.Sections[0].Blocks.Count);
            DocumentAsserter.AssertEmptyParagraph(doc.Sections[0].Blocks[1]);

            Table table = doc.Sections[0].Blocks[0] as Table;
            Table expected = new RadFlowDocument()
                .Sections.AddSection()
                .Blocks.AddTable();
            TableHelper.FillTableWithVerticalAndHorizontalMerge(expected);

            StructureAsserter.AssertTablesAreEqual(expected, table);
        }
    }
}
