﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Watermarks
{
    [TestClass]
    public class CommonWatermarkTests : WatermarkTestsBase
    {
        [TestMethod]
        public void Watermark_WithLargeAngle_ExportImport_Docx()
        {
            RadFlowDocument originalDocument = new RadFlowDocument();
            originalDocument.Sections.AddSection();

            CommonWatermarkTests.SetImageWatermarkToHeader(originalDocument.Sections.First().Headers.Add(), 100, 50, 5000, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));

            RadFlowDocument importedDocument = this.ExportImportDocument(originalDocument, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(originalDocument, importedDocument);
        }

        [TestMethod]
        public void Watermark_WithLargeAngle_Import_Docx()
        {
            string fileName = "Watermarks/WordImageWatermarkLargeAngle.docx";

            RadFlowDocument expectedDocument = new RadFlowDocument();
            expectedDocument.Sections.AddSection().Blocks.AddParagraph();

            Header defaultHeader = expectedDocument.Sections.First().Headers.Add();
            Header evenHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.Even);
            Header firstHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.First);

            double width = 90;
            double height = 30;
            double angle = 3500;
            ImageSource imageSource = new ImageSource(TestHelper.GetImageStream("test.png"), "png");

            CommonWatermarkTests.SetImageWatermarkToHeader(defaultHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(evenHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(firstHeader, width, height, angle, imageSource);

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(fileName, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(expectedDocument, importedDocument);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Watermark)]
        public void Watermark_InAllTypesOfHeaders_ExportImport_Docx()
        {
            RadFlowDocument originalDocument = new RadFlowDocument();
            originalDocument.Sections.AddSection();

            CommonWatermarkTests.SetImageWatermarkToHeader(originalDocument.Sections.First().Headers.Add(), 50, 100, 10, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));
            CommonWatermarkTests.SetImageWatermarkToHeader(originalDocument.Sections.First().Headers.Add(HeaderFooterType.Even), 100, 50, 10, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));
            CommonWatermarkTests.SetImageWatermarkToHeader(originalDocument.Sections.First().Headers.Add(HeaderFooterType.First), 100, 50, 10, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));

            RadFlowDocument importedDocument = this.ExportImportDocument(originalDocument, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(originalDocument, importedDocument);

#if VISUAL
            AssertVisual(originalDocument, VisualTestsFormats);
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Watermark)]
        public void Watermark_MultipleWatermarks_ExportImport_Docx()
        {
            RadFlowDocument originalDocument = new RadFlowDocument();
            originalDocument.Sections.AddSection();

            Header header = originalDocument.Sections.First().Headers.Add();
            CommonWatermarkTests.SetImageWatermarkToHeader(header, 100, 50, 10, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));
            CommonWatermarkTests.SetImageWatermarkToHeader(header, 101, 51, 11, new ImageSource(TestHelper.GetImageStream("test.bmp"), "bmp"));
            CommonWatermarkTests.SetImageWatermarkToHeader(header, 102, 52, 12, new ImageSource(TestHelper.GetImageStream("test.png"), "png"));

            RadFlowDocument importedDocument = this.ExportImportDocument(originalDocument, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(originalDocument, importedDocument);

#if VISUAL
            AssertVisual(originalDocument, VisualTestsFormats);
#endif
        }

        [TestMethod]
        public void Watermark_MultipleWatermarks_Import_Docx()
        {
            string fileName = "Watermarks/WordMultipleWatermarks.docx";

            RadFlowDocument expectedDocument = new RadFlowDocument();
            expectedDocument.Sections.AddSection().Blocks.AddParagraph();

            Header defaultHeader = expectedDocument.Sections.First().Headers.Add();
            Header evenHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.Even);
            Header firstHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.First);

            double width = 90;
            double height = 30;
            double angle = 100;
            ImageSource imageSource = new ImageSource(TestHelper.GetImageStream("test.png"), "png");

            CommonWatermarkTests.SetImageWatermarkToHeader(defaultHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(defaultHeader, width, height, angle + 1, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(defaultHeader, width, height, angle + 2, imageSource);

            CommonWatermarkTests.SetImageWatermarkToHeader(evenHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(evenHeader, width, height, angle + 1, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(evenHeader, width, height, angle + 2, imageSource);

            CommonWatermarkTests.SetImageWatermarkToHeader(firstHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(firstHeader, width, height, angle + 1, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(firstHeader, width, height, angle + 2, imageSource);

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(fileName, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(expectedDocument, importedDocument);
        }

        [TestMethod]
        public void Watermark_SizeSetInInches_Import_Docx()
        {
            string fileName = "Watermarks/WordImageWatermarkInches.docx";

            RadFlowDocument expectedDocument = CreateFlowDocumentWithFootersAndSectionWithParagraph();

            Header defaultHeader = expectedDocument.Sections.First().Headers.Add();
            Header evenHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.Even);
            Header firstHeader = expectedDocument.Sections.First().Headers.Add(HeaderFooterType.First);

            double width = 1152;
            double height = 384;
            double angle = 0;
            ImageSource imageSource = new ImageSource(TestHelper.GetImageStream("test.png"), "png");

            CommonWatermarkTests.SetImageWatermarkToHeader(defaultHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(evenHeader, width, height, angle, imageSource);
            CommonWatermarkTests.SetImageWatermarkToHeader(firstHeader, width, height, angle, imageSource);

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(fileName, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(expectedDocument, importedDocument);
        }

        [TestMethod]
        public void IncorrectCreationOfWatermark_FromTextBox_Import_Docx()
        {
            string fileName = "Watermarks/IncorrectCreationOfWatermarkFromTextBox.docx";

            RadFlowDocument expectedDocument = new RadFlowDocument();
            Section section = expectedDocument.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();
            paragraph.Inlines.AddRun("hello");

            paragraph = section.Blocks.AddParagraph();

            paragraph = section.Blocks.AddParagraph();
            paragraph.Inlines.AddRun("world");

            RadFlowDocument importedDocument = TestHelper.GetDocumentFromFile(fileName, new DocxFormatProvider());

            StructureAsserter.AssertDocumentsAreEqual(expectedDocument, importedDocument);
        }

#if VISUAL

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Watermark)]
        [Description("Old test name Watermark_WhenMultipleSectionsAndOtherSectionHasNotDefinedHeader_IsGetFromLastNotNullHeader")]
        public void Watermark_WhenMultipleSectionsAndOtherSectionHasNotDefinedHeader()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            document.HasDifferentEvenOddPageHeadersFooters = true;

            Section section = document.Sections.AddSection();
            section.HasDifferentFirstPageHeaderFooter = true;
            section.Headers.Add();
            section.Headers.Add(HeaderFooterType.First);
            section.Headers.Add(HeaderFooterType.Even);

            document.Sections.AddSection();
            document.Sections.AddSection();
            document.Sections.AddSection();

            // Act
            SetTextWatermarkToHeader(section.Headers.First, 250, 100, 45, 0.5, System.Windows.Media.Colors.Blue, "First", new System.Windows.Media.FontFamily("Verdana"));
            SetTextWatermarkToHeader(section.Headers.Default, 250, 100, 10, 0.5, System.Windows.Media.Colors.Red, "Default", new System.Windows.Media.FontFamily("Arial"));
            SetTextWatermarkToHeader(section.Headers.Even, 250, 100, 25, 0.5, System.Windows.Media.Colors.Green, "Even", new System.Windows.Media.FontFamily("Verdana"));

            // Assert
            AssertVisual(document, VisualTestsFormats, Telerik.Windows.Documents.Flow.Tests.Utils.UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Watermark)]
        [Description("Old test name Watermark_WhenMultipleSectionsAndOtherSectionHasDefinedHeader_IsGetFromLastNotNullHeader")]
        // Exisiting bug TFS ID: 241697. Will fail once the issue is fixed.
        public void Watermark_WhenMultipleSectionsAndOtherSectionHasDefinedHeader()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            document.HasDifferentEvenOddPageHeadersFooters = true;

            Section section = document.Sections.AddSection();
            section.Headers.Add();
            section.Headers.Add(HeaderFooterType.Even);
            document.Sections.AddSection();

            var otherSection = document.Sections.AddSection();
            otherSection.Headers.Add();
            document.Sections.AddSection();

            // Act
            SetTextWatermarkToHeader(section.Headers.Default, 250, 100, 10, 0.5, System.Windows.Media.Colors.Red, "Default", new System.Windows.Media.FontFamily("Arial"));
            SetTextWatermarkToHeader(section.Headers.Even, 250, 100, 25, 0.5, System.Windows.Media.Colors.Green, "Even", new System.Windows.Media.FontFamily("Verdana"));
            SetTextWatermarkToHeader(otherSection.Headers.Default, 250, 100, 45, 0.5, System.Windows.Media.Colors.Gold, "Other section default", new System.Windows.Media.FontFamily("Verdana"));

            // Assert
            AssertVisual(document, VisualTestsFormats, Telerik.Windows.Documents.Flow.Tests.Utils.UIInteractionHelper.PageDown(), 3);
        }

#endif
    }
}
