﻿using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Watermarks;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Watermarks
{
    public class WatermarkTestsBase : TestsBase
    {
#if VISUAL
        protected const string VISUALTESTCATEGORY = "VisualTests";

        protected static readonly string[] VisualTestsFormats = new string[]
        {
            DOCXFILEEXTENSION,
            PDFFILEEXTENSION,
            RTFFILEEXTENSION
        };
#endif
        public static RadFlowDocument CreateFlowDocumentWithFootersAndSectionWithParagraph()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddParagraph();

            document.Sections.First().Footers.Add().Blocks.AddParagraph();
            document.Sections.First().Footers.Add(HeaderFooterType.Even).Blocks.AddParagraph();
            document.Sections.First().Footers.Add(HeaderFooterType.First).Blocks.AddParagraph();

            return document;
        }

        public static void SetTextWatermarkToHeader(Header header, double width, double height, double angle, double opacity, Color foregroundColor, string text = null, FontFamily fontFamily = null)
        {
            TextWatermarkSettings textSettings = new TextWatermarkSettings();
            textSettings.Width = width;
            textSettings.Height = height;
            textSettings.Angle = angle;
            textSettings.Opacity = opacity;
            textSettings.ForegroundColor = foregroundColor;

            if (text != null)
            {
                textSettings.Text = text;
            }

            if (fontFamily != null)
            {
                textSettings.FontFamily = fontFamily;
            }

            Watermark textWatermark = new Watermark(textSettings);
            header.Watermarks.Add(textWatermark);
        }

        public static void SetImageWatermarkToHeader(Header header, double width, double height, double angle, Telerik.Windows.Documents.Media.ImageSource imageSource)
        {
            Watermark imageWatermark = new Watermark(new ImageWatermarkSettings()
            {
                Width = width,
                Height = height,
                Angle = angle,
                ImageSource = imageSource
            });

            header.Watermarks.Add(imageWatermark);
        }
    }
}
