﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Validation
{
    [TestClass]
    public class DocxRepairTests
    {
        [TestMethod]
        public void Test_Repair_EmptyTable()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddTable();

            DocxValidator.Repair(document);

            ValidationResult result = DocxValidator.Validate(document);

            Assert.AreEqual(ValidationResultType.Success, result.ResultType);
        }

        [TestMethod]
        public void Test_Repair_EmptyTableRow()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();

            DocxValidator.Repair(document);

            ValidationResult result = DocxValidator.Validate(document);

            Assert.AreEqual(ValidationResultType.Success, result.ResultType);
        }

        [TestMethod]
        public void Test_Repair_EmptyTableCell()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();

            DocxValidator.Repair(document);

            ValidationResult result = DocxValidator.Validate(document);

            Assert.AreEqual(ValidationResultType.Success, result.ResultType);
        }

        [TestMethod]
        public void Test_Repair_EmptyNonLastSection()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection();
            document.Sections.AddSection();

            DocxValidator.Repair(document);

            ValidationResult result = DocxValidator.Validate(document);

            Assert.AreEqual(ValidationResultType.Success, result.ResultType);
        }

        [TestMethod]
        public void Test_Repair_NotSupportedImageSourceExtension()
        {
            RadFlowDocument document = new RadFlowDocument();
            ImageInline imageInline = document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            imageInline.Image.ImageSource = new Media.ImageSource(new byte[] { 1, 1, 2, 3, 4 }, "alabala");
            imageInline.Image.Width = 100;
            imageInline.Image.Height = 100;

            DocxValidator.Repair(document);

            Assert.IsNull(imageInline.Image.ImageSource);
        }

        [TestMethod]
        public void Test_Repair_ImageWithSizeEmpty()
        {
            RadFlowDocument document = new RadFlowDocument();
            ImageInline imageInline = document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            imageInline.Image.ImageSource = new Media.ImageSource(new byte[] { 1, 1, 2, 3, 4 }, "png");
            imageInline.Image.Size = Size.Empty;

            DocxValidator.Repair(document);

            Assert.AreEqual(1, imageInline.Image.Width);
            Assert.AreEqual(1, imageInline.Image.Height);
        }
    }
}
