﻿using System.Collections.Generic;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx.Validation;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;

namespace Telerik.Windows.Documents.Flow.Tests.Integration.Validation
{
    [TestClass]
    public class DocxValidtionTests
    {
        [TestMethod]
        public void Test_Validation_EmptyTable()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();

            ValidationResult result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(1, errors.Count());
            Assert.AreEqual(table, errors[0].DocumentElement);
        }

        [TestMethod]
        public void Test_Validation_EmptyTableRow()
        {
            RadFlowDocument document = new RadFlowDocument();
            TableRow tableRow = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow();

            ValidationResult result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(1, errors.Count());
            Assert.AreEqual(tableRow, errors[0].DocumentElement);
        }

        [TestMethod]
        public void Test_Validation_EmptyTableCell()
        {
            RadFlowDocument document = new RadFlowDocument();
            TableCell tableCell = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();

            ValidationResult result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(1, errors.Count());
            Assert.AreEqual(tableCell, errors[0].DocumentElement);
        }

        [TestMethod]
        public void Test_Validation_EmptyNonLastSection()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            ValidationResult result = DocxValidator.Validate(document);

            Assert.AreEqual(ValidationResultType.Success, result.ResultType);

            document.Sections.AddSection();
            result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(2, errors.Count());
            Assert.AreEqual(section, errors[0].DocumentElement);
            Assert.AreEqual(section, errors[1].DocumentElement);
        }

        [TestMethod]
        public void Test_Validation_NotSupportedImageSourceExtension()
        {
            RadFlowDocument document = new RadFlowDocument();
            ImageInline imageInline = document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            imageInline.Image.ImageSource = new Media.ImageSource(new byte[] { 1, 1, 2, 3, 4 }, "alabala");
            imageInline.Image.Width = 100;
            imageInline.Image.Height = 100;

            ValidationResult result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(1, errors.Count());
            Assert.AreEqual(imageInline, errors[0].DocumentElement);
        }

        [TestMethod]
        public void Test_Validation_ImageWithSizeEmpty()
        {
            RadFlowDocument document = new RadFlowDocument();
            ImageInline imageInline = document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddImageInline();
            imageInline.Image.ImageSource = new Media.ImageSource(new byte[] { 1, 1, 2, 3, 4 }, "png");
            imageInline.Image.Size = Size.Empty;

            ValidationResult result = DocxValidator.Validate(document);
            List<ValidationError> errors = new List<ValidationError>(result.ValidationErrors);

            Assert.AreEqual(ValidationResultType.Error, result.ResultType);
            Assert.AreEqual(1, errors.Count());
            Assert.AreEqual(imageInline, errors[0].DocumentElement);
        }
    }
}
