﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class CommentsTests
    {
        [TestMethod]
        public void Comment_WhenIsAddedAsInline_IsInsertedCorrect()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Paragraph paragraph = section.Blocks.AddParagraph();

            // Act
            Comment comment = document.Comments.AddComment();

            paragraph.Inlines.Add(comment.CommentRangeStart);
            paragraph.Inlines.Add(comment.CommentRangeEnd);

            // Assert
            Assert.AreEqual(document, comment.Document);
            Assert.AreEqual(paragraph, comment.CommentRangeStart.Paragraph);
            Assert.AreEqual(paragraph, comment.CommentRangeEnd.Paragraph);
        }

        [TestMethod]
        public void Comment_WhenIsAddedThroughEditor_DocumentCommentsCollectionIsUpdated()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.InsertComment("test");

            // Assert
            Assert.AreEqual(1, editor.Document.Comments.Count);
        }

        [TestMethod]
        public void Comment_WhenIsRemovedFromDocumentCommentsCollection_CommentStartEndsAreRemovedFromDocument()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            Comment comment = editor.InsertComment("test");

            // Act
            editor.Document.Comments.Remove(comment);

            // Assert
            Assert.AreEqual(0, editor.Document.Comments.Count);
            Assert.IsFalse(editor.Document.EnumerateChildrenOfType<CommentRangeStart>().Any());
            Assert.IsFalse(editor.Document.EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }
    }
}