﻿using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Model.Editing
{
    [TestClass]
    public class RadFlowDocumentEditorFormattingTests
    {
        [TestMethod]
        public void InsertText_WithFormatting()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.InsertText("normal ");
            editor.CharacterFormatting.FontWeight.LocalValue = FontWeights.Bold;
            editor.CharacterFormatting.StyleId = "CustomStyle";
            editor.InsertText("bold ");
            editor.CharacterFormatting.ClearLocalValues();
            editor.InsertText("clear.");

            // Assert
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.AddRun().Text = "normal ";
            Run r = p.Inlines.AddRun();
            r.Text = "bold ";
            r.StyleId = "CustomStyle";
            r.FontWeight = FontWeights.Bold;
            p.Inlines.AddRun().Text = "clear.";
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Paragraph>(expected, editor.Document);
        }

        [TestMethod]
        public void Insert_WithEmptyDoc_AppliesFormattingToTheFirstParagraph()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.ParagraphFormatting.LeftIndent.LocalValue = 20;
            editor.InsertText("test");

            // Assert
            RadFlowDocument expected = new RadFlowDocument();
            Paragraph p = expected.Sections.AddSection().Blocks.AddParagraph();
            p.Indentation.LeftIndent = 20;
            p.Inlines.AddRun("test");
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Paragraph>(expected, editor.Document);
        }

        [TestMethod]
        public void InsertParagrah_WithFormatting()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.ParagraphFormatting.LeftIndent.LocalValue = 42;
            editor.ParagraphFormatting.StyleId = "CustomStyle";
            editor.InsertParagraph();
            editor.ParagraphFormatting.ClearLocalValues();
            editor.InsertParagraph();

            // Assert
            RadFlowDocument expected = new RadFlowDocument();
            Section s = expected.Sections.AddSection();
            Paragraph p = s.Blocks.AddParagraph();
            p.StyleId = "CustomStyle";
            p.Indentation.LeftIndent = 42;
            s.Blocks.AddParagraph();
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Run>(expected, editor.Document);
        }

        [TestMethod]
        public void InsertTable_WithFormatting()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.TableFormatting.BackgroundColor.LocalValue = new ThemableColor(Colors.Aqua);
            editor.TableFormatting.StyleId = "CustomStyle";
            editor.InsertTable();
            editor.InsertParagraph();

            // Assert
            RadFlowDocument expected = new RadFlowDocument();
            Section s = expected.Sections.AddSection();
            Table t = s.Blocks.AddTable();
            t.StyleId = "CustomStyle";
            t.Shading.BackgroundColor = new ThemableColor(Colors.Aqua);
            s.Blocks.AddParagraph();
            StylePropertyAsserter.AssertPropertiesAreEqualForAllElementsOfType<Table>(expected, editor.Document);
        }

        [TestMethod]
        public void InsertHyperlink_WhenNoStyle_InsertsHyperlinkWithStyle()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            editor.InsertHyperlink("test that", "uri", false);

            // Assert
            Run run = editor.Document
                .EnumerateChildrenOfType<Run>()
                .Where(r => r.Text == "test that")
                .First();
            Assert.AreEqual(BuiltInStyleNames.HyperlinkStyleId, run.StyleId);
        }

        [TestMethod]
        public void InsertHyperlink_WhenStyleIsSet_InsertsHyperlinkWithStyle()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.CharacterFormatting.StyleId = "CustomStyle";

            // Act
            editor.InsertHyperlink("test that", "uri", false);

            // Assert
            Run run = editor.Document
                .EnumerateChildrenOfType<Run>()
                .Where(r => r.Text == "test that")
                .First();

            Assert.AreEqual("CustomStyle", run.StyleId);
        }
    }
}
