﻿using System;
using System.IO;
#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Model.Shapes;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Editing
{
    [TestClass]
    public class RadFlowDocumentEditorImageTests
    {
        private const string ImageName = "test.png";

        private static readonly Size pngTestImageSize = new Size(90, 30);

        [TestMethod]
        public void InsertInlineImage_WhenSizeIsWithPositiveInfinity_ThrowsArgumentException()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Action action = new Action(() =>
            {
                using (Stream stream = TestHelper.GetImageStream(ImageName))
                {
                    editor.InsertImageInline(stream, "png", new Size(double.PositiveInfinity, double.PositiveInfinity));
                }
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<ArgumentException>(action);
        }

        [TestMethod]
        public void InsertInlineImage_WhenSizeIsNaN_ThrowsArgumentException()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Action action = new Action(() =>
            {
                using (Stream stream = TestHelper.GetImageStream(ImageName))
                {
                    editor.InsertImageInline(stream, "png", new Size(double.NaN, double.NaN));
                };
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<ArgumentException>(action);
        }

        [TestMethod]
        public void InsertInlineImage_WhenSizeIsNotSpecified_GetsImageSizeAutomatically()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            ImageInline image;
            using (Stream stream = TestHelper.GetImageStream(ImageName))
            {
                image = editor.InsertImageInline(stream, "png");
            }

            // Assert
            Assert.AreEqual(pngTestImageSize, image.Image.Size);
        }

        [TestMethod]
        public void InsertFloatingImage_WhenSizeIsNotSpecified_GetsImageSizeAutomatically()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            // Act
            FloatingImage image;
            using (Stream stream = TestHelper.GetImageStream(ImageName))
            {
                image = editor.InsertFloatingImage(stream, "png");
            }

            // Assert
            Assert.AreEqual(pngTestImageSize, image.Image.Size);
        }
    }
}