﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Watermarks;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Editing
{
    [TestClass]
    public class RadFlowDocumentEditorWatermarksTests
    {
        [TestMethod]
        public void SetWatermark_OverwriteExistingWatermarks()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Header header = section.Headers.Add();

            header.Watermarks.Add(new Watermark(new TextWatermarkSettings()));
            header.Watermarks.Add(new Watermark(new ImageWatermarkSettings()));

            Assert.AreEqual(2, header.Watermarks.Count());

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
            editor.SetWatermark(expectedWatermark, header);

            Assert.IsNull(section.Headers.Even);
            Assert.IsNull(section.Headers.First);

            Assert.AreEqual(1, header.Watermarks.Count());
            Assert.AreEqual(expectedWatermark, header.Watermarks.First());
        }

        [TestMethod]
        public void SetWatermark_ToDefaultHeaderOfSection()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section firstSection = document.Sections.AddSection();
            Section secondSection = document.Sections.AddSection();

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
            editor.SetWatermark(expectedWatermark, secondSection, HeaderFooterType.Default);

            Header defaultHeader = secondSection.Headers.Default;

            Assert.IsNotNull(defaultHeader);
            Assert.IsNull(secondSection.Headers.Even);
            Assert.IsNull(secondSection.Headers.First);

            Assert.IsNull(firstSection.Headers.Default);
            Assert.IsNull(firstSection.Headers.Even);
            Assert.IsNull(firstSection.Headers.First);

            Assert.AreEqual(1, defaultHeader.Watermarks.Count());
            Assert.AreEqual(expectedWatermark, defaultHeader.Watermarks.First());
        }

        [TestMethod]
        public void SetWatermark_ToEvenHeaderOfSection()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section firstSection = document.Sections.AddSection();
            Section secondSection = document.Sections.AddSection();

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
            editor.SetWatermark(expectedWatermark, secondSection, HeaderFooterType.Even);

            Header evenHeader = secondSection.Headers.Even;

            Assert.IsNotNull(evenHeader);
            Assert.IsNull(secondSection.Headers.Default);
            Assert.IsNull(secondSection.Headers.First);

            Assert.IsNull(firstSection.Headers.Default);
            Assert.IsNull(firstSection.Headers.Even);
            Assert.IsNull(firstSection.Headers.First);

            Assert.AreEqual(1, evenHeader.Watermarks.Count());
            Assert.AreEqual(expectedWatermark, evenHeader.Watermarks.First());
        }

        [TestMethod]
        public void SetWatermark_ToFirstHeaderOfSection()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section firstSection = document.Sections.AddSection();
            Section secondSection = document.Sections.AddSection();

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
            editor.SetWatermark(expectedWatermark, secondSection, HeaderFooterType.First);

            Header firstHeader = secondSection.Headers.First;

            Assert.IsNotNull(firstHeader);
            Assert.IsNull(secondSection.Headers.Default);
            Assert.IsNull(secondSection.Headers.Even);

            Assert.IsNull(firstSection.Headers.Default);
            Assert.IsNull(firstSection.Headers.Even);
            Assert.IsNull(firstSection.Headers.First);

            Assert.AreEqual(1, firstHeader.Watermarks.Count());
            Assert.AreEqual(expectedWatermark, firstHeader.Watermarks.First());
        }

        //TODO: 159588 See story Add SetWatermark method overload in the RadFlowDocumentEditor to add a watermark to the current section.
        //[TestMethod]
        //public void SetWatermark_CreateFirstSection()
        //{
        //    RadFlowDocument document = new RadFlowDocument();

        //    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
        //    Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
        //    editor.SetWatermark(expectedWatermark, HeaderFooterType.Default);

        //    Section firstSection = document.Sections.FirstOrDefault();

        //    Assert.IsNotNull(firstSection);

        //    Header defaultHeader = firstSection.Headers.Default;

        //    Assert.IsNotNull(defaultHeader);
        //    Assert.IsNull(firstSection.Headers.Even);
        //    Assert.IsNull(firstSection.Headers.First);

        //    Assert.AreEqual(1, defaultHeader.Watermarks.Count());
        //    Assert.AreEqual(expectedWatermark, defaultHeader.Watermarks.First());
        //}

        //TODO: 159588 See story Add SetWatermark method overload in the RadFlowDocumentEditor to add a watermark to the current section.
        //[TestMethod]
        //public void SetWatermark_ToCurrentSection()
        //{
        //    RadFlowDocument document = new RadFlowDocument();

        //    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
        //    editor.InsertSection();
        //    Section currentSection = editor.InsertSection();

        //    Watermark expectedWatermark = new Watermark(new TextWatermarkSettings());
        //    editor.SetWatermark(expectedWatermark, HeaderFooterType.Default);

        //    Header defaultHeader = currentSection.Headers.Default;

        //    Assert.IsNotNull(defaultHeader);
        //    Assert.IsNull(currentSection.Headers.Even);
        //    Assert.IsNull(currentSection.Headers.First);

        //    Assert.AreEqual(1, defaultHeader.Watermarks.Count());
        //    Assert.AreEqual(expectedWatermark, defaultHeader.Watermarks.First());
        //}
    }
}
