﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Fields
{
    [TestClass]
    public class DocumentVariableFieldTests : UpdateFieldTestsBase
    {
        private static readonly string errorResult = "Error! No document variable supplied.";

        [TestMethod]
        public void InsertField_WithDocVariableCode_CreatesAndInsertsDocVariableField()
        {
            this.AssertInsertFieldCreatesFieldOfType("DOCVARIABLE name", typeof(DocumentVariableField));
        }

        [TestMethod]
        public void InsertField_WithDocVariableCodeAndDefinedNames_UpdatesTheResultWithCorrectValue()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.DocumentVariables.Add("single", "first result");
            document.DocumentVariables.Add("multiple words", "second result");

            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.Add("docvariable single", "first result");
            testCases.Add("docvariable \"multiple words\"", "second result");

            this.AssertTestCases(testCases, document);
        }

        [TestMethod]
        public void InsertField_WithDocVariableCode_WhenDocVariableNameIsOverriden_UpdatesTheResultWithLastAddedValue()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.DocumentVariables.Add("single", "first result");
            document.DocumentVariables.Add("single", "first result override");

            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.Add("docvariable single", "first result override");

            Assert.AreEqual(1, document.DocumentVariables.Count);
            this.AssertTestCases(testCases, document);
        }

        [TestMethod]
        public void InsertField_WithDocVariableCode_WhenDocVariableNameIsNotDefinedIn_UpdatesTheResultWithErrorMessage()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.Add("docvariable name", errorResult);

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateFields_WhenDocVariableFieldIsNested_UpdatesAllDocVariableFields()
        {
            // Arrange
            // { DOCVARIABLE { DOCVARIABLE name }} -> result is 'test user'

            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test");
            document.DocumentVariables.Add("test", "test user");

            FieldInfo outerField = editor.InsertField("", "to be updated");
            editor.MoveToInlineEnd(outerField.Start);
            editor.InsertText("DOCVARIABLE ");
            FieldInfo innerField = editor.InsertField("DOCVARIABLE name", "to be updated");

            // Act
            document.UpdateFields();

            // Assert
            Assert.AreEqual("test", innerField.GetResult());
            Assert.AreEqual("test user", outerField.GetResult());
        }

        // TODO: Result fragment is notified about nested fields.
        //[TestMethod]
        //public void UpdateFields_WhenDocVariableFieldIsNested_AndMultipleStringsAreResultFromNestedDocVariable_UpdatesAllDocVariableFields()
        //{
        //    // Arrange
        //    // { DOCVARIABLE { DOCVARIABLE name }} -> result is 'test user'

        //    RadFlowDocument document = new RadFlowDocument();
        //    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

        //    document.DocumentVariables.Add("name", "test name");
        //    document.DocumentVariables.Add("test name", "test user");

        //    FieldInfo outerField = editor.InsertField("", "to be updated");
        //    editor.MoveToInlineEnd(outerField.Start);
        //    editor.InsertText("DOCVARIABLE ");
        //    FieldInfo innerField = editor.InsertField("DOCVARIABLE name", "to be updated");

        //    // Act
        //    document.UpdateFields();

        //    // Assert
        //    Assert.AreEqual("test name", innerField.GetResult());
        //    Assert.AreEqual("test user", outerField.GetResult());
        //}
    }
}