using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Fields
{
    [TestClass]
    public class HyperlinkFieldTests : UpdateFieldTestsBase
    {
        [TestMethod]
        public void InsertField_WithCompareCode_CreatesCompareField()
        {
            this.AssertInsertFieldCreatesFieldOfType("HYPERLINK \"telerik.com\"", typeof(Hyperlink));
        }

        [TestMethod]
        public void UpdateHyperlinkField_WhenResultIsEmpty_InsertsUri()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField("HYPERLINK \"telerik.com\"", null);
            field.UpdateField();

            Assert.AreEqual("telerik.com", field.GetResult());
        }

        [TestMethod]
        public void UpdateHyperlinkField_PointingToBookmark_WhenResultIsEmpty_InsertsUri()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField("HYPERLINK \\l bookmark", null);
            field.UpdateField();

            Assert.AreEqual("bookmark", field.GetResult());
        }

        [TestMethod]
        public void UpdateHyperlinkField_WhenResultIsNotEmpty_DoesNotUpdateResult()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField("HYPERLINK \"telerik.com\"", "test");
            field.UpdateField();

            Assert.AreEqual("test", field.GetResult());
        }

        [TestMethod]
        public void UpdateHyperlinkField_PointingToBookmark_WhenResultIsNotEmpty_DoesNotUpdateResult()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo field = editor.InsertField("HYPERLINK \\l bookmark", "test");
            field.UpdateField();

            Assert.AreEqual("test", field.GetResult());
        }
    }
}