﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Fields
{
    [TestClass]
    public class IfFieldTests : UpdateFieldTestsBase
    {
        [TestMethod]
        public void InsertField_WithIfCode_CreatesIfField()
        {
            this.AssertInsertFieldCreatesFieldOfType("IF 2 = 2 true false", typeof(IfField));
        }

        [TestMethod]
        public void UpdateIfField_WithSimpleNumericValues_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            testCases.Add("IF 2 = 2 true false", "true");
            testCases.Add("IF 1 = 2 true false", "false");
            testCases.Add("IF 2 = 1 true false", "false");

            testCases.Add("IF 2 >= 2 true false", "true");
            testCases.Add("IF 1 >= 2 true false", "false");
            testCases.Add("IF 2 >= 1 true false", "true");

            testCases.Add("IF 2 <= 2 true false", "true");
            testCases.Add("IF 1 <= 2 true false", "true");
            testCases.Add("IF 2 <= 1 true false", "false");

            testCases.Add("IF 2 > 2 true false", "false");
            testCases.Add("IF 1 > 2 true false", "false");
            testCases.Add("IF 2 > 1 true false", "true");

            testCases.Add("IF 2 < 2 true false", "false");
            testCases.Add("IF 1 < 2 true false", "true");
            testCases.Add("IF 2 < 1 true false", "false");

            testCases.Add("IF 2 <> 2 true false", "false");
            testCases.Add("IF 1 <> 2 true false", "true");
            testCases.Add("IF 2 <> 1 true false", "true");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithNumericExpressions_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            testCases.Add("IF 2*2 = 4 true false", "true");
            testCases.Add("IF 2*2 >= 4 true false", "true");
            testCases.Add("IF 2*2 <= 4 true false", "true");

            testCases.Add("IF 2*2 > 4 true false", "false");
            testCases.Add("IF 2*2 < 4 true false", "false");
            testCases.Add("IF 2*2 <> 4 true false", "false");

            testCases.Add("IF 4 = 2*2 true false", "true");
            testCases.Add("IF 4 >= 2*2 true false", "true");
            testCases.Add("IF 4 <= 2*2 true false", "true");

            testCases.Add("IF 4 > 2*2 true false", "false");
            testCases.Add("IF 4 < 2*2 true false", "false");
            testCases.Add("IF 4 <> 2*2 true false", "false");

            testCases.Add("IF 2*2 = 2*2 true false", "true");
            testCases.Add("IF 2*2 >= 2*2 true false", "true");
            testCases.Add("IF 2*2 <= 2*2 true false", "true");

            testCases.Add("IF 2*2 > 2*2 true false", "false");
            testCases.Add("IF 2*2 < 2*2 true false", "false");
            testCases.Add("IF 2*2 <> 2*2 true false", "false");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithParenthesisExpressions_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            testCases.Add("IF (2*2) = (2+2) true false", "true");
            testCases.Add("IF (2*2) >= (2+2) true false", "true");
            testCases.Add("IF (2*2) <= (2+2) true false", "true");
            testCases.Add("IF (2*2) > (2+2) true false", "false");
            testCases.Add("IF (2*2) < (2+2) true false", "false");
            testCases.Add("IF (2*2) <> (2+2) true false", "false");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithFunctionExpressions_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            testCases.Add("IF SUM(2,2) = (2+2) true false", "true");
            testCases.Add("IF SUM(2,2) >= (2+2) true false", "true");
            testCases.Add("IF SUM(2,2) <= (2+2) true false", "true");
            testCases.Add("IF SUM(2,2) > (2+2) true false", "false");
            testCases.Add("IF SUM(2,2) < (2+2) true false", "false");
            testCases.Add("IF SUM(2,2) <> (2+2) true false", "false");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithBookamrkExpressions_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.AddBookmark("bmk", "2");

            testCases.Add("IF bmk = 2 true false", "true");
            testCases.Add("IF bmk >= 2 true false", "true");
            testCases.Add("IF bmk <= 2 true false", "true");
            testCases.Add("IF bmk > 2 true false", "false");
            testCases.Add("IF bmk < 2 true false", "false");
            testCases.Add("IF bmk <> 2 true false", "false");

            testCases.Add("IF 2 = bmk true false", "true");
            testCases.Add("IF 2 >= bmk true false", "true");
            testCases.Add("IF 2 <= bmk true false", "true");
            testCases.Add("IF 2 > bmk true false", "false");
            testCases.Add("IF 2 < bmk true false", "false");
            testCases.Add("IF 2 <> bmk true false", "false");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithQuotedText_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            testCases.Add("IF 3 = 3 true false", "true");
            testCases.Add("IF \"3\" = 3 true false", "true"); // quoted number equal to number value
            testCases.Add("IF 3 = \"3\" true false", "true"); // quoted number equal to number value
            testCases.Add("IF \"3\" = \"3\" true false", "true");

            testCases.Add("IF 3+3 = 3+3 true false", "true");
            testCases.Add("IF \"3+3\" = 3+3 true false", "true");
            testCases.Add("IF 3+3 = \"3+3\" true false", "true");
            testCases.Add("IF \"3+3\" = \"3+3\" true false", "true");


            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithInexistentBookmark_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();

            // There is no such bookmark so only the text matters.
            testCases.Add("IF notBmk = notBmk true false", "true");
            testCases.Add("IF \"notBmk\" = notBmk true false", "true");
            testCases.Add("IF notBmk = \"notBmk\" true false", "true");
            testCases.Add("IF \"notBmk\" = \"notBmk\" true false", "true");

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithNumericBookmark_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.AddBookmark("numBmk", "3");

            testCases.Add("IF numBmk = 3 true false", "true");
            testCases.Add("IF \"numBmk\" = 3 true false", "false"); // quoted text is note treated as bookmark
            testCases.Add("IF numBmk = \"3\" true false", "true"); // quoted number equal to number value
            testCases.Add("IF \"numBmk\" = \"3\" true false", "false"); // quoted text is note treated as bookmark

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateIfField_WithTextBookmark_UpdatesCorrectly()
        {
            FieldUpdateTestCaseCollection testCases = new FieldUpdateTestCaseCollection();
            testCases.AddBookmark("textBmk", "textValue");

            testCases.Add("IF textBmk = textValue true false", "true");
            testCases.Add("IF \"textBmk\" = textValue true false", "false"); // quoted text is note treated as bookmark
            testCases.Add("IF textBmk = \"textValue\" true false", "true"); // quoted number equal to number value
            testCases.Add("IF \"textBmk\" = \"textValue\" true false", "false"); // quoted text is note treated as bookmark

            this.AssertTestCases(testCases);
        }

        //TODO: Tests With WildCards
    }

}
