﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Fields
{
    public class UpdateFieldTestsBase
    {
        protected void AssertTestCases(FieldUpdateTestCaseCollection tests, RadFlowDocument document = null)
        {
            StringBuilder sb = new StringBuilder();

            foreach (FieldUpdateTestCase testCase in tests.TestCases)
            {
                try
                {
                    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document ?? new RadFlowDocument());

                    foreach (BookmarkSetup bmk in tests.BookamrksToAdd)
                    {
                        Run bookmarkedRun = editor.InsertText(bmk.Content);
                        editor.InsertBookmark(bmk.Name, bookmarkedRun, bookmarkedRun);
                        editor.InsertParagraph();
                    }

                    FieldInfo field = editor.InsertField(testCase.FieldCode, "test");
                    field.UpdateField();

                    string actualResult = field.GetResult();
                    if (testCase.ExpectedResult != actualResult)
                    {
                        sb.AppendFormat("Wrong result expression \"{0}\" expected: {1}  actual: {2}",
                            testCase.FieldCode, testCase.ExpectedResult, actualResult);
                        sb.AppendLine();
                    }
                }
                catch (Exception ex)
                {
                    sb.AppendFormat("Exception in expression \"{0}\" Exception:{1}", testCase.FieldCode, ex.Message);
                    sb.AppendLine();
                }
            }
            Assert.IsTrue(sb.Length == 0, sb.ToString());
        }

        protected void AssertInsertFieldCreatesFieldOfType(string input, Type expectedFieldType)
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            FieldInfo field = editor.InsertField(input, "test");
            Assert.IsInstanceOfType(field.Field, expectedFieldType);
        }
    }

    public class FieldUpdateTestCaseCollection
    {
        private readonly List<FieldUpdateTestCase> testCases = new List<FieldUpdateTestCase>();
        private readonly List<BookmarkSetup> bookmarks = new List<BookmarkSetup>();

        public IEnumerable<FieldUpdateTestCase> TestCases
        {
            get { return this.testCases; }
        }


        public IEnumerable<BookmarkSetup> BookamrksToAdd
        {
            get { return this.bookmarks; }
        }

        public void Add(string input, string result)
        {
            this.testCases.Add(new FieldUpdateTestCase(input, result));
        }

        public void AddBookmark(string name, string value)
        {
            this.bookmarks.Add(new BookmarkSetup(name, value));
        }
    }

    public class FieldUpdateTestCase
    {
        public string FieldCode { get; set; }
        public string ExpectedResult { get; set; }
        public FieldUpdateTestCase(string input, string result)
        {
            this.FieldCode = input;
            this.ExpectedResult = result;
        }
    }

    public class BookmarkSetup
    {
        public string Name { get; set; }
        public string Content { get; set; }
        public BookmarkSetup(string input, string result)
        {
            this.Name = input;
            this.Content = result;
        }
    }


}
