﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Fields
{
    [TestClass]
#if !VISUAL
    public class UpdateFieldsAdvancedTests
#else
    public class UpdateFieldsAdvancedTests : Telerik.Windows.Documents.Fixed.Tests.Integration.DocumentFlowVisualBaseTest
#endif
    {
        public const string TestFilesFolder = "Fields/";

        [TestMethod]
        public void UpdateFields_WithManyFields_UdpatesAllFields()
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            for (int i = 0; i < 10; i++)
            {
                editor.InsertField("=10*" + i.ToString(), "to be updated");
            }

            // Act
            editor.Document.UpdateFields();

            List<FieldInfo> fieldsInfos = editor.Document
                                                .EnumerateChildrenOfType<FieldCharacter>()
                                                .Where(fc => fc.FieldCharacterType == FieldCharacterType.Start)
                                                .Select(fc => fc.FieldInfo)
                                                .ToList();

            // Assert
            Assert.AreEqual(10, fieldsInfos.Count);
            for (int i = 0; i < 10; i++)
            {
                Assert.AreEqual((i * 10).ToString(), fieldsInfos[i].GetResult());
            }
        }


        [TestMethod]
        public void UpdateFields_WithNestedFieldsInCode_OneLevelNesting_UpdatesFields()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.Document.UpdateFields(); };
            this.Update_WithNestedFieldsInCode_OneLevelNesting_UpdatesCorrectly_Template(updateFieldAction);
        }

        [TestMethod]
        public void UpdateSingleField_WithNestedFieldsInCode_OneLevelNesting_UpdatesFields()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.UpdateField(); };
            this.Update_WithNestedFieldsInCode_OneLevelNesting_UpdatesCorrectly_Template(updateFieldAction);
        }

        private void Update_WithNestedFieldsInCode_OneLevelNesting_UpdatesCorrectly_Template(Action<FieldInfo> updateAction)
        {
            // Arrange
            // { IF { COMPARE 10 >= 4 } = 1 "True" "False"} -> result is True
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo outerField = editor.InsertField("", "to be updated");
            editor.MoveToInlineEnd(outerField.Start);
            editor.InsertText("IF ");
            FieldInfo innerField = editor.InsertField("COMPARE 10 >= 4", "to be updated");
            editor.InsertText(" = 1 \"True\" \"False\"");

            // Act
            updateAction(outerField);

            // Assert
            Assert.AreEqual("1", innerField.GetResult());
            Assert.AreEqual("True", outerField.GetResult());
        }


        [TestMethod]
        public void UpdateFields_WithNestedFieldsInCode_TwoLevelNesting_UpdatesFields()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.Document.UpdateFields(); };
            this.Update_WithNestedFieldsInCode_TwoLevelNesting_UpdatesCorrectly_Template(updateFieldAction);
        }

        [TestMethod]
        public void UpdateSingleField_WithNestedFieldsInCode_TwoLevelNesting_UpdatesFields()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.UpdateField(); };
            this.Update_WithNestedFieldsInCode_TwoLevelNesting_UpdatesCorrectly_Template(updateFieldAction);
        }

        private void Update_WithNestedFieldsInCode_TwoLevelNesting_UpdatesCorrectly_Template(Action<FieldInfo> updateAction)
        {
            // Arrange
            // { IF { = OR ( { COMPARE 10 >= 4 }, { COMPARE 10 <= 9 } ) } = 1 "True" "False"}
            //    |      |        |                    |
            //  True     1        1                    0

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo outerField = editor.InsertField("", "to be updated");

            editor.MoveToInlineEnd(outerField.Start);
            editor.InsertText("IF ");

            FieldInfo middleField = editor.InsertField("", "to be updated");
            editor.MoveToInlineEnd(middleField.Start);
            editor.InsertText(" = OR ( ");


            FieldInfo innerField1 = editor.InsertField(" COMPARE 10 >= 4 ", "to be updated");
            editor.InsertText(", ");
            FieldInfo innerField2 = editor.InsertField(" COMPARE 10 <= 9 ", "to be updated");
            editor.InsertText(" ) ");

            editor.MoveToInlineEnd(middleField.End);
            editor.InsertText(" = 1 \"True\" \"False\"");

            // Act
            updateAction(outerField);

            // Assert
            Assert.AreEqual("1", innerField1.GetResult());
            Assert.AreEqual("0", innerField2.GetResult());
            Assert.AreEqual("1", middleField.GetResult());
            Assert.AreEqual("True", outerField.GetResult());
        }


        [TestMethod]
        public void UpdateFields_WithNestedFieldsInResult_UpdatesOnlyOuterField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.Document.UpdateFields(); };
            this.Update_WithNestedFieldsInResult_UpdatesOnlyOuterField_Template(updateFieldAction);
        }

        [TestMethod]
        public void UpdateSingleField_WithNestedFieldsInResult_UpdatesOnlyOuterField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.UpdateField(); };
            this.Update_WithNestedFieldsInResult_UpdatesOnlyOuterField_Template(updateFieldAction);
        }

        private void Update_WithNestedFieldsInResult_UpdatesOnlyOuterField_Template(Action<FieldInfo> updateAction)
        {
            // Arrange
            // { IF 3 = 3 True False | { COMPARE 10 >= 4 } }
            //    |                             |
            //  True                       this should be removed             

            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            FieldInfo outerField = editor.InsertField("IF 3 = 3 True False", null);

            editor.MoveToInlineEnd(outerField.Separator);
            FieldInfo innerField = editor.InsertField("COMPARE 10 >= 4", "NOT to be updated");

            // Act
            updateAction(outerField);

            // Assert
            Assert.IsNull(innerField.Start.Parent);
            Assert.IsNull(innerField.Separator.Parent);
            Assert.IsNull(innerField.End.Parent);
            Assert.AreEqual(string.Empty, innerField.GetResult());
            Assert.AreEqual("True", outerField.GetResult());
        }


        [TestMethod]
        public void UpdateFields_WhenFieldInfoIsLocked_DoesNotUpdateField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.Document.UpdateFields(); };
            this.Update_WhenFieldInfoIsLocked_DoesNotUpdateField_Template(updateFieldAction);
        }

        [TestMethod]
        public void UpdateSingleField_WhenFieldInfoIsLocked_DoesNotUpdateField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.UpdateField(); };
            this.Update_WhenFieldInfoIsLocked_DoesNotUpdateField_Template(updateFieldAction);
        }

        private void Update_WhenFieldInfoIsLocked_DoesNotUpdateField_Template(Action<FieldInfo> updateAction)
        {
            // Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            FieldInfo outerField = editor.InsertField("COMPARE 10 >= 4", "fake result");
            outerField.IsLocked = true;

            // Act
            updateAction(outerField);

            // Assert
            Assert.AreEqual("fake result", outerField.GetResult());
        }

        [TestMethod]
        public void UpdateFields_WithLockedNestedFieldsInCode_DoesNotUpdateLockedField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.Document.UpdateFields(); };
            this.Update_WithLockedNestedFieldsInCode_DoesNotUpdateLockedFieldTemplate(updateFieldAction);
        }

        [TestMethod]
        public void UpdateSingleField_WithLockedNestedFieldsInCode_DoesNotUpdateLockedField()
        {
            Action<FieldInfo> updateFieldAction = (fieldInfo) => { fieldInfo.UpdateField(); };
            this.Update_WithLockedNestedFieldsInCode_DoesNotUpdateLockedFieldTemplate(updateFieldAction);
        }

        private void Update_WithLockedNestedFieldsInCode_DoesNotUpdateLockedFieldTemplate(Action<FieldInfo> updateAction)
        {
            // Arrange
            // { IF { COMPARE 10 >= 4 } = 1 "True" "False"} -> result is True
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            FieldInfo outerField = editor.InsertField("", "to be updated");
            editor.MoveToInlineEnd(outerField.Start);
            editor.InsertText("IF ");
            FieldInfo innerField = editor.InsertField("COMPARE 10 >= 4", "0");
            innerField.IsLocked = true;
            editor.InsertText(" = 1 \"True\" \"False\"");

            // Act
            updateAction(outerField);

            // Assert
            Assert.AreEqual("0", innerField.GetResult()); // Locked field is not updated.
            Assert.AreEqual("False", outerField.GetResult());
        }

        [TestMethod]
        public void UpdateField_WhenFieldHasFormatting_IsFormattingPreserved()
        {
            //Arrange
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            editor.CharacterFormatting.FontSize.LocalValue = 9;
            FieldInfo fieldInfo = editor.InsertField("DATE", "10/11/2012");

            //Act
            editor.Document.UpdateFields();

            //Assert
            Paragraph paragraph = fieldInfo.Separator.Paragraph;
            int runIndex = paragraph.Inlines.IndexOf(fieldInfo.Separator) + 1;
            Run run = paragraph.Inlines[runIndex] as Run;

            Assert.IsNotNull(run);
            Assert.AreEqual(DateTime.Now.ToShortDateString(), run.Text);
            Assert.AreEqual(9, run.Properties.FontSize.LocalValue);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Fields)]
        public void UpdateField_WhenFieldHasMultipleFormattedRuns_IsFormattingAsInFirstRun()
        {
            //Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();

            FieldInfo field = new FieldInfo(document);

            ThemableColor expectedForegroundColor = new ThemableColor(Colors.Blue);
            double expectedFontSize = 9;

            paragraph.Inlines.Add(field.Start);
            Run firstRun = paragraph.Inlines.AddRun("=");
            firstRun.Properties.FontSize.LocalValue = expectedFontSize;
            firstRun.Properties.ForegroundColor.LocalValue = expectedForegroundColor;
            Run secondRun = paragraph.Inlines.AddRun("1");
            secondRun.Properties.FontWeight.LocalValue = FontWeights.Bold;
            Run thirdRun = paragraph.Inlines.AddRun("+");
            thirdRun.Properties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            Run fourthRun = paragraph.Inlines.AddRun("2");
            fourthRun.Properties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Times New Roman"));
            paragraph.Inlines.Add(field.Separator);
            paragraph.Inlines.Add(field.End);

            //Act
            document.UpdateFields();

            //Assert
            int runIndex = paragraph.Inlines.IndexOf(field.Separator) + 1;
            Run run = paragraph.Inlines[runIndex] as Run;

            Assert.IsNotNull(run);
            Assert.AreEqual(3.ToString(), run.Text);
            Assert.AreEqual(expectedFontSize, run.Properties.FontSize.LocalValue);
            Assert.AreEqual(expectedForegroundColor, run.Properties.ForegroundColor.LocalValue);
            Assert.IsNull(run.Properties.FontWeight.LocalValue);
            Assert.IsNull(run.Properties.FontFamily.LocalValue);
            Assert.IsNull(run.Properties.BackgroundColor.LocalValue);

#if VISUAL
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
#endif
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void UpdateFields_WhenFieldIsInHeaderFooter_UpdatesFields()
        {
            // Arrange
            // { DOCVARIABLE name } -> result is 'test'

            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            document.DocumentVariables.Add("name", "test");

            Paragraph paragraph = document.Sections.AddSection().Headers.Add().Blocks.AddParagraph();
            editor.MoveToParagraphStart(paragraph);
            FieldInfo innerField = editor.InsertField("DOCVARIABLE name", "to be updated");

            // Act
            document.UpdateFields();

            // Assert
            Assert.AreEqual("test", innerField.GetResult());

#if VISUAL
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
#endif
        }

#if VISUAL
        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.Fields)]

        public void UpdateFields_WhenFieldIsInEvenHeaderFooter_UpdatesFields()
        {
            // Arrange
            // { DOCVARIABLE name } -> result is 'test'

            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            document.DocumentVariables.Add("name", "test");

            Paragraph paragraph = document.Sections.AddSection().Headers.Add(HeaderFooterType.Even).Blocks.AddParagraph();
            paragraph.Inlines.AddRun(" run");
            document.Properties.HasDifferentEvenOddPageHeadersFooters.LocalValue = true;
            editor.MoveToParagraphStart(paragraph);
            FieldInfo innerField = editor.InsertField("DOCVARIABLE name", "to be updated");
            // Act

            document.Sections.AddSection();
            document.UpdateFields();

            // Assert
            Assert.AreEqual("test", innerField.GetResult());
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        public void UpdateFields_WhenFieldIsInFirstHeaderFooter_UpdatesFields()
        {
            // Arrange
            // { DOCVARIABLE name } -> result is 'test'

            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            document.DocumentVariables.Add("name", "test");

            Paragraph paragraph = document.Sections.AddSection().Headers.Add(HeaderFooterType.First).Blocks.AddParagraph();
            paragraph.Inlines.AddRun(" run");
            document.Sections.First().Properties.HasDifferentFirstPageHeaderFooter.LocalValue = true;
            editor.MoveToParagraphStart(paragraph);
            FieldInfo innerField = editor.InsertField("DOCVARIABLE name", "to be updated");
            // Act

            document.Sections.AddSection();
            document.UpdateFields();

            // Assert
            Assert.AreEqual("test", innerField.GetResult());
            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }
#endif

        [TestMethod]
        public void UpdateFields_DocumentContainingFieldsWithoutSeparator_UpdateFieldsDoesNotThrowNullReferenceException()
        {
            RadFlowDocument imported = TestHelper.GetDocumentFromFile(
                string.Format("{0}CodeFields/SimpleFieldWithoutSeparator.docx", TestFilesFolder), new DocxFormatProvider());

            try
            {
                imported.UpdateFields();
            }
            catch (NullReferenceException)
            {
                Assert.Fail("Update fields should not throw NullReferenceException!");
            }
        }
    }
}
