﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class FootersTests
    {
        [TestMethod]
        public void Constructor_ByDefault_AllFootersAreNull()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            Assert.IsNull(footers.Default);
            Assert.IsNull(footers.Even);
            Assert.IsNull(footers.First);
        }

        [TestMethod]
        public void Add_WithoutParameter_AddsDefaultFooter()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            Footer addedFooter = footers.Add();

            Assert.AreEqual(addedFooter, footers.Default);
            Assert.IsNotNull(footers.Default);
            Assert.AreEqual(document, footers.Default.Document);
            Assert.AreEqual(section, footers.Default.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsDefault_AddsValidDefaultFooter()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            Footer addedFooter = footers.Add(HeaderFooterType.Default);

            Assert.AreEqual(addedFooter, footers.Default);
            Assert.IsNotNull(footers.Default);
            Assert.AreEqual(document, footers.Default.Document);
            Assert.AreEqual(section, footers.Default.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsEven_AddsValidEvenFooter()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            Footer addedFooter = footers.Add(HeaderFooterType.Even);

            Assert.AreEqual(addedFooter, footers.Even);
            Assert.IsNotNull(footers.Even);
            Assert.AreEqual(document, footers.Even.Document);
            Assert.AreEqual(section, footers.Even.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsFirst_AddsValidFirstFooter()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            Footer addedFooter = footers.Add(HeaderFooterType.First);

            Assert.AreEqual(addedFooter, footers.First);
            Assert.IsNotNull(footers.First);
            Assert.AreEqual(document, footers.First.Document);
            Assert.AreEqual(section, footers.First.Parent);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsDefault_ClearsDefaultFooter()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            footers.Add(HeaderFooterType.Default);

            // Act
            footers.Remove(HeaderFooterType.Default);

            // Assert
            Assert.IsNull(footers.Default);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsEven_ClearsEvenFooter()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            footers.Add(HeaderFooterType.Even);

            // Act
            footers.Remove(HeaderFooterType.Even);

            // Assert
            Assert.IsNull(footers.Even);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsFirst_ClearsFirstFooter()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Footers footers = section.Footers;

            footers.Add(HeaderFooterType.First);

            // Act
            footers.Remove(HeaderFooterType.First);

            // Assert
            Assert.IsNull(footers.First);
        }
    }
}
