﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class HeadersTests
    {
        [TestMethod]
        public void Constructor_ByDefault_AllHeadersAreNull()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            Assert.IsNull(headers.Default);
            Assert.IsNull(headers.Even);
            Assert.IsNull(headers.First);
        }

        [TestMethod]
        public void Add_WithoutParameter_AddsDefaultHeader()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            Header addedHeader = headers.Add();

            Assert.AreEqual(addedHeader, headers.Default);
            Assert.IsNotNull(headers.Default);
            Assert.AreEqual(document, headers.Default.Document);
            Assert.AreEqual(section, headers.Default.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsDefault_AddsValidOddHeader()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            Header addedHeader = headers.Add(HeaderFooterType.Default);

            Assert.AreEqual(addedHeader, headers.Default);
            Assert.IsNotNull(headers.Default);
            Assert.AreEqual(document, headers.Default.Document);
            Assert.AreEqual(section, headers.Default.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsEven_AddsValidEvenHeader()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            Header addedHeader = headers.Add(HeaderFooterType.Even);

            Assert.AreEqual(addedHeader, headers.Even);
            Assert.IsNotNull(headers.Even);
            Assert.AreEqual(document, headers.Even.Document);
            Assert.AreEqual(section, headers.Even.Parent);
        }

        [TestMethod]
        public void Add_WhenHeaderFooterTypeIsFirst_AddsValidFirstHeader()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            Header addedHeader = headers.Add(HeaderFooterType.First);

            Assert.AreEqual(addedHeader, headers.First);
            Assert.IsNotNull(headers.First);
            Assert.AreEqual(document, headers.First.Document);
            Assert.AreEqual(section, headers.First.Parent);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsDefault_ClearsDefaulHeader()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            headers.Add(HeaderFooterType.Default);

            // Act
            headers.Remove(HeaderFooterType.Default);

            // Assert
            Assert.IsNull(headers.Default);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsEven_ClearsEvenHeader()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            headers.Add(HeaderFooterType.Even);

            // Act
            headers.Remove(HeaderFooterType.Even);

            // Assert
            Assert.IsNull(headers.Even);
        }

        [TestMethod]
        public void Remove_WhenHeaderFooterTypeIsFirst_ClearsFirstHeader()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();
            Headers headers = section.Headers;

            headers.Add(HeaderFooterType.First);

            // Act
            headers.Remove(HeaderFooterType.First);

            // Assert
            Assert.IsNull(headers.First);
        }
    }
}
