﻿using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Lists;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Lists
{
    [TestClass]
    public class ListsPropertyEvaluationTests
    {
        [TestMethod]
        public void EvaluateCharacterStyleProperty()
        {
            RadFlowDocument document = new RadFlowDocument();

            Color defaultColor = Colors.Red;

            document.DefaultStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(defaultColor);
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].CharacterProperties.ForegroundColor, new ThemableColor(defaultColor));
            }
        }

        [TestMethod]
        public void EvaluateParagraphStyleProperty()
        {
            RadFlowDocument document = new RadFlowDocument();

            Color defaultColor = Colors.Red;

            document.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(defaultColor);
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].ParagraphProperties.BackgroundColor, new ThemableColor(defaultColor));
            }
        }

        [TestMethod]
        public void SetLocalChacaterPropertyInTheFirstListLEvel_EvaluateStyleProperty_InList()
        {
            RadFlowDocument document = new RadFlowDocument();

            Color defaultColor = Colors.Red;

            document.DefaultStyle.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(defaultColor);
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Color localColor = Colors.Orange;

            list.Levels[0].CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(localColor);

            Assert.IsTrue(list.Levels[0].CharacterProperties.ForegroundColor.GetActualValue().LocalValue == localColor);

            for (int i = 1; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].CharacterProperties.ForegroundColor, new ThemableColor(defaultColor));
            }
        }

        [TestMethod]
        public void SetLocalParagraphPropertyInTheFirstListLEvel_EvaluateStyleProperty_InList()
        {
            RadFlowDocument document = new RadFlowDocument();

            Color defaultColor = Colors.Red;

            document.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(defaultColor);
            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Color localColor = Colors.Orange;

            list.Levels[0].ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(localColor);

            StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[0].ParagraphProperties.BackgroundColor, new ThemableColor(localColor));

            for (int i = 1; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].ParagraphProperties.BackgroundColor, new ThemableColor(defaultColor));
            }
        }

        [TestMethod]
        public void EvaluateStyleProperties_OfListLevel_WithAppliedParagraphStyleOnListLevel()
        {
            RadFlowDocument document = new RadFlowDocument();

            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(2));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(3));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(4));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(5));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(6));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(7));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(8));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(9));

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                list.Levels[i].StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
            }

            for (int i = 0; i < list.Levels.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].ParagraphProperties.SpacingAfter,
                    document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).ParagraphProperties.SpacingAfter.LocalValue);

                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].ParagraphProperties.SpacingBefore,
                    document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).ParagraphProperties.SpacingBefore.LocalValue);

                StylePropertyAsserter.ConfirmStylePropertyValue(list.Levels[i].CharacterProperties.ForegroundColor,
                     document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).CharacterProperties.ForegroundColor.LocalValue);
            }
        }

        [TestMethod]
        public void EvaluateStyleProperties_OfParagraph_WithAppliedParagraphStyleOnListLevel()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(2));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(3));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(4));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(5));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(6));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(7));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(8));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(9));

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                list.Levels[i].StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
                Paragraph paragraph = section.Blocks.AddParagraph();
                paragraph.ListId = list.Id;
                paragraph.ListLevel = i;

                paragraph.Inlines.AddRun("Text - " + i);
            }

            List<Paragraph> paragraphs = section.EnumerateChildrenOfType<Paragraph>().ToList();

            Assert.IsTrue(paragraphs.Count == 9);

            for (int i = 0; i < paragraphs.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(paragraphs[i].Properties.SpacingAfter,
                    document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).ParagraphProperties.SpacingAfter.LocalValue);

                StylePropertyAsserter.ConfirmStylePropertyValue(paragraphs[i].Properties.SpacingBefore,
                    document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).ParagraphProperties.SpacingBefore.LocalValue);

                StylePropertyAsserter.ConfirmStylePropertyValue((paragraphs[i].Inlines.First() as Run).Properties.ForegroundColor,
                     document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).CharacterProperties.ForegroundColor.LocalValue);
            }
        }

        [TestMethod]
        public void EvaluateStyleProperties_OfParagraph_WithAppliedParagraphStyleOnListLevel_AndLocalPropertyValue()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(2));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(3));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(4));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(5));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(6));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(7));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(8));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(9));

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                list.Levels[i].StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
                Paragraph paragraph = section.Blocks.AddParagraph();
                paragraph.ListId = list.Id;
                paragraph.ListLevel = i;

                paragraph.Inlines.AddRun("Text - " + i);
                paragraph.Spacing.SpacingAfter = 44D;
            }

            List<Paragraph> paragraphs = section.EnumerateChildrenOfType<Paragraph>().ToList();

            Assert.IsTrue(paragraphs.Count == 9);

            for (int i = 0; i < paragraphs.Count; i++)
            {
                StylePropertyAsserter.ConfirmStylePropertyValue(paragraphs[i].Properties.SpacingAfter, 44D);

                StylePropertyAsserter.ConfirmStylePropertyValue(paragraphs[i].Properties.SpacingBefore,
                    document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).ParagraphProperties.SpacingBefore.LocalValue);

                StylePropertyAsserter.ConfirmStylePropertyValue((paragraphs[i].Inlines.First() as Run).Properties.ForegroundColor,
                     document.StyleRepository.GetStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1)).CharacterProperties.ForegroundColor.LocalValue);
            }
        }

        [TestMethod]
        public void EvaluateStyleProperties_OfRunInParagraph_WithAppliedCustomStyleOnParagraph_AfterAppliedListWithStyle()
        {
            RadFlowDocument document = new RadFlowDocument();
            Section section = document.Sections.AddSection();

            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(1));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(2));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(3));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(4));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(5));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(6));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(7));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(8));
            document.StyleRepository.AddBuiltInStyle(BuiltInStyleNames.GetHeadingStyleIdByIndex(9));

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < list.Levels.Count; i++)
            {
                list.Levels[i].StyleId = BuiltInStyleNames.GetHeadingStyleIdByIndex(i + 1);
                Paragraph paragraph = section.Blocks.AddParagraph();
                paragraph.ListId = list.Id;
                paragraph.ListLevel = i;

                paragraph.Inlines.AddRun("Text - " + i);
                paragraph.Spacing.SpacingAfter = 44D;
            }

            List<Paragraph> paragraphs = section.EnumerateChildrenOfType<Paragraph>().ToList();

            Assert.IsTrue(paragraphs.Count == 9);

            Style style = new Style("paragraphStyle", StyleType.Paragraph);
            style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Cyan);

            document.StyleRepository.Add(style);

            paragraphs[0].StyleId = style.Id;

            StylePropertyAsserter.ConfirmStylePropertyValue((paragraphs[0].Inlines.First() as Run).Properties.ForegroundColor,
                 document.StyleRepository.GetStyle("paragraphStyle").CharacterProperties.ForegroundColor.LocalValue);

        }
    }
}
