﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Integration.ImportExport.Fields;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class MailMergeDocumentsTests
    {
        [TestMethod]
        public void MailMerge_WhenCollectionIsNull_ThrowsArgumentNullException()
        {
            RadFlowDocument document = new RadFlowDocument();

            try
            {
                document.MailMerge(null);
                Assert.Fail("Should throw ArgumentNullException.");
            }
            catch (ArgumentNullException)
            {
            }
        }

        [TestMethod]
        public void MailMerge_WhenCollectionEmpty_ThrowsArgumentException()
        {
            RadFlowDocument document = new RadFlowDocument();

            try
            {
                document.MailMerge(Enumerable.Empty<object>());
                Assert.Fail("Should throw ArgumentException.");
            }
            catch (ArgumentException)
            {
            }
        }

        [TestMethod]
        public void MailMerge_DocumentDefaultStyleProperties_MergedDocumentHasSameDefaultStyleProperties()
        {
            RadFlowDocument document = new RadFlowDocument();

            document.DefaultStyle.CharacterProperties.FontSize.LocalValue = 30;
            document.DefaultStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            document.DefaultStyle.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;

            document.DefaultStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Right;
            document.DefaultStyle.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.Blue);
            document.DefaultStyle.ParagraphProperties.AutomaticSpacingBefore.LocalValue = true;

            RadFlowDocument mailMerged = document.MailMerge(Person.GetList(2));

            StylePropertyAsserter.AssertDefaultStylePropertiesAreEqual(document.DefaultStyle, mailMerged.DefaultStyle);
        }

        [TestMethod]
        public void MailMerge_DocumentDefaultStyleProperties_MergedHasSameInitialValues()
        {
            RadFlowDocument document = new RadFlowDocument();

            document.DefaultStyle.CharacterProperties.FontSize.ClearValue();

            document.DefaultStyle.ParagraphProperties.LineSpacing.ClearValue();

            RadFlowDocument clonedDocument = document.MailMerge(Person.GetList(2));

            StylePropertyAsserter.AssertDefaultStylePropertiesAreEqual(document.DefaultStyle, clonedDocument.DefaultStyle);
        }
    }
}
