﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class RadFlowDocumentTests
    {
        [TestMethod]
        public void EnumerateChildrenOfType_ForSection_ReturnsAllSections()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section1 = document.Sections.AddSection();

            // Act
            List<Section> children = document.EnumerateChildrenOfType<Section>().ToList();

            // Assert
            Assert.AreEqual(1, children.Count);
            Assert.AreEqual(section1, children[0]);
        }

        [TestMethod]
        public void EnumerateChildrenOfType_ForDocumentElementBaseOnDocumentWithOneChild_ReturnsAllChildren()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section1 = document.Sections.AddSection();

            // Act
            List<DocumentElementBase> children = document.EnumerateChildrenOfType<DocumentElementBase>().ToList();

            // Assert
            Assert.AreEqual(1, children.Count);
            Assert.AreEqual(section1, children[0]);
        }

        [TestMethod]
        public void EnumerateChildrenOfType_ForDocumentElementBaseOnDocumentWithMultipleChildren_ReturnsAllChildren()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section1 = document.Sections.AddSection();
            Paragraph paragraph1 = section1.Blocks.AddParagraph();
            Run run1 = paragraph1.Inlines.AddRun();

            // Act
            List<DocumentElementBase> children = document.EnumerateChildrenOfType<DocumentElementBase>().ToList();

            // Assert
            Assert.AreEqual(3, children.Count);
            Assert.AreEqual(section1, children[0]);
            Assert.AreEqual(paragraph1, children[1]);
            Assert.AreEqual(run1, children[2]);
        }

        [TestMethod]
        public void EnumerateChildrenOfType_ForRun_ReturnsAllRuns()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section1 = document.Sections.AddSection();
            Paragraph paragraph1 = section1.Blocks.AddParagraph();
            Run run1 = paragraph1.Inlines.AddRun();

            // Act
            List<Run> children = document.EnumerateChildrenOfType<Run>().ToList();

            // Assert
            Assert.AreEqual(1, children.Count);
            Assert.AreEqual(run1, children[0]);
        }

        [TestMethod]
        public void EnumerateChildrenOfType_ForTableWithDocumentWithoutTables_ReturnsEmptyEnumerable()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Section section1 = document.Sections.AddSection();
            section1.Blocks.AddParagraph();
            section1.Blocks.AddParagraph();

            // Act
            List<Table> children = document.EnumerateChildrenOfType<Table>().ToList();

            // Assert
            Assert.AreEqual(0, children.Count);
        }

        [TestMethod]
        public void EnumerateChildrenOfType_ForRunWithEmptyDocument_ReturnsEmptyEnumerable()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            // Act
            List<Run> children = document.EnumerateChildrenOfType<Run>().ToList();

            // Assert
            Assert.AreEqual(0, children.Count);
        }
    }
}
