﻿#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Style = Telerik.Windows.Documents.Flow.Model.Styles.Style;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Styles
{
    [TestClass]
    public class LinkedStylesTests
    {
        [TestMethod]
        public void EvaluationOfStyleProperty_SetFontWeightToCharacterPart()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            document.StyleRepository.Add(characterPartStyle);
            document.StyleRepository.Add(paragraphPartStyle);

            characterPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;

            StylePropertyAsserter.AssertPropertyValue(characterPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
        }

        [TestMethod]
        public void EvaluationOfStyleProperty_SetFontWeightToBothParts()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            document.StyleRepository.Add(characterPartStyle);
            document.StyleRepository.Add(paragraphPartStyle);

            characterPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            paragraphPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Normal;

            StylePropertyAsserter.AssertPropertyValue(characterPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Normal);
        }

        [TestMethod]
        public void EvaluationOfStyleProperty_LinkedStyleBasedOnParagraphStyle_SetLocalPropertiesToBaseStyle()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style baseParagraphPartStyle = new Style("baseParagraphPartStyle", StyleType.Paragraph);
            document.StyleRepository.Add(baseParagraphPartStyle);

            baseParagraphPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            baseParagraphPartStyle.ParagraphProperties.FlowDirection.LocalValue = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection.RightToLeft;

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            paragraphPartStyle.BasedOnStyleId = baseParagraphPartStyle.Id;

            document.StyleRepository.Add(paragraphPartStyle);
            document.StyleRepository.Add(characterPartStyle);

            StylePropertyAsserter.AssertPropertyValue(characterPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Paragraph.FlowDirectionPropertyDefinition,
                Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection.RightToLeft);
        }

        [TestMethod]
        public void EvaluationOfStyleProperty_LinkedStyleBasedOnLinkedStyle_SetFontWeightToBaseCharacterPartStyle()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style baseParagraphPartStyle = new Style("baseParagraphPartStyle", StyleType.Paragraph);
            Style baseCharacterPartStyle = new Style("baseCharacterPartStyle", StyleType.Character);

            baseParagraphPartStyle.LinkedStyleId = baseCharacterPartStyle.Id;
            baseCharacterPartStyle.LinkedStyleId = baseParagraphPartStyle.Id;

            document.StyleRepository.Add(baseParagraphPartStyle);
            document.StyleRepository.Add(baseCharacterPartStyle);

            baseCharacterPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            paragraphPartStyle.BasedOnStyleId = baseParagraphPartStyle.Id;
            characterPartStyle.BasedOnStyleId = baseCharacterPartStyle.Id;

            document.StyleRepository.Add(paragraphPartStyle);
            document.StyleRepository.Add(characterPartStyle);

            StylePropertyAsserter.AssertPropertyValue(characterPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
        }

        [TestMethod]
        public void EvaluationOfStyleProperty_LinkedStyleBasedOnLinkedStyle_SetFontWeightToBothPartsFromBaseStyle()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style baseParagraphPartStyle = new Style("baseParagraphPartStyle", StyleType.Paragraph);
            Style baseCharacterPartStyle = new Style("baseCharacterPartStyle", StyleType.Character);

            baseParagraphPartStyle.LinkedStyleId = baseCharacterPartStyle.Id;
            baseCharacterPartStyle.LinkedStyleId = baseParagraphPartStyle.Id;

            document.StyleRepository.Add(baseParagraphPartStyle);
            document.StyleRepository.Add(baseCharacterPartStyle);

            baseParagraphPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Normal;
            baseCharacterPartStyle.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            paragraphPartStyle.BasedOnStyleId = baseParagraphPartStyle.Id;
            characterPartStyle.BasedOnStyleId = baseCharacterPartStyle.Id;

            document.StyleRepository.Add(paragraphPartStyle);
            document.StyleRepository.Add(characterPartStyle);

            StylePropertyAsserter.AssertPropertyValue(characterPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Run.FontWeightPropertyDefinition, FontWeights.Bold);
        }

        [TestMethod]
        public void EvaluationOfStyleProperty_LinkedStyleBasedOnLinkedStyle_SetFlowDirectionToBaseParagraphPart()
        {
            RadFlowDocument document = new RadFlowDocument();

            Style baseParagraphPartStyle = new Style("baseParagraphPartStyle", StyleType.Paragraph);
            Style baseCharacterPartStyle = new Style("baseCharacterPartStyle", StyleType.Character);

            baseParagraphPartStyle.LinkedStyleId = baseCharacterPartStyle.Id;
            baseCharacterPartStyle.LinkedStyleId = baseParagraphPartStyle.Id;

            document.StyleRepository.Add(baseParagraphPartStyle);
            document.StyleRepository.Add(baseCharacterPartStyle);

            baseParagraphPartStyle.ParagraphProperties.FlowDirection.LocalValue = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection.RightToLeft;

            Style paragraphPartStyle = new Style("paragraphPartStyle", StyleType.Paragraph);
            Style characterPartStyle = new Style("characterPartStyle", StyleType.Character);

            characterPartStyle.LinkedStyleId = paragraphPartStyle.Id;
            paragraphPartStyle.LinkedStyleId = characterPartStyle.Id;

            paragraphPartStyle.BasedOnStyleId = baseParagraphPartStyle.Id;
            characterPartStyle.BasedOnStyleId = baseCharacterPartStyle.Id;

            document.StyleRepository.Add(paragraphPartStyle);
            document.StyleRepository.Add(characterPartStyle);

            StylePropertyAsserter.AssertPropertyValue(paragraphPartStyle, Paragraph.FlowDirectionPropertyDefinition,
                Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection.RightToLeft);
        }
    }
}
