﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
using Style = Telerik.Windows.Documents.Flow.Model.Styles.Style;
using FlowDirection = Telerik.Windows.Documents.Flow.Model.Styles.FlowDirection;
using VerticalAlignment = Telerik.Windows.Documents.Flow.Model.Styles.VerticalAlignment;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Primitives;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Model.Styles
{
    [TestClass]
    public class StyleTests
    {
        [TestMethod]
        public void Clone_WhenCharacterPropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Character);
            style.CharacterProperties.BackgroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade2);
            style.CharacterProperties.FontFamily.LocalValue = new ThemableFontFamily(new FontFamily("Comic Sans MS"));
            style.CharacterProperties.FontSize.LocalValue = 30;
            style.CharacterProperties.FontStyle.LocalValue = FontStyles.Italic;
            style.CharacterProperties.FontWeight.LocalValue = FontWeights.Bold;
            style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3);
            style.CharacterProperties.HighlightColor.LocalValue = Colors.Red;
            style.CharacterProperties.ShadingPattern.LocalValue = ShadingPattern.DiagonalStripe;
            style.CharacterProperties.ShadingPatternColor.LocalValue = new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4);
            style.CharacterProperties.Strikethrough.LocalValue = true;
            style.CharacterProperties.UnderlineColor.LocalValue = new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade5);
            style.CharacterProperties.UnderlinePattern.LocalValue = UnderlinePattern.DashLongHeavy;

            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style.CharacterProperties, clone.CharacterProperties);
        }

        [TestMethod]
        public void Clone_WhenParagraphPropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Paragraph);
            style.ParagraphProperties.BackgroundColor.LocalValue = new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade2);
            style.ParagraphProperties.Borders.LocalValue = new ParagraphBorders(new Border(5, BorderStyle.DashSmallGap, new ThemableColor(Colors.Red)));
            style.ParagraphProperties.FirstLineIndent.LocalValue = 20;
            style.ParagraphProperties.LineSpacingType.LocalValue = HeightType.Exact;
            style.ParagraphProperties.LineSpacing.LocalValue = 4;
            style.ParagraphProperties.ListLevel.LocalValue = 7;
            style.ParagraphProperties.TextAlignment.LocalValue = Alignment.HighKashida;
            style.ParagraphProperties.TabStops.LocalValue = new TabStopCollection().Insert(new TabStop(100, TabStopType.Bar, TabStopLeader.Dot))
                                                                                   .Insert(new TabStop(10, TabStopType.Center));
            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style.ParagraphProperties, clone.ParagraphProperties);
        }

        [TestMethod]
        public void Clone_WhenTablePropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Table);
            style.TableProperties.Alignment.LocalValue = Alignment.Right;
            style.TableProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.Red);
            style.TableProperties.Borders.LocalValue = new TableBorders(new Border(5, BorderStyle.Dotted, new ThemableColor(Colors.Green)));
            style.TableProperties.ColumnBanding.LocalValue = 3;
            style.TableProperties.FlowDirection.LocalValue = FlowDirection.RightToLeft;
            style.TableProperties.Indent.LocalValue = 40;
            style.TableProperties.LayoutType.LocalValue = TableLayoutType.FixedWidth;
            style.TableProperties.Looks.LocalValue = TableLooks.LastColumn;
            style.TableProperties.Overlap.LocalValue = false;
            style.TableProperties.PreferredWidth.LocalValue = new TableWidthUnit(TableWidthUnitType.Percent, 50);
            style.TableProperties.RowBanding.LocalValue = 4;
            style.TableProperties.ShadingPattern.LocalValue = ShadingPattern.DiagonalStripe;
            style.TableProperties.ShadingPatternColor.LocalValue = new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4);
            style.TableProperties.TableCellPadding.LocalValue = new Padding(7);
            style.TableProperties.TableCellSpacing.LocalValue = 3;

            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style.TableProperties, clone.TableProperties);
        }

        [TestMethod]
        public void Clone_WhenTableRowPropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Table);
            style.TableRowProperties.CanSplit.LocalValue = false;
            style.TableRowProperties.Height.LocalValue = new TableRowHeight(HeightType.AtLeast, 100);
            style.TableRowProperties.RepeatOnEveryPage.LocalValue = true;
            style.TableRowProperties.TableCellSpacing.LocalValue = 15;


            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style.TableRowProperties, clone.TableRowProperties);
        }

        [TestMethod]
        public void Clone_WhenTableCellPropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Table);
            style.TableCellProperties.BackgroundColor.LocalValue = new ThemableColor(Colors.Blue);
            style.TableCellProperties.Borders.LocalValue = new TableCellBorders(new Border(5, BorderStyle.Double, new ThemableColor(Colors.Yellow)));
            style.TableCellProperties.CanWrapContent.LocalValue = false;
            style.TableCellProperties.ColumnSpan.LocalValue = 2;
            style.TableCellProperties.IgnoreCellMarkerInRowHeightCalculation.LocalValue = true;
            style.TableCellProperties.Padding.LocalValue = new Padding(13);
            style.TableCellProperties.PreferredWidth.LocalValue = new TableWidthUnit(TableWidthUnitType.Fixed, 50);
            style.TableCellProperties.RowSpan.LocalValue = 2;
            style.TableCellProperties.ShadingPattern.LocalValue = ShadingPattern.Percent30;
            style.TableCellProperties.ShadingPatternColor.LocalValue = new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade5);
            style.TableCellProperties.VerticalAlignment.LocalValue = VerticalAlignment.Bottom;

            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style.TableCellProperties, clone.TableCellProperties);
        }

        [TestMethod]
        public void Clone_WhenStylePropertiesAreSetOnStyle_CloningHasSameProperties()
        {
            // Arrange
            Style style = new Style("test", StyleType.Paragraph);
            style.BasedOnStyleId = BuiltInStyleNames.NormalStyleId;
            style.NextStyleId = BuiltInStyleNames.NormalStyleId;
            style.LinkedStyleId = "pseudoLinkedStyleId";
            style.IsDefault = true;
            style.IsCustom = true;
            style.IsPrimary = true;
            style.UIPriority = 15;

            // Act
            Style clone = style.Clone();

            // Assert
            StylePropertyAsserter.AssertStylesAreEqual(style, clone);

            // Verify (partially) this is deep copy.
            Assert.AreNotSame(style, clone);
        }
    }
}
