﻿#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Model.Styles
{
    [TestClass]
    public class TestBorders
    {
        private static readonly Border oldBorder = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red), true, true, 20);

        [TestMethod]
        public void TestParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            Assert.AreEqual(borders.Top, borders.Left);
            Assert.AreEqual(borders.Top, borders.Bottom);
            Assert.AreEqual(borders.Top, borders.Right);
            Assert.AreEqual(borders.Top, borders.Between);
        }

        [TestMethod]
        public void TestChangeLeftBorderInParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            ParagraphBorders newBorders = BordersAsserter.AssertParagraphBorders(borders, oldBorder, borders.SetLeft);

            Assert.AreEqual(borders.Left, oldBorder);
            Assert.AreNotEqual(newBorders.Left, oldBorder);
        }

        [TestMethod]
        public void TestChangeTopBorderInParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            ParagraphBorders newBorders = BordersAsserter.AssertParagraphBorders(borders, oldBorder, borders.SetTop);

            Assert.AreEqual(borders.Top, oldBorder);
            Assert.AreNotEqual(newBorders.Top, oldBorder);
        }

        [TestMethod]
        public void TestChangeRightBorderInParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            ParagraphBorders newBorders = BordersAsserter.AssertParagraphBorders(borders, oldBorder, borders.SetRight);

            Assert.AreEqual(borders.Right, oldBorder);
            Assert.AreNotEqual(newBorders.Right, oldBorder);
        }

        [TestMethod]
        public void TestChangeBottomBorderInParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            ParagraphBorders newBorders = BordersAsserter.AssertParagraphBorders(borders, oldBorder, borders.SetBottom);

            Assert.AreEqual(borders.Bottom, oldBorder);
            Assert.AreNotEqual(newBorders.Bottom, oldBorder);
        }

        [TestMethod]
        public void TestChangeBetweenBorderInParagraphBorders()
        {
            ParagraphBorders borders = new ParagraphBorders(oldBorder);
            ParagraphBorders newBorders = BordersAsserter.AssertParagraphBorders(borders, oldBorder, (Border border) => { return new ParagraphBorders(borders, between: border); });

            Assert.AreEqual(borders.Between, oldBorder);
            Assert.AreNotEqual(newBorders.Between, oldBorder);
        }

        [TestMethod]
        public void TestTableBorders()
        {
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red), true, true, 20);

            TableBorders borders = new TableBorders(border);
            Assert.AreEqual(borders.Top, borders.Left);
            Assert.AreEqual(borders.Top, borders.Bottom);
            Assert.AreEqual(borders.Top, borders.Right);
            Assert.AreEqual(borders.Top, borders.InsideHorizontal);
            Assert.AreEqual(borders.Top, borders.InsideVertical);
        }

        [TestMethod]
        public void TestChangeLeftBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, leftBorder: border); });

            Assert.AreEqual(borders.Left, oldBorder);
            Assert.AreNotEqual(newBorders.Left, oldBorder);
        }

        [TestMethod]
        public void TestChangeTopBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, topBorder: border); });

            Assert.AreEqual(borders.Top, oldBorder);
            Assert.AreNotEqual(newBorders.Top, oldBorder);
        }

        [TestMethod]
        public void TestChangeRightBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, rightBorder: border); });

            Assert.AreEqual(borders.Right, oldBorder);
            Assert.AreNotEqual(newBorders.Right, oldBorder);
        }

        [TestMethod]
        public void TestChangeBottomBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, bottomBorder: border); });

            Assert.AreEqual(borders.Bottom, oldBorder);
            Assert.AreNotEqual(newBorders.Bottom, oldBorder);
        }

        [TestMethod]
        public void TestChangeInsideHorizontalBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, insideHorizontalBorder: border); });

            Assert.AreEqual(borders.InsideHorizontal, oldBorder);
            Assert.AreNotEqual(newBorders.InsideHorizontal, oldBorder);
        }

        [TestMethod]
        public void TestChangeInsideVerticalBorderInTableBorders()
        {
            TableBorders borders = new TableBorders(oldBorder);
            TableBorders newBorders = BordersAsserter.AssertTableBorders(borders, oldBorder, (Border border) => { return new TableBorders(borders, insideVerticalBorder: border); });

            Assert.AreEqual(borders.InsideVertical, oldBorder);
            Assert.AreNotEqual(newBorders.InsideVertical, oldBorder);
        }

        [TestMethod]
        public void TestTableCellBorders()
        {
            Border border = new Border(1, BorderStyle.Single, new ThemableColor(Colors.Red), true, true, 20);

            TableCellBorders borders = new TableCellBorders(border, border, border, border, border, border, border, border);
            Assert.AreEqual(borders.Top, borders.Left);
            Assert.AreEqual(borders.Top, borders.Bottom);
            Assert.AreEqual(borders.Top, borders.Right);
            Assert.AreEqual(borders.Top, borders.InsideHorizontal);
            Assert.AreEqual(borders.Top, borders.InsideVertical);
            Assert.AreEqual(borders.Top, borders.DiagonalDown);
            Assert.AreEqual(borders.Top, borders.DiagonalUp);
        }

        [TestMethod]
        public void TestChangeLeftBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, leftBorder: border); });

            Assert.AreEqual(borders.Left, oldBorder);
            Assert.AreNotEqual(newBorders.Left, oldBorder);
        }

        [TestMethod]
        public void TestChangeTopBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, topBorder: border); });

            Assert.AreEqual(borders.Top, oldBorder);
            Assert.AreNotEqual(newBorders.Top, oldBorder);
        }

        [TestMethod]
        public void TestChangeRightBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, rightBorder: border); });

            Assert.AreEqual(borders.Right, oldBorder);
            Assert.AreNotEqual(newBorders.Right, oldBorder);
        }

        [TestMethod]
        public void TestChangeBottomBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, bottomBorder: border); });

            Assert.AreEqual(borders.Bottom, oldBorder);
            Assert.AreNotEqual(newBorders.Bottom, oldBorder);
        }

        [TestMethod]
        public void TestChangeInsideHorizontalBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, insideHorizontalBorder: border); });

            Assert.AreEqual(borders.InsideHorizontal, oldBorder);
            Assert.AreNotEqual(newBorders.InsideHorizontal, oldBorder);
        }

        [TestMethod]
        public void TestChangeInsideVerticalBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, insideVerticalBorder: border); });

            Assert.AreEqual(borders.InsideVertical, oldBorder);
            Assert.AreNotEqual(newBorders.InsideVertical, oldBorder);
        }

        [TestMethod]
        public void TestChangeDiagonalDownBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, diagonalDownBorder: border); });

            Assert.IsTrue(borders.DiagonalDown.Equals(oldBorder));
            Assert.AreNotEqual(newBorders.DiagonalDown, oldBorder);
        }

        [TestMethod]
        public void TestChangeDiagonalUpBorderInTableCellBorders()
        {
            TableCellBorders borders = new TableCellBorders(oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder, oldBorder);
            TableCellBorders newBorders = BordersAsserter.AssertTableCellBorders(borders, oldBorder, (Border border) => { return new TableCellBorders(borders, diagonalUpBorder: border); });

            Assert.AreEqual(borders.DiagonalUp, oldBorder);
            Assert.AreNotEqual(newBorders.DiagonalUp, oldBorder);
        }
    }
}
