﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Model.Styles
{
    [TestClass]
    public class TestInheritanceFromDefaultStyleToDocumentElement
    {
        [TestMethod]
        public void TestInheritanceFromDefaultCharacterStyleToDocumentElement()
        {
            // The default character style is not respected by MS Word.

            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            Run run = paragraph.Inlines.AddRun();
            run.Text = "test";

            document.DefaultStyle.CharacterProperties.FontSize.LocalValue = 40;
            document.DefaultStyle.CharacterProperties.UnderlinePattern.LocalValue = UnderlinePattern.Double;

            StylePropertyAsserter.ConfirmStylePropertyValue(
                run.Properties.FontSize,
                document.DefaultStyle.CharacterProperties.FontSize.LocalValue);

            StylePropertyAsserter.ConfirmStylePropertyValue(
                run.Properties.UnderlinePattern,
                document.DefaultStyle.CharacterProperties.UnderlinePattern.LocalValue);
        }

        [TestMethod]
        public void TestInheritanceFromDefaultParagraphStyleToDocumentElement()
        {
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();

            document.DefaultStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Right;

            Style normalStyle = new Style("Normal", StyleType.Paragraph);
            normalStyle.IsDefault = true;
            normalStyle.ParagraphProperties.TextAlignment.LocalValue = Alignment.Center;

            document.StyleRepository.Add(normalStyle);

            StylePropertyAsserter.ConfirmStylePropertyValue(paragraph.Properties.TextAlignment, normalStyle.ParagraphProperties.TextAlignment.LocalValue);
        }

        [TestMethod]
        public void TestInheritanceFromDefaultTableStyleToDocumentElement()
        {
            RadFlowDocument document = new RadFlowDocument();
            Table table = document.Sections.AddSection().Blocks.AddTable();

            Style tableNormalStyle = new Style("TableNormal", StyleType.Table);
            tableNormalStyle.IsDefault = true;
            tableNormalStyle.TableProperties.Alignment.LocalValue = Alignment.Center;

            document.StyleRepository.Add(tableNormalStyle);

            StylePropertyAsserter.ConfirmStylePropertyValue(table.Properties.Alignment, tableNormalStyle.TableProperties.Alignment.LocalValue);
        }
    }
}
