﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
#else
using System.Windows;
#endif

namespace Telerik.Windows.Documents.Flow.Tests.Model.Styles
{
    [TestClass]
    public class TestStylePropertiesValidation
    {
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFontSize()
        {
            Run run = new Run(new RadFlowDocument());
            run.FontSize = -5;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFontWeight()
        {
            Run run = new Run(new RadFlowDocument());
            run.FontWeight = FontWeights.Heavy;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFontStyle()
        {
            Run run = new Run(new RadFlowDocument());
            run.FontStyle = FontStyles.Oblique;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFirstLineIndent()
        {
            Paragraph paragraph = new Paragraph(new RadFlowDocument());
            paragraph.Indentation.FirstLineIndent = -1;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHangingIndent()
        {
            Paragraph paragraph = new Paragraph(new RadFlowDocument());
            paragraph.Indentation.HangingIndent = -1;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestColumnSpan()
        {
            TableCell cell = new TableCell(new RadFlowDocument());
            cell.ColumnSpan = 0;

            cell = new TableCell(new RadFlowDocument());
            cell.ColumnSpan = -1;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRowSpan()
        {
            TableCell cell = new TableCell(new RadFlowDocument());
            cell.RowSpan = 0;

            cell = new TableCell(new RadFlowDocument());
            cell.RowSpan = -1;
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDocumentDefaultTabStopWidth()
        {
            RadFlowDocument document = new RadFlowDocument();
            document.DefaultTabStopWidth = -1;
        }
    }
}
