﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class TableCellTests
    {
        [TestMethod]
        public void CloneWithParameter_WhenInvokedWithNull_ThrowsArgumentNullException()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();
            TableCell tableCell = new TableCell(document1);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<ArgumentNullException>(() => tableCell.Clone(null));
        }

        [TestMethod]
        public void Clone_WithDocument_ClonesTheCellForTheDocument()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableCell tableCell = new TableCell(document1);
            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableCell clonedTableCell = tableCell.Clone(document2);

            // Assert
            Assert.AreNotSame(tableCell, clonedTableCell);
            Assert.AreSame(document2, clonedTableCell.Document);
        }

        [TestMethod]
        public void Clone_WithoutDocument_ClonesTheCellForTheCurrentDocument()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            TableCell tableCell = new TableCell(document);

            // Act
            TableCell clonedTableCell = tableCell.Clone();

            // Assert
            Assert.AreNotSame(tableCell, clonedTableCell);
            Assert.AreSame(document, clonedTableCell.Document);
        }

        [TestMethod]
        public void Clone_WhenPropertiesAreSet_CloningHasSameProperties()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableCell tableCell = new TableCell(document1);
            tableCell.ColumnSpan = 2;
            tableCell.RowSpan = 2;

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableCell clonedTableCell = tableCell.Clone(document2);

            // Assert
            StylePropertyAsserter.AssertPropertiesAreEqual(tableCell.Properties, clonedTableCell.Properties);
        }

        [TestMethod]
        public void Clone_WhenHasChildParagraphs_CloningHasSameStructure()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableCell tableCell = new TableCell(document1);
            tableCell.Blocks.AddParagraph().Inlines.AddRun("some text");
            tableCell.Blocks.AddParagraph().Inlines.AddRun("some text");

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableCell clonedTableCell = tableCell.Clone(document2);

            // Assert
            StructureAsserter.AssertTableCellsAreEqual(tableCell, clonedTableCell);
        }
    }
}
