﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class TableRowTests
    {
        [TestMethod]
        public void CloneWithParameter_WhenInvokedWithNull_ThrowsArgumentNullException()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();
            TableRow tableRow = new TableRow(document1);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<ArgumentNullException>(() => tableRow.Clone(null));
        }

        [TestMethod]
        public void Clone_WithDocument_ClonesTheRowForTheDocument()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableRow tableRow = new TableRow(document1);
            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableRow clonedTableRow = tableRow.Clone(document2);

            // Assert
            Assert.AreNotSame(tableRow, clonedTableRow);
            Assert.AreSame(document2, clonedTableRow.Document);
        }

        [TestMethod]
        public void Clone_WithoutDocument_ClonesTheRowForTheCurrentDocument()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            TableRow tableRow = new TableRow(document);

            // Act
            TableRow clonedTableRow = tableRow.Clone();

            // Assert
            Assert.AreNotSame(tableRow, clonedTableRow);
            Assert.AreSame(document, clonedTableRow.Document);
        }

        [TestMethod]
        public void Clone_WhenPropertiesAreSet_CloningHasSameProperties()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableRow tableRow = new TableRow(document1);
            tableRow.Height = new TableRowHeight(HeightType.Exact, 123);
            tableRow.RepeatOnEveryPage = true;

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableRow clonedTableRow = tableRow.Clone(document2);

            // Assert
            StylePropertyAsserter.AssertPropertiesAreEqual(tableRow.Properties, clonedTableRow.Properties);
        }

        [TestMethod]
        public void Clone_WhenHasChildTableCells_CloningHasSameStructure()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            TableRow tableRow = new TableRow(document1);
            tableRow.Cells.AddTableCell();
            tableRow.Cells.AddTableCell();
            tableRow.Cells.AddTableCell();

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            TableRow clonedTableRow = tableRow.Clone(document2);

            // Assert
            StructureAsserter.AssertTableRowsAreEqual(tableRow, clonedTableRow);
        }
    }
}
