﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;

namespace Telerik.Windows.Documents.Flow.Tests.Model
{
    [TestClass]
    public class TableTests
    {
        [TestMethod]
        public void CloneWithParameter_WhenInvokedWithNull_ThrowsArgumentNullException()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();
            Table table = new Table(document1);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<ArgumentNullException>(() => table.Clone(null));
        }

        [TestMethod]
        public void Clone_WithDocument_ClonesTheTableForTheDocument()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            Table table = new Table(document1);
            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            Table clonedTable = table.Clone(document2);

            // Assert
            Assert.AreNotSame(table, clonedTable);
            Assert.AreSame(document2, clonedTable.Document);
        }

        [TestMethod]
        public void Clone_WithoutDocument_ClonesTheTableForTheCurrentDocument()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Table table = new Table(document);

            // Act
            Table clonedTable = table.Clone();

            // Assert
            Assert.AreNotSame(table, clonedTable);
            Assert.AreSame(document, clonedTable.Document);
        }

        [TestMethod]
        public void Clone_WhenPropertiesAreSet_CloningHasSameProperties()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            Table table = new Table(document1);
            table.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 500);
            table.TableCellPadding = new Primitives.Padding(7);

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            Table clonedTable = table.Clone(document2);

            // Assert
            StylePropertyAsserter.AssertPropertiesAreEqual(table.Properties, clonedTable.Properties);
        }

        [TestMethod]
        public void Clone_WhenHasChildTableRows_CloningHasSameStructure()
        {
            // Arrange
            RadFlowDocument document1 = new RadFlowDocument();

            Table table = new Table(document1);
            table.Rows.AddTableRow();
            table.Rows.AddTableRow();
            table.Rows.AddTableRow();

            RadFlowDocument document2 = new RadFlowDocument();

            // Act
            Table clonedTable = table.Clone(document2);

            // Assert
            StructureAsserter.AssertTablesAreEqual(table, clonedTable);
        }
    }
}
