﻿using System;
using System.Linq;
using System.Text;
#if NETSTANDARD
using Telerik.Documents.Core.Fonts;
using Telerik.Documents.Media;
#else
using System.Windows;
using System.Windows.Media;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.Model.Styles;

namespace Telerik.Windows.Documents.Flow.Tests.TextSearch
{
    [TestClass]
    public abstract class ReplaceTestsBase
    {
        private static readonly ThemableColor foreground = new ThemableColor(Colors.Red);
        private static readonly Color highlightColor = Colors.Yellow;
        private static readonly FontWeight fontWeight = FontWeights.Bold;

        protected Func<Run, bool> runWithCapitalLettersFunc = run => run.Text.Any(c => char.IsUpper(c));

        protected static void AssertDocumentText(RadFlowDocument expected, RadFlowDocument actual)
        {
            string expectedDocumentText = GetDocumentText(expected);
            AssertDocumentText(expectedDocumentText, actual);
        }

        protected static void AssertDocumentText(string expectedDocumentText, RadFlowDocument document)
        {
            string actualDocumentText = GetDocumentText(document);

            Assert.AreEqual(expectedDocumentText, actualDocumentText);
        }

        protected static string GetDocumentText(RadFlowDocument document)
        {
            StringBuilder builder = new StringBuilder();

            foreach (Run run in document.EnumerateChildrenOfType<Run>())
            {
                builder.Append(run.Text);
            }

            return builder.ToString();
        }

        protected static void AssertCharacterPropertiesForRun(RadFlowDocument document, string runText)
        {
            AssertCharacterPropertiesForRun(document, run => run.Text == runText);
        }

        protected static void AssertCharacterPropertiesForRun(RadFlowDocument document, Func<Run, bool> runConditionFunc)
        {
            foreach (Run run in document.EnumerateChildrenOfType<Run>().Where(runConditionFunc))
            {
                Assert.AreEqual(foreground, run.Properties.ForegroundColor.LocalValue);
                Assert.AreEqual(highlightColor, run.Properties.HighlightColor.LocalValue);
                Assert.AreEqual(fontWeight, run.Properties.FontWeight.LocalValue);
            }
        }

        protected static Action<CharacterProperties> GetReplacePropertiesHandler()
        {
            return (properties) =>
            {
                properties.ForegroundColor.LocalValue = foreground;
                properties.HighlightColor.LocalValue = highlightColor;
                properties.FontWeight.LocalValue = fontWeight;
            };
        }
    }
}