﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model.Styles.Core;

namespace Telerik.Windows.Documents.Flow.Tests.Utilities
{
    public class StylePropertiesAsserterInfo
    {
        private List<IStylePropertyDefinition> propertiesToCheck;
        private readonly HashSet<string> propertiesThatShouldNotCompareHasLocalValue;

        public StylePropertiesAsserterInfo()
        {
            this.propertiesThatShouldNotCompareHasLocalValue = new HashSet<string>();
            this.propertiesToCheck = null;
        }

        public void RegisterPropertiesThatShouldNotAssertHasLocalValue(IEnumerable<string> propertyNames)
        {
            foreach (string propertyName in propertyNames)
            {
                this.propertiesThatShouldNotCompareHasLocalValue.Add(propertyName);
            }
        }

        public void RegisterPropertiesToCheck(IEnumerable<IStylePropertyDefinition> propertiesToCheck)
        {
            if (this.propertiesToCheck == null)
            {
                this.propertiesToCheck = new List<IStylePropertyDefinition>();
            }

            this.propertiesToCheck.AddRange(propertiesToCheck);
        }

        // TODO: this method has contriversion naming. Should the property be compared or not ?
        public bool ShouldCompareHasLocalValue(string propertyName)
        {
            return this.propertiesThatShouldNotCompareHasLocalValue.Contains(propertyName);
        }

        // this method is currently used only for StyleId as the property definition is protected and we don't have acces to use it directly.
        public bool ShouldCheckProperty(string propertyName)
        {
            return this.propertiesToCheck == null || this.propertiesToCheck.Any(s => s.PropertyName == propertyName);
        }

        public bool ShouldCheckProperty(IStylePropertyDefinition stylePropertyDefinition)
        {
            return this.propertiesToCheck == null || this.propertiesToCheck.Any(s => s == stylePropertyDefinition);
        }
    }
}
