﻿using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Validation;
using System;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation
{
    internal class XmlValidationHelper
    {
        private const string XmlValidationErrorsFilePath = "Utilities/XmlValidation/XmlValidationErrors.txt";

        private static StringBuilder GetWhiteListErrors()
        {
            StringBuilder whiteList = new StringBuilder();
            using (StreamReader streamReader = new StreamReader(TestHelper.GetResourceStream(XmlValidationErrorsFilePath)))
            {
                String line;
                while ((line = streamReader.ReadLine()) != null)
                {
                    whiteList.AppendLine(line);
                }
            }

            return whiteList;
        }

        //private static void AddErrorToWhiteList(ValidationErrorInfo error)
        //{
        //    StringBuilder whiteList = GetWhiteListErrors();
        //    whiteList.AppendLine(
        //                            error.ErrorType.ToString() + "\t" +                        
        //                            error.Node.ToString() + "\t" +
        //                            error.Part.Uri.ToString() + "\t" +
        //                            error.Description + "\n"
        //                        );

        //    using (StreamWriter streamWriter = new StreamWriter(@"C:\Work\WPF_Scrum\Development\Documents\Flow\Tests\Flow\Utilities\XmlValidation\XmlValidationErrors.txt", false))
        //    {
        //        streamWriter.Write(whiteList.ToString());
        //    }
        //}

        public static bool AssertErrorIsInWhiteList(ValidationErrorInfo error)
        {
            string errors = GetWhiteListErrors().ToString();
            if (errors.Contains(error.Description))
            {
                return true;
            }
            else
            {
                //AddErrorToWhiteList(error);
                return false;
            }
        }

        public static void ExportDocumentToFile(RadFlowDocument document, string filePath)
        {
            DocxFormatProvider provider = new DocxFormatProvider();
            using (Stream stream = new FileStream(Path.Combine(filePath, "Test.docx"), FileMode.OpenOrCreate))
            {
                provider.Export(document, stream);
            }
        }

        public static string GetMainDocumentXmlFromDocument(RadFlowDocument document)
        {
            string mainPartXml = null;
            string fileDirectory = Directory.GetCurrentDirectory();

            try
            {
                XmlValidationHelper.ExportDocumentToFile(document, fileDirectory);

                using (WordprocessingDocument wordprocessingDocument = WordprocessingDocument.Open(Path.Combine(fileDirectory, "Test.docx"), false))
                {
                    OpenXmlPart mainpart = wordprocessingDocument.MainDocumentPart;

                    using (StreamReader streamReader = new StreamReader(mainpart.GetStream()))
                    {
                        mainPartXml = streamReader.ReadToEnd();
                    }

                    wordprocessingDocument.Close();
                }
            }
            finally
            {
                File.Delete(Path.Combine(fileDirectory, "Test.docx"));
            }

            return mainPartXml;
        }
    }
}
