﻿using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Validation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.Tests.Utilities.XmlValidation
{
    internal static class XmlValidator
    {
        private static readonly string fileDirectory = Directory.GetCurrentDirectory();

        public static void ValidateOpenXmlDocument(RadFlowDocument document)
        {
            try
            {
                StringBuilder stringBuilder = new StringBuilder("Errors:\n");
                XmlValidationHelper.ExportDocumentToFile(document, fileDirectory);

                using (WordprocessingDocument wordprocessingDocument = WordprocessingDocument.Open(Path.Combine(fileDirectory, "Test.docx"), false))
                {
                    int count = 0;
                    OpenXmlValidator validator = new OpenXmlValidator();

                    foreach (ValidationErrorInfo error in validator.Validate(wordprocessingDocument))
                    {
                        if (XmlValidationHelper.AssertErrorIsInWhiteList(error))
                        {
                            continue;
                        }
                        else
                        {
                            count++;
                            stringBuilder.AppendLine(
                                    error.ErrorType.ToString() + "\t" +
                                    error.Node.ToString() + "\t" +
                                    error.Part.Uri.ToString() + "\t" +
                                    error.Description + "\n"
                                );
                        }
                    }

                    wordprocessingDocument.Close();
                    Assert.AreEqual(0, count, stringBuilder.ToString());
                }
            }
            finally
            {
                File.Delete(Path.Combine(fileDirectory, "Test.docx"));
            }
        }
    }
}
