﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public static class AssertVisualTest
    {
        public static void AssertAreVisualyEqualDocx(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            DocxFormatProvider provider = new DocxFormatProvider();
            byte[] data = provider.Export(document);
            VisualTestHelper.Assert(data, FileExtenssion.DOCXFILEEXTENSION, rectangle);
#endif
        }

        public static void AssertAreVisualyEqualPdf(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            byte[] data = provider.Export(document);
            VisualTestHelper.Assert(data, FileExtenssion.PDFFILEEXTENSION, rectangle);
#endif
        }

        public static void AssertAreVisualyEqualHtml(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            HtmlFormatProvider provider = new HtmlFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            VisualTestHelper.Assert(data, FileExtenssion.HTMLFILEEXTENSION, rectangle);
#endif
        }

        public static void AssertAreVisualyEqualRtf(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            RtfFormatProvider provider = new RtfFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            VisualTestHelper.Assert(data, FileExtenssion.RTFFILEEXTENSION, rectangle);
#endif
        }


        public static void AssertAreVisualyEqualDocx(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            DocxFormatProvider provider = new DocxFormatProvider();
            byte[] data = provider.Export(document);
            VisualTestHelper.Assert(data, FileExtenssion.DOCXFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }


        public static void AssertAreVisualyEqualPdf(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            ExportFlowToFixeddocument(document, action, repeatActionCounter, actions, provider);
#endif
        }

        internal static void AssertAreVisualyEqualPdf(RadFlowDocument document, PdfExportSettings settings, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            provider.ExportSettings = settings;
            ExportFlowToFixeddocument(document, action, repeatActionCounter, actions, provider);
#endif
        }

        public static void AssertAreVisualyEqualRtf(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            RtfFormatProvider provider = new RtfFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            VisualTestHelper.Assert(data, FileExtenssion.RTFFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }

        public static void AssertAreVisualyEqualHtml(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            HtmlFormatProvider provider = new HtmlFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            VisualTestHelper.Assert(data, FileExtenssion.HTMLFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }

        private static void ExportFlowToFixeddocument(RadFlowDocument document, Action action, int repeatActionCounter, Action[] actions, PdfFormatProvider provider)
        {
            byte[] data = provider.Export(document);
            VisualTestHelper.Assert(data, FileExtenssion.PDFFILEEXTENSION, action, repeatActionCounter, 4000, actions);
        }
    }
}
