﻿using ArtOfTest.Common.Win32;
using ArtOfTest.WebAii.Win32.Dialogs;
using System;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public static class DialogManager
    {
        public static void HandleDialog(string processName, string documentName)
        {
            switch (processName)
            {
                case TestSettings.WORDPROCESS:
                    HandleWordsDilaogs(documentName);
                    break;
                case TestSettings.CSVFILEEXTENSION:
                case TestSettings.EXCELPROCESS:
                    HandleExcelDilaogs(documentName);
                    break;
                default:
                    break;
            }
        }

        private static void HandleExcelDilaogs(string documentName)
        {
            Window excelProcess = FindDilaogByOwnerProcess(DialogMonitor.GetAllDialogs(), TestSettings.EXCELPROCESS);

            if (excelProcess != null && excelProcess.AllChildren != null)
            {
                foreach (Window window in excelProcess.AllChildren)
                {
                    if (window.GetWindowText().Contains("Want to save changes to"))
                    {
                        Window saveDilaog = FindDilaogByCaption(excelProcess.AllChildren, DialogElement.DontSave, true);
                        ClickOnElement(saveDilaog, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleExcelDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains(@"Safe mode could help you troubleshoot the problem, but some features might not be available in this mode"))
                    {
                        Window child = FindDilaogByCaption(excelProcess.AllChildren, DialogElement.No, true);
                        ClickOnElement(child, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleExcelDilaogs(documentName);
                    }
                }
            }
        }

        private static string GetCaption(string processName)
        {
            Process process = Process.GetProcessesByName(processName).FirstOrDefault();

            return process.MainWindowTitle;
        }

        private static void HandleWordsDilaogs(string documentName)
        {
            Window wordProcess = FindDilaogByOwnerProcess(DialogMonitor.GetAllDialogs(), TestSettings.WORDPROCESS);

            if (wordProcess != null && wordProcess.AllChildren != null)
            {
                foreach (Window window in wordProcess.AllChildren)
                {
                    if (window.GetWindowText().Contains(@"Safe mode could help you troubleshoot the problem, but some features might not be available in this mode"))
                    {
                        Window child = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.No, true);
                        ClickOnElement(child, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains("The last time you opened '" + documentName + "', it caused a serious error. Do you still want to open"))
                    {
                        Window child = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.Yes, true);
                        ClickOnElement(child, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains("The document '" + documentName + "' caused a serious error the last time it was opened.  You may continue opening it or perform data recovery by clicking 'Recover Data'.  Clicking 'Recover Data' will extract text out of your document but all formatting and pictures will be lost."))
                    {
                        Window open1 = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.Open, true);
                        ClickOnElement(open1, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains("This file cannot be opened by using Microsoft Word. Do you want to search Office.com for a converter that can open the file?"))
                    {
                        Window open1 = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.Open, true);
                        ClickOnElement(open1, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains("This document contains fields that may refer to other files. Do you want to update the fields in this document?"))
                    {
                        Window open1 = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.Yes, true);
                        ClickOnElement(open1, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }

                    if (window.GetWindowText().Contains("Want to save changes to"))
                    {
                        Window saveDilaog = FindDilaogByCaption(wordProcess.AllChildren, DialogElement.DontSave, true);
                        ClickOnElement(saveDilaog, UIInteractionHelper.LeftMouseClick);
                        Thread.Sleep(500);

                        HandleWordsDilaogs(documentName);
                    }
                }
            }
        }

        private static Window FindDilaogByOwnerProcess(WindowCollection dialogs, string processName)
        {
            Window window = null;
            foreach (Window dialog in dialogs)
            {
                if (dialog.OwnerProcess.ProcessName == processName)
                {
                    window = dialog;
                }
            }

            return window;
        }

        private static Window FindDilaogByCaption(WindowCollection dialogs, string content, bool shouldSetActiveDialog)
        {
            Window window = null;
            foreach (Window dialog in dialogs)
            {
                if (dialog.Caption == content)
                {
                    if (shouldSetActiveDialog)
                    {
                        dialog.SetActive();
                    }

                    window = dialog;

                    break;
                }
            }

            return window;
        }

        private static void ClickOnElement(Window element, Func<int, int, Action> action)
        {
            element.SetFocus();
            var rect = element.Rectangle;
            action(rect.X + 5, rect.Y + 5)();
        }

        private class DialogElement
        {
            public static string Yes = "&Yes";
            public static string No = "&No";
            public static string Open = "&Open";
            public static string DontSave = "&Don't Save";

        }
    }
}
