﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Common.FormatProviders;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using System.IO;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration
{
    public class DocumentFlowVisualBaseTest : IntegrationTestClassBase
    {
#if WordsProcessing
        /// <summary>
        /// Export document to all formats 
        /// </summary>
        /// <param name="document"></param>
        /// <param name="pageDown"></param>
        protected virtual void AssertVisual(RadFlowDocument document, int pageDown = 0)
        {
            if (pageDown == 0)
            {
                this.AssertVisual(document, new string[] { HTMLFILEEXTENSION, DOCXFILEEXTENSION, PDFFILEEXTENSION, RTFFILEEXTENSION });
            }
            else
            {
                this.AssertVisual(document, new string[] { HTMLFILEEXTENSION, DOCXFILEEXTENSION, PDFFILEEXTENSION, RTFFILEEXTENSION }, UIInteractionHelper.PageDown(), pageDown, null);
            }
        }

        protected virtual void AssertVisual(RadFlowDocument document, string[] extenssions, System.Drawing.Rectangle screenRectangle)
        {
            for (int i = 0; i < extenssions.Length; i++)
            {
                switch (extenssions[i])
                {
                    case PDFFILEEXTENSION: this.AssertAreVisualyEqualPdf(document, screenRectangle); break;
                    case DOCXFILEEXTENSION: this.AssertAreVisualyEqualDocx(document, screenRectangle); break;
                    //case HTMLFILEEXTENSION: this.AssertAreVisualyEqualHtml(document, null); break;
                    case RTFFILEEXTENSION: this.AssertAreVisualyEqualRtf(document, screenRectangle); break;
                    default:
                        break;
                }
            }
            this.Setrectangle(screenRectangle);
            this.AssertWithWordPad(document, new string[] { RTFFILEEXTENSION });
        }

        /// <summary>
        /// Asserts test like export to desired file format
        /// </summary>
        /// <param name="document"></param>
        /// <param name="extenssions">DOCXFILEEXTENSSION,PDFFILEEXTENSION,HTMLFILEEXTENSSION.</param>
        /// <param name="actions">UIInteractionHelper.</param>
        /// <param name="repeat">Repeat action. Must be grater than 0</param>
        protected virtual void AssertVisual(RadFlowDocument document, string[] extenssions, Action action, int repeat, Action[] preDefinedActions = null)
        {
            if (action != null && repeat <= 0)
            {
                throw new ArgumentOutOfRangeException("repeat action must be positive number");
            }

            for (int i = 0; i < extenssions.Length; i++)
            {
                switch (extenssions[i])
                {
                    case PDFFILEEXTENSION: this.AssertAreVisualyEqualPdf(document, action, repeat, preDefinedActions); break;
                    case DOCXFILEEXTENSION: this.AssertAreVisualyEqualDocx(document, action, repeat, preDefinedActions); break;
                    //case HTMLFILEEXTENSION: this.AssertAreVisualyEqualHtml(document, action, repeat, preDefinedActions); break;
                    case RTFFILEEXTENSION: this.AssertAreVisualyEqualRtf(document, action, repeat, preDefinedActions); break;
                    default:
                        break;
                }
            }

            this.AssertWithWordPad(document, new string[] { RTFFILEEXTENSION }, action, repeat, preDefinedActions);
        }

        protected virtual void AssertVisual(RadFlowDocument document, string[] extenssions, bool assertWithWordPad = true)
        {
            for (int i = 0; i < extenssions.Length; i++)
            {
                switch (extenssions[i])
                {
                    case PDFFILEEXTENSION: this.AssertAreVisualyEqualPdf(document, null, 0, null); break;
                    case DOCXFILEEXTENSION: this.AssertAreVisualyEqualDocx(document, null, 0, null); break;
                    //case HTMLFILEEXTENSION: this.AssertAreVisualyEqualHtml(document, null, 0, null); break;
                    case RTFFILEEXTENSION: this.AssertAreVisualyEqualRtf(document, null, 0, null); break;
                    default:
                        break;
                }
            }
            if (assertWithWordPad)
            {
                this.AssertWithWordPad(document, new string[] { RTFFILEEXTENSION }, null, 0, null);
            }
        }

        protected void AssertWithWordPad(RadFlowDocument document, string[] extenssions, Action action = null, int repeat = 0, Action[] preDefinedActions = null)
        {
#if VISUAL
            foreach (var extenssion in extenssions)
            {
                switch (extenssion)
                {
                    case RTFFILEEXTENSION:
                        RtfFormatProvider rtfProvider = new RtfFormatProvider();
                        byte[] rtfData = Encoding.UTF8.GetBytes(rtfProvider.Export(document));
                        this.VisualTestHelper.AssertWordPad(rtfData, extenssion, action, repeat, 4000, preDefinedActions);
                        break;
                    case DOCXFILEEXTENSION:
                        DocxFormatProvider docxProvider = new DocxFormatProvider();
                        byte[] docxData = docxProvider.Export(document);
                        this.VisualTestHelper.AssertWordPad(docxData, extenssion, action, repeat, 4000, preDefinedActions);
                        break;
                    default:
                        throw new NotSupportedException("This format provider is not supported for WordPad");
                }
            }
#endif
        }

        protected void AssertAreVisualyEqualPdf(RadFlowDocument document, PdfExportSettings settings, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            provider.ExportSettings = settings;
            this.ExportFlowToFixedDocument(document, action, repeatActionCounter, actions, provider);
#endif
        }

        private void AssertAreVisualyEqualDocx(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            DocxFormatProvider provider = new DocxFormatProvider();
            byte[] data = provider.Export(document);
            this.Setrectangle(rectangle);
            this.VisualTestHelper.Assert(data, DOCXFILEEXTENSION, null, 0);
#endif
        }

        private void AssertAreVisualyEqualPdf(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            byte[] data = provider.Export(document);
            this.Setrectangle(rectangle);
            this.VisualTestHelper.Assert(data, PDFFILEEXTENSION, null, 0);
#endif
        }

        private void AssertAreVisualyEqualHtml(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            HtmlFormatProvider provider = new HtmlFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            this.Setrectangle(rectangle);
            this.VisualTestHelper.Assert(data, HTMLFILEEXTENSION, null, 0);
#endif
        }

        private void AssertAreVisualyEqualRtf(RadFlowDocument document, System.Drawing.Rectangle? rectangle = null)
        {
#if VISUAL
            RtfFormatProvider provider = new RtfFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            this.Setrectangle(rectangle);
            this.VisualTestHelper.Assert(data, RTFFILEEXTENSION, null, 0);
#endif
        }

        private void AssertAreVisualyEqualDocx(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            DocxFormatProvider provider = new DocxFormatProvider();
            byte[] data = provider.Export(document);
            this.VisualTestHelper.Assert(data, DOCXFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }

        private void AssertAreVisualyEqualPdf(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            PdfFormatProvider provider = new PdfFormatProvider();
            this.ExportFlowToFixedDocument(document, action, repeatActionCounter, actions, provider);
#endif
        }

        private void AssertAreVisualyEqualRtf(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            RtfFormatProvider provider = new RtfFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            this.VisualTestHelper.Assert(data, RTFFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }

        private void AssertAreVisualyEqualHtml(RadFlowDocument document, Action action, int repeatActionCounter = 1, params Action[] actions)
        {
#if VISUAL
            HtmlFormatProvider provider = new HtmlFormatProvider();
            byte[] data = Encoding.UTF8.GetBytes(provider.Export(document));
            this.VisualTestHelper.Assert(data, HTMLFILEEXTENSION, action, repeatActionCounter, 4000, actions);
#endif
        }

        private void ExportFlowToFixedDocument(RadFlowDocument document, Action action, int repeatActionCounter, Action[] actions, PdfFormatProvider provider)
        {
            byte[] data = provider.Export(document);
            this.VisualTestHelper.Assert(data, PDFFILEEXTENSION, action, repeatActionCounter, 4000, actions);
        }

        protected RadFlowDocument LoadVisualTestDocument(string documentName, IFormatProvider<RadFlowDocument> provider)
        {
            var source = VisualTestHelper.LoadVisualTestDocumentFromStream(documentName);
            RadFlowDocument document = provider.Import(source);

            return document;
        }

        private void Setrectangle(System.Drawing.Rectangle? rectangle)
        {
            if (rectangle != null)
            {
                this.VisualTestHelper.DefaultRectangle = new System.Drawing.Rectangle(rectangle.Value.X, rectangle.Value.Y, rectangle.Value.Width, rectangle.Value.Height);
            }
            else
            {
                this.VisualTestHelper.DefaultRectangle = new System.Drawing.Rectangle(0, 0, 0, 0);
            }
        }
#endif
    }
}
