﻿using System;
using System.Linq;
using System.Diagnostics;
using System.IO;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using System.Threading;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;
#if SpreadsheetStreaming
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
#endif
namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public class DocumentLauncher
    {
        private const string wordPadFullPath = @"C:\Program Files\Windows NT\Accessories\wordpad.exe";

        private int documentsCounter;
        private string currentDocumentFileExtension;
        public Process process;
        private ProcessStartInfo processStartInfo;
        public DocumentLauncher(string testName)
        {
            this.documentsCounter = 0;
            this.TestName = testName;
        }

        public string TestName { get; set; }
        public string Caption { get; set; }
        public bool ShouldUseWordPadProcess { get; set; }

        public Process ExportAndLoadDocument(byte[] data, string fileExtension, int wait = 6000, params Action[] actions)
        {
            this.currentDocumentFileExtension = fileExtension;

            this.Caption = this.TestName + "_" + this.documentsCounter + fileExtension;

            string fileName = FolderSettings.DplDocumentsTempFolder + this.Caption;

            // Add " - Adobe Acrobat Reader DC" to the Caption to ensure that later the process closes not the tab
            // but the Window.
            // Adobe Reader DC adds its name " - Adobe Acrobat Reader DC" to the title of the window.
            //if (currentDocumentFileExtenssion == TestSettings.PDFFILEEXTENSION)
            //{
            //    this.Caption = this.Caption + " - Adobe Acrobat Reader DC";
            //}

            //else if (currentDocumentFileExtenssion == TestSettings.CSVFILEEXTENSION || currentDocumentFileExtenssion == TestSettings.XLSXFILEEXTENSION)
            //{
            //    this.Caption = this.Caption + " - Excel";
            //}

            File.WriteAllBytes(fileName, data);
            this.process = null;

            if (this.currentDocumentFileExtension == TestSettings.HTMLFILEEXTENSION)
            {
                IntegrationTestClassBase.StartIEBrowser(fileName);
            }
            else
            {
                if (this.ShouldUseWordPadProcess)
                {
                    this.processStartInfo = new ProcessStartInfo(wordPadFullPath);
                    this.processStartInfo.Arguments = fileName;
                }
                else
                {
                    this.processStartInfo = new ProcessStartInfo(fileName);
                }
                this.processStartInfo.WindowStyle = ProcessWindowStyle.Maximized;
                this.processStartInfo.ErrorDialog = false;

                System.Threading.Tasks.Parallel.Invoke(
                                                    () => this.GetProcessStarted(),
                                                    () => this.KillCurrentProcess());
                if (process == null)
                {
                    Microsoft.VisualStudio.TestTools.UnitTesting.Assert.Fail("Document " + this.Caption + " failed to load.");
                }

                this.EnsureProcessMainWindowIsLoaded(wait);
            }

            this.VerifyDialogs();
            this.ExecuteAfterLoadingDocumentActions(actions);

            this.documentsCounter++;

            return process;
        }

        public void VerifyDialogs()
        {
            if (this.process != null)
            {
                DialogManager.HandleDialog(this.process.ProcessName, this.Caption);
            }
        }

        public Process ExportAndLoadDocument(string fileName, string fileExtension, int wait = 3000)
        {
            this.currentDocumentFileExtension = fileExtension;
            this.Caption = this.TestName + "_" + this.documentsCounter + fileExtension;

            this.process = null;

            if (this.currentDocumentFileExtension == TestSettings.HTMLFILEEXTENSION)
            {
                IntegrationTestClassBase.StartIEBrowser(fileName);
            }
            else
            {
                if (this.ShouldUseWordPadProcess)
                {
                    this.processStartInfo = new ProcessStartInfo(wordPadFullPath);
                    this.processStartInfo.Arguments = fileName;
                }
                else
                {
                    this.processStartInfo = new ProcessStartInfo(fileName);
                }
                this.processStartInfo.WindowStyle = ProcessWindowStyle.Maximized;
                this.processStartInfo.ErrorDialog = false;

                System.Threading.Tasks.Parallel.Invoke(
                                                    () => this.GetProcessStarted(),
                                                    () => this.KillCurrentProcess());
                if (process == null)
                {
                    Microsoft.VisualStudio.TestTools.UnitTesting.Assert.Fail("Document " + this.Caption + " failed to load.");
                }
            }

            if (this.process != null)
            {
                DialogManager.HandleDialog(this.process.ProcessName, this.Caption);
            }

            this.documentsCounter++;
            System.Threading.Thread.Sleep(wait);

            return process;
        }

        private void EnsureProcessMainWindowIsLoaded(int wait)
        {
            Process[] processes = Process.GetProcessesByName(this.process.ProcessName);
            int delay = 250;

            if (this.process.MainWindowHandle.ToInt32() == 0)
            {
                while (delay <= wait && (this.process.MainWindowHandle.ToInt32() == 0 || !this.process.MainWindowTitle.Contains(this.Caption)))
                {
                    System.Threading.Thread.Sleep(delay);

                    foreach (var pr in processes)
                    {
                        pr.Refresh();
                    }

                    processes = Process.GetProcessesByName(this.process.ProcessName);

                    this.process = FindProcess(processes, this.currentDocumentFileExtension);
                    // Depending on the ending of the title the Caption is properly set so the later logic for finding the window is not changed
                    this.Caption = this.process.MainWindowTitle;
                    Console.WriteLine("The process has loaded : " + this.process.MainWindowHandle.ToInt32());

                    delay += 250;
                }

                if (this.process.MainWindowHandle.ToInt32() == 0)
                {
                    Microsoft.VisualStudio.TestTools.UnitTesting.Assert.Fail("Document " + this.Caption + " failed to load.");
                }
            }
        }

        private static Process FindProcess(Process[] processes, string fileExtension)
        {
            Process newProcess;

            // Assure that the appropriate window is found depending on the window's title. 
            // Windows 10 adds the application's name at the end of the title
            // For instance: A file named "Document 1" open with Adobe Reader DC, the title of the window will be
            // "Document 1 - Adobe Acrobat Reader DC" and "Document 1 - Word" if open with Word.

            switch (fileExtension)
            {
                case TestSettings.PDFFILEEXTENSION:
                    newProcess = processes.Where(x => x.MainWindowTitle.EndsWith(TestSettings.PdfProcessWindowCaptionSuffix)).FirstOrDefault();
                    break;
                case TestSettings.XLSXFILEEXTENSION:
                case TestSettings.CSVFILEEXTENSION:
                    newProcess = processes.Where(x => x.MainWindowTitle.EndsWith(TestSettings.ExcelProcessWindowCaptionSuffix)).FirstOrDefault();
                    break;
                case TestSettings.DOCXFILEEXTENSION:
                case TestSettings.RTFFILEEXTENSION:
                    newProcess = processes.Where(x => x.MainWindowTitle.EndsWith(TestSettings.WordPadProcessWindowCaptionSuffix) ||
                    x.MainWindowTitle.EndsWith(TestSettings.WordProcessWindowCaptionSuffix)).FirstOrDefault();
                    break;
                default:
                    throw new Exception(string.Format("Cannot find process for {0}!", fileExtension));
            }

            return newProcess;
        }

        private void ExecuteAfterLoadingDocumentActions(Action[] actions)
        {
            if (actions != null && actions.Length > 0 && process.MainWindowTitle != this.Caption)
            {
                for (int i = 0; i < actions.Length; i++)
                {
                    actions[i].Invoke();
                    Thread.Sleep(250);
                    if (i == actions.Length - 1)
                    {
                        Thread.Sleep(650);
                    }
                }
            }
        }

        private void GetProcessStarted()
        {
            this.process = Process.Start(this.processStartInfo);
            Thread.Sleep(1000);
        }

        private void KillCurrentProcess(int wait = 3000)
        {
            Thread.Sleep(wait);
            int delay = 0;

            while (this.process == null || delay >= 60000)
            {
                Thread.Sleep(1000);
                delay += 1000;
            }

            if (this.process == null)
            {
                ProcessKiller.KillProcess(this.currentDocumentFileExtension, this.Caption);
                this.Caption = string.Empty;
            }
        }

    }
}
