﻿#if SpreadsheetStreaming
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
#endif
using System;

using System.IO;
using System.Linq;
using System.Reflection;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;
using System.Text;
using System.Diagnostics;

namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public class FolderSettings
    {
        private const string IMAGES_CONTAINER_FOLDER = @"Images";
        private const string IMAGE_VERIFICATION_RESULTS_FOLDER = @"ImageVerificationResults\";
        private const string VISUAL_TESTS_DOCUMENTS_FOLDER = @"VisualTestsDocuments";
        private const string SOURCES = "Sources";
        private const string DOCUMENTS = "Documents";
        private const string TEST_RESULTS = "TestResults";
        private const string BUILD_PC_OUTPUT_RESULTS = "DPTEAMPC2";
        internal const string TEST_RUN_TYPE_LOCAL = @"LocalRun\";
        internal const string TEST_RUN_TYPE_BUILD = @"BuildRun\";

        private static string platform = "";
        public static string ExpectedImagesStorageFolder { get; set; }
        public static string ImageVerificationResultsFolder { get; set; }
        public static string VisualTestsDocuments { get; set; }
        public static string Control { get; set; }
        public static string ResultDirName { get; set; }
        public static string TeamBuildResultsFolder = @"\\" + BUILD_PC_OUTPUT_RESULTS + @"\" + @"TeamBuildResults\";

        /// <summary>
        /// TestRunType can be Local run or Build run.
        /// </summary>
        public static string TestRunType { get; set; }

        public static string DplDocumentsTempFolder = TestSettings.DPL_TEMP_FOLDER;
        public static string ImagesTempFolder = TestSettings.IMAGES_TEMP_FOLDER;
        public static string VideosTempFolder = TestSettings.VIDEOS_TEMP_FOLDER;
        public static string TestsTempFolder = TestSettings.TESTS_TEMP_FOLDER;


        public static StringBuilder SettingsInfo { get; set; }

        static FolderSettings()
        {
            InitializeFolders();
        }

        private static void InitializeFolders()
        {
            //var buildMachine = @"\\" + Environment.MachineName + @"\";
            var buildMachine = @"\\" + BUILD_PC_OUTPUT_RESULTS + @"\"; // Test Build PC where all the images are stored
            ImageVerificationResultsFolder = buildMachine + IMAGE_VERIFICATION_RESULTS_FOLDER;

#if WPF
            platform = @"\WPF";
#elif SL || SILVERLIGHT
            platform = @"\SL";
#else
            platform = string.Empty;
#endif
            Control = string.Empty;
#if WordsProcessing
            Control = "WordProcessing";
#endif
#if PdfProcessing
            Control = "PdfProcessing";
#endif
#if SpreadsheetStreaming
            Control = "SpreadStreamProcessing";
#endif
            ExpectedImagesStorageFolder = CreateExpectedImageFolder(buildMachine, IMAGES_CONTAINER_FOLDER, platform, Control);
            VisualTestsDocuments = GetVisualTestsDocumentsFolder();
            ResultDirName = FolderSettings.DplDocumentsTempFolder;

        }

        private static string GetVisualTestsDocumentsFolder()
        {
            var currentDir = Directory.GetCurrentDirectory();
#if !DEBUG
             // When Functional Build is ran, the name of the current dir is TestResults 
            var parentDir = GetParentFolder("TestResults", currentDir); 
#else
            // When test is ran locally, the current dir is ...bin\WPF\4.0\Debug
            var parentDir = GetParentFolder(DOCUMENTS, currentDir);
#endif
            return parentDir + "\\" + DOCUMENTS + "\\" + VISUAL_TESTS_DOCUMENTS_FOLDER + "\\";
        }

        // This method is used for debugging purposes.
        internal static string GenerateSettingsInfo()
        {
            SettingsInfo = new StringBuilder();
            SettingsInfo.Append("Environment.MachineName: " + Environment.MachineName + Environment.NewLine);
            SettingsInfo.Append("ExpectedImagesStorageFolder: " + ExpectedImagesStorageFolder + Environment.NewLine);
            SettingsInfo.Append("VisualTestsDocuments: " + VisualTestsDocuments + Environment.NewLine);
            SettingsInfo.Append("TeamBuildResultsFolder: " + TeamBuildResultsFolder + Environment.NewLine);
            SettingsInfo.Append("ResultDirName: " + ResultDirName + Environment.NewLine);
            SettingsInfo.Append("TestRunType: " + TestRunType + Environment.NewLine);
            SettingsInfo.Append("ImagesTempFolder: " + FolderSettings.ImagesTempFolder + Environment.NewLine);
            SettingsInfo.Append("VideosTempFolder: " + FolderSettings.VideosTempFolder + Environment.NewLine);
            return SettingsInfo.ToString();
        }

        private static string CreateExpectedImageFolder(string buildMachine, string imagesFolder, string platform, string folderName)
        {
            return buildMachine + imagesFolder + platform + @"\DPL\" + folderName + @"\";
        }
        
        private static string GetParentFolder(string folderName, string currentFolder)
        {
            DirectoryInfo dir = new DirectoryInfo(currentFolder);

            var subDir = dir;

            var directory = subDir;

            while (directory.Name != folderName)
            {
                directory = subDir;
                subDir = subDir.Parent;

                if (directory.Name == dir.Root.Name)
                {
                    return "";
                }
            }

            return directory.Parent.FullName;
        }

        private static string GetApplicationPhysicalFolder()
        {
            var testAssemblyPath = Assembly.GetExecutingAssembly().Location;
            var testAssemblyFolder = System.IO.Path.GetDirectoryName(testAssemblyPath);

            return testAssemblyFolder;
        }
    }
}
