﻿using ArtOfTest.Common.Win32;
using ArtOfTest.WebAii.Controls.Xaml;
using ArtOfTest.WebAii.Silverlight;
using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Threading;

namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public class ImageComparer
    {
        public static Bitmap CaptureImage(System.Drawing.Rectangle rectangle)
        {
            var image = CreateScreenImage(rectangle);

            return image;
        }

        public static Bitmap GetBitMapRectangleByCaption(string caption, System.Drawing.Rectangle bounds)
        {
            Window windowRecur = GetWindow(caption);

            Bitmap picture = windowRecur.GetBitmap(bounds);

            return picture;
        }

        public static System.Drawing.Rectangle GetSceenRectOfDesktop()
        {
            var window = Extern.GetForegroundWindow();
            var wait = 0;
            while (window.ToInt32() == 0 && wait < 2000)
            {
                Thread.Sleep(150);
                window = Extern.GetForegroundWindow();
                wait += 150;
            }

            Rect rect = new Rect();
            Extern.GetWindowRect(window, ref rect);

            Rectangle visualRectangle = CalculateRectangle(rect);

            return visualRectangle;
        }

        public static System.Drawing.Rectangle GetScreenRectangleByCaption(string caption)
        {
            var vsWindowRecur = GetWindow(caption);

            return vsWindowRecur.Rectangle;
        }

        public static System.Drawing.Rectangle GetScreenRectangleByCaption(string caption, System.Drawing.Rectangle bounds)
        {
            var vsWindowRectangle = GetScreenRectangleByCaption(caption);

            return CalculateRectangle(vsWindowRectangle, bounds);
        }

        public static System.Drawing.Rectangle GetScreenRectangleOfFramworkelement(IApplication app, IFrameworkElement element)
        {
            var rect = app.GetScreenRectangle((FrameworkElement)element);
            return rect;
        }

        public static System.Drawing.Rectangle GetScreenRectangleOfProcess(string procName)
        {
            Process[] processes = Process.GetProcessesByName(procName);
            Process proc = processes.Where(x => x.MainWindowHandle.ToInt32() > 0).FirstOrDefault();

            var rect = GetScreenRectangleOfProcess(proc);

            return rect;
        }

        public static System.Drawing.Rectangle GetScreenRectangleOfProcess(string procName, System.Drawing.Rectangle bounds)
        {
            System.Drawing.Rectangle rect = GetScreenRectangleOfProcess(procName);

            System.Drawing.Rectangle rectangle = CalculateRectangle(rect, bounds);

            return rectangle;
        }

        public static System.Drawing.Rectangle GetScreenRectangleOfProcess(Process process)
        {
            Process[] processes = Process.GetProcessesByName(process.ProcessName);

            int wait = 250;

            if (process.MainWindowHandle.ToInt32() == 0)
            {
                while (wait < 4000 && process.MainWindowHandle.ToInt32() == 0)
                {
                    System.Threading.Thread.Sleep(250);

                    foreach (var pr in processes)
                    {
                        pr.Refresh();
                    }

                    processes = Process.GetProcessesByName(process.ProcessName);
                    var newProcess = processes.Where(x => x.MainWindowHandle.ToInt32() > 0).FirstOrDefault();
                    if (newProcess != null)
                    {
                        process = newProcess;
                        Console.WriteLine(" GetScreenRectangleOfProcess: " + newProcess.MainWindowHandle.ToInt32());
                    }
                    wait += 250;
                }
            }

            if (process == null)
            {
                throw new ArgumentNullException(String.Format("Process of {0} is not found!", process.ProcessName));
            }

            IntPtr ptr = process.MainWindowHandle;
            Rect rect = new Rect();
            Extern.GetWindowRect(ptr, ref rect);

            System.Drawing.Rectangle rectangle = CalculateRectangle(rect);
            Console.WriteLine("{0}{1}{2}{3}", rectangle.X, rectangle.Y, rectangle.Width, rectangle.Height);
            return rectangle;
        }

        public static System.Drawing.Rectangle GetScreenRectangleOfProcess(Process process, System.Drawing.Rectangle visualRectangle)
        {
            var rect = GetScreenRectangleOfProcess(process);

            System.Drawing.Rectangle rectangle = CalculateRectangle(rect, visualRectangle);

            return rectangle;
        }

        public static Bitmap GetSavedImage(string imageFullName)
        {
            Bitmap exp = new Bitmap(imageFullName);

            return exp;
        }

        public static bool Compare(Bitmap expectedImage, Bitmap newActualImage, out Bitmap container, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance, string imageName)
        {
            var result = ImageComparerResult(expectedImage, newActualImage, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance, out container, imageName);

            return result;
        }

        private static Window GetWindow(string caption)
        {
            Window vsWindowRecur = WindowManager.FindWindowRecursively(IntPtr.Zero, caption, true, 10);
            int delay = 0;

            while (vsWindowRecur == null && delay < 3000)
            {
                vsWindowRecur = WindowManager.FindWindowRecursively(IntPtr.Zero, caption, true, 10);
                delay += 250;
            }

            if (vsWindowRecur == null)
            {
                throw new ArgumentNullException(string.Format("window of process {0} is not found", caption));
            }

            vsWindowRecur.WaitForVisibility(false, 2000);
            vsWindowRecur.Maximize();
            vsWindowRecur.SetActive();
            vsWindowRecur.SetFocus();
            Thread.Sleep(250);

            return vsWindowRecur;
        }

        private static Rectangle CalculateRectangle(Rect rect)
        {
            Rectangle visualRectangle = new Rectangle(rect.Left, rect.Top, rect.Right - rect.Left, rect.Bottom - rect.Top);

            return visualRectangle;
        }

        private static Rectangle CalculateRectangle(Rect rect, System.Drawing.Rectangle rectangle)
        {
            Rectangle visualRectangle = new Rectangle(rect.Left + rectangle.X, rect.Top + rectangle.Y, rect.Right - rect.Left - rectangle.Width, rect.Bottom - rect.Top - rectangle.Height);

            return visualRectangle;
        }

        private static Rectangle CalculateRectangle(System.Drawing.Rectangle rect, System.Drawing.Rectangle rectangle)
        {
            Rectangle visualRectangle = new Rectangle(rect.X + rectangle.X, rect.Y + rectangle.Y, rect.Width - rectangle.Width, rect.Height - rectangle.Height);
            
            return visualRectangle;
        }

        // Compare images of frameworkElements
        private static bool ImageComparerResult(Bitmap exp, Bitmap act, double similarPixelArgbTolerance, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, out Bitmap diff, string imageName)
        {
            PlaceCursor(0, 0);
            bool equal = true;

            int w = Math.Min(exp.Width, act.Width);
            int h = Math.Min(exp.Height, act.Height);

            int diferentPixels = 0;
            int similarPixels = 0;
            int totalPixels = w * h;

            diff = new Bitmap(w, h);
            for (int i = 0; i < w; ++i)
            {
                for (int j = 0; j < h; ++j)
                {
                    System.Drawing.Color secondColor = act.GetPixel(i, j);
                    System.Drawing.Color firstColor = exp.GetPixel(i, j);

                    if (firstColor != secondColor)
                    {
                        if ((Math.Abs(firstColor.A - secondColor.A) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.B - secondColor.B) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.G - secondColor.G) < similarPixelArgbTolerance)
                            && (Math.Abs(firstColor.R - secondColor.R) < similarPixelArgbTolerance))
                        {
                            similarPixels++;
                            diff.SetPixel(i, j, System.Drawing.Color.Green);
                        }
                        else
                        {
                            diferentPixels++;
                            diff.SetPixel(i, j, System.Drawing.Color.Red);
                        }
                    }
                    else
                    {
                        diff.SetPixel(i, j, firstColor);
                    }
                }
            }

            double diffPercent = ((double)diferentPixels / (double)totalPixels) * 100;
            double similarPercent = ((double)similarPixels / (double)totalPixels) * 100;

            if ((diffPercent > differentPixelsPercentTolerance) ||
                (similarPercent > similarPixelsPercentTolerance))
            {
                equal = false;
                DoLog(String.Format("Image name: {0}, Total pixels: {1}, Different pixels: {2}, diffPercent: {3}, differentPixelsPercentTolerance: {4}, similarPercent {5}, similarPixelsPercentTolerance: {6}",
                                        imageName, totalPixels, diferentPixels, diffPercent, differentPixelsPercentTolerance,
                                        similarPercent, similarPixelsPercentTolerance), FolderSettings.ImageVerificationResultsFolder);
            }

            return equal;
        }

        private static Bitmap CreateScreenImage(Rectangle visualRect)
        {
            if (visualRect.Width <= 0 || visualRect.Height <= 0)
            {
                throw new ArgumentOutOfRangeException("The rectangle has invalid width: " + visualRect.Width + " height: " + visualRect.Height);
            }

            System.Drawing.Rectangle bounds = new System.Drawing.Rectangle(visualRect.X, visualRect.Y, visualRect.Width, visualRect.Height);
            Console.WriteLine("In CreateScreenImage rectangle {0} {1} {2} {3}", visualRect.X, visualRect.Y, visualRect.Width, visualRect.Height);

            var result = new Bitmap(visualRect.Width, visualRect.Height);

            try
            {
                using (var g = Graphics.FromImage(result))
                {
                    g.CopyFromScreen(new System.Drawing.Point(visualRect.X, visualRect.Y), new System.Drawing.Point(0, 0), result.Size, CopyPixelOperation.SourceCopy);
                }
            }
            catch (Exception ex)
            {

                throw new Exception(ex.Message + " InnerException: " + ex.InnerException);
            }

            return result;
        }

        private static void PlaceCursor(int x, int y)
        {
            System.Drawing.Point p = new System.Drawing.Point(x, y);
            System.Windows.Forms.Cursor.Position = p;
            System.Threading.Thread.Sleep(25);
        }

        public static void DoLog(string logMessage, string folderPath)
        {
            DateTime d = DateTime.Now;
            string date = String.Format("_{0:00}-{1:00}-{2:00}", d.Year, d.Month, d.Day);

            string logName = "log" + date + ".txt";
            string docPath = System.IO.Path.Combine(folderPath, logName);

            using (StreamWriter w = new StreamWriter(docPath, true))
            {
                w.Write("\r\nLog Entry : ");

                w.WriteLine("{0} {1}:", DateTime.Now.ToLongTimeString(),
                    DateTime.Now.ToLongDateString());
                w.WriteLine("\r\n{0}", logMessage);
                w.WriteLine("-------------------------------");
            }
        }
    }
}
