﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Threading;
using Telerik.Windows.Documents.Fixed.Tests.Integration;


namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public class ImageComparerHelper : IntegrationTestClassBase
    {
        public static bool CompareImages(string expectedImage, System.Drawing.Rectangle visualRect, Process process, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            PlaceCursor(0, 0);

            Rectangle screenRectangle = ImageComparer.GetScreenRectangleOfProcess(process, visualRect);

            bool equality = Compare(screenRectangle, expectedImage, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance);

            return equality;
        }

        public static bool CompareImages(string expectedImage, System.Drawing.Rectangle visualRect, string windowName, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            PlaceCursor(0, 0);

            Rectangle screenRectangle = ImageComparer.GetScreenRectangleByCaption(windowName, visualRect);

            bool equality = Compare(screenRectangle, expectedImage, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance);

            return equality;
        }

        public static bool CompareImagesFromIE(string expectedImage, System.Drawing.Rectangle visualRect, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            PlaceCursor(0, 0);

            bool equality = Compare(visualRect, expectedImage, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance);

            return equality;
        }

        //TODO: Check comparison and see why it compare images twice
        private static bool Compare(Rectangle screenRectangle, string imageName, double differentPixelsPercentTolerance, double similarPixelsPercentTolerance, double similarPixelArgbTolerance)
        {
            PlaceCursor(0, 0);

            bool isFirstTimeComparing = false;
            var imageFullName = FolderSettings.ExpectedImagesStorageFolder + imageName;
            Bitmap exp;

            try
            {
                exp = ImageComparer.GetSavedImage(imageFullName);
            }
            catch
            {
                Thread.Sleep(1500);
                exp = ImageComparer.CaptureImage(screenRectangle);
                exp.Save(imageFullName);
                isFirstTimeComparing = true;
            }

            bool equality = true;

            if (isFirstTimeComparing == false)
            {
                Thread.Sleep(1500);
                Bitmap actual = ImageComparer.CaptureImage(screenRectangle);

                Bitmap diff;
                equality = ImageComparer.Compare(exp, actual, out diff, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance, imageName);

                if (equality == false)
                {
                    Thread.Sleep(1500);

                    actual = ImageComparer.CaptureImage(screenRectangle);

                    equality = ImageComparer.Compare(exp, actual, out diff, differentPixelsPercentTolerance, similarPixelsPercentTolerance, similarPixelArgbTolerance, imageName);
                }

                if (equality == false)
                {
                    CreateImageResults(imageName, exp, actual, diff);
                }
            }

            return equality;
        }

        private static void CreateImageResults(string imageFullName, Bitmap exp, Bitmap actual, Bitmap diff)
        {
            if (!Directory.Exists(FolderSettings.ImagesTempFolder))
            {
                Directory.CreateDirectory(FolderSettings.ImagesTempFolder);
            }

            string imageName = System.IO.Path.GetFileNameWithoutExtension(imageFullName);

            //FolderSettings.CreateLog(FolderSettings.GenerateSettingsInfo());
            exp.Save(FolderSettings.ImagesTempFolder + imageName + "_expected.png", System.Drawing.Imaging.ImageFormat.Png);
            diff.Save(FolderSettings.ImagesTempFolder + imageName + "_zdiff.png", System.Drawing.Imaging.ImageFormat.Png);
            actual.Save(FolderSettings.ImagesTempFolder + imageName + ".png", System.Drawing.Imaging.ImageFormat.Png);
        }

        private static void PlaceCursor(int x, int y)
        {
            System.Drawing.Point p = new System.Drawing.Point(x, y);
            System.Windows.Forms.Cursor.Position = p;
            System.Threading.Thread.Sleep(25);
        }
    }
}
