﻿using ArtOfTest.Common.Win32;
using ArtOfTest.WebAii.Core;
using ArtOfTest.WebAii.TestTemplates;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Flow.Tests.Flow.Utils;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using System.Collections.Generic;
using Microsoft.Win32;

namespace Telerik.Windows.Documents.Fixed.Tests.Integration
{
    [TestClass]
    public partial class IntegrationTestClassBase : BaseWpfTest
    {
        private TestContext testContextInstance = null;

        protected static VideoRecorder VideoRecorder;

        protected VisualTestHelper VisualTestHelper;
        protected static string LatestBuildFolder;

        public const string DOCXFILEEXTENSION = TestSettings.DOCXFILEEXTENSION;
        public const string PDFFILEEXTENSION = TestSettings.PDFFILEEXTENSION;
        public const string HTMLFILEEXTENSION = TestSettings.HTMLFILEEXTENSION;
        public const string RTFFILEEXTENSION = TestSettings.RTFFILEEXTENSION;
        public const string XLSXFILEEXTENSION = TestSettings.XLSXFILEEXTENSION;
        public const string CSVFILEEXTENSION = TestSettings.CSVFILEEXTENSION;
        public static string HtmlXPath = @"/html";
        public static Manager MyManager { get; set; }

        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        [AssemblyInitialize]
        public static void AssemblyInitialize(TestContext testContext)
        {
            if (!Directory.Exists(FolderSettings.DplDocumentsTempFolder))
            {
                Directory.CreateDirectory(FolderSettings.DplDocumentsTempFolder);
            }


            VideoRecorder = new VideoRecorder();

        }

        [ClassInitialize()]
        public void MyClassInitialize(TestContext testContext)
        {
        }

        [TestInitialize()]
        public void MyTestInitialize()
        {
            // Delete all Excel and WOrd DocumentRecovery subkeys excluding the root one created when 
            // Excel is forced to close unexpectedly usually during UI test
            DeleteRegistrySubKeysByGivenRegistryPath(TestSettings.EXCEL_DOCUMENT_RECOVERY_PATH, Registry.CurrentUser);
            DeleteRegistrySubKeysByGivenRegistryPath(TestSettings.WORD_DOCUMENT_RECOVERY_PATH, Registry.CurrentUser);
            DeleteRegistryKeyByGivenRegistryPath(TestSettings.EXCEL_DISABLED_ITEMS_PATH, Registry.CurrentUser);
            DeleteRegistryKeyByGivenRegistryPath(TestSettings.WORD_DISABLED_ITEMS_PATH, Registry.CurrentUser);
            DeleteRegistryKeyByGivenRegistryPath(TestSettings.WORD_READING_LOCATIONS, Registry.CurrentUser);
            DeleteRegistryKeyByGivenRegistryPath(TestSettings.WORD_File_MRU, Registry.CurrentUser);
            DeleteRegistryKeyByGivenRegistryPath(TestSettings.WORD_User_MRU, Registry.CurrentUser);

            EnsureApplicationsAreNotAlive();
            Settings settings = GetSettings();
            settings.Web.DefaultBrowser = BrowserType.InternetExplorer;
            settings.Web.KillBrowserProcessOnClose = true;
            settings.Web.RecycleBrowser = true;
            MyManager = new Manager(settings);
            MyManager.Start();
            Initialize(settings, new TestContextWriteLine(this.TestContext.WriteLine));
            SetTestMethod(this, (string)TestContext.Properties["TestName"]);
            string testName = this.TestContext.TestName;
            this.VisualTestHelper = new VisualTestHelper(testName);
            UpdateTempFoldersNamesInDebug();


            VideoRecorder = new VideoRecorder();
            VideoRecorder.StartRecordingVideo(FolderSettings.ExpectedImagesStorageFolder, FolderSettings.VideosTempFolder, testName);

        }
        /// <summary>
        /// Deletes all registry SubKeys by given registry path and Windows registry base.
        /// </summary>
        /// <param name="registryPath">Desired registry path</param>
        /// <param name="windowsRegistryBase">Windows registry base. For instance Registry.CurrentUser or Registry.LocalMachine.</param>
        private static void DeleteRegistrySubKeysByGivenRegistryPath(string registryPath, RegistryKey windowsRegistryBase)
        {
            RegistryKey registryCurrentUserKey = windowsRegistryBase;
            
            var registryKey = registryCurrentUserKey.OpenSubKey(registryPath);
            if (registryKey != null)
            {
                var registryKeySubKeys = registryKey.GetSubKeyNames();

                if (registryKeySubKeys.Length > 0)
                {
                    foreach (var item in registryKeySubKeys)
                    {
                        var subKeyToBeDeletedPath = registryPath + item;
                        registryCurrentUserKey.DeleteSubKeyTree(subKeyToBeDeletedPath);
                    }
                }
            }
            
        }

        /// <summary>
        /// Deletes registry Key by given registry path and Windows registry base.
        /// </summary>
        /// <param name="registryPath">Desired registry path</param>
        /// <param name="windowsRegistryBase">Windows registry base. For instance Registry.CurrentUser or Registry.LocalMachine.</param>
        private static void DeleteRegistryKeyByGivenRegistryPath(string registryPath, RegistryKey windowsRegistryBase)
        {
            RegistryKey registryCurrentUserKey = windowsRegistryBase;

            var registryKey = registryCurrentUserKey.OpenSubKey(registryPath);
            if (registryKey != null)
            {
                registryCurrentUserKey.DeleteSubKeyTree(registryPath);
            }
        }

        [TestCleanup()]
        public void MyTestCleanup()
        {
            this.CheckIfImagesHasPassed(this.VisualTestHelper.FailedImagesInfo);


            if (this.TestContext.CurrentTestOutcome != UnitTestOutcome.Passed)
            {
                VideoRecorder.DisposeCapture(false);
            }
            else
            {
                VideoRecorder.DisposeCapture(true);
            }

            if (Manager != null)
            {
                Manager.Dispose();
            }
            ShutDown();
            this.CleanUp();

            EnsureApplicationsAreNotAlive();
        }

        [ClassCleanup()]
        public static void MyClassCleanup()
        {
            EnsureApplicationsAreNotAlive();
            DeleteTempFolder();
            ShutDown();
        }

        [AssemblyCleanup]
        public static void AssemblyCleanup()
        {
            EnsureApplicationsAreNotAlive();

            DeleteTempFolder(true);
        }

        [Conditional("DEBUG")]
        private static void UpdateTempFoldersNamesInDebug()
        {
            string format = "dd-MM-yyyy_HH-mm-ss";
            string time = DateTime.Now.ToString(format);
            FolderSettings.VideosTempFolder = FolderSettings.TestsTempFolder + @"\Test_" + time + @"\Videos\";
            FolderSettings.ImagesTempFolder = FolderSettings.TestsTempFolder + @"\Test_" + time + @"\Images\";
        }

        public static void StartIEBrowser(string fullFileName)
        {
            MyManager.LaunchNewBrowser();

            MyManager.ActiveBrowser.NavigateTo(fullFileName);
            MyManager.ActiveBrowser.Window.Maximize();
            MyManager.ActiveBrowser.WaitUntilReady();
        }

        public static System.Drawing.Rectangle GetScreenRectangle(string xPath)
        {
            var element = MyManager.ActiveBrowser.Find.ByXPath(xPath);
            element.Wait.ForExists(1000);
            return element.GetRectangle();
        }

        private void CheckIfImagesHasPassed(Dictionary<string, bool> failedImagesInfo)
        {
            var results = failedImagesInfo;
            bool isFalse = false;

            foreach (string result in results.Keys)
            {
                if (results[result] == false)
                {
                    isFalse = true;
                    Console.WriteLine(String.Format("{0} image is not as actual", result));
                }
            }

            if (isFalse)
            {
                Assert.IsTrue(false, "Images are not the same as expected!");
            }
        }

        private static void EnsureApplicationsAreNotAlive()
        {
#if WordsProcessing
            ProcessKiller.KillProcess(TestSettings.DOCXFILEEXTENSION, string.Empty);
            ProcessKiller.KillProcess(TestSettings.HTMLFILEEXTENSION, string.Empty);
            ProcessKiller.KillProcess(TestSettings.PDFFILEEXTENSION, string.Empty);
#endif
#if PdfProcessing
            ProcessKiller.KillProcess(TestSettings.PDFFILEEXTENSION, string.Empty);
#endif
#if SpreadsheetStreaming
            ProcessKiller.KillProcess(TestSettings.XLSXFILEEXTENSION, string.Empty);
            ProcessKiller.KillProcess(TestSettings.CSVFILEEXTENSION, string.Empty);
#endif
        }

        private static void DeleteTempFolder(bool recursive = false)
        {
#if Release
            Directory.Delete(TempFolder, recursive);
#endif
        }
    }
}