﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Telerik.Windows.Documents.Flow.Tests.Utils
{
    public static class UIInteractionHelper
    {
        private const int MOUSEEVENTF_LEFTDOWN = 0x02;
        private const int MOUSEEVENTF_LEFTUP = 0x04;

        /// <summary>
        /// Page down
        /// </summary>
        /// <param name="repeatCount">The count of execution of this action</param>
        /// <returns>Action</returns>
        public static Action PageDown(int repeatCount = 1)
        {
            return () => PageDownInternal(repeatCount);
        }

        public static Action PageUp(int repeatCount = 1)
        {
            return () => PageUpInternal(repeatCount);
        }

        public static Action AltF4KeysCombination()
        {
            return () => KeyPressInternal("%" + "{F4}", 1);
        }

        public static void ClickOnCloseBtn(string caption)
        {
            var rect = ImageComparer.GetScreenRectangleByCaption(caption);
            LeftMouseClick(rect.X + rect.Width - 25, rect.Y + 15)();
        }

        public static Action KeyPress(string key, int repeatCount = 1)
        {
            return () => KeyPressInternal("{" + key + "}", repeatCount);
        }

        private static void PageUpInternal(int repeatCount)
        {
            KeyPressInternal("{PGUP}", repeatCount);
        }

        private static void PageDownInternal(int repeatCount)
        {
            KeyPressInternal("{PGDN}", repeatCount);
        }

        private static void KeyPressInternal(string key, int repeatCount)
        {
            for (int i = 0; i < repeatCount; i++)
            {
                System.Windows.Forms.SendKeys.SendWait(key);
                System.Threading.Thread.Sleep(200);
            }
        }

#if VISUAL
        //This simulates a left mouse click
        public static Action LeftMouseClick(int xpos, int ypos)
        {
            return () =>
            {
                Extern.SetCursorPos(xpos, ypos);
                Extern.mouse_event(MOUSEEVENTF_LEFTDOWN, xpos, ypos, 0, 0);
                //System.Threading.Thread.Sleep(100000);
                Extern.mouse_event(MOUSEEVENTF_LEFTUP, xpos, ypos, 0, 0);
            };
        }
#endif
    }
}
