﻿#if SpreadsheetStreaming
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
#endif
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.Flow.Utils
{
    public class VisualTestHelper
    {
        private static readonly System.Drawing.Rectangle defaultFlowRectangle = new System.Drawing.Rectangle(298, 64, 354, 114);
        //measures after test pc upgrade
        //private static System.Drawing.Rectangle defaultFlowRectangle = new System.Drawing.Rectangle(8, 166, 16, 196);
        private static readonly System.Drawing.Rectangle defaultFixedRectangle = new System.Drawing.Rectangle(0, 30, 0, 92);
        private System.Drawing.Rectangle currentRectangle;

        private DocumentLauncher launcher;
        private string fileExtension;
        private int imagesCounter;
        private string imageName;        

        private Dictionary<string, bool> failedImagesInfo;

        private const double DefaultDifferentPixelsPercentTolerance = 0.0001;
        private const double DefaultSimilarPixelsPercentTolerance = 0.005;
        private const double DefaultSimilarPixelArgbTolerance = 0.0001;

        public VisualTestHelper(string testName)
        {
            this.failedImagesInfo = new Dictionary<string, bool>();
            this.launcher = new DocumentLauncher(testName);
            imagesCounter = 0;
            DifferentPixelsPercentTolerance = DefaultDifferentPixelsPercentTolerance;
            SimilarPixelsPercentTolerance = DefaultSimilarPixelsPercentTolerance;
            SimilarPixelArgbTolerance = DefaultSimilarPixelArgbTolerance;
        }

        public System.Drawing.Rectangle DefaultRectangle { get; set; }

        public double DifferentPixelsPercentTolerance { get; set; }
        public double SimilarPixelsPercentTolerance { get; set; }
        public double SimilarPixelArgbTolerance { get; set; }

        public Dictionary<string, bool> FailedImagesInfo
        {
            get
            {
                return this.failedImagesInfo;
            }
        }

        public static Stream LoadVisualTestDocumentFromStream(string documentName)
        {
            MemoryStream memory = new MemoryStream();

            using (Stream stream = File.OpenRead(FolderSettings.VisualTestsDocuments + documentName))
            {
                stream.CopyTo(memory);
                memory.Seek(0, SeekOrigin.Begin);
            }

            return memory;
        }

        public static byte[] LoadVisualTestDocumentFromByteArray(string documentName)
        {
            var bytes = File.ReadAllBytes(FolderSettings.VisualTestsDocuments + documentName);

            return bytes;
        }

        public void AssertWordPad(byte[] data, string extension, Action action, int repeatAction, int wait = 4000, params Action[] actions)
        {
            this.launcher.ShouldUseWordPadProcess = true;
            this.Assert(data, extension, action, repeatAction, wait, actions);
            this.launcher.ShouldUseWordPadProcess = false;
        }

        public void Assert(byte[] data, string extension, Action action, int repeatAction, int wait = 4000, params Action[] actions)
        {
            launcher.ExportAndLoadDocument(data, extension, wait, actions);

            this.fileExtension = extension;
            this.CheckRectangle();
            this.UpdateImageName();
            bool shouldKillProcess = false;

            if (repeatAction == 0 || action == null)
            {
                shouldKillProcess = true;
            }

            this.AssertHelper(shouldKillProcess);

            if (action != null)
            {
                for (int i = 0; i < repeatAction; i++)
                {
                    if (action != null)
                    {
                        action();
                        Thread.Sleep(450);
                    }

                    this.UpdateImageName();

                    if (i <= repeatAction - 2)
                    {
                        this.AssertHelper(false);
                    }
                    else
                    {
                        this.AssertHelper(true);
                    }
                }
            }

            this.DefaultRectangle = new System.Drawing.Rectangle();

        }

        public void AssertFromProcess(string filePath, string extension, Action action, int repeatAction, int wait = 4000, params Action[] actions)
        {
            launcher.ExportAndLoadDocument(filePath, extension);

            this.fileExtension = extension;
            this.CheckRectangle();
            this.UpdateImageName();
            bool shouldKillProcess = false;

            if (repeatAction == 0 || action == null)
            {
                shouldKillProcess = true;
            }

            this.ProcessAssertHelper(shouldKillProcess);

            if (action != null)
            {
                for (int i = 0; i < repeatAction; i++)
                {
                    if (action != null)
                    {
                        action();
                        Thread.Sleep(450);
                    }

                    this.UpdateImageName();

                    if (i <= repeatAction - 2)
                    {
                        this.ProcessAssertHelper(false);
                    }
                    else
                    {
                        this.ProcessAssertHelper(true);
                    }
                }
            }

            this.DefaultRectangle = new System.Drawing.Rectangle();
        }

        private void AssertHelper(bool shouldKillProcess = true)
        {
            bool equality = this.AssertImageEquality();

            if (shouldKillProcess)
            {
                ProcessKiller.KillProcess(this.fileExtension, launcher.Caption);
            }

            this.failedImagesInfo.Add(this.imageName, equality);
        }

        private void ProcessAssertHelper(bool shouldKillProcess = true)
        {
            bool equality = this.AssertImageEqualityFromProcess();

            if (shouldKillProcess)
            {
                ProcessKiller.KillProcess(this.fileExtension, launcher.Caption);
            }

            this.failedImagesInfo.Add(this.imageName, equality);
        }

        private bool AssertImageEquality()
        {
            bool equality = this.GetIsImageEqual();

            return equality;
        }

        private bool AssertImageEqualityFromProcess()
        {
            bool equality = this.GetIsImageEqualFromProcess();

            return equality;
        }

        private bool GetIsImageEqual()
        {
            bool equality = false;

            if (this.fileExtension == TestSettings.HTMLFILEEXTENSION)
            {
                equality = ImageComparerHelper.CompareImagesFromIE(imageName, this.currentRectangle, DifferentPixelsPercentTolerance, SimilarPixelsPercentTolerance, SimilarPixelArgbTolerance);
            }
            else
            {
                equality = ImageComparerHelper.CompareImages(imageName, this.currentRectangle, this.launcher.Caption, DifferentPixelsPercentTolerance, SimilarPixelsPercentTolerance, SimilarPixelArgbTolerance);
            }

            return equality;
        }

        private bool GetIsImageEqualFromProcess()
        {
            bool equality = false;

            if (this.fileExtension == TestSettings.HTMLFILEEXTENSION)
            {
                equality = ImageComparerHelper.CompareImagesFromIE(imageName, this.currentRectangle, DifferentPixelsPercentTolerance, SimilarPixelsPercentTolerance, SimilarPixelArgbTolerance);
            }
            else
            {
                equality = ImageComparerHelper.CompareImages(imageName, this.currentRectangle, this.launcher.process, DifferentPixelsPercentTolerance, SimilarPixelsPercentTolerance, SimilarPixelArgbTolerance);
            }

            return equality;
        }

        private void UpdateImageName()
        {
            this.imagesCounter++;

            this.imageName = this.launcher.TestName + "_" + this.imagesCounter + ".png";
        }

        private void CheckRectangle()
        {
            if (this.DefaultRectangle == null || (this.DefaultRectangle.X == 0 && this.DefaultRectangle.Y == 0 && this.DefaultRectangle.Width == 0 && this.DefaultRectangle.Height == 0))
            {
                if (this.fileExtension == TestSettings.DOCXFILEEXTENSION)
                {
                    this.currentRectangle = defaultFlowRectangle;
                }
                else if (this.fileExtension == TestSettings.RTFFILEEXTENSION)
                {
                    this.currentRectangle = defaultFlowRectangle;
                }
                else if (this.fileExtension == TestSettings.PDFFILEEXTENSION)
                {
                    this.currentRectangle = defaultFixedRectangle;
                }
                else if (this.fileExtension == TestSettings.HTMLFILEEXTENSION)
                {
                    this.currentRectangle = IntegrationTestClassBase.GetScreenRectangle(IntegrationTestClassBase.HtmlXPath);
                    ;
                }
                else if (this.fileExtension == TestSettings.XLSXFILEEXTENSION)
                {
                    this.currentRectangle = defaultFixedRectangle;
                }
                else
                {
                    throw new ArgumentException("Missing boundery rectangle");
                }
            }
            else
            {
                this.currentRectangle = this.DefaultRectangle;
            }
        }

        public static MemoryStream ConvertStreamToMemoryStream(Stream stream)
        {
            MemoryStream memoryStream = new MemoryStream();

            if (stream != null)
            {
                byte[] buffer = ReadFully(stream);

                if (buffer != null)
                {
                    var binaryWriter = new BinaryWriter(memoryStream);
                    binaryWriter.Write(buffer);
                }
            }

            return memoryStream;
        }

        private static byte[] ReadFully(Stream input)
        {
            input.Seek(0, SeekOrigin.Begin);
            byte[] buffer = new byte[input.Length];
            using (MemoryStream ms = new MemoryStream())
            {
                int read;
                while ((read = input.Read(buffer, 0, buffer.Length)) > 0)
                {
                    ms.Write(buffer, 0, read);
                }
                return ms.ToArray();
            }
        }
    }
}
