﻿using ArtOfTest.WebAii.Controls.Xaml.Wpf;
using DocumentFormat.OpenXml.Presentation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;

namespace Telerik.Windows.Documents.Flow.Tests.VisualTests
{
    [TestClass]
    public class GenerateAndExportDocuments : DocumentFlowVisualBaseTest
    {
        private System.Drawing.Rectangle wordRectangle300x300 = new System.Drawing.Rectangle() { X = 300, Y = 200, Width = 300, Height = 300 };
        private System.Drawing.Rectangle pdfRectangle300x300 = new System.Drawing.Rectangle() { X = 100, Y = 50, Width = 300, Height = 300 };

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("id:155223 tfs: 234660")]
        public void ApplyMixedIndentsOfBlock()
        {
            RadFlowDocument document = new RadFlowDocument();
            var section = document.Sections.AddSection();

            // Add block with negative right indent
            InsertParagraph(section, 0, -30);

            // Add block with negative left indent
            InsertParagraph(section, -30, 0);

            // Add block with negative left and right indent
            InsertParagraph(section, -20, -20);

            // Add block with positive left and negative right indent
            InsertParagraph(section, 20, -20);

            // Add block with negative left and positive right indent
            InsertParagraph(section, -20, 20);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("id:155223 tfs: 234660")]
        public void ApplyMixedIndentsOfBlockInTable()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            var section = document.Sections.AddSection();

            section.Blocks.AddParagraph().Inlines.AddRun("AutoFitTable");
            Table tableAutoFit = section.Blocks.AddTable();
            CreateTableRow(tableAutoFit, new int[] { 0, -20, -20, 0, -20, -20 });
            CreateTableRow(tableAutoFit, new int[] { -20, -20, 20, -10, -20, 10 });
            SetTableProperties(tableAutoFit, TableLayoutType.AutoFit);

            section.Blocks.AddParagraph().Inlines.AddRun("FixedWidthTable");
            Table tableFixedWidth = section.Blocks.AddTable();
            tableFixedWidth.PreferredWidth = new TableWidthUnit(TableWidthUnitType.Fixed, 300);
            CreateTableRow(tableFixedWidth, new int[] { 0, -20, -20, 0, -20, -20 });
            CreateTableRow(tableFixedWidth, new int[] { -20, -20, 20, -10, 20, 10 });
            SetTableProperties(tableFixedWidth, TableLayoutType.FixedWidth);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("id:155223 tfs: 234660")]
        public void ApplyMixedIndentsOfBlocksInTableInTable()
        {
            RadFlowDocument document = new RadFlowDocument();
            var section = document.Sections.AddSection();
            Table mainTable = section.Blocks.AddTable();

            var firstRow = mainTable.Rows.AddTableRow();

            var firstCell = firstRow.Cells.AddTableCell();
            Table tableAutoFit = firstCell.Blocks.AddTable();
            tableAutoFit.Alignment = Alignment.Center;
            CreateTableRow(tableAutoFit, new int[] { 0, -20, -20, 0, -20, -20 });
            CreateTableRow(tableAutoFit, new int[] { -20, -20, 20, -10, -20, 10 });
            SetTableProperties(tableAutoFit, TableLayoutType.AutoFit);

            var secondCell = firstRow.Cells.AddTableCell();
            Table tableFixedWidth = secondCell.Blocks.AddTable();
            tableFixedWidth.Alignment = Alignment.Right;
            CreateTableRow(tableFixedWidth, new int[] { 0, -20, -20, 0, -20, -20 });
            CreateTableRow(tableFixedWidth, new int[] { -20, -20, 20, -10, 20, 10 });
            SetTableProperties(tableFixedWidth, TableLayoutType.FixedWidth);
            SetTableProperties(mainTable, TableLayoutType.AutoFit, 1, 2);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("Team Pulse ID:169549 TFS: 242673")]
        public void ExportDocToPdfWithLowImageQuality()
        {
            var document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            MemoryStream memory = new MemoryStream(File.ReadAllBytes(FolderSettings.VisualTestsDocuments + @"\png\flying.png"));
            editor.InsertImageInline(memory, "png");
            PdfExportSettings settings = new PdfExportSettings();
            settings.ImageQuality = Fixed.FormatProviders.Pdf.Export.ImageQuality.Low;

            this.AssertAreVisualyEqualPdf(document, settings, null);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("Team Pulse ID:169549 TFS: 242673")]
        public void ExportDocToPdfWithMediumImageQuality()
        {
            var document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            MemoryStream memory = new MemoryStream(File.ReadAllBytes(FolderSettings.VisualTestsDocuments + @"\png\flying.png"));
            editor.InsertImageInline(memory, "png");
            PdfExportSettings settings = new PdfExportSettings();
            settings.ImageQuality = Fixed.FormatProviders.Pdf.Export.ImageQuality.Medium;

            this.AssertAreVisualyEqualPdf(document, settings, null);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("Team Pulse ID:169549 TFS: 242673")]
        public void ExportDocToPdfWithHighImageQuality()
        {
            var document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);
            MemoryStream memory = new MemoryStream(File.ReadAllBytes(FolderSettings.VisualTestsDocuments + @"\png\flying.png"));
            editor.InsertImageInline(memory, "png");

           this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("TFS: 283608 - WordsProcessing: ArgumentOutOfRangeException is thrown when exporting table without cells to PDF")]
        public void ExportDocWithTableWithoutCellsToPdf()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertText("text");
            editor.InsertTable().Rows.AddTableRow();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("TFS ID: 243053 - WordsProcessing: Images with upper case extension cannot be exported to DOCX")]
        public void ExportImagesWithUpperCaseExtensionToDocxTest()
        {
            RadFlowDocument document = new RadFlowDocument();
            ImageInline imageUpperCaseExt;
            ImageInline imageLowerCaseExt;

            using (var stream = TestHelper.GetImageStream("minion.JPG"))
            {
                imageUpperCaseExt = new ImageInline(document);
                imageUpperCaseExt.Image.ImageSource = new Media.ImageSource(stream, "JPG");
            }
            using (var stream = TestHelper.GetImageStream("test.png"))
            {
                imageLowerCaseExt = new ImageInline(document);
                imageLowerCaseExt.Image.ImageSource = new Media.ImageSource(stream, "png");
            }
            Paragraph p = document.Sections.AddSection().Blocks.AddParagraph();
            p.Inlines.Add(imageUpperCaseExt);
            p.Inlines.Add(imageLowerCaseExt);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.CreateExportDocument)]
        [Description("TFS ID: 286984 - WordsProcessing: Exporting Runs without set text to PDF throws NullReferenceException")]
        public void ExportRunsToPdfWithoutSetTextTest()
        {
            RadFlowDocument document = new RadFlowDocument();
            Telerik.Windows.Documents.Flow.Model.Run run = new Telerik.Windows.Documents.Flow.Model.Run(document);

            document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun();

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, false);
        }

        private void SetTableProperties(Table table, Documents.Flow.Model.Styles.TableLayoutType layoutType, int rowCount = 2, int columnCount = 3)
        {
            table.LayoutType = layoutType;
            Documents.Flow.Model.Styles.Border cellBorders = new Documents.Flow.Model.Styles.Border(1, BorderStyle.Single, new Spreadsheet.Model.ThemableColor(Colors.Blue));

            for (int rows = 0; rows < rowCount; rows++)
            {
                for (int columns = 0; columns < columnCount; columns++)
                {
                    table.Rows[rows].Cells[columns].Borders = new TableCellBorders(cellBorders, cellBorders, cellBorders, cellBorders);
                }
            }
        }

        private static void CreateTableRow(Table table, int[] indents)
        {
            TableRow tableRows = table.Rows.AddTableRow();

            for (int j = 0; j < indents.Length - 1; j += 2)
            {
                TableCell tableCell = tableRows.Cells.AddTableCell();
                var p = tableCell.Blocks.AddParagraph();
                var run = p.Inlines.AddRun(String.Format("Block with left indent: {0} / right indent: {1}", indents[j], indents[j + 1]));
                p.Indentation.LeftIndent = indents[j];
                p.Indentation.RightIndent = indents[j + 1];

            }
        }

        private static void InsertParagraph(Section section, int leftIndent, int rightIndent)
        {
            var p = section.Blocks.AddParagraph();

            var run = p.Inlines.AddRun(String.Format("Block with left indent: {0} / right indent: {1}", leftIndent, rightIndent));
            p.Indentation.RightIndent = rightIndent;
            p.Indentation.LeftIndent = leftIndent;
        }

    }
}
