﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Tests.Integration;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.FormatProviders.Rtf;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Flow.Tests.Utilities;
using Telerik.Windows.Documents.Flow.Tests.Utils;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Export;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;

namespace Telerik.Windows.Documents.Flow.Tests.VisualTests
{
    [TestClass]
    public class ImportExportDocuments : DocumentFlowVisualBaseTest
    {
        public static string TestFilesFolder = "VisualTestsDocuments/";

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void ImportDocumentContainingListWithoutExplicitlySpecifiedMultilevelType()
        {
            RadFlowDocument document = new HtmlFormatProvider().Import("<html><style>.tdborder{border: solid 1px black}</style><body><table><tr><td class=\"tdborder\"><p>&nbsp;<br />&nbsp;</p></td></tr></table></body></html>");

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void ImportDocumentWithTableWithWithPercentageWidth()
        {
            RadFlowDocument document = new HtmlFormatProvider().Import("<html><body>HTML to PDG Conversion<table style='width: 500px;border:1px solid green;'><tr><td style='width: 25%;border:1px solid blue;'>Hello</td><td style='width: 75%;border:1px solid red;'>World</td></tr></table></body></html>");

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void ImportExportTableWithSingleCell()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "DynamicFontsIssues.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION, DOCXFILEEXTENSION, HTMLFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        public void ImportExportDocumentWithMissingDefaultStyleProperties()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "Input.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID: 159645")]
        public void ImportExportDocumentWithPageBreakBeforeParagraph()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "PageBreakBeforeParagraph.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:159655 TFS: 237224")]
        public void ImportExportDocTriesToDrawSameLayoutElementTwiceToPDF1()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("EBDev-000001.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:159655 TFS: 237224")]
        public void ImportExportDocTriesToDrawSameLayoutElementTwiceToPDF2()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("Word2013.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION }, UIInteractionHelper.PageDown(), 1);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:164505 TFS: 240216")]
        public void ImportExportDocWithCellsWithPrefferedWidth1()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "Table.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:164505 TFS: 240216")]
        public void ImportExportDocWithCellsWithPrefferedWidth2()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "TableBad.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:164505 TFS: 240216")]
        public void ImportExportDocWithCellsWithPrefferedWidth3()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "TableImage.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:164505 TFS: 240216")]
        public void ImportExportDocWithCellsWithPrefferedWidth4()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "TableImage1.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse bug ID:164505 TFS: 240216")]
        public void ImportExportDocWithCellsWithPrefferedWidth5()
        {
            RadFlowDocument document = TestHelper.GetDocumentFromFile(TestFilesFolder + "TableImageText.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [Description("Team Pulse ID:165210 TFS: 240666")]
        public void ForegroundAndHighlightPropertiesWhitTransparentForeColorAndHighlightColor()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            Paragraph paragraph = new Paragraph(document);
            Run run = new Run(document);
            run.Text = "test";
            run.HighlightColor = Colors.Transparent;
            run.ForegroundColor = new ThemableColor(Colors.Transparent);
            paragraph.Inlines.Add(run);
            document.Sections.AddSection().Blocks.Add(paragraph);

            // Act
            RadFlowDocument imported = new RtfFormatProvider().Import(new RtfFormatProvider().Export(document));

            // Assert
            Run importedRun = imported.EnumerateChildrenOfType<Run>().FirstOrDefault();
            Assert.AreEqual(new ThemableColor(Colors.Black, false), importedRun.ForegroundColor);
            Assert.AreEqual(Run.HighlightColorPropertyDefinition.DefaultValue.Value, importedRun.HighlightColor);

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Paragraph)]
        [Description("Team Pulse ID:165053 TFS: 240582")]
        public void IncorrectLinesHightByExportToPd1()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("InfoBrief_Lorem.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Paragraph)]
        [Description("Team Pulse ID:165053 TFS: 240582")]
        public void IncorrectLinesHightByExportToPdf2()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("HealthCertificate.en-GB.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION }, () => UIInteractionHelper.PageDown(), 3);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Paragraph)]
        [Description("Team Pulse ID:165053 TFS: 240582")]
        public void IncorrectLinesHightByExportToPdf3()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("Test Setup requirements.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Paragraph)]
        [Description("Team Pulse ID:165053 TFS: 240582")]
        public void IncorrectLinesHightByExportToPdf4()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("TravelCertificate.en-GB.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:171984 TFS ID:243318")]
        public void CloneDocumentContainingFieldWhichDoesNotHaveSeparator()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("TestDocument.docx", new DocxFormatProvider());

            RadFlowDocument clonedDocument = document.Clone();

            this.AssertVisual(clonedDocument, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION, PDFFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:171984 TFS ID:243318")]
        public void MailMergeDocumentContainingFieldWhichDoesNotHaveSeparator()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("mail merge.docx", new DocxFormatProvider());

            RadFlowDocument mergedDocument = document.MailMerge(new List<MailMergeRecord>() { new MailMergeRecord("First user"), new MailMergeRecord("Second user") });

            this.AssertVisual(mergedDocument, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Fields)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:169783 TFS ID:242744")]
        public void UpdateFieldsInDocumentContainingFieldsWhichDoesNotHaveSeparator()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("DocVariable.docx", new DocxFormatProvider());

            document.UpdateFields();

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.List)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:189945 TFS ID:249959")]
        public void StyledTextInListItemIsNotSplitToMultipleParagraphs()
        {
            RadFlowDocument document = new RadFlowDocument();
            string[] htmlDocs = new string[] { "problem.html", "Problem with whitespaces.html", "Problem with whitespaces_2.html", "ListTagBug.html", "DOCUMENT.html" };

            for (int i = 0; i < htmlDocs.Length; i++)
            {
                document = this.LoadVisualTestDocument(htmlDocs[i], new HtmlFormatProvider());
                this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, false);
            }
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.List)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:189945 TFS ID:249959")]
        public void StyledTextInListItemIsNotSplitToMultipleParagraphs_ExportToHtml()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("problem.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { HTMLFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.MailMerge)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:175863 TFS ID:244735")]
        public void MailMergeRespectsNewLinesFromMailMergeSource()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            editor.InsertField("MERGEFIELD FirstName ", "«FirstName»");

            RadFlowDocument mergedDocument = document.MailMerge(new List<MailMergeRecord>() 
                                                                { 
                                                                    new MailMergeRecord("First \r\nuser"), 
                                                                    new MailMergeRecord("Second \vuser"),
                                                                    new MailMergeRecord("Third" + Environment.NewLine + "user")});
            this.AssertVisual(mergedDocument, 2);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("Team Pulse ID:126095 TFS ID:215356")]
        public void ImportOfImageAsShapePlacedInsidePictElementIsImportedExportedCorrectly()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("DifferentFormatImagesInPictElementAndImageWatermark.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION, RTFFILEEXTENSION, PDFFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 251254 - WordsProcessing: Table with specified width does not expand to its full width when exported to PDF")]
        public void TableWithSpecifiedWidthExportToPdfTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("TableWithSpecifiedWidth.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION }, false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 406511 - WordsProcessing: UriFormatException is thrown when importing HTML content containing embedded images with leading white spaces")]
        public void HTMLContentWithEmbeddedImagesWithLeadingWhiteSpacesTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("SampleHTMLCertificate.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, () => UIInteractionHelper.PageDown(), 2);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 410479 - WordsProcessing: Export to PDF freezes when DefaultTabStopWidth is zero")]
        public void ExportToPdfWhenDefaultTabStopWidthIsZero()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("LightReport_2_Telerik.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 265664 - WordsProcessing: ArgumentException is thrown when importing HTML document containing pseudo-element")]
        public void PseudoElementsImportTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("PseudoElementsTest.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 288574 - WordsProcessing: KeyNotFoundException is thrown when importing document with an image with invalid or not supported extension")]
        public void DocWithInvalidImageImportTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("resume_300075.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 282773 - WordsProcessing: The size of the image is wrongly " +
            "calculated when width or height is not specified and the source is a URI or base64 string")]
        public void ImportImageFromHtmlWithOnlyWidthOrHeightAndExportToPdfTest1()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("pdfPicB64_1.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 282773 - WordsProcessing: The size of the image is wrongly " +
            "calculated when width or height is not specified and the source is a URI or base64 string")]
        public void ImportImageFromHtmlWithOnlyWidthOrHeightAndExportToPdfTest2()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("pdfPicB64_2.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 282773 - WordsProcessing: The size of the image is wrongly " +
            "calculated when width or height is not specified and the source is a URI or base64 string")]
        public void ImportImageFromHtmlWithOnlyWidthOrHeightAndExportToPdfTest3()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("pdfPicB64_3.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 282773 - WordsProcessing: The size of the image is wrongly " +
            "calculated when width or height is not specified and the source is a URI or base64 string")]
        public void ImportImageFromHtmlWithOnlyWidthOrHeightAndExportToPdfTest4()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("pdfPicB64_4.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }


        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 247829 - WordsProcessing: ArgumentException is thrown when exporting HTML document" +
            " with specifically nested tables")]
        public void ExportHtmlWithSpecificallyNestedTablesTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("file.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { PDFFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 283529 - WordsProcessing: KeyNotFoundException is thrown when importing document with an image defined as a resource but not found")]
        public void ImportDocumentWithImageDefinedWithWringResourceTest1()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("corrupted1.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 283529 - WordsProcessing: KeyNotFoundException is thrown when importing document with an image defined as a resource but not found")]
        public void ImportDocumentWithImageDefinedWithWringResourceTest2()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("corrupted2.docx", new DocxFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, assertWithWordPad:false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 283590 - WordsProcessing: ArgumentException when importing html containing attributes with empty string values")]
        public void AttributesWithEmptyStringsTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("AttributesWithEmptyStrings.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION }, assertWithWordPad:false);
        }

        [TestMethod]
        [TestCategory(WordsProcessingCategory.ImportExportDocument)]
        [TestCategory(WordsProcessingCategory.Images)]
        [TestCategory(WordsProcessingCategory.VisualTest)]
        [Description("TFS ID: 410239 - WordsProcessing: FormatException is thrown when importing HTML with img tag containing non-numeric " +
            "'width' or 'height' values")]
        public void ImgTagWithNonNumericWidthOrHeightValuesTest()
        {
            RadFlowDocument document = this.LoadVisualTestDocument("ImageWithWidthAndHeight.html", new HtmlFormatProvider());

            this.AssertVisual(document, new string[] { DOCXFILEEXTENSION });
        }

        public class MailMergeRecord
        {
            public string FirstName { get; set; }

            public MailMergeRecord(string name)
            {
                this.FirstName = name;
            }
        }
    }
}
