﻿#if NETSTANDARD
using Telerik.Documents.Primitives;
#else
using System.Windows;
#endif
using System;
using System.IO;
using System.Linq;
using System.Windows;
using CsQuery;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Shapes;
using Telerik.Windows.Documents.Media;

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export
{
    [TestClass]
    public class ImageTests
    {
        [TestMethod]
        public void HasHeightHasWidth_ShouldUpdate_WhenHeightWidthAreSet()
        {
            RadFlowDocument doc = new RadFlowDocument();
            ImageInline image1 = new ImageInline(doc);
            image1.Image.LockAspectRatio = false;
            image1.Image.Width = 100;
            image1.Image.Height = 100;
            
            ImageInline image2 = new ImageInline(doc);
            image1.Image.LockAspectRatio = false;

            Assert.IsTrue(image1.Image.HasHeight && image1.Image.HasWidth);
            Assert.IsFalse(image2.Image.HasWidth || image2.Image.HasWidth);
        }

        [TestMethod]
        public void HasHeightHasWidth_ShouldBeCopied_WhenCopyConstructorIsUsed()
        {
            RadFlowDocument doc = new RadFlowDocument();
            ImageInline image1 = new ImageInline(doc);
            image1.Image.LockAspectRatio = false;
            image1.Image.Width = 100;
            image1.Image.Height = 100;

            ImageInline image2 = new ImageInline(doc, image1.Image);

            Assert.IsTrue(image2.Image.HasHeight && image2.Image.HasWidth);
        }

        [TestMethod]
        public void RecalculateImageSize_ShouldSetValidSize_WhenLockAspectRatioIsFalse()
        {
            RadFlowDocument doc = new RadFlowDocument();
            ImageInline image1 = new ImageInline(doc);
            image1.Image.LockAspectRatio = false;
            image1.Image.Width = 100;
            image1.Image.Height = 100;
            image1.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image1.Image.RecalculateSize();

            ImageInline image2 = new ImageInline(doc);
            image2.Image.LockAspectRatio = false;
            image2.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image2.Image.RecalculateSize();

            ImageInline image3 = new ImageInline(doc);
            image3.Image.LockAspectRatio = false;
            image3.Image.Width = 100;
            image3.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image3.Image.RecalculateSize();

            ImageInline image4 = new ImageInline(doc);
            image4.Image.LockAspectRatio = false;
            image4.Image.Height = 100;
            image4.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image4.Image.RecalculateSize();

            Assert.AreEqual(new Size(100, 100), image1.Image.Size);
            Assert.AreEqual(new Size(5, 5), image2.Image.Size);
            Assert.AreEqual(new Size(100, 5), image3.Image.Size);
            Assert.AreEqual(new Size(5, 100), image4.Image.Size);
        }

        [TestMethod]
        public void RecalculateImageSize_ShouldSetValidSize_WhenLockAspectRatioIsTrue()
        {
            RadFlowDocument doc = new RadFlowDocument();
            ImageInline image1 = new ImageInline(doc);
            image1.Image.LockAspectRatio = true;
            image1.Image.Width = 100;
            image1.Image.Height = 100;
            image1.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image1.Image.RecalculateSize();

            ImageInline image2 = new ImageInline(doc);
            image2.Image.LockAspectRatio = true;
            image2.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image2.Image.RecalculateSize();

            ImageInline image3 = new ImageInline(doc);
            image3.Image.LockAspectRatio = true;
            image3.Image.Width = 100;
            image3.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image3.Image.RecalculateSize();

            ImageInline image4 = new ImageInline(doc);
            image4.Image.LockAspectRatio = true;
            image4.Image.Height = 100;
            image4.Image.ImageSource = new ImageSource(Convert.FromBase64String("iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="), "png");
            image4.Image.RecalculateSize();

            Assert.AreEqual(new Size(100, 100), image1.Image.Size);
            Assert.AreEqual(new Size(5, 5), image2.Image.Size);
            Assert.AreEqual(new Size(100, 100), image3.Image.Size);
            Assert.AreEqual(new Size(100, 100), image4.Image.Size);
        }

        [TestMethod]
        public void TitleAttribute_WhenTitleIsSetInExportEventArgs_IsExported()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithInlineImage();

            // Act
            HtmlExportSettings settings = new HtmlExportSettings();
            settings.ImageExporting += (s, e) =>
                {
                    e.Title = "test";
                };

            CQ cq = ExportHelper.ExportDocumentToCq(document, settings);

            // Assert
            IDomObject image = cq.Select("img").ToList()[0];
            Assert.IsNotNull(image);
            Assert.AreEqual("test", image.Attributes["title"]);
        }

        [TestMethod]
        public void WidthAndHeightAttributes_WhenIsExportSizeIsFalseInExportEventArgs_AreNotExported()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithInlineImage();

            // Act
            HtmlExportSettings settings = new HtmlExportSettings();
            settings.ImageExporting += (s, e) =>
            {
                e.ExportSize = false;
            };

            CQ cq = ExportHelper.ExportDocumentToCq(document, settings);

            // Assert
            IDomObject image = cq.Select("img").ToList()[0];
            Assert.IsNotNull(image);
            Assert.IsNull(image.Attributes["width"]);
            Assert.IsNull(image.Attributes["height"]);
        }

        [TestMethod]
        public void WidthAndHeightAttributes_WhenImageSizeIsEmpty_AreNotExported()
        {
            // Arrange
            RadFlowDocument document = CreateDocumentWithInlineImage();

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            IDomObject image = cq.Select("img").ToList()[0];
            Assert.IsNotNull(image);
            Assert.IsNull(image.Attributes["width"]);
            Assert.IsNull(image.Attributes["height"]);
        }

        private static RadFlowDocument CreateDocumentWithInlineImage()
        {
            RadFlowDocument document = new RadFlowDocument();

            ImageInline image = new ImageInline(document);

            Stream stream = new MemoryStream();

            image.Image.ImageSource = new Media.ImageSource(stream, "png");

            document.Sections.AddSection().Blocks.AddParagraph().Inlines.Add(image);

            return document;
        }
    }
}