﻿using CsQuery;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export
{
    [TestClass]
    public class ListLevelValueExportTests : ListsExportBase
    {
        private void AssertListLevelNameAndValue(IDomElement domElement, int expectedValue, bool isExpectedValueMandatory = false)
        {
            Assert.AreEqual("li", domElement.NodeName.ToLowerInvariant());

            string value = domElement.GetAttribute("value");
            if (!isExpectedValueMandatory)
            {
                Assert.IsTrue(string.IsNullOrEmpty(value) || value.ToLowerInvariant() == expectedValue.ToString().ToLowerInvariant());
            }
            else
            {
                Assert.IsTrue(value.ToLowerInvariant() == expectedValue.ToString().ToLowerInvariant());
            }
        }

        [TestMethod]
        public void ListLevelValueExport_SkipingFewLevels_IsCorrectValueInHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            list.Levels[2].RestartAfterLevel = 0;

            for (int i = 0; i < 3; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            this.AddParagraphsToDocument(editor, list, 1); // index: 2
            this.AddParagraphsToDocument(editor, list, 2); // index: 2

            this.AddParagraphsToDocument(editor, list, 2); // index: 3
            this.AddParagraphsToDocument(editor, list, 0); // - restart level

            this.AddParagraphsToDocument(editor, list, 2); // index: 1

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> htmlList = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(4, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", htmlList[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", htmlList[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", htmlList[3].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> firstElementInFirstLevel = htmlList[1].ChildElements.ToList();
            Assert.AreEqual(4, firstElementInFirstLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInFirstLevel[0], 1);
            Assert.AreEqual("ol", firstElementInFirstLevel[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(firstElementInFirstLevel[2], 2);
            Assert.AreEqual("ol", firstElementInFirstLevel[3].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> firstElementInSecondLevel = firstElementInFirstLevel[1].ChildElements.ToList();
            Assert.AreEqual(1, firstElementInSecondLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInSecondLevel[0], 1);

            System.Collections.Generic.List<IDomElement> secondElementInSecondLevel = firstElementInFirstLevel[3].ChildElements.ToList();
            Assert.AreEqual(2, secondElementInSecondLevel.Count);
            this.AssertListLevelNameAndValue(secondElementInSecondLevel[0], 2, true);
            this.AssertListLevelNameAndValue(secondElementInSecondLevel[1], 3);

            System.Collections.Generic.List<IDomElement> secondElementInFirstLevel = htmlList[3].ChildElements.ToList();
            Assert.AreEqual(1, secondElementInFirstLevel.Count);
            Assert.AreEqual("ol", secondElementInFirstLevel[0].NodeName.ToLowerInvariant());

            firstElementInSecondLevel = secondElementInFirstLevel[0].ChildElements.ToList();
            Assert.AreEqual(1, firstElementInSecondLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInSecondLevel[0], 1, true);
        }

        [TestMethod]
        public void ListLevelValueExport_IsCorrectValueInHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            list.Levels[2].RestartAfterLevel = 0;

            for (int i = 0; i < 4; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            this.AddParagraphsToDocument(editor, list, 1); // index: 2
            this.AddParagraphsToDocument(editor, list, 2); // index: 2
            this.AddParagraphsToDocument(editor, list, 3); // index: 1

            this.AddParagraphsToDocument(editor, list, 2); // index: 3
            this.AddParagraphsToDocument(editor, list, 2); // index: 4
            this.AddParagraphsToDocument(editor, list, 0); // - restart level

            this.AddParagraphsToDocument(editor, list, 2); // index: 1
            this.AddParagraphsToDocument(editor, list, 3); // index: 1
            this.AddParagraphsToDocument(editor, list, 2); // index: 2

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> htmlList = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(4, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", htmlList[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", htmlList[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", htmlList[3].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> firstElementInFirstLevel = htmlList[1].ChildElements.ToList();
            Assert.AreEqual(4, firstElementInFirstLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInFirstLevel[0], 1);
            Assert.AreEqual("ol", firstElementInFirstLevel[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(firstElementInFirstLevel[2], 2);
            Assert.AreEqual("ol", firstElementInFirstLevel[3].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> firstElementInSecondLevel = firstElementInFirstLevel[1].ChildElements.ToList();
            Assert.AreEqual(4, firstElementInFirstLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInSecondLevel[0], 1);
            Assert.AreEqual("ol", firstElementInSecondLevel[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> firstElementInThirdLevel = firstElementInSecondLevel[1].ChildElements.ToList();
            Assert.AreEqual(1, firstElementInThirdLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInThirdLevel[0], 1);

            System.Collections.Generic.List<IDomElement> secondElementInSecondLevel = firstElementInFirstLevel[3].ChildElements.ToList();
            Assert.AreEqual(4, secondElementInSecondLevel.Count);
            this.AssertListLevelNameAndValue(secondElementInSecondLevel[0], 2, true);
            Assert.AreEqual("ol", secondElementInSecondLevel[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(secondElementInSecondLevel[2], 3);
            this.AssertListLevelNameAndValue(secondElementInSecondLevel[3], 4);

            System.Collections.Generic.List<IDomElement> secondElementInFirstLevel = htmlList[3].ChildElements.ToList();
            Assert.AreEqual(1, secondElementInFirstLevel.Count);
            Assert.AreEqual("ol", secondElementInFirstLevel[0].NodeName.ToLowerInvariant());

            firstElementInSecondLevel = secondElementInFirstLevel[0].ChildElements.ToList();
            Assert.AreEqual(3, firstElementInSecondLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInSecondLevel[0], 1, true);
            Assert.AreEqual("ol", firstElementInSecondLevel[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(firstElementInSecondLevel[2], 2);

            firstElementInThirdLevel = firstElementInSecondLevel[1].ChildElements.ToList();
            Assert.AreEqual(1, firstElementInThirdLevel.Count);
            this.AssertListLevelNameAndValue(firstElementInThirdLevel[0], 1);
        }

        [TestMethod]
        public void SixParagraphsInOneListDevidedIntoTwoGroupsFromTwoParagraphsWihtoutList_AreValuesCorrectInHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            this.AddParagraphsToDocument(editor, list, 0); // index: 1
            this.AddParagraphsToDocument(editor, list, 0); // index: 2
            this.AddParagraphsToDocument(editor, list, 0); // index: 3

            editor.InsertParagraph();
            editor.InsertParagraph();

            this.AddParagraphsToDocument(editor, list, 0); // index: 4
            this.AddParagraphsToDocument(editor, list, 0); // index: 5
            this.AddParagraphsToDocument(editor, list, 0); // index: 6

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(5, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[3].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[4].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> htmlList = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(3, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 1, true);
            Assert.AreEqual("li", htmlList[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[1], 2, true);
            Assert.AreEqual("li", htmlList[2].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[2], 3, true);

            htmlList = bodyChilds[4].ChildElements.ToList();
            Assert.AreEqual(3, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 4, true);
            Assert.AreEqual("li", htmlList[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[1], 5, true);
            Assert.AreEqual("li", htmlList[2].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[2], 6, true);
        }

        [TestMethod]
        public void TwoGroupsOfTreeParagraphsWithDifferentLevelsDevidedFromTwoParagraphsWithoutList_AreValuesCorrectInHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            this.AddParagraphsToDocument(editor, list, 0); // index: 1
            this.AddParagraphsToDocument(editor, list, 1); // index: 1
            this.AddParagraphsToDocument(editor, list, 1); // index: 2

            editor.InsertParagraph();
            editor.InsertText("Simple paragraph");
            editor.InsertParagraph();
            editor.InsertText("Simple paragraph");

            this.AddParagraphsToDocument(editor, list, 0); // index: 2
            this.AddParagraphsToDocument(editor, list, 1); // index: 1
            this.AddParagraphsToDocument(editor, list, 1); // index: 2

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(5, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[3].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[4].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> htmlList = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(2, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 1, true);
            Assert.AreEqual("ol", htmlList[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> childElements = htmlList[1].ChildElements.ToList();
            Assert.AreEqual(2, childElements.Count);
            Assert.AreEqual("li", childElements[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(childElements[0], 1, true);
            Assert.AreEqual("li", childElements[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(childElements[1], 2, true);

            htmlList = bodyChilds[4].ChildElements.ToList();
            Assert.AreEqual(2, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 2, true);
            Assert.AreEqual("ol", htmlList[1].NodeName.ToLowerInvariant());

            childElements = htmlList[1].ChildElements.ToList();
            Assert.AreEqual(2, childElements.Count);
            Assert.AreEqual("li", childElements[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(childElements[0], 1, true);
            Assert.AreEqual("li", childElements[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(childElements[1], 2, true);
        }

        [TestMethod]
        public void ThreeGroupsOfTwoParagraphsDevidedFromOneParagraphWithoutListTheLastGroupIsWithDifferentList_AreValuesCorrectInHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list1.Levels[i].NumberingStyle = NumberingStyle.Decimal;
                list2.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            this.AddParagraphsToDocument(editor, list1, 0); // index: 1
            this.AddParagraphsToDocument(editor, list1, 0); // index: 2

            editor.InsertParagraph();
            editor.InsertText("Simple paragraph");

            this.AddParagraphsToDocument(editor, list1, 0); // index: 3
            this.AddParagraphsToDocument(editor, list1, 0); // index: 4

            editor.InsertParagraph();
            editor.InsertText("Simple paragraph");

            // different list
            this.AddParagraphsToDocument(editor, list2, 0); // index: 1
            this.AddParagraphsToDocument(editor, list2, 0); // index: 1

            // Act
            CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(6, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[3].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[4].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[5].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<IDomElement> htmlList = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(2, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 1, true);
            Assert.AreEqual("li", htmlList[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[1], 2, true);

            htmlList = bodyChilds[3].ChildElements.ToList();
            Assert.AreEqual(2, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 3, true);
            Assert.AreEqual("li", htmlList[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[1], 4, true);

            htmlList = bodyChilds[5].ChildElements.ToList();
            Assert.AreEqual(2, htmlList.Count);
            Assert.AreEqual("li", htmlList[0].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[0], 1, true);
            Assert.AreEqual("li", htmlList[1].NodeName.ToLowerInvariant());
            this.AssertListLevelNameAndValue(htmlList[1], 2, true);
        }
    }
}
