﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Lists;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export
{
    [TestClass]
    public class ListsExportTests : ListsExportBase
    {
        [TestMethod]
        public void ListWithNineNestedLevles_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());


            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }
        }

        [TestMethod]
        public void TwoListsWithNineNestedLevelsSeparatedFromParagraph_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list1.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            p = editor.InsertParagraph();
            editor.InsertText("Separator");

            p = editor.InsertParagraph();
            editor.InsertText("Separator");

            p = editor.InsertParagraph();
            editor.InsertText("Separator");

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list2.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(6, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[3].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[4].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[5].NodeName.ToLowerInvariant());


            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }

            childs = bodyChilds[5].ChildElements.ToList();
            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }
        }

        [TestMethod]
        public void ListWithOneNestedLevelFollowedByParagraphAndListWithTwoNestedLevels_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list1 = document.Lists.Add(ListTemplateType.NumberedDefault);
            List list2 = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("Hello 0").Paragraph;
            p.ListId = list1.Id;
            p.ListLevel = 0;

            p = editor.InsertParagraph();
            editor.InsertText("Hello 1");
            p.ListId = list1.Id;
            p.ListLevel = 1;

            p = editor.InsertParagraph();
            editor.InsertText("Separator");

            p = editor.InsertParagraph();
            p.ListId = list2.Id;
            p.ListLevel = 0;
            editor.InsertText("Hello 3");

            p = editor.InsertParagraph();
            p.ListId = list2.Id;
            p.ListLevel = 1;
            editor.InsertText("Hello 4");

            p = editor.InsertParagraph();
            p.ListId = list2.Id;
            p.ListLevel = 2;
            editor.InsertText("Hello 5");

            p = editor.InsertParagraph();
            p.ListId = list2.Id;
            p.ListLevel = 1;
            editor.InsertText("Hello 6");

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(3, bodyChilds.Count);
            Assert.AreEqual("ol", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("p", bodyChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[2].NodeName.ToLowerInvariant());

            // the first list

            CsQuery.IDomElement firstList = bodyChilds[0];

            System.Collections.Generic.List<CsQuery.IDomElement> firstListChilds = firstList.ChildElements.ToList();
            Assert.AreEqual(2, firstListChilds.Count);
            Assert.AreEqual("li", firstListChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", firstListChilds[1].NodeName.ToLowerInvariant());

            // first inner list
            Assert.AreEqual(1, firstListChilds[1].ChildElements.ToList().Count);
            Assert.AreEqual("li", firstListChilds[1].ChildElements.ToList()[0].NodeName.ToLowerInvariant());

            // the second list
            CsQuery.IDomElement secondList = bodyChilds[2];

            System.Collections.Generic.List<CsQuery.IDomElement> secondListChilds = secondList.ChildElements.ToList();
            Assert.AreEqual(2, secondListChilds.Count);
            Assert.AreEqual("li", secondListChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", secondListChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> innerListChilds = secondListChilds[1].ChildElements.ToList();
            Assert.AreEqual(3, innerListChilds.Count);
            Assert.AreEqual("li", innerListChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", innerListChilds[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", innerListChilds[2].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> innerInerListChilds = innerListChilds[1].ChildElements.ToList();
            Assert.AreEqual(1, innerInerListChilds.Count);
            Assert.AreEqual("li", innerInerListChilds[0].NodeName.ToLowerInvariant());
        }

        [TestMethod]
        public void ListWithNineIncreasinglyNestedLevelesFollowedByNineDecreasinglyLevels_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            for (int i = 7; i >= 0; i--)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());


            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(3, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[2].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(3, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
                Assert.AreEqual("li", childs[2].NodeName.ToLowerInvariant());
            }
        }

        [TestMethod]
        public void ListWithNineElementsFromOneLevelFollowedByNineElementsFromTheNextLevel_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 9; j++)
                {
                    p = editor.InsertParagraph();
                    p.ListId = list.Id;
                    p.ListLevel = i;
                    editor.InsertText("Hello " + i + "-" + j);
                }
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(10, childs.Count);

            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 9; j++)
                {
                    Assert.AreEqual("li", childs[j].NodeName.ToLowerInvariant());
                }

                if (i < 8)
                {
                    Assert.AreEqual("ol", childs[9].NodeName.ToLowerInvariant());
                    childs = childs[9].ChildElements.ToList();
                }
            }
        }

        private void TestAndAssertListLevelBullet(NumberingStyle numberingStyle, string expectedResult)
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = numberingStyle;

                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> childs = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("p", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: " + expectedResult + ";");

            for (int i = 0; i < 8; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
                Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: " + expectedResult + ";");
            }
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleDecimal_IsExportedAsDecimal()
        {
            this.TestAndAssertListLevelBullet(NumberingStyle.Decimal, "decimal");
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleLowerLetter_IsExportedAsLowerLatin()
        {
            this.TestAndAssertListLevelBullet(NumberingStyle.LowerLetter, "lower-latin");
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleUpperLetter_IsExportedAsUpperLatin()
        {
            this.TestAndAssertListLevelBullet(NumberingStyle.UpperLetter, "upper-latin");
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleUpperRoman_IsExportedAsUpperRoman()
        {
            this.TestAndAssertListLevelBullet(NumberingStyle.UpperRoman, "upper-roman");
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleLowerRoman_IsExportedAsLowerRoman()
        {
            this.TestAndAssertListLevelBullet(NumberingStyle.LowerRoman, "lower-roman");
        }

        [TestMethod]
        public void ListLevelWithNumberingStyleBullet_IsExportedAsCorespondingBullet()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.BulletDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            string[] bulletTypes = new string[] { "disc", "circle", "square" };

            System.Collections.Generic.List<CsQuery.IDomElement> childs = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("p", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ul", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: " + bulletTypes[0] + ";");

            for (int i = 1; i < 8; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ul", childs[1].NodeName.ToLowerInvariant());

                Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: " + bulletTypes[i % 3] + ";");
            }
        }

        [TestMethod]
        public void ListLevelWithNumberingTextFormatSymbol_IsExportedAsDiscBullet()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.BulletDefault);
            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberTextFormat = "$";

                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> childs = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, childs.Count);
            Assert.AreEqual("p", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ul", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: disc;");

            for (int i = 1; i < 8; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ul", childs[1].NodeName.ToLowerInvariant());

                Assert.AreEqual(childs[1].Attributes["style"], "list-style-type: disc;");
            }
        }

        [TestMethod]
        public void ListWithNineIncreasinglyNestedLevelesFollowedByNineIncreasinglyNestedLevels_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            editor.InsertText("Simple paragraph");

            for (int i = 0; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            for (int i = 0; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(4, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[3].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }

            childs = new System.Collections.Generic.List<CsQuery.IDomElement>(2);
            childs.Add(null);
            childs.Add(bodyChilds[1].ChildElements.ToList()[3]);

            for (int i = 0; i < 7; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }
        }

        [TestMethod]
        public void ListWithNineIncreadinglyNestedLevelsEachLevelWithTwoItemsFollowedByNineDecreasinglyNestedLevelsEachWithTwoItems_IsCorrectlyExportedToHtml()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            editor.InsertText("Simple paragraph");

            for (int i = 0; i < 9; i++)
            {
                this.AddParagraphsToDocument(editor, list, i);
                this.AddParagraphsToDocument(editor, list, i);
            }

            for (int i = 8; i >= 0; i--)
            {
                this.AddParagraphsToDocument(editor, list, i);
                this.AddParagraphsToDocument(editor, list, i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(5, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[3].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[4].NodeName.ToLowerInvariant());

            for (int i = 0; i < 7; i++)
            {
                childs = childs[2].ChildElements.ToList();
                Assert.AreEqual(5, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("li", childs[1].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[2].NodeName.ToLowerInvariant());
                Assert.AreEqual("li", childs[3].NodeName.ToLowerInvariant());
                Assert.AreEqual("li", childs[4].NodeName.ToLowerInvariant());
            }

            childs = childs[2].ChildElements.ToList();
            Assert.AreEqual(4, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[3].NodeName.ToLowerInvariant());
        }

        [TestMethod]
        public void ListWithNestedLevelsWithSkeepingTheSecondAndThirdLevels()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);

            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;

            this.AddParagraphsToDocument(editor, list, 0);

            for (int i = 3; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            this.AddParagraphsToDocument(editor, list, 0);

            for (int i = 3; i < 9; i++)
            {
                p = editor.InsertParagraph();
                p.ListId = list.Id;
                p.ListLevel = i;
                editor.InsertText("Hello " + i);
            }

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(2, bodyChilds.Count);
            Assert.AreEqual("p", bodyChilds[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", bodyChilds[1].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> childs = bodyChilds[1].ChildElements.ToList();
            Assert.AreEqual(4, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            Assert.AreEqual("li", childs[2].NodeName.ToLowerInvariant());
            Assert.AreEqual("ol", childs[3].NodeName.ToLowerInvariant());

            childs = childs[1].ChildElements.ToList();
            Assert.AreEqual(1, childs.Count);
            Assert.AreEqual("ol", childs[0].NodeName.ToLowerInvariant());

            childs = childs[0].ChildElements.ToList();
            Assert.AreEqual(1, childs.Count);
            Assert.AreEqual("ol", childs[0].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> elements = childs;
            childs = new System.Collections.Generic.List<CsQuery.IDomElement>(2);
            childs.Add(null);
            childs.Add(elements[0]);

            for (int i = 0; i < 5; i++)
            {
                childs = childs[1].ChildElements.ToList();
                Assert.AreEqual(2, childs.Count);
                Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
                Assert.AreEqual("ol", childs[1].NodeName.ToLowerInvariant());
            }

            childs = childs[1].ChildElements.ToList();
            Assert.AreEqual(1, childs.Count);
            Assert.AreEqual("li", childs[0].NodeName.ToLowerInvariant());
        }

        [TestMethod]
        public void ParagraphWithStyleAndWithList_IsCorrectInHmtl()
        {
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Style style = new Style("PStyle", StyleType.Paragraph);
            style.CharacterProperties.ForegroundColor.LocalValue = new ThemableColor(Colors.Red);
            document.StyleRepository.Add(style);

            List list = document.Lists.Add(ListTemplateType.NumberedDefault);
            for (int i = 0; i < 9; i++)
            {
                list.Levels[i].NumberingStyle = NumberingStyle.Decimal;
            }

            Paragraph p = editor.InsertText("Simple paragraph").Paragraph;
            p.ListId = list.Id;
            p.ListLevel = 0;
            p.StyleId = style.Id;

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomElement> bodyChilds = cq.Document.Body.ChildElements.ToList();

            Assert.AreEqual(1, bodyChilds.Count);
            Assert.AreEqual("ol", bodyChilds[0].NodeName.ToLowerInvariant());

            System.Collections.Generic.List<CsQuery.IDomElement> childElements = bodyChilds[0].ChildElements.ToList();
            Assert.AreEqual(1, childElements.Count);
            Assert.AreEqual("li", childElements[0].NodeName.ToLowerInvariant());

            Assert.IsTrue(childElements[0].Classes.Contains(style.Id));
        }
    }
}
