﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export
{
    [TestClass]
    public class ParagraphSpacingExportTests
    {
        [TestMethod]
        public void ParagraphWithAutomaticSpacingAfter_SpacingAfterIsNotExported()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("text");
            paragraph.Spacing.SpacingAfter = 40;
            paragraph.Spacing.AutomaticSpacingAfter = true;

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> paragraphs = cq.Select("p").ToList();
            Assert.AreEqual(1, paragraphs.Count);
            Assert.IsNull(paragraphs[0].Style["margin-bottom"]);
        }

        [TestMethod]
        public void ParagraphWithoutAutomaticSpacingAfter_SpacingAfterIsExported()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("text");
            paragraph.Spacing.SpacingAfter = 40;

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> paragraphs = cq.Select("p").ToList();
            Assert.AreEqual(1, paragraphs.Count);
            Assert.AreEqual("40px", paragraphs[0].Style["margin-bottom"]);
        }

        [TestMethod]
        public void ParagraphWithAutomaticSpacingBefore_SpacingBeforeIsNotExported()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("text");
            paragraph.Spacing.SpacingBefore = 40;
            paragraph.Spacing.AutomaticSpacingBefore = true;

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> paragraphs = cq.Select("p").ToList();
            Assert.AreEqual(1, paragraphs.Count);
            Assert.IsNull(paragraphs[0].Style["margin-top"]);
        }

        [TestMethod]
        public void ParagraphWithoutAutomaticSpacingBefore_SpacingBeforeIsExported()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Paragraph paragraph = document.Sections.AddSection().Blocks.AddParagraph();
            paragraph.Inlines.AddRun("text");
            paragraph.Spacing.SpacingBefore = 40;

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> paragraphs = cq.Select("p").ToList();
            Assert.AreEqual(1, paragraphs.Count);
            Assert.AreEqual("40px", paragraphs[0].Style["margin-top"]);
        }
    }
}
