﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if NETSTANDARD
using Telerik.Documents.Media;
#else
using System.Windows.Media;
#endif
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using PreMailer.Net;
using Telerik.Windows.Documents.Flow.FormatProviders.Html;

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export.Settings
{
    [TestClass]
    public class BordersMinimalThicknessTests
    {
        [TestMethod]
        public void DefautValue_TableBordersAreExportedWithRealValue()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Table table = new Table(document, 1, 1);
            document.Sections.AddSection().Blocks.Add(table);
            table.Borders = new TableBorders(new Border(0.7, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> tables = cq.Select("table").ToList();
            Assert.AreEqual(1, tables.Count);

            StyleClass style = new CssParser().ParseStyleClass("temp", tables[0].Attributes["style"]);

            Assert.IsTrue(style.Attributes["border"].Contains("0.7px"));
        }

        [TestMethod]
        public void DefautValue_TableCellBordersAreExportedWithRealValue()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            TableCell tableCell = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(new Border(0.7, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> cells = cq.Select("td").ToList();
            Assert.AreEqual(1, cells.Count);

            StyleClass style = new CssParser().ParseStyleClass("temp", cells[0].Attributes["style"]);

            Assert.IsTrue(style.Attributes["border"].Contains("0.7px"));
        }

        [TestMethod]
        public void IsSetToOne_TableBordersBelowOnePixelAreExportedWithMinimalValue()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            Table table = new Table(document, 1, 1);
            document.Sections.AddSection().Blocks.Add(table);
            table.Borders = new TableBorders(new Border(0.7, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document, new HtmlExportSettings() { BordersMinimalThickness = 1 });

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> tables = cq.Select("table").ToList();
            Assert.AreEqual(1, tables.Count);

            StyleClass style = new CssParser().ParseStyleClass("temp", tables[0].Attributes["style"]);

            Assert.IsTrue(style.Attributes["border"].Contains("1px"));
        }

        [TestMethod]
        public void IsSetToOne_TableCellBordersBelowOnePixelAreExportedWithMinimalValue()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            TableCell tableCell = document.Sections.AddSection().Blocks.AddTable().Rows.AddTableRow().Cells.AddTableCell();
            tableCell.Borders = new TableCellBorders(new Border(0.7, BorderStyle.Single, new ThemableColor(Colors.Black)));

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document, new HtmlExportSettings() { BordersMinimalThickness = 1 });

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> cells = cq.Select("td").ToList();
            Assert.AreEqual(1, cells.Count);

            StyleClass style = new CssParser().ParseStyleClass("temp", cells[0].Attributes["style"]);

            Assert.IsTrue(style.Attributes["border"].Contains("1px"));
        }
    }
}
