﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;

namespace Telerik.Windows.Documents.Flow.InternalTests.Integration.Html.Export
{
    [TestClass]
    public class SpanExportTests
    {
        [TestMethod]
        public void Run_WhenContainsOnlyTab_IsExportedAsTwoNonBreakingSpaceChars()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("\t");

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> spans = cq.Select("span").ToList();
            Assert.AreEqual(1, spans.Count);
            Assert.AreEqual(string.Format("{0}{1}{0}{1}", (char)160, (char)32), spans[0].TextContent);
        }

        [TestMethod]
        public void Run_WhenContainsSingleTab_IsExportedAsTwoNonBreakingSpaceChars()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test\ttest");

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> spans = cq.Select("span").ToList();
            Assert.AreEqual(1, spans.Count);
            Assert.AreEqual(string.Format("test{0}{1}{0}{1}test", (char)160, (char)32), spans[0].TextContent);
        }

        [TestMethod]
        public void Run_WhenContainsMultipleTabs_IsExportedAsMultpleNonBreakingSpaceChars()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.AddRun("test\t\t\ttest");

            // Act
            CsQuery.CQ cq = ExportHelper.ExportDocumentToCq(document);

            // Assert
            System.Collections.Generic.List<CsQuery.IDomObject> spans = cq.Select("span").ToList();
            Assert.AreEqual(1, spans.Count);
            Assert.AreEqual(string.Format("test{0}{1}{0}{1}{0}{1}{0}{1}{0}{1}{0}{1}test", (char)160, (char)32), spans[0].TextContent);
        }
    }
}