﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Collections;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Collections
{
    [TestClass]
    public class CommentsCollectionTests
    {
        [TestMethod]
        public void AddComment_AddsCommentWithCorrectParentAndDocument()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            CommentCollection commentCollection = new CommentCollection(document);

            // Act
            Comment comment = commentCollection.AddComment();

            // Assert
            Assert.AreEqual(document, comment.Document);
            Assert.AreEqual(document, comment.Parent);
        }

        [TestMethod]
        public void Remove_RemovesCommentAndInsertedCommentRangesInDocument()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();

            CommentCollection commentCollection = new CommentCollection(document);
            Comment comment = commentCollection.AddComment();
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.Add(comment.CommentRangeStart);
            document.Sections.AddSection().Blocks.AddParagraph().Inlines.Add(comment.CommentRangeEnd);

            // Act
            commentCollection.Remove(comment);

            // Assert
            Assert.IsFalse(document.EnumerateChildrenOfType<CommentRangeStart>().Any());
            Assert.IsFalse(document.EnumerateChildrenOfType<CommentRangeEnd>().Any());
        }
    }
}