﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Collections;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Collections.Common
{
    [TestClass]
    public class InlineCollectionTests : DocumentElementCollectionTestsBase<InlineBase, Paragraph>
    {
        public override DocumentElementCollection<InlineBase, Paragraph> GetDocumentElementCollection(Paragraph owner)
        {
            return owner.Inlines;
        }

        public override InlineBase CreateChild(RadFlowDocument document)
        {
            return new Run(document);
        }

        public override Paragraph CreateOwner(RadFlowDocument document)
        {
            return new Paragraph(document);
        }

        [TestMethod]
        public void Add_WhenElementIsFromAnotherDocument_ThrowsArgumentException()
        {
            this.AssertAdd_WhenElementIsFromAnotherDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void Add_WhenElementIsFromTheSameDocument_AddsTheElement()
        {
            this.AssertAdd_WhenElementIsFromTheSameDocument_AddsTheElement();
        }

        [TestMethod]
        public void Add_WhenElementIsNull_ThrowsArgumentNullException()
        {
            this.AssertAdd_WhenElementIsNull_ThrowsArgumentNullException();
        }

        [TestMethod]
        public void IndexerSet_WhenElementIsFromAnotherDocument_ThrowsArgumentException()
        {
            this.AssertIndexerSet_WhenElementIsFromAnotherDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void IndexerSet_WhenElementIsFromTheSameDocument_AddsTheElement()
        {
            this.AssertIndexerSet_WhenElementIsFromTheSameDocument_AddsTheElement();
        }

        [TestMethod]
        public void IndexerSet_WhenElementIsNull_ThrowsArgumentNullException()
        {
            this.AssertIndexerSet_WhenElementIsNull_ThrowsArgumentNullException();
        }

        [TestMethod]
        public void Insert_WhenElementIsFromAnotherDocument_ThrowsArgumentException()
        {
            this.AssertInsert_WhenElementIsFromAnotherDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void Insert_WhenElementIsFromTheSameDocument_AddsTheElement()
        {
            this.AssertInsert_WhenElementIsFromTheSameDocument_AddsTheElement();
        }

        [TestMethod]
        public void Insert_WhenElementIsNull_ThrowsArgumentNullException()
        {
            this.AssertInsert_WhenElementIsNull_ThrowsArgumentNullException();
        }

        [TestMethod]
        public void Add_WhenElementIsFromTheSameDocument_SetsTheParent()
        {
            this.AssertAdd_WhenElementIsFromTheSameDocument_SetsTheParent();
        }

        [TestMethod]
        public void IndexerSet_WhenElementIsFromTheSameDocument_SetsTheParent()
        {
            this.AssertIndexerSet_WhenElementIsFromTheSameDocument_SetsTheParent();
        }

        [TestMethod]
        public void Insert_WhenElementIsFromTheSameDocument_SetsTheParentToBothElements()
        {
            this.AssertInsert_WhenElementIsFromTheSameDocument_SetsTheParentToBothElements();
        }

        [TestMethod]
        public void Remove_WhenElementIsAdded_ClearsTheParent()
        {
            this.AssertRemove_WhenElementIsAdded_ClearsTheParent();
        }

        [TestMethod]
        public void RemoveAt_WhenElementIsAdded_ClearsTheParent()
        {
            this.AssertRemoveAt_WhenElementIsAdded_ClearsTheParent();
        }

        [TestMethod]
        public void Add_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException()
        {
            this.AssertAdd_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void IndexerSet_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException()
        {
            this.AssertIndexerSet_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void Insert_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException()
        {
            this.AssertInsert_WhenElementIsInAnotherCollectionInTheSameDocument_ThrowsArgumentException();
        }

        [TestMethod]
        public void Add_WhenElementIsAlreadyAddedToSameCollection_ThrowsArgumentException()
        {
            this.AssertAdd_WhenElementIsAlreadyAddedToSameCollection_ThrowsArgumentException();
        }

        [TestMethod]
        public void Insert_WhenElementIsAlreadyAddedToSameCollection_ThrowsArgumentException()
        {
            this.AssertInsert_WhenElementIsAlreadyAddedToSameCollection_ThrowsArgumentException();
        }
    }
}
