﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Fields.Expressions;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields.Expressions
{
    [TestClass]
    public class ExpressionErrorsTests
    {
        [TestMethod]
        public void ExpressionParser_WhenDivideByZero_ResultHasAnError()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "=100/0";

            // Act 
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        [TestMethod]
        public void ExpressionParser_WhenPowerZeroOnNegativeNumber_ResultHasAnError()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "=0^-1";

            // Act 
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        [TestMethod]
        public void ExpressionParser_WhenPowerNegativeNumberOnNotIntegerNumber_ResultHasAnError()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "=-1^-1.5";

            // Act 
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        [TestMethod]
        public void ExpressionParser_WhenUndefinedBookmarkIsParsed_ResultHasAnError()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "=BMK";

            // Act 
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        [TestMethod]
        public void ExpressionParser_WhenParseMultipleStrings_ExpressionExceptionIsThrown()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "=BMK test test2";

            // Act
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        [TestMethod]
        public void ExpressionParser_WhenParseFunctionWithSyntaxError_ExpressionExceptionIsThrown()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            string input = "SUM 2 + 3)";

            // Act
            ExpressionResult result = ExpressionParser.Parse(input, document).GetResult();

            // Assert
            this.AssertExpressionResultHasError(result);
        }

        private void AssertExpressionResultHasError(ExpressionResult result)
        {
            Assert.AreEqual(0, result.Value);
            Assert.IsNotNull(result.Error);
        }
    }
}