﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Text;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields.Expressions;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields.Expressions
{
    [TestClass]
    public class ExpressionParserTests
    {
        [TestMethod]
        public void Constant_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3", 3),
                new ExpressionTestCase("123456", 123456),
                new ExpressionTestCase("3.6", 3.6),
                new ExpressionTestCase("3.12345", 3.12345),
                new ExpressionTestCase(".3", 0.3),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void UnaryMinus_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("-3", -3),
                new ExpressionTestCase("-123456", -123456),
                new ExpressionTestCase("-3.6", -3.6),
                new ExpressionTestCase("-0.3", -0.3),
                new ExpressionTestCase("-(3)", -3),
                new ExpressionTestCase("-(-3)", 3),
                new ExpressionTestCase("--3", 3),
                new ExpressionTestCase("-BMK", -123),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void PercentOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("10 %", 0.1),
                new ExpressionTestCase("0%", 0.0),
                new ExpressionTestCase("110%", 1.1),
                new ExpressionTestCase("-8% ", -0.08),
                new ExpressionTestCase("-8%", -0.08),
                new ExpressionTestCase("8 %", 0.08),
                new ExpressionTestCase(" -8%", -0.08),
                new ExpressionTestCase("(10)%", 0.1),
                new ExpressionTestCase("-(10)%", -0.1),
                new ExpressionTestCase("(-10)%", -0.1),
                new ExpressionTestCase(" (10) %", 0.1),
                new ExpressionTestCase("- (10) %", -0.1),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void AdditionOperator_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3+4", 7),
                new ExpressionTestCase("3 +4", 7),
                new ExpressionTestCase("3+ 4", 7),
                new ExpressionTestCase("3 + 4", 7),
                new ExpressionTestCase("(3 + 4)", 7),
                new ExpressionTestCase("-3 + 4", 1),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void SubstractOperator_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3-4", -1),
                new ExpressionTestCase("3 -4", -1),
                new ExpressionTestCase("3- 4", -1),
                new ExpressionTestCase("3 - 4", -1),
                new ExpressionTestCase("(3 - 4)", -1),
                new ExpressionTestCase("-3 - 4", -7),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void DivideOperator_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3/4", 0.75),
                new ExpressionTestCase("3 /4", 0.75),
                new ExpressionTestCase("3/ 4", 0.75),
                new ExpressionTestCase("3 / 4", 0.75),
                new ExpressionTestCase("(3 / 4)", 0.75),
                new ExpressionTestCase("-3 / 4", -0.75),
                new ExpressionTestCase("-3 / 0", 0),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void PowerOperator_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("2^3", 8),
                new ExpressionTestCase("2 ^3", 8),
                new ExpressionTestCase("2^ 3", 8),
                new ExpressionTestCase("2 ^ 3", 8),
                new ExpressionTestCase("(2 ^ 3)", 8),
                new ExpressionTestCase("-2 ^ 3", -8),
                new ExpressionTestCase("-2 ^ 3", -8),
                new ExpressionTestCase("16 ^ (1 / 2)", 4),
                new ExpressionTestCase("16 ^ 0.5", 4),
                new ExpressionTestCase("0 ^ 0", 1),
                new ExpressionTestCase("2 ^ -3", 0.125),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void EqualOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3=4", 0),
                new ExpressionTestCase("3 =4", 0),
                new ExpressionTestCase("3= 4", 0),
                new ExpressionTestCase("3 = 4", 0),
                new ExpressionTestCase("(3 = 4)", 0),
                new ExpressionTestCase("-4 = 4", 0),
                new ExpressionTestCase("4=4", 1),
                new ExpressionTestCase("0=0", 1),
                new ExpressionTestCase("4 =4", 1),
                new ExpressionTestCase("4= 4", 1),
                new ExpressionTestCase("4 = 4", 1),
                new ExpressionTestCase("(4 = 4)", 1),
                new ExpressionTestCase("-4 = 4", 0),
                new ExpressionTestCase("BMK = 123", 1),
                new ExpressionTestCase("-BMK = BMK", 0),
                new ExpressionTestCase("-BMK = -BMK", 1),
                new ExpressionTestCase("-BMK = -BMKSAME", 1),
                new ExpressionTestCase("BMK = BMKSAME", 1),
                new ExpressionTestCase("-BMK = BMKSAME", 0),
                new ExpressionTestCase("-BMKOTHER = BMK", 0),
                new ExpressionTestCase("BMKOTHER = BMK", 0),
            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void LessThanOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3<4", 1),
                new ExpressionTestCase("3 <4", 1),
                new ExpressionTestCase("3< 4", 1),
                new ExpressionTestCase("3 < 4", 1),
                new ExpressionTestCase("(3 < 4)", 1),
                new ExpressionTestCase("-4 < 4", 1),
                new ExpressionTestCase("4<4", 0),
                new ExpressionTestCase("5 <4", 0),
                new ExpressionTestCase("5< 4", 0),
                new ExpressionTestCase("5 < 4", 0),
                new ExpressionTestCase("(5 < 4)", 0),
                new ExpressionTestCase("-5 < 4", 1),
                new ExpressionTestCase("BMK < 123", 0),
                new ExpressionTestCase("-BMK < BMK", 1),
                new ExpressionTestCase("-BMK < -BMKSAME", 0),
                new ExpressionTestCase("-BMK < BMKSAME", 1),
                new ExpressionTestCase("-BMKOTHER < BMK", 1),
                new ExpressionTestCase(" BMK < BMKOTHER ", 0),
            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void LessThanOrEqualToOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3<=4", 1),
                new ExpressionTestCase("3 <=4", 1),
                new ExpressionTestCase("3<= 4", 1),
                new ExpressionTestCase("3 <= 4", 1),
                new ExpressionTestCase("(3 <= 4)", 1),
                new ExpressionTestCase("-4 <= 4", 1),
                new ExpressionTestCase("4<=4", 1),
                new ExpressionTestCase("5 <=4", 0),
                new ExpressionTestCase("5<= 4", 0),
                new ExpressionTestCase("5 <= 4", 0),
                new ExpressionTestCase("(5 <= 4)", 0),
                new ExpressionTestCase("-5 <= 4", 1),
                new ExpressionTestCase("BMK <= 123", 1),
                new ExpressionTestCase("-BMK <= BMK", 1),
                new ExpressionTestCase("-BMK <= -BMKSAME", 1),
                new ExpressionTestCase("-BMK <= BMKSAME", 1),
                new ExpressionTestCase("-BMKOTHER <= BMK", 1),
                new ExpressionTestCase(" BMK <= BMKOTHER ", 0),
            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void GreaterThanOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3>4", 0),
                new ExpressionTestCase("3 >4", 0),
                new ExpressionTestCase("3> 4", 0),
                new ExpressionTestCase("3 > 4", 0),
                new ExpressionTestCase("(3 > 4)", 0),
                new ExpressionTestCase("-4 > 4", 0),
                new ExpressionTestCase("4>4", 0),
                new ExpressionTestCase("5 >4", 1),
                new ExpressionTestCase("5> 4", 1),
                new ExpressionTestCase("5 > 4", 1),
                new ExpressionTestCase("(5 > 4)", 1),
                new ExpressionTestCase("-5 > 4", 0),
                new ExpressionTestCase("BMK > 123", 0),
                new ExpressionTestCase("-BMK > BMK", 0),
                new ExpressionTestCase("-BMK > -BMKSAME", 0),
                new ExpressionTestCase("-BMK > BMKSAME", 0),
                new ExpressionTestCase("-BMKOTHER > BMK", 0),
                new ExpressionTestCase(" BMK > BMKOTHER ", 1),

            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3>=4", 0),
                new ExpressionTestCase("3 >=4", 0),
                new ExpressionTestCase("3>= 4", 0),
                new ExpressionTestCase("3 >= 4", 0),
                new ExpressionTestCase("(3 >= 4)", 0),
                new ExpressionTestCase("-4 >= 4", 0),
                new ExpressionTestCase("4>=4", 1),
                new ExpressionTestCase("5 >=4", 1),
                new ExpressionTestCase("5>= 4", 1),
                new ExpressionTestCase("5 >= 4", 1),
                new ExpressionTestCase("(5 >= 4)", 1),
                new ExpressionTestCase("-5 >= 4", 0),
                 new ExpressionTestCase("BMK >= 123", 1),
                new ExpressionTestCase("-BMK >= BMK", 0),
                new ExpressionTestCase("-BMK >= -BMKSAME", 1),
                new ExpressionTestCase("-BMK >= BMKSAME", 0),
                new ExpressionTestCase("-BMKOTHER >= BMK", 0),
                new ExpressionTestCase(" BMK >= BMKOTHER ", 1),
            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void NotEqualOperator_IsEvaluatedCorrectly()
        {
            RadFlowDocument document = this.InsertMultipleBookmarks();

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("3<>4", 1),
                new ExpressionTestCase("3 <>4", 1),
                new ExpressionTestCase("3<> 4", 1),
                new ExpressionTestCase("3 <> 4", 1),
                new ExpressionTestCase("(3 <> 4)", 1),
                new ExpressionTestCase("-4 <> 4", 1),
                new ExpressionTestCase("4<>4", 0),
                new ExpressionTestCase("5 <>4", 1),
                new ExpressionTestCase("5<> 4", 1),
                new ExpressionTestCase("5 <> 4", 1),
                new ExpressionTestCase("(5 <> 4)", 1),
                new ExpressionTestCase("-5 <> 4", 1),
                new ExpressionTestCase("-5 <> -5", 0),
                 new ExpressionTestCase("BMK <> 123", 0),
                new ExpressionTestCase("-BMK <> BMK", 1),
                new ExpressionTestCase("-BMK <> -BMKSAME", 0),
                new ExpressionTestCase("-BMK <> BMKSAME", 1),
                new ExpressionTestCase("-BMKOTHER <> BMK", 1),
                new ExpressionTestCase(" BMK <> BMKOTHER ", 1),
            };

            this.AssertTestCases(tests, document);
        }

        [TestMethod]
        public void SumFunction_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("SUM (3)", 3),
                new ExpressionTestCase("SUM(3,4)", 7),
                new ExpressionTestCase("SUM (3,4,5)", 12),
                new ExpressionTestCase("SUM  (3,4,5)", 12),
                new ExpressionTestCase("SUM (3,4,5,6)", 18),
                new ExpressionTestCase("SUM( 3,4)", 7),
                new ExpressionTestCase("SUM(3 ,4)", 7),
                new ExpressionTestCase("SUM(3, 4)", 7),
                new ExpressionTestCase("SUM(3,4 )", 7),
                new ExpressionTestCase("SUM(3,4+1 )", 8),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void AbsFunction_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("ABS(3)", 3),
                new ExpressionTestCase("ABS(-3)", 3),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void ModFunction_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("MOD   (21,5)", 1),
                new ExpressionTestCase("MOD(21,-5)", 1),
                new ExpressionTestCase("MOD (-21,5)", -1),
                new ExpressionTestCase("MOD (-21,-5)", -1),
                new ExpressionTestCase("MOD(21.5,5)", 1.5),
                new ExpressionTestCase("MOD(23,5.5)", 1),
                new ExpressionTestCase("MOD(23.5,5.5)", 1.5),
                new ExpressionTestCase("MOD ( 21,5)", 1),
                new ExpressionTestCase("MOD(21 ,5)", 1),
                new ExpressionTestCase("MOD (21, 5)", 1),

            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void AndFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("AND (1>=BMK, 100> 0.1)", 0),
                new ExpressionTestCase("AND(BMK>1, 100> 0.1)", 1),
                new ExpressionTestCase("AND (BMK>BMK*2, 100% > 0.5)", 0),
                new ExpressionTestCase("AND(BMK >= BMK, 100% > 0.5)", 1),
                new ExpressionTestCase("AND(5>3,3>=3)", 1),
                new ExpressionTestCase("AND(-3 < -2 , -3 < -2)", 1),
                new ExpressionTestCase("AND(0=5, 0 = 0)", 0),
                new ExpressionTestCase("AND(0 =0, 0= 5)", 0),
                new ExpressionTestCase("AND(0=0,0=0)", 1),

            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void AverageFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("AVERAGE(1,1, 1,1 ,1)", 1),
                new ExpressionTestCase("AVERAGE(BMK,1)", 62),
                new ExpressionTestCase("AVERAGE(-1,BMK,1)", 41),
                new ExpressionTestCase("AVERAGE (BMK/2)", 61.5),
                new ExpressionTestCase("AVERAGE(BMK)", 123),
                new ExpressionTestCase("AVERAGE (100%, 2*2, 0, 3)", 2),
                new ExpressionTestCase("AVERAGE(-3, 3)", 0),
                new ExpressionTestCase("AVERAGE(-3, 30, 3)", 10),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void CountFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("COUNT(1,1, 1,1 ,1)", 5),
                new ExpressionTestCase("COUNT(BMK,1)", 2),
                new ExpressionTestCase("COUNT(BMK)", 1),
                new ExpressionTestCase("COUNT(BMK, 2*2, 100%, 5>1, 5<0)", 5),
                new ExpressionTestCase("COUNT()", 0),
                new ExpressionTestCase("COUNT(-3, 3)", 2),
                new ExpressionTestCase("COUNT(-3, 30, 3)", 3),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        // TODO: Tests with fields.
        [TestMethod]
        public void DefinedFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("DEFINED(1)", 1),
                new ExpressionTestCase("DEFINED(BMK)", 1),
                new ExpressionTestCase("DEFINED(BMK/2)", 1),
                new ExpressionTestCase("DEFINED(BMKNOTEXISTING)", 0),
                new ExpressionTestCase("DEFINED(asd123)", 0),
                new ExpressionTestCase("DEFINED(1>1)", 1),
                new ExpressionTestCase("DEFINED(100%)", 1),
                new ExpressionTestCase("DEFINED(2+2+2)", 1),

            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void IntFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("INT(1)", 1),
                new ExpressionTestCase("INT(BMK)", 123),
                new ExpressionTestCase("INT(BMK/2)", 61),
                new ExpressionTestCase("INT(100% + 5.5)", 6),
                new ExpressionTestCase("INT(0.12345678901)", 0),
                new ExpressionTestCase("INT(-0.12345678901)", 0),
                new ExpressionTestCase("INT(-3.333)", -3),
                new ExpressionTestCase("INT(-305.01333)", -305),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void MaxFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("MAX(1)", 1),
                new ExpressionTestCase("MAX(BMK)", 123),
                new ExpressionTestCase("MAX(BMK/2, -300, 61.49)", 61.5),
                new ExpressionTestCase("MAX(100% + 5.5, 10, 1000, -9000)", 1000),
                new ExpressionTestCase("MAX(-0.12345678901, 0 , 1,2,3, -6)", 3),
                new ExpressionTestCase("MAX(-3.333,-5,-10,-10,-10)", -3.333),
                new ExpressionTestCase("MAX(1000, 2)", 1000),
                new ExpressionTestCase("MAX(1,2)", 2),

            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void MinFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("MIN(1)", 1),
                new ExpressionTestCase("MIN(BMK)", 123),
                new ExpressionTestCase("MIN(-BMK/2, 300, -61.49)", -61.5),
                new ExpressionTestCase("MIN(100% + 5.5, 10, 1000, -9000)", -9000),
                new ExpressionTestCase("MIN(-6.01,-0.12345678901, 0 , 1,2,3, -6)", -6.01),
                new ExpressionTestCase("MIN(-3.333,-5,-10,-10,-10)", -10),
                new ExpressionTestCase("MIN(1000, 2)", 2),
                new ExpressionTestCase("MIN(1,2)", 1),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void NotFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("NOT(1)", 0),
                new ExpressionTestCase("NOT(0)", 1),
                new ExpressionTestCase("NOT(-1)", 0),
                new ExpressionTestCase("NOT(1>0)", 0),
                new ExpressionTestCase("NOT(1< 0)", 1),
                new ExpressionTestCase("NOT(-3 = -3)", 0),
                new ExpressionTestCase("NOT(-BMK/2 > -3)", 1),
                new ExpressionTestCase("NOT(100% + 5.5 <> 6.5)", 1),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void OrFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("OR(1> 0, 0 >0)", 1),
                new ExpressionTestCase("OR(1 >0, 1 > 0)", 1),
                new ExpressionTestCase("OR(1< 0, 1 <>1)", 0),
                new ExpressionTestCase("OR(-3 = -3, -3>=-3)", 1),
                new ExpressionTestCase("OR(BMK/2 > BMK/2, BMK*2 < BMK*2 )", 0),
                new ExpressionTestCase("OR(BMK/2 >= BMK/2, BMK*2 <= BMK*2 )", 1),
                new ExpressionTestCase("OR(100% + 5.5 <> 6.5, 100% = 1)", 1),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void ProductFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("PRODUCT(1)", 1),
                new ExpressionTestCase("PRODUCT(1,0)", 0),
                new ExpressionTestCase("PRODUCT(BMK)", 123),
                new ExpressionTestCase("PRODUCT(BMK, 1)", 123),
                new ExpressionTestCase("PRODUCT(BMK/2, 2, 2)", 246),
                new ExpressionTestCase("PRODUCT(100% + 1, 10, 1000, -1)", -20000),
                new ExpressionTestCase("PRODUCT(-6.01,-0.12345678901, 0 , 1,2,3, -6)", 0),
                new ExpressionTestCase("PRODUCT(-10,-10,-10)", -1000),
                new ExpressionTestCase("PRODUCT(-10,-10,10)", 1000),
                new ExpressionTestCase("PRODUCT(1,2)", 2),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void RoundFunction_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("ROUND(3.2, 0)",3),
                new ExpressionTestCase("ROUND(3.2, 1.5)", 3.2),
                new ExpressionTestCase("ROUND(76.9, 0)", 76),
                new ExpressionTestCase("ROUND(3.14159, 3)", 3.141),
                new ExpressionTestCase("ROUND(-3.14159, 1)", -3.1),
                new ExpressionTestCase("ROUND(31415.92654, -2)", 31400),
                new ExpressionTestCase("ROUND(123.456, 3)", 123.456),
                new ExpressionTestCase("ROUND(123.456, 2)", 123.45),
                new ExpressionTestCase("ROUND(123.456, 0)", 123),
                new ExpressionTestCase("ROUND(123.456, -1)", 120),
                new ExpressionTestCase("ROUND(123.456, -1.3)", 120),
                new ExpressionTestCase("ROUND(123.456, -1.7)", 120),
                new ExpressionTestCase("ROUND(123.456, -2)", 100),
                new ExpressionTestCase("ROUND(654.321, 3)", 654.321),
                new ExpressionTestCase("ROUND(654.321, 2)", 654.32),
                new ExpressionTestCase("ROUND(654.321, 0)", 654),
                new ExpressionTestCase("ROUND(654.321, -1)", 650),
                new ExpressionTestCase("ROUND(654.321, -1.3)", 650),
                new ExpressionTestCase("ROUND(654.321, -1.7)", 650),
                new ExpressionTestCase("ROUND(654.321, -2)", 600),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void SignFunction_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("SIGN(0.5)", 1),
                new ExpressionTestCase("SIGN(0)", 0),
                new ExpressionTestCase("SIGN(-0.5)", -1),
                new ExpressionTestCase("SIGN(BMK)", 1),
                new ExpressionTestCase("SIGN(-BMK)", -1),
                new ExpressionTestCase("SIGN(BMK*0)", 0),
                new ExpressionTestCase("SIGN(0%)", 0),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void TrueLiteral_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("TRUE", 1),
                new ExpressionTestCase("TrUe", 1),
                new ExpressionTestCase("true", 1),
                new ExpressionTestCase(" TrUe", 1),
                new ExpressionTestCase("TrUe ", 1),
                new ExpressionTestCase(" TrUe ", 1),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void FalseLiteral_IsEvaluatedCorrectly()
        {
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("FALSE", 0),
                new ExpressionTestCase("FaLsE", 0),
                new ExpressionTestCase("false", 0),
                new ExpressionTestCase(" FaLsE", 0),
                new ExpressionTestCase("FaLsE ", 0),
                new ExpressionTestCase(" FaLsE ", 0),
            };

            this.AssertTestCases(tests, new RadFlowDocument());
        }

        [TestMethod]
        public void Bookmark_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            Run bmkRun = editor.InsertText("100/0");
            editor.InsertBookmark("BMK2", bmkRun, bmkRun);
            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("BMK", 123),
                new ExpressionTestCase(" BMK", 123),
                new ExpressionTestCase("BMK ", 123),
                new ExpressionTestCase("(BMK)", 123),
                new ExpressionTestCase("BMK2", 0),
                new ExpressionTestCase("BMK2 + 10", 10),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        [TestMethod]
        public void ComplexExpression_IsEvaluatedCorrectly()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);

            Run smallerBookmarkedRun = editor.InsertText("4 ");
            editor.InsertBookmark("BMKSMALL", smallerBookmarkedRun, smallerBookmarkedRun);

            List<ExpressionTestCase> tests = new List<ExpressionTestCase>() {
                new ExpressionTestCase("(-3+(BMK*2)+TRUE)", 244),
                new ExpressionTestCase("BMK*FALSE + 2*BMK", 246),
                new ExpressionTestCase("BMK*FALSE + 2*BMK + BMK^2 - 1537500%", 0),
                new ExpressionTestCase("((-1 + BMKSMALL^2) * 3 - 100%)/2", 22),
                new ExpressionTestCase("((-1 + BMKSMALL^2) * 3 - 100%)/2 > 22", 0),
                new ExpressionTestCase("((-1 + BMKSMALL^2) * 3 - 100%)/2 >= 22 * 100 %", 1),
                new ExpressionTestCase("BMKSMALL ^ 0.5 < 4", 1),
                new ExpressionTestCase("BMKSMALL ^ 0.5 = 2", 1),
                new ExpressionTestCase("(BMKSMALL > 1) + 1 = 2", 1),
                new ExpressionTestCase("BMK/100%", 123),
                new ExpressionTestCase("-BMK/100%", -123),
                new ExpressionTestCase("-BMK/-100%", 123),
                new ExpressionTestCase("-BMK/-100%^3", 123),
            };

            this.AssertTestCases(tests, editor.Document);
        }

        private void AssertTestCases(List<ExpressionTestCase> tests, RadFlowDocument doc)
        {
            StringBuilder sb = new StringBuilder();
            foreach (ExpressionTestCase testCase in tests)
            {
                try
                {
                    Expression result = ExpressionParser.Parse(testCase.Input, doc);
                    double actualResult = result.GetResult().Value;
                    if (testCase.ExpectedResult != actualResult)
                    {
                        sb.AppendFormat("Wrong result expression \"{0}\" expected: {1}  actual: {2}",
                            testCase.Input, testCase.ExpectedResult, actualResult);
                        sb.AppendLine();
                    }
                    else if (result is ErrorExpression)
                    {
                        sb.AppendFormat("Error Exception \"{0}\" was raised in \"{1}\" expression.",
                            ((ErrorExpression)result).ErrorMessage, testCase.Input);
                        sb.AppendLine();
                    }
                }
                catch (Exception ex)
                {
                    sb.AppendFormat("Exception in expression \"{0}\" Exception:{1}", testCase.Input, ex.Message);
                    sb.AppendLine();
                }
            }
            Assert.IsTrue(sb.Length == 0, sb.ToString());
        }

        private RadFlowDocument InsertMultipleBookmarks()
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

            Run bookmarkedRun = editor.InsertText(" 123 ");
            Bookmark bmk = editor.InsertBookmark("BMK", bookmarkedRun, bookmarkedRun);
            editor.InsertBookmark("BMKSAME", bookmarkedRun, bookmarkedRun);
            editor.MoveToInlineEnd(bmk.BookmarkRangeEnd);
            Run otherBookmarkedRun = editor.InsertText("5");
            editor.InsertBookmark("BMKOTHER", otherBookmarkedRun, otherBookmarkedRun);

            return editor.Document;
        }

        private class ExpressionTestCase
        {
            public string Input { get; set; }
            public double ExpectedResult { get; set; }
            public ExpressionTestCase(string input, double result)
            {
                this.Input = input;
                this.ExpectedResult = result;
            }
        }
    }
}