﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.InternalTests.Utilities;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields
{
    [TestClass]
    public class FieldInfoTests
    {
        [TestMethod]
        public void FieldInfoConstructor_WithUsedStartFieldCharacter_ThrowsException()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            FieldInfo field1 = new FieldInfo(doc);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(field1.Start, new FieldCharacter(doc, FieldCharacterType.Separator), new FieldCharacter(doc, FieldCharacterType.End));
            });
        }

        [TestMethod]
        public void FieldInfoConstructor_WithUsedSeparatorFieldCharacter_ThrowsException()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            FieldInfo field1 = new FieldInfo(doc);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(new FieldCharacter(doc, FieldCharacterType.Start), field1.Separator, new FieldCharacter(doc, FieldCharacterType.End));
            });
        }

        [TestMethod]
        public void FieldInfoConstructor_WithUsedEndFieldCharacter_ThrowsException()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            FieldInfo field1 = new FieldInfo(doc);

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(new FieldCharacter(doc, FieldCharacterType.Start), new FieldCharacter(doc, FieldCharacterType.Separator), field1.End);
            });
        }

        [TestMethod]
        public void FieldInfoConstructor_WithWrongCharacterTypes_ThrowsException()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(doc, FieldCharacterType.End),
                    new FieldCharacter(doc, FieldCharacterType.Separator),
                    new FieldCharacter(doc, FieldCharacterType.End));
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(doc, FieldCharacterType.Start),
                    new FieldCharacter(doc, FieldCharacterType.Start),
                    new FieldCharacter(doc, FieldCharacterType.End));
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(doc, FieldCharacterType.Start),
                    new FieldCharacter(doc, FieldCharacterType.Separator),
                    new FieldCharacter(doc, FieldCharacterType.Start));
            });
        }

        [TestMethod]
        public void FieldInfoConstructor_WithCharacterFromAnotherDocument_ThrowsException()
        {
            // Arrange
            RadFlowDocument doc = new RadFlowDocument();
            RadFlowDocument anotherDocument = new RadFlowDocument();

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(anotherDocument, FieldCharacterType.Start),
                    new FieldCharacter(doc, FieldCharacterType.Separator),
                    new FieldCharacter(doc, FieldCharacterType.End));
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(doc, FieldCharacterType.Start),
                    new FieldCharacter(anotherDocument, FieldCharacterType.Separator),
                    new FieldCharacter(doc, FieldCharacterType.End));
            });

            // Act & Assert
            TestHelper.AssertExceptionIsThrown<InvalidOperationException>(() =>
            {
                new FieldInfo(
                    new FieldCharacter(doc, FieldCharacterType.Start),
                    new FieldCharacter(doc, FieldCharacterType.Separator),
                    new FieldCharacter(anotherDocument, FieldCharacterType.End));
            });
        }
    }
}
