﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;
using Telerik.Windows.Documents.Flow.Model.Fields;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields.FieldParser
{
    [TestClass]
    public class FieldParserTests
    {
        [TestMethod]
        public void InsertField_WithHyperlinkCode_CreatesHyperlinkField()
        {
            this.AssertInsertFieldCreatesFieldOfType("HYPERLINK", typeof(Hyperlink));
            this.AssertInsertFieldCreatesFieldOfType("HYPERLINK \"telerik.com\" \\o \"Telerik Site\" ", typeof(Hyperlink));
        }

        [TestMethod]
        public void InsertField_WithExpressionCode_CreatesExpressionField()
        {
            this.AssertInsertFieldCreatesFieldOfType("=2", typeof(ExpressionField));
            this.AssertInsertFieldCreatesFieldOfType("=2+2", typeof(ExpressionField));
            this.AssertInsertFieldCreatesFieldOfType("=((-1 + X^2) * 3 - Y)/2", typeof(ExpressionField));
            this.AssertInsertFieldCreatesFieldOfType("=100/0", typeof(ExpressionField));
        }

        [TestMethod]
        public void InsertField_WithIFCode_CreatesIfField()
        {
            this.AssertInsertFieldCreatesFieldOfType("IF", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF 200 >= 100 \"true argument\" \"false argument\" ", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF order >= 100 \"true argument\" \"false argument\" ", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF (3-7) > 6 \"true argument\" \"false argument\" ", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF (3-7) > 6 \"true argument\" \"false argument\" \"third ignored argument\" ", typeof(IfField));

        }

        [TestMethod]
        public void InsertField_WithDateCode_CreatesDateField()
        {
            this.AssertInsertFieldCreatesFieldOfType("DATE", typeof(DateField));
            this.AssertInsertFieldCreatesFieldOfType("DATE \\@ \"M/d/yy\"", typeof(DateField));
            this.AssertInsertFieldCreatesFieldOfType("DATE \\@ \"dddd, MMMM dd, yyyy HH:mm:ss\"", typeof(DateField));
            this.AssertInsertFieldCreatesFieldOfType("DATE \\@ \"dddd, MMMM dd, yyyy HH:mm:ss\" \\h", typeof(DateField));
            this.AssertInsertFieldCreatesFieldOfType("DATE \\@ \"dddd, MMMM dd, yyyy HH:mm:ss\" \\s", typeof(DateField));
        }

        [TestMethod]
        public void InsertField_WithNotKnownCode_CreatesCodeField()
        {
            this.AssertInsertFieldCreatesFieldOfType("CUSTOMFIELDCODE ", typeof(CustomCodeField));
        }

        [TestMethod]
        public void InsertField_WithDateFormatting_LoadsDateFormatting()
        {
            Action<string, string> assertDateSwitchFromatting = (code, expextedDateFromat) =>
                {
                    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
                    FieldInfo field = editor.InsertField(code, "test");
                    Assert.AreEqual(expextedDateFromat, field.Field.DateTimeFormatting);
                };

            assertDateSwitchFromatting("DATE", null);
            assertDateSwitchFromatting("DATE \\@", string.Empty);
            assertDateSwitchFromatting("DATE \\@ M", "M");
            assertDateSwitchFromatting("DATE \\@ M/d/yy", "M/d/yy");
            assertDateSwitchFromatting("DATE \\@ \"M/d/yy\"", "M/d/yy");
            assertDateSwitchFromatting("DATE \\@ \"M / d / yy\"", "M / d / yy");
        }

        [TestMethod]
        public void InsertField_WithNumericForamtting_LoadsNumericForamtting()
        {
            Action<string, string> assertNumericSwitchFromatting = (code, expextedDateFromat) =>
            {
                RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
                FieldInfo field = editor.InsertField(code, "test");
                Assert.AreEqual(expextedDateFromat, field.Field.NumericFormatting);
            };

            assertNumericSwitchFromatting("PAGE", null);
            assertNumericSwitchFromatting("PAGE \\#", string.Empty);
            assertNumericSwitchFromatting("PAGE \\# #,##0", "#,##0");
            assertNumericSwitchFromatting("PAGE \\# \"#,##0\"", "#,##0");
            assertNumericSwitchFromatting("PAGE \\# \"# , ##0\"", "# , ##0");
            assertNumericSwitchFromatting("PAGE \\# \"$#,##0.00;-$#,##0.00;$0\"", "$#,##0.00;-$#,##0.00;$0");
        }

        [TestMethod]
        public void InsertField_WithGeneralForamtting_LoadsGeneralForamtting()
        {
            Action<string, string> assertGeneralSwitchFromatting = (code, expextedDateFromat) =>
            {
                RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
                FieldInfo field = editor.InsertField(code, "test");
                Assert.AreEqual(expextedDateFromat, field.Field.GeneralFormatting);
            };

            assertGeneralSwitchFromatting("PAGE", null);
            assertGeneralSwitchFromatting("PAGE \\*", string.Empty);
            assertGeneralSwitchFromatting("PAGE \\* alphabetic", "alphabetic");
            assertGeneralSwitchFromatting("PAGE \\* ALPHABETIC", "ALPHABETIC");
            assertGeneralSwitchFromatting("PAGE \\* \"Roman\"", "Roman");
            assertGeneralSwitchFromatting("PAGE \\* \"roman\"", "roman");
        }

        [TestMethod]
        public void InsertField_WithMultipleGeneralForamtting_LoadsTheLastSetGeneralForamtting()
        {
            Action<string, string> assertGeneralSwitchFromatting = (code, expextedDateFromat) =>
            {
                RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
                FieldInfo field = editor.InsertField(code, "test");
                Assert.AreEqual(expextedDateFromat, field.Field.GeneralFormatting);
            };

            assertGeneralSwitchFromatting("PAGE \\* alphabetic \\* Upper", "Upper");
            assertGeneralSwitchFromatting("PAGE \\* roman \\* ALPHABETIC", "ALPHABETIC");
            assertGeneralSwitchFromatting("PAGE \\* \"Roman\" \\* \"Upper\" \\* \"Lower\"", "Lower");
            assertGeneralSwitchFromatting("PAGE \\* \"Roman\" \\* \"Upper\"", "Upper");
            assertGeneralSwitchFromatting("PAGE \\* ALPHABETIC \\* Upper \\* alphabetic", "alphabetic");
            assertGeneralSwitchFromatting("PAGE \\* ALPHABETIC \\* Upper \\* alphabetic \\* roman", "roman");
            assertGeneralSwitchFromatting("PAGE \\* ALPHABETIC \\* Upper \\* alphabetic \\* roman \\* ROMAN", "ROMAN");
            assertGeneralSwitchFromatting("PAGE \\* ALPHABETIC \\* Upper \\* alphabetic \\* roman \\* ROMAN \\* Lower", "Lower");
        }

        // Error fields.
        [TestMethod]
        public void InsertField_WithParsingException_ReturnsErrorParseField()
        {
            this.AssertInsertFieldCreatesFieldOfType("\\IF 200 >= 100 \"true argument\" \"false argument\"", typeof(ParseErrorField));
            this.AssertInsertFieldCreatesFieldOfType("\\IF 200 >>= 100 \"true argument\" \"false argument\"", typeof(ParseErrorField));
            this.AssertInsertFieldCreatesFieldOfType("\\*IF 200 >>= 100 \"true argument\" \"false argument\"", typeof(ParseErrorField));
        }

        [TestMethod]
        public void InsertField_WithWrongArguments_ReturnsRightTypeOfFieldField_WithError()
        {
            this.AssertInsertFieldCreatesFieldOfType("IF 200 >>= 100 \"true argument\" \"false argument\"", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF 200 != 100 \"true argument\" \"false argument\"", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF 200 <>> 100 \"true argument\" \"false argument\"", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF 200 == 100 \"true argument\" \"false argument\"", typeof(IfField));
            this.AssertInsertFieldCreatesFieldOfType("IF 20+11 = 30 \"true argument\" \"false argument", typeof(IfField));
        }

        private void AssertInsertFieldCreatesFieldOfType(string input, Type expectedFieldType)
        {
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());
            FieldInfo field = editor.InsertField(input, "test");
            Assert.IsInstanceOfType(field.Field, expectedFieldType);
        }
    }
}
