﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using Telerik.Windows.Documents.Core.Sevices;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields.Update
{
    [TestClass]
    public class DateFieldTests : DateTimeFieldsTestsBase
    {
        [TestMethod]
        public void UpdateDateField_NoFormatting_UpdatesCorrectly()
        {
            string expected = DateTimeService.GetDateTime().ToShortDateString();

            List<TestCase> testCases = new List<TestCase>();
            testCases.Add(new TestCase("DATE", expected));
            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateDateField_WithDateFormatting_UpdatesCorrectly()
        {
            List<TestCase> testCases = new List<TestCase>();

            testCases.Add(new TestCase("DATE \\@ \"M/d/yyyy\"", "1/3/2006"));
            testCases.Add(new TestCase("DATE \\@ \"dddd, MMMM dd, yyyy\"", "Tuesday, January 03, 2006"));
            testCases.Add(new TestCase("DATE \\@ \"MMMM d, yyyy\"", "January 3, 2006"));
            testCases.Add(new TestCase("DATE \\@ \"M/d/yy\"", "1/3/06"));
            testCases.Add(new TestCase("DATE \\@ \"yyyy-MM-dd\"", "2006-01-03"));
            testCases.Add(new TestCase("DATE \\@ \"d-MMM-yy\"", "3-Jan-06"));
            testCases.Add(new TestCase("DATE \\@ \"M.d.yyyy\"", "1.3.2006"));
            testCases.Add(new TestCase("DATE \\@ \"MMM. d, yy\"", "Jan. 3, 06"));
            testCases.Add(new TestCase("DATE \\@ \"d MMMM yyyy\"", "3 January 2006"));
            testCases.Add(new TestCase("DATE \\@ \"MMMM yy\"", "January 06"));
            testCases.Add(new TestCase("DATE \\@ \"MMM-yy\"", "Jan-06"));

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateDateField_WithTimeFormatting_UpdatesCorrectly()
        {
            List<TestCase> testCases = new List<TestCase>();

            testCases.Add(new TestCase("DATE \\@ \"h:mm am/pm\"", "5:28 PM"));
            testCases.Add(new TestCase("DATE \\@ \"h:mm:ss am/pm\"", "5:28:34 PM"));

            testCases.Add(new TestCase("DATE \\@ \"HH:mm\"", "17:28"));
            testCases.Add(new TestCase("DATE \\@ \"'Today is 'HH:mm:ss\"", "Today is 17:28:34"));

            this.AssertTestCases(testCases);
        }

        [TestMethod]
        public void UpdateDateField_WithDateAndTimeFormatting_UpdatesCorrectly()
        {
            List<TestCase> testCases = new List<TestCase>();

            testCases.Add(new TestCase("DATE \\@ \"M/d/yyyy h:mm am/pm\"", "1/3/2006 5:28 PM"));
            testCases.Add(new TestCase("DATE \\@ \"M/d/yyyy h:mm:ss am/pm\"", "1/3/2006 5:28:34 PM"));

            this.AssertTestCases(testCases);
        }
    }
}
