using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Core.Sevices;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Editing;

namespace Telerik.Windows.Documents.Flow.InternalTests.Model.Fields.Update
{
    public class DateTimeFieldsTestsBase
    { 
        private IDateTimeProvider previousProvider;

        [TestInitialize]
        public void Init()
        {
            this.previousProvider = DateTimeService.DateTimeProvider;
            DateTimeService.DateTimeProvider = new TestDateTimeProvider();
        }

        [TestCleanup]
        public void Clean()
        {
            DateTimeService.DateTimeProvider = this.previousProvider;
            this.previousProvider = null;
        }

        protected void AssertTestCases(List<TestCase> tests)
        {
            StringBuilder sb = new StringBuilder();
            foreach (var testCase in tests)
            {
                try
                {
                    RadFlowDocumentEditor editor = new RadFlowDocumentEditor(new RadFlowDocument());

                    var field = editor.InsertField(testCase.FieldCode, "test");
                    field.UpdateField();

                    string actualResult = field.GetResult();
                    if (testCase.ExpectedResult != actualResult)
                    {
                        sb.AppendFormat("Wrong result expression \"{0}\" expected: {1}  actual: {2}",
                            testCase.FieldCode, testCase.ExpectedResult, actualResult);
                        sb.AppendLine();
                    }
                }
                catch (Exception ex)
                {
                    sb.AppendFormat("Exception in expression \"{0}\" Exception:{1}", testCase.FieldCode, ex.Message);
                    sb.AppendLine();
                }
            }
            Assert.IsTrue(sb.Length == 0, sb.ToString());
        }

        protected class TestDateTimeProvider : IDateTimeProvider
        {
            public DateTime GetDateTime()
            {
                return new DateTime(2006, 1, 3, 17, 28, 34);
            }
        }

        protected class TestCase
        {
            public string FieldCode { get; set; }

            public string ExpectedResult { get; set; }

            public TestCase(string input, string result)
            {
                this.FieldCode = input;
                this.ExpectedResult = result;
            }
        }
    }
}