﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
#if WPF || SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model.Editing;

namespace Telerik.Windows.Documents.Flow.FormatProviders.Pdf.Tests.Model.Styles
{
    [TestClass]
    public class TabStopsTests : TestsBase
    {
        [TestMethod]
        [TestCategory(VISUALTESTCATEGORY)]
        public void TabStops_WhenAreDecimal_Export()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Paragraph p;
            p = editor.InsertText("test\ttest5.55545454545454545").Paragraph;
            p.TabStops = p.TabStops.Insert(new TabStop(300, TabStopType.Decimal, TabStopLeader.Dot));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            editor.MoveToInlineEnd(p.Inlines.LastOrDefault());
            editor.InsertParagraph();
            p = editor.InsertText("test\t5.1").Paragraph;
            p.TabStops = p.TabStops.Insert(new TabStop(300, TabStopType.Decimal, TabStopLeader.Dot));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(document, new DocxFormatProvider());

            // Assert
            this.AssertDocumentVisual(imported);
        }

        [TestMethod]
        [TestCategory(VISUALTESTCATEGORY)]
        public void TabStops_WhenAreMultipleAndFromEveryType_Export()
        {
            // Arrange
            RadFlowDocument document = new RadFlowDocument();
            RadFlowDocumentEditor editor = new RadFlowDocumentEditor(document);

            Paragraph p;
            Paragraph p2;
            p = editor.InsertText("test\ttest\tmeest\tteeestss\tagain").Paragraph;
            p.TabStops = p.TabStops.Insert(new TabStop(300, TabStopType.Left, TabStopLeader.Dot));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            p = editor.InsertText("test\ttest").Paragraph;
            p.TabStops = p.TabStops.Insert(new TabStop(900, TabStopType.Left, TabStopLeader.Dot));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            editor.MoveToInlineEnd(p.Inlines.LastOrDefault());
            editor.InsertParagraph();
            p = editor.InsertText("test\ttest").Paragraph;
            p.TabStops = p.TabStops.Insert(new TabStop(900, TabStopType.Left, TabStopLeader.Dot));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            editor.MoveToInlineEnd(p.Inlines.LastOrDefault());
            editor.InsertParagraph();
            p = editor.InsertText("test\ttest").Paragraph;
            editor.InsertText(" very very long \tteextClick Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated!");
            p.TabStops = p.TabStops.Insert(new TabStop(200, TabStopType.Right, TabStopLeader.Dot));
            p.TabStops = p.TabStops.Insert(new TabStop(500, TabStopType.Right, TabStopLeader.Hyphen));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            editor.MoveToInlineEnd(p.Inlines.LastOrDefault());
            editor.InsertParagraph();
            p = editor.InsertText("test\ttest").Paragraph;
            editor.InsertText(" very very long \tteext Click Insert and then choose the elements you want from the different galleries. Themes and styles also help keep your document coordinated!");
            p.TabStops = p.TabStops.Insert(new TabStop(200, TabStopType.Left, TabStopLeader.None));
            p.TabStops = p.TabStops.Insert(new TabStop(500, TabStopType.Left, TabStopLeader.Underscore));
            (p.Inlines.First() as Run).ForegroundColor = ThemableColor.FromArgb(255, 255, 0, 0);

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("bar hyphen\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Bar, TabStopLeader.Hyphen));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("left hyphen\ttab \ta lot of text \tnext is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Left, TabStopLeader.Hyphen));
            p2.TabStops = p2.TabStops.Insert(new TabStop(400, TabStopType.Left, TabStopLeader.Dot));
            p2.TabStops = p2.TabStops.Insert(new TabStop(900, TabStopType.Left, TabStopLeader.None));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("right middle dot\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Right, TabStopLeader.MiddleDot));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("right middle dot\ttab \ta lot of text \tnext is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Left, TabStopLeader.MiddleDot));
            p2.TabStops = p2.TabStops.Insert(new TabStop(400, TabStopType.Left, TabStopLeader.MiddleDot));
            p2.TabStops = p2.TabStops.Insert(new TabStop(600, TabStopType.Left, TabStopLeader.MiddleDot));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("decimal underscore\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Decimal, TabStopLeader.Underscore));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("left none\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Left, TabStopLeader.None));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("clear\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Clear, TabStopLeader.Underscore));

            p2 = editor.InsertParagraph();
            editor.MoveToInlineEnd(p2.Inlines.AddRun("center middle dot\ttab a lot of text next is there which shall be splitted"));
            p2.TabStops = p2.TabStops.Insert(new TabStop(200, TabStopType.Center, TabStopLeader.MiddleDot));

            // Act
            RadFlowDocument imported = this.ExportImportDocument(document, new DocxFormatProvider());

            // Assert
            this.AssertDocumentVisual(imported);
        }
    }
}
