﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Commands
{
    /// <summary>
    /// Represents a base for the event arguments classes for the command execution events.
    /// </summary>
    public abstract class CommandExecuteEventArgsBase : EventArgs
    {
        #region Properties

        internal IWorkbookCommand Command { get; private set; }
        internal WorkbookCommandContextBase CommandContext { get; private set; }

        /// <summary>
        /// Gets a value indicating whether the event has occurred while in undo.
        /// </summary>
        /// <value>The value indicating whether the event has occurred while in undo.</value>
        public bool IsInUndo { get; private set; }

        /// <summary>
        /// Gets the name of the command.
        /// </summary>
        /// <value>The name of the command.</value>
        public string CommandName
        {
            get
            {
                return this.Command.Name;
            }
        }
		
		#endregion


		#region Constructors

        internal CommandExecuteEventArgsBase(IWorkbookCommand command, WorkbookCommandContextBase commandContext, bool isInUndo)
		{
			Guard.ThrowExceptionIfNull(command, "command");
			Guard.ThrowExceptionIfNull(commandContext, "commandContext");

			this.Command = command;
			this.CommandContext = commandContext;
			this.IsInUndo = isInUndo;
		}

		#endregion
    }
}