﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Commands
{
	internal abstract class WorkbookCommandBase<T> : IWorkbookCommand
		where T : WorkbookCommandContextBase
    {
        #region Fields

        private string name;

        #endregion


        #region Properties

        public string Name
        {
            get
            {
                if (string.IsNullOrEmpty(this.name))
                {
                    this.name = this.GetType().Name;
                }

                return this.name;
            }
        }

        #endregion


        #region Constructors

        internal WorkbookCommandBase()
		{
		}

		#endregion


		#region Methods

		public bool AffectsLayout(WorkbookCommandContextBase context)
		{
			Guard.ThrowExceptionIfNull(context, "context");
			return this.AffectsLayoutOverride((T)context);
		}

		protected abstract bool AffectsLayoutOverride(T context);

		public bool CanExecute(WorkbookCommandContextBase context)
		{
			Guard.ThrowExceptionIfNull(context, "context");
			return this.CanExecuteOverride((T)context);
		}

		protected virtual bool CanExecuteOverride(T context)
		{
			return true;
		}

		public bool Execute(WorkbookCommandContextBase context)
		{
			Guard.ThrowExceptionIfNull(context, "context");

			if (!this.CanExecute(context))
			{
				return false;
			}

			CommandExecutingEventArgs e = new CommandExecutingEventArgs(this, context);
			context.Workbook.OnCommandExecuting(e);
			if (e.Canceled)
			{
				return false;
			}
			context.OnExecuting();
			this.OnExecuting(context);

			try
			{
				this.ExecuteOverride((T)context);
			}
			catch (Exception ex)
			{
				CommandErrorEventArgs errorArgs = new CommandErrorEventArgs(ex, this, context);
                context.Workbook.OnCommandError(errorArgs);
				return false;
			}

			context.OnExecuted();
            context.Workbook.OnCommandExecuted(new CommandExecutedEventArgs(this, context));

			return true;
		}

		protected virtual void OnExecuting(WorkbookCommandContextBase context)
		{
		}

		protected abstract void ExecuteOverride(T context);

		#endregion
	}
}