﻿using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Commands
{
	internal abstract class WorkbookCommandContextBase
	{
		#region Fields

		private readonly Workbook workbook;
		private bool isFirstExecution;
        private Sheet activeSheet;

		#endregion


		#region Properties

        public Workbook Workbook
        {
            get
            {
                return this.workbook;
            }
        }

        public Sheet ActiveSheet
        {
            get
            {
                return this.activeSheet;
            }
        }

		public bool IsFirstExecution
		{
			get
			{
				return this.isFirstExecution;
			}
		}

		#endregion


		#region Constructors

		public WorkbookCommandContextBase(Workbook workbook)
		{
            Guard.ThrowExceptionIfNull(workbook, "workbook");

            this.workbook = workbook;
			this.isFirstExecution = true;
		}

		#endregion


		#region Methods

		internal void OnExecuting()
		{
            if (this.isFirstExecution)
            {
                this.activeSheet = this.Workbook.ActiveSheet;
            }
		}

		internal void OnExecuted()
		{
			this.isFirstExecution = false;
		}

		#endregion


        #region Methods

        internal virtual void InvalidateLayout()
        {
            this.Workbook.InvalidateLayout();
        }

        #endregion
    }
}