﻿using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Windows.Documents.Spreadsheet.Commands.WorkbookCommands
{
    internal class ReapplyCellStyleCommand : UndoableWorkbookCommandBase<ReapplyCellStyleCommandContext>
    {
        #region Methods

        protected override bool AffectsLayoutOverride(ReapplyCellStyleCommandContext context)
        {
            return true;
        }

        protected override bool CanExecuteOverride(ReapplyCellStyleCommandContext context)
        {
            return true;
        }

        protected override void PreserveStateBeforeExecute(ReapplyCellStyleCommandContext context)
        {
            // Nothing to preserve.
        }

        protected override void ExecuteOverride(ReapplyCellStyleCommandContext context)
        {
            this.ResetStyle(context.Workbook, context.NewValue);
        }

        protected override void UndoOverride(ReapplyCellStyleCommandContext context)
        {
            this.ResetStyle(context.Workbook, context.OldValue);
        }

        private void ResetStyle(Workbook workbook, string styleName)
        {
            using (new UpdateScope(workbook.History.BeginUndoGroup, workbook.History.EndUndoGroup))
            {
                foreach (var sheet in workbook.Worksheets)
                {
                    var styleNamePropertyCollection = sheet.Cells.PropertyBag.GetPropertyValueCollection(CellPropertyDefinitions.StyleNameProperty);
                    var ranges = styleNamePropertyCollection.Where(x => x.Value == styleName);

                    foreach (var range in ranges)
                    {
                        CellRange cellRange = CellsPropertyBag.ConvertLongCellRangeToCellRange(range.Start, range.End);
                        sheet.Cells[cellRange].SetStyleNameAndExpandToFitNumberValuesWidth(range.Value);
                    }
                }
            }
        }

        #endregion
    }
}