﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands
{
    internal class SetColumnBordersCommandContext : SetBordersCommandContextBase
    {
        public SetColumnBordersCommandContext(Worksheet worksheet, CellRange cellRange, CellBorders newValue)
            : base(worksheet, cellRange, newValue)
        {
        }

        public override ICompressedList<CellBorder> GetBorderProperty(IPropertyDefinition<CellBorder> borderProperty, CellRange range)
        {
            return this.Worksheet.Columns.PropertyBag.GetPropertyValue(borderProperty, range.FromIndex.ColumnIndex, range.ToIndex.ColumnIndex);
        }

        public override void SetBorderProperty(IPropertyDefinition<CellBorder> borderProperty, CellRange range, ICompressedList<CellBorder> values)
        {
            ICompressedList<CellBorder> newValues = values.GetValue(range.FromIndex.ColumnIndex, range.ToIndex.ColumnIndex);
            IProperty<CellBorder> property;
            this.Worksheet.Columns.TryGetProperyFromPropertyDefinition(borderProperty, out property);
            foreach (Range<long, CellBorder> valueRange in newValues.GetNonDefaultRanges())
            {
                this.Worksheet.Columns[(int)valueRange.Start, (int)valueRange.End].SetPropertyValue(property, valueRange.Value);
            }
        }

        protected override void SetValuesInRange(CellRange range, ICompressedList<CellBorder> collection, CellBorder value)
        {
            collection.SetValue(range.FromIndex.ColumnIndex, range.ToIndex.ColumnIndex, value);
        }

        protected override void ClearValuesInRange(CellRange range, ICompressedList collection)
        {
            collection.ClearValue(range.FromIndex.ColumnIndex, range.ToIndex.ColumnIndex);
        }
    }
}