﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands
{
    internal class SetShapeRotationAngleCommandContext : WorksheetCommandContextBase
    {
        private static readonly int ExcelMinRotationAngle = -35791;
        private static readonly int ExcelMaxRotationAngle = 35791;

        private readonly FloatingShapeBase shape;
        private readonly double angle;
        private double oldAngle;
        private readonly bool adjustCellindex;

        public FloatingShapeBase FloatingShape
        {
            get
            {
                return this.shape;
            }
        }

        public double Angle
        {
            get
            {
                return this.angle;
            }
        }

        public double OldAngle
        {
            get
            {
                return this.oldAngle;
            }
            set
            {
                Guard.ThrowExceptionIfOutOfRange<double>(ExcelMinRotationAngle, ExcelMaxRotationAngle, value, "value");
                this.oldAngle = value;
            }
        }

        public bool AdjustCellindex
        {
            get
            {
                return this.adjustCellindex;
            }
        }

        public SetShapeRotationAngleCommandContext(Worksheet worksheet, FloatingShapeBase shape, double angle, bool adjustCellindex) : base(worksheet)
        {
            Guard.ThrowExceptionIfNull(shape, "shape");
            Guard.ThrowExceptionIfOutOfRange<double>(ExcelMinRotationAngle, ExcelMaxRotationAngle, angle, "angle");

            this.shape = shape;
            this.angle = angle;
            this.adjustCellindex = adjustCellindex;
        }
    }
}
