﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands
{
    internal class SetSortCommand : UndoableWorksheetCommandBase<SetSortCommandContext>
    {
        protected override bool AffectsLayoutOverride(SetSortCommandContext context)
        {
            return true;
        }

        protected override void PreserveStateBeforeExecute(SetSortCommandContext context)
        {
            int rowCount = context.SortRange.RowCount;

            int[] values = new int[rowCount];
            int[] indexes = new int[rowCount];

            for (int i = 0; i < rowCount; i++)
            {
                values[i] = context.SortedIndexes[i];
                indexes[i] = i;
            }

            Array.Sort(values, indexes);

            int[] sortedIndexes = new int[rowCount];
            for (int i = 0; i < rowCount; i++)
            {
                sortedIndexes[i] = indexes[i];
            }

            context.OldSortedIndexes = sortedIndexes;
        }

        protected override void UndoOverride(SetSortCommandContext context)
        {
            Worksheet worksheet = context.Worksheet;

            worksheet.SortState.RemoveFirst();
            worksheet.Cells.PropertyBag.Sort(worksheet, context.SortRange, context.OldSortedIndexes);
            worksheet.SortState.SetSortRangeInternal(context.SortRange);
        }

        protected override void ExecuteOverride(SetSortCommandContext context)
        {
            Worksheet worksheet = context.Worksheet;

            worksheet.Cells.PropertyBag.Sort(worksheet, context.SortRange, context.SortedIndexes);
            worksheet.SortState.AddFirstInternal(context.SortCondition);
        }
    }
}