﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands
{
    internal abstract class UpdatePropertyInRangeCommandContext<T> : WorksheetCommandContextBase
	{
		#region Fields

		private readonly IPropertyDefinition<T> property;
        private ICompressedList<T> oldValues;
		private readonly Func<T, T> newValueTransform;

		#endregion


		#region Properties

		public IPropertyDefinition<T> Property
		{
			get
			{
				return this.property;
			}
		}

        public ICompressedList<T> OldValues
		{
			get
			{
				return this.oldValues;
			}
			internal set
			{
				this.oldValues = value;
			}
		}

		public Func<T, T> NewValueTransform
		{
			get
			{
				return this.newValueTransform;
			}
		}

		#endregion


		#region Constructors

        public UpdatePropertyInRangeCommandContext(Worksheet worksheet, IPropertyDefinition<T> property, Func<T, T> newValueTransform)
			: base(worksheet)
		{
			Guard.ThrowExceptionIfNull(property, "property");
            Guard.ThrowExceptionIfNull(newValueTransform, "newValueTransform");

			this.property = property;
            this.newValueTransform = newValueTransform;
		}

		#endregion
	}
}