﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Copying
{
    internal class CopyContext
    {
        private readonly Dictionary<string, string> originalToRenamedStyleNames;
        private readonly Worksheet targetWorksheet;
        private readonly Worksheet sourceWorksheet;

        /// <summary>
        /// Initializes a new instance of the <see cref="CopyContext" /> class.
        /// </summary>
        /// <param name="target">The worksheet to copy to.</param>
        /// <param name="source">The worksheet to copy from.</param>
        public CopyContext(Worksheet target, Worksheet source)
        {
            Guard.ThrowExceptionIfNull(target, "target");
            Guard.ThrowExceptionIfNull(source, "source");

            this.targetWorksheet = target;
            this.sourceWorksheet = source;
            this.originalToRenamedStyleNames = new Dictionary<string, string>();
        }

        /// <summary>
        /// Gets the target worksheet - the worksheet to copy to.
        /// </summary>
        /// <value>The target worksheet.</value>
        public Worksheet TargetWorksheet
        {
            get
            {
                return this.targetWorksheet;
            }
        }

        /// <summary>
        /// Gets the source worksheet - the worksheet to copy from.
        /// </summary>
        /// <value>The source worksheet.</value>
        public Worksheet SourceWorksheet
        {
            get
            {
                return this.sourceWorksheet;
            }
        }

        public Dictionary<string, string> OriginalToRenamedStyleNames
        {
            get
            {
                return this.originalToRenamedStyleNames;
            }
        }

        public bool SpreadsheetNameExistsInTargetWorkbook { get; set; }

        public FloatingShapeBase SourceFloatingShape { get; set; }
    }
}