﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Core.DataStructures
{
    internal class Range<TIndex, TValue> : IComparable<Range<TIndex, TValue>>, IEquatable<Range<TIndex, TValue>>
        where TIndex : IComparable<TIndex>
    {
        #region Fields

        private TIndex start;
        private TIndex end;
        private readonly bool isDefault;

        #endregion


        #region Properties

        public TValue Value { get; private set; }

        public TIndex Start
        {
            get
            {
                return this.start;
            }
            internal set
            {
                this.start = value;
            }
        }

        public TIndex End
        {
            get
            {
                return this.end;
            }
            internal set
            {
                this.end = value;
            }
        }

        public bool IsDefault
        {
            get
            {
                return this.isDefault;
            }
        }

        #endregion


        #region Constructors

        public Range(TIndex start, TIndex end, bool isDefault, TValue value = default(TValue))
        {
            Guard.ThrowExceptionIfLessThan(start, end, "end");

            this.Start = start;
            this.End = end;
            this.Value = value;
            this.isDefault = isDefault;
        }

        #endregion


        #region Methods

        public bool Contains(TIndex position)
        {
            return position.CompareTo(this.Start) >= 0 && position.CompareTo(this.End) <= 0;
        }

        public bool Contains(Range<TIndex, TValue> range)
        {
            Guard.ThrowExceptionIfNull(range, "range");
            return ((range.Start.CompareTo(this.Start) >= 0) && (range.End.CompareTo(this.End) <= 0));
        }

        public bool IntersectsWith(Range<TIndex, TValue> range)
        {
            Guard.ThrowExceptionIfNull(range, "range");
            return range.Start.CompareTo(this.End) <= 0 && range.End.CompareTo(this.Start) >= 0;
        }

        public bool Equals(Range<TIndex, TValue> other)
        {
            if (other == null)
            {
                return false;
            }

            return ((other.Start.Equals(this.Start)) && (other.End.Equals(this.End)));
        }

        public override bool Equals(object obj)
        {
            Range<TIndex, TValue> other = obj as Range<TIndex, TValue>;
            return this.Equals(other);
        }

        public override int GetHashCode()
        {
            int startHashCode = this.Start.GetHashCode();
            int endHashCode = this.End.GetHashCode();

            return TelerikHelper.CombineHashCodes(startHashCode, endHashCode);
        }

        public override string ToString()
        {
            return string.Format("[{0}..{1}]", this.Start, this.End);
        }

        int IComparable<Range<TIndex, TValue>>.CompareTo(Range<TIndex, TValue> other)
        {
            //Guard.ThrowExceptionIfNull(other, "other");
            return this.start.CompareTo(other.start);
        }

        internal Range<TIndex, TValue> CreateCopy()
        {
            return new Range<TIndex, TValue>(this.Start, this.End, this.IsDefault, this.Value);
        }

        #endregion
    }
}