﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Core.DataStructures
{
	internal class RangeTreeCollection<TIndex, TValue> : AATree<Range<TIndex, TValue>>
		where TIndex : IComparable<TIndex>
	{
		#region Methods

		private void GetIntersectingRanges(AATreeNode node, Range<TIndex, TValue> range, ref List<Range<TIndex, TValue>> intersectingRanges)
		{
            if (node == sentinel)
			{
				return;
			}

			if (node.value.Start.CompareTo(range.Start) > 0)
			{
				this.GetIntersectingRanges(node.left, range, ref intersectingRanges);
			}

			if (node.value.IntersectsWith(range))
			{
				intersectingRanges.Add(node.value);
			}

			if (node.value.End.CompareTo(range.End) < 0)
			{
				this.GetIntersectingRanges(node.right, range, ref intersectingRanges);
			}
		}

		internal List<Range<TIndex, TValue>> GetIntersectingRanges(Range<TIndex, TValue> range)
		{
			List<Range<TIndex, TValue>> intersectingRanges = new List<Range<TIndex, TValue>>();
			this.GetIntersectingRanges(this.root, range, ref intersectingRanges);

			return intersectingRanges;
		}

		private Range<TIndex, TValue> GetContainingRange(AATreeNode node, TIndex position)
		{
            while (node != sentinel)
            {
                if (position.CompareTo(node.value.Start) < 0)
                {
                    node = node.left;
                }
                else if (position.CompareTo(node.value.End) > 0)
                {
                    node = node.right;
                }
                else
                {
                    return node.value;
                }
            }

            return null;
		}

		internal Range<TIndex, TValue> GetContainingRange(TIndex position)
		{
			return this.GetContainingRange(this.root, position);
		}

		#endregion
	}
}