﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Core
{
    /// <summary>
    /// Represents named objects item replace event arguments.
    /// </summary>
    public class NamedObjectsItemReplaceEventArgs : EventArgs
    {
        #region Properties

        /// <summary>
        /// Gets the name of the item.
        /// </summary>
        /// <value>The name of the item.</value>
        public string ItemName { get; private set; }

        /// <summary>
        /// Gets the new name of the item.
        /// </summary>
        /// <value>The new name of the item.</value>
        public string NewItemName { get; private set; }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="NamedObjectsItemReplaceEventArgs" /> class.
        /// </summary>
        /// <param name="itemName">Name of the item.</param>
        /// <param name="newItemName">New name of the item.</param>
        public NamedObjectsItemReplaceEventArgs(string itemName, string newItemName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(itemName, "itemName");
            Guard.ThrowExceptionIfNullOrEmpty(newItemName, "newItemName");

            this.ItemName = itemName;
            this.NewItemName = newItemName;
        }

        #endregion
    }
}