﻿using System.ComponentModel;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Core
{
    /// <summary>
    /// Provides the base class from which the classes that represent INotifyPropertyChanged objects are derived.
    /// </summary>
    public abstract class NotifyPropertyChangedBase : INotifyPropertyChanged
    {
		#region Events

        /// <summary>
        /// Called when when property is changed.
        /// </summary>
        /// <param name="propertyName">Name of the property.</param>
        protected void OnPropertyChanged(string propertyName)
        {
			Guard.ThrowExceptionIfNullOrEmpty(propertyName, "propertyName");
			this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
		}

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises the <see cref="E:PropertyChanged" /> event.
        /// </summary>
        /// <param name="args">The <see cref="System.ComponentModel.PropertyChangedEventArgs" /> instance containing the event data.</param>
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
        {
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, args);
			}
		}

		#endregion
	}
}