﻿using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace Telerik.Windows.Documents.Spreadsheet.Core
{
	internal class ObjectPool
	{
		#region Fields

		private static readonly object lockObject = new object();
        private readonly Dictionary<Type, Stack<object>> typeToPool;
        private readonly Func<Type, object> createInstanceFunc;

		#endregion


		#region Constructors

        public ObjectPool()
        {
			this.typeToPool = new Dictionary<Type, Stack<object>>();
            this.createInstanceFunc = (type) =>
                {
                    return Activator.CreateInstance(type);
                };
		}

        public ObjectPool(Func<Type, object> createInstanceFunc)
        {
            this.typeToPool = new Dictionary<Type, Stack<object>>();
            this.createInstanceFunc = createInstanceFunc;
        }

		#endregion


		#region Methods

		public T CreateObject<T>()
		{
			lock (lockObject)
			{
				Type uiElementType = typeof(T);
				Stack<object> availableElements = null;
				if (this.typeToPool.TryGetValue(uiElementType, out availableElements))
				{
					if (availableElements.Count > 0)
					{
						return (T)availableElements.Pop();
					}
				}

                return (T)this.createInstanceFunc(uiElementType);
			}
		}

		public void ReleaseObject(object obj)
		{
			lock (lockObject)
			{
				Stack<object> availableElements;
				Type uiElementType = obj.GetType();
				if (!typeToPool.TryGetValue(uiElementType, out availableElements))
				{
					availableElements = new Stack<object>();
					this.typeToPool[uiElementType] = availableElements;
				}

                // TODO: Remove this check!
                bool isInPool = false;
#if DEBUG
				foreach (var x in availableElements)
				{
                    if (x == obj)
                    {
                        isInPool = true;
                        Debug.Assert(false);
                        break;
                    }
				}
#endif

				// TODO: if count < MAX_ELEMENTS?
                if (!isInPool)
                {
                    availableElements.Push(obj);
                }
			}
		}

		#endregion
	}
}