﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Core
{
	internal class RadWeakEventListener<TInstance, TSource, TEventArgs> where TInstance : class
	{
		#region Fields

		private readonly WeakReference weakInstance;
		private readonly WeakReference weakSource;
		private Action<TInstance, object, TEventArgs> onEventAction;
		private Action<RadWeakEventListener<TInstance, TSource, TEventArgs>, TSource> onDetachAction;

		#endregion


		#region Properties

		public Action<TInstance, object, TEventArgs> OnEventAction
		{
			get
			{
				return this.onEventAction;
			}
			set
			{
				if (value != null && !value.Method.IsStatic)
				{
					throw new ArgumentException("OnEventAction method must be static otherwise the event WeakEventListner class does not prevent memory leaks.");
				}

				this.onEventAction = value;
			}
		}

		internal Action<RadWeakEventListener<TInstance, TSource, TEventArgs>, TSource> OnDetachAction
		{
			get 
			{ 
				return onDetachAction; 
			}
			set
			{
				if (value != null && !value.Method.IsStatic)
				{
					throw new ArgumentException("OnDetachAction method must be static otherwise the event WeakEventListner cannot guarantee to unregister the handler.");
				}

				onDetachAction = value;
			}
		}

		#endregion


		#region Constructors

		public RadWeakEventListener(TInstance instance, TSource source)
		{
			Guard.ThrowExceptionIfNull(instance, "instance");
			Guard.ThrowExceptionIfNull(source, "source");

			this.weakInstance = new WeakReference(instance);
			this.weakSource = new WeakReference(source);
		}

		#endregion


		#region Methods

		public void OnEvent(object source, TEventArgs eventArgs)
		{
			TInstance target = (TInstance)weakInstance.Target;
			if (null != target)
			{
				if (null != OnEventAction)
				{
					this.OnEventAction(target, source, eventArgs);
				}
			}
			else
			{
				this.Detach();
			}
		}

		public void Detach()
		{
			TSource source = (TSource)weakSource.Target;            
			if (null != this.OnDetachAction && source != null)
			{
				this.OnDetachAction(this, source);
				this.OnDetachAction = null;
			}
		}

		#endregion
	}
}