﻿using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    // TODO: Thread safety?
    internal class AutoFillDataSeriesManager
    {
        #region Fields

        private readonly NamedObjectList<IAutoFillGenerator> autoFillGenerators;
        private readonly Worksheet worksheet;

        #endregion


        #region Properties

        public NamedObjectList<IAutoFillGenerator> AutoFillGenerators
        {
            get
            {
                return this.autoFillGenerators;
            }
        }

        #endregion


        #region Constructors

        public AutoFillDataSeriesManager(Worksheet worksheet)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            this.worksheet = worksheet;

            this.autoFillGenerators = new NamedObjectList<IAutoFillGenerator>();
            this.InitBuiltInGenerators();
        }

        #endregion


        #region Methods

        private void InitBuiltInGenerators()
        {
            // default generator
            this.autoFillGenerators.AddLast(new RepeatAutoFillGenerator());

            // number generator
            autoFillGenerators.AddLast(new NumberAutoFillGenerator());

            // numbered text generator
            autoFillGenerators.AddLast(new NumberedTextAutoFillGenerator());

            // quarters generator
            autoFillGenerators.AddLast(new QuarterAutoFillGenerator());

            // weekdays generator
            autoFillGenerators.AddLast(new ShortLowerCaseWeekDaysAutoFillGenerator());
            autoFillGenerators.AddLast(new ShortUpperCaseWeekDaysAutoFillGenerator());
            autoFillGenerators.AddLast(new LongLowerCaseWeekDaysAutoFillGenerator());
            autoFillGenerators.AddLast(new LongUpperCaseWeekDaysAutoFillGenerator());

            // months generator
            autoFillGenerators.AddLast(new ShortLowerCaseMonthsAutoFillGenerator());
            autoFillGenerators.AddLast(new ShortUpperCaseMonthsAutoFillGenerator());
            autoFillGenerators.AddLast(new LongLowerCaseMonthsAutoFillGenerator());
            autoFillGenerators.AddLast(new LongUpperCaseMonthsAutoFillGenerator());

            // ordinal numbers generator
            autoFillGenerators.AddLast(new OrdinalNumbersFillGenerator());
        }

        public void FillAuto(CellIndex[] indexes, int initialIndexesCount)
        {
            foreach (var generator in this.autoFillGenerators)
            {
                generator.FillSeries(this.worksheet, indexes, initialIndexesCount);
            }
        }

        #endregion
    }
}