﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class LongLowerCaseMonthsAutoFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "LongLowerCaseMonths";
        private static readonly int RangeLowerBound = 1;
        private static readonly int RangeUpperBound = 12;
        private static readonly bool AllowReversingBelowLowerBound = false;

        #endregion


        #region Fields

        private readonly Dictionary<string, int> monthsToInt;
        private readonly string[] monthsToIntKeys;

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get 
            {
                return AllowReversingBelowLowerBound;
            }
        }

        #endregion


        #region Constructors

        public LongLowerCaseMonthsAutoFillGenerator()
        {
            this.monthsToInt = new Dictionary<string, int>();

            this.monthsToInt.Add("january", 1);
            this.monthsToInt.Add("february", 2);
            this.monthsToInt.Add("march", 3);
            this.monthsToInt.Add("april", 4);
            this.monthsToInt.Add("may", 5);
            this.monthsToInt.Add("june", 6);
            this.monthsToInt.Add("july", 7);
            this.monthsToInt.Add("august", 8);
            this.monthsToInt.Add("september", 9);
            this.monthsToInt.Add("october", 10);
            this.monthsToInt.Add("november", 11);
            this.monthsToInt.Add("december", 12);

            this.monthsToIntKeys = this.monthsToInt.Keys.ToArray();
        }

        #endregion


        #region Methods

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType != CellValueType.Text)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            if (this.monthsToInt.ContainsKey(text))
            {
                number = this.monthsToInt[text];
                return true;
            }

            return false;
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            return new TextCellValue(this.monthsToIntKeys[(int)value - 1]);
        }

        #endregion
    }
}