﻿using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class NumberAutoFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "Number";
        private static readonly double RangeLowerBound = double.MinValue;
        private static readonly double RangeUpperBound = double.MaxValue;
        private static readonly bool AllowReversingBelowLowerBound = false;
        private static readonly int MinAllowedInitialValues = 2;

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get
            {
                return AllowReversingBelowLowerBound;
            }
        }

        protected override int MinAllowedInitialValuesCount
        {
            get
            {
                return MinAllowedInitialValues;
            }
        }

        #endregion


        #region Methods

        protected override bool IsValueFitInDesiredBounds(double? value)
        {
            return base.IsValueFitInDesiredBounds(value) || value == null;
        }

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType == CellValueType.Boolean ||
                value.ValueType == CellValueType.Formula ||
                value.ValueType == CellValueType.Empty)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);

            return double.TryParse(text, out number);
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            return value.ToCellValue();
        }

        protected override bool HaveTheSameOrgin(ICellValue previousCellValue, ICellValue currentCellValue, double? previousNumberValue, double? currentNumberValues)
        {
            return true;
        }

        #endregion
    }
}