﻿using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class NumberedTextAutoFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "NumberedText";
        private static readonly int RangeLowerBound = 0;
        private static readonly int RangeUpperBound = int.MaxValue;
        private static readonly bool AllowReversingBelowLowerBound = true;

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get
            {
                return AllowReversingBelowLowerBound;
            }
        }

        #endregion


        #region Methods

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType != CellValueType.Text)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            if (text == null)
            {
                number = -1;
                return false;
            }

            int firstDigitIndex = text.Length;

            StringBuilder numberPart = new StringBuilder();
            while (firstDigitIndex > 0)
            {
                firstDigitIndex--;
                if (!char.IsDigit(text[firstDigitIndex]))
                {
                    firstDigitIndex++;
                    break;
                }

                char digit = text[firstDigitIndex];
                numberPart.Insert(0, new char[] { digit });
            }

            number = -1;
            int numberResult;

            if(int.TryParse(numberPart.ToString(), out numberResult))
            {
                number = numberResult;
                return true;
            }

            return false;
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            if (firstInitialValue.ValueType != CellValueType.Text)
            {
                return null;
            }

            string stringValue = firstInitialValue.GetResultValueAsString(CellValueFormat.GeneralFormat);

            double number;
            if (this.TryParse(firstInitialValue, out number))
            {
                int numberIndex = stringValue.LastIndexOf(((int)number).ToString());

                string prefix = stringValue.Substring(0, numberIndex);

                return new TextCellValue(prefix + value.ToString());
            }
            else
            {
                return null;
            }
        }

        protected override bool HaveTheSameOrgin(ICellValue previousCellValue, ICellValue currentCellValue, double? previousNumberValue, double? currentNumberValues)
        {
            if (!previousNumberValue.HasValue || !currentNumberValues.HasValue)
            {
                return false;
            }

            if (previousCellValue == null ||
                currentCellValue == null ||
                (previousNumberValue > this.ValuesRangeUpperBound || previousNumberValue < this.ValuesRangeLowerBound) ||
                (currentNumberValues > this.ValuesRangeUpperBound || currentNumberValues < this.ValuesRangeLowerBound))
            {
                return true;
            }

            string previousStringValue = 
                (previousCellValue.ValueType == CellValueType.Text) ? previousCellValue.GetResultValueAsString(CellValueFormat.GeneralFormat) : null;
            string currentStringValue =
                (currentCellValue.ValueType == CellValueType.Text) ? currentCellValue.GetResultValueAsString(CellValueFormat.GeneralFormat) : null;

            string previousPrefix = previousStringValue.Remove(previousStringValue.LastIndexOf(previousNumberValue.ToString()));
            string currentPrefix = currentStringValue.Remove(currentStringValue.LastIndexOf(currentNumberValues.ToString()));

            return currentPrefix == previousPrefix;
        }

        #endregion
    }
}