﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class OrdinalNumbersFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "OrdinalNumbers";
        private static readonly int RangeLowerBound = 0;
        private static readonly int RangeUpperBound = int.MaxValue;
        private static readonly bool AllowReversingBelowLowerBound = true;

        private const string stSuffix = "st";
        private const string ndSuffix = "nd";
        private const string rdSuffix = "rd";
        private const string thSuffix = "th";

        private static readonly List<string> ordinalSuffixes = new List<string>() { stSuffix, ndSuffix, rdSuffix, thSuffix };

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get
            {
                return AllowReversingBelowLowerBound;
            }
        }

        #endregion


        #region Methods

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType != CellValueType.Text)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            number = -1;

            if(text.Length < 3 || !ordinalSuffixes.Any(sf => text.EndsWith(sf)))
            {
                return false;
            }

            string numberPart = text.Substring(0, text.Length - 2);
            return double.TryParse(numberPart, out number);
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            string resultString;

            int result = ((int)value) % 10;
            switch (result)
            {
                case 1: resultString = value + stSuffix;
                    break;
                case 2: resultString = value + ndSuffix;
                    break;
                case 3: resultString = value + rdSuffix;
                    break;
                default: resultString = value + thSuffix;
                    break;
            }

            switch ((int)value)
            {
                case 11:
                case 12:
                case 13: resultString = value + thSuffix;
                    break;
            }

            return new TextCellValue(resultString);
        }

        #endregion
    }
}