﻿using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class QuarterAutoFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "Quarter";
        private static readonly int RangeLowerBound = 1;
        private static readonly int RangeUpperBound = 4;
        private static readonly bool AllowReversingBelowLowerBound = false;

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get
            {
                return AllowReversingBelowLowerBound;
            }
        }

        #endregion


        #region Methods

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType != CellValueType.Text)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            number = -1;
            if (text[0] == 'q' ||
                text[0] == 'Q')
            {
                string postfix = text.Substring(1);
                if (double.TryParse(postfix, out number))
                {
                    if (number >= RangeLowerBound &&
                        number <= RangeUpperBound)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            string firstInitialStringValue = firstInitialValue.GetResultValueAsString(CellValueFormat.GeneralFormat);
            char prefix = firstInitialStringValue[0];
            return new TextCellValue(prefix + value.ToString());
        }

        #endregion
    }
}