﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.DataSeries.AutoFill
{
    internal class ShortLowerCaseWeekDaysAutoFillGenerator : NumberRelatedAutoFillGeneratorBase
    {
        #region Constants

        private static readonly string RuleName = "ShortLowerCaseWeekDays";
        private static readonly int RangeLowerBound = 1;
        private static readonly int RangeUpperBound = 7;
        private static readonly bool AllowReversingBelowLowerBound = false;

        #endregion


        #region Fields

        private readonly Dictionary<string, int> weekDayToInt;
        private readonly string[] weekDayToIntKeys;

        #endregion


        #region Properties

        public override string Name
        {
            get
            {
                return RuleName;
            }
        }

        protected override double ValuesRangeLowerBound
        {
            get
            {
                return RangeLowerBound;
            }
        }

        protected override double ValuesRangeUpperBound
        {
            get
            {
                return RangeUpperBound;
            }
        }

        protected override bool AllowReversingOfTheFitBelowTheLowerBoundValue
        {
            get
            {
                return AllowReversingBelowLowerBound;
            }
        }

        #endregion


        #region Constructors

        public ShortLowerCaseWeekDaysAutoFillGenerator()
        {
            this.weekDayToInt = new Dictionary<string, int>();

            this.weekDayToInt.Add("mon", 1);
            this.weekDayToInt.Add("tue", 2);
            this.weekDayToInt.Add("wed", 3);
            this.weekDayToInt.Add("thu", 4);
            this.weekDayToInt.Add("fri", 5);
            this.weekDayToInt.Add("sat", 6);
            this.weekDayToInt.Add("sun", 7);

            this.weekDayToIntKeys = this.weekDayToInt.Keys.ToArray();
        }

        #endregion


        #region Methods

        protected override bool TryParse(ICellValue value, out double number)
        {
            number = -1;
            if (value.ValueType != CellValueType.Text)
            {
                return false;
            }

            string text = value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            number = -1;
            if (this.weekDayToInt.ContainsKey(text))
            {
                number = this.weekDayToInt[text];
                return true;
            }

            return false;
        }

        protected override ICellValue ValueToResultCellValue(double value, ICellValue firstInitialValue)
        {
            return new TextCellValue(this.weekDayToIntKeys[(int)value - 1]);
        }

        #endregion
    }
}