﻿namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using AdditionExpression.
    /// </summary>
    public class AdditionExpression : NumberBinaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Plus;
            }
        }

        #endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AdditionExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public AdditionExpression(RadExpression left, RadExpression right) : base(left, right)
		{
		}

		#endregion


		#region Methods

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new AdditionExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(double[] operands)
        {
            return NumberExpression.CreateValidNumberOrErrorExpression(operands[0] + operands[1]);
        }

		#endregion
	}
}