﻿using System;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using AmpersandExpression, that concatenates the two operands as Strings.
    /// </summary>
    public class AmpersandExpression : StringBinaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Ampersand;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AmpersandExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public AmpersandExpression(RadExpression left, RadExpression right) : base(left, right)
		{
		}

		#endregion


		#region Methods

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(string[] operands)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(operands[0]);
            builder.Append(operands[1]);

            return new StringExpression(builder.ToString());
        }

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new AmpersandExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

		#endregion
    }
}
