﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Abstract class representing a cell reference binary operator expression.
    /// </summary>
    public abstract class CellReferenceBinaryOperatorExpression : BinaryOperatorExpression<CellReferenceRangeExpression>
    {
        #region Properties
        
        /// <summary>
        /// Gets the type of the operands.
        /// </summary>
        /// <returns>The type of the operands as ArgumentType.</returns>
        /// <value>The type of the operands.</value>
        public override ArgumentType OperandsType
        {
            get 
            {
                return ArgumentType.Reference; 
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CellReferenceBinaryOperatorExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        protected CellReferenceBinaryOperatorExpression(RadExpression left, RadExpression right)
            : base(left, right)
		{
		}

		#endregion


        #region Methods

        /// <summary>
        /// Toes the string.
        /// </summary>
        /// <param name="cultureInfo">The culture info.</param>
        /// <returns>The result string.</returns>
        internal override string ToString(Utilities.SpreadsheetCultureHelper cultureInfo)
        {
            if (this.Left.ToString(cultureInfo) == SpreadsheetCultureHelper.ReferenceErrorValue ||
                  this.Right.ToString(cultureInfo) == SpreadsheetCultureHelper.ReferenceErrorValue)
            {
                return SpreadsheetCultureHelper.ReferenceErrorValue;
            }

            return base.ToString(cultureInfo);
        }

        #endregion
    }
}