﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Abstract class providing methods and properties for using ComparisonOperatorExpression.
    /// </summary>
    public abstract class ComparisonOperatorExpression : BinaryOperatorExpression<ConstantExpression>
    {
        #region Properties
                
        /// <summary>
        /// Gets the type of the operands.
        /// </summary>
        /// <returns>The type of the operands as ArgumentType.</returns>
        /// <value>The type of the operands.</value>
        public override Functions.ArgumentType OperandsType
        {
            get 
            {
                return ArgumentType.Any;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ComparisonOperatorExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        protected ComparisonOperatorExpression(RadExpression left, RadExpression right) : base(left, right)
        {
        }

        #endregion


        #region Methods
                
        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(ConstantExpression[] operands)
        {
            bool result = false;

            ConstantExpression leftOperandValue = operands[0] is EmptyExpression ? NumberExpression.Zero: operands[0];
            ConstantExpression rightOperandValue = operands[1] is EmptyExpression ? NumberExpression.Zero : operands[1];

            if (leftOperandValue.GetType().Equals(rightOperandValue.GetType()))
            {
                result = this.CompareSameTypeExpressions(leftOperandValue, rightOperandValue);
            }
            else
            {
                result = this.CompareDifferentTypeExpressions(leftOperandValue, rightOperandValue);
            }
          
            return result.ToBooleanExpression();
        }

        private bool CompareSameTypeExpressions(RadExpression leftOperandValue, RadExpression rightOperandValue)
        {
            NumberExpression leftNumberExpression = leftOperandValue as NumberExpression;
            NumberExpression rightNumberExpression = rightOperandValue as NumberExpression;

            if (leftNumberExpression != null && rightNumberExpression != null)
            {
                return this.CompareNumberExpressions(leftNumberExpression, rightNumberExpression);
            }

            StringExpression leftStringExpression = leftOperandValue as StringExpression;
            StringExpression rightStringExpression = rightOperandValue as StringExpression;

            if (leftStringExpression != null && rightStringExpression != null)
            {
                return this.CompareStringExpressions(leftStringExpression, rightStringExpression);
            }

            BooleanExpression leftBooleanExpression = leftOperandValue as BooleanExpression;
            BooleanExpression rightBooleanExpression = rightOperandValue as BooleanExpression;

            if (leftBooleanExpression != null && rightBooleanExpression != null)
            {
                return this.CompareBooleanExpressions(leftBooleanExpression, rightBooleanExpression);
            }

            EmptyExpression leftEmptyExpression = leftOperandValue as EmptyExpression;
            EmptyExpression rightEmtpyExpression = rightOperandValue as EmptyExpression;

            if (leftEmptyExpression != null && rightEmtpyExpression != null)
            {
                return true;
            }

            return false;
        }

        internal int GetExpressionTypeNumber(ConstantExpression expression)
        {
            int result = 0;

            if (expression is NumberExpression)
            {
                result = 1;
            }
            else if (expression is StringExpression)
            {
                result = 2;
            }
            else if (expression is BooleanExpression)
            {
                result = 4;
            }

            return result;
        }

        /// <summary>
        /// Compares the string expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected abstract bool CompareStringExpressions(StringExpression left, StringExpression right);

        /// <summary>
        /// Compares the number expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected abstract bool CompareNumberExpressions(NumberExpression left, NumberExpression right);

        /// <summary>
        /// Compares the boolean expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected abstract bool CompareBooleanExpressions(BooleanExpression left, BooleanExpression right);

        /// <summary>
        /// Compares the different type expressions.
        /// </summary>
        /// <param name="leftOperandValue">The left operand value.</param>
        /// <param name="rightOperandValue">The right operand value.</param>
        /// <returns>Boolean result.</returns>
        protected abstract bool CompareDifferentTypeExpressions(ConstantExpression leftOperandValue, ConstantExpression rightOperandValue);

        #endregion
    }
}
