﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Abstract class providing methods and properties for using ConstantExpression with value of type T.
    /// </summary>
    /// <typeparam name="T">The type T of the expression value.</typeparam>
    public abstract class ConstantExpression<T> : ConstantExpression
    {
        #region Fields

		private readonly T value;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <value>The value.</value>
        /// <returns>The value of type T.</returns>
        public T Value
        {
			get
			{
				return this.value;
			}
		}

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ConstantExpression{T}" /> class.
        /// </summary>
        /// <param name="value">The value.</param>
        protected ConstantExpression(T value)
        {
			this.value = value;
		}

		#endregion


		#region Methods

        internal override bool SimilarEquals(object obj)
        {
            ConstantExpression<T> other = obj as ConstantExpression<T>;

            if (other == null)
            {
                return false;
            }

            return TelerikHelper.EqualsOfT(this.Value, other.Value);
        }

        internal override string GetValueAsString(SpreadsheetCultureHelper cultureInfo)
		{
			return this.Value.ToString();
		}

        internal override string ToString(SpreadsheetCultureHelper cultureInfo)
        {
            return this.GetValueAsString(cultureInfo);
        }

		#endregion
    }
}