﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using EmptyExpression.
    /// </summary>
	public class ErrorExpression : ConstantExpression<string>
	{
		#region Fields

        private readonly string message;
        private readonly Exception exception;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the exception.
        /// </summary>
        /// <value>The exception.</value>
        /// <returns>The exception of the ErrorExpression.</returns>
        public Exception Exception
        {
            get
            {
                return this.exception;
            }
        }

        /// <summary>
        /// Gets the message.
        /// </summary>
        /// <value>The message.</value>
        /// <returns>The error message.</returns>
        public string Message
        {
			get
			{
				return this.message;
			}
		}

		#endregion


		#region Constructors

        internal ErrorExpression(string value, string message, Exception exception = null)
            : base(value)
		{
            Guard.ThrowExceptionIfNullOrEmpty(value, "value");
			Guard.ThrowExceptionIfNullOrEmpty(message, "message");

			this.message = message;
            this.exception = exception;
		}

		#endregion
    }
}