﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides a set of RadSpreadsheet's error expressions.
    /// </summary>
    public static class ErrorExpressions
    {
		#region Fields

        /// <summary>
        /// Represents null error error expression.
        /// </summary>
        public static readonly ErrorExpression NullError;

        /// <summary>
        /// Represents division by zero error expression.
        /// </summary>
        public static readonly ErrorExpression DivisionByZero;

        /// <summary>
        /// Represents value error error expression.
        /// </summary>
        public static readonly ErrorExpression ValueError;

        /// <summary>
        /// Represents name error error expression.
        /// </summary>
        public static readonly ErrorExpression NameError;

        /// <summary>
        /// Represents reference error error expression.
        /// </summary>
        public static readonly ErrorExpression ReferenceError;

        /// <summary>
        /// Represents number error error expression.
        /// </summary>
        public static readonly ErrorExpression NumberError;

        /// <summary>
        /// Represents not available error error expression.
        /// </summary>
        public static readonly ErrorExpression NotAvailableError;

        /// <summary>
        /// Represents cyclic reference error expression.
        /// </summary>
        public static readonly ErrorExpression CyclicReference;
        
        private static readonly Dictionary<string, ErrorExpression> errors; 

		#endregion


        #region Constructors

        static ErrorExpressions()
        {
            errors = new Dictionary<string, ErrorExpression>();

            NullError = CreateErrorExpression(SpreadsheetCultureHelper.NullErrorValue, SpreadsheetStrings.NullErrorMessage);
            DivisionByZero = CreateErrorExpression(SpreadsheetCultureHelper.DivisionByZeroErrorValue, SpreadsheetStrings.DivideByZeroErrorMessage);
            ValueError = CreateErrorExpression(SpreadsheetCultureHelper.ValueErrorValue, SpreadsheetStrings.ValueErrorMessage);
            NameError = CreateErrorExpression(SpreadsheetCultureHelper.NameErrorValue, SpreadsheetStrings.NameErrorMessage);
            ReferenceError = CreateErrorExpression(SpreadsheetCultureHelper.ReferenceErrorValue, SpreadsheetStrings.ReferenceErrorMessage);
            NumberError = CreateErrorExpression(SpreadsheetCultureHelper.NumberErrorValue, SpreadsheetStrings.NumberErrorMessage);
            NotAvailableError = CreateErrorExpression(SpreadsheetCultureHelper.NotAvailableErrorValue, SpreadsheetStrings.NotAvailableErrorMessage);
            CyclicReference = CreateErrorExpression("0", SpreadsheetStrings.CyclicReferenceMessage, 
                new ExpressionException(SpreadsheetStrings.CyclicReferenceErrorMessage, "Spreadsheet_ErrorExpressions_CyclicReferenceErrorMessage"));
        }

        #endregion


        #region Methods

        internal static ErrorExpression FindErrorExpression(string value)
        {
            Guard.ThrowExceptionIfNullOrEmpty(value, "value");

            return errors[value];
        }

        private static ErrorExpression CreateErrorExpression(string value, string message, Exception exception = null)
        {
            Guard.ThrowExceptionIfNullOrEmpty(value, "value");
            Guard.ThrowExceptionIfNullOrEmpty(message, "message");

            ErrorExpression error = new ErrorExpression(value, message, exception);
            errors[value] = error;

            return error;
        }

        #endregion
    }
}