﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    internal class ExpressionCloneAndTranslateContext
    {
        public Worksheet Worksheet { get; private set; }
        public int RowIndex { get; private set; }
        public int ColumnIndex { get; private set; }
        public InputStringCollection OldFormatCollection { get; private set; }
        public InputStringCollection NewFormatCollection { get; private set; }

        public bool IsInsertingRemovingCells { get; private set; }
        public bool IsHorizontal { get; private set; }
        public bool IsRemove { get; private set; }
        public CellRange Range { get; private set; }
        public bool OnOverflowStartOver { get; set; }

        public ExpressionCloneAndTranslateContext(Worksheet worksheet, int rowIndex, int columnIndex, bool onOverflowStartOver)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            this.Worksheet = worksheet;
            this.RowIndex = rowIndex;
            this.ColumnIndex = columnIndex;
            this.OnOverflowStartOver = onOverflowStartOver;
        }

        public ExpressionCloneAndTranslateContext(Worksheet worksheet, int rowIndex, int columnIndex, InputStringCollection oldFormatCollection, InputStringCollection newFormatCollection, bool onOverflowStartOver)
            : this(worksheet, rowIndex, columnIndex, onOverflowStartOver)
        {
            Guard.ThrowExceptionIfNull(oldFormatCollection, "oldFormatCollection");
            Guard.ThrowExceptionIfNull(newFormatCollection, "newFormatCollection");

            this.OldFormatCollection = oldFormatCollection;
            this.NewFormatCollection = newFormatCollection;
        }

        public ExpressionCloneAndTranslateContext(Worksheet worksheet, int rowIndex, int columnIndex, InputStringCollection oldFormatCollection,
            InputStringCollection newFormatCollection, bool isHorizontal, CellRange range, bool isRemove, bool onOverflowStartOver)
            : this(worksheet, rowIndex, columnIndex, oldFormatCollection, newFormatCollection, onOverflowStartOver)
        {
            this.IsInsertingRemovingCells = true;
            
            this.IsHorizontal = isHorizontal;
            this.Range = range;
            this.IsRemove = isRemove;
        }
    }
}
