﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    internal class ExpressionTranslateContext
    {
        public Worksheet RenamedWorksheet { get; private set; }
        public Workbook RenamedWorkbook { get; private set; }
        public string OldSpreadsheetName { get; private set; }
        public string NewSpreadsheetName { get; private set; }

        public bool IsWorkbookRenamed
        {
            get
            {
                return this.RenamedWorkbook != null;
            }
        }

        public bool IsWorksheetRenamed
        {
            get
            {
                return this.RenamedWorksheet != null;
            }
        }

        public bool IsNameRenamed
        {
            get
            {
                return !string.IsNullOrEmpty(OldSpreadsheetName) && 
                    !string.IsNullOrEmpty(NewSpreadsheetName);
            }
        }

        public ExpressionTranslateContext(Worksheet worksheet)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");

            this.RenamedWorksheet = worksheet;
        }

        public ExpressionTranslateContext(Workbook workbook)
        {
            Guard.ThrowExceptionIfNull(workbook, "workbook");

            this.RenamedWorkbook = workbook;
        }

        public ExpressionTranslateContext(string oldName, string newName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(oldName, "oldName");
            Guard.ThrowExceptionIfNullOrEmpty(newName, "newName");

            this.OldSpreadsheetName = oldName;
            this.NewSpreadsheetName = newName;
        }
    }
}
