﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ERFC function.
    /// </summary>
    public class ErfC : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ERFC";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <returns>The argument conversion rules as ArgumentConversionRules.</returns>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return ArgumentConversionRules.NonBoolNumberFunctionConversion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static ErfC()
        {
            string description = "Returns the complementary ERF function integrated between x and infinity.";
            string descriptionKey = "Spreadsheet_Functions_ErfC_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("X",
                                 "The lower bound for integrating ERFC.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_X",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_ErfC_XInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Engineering, description, requiredArguments, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double result = Telerik.Windows.Documents.Spreadsheet.Maths.Functions.ErfC(context.Arguments[0]);

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

        #endregion
    }
}