﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Telerik.Windows.Documents.Spreadsheet.Maths;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the IMDIV function.
    /// </summary>
    public class ImDiv : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "IMDIV";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static ImDiv()
        {
            string description = "Returns the quotient of two complex numbers in x + yi or x + yj text format.";
            string descriptionKey = "Spreadsheet_Functions_ImDiv_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo(
                    "Inumber1",
                    "The complex numerator or dividend.",
                    ArgumentType.Text,
                    nameLocalizationKey:"Spreadsheet_Functions_Args_Inumber1",
                    descriptionLocalizationKey:"Spreadsheet_Functions_ImDiv_Inumber1Info"),

                new ArgumentInfo(
                    "Inumber2",
                    "The complex denominator or divisor.",
                    ArgumentType.Text,
                    nameLocalizationKey:"Spreadsheet_Functions_Args_Inumber2",
                    descriptionLocalizationKey:"Spreadsheet_Functions_ImDiv_Inumber2Info"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Engineering, description, requiredArguments, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            try
            {
                string imaginarySymbol;

                if (MathUtility.TryGetImaginarySymbol(context.Arguments, out imaginarySymbol))
                {
                    var complexResult = EngineeringFunctions.IMDIV(context.Arguments[0], context.Arguments[1]);
                    string result = EngineeringFunctions.COMPLEX(complexResult.Real, complexResult.Imaginary, imaginarySymbol);

                    return new StringExpression(result);
                }
                else
                {
                    return ErrorExpressions.ValueError;
                }
            }
            catch (Exception)
            {
                return ErrorExpressions.NumberError;
            }
        }

        #endregion
    }
}