﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Telerik.Windows.Documents.Spreadsheet.Maths;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the IMLOG2 function.
    /// </summary>
    public class ImLog2 : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "IMLOG2";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static ImLog2()
        {
            string description = "Returns the base-2 logarithm of a complex number in x + yi or x + yj text format.";
            string descriptionKey = "Spreadsheet_Functions_ImLog2_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo(
                    "Inumber",
                    "A complex number for which you want the base-2 logarithm.",
                    ArgumentType.Text,
                    nameLocalizationKey:"Spreadsheet_Functions_Args_Inumber",
                    descriptionLocalizationKey:"Spreadsheet_Functions_ImLog2_InumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Engineering, description, requiredArguments, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            try
            {
                string imaginarySymbol = MathUtility.GetImaginarySymbol(context.Arguments[0]);
                var complexResult = EngineeringFunctions.IMLOG2(context.Arguments[0]);
                string result = EngineeringFunctions.COMPLEX(complexResult.Real, complexResult.Imaginary, imaginarySymbol);

                return new StringExpression(result);
            }
            catch (Exception)
            {
                return ErrorExpressions.NumberError;
            }
        }

        #endregion
    }
}