﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Class representing function evaluation context.
    /// </summary>
    /// <typeparam name="T">Type of the arguments.</typeparam>
    public class FunctionEvaluationContext<T>
    {
        /// <summary>
        /// Gets the arguments.
        /// </summary>
        /// <value>The arguments.</value>
        public T[] Arguments { get; private set; }

        /// <summary>
        /// Gets the worksheet.
        /// </summary>
        /// <value>The worksheet.</value>
        public Worksheet Worksheet { get; private set; }

        /// <summary>
        /// Gets the index of the row.
        /// </summary>
        /// <value>The index of the row.</value>
        public int RowIndex { get; private set; }

        /// <summary>
        /// Gets the index of the column.
        /// </summary>
        /// <value>The index of the column.</value>
        public int ColumnIndex { get; private set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="FunctionEvaluationContext{T}" /> class.
        /// </summary>
        /// <param name="arguments">The arguments.</param>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        public FunctionEvaluationContext(T[] arguments, Worksheet worksheet, int rowIndex, int columnIndex)
        {
            this.Arguments = arguments;
            this.Worksheet = worksheet;
            this.RowIndex = rowIndex;
            this.ColumnIndex = columnIndex;
        }
    }
}