﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Abstract class providing methods and properties for creating an RadSpreadsheet function.
    /// </summary>
    public abstract class FunctionWithArguments : FunctionBase
    {
        #region Methods

        private ArgumentInfo GetArgumentInfoByIndex(ArgumentInfo[] argumentsInfo, int requiredArgumentsCount, int optionalArgumentsCount, int index)
        {
            ArgumentInfo result = null;

            if (index < requiredArgumentsCount)
            {
                result = argumentsInfo[index];
            }
            else
            {
                int newIndex = (index - requiredArgumentsCount) % optionalArgumentsCount + requiredArgumentsCount;
                result = argumentsInfo[newIndex];
            }

            return result;
        }

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected sealed override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            ErrorExpression errorExpression = null;
            List<object> convertedArguments = new List<object>();

            ArgumentInfo[] argumentsInfo = this.FunctionInfo.GetArguments(0).ToArray();

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                ArgumentInfo argumentInfo = GetArgumentInfoByIndex(argumentsInfo, this.FunctionInfo.RequiredArgumentsCount, this.FunctionInfo.OptionalArgumentsCount, i);

                errorExpression = FunctionHelper.TryConvertArgument(context.Arguments[i], this.ArgumentConversionRules, argumentInfo.Type, convertedArguments);

                if (errorExpression != null)
                {
                    return errorExpression;
                }
            }

            if (!this.FunctionInfo.IsDefaultValueFunction && !this.IsArgumentNumberValid(convertedArguments.Count))
            {
                return ErrorExpressions.ValueError;
            }

            FunctionEvaluationContext<object> newContext = new FunctionEvaluationContext<object>(convertedArguments.ToArray(), context.Worksheet, context.RowIndex, context.ColumnIndex);
            return EvaluateOverride(newContext);
        }

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected virtual RadExpression EvaluateOverride(FunctionEvaluationContext<object> context)
        {
            throw new NotSupportedException();
        }

        #endregion
    }
}