﻿using System;
using System.ComponentModel;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Provides methods and properties for creating a RadSpreadsheet functions with same arguments type.
    /// </summary>
    /// <typeparam name="T">The type T of functions arguments.</typeparam>
    public abstract class FunctionWithSameTypeArguments<T> : FunctionWithArguments
    {
        #region Methods

        private T[] ConvertArguments(object[] arguments)
        {
            T[] argumentsT = new T[arguments.Length];

            for (int i = 0; i < arguments.Length; i++)
            {
                argumentsT[i] = (T)arguments[i];
            }
            return argumentsT;
        }

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context"></param>
        /// <returns>Functions result as RadExpression.</returns>
        protected sealed override RadExpression EvaluateOverride(FunctionEvaluationContext<object> context)
        {
            T[] argumentsT = this.ConvertArguments(context.Arguments.ToArray());

            FunctionEvaluationContext<T> newContext = new FunctionEvaluationContext<T>(argumentsT, context.Worksheet, context.RowIndex, context.ColumnIndex);
            return this.EvaluateOverride(newContext);
        }

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected virtual RadExpression EvaluateOverride(FunctionEvaluationContext<T> context)
        {
            throw new NotSupportedException();
        }

        #endregion
    }
}