﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ISBLANK function.
    /// </summary>
    public class IsBlank : FunctionBase
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ISBLANK";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static IsBlank()
        {
            string description = "Returns the logical value TRUE if the value argument is a reference to an empty cell; otherwise it returns FALSE";
            string descriptionKey = "Spreadsheet_Functions_IsBlank_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Value",
                                 "The value that you want tested. The value argument can be a blank (empty cell), error, logical value, text, number, or reference value, or a name referring to any of these.",
                                 ArgumentType.Any,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_IsBlank_ValueInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            RadExpression expression = context.Arguments[0];
            RadExpression expressionValue = expression.GetValueAsConstantExpression();

            if (expressionValue.Equals(ErrorExpressions.CyclicReference))
            {
                return ErrorExpressions.CyclicReference;
            }

            bool result = expression is EmptyExpression || expressionValue is EmptyExpression;

            return result.ToBooleanExpression();
        }

        #endregion

    }
}