﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ISERR function.
    /// </summary>
    public class IsErr : FunctionBase
    {
        #region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ISERR";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static IsErr()
        {
            string description = "Returns TRUE if the value is any error value except #N/A.";
            string descriptionKey = "Spreadsheet_Functions_IsErr_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Value",
                                 "is the value you want tested. Value can be a blank (empty cell), error, logical, text, number, or reference value, or a name referring to any of these, that you want to test.",
                                 ArgumentType.Any,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_IsErr_ValueInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            RadExpression expressionValue = context.Arguments[0].GetValueAsConstantExpression();

            if (expressionValue.Equals(ErrorExpressions.CyclicReference))
            {
                return ErrorExpressions.CyclicReference;
            }

            bool result = expressionValue is ErrorExpression && expressionValue != ErrorExpressions.NotAvailableError;

            return result.ToBooleanExpression();
        }

		#endregion

    }
}