﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ISFORMULA function.
    /// </summary>
    public class IsFormula : FunctionWithArguments
    {
        #region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ISFORMULA";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static IsFormula()
        {
            string description = "Checks whether there is a reference to a cell that contains a formula, and returns TRUE or FALSE.";
            string descriptionKey = "Spreadsheet_Functions_IsFormula_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Reference",
                                 "Reference is a reference to the cell you want to test. Reference can be a cell reference, a formula, or a name that refers to a cell.",
                                 ArgumentType.Reference,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Reference",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_IsFormula_ReferenceInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context"></param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<object> context)
        {
            CellReferenceRangeExpression cellReferenceRangeExpression = context.Arguments[0] as CellReferenceRangeExpression;
            if (cellReferenceRangeExpression != null && cellReferenceRangeExpression.IsValid)
            {
                int rowIndex = cellReferenceRangeExpression.CellReferenceRange.FromCellReference.ActualRowIndex;
                int columnIndex = cellReferenceRangeExpression.CellReferenceRange.FromCellReference.ActualColumnIndex;

                FormulaCellValue formulaCellValue = cellReferenceRangeExpression.Worksheet.
                    Cells[rowIndex, columnIndex].GetValue().Value as FormulaCellValue;

                if (formulaCellValue != null)
                {
                    return BooleanExpression.True;
                }
            }

            return BooleanExpression.False;
        }

		#endregion

    }
}