﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the N function.
    /// </summary>
    public class N : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "N";
        private static readonly FunctionInfo Info;

        private static readonly ArgumentConversionRules argumentConvertionRules = new ArgumentConversionRules
            (numberDirectArgument: ArgumentInterpretation.UseAsIs,
             boolDirectArgument : ArgumentInterpretation.UseAsIs,
             textNumberDirectArgument : ArgumentInterpretation.ConvertToDefault,
             nonTextNumberDirectArgument: ArgumentInterpretation.ConvertToDefault,
             emptyIndirectArgument: ArgumentInterpretation.ConvertToDefault,
             numberIndirectArgument : ArgumentInterpretation.UseAsIs,
             boolIndirectArgument : ArgumentInterpretation.UseAsIs,
             textNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault,
             nonTextNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault,
             arrayArgument : ArrayArgumentInterpretation.UseFirstElement);
                                        

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get 
            {
                return Info;
            }
        }

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <returns>The argument conversion rules as ArgumentConversionRules.</returns>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return argumentConvertionRules;
            }
        }

        #endregion


        #region Constructors

        static N()
        {
            string description = "Returns a value converted to a number.";
            string descriptionKey = "Spreadsheet_Functions_N_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Value",
                                 "The value you want converted. N converts values listed in the following table.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_N_ValueInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods
        
        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            return NumberExpression.CreateValidNumberOrErrorExpression(context.Arguments[0]);
        }

        #endregion
    }
}