﻿using System;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the NA function.
    /// </summary>
    public class NA : FunctionBase
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "NA";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static NA()
        {
            string description = "Returns the error value #N/A. #N/A is the error value that means \"no value is available.\" Use NA to mark empty cells. By entering #N/A in cells where you are missing information, you can avoid the problem of unintentionally including empty cells in your calculations. (When a formula refers to a cell containing #N/A, the formula returns the #N/A error value.)";
            string descriptionKey = "Spreadsheet_Functions_NA_Info";

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            return ErrorExpressions.NotAvailableError;
        }

        #endregion
    }
}