﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the TYPE function.
    /// </summary>
    public class TypeFunction : FunctionBase
    {
        #region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "TYPE";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static TypeFunction()
        {
            string description = "Returns the type of value. Use TYPE when the behavior of another function depends on the type of value in a particular cell.";
            string descriptionKey = "Spreadsheet_Functions_Type_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Value",
                                 "can be any RadSpreadsheet value, such as a number, text, logical value, and so on.",
                                 ArgumentType.Any,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Type_ValueInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Information, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            double result = 1;
            RadExpression valueExpression = context.Arguments[0].GetValueAsConstantExpression();

            if (valueExpression.Equals(ErrorExpressions.CyclicReference))
            {
                return ErrorExpressions.CyclicReference;
            }

            if (valueExpression is StringExpression)
            {
                result = 2;
            }
            else if (valueExpression is BooleanExpression)
            {
                result = 4;
            }
            else if (valueExpression is ErrorExpression)
            {
                result = 16;
            }
            else if (valueExpression is ArrayExpression)
            {
                result = 64;
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion

    }
}