﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the AND function.
    /// </summary>
    public class And : BooleansInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "AND";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static And()
        {
            string description = "Returns TRUE if all its arguments are TRUE; returns FALSE if one or more argument is FALSE.";
            string descriptionKey = "Spreadsheet_Functions_And_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Logical",
                                 "Logical1, logical2, ...     are 1 to 30 conditions you want to test that can be either TRUE or FALSE.",
                                 ArgumentType.Logical,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Logical",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_And_LogicalInfo"),
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Logical",
                                 "Logical1, logical2, ...     are 1 to 30 conditions you want to test that can be either TRUE or FALSE.",
                                 ArgumentType.Logical,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Logical",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_And_LogicalInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Logical, description, requiredArguments, optionalArguments, 254, false, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<bool> context)
        {
            bool result = true;

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                if (!context.Arguments[i])
                {
                    result = false;
                    break;
                }
            }

            return result ? BooleanExpression.True : BooleanExpression.False;
        }

        #endregion
    }
}