﻿using System;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the FALSE function.
    /// </summary>
    public class False : FunctionBase
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "FALSE";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static False()
        {
            string description = "Returns the logical value FALSE.";
            string descriptionKey = "Spreadsheet_Functions_False_Info";

            Info = new FunctionInfo(FunctionName, FunctionCategory.Logical, description, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method that evaluates the function with the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Function result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<RadExpression> context)
        {
            return BooleanExpression.False;
        }

        #endregion
    }
}