﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the NOT function.
    /// </summary>
    public class Not : BooleansInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "NOT";
        private static readonly FunctionInfo Info;
        private static readonly ArgumentConversionRules SingleArgumentBoolFunctionConvertion = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.Ignore,
                                                                                                                           textNumberIndirectArgument: ArgumentInterpretation.Ignore,
                                                                                                                           nonTextNumberDirectArgument: ArgumentInterpretation.Ignore,
                                                                                                                           nonTextNumberIndirectArgument: ArgumentInterpretation.Ignore,
                                                                                                                           arrayArgument: ArrayArgumentInterpretation.UseFirstElement);

        #endregion


        #region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <returns>The argument conversion rules as ArgumentConversionRules.</returns>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return SingleArgumentBoolFunctionConvertion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Not()
        {
            string description = "Reverses the value of its argument. Use NOT when you want to make sure a value is not equal to one particular value.";
            string descriptionKey = "Spreadsheet_Functions_Not_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Logical",
                                 "is a value or expression that can be evaluated to TRUE or FALSE.",
                                 ArgumentType.Logical,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Logical",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Not_LogicalInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Logical, description, requiredArguments, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<bool> context)
        {
            return context.Arguments[0] ? BooleanExpression.False : BooleanExpression.True;
        }

        #endregion
    }
}