﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the XOR function.
    /// </summary>
    public class Xor : BooleansInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "XOR";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Xor()
        {
            string description = "The result of XOR is TRUE when the number of TRUE inputs is odd and FALSE when the number of TRUE inputs is even.";
            string descriptionKey = "Spreadsheet_Functions_Xor_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Logical",
                                 "Logical1 is required, subsequent logical values are optional. 1 to 254 conditions you want to test that can be either TRUE or FALSE, and can be logical values, arrays, or references.",
                                 ArgumentType.Logical,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Logical",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Xor_LogicalInfo"),
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Logical",
                                 "Logical1 is required, subsequent logical values are optional. 1 to 254 conditions you want to test that can be either TRUE or FALSE, and can be logical values, arrays, or references.",
                                 ArgumentType.Logical,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Logical",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Xor_LogicalInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Logical, description, requiredArguments, optionalArguments, 254, false, descriptionLocalizationKey:descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<bool> context)
        {
            int numberOfTrueValues = 0;

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                if (context.Arguments[i])
                {
                    numberOfTrueValues++;
                }
            }

            return numberOfTrueValues % 2 > 0 ? BooleanExpression.True : BooleanExpression.False;
        }

        #endregion
    }
}