﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions.LookupAndReference
{
    /// <summary>
    /// Represents the CHOOSE function.
    /// </summary>
    public class Choose : FunctionWithArguments
    {
        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "CHOOSE";
        private static readonly FunctionInfo Info;

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        static Choose()
        {
            string description = "Uses the number parameter to return a value from the list of value arguments.";
            string descriptionKey = "Spreadsheet_Functions_Choose_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                 new ArgumentInfo("Index",
                     "Specifies which value argument is selected. Must be a number between 1 and 254, or a formula or reference to a cell containing a number between 1 and 254.",
                     ArgumentType.Number,
                     nameLocalizationKey:"Spreadsheet_Functions_Args_Index",
                     descriptionLocalizationKey:"Spreadsheet_Functions_Choose_IndexInfo"),

                 new ArgumentInfo("Value",
                     "1 to 254 value arguments from which CHOOSE selects a value or an action to perform based on index_num. The arguments can be numbers, cell references, defined names, formulas, functions, or text.",
                     ArgumentType.Any,
                     nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                     descriptionLocalizationKey:"Spreadsheet_Functions_Choose_ValueInfo"),
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
           {
                new ArgumentInfo("Value",
                    "Value 1 to 254 value arguments from which CHOOSE selects a value or an action to perform based on index_num. The arguments can be numbers, cell references, defined names, formulas, functions, or text.",
                    ArgumentType.Any,
                    nameLocalizationKey:"Spreadsheet_Functions_Args_Value",
                    descriptionLocalizationKey:"Spreadsheet_Functions_Choose_ValueInfo"),
           };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description,
                       requiredArguments, optionalArguments, optionalArgumentsRepeatCount: 254,
                       isDefaultValueFunction: false, descriptionLocalizationKey: descriptionKey);
        }

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context"></param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<object> context)
        {
            object[] arguments = context.Arguments;
            int returnIndex = System.Convert.ToInt32(arguments[0]);
            RadExpression result;
            try
            {
                result = (RadExpression) arguments[returnIndex];
            }
            catch (IndexOutOfRangeException)
            {
                return ErrorExpressions.ValueError;
            }

            return result;
        }
    }
}
