﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ABS function.
    /// </summary>
    public class Abs : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ABS";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get 
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Abs()
        {
            string description = "Returns the absolute value of a number. The absolute value of a number is the number without its sign.";
            string descriptionKey = "Spreadsheet_Functions_Abs_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                    "is the real number for which you want the absolute value.",
                    ArgumentType.Number,
                    nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                    descriptionLocalizationKey:"Spreadsheet_Functions_Abs_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods
        
        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double result = Math.Abs(context.Arguments[0]);
            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

        #endregion
    }
}