﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ACOSH function.
    /// </summary>
    public class Acosh : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ACOSH";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Acosh()
        {
            string description = "Returns the inverse hyperbolic cosine of a number. The number must be greater than or equal to 1. The inverse hyperbolic cosine is the value whose hyperbolic cosine is number, so ACOSH(COSH(number)) equals number.";
            string descriptionKey = "Spreadsheet_Functions_Acosh_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "is any real number equal to or greater than 1.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Acosh_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double argument = context.Arguments[0];

            if (argument < 1)
            {
                return ErrorExpressions.NumberError;
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(MathUtility.Acosh(argument));
        }

        #endregion
    }
}