﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the ASIN function.
    /// </summary>
	public class Asin : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "ASIN";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static Asin()
        {
            string description = "Returns the arcsine, or inverse sine, of a number. The arcsine is the angle whose sine is number. The returned angle is given in radians in the range -pi/2 to pi/2.";
            string descriptionKey = "Spreadsheet_Functions_Asin_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "is the sine of the angle you want and must be from -1 to 1.",
                                 ArgumentType.Number,
                                 nameLocalizationKey: "Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Asin_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double argument = context.Arguments[0];

            if (argument > 1 || argument < -1)
            {
                return ErrorExpressions.NumberError;
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(Math.Asin(argument));
        }

		#endregion
	}
}