﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the DEGREES function.
    /// </summary>
	public class Degrees : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "DEGREES";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static Degrees()
        {
            string description = "Converts radians into degrees.";
            string descriptionKey = "Spreadsheet_Functions_Degrees_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Angle",
                                 "is the angle in radians that you want to convert.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Angle",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Degrees_AngleInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double result = (context.Arguments[0] * 180) / Math.PI;
            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion
	}
}