﻿using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the FACT function.
    /// </summary>
    public class Fact : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "FACT";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Fact()
        {
            string description = "Returns the factorial of a number. The factorial of a number is equal to 1*2*3*...* number.";
            string descriptionKey = "Spreadsheet_Functions_Fact_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "is the nonnegative number you want the factorial of. If number is not an integer, it is truncated.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Fact_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            int number = (int)context.Arguments[0];

            if (number < 0 || number > 170)
            {
                return ErrorExpressions.NumberError;
            }

            double result = MathUtility.Factorial(number);
            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

        #endregion
    }
}