﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the LCM function.
    /// </summary>
    public class Lcm : NumbersInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "LCM";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <returns>The argument conversion rules as ArgumentConversionRules.</returns>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return ArgumentConversionRules.NonBoolNaryFunctionConversion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Lcm()
        {
            string description = "Returns the least common multiple of integers. The least common multiple is the smallest positive integer that is a multiple of all integer arguments number1, number2, and so on. Use LCM to add fractions with different denominators.";
            string descriptionKey = "Spreadsheet_Functions_Lcm_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "Number1, number2, ...     are 1 to 29 values for which you want the least common multiple. If value is not an integer, it is truncated.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Lcm_NumberInfo")
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "Number1, number2, ...     are 1 to 29 values for which you want the least common multiple. If value is not an integer, it is truncated.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Lcm_NumberInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, optionalArguments, 254, false, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            for (int i = 0; i < context.Arguments.Length; i++)
            {
                if (context.Arguments[i] < 0)
                {
                    return ErrorExpressions.NumberError;
                }
            }

            double result = MathUtility.LeastCommonMultiple(context.Arguments);

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

        #endregion
    }
}