﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the POWER function.
    /// </summary>
    public class Power : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "POWER";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static Power()
        {
            string description = "Returns the result of a number raised to a power.";
            string descriptionKey = "Spreadsheet_Functions_Power_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "is the base number, any real number.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Power_NumberInfo"),
                new ArgumentInfo("Power",
                                 "is the exponent, to which the base number is raised.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Power",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Power_PowerInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double number = context.Arguments[0];
            double power = context.Arguments[1];

            if (number == 0)
            {
                if (power == 0)
                {
                    return ErrorExpressions.NumberError;
                }
                else if (power < 0)
                {
                    return ErrorExpressions.DivisionByZero;
                }
            }

            double result = Math.Pow(number, power);
            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion
	}
}