﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the SUM function.
    /// </summary>
    public class Sum : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "SUM";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return ArgumentConversionRules.NaryIgnoreIndirectNumberFunctionConversion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static Sum()
        {
            string description = "Adds all the numbers in range of cells.";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "number1, number2,... are the numbers to sum. Logical values and text are ignored in cells, included if typed as arguments.",
                                 ArgumentType.Number,
                                 nameLocalizationKey: "Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey: "Spreadsheet_Functions_Sum_NumberInfo"),
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "number1, number2,... are the numbers to sum. Logical values and text are ignored in cells, included if typed as arguments.",
                                 ArgumentType.Number,
                                 nameLocalizationKey: "Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey: "Spreadsheet_Functions_Sum_NumberInfo"),
            };

            string descriptionKey = "Spreadsheet_Functions_Sum_Info";

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, optionalArguments, 254, true, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double result = 0;

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                result += context.Arguments[i];
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion
	}
}