﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the SUMSQ function.
    /// </summary>
    public class SumSq : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "SUMSQ";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return ArgumentConversionRules.NaryIgnoreIndirectNumberFunctionConversion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


		#region Constructors

        static SumSq()
        {
            string description = "Returns the sum of the squares of the arguments.";
            string descriptionKey = "Spreadsheet_Functions_SumSq_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "Number1, number2, ... are 1 to 30 arguments for which you want the sum of the squares. You can also use a single array or a reference to an array instead of arguments separated by commas.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_SumSq_NumberInfo"),
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "Number1, number2, ... are 1 to 30 arguments for which you want the sum of the squares. You can also use a single array or a reference to an array instead of arguments separated by commas.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_SumSq_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.MathTrig, description, requiredArguments, optionalArguments, 254, true, descriptionLocalizationKey: descriptionKey);
        }

		#endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            double result = 0;

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                result += Math.Pow(context.Arguments[i], 2);
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion
	}
}