﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the STDEVPA function.
    /// </summary>
    public class StdevPA : NumbersInFunction
	{
		#region Constants

		///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "STDEVPA";
        private static readonly FunctionInfo Info;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the argument conversion rules.
        /// </summary>
        /// <value>The argument conversion rules.</value>
        public override ArgumentConversionRules ArgumentConversionRules
        {
            get
            {
                return ArgumentConversionRules.DefaultValueNumberFunctionConversion;
            }
        }

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
			get
			{
				return FunctionName;
			}
		}

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

		#endregion


        #region Constructors

        static StdevPA()
        {
            string description = "Calculates standard deviation based on the entire population given as arguments, including text and logical values. The standard deviation is a measure of how widely values are dispersed from the average value (the mean).";
            string descriptionKey = "Spreadsheet_Functions_StdevPA_Info";
            
            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "number1, number2,... are 1 to 30 values corresponding to a population. You can also use a single array or a reference to an array instead of arguments separated by commas.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_StdevPA_NumberInfo"),
            };
            
            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Number",
                                 "number1, number2,... are 1 to 30 values corresponding to a population. You can also use a single array or a reference to an array instead of arguments separated by commas.",
                                 ArgumentType.Number,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Number",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_StdevPA_NumberInfo"),
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Statistical, description, requiredArguments, optionalArguments, 254, true, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


		#region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<double> context)
        {
            if (context.Arguments.Length < 2)
            {
                return ErrorExpressions.DivisionByZero;
            }

            double result = MathUtility.StandardDeviation(context.Arguments, true);

            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

		#endregion
	}
}