﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the CODE function.
    /// </summary>
    public class Code : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "CODE";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Code()
        {
            string description = "Returns a numeric code for the first character in a text string. The returned code corresponds to the character set used by your computer.";
            string descriptionKey = "Spreadsheet_Functions_Code_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Text",
                                 "is the text for which you want the code of the first character.",
                                 ArgumentType.Text,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Code_TextInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            string text = context.Arguments[0];

            if (text.Length == 0)
            {
                return ErrorExpressions.ValueError;
            }

            int result = text[0];
            return NumberExpression.CreateValidNumberOrErrorExpression(result);
        }

        #endregion
    }
}