﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the CONCATENATE function.
    /// </summary>
    public class Concatenate : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "CONCATENATE";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get 
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Concatenate()
        {
            string description = "Joins several text strings into one text string.";
            string descriptionKey = "Spreadsheet_Functions_Concatenate_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Text",
                                 "Text1, text2, ...     are 1 to 30 text items to be joined into a single text item. The text items can be text strings, numbers, or single-cell references.",
                                 ArgumentType.Text,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Concatenate_TextInfo")
            };

            IEnumerable<ArgumentInfo> optionalArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Text",
                                 "Text1, text2, ...     are 1 to 30 text items to be joined into a single text item. The text items can be text strings, numbers, or single-cell references.",
                                 ArgumentType.Text,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Concatenate_TextInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, requiredArguments, optionalArguments, 254, true, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < context.Arguments.Length; i++)
            {
                builder.Append(context.Arguments[i]);
            }

            return new StringExpression(builder.ToString());
        }

        #endregion
    }
}