﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the EXACT function.
    /// </summary>
    public class Exact : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "EXACT";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get 
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Exact()
        {
            string description = "Compares two text strings and returns TRUE if they are exactly the same, FALSE otherwise. EXACT is case-sensitive but ignores formatting differences. Use EXACT to test text being entered into a document.";
            string descriptionKey = "Spreadsheet_Functions_Exact_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                 new ArgumentInfo("Text1",
                                  "is the first text string.",
                                  ArgumentType.Text,
                                  nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                  descriptionLocalizationKey:"Spreadsheet_Functions_Exact_Text1Info"),
                 new ArgumentInfo("Text2",
                                  "is the second text string.",
                                  ArgumentType.Text,
                                  nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                  descriptionLocalizationKey:"Spreadsheet_Functions_Exact_Text2Info")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods
        
        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            string text1 = context.Arguments[0];
            string text2 = context.Arguments[1];

            bool result = false;

            if (text1.CompareTo(text2) == 0)
            {
                result = true;
            }

            return result.ToBooleanExpression();
        }

        #endregion
    }
}