﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the LEN function.
    /// </summary>
    public class Len : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "LEN";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Len()
        {
            string description = "Returns the number of characters in a text string.";
            string descriptionKey = "Spreadsheet_Functions_Len_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Text",
                                 "is the text whose length you want to find. Spaces count as characters.",
                                 ArgumentType.Text,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Len_TextInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            return NumberExpression.CreateValidNumberOrErrorExpression(context.Arguments[0].Length);
        }

        #endregion
    }
}