﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the TRIM function.
    /// </summary>
    public class Trim : StringsInFunction
    {

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "TRIM";
        private static readonly FunctionInfo Info;

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get
            {
                return Info;
            }
        }

        static Trim()
        {
            string description = "Removes all spaces from text except for single spaces between words.";
            string descriptionKey = "Spreadsheet_Functions_Trim_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                 new ArgumentInfo("Text", "is the text you want to trim.",
                     ArgumentType.Text,
                     nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                     descriptionLocalizationKey:"Spreadsheet_Functions_Trim_TextInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, 
                requiredArguments, isDefaultValueFunction: false, descriptionLocalizationKey: descriptionKey);
        }

        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context"></param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            object[] arguments = context.Arguments;
            string text = arguments[0].ToString();

            RegexOptions regexOptions = RegexOptions.None;
            string regexPattern = "[ ]{2,}";
            Regex regex = new Regex(regexPattern, regexOptions);

            char spaceAsChar = ' ';
            string result = regex.Replace(text, spaceAsChar.ToString()).Trim(spaceAsChar);

            return new StringExpression(result);
        }
    }
}