﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Functions
{
    /// <summary>
    /// Represents the UPPER function.
    /// </summary>
    public class Upper : StringsInFunction
    {
        #region Constants

        ///<summary>
        /// The name of the function.
        ///</summary>
        public static readonly string FunctionName = "UPPER";
        private static readonly FunctionInfo Info;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the function.
        /// </summary>
        /// <returns>The name as String.</returns>
        /// <value>The name.</value>
        public override string Name
        {
            get
            {
                return FunctionName;
            }
        }

        /// <summary>
        /// Gets the function info.
        /// </summary>
        /// <returns>The function info as FunctionInfo.</returns>
        /// <value>The function info.</value>
        public override FunctionInfo FunctionInfo
        {
            get 
            {
                return Info;
            }
        }

        #endregion


        #region Constructors

        static Upper()
        {
            string description = "Converts text to uppercase.";
            string descriptionKey = "Spreadsheet_Functions_Upper_Info";

            IEnumerable<ArgumentInfo> requiredArguments = new ArgumentInfo[]
            {
                new ArgumentInfo("Text",
                                 "is the text you want converted to uppercase. Text can be a reference or text string.",
                                 ArgumentType.Text,
                                 nameLocalizationKey:"Spreadsheet_Functions_Args_Text",
                                 descriptionLocalizationKey:"Spreadsheet_Functions_Upper_TextInfo")
            };

            Info = new FunctionInfo(FunctionName, FunctionCategory.Text, description, requiredArguments, descriptionLocalizationKey: descriptionKey);
        }

        #endregion


        #region Methods
        
        /// <summary>
        /// Virtual method evaluating the function with System.Object arguments array.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>Functions result as RadExpression.</returns>
        protected override RadExpression EvaluateOverride(FunctionEvaluationContext<string> context)
        {
            string text = context.Arguments[0];

            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < text.Length; i++)
            {
                char c = char.ToUpperInvariant(text[i]);
                builder.Append(c);
            }

            return new StringExpression(builder.ToString());
        }
        #endregion
    }
}