﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using GreaterThanExpression.
    /// </summary>
    public class GreaterThanExpression : ComparisonOperatorExpression
	{
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.GreaterThan;
            }
        }

        #endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="GreaterThanExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public GreaterThanExpression(RadExpression left, RadExpression right) : base(left, right)
		{
		}

		#endregion


		#region Methods
        
        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new GreaterThanExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Compares the string expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected override bool CompareStringExpressions(StringExpression left, StringExpression right)
        {
            return String.Compare(left.Value, right.Value, StringComparison.OrdinalIgnoreCase) > 0;
        }

        /// <summary>
        /// Compares the number expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected override bool CompareNumberExpressions(NumberExpression left, NumberExpression right)
        {
            return left.Value > right.Value;
        }

        /// <summary>
        /// Compares the boolean expressions.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>Boolean result.</returns>
        protected override bool CompareBooleanExpressions(BooleanExpression left, BooleanExpression right)
        {
            return left.Value && !right.Value;
        }

        /// <summary>
        /// Compares the different type expressions.
        /// </summary>
        /// <param name="leftOperandValue">The left operand value.</param>
        /// <param name="rightOperandValue">The right operand value.</param>
        /// <returns>Boolean result.</returns>
        protected override bool CompareDifferentTypeExpressions(ConstantExpression leftOperandValue, ConstantExpression rightOperandValue)
        {
            int leftOperandPrecedence = this.GetExpressionTypeNumber(leftOperandValue);
            int rightOperandPrecedence = this.GetExpressionTypeNumber(rightOperandValue);

            return leftOperandPrecedence > rightOperandPrecedence;
        }

		#endregion
	}
}
