﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Represents the input string collection class.
    /// </summary>
    internal class InputStringCollection : IEnumerable<UserInputStringBase>
    {
        #region Fields

        private readonly List<UserInputStringBase> innerList;

        #endregion


        #region Properties

        /// <summary>
        /// Gets a value indicating whether the cell value depends on a spreadsheet name.
        /// </summary>
        /// <value>The name of the contains spreadsheet.</value>
        public bool ContainsSpreadsheetName
        {
            get
            {
                return this.innerList.Any((item) => { return item is DefinedNameInputString; });
            }
        }

        /// <summary>
        /// Gets a value indicating whether the cell value depends on a sheet name.
        /// </summary>
        /// <value>The is sheet name dependent.</value>
        public bool IsSheetNameDependent
        {
            get
            {
                return this.innerList.Any((item) => { return item.IsSheetNameDependent; });
            }
        }

        /// <summary>
        /// Gets a value indicating whether the cell value depends on the workbook name.
        /// </summary>
        /// <value>The is workbook name dependent.</value>
        public bool IsWorkbookNameDependent
        {
            get
            {
                return this.innerList.Any((item) => { return item.IsWorkbookNameDependent; });
            }
        }

        /// <summary>
        /// Gets a value indicating whether the cell value contains cell references.
        /// </summary>
        /// <value>The is cells name dependent.</value>
        public bool IsCellsNameDependent
        {
            get
            {
                return this.innerList.Any((item) => { return item is CellReferenceInputString; });
            }
        }

        /// <summary>
        /// Gets the count.
        /// </summary>
        /// <value>The count.</value>
        public int Count
        {
            get
            {
                return this.innerList.Count;
            }
        }

        /// <summary>
        /// Gets the <see cref="Telerik.Windows.Documents.Spreadsheet.Expressions.UserInputStringBase" /> at the specified index.
        /// </summary>
        /// <value></value>
        public UserInputStringBase this[int index]
        {
            get
            {
                return this.innerList[index];
            }
        }

        #endregion


        #region Contructor

        /// <summary>
        /// Initializes a new instance of the <see cref="InputStringCollection" /> class.
        /// </summary>
        public InputStringCollection()
        {
            this.innerList = new List<UserInputStringBase>();
        }

        #endregion


        #region Methods

        public void Add(UserInputStringBase stringExpression)
        {
            this.innerList.Add(stringExpression);
        }

        public string BuildStringInCulture(SpreadsheetCultureHelper cultureInfo)
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < innerList.Count; i++)
            {
                builder.Append(innerList[i].ToString(cultureInfo));
            }

            return builder.ToString();
        }

        public string BuildString()
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < innerList.Count; i++)
            {
                builder.Append(innerList[i].ToString(FormatHelper.DefaultSpreadsheetCulture));
            }

            return builder.ToString();
        }

        public InputStringCollection Clone()
        {
            InputStringCollection newCollection = new InputStringCollection();

            for (int i = 0; i < this.innerList.Count; i++)
            {
                newCollection.Add(this.innerList[i].Clone());
            }

            return newCollection;
        }

        #endregion


        #region IEnumerable

        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable<UserInputStringBase>)this).GetEnumerator();
        }

        public IEnumerator<UserInputStringBase> GetEnumerator()
        {
            return this.innerList.GetEnumerator();
        }

        #endregion
    }
}
