﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using IntersectionExpression, that calculates the intersections of two CellReferenceRangeExpression instances.
    /// </summary>
    public class IntersectionExpression : CellReferenceBinaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Intersection;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="IntersectionExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public IntersectionExpression(RadExpression left, RadExpression right) 
            : base(left, right)
		{

		}

		#endregion


		#region Methods

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new IntersectionExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(CellReferenceRangeExpression[] operands)
        {
            CellReferenceRangeExpression newRangeExpression;

            if (CellReferenceRangeExpression.TryCreateIntersection(operands[0], operands[1], out newRangeExpression))
            {
                this.AttachToChildEvent(newRangeExpression);

                return newRangeExpression;
            }

            return ErrorExpressions.NullError;
        }

		#endregion
	}
}
