﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using NumberExpression.
    /// </summary>
    public class NumberExpression : ConstantExpression<double>
    {
        #region Constants

        /// <summary>
        /// The Zero NumberExpression.
        /// </summary>
        public static readonly NumberExpression Zero = new NumberExpression(0);

        /// <summary>
        /// The One NumberExpression.
        /// </summary>
        public static readonly NumberExpression One = new NumberExpression(1);

        /// <summary>
        /// The PI NumberExpression.
        /// </summary>
        public static readonly NumberExpression PI = new NumberExpression(Math.PI);

        /// <summary>
        /// The Napier's constant NumberExpression.
        /// </summary>
        public static readonly NumberExpression E = new NumberExpression(Math.E);

        #endregion


        #region Properties

        /// <summary>
        /// Gets the integer value.
        /// </summary>
        /// <value>The integer value.</value>
        /// <returns>The integer value.</returns>
        public int IntValue
        {
            get
            {
                return (int)this.Value;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="NumberExpression" /> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public NumberExpression(double value) : base(value)
        {
            Guard.ThrowExceptionIfInvalidDouble(value, "value");
        }

        #endregion


        #region Overrides

        internal override string GetValueAsString(SpreadsheetCultureHelper cultureInfo)
        {
            return this.ToString(cultureInfo);
        }

        internal override string ToString(SpreadsheetCultureHelper cultureInfo)
        {
            return cultureInfo.ToString(this.Value);
        }

        internal static RadExpression CreateValidNumberOrErrorExpression(double number)
        {
            if (double.IsInfinity(number))
            {
                return ErrorExpressions.NumberError;
            }

            if (double.IsNaN(number))
            {
                return ErrorExpressions.DivisionByZero;
            }

            return new NumberExpression(number);
        }

        #endregion
    }
}