﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Abstract class providing methods and properties for using OperatorExpression.
    /// </summary>
    public abstract class OperatorExpression : RadExpression
    {
        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <value>The operator info.</value>
        /// <returns>The operator info as OperatorInfo.</returns>
        public abstract OperatorInfo OperatorInfo { get; }

        internal virtual string GetOperandAsString(RadExpression expression, SpreadsheetCultureHelper cultureInfo)
        {
            Guard.ThrowExceptionIfNull(expression, "expression");
            Guard.ThrowExceptionIfNull(cultureInfo, "cultureInfo");

            string operandFormat = "{0}";
            OperatorExpression operatorExpression = expression as OperatorExpression;
            if (operatorExpression != null)
            {
                int operatorPrecedence = OperatorInfos.ComparePrecedence(this.OperatorInfo.Precedence, operatorExpression.OperatorInfo.Precedence);
                if (operatorPrecedence <= 0 && !(this.OperatorInfo.Symbol == operatorExpression.OperatorInfo.Symbol && this.IsOperatorAssociative()))
                {
                    operandFormat = "({0})";
                }
            }

            return string.Format(operandFormat, expression.ToString(cultureInfo));
        }

        private bool IsOperatorAssociative()
        {
            return (this.OperatorInfo.Symbol != SpreadsheetCultureHelper.Minus || this.OperatorInfo.Associativity == OperatorAssociativity.Right) &&
                this.OperatorInfo.Symbol != SpreadsheetCultureHelper.Power &&
                this.OperatorInfo.Symbol != SpreadsheetCultureHelper.Division;
        }
	}
}