﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Parser
{
    /// <summary>
    /// The class provides context for the ExpressionLexer class. It stores information about the tokens already sent 
    /// to the ExpressionParser and allows the lexer to resolve the ambiguity between the list separator and the union operator.
    /// </summary>
    internal class ExpressionLexerContext
    {
        private readonly Stack<bool> functionStack;

        public ExpressionLexerContext()
        {
            this.functionStack = new Stack<bool>();
        }

        public void RegisterFunction()
        {
            this.functionStack.Push(true);
        }

        public void RegisterLeftParenthesis()
        {
            this.functionStack.Push(false);
        }

        public void RegisterRightParenthesis()
        {
            if (this.functionStack.Count > 0)
            {
                this.functionStack.Pop();
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current list separator should be treated as a union operator.
        /// </summary>
        /// <value>The is union operator.</value>
        public bool IsUnionOperator
        {
            get
            {
                return this.functionStack.Count == 0 || !this.functionStack.Peek();
            }
        }
    }
}
