﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions.Parser
{
	internal class ExpressionToken
	{
		#region Constants

		public static readonly ExpressionToken UnaryPlus = new ExpressionToken(ExpressionTokenType.UnaryPlus, SpreadsheetCultureHelper.UnaryPlus);
        public static readonly ExpressionToken UnaryMinus = new ExpressionToken(ExpressionTokenType.UnaryMinus, SpreadsheetCultureHelper.UnaryMinus);
        public static readonly ExpressionToken Percent = new ExpressionToken(ExpressionTokenType.Percent, SpreadsheetCultureHelper.Percent);

        public static readonly ExpressionToken Plus = new ExpressionToken(ExpressionTokenType.Plus, SpreadsheetCultureHelper.Plus);
        public static readonly ExpressionToken Minus = new ExpressionToken(ExpressionTokenType.Minus, SpreadsheetCultureHelper.Minus);
        public static readonly ExpressionToken Multiply = new ExpressionToken(ExpressionTokenType.Multiply, SpreadsheetCultureHelper.Multiplication);
        public static readonly ExpressionToken Divide = new ExpressionToken(ExpressionTokenType.Divide, SpreadsheetCultureHelper.Division);
        public static readonly ExpressionToken Power = new ExpressionToken(ExpressionTokenType.Power, SpreadsheetCultureHelper.Power);

        public static readonly ExpressionToken Equal = new ExpressionToken(ExpressionTokenType.Equal, SpreadsheetCultureHelper.Equal);
        public static readonly ExpressionToken LessThan = new ExpressionToken(ExpressionTokenType.LessThan, SpreadsheetCultureHelper.LessThan);
        public static readonly ExpressionToken LessThanOrEqualTo = new ExpressionToken(ExpressionTokenType.LessThanOrEqualTo, SpreadsheetCultureHelper.LessThanOrEqualTo);
        public static readonly ExpressionToken GreaterThan = new ExpressionToken(ExpressionTokenType.GreaterThan, SpreadsheetCultureHelper.GreaterThan);
        public static readonly ExpressionToken GreaterThanOrEqualTo = new ExpressionToken(ExpressionTokenType.GreaterThanOrEqualTo, SpreadsheetCultureHelper.GreaterThanOrEqualTo);
        public static readonly ExpressionToken NotEqual = new ExpressionToken(ExpressionTokenType.NotEqual, SpreadsheetCultureHelper.NotEqual);

        public static readonly ExpressionToken Ampersand = new ExpressionToken(ExpressionTokenType.Ampersand, SpreadsheetCultureHelper.Ampersand);
        public static readonly ExpressionToken Range = new ExpressionToken(ExpressionTokenType.Range, SpreadsheetCultureHelper.Colon);

        public static readonly ExpressionToken LeftParenthesis = new ExpressionToken(ExpressionTokenType.LeftParenthesis, SpreadsheetCultureHelper.LeftParenthesis);
        public static readonly ExpressionToken RightParenthesis = new ExpressionToken(ExpressionTokenType.RightParenthesis, SpreadsheetCultureHelper.RightParenthesis);

        public static readonly ExpressionToken True = new ExpressionToken(ExpressionTokenType.True, SpreadsheetCultureHelper.TrueLiteral);
        public static readonly ExpressionToken False = new ExpressionToken(ExpressionTokenType.False, SpreadsheetCultureHelper.FalseLiteral);

        internal static readonly ExpressionToken MissingValue = new ExpressionToken(ExpressionTokenType.MissingValue, string.Empty);
		internal static readonly ExpressionToken FunctionStart = new ExpressionToken(ExpressionTokenType.FunctionStart, "f");

        public static readonly ExpressionToken NullError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.NullErrorValue);
        public static readonly ExpressionToken DivisionByZeroError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.DivisionByZeroErrorValue);
        public static readonly ExpressionToken ValueError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.ValueErrorValue);
        public static readonly ExpressionToken ReferenceError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.ReferenceErrorValue);
        public static readonly ExpressionToken NameError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.NameErrorValue);
        public static readonly ExpressionToken NumberError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.NumberErrorValue);
        public static readonly ExpressionToken NotAvailableError = new ExpressionToken(ExpressionTokenType.Error, SpreadsheetCultureHelper.NotAvailableErrorValue);

		#endregion


		#region Fields

		private readonly ExpressionTokenType tokenType;
		private readonly string value;
		private readonly double? numberValue;

		#endregion


		#region Properties

		public string Value
		{
			get
			{
				return this.value;
			}
		}

		public ExpressionTokenType TokenType
		{
			get
			{
				return this.tokenType;
			}
		}

		public double NumberValue
		{
			get
			{
				return this.numberValue.Value;
			}
		}
   
		#endregion


		#region Constructors

		public ExpressionToken(ExpressionTokenType type, string value)
		{
            Guard.ThrowExceptionIfNull(value, "value");

			this.tokenType = type;
			this.value = value;
		}

		public ExpressionToken(double value, string valueAsString)
            : this(ExpressionTokenType.Number, valueAsString)
		{
			this.numberValue = value;
		}

		#endregion


		#region Methods

		public override string ToString()
		{
			return this.Value;
		}

		#endregion
	}
}