﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using PercentExpression.
    /// </summary>
    public class PercentExpression : UnaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Percent;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PercentExpression" /> class.
        /// </summary>
        /// <param name="operand">The operand as RadExpression.</param>
        public PercentExpression(RadExpression operand) : base(operand)
		{
		}

		#endregion


		#region Methods

        /// <summary>
        /// Gets the value of the expression.
        /// </summary>
        /// <param name="operand">The operand as double.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(double operand)
        {
            return NumberExpression.CreateValidNumberOrErrorExpression(operand / 100);
        }

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
             return new PercentExpression(this.Operand.CloneAndTranslate(context));
        }

        internal override string ToString(SpreadsheetCultureHelper cultureInfo)
        {
            return this.GetOperandAsString(this.Operand, cultureInfo) + this.OperatorInfo.Symbol;
        }

		#endregion
	}
}
