﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using PowerExpression.
    /// </summary>
	public class PowerExpression : NumberBinaryOperatorExpression
	{
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Power;
            }
        }

        #endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PowerExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public PowerExpression(RadExpression left, RadExpression right) : base(left, right)
		{
		}

		#endregion


		#region Methods

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new PowerExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(double[] operands)
        {
            double number = operands[0];
            double power = operands[1];

            if (number == 0)
            {
                if (power == 0)
                {
                    return ErrorExpressions.NumberError;
                }
                else if (power < 0)
                {
                    return ErrorExpressions.DivisionByZero;
                }
            }
            else if (number < 0)
            {                         
                if (MathUtility.IsOddEgyptianFraction(power))
                {
                    return NumberExpression.CreateValidNumberOrErrorExpression(-Math.Pow(-number, power));
                }                
                else if ((int)power != power)
                {
                    return ErrorExpressions.NumberError;
                }
            }

            return NumberExpression.CreateValidNumberOrErrorExpression(Math.Pow(number, power));
		}

		#endregion
	}
}