﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using RangeExpression.
    /// </summary>
    public class RangeExpression : CellReferenceBinaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Range;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RangeExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public RangeExpression(RadExpression left, RadExpression right) 
            : base(left, right)
		{

		}

		#endregion


		#region Methods

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new RangeExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(CellReferenceRangeExpression[] operands)
        {
            CellReferenceRangeExpression newRangeExpression;

            if (CellReferenceRangeExpression.TryCreateRange(operands[0], operands[1], out newRangeExpression))
            {
                this.AttachToChildEvent(newRangeExpression);
                return newRangeExpression;
            }

            if (operands[0].CellReferenceRange == null || !operands[0].CellReferenceRange.IsInRange ||
                operands[1].CellReferenceRange == null || !operands[1].CellReferenceRange.IsInRange)
            {
                return ErrorExpressions.ReferenceError;
            }

            return ErrorExpressions.ValueError;
        }

		#endregion
    }
}
