﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Provides methods and properties for using UnaryPlusExpression.
    /// </summary>
    public class UnaryPlusExpression : UnaryOperatorExpression
    {
        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.UnaryPlus;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="UnaryPlusExpression" /> class.
        /// </summary>
        /// <param name="operand">The operand.</param>
        public UnaryPlusExpression(RadExpression operand) : base(operand)
		{
		}

		#endregion


		#region Methods

        /// <summary>
        /// Gets the value of the expression.
        /// </summary>
        /// <param name="operand">The operand as double.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(double operand)
        {
            return NumberExpression.CreateValidNumberOrErrorExpression(operand);
        }

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new UnaryPlusExpression(this.Operand.CloneAndTranslate(context));
        }

		#endregion
	}
}