﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Expressions
{
    /// <summary>
    /// Represents the union expression.
    /// </summary>
    public class UnionExpression : CellReferenceBinaryOperatorExpression
    {
        #region Fields

        private string toStringValueCache;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the operator info.
        /// </summary>
        /// <returns>The operator info as OperatorInfo.</returns>
        /// <value>The operator info.</value>
        public override OperatorInfo OperatorInfo
        {
            get
            {
                return OperatorInfos.Union;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="UnionExpression" /> class.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        public UnionExpression(RadExpression left, RadExpression right)
            : base(left, right)
        {

        }

        #endregion

        
        #region Methods

        internal override RadExpression CloneAndTranslate(ExpressionCloneAndTranslateContext context)
        {
            return new UnionExpression(this.Left.CloneAndTranslate(context), this.Right.CloneAndTranslate(context));
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="operands">The operands of type T.</param>
        /// <returns>Value as RadExpression.</returns>
        protected override RadExpression GetValueOverride(CellReferenceRangeExpression[] operands)
        {
            if (!operands[0].CellReferenceRange.IsInRange || !operands[1].CellReferenceRange.IsInRange)
            {
                return ErrorExpressions.ReferenceError;
            }

            CellReferenceRangeExpression newRangeExpression;

            if (CellReferenceRangeExpression.TryCreateUnion(operands[0], operands[1], out newRangeExpression))
            { 
                this.AttachToChildEvent(newRangeExpression);
                return newRangeExpression;
            }

            return ErrorExpressions.ValueError;
        }

        internal override string ToString(SpreadsheetCultureHelper cultureInfo)
        {
            if (this.Left.ToString(cultureInfo) == SpreadsheetCultureHelper.ReferenceErrorValue ||
                this.Right.ToString(cultureInfo) == SpreadsheetCultureHelper.ReferenceErrorValue)
            {
                return SpreadsheetCultureHelper.ReferenceErrorValue;
            }

            if (this.toStringValueCache == null)
            {
                StringBuilder sb = new StringBuilder();

                sb.Append(base.GetOperandAsString(this.Left, cultureInfo));
                sb.Append(cultureInfo.ListSeparator);
                sb.Append(base.GetOperandAsString(this.Right, cultureInfo));

                this.toStringValueCache = sb.ToString();
            }

            return this.toStringValueCache;
        }

        #endregion
    }
}
