﻿using System.IO;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders
{
    /// <summary>
    /// Provides the base class from which the classes that represent binary workbook format providers are derived. 
    /// </summary>
    public abstract class BinaryWorkbookFormatProviderBase : WorkbookFormatProviderBase, IBinaryWorkbookFormatProvider
    {
        #region Methods

        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The workbook.</returns>
        public Workbook Import(byte[] input)
        {
            using (MemoryStream inputStream = new MemoryStream(input))
            {
                return this.Import(inputStream);
            }
        }

        /// <summary>
        /// Exports the specified workbook.
        /// </summary>
        /// <param name="workbook">The workbook.</param>
        /// <returns>The result byte array.</returns>
        public byte[] Export(Workbook workbook)
        {
            using (MemoryStream outputStream = new MemoryStream())
            {
                this.Export(workbook, outputStream);
                return outputStream.ToArray();
            }
        }

        #endregion
    }
}