﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders
{
    /// <summary>
    /// Provides properties describing what should be exported from the document.
    /// </summary>
    public class ExportWhatSettings
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ExportWhatSettings" /> class.
        /// </summary>
        internal ExportWhatSettings() 
            : this(ExportWhat.ActiveSheet, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExportWhatSettings" /> class.
        /// </summary>
        /// <param name="exportWhat">The ExportWhat option.</param>
        /// <param name="ignorePrintArea">Value indicating whether to ignore print area.</param>
        public ExportWhatSettings(ExportWhat exportWhat, bool ignorePrintArea)
        {
            this.ExportWhat = exportWhat;
            this.IgnorePrintArea = ignorePrintArea;
        }

        /// <summary>
        /// Gets what should be exported.
        /// </summary>
        /// <value>The ExportWhat option value.</value>
        public ExportWhat ExportWhat { get; private set; }

        /// <summary>
        /// Gets whether to ignore print area.
        /// </summary>
        /// <value>Value indicating whether the print area should be ignored.</value>
        public bool IgnorePrintArea { get; private set; }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// Returns true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            ExportWhatSettings other = obj as ExportWhatSettings;
            if (other == null)
            {
                return false;
            }

            return this.ExportWhat == other.ExportWhat && this.IgnorePrintArea == other.IgnorePrintArea;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return this.ExportWhat.GetHashCode() ^ this.IgnorePrintArea.GetHashCode();
        }
    }
}
