﻿using System.Collections.Generic;
using System.IO;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders
{
    /// <summary>
    /// Represents workbook format provider.
    /// </summary>
    public interface IWorkbookFormatProvider
    {
        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name of the provider.</value>
        string Name { get; }

        /// <summary>
        /// Gets the files description.
        /// </summary>
        /// <value>The files description.</value>
        string FilesDescription { get; }

        /// <summary>
        /// Gets the supported extensions.
        /// </summary>
        /// <value>The supported extensions.</value>
        IEnumerable<string> SupportedExtensions { get; }

        /// <summary>
        /// Gets a value indicating whether can import.
        /// </summary>
        /// <value>The value indicating whether can import.</value>
        bool CanImport { get; }

        /// <summary>
        /// Gets a value indicating whether can export.
        /// </summary>
        /// <value>The value indicating whether can export.</value>
        bool CanExport { get; }

        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The imported workbook.</returns>
        Workbook Import(Stream input);

        /// <summary>
        /// Exports the specified workbook.
        /// </summary>
        /// <param name="workbook">The workbook.</param>
        /// <param name="output">The output.</param>
        void Export(Workbook workbook, Stream output);
    }
}