﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Core
{
    internal class CsvTokenizer
    {
        #region Fields

        private readonly List<Func<CsvToken>> tokenReaders;

        private readonly TextReader textReader;
        private readonly CsvSettings settings;

        #endregion


        #region Properties

        public bool IsAtEndOfFile
        {
            get
            {
                return this.textReader.Peek() == -1;
            }
        }

        #endregion


        #region Constructors
        
        public CsvTokenizer(TextReader textReader, CsvSettings settings)
        {
            Guard.ThrowExceptionIfNull(textReader, "textReader");
           
            this.textReader = textReader;
            this.settings = settings ?? new CsvSettings();

            this.tokenReaders = new List<Func<CsvToken>>();

            this.InitTokenHadlers();
        }

        #endregion


        #region Methods

        private void InitTokenHadlers()
        {
            this.tokenReaders.Add(() => { return this.ReadSingleCharToken(CsvTokenType.Delimiter, settings.Delimiter); });
            this.tokenReaders.Add(() => { return this.ReadSingleCharToken(CsvTokenType.Quote, settings.Quote); });
            this.tokenReaders.Add(() => { return this.ReadEndOfLineToken(); });
            this.tokenReaders.Add(() => { return this.ReadSingleCharToken(CsvTokenType.Symbol); });
        }

        public CsvToken Read()
        {
            CsvToken result = null;
            foreach (var tokenReader in this.tokenReaders)
            {
                result = tokenReader();
                if (result != null)
                {
                    break;
                }
            }

            return result;
        }

        private CsvToken ReadSingleCharToken(CsvTokenType csvTokenType, char? tokenChar = null)
        {
            int nextCharIndex = this.textReader.Peek();
            if (nextCharIndex != -1)
            {
                char nextChar = (char)nextCharIndex;
                if (!tokenChar.HasValue || nextChar == tokenChar)
                {
                    this.textReader.Read();
                    return new CsvToken(csvTokenType, nextChar.ToString());
                }
            }

            return null;
        }

        private CsvToken ReadEndOfLineToken()
        {
            char nextChar = (char)this.textReader.Peek();

            if (nextChar == '\r' || nextChar == '\n')
            {
                StringBuilder builder = new StringBuilder();
                builder.Append(nextChar);
                this.textReader.Read();
                nextChar = (char)this.textReader.Peek();

                if ((nextChar == '\r' && builder[0] == '\n') ||
                    (nextChar == '\n' && builder[0] == '\r'))
                {
                    builder.Append(nextChar);
                    this.textReader.Read();
                }

                return new CsvToken(CsvTokenType.EndOfLine, builder.ToString());
            }

            return null;
        }

        #endregion
    }
}