﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Core;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Csv
{
    /// <summary>
    /// Represents CSV format provider.
    /// </summary>
    public class CsvFormatProvider : DelimitedValuesFormatProviderBase
    {
        #region Constants

        private static readonly string NAME = "CsvFormatProvider";
        private static readonly string FILE_DESCRIPTION = "CSV (Comma delimited)";
        private static readonly IEnumerable<string> SUPPORTED_EXTENSIONS = new string[] { ".csv" };
        
        #endregion


        #region Fields

        private readonly CsvSettings csvSettings;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name of the provider.</value>
        public override string Name
        {
            get
            {
                return NAME;
            }
        }

        /// <summary>
        /// Gets the files description.
        /// </summary>
        /// <value>The files description.</value>
        public override string FilesDescription
        {
            get
            {
                return FILE_DESCRIPTION;
            }
        }

        /// <summary>
        /// Gets the supported extensions.
        /// </summary>
        /// <value>The supported extensions.</value>
        public override IEnumerable<string> SupportedExtensions
        {
            get
            {
                return SUPPORTED_EXTENSIONS;
            }
        }

        /// <summary>
        /// Gets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public override CsvSettings Settings
        {
            get 
            {
                return this.csvSettings;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CsvFormatProvider" /> class.
        /// </summary>
        public CsvFormatProvider()
        {
            this.csvSettings = new CsvSettings();
        }

        #endregion
    }
}