﻿using System.IO;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders
{
    /// <summary>
    /// Provides the base class from which the classes that represent text based workbook format providers are derived. 
    /// </summary>
    public abstract class TextBasedWorkbookFormatProviderBase : WorkbookFormatProviderBase, ITextBasedWorkbookFormatProvider
    {
        #region Methods

        /// <summary>
        /// Imports the specified input.
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>The result workbook.</returns>
        public Workbook Import(string input)
        {
            using (MemoryStream inputStream = new MemoryStream())
            {
                using (StreamWriter writer = new StreamWriter(inputStream))
                {
                    writer.Write(input);
                    writer.Flush();
                    inputStream.Position = 0;

                    return this.Import(inputStream);
                }
            }
        }

        /// <summary>
        /// Exports the specified workbook.
        /// </summary>
        /// <param name="workbook">The workbook.</param>
        /// <returns>The result string.</returns>
        public string Export(Workbook workbook)
        {
            using (MemoryStream outputStream = new MemoryStream())
            {
                this.Export(workbook, outputStream);
                outputStream.Seek(0, SeekOrigin.Begin);
                
                using (StreamReader reader = new StreamReader(outputStream))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        #endregion
    }
}