﻿using System;
using System.Globalization;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting
{
    internal static class CurrencyParser
    {
        #region Constants

        private static readonly string DefaultCurrencyFormat = "#{0}##0";
        private static readonly string DefaultCurrencyDecimalFormat = "#{0}##0{1}00";

        private static readonly string[] NegativeToPositivePatternStrings = 
                               {
                                 "{0}_)", "{0}", "{0}", "{0}_-", "{0}_)", "{0}", "{0}", "{0}_-", 
                                 "{0}", "{0}", "{0}_-", "{0}_-", "{0}", "{0}", "{0}_)", "{0}_)" 
                               };

        #endregion


        #region Methods

        public static bool TryParse(string value, out double result, out string format)
        {
            format = FormatHelper.GeneralFormatString;

            //CultureInfo currentCultureInfo = FormatHelper.GetCurrentCulture();
            //NumberStyles styles = NumberStyles.Number | NumberStyles.AllowCurrencySymbol;
            //int index = value.IndexOf(currentCultureInfo.NumberFormat.CurrencySymbol);

            //bool canParse = (index != -1 && double.TryParse(value, styles, currentCultureInfo, out result));

            //canParse &= FormatHelper.ApplyAdditionalNumberConstraints(value);

            bool canParse = FormatHelper.DefaultSpreadsheetCulture.TryParseCurrency(value, out result);

            if (canParse)
            {
                format = GetCurrencyFormat(value);
            }

            return canParse;
        }

        private static string GetCurrencyFormat(string value)
        {
            CultureInfo currentCulture = FormatHelper.DefaultSpreadsheetCulture.CultureInfo;

            string stringVal = value.Replace(currentCulture.NumberFormat.CurrencySymbol, "");
            string format;

            format = string.Format(DefaultCurrencyFormat, currentCulture.NumberFormat.CurrencyGroupSeparator);
            if (stringVal.IndexOf(currentCulture.NumberFormat.NumberDecimalSeparator) != -1)
            {
                format = string.Format(DefaultCurrencyDecimalFormat, currentCulture.NumberFormat.CurrencyGroupSeparator,
                    currentCulture.NumberFormat.CurrencyDecimalSeparator);
            }

            int positivePatternIndex = currentCulture.NumberFormat.CurrencyPositivePattern;
            int negativePatternIndex = currentCulture.NumberFormat.CurrencyNegativePattern;

            string positive = FormatHelper.PositivePatternStrings[positivePatternIndex];
            string negative = FormatHelper.NegativePatternStrings[negativePatternIndex];

            string negativePattern = "[Red]" + negative.Replace("$", GetLiteralCurrencySymbol())
                .Replace("-", currentCulture.NumberFormat.NegativeSign).Replace("n", "{0}");

            positive = string.Format(NegativeToPositivePatternStrings[negativePatternIndex],
                positive);
            string positivePattern = positive.Replace("$", GetLiteralCurrencySymbol())
                .Replace("-", currentCulture.NumberFormat.NegativeSign).Replace("n", "{0}");

            string result = string.Format(positivePattern + ";" + negativePattern, format);

            return result;
        }

        private static string GetLiteralCurrencySymbol()
        {
            string currencySymbol = FormatHelper.DefaultSpreadsheetCulture.CultureInfo.NumberFormat.CurrencySymbol;

            return string.Format("\"{0}\"", currencySymbol);

        }

        #endregion
    }
}