﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Builders
{
    internal static class BuildersHelper
    {
        private const string InvariantCurrencySymbol = "$";
        private const string InvariantNumberDecimalSeparator = ".";
        private const string InvariantNumberGroupSeparator = ",";

        public static string GetCultureDependentFormatCode(string invariantCultureFormatCode)
        {
            string currencySymbol = FormatHelper.DefaultSpreadsheetCulture.CultureInfo.NumberFormat.CurrencySymbol;
            string numberDecimalSeparator = FormatHelper.DefaultSpreadsheetCulture.NumberDecimalSeparator;
            string numberGroupSeparator = FormatHelper.DefaultSpreadsheetCulture.NumberGroupSeparator;

            string currentCultureFormatCode = invariantCultureFormatCode
                .Replace(InvariantNumberDecimalSeparator, numberDecimalSeparator)
                .Replace(InvariantNumberGroupSeparator, numberGroupSeparator)
                .Replace(InvariantCurrencySymbol, currencySymbol);

            return currentCultureFormatCode;
        }

        public static List<string> GetCurrencyStringsForCurrentCulture(CultureInfo cultureInfo, string currencyCode, int decimalPlaces, 
            bool useThousandSeparator = true)
        {
            List<string> patterns = new List<string>();

            string valuePattern = FormatHelper.GetValuePatternConsiderDecimalPlaces(decimalPlaces, useThousandSeparator);
            int currencyNegativePatternIndex = cultureInfo.NumberFormat.CurrencyNegativePattern;
            string positiveSystemPattern = FormatHelper.PositivePatternStrings[cultureInfo.NumberFormat.CurrencyPositivePattern];
            string negativeSystemPattern = FormatHelper.NegativePatternStrings[currencyNegativePatternIndex];

            string positivePattern = positiveSystemPattern.Replace("n", valuePattern);
            positivePattern = positivePattern.Replace("$", currencyCode);
            positivePattern = positivePattern.Trim();

            string negativePattern = negativeSystemPattern.Replace("n", valuePattern);
            negativePattern = negativePattern.Replace("$", currencyCode);
            negativePattern = negativePattern.Trim();

            patterns.Add(string.Format("{0}{1}", positivePattern, 
                BuildersHelper.GetAlignmentStringForPattern(currencyNegativePatternIndex, true)));
            patterns.Add(string.Format("{0};[Red]{0}", positivePattern));
            patterns.Add(string.Format("{0}{1};{2}", positivePattern, 
                BuildersHelper.GetAlignmentStringForPattern(currencyNegativePatternIndex, false), 
                negativePattern));
            patterns.Add(string.Format("{0}{1};[Red]{2}", positivePattern, 
                BuildersHelper.GetAlignmentStringForPattern(currencyNegativePatternIndex, false), 
                negativePattern));

            return patterns;
        }

        public static string MakeCurrencySymbolInvisible(string currencySymbol)
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < currencySymbol.Length; i++)
            {
                sb.Append("_");
                sb.Append(currencySymbol[i]);
            }

            return sb.ToString();
        }

        private static string GetAlignmentStringForPattern(int patternIndex, bool isFirstNegativePattern)
        {
            switch (patternIndex)
            {
                case 0:
                case 4:
                case 14:
                case 15:
                    if (isFirstNegativePattern)
                    {
                        return string.Empty;
                    }
                    return "_)";
                case 3:
                case 7:
                case 10:
                case 11:
                    return "_-";
                default:
                    return string.Empty;
            }
        }
    }
}
