﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Builders
{
    /// <summary>
    /// Represents the currency format string builder class.
    /// </summary>
    public static class CurrencyFormatStringBuilder
    {
        #region Fields

        private static readonly List<Func<CurrencyInfo, int, List<string>>> formatStringHandlers;

        #endregion


        #region Constructors

        static CurrencyFormatStringBuilder()
        {
            formatStringHandlers = new List<Func<CurrencyInfo, int, List<string>>>();
            formatStringHandlers.Add(DefaultCurrencyFormatStringsHandler);
            formatStringHandlers.Add(CurrencyWithCultureFormatStringsHandler);
        }

        #endregion


        #region Methods

        /// <summary>
        /// Currencies the with culture format strings handler.
        /// </summary>
        /// <param name="currencyInfo">The currency info.</param>
        /// <param name="decimalPlaces">The decimal places.</param>
        /// <returns></returns>
        public static List<string> CurrencyWithCultureFormatStringsHandler(CurrencyInfo currencyInfo, int decimalPlaces)
        {
            if (currencyInfo.Culture == null)
            {
                return null;
            }

            string currencyCode = CurrencyFormatCategoryManager.CultureInfoToCurrencyCode[currencyInfo.Culture];

            return BuildersHelper.GetCurrencyStringsForCurrentCulture(currencyInfo.Culture, currencyCode, decimalPlaces);
        }

        /// <summary>
        /// Builds the format strings.
        /// </summary>
        /// <param name="currencyInfo">The currency info.</param>
        /// <param name="decimalPlaces">The decimal places.</param>
        /// <returns></returns>
        public static List<string> BuildFormatStrings(CurrencyInfo currencyInfo, int decimalPlaces)
        {
            List<string> result = null;
            foreach (var handler in formatStringHandlers)
            {
                result = handler(currencyInfo, decimalPlaces);
                if (result != null)
                {
                    break;
                }
            }

            return result;
        }

        private static List<string> DefaultCurrencyFormatStringsHandler(CurrencyInfo currencyInfo, int decimalPlaces)
        {
            if (TelerikHelper.EqualsOfT(currencyInfo, CurrencyInfo.None))
            {
                string currencySymbol = FormatHelper.DefaultSpreadsheetCulture.CultureInfo.NumberFormat.CurrencySymbol;

                return BuildersHelper.GetCurrencyStringsForCurrentCulture(FormatHelper.DefaultSpreadsheetCulture.CultureInfo,
                    BuildersHelper.MakeCurrencySymbolInvisible(currencySymbol), decimalPlaces);
            }
            
            if (TelerikHelper.EqualsOfT(currencyInfo, CurrencyInfo.Symbol))
            {
                return BuildersHelper.GetCurrencyStringsForCurrentCulture(FormatHelper.DefaultSpreadsheetCulture.CultureInfo,
                    currencyInfo.EscapedCurrencySymbol, decimalPlaces);
            }

            return null;
        }

        #endregion
    }
}