﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers
{
    /// <summary>
    /// Represents the fraction format category manager class.
    /// </summary>
    public static class FractionFormatCategoryManager
    {
        #region Fields

        private static readonly Dictionary<FractionDenominatorType, string> fractionTypeToFormatString;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the fraction type to format string.
        /// </summary>
        /// <value>The fraction type to format string.</value>
        public static Dictionary<FractionDenominatorType, string> FractionTypeToFormatString
        {
            get
            {
                return fractionTypeToFormatString;
            }
        }

        #endregion


        #region Constructors

        static FractionFormatCategoryManager()
        {
            fractionTypeToFormatString = new Dictionary<FractionDenominatorType, string>();
            InitFractionDenominatorTypes();
        }

        #endregion


        #region Methods

        private static void InitFractionDenominatorTypes()
        {
            AddFractionType(FractionDenominatorType.UpToOneDigit, "# ?/?");
            AddFractionType(FractionDenominatorType.UpToTwoDigits, "# ??/??");
            AddFractionType(FractionDenominatorType.UpToThreeDigits, "# ???/???");
            AddFractionType(FractionDenominatorType.AsHalves, "# ?/2");
            AddFractionType(FractionDenominatorType.AsQuarters, "# ?/4");
            AddFractionType(FractionDenominatorType.AsEighths, "# ?/8");
            AddFractionType(FractionDenominatorType.AsSixteenths, "# ??/16");
            AddFractionType(FractionDenominatorType.AsTenths, "# ?/10");
            AddFractionType(FractionDenominatorType.AsHundredths, "# ??/100");
        }

        private static void AddFractionType(FractionDenominatorType fractionType, string formatString)
        {
            fractionTypeToFormatString.Add(fractionType, formatString);
        }

        #endregion
    }
}
