﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings
{
    internal class FormatDescriptor
    {
        #region Fields

        private readonly List<FormatDescriptorItem> items;

        #endregion


        #region Properties

        public CultureInfo Culture
        {
            get;
            internal set;
        }

        public Color? Foreground
        {
            get;
            internal set;
        }

        public Predicate<double> Condition
        {
            get;
            internal set;
        }

        internal int ItemsCount
        {
            get
            {
                return this.items.Count;
            }
        }

        public IEnumerable<FormatDescriptorItem> Items
        {
            get
            {
                return this.items.AsEnumerable();
            }
        }

        internal string Text
        {
            get
            {
                StringBuilder sb = new StringBuilder();
                foreach (FormatDescriptorItem item in this.Items)
                {
                    sb.Append(item.Text);
                }

                return sb.ToString();
            }
        }

        internal bool ShouldExpand
        {
            get
            {
                return this.Items.Any(fspi => fspi.ShouldExpand);
            }
        }

        #endregion


        #region Constructors

        public FormatDescriptor()
        {
            this.Condition = FormatHelper.DefaultFormatCondition;
            this.Foreground = FormatHelper.DefaultFormatForeground;
            this.Culture = FormatHelper.DefaultFormatCulture;
            this.items = new List<FormatDescriptorItem>();
        }

        #endregion


        #region Methods

        internal void AddItem(FormatDescriptorItem item)
        {
            this.items.Add(item);
        }

        #endregion
    }
}