﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings
{
    internal class FormatDescriptorItem
    {
        #region Fields

        private string text;
        private readonly bool isTransparent;
        private readonly bool shouldExpand;
        private readonly bool applyFormat;
        private readonly Func<double?, string> applyFormatToDoubleValue;
        private readonly Func<DateTime?, string> applyFormatToDateTimeValue;

        #endregion


        #region Properties

        internal string Text
        {
            get
            {
                return this.text;
            }
            set
            {
                this.text = value;
            }
        }

        public bool IsTransparent
        {
            get
            {
                return this.isTransparent;
            }
        }

        public bool ShouldExpand
        {
            get
            {
                return this.shouldExpand;
            }
        }

        public bool ApplyFormat
        {
            get
            {
                return this.applyFormat;
            }
        }


        #endregion


        #region Constructors

        private FormatDescriptorItem(bool isTransparent, bool shouldExapnd, bool applyFormat)
        {
            this.text = string.Empty;
            this.isTransparent = isTransparent;
            this.shouldExpand = shouldExapnd;
            this.applyFormat = applyFormat;
        }

        public FormatDescriptorItem(string text, bool isTransparent, bool shouldExapnd = false, bool applyFormat = true)
            : this(isTransparent, shouldExapnd, applyFormat)
        {
            this.text = text;
        }

        public FormatDescriptorItem(Func<double?, string> applyFormatToDoubleValue, bool isTransparent, bool shouldExapnd = false,
            bool applyFormat = true)
            : this(isTransparent, shouldExapnd, applyFormat)
        {
            this.applyFormatToDoubleValue = applyFormatToDoubleValue;
        }

        public FormatDescriptorItem(Func<DateTime?, string> applyFormatToDateTimeValue, bool isTransparent, bool shouldExapnd = false,
            bool applyFormat = true)
            : this(isTransparent, shouldExapnd, applyFormat)
        {
            this.applyFormatToDateTimeValue = applyFormatToDateTimeValue;
        }

        #endregion


        #region Methods

        public string ApplyFormatToValue(double? doubleValue)
        {
            if (this.applyFormatToDoubleValue != null)
            {
                return this.applyFormatToDoubleValue(doubleValue);
            }

            return this.Text;
        }

        public string ApplyFormatToValue(DateTime? dateTimeValue)
        {
            if (this.applyFormatToDateTimeValue != null)
            {
                return this.applyFormatToDateTimeValue(dateTimeValue);
            }

            return this.Text;
        }

        public virtual IEnumerable<FormatDescriptorItem> GetItems(double? doubleValue)
        {
            yield return this;
        }

        public string GetTextReplacePlaceholder()
        {
            string text = this.Text.Replace(FormatHelper.TextPlaceholder, "{0}");
            text = text.Replace("\\", string.Empty);
            text = text.Replace("\"", string.Empty);
            
            return text;
        }

        #endregion
    }
}