﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings
{
    internal static class FormatStringValidator
    {
        #region Methods

        public static void Validate(string formatString)
        {
            FormatStringType? formatStringType;
            bool isValid = IsValid(formatString, out formatStringType);
            Guard.ThrowExceptionIfFalse(isValid, "isValid");
        }

        public static bool ValidateAndGetFormatStringType(string formatString, out FormatStringType? formatStringType)
        {
            bool isValid = IsValid(formatString, out formatStringType);

            return isValid;
        }

        private static bool IsValid(string formatString, out FormatStringType? formatStringType)
        {
            formatStringType = null;

            if (string.IsNullOrEmpty(formatString))
            {
                return true;
            }

            if (FormatHelper.ContainsTotalHoursMinutesSecondsModifiers(formatString))
            {
                formatStringType = FormatStringType.DateTime;
            }

            formatString = FormatHelper.StripBrackets(formatString);

            string[] parts = formatString.Split(';');

            if (parts.Count() == 1 && parts[0].Contains(FormatHelper.TextPlaceholder))
            {
                return true;
            }

            Dictionary<FormatStringType, int> formatStringTypeCounts = GetFormatStringTypeToOccurenceCountDictionary();
            for (int i = 0; i < parts.Length; i++)
            {
                //If type of part is null we have invalid part.
                formatStringType = GetPartType(parts[i], formatStringType);
                if (!formatStringType.HasValue)
                {
                    return false;
                }

                formatStringTypeCounts[formatStringType.Value]++;
            }
            var keyValuePair = formatStringTypeCounts.OrderByDescending(kv => kv.Value).FirstOrDefault();
            formatStringType = keyValuePair.Key;

            return true;
        }

        private static Dictionary<FormatStringType, int> GetFormatStringTypeToOccurenceCountDictionary()
        {
            Dictionary<FormatStringType, int> formatStringTypeCounts = new Dictionary<FormatStringType, int>();
            formatStringTypeCounts.Add(FormatStringType.Number, 0);
            formatStringTypeCounts.Add(FormatStringType.DateTime, 0);
            formatStringTypeCounts.Add(FormatStringType.Text, 0);

            return formatStringTypeCounts;
        }

        private static FormatStringType? GetPartType(string part, FormatStringType? formatType)
        {
            if (part.Contains(FormatHelper.TextPlaceholder))
            {
                return FormatStringType.Text;
            }

            //StringBuilder sbPart = new StringBuilder(part);
            if (!string.IsNullOrEmpty(part))
            {
                part = FormatHelper.StripBrackets(part);
                part = FormatHelper.StripUnderscores(part);
                part = FormatHelper.StripCurrencyEscapingsAndSlashes(part);
            }

            bool isLiteral = false;
            for (int i = 0; i < part.Length; i++)
            {
                if (part[i] == '\"')
                {
                    isLiteral = !isLiteral;
                }
                else if (!isLiteral)
                {
                    if (FormatHelper.NumberChars.Contains(part[i]))
                    {
                        if (!formatType.HasValue || formatType == FormatStringType.Text)
                        {
                            formatType = FormatStringType.Number;
                        }
                        else if (formatType == FormatStringType.DateTime && !IsZeroValidInDateTimeContext(part, i))
                        {
                            return null;
                        }
                    }
                    else if (FormatHelper.DateChars.Contains(part[i]) || FormatHelper.TimeChars.Contains(part[i]))
                    {
                        if (!formatType.HasValue || formatType == FormatStringType.Text)
                        {
                            formatType = FormatStringType.DateTime;
                        }
                        else if (formatType != FormatStringType.DateTime)
                        {
                            return null;
                        }
                    }
                }
            }

            return formatType;
        }

        private static bool IsZeroValidInDateTimeContext(string part, int index)
        {
            bool isZeroValidInDateTimeContext = false;
            if (part[index] == '0')
            {
                int j = 1;
                isZeroValidInDateTimeContext = index - j >= 0;
                while (isZeroValidInDateTimeContext)
                {
                    if (part[index - j] == '.')
                    {
                        break;
                    }

                    isZeroValidInDateTimeContext = index - j >= 0;
                    isZeroValidInDateTimeContext = part[index - j] == '0';
                    j++;
                }
            }

            return isZeroValidInDateTimeContext;
        }

        #endregion


        #region OLD

        //private static void Sanitize(string formatString)
        //{
        //    if (isValid && formatType == FormatStringType.DateTime)
        //    {
        //        string[] parts = formatString.Split(';');
        //        string result = SanitizePart(parts[0]);
        //        if (parts.Length > 0)
        //        {
        //            for (int i = 1; i < parts.Length; i++)
        //            {
        //                result += ";";
        //                result += SanitizePart(parts[i]);
        //            }
        //        }

        //        formatString = result;
        //    }

        //    formatString = formatString.Replace("\\", string.Empty);
        //    formatString = formatString.Replace("\"", string.Empty);
        //}

        //private static string SanitizePart(string part)
        //{
        //    StringBuilder sbPart = new StringBuilder(part);
        //    FormatHelper.StripBrackets(sbPart);

        //    int startingIndex = part.Length - sbPart.Length;
        //    StringBuilder sbResult = new StringBuilder(part);

        //    List<Tuple<char, int, int>> repeatingChars = GetRepeatingChars(sbResult, startingIndex);
        //    for (int i = repeatingChars.Count - 1; i >= 0; i--)
        //    {
        //        ReplaceRepeatingCharSequence(repeatingChars[i], sbResult);
        //    }

        //    return sbResult.ToString();
        //}

        //private static void ReplaceRepeatingCharSequence(Tuple<char, int, int> repeatingChar, StringBuilder sbPart)
        //{
        //    char charToReplace = repeatingChar.Item1;
        //    int index = repeatingChar.Item2;
        //    int count = repeatingChar.Item3;
        //    int newCount = count;

        //    if (FormatHelper.TimeChars.Contains(charToReplace) && count > 2)
        //    {
        //        newCount = 2;
        //    }
        //    else if (FormatHelper.DateChars.Contains(charToReplace))
        //    {
        //        if (charToReplace == 'y')
        //        {
        //            if (count == 1)
        //            {
        //                newCount = 2;
        //            }
        //            else if (count > 2)
        //            {
        //                newCount = 4;
        //            }
        //        }
        //        else if (charToReplace == 'm')
        //        {
        //            if (count > 5)
        //            {
        //                newCount = 4;
        //            }
        //        }
        //        else if (charToReplace == 'd')
        //        {
        //            if (count > 4)
        //            {
        //                newCount = 4;
        //            }
        //        }
        //    }

        //    sbPart.Remove(index, count);
        //    sbPart.Insert(index, string.Concat(Enumerable.Repeat<char>(charToReplace, newCount)).ToLower());
        //}

        //// NOTE: Char, index, count.
        //private static List<Tuple<char, int, int>> GetRepeatingChars(StringBuilder sbPart, int startingIndex)
        //{
        //    List<Tuple<char, int, int>> repeatingChars = new List<Tuple<char, int, int>>();
        //    char currentChar = 'ñ';
        //    int currentCount = 0;
        //    int currentIndex = -1;
        //    for (int i = startingIndex; i < sbPart.Length; i++)
        //    {
        //        if (char.ToLower(sbPart[i]) == currentChar)
        //        {
        //            currentCount++;
        //        }
        //        else
        //        {
        //            if (FormatHelper.DateChars.Contains(currentChar) || FormatHelper.TimeChars.Contains(currentChar))
        //            {
        //                repeatingChars.Add(new Tuple<char, int, int>(currentChar, currentIndex, currentCount));
        //            }
        //            currentChar = char.ToLower(sbPart[i]);
        //            currentCount = 1;
        //            currentIndex = i;
        //        }
        //    }

        //    if (FormatHelper.DateChars.Contains(currentChar) || FormatHelper.TimeChars.Contains(currentChar))
        //    {
        //        repeatingChars.Add(new Tuple<char, int, int>(currentChar, currentIndex, currentCount));
        //    }

        //    return repeatingChars;
        //}

        #endregion
    }
}