﻿using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Builders;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the accounting format string info class.
    /// </summary>
    public class AccountingFormatStringInfo : ICategorizedFormatStringInfo
    {
        #region Fields

        private readonly int decimalPlaces;
        private readonly CurrencyInfo currencyInfo;
        private readonly string formatString;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the decimal places.
        /// </summary>
        /// <value>The decimal places.</value>
        public int DecimalPlaces
        {
            get
            {
                return this.decimalPlaces;
            }
        }

        /// <summary>
        /// Gets the currency info.
        /// </summary>
        /// <value>The currency info.</value>
        public CurrencyInfo CurrencyInfo
        {
            get
            {
                return this.currencyInfo;
            }
        }

        /// <summary>
        /// Gets the format string.
        /// </summary>
        /// <value>The format string.</value>
        public string FormatString
        {
            get
            {
                return this.formatString;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountingFormatStringInfo" /> class.
        /// </summary>
        /// <param name="decimalPlaces">The decimal places.</param>
        /// <param name="currencyInfo">The currency info.</param>
        /// <param name="formatString">The format string.</param>
        public AccountingFormatStringInfo(int decimalPlaces, CurrencyInfo currencyInfo, string formatString)
        {
            Guard.ThrowExceptionIfLessThan(0, decimalPlaces, "decimalPlaces");
            Guard.ThrowExceptionIfNull(currencyInfo, "currencyInfo");
            Guard.ThrowExceptionIfNull(formatString, "formatString");

            this.decimalPlaces = decimalPlaces;
            this.currencyInfo = currencyInfo;
            this.formatString = formatString;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out AccountingFormatStringInfo formatStringInfo)
        {
            int decimalPlaces;
            if (FormatHelper.TryGetDecimalPlacesForFormatString(formatString, 
                FormatHelper.DefaultSpreadsheetCulture.CultureInfo, out decimalPlaces))
            {
                foreach (CurrencyInfo currencyInfo in CurrencyFormatCategoryManager.CurrencyInfos)
                {
                    string accountingFormatString = AccountingFormatStringBuilder.BuildFormatString(currencyInfo, decimalPlaces);
                    if (formatString.CompareIgnoringSpecialSymbols(accountingFormatString))
                    {
                        formatStringInfo = new AccountingFormatStringInfo(decimalPlaces, currencyInfo, accountingFormatString);
                        return true;
                    }
                }
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}
