﻿using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the date format string info class.
    /// </summary>
    public class DateFormatStringInfo : ICategorizedFormatStringInfo
    {
        #region Fields

        private readonly CultureInfo culture;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        public CultureInfo Culture
        {
            get
            {
                return this.culture;
            }
        }

        #endregion
        

        #region Cosntructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DateFormatStringInfo" /> class.
        /// </summary>
        /// <param name="culture">The culture.</param>
        public DateFormatStringInfo(CultureInfo culture)
        {
            Guard.ThrowExceptionIfNull(culture, "culture");
            this.culture = culture;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out DateFormatStringInfo formatStringInfo)
        {
            string loweredFormatString = formatString.ToLower();

            if (DateTimeFormatCategoryManager.CultureInfoToDateFormats.ContainsKey(FormatHelper.DefaultSpreadsheetCulture.CultureInfo))
            {
                IList<string> dateFormatsForCulture = DateTimeFormatCategoryManager.CultureInfoToDateFormats[FormatHelper.DefaultSpreadsheetCulture.CultureInfo];

                if (dateFormatsForCulture.Any(x => x.Contains(loweredFormatString) && loweredFormatString != "@"))
                {
                    formatStringInfo = new DateFormatStringInfo(FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                    return true;
                }
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}