﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Builders;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the number format string info class.
    /// </summary>
    public class NumberFormatStringInfo : ICategorizedFormatStringInfo
    {
        #region Fields

        private readonly int decimalPlaces;
        private readonly bool useThousandSeparator;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the decimal places.
        /// </summary>
        /// <value>The decimal places.</value>
        public int DecimalPlaces
        {
            get
            {
                return this.decimalPlaces;
            }
        }

        /// <summary>
        /// Gets the use thousand separator.
        /// </summary>
        /// <value>The use thousand separator.</value>
        public bool UseThousandSeparator
        {
            get
            {
                return this.useThousandSeparator;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="NumberFormatStringInfo" /> class.
        /// </summary>
        /// <param name="decimalPlaces">The decimal places.</param>
        /// <param name="useThousandSeparator">The use thousand separator.</param>
        public NumberFormatStringInfo(int decimalPlaces, bool useThousandSeparator)
        {
            Guard.ThrowExceptionIfLessThan(0, decimalPlaces, "decimalPlaces");

            this.decimalPlaces = decimalPlaces;
            this.useThousandSeparator = useThousandSeparator;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out NumberFormatStringInfo formatStringInfo)
        {
            int decimalPlaces;
            bool useThousandSeparator;
            if (FormatHelper.TryGetDecimalPlacesForFormatString(formatString,
                FormatHelper.DefaultSpreadsheetCulture.CultureInfo, out decimalPlaces, out useThousandSeparator))
            {
                List<string> formatStrings = NumberFormatStringBuilder.BuildFormatStrings(decimalPlaces, useThousandSeparator);
                if (formatStrings.Contains(formatString))
                {
                    formatStringInfo = new NumberFormatStringInfo(decimalPlaces, useThousandSeparator);
                    return true;
                }
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}