﻿using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents percentage format string info.
    /// </summary>
    public class PercentageFormatStringInfo : ICategorizedFormatStringInfo
    {
        #region Constants

        private static readonly string PercentPattern = string.Format(@"^0(\.(?<{0}>0+))?%$", FormatHelper.ZeroCountGroupName);
        private static readonly Regex PercentRegex = new Regex(PercentPattern);

        #endregion


        #region Fields

        private readonly int decimalPlaces;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the decimal places.
        /// </summary>
        /// <value>The decimal places.</value>
        public int DecimalPlaces
        {
            get
            {
                return this.decimalPlaces;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PercentageFormatStringInfo" /> class.
        /// </summary>
        /// <param name="decimalPlaces">The decimal places.</param>
        public PercentageFormatStringInfo(int decimalPlaces)
        {
            Guard.ThrowExceptionIfLessThan(0, decimalPlaces, "decimalPlaces");

            this.decimalPlaces = decimalPlaces;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out PercentageFormatStringInfo formatStringInfo)
        {
            Match match = PercentRegex.Match(formatString);

            if (match.Success)
            {
                int decimalPlaces = match.Groups[FormatHelper.ZeroCountGroupName].Value.Length;

                formatStringInfo = new PercentageFormatStringInfo(decimalPlaces);
                return true;
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}