﻿using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the scientific format string info class.
    /// </summary>
    public class ScientificFormatStringInfo
    {
        #region Constants

        private static readonly string ScientificPattern = string.Format(@"^0(\.(?<{0}>0*))?E\+00$", FormatHelper.ZeroCountGroupName);
        private static readonly Regex ScientificRegex = new Regex(ScientificPattern);

        #endregion


        #region Fields

        private readonly int decimalPlaces;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the decimal places.
        /// </summary>
        /// <value>The decimal places.</value>
        public int DecimalPlaces
        {
            get
            {
                return this.decimalPlaces;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ScientificFormatStringInfo" /> class.
        /// </summary>
        /// <param name="decimalPlaces">The decimal places.</param>
        public ScientificFormatStringInfo(int decimalPlaces)
        {
            Guard.ThrowExceptionIfLessThan(0, decimalPlaces, "decimalPlaces");

            this.decimalPlaces = decimalPlaces;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out ScientificFormatStringInfo formatStringInfo)
        {
            Match match = ScientificRegex.Match(formatString);

            if (match.Success)
            {
                int decimalPlaces = match.Groups[FormatHelper.ZeroCountGroupName].Value.Length;

                formatStringInfo = new ScientificFormatStringInfo(decimalPlaces);
                return true;
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}