﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the special format string info class.
    /// </summary>
    public class SpecialFormatStringInfo : ICategorizedFormatStringInfo
    {
        #region Fields

        private readonly CultureInfo culture;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        public CultureInfo Culture
        {
            get
            {
                return this.culture;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SpecialFormatStringInfo" /> class.
        /// </summary>
        /// <param name="culture">The culture.</param>
        public SpecialFormatStringInfo(CultureInfo culture)
        {
            Guard.ThrowExceptionIfNull(culture, "culture");
            this.culture = culture;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out SpecialFormatStringInfo formatStringInfo)
        {
            foreach (KeyValuePair<CultureInfo, IList<SpecialFormatInfo>> pair in SpecialFormatCategoryManager.CultureInfoToFormatInfo)
            {
                SpecialFormatInfo formatInfo = pair.Value.Where(sfi => sfi.FormatString == formatString).FirstOrDefault();
                if (formatInfo != null)
                {
                    formatStringInfo = new SpecialFormatStringInfo(pair.Key);
                    return true;
                }
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}