﻿using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.CategoryManagers;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Infos
{
    /// <summary>
    /// Represents the time format string info class.
    /// </summary>
    public class TimeFormatStringInfo: ICategorizedFormatStringInfo
    {
        #region Fields

        private readonly CultureInfo culture;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        public CultureInfo Culture
        {
            get
            {
                return this.culture;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TimeFormatStringInfo" /> class.
        /// </summary>
        /// <param name="culture">The culture.</param>
        public TimeFormatStringInfo(CultureInfo culture)
        {
            Guard.ThrowExceptionIfNull(culture, "culture");
            this.culture = culture;
        }

        #endregion


        #region Methods

        internal static bool TryCreate(string formatString, out TimeFormatStringInfo formatStringInfo)
        {
            string loweredFormatString = formatString.ToLower();
            loweredFormatString = loweredFormatString.Replace("tt", "AM/PM");

            if (DateTimeFormatCategoryManager.CultureInfoToTimeFormats.ContainsKey(FormatHelper.DefaultSpreadsheetCulture.CultureInfo))
            {
                IList<string> timeFormatsForCulture = DateTimeFormatCategoryManager.CultureInfoToTimeFormats[FormatHelper.DefaultSpreadsheetCulture.CultureInfo];

                if (timeFormatsForCulture.Any(x => x.Contains(loweredFormatString) && loweredFormatString != "@"))
                {
                    formatStringInfo = new TimeFormatStringInfo(FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                    return true;
                }
            }

            formatStringInfo = null;
            return false;
        }

        #endregion
    }
}
