﻿using System;
using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;

namespace Telerik.Windows.Documents.Spreadsheet.Formatting
{
    internal static class FractionParser
    {
        #region Constants

        private static readonly string WholeGroupName = "whole";
        private static readonly string NumeratorGroupName = "numerator";
        private static readonly string DenominatorGroupName = "denominator";
        private static readonly string MinusSignGroupName = "minusSign";

        private static readonly Regex FractionsRegex = new Regex(string.Format(@"^(?<{0}>-)?\s*(?<{1}>\d+) (?<{2}>\d+)/(?<{3}>\d+)$",
            MinusSignGroupName, WholeGroupName, NumeratorGroupName, DenominatorGroupName));
        private static readonly Regex FractionsShortRegex = new Regex(string.Format(@"^(?<{0}>-)?\s*(?<{1}>\d+)/(?<{2}>\d+)$", MinusSignGroupName,
            NumeratorGroupName, DenominatorGroupName));

        private static readonly string DefaultFractionsUpToOneDigitFormat = "# ?/?";
        private static readonly string DefaultFractionsUpToTwoDigitFormat = "# ??/??";
        //private static readonly string DefaultFractionsUpToThreeDigitFormat = "# ???/???";

        #endregion


        #region Methods

        public static bool TryParse(string value, FormatStringType? forceParsingAsSpecificType, out double result, out string format)
        {
            format = FormatHelper.GeneralFormatString;
            int denominatorLength = -1;
            result = 0;
            bool canParse = FractionsRegex.IsMatch(value);

            if (canParse)
            {
                Match match = FractionsRegex.Match(value);

                double whole = int.Parse(match.Groups[WholeGroupName].Value);
                double numerator = int.Parse(match.Groups[NumeratorGroupName].Value);
                double denominator = int.Parse(match.Groups[DenominatorGroupName].Value);

                denominatorLength = denominator.ToString().Length;

                result = whole + numerator / denominator;

                if (match.Groups[MinusSignGroupName].Success)
                {
                    result *= -1;
                }
            }
            else if (!canParse && forceParsingAsSpecificType == FormatStringType.Number)
            {
                canParse = FractionsShortRegex.IsMatch(value);

                if (canParse)
                {
                    Match match = FractionsShortRegex.Match(value);

                    double numerator = int.Parse(match.Groups[NumeratorGroupName].Value);
                    double denominator = int.Parse(match.Groups[DenominatorGroupName].Value);
                    denominatorLength = denominator.ToString().Length;

                    result = numerator / denominator;

                    if (match.Groups[MinusSignGroupName].Success)
                    {
                        result *= -1;
                    }
                }
            }

            if (canParse)
            {
                format = GetFractionFormat(denominatorLength);
            }

            return canParse;
        }

        private static string GetFractionFormat(int denominatorLength)
        {
            switch (denominatorLength)
            {
                case 1:
                    return DefaultFractionsUpToOneDigitFormat;

                case 2:
                    return DefaultFractionsUpToTwoDigitFormat;

                default:
                    return DefaultFractionsUpToTwoDigitFormat;
            }
        }

        #endregion
    }
}
