﻿using Telerik.Windows.Documents.Spreadsheet.Commands;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.History
{
    /// <summary>
    /// Represents a change in a workbook.
    /// </summary>
    public class WorkbookChange
    {
		#region Fields

        private readonly IUndoableWorkbookCommand command;
        private readonly WorkbookCommandContextBase commandContext;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the name of the command.
        /// </summary>
        /// <value>The name of the command.</value>
        public string CommandName
        {
            get
            {
                return this.Command.Name;
            }
        }

        internal IUndoableWorkbookCommand Command
        {
            get
			{
				return this.command;
			}
        }

        internal WorkbookCommandContextBase CommandContext
        {
            get
			{
				return this.commandContext;
			}
        }

        #endregion


        #region Constructors

		internal WorkbookChange(IUndoableWorkbookCommand command, WorkbookCommandContextBase commandContext)
        {
			Guard.ThrowExceptionIfNull(command, "command");
			Guard.ThrowExceptionIfNull(commandContext, "commandContext");

			this.command = command;
			this.commandContext = commandContext;
        }

        #endregion


        #region Methods

		internal bool Undo()
		{
			return this.Command.Undo(this.CommandContext);
		}

		internal bool Redo()
        {
            return this.Command.Redo(this.CommandContext);
        }

        #endregion
	}
}