﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.History
{
    /// <summary>
    /// Represents an action in the workbook history.
    /// </summary>
    public class WorkbookHistoryAction
    {
		#region Fields

        private readonly List<WorkbookChange> changes;
        private readonly Sheet activeSheet;

        #endregion


        #region Properties

        /// <summary>
        /// Gets or sets the display text.
        /// </summary>
        /// <value>The display text.</value>
        public string DisplayText { get; protected set; }

        internal object Context { get; set; }

        /// <summary>
        /// Gets the changes of the action.
        /// </summary>
        /// <value>The changes.</value>
        public IEnumerable<WorkbookChange> Changes
        {
            get
            {
                return this.changes;
            }
        }

        /// <summary>
        /// Gets the active sheet.
        /// </summary>
        /// <value>The active sheet.</value>
        public Sheet ActiveSheet
        {
            get
            {
                return this.activeSheet;
            }
        }

        #endregion


        #region Constructor

		internal WorkbookHistoryAction(IEnumerable<WorkbookChange> changes, string displayText)
        {
            this.changes = new List<WorkbookChange>(changes);
            this.activeSheet = this.changes[0].CommandContext.ActiveSheet;
            this.DisplayText = displayText;
        }

        #endregion


        #region Methods

        internal bool Undo()
        {
			bool result = true;
			WorkbookChange firstChange = this.Changes.FirstOrDefault();
			if (firstChange != null)
			{
                var workbook = firstChange.CommandContext.Workbook;
                using (new UpdateScope(workbook.SuspendLayoutUpdate, workbook.ResumeLayoutUpdate))
                {
                    foreach (WorkbookChange change in this.Changes.Reverse())
                    {
                        result |= change.Undo();
                    }
                }
			}
			

			return result;
        }

		internal bool Redo()
        {
			bool result = true;
			WorkbookChange firstChange = this.Changes.FirstOrDefault();
			if (firstChange != null)
			{
                var workbook = firstChange.CommandContext.Workbook;
                using (new UpdateScope(workbook.SuspendLayoutUpdate, workbook.ResumeLayoutUpdate))
                {
                    foreach (WorkbookChange change in this.Changes)
                    {
                        result |= change.Redo();
                    }
                }
			}

			return result;
        }

		internal bool Merge(/*WorkbookHistoryAction action*/)
        {
			// TODO: Introduce merge logic if needed.
			//if (action.changes.Count == 1)
			//{
			//    TextChange lastTextChange = this.actions.Last();
			//    TextChange newTextChange = action.changes[0];
			//    if (newTextChange.OldText.Length == 0 &&
			//        newTextChange.InsertedLineCount == 0 &&
			//        lastTextChange.NewSpan.End == newTextChange.OldSpan.Start)
			//    {
			//        CaretPosition position = new CaretPosition(this.Sheet);
			//        position.MoveToIndex(newTextChange.NewSpan.Start);
			//        if (!position.IsAtWordStart())
			//        {
			//            this.actions.AddRange(action.changes);
			//            return true;
			//        }
			//    }
			//}

            return false;
        }

        #endregion 
	}
}