﻿using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Layout
{
    /// <summary>
    /// Represents a column layout box.
    /// </summary>
    public class ColumnLayoutBox : LayoutBox
    {
		#region Fields

		private readonly int columnIndex;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the index of the column.
        /// </summary>
        /// <value>The index of the column.</value>
        public int ColumnIndex
        {
			get
			{
				return this.columnIndex;
			}
		}

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ColumnLayoutBox" /> class.
        /// </summary>
        /// <param name="columnIndex">Index of the column.</param>
        /// <param name="rect">The rectangle.</param>
        public ColumnLayoutBox(int columnIndex, Rect rect) 
            : base(rect)
		{
			Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);
			this.columnIndex = columnIndex;
		}

		#endregion


		#region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
			ColumnLayoutBox other = obj as ColumnLayoutBox;
			if (other == null)
			{
				return false;
			}

			return this.ColumnIndex == other.ColumnIndex;
		}

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
			return this.columnIndex;
		}

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return string.Format("ColumnLayoutBox {0}: {1}", this.ColumnIndex, base.ToString());
        }

		#endregion
	}
}