﻿using System;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class BorderRenderHelper
    {
        public static DoubleCollection GetStrokeDashArrayByBorderStyle(CellBorderStyle borderStyle)
        {
            switch (borderStyle)
            {
                case CellBorderStyle.None:
                    return null;

                case CellBorderStyle.DashDot:
                    return new DoubleCollection { 8, 2, 2, 2 };

                case CellBorderStyle.DashDotDot:
                    return new DoubleCollection { 8, 2, 2, 2, 2, 2 };

                case CellBorderStyle.Dashed:
                    return new DoubleCollection { 3, 1 };

                case CellBorderStyle.Dotted:
                    return new DoubleCollection { 2, 2 };

                case CellBorderStyle.Hair:
                    return new DoubleCollection { 1.5, 1.5 };

                // TODO:
                case CellBorderStyle.Double:
                case CellBorderStyle.Medium:
                    return null;
                // TODO:
                case CellBorderStyle.SlantDashDot:
                case CellBorderStyle.MediumDashDot:
                    return new DoubleCollection { 4, 1, 1, 1 };

                case CellBorderStyle.MediumDashDotDot:
                    return new DoubleCollection { 4, 1, 1, 1, 1, 1 };

                case CellBorderStyle.MediumDashed:
                    return new DoubleCollection { 2, 1 };

                case CellBorderStyle.Thick:
                    return null;

                case CellBorderStyle.Thin:
                    return null;

                default:
                    return null;
            }
        }

        public static CellBorder GetRightBorderToDisplay(WorksheetRenderUpdateContext context, int rowIndex, int columnIndex)
        {
            CellBorder rightBorder = context.GetRightBorder(CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex));//worksheet.Cells[rowIndex, columnIndex].GetRightBorder().Value;
            if (columnIndex == SpreadsheetDefaultValues.ColumnCount - 1)
            {
                return rightBorder;
            }

            CellBorder leftBorder = context.GetLeftBorder(CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex + 1)); //worksheet.Cells[rowIndex, columnIndex + 1].GetLeftBorder().Value;

            return CellBorder.GetWithMaxPriority(leftBorder, rightBorder, context.CurrentTheme.ColorScheme);
        }

        public static CellBorder GetBottomBorderToDisplay(WorksheetRenderUpdateContext context, int rowIndex, int columnIndex)
        {
            CellBorder bottomBorder = context.GetBottomBorder(CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex));//worksheet.Cells[rowIndex, columnIndex].GetBottomBorder().Value;
            if (rowIndex == SpreadsheetDefaultValues.RowCount - 1)
            {
                return bottomBorder;
            }

            CellBorder topBorder = context.GetTopBorder(CellsPropertyBag.ConvertCellIndexToLong(rowIndex + 1, columnIndex));//worksheet.Cells[rowIndex + 1, columnIndex].GetTopBorder().Value;

            return CellBorder.GetWithMaxPriority(bottomBorder, topBorder, context.CurrentTheme.ColorScheme);
        }
    }
}
