﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal abstract class BoxBasedLayerBase<T> : WorksheetLayerBase
        where T : LayoutBox
    {
        protected override void UpdateRenderOverride(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            foreach (var pair in worksheetUpdateContext.VisibleCellLayoutBoxes)
            {
                ViewportPaneType paneType = pair.Key;

                foreach (T box in this.GetVisibleBoxes(paneType, worksheetUpdateContext))
                {
                    IRenderable renderable = this.CreateRenderableElementForBox(paneType, box, worksheetUpdateContext);

                    if (renderable != null)
                    {
                        this.ContainerManager.Add(renderable, paneType);
                    }
                }
            }
        }

        protected abstract IEnumerable<T> GetVisibleBoxes(ViewportPaneType viewportPaneType, WorksheetRenderUpdateContext updateContext);

        protected abstract IRenderable CreateRenderableElementForBox(ViewportPaneType viewportPaneType, T box,  WorksheetRenderUpdateContext updateContext);
    }
}
