﻿using System;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class ColumnHeadingMarginLayer : ColumnBasedLayerBase
    {
        private readonly IRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>> columnHeadingRenderer;

        public ColumnHeadingMarginLayer(IRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>> columnHeadingRenderer)
        {
            this.columnHeadingRenderer = columnHeadingRenderer;
        }

        public override string Name
        {
            get
            {
                return WorksheetPredefinedLayers.ColumnHeadingMargin;
            }
        }

        public double MaxHeight { get; private set; }

        internal IRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>> ColumnHeadingRenderer
        {
            get
            {
                return this.columnHeadingRenderer;
            }
        }

        internal static bool ShouldCreateColumnHeadingElementForBox(ViewportPaneType viewportPaneType, WorksheetRenderUpdateContext updateContext)
        {
            SheetViewport sheetViewpot = updateContext.SheetViewport;
            if (sheetViewpot[ViewportPaneType.Fixed].IsEmpty &&
                sheetViewpot[ViewportPaneType.VerticalScrollable].IsEmpty &&
                !sheetViewpot[ViewportPaneType.HorizontalScrollable].IsEmpty &&
                viewportPaneType == ViewportPaneType.Scrollable)
            {
                return false;
            }

            if ((!sheetViewpot[ViewportPaneType.Fixed].IsEmpty &&
                 !sheetViewpot[ViewportPaneType.Scrollable].IsEmpty) &&
                (viewportPaneType == ViewportPaneType.VerticalScrollable || viewportPaneType == ViewportPaneType.Scrollable))
            {
                return false;
            }

            return true;
        }

        protected override void UpdateRenderOverride(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            this.MaxHeight = SpreadsheetDefaultValues.RowColumnHeadingMinimumSize.Height;
            base.UpdateRenderOverride(worksheetUpdateContext);
        }

        protected override IRenderable CreateRenderableElementForBox(ViewportPaneType viewportPaneType, ColumnLayoutBox box, WorksheetRenderUpdateContext updateContext)
        {
            if (!ColumnHeadingMarginLayer.ShouldCreateColumnHeadingElementForBox(viewportPaneType, updateContext))
            {
                return null;
            }

            Size columnHeadingSize = updateContext.Layout.GetColumnHeadingSize(box.ColumnIndex);
            this.MaxHeight = Math.Max(this.MaxHeight, columnHeadingSize.Height);
            Worksheet worksheet = updateContext.Worksheet;
            CellRange visibleRange = updateContext.SheetViewport[viewportPaneType].VisibleRange;

            return new RowColumnHeadingRenderable<ColumnLayoutBox>()
            {
                LayoutBox = box,
                Text = updateContext.Worksheet.HeaderNameRenderingConverter.ConvertColumnIndexToName(new HeaderNameRenderingConverterContext(worksheet, visibleRange), box.ColumnIndex), // NameConverter.ConvertColumnIndexToName(box.ColumnIndex),
                Rectangle = new Rect(new Point(box.Left, 0), columnHeadingSize)
            };
        }

        protected override void TranslateAndScale(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            foreach (ViewportPaneType paneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                foreach (IRenderable renderable in this.ContainerManager.GetElementsForViewportPane(paneType))
                {
                    RowColumnHeadingRenderable<ColumnLayoutBox> columnHeading = (RowColumnHeadingRenderable<ColumnLayoutBox>)renderable;
                    Rect translatedRect = this.Translate(columnHeading.Rectangle, paneType, worksheetUpdateContext);
                    columnHeading.Rectangle = new Rect(translatedRect.Left, 0, translatedRect.Width, this.MaxHeight);
                    columnHeading.ScaleFactor = worksheetUpdateContext.ScaleFactor;

                    this.columnHeadingRenderer.Render(columnHeading, paneType);
                }
            }
        }
    }
}
