﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal static class GradientsRenderHelper
    {
        private static readonly Dictionary<GradientType, Func<GradientFill, ThemeColorScheme, Size, IEnumerable<LinearGradientBox>>> gradientToGradientBoxes;

        static GradientsRenderHelper()
        {
            GradientsRenderHelper.gradientToGradientBoxes = new Dictionary<GradientType, Func<GradientFill, ThemeColorScheme, Size, IEnumerable<LinearGradientBox>>>()
            {
                { GradientType.Horizontal, GradientsRenderHelper.GenerateHorizontalGradient},
                { GradientType.HorizontalReversed, GradientsRenderHelper.GenerateHorizontalReversedGradient},
                { GradientType.HorizontalRepeated, GradientsRenderHelper.GenerateHorizontalRepeatedGradient},
                { GradientType.Vertical, GradientsRenderHelper.GenerateVerticalGradient},
                { GradientType.VerticalReversed, GradientsRenderHelper.GenerateVerticalReversedGradient},
                { GradientType.VerticalRepeated, GradientsRenderHelper.GenerateVerticalRepeatedGradient},
                { GradientType.DiagonalUp, GradientsRenderHelper.GenerateDiagonalUpGradient},
                { GradientType.DiagonalUpReversed, GradientsRenderHelper.GenerateDiagonalUpReversedGradient},
                { GradientType.DiagonalUpRepeated, GradientsRenderHelper.GenerateDiagonalUpRepeatedGradient},
                { GradientType.DiagonalDown, GradientsRenderHelper.GenerateDiagonalDownGradient},
                { GradientType.DiagonalDownReversed, GradientsRenderHelper.GenerateDiagonalDownReversedGradient},
                { GradientType.DiagonalDownRepeated, GradientsRenderHelper.GenerateDiagonalDownRepeatedGradient},
                { GradientType.FromTopLeftCorner, GradientsRenderHelper.GenerateFromTopLeftCornerGradient},
                { GradientType.FromTopRightCorner, GradientsRenderHelper.GenerateFromTopRightCornerGradient},
                { GradientType.FromBottomLeftCorner, GradientsRenderHelper.GenerateFromBottomLeftCornerGradient},
                { GradientType.FromBottomRightCorner, GradientsRenderHelper.GenerateFromBottomRightCornerGradient},
                { GradientType.FromCenter, GradientsRenderHelper.GenerateFromCenterGradient}
            };
        }

        public static IEnumerable<LinearGradientBox> GenerateGradientBoxes(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            Func<GradientFill, ThemeColorScheme, Size, IEnumerable<LinearGradientBox>> gradientBoxesGenerator = GradientsRenderHelper.gradientToGradientBoxes[fill.GradientType];

            return gradientBoxesGenerator(fill, colorScheme, containerSize);
        }

        private static IEnumerable<LinearGradientBox> GenerateHorizontalGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateHorizontalReversedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateHorizontalRepeatedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 0.5;
            double gradientStopOffsetColor1Repeat = 1;

            LinearGradientBox gradient = GenerateRepeatedGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2, gradientStopOffsetColor1Repeat: gradientStopOffsetColor1Repeat,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateVerticalGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateVerticalReversedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(1, 0), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateVerticalRepeatedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 0.5;
            double gradientStopOffsetColor1Repeat = 1;

            LinearGradientBox gradient = GenerateRepeatedGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2, gradientStopOffsetColor1Repeat: gradientStopOffsetColor1Repeat,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalUpGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalUpReversedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(1, 1), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalUpRepeatedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 0.5;
            double gradientStopOffsetColor1Repeat = 1;

            LinearGradientBox gradient = GenerateRepeatedGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2, gradientStopOffsetColor1Repeat: gradientStopOffsetColor1Repeat,
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalDownGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(1, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalDownReversedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 1;

            LinearGradientBox gradient = GenerateGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateDiagonalDownRepeatedGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0;
            double gradientStopOffsetColor2 = 0.5;
            double gradientStopOffsetColor1Repeat = 1;

            LinearGradientBox gradient = GenerateRepeatedGradientRectangleWithRectangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1,
                gradientStopOffsetColor2: gradientStopOffsetColor2, gradientStopOffsetColor1Repeat: gradientStopOffsetColor1Repeat,
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return gradient;
        }

        private static IEnumerable<LinearGradientBox> GenerateFromTopLeftCornerGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0.2;
            double gradientStopOffsetColor2 = 1;

            double width = containerSize.Width;
            double height = containerSize.Height;

            LinearGradientBox topGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, height), clipThirdPoint: new Point(width, 0),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            LinearGradientBox bottomGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, height), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return topGradientBox;
            yield return bottomGradientBox;
        }

        private static IEnumerable<LinearGradientBox> GenerateFromTopRightCornerGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0.2;
            double gradientStopOffsetColor2 = 1;

            double width = containerSize.Width;
            double height = containerSize.Height;

            LinearGradientBox topGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, 0), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(1, 0), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            LinearGradientBox bottomGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(width, height), clipSecondPoint: new Point(width, 0), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return topGradientBox;
            yield return bottomGradientBox;
        }

        private static IEnumerable<LinearGradientBox> GenerateFromBottomLeftCornerGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0.2;
            double gradientStopOffsetColor2 = 1;

            double width = containerSize.Width;
            double height = containerSize.Height;

            LinearGradientBox topGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, 0), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            LinearGradientBox bottomGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(width, height), clipSecondPoint: new Point(width, 0), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return topGradientBox;
            yield return bottomGradientBox;
        }

        private static IEnumerable<LinearGradientBox> GenerateFromBottomRightCornerGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0.2;
            double gradientStopOffsetColor2 = 1;

            double width = containerSize.Width;
            double height = containerSize.Height;

            LinearGradientBox topGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, height), clipThirdPoint: new Point(width, 0),
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            LinearGradientBox bottomGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width, height), clipThirdPoint: new Point(0, height),
                gradientBrushStartPoint: new Point(1, 0), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(new Point(0, 0), containerSize));

            yield return topGradientBox;
            yield return bottomGradientBox;
        }

        private static IEnumerable<LinearGradientBox> GenerateFromCenterGradient(GradientFill fill, ThemeColorScheme colorScheme, Size containerSize)
        {
            double gradientStopOffsetColor1 = 0.75;
            double gradientStopOffsetColor2 = 0;

            double width = containerSize.Width;
            double height = containerSize.Height;

            LinearGradientBox leftGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(0, height), clipThirdPoint: new Point(width / 2, height / 2),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(1, 0), boundingBox: new Rect(0, 0, width / 2, height));

            LinearGradientBox rightGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(width / 2, 0), clipSecondPoint: new Point(width / 2, height), clipThirdPoint: new Point(0, height / 2),
                gradientBrushStartPoint: new Point(1, 0), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(width / 2, 0, width / 2, height));

            LinearGradientBox topGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, 0), clipSecondPoint: new Point(width / 2, height / 2), clipThirdPoint: new Point(width, 0),
                gradientBrushStartPoint: new Point(0, 0), gradientBrushEndPoint: new Point(0, 1), boundingBox: new Rect(0, 0, width, height / 2));

            LinearGradientBox bottomGradientBox = GenerateGradientRectangleWithTriangleClip(
                fill: fill, colorScheme: colorScheme, gradientStopOffsetColor1: gradientStopOffsetColor1, gradientStopOffsetColor2: gradientStopOffsetColor2,
                clipFirstPoint: new Point(0, height / 2), clipSecondPoint: new Point(width, height / 2), clipThirdPoint: new Point(width / 2, 0),
                gradientBrushStartPoint: new Point(0, 1), gradientBrushEndPoint: new Point(0, 0), boundingBox: new Rect(0, height / 2, width, height / 2));

            yield return leftGradientBox;
            yield return rightGradientBox;
            yield return topGradientBox;
            yield return bottomGradientBox;
        }

        private static LinearGradientBox GenerateGradientRectangleWithTriangleClip(GradientFill fill, ThemeColorScheme colorScheme, double gradientStopOffsetColor1, double gradientStopOffsetColor2, Point clipFirstPoint, Point clipSecondPoint,
            Point clipThirdPoint, Point gradientBrushStartPoint, Point gradientBrushEndPoint, Rect boundingBox)
        {
            PathFigure rectClipPathFigure = new PathFigure();
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = clipSecondPoint });
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = clipThirdPoint });
            rectClipPathFigure.StartPoint = clipFirstPoint;

            LinearGradientBox gradient = GenerateGradientRectangle(
                fill: fill,
                colorScheme: colorScheme,
                gradientStopOffsetColor1: gradientStopOffsetColor1,
                gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: gradientBrushStartPoint,
                gradientBrushEndPoint: gradientBrushEndPoint,
                rectClipPathFigure: rectClipPathFigure,
                boundingBox: boundingBox);

            return gradient;
        }

        private static LinearGradientBox GenerateGradientRectangleWithRectangleClip(GradientFill fill, ThemeColorScheme colorScheme, double gradientStopOffsetColor1, double gradientStopOffsetColor2,
            Point gradientBrushStartPoint, Point gradientBrushEndPoint, Rect boundingBox)
        {
            PathFigure rectClipPathFigure = new PathFigure();
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(0, boundingBox.Height) });
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(boundingBox.Width, boundingBox.Height) });
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(boundingBox.Width, 0) });
            rectClipPathFigure.StartPoint = new Point(0, 0);

            LinearGradientBox gradient = GenerateGradientRectangle(
                fill: fill,
                colorScheme: colorScheme,
                gradientStopOffsetColor1: gradientStopOffsetColor1,
                gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientBrushStartPoint: gradientBrushStartPoint,
                gradientBrushEndPoint: gradientBrushEndPoint,
                rectClipPathFigure: rectClipPathFigure,
                boundingBox: boundingBox);

            return gradient;
        }

        private static LinearGradientBox GenerateRepeatedGradientRectangleWithRectangleClip(GradientFill fill, ThemeColorScheme colorScheme, double gradientStopOffsetColor1, double gradientStopOffsetColor2, double gradientStopOffsetColor1Repeat,
            Point gradientBrushStartPoint, Point gradientBrushEndPoint, Rect boundingBox)
        {
            PathFigure rectClipPathFigure = new PathFigure();
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(0, boundingBox.Height) });
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(boundingBox.Width, boundingBox.Height) });
            rectClipPathFigure.Segments.Add(new LineSegment() { Point = new Point(boundingBox.Width, 0) });
            rectClipPathFigure.StartPoint = new Point(0, 0);

            LinearGradientBox gradient = GenerateRepeatedGradientRectangle(
                fill: fill,
                colorScheme: colorScheme,
                gradientStopOffsetColor1: gradientStopOffsetColor1,
                gradientStopOffsetColor2: gradientStopOffsetColor2,
                gradientStopOffsetColor1Repeat: gradientStopOffsetColor1Repeat,
                gradientBrushStartPoint: gradientBrushStartPoint,
                gradientBrushEndPoint: gradientBrushEndPoint,
                rectClipPathFigure: rectClipPathFigure,
                boundingBox: boundingBox);

            return gradient;
        }

        private static LinearGradientBox GenerateGradientRectangle(GradientFill fill, ThemeColorScheme colorScheme, double gradientStopOffsetColor1, double gradientStopOffsetColor2, Point gradientBrushStartPoint, Point gradientBrushEndPoint,
            PathFigure rectClipPathFigure, Rect boundingBox)
        {
            GradientStopCollection gradientStopCollection = new GradientStopCollection();

            GradientStop gradientStopColor1 = new GradientStop();
            gradientStopColor1.Color = fill.Color1.GetActualValue(colorScheme);
            gradientStopColor1.Offset = gradientStopOffsetColor1;
            gradientStopCollection.Add(gradientStopColor1);

            GradientStop gradientStopColor2 = new GradientStop();
            gradientStopColor2.Color = fill.Color2.GetActualValue(colorScheme);
            gradientStopColor2.Offset = gradientStopOffsetColor2;
            gradientStopCollection.Add(gradientStopColor2);

            LinearGradientBrush gradientBrush = new LinearGradientBrush(gradientStopCollection, 0);
            gradientBrush.StartPoint = gradientBrushStartPoint;
            gradientBrush.EndPoint = gradientBrushEndPoint;

            PathGeometry rectClipGeometry = new PathGeometry();
            rectClipGeometry.Figures.Add(rectClipPathFigure);

            LinearGradientBox gradient = new LinearGradientBox();
            gradient.BoundingBox = boundingBox;
            gradient.Fill = gradientBrush;
            gradient.Clip = rectClipGeometry;

            return gradient;
        }

        private static LinearGradientBox GenerateRepeatedGradientRectangle(GradientFill fill, ThemeColorScheme colorScheme, double gradientStopOffsetColor1, double gradientStopOffsetColor2, double gradientStopOffsetColor1Repeat,
            Point gradientBrushStartPoint, Point gradientBrushEndPoint, PathFigure rectClipPathFigure, Rect boundingBox)
        {
            GradientStopCollection gradientStopCollection = new GradientStopCollection();

            GradientStop gradientStopColor1 = new GradientStop();
            gradientStopColor1.Color = fill.Color1.GetActualValue(colorScheme);
            gradientStopColor1.Offset = gradientStopOffsetColor1;
            gradientStopCollection.Add(gradientStopColor1);

            GradientStop gradientStopColor2 = new GradientStop();
            gradientStopColor2.Color = fill.Color2.GetActualValue(colorScheme);
            gradientStopColor2.Offset = gradientStopOffsetColor2;
            gradientStopCollection.Add(gradientStopColor2);

            GradientStop gradientStopColor1Repeat = new GradientStop();
            gradientStopColor1Repeat.Color = fill.Color1.GetActualValue(colorScheme);
            gradientStopColor1Repeat.Offset = gradientStopOffsetColor1Repeat;
            gradientStopCollection.Add(gradientStopColor1Repeat);

            LinearGradientBrush gradientBrush = new LinearGradientBrush(gradientStopCollection, 0);
            gradientBrush.StartPoint = gradientBrushStartPoint;
            gradientBrush.EndPoint = gradientBrushEndPoint;

            PathGeometry rectClipGeometry = new PathGeometry();
            rectClipGeometry.Figures.Add(rectClipPathFigure);

            LinearGradientBox gradient = new LinearGradientBox();
            gradient.BoundingBox = boundingBox;
            gradient.Fill = gradientBrush;
            gradient.Clip = rectClipGeometry;

            return gradient;
        }

    }
}
