﻿using System.Collections.Generic;
using System.IO;
using System.Windows.Media.Imaging;
using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class ImageSourceCache : IShapeSourceFactory<FloatingImage>
    {
        private readonly Dictionary<int, BitmapShapeSource> imageSourceCache;

        public ImageSourceCache()
        {
            this.imageSourceCache = new Dictionary<int, BitmapShapeSource>();
        }
        
        public IShapeSource GetShapeSource(FloatingImage shape)
        {
            int resourceId = ((IResource)shape.ImageSource).Id;
            if (!this.imageSourceCache.ContainsKey(resourceId))
            {
                MemoryStream stream = new MemoryStream(shape.Image.ImageSource.Data);
                BitmapSource bitmapImage = SpreadsheetHelper.GetImageSourceOrNoImagePlaceHolder(stream);
                this.imageSourceCache.Add(resourceId, new BitmapShapeSource(bitmapImage));
            }

            return this.imageSourceCache[resourceId];
        }
    }
}
