﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class LayerContainerManager
    {
        private readonly Dictionary<ViewportPaneType, List<IRenderable>> viewportPaneTypeToIRenderable;

        public LayerContainerManager()
        {
            this.viewportPaneTypeToIRenderable = new Dictionary<ViewportPaneType, List<IRenderable>>();

            foreach (ViewportPaneType viewportPaneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                this.viewportPaneTypeToIRenderable[viewportPaneType] = new List<IRenderable>();
            }
        }

        public IEnumerable<IRenderable> GetElementsForViewportPane(ViewportPaneType viewportPaneType)
        {
            return this.viewportPaneTypeToIRenderable[viewportPaneType];
        }

        public void Add(IRenderable renderable, ViewportPaneType paneType)
        {
            this.viewportPaneTypeToIRenderable[paneType].Add(renderable);
        }

        public void Clear()
        {
            foreach (ViewportPaneType viewportPaneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                this.viewportPaneTypeToIRenderable[viewportPaneType].Clear();
            }
        }
    }
}
