﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal static class PatternsRenderHelper
    {
        private static readonly Dictionary<PatternType, int[,]> patterns;

        static PatternsRenderHelper()
        {
            PatternsRenderHelper.patterns = new Dictionary<PatternType, int[,]>();

            PatternsRenderHelper.patterns.Add(PatternType.Gray6Percent, new int[,]
                {
                    { 0, 0, 0, 0, 1, 0, 0, 0 },
                    { 0, 0, 0, 0, 0, 0, 0, 0 },
                    { 1, 0, 0, 0, 0, 0, 0, 0 },
                    { 0, 0, 0, 0, 0, 0, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.Gray12Percent, new int[,]
                {
                    { 0, 0, 1, 0 },
                    { 0, 0, 0, 0 },
                    { 1, 0, 0, 0 },
                    { 0, 0, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.Gray25Percent, new int[,]
                {
                    { 0, 0, 1, 0 },
                    { 1, 0, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.Gray50Percent, new int[,]
                {
                    { 1, 0 },
                    { 0, 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.Gray75Percent, new int[,]
                {
                    { 1, 1, 1, 0 },
                    { 1, 0, 1, 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.HorizontalStripe, new int[,]
                {
                    { 1 },
                    { 0 },
                    { 0 },
                    { 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.VerticalStripe, new int[,]
                {
                    { 1, 1, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.ReverseDiagonalStripe, new int[,]
                {
                    { 0, 0, 1, 1 },
                    { 1, 0, 0, 1 },
                    { 1, 1, 0, 0 },
                    { 0, 1, 1, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.DiagonalStripe, new int[,]
                {
                    { 0, 0, 1, 1 },
                    { 0, 1, 1, 0 },
                    { 1, 1, 0, 0 },
                    { 1, 0, 0, 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.DiagonalCrosshatch, new int[,]
                {
                    { 0, 0, 1, 1 },
                    { 0, 0, 1, 1 },
                    { 1, 1, 0, 0 },
                    { 1, 1, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThickDiagonalCrosshatch, new int[,]
                {
                    { 0, 0, 1, 1 },
                    { 1, 1, 1, 1 },
                    { 1, 1, 0, 0 },
                    { 1, 1, 1, 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinHorizontalStripe, new int[,]
                {
                    { 0 },
                    { 0 },
                    { 0 },
                    { 1 },
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinVerticalStripe, new int[,]
                {
                    { 1, 0, 0, 0 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinReverseDiagonalStripe, new int[,]
                {
                    { 0, 0, 1, 0 },
                    { 0, 0, 0, 1 },
                    { 1, 0, 0, 0 },
                    { 0, 1, 0, 0 },
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinDiagonalStripe, new int[,]
                {
                    { 0, 0, 0, 1 },
                    { 0, 0, 1, 0 },
                    { 0, 1, 0, 0 },
                    { 1, 0, 0, 0 },
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinHorizontalCrossHatch, new int[,]
                {
                    { 0, 1, 0, 0 },
                    { 0, 1, 0, 0 },
                    { 0, 1, 0, 0 },
                    { 1, 1, 1, 1 }
                });

            PatternsRenderHelper.patterns.Add(PatternType.ThinDiagonalCrosshatch, new int[,]
                {
                    { 1, 0, 1, 0 },
                    { 0, 0, 0, 1 },
                    { 1, 0, 1, 0 },
                    { 0, 1, 0, 0 }
                });
        }

        public static int[,] GetPattern(PatternType patternType)
        {
            if (patternType == PatternType.Solid)
            {
                throw new ArgumentException("Pattern type cannot be Solid!");
            }

            return PatternsRenderHelper.patterns[patternType];
        }
    }
}
