﻿using System;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class RowHeadingMarginLayer : RowBasedLayerBase
    {
        private readonly IRenderer<RowColumnHeadingRenderable<RowLayoutBox>> rowHeadingRenderer;

        public RowHeadingMarginLayer(IRenderer<RowColumnHeadingRenderable<RowLayoutBox>> rowHeadingRenderer)
        {
            this.rowHeadingRenderer = rowHeadingRenderer;
        }

        public override string Name
        {
            get
            {
                return WorksheetPredefinedLayers.RowHeadingMargin;
            }
        }

        public double MaxWidth { get; private set; }

        internal IRenderer<RowColumnHeadingRenderable<RowLayoutBox>> RowHeadingRenderer
        {
            get
            {
                return this.rowHeadingRenderer;
            }
        }

        internal static bool ShouldCreateRowHeadingElementForBox(ViewportPaneType viewportPaneType, WorksheetRenderUpdateContext updateContext)
        {
            SheetViewport sheetViewpot = updateContext.SheetViewport;
            if (sheetViewpot[ViewportPaneType.Fixed].IsEmpty &&
                sheetViewpot[ViewportPaneType.HorizontalScrollable].IsEmpty &&
                !sheetViewpot[ViewportPaneType.VerticalScrollable].IsEmpty &&
                viewportPaneType == ViewportPaneType.Scrollable)
            {
                return false;
            }

            if ((!sheetViewpot[ViewportPaneType.Fixed].IsEmpty &&
                 !sheetViewpot[ViewportPaneType.Scrollable].IsEmpty) &&
                (viewportPaneType == ViewportPaneType.HorizontalScrollable || viewportPaneType == ViewportPaneType.Scrollable))
            {
                return false;
            }

            return true;
        }

        protected override void UpdateRenderOverride(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            this.MaxWidth = SpreadsheetDefaultValues.RowColumnHeadingMinimumSize.Width;
            base.UpdateRenderOverride(worksheetUpdateContext);
        }

        protected override IRenderable CreateRenderableElementForBox(ViewportPaneType viewportPaneType, RowLayoutBox box, WorksheetRenderUpdateContext updateContext)
        {
            if (!RowHeadingMarginLayer.ShouldCreateRowHeadingElementForBox(viewportPaneType, updateContext))
            {
                return null;
            }

            CellRange visibleRange = updateContext.SheetViewport[viewportPaneType].VisibleRange;
            Size rowHeadingSize = updateContext.Layout.GetRowHeadingSize(visibleRange, box.RowIndex);
            this.MaxWidth = Math.Max(this.MaxWidth, rowHeadingSize.Width);
            Worksheet worksheet = updateContext.Worksheet;

            return new RowColumnHeadingRenderable<RowLayoutBox>()
            {
                LayoutBox = box,
                Text = updateContext.Worksheet.HeaderNameRenderingConverter.ConvertRowIndexToName(new HeaderNameRenderingConverterContext(worksheet, visibleRange), box.RowIndex),
                Rectangle = new Rect(new Point(0, box.Top), rowHeadingSize)
            };
        }

        protected override void TranslateAndScale(WorksheetRenderUpdateContext worksheetUpdateContext)
        {        
            foreach (ViewportPaneType paneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                foreach (IRenderable renderable in this.ContainerManager.GetElementsForViewportPane(paneType))
                {
                    RowColumnHeadingRenderable<RowLayoutBox> rowHeading = (RowColumnHeadingRenderable<RowLayoutBox>)renderable;
                    Rect translatedRect = this.Translate(rowHeading.Rectangle, paneType, worksheetUpdateContext);
                    rowHeading.Rectangle = new Rect(0, translatedRect.Top, this.MaxWidth, translatedRect.Height);
                    rowHeading.ScaleFactor = worksheetUpdateContext.ScaleFactor;

                    this.rowHeadingRenderer.Render(rowHeading, paneType);
                }
            }
        }
    }
}
