﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class ShapeRenderableFactory
    {
        private readonly ShapeSourceFactory shapeSourceFactory;

        public ShapeRenderableFactory(ShapeSourceFactory shapeSourceFactory)
        {
            this.shapeSourceFactory = shapeSourceFactory;
        }

        public ShapeRenderable GetShapeRenderableFromFloatingShape(ShapeCollection shapes, FloatingShapeBase shape)
        {
            FloatingImage picture = shape as FloatingImage;
#if !SILVERLIGHT
            FloatingChartShape chartShape = shape as FloatingChartShape;
#endif
            ShapeRenderable resultRenderable = null;

            if (picture != null)
            {
                resultRenderable = new ImageRenderable()
                {
                    Shape = picture,
                    ShapeSource = this.shapeSourceFactory.GetImageSource(picture),
                    ZIndex = shapes.GetZIndexById(shape.Id)
                };
            }
#if !SILVERLIGHT
            else if (chartShape != null)
            {
                resultRenderable = new ChartRenderable()
                {
                    Shape = chartShape,
                    ShapeSource = this.shapeSourceFactory.GetChartSource(chartShape),
                    ZIndex = shapes.GetZIndexById(shape.Id)
                };
            }
#endif
            else
            {
                throw new NotSupportedException();
            }

            return resultRenderable;
        }
    }
}
