﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Maths;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Layout.Layers
{
    internal class ShapesLayer : WorksheetLayerBase
    {
        private static readonly Point ShapesRenderTransformOrigin = new Point(0.5, 0.5);
        private readonly IRenderer<ShapeRenderable> shapeRenderer;
        private readonly ShapeRenderableFactory shapeRenderableFactory;

        public ShapesLayer(IRenderer<ShapeRenderable> shapeRenderer, ShapeSourceFactory cache)
        {
            this.shapeRenderer = shapeRenderer;
            this.shapeRenderableFactory = new ShapeRenderableFactory(cache);
        }

        public override string Name
        {
            get
            {
                return WorksheetPredefinedLayers.Shapes;
            }
        }

        internal IRenderer<ShapeRenderable> ShapeRenderer
        {
            get
            {
                return this.shapeRenderer;
            }
        }

        protected override void UpdateRenderOverride(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            foreach (FloatingShapeBase shape in worksheetUpdateContext.Shapes)
            {
                ShapeLayoutBox shapeLayoutBox = worksheetUpdateContext.Layout.GetShapeLayoutBox(shape.Shape.Id);

                foreach (var pane in worksheetUpdateContext.SheetViewport.ViewportPanes)
                {
                    if (pane.Rect.IntersectsWith(shapeLayoutBox.BoundingRectangle))
                    {
                        ShapeRenderable renderable = this.shapeRenderableFactory.GetShapeRenderableFromFloatingShape(worksheetUpdateContext.Worksheet.Shapes, shape);

                        renderable.ShapeSource.IsLocked = true;
                        this.ContainerManager.Add(renderable, pane.ViewportPaneType);
                    }
                }
            }
        }

        protected override void TranslateAndScale(WorksheetRenderUpdateContext worksheetUpdateContext)
        {
            foreach (ViewportPane pane in worksheetUpdateContext.SheetViewport.ViewportPanes)
            {
                foreach (ShapeRenderable shape in this.ContainerManager.GetElementsForViewportPane(pane.ViewportPaneType).OrderBy(x => ((ShapeRenderable)x).ZIndex))
                {
                    Point shapeTopleft = ShapesResizeHelper.GetShapeTopLeftConsideringAdjustmentForRotation(shape.ShapeBase, worksheetUpdateContext.Layout);
                    shape.TopLeft = base.TranslateAndScale(shapeTopleft, pane.ViewportPaneType, worksheetUpdateContext);

                    shape.Height = shape.ShapeBase.Height * worksheetUpdateContext.ScaleFactor.Width;
                    shape.Width = shape.ShapeBase.Width * worksheetUpdateContext.ScaleFactor.Height;

                    ImageRenderable image = shape as ImageRenderable;

                    if (image != null)
                    {
                        image.RenderTransformOrigin = ShapesLayer.ShapesRenderTransformOrigin;
                        image.RenderTransform = GetImageRenderTransform(image.Shape);
                    }

                    this.ShapeRenderer.Render(shape, pane.ViewportPaneType);
                }
            }
        }

        protected override void Clear()
        {
            foreach (ViewportPaneType paneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                var elements = this.ContainerManager.GetElementsForViewportPane(paneType);

                foreach (var element in elements)
                {
                    ShapeRenderable shape = element as ShapeRenderable;

                    if (shape != null)
                    {
                        shape.ShapeSource.IsLocked = false;
                    }
                }
            }

            base.Clear();
        }

        private static Transform GetImageRenderTransform(FloatingShapeBase shape)
        {
            ScaleTransform flipTransform = new ScaleTransform();
            flipTransform.ScaleX = shape.IsHorizontallyFlipped ? -1 : 1;
            flipTransform.ScaleY = shape.IsVerticallyFlipped ? -1 : 1;

            RotateTransform rotateTransform = new RotateTransform();
            rotateTransform.Angle = shape.RotationAngle;

            TransformGroup transformGroup = new TransformGroup();
            transformGroup.Children.Add(flipTransform);
            transformGroup.Children.Add(rotateTransform);

            return transformGroup;
        }
    }
}
