﻿using System;
using System.Windows;

namespace Telerik.Windows.Documents.Spreadsheet.Layout
{
    /// <summary>
    /// Represents a layout box element.
    /// </summary>
    public abstract class LayoutBox
    {
        #region Fields

        private double top;
        private double left;
		private double width;
		private double height;
        private Rect? boundingRectangle;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the position of the box according to the x-axis.
        /// </summary>
        /// <value>The position of the box according to the x-axis.</value>
        public double Left
        {
            get
            {
                return this.left;
            }
			//internal set
			//{
			//	this.left = value;
            //    this.InvalidateBoundingRectangle();
			//}
        }

        /// <summary>
        /// Gets the position of the box according to the y-axis.
        /// </summary>
        /// <value>The position of the box according to the y-axis.</value>
        public double Top
        {
            get
            {
                return this.top;
            }
			//internal set
			//{
			//	this.top = value;
            //    this.InvalidateBoundingRectangle();
			//}
        }

        /// <summary>
        /// Gets the width of the box.
        /// </summary>
        /// <value>The width of the box.</value>
        public double Width
        {
			get
			{
				return this.width;
			}
		}

        /// <summary>
        /// Gets the height of the box.
        /// </summary>
        /// <value>The height of the box.</value>
        public double Height
        {
			get
			{
				return this.height;
			}
		}
        
        /// <summary>
        /// Gets or sets the size of the box.
        /// </summary>
        /// <value>The size of the box.</value>
        public Size Size
        {
            get
            {
                return new Size(this.Width, this.Height);
            }
            //protected set
            //{
            //    this.width = value.Width;
            //    this.height = value.Height;

            //    // TODO: Niki, review this. Kamen!
            //    this.InvalidateBoundingRectangle();
            //}
        }

        /// <summary>
        /// Gets the bounding rectangle.
        /// </summary>
        /// <value>The bounding rectangle.</value>
        public Rect BoundingRectangle
        {
            get
            {
                if (!this.boundingRectangle.HasValue)
                {
                    this.boundingRectangle = new Rect(Math.Round(this.Left, 3), Math.Round(this.Top, 3), Math.Round(this.Width, 3), Math.Round(this.Height, 3));
                }

                return this.boundingRectangle.Value;
            }
        }

		//public bool IsMeasureValid { get; private set; }
		//public bool IsArrangeValid { get; private set; }

        #endregion


        #region Constructors

        internal LayoutBox(Rect rect)
        {
			this.left = rect.Left;
			this.top = rect.Top;
			this.width = rect.Width;
			this.height = rect.Height;
        }

        #endregion


        #region Methods

        //private void InvalidateBoundingRectangle()
        //{
        //    this.boundingRectangle = null;
        //}

		//public override bool Equals(object obj)
		//{
		//    LayoutBox other = obj as LayoutBox;
		//    if (other == null)
		//    {
		//        return false;
		//    }

		//    return this.BoundingRectangle.Equals(other.BoundingRectangle);
		//}

		//public override int GetHashCode()
		//{
		//    return this.BoundingRectangle.GetHashCode();
		//}

		//internal virtual void InvalidateLayout()
		//{
		//    this.InvalidateMeasure();
		//    this.InvalidateArrange();

		//    this.OnLayoutInvalidated();
		//}

		//public event EventHandler LayoutInvalidated;
		//protected virtual void OnLayoutInvalidated()
		//{
		//    if (this.LayoutInvalidated != null)
		//    {
		//        this.LayoutInvalidated(this, EventArgs.Empty);
		//    }
		//}

		//private void InvalidateMeasure()
		//{
		//    this.IsMeasureValid = false;
		//}

		//private void InvalidateArrange()
		//{
		//    this.IsArrangeValid = false;
		//}

		//internal void Measure()
		//{
		//    if (!this.IsMeasureValid)
		//    {
		//        this.MeasureOverride();
		//        this.IsMeasureValid = true;
		//    }
		//}

		//internal void Arrange(double top)
		//{
		//    this.top = top;
		//    if (!this.IsArrangeValid)
		//    {
		//        this.ArrangeOverride();
		//        this.IsArrangeValid = true;
		//    }
		//}

		//protected abstract void MeasureOverride();
		//protected abstract void ArrangeOverride();

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return String.Format("BoundingRectangle:{0}", this.BoundingRectangle);
        }

        #endregion
    }
}