﻿using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Layout
{
    /// <summary>
    /// Represents a row layout box.
    /// </summary>
	public class RowLayoutBox : LayoutBox
	{
		#region Fields

		private readonly int rowIndex;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the index of the row.
        /// </summary>
        /// <value>The index of the row.</value>
        public int RowIndex
        {
			get
			{
				return this.rowIndex;
			}
		}

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RowLayoutBox" /> class.
        /// </summary>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="rect">The rectangle.</param>
        public RowLayoutBox(int rowIndex, Rect rect)
            : base(rect)
		{
			Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
			this.rowIndex = rowIndex;
		}

		#endregion


		#region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
			RowLayoutBox other = obj as RowLayoutBox;
			if (other == null)
			{
				return false;
			}

			return this.RowIndex == other.RowIndex;
		}

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
			return this.rowIndex;
		}

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return string.Format("RowLayoutBox {0}: {1}", this.RowIndex, base.ToString());
        }

		#endregion
	}
}