namespace Telerik.Windows.Documents.Spreadsheet.Maths
{
    /// <summary>
    /// Enumeration of supported volume units.
    /// </summary>
    internal enum VolumeUnit
    {
        /// <summary>
        ///  Teaspoon "tsp" 
        /// </summary>
        Teaspoon,
        /// <summary>
        /// Modern teaspoon "tspm" 
        /// </summary>
        ModernTeaspoon,
        /// <summary>
        /// Tablespoon "tbs" 
        /// </summary>
        TableSpoon,
        /// <summary>
        /// Fluid ounce "oz" 
        /// </summary>
        FluidOunce,
        /// <summary>
        /// Cup "cup" 
        /// </summary>
        Cup,
        /// <summary>
        /// U.S. pint "pt" (or "us_pt") 
        /// </summary>
        USPint,
        /// <summary>
        /// U.K. pint "uk_pt" 
        /// </summary>
        UKPint,
        /// <summary>
        /// Quart "qt" 
        /// </summary>
        Quart,
        /// <summary>
        /// Imperial quart (U.K.) "uk_qt" 
        /// </summary>
        ImperialQuart,
        /// <summary>
        /// Gallon "gal" 
        /// </summary>
        Gallon,
        /// <summary>
        /// Imperial gallon (U.K.) "uk_gal" 
        /// </summary>
        ImperialGallon,
        /// <summary>
        /// Liter "l" or "L" ("lt") 
        /// </summary>
        Liter,
        /// <summary>
        /// Cubic angstrom "ang3" or "ang^3" 
        /// </summary>
        CubicAngstrom,
        /// <summary>
        /// U.S. oil barrel "barrel" 
        /// </summary>
        USOilBarrel,
        /// <summary>
        /// U.S. bushel "bushel" 
        /// </summary>
        USBushel,
        /// <summary>
        /// Cubic feet "ft3" or "ft^3" 
        /// </summary>
        CubicFeet,
        /// <summary>
        /// Cubic inch "in3" or "in^3" 
        /// </summary>
        CubicInch,
        /// <summary>
        /// Cubic light-year "ly3" or "ly^3"
        /// </summary>
        CubicLightYear,
        /// <summary>
        /// Cubic meter "m3" or "m^3" 
        /// </summary>
        CubicMeter,
        /// <summary>
        /// Cubic Mile "mi3" or "mi^3" 
        /// </summary>
        CubicMile,
        /// <summary>
        /// Cubic yard "yd3" or "yd^3" 
        /// </summary>
        CubicYard,
        /// <summary>
        /// Cubic nautical mile "Nmi3" or "Nmi^3" 
        /// </summary>
        CubicNauticalMile,
        /// <summary>
        /// Cubic Pica "Picapt3", "Picapt^3", "Pica3" or "Pica^3" 
        /// </summary>
        CubicPica,
        /// <summary>
        /// Gross Registered Ton "GRT" ("regton") 
        /// </summary>
        GrossRegisteredTon,
        /// <summary>
        /// Measurement ton (freight ton) "MTON" 
        /// </summary>
        MeasurementTon

    }
}