﻿using System.Windows;

namespace Telerik.Windows.Documents.Spreadsheet.Maths
{
	/// <summary>
	/// A vector structure implementation.
	/// </summary>
    internal struct Vector
	{
		public double X { get; set; }
		public double Y { get; set; }

		#region Constructor
		public Vector(double x, double y)
			: this()
		{
			X = x;
			Y = y;
		}

		public Vector(Point p)
			: this()
		{
			X = p.X;
			Y = p.Y;
		}
		#endregion

		public static Vector operator -(Vector v)
		{
			return new Vector(-v.X, -v.Y);
		}

		public static Point operator -(Point p, Vector v)
		{
			return new Point(p.X - v.X, p.Y - v.Y);
		}

		public static Point operator +(Point p, Vector v)
		{
			return new Point(p.X + v.X, p.Y + v.Y);
		}

		public static Vector operator *(Vector v, double d)
		{
			return new Vector(v.X * d, v.Y * d);
		}

		public static Vector operator /(Vector v, double d)
		{
			return new Vector(v.X / d, v.Y / d);
		}

		public override string ToString()
		{
			return "{" + X + ", " + Y + "}";
		}
		public double Length
		{
			get { return System.Math.Sqrt(X * X + Y * Y); }
		}
		public static Vector Empty
		{
			get
			{
				return new Vector(0, 0);
			}
		}
		public void Normalize()
		{
			var u = this.ToUnit();
			X = u.X;
			Y = u.Y;
		}

		public Vector ToUnit()
		{
			if (Length == 0) return Empty;
			return this / Length;
		}
	}
}