﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Maths.Number
{
    internal class FiniteInterval<T> 
        where T : IComparable
    {
        public T Start { get; private set; }
        public T End { get; private set; }
        public bool IsStartOpened { get; private set; }
        public bool IsEndOpened { get; private set; }

        public FiniteInterval(T start, T end, bool isStartOpened = false, bool isEndOpened = false)
        {
            if (start.CompareTo(end) > 0)
            {
                throw new ArgumentException("Interval start value must be less than or equal to interval end value!");
            }

            this.Start = start;
            this.IsStartOpened = isStartOpened;
            this.End = end;
            this.IsEndOpened = isEndOpened;
        }

        public bool Contains(T value)
        {
            return this.CompareTo(value) == 0;
        }

        public int CompareTo(T value)
        {
            bool isValueSmallerThanInterval = this.IsStartOpened ? value.CompareTo(this.Start) <= 0 : value.CompareTo(this.Start) < 0;
            if (isValueSmallerThanInterval)
            {
                return 1;
            }

            bool isValueBiggerThanInterval = this.IsEndOpened ? value.CompareTo(this.End) >= 0 : value.CompareTo(this.End) > 0;
            if (isValueBiggerThanInterval)
            {
                return -1;
            }

            return 0;
        }

        public override string ToString()
        {
            Func<bool, bool, string> getEndingBracket = (isOpened, isLeft) =>
            {
                if (isOpened)
                {
                    return isLeft ? "(" : ")";
                }
                else
                {
                    return isLeft ? "[" : "]";
                }
            };

            return String.Format("{0}{1};{2}{3}",
                getEndingBracket(this.IsStartOpened, true),
                this.Start,
                this.End,
                getEndingBracket(this.IsEndOpened, false));
        }

        public override bool Equals(object obj)
        {
            FiniteInterval<T> other = obj as FiniteInterval<T>;

            if (other == null)
            {
                return false;
            }

            return this.Start.Equals(other.Start) &&
                this.End.Equals(other.End) &&
                this.IsStartOpened.Equals(other.IsStartOpened) &&
                this.IsEndOpened.Equals(other.IsEndOpened);
        }

        public override int GetHashCode()
        {
            return this.Start.GetHashCode() ^
                this.End.GetHashCode() ^
                this.IsStartOpened.GetHashCode() ^
                this.IsEndOpened.GetHashCode();
        }
    }
}
