﻿using System;
using System.Windows;
using Telerik.Windows.Documents.Core.TextMeasurer;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Measurement
{
    internal static class RadTextMeasurer
    {
        private static readonly Documents.Core.TextMeasurer.RadTextMeasurer textMeasurer;
     
        static RadTextMeasurer()
        {
            textMeasurer = new Documents.Core.TextMeasurer.RadTextMeasurer();
        }

        public static Size MeasureMultiline(string text, FontProperties fontProperties, double? wrappingWidth)
        {
            Guard.ThrowExceptionIfNull(fontProperties, "fontProperties");

            if (wrappingWidth.HasValue)
            {
                Guard.ThrowExceptionIfLessThan(0, wrappingWidth.Value, "wrappingWidth");
            }

            double maxWidth = 0;
            double totalHeight = 0;

            text = TextHelper.SanitizeNewLines(text);
            foreach (var line in text.Split(LineBreak.NewLine))
            {
                var measurementInfo = Measure(line, fontProperties,  wrappingWidth);
                maxWidth = Math.Max(maxWidth, measurementInfo.Size.Width);
                totalHeight += measurementInfo.Size.Height;
            }

            return new Size(maxWidth, totalHeight);
        }

        public static TextMeasurementInfo Measure(string text, FontProperties fontProperties, double? wrappingWidth = null)
        {
            Guard.ThrowExceptionIfNull(fontProperties, "fontProperties");

            if (wrappingWidth.HasValue)
            {
                Guard.ThrowExceptionIfLessThan(0, wrappingWidth.Value, "wrappingWidth");
            }

            var fp = new Documents.Core.Fonts.FontProperties(fontProperties.FontFamily, fontProperties.FontStyle, fontProperties.FontWeight);
            var textProperties = new TextProperties(text, fontProperties.FontSize);
            if (wrappingWidth.HasValue)
            {
                return Documents.Core.TextMeasurer.RadTextMeasurer.MeasureTextWithWrapping(textProperties, fp, wrappingWidth.Value);
            }
            else
            {
                return textMeasurer.MeasureText(textProperties , fp);
            }
        }
    }
}