﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    internal class CellRangeEnumerator : IEnumerator<CellIndex>
    {
        #region Fields

        private readonly CellRange cellRange;
        private readonly CellOrientation orientation;
        private readonly bool isReversed;

        private CellIndex current;

        #endregion


        #region Properties

        public CellIndex Current
        {
            get
            {
                Guard.ThrowExceptionIfNull(this.current, "current");
                return this.current;
            }
        }

        object System.Collections.IEnumerator.Current
        {
            get
            {
                return ((IEnumerator<CellIndex>)this.Current);
            }
        }

        public bool IsAtRowColumnStart
        {
            get
            {
                if (this.current == null)
                {
                    return false;
                }

                if (this.orientation == CellOrientation.Horizontal)
                {
                    if (!this.isReversed)
                    {
                        return this.current.ColumnIndex == this.cellRange.FromIndex.ColumnIndex;
                    }
                    else
                    {
                        return this.current.ColumnIndex == this.cellRange.ToIndex.ColumnIndex;
                    }
                }
                else
                {
                    if (!this.isReversed)
                    {
                        return this.current.RowIndex == this.cellRange.FromIndex.RowIndex;
                    }
                    else
                    {
                        return this.current.RowIndex == this.cellRange.ToIndex.RowIndex;
                    }
                }
            }
        }

        public int RowColumnLength
        {
            get
            {
                if (this.orientation == CellOrientation.Horizontal)
                {
                    return this.cellRange.ColumnCount;
                }

                return this.cellRange.RowCount;
            }
        }

        #endregion


        #region Constructors

        public CellRangeEnumerator(CellRange cellRange, CellOrientation orientation, bool isReversed)
        {
            Guard.ThrowExceptionIfNull(cellRange, "cellRange");

            this.cellRange = cellRange;
            this.orientation = orientation;
            this.isReversed = isReversed;
        }

        #endregion


        #region Methods

        public void Reset()
        {
            this.current = null;
        }

        public bool MoveNext()
        {
            if (this.current == null)
            {
                if (!isReversed)
                {
                    this.current = cellRange.FromIndex;
                }
                else
                {
                    if (this.orientation == CellOrientation.Horizontal)
                    {
                        this.current = new CellIndex(cellRange.FromIndex.RowIndex, cellRange.ToIndex.ColumnIndex);
                    }
                    else
                    {
                        this.current = new CellIndex(cellRange.ToIndex.RowIndex, cellRange.FromIndex.ColumnIndex);
                    }
                }

                return true;
            }

            if (this.orientation == CellOrientation.Horizontal)
            {
                if (!this.isReversed)
                {
                    if (this.current.ColumnIndex + 1 <= this.cellRange.ToIndex.ColumnIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex, this.current.ColumnIndex + 1);
                        return true;
                    }
                    else if (this.current.RowIndex + 1 <= this.cellRange.ToIndex.RowIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex + 1, this.cellRange.FromIndex.ColumnIndex);
                        return true;
                    }
                }
                else
                {
                    if (this.current.ColumnIndex - 1 >= this.cellRange.FromIndex.ColumnIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex, this.current.ColumnIndex - 1);
                        return true;
                    }
                    else if (this.current.RowIndex + 1 <= this.cellRange.ToIndex.RowIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex + 1, this.cellRange.ToIndex.ColumnIndex);
                        return true;
                    }
                }
            }
            else
            {
                if (!this.isReversed)
                {
                    if (this.current.RowIndex + 1 <= this.cellRange.ToIndex.RowIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex + 1, this.current.ColumnIndex);
                        return true;
                    }
                    else if (this.current.ColumnIndex + 1 <= this.cellRange.ToIndex.ColumnIndex)
                    {
                        this.current = new CellIndex(this.cellRange.FromIndex.RowIndex, this.current.ColumnIndex + 1);
                        return true;
                    }
                }
                else
                {
                    if (this.current.RowIndex - 1 >= this.cellRange.FromIndex.RowIndex)
                    {
                        this.current = new CellIndex(this.current.RowIndex - 1, this.current.ColumnIndex);
                        return true;
                    }
                    else if (this.current.ColumnIndex + 1 <= this.cellRange.ToIndex.ColumnIndex)
                    {
                        this.current = new CellIndex(this.cellRange.ToIndex.RowIndex, this.current.ColumnIndex + 1);
                        return true;
                    }
                }
            }

            return false;
        }

        public void Dispose()
        {
        }

        #endregion
    }
}