﻿using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    internal class CellRangeFontProperties
    {
        #region Fields

        private readonly bool onlyPropertiesAffectingLayout;
        private readonly ICompressedList<ThemableFontFamily> fontFamilyValues;
        private readonly ICompressedList<double> fontSizeValues;
        private readonly ICompressedList<bool> isBoldValues;
        private readonly ICompressedList<bool> isItalicValues;
        private readonly ICompressedList<UnderlineType> underlineValues;
        private readonly ICompressedList<ThemableColor> foreColorValues;

        #endregion


        #region Constructors

        public CellRangeFontProperties(Cells cells, CellRange cellRange, bool onlyPropertiesAffectingLayout = false)
        {
            this.onlyPropertiesAffectingLayout = onlyPropertiesAffectingLayout;

            this.fontFamilyValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.FontFamilyProperty, cells.Worksheet, cellRange);
            this.fontSizeValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.FontSizeProperty, cells.Worksheet, cellRange);
            this.isBoldValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.IsBoldProperty, cells.Worksheet, cellRange);

            if (!onlyPropertiesAffectingLayout)
            {
                this.isItalicValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.IsItalicProperty, cells.Worksheet, cellRange);
                this.underlineValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.UnderlineProperty, cells.Worksheet, cellRange);
                this.foreColorValues = cells.PropertyBag.GetPropertyValueRespectingStyle(CellPropertyDefinitions.ForeColorProperty, cells.Worksheet, cellRange);
            }
        }

        #endregion


        #region Methods

        public FontProperties GetFontProperties(long index, DocumentTheme theme)
        {
            FontProperties fontProperties = new FontProperties(
                this.fontFamilyValues.GetValue(index).GetActualValue(theme),
                this.fontSizeValues.GetValue(index),
                this.isBoldValues.GetValue(index));

            if (!this.onlyPropertiesAffectingLayout)
            {
                fontProperties.IsItalic = this.isItalicValues.GetValue(index);
                fontProperties.Underline = this.underlineValues.GetValue(index);
                fontProperties.ForeColor = this.foreColorValues.GetValue(index);
            }

            return fontProperties;
        }

        #endregion
    }
}