﻿using System;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents cell range inserted or removed aware ordered collection.
    /// </summary>
    public abstract class CellRangeInsertedOrRemovedAwareOrderedCollection : CellRangeInsertedOrRemovedAwareCollectionBase
    {
        private readonly List<CellRange> rangesList;

        /// <summary>
        /// Initializes a new instance of the <see cref="CellRangeInsertedOrRemovedAwareOrderedCollection" /> class.
        /// </summary>
        /// <param name="cells">The cells.</param>
        protected CellRangeInsertedOrRemovedAwareOrderedCollection(Cells cells) : base(cells)
        {
            this.rangesList = new List<CellRange>();
        }

        /// <summary>
        /// Gets the ranges list.
        /// </summary>
        /// <value>The ranges list.</value>
        protected List<CellRange> RangesList
        {
            get
            {
                return this.rangesList;
            }
        }

        /// <summary>
        /// Gets the cell ranges.
        /// </summary>
        /// <value>The cell ranges.</value>
        protected override ICollection<CellRange> CellRanges
        {
            get 
            {
                return this.RangesList; 
            }
        }

        /// <summary>
        /// Translates the range.
        /// </summary>
        /// <param name="oldRange">The old range.</param>
        /// <param name="newRange">The new range.</param>
        protected override void TranslateRange(CellRange oldRange, CellRange newRange)
        {
            int index = this.RangesList.IndexOf(oldRange);

            if (index > -1)
            {
                this.RemoveCellRangeAt(index);

                if (newRange != null)
                {
                    this.Insert(index, newRange);
                }
            }
        }

        /// <summary>
        /// Removes the cell range at.
        /// </summary>
        /// <param name="index">The index.</param>
        protected virtual void RemoveCellRangeAt(int index)
        {
            this.RangesList.RemoveAt(index);
        }

        /// <summary>
        /// Inserts the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <param name="range">The range.</param>
        protected virtual void Insert(int index, CellRange range)
        {
            this.RangesList.Insert(index, range);
        }
    }
}
