﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    internal static class CellValuesExtensions
    {
        private static int NumberCellValueComparedToTextCellValue = -1;
        private static int NumberCellValueComparedToBooleanCellValue = -1;
        private static int NumberCellValueComparedToEmptyCellValue = -1;
        private static int NumberCellValueComparedToErrorCellValue = -1;

        private static int TextCellValueComparedToBooleanCellValue = -1;
        private static int TextCellValueComparedToEmptyCellValue = -1;
        private static int TextCellValueComparedToErrorCellValue = -1;

        private static int BooleanCellValueComparedToEmptyCellValue = -1;
        private static int BooleanCellValueComparedToErrorCellValue = -1;

        private static int EmptyCellValueComparedToEmptyCellValue = 0;

        private static int EmptyCellValueComparedToErrorCellValue = 1;

        private static int ErrorCellValueComparedToErrorCellValue = 0;

        public static int CompareTo(this ICellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value1 as FormulaCellValue;
            if (formulaValue != null)
            {
                return formulaValue.CompareTo(value2);
            }

            NumberCellValue numberValue = value1 as NumberCellValue;
            if (numberValue != null)
            {
                return numberValue.CompareTo(value2);
            }

            TextCellValue textValue = value1 as TextCellValue;
            if (textValue != null)
            {
                return textValue.CompareTo(value2);
            }

            BooleanCellValue booleanValue = value1 as BooleanCellValue;
            if (booleanValue != null)
            {
                return booleanValue.CompareTo(value2);
            }

            EmptyCellValue emptyCellValue = value1 as EmptyCellValue;
            if (emptyCellValue != null)
            {
                return emptyCellValue.CompareTo(value2);
            }

            ErrorCellValue errorValue = value1 as ErrorCellValue;
            if (errorValue != null)
            {
                return errorValue.CompareTo(value2);
            }

            throw new InvalidOperationException();
        }


        public static int CompareTo(this FormulaCellValue value1, ICellValue value2)
        {
            return value1.GetResultValueAsCellValue().CompareTo(value2);
        }

        public static int CompareTo(this NumberCellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value2 as FormulaCellValue;
            if (formulaValue != null)
            {
                return -1 * formulaValue.CompareTo(value1);
            }

            NumberCellValue numberValue = value2 as NumberCellValue;
            if (numberValue != null)
            {
                return value1.CompareTo(numberValue);
            }
            
            if (value2 is TextCellValue)
            {
                return NumberCellValueComparedToTextCellValue;
            }
            
            if (value2 is BooleanCellValue)
            {
                return NumberCellValueComparedToBooleanCellValue;
            }
            
            if (value2 is EmptyCellValue)
            {
                return NumberCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is ErrorCellValue)
            {
                return NumberCellValueComparedToErrorCellValue;
            }

            throw new InvalidOperationException();
        }

        public static int CompareTo(this TextCellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value2 as FormulaCellValue;
            if (formulaValue != null)
            {
                return -1 * formulaValue.CompareTo(value1);
            }
            
            if (value2 is NumberCellValue)
            {
                return -1 * NumberCellValueComparedToTextCellValue;
            }

            TextCellValue textValue = value2 as TextCellValue;
            if (textValue != null)
            {
                return value1.CompareTo(textValue);
            }
            
            if (value2 is BooleanCellValue)
            {
                return TextCellValueComparedToBooleanCellValue;
            }
            
            if (value2 is EmptyCellValue)
            {
                return TextCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is ErrorCellValue)
            {
                return TextCellValueComparedToErrorCellValue;
            }

            throw new InvalidOperationException();
        }

        public static int CompareTo(this BooleanCellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value2 as FormulaCellValue;
            if (formulaValue != null)
            {
                return -1 * formulaValue.CompareTo(value1);
            }
            
            if (value2 is NumberCellValue)
            {
                return -1 * NumberCellValueComparedToBooleanCellValue;
            }
            
            if (value2 is TextCellValue)
            {
                return -1 * TextCellValueComparedToBooleanCellValue;
            }

            BooleanCellValue booleanValue = value2 as BooleanCellValue;
            if (booleanValue != null)
            {
                return value1.CompareTo(booleanValue);
            }
            
            if (value2 is EmptyCellValue)
            {
                return BooleanCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is ErrorCellValue)
            {
                return BooleanCellValueComparedToErrorCellValue;
            }

            throw new InvalidOperationException();
        }

        public static int CompareTo(this EmptyCellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value2 as FormulaCellValue;
            if (formulaValue != null)
            {
                return -1 * formulaValue.CompareTo(value1);
            }
            
            if (value2 is NumberCellValue)
            {
                return -1 * NumberCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is TextCellValue)
            {
                return -1 * TextCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is BooleanCellValue)
            {
                return -1 * BooleanCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is EmptyCellValue)
            {
                return EmptyCellValueComparedToEmptyCellValue;
            }
            
            if (value2 is ErrorCellValue)
            {
                return EmptyCellValueComparedToErrorCellValue;
            }

            throw new InvalidOperationException();
        }

        public static int CompareTo(this ErrorCellValue value1, ICellValue value2)
        {
            FormulaCellValue formulaValue = value2 as FormulaCellValue;
            if (formulaValue != null)
            {
                return -1 * formulaValue.CompareTo(value1);
            }
            
            if (value2 is NumberCellValue)
            {
                return -1 * NumberCellValueComparedToErrorCellValue;
            }
            
            if (value2 is TextCellValue)
            {
                return -1 * TextCellValueComparedToErrorCellValue;
            }
            
            if (value2 is BooleanCellValue)
            {
                return -1 * BooleanCellValueComparedToErrorCellValue;
            }
            
            if (value2 is EmptyCellValue)
            {
                return -1 * EmptyCellValueComparedToErrorCellValue;
            }
            
            if (value2 is ErrorCellValue)
            {
                return ErrorCellValueComparedToErrorCellValue;
            }

            throw new InvalidOperationException();
        }


        public static int CompareTo(this NumberCellValue value1, NumberCellValue value2)
        {
            return value1.Value.CompareTo(value2.Value);
        }

        public static int CompareTo(this TextCellValue value1, TextCellValue value2)
        {
            return value1.Value.CompareTo(value2.Value);
        }

        public static int CompareTo(this BooleanCellValue value1, BooleanCellValue value2)
        {
            return value1.Value.CompareTo(value2.Value);
        }
    }
}