﻿using Telerik.Windows.Documents.Spreadsheet.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents the value of a cell which has no content.
    /// </summary>
    public class EmptyCellValue : CellValueBase<string>, IExpressionAwareCellValue
    {
        #region Constants

        /// <summary>
        /// Represents Empty cell value.
        /// </summary>
        public static readonly EmptyCellValue EmptyValue = new EmptyCellValue();

        #endregion


        #region Properties

        /// <summary>
        /// Gets the type of the value.
        /// </summary>
        /// <value>The type of the value.</value>
        public override CellValueType ValueType
        {
            get
            {
                return CellValueType.Empty;
            }
        }

        #endregion


        #region Constructors

        private EmptyCellValue()
            : base(string.Empty)
        {
        }

        #endregion


        #region Methods

        /// <summary>
        /// Gets the value as string override.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <returns></returns>
        protected override string GetValueAsStringOverride(CellValueFormat format)
        {
            return string.Empty;
        }

        /// <summary>
        /// Gets the result value as string.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <returns></returns>
        public override string GetResultValueAsString(CellValueFormat format)
        {
            return this.GetValueAsString(format);
        }

        RadExpression IExpressionAwareCellValue.GetCellExpressionOrNull()
        {
            return EmptyExpression.Empty;
        }

        #endregion
    }
}