﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Formatting;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents the value of a cell which contains text.
    /// </summary>
    public class TextCellValue : CellValueBase<string>, IExpressionAwareCellValue
    {
        #region Fields

        private StringExpression expression;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the type of the value.
        /// </summary>
        /// <value>The type of the value.</value>
        public override CellValueType ValueType
        {
			get
			{
				return CellValueType.Text;
			}
		}

        /// <summary>
        /// Gets the raw value.
        /// </summary>
        /// <value>The raw value.</value>
        public override string RawValue
        {
            get
            {
                return this.Value.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
            }
        }

		#endregion


		#region Constructors

        internal TextCellValue(string value)
            : base(value)
        {
        }

		#endregion


        #region Methods

        /// <summary>
        /// Gets the value as string override.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <returns></returns>
        protected override string GetValueAsStringOverride(CellValueFormat format = null)
        {
            return this.RawValue;
        }

        RadExpression IExpressionAwareCellValue.GetCellExpressionOrNull()
        {
            if (this.expression == null)
            {
                this.expression = new StringExpression(this.RawValue);
            }

            return this.expression;
        }

        #endregion
    }
}