﻿using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Charts
{
    internal static class AxisFactory
    {
        public static Axis GetHorizontalAxis(RangeParseResultInfo parseResult, Worksheet worksheet, ChartType[] chartTypes)
        {
            CellRange categoriesRange = parseResult.CategoriesRange;
            bool seriesRangesAreVertical = parseResult.SeriesRangesAreVertical;

            bool categoriesRangeIsSingleRowOrColumn =
                categoriesRange == null ? false :
                seriesRangesAreVertical ? categoriesRange.ColumnCount == 1 : categoriesRange.RowCount == 1;
            bool axisIsDateAxis = false;

            if (categoriesRangeIsSingleRowOrColumn)
            {
                bool thereAreValuesProhibitingDateAxis = false;
                bool atLeasetOneValueIsDate = false;

                for (int row = categoriesRange.FromIndex.RowIndex; row <= categoriesRange.ToIndex.RowIndex; row++)
                {
                    for (int col = categoriesRange.FromIndex.ColumnIndex; col <= categoriesRange.ToIndex.ColumnIndex; col++)
                    {
                        ICellValue value = worksheet.Cells[row, col].GetValue().Value;
                        CellValueFormat format = worksheet.Cells[row, col].GetFormat().Value;

                        bool categoryIsDate = value.ValueType == CellValueType.Number && format.FormatStringInfo.Category == FormatStringCategory.Date;
                        bool categoryAllowsDateAxis = categoryIsDate || value.ValueType == CellValueType.Empty;

                        if (!categoryAllowsDateAxis)
                        {
                            thereAreValuesProhibitingDateAxis = true;
                        }

                        if (categoryIsDate)
                        {
                            atLeasetOneValueIsDate = true;
                        }
                    }
                }

                axisIsDateAxis = !thereAreValuesProhibitingDateAxis && atLeasetOneValueIsDate;
            }

            Axis result;

            if (axisIsDateAxis)
            {
                result = new DateAxis();
            }
            else
            {
                AxesType resultAxesType = AxesType.CategoriesValues;

                for (int i = 0; i < chartTypes.Length; i++)
                {
                    ChartType chartType = chartTypes[i];
                    AxesType axesType = ChartTypeToAxesTypeMapper.GetAxesType(chartType);

                    if (axesType != AxesType.CategoriesValues)
                    {
                        resultAxesType = axesType;
                        break;
                    }
                }

                if (resultAxesType == AxesType.XValuesYValues)
                {
                    result = new ValueAxis();
                }
                else
                {
                    result = new CategoryAxis();
                }
            }

            if (categoriesRange != null)
            {
                CellValueFormat axisFormat = worksheet.Cells[categoriesRange.FromIndex].GetFormat().Value;
                result.NumberFormat = axisFormat.FormatString;
            }

            AddDefaultShapePropertiesToAxis(result);

            return result;
        }

        public static Axis GetVerticalAxis(RangeParseResultInfo parseResult, Worksheet worksheet)
        {
            CellValueFormat format = worksheet.Cells[parseResult.DataRange.FromIndex].GetFormat().Value;

            Axis result = new ValueAxis();
            result.NumberFormat = format.FormatString;

            AddDefaultShapePropertiesToAxis(result);
            AddDefaultGridlinesToAxis(result);

            return result;
        }

        private static void AddDefaultShapePropertiesToAxis(Axis axis)
        {
            axis.Outline.Fill = SpreadsheetDefaultValues.ChartLinesDefaultFill;
            axis.Outline.Width = SpreadsheetDefaultValues.ChartLinesDefaultWidth;
        }

        private static void AddDefaultGridlinesToAxis(Axis axis)
        {
            axis.MajorGridlines.Outline.Fill = SpreadsheetDefaultValues.ChartLinesDefaultFill;
            axis.MajorGridlines.Outline.Width = SpreadsheetDefaultValues.ChartLinesDefaultWidth;
        }

    }
}
