﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Charts
{
    internal static class SeriesGroupFactory
    {
        static Dictionary<ChartType, Func<SeriesGroup>> seriesGroupConstructors;

        static SeriesGroupFactory()
        {
            seriesGroupConstructors = new Dictionary<ChartType, Func<SeriesGroup>>();

            seriesGroupConstructors.Add(ChartType.Bar, () =>
            {
                BarSeriesGroup result = new BarSeriesGroup();
                result.BarDirection = BarDirection.Bar;
                return result;
            });

            seriesGroupConstructors.Add(ChartType.Column, () =>
            {
                BarSeriesGroup result = new BarSeriesGroup();
                result.BarDirection = BarDirection.Column;
                return result;
            });

            seriesGroupConstructors.Add(ChartType.Line, () =>
            {
                return new LineSeriesGroup();
            });

            seriesGroupConstructors.Add(ChartType.Pie, () =>
            {
                return new PieSeriesGroup();
            });

            seriesGroupConstructors.Add(ChartType.Area, () =>
            {
                return new AreaSeriesGroup();
            });

            seriesGroupConstructors.Add(ChartType.Doughnut, () =>
            {
                DoughnutSeriesGroup result = new DoughnutSeriesGroup();
                result.HoleSizePercent = SpreadsheetDefaultValues.DoughnutDefaultHoleSize;
                return result;
            });

            seriesGroupConstructors.Add(ChartType.Scatter, () =>
            {
                return new ScatterSeriesGroup();
            });

            seriesGroupConstructors.Add(ChartType.Bubble, () =>
            {
                return new BubbleSeriesGroup();
            });
        }

        public static SeriesGroup GetSeriesGroup(ChartType chartType)
        {
            if(!seriesGroupConstructors.ContainsKey(chartType))
            {
                throw new NotSupportedException("This chart type is not supported.");
            }

            return seriesGroupConstructors[chartType]();
        }
    }
}
