﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents a base class for copy - paste result.
    /// </summary>
    public class CopyPasteResultBase
    {
        #region Fields
        private readonly bool success;
        private readonly string errorMessage;
        private readonly string errorMessageLocalizationKey;

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether the paste was successful.
        /// </summary>
        /// <value>The value indicating whether the paste was successful.</value>
        public bool Success
        {
            get
            {
                return this.success;
            }
        }

        /// <summary>
        /// Gets the error message.
        /// </summary>
        /// <value>The error message.</value>
        public string ErrorMessage
        {
            get
            {
                return this.errorMessage;
            }
        }

        internal string ErrorMessageLocalizationKey
        {
            get
            {
                return this.errorMessageLocalizationKey;
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CopyPasteResultBase" /> class.
        /// </summary>
        /// <param name="success">The success.</param>
        /// <param name="errorMessage">The error message.</param>
        /// <param name="errorMessageLocalizationKey">The error message localization key.</param>
        protected CopyPasteResultBase(bool success, string errorMessage = "", string errorMessageLocalizationKey = "")
        {
            this.success = success;
            this.errorMessage = errorMessage;
            this.errorMessageLocalizationKey = errorMessageLocalizationKey;
        }
        #endregion
    }
}
