﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents a base class for data validation rule context.
    /// </summary>
    public abstract class DataValidationRuleContextBase
    {
        /// <summary>
        /// Gets or sets a value indicating whether an input message should be shown.
        /// </summary>
        /// <value>The value indicating whether an input message should be shown.</value>
        public bool ShowInputMessage { get; set; }

        /// <summary>
        /// Gets or sets the input message title.
        /// </summary>
        /// <value>The input message title.</value>
        public string InputMessageTitle { get; set; }

        /// <summary>
        /// Gets or sets the content of the input message.
        /// </summary>
        /// <value>The content of the input message.</value>
        public string InputMessageContent { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether an error message should be shown.
        /// </summary>
        /// <value>The value indicating whether an input message should be shown.</value>
        public bool ShowErrorMessage { get; set; }

        /// <summary>
        /// Gets or sets the error style.
        /// </summary>
        /// <value>The error style.</value>
        public ErrorStyle ErrorStyle { get; set; }

        /// <summary>
        /// Gets or sets the error message title.
        /// </summary>
        /// <value>The error message title.</value>
        public string ErrorAlertTitle { get; set; }

        /// <summary>
        /// Gets or sets the content of the error alert.
        /// </summary>
        /// <value>The content of the error alert.</value>
        public string ErrorAlertContent { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="DataValidationRuleContextBase" /> class.
        /// </summary>
        protected DataValidationRuleContextBase()
        {
            this.ShowInputMessage = true;
            this.InputMessageTitle = string.Empty;
            this.InputMessageContent = string.Empty;

            this.ShowErrorMessage = true;
            this.ErrorStyle = ErrorStyle.Stop;
            this.ErrorAlertTitle = string.Empty;
            this.ErrorAlertContent = string.Empty;
        }
    }
}