﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents list data validation rule context.
    /// </summary>
    public class ListDataValidationRuleContext : SingleArgumentDataValidationRuleContext
    {
        /// <summary>
        /// Gets or sets a value indicating if an cell dropdown should be shown.
        /// </summary>
        /// <value>The a value indicating if an cell dropdown should be shown.</value>
        public bool InCellDropdown { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="ListDataValidationRuleContext" /> class.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        public ListDataValidationRuleContext(Worksheet worksheet, int rowIndex, int columnIndex)
            : this(worksheet, new CellIndex(rowIndex, columnIndex))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ListDataValidationRuleContext" /> class.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="cellIndex">Index of the cell.</param>
        public ListDataValidationRuleContext(Worksheet worksheet, CellIndex cellIndex)
            : base(worksheet, cellIndex)
        {
            this.InCellDropdown = true;
        }
    }
}