﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents a single argument data validation rule context.
    /// </summary>
    public class SingleArgumentDataValidationRuleContext : DataValidationRuleContextBase
    {
        private SpreadsheetCultureHelper cultureInfo;

        /// <summary>
        /// Gets or sets a value indicating if blank cells should be ignored.
        /// </summary>
        /// <value>The value indicating if blank cells should be ignored.</value>
        public bool IgnoreBlank { get; set; }

        /// <summary>
        /// Gets or sets the first argument.
        /// </summary>
        /// <value>The first argument.</value>
        public string Argument1 { get; set; }

        /// <summary>
        /// Gets or sets the index of the cell in which the Validation rule is created.
        /// </summary>
        /// <value>The index of the cell.</value>
        public CellIndex CellIndex { get; set; }

        /// <summary>
        /// Gets or sets the worksheet.
        /// </summary>
        /// <value>The worksheet.</value>
        public Worksheet Worksheet { get; set; }

        internal SpreadsheetCultureHelper CultureInfo
        {
            get
            {
                return this.cultureInfo;
            }
            set
            {
                this.cultureInfo = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SingleArgumentDataValidationRuleContext" /> class.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        public SingleArgumentDataValidationRuleContext(Worksheet worksheet, int rowIndex, int columnIndex) 
            : this(worksheet, new CellIndex(rowIndex, columnIndex))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SingleArgumentDataValidationRuleContext" /> class.
        /// </summary>
        public SingleArgumentDataValidationRuleContext(Worksheet worksheet, CellIndex cellIndex)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfNull(cellIndex, "cellIndex");

            this.IgnoreBlank = true;
            this.Worksheet = worksheet;
            this.CellIndex = cellIndex;

            this.cultureInfo = FormatHelper.DefaultSpreadsheetCulture;
        }
    }
}