﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents an interface for data validation rule.
    /// </summary>
    public interface IDataValidationRule
    {
        /// <summary>
        /// Gets or sets a value indicating whether an input message should be shown.
        /// </summary>
        /// <value>The value indicating whether an input message should be shown.</value>
        bool ShowInputMessage { get; }

        /// <summary>
        /// Gets or sets the input message title.
        /// </summary>
        /// <value>The input message title.</value>
        string InputMessageTitle { get; }

        /// <summary>
        /// Gets or sets the content of the input message.
        /// </summary>
        /// <value>The content of the input message.</value>
        string InputMessageContent { get; }

        /// <summary>
        /// Gets or sets a value indicating whether an error message should be shown.
        /// </summary>
        /// <value>The value indicating whether an input message should be shown.</value>
        bool ShowErrorMessage { get; }

        /// <summary>
        /// Gets or sets the error style.
        /// </summary>
        /// <value>The error style.</value>
        ErrorStyle ErrorStyle { get; }

        /// <summary>
        /// Gets the error alert title.
        /// </summary>
        /// <value>The error alert title.</value>
        string ErrorAlertTitle { get; }

        /// <summary>
        /// Gets the content of the error alert.
        /// </summary>
        /// <value>The content of the error alert.</value>
        string ErrorAlertContent { get; }

        /// <summary>
        /// Evaluates the data validation rule.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        /// <param name="cellValue">The cell value.</param>
        /// <returns>Retrurns True if the rule is satisfied, else returns False.</returns>
        bool Evaluate(Worksheet worksheet, int rowIndex, int columnIndex, ICellValue cellValue);
    }
}