﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents a custom data validation rule.
    /// </summary>
    public sealed class CustomDataValidationRule : SingleArgumentDataValidationRuleBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="CustomDataValidationRule" /> class.
        /// </summary>
        /// <param name="context">The context.</param>
        public CustomDataValidationRule(SingleArgumentDataValidationRuleContext context)
            : base(context)
        {
        }

        /// <summary>
        /// Evaluates the data validation rule.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        /// <param name="cellValue">The cell value.</param>
        /// <returns>Retrurns True if the rule is satisfied, else returns False.</returns>
        protected override bool EvaluateOverride(Worksheet worksheet, int rowIndex, int columnIndex, ICellValue cellValue)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);
            Guard.ThrowExceptionIfNull(cellValue, "cellValue");

            if (this.Argument1 is EmptyCellValue)
            {
                return true;
            }

            if (this.ShouldIgnore(cellValue))
            {
                return true;
            }

            bool result = false;
            ICellValue argumentCellValue = this.GetTranslatedArgument(this.Argument1, worksheet, rowIndex, columnIndex);

            string cellValueResult = argumentCellValue.GetResultValueAsString(CellValueFormat.GeneralFormat);

            if (!string.IsNullOrEmpty(cellValueResult))
            {
                bool.TryParse(cellValueResult, out result);
            }

            return result;
        }
    }
}