﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    internal static class DataValidationRulesExtensionMethods
    {
        public static bool Evaluate(this IDataValidationRule rule, Worksheet worksheet, int rowIndex, int columnIndex)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfNull(rule, "rule");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            long index = CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex);

            ICellValue cellValue = worksheet.Cells.PropertyBag.GetPropertyValue(CellPropertyDefinitions.ValueProperty, index);

            return rule.Evaluate(worksheet, rowIndex, columnIndex, cellValue);
        }

        public static bool Evaluate(this IDataValidationRule rule, Worksheet worksheet, CellIndex cellIndex, string cellValue)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfNull(rule, "rule");
            Guard.ThrowExceptionIfNull(cellIndex, "cellIndex");

            return rule.Evaluate(worksheet, cellIndex.RowIndex, cellIndex.ColumnIndex, cellValue);
        }

        public static bool Evaluate(this IDataValidationRule rule, Worksheet worksheet, int rowIndex, int columnIndex, string cellValue)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfNull(rule, "rule");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            ICellValue value = cellValue.ToCellValue(worksheet, rowIndex, columnIndex);

            return rule.Evaluate(worksheet, rowIndex, columnIndex, value);
        }
    }
}