﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.DataValidation
{
    /// <summary>
    /// Represents text length data validation rule.
    /// </summary>
    public sealed class TextLengthDataValidationRule : NumberDataValidationRuleBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TextLengthDataValidationRule" /> class.
        /// </summary>
        /// <param name="context">The context.</param>
        public TextLengthDataValidationRule(NumberDataValidationRuleContext context)
            : base(context)
        {
        }

        /// <summary>
        /// Evaluates the data validation rule.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        /// <param name="cellValue">The cell value.</param>
        /// <returns>Retrurns True if the rule is satisfied, else returns False.</returns>
        protected override bool EvaluateOverride(Worksheet worksheet, int rowIndex, int columnIndex, ICellValue cellValue)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);
            Guard.ThrowExceptionIfNull(cellValue, "cellValue");
            Guard.ThrowExceptionIfNull(this.Argument1, "Argument1");

            if (this.ShouldIgnore(cellValue))
            {
                return true;
            }

            CellValueFormat generalFormat = CellValueFormat.GeneralFormat;

            double cellValueTextLength = cellValue.GetResultValueAsString(generalFormat).Length;
            double? argument1Result = base.EvaluateArgument(this.Argument1, worksheet, rowIndex, columnIndex);
            double? argument2Result = base.EvaluateArgument(this.Argument2, worksheet, rowIndex, columnIndex);

            if (!argument1Result.HasValue || !argument2Result.HasValue)
            {
                return true;
            }

            return this.CompareValues(cellValueTextLength, argument1Result, argument2Result);
        }
    }
}