﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Filtering
{
    /// <summary>
    /// Describes a criteria used by the CustomFilter
    /// </summary>
    public class CustomFilterCriteria
    {
        private readonly ComparisonOperator comparisonOperator;
        private readonly string filterValue;
        private readonly IComparable comparableFilterValue;

        /// <summary>
        /// Gets the comparison operator.
        /// </summary>
        /// <value>The comparison operator.</value>
        public ComparisonOperator ComparisonOperator
        {
            get
            {
                return this.comparisonOperator;
            }
        }

        /// <summary>
        /// Gets the filter value to which the cell value will be compared.
        /// </summary>
        /// <value>The filter value.</value>
        public string FilterValue
        {
            get
            {
                return this.filterValue;
            }
        }

        internal IComparable ComparableFilterValue
        {
            get
            {
                return this.comparableFilterValue;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CustomFilterCriteria" /> class.
        /// </summary>
        /// <param name="comparisonOperator">The comparison operator.</param>
        /// <param name="filterValue">The filter value to which the cell value will be compared.</param>
        public CustomFilterCriteria(ComparisonOperator comparisonOperator, string filterValue)
        {
            this.comparisonOperator = comparisonOperator;
            this.filterValue = filterValue;
            this.comparableFilterValue = this.GetComparableValue(this.filterValue);
        }

        private IComparable GetComparableValue(string value)
        {
            if (value == null)
            {
                return null;
            }

            double numberCriteria;
            if (double.TryParse(value, out numberCriteria))
            {
                return numberCriteria;
            }

            bool boolCriteria;
            if (bool.TryParse(value, out boolCriteria))
            {
                return boolCriteria;
            }

            return value;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            CustomFilterCriteria other = obj as CustomFilterCriteria;

            if (other == null)
            {
                return false;
            }

            return this.ComparisonOperator.Equals(other.ComparisonOperator) &&
                TelerikHelper.EqualsOfT(this.FilterValue, other.FilterValue);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                this.ComparisonOperator.GetHashCode(),
                this.FilterValue.GetHashCodeOrZero());
        }
    }
}