﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Filtering
{
    /// <summary>
    /// Defines the common behaviour of the filter classes.
    /// </summary>
    public interface IFilter
    {
        /// <summary>
        /// Gets the index of the column to which the filter is applied. The index is relative to the beginning of the filter range.
        /// </summary>
        /// <value>The relative index of the column.</value>
        int RelativeColumnIndex { get; }

        /// <summary>
        /// Gets the value of the cell at the specified index. This value is used to determine whether the row will be hidden by the filter.
        /// </summary>
        /// <param name="cells">The cells of the worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        /// <returns>The value of the cell.</returns>
        object GetValue(Cells cells, int rowIndex, int columnIndex);

        /// <summary>
        /// Determines whether the row which contains the specified value will be shown.
        /// </summary>
        /// <param name="value">The value retrieved by the GetValue method.</param>
        /// <returns> A value indicating whether the row which contains the specified value will be shown</returns>
        bool ShouldShowValue(object value);
    }
}