﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents the result of a find operation.
    /// </summary>
    public class FindResult
    {
        #region Fields

        private readonly WorksheetCellIndex foundCell;
        private string resultValue;
        private string rawValue;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the found cell.
        /// </summary>
        /// <value>The found cell.</value>
        public WorksheetCellIndex FoundCell
        {
            get
            {
                return this.foundCell;
            }
        }

        /// <summary>
        /// Gets the result value.
        /// </summary>
        /// <value>The result value.</value>
        public string ResultValue
        {
            get
            {
                if (this.resultValue == null)
                {
                    this.resultValue = this.GetResultValue();
                }

                return this.resultValue;
            }
        }

        /// <summary>
        /// Gets the raw value.
        /// </summary>
        /// <value>The raw value.</value>
        public string RawValue
        {
            get
            {
                if (this.rawValue == null)
                {
                    this.rawValue = this.GetRawValue();
                }

                return this.rawValue;
            }
        }

        #endregion


        #region Constructors

        internal FindResult(WorksheetCellIndex foundCell)
        {
            this.foundCell = foundCell;
        }

        #endregion


        #region Methods

        private string GetResultValue()
        {
            CellSelection selection = this.FoundCell.Worksheet.Cells[this.FoundCell.CellIndex];
            CellValueFormat format = selection.GetFormat().Value;

            return selection.GetValue().Value.GetResultValueAsString(format);
        }

        private string GetRawValue()
        {
            CellSelection selection = this.FoundCell.Worksheet.Cells[this.FoundCell.CellIndex];
            return selection.GetValue().Value.RawValue;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            FindResult other = obj as FindResult;

            if (other == null)
            {
                return false;
            }

            return TelerikHelper.EqualsOfT<WorksheetCellIndex>(this.FoundCell, other.FoundCell);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return this.FoundCell.GetHashCodeOrZero();
        }

        #endregion
    }
}