﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents the replace options.
    /// </summary>
    public class ReplaceOptions : FindOptions
    {
        #region Fields

        private string replaceWith;

        #endregion


        #region Properties

        /// <summary>
        /// Gets or sets the value to replace with.
        /// </summary>
        /// <value>The value to replace with.</value>
        public string ReplaceWith
        {
            get
            {
                if (this.replaceWith == null)
                {
                    this.replaceWith = string.Empty;
                }

                return this.replaceWith;
            }
            set
            {
                if (this.replaceWith != value)
                {
                    this.replaceWith = value;
                }
            }
        }

        #endregion


        #region Methods

        /// <summary>
        /// Copies the properties from the specified options.
        /// </summary>
        /// <param name="options">The options.</param>
        public void CopyPropertiesFrom(ReplaceOptions options)
        {
            base.CopyPropertiesFrom(options);
            this.ReplaceWith = options.ReplaceWith;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            ReplaceOptions other = obj as ReplaceOptions;

            if (other == null)
            {
                return false;
            }

            return base.Equals(obj) && this.ReplaceWith.Equals(other.ReplaceWith);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(base.GetHashCode(), this.ReplaceWith.GetHashCodeOrZero());
        }

        #endregion
    }
}