﻿using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Describes a gradient type of fill.
    /// </summary>
    public class GradientFill : IFill
    {
        #region Fields

        private readonly GradientType gradientType;
        private readonly ThemableColor color1;
        private readonly ThemableColor color2;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the type of the gradient.
        /// </summary>
        /// <value>The type of the gradient.</value>
        public GradientType GradientType
        {
            get
            {
                return this.gradientType;
            }
        }

        /// <summary>
        /// Gets the first color.
        /// </summary>
        /// <value>The first color.</value>
        public ThemableColor Color1
        {
            get
            {
                return this.color1;
            }
        }

        /// <summary>
        /// Gets the second color.
        /// </summary>
        /// <value>The second color.</value>
        public ThemableColor Color2
        {
            get
            {
                return this.color2;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="GradientFill" /> class.
        /// </summary>
        /// <param name="gradientType">Type of the gradient.</param>
        /// <param name="color1">The first color.</param>
        /// <param name="color2">The second color.</param>
        public GradientFill(GradientType gradientType, Color color1, Color color2)
            : this(gradientType, new ThemableColor(color1), new ThemableColor(color2))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GradientFill" /> class.
        /// </summary>
        /// <param name="gradientType">Type of the gradient.</param>
        /// <param name="color1">The first color.</param>
        /// <param name="color2">The second color.</param>
        public GradientFill(GradientType gradientType, ThemableColor color1, ThemableColor color2)
        {
            this.gradientType = gradientType;
            this.color1 = color1;
            this.color2 = color2;
        }

        #endregion


        #region Operators & Overrides

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The object to compare with the current object.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            GradientFill other = obj as GradientFill;
            if (other == null)
            {
                return false;
            }

            return this.GradientType == other.GradientType &&
                TelerikHelper.EqualsOfT(this.Color1, other.Color1) &&
                TelerikHelper.EqualsOfT(this.Color2, other.Color2);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                this.GradientType.GetHashCode(),
                this.Color1.GetHashCode(),
                this.Color2.GetHashCode());
        }

        #endregion
    }
}