﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents a hyperlink in the RadSpreadsheet.
    /// </summary>
    public class SpreadsheetHyperlink
    {
        #region Fields

        private readonly HyperlinkInfo hyperlinkInfo;
        private readonly CellRange cellRange;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the cell range.
        /// </summary>
        /// <value>The cell range.</value>
        public CellRange Range
        {
            get
            {
                return this.cellRange;
            }
        }

        /// <summary>
        /// Gets the hyperlink information.
        /// </summary>
        /// <value>The hyperlink information.</value>
        public HyperlinkInfo HyperlinkInfo
        {
            get
            {
                return this.hyperlinkInfo;
            }
        }

        #endregion


        #region Constructors
        
        internal SpreadsheetHyperlink(CellRange cellRange, HyperlinkInfo hyperlinkInfo)
        {
            this.cellRange = cellRange;
            this.hyperlinkInfo = hyperlinkInfo;
        }

        #endregion


        #region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            SpreadsheetHyperlink other = obj as SpreadsheetHyperlink;

            if (other == null)
            {
                return false;
            }

            return this.Range.Equals(other.Range) &&
                this.HyperlinkInfo.Equals(other.HyperlinkInfo);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(this.Range.GetHashCodeOrZero(), this.HyperlinkInfo.GetHashCodeOrZero());
        }

        #endregion
    }
}