﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Defines members for the sheet classes.
    /// </summary>
    public interface ISheet
    {
        /// <summary>
        /// Gets or sets the name of the sheet.
        /// </summary>
        /// <value>The name of the sheet.</value>
        string Name { get; set; }

        /// <summary>
        /// Gets the workbook.
        /// </summary>
        /// <value>The workbook.</value>
        Workbook Workbook { get; }

        /// <summary>
        /// Gets the visibility.
        /// </summary>
        /// <value>The visibility.</value>
        SheetVisibility Visibility { get; }

        /// <summary>
        /// Gets the type of the sheet.
        /// </summary>
        /// <value>The type of the sheet.</value>
        SheetType Type { get; }

        /// <summary>
        /// Gets the view state of the sheet.
        /// </summary>
        /// <value>The view state of the sheet.</value>
        ISheetViewState ViewState { get; }

        /// <summary>
        /// Gets the value indicating whether the layout update is suspended.
        /// </summary>
        /// <value>The value indicating whether the layout update is suspended.</value>
        bool IsLayoutUpdateSuspended { get; }

        /// <summary>
        /// Invalidates the layout.
        /// </summary>
        void InvalidateLayout();

        /// <summary>
        /// Suspends the layout update.
        /// </summary>
        void SuspendLayoutUpdate();

        /// <summary>
        /// Resumes the layout update.
        /// </summary>
        void ResumeLayoutUpdate();

        /// <summary>
        /// Occurs when the layout is invalidated.
        /// </summary>
        event EventHandler LayoutInvalidated;
    }
}