﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Copying;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Describes the owner of a spreadsheet name collection.
    /// </summary>
    public class SpreadsheetNameCollectionScope : ICopyable<SpreadsheetNameCollectionScope>
    {
        #region Fields

        private readonly Workbook workbook;
        private readonly Worksheet worksheet;

        #endregion


        #region Properties

        /// <summary>;
        /// Gets a value indicating whether the owner is a workbook.
        /// </summary>
        /// <value>The value indicating whether the owner is a workbook.</value>
        public bool IsGlobal
        {
            get
            {
                return this.worksheet == null;
            }
        }

        internal Worksheet CurrentWorksheet
        {
            get
            {
                if (this.IsGlobal)
                {
                    return workbook.ActiveWorksheet;
                }

                return this.worksheet;
            }
        }

        internal Workbook Workbook
        {
            get
            {
                return this.workbook;
            }
        }

        #endregion


        #region Constructors

        internal SpreadsheetNameCollectionScope(Worksheet worksheet)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");

            this.worksheet = worksheet;
            this.workbook = worksheet.Workbook;
        }

        internal SpreadsheetNameCollectionScope(Workbook workbook)
        {
            Guard.ThrowExceptionIfNull(workbook, "workbook");
            
            this.workbook = workbook;
        }

        #endregion


        #region Methods

        SpreadsheetNameCollectionScope ICopyable<SpreadsheetNameCollectionScope>.Copy(CopyContext context)
        {
            if (this.IsGlobal && !context.SpreadsheetNameExistsInTargetWorkbook)
            {
                return new SpreadsheetNameCollectionScope(context.TargetWorksheet.Workbook);
            }
            else
            {
                return new SpreadsheetNameCollectionScope(context.TargetWorksheet);
            }
        }

        #endregion
    }
}