﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Describes the paste options.
    /// </summary>
    public class PasteOptions
    {
        #region Constants

        /// <summary>
        /// Paste Options with paste type all.
        /// </summary>
        public static readonly PasteOptions All = new PasteOptions(PasteType.All);

        #endregion


        #region Fields

        private readonly PasteType pasteType;
        private readonly bool skipBlanks;
        private readonly bool skipMergedCells;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the type of the paste.
        /// </summary>
        /// <value>The type of the paste.</value>
        public PasteType PasteType
        {
            get
            {
                return this.pasteType;
            }
        }

        /// <summary>
        /// Gets a value indicating whether blanks should be skipped.
        /// </summary>
        /// <value>The value indicating whether blanks should be skipped.</value>
        public bool SkipBlanks
        {
            get 
            {
                return this.skipBlanks;
            }
        }

        /// <summary>
        /// Gets a value indicating whether merged cells should be skipped.
        /// </summary>
        /// <value>The value indicating whether merged cells should be skipped.</value>
        internal bool SkipMergedCells
        {
            get
            {
                return this.skipMergedCells;
            }
        }

        #endregion


        #region Cosntructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PasteOptions" /> class.
        /// </summary>
        /// <param name="pasteType">Type of the paste.</param>
        /// <param name="skipBlanks">The value indicating whether blanks should be skipped.</param>
        public PasteOptions(PasteType pasteType, bool skipBlanks = false)
        {
            this.pasteType = pasteType;
            this.skipBlanks = skipBlanks;
            this.skipMergedCells = false;
        }

        internal PasteOptions(PasteType pasteType, bool skipBlanks, bool skipMergedCells)
            :this(pasteType, skipBlanks)
        {
            this.skipMergedCells = skipMergedCells;
        }

        #endregion
    }
}