﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Printing
{
    /// <summary>
    /// The section content of headers and footers.
    /// </summary>
    public class HeaderFooterSection
    {
        private readonly Action updateOnChange;
        private string text;

        internal HeaderFooterSection(Action updateOnChange)
        {
            Guard.ThrowExceptionIfNull(updateOnChange, "updateOnChange");

            this.updateOnChange = updateOnChange;
            this.text = string.Empty;
        }

        internal HeaderFooterSection(HeaderFooterSection other, Action updateOnChange)
        {
            Guard.ThrowExceptionIfNull(updateOnChange, "updateOnChange");

            this.updateOnChange = updateOnChange;
            this.text = other.text;
        }

        /// <summary>
        /// Gets or sets the text content.
        /// </summary>
        /// <value>The text content.</value>
        public string Text
        {
            get
            {
                return this.text;
            }
            set
            {
                if (this.text != value)
                {
                    this.text = value;
                    this.updateOnChange();
                }
            }
        }

        internal bool IsEmpty
        {
            get
            {
                HeaderFooterSectionTextSanitizer sanitizer = new HeaderFooterSectionTextSanitizer();
                string result = sanitizer.Sanitize(text);

                return string.IsNullOrEmpty(result);
            }
        }

        // TODO: Add PictureInfo property that defines the only picture in this section
        // public PictureInfo Picture { get; set; }

        internal void CopyFrom(HeaderFooterSection fromHeaderFooterSection)
        {
            this.Text = fromHeaderFooterSection.Text;

            // TODO: Copy picture information.
            //this.Picture = fromHeaderFooterSection.Picture;
        }

        internal void Clear()
        {
            this.Text = string.Empty;

            // TODO: Clear picture info.
            //this.PictureInfo = null;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            HeaderFooterSection other = obj as HeaderFooterSection;

            if (other == null)
            {
                return false;
            }

            // TODO: Add picture equality comparison.
            return this.Text.Equals(other.Text);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            // TODO: Add picture hash code.
            return this.Text.GetHashCode();
        }
    }
}
