﻿using System;
using System.Windows;
using System.Windows.Media;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
#if SILVERLIGHT
using BaselineAlignmentType = Telerik.Windows.Documents.Spreadsheet.Model.BaselineAlignment;
#else
using BaselineAlignmentType = System.Windows.BaselineAlignment;
#endif
using Telerik.Windows.Documents.Spreadsheet.Utilities;
using System.Globalization;
using System.Windows.Documents;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Printing
{
    internal abstract class HeaderFooterSectionTextRendererBase : IHeaderFooterSectionTextParseHandler
    {
        public const string Bold = "Bold";
        public const string Italic = "Italic";
        public const string Regular = "Regular";
        public const int DefaultFontSize = 11;
        private static readonly int ThemeColorTypesCount;
        private static readonly ThemableColor DefaultFontColor;
        private const double TintAndShadeMultiplier = 100;
        private const int TintAndShadeSignPositionIndex = 2;
        private readonly HeaderFooterSectionTextParser parser;
        private FontFamily fontFamily;
        private FontWeight fontWeight;
        private FontStyle fontStyle;
        private int fontSize;
        private ThemableColor fontColor;
        private UnderlineType underlineType;
        private BaselineAlignment baselineAlignment;
        private bool isStrikeThrough;

        static HeaderFooterSectionTextRendererBase()
        {
            ThemeColorTypesCount = Enum.GetValues(typeof(ThemeColorType)).Length;
            DefaultFontColor = new ThemableColor(Colors.Black, true);
        }

        protected HeaderFooterSectionTextRendererBase()
        {
            this.parser = new HeaderFooterSectionTextParser(this);
        }

        protected FontFamily FontFamily
        {
            get
            {
                return this.fontFamily;
            }
        }

        protected FontWeight FontWeight
        {
            get
            {
                return this.fontWeight;
            }
        }

        protected FontStyle FontStyle
        {
            get
            {
                return this.fontStyle;
            }
        }

        protected double FontSize
        {
            get
            {
                return UnitHelper.PointToDip(this.fontSize);
            }
        }

        protected bool IsStrikeThrough
        {
            get
            {
                return this.isStrikeThrough;
            }
        }

        protected UnderlineType UnderlineType
        {
            get
            {
                return this.underlineType;
            }
        }

        protected ThemableColor FontColor
        {
            get
            {
                return this.fontColor;
            }
        }

        protected Color FontColorActualValue
        {
            get
            {
                return this.fontColor.GetActualValue(this.SheetContext.Workbook.Theme);
            }
        }

        protected BaselineAlignment BaselineAlignment
        {
            get
            {
                return this.baselineAlignment;
            }
        }

        public static string GetNormalFontFamily(Workbook workbook)
        {
            return workbook.Styles["Normal"].FontFamily.GetActualValue(workbook.Theme).ToString();
        }

        protected abstract Sheet SheetContext { get; }

        protected void Render(string headerFooterSectionText)
        {
            this.SetInitialStates();
            this.parser.Parse(headerFooterSectionText);
        }

        protected Run CreateRun(string text)
        {
            Run run = new Run();
            run.FontFamily = this.FontFamily;
            run.FontStyle = this.FontStyle;
            run.FontWeight = this.FontWeight;
            run.FontSize = this.FontSize;

            this.SetTextDecorations(run);
#if !SILVERLIGHT
            run.BaselineAlignment = this.BaselineAlignment;
#endif
            run.Foreground = new SolidColorBrush(this.FontColorActualValue);
            run.Text = text;

            return run;
        }

        protected abstract void OnAppendingAmpersandOverride();
        protected abstract void OnAppendingFileNameOverride();
        protected abstract void OnAppendingPageNumberOverride();
        protected abstract void OnAppendingAddToPageNumberOverride();
        protected abstract void OnAppendingSubstractFromPageNumberOverride();
        protected abstract void OnAppendingNumberOfPagesOverride();
        protected abstract void OnAppendingSheetNameOverride();
        protected abstract void OnAppendingFilePathOverride();
        protected abstract void OnAppendingPictureOverride();
        protected abstract void OnAppendingDateOverride();
        protected abstract void OnAppendingTimeOverride();
        protected abstract void OnAppendingTextFragmentOverride(string text);

        void IHeaderFooterSectionTextParseHandler.OnAmpersandParse()
        {
            this.OnAppendingAmpersandOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnFileNameParse()
        {
            this.OnAppendingFileNameOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnPageNumberParse()
        {
            this.OnAppendingPageNumberOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnAddToPageNumberParse()
        {
            this.OnAppendingAddToPageNumberOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnSubstractFromPageNumberParse()
        {
            this.OnAppendingSubstractFromPageNumberOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnNumberOfPagesParse()
        {
            this.OnAppendingNumberOfPagesOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnTextFragmentParse(string text)
        {
            this.OnAppendingTextFragmentOverride(text);
        }

        void IHeaderFooterSectionTextParseHandler.OnSheetNameParse()
        {
            this.OnAppendingSheetNameOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnDateParse()
        {
            this.OnAppendingDateOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnTimeParse()
        {
            this.OnAppendingTimeOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnFilePathParse()
        {
            this.OnAppendingFilePathOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnPictureParse()
        {
            this.OnAppendingPictureOverride();
        }

        void IHeaderFooterSectionTextParseHandler.OnOutlineStyleParse()
        {
            // Do nothing.
        }

        void IHeaderFooterSectionTextParseHandler.OnShadowStyleParse()
        {
            // Do nothing.
        }

        void IHeaderFooterSectionTextParseHandler.OnDoubleUnderlineParse()
        {
            if (this.underlineType == Model.UnderlineType.Double)
            {
                this.underlineType = Model.UnderlineType.None;
            }
            else
            {
                this.underlineType = Model.UnderlineType.Double;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnBoldParse()
        {
            if (this.fontWeight == FontWeights.Bold)
            {
                this.fontWeight = FontWeights.Normal;
            }
            else
            {
                this.fontWeight = FontWeights.Bold;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnItalicParse()
        {
            if (this.fontStyle == FontStyles.Italic)
            {
                this.fontStyle = FontStyles.Normal;
            }
            else
            {
                this.fontStyle = FontStyles.Italic;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnUnderlineParse()
        {
            if (this.underlineType == Model.UnderlineType.Single)
            {
                this.underlineType = Model.UnderlineType.None;
            }
            else
            {
                this.underlineType = Model.UnderlineType.Single;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnFontSizeParse(int fontSize)
        {
            this.fontSize = fontSize;
        }

        void IHeaderFooterSectionTextParseHandler.OnStrikeThroughParse()
        {
            this.isStrikeThrough = !this.isStrikeThrough;
        }

        void IHeaderFooterSectionTextParseHandler.OnSubscriptParse()
        {
            if (this.baselineAlignment == BaselineAlignmentType.Subscript)
            {
                this.baselineAlignment = BaselineAlignmentType.Baseline;
            }
            else
            {
                this.baselineAlignment = BaselineAlignmentType.Subscript;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnSuperscriptParse()
        {
            if (this.baselineAlignment == BaselineAlignmentType.Superscript)
            {
                this.baselineAlignment = BaselineAlignmentType.Baseline;
            }
            else
            {
                this.baselineAlignment = BaselineAlignmentType.Superscript;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnFontPropertiesParse(string fontPropertiesValue)
        {
            string[] properties = fontPropertiesValue.Split(HeaderFooterFieldSymbols.FontPropertiesSeparator);

            bool areValidProperties = properties.Length == 2;

            if (areValidProperties)
            {
                string fontFamilyProperty = properties[0].Trim();
                string fontWeightAndStyleProperties = properties[1];
                bool isNormalFont = fontFamilyProperty == HeaderFooterFieldSymbols.FontPropertiesNormalFontFamily.ToString();

                this.fontFamily = new FontFamily(isNormalFont ? GetNormalFontFamily(this.SheetContext.Workbook) : fontFamilyProperty);
                this.fontWeight = fontWeightAndStyleProperties.Contains(Bold) ? FontWeights.Bold : FontWeights.Normal;
                this.fontStyle = fontWeightAndStyleProperties.Contains(Italic) ? FontStyles.Italic : FontStyles.Normal;
            }
        }

        void IHeaderFooterSectionTextParseHandler.OnColorParse(string colorValue)
        {
            char symbolOnSignPosition = colorValue[TintAndShadeSignPositionIndex];

            if (IsSign(symbolOnSignPosition))
            {
                this.ParseThemeColor(colorValue);
            }
            else
            {
                this.ParseHexColor(colorValue);
            }
        }

        private void ParseThemeColor(string colorValue)
        {
            string colorIdText = colorValue.Substring(0, TintAndShadeSignPositionIndex);
            string tintAndShadeText = colorValue.Substring(TintAndShadeSignPositionIndex + 1);

            int id, tintAndShade;
            if (int.TryParse(colorIdText, out id) && int.TryParse(tintAndShadeText, out tintAndShade) &&
                id < ThemeColorTypesCount && Math.Abs(tintAndShade) <= TintAndShadeMultiplier)
            {
                char sign = colorValue[TintAndShadeSignPositionIndex];
                double multiplier = (sign == '+' ? 1 : -1) / TintAndShadeMultiplier;
                ThemeColorType colorType = (ThemeColorType)id;

                this.fontColor = new ThemableColor(colorType, tintAndShade * multiplier);
            }
        }

        private void ParseHexColor(string colorValue)
        {
            string redText = colorValue.Substring(0, 2);
            string greenText = colorValue.Substring(2, 2);
            string blueText = colorValue.Substring(4);

            byte r, g, b;
            if (byte.TryParse(redText, NumberStyles.HexNumber, CultureInfo.InvariantCulture, out r) &&
                byte.TryParse(greenText, NumberStyles.HexNumber, CultureInfo.InvariantCulture, out g) &&
                byte.TryParse(blueText, NumberStyles.HexNumber, CultureInfo.InvariantCulture, out b))
            {
                this.fontColor = new ThemableColor(Color.FromArgb(255, r, g, b));
            }
        }

        private static bool IsSign(char symbol)
        {
            return symbol == '+' || symbol == '-';
        }

        private void SetInitialStates()
        {
            this.fontFamily = new FontFamily(GetNormalFontFamily(this.SheetContext.Workbook));
            this.fontWeight = FontWeights.Normal;
            this.fontStyle = FontStyles.Normal;
            this.isStrikeThrough = false;
            this.underlineType = UnderlineType.None;
            this.baselineAlignment = BaselineAlignmentType.Baseline;
            this.fontSize = DefaultFontSize;
            this.fontColor = DefaultFontColor;
        }

        private void SetTextDecorations(Run run)
        {
#if SILVERLIGHT
            if (this.UnderlineType != UnderlineType.None)
            {
                run.TextDecorations = TextDecorations.Underline;
            }
#else
            run.TextDecorations = this.GetTextDecorations();
#endif
        }

#if !SILVERLIGHT
        private TextDecorationCollection GetTextDecorations()
        {
            TextDecorationCollection textDecorations = new TextDecorationCollection();

            if (this.UnderlineType != UnderlineType.None)
            {
                textDecorations.Add(TextDecorations.Underline);
            }

            if (this.IsStrikeThrough)
            {
                textDecorations.Add(TextDecorations.Strikethrough);
            }

            return textDecorations;
        }
#endif
    }
}
