﻿using System;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Printing
{
    internal class HeaderFooterSectionTextSanitizer : IHeaderFooterSectionTextParseHandler
    {
        private readonly StringBuilder resultText;
        private readonly HeaderFooterSectionTextParser parser;

        public HeaderFooterSectionTextSanitizer()
        {
            this.parser = new HeaderFooterSectionTextParser(this);
            this.resultText = new StringBuilder();
        }

        public string Sanitize(string sectionText)
        {
            this.parser.Parse(sectionText);
            string result = this.resultText.ToString();
            this.resultText.Clear();

            return result;
        }

        void IHeaderFooterSectionTextParseHandler.OnColorParse(string colorValue)
        {
            this.WriteField(string.Format("{0}{1}", HeaderFooterFieldSymbols.ColorSymbol, colorValue));
        }

        void IHeaderFooterSectionTextParseHandler.OnFontPropertiesParse(string fontPropertiesValue)
        {
            this.WriteField(string.Format("{0}{1}{2}", HeaderFooterFieldSymbols.FontPropertiesStartSymbol, fontPropertiesValue, HeaderFooterFieldSymbols.FontPropertiesEndSymbol));
        }

        void IHeaderFooterSectionTextParseHandler.OnBoldParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.BoldSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnItalicParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.ItalicSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnUnderlineParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.UnderlineSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnDoubleUnderlineParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.DoubleUnderlineSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnStrikeThroughParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.StrikeThroughSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnSubscriptParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.SubscriptSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnSuperscriptParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.SuperscriptSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnFontSizeParse(int fontSize)
        {
            string numberText = fontSize.ToString();
            this.WriteField(numberText);
        }

        void IHeaderFooterSectionTextParseHandler.OnPageNumberParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.PageNumberSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnAddToPageNumberParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.AddToPageNumberSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnSubstractFromPageNumberParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.SubstractFromPageNumberSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnNumberOfPagesParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.NumberOfPagesSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnDateParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.DateSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnTimeParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.TimeSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnFilePathParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.FilePathSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnFileNameParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.FileNameSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnSheetNameParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.SheetNameSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnPictureParse()
        {
            // TODO: Do nothing because pictures import/export is more complex and currently unsupported.
            // However when pictures are implemented only the first picture should be parsed as only a single picture in section is allowed.
            // All pictures after the first picture should be skipped.
        }

        void IHeaderFooterSectionTextParseHandler.OnAmpersandParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.AmpersandSymbol);
        }

        void IHeaderFooterSectionTextParseHandler.OnOutlineStyleParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.OutlineStyle);
        }

        void IHeaderFooterSectionTextParseHandler.OnShadowStyleParse()
        {
            this.WriteField(HeaderFooterFieldSymbols.ShadowStyle);
        }

        void IHeaderFooterSectionTextParseHandler.OnTextFragmentParse(string text)
        {
            this.resultText.Append(text);
        }

        private void WriteField(char symbol)
        {
            this.resultText.AppendFormat("{0}{1}", HeaderFooterFieldSymbols.FieldStartSymbol, symbol);
        }

        private void WriteField(string parameter)
        {
            this.resultText.AppendFormat("{0}{1}", HeaderFooterFieldSymbols.FieldStartSymbol, parameter);
        }
    }
}
