﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Printing
{
    /// <summary>
    /// Represents a base class for sheet print options.
    /// </summary>
    public abstract class SheetPrintOptionsBase
    {
        #region Fields

        private readonly SheetPageSetupBase pageSetup;
        private bool printBlackAndWhite;
        private bool isDraftQuality;

        #endregion


        #region Properties

        internal bool PrintBlackAndWhite 
        {
            get
            {
                return this.printBlackAndWhite;
            }
            set
            {
                if (this.printBlackAndWhite != value)
                {
                    this.printBlackAndWhite = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        internal bool IsDraftQuality
        {
            get
            {
                return this.isDraftQuality;
            }
            set
            {
                if (this.isDraftQuality != value)
                {
                    this.isDraftQuality = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        #endregion


        #region Constructors

        internal SheetPrintOptionsBase(SheetPageSetupBase pageSetup)
        {
            this.pageSetup = pageSetup;
        }

        #endregion


        #region Methods

        internal virtual void CopyFrom(SheetPrintOptionsBase fromSheetPrintOptionsBase)
        {
            this.printBlackAndWhite = fromSheetPrintOptionsBase.PrintBlackAndWhite;
            this.isDraftQuality = fromSheetPrintOptionsBase.IsDraftQuality;
        }

        internal void OnPrintOptionChanged()
        {
            this.pageSetup.OnPageSetupChanged();
        }

        #endregion
    }
}