﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Printing
{
    /// <summary>
    /// Represents the worksheet print options.
    /// </summary>
    public class WorksheetPrintOptions : SheetPrintOptionsBase
    {
        private bool printGridlines;
        private bool printRowColumnHeadings;
        private ErrorsPrintStyle errorsPrintStyle;
        private PrintLocation printLocation;

        /// <summary>
        /// Gets or sets the value indicating whether to print gridlines.
        /// </summary>
        /// <value>The value indicating whether to print gridlines.</value>
        public bool PrintGridlines
        {
            get
            {
                return this.printGridlines;
            }
            set
            {
                if (this.printGridlines != value)
                {
                    this.printGridlines = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the value indicating whether to print the row and column headings.
        /// </summary>
        /// <value>The value indicating whether to print the row and column headings.</value>
        public bool PrintRowColumnHeadings
        {
            get
            {
                return this.printRowColumnHeadings;
            }
            set
            {
                if (this.printRowColumnHeadings != value)
                {
                    this.printRowColumnHeadings = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        internal ErrorsPrintStyle ErrorsPrintStyle
        {
            get
            {
                return this.errorsPrintStyle;
            }
            set
            {
                if (this.errorsPrintStyle != value)
                {
                    this.errorsPrintStyle = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        internal PrintLocation Comments
        {
            get
            {
                return this.printLocation;
            }
            set
            {
                if (this.printLocation != value)
                {
                    this.printLocation = value;
                    this.OnPrintOptionChanged();
                }
            }
        }

        internal bool PrintGridlinesOutline
        {
            get
            {
                return this.PrintGridlines || this.PrintRowColumnHeadings;
            }
        }

        internal WorksheetPrintOptions(WorksheetPageSetup pageSetup)
            : base(pageSetup)
        {
        }

        internal override void CopyFrom(SheetPrintOptionsBase fromSheetPrintOptionsBase)
        {
            base.CopyFrom(fromSheetPrintOptionsBase);

            WorksheetPrintOptions fromWorksheetPrintOptions = (WorksheetPrintOptions)fromSheetPrintOptionsBase;
            this.errorsPrintStyle = fromWorksheetPrintOptions.ErrorsPrintStyle;
            this.printGridlines = fromWorksheetPrintOptions.PrintGridlines;
            this.printRowColumnHeadings = fromWorksheetPrintOptions.PrintRowColumnHeadings;
            this.printLocation = fromWorksheetPrintOptions.Comments;
        }
    }
}