﻿using System;
using Telerik.Windows.Documents.Common.Model.Protection;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Protection
{
    /// <summary>
    /// Contains helper methods for generating protection salt and hash values.
    /// </summary>
    internal class SpreadsheetProtectionHelper : ProtectionHelperBase
    {
        private static readonly char nConstant = 'N';
        private static readonly char kConstant = 'K';
        private static readonly SpreadsheetProtectionHelper instance;

        static SpreadsheetProtectionHelper()
        {
            instance = new SpreadsheetProtectionHelper();
        }

        internal static SpreadsheetProtectionHelper Instance
        {
            get
            {
                return instance;
            }
        }

        /// <summary>
        /// Generates the password using OOXML legacy scheme.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        public String GeneratePasswordLegacyScheme(String password)
        {
            Guard.ThrowExceptionIfNullOrEmpty(password, "password");

            Int32 pLength = password.Length;
            Int32 hash = 0;
            if (pLength == 0) return hash.ToString("X");

            for (Int32 i = pLength - 1; i >= 0; i--)
            {
                hash = hash >> 14 & 0x01 | hash << 1 & 0x7fff;
                hash ^= password[i];
            }
            hash = hash >> 14 & 0x01 | hash << 1 & 0x7fff;
            hash ^= 0x8000 | nConstant << 8 | kConstant;
            hash ^= pLength;
            return hash.ToString("X");
        }
    }
}
