﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents a property value applied to an entire range.
    /// </summary>
    /// <typeparam name="T">The type of the property value.</typeparam>
    public class RangePropertyValue<T>
    {
        #region Fields

        private readonly bool isIndeterminate;        
        private readonly T value;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the value indicating whether the property value is indeterminate.
        /// </summary>
        /// <value>The value indicating whether the property value is indeterminate.</value>
        public bool IsIndeterminate
        {
            get
            {
                return this.isIndeterminate;
            }
        }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <value>The value.</value>
        public T Value
        {
            get 
            {
                return this.value;
            }
        }

        #endregion


        #region Constructors

        internal RangePropertyValue(bool isIndeterminate, T value)
        {
            this.isIndeterminate = isIndeterminate;
            this.value = value;
        }

        #endregion


        #region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            RangePropertyValue<T> other = obj as RangePropertyValue<T>;
            if (other == null)
            {
                return false;
            }

            return TelerikHelper.EqualsOfT(this.Value, other.Value) && this.IsIndeterminate == other.IsIndeterminate;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            int valueHashCode = 0;
            if (this.Value != null)
            {
                valueHashCode = this.Value.GetHashCode();
            }

            return TelerikHelper.CombineHashCodes(
                valueHashCode, 
                this.IsIndeterminate.GetHashCode());
        }

        #endregion
    }
}