﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Describes the types of change which can be applied to a sheet collection.
    /// </summary>
    public enum SheetCollectionChangeType
    {
        /// <summary>
        /// Add sheet collection change type.
        /// </summary>
        Add,

        /// <summary>
        /// Move sheet collection change type.
        /// </summary>
        Move,

        /// <summary>
        /// Remove sheet collection change type.
        /// </summary>
        Remove
    }

    /// <summary>
    /// Arguments for the Changed event of the SheetCollection.
    /// </summary>
    public class SheetCollectionChangedEventArgs : EventArgs
    {
        #region Fields

        private readonly SheetCollectionChangeType changeType;
        private readonly Sheet sheet;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the change type.
        /// </summary>
        /// <value>The change type.</value>
        public SheetCollectionChangeType ChangeType
        {
            get
            {
                return this.changeType;
            }
        }

        internal Sheet Sheet
        {
            get
            {
                return this.sheet;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SheetCollectionChangedEventArgs" /> class.
        /// </summary>
        /// <param name="changeType">Type of the change.</param>
        public SheetCollectionChangedEventArgs(SheetCollectionChangeType changeType)
        {
            this.changeType = changeType;
        }

        internal SheetCollectionChangedEventArgs(SheetCollectionChangeType changeType, Sheet sheet)
            : this(changeType)
        {
            this.sheet = sheet;
        }

        #endregion
    }
}