﻿using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents base for the WorksheetViewState class.
    /// </summary>
    public abstract class SheetViewStateBase : ISheetViewState
    {
        #region Fields

        private ThemableColor tabColor;
        private bool isInvalidated;

        #endregion


        #region Properties

        /// <summary>
        /// Gets or sets the color of the tab.
        /// </summary>
        /// <value>The color of the tab.</value>
        public ThemableColor TabColor
        {
            get
            {
                return this.tabColor;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");
                this.tabColor = value;
            }
        }

        /// <summary>
        /// Gets or sets the is invalidated.
        /// </summary>
        /// <value>The is invalidated.</value>
        public bool IsInvalidated
        {
            get
            {
                return this.isInvalidated;
            }
            set
            {
                Guard.ThrowExceptionIfNull(value, "value");
                this.isInvalidated = value;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SheetViewStateBase" /> class.
        /// </summary>
        protected SheetViewStateBase()
        {
            this.TabColor = new ThemableColor(ColorsHelper.HexStringToColor("#FFE4E8EB"));
        }

        #endregion
    }
}