﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Sorting
{
    internal class CustomValuesSortConditionComparer : ValuesSortConditionComparerBase
    {
        private readonly string[] customList;
        private readonly SortOrder sortOrder;

        public CustomValuesSortConditionComparer(string[] customList, SortOrder sortOrder)
        {
            Guard.ThrowExceptionIfLessThan(1, customList.Length, "customList");
            this.customList = customList;
            this.sortOrder = sortOrder;
        }

        public override int Compare(SortValue x, SortValue y)
        {
            ICellValue value1 = x.Value as ICellValue;
            ICellValue value2 = y.Value as ICellValue;

            string stringValue1 = value1.GetValueAsString(CellValueFormat.GeneralFormat);
            string stringValue2 = value2.GetValueAsString(CellValueFormat.GeneralFormat);

            int stringValue1Index = Array.IndexOf(this.customList, stringValue1);
            int stringValue2Index = Array.IndexOf(this.customList, stringValue2);

            if (stringValue1Index >= 0 && stringValue2Index >= 0)
            {
                int result = stringValue1Index - stringValue2Index;

                result = (sortOrder == SortOrder.Ascending) ? result : result * -1;
                result = (result == 0) ? x.Index.CompareTo(y.Index) : result;

                return result;
            }

            if (stringValue1Index >= 0 && stringValue2Index < 0)
            {
                int result = -1;
                result = (sortOrder == SortOrder.Ascending) ? result : result * -1;

                return result;
            }

            if (stringValue1Index < 0 && stringValue2Index >= 0)
            {
                int result = 1;
                result = (sortOrder == SortOrder.Ascending) ? result : result * -1;

                return result;
            }

            return base.Compare(x, y);
        }
    }
}