﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Sorting
{
    /// <summary>
    /// Represents a sort condition.
    /// </summary>
    public interface ISortCondition
    {
        /// <summary>
        /// Gets the index of the row or column that is key for sorting relative to the sorted range.
        /// </summary>
        /// <value>The index of the row or column that is key for sorting relative to the sorted range.</value>
        int RelativeIndex { get; }

        /// <summary>
        /// Gets the comparer used for sorting.
        /// </summary>
        /// <value>The comparer used for sorting.</value>
        IComparer<SortValue> Comparer { get; }

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <param name="cells">The cells.</param>
        /// <param name="rowIndex">The row index.</param>
        /// <param name="columnIndex">The column index.</param>
        /// <returns>The value of the specified cell.</returns>
        object GetValue(Cells cells, int rowIndex, int columnIndex);
    }
}