﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Sorting
{
    /// <summary>
    /// Represents a base class for color sort conditions.
    /// </summary>
    /// <typeparam name="T">The type of the T.</typeparam>
    public abstract class OrderedSortConditionBase<T> : SortConditionBase<T>
    {
        #region Fields

        private readonly SortOrder sortOrder;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the sort order.
        /// </summary>
        /// <value>The sort order.</value>
        public SortOrder SortOrder
        {
            get
            {
                return this.sortOrder;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="OrderedSortConditionBase{T}" /> class.
        /// </summary>
        /// <param name="relativeIndex">Gets the index of the row or column that is key for sorting relative to the sorted range..</param>
        /// <param name="sortOrder">The sort order.</param>
        protected OrderedSortConditionBase(int relativeIndex, SortOrder sortOrder) 
            : base(relativeIndex)
        {
            this.sortOrder = sortOrder;
        }

        #endregion


        #region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The object to compare with the current object.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            OrderedSortConditionBase<T> other = obj as OrderedSortConditionBase<T>;

            if (other == null)
            {
                return false;
            }

            return this.SortOrder.Equals(other.SortOrder) && base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                base.GetHashCode(),
                this.SortOrder.GetHashCode());
        }

        #endregion
    }
}
