﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Windows.Documents.Spreadsheet.Model.Sorting
{
    /// <summary>
    /// Represents a sort condition for sorting values.
    /// </summary>
    public sealed class ValuesSortCondition : OrderedSortConditionBase<ICellValue>
    {
        #region Constants

        private static readonly ValuesSortConditionComparer DefaultValuesSortComparer = new ValuesSortConditionComparer(false);
        private static readonly ValuesSortConditionComparer InvertedValuesSortComparer = new ValuesSortConditionComparer(true);

        #endregion


        #region Properties

        /// <summary>
        /// Gets the property definition of the property which is used for sorting.
        /// </summary>
        /// <value>
        /// The property definition of the property which is used for sorting.
        /// </value>
        protected override IPropertyDefinition<ICellValue> PropertyDefinition
        {
            get
            {
                return CellPropertyDefinitions.ValueProperty;
            }
        }

        /// <summary>
        /// Gets the comparer used for sorting.
        /// </summary>
        /// <value>The comparer used for sorting.</value>
        public override IComparer<SortValue> Comparer
        {
            get
            {
                switch (this.SortOrder)
                {
                    case SortOrder.Ascending:
                        return ValuesSortCondition.DefaultValuesSortComparer;
                    case SortOrder.Descending:
                        return ValuesSortCondition.InvertedValuesSortComparer;
                }

                return null;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ValuesSortCondition" /> class.
        /// </summary>
        /// <param name="relativeIndex">Gets the index of the row or column that is key for sorting relative to the sorted range.</param>
        /// <param name="sortOrder">The sort order.</param>
        public ValuesSortCondition(int relativeIndex, SortOrder sortOrder)
            : base(relativeIndex, sortOrder)
        {
        }

        #endregion


        #region Methods

        internal override ISortCondition Copy(int newRelativeColumnIndex)
        {
            return new ValuesSortCondition(newRelativeColumnIndex, this.SortOrder);
        }

        #endregion
    }
}