﻿using System;
using System.Windows;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Model.Drawing.Theming;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Spreadsheet.Utilities;
namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Provides the default values of RadSpreadsheet.
    /// </summary>
    public static class SpreadsheetDefaultValues
    {
        #region Constants

        /// <summary>
        /// The default width of the columns.
        /// </summary>
        public static readonly double DefaultColumnWidth = 65;

        /// <summary>
        /// The default height of the rows.
        /// </summary>
        public static readonly double DefaultRowHeight = 20;

        /// <summary>
        /// The minimum width of a column.
        /// </summary>
        public static readonly double MinColumnWidth = 0;

        /// <summary>
        /// The maximum width of a column.
        /// </summary>
		public static readonly double MaxColumnWidth = 2000;

        /// <summary>
        /// The minimum height of a row.
        /// </summary>
        public static readonly double MinRowHeight = 0;

        /// <summary>
        /// The maximum height of a row.
        /// </summary>
		public static readonly double MaxRowHeight = 600;

        /// <summary>
        /// The row count in a worksheet in RadSpreadsheet.
        /// </summary>
		public static readonly int RowCount = 1048576;

        /// <summary>
        /// The column count in a worksheet in RadSpreadsheet.
        /// </summary>
		public static readonly int ColumnCount = 16384;

        /// <summary>
        /// The cell count in a worksheet in RadSpreadsheet.
        /// </summary>
		public static readonly long CellCount = (long)RowCount * ColumnCount;

        /// <summary>
        /// The maximum indent value.
        /// </summary>
        public static readonly int MaxIndent = 250;

        /// <summary>
        /// The indent step size which is multiplied by the indent factor.
        /// </summary>
        public static readonly double IndentStep = 10;

        /// <summary>
        /// The left margin of a cell.
        /// </summary>
        public static readonly double LeftCellMargin = 2 + CellBorder.Default.Thickness / 2;

        /// <summary>
        /// The right margin of a cell.
        /// </summary>
        public static readonly double RightCellMargin = 2 + CellBorder.Default.Thickness / 2;

        /// <summary>
        /// The sum of the left and right cell margins.
        /// </summary>
        public static readonly double TotalHorizontalCellMargin = LeftCellMargin + RightCellMargin;

        /// <summary>
        /// The cell range encompassing the entire worksheet.
        /// </summary>
		public static readonly CellRange TotalWorksheetCellRange = new CellRange(0, 0, RowCount - 1, ColumnCount - 1);

        /// <summary>
        /// The default font color.
        /// </summary>
        public static readonly ThemableColor DefaultForeColor = new ThemableColor(ThemeColorType.Text1);

        /// <summary>
        /// The default font family.
        /// </summary>
        public static readonly ThemableFontFamily DefaultFontFamily = new ThemableFontFamily(ThemeFontType.Minor);

        /// <summary>
        /// The default font size measured in points.
        /// </summary>
        public static readonly double DefaultFontSizeInPoints = 11;

        /// <summary>
        /// The default font size in dip.
        /// </summary>
        public static readonly double DefaultFontSize = UnitHelper.PointToDip(DefaultFontSizeInPoints);

        /// <summary>
        /// The default style name.
        /// </summary>
        public static readonly string DefaultStyleName = "Normal";

        /// <summary>
        /// The default style name for hyperlinks.
        /// </summary>
        public static readonly string HyperlinkStyleName = "Hyperlink";

        /// <summary>
        /// The minimum scale factor.
        /// </summary>
		public static readonly double MinScaleFactor = 0.5;

        /// <summary>
        /// The maximum scale factor.
        /// </summary>
		public static readonly double MaxScaleFactor = 4;

        /// <summary>
        /// The default scale factor.
        /// </summary>
        public const double DefaultScaleFactor = 1;

        /// <summary>
        /// The default scale factor size.
        /// </summary>
        public static readonly Size DefaultScaleFactorSize = new Size(1, 1);

        /// <summary>
        /// The minimum scale factor in the context of printing and pdf export.
        /// </summary>
        public static readonly double MinPageScaleFactor = 0.1;

        /// <summary>
        /// The maximum scale factor in the context of printing and pdf export.
        /// </summary>
		public static readonly double MaxPageScaleFactor = 4;

        /// <summary>
        /// The minimum count of pages to fit.
        /// </summary>
        public const int MinFitToPagesCount = 0;

        /// <summary>
        /// The maximum count of pages to fit.
        /// </summary>
        public const int MaxFitToPagesCount = Int16.MaxValue;

        /// <summary>
        /// The minimum number allowed to be used for first page number.
        /// </summary>
        public const int MinFirstPageNumber = Int16.MinValue + 3;

        /// <summary>
        /// The maximum number allowed to be used for first page number.
        /// </summary>
        public const int MaxFirstPageNumber = Int16.MaxValue;

        internal const double RowColumnHeadingSymbolWidth = 6.5;
        internal static readonly Size RowColumnHeadingMinimumSize = new Size(25, 20);
        internal static readonly Thickness RowColumnHeadingsPadding = new Thickness(3, 2, 3, 2);
        internal static readonly SizeI VisibleSize = new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount);
        internal static readonly Size ChartDefaultSize = new Size(480, 288);
        internal static readonly int DoughnutDefaultHoleSize = 75;
        internal static readonly double ChartLinesDefaultWidth = 0.75;

        internal static readonly SolidFill ChartLinesDefaultFill = new SolidFill(new ThemableColor(ThemeColorType.Text1, 0.85));
        internal static readonly SolidFill ChartDefaultFill = new SolidFill(new ThemableColor(ThemeColorType.Background1));

        #endregion
    }
}