﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// Represents base for the RowWidth and ColumnWidth classes.
    /// </summary>
    public abstract class WidthHeightBase
    {
        #region Fields

		private readonly double value;
		private readonly bool isCustom;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the value.
        /// </summary>
        /// <value>The value.</value>
        public double Value
        {
			get
			{
				return this.value;
			}
		}

        /// <summary>
        /// Gets a value indicating whether the width / height is custom.
        /// </summary>
        /// <value>The value indicating whether the width / height is custom.</value>
        public bool IsCustom
        {
			get
			{
				return this.isCustom;
			}
		}

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="WidthHeightBase" /> class.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="isCustom">The is custom.</param>
        protected WidthHeightBase(double value, bool isCustom)
        {
			this.value = value;
			this.isCustom = isCustom;
		}

		#endregion


        #region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            WidthHeightBase other = obj as WidthHeightBase;
            if (other == null)
            {
                return false;
            }

            return this.value == other.value && this.isCustom == other.isCustom;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(this.value.GetHashCode(), this.isCustom.GetHashCode());
        }

        /// <summary>
        /// Returns a <see cref="T:System.String" /> that represents the current
        /// <see cref="T:System.Object" />.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String" /> that represents the current <see cref="T:System.Object" />.
        /// </returns>
        public override string ToString()
        {
            return string.Format("[{0};{1}]", this.Value, this.IsCustom);
        }

        #endregion
    }
}