﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    /// <summary>
    /// A cell index in a specific worksheet.
    /// </summary>
    public class WorksheetCellIndex
    {
        #region Fields

        private readonly CellIndex cellIndex;
        private readonly Worksheet worksheet;

        #endregion


        #region Properties

        /// <summary>
        /// Gets the index of the cell.
        /// </summary>
        /// <value>The index of the cell.</value>
        public CellIndex CellIndex
        {
            get
            {
                return this.cellIndex;
            }
        }

        /// <summary>
        /// Gets the worksheet.
        /// </summary>
        /// <value>The worksheet.</value>
        public Worksheet Worksheet
        {
            get
            {
                return this.worksheet;
            }
        }

        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="WorksheetCellIndex" /> class.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="columnIndex">Index of the column.</param>
        public WorksheetCellIndex(Worksheet worksheet, int rowIndex, int columnIndex) 
            : this(worksheet, new CellIndex(rowIndex, columnIndex))
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorksheetCellIndex" /> class.
        /// </summary>
        /// <param name="worksheet">The worksheet.</param>
        /// <param name="cellIndex">Index of the cell.</param>
        public WorksheetCellIndex(Worksheet worksheet, CellIndex cellIndex)
        {
            Guard.ThrowExceptionIfNull(worksheet, "worksheet");
            Guard.ThrowExceptionIfNull(worksheet.Workbook, "worksheet.Workbook");
            Guard.ThrowExceptionIfNull(cellIndex, "cellIndex");

            this.worksheet = worksheet;
            this.cellIndex = cellIndex;
        }

        #endregion


        #region Methods

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            WorksheetCellIndex other = obj as WorksheetCellIndex;

            if (other == null)
            {
                return false;
            }

            return this.CellIndex.Equals(other.CellIndex) &&
                this.Worksheet.Equals(other.Worksheet);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(this.CellIndex.GetHashCode(), this.Worksheet.GetHashCode());
        }

        #endregion
    }
}