﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Model
{
    internal class WorksheetReference
    {
        private readonly Workbook workbook;
        private readonly bool isWorksheetAbsolute;
        private readonly Worksheet worksheet;
        private string worksheetName;

        public bool IsWorksheetAbsolute
        {
            get
            {
                return this.isWorksheetAbsolute;
            }
        }

        public string WorksheetName
        {
            get
            {
                return this.worksheetName;
            }
        }

        public Worksheet Worksheet
        {
            get
            {
                return this.worksheet;
            }
        }

        public Workbook Workbook
        {
            get
            {
                return this.workbook;
            }
        }

        internal WorksheetReference(Workbook workbook, string worksheetName, bool isWorksheetAbsolute)
        {
            Guard.ThrowExceptionIfNull(workbook, "workbook");
            Guard.ThrowExceptionIfNullOrEmpty(worksheetName, "worksheetName");

            this.workbook = workbook;
            this.worksheetName = worksheetName;
            this.isWorksheetAbsolute = isWorksheetAbsolute;

            this.worksheet = this.GetWorksheet();
        }

        private Worksheet GetWorksheet()
        {
            string decodedWorksheetName = TextHelper.DecodeWorksheetName(this.worksheetName);
            return SpreadsheetHelper.GetWorksheetByName(this.workbook, decodedWorksheetName);
        }

        // TODO:
        public override string ToString()
        {
            StringBuilder builder = new StringBuilder();

            if (this.isWorksheetAbsolute)
            {
                builder.Append(this.worksheetName);
                builder.Append(SpreadsheetCultureHelper.ExclamationMark);
            }

            return builder.ToString();
        }

        public WorksheetReference CloneAndTranslate(Worksheet worksheet)
        {
            string translatedWorksheetName = this.IsWorksheetAbsolute ? this.worksheetName : worksheet.Name;

            return new WorksheetReference(worksheet.Workbook, translatedWorksheetName, this.isWorksheetAbsolute);
        }

        internal void Translate()
        {
            if (this.Worksheet != null)
            {
                this.worksheetName = this.Worksheet.Name;
            }
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// True if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            WorksheetReference other = obj as WorksheetReference;

            if (other == null)
            {
                return false;
            }

            return TelerikHelper.EqualsOfT(this.workbook, other.workbook) &&
                TelerikHelper.EqualsOfT(this.worksheet, other.worksheet) &&
                TelerikHelper.EqualsOfT(this.isWorksheetAbsolute, other.isWorksheetAbsolute);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                this.workbook.GetHashCode(),
                this.worksheet.GetHashCodeOrZero(),
                this.isWorksheetAbsolute.GetHashCodeOrZero());
        }
    }
}
