﻿using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    internal class CellProperty<T> : PropertyBase<T, SetCellPropertyCommandContext<T>>
    {
        #region Properties

        protected CellsPropertyBag PropertyBag
        {
            get
            {
                return this.Worksheet.Cells.PropertyBag;
            }
        }

        #endregion


        #region Constructors

        public CellProperty(Worksheet worksheet, IPropertyDefinition<T> propertyDefinition)
            : base(worksheet, propertyDefinition)
        {
        }

        #endregion


        #region Methods

        protected override UndoableWorksheetCommandBase<SetCellPropertyCommandContext<T>> CreateSetPropertyCommand()
        {
            return new SetCellPropertyCommand<T>();
        }

        protected override SetCellPropertyCommandContext<T> CreateSetPropertyCommandContext(Worksheet worksheet, IPropertyDefinition<T> property, CellRange cellRange, T value)
        {
            return new SetCellPropertyCommandContext<T>(this.Worksheet, property, cellRange, value);
        }

        protected override SetCellPropertyCommandContext<T> CreateClearPropertyCommandContext(Worksheet worksheet, IPropertyDefinition<T> property, CellRange cellRange)
        {
            return new ClearCellPropertyCommandContext<T>(this.Worksheet, property, cellRange);
        }

        public override RangePropertyValue<T> GetValue(CellRange cellRange)
        {
            T value;
            bool isIndeterminate = false;
            IEnumerable<LongRange> rangeParts = CellsPropertyBag.ConvertCellRangeToLongRanges(cellRange);

            bool isFirstSet = true;
            T result = default(T);
            foreach (LongRange rangePart in rangeParts)
            {
                var rangePartValues = this.Worksheet.Cells.PropertyBag.GetPropertyValueRespectingStyle(this.PropertyDefinition, this.Worksheet, rangePart.Start, rangePart.End);
                foreach (var rangePartValue in rangePartValues)
                {
                    if (isFirstSet)
                    {
                        result = rangePartValue.Value;
                        isFirstSet = false;
                    }
                    else if (!TelerikHelper.EqualsOfT(result, rangePartValue.Value))
                    {
                        value = this.GetDefaultValue();
                        isIndeterminate = true;
                    }
                }
            }

            value = result;

            return new RangePropertyValue<T>(isIndeterminate, value);
        }

        protected override bool ValidateCellRange(CellRange cellRange)
        {
            return true;
        }

        #endregion
    }
}