﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    /// <summary>
    /// Represents cell property changed event arguments.
    /// </summary>
    public class CellPropertyChangedEventArgs : EventArgs
    {
		#region Fields

		private readonly IPropertyDefinition property;
		private readonly CellRange cellRange;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the property.
        /// </summary>
        /// <value>The property.</value>
        public IPropertyDefinition Property
        {
			get
			{
				return this.property;
			}
		}

        /// <summary>
        /// Gets the cell range.
        /// </summary>
        /// <value>The cell range.</value>
        public CellRange CellRange
        {
			get
			{
				return this.cellRange;
			}
		}

        internal bool ShouldRespectAffectLayoutProperty { get; set; }

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CellPropertyChangedEventArgs" /> class.
        /// </summary>
        /// <param name="property">The property.</param>
        /// <param name="cellRange">The cell range.</param>
        public CellPropertyChangedEventArgs(IPropertyDefinition property, CellRange cellRange)
        {
			Guard.ThrowExceptionIfNull(property, "property");
			Guard.ThrowExceptionIfNull(cellRange, "cellRange");

			this.property = property;
			this.cellRange = cellRange;
            this.ShouldRespectAffectLayoutProperty = true;
		}

		#endregion
	}
}