﻿namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    /// <summary>
    /// Represents property definition.
    /// </summary>
    public interface IPropertyDefinition
    {
        /// <summary>
        /// Gets a value indicating whether to use same value as previous on insert.
        /// </summary>
        /// <value>The value indicating whether to use same value as previous on insert.</value>
        bool UseSameValueAsPreviousOnInsert { get; }

        /// <summary>
        /// Gets the name of the property.
        /// </summary>
        /// <value>The name of the property.</value>
        string Name { get; }

        /// <summary>
        /// Gets a value indicating whether the property affects layout.
        /// </summary>
        /// <value>The value indicating whether the property affects layout.</value>
        bool AffectsLayout { get; }

        /// <summary>
        /// Gets the style property group.
        /// </summary>
        /// <value>The style property group.</value>
        StylePropertyGroup StylePropertyGroup { get; }
    }

    /// <summary>
    /// Represents property definition.
    /// </summary>
    /// <typeparam name="T">The type of T.</typeparam>
    public interface IPropertyDefinition<T> : IPropertyDefinition
    { 
        /// <summary>
        /// Gets the default value.
        /// </summary>
        /// <value>The default value.</value>
        T DefaultValue { get; }
    }
}