﻿using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    /// <summary>
    /// Represents property definition.
    /// </summary>
    /// <typeparam name="T">The type of the T.</typeparam>
    public class PropertyDefinition<T> : IPropertyDefinition<T>
    {
		#region Fields

		private readonly string name;
		private readonly T defaultValue;
		private readonly bool affectsLayout;
        private readonly bool useSameValueAsPreviousOnInsert;
        private readonly StylePropertyGroup stylePropertyGroup;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the name of the property.
        /// </summary>
        /// <value>The name of the property.</value>
        public string Name
        {
			get
			{
				return this.name;
			}
		}

        /// <summary>
        /// Gets the default value.
        /// </summary>
        /// <value>The default value.</value>
        public T DefaultValue
        {
			get
			{
				return this.defaultValue;
			}
		}

        /// <summary>
        /// Gets a value indicating whether the property affects layout.
        /// </summary>
        /// <value>The value indicating whether the property affects layout.</value>
        public bool AffectsLayout
        {
			get
			{
				return this.affectsLayout;
			}
		}

        /// <summary>
        /// Gets a value indicating whether to use same value as previous on insert.
        /// </summary>
        /// <value>
        /// The value indicating whether to use same value as previous on insert.
        /// </value>
        public bool UseSameValueAsPreviousOnInsert
        {
            get
            {
                return this.useSameValueAsPreviousOnInsert;
            }
        }

        /// <summary>
        /// Gets the style property group.
        /// </summary>
        /// <value>The style property group.</value>
        public StylePropertyGroup StylePropertyGroup
        {
            get
            {
                return this.stylePropertyGroup;
            }
        }

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PropertyDefinition{T}" /> class.
        /// </summary>
        /// <param name="name">The name of the property.</param>
        /// <param name="affectsLayout">The affects layout.</param>
        /// <param name="stylePropertyGroup">The style property group.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="useSameValueAsPreviousOnInsert">The use same value as previous on insert.</param>
        public PropertyDefinition(
            string name,
            bool affectsLayout,
            StylePropertyGroup stylePropertyGroup,
            T defaultValue = default(T),
            bool useSameValueAsPreviousOnInsert = true)
		{
			Guard.ThrowExceptionIfNullOrEmpty(name, "name");

			this.name = name;
			this.affectsLayout = affectsLayout;
            this.stylePropertyGroup = stylePropertyGroup;
			this.defaultValue = defaultValue;
            this.useSameValueAsPreviousOnInsert = useSameValueAsPreviousOnInsert;
		}

		#endregion
	}
}