﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Commands.WorksheetCommands;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    internal abstract class RowColumnProperty<TValue, TContext> : PropertyBase<TValue, TContext>
        where TContext : SetRowColumnPropertyCommandContext<TValue>
    {
        protected abstract RowColumnPropertyBagBase PropertyBag
        {
            get;
        }

        public RowColumnProperty(Worksheet worksheet, IPropertyDefinition<TValue> propertyDefinition)
            : base(worksheet, propertyDefinition)
        {
        }

        private RangePropertyValue<TValue> GetRangePropertyValue<T>(IPropertyDefinition<TValue> property, int fromIndex, int toIndex)
        {
            TValue value = default(TValue);
            bool isIndeterminate = false;

            var rangeValues = this.PropertyBag.GetPropertyValueRespectingStyle(property, this.Worksheet, fromIndex, toIndex);
            bool isFirstSet = true;

            foreach (var rangePartValue in rangeValues)
            {
                if (isFirstSet)
                {
                    value = rangePartValue.Value;
                    isFirstSet = false;
                }
                else if (!TelerikHelper.EqualsOfT(value, rangePartValue.Value))
                {
                    value = this.GetDefaultValue();
                    isIndeterminate = true;
                }
            }

            return new RangePropertyValue<TValue>(isIndeterminate, value);
        }

        internal abstract int GetRowColumnIndex(CellIndex index);

        public override RangePropertyValue<TValue> GetValue(CellRange cellRange)
        {
            int fromIndex = this.GetRowColumnIndex(cellRange.FromIndex);
            int toIndex = this.GetRowColumnIndex(cellRange.ToIndex);

            return this.GetRangePropertyValue<TValue>(this.PropertyDefinition, fromIndex, toIndex);
        }

        protected override TContext CreateSetPropertyCommandContext(Worksheet worksheet, IPropertyDefinition<TValue> property, CellRange cellRange, TValue value)
        {
            int fromIndex = this.GetRowColumnIndex(cellRange.FromIndex);
            int toIndex = this.GetRowColumnIndex(cellRange.ToIndex);

            return (TContext)Activator.CreateInstance(typeof(TContext), worksheet, property, fromIndex, toIndex, value);
        }

        protected override TContext CreateClearPropertyCommandContext(Worksheet worksheet, IPropertyDefinition<TValue> property, CellRange cellRange)
        {
            int fromIndex = this.GetRowColumnIndex(cellRange.FromIndex);
            int toIndex = this.GetRowColumnIndex(cellRange.ToIndex);

            return new ClearRowColumnPropertyCommandContext<TValue>(worksheet, property, fromIndex, toIndex) as TContext;
        }
    }
}