﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    /// <summary>
    /// Represents row or column property changed event arguments.
    /// </summary>
    public class RowColumnPropertyChangedEventArgs : EventArgs
    {
		#region Fields

		private readonly IPropertyDefinition property;
		private readonly int fromIndex;
		private readonly int toIndex;

		#endregion


		#region Properties

        /// <summary>
        /// Gets the changed property.
        /// </summary>
        /// <value>The changed property.</value>
        public IPropertyDefinition Property
        {
			get
			{
				return this.property;
			}
		}

        /// <summary>
        /// Gets the first index of the changed range.
        /// </summary>
        /// <value>The first index of the changed range.</value>
        public int FromIndex
        {
			get
			{
				return this.fromIndex;
			}
		}

        /// <summary>
        /// Gets the last index of the changed range.
        /// </summary>
        /// <value>The last index of the changed range.</value>
        public int ToIndex
        {
			get
			{
				return this.toIndex;
			}
		}

		#endregion


		#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RowColumnPropertyChangedEventArgs" /> class.
        /// </summary>
        /// <param name="property">The property.</param>
        /// <param name="fromIndex">The first index of the changed range.</param>
        /// <param name="toIndex">The last index of the changed range.</param>
        public RowColumnPropertyChangedEventArgs(IPropertyDefinition property, int fromIndex, int toIndex)
        {
			Guard.ThrowExceptionIfNull(property, "property");
			Guard.ThrowExceptionIfLessThan(0, fromIndex, "fromIndex");
			Guard.ThrowExceptionIfLessThan(0, toIndex, "toIndex");

			this.property = property;
			this.fromIndex = fromIndex;
			this.toIndex = toIndex;
		}

		#endregion
	}
}
