﻿using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
	internal class RowsPropertyBag : RowColumnPropertyBagBase
	{
		#region Property Definitions

		public static readonly IPropertyDefinition<RowHeight> HeightProperty =
			new PropertyDefinition<RowHeight>("RowHeight", true, StylePropertyGroup.None, new RowHeight(SpreadsheetDefaultValues.DefaultRowHeight, false));

        private static readonly RowHeight HiddenHeight = new RowHeight(0, true);

		#endregion


		#region Properties

        protected override WidthHeightBase HiddenSize
        {
            get 
            { 
                return HiddenHeight;
            }
        }

		protected override long ToItemIndex
		{
			get
			{
				return SpreadsheetDefaultValues.RowCount - 1;
			}
		}

		#endregion


		#region Constructors

		public RowsPropertyBag()
		{
			base.RegisterProperty(RowsPropertyBag.HeightProperty);
		}

		#endregion


		#region Methods

        internal override int GetRowColumnIndex(int rowIndex, int columnIndex)
        {
            return rowIndex;
        }

		public void CopyPropertiesFrom(RowsPropertyBag fromProperties, long fromIndex, long toIndex)
		{
			base.CopyPropertiesFrom(fromProperties, fromIndex, toIndex);
		}

        public ICompressedList<RowHeight> GetRowHeightPropertyValueRespectingHidden()
        {
            return base.GetSizePropertyValueRespectingHidden<RowHeight>(HeightProperty);
        }

        internal void Sort(CellRange sortedRange, int[] sortedIndexes)
        {
            var fromLongIndex = CellsPropertyBag.ConvertCellIndexToLong(sortedRange.FromIndex.RowIndex, 0);
            var toLongIndex = CellsPropertyBag.ConvertCellIndexToLong(sortedRange.ToIndex.RowIndex, 0);
            this.GetPropertyValueCollection(RowsPropertyBag.HeightProperty).Sort(fromLongIndex, toLongIndex, sortedIndexes);

            this.OnPropertyChanged(RowsPropertyBag.HeightProperty, fromLongIndex, toLongIndex);
        }

		#endregion
	}
}