﻿using Telerik.Windows.Documents.Spreadsheet.Commands.WorkbookCommands;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.PropertySystem
{
    internal class StyleProperty<T> : StylePropertyBase
    {
        #region Fields

        private readonly CellStyle cellStyle;
        private readonly IPropertyDefinition<T> propertyDefinition;
        private T value;
        private bool isValueSet;

        #endregion


        #region Properties

        public bool IsValueSet
        {
            get
            {
                return this.isValueSet;
            }
        }

        public IPropertyDefinition<T> PropertyDefinition
        {
            get
            {
                return this.propertyDefinition;
            }
        }

        public override bool AffectsLayout
        {
            get
            {
                return this.propertyDefinition.AffectsLayout;
            }
        }

        #endregion


        #region Constructors

        internal StyleProperty(CellStyle cellStyle, IPropertyDefinition<T> propertyDefinition)
        {
            Guard.ThrowExceptionIfNull(cellStyle, "cellStyle");
            Guard.ThrowExceptionIfNull(propertyDefinition, "propertyDefinition");

            this.cellStyle = cellStyle;
            this.propertyDefinition = propertyDefinition;
            this.value = propertyDefinition.DefaultValue;
        }

        #endregion


        #region Methods

        public override object GetValueAsObject()
        {
            return this.GetValue();
        }

        public override void SetValue(object value)
        {
            this.SetValue((T)value);
        }

        public T GetValue()
        {
            return this.value;
        }

        //public void ClearValue()
        //{
        //}

        public void SetValue(T value)
        {
            if (!this.cellStyle.IsRemoved)
            {
                var context = new SetStylePropertyCommandContext(this.cellStyle.Workbook, this, this.value, value);
                this.cellStyle.Workbook.ExecuteCommand(WorkbookCommands.SetStyleProperty, context);
            }
            else
            {
                this.SetValueInternal(value);
            }
        }

        internal override void SetValueInternal(object value)
        {
            this.SetValueInternal((T)value);
        }

        internal void SetValueInternal(T value)
        {
            this.isValueSet = true;

            if (!TelerikHelper.EqualsOfT(this.value, value))
            {
                this.value = value;
                this.OnValueChanged();
            }
        }

        private void OnValueChanged(/*T oldValue, T newValue*/)
        {
            // TODO: Using old/new values?
            this.cellStyle.OnPropertyChanged(this.propertyDefinition);
        }

        #endregion
    }
}