﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Copying;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    internal class CellStyleCollectionsMerger
    {
        private readonly CopyContext context;
        private readonly CellStyleCollection targetStyles;
        private readonly CellStyleCollection sourceStyles;

        private enum MergeOperation
        {
            None,
            Add,
            RenameAndAdd
        }

        public CellStyleCollectionsMerger(CellStyleCollection targetStyles, CellStyleCollection sourceStyles, CopyContext context)
        {
            Guard.ThrowExceptionIfNull(targetStyles, "targetStyles");
            Guard.ThrowExceptionIfNull(sourceStyles, "sourceStyles");
            Guard.ThrowExceptionIfNull(context, "context");

            this.targetStyles = targetStyles;
            this.sourceStyles = sourceStyles;
            this.context = context;
        }

        public void Merge()
        {
            foreach (CellStyle style in this.sourceStyles)
            {
                MergeOperation operation = GetMergeOperationForStyle(style);
                switch (operation)
                {
                    case MergeOperation.Add:
                        CellStyle newStyle = this.targetStyles.Add(style.Name, style.Category, style.IsRemovable);
                        newStyle.CopyPropertiesFrom(style);
                        break;

                    case MergeOperation.RenameAndAdd:
                        string renamedStyleName = ComputeRenamedStyleName(style.Name);
                        CellStyle renamedStyle = this.targetStyles.Add(renamedStyleName, style.Category, style.IsRemovable);
                        renamedStyle.CopyPropertiesFrom(style);

                        this.context.OriginalToRenamedStyleNames.Add(style.Name, renamedStyleName);
                        break;

                    case MergeOperation.None:
                    default:
                        break;
                }
            }
        }

        private string ComputeRenamedStyleName(string styleName)
        {
            int index = 1;
            string computedStyleName = styleName;

            do
            {
                computedStyleName = string.Format("{0} {1}", styleName, index);
                index++;
            }
            while (this.targetStyles.Contains(computedStyleName));

            return computedStyleName;
        }

        private MergeOperation GetMergeOperationForStyle(CellStyle style)
        {
            if (!this.targetStyles.Contains(style.Name))
            {
                return MergeOperation.Add;
            }
            else
            {
                CellStyle targetStyle = this.targetStyles[style.Name];
                if (!targetStyle.Equals(style))
                {
                    return MergeOperation.RenameAndAdd;
                }
            }

            return MergeOperation.None;
        }
    }
}