﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    internal static class FindAndReplaceHelper
    {
        public static bool MoreThan(this CellIndex left, CellIndex right, FindBy searchBy)
        {
            if (searchBy == FindBy.Rows)
            {
                return left > right;
            }
            else
            {
                if (left == null)
                {
                    return false;
                }

                if (left.ColumnIndex > right.ColumnIndex)
                {
                    return true;
                }

                if (left.ColumnIndex == right.ColumnIndex)
                {
                    return left.RowIndex > right.RowIndex;
                }

                return false;
            }
        }

        public static bool MoreThan(this WorksheetCellIndex left, WorksheetCellIndex right, FindBy searchBy)
        {
            Guard.ThrowExceptionIfNotEqual(left.Worksheet.Workbook, right.Worksheet.Workbook, 
                "worksheets should be in the same workbook");

            int leftWorksheetIndex = GetWorksheetIndex(left.Worksheet);
            int rightWorksheetIndex = GetWorksheetIndex(right.Worksheet);

            if (leftWorksheetIndex == rightWorksheetIndex)
            {
                return left.CellIndex.MoreThan(right.CellIndex, searchBy);
            }
            else
            {
                return leftWorksheetIndex > rightWorksheetIndex;
            }
        }

        public static bool LessOrEqual(this WorksheetCellIndex left, WorksheetCellIndex right, FindBy searchBy)
        {
            return !left.MoreThan(right, searchBy);
        }

        public static IEnumerable<FindResult> OrderResults(FindOptions findOptions, IEnumerable<FindResult> findResults)
        {
            IEnumerable<FindResult> resultsPart1 = findResults.TakeWhile(
                (result) =>
                {
                    bool res = result.FoundCell.LessOrEqual(findOptions.StartCell, findOptions.FindBy);
                    return res;
                });

            var resultsPart2 = findResults.Skip(resultsPart1.Count());

            findResults = resultsPart2.Concat(resultsPart1);
            return findResults;
        }

        public static bool FindInRawValue(Range<CellIndex, ICellValue> range, FindOptions findOptions)
        {
            string rangeValue = range.Value.RawValue;

            return FindInString(rangeValue, findOptions);
        }

        public static bool FindInResultValue(Cells cells, Range<CellIndex, ICellValue> range, FindOptions findOptions)
        {
            CellValueFormat format = cells[range.Start].GetFormat().Value;
            string rangeValue = range.Value.GetResultValueAsString(format);

            return FindInString(rangeValue, findOptions);
        }

        public static bool FindInString(string rangeValue, FindOptions findOptions)
        {
            if (string.IsNullOrEmpty(rangeValue))
            {
                return false;
            }
            else
            {
                return Regex.Match(rangeValue, findOptions.FindWhatRegex, findOptions.FindWhatRegexOptions).Success;
            }
        }

        public static int GetWorksheetIndex(Worksheet worksheet)
        {
            return worksheet.Workbook.Worksheets.IndexOf(worksheet);
        }
    }
}