﻿using System;
using System.Linq;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    /// <summary>
    /// Represents a base class for header name rendering converter.
    /// </summary>
    public abstract class HeaderNameRenderingConverterBase
    {
        internal string ConvertRowIndexToName(HeaderNameRenderingConverterContext context, int rowIndex)
        {
            return this.ConvertRowIndexToNameOverride(context, rowIndex);
        }

        internal string ConvertColumnIndexToName(HeaderNameRenderingConverterContext context, int columnIndex)
        {
            return this.ConvertColumnIndexToNameOverride(context, columnIndex);
        }

        /// <summary>
        /// Converts the name of the row index to.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <returns>The row name.</returns>
        protected virtual string ConvertRowIndexToNameOverride(HeaderNameRenderingConverterContext context, int rowIndex)
        {
            return NameConverter.ConvertRowIndexToName(rowIndex);
        }

        /// <summary>
        /// Converts the name of the column index to.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <param name="columnIndex">Index of the column.</param>
        /// <returns>The column name.</returns>
        protected virtual string ConvertColumnIndexToNameOverride(HeaderNameRenderingConverterContext context, int columnIndex)
        {
            return NameConverter.ConvertColumnIndexToName(columnIndex);
        }
    }
}