﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    /// <summary>
    /// Represents localizable exception.
    /// </summary>
    public class LocalizableException : Exception
    {
        private string localizationKey;
        private string[] formatStringArguments = new string[0];

        /// <summary>
        /// Gets or sets the localization key.
        /// </summary>
        /// <value>The localization key.</value>
        public string LocalizationKey
        {
            get
            {
                return this.localizationKey;
            }
            protected set
            {
                this.localizationKey = value;
            }
        }

        /// <summary>
        /// Gets the format string arguments.
        /// </summary>
        /// <value>The format string arguments.</value>
        public string[] FormatStringArguments
        {
            get
            {
                return this.formatStringArguments;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LocalizableException" /> class.
        /// </summary>
        public LocalizableException() : base()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LocalizableException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public LocalizableException(string message) : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LocalizableException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="key">The key.</param>
        /// <param name="formatStringArguments">The format string arguments.</param>
        public LocalizableException(string message, string key, string[] formatStringArguments = null) : base(message)
        {
            this.localizationKey = key;

            if (formatStringArguments != null)
            {
                this.formatStringArguments = formatStringArguments;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LocalizableException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="innerException">The inner exception.</param>
        public LocalizableException(string message, Exception innerException) : base(message, innerException)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LocalizableException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="innerException">The inner exception.</param>
        /// <param name="key">The key.</param>
        /// <param name="formatStringArguments">The format string arguments.</param>
        public LocalizableException(string message, Exception innerException, string key, string[] formatStringArguments = null) : base(message, innerException)
        {
            this.localizationKey = key;

            if (formatStringArguments != null)
            {
                this.formatStringArguments = formatStringArguments;
            }
        }
    }
}
