﻿using System;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities.Performance
{
	internal class SnippetMemoryCounter
	{
		#region Fields

		private ILogger logger;
		private string snippetName;
		private long totalMemory;
		private bool isStartCalled;
		private long currentStartMemory;
		private bool hasBeenStarted;
		private int hitCounter;

		#endregion


		#region Properties

		public double TotalMemory
		{
			get
			{
				return this.totalMemory;
			}
		}

		public double TotalMemoryMB
		{
			get
			{
				return ((this.totalMemory / 1024f) / 1024f);
			}
		}
		
		#endregion


		#region Constructors

		public SnippetMemoryCounter(string snippetName)
		{
			this.logger = new DebugLogger();
			this.snippetName = snippetName;
			this.totalMemory = 0;
			this.isStartCalled = false;
			this.hasBeenStarted = false;
		}

		#endregion


		#region Methods

		public void Start()
		{
			if (!this.hasBeenStarted)
			{
				this.logger.Log(string.Format("[{0}]: FIRST TIME STARTED", this.snippetName));
				this.hasBeenStarted = true;
			}

			if (!this.isStartCalled)
			{
				GC.WaitForPendingFinalizers();
				GC.Collect();

				//Thread.CurrentThread.
				this.currentStartMemory = GC.GetTotalMemory(true);
				this.isStartCalled = true;
			}

			hitCounter++;
		}

		public void Stop()
		{
			if (this.isStartCalled)
			{
				GC.Collect();
				GC.WaitForPendingFinalizers();
				GC.Collect();

				this.totalMemory += (GC.GetTotalMemory(true) - this.currentStartMemory);
				this.isStartCalled = false;
			}
		}

		public void End()
		{
			this.Stop();

			if (this.hasBeenStarted)
			{
				this.logger.Log(string.Format("[{0}]: ENDED WITH TOTAL MEMORY:{1}; TIMES HIT: {2}; TOTAL MEMORY: {3}MB",
					this.snippetName, this.totalMemory, this.hitCounter, GC.GetTotalMemory(true)));
			}
		}

		#endregion
	}
}