﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    /// <summary>
    /// A converter for the ThemableColor class. 
    /// </summary>
    public class ThemableColorTypeConverter : TypeConverter
    {
        #region Constants

        private static readonly Dictionary<string, Color> ColorNameToColor;

        #endregion


        #region Constructors

        static ThemableColorTypeConverter()
        {
            ColorNameToColor = new Dictionary<string, Color>();

            ColorNameToColor.Add("Black", Colors.Black);
            ColorNameToColor.Add("Blue", Colors.Blue);
            ColorNameToColor.Add("Brown", Colors.Brown);
            ColorNameToColor.Add("Cyan", Colors.Cyan);
            ColorNameToColor.Add("DarkGray", Colors.DarkGray);
            ColorNameToColor.Add("Gray", Colors.Gray);
            ColorNameToColor.Add("Green", Colors.Green);
            ColorNameToColor.Add("LightGray", Colors.LightGray);
            ColorNameToColor.Add("Magenta", Colors.Magenta);
            ColorNameToColor.Add("Orange", Colors.Orange);
            ColorNameToColor.Add("Purple", Colors.Purple);
            ColorNameToColor.Add("Red", Colors.Red);
            ColorNameToColor.Add("Transparent", Colors.Transparent);
            ColorNameToColor.Add("White", Colors.White);
            ColorNameToColor.Add("Yellow", Colors.Yellow);
        }

        #endregion


        #region Methods

        /// <inheritdoc />
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return (sourceType == typeof(string));
        }

        /// <inheritdoc />
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return (destinationType == typeof(string));
        }

        /// <inheritdoc />
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            string color = (string)value;

            if (ColorNameToColor.ContainsKey(color))
            {
                return new ThemableColor(ColorNameToColor[color]);
            }

            color = color.Substring(1, color.Length - 1);

            UInt32 hexColor = UInt32.Parse(color, System.Globalization.NumberStyles.HexNumber);

            byte a = (byte)(hexColor >> 24);
            byte r = (byte)(hexColor >> 16);
            byte g = (byte)(hexColor >> 8);
            byte b = (byte)(hexColor);

            return new ThemableColor(Color.FromArgb(a, r, g, b));
        }

        #endregion
    }
}