﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    internal static class SortAndFilterHelper
    {
        public static bool IsValidSelection(IEnumerable<CellRange> cellRanges)
        {
            if (cellRanges.Count() > 1)
            {
                return false;
            }

            return true;
        }

        public static bool IsEmptySortRange(CellSelection selection)
        {
            RangePropertyValue<ICellValue> value = selection.GetValue();
            RangePropertyValue<IFill> fill = selection.GetFill();
            RangePropertyValue<ThemableColor> foreColor = selection.GetForeColor();

            bool boolHasValue = true;
            if (!value.IsIndeterminate)
            {
                boolHasValue = !(value.Value is EmptyCellValue);
            }

            bool hasFill = true;
            if (!fill.IsIndeterminate)
            {
                hasFill = !fill.Value.Equals(PatternFill.Default);
            }

            bool hasForeColor = true;
            if (!foreColor.IsIndeterminate)
            {
                hasForeColor = !foreColor.Value.Equals(SpreadsheetDefaultValues.DefaultForeColor);
            }

            if (!(boolHasValue || hasFill || hasForeColor))
            {
                return true;
            }

            return false;
        }

        public static IFill GetIFillWithLocalColors(IFill fill, ThemeColorScheme scheme)
        {
            PatternFill patternFill = fill as PatternFill;
            if (patternFill != null)
            {
                if (patternFill.PatternColor.IsFromTheme)
                {
                    Color patternActualColor = patternFill.PatternColor.GetActualValue(scheme);
                    return new PatternFill(patternFill.PatternType, new ThemableColor(patternActualColor), patternFill.BackgroundColor);
                }
            }

            GradientFill gradientFill = fill as GradientFill;
            if (gradientFill != null)
            {
                if (gradientFill.Color1.IsFromTheme || gradientFill.Color2.IsFromTheme)
                {
                    Color color1ActualColor = gradientFill.Color1.GetActualValue(scheme);
                    Color color2ActualColor = gradientFill.Color2.GetActualValue(scheme);

                    return new GradientFill(gradientFill.GradientType, color1ActualColor, color2ActualColor);
                }
            }

            return fill;
        }

        public static ThemableColor GetThemableColorWithLocalColors(ThemableColor color, ThemeColorScheme scheme)
        {
            if (color.IsFromTheme)
            {
                Color actualColor = color.GetActualValue(scheme);

                return new ThemableColor(actualColor);
            }

            return color;
        }

        public static CellRange GetActualFilteredRange(CellRange filterRange, Worksheet worksheet)
        {
            if (filterRange == null)
            {
                return null;
            }

            int fromIndexRow = filterRange.FromIndex.RowIndex + 1;
            int fromIndexCol = filterRange.FromIndex.ColumnIndex;
            int toIndexRow = Math.Min(filterRange.ToIndex.RowIndex, worksheet.UsedCellRange.ToIndex.RowIndex);
            int toIndexCol = filterRange.ToIndex.ColumnIndex;

            if (fromIndexRow > toIndexRow)
            {
                return null;
            }

            return new CellRange(fromIndexRow, fromIndexCol, toIndexRow, toIndexCol);
        }
    }
}
