﻿using System;
using System.IO;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
	internal static class TelerikHelper
	{
		public static int CombineHashCodes(int h1, int h2)
		{
			return (((h1 << 5) + h1) ^ h2);
		}

		public static int CombineHashCodes(int h1, int h2, int h3)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2), h3);
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2), CombineHashCodes(h3, h4));
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4, int h5)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2, h3, h4), h5);
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4, int h5, int h6)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2, h3, h4), CombineHashCodes(h5, h6));
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4, int h5, int h6, int h7)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2, h3, h4), CombineHashCodes(h5, h6, h7));
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4, int h5, int h6, int h7, int h8)
		{
			return CombineHashCodes(CombineHashCodes(h1, h2, h3, h4), CombineHashCodes(h5, h6, h7, h8));
		}

		public static int CombineHashCodes(int h1, int h2, int h3, int h4, int h5, int h6, int h7, int h8, int h9)
		{
			return CombineHashCodes(CombineHashCodes(CombineHashCodes(h1, h2, h3, h4), CombineHashCodes(h5, h6, h7, h8)), h9);
		}

        // TODO: Test if this is correct.
        public static int CombineHashCodes(int h1, int h2, params int[] h)
        {
            int result = CombineHashCodes(h1, h2);

            foreach (int hash in h)
            {
                result = CombineHashCodes(result, hash);
            }

            return result;
        }

		public static void Swap<T>(ref T first, ref T second)
		{
			T temp = first;
			first = second;
			second = temp;
		}

		public static T Max<T>(T first, T second)
			where T : IComparable<T>
		{
			return (first.CompareTo(second) > 0) ? first : second;
		}

		public static T Min<T>(T first, T second)
			where T : IComparable<T>
		{
			return (first.CompareTo(second) < 0) ? first : second;
		}

#if SILVERLIGHT
        public static DependencyObject GetFocusedElement()
        {
            return (DependencyObject)FocusManager.GetFocusedElement();
        }
#else
		public static DependencyObject GetFocusedElement(UIElement element)
		{
            return (DependencyObject)FocusManager.GetFocusedElement(FocusManager.GetFocusScope(element));
		}
#endif
        public static bool EqualsOfT<T>(T first, T second)
        {
            return (first == null && second == null) || (first != null && first.Equals(second));
        }

        public static bool IsValidRowIndex(int rowIndex)
        {
            return rowIndex >= 0 && rowIndex < SpreadsheetDefaultValues.RowCount;
        }

        public static bool IsValidFontSizeInPoints(double fontSize)
        {
            return fontSize >= 1 && fontSize <= 409;
        }

        public static bool IsValidColumnIndex(int columnIndex)
        {
            return columnIndex >= 0 && columnIndex < SpreadsheetDefaultValues.ColumnCount;
        }
    }
}