﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Telerik.Windows.Documents.Spreadsheet.Utilities
{
    internal static class WildcardStringExtensions
    {
        #region Fields

        private static readonly Dictionary<string, string> wildcardSymbolToRegex;

        #endregion


        #region Constructor

        static WildcardStringExtensions()
        {
            WildcardStringExtensions.wildcardSymbolToRegex = new Dictionary<string, string>();
            wildcardSymbolToRegex[SpreadsheetCultureHelper.Asterisk] = ".*";
            wildcardSymbolToRegex[SpreadsheetCultureHelper.QuestionMark] = ".{1}";
            wildcardSymbolToRegex[SpreadsheetCultureHelper.Tilde] = Regex.Escape(SpreadsheetCultureHelper.Tilde);
        }

        #endregion


        #region Methods

        public static string FromWildcardStringToRegex(this string wildcardString)
        {
            StringBuilder regex = new StringBuilder();

            foreach (string token in wildcardString.TokenizeWildcardString())
            {
                if (token.IsWildcardCharacter())
                {
                    regex.Append(WildcardStringExtensions.wildcardSymbolToRegex[token]);
                }
                else
                {
                    regex.Append(Regex.Escape(token));
                }
            }

            return regex.ToString();
        }

        public static bool IsWildcardCharacter(this char character)
        {
            return character == SpreadsheetCultureHelper.Asterisk[0] ||
                character == SpreadsheetCultureHelper.QuestionMark[0] ||
                character == SpreadsheetCultureHelper.Tilde[0];
        }

        public static bool IsWildcardCharacter(this string character)
        {
            return character.Length == 1 && character[0].IsWildcardCharacter();
        }

        public static bool IsNonTildeWildcardCharacter(this char character)
        {
            return character != SpreadsheetCultureHelper.Tilde[0] && character.IsWildcardCharacter();
        }

        private static IEnumerable<string> TokenizeWildcardString(this string wildcardString)
        {
            StringBuilder token = new StringBuilder();

            for (int i = 0; i < wildcardString.Length; i++)
            {
                char character = wildcardString[i];

                if (character.IsWildcardCharacter())
                {
                    if (character == SpreadsheetCultureHelper.Tilde[0])
                    {
                        if (i < wildcardString.Length - 1 && wildcardString[i + 1].IsWildcardCharacter())
                        {
                            token.Append(wildcardString[i + 1]);
                            i++;
                        }
                    }
                    else
                    {
                        string tokenToYield = token.ToString();
                        if (tokenToYield.Length > 0)
                        {
                            yield return tokenToYield;
                            token.Clear();
                        }

                        yield return character.ToString();
                    }
                }
                else
                {
                    token.Append(character);
                }
            }

            string lastToken = token.ToString();
            if (lastToken.Length > 0)
            {
                yield return lastToken;
            }
        }

        #endregion
    }
}
