﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Data
{
    internal class XlsxDataWorksheetExportContext : IXlsxWorksheetExportContext
    {
        private readonly DataWorksheet worksheet;
        private readonly XlsxDataWorkbookExportContext context;

        public XlsxDataWorksheetExportContext(XlsxDataWorkbookExportContext context, DataWorksheet worksheet)
        {
            this.context = context;
            this.worksheet = worksheet;
        }

        public IXlsxWorkbookExportContext WorkbookContext
        {
            get
            {
                return this.context;
            }
        }

        public int SheetNo
        {
            get
            {
                return this.worksheet.No;
            }
        }

        public IEnumerable<RowInfo> GetNonEmptyRows()
        {
            return this.worksheet.GetNonEmptyRows();
        }

        public IEnumerable<CellInfo> GetNonEmptyCellsInRow(int rowIndex)
        {
            return this.worksheet.GetNonEmptyCellsInRow(rowIndex);
        }

        public int GetSharedStringIndex(TextCellValue textCellValue)
        {
            return -1;
        }

        public double GetDefaultColumnWidth()
        {
            return 10;
        }

        public RowHeight GetDefaultRowHeight()
        {
            return new RowHeight(100, false);
        }

        public string WorksheetName
        {
            get
            {
                return this.worksheet.Name;
            }
        }

        public DataWorksheet Worksheet
        {
            get
            {
                return this.worksheet;
            }
        }

        public IEnumerable<MergedCellInfo> GetMergedCells()
        {
            return Enumerable.Empty<MergedCellInfo>();
        }

        public IEnumerable<ColumnInfo> GetNonEmptyColumns()
        {
            return Enumerable.Empty<ColumnInfo>();
        }

        public IEnumerable<SpreadsheetHyperlink> Hyperlinks
        {
            get
            {
                return Enumerable.Empty<SpreadsheetHyperlink>();
            }
        }

        public int? GetFormattingRecordIndex(CellIndex cellIndex)
        {
            return null;
        }

        public IEnumerable<FontInfo> GetFontInfos()
        {
            return Enumerable.Empty<FontInfo>();
        }

        public IEnumerable<IFill> GetFills()
        {
            return Enumerable.Empty<IFill>();
        }

        public IEnumerable<CellValueFormat> GetCellValueFormats()
        {
            return Enumerable.Empty<CellValueFormat>();
        }

        public IEnumerable<BordersInfo> GetBorderInfos()
        {
            return Enumerable.Empty<BordersInfo>();
        }

        public IEnumerable<FormattingRecord> GetDirectFormattings()
        {
            return Enumerable.Empty<FormattingRecord>();
        }

        public string RelationshipId
        {
            get;
            set;
        }
    }
}
