﻿using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal struct FontInfo
    {
        public FontInfo(CellStyle cellStyle)
            : this()
        {
            this.Bold = cellStyle.IsBold;
            this.Italic = cellStyle.IsItalic;
            this.FontSize = cellStyle.FontSize;
            this.FontFamily = cellStyle.FontFamily;
            this.ForeColor = cellStyle.ForeColor;
            this.UnderlineType = cellStyle.Underline;
        }

        public bool? Bold { get; set; }

        public bool? Italic { get; set; }

        public double? FontSize { get; set; }

        public ThemableFontFamily FontFamily { get; set; }

        public ThemableColor ForeColor { get; set; }

        public UnderlineType? UnderlineType { get; set; }

        public FontInfo MergeWith(FontInfo other)
        {
            if (!this.Bold.HasValue)
            {
                this.Bold = other.Bold;
            }

            if (!this.Italic.HasValue)
            {
                this.Italic = other.Italic;
            }

            if (!this.FontSize.HasValue)
            {
                this.FontSize = other.FontSize;
            }

            if (this.FontFamily == null)
            {
                this.FontFamily = other.FontFamily;
            }

            if (this.ForeColor == null)
            {
                this.ForeColor = other.ForeColor;
            }

            if (!this.UnderlineType.HasValue)
            {
                this.UnderlineType = other.UnderlineType;
            }

            return this;
        }

        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is FontInfo))
            {
                return false;
            }

            FontInfo other = (FontInfo)obj;

            return TelerikHelper.EqualsOfT(this.Bold, other.Bold) &&
                TelerikHelper.EqualsOfT(this.Italic, other.Italic) &&
                TelerikHelper.EqualsOfT(this.FontSize, other.FontSize) &&
                TelerikHelper.EqualsOfT(this.FontFamily, other.FontFamily) &&
                TelerikHelper.EqualsOfT(this.ForeColor, other.ForeColor) &&
                TelerikHelper.EqualsOfT(this.UnderlineType, other.UnderlineType);
        }

        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                this.Bold.GetHashCodeOrZero(),
                this.Italic.GetHashCodeOrZero(),
                this.FontFamily.GetHashCodeOrZero(),
                this.FontSize.GetHashCodeOrZero(),
                this.ForeColor.GetHashCodeOrZero(),
                this.UnderlineType.GetHashCodeOrZero());
        }

        public override string ToString()
        {
            return string.Format("Bold: {0};Italic: {1}; FontSize: {2}; FontFamily: {3}; ForeColor: {4}; Underline: {5}",
                this.Bold,
                this.Italic,
                this.FontSize,
                this.FontFamily,
                this.ForeColor,
                this.UnderlineType);
        }

        internal static FontInfo GetDefaultValue()
        {
            FontInfo info = new FontInfo();
            info.Bold = false;
            info.FontFamily = SpreadsheetDefaultValues.DefaultFontFamily;
            info.FontSize = SpreadsheetDefaultValues.DefaultFontSize;
            info.ForeColor = SpreadsheetDefaultValues.DefaultForeColor;
            info.Italic = false;
            info.UnderlineType = Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.None;

            return info;
        }
    }
}