﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal struct FormattingRecord
    {
        public static readonly FormattingRecord Empty = new FormattingRecord();

        public FormattingRecord(FormattingRecord record)
            : this()
        {
            this.FillId = record.FillId;
            this.NumberFormatId = record.NumberFormatId;
            this.FontInfoId = record.FontInfoId;
            this.BordersInfoId = record.BordersInfoId;
            this.HorizontalAlignment = record.HorizontalAlignment;
            this.VerticalAlignment = record.VerticalAlignment;
            this.Indent = record.Indent;
            this.WrapText = record.WrapText;
            this.IsLocked = record.IsLocked;

            this.ApplyNumberFormat = record.ApplyNumberFormat;
            this.ApplyAlignment = record.ApplyAlignment;
            this.ApplyFont = record.ApplyFont;
            this.ApplyBorder = record.ApplyBorder;
            this.ApplyFill = record.ApplyFill;
            this.ApplyProtection = record.ApplyProtection;

            this.StyleFormattingRecordId = record.StyleFormattingRecordId;
        }

        public int? FillId { get; set; }
      
        public int? NumberFormatId { get; set; }
      
        public int? FontInfoId { get; set; }
     
        public int? BordersInfoId { get; set; }
     
        public RadHorizontalAlignment? HorizontalAlignment { get; set; }
     
        public RadVerticalAlignment? VerticalAlignment { get; set; }
    
        public int? Indent { get; set; }
    
        public bool? WrapText { get; set; }

        public bool? IsLocked { get; set; }

        public bool? ApplyNumberFormat { get; set; }
     
        public bool? ApplyAlignment { get; set; }
        
        public bool? ApplyFont { get; set; }
        
        public bool? ApplyBorder { get; set; }
        
        public bool? ApplyFill { get; set; }

        public bool? ApplyProtection { get; set; }

        public int? StyleFormattingRecordId { get; set; }
        
        public bool IsEmpty()
        {
            return this.Equals(FormattingRecord.Empty);
        }

        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is FormattingRecord))
            {
                return false;
            }

            FormattingRecord other = (FormattingRecord)obj;

            return TelerikHelper.EqualsOfT(this.FillId, other.FillId) &&
                TelerikHelper.EqualsOfT(this.NumberFormatId, other.NumberFormatId) &&
                TelerikHelper.EqualsOfT(this.FontInfoId, other.FontInfoId) &&
                TelerikHelper.EqualsOfT(this.BordersInfoId, other.BordersInfoId) &&
                TelerikHelper.EqualsOfT(this.HorizontalAlignment, other.HorizontalAlignment) &&
                TelerikHelper.EqualsOfT(this.VerticalAlignment, other.VerticalAlignment) &&
                TelerikHelper.EqualsOfT(this.Indent, other.Indent) &&
                TelerikHelper.EqualsOfT(this.WrapText, other.WrapText) &&
                TelerikHelper.EqualsOfT(this.IsLocked, other.IsLocked) &&
                TelerikHelper.EqualsOfT(this.StyleFormattingRecordId, other.StyleFormattingRecordId) &&
                TelerikHelper.EqualsOfT(this.ApplyNumberFormat, other.ApplyNumberFormat) &&
                TelerikHelper.EqualsOfT(this.ApplyAlignment, other.ApplyAlignment) &&
                TelerikHelper.EqualsOfT(this.ApplyFont, other.ApplyFont) &&
                TelerikHelper.EqualsOfT(this.ApplyBorder, other.ApplyBorder) &&
                TelerikHelper.EqualsOfT(this.ApplyFill, other.ApplyFill) &&
                TelerikHelper.EqualsOfT(this.ApplyProtection, other.ApplyProtection);
        }

        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(
                this.FillId.GetHashCodeOrZero(),
                this.NumberFormatId.GetHashCodeOrZero(),
                this.FontInfoId.GetHashCodeOrZero(),
                this.BordersInfoId.GetHashCodeOrZero(),
                this.HorizontalAlignment.GetHashCodeOrZero(),
                this.VerticalAlignment.GetHashCodeOrZero(),
                this.Indent.GetHashCodeOrZero(),
                this.WrapText.GetHashCodeOrZero(),
                this.IsLocked.GetHashCodeOrZero(),
                this.StyleFormattingRecordId.GetHashCodeOrZero(), 
                this.ApplyNumberFormat.GetHashCodeOrZero(),
                this.ApplyAlignment.GetHashCodeOrZero(),
                this.ApplyFont.GetHashCodeOrZero(),
                this.ApplyBorder.GetHashCodeOrZero(),
                this.ApplyFill.GetHashCodeOrZero(),
                this.ApplyProtection.GetHashCodeOrZero());
        }
    }
}