﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal class RowInfo
    {
        private int? rowIndex;

        public RowInfo()
        {
        }

        public RowInfo(int rowIndex, Rows rows)
        {
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfNull(rows, "rows");

            this.RowIndex = rowIndex;
            
            ValueBox<RowHeight> rowHeightBox = rows.PropertyBag.GetPropertyValueCollection(RowsPropertyBag.HeightProperty).GetValueInternal(rowIndex);
            this.IsDefault = rowHeightBox == null;

            RowHeight rowHeight = this.IsDefault ? RowsPropertyBag.HeightProperty.DefaultValue : rowHeightBox.Value;

            this.Height = rowHeight.Value;
            this.IsCustom = rowHeight.IsCustom;

            this.OutlineLevel = rows.PropertyBag.GetPropertyValueCollection(RowsPropertyBag.OutlineLevelProperty).GetValue(rowIndex);

            this.Hidden = rows.PropertyBag.GetPropertyValueCollection(RowsPropertyBag.HiddenProperty).GetValue(rowIndex);
        }

        public int? RowIndex
        {
            get
            {
                return this.rowIndex;
            }
            set
            {
                if (value != null)
                {
                    Guard.ThrowExceptionIfInvalidRowIndex(value.Value);
                }

                this.rowIndex = value;
            }
        }

        public bool IsCustom { get; set; }
        
        public bool IsDefault { get; set; }
       
        public double? Height { get; set; }

        public bool Hidden { get; set; }

        public int OutlineLevel { get; set; }

        public int StyleIndex { get; set; }

        public override bool Equals(object obj)
        {
            RowInfo other = (RowInfo)obj;

            return this.RowIndex == other.RowIndex &&
                this.IsCustom == other.IsCustom &&
                this.IsDefault == other.IsDefault &&
                this.Height == other.Height &&
                this.Hidden == other.Hidden &&
                this.OutlineLevel == other.OutlineLevel &&
                this.StyleIndex == other.StyleIndex;
        }

        public override int GetHashCode()
        {
            return TelerikHelper.CombineHashCodes(this.RowIndex.GetHashCode(),
                this.IsCustom.GetHashCode(),
                this.IsDefault.GetHashCode(),
                this.Height.GetHashCode(),
                this.Hidden.GetHashCode(),
                this.OutlineLevel.GetHashCode(),
                this.StyleIndex.GetHashCode());
        }
    }
}