﻿using System;
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal class StyleSheet
    {
        private readonly ResourceIndexedTable<FontInfo> fontInfoTable;
        private readonly ResourceIndexedTable<IFill> fillTable;
        private readonly ResourceIndexedTable<CellValueFormat> cellValueFormatTable;
        private readonly ResourceIndexedTable<BordersInfo> bordersInfoTable;
        private readonly ResourceIndexedTable<StyleInfo> styleInfoTable;
        private readonly ResourceIndexedTable<FormattingRecord> directFormattingTable;
        private readonly ResourceIndexedTable<FormattingRecord> styleFormattingTable;
        private readonly ResourceIndexedTable<Color> indexedColorTable;
        
        public StyleSheet(bool isExport)
        {
            this.fontInfoTable = new ResourceIndexedTable<FontInfo>(isExport);
            this.fillTable = new ResourceIndexedTable<IFill>(isExport);
            this.cellValueFormatTable = new ResourceIndexedTable<CellValueFormat>(isExport, NumberFormatTypes.FirstCustomFormatId);
            this.bordersInfoTable = new ResourceIndexedTable<BordersInfo>(isExport);
            this.styleInfoTable = new ResourceIndexedTable<StyleInfo>(isExport);
            this.indexedColorTable = new ResourceIndexedTable<Color>(isExport);

            this.directFormattingTable = new ResourceIndexedTable<FormattingRecord>(isExport);
            this.styleFormattingTable = new ResourceIndexedTable<FormattingRecord>(isExport);
        }

        public ResourceIndexedTable<FontInfo> FontInfoTable
        {
            get
            {
                return this.fontInfoTable;
            }
        }

        public ResourceIndexedTable<IFill> FillTable
        {
            get
            {
                return this.fillTable;
            }
        }

        public ResourceIndexedTable<CellValueFormat> CellValueFormatTable
        {
            get
            {
                return this.cellValueFormatTable;
            }
        }

        public ResourceIndexedTable<BordersInfo> BordersInfoTable
        {
            get
            {
                return this.bordersInfoTable;
            }
        }

        public ResourceIndexedTable<StyleInfo> StyleInfoTable
        {
            get
            {
                return this.styleInfoTable;
            }
        }

        public ResourceIndexedTable<FormattingRecord> DirectFormattingTable
        {
            get
            {
                return this.directFormattingTable;
            }
        }

        public ResourceIndexedTable<FormattingRecord> StyleFormattingTable
        {
            get
            {
                return this.styleFormattingTable;
            }
        }

        public ResourceIndexedTable<Color> IndexedColorTable
        {
            get
            {
                return this.indexedColorTable;
            }
        }

        public StyleInfo? GetStyleInfoByStyleFormattingRecordId(int formattingRecordId)
        {
            foreach (var styleInfo in this.StyleInfoTable)
            {
                if (styleInfo.FormattingRecordId == formattingRecordId)
                {
                    return styleInfo;
                }
            }

            return null;
        }
    }
}