﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.Core.Data;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Model.Drawing.Charts;
#endif
using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Spreadsheet.Model.Charts;
#endif
using Telerik.Windows.Documents.Spreadsheet.Model.Protection;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal class XlsxWorkbookImportContext : XlsxWorkbookContextBase<XlsxWorksheetImportContext>, IXlsxWorkbookImportContext
    {
        private readonly Dictionary<string, IResource> resourcesMapping;
        private readonly DifferentialFormatsImportContext formats;
#if !SILVERLIGHT
        private ValueMapper<AxisGroupName, System.Tuple<int, int>> axesGroupNamesToAxesIndices;
        private ValueMapper<ISupportAxes, System.Tuple<int, int>> seriesGroupsToAxesIndices;
#endif

        public XlsxWorkbookImportContext()
            : base(false, new Workbook())
        {
            this.resourcesMapping = new Dictionary<string, IResource>();
            this.formats = new DifferentialFormatsImportContext();
#if !SILVERLIGHT
            this.axesGroupNamesToAxesIndices = new ValueMapper<AxisGroupName, System.Tuple<int, int>>();
            this.seriesGroupsToAxesIndices = new ValueMapper<ISupportAxes, System.Tuple<int, int>>();
#endif
        }

        public DifferentialFormatsImportContext DifferentialFormatsContext
        {
            get
            {
                return this.formats;
            }
        }

        public bool IsImportSuspended
        {
            get
            {
                return false;
            }
        }

        public bool SupportCharts
        {
            get
            {
                return true;
            }
        } 

        public void BeginImport()
        {
            this.Workbook.History.IsEnabled = false;
            this.Workbook.SuspendLayoutUpdate();
            this.Workbook.SuspendPropertyChanged();

            this.InitDefaultFormatting();
        }

        public void EndImport()
        {
            foreach (var worksheetContext in this.GetWorksheetContexts())
            {
                worksheetContext.EndImport(this);
            }

            this.ImportDefinedNames();

            this.Workbook.ResumePropertyChanged();
            this.Workbook.ResumeLayoutUpdate();
            this.Workbook.History.IsEnabled = true;
        }

        public IXlsxWorksheetImportContext AddWorksheet(string worksheetName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(worksheetName, "worksheetName");

            Worksheet worksheet = this.Workbook.Worksheets.Add();
            worksheet.Name = worksheetName;

            XlsxWorksheetImportContext context = new XlsxWorksheetImportContext(this, worksheet);
            this.AddWorksheetContext(context);

            return context;
        }

        public IXlsxWorksheetImportContext GetWorksheetContextFromWorksheetPart(WorksheetPart worksheetPart)
        {
            Guard.ThrowExceptionIfNull(worksheetPart, "worksheetPart");

            return this.GetWorksheetContext(this.GetWorksheetFromWorksheetPart(worksheetPart));
        }

        public IXlsxWorksheetImportContext GetWorksheetContextFromDrawingPart(DrawingPart drawingPart)
        {
            Guard.ThrowExceptionIfNull(drawingPart, "drawingPart");

            return this.GetWorksheetContext(this.GetWorksheetFromDrawingPart(drawingPart));
        }

        IXlsxWorksheetImportContext IXlsxWorkbookImportContext.GetWorksheetContext(string worksheetName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(worksheetName, "worksheetName");

            return this.GetWorksheetContext(worksheetName);
        }

        public void SetActiveTabIndex(int activeTabIndex)
        {
            this.Workbook.ActiveTabIndex = activeTabIndex;
        }

        public void ApplyWorkbookProtectionInfo(WorkbookProtectionInfo workbookProtectionInfo)
        {
            Guard.ThrowExceptionIfNull(workbookProtectionInfo, "workbookProtectionInfo");

            this.Workbook.ProtectionData.Enforced = true;

            if (string.IsNullOrEmpty(workbookProtectionInfo.Password))
            {
                this.Workbook.ProtectionData.AlgorithmName = workbookProtectionInfo.AlgorithmName;
                this.Workbook.ProtectionData.Salt = workbookProtectionInfo.SaltValue;
                this.Workbook.ProtectionData.Hash = workbookProtectionInfo.HashValue;
                this.Workbook.ProtectionData.SpinCount = workbookProtectionInfo.SpinCount;
            }
            else
            {
                this.Workbook.ProtectionData.Password = workbookProtectionInfo.Password;
            }

            if (workbookProtectionInfo.Enforced)
            {
                this.Workbook.ProtectionOptions = new WorkbookProtectionOptions(workbookProtectionInfo.LockStructure,
                    workbookProtectionInfo.LockWindows);
            }
            else
            {
                this.Workbook.ProtectionOptions = WorkbookProtectionOptions.Default;
            }
        }

        public void RegisterResource(string resourceKey, IResource resource)
        {
            this.resourcesMapping[resourceKey] = resource;
        }

        public IResource GetResourceByResourceKey(string resourceKey)
        {
            return this.resourcesMapping[resourceKey];
        }

        public void ImportStyles()
        {
            foreach (StyleInfo styleInfo in this.StyleSheet.StyleInfoTable)
            {
                CellStyle cellStyle;

                if (styleInfo.BuiltInId.HasValue)
                {
                    CellStyle builtinStyle = this.Workbook.Styles.GetByName(styleInfo.Name);
                    if (builtinStyle != null)
                    {
                        cellStyle = this.CreateStyleFromStyleInfo(styleInfo, builtinStyle);
                        using (new UpdateScope(builtinStyle.SuspendForcingLayoutUpdate, builtinStyle.ResumeForcingLayoutUpdate))
                        {
                            builtinStyle.CopyPropertiesFrom(cellStyle);
                        }

                        continue;
                    }
                }

                cellStyle = this.CreateStyleFromStyleInfo(styleInfo);
                var style = this.Workbook.Styles.GetByName(cellStyle.Name);
                if (style == null || this.Workbook.Styles.Remove(cellStyle.Name))
                {
                    CellStyle newCustomStyle = this.Workbook.Styles.Add(cellStyle.Name);
                    using (new UpdateScope(newCustomStyle.SuspendForcingLayoutUpdate, newCustomStyle.ResumeForcingLayoutUpdate))
                    {
                        newCustomStyle.CopyPropertiesFrom(cellStyle);
                    }
                }
                else
                {
                    style.CopyPropertiesFrom(cellStyle);
                }
            }
        }

#if !SILVERLIGHT
        public FormulaChartData GetFormulaChartData(string formula)
        {
            return new WorkbookFormulaChartData(this.Workbook, formula);
        }

        public void RegisterSeriesGroupAwaitingAxisGroupName(ISupportAxes seriesGroup, int catAxisId, int valAxisId)
        {
            this.seriesGroupsToAxesIndices.AddPair(seriesGroup, new System.Tuple<int, int>(catAxisId, valAxisId));
        }

        public void RegisterAxisGroup(AxisGroupName groupName, int thisId, int otherId)
        {
            if (!this.axesGroupNamesToAxesIndices.ContainsFromValue(groupName))
            {
                this.axesGroupNamesToAxesIndices.AddPair(groupName, new System.Tuple<int, int>(thisId, otherId));
            }
        }

        public void PairSeriesGroupsWithAxes()
        {
            foreach (ISupportAxes seriesGroup in this.seriesGroupsToAxesIndices.FromValues)
            {
                System.Tuple<int, int> expectedIndices = this.seriesGroupsToAxesIndices.GetToValue(seriesGroup);

                System.Tuple<int, int> correspondingRegisteredPair = this.axesGroupNamesToAxesIndices.ToValues.FirstOrDefault((registeredPair) =>
                {
                    if ((expectedIndices.Item1 == registeredPair.Item1 && expectedIndices.Item2 == registeredPair.Item2) ||
                    (expectedIndices.Item1 == registeredPair.Item2 && expectedIndices.Item2 == registeredPair.Item1))
                    {
                        return true;
                    }

                    return false;
                });

                if (correspondingRegisteredPair == null)
                {
                    throw new InvalidOperationException("The axes are incorrectly paired.");
                }

                AxisGroupName groupName = this.axesGroupNamesToAxesIndices.GetFromValue(correspondingRegisteredPair);
                seriesGroup.AxisGroupName = groupName;
            }

            this.seriesGroupsToAxesIndices = new ValueMapper<ISupportAxes, System.Tuple<int, int>>();
            this.axesGroupNamesToAxesIndices = new ValueMapper<AxisGroupName, System.Tuple<int, int>>();
        }
#endif

        private void InitDefaultFormatting()
        {
            this.InitDefaultNumberFormatting();
            this.InitDefaultIndexedColors();
        }

        private void InitDefaultIndexedColors()
        {
            for (int i = 0; i < IndexedColors.PredefinedIndexedColors.Length; i++)
            {
                this.StyleSheet.IndexedColorTable.Add(IndexedColors.PredefinedIndexedColors[i].Color, i);
            }
        }

        private void ImportDefinedNames()
        {
            foreach (DefinedNameInfo definedNameInfo in this.DefinedNames)
            {
                int rowIndex = 0;
                int columnIndex = 0;
                RadExpression expression;
                InputStringCollection inputStringCollection;

                Worksheet worksheet = definedNameInfo.IsGlobal ? this.Workbook.Worksheets[0] : this.Workbook.Worksheets[definedNameInfo.LocalSheetId];
                ParseResult result = RadExpression.TryParse(definedNameInfo.Value, worksheet, rowIndex, columnIndex, XlsxHelper.CultureInfo, out expression, out inputStringCollection);

                if (result == ParseResult.Successful)
                {
                    string localizedValue = SpreadsheetCultureHelper.PrepareFormulaValue(expression.ToString(FormatHelper.DefaultSpreadsheetCulture));

                    if (definedNameInfo.IsGlobal)
                    {
                        if (!DefinedName.AreNamesEqual(definedNameInfo.Name, DefinedName.PrintAreaDefinedName) && !DefinedName.AreNamesEqual(definedNameInfo.Name, DefinedName.FilterDefinedName))
                        {
                            this.Workbook.Names.Add(definedNameInfo.Name, localizedValue, rowIndex, columnIndex, definedNameInfo.Comment, !definedNameInfo.Hidden);
                        }
                    }
                    else
                    {
                        if (definedNameInfo.Name == DefinedName.PrintAreaDefinedName)
                        {
                            this.ImportPrintArea(expression, worksheet);
                        }
                        else if (!string.Equals(DefinedName.FilterDefinedName, definedNameInfo.Name, StringComparison.OrdinalIgnoreCase))
                        {
                            worksheet.Names.Add(definedNameInfo.Name, localizedValue, rowIndex, columnIndex, definedNameInfo.Comment, !definedNameInfo.Hidden);
                        }
                    }
                }
            }
        }

        // TODO: Remove the print area logic. Print areas should be synchronized with defined names in the model, not here.
        private void ImportPrintArea(RadExpression expression, Worksheet worksheet)
        {
            CellReferenceRangeExpression cellReferenceRangeExpression = expression.GetValueAsConstantOrCellReference() as CellReferenceRangeExpression;

            if (cellReferenceRangeExpression == null || !cellReferenceRangeExpression.IsValid)
            {
                return;
            }

            List<CellRange> ranges = new List<CellRange>();
            foreach (CellReferenceRange cellReferenceRange in cellReferenceRangeExpression.CellReferenceRanges)
            {
                ranges.Add(new CellRange(cellReferenceRange.FromCellReference.ActualRowIndex,
                    cellReferenceRange.FromCellReference.ActualColumnIndex,
                    cellReferenceRange.ToCellReference.ActualRowIndex,
                    cellReferenceRange.ToCellReference.ActualColumnIndex));
            }

            worksheet.WorksheetPageSetup.PrintArea.SetPrintArea(ranges);
        }

        private CellStyle CreateStyleFromStyleInfo(StyleInfo styleInfo, CellStyle builtInStyle = null)
        {
            CellStyle cellStyle = new CellStyle(this.Workbook, styleInfo.Name, CellStyleCategory.Custom);

            using (new UpdateScope(cellStyle.SuspendForcingLayoutUpdate, cellStyle.ResumeForcingLayoutUpdate))
            {
                if (builtInStyle != null)
                {
                    cellStyle.CopyPropertiesFrom(builtInStyle);
                }

                FormattingRecord formattingRecord = this.StyleSheet.StyleFormattingTable[styleInfo.FormattingRecordId];

                if (formattingRecord.FillId.HasValue)
                {
                    IFill fill = this.StyleSheet.FillTable[formattingRecord.FillId.Value];
                    if (!(fill is NoneFill))
                    {
                        cellStyle.Fill = fill;
                    }
                }

                if (formattingRecord.NumberFormatId.HasValue)
                {
                    cellStyle.Format = this.StyleSheet.CellValueFormatTable[formattingRecord.NumberFormatId.Value];
                }

                if (formattingRecord.FontInfoId.HasValue)
                {
                    FontInfo fontInfo = this.StyleSheet.FontInfoTable[formattingRecord.FontInfoId.Value];
                    ApplyFontInfoToStyle(fontInfo, cellStyle);
                }

                if (formattingRecord.BordersInfoId.HasValue)
                {
                    BordersInfo borderInfo = this.StyleSheet.BordersInfoTable[formattingRecord.BordersInfoId.Value];
                    ApplyBorderInfoToStyle(borderInfo, cellStyle);
                }

                if (formattingRecord.HorizontalAlignment.HasValue)
                {
                    cellStyle.HorizontalAlignment = formattingRecord.HorizontalAlignment.Value;
                }

                if (formattingRecord.VerticalAlignment.HasValue)
                {
                    cellStyle.VerticalAlignment = formattingRecord.VerticalAlignment.Value;
                }

                if (formattingRecord.Indent.HasValue)
                {
                    cellStyle.Indent = formattingRecord.Indent.Value;
                }

                if (formattingRecord.WrapText.HasValue)
                {
                    cellStyle.IsWrapped = formattingRecord.WrapText.Value;
                }

                if (formattingRecord.IsLocked.HasValue)
                {
                    cellStyle.IsLocked = formattingRecord.IsLocked.Value;
                }

                if (formattingRecord.ApplyNumberFormat.HasValue)
                {
                    cellStyle.IncludeNumber = formattingRecord.ApplyNumberFormat.Value;
                }

                if (formattingRecord.ApplyAlignment.HasValue)
                {
                    cellStyle.IncludeAlignment = formattingRecord.ApplyAlignment.Value;
                }

                if (formattingRecord.ApplyFont.HasValue)
                {
                    cellStyle.IncludeFont = formattingRecord.ApplyFont.Value;
                }

                if (formattingRecord.ApplyBorder.HasValue)
                {
                    cellStyle.IncludeBorder = formattingRecord.ApplyBorder.Value;
                }

                if (formattingRecord.ApplyFill.HasValue)
                {
                    cellStyle.IncludeFill = formattingRecord.ApplyFill.Value;
                }

                if (formattingRecord.ApplyProtection.HasValue)
                {
                    cellStyle.IncludeProtection = formattingRecord.ApplyProtection.Value;
                }
            }

            return cellStyle;
        }

        private void ApplyFontInfoToStyle(FontInfo fontInfo, CellStyle cellStyle)
        {
            if (fontInfo.Bold.HasValue)
            {
                cellStyle.IsBold = fontInfo.Bold.Value;
            }

            if (fontInfo.Italic.HasValue)
            {
                cellStyle.IsItalic = fontInfo.Italic.Value;
            }

            if (fontInfo.FontSize.HasValue)
            {
                cellStyle.FontSize = fontInfo.FontSize.Value;
            }

            if (fontInfo.FontFamily != null)
            {
                cellStyle.FontFamily = fontInfo.FontFamily;
            }

            if (fontInfo.ForeColor != null)
            {
                cellStyle.ForeColor = fontInfo.ForeColor;
            }

            if (fontInfo.UnderlineType.HasValue)
            {
                cellStyle.Underline = fontInfo.UnderlineType.Value;
            }
        }

        private void ApplyBorderInfoToStyle(BordersInfo borderInfo, CellStyle cellStyle)
        {
            if (borderInfo.Left != null)
            {
                cellStyle.LeftBorder = borderInfo.Left;
            }

            if (borderInfo.Top != null)
            {
                cellStyle.TopBorder = borderInfo.Top;
            }

            if (borderInfo.Right != null)
            {
                cellStyle.RightBorder = borderInfo.Right;
            }

            if (borderInfo.Bottom != null)
            {
                cellStyle.BottomBorder = borderInfo.Bottom;
            }

            if (borderInfo.DiagonalUp != null)
            {
                cellStyle.DiagonalUpBorder = borderInfo.DiagonalUp;
            }

            if (borderInfo.DiagonalDown != null)
            {
                cellStyle.DiagonalDownBorder = borderInfo.DiagonalDown;
            }
        }
    }
}