﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
#if !SILVERLIGHT
#endif
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Core;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal class XlsxWorksheetExportContext : XlsxWorksheetContextBase, IXlsxWorksheetExportContext
    {
        private readonly CellPropertyDataInfo directFormattingPropertyDataInfo;
        private readonly ICompressedList<ColumnInfo> columnInfoCompressedList;
        private readonly List<CellPropertyDataInfo> rowPropertyDataInfosToRespect;
        private readonly XlsxWorkbookExportContext workbookContext;
        private readonly int sheetNo;
        private readonly Dictionary<Image, string> pictureToRelationshipId;
        private readonly Dictionary<IDataValidationRule, IEnumerable<CellRange>> dataValidationRuleToCellRanges;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<FormattingRecord>> worksheetEntryToDirectFormattingCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<FontInfo>> worksheetEntityToFontInfoCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<BordersInfo>> worksheetEntityToBordersInfoCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<string>> worksheetEntityToStyleNameCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<IFill>> worksheetEntityToFillCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<CellValueFormat>> worksheetEntityToCellValueFormatCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<RadHorizontalAlignment>> worksheetEntityToHorizontalAlignmentCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<RadVerticalAlignment>> worksheetEntityToVerticalAlignmentCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<int>> worksheetEntityToIndentCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<bool>> worksheetEntityToIsWrappedCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<bool>> worksheetEntityToIsLockedCompressedList;
        private readonly Dictionary<WorksheetEntityBase, ICompressedList<bool>> worksheetEntityToNonDefaultFormattingRecordIndex;

        private Dictionary<IPropertyDefinition, Func<ICompressedList, bool>> propertyToIsDefaultFunc;

        private bool areDirectFormattingRecordsInitialized;

        private ICompressedList<IDataValidationRule> dataValidationRulesCompressedList;

        public XlsxWorksheetExportContext(XlsxWorkbookExportContext workbookContext, Worksheet worksheet, int sheetNo)
            : base(worksheet)
        {
            Guard.ThrowExceptionIfNull(workbookContext, "workbookContext");

            this.sheetNo = sheetNo;
            this.workbookContext = workbookContext;

            this.worksheetEntityToFontInfoCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<FontInfo>>();
            this.worksheetEntityToFontInfoCompressedList[this.Worksheet.Cells] = new CompressedList<FontInfo>(0, SpreadsheetDefaultValues.CellCount - 1, FontInfo.GetDefaultValue());
            this.worksheetEntityToFontInfoCompressedList[this.Worksheet.Rows] = new CompressedList<FontInfo>(0, SpreadsheetDefaultValues.RowCount - 1, FontInfo.GetDefaultValue());
            this.worksheetEntityToFontInfoCompressedList[this.Worksheet.Columns] = new CompressedList<FontInfo>(0, SpreadsheetDefaultValues.ColumnCount - 1, FontInfo.GetDefaultValue());

            this.worksheetEntityToBordersInfoCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<BordersInfo>>();
            this.worksheetEntityToBordersInfoCompressedList[this.Worksheet.Cells] = new CompressedList<BordersInfo>(0, SpreadsheetDefaultValues.CellCount - 1, BordersInfo.GetDefaultValue());
            this.worksheetEntityToBordersInfoCompressedList[this.Worksheet.Rows] = new CompressedList<BordersInfo>(0, SpreadsheetDefaultValues.RowCount - 1, BordersInfo.GetDefaultValue());
            this.worksheetEntityToBordersInfoCompressedList[this.Worksheet.Columns] = new CompressedList<BordersInfo>(0, SpreadsheetDefaultValues.ColumnCount - 1, BordersInfo.GetDefaultValue());

            this.worksheetEntityToStyleNameCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<string>>();
            this.worksheetEntityToFillCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<IFill>>();
            this.worksheetEntityToCellValueFormatCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<CellValueFormat>>();
            this.worksheetEntityToHorizontalAlignmentCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<RadHorizontalAlignment>>();
            this.worksheetEntityToVerticalAlignmentCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<RadVerticalAlignment>>();
            this.worksheetEntityToIndentCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<int>>();
            this.worksheetEntityToIsWrappedCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<bool>>();
            this.worksheetEntityToIsLockedCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<bool>>();

            this.worksheetEntityToNonDefaultFormattingRecordIndex = new Dictionary<WorksheetEntityBase, ICompressedList<bool>>();
            this.worksheetEntityToNonDefaultFormattingRecordIndex[this.Worksheet.Cells] = new CompressedList<bool>(0, SpreadsheetDefaultValues.CellCount - 1, false);
            this.worksheetEntityToNonDefaultFormattingRecordIndex[this.Worksheet.Rows] = new CompressedList<bool>(0, SpreadsheetDefaultValues.RowCount - 1, false);
            this.worksheetEntityToNonDefaultFormattingRecordIndex[this.Worksheet.Columns] = new CompressedList<bool>(0, SpreadsheetDefaultValues.ColumnCount - 1, false);

            this.worksheetEntryToDirectFormattingCompressedList = new Dictionary<WorksheetEntityBase, ICompressedList<FormattingRecord>>();
            this.worksheetEntryToDirectFormattingCompressedList[this.Worksheet.Cells] = new CompressedList<FormattingRecord>(0, SpreadsheetDefaultValues.CellCount - 1, FormattingRecord.Empty);
            this.worksheetEntryToDirectFormattingCompressedList[this.Worksheet.Rows] = new CompressedList<FormattingRecord>(0, SpreadsheetDefaultValues.RowCount - 1, FormattingRecord.Empty);
            this.worksheetEntryToDirectFormattingCompressedList[this.Worksheet.Columns] = new CompressedList<FormattingRecord>(0, SpreadsheetDefaultValues.ColumnCount - 1, FormattingRecord.Empty);

            this.directFormattingPropertyDataInfo = new CellPropertyDataInfo();

            ColumnInfo defaultWidthColumnInfo = new ColumnInfo(XlsxHelper.ConvertColumnPixelWidthToExcelWidth(this.Worksheet.Workbook, SpreadsheetDefaultValues.DefaultColumnWidth));
            this.columnInfoCompressedList = new CompressedList<ColumnInfo>(0, SpreadsheetDefaultValues.ColumnCount - 1, defaultWidthColumnInfo);

            this.rowPropertyDataInfosToRespect = new List<CellPropertyDataInfo>();

            this.rowPropertyDataInfosToRespect.Add(WorksheetExportContext.GetValuePropertyDataInfo(this.Worksheet));
            this.rowPropertyDataInfosToRespect.Add(this.directFormattingPropertyDataInfo);

            this.pictureToRelationshipId = new Dictionary<Image, string>();

            this.dataValidationRuleToCellRanges = new Dictionary<IDataValidationRule, IEnumerable<CellRange>>();

            this.SetRowAndColumnPropertiesAsLocalInCellsWhereCellsHasValueProperty();

            this.InitializePropertyToIsDefault();

            this.Initialize();
        }

        public Common.Model.Data.ResourceManager Resources
        {
            get { throw new NotImplementedException(); }
        }

        public IXlsxWorkbookExportContext WorkbookContext
        {
            get
            {
                return this.workbookContext;
            }
        }

        public int SheetNo
        {
            get
            {
                return this.sheetNo;
            }
        }

        public string WorksheetName
        {
            get
            {
                return this.Worksheet.Name;
            }
        }

        public IEnumerable<SpreadsheetHyperlink> Hyperlinks
        {
            get
            {
                return this.Worksheet.Hyperlinks;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<string>> StyleNameCompressedList
        {
            get
            {
                return this.worksheetEntityToStyleNameCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<IFill>> FillCompressedList
        {
            get
            {
                return this.worksheetEntityToFillCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<CellValueFormat>> CellValueFormatCompressedList
        {
            get
            {
                return this.worksheetEntityToCellValueFormatCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<FontInfo>> FontInfoCompressedList
        {
            get
            {
                return this.worksheetEntityToFontInfoCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<BordersInfo>> BordersInfoCompressedList
        {
            get
            {
                return this.worksheetEntityToBordersInfoCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<FormattingRecord>> DirectFormattingCompressedList
        {
            get
            {
                return this.worksheetEntryToDirectFormattingCompressedList;
            }
        }

        public Dictionary<WorksheetEntityBase, ICompressedList<bool>> NonDefaultFormattingCompressedList
        {
            get
            {
                return this.worksheetEntityToNonDefaultFormattingRecordIndex;
            }
        }

        public DocumentTheme Theme
        {
            get
            {
                return this.WorkbookContext.Theme;
            }
        }

        public IEnumerable<FloatingShapeBase> Shapes
        {
            get
            {
                return this.Worksheet.Shapes;
            }
        }

#if !SILVERLIGHT
        public IEnumerable<FloatingChartShape> Charts
        {
            get
            {
                return this.Worksheet.Shapes.Charts;
            }
        }
#endif

        public WorksheetViewState WorksheetViewState
        {
            get
            {
                return (WorksheetViewState)((ISheet)this.Worksheet).ViewState;
            }
        }

        public void InitDirectFormattingRecords()
        {
            if (this.areDirectFormattingRecordsInitialized)
            {
                return;
            }

            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.InitDirectFormattingRecords(entity);
            }

            WorksheetExportContext.CalculateCellPropertyPropertyUsedValueRanges(this.directFormattingPropertyDataInfo, this.NonDefaultFormattingCompressedList[this.Worksheet.Cells], true);

            this.areDirectFormattingRecordsInitialized = true;
        }

        public void InitDirectFormattingRecords(WorksheetEntityBase tableEntity)
        {
            this.UpdateProperty(tableEntity,
                this.worksheetEntityToFillCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.FillId = this.workbookContext.StyleSheet.FillTable.GetIndex(value);
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToCellValueFormatCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.NumberFormatId = this.workbookContext.StyleSheet.CellValueFormatTable.GetIndex(value);
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToFontInfoCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.FontInfoId = this.workbookContext.StyleSheet.FontInfoTable.GetIndex(value);
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToBordersInfoCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.BordersInfoId = this.workbookContext.StyleSheet.BordersInfoTable.GetIndex(value);
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToVerticalAlignmentCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.VerticalAlignment = value;
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToHorizontalAlignmentCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.HorizontalAlignment = value;
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToIndentCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.Indent = value;
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToIsWrappedCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.WrapText = value;
                    return formattingRecord;
                });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToIsLockedCompressedList[tableEntity],
               this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
               (formattingRecord, value) =>
               {
                   formattingRecord.IsLocked = value;
                   return formattingRecord;
               });

            this.UpdateProperty(tableEntity,
                this.worksheetEntityToStyleNameCompressedList[tableEntity],
                this.worksheetEntryToDirectFormattingCompressedList[tableEntity],
                (formattingRecord, value) =>
                {
                    formattingRecord.StyleFormattingRecordId = this.workbookContext.GetStyleFormattingRecordId(value);
                    return formattingRecord;
                });
        }

        public int GetSharedStringIndex(TextCellValue textValue)
        {
            return this.workbookContext.SharedStrings.GetIndex(new TextSharedString(textValue.Value));
        }

        public string GetPictureRelationshipId(Image picture)
        {
            return this.pictureToRelationshipId[picture];
        }

        public bool ContainsNonDefaultFormattingRecordIndex(WorksheetEntityBase worksheetEntity, long index)
        {
            bool result = this.worksheetEntityToNonDefaultFormattingRecordIndex[worksheetEntity].GetValue(index);

            return result;
        }

        public int? GetFormattingRecordIndex(WorksheetEntityBase tableEntity, long index)
        {
            int? formattingRecordIndex = null;

            FormattingRecord record = this.worksheetEntryToDirectFormattingCompressedList[tableEntity].GetValue(index);

            if (!record.IsEmpty())
            {
                formattingRecordIndex = this.workbookContext.StyleSheet.DirectFormattingTable.GetIndex(record);
            }

            return formattingRecordIndex;
        }

        public string GetRelationshipIdByResource(Common.Model.Data.IResource resource)
        {
            throw new NotImplementedException();
        }

        public void RegisterResource(string relationshipId, Common.Model.Data.IResource resource)
        {
            throw new NotImplementedException();
        }

#if !SILVERLIGHT
        public ChartPart GetChartPartForChart(ChartShape chart)
        {
            return this.workbookContext.GetChartPartForChart(chart);
        }

        public ChartShape GetChartForChartPart(ChartPart chartPart)
        {
            return this.workbookContext.GetChartForChartPart(chartPart);
        }
#endif

        public IEnumerable<RowInfo> GetNonEmptyRows()
        {
            Rows rows = this.Worksheet.Rows;

            CompressedList<bool> usedRows = new CompressedList<bool>(0, SpreadsheetDefaultValues.RowCount - 1, false);

            foreach (IProperty property in rows.Properties)
            {
                ICompressedList propertyValueCollection = rows.PropertyBag.GetPropertyValueCollection(property.PropertyDefinition);

                IEnumerable<LongRange> nonDefaultRanges = propertyValueCollection.GetRanges(getDefaultRanges: false);

                foreach (LongRange range in nonDefaultRanges)
                {
                    usedRows.SetValue(range.Start, range.End, true);
                }
            }

            CellRange usedCellRange = this.GetUsedCellRange();

            int startRowIndex = usedCellRange.FromIndex.RowIndex;
            int endRowIndex = usedCellRange.ToIndex.RowIndex;

            foreach (IProperty property in rows.Properties)
            {
                ICompressedList propertyValueCollection = rows.PropertyBag.GetPropertyValueCollection(property.PropertyDefinition);
                long? lastNonDefaultRowRangeEnd = propertyValueCollection.GetLastNonEmptyRangeEnd(0, SpreadsheetDefaultValues.RowCount - 1);
                endRowIndex = (int)Math.Max(endRowIndex, lastNonDefaultRowRangeEnd ?? 0);
            }

            IEnumerable<LongRange> defaultRanges = usedRows.GetRanges(getDefaultRanges: true);

            foreach (LongRange range in defaultRanges)
            {
                int start = (int)range.Start;
                int end = (int)range.End;
                Tuple<int, int> intersection = Maths.Utils.GetIntersection(start, end, startRowIndex, endRowIndex);

                if (intersection == null)
                {
                    continue;
                }

                for (int rowIndex = intersection.Item1; rowIndex <= intersection.Item2; rowIndex++)
                {
                    foreach (CellPropertyDataInfo propertyDataInfo in this.rowPropertyDataInfosToRespect)
                    {
                        if (!propertyDataInfo.GetRowIsEmpty(rowIndex))
                        {
                            usedRows.SetValue(rowIndex, true);
                            break;
                        }
                    }
                }
            }

            foreach (Range<long, bool> range in usedRows.GetNonDefaultRanges())
            {
                int start = (int)range.Start;
                int end = (int)range.End;
                for (int rowIndex = start; rowIndex <= end; rowIndex++)
                {
                    yield return new RowInfo(rowIndex, this.Worksheet.Rows);
                }
            }
        }

        public IEnumerable<Range<long, ColumnInfo>> GetNonEmptyColumns()
        {
            foreach (Range<long, ColumnInfo> columnRange in this.columnInfoCompressedList.GetNonDefaultRanges())
            {
                yield return columnRange;
            }
        }

        public IEnumerable<MergedCellInfo> GetMergedCells()
        {
            foreach (CellRange cellRange in this.Worksheet.Cells.MergedCellRanges.Ranges)
            {
                yield return new MergedCellInfo(cellRange);
            }
        }

        public IEnumerable<CellInfo> GetNonEmptyCellsInRow(int rowIndex)
        {
            ICompressedList<bool> usedRangesInRow = new CompressedList<bool>(0, SpreadsheetDefaultValues.ColumnCount - 1);
            foreach (CellPropertyDataInfo propertyDataInfo in this.rowPropertyDataInfosToRespect)
            {
                Core.DataStructures.Range range = propertyDataInfo.GetRowUsedRange(rowIndex);
                if (range != null)
                {
                    usedRangesInRow.SetValue(range.Start, range.End, true);
                }
            }

            ICompressedList<bool> nonDefaultFormattingCompressedList = this.NonDefaultFormattingCompressedList[this.Worksheet.Cells];
            ICompressedList<ICellValue> valuesCompressedList = this.Worksheet.Cells.PropertyBag.GetPropertyValueCollection(CellPropertyDefinitions.ValueProperty);

            foreach (Range<long, bool> range in usedRangesInRow.GetNonDefaultRanges())
            {
                for (int columnIndex = (int)range.Start; columnIndex <= range.End; columnIndex++)
                {
                    long index = CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex);
                    bool containsFormatting = nonDefaultFormattingCompressedList.ContainsNonDefaultValues(index, index);
                    bool containsValues = valuesCompressedList.ContainsNonDefaultValues(index, index);

                    if (containsFormatting || containsValues)
                    {
                        yield return new CellInfo(rowIndex, columnIndex, this.Worksheet.Cells);
                    }
                }
            }
        }

        public WorksheetProtectionInfo GetSheetProtectionInfo()
        {
            WorksheetProtectionInfo info = new WorksheetProtectionInfo();

            info.Enforced = this.Worksheet.ProtectionData.Enforced;

            info.AlgorithmName = this.Worksheet.ProtectionData.AlgorithmName;
            info.HashValue = this.Worksheet.ProtectionData.Hash;
            info.SaltValue = this.Worksheet.ProtectionData.Salt;
            info.SpinCount = this.Worksheet.ProtectionData.SpinCount;
            info.Password = this.Worksheet.ProtectionData.Password;

            info.InsertColumns = !this.Worksheet.ProtectionOptions.AllowInsertColumns;
            info.InsertRows = !this.Worksheet.ProtectionOptions.AllowInsertRows;
            info.DeleteColumns = !this.Worksheet.ProtectionOptions.AllowDeleteColumns;
            info.DeleteRows = !this.Worksheet.ProtectionOptions.AllowDeleteRows;
            info.FormatCells = !this.Worksheet.ProtectionOptions.AllowFormatCells;
            info.FormatColumns = !this.Worksheet.ProtectionOptions.AllowFormatColumns;
            info.FormatRows = !this.Worksheet.ProtectionOptions.AllowFormatRows;
            info.AutoFilter = !this.Worksheet.ProtectionOptions.AllowFiltering;
            info.Sort = !this.Worksheet.ProtectionOptions.AllowSorting;

            return info;
        }

        public PrintOptionsInfo GetPrintOptionsInfo()
        {
            PrintOptionsInfo printOptionsInfo = new PrintOptionsInfo();
            WorksheetPageSetup pageSetup = this.Worksheet.WorksheetPageSetup;

            printOptionsInfo.HorizontalCentered = pageSetup.CenterHorizontally;
            printOptionsInfo.VerticalCentered = pageSetup.CenterVertically;
            printOptionsInfo.Headings = pageSetup.PrintOptions.PrintRowColumnHeadings;
            printOptionsInfo.GridLines = pageSetup.PrintOptions.PrintGridlines;

            return printOptionsInfo;
        }

        public PageMarginsInfo GetPageMarginsInfo()
        {
            PageMarginsInfo pageMarginsInfo = new PageMarginsInfo();

            pageMarginsInfo.Bottom = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Bottom);
            pageMarginsInfo.Footer = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Footer);
            pageMarginsInfo.Header = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Header);
            pageMarginsInfo.Left = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Left);
            pageMarginsInfo.Right = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Right);
            pageMarginsInfo.Top = UnitHelper.DipToInch(this.Worksheet.WorksheetPageSetup.Margins.Top);

            return pageMarginsInfo;
        }

        public PageSetupInfo GetPageSetupInfo()
        {
            PageSetupInfo pageSetupInfo = new PageSetupInfo();

            pageSetupInfo.PaperType = this.Worksheet.WorksheetPageSetup.PaperType;
            pageSetupInfo.PageOrientation = this.Worksheet.WorksheetPageSetup.PageOrientation;
            pageSetupInfo.PageOrder = this.Worksheet.WorksheetPageSetup.PageOrder;
            pageSetupInfo.Errors = this.Worksheet.WorksheetPageSetup.PrintOptions.ErrorsPrintStyle;
            pageSetupInfo.FirstPageNumber = this.Worksheet.WorksheetPageSetup.FirstPageNumber;
            pageSetupInfo.FitToHeight = this.Worksheet.WorksheetPageSetup.FitToPagesTall;
            pageSetupInfo.FitToWidth = this.Worksheet.WorksheetPageSetup.FitToPagesWide;
            pageSetupInfo.Scale = SpreadsheetHelper.PercentFromScaleFactor(this.Worksheet.WorksheetPageSetup.ScaleFactor);

            return pageSetupInfo;
        }

        public PageBreaksInfo GetPageBreaksInfo()
        {
            PageBreaksInfo pageBreaksInfo = new PageBreaksInfo();

            List<PageBreakInfo> horizontalPageBreaks = new List<PageBreakInfo>();

            foreach (PageBreak pageBreak in this.Worksheet.WorksheetPageSetup.PageBreaks.SortedHorizontalPageBreaks)
            {
                PageBreakInfo breakInfo = new PageBreakInfo();

                breakInfo.Id = pageBreak.Index;
                breakInfo.Manual = true;
                breakInfo.Max = pageBreak.ToIndex;
                breakInfo.Min = pageBreak.FromIndex;

                horizontalPageBreaks.Add(breakInfo);
            }

            pageBreaksInfo.HorizontalPageBreaks = horizontalPageBreaks;

            List<PageBreakInfo> verticalPageBreaks = new List<PageBreakInfo>();

            foreach (PageBreak pageBreak in this.Worksheet.WorksheetPageSetup.PageBreaks.SortedVerticalPageBreaks)
            {
                PageBreakInfo breakInfo = new PageBreakInfo();

                breakInfo.Id = pageBreak.Index;
                breakInfo.Manual = true;
                breakInfo.Max = pageBreak.ToIndex;
                breakInfo.Min = pageBreak.FromIndex;

                verticalPageBreaks.Add(breakInfo);
            }

            pageBreaksInfo.VerticalPageBreaks = verticalPageBreaks;

            return pageBreaksInfo;
        }

        public CellRefRange GetAutoFilterRange()
        {
            if (this.Worksheet.Filter.FilterRange != null)
            {
                return new CellRefRange(this.Worksheet.Filter.FilterRange);
            }

            return null;
        }

        public IEnumerable<FilterColumnInfo> GetFilterColumnInfos()
        {
            for (int i = 0; i < this.Worksheet.Filter.Filters.Count; i++)
            {
                FilterColumnInfo filterColumnInfo = new FilterColumnInfo(this.Worksheet.Filter.Filters[i].RelativeColumnIndex);

                yield return filterColumnInfo;
            }
        }

        public IFilterInfo GetFilterInfo(int columnId)
        {
            IFilter filter = this.Worksheet.Filter.GetFilter(columnId);

            if (filter is ValuesCollectionFilter)
            {
                return this.GetFiltersInfo(columnId);
            }

            if (filter is CustomFilter)
            {
                return this.GetCustomFilterInfo(columnId);
            }

            if (filter is DynamicFilter)
            {
                return this.GetDynamicFilterInfo(columnId);
            }

            if (filter is TopFilter)
            {
                return this.GetTop10FilterInfo(columnId);
            }

            if (filter is FillColorFilter || filter is ForeColorFilter)
            {
                return this.GetColorFilterInfo(columnId);
            }

            return null;
        }

        public FiltersInfo GetFiltersInfo(int columnId)
        {
            ValuesCollectionFilter valuesCollectionFilter = (ValuesCollectionFilter)this.Worksheet.Filter.GetFilter(columnId);

            FiltersInfo filtersInfo = new FiltersInfo();
            filtersInfo.Blank = valuesCollectionFilter.Blank;

            foreach (string value in valuesCollectionFilter.StringValues)
            {
                filtersInfo.StringFilters.Add(value);
            }

            foreach (DateGroupItem item in valuesCollectionFilter.DateItems)
            {
                filtersInfo.DateFilters.Add(new DateGroupItemInfo(item));
            }

            return filtersInfo;
        }

        public CustomFiltersInfo GetCustomFilterInfo(int columnId)
        {
            CustomFilter customFilter = (CustomFilter)this.Worksheet.Filter.GetFilter(columnId);
            CustomFiltersInfo customFiltersInfo = new CustomFiltersInfo();

            CustomFilterCriteriaInfo condition1 = new CustomFilterCriteriaInfo(
                new CustomFilterCriteria(customFilter.Criteria1.ComparisonOperator, customFilter.Criteria1.ComparableFilterValue.ToString()));
            customFiltersInfo.CustomFilters.Add(condition1);

            if (customFilter.Criteria2 != null)
            {
                CustomFilterCriteriaInfo condition2 = new CustomFilterCriteriaInfo(
                    new CustomFilterCriteria(customFilter.Criteria2.ComparisonOperator, customFilter.Criteria2.ComparableFilterValue.ToString()));
                customFiltersInfo.CustomFilters.Add(condition2);
                customFiltersInfo.IsAnd = customFilter.LogicalOperator == LogicalOperator.And;
            }

            return customFiltersInfo;
        }

        public DynamicFilterInfo GetDynamicFilterInfo(int columnId)
        {
            DynamicFilter dynamicFilter = (DynamicFilter)this.Worksheet.Filter.GetFilter(columnId);

            DynamicFilterInfo dynamicFilterInfo = new DynamicFilterInfo();
            dynamicFilterInfo.DynamicFilterType = dynamicFilter.DynamicFilterType;

            return dynamicFilterInfo;
        }

        public Top10FilterInfo GetTop10FilterInfo(int columnId)
        {
            TopFilter top10Filter = (TopFilter)this.Worksheet.Filter.GetFilter(columnId);

            Top10FilterInfo top10FilterInfo = new Top10FilterInfo();

            top10FilterInfo.Value = top10Filter.Value;
            top10FilterInfo.Top = top10Filter.TopFilterType == TopFilterType.TopNumber || top10Filter.TopFilterType == TopFilterType.TopPercent;
            top10FilterInfo.Percent = top10Filter.TopFilterType == TopFilterType.BottomPercent || top10Filter.TopFilterType == TopFilterType.TopPercent;

            return top10FilterInfo;
        }

        public ColorFilterInfo GetColorFilterInfo(int columnId)
        {
            ColorFilterInfo colorFilterInfo = new ColorFilterInfo();

            FillColorFilter fillFilter = this.Worksheet.Filter.GetFilter(columnId) as FillColorFilter;
            if (fillFilter != null)
            {
                colorFilterInfo.CellColor = true;
                colorFilterInfo.DxfId = this.WorkbookContext.DifferentialFormatsContext.RegisterFormatAndGetId(fillFilter.Fill);
                return colorFilterInfo;
            }

            ForeColorFilter foreColorFilter = (ForeColorFilter)this.Worksheet.Filter.GetFilter(columnId);
            colorFilterInfo.CellColor = false;
            PatternFill fill = new PatternFill(PatternType.Solid, foreColorFilter.Color, ThemableColor.FromArgb(255, 255, 255, 255));
            colorFilterInfo.DxfId = this.WorkbookContext.DifferentialFormatsContext.RegisterFormatAndGetId(fill);

            return colorFilterInfo;
        }

        public SortStateInfo GetSortStateInfo()
        {
            SortStateInfo info = new SortStateInfo();

            //// TODO: add case sensitive and columns options
            info.Range = this.Worksheet.SortState.SortRange;

            return info;
        }

        public IEnumerable<SortConditionInfo> GetSortConditionInfos()
        {
            foreach (ISortCondition condition in this.Worksheet.SortState.SortConditions)
            {
                SortConditionInfo info = new SortConditionInfo();

                //// TODO: Fix false when Sort by column option is available
                info.Range = this.CalculateSortRange(this.Worksheet.SortState.SortRange, condition.RelativeIndex, false);

                ValuesSortCondition valuesSort = condition as ValuesSortCondition;
                if (valuesSort != null)
                {
                    info.Descending = valuesSort.SortOrder == SortOrder.Descending;
                    info.SortBy = SortBy.Value;
                }

                CustomValuesSortCondition customValuesSort = condition as CustomValuesSortCondition;
                if (customValuesSort != null)
                {
                    info.CustomList = this.GetCustomListValues(customValuesSort.CustomList);
                    info.SortBy = SortBy.Value;
                }

                FillColorSortCondition fillColorSortCondition = condition as FillColorSortCondition;
                if (fillColorSortCondition != null)
                {
                    info.Descending = fillColorSortCondition.SortOrder == SortOrder.Descending;
                    info.SortBy = SortBy.CellColor;
                    info.DxfId = this.WorkbookContext.DifferentialFormatsContext.RegisterFormatAndGetId(fillColorSortCondition.Fill);
                }

                ForeColorSortCondition foreColorCondition = condition as ForeColorSortCondition;
                if (foreColorCondition != null)
                {
                    info.Descending = foreColorCondition.SortOrder == SortOrder.Descending;
                    info.SortBy = SortBy.FontColor;
                    info.DxfId = this.WorkbookContext.DifferentialFormatsContext.RegisterFormatAndGetId(foreColorCondition.Color);
                }

                yield return info;
            }
        }

        public double GetDefaultColumnWidth()
        {
            return XlsxHelper.ConvertColumnPixelWidthToExcelWidth(this.Worksheet.Workbook, this.Worksheet.Columns.GetDefaultWidth().Value);
        }

        public RowHeight GetDefaultRowHeight()
        {
            return this.Worksheet.Rows.GetDefaultHeight();
        }

        public void InitFontInfos()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.InitFontInfos(entity);
            }
        }

        public void InitBordersInfos()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.InitBordersInfos(entity);
            }
        }

        public void RegisterPictureToIdRelationshipId(Image image, string relationshipId)
        {
            Guard.ThrowExceptionIfNull(image, "image");
            Guard.ThrowExceptionIfNullOrEmpty(relationshipId, "relationshipId");

            this.pictureToRelationshipId.Add(image, relationshipId);
        }

        public void RegisterWorksheetPart(Worksheet worksheet, Model.Parts.WorksheetPart worksheetPart)
        {
            this.workbookContext.RegisterWorksheetPart(worksheet, worksheetPart);
        }

        public int GetDataValidationsCount()
        {
            return this.dataValidationRuleToCellRanges.Count;
        }

        public IEnumerable<IDataValidationRule> GetDataValidationRules()
        {
            return this.dataValidationRuleToCellRanges.Keys;
        }

        public IEnumerable<CellRange> GetDataValidationRuleCellRanges(IDataValidationRule rule)
        {
            return this.dataValidationRuleToCellRanges[rule];
        }

        public CellRange GetUsedCellRange()
        {
            IEnumerable<IPropertyDefinition> properties =
                this.Worksheet.Cells.Properties.Select(p => p.PropertyDefinition)
                .Except(new IPropertyDefinition[] { CellPropertyDefinitions.DataValidationRuleProperty });

            CellRange usedCellRange = this.Worksheet.GetUsedCellRange(properties);

            return usedCellRange;
        }

        private void SetRowAndColumnPropertiesAsLocalInCellsWhereCellsHasValueProperty()
        {
            ICompressedList<ICellValue> valuesCollection = this.Worksheet.Cells.PropertyBag.GetPropertyValueCollection(CellPropertyDefinitions.ValueProperty);

            List<LongRange> nonDefaultRanges = new List<LongRange>();

            IEnumerable<Range<long, ICellValue>> ranges = valuesCollection.GetNonDefaultRanges();

            Range<long, ICellValue> firstRange = ranges.FirstOrDefault();
            if (firstRange == null || !ranges.Skip(1).Any())
            {
                return;
            }

            long start = firstRange.Start;
            long end = firstRange.End;

            foreach (Range<long, ICellValue> range in ranges.Skip(1))
            {
                if (end + 1 == range.Start)
                {
                    end = range.End;
                }
                else
                {
                    nonDefaultRanges.Add(new LongRange(start, end));
                    start = range.Start;
                    end = range.End;
                }
            }

            if (nonDefaultRanges.Count > 0 && nonDefaultRanges[nonDefaultRanges.Count - 1].End != end)
            {
                nonDefaultRanges.Add(new LongRange(start, end));
            }

            foreach (LongRange range in nonDefaultRanges)
            {
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.BottomBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.DiagonalDownBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.DiagonalUpBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.FillProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.FontFamilyProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.FontSizeProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.ForeColorProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.FormatProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.HorizontalAlignmentProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.IndentProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.IsBoldProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.IsItalicProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.IsLockedProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.IsWrappedProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.LeftBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.RightBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.TopBorderProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.UnderlineProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.VerticalAlignmentProperty, range.Start, range.End);
                this.SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty(CellPropertyDefinitions.StyleNameProperty, range.Start, range.End);
            }
        }

        private void SetRowAndColumnPropertyAsLocalInCellsWhereCellsHasValueProperty<T>(IPropertyDefinition<T> propertyDefinition, long fromIndex, long toIndex)
        {
            ICompressedList<T> value = this.Worksheet.Cells.PropertyBag.GetPropertyValueRespectingStyle(propertyDefinition, this.Worksheet, fromIndex, toIndex);
            this.Worksheet.Cells.PropertyBag.GetPropertyValueCollection(propertyDefinition).SetValue(value);
        }

        private void InitFontInfos(WorksheetEntityBase tableEntity)
        {
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.IsBoldProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.Bold = value; return fontInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.IsItalicProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.Italic = value; return fontInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.FontSizeProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.FontSize = value; return fontInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.FontFamilyProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.FontFamily = value; return fontInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.ForeColorProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.ForeColor = value; return fontInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.UnderlineProperty, this.worksheetEntityToFontInfoCompressedList[tableEntity], (fontInfo, value) => { fontInfo.UnderlineType = value; return fontInfo; });
            this.UpdateProperty(
                tableEntity,
                CellPropertyDefinitions.StyleNameProperty,
                this.worksheetEntityToFontInfoCompressedList[tableEntity],
                (fontInfo, value) =>
                {
                    int styleFormattingRecordIndex = this.workbookContext.GetStyleFormattingRecordId(value);
                    FormattingRecord styleFormattingRecord = this.workbookContext.StyleSheet.StyleFormattingTable[styleFormattingRecordIndex];

                    if (styleFormattingRecord.FontInfoId.HasValue)
                    {
                        FontInfo styleFontInfo = this.workbookContext.StyleSheet.FontInfoTable[styleFormattingRecord.FontInfoId.Value];
                        fontInfo = fontInfo.MergeWith(styleFontInfo);
                    }

                    return fontInfo;
                });
        }

        private void InitBordersInfos(WorksheetEntityBase tableEntity)
        {
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.LeftBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.Left = value; return bordersInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.TopBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.Top = value; return bordersInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.RightBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.Right = value; return bordersInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.BottomBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.Bottom = value; return bordersInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.DiagonalUpBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.DiagonalUp = value; return bordersInfo; });
            this.UpdateProperty(tableEntity, CellPropertyDefinitions.DiagonalDownBorderProperty, this.worksheetEntityToBordersInfoCompressedList[tableEntity], (bordersInfo, value) => { bordersInfo.DiagonalDown = value; return bordersInfo; });
            this.UpdateProperty(
                tableEntity,
                CellPropertyDefinitions.StyleNameProperty,
                this.worksheetEntityToBordersInfoCompressedList[tableEntity],
                (bordersInfo, value) =>
                {
                    int styleFormattingRecordIndex = this.workbookContext.GetStyleFormattingRecordId(value);
                    FormattingRecord styleFormattingRecord = this.workbookContext.StyleSheet.StyleFormattingTable[styleFormattingRecordIndex];

                    if (styleFormattingRecord.BordersInfoId.HasValue)
                    {
                        BordersInfo styleBordersInfo = this.workbookContext.StyleSheet.BordersInfoTable[styleFormattingRecord.BordersInfoId.Value];
                        bordersInfo = bordersInfo.MergeWith(styleBordersInfo);
                    }

                    return bordersInfo;
                });
        }

        private void Initialize()
        {
            this.InitStyleNames();
            this.InitFills();
            this.InitCellValueFormats();
            this.InitAlignments();
            this.InitProtection();
            this.InitDataValidationRules();
            this.InitializeColumnProperties();
        }

        private void InitStyleNames()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.worksheetEntityToStyleNameCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.StyleNameProperty);
            }
        }

        private void InitFills()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.worksheetEntityToFillCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.FillProperty);
            }
        }

        private void InitCellValueFormats()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.worksheetEntityToCellValueFormatCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.FormatProperty);
            }
        }

        private void InitAlignments()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.worksheetEntityToHorizontalAlignmentCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.HorizontalAlignmentProperty);
                this.worksheetEntityToVerticalAlignmentCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.VerticalAlignmentProperty);
                this.worksheetEntityToIndentCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.IndentProperty);
                this.worksheetEntityToIsWrappedCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.IsWrappedProperty);
            }
        }

        private void InitProtection()
        {
            foreach (WorksheetEntityBase entity in this.WorkbookContext.GetWorksheetEntitiesFromWorksheet(this.Worksheet))
            {
                this.worksheetEntityToIsLockedCompressedList[entity] = entity.PropertyBagBase.GetPropertyValueCollection(CellPropertyDefinitions.IsLockedProperty);
            }
        }

        private void InitDataValidationRules()
        {
            CellsPropertyBag propertyBag = this.Worksheet.Cells.PropertyBag;
            IPropertyDefinition<IDataValidationRule> dataValidationPropertyDefinition = CellPropertyDefinitions.DataValidationRuleProperty;
            this.dataValidationRulesCompressedList = propertyBag.GetPropertyValueCollection(dataValidationPropertyDefinition);

            foreach (Range<long, IDataValidationRule> range in this.dataValidationRulesCompressedList.GetNonDefaultRanges())
            {
                IDataValidationRule rule = range.Value;
                if (!this.dataValidationRuleToCellRanges.ContainsKey(rule))
                {
                    IEnumerable<CellRange> cellRanges = propertyBag.GetValueContainingCellRanges(dataValidationPropertyDefinition, rule);
                    this.dataValidationRuleToCellRanges.Add(rule, cellRanges);
                }
            }
        }

        private void InitializeColumnProperties()
        {
            ICompressedList<ColumnWidth> columnWidthCompressedList = this.Worksheet.Columns.PropertyBag.GetPropertyValueCollection(ColumnsPropertyBag.WidthProperty);
            ICompressedList<bool> columnHiddenCompressedList = this.Worksheet.Columns.PropertyBag.GetPropertyValueCollection(ColumnsPropertyBag.HiddenProperty);
            ICompressedList<int> columnOutlineLevelCompressedList = this.Worksheet.Columns.PropertyBag.GetPropertyValueCollection(ColumnsPropertyBag.OutlineLevelProperty);

            this.UpdateProperty(this.Worksheet.Columns,
                columnWidthCompressedList,
                this.columnInfoCompressedList,
                (columnInfo, value) =>
                {
                    columnInfo.IsCustom = value.IsCustom;
                    columnInfo.BestFit = value.IsAutoFit;
                    columnInfo.Width = XlsxHelper.ConvertColumnPixelWidthToExcelWidth(this.Worksheet.Workbook, value.Value);

                    return columnInfo;
                });

            this.UpdateProperty(this.Worksheet.Columns, columnHiddenCompressedList, this.columnInfoCompressedList, (columnInfo, value) => { columnInfo.Hidden = value; return columnInfo; });
            this.UpdateProperty(this.Worksheet.Columns, columnOutlineLevelCompressedList, this.columnInfoCompressedList, (columnInfo, value) => { columnInfo.OutlineLevel = value; return columnInfo; });

            this.InitializeColumnProperty(CellPropertyDefinitions.BottomBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.DiagonalDownBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.DiagonalUpBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.FillProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.FontFamilyProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.FontSizeProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.ForeColorProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.FormatProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.HorizontalAlignmentProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.IndentProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.IsBoldProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.IsItalicProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.IsLockedProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.IsWrappedProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.LeftBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.RightBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.StyleNameProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.TopBorderProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.UnderlineProperty);
            this.InitializeColumnProperty(CellPropertyDefinitions.VerticalAlignmentProperty);
        }

        private void InitializeColumnProperty<T>(IPropertyDefinition<T> propertyDefinition)
        {
            ICompressedList<T> propertyCollection = this.Worksheet.Columns.PropertyBag.GetPropertyValueCollection(propertyDefinition);
            this.UpdateProperty(
                this.Worksheet.Columns,
                propertyCollection,
                this.columnInfoCompressedList,
                (columnInfo, value) =>
                {
                    return columnInfo;
                });
        }

        private void InitializePropertyToIsDefault()
        {
            this.propertyToIsDefaultFunc = new Dictionary<IPropertyDefinition, Func<ICompressedList, bool>>();
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.BottomBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.DataValidationRuleProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.DiagonalDownBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.DiagonalUpBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.FillProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.FontFamilyProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.FontSizeProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.ForeColorProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.FormatProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.HorizontalAlignmentProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.IndentProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.IsBoldProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.IsItalicProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.IsLockedProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.IsWrappedProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.LeftBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.RightBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.StyleNameProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.TopBorderProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.UnderlineProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.ValueProperty);
            this.AddPropertyToIsDefaultFuncDefault(CellPropertyDefinitions.VerticalAlignmentProperty);

            this.propertyToIsDefaultFunc.Add(
                RowsPropertyBag.HeightProperty,
                (compressedList) =>
                {
                    CompressedList<RowHeight> compressedListOfT = compressedList as CompressedList<RowHeight>;
                    ValueBox<RowHeight> value = compressedListOfT.GetValueInternal(compressedListOfT.FromIndex);

                    return value == null || !value.Value.IsCustom;
                });
            this.AddPropertyToIsDefaultFuncDefault(RowsPropertyBag.HiddenProperty);
            this.AddPropertyToIsDefaultFuncDefault(RowsPropertyBag.OutlineLevelProperty);

            this.AddPropertyToIsDefaultFuncDefault(ColumnsPropertyBag.HiddenProperty);
            this.AddPropertyToIsDefaultFuncDefault(ColumnsPropertyBag.OutlineLevelProperty);
            this.AddPropertyToIsDefaultFuncDefault(ColumnsPropertyBag.WidthProperty);
        }

        private void AddPropertyToIsDefaultFuncDefault<T>(IPropertyDefinition<T> propertyDefinition)
        {
            if (this.propertyToIsDefaultFunc.ContainsKey(propertyDefinition))
            {
                return;
            }

            this.propertyToIsDefaultFunc.Add(propertyDefinition,
                (compressedList) =>
                {
                    CompressedList<T> compressedListOfT = compressedList as CompressedList<T>;
                    ValueBox<T> value = compressedListOfT.GetValueInternal(compressedListOfT.FromIndex);

                    return value == null;
                });
        }

        private void UpdateProperty<TSource, TTarget>(
            WorksheetEntityBase tableEntity,
            IPropertyDefinition<TSource> sourcePropertyDefinition,
            ICompressedList<TTarget> targetCompressedList,
            Func<TTarget, TSource, TTarget> propertySetter)
            where TTarget : struct
        {
            this.UpdateProperty(tableEntity, tableEntity.PropertyBagBase.GetPropertyValueCollection(sourcePropertyDefinition), targetCompressedList, propertySetter);
        }

        // Updates the targetCompressedList with values from the source compressed list using the propertySetter function.
        // Eg: if we want to collect fontInfos, we first start with empty font infos, then update it with bold property, then with italic
        // Empty[-----]
        // Bold[b-b--]
        // Italic after bold[b-b--
        //                   i--i-]
        private void UpdateProperty<TSource, TTarget>(
            WorksheetEntityBase tableEntity,
            ICompressedList<TSource> sourceCompressedList,
            ICompressedList<TTarget> targetCompressedList,
            Func<TTarget, TSource, TTarget> propertySetter)
            where TTarget : struct
        {
            foreach (Range<long, TSource> sourceRange in sourceCompressedList.GetNonDefaultRanges())
            {
                this.NonDefaultFormattingCompressedList[tableEntity].SetValue(sourceRange.Start, sourceRange.End, true);

                if (TelerikHelper.EqualsOfT(sourceRange.Value, sourceCompressedList.GetDefaultValue()))
                {
                    continue;
                }

                ICompressedList<TTarget> targetSublist = targetCompressedList.GetValue(sourceRange.Start, sourceRange.End);
                CompressedList<TTarget> modifiedTargetSublist = new CompressedList<TTarget>(targetSublist);
                foreach (Range<long, TTarget> targetRange in targetSublist)
                {
                    TTarget targetValue = targetRange.Value;

                    TTarget modifiedTargetValue = propertySetter(targetValue, sourceRange.Value);

                    modifiedTargetSublist.SetValue(targetRange.Start, targetRange.End, modifiedTargetValue);
                }

                targetCompressedList.SetValue(modifiedTargetSublist);
            }
        }

        private CellRange CalculateSortRange(CellRange sortRange, int index, bool sortByColumns)
        {
            if (sortByColumns)
            {
                int absoluteRowIndex = sortRange.FromIndex.RowIndex + index;
                return new CellRange(absoluteRowIndex, sortRange.FromIndex.ColumnIndex, absoluteRowIndex, sortRange.ToIndex.RowIndex);
            }

            int absoluteColumnIndex = sortRange.FromIndex.ColumnIndex + index;
            return new CellRange(sortRange.FromIndex.RowIndex, absoluteColumnIndex, sortRange.ToIndex.RowIndex, absoluteColumnIndex);
        }

        private string GetCustomListValues(string[] customList)
        {
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < customList.Length; i++)
            {
                builder.Append(customList[i]);
                if (i < customList.Length - 1)
                {
                    builder.Append(SpreadsheetCultureHelper.Comma);
                }
            }

            return builder.ToString();
        }
    }
}