﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
#endif
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Model.Protection;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts
{
    internal class XlsxWorksheetImportContext : XlsxWorksheetContextBase, IXlsxWorksheetImportContext
    {
        private readonly ICompressedList<int> cellStyleIndexCompressedList;
        private readonly ICompressedList<int> rowStyleIndexCompressedList;
        private readonly ICompressedList<int> columnStyleIndexCompressedList;
        private readonly XlsxWorkbookImportContext workbookContext;
        private readonly Dictionary<int, SharedFormulaInfo> sharedFormulas;
        private readonly Dictionary<FloatingShapeBase, TwoCellAnchorInfo> shapesIndices;
        private readonly ReferenceCounter referenceCounter;

        public XlsxWorksheetImportContext(XlsxWorkbookImportContext workbookContext, Worksheet worksheet)
            : base(worksheet)
        {
            Guard.ThrowExceptionIfNull(workbookContext, "workbookContext");

            this.workbookContext = workbookContext;
            this.cellStyleIndexCompressedList = new CompressedList<int>(0, SpreadsheetDefaultValues.CellCount - 1);
            this.rowStyleIndexCompressedList = new CompressedList<int>(0, SpreadsheetDefaultValues.RowCount - 1);
            this.columnStyleIndexCompressedList = new CompressedList<int>(0, SpreadsheetDefaultValues.ColumnCount - 1);
            this.sharedFormulas = new Dictionary<int, SharedFormulaInfo>();
            this.shapesIndices = new Dictionary<FloatingShapeBase, TwoCellAnchorInfo>();
            this.referenceCounter = new ReferenceCounter();
        }

        public AutoFilterInfo AutoFilterInfo { get; set; }

        public Dictionary<FloatingShapeBase, TwoCellAnchorInfo> ShapesAnchorsWaitingSize
        {
            get
            {
                return this.shapesIndices;
            }
        }

        public IXlsxWorkbookImportContext WorkbookContext
        {
            get
            {
                return this.workbookContext;
            }
        }

        public ResourceManager Resources
        {
            get
            {
                return this.workbookContext.Resources;
            }
        }

        public DocumentTheme Theme
        {
            get
            {
                return this.workbookContext.Theme;
            }
            set
            {
                this.workbookContext.Theme = value;
            }
        }

        public bool IsImportSuspended
        {
            get
            {
                return false;
            }
        }

        public bool SupportCharts
        {
            get
            {
                return true;
            }
        }

        public void RegisterSharedFormula(int index, SharedFormulaInfo sharedFormula)
        {
            this.sharedFormulas[index] = sharedFormula;
        }

        public SharedFormulaInfo GetSharedFormula(int index)
        {
            SharedFormulaInfo sharedFormula;
            if (this.sharedFormulas.TryGetValue(index, out sharedFormula))
            {
                return sharedFormula;
            }

            return null;
        }

        public void ApplyRowInfo(RowInfo rowInfo)
        {
            Guard.ThrowExceptionIfNull(rowInfo, "rowInfo");

            int rowIndex = this.referenceCounter.GetRow(rowInfo);

            if (rowInfo.Hidden)
            {
                this.Worksheet.Rows[rowIndex].SetHidden(rowInfo.Hidden);
            }

            if (rowInfo.Height.HasValue)
            {
                RowHeight height = new RowHeight(rowInfo.Height.Value, rowInfo.IsCustom);
                this.Worksheet.Rows[rowIndex].SetHeight(height);
            }

            if (rowInfo.OutlineLevel != 0)
            {
                this.Worksheet.Rows[rowIndex].SetOutlineLevel(rowInfo.OutlineLevel);
            }

            if (rowInfo.StyleIndex != this.rowStyleIndexCompressedList.GetDefaultValue())
            {
                this.rowStyleIndexCompressedList.SetValue(rowIndex, rowInfo.StyleIndex);
            }
        }

        public void ApplyColumnInfo(Range<long, ColumnInfo> columnRange)
        {
            Guard.ThrowExceptionIfNull(columnRange, "columnRange");
            ColumnInfo columnInfo = columnRange.Value;

            if (columnInfo.OutlineLevel != 0)
            {
                this.Worksheet.Columns[(int)columnRange.Start, (int)columnRange.End].SetOutlineLevel(columnInfo.OutlineLevel);
            }

            if (columnInfo.Hidden)
            {
                this.Worksheet.Columns[(int)columnRange.Start, (int)columnRange.End].SetHidden(columnInfo.Hidden);
            }

            ColumnWidth width = new ColumnWidth(columnInfo.Width, columnInfo.IsCustom);
            this.Worksheet.Columns[(int)columnRange.Start, (int)columnRange.End].SetWidth(width);

            if (columnInfo.StyleIndex != this.columnStyleIndexCompressedList.GetDefaultValue())
            {
                this.columnStyleIndexCompressedList.SetValue(columnRange.Start, columnRange.End, columnInfo.StyleIndex);
            }
        }

        public void ApplyCellInfo(CellInfo cellInfo)
        {
            Guard.ThrowExceptionIfNull(cellInfo, "cellInfo");

            int rowIndex;
            int columnIndex;
            this.referenceCounter.GetCellIndex(cellInfo, out rowIndex, out columnIndex);

            long longIndex = CellsPropertyBag.ConvertCellIndexToLong(rowIndex, columnIndex);
            if (cellInfo.CellValue != null)
            {
                CellsPropertyBag propertyBag = this.Worksheet.Cells.PropertyBag;
                ICompressedList<ICellValue> values = propertyBag.GetPropertyValueCollection(CellPropertyDefinitions.ValueProperty);
                values.SetValue(longIndex, cellInfo.CellValue);
            }

            this.cellStyleIndexCompressedList.SetValue(longIndex, cellInfo.StyleIndex);
        }

        public void ApplyMergeCellInfo(MergedCellInfo mergeCellInfo)
        {
            if (mergeCellInfo.CellRange.ColumnCount > 1 || mergeCellInfo.CellRange.RowCount > 1)
            {
                this.Worksheet.Cells.MergedCellRanges.AddMergedRange(mergeCellInfo.CellRange);
            }
        }

        public void ApplySheetProtectionInfo(WorksheetProtectionInfo info)
        {
            Guard.ThrowExceptionIfNull(info, "sheetProtectionInfo");

            this.Worksheet.ProtectionData.Enforced = info.Enforced;
            this.Worksheet.ProtectionData.AlgorithmName = info.AlgorithmName;
            this.Worksheet.ProtectionData.Hash = info.HashValue;
            this.Worksheet.ProtectionData.Salt = info.SaltValue;
            this.Worksheet.ProtectionData.SpinCount = info.SpinCount;
            this.Worksheet.ProtectionData.Password = info.Password;

            this.Worksheet.ProtectionOptions = new WorksheetProtectionOptions(allowDeleteColumns: !info.DeleteColumns,
                allowInsertRows: !info.InsertRows,
                allowDeleteRows: !info.DeleteRows,
                allowInsertColumns: !info.InsertColumns,
                allowFormatCells: !info.FormatCells,
                allowFormatColumns: !info.FormatColumns,
                allowFormatRows: !info.FormatRows,
                allowFiltering: !info.AutoFilter,
                allowSorting: !info.Sort);
        }

        public void ApplyDefaultColumnWidth(ColumnWidth width)
        {
            Guard.ThrowExceptionIfNull(width, "width");

            this.Worksheet.Columns.SetDefaultWidth(width);
        }

        public void ApplyDefaultColumnWidthDefaultRowHeight(RowHeight height)
        {
            Guard.ThrowExceptionIfNull(height, "height");

            this.Worksheet.Rows.SetDefaultHeight(height);
        }

        public void BeginImport()
        {
            throw new NotImplementedException();
        }

        public void EndImport()
        {
            throw new NotImplementedException();
        }

        public void EndImport(XlsxWorkbookImportContext workbookContext)
        {
            Guard.ThrowExceptionIfNull(workbookContext, "workbookContext");

            foreach (Range<long, int> cellStyleIndexRange in this.cellStyleIndexCompressedList.GetNonDefaultRanges())
            {
                FormattingRecord directFormatting = workbookContext.StyleSheet.DirectFormattingTable[cellStyleIndexRange.Value];
                this.ImportDirectFormattingInRange(workbookContext, directFormatting, this.Worksheet.Cells.PropertyBag, cellStyleIndexRange.Start, cellStyleIndexRange.End);
            }

            foreach (Range<long, int> columnStyleIndexRange in this.columnStyleIndexCompressedList.GetNonDefaultRanges())
            {
                FormattingRecord directFormatting = workbookContext.StyleSheet.DirectFormattingTable[columnStyleIndexRange.Value];
                this.ImportDirectFormattingInRange(workbookContext, directFormatting, this.Worksheet.Columns.PropertyBag, columnStyleIndexRange.Start, columnStyleIndexRange.End);
            }

            foreach (Range<long, int> rowStyleIndexRange in this.rowStyleIndexCompressedList.GetNonDefaultRanges())
            {
                FormattingRecord directFormatting = workbookContext.StyleSheet.DirectFormattingTable[rowStyleIndexRange.Value];
                this.ImportDirectFormattingInRange(workbookContext, directFormatting, this.Worksheet.Rows.PropertyBag, rowStyleIndexRange.Start, rowStyleIndexRange.End);
            }

            this.ImportShapesSizes();
            this.ApplyAutoFilter();
        }

        public void RegisterResource(string relationshipId, IResource resource)
        {
            this.workbookContext.RegisterResource(relationshipId, resource);
        }

#if !SILVERLIGHT
        public ChartShape GetChartForChartPart(ChartPart chartPart)
        {
            return this.workbookContext.GetChartForChartPart(chartPart);
        }

        public void RegisterChartPartForChart(ChartShape chart, ChartPart chartPart)
        {
            this.workbookContext.RegisterChartPartForChart(chart, chartPart);
        }

        public FormulaChartData GetFormulaChartData(string formula)
        {
            return this.workbookContext.GetFormulaChartData(formula);
        }

        public void RegisterSeriesGroupAwaitingAxisGroupName(ISupportAxes seriesGroup, int catAxisId, int valAxisId)
        {
            throw new NotImplementedException();
        }

        public void RegisterAxisGroup(AxisGroupName groupName, int thisId, int otherId)
        {
            throw new NotImplementedException();
        }

        public void PairSeriesGroupsWithAxes()
        {
            throw new NotImplementedException();
        }
#endif

        public string GetSharedStringValue(int index)
        {
            SharedStringType currentSharedStringType = this.workbookContext.SharedStrings[index].Type;
            string sharedStringValue = string.Empty;

            switch (currentSharedStringType)
            {
                case SharedStringType.Text:
                    {
                        TextSharedString textSharedString = this.workbookContext.SharedStrings[index] as TextSharedString;
                        if (textSharedString != null)
                        {
                            sharedStringValue = textSharedString.Text;
                        }
                    }
                    break;

                case SharedStringType.RichText:
                    {
                        RichTextRunSharedString richTextRunSharedString = this.workbookContext.SharedStrings[index] as RichTextRunSharedString;
                        if (richTextRunSharedString != null)
                        {
                            sharedStringValue = richTextRunSharedString.Text;
                        }
                    }
                    break;

                default:
                    sharedStringValue = string.Empty;
                    break;
            }

            return sharedStringValue;
        }

        public IResource GetResourceByResourceKey(string relationshipId)
        {
            return this.workbookContext.GetResourceByResourceKey(relationshipId);
        }

        public void ApplyPrintOptions(PrintOptionsInfo printOptionsInfo)
        {
            Guard.ThrowExceptionIfNull(printOptionsInfo, "printOptionsInfo");

            WorksheetPageSetup pageSetup = this.Worksheet.WorksheetPageSetup;

            pageSetup.CenterHorizontally = printOptionsInfo.HorizontalCentered;
            pageSetup.CenterVertically = printOptionsInfo.VerticalCentered;
            pageSetup.PrintOptions.PrintGridlines = printOptionsInfo.GridLines;
            pageSetup.PrintOptions.PrintRowColumnHeadings = printOptionsInfo.Headings;
        }

        public void ApplyPageMargins(PageMarginsInfo pageMarginsInfo)
        {
            Guard.ThrowExceptionIfNull(pageMarginsInfo, "pageMarginsInfo");

            this.Worksheet.WorksheetPageSetup.Margins = new PageMargins(UnitHelper.InchToDip(pageMarginsInfo.Left),
                UnitHelper.InchToDip(pageMarginsInfo.Top),
                UnitHelper.InchToDip(pageMarginsInfo.Right),
                UnitHelper.InchToDip(pageMarginsInfo.Bottom),
                UnitHelper.InchToDip(pageMarginsInfo.Header),
                UnitHelper.InchToDip(pageMarginsInfo.Footer));
        }

        public void ApplyPageSetup(PageSetupInfo pageSetupInfo)
        {
            Guard.ThrowExceptionIfNull(pageSetupInfo, "pageSetupInfo");

            this.Worksheet.WorksheetPageSetup.PaperType = pageSetupInfo.PaperType;
            this.Worksheet.WorksheetPageSetup.PageOrientation = pageSetupInfo.PageOrientation;
            this.Worksheet.WorksheetPageSetup.PageOrder = pageSetupInfo.PageOrder;
            this.Worksheet.WorksheetPageSetup.PrintOptions.ErrorsPrintStyle = pageSetupInfo.Errors;
            this.Worksheet.WorksheetPageSetup.FirstPageNumber = pageSetupInfo.FirstPageNumber;
            this.Worksheet.WorksheetPageSetup.FitToPagesTall = pageSetupInfo.FitToHeight;
            this.Worksheet.WorksheetPageSetup.FitToPagesWide = pageSetupInfo.FitToWidth;
            this.Worksheet.WorksheetPageSetup.ScaleFactor = SpreadsheetHelper.ScaleFactorFromPercent(pageSetupInfo.Scale);
        }

        public void ApplyHorizontalPageBreakInfo(PageBreakInfo pageBreakInfo)
        {
            Guard.ThrowExceptionIfNull(pageBreakInfo, "pageBreakInfo");

            this.Worksheet.WorksheetPageSetup.PageBreaks.AddPageBreakInternal(new PageBreak(PageBreakType.Horizontal, pageBreakInfo.Id, pageBreakInfo.Min, pageBreakInfo.Max));
        }

        public void ApplyVerticalPageBreakInfo(PageBreakInfo pageBreakInfo)
        {
            Guard.ThrowExceptionIfNull(pageBreakInfo, "pageBreakInfo");

            this.Worksheet.WorksheetPageSetup.PageBreaks.AddPageBreakInternal(new PageBreak(PageBreakType.Vertical, pageBreakInfo.Id, pageBreakInfo.Min, pageBreakInfo.Max));
        }

        public void ApplyAutoFilter()
        {
            if (this.AutoFilterInfo == null)
            {
                return;
            }

            this.Worksheet.Filter.FilterRange = this.AutoFilterInfo.Range;

            if (this.AutoFilterInfo.FilterColumnInfos == null)
            {
                return;
            }

            for (int j = 0; j < this.AutoFilterInfo.FilterColumnInfos.Count; j++)
            {
                FilterColumnInfo filterColumnInfo = this.AutoFilterInfo.FilterColumnInfos[j];
                if (filterColumnInfo != null)
                {
                    IFilter filter = this.GetFilter(filterColumnInfo);
                    if (filter != null)
                    {
                        this.Worksheet.Filter.SetFilter(filter);
                    }
                }
            }
        }

        public void ApplySortState(SortStateInfo sortStateInfo)
        {
            Guard.ThrowExceptionIfNull(sortStateInfo, "sortStateInfo");

            List<ISortCondition> conditions = new List<ISortCondition>();
            for (int i = 0; i < sortStateInfo.Conditions.Count; i++)
            {
                ISortCondition sortCondition = this.GetSortCondition(sortStateInfo.Range, sortStateInfo.Conditions[i]);
                if (sortCondition != null)
                {
                    conditions.Add(sortCondition);
                }
            }

            if (conditions.Count > 0)
            {
                this.Worksheet.SortState.SetInternal(sortStateInfo.Range, conditions.ToArray());
            }
        }

        private ISortCondition GetSortCondition(CellRange sortRange, SortConditionInfo conditionInfo)
        {
            Guard.ThrowExceptionIfNull(conditionInfo, "conditionInfo");

            int index = conditionInfo.Range.FromIndex.ColumnIndex - sortRange.FromIndex.ColumnIndex; //// TODO: fix the index when sorting by columns is ready
            SortOrder sortOrder = conditionInfo.Descending ? SortOrder.Descending : SortOrder.Ascending;

            if (conditionInfo.SortBy == SortBy.Value)
            {
                if (!string.IsNullOrEmpty(conditionInfo.CustomList))
                {
                    string[] customList = conditionInfo.CustomList.Split(SpreadsheetCultureHelper.Comma[0]);
                    return new CustomValuesSortCondition(index, customList, SortOrder.Ascending);
                }

                return new ValuesSortCondition(index, sortOrder);
            }

            if (conditionInfo.SortBy == SortBy.CellColor)
            {
                // In case fill for the condition cannot be retrieved from the differential formats, ignore the condition
                IFill fill = this.WorkbookContext.DifferentialFormatsContext.GetFillById((int)conditionInfo.DxfId);
                if (fill != null)
                {
                    return new FillColorSortCondition(index, fill, sortOrder);
                }
            }

            if (conditionInfo.SortBy == SortBy.FontColor)
            {
                // In case fill for the condition cannot be retrieved from the differential formats, ignore the condition
                ThemableColor color = this.WorkbookContext.DifferentialFormatsContext.GetColorById((int)conditionInfo.DxfId);
                if (color != null)
                {
                    return new ForeColorSortCondition(index, color, sortOrder);
                }
            }

            return null;
        }

        private IFilter GetFilter(FilterColumnInfo filterColumnInfo)
        {
            Guard.ThrowExceptionIfNull(filterColumnInfo, "filterColumnInfo");

            FiltersInfo filtersInfo = filterColumnInfo.FilterInfo as FiltersInfo;
            if (filtersInfo != null)
            {
                return this.GetValuesCollectionFilter(filterColumnInfo.ColumnId, filtersInfo);
            }

            CustomFiltersInfo customFiltersInfo = filterColumnInfo.FilterInfo as CustomFiltersInfo;
            if (customFiltersInfo != null)
            {
                return this.GetCustomFilter(filterColumnInfo.ColumnId, customFiltersInfo);
            }

            DynamicFilterInfo dynamicFilterInfo = filterColumnInfo.FilterInfo as DynamicFilterInfo;
            if (dynamicFilterInfo != null)
            {
                return this.GetDynamicFilter(filterColumnInfo.ColumnId, dynamicFilterInfo);
            }

            Top10FilterInfo top10FilterInfo = filterColumnInfo.FilterInfo as Top10FilterInfo;
            if (top10FilterInfo != null)
            {
                return this.GetTop10Filter(filterColumnInfo.ColumnId, top10FilterInfo);
            }

            ColorFilterInfo colorFilterInfo = filterColumnInfo.FilterInfo as ColorFilterInfo;
            if (colorFilterInfo != null)
            {
                return this.GetColorFilter(filterColumnInfo.ColumnId, colorFilterInfo);
            }

            return null;
        }

        private ValuesCollectionFilter GetValuesCollectionFilter(int columnId, FiltersInfo filtersInfo)
        {
            Guard.ThrowExceptionIfNull(filtersInfo, "filtersInfo");

            List<DateGroupItem> dateItems = new List<DateGroupItem>();
            for (int i = 0; i < filtersInfo.DateFilters.Count; i++)
            {
                DateGroupItemInfo itemInfo = filtersInfo.DateFilters[i];
                DateGroupItem item = new DateGroupItem(itemInfo.DateTimeGroupingType,
                    itemInfo.Year,
                    itemInfo.Month,
                    itemInfo.Day,
                    itemInfo.Hour,
                    itemInfo.Minute,
                    itemInfo.Second);

                dateItems.Add(item);
            }

            return new ValuesCollectionFilter(columnId, filtersInfo.StringFilters, dateItems, filtersInfo.Blank);
        }

        private CustomFilter GetCustomFilter(int columnId, CustomFiltersInfo customFiltersInfo)
        {
            Guard.ThrowExceptionIfNull(customFiltersInfo, "customFiltersInfo");

            CustomFilterCriteriaInfo criteriaInfo1 = customFiltersInfo.CustomFilters[0];

            if (customFiltersInfo.CustomFilters.Count == 1)
            {
                return new CustomFilter(columnId, criteriaInfo1.ToCriteria());
            }

            LogicalOperator conjunctionOperator = customFiltersInfo.IsAnd ? LogicalOperator.And : LogicalOperator.Or;
            CustomFilterCriteriaInfo criteriaInfo2 = customFiltersInfo.CustomFilters[1];

            return new CustomFilter(columnId, criteriaInfo1.ToCriteria(), conjunctionOperator, criteriaInfo2.ToCriteria());
        }

        private DynamicFilter GetDynamicFilter(int columnId, DynamicFilterInfo dynamicFilterInfo)
        {
            Guard.ThrowExceptionIfNull(dynamicFilterInfo, "dynamicFilterInfo");

            return new DynamicFilter(columnId, dynamicFilterInfo.DynamicFilterType);
        }

        private TopFilter GetTop10Filter(int columnId, Top10FilterInfo top10FilterInfo)
        {
            Guard.ThrowExceptionIfNull(top10FilterInfo, "top10FilterInfo");

            TopFilterType type;
            if (top10FilterInfo.Top)
            {
                type = top10FilterInfo.Percent ? TopFilterType.TopPercent : TopFilterType.TopNumber;
            }
            else
            {
                type = top10FilterInfo.Percent ? TopFilterType.BottomPercent : TopFilterType.BottomNumber;
            }

            return new TopFilter(columnId, type, top10FilterInfo.Value);
        }

        private IFilter GetColorFilter(int columnId, ColorFilterInfo colorFilterInfo)
        {
            //// In case fill for the filter cannot be retrieved from the differential formats, ignore the filter.
            IFill fill = this.WorkbookContext.DifferentialFormatsContext.GetFillById(colorFilterInfo.DxfId);
            if (fill != null)
            {
                if (colorFilterInfo.CellColor)
                {
                    return new FillColorFilter(columnId, fill);
                }

                PatternFill pattern = fill as PatternFill;
                if (pattern != null)
                {
                    //// Note that the ForeColor of the filter is stored in the dxfs as a pattern fill!
                    return new ForeColorFilter(columnId, pattern.PatternColor);
                }
            }

            return null;
        }

        /// <summary>
        /// Imports the shapes sizes from the from cell and to cell.
        /// Used in case the shape size is not indicated in the TransformElement.
        /// </summary>
        private void ImportShapesSizes()
        {
            if (this.ShapesAnchorsWaitingSize.Keys.Count == 0)
            {
                return;
            }

            RadWorksheetLayout worksheetLayout = this.Worksheet.Workbook.GetWorksheetLayout(this.Worksheet, false);

            foreach (FloatingShapeBase shape in this.ShapesAnchorsWaitingSize.Keys)
            {
                if (this.Worksheet.Shapes.Contains(shape))
                {
                    TwoCellAnchorInfo info = this.ShapesAnchorsWaitingSize[shape];

                    Point fromIndexTopLeft = worksheetLayout.GetTopLeftPointFromCellIndex(info.FromIndex);
                    Point toIndexTopLeft = worksheetLayout.GetTopLeftPointFromCellIndex(info.ToIndex);

                    double restrictedRotationAngle = SpreadsheetHelper.RestrictRotationAngle(shape.RotationAngle);

                    bool rotationAngleRequiresCellIndexChange = (restrictedRotationAngle >= 45 && restrictedRotationAngle < 135) ||
                        (restrictedRotationAngle > 225 && restrictedRotationAngle < 315);

                    double width;
                    double height;

                    if (rotationAngleRequiresCellIndexChange)
                    {
                        width = toIndexTopLeft.Y + info.ToOffsetY - (fromIndexTopLeft.Y + info.FromOffsetY);
                        height = toIndexTopLeft.X + info.ToOffsetX - (fromIndexTopLeft.X + info.FromOffsetX);
                    }
                    else
                    {
                        width = toIndexTopLeft.X + info.ToOffsetX - (fromIndexTopLeft.X + info.FromOffsetX);
                        height = toIndexTopLeft.Y + info.ToOffsetY - (fromIndexTopLeft.Y + info.FromOffsetY);
                    }

                    shape.Width = width;
                    shape.Height = height;
                }
            }
        }

        private void ImportDirectFormattingInRange(XlsxWorkbookImportContext workbookContext, FormattingRecord directFormatting, PropertyBagBase propertyBag, long start, long end)
        {
            if (directFormatting.FillId.HasValue)
            {
                IFill fill = workbookContext.StyleSheet.FillTable[directFormatting.FillId.Value];
                if (!(fill is NoneFill))
                {
                    this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.FillProperty, start, end, fill);
                }
            }

            if (directFormatting.NumberFormatId.HasValue)
            {
                CellValueFormat cellValueFormat = workbookContext.StyleSheet.CellValueFormatTable[directFormatting.NumberFormatId.Value];
                this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.FormatProperty, start, end, cellValueFormat);
            }

            if (directFormatting.FontInfoId.HasValue)
            {
                FontInfo fontInfo = workbookContext.StyleSheet.FontInfoTable[directFormatting.FontInfoId.Value];
                this.ImportFontInRange(propertyBag, fontInfo, start, end);
            }

            if (directFormatting.BordersInfoId.HasValue)
            {
                BordersInfo bordersInfo = workbookContext.StyleSheet.BordersInfoTable[directFormatting.BordersInfoId.Value];
                this.ImportBordersInRange(propertyBag, bordersInfo, start, end);
            }

            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.HorizontalAlignmentProperty, start, end, directFormatting.HorizontalAlignment);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.VerticalAlignmentProperty, start, end, directFormatting.VerticalAlignment);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.IndentProperty, start, end, directFormatting.Indent);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.IsWrappedProperty, start, end, directFormatting.WrapText);

            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.IsLockedProperty, start, end, directFormatting.IsLocked);

            if (directFormatting.StyleFormattingRecordId.HasValue)
            {
                StyleInfo? styleInfo = workbookContext.StyleSheet.GetStyleInfoByStyleFormattingRecordId(directFormatting.StyleFormattingRecordId.Value);
                if (styleInfo.HasValue)
                {
                    this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.StyleNameProperty, start, end, styleInfo.Value.Name);
                }
            }
        }

        private void ImportFontInRange(PropertyBagBase propertyBag, FontInfo fontInfo, long start, long end)
        {
            Guard.ThrowExceptionIfNull(fontInfo, "fontInfo");

            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.IsBoldProperty, start, end, fontInfo.Bold);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.IsItalicProperty, start, end, fontInfo.Italic);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.FontSizeProperty, start, end, fontInfo.FontSize);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.FontFamilyProperty, start, end, fontInfo.FontFamily);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.ForeColorProperty, start, end, fontInfo.ForeColor);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.UnderlineProperty, start, end, fontInfo.UnderlineType);
        }

        private void ImportBordersInRange(PropertyBagBase propertyBag, BordersInfo bordersInfo, long start, long end)
        {
            Guard.ThrowExceptionIfNull(bordersInfo, "bordersInfo");

            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.LeftBorderProperty, start, end, bordersInfo.Left);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.TopBorderProperty, start, end, bordersInfo.Top);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.RightBorderProperty, start, end, bordersInfo.Right);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.BottomBorderProperty, start, end, bordersInfo.Bottom);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.DiagonalUpBorderProperty, start, end, bordersInfo.DiagonalUp);
            this.UpdatePropertyInRangeIfNotNull(propertyBag, CellPropertyDefinitions.DiagonalDownBorderProperty, start, end, bordersInfo.DiagonalDown);
        }

        private void UpdatePropertyInRangeIfNotNull<T>(PropertyBagBase propertyBag, IPropertyDefinition<T> propertyDefinition, long start, long end, T? value)
            where T : struct
        {
            if (value.HasValue)
            {
                this.UpdatePropertyInRangeInternal(propertyBag, propertyDefinition, start, end, value.Value);
            }
        }

        private void UpdatePropertyInRangeIfNotNull<T>(PropertyBagBase propertyBag, IPropertyDefinition<T> propertyDefinition, long start, long end, T value)
            where T : class
        {
            if (value != null)
            {
                this.UpdatePropertyInRangeInternal(propertyBag, propertyDefinition, start, end, value);
            }
        }

        private void UpdatePropertyInRangeInternal<T>(PropertyBagBase propertyBag, IPropertyDefinition<T> propertyDefinition, long start, long end, T value)
        {
            propertyBag.GetPropertyValueCollection(propertyDefinition).SetValue(start, end, value);
        }
    }
}