﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Export;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
#endif
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export
{
    internal class XlsxExporter : OpenXmlExporter<XlsxPartsManager, IXlsxWorkbookExportContext>
    {
        protected override XlsxPartsManager CreatePartsManager()
        {
            return new XlsxPartsManager();
        }

        protected override void InitializeParts(XlsxPartsManager partsManager, IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNull(context, "context");

            base.InitializeParts(partsManager, context);

            WorkbookPart workbookPart = new WorkbookPart(partsManager);

            partsManager.RegisterPart(new SharedStringsPart(partsManager));
            partsManager.RegisterPart(new StylesPart(partsManager));

            ThemePart themePart = new ThemePart(partsManager, XlsxPartNames.ThemePartName);
            partsManager.RegisterPart(themePart);
            partsManager.CreateWorkbookRelationship(themePart.Name, OpenXmlRelationshipTypes.ThemeRelationshipType);

            foreach (IXlsxWorksheetExportContext worksheetExportContext in context.WorksheetContexts)
            {
                this.InitializeWorksheetPart(partsManager, context, worksheetExportContext);
            }

#if !SILVERLIGHT
            int chartPartNumber = 1;

            foreach (IXlsxWorksheetExportContext worksheetExportContext in context.WorksheetContexts)
            {
                foreach (FloatingChartShape chartShape in worksheetExportContext.Charts)
                {
                    InitializeChartPart(partsManager, context, chartPartNumber, (ChartShape)chartShape.Shape);
                    chartPartNumber++;
                }
            }
#endif

            partsManager.RegisterPart(workbookPart);
        }

        protected override string CreateResourceName(IResource resource)
        {
            return XlsxHelper.CreateResourceName(resource);
        }

        private void InitializeWorksheetPart(XlsxPartsManager partsManager, IXlsxWorkbookExportContext workbookContext, IXlsxWorksheetExportContext worksheetContext)
        {
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");
            Guard.ThrowExceptionIfNull(workbookContext, "workbookContext");
            Guard.ThrowExceptionIfNull(worksheetContext, "worksheetContext");

            WorksheetPart worksheetPart = new WorksheetPart(partsManager, worksheetContext);
            workbookContext.RegisterWorksheetPart(worksheetContext.Worksheet, worksheetPart);
            partsManager.RegisterPart(worksheetPart);

            if (this.HasShapes(worksheetContext))
            {
                InitializeDrawingPart(partsManager, worksheetContext);
            }
        }

        private void InitializeDrawingPart(XlsxPartsManager partsManager, IXlsxWorksheetExportContext context)
        {
            DrawingPart drawingPart = new DrawingPart(partsManager, context);
            partsManager.RegisterPart(drawingPart);
            context.WorkbookContext.RegisterDrawingPart(context.Worksheet, drawingPart);
        }

#if !SILVERLIGHT
        private void InitializeChartPart(XlsxPartsManager partsManager, IXlsxWorkbookExportContext workbookContext, int chartNumber, ChartShape chart)
        {
            ChartPart chartPart = new ChartPart(partsManager, string.Format(XlsxPartNames.ChartPartName, chartNumber));
            partsManager.RegisterPart(chartPart);
            workbookContext.RegisterChartPartForChart(chart, chartPart);
        }
#endif

        private bool HasShapes(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return context.Shapes.Any();
        }
    }
}