﻿using System;
using System.Windows.Media;
using Telerik.Windows.Documents.Licensing;
using Telerik.Windows.Documents.Spreadsheet.Model;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
{
    internal class LicenseCheck : IDisposable
    {
        private static readonly string licenseSheetName = "License";
        private readonly Workbook workbook;
        private readonly string activeSheetName;
        private bool isDisposed;

        public LicenseCheck(Workbook workbook)
        {
            this.workbook = workbook;

            LicensingMode licensingMode = TelerikLicense.GetLicensingMode();

            if (licensingMode != LicensingMode.Dev)
            {
                this.activeSheetName = workbook.ActiveSheet.Name;

                Worksheet worksheet = workbook.Worksheets.Add();
                worksheet.Name = this.GetAvailableSheetName();

                string message;
                if (licensingMode == LicensingMode.Trial)
                {
                    message = TrialMessages.DocumentProcessingTrialMessage;
                }
                else
                {
                    message = TrialMessages.DocumentProcessingUnlicencedMessage;
                }

                worksheet.Cells[0, 0].SetValue(message);
                worksheet.Cells[0, 0].SetForeColor(new ThemableColor(Colors.Red));

                workbook.ActiveSheet = worksheet;
            }
        }

        public void Dispose()
        {
            if (!this.isDisposed)
            {
                if (!string.IsNullOrEmpty(this.activeSheetName))
                {
                    this.workbook.Worksheets.RemoveAt(this.workbook.Worksheets.Count - 1);
                    this.workbook.ActiveSheet = this.workbook.Sheets[this.activeSheetName];
                }
                this.isDisposed = true;
            }
        }

        private string GetAvailableSheetName()
        {
            string testName = licenseSheetName;
            int index = 1;
            while (this.workbook.Sheets.Contains(testName))
            {
                testName = licenseSheetName + index.ToString();
                index++;
            }

            return testName;
        }
    }
}
