﻿using System.Linq;
using System.Windows.Media;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
#if (WPF || SILVERLIGHT) && !NETCORE
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
#else
using Telerik.Documents.Common.Model;
#endif
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common
{
    internal abstract class ColorElementBase : XlsxElementBase
    {
        private readonly IntOpenXmlAttribute indexed;
        private readonly ConvertedOpenXmlAttribute<UnsignedIntHex> rgb;
        private readonly IntOpenXmlAttribute themeColorId;
        private readonly OpenXmlAttribute<double> tint;

        public ColorElementBase(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.indexed = this.RegisterAttribute(new IntOpenXmlAttribute("indexed"));
            this.rgb = this.RegisterAttribute(new ConvertedOpenXmlAttribute<UnsignedIntHex>("rgb", Converters.UnsignedIntHexConverter));
            this.themeColorId = this.RegisterAttribute(new IntOpenXmlAttribute("theme"));
            this.tint = this.RegisterAttribute<double>("tint", 0.0);
        }

        public int Indexed
        {
            get
            {
                return this.indexed.Value;
            }
            set
            {
                this.indexed.Value = value;
            }
        }

        public UnsignedIntHex Rgb
        {
            get
            {
                return this.rgb.Value;
            }
            set
            {
                this.rgb.Value = value;
            }
        }

        public int ThemeColorId
        {
            get
            {
                return this.themeColorId.Value;
            }
            set
            {
                this.themeColorId.Value = value;
            }
        }

        public double Tint
        {
            get
            {
                return this.tint.Value;
            }
            set
            {
                this.tint.Value = value;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, ThemableColor color)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(color, "border");

            if (color.IsFromTheme)
            {
                this.ThemeColorId = (int)color.ThemeColorType;
                if (color.ColorShadeType.HasValue)
                {
                    this.Tint = context.Theme.ColorScheme.GetTintAndShade(color.ThemeColorType, color.ColorShadeType.Value);
                }
                else if (color.TintAndShade.HasValue)
                {
                    this.Tint = color.TintAndShade.Value;
                }
            }
            else
            {
                this.Rgb = new UnsignedIntHex(color.GetActualValue(context.Theme));
            }
        }

        public ThemableColor CreateThemableColor(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.themeColorId.HasValue)
            {
                ThemeColorType colorType = ThemeColorTypes.GetThemeColorTypeByIndex(this.ThemeColorId);
                Color baseColor = context.Theme.ColorScheme[colorType].Color;
                ColorShadeType? shadeType = ColorsHelper.GetColorShadeTypeForColorAndTint(baseColor, this.Tint);

                if (shadeType.HasValue)
                {
                    return new ThemableColor(colorType, shadeType);
                }

                if (this.tint.HasValue)
                {
                    return new ThemableColor(colorType, this.Tint);
                }

                return new ThemableColor(colorType);
            }
            else if (this.rgb.HasValue)
            {
                return new ThemableColor(this.Rgb.Color);
            }
            else if (this.indexed.HasValue)
            {
                ResourceIndexedTable<Color> indexedColorTable = context.StyleSheet.IndexedColorTable;
                int indexedColorTableCount = indexedColorTable.Count();
                int indexed = this.Indexed;

                if (indexed < 0 || indexed >= indexedColorTableCount)
                {
                    indexed = (indexed % 2 == 0) ? indexedColorTableCount - 2 : indexedColorTableCount - 1;
                }

                return new ThemableColor(context.StyleSheet.IndexedColorTable[indexed]);
            }

            return new ThemableColor(Colors.Black);
        }
    }
}