﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal class AbsoluteAnchorElement : AnchorElementBase
    {
        private readonly OpenXmlChildElement<OffsetElement> pos;
        private readonly OpenXmlChildElement<SizeElement> ext;

        public AbsoluteAnchorElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.pos = this.RegisterChildElement<OffsetElement>(XlsxElementNames.Position);
            this.ext = this.RegisterChildElement<SizeElement>(XlsxElementNames.Extent);
            this.RegisterChildElements();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.AbsoluteAnchorElementName;
            }
        }

        public OffsetElement PositionElement
        {
            get
            {
                return this.pos.Element;
            }
        }

        public SizeElement SizeElement
        {
            get
            {
                return this.ext.Element;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CellIndex fromIndex = new CellIndex(0, 0);
            double fromOffsetX = 0;
            double fromOffsetY = 0;

            if (this.PositionElement != null)
            {
                RadWorksheetLayout worksheetlayout = context.Worksheet.Workbook.GetWorksheetLayout(context.Worksheet, false);
                fromIndex = worksheetlayout.GetCellIndexAndOffsetFromPoint(new Point(this.PositionElement.X, this.PositionElement.Y), out fromOffsetX, out fromOffsetY);

                this.ReleaseElement(this.pos);
            }

            double width = 0;
            double height = 0;

            if (this.SizeElement != null)
            {
                width = this.SizeElement.ExtentWidth;
                height = this.SizeElement.ExtentHeight;
                this.ReleaseElement(this.ext);
            }

            FloatingShapeBase shape = this.CreateShapeAndCopyPropertiesTo(context, fromIndex, fromOffsetX, fromOffsetY);

            if (shape != null)
            {
                shape.Shape.Width = width;
                shape.Shape.Height = height;

                context.Worksheet.Shapes.Add(shape);
            }

            base.OnAfterRead(context);
        }
    }
}
