﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Model.Drawing.Charts;
#endif
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal abstract class AnchorElementBase : WorksheetDrawingElementBase
    {
        private readonly OpenXmlChildElement<PictureElement> picture;
        private readonly OpenXmlChildElement<GraphicFrameElement> graphicFrame;
        private readonly OpenXmlChildElement<ClientDataElement> clientData;

        public AnchorElementBase(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.picture = new OpenXmlChildElement<PictureElement>(XlsxElementNames.Picture);
            this.graphicFrame = new OpenXmlChildElement<GraphicFrameElement>(OpenXmlElementNames.GraphicFrame);
            this.clientData = new OpenXmlChildElement<ClientDataElement>(XlsxElementNames.ClientData);
        }

        public PictureElement PictureElement
        {
            get
            {
                return this.picture.Element;
            }
        }

        public GraphicFrameElement GraphicFrameElement
        {
            get
            {
                return this.graphicFrame.Element;
            }
        }

        public virtual void CopyPropertiesFrom(IXlsxWorksheetExportContext context, FloatingShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            Image image = shape.Shape as Image;
            if (image != null)
            {
                this.CreateElement(this.picture);
                this.PictureElement.CopyPropertiesFrom(context, image);
            }

#if !SILVERLIGHT
            ChartShape chart = shape.Shape as ChartShape;
            if (chart != null)
            {
                this.CreateElement(this.graphicFrame);
                this.GraphicFrameElement.CopyPropertiesFrom(context, chart);
            }
#endif

            this.CreateElement(this.clientData);
        }

        protected FloatingShapeBase CreateShapeAndCopyPropertiesTo(IXlsxWorksheetImportContext context, CellIndex fromIndex, double fromOffsetX, double fromOffsetY)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            FloatingShapeBase shape = null;

            if (this.PictureElement != null)
            {
                FloatingImage image = new FloatingImage(context.Worksheet, fromIndex, fromOffsetX, fromOffsetY);
                this.PictureElement.CopyPropertiesTo(context, image.Image);

                shape = image;
            }
            else if (this.GraphicFrameElement != null)
            {
                ShapeBase coreShape = this.GraphicFrameElement.CreateShapeAndCopyPropertiesTo(context);

                if (coreShape is Image)
                {
                    FloatingImage image = new FloatingImage(context.Worksheet, fromIndex, fromOffsetX, fromOffsetY, coreShape as Image);
                    shape = image;
                }
#if!SILVERLIGHT
                else if (coreShape is ChartShape)
                {
                    FloatingChartShape chartShape = new FloatingChartShape(context.Worksheet, fromIndex, coreShape as ChartShape);
                    chartShape.OffsetX = fromOffsetX;
                    chartShape.OffsetY = fromOffsetY;
                    shape = chartShape;
                }
#endif
            }

            return shape;
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.picture);
            this.ReleaseElement(this.graphicFrame);
            this.ReleaseElement(this.clientData);
        }

        protected void RegisterChildElements()
        {
            this.RegisterChildElement(this.picture);
            this.RegisterChildElement(this.graphicFrame);
            this.RegisterChildElement(this.clientData);
        }
    }
}
