﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal class BlipFillElement : XlsxElementBase
    {
        private readonly OpenXmlChildElement<BlipElement> blip;
        private readonly OpenXmlChildElement<StretchElement> stretch;

        public BlipFillElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.blip = this.RegisterChildElement<BlipElement>(OpenXmlElementNames.BlipElementName);
            this.stretch = this.RegisterChildElement<StretchElement>(OpenXmlElementNames.StretchElementName);
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.SpreadsheetDrawingMLNamespace;
            }
        }

        public override string ElementName
        {
            get 
            {
                return XlsxElementNames.BlipFill;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public BlipElement BlipElement
        {
            get
            {
                return this.blip.Element;
            }
            set
            {
                this.blip.Element = value;
            }
        }

        public StretchElement StretchElement
        {
            get
            {
                return this.stretch.Element;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorksheetExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.blip);

            ImageSource resource = image.ImageSource;
            string resourceName = XlsxHelper.CreateResourceName(resource);
            this.BlipElement.RelationshipId = this.PartsManager.CreateRelationship(this.Part.Name, resourceName, OpenXmlRelationshipTypes.GetRelationshipTypeByExtension(resource.Extension));

            this.CreateElement(this.stretch);
            this.StretchElement.CopyPropertiesFrom(context, image);
        }

        public void CopyPropertiesTo(IXlsxWorksheetImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            if (this.BlipElement != null)
            {
                string target = this.PartsManager.GetRelationshipTarget(this.Part.Name, this.BlipElement.RelationshipId);
                string resourceKey = this.Part.GetResourceName(target);

                image.ImageSource = (ImageSource)context.GetResourceByResourceKey(resourceKey);
                this.ReleaseElement(this.blip);
            }
        }
    }
}
