﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Maths;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal class OneCellAnchorElement : AnchorElementBase
    {
        private readonly OpenXmlChildElement<MarkerElement> from;
        private readonly OpenXmlChildElement<SizeElement> ext;

        public OneCellAnchorElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.from = this.RegisterChildElement<MarkerElement>(XlsxElementNames.From);
            this.ext = this.RegisterChildElement<SizeElement>(XlsxElementNames.Extent);
            this.RegisterChildElements();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.TwoCellAnchorElementName;
            }
        }

        public MarkerElement FromElement
        {
            get
            {
                return this.from.Element;
            }
        }

        public SizeElement SizeElement
        {
            get
            {
                return this.ext.Element;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CellIndex fromIndex = new CellIndex(0, 0);
            double fromOffsetX = 0;
            double fromOffsetY = 0;
            
            if (this.FromElement != null)
            {
                this.FromElement.CopyPropertiesTo(context, out fromIndex, out fromOffsetX, out fromOffsetY);
            }

            FloatingShapeBase shape = this.CreateShapeAndCopyPropertiesTo(context, fromIndex, fromOffsetX, fromOffsetY);

            if (shape != null)
            {
                if (this.SizeElement != null)
                {
                    shape.Width = this.SizeElement.ExtentWidth;
                    shape.Height = this.SizeElement.ExtentHeight;
                }

                this.AdjustShapeForRotation(shape);
                context.Worksheet.Shapes.Add(shape);
            }

            this.ReleaseElement(this.from);
            this.ReleaseElement(this.ext);

            base.OnAfterRead(context);
        }

        private void AdjustShapeForRotation(FloatingShapeBase shape)
        {
            double restrictedRotationAngle = SpreadsheetHelper.RestrictRotationAngle(shape.RotationAngle);

            bool rotationAngleRequiresCellIndexChange = (restrictedRotationAngle >= 45 && restrictedRotationAngle < 135)
              || (restrictedRotationAngle > 225 && restrictedRotationAngle < 315);

            if (rotationAngleRequiresCellIndexChange)
            {
                RadWorksheetLayout worksheetLayout = shape.Worksheet.Workbook.GetWorksheetLayout(shape.Worksheet, false);
                Point topLeftPoint = worksheetLayout.GetPointFromCellIndexAndOffset(shape.CellIndex, shape.OffsetY, shape.OffsetY);

                double width = shape.Width;
                double height = shape.Height;

                Matrix m = Matrix.Identity;
                m = m.RotateMatrixAt(-90, topLeftPoint.X + (width / 2), topLeftPoint.Y + (height / 2));

                Point rotatedPoint = m.Transform(topLeftPoint);
                rotatedPoint.Y -= width;

                CellIndex newCellIndex;
                double newOffsetX;
                double newOffsetY;
                newCellIndex = worksheetLayout.GetCellIndexAndOffsetFromPoint(rotatedPoint, out newOffsetX, out newOffsetY);

                shape.CellIndex = newCellIndex;
                shape.OffsetX = newOffsetX;
                shape.OffsetY = newOffsetY;
                shape.Shape.Width = height;
                shape.Shape.Height = width;
            }
        }
    }
}
