﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.Model.Drawing.Shapes;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal class PictureElement : WorksheetDrawingElementBase
    {
        private readonly OpenXmlChildElement<NonVisualPicturePropertiesElement> nonVisualPictureProperties;
        private readonly OpenXmlChildElement<BlipFillElement> blipFill;
        private readonly OpenXmlChildElement<ShapePropertiesElement> shapeProperties;

        public PictureElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.nonVisualPictureProperties = this.RegisterChildElement<NonVisualPicturePropertiesElement>(OpenXmlElementNames.NonVisualPicturePropertiesElementName);
            this.blipFill = this.RegisterChildElement<BlipFillElement>(XlsxElementNames.BlipFill);
            this.shapeProperties = this.RegisterChildElement<ShapePropertiesElement>(OpenXmlElementNames.ShapePropertiesElementName);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.Picture;
            }
        }

        public NonVisualPicturePropertiesElement NonVisualPicturePropertiesElement
        {
            get
            {
                return this.nonVisualPictureProperties.Element;
            }
            set
            {
                this.nonVisualPictureProperties.Element = value;
            }
        }

        public BlipFillElement BlipFillElement
        {
            get
            {
                return this.blipFill.Element;
            }
            set
            {
                this.blipFill.Element = value;
            }
        }

        public ShapePropertiesElement ShapePropertiesElement
        {
            get
            {
                return this.shapeProperties.Element;
            }
            set
            {
                this.shapeProperties.Element = value;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorksheetExportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(image, "image");

            this.CreateElement(this.nonVisualPictureProperties);
            this.NonVisualPicturePropertiesElement.CopyPropertiesFrom(context, image);

            this.CreateElement(this.blipFill);
            this.BlipFillElement.CopyPropertiesFrom(context, image);

            this.CreateElement(this.shapeProperties);
            this.ShapePropertiesElement.CopyPropertiesFrom(context, image);
        }

        public void CopyPropertiesTo(IXlsxWorksheetImportContext context, Image image)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.BlipFillElement != null)
            {
                this.BlipFillElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.blipFill);
            }

            if (this.NonVisualPicturePropertiesElement != null)
            {
                this.NonVisualPicturePropertiesElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.nonVisualPictureProperties);
            }

            if (this.ShapePropertiesElement != null)
            {
                this.ShapePropertiesElement.CopyPropertiesTo(context, image);
                this.ReleaseElement(this.shapeProperties);
            }
        }
    }
}
