﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
#if !SILVERLIGHT
using Telerik.Windows.Documents.Model.Drawing.Charts;
#endif
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing
{
    internal class TwoCellAnchorElement : AnchorElementBase
    {
        private readonly OpenXmlChildElement<MarkerElement> from;
        private readonly OpenXmlChildElement<MarkerElement> to;
        private readonly OpenXmlAttribute<string> editAs;

        public TwoCellAnchorElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.editAs = this.RegisterAttribute<string>("editAs", "oneCell", isRequired: false);

            this.from = this.RegisterChildElement<MarkerElement>(XlsxElementNames.From);
            this.to = this.RegisterChildElement<MarkerElement>(XlsxElementNames.To);
            this.RegisterChildElements();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.TwoCellAnchorElementName;
            }
        }

        public MarkerElement FromElement
        {
            get
            {
                return this.from.Element;
            }
        }

        public MarkerElement ToElement
        {
            get
            {
                return this.to.Element;
            }
        }

        public string EditAs
        {
            get
            {
                return this.editAs.Value;
            }
            set
            {
                this.editAs.Value = value;
            }
        }

        public override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, FloatingShapeBase shape)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(shape, "shape");

            CellIndex fromCellIndex = shape.CellIndex;
            double fromOffsetX = shape.OffsetX;
            double fromOffsetY = shape.OffsetY;

            this.CreateElement(this.from);
            this.FromElement.CopyPropertiesFrom(context, fromCellIndex, fromOffsetX, fromOffsetY);

            CellIndex toCellIndex;
            double toOffsetX;
            double toOffsetY;

            this.GetToCellIndex(context.Worksheet, shape, out toCellIndex, out toOffsetX, out toOffsetY);

            this.CreateElement(this.to);
            this.ToElement.CopyPropertiesFrom(context, toCellIndex, toOffsetX, toOffsetY);

            this.EditAs = "oneCell";

            base.CopyPropertiesFrom(context, shape);
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CellIndex fromIndex;
            double fromOffsetX;
            double fromOffsetY;

            CellIndex toIndex;
            double toOffsetX;
            double toOffsetY;

            this.FromElement.CopyPropertiesTo(context, out fromIndex, out fromOffsetX, out fromOffsetY);
            this.ToElement.CopyPropertiesTo(context, out toIndex, out toOffsetX, out toOffsetY);

            FloatingShapeBase shape = this.CreateShapeAndCopyPropertiesTo(context, fromIndex, fromOffsetX, fromOffsetY);

            // This is in case the TransformElement is not declared or its inner ExtentElement is missing.
            if (shape != null && shape.Shape.SizeInternal.IsEmpty)
            {
                TwoCellAnchorInfo info = new TwoCellAnchorInfo()
                {
                    FromIndex = fromIndex,
                    ToIndex = toIndex,
                    FromOffsetX = fromOffsetX,
                    FromOffsetY = fromOffsetY,
                    ToOffsetX = toOffsetX,
                    ToOffsetY = toOffsetY,
                };

                context.ShapesAnchorsWaitingSize.Add(shape, info);
            }

            if (shape != null)
            {
                context.Worksheet.Shapes.Add(shape);
            }

            this.ReleaseElement(this.from);
            this.ReleaseElement(this.to);

            base.OnAfterRead(context);
        }

        private void GetToCellIndex(Worksheet worksheet, FloatingShapeBase shape, out CellIndex toCellIndex, out double toOffsetX, out double toOffsetY)
        {
            RadWorksheetLayout worksheetLayout = worksheet.Workbook.GetWorksheetLayout(worksheet, false);

            Point point = worksheetLayout.GetTopLeftPointFromCellIndex(shape.CellIndex);

            if (shape.DoesRotationAngleRequireCellIndexChange())
            {
                point.X += shape.OffsetX + shape.Height;
                point.Y += shape.OffsetY + shape.Width;
            }
            else
            {
                point.X += shape.OffsetX + shape.Width;
                point.Y += shape.OffsetY + shape.Height;
            }

            toCellIndex = worksheetLayout.GetCellIndexAndOffsetFromPoint(point, out toOffsetX, out toOffsetY);
        }
    }
}
