﻿using System;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.SharedStrings
{
    internal class StringItemElement : SharedStringElementBase
    {
        private readonly OpenXmlChildElement<TextElement> text;
        private StringBuilder builder;

        public StringItemElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.text = this.RegisterChildElement<TextElement>(XlsxElementNames.TextElementName, XlsxElementUniquePoolIds.TextElementUniquePoolId);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.StringItemElementName;
            }
        }

        public TextElement TextElement
        {
            get
            {
                return this.text.Element;
            }
            set
            {
                this.text.Element = value;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, SharedString sharedString)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(sharedString, "sharedString");

            switch (sharedString.Type)
            {
                case SharedStringType.Text:
                    this.CreateElement(this.text);
                    this.TextElement.CopyPropertiesFrom(context, (TextSharedString)sharedString);

                    break;
            }
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.TextElement != null)
            {
                string sharedStringValue = this.TextElement.InnerText ?? string.Empty;
                this.ReleaseElement(this.text);

                context.SharedStrings.Add(new TextSharedString(sharedStringValue));
            }
            else if (this.builder != null)
            {
                string sharedStringValue = this.builder.ToString();

                context.SharedStrings.Add(new RichTextRunSharedString(sharedStringValue));

                this.builder = null;
            }
            else
            {
                context.SharedStrings.Add(new TextSharedString(string.Empty));
            }
        }

        protected override void OnAfterReadChildElement(IXlsxWorkbookImportContext context, OpenXmlElementBase element)
        {
            RichTextRunElement richTextRunElement = element as RichTextRunElement;

            if (richTextRunElement == null)
            {
                return;
            }

            if (this.builder == null)
            {
                this.builder = new StringBuilder();
            }

            this.builder.Append(richTextRunElement.TextElement.InnerText);
        }
    }
}