﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class AlignmentElement : StyleSheetElementBase
    {
        private readonly OpenXmlAttribute<string> horizontalAlignment;
        private readonly OpenXmlAttribute<string> verticalAlignment;
        private readonly BoolOpenXmlAttribute wrapText;
        private readonly IntOpenXmlAttribute indent;

        public AlignmentElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.horizontalAlignment = this.RegisterAttribute(new OpenXmlAttribute<string>("horizontal"));
            this.verticalAlignment = this.RegisterAttribute(new OpenXmlAttribute<string>("vertical"));
            this.indent = this.RegisterAttribute(new IntOpenXmlAttribute("indent"));
            this.wrapText = this.RegisterAttribute(new BoolOpenXmlAttribute("wrapText"));
        }

        public string HorizontalAlignment
        {
            get
            {
                return this.horizontalAlignment.Value;
            }
            set
            {
                this.horizontalAlignment.Value = value;
            }
        }

        public string VerticalAlignment
        {
            get
            {
                return this.verticalAlignment.Value;
            }
            set
            {
                this.verticalAlignment.Value = value;
            }
        }

        public bool WrapText
        {
            get
            {
                return this.wrapText.Value;
            }
            set
            {
                this.wrapText.Value = value;
            }
        }

        public int Indent
        {
            get
            {
                return this.indent.Value;
            }
            set
            {
                this.indent.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.AlignmentElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, FormattingRecord format)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(format, "format");

            if (format.HorizontalAlignment.HasValue)
            {
                this.HorizontalAlignment = HorizontalAlignments.GetHorizontalAlignmentName(format.HorizontalAlignment.Value);
            }

            if (format.VerticalAlignment.HasValue)
            {
                this.VerticalAlignment = VerticalAlignments.GetVerticalAlignmentName(format.VerticalAlignment.Value);
            }

            if (format.Indent.HasValue)
            {
                this.Indent = format.Indent.Value;
            }

            if (format.WrapText.HasValue)
            {
                this.WrapText = format.WrapText.Value;
            }
        }

        public void ApplyAllignment(IXlsxWorkbookImportContext context, ref FormattingRecord format)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(format, "format");

            if (this.horizontalAlignment.HasValue)
            {
                format.HorizontalAlignment = HorizontalAlignments.GetHorizontalAlignment(this.HorizontalAlignment);
            }

            if (this.verticalAlignment.HasValue)
            {
                format.VerticalAlignment = VerticalAlignments.GetVerticalAlignment(this.VerticalAlignment);
            }

            if (this.indent.HasValue)
            {
                format.Indent = this.Indent;
            }

            if (this.wrapText.HasValue)
            {
                format.WrapText = this.WrapText;
            }
        }
    }
}