﻿using System.Windows.Media;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal abstract class BorderTypeElementBase : StyleSheetElementBase
    {
        private readonly OpenXmlAttribute<string> style;
        private readonly OpenXmlChildElement<ColorElement> color;

        public BorderTypeElementBase(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.style = this.RegisterAttribute<string>("style", BorderStyles.None);
            this.color = this.RegisterChildElement<ColorElement>(XlsxElementNames.ColorElementName);
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        public string Style
        {
            get
            {
                return this.style.Value;
            }
            set
            {
                this.style.Value = value;
            }
        }

        public ColorElement ColorElement
        {
            get
            {
                return this.color.Element;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, CellBorder border)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(border, "border");

            this.Style = BorderStyles.GetBorderStyleName(border.Style);

            if (border.Style != CellBorderStyle.None && border.Color != null)
            {
                this.CreateElement(this.color);
                this.ColorElement.CopyPropertiesFrom(context, border.Color);
            }
        }

        public CellBorder CreateCellBorder(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CellBorder cellBorder = null;
            if (this.Style != BorderStyles.None)
            {
                ThemableColor themableColor = this.ColorElement != null ?
                    this.ColorElement.CreateThemableColor(context) :
                    new ThemableColor(Colors.Black);

                cellBorder = new CellBorder(BorderStyles.GetBorderStyle(this.Style), themableColor);
            }

            this.ReleaseElement(this.color);

            return cellBorder;
        }
    }
}