﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class CellStyleElement : StyleSheetElementBase
    {
        private readonly IntOpenXmlAttribute builtinId;
        private readonly BoolOpenXmlAttribute customBuiltin;
        private readonly OpenXmlAttribute<string> name;
        private readonly IntOpenXmlAttribute formatId;

        public CellStyleElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.name = this.RegisterAttribute(new OpenXmlAttribute<string>("name"));
            this.formatId = this.RegisterAttribute(new IntOpenXmlAttribute("xfId", true));
            this.builtinId = this.RegisterAttribute(new IntOpenXmlAttribute("builtinId"));
            this.customBuiltin = this.RegisterAttribute(new BoolOpenXmlAttribute("customBuiltin"));
        }

        public int BuiltinId
        {
            get
            {
                return this.builtinId.Value;
            }
            set
            {
                this.builtinId.Value = value;
            }
        }

        public bool CustomBuiltin
        {
            get
            {
                return this.customBuiltin.Value;
            }
            set
            {
                this.customBuiltin.Value = value;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public int FormatId
        {
            get
            {
                return this.formatId.Value;
            }
            set
            {
                this.formatId.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.CellStyleElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, StyleInfo style)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(style, "style");

            this.Name = style.Name;
            this.FormatId = style.FormattingRecordId;

            if (style.BuiltInId.HasValue)
            {
                this.BuiltinId = style.BuiltInId.Value;
            }
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            StyleInfo styleInfo = new StyleInfo();
            styleInfo.Name = this.Name;
            styleInfo.FormattingRecordId = this.FormatId;

            if (this.builtinId.HasValue)
            {
                styleInfo.BuiltInId = this.BuiltinId;
            }

            context.StyleSheet.StyleInfoTable.Add(styleInfo);
        }
    }
}