﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class CellStylesElement : StyleSheetElementBase
    {
        private readonly OpenXmlCountAttribute count;

        public CellStylesElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterCountAttribute();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.CellStylesElementName;
            }
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.StyleSheet.StyleInfoTable.Count();
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            
            foreach (StyleInfo style in context.StyleSheet.StyleInfoTable)
            {
                CellStyleElement cellStyleElement = this.CreateElement<CellStyleElement>(XlsxElementNames.CellStyleElementName);
                cellStyleElement.CopyPropertiesFrom(context, style);

                yield return cellStyleElement;
            }
        }
    }
}