﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class DifferentialFormatElement : StyleSheetElementBase
    {
        private readonly OpenXmlChildElement<FontElement> font;
        private readonly OpenXmlChildElement<FillElement> fill;

        public DifferentialFormatElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.font = this.RegisterChildElement<FontElement>(XlsxElementNames.FontElementName);
            this.fill = this.RegisterChildElement<FillElement>(XlsxElementNames.FillElementName);
        }

        public FontElement Font
        {
            get
            {
                return this.font.Element;
            }
            set
            {
                this.font.Element = value;
            }
        }

        public FillElement Fill
        {
            get
            {
                return this.fill.Element;
            }
            set
            {
                this.fill.Element = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DifferentialFormatElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, DifferentialFormatInfo info)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(info, "info");

            if (info.FontInfo != null)
            {
                this.CreateElement(this.font);
                this.Font.CopyPropertiesFrom(context, (FontInfo)info.FontInfo);
            }

            if (info.Fill != null)
            {
                this.CreateElement(this.fill);
                this.Fill.CopyPropertiesFrom(context, info.Fill);
            }
        }

        protected override void OnAfterRead(Import.IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.Font != null)
            {
                FontInfo info = (FontInfo)this.Font.FontInfo;
                context.DifferentialFormatsContext.RegisterFont(info);
                this.ReleaseElement(this.font);
            }

            if (this.Fill != null)
            {
                IFill fill = this.Fill.Fill;
                context.DifferentialFormatsContext.RegisterFill(fill);
                this.ReleaseElement(this.fill);
            }
        }
    }
}