﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class DifferentialFormatsElement : StyleSheetElementBase
    {
        private readonly OpenXmlCountAttribute count;

        public DifferentialFormatsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterCountAttribute();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DifferentialFormatsElementName;
            }
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        protected override bool ShouldExport(IXlsxWorkbookExportContext context)
        {
            return base.ShouldExport(context) || context.DifferentialFormatsContext.Count > 0;
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.DifferentialFormatsContext.Count;
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (DifferentialFormatInfo info in context.DifferentialFormatsContext.GetRegisteredDifferentialFormats())
            {
                DifferentialFormatElement formatElement = this.CreateElement<DifferentialFormatElement>(XlsxElementNames.DifferentialFormatElementName);
                formatElement.CopyPropertiesFrom(context, info);

                yield return formatElement;
            }
        }
    }
}