﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class FillElement : StyleSheetElementBase
    {
        private readonly OpenXmlChildElement<GradientFillElement> gradientFill;
        private readonly OpenXmlChildElement<PatternFillElement> patternFill;
        private IFill fill;

        public FillElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.gradientFill = this.RegisterChildElement<GradientFillElement>(XlsxElementNames.GradientFillElementName);
            this.patternFill = this.RegisterChildElement<PatternFillElement>(XlsxElementNames.PatternFillElementName);
        }

        public IFill Fill
        {
            get
            {
                return this.fill;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FillElementName;
            }
        }

        public GradientFillElement GradientFillElement
        {
            get
            {
                return this.gradientFill.Element;
            }
            set
            {
                this.gradientFill.Element = value;
            }
        }

        public PatternFillElement PatternFillElement
        {
            get
            {
                return this.patternFill.Element;
            }
            set
            {
                this.patternFill.Element = value;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, IFill fill)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(fill, "fill");
            
            GradientFill gradientFill = fill as GradientFill;
            PatternFill patternFill = fill as PatternFill;
            NoneFill noneFill = fill as NoneFill;

            if (gradientFill != null)
            {
                this.CreateElement(this.gradientFill);
                this.GradientFillElement.CopyPropertiesFrom(context, gradientFill);
            }

            if (patternFill != null)
            {
                this.CreateElement(this.patternFill);
                this.PatternFillElement.CopyPropertiesFrom(context, patternFill);
            }

            if (noneFill != null)
            {
                this.CreateElement(this.patternFill);
                this.PatternFillElement.CopyPropertiesFrom(context, noneFill);
            }
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.fill = this.GetFill(context);

            this.ReleaseElement(this.gradientFill);
            this.ReleaseElement(this.patternFill);
        }

        protected override void ClearOverride()
        {
            this.fill = null;
        }

        private IFill GetFill(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            IFill fill = NoneFill.Instance;
            if (this.GradientFillElement != null)
            {
                fill = this.GradientFillElement.CreateFill(context);
            }

            if (this.PatternFillElement != null)
            {
                fill = this.PatternFillElement.CreateFill(context);
            }

            return fill;
        }
    }
}