﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class FillsElement : StyleSheetElementBase
    {
        private readonly OpenXmlCountAttribute count;

        public FillsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterCountAttribute();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FillsElementName;
            }
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.StyleSheet.FillTable.Count();
        }

        protected override void OnAfterReadChildElement(Import.IXlsxWorkbookImportContext context, OpenXmlElementBase childElement)
        {
            context.StyleSheet.FillTable.Add(((FillElement)childElement).Fill);
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (IFill fill in context.StyleSheet.FillTable)
            {
                FillElement fillElement = this.CreateElement<FillElement>(XlsxElementNames.FillElementName);
                fillElement.CopyPropertiesFrom(context, fill);

                yield return fillElement;
            }
        }
    }
}