﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class FontsElement : StyleSheetElementBase
    {
        private readonly OpenXmlCountAttribute count;

        public FontsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterCountAttribute();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FontsElementName;
            }
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.StyleSheet.FontInfoTable.Count();
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (FontInfo font in context.StyleSheet.FontInfoTable)
            {
                FontElement fontElement = this.CreateElement<FontElement>(XlsxElementNames.FontElementName);
                fontElement.CopyPropertiesFrom(context, font);

                yield return fontElement;
            }
        }

        protected override void OnAfterReadChildElement(Import.IXlsxWorkbookImportContext context, OpenXmlElementBase childElement)
        {
            FontElement fontElement = (FontElement)childElement;
            FontInfo fontInfo = (FontInfo)fontElement.FontInfo;
            context.StyleSheet.FontInfoTable.Add(fontInfo);

            base.OnAfterReadChildElement(context, childElement);
        }
    }
}