﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class NumberFormatsElement : StyleSheetElementBase
    {
        private readonly OpenXmlCountAttribute count;

        public NumberFormatsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterCountAttribute();
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.NumberFormatsElementName;
            }
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.StyleSheet.CellValueFormatTable.Where(f => !NumberFormatTypes.IsBuiltInFormat(f.FormatString)).Count();
        }

        protected override IEnumerable<OpenXmlElementBase> EnumerateChildElements(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (CellValueFormat cellValueFormat in context.StyleSheet.CellValueFormatTable.Where(f => !NumberFormatTypes.IsBuiltInFormat(f.FormatString)))
            {
                NumberFormatElement numberFormatElement = this.CreateElement<NumberFormatElement>(XlsxElementUniquePoolIds.NumberFormatElementUniquePoolId);
                numberFormatElement.CopyPropertiesFrom(context, cellValueFormat);

                yield return numberFormatElement;
            }
        }

        protected override void OnAfterReadChildElement(Import.IXlsxWorkbookImportContext context, OpenXmlElementBase childElement)
        {
            NumberFormatElement numberFormatElement = (NumberFormatElement)childElement;
            CellValueFormat format = new CellValueFormat(numberFormatElement.FormatCode, false);
            
            if (numberFormatElement.HasFormatId)
            {
                context.StyleSheet.CellValueFormatTable[numberFormatElement.FormatId] = format;
            }
            else
            {
                context.StyleSheet.CellValueFormatTable.Add(format);
            }
        }
    }
}