﻿using System.Windows.Media;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
#if NETCORE || NETSTANDARD
using Telerik.Documents.Common.Model;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class PatternFillElement : StyleSheetElementBase
    {
        private readonly OpenXmlAttribute<string> patternType;

        private readonly OpenXmlChildElement<BackgroundColorElement> backgroundColor;
        private readonly OpenXmlChildElement<ForegroundColorElement> foregroundColor;

        public PatternFillElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.patternType = this.RegisterAttribute<string>("patternType", PatternTypes.None);

            this.foregroundColor = this.RegisterChildElement<ForegroundColorElement>(XlsxElementNames.ForegroundColorElementName);
            this.backgroundColor = this.RegisterChildElement<BackgroundColorElement>(XlsxElementNames.BackgroundColorElementName);
        }

        public string PatternType
        {
            get
            {
                return this.patternType.Value;
            }
            set
            {
                this.patternType.Value = value;
            }
        }

        public BackgroundColorElement BackgroundColorElement
        {
            get
            {
                return this.backgroundColor.Element;
            }
            set
            {
                this.backgroundColor.Element = value;
            }
        }

        public ForegroundColorElement ForegroundColorElement
        {
            get
            {
                return this.foregroundColor.Element;
            }
            set
            {
                this.foregroundColor.Element = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PatternFillElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, NoneFill fill)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(fill, "fill");

            this.PatternType = PatternTypes.None;
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, PatternFill fill)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(fill, "fill");

            this.PatternType = PatternTypes.GetPatternTypeName(fill.PatternType);

            ThemableColor backgroundColor = fill.BackgroundColor;
            ThemableColor patternColor = fill.PatternColor;

            if (backgroundColor != null)
            {
                this.CreateElement(this.backgroundColor);
                this.BackgroundColorElement.CopyPropertiesFrom(context, backgroundColor);
            }

            if (patternColor != null)
            {
                this.CreateElement(this.foregroundColor);
                this.ForegroundColorElement.CopyPropertiesFrom(context, patternColor);
            }
        }

        public IFill CreateFill(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            if (this.PatternType == PatternTypes.None)
            {
                return NoneFill.Instance;
            }

            Spreadsheet.Model.PatternType patternType = PatternTypes.GetPatternType(this.PatternType);
            ThemableColor backgroundColor = null;
            ThemableColor patternColor = null;

            if (this.BackgroundColorElement != null)
            {
                backgroundColor = this.BackgroundColorElement.CreateThemableColor(context);
                this.ReleaseElement(this.backgroundColor);
            }
            else
            {
                backgroundColor = new ThemableColor(Colors.Transparent);
            }

            if (this.ForegroundColorElement != null)
            {
                patternColor = this.ForegroundColorElement.CreateThemableColor(context);
                this.ReleaseElement(this.foregroundColor);
            }
            else
            {
                patternColor = new ThemableColor(Colors.Black);
            }

            return new PatternFill(patternType, patternColor, backgroundColor);
        }
    }
}