﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class ProtectionElement : StyleSheetElementBase
    {
        private readonly BoolOpenXmlAttribute locked;

        public ProtectionElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.locked = this.RegisterAttribute(new BoolOpenXmlAttribute("locked"));
        }

        public bool Locked
        {
            get
            {
                return this.locked.Value;
            }
            set
            {
                this.locked.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.ProtectionElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, FormattingRecord format)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(format, "format");

            if (format.IsLocked.HasValue)
            {
                this.Locked = format.IsLocked.Value;
            }
        }

        public void ApplyProtection(IXlsxWorkbookImportContext context, ref FormattingRecord format)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(format, "format");

            if (this.locked.HasValue)
            {
                format.IsLocked = this.Locked;
            }
        }
    }
}