﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class StopElement : StyleSheetElementBase
    {
        private readonly OpenXmlAttribute<double> position;
        private readonly OpenXmlChildElement<ColorElement> color;

        public StopElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.position = this.RegisterAttribute<double>("position", true);

            this.color = this.RegisterChildElement<ColorElement>(XlsxElementNames.ColorElementName);
        }

        public double Position
        {
            get
            {
                return this.position.Value;
            }
            set
            {
                this.position.Value = value;
            }
        }

        public ColorElement ColorElement
        {
            get
            {
                return this.color.Element;
            }
            set
            {
                this.color.Element = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.StopElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, GradientStop stop)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(stop, "stop");

            this.Position = stop.Position;

            this.CreateElement(this.color);
            this.ColorElement.CopyPropertiesFrom(context, stop.ThemableColor);
        }

        public GradientStop ToGradientStop(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            GradientStop result = null;
            if (this.ColorElement != null)
            {
                result = new GradientStop(this.Position, this.ColorElement.CreateThemableColor(context));
                this.ReleaseElement(this.color);
            }

            return result;
        }
    }
}