using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet
{
    internal class StyleSheetElement : XlsxPartRootElementBase
    {
        private readonly OpenXmlChildElement<BordersElement> borders;
        private readonly OpenXmlChildElement<CellStylesElement> cellStyles;
        private readonly OpenXmlChildElement<CellStyleFormatsElement> cellStyleFormats;
        private readonly OpenXmlChildElement<CellFormatsElement> cellFormats;
        private readonly OpenXmlChildElement<ColorsElement> colors;
        private readonly OpenXmlChildElement<FillsElement> fills;
        private readonly OpenXmlChildElement<FontsElement> fonts;
        private readonly OpenXmlChildElement<NumberFormatsElement> numberFormats;
        private readonly OpenXmlChildElement<DifferentialFormatsElement> differentialFormats;
        
        public StyleSheetElement(XlsxPartsManager partsManager, OpenXmlPartBase part) 
            : base(partsManager, part)
        {
            this.numberFormats = this.RegisterChildElement<NumberFormatsElement>(XlsxElementNames.NumberFormatsElementName);
            this.fonts = this.fonts = this.RegisterChildElement<FontsElement>(XlsxElementNames.FontsElementName);
            this.fills = this.RegisterChildElement<FillsElement>(XlsxElementNames.FillsElementName);
            this.borders = this.RegisterChildElement<BordersElement>(XlsxElementNames.BordersElementName);
            this.cellStyleFormats = this.RegisterChildElement<CellStyleFormatsElement>(XlsxElementNames.CellStyleFormatsElementName);
            this.cellFormats = this.RegisterChildElement<CellFormatsElement>(XlsxElementNames.CellFormatsElementName);
            this.cellStyles = this.RegisterChildElement<CellStylesElement>(XlsxElementNames.CellStylesElementName);
            this.differentialFormats = this.RegisterChildElement<DifferentialFormatsElement>(XlsxElementNames.DifferentialFormatsElementName);
            this.colors = this.RegisterChildElement<ColorsElement>(XlsxElementNames.ColorsElementName);
        }   

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.SpreadsheetMLNamespace;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.StyleSheetElementName;
            }
        }

        protected override void OnBeforeWrite(Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export.IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.numberFormats);
            this.CreateElement(this.fonts);
            this.CreateElement(this.fills);
            this.CreateElement(this.borders);
            this.CreateElement(this.cellStyleFormats);
            this.CreateElement(this.cellFormats);
            this.CreateElement(this.cellStyles);
            this.CreateElement(this.differentialFormats);
            this.CreateElement(this.colors);
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            this.ReleaseElement(this.numberFormats);
            this.ReleaseElement(this.fonts);
            this.ReleaseElement(this.fills);
            this.ReleaseElement(this.borders);
            this.ReleaseElement(this.cellStyleFormats);
            this.ReleaseElement(this.cellFormats);
            this.ReleaseElement(this.cellStyles);
            this.ReleaseElement(this.differentialFormats);
            this.ReleaseElement(this.colors);

            context.ImportStyles();
        }
    }
}