﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Workbooks
{
    internal class DefinedNameElement : WorkbookElementBase
    {
        private readonly OpenXmlAttribute<string> name;
        private readonly IntOpenXmlAttribute localSheetId;
        private readonly OpenXmlAttribute<string> comment;
        private readonly BoolOpenXmlAttribute hidden;

        public DefinedNameElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.name = this.RegisterAttribute<string>("name", true);
            this.localSheetId = this.RegisterAttribute(new IntOpenXmlAttribute("localSheetId", -1, false));
            this.comment = this.RegisterAttribute<string>("comment", string.Empty, false);
            this.hidden = this.RegisterAttribute(new BoolOpenXmlAttribute("hidden"));
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public int LocalSheetId
        {
            get
            {
                return this.localSheetId.Value;
            }
            set
            {
                this.localSheetId.Value = value;
            }
        }

        public string Comment
        {
            get
            {
                return this.comment.Value;
            }
            set
            {
                this.comment.Value = value;
            }
        }

        public bool Hidden
        {
            get
            {
                return this.hidden.Value;
            }
            set
            {
                this.hidden.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DefinedNameElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, DefinedNameInfo nameInfo)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(nameInfo, "nameInfo");

            this.Name = nameInfo.Name;

            if (nameInfo.Value.Length > 1 && SpreadsheetCultureHelper.IsCharEqualTo(nameInfo.Value[0], SpreadsheetCultureHelper.Equal))
            {
                this.InnerText = nameInfo.Value.Substring(1);
            }
            else
            {
                this.InnerText = nameInfo.Value;
            }

            if (nameInfo.LocalSheetId != -1)
            {
                this.LocalSheetId = nameInfo.LocalSheetId;
            }

            if (!string.IsNullOrEmpty(nameInfo.Comment))
            {
                this.Comment = nameInfo.Comment;
            }

            this.Hidden = nameInfo.Hidden;
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            DefinedNameInfo definedNameInfo = new DefinedNameInfo(this.Name, 
                SpreadsheetCultureHelper.PrepareFormulaValue(this.InnerText),
                this.LocalSheetId,
                this.Comment,
                this.Hidden);

            context.DefinedNames.Add(definedNameInfo);
        }
    }
}
