﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Workbooks
{
    internal class SheetElement : WorkbookElementBase
    {
        private readonly IntOpenXmlAttribute sheetId;
        private readonly OpenXmlAttribute<string> name;
        private readonly OpenXmlAttribute<string> relationId;
        private readonly MappedOpenXmlAttribute<SheetVisibility> state;

        public SheetElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.sheetId = this.RegisterAttribute(new IntOpenXmlAttribute("sheetId", true));
            this.name = this.RegisterAttribute<string>("name", true);
            this.state = this.RegisterAttribute(new MappedOpenXmlAttribute<SheetVisibility>("state", null, TypeMappers.SheetVisibilityMapper));
            this.relationId = this.RegisterAttribute(new OpenXmlAttribute<string>("id", OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace, true));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SheetElementName;
            }
        }

        public string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        public int SheetId
        {
            get
            {
                return this.sheetId.Value;
            }
            set
            {
                this.sheetId.Value = value;
            }
        }

        public string RelationshipId
        {
            get
            {
                return this.relationId.Value;
            }
            set
            {
                this.relationId.Value = value;
            }
        }

        public SheetVisibility State
        {
            get
            {
                return this.state.Value;
            }
            set
            {
                this.state.Value = value;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorkbookExportContext context, IXlsxWorksheetExportContext sheetContext, int sheetId)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Name = sheetContext.WorksheetName;
            this.SheetId = sheetId;
            this.State = sheetContext.Worksheet.Visibility;

            WorksheetPart worksheetPart = context.GetWorksheetPartFromWorksheet(sheetContext.Worksheet);
            this.RelationshipId = this.PartsManager.CreateWorkbookRelationship(worksheetPart.Name, XlsxRelationshipTypes.WorksheetRelationshipType);
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string target = this.PartsManager.GetRelationshipTarget(this.Part.Name, this.RelationshipId);
            string partName = this.Part.GetResourceName(target);

            SheetPartBase sheetPart = this.PartsManager.GetPart<SheetPartBase>(partName);
            if (sheetPart != null && sheetPart.SheetType == SheetPartType.Worksheet)
            {
                IXlsxWorksheetImportContext worksheetContext = context.AddWorksheet(this.Name);
                context.RegisterWorksheetPart(worksheetContext.Worksheet, (WorksheetPart)sheetPart);
                worksheetContext.Worksheet.Visibility = this.State;
            }
        }
    }
}