﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Workbooks
{
    internal class WorkbookElement : XlsxPartRootElementBase
    {
        private readonly OpenXmlChildElement<WorkbookProtectionElement> workbookProtection;
        private readonly OpenXmlChildElement<BookViewsElement> bookViews;
        private readonly OpenXmlChildElement<SheetsElement> sheets;
        private readonly OpenXmlChildElement<DefinedNamesElement> definedNames;

        public WorkbookElement(XlsxPartsManager partsManager, OpenXmlPartBase part) 
            : base(partsManager, part)
        {
            this.workbookProtection = this.RegisterChildElement<WorkbookProtectionElement>(XlsxElementNames.WorkbookProtectionElementName);
            this.bookViews = this.RegisterChildElement<BookViewsElement>(XlsxElementNames.BookViewsElementName);
            this.sheets = this.RegisterChildElement<SheetsElement>(XlsxElementNames.SheetsElementName);
            this.definedNames = this.RegisterChildElement<DefinedNamesElement>(XlsxElementNames.DefinedNamesElementName);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.WorkbookElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.SpreadsheetMLNamespace;
            }
        }

        public override IEnumerable<OpenXmlNamespace> Namespaces
        {
            get
            {
                yield return OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.workbookProtection);
            this.CreateElement(this.bookViews);
            this.CreateElement(this.sheets);
            this.CreateElement(this.definedNames);
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.workbookProtection);
            this.ReleaseElement(this.bookViews);
            this.ReleaseElement(this.sheets);
            this.ReleaseElement(this.definedNames);
        }
    }
}