﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal abstract class BreaksElementBase : WorksheetElementBase
    {
        private readonly OpenXmlCountAttribute countAttribute;
        private readonly IntOpenXmlAttribute manualBreakCountAttribute;

        public BreaksElementBase(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.countAttribute = this.RegisterCountAttribute();
            this.manualBreakCountAttribute = this.RegisterAttribute(new IntOpenXmlAttribute("manualBreakCount", 0));
        }

        public int Count
        {
            get
            {
                return this.countAttribute.Value;
            }
            set
            {
                this.countAttribute.Value = value;
            }
        }

        public int ManualBreakCount
        {
            get
            {
                return this.manualBreakCountAttribute.Value;
            }
            set
            {
                this.manualBreakCountAttribute.Value = value;
            }
        }

        protected abstract IEnumerable<PageBreakInfo> EnumeratePageBreaks(IXlsxWorksheetExportContext context);

        protected abstract void ApplyPageBreakInfo(IXlsxWorksheetImportContext context, PageBreakInfo pageBreakInfo);

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            int breaksCount = this.EnumeratePageBreaks(context).Count();
            if (breaksCount > 0)
            {
                this.Count = breaksCount;
                this.ManualBreakCount = breaksCount;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return this.EnumeratePageBreaks(context).Any();
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (PageBreakInfo pageBreak in this.EnumeratePageBreaks(context))
            {
                BreakElement breakElement = this.CreateElement<BreakElement>(XlsxElementNames.BreakElementName);
                breakElement.OnBeforeWrite(pageBreak);

                yield return breakElement;
            }
        }

        protected override void OnAfterReadChildElement(IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (childElement.ElementName == XlsxElementNames.BreakElementName)
            {
                BreakElement breakElement = (BreakElement)childElement;
                this.ApplyPageBreakInfo(context, breakElement.PageBreakInfo);
            }
        }
    }
}
