﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class ColumnElement : WorksheetElementBase
    {
        private readonly IntOpenXmlAttribute min;
        private readonly IntOpenXmlAttribute max;
        private readonly BoolOpenXmlAttribute bestFit;
        private readonly BoolOpenXmlAttribute customWidth;
        private readonly IntOpenXmlAttribute styleIndex;
        private readonly OpenXmlAttribute<double> width;
        private readonly BoolOpenXmlAttribute hidden;
        private readonly IntOpenXmlAttribute outlineLevel;

        public ColumnElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.min = this.RegisterAttribute(new IntOpenXmlAttribute("min", true));
            this.max = this.RegisterAttribute(new IntOpenXmlAttribute("max", true));
            this.bestFit = this.RegisterAttribute(new BoolOpenXmlAttribute("bestFit"));
            this.customWidth = this.RegisterAttribute(new BoolOpenXmlAttribute("customWidth"));
            this.styleIndex = this.RegisterAttribute(new IntOpenXmlAttribute("style", 0));
            this.width = this.RegisterAttribute<double>("width");
            this.hidden = this.RegisterAttribute(new BoolOpenXmlAttribute("hidden"));
            this.outlineLevel = this.RegisterAttribute(new IntOpenXmlAttribute("outlineLevel"));
        }

        public int Min
        {
            get
            {
                return this.min.Value;
            }
            set
            {
                this.min.Value = value;
            }
        }

        public int Max
        {
            get
            {
                return this.max.Value;
            }
            set
            {
                this.max.Value = value;
            }
        }

        public bool BestFit
        {
            get
            {
                return this.bestFit.Value;
            }
            set
            {
                this.bestFit.Value = value;
            }
        }

        public bool CustomWidth
        {
            get
            {
                return this.customWidth.Value;
            }
            set
            {
                this.customWidth.Value = value;
            }
        }

        public int StyleIndex
        {
            get
            {
                return this.styleIndex.Value;
            }
            set
            {
                this.styleIndex.Value = value;
            }
        }

        public double Width
        {
            get
            {
                return this.width.Value;
            }
            set
            {
                this.width.Value = value;
            }
        }

        public bool Hidden
        {
            get
            {
                return this.hidden.Value;
            }
            set 
            {
                this.hidden.Value = value;
            }
        }

        public int OutlineLevel
        {
            get
            {
                return this.outlineLevel.Value;
            }
            set
            {
                this.outlineLevel.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.ColumnElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorksheetExportContext context, Range<long, ColumnInfo> columnRange)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(columnRange, "columnRange");

            ColumnInfo column = columnRange.Value;

            Guard.ThrowExceptionIfNull(column, "column");

            this.Min = (int)columnRange.Start + 1;
            this.Max = (int)columnRange.End + 1;
            this.BestFit = column.BestFit;
            this.Width = column.Width;
            this.OutlineLevel = column.OutlineLevel;
            this.Hidden = column.Hidden;

            int? styleIndex = context.GetFormattingRecordIndex(context.Worksheet.Columns, columnRange.Start);
            if (styleIndex.HasValue)
            {
                this.StyleIndex = styleIndex.Value;
            }

            if (column.IsCustom)
            {
                this.CustomWidth = column.IsCustom;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            ColumnInfo columnInfo = new ColumnInfo();
            columnInfo.BestFit = this.BestFit;
            columnInfo.IsCustom = this.CustomWidth;
            columnInfo.Hidden = this.Hidden;
            columnInfo.OutlineLevel = this.OutlineLevel;
            columnInfo.Width = XlsxHelper.ConvertColumnExcelWidthToPixelWidth(context.Worksheet.Workbook, this.Width);
            columnInfo.StyleIndex = this.StyleIndex;

            Range<long, ColumnInfo> columnRange = new Range<long, ColumnInfo>(this.Min - 1, this.Max - 1, true, columnInfo);

            context.ApplyColumnInfo(columnRange);
        }
    }
}