﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal abstract class DataValidationFormulaElementBase : WorksheetElementBase
    {
        private OpenXmlChildElement<FormulaElement> formulaElement;

        public DataValidationFormulaElementBase(XlsxPartsManager partsManager) : base(partsManager)
        {
            this.formulaElement = this.RegisterChildElement<FormulaElement>(XlsxElementNames.FormulaElementName, XlsxElementUniquePoolIds.FormulaElementUniquePoolId);
        }

        public FormulaElement FormulaElement
        {
            get
            {
                return this.formulaElement.Element;
            }
            set
            {
                this.formulaElement.Element = value;
            }
        }

        public string GetValue()
        {
            string value = this.InnerText;
            if (string.IsNullOrEmpty(value))
            {
                value = this.FormulaElement.InnerText;
            }

            double doubleValue;
            if (!double.TryParse(value, out doubleValue))
            {
                value = this.PrepareFormulaText(value);
            }

            return value;
        }

        private string PrepareFormulaText(string formulaText)
        {
            if (formulaText.Contains('\"'))
            {
                formulaText = formulaText.Trim('\"');
                formulaText = formulaText.Replace(SpreadsheetCultureHelper.InvariantSpreadsheetCultureInfo.ListSeparator, FormatHelper.DefaultSpreadsheetCulture.ListSeparator);
            }
            else
            {
                formulaText = SpreadsheetCultureHelper.PrepareFormulaValue(formulaText);
            }

            return formulaText;
        }
    }
}