﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class DataValidationsElement : WorksheetElementBase
    {
        private readonly IntOpenXmlAttribute count;

        public DataValidationsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.count = this.RegisterAttribute(new IntOpenXmlAttribute("count", 0));
        }

        public int Count
        {
            get
            {
                return this.count.Value;
            }
            set
            {
                this.count.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DataValidationsElementName;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return this.Count > 0;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.Count = context.GetDataValidationsCount();
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (IDataValidationRule rule in context.GetDataValidationRules())
            {
                DataValidationElement dataValidationElement = this.CreateElement<DataValidationElement>(XlsxElementNames.DataValidationElementName);
                dataValidationElement.CopyPropertiesFrom(context, rule);

                yield return dataValidationElement;
            }
        }
    }
}