﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class DrawingElement : WorksheetElementBase
    {
        private readonly OpenXmlAttribute<string> relationId;

        public DrawingElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.relationId = this.RegisterAttribute<string>("id", OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DrawingElementName;
            }
        }

        public string RelationshipId
        {
            get
            {
                return this.relationId.Value;
            }
            set
            {
                this.relationId.Value = value;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return context.Shapes.Any() || base.ShouldExport(context);
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (context.Shapes.Any())
            {
                DrawingPart drawingPart = context.WorkbookContext.GetDrawingPartFromWorksheet(context.Worksheet);
                this.RelationshipId = this.PartsManager.CreateRelationship(this.Part.Name, drawingPart.Name, XlsxRelationshipTypes.DrawingRelationshipType);
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            string target = this.PartsManager.GetRelationshipTarget(this.Part.Name, this.RelationshipId);
            string drawingPartName = this.Part.GetResourceName(target);

            DrawingPart drawingPart = this.PartsManager.GetPart<DrawingPart>(drawingPartName);
            context.WorkbookContext.RegisterDrawingPart(context.Worksheet, drawingPart);
        }
    }
}
