﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class AutoFilterElement : WorksheetElementBase
    {
        private readonly ConvertedOpenXmlAttribute<CellRefRange> reference;
        private List<FilterColumnInfo> filterColumnInfos;

        public AutoFilterElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.reference = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRefRange>("ref", XlsxConverters.CellRefRangeConverter, true));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.AutoFilterElementName;
            }
        }

        public CellRefRange Reference
        {
            get
            {
                return this.reference.Value;
            }
            set
            {
                this.reference.Value = value;
            }
        }

        private List<FilterColumnInfo> FilterColumnInfos
        {
            get
            {
                if (this.filterColumnInfos == null)
                {
                    this.filterColumnInfos = new List<FilterColumnInfo>();
                }

                return this.filterColumnInfos;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return context.GetAutoFilterRange() != null;
        }

        protected override void OnAfterReadChildElement(Import.IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            FilterColumnElement filterColumnElement = childElement as FilterColumnElement;
            if (filterColumnElement != null)
            {
                this.FilterColumnInfos.Add(filterColumnElement.GetInfo());
            }
        }

        protected override void OnAfterRead(Import.IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            context.AutoFilterInfo = new AutoFilterInfo(this.Reference.ToCellRange(), this.filterColumnInfos);
        }

        protected override void ClearOverride()
        {
            this.filterColumnInfos = null;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CellRefRange range = context.GetAutoFilterRange();
            if (range != null)
            {
                this.Reference = range;
            }
        }
        
        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (FilterColumnInfo filterColumnInfo in context.GetFilterColumnInfos())
            {
                FilterColumnElement filterColumnElement = this.CreateElement<FilterColumnElement>(XlsxElementNames.FilterColumnElementName);
                filterColumnElement.CopyPropertiesFrom(context, filterColumnInfo.ColumnId);

                yield return filterColumnElement;
            }
        }
    }
}
