﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class ColorFilterElement : FilterElementBase
    {
        private readonly BoolOpenXmlAttribute cellColor;
        private readonly IntOpenXmlAttribute dxfId;

        public ColorFilterElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.dxfId = this.RegisterAttribute(new IntOpenXmlAttribute("dxfId", isRequired: false));
            this.cellColor = this.RegisterAttribute(new BoolOpenXmlAttribute("cellColor", true, isRequired: false));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.ColorFilterElementName;
            }
        }

        public bool CellColor
        {
            get
            {
                return this.cellColor.Value;
            }
            set
            {
                this.cellColor.Value = value;
            }
        }

        public int DxfId
        {
            get
            {
                return this.dxfId.Value;
            }
            set
            {
                this.dxfId.Value = value;
            }
        }

        internal override IFilterInfo GetInfo()
        {
            ColorFilterInfo info = new ColorFilterInfo();

            info.DxfId = this.DxfId;
            info.CellColor = this.CellColor;

            return info;
        }

        internal override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            ColorFilterInfo colorFilterInfo = context.GetColorFilterInfo(columnId);
            
            this.DxfId = colorFilterInfo.DxfId;

            if (colorFilterInfo.CellColor != this.cellColor.DefaultValue)
            {
                this.CellColor = colorFilterInfo.CellColor;
            }
        }
    }
}