﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class CustomFiltersElement : FilterElementBase
    {
        private readonly BoolOpenXmlAttribute isAnd;
        private CustomFiltersInfo customFiltersInfo;
        private int? columnId;

        public CustomFiltersElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.isAnd = this.RegisterAttribute(new BoolOpenXmlAttribute("and"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.CustomFiltersElementName;
            }
        }

        public bool IsAnd
        {
            get
            {
                return this.isAnd.Value;
            }
            set
            {
                this.isAnd.Value = value;
            }
        }

        private CustomFiltersInfo CustomFiltersInfo
        {
            get
            {
                if (this.customFiltersInfo == null)
                {
                    this.customFiltersInfo = new CustomFiltersInfo();
                }

                return this.customFiltersInfo;
            }
        }

        internal override IFilterInfo GetInfo()
        {
            return this.customFiltersInfo;
        }

        internal override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            this.columnId = columnId;
        }

        protected override void ClearOverride()
        {
            this.customFiltersInfo = null;
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return base.ShouldExport(context) || this.columnId != null;
        }
        
        protected override void OnAfterReadChildElement(Import.IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            CustomFilterElement customFilterElement = childElement as CustomFilterElement;
            if (customFilterElement != null)
            {
                CustomFilterCriteriaInfo customFilterInfo = new CustomFilterCriteriaInfo(
                    new CustomFilterCriteria(customFilterElement.FilterOperator, customFilterElement.Value));
                this.CustomFiltersInfo.CustomFilters.Add(customFilterInfo);
            }
        }

        protected override void OnAfterRead(Import.IXlsxWorksheetImportContext context)
        {
            this.CustomFiltersInfo.IsAnd = this.IsAnd;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            CustomFiltersInfo customFilters = context.GetCustomFilterInfo((int)this.columnId);

            if (customFilters.IsAnd)
            {
                this.IsAnd = customFilters.IsAnd;
            }
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            CustomFiltersInfo customFilters = context.GetCustomFilterInfo((int)this.columnId);

            CustomFilterElement customFilterElement1 = this.CreateElement<CustomFilterElement>(XlsxElementNames.CustomFilterElementName);
            customFilterElement1.CopyPropertiesFrom(customFilters.CustomFilters[0]);
            yield return customFilterElement1;

            if (customFilters.CustomFilters.Count > 1)
            {
                CustomFilterElement customFilterElement2 = this.CreateElement<CustomFilterElement>(XlsxElementNames.CustomFilterElementName);
                customFilterElement2.CopyPropertiesFrom(customFilters.CustomFilters[1]);
                yield return customFilterElement2;
            }
        }
    }
}
