﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class DateGroupItemElement : WorksheetElementBase
    {
        private readonly ConvertedOpenXmlAttribute<DateTimeGroupingType> dateTimeGrouping;
        private readonly IntOpenXmlAttribute year;
        private readonly IntOpenXmlAttribute month;
        private readonly IntOpenXmlAttribute day;
        private readonly IntOpenXmlAttribute hour;
        private readonly IntOpenXmlAttribute minute;
        private readonly IntOpenXmlAttribute second;

        public DateGroupItemElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.year = this.RegisterAttribute(new IntOpenXmlAttribute("year", isRequired: true));
            this.month = this.RegisterAttribute(new IntOpenXmlAttribute("month"));
            this.day = this.RegisterAttribute(new IntOpenXmlAttribute("day"));
            this.hour = this.RegisterAttribute(new IntOpenXmlAttribute("hour"));
            this.minute = this.RegisterAttribute(new IntOpenXmlAttribute("minute"));
            this.second = this.RegisterAttribute(new IntOpenXmlAttribute("second"));
            this.dateTimeGrouping = this.RegisterAttribute(new ConvertedOpenXmlAttribute<DateTimeGroupingType>("dateTimeGrouping", null, XlsxConverters.DateTimeGroupingTypeConverter, DateTimeGroupingTypeConverter.DefaultExportDateTimeGroupingType));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DateGroupItemElementName;
            }
        }

        public DateTimeGroupingType DateTimeGrouping
        {
            get
            {
                return this.dateTimeGrouping.Value;
            }
            set
            {
                this.dateTimeGrouping.Value = value;
            }
        }

        public int Year
        {
            get
            {
                return this.year.Value;
            }
            set
            {
                this.year.Value = value;
            }
        }

        public int Month
        {
            get
            {
                return this.month.Value;
            }
            set
            {
                this.month.Value = value;
            }
        }

        public int Day
        {
            get
            {
                return this.day.Value;
            }
            set
            {
                this.day.Value = value;
            }
        }

        public int Hour
        {
            get
            {
                return this.hour.Value;
            }
            set
            {
                this.hour.Value = value;
            }
        }

        public int Minute
        {
            get
            {
                return this.minute.Value;
            }
            set
            {
                this.minute.Value = value;
            }
        }

        public int Second
        {
            get
            {
                return this.second.Value;
            }
            set
            {
                this.second.Value = value;
            }
        }
        
        public void CopyPropertiesFrom(DateGroupItemInfo dateFilter)
        {
            this.DateTimeGrouping = dateFilter.DateTimeGroupingType;
            this.Year = dateFilter.Year;

            if (dateFilter.Month != 0)
            {
                this.Month = dateFilter.Month;
            }

            if (dateFilter.Day != 0)
            {
                this.Day = dateFilter.Day;
            }

            if (dateFilter.Hour != 0)
            {
                this.Hour = dateFilter.Hour;
            }

            if (dateFilter.Minute != 0)
            {
                this.Minute = dateFilter.Minute;
            }

            if (dateFilter.Second != 0)
            {
                this.Second = dateFilter.Second;
            }
        }
    }
}
