﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class DynamicFilterElement : FilterElementBase
    {
        private readonly ConvertedOpenXmlAttribute<DynamicFilterType> filterType;
        private readonly OpenXmlAttribute<double> value;
        private readonly OpenXmlAttribute<double> valueIso;
        private readonly OpenXmlAttribute<double> maxValueIso;
        
        public DynamicFilterElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.filterType = this.RegisterAttribute(new ConvertedOpenXmlAttribute<DynamicFilterType>("type", null, XlsxConverters.DynamicFilterTypeConverter, DynamicFilterTypeConverter.DefaultExportDynamicFilterType));
            this.value = this.RegisterAttribute<double>("val");
            this.valueIso = this.RegisterAttribute<double>("valIso");
            this.maxValueIso = this.RegisterAttribute<double>("maxValIso");
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.DynamicFilterElementName;
            }
        }

        public DynamicFilterType Type
        {
            get
            {
                return this.filterType.Value;
            }
            set
            {
                this.filterType.Value = value;
            }
        }

        public double Value
        {
            get
            {
                return this.value.Value;
            }
            set
            {
                this.value.Value = value;
            }
        }

        public double ValueIso
        {
            get
            {
                return this.valueIso.Value;
            }
            set
            {
                this.valueIso.Value = value;
            }
        }

        public double MaxValueIso
        {
            get
            {
                return this.maxValueIso.Value;
            }
            set
            {
                this.maxValueIso.Value = value;
            }
        }

        internal override IFilterInfo GetInfo()
        {
            DynamicFilterInfo dynamicFilterInfo = new DynamicFilterInfo();
            dynamicFilterInfo.DynamicFilterType = this.Type;

            return dynamicFilterInfo;
        }

        internal override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            DynamicFilterInfo dynamicFilterInfo = context.GetDynamicFilterInfo(columnId);

            this.Type = dynamicFilterInfo.DynamicFilterType;
        }
    }
}
