﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class FilterColumnElement : WorksheetElementBase
    {
        private readonly IntOpenXmlAttribute columnId;
        private FilterColumnInfo filterColumnInfo;

        public FilterColumnElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.columnId = this.RegisterAttribute(new IntOpenXmlAttribute("colId", true));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FilterColumnElementName;
            }
        }

        public int ColumnId
        {
            get
            {
                return this.columnId.Value;
            }
            set
            {
                this.columnId.Value = value;
            }
        }

        private FilterColumnInfo FilterColumnInfo
        {
            get
            {
                if (this.filterColumnInfo == null)
                {
                    this.filterColumnInfo = new FilterColumnInfo(this.ColumnId);
                }

                return this.filterColumnInfo;
            }
        }

        internal FilterColumnInfo GetInfo()
        {
            return this.filterColumnInfo;
        }

        internal void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ColumnId = columnId;
        }

        protected override void OnAfterReadChildElement(IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(childElement, "childElement");

            FilterElementBase filterElement = childElement as FilterElementBase;
            if (filterElement != null)
            {
                this.FilterColumnInfo.FilterInfo = filterElement.GetInfo();
            }
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            
            FilterElementBase element = null;
            IFilterInfo filterInfo = context.GetFilterInfo(this.ColumnId);

            if (filterInfo is FiltersInfo)
            {
                element = this.CreateElement<FiltersElement>(XlsxElementNames.FiltersElementName);
            }
            else if (filterInfo is CustomFiltersInfo)
            {
                element = this.CreateElement<CustomFiltersElement>(XlsxElementNames.CustomFiltersElementName);
            }
            else if (filterInfo is DynamicFilterInfo)
            {
                element = this.CreateElement<DynamicFilterElement>(XlsxElementNames.DynamicFilterElementName);
            }
            else if (filterInfo is Top10FilterInfo)
            {
                element = this.CreateElement<Top10FilterElement>(XlsxElementNames.Top10FilterElementName);
            }
            else if (filterInfo is ColorFilterInfo)
            {
                element = this.CreateElement<ColorFilterElement>(XlsxElementNames.ColorFilterElementName);
            }

            if (element != null)
            {
                element.CopyPropertiesFrom(context, this.ColumnId);
                yield return element;
            }
        }

        protected override void ClearOverride()
        {
            this.filterColumnInfo = null;
        }
    }
}
