﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class FiltersElement : FilterElementBase
    {
        private readonly BoolOpenXmlAttribute blank;
        private int? columnId;
        private FiltersInfo filtersInfo;

        public FiltersElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.blank = this.RegisterAttribute(new BoolOpenXmlAttribute("blank"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FiltersElementName;
            }
        }

        public bool Blank
        {
            get 
            {
                return this.blank.Value;
            }
            set
            {
                this.blank.Value = value;
            }
        }

        private FiltersInfo FiltersInfo
        {
            get
            {
                if (this.filtersInfo == null)
                {
                    this.filtersInfo = new FiltersInfo();
                }

                return this.filtersInfo;
            }
        }

        internal override IFilterInfo GetInfo()
        {
            return this.filtersInfo;
        }

        internal override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            this.columnId = columnId;
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return true;
        }

        protected override void ClearOverride()
        {
            this.filtersInfo = null;
        }

        protected override void OnAfterReadChildElement(IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            FilterElement filterElement = childElement as FilterElement;
            if (filterElement != null)
            {
                this.FiltersInfo.StringFilters.Add(filterElement.Value);
            }

            DateGroupItemElement dateGroupItemElement = childElement as DateGroupItemElement;
            if (dateGroupItemElement != null)
            {
                DateGroupItemInfo dateGroupItemInfo = new DateGroupItemInfo(dateGroupItemElement.DateTimeGrouping,
                    dateGroupItemElement.Year,
                    dateGroupItemElement.Month,
                    dateGroupItemElement.Day, 
                    dateGroupItemElement.Hour, 
                    dateGroupItemElement.Minute, 
                    dateGroupItemElement.Second);

                this.FiltersInfo.DateFilters.Add(dateGroupItemInfo);
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            this.FiltersInfo.Blank = this.Blank;
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            FiltersInfo filterInfo = context.GetFiltersInfo((int)this.columnId);

            for (int i = 0; i < filterInfo.StringFilters.Count; i++)
            {
                FilterElement filterElement = this.CreateElement<FilterElement>(XlsxElementNames.FilterElementName);
                filterElement.CopyPropertiesFrom(filterInfo.StringFilters[i]);

                yield return filterElement;
            }

            for (int i = 0; i < filterInfo.DateFilters.Count; i++)
            {
                DateGroupItemElement dateGroupItemElement = this.CreateElement<DateGroupItemElement>(XlsxElementNames.DateGroupItemElementName);
                dateGroupItemElement.CopyPropertiesFrom(filterInfo.DateFilters[i]);

                yield return dateGroupItemElement;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            FiltersInfo filterInfo = context.GetFiltersInfo((int)this.columnId);

            if (filterInfo.Blank != this.blank.DefaultValue)
            {
                this.Blank = filterInfo.Blank;
            }
        }
    }
}
