﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class Top10FilterElement : FilterElementBase
    {
        private readonly OpenXmlAttribute<double> fitlerValue;
        private readonly BoolOpenXmlAttribute percent;
        private readonly BoolOpenXmlAttribute top;
        private readonly OpenXmlAttribute<double> value;
        
        public Top10FilterElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.fitlerValue = this.RegisterAttribute<double>("filterVal");
            this.percent = this.RegisterAttribute(new BoolOpenXmlAttribute("percent"));
            this.top = this.RegisterAttribute(new BoolOpenXmlAttribute("top", true, isRequired: false));
            this.value = this.RegisterAttribute<double>("val", isRequired: true);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.Top10FilterElementName;
            }
        }

        public double FitlerValue
        {
            get
            {
                return this.fitlerValue.Value;
            }
            set
            {
                this.fitlerValue.Value = value;
            }
        }

        public bool Percent
        {
            get
            {
                return this.percent.Value;
            }
            set
            {
                this.percent.Value = value;
            }
        }

        public bool Top
        {
            get
            {
                return this.top.Value;
            }
            set
            {
                this.top.Value = value;
            }
        }

        public double Value
        {
            get
            {
                return this.value.Value;
            }
            set
            {
                this.value.Value = value;
            }
        }

        internal override IFilterInfo GetInfo()
        {
            Top10FilterInfo top10FilterInfo = new Top10FilterInfo();

            top10FilterInfo.Value = this.Value;
            top10FilterInfo.Top = this.Top;
            top10FilterInfo.Percent = this.Percent;

            return top10FilterInfo;
        }

        internal override void CopyPropertiesFrom(IXlsxWorksheetExportContext context, int columnId)
        {
            Top10FilterInfo top10FilterInfo = context.GetTop10FilterInfo(columnId);

            this.Value = top10FilterInfo.Value;

            if (top10FilterInfo.Top != this.top.DefaultValue)
            {
                this.Top = top10FilterInfo.Top;
            }

            if (top10FilterInfo.Percent != this.percent.DefaultValue)
            {
                this.Percent = top10FilterInfo.Percent;
            }
        }
    }
}
