﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class FormulaElement : XlsxElementBase
    {
        private readonly OpenXmlAttribute<string> formulaType;
        private readonly IntOpenXmlAttribute sharedGroupIndex;
        private readonly ConvertedOpenXmlAttribute<Ref> reference;

        public FormulaElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.formulaType = this.RegisterAttribute<string>("t", FormulaTypes.Normal);
            this.reference = this.RegisterAttribute(new ConvertedOpenXmlAttribute<Ref>("ref", XlsxConverters.RefConverter));
            this.sharedGroupIndex = this.RegisterAttribute(new IntOpenXmlAttribute("si"));
        }

        public string FormulaType
        {
            get
            {
                return this.formulaType.Value;
            }
            set
            {
                this.formulaType.Value = value;
            }
        }

        public Ref Reference
        {
            get
            {
                return this.reference.Value;
            }
            set
            {
                this.reference.Value = value;
            }
        }

        public int SharedGroupIndex
        {
            get
            {
                return this.sharedGroupIndex.Value;
            }
            set
            {
                this.sharedGroupIndex.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FormulaElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorksheetExportContext context, FormulaCellValue formula)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(formula, "formula");

            this.InnerText = SpreadsheetCultureHelper.ClearFormulaValue(formula.GetExportString());
        }

        public bool CopyPropertiesTo(IXlsxWorksheetImportContext context, int rowIndex, int columnIndex, out ICellValue cellValue)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            if (this.FormulaType == FormulaTypes.Shared)
            {
                cellValue = CreateSharedCellValue(context, rowIndex, columnIndex);
            }
            else if (!string.IsNullOrEmpty(this.InnerText))
            {
                cellValue = CellValueFactory.Create(SpreadsheetCultureHelper.PrepareFormulaValue(this.InnerText), context.Worksheet, rowIndex, columnIndex, XlsxHelper.CultureInfo);
            }
            else
            {
                cellValue = null;

                return false;
            }

            return true;
        }

        private ICellValue CreateSharedCellValue(IXlsxWorksheetImportContext context, int rowIndex, int columnIndex)
        {
            ICellValue cellValue = null;

            if (this.reference.HasValue)
            { 
                cellValue = CellValueFactory.Create(SpreadsheetCultureHelper.PrepareFormulaValue(this.InnerText), context.Worksheet, rowIndex, columnIndex, XlsxHelper.CultureInfo);
                FormulaCellValue value = cellValue as FormulaCellValue;

                // TODO: This needs to be fixed after refactoring the parser. It temprorary solves the issue with parsing (A1,A2,A3,A2:B4) as 4 function arguments instead of simply one CellReferenceRange expression containing a list of CellReferenceRanges.
                if (value != null)
                {
                    context.RegisterSharedFormula(this.SharedGroupIndex, new SharedFormulaInfo(this.Reference, value.CloneAndTranslate(context.Worksheet, rowIndex, columnIndex)));
                }
                else
                {
                    CellValueFormat newFormat;
                    CellValueFactory.Create(ErrorExpressions.NameError.Value, context.Worksheet, rowIndex, columnIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);
                }
            }
            else
            {
                SharedFormulaInfo sharedFormula = context.GetSharedFormula(this.SharedGroupIndex);

                // TODO: This needs to be fixed after refactoring the parser. It temprorary solves the issue with parsing (A1,A2,A3,A2:B4) as 4 function arguments instead of simply one CellReferenceRange expression containing a list of CellReferenceRanges.
                if (sharedFormula != null)
                {
                    cellValue = sharedFormula.FormulaCellValue.CloneAndTranslate(context.Worksheet, rowIndex, columnIndex);
                }
                else
                {
                    CellValueFormat newFormat;
                    CellValueFactory.Create(ErrorExpressions.NameError.Value, context.Worksheet, rowIndex, columnIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);
                }
            }

            return cellValue;
        }
    }
}