﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class MergedCellElement : WorksheetElementBase
    {
        private readonly ConvertedOpenXmlAttribute<Ref> reference;

        public MergedCellElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.reference = this.RegisterAttribute(new ConvertedOpenXmlAttribute<Ref>("ref", XlsxConverters.RefConverter, true));
        }

        public Ref Reference
        {
            get
            {
                return this.reference.Value;
            }
            set
            {
                this.reference.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.MergedCellElementName;
            }
        }

        public void CopyPropertiesFrom(IXlsxWorksheetExportContext context, MergedCellInfo mergedCell)
        {
            Guard.ThrowExceptionIfNull(context, "context");
            Guard.ThrowExceptionIfNull(mergedCell, "mergedCell");

            this.Reference = new Ref(mergedCell.CellRange);
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            MergedCellInfo mergedCellInfo = new MergedCellInfo();
            mergedCellInfo.CellRange = this.Reference.ToCellRange();

            context.ApplyMergeCellInfo(mergedCellInfo);
        }
    }
}