﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class OutlinePropertiesElement : WorksheetElementBase
    {
        private readonly BoolOpenXmlAttribute summaryBelow;
        private readonly BoolOpenXmlAttribute summaryRight;

        public OutlinePropertiesElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.summaryBelow = this.RegisterAttribute(new BoolOpenXmlAttribute("summaryBelow", true, isRequired: false));
            this.summaryRight = this.RegisterAttribute(new BoolOpenXmlAttribute("summaryRight", true, isRequired: false));
        }

        public override string ElementName
        {
            get 
            {
                return XlsxElementNames.OutlinePropertiesElementName;
            }
        }

        public bool SummaryBelow
        {
            get
            {
                return this.summaryBelow.Value;
            }
            set
            {
                this.summaryBelow.Value = value;
            }
        }

        public bool SummaryRight
        {
            get
            {
                return this.summaryRight.Value;
            }
            set
            {
                this.summaryRight.Value = value;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return !context.Worksheet.GroupingProperties.SummaryRowIsBelow ||
                !context.Worksheet.GroupingProperties.SummaryColumnIsToRight;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.SummaryBelow = context.Worksheet.GroupingProperties.SummaryRowIsBelow;
            this.SummaryRight = context.Worksheet.GroupingProperties.SummaryColumnIsToRight;
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            context.Worksheet.GroupingProperties.SummaryRowIsBelow = this.SummaryBelow;
            context.Worksheet.GroupingProperties.SummaryColumnIsToRight = this.SummaryRight;
        }
    }
}
