﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class PageMarginsElement : WorksheetElementBase
    {
        private readonly OpenXmlAttribute<double> bottom;
        private readonly OpenXmlAttribute<double> footer;
        private readonly OpenXmlAttribute<double> header;
        private readonly OpenXmlAttribute<double> left;
        private readonly OpenXmlAttribute<double> right;
        private readonly OpenXmlAttribute<double> top;

        public PageMarginsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.bottom = this.RegisterAttribute<double>("bottom", isRequired: true);
            this.footer = this.RegisterAttribute<double>("footer", isRequired: true);
            this.header = this.RegisterAttribute<double>("header", isRequired: true);
            this.left = this.RegisterAttribute<double>("left", isRequired: true);
            this.right = this.RegisterAttribute<double>("right", isRequired: true);
            this.top = this.RegisterAttribute<double>("top", isRequired: true);
        }

        public double Bottom
        {
            get
            {
                return this.bottom.Value;
            }
            set
            {
                this.bottom.Value = value;
            }
        }

        public double Footer
        {
            get
            {
                return this.footer.Value;
            }
            set
            {
                this.footer.Value = value;
            }
        }

        public double Header
        {
            get
            {
                return this.header.Value;
            }
            set
            {
                this.header.Value = value;
            }
        }

        public double Left
        {
            get
            {
                return this.left.Value;
            }
            set
            {
                this.left.Value = value;
            }
        }

        public double Right
        {
            get
            {
                return this.right.Value;
            }
            set
            {
                this.right.Value = value;
            }
        }

        public double Top
        {
            get
            {
                return this.top.Value;
            }
            set
            {
                this.top.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PageMarginsElementName;
            }
        }

        public override bool AlwaysExport
        {
            get
            {
                return true;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            PageMarginsInfo pageMarginsInfo = new PageMarginsInfo();

            pageMarginsInfo.Bottom = this.Bottom;
            pageMarginsInfo.Footer = this.Footer;
            pageMarginsInfo.Header = this.Header;
            pageMarginsInfo.Left = this.Left;
            pageMarginsInfo.Right = this.Right;
            pageMarginsInfo.Top = this.Top;

            context.ApplyPageMargins(pageMarginsInfo);
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            PageMarginsInfo pageMarginsInfo = context.GetPageMarginsInfo();

            this.Bottom = pageMarginsInfo.Bottom;
            this.Footer = pageMarginsInfo.Footer;
            this.Header = pageMarginsInfo.Header;
            this.Left = pageMarginsInfo.Left;
            this.Right = pageMarginsInfo.Right;
            this.Top = pageMarginsInfo.Top;
        }
    }
}
