﻿using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class PageSetUpPropertiesElement : WorksheetElementBase
    {
        private readonly BoolOpenXmlAttribute fitToPage;

        public PageSetUpPropertiesElement(XlsxPartsManager partsManager) : base(partsManager)
        {
            this.fitToPage = this.RegisterAttribute(new BoolOpenXmlAttribute("fitToPage", true, isRequired: false));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PageSetUpPropertiesElementName;
            }
        }

        public bool FitToPage
        {
            get
            {
                return this.fitToPage.Value;
            }
            set
            {
                this.fitToPage.Value = value;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return context.Worksheet.WorksheetPageSetup.FitToPages;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.FitToPage = context.Worksheet.WorksheetPageSetup.FitToPages;
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            context.Worksheet.WorksheetPageSetup.FitToPages = this.FitToPage;
        }
    }
}
