﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("StyleCop.CSharp.NamingRules", "SA1305:FieldNamesMustNotUseHungarianNotation", Justification = "Using attribute names for consistency.")]
    internal class PaneElement : WorksheetElementBase
    {
        private readonly IntOpenXmlAttribute xSplit;
        private readonly IntOpenXmlAttribute ySplit;
        private readonly ConvertedOpenXmlAttribute<CellRef> topLeftCell;
        private readonly MappedOpenXmlAttribute<PaneState> state;
        private readonly MappedOpenXmlAttribute<ViewportPaneType> activePane;

        public PaneElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.xSplit = this.RegisterAttribute(new IntOpenXmlAttribute("xSplit", 0));
            this.ySplit = this.RegisterAttribute(new IntOpenXmlAttribute("ySplit", 0));
            this.topLeftCell = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("topLeftCell", XlsxConverters.CellRefConverter, true));

            this.activePane = this.RegisterAttribute(new MappedOpenXmlAttribute<ViewportPaneType>("activePane", null, TypeMappers.ViewportTypeMapper));
            this.state = this.RegisterAttribute(new MappedOpenXmlAttribute<PaneState>("state", null, TypeMappers.ViewportStateMapper));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PaneElementName;
            }
        }

        private int XSplit
        {
            get
            {
                return this.xSplit.Value;
            }
            set
            {
                this.xSplit.Value = value;
            }
        }

        private int YSplit
        {
            get
            {
                return this.ySplit.Value;
            }
            set
            {
                this.ySplit.Value = value;
            }
        }

        private CellRef TopLeftCell
        {
            get
            {
                return this.topLeftCell.Value;
            }
            set
            {
                this.topLeftCell.Value = value;
            }
        }

        private ViewportPaneType ActivePane
        {
            get
            {
                return this.activePane.Value;
            }
            set
            {
                this.activePane.Value = value;
            }
        }

        private PaneState State
        {
            get
            {
                return this.state.Value;
            }
            set
            {
                this.state.Value = value;
            }
        }

        public Pane ToPane()
        {
            CellIndex topLeftCellIndex = this.TopLeftCell.ToCellIndex();
            Pane pane = new Pane(topLeftCellIndex, this.XSplit, this.YSplit, this.ActivePane, this.State);

            return pane;
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            if (context.WorksheetViewState.Pane == null)
            {
                return false;
            }

            Pane pane = context.WorksheetViewState.Pane;

            if (pane.XSplit == 0 && pane.YSplit == 0)
            {
                return false;
            }

            return true;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            if (context.WorksheetViewState.Pane != null)
            {
                Pane contextPane = context.WorksheetViewState.Pane;

                this.ActivePane = ViewportPaneHelper.GetActivePaneType(context.WorksheetViewState.Pane);

                this.TopLeftCell = new CellRef(contextPane.TopLeftCellIndex.RowIndex, contextPane.TopLeftCellIndex.ColumnIndex);
                this.XSplit = contextPane.XSplit;
                this.YSplit = contextPane.YSplit;
                this.State = contextPane.State;
            }
        }
    }
}