﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class PrintOptionsElement : WorksheetElementBase
    {
        private readonly BoolOpenXmlAttribute horizontalCentered;
        private readonly BoolOpenXmlAttribute verticalCentered;
        private readonly BoolOpenXmlAttribute headings;
        private readonly BoolOpenXmlAttribute gridLines;

        public PrintOptionsElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.horizontalCentered = this.RegisterAttribute(new BoolOpenXmlAttribute("horizontalCentered"));
            this.verticalCentered = this.RegisterAttribute(new BoolOpenXmlAttribute("verticalCentered"));
            this.headings = this.RegisterAttribute(new BoolOpenXmlAttribute("headings"));
            this.gridLines = this.RegisterAttribute(new BoolOpenXmlAttribute("gridLines"));
        }

        public bool HorizontalCentered
        {
            get
            {
                return this.horizontalCentered.Value;
            }
            set
            {
                this.horizontalCentered.Value = value;
            }
        }

        public bool VerticalCentered
        {
            get
            {
                return this.verticalCentered.Value;
            }
            set
            {
                this.verticalCentered.Value = value;
            }
        }

        public bool Headings
        {
            get
            {
                return this.headings.Value;
            }
            set
            {
                this.headings.Value = value;
            }
        }

        public bool GridLines
        {
            get
            {
                return this.gridLines.Value;
            }
            set
            {
                this.gridLines.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PrintOptionsElementName;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            PrintOptionsInfo printOptionsInfo = new PrintOptionsInfo();

            printOptionsInfo.HorizontalCentered = this.HorizontalCentered;
            printOptionsInfo.VerticalCentered = this.VerticalCentered;
            printOptionsInfo.GridLines = this.GridLines;
            printOptionsInfo.Headings = this.Headings;

            context.ApplyPrintOptions(printOptionsInfo);
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            PrintOptionsInfo printOptionsInfo = context.GetPrintOptionsInfo();

            if (printOptionsInfo.HorizontalCentered != this.horizontalCentered.DefaultValue)
            {
                this.HorizontalCentered = printOptionsInfo.HorizontalCentered;
            }

            if (printOptionsInfo.VerticalCentered != this.verticalCentered.DefaultValue)
            {
                this.VerticalCentered = printOptionsInfo.VerticalCentered;
            }

            if (printOptionsInfo.GridLines != this.gridLines.DefaultValue)
            {
                this.GridLines = printOptionsInfo.GridLines;
            }

            if (printOptionsInfo.Headings != this.headings.DefaultValue)
            {
                this.Headings = printOptionsInfo.Headings;
            }
        }
    }
}
