﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SelectionElement : WorksheetElementBase
    {
        private readonly ConvertedOpenXmlAttribute<CellRef> activeCell;
        private readonly ConvertedOpenXmlAttribute<CellRefRangeSequence> sqref;
        private readonly MappedOpenXmlAttribute<ViewportPaneType> pane;

        public SelectionElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.activeCell = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("activeCell", XlsxConverters.CellRefConverter, true));
            this.sqref = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRefRangeSequence>("sqref", XlsxConverters.CellRefRangeSequenceConverter, true));

            this.pane = this.RegisterAttribute(new MappedOpenXmlAttribute<ViewportPaneType>("pane", null, TypeMappers.ViewportTypeMapper));
        }

        public CellRef ActiveCell
        {
            get
            {
                return this.activeCell.Value;
            }
            set
            {
                this.activeCell.Value = value;
            }
        }

        public CellRefRangeSequence SqRef
        {
            get
            {
                return this.sqref.Value;
            }
            set
            {
                this.sqref.Value = value;
            }
        }

        public ViewportPaneType Pane
        {
            get
            {
                return this.pane.Value;
            }
            set
            {
                this.pane.Value = value;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SelectionElementName;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            if (context.WorksheetViewState.Pane != null)
            {
                return true;
            }

            SelectionState selectionState = context.WorksheetViewState.SelectionState;

            bool hasMultipleSelections = selectionState.SelectedRanges.Count() > 1;
            if (hasMultipleSelections)
            {
                return true;
            }

            CellIndex firstCell = new CellIndex(0, 0);
            CellRange selectedRange = selectionState.SelectedRanges.First();

            if ((this.ActiveCell != null &&
                this.ActiveCell.ToCellIndex() != firstCell) ||
                !selectedRange.IsSingleCell)
            {
                return true;
            }

            return false;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            SelectionState selectionState = context.WorksheetViewState.SelectionState;

            if (context.WorksheetViewState.Pane != null)
            {
                this.Pane = ViewportPaneHelper.GetActivePaneType(context.WorksheetViewState.Pane);
            }

            this.ActiveCell = new CellRef(selectionState.ActiveCellIndex.RowIndex, selectionState.ActiveCellIndex.ColumnIndex);

            this.RetrieveAndSetCellReferenceSequence(selectionState.SelectedRanges, this);
        }

        private void RetrieveAndSetCellReferenceSequence(IEnumerable<CellRange> cellRanges, SelectionElement selectionElement)
        {
            StringBuilder builder = new StringBuilder();

            foreach (var cellRange in cellRanges)
            {
                builder.Append(NameConverter.ConvertCellRangeToName(cellRange.FromIndex, cellRange.ToIndex));
                builder.Append(' ');
            }

            selectionElement.SqRef = new CellRefRangeSequence(builder.ToString());
        }
    }
}