﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SheetFormatPropertiesElement : WorksheetElementBase
    {
        private readonly BoolOpenXmlAttribute customHeight;
        private readonly OpenXmlAttribute<double> defaultColumnWidth;
        private readonly OpenXmlAttribute<double> defaultRowHeight;

        public SheetFormatPropertiesElement(XlsxPartsManager partsManager) 
            : base(partsManager)
        {
            this.customHeight = this.RegisterAttribute(new BoolOpenXmlAttribute("customHeight"));
            this.defaultColumnWidth = this.RegisterAttribute<double>("defaultColWidth");
            this.defaultRowHeight = this.RegisterAttribute<double>("defaultRowHeight");
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SheetFormatPropertiesElementName;
            }
        }

        public bool CustomHeight
        {
            get
            {
                return this.customHeight.Value;
            }
            set
            {
                this.customHeight.Value = value;
            }
        }

        public double DefaultColumnWidth
        {
            get
            {
                return this.defaultColumnWidth.Value;
            }
            set
            {
                this.defaultColumnWidth.Value = value;
            }
        }

        public double DefaultRowHeight
        {
            get
            {
                return this.defaultRowHeight.Value;
            }
            set
            {
                this.defaultRowHeight.Value = value;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.DefaultColumnWidth = context.GetDefaultColumnWidth();
            
            RowHeight height = context.GetDefaultRowHeight();
            this.CustomHeight = height.IsCustom;
            this.DefaultRowHeight = UnitHelper.DipToPoint(height.Value);
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            if (this.defaultColumnWidth.HasValue)
            {
                context.ApplyDefaultColumnWidth(new ColumnWidth(XlsxHelper.ConvertColumnExcelWidthToPixelWidth(context.Worksheet.Workbook, this.DefaultColumnWidth), false));
            }

            if (this.defaultRowHeight.HasValue)
            {
                context.ApplyDefaultColumnWidthDefaultRowHeight(new RowHeight(UnitHelper.PointToDip(this.DefaultRowHeight), this.CustomHeight));
            }
        }
    }
}
