﻿using System;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SheetPropertiesElement : WorksheetElementBase
    {
        private readonly OpenXmlChildElement<OutlinePropertiesElement> outlinePropertiesElement;
        private readonly OpenXmlChildElement<PageSetUpPropertiesElement> pageSetUpPropertiesElement;

        public SheetPropertiesElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.outlinePropertiesElement = this.RegisterChildElement<OutlinePropertiesElement>(XlsxElementNames.OutlinePropertiesElementName);
            this.pageSetUpPropertiesElement = this.RegisterChildElement<PageSetUpPropertiesElement>(XlsxElementNames.PageSetUpPropertiesElementName);
        }

        public override string ElementName
        {
            get 
            { 
                 return XlsxElementNames.SheetPropertiesElementName;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.outlinePropertiesElement);
            this.CreateElement(this.pageSetUpPropertiesElement);
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.outlinePropertiesElement);
            this.ReleaseElement(this.pageSetUpPropertiesElement);
        }
    }
}
