﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SheetProtectionElement : WorksheetElementBase
    {
        private readonly OpenXmlAttribute<string> algorithmName;
        private readonly OpenXmlAttribute<string> hashValue;
        private readonly OpenXmlAttribute<string> saltValue;
        private readonly IntOpenXmlAttribute spinCount;
        private readonly OpenXmlAttribute<string> password;
        private readonly BoolOpenXmlAttribute sheet;
        private readonly BoolOpenXmlAttribute formatCells;
        private readonly BoolOpenXmlAttribute formatColumns;
        private readonly BoolOpenXmlAttribute formatRows;
        private readonly BoolOpenXmlAttribute insertColumns;
        private readonly BoolOpenXmlAttribute insertRows;
        private readonly BoolOpenXmlAttribute deleteColumns;
        private readonly BoolOpenXmlAttribute deleteRows;
        private readonly BoolOpenXmlAttribute autoFilter;
        private readonly BoolOpenXmlAttribute sort;

        public SheetProtectionElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.algorithmName = this.RegisterAttribute<string>("algorithmName", string.Empty);
            this.hashValue = this.RegisterAttribute<string>("hashValue", string.Empty);
            this.saltValue = this.RegisterAttribute<string>("saltValue", string.Empty);
            this.spinCount = this.RegisterAttribute(new IntOpenXmlAttribute("spinCount"));
            this.password = this.RegisterAttribute<string>("password", string.Empty);
            this.sheet = this.RegisterAttribute(new BoolOpenXmlAttribute("sheet"));
            this.formatCells = this.RegisterAttribute(new BoolOpenXmlAttribute("formatCells", true, isRequired: false));
            this.formatColumns = this.RegisterAttribute(new BoolOpenXmlAttribute("formatColumns", true, isRequired: false));
            this.formatRows = this.RegisterAttribute(new BoolOpenXmlAttribute("formatRows", true, isRequired: false));
            this.insertColumns = this.RegisterAttribute(new BoolOpenXmlAttribute("insertColumns", true, isRequired: false));
            this.insertRows = this.RegisterAttribute(new BoolOpenXmlAttribute("insertRows", true, isRequired: false));
            this.deleteColumns = this.RegisterAttribute(new BoolOpenXmlAttribute("deleteColumns", true, isRequired: false));
            this.deleteRows = this.RegisterAttribute(new BoolOpenXmlAttribute("deleteRows", true, isRequired: false));
            this.autoFilter = this.RegisterAttribute(new BoolOpenXmlAttribute("autoFilter", true, isRequired: false));
            this.sort = this.RegisterAttribute(new BoolOpenXmlAttribute("sort", true, isRequired: false));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SheetProtectionElementName;
            }
        }

        public bool Sheet
        {
            get
            {
                return this.sheet.Value;
            }
            set
            {
                this.sheet.Value = value;
            }
        }

        public string AlgorithmName
        {
            get
            {
                return this.algorithmName.Value;
            }
            set
            {
                this.algorithmName.Value = value;
            }
        }

        public string HashValue
        {
            get
            {
                return this.hashValue.Value;
            }
            set
            {
                this.hashValue.Value = value;
            }
        }

        public string SaltValue
        {
            get
            {
                return this.saltValue.Value;
            }
            set
            {
                this.saltValue.Value = value;
            }
        }

        public int SpinCount
        {
            get
            {
                return this.spinCount.Value;
            }
            set
            {
                this.spinCount.Value = value;
            }
        }

        public string Password
        {
            get
            {
                return this.password.Value;
            }
            set
            {
                this.password.Value = value;
            }
        }

        public bool FormatCells
        {
            get
            {
                return this.formatCells.Value;
            }
            set
            {
                this.formatCells.Value = value;
            }
        }

        public bool FormatColumns
        {
            get
            {
                return this.formatColumns.Value;
            }
            set
            {
                this.formatColumns.Value = value;
            }
        }

        public bool FormatRows
        {
            get
            {
                return this.formatRows.Value;
            }
            set
            {
                this.formatRows.Value = value;
            }
        }

        public bool InsertColumns
        {
            get
            {
                return this.insertColumns.Value;
            }
            set
            {
                this.insertColumns.Value = value;
            }
        }

        public bool InsertRows
        {
            get
            {
                return this.insertRows.Value;
            }
            set
            {
                this.insertRows.Value = value;
            }
        }

        public bool DeleteColumns
        {
            get
            {
                return this.deleteColumns.Value;
            }
            set
            {
                this.deleteColumns.Value = value;
            }
        }

        public bool DeleteRows
        {
            get
            {
                return this.deleteRows.Value;
            }
            set
            {
                this.deleteRows.Value = value;
            }
        }

        public bool AutoFilter
        {
            get
            {
                return this.autoFilter.Value;
            }
            set
            {
                this.autoFilter.Value = value;
            }
        }

        public bool Sort
        {
            get
            {
                return this.sort.Value;
            }
            set
            {
                this.sort.Value = value;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            return context.GetSheetProtectionInfo().Enforced;
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            WorksheetProtectionInfo info = context.GetSheetProtectionInfo();

            if (info.Enforced)
            {
                this.Sheet = true;

                if (!string.IsNullOrEmpty(info.Password))
                {
                    this.Password = info.Password;
                }
                else if (!string.IsNullOrEmpty(info.HashValue))
                {
                    this.AlgorithmName = info.AlgorithmName;
                    this.HashValue = info.HashValue;
                    this.SpinCount = info.SpinCount;

                    if (!string.IsNullOrEmpty(info.SaltValue))
                    {
                        this.SaltValue = info.SaltValue;
                    }
                }
            }

            if (this.formatCells.DefaultValue != info.FormatCells)
            {
                this.FormatCells = info.FormatCells;
            }

            if (this.formatColumns.DefaultValue != info.FormatColumns)
            {
                this.FormatColumns = info.FormatColumns;
            }

            if (this.formatRows.DefaultValue != info.FormatRows)
            {
                this.FormatRows = info.FormatRows;
            }

            if (this.insertColumns.DefaultValue != info.InsertColumns)
            {
                this.InsertColumns = info.InsertColumns;
            }

            if (this.insertRows.DefaultValue != info.InsertRows)
            {
                this.InsertRows = info.InsertRows;
            }

            if (this.deleteColumns.DefaultValue != info.DeleteColumns)
            {
                this.DeleteColumns = info.DeleteColumns;
            }

            if (this.deleteRows.DefaultValue != info.DeleteRows)
            {
                this.DeleteRows = info.DeleteRows;
            }

            if (this.autoFilter.DefaultValue != info.AutoFilter)
            {
                this.AutoFilter = info.AutoFilter;
            }

            if (this.sort.DefaultValue != info.Sort)
            {
                this.Sort = info.Sort;
            }
        }

        protected override void OnAfterRead(IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            WorksheetProtectionInfo info = new WorksheetProtectionInfo();

            info.Password = this.Password;
            info.AlgorithmName = this.AlgorithmName;
            info.HashValue = this.HashValue;
            info.SaltValue = this.SaltValue;
            info.SpinCount = this.SpinCount;
            info.Enforced = this.Sheet;
            info.FormatCells = this.FormatCells;
            info.FormatColumns = this.FormatColumns;
            info.FormatRows = this.FormatRows;
            info.InsertColumns = this.InsertColumns;
            info.InsertRows = this.InsertRows;
            info.DeleteColumns = this.DeleteColumns;
            info.DeleteRows = this.DeleteRows;
            info.AutoFilter = this.AutoFilter;
            info.Sort = this.Sort;

            context.ApplySheetProtectionInfo(info);
        }
    }
}
