﻿using System;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SortConditionElement : WorksheetElementBase
    {
        private readonly ConvertedOpenXmlAttribute<Ref> range;
        private readonly OpenXmlAttribute<string> customList;
        private readonly BoolOpenXmlAttribute descending;
        private readonly OpenXmlAttribute<string> sortBy;
        private readonly IntOpenXmlAttribute dxfId;

        public SortConditionElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.range = this.RegisterAttribute(new ConvertedOpenXmlAttribute<Ref>("ref", XlsxConverters.RefConverter, true));
            this.customList = this.RegisterAttribute<string>("customList", false);
            this.descending = this.RegisterAttribute(new BoolOpenXmlAttribute("descending"));
            this.sortBy = this.RegisterAttribute<string>("sortBy", Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types.SortBy.Value, false);
            this.dxfId = this.RegisterAttribute(new IntOpenXmlAttribute("dxfId", false));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SortConditionElementName;
            }
        }

        public Ref Range
        {
            get
            {
                return this.range.Value;
            }
            set
            {
                this.range.Value = value;
            }
        }

        public string CustomList
        {
            get
            {
                return this.customList.Value;
            }
            set
            {
                this.customList.Value = value;
            }
        }

        public bool Descending
        {
            get
            {
                return this.descending.Value;
            }
            set
            {
                this.descending.Value = value;
            }
        }

        public string SortBy
        {
            get
            {
                return this.sortBy.Value;
            }
            set
            {
                this.sortBy.Value = value;
            }
        }

        public int DxfId
        {
            get
            {
                return this.dxfId.Value;
            }
            set
            {
                this.dxfId.Value = value;
            }
        }

        internal void CopyPropertiesFrom(SortConditionInfo sortConditionInfo)
        {
            Guard.ThrowExceptionIfNull(sortConditionInfo, "sortConditionInfo");

            this.Range = new Ref(sortConditionInfo.Range);

            if (sortConditionInfo.Descending != this.descending.DefaultValue)
            {
                this.Descending = sortConditionInfo.Descending;
            }

            if (sortConditionInfo.SortBy != this.sortBy.DefaultValue)
            {
                this.SortBy = sortConditionInfo.SortBy;
            }

            if (sortConditionInfo.CustomList != this.customList.DefaultValue)
            {
                this.CustomList = sortConditionInfo.CustomList;
            }

            if (sortConditionInfo.DxfId.HasValue)
            {
                this.DxfId = (int)sortConditionInfo.DxfId;
            }
        }

        internal SortConditionInfo GetInfo()
        {
            SortConditionInfo info = new SortConditionInfo();
            
            info.CustomList = this.CustomList;
            info.Descending = this.Descending;
            info.Range = this.Range.ToCellRange();
            info.SortBy = this.SortBy;
            info.DxfId = this.DxfId;

            return info;
        }
    }
}