﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class SortStateElement : WorksheetElementBase
    {
        private readonly BoolOpenXmlAttribute caseSensitive;
        private readonly BoolOpenXmlAttribute columnSort;
        private readonly ConvertedOpenXmlAttribute<Ref> sortRange;
        private List<SortConditionInfo> conditions;

        public SortStateElement(XlsxPartsManager partsManager)
            : base(partsManager)
        {
            this.caseSensitive = this.RegisterAttribute(new BoolOpenXmlAttribute("caseSensitive"));
            this.columnSort = this.RegisterAttribute(new BoolOpenXmlAttribute("columnSort"));
            this.sortRange = this.RegisterAttribute(new ConvertedOpenXmlAttribute<Ref>("ref", XlsxConverters.RefConverter, true));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SortStateElementName;
            }
        }

        public bool CaseSensitive
        {
            get
            {
                return this.caseSensitive.Value;
            }
            set
            {
                this.caseSensitive.Value = value;
            }
        }

        public bool ColumnSort
        {
            get
            {
                return this.columnSort.Value;
            }
            set
            {
                this.columnSort.Value = value;
            }
        }

        public Ref SortRange
        {
            get
            {
                return this.sortRange.Value;
            }
            set
            {
                this.sortRange.Value = value;
            }
        }

        private List<SortConditionInfo> Conditions
        {
            get
            {
                if (this.conditions == null)
                {
                    this.conditions = new List<SortConditionInfo>();
                }

                return this.conditions;
            }
        }

        protected override bool ShouldExport(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            return context.GetSortStateInfo().Range != null;
        }

        protected override IEnumerable<XlsxElementBase> EnumerateChildElements(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            foreach (SortConditionInfo sortCondition in context.GetSortConditionInfos())
            {
                SortConditionElement sortConditionElement = this.CreateElement<SortConditionElement>(XlsxElementNames.SortConditionElementName);
                sortConditionElement.CopyPropertiesFrom(sortCondition);

                yield return sortConditionElement;
            }
        }

        protected override void OnBeforeWrite(IXlsxWorksheetExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            SortStateInfo info = context.GetSortStateInfo();
            if (info.Range != null)
            {
                this.SortRange = new Ref(info.Range);
            }
        }

        protected override void OnAfterReadChildElement(Import.IXlsxWorksheetImportContext context, OpenXmlElementBase childElement)
        {
            SortConditionElement sortConditionElement = childElement as SortConditionElement;
            if (sortConditionElement != null)
            {
                this.Conditions.Add(sortConditionElement.GetInfo());
            }
        }

        protected override void OnAfterRead(Import.IXlsxWorksheetImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            SortStateInfo sortStateInfo = new SortStateInfo();
            sortStateInfo.Range = this.SortRange.ToCellRange();
            sortStateInfo.Conditions = this.Conditions;

            context.ApplySortState(sortStateInfo);
        }

        protected override void ClearOverride()
        {
            this.conditions = null;
        }
    }
}
