﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Core;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Import;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets
{
    internal class WorksheetElement : XlsxPartRootElementBase
    {
        private readonly OpenXmlChildElement<SheetPropertiesElement> sheetProperties;
        private readonly OpenXmlChildElement<SheetDataElement> sheetData;
        private readonly OpenXmlChildElement<SheetProtectionElement> sheetProtection;
        private readonly OpenXmlChildElement<MergedCellsElement> mergedCells;
        private readonly OpenXmlChildElement<ColumnsElement> columns;
        private readonly OpenXmlChildElement<SheetFormatPropertiesElement> sheetFormatProperties;
        private readonly OpenXmlChildElement<AutoFilterElement> autoFilter;
        private readonly OpenXmlChildElement<SortStateElement> sortState;
        private readonly OpenXmlChildElement<HyperlinksElement> hyperlinks;
        private readonly OpenXmlChildElement<DrawingElement> drawing;
        private readonly OpenXmlChildElement<DimensionElement> dimension;
        private readonly OpenXmlChildElement<SheetViewsElement> sheetViews;
        private readonly OpenXmlChildElement<PrintOptionsElement> printOptions;
        private readonly OpenXmlChildElement<PageMarginsElement> pageMargins;
        private readonly OpenXmlChildElement<PageSetupElement> pageSetup;
        private readonly OpenXmlChildElement<HeaderFooterElement> headerFooter;
        private readonly OpenXmlChildElement<RowBreaksElement> rowBreaks;
        private readonly OpenXmlChildElement<ColumnBreaksElement> columnBreaks;
        private readonly OpenXmlChildElement<DataValidationsElement> dataValidations;
        private readonly OpenXmlChildElement<FutureFeatureDataStorageAreaElement> extList;

        public WorksheetElement(XlsxPartsManager partsManager, OpenXmlPartBase part) : base(partsManager, part)
        {
            this.sheetProperties = this.RegisterChildElement<SheetPropertiesElement>(XlsxElementNames.SheetPropertiesElementName);
            this.dimension = this.RegisterChildElement<DimensionElement>(XlsxElementNames.DimensionElementName);
            this.sheetViews = this.RegisterChildElement<SheetViewsElement>(XlsxElementNames.SheetViewsElementName);
            this.sheetFormatProperties = this.RegisterChildElement<SheetFormatPropertiesElement>(XlsxElementNames.SheetFormatPropertiesElementName);
            this.columns = this.RegisterChildElement<ColumnsElement>(XlsxElementNames.ColumnsElementName);
            this.sheetData = this.RegisterChildElement<SheetDataElement>(XlsxElementNames.SheetDataElementName);
            this.sheetProtection = this.RegisterChildElement<SheetProtectionElement>(XlsxElementNames.SheetProtectionElementName);
            this.autoFilter = this.RegisterChildElement<AutoFilterElement>(XlsxElementNames.AutoFilterElementName);
            this.sortState = this.RegisterChildElement<SortStateElement>(XlsxElementNames.SortStateElementName);
            this.mergedCells = this.RegisterChildElement<MergedCellsElement>(XlsxElementNames.MergedCellsElementName);
            this.hyperlinks = this.RegisterChildElement<HyperlinksElement>(XlsxElementNames.HyperlinksElementName);
            this.dataValidations = this.RegisterChildElement<DataValidationsElement>(XlsxElementNames.DataValidationsElementName);
            this.printOptions = this.RegisterChildElement<PrintOptionsElement>(XlsxElementNames.PrintOptionsElementName);
            this.pageMargins = this.RegisterChildElement<PageMarginsElement>(XlsxElementNames.PageMarginsElementName);
            this.pageSetup = this.RegisterChildElement<PageSetupElement>(XlsxElementNames.PageSetupElementName);
            this.headerFooter = this.RegisterChildElement<HeaderFooterElement>(XlsxElementNames.HeaderFooterElementName);
            this.rowBreaks = this.RegisterChildElement<RowBreaksElement>(XlsxElementNames.RowBreaksElementName);
            this.columnBreaks = this.RegisterChildElement<ColumnBreaksElement>(XlsxElementNames.ColumnBreaksElementName);
            this.drawing = this.RegisterChildElement<DrawingElement>(XlsxElementNames.DrawingElementName);
            this.extList = this.RegisterChildElement<FutureFeatureDataStorageAreaElement>(OpenXmlElementNames.FutureFeatureDataStorageAreaElementName);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.WorksheetElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.SpreadsheetMLNamespace;
            }
        }

        public override IEnumerable<OpenXmlNamespace> Namespaces
        {
            get
            {
                yield return OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace;
            }
        }

        protected override void OnBeforeWrite(Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Export.IXlsxWorkbookExportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.CreateElement(this.sheetProperties);
            this.CreateElement(this.dimension);
            this.CreateElement(this.sheetViews);
            this.CreateElement(this.sheetFormatProperties);
            this.CreateElement(this.columns);
            this.CreateElement(this.sheetData);
            this.CreateElement(this.sheetProtection);
            this.CreateElement(this.autoFilter);
            this.CreateElement(this.sortState);
            this.CreateElement(this.mergedCells);
            this.CreateElement(this.hyperlinks);
            this.CreateElement(this.dataValidations);
            this.CreateElement(this.printOptions);
            this.CreateElement(this.pageMargins);
            this.CreateElement(this.pageSetup);
            this.CreateElement(this.headerFooter);
            this.CreateElement(this.rowBreaks);
            this.CreateElement(this.columnBreaks);
            this.CreateElement(this.drawing);
            this.CreateElement(this.extList);
        }

        protected override void OnAfterRead(IXlsxWorkbookImportContext context)
        {
            Guard.ThrowExceptionIfNull(context, "context");

            this.ReleaseElement(this.sheetProperties);
            this.ReleaseElement(this.dimension);
            this.ReleaseElement(this.sheetViews);
            this.ReleaseElement(this.sheetFormatProperties);
            this.ReleaseElement(this.columns);
            this.ReleaseElement(this.sheetData);
            this.ReleaseElement(this.sheetProtection);
            this.ReleaseElement(this.autoFilter);
            this.ReleaseElement(this.sortState);
            this.ReleaseElement(this.mergedCells);
            this.ReleaseElement(this.hyperlinks);
            this.ReleaseElement(this.dataValidations);
            this.ReleaseElement(this.printOptions);
            this.ReleaseElement(this.pageMargins);
            this.ReleaseElement(this.pageSetup);
            this.ReleaseElement(this.headerFooter);
            this.ReleaseElement(this.rowBreaks);
            this.ReleaseElement(this.columnBreaks);
            this.ReleaseElement(this.drawing);
            this.ReleaseElement(this.extList);
        }
    }
}