﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class CellRefRange
    {
        private readonly CellRef fromCell;
        private readonly CellRef toCell;

        public CellRefRange(CellRange range)
            : this(new CellRef(range.FromIndex.RowIndex, range.FromIndex.ColumnIndex), new CellRef(range.ToIndex.RowIndex, range.ToIndex.ColumnIndex))
        {
        }

        public CellRefRange(CellRef fromCell, CellRef toCell)
        {
            this.fromCell = fromCell;
            this.toCell = toCell;
        }

        public CellRefRange(string cellRefRange)
        {
            CellRange cellRange;
            NameConverter.ConvertCellNameToCellRange(cellRefRange, out cellRange);

            this.fromCell = new CellRef(cellRange.FromIndex.RowIndex, cellRange.FromIndex.ColumnIndex);
            this.toCell = new CellRef(cellRange.ToIndex.RowIndex, cellRange.ToIndex.ColumnIndex);
        }

        public CellRef FromCell
        {
            get
            {
                return this.fromCell;
            }
        }

        public CellRef ToCell
        {
            get
            {
                return this.toCell;
            }
        }

        public CellRange ToCellRange()
        {
            return new CellRange(this.FromCell.ToCellIndex(), this.ToCell.ToCellIndex());
        }

        public override string ToString()
        {
            CellIndex fromIndex = this.FromCell.ToCellIndex();
            CellIndex toIndex = this.ToCell.ToCellIndex();
            string resultString = string.Empty;

            if (fromIndex == toIndex)
            {
                resultString = this.FromCell.ToString();
            }
            else
            {
                resultString = NameConverter.ConvertCellRangeToName(fromIndex, toIndex);
            }

            return resultString;
        }
    }
}
