﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal static class CellTypes
    {
        public static readonly string Boolean = "b";
        public static readonly string Error = "e";
        public static readonly string InlineString = "inlineStr";
        public static readonly string Number = "n";
        public static readonly string SharedString = "s";
        public static readonly string String = "str";
        public static readonly string Date = "d";

        private static readonly Dictionary<string, CellValueType> nameToCellValueType;
        private static readonly Dictionary<CellValueType, string> cellValueTypeToName;

        static CellTypes()
        {
            nameToCellValueType = new Dictionary<string, CellValueType>();
            cellValueTypeToName = new Dictionary<CellValueType, string>();

            RegisterPair(Boolean, CellValueType.Boolean);
            RegisterPair(Error, CellValueType.Error);
            RegisterPair(InlineString, CellValueType.RichText);
            RegisterPair(Number, CellValueType.Number);

            nameToCellValueType.Add(String, CellValueType.Text);
            cellValueTypeToName.Add(CellValueType.Text, SharedString);            
        }

        public static string GetCellTypeName(CellValueType valueType)
        {
            return cellValueTypeToName[valueType];
        }

        public static CellValueType GetCellValueType(string name)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            return nameToCellValueType[name];
        }

        private static void RegisterPair(string name, CellValueType value)
        {
            nameToCellValueType.Add(name, value);
            cellValueTypeToName.Add(value, name);
        }
    }
}