﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class ComparisonOperatorConverter : IStringConverter<ComparisonOperator>
    {
        public const ComparisonOperator DefaultExportComparisonOperator = ComparisonOperator.EqualsTo;

        private static readonly Dictionary<ComparisonOperator, string> comparisonOperatorToId;
        private static readonly Dictionary<string, ComparisonOperator> operatorIdToComparisonOperator;
        private static readonly string defaultId;

        static ComparisonOperatorConverter()
        {
            comparisonOperatorToId = new Dictionary<ComparisonOperator, string>();
            operatorIdToComparisonOperator = new Dictionary<string, ComparisonOperator>();

            AddId("equal", ComparisonOperator.EqualsTo);
            AddId("lessThan", ComparisonOperator.LessThan);
            AddId("lessThanOrEqual", ComparisonOperator.LessThanOrEqualsTo);
            AddId("notEqual", ComparisonOperator.NotEqualsTo);
            AddId("greaterThanOrEqual", ComparisonOperator.GreaterThanOrEqualsTo);
            AddId("greaterThan", ComparisonOperator.GreaterThan);
            AddId("between", ComparisonOperator.Between);
            AddId("notBetween", ComparisonOperator.NotBetween);

            defaultId = comparisonOperatorToId[DefaultExportComparisonOperator];
        } 

        public ComparisonOperator ConvertFromString(string value)
        {
            ComparisonOperator comparisonOperator;
            if (operatorIdToComparisonOperator.TryGetValue(value, out comparisonOperator))
            {
                return comparisonOperator;
            }

            return DefaultExportComparisonOperator;
        }

        public string ConvertToString(ComparisonOperator value)
        {
            string id;
            if (comparisonOperatorToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, ComparisonOperator comparisonOperator)
        {
            operatorIdToComparisonOperator[id] = comparisonOperator;

            if (!comparisonOperatorToId.ContainsKey(comparisonOperator))
            {
                comparisonOperatorToId[comparisonOperator] = id;
            }
        }
    }
}
