﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Contexts;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class DataValidationRuleTypeConverter : IStringConverter<DataValidationRuleType>
    {
        public const DataValidationRuleType DefaultExportDataValidationRuleType = DataValidationRuleType.None;

        private static readonly string Custom = "custom";
        private static readonly string Date = "date";
        private static readonly string Decimal = "decimal";
        private static readonly string List = "list";
        private static readonly string None = "none";
        private static readonly string TextLength = "textLength";
        private static readonly string Time = "time";
        private static readonly string Whole = "whole";

        private static readonly Dictionary<DataValidationRuleType, string> dataValidationRuleTypeToId;
        private static readonly Dictionary<string, DataValidationRuleType> typeIdToDataValidationRuleType;
        private static readonly string defaultId;

        static DataValidationRuleTypeConverter()
        {
            dataValidationRuleTypeToId = new Dictionary<DataValidationRuleType, string>();
            typeIdToDataValidationRuleType = new Dictionary<string, DataValidationRuleType>();

            AddId(Custom, DataValidationRuleType.Custom);
            AddId(Date, DataValidationRuleType.Date);
            AddId(Decimal, DataValidationRuleType.Decimal);
            AddId(List, DataValidationRuleType.List);
            AddId(None, DataValidationRuleType.None);
            AddId(TextLength, DataValidationRuleType.TextLength);
            AddId(Time, DataValidationRuleType.Time);
            AddId(Whole, DataValidationRuleType.Whole);

            defaultId = dataValidationRuleTypeToId[DefaultExportDataValidationRuleType];
        }

        public DataValidationRuleType ConvertFromString(string value)
        {
            DataValidationRuleType dataValidationRuleType;
            if (typeIdToDataValidationRuleType.TryGetValue(value, out dataValidationRuleType))
            {
                return dataValidationRuleType;
            }

            return DefaultExportDataValidationRuleType;
        }

        public string ConvertToString(DataValidationRuleType value)
        {
            string id;
            if (dataValidationRuleTypeToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, DataValidationRuleType dataValidationRuleType)
        {
            typeIdToDataValidationRuleType[id] = dataValidationRuleType;

            if (!dataValidationRuleTypeToId.ContainsKey(dataValidationRuleType))
            {
                dataValidationRuleTypeToId[dataValidationRuleType] = id;
            }
        }
    }
}
