﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class DateTimeGroupingTypeConverter : IStringConverter<DateTimeGroupingType>
    {
        public const DateTimeGroupingType DefaultExportDateTimeGroupingType = DateTimeGroupingType.Year;

        private static readonly Dictionary<DateTimeGroupingType, string> dateTimeGroupingTypeToId;
        private static readonly Dictionary<string, DateTimeGroupingType> typeIdToDateTimeGroupingType;
        private static readonly string defaultId;

        static DateTimeGroupingTypeConverter()
        {
            dateTimeGroupingTypeToId = new Dictionary<DateTimeGroupingType, string>();
            typeIdToDateTimeGroupingType = new Dictionary<string, DateTimeGroupingType>();

            AddId("year", DateTimeGroupingType.Year);
            AddId("month", DateTimeGroupingType.Month);
            AddId("day", DateTimeGroupingType.Day);
            AddId("hour", DateTimeGroupingType.Hour);
            AddId("minute", DateTimeGroupingType.Minute);
            AddId("second", DateTimeGroupingType.Second);

            defaultId = dateTimeGroupingTypeToId[DefaultExportDateTimeGroupingType];
        }

        public DateTimeGroupingType ConvertFromString(string value)
        {
            DateTimeGroupingType dateTimeGroupingType;
            if (typeIdToDateTimeGroupingType.TryGetValue(value, out dateTimeGroupingType))
            {
                return dateTimeGroupingType;
            }

            return DefaultExportDateTimeGroupingType;
        }

        public string ConvertToString(DateTimeGroupingType value)
        {
            string id;
            if (dateTimeGroupingTypeToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, DateTimeGroupingType dateTimeGroupingType)
        {
            typeIdToDateTimeGroupingType[id] = dateTimeGroupingType;

            if (!dateTimeGroupingTypeToId.ContainsKey(dateTimeGroupingType))
            {
                dateTimeGroupingTypeToId[dateTimeGroupingType] = id;
            }
        }
    }
}
