﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class DynamicFilterTypeConverter : IStringConverter<DynamicFilterType>
    {
        public const DynamicFilterType DefaultExportDynamicFilterType = DynamicFilterType.None;

        private static readonly Dictionary<DynamicFilterType, string> dynamicFilterTypeToId;
        private static readonly Dictionary<string, DynamicFilterType> filterIdToDynamicFilterType;
        private static readonly string defaultId;

        static DynamicFilterTypeConverter()
        {
            dynamicFilterTypeToId = new Dictionary<DynamicFilterType, string>();
            filterIdToDynamicFilterType = new Dictionary<string, DynamicFilterType>();

            AddId("null", DynamicFilterType.None);
            AddId("aboveAverage", DynamicFilterType.AboveAverage);
            AddId("belowAverage", DynamicFilterType.BelowAverage);
            AddId("tomorrow", DynamicFilterType.Tomorrow);
            AddId("today", DynamicFilterType.Today);
            AddId("yesterday", DynamicFilterType.Yesterday);
            AddId("nextWeek", DynamicFilterType.NextWeek);
            AddId("thisWeek", DynamicFilterType.ThisWeek);
            AddId("lastWeek", DynamicFilterType.LastWeek);
            AddId("nextMonth", DynamicFilterType.NextMonth);
            AddId("thisMonth", DynamicFilterType.ThisMonth);
            AddId("lastMonth", DynamicFilterType.LastMonth);
            AddId("nextQuarter", DynamicFilterType.NextQuarter);
            AddId("thisQuarter", DynamicFilterType.ThisQuarter);
            AddId("lastQuarter", DynamicFilterType.LastQuarter);
            AddId("nextYear", DynamicFilterType.NextYear);
            AddId("thisYear", DynamicFilterType.ThisYear);
            AddId("lastYear", DynamicFilterType.LastYear);
            AddId("yearToDate", DynamicFilterType.YearToDate);
            AddId("Q1", DynamicFilterType.Quarter1);
            AddId("Q2", DynamicFilterType.Quarter2);
            AddId("Q3", DynamicFilterType.Quarter3);
            AddId("Q4", DynamicFilterType.Quarter4);
            AddId("M1", DynamicFilterType.January);
            AddId("M2", DynamicFilterType.February);
            AddId("M3", DynamicFilterType.March);
            AddId("M4", DynamicFilterType.April);
            AddId("M5", DynamicFilterType.May);
            AddId("M6", DynamicFilterType.June);
            AddId("M7", DynamicFilterType.July);
            AddId("M8", DynamicFilterType.August);
            AddId("M9", DynamicFilterType.September);
            AddId("M10", DynamicFilterType.October);
            AddId("M11", DynamicFilterType.November);
            AddId("M12", DynamicFilterType.December);

            defaultId = dynamicFilterTypeToId[DefaultExportDynamicFilterType];
        }

        public DynamicFilterType ConvertFromString(string value)
        {
            DynamicFilterType dynamicFilterType;
            if (filterIdToDynamicFilterType.TryGetValue(value, out dynamicFilterType))
            {
                return dynamicFilterType;
            }

            return DefaultExportDynamicFilterType;
        }

        public string ConvertToString(DynamicFilterType value)
        {
            string id;
            if (dynamicFilterTypeToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, DynamicFilterType dynamicFilterType)
        {
            filterIdToDynamicFilterType[id] = dynamicFilterType;

            if (!dynamicFilterTypeToId.ContainsKey(dynamicFilterType))
            {
                dynamicFilterTypeToId[dynamicFilterType] = id;
            }
        }
    }
}
