﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class ErrorStyleConverter : IStringConverter<ErrorStyle>
    {
        public const ErrorStyle DefaultExportErrorStyle = ErrorStyle.Stop;

        private static readonly string Stop = "stop";
        private static readonly string Warning = "warning";
        private static readonly string Information = "information";

        private static readonly Dictionary<ErrorStyle, string> errorStyleToId;
        private static readonly Dictionary<string, ErrorStyle> typeIdToErrorStyle;
        private static readonly string defaultId;

        static ErrorStyleConverter()
        {
            errorStyleToId = new Dictionary<ErrorStyle, string>();
            typeIdToErrorStyle = new Dictionary<string, ErrorStyle>();

            AddId(Stop, ErrorStyle.Stop);
            AddId(Warning, ErrorStyle.Warning);
            AddId(Information, ErrorStyle.Information);

            defaultId = errorStyleToId[DefaultExportErrorStyle];
        }

        public ErrorStyle ConvertFromString(string value)
        {
            ErrorStyle errorStyle;
            if (typeIdToErrorStyle.TryGetValue(value, out errorStyle))
            {
                return errorStyle;
            }

            return DefaultExportErrorStyle;
        }

        public string ConvertToString(ErrorStyle value)
        {
            string id;
            if (errorStyleToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, ErrorStyle errorStyle)
        {
            typeIdToErrorStyle[id] = errorStyle;

            if (!errorStyleToId.ContainsKey(errorStyle))
            {
                errorStyleToId[errorStyle] = id;
            }
        }
    }
}
