﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal static class HorizontalAlignments
    {
        public static readonly string Center = "center";
        public static readonly string CenterContinuous = "centerContinuous";
        public static readonly string Distributed = "distributed";
        public static readonly string Fill = "fill";
        public static readonly string General = "general";
        public static readonly string Justify = "justify";
        public static readonly string Left = "left";
        public static readonly string Right = "right";

        private static readonly ValueMapper<string, RadHorizontalAlignment> nameToValueMapper;

        static HorizontalAlignments()
        {
            nameToValueMapper = new ValueMapper<string, RadHorizontalAlignment>();
            nameToValueMapper.AddPair(Center, RadHorizontalAlignment.Center);
            nameToValueMapper.AddPair(CenterContinuous, RadHorizontalAlignment.CenterContinuous);
            nameToValueMapper.AddPair(Distributed, RadHorizontalAlignment.Distributed);
            nameToValueMapper.AddPair(Fill, RadHorizontalAlignment.Fill);
            nameToValueMapper.AddPair(General, RadHorizontalAlignment.General);
            nameToValueMapper.AddPair(Justify, RadHorizontalAlignment.Justify);
            nameToValueMapper.AddPair(Left, RadHorizontalAlignment.Left);
            nameToValueMapper.AddPair(Right, RadHorizontalAlignment.Right);
        }

        public static string GetHorizontalAlignmentName(RadHorizontalAlignment horizontalAlignment)
        {
            return nameToValueMapper.GetFromValue(horizontalAlignment);
        }

        public static RadHorizontalAlignment GetHorizontalAlignment(string name)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            return nameToValueMapper.GetToValue(name);
        }
    }
}