﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Types;
using Telerik.Windows.Documents.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class PaperTypesConverter : IStringConverter<PaperTypes>
    {
        public const PaperTypes DefaultExportPaperType = PaperTypes.Letter;

        private static readonly Dictionary<PaperTypes, string> paperTypesToId;
        private static readonly Dictionary<string, PaperTypes> paperIdToPaperTypes;
        private static readonly string defaultId;
        
        static PaperTypesConverter()
        {
            paperTypesToId = new Dictionary<PaperTypes, string>();
            paperIdToPaperTypes = new Dictionary<string, PaperTypes>();

            AddId("1", PaperTypes.Letter);
            AddId("2", PaperTypes.Letter);
            AddId("3", PaperTypes.Tabloid);
            AddId("4", PaperTypes.Ledger);
            AddId("5", PaperTypes.Legal);
            AddId("6", PaperTypes.Statement);
            AddId("7", PaperTypes.Executive);
            AddId("8", PaperTypes.A3);
            AddId("9", PaperTypes.A4);
            AddId("10", PaperTypes.A4);
            AddId("11", PaperTypes.A5);
            AddId("12", PaperTypes.B4);
            AddId("13", PaperTypes.B5);
            AddId("14", PaperTypes.Folio);
            AddId("15", PaperTypes.Quarto);

            AddId("42", PaperTypes.B4);

            AddId("50", PaperTypes.Letter);
            AddId("51", PaperTypes.Legal);
            AddId("52", PaperTypes.Tabloid);
            AddId("53", PaperTypes.A4);

            AddId("59", PaperTypes.Letter);
            AddId("60", PaperTypes.A4);

            AddId("63", PaperTypes.A3);
            AddId("64", PaperTypes.A5);
            AddId("65", PaperTypes.B5);
            AddId("66", PaperTypes.A2);

            defaultId = paperTypesToId[DefaultExportPaperType];
        }

        public PaperTypes ConvertFromString(string value)
        {
            PaperTypes paperType;
            if (paperIdToPaperTypes.TryGetValue(value, out paperType))
            {
                return paperType;
            }

            return DefaultExportPaperType;
        }

        public string ConvertToString(PaperTypes value)
        {
            string id;
            if (paperTypesToId.TryGetValue(value, out id))
            {
                return id;
            }

            return defaultId;
        }

        private static void AddId(string id, PaperTypes paperType)
        {
            paperIdToPaperTypes[id] = paperType;

            if (!paperTypesToId.ContainsKey(paperType))
            {
                paperTypesToId[paperType] = id;
            }
        }
    }
}
