﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal static class PatternTypes
    {
        public static readonly string DarkDown = "darkDown";
        public static readonly string DarkGray = "darkGray";
        public static readonly string DarkGrid = "darkGrid";
        public static readonly string DarkHorizontal = "darkHorizontal";
        public static readonly string DarkTrellis = "darkTrellis";
        public static readonly string DarkUp = "darkUp";
        public static readonly string DarkVertical = "darkVertical";
        public static readonly string Gray0625 = "gray0625";
        public static readonly string Gray125 = "gray125";
        public static readonly string LightDown = "lightDown";
        public static readonly string LightGray = "lightGray";
        public static readonly string LightGrid = "lightGrid";
        public static readonly string LightHorizontal = "lightHorizontal";
        public static readonly string LightTrellis = "lightTrellis";
        public static readonly string LightUp = "lightUp";
        public static readonly string LightVertical = "lightVertical";
        public static readonly string MediumGray = "mediumGray";
        public static readonly string None = "none";
        public static readonly string Solid = "solid";

        private static readonly ValueMapper<string, PatternType> nameToValueMapper;

        static PatternTypes()
        {
            nameToValueMapper = new ValueMapper<string, PatternType>();

            nameToValueMapper.AddPair(Solid, PatternType.Solid);
            nameToValueMapper.AddPair(DarkGray, PatternType.Gray75Percent);
            nameToValueMapper.AddPair(MediumGray, PatternType.Gray50Percent);
            nameToValueMapper.AddPair(LightGray, PatternType.Gray25Percent);
            nameToValueMapper.AddPair(Gray125, PatternType.Gray12Percent);
            nameToValueMapper.AddPair(Gray0625, PatternType.Gray6Percent);
            nameToValueMapper.AddPair(DarkHorizontal, PatternType.HorizontalStripe);
            nameToValueMapper.AddPair(DarkVertical, PatternType.VerticalStripe);
            nameToValueMapper.AddPair(DarkDown, PatternType.ReverseDiagonalStripe);
            nameToValueMapper.AddPair(DarkUp, PatternType.DiagonalStripe);
            nameToValueMapper.AddPair(DarkGrid, PatternType.DiagonalCrosshatch);
            nameToValueMapper.AddPair(DarkTrellis, PatternType.ThickDiagonalCrosshatch);
            nameToValueMapper.AddPair(LightHorizontal, PatternType.ThinHorizontalStripe);
            nameToValueMapper.AddPair(LightVertical, PatternType.ThinVerticalStripe);
            nameToValueMapper.AddPair(LightDown, PatternType.ThinReverseDiagonalStripe);
            nameToValueMapper.AddPair(LightUp, PatternType.ThinDiagonalStripe);
            nameToValueMapper.AddPair(LightGrid, PatternType.ThinHorizontalCrossHatch);
            nameToValueMapper.AddPair(LightTrellis, PatternType.ThinDiagonalCrosshatch);
        }

        public static string GetPatternTypeName(PatternType pattern)
        {
            return nameToValueMapper.GetFromValue(pattern);
        }

        public static PatternType GetPatternType(string name)
        {
            Guard.ThrowExceptionIfNullOrEmpty(name, "name");

            return nameToValueMapper.GetToValue(name);
        }
    }
}