﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Types
{
    internal class Ref
    {
        private readonly CellRef startCellRef;
        private readonly CellRef endCellRef;

        public Ref(CellRange range)
        {
            this.startCellRef = new CellRef(range.FromIndex.RowIndex, range.FromIndex.ColumnIndex);
            this.endCellRef = new CellRef(range.ToIndex.RowIndex, range.ToIndex.ColumnIndex);
        }

        public Ref(CellRef startCellRef, CellRef endCellRef)
        {
            this.startCellRef = startCellRef;
            this.endCellRef = endCellRef;
        }

        public Ref(string range)
        {
            Guard.ThrowExceptionIfNullOrEmpty(range, "range");

            string[] rangeParts = range.Split(new string[] { SpreadsheetCultureHelper.Colon }, StringSplitOptions.RemoveEmptyEntries);
            
            this.startCellRef = new CellRef(rangeParts[0]);

            this.endCellRef = rangeParts.Length > 1 ? new CellRef(rangeParts[1]) : new CellRef(rangeParts[0]);
        }

        public CellRef StartCellRef
        {
            get
            {
                return this.startCellRef;
            }
        }

        public CellRef EndCellRef
        {
            get
            {
                return this.endCellRef;
            }
        }

        public CellRange ToCellRange()
        {
            return new CellRange(this.StartCellRef.ToCellIndex(), this.EndCellRef.ToCellIndex());
        }

        public override string ToString()
        {
            return NameConverter.ConvertCellRangeToName(this.StartCellRef.ToCellIndex(), this.EndCellRef.ToCellIndex());
        }
    }
}