﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Common;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements.Drawing;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Common;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Drawing;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.SharedStrings;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.StyleSheet;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Workbooks;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.Worksheets;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
using SST = Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Elements.SharedStrings;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model
{
    internal static class XlsxElementsFactory
    {
        private static readonly Dictionary<string, Func<XlsxPartsManager, OpenXmlElementBase>> elementNameToFactoryMethod;

        static XlsxElementsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<XlsxPartsManager, OpenXmlElementBase>>();

            RegisterFactoryMethod(XlsxElementNames.StringItemElementName, (pm) => new StringItemElement(pm));
            RegisterFactoryMethod(XlsxElementNames.TextElementName, (pm) => new SST.TextElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RichTextRunElementName, (pm) => new SST.RichTextRunElement(pm));
            RegisterFactoryMethod(XlsxElementUniquePoolIds.TextElementUniquePoolId, (pm) => new SST.TextElement(pm));
            RegisterFactoryMethod(XlsxElementUniquePoolIds.RichTextRunElementUniquePoolId, (pm) => new SST.RichTextRunElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RichTextInlineElementName, (pm) => new RichTextInlineElement(pm));
            RegisterFactoryMethod(XlsxElementNames.UnderlineElementName, (pm) => new UnderlineElement(pm));
            RegisterFactoryMethod(XlsxElementNames.VerticalAlignmentElementName, (pm) => new VerticalAlignmentElement(pm));
            RegisterFactoryMethod(XlsxElementNames.AlignmentElementName, (pm) => new AlignmentElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ProtectionElementName, (pm) => new ProtectionElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BackgroundColorElementName, (pm) => new BackgroundColorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BoldElementName, (pm) => new BoldElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BorderElementName, (pm) => new BorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.LeftBorderElementName, (pm) => new LeftBorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RightBorderElementName, (pm) => new RightBorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.TopBorderElementName, (pm) => new TopBorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BottomBorderElementName, (pm) => new BottomBorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DiagonalBorderElementName, (pm) => new DiagonalBorderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BordersElementName, (pm) => new BordersElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColorElementName, (pm) => new ColorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellFormatsElementName, (pm) => new CellFormatsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FormatElementName, (pm) => new FormatElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellStyleElementName, (pm) => new CellStyleElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellStyleFormatsElementName, (pm) => new CellStyleFormatsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellStylesElementName, (pm) => new CellStylesElement(pm));
            RegisterFactoryMethod(XlsxElementNames.IndexedColorsElementName, (pm) => new IndexedColorsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.GradientFillElementName, (pm) => new GradientFillElement(pm));
            RegisterFactoryMethod(XlsxElementNames.PatternFillElementName, (pm) => new PatternFillElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FillElementName, (pm) => new FillElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FillsElementName, (pm) => new FillsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FontNameElementName, (pm) => new FontNameElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FontFamilyElementName, (pm) => new FontFamilyElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ItalicElementName, (pm) => new ItalicElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FontSizeElementName, (pm) => new FontSizeElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SchemeElementName, (pm) => new SchemeElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FontElementName, (pm) => new FontElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FontsElementName, (pm) => new FontsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ForegroundColorElementName, (pm) => new ForegroundColorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.StopElementName, (pm) => new StopElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RgbColorElementName, (pm) => new RgbColorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.NumberFormatElementName, (pm) => new NumberFormatElement(pm));
            RegisterFactoryMethod(XlsxElementUniquePoolIds.NumberFormatElementUniquePoolId, (pm) => new NumberFormatElement(pm));
            RegisterFactoryMethod(XlsxElementNames.NumberFormatsElementName, (pm) => new NumberFormatsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColorsElementName, (pm) => new ColorsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetElementName, (pm) => new SheetElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetsElementName, (pm) => new SheetsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BookViewsElementName, (pm) => new BookViewsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.WorkbookViewElementName, (pm) => new WorkbookViewElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FormulaElementName, (pm) => new FormulaElement(pm));
            RegisterFactoryMethod(XlsxElementUniquePoolIds.FormulaElementUniquePoolId, (pm) => new FormulaElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DefinedNamesElementName, (pm) => new DefinedNamesElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DefinedNameElementName, (pm) => new DefinedNameElement(pm));
            RegisterFactoryMethod(XlsxElementNames.HyperlinksElementName, (pm) => new HyperlinksElement(pm));
            RegisterFactoryMethod(XlsxElementNames.HyperlinkElementName, (pm) => new HyperlinkElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellValueElementName, (pm) => new CellValueElement(pm));
            RegisterFactoryMethod(XlsxElementUniquePoolIds.CellValueElementUniquePoolId, (pm) => new CellValueElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColumnElementName, (pm) => new ColumnElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColumnsElementName, (pm) => new ColumnsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.MergedCellElementName, (pm) => new MergedCellElement(pm));
            RegisterFactoryMethod(XlsxElementNames.MergedCellsElementName, (pm) => new MergedCellsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RowElementName, (pm) => new RowElement(pm));
            RegisterFactoryMethod(XlsxElementNames.PaneElementName, (pm) => new PaneElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SelectionElementName, (pm) => new SelectionElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetViewElementName, (pm) => new SheetViewElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetViewsElementName, (pm) => new SheetViewsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetDataElementName, (pm) => new SheetDataElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetProtectionElementName, (pm) => new SheetProtectionElement(pm));
            RegisterFactoryMethod(XlsxElementNames.WorkbookProtectionElementName, (pm) => new WorkbookProtectionElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetFormatPropertiesElementName, (pm) => new SheetFormatPropertiesElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CellElementName, (pm) => new CellElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DifferentialFormatsElementName, (pm) => new DifferentialFormatsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DifferentialFormatElementName, (pm) => new DifferentialFormatElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DimensionElementName, (pm) => new DimensionElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SqRefElementName, (pm) => new SqRefElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SheetPropertiesElementName, (pm) => new SheetPropertiesElement(pm));
            RegisterFactoryMethod(XlsxElementNames.OutlinePropertiesElementName, (pm) => new OutlinePropertiesElement(pm));
            RegisterFactoryMethod(XlsxElementNames.PageSetUpPropertiesElementName, (pm) => new PageSetUpPropertiesElement(pm));

            // Printing
            RegisterFactoryMethod(XlsxElementNames.PrintOptionsElementName, (pm) => new PrintOptionsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.PageMarginsElementName, (pm) => new PageMarginsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.PageSetupElementName, (pm) => new PageSetupElement(pm));
            RegisterFactoryMethod(XlsxElementNames.RowBreaksElementName, (pm) => new RowBreaksElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColumnBreaksElementName, (pm) => new ColumnBreaksElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BreakElementName, (pm) => new BreakElement(pm));
            RegisterFactoryMethod(XlsxElementNames.HeaderFooterElementName, (pm) => new HeaderFooterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.OddHeaderElementName, (pm) => new OddHeaderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.OddFooterElementName, (pm) => new OddFooterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.EvenHeaderElementName, (pm) => new EvenHeaderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.EvenFooterElementName, (pm) => new EvenFooterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FirstHeaderElementName, (pm) => new FirstHeaderElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FirstFooterElementName, (pm) => new FirstFooterElement(pm));

            // Filtering
            RegisterFactoryMethod(XlsxElementNames.AutoFilterElementName, (pm) => new AutoFilterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FilterColumnElementName, (pm) => new FilterColumnElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FiltersElementName, (pm) => new FiltersElement(pm));
            RegisterFactoryMethod(XlsxElementNames.FilterElementName, (pm) => new FilterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DateGroupItemElementName, (pm) => new DateGroupItemElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CustomFiltersElementName, (pm) => new CustomFiltersElement(pm));
            RegisterFactoryMethod(XlsxElementNames.CustomFilterElementName, (pm) => new CustomFilterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.Top10FilterElementName, (pm) => new Top10FilterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DynamicFilterElementName, (pm) => new DynamicFilterElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ColorFilterElementName, (pm) => new ColorFilterElement(pm));

            // Sorting
            RegisterFactoryMethod(XlsxElementNames.SortStateElementName, (pm) => new SortStateElement(pm));
            RegisterFactoryMethod(XlsxElementNames.SortConditionElementName, (pm) => new SortConditionElement(pm));

            // Drawing
            RegisterFactoryMethod(XlsxElementNames.ColumnOffset, (pm) => new OneDimentionOffsetElement(pm, XlsxElementNames.ColumnOffset, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(XlsxElementNames.RowOffset, (pm) => new OneDimentionOffsetElement(pm, XlsxElementNames.RowOffset, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(XlsxElementNames.From, (pm) => new MarkerElement(pm, XlsxElementNames.From));
            RegisterFactoryMethod(XlsxElementNames.To, (pm) => new MarkerElement(pm, XlsxElementNames.To));
            RegisterFactoryMethod(XlsxElementNames.AbsoluteAnchorElementName, (pm) => new AbsoluteAnchorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.Position, (pm) => new OffsetElement(pm, XlsxElementNames.Position, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(XlsxElementNames.OneCellAnchorElementName, (pm) => new OneCellAnchorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.TwoCellAnchorElementName, (pm) => new TwoCellAnchorElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DrawingElementName, (pm) => new DrawingElement(pm));
            RegisterFactoryMethod(XlsxElementNames.Picture, (pm) => new PictureElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ClientData, (pm) => new ClientDataElement(pm));
            RegisterFactoryMethod(XlsxElementNames.BlipFill, (pm) => new BlipFillElement(pm));
            RegisterFactoryMethod(XlsxElementNames.ShapePropertiesElementName, (pm) => new ShapePropertiesElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualPicturePropertiesElementName, (pm) => new NonVisualPicturePropertiesElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualPictureDrawingPropertiesElementName, (pm) => new NonVisualPictureDrawingPropertiesElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualDrawingPropertiesElementName, (pm) => new NonVisualDrawingPropertiesElement(pm, OpenXmlElementNames.NonVisualDrawingPropertiesElementName, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.GraphicFrame, (pm) => new GraphicFrameElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualGraphicFramePropertiesElementName, (pm) => new NonVisualGraphicFramePropertiesElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));
            RegisterFactoryMethod(OpenXmlElementNames.NonVisualGraphicFrameDrawingPropertiesElementName, (pm) => new NonVisualGraphicFrameDrawingPropertiesElement(pm, OpenXmlNamespaces.SpreadsheetDrawingMLNamespace));

            // DataValidation
            RegisterFactoryMethod(XlsxElementNames.DataValidationsElementName, (pm) => new DataValidationsElement(pm));
            RegisterFactoryMethod(XlsxElementNames.DataValidationElementName, (pm) => new DataValidationElement(pm));
            RegisterFactoryMethod(XlsxElementNames.Formula1ElementName, (pm) => new Formula1Element(pm));
            RegisterFactoryMethod(XlsxElementNames.Formula2ElementName, (pm) => new Formula2Element(pm));

            // FutureFeatureDataStorageArea
            RegisterFactoryMethod(OpenXmlElementNames.FutureFeatureDataStorageAreaElementName, (pm) => new FutureFeatureDataStorageAreaElement(pm));
        }

        public static OpenXmlElementBase CreateInstance(string elementName, XlsxPartsManager partsManager)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");

            return elementNameToFactoryMethod[elementName](partsManager);
        }

        public static bool CanCreateInstance(string elementName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");

            return elementNameToFactoryMethod.ContainsKey(elementName);
        }

        private static void RegisterFactoryMethod(string elementName, Func<XlsxPartsManager, OpenXmlElementBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(elementName, factoryMethod);
        }
    }
}