﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model
{
    internal static class XlsxPartsFactory
    {
        private static readonly Dictionary<string, Func<XlsxPartsManager, string, XlsxPartBase>> elementNameToFactoryMethod;

        static XlsxPartsFactory()
        {
            elementNameToFactoryMethod = new Dictionary<string, Func<XlsxPartsManager, string, XlsxPartBase>>();

            RegisterFactoryMethod(XlsxContentTypeNames.SharedStringsContentType, (pm, pn) => new SharedStringsPart(pm, pn));
            RegisterFactoryMethod(XlsxContentTypeNames.StylesContentType, (pm, pn) => new StylesPart(pm, pn));
            RegisterFactoryMethod(XlsxContentTypeNames.WorkbookContentType, (pm, pn) => new WorkbookPart(pm, pn));
            RegisterFactoryMethod(XlsxContentTypeNames.MacroEnabledWorkbookContentType, (pm, pn) => new WorkbookPart(pm, pn));
            RegisterFactoryMethod(XlsxContentTypeNames.WorksheetContentType, (pm, pn) => new WorksheetPart(pm, pn));
            RegisterFactoryMethod(XlsxContentTypeNames.DrawingContentType, (pm, pn) => new DrawingPart(pm, pn));
        }

        public static XlsxPartBase CreateInstance(string partType, XlsxPartsManager partsManager, string partName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(partsManager, "partsManager");

            Func<XlsxPartsManager, string, XlsxPartBase> constructor;
            if (!elementNameToFactoryMethod.TryGetValue(partType, out constructor))
            {
                return null;
            }

            return constructor(partsManager, partName);
        }

        public static bool CanCreateInstance(string partType)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");

            return elementNameToFactoryMethod.ContainsKey(partType);
        }

        private static void RegisterFactoryMethod(string partType, Func<XlsxPartsManager, string, XlsxPartBase> factoryMethod)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNull(factoryMethod, "factoryMethod");

            elementNameToFactoryMethod.Add(partType, factoryMethod);
        }
    }
}
