﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Common.Model.Data;
using Telerik.Windows.Documents.Core.Data;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Elements;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model.Parts;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Model
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
    internal class XlsxPartsManager : OpenXmlPartsManager
    {
        private readonly XlsxElementsPool elementsPool;
 
        public XlsxPartsManager()
        {
            this.elementsPool = new XlsxElementsPool(this);
        }

        public override OpenXmlElementBase CreateElement(string elementName, OpenXmlPartBase part)
        {
            Guard.ThrowExceptionIfNullOrEmpty(elementName, "elementName");
            Guard.ThrowExceptionIfNull(part, "part");

            return this.elementsPool.CreateElement(elementName, part);
        }

        public override OpenXmlPartBase CreatePart(string partType, string partName)
        {
            Guard.ThrowExceptionIfNullOrEmpty(partType, "partType");
            Guard.ThrowExceptionIfNullOrEmpty(partName, "partName");

            if (XlsxPartsFactory.CanCreateInstance(partType))
            {
                return XlsxPartsFactory.CreateInstance(partType, this, partName);
            }

            return base.CreatePart(partType, partName);
        }

        public override void ReleaseElement(OpenXmlElementBase element)
        {
            Guard.ThrowExceptionIfNull(element, "element");

            this.elementsPool.ReleaseElement(element);
        }

        public string CreateWorkbookRelationship(string target, string type, string targetMode = null)
        {
            Guard.ThrowExceptionIfNullOrEmpty(target, "target");
            Guard.ThrowExceptionIfNullOrEmpty(type, "type");

            return this.CreateRelationship(XlsxPartNames.WorkbookPartName, target, type, targetMode);
        }

        public string GetWorkbookRelationshipId(string target)
        {
            Guard.ThrowExceptionIfNullOrEmpty(target, "target");

            return this.GetRelationshipId(XlsxPartNames.WorkbookPartName, target);
        }

        public string CreateWorksheetRelationship(WorksheetPart worksheetPart, string target, string type, string targetMode = null)
        {
            Guard.ThrowExceptionIfNull(worksheetPart, "worksheetPart");
            Guard.ThrowExceptionIfNullOrEmpty(target, "target");
            Guard.ThrowExceptionIfNullOrEmpty(type, "type");

            return this.CreateRelationship(worksheetPart.Name, target, type, targetMode);
        }

        public string GetWorksheetRelationshipTarget(WorksheetPart worksheetPart, string relationshipId)
        {
            Guard.ThrowExceptionIfNull(worksheetPart, "worksheetPart");
            Guard.ThrowExceptionIfNullOrEmpty(relationshipId, "relationshipId");

            return this.GetRelationshipTarget(worksheetPart.Name, relationshipId);
        }
    }
}