﻿using System;
using System.Collections.Generic;
using System.Globalization;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings.Builders;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx.Utilities
{
    internal static class BuiltInNumberFormatsGenerator
    {
        public static Dictionary<string, int> GetBuiltInFormatsForCurrentCulture()
        {
            Dictionary<string, int> builtInFormats = new Dictionary<string, int>();

            RegisterBuiltInFormat(builtInFormats, FormatHelper.GeneralFormatString, 0);
            RegisterBuiltInFormat(builtInFormats, "0", 1);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(2, false, 0), 2);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(0, true, 0), 3);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(2, true, 0), 4);
            RegisterBuiltInFormat(builtInFormats, GetCurrencyFormat(0, 2), 5);
            RegisterBuiltInFormat(builtInFormats, GetCurrencyFormat(0, 3), 6);
            RegisterBuiltInFormat(builtInFormats, GetCurrencyFormat(2, 2), 7);
            RegisterBuiltInFormat(builtInFormats, GetCurrencyFormat(2, 3), 8);
            RegisterBuiltInFormat(builtInFormats, "0%", 9);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormatWithPercent(2, false, 0), 10);
            RegisterBuiltInFormat(builtInFormats, GetScientificFormat1(), 11);
            RegisterBuiltInFormat(builtInFormats, "# ?/?", 12);
            RegisterBuiltInFormat(builtInFormats, "# ??/??", 13);
            RegisterBuiltInFormat(builtInFormats, GetShortDateFormat(), 14);
            RegisterBuiltInFormat(builtInFormats, GetDayMonthYearFormat(), 15);
            RegisterBuiltInFormat(builtInFormats, GetDayMonthFormat(), 16);
            RegisterBuiltInFormat(builtInFormats, GetMonthYearFormat(), 17);
            RegisterBuiltInFormat(builtInFormats, GetHourMinuteAMPMFormat(), 18);
            RegisterBuiltInFormat(builtInFormats, GetHourMinuteSecondAMPMFormat(), 19);
            RegisterBuiltInFormat(builtInFormats, GetHourMinuteFormat(), 20);
            RegisterBuiltInFormat(builtInFormats, GetHourMinuteSecondFormat(), 21);
            RegisterBuiltInFormat(builtInFormats, GetShortDateHourMinuteFormat(), 22);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(0, true, 2), 37);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(0, true, 3), 38);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(2, true, 2), 39);
            RegisterBuiltInFormat(builtInFormats, GetNumberFormat(2, true, 3), 40);
            RegisterBuiltInFormat(builtInFormats, BuildersHelper.GetCultureDependentFormatCode("_(*#,##0_);_(*(#,##0);_(* \"-\"_);_(@_)"), 41);
            RegisterBuiltInFormat(builtInFormats, BuildersHelper.GetCultureDependentFormatCode("_($*#,##0_);_($*(#,##0);_($* \"-\"_);_(@_)"), 42);
            RegisterBuiltInFormat(builtInFormats, BuildersHelper.GetCultureDependentFormatCode("_(*#,##0.00_);_(*(#,##0.00);_(*\"-\"??_);_(@_)"), 43);
            RegisterBuiltInFormat(builtInFormats, BuildersHelper.GetCultureDependentFormatCode("_($*#,##0.00_);_($*(#,##0.00);_($*\"-\"??_);_(@_)"), 44);
            RegisterBuiltInFormat(builtInFormats, GetMonthSecondFormat(), 45);
            RegisterBuiltInFormat(builtInFormats, GetTotalHoursMonthSecondFormat(), 46);
            RegisterBuiltInFormat(builtInFormats, GetMonthSecondMillisecondFormat(), 47);
            RegisterBuiltInFormat(builtInFormats, GetScientificFormat2(), 48);
            RegisterBuiltInFormat(builtInFormats, "@", 49);

            return builtInFormats;
        }

        private static void RegisterBuiltInFormat(Dictionary<string, int> builtInFormats, string format, int id)
        {
            builtInFormats[format] = id;
        }

        private static string GetNumberFormat(int decimalPlaces, bool useThousandSeparator, int formatNumber)
        {
            return NumberFormatStringBuilder.BuildFormatStrings(decimalPlaces, useThousandSeparator)[formatNumber];
        }

        private static string GetCurrencyFormat(int decimalPlaces, int formatNumber)
        {
            string currencySymbol = FormatHelper.DefaultSpreadsheetCulture.CultureInfo.NumberFormat.CurrencySymbol;
            return BuildersHelper.GetCurrencyStringsForCurrentCulture(FormatHelper.DefaultSpreadsheetCulture.CultureInfo, currencySymbol, decimalPlaces, true)[formatNumber];
        }

        private static string GetNumberFormatWithPercent(int decimalPlaces, bool useThousandSeparator, int formatNumber)
        {
            return NumberFormatStringBuilder.BuildFormatStrings(decimalPlaces, useThousandSeparator)[formatNumber] + "%";
        }

        private static string GetScientificFormat1()
        {
            return string.Format("0{0}00E+00", FormatHelper.DefaultSpreadsheetCulture.NumberDecimalSeparator);
        }

        private static string GetShortDateFormat()
        {
            return FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern.ToLower();
        }

        private static string GetDayMonthYearFormat()
        {
            return string.Format("d{0}mmm{0}yy", FormatHelper.DefaultSpreadsheetCulture.DateSeparator);
        }

        private static string GetDayMonthFormat()
        {
            return string.Format("d{0}mmm", FormatHelper.DefaultSpreadsheetCulture.DateSeparator);
        }

        private static string GetMonthYearFormat()
        {
            return string.Format("mmm{0}yy", FormatHelper.DefaultSpreadsheetCulture.DateSeparator);
        }

        private static string GetHourMinuteAMPMFormat()
        {
            return string.Format("h{0}mm AM/PM", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetHourMinuteSecondAMPMFormat()
        {
            return string.Format("h{0}mm{0}ss AM/PM", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetHourMinuteFormat()
        {
            return string.Format("h{0}mm", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetHourMinuteSecondFormat()
        {
            return string.Format("h{0}mm{0}ss", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetShortDateHourMinuteFormat()
        {
            return string.Format("{0} {1}", GetShortDateFormat(), GetHourMinuteFormat());
        }

        private static string GetMonthSecondFormat()
        {
            return string.Format("mm{0}ss", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetTotalHoursMonthSecondFormat()
        {
            return string.Format("[h]{0}mm{0}ss", FormatHelper.DefaultSpreadsheetCulture.TimeSeparator);
        }

        private static string GetMonthSecondMillisecondFormat()
        {
            return string.Format("mm{0}ss{1}0", 
                FormatHelper.DefaultSpreadsheetCulture.TimeSeparator,
                FormatHelper.DefaultSpreadsheetCulture.CultureInfo.NumberFormat.NumberDecimalSeparator);
        }

        private static string GetScientificFormat2()
        {
            return string.Format("##0{0}0E+0", FormatHelper.DefaultSpreadsheetCulture.NumberDecimalSeparator);
        }
    }
}
