﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers
{
    internal class PdfGradientFillRenderer : PdfRenderer<GradientFillRenderable>
    {
        public override void RenderOverride(GradientFillRenderable renderable, ViewportPaneType paneType)
        {
            this.Editor.GraphicProperties.StrokeThickness = 0;
            this.Editor.GraphicProperties.IsStroked = false;
            Size gradientSize = new Size(renderable.BoundingRectangle.Width, renderable.BoundingRectangle.Height);
            IEnumerable<LinearGradientBox> gradientBoxes = GradientsRenderHelper.GenerateGradientBoxes(renderable.GradientFill, renderable.ColorScheme, gradientSize);

            this.Editor.Position.Translate(renderable.BoundingRectangle.Left, renderable.BoundingRectangle.Top);

            foreach (LinearGradientBox gradientBox in gradientBoxes)
            {
                this.Editor.Position.Translate(gradientBox.BoundingBox.Left, gradientBox.BoundingBox.Top);

                Matrix gradientMatrix = new Matrix()
                    .ScaleMatrix(gradientBox.BoundingBox.Width, gradientBox.BoundingBox.Height)
                    .MultiplyBy(this.Editor.Position.Matrix);

                this.Editor.GraphicProperties.FillColor = gradientBox.Fill.ToPdfGradient(gradientMatrix);
                var path = this.CalculatePath(gradientBox);
                this.Editor.DrawPath(path);

                this.Editor.Position.Translate(-gradientBox.BoundingBox.Left, -gradientBox.BoundingBox.Top);
            }                        
        }

        private Telerik.Windows.Documents.Fixed.Model.Graphics.PathGeometry CalculatePath(LinearGradientBox gradientBox)
        {
            var path = new Telerik.Windows.Documents.Fixed.Model.Graphics.PathGeometry();

            foreach (var clipFigure in gradientBox.Clip.Figures)
            {
                var figure = path.Figures.AddPathFigure();
                figure.IsClosed = clipFigure.IsClosed;
                figure.StartPoint = clipFigure.StartPoint;

                foreach (PathSegment segment in clipFigure.Segments)
                {
                    LineSegment lineSegment = segment as LineSegment;
                    if (lineSegment != null)
                    {
                        figure.Segments.AddLineSegment().Point = lineSegment.Point;
                    }
                }
            }

            return path;
        }
    }
}
