﻿using System;
using System.Windows;
using Telerik.Windows.Documents.Fixed.Model.ColorSpaces;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers
{
    internal class PdfPatternFillRenderer : PdfRenderer<PatternFillRenderable>
    {
        public override void RenderOverride(PatternFillRenderable renderable, ViewportPaneType paneType)
        {
            this.Editor.GraphicProperties.StrokeThickness = 0;
            this.Editor.GraphicProperties.IsStroked = false;

            var background = renderable.PatternFill.GetBackgroundDependingOnPatternType().GetActualValue(renderable.ColorScheme).ToPdfRgbColor();

            if (renderable.PatternFill.PatternType == PatternType.Solid)
            {
                this.Editor.GraphicProperties.FillColor = background;                    
            }
            else
            {
                var foreground = renderable.PatternFill.PatternColor.GetActualValue(renderable.ColorScheme).ToPdfRgbColor();
                int[,] pattern = PatternsRenderHelper.GetPattern(renderable.PatternFill.PatternType);
                this.Editor.GraphicProperties.FillColor = PdfPatternFillRenderer.CalculateTiling(pattern, foreground, background);
            }            

            foreach (Rect rect in renderable.VisibleBoxes)
            {
                this.Editor.DrawRectangle(rect);
            }
        }

        private static Tiling CalculateTiling(int[,] pattern, Fixed.Model.ColorSpaces.RgbColor foreground, Fixed.Model.ColorSpaces.RgbColor background)
        {
            int rows = pattern.GetLength(0);
            int columns = pattern.GetLength(1);

            var tiling = new Tiling();
            tiling.BoundingBox = new Rect(0, 0, rows, columns);

            FixedContentEditor editor = new FixedContentEditor(tiling);
            editor.GraphicProperties.StrokeThickness = 0;
            editor.GraphicProperties.IsStroked = false;

            editor.GraphicProperties.FillColor = background;
            editor.DrawRectangle(tiling.BoundingBox);

            editor.GraphicProperties.FillColor = foreground;

            for (int row = 0; row < rows; row++)
            {
                for (int column = 0; column < columns; column++)
                {
                    if (pattern[row, column] == 1)
                    {
                        editor.DrawRectangle(new Rect(column, row, 1, 1));
                    }
                }
            }

            return tiling;
        }
    }
}
