﻿using System;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers
{
    internal abstract class PdfRowColumnHeadingRenderer<T> : PdfRenderer<RowColumnHeadingRenderable<T>>
        where T : LayoutBox
    {
        public sealed override void RenderOverride(RowColumnHeadingRenderable<T> renderable, ViewportPaneType paneType)
        {
            Point topLeftCorner = this.CalculateTopLeft(renderable);

            Matrix topLeftCornerEditorMatrix = new Matrix()
                .ScaleMatrix(renderable.ScaleFactor.Width, renderable.ScaleFactor.Height)
                .TranslateMatrix(topLeftCorner.X, topLeftCorner.Y)
                .MultiplyBy(this.Editor.Position.Matrix);

            this.Editor.Position = new MatrixPosition(topLeftCornerEditorMatrix);

            Point topLeft = new Point(0, 0);
            Point topRight = new Point(renderable.Rectangle.Width, 0);
            Point bottomLeft = new Point(0, renderable.Rectangle.Height);
            Point bottomRight = new Point(renderable.Rectangle.Width, renderable.Rectangle.Height);

            this.Editor.DrawLine(topLeft, topRight);
            this.Editor.DrawLine(topLeft, bottomLeft);
            this.Editor.DrawLine(topRight, bottomRight);
            this.Editor.DrawLine(bottomLeft, bottomRight);

            this.Editor.PushTransformedClipping(new Rect(0, 0, renderable.Rectangle.Width, renderable.Rectangle.Height));

            this.Editor.DrawTextBlock(renderable.Text, 
                renderable.Rectangle.Width, 
                renderable.Rectangle.Height, 
                Fixed.Model.Editing.Flow.HorizontalAlignment.Center, 
                Fixed.Model.Editing.Flow.VerticalAlignment.Center);

            this.Editor.PopClipping();
        }

        protected abstract Point CalculateTopLeft(RowColumnHeadingRenderable<T> renderable);
    }
}
