﻿using System;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers
{
    internal class PdfShapeRenderer : PdfRenderer<ShapeRenderable>
    {
#if !SILVERLIGHT
        public IPdfChartRenderer ChartRenderer { get; internal set; }
#endif

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1800:DoNotCastUnnecessarily")]
        public override void RenderOverride(ShapeRenderable renderable, ViewportPaneType paneType)
        {
            if (IsZeroSized(renderable))
            {
                return;
            }

            if (renderable is ImageRenderable)
            {
                this.RenderImage(renderable as ImageRenderable);
            }
#if !SILVERLIGHT
            else if (renderable is ChartRenderable)
            {
                this.RenderChart(renderable as ChartRenderable);
            }
#endif
        }

        private static bool IsZeroSized(ShapeRenderable renderable)
        {
            return renderable.Width == 0 || renderable.Height == 0;
        }

#if !SILVERLIGHT
        private void RenderChart(ChartRenderable chartRenderable)
        {
            IPdfChartRenderer renderer = this.ChartRenderer;

            if (renderer == null && ChartModelToImageConverterContainer.ChartModelToImageConverter != null)
            {
                renderer = new DefaultPdfChartToImageRenderer(ChartModelToImageConverterContainer.ChartModelToImageConverter);
            }

            if (renderer == null)
            {
                return;
            }            

            double x = chartRenderable.TopLeft.X;
            double y = chartRenderable.TopLeft.Y;
            double scaleX = chartRenderable.Width / chartRenderable.Shape.Width;
            double scaleY = chartRenderable.Height / chartRenderable.Shape.Height;

            Matrix imageMatrix = new Matrix()
                .TranslateMatrix(x, y)
                .ScaleMatrixAt(scaleX, scaleY, x, y)
                .MultiplyBy(this.Editor.Position.Matrix);

            this.Editor.Position = new MatrixPosition(imageMatrix);

            FloatingChartShape shape = (chartRenderable.ShapeSource as ChartPdfSource).ChartShape;
            renderer.RenderChart(this.Editor, shape);
        }
#endif

        private void RenderImage(ImageRenderable renderable)
        {
            BitmapShapeSource shapeSource = (BitmapShapeSource)renderable.ShapeSource;
            BitmapSource imageSource = shapeSource.Bitmap;

            double x = renderable.TopLeft.X;
            double y = renderable.TopLeft.Y;
            double scaleX = renderable.Width / imageSource.PixelWidth;
            double scaleY = renderable.Height / imageSource.PixelHeight;
            double centerX = x + (renderable.Width / 2);
            double centerY = y + (renderable.Height / 2);

            double flipX = renderable.Shape.IsHorizontallyFlipped ? -1 : 1;
            double flipY = renderable.Shape.IsVerticallyFlipped ? -1 : 1;
            double angle = renderable.Shape.RotationAngle;

            Matrix imageMatrix = new Matrix()
                .TranslateMatrix(x, y)
                .ScaleMatrixAt(scaleX, scaleY, x, y)
                .ScaleMatrixAt(flipX, flipY, centerX, centerY)
                .RotateMatrixAt(angle, centerX, centerY)
                .MultiplyBy(this.Editor.Position.Matrix);

            this.Editor.Position = new MatrixPosition(imageMatrix);

            this.Editor.DrawImage(new MemoryStream(renderable.Shape.ImageSource.Data));
        }
    }
}
