﻿using System;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers
{
    internal class PdfTextRenderer : PdfRenderer<TextBlockRenderable>
    {
        private static readonly string[] environmentNewLineSplitter = new string[] { "\r\n", "\n", "\r" };

        public override void RenderOverride(TextBlockRenderable renderable, ViewportPaneType paneType)
        {
            Matrix newMatrix = new Matrix()
                .ScaleMatrix(renderable.ScaleFactor.Width, renderable.ScaleFactor.Height)
                .TranslateMatrix(renderable.TopLeft.X * renderable.ScaleFactor.Width, renderable.TopLeft.Y * renderable.ScaleFactor.Height)
                .TranslateMatrix(renderable.Padding.Left * renderable.ScaleFactor.Width, renderable.Padding.Top * renderable.ScaleFactor.Height)
                .MultiplyBy(this.Editor.Position.Matrix);

            this.Editor.Position = new MatrixPosition(newMatrix);

            Block block = new Block();
            block.GraphicProperties.FillColor = renderable.Foreground.ToPdfRgbColor();

            TextProperties textProperties = block.TextProperties;
            textProperties.TrySetFont(renderable.FontFamily, renderable.FontStyle, renderable.FontWeight);
            textProperties.FontSize = renderable.FontSize;
            textProperties.UnderlinePattern = renderable.TextDecorations.ToPdfTextDecorations();

            Fixed.Model.Editing.Flow.HorizontalAlignment horizontalAlignment = Fixed.Model.Editing.Flow.HorizontalAlignment.Left;

            double wrappingWidth = double.PositiveInfinity;
            if (renderable.TextWrapping == TextWrapping.Wrap)
            {
                wrappingWidth = renderable.CellBox.Width - renderable.Padding.Left - renderable.Padding.Right;
                horizontalAlignment = renderable.TextAlignment.ToPdfHorizontalTextAlignment();
            }

            block.HorizontalAlignment = horizontalAlignment;

            Rect clipping = renderable.Clip;
            if (!clipping.IsEmpty)
            {
                clipping = new Rect(clipping.X - renderable.Padding.Left, clipping.Y, clipping.Width, clipping.Height);

                this.Editor.PushTransformedClipping(clipping);
                this.DrawText(renderable, block, wrappingWidth);
                this.Editor.PopClipping();
            }
            else
            {
                this.DrawText(renderable, block, wrappingWidth);
            }
        }

        internal static void InsertText(Block parentBlock, string text)
        {
            var textLines = text.Split(environmentNewLineSplitter, StringSplitOptions.None);
            for (int currentLineIndex = 0; currentLineIndex < textLines.Length; currentLineIndex++)
            {
                parentBlock.InsertText(textLines[currentLineIndex]);
                if (currentLineIndex < textLines.Length - 1)
                {
                    parentBlock.InsertLineBreak();
                }
            }
        }

        private void DrawText(TextBlockRenderable renderable, Block block, double wrappingWidth)
        {
            if (renderable.Text == null)
            {
                foreach (RunRenderable run in renderable.Runs)
                {
                    this.DrawRun(block, run);
                }
            }
            else
            {
                InsertText(block, renderable.Text);
            }

            this.Editor.DrawBlock(block, new Size(wrappingWidth, double.PositiveInfinity));
        }

        private void DrawRun(Block block, RunRenderable run)
        {
            using (block.SaveProperties())
            {
                if (run.FontFamily != null)
                {
                    block.TextProperties.TrySetFont(run.FontFamily);
                }

                if (run.FontSize != null)
                {
                    block.TextProperties.FontSize = run.FontSize.Value;
                }

                if (run.Foreground != null)
                {
                    block.GraphicProperties.FillColor = run.Foreground.Color.ToPdfRgbColor();
                }

                if (run.Foreground != null && run.Foreground.Color == Colors.Transparent)
                {
                    //// TODO: Handle invisible symbols.
                    //// Size measuredSize = this.Editor.MeasureText(run.Text, null);
                    //// this.Editor.TranslateBlockPosition(measuredSize.Width);
                }
                else
                {
                    InsertText(block, run.Text);
                }
            }
        }
    }
}