﻿using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;
using FixedExport = Telerik.Windows.Documents.Fixed.FormatProviders.Pdf.Export;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export
{
    /// <summary>
    /// Provides properties specifying what to export to PDF.
    /// </summary>
    public class PdfExportSettings : ExportWhatSettings
    {
        private readonly List<CellRange> selectedRanges;

        /// <summary>
        /// Initializes a new instance of the <see cref="PdfExportSettings" /> class.
        /// </summary>
        /// <param name="exportWhat">The ExportWhat option.</param>
        /// <param name="ignorePrintArea">The value indicating whether to ignore print area.</param>
        public PdfExportSettings(ExportWhat exportWhat, bool ignorePrintArea)
            : base(exportWhat, ignorePrintArea)
        {
            this.selectedRanges = new List<CellRange>();
            this.PdfFileSettings = new FixedExport.PdfExportSettings();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PdfExportSettings" /> class.
        /// </summary>
        /// <param name="selection">The ranges to export.</param>
        public PdfExportSettings(IEnumerable<CellRange> selection)
            : this(ExportWhat.Selection, true)
        {
            this.SelectedRanges.AddRange(selection);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PdfExportSettings" /> class.
        /// </summary>
        internal PdfExportSettings()
            : base()
        {
            this.selectedRanges = new List<CellRange>();
            this.PdfFileSettings = new FixedExport.PdfExportSettings();
        }

        /// <summary>
        /// Gets the selected ranges.
        /// </summary>
        /// <value>The selected ranges.</value>
        public List<CellRange> SelectedRanges
        {
            get
            {
                return this.selectedRanges;
            }
        }

        /// <summary>
        /// Gets or sets the PDF settings used when the <see cref="Telerik.Windows.Documents.Spreadsheet.Model.Workbook"/> is exported to a file.
        /// </summary>
        /// <value>The PDF export settings.</value>
        public FixedExport.PdfExportSettings PdfFileSettings { get; set; }

#if !SILVERLIGHT
        /// <summary>
        /// Gets or sets the renderer which will be used to render charts.
        /// </summary>
        public IPdfChartRenderer ChartRenderer { get; set; }
#endif
    }
}