﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Fixed.Model;
using Telerik.Windows.Documents.Fixed.Model.Data;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export.Layers;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf.Export
{
    internal class PdfExporter
    {
        private readonly PageLayout pageLayout;
        private readonly PdfLayerStack layers;
        private readonly PdfLayersBuilder layersBuilder;

        public PdfExporter()
        {
            this.pageLayout = new PageLayout();
            this.layers = new PdfLayerStack();
            this.layersBuilder = new PdfLayersBuilder();
        }

        public PdfLayersBuilder LayersBuilder
        {
            get
            {
                return this.layersBuilder;
            }
        }

        public Fixed.FormatProviders.Pdf.Export.PdfExportSettings PdfExportSettings { get; set; }

#if !SILVERLIGHT
        public IPdfChartRenderer ChartRenderer { get; set; }
#endif

        private PageLayout PageLayout
        {
            get
            {
                return this.pageLayout;
            }
        }

        private PdfLayerStack Layers
        {
            get
            {
                return this.layers;
            }
        }

        public void Export(Workbook workbook, bool ignorePrintArea, Stream output)
        {
            RadFixedDocument document = this.CreateFixedDocument(workbook, ignorePrintArea);
            this.Export(document, output);
        }

        public void Export(Worksheet worksheet, bool ignorePrintArea, Stream output)
        {
            RadFixedDocument document = this.CreateFixedDocument(worksheet, ignorePrintArea);
            this.Export(document, output);
        }

        public void Export(Worksheet worksheet, IEnumerable<CellRange> selectedRanges, Stream output)
        {
            RadFixedDocument document = this.CreateFixedDocument(worksheet, selectedRanges);
            this.Export(document, output);
        }

        public RadFixedDocument CreateFixedDocument(Workbook workbook, bool ignorePrintArea)
        {
            this.PageLayout.PreparePages(workbook, ignorePrintArea);
            return this.CreateFixedDocument();
        }

        public RadFixedDocument CreateFixedDocument(Worksheet worksheet, bool ignorePrintArea)
        {
            this.PageLayout.PreparePages(worksheet, ignorePrintArea);
            return this.CreateFixedDocument();
        }

        public RadFixedDocument CreateFixedDocument(Worksheet worksheet, IEnumerable<CellRange> selectedRanges)
        {
            this.PageLayout.PreparePages(worksheet, selectedRanges);
            return this.CreateFixedDocument();
        }

        private void Export(RadFixedDocument document, Stream output)
        {
            Fixed.FormatProviders.Pdf.PdfFormatProvider provider = new Fixed.FormatProviders.Pdf.PdfFormatProvider();
            provider.ExportSettings = this.PdfExportSettings;
            provider.Export(document, output);
        }
 
        private RadFixedDocument CreateFixedDocument()
        {
            this.Layers.Clear();
            this.LayersBuilder.BuildLayersOverride(this.Layers);

            RadFixedDocument document = new RadFixedDocument();

            foreach (WorksheetPageLayoutBox page in this.PageLayout.Pages)
            {
                this.RenderPage(document, page);
            }

            return document;
        }

        private void RenderPage(RadFixedDocument document, WorksheetPageLayoutBox worksheetPage)
        {
            WorksheetPageSetup pageSetup = worksheetPage.Worksheet.WorksheetPageSetup;

            RadFixedPage page = document.Pages.AddPage();
            page.Size = pageSetup.RotatedPageSize;
            FixedContentEditor editor = new FixedContentEditor(page, MatrixPosition.Default);

            RadWorksheetLayout layout = worksheetPage.Worksheet.Workbook.GetWorksheetLayout(worksheetPage.Worksheet, true);
            Point offset = new Point(worksheetPage.Left, worksheetPage.Top);
            SheetViewport sheetViewPort = SheetViewportFactory.CreateSheetViewport(layout, worksheetPage.Size, offset, worksheetPage.ScaleFactor);

#if !SILVERLIGHT
            PdfPrintWorksheetRenderUpdateContext updateContext = new PdfPrintWorksheetRenderUpdateContext(layout, sheetViewPort, worksheetPage.ScaleFactor, worksheetPage.HeaderFooterContext, this.ChartRenderer);
#else
            PdfPrintWorksheetRenderUpdateContext updateContext = new PdfPrintWorksheetRenderUpdateContext(layout, sheetViewPort, worksheetPage.ScaleFactor, worksheetPage.HeaderFooterContext);
#endif

            double leftOffset = worksheetPage.PageContentBox.Left + worksheetPage.RowColumnHeadingsSize.Width;
            double topOffset = worksheetPage.PageContentBox.Top + worksheetPage.RowColumnHeadingsSize.Height;
            editor.Position.Translate(leftOffset, topOffset);

            this.SetShouldRender(pageSetup.PrintOptions.PrintRowColumnHeadings, WorksheetPredefinedPdfLayers.RowHeadingMargin);
            this.SetShouldRender(pageSetup.PrintOptions.PrintRowColumnHeadings, WorksheetPredefinedPdfLayers.ColumnHeadingMargin);
            this.SetShouldRender(pageSetup.PrintOptions.PrintGridlines, WorksheetPredefinedPdfLayers.Gridlines);
            this.SetShouldRender(pageSetup.PrintOptions.PrintGridlinesOutline, WorksheetPredefinedPdfLayers.GridlinesOutline);

            this.Layers.UpdateRender(updateContext, editor);
        }

        private void SetShouldRender(bool shouldRender, string layerName)
        {
            PdfLayerBase layer = this.Layers.GetByName(layerName);

            if (layer != null)
            {
                layer.ShouldRender = shouldRender;
            }
        }
    }
}
