﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Fixed.Model.Editing;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.FormatProviders.Pdf
{
    internal static class PdfFormatProviderExtensions
    {
        public static Telerik.Windows.Documents.Fixed.Model.ColorSpaces.RgbColor ToPdfRgbColor(this Color color)
        {
            return new Telerik.Windows.Documents.Fixed.Model.ColorSpaces.RgbColor(color.A, color.R, color.G, color.B);
        }

        public static Telerik.Windows.Documents.Fixed.Model.ColorSpaces.GradientStop ToPdfGradientStop(this GradientStop gradientStop)
        {
            return new Telerik.Windows.Documents.Fixed.Model.ColorSpaces.GradientStop(gradientStop.Color.ToPdfRgbColor(), gradientStop.Offset);
        }

        public static Telerik.Windows.Documents.Fixed.Model.ColorSpaces.Gradient ToPdfGradient(this LinearGradientBrush linearGradientBrush, Matrix matrix)
        {
            Point gradientStartPoint = matrix.Transform(linearGradientBrush.StartPoint);
            Point gradientEndPoint = matrix.Transform(linearGradientBrush.EndPoint);
            var linearGradient = new Telerik.Windows.Documents.Fixed.Model.ColorSpaces.LinearGradient(gradientStartPoint, gradientEndPoint);
            IEnumerable<GradientStop> sortedStops = linearGradientBrush.GradientStops.OrderBy(stop => stop.Offset);

            foreach (GradientStop stop in sortedStops)
            {
                linearGradient.GradientStops.Add(stop.ToPdfGradientStop());
            }

            return linearGradient;
        }

        public static Telerik.Windows.Documents.Fixed.Model.Editing.Flow.HorizontalAlignment ToPdfHorizontalTextAlignment(this TextAlignment textAlignment)
        {
            switch (textAlignment)
            {
                case TextAlignment.Left:
                    return Telerik.Windows.Documents.Fixed.Model.Editing.Flow.HorizontalAlignment.Left;
                case TextAlignment.Right:
                    return Telerik.Windows.Documents.Fixed.Model.Editing.Flow.HorizontalAlignment.Right;
                default:
                    return Telerik.Windows.Documents.Fixed.Model.Editing.Flow.HorizontalAlignment.Center;
            }
        }

        public static Telerik.Windows.Documents.Fixed.Model.Editing.Flow.BaselineAlignment ToPdfBaselineAlignment(this BaselineAlignment baselineAlignment)
        {
            switch (baselineAlignment)
            {
                case BaselineAlignment.Superscript:
                    return Fixed.Model.Editing.Flow.BaselineAlignment.Superscript;
                case BaselineAlignment.Subscript:
                    return Fixed.Model.Editing.Flow.BaselineAlignment.Subscript;
                default:
                    return Fixed.Model.Editing.Flow.BaselineAlignment.Baseline;
            }
        }

        public static Telerik.Windows.Documents.Fixed.Model.Graphics.PathGeometry TransformRectangle(this Matrix matrix, Rect rectangle)
        {
            Point topLeft = matrix.Transform(new Point(rectangle.Left, rectangle.Top));
            Point topRight = matrix.Transform(new Point(rectangle.Right, rectangle.Top));
            Point bottomRight = matrix.Transform(new Point(rectangle.Right, rectangle.Bottom));
            Point bottomLeft = matrix.Transform(new Point(rectangle.Left, rectangle.Bottom));

            var path = new Telerik.Windows.Documents.Fixed.Model.Graphics.PathGeometry();
            var figure = path.Figures.AddPathFigure();
            figure.IsClosed = true;

            figure.StartPoint = topLeft;
            figure.Segments.AddLineSegment().Point = topRight;
            figure.Segments.AddLineSegment().Point = bottomRight;
            figure.Segments.AddLineSegment().Point = bottomLeft;

            return path;
        }

        public static Telerik.Windows.Documents.Fixed.Model.Editing.Flow.UnderlinePattern ToPdfUnderlinePattern(this UnderlineType underlineType)
        {
            if (underlineType == Model.UnderlineType.None)
            {
                return Fixed.Model.Editing.Flow.UnderlinePattern.None;
            }
            else
            {
                return Fixed.Model.Editing.Flow.UnderlinePattern.Single;
            }
        }

        public static Telerik.Windows.Documents.Fixed.Model.Editing.Flow.UnderlinePattern ToPdfTextDecorations(this TextDecorationCollection textDecorations)
        {
            if (textDecorations == TextDecorations.Underline)
            {
                return Telerik.Windows.Documents.Fixed.Model.Editing.Flow.UnderlinePattern.Single;
            }

            return Telerik.Windows.Documents.Fixed.Model.Editing.Flow.UnderlinePattern.None;
        }

        public static void PushTransformedClipping(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor, Rect rect)
        {
            editor.PushClipping(editor.Position.Matrix.TransformRectangle(rect));
        }

        public static Matrix PreserveStates(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor)
        {
            Matrix oldEditorMatrix = editor.Position.Matrix;
            editor.SaveProperties();

            return oldEditorMatrix;
        }

        public static void RestoreStates(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor, Matrix oldEditorMatrix)
        {
            editor.RestoreProperties();
            editor.Position = new Telerik.Windows.Documents.Fixed.Model.Data.MatrixPosition(oldEditorMatrix);
        }

        public static Size MeasureText(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor, string text, FontProperties fontProperties, double? wrappingWidth)
        {
            Block block = new Block();
            block.TextProperties.CopyFrom(editor.TextProperties);
            block.TextProperties.TrySetFont(fontProperties.FontFamily, fontProperties.FontStyle, fontProperties.FontWeight);
            block.TextProperties.FontSize = fontProperties.FontSize;
            block.InsertText(text);
            
            return block.Measure(new Size(wrappingWidth.HasValue ? wrappingWidth.Value : double.PositiveInfinity, double.PositiveInfinity));
        }

        public static Size MeasureText(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor, string text, double? wrappingWidth)
        {
            Block block = new Block();
            block.TextProperties.CopyFrom(editor.TextProperties);
            block.InsertText(text);

            return block.Measure(new Size(wrappingWidth.HasValue ? wrappingWidth.Value : double.PositiveInfinity, double.PositiveInfinity));
        }

        public static void DrawTextBlock(this Telerik.Windows.Documents.Fixed.Model.Editing.FixedContentEditor editor, string text, double width, double height, Fixed.Model.Editing.Flow.HorizontalAlignment horizontalAlignment, Fixed.Model.Editing.Flow.VerticalAlignment verticalAlignment)
        {
            Block block = new Block();
            block.HorizontalAlignment = horizontalAlignment;
            block.VerticalAlignment = verticalAlignment;
            block.TextProperties.CopyFrom(editor.TextProperties);
            block.GraphicProperties.CopyFrom(editor.GraphicProperties);
            block.InsertText(text);

            editor.DrawBlock(block, new Size(width, height));
        }
    }
}
