﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.CellValueTests
{
    [TestClass]
    public class ErrorCellValueTests
    {
        private static void AssertCellValueCorrectness(string inputString, string expectedString, CellValueType type)
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            Worksheet worksheet = TestHelper.CreateWorksheet();

            CellValueFactory.Create(inputString.ReplaceCultureDependentSeparators(), worksheet, new CellIndex(0, 0), CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            Assert.AreEqual(cellValue.ValueType, type);
            Assert.AreEqual<string>(expectedString.ReplaceCultureDependentSeparators(), cellValue.GetValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void CreateNullErrorCellValueTest()
        {
            string testValue = "#NULL!";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateDivisionByZeroErrorCellValueTest()
        {
            string testValue = "#DIV/0!";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateValueErrorCellValueTest()
        {
            string testValue = "#VALUE!";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateNameErrorCellValueTest()
        {
            string testValue = "#NAME?";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateReferenceErrorCellValueTest()
        {
            string testValue = "#REF!";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateNumberErrorCellValueTest()
        {
            string testValue = "#NUM!";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateNotAvailableErrorCellValueTest()
        {
            string testValue = "#N/A";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateNullErrorWithLowerCasingCellValueTest()
        {
            string testValue = "#null!";
            string expectedValue = "#NULL!";
            AssertCellValueCorrectness(testValue, expectedValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateNullErrorWithMixedCasingCellValueTest()
        {
            string testValue = "#Div/0!";
            string expectedValue = "#DIV/0!";
            AssertCellValueCorrectness(testValue, expectedValue, CellValueType.Error);
        }

        [TestMethod]
        public void CreateTextCellValueThatIsOnlyHashTest()
        {
            string testValue = "#";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Text);
        }

        [TestMethod]
        public void CreateTextCellValueContainingHashAndNumbersTest()
        {
            string testValue = "#203";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Text);
        }

        [TestMethod]
        public void CreateTextCellValueContainingHashAndTextTest()
        {
            string testValue = "#test";
            AssertCellValueCorrectness(testValue, testValue, CellValueType.Text);
        }
    }
}
