﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Charts;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ChartsTests
{
    [TestClass]
    public class BubbleChartTests
    {
        [TestMethod]
        public void AssertCreateBubbleChartFromDataRangeWithAutoOrientation()
        {
            Workbook workbook = GenerateChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 2, 2),
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateSingleRowBubbleChart()
        {
            Workbook workbook = GenerateChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 3, 0),
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateSingleColumnBubbleChart()
        {
            Workbook workbook = GenerateChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 0, 3),
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromDataRangeWithHorizontalOrientation()
        {
            Workbook workbook = GenerateChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 2, 2),
                SeriesRangesOrientation.Horizontal,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromDataRangeWithVerticalOrientation()
        {
            Workbook workbook = GenerateChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 2, 2),
                SeriesRangesOrientation.Vertical,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromLargeDataRangeWithAutoOrientation()
        {
            Workbook workbook = TestHelper.GetWorkbookFromFile("ChartTemplate.xlsx");

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 12, 5),
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(2, seriesGroup.Series.Count());
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromLargeDataRangeWithHorizontalOrientation()
        {
            Workbook workbook = TestHelper.GetWorkbookFromFile("ChartTemplate.xlsx");

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 12, 5),
                SeriesRangesOrientation.Horizontal,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(6, seriesGroup.Series.Count());
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromLargeDataRangeWithVerticalOrientation()
        {
            Workbook workbook = TestHelper.GetWorkbookFromFile("ChartTemplate.xlsx");

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 12, 5),
                SeriesRangesOrientation.Vertical,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(2, seriesGroup.Series.Count());
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromNumbersRangeOnlyHorizontal()
        {
            Workbook workbook = GenerateOnlyNumberChartData();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 2, 2),
                SeriesRangesOrientation.Horizontal,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromNumbersRangeOnlyVertical()
        {
            Workbook workbook = GenerateOnlyNumberChartData();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 2, 2),
                SeriesRangesOrientation.Vertical,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromLargeDataRangeOnlyHorizontal()
        {
            Workbook workbook = GenerateLargeChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 7, 2),
                SeriesRangesOrientation.Horizontal,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(3, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        [TestMethod]
        public void AssertCreateBubbleChartFromLargeDataRangeOnlyVertical()
        {
            Workbook workbook = GenerateLargeChartDataWithHeaders();

            FloatingChartShape chart = workbook.ActiveWorksheet.Shapes.AddChart(
                new CellIndex(0, 0),
                new CellRange(0, 0, 7, 2),
                SeriesRangesOrientation.Vertical,
                ChartType.Bubble);

            SeriesGroupCollection seriesGroups = chart.Chart.SeriesGroups;
            Assert.AreEqual(1, seriesGroups.Count());

            SeriesGroup seriesGroup = seriesGroups.First();
            Assert.AreEqual(1, seriesGroup.Series.Count());

            BubbleSeries series = (BubbleSeries)seriesGroup.Series.First();

            Assert.IsNotNull(series.XValues);
            Assert.IsNotNull(series.YValues);
            Assert.IsNotNull(series.BubbleSizes);
        }

        private static Workbook GenerateLargeChartDataWithHeaders()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            worksheet.Cells[0, 0].SetValue("Ads");
            worksheet.Cells[0, 1].SetValue("Sales");
            worksheet.Cells[0, 2].SetValue("Value");

            worksheet.Cells[1, 0].SetValue(20);
            worksheet.Cells[1, 1].SetValue(1000);
            worksheet.Cells[1, 2].SetValue(1);

            worksheet.Cells[2, 0].SetValue(30);
            worksheet.Cells[2, 1].SetValue(2000);
            worksheet.Cells[2, 2].SetValue(2);

            worksheet.Cells[3, 0].SetValue(15);
            worksheet.Cells[3, 1].SetValue(2500);
            worksheet.Cells[3, 2].SetValue(3);

            worksheet.Cells[4, 0].SetValue(12);
            worksheet.Cells[4, 1].SetValue(1800);
            worksheet.Cells[4, 2].SetValue(3);

            worksheet.Cells[5, 0].SetValue(40);
            worksheet.Cells[5, 1].SetValue(3000);
            worksheet.Cells[5, 2].SetValue(4);

            worksheet.Cells[6, 0].SetValue(12);
            worksheet.Cells[6, 1].SetValue(1000);
            worksheet.Cells[6, 2].SetValue(4);

            worksheet.Cells[7, 0].SetValue(18);
            worksheet.Cells[7, 1].SetValue(2700);
            worksheet.Cells[7, 2].SetValue(7);

            return workbook;
        }

        private static Workbook GenerateChartDataWithHeaders()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            worksheet.Cells[0, 0].SetValue("a");
            worksheet.Cells[0, 1].SetValue("b");
            worksheet.Cells[0, 2].SetValue("c");

            worksheet.Cells[1, 0].SetValue(1);
            worksheet.Cells[1, 1].SetValue(2);
            worksheet.Cells[1, 2].SetValue(3);

            worksheet.Cells[2, 0].SetValue(2);
            worksheet.Cells[2, 1].SetValue(3);
            worksheet.Cells[2, 2].SetValue(4);
            return workbook;
        }

        private static Workbook GenerateOnlyNumberChartData()
        {
            Workbook workbook = new Workbook();

            Worksheet worksheet = workbook.Worksheets.Add();
            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Cells[0, 1].SetValue(2);
            worksheet.Cells[0, 2].SetValue(3);

            worksheet.Cells[1, 0].SetValue(2);
            worksheet.Cells[1, 1].SetValue(3);
            worksheet.Cells[1, 2].SetValue(4);

            worksheet.Cells[2, 0].SetValue(3);
            worksheet.Cells[2, 1].SetValue(4);
            worksheet.Cells[2, 2].SetValue(5);
            return workbook;
        }
    }
}
