﻿using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    public struct CellRangeValue
    {
        #region Fields

        private CellPropertiesValues properties;

        #endregion


        #region Properties

        public CellRange CellRange { get; set; }

        public string Value { get; set; }

        public CellPropertiesValues Properties
        {
            get
            {
                if (this.properties == null)
                {
                    this.properties = new CellPropertiesValues();
                }

                return this.properties;
            }
            set
            {
                if (this.properties != value)
                {
                    this.properties = value;
                }
            }
        }

        #endregion
    }

    [TestClass]
    public class AutoFillMergedCellsTests
    {
        private static bool IsExpectedRangeMerged(CellRange cellRange)
        {
            return cellRange.ColumnCount > 1 || cellRange.RowCount > 1;
        }

        private static bool IsActualRangeMerged(Worksheet worksheet, CellRange cellRange)
        {
            CellRange mergedRange;
            if (!worksheet.Cells.TryGetContainingMergedRange(cellRange.FromIndex, out mergedRange))
            {
                return false;
            }

            return mergedRange.Equals(cellRange);
        }

        private static Worksheet AssertMergeStateAndValue(CellRangeValue[] initialValues, CellRange selectionRange, CellRangeValue[] expectedValues,
            CellOrientation cellOrientation = CellOrientation.Horizontal)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            foreach (CellRangeValue initialValue in initialValues)
            {
                CellSelection valueSelection = worksheet.Cells[initialValue.CellRange];
                valueSelection.Merge();
                valueSelection = worksheet.Cells[initialValue.CellRange.FromIndex];
                valueSelection.SetProperties(initialValue.Properties);
                valueSelection.SetValue(initialValue.Value);
            }

            CellSelection selection = worksheet.Cells[selectionRange];
            selection.FillDataSeriesAuto(cellOrientation, false);

            foreach (CellRangeValue expectedValue in expectedValues)
            {
                bool isExpectedRangeMerged = IsExpectedRangeMerged(expectedValue.CellRange);
                bool isActualRangeMerged = IsActualRangeMerged(worksheet, expectedValue.CellRange);

                Assert.AreEqual(isExpectedRangeMerged, isActualRangeMerged);

                CellSelection valueSelection = worksheet.Cells[expectedValue.CellRange.FromIndex];
                CellValueFormat format = valueSelection.GetFormat().Value;
                string actualValue = valueSelection.GetValue().Value.GetResultValueAsString(format);
                Assert.AreEqual(expectedValue.Value, actualValue);

                CellPropertiesValues cellProperties = valueSelection.GetFormatProperties();
                Assert.AreEqual(expectedValue.Properties, cellProperties);
            }

            return worksheet;
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest1()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                }
            };
            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 3),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 0, 5),
                    Value = "3",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 0, 7),
                    Value = "4",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 0, 9),
                    Value = "5",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 10, 0, 11),
                    Value = "6",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Center
                    }
                }
            };
            CellRange selectionRange = new CellRange(0, 0, 0, 11);
            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest2()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = new PatternFill(PatternType.Gray25Percent, Colors.Blue, Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Left,
                        ForeColor = new ThemableColor(Colors.Yellow)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 3),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Blue),
                        HorizontalAlignment = RadHorizontalAlignment.Center,
                        ForeColor = new ThemableColor(Colors.Red)
                    }
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = new PatternFill(PatternType.Gray25Percent, Colors.Blue, Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Left,
                        ForeColor = new ThemableColor(Colors.Yellow)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 3),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Blue),
                        HorizontalAlignment = RadHorizontalAlignment.Center,
                        ForeColor = new ThemableColor(Colors.Red)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 0, 5),
                    Value = "3",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = new PatternFill(PatternType.Gray25Percent, Colors.Blue, Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Left,
                        ForeColor = new ThemableColor(Colors.Yellow)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 0, 7),
                    Value = "4",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Blue),
                        HorizontalAlignment = RadHorizontalAlignment.Center,
                        ForeColor = new ThemableColor(Colors.Red)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 0, 9),
                    Value = "5",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = new PatternFill(PatternType.Gray25Percent, Colors.Blue, Colors.Red),
                        HorizontalAlignment = RadHorizontalAlignment.Left,
                        ForeColor = new ThemableColor(Colors.Yellow)
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 10, 0, 11),
                    Value = "6",
                    Properties = new CellPropertiesValues()
                    {
                        Fill = PatternFill.CreateSolidFill(Colors.Blue),
                        HorizontalAlignment = RadHorizontalAlignment.Center,
                        ForeColor = new ThemableColor(Colors.Red)
                    }
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 0, 11);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest3()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 4),
                    Value = "2"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 4),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 5, 0, 6),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 7, 0, 9),
                    Value = "4"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 10, 0, 11),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 12, 0, 14),
                    Value = "6"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 0, 14);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest4()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 2),
                    Value = "2"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 2),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 4),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 5, 0, 5),
                    Value = "4"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 0, 7),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 0, 8),
                    Value = "6"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 0, 8);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest5()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 2),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 5),
                    Value = "3"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 2),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 5),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 0, 7),
                    Value = (3.5).ToCultureDependentString()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 0, 8),
                    Value = (4.78571428571429).ToCultureDependentString()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 9, 0, 11),
                    Value = (5.42857142857143).ToCultureDependentString()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 12, 0, 13),
                    Value = (6.07142857142857).ToCultureDependentString()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 14, 0, 14),
                    Value = (7.35714285714286).ToCultureDependentString()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 15, 0, 17),
                    Value = "8"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 0, 17);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest6()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 1, 5),
                    Value = "3"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 1),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 2),
                    Value = string.Empty
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 1, 5),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 1, 7),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 0, 8),
                    Value = string.Empty
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 9, 1, 11),
                    Value = "7"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 12, 1, 13),
                    Value = "9"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 14, 0, 14),
                    Value = string.Empty
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 15, 1, 17),
                    Value = "11"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 1, 17);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest7()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "2"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 1, 2),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 3),
                    Value = "4"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 1, 4),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 5, 0, 5),
                    Value = "6"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 1, 6),
                    Value = "7"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 7, 0, 7),
                    Value = "8"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 8, 1, 8),
                    Value = "9"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 1, 8);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest8()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "2"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 1, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 1, 2),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 3),
                    Value = "4"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 1, 4),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 5, 0, 5),
                    Value = "6"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 1, 6),
                    Value = "7"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 7, 0, 7),
                    Value = "8"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 1, 7);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest9()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 1, 2),
                    Value = "2"
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 0),
                    Value = "1"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 1, 2),
                    Value = "2"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 3, 0, 3),
                    Value = "3"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 1, 5),
                    Value = "4"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 6, 0, 6),
                    Value = "5"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 7, 1, 8),
                    Value = "6"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 9, 0, 9),
                    Value = "7"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 10, 1, 11),
                    Value = "8"
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 12, 0, 12),
                    Value = "9"
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 1, 12);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest10()
        {
            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 2, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Bad"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 3),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Good"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 2, 2, 2),
                    Value = "3",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Neutral"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 0, 4),
                    Value = "4",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Calculation"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 3, 2, 4),
                    Value = "5",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Check Cell"
                    }
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 2, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Bad"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 2, 0, 3),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Good"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 2, 2, 2),
                    Value = "3",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Neutral"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 4, 0, 4),
                    Value = "4",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Calculation"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 3, 2, 4),
                    Value = "5",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Check Cell"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 5, 2, 6),
                    Value = (4.58333333333333).ToCultureDependentString(),
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Bad"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 7, 0, 8),
                    Value = (6.08333333333333).ToCultureDependentString(),
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Good"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 7, 2, 7),
                    Value = "7",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Neutral"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 8, 2, 9),
                    Value = "9",
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Check Cell"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 9, 0, 9),
                    Value = (7.58333333333333).ToCultureDependentString(),
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Calculation"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 10, 2, 11),
                    Value = (8.33333333333333).ToCultureDependentString(),
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Bad"
                    }
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 12, 0, 13),
                    Value = (9.83333333333333).ToCultureDependentString(),
                    Properties = new CellPropertiesValues()
                    {
                        StyleName = "Good"
                    }
                }
            };

            CellRange selectionRange = new CellRange(0, 0, 2, 13);

            AssertMergeStateAndValue(initialValues, selectionRange, expectedValues);
        }

        [TestMethod]
        public void AutoFillMergedCellsValuesTest11()
        {
            CellRange initiallyMergedRange = new CellRange(4, 0, 4, 1);

            CellRangeValue[] initialValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 0),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 0, 1, 1),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = initiallyMergedRange,
                    Properties = new CellPropertiesValues()
                }
            };

            CellRangeValue[] expectedValues = new CellRangeValue[]
            {
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 0, 0, 0),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(0, 1, 0, 1),
                    Value = "1",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 0, 1, 1),
                    Value = "2",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(1, 1, 1, 1),
                    Value = string.Empty,
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(2, 0, 2, 0),
                    Value = "3",
                    Properties = new CellPropertiesValues()
                },
                //new CellRangeValue()
                //{
                //    CellRange = new CellRange(2, 1, 2, 1),
                //    Value = "1",
                //    Properties = new CellPropertiesValues()
                //},
                new CellRangeValue()
                {
                    CellRange = new CellRange(3, 0, 3, 1),
                    Value = "4",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(3, 1, 3, 1),
                    Value = string.Empty,
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(4, 0, 4, 0),
                    Value = "5",
                    Properties = new CellPropertiesValues()
                },
                //new CellRangeValue()
                //{
                //    CellRange = new CellRange(4, 1, 4, 1),
                //    Value = "1",
                //    Properties = new CellPropertiesValues()
                //},
                new CellRangeValue()
                {
                    CellRange = new CellRange(5, 0, 5, 1),
                    Value = "6",
                    Properties = new CellPropertiesValues()
                },
                new CellRangeValue()
                {
                    CellRange = new CellRange(5, 1, 5, 1),
                    Value = string.Empty,
                    Properties = new CellPropertiesValues()
                },
            };

            CellRange selectionRange = new CellRange(0, 0, 5, 1);

            Worksheet worksheet = AssertMergeStateAndValue(initialValues, selectionRange, expectedValues, CellOrientation.Vertical);

            bool isInitiallyMergedRangeMerged = IsActualRangeMerged(worksheet, initiallyMergedRange);
            Assert.IsFalse(isInitiallyMergedRangeMerged);
        }

    }
}