﻿using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    [TestClass]
    public class AutoFillPropertiesTests
    {
        #region One Initial Value Tests

        [TestMethod]
        public void AutoFillOneInitialValueTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Borders = new CellBorders(
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.DashDotDot, new ThemableColor(ThemeColorType.Accent2)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent4)),
                         null,
                         null,
                         new CellBorder(CellBorderStyle.MediumDashDot, new ThemableColor(ThemeColorType.Background1)),
                         new CellBorder(CellBorderStyle.MediumDashDotDot, new ThemableColor(ThemeColorType.Background2)))
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal,
                new CellPropertiesValues()
                {
                    Borders = new CellBorders(
                        new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                        CellBorder.Default,
                        CellBorder.Default,
                        CellBorder.Default,
                        null,
                        null,
                        CellBorder.Default,
                        CellBorder.Default)
                });
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest3()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Fill = new PatternFill(PatternType.Gray12Percent, new ThemableColor(ThemeColorType.Accent1), new ThemableColor(ThemeColorType.Accent2))
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest4()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     FontFamily = new ThemableFontFamily(ThemeFontType.Major)
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest5()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     FontSize = 18
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest6()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     ForeColor = new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4)
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest7()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Format = new CellValueFormat("$#,##0_);[Red]($#,##0)")
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest8()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                    HorizontalAlignment = RadHorizontalAlignment.Left,
                    Indent = 5
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest9()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsBold = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest10()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsItalic = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest11()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsWrapped = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest12()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     StyleName = "Good"
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest13()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Underline = UnderlineType.Single
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest14()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Bottom
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest15()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Top
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest16()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     Indent = 5
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillOneInitialValueTest17()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     Indent = 5
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        #endregion


        #region Two Initial Values Tests

        [TestMethod]
        public void AutoFillTwoInitialValuesTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Borders = new CellBorders(
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.DashDotDot, new ThemableColor(ThemeColorType.Accent2)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent4)),
                         null,
                         null,
                         new CellBorder(CellBorderStyle.MediumDashDot, new ThemableColor(ThemeColorType.Background1)),
                         new CellBorder(CellBorderStyle.MediumDashDotDot, new ThemableColor(ThemeColorType.Background2)))
                },
                new CellPropertiesValues()
                {
                     Borders = new CellBorders(
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent2)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent4)),
                         null,
                         null,
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Background1)),
                         new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Background2)))
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal,
                new CellPropertiesValues()
                {
                    Borders = new CellBorders(
                        new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent1)),
                        CellBorder.Default,
                        CellBorder.Default,
                        CellBorder.Default,
                        null,
                        null,
                        CellBorder.Default,
                        CellBorder.Default)
                });
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest3()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Fill = new PatternFill(PatternType.Gray12Percent, new ThemableColor(ThemeColorType.Accent1), new ThemableColor(ThemeColorType.Accent2))
                },
                new CellPropertiesValues()
                {
                     Fill = new GradientFill(GradientType.DiagonalDown, new ThemableColor(ThemeColorType.Accent1), new ThemableColor(ThemeColorType.Accent2))
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest4()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     FontFamily = new ThemableFontFamily(ThemeFontType.Major)
                },
                new CellPropertiesValues()
                {
                     FontFamily = new ThemableFontFamily(ThemeFontType.Minor)
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest5()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     FontSize = 18
                },
                new CellPropertiesValues()
                {
                     FontSize = 23
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest6()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     ForeColor = new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4)
                },
                new CellPropertiesValues()
                {
                     ForeColor = new ThemableColor(ThemeColorType.Accent1, ColorShadeType.Shade2)
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest7()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Format = new CellValueFormat("$#,##0_);[Red]($#,##0)")
                },
                new CellPropertiesValues()
                {
                     Format = CellValueFormat.GeneralFormat
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest8()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                    HorizontalAlignment = RadHorizontalAlignment.Right,
                    Indent = 5
                },
                new CellPropertiesValues()
                {
                    HorizontalAlignment = RadHorizontalAlignment.Left,
                    Indent = 10
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest9()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsBold = true
                },
                new CellPropertiesValues()
                {
                     IsBold = false
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest10()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsItalic = true
                },
                new CellPropertiesValues()
                {
                     IsItalic = false
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest11()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     IsWrapped = true
                },
                new CellPropertiesValues()
                {
                     IsWrapped = false
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest12()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     StyleName = "Good"
                },
                new CellPropertiesValues()
                {
                     StyleName = "Bad"
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest13()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     Underline = UnderlineType.Single
                },
                new CellPropertiesValues()
                {
                     Underline = UnderlineType.None
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest14()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Bottom
                },
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Top
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest15()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Top
                },
                new CellPropertiesValues()
                {
                     VerticalAlignment = RadVerticalAlignment.Bottom
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest16()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     Indent = 5
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     Indent = 2
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillTwoInitialValuesTest17()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     Indent = 5
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     Indent = 6
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        #endregion


        #region Three Initial Values Tests

        [TestMethod]
        public void AutoFillThreeInitialValuesTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right
                }
                ,
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Center
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        [TestMethod]
        public void AutoFillThreeInitialValuesTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     ForeColor = new ThemableColor(Colors.Red)
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     Fill = new PatternFill(PatternType.Gray6Percent, Colors.Blue, Colors.Green)
                }
                ,
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Center,
                     Fill = new GradientFill(GradientType.DiagonalUpReversed, Colors.Green, Colors.Red)
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertAutoFillPropertiesCorrectness(initinalValues, expectedValuesCount, CellOrientation.Horizontal, CellPropertiesValues.Default);
        }

        #endregion
    }
}