﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    [TestClass]
    public class AutoFillValuesTests
    {
        #region Methods

        private static void AssertParsedValuesEquality(string expectedString, string actualString)
        {
            double expectedNumber;
            double actualNumber;

            if (double.TryParse(expectedString, out expectedNumber) &&
                double.TryParse(actualString, out actualNumber))
            {
                Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(expectedNumber, actualNumber));
            }
            else
            {
                Assert.AreEqual(expectedString, actualString);
            }
        }

        private void TestAutoFillHorizontal(string[] initialValues, string[] expectedResult)
        {
            this.TestAutoFillHorizontal(initialValues, expectedResult, CellValueFormat.GeneralFormat);
        }

        private void TestAutoFillHorizontal(string[] initialValues, string[] expectedResult, CellValueFormat format)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = 0; i < initialValues.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                cellSelection.SetValue(initialValues[i]);
                cellSelection.SetFormat(format);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedResult.Length - 1);
            cellSelection.FillDataSeriesAuto(CellOrientation.Horizontal, false, initialValues.Length);

            for (int i = 0; i < expectedResult.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.GetResultValueAsString(cellSelection.GetFormat().Value);

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillHorizontalReversed(string[] initialValues, string[] expectedResult)
        {
            this.TestAutoFillHorizontalReversed(initialValues, expectedResult, CellValueFormat.GeneralFormat);
        }

        private void TestAutoFillHorizontalReversed(string[] initialValues, string[] expectedResult, CellValueFormat format)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = initialValues.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i + expectedResult.Length - initialValues.Length, 0, i + expectedResult.Length - initialValues.Length);
                cellSelection.SetValue(initialValues[initialValues.Length - 1 - i]);
                cellSelection.SetFormat(format);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, expectedResult.Length - 1, 0, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Horizontal, true, initialValues.Length);

            for (int i = expectedResult.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[expectedResult.Length - 1 - i];
                string actual = value.GetResultValueAsString(cellSelection.GetFormat().Value);

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillVertical(string[] initialValues, string[] expectedResult)
        {
            this.TestAutoFillVertical(initialValues, expectedResult, CellValueFormat.GeneralFormat);
        }

        private void TestAutoFillVertical(string[] initialValues, string[] expectedResult, CellValueFormat format)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = 0; i < initialValues.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                cellSelection.SetValue(initialValues[i]);
                cellSelection.SetFormat(format);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, 0, expectedResult.Length - 1, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Vertical, false, initialValues.Length);

            for (int i = 0; i < expectedResult.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.GetResultValueAsString(cellSelection.GetFormat().Value);

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillVerticalReversed(string[] initialValues, string[] expectedResult)
        {
            this.TestAutoFillVerticalReversed(initialValues, expectedResult, CellValueFormat.GeneralFormat);
        }

        private void TestAutoFillVerticalReversed(string[] initialValues, string[] expectedResult, CellValueFormat format)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = initialValues.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i + expectedResult.Length - initialValues.Length, 0, i + expectedResult.Length - initialValues.Length, 0);
                cellSelection.SetValue(initialValues[initialValues.Length - 1 - i]);
                cellSelection.SetFormat(format);
            }

            cellSelection = worksheet.Cells.GetCellSelection(expectedResult.Length - 1, 0, 0, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Vertical, true, initialValues.Length);

            for (int i = expectedResult.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[expectedResult.Length - 1 - i];
                string actual = value.GetResultValueAsString(cellSelection.GetFormat().Value);

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillormulasHorizontal(string[] initialValues, string[] expectedResult)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = 0; i < initialValues.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                cellSelection.SetValue(initialValues[i]);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedResult.Length - 1);
            cellSelection.FillDataSeriesAuto(CellOrientation.Horizontal, false, initialValues.Length);

            for (int i = 0; i < expectedResult.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.RawValue;

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillFormulasHorizontalReversed(string[] initialValues, string[] expectedResult)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = initialValues.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i + expectedResult.Length - initialValues.Length, 0, i + expectedResult.Length - initialValues.Length);
                cellSelection.SetValue(initialValues[i]);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, expectedResult.Length - 1, 0, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Horizontal, true, initialValues.Length);

            for (int i = expectedResult.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(0, i, 0, i);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.RawValue;

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillFormulasVertical(string[] initialValues, string[] expectedResult)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = 0; i < initialValues.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                cellSelection.SetValue(initialValues[i]);
            }

            cellSelection = worksheet.Cells.GetCellSelection(0, 0, expectedResult.Length - 1, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Vertical, false, initialValues.Length);

            for (int i = 0; i < expectedResult.Length; i++)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.RawValue;

                AssertParsedValuesEquality(expected, actual);
            }
        }

        private void TestAutoFillFormulasVerticalReversed(string[] initialValues, string[] expectedResult)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cellSelection;

            for (int i = initialValues.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i + expectedResult.Length - initialValues.Length, 0, i + expectedResult.Length - initialValues.Length, 0);
                cellSelection.SetValue(initialValues[i]);
            }

            cellSelection = worksheet.Cells.GetCellSelection(expectedResult.Length - 1, 0, 0, 0);
            cellSelection.FillDataSeriesAuto(CellOrientation.Vertical, true, initialValues.Length);

            for (int i = expectedResult.Length - 1; i >= 0; i--)
            {
                cellSelection = worksheet.Cells.GetCellSelection(i, 0, i, 0);
                var value = cellSelection.GetValue().Value;

                string expected = expectedResult[i];
                string actual = value.RawValue;

                AssertParsedValuesEquality(expected, actual);
            }
        }

        #endregion


        #region Auto Fill Repeat Tests

        [TestMethod]
        public void RepeatGenerator1()
        {
            string[] simpleInitialValues = new string[]
            {
                "TRUE", "FALSE"
            };

            var expectedResult = new string[]
            {
                 "TRUE", "FALSE","TRUE", "FALSE","TRUE", "FALSE","TRUE", "FALSE","TRUE", "FALSE","TRUE",
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void RepeatGeneratorNumberAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "1"
            };

            var expectedResult = new string[]
            {
                 "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void RepeatGeneratorTextNumberNotOrdinalAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "123"
            };

            var expectedResult = new string[]
            {
                 "123", "124" 
                 // The actual result should be "123", "123" However, this test checks the fix of a bug which turned the result to ordinal numbers.
                 // Fixing the behavior from increasing numbering to repetition is not a priority at the moment.
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult, new CellValueFormat("@"));
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult, new CellValueFormat("@"));
            this.TestAutoFillVertical(simpleInitialValues, expectedResult, new CellValueFormat("@"));
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult, new CellValueFormat("@"));
        }

        [TestMethod]
        public void RepeatGeneratorAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "as", "bd", "te"
            };

            var expectedResult = new string[]
            {
                 "as", "bd", "te", "as", "bd", "te", "as", "bd", "te", "as", "bd", "te", "as", "bd", "te"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void SimpleRepeatGeneratorAutoFillHorizontalFormulasTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "=A2"
            };

            string[] simpleReversedInitialValues = new string[]
            {
                "=O2"
            };

            var expectedResultHorizontal = new string[]
            {
                 "=A2", "=B2", "=C2", "=D2", "=E2", "=F2", "=G2", "=H2", "=I2", "=J2", "=K2", "=L2", "=M2", "=N2", "=O2"
            };

            this.TestAutoFillormulasHorizontal(simpleInitialValues, expectedResultHorizontal);
            this.TestAutoFillFormulasHorizontalReversed(simpleReversedInitialValues, expectedResultHorizontal);
        }

        [TestMethod]
        public void SimpleRepeatGeneratorAutoFillVerticalFormulasTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "=B1"
            };

            string[] simpleReversedInitialValues = new string[]
            {
                "=B15"
            };

            var expectedResultVertical = new string[]
            {
                 "=B1", "=B2", "=B3", "=B4", "=B5", "=B6", "=B7", "=B8", "=B9", "=B10", "=B11", "=B12", "=B13", "=B14", "=B15"
            };

            this.TestAutoFillFormulasVertical(simpleInitialValues, expectedResultVertical);
            this.TestAutoFillFormulasVerticalReversed(simpleReversedInitialValues, expectedResultVertical);
        }

        [TestMethod]
        public void RepeatGeneratorAutoFillHorizontalFormulasTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "=A2", "=B3"
            };

            string[] simpleReversedInitialValues = new string[]
            {
                "=O2", "=P3"
            };

            var expectedResultHorizontal = new string[]
            {
                 "=A2", "=B3", "=C2", "=D3", "=E2", "=F3", "=G2", "=H3", "=I2", "=J3", "=K2", "=L3", "=M2", "=N3", "=O2", "=P3"
            };

            this.TestAutoFillormulasHorizontal(simpleInitialValues, expectedResultHorizontal);
            this.TestAutoFillFormulasHorizontalReversed(simpleReversedInitialValues, expectedResultHorizontal);
        }

        [TestMethod]
        public void RepeatGeneratorAutoFillVerticalFormulasTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "=D1", "=G2"
            };

            string[] simpleReversedInitialValues = new string[]
            {
                "=D15", "=G16"
            };

            var expectedResultVertical = new string[]
            {
                 "=D1", "=G2", "=D3", "=G4", "=D5", "=G6", "=D7", "=G8", "=D9", "=G10", "=D11", "=G12", "=D13", "=G14", "=D15", "=G16"
            };

            this.TestAutoFillFormulasVertical(simpleInitialValues, expectedResultVertical);
            this.TestAutoFillFormulasVerticalReversed(simpleReversedInitialValues, expectedResultVertical);
        }

        #endregion


        #region Auto Fill Quarter Tests

        [TestMethod]
        public void UpperCaseQuarterGeneratorAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "Q1"
            };

            var expectedResult = new string[]
            {
                 "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseQuarterGeneratorAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "Q3"
            };

            var expectedResult = new string[]
            {
                 "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseQuarterGeneratorAutoFillTest3()
        {
            string[] simpleInitialValues = new string[]
            {
                "Q1", "Q2", "Q3"
            };

            var expectedResult = new string[]
            {
                 "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3", "Q4", "Q1", "Q2", "Q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "q1"
            };

            var expectedResult = new string[]
            {
                 "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "q4"
            };

            var expectedResult = new string[]
            {
                 "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorAutoFillTest3()
        {
            string[] simpleInitialValues = new string[]
            {
                "q1", "q2", "q3"
            };

            var expectedResult = new string[]
            {
                 "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorStartingFromQ3AutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "q3", "q4"
            };

            var expectedResult = new string[]
            {
                 "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1", "q2", "q3", "q4", "q1"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorStartingFromQ3ReversedAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "q3", "q2"
            };

            var expectedResult = new string[]
            {
                 "q3", "q2", "q1", "q4", "q3", "q2", "q1", "q4", "q3", "q2", "q1", "q4", "q3", "q2", "q1"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorStartingFromQ3ReversedAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "Q3", "q2"
            };

            var expectedResult = new string[]
            {
                 "Q3", "q2", "Q1", "q4", "Q3", "q2", "Q1", "q4", "Q3", "q2", "Q1", "q4", "Q3", "q2", "Q1"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorAndLowerCaseShortWeekDaysAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "q3", "q2", "mon", "tue"
            };

            var expectedResult = new string[]
            {
                 "q3", "q2", "mon", "tue", "q1", "q4", "wed", "thu", "q3", "q2", "fri", "sat", "q1", "q4", "sun"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseQuarterGeneratorAndUpperCaseShortWeekDaysAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "q3", "q2", "Mon", "Tue"
            };

            var expectedResult = new string[]
            {
                 "q3", "q2", "Mon", "Tue", "q1", "q4", "Wed", "Thu", "q3", "q2", "Fri", "Sat", "q1", "q4", "Sun"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Auto Fill Week Days Tests

        [TestMethod]
        public void LowerAndUpperCaseShortWeekDaysAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "Mon", "Tue", "mon", "tue"
            };

            var expectedResult = new string[]
            {
                 "Mon", "Tue", "mon", "tue", "Wed", "Thu", "wed", "thu", "Fri", "Sat", "fri", "sat", "Sun", "Mon", "sun"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseShortWeekDaysAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "mon", "tue"
            };

            var expectedResult = new string[]
            {
                 "mon", "tue", "wed", "thu", "fri", "sat", "sun", "mon", "tue", "wed", "thu", "fri", "sat", "sun", "mon"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseShortWeekDaysAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "Mon", "Tue"
            };

            var expectedResult = new string[]
            {
                 "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun", "Mon"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseShortWeekDaysAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "Mon"
            };

            var expectedResult = new string[]
            {
                 "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun", "Mon"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseLongWeekDaysAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "monday", "tuesday"
            };

            var expectedResult = new string[]
            {
                 "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday",
                 "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday",
                 "monday"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseLongWeekDaysAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "Monday"
            };

            var expectedResult = new string[]
            {
                 "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday",
                 "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday",
                 "Monday"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseLongWeekDaysAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "Monday", "Tuesday"
            };

            var expectedResult = new string[]
            {
                 "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday",
                 "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday",
                 "Monday"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Auto Fill Months Tests

        [TestMethod]
        public void LowerCaseShortMonthsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "jan", "feb"
            };

            var expectedResult = new string[]
            {
                 "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec", "jan", "feb", "mar"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void LowerCaseLongMonthsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "january", "february"
            };

            var expectedResult = new string[]
            {
                 "january", "february", "march", "april", "may", "june", "july", "august",
                 "september", "october", "november", "december", "january", "february", "march"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseShortMonthsAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "Jan"
            };

            var expectedResult = new string[]
            {
                 "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseShortMonthsAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "Jan", "Feb"
            };

            var expectedResult = new string[]
            {
                 "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseLongMonthsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "January", "February"
            };

            var expectedResult = new string[]
            {
                 "January", "February", "March", "April", "May", "June", "July", "August", "September",
                 "October", "November", "December", "January", "February", "March"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void UpperCaseLongMonthsOneInitialItemAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "February"
            };

            var expectedResult = new string[]
            {
                 "February", "March", "April", "May", "June", "July", "August", "September",
                 "October", "November", "December", "January", "February", "March", "April"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Auto Fill Ordinal Numbers Tests

        [TestMethod]
        public void OrdinalNumbersOneInitialItemAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "1st"
            };

            var expectedResult = new string[]
            {
                 "1st", "2nd", "3rd", "4th", "5th", "6th", "7th", "8th",
                 "9th", "10th", "11th", "12th", "13th", "14th", "15th",
                 "16th", "17th", "18th", "19th", "20th", "21st", "22nd"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void OrdinalNumbersReversedAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "3rd", "2nd"
            };

            var expectedResult = new string[]
            {
                 "3rd", "2nd", "1st", "0th", "1st", "2nd", "3rd", "4th",
                 "5th", "6th", "7th", "8th", "9th", "10th", "11th", "12th",
                 "13th", "14th", "15th", "16th", "17th", "18th"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Auto Fill Numbered Text Tests

        [TestMethod]
        public void NumberedTextAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test5", "test7"
            };

            var expectedResult = new string[]
            {
                 "test5", "test7", "test9", "test11", "test13", "test15", "test17", "test19", "test21", "test23", "test25", "test27", "test29"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumberedTextReversedSimpleAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test7", "test5"
            };

            var expectedResult = new string[]
            {
                 "test7", "test5", "test3", "test1", "test1", "test3", "test5", "test7", "test9", "test11", "test13", "test15", "test17"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumberedTextReversedAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test5", "test2"
            };

            var expectedResult = new string[]
            {
                 "test5", "test2", "test1", "test4", "test7", "test10", "test13", "test16", "test19", "test22", "test25", "test28", "test31"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumberedTextReversedComplexAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test 22", "test 15"
            };

            var expectedResult = new string[]
            {
                 "test 22", "test 15", "test 8", "test 1", "test 6", "test 13", "test 20", "test 27", "test 34", "test 41", "test 48", "test 55", "test 62"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumberedTextReversedTwoGroupsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test 2", "test 1.5"
            };

            var expectedResult = new string[]
            {
                 "test 2", "test 1.5", "test 3", "test 1.6", "test 4", "test 1.7", "test 5", "test 1.8", "test 6", "test 1.9", "test 7", "test 1.10", "test 8"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumberedTextLargeNumberAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "test123456789000"
            };

            var expectedResult = new string[]
            {
                 "test123456789000", "test123456789000"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }


        #endregion


        #region Auto Fill Numbers Tests

        [TestMethod]
        public void SimpleNumbersAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "4", "3"
            };

            var expectedResult = new string[]
            {
                 "4", "3", "2", "1", "0", "-1", "-2", "-3", "-4", "-5", "-6", "-7", "-8"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", "3", (3.23076923076923).ToCultureDependentString(), (3.69230769230769).ToCultureDependentString(), 
                 (4.15384615384615).ToCultureDependentString(), (4.61538461538462).ToCultureDependentString(),
                 (5.07692307692308).ToCultureDependentString(), (5.53846153846154).ToCultureDependentString(), 
                 "6", (6.46153846153846).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", "3", (3.23076923076923).ToCultureDependentString(), (3.69230769230769).ToCultureDependentString(), 
                 (4.15384615384615).ToCultureDependentString(), (4.61538461538461).ToCultureDependentString(), (5.07692307692307).ToCultureDependentString(),
                 (5.53846153846154).ToCultureDependentString(), "6", (6.46153846153846).ToCultureDependentString(), (6.92307692307692).ToCultureDependentString(),
                 (7.38461538461538).ToCultureDependentString(), (7.84615384615384).ToCultureDependentString(), (8.30769230769231).ToCultureDependentString(), 
                 (8.76923076923077).ToCultureDependentString(), (9.23076923076923).ToCultureDependentString(), (9.69230769230769).ToCultureDependentString(),
                 (10.1538461538462).ToCultureDependentString(), (10.6153846153846).ToCultureDependentString(), (11.0769230769231).ToCultureDependentString(),
                 (11.5384615384615).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", "3", string.Empty, "4", "5", string.Empty,
                 "6", "7", string.Empty, "8", "9", string.Empty, "10", "11"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest2()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, "3", string.Empty, "4", string.Empty, "5", string.Empty,
                 "6", string.Empty, "7", string.Empty, "8", string.Empty, "9", string.Empty, "10", string.Empty, "11"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest3()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, string.Empty, "3", string.Empty, "4", string.Empty, string.Empty, "5", string.Empty, "6",
                 string.Empty, string.Empty, "7", string.Empty, "8", string.Empty, string.Empty, "9", string.Empty, "10", string.Empty, string.Empty, "11"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest4()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", "3", (3.23076923076923).ToCultureDependentString(), (3.69230769230769).ToCultureDependentString(),
                 (4.15384615384615).ToCultureDependentString(), (4.61538461538461).ToCultureDependentString(), (5.07692307692307).ToCultureDependentString(),
                 (5.53846153846154).ToCultureDependentString(), "6", (6.46153846153846).ToCultureDependentString(), (6.92307692307692).ToCultureDependentString(),
                 (7.38461538461538).ToCultureDependentString(), (7.84615384615384).ToCultureDependentString(), (8.30769230769231).ToCultureDependentString(),
                 (8.76923076923077).ToCultureDependentString(), (9.23076923076923).ToCultureDependentString(), (9.69230769230769).ToCultureDependentString(),
                 (10.1538461538462).ToCultureDependentString(), (10.6153846153846).ToCultureDependentString(), (11.0769230769231).ToCultureDependentString(),
                 (11.5384615384615).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest5()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", "3", string.Empty, string.Empty, "4", "5", string.Empty, string.Empty, "6", "7", string.Empty, string.Empty,
                 "8", "9", string.Empty, string.Empty, "10", "11", string.Empty, string.Empty, "12"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest6()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", "3", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", "3", string.Empty, (3.23076923076923).ToCultureDependentString(), 
                 (3.69230769230769).ToCultureDependentString(), (4.15384615384615).ToCultureDependentString(), (4.61538461538461).ToCultureDependentString(), 
                 (5.07692307692307).ToCultureDependentString(), string.Empty, (5.53846153846154).ToCultureDependentString(), "6",
                 (6.46153846153846).ToCultureDependentString(), (6.92307692307692).ToCultureDependentString(), (7.38461538461538).ToCultureDependentString(),
                 string.Empty, (7.84615384615384).ToCultureDependentString(), (8.30769230769231).ToCultureDependentString(),
                 (8.76923076923077).ToCultureDependentString(), (9.23076923076923).ToCultureDependentString(), (9.69230769230769).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest7()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, string.Empty, "2"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, string.Empty, "2", "3", string.Empty, string.Empty, string.Empty, "4", "5", string.Empty, string.Empty, string.Empty,
                 "6", "7", string.Empty, string.Empty, string.Empty, "8", "9", string.Empty, string.Empty, string.Empty, "10"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest8()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, string.Empty, "2", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, string.Empty, "2", string.Empty, "3", string.Empty, string.Empty, string.Empty, "4", string.Empty, "5",
                 string.Empty, string.Empty, string.Empty, "6", string.Empty, "7", string.Empty, string.Empty, string.Empty, "8", string.Empty, "9",
                 string.Empty, string.Empty, string.Empty, "10"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest9()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, string.Empty, "2", "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, string.Empty, "2", "3", (3.07142857142857).ToCultureDependentString(),
                 (3.42857142857143).ToCultureDependentString(), (3.78571428571429).ToCultureDependentString(), (4.14285714285715).ToCultureDependentString(),
                 (4.5).ToCultureDependentString(), (4.85714285714286).ToCultureDependentString(), (5.21428571428572).ToCultureDependentString(),
                 (5.57142857142857).ToCultureDependentString(), (5.92857142857143).ToCultureDependentString(), (6.28571428571429).ToCultureDependentString(),
                 (6.64285714285715).ToCultureDependentString(), "7"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest10()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, "3", "4", string.Empty, "5", string.Empty, "6", "7", string.Empty, "8", string.Empty,
                 "9", "10", string.Empty, "11", string.Empty, "12", "13"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest11()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, "3", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, "3", string.Empty, "4", string.Empty, "5", string.Empty, "6", string.Empty, "7", string.Empty, "8", string.Empty,
                 "9", string.Empty, "10"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest12()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, string.Empty, "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, string.Empty, "3", (3.44736842105263).ToCultureDependentString(), (3.84210526315789).ToCultureDependentString(),
                 (4.23684210526316).ToCultureDependentString(), (4.63157894736842).ToCultureDependentString(), (5.02631578947368).ToCultureDependentString(), 
                 (5.42105263157894).ToCultureDependentString(), (5.81578947368421).ToCultureDependentString(), (6.21052631578947).ToCultureDependentString(), 
                 (6.60526315789473).ToCultureDependentString(), "7", (7.39473684210526).ToCultureDependentString(), (7.78947368421052).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest13()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", string.Empty, "3"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", string.Empty, "3", (3.31578947368421).ToCultureDependentString(), (3.71052631578947).ToCultureDependentString(),
                 (4.10526315789474).ToCultureDependentString(), (4.5).ToCultureDependentString(), (4.89473684210526).ToCultureDependentString(),
                 (5.28947368421053).ToCultureDependentString(), (5.68421052631579).ToCultureDependentString(), (6.07894736842105).ToCultureDependentString(),
                 (6.47368421052631).ToCultureDependentString(), (6.86842105263158).ToCultureDependentString(), (7.26315789473684).ToCultureDependentString(),
                 (7.6578947368421).ToCultureDependentString(), (8.05263157894737).ToCultureDependentString(), (8.44736842105263).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest14()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, string.Empty, "3", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, string.Empty, "3", string.Empty, (3.44736842105263).ToCultureDependentString(),
                 (3.84210526315789).ToCultureDependentString(), (4.23684210526316).ToCultureDependentString(), (4.63157894736842).ToCultureDependentString(),
                 (5.02631578947368).ToCultureDependentString(), (5.42105263157894).ToCultureDependentString(), string.Empty, (5.81578947368421).ToCultureDependentString(),
                 (6.21052631578947).ToCultureDependentString(), (6.60526315789473).ToCultureDependentString(), "7", (7.39473684210526).ToCultureDependentString(), 
                 (7.78947368421052).ToCultureDependentString(), string.Empty, (8.18421052631579).ToCultureDependentString(), (8.57894736842105).ToCultureDependentString(), 
                 (8.97368421052631).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest15()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, string.Empty, "2", string.Empty, "3", string.Empty
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, string.Empty, "2", string.Empty, "3", string.Empty, (3.31578947368421).ToCultureDependentString(), 
                 (3.71052631578947).ToCultureDependentString(), (4.10526315789474).ToCultureDependentString(), (4.5).ToCultureDependentString(),
                (4.89473684210526).ToCultureDependentString(), (5.28947368421053).ToCultureDependentString(), string.Empty, 
                (5.68421052631579).ToCultureDependentString(), (6.07894736842105).ToCultureDependentString(), (6.47368421052631).ToCultureDependentString(), 
                (6.86842105263158).ToCultureDependentString(), (7.26315789473684).ToCultureDependentString(), (7.6578947368421).ToCultureDependentString(), 
                string.Empty, (8.05263157894737).ToCultureDependentString(), (8.44736842105263).ToCultureDependentString(), (8.84210526315789).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest16()
        {
            string[] simpleInitialValues = new string[]
            {
                "2", string.Empty, "4", "6"
            };

            var expectedResult = new string[]
            {
                 "2", string.Empty, "4", "6", "7", (8.28571428571429).ToCultureDependentString(), (9.57142857142858).ToCultureDependentString(),
                 (10.8571428571429).ToCultureDependentString(), (12.1428571428572).ToCultureDependentString(), (13.4285714285715).ToCultureDependentString(),
                 (14.7142857142858).ToCultureDependentString(), "16", (17.2857142857143).ToCultureDependentString(), (18.5714285714286).ToCultureDependentString(), 
                 (19.8571428571429).ToCultureDependentString(), (21.1428571428572).ToCultureDependentString(), (22.4285714285715).ToCultureDependentString(),
                 (23.7142857142858).ToCultureDependentString(), "25", (26.2857142857143).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest17()
        {
            string[] simpleInitialValues = new string[]
            {
                "2", string.Empty, "4", string.Empty
            };

            var expectedResult = new string[]
            {
                 "2", string.Empty, "4", string.Empty, "6", string.Empty, "8", string.Empty, "10", string.Empty, "12", string.Empty, "14", string.Empty, "16",
                 string.Empty, "18", string.Empty, "20", string.Empty, "22", string.Empty, "24", string.Empty, "26"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest18()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", "2", string.Empty, "3", "4"
            };

            var expectedResult = new string[]
            {
                 "1", "2", string.Empty, "3", "4", (4.6).ToCultureDependentString(), (5.3).ToCultureDependentString(), "6", (6.7).ToCultureDependentString(),
                 (7.4).ToCultureDependentString(), (8.1).ToCultureDependentString(), (8.8).ToCultureDependentString(), (9.5).ToCultureDependentString(),
                 (10.2).ToCultureDependentString(), (10.9).ToCultureDependentString(), (11.6).ToCultureDependentString(), (12.3).ToCultureDependentString(),
                 "13", (13.7).ToCultureDependentString(), (14.4).ToCultureDependentString(), (15.1).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest19()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", "2", string.Empty, string.Empty, "3", "4"
            };

            var expectedResult = new string[]
            {
                 "1", "2", string.Empty, string.Empty, "3", "4", (4.35294117647058).ToCultureDependentString(), (4.88235294117647).ToCultureDependentString(), 
                 (5.41176470588235).ToCultureDependentString(), (5.94117647058823).ToCultureDependentString(), (6.47058823529411).ToCultureDependentString(), "7",
                 (7.52941176470588).ToCultureDependentString(), (8.05882352941176).ToCultureDependentString(), (8.58823529411764).ToCultureDependentString(),
                 (9.11764705882353).ToCultureDependentString(), (9.64705882352941).ToCultureDependentString(), (10.1764705882353).ToCultureDependentString(),
                 (10.7058823529412).ToCultureDependentString(), (11.2352941176471).ToCultureDependentString(), (11.7647058823529).ToCultureDependentString(),
                 (12.2941176470588).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersContainingEmptyCellsAutoFillTest20()
        {
            string[] simpleInitialValues = new string[]
            {
                "1", string.Empty, "2", string.Empty, "4"
            };

            var expectedResult = new string[]
            {
                 "1", string.Empty, "2", string.Empty, "4", (4.58333333333333).ToCultureDependentString(), (5.33333333333333).ToCultureDependentString(), 
                 (6.08333333333333).ToCultureDependentString(), (6.83333333333333).ToCultureDependentString(), (7.58333333333333).ToCultureDependentString(),
                 (8.33333333333333).ToCultureDependentString(), (9.08333333333333).ToCultureDependentString(), (9.83333333333333).ToCultureDependentString(),
                 (10.5833333333333).ToCultureDependentString(), (11.3333333333333).ToCultureDependentString(), (12.0833333333333).ToCultureDependentString(), 
                 (12.8333333333333).ToCultureDependentString(), (13.5833333333333).ToCultureDependentString(), (14.3333333333333).ToCultureDependentString(), 
                 (15.0833333333333).ToCultureDependentString(), (15.8333333333333).ToCultureDependentString(), (16.5833333333333).ToCultureDependentString(),
                 (17.3333333333333).ToCultureDependentString(), (18.0833333333333).ToCultureDependentString(), (18.8333333333333).ToCultureDependentString()
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersStartingWithEmptyCellsAutoFillTest()
        {
            string[] simpleInitialValues = new string[]
            {
                string.Empty, string.Empty, "1", string.Empty, string.Empty, "2", "3"
            };

            var expectedResult = new string[]
            {
                 string.Empty, string.Empty, "1", string.Empty, string.Empty, "2", "3", string.Empty, string.Empty,
                 (3.23076923076923).ToCultureDependentString(), (3.69230769230769).ToCultureDependentString(), (4.15384615384615).ToCultureDependentString(), 
                 (4.61538461538462).ToCultureDependentString(), (5.07692307692308).ToCultureDependentString(), string.Empty, string.Empty,
                 (5.53846153846154).ToCultureDependentString(), "6"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void NumbersStartingWithEmptyCellsAutoFillTest1()
        {
            string[] simpleInitialValues = new string[]
            {
                string.Empty, string.Empty, string.Empty, string.Empty, "123"
            };

            var expectedResult = new string[]
            {
                 string.Empty, string.Empty, string.Empty, string.Empty, "123", string.Empty, string.Empty, string.Empty, string.Empty, "124",
                 string.Empty, string.Empty, string.Empty, string.Empty, "125", string.Empty, string.Empty, string.Empty, string.Empty, "126",
                 string.Empty, string.Empty, string.Empty, string.Empty, "127", string.Empty, string.Empty, string.Empty, string.Empty, "128"
            };

            this.TestAutoFillHorizontal(simpleInitialValues, expectedResult);
            this.TestAutoFillHorizontalReversed(simpleInitialValues, expectedResult);
            this.TestAutoFillVertical(simpleInitialValues, expectedResult);
            this.TestAutoFillVerticalReversed(simpleInitialValues, expectedResult);
        }

        #endregion
    }
}