﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    public class CellPropertiesValues
    {
        #region Constants

        public static readonly CellPropertiesValues Default = new CellPropertiesValues();

        #endregion


        #region Fields

        private CellBorders borders;
        private IFill fill;
        private ThemableFontFamily fontFamily;
        private double fontSize;
        private ThemableColor foreColor;
        private CellValueFormat format;
        private RadHorizontalAlignment horizontalAlignment;
        private int indent;
        private bool isBold;
        private bool isItalic;
        private bool isWrapped;
        private string styleName;
        private UnderlineType underline;
        private RadVerticalAlignment verticalAlignment;
        private bool isLocked;

        private bool isBordersSet;
        private bool isFillSet;
        private bool isFontFamilySet;
        private bool isFontSizeSet;
        private bool isForeColorSet;
        private bool isFormatSet;
        private bool isHorizontalAlignmentSet;
        private bool isIndentSet;
        private bool isIsBoldSet;
        private bool isIsItalicSet;
        private bool isIsWrappedSet;
        private bool isStyleNameSet;
        private bool isUnderlineSet;
        private bool isVertivalAlignmentSet;
        private bool isIsLockedSet;

        #endregion


        #region Properties

        public CellBorders Borders
        {
            get
            {
                return this.borders;
            }
            set
            {
                if (this.borders != value)
                {
                    this.borders = value;
                    this.isBordersSet = true;
                }
            }
        }

        public IFill Fill
        {
            get
            {
                return this.fill;
            }
            set
            {
                if (this.fill != value)
                {
                    this.fill = value;
                    this.isFillSet = true;
                }
            }
        }

        public ThemableFontFamily FontFamily
        {
            get
            {
                return this.fontFamily;
            }
            set
            {
                if (this.fontFamily != value)
                {
                    this.fontFamily = value;
                    this.isFontFamilySet = true;
                }
            }
        }

        public double FontSize
        {
            get
            {
                return this.fontSize;
            }
            set
            {
                if (this.fontSize != value)
                {
                    this.fontSize = value;
                    this.isFontSizeSet = true;
                }
            }
        }

        public ThemableColor ForeColor
        {
            get
            {
                return this.foreColor;
            }
            set
            {
                if (this.foreColor != value)
                {
                    this.foreColor = value;
                    this.isForeColorSet = true;
                }
            }
        }

        public CellValueFormat Format
        {
            get
            {
                return this.format;
            }
            set
            {
                if (this.format != value)
                {
                    this.format = value;
                    this.isFormatSet = true;
                }
            }
        }

        public RadHorizontalAlignment HorizontalAlignment
        {
            get
            {
                return this.horizontalAlignment;
            }
            set
            {
                if (this.horizontalAlignment != value)
                {
                    this.horizontalAlignment = value;
                    this.isHorizontalAlignmentSet = true;
                }
            }
        }

        public int Indent
        {
            get
            {
                return this.indent;
            }
            set
            {
                if (this.indent != value)
                {
                    this.indent = value;
                    this.isIndentSet = true;
                }
            }
        }

        public bool IsBold
        {
            get
            {
                return this.isBold;
            }
            set
            {
                if (this.isBold != value)
                {
                    this.isBold = value;
                    this.isIsBoldSet = true;
                }
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.isItalic;
            }
            set
            {
                if (this.isItalic != value)
                {
                    this.isItalic = value;
                    this.isIsItalicSet = true;
                }
            }
        }

        public bool IsWrapped
        {
            get
            {
                return this.isWrapped;
            }
            set
            {
                if (this.isWrapped != value)
                {
                    this.isWrapped = value;
                    this.isIsWrappedSet = true;
                }
            }
        }

        public string StyleName
        {
            get
            {
                return this.styleName;
            }
            set
            {
                if (this.styleName != value)
                {
                    this.styleName = value;
                    this.isStyleNameSet = true;
                }
            }
        }

        public UnderlineType Underline
        {
            get
            {
                return this.underline;
            }
            set
            {
                if (this.underline != value)
                {
                    this.underline = value;
                    this.isUnderlineSet = true;
                }
            }
        }

        public RadVerticalAlignment VerticalAlignment
        {
            get
            {
                return this.verticalAlignment;
            }
            set
            {
                if (this.verticalAlignment != value)
                {
                    this.verticalAlignment = value;
                    this.isVertivalAlignmentSet = true;
                }
            }
        }

        public bool IsLocked
        {
            get
            {
                return this.isLocked;
            }
            set
            {
                if (this.isLocked != value)
                {
                    this.isLocked = value;
                    this.isIsLockedSet = true;
                }
            }
        }

        public double Value { get; set; }

        #endregion


        #region Constructors

        public CellPropertiesValues()
        {
            this.Value = 1;
            this.styleName = SpreadsheetDefaultValues.DefaultStyleName;
            this.borders = CellBorders.Default;
            this.borders.InsideHorizontal = null;
            this.borders.InsideVertical = null;
            this.fill = PatternFill.Default;
            this.fontFamily = SpreadsheetDefaultValues.DefaultFontFamily;
            this.fontSize = SpreadsheetDefaultValues.DefaultFontSize;
            this.foreColor = SpreadsheetDefaultValues.DefaultForeColor; 
            this.format = CellValueFormat.GeneralFormat;
        }

        #endregion


        #region Methods

        public override bool Equals(object obj)
        {
            CellPropertiesValues other = obj as CellPropertiesValues;

            if (other == null)
            {
                return false;
            }

            bool result = true;

            if (isBordersSet)
            {
                result = result && this.Borders.Equals(other.Borders);
            }

            if (isFillSet)
            {
                result = result && this.Fill.Equals(other.Fill);
            }

            if (isFontFamilySet)
            {
                result = result && this.FontFamily.Equals(other.FontFamily);
            }

            if (isFontSizeSet)
            {
                result = result && this.FontSize.Equals(other.FontSize);
            }

            if (isForeColorSet)
            {
                result = result && this.ForeColor.Equals(other.ForeColor);
            }

            if (isFormatSet)
            {
                result = result && this.Format.Equals(other.Format);
            }

            if (isHorizontalAlignmentSet)
            {
                result = result && this.HorizontalAlignment.Equals(other.HorizontalAlignment);
            }

            if (isIndentSet)
            {
                result = result && this.Indent.Equals(other.Indent);
            }

            if (isIsBoldSet)
            {
                result = result && this.IsBold.Equals(other.IsBold);
            }

            if (isIsItalicSet)
            {
                result = result && this.IsItalic.Equals(other.IsItalic);
            }

            if (isIsWrappedSet)
            {
                result = result && this.IsWrapped.Equals(other.IsWrapped);
            }

            if (isStyleNameSet)
            {
                result = result && this.StyleName.Equals(other.StyleName);
            }

            if (isUnderlineSet)
            {
                result = result && this.Underline.Equals(other.Underline);
            }

            if (isVertivalAlignmentSet)
            {
                result = result && this.VerticalAlignment.Equals(other.VerticalAlignment);
            }

            if (isIsLockedSet)
            {
                result = result && this.IsLocked.Equals(other.IsLocked);
            }

            return result;
        }

        // TODO: Implement if needed
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        #endregion
    }
}