﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    [TestClass]
    public class DateFillValuesTests
    {
        #region Methods

        private static DateTime[] GenerateDateExpectedValues(int expectedValuesCount, double stepValue, Func<DateTime, int, DateTime> transform)
        {
            DateTime date = DateTime.Now;

            List<DateTime> result = new List<DateTime>();

            int intStep;
            double residue = 0;
            double originalStep = stepValue;

            result.Add(date);

            while (result.Count < expectedValuesCount)
            {
                intStep = (int)Math.Floor(stepValue);
                residue += (stepValue - intStep);
                if (residue >= 1)
                {
                    intStep++;
                    residue--;
                }

                result.Add(transform(result.Last(), intStep));

                stepValue = originalStep;
            }

            return result.ToArray();
        }

        private static DateTime[] GenerateWeekdayExpectedValues(int expectedValuesCount, double stepValue)
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            List<DateTime> result = new List<DateTime>();

            int intStep;
            double residue = 0;
            double originalStep = stepValue;

            result.Add(weekDay);

            while (result.Count < expectedValuesCount)
            {
                intStep = (int)Math.Floor(stepValue);
                residue += (stepValue - intStep);
                if (residue >= 1)
                {
                    intStep++;
                    residue--;
                }

                result.Add(GetNextWeekDay(result.Last(), intStep));

                stepValue = originalStep;
            }

            return result.ToArray();
        }

        private static DateTime GetNextWeekDay(DateTime weekDay, int stepValue = 1)
        {
            for (int i = 0; i < stepValue; i++)
            {
                weekDay = weekDay.AddDays(1);
                while (weekDay.DayOfWeek == DayOfWeek.Saturday ||
                       weekDay.DayOfWeek == DayOfWeek.Sunday)
                {
                    weekDay = weekDay.AddDays(1);
                }
            }

            return weekDay;
        }

        #endregion


        #region Tests

        // TODO: many fill tests fail in German culture, because the value is not set properly
        [TestMethod]
        public void FillDataSeriesDaysTest1()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 1;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest2()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest3()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2.5;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest4()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2.2;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest5()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 1;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            int selectionLength = expectedValues.Length + 5;

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue, selectionLength);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest6()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            int selectionLength = expectedValues.Length + 5;

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue, selectionLength);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest7()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2.5;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            int selectionLength = expectedValues.Length + 5;

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue, selectionLength);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest8()
        {
            string[] initialValues = new string[]
            {
                DateTime.Now.ToShortDateString(),
            };

            double stepValue = 2.2;

            DateTime[] expectedValues = GenerateDateExpectedValues(10, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddDays(step);
                });

            int selectionLength = expectedValues.Length + 5;

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue, selectionLength);
        }

        [TestMethod]
        public void FillDataSeriesDaysTest9()
        {
            string[] initialValues = new string[]
            {
                "TRUE"
            };

            double stepValue = 2.2;

            string[] expectedValues = new string[]
            {
                "TRUE", string.Empty, string.Empty, string.Empty, string.Empty, string.Empty
            };

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Day, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest1()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            int stepValue = 1;
            DateTime[] expectedValues = GenerateWeekdayExpectedValues(expectedValuesCount, stepValue);

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest2()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2;
            DateTime[] expectedValues = GenerateWeekdayExpectedValues(expectedValuesCount, (int)stepValue);

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest3()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 3;
            DateTime[] expectedValues = GenerateWeekdayExpectedValues(expectedValuesCount, (int)stepValue);

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest4()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 3.5;
            DateTime[] expectedValues = GenerateWeekdayExpectedValues(expectedValuesCount, stepValue);

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest5()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2.2;
            DateTime[] expectedValues = GenerateWeekdayExpectedValues(expectedValuesCount, stepValue);

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesWeekDaysTest6()
        {
            DateTime weekDay = DateTime.Now;
            weekDay = GetNextWeekDay(weekDay);

            string[] initialValues = new string[]
            {
                "FALSE"
            };

            double stepValue = 2.2;
            string[] expectedValues = new string[]
            {
                "FALSE", string.Empty, string.Empty, string.Empty, string.Empty, string.Empty
            };

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Weekday, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesMonthsTest1()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 1;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddMonths(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Month, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesMonthsTest2()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddMonths(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Month, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesMonthsTest3()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2.5;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddMonths(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Month, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesMonthsTest4()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2.2;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddMonths(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Month, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesMonthsTest5()
        {
            string[] initialValues = new string[]
            {
                "TRUE"
            };

            double stepValue = 2.2;
            string[] expectedValues = new string[]
            {
                "TRUE", string.Empty, string.Empty, string.Empty, string.Empty, string.Empty
            };

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Month, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesYearsTest1()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 1;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddYears(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Year, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesYearsTest2()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddYears(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Year, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesYearsTest3()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2.5;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddYears(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Year, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesYearsTest4()
        {
            DateTime weekDay = DateTime.Now;

            string[] initialValues = new string[]
            {
                weekDay.ToShortDateString(),
            };

            int expectedValuesCount = 10;
            double stepValue = 2.2;
            DateTime[] expectedValues = GenerateDateExpectedValues(expectedValuesCount, stepValue,
                (lastDate, step) =>
                {
                    return lastDate.AddYears(step);
                });

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Year, stepValue);
        }

        [TestMethod]
        public void FillDataSeriesYearsTest5()
        {
            string[] initialValues = new string[]
            {
                "FALSE"
            };

            double stepValue = 2.2;
            string[] expectedValues = new string[]
            {
                "FALSE", string.Empty, string.Empty, string.Empty, string.Empty, string.Empty
            };

            FillSeriesTestHelper.AssertFillDateCorrectness(initialValues, expectedValues, DateUnitType.Year, stepValue);
        }

        #endregion
    }
}