﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    public static class FillSeriesTestHelper
    {
        private static string[] ToStringArray(this DateTime[] input)
        {
            string[] expectedValues = new string[input.Length];

            for (int i = 0; i < input.Length; i++)
            {
                expectedValues[i] = input[i].ToShortDateString();
            }

            return expectedValues;
        }

        private static void FillInitialValues(CellPropertiesValues[] initinalValues, Worksheet worksheet)
        {
            for (int i = 0; i < initinalValues.Length; i++)
            {
                CellSelection cells = worksheet.Cells[0, i];
                cells.SetProperties(initinalValues[i]);
            }
        }

        private static void FillInitialValues(string[] initinalValues, Worksheet worksheet)
        {
            for (int i = 0; i < initinalValues.Length; i++)
            {
                CellSelection cells = worksheet.Cells[0, i];
                cells.SetValue(initinalValues[i]);
            }
        }

        private static void AssertCellPropertiesValuesAreEqual(int expectedValuesCount, CellPropertiesValues[] initinalValues, Worksheet worksheet,
            CellPropertiesValues lastCellExpectedValue)
        {
            int i = 0;

            while (i < expectedValuesCount)
            {
                for (int j = 0; j < initinalValues.Length; j++)
                {
                    CellSelection cells = worksheet.Cells[0, i];
                    CellPropertiesValues properties = cells.GetFormatProperties();

                    Assert.AreEqual(initinalValues[j], properties);

                    i++;

                    if (i >= expectedValuesCount)
                    {
                        break;
                    }
                }
            }

            CellPropertiesValues nextCellProperties = worksheet.Cells[0, expectedValuesCount].GetFormatProperties();

            Assert.AreEqual(nextCellProperties, lastCellExpectedValue);
        }

        private static void AssertCellPropertiesValuesAreEqual(int expectedValuesCount, CellPropertiesValues initinalValue, Worksheet worksheet)
        {
            for (int i = 0; i < expectedValuesCount; i++)
            {
                CellSelection cells = worksheet.Cells[0, i];
                CellPropertiesValues properties = cells.GetFormatProperties();

                Assert.AreEqual(initinalValue, properties);
            }

            CellPropertiesValues nextCellProperties = worksheet.Cells[0, expectedValuesCount].GetFormatProperties();

            Assert.AreEqual(nextCellProperties, CellPropertiesValues.Default);
        }

        public static void AssertFillPropertiesCorrectness(CellPropertiesValues[] initinalValues, int expectedValuesCount, Action<CellSelection> action)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            FillInitialValues(initinalValues, worksheet);

            CellSelection cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedValuesCount - 1);
            action(cellSelection);

            AssertCellPropertiesValuesAreEqual(expectedValuesCount, initinalValues[0], worksheet);
        }

        public static void AssertAutoFillPropertiesCorrectness(CellPropertiesValues[] initinalValues, int expectedValuesCount, CellOrientation orientation,
             CellPropertiesValues lastCellExpectedValue, bool isReversed = false, int? initialValueCount = null)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            FillInitialValues(initinalValues, worksheet);

            CellSelection cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedValuesCount - 1);
            cellSelection.FillDataSeriesAuto(orientation, isReversed, initialValueCount);

            AssertCellPropertiesValuesAreEqual(expectedValuesCount, initinalValues, worksheet, lastCellExpectedValue);
        }

        private static void AssertValuesAreEqual(string[] expectedValues, Worksheet worksheet)
        {
            for (int i = 0; i < expectedValues.Length; i++)
            {
                CellSelection cells = worksheet.Cells[0, i];
                string value = cells.GetValue().Value.GetResultValueAsString(cells.GetFormat().Value);

                Assert.AreEqual(expectedValues[i], value);
            }

            //CellPropertiesValues nextCellProperties = worksheet.Cells[0, expectedValuesCount].GetFormatProperties();

            //Assert.AreEqual(nextCellProperties, CellPropertiesValues.Default);
        }

        public static void AssertFillDateCorrectness(string[] initialValues, DateTime[] expectedValues, DateUnitType dateUnitType)
        {
            AssertFillDateCorrectness(initialValues, expectedValues, dateUnitType, 1);
        }

        public static void AssertFillDateCorrectness(string[] initialValues, DateTime[] expectedValues, DateUnitType dateUnitType, double stepValue)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            FillInitialValues(initialValues, worksheet);

            CellSelection cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedValues.Length - 1);
            cellSelection.FillDataSeriesDate(CellOrientation.Horizontal, dateUnitType, stepValue, null);

            AssertValuesAreEqual(expectedValues.ToStringArray(), worksheet);
        }

        public static void AssertFillDateCorrectness(string[] initialValues, string[] expectedValues, DateUnitType dateUnitType, double stepValue)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            FillInitialValues(initialValues, worksheet);

            CellSelection cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedValues.Length - 1);
            cellSelection.FillDataSeriesDate(CellOrientation.Horizontal, dateUnitType, stepValue, null);

            AssertValuesAreEqual(expectedValues, worksheet);
        }

        public static void AssertFillDateCorrectness(string[] initialValues, DateTime[] expectedValues, DateUnitType dateUnitType, double stepValue, int selectionLength)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            FillInitialValues(initialValues, worksheet);

            CellSelection cellSelection = worksheet.Cells.GetCellSelection(0, 0, 0, expectedValues.Length - 1);

            double? stopValue = FormatHelper.ConvertDateTimeToDouble(expectedValues.Last());
            stopValue = Math.Ceiling(stopValue.Value);
            cellSelection.FillDataSeriesDate(CellOrientation.Horizontal, dateUnitType, stepValue, stopValue);

            List<string> expected = new List<string>(expectedValues.ToStringArray());
            while(expected.Count < selectionLength)
            {
                expected.Add(string.Empty);
            }

            AssertValuesAreEqual(expected.ToArray(), worksheet);
        }
    }
}