﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FillSeriesTests
{
    [TestClass]
    public class LinearFillPropertiesTests
    {
        #region Linear Trend Fill Tests

        [TestMethod]
        public void LinearTrendFillOneInitialValueTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinearTrend(CellOrientation.Horizontal);
                });
        }

        [TestMethod]
        public void LinearTrendFillOneInitialValueTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     Fill = new PatternFill(PatternType.ReverseDiagonalStripe, new ThemableColor(ThemeColorType.Accent3), new ThemableColor(ThemeColorType.Accent6))
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinearTrend(CellOrientation.Horizontal);
                });
        }

        [TestMethod]
        public void LinearTrendFillTwoInitialValuesTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinearTrend(CellOrientation.Horizontal);
                });
        }

        [TestMethod]
        public void LinearTrendFillTwoInitialValuesTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     ForeColor = new ThemableColor(ThemeColorType.Accent2)
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     IsBold = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinearTrend(CellOrientation.Horizontal);
                });
        }

        #endregion


        #region Linear Fill Tests

        [TestMethod]
        public void LinearFillOneInitialValueTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinear(CellOrientation.Horizontal, 1);
                });
        }

        [TestMethod]
        public void LinearFillOneInitialValueTest2()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     IsItalic = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinear(CellOrientation.Horizontal, 1);
                });
        }

        [TestMethod]
        public void LinearFillTwoInitialValuesTest1()
        {
            CellPropertiesValues[] initinalValues = new CellPropertiesValues[]
            {
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Left,
                     ForeColor = new ThemableColor(ThemeColorType.Accent1)
                },
                new CellPropertiesValues()
                {
                     HorizontalAlignment = RadHorizontalAlignment.Right,
                     IsWrapped = true
                }
            };

            int expectedValuesCount = 15;

            FillSeriesTestHelper.AssertFillPropertiesCorrectness(initinalValues, expectedValuesCount,
                (cellSelection) =>
                {
                    cellSelection.FillDataSeriesLinear(CellOrientation.Horizontal, 1);
                });
        }

        #endregion
    }
}