﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Filtering
{
    [TestClass]
    public class CustomFilterTests
    {
        private static readonly string stringCriteria = "test";
        private static readonly string numberCriteria = "10";
        private static readonly string boolCriteria = "true";

        private static readonly Worksheet worksheet = TestHelper.CreateWorksheet();
        private static readonly CellIndex cellIndex = new CellIndex(0, 0);

        #region String criteria tests

        [TestMethod]
        public void ShouldShowValueTestStringEqualsPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("test", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringEqualsCaseInsensitive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TeST", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringEqualsNegative()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("other", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringGreaterThanPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("werty", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringGreaterThanNegative()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TEST", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringGreaterThanOrEqualPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TEST", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringGreaterThanOrEqualNegative()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TES", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringLessThanPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TES", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringLessThanNegative1()
        {
            ICellValue cellValue = CellValueFactory.Create(1);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringLessThanNegative2()
        {
            ICellValue cellValue = CellValueFactory.Create(true);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringLessThanOrEqualPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("TEST", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestStringLessThanOrEqualNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(1);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, stringCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        #endregion


        #region Number criteria tests

        [TestMethod]
        public void ShouldShowValueTestNumberEqualsPositive()
        {
            ICellValue cellValue = CellValueFactory.Create(10.0);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberEqualsPositive2()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("10", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberEqualsNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(1.2);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        //TODO:
        //[TestMethod]
        //public void ShouldShowValueTestNumberEqualsNegative2()
        //{
        //    ICellValue cellValue;
        //    CellValueFormat newFormat;
        //    CellValueFactory.Create("10.0", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

        //    CustomFilter filter = new CustomFilter(new CustomFilterCriteria(ComparisonOperator.EqualsTo, numberCriteria));
        //    bool shouldShow = filter.ShouldShowValue(cellValue);

        //    Assert.IsFalse(shouldShow);
        //}

        [TestMethod]
        public void ShouldShowValueTestNumberGreaterThanPositive()
        {
            ICellValue cellValue = CellValueFactory.Create(10.5);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberGreaterThanNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(10);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberGreaterThanNegative2()
        {
            ICellValue cellValue = CellValueFactory.Create(true);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberGreaterThanNegative3()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("test", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestNumberLessThanOrEqualPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("=2+3+5", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        #endregion


        #region Bool criteria

        [TestMethod]
        public void ShouldShowValueTestBoolEqualsPositive()
        {
            ICellValue cellValue = CellValueFactory.Create(true);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        //TODO:
        //[TestMethod]
        //public void ShouldShowValueTestBoolEqualsPositive2()
        //{
        //    ICellValue cellValue;
        //    CellValueFormat newFormat;
        //    CellValueFactory.Create("=\"TRUE\"", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

        //    CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, boolCriteria));
        //    bool shouldShow = filter.ShouldShowValue(cellValue);

        //    Assert.IsTrue(shouldShow);
        //}

        [TestMethod]
        public void ShouldShowValueTestBoolEqualsNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(1);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolGreaterThanNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(true);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolGreaterThanNegative2()
        {
            ICellValue cellValue = CellValueFactory.Create(1);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolLessThanPositive()
        {
            ICellValue cellValue = CellValueFactory.Create(false);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolLessThanNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(true);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolLessThanNegative2()
        {
            ICellValue cellValue = CellValueFactory.Create(1);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.LessThan, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestBoolLessThenOrEqualNegative()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("=\"TRUE\"", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.EqualsTo, boolCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        #endregion

        #region Two conditions

        [TestMethod]
        public void ShouldShowValueTestLessThanOrGreaterThanPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("=2+2+5", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, 
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, numberCriteria),
                LogicalOperator.Or,
                new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestLessThanOrGreaterThanNegative()
        {
            ICellValue cellValue = CellValueFactory.Create(11);

            CustomFilter filter = new CustomFilter(0, 
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, numberCriteria),
                LogicalOperator.Or,
                new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, "12"));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestLessThanAndGreaterThanPositive()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("=2+4+5", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, 
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, "12"),
                LogicalOperator.And,
                new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsTrue(shouldShow);
        }

        [TestMethod]
        public void ShouldShowValueTestLessThanAndGreaterThanNegative()
        {
            ICellValue cellValue;
            CellValueFormat newFormat;
            CellValueFactory.Create("=2+2+5", worksheet, cellIndex, CellValueFormat.GeneralFormat, out cellValue, out newFormat);

            CustomFilter filter = new CustomFilter(0, 
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, numberCriteria),
                LogicalOperator.And,
                new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, numberCriteria));
            bool shouldShow = filter.ShouldShowValue(cellValue);

            Assert.IsFalse(shouldShow);
        }

        #endregion
    }
}
