﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Filtering
{
    [TestClass]
    public class TopFilterTests
    {
        private static readonly CellRange range = new CellRange(0, 0, 10, 0);

        [TestMethod]
        [ExpectedException(typeof(FilteringException))]
        public void UseFilterBeforeAssigning()
        {
            ICellValue cellValue = CellValueFactory.Create(2);
            IFilter filter = new TopFilter(0, TopFilterType.TopNumber, 5);

            filter.ShouldShowValue(cellValue);
        }

        [TestMethod]
        public void ShouldShowValueTopItems()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelper.SetValuesToWorksheetColumn(worksheet, 1, 0, new List<object>()
            {
                1d, 2d, 3d, 4d, 5d, 6d, 7d, "test1", "test2", true
            });

            IFilter filter = new TopFilter(0, TopFilterType.TopNumber, 5);
            worksheet.Filter.FilterRange = range;
            worksheet.Filter.SetFilter(filter);

            this.AssertShouldShow(worksheet, filter, new CellIndex(1, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(2, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(3, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(4, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(5, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(6, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(7, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(8, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(9, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(10, 0), false);
        }

        [TestMethod]
        public void ShouldShowValueBottomItems()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelper.SetValuesToWorksheetColumn(worksheet, 1, 0, new List<object>()
            {
                1d, 2d, 3d, 4d, 5d, 6d, 7d, "test1", "test2", true
            });

            IFilter filter = new TopFilter(0, TopFilterType.BottomNumber, 5);
            worksheet.Filter.FilterRange = range;
            worksheet.Filter.SetFilter(filter);

            this.AssertShouldShow(worksheet, filter, new CellIndex(1, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(2, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(3, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(4, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(5, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(6, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(7, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(8, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(9, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(10, 0), false);
        }

        [TestMethod]
        public void ShouldShowValueTopPercent()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelper.SetValuesToWorksheetColumn(worksheet, 1, 0, new List<object>()
            {
                1d, 2d, 3d, 4d, 5d, 6d, 7d, "test1", "test2", true
            });

            IFilter filter = new TopFilter(0, TopFilterType.TopPercent, 50);
            worksheet.Filter.FilterRange = range;
            worksheet.Filter.SetFilter(filter);

            this.AssertShouldShow(worksheet, filter, new CellIndex(1, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(2, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(3, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(4, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(5, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(6, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(7, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(8, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(9, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(10, 0), false);
        }

        [TestMethod]
        public void ShouldShowValueBottomPercent()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelper.SetValuesToWorksheetColumn(worksheet, 1, 0, new List<object>()
            {
                1d, 2d, 3d, 4d, 5d, 6d, 7d, "test1", "test2", true
            });

            IFilter filter = new TopFilter(0, TopFilterType.BottomPercent, 50);
            worksheet.Filter.FilterRange = range;
            worksheet.Filter.SetFilter(filter);

            this.AssertShouldShow(worksheet, filter, new CellIndex(1, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(2, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(3, 0), true);
            this.AssertShouldShow(worksheet, filter, new CellIndex(4, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(5, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(6, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(7, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(8, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(9, 0), false);
            this.AssertShouldShow(worksheet, filter, new CellIndex(10, 0), false);
        }

        private void AssertShouldShow(Worksheet worksheet, IFilter filter, CellIndex cell, bool expected)
        {
            ICellValue cellValue = worksheet.Cells[cell].GetValue().Value;
            Assert.AreEqual(expected, filter.ShouldShowValue(cellValue));
        }
    }
}
