﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class FindAndReplaceTests
    {
        #region Methods

        private Workbook InitializeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            workbook.Worksheets.Add();

            Worksheet worksheet = workbook.Worksheets[0];

            worksheet.Cells[1, 1].SetValue("test");
            worksheet.Cells[2, 1].SetValue("test");
            worksheet.Cells[2, 3].SetValue("test");
            worksheet.Cells[2, 5].SetValue("test");
            worksheet.Cells[3, 1].SetValue("test123");
            worksheet.Cells[4, 1].SetValue("Test123");
            worksheet.Cells[5, 3].SetValue("Test");
            worksheet.Cells[7, 3].SetValue("sum");
            worksheet.Cells[7, 5].SetValue("Sum");
            worksheet.Cells[10, 6].SetValue("=sum(5, 4)");

            Worksheet worksheet1 = workbook.Worksheets[1];

            worksheet1.Cells[1, 1].SetValue("test");
            worksheet1.Cells[2, 1].SetValue("test");
            worksheet1.Cells[2, 3].SetValue("test");
            worksheet1.Cells[2, 5].SetValue("test");
            worksheet1.Cells[3, 1].SetValue("test123");
            worksheet1.Cells[4, 1].SetValue("Test123");
            worksheet1.Cells[5, 3].SetValue("Test");
            worksheet1.Cells[7, 3].SetValue("sum");
            worksheet1.Cells[7, 5].SetValue("Sum");
            worksheet1.Cells[10, 6].SetValue("=sum(5, 4)");

            workbook.Sheets.ActiveSheet = worksheet;

            return workbook;
        }

        private ReplaceOptions InitializeReplaceOptions(string findWhat, string replaceWith)
        {
            return new ReplaceOptions()
            {
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = findWhat,
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = replaceWith
            };
        }

        private void AssertReplaceAllTest(ReplaceOptions replaceOptions, int expectedReplacesCount, KeyValuePair<string, string>[] beforeAndAfterReplaceValues)
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();

            Worksheet worksheet = workbook.Worksheets[0];

            replaceOptions.StartCell = new WorksheetCellIndex(worksheet, 0, 0);

            for (int i = 0; i < beforeAndAfterReplaceValues.Length; i++)
            {
                worksheet.Cells[i, i].SetValue(beforeAndAfterReplaceValues[i].Key);
            }

            int replacesCount = workbook.ReplaceAll(replaceOptions);

            Assert.AreEqual(expectedReplacesCount, replacesCount);

            for (int i = 0; i < beforeAndAfterReplaceValues.Length; i++)
            {
                string actualValue = worksheet.Cells[i, i].GetValue().Value.RawValue;
                Assert.AreEqual(beforeAndAfterReplaceValues[i].Value, actualValue);
            }
        }

        #endregion


        #region Find Tests

        [TestMethod]
        public void FindSingleValueTest()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest1()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 10, 6));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 10, 6));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest2()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Values,
                FindWhat = "sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest3()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "Sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = true,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest4()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "SUM",
                FindWithin = FindWithin.Workbook,
                MatchCase = true,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 10, 6));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 10, 6));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest5()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = true,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest6()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = true,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest7()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Columns,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest8()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                SearchRanges = new CellRange[]
                {
                    new CellRange(1, 1, 3, 5)
                }
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        [TestMethod]
        public void FindSingleValueTest9()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
                SearchRanges = new CellRange[]
                {
                    new CellRange(1, 1, 3, 5)
                }
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));

            FindResult result;
            for (int i = 0; i < expectedResults.Count; i++)
            {
                result = workbook.Find(options);
                options.StartCell = result.FoundCell;

                Assert.AreEqual(expectedResults[i], result.FoundCell);
            }
        }

        #endregion


        #region Find All Tests

        [TestMethod]
        public void FindAllValueTest()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest1()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 10, 6));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 10, 6));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest2()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Values,
                FindWhat = "sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest3()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "Sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = true,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest4()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "SUM",
                FindWithin = FindWithin.Workbook,
                MatchCase = true,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 10, 6));

            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 10, 6));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest5()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = true,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest6()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = true,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest7()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Columns,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest8()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                SearchRanges = new CellRange[]
                {
                    new CellRange(1, 1, 3, 5)
                }
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        [TestMethod]
        public void FindAllValueTest9()
        {
            Workbook workbook = InitializeWorkbook();

            FindOptions options = new FindOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
                SearchRanges = new CellRange[]
                {
                    new CellRange(1, 1, 3, 5)
                }
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));

            List<FindResult> results = workbook.FindAll(options).ToList();

            Assert.AreEqual(expectedResults.Count, results.Count);

            for (int i = 0; i < results.Count; i++)
            {
                Assert.AreEqual(expectedResults[i], results[i].FoundCell);
            }
        }

        #endregion


        #region Replace Tests

        [TestMethod]
        public void ReplaceSingleValueTest()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

           Assert.IsFalse(workbook.Replace(options));
        }

        [TestMethod]
        public void ReplaceSingleValueTest1()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

            Assert.IsFalse(workbook.Replace(options));
        }

        [TestMethod]
        public void ReplaceSingleValueTest2()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 1, 1),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

            Assert.IsTrue(workbook.Replace(options));

            Assert.AreEqual("alabala", workbook.Worksheets[0].Cells[1, 1].GetValue().Value.RawValue);

            Assert.AreEqual("test", workbook.Worksheets[0].Cells[2, 1].GetValue().Value.RawValue);

            options.StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 3, 1);
            Assert.IsTrue(workbook.Replace(options));

            Assert.AreEqual("alabala123", workbook.Worksheets[0].Cells[3, 1].GetValue().Value.RawValue);

            Assert.AreEqual("test", workbook.Worksheets[0].Cells[2, 1].GetValue().Value.RawValue);
        }

        #endregion


        #region Replace All Tests

        [TestMethod]
        public void ReplaceAllValueTest()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 5, 3));

            int replacesCount = workbook.ReplaceAll(options);

            Assert.AreEqual(expectedResults.Count, replacesCount);

            foreach (var result in expectedResults)
            {
                Assert.IsTrue(result.Worksheet.Cells[result.CellIndex].GetValue().Value.RawValue.Contains(options.ReplaceWith));
            }
        }

        [TestMethod]
        public void ReplaceAllValueTest1()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "test",
                FindWithin = FindWithin.Sheet,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 1, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 2, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 3, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 4, 1));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 5, 3));

            int replacesCount = workbook.ReplaceAll(options);

            Assert.AreEqual(expectedResults.Count, replacesCount);

            foreach (var result in expectedResults)
            {
                Assert.IsTrue(result.Worksheet.Cells[result.CellIndex].GetValue().Value.RawValue.Contains(options.ReplaceWith));
            }
        }

        [TestMethod]
        public void ReplaceAllValueTest2()
        {
            Workbook workbook = InitializeWorkbook();

            ReplaceOptions options = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 0, 0),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "sum",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "alabala"
            };

            List<WorksheetCellIndex> expectedResults = new List<WorksheetCellIndex>();
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[0], 10, 6));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 3));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 7, 5));
            expectedResults.Add(new WorksheetCellIndex(workbook.Worksheets[1], 10, 6));

            int replacesCount = workbook.ReplaceAll(options);

            Assert.AreEqual(expectedResults.Count, replacesCount);

            foreach (var result in expectedResults)
            {
                string rawValue = result.Worksheet.Cells[result.CellIndex].GetValue().Value.RawValue;
                Assert.IsTrue(rawValue.ToLower().Contains(options.ReplaceWith.ToLower()));
            }
        }

        [TestMethod]
        public void ReplaceCaseInsensitive()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("sha", "d");

            this.AssertReplaceAllTest(replaceOptions, 10, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d d"),
                new KeyValuePair<string, string>("sHaa", "da"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "dlala"),
                new KeyValuePair<string, string>("shalalashasha", "dlaladd"),
                new KeyValuePair<string, string>("  sha  ", "  d  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceCaseInsensitiveWithWholeMatch()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("sha", "d");
            replaceOptions.MatchEntireCellContents = true;

            this.AssertReplaceAllTest(replaceOptions, 2, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "sha sha"),
                new KeyValuePair<string, string>("sHaa", "sHaa"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "shalala"),
                new KeyValuePair<string, string>("shalalashasha", "shalalashasha"),
                new KeyValuePair<string, string>("  sha  ", "  sha  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceCaseSensitive()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("sha", "d");
            replaceOptions.MatchCase = true;

            this.AssertReplaceAllTest(replaceOptions, 7, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d d"),
                new KeyValuePair<string, string>("sHaa", "sHaa"),
                new KeyValuePair<string, string>("Sha", "Sha"),
                new KeyValuePair<string, string>("shalala", "dlala"),
                new KeyValuePair<string, string>("shalalashAsha", "dlalashAd"),
                new KeyValuePair<string, string>("  sha  ", "  d  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceCaseSensitiveWithWholeMatch()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("sha", "d");
            replaceOptions.MatchCase = true;
            replaceOptions.MatchEntireCellContents = true;

            this.AssertReplaceAllTest(replaceOptions, 1, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "sha sha"),
                new KeyValuePair<string, string>("sHaa", "sHaa"),
                new KeyValuePair<string, string>("Sha", "Sha"),
                new KeyValuePair<string, string>("shalala", "shalala"),
                new KeyValuePair<string, string>("shalalashasha", "shalalashasha"),
                new KeyValuePair<string, string>("  sha  ", "  sha  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        #endregion


        #region Wildcard characters tests

        [TestMethod]
        public void ReplaceSingleWildcard()
        {
            Workbook workbook = InitializeWorkbook();
            ReplaceOptions replaceOptions = new ReplaceOptions()
            {
                StartCell = new WorksheetCellIndex(workbook.Worksheets[0], 1, 1),
                FindBy = FindBy.Rows,
                FindIn = FindInContentType.Formulas,
                FindWhat = "*",
                FindWithin = FindWithin.Workbook,
                MatchCase = false,
                MatchEntireCellContents = false,
                ReplaceWith = "d"
            };

            bool result = workbook.Replace(replaceOptions);

            Assert.IsTrue(result);
            Assert.AreEqual("d", workbook.Worksheets[0].Cells[1, 1, 1, 1].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void ReplaceWildcardsCaseInsensitive()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("?a", "d");

            this.AssertReplaceAllTest(replaceOptions, 14, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "sd"),
                new KeyValuePair<string, string>("sha sha", "sd sd"),
                new KeyValuePair<string, string>("sHaa", "sda"),
                new KeyValuePair<string, string>("Sha", "Sd"),
                new KeyValuePair<string, string>("shalala", "sddd"),
                new KeyValuePair<string, string>("shalalashasha", "sdddsdsd"),
                new KeyValuePair<string, string>("  sha  ", "  sd  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceWildcardsCaseInsensitiveWithWholeMatch()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("sh*", "d");
            replaceOptions.MatchEntireCellContents = true;

            this.AssertReplaceAllTest(replaceOptions, 6, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d"),
                new KeyValuePair<string, string>("sHaa", "d"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "d"),
                new KeyValuePair<string, string>("shalalashasha", "d"),
                new KeyValuePair<string, string>("  sha  ", "  sha  "),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceWildcardsCaseSensitive()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("?ha*", "d");
            replaceOptions.MatchCase = true;

            this.AssertReplaceAllTest(replaceOptions, 6, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d"),
                new KeyValuePair<string, string>("sHaa", "sHaa"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "d"),
                new KeyValuePair<string, string>("shalalashAsha", "d"),
                new KeyValuePair<string, string>("  sha  ", "  d"),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "  "),
            });
        }

        [TestMethod]
        public void ReplaceWildcardsCaseSensitiveWithWholeMatch()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("*", "d");
            replaceOptions.MatchEntireCellContents = true;

            this.AssertReplaceAllTest(replaceOptions, 8, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d"),
                new KeyValuePair<string, string>("sHaa", "d"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "d"),
                new KeyValuePair<string, string>("shalalashasha", "d"),
                new KeyValuePair<string, string>("  sha  ", "d"),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "d"),
            });
        }

        [TestMethod]
        public void ReplaceWildcardsAll()
        {
            ReplaceOptions replaceOptions = this.InitializeReplaceOptions("*", "d");

            this.AssertReplaceAllTest(replaceOptions, 8, new KeyValuePair<string, string>[]
            {
                new KeyValuePair<string, string>("sha", "d"),
                new KeyValuePair<string, string>("sha sha", "d"),
                new KeyValuePair<string, string>("sHaa", "d"),
                new KeyValuePair<string, string>("Sha", "d"),
                new KeyValuePair<string, string>("shalala", "d"),
                new KeyValuePair<string, string>("shalalashasha", "d"),
                new KeyValuePair<string, string>("  sha  ", "d"),
                new KeyValuePair<string, string>("", ""),
                new KeyValuePair<string, string>("  ", "d"),
            });
        }

        #endregion
    }
}
