﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class FreezePanesTests
    {
        [TestMethod]
        public void FreezeBothRowAndColumnNotScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.ViewState.FreezePanes(2, 3);

            Pane expectedPane = new Pane(new CellIndex(2, 3), 3, 2, ViewportPaneType.Scrollable);
            CellIndex expectedWorksheetTopLeft = new CellIndex(0,0);
            Assert.AreEqual(expectedWorksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeBothRowAndColumnScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(4, 5);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 3);

            Pane expectedPane = new Pane(new CellIndex(6, 8), 3, 2, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeBothRowAndColumnScrollablePaneScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(0, 0);
            CellIndex scrollablePaneTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 3, scrollablePaneTopLeft);

            Pane expectedPane = new Pane(scrollablePaneTopLeft, 3, 2, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void FreezeBothRowAndColumnScrollablePaneScrolledInvalidTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(0, 0);
            CellIndex scrollablePaneTopLeft = new CellIndex(1, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 3, scrollablePaneTopLeft);
        }

        [TestMethod]
        public void FreezeRowNotScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            
            worksheet.ViewState.FreezePanes(2, 0);

            CellIndex expectedWorksheetTopLeft = new CellIndex(0, 0);
            Pane expectedPane = new Pane(new CellIndex(2, 0), 0, 2, ViewportPaneType.Scrollable);
            Assert.AreEqual(expectedWorksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeColumnNotScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            
            worksheet.ViewState.FreezePanes(0, 3);

            CellIndex expectedWorksheetTopLeft = new CellIndex(0, 0);
            Pane expectedPane = new Pane(new CellIndex(0, 3), 3, 0, ViewportPaneType.Scrollable);
            Assert.AreEqual(expectedWorksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeRowScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 0);

            Pane expectedPane = new Pane(new CellIndex(12, 11), 0, 2, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeColumnScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 0, 3);

            Pane expectedPane = new Pane(new CellIndex(10, 14), 3, 0, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeRowScrollablePaneScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(0, 0);
            CellIndex scrollablePaneTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 0, scrollablePaneTopLeft);

            Pane expectedPane = new Pane(scrollablePaneTopLeft, 0, 2, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        public void FreezeColumnScrollablePaneScrolledTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(0, 0);
            CellIndex scrollablePaneTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 0, 3, scrollablePaneTopLeft);

            Pane expectedPane = new Pane(scrollablePaneTopLeft, 3, 0, ViewportPaneType.Scrollable);
            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.AreEqual(expectedPane, worksheet.ViewState.Pane);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void FreezeRowScrollablePaneScrolledInvalidTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(10, 11);
            CellIndex scrollablePaneTopLeft = new CellIndex(5, 12);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 0, scrollablePaneTopLeft);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void FreezeColumnScrollablePaneScrolledInvalidTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(10, 11);
            CellIndex scrollablePaneTopLeft = new CellIndex(11, 5);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 0, 3, scrollablePaneTopLeft);
        }

        [TestMethod]
        public void UnfreezePanesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex worksheetTopLeft = new CellIndex(3, 4);
            CellIndex scrollablePaneTopLeft = new CellIndex(10, 11);

            worksheet.ViewState.FreezePanes(worksheetTopLeft, 2, 3, scrollablePaneTopLeft);
            worksheet.ViewState.FreezePanes(0, 0);

            Assert.AreEqual(worksheetTopLeft, worksheet.ViewState.TopLeftCellIndex);
            Assert.IsNull(worksheet.ViewState.Pane);
        }
    }
}
