﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class DateFunctionsTests
    {
        [TestMethod]
        public void Date()
        {
            Date date = new Date();

            FunctionsTestHelper.AssertFunctionResult(date, 367, 1, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 1, 0, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 59, 0, 2, 28);
            FunctionsTestHelper.AssertFunctionResult(date, 61, 0, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 61, 1900, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 426, 1901, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 18323, 1950, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 675760, 1850, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 1, -1, 13, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 1, 0, 4, -90);
            FunctionsTestHelper.AssertFunctionResult(date, 1, 0, 3, -59);
            FunctionsTestHelper.AssertFunctionResult(date, 92, 1, -8, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 183, 1, -5, 1);
            FunctionsTestHelper.AssertFunctionResult(date, 41414, 2013, 5, 20);
            FunctionsTestHelper.AssertFunctionResult(date, 367, true, true, true);
            FunctionsTestHelper.AssertFunctionResult(date, 1, false, true, true);
            FunctionsTestHelper.AssertFunctionResult(date, 675760, "1850", "3", "1");

            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, -1, 12, 0);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, 0, 12, -365);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, -1, 1, 400);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, -1, 1, 800);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, -1, 13, -1);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.ValueError, "", 1, 1);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.ValueError, "text", true, true);
        }

        [TestMethod]
        public void Day()
        {
            Day day = new Day();

            FunctionsTestHelper.AssertFunctionResult(day, 31, 0);
            FunctionsTestHelper.AssertFunctionResult(day, 31, false);
            FunctionsTestHelper.AssertFunctionResult(day, 10, 10);
            FunctionsTestHelper.AssertFunctionResult(day, 10, "10");
            FunctionsTestHelper.AssertFunctionResult(day, 1, true);
            FunctionsTestHelper.AssertFunctionResult(day, 15, 41320);
            FunctionsTestHelper.AssertFunctionResult(day, 20, 39558);
            FunctionsTestHelper.AssertFunctionResult(day, 15, "41320");
            FunctionsTestHelper.AssertFunctionResult(day, 20, "39558");

            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.NumberError, -10);
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.NumberError, 3000000);
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Month()
        {
            Month month = new Month();

            FunctionsTestHelper.AssertFunctionResult(month, 12, 0);
            FunctionsTestHelper.AssertFunctionResult(month, 12, false);

            FunctionsTestHelper.AssertFunctionResult(month, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(month, 1, true);
            FunctionsTestHelper.AssertFunctionResult(month, 1, 10);
            FunctionsTestHelper.AssertFunctionResult(month, 1, "10");
            FunctionsTestHelper.AssertFunctionResult(month, 2, 41320);
            FunctionsTestHelper.AssertFunctionResult(month, 4, 39558);
            FunctionsTestHelper.AssertFunctionResult(month, 2, "41320");
            FunctionsTestHelper.AssertFunctionResult(month, 4, "39558");

            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.NumberError, -10);
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.NumberError, 3000000);
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Year()
        {
            Year year = new Year();

            FunctionsTestHelper.AssertFunctionResult(year, 1899, 0);
            FunctionsTestHelper.AssertFunctionResult(year, 1899, false);
            FunctionsTestHelper.AssertFunctionResult(year, 1900, 1);
            FunctionsTestHelper.AssertFunctionResult(year, 1900, true);
            FunctionsTestHelper.AssertFunctionResult(year, 1900, 10);
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "10");
            FunctionsTestHelper.AssertFunctionResult(year, 2013, 41320);
            FunctionsTestHelper.AssertFunctionResult(year, 2008, 39558);
            FunctionsTestHelper.AssertFunctionResult(year, 2013, "41320");
            FunctionsTestHelper.AssertFunctionResult(year, 2008, "39558");

            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.NumberError, -10);
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.NumberError, 3000000);
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.ValueError, "text");
        }

        // TODO: Fails in Korean culture ko-KR
        [TestMethod]
        public void DateValue()
        {
            DateValue dateValue = new DateValue();
            Date date = new Date();

            FunctionsTestHelper.AssertFunctionResult(dateValue, 39682, new DateTime(2008, 8, 22).ToShortDateString());
            FunctionsTestHelper.AssertFunctionResult(dateValue, 39682, "22-Aug-2008");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 39501, "2008/02/23");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 41635, "2013/12/27");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 407255, "3015/01/09");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 10, "1900/01/10");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 336, "1900-12-1");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 0, "19:54:30");
            FunctionsTestHelper.AssertFunctionResult(dateValue, 0, "6:00 AM");

            RadExpression[] arguments = new RadExpression[] { new NumberExpression(DateTime.Now.Year - 1900), new NumberExpression(7), new NumberExpression(5) };
            FunctionEvaluationContext<RadExpression> context = new FunctionEvaluationContext<RadExpression>(arguments, null, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(dateValue, date.Evaluate(context), "5-JUL");

            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "1900/01/350");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "1900/01/35");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "1900/13/01");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "-1900/13/01");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, 0);
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "text");
   
        }

        [TestMethod]
        public void Time()
        {
            Time time = new Time();

            FunctionsTestHelper.AssertFunctionResult(time, 0, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(time, 0.298784722222222, 7, 10, 15);
            FunctionsTestHelper.AssertFunctionResult(time, 0.112268518518519, 1, 100, 100);
            FunctionsTestHelper.AssertFunctionResult(time, 0.0451388888888889, 1, 5, 0);
            FunctionsTestHelper.AssertFunctionResult(time, 0.1701388888888889, 100, 5, 0);
            FunctionsTestHelper.AssertFunctionResult(time, 0.0423726851851852, 1, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(time, 0, false, false, false);
            FunctionsTestHelper.AssertFunctionResult(time, 0.0423726851851852, true, true, true);


            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.ValueError, "", 1, 1);
            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.ValueError, "text", true, true);

        }

        [TestMethod]
        public void Second()
        {
            Second second = new Second();

            FunctionsTestHelper.AssertFunctionResult(second, 59, 0.573599537037037);
            FunctionsTestHelper.AssertFunctionResult(second, 40, 0.112268518518519);
            FunctionsTestHelper.AssertFunctionResult(second, 0, 0.0451388888888889);
            FunctionsTestHelper.AssertFunctionResult(second, 38, 0.131);
            FunctionsTestHelper.AssertFunctionResult(second, 12, 0.78);
            FunctionsTestHelper.AssertFunctionResult(second, 10, 0.86400462962963);
            FunctionsTestHelper.AssertFunctionResult(second, 24, 0.91);
            FunctionsTestHelper.AssertFunctionResult(second, 59, (0.573599537037037).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionResult(second, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(second, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(second, 0, "1");
            FunctionsTestHelper.AssertFunctionResult(second, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(second, 0, true);
            FunctionsTestHelper.AssertFunctionResult(second, 0, false);


            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.NumberError, "-5");
            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Minute()
        {
            Minute minute = new Minute();

            FunctionsTestHelper.AssertFunctionResult(minute, 45, 0.573599537037037);
            FunctionsTestHelper.AssertFunctionResult(minute, 45, (0.573599537037037).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(minute, 41, 0.112268518518519);
            FunctionsTestHelper.AssertFunctionResult(minute, 5, 0.0451388888888889);
            FunctionsTestHelper.AssertFunctionResult(minute, 50, 0.91);
            FunctionsTestHelper.AssertFunctionResult(minute, 17, 0.0123);
            FunctionsTestHelper.AssertFunctionResult(minute, 53, 0.412);
            FunctionsTestHelper.AssertFunctionResult(minute, 7, 0.38);

            FunctionsTestHelper.AssertFunctionResult(minute, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(minute, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(minute, 0, "1");
            FunctionsTestHelper.AssertFunctionResult(minute, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(minute, 0, true);
            FunctionsTestHelper.AssertFunctionResult(minute, 0, false);


            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.NumberError, "-5");
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Hour()
        {
            Hour hour = new Hour();

            FunctionsTestHelper.AssertFunctionResult(hour, 13, 0.573599537037037);
            FunctionsTestHelper.AssertFunctionResult(hour, 13, (0.573599537037037).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(hour, 2, 0.112268518518519);
            FunctionsTestHelper.AssertFunctionResult(hour, 1, 0.0451388888888889);
            FunctionsTestHelper.AssertFunctionResult(hour, 9, 0.38);
            FunctionsTestHelper.AssertFunctionResult(hour, 21, 0.91);
            FunctionsTestHelper.AssertFunctionResult(hour, 16, 0.67);
            FunctionsTestHelper.AssertFunctionResult(hour, 2, 0.10101);

            FunctionsTestHelper.AssertFunctionResult(hour, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(hour, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(hour, 0, "1");
            FunctionsTestHelper.AssertFunctionResult(hour, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(hour, 0, true);
            FunctionsTestHelper.AssertFunctionResult(hour, 0, false);


            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.NumberError, "-5");
            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void TimeValue()
        {
            TimeValue timeValue = new TimeValue();

            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.1, "2:24 AM");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.274305555554747, "22-Aug-2008 6:35 AM");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.787847222222222, "18:54:30");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.999988425925926, "23:59:59");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.999988425925926, "23:59:59");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.834027777777778, "8:01 PM");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.334027777777778, "8:01 AM");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0.334027777777778, "08:01 AM");
            FunctionsTestHelper.AssertFunctionResult(timeValue, 0, "24:00:00");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.000520833333333, "24:00:45");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.0421845, "25:00:45");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.75, "42:00:00");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.298611111111111, "6:70:00");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.347222222222222, "6:140:00");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.028935185185185, "0:40:100");
            //FunctionsTestHelper.AssertFunctionResult(timeValue, 0.347222222222222, "6:140:00");

            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "25:70:00");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "25:40:100");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "0:70:100");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, false);

            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, 0);
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "text");

        }

        [TestMethod]
        public void Days()
        {
            Days days = new Days();

            FunctionsTestHelper.AssertFunctionResult(days, 1, 42959, 42958);
            FunctionsTestHelper.AssertFunctionResult(days, -1, 42958, 42959);
            FunctionsTestHelper.AssertFunctionResult(days, 1, "8/12/2017", "8/11/2017");
            FunctionsTestHelper.AssertFunctionResult(days, -1, "8/11/2017", "8/12/2017");
            FunctionsTestHelper.AssertFunctionError(days, ErrorExpressions.ValueError, "test", "test");
            FunctionsTestHelper.AssertFunctionError(days, ErrorExpressions.NumberError, 2958466, 2958465);
            FunctionsTestHelper.AssertFunctionResult(days, 1, 42959.5, 42958.6);
            FunctionsTestHelper.AssertFunctionResult(days, 2, "3/1/1900", "2/28/1900");
            FunctionsTestHelper.AssertFunctionResultWithReferenceToArgumentsInSeparateCells(days, "1", "4/2/1986", "4/1/1986");
            FunctionsTestHelper.AssertFunctionResultWithReferenceToArgumentsInSeparateCells(days, "1", "true", "false");
            FunctionsTestHelper.AssertFunctionResultWithReferenceToArgumentsInSeparateCells(days, "0", "", "");
            FunctionsTestHelper.AssertFunctionResultWithReferenceToArgumentsInSeparateCells(days, "-59", "", "2/28/1900");
        }
    }
}
