﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class EngineeringFunctionsTests
    {
        #region BesselI tests

        [TestMethod]
        public void BesselICorrectnessTest()
        {
            BesselI besselI = new BesselI();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0.981666428, accuracy, 1.5, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 1, accuracy, 0, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0, accuracy, 0, 100000);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 2.82960561, accuracy, 2.3, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0.349223222, accuracy, 2.3, 3.4);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0.565159098, accuracy, "1", "1");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0.565159098, accuracy, 1, 1.618);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 0.144003421, accuracy, 23, 34);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 2.618494818, accuracy, Math.PI, Math.E);

            accuracy = 1E-6;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 3.32084E+12, accuracy, 60, "60");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 6.33658E+14, accuracy, "-70", 70);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 2.23438690749371E+25, accuracy, 100, 90);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 1.617319483915E+24, accuracy, 70, 40);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 1.20158844027E+29, accuracy, 70, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselI, 5.85925129038407E+28, accuracy, 70, 10);
        }

        [TestMethod]
        public void BesselIErrorTest()
        {
            BesselI besselI = new BesselI();
            
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.NumberError, 1, -0.4);
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.NumberError, 10.5, -5);
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.NumberError, 0, -1);

            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.ValueError, 10.5, "asfd");
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.ValueError, "", "e");
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.ValueError, "ass", -1);
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.ValueError, 1.5, true);
            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.ValueError, false, true);
        }

        [TestMethod]
        public void BesselITooBigArgumentsValuesTest()
        {
            BesselI besselI = new BesselI();

            FunctionsTestHelper.AssertFunctionError(besselI, ErrorExpressions.NumberError, 1000, 1000);
        }

        #endregion


        #region BesselJ tests

        [TestMethod]
        public void BesselJCorrectnessTest()
        {
            BesselJ besselJ = new BesselJ();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.557936507890804, accuracy, 1.5, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 1, accuracy, 0, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0, accuracy, 0, 100000);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.44005058567713, accuracy, "1", "1");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.44005058567713, accuracy, 1, 1.618);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.485433935293123, accuracy, Math.PI, Math.E);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.0555397865782638, accuracy, 2.3, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.179978934935148, accuracy, 2.3, 3.4);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.329925828669785, accuracy, 1.9, 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, -0.124230136902855, accuracy, -70, 60);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, -0.113948667359755, accuracy, -70, 50);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, -0.0914718040406352, accuracy, 60, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.0558123275994173, accuracy, 50, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.00222052182115523, accuracy, 80, 90);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 4.60655306482347E-06, accuracy, -80, 100);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 7.10986139163999E-05, accuracy, 23, 34);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselJ, 0.114252082213003, accuracy, 60, "60");
        }

        [TestMethod]
        public void BesselJErrorTest()
        {
            BesselJ besselJ = new BesselJ();

            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.NumberError, 1, -0.4);
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.NumberError, 10.5, -5);
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.NumberError, 0, -1);

            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.ValueError, 10.5, "asfd");
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.ValueError, "", "e");
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.ValueError, "ass", -1);
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.ValueError, 1.5, true);
            FunctionsTestHelper.AssertFunctionError(besselJ, ErrorExpressions.ValueError, false, true);
        }

        #endregion


        #region BesselK tests

        [TestMethod]
        public void BesselKCorrectnessTest()
        {
            BesselK besselK = new BesselK();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.277387804, accuracy, 1.5, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.079139931, accuracy, 2.3, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.376257862, accuracy, 2.3, 3.4);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0, accuracy, 1000, 1000);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.601907232, accuracy, "1", "1");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.601907232, accuracy, 1, 1.618);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.084580979, accuracy, 23, 34);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 0.05109869, accuracy, Math.PI, Math.E);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 1.77439E-15, accuracy, 60, "60");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 1.6633E-28, accuracy, 100, 90);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 3.83456E-27, accuracy, 70, 40);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 5.94466E-32, accuracy, 70, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselK, 1.20685E-31, accuracy, 70, 10);
        }

        [TestMethod]
        public void BesselKErrorTest()
        {
            BesselK besselK = new BesselK();
            
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, 0, 100000);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, "-70", 70);

            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, 1, -0.4);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, 10.5, -5);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.NumberError, 0, -1);

            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.ValueError, 10.5, "asfd");
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.ValueError, "", "e");
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.ValueError, "ass", -1);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.ValueError, 1.5, true);
            FunctionsTestHelper.AssertFunctionError(besselK, ErrorExpressions.ValueError, false, true);
        }

        #endregion


        #region BesselY tests

        [TestMethod]
        public void BesselYCorrectnessTest()
        {
            BesselY besselY = new BesselY();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.412308627, accuracy, 1.5, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, 0.518075392, accuracy, 2.3, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.874219675, accuracy, 2.3, 3.4);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.077476002, accuracy, 1000, 1000);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.781212821, accuracy, "1", "1");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.781212821, accuracy, 1, 1.618);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -179.0514609, accuracy, 23, 34);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.09990072, accuracy, Math.PI, Math.E);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.197912521, accuracy, 60, "60");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, 0.09815246, accuracy, 100, 90);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.067431169, accuracy, 70, 40);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, -0.069639384, accuracy, 70, 10);

            accuracy = 1E-7;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(besselY, 0.009309666, accuracy, 70, 0);
        }

        [TestMethod]
        public void BesselYErrorTest()
        {
            BesselY besselY = new BesselY();

            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, 0, 100000);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, "-70", 70);

            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, 1, -0.4);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, 10.5, -5);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.NumberError, 0, -1);

            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.ValueError, 10.5, "asfd");
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.ValueError, "", "e");
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.ValueError, "ass", -1);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.ValueError, 1.5, true);
            FunctionsTestHelper.AssertFunctionError(besselY, ErrorExpressions.ValueError, false, true);
        }

        #endregion


        #region Bin2Dec

        [TestMethod]
        public void Bin2DecCorrectnessTests()
        {
            Bin2Dec bin2Dec = new Bin2Dec();

            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 2, 10);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 0, "000");
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 0, "00000000000000000000000");
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 1, "0000000001");
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, -1, "1111111111");
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, -1, "00000000000000000001111111111");
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, -512, 1000000000);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, -509, 1000000011);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, -256, 1100000000);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 256, 100000000);
            FunctionsTestHelper.AssertFunctionResult(bin2Dec, 31, "11{gs}111".ReplaceCultureDependentSeparators());
        }

        [TestMethod]
        public void Bin2DecErrorTests()
        {
            Bin2Dec bin2Dec = new Bin2Dec();

            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.NumberError, "11111111111");
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.NumberError, 1002);
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.NumberError, 1.100);

            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, "1{gs}11".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, "true");
            FunctionsTestHelper.AssertFunctionError(bin2Dec, ErrorExpressions.ValueError, "false");
        }

        #endregion


        #region Bin2Oct

        [TestMethod]
        public void Bin2OctCorrectnessTests()
        {
            Bin2Oct bin2Oct = new Bin2Oct();

            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "10", 1000);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "10", "1{gs}000".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "7777777777", "1111111111", "10");
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "7777777777", 1111111111, "5");
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "7777777777", "1{gs}111{gs}111{gs}111".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "777", 111111111);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "154", 00000001101100);
            FunctionsTestHelper.AssertFunctionResult(bin2Oct, "154", "00000001101100");
        }

        [TestMethod]
        public void Bin2OctErrorTests()
        {
            Bin2Oct bin2Oct = new Bin2Oct();

            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, 11111111111);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, 111111111, 2);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.NumberError, "11111111111");

            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(bin2Oct, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Bin2Hex

        [TestMethod]
        public void Bin2HexCorrectnessTests()
        {
            Bin2Hex bin2Hex = new Bin2Hex();

            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "10", 10000);
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "10", "10{gs}000".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "FFFFFFFFFF", "1111111111", "10");
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "FFFFFFFFFF", 1111111111, "5");
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "FFFFFFFFFF", "1{gs}111{gs}111{gs}111".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "1FF", 111111111);
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "6C", "00000001101100");
            FunctionsTestHelper.AssertFunctionResult(bin2Hex, "6C", 00000001101100);
        }

        [TestMethod]
        public void Bin2HexErrorTests()
        {
            Bin2Hex bin2Hex = new Bin2Hex();

            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, 549755813886, 9);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, 549755813887);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, -549755813888);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, 11111111111);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, 111111111, 2);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.NumberError, "11111111111");

            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(bin2Hex, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Dec2Oct

        [TestMethod]
        public void Dec2OctCorrectnessTests()
        {
            Dec2Oct dec2Oct = new Dec2Oct();

            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "10", 8);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "0000000011", "9", "10");
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "4000000000", -536870912);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "4000000000", -536870912, 4);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "3777777777", 536870911);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "7743546000", -7418880);
            FunctionsTestHelper.AssertFunctionResult(dec2Oct, "34232000", 7418880);
        }

        [TestMethod]
        public void Dec2OctErrorTests()
        {
            Dec2Oct dec2Oct = new Dec2Oct();

            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, "9", "11");
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, "9", "-1");
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.NumberError, -536870913);

            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(dec2Oct, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Dec2Bin

        [TestMethod]
        public void Dec2BinCorrectnessTests()
        {
            Dec2Bin dec2Bin = new Dec2Bin();

            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "1000", 8);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "0000001001", "9", "10");
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "1000000000", -512);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "1000000000", -512, 4);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "111111111", 511);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "1110010100", -108);
            FunctionsTestHelper.AssertFunctionResult(dec2Bin, "1101100", 108);
        }

        [TestMethod]
        public void Dec2BinErrorTests()
        {
            Dec2Bin dec2Bin = new Dec2Bin();

            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, "9", "11");
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, "9", "-1");
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, 511, 8);
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, 512);
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.NumberError, -513);

            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(dec2Bin, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Dec2Hex

        [TestMethod]
        public void Dec2HexCorrectnessTests()
        {
            Dec2Hex dec2Hex = new Dec2Hex();

            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "100", 256);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "00000000FF", "255", "10");
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "FFFFFFFE00", -512);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "1FF", 511);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "FFFF8ECC00", -7418880);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "713400", 7418880);

            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "FFFFFFFE00", -512, 4);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "8000000000", -549755813888);
            FunctionsTestHelper.AssertFunctionResult(dec2Hex, "7FFFFFFFFF", 549755813887);
        }

        [TestMethod]
        public void Dec2HexErrorTests()
        {
            Dec2Hex dec2Hex = new Dec2Hex();

            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, "9", "11");
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, "9", "-1");
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, 511, 2);
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, -549755813889);
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.NumberError, 549755813888);

            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(dec2Hex, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Oct2Bin

        [TestMethod]
        public void Oct2BinCorrectnessTests()
        {
            Oct2Bin oct2Bin = new Oct2Bin();

            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1000000", 100);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1111111111", "7777777777", "10");
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1111111111", 7777777777, "5");
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1111111111", "7{gs}777{gs}777{gs}777".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "111111111", 777);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1101100", 0000000154);
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1101100", "0000000154");
            FunctionsTestHelper.AssertFunctionResult(oct2Bin, "1001001", "111");
        }

        [TestMethod]
        public void Oct2BinErrorTests()
        {
            Oct2Bin oct2Bin = new Oct2Bin();

            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 536870911);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 11111111111);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 1000);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, 111, 2);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.NumberError, "1111");

            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(oct2Bin, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Oct2Dec

        [TestMethod]
        public void Oct2DecCorrectnessTests()
        {
            Oct2Dec oct2Dec = new Oct2Dec();

            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 8, 10);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 0, "000");
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 0, "00000000000000000000000");
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 1, "0000000001");
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, -1, "7777777777");
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, -1, "00000000000000000007777777777");
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, -536870912, 4000000000);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, -536870903, 4000000011);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, -419430400, 4700000000);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 536870911, 3777777777);
            FunctionsTestHelper.AssertFunctionResult(oct2Dec, 4681, "11{gs}111".ReplaceCultureDependentSeparators());
        }

        [TestMethod]
        public void Oct2DecErrorTests()
        {
            Oct2Dec oct2Dec = new Oct2Dec();

            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.NumberError, "11111111111");
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.NumberError, 1008);
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.NumberError, 1.100);

            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, "1{gs}11".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, "true");
            FunctionsTestHelper.AssertFunctionError(oct2Dec, ErrorExpressions.ValueError, "false");
        }

        #endregion


        #region Oct2Hex

        [TestMethod]
        public void Oct2HexCorrectnessTests()
        {
            Oct2Hex oct2Hex = new Oct2Hex();

            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "40", 100);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "1C0", "000000000000700");
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFFFFFFFFF", "7777777777", "10");
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFFFFFFFFF", 7777777777, "5");
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFFFFFFFFF", "7{gs}777{gs}777{gs}777".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFE0002641", 4000023101, "5");
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFE0002641", 4000023101);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "FFE0000000", 4000000000);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "7FFFFFF", 777777777);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "6C", 0000000154);
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "6C", "0000000154");
            FunctionsTestHelper.AssertFunctionResult(oct2Hex, "000049", "111", 6);
        }

        [TestMethod]
        public void Oct2HexErrorTests()
        {
            Oct2Hex oct2Hex = new Oct2Hex();

            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 536870911);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 11111111111);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 8000000000);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, 500000000, 2);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.NumberError, "10000000000");

            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.ValueError, "asdas", -1);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(oct2Hex, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Hex2Bin

        [TestMethod]
        public void Hex2BinCorrectnessTests()
        {
            Hex2Bin hex2Bin = new Hex2Bin();

            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "100000000", 100);
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "1111111111", "FFFFFFFFFF", "10");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "111111111", "1FF");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "101010100", 0000000154);
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "101010100", "0000000154");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "100010001", "111");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "10101111", "af");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "10101111", "aF");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "10101111", "Af");
            FunctionsTestHelper.AssertFunctionResult(hex2Bin, "1100000001", "FFFFFFFF01");
        }

        [TestMethod]
        public void Hex2BinErrorTests()
        {
            Hex2Bin hex2Bin = new Hex2Bin();

            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 536870911);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 1111);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 1000);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, 111, 2);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "asdas");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "1111");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "asvas");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "false");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "FALSE");
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.NumberError, "TRUE");

            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(hex2Bin, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Hex2Oct

        [TestMethod]
        public void Hex2OctCorrectnessTests()
        {
            Hex2Oct hex2Oct = new Hex2Oct();

            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "0", 0);
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "0", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "000000", 0, 6);
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "400", 100);
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "7777777777", "FFFFFFFFFF", "10");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "3777777777", "1FFFFFFF");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "524", 0000000154);
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "524", "0000000154");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "421", "111");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "52746757", "abcdef");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "52746757", "AbcDeF");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "52746757", "aBCdEf");
            FunctionsTestHelper.AssertFunctionResult(hex2Oct, "7777777401", "FFFFFFFF01");
        }

        [TestMethod]
        public void Hex2OctErrorTests()
        {
            Hex2Oct hex2Oct = new Hex2Oct();

            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "11", "11");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "11", "-1");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 536870911, 9);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 536870911);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 536870912);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 111111111);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "1FFFFFFFF");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, 111, 2);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "asdas");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "111111111");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "asvas");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "false");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "FALSE");
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.NumberError, "TRUE");

            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.ValueError, 234224, true);
            FunctionsTestHelper.AssertFunctionError(hex2Oct, ErrorExpressions.ValueError, false, 0);
        }

        #endregion


        #region Hex2Dec

        [TestMethod]
        public void Hex2DecCorrectnessTests()
        {
            Hex2Dec hex2Dec = new Hex2Dec();

            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 256, 100);
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, -1, "FFFFFFFFFF");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 549755813887, "7fffffffff");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, -549755813888, "8000000000");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 340, 000000000000154);
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 340, "0000000154");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 273, "111");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 11259375, "abcdef");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 11259375, "AbcDeF");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, 11259375, "aBCdEf");
            FunctionsTestHelper.AssertFunctionResult(hex2Dec, -255, "FFFFFFFF01");
        }

        [TestMethod]
        public void Hex2DecErrorTests()
        {
            Hex2Dec hex2Dec = new Hex2Dec();

            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "7fffffffff0");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "07fffffffff");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "80000000000");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, -536870913);
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "00000000001");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "asdas");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "asvas");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "false");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "FALSE");
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.NumberError, "TRUE");

            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(hex2Dec, ErrorExpressions.ValueError, false);
        }

        #endregion


        #region BitAnd

        [TestMethod]
        public void BitAndCorrectnessTests()
        {
            BitAnd bitAnd = new BitAnd();

            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 7418880, 7418880, 7418880);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 281474976710654L, 281474976710655L, 281474976710654L);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, 281474976710655L, false);
            FunctionsTestHelper.AssertFunctionResult(bitAnd, 0, true, 281474976710654L);
        }

        [TestMethod]
        public void BitAndErrorTests()
        {
            BitAnd bitAnd = new BitAnd();

            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.NumberError, 0, 0.5);
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.NumberError, 4535.0000001, 2);
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.NumberError, 1, 281474976710656L);
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.NumberError, 32, -1);
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.NumberError, -32, 1);

            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(bitAnd, ErrorExpressions.ValueError, "abv", "asfd");
        }

        #endregion


        #region BitOr

        [TestMethod]
        public void BitOrCorrectnessTests()
        {
            BitOr bitOr = new BitOr();

            FunctionsTestHelper.AssertFunctionResult(bitOr, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 3, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 3, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 7418880, 7418880, 7418880);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 281474976710655L, 281474976710655L, 281474976710654L);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 281474976710655L, 281474976710655L, false);
            FunctionsTestHelper.AssertFunctionResult(bitOr, 281474976710655L, true, 281474976710654L);
        }

        [TestMethod]
        public void BitOrErrorTests()
        {
            BitOr bitOr = new BitOr();

            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.NumberError, 0, 0.5);
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.NumberError, 4535.0000001, 2);
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.NumberError, 1, 281474976710656L);
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.NumberError, 32, -1);
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.NumberError, -32, 1);

            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(bitOr, ErrorExpressions.ValueError, "abv", "asfd");
        }

        #endregion


        #region BitXor

        [TestMethod]
        public void BitXorCorrectnessTests()
        {
            BitXor bitXor = new BitXor();

            FunctionsTestHelper.AssertFunctionResult(bitXor, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 3, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 3, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 0, 7418880, 7418880);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 1, 281474976710655L, 281474976710654L);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 281474976710655L, 281474976710655L, false);
            FunctionsTestHelper.AssertFunctionResult(bitXor, 281474976710655L, true, 281474976710654L);
        }

        [TestMethod]
        public void BitXorErrorTests()
        {
            BitXor bitXor = new BitXor();

            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.NumberError, 0, 0.5);
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.NumberError, 4535.0000001, 2);
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.NumberError, 1, 281474976710656L);
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.NumberError, 32, -1);
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.NumberError, -32, 1);

            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(bitXor, ErrorExpressions.ValueError, "abv", "asfd");
        }

        #endregion


        #region BitLShift

        [TestMethod]
        public void BitLShiftCorrectnessTests()
        {
            BitLShift bitLShift = new BitLShift();

            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 4, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 4, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 2, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 1, -1);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 0, 53);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 0, -53);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 281474976710655L, -53);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 0, 281474976710655L, -48);
            FunctionsTestHelper.AssertFunctionResult(bitLShift, 281474976710655L, 281474976710655L, false);
        }

        [TestMethod]
        public void BitLShiftErrorTests()
        {
            BitLShift bitLShift = new BitLShift();

            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 0, 0.5);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 4535.0000001, 2);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 1, 281474976710656L);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, -32, 1);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 1, 54);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 1, -54);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 1, 53);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, 1, 48);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.NumberError, true, 281474976710654L);

            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(bitLShift, ErrorExpressions.ValueError, "abv", "asfd");
        }

        #endregion


        #region BitRShift

        [TestMethod]
        public void BitRShiftCorrectnessTests()
        {
            BitRShift bitRShift = new BitRShift();

            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 4, 2, -1);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 4, 1, -2);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 2, 1, -1);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 2, 8, 2);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 0, 53);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 0, -53);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 281474976710655L, 53);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 0, 281474976710655L, 48);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 1, 281474976710655L, 47);
            FunctionsTestHelper.AssertFunctionResult(bitRShift, 281474976710655L, 281474976710655L, false);
        }

        [TestMethod]
        public void BitRShiftErrorTests()
        {
            BitRShift bitRShift = new BitRShift();

            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 0, 0.5);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 4535.0000001, 2);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 1, -281474976710656L);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, -32, 1);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 1, 54);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 1, -54);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 1, -53);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, 1, -48);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.NumberError, true, -281474976710654L);

            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(bitRShift, ErrorExpressions.ValueError, "abv", "asfd");
        }

        #endregion


        #region Convert

        [TestMethod]
        public void ConvertCorrectnessTests()
        {
            var convert = new Telerik.Windows.Documents.Spreadsheet.Expressions.Functions.Convert();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 0, accuracy, 0, "kg", "g");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 1E+3, accuracy, 1, "kg", "g");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 1E-3, accuracy, 1, "mg", "g");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 1741.600705, accuracy, 1.618, "ar", "ft2");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 1.618, accuracy, 1741.600705, "ft2", "ar");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, 34.9124, accuracy, 1.618, "C", "F");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(convert, -16.87888889, accuracy, 1.618, "F", "C");
        }

        [TestMethod]
        public void ConvertErrorTests()
        {
            var convert = new Telerik.Windows.Documents.Spreadsheet.Expressions.Functions.Convert();

            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.NotAvailableError, 0, "asfd", "g");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.NotAvailableError, 1, "g", 0);
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.NotAvailableError, 1, 5, "g");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.NotAvailableError, 1, "ar", "g");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.NotAvailableError, 1, "kg", "ft2");

            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.ValueError, true, "g", "kg");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.ValueError, false, "g", "kg");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.ValueError, 1, true, "kg");
            FunctionsTestHelper.AssertFunctionError(convert, ErrorExpressions.ValueError, 1, "g", false);
        }

        #endregion


        #region Delta

        [TestMethod]
        public void DeltaCorrectnessTests()
        {
            Delta delta = new Delta();

            FunctionsTestHelper.AssertFunctionResult(delta, 1, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(delta, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(delta, 0, 1E-8);
            FunctionsTestHelper.AssertFunctionResult(delta, 0, -1E-8);
            FunctionsTestHelper.AssertFunctionResult(delta, 0, 7418880, 7418880.00000001);
            FunctionsTestHelper.AssertFunctionResult(delta, 1, 7418880, 7418880);
            FunctionsTestHelper.AssertFunctionResult(delta, 0, -1E-8, 1E-8);
        }

        [TestMethod]
        public void DeltaErrorTests()
        {
            Delta delta = new Delta();

            FunctionsTestHelper.AssertFunctionError(delta, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(delta, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(delta, ErrorExpressions.ValueError, "abv", "asfd");
            FunctionsTestHelper.AssertFunctionError(delta, ErrorExpressions.ValueError, true, 0);
            FunctionsTestHelper.AssertFunctionError(delta, ErrorExpressions.ValueError, 1, false);
        }

        #endregion


        #region Erf

        [TestMethod]
        public void ErfCorrectnessTests()
        {
            Erf erf = new Erf();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 0.70792892, accuracy, 0.745);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 0, accuracy, 0, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 0.995322265, accuracy, 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, -0.995322265, accuracy, -2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 0.84270079, accuracy, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, -0.84270079, accuracy, -1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 1, accuracy, 5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, -1, accuracy, -5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, 0.221601087, accuracy, 0.5, 0.8);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erf, -0.221601087, accuracy, 0.8, 0.5);
        }

        [TestMethod]
        public void ErfErrorTests()
        {
            Erf erf = new Erf();

            FunctionsTestHelper.AssertFunctionError(erf, ErrorExpressions.ValueError, "asfd", 0.5);
            FunctionsTestHelper.AssertFunctionError(erf, ErrorExpressions.ValueError, 0, "asfd");
            FunctionsTestHelper.AssertFunctionError(erf, ErrorExpressions.ValueError, "abv", "asfd");
            FunctionsTestHelper.AssertFunctionError(erf, ErrorExpressions.ValueError, true, 0);
            FunctionsTestHelper.AssertFunctionError(erf, ErrorExpressions.ValueError, 1, false);
        }

        #endregion


        #region ErfPrecise

        [TestMethod]
        public void ErfPreciseCorrectnessTests()
        {
            ErfPrecise erfPrecise = new ErfPrecise();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0, accuracy, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0.70792892, accuracy, 0.745);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0.995322265, accuracy, 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, -0.995322265, accuracy, -2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0.84270079, accuracy, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, -0.84270079, accuracy, -1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 1, accuracy, 5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, -1, accuracy, -5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0.742100965, accuracy, 0.8);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfPrecise, 0.520499878, accuracy, 0.5);
        }

        [TestMethod]
        public void ErfPreciseErrorTests()
        {
            ErfPrecise erfPrecise = new ErfPrecise();

            FunctionsTestHelper.AssertFunctionError(erfPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfPrecise, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(erfPrecise, ErrorExpressions.ValueError, false);
        }

        #endregion


        #region ErfC

        [TestMethod]
        public void ErfCCorrectnessTests()
        {
            ErfC erfC = new ErfC();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 1, accuracy, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 0.29207108, accuracy, 0.745);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 0.004677735, accuracy, 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 1.995322265, accuracy, -2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 0.157299207, accuracy, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 1.842700793, accuracy, -1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 2, accuracy, -5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 0.257899035, accuracy, 0.8);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 0.479500122, accuracy, 0.5);

            accuracy = 1E-6;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfC, 1.53746E-12, accuracy, 5);
        }

        [TestMethod]
        public void ErfCErrorTests()
        {
            ErfC erfC = new ErfC();

            FunctionsTestHelper.AssertFunctionError(erfC, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfC, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfC, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfC, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(erfC, ErrorExpressions.ValueError, false);
        }

        #endregion


        #region ErfCPrecise

        [TestMethod]
        public void ErfCPreciseCorrectnessTests()
        {
            ErfCPrecise erfCPrecise = new ErfCPrecise();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 1, accuracy, 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 0.29207108, accuracy, 0.745);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 0.004677735, accuracy, 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 1.995322265, accuracy, -2);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 0.157299207, accuracy, 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 1.842700793, accuracy, -1);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 2, accuracy, -5);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 0.257899035, accuracy, 0.8);
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 0.479500122, accuracy, 0.5);

            accuracy = 1E-6;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(erfCPrecise, 1.53746E-12, accuracy, 5);
        }

        [TestMethod]
        public void ErfCPreciseErrorTests()
        {
            ErfCPrecise erfCPrecise = new ErfCPrecise();

            FunctionsTestHelper.AssertFunctionError(erfCPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfCPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfCPrecise, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(erfCPrecise, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(erfCPrecise, ErrorExpressions.ValueError, false);
        }

        #endregion


        #region GEStep

        [TestMethod]
        public void GEStepCorrectnessTests()
        {
            GEStep geStep = new GEStep();

            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 0.745);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(geStep, 0, -2);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(geStep, 0, -1E-100);
            FunctionsTestHelper.AssertFunctionResult(geStep, 0, 0, 1E-100);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 0, -0.1);
            FunctionsTestHelper.AssertFunctionResult(geStep, 1, 4.2, 2.3);
            FunctionsTestHelper.AssertFunctionResult(geStep, 0, 2.3, 4.2);
        }

        [TestMethod]
        public void GEStepErrorTests()
        {
            GEStep geStep = new GEStep();

            FunctionsTestHelper.AssertFunctionError(geStep, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(geStep, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(geStep, ErrorExpressions.ValueError, "asfd");
            FunctionsTestHelper.AssertFunctionError(geStep, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(geStep, ErrorExpressions.ValueError, false);
        }

        #endregion


        #region Complex helper methods

        public void RunComplexArgumentsFunctionNumberErrorTests(FunctionBase functionWithComplexArgument)
        {
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "k");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "  ");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, " ");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "   1E- 100 +   2e10  i   ");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, " +-  1E-100 +   2e10  i   ");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, " + +  1E-100 +   2e10  i   ");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "++2e10i");

            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "2e +10i");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "2e+ 10i");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "2e1 0i");
            FunctionsTestHelper.AssertFunctionError(functionWithComplexArgument, ErrorExpressions.NumberError, "1E-i");
        }

        #endregion


        #region Complex

        [TestMethod]
        public void ComplexCorrectnessTests()
        {
            Complex complex = new Complex();

            FunctionsTestHelper.AssertFunctionResult(complex, "1", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(complex, "i", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(complex, "-i", 0, -1);
            FunctionsTestHelper.AssertFunctionResult(complex, "1-i", 1, -1);
            FunctionsTestHelper.AssertFunctionResult(complex, "-1+i", -1, +1);
            FunctionsTestHelper.AssertFunctionResult(complex, "0", "", "");
            FunctionsTestHelper.AssertFunctionResult(complex, "0", "", "", "");
            FunctionsTestHelper.AssertFunctionResult(complex, "3+2j", "3", "2", "j");
            FunctionsTestHelper.AssertFunctionResult(complex, "2j", "", "2", "j");
            FunctionsTestHelper.AssertFunctionResult(complex, "3-2i", "3", -2, "i");
            FunctionsTestHelper.AssertFunctionResult(complex, "-4{ds}3-1E-100i".ReplaceCultureDependentSeparators(), -4.3, -1E-100);
            FunctionsTestHelper.AssertFunctionResult(complex, "1E-100i", 0, 1E-100);
            FunctionsTestHelper.AssertFunctionResult(complex, "-3{ds}14".ReplaceCultureDependentSeparators(), -3.14, "", "j");
        }

        [TestMethod]
        public void ComplexErrorTests()
        {
            Complex complex = new Complex();

            FunctionsTestHelper.AssertFunctionError(complex, ErrorExpressions.ValueError, "asfd", "0");
            FunctionsTestHelper.AssertFunctionError(complex, ErrorExpressions.ValueError, "0", "sd");
            FunctionsTestHelper.AssertFunctionError(complex, ErrorExpressions.ValueError, true, "20");
            FunctionsTestHelper.AssertFunctionError(complex, ErrorExpressions.ValueError, "2", false);
            FunctionsTestHelper.AssertFunctionError(complex, ErrorExpressions.ValueError, "0", "3", "k");
        }

        #endregion


        #region ImAbs

        [TestMethod]
        public void ImAbsCorrectnessTests()
        {
            ImAbs imAbs = new ImAbs();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1, accuracy, "i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1, accuracy, "j");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 5, accuracy, "3-4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1E10, accuracy, "-1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1E10, accuracy, "+1E+10");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1.23E-100, accuracy, "-1{ds}23E-100i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1, accuracy, "  - 1 i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1E10, accuracy, "  - 1E10 i  ");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1E10, accuracy, "- 1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1, accuracy, "- 1i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1, accuracy, "  -  i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 5, accuracy, " 3 - 4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1.3, accuracy, "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imAbs, 1.3, accuracy, " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());
        }
        
        [TestMethod]
        public void ImAbsErrorTests()
        {
            ImAbs imAbs = new ImAbs();

            this.RunComplexArgumentsFunctionNumberErrorTests(imAbs);
        }

        #endregion


        #region Imaginary

        [TestMethod]
        public void ImaginaryCorrectnessTests()
        {
            Imaginary imaginary = new Imaginary();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, 1, accuracy, "i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, 1, accuracy, "j");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1E10, accuracy, "-1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, 0, accuracy, "+1E+10");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1.23E-100, accuracy, "-1{ds}23E-100i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -4, accuracy, "3-4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1, accuracy, "  - 1 i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1E10, accuracy, "  - 1E10 i  ");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1E10, accuracy, "- 1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1, accuracy, "- 1i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1, accuracy, "  -  i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -4, accuracy, " 3 - 4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1.2, accuracy, "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imaginary, -1.2, accuracy, " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());
        }

        [TestMethod]
        public void ImaginaryErrorTests()
        {
            Imaginary imaginary = new Imaginary();

            this.RunComplexArgumentsFunctionNumberErrorTests(imaginary);
        }

        #endregion


        #region ImReal

        [TestMethod]
        public void ImRealCorrectnessTests()
        {
            ImReal imReal = new ImReal();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0, accuracy, "i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0, accuracy, "j");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0, accuracy, "-1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 1E10, accuracy, "+1E+10");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, -1.23E-100, accuracy, "-1{ds}23E-100-1{ds}23E-100i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 3, accuracy, "3-4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0, accuracy, "  - 1 i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, -1E10, accuracy, " - 1E10 - 1E10 i  ");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 2, accuracy, "2- 1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, -2, accuracy, "- 2 - 1i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0, accuracy, "  -  i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 3, accuracy, " 3 - 4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0.5, accuracy, "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imReal, 0.5, accuracy, " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());
        }
        
        [TestMethod]
        public void ImRealErrorTests()
        {
            ImReal imReal = new ImReal();

            this.RunComplexArgumentsFunctionNumberErrorTests(imReal);
        }

        #endregion


        #region ImArgument

        [TestMethod]
        public void ImArgumentCorrectnessTests()
        {
            ImArgument imArgument = new ImArgument();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, Math.PI / 2, accuracy, "i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, Math.PI / 2, accuracy, "j");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, 0, accuracy, "+1E+10");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "-1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "-1{ds}23E-100i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -0.927295218001612, accuracy, "3-4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "  - 1 i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "  - 1E10 i  ");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "- 1E10i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "- 1i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -Math.PI / 2, accuracy, "  -  i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -0.927295218001612, accuracy, " 3 - 4i");
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -1.17600520709514, accuracy, "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertAreEssentiallyEqualFunctionResult(imArgument, -1.17600520709514, accuracy, " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());
        
        }

        [TestMethod]
        public void ImArgumentErrorTests()
        {
            ImArgument imArgument = new ImArgument();

            this.RunComplexArgumentsFunctionNumberErrorTests(imArgument);

            FunctionsTestHelper.AssertFunctionError(imArgument, ErrorExpressions.DivisionByZero, "");
            FunctionsTestHelper.AssertFunctionError(imArgument, ErrorExpressions.DivisionByZero, "0");
            FunctionsTestHelper.AssertFunctionError(imArgument, ErrorExpressions.DivisionByZero, "0i");
            FunctionsTestHelper.AssertFunctionError(imArgument, ErrorExpressions.DivisionByZero, "0-0i");
        }

        #endregion


        #region ImConjugate

        [TestMethod]
        public void ImConjugateCorrectnessTests()
        {
            ImConjugate imConjugate = new ImConjugate();

            FunctionsTestHelper.AssertFunctionResult(imConjugate, "-i", "i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "-j", "j");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "10", "10");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "10i", "-10i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate,
                "1{ds}23i".ReplaceCultureDependentSeparators(),
                "-1{ds}23i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertFunctionResult(imConjugate, "3+4i", "3-4i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "i", "  - 1 i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "i", "- 1i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "i", "  -  i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate, "3+4i", " 3 - 4i");
            FunctionsTestHelper.AssertFunctionResult(imConjugate,
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators(),
                "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(imConjugate,
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators(),
                " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());
        }

        [TestMethod]
        public void ImConjugateErrorTests()
        {
            ImConjugate imConjugate = new ImConjugate();

            this.RunComplexArgumentsFunctionNumberErrorTests(imConjugate);
        }

        #endregion


        #region ImSum

        [TestMethod]
        public void ImSumCorrectnessTests()
        {
            ImSum imSum = new ImSum();

            FunctionsTestHelper.AssertFunctionResult(imSum, "-i", "-i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "j", "j");
            FunctionsTestHelper.AssertFunctionResult(imSum, "10", "10");
            FunctionsTestHelper.AssertFunctionResult(imSum, "10i", "10i");
            FunctionsTestHelper.AssertFunctionResult(imSum,
                "1{ds}23i".ReplaceCultureDependentSeparators(),
                "1{ds}23i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertFunctionResult(imSum, "3+4i", "3+4i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "-i", "  - 1 i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "-i", "- 1i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "-i", "  -  i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "3-4i", " 3 - 4i");
            FunctionsTestHelper.AssertFunctionResult(imSum,
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators(),
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(imSum,
                "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators(),
                " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertFunctionResult(imSum, "2", "-i", "2 +i");
            FunctionsTestHelper.AssertFunctionResult(imSum, "0", "-  j   ", "2 +j", "-1", "-1");
            FunctionsTestHelper.AssertFunctionResult(imSum, "-1-j", "- 2 j   ", "2 +j", "-1", "-2");
            FunctionsTestHelper.AssertFunctionResult(imSum, "1+i", "- 0 i   ", "2 +i", "-1", "-0");
            FunctionsTestHelper.AssertFunctionResult(imSum, "1", "-i", "2 +i", "-1", "");
        }

        [TestMethod]
        public void ImSumErrorTests()
        {
            ImSum imSum = new ImSum();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSum);

            FunctionsTestHelper.AssertFunctionError(imSum, ErrorExpressions.ValueError, "2", "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imSum, ErrorExpressions.ValueError, "2", "-j  ", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imSum, ErrorExpressions.ValueError, "2", "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imSum, ErrorExpressions.ValueError, "2i", "3-j ", "2");
            FunctionsTestHelper.AssertFunctionError(imSum, ErrorExpressions.ValueError, "2j", "3-i ", "2");
        }

        #endregion


        #region ImProduct

        [TestMethod]
        public void ImProductCorrectnessTests()
        {
            ImProduct imProduct = new ImProduct();

            FunctionsTestHelper.AssertFunctionResult(imProduct, "-i", "-i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "j", "j");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "10", "10");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "10i", "10i");
            FunctionsTestHelper.AssertFunctionResult(imProduct,
                "1{ds}23i".ReplaceCultureDependentSeparators(),
                "1{ds}23i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertFunctionResult(imProduct, "3+4i", "3+4i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "-i", "  - 1 i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "-i", "- 1i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "-i", "  -  i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "3-4i", " 3 - 4i");
            FunctionsTestHelper.AssertFunctionResult(imProduct,
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators(),
                "0{ds}5+1{ds}2i".ReplaceCultureDependentSeparators());
            FunctionsTestHelper.AssertFunctionResult(imProduct,
                "0{ds}5-1{ds}2i".ReplaceCultureDependentSeparators(),
                " 0{ds}5 - 1{ds}2i".ReplaceCultureDependentSeparators());

            FunctionsTestHelper.AssertFunctionResult(imProduct, "1-2i", "-i", "2 +i");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "1-2j", "-  j   ", "2 +j", "-1", "-1");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "4-8j", "- 2 j   ", "2 +j", "-1", "-2");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "0", "- 0 i   ", "2 +i", "-1", "-0");
            FunctionsTestHelper.AssertFunctionResult(imProduct, "0", "-i", "2 +i", "-1", "");
        }

        [TestMethod]
        public void ImProductErrorTests()
        {
            ImProduct imProduct = new ImProduct();

            this.RunComplexArgumentsFunctionNumberErrorTests(imProduct);

            FunctionsTestHelper.AssertFunctionError(imProduct, ErrorExpressions.ValueError, "2", "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imProduct, ErrorExpressions.ValueError, "2", "-j  ", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imProduct, ErrorExpressions.ValueError, "2", "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imProduct, ErrorExpressions.ValueError, "2i", "3-j ", "2");
            FunctionsTestHelper.AssertFunctionError(imProduct, ErrorExpressions.ValueError, "2j", "3-i ", "2");
        }

        #endregion


        #region ImSub

        [TestMethod]
        public void ImSubCorrectnessTests()
        {
            ImSub imSub = new ImSub();

            FunctionsTestHelper.AssertFunctionResult(imSub, "-2-2i", "-i", "2 +i");
            FunctionsTestHelper.AssertFunctionResult(imSub, "-2-2j", "-  j   ", "2 +j");
            FunctionsTestHelper.AssertFunctionResult(imSub, "2-2j", "- 2 j   ", "-2");
            FunctionsTestHelper.AssertFunctionResult(imSub, "0", "- 0 i   ", "-0");
            FunctionsTestHelper.AssertFunctionResult(imSub, "-2+i", "", "2 -i");
        }

        [TestMethod]
        public void ImSubErrorTests()
        {
            ImSub imSub = new ImSub();

            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "k", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "  ", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, " ", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "   1E- 100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, " +-  1E-100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, " + +  1E-100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "++2e10i", "0");

            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "0", "2e +10i");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "0", "2e+ 10i");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "0", "2e1 0i");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.NumberError, "1E-i", "0");

            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.ValueError, "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.ValueError, "2i", "3-j ");
            FunctionsTestHelper.AssertFunctionError(imSub, ErrorExpressions.ValueError, "2j", "3-i ");
        }

        #endregion


        #region ImDiv

        [TestMethod]
        public void ImDivCorrectnessTests()
        {
            ImDiv imDiv = new ImDiv();

            FunctionsTestHelper.AssertFunctionResult(imDiv, "-0{ds}2-0{ds}4i", "-i", "2 +i");
            FunctionsTestHelper.AssertFunctionResult(imDiv, "-0{ds}2-0{ds}4j", "-  j   ", "2 +j");
            FunctionsTestHelper.AssertFunctionResult(imDiv, "j", "- 2 j   ", "-2");
            FunctionsTestHelper.AssertFunctionResult(imDiv, "0", "", "2 -i");
        }

        [TestMethod]
        public void ImDivErrorTests()
        {
            ImDiv imDiv = new ImDiv();

            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "1", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "1", "");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "", "");

            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "k", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "  ", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, " ", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "   1E- 100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, " +-  1E-100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, " + +  1E-100 +   2e10  i   ", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "++2e10i", "0");

            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "2 e+10i", "0");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "0", "2e +10i");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "0", "2e+ 10i");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "0", "2e1 0i");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.NumberError, "1E-i", "0");

            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.ValueError, "-j", "2 +i");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.ValueError, "2i", "3-j ");
            FunctionsTestHelper.AssertFunctionError(imDiv, ErrorExpressions.ValueError, "2j", "3-i ");
        }

        #endregion


        #region ImPower

        [TestMethod]
        public void ImPowerCorrectnessTests()
        {
            ImPower imPower = new ImPower();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "1", accuracy, "i", 0);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "i", accuracy, "i", 1);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "-j", accuracy, "j", -1);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "-1", accuracy, "j", 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "3-4j", accuracy, "2-j", 2);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower, "i", accuracy, "-1", 0.5);            
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower,
                "20{ds}1247835664355-66{ds}6690231177843i".ReplaceCultureDependentSeparators(), accuracy,
                "3{ds}14 - 1{ds}2 i".ReplaceCultureDependentSeparators(), 3.5);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower,
                "0{ds}065942486565336+0{ds}0590221125417326i".ReplaceCultureDependentSeparators(), accuracy,
                "3{ds}14 - 1{ds}2 i".ReplaceCultureDependentSeparators(), -2);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower,
                "1{ds}80298194193065-0{ds}332782035164209i".ReplaceCultureDependentSeparators(), accuracy,
                "3{ds}14 - 1{ds}2 i".ReplaceCultureDependentSeparators(), 0.5);
            FunctionsTestHelper.AssertAreEssentiallyEqualComplexNumberFunctionResult(imPower,
                "0{ds}707106781186548+0{ds}707106781186547i".ReplaceCultureDependentSeparators(), accuracy,
                "i", 0.5);
        }

        [TestMethod]
        public void ImPowerErrorTests()
        {
            ImPower imPower = new ImPower();

            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "k", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "  ", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, " ", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "   1E- 100 +   2e10  i   ", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, " +-  1E-100 +   2e10  i   ", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, " + +  1E-100 +   2e10  i   ", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "++2e10i", 0);

            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "2e +10i", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "2e+ 10i", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "2e1 0i", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "1E-i", 0);

            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "0", 0);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.NumberError, "", 0);

            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "-j", "2");
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "-j", "");
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "i", "0");
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "2i", true);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "2j", false);
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, "k", "0");
            FunctionsTestHelper.AssertFunctionError(imPower, ErrorExpressions.ValueError, 0, "2e +10i");
        }

        #endregion


        #region ImCos

        [TestMethod]
        public void ImCosCorrectnessTests()
        {
            ImCos imCos = new ImCos();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imCos, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("-0{ds}64214812471552+1{ds}06860742138278i", "2-i"),
                new KeyValuePair<string, string>("1{ds}54308063481524", "j"),
                new KeyValuePair<string, string>("0{ds}54030230586814", "-1"),
                new KeyValuePair<string, string>("1", "0-0j"),
                new KeyValuePair<string, string>("-1", "3{ds}141592654"),
                new KeyValuePair<string, string>("", "1{ds}570796327"),
                new KeyValuePair<string, string>("0", "1{ds}570796327"),
                new KeyValuePair<string, string>("-8{ds}09051946670641+0{ds}0127865999570397j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("1", ""),
            });
        }

        [TestMethod]
        public void ImCosErrorTests()
        {
            ImCos imCos = new ImCos();

            this.RunComplexArgumentsFunctionNumberErrorTests(imCos);
        }

        #endregion


        #region ImCosh

        [TestMethod]
        public void ImCoshCorrectnessTests()
        {
            ImCosh imCosh = new ImCosh();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imCosh, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("2{ds}03272300701967-3{ds}0518977991518i", "2-i"),
                new KeyValuePair<string, string>("0{ds}54030230586814", "j"),
                new KeyValuePair<string, string>("1{ds}54308063481524", "-1"),
                new KeyValuePair<string, string>("1", "0-0j"),
                new KeyValuePair<string, string>("2{ds}50917847865806", "1{ds}570796327"),
                new KeyValuePair<string, string>("11{ds}5919532755215", "3{ds}141592654"),
                new KeyValuePair<string, string>("-3{ds}56076735314386E+46+1{ds}83674610037668E+46i", "108+7418880i"),
                new KeyValuePair<string, string>("-10{ds}8251648219767-4{ds}07900621126902j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("1", ""),
            });
        }

        [TestMethod]
        public void ImCoshErrorTests()
        {
            ImCosh imCosh = new ImCosh();

            this.RunComplexArgumentsFunctionNumberErrorTests(imCosh);
        }

        #endregion


        #region ImCot

        [TestMethod]
        public void ImCotCorrectnessTests()
        {
            ImCot imCot = new ImCot();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imCot, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("-0{ds}171383612909185+0{ds}821329797493852i", "2-i"),
                new KeyValuePair<string, string>("-1{ds}31303528549933j", "j"),
                new KeyValuePair<string, string>("-0{ds}642092615934331", "-1"),
                new KeyValuePair<string, string>("0", "1{ds}570796327"),
                new KeyValuePair<string, string>("-0{ds}0000247088594926504+1{ds}00772725376298j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImCotErrorTests()
        {
            ImCot imCot = new ImCot();

            this.RunComplexArgumentsFunctionNumberErrorTests(imCot);

            FunctionsTestHelper.AssertFunctionError(imCot, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(imCot, ErrorExpressions.NumberError, "");          
        }

        #endregion


        #region ImCsc

        [TestMethod]
        public void ImCscCorrectnessTests()
        {
            ImCsc imCsc = new ImCsc();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imCsc, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("0{ds}6354937992539-0{ds}221500930850509i", "2-i"),
                new KeyValuePair<string, string>("-0{ds}850918128239322j", "j"),
                new KeyValuePair<string, string>("-1{ds}18839510577812", "-1"),
                new KeyValuePair<string, string>("1", "1{ds}570796327"),
                new KeyValuePair<string, string>("0{ds}000199908015800742-0{ds}124556241631464j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImCscErrorTests()
        {
            ImCsc imCsc = new ImCsc();

            this.RunComplexArgumentsFunctionNumberErrorTests(imCsc);

            FunctionsTestHelper.AssertFunctionError(imCsc, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(imCsc, ErrorExpressions.NumberError, "");
        }

        #endregion


        #region ImCsch

        [TestMethod]
        public void ImCschCorrectnessTests()
        {
            ImCsch imCsch = new ImCsch();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imCsch, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("0{ds}141363021612408+0{ds}228375065599687i", "2-i"),
                new KeyValuePair<string, string>("-1{ds}18839510577812j", "j"),
                new KeyValuePair<string, string>("-0{ds}850918128239322", "-1"),
                new KeyValuePair<string, string>("0{ds}434537208094696", "1{ds}570796327"),
                new KeyValuePair<string, string>("-0{ds}0810434759006294+0{ds}0307675094311154j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImCschErrorTests()
        {
            ImCsch imCsch = new ImCsch();

            this.RunComplexArgumentsFunctionNumberErrorTests(imCsch);

            FunctionsTestHelper.AssertFunctionError(imCsch, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(imCsch, ErrorExpressions.NumberError, "");
        }

        #endregion


        #region ImExp

        [TestMethod]
        public void ImExpCorrectnessTests()
        {
            ImExp imExp = new ImExp();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imExp, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("3{ds}99232404844127-6{ds}21767631236797i", "2-i"),
                new KeyValuePair<string, string>("0{ds}54030230586814+0{ds}841470984807897j", "j"),
                new KeyValuePair<string, string>("0{ds}367879441171442", "-1"),
                new KeyValuePair<string, string>("1", "0-0j"),
                new KeyValuePair<string, string>("4{ds}81047738096535", "1{ds}570796327"),
                new KeyValuePair<string, string>("23{ds}1406926327793", "3{ds}141592654"),
                new KeyValuePair<string, string>("-7{ds}12153470628771E+46+3{ds}67349220075335E+46i", "108+7418880i"),
                new KeyValuePair<string, string>("-21{ds}6098457460916-8{ds}17332433320063j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("1", ""),
            });
        }

        [TestMethod]
        public void ImExpErrorTests()
        {
            ImExp imExp = new ImExp();

            this.RunComplexArgumentsFunctionNumberErrorTests(imExp);
        }

        #endregion


        #region ImLn

        [TestMethod]
        public void ImLnCorrectnessTests()
        {
            ImLn imLn = new ImLn();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imLn, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("0{ds}80471895621705-0{ds}463647609000806i", "2-i"),
                new KeyValuePair<string, string>("1{ds}5707963267949j", "j"),
                new KeyValuePair<string, string>("3{ds}14159265358979i", "-1"),
                new KeyValuePair<string, string>("0{ds}451582705289455", "1{ds}570796327"),
                new KeyValuePair<string, string>("1{ds}1447298858494", "3{ds}141592654"),
                new KeyValuePair<string, string>("15{ds}8195386604602+1{ds}57078176934148i", "108+7418880i"),
                new KeyValuePair<string, string>("1{ds}43360842565928-0{ds}724662145250158j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImLnErrorTests()
        {
            ImLn imLn = new ImLn();

            this.RunComplexArgumentsFunctionNumberErrorTests(imLn);

            FunctionsTestHelper.AssertFunctionError(imLn, ErrorExpressions.NumberError, "0-0j");
            FunctionsTestHelper.AssertFunctionError(imLn, ErrorExpressions.NumberError, "");           
        }

        #endregion


        #region ImLog10

        [TestMethod]
        public void ImLog10CorrectnessTests()
        {
            ImLog10 imLog10 = new ImLog10();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imLog10, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("0{ds}349485002168009-0{ds}201359598136687i", "2-i"),
                new KeyValuePair<string, string>("0{ds}682188176920921j", "j"),
                new KeyValuePair<string, string>("1{ds}36437635384184i", "-1"),
                new KeyValuePair<string, string>("0{ds}196119877030153", "1{ds}570796327"),
                new KeyValuePair<string, string>("0{ds}497149872694134", "3{ds}141592654"),
                new KeyValuePair<string, string>("6{ds}87033834649302+0{ds}682181854699232i", "108+7418880i"),
                new KeyValuePair<string, string>("0{ds}622608228473834-0{ds}314716770926316j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImLog10ErrorTests()
        {
            ImLog10 imLog10 = new ImLog10();

            this.RunComplexArgumentsFunctionNumberErrorTests(imLog10);

            FunctionsTestHelper.AssertFunctionError(imLog10, ErrorExpressions.NumberError, "0-0j");
            FunctionsTestHelper.AssertFunctionError(imLog10, ErrorExpressions.NumberError, "");
        }

        #endregion


        #region ImLog2

        [TestMethod]
        public void ImLog2CorrectnessTests()
        {
            ImLog2 imLog2 = new ImLog2();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imLog2, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("1{ds}16096404744368-0{ds}668902106225488i", "2-i"),
                new KeyValuePair<string, string>("2{ds}2661800709136j", "j"),
                new KeyValuePair<string, string>("4{ds}53236014182719i", "-1"),
                new KeyValuePair<string, string>("0{ds}651496129472319", "1{ds}570796327"),
                new KeyValuePair<string, string>("1{ds}65149612947232", "3{ds}141592654"),
                new KeyValuePair<string, string>("22{ds}8227699745972+2{ds}26615906894775i", "108+7418880i"),
                new KeyValuePair<string, string>("2{ds}06825976627528-1{ds}04546648327236j", "   3{ds}14 - 2{ds}78 j "),
            });
        }

        [TestMethod]
        public void ImLog2ErrorTests()
        {
            ImLog2 imLog2 = new ImLog2();

            this.RunComplexArgumentsFunctionNumberErrorTests(imLog2);

            FunctionsTestHelper.AssertFunctionError(imLog2, ErrorExpressions.NumberError, "0-0j");
            FunctionsTestHelper.AssertFunctionError(imLog2, ErrorExpressions.NumberError, "");
        }

        #endregion


        #region ImSqrt

        [TestMethod]
        public void ImSqrtCorrectnessTests()
        {
            ImSqrt imSqrt = new ImSqrt();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imSqrt, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("1{ds}45534669022535-0{ds}343560749722512i", "2-i"),
                new KeyValuePair<string, string>("0{ds}707106781186548+0{ds}707106781186547j", "j"),
                new KeyValuePair<string, string>("i", "-1"),
                new KeyValuePair<string, string>("0", "0-0j"),
                new KeyValuePair<string, string>("1{ds}2533141373155", "1{ds}570796327"),
                new KeyValuePair<string, string>("1{ds}77245385090552", "3{ds}141592654"),
                new KeyValuePair<string, string>("1926{ds}00467299357+1925{ds}97663547434i", "108+7418880i"),
                new KeyValuePair<string, string>("1{ds}91491578860933-0{ds}725880484284619j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("0", ""),
            });
        }

        [TestMethod]
        public void ImSqrtErrorTests()
        {
            ImSqrt imSqrt = new ImSqrt();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSqrt);
        }

        #endregion


        #region ImSinh

        [TestMethod]
        public void ImSinhCorrectnessTests()
        {
            ImSinh imSinh = new ImSinh();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imSinh, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("1{ds}95960104142161-3{ds}16577851321617i", "2-i"),
                new KeyValuePair<string, string>("0{ds}841470984807897j", "j"),
                new KeyValuePair<string, string>("-1{ds}1752011936438", "-1"),
                new KeyValuePair<string, string>("0", "0-0j"),
                new KeyValuePair<string, string>("2{ds}30129890230729", "1{ds}570796327"),
                new KeyValuePair<string, string>("11{ds}5487393572577", "3{ds}141592654"),
                new KeyValuePair<string, string>("-3{ds}56076735314386E+46+1{ds}83674610037668E+46i", "108+7418880i"),
                new KeyValuePair<string, string>("-10{ds}7846809241149-4{ds}09431812193161j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("0", ""),
            });
        }

        [TestMethod]
        public void ImSinhErrorTests()
        {
            ImSinh imSinh = new ImSinh();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSinh);
        }

        #endregion


        #region ImSin

        [TestMethod]
        public void ImSinCorrectnessTests()
        {
            ImSin imSin = new ImSin();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imSin, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("1{ds}40311925062204+0{ds}489056259041294i", "2-i"),
                new KeyValuePair<string, string>("1{ds}1752011936438j", "j"),
                new KeyValuePair<string, string>("-0{ds}841470984807897", "-1"),
                new KeyValuePair<string, string>("0", "0-0j"),
                new KeyValuePair<string, string>("1", "1{ds}570796327"),
                new KeyValuePair<string, string>("1{ds}22514845490862E-16", "3{ds}141592654"),
                new KeyValuePair<string, string>("0{ds}0128854057667487+8{ds}02848103801078j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("0", ""),
            });
        }

        [TestMethod]
        public void ImSinErrorTests()
        {
            ImSin imSin = new ImSin();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSin);

            FunctionsTestHelper.AssertFunctionError(imSin, ErrorExpressions.NumberError, "108+7418880i");
        }

        #endregion


        #region ImSec

        [TestMethod]
        public void ImSecCorrectnessTests()
        {
            ImSec imSec = new ImSec();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imSec, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("-0{ds}41314934426694-0{ds}687527438655479i", "2-i"),
                new KeyValuePair<string, string>("0{ds}648054273663885", "j"),
                new KeyValuePair<string, string>("1{ds}85081571768093", "-1"),
                new KeyValuePair<string, string>("1", "0-0j"),
                new KeyValuePair<string, string>("-1", "3{ds}141592654"),
                new KeyValuePair<string, string>("-0{ds}123601149014375-0{ds}000195344495885706j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("1", ""),
            });
        }

        [TestMethod]
        public void ImSecErrorTests()
        {
            ImSec imSec = new ImSec();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSec);
        }

        #endregion


        #region ImSech

        [TestMethod]
        public void ImSechCorrectnessTests()
        {
            ImSech imSech = new ImSech();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imSech, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("0{ds}151176298265577+0{ds}226973675393722i", "2-i"),
                new KeyValuePair<string, string>("1{ds}85081571768093", "j"),
                new KeyValuePair<string, string>("0{ds}648054273663885", "-1"),
                new KeyValuePair<string, string>("1", "0-0j"),
                new KeyValuePair<string, string>("0{ds}398536815338387", "1{ds}570796327"),
                new KeyValuePair<string, string>("0{ds}0862667383340544", "3{ds}141592654"),
                new KeyValuePair<string, string>("-2{ds}21817343422E-47-1{ds}14419758473283E-47i", "108+7418880i"),
                new KeyValuePair<string, string>("-0{ds}0808919727826736+0{ds}0304807238364136j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("1", ""),
            });
        }

        [TestMethod]
        public void ImSechErrorTests()
        {
            ImSech imSech = new ImSech();

            this.RunComplexArgumentsFunctionNumberErrorTests(imSech);
        }

        #endregion


        #region ImTan

        [TestMethod]
        public void ImTanCorrectnessTests()
        {
            ImTan imTan = new ImTan();

            double accuracy = 1E-8;
            FunctionsTestHelper.AssertMultipleTestsForAComplexComplexFunction(imTan, accuracy, new KeyValuePair<string, string>[]{
                new KeyValuePair<string, string>("-0{ds}243458201185725-1{ds}16673625724092i", "2-i"),
                new KeyValuePair<string, string>("0{ds}761594155955765j", "j"),
                new KeyValuePair<string, string>("-1{ds}5574077246549", "-1"),
                new KeyValuePair<string, string>("0", "0-0j"),
                new KeyValuePair<string, string>("-1{ds}22514845490862E-16", "3{ds}141592654"),
                new KeyValuePair<string, string>("-0{ds}0000243313771884217-0{ds}992331998231345j", "   3{ds}14 - 2{ds}78 j "),
                new KeyValuePair<string, string>("0", ""),
            });
        }

        [TestMethod]
        public void ImTanErrorTests()
        {
            ImTan imTan = new ImTan();

            this.RunComplexArgumentsFunctionNumberErrorTests(imTan);
        }

        #endregion
    }
}
