﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class FinancialFunctionsTests
    {
        [TestMethod]
        public void AccrIntMCorrectnessTest()
        {
            AccrIntM accrIntM = new AccrIntM();

            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.5555555555556, 39539, 39614, 0.1, 1000, 0);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.4918032786885, 39539, 39614, 0.1, 1000, 1);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.8333333333333, 39539, 39614, 0.1, 1000, 2);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.5479452054795, 39539, 39614, 0.1, 1000, 3);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.5555555555556, 39539, 39614, 0.1, 1000, 4);
        }

        [TestMethod]
        public void AccrIntMErrorTest()
        {
            AccrIntM accrIntM = new AccrIntM();

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, -1, 39614, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39614, -1, 0.1, 1000);

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, 0, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, -0.5, 1000);

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, 0.1, 0);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, 0.1, -1000);

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, 0.1, 1000, -1);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39539, 39614, 0.1, 1000, 5);

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39614, 39539, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39614, 39614, 0.1, 1000);
        }

        [TestMethod]
        public void AmorDegrcCorrectnessTest()
        {
            AmorDegrc amorDegrc = new AmorDegrc();

            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, 300, 1, 0.15, 1);
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, 300, 1, 0.15, 3);
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 777, 2400, 39679, 39813, 300, 1, 0.15, 4);
        }


        [TestMethod]
        public void AmorDegrcErrorTest()
        {
            AmorDegrc amorDegrc = new AmorDegrc();

            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, -1);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, 5);

            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, -1, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, -1, 39679, 39813, 0, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 299, 39679, 39813, 300, 1, 0.15, 0);

            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, -1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, -0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0, 0);
        }


        [TestMethod]
        public void AmorLincCorrectnessTest()
        {
            AmorLinc amorLinc = new AmorLinc();

            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, 300, 1, 0.15, 1);
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, 300, 1, 0.15, 3);
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, 300, 1, 0.15, 4);
        }


        [TestMethod]
        public void AmorLincErrorTest()
        {
            AmorLinc amorLinc = new AmorLinc();

            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, -1);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, 5);

            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, -1, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, -1, 39679, 39813, 0, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 299, 39679, 39813, 300, 1, 0.15, 0);

            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, -1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, -0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0, 0);
        }

        [TestMethod]
        public void CoupDayBsCorrectnessTest()
        {
            CoupDayBs coupDayBs = new CoupDayBs();

            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 71, 40568, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 71, 40568, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 71, 40568, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, 2, 4);
        }

        [TestMethod]
        public void CoupDayBsErrorTest()
        {
            CoupDayBs coupDayBs = new CoupDayBs();

            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CoupDaysCorrectnessTest()
        {
            CoupDays coupDays = new CoupDays();

            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, 40568, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 181, 40568, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, 40568, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 182.5, 40568, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, 40568, 40862, 2, 4);
        }


        [TestMethod]
        public void CoupDaysErrorTest()
        {
            CoupDays coupDays = new CoupDays();

            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CoupDaysNcCorrectnessTest()
        {
            CoupDaysNc coupDaysNc = new CoupDaysNc();

            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 290, 40568, 40862, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 294, 40568, 40862, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 294, 40568, 40862, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 294, 40568, 40862, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 290, 40568, 40862, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 20, 40568, 40862, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 21, 40568, 40862, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 21, 40568, 40862, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 21, 40568, 40862, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 20, 40568, 40862, 4, 4);
        }

        [TestMethod]
        public void CoupDaysNcErrorTest()
        {
            CoupDaysNc coupDaysNc = new CoupDaysNc();

            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CoupNcdCorrectnessTest()
        {
            CoupNcd coupNcd = new CoupNcd();

            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40862, 40568, 40862, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40862, 40568, 40862, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40862, 40568, 40862, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40862, 40568, 40862, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40862, 40568, 40862, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, 4, 4);
        }


        [TestMethod]
        public void CoupNcdErrorTest()
        {
            CoupNcd coupNcd = new CoupNcd();

            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CoupNumCorrectnessTest()
        {
            CoupNum coupNum = new CoupNum();

            FunctionsTestHelper.AssertFunctionResult(coupNum, 1, 40568, 40862, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 1, 40568, 40862, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 1, 40568, 40862, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 1, 40568, 40862, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 1, 40568, 40862, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, 4, 4);
        }


        [TestMethod]
        public void CoupNumErrorTest()
        {
            CoupNum coupNum = new CoupNum();

            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CoupPcdCorrectnessTest()
        {
            CoupPcd coupPcd = new CoupPcd();

            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40497, 40862, 4, 4);
        }


        [TestMethod]
        public void CoupPcdErrorTest()
        {
            CoupPcd coupPcd = new CoupPcd();

            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, -1, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, -1, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 0, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 3, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 5, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, -1, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 1, -1);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 1, 5);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40862, 40568, 1, 0);
        }

        [TestMethod]
        public void CumIPmtCorrectnessTest()
        {
            CumIPmt cumIPmt = new CumIPmt();

            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -11135.2321307508, 0.0075, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -11052.3395838718, 0.0075, 360, 125000, 13, 24, 1);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -937.5, 0.0075, 360, 125000, 1, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, 0, 0.0075, 360, 125000, 1, 1, 1);
        }

        [TestMethod]
        public void CumIPmtErrorTest()
        {
            CumIPmt cumIPmt = new CumIPmt();

            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, -1, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 0, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, -1, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 0, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, -1, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 0, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 0, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 5, 4, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, -1);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, 2);
        }

        [TestMethod]
        public void CumPrincCorrectnessTest()
        {
            CumPrinc cumPrinc = new CumPrinc();

            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -934.107123420877, 0.0075, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -927.153472378042, 0.0075, 360, 125000, 13, 24, 1);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -68.2782711809784, 0.0075, 360, 125000, 1, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -998.291088020822, 0.0075, 360, 125000, 1, 1, 1);
        }

        [TestMethod]
        public void CumPrincErrorTest()
        {
            CumPrinc cumPrinc = new CumPrinc();

            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, -1, 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 0, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, -1, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 0, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, -1, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 0, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 0, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 5, 4, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, -1);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, 2);
        }

        [TestMethod]
        public void DbCorrectnessTest()
        {
            Db db = new Db();

            FunctionsTestHelper.AssertFunctionResult(db, 319000, 1000000, 100000, 6, 1);
            FunctionsTestHelper.AssertFunctionResult(db, 217239, 1000000, 100000, 6, 2);
            FunctionsTestHelper.AssertFunctionResult(db, 147939.759, 1000000, 100000, 6, 3);
            FunctionsTestHelper.AssertFunctionResult(db, 100746.975879, 1000000, 100000, 6, 4);
            FunctionsTestHelper.AssertFunctionResult(db, 68608.690573599, 1000000, 100000, 6, 5);

            FunctionsTestHelper.AssertFunctionResult(db, 0, 0, 100000, 6, 1);
            FunctionsTestHelper.AssertFunctionResult(db, 1000000, 1000000, 0, 6, 1);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 2);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 3);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 4);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 5);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 6);

            FunctionsTestHelper.AssertFunctionResult(db, 186083.333333333, 1000000, 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 259639.416666667, 1000000, 100000, 6, 2, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 176814.44275, 1000000, 100000, 6, 3, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 120410.63551275, 1000000, 100000, 6, 4, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 81999.6427841827, 1000000, 100000, 6, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 55841.7567360285, 1000000, 100000, 6, 6, 7);
           
            FunctionsTestHelper.AssertFunctionResult(db, 0, 0, 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 583333.333333333, 1000000, 0, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 416666.666666667, 1000000, 0, 6, 2, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 3, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 4, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 6, 7);

            //FunctionsTestHelper.AssertFunctionResult(db, 0, 1000000, 0, 6, 7, 7);
            FunctionsTestHelper.AssertFunctionResult(db, 46722.5182806209, 1000000, 100000, 6, 6);
            //FunctionsTestHelper.AssertFunctionResult(db, 15845.0984738481, 1000000, 100000, 6, 7, 7);
        }

        [TestMethod]
        public void DbErrorTest()
        {
            Db db = new Db();

            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, -1, 100000, 6, 1);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, -1, 6, 1);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 0, 1);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, -1, 1);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 6, 7);

            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, -1, 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, -1, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 0, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, -1, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 6, 1, 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 6, 1, -1);
        }

        [TestMethod]
        public void DiscCorrectnessTest()
        {
            Disc disc = new Disc();

            FunctionsTestHelper.AssertFunctionResult(disc, 0.0520714285714288, 39107, 39248, 97.975, 100, 0);
            FunctionsTestHelper.AssertFunctionResult(disc, 0.0524202127659577, 39107, 39248, 97.975, 100, 1);
            FunctionsTestHelper.AssertFunctionResult(disc, 0.0517021276595747, 39107, 39248, 97.975, 100, 2);
            FunctionsTestHelper.AssertFunctionResult(disc, 0.0524202127659577, 39107, 39248, 97.975, 100, 3);
            FunctionsTestHelper.AssertFunctionResult(disc, 0.0520714285714288, 39107, 39248, 97.975, 100, 4);
        }

        [TestMethod]
        public void DiscErrorTest()
        {
            Disc disc = new Disc();

            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, -1, 39248, 97.975, 100, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, -1, 97.975, 100, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39248, 39107, 97.975, 100, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, 0, 100, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, -1, 100, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, 97.975, 0, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, 97.975, -1, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, 97.975, 100, -1);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.NumberError, 39107, 39248, 97.975, 100, 5);
        }

        [TestMethod]
        public void DollarDeCorrectnessTest()
        {
            DollarDe dollarDe = new DollarDe();

            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.125, 1.02, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.0625, 1.02, 32);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.625, 1.1, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.3125, 1.1, 32);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 0, 0, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, -1, -1, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1, 1, 1);
        }

        [TestMethod]
        public void DollarDeErrorTest()
        {
            DollarDe dollarDe = new DollarDe();

            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.DivisionByZero, 1.02, 0);
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.NumberError, 1.02, -0.1);
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.NumberError, 1.02, -1);
        }

        [TestMethod]
        public void DollarFrCorrectnessTest()
        {
            DollarFr dollarFr = new DollarFr();

            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.0032, 1.02, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.0064, 1.02, 32);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.016, 1.1, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.032, 1.1, 32);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 0, 0, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, -1, -1, 16);
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1, 1, 1);
        }

        [TestMethod]
        public void DollarFrErrorTest()
        {
            DollarFr dollarFr = new DollarFr();

            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.DivisionByZero, 1.02, 0);
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.NumberError, 1.02, -0.1);
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.NumberError, 1.02, -1);
        }

        [TestMethod]
        public void DurationCorrectnessTest()
        {
            Duration duration = new Duration();

            FunctionsTestHelper.AssertFunctionResult(duration, 6.14906788677325, 39448, 42370, 0.08, 0.09, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(duration, 6.14906788677324, 39448, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 6.14906788677324, 39448, 42370, 0.08, 0.09, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(duration, 6.14906788677324, 39448, 42370, 0.08, 0.09, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(duration, 6.14906788677324, 39448, 42370, 0.08, 0.09, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(duration, 5.99377495554519, 39448, 42370, 0.08, 0.09, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.99377495554519, 39448, 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.99377495554519, 39448, 42370, 0.08, 0.09, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.99377495554519, 39448, 42370, 0.08, 0.09, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.99377495554519, 39448, 42370, 0.08, 0.09, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(duration, 5.91465300919712, 39448, 42370, 0.08, 0.09, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.91465300919712, 39448, 42370, 0.08, 0.09, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.91465300919712, 39448, 42370, 0.08, 0.09, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.91465300919712, 39448, 42370, 0.08, 0.09, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(duration, 5.91465300919712, 39448, 42370, 0.08, 0.09, 4, 4);
        }

        [TestMethod]
        public void DurationErrorTest()
        {
            Duration duration = new Duration();
            
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, -1, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, -1, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 42370, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 42370, 39448, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, -1, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, -1, 1, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 0, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 3, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 5, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 1, -1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 1, 5);
        }

        [TestMethod]
        public void EffectCorrectnessTest()
        {
            Effect effect = new Effect();

            FunctionsTestHelper.AssertFunctionResult(effect, 0.0535426673707582, 0.0525, 4);
            FunctionsTestHelper.AssertFunctionResult(effect, 0.0534241093750003, 0.0525, 3);
            FunctionsTestHelper.AssertFunctionResult(effect, 0.0531890625000002, 0.0525, 2);
            FunctionsTestHelper.AssertFunctionResult(effect, 0.0525, 0.0525, 1);

            FunctionsTestHelper.AssertFunctionResult(effect, 1.44140625, 1, 4);
            FunctionsTestHelper.AssertFunctionResult(effect, 1.37037037037037, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(effect, 1.25, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(effect, 1, 1, 1);
        }


        [TestMethod]
        public void EffectErrorTest()
        {
            Effect effect = new Effect();

            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.NumberError, 0, 4);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.NumberError, -1, 4);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.NumberError, 0.0525, 0.99);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.NumberError, 0.0525, 0);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.NumberError, 0.0525, -1);
        }

        [TestMethod]
        public void FvCorrectnessTest()
        {
            Fv fv = new Fv();

            FunctionsTestHelper.AssertFunctionResult(fv, 2571.17534765198, 0.005, 10, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 2581.40337406013, 0.005, 10, -200, -500, 1);
            FunctionsTestHelper.AssertFunctionResult(fv, 12682.503013197, 0.01, 12, -1000);
            FunctionsTestHelper.AssertFunctionResult(fv, 2295.23404915446, 0.005, 12, -100, -1000, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 2301.40183034091, 0.005, 12, -100, -1000, 1);

            FunctionsTestHelper.AssertFunctionResult(fv, 2500, 0, 10, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 500, 0.005, 0, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 525.570066020395, 0.005, 10, 0, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 2045.60528163157, 0.005, 10, -200, 0, 0);

            FunctionsTestHelper.AssertFunctionResult(fv, 0, 0.005, 10, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 500, 0, 0, -200, -500, 0);

            FunctionsTestHelper.AssertFunctionResult(fv, 2431.14984660201, -0.005, 10, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, -1470.40840180916, 0.005, -10, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, -1520.03521561117, 0.005, 10, 200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 1520.03521561117, 0.005, 10, -200, 500, 0);

            FunctionsTestHelper.AssertFunctionResult(fv, -2571.17534765198, 0.005, 10, 200, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 2571.17534765198, 0.005, 10, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 1530.41665180908, -0.005, -10, 200, 500, 0);

            //FunctionsTestHelper.AssertFunctionResult(fv, 2581.40337406012, 0.11, 35, -2000, null, 1);
        }

        [TestMethod]
        public void FvErrorTest()
        {
            Fv fv = new Fv();

            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.NumberError, 0.005, 10, -200, -500, -1);
            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.NumberError, 0.005, 10, -200, -500, 2);
        }

        [TestMethod]
        public void IntRateCorrectnessTest()
        {
            IntRate intRate = new IntRate();

            FunctionsTestHelper.AssertFunctionResult(intRate, 0.05768, 39493, 39583, 1000000, 1014420, 0);
            FunctionsTestHelper.AssertFunctionResult(intRate, 0.0586413333333333, 39493, 39583, 1000000, 1014420, 1);
            FunctionsTestHelper.AssertFunctionResult(intRate, 0.05768, 39493, 39583, 1000000, 1014420, 2);
            FunctionsTestHelper.AssertFunctionResult(intRate, 0.0584811111111111, 39493, 39583, 1000000, 1014420, 3);
            FunctionsTestHelper.AssertFunctionResult(intRate, 0.05768, 39493, 39583, 1000000, 1014420, 4);
        }


        [TestMethod]
        public void IntRateErrorTest()
        {
            IntRate intRate = new IntRate();

            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, -1, 39583, 1000000, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, -1, 1000000, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39583, 39493, 1000000, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39493, 1000000, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, 0, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, -1, 1014420, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, 1000000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, 1000000, -1, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, 1000000, 1014420, -1);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.NumberError, 39493, 39583, 1000000, 1014420, 5);
        }

        [TestMethod]
        public void IpmtCorrectnessTest()
        {
            Ipmt ipmt = new Ipmt();

            FunctionsTestHelper.AssertFunctionResult(ipmt, -66.6666666666667, 0.00833333333333333, 1, 36, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(ipmt, 0, 0.00833333333333333, 1, 36, 8000, 500, 1);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -260.725075528701, 0.1, 3, 3, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -237.022795935183, 0.1, 3, 3, 8000, 500, 1);
        }


        [TestMethod]
        public void IpmtErrorTest()
        {
            Ipmt ipmt = new Ipmt();

            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, -1, 1, 36, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 0, 36, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, -1, 36, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 37, 36, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 1, 0, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 1, -1, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 1, 36, 8000, 500, -1);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.NumberError, 0.00833333333333333, 1, 36, 8000, 500, 2);
        }

        [TestMethod]
        public void IspmtCorrectnessTest()
        {
            Ispmt ispmt = new Ispmt();

            FunctionsTestHelper.AssertFunctionResult(ispmt, -64814.8148148148, 0.00833333333333333, 1, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -62962.962962963, 0.00833333333333333, 2, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, 64814.8148148148, -0.00833333333333333, 1, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, 0, 0, 1, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -66666.6666666667, 0.00833333333333333, 0, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -68518.5185185185, 0.00833333333333333, -1, 36, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -133333.333333333, 0.00833333333333333, 1, -1, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, 0, 0.00833333333333333, 1, 36, 0);
            FunctionsTestHelper.AssertFunctionResult(ispmt, 0.00810185185185185, 0.00833333333333333, 1, 36, -1);
            FunctionsTestHelper.AssertFunctionResult(ispmt, 3703.7037037037, 0.00833333333333333, 38, 36, 8000000);
        }

        [TestMethod]
        public void IspmtErrorTest()
        {
            Ispmt ispmt = new Ispmt();

            FunctionsTestHelper.AssertFunctionError(ispmt, ErrorExpressions.DivisionByZero, 0.00833333333333333, 1, 0, 8000000);
        }

        [TestMethod]
        public void MDurationCorrectnessTest()
        {
            MDuration mDuration = new MDuration();

            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.64134668511307, 39448, 42370, 0.08, 0.09, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.64134668511307, 39448, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.64134668511307, 39448, 42370, 0.08, 0.09, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.64134668511307, 39448, 42370, 0.08, 0.09, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.64134668511307, 39448, 42370, 0.08, 0.09, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.73566981391884, 39448, 42370, 0.08, 0.09, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.73566981391884, 39448, 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.73566981391884, 39448, 42370, 0.08, 0.09, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.73566981391884, 39448, 42370, 0.08, 0.09, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.73566981391884, 39448, 42370, 0.08, 0.09, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.78450172048619, 39448, 42370, 0.08, 0.09, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.78450172048619, 39448, 42370, 0.08, 0.09, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.78450172048619, 39448, 42370, 0.08, 0.09, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.78450172048619, 39448, 42370, 0.08, 0.09, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 5.78450172048619, 39448, 42370, 0.08, 0.09, 4, 4);
        }

        [TestMethod]
        public void MDurationErrorTest()
        {
            MDuration mDuration = new MDuration();

            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, -1, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, -1, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 42370, 42370, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 42370, 39448, 0.08, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, -1, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, -1, 1, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 0, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 3, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 5, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 1, -1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.NumberError, 39448, 42370, 0.08, 0.09, 1, 5);
        }

        [TestMethod]
        public void NominalCorrectnessTest()
        {
            Nominal nominal = new Nominal();

            FunctionsTestHelper.AssertFunctionResult(nominal, 0.052500319868356, 0.053543, 4);
            FunctionsTestHelper.AssertFunctionResult(nominal, 0.0526148318698652, 0.053543, 3);
            FunctionsTestHelper.AssertFunctionResult(nominal, 0.0528448553166405, 0.053543, 2);
            FunctionsTestHelper.AssertFunctionResult(nominal, 0.0535429999999999, 0.053543, 1);

            FunctionsTestHelper.AssertFunctionResult(nominal, 0.756828460010884, 1, 4);
            FunctionsTestHelper.AssertFunctionResult(nominal, 0.77976314968462, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(nominal, 0.82842712474619, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(nominal, 1, 1, 1);
        }

        [TestMethod]
        public void NominalErrorTest()
        {
            Nominal nominal = new Nominal();

            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.NumberError, 0, 4);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.NumberError, -1, 4);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.NumberError, 0.0525, 0.99);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.NumberError, 0.0525, 0);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.NumberError, 0.0525, -1);
        }

        [TestMethod]
        public void NPerCorrectnessTest()
        {
            NPer nPer = new NPer();

            FunctionsTestHelper.AssertFunctionResult(nPer, -9.57859403981316, 0.01, -100, -1000);
            FunctionsTestHelper.AssertFunctionResult(nPer, -10, 0, -100, -1000);
            FunctionsTestHelper.AssertFunctionResult(nPer, -240.986486598574, 0.01, -1, -1000);
            FunctionsTestHelper.AssertFunctionResult(nPer, 240.986486598574, 0.01, 11, -1000);
            FunctionsTestHelper.AssertFunctionResult(nPer, 0, 0.01, -100, 0);
            FunctionsTestHelper.AssertFunctionResult(nPer, 0.100549453917389, 0.01, -100, 10);
            FunctionsTestHelper.AssertFunctionResult(nPer, 10.5886444594232, 0.01, -100, 1000);

            FunctionsTestHelper.AssertFunctionResult(nPer, 60.0821228537617, 0.01, -100, -1000, 10000, 0);
            FunctionsTestHelper.AssertFunctionResult(nPer, 59.6738656742946, 0.01, -100, -1000, 10000, 1);
            FunctionsTestHelper.AssertFunctionResult(nPer, -473.303880123028, 0.01, -100, -1000, -10000, 1);

            //FunctionsTestHelper.AssertFunctionResult(nPer, 2581.40337406012, 0.11, 35, -2000, null, 1);
        }


        [TestMethod]
        public void NpvCorrectnessTest()
        {
            Npv npv = new Npv();

            FunctionsTestHelper.AssertFunctionResult(npv, 0, 0.1, 0);
            FunctionsTestHelper.AssertFunctionResult(npv, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(npv, 1111.11111111111, -10, -10000);
            FunctionsTestHelper.AssertFunctionResult(npv, -1111.11111111111, -10, 10000);
            FunctionsTestHelper.AssertFunctionResult(npv, 1188.44341233522, 0.1, -10000, 3000, 4200, 6800);
            FunctionsTestHelper.AssertFunctionResult(npv, 4000, 0, -10000, 3000, 4200, 6800);
            FunctionsTestHelper.AssertFunctionResult(npv, 8718.18320377991, -0.1, -10000, 3000, 4200, 6800);
            FunctionsTestHelper.AssertFunctionResult(npv, -880.677549347722, 10, -10000, 3000, 4200, 6800);
            FunctionsTestHelper.AssertFunctionResult(npv, 1188.44341233522, 0.1, -10000, 3000, 4200, 6800);
        }

        [TestMethod]
        public void NPerErrorTest()
        {
            NPer nPer = new NPer();

            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, -1, -100, -1000);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, 0, -1000);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, 1, -1000);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, 7, -1000);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, 10, -1000);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, -100, 10000);

            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, -100, -1000, -10000, 0);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, -100, -1000, 10000, -1);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.NumberError, 0.01, -100, -1000, 10000, 2);
        }

        [TestMethod]
        public void OddFPriceCorrectnessTest()
        {
            OddFPrice oddFPrice = new OddFPrice();

            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.498148746163, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.494585545507, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.497748534154, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.494585545507, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.498148746163, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.599205828238, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.597717474079, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.598799608325, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.596112595205, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.599205828238, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.650687054409, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.650021611091, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.650277739118, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.649958352793, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 113.650687054409, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 4, 4);

            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 47.4250482962375, 39763, 44256, 39736, 39873, 0, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 196.598611111111, 39763, 44256, 39736, 39873, 0.0785, 0, 100, 1, 0);
        }

        [TestMethod]
        public void OddFPriceErrorTest()
        {
            OddFPrice oddFPrice = new OddFPrice();
            
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, -1, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, -1, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, -1, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, -1, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 39763, 39763, 39763, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39763, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39873, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 44256, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39764, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39874, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 44257, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, -0.1, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, -0.1, 100, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, -1, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 5);
        }

        [TestMethod]
        public void OddFYieldCorrectnessTest()
        {
            OddFYield oddFYield = new OddFYield();

            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0775403902082764, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.07754442457188, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0775398696000414, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.07754442457188, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0775403902082764, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0772455415972989, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0772470625974191, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0772450219206795, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0772500780535923, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0772455415972989, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.077097558315507, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0770979399217127, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.0770970390819706, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.07709816193298, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.077097558315507, 39763, 44256, 39736, 39873, 0.0575, 84.5, 100, 4, 4);
        }

        [TestMethod]
        public void OddFYieldErrorTest()
        {
            OddFYield oddFYield = new OddFYield();

            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, -1, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, -1, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, -1, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, -1, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 39763, 39763, 39763, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39763, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39873, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 44256, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39764, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39874, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 44257, 0.0785, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, -0.1, 0.0625, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, -0.1, 100, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, -1, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.NumberError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 100, 1, 5);
        }

        [TestMethod]
        public void OddLPriceCorrectnessTest()
        {
            OddLPrice oddLPrice = new OddLPrice();

            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8791676815291, 39485, 39614, 39370, 0.0375, 0.0405, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8785673299155, 39485, 39614, 39370, 0.0375, 0.0405, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8785673299155, 39485, 39614, 39370, 0.0375, 0.0405, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8785673299155, 39485, 39614, 39370, 0.0375, 0.0405, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 99.8782860147213, 39485, 39614, 39370, 0.0375, 0.0405, 100, 4, 4);
        }

        [TestMethod]
        public void OddLPriceErrorTest()
        {
            OddLPrice oddLPrice = new OddLPrice();

            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, -1, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, -1, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, -1, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39485, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39486, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39614, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39615, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39370, 39370, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, -0.1, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, -0.1, 100, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, -1, 1, 0);

            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 3, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 5, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 5);
        }

        [TestMethod]
        public void OddLYieldCorrectnessTest()
        {
            OddLYield oddLYield = new OddLYield();

            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0451922356291692, 39558, 39614, 39440, 0.0375, 99.875, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0451922356291692, 39558, 39614, 39440, 0.0375, 99.875, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0451922356291692, 39558, 39614, 39440, 0.0375, 99.875, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.045179885491872, 39558, 39614, 39440, 0.0375, 99.875, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0451922356291692, 39558, 39614, 39440, 0.0375, 99.875, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0408836050254386, 39485, 39614, 39440, 0.0375, 99.875, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0409134403872861, 39485, 39614, 39440, 0.0375, 99.875, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0409134403872861, 39485, 39614, 39440, 0.0375, 99.875, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0409134403872861, 39485, 39614, 39440, 0.0375, 99.875, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, 0.0408836050254386, 39485, 39614, 39440, 0.0375, 99.875, 100, 4, 4);
        }

        [TestMethod]
        public void OddLYieldErrorTest()
        {
            OddLYield oddLYield = new OddLYield();
          
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, -1, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, -1, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, -1, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39485, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39486, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39614, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39615, 39614, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39370, 39370, 39370, 0.0375, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, -0.1, 0.0405, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, -0.1, 100, 1, 0);
                                   
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, -1, 1, 0);
                                        
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 3, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 5, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.NumberError, 39485, 39614, 39370, 0.0375, 0.0405, 100, 1, 5);
        }

        [TestMethod]
        public void PDurationCorrectnessTest()
        {
            PDuration pDuration = new PDuration();

            FunctionsTestHelper.AssertFunctionResult(pDuration, 3.85986616262266, 0.025, 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 87.6054764193714, 0.00208333333333333, 1000, 1200);
        }

        [TestMethod]
        public void PDurationErrorTest()
        {
            PDuration pDuration = new PDuration();

            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, 0, 2000, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, -1, 2000, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, 0.025, 0, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, 0.025, -1, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, 0.025, 2000, 0);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.NumberError, 0.025, 2000, -1);
        }

        [TestMethod]
        public void PpmtCorrectnessTest()
        {
            Ppmt ppmt = new Ppmt();

            FunctionsTestHelper.AssertFunctionResult(ppmt, -75.6231860083663, 0.00833333333333333, 1, 24, 2000);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -27598.0534624214, 0.08, 10, 10, 200000);
            
            FunctionsTestHelper.AssertFunctionResult(ppmt, -75.6609976013705, 0.00833333333333333, 1, 24, 2000, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -91.5646257203675, 0.00833333333333333, 1, 24, 2000, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -91.5646257203675, 0.00833333333333333, 1, 24, 2000, 1, -1);

            FunctionsTestHelper.AssertFunctionResult(ppmt, -75.6988091943747, 0.00833333333333333, 1, 24, 2000, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -91.6021248208675, 0.00833333333333333, 1, 24, 2000, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -91.6021248208675, 0.00833333333333333, 1, 24, 2000, 2, -1);

            FunctionsTestHelper.AssertFunctionResult(ppmt, -27598.1914526887, 0.08, 10, 10, 200000, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -25553.8809747117, 0.08, 10, 10, 200000, 1, 1);
        }

        [TestMethod]
        public void PpmtErrorTest()
        {
            Ppmt ppmt = new Ppmt();

            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, -1, 1, 24, 2000);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, 0.08, 0, 10, 200000);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, 0.08, -1, 10, 200000);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, 0.08, 11, 10, 200000);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, 0.08, 11, 0, 200000);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.NumberError, 0.08, 11, -1, 200000);
        }

        [TestMethod]
        public void PriceCorrectnessTest()
        {
            Price price = new Price();

            FunctionsTestHelper.AssertFunctionResult(price, 94.6721500072852, 39493, 43054, 0.0575, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6725635950793, 39493, 43054, 0.0575, 0.065, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(price, 94.673836367238,  39493, 43054, 0.0575, 0.065, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6727723583542, 39493, 43054, 0.0575, 0.065, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6721500072852, 39493, 43054, 0.0575, 0.065, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(price, 94.6343616213221, 39493, 43054, 0.0575, 0.065, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6354492078772, 39493, 43054, 0.0575, 0.065, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6365640300251, 39493, 43054, 0.0575, 0.065, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6351747967845, 39493, 43054, 0.0575, 0.065, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6343616213221, 39493, 43054, 0.0575, 0.065, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(price, 94.6150939521377, 39493, 43054, 0.0575, 0.065, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6150939521377, 39493, 43054, 0.0575, 0.065, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6150939521377,  39493, 43054, 0.0575, 0.065, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6150939521377, 39493, 43054, 0.0575, 0.065, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(price, 94.6150939521377, 39493, 43054, 0.0575, 0.065, 100, 4, 4);
        }

        [TestMethod]
        public void PriceErrorTest()
        {
            Price price = new Price();

            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, -1, 43054, 0.0575, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, -1, 0.0575, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 39493, 0.0575, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39494, 39493, 0.0575, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, -1, 0.065, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, -1, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, -1, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 100, 3, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 100, 5, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.NumberError, 39493, 43054, 0.0575, 0.065, 100, 1, 5);
        }

        [TestMethod]
        public void PriceDiscCorrectnessTest()
        {
            PriceDisc priceDisc = new PriceDisc();

            FunctionsTestHelper.AssertFunctionResult(priceDisc, 99.78125, 39494, 39508, 0.0525, 100, 0);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 99.7991803278689, 39494, 39508, 0.0525, 100, 1);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 99.7958333333333, 39494, 39508, 0.0525, 100, 2);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 99.7986301369863, 39494, 39508, 0.0525, 100, 3);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 99.78125, 39494, 39508, 0.0525, 100, 4);
        }

        [TestMethod]
        public void PriceDiscErrorTest()
        {
            PriceDisc priceDisc = new PriceDisc();
            
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, -1, 39508, 0.0525, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, -1, 0.0525, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39494, 0.0525, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39508, 39494, 0.0525, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, 0, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, -1, 100, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, 0.0525, 0, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, 0.0525, -1, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, 0.0525, 100, -1);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.NumberError, 39494, 39508, 0.0525, 100, 5);
        }

        [TestMethod]
        public void PvCorrectnessTest()
        {
            Pv pv = new Pv();

            FunctionsTestHelper.AssertFunctionResult(pv, -59777145.8511878, 0.00666666666666667, 240, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -60175660.1568624, 0.00666666666666667, 240, 500000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, -120000000, 0, 240, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -120000000, 0, 240, 500000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, -2, 240, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, -2, 240, 500000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, 0.00666666666666667, 0, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, 0.00666666666666667, 0, 500000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 499999.999999994, 0.00666666666666667, -1, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 503333.333333327, 0.00666666666666667, -1, 500000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, 0.00666666666666667, 240, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 0, 0.00666666666666667, 240, 0, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 119.554291702376, 0.00666666666666667, 240, -1, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 120.351320313725, 0.00666666666666667, 240, -1, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, -80, 0.00666666666666667, 0, 500000, 80, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -80, 0.00666666666666667, 0, 500000, 80, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, 80, 0.00666666666666667, 0, 500000, -80, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, 80, 0.00666666666666667, 0, 500000, -80, 1);
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, 10, 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0033752808, -0.00000001, 10, 5000, 9000, 1);
        }

        [TestMethod]
        public void PvErrorTest()
        {
            Pv pv = new Pv();

            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.NumberError, -500000, 240, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.NumberError, 100, 240, 500000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.NumberError, 20, 240, 500000, 0, 0);
        }

        [TestMethod]
        public void RateCorrectnessTest()
        {
            Rate rate = new Rate();

            FunctionsTestHelper.AssertFunctionResult(rate, 0.00770147248820137, 48, -200, 8000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(rate, 0.00805298192390656, 48, -200, 8000, 0, 1);
        }

        [TestMethod]
        public void RateErrorTest()
        {
            Rate rate = new Rate();

            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.NumberError, 0, -200, 8000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.NumberError, -1, -200, 8000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.NumberError, 48, 0, 8000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.NumberError, 48, 100, 8000, 0, 0);
        }

        [TestMethod]
        public void ReceivedCorrectnessTest()
        {
            Received received = new Received();

            FunctionsTestHelper.AssertFunctionResult(received, 1014584.6544071, 39493, 39583, 1000000, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionResult(received, 1014342.13261276, 39493, 39583, 1000000, 0.0575, 1);
            FunctionsTestHelper.AssertFunctionResult(received, 1014584.6544071, 39493, 39583, 1000000, 0.0575, 2);
            FunctionsTestHelper.AssertFunctionResult(received, 1014381.99124574, 39493, 39583, 1000000, 0.0575, 3);
            FunctionsTestHelper.AssertFunctionResult(received, 1014584.6544071, 39493, 39583, 1000000, 0.0575, 4);
        }

        [TestMethod]
        public void ReceivedErrorTest()
        {
            Received received = new Received();

            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, -1, 39583, 1000000, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, -1, 1000000, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39493, 1000000, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39494, 0, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39494, -1, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39583, 1000000, 0, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39583, 1000000, -1, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39583, 1000000, 0.0575, -1);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.NumberError, 39493, 39583, 1000000, 0.0575, 5);
        }

        [TestMethod]
        public void RriCorrectnessTest()
        {
            Rri rri = new Rri();

            FunctionsTestHelper.AssertFunctionResult(rri, 0, 8, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.0119850241404, 8, 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.0119850241404, 8, -10000, -11000);
            FunctionsTestHelper.AssertFunctionResult(rri, -1, 8, 10000, 0);
            FunctionsTestHelper.AssertFunctionResult(rri, -1, 8, -10000, 0);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.1, 1, 10000, 11000);
        }

        [TestMethod]
        public void RriErrorTest()
        {
            Rri rri = new Rri();

            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, 0, 10000, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, -1, 10000, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, 8, -10000, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, 8, 10000, -11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, 8, 0, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.NumberError, 8, 0, -11000);
        }

        [TestMethod]
        public void SlnCorrectnessTest()
        {
            Sln sln = new Sln();

            FunctionsTestHelper.AssertFunctionResult(sln, 2250, 30000, 7500, 10);
            FunctionsTestHelper.AssertFunctionResult(sln, -750, 0, 7500, 10);
            FunctionsTestHelper.AssertFunctionResult(sln, -770, -200, 7500, 10);
            FunctionsTestHelper.AssertFunctionResult(sln, 3000, 30000, 0, 10);
            FunctionsTestHelper.AssertFunctionResult(sln, 3020, 30000, -200, 10);
            FunctionsTestHelper.AssertFunctionResult(sln, -22500, 30000, 7500, -1);
            FunctionsTestHelper.AssertFunctionResult(sln, -225, 30000, 7500, -100);
            FunctionsTestHelper.AssertFunctionResult(sln, 225, -30000, -7500, -100);
        }

        [TestMethod]
        public void SlnErrorTest()
        {
            Sln sln = new Sln();

            FunctionsTestHelper.AssertFunctionError(sln, ErrorExpressions.DivisionByZero, 30000, 7500, 0);
        }

        [TestMethod]
        public void SydCorrectnessTest()
        {
            Syd syd = new Syd();

            FunctionsTestHelper.AssertFunctionResult(syd, 4090.90909090909, 30000, 7500, 10, 1);
            FunctionsTestHelper.AssertFunctionResult(syd, 409.090909090909, 30000, 7500, 10, 10);
            FunctionsTestHelper.AssertFunctionResult(syd, 187.5, 30000, 7500, 15, 15);
            FunctionsTestHelper.AssertFunctionResult(syd, 250, 30000, 0, 15, 15);
            FunctionsTestHelper.AssertFunctionResult(syd, 187.5, 30000, 7500, 15, 15);
            FunctionsTestHelper.AssertFunctionResult(syd, -62.5, 0, 7500, 15, 15);
            FunctionsTestHelper.AssertFunctionResult(syd, -65, -300, 7500, 15, 15);
        }

        [TestMethod]
        public void SydErrorTest()
        {
            Syd syd = new Syd();

            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.NumberError, 30000, 7500, 10, 0);
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.NumberError, 30000, 7500, 10, -1);
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.NumberError, 30000, 7500, 10, 11);
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.NumberError, 30000, -1, 10, 11);
        }

        [TestMethod]
        public void TBillEqCorrectnessTest()
        {
            TBillEq tBillEq = new TBillEq();

            FunctionsTestHelper.AssertFunctionResult(tBillEq, 0.094151493565943, 39538, 39600, 0.0914000);
            FunctionsTestHelper.AssertFunctionResult(tBillEq, 36.5, 39538, 39600, 5);
            FunctionsTestHelper.AssertFunctionResult(tBillEq, 0.554711246200608, 39538, 39600, 0.5);
        }


        [TestMethod]
        public void TBillEqErrorTest()
        {
            TBillEq tBillEq = new TBillEq();

            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, -1, 39600, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, 39538, -1, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, 39538, 39538, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, 39600, 39538, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, 39538, 39600, 0);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.NumberError, 39538, 39600, -1);
        }

        [TestMethod]
        public void TBillPriceCorrectnessTest()
        {
            TBillPrice tBillPrice = new TBillPrice();

            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 98.45, 39538, 39600, 0.09);
            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 82.95, 39538, 39600, 0.99);
            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 82.7777777777778, 39538, 39600, 1);
        }


        [TestMethod]
        public void TBillPriceErrorTest()
        {
            TBillPrice tBillPrice = new TBillPrice();

            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, -1, 39600, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, 39538, -1, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, 39538, 39538, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, 39600, 39538, 0.0914000);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, 39538, 39600, 0);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.NumberError, 39538, 39600, -1);
        }

        [TestMethod]
        public void TBillYieldCorrectnessTest()
        {
            TBillYield tBillYield = new TBillYield();

            FunctionsTestHelper.AssertFunctionResult(tBillYield, 0.0914169629253426, 39538, 39600, 98.45);
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 52.258064516129, 39538, 39600, 10);
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 23.2258064516129, 39538, 39600, 20);
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 574.838709677419, 39538, 39600, 1);
        }


        [TestMethod]
        public void TBillYieldErrorTest()
        {
            TBillYield tBillYield = new TBillYield();

            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, -1, 39600, 98.45);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, 39538, -1, 98.45);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, 39538, 39538, 98.45);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, 39538, 39537, 98.45);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, 39538, 39600, 0);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.NumberError, 39538, 39600, -1);
        }

        [TestMethod]
        public void VdbCorrectnessTest()
        {
            Vdb vdb = new Vdb();

            FunctionsTestHelper.AssertFunctionResult(vdb, 1.31506849315068, 2400, 300, 3650, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, 10, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(vdb, 396.306053264751, 2400, 300, 120, 6, 18);
            FunctionsTestHelper.AssertFunctionResult(vdb, 311.808936658234, 2400, 300, 120, 6, 18, 1.5);
            FunctionsTestHelper.AssertFunctionResult(vdb, 315, 2400, 300, 10, 0, 0.875, 1.5);
            FunctionsTestHelper.AssertFunctionResult(vdb, 0, 2400, 300, 3650, 0, 0);
        }

        [TestMethod]
        public void VdbErrorTest()
        {
            Vdb vdb = new Vdb();

            //FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 0, 300, 10, 6, 18, 1.5);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, 300, 3650, -1, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, 300, 3650, 0, -1);
            
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, -1, 300, 3650, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, -1, 3650, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, 300, 0, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, 300, -1, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.NumberError, 2400, 300, 3650, 0, 1, -1);
        }

        [TestMethod]
        public void YieldCorrectnessTest()
        {
            Yield yield = new Yield();

            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650556801479499, 39493, 42689, 0.0575, 95.04287, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650563966931057, 39493, 42689, 0.0575, 95.04287, 100, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650586021766687, 39493, 42689, 0.0575, 95.04287, 100, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650567584013553, 39493, 42689, 0.0575, 95.04287, 100, 1, 3);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650556801479499, 39493, 42689, 0.0575, 95.04287, 100, 1, 4);

            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650000068807314, 39493, 42689, 0.0575, 95.04287, 100, 2, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650018206055232, 39493, 42689, 0.0575, 95.04287, 100, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650036803304293, 39493, 42689, 0.0575, 95.04287, 100, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650013629283167, 39493, 42689, 0.0575, 95.04287, 100, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0650000068807314, 39493, 42689, 0.0575, 95.04287, 100, 2, 4);

            FunctionsTestHelper.AssertFunctionResult(yield, 0.0649721173340935, 39493, 42689, 0.0575, 95.04287, 100, 4, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0649721173340936, 39493, 42689, 0.0575, 95.04287, 100, 4, 1);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0649721173340906, 39493, 42689, 0.0575, 95.04287, 100, 4, 2);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.064972117334093,  39493, 42689, 0.0575, 95.04287, 100, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(yield, 0.0649721173340935, 39493, 42689, 0.0575, 95.04287, 100, 4, 4);
        }

        [TestMethod]
        public void YieldErrorTest()
        {
            Yield yield = new Yield();

            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, -1, 42689, 0.0575, 95.04287, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, -1, 0.0575, 95.04287, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 39493, 0.0575, 95.04287, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 39492, 0.0575, 95.04287, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 100, 0, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, -1, 1, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 100, 3, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 100, 5, 0);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 100, 1, -1);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.NumberError, 39493, 42689, 0.0575, 95.04287, 100, 1, 5);
        }

        [TestMethod]
        public void YieldDiscCorrectnessTest()
        {
            YieldDisc yieldDisc = new YieldDisc();

            FunctionsTestHelper.AssertFunctionResult(yieldDisc, 0.0493010671877344, 39494, 39508, 99.7950, 100, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, 0.0537029481866393, 39494, 39508, 99.7950, 100, 1);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, 0.0528225719868583, 39494, 39508, 99.7950, 100, 2);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, 0.0535562188200091, 39494, 39508, 99.7950, 100, 3);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, 0.0493010671877344, 39494, 39508, 99.7950, 100, 4);
        }


        [TestMethod]
        public void YieldDiscErrorTest()
        {
            YieldDisc yieldDisc = new YieldDisc();

            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, -1, 39508, 99.7950, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, -1, 99.7950, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39494, 99.7950, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39493, 99.7950, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, 0, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, -1, 100, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, 99.7950, 0, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, 99.7950, -1, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, 99.7950, 100, -1);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.NumberError, 39494, 39508, 99.7950, 100, 5);
        }

        [TestMethod]
        public void YieldMatCorrectnessTest()
        {
            YieldMat yieldMat = new YieldMat();

            FunctionsTestHelper.AssertFunctionResult(yieldMat, 0.0609543336915387, 39522, 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 0.0609666856464936, 39522, 39755, 39394, 0.0625, 100.0123, 1);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 0.060948059155698, 39522, 39755, 39394, 0.0625, 100.0123, 2);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 0.0609636299211303, 39522, 39755, 39394, 0.0625, 100.0123, 3);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 0.0609543336915387, 39522, 39755, 39394, 0.0625, 100.0123, 4);
        }

        [TestMethod]
        public void YieldMatErrorTest()
        {
            YieldMat yieldMat = new YieldMat();

            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, -1, 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, -1, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, -1, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39522, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39523, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39755, 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39756, 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, 0, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, -1, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, 0.0625, 0, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, 0.0625, -1, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, 0.0625, 100.0123, -1);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.NumberError, 39522, 39755, 39394, 0.0625, 100.0123, 5);
        }
    }
}


