﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class FunctionCellReferenceRangesTests
    {
        [TestMethod]
        public void FunctionCellReferenceUnionInvalidationTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[3, 3].SetValue("=sum(A1:A2{ls}B1:B2)".ReplaceCultureDependentSeparators());
            worksheet.Cells[1, 1].SetValue(1);

            FormulaCellValue formulaCellValue = (FormulaCellValue)worksheet.Cells[3, 3].GetValue().Value;
            Assert.AreEqual(1, formulaCellValue.Value.GetValueAsConstantExpression().NumberValue());

            worksheet.Cells[0, 0].SetValue("=B2");
            worksheet.Cells[1, 1].SetValue(2);

            formulaCellValue = (FormulaCellValue)worksheet.Cells[3, 3].GetValue().Value;
            Assert.AreEqual(4, formulaCellValue.Value.GetValueAsConstantExpression().NumberValue());
        }

        [TestMethod]
        public void FunctionCellReferenceUnionInvalidationInFirstRangeTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cell = worksheet.Cells[3, 3];

            cell.SetValue("=sum(A1:A3{ls}B1:D2)".ReplaceCultureDependentSeparators());

            string value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("0", value);

            worksheet.Cells[0, 0].SetValue(1);

            value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("1", value);

            worksheet.Cells[1, 0].SetValue(1);

            value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("2", value);

            worksheet.Cells[2, 0].SetValue(1);

            value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("3", value);
        }

        [TestMethod]
        public void FunctionCellReferenceUnionInvalidationInSecondRangeTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellSelection cell = worksheet.Cells[3, 3];

            cell.SetValue("=sum(A1:A3{ls}B1:D2)".ReplaceCultureDependentSeparators());
            worksheet.Cells[0, 1].SetValue(1);

            string value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("1", value);

            worksheet.Cells[1, 2].SetValue(1);

            value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("2", value);

            worksheet.Cells[1, 3].SetValue(1);

            value = cell.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("3", value);
        }

        [TestMethod]
        public void FunctionCellReferenceRangeInvalidationTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetValue("=sum(B1:B5)");
            worksheet.Cells[1, 1].SetValue(1);

            FormulaCellValue formulaCellValue = (FormulaCellValue)worksheet.Cells[0, 0].GetValue().Value;
            Assert.AreEqual(1, formulaCellValue.Value.GetValueAsConstantExpression().NumberValue());

            worksheet.Cells[2, 1].SetValue("=C3");
            worksheet.Cells[2, 2].SetValue(2);

            formulaCellValue = (FormulaCellValue)worksheet.Cells[0, 0].GetValue().Value;
            Assert.AreEqual(3, formulaCellValue.Value.GetValueAsConstantExpression().NumberValue());
        }

        [TestMethod]
        public void FunctionsCellReferenceRangeTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetValue("=sum(B1:B7)");
            worksheet.Cells[1, 0].SetValue("=abs(B1:B7)");

            worksheet.Cells[0, 1].SetValue(1);
            worksheet.Cells[1, 1].SetValue(2);
            worksheet.Cells[2, 1].SetValue(3);
            worksheet.Cells[3, 1].SetValue("=C4");
            worksheet.Cells[4, 1].SetValue(5);
            worksheet.Cells[5, 1].SetValue(6);
            worksheet.Cells[6, 1].SetValue(7);

            worksheet.Cells[3, 2].SetValue(4);
            worksheet.Cells[4, 2].SetValue(3);
            worksheet.Cells[5, 2].SetValue(3);

            Sum sum = new Sum();
            Abs abs = new Abs();

            CellReferenceRangeExpression A1 = (CellReferenceRangeExpression)ExpressionTestHelper.ParseExpression("=$A$1", worksheet, new CellIndex(10, 10));
            CellReferenceRangeExpression A2 = (CellReferenceRangeExpression)ExpressionTestHelper.ParseExpression("=$A$2", worksheet, new CellIndex(10, 10));

            FunctionsTestHelper.AssertFunctionResult(sum, 28, A1);
            FunctionsTestHelper.AssertFunctionResult(abs, 1, A2);

            worksheet.Cells[4, 1].SetValue("=sum(C5:C6)");

            FunctionsTestHelper.AssertFunctionResult(sum, 29, A1);
            FunctionsTestHelper.AssertFunctionResult(abs, 1, A2);
        }
    }
}
