﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class InformationFunctionsTests
    {
        [TestMethod]
        public void ErrorType()
        {
            ErrorType errorType = new ErrorType();

            FunctionsTestHelper.AssertFunctionResult(errorType, 1, ErrorExpressions.NullError);
            FunctionsTestHelper.AssertFunctionResult(errorType, 2, ErrorExpressions.DivisionByZero);
            FunctionsTestHelper.AssertFunctionResult(errorType, 3, ErrorExpressions.ValueError);
            FunctionsTestHelper.AssertFunctionResult(errorType, 4, ErrorExpressions.ReferenceError);
            FunctionsTestHelper.AssertFunctionResult(errorType, 6, ErrorExpressions.NumberError);
            FunctionsTestHelper.AssertFunctionResult(errorType, 7, ErrorExpressions.NotAvailableError);


            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, 1);
            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, false);
            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, "");
        }

        [TestMethod]
        public void IsBlank()
        {
            IsBlank isBlank = new IsBlank();

            FunctionsTestHelper.AssertFunctionResult(isBlank, true, EmptyExpression.Empty);


            FunctionsTestHelper.AssertFunctionResult(isBlank, false, "");
            FunctionsTestHelper.AssertFunctionResult(isBlank, false, 5);
            FunctionsTestHelper.AssertFunctionResult(isBlank, false, true);
            FunctionsTestHelper.AssertFunctionResult(isBlank, false, 0);
        }

        [TestMethod]
        public void IsNumber()
        {
            IsNumber isNumber = new IsNumber();

            FunctionsTestHelper.AssertFunctionResult(isNumber, true, 5);
            FunctionsTestHelper.AssertFunctionResult(isNumber, true, 0);
            FunctionsTestHelper.AssertFunctionResult(isNumber, true, -5);

            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "5");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "-5");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, true);
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, false);
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "text");
        }

        [TestMethod]
        public void IsText()
        {
            IsText isText = new IsText();

            FunctionsTestHelper.AssertFunctionResult(isText, true, "text");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "0");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "500");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "-500");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "");
            FunctionsTestHelper.AssertFunctionResult(isText, false, 5);
            FunctionsTestHelper.AssertFunctionResult(isText, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isText, false, true);
            FunctionsTestHelper.AssertFunctionResult(isText, false, false);
        }

        [TestMethod]
        public void IsNonText()
        {
            IsNonText isNonText = new IsNonText();

            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "500");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "-500");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "");

            FunctionsTestHelper.AssertFunctionResult(isNonText, true, 5);
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, 0);

            FunctionsTestHelper.AssertFunctionResult(isNonText, true, false);
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, true);

        }

        [TestMethod]
        public void IsLogical()
        {
            IsLogical isLogical = new IsLogical();

            FunctionsTestHelper.AssertFunctionResult(isLogical, true, true);
            FunctionsTestHelper.AssertFunctionResult(isLogical, true, false);
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "false");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "true");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "500");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "-500");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, 5);
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, 0);
        }

        [TestMethod]
        public void IsRef()
        {
            IsRef isRef = new IsRef();

            FunctionsTestHelper.AssertFunctionResult(isRef, false, 1);
            FunctionsTestHelper.AssertFunctionResult(isRef, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isRef, false, "1");
            FunctionsTestHelper.AssertFunctionResult(isRef, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isRef, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isRef, false, true);
            FunctionsTestHelper.AssertFunctionResult(isRef, false, false);
        }

        [TestMethod]
        public void IsError()
        {
            IsError isError = new IsError();

            FunctionsTestHelper.AssertFunctionResult(isError, true, new DivisionExpression(new NumberExpression(5), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isError, true, new PowerExpression(new NumberExpression(0), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isError, true, ErrorExpressions.ValueError);
            FunctionsTestHelper.AssertFunctionResult(isError, true, ErrorExpressions.NotAvailableError);
            FunctionsTestHelper.AssertFunctionResult(isError, true, ErrorExpressions.DivisionByZero);
            FunctionsTestHelper.AssertFunctionResult(isError, true, ErrorExpressions.NumberError);


            FunctionsTestHelper.AssertFunctionResult(isError, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isError, false, -1);
            FunctionsTestHelper.AssertFunctionResult(isError, false, 1);
            FunctionsTestHelper.AssertFunctionResult(isError, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "-1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "true");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "false");
            FunctionsTestHelper.AssertFunctionResult(isError, false, true);
            FunctionsTestHelper.AssertFunctionResult(isError, false, false);
        }

        [TestMethod]
        public void IsErr()
        {
            IsErr isErr = new IsErr();


            FunctionsTestHelper.AssertFunctionResult(isErr, true, new DivisionExpression(new NumberExpression(5), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isErr, true, new PowerExpression(new NumberExpression(0), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isErr, true, ErrorExpressions.ValueError);
            FunctionsTestHelper.AssertFunctionResult(isErr, true, ErrorExpressions.DivisionByZero);
            FunctionsTestHelper.AssertFunctionResult(isErr, true, ErrorExpressions.NumberError);

            FunctionsTestHelper.AssertFunctionResult(isErr, false, ErrorExpressions.NotAvailableError);

            FunctionsTestHelper.AssertFunctionResult(isErr, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isErr, false, -1);
            FunctionsTestHelper.AssertFunctionResult(isErr, false, 1);
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "-1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "true");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "false");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, true);
            FunctionsTestHelper.AssertFunctionResult(isErr, false, false);
        }

        [TestMethod]
        public void IsNA()
        {
            IsNA isNa = new IsNA();

            FunctionsTestHelper.AssertFunctionResult(isNa, true, ErrorExpressions.NotAvailableError);

            FunctionsTestHelper.AssertFunctionResult(isNa, false, new DivisionExpression(new NumberExpression(5), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isNa, false, new PowerExpression(new NumberExpression(0), new NumberExpression(0)));
            FunctionsTestHelper.AssertFunctionResult(isNa, false, ErrorExpressions.ValueError);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, ErrorExpressions.DivisionByZero);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, ErrorExpressions.NumberError);

            FunctionsTestHelper.AssertFunctionResult(isNa, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, -1);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, 1);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "-1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "text");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "true");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "false");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, true);
            FunctionsTestHelper.AssertFunctionResult(isNa, false, false);
        }

        [TestMethod]
        public void IsEven()
        {
            IsEven isEven = new IsEven();

            FunctionsTestHelper.AssertFunctionResult(isEven, true, 0);
            FunctionsTestHelper.AssertFunctionResult(isEven, true, 4);

            FunctionsTestHelper.AssertFunctionResult(isEven, true, "0");
            FunctionsTestHelper.AssertFunctionResult(isEven, true, "4");


            FunctionsTestHelper.AssertFunctionResult(isEven, false, 1);
            FunctionsTestHelper.AssertFunctionResult(isEven, false, -1);

            FunctionsTestHelper.AssertFunctionResult(isEven, false, "5");
            FunctionsTestHelper.AssertFunctionResult(isEven, false, "-11");

            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, false);
     
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, "");
        }

        [TestMethod]
        public void IsOdd()
        {
            IsOdd isOdd = new IsOdd();

            FunctionsTestHelper.AssertFunctionResult(isOdd, true, 1);
            FunctionsTestHelper.AssertFunctionResult(isOdd, true, -1);

            FunctionsTestHelper.AssertFunctionResult(isOdd, true, "5");
            FunctionsTestHelper.AssertFunctionResult(isOdd, true, "-11");

            FunctionsTestHelper.AssertFunctionResult(isOdd, false, 0);
            FunctionsTestHelper.AssertFunctionResult(isOdd, false, 4);
            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "0");
            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "4");

            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, false);

            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, "");
        }

        [TestMethod]
        public void N()
        {
            N nFunction = new N();

            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(nFunction, 10, 10);
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0.234, 0.234);
            FunctionsTestHelper.AssertFunctionResult(nFunction, 9.876, 9.876);
            FunctionsTestHelper.AssertFunctionResult(nFunction, -9.876, -9.876);
            FunctionsTestHelper.AssertFunctionResult(nFunction, -10000, -10000);

            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "10");
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, (0.234).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, (9.876).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, (-9.876).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "-10000");
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "anyString");
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "123anyString");


            FunctionsTestHelper.AssertFunctionResult(nFunction, 1, true);
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, false);


        }

        [TestMethod]
        public void TypeFunction()
        {
            TypeFunction typeFunction = new TypeFunction();

            FunctionsTestHelper.AssertFunctionResult(typeFunction, 1, 5);
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 1, -50);
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 2, "string");
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 2, "100");
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 4, true);
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 4, false);
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 16, ErrorExpressions.NotAvailableError);
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 16, ErrorExpressions.DivisionByZero);

        }
    }
}
