﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class LogicalFunctionsTests
    {
        [TestMethod]
        public void AndFunctionCorrectnessTest()
        {
            And and = new And();

            FunctionsTestHelper.AssertFunctionResult(and, true, true);
            FunctionsTestHelper.AssertFunctionResult(and, false, false);
            FunctionsTestHelper.AssertFunctionResult(and, false, false, false);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, true, true);
            FunctionsTestHelper.AssertFunctionResult(and, false, true, true, false);
        }

        [TestMethod]
        public void AndFunctionIgnoreTextNumberIndirectArgumentsTest()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.ActiveWorksheet;

            worksheet.Cells[0, 0].SetValue(@"=""123""");
            worksheet.Names.Add("indirect", @"=A1", new CellIndex(0, 1));
            worksheet.Cells[0, 1].SetValue("=indirect");
            worksheet.Cells[0, 2].SetValue(@"=A1");
            RadExpression nameExpression = ((FormulaCellValue)worksheet.Cells[0, 1].GetValue().Value).Value;
            RadExpression cellReferenceExpression = ((FormulaCellValue)worksheet.Cells[0, 2].GetValue().Value).Value;

            And and = new And();

            FunctionsTestHelper.AssertFunctionResult(and, true, true, cellReferenceExpression);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, nameExpression);
        }

        [TestMethod]
        public void AndFunctionTreatAsErrorTextNumberDirectArgumentsTest()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.ActiveWorksheet;

            worksheet.Names.Add("direct", @"=""123""", new CellIndex(0, 0));
            worksheet.Cells[0, 1].SetValue("=direct");
            RadExpression nameExpression = ((FormulaCellValue)worksheet.Cells[0, 1].GetValue().Value).Value;

            And and = new And();

            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, true, "123");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, true, nameExpression);
        }

        [TestMethod]
        public void OrFunctionCorrectnessTest()
        {
            Or or = new Or();

            FunctionsTestHelper.AssertFunctionResult(or, true, true);
            FunctionsTestHelper.AssertFunctionResult(or, false, false);
            FunctionsTestHelper.AssertFunctionResult(or, true, false, true, false);
        }

        [TestMethod]
        public void NotFucntionCorrectnessTest()
        {
            Not not = new Not();

            FunctionsTestHelper.AssertFunctionResult(not, false, true);
            FunctionsTestHelper.AssertFunctionResult(not, true, false);
        }

        [TestMethod]
        public void XorFunctionCorrectnessTest()
        {
            Xor xor = new Xor();

            FunctionsTestHelper.AssertFunctionResult(xor, true, true);
            FunctionsTestHelper.AssertFunctionResult(xor, false, false);
            FunctionsTestHelper.AssertFunctionResult(xor, true, false, true, false);
        }

        [TestMethod]
        public void IfFunctionMissingValuesTest()
        {
            ExpressionTestHelper.AssertExpressionResult("IF(true{ls})", typeof(FunctionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("IF(true{ls}{ls})", typeof(FunctionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("IF(false{ls})", typeof(FunctionExpression), false);
            ExpressionTestHelper.AssertExpressionResult("IF(false{ls}{ls})", typeof(FunctionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("IF(3<1{ls}\"OK\")", typeof(FunctionExpression), false);
            ExpressionTestHelper.AssertExpressionResult("IF(3<1{ls}\"OK\"{ls})", typeof(FunctionExpression), 0);
        }

        [TestMethod]
        public void IfFunctionCorrectnessTest1()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[0, 0].SetValue(50);

            ExpressionTestHelper.AssertExpressionResult("IF(A2<=100{ls}\"Within budget\"{ls}\"Over budget\")", typeof(FunctionExpression), "Within budget", worksheet);
            ExpressionTestHelper.AssertExpressionResult("IF(A2=100{ls}A2+B2{ls}\"\")", typeof(FunctionExpression), String.Empty, worksheet);
        }

        [TestMethod]
        public void IfFunctionCorrectnessTest2()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[1, 0].SetValue(1500);
            worksheet.Cells[2, 0].SetValue(500);
            worksheet.Cells[3, 0].SetValue(500);

            worksheet.Cells[1, 1].SetValue(900);
            worksheet.Cells[2, 1].SetValue(900);
            worksheet.Cells[3, 1].SetValue(925);

            ExpressionTestHelper.AssertExpressionResult("IF(A2>B2{ls}\"Over Budget\"{ls}\"OK\")", typeof(FunctionExpression), "Over Budget", worksheet);
            ExpressionTestHelper.AssertExpressionResult("IF(A3>B3{ls}\"Over Budget\"{ls}\"OK\")", typeof(FunctionExpression), "OK", worksheet);
        }

        [TestMethod]
        public void IfFunctionCorrectnessTest3()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[1, 0].SetValue(45);
            worksheet.Cells[2, 0].SetValue(90);
            worksheet.Cells[3, 0].SetValue(78);

            ExpressionTestHelper.AssertExpressionResult("IF(A2>89{ls}\"A\"{ls}IF(A2>79{ls}\"B\"{ls} IF(A2>69{ls}\"C\"{ls}IF(A2>59{ls}\"D\"{ls}\"F\"))))",
                typeof(FunctionExpression), "F", worksheet);
            ExpressionTestHelper.AssertExpressionResult("IF(A3>89{ls}\"A\"{ls}IF(A3>79{ls}\"B\"{ls} IF(A3>69{ls}\"C\"{ls}IF(A3>59{ls}\"D\"{ls}\"F\"))))", 
                typeof(FunctionExpression), "A", worksheet);
            ExpressionTestHelper.AssertExpressionResult("IF(A4>89{ls}\"A\"{ls}IF(A4>79{ls}\"B\"{ls} IF(A4>69{ls}\"C\"{ls}IF(A4>59{ls}\"D\"{ls}\"F\"))))", 
                typeof(FunctionExpression), "C", worksheet);
        }
    }
}
