﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class MathAndTrigonometryFunctionsTests
    {
        [TestMethod]
        public void AbsFunctionTest()
        {
            Abs abs = new Abs();

            FunctionsTestHelper.AssertFunctionResult(abs, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(abs, 2, -2);
            FunctionsTestHelper.AssertFunctionResult(abs, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(abs, 1.7, 1.7);
            FunctionsTestHelper.AssertFunctionResult(abs, 1.3, -1.3);
            FunctionsTestHelper.AssertFunctionResult(abs, 1, true);
            FunctionsTestHelper.AssertFunctionResult(abs, 0, false);
            FunctionsTestHelper.AssertFunctionResult(abs, 123, "123");
            FunctionsTestHelper.AssertFunctionResult(abs, 123, "-123");

            FunctionsTestHelper.AssertFunctionResult(abs, 10, new object[,] { { -10, 5 } });
            FunctionsTestHelper.AssertFunctionResult(abs, 0, new object[,] { { 0, "qwe" } });
        }

        [TestMethod]
        public void AbsFunctionErrorTest()
        {
            Abs abs = new Abs();

            FunctionsTestHelper.AssertFunctionError(abs, ErrorExpressions.ValueError, new object[,] { { "qwe" } });
            FunctionsTestHelper.AssertFunctionError(abs, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void AcosFunctionTest()
        {
            Acos acos = new Acos();

            FunctionsTestHelper.AssertFunctionResult(acos, 3.14159265358979, -1);
            FunctionsTestHelper.AssertFunctionResult(acos, 1.5707963267949, 0);
            FunctionsTestHelper.AssertFunctionResult(acos, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(acos, 1.5707963267949, "0");
            FunctionsTestHelper.AssertFunctionResult(acos, 0, true);
            FunctionsTestHelper.AssertFunctionResult(acos, 1.5707963267949, false);
        }

        [TestMethod]
        public void AcosFunctionErrorTest()
        {
            Acos acos = new Acos();

            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, -1.1);
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, 1.1);
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, -100);
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, 100);
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, "2");
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void AcoshFunctionTest()
        {
            Acosh acosh = new Acosh();

            FunctionsTestHelper.AssertFunctionResult(acosh, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(acosh, 0, true);
            FunctionsTestHelper.AssertFunctionResult(acosh, 0.443568254385115, 1.1);
            FunctionsTestHelper.AssertFunctionResult(acosh, 5.29829236561049, 100);
            FunctionsTestHelper.AssertFunctionResult(acosh, 1.31695789692482, "2");
            FunctionsTestHelper.AssertFunctionResult(acosh, 3.6882538673613, "20");
        }

        [TestMethod]
        public void AcoshFunctionErrorTest()
        {
            Acosh acosh = new Acosh();

            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, false);
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, 0);
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, 0.9);
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, -1.1);
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, -100);
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void AcotFunctionTest()
        {
            Acot acot = new Acot();

            FunctionsTestHelper.AssertFunctionResult(acot, 0.785398163397448, 1);
            FunctionsTestHelper.AssertFunctionResult(acot, 0.785398163397448, true);
            FunctionsTestHelper.AssertFunctionResult(acot, 1.5707963267949, 0);
            FunctionsTestHelper.AssertFunctionResult(acot, 1.5707963267949, false);
            FunctionsTestHelper.AssertFunctionResult(acot, 0.308315662195428, 3.14);
            FunctionsTestHelper.AssertFunctionResult(acot, 2.35619449019234, -1);
            FunctionsTestHelper.AssertFunctionResult(acot, 2.0344439357957, -0.5);
            FunctionsTestHelper.AssertFunctionResult(acot, 3.13159298690313, -100);
            FunctionsTestHelper.AssertFunctionResult(acot, 0.00999966668666524, 100);
            FunctionsTestHelper.AssertFunctionResult(acot, 0.463647609000806, "2");
            FunctionsTestHelper.AssertFunctionResult(acot, 0.0499583957219428, "20");
            FunctionsTestHelper.AssertFunctionResult(acot, 0.099668652491162, 10);
        }

        [TestMethod]
        public void AcotFunctionErrorTest()
        {
            Acot acot = new Acot();

            FunctionsTestHelper.AssertFunctionError(acot, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void AcothFunctionTest()
        {
            Acoth acoth = new Acoth();

            FunctionsTestHelper.AssertFunctionResult(acoth, 8.4056214404672, 1.0000001);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.329944979401731, 3.14);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.0100003333533347, 100);
            FunctionsTestHelper.AssertFunctionResult(acoth, -0.0100003333533348, -100);
            FunctionsTestHelper.AssertFunctionResult(acoth, -0.0200026673068496, -50);
            FunctionsTestHelper.AssertFunctionResult(acoth, -0.14384103622589, -7);
            FunctionsTestHelper.AssertFunctionResult(acoth, -0.549306144334055, -2);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.549306144334055, 2);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.0500417292784913, 20);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.100335347731076, 10);
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.549306144334055, "2");
        }

        [TestMethod]
        public void AcothFunctionErrorTest()
        {
            Acoth acoth = new Acoth();

            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, 1);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, true);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, 0);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, false);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, 0.5);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, -0.5);
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.ValueError, "string");

        }

        [TestMethod]
        public void AsinFunctionTest()
        {
            Asin asin = new Asin();

            FunctionsTestHelper.AssertFunctionResult(asin, -1.5707963267949, -1);
            FunctionsTestHelper.AssertFunctionResult(asin, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(asin, 1.5707963267949, 1);
            FunctionsTestHelper.AssertFunctionResult(asin, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(asin, 1.5707963267949, true);
            FunctionsTestHelper.AssertFunctionResult(asin, 0, false);

        }

        [TestMethod]
        public void AsinFunctionErrorTest()
        {
            Asin asin = new Asin();

            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, -1.1);
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, 1.1);
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, -100);
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, 100);
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, "2");

            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.ValueError, "string");

        }

        [TestMethod]
        public void AsinhFunctionTest()
        {
            Asinh asinh = new Asinh();

            FunctionsTestHelper.AssertFunctionResult(asinh, 0, false);
            FunctionsTestHelper.AssertFunctionResult(asinh, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(asinh, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(asinh, 0.881373587019543, 1);
            FunctionsTestHelper.AssertFunctionResult(asinh, 0.881373587019543, true);
            FunctionsTestHelper.AssertFunctionResult(asinh, 0.950346929821134, 1.1);
            FunctionsTestHelper.AssertFunctionResult(asinh, 5.29834236561059, 100);
            FunctionsTestHelper.AssertFunctionResult(asinh, 1.44363547517881, "2");
            FunctionsTestHelper.AssertFunctionResult(asinh, 3.68950386898891, "20");
            FunctionsTestHelper.AssertFunctionResult(asinh, 0.808866935652783, 0.9);
            FunctionsTestHelper.AssertFunctionResult(asinh, -0.950346929821134, -1.1);
            FunctionsTestHelper.AssertFunctionResult(asinh, -5.2983423656108, -100);
        }

        [TestMethod]
        public void AsinhFunctionErrorTest()
        {
            Asinh asinh = new Asinh();

            FunctionsTestHelper.AssertFunctionError(asinh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Atan()
        {
            Atan atan = new Atan();

            FunctionsTestHelper.AssertFunctionResult(atan, 0, false);
            FunctionsTestHelper.AssertFunctionResult(atan, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(atan, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(atan, 0.785398163397448, 1);
            FunctionsTestHelper.AssertFunctionResult(atan, 0.785398163397448, true);
            FunctionsTestHelper.AssertFunctionResult(atan, 0.832981266674432, 1.1);
            FunctionsTestHelper.AssertFunctionResult(atan, 1.56079666010823, 100);
            FunctionsTestHelper.AssertFunctionResult(atan, 1.10714871779409, "2");
            FunctionsTestHelper.AssertFunctionResult(atan, 1.52083793107295, "20");
            FunctionsTestHelper.AssertFunctionResult(atan, 0.732815101786507, 0.9);
            FunctionsTestHelper.AssertFunctionResult(atan, -0.832981266674432, -1.1);
            FunctionsTestHelper.AssertFunctionResult(atan, -1.56079666010823, -100);

            FunctionsTestHelper.AssertFunctionError(atan, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Atan2()
        {
            Atan2 atan2 = new Atan2();

            FunctionsTestHelper.AssertFunctionResult(atan2, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(atan2, 0, 20, 0);
            FunctionsTestHelper.AssertFunctionResult(atan2, 3.14159265358979, -1, 0);
            FunctionsTestHelper.AssertFunctionResult(atan2, 3.14159265358979, -10, 0);
            FunctionsTestHelper.AssertFunctionResult(atan2, 1.5707963267949, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(atan2, 1.5707963267949, 0, true);
            FunctionsTestHelper.AssertFunctionResult(atan2, 1.5707963267949, false, true);
            FunctionsTestHelper.AssertFunctionResult(atan2, 0.463647609000806, "2", "1");
            FunctionsTestHelper.AssertFunctionResult(atan2, 0.0499583957219428, "20", "1");
            FunctionsTestHelper.AssertFunctionResult(atan2, 0.732815101786507, 100, 90);
            FunctionsTestHelper.AssertFunctionResult(atan2, -0.732815101786507, 100, -90);
            FunctionsTestHelper.AssertFunctionResult(atan2, -2.40877755180329, -100, -90);

            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.DivisionByZero, false, false);
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.DivisionByZero, "0", "0");
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.DivisionByZero, 0, 0);
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.ValueError, "string", 5);
        }

        [TestMethod]
        public void Atanh()
        {
            Atanh atanh = new Atanh();

            FunctionsTestHelper.AssertFunctionResult(atanh, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(atanh, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(atanh, 0, false);
            FunctionsTestHelper.AssertFunctionResult(atanh, 0.100335347731076, 0.1);
            FunctionsTestHelper.AssertFunctionResult(atanh, 1.47221948958322, 0.9);
            FunctionsTestHelper.AssertFunctionResult(atanh, 0.549306144334055, 0.5);
            FunctionsTestHelper.AssertFunctionResult(atanh, -0.100335347731076, -0.1);
            FunctionsTestHelper.AssertFunctionResult(atanh, -1.47221948958322, -0.9);
            FunctionsTestHelper.AssertFunctionResult(atanh, -0.549306144334055, -0.5);

            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, true);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, 1);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, -1.1);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, 1.1);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, -100);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, 100);
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, "2");

            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Ceiling()
        {
            Ceiling ceiling = new Ceiling();

            FunctionsTestHelper.AssertFunctionResult(ceiling, 6, 4.5, 2);
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.NumberError, 4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -4, -4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -6, -4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, 0, -2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, 5, 0);

            FunctionsTestHelper.AssertFunctionResult(ceiling, 3, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -2, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -2, -2.5, 2);

            FunctionsTestHelper.AssertFunctionResult(ceiling, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.NumberError, 1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -1.5, -1.5, -0.1);

            FunctionsTestHelper.AssertFunctionResult(ceiling, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -0.23, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -0.24, -0.234, -0.01);

            FunctionsTestHelper.AssertFunctionResult(ceiling, 7, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -8, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -4, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 5, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -4, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 6, 4.3, 2);

            FunctionsTestHelper.AssertFunctionResult(ceiling, -4, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 1.4, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -1.4, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 654.33, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 654.4, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 655, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 660, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -654.321, -654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -654.32, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -654.3, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, -654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -654, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -650, -654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 3, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(ceiling, 654.321, (654.321).ToString(), (0.001).ToString());

            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.NumberError, 1.3, -0.2);
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.NumberError, 2.5, -2);

            //by definition the following two cases should return #NUM!
            //in excel, however, they are valid scenarios, although they return meaningless result
            FunctionsTestHelper.AssertFunctionResult(ceiling, -2, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceiling, -1.2, -1.3, 0.2);

            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, "text", "text");

        }

        [TestMethod]
        public void CeilingMath()
        {
            CeilingMath ceilingMath = new CeilingMath();

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 3, 2.5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 5, 4.5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 2, 1.5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 1, 0.234);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0, -0.234);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 25, 24.3);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 7, 6.7);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -8, -8.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -5, -5.5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 5, 4.3);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -4, -4.3);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 7, 6.789);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -6, -6.789);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0, 0);

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 3, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 1.5, 1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -1.5, -1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -0.23, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -0.23, -0.234, -0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 7, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -8, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -4, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 5, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -4, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 6, 4.3, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 1.4, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -1.2, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 654.33, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 654.4, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 655, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 660, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -654.321, -654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -654.32, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -654.3, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0, -654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -654, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -650, -654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 3, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 0, true, false);

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 654.321, (654.321).ToCultureDependentString(), (0.001).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -3, -2.5, 1, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -4, -2.5, 2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -0.24, -0.234, 0.01, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -0.24, -0.234, -0.01, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -10, -8.1, 2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -6, -5.5, 2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -6, -4.3, 2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -6, -4.3, -2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -7.9, -7.89, 0.1, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -7.9, -7.89, -0.1, true);

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -7.8, -7.89, 0.1, false);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -7.8, -7.89, -0.1, false);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 6, 4.3, 2, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 6, 4.3, 2, false);

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -3, -2.5, 1, 5);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -4, -2.5, 2, 10);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, -2, -2.5, 2, 0);

            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void CeilingPrecise()
        {
            CeilingPrecise ceilingPrecise = new CeilingPrecise();

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 6, 4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 6, 4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -4, -4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -4, -4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, 0, -2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, 5, 0);

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 3, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -2, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -2, -2.5, 2);

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 1.5, 1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -1.5, -1.5, -0.1);

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -0.23, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -0.23, -0.234, -0.01);

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 7, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -8, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -4, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 5, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -4, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 6, 4.3, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -2, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0.24, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 1.4, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -1.2, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 654.33, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 654.4, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 655, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 660, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -654.321, -654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -654.32, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -654.3, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, -654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -654, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -650, -654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 3, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 654.321, (654.321).ToCultureDependentString(), (0.001).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 1.4, 1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 4, 2.5, -2);

            //by definition the following two cases should return #NUM!
            //in excel, however, they are valid scenarios, although they return meaningless result
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -2, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, -1.2, -1.3, 0.2);

            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, "text", "text");

        }

        [TestMethod]
        public void Combin()
        {
            Combin combin = new Combin();

            FunctionsTestHelper.AssertFunctionResult(combin, 1, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(combin, 1, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(combin, 10, 10, 1);
            FunctionsTestHelper.AssertFunctionResult(combin, 45, 10, 2);
            FunctionsTestHelper.AssertFunctionResult(combin, 252, 10, 5);
            FunctionsTestHelper.AssertFunctionResult(combin, 28, 8, 2);
            FunctionsTestHelper.AssertFunctionResult(combin, 252, "10", "5");

            FunctionsTestHelper.AssertFunctionResult(combin, 20, 20, true);
            FunctionsTestHelper.AssertFunctionResult(combin, 1, true, true);

            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.NumberError, -1, 0);
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.NumberError, 0, -1);
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.NumberError, 2, 4);
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, "text", 4);
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, 5, "text");
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Cos()
        {
            Cos cos = new Cos();

            FunctionsTestHelper.AssertFunctionResult(cos, 1, false);
            FunctionsTestHelper.AssertFunctionResult(cos, 1, "0");
            FunctionsTestHelper.AssertFunctionResult(cos, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.54030230586814, 1);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.54030230586814, true);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.453596121425577, 1.1);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.862318872287684, 100);
            FunctionsTestHelper.AssertFunctionResult(cos, -0.416146836547142, "2");
            FunctionsTestHelper.AssertFunctionResult(cos, 0.408082061813392, "20");
            FunctionsTestHelper.AssertFunctionResult(cos, 0.621609968270664, 0.9);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.453596121425577, -1.1);
            FunctionsTestHelper.AssertFunctionResult(cos, 0.862318872287684, -100);

            FunctionsTestHelper.AssertFunctionError(cos, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Cosh()
        {
            Cosh cosh = new Cosh();


            FunctionsTestHelper.AssertFunctionResult(cosh, 1, false);
            FunctionsTestHelper.AssertFunctionResult(cosh, 1, "0");
            FunctionsTestHelper.AssertFunctionResult(cosh, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(cosh, 1.54308063481524, 1);
            FunctionsTestHelper.AssertFunctionResult(cosh, 1.54308063481524, true);
            FunctionsTestHelper.AssertFunctionResult(cosh, 1.66851855382226, 1.1);
            FunctionsTestHelper.AssertFunctionResult(cosh, 2592352764293540000000d, 50);
            FunctionsTestHelper.AssertFunctionResult(cosh, 3.76219569108363, "2");
            FunctionsTestHelper.AssertFunctionResult(cosh, 242582597.704895, "20");

            FunctionsTestHelper.AssertFunctionResult(cosh, 1.43308638544877, 0.9);
            FunctionsTestHelper.AssertFunctionResult(cosh, 1.66851855382226, -1.1);
            FunctionsTestHelper.AssertFunctionResult(cosh, 2592352764293540000000d, -50);

            FunctionsTestHelper.AssertFunctionError(cosh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Cot()
        {
            Cot cot = new Cot();

            FunctionsTestHelper.AssertFunctionResult(cot, 0.642092615934331, 1);
            FunctionsTestHelper.AssertFunctionResult(cot, 0.642092615934331, true);

            FunctionsTestHelper.AssertFunctionResult(cot, -0.642092615934331, -1);
            FunctionsTestHelper.AssertFunctionResult(cot, 1.83048772171245, 0.5);
            FunctionsTestHelper.AssertFunctionResult(cot, -1.83048772171245, -0.5);
            FunctionsTestHelper.AssertFunctionResult(cot, -627.882397586913, 3.14);
            FunctionsTestHelper.AssertFunctionResult(cot, -1.70295691942647, 100);
            FunctionsTestHelper.AssertFunctionResult(cot, 1.70295691942647, -100);
            FunctionsTestHelper.AssertFunctionResult(cot, 3.67781445085057, -50);
            FunctionsTestHelper.AssertFunctionResult(cot, -1.14751542240514, -7);
            FunctionsTestHelper.AssertFunctionResult(cot, 0.457657554360286, -2);
            FunctionsTestHelper.AssertFunctionResult(cot, -0.457657554360286, 2);
            FunctionsTestHelper.AssertFunctionResult(cot, 0.446995108948917, 20);
            FunctionsTestHelper.AssertFunctionResult(cot, 1.54235104535692, 10);
            FunctionsTestHelper.AssertFunctionResult(cot, -0.457657554360286, "2");

            FunctionsTestHelper.AssertFunctionResult(cot, -0.306893777106121, -134217727);
            FunctionsTestHelper.AssertFunctionResult(cot, 0.306893777106121, 134217727);

            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.NumberError, 134217728);
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.NumberError, -134217728);
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.DivisionByZero, 0);
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.DivisionByZero, false);
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.ValueError, "string");

        }

        [TestMethod]
        public void Coth()
        {
            Coth coth = new Coth();

            FunctionsTestHelper.AssertFunctionResult(coth, 1.31303528549933, 1);
            FunctionsTestHelper.AssertFunctionResult(coth, 1.31303528549933, true);
            FunctionsTestHelper.AssertFunctionResult(coth, -1.31303528549933, -1);
            FunctionsTestHelper.AssertFunctionResult(coth, 2.16395341373865, 0.5);
            FunctionsTestHelper.AssertFunctionResult(coth, -2.16395341373865, -0.5);
            FunctionsTestHelper.AssertFunctionResult(coth, 1.00375383362258, 3.14);
            FunctionsTestHelper.AssertFunctionResult(coth, 1, 100);
            FunctionsTestHelper.AssertFunctionResult(coth, -1, -100);
            FunctionsTestHelper.AssertFunctionResult(coth, -1, -50);
            FunctionsTestHelper.AssertFunctionResult(coth, -1.00000166305882, -7);
            FunctionsTestHelper.AssertFunctionResult(coth, -1.03731472072755, -2);
            FunctionsTestHelper.AssertFunctionResult(coth, 1.03731472072755, 2);
            FunctionsTestHelper.AssertFunctionResult(coth, 1, 20);
            FunctionsTestHelper.AssertFunctionResult(coth, 1.00000000412231, 10);
            FunctionsTestHelper.AssertFunctionResult(coth, 1.03731472072755, "2");

            FunctionsTestHelper.AssertFunctionResult(coth, -1, -134217727);
            FunctionsTestHelper.AssertFunctionResult(coth, 1, 134217727);

            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.DivisionByZero, 0);
            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.DivisionByZero, false);
            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Csc()
        {
            Csc csc = new Csc();

            FunctionsTestHelper.AssertFunctionResult(csc, 1.18839510577812, 1);
            FunctionsTestHelper.AssertFunctionResult(csc, 1.18839510577812, true);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.18839510577812, -1);
            FunctionsTestHelper.AssertFunctionResult(csc, 2.08582964293349, 0.5);
            FunctionsTestHelper.AssertFunctionResult(csc, -2.08582964293349, -0.5);
            FunctionsTestHelper.AssertFunctionResult(csc, 627.883193913876, 3.14);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.9748575314241, 100);
            FunctionsTestHelper.AssertFunctionResult(csc, 1.9748575314241, -100);
            //FunctionsTestHelper.AssertFunctionResult(csc, 3.81134085787211, -50);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.52210106256373, -7);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.09975017029462, -2);
            FunctionsTestHelper.AssertFunctionResult(csc, 1.09975017029462, 2);
            FunctionsTestHelper.AssertFunctionResult(csc, 1.095355936408, 20);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.83816396088967, 10);
            FunctionsTestHelper.AssertFunctionResult(csc, 1.09975017029462, "2");

            FunctionsTestHelper.AssertFunctionResult(csc, 1.04603240409963, -134217727);
            FunctionsTestHelper.AssertFunctionResult(csc, -1.04603240409963, 134217727);

            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.NumberError, 134217728);
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.NumberError, -134217728);
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.DivisionByZero, 0);
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.DivisionByZero, false);
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Csch()
        {
            Csch csch = new Csch();

            FunctionsTestHelper.AssertFunctionResult(csch, 0.850918128239322, 1);
            FunctionsTestHelper.AssertFunctionResult(csch, 0.850918128239322, true);
            FunctionsTestHelper.AssertFunctionResult(csch, -0.850918128239322, -1);
            FunctionsTestHelper.AssertFunctionResult(csch, 1.91903475133494, 0.5);
            FunctionsTestHelper.AssertFunctionResult(csch, -1.91903475133494, -0.5);
            FunctionsTestHelper.AssertFunctionResult(csch, 0.0867280722259757, 3.14);
            FunctionsTestHelper.AssertFunctionResult(csch, -0.00182376544762238, -7);
            FunctionsTestHelper.AssertFunctionResult(csch, -0.275720564771783, -2);
            FunctionsTestHelper.AssertFunctionResult(csch, 0.275720564771783, 2);
            FunctionsTestHelper.AssertFunctionResult(csch, 0.0000907998597121222, 10);
            FunctionsTestHelper.AssertFunctionResult(csch, 0.275720564771783, "2");

            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.DivisionByZero, 0);
            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.DivisionByZero, false);
            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Degrees()
        {
            Degrees degrees = new Degrees();

            FunctionsTestHelper.AssertFunctionResult(degrees, 180, Math.PI);
            FunctionsTestHelper.AssertFunctionResult(degrees, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(degrees, -1145.91559026165, -20);
            FunctionsTestHelper.AssertFunctionResult(degrees, 0, false);
            FunctionsTestHelper.AssertFunctionResult(degrees, 57.2957795130823, true);

            FunctionsTestHelper.AssertFunctionResult(degrees, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(degrees, 572.957795130823, "10");
            FunctionsTestHelper.AssertFunctionResult(degrees, 57.2957795130823, "1");
 
            FunctionsTestHelper.AssertFunctionError(degrees, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Even()
        {
            Even even = new Even();

            FunctionsTestHelper.AssertFunctionResult(even, 2, 1.5);
            FunctionsTestHelper.AssertFunctionResult(even, 4, 3);
            FunctionsTestHelper.AssertFunctionResult(even, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(even, -2, -1);
            FunctionsTestHelper.AssertFunctionResult(even, 2, (1.5).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(even, 4, "3");
            FunctionsTestHelper.AssertFunctionResult(even, 2, true);
            FunctionsTestHelper.AssertFunctionResult(even, 0, false);

            FunctionsTestHelper.AssertFunctionError(even, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Exp()
        {
            Exp exp = new Exp();

            FunctionsTestHelper.AssertFunctionResult(exp, Math.E, 1);
            FunctionsTestHelper.AssertFunctionResult(exp, Math.E, "1");
            FunctionsTestHelper.AssertFunctionResult(exp, Math.E, true);
            FunctionsTestHelper.AssertFunctionResult(exp, Double.Parse((Math.E * Math.E).ToString()), 2);

            FunctionsTestHelper.AssertFunctionError(exp, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Fact()
        {
            Fact fact = new Fact();

            FunctionsTestHelper.AssertFunctionResult(fact, 120, 5);
            FunctionsTestHelper.AssertFunctionResult(fact, 1, 1.9);
            FunctionsTestHelper.AssertFunctionResult(fact, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(fact, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(fact, 1, true);
            FunctionsTestHelper.AssertFunctionResult(fact, 3628800, 10);
            FunctionsTestHelper.AssertFunctionResult(fact, 3628800, "10");

            FunctionsTestHelper.AssertFunctionError(fact, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(fact, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void FactDouble()
        {
            FactDouble factDouble = new FactDouble();

            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, -1);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 3, 3);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 48, 6);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 105, 7);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 3840, 10);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 3840, "10");
            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, false);
            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, true);

            FunctionsTestHelper.AssertFunctionError(factDouble, ErrorExpressions.NumberError, -2);
            FunctionsTestHelper.AssertFunctionError(factDouble, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void FloorFunctionCorrectnessTest()
        {
            Floor floor = new Floor();

            FunctionsTestHelper.AssertFunctionResult(floor, 4, 4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floor, -6, -4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floor, -4, -4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floor, 0, 0, -2);

            FunctionsTestHelper.AssertFunctionResult(floor, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(floor, 0, 0, 5);

            FunctionsTestHelper.AssertFunctionResult(floor, 2, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -3, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -4, -2.5, 2);

            FunctionsTestHelper.AssertFunctionResult(floor, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floor, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floor, -1.5, -1.5, -0.1);

            FunctionsTestHelper.AssertFunctionResult(floor, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floor, -0.24, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floor, -0.23, -0.234, -0.01);

            FunctionsTestHelper.AssertFunctionResult(floor, 6, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -10, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(floor, -6, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floor, 4, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -5, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, 4, 4.3, 2);

            FunctionsTestHelper.AssertFunctionResult(floor, 2, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -2, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floor, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floor, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floor, 1.2, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(floor, -1.2, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(floor, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(floor, 654.32, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floor, 654.3, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floor, 654, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, 650, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(floor, -654.33, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floor, -654.4, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floor, -655, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floor, -660, -654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(floor, 2, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(floor, 654.321, (654.321).ToCultureDependentString(), (0.001).ToCultureDependentString());

            //by definition the following two cases should return #NUM!
            //in excel, however, they are valid scenarios, although they return meaningless result
            FunctionsTestHelper.AssertFunctionResult(floor, -4, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floor, -1.4, -1.3, 0.2);
        }

        [TestMethod]
        public void FloorFuncitonErrorTest()
        {
            Floor floor = new Floor();

            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.NumberError, 4.5, -2);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.NumberError, 1.5, -0.1);

            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.DivisionByZero, 5, 0);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.DivisionByZero, -1, 0);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.DivisionByZero, 1, 0);

            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.NumberError, 1.3, -0.2);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.NumberError, 2.5, -2);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.DivisionByZero, true, false);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void FloorMath()
        {
            FloorMath floorMath = new FloorMath();

            FunctionsTestHelper.AssertFunctionResult(floorMath, 2, 2.5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -3, -2.5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1, 1.5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 0, 0.234);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -1, -0.234);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 24, 24.3);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 6, 6.7);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -9, -8.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -6, -5.5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.3);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -5, -4.3);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 6, 6.789);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -7, -6.789);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 0, 0);

            FunctionsTestHelper.AssertFunctionResult(floorMath, 2, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -3, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1.5, 1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -1.5, -1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -0.24, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -0.24, -0.234, -0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 6, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -10, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -6, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -5, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.3, 2);

            FunctionsTestHelper.AssertFunctionResult(floorMath, 2, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1.2, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -1.4, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 654.32, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 654.3, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 654, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 650, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -654.33, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -654.4, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -655, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -660, -654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 2, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 654.321, (654.321).ToCultureDependentString(), (0.001).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionResult(floorMath, -2, -2.5, 1, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -2, -2.5, 2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -0.23, -0.234, 0.01, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -0.23, -0.234, -0.01, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -8, -8.1, 2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -5.5, 2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -4.3, 2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -4.3, -2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -7.8, -7.89, 0.1, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -7.8, -7.89, -0.1, true);

            FunctionsTestHelper.AssertFunctionResult(floorMath, -7.9, -7.89, 0.1, false);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -7.9, -7.89, -0.1, false);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.3, 2, true);
            FunctionsTestHelper.AssertFunctionResult(floorMath, 4, 4.3, 2, false);

            FunctionsTestHelper.AssertFunctionResult(floorMath, -2, -2.5, 1, 5);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -2, -2.5, 2, 10);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -3, -2.5, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(floorMath, -4, -2.5, 2, 0);

            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void FloorPrecise()
        {
            FloorPrecise floorPrecise = new FloorPrecise();

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 4, 4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 4, 4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -6, -4.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -6, -4.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, 0, -2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, 5, 0);

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 2, 2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -3, -2.5, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -4, -2.5, 2);

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 1.5, 1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 1.5, 1.5, -0.1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -1.5, -1.5, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -1.5, -1.5, -0.1);

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -0.24, -0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -0.24, -0.234, -0.01);

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 6, 6.7, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -10, -8.1, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -6, -5.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 4, 4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -5, -4.3, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 4, 4.3, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -4, -2.5, -2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0.23, 0.234, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 1.2, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -1.4, -1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 654.321, 654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 654.32, 654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 654.3, 654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 654, 654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 650, 654.321, 10);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -654.321, -654.321, 0.001);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -654.33, -654.321, 0.01);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -654.4, -654.321, 0.1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, -654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -655, -654.321, 1);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -660, -654.321, 10);

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 2, 2.5, true);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 654.321, (654.321).ToCultureDependentString(), (0.001).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 1.2, 1.3, -0.2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 2, 2.5, -2);

            //by definition the following two cases should return #NUM!
            //in excel, however, they are valid scenarios, although they return meaningless result
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -4, -2.5, 2);
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, -1.4, -1.3, 0.2);

            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Gcd()
        {
            Gcd gcd = new Gcd();

            FunctionsTestHelper.AssertFunctionResult(gcd, 5, 5);
            FunctionsTestHelper.AssertFunctionResult(gcd, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(gcd, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(gcd, 1, 5, 2);
            FunctionsTestHelper.AssertFunctionResult(gcd, 12, 24, 36);
            FunctionsTestHelper.AssertFunctionResult(gcd, 1, 7, 1);
            FunctionsTestHelper.AssertFunctionResult(gcd, 0, 5, 0);
            FunctionsTestHelper.AssertFunctionResult(gcd, 5, "5");
            FunctionsTestHelper.AssertFunctionResult(gcd, 3, 48, 18, 3);
            FunctionsTestHelper.AssertFunctionResult(gcd, 3, "48", "18", "3");
            FunctionsTestHelper.AssertFunctionResult(gcd, 1, 10, 7);


            FunctionsTestHelper.AssertFunctionResult(gcd, 4, new object[,] { { 12, 16, 4 } });
            FunctionsTestHelper.AssertFunctionResult(gcd, 2, new object[,] { { 12, 16 }, { 2, 4 } });

            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 5, false);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 7, true);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.NumberError, 0, 0, -1);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.NumberError, -2);
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 1, "text");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Int()
        {
            Int intFunction = new Int();

            FunctionsTestHelper.AssertFunctionResult(intFunction, 8, 8.9);
            FunctionsTestHelper.AssertFunctionResult(intFunction, -9, -8.9);
            FunctionsTestHelper.AssertFunctionResult(intFunction, 8, 8.1);
            FunctionsTestHelper.AssertFunctionResult(intFunction, -9, -8.1);
            FunctionsTestHelper.AssertFunctionResult(intFunction, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(intFunction, 0, false);
            FunctionsTestHelper.AssertFunctionResult(intFunction, 1, true);
            FunctionsTestHelper.AssertFunctionResult(intFunction, 8, (8.9).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(intFunction, 10, "10");

            FunctionsTestHelper.AssertFunctionError(intFunction, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Lcm()
        {
            Lcm lcm = new Lcm();

            FunctionsTestHelper.AssertFunctionResult(lcm, 5, 5);
            FunctionsTestHelper.AssertFunctionResult(lcm, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(lcm, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(lcm, 10, 5, 2);
            FunctionsTestHelper.AssertFunctionResult(lcm, 72, 24, 36);
            FunctionsTestHelper.AssertFunctionResult(lcm, 0, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(lcm, 72, "24", "36");
            FunctionsTestHelper.AssertFunctionResult(lcm, 70, 10, 7);

            FunctionsTestHelper.AssertFunctionResult(lcm, 48, new object[,] { { 12, 16 }, { 2, 48 } });
            FunctionsTestHelper.AssertFunctionResult(lcm, 96, new object[,] { { 12, 16 }, { 2, 96 } });

            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 5, false);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 7, true);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.NumberError, 0, 0, -1);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.NumberError, -2);
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 1, "text");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Ln()
        {
            Ln ln = new Ln();

            FunctionsTestHelper.AssertFunctionResult(ln, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(ln, 2.30258509299405, 10);
            FunctionsTestHelper.AssertFunctionResult(ln, 1.14472988584940, Math.PI);
            FunctionsTestHelper.AssertFunctionResult(ln, 0, true);
            FunctionsTestHelper.AssertFunctionResult(ln, 0.693147180559945, "2");

            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.NumberError, 0);
            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.NumberError, false);
            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Log()
        {
            Log log = new Log();

            FunctionsTestHelper.AssertFunctionResult(log, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(log, 1, 10);
            FunctionsTestHelper.AssertFunctionResult(log, 2, 100);
            FunctionsTestHelper.AssertFunctionResult(log, -2, 0.01);
            FunctionsTestHelper.AssertFunctionResult(log, 0, "1");
            FunctionsTestHelper.AssertFunctionResult(log, 1, "10");
            FunctionsTestHelper.AssertFunctionResult(log, 2, "100");
            FunctionsTestHelper.AssertFunctionResult(log, 0, true);

            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, "-1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, 0);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, false);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.ValueError, "text");

            FunctionsTestHelper.AssertFunctionResult(log, 0, 1, 5);
            FunctionsTestHelper.AssertFunctionResult(log, 3, 8, 2);

            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, "1", "0");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, 1, 0);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, "1", "-1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, 1, -1);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.DivisionByZero, "1", "1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.DivisionByZero, 1, 1);
        }

        [TestMethod]
        public void Log10()
        {
            Log10 log10 = new Log10();

            FunctionsTestHelper.AssertFunctionResult(log10, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(log10, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(log10, 1, 10);
            FunctionsTestHelper.AssertFunctionResult(log10, 2, 100);
            FunctionsTestHelper.AssertFunctionResult(log10, -2, 0.01);
            FunctionsTestHelper.AssertFunctionResult(log10, 0, "1");
            FunctionsTestHelper.AssertFunctionResult(log10, 1, "10");
            FunctionsTestHelper.AssertFunctionResult(log10, 2, "100");
            FunctionsTestHelper.AssertFunctionResult(log10, 0, true);

            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.NumberError, "-1");
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.NumberError, 0);
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.NumberError, "0");
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Mod()
        {
            Mod mod = new Mod();

            FunctionsTestHelper.AssertFunctionResult(mod, 1, 3, 2);
            FunctionsTestHelper.AssertFunctionResult(mod, 1, -3, 2);
            FunctionsTestHelper.AssertFunctionResult(mod, -1, 3, -2);
            FunctionsTestHelper.AssertFunctionResult(mod, -1, -3, -2);
            FunctionsTestHelper.AssertFunctionResult(mod, 0.1, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(mod, 0.34, (12.34).ToCultureDependentString(), 1);
            FunctionsTestHelper.AssertFunctionResult(mod, 0.34, (12.34).ToCultureDependentString(), true);

            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.DivisionByZero, 5, 0);
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.DivisionByZero, 5, false);
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Multinomial()
        {
            Multinomial multinomial = new Multinomial();

            FunctionsTestHelper.AssertFunctionResult(multinomial, 1, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(multinomial, 3, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(multinomial, 3, 1.5, 2.5);
            FunctionsTestHelper.AssertFunctionResult(multinomial, 1260, 2, 3, 4);
            FunctionsTestHelper.AssertFunctionResult(multinomial, 3, "1", "2");

            FunctionsTestHelper.AssertFunctionResult(multinomial, 1680, new object[,] { { 1, 2 }, { 2, 3 } });
            FunctionsTestHelper.AssertFunctionResult(multinomial, 3, new object[,] { { 1 }, { 2 } });
            FunctionsTestHelper.AssertFunctionResult(multinomial, 1, new object[,] { { 0 }, { 0 } });

            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.NumberError, -1);
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.NumberError, 5, 6, -1);
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, 1, "text");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, 1, false);
        }

        [TestMethod]
        public void MRound()
        {
            MRound mRound = new MRound();

            FunctionsTestHelper.AssertFunctionResult(mRound, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(mRound, 0, 123, 0);
            FunctionsTestHelper.AssertFunctionResult(mRound, 9, 10, 3);
            FunctionsTestHelper.AssertFunctionResult(mRound, -9, -10, -3);
            FunctionsTestHelper.AssertFunctionResult(mRound, 1.4, 1.3, 0.2);
            FunctionsTestHelper.AssertFunctionResult(mRound, 333.5, 333.7, 0.5);
            FunctionsTestHelper.AssertFunctionResult(mRound, 334, 333.8, 1);
            FunctionsTestHelper.AssertFunctionResult(mRound, 334, 333.3, 2);
            FunctionsTestHelper.AssertFunctionResult(mRound, 400, 555.3, 400);
            FunctionsTestHelper.AssertFunctionResult(mRound, 1000, 555, 1000);
            FunctionsTestHelper.AssertFunctionResult(mRound, -556, -555.7, -1);
            FunctionsTestHelper.AssertFunctionResult(mRound, -555, -555.4, -1);
            FunctionsTestHelper.AssertFunctionResult(mRound, -2000, -1555, -1000);
            FunctionsTestHelper.AssertFunctionResult(mRound, 334, (333.8).ToCultureDependentString(), "1");
            FunctionsTestHelper.AssertFunctionResult(mRound, 1000, "555", "1000");
            FunctionsTestHelper.AssertFunctionResult(mRound, 120, 123.456, 10);

            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, 1, true);
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, 1, false);
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.NumberError, 5, -2);
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, "text", 1);
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, 1, "text");
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Odd()
        {
            Odd odd = new Odd();

            FunctionsTestHelper.AssertFunctionResult(odd, 3, 1.5);
            FunctionsTestHelper.AssertFunctionResult(odd, 3, 3);
            FunctionsTestHelper.AssertFunctionResult(odd, 3, 2);
            FunctionsTestHelper.AssertFunctionResult(odd, -1, -1);
            FunctionsTestHelper.AssertFunctionResult(odd, -3, -2);
            FunctionsTestHelper.AssertFunctionResult(odd, 3, (1.5).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(odd, 3, "3");
            FunctionsTestHelper.AssertFunctionResult(odd, 1, true);
            FunctionsTestHelper.AssertFunctionResult(odd, 1, false);

            FunctionsTestHelper.AssertFunctionError(odd, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Power()
        {
            Power power = new Power();

            FunctionsTestHelper.AssertFunctionResult(power, 25, 5, 2);
            FunctionsTestHelper.AssertFunctionResult(power, 5, 5, 1);
            FunctionsTestHelper.AssertFunctionResult(power, 1, 5, 0);
            FunctionsTestHelper.AssertFunctionResult(power, 0.2, 5, -1);
            FunctionsTestHelper.AssertFunctionResult(power, -1, -1, -1);
            FunctionsTestHelper.AssertFunctionResult(power, 0.25, -2, -2);
            FunctionsTestHelper.AssertFunctionResult(power, 2, 4, 0.5);
            FunctionsTestHelper.AssertFunctionResult(power, 0, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(power, 0, false, true);
            FunctionsTestHelper.AssertFunctionResult(power, 100, "10", "2");
            FunctionsTestHelper.AssertFunctionResult(power, 400, "20", "2");

            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.NumberError, 0, 0);
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.NumberError, false, false);
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.DivisionByZero, 0, -2);
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Product()
        {
            Product product = new Product();

            FunctionsTestHelper.AssertFunctionResult(product, 20, 5, 4);
            FunctionsTestHelper.AssertFunctionResult(product, 9, 1, 9);
            FunctionsTestHelper.AssertFunctionResult(product, 1, 1, true);
            FunctionsTestHelper.AssertFunctionResult(product, 0, false, true);
            FunctionsTestHelper.AssertFunctionResult(product, 45, 5, "9");

            FunctionsTestHelper.AssertFunctionResult(product, 30, new object[,] { { 2, 3 } }, new object[,] { { 5 } });
            FunctionsTestHelper.AssertFunctionResult(product, 10, new object[,] { { 2, "qwe" } }, new object[,] { { 5 } });
            FunctionsTestHelper.AssertFunctionResult(product, 0, new object[,] { { "str", "qwe" } }, new object[,] { { "string" } });

            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.ValueError, "qwe");
            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.ValueError, 1, "qwe");
            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.ValueError, 1, 2, "qwe");
        }

        [TestMethod]
        public void Quotient()
        {
            Quotient quotient = new Quotient();

            FunctionsTestHelper.AssertFunctionResult(quotient, 2, 5, 2);
            FunctionsTestHelper.AssertFunctionResult(quotient, 1, 4.5, 3.1);
            FunctionsTestHelper.AssertFunctionResult(quotient, -3, -10, 3);
            FunctionsTestHelper.AssertFunctionResult(quotient, 45678, 456.789, 0.01);
            FunctionsTestHelper.AssertFunctionResult(quotient, -45678, -456.789, 0.01);
            FunctionsTestHelper.AssertFunctionResult(quotient, -45678, 456.789, -0.01);
            FunctionsTestHelper.AssertFunctionResult(quotient, 45678, -456.789, -0.01);
            FunctionsTestHelper.AssertFunctionResult(quotient, 3, "10", "3");
            FunctionsTestHelper.AssertFunctionResult(quotient, 6, "20", "3");

            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, 2, true);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, 2, false);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, true, 3);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, false, 5);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.DivisionByZero, 2, 0);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Radians()
        {
            Radians radians = new Radians();

            FunctionsTestHelper.AssertFunctionResult(radians, Math.PI, 180);
            FunctionsTestHelper.AssertFunctionResult(radians, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(radians, -0.349065850398866, -20);
            FunctionsTestHelper.AssertFunctionResult(radians, 0, false);
            FunctionsTestHelper.AssertFunctionResult(radians, 0.0174532925199433, true);

            FunctionsTestHelper.AssertFunctionResult(radians, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(radians, 0.174532925199433, "10");
            FunctionsTestHelper.AssertFunctionResult(radians, 0.0174532925199433, "1");

            FunctionsTestHelper.AssertFunctionError(radians, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Round()
        {
            Round round = new Round();

            FunctionsTestHelper.AssertFunctionResult(round, 2.2, 2.15, 1);
            FunctionsTestHelper.AssertFunctionResult(round, 2.1, 2.149, 1);
            FunctionsTestHelper.AssertFunctionResult(round, -1.48, -1.475, 2);
            FunctionsTestHelper.AssertFunctionResult(round, 20, 21.5, -1);
            FunctionsTestHelper.AssertFunctionResult(round, 0, 0, -2);
            FunctionsTestHelper.AssertFunctionResult(round, 0, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(round, 98765.432, 98765.4321, 3);
            FunctionsTestHelper.AssertFunctionResult(round, 98765.43, 98765.4321, 2);
            FunctionsTestHelper.AssertFunctionResult(round, 98765.4, 98765.4321, 1.7);
            FunctionsTestHelper.AssertFunctionResult(round, 98765.4, 98765.4321, 1.3);
            FunctionsTestHelper.AssertFunctionResult(round, 98765.4, 98765.4321, 1);
            FunctionsTestHelper.AssertFunctionResult(round, 98765, 98765.4321, 0);
            FunctionsTestHelper.AssertFunctionResult(round, 98770, 98765.4321, -1);
            FunctionsTestHelper.AssertFunctionResult(round, 98770, 98765.4321, -1.3);
            FunctionsTestHelper.AssertFunctionResult(round, 98770, 98765.4321, -1.7);
            FunctionsTestHelper.AssertFunctionResult(round, 98800, 98765.4321, -2);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.568, 1234.56789, 3);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.57, 1234.56789, 2);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.6, 1234.56789, 1.7);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.6, 1234.56789, 1.3);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.6, 1234.56789, 1);
            FunctionsTestHelper.AssertFunctionResult(round, 1235, 1234.56789, 0);
            FunctionsTestHelper.AssertFunctionResult(round, 1230, 1234.56789, -1);
            FunctionsTestHelper.AssertFunctionResult(round, 1230, 1234.56789, -1.3);
            FunctionsTestHelper.AssertFunctionResult(round, 1230, 1234.56789, -1.7);
            FunctionsTestHelper.AssertFunctionResult(round, 1200, 1234.56789, -2);
            FunctionsTestHelper.AssertFunctionResult(round, 1234.6, 1234.56789, true);

            FunctionsTestHelper.AssertFunctionResult(round, 1235, 1234.56789, false);
            FunctionsTestHelper.AssertFunctionResult(round, 1235, (1234.56789).ToCultureDependentString(), "0");
            FunctionsTestHelper.AssertFunctionResult(round, 1230, (1234.56789).ToCultureDependentString(), (-1.3).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void RoundDown()
        {
            RoundDown roundDown = new RoundDown();

            FunctionsTestHelper.AssertFunctionResult(roundDown, 3, 3.2, 0);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 3.2, 3.2, 1.5);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 76, 76.9, 0);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 3.141, 3.14159, 3);
            FunctionsTestHelper.AssertFunctionResult(roundDown, -3.1, -3.14159, 1);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 31400, 31415.92654, -2);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 123.456, 123.456, 3);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 123.45, 123.456, 2);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 123, 123.456, 0);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 120, 123.456, -1);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 120, 123.456, -1.3);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 120, 123.456, -1.7);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 100, 123.456, -2);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 654.321, 654.321, 3);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 654.32, 654.321, 2);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 654, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 650, 654.321, -1);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 650, 654.321, -1.3);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 650, 654.321, -1.7);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 600, 654.321, -2);
            FunctionsTestHelper.AssertFunctionResult(roundDown, -3.1, -3.14159, true);

            FunctionsTestHelper.AssertFunctionResult(roundDown, 123, 123.456, false);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 10, "10", 1);

            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void RoundUp()
        {
            RoundUp roundUp = new RoundUp();

            FunctionsTestHelper.AssertFunctionResult(roundUp, 4, 3.2, 0);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 77, 76.9, 0);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 3.142, 3.14159, 3);
            FunctionsTestHelper.AssertFunctionResult(roundUp, -3.2, -3.14159, 1);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 31500, 31415.92654, -2);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 123.456, 123.456, 3);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 123.46, 123.456, 2);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 124, 123.456, 0);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 130, 123.456, -1);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 130, 123.456, -1.3);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 130, 123.456, -1.7);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 200, 123.456, -2);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 654.321, 654.321, 3);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 654.33, 654.321, 2);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 655, 654.321, 0);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 660, 654.321, -1);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 660, 654.321, -1.3);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 660, 654.321, -1.7);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 700, 654.321, -2);
            FunctionsTestHelper.AssertFunctionResult(roundUp, -3.2, -3.14159, true);

            FunctionsTestHelper.AssertFunctionResult(roundUp, 124, 123.456, false);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 10, "10", 1);

            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Sec()
        {
            Sec sec = new Sec();

            FunctionsTestHelper.AssertFunctionResult(sec, 1.85081571768093, 1);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.85081571768093, true);
            FunctionsTestHelper.AssertFunctionResult(sec, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(sec, 1, false);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.85081571768093, -1);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.13949392732455, 0.5);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.13949392732455, -0.5);
            FunctionsTestHelper.AssertFunctionResult(sec, -1.00000126827407, 3.14);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.15966382290469, 100);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.15966382290469, -100);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.036305911787, -50);
            FunctionsTestHelper.AssertFunctionResult(sec, 1.3264319004737, -7);
            FunctionsTestHelper.AssertFunctionResult(sec, -2.40299796172238, -2);
            FunctionsTestHelper.AssertFunctionResult(sec, -2.40299796172238, 2);
            FunctionsTestHelper.AssertFunctionResult(sec, 2.45048752095671, 20);
            FunctionsTestHelper.AssertFunctionResult(sec, -1.1917935066879, 10);
            FunctionsTestHelper.AssertFunctionResult(sec, -2.40299796172238, "2");

            FunctionsTestHelper.AssertFunctionResult(sec, -3.40845100856485, -134217727);
            FunctionsTestHelper.AssertFunctionResult(sec, -3.40845100856485, 134217727);

            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.NumberError, 134217728);
            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.NumberError, -134217728);
            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Sech()
        {
            Sech sech = new Sech();

            FunctionsTestHelper.AssertFunctionResult(sech, 0.648054273663885, 1);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.648054273663885, true);
            FunctionsTestHelper.AssertFunctionResult(sech, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(sech, 1, false);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.648054273663885, -1);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.886818883970074, 0.5);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.886818883970074, -0.5);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.0864037270104075, 3.14);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.00182376241459821, -7);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.26580222883408, -2);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.26580222883408, 2);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.0000907998593378173, 10);
            FunctionsTestHelper.AssertFunctionResult(sech, 0.26580222883408, "2");

            FunctionsTestHelper.AssertFunctionError(sech, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Sign()
        {
            Sign sign = new Sign();

            FunctionsTestHelper.AssertFunctionResult(sign, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(sign, 0, false);
            FunctionsTestHelper.AssertFunctionResult(sign, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(sign, 1, true);

            FunctionsTestHelper.AssertFunctionResult(sign, -1, -50);
            FunctionsTestHelper.AssertFunctionResult(sign, 1, 50);
            FunctionsTestHelper.AssertFunctionResult(sign, -1, (-1.5).ToCultureDependentString());
            FunctionsTestHelper.AssertFunctionResult(sign, 1, (1.5).ToCultureDependentString());

            FunctionsTestHelper.AssertFunctionError(sign, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void Sin()
        {
            Sin sin = new Sin();

            FunctionsTestHelper.AssertFunctionResult(sin, 0, false);
            FunctionsTestHelper.AssertFunctionResult(sin, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(sin, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(sin, 0.841470984807897, 1);
            FunctionsTestHelper.AssertFunctionResult(sin, 0.841470984807897, true);

            FunctionsTestHelper.AssertFunctionResult(sin, 0.891207360061435, 1.1);
            FunctionsTestHelper.AssertFunctionResult(sin, -0.506365641109759, 100);
            FunctionsTestHelper.AssertFunctionResult(sin, 0.909297426825682, "2");
            FunctionsTestHelper.AssertFunctionResult(sin, 0.912945250727628, "20");

            FunctionsTestHelper.AssertFunctionResult(sin, 0.783326909627483, 0.9);
            FunctionsTestHelper.AssertFunctionResult(sin, -0.891207360061435, -1.1);
            FunctionsTestHelper.AssertFunctionResult(sin, 0.506365641109759, -100);

            FunctionsTestHelper.AssertFunctionError(sin, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Sinh()
        {
            Sinh sinh = new Sinh();

            FunctionsTestHelper.AssertFunctionResult(sinh, 0, false);
            FunctionsTestHelper.AssertFunctionResult(sinh, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(sinh, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(sinh, 1.1752011936438, 1);
            FunctionsTestHelper.AssertFunctionResult(sinh, 1.1752011936438, true);

            FunctionsTestHelper.AssertFunctionResult(sinh, 1.33564747012418, 1.1);
            FunctionsTestHelper.AssertFunctionResult(sinh, 2592352764293540000000d, 50);
            FunctionsTestHelper.AssertFunctionResult(sinh, 3.62686040784702, "2");
            FunctionsTestHelper.AssertFunctionResult(sinh, 242582597.704895, "20");

            FunctionsTestHelper.AssertFunctionResult(sinh, 1.02651672570818, 0.9);
            FunctionsTestHelper.AssertFunctionResult(sinh, -1.33564747012418, -1.1);
            FunctionsTestHelper.AssertFunctionResult(sinh, -2592352764293540000000d, -50);

            FunctionsTestHelper.AssertFunctionError(sinh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Sqrt()
        {
            Sqrt sqrt = new Sqrt();

            FunctionsTestHelper.AssertFunctionResult(sqrt, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(sqrt, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(sqrt, 5, 25);
            FunctionsTestHelper.AssertFunctionResult(sqrt, 2, 4);
            FunctionsTestHelper.AssertFunctionResult(sqrt, 0, false);
            FunctionsTestHelper.AssertFunctionResult(sqrt, 1, true);

            FunctionsTestHelper.AssertFunctionResult(sqrt, 4, "16");
            FunctionsTestHelper.AssertFunctionResult(sqrt, 10, "100");

            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.NumberError, -2);
            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.NumberError, "-1");
            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.ValueError, "text");
        }

        [TestMethod]
        public void SqrtPi()
        {
            SqrtPi sqrtPi = new SqrtPi();

            FunctionsTestHelper.AssertFunctionResult(sqrtPi, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(sqrtPi, Math.PI, Math.PI);
            FunctionsTestHelper.AssertFunctionResult(sqrtPi, 5.60499121639793, 10);
            FunctionsTestHelper.AssertFunctionResult(sqrtPi, 5.60499121639793, "10");

            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.NumberError, -2);
            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.NumberError, "-1");
            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.ValueError, true);

            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.ValueError, false);
        }

        [TestMethod]
        public void Sum()
        {
            Sum sum = new Sum();

            FunctionsTestHelper.AssertFunctionResult(sum, 10, 1, 9);
            FunctionsTestHelper.AssertFunctionResult(sum, 2, 1, true);
            FunctionsTestHelper.AssertFunctionResult(sum, 1, false, true);
            FunctionsTestHelper.AssertFunctionResult(sum, 10, 1, "9");

            FunctionsTestHelper.AssertFunctionResult(sum, 0, new object[,] { { "text" }, { "text" } }, new object[,] { { "text" } });
            FunctionsTestHelper.AssertFunctionResult(sum, 5, new object[,] { { "text" }, { 5 } }, new object[,] { { "text" } });
            FunctionsTestHelper.AssertFunctionResult(sum, 0, new object[,] { { "text" }, { "5" } }, new object[,] { { "text" } });

            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.ValueError, "qwe");
            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.ValueError, 1, "qwe");
            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.ValueError, 1, 2, "qwe");
        }

        [TestMethod]
        public void SumSq()
        {
            SumSq sumSq = new SumSq();

            FunctionsTestHelper.AssertFunctionResult(sumSq, 82, 1, 9);
            FunctionsTestHelper.AssertFunctionResult(sumSq, 2, 1, true);
            FunctionsTestHelper.AssertFunctionResult(sumSq, 1, false, true);
            FunctionsTestHelper.AssertFunctionResult(sumSq, 82, 1, "9");

            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.ValueError, "qwe");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.ValueError, 1, "qwe");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.ValueError, 1, 2, "qwe");
        }

        [TestMethod]
        public void Tan()
        {
            Tan tan = new Tan();

            FunctionsTestHelper.AssertFunctionResult(tan, 0, false);
            FunctionsTestHelper.AssertFunctionResult(tan, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(tan, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(tan, 1.5574077246549, 1);
            FunctionsTestHelper.AssertFunctionResult(tan, 1.5574077246549, true);

            FunctionsTestHelper.AssertFunctionResult(tan, 1.96475965724865, 1.1);
            FunctionsTestHelper.AssertFunctionResult(tan, -0.587213915156929, 100);
            FunctionsTestHelper.AssertFunctionResult(tan, -2.18503986326152, "2");
            FunctionsTestHelper.AssertFunctionResult(tan, 2.23716094422474, "20");

            FunctionsTestHelper.AssertFunctionResult(tan, 1.26015821755034, 0.9);
            FunctionsTestHelper.AssertFunctionResult(tan, -1.96475965724865, -1.1);
            FunctionsTestHelper.AssertFunctionResult(tan, 0.587213915156929, -100);

            FunctionsTestHelper.AssertFunctionError(tan, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Tanh()
        {
            Tanh tanh = new Tanh();

            FunctionsTestHelper.AssertFunctionResult(tanh, 0, false);
            FunctionsTestHelper.AssertFunctionResult(tanh, 0, "0");
            FunctionsTestHelper.AssertFunctionResult(tanh, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.761594155955765, 1);
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.761594155955765, true);
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.80049902176063, 1.1);
            FunctionsTestHelper.AssertFunctionResult(tanh, 1, 100);
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.964027580075817, "2");
            FunctionsTestHelper.AssertFunctionResult(tanh, 1, "20");
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.716297870199024, 0.9);
            FunctionsTestHelper.AssertFunctionResult(tanh, -0.80049902176063, -1.1);
            FunctionsTestHelper.AssertFunctionResult(tanh, -1, -100);

            FunctionsTestHelper.AssertFunctionError(tanh, ErrorExpressions.ValueError, "string");
        }

        [TestMethod]
        public void Trunc()
        {
            Trunc trunc = new Trunc();

            FunctionsTestHelper.AssertFunctionResult(trunc, 8, 8.9);
            FunctionsTestHelper.AssertFunctionResult(trunc, -8, -8.9);
            FunctionsTestHelper.AssertFunctionResult(trunc, 8, 8.1);
            FunctionsTestHelper.AssertFunctionResult(trunc, -8, -8.1);
            FunctionsTestHelper.AssertFunctionResult(trunc, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(trunc, 0, 1234.5678, -10);
            FunctionsTestHelper.AssertFunctionResult(trunc, 0, 1234.5678, -4);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1000, 1234.5678, -3);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1200, 1234.5678, -2);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1230, 1234.5678, -1);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234, 1234.5678, 0);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.5, 1234.5678, 1);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.56, 1234.5678, 2);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.567, 1234.5678, 3);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.5678, 1234.5678, 10);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234, 1234.5678, false);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.5, 1234.5678, true);
            FunctionsTestHelper.AssertFunctionResult(trunc, 1.0000012345, 1.0000012345678, 10);

            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, "text", "text");
        }

        #region SumIf tests

        [TestMethod]
        public void SumIfWildCharactersCriteriaTests()
        {
            this.AssertSumIfCorrectnessTest("someText56~?~~7", new string[,] { { "someText56?~7" } }, new string[,] { { "32" } }, 32);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?7" } }, new string[,] { { "3{ds}14".ReplaceCultureDependentSeparators() } }, 3.14);
            this.AssertSumIfCorrectnessTest("someText56?~7", new string[,] { { "someText56?7" } }, new string[,] { { "3{ds}14".ReplaceCultureDependentSeparators() } }, 3.14);
            this.AssertSumIfCorrectnessTest("someText56*~7", new string[,] { { "someText56?7" } }, new string[,] { { "3{ds}14".ReplaceCultureDependentSeparators() } }, 3.14);
            this.AssertSumIfCorrectnessTest("someText56~7", new string[,] { { "someText56?7" } }, new string[,] { { "3{ds}14".ReplaceCultureDependentSeparators() } }, 0);
            this.AssertSumIfCorrectnessTest("someText56~?7", new string[,] { { "someText56?7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 2.78);
            this.AssertSumIfCorrectnessTest("someText56~??7", new string[,] { { "someText56?Q7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 2.78);
            this.AssertSumIfCorrectnessTest("someText56~??7~", new string[,] { { "someText56?Q7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 2.78);
            this.AssertSumIfCorrectnessTest("~so~me~Text56~??7~", new string[,] { { "someText56?Q7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 2.78);
            this.AssertSumIfCorrectnessTest("~so~me~Text56~??~7~", new string[,] { { "someText56?Q~7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 0);
            this.AssertSumIfCorrectnessTest("~so~me~Text56~??~~7~", new string[,] { { "someText56?Q~7" } }, new string[,] { { "2{ds}78".ReplaceCultureDependentSeparators() } }, 2.78);
            this.AssertSumIfCorrectnessTest("someText56~?*~~~7", new string[,] { { "someText56?Q~7" } }, new string[,] { { "7418880" } }, 7418880);
            this.AssertSumIfCorrectnessTest("someText56~?*7", new string[,] { { "someText56?Q~7" } }, new string[,] { { "7418880" } }, 7418880);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?7" } }, new string[,] { { "7418880" } }, 7418880);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?~7" } }, new string[,] { { "7418880" } }, 0);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56Q7" } }, new string[,] { { "7418880" } }, 0);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56~?7" } }, new string[,] { { "7418880" } }, 0);
            this.AssertSumIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56~?~7" } }, new string[,] { { "7418880" } }, 0); 
        }

        [TestMethod]
        public void SumIfComparissonOperatorsTests()
        {
            this.AssertSumIfCorrectnessTest(
                ">=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                7);

            this.AssertSumIfCorrectnessTest(
                "<=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                9);

            this.AssertSumIfCorrectnessTest(
                "<=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "\"8\"" } },
                1);

            this.AssertSumIfCorrectnessTest(
                "=\"=q1234\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                1);

            this.AssertSumIfCorrectnessTest(
                "q1234",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                1);

            this.AssertSumIfCorrectnessTest(
                "=\"=q123?\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                13);

            this.AssertSumIfCorrectnessTest(
                "=\"=q12?3\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                8);

            this.AssertSumIfCorrectnessTest(
                ">=q1234*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                4);

            this.AssertSumIfCorrectnessTest(
                ">=q*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                13);

            this.AssertSumIfCorrectnessTest(
                ">=q12?4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                13);

            this.AssertSumIfCorrectnessTest(
                ">=q12??4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                13);

            this.AssertSumIfCorrectnessTest(
                "",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                0);

            this.AssertSumIfCorrectnessTest(
                "<=q1234*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                11);

            this.AssertSumIfCorrectnessTest(
                "<=q*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                2);

            this.AssertSumIfCorrectnessTest(
                "<=q12?4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                2);

            this.AssertSumIfCorrectnessTest(
                "<=q12??4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                new string[,] { { "1", "2" }, { "4", "8" } },
                2);
        }

        [TestMethod]
        public void SumIfNumberCellValuesTests()
        {
            this.AssertSumIfCorrectnessTest(
                "<  3 ",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                48);

            this.AssertSumIfCorrectnessTest(
                "<=3 ",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                56);

            this.AssertSumIfCorrectnessTest(
                ">3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                7);

            this.AssertSumIfCorrectnessTest(
                ">=3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                15);

            this.AssertSumIfCorrectnessTest(
                "=\"3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                8);

            this.AssertSumIfCorrectnessTest(
                "=\"=3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                8);

            this.AssertSumIfCorrectnessTest(
                "=\"<>3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                55);

            this.AssertSumIfCorrectnessTest(
                "=3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                8);

            this.AssertSumIfCorrectnessTest(
                "=1",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                32);

            this.AssertSumIfCorrectnessTest(
                "=TRUE",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                0);

            this.AssertSumIfCorrectnessTest(
                "TRUE",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                0);
        }

        [TestMethod]
        public void SumIfBooleanCellValueTests()
        {
            this.AssertSumIfCorrectnessTest(
                "<tRuE",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                20);

            this.AssertSumIfCorrectnessTest(
                "<=true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                60);

            this.AssertSumIfCorrectnessTest(
                ">true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                0);

            this.AssertSumIfCorrectnessTest(
                ">=true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                40);

            this.AssertSumIfCorrectnessTest(
                "=\"=true\"",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                40);

            this.AssertSumIfCorrectnessTest(
                "<>true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                23);

            this.AssertSumIfCorrectnessTest(
                "<>False",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                43);

            this.AssertSumIfCorrectnessTest(
                 "< true",
                 new string[,] { { "=\"TRUE\"", "=\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 2);

            this.AssertSumIfCorrectnessTest(
                 "< true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 2);

            this.AssertSumIfCorrectnessTest(
                 "> true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 60);

            this.AssertSumIfCorrectnessTest(
                 "<> true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 62);

            this.AssertSumIfCorrectnessTest(
                 "=\" true\"",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 1);

            this.AssertSumIfCorrectnessTest(
                 "=\"= true\"",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 new string[,] { { "32", "16" }, { "8", "4" }, { "2", "1" } },
                 1);
        }

        [TestMethod]
        public void SumIfErrorsInReferenceArgumentTest()
        {
            string errorValue = "=sum(\"\")";

            this.AssertSumIfCorrectnessTest(
                "=0",
                new string[,] { { errorValue, errorValue, "0" }, { errorValue, errorValue, "1" }, { "", errorValue, "0" } },
                new string[,] { { "256", "128", "64" }, { "32", "16", "8" }, { "4", "2", "1" } },
                65);
        }

        [TestMethod]
        public void SumIfMultupleCellRangesValueErrorNotPoppingUpReferenceErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=SUMIF((A1{ls}A2){ls}0)", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SumIfMultupleCellRangesValueErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=SUMIF((B1{ls}B2{ls}B3){ls}0)", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SumIfMultupleCellRangesInOptionalArgumentValueErrorNotPoppingUpReferenceErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=SUMIF(A2{ls}0{ls}(A1{ls}A2))", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SumIfMultupleCellRangesInOptionalArgumentValueErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=SUMIF(C3{ls}0{ls}(B1{ls}B2{ls}B3))", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SumIfErrorCriteriaSummingTest()
        {
            string errorValue = "=sum(\"\")";
            string stringErrorValue = ErrorExpressions.ValueError.Value;
            string stringValue = String.Format("=\"{0}\"", stringErrorValue);
            string refError = ErrorExpressions.ReferenceError.Value;
            string divError = ErrorExpressions.DivisionByZero.Value;

            this.AssertSumIfCorrectnessTest(
                errorValue,
                new string[,] { { errorValue, refError, divError }, { "0", "asdf", errorValue }, { stringValue, stringErrorValue, errorValue } },
                new string[,] { { "256", "128", "64" }, { "32", "16", "8" }, { "4", "2", "1" } },
                267);
        }

        [TestMethod]
        public void SumIfReferenceErrorVariationsTests()
        {
            FunctionsTestHelper.AssertFunctionReferenceErrorVariationsTests("Sumif", 2, 3, "1");
        }

        private void AssertSumIfCorrectnessTest(string criteria, string[,] cellValues, string[,] summands, double expectedValue)
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            int rangesMaxSize = 10;

            worksheet.Cells[0, 1].SetValue(criteria);

            for (int i = 0; i < cellValues.GetLength(0); i++)
            {
                for (int j = 0; j < cellValues.GetLength(1); j++)
                {
                    worksheet.Cells[i + 1, j + 1].SetValue(cellValues[i, j]);
                }
            }

            for (int i = 0; i < summands.GetLength(0); i++)
            {
                for (int j = 0; j < summands.GetLength(1); j++)
                {
                    worksheet.Cells[i + 1 + rangesMaxSize, j + 1 + rangesMaxSize].SetValue(summands[i, j]);
                }
            }

            worksheet.Cells[0, 0].SetValue("=SUMIF(B2:K11{ls}B1{ls}L12)".ReplaceCultureDependentSeparators());
            FormulaCellValue formula = worksheet.Cells[0, 0].GetValue().Value as FormulaCellValue;
            string cellValueOfA1 = formula.GetResultValueAsCellValue().RawValue;
            double actualValue = double.Parse(cellValueOfA1);

            Assert.AreEqual(expectedValue, actualValue);
        }

        #endregion
    }
}
