﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class StatisticalFunctionsTests
    {
        [TestMethod]
        public void Average()
        {
            Average average = new Average();

            FunctionsTestHelper.AssertFunctionResult(average, 6, 4, 8);
            FunctionsTestHelper.AssertFunctionResult(average, 1, 1, true);
            FunctionsTestHelper.AssertFunctionResult(average, 0.5, false, true);
            FunctionsTestHelper.AssertFunctionResult(average, 8, 7, "9");
            FunctionsTestHelper.AssertFunctionResult(average, 50, "0", "100");

            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.ValueError, "qwe");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.ValueError, 1, "qwe");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.ValueError, 1, 2, "qwe");

        }

        [TestMethod]
        public void AverageA()
        {
            AverageA averagea = new AverageA();

            FunctionsTestHelper.AssertFunctionResult(averagea, 6, 4, 8);
            FunctionsTestHelper.AssertFunctionResult(averagea, 1, 1, true);
            FunctionsTestHelper.AssertFunctionResult(averagea, 0.5, false, true);
            FunctionsTestHelper.AssertFunctionResult(averagea, 8, 7, "9");
            FunctionsTestHelper.AssertFunctionResult(averagea, 50, "0", "100");

            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.ValueError, "qwe");
            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.ValueError, 1, "qwe");
            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.ValueError, 1, 2, "qwe");
        }

        [TestMethod]
        public void Max()
        {
            Max max = new Max();

            FunctionsTestHelper.AssertFunctionResult(max, 2, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(max, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(max, 1, true, true);
            FunctionsTestHelper.AssertFunctionResult(max, 2, "1", "2");

            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.ValueError, "1", "text");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.ValueError, -1, "text");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void MaxA()
        {
            MaxA maxa = new MaxA();

            FunctionsTestHelper.AssertFunctionResult(maxa, 2, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(maxa, 1, true, false);
            FunctionsTestHelper.AssertFunctionResult(maxa, 1, true, true);
            FunctionsTestHelper.AssertFunctionResult(maxa, 2, "1", "2");

            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.ValueError, "1", "text");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.ValueError, -1, "text");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Median()
        {
            Median median = new Median();

            FunctionsTestHelper.AssertFunctionResult(median, 2, 1, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(median, 2, 3, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(median, 2, 2, 3, 1);
            FunctionsTestHelper.AssertFunctionResult(median, 1.5, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(median, 1.5, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(median, 1.5, 1, 1, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(median, 1.5, 1, 3, 2, 1);
            FunctionsTestHelper.AssertFunctionResult(median, -1, -1);
            FunctionsTestHelper.AssertFunctionResult(median, 5, "5");
            FunctionsTestHelper.AssertFunctionResult(median, 7.5, "5", "10");
            FunctionsTestHelper.AssertFunctionResult(median, 1, true, true);
            FunctionsTestHelper.AssertFunctionResult(median, 0.5, true, false);
            FunctionsTestHelper.AssertFunctionResult(median, 5, 10, false);

            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.ValueError, "1", "text");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.ValueError, 10, "text");
        }

        [TestMethod]
        public void Min()
        {
            Min min = new Min();

            FunctionsTestHelper.AssertFunctionResult(min, 1, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(min, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(min, 1, true, true);
            FunctionsTestHelper.AssertFunctionResult(min, 1, "1", "2");

            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.ValueError, "1", "text");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.ValueError, -1, "text");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void MinA()
        {
            MinA mina = new MinA();

            FunctionsTestHelper.AssertFunctionResult(mina, 1, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(mina, 0, true, false);
            FunctionsTestHelper.AssertFunctionResult(mina, 1, true, true);
            FunctionsTestHelper.AssertFunctionResult(mina, 1, "1", "2");

            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.ValueError, "1", "text");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.ValueError, -1, "text");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Mode()
        {
            Mode mode = new Mode();

            FunctionsTestHelper.AssertFunctionResult(mode, 1, 1, 1, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(mode, 3, 3, 3, 2, 2);
            FunctionsTestHelper.AssertFunctionResult(mode, 3, 3, 2, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(mode, 2, 2, 3, 3, 2);
            FunctionsTestHelper.AssertFunctionResult(mode, 1, 1, 1, 2, 3, 3);
            FunctionsTestHelper.AssertFunctionResult(mode, 2, 1, 2, 2, 3, 4);
            FunctionsTestHelper.AssertFunctionResult(mode, 3, 4, 3, 2, 2, 3);
            FunctionsTestHelper.AssertFunctionResult(mode, 0, -1, 5, 0, -2, 0);

            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NotAvailableError, 100);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NotAvailableError, 5, 10);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NotAvailableError, 1, 2, 3);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "1", "1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "1", 1);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, true);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, true, 2, 2);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, 50, 50, false);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "text", 0);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, 0, "text");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "text", "text");
        }

        [TestMethod]
        public void Stdev()
        {
            Stdev stdev = new Stdev();

            FunctionsTestHelper.AssertFunctionResult(stdev, 2.5, 1, 4, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0.707106781186548, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(stdev, 7.07106781186548, 10, 20);
            FunctionsTestHelper.AssertFunctionResult(stdev, 7.07106781186548, "10", "20");
            FunctionsTestHelper.AssertFunctionResult(stdev, 2.5, "1", "4", "5", "7");
            FunctionsTestHelper.AssertFunctionResult(stdev, 0, true, true);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0, false, false);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0.707106781186548, false, true);

            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.ValueError, "text", 1);
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.ValueError, "text", 1, 1);
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.ValueError, "text", 1, 1, 2);
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, 1);
        }

        [TestMethod]
        public void StdevP()
        {
            StdevP stdevp = new StdevP();

            FunctionsTestHelper.AssertFunctionResult(stdevp, 2.1650635094611, 1, 4, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0.5, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 5, 10, 20);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 5, "10", "20");
            FunctionsTestHelper.AssertFunctionResult(stdevp, 2.1650635094611, "1", "4", "5", "7");
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0, true, true);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0, false, false);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0.5, false, true);

            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, 1); 
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.ValueError, "text", 1);
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.ValueError, "text", 1, 1);
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.ValueError, "text", 1, 1, 2);
        }

        [TestMethod]
        public void StdevA()
        {
            StdevA stdeva = new StdevA();

            FunctionsTestHelper.AssertFunctionResult(stdeva, 2.5, 1, 4, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 7.07106781186548, 10, 20);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 7.07106781186548, "10", "20");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 2.5, "1", "4", "5", "7");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0, true, true);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0, false, false);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, false, true);

            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.ValueError, "text", 1);
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.ValueError, "text", 1, 1);
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.ValueError, "text", 1, 1, 2);
        }

        [TestMethod]
        public void StdevPA()
        {
            StdevPA stdevpa = new StdevPA();

            FunctionsTestHelper.AssertFunctionResult(stdevpa, 2.1650635094611, 1, 4, 5, 7);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, 1, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 5, 10, 20);

            FunctionsTestHelper.AssertFunctionResult(stdevpa, 5, "10", "20");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 2.1650635094611, "1", "4", "5", "7");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0, true, true);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0, false, false);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, false, true);

            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.ValueError, "text", 1);
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.ValueError, "text", 1, 1);
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.ValueError, "text", 1, 1, 2);
        }

        #region CountIf

        [TestMethod]
        public void CountIfWildCharactersCriteriaTests()
        {
            this.AssertCountIfCorrectnessTest("someText56~?~~7", new string[,] { { "someText56?~7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56?~7", new string[,] { { "someText56?7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56*~7", new string[,] { { "someText56?7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~7", new string[,] { { "someText56?7" } }, 0);
            this.AssertCountIfCorrectnessTest("someText56~?7", new string[,] { { "someText56?7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~??7", new string[,] { { "someText56?Q7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~??7~", new string[,] { { "someText56?Q7" } }, 1);
            this.AssertCountIfCorrectnessTest("~so~me~Text56~??7~", new string[,] { { "someText56?Q7" } }, 1);
            this.AssertCountIfCorrectnessTest("~so~me~Text56~??~7~", new string[,] { { "someText56?Q~7" } }, 0);
            this.AssertCountIfCorrectnessTest("~so~me~Text56~??~~7~", new string[,] { { "someText56?Q~7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~?*~~~7", new string[,] { { "someText56?Q~7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~?*7", new string[,] { { "someText56?Q~7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?7" } }, 1);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56?~7" } }, 0);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56Q7" } }, 0);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56~?7" } }, 0);
            this.AssertCountIfCorrectnessTest("someText56~?~7", new string[,] { { "someText56~?~7" } }, 0);
        }

        [TestMethod]
        public void CountIfComparissonOperatorsTests()
        {
            this.AssertCountIfCorrectnessTest(
                ">=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                3);

            this.AssertCountIfCorrectnessTest(
                "<=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "<=q1234",
                new string[,] { { "q1234", "q12345" }, { "q1235", "q1233" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "=\"=q1234\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "q1234",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "=\"=q123?\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q123*", "q123?" } },
                3);

            this.AssertCountIfCorrectnessTest(
                "=\"=q12?3\"",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                1);

            this.AssertCountIfCorrectnessTest(
                ">=q1234*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                1);

            this.AssertCountIfCorrectnessTest(
                ">=q*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                3);

            this.AssertCountIfCorrectnessTest(
                ">=q12?4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                3);

            this.AssertCountIfCorrectnessTest(
                ">=q12??4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                3);

            this.AssertCountIfCorrectnessTest(
                "",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                0);

            this.AssertCountIfCorrectnessTest(
                "<=q1234*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                3);

            this.AssertCountIfCorrectnessTest(
                "<=q*",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "<=q12?4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "<=q12??4",
                new string[,] { { "q1234", "=\"=q1234\"" }, { "q1235", "q1233" } },
                1);
        }

        [TestMethod]
        public void CountIfNumberCellValuesTests()
        {
            this.AssertCountIfCorrectnessTest(
                "<  3 ",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "<=3 ",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                3);

            this.AssertCountIfCorrectnessTest(
                ">3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                3);

            this.AssertCountIfCorrectnessTest(
                ">=3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                4);

            this.AssertCountIfCorrectnessTest(
                "=\"3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "=\"=3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "=\"<>3\"",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                99);

            this.AssertCountIfCorrectnessTest(
                "=3",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "=1",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                1);

            this.AssertCountIfCorrectnessTest(
                "=TRUE",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                0);

            this.AssertCountIfCorrectnessTest(
                "TRUE",
                new string[,] { { "1{ds}00".ReplaceCultureDependentSeparators(), "2" }, { "3", "4" }, { "5{ds}00".ReplaceCultureDependentSeparators(), "6" } },
                0);
        }

        [TestMethod]
        public void CountIfBooleanCellValueTests()
        {
            this.AssertCountIfCorrectnessTest(
                "<tRuE",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "<=true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                4);

            this.AssertCountIfCorrectnessTest(
                ">true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                0);

            this.AssertCountIfCorrectnessTest(
                ">=true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "=\"=true\"",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                2);

            this.AssertCountIfCorrectnessTest(
                "<>true",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                98);

            this.AssertCountIfCorrectnessTest(
                "<>False",
                new string[,] { { "TRUE", "FALSE" }, { "=TRUE", "=FALSE" }, { "=\"false\"", "=\"true\"" } },
                98);

            this.AssertCountIfCorrectnessTest(
                 "< true",
                 new string[,] { { "=\"TRUE\"", "=\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 1);

            this.AssertCountIfCorrectnessTest(
                 "< true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 1);

            this.AssertCountIfCorrectnessTest(
                 "> true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 4);

            this.AssertCountIfCorrectnessTest(
                 "<> true",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 99);

            this.AssertCountIfCorrectnessTest(
                 "=\" true\"",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 1);

            this.AssertCountIfCorrectnessTest(
                 "=\"= true\"",
                 new string[,] { { "\"TRUE\"", "\"FALSE\"" }, { "=\"TRUE\"", "=\"FALSE\"" }, { "=\" false\"", "=\" true\"" } },
                 1);
        }

        [TestMethod]
        public void CountIfErrorsInReferenceArgumentTest()
        {
            string errorValue = "=sum(\"\")";

            this.AssertCountIfCorrectnessTest(
                "=0",
                new string[,] { { errorValue, errorValue, "0" }, { errorValue, errorValue, "1" }, { "", errorValue, "0" } },
                2);
        }

        [TestMethod]
        public void CountIfMultupleCellRangesValueErrorNotPoppingUpReferenceErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=COUNTIF((A1{ls}A2){ls}0)", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void CountIfMultupleCellRangesValueErrorTest()
        {
            FunctionsTestHelper.AssertErrorCellValue("=COUNTIF((B1{ls}B2{ls}B3){ls}0)", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void CountIfErrorCriteriaCountingTest()
        {
            string errorValue = "=sum(\"\")";
            string stringErrorValue = ErrorExpressions.ValueError.Value;
            string stringValue = String.Format("=\"{0}\"", stringErrorValue);
            string refError = ErrorExpressions.ReferenceError.Value;
            string divError = ErrorExpressions.DivisionByZero.Value;

            this.AssertCountIfCorrectnessTest(
                errorValue,
                new string[,] { { errorValue, refError, divError }, { "0", "asdf", errorValue }, { stringValue, stringErrorValue, errorValue } },
                4);
        }

        [TestMethod]
        public void CountIfReferenceErrorVariationsTests()
        {
            FunctionsTestHelper.AssertFunctionReferenceErrorVariationsTests("Countif", 2, 2, "1");
        }

        private void AssertCountIfCorrectnessTest(string criteria, string[,] cellValues, double expectedCount)
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            int rangesMaxSize = 10;

            worksheet.Cells[0, 1].SetValue(criteria);

            for (int i = 0; i < cellValues.GetLength(0) && i < rangesMaxSize; i++)
            {
                for (int j = 0; j < cellValues.GetLength(1) && j < rangesMaxSize; j++)
                {
                    worksheet.Cells[i + 1, j + 1].SetValue(cellValues[i, j]);
                }
            }

            worksheet.Cells[0, 0].SetValue("=COUNTIF(B2:K11{ls}B1)".ReplaceCultureDependentSeparators());
            FormulaCellValue formula = worksheet.Cells[0, 0].GetValue().Value as FormulaCellValue;
            string cellValueOfA1 = formula.GetResultValueAsCellValue().RawValue;
            double actualCount = double.Parse(cellValueOfA1);

            Assert.AreEqual(expectedCount, actualCount);
        }

        #endregion
    }
}
