﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.FunctionsTests
{
    [TestClass]
    public class TextFunctionsTests
    {
        [TestMethod]
        public void CharFunctionCorrectnessTest()
        {
            CharFunction charFunction = new CharFunction();

            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)1).ToString(), 1);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)48).ToString(), 48);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)64).ToString(), 64);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)45).ToString(), 45);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)65).ToString(), 65);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)255).ToString(), 255);
        }

        [TestMethod]
        public void ExactFunctionCorrectnessTest()
        {
            Exact exact = new Exact();

            FunctionsTestHelper.AssertFunctionResult(exact, true, "", "");
            FunctionsTestHelper.AssertFunctionResult(exact, true, "string", "string");
            FunctionsTestHelper.AssertFunctionResult(exact, false, "string", "String");
            FunctionsTestHelper.AssertFunctionResult(exact, false, " string", "string");
        }

        [TestMethod]
        public void FindFunctionCorrectnessTest()
        {
            Find find = new Find();

            FunctionsTestHelper.AssertFunctionResult(find, 1, "", "");
            FunctionsTestHelper.AssertFunctionResult(find, 1, "qwe", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(find, 2, "wer", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(find, 3, "ert", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(find, 4, "rty", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(find, 1, "M", "Miriam McGovern");
            FunctionsTestHelper.AssertFunctionResult(find, 6, "m", "Miriam McGovern");
            FunctionsTestHelper.AssertFunctionResult(find, 4, "i", "string");

            FunctionsTestHelper.AssertFunctionResult(find, 6, "m", "Miriam McGovern", 3);
            FunctionsTestHelper.AssertFunctionResult(find, 1, "M", "Miriam McGovern", 1);
            FunctionsTestHelper.AssertFunctionResult(find, 2, "i", "Miriam McGovern", 1);
            FunctionsTestHelper.AssertFunctionResult(find, 4, "i", "Miriam McGovern", 3);
            FunctionsTestHelper.AssertFunctionResult(find, 4, "i", "Miriam McGovern", 4);
        }

        [TestMethod]
        public void LeftFunctionCorrectnessTest()
        {
            Left left = new Left();

            FunctionsTestHelper.AssertFunctionResult(left, "", "");
            FunctionsTestHelper.AssertFunctionResult(left, "q", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(left, "s", "string");
            FunctionsTestHelper.AssertFunctionResult(left, "", "", 10);
            FunctionsTestHelper.AssertFunctionResult(left, "", "qwerty", 0);
            FunctionsTestHelper.AssertFunctionResult(left, "", "", 0);
            FunctionsTestHelper.AssertFunctionResult(left, "qw", "qwerty", 2);
            FunctionsTestHelper.AssertFunctionResult(left, "qwerty", "qwerty", 10);
            FunctionsTestHelper.AssertFunctionResult(left, "string", "string", 10);
        }

        [TestMethod]
        public void LowerFunctionCorrectnessTest()
        {
            Lower lower = new Lower();

            FunctionsTestHelper.AssertFunctionResult(lower, "", "");
            FunctionsTestHelper.AssertFunctionResult(lower, "e. e. cummings", "E. E. Cummings");
            FunctionsTestHelper.AssertFunctionResult(lower, "apt. 2b", "Apt. 2B");
        }

        [TestMethod]
        public void ReptFunctionCorrectnessTest()
        {
            Rept rept = new Rept();

            FunctionsTestHelper.AssertFunctionResult(rept, "", "a", 0);
            FunctionsTestHelper.AssertFunctionResult(rept, "", "", 0);
            FunctionsTestHelper.AssertFunctionResult(rept, "a", "a", 1);
            FunctionsTestHelper.AssertFunctionResult(rept, "aa", "a", 2);
            FunctionsTestHelper.AssertFunctionResult(rept, "aaaaa", "a", 5);
            FunctionsTestHelper.AssertFunctionResult(rept, "", "*-", 0);
            FunctionsTestHelper.AssertFunctionResult(rept, "*-", "*-", 1);
            FunctionsTestHelper.AssertFunctionResult(rept, "*-*-", "*-", 2);
            FunctionsTestHelper.AssertFunctionResult(rept, "*-*-*-", "*-", 3);
            FunctionsTestHelper.AssertFunctionResult(rept, "111", 1, 3);
        }

        [TestMethod]
        public void RightFunctionCorrectnessTest()
        {
            Right right = new Right();

            FunctionsTestHelper.AssertFunctionResult(right, "", "");
            FunctionsTestHelper.AssertFunctionResult(right, "y", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(right, "g", "string");

            FunctionsTestHelper.AssertFunctionResult(right, "", "", 10);
            FunctionsTestHelper.AssertFunctionResult(right, "", "qwerty", 0);
            FunctionsTestHelper.AssertFunctionResult(right, "", "", 0);
            FunctionsTestHelper.AssertFunctionResult(right, "ty", "qwerty", 2);
            FunctionsTestHelper.AssertFunctionResult(right, "qwerty", "qwerty", 10);
            FunctionsTestHelper.AssertFunctionResult(right, "string", "string", 10);
        }

        [TestMethod]
        public void SearchFunctionCorrectnessTest()
        {
            Search search = new Search();

            FunctionsTestHelper.AssertFunctionResult(search, 1, "", "");
            FunctionsTestHelper.AssertFunctionResult(search, 1, "qwe", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(search, 2, "wer", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(search, 3, "ert", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(search, 4, "rty", "qwerty");
            FunctionsTestHelper.AssertFunctionResult(search, 1, "M", "Miriam McGovern");
            FunctionsTestHelper.AssertFunctionResult(search, 1, "m", "Miriam McGovern");
            FunctionsTestHelper.AssertFunctionResult(search, 4, "i", "string");
            
            FunctionsTestHelper.AssertFunctionResult(search, 6, "m", "Miriam McGovern", 3);
            FunctionsTestHelper.AssertFunctionResult(search, 1, "M", "Miriam McGovern", 1);
            FunctionsTestHelper.AssertFunctionResult(search, 8, "m", "Miriam McGovern", 7);
            FunctionsTestHelper.AssertFunctionResult(search, 2, "i", "Miriam McGovern", 1);
            FunctionsTestHelper.AssertFunctionResult(search, 4, "i", "Miriam McGovern", 3);
            FunctionsTestHelper.AssertFunctionResult(search, 4, "i", "Miriam McGovern", 4);
        }

        [TestMethod]
        public void UpperFunctionCorrectnessTest()
        {
            Upper upper = new Upper();

            FunctionsTestHelper.AssertFunctionResult(upper, "", "");
            FunctionsTestHelper.AssertFunctionResult(upper, "E. E. CUMMINGS", "E. E. Cummings");
            FunctionsTestHelper.AssertFunctionResult(upper, "APT. 2B", "Apt. 2B");
        }

        [TestMethod]
        public void TrimFunctionCorrectnessTest()
        {
            Trim trim = new Trim();
            FunctionsTestHelper.AssertFunctionResult(trim, "", "");
            FunctionsTestHelper.AssertFunctionResult(trim, "E. E. Cummings", "    E. E.     Cummings");
            FunctionsTestHelper.AssertFunctionResult(trim, "Apt. 2B", "Apt. 2B     ");
            FunctionsTestHelper.AssertFunctionResult(trim, "33 33 33", "   33    33 33        ");
        }

        [TestMethod]
        public void TrimFunctionCorrectnessWithNonBreakingSpacesTest()
        {
            Trim trim = new Trim();

            FunctionsTestHelper.AssertFunctionResult(trim, "test   test", "test   test");
            FunctionsTestHelper.AssertFunctionResult(trim, "     test    test     ", "     test    test     ");
        }
    }
}
