﻿using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class AlignmentAndIndentTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            cells[0, 0].SetVerticalAlignment(RadVerticalAlignment.Top);
            cells[0, 1].SetVerticalAlignment(RadVerticalAlignment.Center);
            cells[0, 2].SetVerticalAlignment(RadVerticalAlignment.Bottom);

            cells[1, 0].SetHorizontalAlignment(RadHorizontalAlignment.Left);
            cells[1, 1].SetHorizontalAlignment(RadHorizontalAlignment.Left);
            cells[1, 2].SetHorizontalAlignment(RadHorizontalAlignment.Center);
            cells[1, 3].SetHorizontalAlignment(RadHorizontalAlignment.Right);
            cells[1, 4].SetHorizontalAlignment(RadHorizontalAlignment.Right);

            cells[1, 1].SetIndent(2);
            cells[1, 3].SetIndent(2);

            cells[2, 0].SetIsWrapped(true);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            Assert.AreEqual(RadVerticalAlignment.Top, cells[0, 0].GetVerticalAlignment().Value);
            Assert.AreEqual(RadVerticalAlignment.Center, cells[0, 1].GetVerticalAlignment().Value);
            Assert.AreEqual(RadVerticalAlignment.Bottom, cells[0, 2].GetVerticalAlignment().Value);

            Assert.AreEqual(RadHorizontalAlignment.Left, cells[1, 0].GetHorizontalAlignment().Value);
            Assert.AreEqual(RadHorizontalAlignment.Left, cells[1, 1].GetHorizontalAlignment().Value);
            Assert.AreEqual(2, cells[1, 1].GetIndent().Value);

            Assert.AreEqual(RadHorizontalAlignment.Center, cells[1, 2].GetHorizontalAlignment().Value);

            Assert.AreEqual(RadHorizontalAlignment.Right, cells[1, 3].GetHorizontalAlignment().Value);
            Assert.AreEqual(2, cells[1, 3].GetIndent().Value);
            
            Assert.AreEqual(RadHorizontalAlignment.Right, cells[1, 4].GetHorizontalAlignment().Value);

            Assert.AreEqual(true, cells[2, 0].GetIsWrapped().Value);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportAlignmentAndIndentTest()
        {
            ImportFileAndAssert("ImportAlignmentAndIndentDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportAlignmentAndIndentTest()
        {
            ArrangeWorkbookExportImportAssert();
        }        
    }
}
