﻿using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Theming;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class BordersTests : ImportExportTestClassBase
    {
        #region Fields

        private static readonly CellBorder borderAccent3 = new CellBorder(CellBorderStyle.Thick, 
            new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade4));

        private static readonly CellBorder borderAccent2 = new CellBorder(CellBorderStyle.Thick,
            new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade4));

        private static readonly CellBorders borders1 = new CellBorders(
            new CellBorder(CellBorderStyle.Dotted, new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.Hair, new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.DashDotDot, new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.DashDot, new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade4)),
            new CellBorder(CellBorderStyle.Thin, new ThemableColor(ThemeColorType.Accent1, ColorShadeType.Shade4)),
            new CellBorder(CellBorderStyle.Dashed, new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade4)),
            new CellBorder(CellBorderStyle.Thin, new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade4)),
            new CellBorder(CellBorderStyle.Thin, new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade4)));

        private static readonly CellBorders borders2 = new CellBorders(
            new CellBorder(CellBorderStyle.SlantDashDot, new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.MediumDashDotDot, new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.MediumDashed, new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.MediumDashDot, new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.Thick, new ThemableColor(ThemeColorType.Accent1, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.Medium, new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.Double, new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade3)),
            new CellBorder(CellBorderStyle.Double, new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade3)));

        private static readonly CellBorders borders3 = new CellBorders(borderAccent3, borderAccent3, borderAccent3, borderAccent3, borderAccent3, borderAccent3,
            CellBorder.Default, CellBorder.Default);

        private static readonly CellBorders borders4 = new CellBorders(borderAccent2, borderAccent2, borderAccent2, borderAccent2, borderAccent2, borderAccent2, 
            CellBorder.Default, CellBorder.Default);

        #endregion


        #region Methods

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            worksheet.Cells[2, 1, 6, 4].SetBorders(borders1);
            worksheet.Cells[2, 7, 6, 11].SetBorders(borders2);
            worksheet.Cells[10, 1, 14, 4].SetBorders(borders3);
            worksheet.Cells[10, 5, 14, 9].SetBorders(borders4);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            CellBorders borders = worksheet.Cells[2, 1, 6, 4].GetBorders();
            Assert.AreEqual(borders1, borders);

            borders = worksheet.Cells[2, 7, 6, 11].GetBorders();
            Assert.AreEqual(borders2, borders); 

            borders = worksheet.Cells[10, 1, 14, 4].GetBorders();
            CellBorders expectedBorders = new CellBorders(borderAccent3, borderAccent3, borderAccent2,
                borderAccent3, borderAccent3, borderAccent3, CellBorder.Default, CellBorder.Default);

            Assert.AreEqual(expectedBorders, borders); 

            borders = worksheet.Cells[10, 5, 14, 9].GetBorders();
            Assert.AreEqual(borders4, borders); 
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportBordersTest()
        {
            ImportFileAndAssert("ImportBordersTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportBordersTest()
        {
            ArrangeWorkbookExportImportAssert();
        }

        #endregion
    }
}