﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.ChartsheetTests
{
    [TestClass]
    public class ImportWorkbookContainingChartsheetTests
    {
        private const string ChartsheetTestsFolder = "Chartsheet";

        [TestMethod]
        public void Workbook_WhenContainingOnlyChartsheet_IsImported()
        {
            // Arrange & Act
            Workbook workbook = GetWorkbookFromFile("ImportWorkbookContainingOnlyChartsheet");

            // Assert
            Assert.AreEqual(0, workbook.Sheets.Count);
        }

        [TestMethod]
        public void Workbook_WhenContainingChartsheetAndWorksheet_OnlyWorksheetIsImported()
        {
            // Arrange & Act
            Workbook workbook = GetWorkbookFromFile("ImportWorkbookContainingChartsheetAndWorksheet");
            Worksheet worksheet = workbook.Worksheets[0];

            // Assert
            Assert.AreEqual(1, workbook.Sheets.Count);
            Assert.AreEqual("test", worksheet.Cells[0, 0].GetValue().Value.RawValue);
        }

        [TestMethod]
        public void Workbook_WhenContainingChartsheetAndWorksheetWithImages_OnlyWorksheetIsImported()
        {
            // Arrange & Act
            Workbook workbook = GetWorkbookFromFile("ImportWorkbookContainingChartsheetAndWorksheetWithImage");
            Worksheet worksheet = workbook.Worksheets[0];

            // Assert
            Assert.AreEqual(1, workbook.Sheets.Count);
            Assert.AreEqual(1, worksheet.Shapes.Count);
        }

        private static Workbook GetWorkbookFromFile(string fileName)
        {
            return TestHelper.GetWorkbookFromFile(string.Format("{0}/{1}.xlsx", ChartsheetTestsFolder, fileName));
        }
    }
}