﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.DataValidationTests
{
    [TestClass]
    public class DataValidationListAndCustomRuleImportExportTest : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            workbook.Worksheets.Add();
            workbook.Worksheets.Add();

            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(0, 0);
            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = "1,2,3,4,5,6,7";
            context.InCellDropdown = true;
            context.IgnoreBlank = true;
            ListDataValidationRule rule = new ListDataValidationRule(context);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule);


            CellIndex cellIndex1 = new CellIndex(0, 1);
            ListDataValidationRuleContext context1 = new ListDataValidationRuleContext(worksheet, cellIndex1);
            context1.Argument1 = "=$C$1:$C$8";
            context1.InCellDropdown = false;
            context1.IgnoreBlank = true;
            ListDataValidationRule rule1 = new ListDataValidationRule(context1);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule1);


            CellIndex cellIndex3 = new CellIndex(0, 3);
            SingleArgumentDataValidationRuleContext context3 = new SingleArgumentDataValidationRuleContext(worksheet, cellIndex3);
            context3.Argument1 = "=D1=SUM(E1,E2)";
            context3.IgnoreBlank = true;
            CustomDataValidationRule rule3 = new CustomDataValidationRule(context3);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule3);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(0, 0);
            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = "1,2,3,4,5,6,7";
            context.InCellDropdown = true;
            context.IgnoreBlank = true;
            ListDataValidationRule expectedRule = new ListDataValidationRule(context);

            IDataValidationRule rule = worksheet.Cells[cellIndex].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule, rule);


            CellIndex cellIndex1 = new CellIndex(0, 1);
            ListDataValidationRuleContext context1 = new ListDataValidationRuleContext(worksheet, cellIndex1);
            context1.Argument1 = "=$C$1:$C$8";
            context1.InCellDropdown = false;
            context1.IgnoreBlank = true;
            ListDataValidationRule expectedRule1 = new ListDataValidationRule(context1);

            IDataValidationRule rule1 = worksheet.Cells[cellIndex1].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule1, rule1);


            CellIndex cellIndex3 = new CellIndex(0, 3);
            SingleArgumentDataValidationRuleContext context3 = new SingleArgumentDataValidationRuleContext(worksheet, cellIndex3);
            context3.Argument1 = "=D1=SUM(E1,E2)";
            context3.IgnoreBlank = true;
            CustomDataValidationRule expectedRule3 = new CustomDataValidationRule(context3);

            IDataValidationRule rule3 = worksheet.Cells[cellIndex3].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule3, rule3);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ImportListAndCustomDataValidationRuleTest()
        {
            base.ImportFileAndAssert("DataValidation\\ListAndCustomDataValidationRuleIDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ExportAndImportListAndCustomDataValidationRuleTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}