﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.DataValidationTests
{
    [TestClass]
    public class DataValidationMissingArgumentsDataValidationRulesImportExportTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            workbook.Worksheets.Add();
            workbook.Worksheets.Add();

            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(1, 0);
            NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
            context.ComparisonOperator = ComparisonOperator.Between;
            context.IgnoreBlank = true;
            WholeNumberDataValidationRule rule = new WholeNumberDataValidationRule(context);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule);

            CellIndex cellIndex1 = new CellIndex(1, 1);
            NumberDataValidationRuleContext context1 = new NumberDataValidationRuleContext(worksheet, cellIndex1);
            context1.ComparisonOperator = ComparisonOperator.Between;
            context1.IgnoreBlank = true;
            DecimalDataValidationRule rule1 = new DecimalDataValidationRule(context1);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule1);

            CellIndex cellIndex2 = new CellIndex(1, 2);
            NumberDataValidationRuleContext context2 = new NumberDataValidationRuleContext(worksheet, cellIndex2);
            context2.ComparisonOperator = ComparisonOperator.Between;
            context2.IgnoreBlank = true;
            DateDataValidationRule rule2 = new DateDataValidationRule(context2);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule2);

            CellIndex cellIndex3 = new CellIndex(1, 3);
            NumberDataValidationRuleContext context3 = new NumberDataValidationRuleContext(worksheet, cellIndex3);
            context3.ComparisonOperator = ComparisonOperator.Between;
            context3.IgnoreBlank = true;
            TimeDataValidationRule rule3 = new TimeDataValidationRule(context3);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule3);

            CellIndex cellIndex4 = new CellIndex(1, 4);
            NumberDataValidationRuleContext context4 = new NumberDataValidationRuleContext(worksheet, cellIndex4);
            context4.ComparisonOperator = ComparisonOperator.Between;
            context4.IgnoreBlank = true;
            TextLengthDataValidationRule rule4 = new TextLengthDataValidationRule(context4);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule4);

            CellIndex cellIndex5 = new CellIndex(1, 5);
            SingleArgumentDataValidationRuleContext context5 = new SingleArgumentDataValidationRuleContext(worksheet, cellIndex5);
            context5.IgnoreBlank = true;
            CustomDataValidationRule rule5 = new CustomDataValidationRule(context5);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule5);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(1, 0);
            NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
            context.ComparisonOperator = ComparisonOperator.Between;
            context.IgnoreBlank = true;
            WholeNumberDataValidationRule expectedRule = new WholeNumberDataValidationRule(context);

            IDataValidationRule rule = worksheet.Cells[cellIndex].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule, rule);

            CellIndex cellIndex1 = new CellIndex(1, 1);
            NumberDataValidationRuleContext context1 = new NumberDataValidationRuleContext(worksheet, cellIndex1);
            context1.ComparisonOperator = ComparisonOperator.Between;
            context1.IgnoreBlank = true;
            DecimalDataValidationRule expectedRule1 = new DecimalDataValidationRule(context1);

            IDataValidationRule rule1 = worksheet.Cells[cellIndex1].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule1, rule1);

            CellIndex cellIndex2 = new CellIndex(1, 2);
            NumberDataValidationRuleContext context2 = new NumberDataValidationRuleContext(worksheet, cellIndex2);
            context2.ComparisonOperator = ComparisonOperator.Between;
            context2.IgnoreBlank = true;
            DateDataValidationRule expectedRule2 = new DateDataValidationRule(context2);

            IDataValidationRule rule2 = worksheet.Cells[cellIndex2].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule2, rule2);

            CellIndex cellIndex3 = new CellIndex(1, 3);
            NumberDataValidationRuleContext context3 = new NumberDataValidationRuleContext(worksheet, cellIndex3);
            context3.ComparisonOperator = ComparisonOperator.Between;
            context3.IgnoreBlank = true;
            TimeDataValidationRule expectedRule3 = new TimeDataValidationRule(context3);

            IDataValidationRule rule3 = worksheet.Cells[cellIndex3].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule3, rule3);

            CellIndex cellIndex4 = new CellIndex(1, 4);
            NumberDataValidationRuleContext context4 = new NumberDataValidationRuleContext(worksheet, cellIndex4);
            context4.ComparisonOperator = ComparisonOperator.Between;
            context4.IgnoreBlank = true;
            TextLengthDataValidationRule expectedRule4 = new TextLengthDataValidationRule(context4);

            IDataValidationRule rule4 = worksheet.Cells[cellIndex4].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule4, rule4);

            CellIndex cellIndex5 = new CellIndex(1, 5);
            SingleArgumentDataValidationRuleContext context5 = new SingleArgumentDataValidationRuleContext(worksheet, cellIndex5);
            context5.IgnoreBlank = true;
            CustomDataValidationRule expectedRule5 = new CustomDataValidationRule(context5);

            IDataValidationRule rule5 = worksheet.Cells[cellIndex5].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule5, rule5);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ImportDateDataValidationRuleTest()
        {
            base.ImportFileAndAssert("DataValidation\\MissingArgumentsDataValidationRules.xlsx");
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ExportAndImportDateDataValidationRuleTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
