﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.DataValidationTests
{
    [TestClass]
    public class DataValidationRulesInputAndErrorMessageImportExportTest : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            AnyValueDataValidationRuleContext context = new AnyValueDataValidationRuleContext();
            context.ShowInputMessage = true;
            context.InputMessageTitle = "InputMessageTitle";
            context.InputMessageContent = "InputMessageContent";
            AnyValueDataValidationRule rule = new AnyValueDataValidationRule(context);

            worksheet.Cells[0, 0].SetDataValidationRule(rule);


            AnyValueDataValidationRuleContext context1 = new AnyValueDataValidationRuleContext();
            context1.ShowErrorMessage = true;
            context1.ErrorStyle = ErrorStyle.Stop;
            context1.ErrorAlertTitle = "StopTitle";
            context1.ErrorAlertContent = "StopContent";
            AnyValueDataValidationRule rule1 = new AnyValueDataValidationRule(context1);

            worksheet.Cells[0, 1].SetDataValidationRule(rule1);


            AnyValueDataValidationRuleContext context2 = new AnyValueDataValidationRuleContext();
            context2.ShowErrorMessage = true;
            context2.ErrorStyle = ErrorStyle.Warning;
            context2.ErrorAlertTitle = "WarningTitle";
            context2.ErrorAlertContent = "WarningContent";
            AnyValueDataValidationRule rule2 = new AnyValueDataValidationRule(context2);

            worksheet.Cells[0, 2].SetDataValidationRule(rule2);


            AnyValueDataValidationRuleContext context3 = new AnyValueDataValidationRuleContext();
            context3.ShowErrorMessage = true;
            context3.ErrorStyle = ErrorStyle.Information;
            context3.ErrorAlertTitle = "InformationTitle";
            context3.ErrorAlertContent = "InformationContent";
            AnyValueDataValidationRule rule3 = new AnyValueDataValidationRule(context3);

            worksheet.Cells[0, 3].SetDataValidationRule(rule3);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets[0];

            AnyValueDataValidationRuleContext context = new AnyValueDataValidationRuleContext();
            context.ShowInputMessage = true;
            context.InputMessageTitle = "InputMessageTitle";
            context.InputMessageContent = "InputMessageContent";
            AnyValueDataValidationRule rule = new AnyValueDataValidationRule(context);

            IDataValidationRule expectedRule = worksheet.Cells[0, 0].GetDataValidationRule().Value;

            Assert.AreEqual(rule, expectedRule);


            AnyValueDataValidationRuleContext context1 = new AnyValueDataValidationRuleContext();
            context1.ShowErrorMessage = true;
            context1.ErrorStyle = ErrorStyle.Stop;
            context1.ErrorAlertTitle = "StopTitle";
            context1.ErrorAlertContent = "StopContent";
            AnyValueDataValidationRule rule1 = new AnyValueDataValidationRule(context1);

            IDataValidationRule expectedRule1 = worksheet.Cells[0, 1].GetDataValidationRule().Value;

            Assert.AreEqual(rule1, expectedRule1);


            AnyValueDataValidationRuleContext context2 = new AnyValueDataValidationRuleContext();
            context2.ShowErrorMessage = true;
            context2.ErrorStyle = ErrorStyle.Warning;
            context2.ErrorAlertTitle = "WarningTitle";
            context2.ErrorAlertContent = "WarningContent";
            AnyValueDataValidationRule rule2 = new AnyValueDataValidationRule(context2);

            IDataValidationRule expectedRule2 = worksheet.Cells[0, 2].GetDataValidationRule().Value;

            Assert.AreEqual(rule2, expectedRule2);


            AnyValueDataValidationRuleContext context3 = new AnyValueDataValidationRuleContext();
            context3.ShowErrorMessage = true;
            context3.ErrorStyle = ErrorStyle.Information;
            context3.ErrorAlertTitle = "InformationTitle";
            context3.ErrorAlertContent = "InformationContent";
            AnyValueDataValidationRule rule3 = new AnyValueDataValidationRule(context3);

            IDataValidationRule expectedRule3 = worksheet.Cells[0, 3].GetDataValidationRule().Value;

            Assert.AreEqual(rule3, expectedRule3);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ImportInputAndErrorMessageFromDataValidationRuleTest()
        {
            base.ImportFileAndAssert("DataValidation\\ImportExportDataValidationRulesInputAndErrorMessageDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ExportAndImportInputAndErrorMessageFromDataValidationRuleTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}