﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.DataValidationTests
{
    [TestClass]
    public class DataValidationTimeRuleImportExportTest : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            workbook.Worksheets.Add();
            workbook.Worksheets.Add();

            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(0, 0);
            NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = "12:00";
            context.Argument2 = "15:00";
            context.ComparisonOperator = ComparisonOperator.Between;
            context.IgnoreBlank = true;
            TimeDataValidationRule rule = new TimeDataValidationRule(context);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule);


            CellIndex cellIndex1 = new CellIndex(0, 1);
            NumberDataValidationRuleContext context1 = new NumberDataValidationRuleContext(worksheet, cellIndex1);
            context1.Argument1 = "12:00";
            context1.Argument2 = "15:00";
            context1.ComparisonOperator = ComparisonOperator.NotBetween;
            context1.IgnoreBlank = false;
            TimeDataValidationRule rule1 = new TimeDataValidationRule(context1);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule1);


            CellIndex cellIndex2 = new CellIndex(0, 2);
            NumberDataValidationRuleContext context2 = new NumberDataValidationRuleContext(worksheet, cellIndex2);
            context2.Argument1 = "12:00";
            context2.Argument2 = "15:00";
            context2.ComparisonOperator = ComparisonOperator.NotBetween;
            context2.IgnoreBlank = true;
            TimeDataValidationRule rule2 = new TimeDataValidationRule(context2);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule2);


            CellIndex cellIndex3 = new CellIndex(0, 3);
            NumberDataValidationRuleContext context3 = new NumberDataValidationRuleContext(worksheet, cellIndex3);
            context3.Argument1 = "12:00";
            context3.Argument2 = "";
            context3.ComparisonOperator = ComparisonOperator.EqualsTo;
            context3.IgnoreBlank = true;
            TimeDataValidationRule rule3 = new TimeDataValidationRule(context3);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule3);


            CellIndex cellIndex4 = new CellIndex(0, 4);
            NumberDataValidationRuleContext context4 = new NumberDataValidationRuleContext(worksheet, cellIndex4);
            context4.Argument1 = "12:00";
            context4.Argument2 = "";
            context4.ComparisonOperator = ComparisonOperator.NotEqualsTo;
            context4.IgnoreBlank = true;
            TimeDataValidationRule rule4 = new TimeDataValidationRule(context4);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule4);


            CellIndex cellIndex5 = new CellIndex(0, 5);
            NumberDataValidationRuleContext context5 = new NumberDataValidationRuleContext(worksheet, cellIndex5);
            context5.Argument1 = "12:00";
            context5.Argument2 = "";
            context5.ComparisonOperator = ComparisonOperator.GreaterThan;
            context5.IgnoreBlank = true;
            TimeDataValidationRule rule5 = new TimeDataValidationRule(context5);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule5);


            CellIndex cellIndex6 = new CellIndex(0, 6);
            NumberDataValidationRuleContext context6 = new NumberDataValidationRuleContext(worksheet, cellIndex6);
            context6.Argument1 = "12:00";
            context6.Argument2 = "";
            context6.ComparisonOperator = ComparisonOperator.LessThan;
            context6.IgnoreBlank = true;
            TimeDataValidationRule rule6 = new TimeDataValidationRule(context6);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule6);


            CellIndex cellIndex7 = new CellIndex(0, 7);
            NumberDataValidationRuleContext context7 = new NumberDataValidationRuleContext(worksheet, cellIndex7);
            context7.Argument1 = "12:00";
            context7.Argument2 = "";
            context7.ComparisonOperator = ComparisonOperator.GreaterThanOrEqualsTo;
            context7.IgnoreBlank = true;
            TimeDataValidationRule rule7 = new TimeDataValidationRule(context7);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule7);


            CellIndex cellIndex8 = new CellIndex(0, 8);
            NumberDataValidationRuleContext context8 = new NumberDataValidationRuleContext(worksheet, cellIndex8);
            context8.Argument1 = "12:00";
            context8.Argument2 = "";
            context8.ComparisonOperator = ComparisonOperator.LessThanOrEqualsTo;
            context8.IgnoreBlank = true;
            TimeDataValidationRule rule8 = new TimeDataValidationRule(context8);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule8);


            CellIndex cellIndex9 = new CellIndex(0, 9);
            NumberDataValidationRuleContext context9 = new NumberDataValidationRuleContext(worksheet, cellIndex9);
            context9.Argument1 = "=Sheet2!A1";
            context9.Argument2 = "=Sheet2!A2";
            context9.ComparisonOperator = ComparisonOperator.Between;
            context9.IgnoreBlank = true;
            TimeDataValidationRule rule9 = new TimeDataValidationRule(context9);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule9);

            workbook.Names.Add("test", "=Sheet2!$A$1", 0, 0, string.Empty);
            CellIndex cellIndex10 = new CellIndex(0, 10);
            NumberDataValidationRuleContext context10 = new NumberDataValidationRuleContext(worksheet, cellIndex10);
            context10.Argument1 = "=test";
            context10.Argument2 = "=Sheet2!A2";
            context10.ComparisonOperator = ComparisonOperator.Between;
            context10.IgnoreBlank = true;
            TimeDataValidationRule rule10 = new TimeDataValidationRule(context10);

            worksheet.Cells[cellIndex].SetDataValidationRule(rule10);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets[0];

            CellIndex cellIndex = new CellIndex(0, 0);
            NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = "12:00";
            context.Argument2 = "15:00";
            context.ComparisonOperator = ComparisonOperator.Between;
            context.IgnoreBlank = true;
            TimeDataValidationRule expectedRule = new TimeDataValidationRule(context);

            IDataValidationRule rule = worksheet.Cells[cellIndex].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule, rule);


            CellIndex cellIndex1 = new CellIndex(0, 1);
            NumberDataValidationRuleContext context1 = new NumberDataValidationRuleContext(worksheet, cellIndex1);
            context1.Argument1 = "12:00";
            context1.Argument2 = "15:00";
            context1.ComparisonOperator = ComparisonOperator.NotBetween;
            context1.IgnoreBlank = false;
            TimeDataValidationRule expectedRule1 = new TimeDataValidationRule(context1);

            IDataValidationRule rule1 = worksheet.Cells[cellIndex1].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule1, rule1);


            CellIndex cellIndex2 = new CellIndex(0, 2);
            NumberDataValidationRuleContext context2 = new NumberDataValidationRuleContext(worksheet, cellIndex2);
            context2.Argument1 = "=A2";
            context2.Argument2 = "=A3";
            context2.ComparisonOperator = ComparisonOperator.NotBetween;
            context2.IgnoreBlank = true;
            TimeDataValidationRule expectedRule2 = new TimeDataValidationRule(context2);

            IDataValidationRule rule2 = worksheet.Cells[cellIndex2].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule2, rule2);


            CellIndex cellIndex3 = new CellIndex(0, 3);
            NumberDataValidationRuleContext context3 = new NumberDataValidationRuleContext(worksheet, cellIndex3);
            context3.Argument1 = "12:00";
            context3.Argument2 = "";
            context3.ComparisonOperator = ComparisonOperator.EqualsTo;
            context3.IgnoreBlank = true;
            TimeDataValidationRule expectedRule3 = new TimeDataValidationRule(context3);

            IDataValidationRule rule3 = worksheet.Cells[cellIndex3].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule3, rule3);


            CellIndex cellIndex4 = new CellIndex(0, 4);
            NumberDataValidationRuleContext context4 = new NumberDataValidationRuleContext(worksheet, cellIndex4);
            context4.Argument1 = "12:00";
            context4.Argument2 = "";
            context4.ComparisonOperator = ComparisonOperator.NotEqualsTo;
            context4.IgnoreBlank = true;
            TimeDataValidationRule expectedRule4 = new TimeDataValidationRule(context4);

            IDataValidationRule rule4 = worksheet.Cells[cellIndex4].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule4, rule4);


            CellIndex cellIndex5 = new CellIndex(0, 5);
            NumberDataValidationRuleContext context5 = new NumberDataValidationRuleContext(worksheet, cellIndex5);
            context5.Argument1 = "12:00";
            context5.Argument2 = "";
            context5.ComparisonOperator = ComparisonOperator.GreaterThan;
            context5.IgnoreBlank = true;
            TimeDataValidationRule expectedRule5 = new TimeDataValidationRule(context5);

            IDataValidationRule rule5 = worksheet.Cells[cellIndex5].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule5, rule5);


            CellIndex cellIndex6 = new CellIndex(0, 6);
            NumberDataValidationRuleContext context6 = new NumberDataValidationRuleContext(worksheet, cellIndex6);
            context6.Argument1 = "12:00";
            context6.Argument2 = "";
            context6.ComparisonOperator = ComparisonOperator.LessThan;
            context6.IgnoreBlank = true;
            TimeDataValidationRule expectedRule6 = new TimeDataValidationRule(context6);

            IDataValidationRule rule6 = worksheet.Cells[cellIndex6].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule6, rule6);


            CellIndex cellIndex7 = new CellIndex(0, 7);
            NumberDataValidationRuleContext context7 = new NumberDataValidationRuleContext(worksheet, cellIndex7);
            context7.Argument1 = "12:00";
            context7.Argument2 = "";
            context7.ComparisonOperator = ComparisonOperator.GreaterThanOrEqualsTo;
            context7.IgnoreBlank = true;
            TimeDataValidationRule expectedRule7 = new TimeDataValidationRule(context7);

            IDataValidationRule rule7 = worksheet.Cells[cellIndex7].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule7, rule7);


            CellIndex cellIndex8 = new CellIndex(0, 8);
            NumberDataValidationRuleContext context8 = new NumberDataValidationRuleContext(worksheet, cellIndex8);
            context8.Argument1 = "12:00";
            context8.Argument2 = "";
            context8.ComparisonOperator = ComparisonOperator.LessThanOrEqualsTo;
            context8.IgnoreBlank = true;
            TimeDataValidationRule expectedRule8 = new TimeDataValidationRule(context8);

            IDataValidationRule rule8 = worksheet.Cells[cellIndex8].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule8, rule8);


            CellIndex cellIndex9 = new CellIndex(0, 9);
            NumberDataValidationRuleContext context9 = new NumberDataValidationRuleContext(worksheet, cellIndex9);
            context9.Argument1 = "=Sheet2!A1";
            context9.Argument2 = "=Sheet2!A2";
            context9.ComparisonOperator = ComparisonOperator.Between;
            context9.IgnoreBlank = true;
            TimeDataValidationRule expectedRule9 = new TimeDataValidationRule(context9);

            IDataValidationRule rule9 = worksheet.Cells[cellIndex9].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule9, rule9);

            CellIndex cellIndex10 = new CellIndex(0, 10);
            NumberDataValidationRuleContext context10 = new NumberDataValidationRuleContext(worksheet, cellIndex10);
            context10.Argument1 = "=test";
            context10.Argument2 = "=Sheet2!A2";
            context10.ComparisonOperator = ComparisonOperator.Between;
            context10.IgnoreBlank = true;
            TimeDataValidationRule expectedRule10 = new TimeDataValidationRule(context10);

            IDataValidationRule rule10 = worksheet.Cells[cellIndex10].GetDataValidationRule().Value;

            Assert.AreEqual(expectedRule10, rule10);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ImportTimeDataValidationRuleTest()
        {
            base.ImportFileAndAssert("DataValidation\\ImportExportTimeRuleDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        [TestCategory("ImportExport")]
        public void ExportAndImportTimeDataValidationRuleTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}