﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Theming;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class FillsTests : ImportExportTestClassBase
    {
        #region Fields

        private static readonly PatternFill[] patternFills = new PatternFill[] { 
            
            new PatternFill(PatternType.Solid,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),

            new PatternFill(PatternType.Gray75Percent,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade3)),

            new PatternFill(PatternType.Gray50Percent,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3)),

            new PatternFill(PatternType.Gray25Percent,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3)),
        
            new PatternFill(PatternType.Gray12Percent,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade3)),

            new PatternFill(PatternType.Gray6Percent,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade2)),

            new PatternFill(PatternType.HorizontalStripe,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade2)),

            new PatternFill(PatternType.VerticalStripe,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade2)),

            new PatternFill(PatternType.ReverseDiagonalStripe,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),

            new PatternFill(PatternType.DiagonalStripe,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade1)),

            new PatternFill(PatternType.DiagonalCrosshatch,
                  new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade2)),

            new PatternFill(PatternType.ThickDiagonalCrosshatch,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),

            new PatternFill(PatternType.ThinHorizontalStripe,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade4)),

            new PatternFill(PatternType.ThinVerticalStripe,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade2)),

            new PatternFill(PatternType.ThinReverseDiagonalStripe,
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),

            new PatternFill(PatternType.ThinDiagonalStripe,
                  new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade1)),

            new PatternFill(PatternType.ThinHorizontalCrossHatch,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade1)),

            new PatternFill(PatternType.ThinDiagonalCrosshatch,
                  new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade5),
                  new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade1))
        };

        private static readonly GradientFill[] gradientFills = new GradientFill[] { 

               new GradientFill(GradientType.Horizontal,
                      new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade2),
                      new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade4)),

               new GradientFill(GradientType.HorizontalReversed,
                      new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade1),
                      new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade5)),

               new GradientFill(GradientType.HorizontalRepeated,
                      new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade1),
                      new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade4)),
        
               new GradientFill(GradientType.Vertical,
                      new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade1),
                      new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3)),

                new GradientFill(GradientType.VerticalReversed,
                      new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade3),
                      new ThemableColor(ThemeColorType.Accent6, ColorShadeType.Shade3)),

               new GradientFill(GradientType.VerticalRepeated,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

               new GradientFill(GradientType.DiagonalUp,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

               new GradientFill(GradientType.DiagonalUpReversed,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

               new GradientFill(GradientType.DiagonalUpRepeated,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.DiagonalDown,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.DiagonalDownReversed,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),
                
                new GradientFill(GradientType.DiagonalDownRepeated,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.FromTopLeftCorner,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.FromTopRightCorner,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.FromBottomLeftCorner,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1)),

                new GradientFill(GradientType.FromBottomRightCorner,
                      new ThemableColor(ThemeColorType.Background1),
                      new ThemableColor(ThemeColorType.Accent1))
           };

        #endregion


        #region Methods

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            for (int i = 0; i < patternFills.Length; i++)
            {
                worksheet.Cells[0, i].SetFill(patternFills[i]);
            }

            for (int i = 0; i < gradientFills.Length; i++)
            {
                worksheet.Cells[2, i].SetFill(gradientFills[i]);
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            for (int i = 0; i < patternFills.Length; i++)
            {
                PatternFill fill = worksheet.Cells[0, i].GetFill().Value as PatternFill;
                Assert.IsNotNull(fill);
                Assert.AreEqual(patternFills[i], fill);
            }

            for (int i = 0; i < gradientFills.Length; i++)
            {
                GradientFill fill = worksheet.Cells[2, i].GetFill().Value as GradientFill;
                Assert.IsNotNull(fill);
                Assert.AreEqual(gradientFills[i], fill);
            }
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportFillsTest()
        {
            ImportFileAndAssert("ImportFillsTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportFillsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }

        #endregion
    }
}