﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class CustomFilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 19, 4);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                for (int j = 0; j < 20; j++)
                {
                    sheet1.Cells[i, j].SetValue(i);
                }
            }

            sheet1.Filter.FilterRange = filterRange;

            sheet1.Filter.SetFilter(new CustomFilter(0, new CustomFilterCriteria(ComparisonOperator.GreaterThan, "2")));

            sheet1.Filter.SetFilter(new CustomFilter(1, new CustomFilterCriteria(ComparisonOperator.LessThan, "20")));

            sheet1.Filter.SetFilter(new CustomFilter(2, 
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, "19"),
                LogicalOperator.And,
                new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, "3")));

            sheet1.Filter.SetFilter(new CustomFilter(3, 
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "10"),
                LogicalOperator.Or,
                new CustomFilterCriteria(ComparisonOperator.EqualsTo, "10")));

            sheet1.Filter.SetFilter(new CustomFilter(4, 
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "11"),
                LogicalOperator.And,
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "12")));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            AssertCustomFilter(sheet1.Filter.GetFilter(0), new CustomFilterCriteria(ComparisonOperator.GreaterThan, "2"));
            AssertCustomFilter(sheet1.Filter.GetFilter(1), new CustomFilterCriteria(ComparisonOperator.LessThan, "20"));
            AssertCustomFilter(sheet1.Filter.GetFilter(2),
                new CustomFilterCriteria(ComparisonOperator.LessThanOrEqualsTo, "19"),
                LogicalOperator.And, new CustomFilterCriteria(ComparisonOperator.GreaterThanOrEqualsTo, "3"));
            AssertCustomFilter(sheet1.Filter.GetFilter(3),
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "10"),
                LogicalOperator.Or,
                new CustomFilterCriteria(ComparisonOperator.EqualsTo, "10"));
            AssertCustomFilter(sheet1.Filter.GetFilter(4),
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "11"),
                LogicalOperator.And,
                new CustomFilterCriteria(ComparisonOperator.NotEqualsTo, "12"));
        }

        private static void AssertCustomFilter(IFilter filter, CustomFilterCriteria criteria)
        {
            CustomFilter customFilter = filter as CustomFilter;
            Assert.IsNotNull(customFilter, "Should be a custom filter");

            Assert.AreEqual(criteria, customFilter.Criteria1);
        }

        private static void AssertCustomFilter(IFilter filter, CustomFilterCriteria criteria1, LogicalOperator conjunction, CustomFilterCriteria criteria2)
        {
            AssertCustomFilter(filter, criteria1);

            CustomFilter customFilter = (CustomFilter)filter;
            Assert.AreEqual(criteria2, customFilter.Criteria2);
            Assert.AreEqual(conjunction, customFilter.LogicalOperator);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportCustomFilterTest()
        {
            ImportFileAndAssert("ImportExportCustomFilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportCustomFilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}