﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class DynamicFilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 19, 21);
        private readonly DynamicFilterType[] filterTypes = new DynamicFilterType[] { DynamicFilterType.AboveAverage,
            DynamicFilterType.BelowAverage,
            DynamicFilterType.Today,
            DynamicFilterType.Yesterday,
            DynamicFilterType.Tomorrow,
            DynamicFilterType.ThisWeek,
            DynamicFilterType.LastWeek,
            DynamicFilterType.NextWeek,
            DynamicFilterType.ThisMonth,
            DynamicFilterType.LastMonth,
            DynamicFilterType.NextMonth,
            DynamicFilterType.ThisQuarter,
            DynamicFilterType.LastQuarter,
            DynamicFilterType.NextQuarter,
            DynamicFilterType.ThisYear,
            DynamicFilterType.LastYear,
            DynamicFilterType.NextYear,
            DynamicFilterType.YearToDate,
            DynamicFilterType.Quarter1,
            DynamicFilterType.Quarter3,
            DynamicFilterType.March,
            DynamicFilterType.December,
        };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue((i + 1).ToString());
                sheet1.Cells[i, 1].SetValue((40 - i).ToString());
                sheet1.Cells[i, 2].SetValue("=TODAY()");
                sheet1.Cells[i, 3].SetValue("=TODAY() - 1");
                sheet1.Cells[i, 4].SetValue("=TODAY() + 1");
                sheet1.Cells[i, 5].SetValue("=TODAY()");
                sheet1.Cells[i, 6].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()), DAY(TODAY()) - 7)");
                sheet1.Cells[i, 7].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()), DAY(TODAY()) + 7)");
                sheet1.Cells[i, 8].SetValue("=TODAY()");
                sheet1.Cells[i, 9].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()) - 1, DAY(TODAY()))");
                sheet1.Cells[i, 10].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()) + 1, DAY(TODAY()))");
                sheet1.Cells[i, 11].SetValue("=TODAY()");
                sheet1.Cells[i, 12].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()) - 3, DAY(TODAY()))");
                sheet1.Cells[i, 13].SetValue("=DATE(YEAR(TODAY()), MONTH(TODAY()) + 3, DAY(TODAY()))");
                sheet1.Cells[i, 14].SetValue("=TODAY()");
                sheet1.Cells[i, 15].SetValue("=DATE(YEAR(TODAY()) - 1, MONTH(TODAY()), DAY(TODAY()))");
                sheet1.Cells[i, 16].SetValue("=DATE(YEAR(TODAY()) + 1, MONTH(TODAY()), DAY(TODAY()))");
                sheet1.Cells[i, 17].SetValue("=TODAY() - 1");
                sheet1.Cells[i, 18].SetValue("1/1/2014");
                sheet1.Cells[i, 19].SetValue("8/20/2014");
                sheet1.Cells[i, 20].SetValue("3/25/2014");
                sheet1.Cells[i, 21].SetValue("12/30/2014");
            }

            sheet1.Filter.FilterRange = filterRange;

            for (int i = 0; i < 22; i++)
            {
                sheet1.Filter.SetFilter(new DynamicFilter(i, filterTypes[i]));
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            for (int i = 0; i < 22; i++)
            {
                AssertDynamicFilter(sheet1.Filter.GetFilter(i), filterTypes[i]);
            }
        }

        private static void AssertDynamicFilter(IFilter filter, DynamicFilterType type)
        {
            DynamicFilter dynamicFilter = filter as DynamicFilter;
            Assert.IsNotNull(dynamicFilter, "Should be a DynamicFilter");

            Assert.AreEqual(type, dynamicFilter.DynamicFilterType);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportDynamicFilterTest()
        {
            ImportFileAndAssert("ImportExportDynamicFilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportDynamicFilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
