﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class ForeColorFilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 9, 1);
        private readonly ThemableColor red = ThemableColor.FromArgb(255, 255, 0, 0);
        private readonly ThemableColor orange = ThemableColor.FromArgb(255, 255, 192, 0);
        private readonly ThemableColor blue = ThemableColor.FromArgb(255, 0, 176, 240);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue(i + 1);
                sheet1.Cells[i, 1].SetValue(i + 1);
                if (i < 5)
                {
                    sheet1.Cells[i, 0].SetForeColor(red);
                }
                else
                {
                    sheet1.Cells[i, 0].SetForeColor(orange); 
                }

                if (i > 8)
                {
                    sheet1.Cells[i, 1].SetForeColor(blue);
                }
            }

            sheet1.Filter.FilterRange = filterRange;
            
            sheet1.Filter.SetFilter(new ForeColorFilter(0, orange));
            sheet1.Filter.SetFilter(new ForeColorFilter(1, workbook.Styles["Normal"].ForeColor));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            AssertForeColorFilter(sheet1.Filter.GetFilter(0), orange);
            AssertForeColorFilter(sheet1.Filter.GetFilter(1), new ThemableColor(workbook.Theme.ColorScheme[Theming.ThemeColorType.Text1].Color));
        }

        private static void AssertForeColorFilter(IFilter filter, ThemableColor color)
        {
            ForeColorFilter foreColorFilter = filter as ForeColorFilter;
            Assert.IsNotNull(foreColorFilter, "Should be a fore color filter");

            Assert.AreEqual(color, foreColorFilter.Color);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportForeColorFilterTest()
        {
            ImportFileAndAssert("ImportExportForeColorFilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportForeColorFilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}