﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using System.Windows.Media;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class GradientFillFilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 9, 1);
        private readonly GradientFill blueFill = new GradientFill(GradientType.Horizontal, Color.FromArgb(255, 91, 155, 213), Color.FromArgb(255, 255, 255, 255));
        private readonly GradientFill greenFill = new GradientFill(GradientType.DiagonalDown, Color.FromArgb(255, 255, 192, 0), Color.FromArgb(255, 112, 173, 71));
        private readonly GradientFill redFill = new GradientFill(GradientType.FromCenter, Color.FromArgb(255, 255, 0, 0), Color.FromArgb(255, 237, 125, 49));

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                if (i < 5)
                {
                    sheet1.Cells[i, 0].SetFill(blueFill);
                }
                else
                {
                    sheet1.Cells[i, 0].SetFill(greenFill);
                }

                if (i > 7)
                {
                    sheet1.Cells[i, 1].SetFill(redFill);
                }
            }

            sheet1.Filter.FilterRange = filterRange;

            sheet1.Filter.SetFilter(new FillColorFilter(0, greenFill));
            sheet1.Filter.SetFilter(new FillColorFilter(1, PatternFill.Default));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            AssertFillColorFilter(sheet1.Filter.GetFilter(0), greenFill);
            AssertFillColorFilter(sheet1.Filter.GetFilter(1), PatternFill.Default);
        }

        private static void AssertFillColorFilter(IFilter filter, IFill fill)
        {
            FillColorFilter fillColorFilter = filter as FillColorFilter;
            Assert.IsNotNull(fillColorFilter, "Should be a fill color filter");

            Assert.AreEqual(fill, fillColorFilter.Fill);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportGradientFillFilterTest()
        {
            ImportFileAndAssert("ImportExportGradientFillFilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportGradientFillFilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}