﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class MultipleSheetsWithFiltersTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 2, 1);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();
            Worksheet sheet3 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                for (int j = 0; j < filterRange.ColumnCount; j++)
                {
                    sheet1.Cells[i, j].SetValue(i);
                    sheet2.Cells[i, j].SetValue(i);
                    sheet3.Cells[i, j].SetValue(i);
                }
            }

            sheet1.Filter.FilterRange = filterRange;
            sheet2.Filter.FilterRange = filterRange;
            sheet3.Filter.FilterRange = filterRange;

            sheet1.Filter.SetFilter(new ValuesCollectionFilter(0, new string[] { "3" }, null));
            sheet3.Filter.SetFilter(new DynamicFilter(1, DynamicFilterType.AboveAverage));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];
            Worksheet sheet2 = workbook.Worksheets[1];
            Worksheet sheet3 = workbook.Worksheets[2];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);
            Assert.AreEqual(filterRange, sheet2.Filter.FilterRange);
            Assert.AreEqual(filterRange, sheet3.Filter.FilterRange);

            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(0), new string[] { "3" });
            AssertDynamicFilter(sheet3.Filter.GetFilter(1), DynamicFilterType.AboveAverage);
        }
        private static void AssertValuesCollectionFilter(IFilter filter, string[] stringItems)
        {
            ValuesCollectionFilter valuesCollectionFilter = filter as ValuesCollectionFilter;
            Assert.IsNotNull(valuesCollectionFilter, "Should be a ValuesCollection filter");

            if (stringItems != null)
            {
                CollectionAssert.AreEquivalent(stringItems, valuesCollectionFilter.StringValues.ToArray());
            }
        }
      
        private static void AssertDynamicFilter(IFilter filter, DynamicFilterType type)
        {
            DynamicFilter dynamicFilter = filter as DynamicFilter;
            Assert.IsNotNull(dynamicFilter, "Should be a DynamicFilter");

            Assert.AreEqual(type, dynamicFilter.DynamicFilterType);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportMultipleFiltersTest()
        {
            ImportFileAndAssert("ImportExportMultipleFiltersTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportMultipleFiltersTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}