﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class Top10FilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 11, 7);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    sheet1.Cells[i, j].SetValue(i);
                }
            }

            sheet1.Filter.FilterRange = filterRange;

            sheet1.Filter.SetFilter(new TopFilter(0, TopFilterType.TopNumber, 10));
            sheet1.Filter.SetFilter(new TopFilter(1, TopFilterType.BottomNumber, 10));
            sheet1.Filter.SetFilter(new TopFilter(2, TopFilterType.TopNumber, 15));
            sheet1.Filter.SetFilter(new TopFilter(3, TopFilterType.BottomNumber, 15));
            sheet1.Filter.SetFilter(new TopFilter(4, TopFilterType.TopPercent, 90));
            sheet1.Filter.SetFilter(new TopFilter(5, TopFilterType.BottomPercent, 90));
            sheet1.Filter.SetFilter(new TopFilter(6, TopFilterType.TopPercent, 120));
            sheet1.Filter.SetFilter(new TopFilter(7, TopFilterType.BottomPercent, 120));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            AssertTop10Filter(sheet1.Filter.GetFilter(0), TopFilterType.TopNumber, 10);
            AssertTop10Filter(sheet1.Filter.GetFilter(1), TopFilterType.BottomNumber, 10);
            AssertTop10Filter(sheet1.Filter.GetFilter(2), TopFilterType.TopNumber, 15);
            AssertTop10Filter(sheet1.Filter.GetFilter(3), TopFilterType.BottomNumber, 15);
            AssertTop10Filter(sheet1.Filter.GetFilter(4), TopFilterType.TopPercent, 90);
            AssertTop10Filter(sheet1.Filter.GetFilter(5), TopFilterType.BottomPercent, 90);
            AssertTop10Filter(sheet1.Filter.GetFilter(6), TopFilterType.TopPercent, 120);
            AssertTop10Filter(sheet1.Filter.GetFilter(7), TopFilterType.BottomPercent, 120);
        }

        private static void AssertTop10Filter(IFilter filter, TopFilterType type, double value)
        {
            TopFilter topFilter = filter as TopFilter;
            Assert.IsNotNull(topFilter, "Should be a top10filter");

            Assert.AreEqual(type, topFilter.TopFilterType);
            Assert.AreEqual(value, topFilter.Value);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportTop10FilterTest()
        {
            ImportFileAndAssert("ImportExportTop10FilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportTop10FilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}