﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.FilteringTests
{
    [TestClass]
    public class ValuesCollectionFilterTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(0, 0, 19, 6);
        private readonly string[] filterStringValues0 = new string[] { "2", "3", "4", "5", "6", "7", "8", "9" };
        private readonly string[] filterStringValues = new string[] { "2", "4", "5", "6", "7", "8", "9" };
        private readonly DateGroupItem[] filterDateGroupItems1 = new DateGroupItem[] { new DateGroupItem(2016) };
        private readonly DateGroupItem[] filterDateGroupItems2 = new DateGroupItem[] { new DateGroupItem(2016, 6), new DateGroupItem(2016, 7), new DateGroupItem(2016, 8), new DateGroupItem(2016, 9), new DateGroupItem(2016, 10), new DateGroupItem(2016, 11) };
        private readonly DateGroupItem[] filterDateGroupItems3 = new DateGroupItem[] { new DateGroupItem(2016, 6, 10), new DateGroupItem(2016, 6, 11, 6) };
        private readonly DateGroupItem[] filterDateGroupItems4 = new DateGroupItem[] { new DateGroupItem(2016, 6, 10, 1), new DateGroupItem(2016, 6, 10, 8), new DateGroupItem(2016, 6, 10, 15) };
        private readonly DateGroupItem[] filterDateGroupItems5 = new DateGroupItem[] { new DateGroupItem(2016, 6, 10, 1, 30) };
        private readonly DateGroupItem[] filterDateGroupItems6 = new DateGroupItem[] { new DateGroupItem(2016, 6, 10, 1, 30, 45) };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Filter.FilterRange = filterRange;

            this.PopulateWorksheet(sheet1);
            this.AddFilters(sheet1);

            return workbook;
        }

        private void PopulateWorksheet(Worksheet sheet)
        {
            for (int i = 0; i < 7; i++)
            {
                if (i != 1)
                {
                    for (int j = 0; j < 10; j++)
                    {

                        sheet.Cells[i, j].SetValue(i);
                    }
                }

                if (i != 0)
                {
                    double start = 42531.0630208333;
                    double step = 31.05;

                    if (i == 2)
                    {
                        step = 31;
                    }
                    else if (i == 3 || i == 4)
                    {
                        step = 0.3;
                    }
                    else if (i == 5 || i == 6)
                    {
                        step = 0.0001;
                    }

                    for (int j = 10; j < 20; j++)
                    {
                        sheet.Cells[i, j].SetValue(start);
                        start += step;
                    }
                }
            }
        }

        private void AddFilters(Worksheet sheet)
        {
            ValuesCollectionFilter filter = new ValuesCollectionFilter(0, filterStringValues0, null, true);
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(1, null, filterDateGroupItems1, true);
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(2, filterStringValues, filterDateGroupItems2, true);
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(3, filterStringValues, filterDateGroupItems3, false);
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(4, filterStringValues, filterDateGroupItems4, false);    
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(5, filterStringValues, filterDateGroupItems5, false);
            sheet.Filter.SetFilter(filter);

            filter = new ValuesCollectionFilter(6, filterStringValues, filterDateGroupItems6, false);
            sheet.Filter.SetFilter(filter);
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);

            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(0), filterStringValues0, null);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(1), null, filterDateGroupItems1);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(2), filterStringValues, filterDateGroupItems2);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(3), filterStringValues, filterDateGroupItems3);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(4), filterStringValues, filterDateGroupItems4);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(5), filterStringValues, filterDateGroupItems5);
            AssertValuesCollectionFilter(sheet1.Filter.GetFilter(6), filterStringValues, filterDateGroupItems6);
        }

        private static void AssertValuesCollectionFilter(IFilter filter, string[] stringItems, DateGroupItem[] dateItems)
        {
            ValuesCollectionFilter valuesCollectionFilter = filter as ValuesCollectionFilter;
            Assert.IsNotNull(valuesCollectionFilter, "Should be a ValuesCollection filter");

            if (stringItems != null)
            {
                CollectionAssert.AreEquivalent(stringItems, valuesCollectionFilter.StringValues.ToArray());
            }
            if (dateItems != null)
            {
                CollectionAssert.AreEquivalent(dateItems, valuesCollectionFilter.DateItems.ToArray());
            }
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportValuesCollectionFilterTest()
        {
            ImportFileAndAssert("ImportExportValuesCollectionFilterTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportValuesCollectionFilterTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}