﻿using System;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class FontTests : ImportExportTestClassBase
    {
        static readonly ThemableColor red = new ThemableColor(Colors.Red);
        static readonly ThemableFontFamily segoeUI = new ThemableFontFamily("Segoe UI");
        static readonly ThemableFontFamily major = new ThemableFontFamily(ThemeFontType.Major);
        static readonly ThemableColor accesnt1Shade3 = new ThemableColor(ThemeColorType.Accent1, ColorShadeType.Shade3);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            CellSelection a1Selection = worksheet.Cells[0, 0];
            CellSelection a2Selection = worksheet.Cells[1, 0];

            a1Selection.SetValue("This is a long sentence");
            a1Selection.SetFontFamily(segoeUI);
            a1Selection.SetIsBold(true);
            a1Selection.SetIsItalic(true);
            a1Selection.SetFontSize(UnitHelper.PointToDip(20));
            a1Selection.SetUnderline(UnderlineType.Single);
            a1Selection.SetForeColor(red);

            a2Selection.SetValue("This is another long sentence");
            a2Selection.SetFontFamily(major);
            a2Selection.SetIsBold(false);
            a2Selection.SetIsItalic(true);
            a2Selection.SetFontSize(UnitHelper.PointToDip(8));
            a2Selection.SetUnderline(UnderlineType.Single);
            a2Selection.SetForeColor(accesnt1Shade3);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            CellSelection cellSelection = workbook.Worksheets["Sheet1"].Cells[0, 0];

            AssertFontProperties(cellSelection, segoeUI, true, true, 20, UnderlineType.Single, red);

            cellSelection = workbook.Worksheets["Sheet1"].Cells[1, 0];

            AssertFontProperties(cellSelection, major, false, true, 8, UnderlineType.Single, accesnt1Shade3);
        }

        public static void AssertFontProperties(CellSelection cellSelection, ThemableFontFamily expectedFontFamily, bool expectedIsBold,
            bool expectedIsItalic, double expectedFontSizeInPoints, UnderlineType expectedUnderlineType, ThemableColor expectedForeColor)
        {
            ThemableFontFamily fontFamily = cellSelection.GetFontFamily().Value;
            Assert.AreEqual(expectedFontFamily, fontFamily);

            bool isBold = cellSelection.GetIsBold().Value;
            Assert.AreEqual(expectedIsBold, isBold);

            bool isItalic = cellSelection.GetIsItalic().Value;
            Assert.AreEqual(expectedIsItalic, isItalic);

            double fontSize = cellSelection.GetFontSize().Value;
            Assert.AreEqual(UnitHelper.PointToDip(expectedFontSizeInPoints), fontSize);

            UnderlineType underline = cellSelection.GetUnderline().Value;
            Assert.AreEqual(expectedUnderlineType, underline);

            ThemableColor foreColor = cellSelection.GetForeColor().Value;
            Assert.AreEqual(expectedForeColor, foreColor);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportFontTest()
        {
            ImportFileAndAssert("ImportFontsTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportFontTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
