﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class GroupingImportExportTests : ImportExportTestClassBase
    {
        private readonly Dictionary<int, int> rowsToOutlineLevel = new Dictionary<int, int>
        {
            {0, 7},
            {1, 7},
            {2, 7},
            {3, 7},
            {4, 6},
            {5, 5},
            {6, 4},
            {7, 3},
            {8, 2},
            {9, 1},
        };

        private readonly Dictionary<int, int> columnsToOutlineLevel = new Dictionary<int, int>
        {
            {0, 1},
            {1, 1},
            {2, 1},
            {4, 1},
            {5, 2},
            {6, 4},
            {7, 3},
            {8, 2},
            {9, 1},
        };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet1 = workbook.Worksheets["Sheet1"];

            foreach (var pair in this.rowsToOutlineLevel)
            {
                int rowIndex = pair.Key;
                int outlineLevel = pair.Value;

                worksheet1.Rows[rowIndex].SetOutlineLevel(outlineLevel);
            }

            foreach (var pair in this.columnsToOutlineLevel)
            {
                int columnIndex = pair.Key;
                int outlineLevel = pair.Value;

                worksheet1.Columns[columnIndex].SetOutlineLevel(outlineLevel);
            }

            Worksheet worksheet2 = workbook.Worksheets.Add();
            worksheet2.GroupingProperties.SummaryRowIsBelow = false;

            Worksheet worksheet3 = workbook.Worksheets.Add();
            worksheet3.GroupingProperties.SummaryColumnIsToRight = false;

            return workbook;
        }

        internal override void AssertExportedBookImportedSpecificValuesCorrectly(Workbook workbook)
        {
            this.AssertCorrectWorkbook(workbook);
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            this.AssertCorrectWorkbook(workbook);
        }

        private void AssertCorrectWorkbook(Workbook workbook)
        {
            Worksheet worksheet1 = workbook.Worksheets["Sheet1"];

            foreach (var pair in this.rowsToOutlineLevel)
            {
                int rowIndex = pair.Key;
                int expectedOutlineLevel = pair.Value;

                int actualOutlineLevel = worksheet1.Rows[rowIndex].GetOutlineLevel().Value;

                Assert.AreEqual(expectedOutlineLevel, actualOutlineLevel);
            }

            foreach (var pair in this.columnsToOutlineLevel)
            {
                int columnIndex = pair.Key;
                int expectedOutlineLevel = pair.Value;

                int actualOutlineLevel = worksheet1.Columns[columnIndex].GetOutlineLevel().Value;

                Assert.AreEqual(expectedOutlineLevel, actualOutlineLevel);
            }

            Assert.IsTrue(worksheet1.GroupingProperties.SummaryRowIsBelow);
            Assert.IsTrue(worksheet1.GroupingProperties.SummaryColumnIsToRight);

            Worksheet worksheet2 = workbook.Worksheets["Sheet2"];
            Assert.IsFalse(worksheet2.GroupingProperties.SummaryRowIsBelow);
            Assert.IsTrue(worksheet2.GroupingProperties.SummaryColumnIsToRight);

            Worksheet worksheet3 = workbook.Worksheets["Sheet3"];
            Assert.IsTrue(worksheet3.GroupingProperties.SummaryRowIsBelow);
            Assert.IsFalse(worksheet3.GroupingProperties.SummaryColumnIsToRight);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportGroupingTest()
        {
            ImportFileAndAssert("ImportGroupingTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportGroupingTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
