﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class HiddenColumnsTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            worksheet.Cells[0, 0, 0, 0].SetValue("Test.");

            for (int i = 1; i <= 4; i++)
            {
                worksheet.Columns[i].SetHidden(true);
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            ColumnWidth actualColumnWidth;
            ColumnWidth expectedColumnWidth = new ColumnWidth(0, true);

            for (int i = 21; i <= 25; i++)
            {
                actualColumnWidth = workbook.ActiveWorksheet.Columns[i].GetWidth().Value;
                Assert.AreEqual(expectedColumnWidth, actualColumnWidth);
            }
        }

        internal override void AssertExportedBookImportedSpecificValuesCorrectly(Workbook workbook)
        {
            for (int i = 1; i <= 4; i++)
            {
                bool isHidden = workbook.ActiveWorksheet.Columns[i].GetHidden().Value;
                Assert.IsTrue(isHidden);
            }
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportHiddenColumnsTest()
        {
            ImportFileAndAssert("ImportHiddenColumnsDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportHiddenColumnsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
