﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class HiddenRowsTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            worksheet.Cells[0, 0, 0, 0].SetValue("Test.");

            for (int i = 1; i <= 4; i++)
            {
                worksheet.Rows[i].SetHidden(true);
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            RowHeight actualRowHeight;
            RowHeight expectedRowHeight = new RowHeight(20, false);

            for (int i = 1; i <= 4; i++)
            {
                actualRowHeight = workbook.ActiveWorksheet.Rows[i].GetHeight().Value;
                Assert.AreEqual(expectedRowHeight, actualRowHeight);

                bool isHidden = workbook.ActiveWorksheet.Rows[i].GetHidden().Value;
                Assert.IsTrue(isHidden);
            }
        }

        internal override void AssertExportedBookImportedSpecificValuesCorrectly(Workbook workbook)
        {
            this.AssertWorkbookImportedCorrectly(workbook);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportHiddenRowsTest()
        {
            ImportFileAndAssert("ImportHiddenRowsDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportHiddenRowsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}