﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class HyperlinksTests : ImportExportTestClassBase
    {
        #region Fields

        private static readonly HyperlinkInfo webAddress = HyperlinkInfo.CreateHyperlink("http://google.com/", "Google");
        private static readonly HyperlinkInfo inDocument = HyperlinkInfo.CreateInDocumentHyperlink("Sheet1!A1:B3", "Go to A1:B3");
        private static readonly HyperlinkInfo mailto = HyperlinkInfo.CreateMailtoHyperlink("someOne@someCompany.com", "someSubject", "Mail to someOne");

        #endregion


        #region Methods

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            worksheet.Hyperlinks.Add(new CellIndex(0, 0), webAddress);
            worksheet.Hyperlinks.Add(new CellIndex(1, 0), inDocument);
            worksheet.Hyperlinks.Add(new CellIndex(2, 0), mailto);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            SpreadsheetHyperlink retrievedWebAddress = worksheet.Hyperlinks.FirstOrDefault(x => x.HyperlinkInfo.Equals(webAddress));
            Assert.IsNotNull(retrievedWebAddress);
            Assert.AreEqual(new CellRange(0, 0, 0, 0), retrievedWebAddress.Range);

            SpreadsheetHyperlink retrievedInDocument = worksheet.Hyperlinks.FirstOrDefault(x => x.HyperlinkInfo.Equals(inDocument));
            Assert.IsNotNull(retrievedInDocument);
            Assert.AreEqual(new CellRange(1, 0, 1, 0), retrievedInDocument.Range);

            SpreadsheetHyperlink retrievedMailto = worksheet.Hyperlinks.FirstOrDefault(x => x.HyperlinkInfo.Equals(mailto));
            Assert.IsNotNull(retrievedMailto);
            Assert.AreEqual(new CellRange(2, 0, 2, 0), retrievedMailto.Range);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportHyperlinksTest()
        {
            ImportFileAndAssert("ImportHyperlinksTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportHyperlinksTest()
        {
            ArrangeWorkbookExportImportAssert();
        }

        #endregion
    }
}
