﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ImagesTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            FloatingImage image = new FloatingImage(worksheet, new CellIndex(0, 0), 0, 0);
            Stream stream = TestHelper.GetStreamFromFile("Penguins.jpg");

            using (stream)
            {
                image.ImageSource = new ImageSource(stream, "jpg");
            }

            image.Width = 100;
            image.Height = 100;
            image.RotationAngle = -90;
            image.IsHorizontallyFlipped = true;
            image.IsHorizontallyFlipped = true;
            image.LockAspectRatio = false;
            image.PreferRelativeToOriginalResize = false;

            worksheet.Shapes.Add(image);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            FloatingShapeBase firstImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 1");
            Assert.IsNotNull(firstImage);
            Assert.AreEqual(new CellIndex(2, 1), firstImage.CellIndex);
            Assert.AreEqual(0, firstImage.OffsetX);
            Assert.AreEqual(0, firstImage.OffsetY);
            Assert.AreEqual(false, firstImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, firstImage.IsVerticallyFlipped);
            Assert.AreEqual(0, firstImage.RotationAngle);
            Assert.AreEqual(318, firstImage.Width);
            Assert.AreEqual(239, firstImage.Height);
            Assert.IsFalse(firstImage.LockAspectRatio);
            Assert.IsTrue((firstImage as FloatingImage).PreferRelativeToOriginalResize);

            FloatingShapeBase secondImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 7");
            Assert.IsNotNull(secondImage);
            Assert.AreEqual(new CellIndex(15, 1), secondImage.CellIndex);
            Assert.AreEqual(0, secondImage.OffsetX);
            Assert.AreEqual(0, secondImage.OffsetY);
            Assert.AreEqual(true, secondImage.IsHorizontallyFlipped);
            Assert.AreEqual(true, secondImage.IsVerticallyFlipped);
            Assert.AreEqual(0, secondImage.RotationAngle);
            Assert.AreEqual(320, secondImage.Width);
            Assert.AreEqual(240, secondImage.Height);
            Assert.IsTrue(secondImage.LockAspectRatio);
            Assert.IsFalse((secondImage as FloatingImage).PreferRelativeToOriginalResize);

            FloatingShapeBase thirdImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 8");
            Assert.IsNotNull(thirdImage);
            Assert.AreEqual(new CellIndex(0, 7), thirdImage.CellIndex);
            Assert.AreEqual(40.51, thirdImage.OffsetX);
            Assert.AreEqual(0.17, thirdImage.OffsetY);
            Assert.AreEqual(false, thirdImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, thirdImage.IsVerticallyFlipped);
            Assert.AreEqual(90, thirdImage.RotationAngle);
            Assert.AreEqual(314, Math.Round(thirdImage.Width, 2));
            Assert.AreEqual(224, Math.Round(thirdImage.Height, 2));
            Assert.IsTrue(thirdImage.LockAspectRatio);
            Assert.IsTrue((thirdImage as FloatingImage).PreferRelativeToOriginalResize);

            FloatingShapeBase fourthImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 9");
            Assert.IsNotNull(fourthImage);
            Assert.AreEqual(new CellIndex(19, 7), fourthImage.CellIndex);
            Assert.AreEqual(0, fourthImage.OffsetX);
            Assert.AreEqual(0, fourthImage.OffsetY);
            Assert.AreEqual(false, fourthImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, fourthImage.IsVerticallyFlipped);
            Assert.AreEqual(332, Math.Round(fourthImage.RotationAngle));
            Assert.AreEqual(318, fourthImage.Width);
            Assert.AreEqual(238, fourthImage.Height);
            Assert.IsTrue(fourthImage.LockAspectRatio);
            Assert.IsTrue((fourthImage as FloatingImage).PreferRelativeToOriginalResize);

            FloatingShapeBase fifthImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 3");
            Assert.IsNotNull(fifthImage);
            Assert.AreEqual(new CellIndex(10, 14), fifthImage.CellIndex);
            Assert.AreEqual(40, fifthImage.OffsetX);
            Assert.AreEqual(0, fifthImage.OffsetY);
            Assert.AreEqual(false, fifthImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, fifthImage.IsVerticallyFlipped);
            Assert.AreEqual(-600, Math.Round(fifthImage.RotationAngle));
            Assert.AreEqual(320, fifthImage.Width);
            Assert.AreEqual(241, Math.Round(fifthImage.Height, 2));
            Assert.IsTrue(fifthImage.LockAspectRatio);
            Assert.IsTrue((fifthImage as FloatingImage).PreferRelativeToOriginalResize);

            FloatingShapeBase sixthImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 6");
            Assert.IsNotNull(sixthImage);
            Assert.AreEqual(new CellIndex(2, 21), sixthImage.CellIndex);
            Assert.AreEqual(5, sixthImage.OffsetX);
            Assert.AreEqual(85, sixthImage.OffsetY);
            Assert.AreEqual(false, sixthImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, sixthImage.IsVerticallyFlipped);
            Assert.AreEqual(-600, Math.Round(sixthImage.RotationAngle));
            Assert.AreEqual(320, Math.Round(sixthImage.Width, 2));
            Assert.AreEqual(244.45, Math.Round(sixthImage.Height, 2));
            Assert.IsTrue(sixthImage.LockAspectRatio);
            Assert.IsTrue((sixthImage as FloatingImage).PreferRelativeToOriginalResize);
            
            FloatingShapeBase seventhImage = worksheet.Shapes.FirstOrDefault(x => x.Name == "Picture 2");
            Assert.IsNotNull(seventhImage);
            Assert.AreEqual(new CellIndex(1, 14), seventhImage.CellIndex);
            Assert.AreEqual(0, seventhImage.OffsetX);
            Assert.AreEqual(0, seventhImage.OffsetY);
            Assert.AreEqual(false, seventhImage.IsHorizontallyFlipped);
            Assert.AreEqual(false, seventhImage.IsVerticallyFlipped);
            Assert.AreEqual(0, Math.Round(seventhImage.RotationAngle));
            Assert.AreEqual(255, Math.Round(seventhImage.Width));
            Assert.AreEqual(237, Math.Round(seventhImage.Height));
            Assert.IsTrue(seventhImage.LockAspectRatio);
            Assert.IsTrue((seventhImage as FloatingImage).PreferRelativeToOriginalResize);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportImagesTest()
        {
            ImportFileAndAssert("ImportImagesTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportImagesTest()
        {
            ArrangeWorkbookExportImportAssert();
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportImagesWithoutSizeElementTest()
        {
            Workbook workbook = TestHelper.GetWorkbookFromFile("ImportImageWithoutSizeElement.xlsx");
            Worksheet worksheet = workbook.ActiveWorksheet;

            FloatingShapeBase image = worksheet.Shapes.FirstOrDefault(x => x.Name == "Imagen 5");
            Assert.IsNotNull(image);
            Assert.AreEqual(new CellIndex(0, 0), image.CellIndex);
            Assert.AreEqual(0, image.OffsetX);
            Assert.AreEqual(0, image.OffsetY);
            Assert.AreEqual(false, image.IsHorizontallyFlipped);
            Assert.AreEqual(false, image.IsVerticallyFlipped);
            Assert.AreEqual(0, image.RotationAngle);
            Assert.AreEqual(666, image.Width);
            Assert.AreEqual(108, image.Height);
            Assert.IsTrue(image.LockAspectRatio);
            Assert.IsTrue((image as FloatingImage).PreferRelativeToOriginalResize);
        }
    }
}
