﻿using System;
using System.IO;
using System.Threading.Tasks;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
#if WPF
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.XmlValidation;
#endif

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    public abstract class ImportExportTestClassBase
    {
        public abstract Workbook ArrangeWorkbook();

        public abstract void AssertWorkbookImportedCorrectly(Workbook workbook);

        internal virtual void AssertExportedBookImportedSpecificValuesCorrectly(Workbook workbook)
        {
        }

        public void ImportFileAndAssert(string fileName)
        {
            Workbook workbook = TestHelper.GetWorkbookFromFile(fileName);
            AssertWorkbookImportedCorrectly(workbook);
        }

        public void ArrangeWorkbookExportImportAssert()
        {
            Workbook expectedWorkbook = this.ArrangeWorkbook();
#if WPF
            this.ValidateSpreadsheetDocument(expectedWorkbook);
#endif
            Test(expectedWorkbook);

            Task.WaitAll(Task.Factory.StartNew(() => { Test(expectedWorkbook); }));
        }

        private void Test(Workbook expectedWorkbook)
        {
            using (Stream stream = new MemoryStream())
            {
                XlsxFormatProvider provider = new XlsxFormatProvider();
                provider.Export(expectedWorkbook, stream);

                Workbook actualWorkbook = provider.Import(stream);

                TestHelper.AssertWorkbooksAreEqual(expectedWorkbook, actualWorkbook);
                AssertExportedBookImportedSpecificValuesCorrectly(actualWorkbook);
            }
        }

#if WPF
        private void ValidateSpreadsheetDocument(Workbook workbook)
        {
            XmlValidator.ValidateOpenXmlDocument(workbook);
        }
#endif
    }
}
