﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ImportGeneneralStringFormatCode : ImportExportTestClassBase
    {
        private string valueOfA1 = "123{ds}45".ReplaceCultureDependentSeparators();
        private string valueOfC3 = "67{ds}89".ReplaceCultureDependentSeparators();

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(valueOfA1);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.ActiveWorksheet;

            CellValueFormat actualFormatValueOfA1 = worksheet.Cells[0, 0].GetFormat().Value;
            string actualValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfA1);
            string actualRawValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.RawValue;

            Assert.AreEqual(valueOfA1, actualValueOfA1);
            Assert.AreEqual(string.Format("={0}", valueOfA1), actualRawValueOfA1);
            Assert.AreEqual("GENERAL", actualFormatValueOfA1.FormatString.ToUpperInvariant());

            CellValueFormat actualFormatValueOfC3 = worksheet.Cells[2, 2].GetFormat().Value;
            string actualValueOfC3 = worksheet.Cells[2, 2].GetValue().Value.GetResultValueAsString(actualFormatValueOfA1);
            string actualRawValueOfC3 = worksheet.Cells[2, 2].GetValue().Value.RawValue;

            Assert.AreEqual(valueOfC3, actualValueOfC3);
            Assert.AreEqual(string.Format("={0}", valueOfC3), actualRawValueOfC3);
            Assert.AreEqual("[$-1010409]GENERAL", actualFormatValueOfC3.FormatString.ToUpperInvariant());
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportGeneneralStringFormatCodeTest()
        {
            ImportFileAndAssert("ImportGeneneralStringFormatCode.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportGeneneralStringFormatCodeTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
