﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ImportNumberFormat42 : ImportExportTestClassBase
    {
        private readonly string valueOfA1 = "23{ds}45".ReplaceCultureDependentSeparators();
        private readonly string formatOfA1 = "_({cs}*#{gs}##0_);_({cs}*(#{gs}##0);_({cs}* \"-\"_);_(@_)".ReplaceCultureDependentSeparators();
        private readonly string formatedValueOfA1 = "$ 23";

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(formatOfA1));
            worksheet.Cells[0, 0].SetValue(valueOfA1);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.ActiveWorksheet;

            CellValueFormat actualFormatValueOfA1 = worksheet.Cells[0, 0].GetFormat().Value;
            string actualValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfA1);
            string actualRawValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.RawValue;

            Assert.AreEqual(formatedValueOfA1, actualValueOfA1);
            Assert.AreEqual(valueOfA1, actualRawValueOfA1);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportNumberFormat42Test()
        {
            ImportFileAndAssert("ImportNumberFormat42.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportNumberFormat42Test()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
