﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ImportNumberFormat5678 : ImportExportTestClassBase
    {
        private readonly string value = "-23{ds}45".ReplaceCultureDependentSeparators();
        private readonly string format5 = "{cs}#{gs}##0_);({cs}#{gs}##0)".ReplaceCultureDependentSeparators();
        private readonly string format6 = "{cs}#{gs}##0_);[Red]({cs}#{gs}##0)".ReplaceCultureDependentSeparators();
        private readonly string format7 = "{cs}#{gs}##0{ds}00_);({cs}#{gs}##0{ds}00)".ReplaceCultureDependentSeparators();
        private readonly string format8 = "{cs}#{gs}##0{ds}00_);[Red]({cs}#{gs}##0{ds}00)".ReplaceCultureDependentSeparators();
        private readonly string formatedValue5 = "({cs}23)".ReplaceCultureDependentSeparators();
        private readonly string formatedValue6 = "({cs}23)".ReplaceCultureDependentSeparators();
        private readonly string formatedValue7 = "({cs}23{ds}45)".ReplaceCultureDependentSeparators();
        private readonly string formatedValue8 = "({cs}23{ds}45)".ReplaceCultureDependentSeparators();

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(format5));
            worksheet.Cells[0, 0].SetValue(value);

            worksheet.Cells[1, 0].SetFormat(new CellValueFormat(format6));
            worksheet.Cells[1, 0].SetValue(value);

            worksheet.Cells[2, 0].SetFormat(new CellValueFormat(format7));
            worksheet.Cells[2, 0].SetValue(value);

            worksheet.Cells[3, 0].SetFormat(new CellValueFormat(format8));
            worksheet.Cells[3, 0].SetValue(value);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.ActiveWorksheet;

            CellValueFormat actualFormatValueOfA1 = worksheet.Cells[0, 0].GetFormat().Value;
            string actualValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfA1);
            string actualRawValueOfA1 = worksheet.Cells[0, 0].GetValue().Value.RawValue;

            CellValueFormat actualFormatValueOfB1 = worksheet.Cells[1, 0].GetFormat().Value;
            string actualValueOfB1 = worksheet.Cells[1, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfB1);
            string actualRawValueOfB1 = worksheet.Cells[1, 0].GetValue().Value.RawValue;

            CellValueFormat actualFormatValueOfC1 = worksheet.Cells[2, 0].GetFormat().Value;
            string actualValueOfC1 = worksheet.Cells[2, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfC1);
            string actualRawValueOfC1 = worksheet.Cells[2, 0].GetValue().Value.RawValue;

            CellValueFormat actualFormatValueOfD1 = worksheet.Cells[3, 0].GetFormat().Value;
            string actualValueOfD1 = worksheet.Cells[3, 0].GetValue().Value.GetResultValueAsString(actualFormatValueOfD1);
            string actualRawValueOfD1 = worksheet.Cells[3, 0].GetValue().Value.RawValue;

            Assert.AreEqual(formatedValue5, actualValueOfA1);
            Assert.AreEqual(value, actualRawValueOfA1);

            Assert.AreEqual(formatedValue6, actualValueOfB1);
            Assert.AreEqual(value, actualRawValueOfB1);

            Assert.AreEqual(formatedValue7, actualValueOfC1);
            Assert.AreEqual(value, actualRawValueOfC1);

            Assert.AreEqual(formatedValue8, actualValueOfD1);
            Assert.AreEqual(value, actualRawValueOfD1);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportNumberFormat5678Test()
        {
            ImportFileAndAssert("ImportNumberFormat5678.xlsx");
        }
    }
}
