﻿using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class MergedCellsTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];
           
            worksheet.Cells[2, 1, 3, 2].Merge();
            worksheet.Cells[2, 1, 3, 2].SetHorizontalAlignment(RadHorizontalAlignment.Center);

            worksheet.Cells[4, 4, 10, 9].Merge();
            worksheet.Cells[4, 4, 10, 9].SetHorizontalAlignment(RadHorizontalAlignment.Center);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            List<CellRange> mergedRanges = worksheet.Cells.GetMergedCellRanges().ToList();

            Assert.AreEqual(mergedRanges[0].FromIndex, new CellIndex(2, 1));
            Assert.AreEqual(mergedRanges[0].ToIndex, new CellIndex(3, 2));

            RangePropertyValue<RadHorizontalAlignment> actualHorizontalAlignment = worksheet.Cells[mergedRanges[0]].GetHorizontalAlignment();
            Assert.AreEqual(false, actualHorizontalAlignment.IsIndeterminate);
            Assert.AreEqual(RadHorizontalAlignment.Center, actualHorizontalAlignment.Value);

            Assert.AreEqual(mergedRanges[1].FromIndex, new CellIndex(4, 4));
            Assert.AreEqual(mergedRanges[1].ToIndex, new CellIndex(10, 9));

            actualHorizontalAlignment = worksheet.Cells[mergedRanges[0]].GetHorizontalAlignment();
            Assert.AreEqual(false, actualHorizontalAlignment.IsIndeterminate);
            Assert.AreEqual(RadHorizontalAlignment.Center, actualHorizontalAlignment.Value);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportMergedCellsTest()
        {
            ImportFileAndAssert("ImportMergedCellsTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportMergedCellsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}