﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class NamesInCellsTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet sheet1 = workbook.Worksheets["Sheet1"];
            Worksheet sheet2 = workbook.Worksheets.Add();

            workbook.Names.Add("gl", @"=""global""", new CellIndex(0, 0), string.Empty);
            sheet1.Names.Add("lo", @"=""local""", new CellIndex(0, 0), string.Empty);

            sheet1.Cells[0, 0].SetValue("=gl");
            sheet1.Cells[0, 1].SetValue("=lo");

            sheet2.Cells[0, 0].SetValue("=Book1!gl");
            sheet2.Cells[0, 1].SetValue("=Sheet1!lo");

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            string a1Value = workbook.Worksheets["Sheet1"].Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("global", a1Value);

            string a2Value = workbook.Worksheets["Sheet1"].Cells[0, 1].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("local", a2Value);

            a1Value = workbook.Worksheets["Sheet2"].Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("global", a1Value);

            a2Value = workbook.Worksheets["Sheet2"].Cells[0, 1].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            Assert.AreEqual("local", a2Value);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportNamesInCellsTest()
        {
            ImportFileAndAssert("ImportNamesTestDocument2.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportNamesInCellsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
