﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class NamesTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet sheet1 = workbook.Worksheets["Sheet1"];
            Worksheet sheet2 = workbook.Worksheets.Add();
            Worksheet sheet3 = workbook.Worksheets.Add();

            workbook.Names.Add("WorkbookName", "=Sheet1!$K$7:Sheet1!$M$13", new CellIndex(0, 0), "WorkbookComment");
            sheet1.Names.Add("Sheet1Name", "=Sheet1!$A$1", new CellIndex(0, 0), "Sheet1Comment");
            sheet2.Names.Add("Sheet2Name", "=Sheet2!$A$1:$C$9", new CellIndex(0, 0), string.Empty);
            sheet3.Names.Add("Sheet3Name", "=Sheet3!$B$9:$D$13", new CellIndex(0, 0), "Sheet3Comment");
            sheet3.Names.Add("Sheet3HiddenName", "=Sheet3!$A$1", new CellIndex(0, 0), string.Empty, false);
         
            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            ISpreadsheetName name;

            workbook.Names.TryGetSpreadsheetName("WorkbookName", out name);
            AssertSpreadsheetnameCorrectness(name, "WorkbookName", "=Sheet1!$K$7:'Sheet1'!$M$13", "WorkbookComment", true);

            workbook.Worksheets["Sheet1"].Names.TryGetSpreadsheetName("Sheet1Name", out name);
            AssertSpreadsheetnameCorrectness(name, "Sheet1Name", "=Sheet1!$A$1", "Sheet1Comment", true);

            workbook.Worksheets["Sheet2"].Names.TryGetSpreadsheetName("Sheet2Name", out name);
            AssertSpreadsheetnameCorrectness(name, "Sheet2Name", "=Sheet2!$A$1:$C$9", String.Empty, true);

            workbook.Worksheets["Sheet3"].Names.TryGetSpreadsheetName("Sheet3Name", out name);
            AssertSpreadsheetnameCorrectness(name, "Sheet3Name", "=Sheet3!$B$9:$D$13", "Sheet3Comment", true);

            workbook.Worksheets["Sheet3"].Names.TryGetSpreadsheetName("Sheet3HiddenName", out name);
            AssertSpreadsheetnameCorrectness(name, "Sheet3HiddenName", "=Sheet3!$A$1", string.Empty, false);
        }

        private void AssertSpreadsheetnameCorrectness(ISpreadsheetName nameEntity, string name, string refersTo, string comment, bool isVisible)
        {
            Assert.IsNotNull(nameEntity);
            Assert.AreEqual(name, nameEntity.Name);
            Assert.AreEqual(refersTo, nameEntity.RefersTo);
            Assert.AreEqual(comment, nameEntity.Comment);
            Assert.AreEqual(isVisible, nameEntity.IsVisible);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportNamesTest()
        {
            ImportFileAndAssert("ImportNamesTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportNamesTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
