﻿using System;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class PaletteColorsTests : ImportExportTestClassBase
    {
        private static readonly Color[] colors = new Color[]
        {
            Color.FromArgb(255,192, 0, 0),
            Color.FromArgb(255,255,0,0),
            Color.FromArgb(255,255,192,0),
            Color.FromArgb(255,255,255,0),
            Color.FromArgb(255,146,208,80),
            Color.FromArgb(255,0,176,80),
            Color.FromArgb(255,0,176,240),
            Color.FromArgb(255,0,112,192),
            Color.FromArgb(255,0,32,96),
            Color.FromArgb(255,112,48,160),
        };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            Array themeColorType = Enum.GetValues(typeof(ThemeColorType));
            Array colorShadeType = Enum.GetValues(typeof(ColorShadeType));

            // TODO: 10 should be themeColorType.Count
            for (int columnIndex = 0; columnIndex < 10; columnIndex++)
            {
                ThemeColorType colorType = (ThemeColorType)themeColorType.GetValue(columnIndex);

                IFill headerColorFill = new PatternFill(PatternType.Solid,
                      new ThemableColor(colorType),
                      PatternFill.Default.BackgroundColor);

                cells[0, columnIndex].SetFill(headerColorFill);

                for (int rowIndex = 0; rowIndex < colorShadeType.Length; rowIndex++)
                {
                    ColorShadeType shadeType = (ColorShadeType)colorShadeType.GetValue(rowIndex);

                    IFill fill = new PatternFill(PatternType.Solid,
                      new ThemableColor(colorType, shadeType),
                      PatternFill.Default.BackgroundColor);

                    cells[rowIndex + 1, columnIndex].SetFill(fill);
                }

                IFill customColorFill = new PatternFill(PatternType.Solid,
                     new ThemableColor(colors[columnIndex]),
                     PatternFill.Default.BackgroundColor);

                cells[colorShadeType.Length + 1, columnIndex].SetFill(customColorFill);
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            Array themeColorType = Enum.GetValues(typeof(ThemeColorType));
            Array colorShadeType = Enum.GetValues(typeof(ColorShadeType));

            for (int columnIndex = 0; columnIndex < 10; columnIndex++)
            {
                ThemeColorType colorType = (ThemeColorType)themeColorType.GetValue(columnIndex);

                IFill headerColorFill = new PatternFill(PatternType.Solid,
                      new ThemableColor(colorType),
                      PatternFill.Default.BackgroundColor);

                Assert.AreEqual(headerColorFill, cells[0, columnIndex].GetFill().Value);

                for (int rowIndex = 0; rowIndex < colorShadeType.Length; rowIndex++)
                {
                    ColorShadeType shadeType = (ColorShadeType)colorShadeType.GetValue(rowIndex);

                    IFill fill = new PatternFill(PatternType.Solid,
                      new ThemableColor(colorType, shadeType),
                      PatternFill.Default.BackgroundColor);

                    Assert.AreEqual(fill, cells[rowIndex + 1, columnIndex].GetFill().Value);
                }

                IFill customColorFill = new PatternFill(PatternType.Solid,
                     new ThemableColor(colors[columnIndex]),
                     PatternFill.Default.BackgroundColor);

                IFill test = cells[colorShadeType.Length + 1, columnIndex].GetFill().Value;
                Assert.AreEqual(customColorFill, test);
            }
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportPaletteColorsTest()
        {
            ImportFileAndAssert("ImportPaletteColorsTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportPaletteColorsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
