﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class HeaderFooterContentTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet worksheet = workbook.Worksheets.Add();
            worksheet.Cells[300, 5].SetValue("header footer content");
            worksheet.WorksheetPageSetup.PrintOptions.PrintGridlines = true;
            worksheet.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings = true;
            HeaderFooterSettings settings = worksheet.WorksheetPageSetup.HeaderFooterSettings;
            settings.DifferentFirstPage = true;
            settings.FirstPageHeader.LeftSection.Text = @"Let's test all fields:
        Ampersand-&&

Color-&KFF0000red
Automatic color-&K000000black
Theme color-&K05-024theme
&""Algerian,Regular""font family change
&""-,Bold Italic""normal font bold italic
&""-,Bold""normal font bold
";
            settings.FirstPageFooter.CenterSection.Text = @"
&""-,Italic""normal font italic,
&""-,Regular""normal font
false ampersand
&Uunderline
&Edouble underline
        tabs        and spaces    and new lines
&Eunderline off


new lines";

            settings.Header.RightSection.Text = @"&Sstrikethrough
&S&Ysubscript
&Xsuperscript
&Xsuperscript off
&17.17 font size
&Ddate
&Ttime
&Zfolder
&Ffile
&Aworksheet
picture
different new lines" + "\n\r\r\n\n\rthe end";
            settings.Footer.CenterSection.Text = "Page &\"-,Bold Italic\"&P &\"-,Regular\"from &\"-,Bold Italic\"&N";

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = workbook.ActiveWorksheet;
            Assert.IsTrue(worksheet.WorksheetPageSetup.PrintOptions.PrintGridlines);
            Assert.IsTrue(worksheet.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings);

            HeaderFooterSettings settings = worksheet.WorksheetPageSetup.HeaderFooterSettings;
            Assert.IsTrue(settings.DifferentFirstPage);
            Assert.IsFalse(settings.DifferentOddAndEvenPages);

            string expectedFirstHeader = @"Let's test all fields:
        Ampersand-&&

Color-&KFF0000red
Automatic color-&K000000black
Theme color-&K05-024theme
&""Algerian,Regular""font family change
&""-,Bold Italic""normal font bold italic
&""-,Bold""normal font bold
";
            string expectedFirstFooter = @"
&""-,Italic""normal font italic,
&""-,Regular""normal font
false ampersand
&Uunderline
&Edouble underline
        tabs        and spaces    and new lines
&Eunderline off


new lines";

            string expectedHeader = @"&Sstrikethrough
&S&Ysubscript
&Xsuperscript
&Xsuperscript off
&17.17 font size
&Ddate
&Ttime
&Zfolder
&Ffile
&Aworksheet
picture
different new lines" + "\n\r\r\n\n\rthe end";
            string expectedFooter = "Page &\"-,Bold Italic\"&P &\"-,Regular\"from &\"-,Bold Italic\"&N";

            TestHelper.AssertHeaderFooterSectionsTextAreEqual(expectedFirstHeader, settings.FirstPageHeader.LeftSection.Text);
            TestHelper.AssertHeaderFooterSectionsTextAreEqual(expectedFirstFooter, settings.FirstPageFooter.CenterSection.Text);
            TestHelper.AssertHeaderFooterSectionsTextAreEqual(expectedHeader, settings.Header.RightSection.Text);
            TestHelper.AssertHeaderFooterSectionsTextAreEqual(expectedFooter, settings.Footer.CenterSection.Text);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookHeaderFooterContentTest()
        {
            ImportFileAndAssert("ImportExportHeaderFooterContent.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportHeaderFooterContentTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
