﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class HeaderFooterSettingsTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            sheet1.Cells[300, 5].SetValue("sheet1 header footer test");
            sheet1.WorksheetPageSetup.PrintOptions.PrintGridlines = true;
            sheet1.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings = true;
            HeaderFooterSettings settings1 = sheet1.WorksheetPageSetup.HeaderFooterSettings;
            settings1.DifferentOddAndEvenPages = true;
            settings1.DifferentFirstPage = true;
            settings1.ScaleWithDocument = false;
            settings1.AlignWithPageMargins = false;
            settings1.FirstPageHeader.RightSection.Text = "first Header";
            settings1.FirstPageFooter.LeftSection.Text = "first footer";
            settings1.Header.CenterSection.Text = "odd header";
            settings1.Footer.CenterSection.Text = "&P is odd";
            settings1.Footer.RightSection.Text = "Page &P of &N";
            settings1.EvenPageHeader.CenterSection.Text = "even header";
            settings1.EvenPageFooter.CenterSection.Text = "&P is even";
            settings1.EvenPageFooter.LeftSection.Text = "Page &P of &N";

            Worksheet sheet2 = workbook.Worksheets.Add();
            sheet1.Cells[300, 5].SetValue("sheet2 header footer test");
            sheet1.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings = true;
            HeaderFooterSettings settings2 = sheet2.WorksheetPageSetup.HeaderFooterSettings;
            settings2.Header.LeftSection.Text = "&P";
            settings2.Header.CenterSection.Text = "OF";
            settings2.Header.RightSection.Text = "&N";

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Assert.AreEqual(workbook.Worksheets.Count, 2);

            Worksheet sheet1 = workbook.Worksheets[0];
            HeaderFooterSettings settings1 = sheet1.WorksheetPageSetup.HeaderFooterSettings;
            Assert.IsTrue(sheet1.WorksheetPageSetup.PrintOptions.PrintGridlines);
            Assert.IsTrue(sheet1.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings);
            Assert.IsTrue(settings1.DifferentOddAndEvenPages);
            Assert.IsTrue(settings1.DifferentFirstPage);
            Assert.IsFalse(settings1.ScaleWithDocument);
            Assert.IsFalse(settings1.AlignWithPageMargins);
            Assert.AreEqual(string.Empty, settings1.FirstPageHeader.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings1.FirstPageHeader.CenterSection.Text);
            Assert.AreEqual("first Header", settings1.FirstPageHeader.RightSection.Text);
            Assert.AreEqual("first footer", settings1.FirstPageFooter.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings1.FirstPageFooter.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings1.FirstPageFooter.RightSection.Text);
            Assert.AreEqual(string.Empty, settings1.Header.LeftSection.Text);
            Assert.AreEqual("odd header", settings1.Header.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings1.Header.RightSection.Text);
            Assert.AreEqual(string.Empty, settings1.Footer.LeftSection.Text);
            Assert.AreEqual("&P is odd", settings1.Footer.CenterSection.Text);
            Assert.AreEqual("Page &P of &N", settings1.Footer.RightSection.Text);
            Assert.AreEqual(string.Empty, settings1.EvenPageHeader.LeftSection.Text);
            Assert.AreEqual("even header", settings1.EvenPageHeader.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings1.EvenPageHeader.RightSection.Text);
            Assert.AreEqual("Page &P of &N", settings1.EvenPageFooter.LeftSection.Text);
            Assert.AreEqual("&P is even", settings1.EvenPageFooter.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings1.EvenPageFooter.RightSection.Text);

            Worksheet sheet2 = workbook.Worksheets[1];
            HeaderFooterSettings settings2 = sheet2.WorksheetPageSetup.HeaderFooterSettings;
            Assert.IsFalse(sheet2.WorksheetPageSetup.PrintOptions.PrintGridlines);
            Assert.IsTrue(sheet2.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings);
            Assert.IsFalse(settings2.DifferentOddAndEvenPages);
            Assert.IsFalse(settings2.DifferentFirstPage);
            Assert.IsTrue(settings2.ScaleWithDocument);
            Assert.IsTrue(settings2.AlignWithPageMargins);
            Assert.AreEqual(string.Empty, settings2.FirstPageHeader.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings2.FirstPageHeader.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings2.FirstPageHeader.RightSection.Text);
            Assert.AreEqual(string.Empty, settings2.FirstPageFooter.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings2.FirstPageFooter.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings2.FirstPageFooter.RightSection.Text);
            Assert.AreEqual("&P", settings2.Header.LeftSection.Text);
            Assert.AreEqual("OF", settings2.Header.CenterSection.Text);
            Assert.AreEqual("&N", settings2.Header.RightSection.Text);
            Assert.AreEqual(string.Empty, settings2.Footer.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings2.Footer.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings2.Footer.RightSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageHeader.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageHeader.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageHeader.RightSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageFooter.LeftSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageFooter.CenterSection.Text);
            Assert.AreEqual(string.Empty, settings2.EvenPageFooter.RightSection.Text);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookHeaderFooterSettingsTest()
        {
            ImportFileAndAssert("ImportExportHeaderFooterSettings.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportHeaderFooterSettingsTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
