﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class PageBreaksInPrintAreaTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();

            PrintArea printArea = sheet1.WorksheetPageSetup.PrintArea;
            printArea.SetPrintArea(new CellRange[]
            {
                new CellRange(1, 2, 3, 5),
                new CellRange(2, 4, 9, 8),
            });

            PageBreaks pageBreaks = sheet1.WorksheetPageSetup.PageBreaks;
            pageBreaks.TryInsertPageBreaks(3, 5);
            pageBreaks.TryInsertPageBreaks(1, 2);
            pageBreaks.TryInsertPageBreaks(11, 7);

            printArea.TryAddToPrintArea(new CellRange(10, 6, 14, 9));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            WorksheetPageSetup pageSetup = workbook.ActiveWorksheet.WorksheetPageSetup;
            PageBreaks pageBreaks = pageSetup.PageBreaks;

            IEnumerable<PageBreakInfo> horizontalBreaks = PageBreakInfo.GetPageBreakInfos(pageBreaks.HorizontalPageBreaks);
            Assert.IsTrue(PageBreakInfo.AreEqualPageBreaksCollections(horizontalBreaks, new PageBreakInfo[]
            {
                new PageBreakInfo(PageBreakType.Horizontal, 3, 2, 8),
                new PageBreakInfo(PageBreakType.Horizontal, 11),
            }));

            IEnumerable<PageBreakInfo> verticalBreaks = PageBreakInfo.GetPageBreakInfos(pageBreaks.VerticalPageBreaks);
            Assert.IsTrue(PageBreakInfo.AreEqualPageBreaksCollections(verticalBreaks, new PageBreakInfo[]
            {
                new PageBreakInfo(PageBreakType.Vertical, 5, 1, 9),
                new PageBreakInfo(PageBreakType.Vertical, 7),
            }));

            TestHelper.AssertCellRangeCollectionsAreEqual(pageSetup.PrintArea.Ranges, new CellRange[]
            {
                new CellRange(1, 2, 3, 5),
                new CellRange(2, 4, 9, 8),
                new CellRange(10, 6, 14, 9),
            });
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookPageBreaksInPrintAreaTest()
        {
            ImportFileAndAssert("ImportExportPageBreaksInPrintArea.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorkbookPageBreaksInPrintAreaTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
