﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class PageBreaksTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            PageBreaks firstPageBreaks = sheet1.WorksheetPageSetup.PageBreaks;
            firstPageBreaks.TryInsertPageBreaks(1, 1);
            firstPageBreaks.TryInsertPageBreaks(0, 4);
            firstPageBreaks.TryInsertPageBreaks(5, 0);
            firstPageBreaks.TryInsertPageBreaks(3, 1);

            Worksheet sheet2 = workbook.Worksheets.Add();
            PageBreaks secondPageBreaks = sheet2.WorksheetPageSetup.PageBreaks;
            secondPageBreaks.TryInsertHorizontalPageBreak(1, 1);
            secondPageBreaks.TryInsertVerticalPageBreak(1, 1);
            secondPageBreaks.TryInsertHorizontalPageBreak(0, 4);
            secondPageBreaks.TryInsertVerticalPageBreak(0, 4);
            secondPageBreaks.TryInsertHorizontalPageBreak(5, 0);
            secondPageBreaks.TryInsertVerticalPageBreak(5, 0);
            secondPageBreaks.TryInsertHorizontalPageBreak(3, 1);
            secondPageBreaks.TryInsertVerticalPageBreak(3, 1);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            this.AssertWorksheet(workbook.Worksheets[0]);
            this.AssertWorksheet(workbook.Worksheets[1]);
        }

        private void AssertWorksheet(Worksheet worksheet)
        {
            PageBreaks pageBreaks = worksheet.WorksheetPageSetup.PageBreaks;

            IEnumerable<PageBreakInfo> horizontalBreaks = PageBreakInfo.GetPageBreakInfos(pageBreaks.HorizontalPageBreaks);
            Assert.IsTrue(PageBreakInfo.AreEqualPageBreaksCollections(horizontalBreaks, new PageBreakInfo[]
            {
                new PageBreakInfo(PageBreakType.Horizontal, 1),
                new PageBreakInfo(PageBreakType.Horizontal, 5),
                new PageBreakInfo(PageBreakType.Horizontal, 3),
            }));

            IEnumerable<PageBreakInfo> verticalBreaks = PageBreakInfo.GetPageBreakInfos(pageBreaks.VerticalPageBreaks);
            Assert.IsTrue(PageBreakInfo.AreEqualPageBreaksCollections(verticalBreaks, new PageBreakInfo[]
            {
                new PageBreakInfo(PageBreakType.Vertical, 1),
                new PageBreakInfo(PageBreakType.Vertical, 4),
            }));
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookPageBreaksTest()
        {
            ImportFileAndAssert("ImportExportPageBreaks.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorkbookPageBreaksTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
