﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class PageSizeTests : ImportExportTestClassBase
    {
        private static readonly List<PaperTypes> papersToTest;

        static PageSizeTests()
        {
            papersToTest = new List<PaperTypes>()
            {
                PaperTypes.Letter, // id="1"
                PaperTypes.Letter, // id="2"
                PaperTypes.Tabloid, // id="3"
                PaperTypes.Ledger, // id="4"
                PaperTypes.Legal, // id="5"
                PaperTypes.Statement, // id="6"
                PaperTypes.Executive, // id="7"
                PaperTypes.A3, // id="8"
                PaperTypes.A4, // id="9"
                PaperTypes.A4, // id="10"
                PaperTypes.A5, // id="11"
                PaperTypes.B4, // id="12"
                PaperTypes.B5, // id="13"
                PaperTypes.Folio, // id="14"
                PaperTypes.Quarto, // id="15"

                PaperTypes.B4, // id="42"

                PaperTypes.Letter, // id="50"
                PaperTypes.Legal, // id="51"
            };
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            foreach (PaperTypes paperType in PageSizeTests.papersToTest)
            {
                workbook.Worksheets.Add().WorksheetPageSetup.PaperType = paperType;
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            int worksheetsCount = workbook.Worksheets.Count;

            Assert.AreEqual(papersToTest.Count, worksheetsCount);

            for (int index = 0; index < worksheetsCount; index++)
            {
                PaperTypes expectedPaper = PageSizeTests.papersToTest[index];
                PaperTypes actualPaper = workbook.Worksheets[index].WorksheetPageSetup.PaperType;

                Assert.AreEqual(expectedPaper, actualPaper);
            }
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookPaperSizesTest()
        {
            ImportFileAndAssert("ImportExportPaperSizes.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorkbookPaperSizesTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
