﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class PrintAreaTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            PrintArea printArea1 = sheet1.WorksheetPageSetup.PrintArea;
            printArea1.SetPrintArea(new CellRange[]
            {
                new CellRange(1, 2, 3, 5),
                new CellRange(2, 4, 9, 8),
            });
            printArea1.TryAddToPrintArea(new CellRange(10, 6, 14, 9));

            Worksheet sheet2 = workbook.Worksheets.Add();
            PrintArea printArea2 = sheet2.WorksheetPageSetup.PrintArea;
            printArea2.SetPrintArea(new CellRange[]
            {
                new CellRange(1, 2, 3, 5)
            });
            printArea2.TryAddToPrintArea(new CellRange(2, 4, 9, 8));
            printArea2.TryAddToPrintArea(new CellRange(10, 6, 14, 9));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            WorksheetPageSetup pageSetup1 = workbook.Worksheets[0].WorksheetPageSetup;
            TestHelper.AssertCellRangeCollectionsAreEqual(pageSetup1.PrintArea.Ranges, new CellRange[]
            {
                new CellRange(1, 2, 3, 5),
                new CellRange(2, 4, 9, 8),
                new CellRange(10, 6, 14, 9),
            });


            WorksheetPageSetup pageSetup2 = workbook.Worksheets[1].WorksheetPageSetup;
            TestHelper.AssertCellRangeCollectionsAreEqual(pageSetup2.PrintArea.Ranges, new CellRange[]
            {
                new CellRange(1, 2, 3, 5),
                new CellRange(10, 6, 14, 9),
            });

            WorksheetPageSetup pageSetup3 = workbook.Worksheets[2].WorksheetPageSetup;
            TestHelper.AssertCellRangeCollectionsAreEqual(pageSetup3.PrintArea.Ranges, Enumerable.Empty<CellRange>());
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookPrintAreaTest()
        {
            ImportFileAndAssert("ImportExportPrintArea.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorkbookPrintAreaTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
