﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Windows;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.PrintingTests
{
    [TestClass]
    public class WorksheetPageSetupTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            WorksheetPageSetup pageSetup1 = sheet1.WorksheetPageSetup;
            pageSetup1.CenterHorizontally = true;
            pageSetup1.CenterVertically = true;
            pageSetup1.Margins = new PageMargins(UnitHelper.InchToDip(2));
            pageSetup1.PageOrder = PageOrder.OverThenDown;
            pageSetup1.PageOrientation = PageOrientation.Landscape;
            pageSetup1.PaperType = PaperTypes.A5;
            pageSetup1.ScaleFactor = new Size(1.5, 1.5);
            pageSetup1.PrintOptions.PrintGridlines = true;
            pageSetup1.PrintOptions.PrintRowColumnHeadings = true;
            pageSetup1.FitToPages = true;
            pageSetup1.FitToPagesTall = 2;
            pageSetup1.FitToPagesWide = 2;

            Worksheet sheet2 = workbook.Worksheets.Add();
            WorksheetPageSetup pageSetup2 = sheet2.WorksheetPageSetup;
            pageSetup2.PaperType = PaperTypes.A3;
            pageSetup2.PrintOptions.PrintGridlines = true;
            pageSetup2.Margins = PageMargins.WideMargins;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            WorksheetPageSetup pageSetup1 = workbook.Worksheets[0].WorksheetPageSetup;
            Assert.IsTrue(pageSetup1.CenterHorizontally);
            Assert.IsTrue(pageSetup1.CenterVertically);
            Assert.AreEqual(pageSetup1.Margins, new PageMargins(UnitHelper.InchToDip(2)));
            Assert.AreEqual(pageSetup1.PageOrder, PageOrder.OverThenDown);
            Assert.AreEqual(pageSetup1.PageOrientation, PageOrientation.Landscape);
            Assert.AreEqual(pageSetup1.PaperType, PaperTypes.A5);
            Assert.AreEqual(pageSetup1.ScaleFactor, new Size(1.5, 1.5));
            Assert.IsTrue(pageSetup1.PrintOptions.PrintGridlines);
            Assert.IsTrue(pageSetup1.PrintOptions.PrintRowColumnHeadings);
            Assert.IsTrue(pageSetup1.FitToPages);
            Assert.AreEqual(2, pageSetup1.FitToPagesTall);
            Assert.AreEqual(2, pageSetup1.FitToPagesWide);

            WorksheetPageSetup pageSetup2 = workbook.Worksheets[1].WorksheetPageSetup;
            Assert.AreEqual(pageSetup2.PaperType, PaperTypes.A3);
            Assert.IsTrue(pageSetup2.PrintOptions.PrintGridlines);
            Assert.IsFalse(pageSetup2.PrintOptions.PrintRowColumnHeadings);
            Assert.AreEqual(pageSetup2.Margins, PageMargins.WideMargins);
            Assert.IsFalse(pageSetup2.FitToPages);
            Assert.AreEqual(1, pageSetup2.FitToPagesTall);
            Assert.AreEqual(1, pageSetup2.FitToPagesWide);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorkbookWorksheetPageSetupsTest()
        {
            ImportFileAndAssert("ImportExportWorksheetPageSetups.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorkbookWorksheetPageSetupsTest()
        {
            base.ArrangeWorkbookExportImportAssert();
        }
    }
}
