﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.ProtectionTests
{
    [TestClass]
    public class ProtectionIsLockedCellProperty : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Cells[0, 0, 1, 1].SetIsLocked(false);
            sheet1.Cells[2, 2].SetIsLocked(false);
            sheet1.Cells[3, 3].SetIsLocked(false);

            sheet1.Protect(string.Empty, null);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.IsTrue(sheet1.IsProtected);

            RangePropertyValue<bool> isLocked = sheet1.Cells[0, 0, 1, 1].GetIsLocked();
            Assert.IsFalse(isLocked.IsIndeterminate);
            Assert.AreEqual(false, isLocked.Value);

            isLocked = sheet1.Cells[2, 2].GetIsLocked();
            Assert.IsFalse(isLocked.IsIndeterminate);
            Assert.AreEqual(false, isLocked.Value);

            isLocked = sheet1.Cells[3, 3].GetIsLocked();
            Assert.IsFalse(isLocked.IsIndeterminate);
            Assert.AreEqual(false, isLocked.Value);

            isLocked = sheet1.Cells[2, 0, 3, 1].GetIsLocked();
            Assert.IsFalse(isLocked.IsIndeterminate);
            Assert.AreEqual(true, isLocked.Value);

            isLocked = sheet1.Cells[0, 2, 1, 3].GetIsLocked();
            Assert.IsFalse(isLocked.IsIndeterminate);
            Assert.AreEqual(true, isLocked.Value);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorksheetProtectionIsLockedCellPropertyTest()
        {
            ImportFileAndAssert("ImportExportLockedCells.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorksheetProtectionIsLockedCellPropertyTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
