﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Protection;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ProtectionOptionsWorksheetTests : ImportExportTestClassBase
    {
        private const string Sheet1Password = "qwerty";
        private static readonly WorksheetProtectionOptions sheet1Options = new WorksheetProtectionOptions(true, true, true, true, true, true, true, true, true);
        private static readonly WorksheetProtectionOptions sheet2Options = new WorksheetProtectionOptions(true, false, true, false, true, true, false, false, true);
        private static readonly WorksheetProtectionOptions sheet3Options = new WorksheetProtectionOptions(false, false, true, true, false, false, true, true, false);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();
            Worksheet sheet3 = workbook.Worksheets.Add();

            sheet1.Protect(Sheet1Password, sheet1Options);
            sheet2.Protect(string.Empty, sheet2Options);
            sheet3.Protect(string.Empty, sheet3Options);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];
            Worksheet sheet2 = workbook.Worksheets[1];
            Worksheet sheet3 = workbook.Worksheets[2];

            Assert.IsTrue(sheet1.IsProtected);
            Assert.AreEqual(sheet1Options, sheet1.ProtectionOptions);

            Assert.IsTrue(sheet2.IsProtected);
            Assert.AreEqual(sheet2Options, sheet2.ProtectionOptions);
            
            Assert.IsTrue(sheet3.IsProtected);
            Assert.AreEqual(sheet3Options, sheet3.ProtectionOptions);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportWorksheetProtectionOptionsTest()
        {
            ImportFileAndAssert("ImportExportWorksheetProtectionOptions2013.xlsx");
            ImportFileAndAssert("ImportExportWorksheetProtectionOptions2010.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportWorksheetProtectionOptionsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
