﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ProtectionWorksheetTests : ImportExportTestClassBase
    {
        private const string Sheet1Password = "qwerty";
        private const string Sheet2Password = "";

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();
            workbook.Worksheets.Add();
            
            sheet1.Protect(Sheet1Password, null);
            sheet2.Protect(Sheet2Password, null);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];
            Worksheet sheet2 = workbook.Worksheets[1];
            Worksheet sheet3 = workbook.Worksheets[2];

            Assert.IsTrue(sheet1.IsProtected);
            Assert.IsTrue(sheet2.IsProtected);
            Assert.IsFalse(sheet3.IsProtected);

            Assert.IsTrue(sheet1.Unprotect(Sheet1Password));
            Assert.IsTrue(sheet2.Unprotect(Sheet2Password));
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportProtectedWorkbookTest()
        {
            ImportFileAndAssert("ImportExportProtectedWorksheet2013.xlsx");
            ImportFileAndAssert("ImportExportProtectedWorksheet2010.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportProtectedWorkbookTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
