﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.RowAndColumnStyles
{
    [TestClass]
    public class ColumnFillThenClearTests : ImportExportTestClassBase
    {
        private readonly IFill fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent5, ColorShadeType.Shade3));
        private readonly CellRange columnRange = CellRange.FromColumnRange(0, 7);
        private readonly CellRange cellRange = new CellRange(12, 1, 30, 6);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            columns[columnRange].SetFill(fill);

            cells[cellRange].Clear(ClearType.All);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            IFill columnFill = columns[columnRange].GetFill().Value;
            Assert.AreEqual(fill, columnFill);

            IFill cellFill = cells[cellRange].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ImportColumnFillThenClearTest()
        {
            ImportFileAndAssert("RowAndColumnStyles/ColumnFillThenClear.xlsx");
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ExportAndImportColumnFillThenClearTest()
        {
            ArrangeWorkbookExportImportAssert();
        } 
    }
}
