﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.RowAndColumnStyles
{
    [TestClass]
    public class ColumnFillThenRowFillClearTests : ImportExportTestClassBase
    {
        private readonly IFill columnFill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3));
        private readonly IFill rowFill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4));
        private readonly CellRange columnRange = CellRange.FromColumnRange(0, 8);
        private readonly CellRange rowRange = CellRange.FromRowRange(7, 27);
        private readonly CellRange cellRange = new CellRange(10, 6, 23, 11);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            columns[columnRange].SetFill(columnFill);
            rows[rowRange].SetFill(rowFill);

            cells[cellRange].Clear(ClearType.All);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            IFill columnFill = columns[columnRange].GetFill().Value;
            Assert.AreEqual(this.columnFill, columnFill);

            IFill rowFill = rows[rowRange].GetFill().Value;
            Assert.AreEqual(this.rowFill, rowFill);

            IFill cellFill = cells[cellRange].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ImportColumnFillThenRowFillClearTest()
        {
            ImportFileAndAssert("RowAndColumnStyles/ColumnFillThenRowFillClear.xlsx");
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ExportAndImportColumnFillThenRowFillClearTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}