﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.RowAndColumnStyles
{
    [TestClass]
    public class RowFillThenClearTests : ImportExportTestClassBase
    {
        private readonly IFill fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade4));
        private readonly CellRange rowRange = CellRange.FromRowRange(0, 29);
        private readonly CellRange cellRange = new CellRange(8, 4, 22, 12);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            rows[rowRange].SetFill(fill);

            cells[cellRange].Clear(ClearType.All);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            IFill rowFill = rows[rowRange].GetFill().Value;
            Assert.AreEqual(fill, rowFill);

            IFill cellFill = cells[cellRange].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void RowFillThenClearTest()
        {
            ImportFileAndAssert("RowAndColumnStyles/RowFillThenClear.xlsx");
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ExportAndImportRowFillThenClearTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
