﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.RowAndColumnStyles
{
    [TestClass]
    public class RowFillThenColumnFillClearTests : ImportExportTestClassBase
    {
        private readonly IFill columnFill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade4));
        private readonly IFill rowFill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent4, ColorShadeType.Shade4));
        private readonly CellRange columnRange = CellRange.FromColumnRange(3, 10);
        private readonly CellRange rowRange = CellRange.FromRowRange(0, 34);
        private readonly CellRange cellRange = new CellRange(9, 1, 41, 17);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            rows[rowRange].SetFill(rowFill);
            columns[columnRange].SetFill(columnFill);

            cells[cellRange].Clear(ClearType.All);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            IFill columnFill = columns[columnRange].GetFill().Value;
            Assert.AreEqual(this.columnFill, columnFill);

            IFill rowFill = rows[rowRange].GetFill().Value;
            Assert.AreEqual(this.rowFill, rowFill);

            IFill cellFill = cells[cellRange].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ImportRowFillThenColumnFillClearTest()
        {
            ImportFileAndAssert("RowAndColumnStyles/RowFillThenColumnFillClear.xlsx");
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ExportAndImportRowFillThenColumnFillClearTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}