﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.RowAndColumnStyles
{
    [TestClass]
    public class RowsThenColumnsFillTests : ImportExportTestClassBase
    {
        private readonly IFill column1Fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent2, ColorShadeType.Shade4));
        private readonly IFill column2Fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent3, ColorShadeType.Shade3));
        private readonly string column3StyleName = "Good";
        private readonly IFill row1Fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Text2, ColorShadeType.Shade3));
        private readonly IFill row2Fill = PatternFill.CreateSolidFill(new ThemableColor(ThemeColorType.Accent1, ColorShadeType.Shade4));
        private readonly string row3StyleName = "Bad";

        private readonly CellRange column1Range = CellRange.FromColumn(1);
        private readonly CellRange column2Range = CellRange.FromColumn(3);
        private readonly CellRange column3Range = CellRange.FromColumn(5);

        private readonly CellRange row1Range = CellRange.FromRow(1);
        private readonly CellRange row2Range = CellRange.FromRow(3);
        private readonly CellRange row3Range = CellRange.FromRow(5);

        private readonly CellRange cellRange = new CellRange(0, 0, 10, 7);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            rows[row1Range].SetFill(row1Fill);
            rows[row2Range].SetFill(row2Fill);
            rows[row3Range].SetStyleName(row3StyleName);

            columns[column1Range].SetFill(column1Fill);
            columns[column2Range].SetFill(column2Fill);
            columns[column3Range].SetStyleName(column3StyleName);

            for (int row = cellRange.FromIndex.RowIndex; row <= cellRange.ToIndex.RowIndex; row++)
            {
                for (int column = cellRange.FromIndex.ColumnIndex; column <= cellRange.ToIndex.ColumnIndex; column++)
                {
                    cells[row, column].SetValue(row + column + 1);
                }
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Columns columns = workbook.Worksheets["Sheet1"].Columns;
            Rows rows = workbook.Worksheets["Sheet1"].Rows;
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            IFill column1Fill = columns[column1Range].GetFill().Value;
            Assert.AreEqual(this.column1Fill, column1Fill);

            IFill column2Fill = columns[column2Range].GetFill().Value;
            Assert.AreEqual(this.column2Fill, column2Fill);

            string column3StyleName = columns[column3Range].GetStyleName().Value;
            Assert.AreEqual(this.column3StyleName, column3StyleName);

            IFill row1Fill = rows[row1Range].GetFill().Value;
            Assert.AreEqual(this.row1Fill, row1Fill);

            IFill row2Fill = rows[row2Range].GetFill().Value;
            Assert.AreEqual(this.row2Fill, row2Fill);

            string row3StyleName = rows[row3Range].GetStyleName().Value;
            Assert.AreEqual(this.row3StyleName, row3StyleName);

            for (int row = cellRange.FromIndex.RowIndex; row <= cellRange.ToIndex.RowIndex; row++)
            {
                for (int column = cellRange.FromIndex.ColumnIndex; column <= cellRange.ToIndex.ColumnIndex; column++)
                {
                    var numberCellValue = cells[row, column].GetValue().Value as NumberCellValue;
                    Assert.AreEqual(row + column + 1, numberCellValue.Value);
                }
            }
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ImportRowsThenColumnsFillTest()
        {
            ImportFileAndAssert("RowAndColumnStyles/RowsThenColumnsFill.xlsx");
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        [TestCategory("ImportExport")]
        public void ExportAndImportRowsThenColumnsFillTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}