﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class RowColumnSizeTests : ImportExportTestClassBase
    {
        private static readonly double firstRowHeight = 91; //68.25;
        private static readonly double secondRowHeight = 10; //7.5;
        private static readonly double thirdRowHeight = 91; //68.25;
        private static readonly double fourthRowHeight = SpreadsheetDefaultValues.DefaultRowHeight; // default height

        private static readonly double firstColumnWidth = 285; //40;
        private static readonly double secondColumnWidth = 27; // 3.14;
        private static readonly double thirdColumnWidth = 65; // 3.14;

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            worksheet.Rows[0].SetHeight(new RowHeight(firstRowHeight, true));
            worksheet.Rows[1].SetHeight(new RowHeight(secondRowHeight, true));
            worksheet.Rows[2].SetHeight(new RowHeight(thirdRowHeight, false));
            worksheet.Rows[3].SetHeight(new RowHeight(SpreadsheetDefaultValues.DefaultRowHeight, true));

            worksheet.Columns[0].SetWidth(new ColumnWidth(firstColumnWidth, true));
            worksheet.Columns[1].SetWidth(new ColumnWidth(secondColumnWidth, true));
            worksheet.Columns[2].SetWidth(new ColumnWidth(thirdColumnWidth, false));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            this.AssertCorrectWorkbook(workbook);
        }

        internal override void AssertExportedBookImportedSpecificValuesCorrectly(Workbook workbook)
        {
            this.AssertCorrectWorkbook(workbook);

            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            //TODO: After the column width calculation is fixed this needs to be asserted for the ImportVariedRowColumnSizeTest as well.
            ColumnSelection firstColumn = worksheet.Columns[0];
            double retrievedFirstColumnWidth = firstColumn.GetWidth().Value.Value;
            Assert.AreEqual(firstColumnWidth, retrievedFirstColumnWidth);

            ColumnSelection secondColumn = worksheet.Columns[1];
            double retrievedSecondColumnWidth = secondColumn.GetWidth().Value.Value;
            Assert.AreEqual(secondColumnWidth, retrievedSecondColumnWidth);

            ColumnSelection thirdColumn = worksheet.Columns[2];
            double retrievedThirdColumnWidth = thirdColumn.GetWidth().Value.Value;
            Assert.AreEqual(thirdColumnWidth, retrievedThirdColumnWidth);
        }

        private void AssertCorrectWorkbook(Workbook workbook)
        {
            Worksheet worksheet = workbook.Worksheets["Sheet1"];

            RowSelection firstRow = worksheet.Rows[0];
            double retrievedFirstRowHeight = firstRow.GetHeight().Value.Value;
            Assert.AreEqual(firstRowHeight, retrievedFirstRowHeight);
            bool retrievedFirstRowIsCustom = firstRow.GetHeight().Value.IsCustom;
            Assert.AreEqual(true, retrievedFirstRowIsCustom);

            RowSelection secondRow = worksheet.Rows[1];
            double retrievedSecondRowHeight = secondRow.GetHeight().Value.Value;
            Assert.AreEqual(secondRowHeight, retrievedSecondRowHeight);
            bool retrievedSecondRowIsCustom = secondRow.GetHeight().Value.IsCustom;
            Assert.AreEqual(true, retrievedSecondRowIsCustom);

            RowSelection thuirdRow = worksheet.Rows[2];
            double retrievedThirdRowHeight = thuirdRow.GetHeight().Value.Value;
            Assert.AreEqual(thirdRowHeight, retrievedThirdRowHeight);
            bool retrievedThirdRowIsCustom = thuirdRow.GetHeight().Value.IsCustom;
            Assert.AreEqual(true, retrievedThirdRowIsCustom);

            RowSelection fourthRow = worksheet.Rows[3];
            double retrievedFourthRowHeight = fourthRow.GetHeight().Value.Value;
            Assert.AreEqual(fourthRowHeight, retrievedFourthRowHeight);
            bool retrievedFourthRowIsCustom = fourthRow.GetHeight().Value.IsCustom;
            Assert.AreEqual(true, retrievedFourthRowIsCustom);

            RowSelection fifthRow = worksheet.Rows[4];
            double retrievedFifthRowHeight = fifthRow.GetHeight().Value.Value;
            Assert.AreEqual(SpreadsheetDefaultValues.DefaultRowHeight, retrievedFifthRowHeight);
            bool retrievedFifthRowIsCustom = fifthRow.GetHeight().Value.IsCustom;
            Assert.AreEqual(false, retrievedFifthRowIsCustom);

            ColumnSelection firstColumn = worksheet.Columns[0];
            double retrievedFirstColumnWidth = firstColumn.GetWidth().Value.Value;
            Assert.AreEqual(firstColumnWidth, retrievedFirstColumnWidth);
            bool retrievedFirstColumnIsCustom = firstColumn.GetWidth().Value.IsCustom;
            Assert.AreEqual(true, retrievedFirstColumnIsCustom);

            ColumnSelection secondColumn = worksheet.Columns[1];
            double retrievedSecondColumnWidth = secondColumn.GetWidth().Value.Value;
            Assert.AreEqual(secondColumnWidth, retrievedSecondColumnWidth);
            bool retrievedSecondColumnIsCustom = secondColumn.GetWidth().Value.IsCustom;
            Assert.AreEqual(true, retrievedSecondColumnIsCustom);

            ColumnSelection thirdColumn = worksheet.Columns[2];
            double retrievedThirdColumnWidth = thirdColumn.GetWidth().Value.Value;
            Assert.AreEqual(thirdColumnWidth, retrievedThirdColumnWidth);
            bool retrievedThirdColumnIsCustom = thirdColumn.GetWidth().Value.IsCustom;
            Assert.AreEqual(false, retrievedThirdColumnIsCustom);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportVariedRowColumnSizeTest()
        {
            ImportFileAndAssert("ImportRowColumnSizeTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportVariedRowColumnSizeTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
