﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class CustomListSortTests : ImportExportTestClassBase
    {
        private readonly CellRange sortRange = new CellRange(0, 0, 14, 3);
        private readonly string[] values0 = new string[] { "Sun", "Mon", "Wed", "Fri", "Fri", "Tue", "Tue", "Thu", "Thu", "Sat", "Sat", "Sat", "Wed", "Wed", "Sun" };
        private readonly string[] values1 = new string[] { "Jan", "Mar", "Apr", "Jul", "Jun", "Mar", "Mar", "Apr", "May", "May", "Feb", "Jun", "Mar", "Mat", "Jun" };
        private readonly string[] values2 = new string[] { "S", "M", "L", "XL", "XS", "M", "M", "L", "L", "XS", "S", "XL", "S", "XS", "XL" };
        private readonly string[] values3 = new string[] { "mm", "cm", "m", "cm", "cm", "mm", "m", "mm", "m", "cm", "cm", "m", "mm", "cm", "mm" };
        private readonly string[] customList0 = new string[] {  "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
        private readonly string[] customList1 = new string[] { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
        private readonly string[] customList2 = new string[] { "XS", "S", "M", "L", "XL" };
        private readonly string[] customList3 = new string[] { "mm", "cm", "m" };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < sortRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue(values0[i]);
                sheet1.Cells[i, 1].SetValue(values1[i]);
                sheet1.Cells[i, 2].SetValue(values2[i]);
                sheet1.Cells[i, 3].SetValue(values3[i]);
            }

            ISortCondition condition0 = new CustomValuesSortCondition(0, customList0, SortOrder.Ascending);
            ISortCondition condition1 = new CustomValuesSortCondition(1, customList1, SortOrder.Ascending);
            ISortCondition condition2 = new CustomValuesSortCondition(2, customList2, SortOrder.Ascending);
            ISortCondition condition3 = new CustomValuesSortCondition(3, customList3, SortOrder.Ascending);

            sheet1.SortState.Set(sortRange, condition0, condition1, condition2, condition3);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);

            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();

            AssertCustomListSortCondition(conditions[0], customList0);
            AssertCustomListSortCondition(conditions[1], customList1);
            AssertCustomListSortCondition(conditions[2], customList2);
            AssertCustomListSortCondition(conditions[3], customList3);
        }

        private static void AssertCustomListSortCondition(ISortCondition condition, string[] expectedCustomList)
        {
            CustomValuesSortCondition customListCondition = condition as CustomValuesSortCondition;
            Assert.IsNotNull(customListCondition, "Should be a custom list sort condition");

            CollectionAssert.AreEquivalent(expectedCustomList, customListCondition.CustomList);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportCustomListTest()
        {
            ImportFileAndAssert("ImportExportCustomListSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportCustomListTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}