﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class FilterAndSortTests : ImportExportTestClassBase
    {
        private readonly CellRange filterRange = new CellRange(1, 1, 15, 3);
        private readonly CellRange sortRange = new CellRange(1, 4, 10, 6);
        private readonly PatternFill orangeFill = new PatternFill(PatternType.Solid, Color.FromArgb(255, 255, 192, 0), Colors.Black);
        private readonly PatternFill redFill = new PatternFill(PatternType.Solid, Color.FromArgb(255, 192, 0, 0), Colors.Black);
        private readonly ThemableColor greenColor = new ThemableColor(Color.FromArgb(255, 0, 176, 80));
        private readonly ThemableColor blueColor = new ThemableColor(Color.FromArgb(255, 0, 176, 240));
        private readonly string[] values = new string[]{"3", "4", "6", "7", "8", "9", "10" };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < filterRange.RowCount; i++)
            {
                int index = i + 1;
                int value = 15 - i;
                sheet1.Cells[index, 1].SetValue(index);
                sheet1.Cells[index, 2].SetValue(value);
                sheet1.Cells[index, 3].SetValue(index);
                sheet1.Cells[index, 4].SetValue(value);
                sheet1.Cells[index, 5].SetValue(index);
                sheet1.Cells[index, 6].SetValue(value);
            }

            for (int i = 0; i < 10; i++)
            {
                sheet1.Cells[i + 1, 1].SetFill(orangeFill);
                if (i > 2)
                {
                    sheet1.Cells[i + 1, 2].SetForeColor(greenColor);
                    sheet1.Cells[i + 1, 5].SetFill(redFill);
                }
            }

            sheet1.Cells[3, 2].SetForeColor(greenColor);
            sheet1.Cells[4, 6].SetForeColor(blueColor);
            sheet1.Cells[6, 6].SetForeColor(blueColor);
            sheet1.Cells[8, 6].SetForeColor(blueColor);
            sheet1.Cells[10, 6].SetForeColor(blueColor);

            sheet1.Filter.FilterRange = filterRange;
            sheet1.Filter.SetFilter(new FillColorFilter(0, orangeFill));
            sheet1.Filter.SetFilter(new ForeColorFilter(1, greenColor));
            ValuesCollectionFilter filter = new ValuesCollectionFilter(2, values, null);
            sheet1.Filter.SetFilter(filter);

            ISortCondition condition0 = new ValuesSortCondition(0, SortOrder.Ascending);
            ISortCondition condition1 = new FillColorSortCondition(1, orangeFill, SortOrder.Ascending);
            ISortCondition condition2 = new ForeColorSortCondition(2, blueColor, SortOrder.Descending);

            sheet1.SortState.Set(sortRange, condition0, condition1, condition2);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];
            
            Assert.AreEqual(filterRange, sheet1.Filter.FilterRange);
            
            Assert.IsInstanceOfType(sheet1.Filter.GetFilter(0), typeof(FillColorFilter));
            Assert.AreEqual(orangeFill, ((FillColorFilter)sheet1.Filter.GetFilter(0)).Fill);

            Assert.IsInstanceOfType(sheet1.Filter.GetFilter(1), typeof(ForeColorFilter));
            Assert.AreEqual(greenColor, ((ForeColorFilter)sheet1.Filter.GetFilter(1)).Color);

            Assert.IsInstanceOfType(sheet1.Filter.GetFilter(2), typeof(ValuesCollectionFilter));
            CollectionAssert.AreEquivalent(values, ((ValuesCollectionFilter)sheet1.Filter.GetFilter(2)).StringValues.ToArray()); 

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);
            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();
            
            Assert.IsInstanceOfType(conditions[0], typeof(ValuesSortCondition));
            Assert.AreEqual(SortOrder.Ascending, ((ValuesSortCondition)conditions[0]).SortOrder);

            Assert.IsInstanceOfType(conditions[1], typeof(FillColorSortCondition));
            Assert.AreEqual(SortOrder.Ascending, ((FillColorSortCondition)conditions[1]).SortOrder);
            Assert.AreEqual(redFill, ((FillColorSortCondition)conditions[1]).Fill);

            Assert.IsInstanceOfType(conditions[2], typeof(ForeColorSortCondition));
            Assert.AreEqual(SortOrder.Descending, ((ForeColorSortCondition)conditions[2]).SortOrder);
            Assert.AreEqual(blueColor, ((ForeColorSortCondition)conditions[2]).Color);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportFilterAndSortTest()
        {
            ImportFileAndAssert("ImportExportFilterAndSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportFilterAndSortTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}