﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;
using System.Windows.Media;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class ForeColorSortTests : ImportExportTestClassBase
    {
        private readonly CellRange sortRange = new CellRange(0, 0, 9, 1);
        private readonly ThemableColor red = ThemableColor.FromArgb(255, 255, 0, 0);
        private readonly ThemableColor orange = ThemableColor.FromArgb(255, 255, 192, 0);
        private readonly ThemableColor blue = ThemableColor.FromArgb(255, 0, 176, 240);
        private readonly ThemableColor black = new ThemableColor(Colors.Black);

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < sortRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue(i + 1);
                sheet1.Cells[i, 1].SetValue(i + 1);
                if (i < 5)
                {
                    sheet1.Cells[i, 0].SetForeColor(red);
                }
                else
                {
                    sheet1.Cells[i, 0].SetForeColor(orange);
                }

                if (i > 8)
                {
                    sheet1.Cells[i, 1].SetForeColor(blue);
                }
            }

            sheet1.SortState.Set(sortRange, 
                new ForeColorSortCondition(0, orange, SortOrder.Ascending),
                new ForeColorSortCondition(1, black, SortOrder.Descending));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);

            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();

            AssertValuesSortCondition(conditions[0], orange, SortOrder.Ascending);
            AssertValuesSortCondition(conditions[1], black, SortOrder.Descending);
        }

        private static void AssertValuesSortCondition(ISortCondition condition, ThemableColor color, SortOrder expectedSortOrder)
        {
            ForeColorSortCondition foreColorSortCondition = condition as ForeColorSortCondition;
            Assert.IsNotNull(foreColorSortCondition, "Should be a ForeColorSortCondition");

            Assert.AreEqual(color, foreColorSortCondition.Color);
            Assert.AreEqual(expectedSortOrder, foreColorSortCondition.SortOrder);
        }
        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportForeColorSortTest()
        {
            ImportFileAndAssert("ImportExportForeColorSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportForeColorSortTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}