﻿using System;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class GradientFillSortTests : ImportExportTestClassBase
    {
        private readonly CellRange sortRange = new CellRange(0, 0, 9, 1);
        private readonly GradientFill blueFill = new GradientFill(GradientType.Horizontal, Color.FromArgb(255, 91, 155, 213), Color.FromArgb(255, 255, 255, 255));
        private readonly GradientFill greenFill = new GradientFill(GradientType.DiagonalDown, Color.FromArgb(255, 255, 192, 0), Color.FromArgb(255, 112, 173, 71));
        private readonly GradientFill redFill = new GradientFill(GradientType.FromCenter, Color.FromArgb(255, 255, 0, 0), Color.FromArgb(255, 237, 125, 49));

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < sortRange.RowCount; i++)
            {
                if (i < 5)
                {
                    sheet1.Cells[i, 0].SetFill(blueFill);
                }
                else
                {
                    sheet1.Cells[i, 0].SetFill(greenFill);
                }

                if (i > 7)
                {
                    sheet1.Cells[i, 1].SetFill(redFill);
                }
            }

            sheet1.SortState.Set(sortRange, 
                new FillColorSortCondition(0, greenFill, SortOrder.Ascending),
                new FillColorSortCondition(1, PatternFill.Default, SortOrder.Descending));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);

            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();

            AssertFillSortCondition(conditions[0], greenFill, SortOrder.Ascending);
            AssertFillSortCondition(conditions[1], PatternFill.Default, SortOrder.Descending);
        }

        private static void AssertFillSortCondition(ISortCondition condition, IFill fill, SortOrder expectedSortOrder)
        {
            FillColorSortCondition fillColorSortCondition = condition as FillColorSortCondition;
            Assert.IsNotNull(fillColorSortCondition, "Should be a FillColorSortCondition");

            Assert.AreEqual(fill, fillColorSortCondition.Fill);
            Assert.AreEqual(expectedSortOrder, fillColorSortCondition.SortOrder);
        }
        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportGradientFillSortTest()
        {
            ImportFileAndAssert("ImportExportGradientFillSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportGradientFillSortTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}