﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class ValuesAndCustomListSortTests : ImportExportTestClassBase
    {
        private readonly CellRange sortRange = new CellRange(0, 0, 19, 1);
        private readonly string[] values = new string[] { "S", "M", "S", "S", "M", "M", "XS", "XS", "L", "L", "M", "S", "S", "L", "XL", "L", "XL", "L", "L", "M" };
        private readonly string[] customList = new string[] { "XS", "S", "M", "L", "XL" };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < sortRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue(i + 1);
                sheet1.Cells[i, 1].SetValue(values[i]);
            }

            ISortCondition condition1 = new CustomValuesSortCondition(1, customList, SortOrder.Ascending);
            ISortCondition condition2 = new ValuesSortCondition(0, SortOrder.Descending);

            sheet1.SortState.Set(sortRange, condition1, condition2);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);

            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();

            AssertCustomListSortCondition(conditions[0], customList);
            AssertValuesSortCondition(conditions[1], SortOrder.Descending);
        }

        private static void AssertCustomListSortCondition(ISortCondition condition, string[] expectedCustomList)
        {
            CustomValuesSortCondition customListCondition = condition as CustomValuesSortCondition;
            Assert.IsNotNull(customListCondition, "Should be a custom list sort condition");

            CollectionAssert.AreEquivalent(expectedCustomList, customListCondition.CustomList);
        }

        private static void AssertValuesSortCondition(ISortCondition condition, SortOrder expectedSortOrder)
        {
            ValuesSortCondition valuesCondition = condition as ValuesSortCondition;
            Assert.IsNotNull(valuesCondition, "Should be a value sort condition");
           
            Assert.AreEqual(expectedSortOrder, valuesCondition.SortOrder);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportValuesAndCustomListTest()
        {
            ImportFileAndAssert("ImportExportValuesAndCustomListSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportValuesAndCustomListTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}