﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.SortingTests
{
    [TestClass]
    public class ValuesSortTests : ImportExportTestClassBase
    {
        private readonly CellRange sortRange = new CellRange(0, 0, 14, 2);
        private readonly int[] values0 = new int[] { 1, 2, 1, 1, 3, 2, 2, 3, 1, 2, 3, 3, 4, 4, 1 };
        private readonly int[] values1 = new int[] { 10, 8, 12, 11, 12, 8, 7, 12, 10, 8, 13, 11, 18, 9, 10 };
        private readonly int[] values2 = new int[] { 1, 3, 1, 3, 1, 3, 5, 3, 2, 2, 1, 3, 5, 5, 1 };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            for (int i = 0; i < sortRange.RowCount; i++)
            {
                sheet1.Cells[i, 0].SetValue(values0[i]);
                sheet1.Cells[i, 1].SetValue(values1[i]);
                sheet1.Cells[i, 2].SetValue(values2[i]);
            }

            ISortCondition condition0 = new ValuesSortCondition(0, SortOrder.Descending);
            ISortCondition condition1 = new ValuesSortCondition(1, SortOrder.Ascending);
            ISortCondition condition2 = new ValuesSortCondition(2, SortOrder.Descending);

            sheet1.SortState.Set(sortRange, condition0, condition1, condition2);

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet sheet1 = workbook.Worksheets[0];

            Assert.AreEqual(sortRange, sheet1.SortState.SortRange);

            ISortCondition[] conditions = sheet1.SortState.SortConditions.ToArray();

            AssertValuesSortCondition(conditions[0], SortOrder.Descending);
            AssertValuesSortCondition(conditions[1], SortOrder.Ascending);
            AssertValuesSortCondition(conditions[2], SortOrder.Descending);
        }

        private static void AssertValuesSortCondition(ISortCondition condition, SortOrder expectedSortOrder)
        {
            ValuesSortCondition valuesCondition = condition as ValuesSortCondition;
            Assert.IsNotNull(valuesCondition, "Should be a value sort condition");

            Assert.AreEqual(expectedSortOrder, valuesCondition.SortOrder);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportValueSortTest()
        {
            ImportFileAndAssert("ImportExportValueSortTest.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportValueSortTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}