﻿using System;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Theming;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class ThemeTests : ImportExportTestClassBase
    {
        private readonly static string themeName = "ExpenseReport";

        private readonly static Color[] colors = new Color[] 
        {
            Color.FromArgb(255, 0, 0, 0),
            Color.FromArgb(255, 255, 255, 255),
            Color.FromArgb(255, 65, 65, 65),
            Color.FromArgb(255, 240, 240, 240),
            Color.FromArgb(255, 116, 202, 218),
            Color.FromArgb(255, 146, 204, 70),
            Color.FromArgb(255, 241, 96, 61),
            Color.FromArgb(255, 143, 145, 158),
            Color.FromArgb(255, 141, 119, 251),
            Color.FromArgb(255, 91, 119, 153),
            Color.FromArgb(255, 5, 99, 193),
            Color.FromArgb(255, 149, 79, 114)
        };

        private readonly static string[] fontFamilies = new[] 
        {
            "Cambria", 
            "Segoe UI"
        };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            
            ThemeColorScheme colorScheme = new ThemeColorScheme(themeName,
                  colors[0],
                  colors[1],
                  colors[2],
                  colors[3],
                  colors[4],
                  colors[5],
                  colors[6],
                  colors[7],
                  colors[8],
                  colors[9],
                  colors[10],
                  colors[11]);

            ThemeFontScheme fontScheme = new ThemeFontScheme(themeName, fontFamilies[0], fontFamilies[1]);

            DocumentTheme theme = new DocumentTheme(themeName, colorScheme, fontScheme);

            workbook.Theme = theme;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            AssertColorScheme(workbook.Theme.ColorScheme);
            AssertFontScheme(workbook.Theme.FontScheme);
        }

        public static void AssertColorScheme(ThemeColorScheme colorScheme)
        {
            Array themeColorTypes = Enum.GetValues(typeof(ThemeColorType));
            for (int i = 0; i < themeColorTypes.Length; i++)
			{
                ThemeColorType themeColorType = (ThemeColorType)themeColorTypes.GetValue(i);
                Assert.AreEqual(colors[i], colorScheme[themeColorType].Color);
			}
        }
        public static void AssertFontScheme(ThemeFontScheme fontScheme)
        {
            Assert.AreEqual(fontFamilies[0], fontScheme[ThemeFontType.Major][FontLanguageType.Latin].FontFamily.Source);
            Assert.AreEqual(fontFamilies[1], fontScheme[ThemeFontType.Minor][FontLanguageType.Latin].FontFamily.Source);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportThemeTest()
        {
            ImportFileAndAssert("ImportExportThemeTestDocument.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportThemeTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}