﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class FreezeFirstColumnTests : ImportExportTestClassBase
    {
        private SelectionState CreateSelection()
        {
            CellIndex selectionActiveCellIndex = new CellIndex(11, 0); // A12
            List<CellRange> sqRef = new List<CellRange>()
            {
                new CellRange(new CellIndex(11, 0), new CellIndex(14, 5)),
            };

            SelectionState expectedSelectionState = new SelectionState(sqRef, selectionActiveCellIndex, ViewportPaneType.HorizontalScrollable);

            return expectedSelectionState;
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)workbook.ActiveWorksheet).ViewState;

            Pane expectedPane = new Pane(new CellIndex(0, 1), 1, 0, ViewportPaneType.HorizontalScrollable, PaneState.Frozen);
            worksheetViewState.Pane = expectedPane;

            SelectionState expectedSelectionState = this.CreateSelection();
            worksheetViewState.SelectionState = expectedSelectionState;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetFreezeFirstColumnName];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertScaleFactorIsEqualTo(100);
            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(0, 0));

            Pane expectedPane = new Pane(new CellIndex(0, 1), 1, 0, ViewportPaneType.HorizontalScrollable, PaneState.Frozen);

            helper.AssertPaneIsEqualTo(expectedPane);

            SelectionState expectedSelectionState = this.CreateSelection();

            helper.AssertSelectionIsEqualTo(expectedSelectionState);
        }
  
        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezeFirstColumnWithSelection_WhenImported_IsCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezeFirstColumnWithSelection_WhenCreatedAndExported_IsImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}