﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests.ViewStateTests
{
    [TestClass]
    public class FreezePanesMethodTests : ImportExportTestClassBase
    {
        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = workbook.ActiveWorksheet.ViewState;

            worksheetViewState.FreezePanes(new CellIndex(1, 2), 5, 3, new CellIndex(17, 10));

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetFreezePanesMethod];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(1, 2));

            Pane expectedPane = new Pane(new CellIndex(17, 10), 3, 5, ViewportPaneType.Scrollable, PaneState.Frozen);
           
            helper.AssertPaneIsEqualTo(expectedPane);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezePanesWithMultipleSelection_WhenImported_AreCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezePanesWithMultipleSelection_WhenCreatedAndExported_AreImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}
