﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class FreezePanesTests : ImportExportTestClassBase
    {
        private SelectionState CreateSelection()
        {
            CellIndex selectionActiveCellIndex = new CellIndex(6, 5); // F7
            List<CellRange> sqRef = new List<CellRange>()
            {
                new CellRange(new CellIndex(3, 1), new CellIndex(11, 6)),
                new CellRange(new CellIndex(3, 17), new CellIndex(10, 19)),
                new CellRange(new CellIndex(27, 1), new CellIndex(32, 5)),
                new CellRange(new CellIndex(6, 5), new CellIndex(21, 11)),
            };

            SelectionState expectedSelectionState = new SelectionState(sqRef, selectionActiveCellIndex, ViewportPaneType.Scrollable);
           
            return expectedSelectionState;
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)workbook.ActiveWorksheet).ViewState;

            Pane expectedPane = new Pane(new CellIndex(17, 4), 3, 5, ViewportPaneType.Scrollable, PaneState.Frozen);
            worksheetViewState.Pane = expectedPane;

            SelectionState expectedSelectionState = this.CreateSelection();
            worksheetViewState.SelectionState = expectedSelectionState;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetFreezePanesName];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertScaleFactorIsEqualTo(120);
            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(0, 0));

            Pane expectedPane = new Pane(new CellIndex(17, 4), 3, 5, ViewportPaneType.Scrollable, PaneState.Frozen);
           
            helper.AssertPaneIsEqualTo(expectedPane);

            SelectionState expectedSelectionState = this.CreateSelection();

            helper.AssertSelectionIsEqualTo(expectedSelectionState);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezePanesWithMultipleSelection_WhenImported_AreCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezePanesWithMultipleSelection_WhenCreatedAndExported_AreImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}