﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class FreezeTopRowTests : ImportExportTestClassBase
    {
        private SelectionState CreateSelection()
        {
            CellIndex selectionActiveCellIndex = new CellIndex(12, 2); // C13
            List<CellRange> sqRef = new List<CellRange>()
            {
                new CellRange(new CellIndex(5, 4), new CellIndex(15, 9)),
                new CellRange(new CellIndex(12, 2), new CellIndex(23, 7)),
            };

            SelectionState expectedSelectionState = new SelectionState(sqRef, selectionActiveCellIndex, ViewportPaneType.VerticalScrollable);

            return expectedSelectionState;
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)workbook.ActiveWorksheet).ViewState;

            Pane expectedPane = new Pane(new CellIndex(1, 0), 0, 1, ViewportPaneType.VerticalScrollable, PaneState.Frozen);
            worksheetViewState.Pane = expectedPane;

            SelectionState expectedSelectionState = this.CreateSelection();
            worksheetViewState.SelectionState = expectedSelectionState;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetFreezeTopRowName];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertScaleFactorIsEqualTo(100);
            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(0, 0));

            Pane expectedPane = new Pane(new CellIndex(1, 0), 0, 1, ViewportPaneType.VerticalScrollable, PaneState.Frozen);

            helper.AssertPaneIsEqualTo(expectedPane);

            SelectionState expectedSelectionState = this.CreateSelection();

            helper.AssertSelectionIsEqualTo(expectedSelectionState);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezeTopWithMultipleSelection_WhenImported_AreCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void FreezeTopWithMultipleSelection_WhenCreatedAndExported_AreImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}