﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class MultipleSelectionTests : ImportExportTestClassBase
    {
        private SelectionState CreateSelection()
        {
            CellIndex selectionActiveCellIndex = new CellIndex(3, 3); // D4
            List<CellRange> sqRef = new List<CellRange>()
            {
                new CellRange(new CellIndex(0, 0), new CellIndex(2, 2)),
                new CellRange(new CellIndex(1, 1), new CellIndex(4, 3)),
                new CellRange(new CellIndex(2, 2), new CellIndex(6, 4)),
                new CellRange(new CellIndex(3, 3), new CellIndex(7, 5)),
            };

            SelectionState expectedSelectionState = new SelectionState(sqRef, selectionActiveCellIndex, ViewportPaneType.Fixed);
            
            return expectedSelectionState;
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)workbook.ActiveWorksheet).ViewState;

            SelectionState expectedSelectionState = this.CreateSelection();
            worksheetViewState.SelectionState = expectedSelectionState;

            return workbook;
        }
        
        public override void AssertWorkbookImportedCorrectly(Model.Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetMultipleSelectionName];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertScaleFactorIsEqualTo(130);
            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(0, 0));

            SelectionState expectedSelectionState = this.CreateSelection();

            helper.AssertSelectionIsEqualTo(expectedSelectionState);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void MultipleSelection_WhenImported_IsCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void MultipleSelection_WhenCreatedAndExported_IsImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}