﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    [TestClass]
    public class SelectionTests : ImportExportTestClassBase
    {
        private SelectionState CreateSelection()
        {
            CellIndex selectionActiveCellIndex = new CellIndex(2, 1); // B3
            CellRange selectionCellRange = new CellRange(selectionActiveCellIndex, new CellIndex(11, 4)); // B3:E12
            
            SelectionState expectedSelectionState = new SelectionState(new List<CellRange> { selectionCellRange }, selectionActiveCellIndex, ViewportPaneType.Fixed);
            
            return expectedSelectionState;
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)workbook.ActiveWorksheet).ViewState;

            SelectionState expectedSelectionState = this.CreateSelection();
            worksheetViewState.SelectionState = expectedSelectionState;

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Worksheet worksheet = (Worksheet)workbook.Sheets[ViewStateTestHelper.SheetSingleSelectionName];
            WorksheetViewState worksheetViewState = (WorksheetViewState)((ISheet)worksheet).ViewState;

            ViewStateTestHelper helper = new ViewStateTestHelper(worksheetViewState);

            helper.AssertScaleFactorIsEqualTo(50);
            helper.AssertTopLeftCellIndexIsEqualTo(new CellIndex(0, 0));

            SelectionState expectedSelectionState = this.CreateSelection();

            helper.AssertSelectionIsEqualTo(expectedSelectionState);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void SingleSelection_WhenImported_IsCorrect()
        {
            this.ImportFileAndAssert(ViewStateTestHelper.ViewStateTestDocument);
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void SingleSelection_WhenCreatedAndExported_IsImportedCorrectly()
        {
            this.ArrangeWorkbookExportImportAssert();
        }
    }
}