﻿using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
	[TestClass]
	public class NameConverterTests
	{
		[TestMethod]
		public void ConvertRowIndexToNameTest()
		{
			for (int i = 0; i < SpreadsheetDefaultValues.RowCount; i++)
			{
				string rowName = NameConverter.ConvertRowIndexToName(i);
				Assert.AreEqual((i + 1).ToString(), rowName);
			}
		}

		[TestMethod]
		public void RowNameToIndexAndRowIndexToNameConversionTest()
		{
			for (int i = 0; i < SpreadsheetDefaultValues.RowCount; i++)
			{
				string rowName = NameConverter.ConvertRowIndexToName(i);
				Assert.AreEqual(i, NameConverter.ConvertRowNameToIndex(rowName));
			}
		}

		[TestMethod]
		public void ConvertColumnIndexToNameTest()
		{
			List<string> allValidColumnNames = NameGenerator.GetAllColumnNames();

			for (int i = 0; i < allValidColumnNames.Count; i++)
			{
				Assert.AreEqual(allValidColumnNames[i], NameConverter.ConvertColumnIndexToName(i));
			}
		}

		[TestMethod]
		public void ColumnIndexToNameAndColumnIndexToNameConversionTest()
		{
			List<string> allValidColumnNames = NameGenerator.GetAllColumnNames();

			for (int i = 0; i < allValidColumnNames.Count; i++)
			{
				string columnName = NameConverter.ConvertColumnIndexToName(i);
				Assert.AreEqual(i, NameConverter.ConvertColumnNameToIndex(columnName));
			}
		}

		[TestMethod]
		public void CellNameToIndexAndCellIndexToNameConversionTest()
		{
			for (int i = 0; i < SpreadsheetDefaultValues.RowCount; i += 1000)
			{
				for (int j = 0; j < SpreadsheetDefaultValues.ColumnCount; j += 1000)
				{
					string cellName = NameConverter.ConvertCellIndexToName(i, j);

					int rowIndex;
					int columnIndex;

					NameConverter.ConvertCellNameToIndex(cellName, out rowIndex, out columnIndex);

					Assert.AreEqual(i, rowIndex);
					Assert.AreEqual(j, columnIndex);
				}
			}
		}
	}
}