﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System.Windows;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayoutTests
{
    [TestClass]
    public class PageScaleFactorCalculatorScaleCalculatorTests
    {
        [TestMethod]
        public void Fit25StandardWidthColumnsToOnePage()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 1;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.4, 0.4);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit25StandardWidthColumnsToZeroPagesTall()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesTall = 0;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.4, 0.4);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit25StandardWidthColumnsToZeroPagesTallAndWide()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 0;
            worksheet.WorksheetPageSetup.FitToPagesTall = 0;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(1, 1);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit100StandardHeightRowsToOnePage()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 100; i++)
            {
                worksheet.Cells[i, 0].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesTall = 1;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.48, 0.48);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit100StandardHeightRowsToZeroPagesWide()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 100; i++)
            {
                worksheet.Cells[i, 0].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 0;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.48, 0.48);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit25StandardWidthColumnsToTwoPages()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for(int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.77, 0.77);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit100StandardHeightRowsToTwoPages()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 100; i++)
            {
                worksheet.Cells[i, 0].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesTall = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.97, 0.97);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitTwoRangesToTwoPages()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;
            
            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet, new List<CellRange> { new CellRange(0, 0, 0, 0), new CellRange(0, 1, 0, 24) });

            // Assert
            Size expectedScaleFactor = new Size(0.84, 0.84);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit5WideColumnsToTwoPages()
        {
            // The columns are such as one page would take 2.5 columns if splitting columns were possible.

            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(225, true));

            worksheet.Cells[0, 1].SetValue(2);
            worksheet.Columns[1].SetWidth(new ColumnWidth(225, true));

            worksheet.Cells[0, 2].SetValue(3);
            worksheet.Columns[2].SetWidth(new ColumnWidth(225, true));

            worksheet.Cells[0, 3].SetValue(4);
            worksheet.Columns[3].SetWidth(new ColumnWidth(225, true));

            worksheet.Cells[0, 4].SetValue(5);
            worksheet.Columns[4].SetWidth(new ColumnWidth(225, true));

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.97, 0.97);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit4ColumnsFirstAndSecondAreWideToTwoPages()
        {
            // The first column is close to a page wide. The second is almost as wide. The algorithm has to set scaling to fit the last three in one page.

            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(630, true));

            worksheet.Cells[0, 1].SetValue(2);
            worksheet.Columns[1].SetWidth(new ColumnWidth(515, true));

            worksheet.Cells[0, 2].SetValue(3);
            worksheet.Columns[2].SetWidth(new ColumnWidth(100, true));

            worksheet.Cells[0, 3].SetValue(4);

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.96, 0.96);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit4ColumnsFirstIsWiderThanAPageToTwoPages()
        {
            // The first column is larger than a page wide. The algorithm has to set scaling to fit the first column and leave the last three for the second page.

            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(1000, true));

            worksheet.Cells[0, 1].SetValue(2);
            worksheet.Cells[0, 2].SetValue(3);
            worksheet.Cells[0, 3].SetValue(4);

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.65, 0.65);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit4Columns_200_480_70_64_ToTwoPages()
        {
            // This scenario breaks Excel algorithm. The solution is to put the first column in one page and set scaling to fit the last three in one page.

            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(200, true));

            worksheet.Cells[0, 1].SetValue(2);
            worksheet.Columns[1].SetWidth(new ColumnWidth(480, true));

            worksheet.Cells[0, 2].SetValue(3);
            worksheet.Columns[2].SetWidth(new ColumnWidth(70, true));

            worksheet.Cells[0, 3].SetValue(4);

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(1, 1);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit25StandardWidthColumnsToOnePageWithLargeMargins()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 25; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 1;
            worksheet.WorksheetPageSetup.Margins = new Model.Printing.PageMargins(150, 0);

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.3, 0.3);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void Fit100StandardHeightRowsToTwoPagesLandscapeOrientation()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            for (int i = 0; i < 100; i++)
            {
                worksheet.Cells[i, 0].SetValue(i);
            }

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesTall = 2;
            worksheet.WorksheetPageSetup.PageOrientation = Documents.Model.PageOrientation.Landscape;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.64, 0.64);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitOneWideColumnToOnePageWithAndWithoutHeadings()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(1000, true));

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 1;
            worksheet.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings = true;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.63, 0.63);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);

            // Arrange
            worksheet.WorksheetPageSetup.PrintOptions.PrintRowColumnHeadings = false;

            // Act
            resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            expectedScaleFactor = new Size(0.65, 0.65);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitColumnsToOnePageWithWhereScaleFactorWillBeBelowMinimum()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(2000, true));

            worksheet.Cells[0, 1].SetValue(1);
            worksheet.Columns[1].SetWidth(new ColumnWidth(2000, true));

            worksheet.Cells[0, 2].SetValue(1);
            worksheet.Columns[2].SetWidth(new ColumnWidth(2000, true));

            worksheet.Cells[0, 3].SetValue(1);
            worksheet.Columns[3].SetWidth(new ColumnWidth(2000, true));

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 1;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.1, 0.1);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitColumnsToThreePagesWhenScaleFactorIsDeterminedByTheLastPage()
        {
            // With this document, it is the last page that has to fit the most content and the scale factor should be taken from it.

            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);
            worksheet.Columns[0].SetWidth(new ColumnWidth(126, true));

            for(int i = 1; i <= 6; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
                worksheet.Columns[i].SetWidth(new ColumnWidth(65, true));
            }

            for (int i = 7; i <= 27; i++)
            {
                worksheet.Cells[0, i].SetValue(i);
                worksheet.Columns[i].SetWidth(new ColumnWidth(120, true));
            }

            worksheet.Cells[0, 28].SetValue(28);
            worksheet.Columns[28].SetWidth(new ColumnWidth(65, true));
            worksheet.Cells[0, 29].SetValue(29);
            worksheet.Columns[29].SetWidth(new ColumnWidth(65, true));

            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 3;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.62, 0.62);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitLastColumnLargerThanTheOtherColumnsAndThanThePage()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 3].SetValue(3);
            worksheet.Columns[3].SetWidth(new ColumnWidth(1000, true));

            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.68, 0.68);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitColumnLargerThanTheOtherColumnsAndThanThePageAndTwoColumnsAfterIt()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 5].SetValue(3);
            worksheet.Columns[3].SetWidth(new ColumnWidth(1000 - 2 * SpreadsheetDefaultValues.DefaultColumnWidth, true));

            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.68, 0.68);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitWhenLastColumnIsZero()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 5].SetValue(3);
            worksheet.Columns[3].SetWidth(new ColumnWidth(1000 - SpreadsheetDefaultValues.DefaultColumnWidth, true));
            worksheet.Columns[5].SetWidth(new ColumnWidth(0, true));

            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 2;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(0.68, 0.68);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }

        [TestMethod]
        public void FitColumnsToOnePageWithLittleContentResultsIs100PercentScaleFactor()
        {
            // Arrange
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();

            worksheet.Cells[0, 0].SetValue(1);

            worksheet.WorksheetPageSetup.PaperType = PaperTypes.A4;
            worksheet.WorksheetPageSetup.FitToPages = true;
            worksheet.WorksheetPageSetup.FitToPagesWide = 1;

            // Act
            Size resultScaleFactor = PageScaleFactorCalculator.CalculateScaleAccordingToFitToPages(worksheet);

            // Assert
            Size expectedScaleFactor = new Size(1, 1);
            Assert.AreEqual(expectedScaleFactor, resultScaleFactor);
        }
    }
}
