﻿using System;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.PropertySystemTests
{
    [TestClass]
    public class RowColumnCellPropertiesTests
    {
        #region Fill Property

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedFill = PatternFill.CreateSolidFill(Colors.Red);
            worksheet.Columns[1, 5].SetFill(expectedFill);

            var actualColumnFill = worksheet.Columns[2].GetFill().Value;
            Assert.AreEqual(expectedFill, actualColumnFill);

            var actualCellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedFill, actualCellFill);

            var actualDefaultColumnFill = worksheet.Columns[8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultColumnFill);

            var actualDefaultCellFill = worksheet.Cells[8, 8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultCellFill);

            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedFill = PatternFill.CreateSolidFill(Colors.Red);
            worksheet.Rows[1, 5].SetFill(expectedFill);

            var actualRowFill = worksheet.Rows[2].GetFill().Value;
            Assert.AreEqual(expectedFill, actualRowFill);

            var actualCellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedFill, actualCellFill);

            var actualDefaultRowFill = worksheet.Rows[8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultRowFill);

            var actualDefaultCellFill = worksheet.Cells[8, 8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultCellFill);

            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnThenRowFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedRowFill = PatternFill.CreateSolidFill(Colors.Red);
            var expectedColumnFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Columns[1, 5].SetFill(expectedColumnFill);
            worksheet.Rows[1, 5].SetFill(expectedRowFill);

            var actualRowFill = worksheet.Rows[2].GetFill().Value;
            Assert.AreEqual(expectedRowFill, actualRowFill);

            var actualColumnFill = worksheet.Columns[2].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, actualColumnFill);

            var actualCellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedRowFill, actualCellFill);

            var actualDefaultColumnFill = worksheet.Rows[8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultColumnFill);

            var actualDefaultCellFill = worksheet.Cells[8, 8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultCellFill);

            worksheet.Workbook.History.Undo();
            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();
            worksheet.Workbook.History.Redo();

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(expectedRowFill, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowThenColumnFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedRowFill = PatternFill.CreateSolidFill(Colors.Red);
            var expectedColumnFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[1, 5].SetFill(expectedRowFill);
            worksheet.Columns[1, 5].SetFill(expectedColumnFill);

            var actualRowFill = worksheet.Rows[2].GetFill().Value;
            Assert.AreEqual(expectedRowFill, actualRowFill);

            var actualColumnFill = worksheet.Columns[2].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, actualColumnFill);

            var actualCellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, actualCellFill);

            var actualDefaultColumnFill = worksheet.Rows[8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultColumnFill);

            var actualDefaultCellFill = worksheet.Cells[8, 8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultCellFill);

            worksheet.Workbook.History.Undo();
            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();
            worksheet.Workbook.History.Redo();

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(expectedRowFill, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellThenRowFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedCellFill = PatternFill.CreateSolidFill(Colors.Green);
            var expectedRowFill = PatternFill.CreateSolidFill(Colors.Red);

            worksheet.Cells[1, 1, 5, 5].SetFill(expectedCellFill);
            worksheet.Rows[1, 5].SetFill(expectedRowFill);

            var actualRowFill = worksheet.Rows[2].GetFill().Value;
            Assert.AreEqual(expectedRowFill, actualRowFill);

            var actualCellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedRowFill, actualCellFill);

            var actualDefaultColumnFill = worksheet.Rows[8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultColumnFill);

            var actualDefaultCellFill = worksheet.Cells[8, 8].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, actualDefaultCellFill);

            worksheet.Workbook.History.Undo();
            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Cells[1, 1, 5, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();
            worksheet.Workbook.History.Redo();

            fill = worksheet.Cells[1, 1, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedRowFill, fill);

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(expectedRowFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowFillThenClearCellFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedRowFill = PatternFill.CreateSolidFill(Colors.Red);

            worksheet.Rows[1, 5].SetFill(expectedRowFill);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var cellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);

            var cellFill1 = worksheet.Cells[1, 1].GetFill().Value;
            Assert.AreEqual(expectedRowFill, cellFill1);

            worksheet.Workbook.History.Undo();
            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Cells[2, 2, 4, 4].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();
            worksheet.Workbook.History.Redo();

            fill = worksheet.Cells[2, 2, 4, 4].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Rows[1, 5].GetFill().Value;
            Assert.AreEqual(expectedRowFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnFillThenClearCellFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedColumnFill = PatternFill.CreateSolidFill(Colors.Red);

            worksheet.Columns[1, 5].SetFill(expectedColumnFill);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var cellFill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, cellFill);

            var cellFill1 = worksheet.Cells[1, 1].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, cellFill1);

            worksheet.Workbook.History.Undo();
            worksheet.Workbook.History.Undo();

            IFill fill = worksheet.Cells[2, 2, 4, 4].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            worksheet.Workbook.History.Redo();
            worksheet.Workbook.History.Redo();

            fill = worksheet.Cells[2, 2, 4, 4].GetFill().Value;
            Assert.AreEqual(PatternFill.Default, fill);

            fill = worksheet.Columns[1, 5].GetFill().Value;
            Assert.AreEqual(expectedColumnFill, fill);
        }

        #endregion


        #region Format Property

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellValueThenApplyNumberFormat()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0, 10, 10].SetValue(15);
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat("@"));

            var value = worksheet.Cells[0, 0].GetValue().Value as TextCellValue;
            Assert.IsNotNull(value);

            var value1 = worksheet.Cells[10, 10].GetValue().Value as NumberCellValue;
            Assert.IsNotNull(value1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellValueThenApplyColumnNumberFormat()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0, 10, 10].SetValue(15);
            worksheet.Columns[0].SetFormat(new CellValueFormat("@"));

            var value = worksheet.Cells[0, 0].GetValue().Value as TextCellValue;
            Assert.IsNotNull(value);

            var value1 = worksheet.Cells[10, 10].GetValue().Value as NumberCellValue;
            Assert.IsNotNull(value1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellValueThenApplyRowNumberFormat()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0, 10, 10].SetValue(15);
            worksheet.Rows[0].SetFormat(new CellValueFormat("@"));

            var value = worksheet.Cells[0, 0].GetValue().Value as TextCellValue;
            Assert.IsNotNull(value);

            var value1 = worksheet.Cells[10, 10].GetValue().Value as NumberCellValue;
            Assert.IsNotNull(value1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowFormatThenClearCellFormatProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedColumnFormat = new CellValueFormat("@");

            worksheet.Columns[1, 5].SetFormat(expectedColumnFormat);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var format = worksheet.Cells[2, 2].GetFormat().Value;
            Assert.AreEqual(CellValueFormat.GeneralFormat, format);

            var format1 = worksheet.Cells[1, 1].GetFormat().Value;
            Assert.AreEqual(expectedColumnFormat, format1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnFormatThenClearCellFormatProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedRowFormat = new CellValueFormat("@");

            worksheet.Rows[1, 5].SetFormat(expectedRowFormat);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var format = worksheet.Cells[2, 2].GetFormat().Value;
            Assert.AreEqual(CellValueFormat.GeneralFormat, format);

            var format1 = worksheet.Cells[1, 1].GetFormat().Value;
            Assert.AreEqual(expectedRowFormat, format1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetNumberValueThenSetTextNumberFormatToColumnsCheckIfCellValueIsConvertedToTextCellValue()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var textFormat = new CellValueFormat("@");

            worksheet.Cells[0, 0, 5, 5].SetValue(1);
            worksheet.Columns[0, 3].SetFormat(textFormat);

            var textCellValue = worksheet.Cells[0, 0].GetValue().Value;
            Assert.IsTrue(textCellValue is TextCellValue);

            var numberCellValue = worksheet.Cells[5, 5].GetValue().Value;
            Assert.IsTrue(numberCellValue is NumberCellValue);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetNumberValueThenSetTextNumberFormatToRowsCheckIfCellValueIsConvertedToTextCellValue()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var textFormat = new CellValueFormat("@");

            worksheet.Cells[0, 0, 5, 5].SetValue(1);
            worksheet.Rows[0, 3].SetFormat(textFormat);

            var textCellValue = worksheet.Cells[0, 0].GetValue().Value;
            Assert.IsTrue(textCellValue is TextCellValue);

            var numberCellValue = worksheet.Cells[5, 5].GetValue().Value;
            Assert.IsTrue(numberCellValue is NumberCellValue);
        }

        #endregion


        #region Borders Properties

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowBorders()
        {
            var worksheet = TestHelper.CreateWorksheet();
            CellBorders borders = new CellBorders(
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.DashDotDot, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Dotted, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Double, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Medium, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.MediumDashDot, new ThemableColor(Colors.Red)));

            worksheet.Rows[0, 5].SetBorders(borders);

            var actualBorders = worksheet.Rows[0, 5].GetBorders();

            Assert.AreEqual(borders, actualBorders);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnBorders()
        {
            var worksheet = TestHelper.CreateWorksheet();
            CellBorders borders = new CellBorders(
                new CellBorder(CellBorderStyle.DashDot, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Double, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Dashed, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Double, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Double, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.Medium, new ThemableColor(Colors.Red)),
                new CellBorder(CellBorderStyle.MediumDashDot, new ThemableColor(Colors.Red)));

            worksheet.Columns[0, 5].SetBorders(borders);

            var actualBorders = worksheet.Columns[0, 5].GetBorders();

            Assert.AreEqual(borders, actualBorders);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellBorders()
        {
            var worksheet = TestHelper.CreateWorksheet();
            CellBorders borders = new CellBorders(
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Black)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Blue)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Brown)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Cyan)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.DarkGray)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Gray)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.Green)),
                new CellBorder(CellBorderStyle.Hair, new ThemableColor(Colors.LightGray)));

            worksheet.Cells[0, 0, 5, 5].SetBorders(borders);

            var actualBorders = worksheet.Cells[0, 0, 5, 5].GetBorders();

            Assert.AreEqual(borders, actualBorders);
        }

        #endregion


        #region CellStyle Property

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnStyleBadThenRowStyleExplanatoryCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Columns[0, 5].SetStyleName("Bad");
            worksheet.Rows[5, 10].SetStyleName("Explanatory Text");

            var badForeColor = worksheet.Cells[0, 0].GetForeColor().Value;
            var explanatoryForeColor = worksheet.Cells[5, 0, 5, 0].GetForeColor().Value;

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            Assert.AreEqual(badStyle.ForeColor, badForeColor);
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnStyleExplanatoryThenRowStyleBadCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Columns[0, 5].SetStyleName("Explanatory Text");
            worksheet.Rows[5, 10].SetStyleName("Bad");

            var explanatoryForeColor = worksheet.Cells[1, 1].GetForeColor().Value;
            var badForeColor = worksheet.Cells[5, 1, 5, 1].GetForeColor().Value;

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            Assert.AreEqual(badStyle.ForeColor, badForeColor);
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowStyleBadThenColumnStyleExplanatoryCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Rows[0, 5].SetStyleName("Bad");
            worksheet.Columns[5, 10].SetStyleName("Explanatory Text");

            var explanatoryForeColor = worksheet.Cells[6, 6].GetForeColor().Value;
            var badForeColor = worksheet.Cells[5, 0, 5, 0].GetForeColor().Value;

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            Assert.AreEqual(badStyle.ForeColor, badForeColor);
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowStyleExplanatoryThenColumnStyleBadCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            worksheet.Rows[0, 5].SetStyleName("Explanatory Text");
            worksheet.Columns[5, 10].SetStyleName("Bad");

            var badForeColor = worksheet.Cells[6, 6].GetForeColor().Value;
            var explanatoryForeColor = worksheet.Cells[5, 1, 5, 1].GetForeColor().Value;

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            Assert.AreEqual(badStyle.ForeColor, badForeColor);
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowStyleBadThenExplanatoryCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            worksheet.Rows[0, 5].SetStyleName(badStyle.Name);

            var badForeColor = worksheet.Cells[2, 2].GetForeColor().Value;
            Assert.AreEqual(badStyle.ForeColor, badForeColor);

            worksheet.Rows[0, 5].SetStyleName(explanatoryStyle.Name);

            var explanatoryForeColor = worksheet.Cells[2, 2].GetForeColor().Value;
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnStyleBadThenExplanatoryCheckTheForeColor()
        {
            var worksheet = TestHelper.CreateWorksheet();

            var styles = worksheet.Workbook.Styles;
            var badStyle = styles.GetByName("Bad");
            var explanatoryStyle = styles.GetByName("Explanatory Text");

            worksheet.Columns[0, 5].SetStyleName(badStyle.Name);

            var badForeColor = worksheet.Cells[2, 2].GetForeColor().Value;
            Assert.AreEqual(badStyle.ForeColor, badForeColor);

            worksheet.Columns[0, 5].SetStyleName(explanatoryStyle.Name);

            var explanatoryForeColor = worksheet.Cells[2, 2].GetForeColor().Value;
            Assert.AreEqual(explanatoryStyle.ForeColor, explanatoryForeColor);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowStyleThenClearCellStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedStyleName = "Bad";

            worksheet.Columns[1, 5].SetStyleName(expectedStyleName);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var style = worksheet.Cells[2, 2].GetStyleName().Value;
            Assert.AreEqual("Normal", style);

            var style1 = worksheet.Cells[1, 1].GetStyleName().Value;
            Assert.AreEqual(expectedStyleName, style1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnStyleThenClearCellStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var expectedStyleName = "Bad";

            worksheet.Rows[1, 5].SetStyleName(expectedStyleName);

            worksheet.Cells[2, 2, 4, 4].Clear(ClearType.All);

            var style = worksheet.Cells[2, 2].GetStyleName().Value;
            Assert.AreEqual("Normal", style);

            var style1 = worksheet.Cells[1, 1].GetStyleName().Value;
            Assert.AreEqual(expectedStyleName, style1);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenColumnStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var goodStyleName = "Good";

            worksheet.Cells[2, 2, 5, 5].SetStyleName(badStyleName);
            worksheet.Columns[2, 5].SetStyleName(goodStyleName);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(goodStyleName, styleName);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenRowStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var goodStyleName = "Good";

            worksheet.Cells[2, 2, 5, 5].SetStyleName(badStyleName);
            worksheet.Rows[2, 5].SetStyleName(goodStyleName);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(goodStyleName, styleName);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnsStyleThenCellsStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var goodStyleName = "Good";

            worksheet.Columns[2, 5].SetStyleName(goodStyleName);
            worksheet.Cells[2, 2, 5, 5].SetStyleName(badStyleName);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowsStyleThenCellsStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var goodStyleName = "Good";

            worksheet.Rows[2, 5].SetStyleName(goodStyleName);
            worksheet.Cells[2, 2, 5, 5].SetStyleName(badStyleName);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenRowFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Cells[2, 2, 2, 8].SetStyleName(badStyleName);
            worksheet.Rows[2].SetFill(expectedFill);

            var actualFill = worksheet.Cells[2, 2, 2, 8].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenColumnFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Cells[2, 2, 8, 2].SetStyleName(badStyleName);
            worksheet.Columns[2].SetFill(expectedFill);

            var actualFill = worksheet.Cells[2, 2, 8, 2].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenColumnFillPropertyUndoRedo()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);
            var cellRange = new CellRange(2, 2, 8, 2);
            var columnIndex = 2;

            worksheet.Cells[cellRange].SetStyleName(badStyleName);

            var styleFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(worksheet.Workbook.Styles.GetByName(badStyleName).Fill, styleFill);

            worksheet.Columns[columnIndex].SetFill(expectedFill);

            var actualFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);

            worksheet.Workbook.History.Undo();

            styleFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(worksheet.Workbook.Styles.GetByName(badStyleName).Fill, styleFill);

            worksheet.Workbook.History.Redo();

            actualFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetCellStyleThenRowFillPropertyUndoRedo()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);
            var cellRange = new CellRange(2, 2, 2, 8);
            int rowIndex = 2;

            worksheet.Cells[cellRange].SetStyleName(badStyleName);

            var styleFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(worksheet.Workbook.Styles.GetByName(badStyleName).Fill, styleFill);

            worksheet.Rows[rowIndex].SetFill(expectedFill);

            var actualFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);

            worksheet.Workbook.History.Undo();

            styleFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(worksheet.Workbook.Styles.GetByName(badStyleName).Fill, styleFill);

            worksheet.Workbook.History.Redo();

            actualFill = worksheet.Cells[cellRange].GetFill().Value;
            Assert.AreEqual(expectedFill, actualFill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnFillThenRowStyle()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);
            int columnIndex = 0;
            int rowIndex = 0;

            worksheet.Columns[columnIndex].SetFill(expectedFill);

            worksheet.Rows[rowIndex].SetStyleName(badStyleName);

            IFill actualFill = worksheet.Cells[1, 0].GetFill().Value;

            Assert.AreEqual(expectedFill, actualFill);
        }

        #endregion


        #region CellStyle and Fill property

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowsStyleThenRowsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[2, 5].SetStyleName(badStyleName);
            worksheet.Rows[2, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnsStyleThenColumnsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Columns[2, 5].SetStyleName(badStyleName);
            worksheet.Columns[2, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnsStyleThenRowsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Columns[2, 5].SetStyleName(badStyleName);
            worksheet.Rows[2, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowssStyleThenColumnsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[2, 5].SetStyleName(badStyleName);
            worksheet.Columns[2, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowsStyleThenCellsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[2, 5].SetStyleName(badStyleName);
            worksheet.Cells[2, 2, 5, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetColumnsStyleThenCellsFillProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Columns[2, 5].SetStyleName(badStyleName);
            worksheet.Cells[2, 2, 5, 5].SetFill(expectedFill);

            var styleName = worksheet.Cells[2, 2, 5, 5].GetStyleName().Value;
            Assert.AreEqual(badStyleName, styleName);

            var fill = worksheet.Cells[2, 2, 5, 5].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestSetRowFillThenColumnStyleProperty()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var badStyleName = "Bad";
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[2].SetFill(expectedFill);
            worksheet.Columns[2].SetStyleName(badStyleName);

            var fill = worksheet.Cells[2, 2].GetFill().Value;
            var style = worksheet.Workbook.Styles.GetByName(badStyleName);
            Assert.AreEqual(style.Fill, fill);
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestChangeNormalStyleAndApplyCellFill()
        {
            var worksheet = TestHelper.CreateWorksheet();
            var normalStyleName = "Normal";
            var normalStyle = worksheet.Workbook.Styles.GetByName(normalStyleName);
            var expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            normalStyle.IncludeFill = true;
            normalStyle.Fill = PatternFill.CreateSolidFill(Colors.White);

            worksheet.Rows[2].SetFill(expectedFill);

            var fill = worksheet.Cells[2, 2].GetFill().Value;
            Assert.AreEqual(expectedFill, fill);
        }

        #endregion


        #region Copy Paste Fill Property

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestCopyPasteFillPropertyForColumns()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            IFill expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Columns[0].SetFill(expectedFill);

            CellRange columnRange = CellRange.FromColumn(0);
            WorksheetFragment fragment = worksheet.Cells[columnRange].Copy();

            CellRange severalColumnsRange = CellRange.FromColumnRange(1, 5);
            worksheet.Cells[severalColumnsRange].Paste(fragment, new PasteOptions(PasteType.All));

            for (int i = 1; i <= 5; i++)
            {
                IFill fill = worksheet.Columns[i].GetFill().Value;
                Assert.AreEqual(expectedFill, fill);
            }
        }

        [TestMethod]
        [TestCategory("PropertySystemTests")]
        public void TestCopyPasteFillPropertyForRows()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            IFill expectedFill = PatternFill.CreateSolidFill(Colors.Green);

            worksheet.Rows[0].SetFill(expectedFill);

            CellRange rowRange = CellRange.FromRow(0);
            WorksheetFragment fragment = worksheet.Cells[rowRange].Copy();

            CellRange severalRowsRange = CellRange.FromRowRange(1, 5);
            worksheet.Cells[severalRowsRange].Paste(fragment, new PasteOptions(PasteType.All));

            for (int i = 1; i <= 5; i++)
            {
                IFill fill = worksheet.Rows[i].GetFill().Value;
                Assert.AreEqual(expectedFill, fill);
            }
        }

        #endregion
    }
}