﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class BinaryOperatorExpressionTests
    {
        [TestMethod]
        public void AdditionExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("0+0", typeof(AdditionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("1+0", typeof(AdditionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("0+1", typeof(AdditionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("-1+1", typeof(AdditionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("-1+0", typeof(AdditionExpression), -1);

            ExpressionTestHelper.AssertExpressionResult("1+12", typeof(AdditionExpression), 13);
            ExpressionTestHelper.AssertExpressionResult("(13+3)", typeof(AdditionExpression), 16);
            ExpressionTestHelper.AssertExpressionResult("-13+19", typeof(AdditionExpression), 6);
            ExpressionTestHelper.AssertExpressionResult("15-3+12-3+1", typeof(AdditionExpression), 22);
            ExpressionTestHelper.AssertExpressionResult("(13-3)+(12-3)", typeof(AdditionExpression), 19);
            ExpressionTestHelper.AssertExpressionResult(" (13 - 3) + (12 - 3) ", typeof(AdditionExpression), 19);
            ExpressionTestHelper.AssertExpressionResult("(13+(3+12)+3)", typeof(AdditionExpression), 31);
            ExpressionTestHelper.AssertExpressionResult("(13+((3)+12)+3)", typeof(AdditionExpression), 31);
        }

        [TestMethod]
        public void AdditionExpressionDifferentArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1+B1", typeof(AdditionExpression), 5, worksheet);
            ExpressionTestHelper.AssertExpressionResult("C1+E1", typeof(AdditionExpression), 21, worksheet);
            ExpressionTestHelper.AssertExpressionResult("F1+E1", typeof(AdditionExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10+E1", typeof(AdditionExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10+B10", typeof(AdditionExpression), 0, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B1+D1", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3+B3", typeof(AdditionExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4+B4", typeof(AdditionExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4+B3:B4", typeof(AdditionExpression), 4, worksheet);
        }

        [TestMethod]
        public void AdditionExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true + false", typeof(AdditionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("true + 7", typeof(AdditionExpression), 8);
            ExpressionTestHelper.AssertExpressionResult("true +  7 + false", typeof(AdditionExpression), 8);
            ExpressionTestHelper.AssertExpressionResult("\"8\" + 7", typeof(AdditionExpression), 15);
            ExpressionTestHelper.AssertExpressionResult("\"8\" + \"-7\"", typeof(AdditionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("{1{ls}2{ls}3}+{4{ls}5{ls}6}".ReplaceCultureDependentSeparators(), typeof(AdditionExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("{1}+{-4{ls}5{ls}6}".ReplaceCultureDependentSeparators(), typeof(AdditionExpression), -3);
        }

        [TestMethod]
        public void InvalidAdditionExpressionTest()
        {
            //TODO: Auto generate expressions according to Equivalent Partitioning and Boudary Value Analisys

            ExpressionTestHelper.AssertThrownException("1 3 + 3", typeof(ExpressionException));//InvalidOperationException
            ExpressionTestHelper.AssertThrownException("1 {ds} 3 + 3".ReplaceCultureDependentSeparators(), typeof(ExpressionException));//FormatException

            ExpressionTestHelper.AssertErrorExpressionResult("7+\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1+\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SubtractionExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("0-0", typeof(SubtractionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("0-1", typeof(SubtractionExpression), -1);
            ExpressionTestHelper.AssertExpressionResult("-1-0", typeof(SubtractionExpression), -1);
            ExpressionTestHelper.AssertExpressionResult("1-1", typeof(SubtractionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("1-0", typeof(SubtractionExpression), 1);

            ExpressionTestHelper.AssertExpressionResult("12-3", typeof(SubtractionExpression), 9);
            ExpressionTestHelper.AssertExpressionResult("(21-4)", typeof(SubtractionExpression), 17);
            ExpressionTestHelper.AssertExpressionResult("-12-3", typeof(SubtractionExpression), -15);
            ExpressionTestHelper.AssertExpressionResult("-12+3-5+2-21", typeof(SubtractionExpression), -33);
            ExpressionTestHelper.AssertExpressionResult("(12+3)-(5+2)", typeof(SubtractionExpression), 8);
            ExpressionTestHelper.AssertExpressionResult(" ( 12 + 3 ) - ( 5 + 2 ) ", typeof(SubtractionExpression), 8);
        }

        [TestMethod]
        public void InvalidSubstractionExpressionTest()
        {
            //TODO: Auto generate expressions according to Equivalent Partitioning and Boudary Value Analisys

            ExpressionTestHelper.AssertThrownException("1 2 - 3", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("1 . 2 - 3", typeof(ExpressionException));

            ExpressionTestHelper.AssertErrorExpressionResult("7-\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1-\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void SubtractionExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true - false", typeof(SubtractionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("true - 7", typeof(SubtractionExpression), -6);
            ExpressionTestHelper.AssertExpressionResult("true -  7 - false", typeof(SubtractionExpression), -6);
            ExpressionTestHelper.AssertExpressionResult("\"8\" - 7", typeof(SubtractionExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("\"8\" - \"-7\"", typeof(SubtractionExpression), 15);
            ExpressionTestHelper.AssertExpressionResult("{1{ls}2{ls}3}-{4{ls}5{ls}6}", typeof(SubtractionExpression), -3);
            ExpressionTestHelper.AssertExpressionResult("{1}-{-4{ls}5{ls}6}", typeof(SubtractionExpression), 5);
        }

        [TestMethod]
        public void SubtractionExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1-C1", typeof(SubtractionExpression), -10, worksheet);
            ExpressionTestHelper.AssertExpressionResult("C1-E1", typeof(SubtractionExpression), 19, worksheet);
            ExpressionTestHelper.AssertExpressionResult("F1-E1", typeof(SubtractionExpression), -1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10-E1", typeof(SubtractionExpression), -1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10-B10", typeof(SubtractionExpression), 0, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A1-D1", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3-B3", typeof(SubtractionExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4-B4", typeof(SubtractionExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4-B3:B4", typeof(SubtractionExpression), 0, worksheet);
        }

        [TestMethod]
        public void InvalidSubtractionExpressionTest()
        {
            //TODO: Auto generate expressions according to Equivalent Partitioning and Boudary Value Analisys

            ExpressionTestHelper.AssertThrownException("1 3 + 3", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("1 . 3 + 3", typeof(ExpressionException));

            ExpressionTestHelper.AssertErrorExpressionResult("7+\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1+\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void DivisionExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5/2", typeof(DivisionExpression), 2.5);
            ExpressionTestHelper.AssertExpressionResult("1/3", typeof(DivisionExpression), 0.333333333333333);
            ExpressionTestHelper.AssertExpressionResult("0/2", typeof(DivisionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("-5/2", typeof(DivisionExpression), -2.5);
            ExpressionTestHelper.AssertExpressionResult("-1/-3", typeof(DivisionExpression), 0.333333333333333);
            ExpressionTestHelper.AssertExpressionResult("0/-2", typeof(DivisionExpression), 0);

            ExpressionTestHelper.AssertExpressionResult("0.001/10", typeof(DivisionExpression), 0.0001);
            ExpressionTestHelper.AssertExpressionResult("0.0001/10", typeof(DivisionExpression), 1e-5);
            ExpressionTestHelper.AssertExpressionResult("0.00001/10", typeof(DivisionExpression), 1e-6);
            ExpressionTestHelper.AssertExpressionResult("1e-05/10", typeof(DivisionExpression), 1e-6);

        }

        [TestMethod]
        public void DivisionExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true/2", typeof(DivisionExpression), 0.5);
            ExpressionTestHelper.AssertExpressionResult("\"8\" / 2", typeof(DivisionExpression), 4);
            ExpressionTestHelper.AssertExpressionResult("\"8\" / \"-2\"", typeof(DivisionExpression), -4);
            ExpressionTestHelper.AssertExpressionResult("{1{ls}2{ls}3}/{4{ls}5{ls}6}", typeof(DivisionExpression), 0.25);
            ExpressionTestHelper.AssertExpressionResult("{1}/{-4{ls}5{ls}6}", typeof(DivisionExpression), -0.25);
        }

        [TestMethod]
        public void DivisionExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1/B1", typeof(DivisionExpression), -2, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1/C1", typeof(DivisionExpression), 0.5, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1/E1", typeof(DivisionExpression), 10, worksheet);
            ExpressionTestHelper.AssertExpressionResult("F1/B1", typeof(DivisionExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10/E1", typeof(DivisionExpression), 0, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("D1/B1", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B1/F1", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A10/B10", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3/B3", typeof(DivisionExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4/B4", typeof(DivisionExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4/B3:B4", typeof(DivisionExpression), 1, worksheet);
        }

        [TestMethod]
        public void InvalidDivisionExpressionTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("5/0", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("true / false", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("7/\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1/\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void MultiplicationExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5*2", typeof(MultiplicationExpression), 10);
            ExpressionTestHelper.AssertExpressionResult("0*3", typeof(MultiplicationExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("-5*2", typeof(MultiplicationExpression), -10);
            ExpressionTestHelper.AssertExpressionResult("0*-3", typeof(MultiplicationExpression), 0);
            ExpressionTestHelper.AssertExpressionResult(string.Format("{0}*-2", (0.5).ToCultureDependentString()), typeof(MultiplicationExpression), -1);
        }

        [TestMethod]
        public void MultiplicationExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true*2", typeof(MultiplicationExpression), 2);
            ExpressionTestHelper.AssertExpressionResult("\"8\" * 2", typeof(MultiplicationExpression), 16);
            ExpressionTestHelper.AssertExpressionResult("\"8\" * \"-2\"", typeof(MultiplicationExpression), -16);
            ExpressionTestHelper.AssertExpressionResult("{2{ls}3}*{4{ls}5{ls}6}", typeof(MultiplicationExpression), 8);
            ExpressionTestHelper.AssertExpressionResult("{2}*{-4{ls}5{ls}6}", typeof(MultiplicationExpression), -8);
        }

        [TestMethod]
        public void MultiplicationExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1*B1", typeof(MultiplicationExpression), -50, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1*C1", typeof(MultiplicationExpression), 200, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1*E1", typeof(MultiplicationExpression), 10, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1*F1", typeof(MultiplicationExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10*E1", typeof(MultiplicationExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10*B10", typeof(MultiplicationExpression), 0, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A1*D1", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3*B3", typeof(MultiplicationExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4*B4", typeof(MultiplicationExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4*B3:B4", typeof(MultiplicationExpression), 4, worksheet);
        }

        [TestMethod]
        public void InvalidMultiplicationExpressionTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("7*\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1*\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void PowerExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5^2^3", typeof(PowerExpression), 15625); // NOTE: In MS Excel the power operator is left associative.
            ExpressionTestHelper.AssertExpressionResult("5^2", typeof(PowerExpression), 25);
            ExpressionTestHelper.AssertExpressionResult("5^1", typeof(PowerExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("5^0", typeof(PowerExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("5^-1", typeof(PowerExpression), 0.2);
            ExpressionTestHelper.AssertExpressionResult("-1^-1", typeof(PowerExpression), -1);
            ExpressionTestHelper.AssertExpressionResult("-2^-2", typeof(PowerExpression), 0.25);
            ExpressionTestHelper.AssertExpressionResult(string.Format("4 ^ {0}", (0.5).ToCultureDependentString()), typeof(PowerExpression), 2);
            ExpressionTestHelper.AssertExpressionResult("0^1", typeof(PowerExpression), 0);
        }

        [TestMethod]
        public void PowerExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5^true", typeof(PowerExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("true^false", typeof(PowerExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("\"8\" ^ 2", typeof(PowerExpression), 64);
            ExpressionTestHelper.AssertExpressionResult("\"8\" ^ \"-2\"", typeof(PowerExpression), 0.015625);
            ExpressionTestHelper.AssertExpressionResult("{2{ls}3}^{4{ls}5{ls}6}", typeof(PowerExpression), 16);
            ExpressionTestHelper.AssertExpressionResult("{2}^{-4{ls}5{ls}6}", typeof(PowerExpression), 0.0625);
        }

        [TestMethod]
        public void PowerExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1^2", typeof(PowerExpression), 100, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1^2", typeof(PowerExpression), 25, worksheet);
            ExpressionTestHelper.AssertExpressionResult("C1^2", typeof(PowerExpression), 400, worksheet);
            ExpressionTestHelper.AssertExpressionResult("E1^1", typeof(PowerExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("E1^F1", typeof(PowerExpression), 1, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10^E1", typeof(PowerExpression), 0, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3^B3", typeof(PowerExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4^B4", typeof(PowerExpression), 4, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4^B3:B4", typeof(PowerExpression), 4, worksheet);
        }

        [TestMethod]
        public void InvalidPowerExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("D1^B1", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("F1^F1", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A10^B10", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A10^B1", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("F1^B1", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void InvalidPowerExpressionTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("0^0", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("false^false", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("0^-2", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("7^\"string\"", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("1^\"\"", ErrorExpressions.ValueError);
        }

        [TestMethod]
        public void PowerExpressionWithNegativeNomeratorAndEvenEgyptianFractionDenominatorTest()
        {
            int testRange = 100;
            double testNomerator = -4389.45;
            for (int i = 2; i <= testRange; i += 2)
            {
                ExpressionTestHelper.AssertErrorExpressionResult(String.Format("({0})^(1/{1})", testNomerator, i), ErrorExpressions.NumberError);
                ExpressionTestHelper.AssertErrorExpressionResult(String.Format("({0})^(1/-{1})", testNomerator, i), ErrorExpressions.NumberError);
            }
        }

        [TestMethod]
        public void PowerExpressionWithNegativeNomeratorAndOddEgyptianFractionDenominatorTest()
        {
            int testRange = 100;
            double testNomerator = -4389.45;
            for (int i = 3; i <= testRange; i += 2)
            {
                ExpressionTestHelper.AssertAreEssentiallyEqualExpressionResult(String.Format("({0})^(1/{1})", testNomerator, i), typeof(PowerExpression), -Math.Pow(-testNomerator, 1.0 / i));
                ExpressionTestHelper.AssertAreEssentiallyEqualExpressionResult(String.Format("({0})^(1/-{1})", testNomerator, i), typeof(PowerExpression), -Math.Pow(-testNomerator, 1.0 / -i));
            }
        }

        [TestMethod]
        public void PowerExpressionWithNegativeNomeratorAndIntegerDenominatorTest()
        {
            int testRange = 10;
            double testNomerator = -4389.45;
            for (int i = 0; i <= testRange; i++)
            {
                ExpressionTestHelper.AssertExpressionResult(String.Format("({0})^({1})", testNomerator, i), typeof(PowerExpression), Math.Pow(testNomerator, i));
                ExpressionTestHelper.AssertExpressionResult(String.Format("({0})^(-{1})", testNomerator, i), typeof(PowerExpression), Math.Pow(testNomerator, -i));
            }
        }

        [TestMethod]
        public void PowerExpressionWithZeroNomeratorTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("0^0", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("0^-10", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult(string.Format("0^-{0}", (0.15).ToCultureDependentString()), ErrorExpressions.DivisionByZero);

            ExpressionTestHelper.AssertExpressionResult("0^(1/2)", typeof(PowerExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("0^10", typeof(PowerExpression), 0);
        }

        [TestMethod]
        public void PowerExpressionWithPositiveNomeratorTest()
        {
            ExpressionTestHelper.AssertExpressionResult("4^(1/2)", typeof(PowerExpression), 2);
            ExpressionTestHelper.AssertExpressionResult("4^(-1/2)", typeof(PowerExpression), 0.5);
        }

        [TestMethod]
        public void AmpersandExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"abc\"&\"def\"", typeof(AmpersandExpression), "abcdef");
            ExpressionTestHelper.AssertExpressionResult("\"abc\"&\"\"", typeof(AmpersandExpression), "abc");
            ExpressionTestHelper.AssertExpressionResult("\"\"&\"abc\"", typeof(AmpersandExpression), "abc");
            ExpressionTestHelper.AssertExpressionResult("\"\"&\"\"", typeof(AmpersandExpression), "");
        }

        [TestMethod]
        public void AmpersandExpressionWithDifferentTypeArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true&false", typeof(AmpersandExpression), "TRUEFALSE");
            ExpressionTestHelper.AssertExpressionResult("7&false", typeof(AmpersandExpression), "7FALSE");
            ExpressionTestHelper.AssertExpressionResult("\"8\" & 2", typeof(AmpersandExpression), "82");
            ExpressionTestHelper.AssertExpressionResult("\"8\" & \"-2\"", typeof(AmpersandExpression), "8-2");
            ExpressionTestHelper.AssertExpressionResult("{2{ls}3} & {4{ls}5{ls}6}", typeof(AmpersandExpression), "24");
            ExpressionTestHelper.AssertExpressionResult("{2} & {-4{ls}5{ls}6}", typeof(AmpersandExpression), "2-4");
            ExpressionTestHelper.AssertExpressionResult("{\"first\"{ls}\"second\"} & {\"third\"{ls}5{ls}6}", typeof(AmpersandExpression), "firstthird");
            ExpressionTestHelper.AssertExpressionResult("{2} & {\"string\"{ls}5{ls}6}", typeof(AmpersandExpression), "2string");

            ExpressionTestHelper.AssertExpressionResult("123&123", typeof(AmpersandExpression), "123123");
            ExpressionTestHelper.AssertExpressionResult("123&\"abc\"", typeof(AmpersandExpression), "123abc");
            ExpressionTestHelper.AssertExpressionResult("123&true", typeof(AmpersandExpression), "123TRUE");
            ExpressionTestHelper.AssertExpressionResult("\"abc\"&123", typeof(AmpersandExpression), "abc123");
            ExpressionTestHelper.AssertExpressionResult("\"abc\"&\"abc\"", typeof(AmpersandExpression), "abcabc");
            ExpressionTestHelper.AssertExpressionResult("\"abc\"&True", typeof(AmpersandExpression), "abcTRUE");
            ExpressionTestHelper.AssertExpressionResult("False&123", typeof(AmpersandExpression), "FALSE123");
            ExpressionTestHelper.AssertExpressionResult("false&\"abc\"", typeof(AmpersandExpression), "FALSEabc");
            ExpressionTestHelper.AssertExpressionResult("False&True", typeof(AmpersandExpression), "FALSETRUE");
            ExpressionTestHelper.AssertExpressionResult("123&\"abc\"&TRUE", typeof(AmpersandExpression), "123abcTRUE");
        }

        [TestMethod]
        public void AmpersandExpressionCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("C1&D1", typeof(AmpersandExpression), "20string", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1&B1", typeof(AmpersandExpression), "10-5", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1&C1", typeof(AmpersandExpression), "1020", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1&C1&D1", typeof(AmpersandExpression), "1020string", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1&E1", typeof(AmpersandExpression), "10TRUE", worksheet);
            ExpressionTestHelper.AssertExpressionResult("F1&B1", typeof(AmpersandExpression), "FALSE-5", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10&B10", typeof(AmpersandExpression), "", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3&B3", typeof(AmpersandExpression), "22", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4&B4", typeof(AmpersandExpression), "22", worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A5&B3:B5", typeof(AmpersandExpression), "22", worksheet);
        }

        [TestMethod]
        public void InvalidAmpersandExpressionTest()
        {
            ExpressionTestHelper.AssertThrownException("\"abc\"&\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("&\"abc\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"abc\"&", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"&\"abc\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"&\"abc\"\"", typeof(ExpressionException));
        }

        [TestMethod]
        public void CombinedOperatorsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("(3+3)*5", typeof(MultiplicationExpression), 30);
            ExpressionTestHelper.AssertExpressionResult("(3+(4/2))", typeof(AdditionExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("((3+5)/2)", typeof(DivisionExpression), 4);
            ExpressionTestHelper.AssertExpressionResult("30/(5*3)", typeof(DivisionExpression), 2);
            ExpressionTestHelper.AssertExpressionResult("4+6*(10*(5-3))/3+(1-3)", typeof(AdditionExpression), 42);
            ExpressionTestHelper.AssertExpressionResult("(0{ds}462-0{ds}32094)*((7{ds}2+9{ds}6)/(4-8))", typeof(MultiplicationExpression), -0.59245200000000009);
            ExpressionTestHelper.AssertExpressionResult("9 + 6 * (8 - 5)", typeof(AdditionExpression), 27);
            ExpressionTestHelper.AssertExpressionResult("(14 - 5) / (9 - 6)", typeof(DivisionExpression), 3);
            ExpressionTestHelper.AssertExpressionResult("5 * 8 + 6 / 6 - 12 * 2", typeof(SubtractionExpression), 17);
            ExpressionTestHelper.AssertExpressionResult("(36-3*4)/(15-9/3)", typeof(DivisionExpression), 2);
        }
    }
}
