﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class ComparisonOperatorTests
    {
        #region Comparison Operator Tests

        [TestMethod]
        public void EqualOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5=5", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0=0", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-1000=-1000", typeof(EqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("5=4", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult(string.Format("{0}={1}", (1.333).ToCultureDependentString(), (1.33).ToCultureDependentString()),
                typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult(string.Format("-1000={0}", (-1000.001).ToCultureDependentString()), typeof(EqualExpression), false);
        }

        [TestMethod]
        public void EqualOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true=true", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("false=false", typeof(EqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("false=true", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("true=false", typeof(EqualExpression), false);
        }

        [TestMethod]
        public void EqualOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"a\"=\"a\"", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"a\"=\"A\"", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"A\"=\"a\"", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"=\"AB\"", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"=\" abc\"", typeof(EqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("\"ab\"=\"a\"", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"=\" ab\"", typeof(EqualExpression), false);
        }

        [TestMethod]
        public void EqualOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!=true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A=3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!=#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A2=D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void EqualOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true=\"1\"", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("true=1", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\"=1", typeof(EqualExpression), false);
            
            ExpressionTestHelper.AssertExpressionResult("7=\"7\"", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("7=\"-7\"", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-7=\"-7\"", typeof(EqualExpression), false);

            ExpressionTestHelper.AssertExpressionResult("7=\"string\"", typeof(EqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1=\"\"", typeof(EqualExpression), false);
        }

        [TestMethod]
        public void EqualOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5=A5", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0=A5", typeof(EqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1=A5", typeof(EqualExpression), false);
        }

        [TestMethod]
        public void EqualOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1=A2", typeof(EqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("\"20\"=C1", typeof(EqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3=B3", typeof(EqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A4=B4", typeof(EqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A3:A4=B3:B4", typeof(EqualExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("A1=E1", typeof(EqualExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1=F1", typeof(EqualExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1=D1", typeof(EqualExpression), false, worksheet);
        }

        [TestMethod]
        public void LessThanOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("1<2", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-1<2", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0<2", typeof(LessThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("2<2", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("3<2", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-2<-3", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("2<-2", typeof(LessThanExpression), false);
        }

        [TestMethod]
        public void LessThanOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("false<true", typeof(LessThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true<true", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("false<false", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("true<false", typeof(LessThanExpression), false);
        }

        [TestMethod]
        public void LessThanOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"a\"<\"ab\"", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"abc\"<\"abd\"", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<\"abd\"", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<\"2\"", typeof(LessThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("\"adc\"<\"abd\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<\"a\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<\" ab\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"a\"<\"a\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"a\"<\"A\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"A\"<\"a\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<\"AB\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<\" abc\"", typeof(LessThanExpression), false);
        }

        [TestMethod]
        public void LessThanOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A<3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A1<D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void LessThanOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("1<true", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<true", typeof(LessThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1<\"1\"", typeof(LessThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true<1", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("true<\"1\"", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<1", typeof(LessThanExpression), false);
        }

        [TestMethod]
        public void LessThanOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5<A5", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("0<A5", typeof(LessThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1<A5", typeof(LessThanExpression), false);
        }   

        [TestMethod]
        public void LessThanOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1<E1", typeof(LessThanExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1<F1", typeof(LessThanExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("A1<B1", typeof(LessThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("\"20\"<C1", typeof(LessThanExpression), false, worksheet);
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("1<=2", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("2<=2", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-1<=2", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-3<=2", typeof(LessThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("3<=2", typeof(LessThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("2<=-2", typeof(LessThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true<=true", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("false<=false", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("false<=true", typeof(LessThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true<=false", typeof(LessThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"a\"<=\"ab\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"abc\"<=\"abd\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<=\"abd\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<=\"2\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"a\"<=\"a\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"a\"<=\"A\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"A\"<=\"a\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<=\"AB\"", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<=\" abc\"", typeof(LessThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("\"adc\"<=\"abd\"", typeof(LessThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<=\"a\"", typeof(LessThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<=\" ab\"", typeof(LessThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<=true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A<=3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<=#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A1<=D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("1<=true", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<=true", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1<=\"1\"", typeof(LessThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true<=1", typeof(LessThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("true<=\"1\"", typeof(LessThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<=1", typeof(LessThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void LessThanOrEqualOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5<=A5", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0<=A5", typeof(LessThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1<=A5", typeof(LessThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void LessThanOrEqualOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("\"20\"<=C1", typeof(LessThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1<=E1", typeof(LessThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1<=F1", typeof(LessThanOrEqualToExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("A1<=B1", typeof(LessThanOrEqualToExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10<=E1", typeof(LessThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10<=F1", typeof(LessThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10<=B10", typeof(LessThanOrEqualToExpression), true, worksheet);
        }

        [TestMethod]
        public void GreaterThanOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("2>-2", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0>-2", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("3>2", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-2>-3", typeof(GreaterThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("1>2", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("2>2", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-1>2", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-3>2", typeof(GreaterThanExpression), false);
        }

        [TestMethod]
        public void GreaterThanOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true>false", typeof(GreaterThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true>true", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("false>false", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("false>true", typeof(GreaterThanExpression), false);
        }

        [TestMethod]
        public void GreaterThanOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"a\">\"ab\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"abc\">\"abd\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">\"abd\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">\"2\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"a\">\"a\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"a\">\"A\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"A\">\"a\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"ab\">\"AB\"", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\">\" abc\"", typeof(GreaterThanExpression), false);

            ExpressionTestHelper.AssertExpressionResult("\"adc\">\"abd\"", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"ab\">\"a\"", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\">\" ab\"", typeof(GreaterThanExpression), true);
        }

        [TestMethod]
        public void GreaterThanOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!>true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A>3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!>#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A1>D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void GreaterThanOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true>1", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("true>\"1\"", typeof(GreaterThanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\">1", typeof(GreaterThanExpression), true);

            ExpressionTestHelper.AssertExpressionResult("1>true", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">true", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1>\"1\"", typeof(GreaterThanExpression), false);
        }

        [TestMethod]
        public void GreaterThanOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5>A5", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("0>A5", typeof(GreaterThanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1>A5", typeof(GreaterThanExpression), true);
        }

        [TestMethod]
        public void GreaterThanOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1>B1", typeof(GreaterThanExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("20>C1", typeof(GreaterThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1>E1", typeof(GreaterThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1>F1", typeof(GreaterThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>E1", typeof(GreaterThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>F1", typeof(GreaterThanExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>B10", typeof(GreaterThanExpression), false, worksheet);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("2>=2", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("3>=2", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("2>=-2", typeof(GreaterThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("1>=2", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-1>=2", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-3>=2", typeof(GreaterThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true>=true", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("false>=false", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("true>=false", typeof(GreaterThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("false>=true", typeof(GreaterThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"adc\">=\"abd\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"ab\">=\"a\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\">=\" ab\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"a\">=\"a\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"a\">=\"A\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"A\">=\"a\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"ab\">=\"AB\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\">=\" abc\"", typeof(GreaterThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("\"a\">=\"ab\"", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"abc\">=\"abd\"", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">=\"abd\"", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">=\"2\"", typeof(GreaterThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!>=true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A>=3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!>=#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A1>=D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void GreaterThanOrEqualToOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true>=1", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("true>=\"1\"", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\">=1", typeof(GreaterThanOrEqualToExpression), true);

            ExpressionTestHelper.AssertExpressionResult("1>=true", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1>=\"1\"", typeof(GreaterThanOrEqualToExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"1\">=true", typeof(GreaterThanOrEqualToExpression), false);
        }

        [TestMethod]
        public void GreaterThanOrEqualOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5>=A5", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("0>=A5", typeof(GreaterThanOrEqualToExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1>=A5", typeof(GreaterThanOrEqualToExpression), true);
        }

        [TestMethod]
        public void GreaterThanOrEqualOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1>=B1", typeof(GreaterThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("\"20\">=C1", typeof(GreaterThanOrEqualToExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>=B10", typeof(GreaterThanOrEqualToExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("20>=C1", typeof(GreaterThanOrEqualToExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1>=E1", typeof(GreaterThanOrEqualToExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1>=F1", typeof(GreaterThanOrEqualToExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>=E1", typeof(GreaterThanOrEqualToExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10>=F1", typeof(GreaterThanOrEqualToExpression), false, worksheet);
        }

        [TestMethod]
        public void NotEqualOperatorWithNumberArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("5<>4", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult(string.Format("{0}<>{1}", (1.333).ToCultureDependentString(), (1.33).ToCultureDependentString()),
                typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult(string.Format("-1000<>{0}", (-1000.001).ToCultureDependentString()), typeof(NotEqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("5<>5", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("0<>0", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("-1000<>-1000", typeof(NotEqualExpression), false);
        }

        [TestMethod]
        public void NotEqualOperatorWithBooleanArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("false<>true", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("true<>false", typeof(NotEqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("true<>true", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("false<>false", typeof(NotEqualExpression), false);
        }

        [TestMethod]
        public void NotEqualOperatorWithStringArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<>\"a\"", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<>\" ab\"", typeof(NotEqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("\"a\"<>\"a\"", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"a\"<>\"A\"", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"A\"<>\"a\"", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\"ab\"<>\"AB\"", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("\" abc\"<>\" abc\"", typeof(NotEqualExpression), false);
        }

        [TestMethod]
        public void NotEqualOperatorWithErrorArgumentsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<>true", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A<>3", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!<>#N/A", ErrorExpressions.ValueError);

            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("A1<>D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void NotEqualOperatorWithDifferentArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true<>\"1\"", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("true<>1", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("\"1\"<>1", typeof(NotEqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("7<>\"7\"", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("7<>\"-7\"", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("-7<>\"-7\"", typeof(NotEqualExpression), true);

            ExpressionTestHelper.AssertExpressionResult("7<>\"string\"", typeof(NotEqualExpression), true);
            ExpressionTestHelper.AssertExpressionResult("1<>\"\"", typeof(NotEqualExpression), true);
        }

        [TestMethod]
        public void NotEqualOperatorWithEmptyArgumentsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("A5<>A5", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("0<>A5", typeof(NotEqualExpression), false);
            ExpressionTestHelper.AssertExpressionResult("1<>A5", typeof(NotEqualExpression), true);
        }

        [TestMethod]
        public void NotEqualOperatorWithCellReferenceArgumentsTest()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertExpressionResult("A1<>B1", typeof(NotEqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("20<>C1", typeof(NotEqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A1<>E1", typeof(NotEqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("B1<>F1", typeof(NotEqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10<>E1", typeof(NotEqualExpression), true, worksheet);
            ExpressionTestHelper.AssertExpressionResult("A10<>F1", typeof(NotEqualExpression), true, worksheet);

            ExpressionTestHelper.AssertExpressionResult("A10<>B10", typeof(NotEqualExpression), false, worksheet);
            ExpressionTestHelper.AssertExpressionResult("\"20\"<>C1", typeof(NotEqualExpression), false, worksheet);
        }

        #endregion
    }
}
