﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class ConstantExpressionTests
    {
        #region Methods

        [TestMethod]
        public void NumberExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("0", typeof(NumberExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("1", typeof(NumberExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("1{ds}".ReplaceCultureDependentSeparators(), typeof(NumberExpression), 1);
            ExpressionTestHelper.AssertExpressionResult("{ds}1".ReplaceCultureDependentSeparators(), typeof(NumberExpression), .1);
            ExpressionTestHelper.AssertExpressionResult("123456789{ds}123456789".ReplaceCultureDependentSeparators(), typeof(NumberExpression), 123456789.123456789);
            ExpressionTestHelper.AssertExpressionResult(" 5 ", typeof(NumberExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("(5)", typeof(NumberExpression), 5);
            ExpressionTestHelper.AssertExpressionResult("1{ds}79769313486231E+308", typeof(NumberExpression), 1.79769313486231E+308);
        }

        [TestMethod]
        public void PowerExpressionInfinityValueTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("4389^7E+37", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("(-4389)^7E+37", ErrorExpressions.NumberError);
        }

        [TestMethod]
        public void NaNValueTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("2^(0/0)", ErrorExpressions.DivisionByZero);
        } 

        [TestMethod]
        public void ScientificNotationParseTest()
        {
            ExpressionTestHelper.AssertExpressionResult("1232E+10", typeof(NumberExpression), 12320000000000);
            ExpressionTestHelper.AssertExpressionResult("1{ds}2345679E+08", typeof(NumberExpression), 123456790);
            ExpressionTestHelper.AssertExpressionResult("1{ds}23457E+08", typeof(NumberExpression), 123457000);
            ExpressionTestHelper.AssertExpressionResult("1{ds}23E+08", typeof(NumberExpression), 123000000);
            ExpressionTestHelper.AssertExpressionResult((123.4567).ToCultureDependentString(), typeof(NumberExpression), 123.4567);
            ExpressionTestHelper.AssertExpressionResult((-123.4567).ToCultureDependentString(), typeof(UnaryMinusExpression), -123.4567);
        }

        [TestMethod]
        public void InvalidNumberExpressionTest()
        {
            ExpressionTestHelper.AssertThrownException(" ", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("1 5", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("1.2.3", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("(5", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("5)", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(")5(", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("5+", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("5-", typeof(ExpressionException));
        }

        [TestMethod]
        public void BooleanExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("true", typeof(BooleanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("TRue", typeof(BooleanExpression), true);
            ExpressionTestHelper.AssertExpressionResult("false", typeof(BooleanExpression), false);
            ExpressionTestHelper.AssertExpressionResult("faLSE", typeof(BooleanExpression), false);
        }

        [TestMethod]
        public void StringExpressionTest()
        {
            ExpressionTestHelper.AssertExpressionResult("\"abc\"", typeof(StringExpression), "abc");
            ExpressionTestHelper.AssertExpressionResult("\"\"\"\"", typeof(StringExpression), "\"");
            ExpressionTestHelper.AssertExpressionResult("\"\"\" \"\"\"", typeof(StringExpression), "\" \"");
            ExpressionTestHelper.AssertExpressionResult("\" \"\" \"\" \"\"  \"", typeof(StringExpression), " \" \" \"  ");
            ExpressionTestHelper.AssertExpressionResult("\"\"\"\"\"\"\"\"", typeof(StringExpression), "\"\"\"");
            ExpressionTestHelper.AssertExpressionResult("\"ab\"\"\"\"cd\"\"\"", typeof(StringExpression), "ab\"\"cd\"");
            ExpressionTestHelper.AssertExpressionResult("\"\"\"a\"\"b\"\"c\"\"\"", typeof(StringExpression), "\"a\"b\"c\"");
            ExpressionTestHelper.AssertExpressionResult("\"a\"\"\"\"\"\"b\"", typeof(StringExpression), "a\"\"\"b");
            ExpressionTestHelper.AssertExpressionResult("\"\"\"ab\"\"cd\"\"ef\"", typeof(StringExpression), "\"ab\"cd\"ef");
            ExpressionTestHelper.AssertExpressionResult("\"123 456 789\"", typeof(StringExpression), "123 456 789");
            ExpressionTestHelper.AssertExpressionResult("\"\"", typeof(StringExpression), "");
        }

        [TestMethod]
        public void InvalidStringExpressionTest()
        {
            ExpressionTestHelper.AssertThrownException("\"abc", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"ab\"\"c", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"ab\"cde\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"\"\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"", typeof(ExpressionException));
        }

        [TestMethod]
        public void ErrorExpressionTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#NUM!", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("#num!", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("#NULL!", ErrorExpressions.NullError);
            ExpressionTestHelper.AssertErrorExpressionResult("#nUll!", ErrorExpressions.NullError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VALUE!", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#Value!", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#VaLuE!", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#vAlUe!", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#DIV/0!", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("#REF!", ErrorExpressions.ReferenceError);
        }

        [TestMethod]
        public void InvalidErrorExpressionTest()
        {
            ExpressionTestHelper.AssertThrownException("#NUM", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#NULL", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#NU", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#N", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#N/", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#N!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#val!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#VALUE", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#DIV/O", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#DIV/", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#div0", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#ref", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#re!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#!", typeof(ExpressionException));
        }

        [TestMethod]
        public void ErrorExpressionsInOperatorsTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("#REF!/5", ErrorExpressions.ReferenceError);
            ExpressionTestHelper.AssertErrorExpressionResult("#num! + 5", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("8+#value!", ErrorExpressions.ValueError);
            ExpressionTestHelper.AssertErrorExpressionResult("#DIV/0!/0", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("0/#DIV/0!", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("sum(#N/A)", ErrorExpressions.NotAvailableError);
            ExpressionTestHelper.AssertErrorExpressionResult("#N/A=#N/A", ErrorExpressions.NotAvailableError);
        }

        [TestMethod]
        public void NameErrorExpressionTest()
        {
            ExpressionTestHelper.AssertErrorExpressionResult("Tralala()", ErrorExpressions.NameError);
            ExpressionTestHelper.AssertErrorExpressionResult("string", ErrorExpressions.NameError);
            ExpressionTestHelper.AssertErrorExpressionResult("sum(parameter{ls} 5)", ErrorExpressions.NameError);
            ExpressionTestHelper.AssertErrorExpressionResult("#NAME?", ErrorExpressions.NameError);
            ExpressionTestHelper.AssertErrorExpressionResult("#NAME?+15", ErrorExpressions.NameError);
            ExpressionTestHelper.AssertErrorExpressionResult("15 + #NAME?", ErrorExpressions.NameError);
        }

        #endregion
    }
}
