﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class NamedRangesExpressionTests
    {
        // TODO: This test should be working when defined name cross worksheet copy paste is performed as in Excel
        // When pasting name in different sheet a new name should be created in the new sheet in order the defined name to work properly.
        // There is an item in teampulse about this bug with id-s: B 134473 TFS 220980
        //[TestMethod]
        //public void TranslateNamedRangeCrossWorksheetNonAbsoluteWorksheetTest()
        //{
        //    Workbook workbook = new Workbook();

        //    Worksheet sheet1 = workbook.Worksheets.Add();
        //    Worksheet sheet2 = workbook.Worksheets.Add();

        //    InputSampleCellValues(sheet1, new CellRange(0, 0, 1, 1), 1);
        //    InputSampleCellValues(sheet2, new CellRange(0, 0, 1, 1), 5);

        //    sheet1.Names.Add("C3Name", "=A1", new CellIndex(2, 2));
        //    sheet1.Cells[2, 2].SetValue("=C3Name");
        //    WorksheetFragment worksheetFragment = sheet1.Cells[2, 2].Copy();
        //    sheet2.Cells[3, 3].Paste(worksheetFragment, PasteOptions.All);

        //    ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 2), "=C3Name", "1");
        //    ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(3, 3), "=C3Name", "8");
        //}

        [TestMethod]
        public void CopyPasteComplexFormulaContainingNamedRange()
        {
            Workbook workbook = new Workbook();
            var formula = "=IF(ISERR(MIN(B1,IF(MyField<B2,0,(MyField-B2)/B3*B4))),0,MIN(B1,IF(MyField<B2,0,(MyField-B2)/B3*B4)))";

            Worksheet templateWorkSheet = workbook.Worksheets.Add();
            CellSelection allCellsToCopy = templateWorkSheet.Cells[0, 0, 10, 3];
            templateWorkSheet.Names.Add("MyField", "=Sheet1!$A$1", new CellIndex(0, 0), "My Field");
            templateWorkSheet.Cells[0, 0].SetValue(5);
            templateWorkSheet.Cells[0, 1].SetValue(1);
            templateWorkSheet.Cells[1, 1].SetValue(2);
            templateWorkSheet.Cells[2, 1].SetValue(3);
            templateWorkSheet.Cells[3, 1].SetValue(3);
            templateWorkSheet.Cells[1, 0].SetValue(formula);
            WorksheetFragment copyFragment = allCellsToCopy.Copy();

            Worksheet worksheet = workbook.Worksheets.Add();
            workbook.ActiveWorksheet = worksheet;
            worksheet.Name = "My New Sheet Added In Code";
            worksheet.Cells[copyFragment.CellRange].Paste(copyFragment, PasteOptions.All);
            worksheet.Names.Add("MyField", "='My New Sheet Added In Code'!$A$1", new CellIndex(0, 0), "My Field");

            var actualValue = templateWorkSheet.Cells[1, 0].GetValue().Value.RawValue;

            Assert.AreEqual(formula, actualValue);
        }

        [TestMethod]
        public void TranslateNamedRangeWithRowColumnAbsoluteValueSameWorksheetTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Names.Add("C3Name", "=$A$1", new CellIndex(2, 2));
            InputSampleCellValues(sheet1, new CellRange(0, 0, 1, 1));

            string nameCellValue = "=C3Name";
            sheet1.Cells[2, 2].SetValue(nameCellValue);
            sheet1.Cells[2, 3].SetValue(nameCellValue);
            sheet1.Cells[3, 2].SetValue(nameCellValue);
            sheet1.Cells[3, 3].SetValue(nameCellValue);

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 3), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 3), nameCellValue, "1");
        }

        [TestMethod]
        public void TranslateNamedRangeWithColumnAbsoluteValueSameWorksheetTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Names.Add("C3Name", "=$A1", new CellIndex(2, 2));
            InputSampleCellValues(sheet1, new CellRange(0, 0, 1, 1));

            string nameCellValue = "=C3Name";
            sheet1.Cells[2, 2].SetValue(nameCellValue);
            sheet1.Cells[2, 3].SetValue(nameCellValue);
            sheet1.Cells[3, 2].SetValue(nameCellValue);
            sheet1.Cells[3, 3].SetValue(nameCellValue);

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 3), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 2), nameCellValue, "3");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 3), nameCellValue, "3");
        }

        [TestMethod]
        public void TranslateNamedRangeWithRowAbsoluteValueSameWorksheetTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Names.Add("C3Name", "=A$1", new CellIndex(2, 2));
            InputSampleCellValues(sheet1, new CellRange(0, 0, 1, 1));

            string nameCellValue = "=C3Name";
            sheet1.Cells[2, 2].SetValue(nameCellValue);
            sheet1.Cells[2, 3].SetValue(nameCellValue);
            sheet1.Cells[3, 2].SetValue(nameCellValue);
            sheet1.Cells[3, 3].SetValue(nameCellValue);

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 3), nameCellValue, "2");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 3), nameCellValue, "2");
        }

        [TestMethod]
        public void TranslateNamedRangeWithRelativeValueSameWorksheetTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();

            sheet1.Names.Add("C3Name", "=A1", new CellIndex(2, 2));
            sheet1.Cells[0, 0].SetValue(1);
            sheet1.Cells[0, 1].SetValue(2);
            sheet1.Cells[1, 0].SetValue(3);
            sheet1.Cells[1, 1].SetValue(4);

            string nameCellValue = "=C3Name";
            sheet1.Cells[2, 2].SetValue(nameCellValue);
            sheet1.Cells[2, 3].SetValue(nameCellValue);
            sheet1.Cells[3, 2].SetValue(nameCellValue);
            sheet1.Cells[3, 3].SetValue(nameCellValue);

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 2), nameCellValue, "1");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(2, 3), nameCellValue, "2");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 2), nameCellValue, "3");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(3, 3), nameCellValue, "4");
        }

        [TestMethod]
        public void ChangeAbsoluteWorksheetNameInDefinedNamesWorksheetNameChangedTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();
            Worksheet sheet3 = workbook.Worksheets.Add();

            sheet1.Names.Add("A1Name", "=Sheet1!B1", new CellIndex(0, 0));
            sheet1.Names.Add("B1Name", "=Sheet2!B1", new CellIndex(0, 1));
            sheet1.Cells[0, 0].SetValue("=A1Name");
            sheet1.Cells[0, 1].SetValue("=B1Name");

            sheet2.Names.Add("A1Name", "=Sheet1!A1", new CellIndex(0, 0));
            sheet2.Names.Add("B1Name", "3", new CellIndex(0, 1));
            sheet2.Names.Add("C1Name", "=Sheet3!B1", new CellIndex(0, 2));
            sheet2.Cells[0, 0].SetValue("=A1Name");
            sheet2.Cells[0, 1].SetValue("=B1Name");
            sheet2.Cells[0, 2].SetValue("=C1Name");

            sheet3.Names.Add("A1Name", "=Sheet1!B1", new CellIndex(0, 0));
            sheet3.Names.Add("B1Name", "=Sheet2!B1", new CellIndex(0, 1));
            sheet3.Names.Add("C1Name", "=Sheet3!A1", new CellIndex(0, 2));
            sheet3.Cells[0, 0].SetValue("=A1Name");
            sheet3.Cells[0, 1].SetValue("=B1Name");
            sheet3.Cells[0, 2].SetValue("=C1Name");

            sheet1.Name = "NewName1";
            sheet2.Name = "NewName2";

            ExpressionTestHelper.AssertCellValueWithName(sheet1, new CellIndex(0, 0), "A1Name", "=NewName1!B1", "3");
            ExpressionTestHelper.AssertCellValueWithName(sheet1, new CellIndex(0, 1), "B1Name", "=NewName2!B1", "3");

            ExpressionTestHelper.AssertCellValueWithName(sheet2, new CellIndex(0, 0), "A1Name", "=NewName1!A1", "3");
            ExpressionTestHelper.AssertCellValueWithName(sheet2, new CellIndex(0, 1), "B1Name", "=3", "3");
            ExpressionTestHelper.AssertCellValueWithName(sheet2, new CellIndex(0, 2), "C1Name", "=Sheet3!B1", "3");

            ExpressionTestHelper.AssertCellValueWithName(sheet3, new CellIndex(0, 0), "A1Name", "=NewName1!B1", "3");
            ExpressionTestHelper.AssertCellValueWithName(sheet3, new CellIndex(0, 1), "B1Name", "=NewName2!B1", "3");
            ExpressionTestHelper.AssertCellValueWithName(sheet3, new CellIndex(0, 2), "C1Name", "=Sheet3!A1", "3");
        }

        [TestMethod]
        public void ChangeNonAbsoluteWorksheetNameInDefinedNamesWhenWorksheetNameChangedTest()
        {
            Workbook workbook = new Workbook();

            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();

            sheet1.Names.Add("B2Name", "=A1", new CellIndex(1, 1));
            sheet2.Names.Add("B2Name", "=A1", new CellIndex(1, 1));

            sheet1.Cells[0, 0].SetValue("sheet1A1");
            sheet1.Cells[1, 1].SetValue("=B2Name");
            sheet2.Cells[0, 0].SetValue("sheet2A1");
            sheet2.Cells[1, 1].SetValue("=B2Name");

            sheet1.Name = "NewName1";
            sheet2.Name = "NewName2";

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(1, 1), "=B2Name", "sheet1A1");
            ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(1, 1), "=B2Name", "sheet2A1");
        }

        [TestMethod]
        public void CreateNamedRageContainingCellsWithLongStringValuesAssertValuesIsCorrect()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            for (int i = 0; i < 5; i++)
            {
                worksheet.Cells[i, 0].SetValue("ValueOfCellAtRow" + i + "Column0");
            }

            var definedName = worksheet.Names.Add("test", "=$A$1:$A$5", new CellIndex(0, 0)) as DefinedName;

            string expectedValue = "{\"\"ValueOfCellAtRow0Column0\"\";\"\"ValueOfCellAtRow1Column0\"\";\"\"ValueOfCellAtRow2Column0\"\";\"\"ValueOfCellAtRow3Column0\"\";\"\"ValueOfCellAtRow4Column0\"\"}";

            Assert.AreEqual(expectedValue, definedName.Value);
        }

        private static void InputSampleCellValues(Worksheet worksheet, CellRange cellRange, int startValue = 1)
        {
            int value = startValue;

            for (int row = cellRange.FromIndex.RowIndex; row <= cellRange.ToIndex.RowIndex; row++)
            {
                for (int column = cellRange.FromIndex.ColumnIndex; column <= cellRange.ToIndex.ColumnIndex; column++)
                {
                    worksheet.Cells[row, column].SetValue(value++);
                }
            }
        }
    }
}
