﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    /// <summary>
    /// Version 1.0
    /// Tests: NumberExpression, UnaryPlusExpression, UnaryMinusExpression, AdditionExpression, SubstractionExpression, PowerExpression
    /// </summary>
    [TestClass]
    public class RadExpressionTests
    {
        #region Error Propagation, Empty and Wrong Number of Arguments Tests

        [TestMethod]
        public void OperatorErrorPropagationTest1()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("-E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("+E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2&E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2+E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2>E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2=E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2^E2", ErrorExpressions.NumberError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("B2/E2", ErrorExpressions.NumberError, worksheet);
        }

        [TestMethod]
        public void OperatorErrorPropagationTest2()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("-D2", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("+D2", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A2*D2", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A2-D2", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A2-D2", ErrorExpressions.DivisionByZero, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("A2&D2", ErrorExpressions.DivisionByZero, worksheet);
        }

        [TestMethod]
        public void OperatorErrorPropagationTest3()
        {
            Worksheet worksheet = ExpressionTestHelper.Worksheet;

            ExpressionTestHelper.AssertErrorExpressionResult("C2-A2&B2", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("-(C2-A2)", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("(C2-A2)*B2", ErrorExpressions.ValueError, worksheet);
            ExpressionTestHelper.AssertErrorExpressionResult("(C2-A2)/B2", ErrorExpressions.ValueError, worksheet);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest1()
        {
            ExpressionTestHelper.AssertExpressionResult("-A5", typeof(UnaryMinusExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("+A5", typeof(UnaryPlusExpression), 0);

        }

        [TestMethod]
        public void OperatorEmptyValuesTest2()
        {
            ExpressionTestHelper.AssertExpressionResult("A5+A5", typeof(AdditionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("10+A5", typeof(AdditionExpression), 10);
            ExpressionTestHelper.AssertExpressionResult("-10+A5", typeof(AdditionExpression), -10);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest3()
        {
            ExpressionTestHelper.AssertExpressionResult("A5-A5", typeof(SubtractionExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("10-A5", typeof(SubtractionExpression), 10);
            ExpressionTestHelper.AssertExpressionResult("-10-A5", typeof(SubtractionExpression), -10);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest4()
        {
            ExpressionTestHelper.AssertExpressionResult("A5*A5", typeof(MultiplicationExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("10*A5", typeof(MultiplicationExpression), 0);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest5()
        {
            ExpressionTestHelper.AssertExpressionResult("A5/10", typeof(DivisionExpression), 0);
            ExpressionTestHelper.AssertErrorExpressionResult("10/A5", ErrorExpressions.DivisionByZero);
            ExpressionTestHelper.AssertErrorExpressionResult("A5/A5", ErrorExpressions.DivisionByZero);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest6()
        {
            ExpressionTestHelper.AssertExpressionResult("A5^1", typeof(PowerExpression), 0);
            ExpressionTestHelper.AssertExpressionResult("5^A5", typeof(PowerExpression), 1);
            ExpressionTestHelper.AssertErrorExpressionResult("A5^A5", ErrorExpressions.NumberError);
            ExpressionTestHelper.AssertErrorExpressionResult("A5^-2", ErrorExpressions.DivisionByZero);
        }

        [TestMethod]
        public void OperatorEmptyValuesTest7()
        {
            ExpressionTestHelper.AssertExpressionResult("A5&A5", typeof(AmpersandExpression), "");
            ExpressionTestHelper.AssertExpressionResult("\"string\"&A5", typeof(AmpersandExpression), "string");
            ExpressionTestHelper.AssertExpressionResult("A5&\"string\"", typeof(AmpersandExpression), "string");
        }

        [TestMethod]
        public void PlusMinusOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("-", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("+", typeof(ExpressionException));
        }

        [TestMethod]
        public void PlusOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("5+", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("true+", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8+10+15+", typeof(ExpressionException));
        }

        [TestMethod]
        public void MinusOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("10-", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("false-", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8-10-15-", typeof(ExpressionException));
        }

        [TestMethod]
        public void MultiplicationOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("*", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10*", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("false*", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("*false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8*10*15*", typeof(ExpressionException));
        }

        [TestMethod]
        public void DivisionOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("/", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10/", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("/false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("false/", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8/10/15/", typeof(ExpressionException));
        }

        [TestMethod]
        public void PowerOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("^", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10^", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("false^", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("^false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8^10^15^", typeof(ExpressionException));
        }

        [TestMethod]
        public void AmpersandOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("&", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10&", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("false&", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("&false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8&10&15&", typeof(ExpressionException));
        }

        [TestMethod]
        public void EqualOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("=false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8=10=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8=10=15=", typeof(ExpressionException));
        }

        [TestMethod]
        public void GreaterThanOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException(">", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10>", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(">false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8>10>", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8>10>15>", typeof(ExpressionException));
        }

        [TestMethod]
        public void GreaterThanOtEqualToOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException(">=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10>=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(">=false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8>=10>=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8>=10>=15>=", typeof(ExpressionException));
        }

        [TestMethod]
        public void LessThanOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("<", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10<", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("<false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<10<", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<10<15<", typeof(ExpressionException));
        }

        [TestMethod]
        public void LessThanOrEqualToOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("<=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10<=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("<=false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<=10<=", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<=10<=15<=", typeof(ExpressionException));
        }

        [TestMethod]
        public void NotEqualOperatorNumberOfArgumentsValidationTest()
        {
            ExpressionTestHelper.AssertThrownException("<>", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("10<>", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("<>false", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<>10<>", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("8<>10<>15<>", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidFormulaSymbolsAtTheEndOfTheExpressionTest()
        {
            ExpressionTestHelper.AssertThrownException(@"5\0", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(@"5\A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(@"5'A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(@"5""A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(@"5@A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException(@"5!A1", typeof(ExpressionException));
        }

        #endregion


        #region Parser Tests

        [TestMethod]
        public void InvalidParserStringTest()
        {
            ExpressionTestHelper.AssertThrownException("5+5 1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("1.2.3", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("{}", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("SUM({})", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("OR(})", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("AND({", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidParserCellReferencesTest1()
        {
            ExpressionTestHelper.AssertThrownException("abc'abc!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'test'", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'test'!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("a\"bc", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("a'bc", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("a'bc'!A1", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidParserCellReferencesTest2()
        {
            ExpressionTestHelper.AssertThrownException("#r!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'Sheet1'!#REF!#REF!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#REF!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'sheet1'!#REF", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'sheet1'!#", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'sheet1'!#R", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("#REF!#REF!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("123Sheet!A1", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidParserCellReferencesTest3()
        {
            ExpressionTestHelper.AssertThrownException(" !A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("a!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"are\"!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("\"are\"'1'!A1", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidParserCellReferencesTest4()
        {
            ExpressionTestHelper.AssertThrownException("-+=&@#%^()!.<>~;  __!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("='__!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("{1}!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("={1}A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("=[]!A1", typeof(ExpressionException));

        }

        [TestMethod]
        public void InvalidParserCellReferencesTest5()
        {
            ExpressionTestHelper.AssertThrownException("'abc''!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'abc'''!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'''abc'!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'''abc'''!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("''''!A1", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'''!A1", typeof(ExpressionException));
        }

        [TestMethod]
        public void InvalidParserCellReferencesTest6()
        {
            ExpressionTestHelper.AssertThrownException("'Sheet2'!{A1}", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'Sheet2'!4234", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'Sheet2'!\"A1\"", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'Sheet2'!!", typeof(ExpressionException));
            ExpressionTestHelper.AssertThrownException("'Sheet2'!'Sheet2'!A1", typeof(ExpressionException));
        }

        //[TestMethod]
        //public void ValidVariabledExpressionTest1()
        //{
        //    ExpressionTestHelper.AssertExpressionResult("'Sheet2'!abd", typeof(VariableExpression));
        //}

        [TestMethod]
        public void ExpressionStartPlusSymbolTest()
        {
            ExpressionTestHelper.AssertExpressionResult("+5+5", typeof(AdditionExpression), 10, addFunctionStart: false);
        }

        [TestMethod]
        public void ExpressionStartMinusSymbolTest()
        {
            ExpressionTestHelper.AssertExpressionResult("-5+3", typeof(AdditionExpression), -2, addFunctionStart: false);
        }

        [TestMethod]
        public void ExpressionStartEqualSymbolTest()
        {
            ExpressionTestHelper.AssertExpressionResult("=5+3", typeof(AdditionExpression), 8, addFunctionStart: false);
        }

        [TestMethod]
        public void ExpressionStartEqualAndMinusSymbolsTest()
        {
            ExpressionTestHelper.AssertExpressionResult("=-5+3", typeof(AdditionExpression), -2, addFunctionStart: false);
        }

        [TestMethod]
        public void ExpressionNotStartNumberTest()
        {
            RadExpression expression = ExpressionTestHelper.ParseExpression("5+5", cellIndex: new CellIndex(0, 0));
            Assert.AreEqual(null, expression);
        }

        [TestMethod]
        public void ExpressionNotStartNumberTest2()
        {
            RadExpression expression = ExpressionTestHelper.ParseExpression("10-5", cellIndex: new CellIndex(0, 0));
            Assert.AreEqual(null, expression);
        }

        // TODO: Move to FormulaCellValue tests!

        [TestMethod]
        public void UpperCaseFunctionsAndCellReferenceNamesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            CellIndex cellIndex = new CellIndex(0, 0);
            FormulaCellValue cellValue;

            cellValue = TestHelper.CreateFormulaCellValue("=sum(b1:c1)", worksheet, cellIndex, false);
            Assert.AreEqual(cellValue.RawValue, "=SUM(B1:C1)");

            cellValue = TestHelper.CreateFormulaCellValue("=aBs(5 * aaa123)", worksheet, cellIndex, false);
            Assert.AreEqual(cellValue.RawValue, "=ABS(5 * AAA123)");

            cellValue = TestHelper.CreateFormulaCellValue("=Error.Type(z100)", worksheet, cellIndex, false);
            Assert.AreEqual(cellValue.RawValue, "=ERROR.TYPE(Z100)");
        }

        #endregion


        #region Expression ToString Tests

        [TestMethod]
        public void ExpressionToStringOperatorsWithDifferentPrecedenceTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(3+2)*4");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=4 * (3+2)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(4 + 5) * (3+2)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=-(3+5)");
        }

        [TestMethod]
        public void ExpressionToStringMultiplicationTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2*2*3");
        }

        [TestMethod]
        public void ExpressionToStringDivisionTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2/2/3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2/(2/3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(2/2)/3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=((2/2/3))");
        }

        [TestMethod]
        public void ExpressionToStringExponentiationTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=3^(5^3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2^(2^3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(2^2)^3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2^2^3");
        }

        [TestMethod]
        public void ExpressionToStringOperatorsWithSamePrecedenceAndParenthesisTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(2)*(2)*(3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=((2)*(2))*(3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(((2)*((2))*(3)))");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2*(3*(((4))))*5");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(1 + (2 + (3 + (4 + (5 + 6)))))");
        }

        [TestMethod]
        public void ExpressionToStringOperatorsWithSamePrecedenceTest()
        {
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2*2/3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2*(2/3)");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(2*2)/3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=2*2/3*3");
            ExpressionTestHelper.AssertExpressionToStringCorrectness("=(2*2)/(3*3)");
        }

        #endregion


        #region Worksheet Name Changed Tests

        [TestMethod]
        public void ChangeAbsoluteWorksheetNameReferencesWhenWorksheetNameChangedTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();
            Worksheet sheet3 = workbook.Worksheets.Add();

            sheet1.Cells[0, 0].SetValue("=Sheet1!B1");
            sheet1.Cells[0, 1].SetValue("=Sheet2!B1");

            sheet2.Cells[0, 0].SetValue("=Sheet1!A1");
            sheet2.Cells[0, 1].SetValue(3);
            sheet2.Cells[0, 2].SetValue("=Sheet3!B1");

            sheet3.Cells[0, 0].SetValue("=Sheet1!B1");
            sheet3.Cells[0, 1].SetValue("=Sheet2!B1");
            sheet3.Cells[0, 2].SetValue("=Sheet3!A1");

            sheet1.Name = "NewName1";
            sheet2.Name = "NewName2";

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(0, 0), "=NewName1!B1", "3");
            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(0, 1), "=NewName2!B1", "3");

            ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(0, 0), "=NewName1!A1", "3");
            ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(0, 1), "3", "3");
            ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(0, 2), "=Sheet3!B1", "3");

            ExpressionTestHelper.AssertCellValue(sheet3, new CellIndex(0, 0), "=NewName1!B1", "3");
            ExpressionTestHelper.AssertCellValue(sheet3, new CellIndex(0, 1), "=NewName2!B1", "3");
            ExpressionTestHelper.AssertCellValue(sheet3, new CellIndex(0, 2), "=Sheet3!A1", "3");
        }

        [TestMethod]
        public void ChangeNonAbsoluteWorksheetNameReferencesWhenWorksheetNameChangedTest()
        {
            Workbook workbook = new Workbook();
            Worksheet sheet1 = workbook.Worksheets.Add();
            Worksheet sheet2 = workbook.Worksheets.Add();

            sheet1.Cells[0, 0].SetValue("=A2");
            sheet1.Cells[1, 0].SetValue(3);
            sheet1.Name = "NewSheet1";

            sheet2.Cells[0, 0].SetValue("=A2");
            sheet2.Cells[1, 0].SetValue(5);
            sheet2.Name = "NewSheet2";

            ExpressionTestHelper.AssertCellValue(sheet1, new CellIndex(0, 0), "=A2", "3");
            ExpressionTestHelper.AssertCellValue(sheet2, new CellIndex(0, 0), "=A2", "5");
        }

        [TestMethod]
        public void SumIfCalculationWorksheetNameChangedTest()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            worksheet.Cells[0, 0].SetValue("sales");
            worksheet.Cells[0, 1].SetValue(815);
            
            string sumIfCellValue = "=SUMIF(A1{ls}\"sales\"{ls}B1)".ReplaceCultureDependentSeparators();
            worksheet.Cells[2, 2].SetValue(sumIfCellValue);

            ExpressionTestHelper.AssertCellValue(worksheet, new CellIndex(2, 2), sumIfCellValue, "815");
            worksheet.Name = "NewWorksheetName";
            ExpressionTestHelper.AssertCellValue(worksheet, new CellIndex(2, 2), sumIfCellValue, "815");
        }

        #endregion
    }
}
