﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Sorting;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Sorting
{
    [TestClass]
    public class SortingInsertCells
    {
        private Worksheet worksheet;
        private readonly CellRange range = new CellRange(6, 5, 19, 7);
        private readonly ISortCondition firstCondition = new ValuesSortCondition(0, SortOrder.Ascending);
        private readonly ISortCondition secondCondition = new ValuesSortCondition(1, SortOrder.Descending);
        private readonly ISortCondition thirdCondition = new ValuesSortCondition(2, SortOrder.Ascending);

        private readonly List<string> firstColumnSortedValues = new List<string>() { "a", "a", "a", "a", "a", "b", "b", "b", "b", "c", "c", "c", "c", "c" };
        private readonly List<string> secondColumnSortedValues = new List<string>() { "3", "2", "2", "1", "1", "3", "2", "2", "1", "3", "3", "2", "2", "1" };
        private readonly List<string> thirdColumnSortedValues = new List<string>() { "a", "a", "c", "a", "c", "b", "b", "c", "b", "a", "c", "a", "a", "c" };

        [TestInitialize]
        public void EnterData()
        {
            this.worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[6, 5, 6, 5].SetValue("a");
            worksheet.Cells[7, 5, 7, 5].SetValue("b");
            worksheet.Cells[8, 5, 8, 5].SetValue("a");
            worksheet.Cells[9, 5, 9, 5].SetValue("a");
            worksheet.Cells[10, 5, 10, 5].SetValue("b");
            worksheet.Cells[11, 5, 11, 5].SetValue("b");
            worksheet.Cells[12, 5, 12, 5].SetValue("b");
            worksheet.Cells[13, 5, 13, 5].SetValue("c");
            worksheet.Cells[14, 5, 14, 5].SetValue("c");
            worksheet.Cells[15, 5, 15, 5].SetValue("c");
            worksheet.Cells[16, 5, 16, 5].SetValue("a");
            worksheet.Cells[17, 5, 17, 5].SetValue("a");
            worksheet.Cells[18, 5, 18, 5].SetValue("c");
            worksheet.Cells[19, 5, 19, 5].SetValue("c");

            worksheet.Cells[6, 6, 6, 6].SetValue("1");
            worksheet.Cells[7, 6, 7, 6].SetValue("1");
            worksheet.Cells[8, 6, 8, 6].SetValue("2");
            worksheet.Cells[9, 6, 9, 6].SetValue("3");
            worksheet.Cells[10, 6, 10, 6].SetValue("2");
            worksheet.Cells[11, 6, 11, 6].SetValue("2");
            worksheet.Cells[12, 6, 12, 6].SetValue("3");
            worksheet.Cells[13, 6, 13, 6].SetValue("3");
            worksheet.Cells[14, 6, 14, 6].SetValue("3");
            worksheet.Cells[15, 6, 15, 6].SetValue("1");
            worksheet.Cells[16, 6, 16, 6].SetValue("1");
            worksheet.Cells[17, 6, 17, 6].SetValue("2");
            worksheet.Cells[18, 6, 18, 6].SetValue("2");
            worksheet.Cells[19, 6, 19, 6].SetValue("2");

            worksheet.Cells[6, 7, 6, 7].SetValue("a");
            worksheet.Cells[7, 7, 7, 7].SetValue("b");
            worksheet.Cells[8, 7, 8, 7].SetValue("c");
            worksheet.Cells[9, 7, 9, 7].SetValue("a");
            worksheet.Cells[10, 7, 10, 7].SetValue("c");
            worksheet.Cells[11, 7, 11, 7].SetValue("b");
            worksheet.Cells[12, 7, 12, 7].SetValue("b");
            worksheet.Cells[13, 7, 13, 7].SetValue("c");
            worksheet.Cells[14, 7, 14, 7].SetValue("a");
            worksheet.Cells[15, 7, 15, 7].SetValue("c");
            worksheet.Cells[16, 7, 16, 7].SetValue("c");
            worksheet.Cells[17, 7, 17, 7].SetValue("a");
            worksheet.Cells[18, 7, 18, 7].SetValue("a");
            worksheet.Cells[19, 7, 19, 7].SetValue("a");

            worksheet.Cells[this.range].Sort(this.firstCondition, this.secondCondition, this.thirdCondition);
        }

        [TestMethod]
        public void InsertRangeBelowAndRightToSortedShiftDown()
        {
            CellRange insertRange = new CellRange(20, 20, 25, 25);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Down);

            this.AssertInitialStateIsUnchanged();
        }

        [TestMethod]
        public void InsertLargerRangeAboveSortedShiftDown()
        {
            CellRange insertRange = new CellRange(3, 3, 3, 13);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Down);

            CellRange expectedSortRange = this.range.Offset(1, 0);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void InsertLargerRangeAboveSortedShiftDownUndoRedo()
        {
            this.InsertLargerRangeAboveSortedShiftDown();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            CellRange expectedSortRange = this.range.Offset(1, 0);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeAboveSortedShiftUp()
        {
            CellRange removeRange = new CellRange(3, 3, 3, 13);

            worksheet.Cells.Remove(removeRange, RemoveShiftType.Up);

            CellRange expectedSortRange = this.range.Offset(-1, 0);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeAboveSortedShiftUpUndoRedo()
        {
            this.DeleteLargerRangeAboveSortedShiftUp();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            CellRange expectedSortRange = this.range.Offset(-1, 0);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void InsertLargerRangeToLeftOfSortedShiftRight()
        {
            CellRange insertRange = new CellRange(3, 3, 20, 3);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Right);

            CellRange expectedSortRange = this.range.Offset(0, 1);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void InsertLargerRangeToLeftOfSortedShiftRightUndoRedo()
        {
            this.InsertLargerRangeToLeftOfSortedShiftRight();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            CellRange expectedSortRange = this.range.Offset(0, 1);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeToLeftOfSortedShiftLeft()
        {
            CellRange removeRange = new CellRange(3, 3, 20, 3);

            worksheet.Cells.Remove(removeRange, RemoveShiftType.Left);

            CellRange expectedSortRange = this.range.Offset(0, -1);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeToLeftOfSortedShiftLeftUndoRedo()
        {
            this.DeleteLargerRangeToLeftOfSortedShiftLeft();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            CellRange expectedSortRange = this.range.Offset(0, -1);
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(expectedSortRange);
        }

        [TestMethod]
        public void InsertLargerRangeWithinSortedShiftRight()
        {
            CellRange insertRange = new CellRange(3, 6, 20, 6);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Right);

            this.AssertResultInsertLargerRangeWithinSortedShiftRight();
        }

        [TestMethod]
        public void InsertLargerRangeWithinSortedShiftRightUndoRedo()
        {
            this.InsertLargerRangeWithinSortedShiftRight();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertLargerRangeWithinSortedShiftRight();
        }

        private void AssertResultInsertLargerRangeWithinSortedShiftRight()
        {
            CellRange expectedSortRange = new CellRange(this.range.FromIndex, this.range.ToIndex.Offset(0, 1));
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            
            Assert.AreEqual(this.firstCondition, worksheet.SortState.SortConditions.ElementAt(0));
            Assert.AreEqual(new ValuesSortCondition(2, SortOrder.Descending), worksheet.SortState.SortConditions.ElementAt(1));
            Assert.AreEqual(new ValuesSortCondition(3, SortOrder.Ascending), worksheet.SortState.SortConditions.ElementAt(2));

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                new List<string>(),
                this.secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinSortedShiftLeft()
        {
            CellRange removeRange = new CellRange(3, 6, 20, 6);

            worksheet.Cells.Remove(removeRange, RemoveShiftType.Left);

            this.AssertResultDeleteLargerRangeWithinSortedShiftLeft();
        }

        [TestMethod]
        public void DeleteLargerRangeWithinSortedShiftLeftUndoRedo()
        {
            this.DeleteLargerRangeWithinSortedShiftLeft();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteLargerRangeWithinSortedShiftLeft();
        }

        private void AssertResultDeleteLargerRangeWithinSortedShiftLeft()
        {
            CellRange expectedSortRange = new CellRange(this.range.FromIndex, this.range.ToIndex.Offset(0, -1));
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);
            
            Assert.AreEqual(this.firstCondition, worksheet.SortState.SortConditions.ElementAt(0));
            Assert.AreEqual(new ValuesSortCondition(1, SortOrder.Ascending), worksheet.SortState.SortConditions.ElementAt(1));

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void InsertLargerRangeWithinSortedShiftDown()
        {
            CellRange insertRange = new CellRange(7, 3, 7, 13);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Down);

            this.AssertResultInsertLargerRangeWithinSortedShiftDown();
        }

        [TestMethod]
        public void InsertLargerRangeWithinSortedShiftDownUndoRedo()
        {
            this.InsertLargerRangeWithinSortedShiftDown();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertLargerRangeWithinSortedShiftDown();
        }

        private void AssertResultInsertLargerRangeWithinSortedShiftDown()
        {
            CellRange expectedSortRange = new CellRange(this.range.FromIndex, this.range.ToIndex.Offset(1, 0));
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> firstColumnSortedValues = new List<string>(this.firstColumnSortedValues);
            firstColumnSortedValues.Insert(1, string.Empty);
            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            secondColumnSortedValues.Insert(1, string.Empty);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);
            thirdColumnSortedValues.Insert(1, string.Empty);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinSortedShiftUp()
        {
            CellRange removeRange = new CellRange(7, 3, 7, 13);

            worksheet.Cells.Remove(removeRange, RemoveShiftType.Up);

            this.AssertResultDeleteLargerRangeWithinSortedShiftUp();
        }

        [TestMethod]
        public void DeleteLargerRangeWithinSortedShiftUpUndoRedo()
        {
            this.DeleteLargerRangeWithinSortedShiftUp();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteLargerRangeWithinSortedShiftUp();
        }

        private void AssertResultDeleteLargerRangeWithinSortedShiftUp()
        {
            CellRange expectedSortRange = new CellRange(this.range.FromIndex, this.range.ToIndex.Offset(-1, 0));
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> firstColumnSortedValues = new List<string>(this.firstColumnSortedValues);
            firstColumnSortedValues.RemoveAt(1);
            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            secondColumnSortedValues.RemoveAt(1);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);
            thirdColumnSortedValues.RemoveAt(1);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void InsertSmallerRangeAboveSortedShiftDown()
        {
            CellRange insertRange = new CellRange(3, 6, 3, 6);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Down);

            this.AssertResultInsertSmallerRangeAboveSortedShiftDown();
        }

        [TestMethod]
        public void InsertSmallerRangeAboveSortedShiftDownUndoRedo()
        {
            this.InsertSmallerRangeAboveSortedShiftDown();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertSmallerRangeAboveSortedShiftDown();
        }

        private void AssertResultInsertSmallerRangeAboveSortedShiftDown()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            secondColumnSortedValues.Insert(0, string.Empty);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeAboveSortedShiftUp()
        {
            CellRange insertRange = new CellRange(3, 6, 3, 6);

            worksheet.Cells.Remove(insertRange, RemoveShiftType.Up);

            this.AssertResultDeleteSmallerRangeAboveSortedShiftUp();
        }

        [TestMethod]
        public void DeleteSmallerRangeAboveSortedShiftUpUndoRedo()
        {
            this.DeleteSmallerRangeAboveSortedShiftUp();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteSmallerRangeAboveSortedShiftUp();
        }

        private void AssertResultDeleteSmallerRangeAboveSortedShiftUp()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            secondColumnSortedValues.RemoveAt(0);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void InsertSmallerRangeToLeftOfSortedShiftRight()
        {
            CellRange insertRange = new CellRange(7, 3, 7, 3);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Right);

            this.AssertResultInsertSmallerRangeToLeftOfSortedShiftRight();
        }

        [TestMethod]
        public void InsertSmallerRangeToLeftOfSortedShiftRightUndoRedo()
        {
            this.InsertSmallerRangeToLeftOfSortedShiftRight();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertSmallerRangeToLeftOfSortedShiftRight();
        }

        private void AssertResultInsertSmallerRangeToLeftOfSortedShiftRight()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> firstColumnSortedValues = new List<string>(this.firstColumnSortedValues);
            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);

            thirdColumnSortedValues[1] = secondColumnSortedValues[1];
            secondColumnSortedValues[1] = firstColumnSortedValues[1];
            firstColumnSortedValues[1] = string.Empty;

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeToLeftOfSortedShiftLeft()
        {
            CellRange insertRange = new CellRange(7, 3, 7, 3);

            worksheet.Cells.Remove(insertRange, RemoveShiftType.Left);

            this.AssertResultDeleteSmallerRangeToLeftOfSortedShiftLeft();
        }

        [TestMethod]
        public void DeleteSmallerRangeToLeftOfSortedShiftLeftUndoRedo()
        {
            this.DeleteSmallerRangeToLeftOfSortedShiftLeft();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteSmallerRangeToLeftOfSortedShiftLeft();
        }

        private void AssertResultDeleteSmallerRangeToLeftOfSortedShiftLeft()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> firstColumnSortedValues = new List<string>(this.firstColumnSortedValues);
            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);

            firstColumnSortedValues[1] = secondColumnSortedValues[1];
            secondColumnSortedValues[1] = thirdColumnSortedValues[1];
            thirdColumnSortedValues[1] = string.Empty;

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void InsertSmallerRangeWithinSortedShiftRight()
        {
            CellRange insertRange = new CellRange(7, 6, 7, 6);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Right);

            this.AssertResultInsertSmallerRangeWithinSortedShiftRight();
        }

        [TestMethod]
        public void InsertSmallerRangeWithinSortedShiftRightUndoRedo()
        {
            this.InsertSmallerRangeWithinSortedShiftRight();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertSmallerRangeWithinSortedShiftRight();
        }

        private void AssertResultInsertSmallerRangeWithinSortedShiftRight()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);

            thirdColumnSortedValues[1] = secondColumnSortedValues[1];
            secondColumnSortedValues[1] = string.Empty;

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinSortedShiftLeft()
        {
            CellRange insertRange = new CellRange(7, 6, 7, 6);

            worksheet.Cells.Remove(insertRange, RemoveShiftType.Left);

            this.AssertResultDeleteSmallerRangeWithinSortedShiftLeft();
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinSortedShiftLeftUndoRedo()
        {
            this.DeleteSmallerRangeWithinSortedShiftLeft();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteSmallerRangeWithinSortedShiftLeft();
        }

        private void AssertResultDeleteSmallerRangeWithinSortedShiftLeft()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            List<string> thirdColumnSortedValues = new List<string>(this.thirdColumnSortedValues);

            secondColumnSortedValues[1] = thirdColumnSortedValues[1];
            thirdColumnSortedValues[1] = string.Empty;

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void InsertSmallerRangeWithinSortedShiftDown()
        {
            CellRange insertRange = new CellRange(7, 6, 7, 6);

            worksheet.Cells.Insert(insertRange, InsertShiftType.Down);

            this.AssertResultInsertSmallerRangeWithinSortedShiftDown();
        }

        [TestMethod]
        public void InsertSmallerRangeWithinSortedShiftDownUndoRedo()
        {
            this.InsertSmallerRangeWithinSortedShiftDown();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultInsertSmallerRangeWithinSortedShiftDown();
        }

        private void AssertResultInsertSmallerRangeWithinSortedShiftDown()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);
            secondColumnSortedValues.Insert(1, string.Empty);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinSortedShiftUp()
        {
            CellRange insertRange = new CellRange(7, 6, 7, 6);

            worksheet.Cells.Remove(insertRange, RemoveShiftType.Up);

            this.AssertResultDeleteSmallerRangeWithinSortedShiftUp();
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinSortedShiftUpUndoRedo()
        {
            this.DeleteSmallerRangeWithinSortedShiftUp();

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            this.AssertResultDeleteSmallerRangeWithinSortedShiftUp();
        }

        private void AssertResultDeleteSmallerRangeWithinSortedShiftUp()
        {
            CellRange expectedSortRange = this.range;
            Assert.AreEqual(expectedSortRange, worksheet.SortState.SortRange);

            this.AssertRelativePositionOfTheSortConditionsIsPreserved();

            List<string> secondColumnSortedValues = new List<string>(this.secondColumnSortedValues);

            secondColumnSortedValues.RemoveAt(1);

            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues,
                secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, expectedSortRange);
        }

        [TestMethod]
        public void DeleteEntireSortedRangeShiftUp()
        {
            worksheet.Cells.Remove(this.range, RemoveShiftType.Up);

            Assert.IsNull(worksheet.SortState.SortRange);
            Assert.AreEqual(0, worksheet.SortState.SortConditions.Count());
            Assert.AreEqual(string.Empty, 
                worksheet.Cells[this.range].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            this.worksheet.Workbook.History.Undo();

            this.AssertInitialStateIsUnchanged();

            this.worksheet.Workbook.History.Redo();

            Assert.IsNull(worksheet.SortState.SortRange);
            Assert.AreEqual(0, worksheet.SortState.SortConditions.Count());
            Assert.AreEqual(string.Empty, 
                worksheet.Cells[this.range].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        private void AssertInitialStateIsUnchanged()
        {
            Assert.AreEqual(this.range, worksheet.SortState.SortRange);
            this.AssertRelativePositionOfTheSortConditionsIsPreserved();
            this.AssertRelativePositionOfTheSortedValuesIsPreserved(this.range);
        }

        private void AssertRelativePositionOfTheSortConditionsIsPreserved()
        {
            Assert.AreEqual(this.firstCondition, worksheet.SortState.SortConditions.ElementAt(0));
            Assert.AreEqual(this.secondCondition, worksheet.SortState.SortConditions.ElementAt(1));
            Assert.AreEqual(this.thirdCondition, worksheet.SortState.SortConditions.ElementAt(2));
        }

        private void AssertRelativePositionOfTheSortedValuesIsPreserved(CellRange range)
        {
            List<List<string>> expectedValues = new List<List<string>>() 
            { 
                this.firstColumnSortedValues, 
                this.secondColumnSortedValues, 
                this.thirdColumnSortedValues
            };

            this.AssertPositionOfSortedValues(expectedValues, range);
        }

        private void AssertPositionOfSortedValues(List<List<string>> expectedValues, CellRange range)
        {
            int columnOffset = range.FromIndex.ColumnIndex;
            int rowOffset = range.FromIndex.RowIndex;

            for (int col = 0; col < expectedValues.Count; col++)
            {
                var expectedColumnValues = expectedValues[col];

                for (int row = 0; row < expectedColumnValues.Count; row++)
                {
                    int currentRow = row + rowOffset;
                    int currentCol = col + columnOffset;

                    string value = worksheet.Cells[currentRow, currentCol, currentRow, currentCol].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat);

                    if (expectedColumnValues.Count > row)
                    {
                        Assert.AreEqual(expectedColumnValues[row], value);
                    }
                }
            }
        }


    }
}
