﻿using System.Collections;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.Charts
{
    internal class MarkerEqualityComparer : SafeEqualityComparerBase<Marker>
    {
        private static readonly IEqualityComparer outlineEqualityComparer = new OutlineEqualityComparer();
        private static readonly IEqualityComparer fillEqualityComparer = new FillEqualityComparer();

        static MarkerEqualityComparer()
        {
            RegisterComparerFunction("Symbol", CompareSymbol);
            RegisterComparerFunction("Size", CompareSize);
            RegisterComparerFunction("Fill", CompareFill);
            RegisterComparerFunction("Outline", CompareOutline);
        }

        private static bool CompareSymbol(Marker marker1, Marker marker2)
        {
            bool result = marker1.Symbol == marker2.Symbol;

#if DEBUG
            System.Diagnostics.Debug.Assert(result);
#endif

            return result;
        }

        private static bool CompareSize(Marker marker1, Marker marker2)
        {
            bool result = marker1.Size == marker2.Size;

#if DEBUG
            System.Diagnostics.Debug.Assert(result);
#endif

            return result;
        }

        private static bool CompareFill(Marker marker1, Marker maker2)
        {
            var value1 = marker1.Fill;
            var value2 = maker2.Fill;

            if (value1 == null && value2 == null)
            {
                return true;
            }

            if (value1 == null || value2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif

                return false;
            }

            return fillEqualityComparer.Equals(value1, value2);
        }

        private static bool CompareOutline(Marker marker1, Marker maker2)
        {
            var value1 = marker1.Outline;
            var value2 = maker2.Outline;

            if (value1 == null && value2 == null)
            {
                return true;
            }

            if (value1 == null || value2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif

                return false;
            }

            return outlineEqualityComparer.Equals(value1, value2);
        }
    }
}
