﻿using System.Collections;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.Charts
{
    internal abstract class SeriesBaseEqualityComparer<T> : SafeEqualityComparerBase<T>
        where T : SeriesBase
    {
        private static readonly GenericTitleEqualityComparer titleEqualityComparer = new GenericTitleEqualityComparer();
        private static readonly GenericChartDataEqualityConverter chartDataEqualityConverter = new GenericChartDataEqualityConverter();
        private static readonly IEqualityComparer outlineEqualityComparer = new OutlineEqualityComparer();

        static SeriesBaseEqualityComparer()
        {
            RegisterComparerFunction("SeriesType", CompareSeriesType);
            RegisterComparerFunction("Title", CompareTitle);
            RegisterComparerFunction("Outline", CompareOutline);
        }

        private static bool CompareSeriesType(SeriesBase series1, SeriesBase series2)
        {
            bool result = series1.SeriesType == series2.SeriesType;

#if DEBUG
            System.Diagnostics.Debug.Assert(result);
#endif

            return result;
        }

        private static bool CompareTitle(SeriesBase series1, SeriesBase series2)
        {
            Title value1 = series1.Title;
            Title value2 = series2.Title;

            if (value1 == null && value2 == null)
            {
                return true;
            }

            if (value1 == null || value2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif

                return false;
            }

            return titleEqualityComparer.Equals(value1, value2);
        }

        private static bool CompareOutline(SeriesBase series1, SeriesBase series2)
        {
            Documents.Model.Drawing.Theming.Outline value1 = series1.Outline;
            Documents.Model.Drawing.Theming.Outline value2 = series2.Outline;

            if (value1 == null && value2 == null)
            {
                return true;
            }

            if (value1 == null || value2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif

                return false;
            }

            return outlineEqualityComparer.Equals(value1, value2);
        }

        protected static bool CompareIChartData(IChartData value1, IChartData value2)
        {
            if (value1 == null && value2 == null)
            {
                return true;
            }

            if (value1 == null || value2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif

                return false;
            }

            return chartDataEqualityConverter.Equals(value1, value2);
        }
    }
}
