﻿using System.Collections;
using System.Collections.Generic;
using Telerik.Windows.Documents.Model.Drawing.Charts;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.Charts
{
    internal abstract class SeriesGroupEqualityComparer<T> : SafeEqualityComparerBase<T>
        where T : SeriesGroup
    {
        private static readonly Dictionary<SeriesType, IEqualityComparer> seriesTypeToEqualityComparer = new Dictionary<SeriesType, IEqualityComparer>();

        static SeriesGroupEqualityComparer()
        {
            RegisterComparerFunction("Series", CompareSeriesGroupSeries);
            RegisterComparerFunction("SeriesType", CompareSeriesGroupSeriesType);

            seriesTypeToEqualityComparer[SeriesType.Scatter] = new ScatterSeriesEqualityComparer();
            seriesTypeToEqualityComparer[SeriesType.Bar] = new BarSeriesEqualityComparer();
            seriesTypeToEqualityComparer[SeriesType.Pie] = new PieSeriesEqualityComparer();
            seriesTypeToEqualityComparer[SeriesType.Line] = new LineSeriesEqualityComparer();
            seriesTypeToEqualityComparer[SeriesType.Bubble] = new BubbleSeriesEqualityComparer();
        }

        private static bool CompareSeriesGroupSeries(SeriesGroup seriesGroup1, SeriesGroup seriesGroup2)
        {
            SeriesCollection series1 = seriesGroup1.Series;
            SeriesCollection series2 = seriesGroup2.Series;

            if (series1 == null && series2 == null)
            {
                return true;
            }

            if (series1 == null || series2 == null)
            {
#if DEBUG
                System.Diagnostics.Debug.Assert(false);
#endif
                return false;
            }

            IEnumerator<SeriesBase> seriesEnumerator1 = series1.GetEnumerator();
            IEnumerator<SeriesBase> seriesEnumerator2 = series2.GetEnumerator();

            while (seriesEnumerator1.MoveNext() && seriesEnumerator2.MoveNext())
            {
                SeriesBase currentSeries1 = seriesEnumerator1.Current;
                SeriesBase currentSeries2 = seriesEnumerator2.Current;

                if (currentSeries1.SeriesType != currentSeries2.SeriesType)
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(false);
#endif
                    return false;
                }

                IEqualityComparer comparer = seriesTypeToEqualityComparer[currentSeries1.SeriesType];

                if (!comparer.Equals(currentSeries1, currentSeries2))
                {
#if DEBUG
                    System.Diagnostics.Debug.Assert(false);
#endif
                    return false;
                }
            }

            return true;
        }

        private static bool CompareSeriesGroupSeriesType(SeriesGroup seriesGroup1, SeriesGroup seriesGroup2)
        {
            bool result = seriesGroup1.SeriesType == seriesGroup2.SeriesType;

#if DEBUG
            System.Diagnostics.Debug.Assert(result);
#endif

            return result;
        }
    }
}
