﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities
{
    public static class CultureStringHelper
    {
        const string Semicolon = ";";
        const string Backslash = "\\";
        const string Comma = ",";

        private static readonly Dictionary<string, Func<string>> mapping;

        static CultureStringHelper()
        {
            mapping = new Dictionary<string, Func<string>>();
            RegisterMapping("{gs}", () => { return CultureInfo.CurrentCulture.NumberFormat.NumberGroupSeparator; });
            RegisterMapping("{ds}", () => { return CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator; });
            RegisterMapping("{ls}", () => { return GetListSeparator(CultureInfo.CurrentCulture); });
            RegisterMapping("{als}", () => { return GetListSeparator(CultureInfo.CurrentCulture) == Semicolon ? Backslash : GetListSeparator(CultureInfo.CurrentCulture); });
            RegisterMapping("{ars}", () => { return Semicolon; });
            RegisterMapping("{cs}", () => { return CultureInfo.CurrentCulture.NumberFormat.CurrencySymbol; });
        }

        private static string GetListSeparator(CultureInfo info)
        {
            string listSeparator = info.TextInfo.ListSeparator;
            if (listSeparator == info.NumberFormat.NumberDecimalSeparator && listSeparator == Comma)
            {
                return Semicolon;
            }

            return listSeparator; 
        }

        private static void RegisterMapping(string mappingId, Func<string> mappingFunc)
        {
            mapping.Add(mappingId, mappingFunc);
        }

        public static string Replace(string value)
        {
            StringBuilder builder = new StringBuilder(value);

            foreach (string key in mapping.Keys)
            {
                builder = builder.Replace(key, mapping[key]());
            }

            return builder.ToString();
        }
    }
}
