﻿using System;
using System.Linq;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities
{
    internal class PageBreakInfo
    {
        public PageBreakType Type { get; set; }
        public int Index { get; set; }
        public int FromIndex { get; set; }
        public int ToIndex { get; set; }

        public PageBreakInfo(PageBreakType type, int index)
            : this(type, index, 0, GetInfiniteToIndex(type))
        {
        }
        
        public PageBreakInfo(PageBreak pageBreak)
            : this(pageBreak.Type, pageBreak.Index, pageBreak.FromIndex, pageBreak.ToIndex)
        {
        }

        public PageBreakInfo(PageBreakType type, int index, int fromIndex, int toIndex)
        {
            this.Type = type;
            this.Index = index;
            this.FromIndex = fromIndex;
            this.ToIndex = toIndex;
        }

        public static bool AreEqualPageBreaksCollections(IEnumerable<PageBreakInfo> firstCollection, IEnumerable<PageBreakInfo> secondCollection)
        {
            PageBreakInfo[] firstSortedCollection = GetSortedCollection(firstCollection);
            PageBreakInfo[] secondSortedCollection = GetSortedCollection(secondCollection);

            if (firstSortedCollection.Length != secondSortedCollection.Length)
            {
                return false;
            }

            for (int index = 0; index < firstSortedCollection.Length; index++)
            {
                PageBreakInfo firstBreak = firstSortedCollection[index];
                PageBreakInfo secondBreak = secondSortedCollection[index];

                if (!firstBreak.Equals(secondBreak))
                {
                    return false;
                }
            }

            return true;
        }

        public static IEnumerable<PageBreakInfo> GetPageBreakInfos(IEnumerable<PageBreak> pageBreaks)
        {
            foreach (PageBreak pageBreak in pageBreaks)
            {
                yield return new PageBreakInfo(pageBreak);
            }
        }

        private static PageBreakInfo[] GetSortedCollection(IEnumerable<PageBreakInfo> collection)
        {
            return collection.OrderBy(x => x.Index).ThenBy(x => x.FromIndex).ToArray();
        }

        private static int GetInfiniteToIndex(PageBreakType type)
        {
            return type == PageBreakType.Horizontal ? SpreadsheetDefaultValues.ColumnCount - 1 : SpreadsheetDefaultValues.RowCount - 1;
        }

        public override bool Equals(object obj)
        {
            PageBreakInfo other = obj as PageBreakInfo;

            if (other == null)
            {
                return false;
            }

            return this.Type == other.Type &&
                this.Index == other.Index &&
                this.FromIndex == other.FromIndex &&
                this.ToIndex == other.ToIndex;
        }

        public override int GetHashCode()
        {
            return this.Type.GetHashCode() ^
                this.Index.GetHashCode() ^
                this.FromIndex.GetHashCode() ^
                this.ToIndex.GetHashCode();
        }        
    }
}
