﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Layout;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ImportExportTests
{
    internal class ViewStateTestHelper
    {
        private readonly WorksheetViewState actualWorksheetViewState;

        public const string ViewStateTestDocument = "ImportExportViewStateDocument.xlsx";

        public const string SheetSingleSelectionName = "singleSelection";
        public const string SheetMultipleSelectionName = "multipleSelection";
        public const string SheetFreezePanesName = "freezePanesMultipleSelection";
        public const string SheetFreezeTopRowName = "freezeTopRow";
        public const string SheetFreezeFirstColumnName = "freezeFirstColumn";
        public const string SheetFreezePanesMethod = "freezePanesMethod";

        public ViewStateTestHelper(WorksheetViewState actualWorksheetViewState)
        {
            this.actualWorksheetViewState = actualWorksheetViewState;
        }

        public void AssertScaleFactorIsEqualTo(double scaleFactor)
        {
            Size actualSize = this.actualWorksheetViewState.ScaleFactor;
            Size expectedSize = new Size(scaleFactor/100, scaleFactor/100);

            Assert.AreEqual(expectedSize, actualSize);
        }

        public void AssertScaleFactorIsEqualTo(Size size)
        {
            this.AssertScaleFactorIsEqualTo(size.Height * 100);
        }

        public void AssertTopLeftCellIndexIsEqualTo(CellIndex expectedCellIndex)
        {
            CellIndex actualCellIndex = this.actualWorksheetViewState.TopLeftCellIndex;

            Assert.AreEqual(expectedCellIndex, actualCellIndex);
        }

        public void AssertSelectionIsEqualTo(SelectionState expectedSelectionState)
        {
            SelectionState actualSelectionState = this.actualWorksheetViewState.SelectionState;

            this.AssertCellIndicesAreEqual(expectedSelectionState.ActiveCellIndex, actualSelectionState.ActiveCellIndex);
            this.AssertCellRangesSequencesAreEqual(expectedSelectionState, actualSelectionState);

            Assert.AreEqual(expectedSelectionState.Pane, actualSelectionState.Pane);
        }

        public void AssertPaneIsEqualTo(Pane expectedPane)
        {
            Pane actualPane = this.actualWorksheetViewState.Pane;

            if (expectedPane == null)
            {
                Assert.IsNull(actualPane);
            }
            else
            {
                Assert.AreEqual(expectedPane.XSplit, actualPane.XSplit);
                Assert.AreEqual(expectedPane.YSplit, actualPane.YSplit);
                this.AssertCellIndicesAreEqual(expectedPane.TopLeftCellIndex, actualPane.TopLeftCellIndex);

                Assert.AreEqual(expectedPane.State, actualPane.State);
                Assert.AreEqual(expectedPane.ActivePane, actualPane.ActivePane);
            }
        }
  
        private void AssertCellRangesSequencesAreEqual(SelectionState expectedSelectionState, SelectionState actualSelectionState)
        {
            Assert.AreEqual(expectedSelectionState.SelectedRanges.Count(), actualSelectionState.SelectedRanges.Count());

            List<CellRange> expectedCellRanges = expectedSelectionState.SelectedRanges.ToList();
            List<CellRange> actualCellRanges = actualSelectionState.SelectedRanges.ToList();

            for (int i = 0; i < expectedCellRanges.Count; i++)
            {
                Assert.AreEqual(expectedCellRanges[i], actualCellRanges[i]);
            }
        }

        private void AssertCellIndicesAreEqual(CellIndex cellIndex1, CellIndex cellIndex2)
        {
            Assert.AreEqual(cellIndex1, cellIndex2);
        }
    }
}