﻿#if WPF

using System;
using System.IO;
using System.Linq;
using DocumentFormat.OpenXml.Validation;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
using System.Collections.Generic;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.XmlValidation
{
    internal class XmlValidationHelper
    {
        private const string XmlValidationErrorsFilePath = "Utilities/XmlValidation/XmlValidationErrors.txt";

        private static readonly List<string> whitelistedErrors;

        static XmlValidationHelper()
        {
            whitelistedErrors = new List<string>();
            InitializeWhitelistedErrors();
        }

        private static void InitializeWhitelistedErrors()
        {
            Stream stream = TestHelper.GetResourceStream(XmlValidationErrorsFilePath);

            using (StreamReader reader = new StreamReader(stream))
            {
                string line = reader.ReadLine();
                while (line != null)
                {
                    if (!line.StartsWith("/") && !string.IsNullOrWhiteSpace(line))
                    {
                        whitelistedErrors.Add(line);
                    }

                    line = reader.ReadLine();
                }
            }
        }

        public static bool IsErrorWhitelisted(ValidationErrorInfo error)
        {
            return whitelistedErrors.Contains(error.Description);
        }
        
        public static void ExportDocumentToFile(Workbook workbook, string filePath)
        {
            using (Stream stream = new FileStream(filePath, FileMode.OpenOrCreate))
            {
                new XlsxFormatProvider().Export(workbook, stream);
            }
        }
    }
}

#endif