﻿#if WPF

using System;
using System.IO;
using System.Linq;
using System.Text;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Validation;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.Utilities.XmlValidation
{
    internal static class XmlValidator
    {
        private static readonly string fileDirectory = Directory.GetCurrentDirectory();
        private static readonly string testFile = "Test.xlsx";

        public static void ValidateOpenXmlDocument(Workbook workbook)
        {
            string filePath = Path.Combine(fileDirectory, testFile);

            try
            {
                XmlValidationHelper.ExportDocumentToFile(workbook, filePath);

                using (SpreadsheetDocument spreadsheetDocument = SpreadsheetDocument.Open(filePath, false))
                {
                    OpenXmlValidator validator = new OpenXmlValidator();
                    StringBuilder stringBuilder = new StringBuilder();

                    foreach (ValidationErrorInfo error in validator.Validate(spreadsheetDocument))
                    {
                        if (!XmlValidationHelper.IsErrorWhitelisted(error))
                        {
                            stringBuilder.Append(error.ErrorType.ToString())
                                .Append('\t')
                                .Append(error.Node.ToString())
                                .Append('\t')
                                .Append(error.Part.Uri.ToString())
                                .Append('\t')
                                .Append(error.Description)
                                .Append('\n');
                        }
                    }

                    spreadsheetDocument.Close();

                    string errorMessage = stringBuilder.ToString();
                    Assert.IsTrue(string.IsNullOrEmpty(errorMessage), errorMessage);
                }
            }
            finally
            {
                File.Delete(filePath);
            }
        }
    }
}

#endif