﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
	[TestClass]
	public class WorkbookWorksheetCollectionTests
	{
		[TestMethod]
		public void AddTest()
		{
			Workbook workbook = new Workbook();
			Assert.AreEqual(0, workbook.Worksheets.Count);
			Assert.AreEqual(0, workbook.Sheets.Count);

			for (int i = 1; i <= 10; i++)
			{
				Worksheet worksheet = workbook.Worksheets.Add();

				Assert.AreEqual(i, workbook.Worksheets.Count);

				Assert.AreEqual(workbook, worksheet.Workbook);
				Assert.AreEqual(SheetType.Worksheet, worksheet.Type);
				Assert.AreEqual(i, workbook.Sheets.Count);
			}
		}

		[TestMethod]
		public void InsertTest()
		{
			Workbook workbook = new Workbook();
			Assert.AreEqual(0, workbook.Worksheets.Count);
			Assert.AreEqual(0, workbook.Sheets.Count);

			for (int i = 1; i <= 10; i++)
			{
				Worksheet worksheet = workbook.Worksheets.Insert(0);

				Assert.AreEqual("Sheet" + i, worksheet.Name);
				Assert.AreEqual(i, workbook.Worksheets.Count);
				Assert.AreEqual(i, workbook.Sheets.Count);
			}

			for (int i = 1; i <= 10; i++)
			{
				Worksheet worksheet = workbook.Worksheets[10 - i];

				string expectedSheetName = ("Sheet" + i);
				Assert.AreEqual(expectedSheetName, worksheet.Name);
				Assert.AreEqual(worksheet, workbook.Worksheets[expectedSheetName]);
			}
		}

		[TestMethod]
		public void AddAndGetByNameTest()
		{
			Workbook workbook = new Workbook();
			Assert.AreEqual(0, workbook.Worksheets.Count);
			Assert.AreEqual(0, workbook.Sheets.Count);

			for (int i = 1; i <= 10; i++)
			{
				Worksheet worksheet = workbook.Worksheets.Add();

				Assert.AreEqual("Sheet" + i, worksheet.Name);
				Assert.AreEqual(i, workbook.Sheets.Count);
			}

			for (int i = 1; i <= 10; i++)
			{
				Worksheet worksheet = workbook.Worksheets[i - 1];

				string expectedSheetName = ("Sheet" + i);
				Assert.AreEqual(expectedSheetName, worksheet.Name);
				Assert.AreEqual(worksheet, workbook.Worksheets[expectedSheetName]);
			}
		}

		[TestMethod]
		public void Remove()
		{
			Workbook workbook = new Workbook();

			Assert.AreEqual(0, workbook.Worksheets.Count);

			Worksheet worksheet = workbook.Worksheets.Add();
			Assert.AreEqual(1, workbook.Worksheets.Count);

			workbook.Worksheets.Remove(worksheet);
			Assert.AreEqual(0, workbook.Worksheets.Count);
		}


        #region Worksheet name validation

        [TestMethod]
		[ExpectedException(typeof(ArgumentNullException))]
		public void SetSheetNameToNullTest()
		{
			Workbook workbook = new Workbook();
			Worksheet worksheet = workbook.Worksheets.Add();
			worksheet.Name = null;
		}

		[TestMethod]
		[ExpectedException(typeof(LocalizableException))]
		public void ChangeNameToExistingNameTest()
		{
			Workbook workbook = new Workbook();

			Worksheet worksheet = workbook.Worksheets.Add();
			Assert.AreEqual(1, workbook.Worksheets.Count);
			Assert.AreEqual("Sheet1", worksheet.Name);

			Worksheet worksheet1 = workbook.Worksheets.Add();
			Assert.AreEqual("Sheet2", worksheet1.Name);
			worksheet1.Name = "Sheet1";
		}
                
        [TestMethod]
        public void ChangingToTheIgnoreCaseSameNameTests()
        {
            Workbook workbook = new Workbook();

            string firstName = "Иван Иванов";
            string secondName = "иВан иВаноВ";

            Worksheet worksheet = workbook.Worksheets.Add();
            worksheet.Name = firstName;

            //Changing to the same name with different casing
            worksheet.Name = secondName;

            Assert.AreEqual(secondName, worksheet.Name);
            Assert.AreNotEqual(firstName, worksheet.Name);
        }

        [TestMethod]
        [ExpectedException(typeof(LocalizableException))]
        public void ChangeNameToExistingNameIgnoreCaseTest()
        {
            Workbook workbook = new Workbook();

            workbook.Worksheets.Add();
            Worksheet worksheet2 = workbook.Worksheets.Add();

            worksheet2.Name = "sHeEt1";
        }

        [TestMethod]
        public void IndexOfIgnoreCaseTest()
        {
            Workbook workbook = new Workbook();

            workbook.Worksheets.Add();
            workbook.Worksheets.Add().Name = "Pesho";
            workbook.Worksheets.Add();

            int resultIndex = workbook.Worksheets.IndexOf("pEsHo");
            int expectedIndex = 1;

            Assert.AreEqual(expectedIndex, resultIndex);
        }

        [TestMethod]
        public void GetByNameIgnoreCaseTest()
        {
            Workbook workbook = new Workbook();

            string worksheetName = "Pesho";
            string differentCasingName = "pEsHo";

            workbook.Worksheets.Add();
            workbook.Worksheets.Add().Name = worksheetName;
            workbook.Worksheets.Add();

            Worksheet worksheet = workbook.Worksheets.GetByName(differentCasingName);

            Assert.AreEqual(worksheetName, worksheet.Name);
            Assert.AreNotEqual(differentCasingName, worksheet.Name);
        }

        #endregion
    }
}