﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Silverlight.Testing;
using Microsoft.Silverlight.Testing.Client;
using Microsoft.Silverlight.Testing.Controls;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    public partial class App : Application
    {
        private TestPage testPage;

        public App()
        {
            this.Startup += this.Application_Startup;
            this.UnhandledException += this.Application_UnhandledException;

            InitializeComponent();
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            this.testPage = (TestPage)UnitTestSystem.CreateTestPage();
            AttachReRunButton();
            AttachCopyTagsButton();
            this.RootVisual = testPage;
        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                Deployment.Current.Dispatcher.BeginInvoke(delegate { ReportErrorToDOM(e); });
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }


        private void AttachReRunButton()
        {
            Grid pageContent = this.testPage.Content as Grid;
            if (pageContent != null)
            {
                Button reRunButton = new Button()
                {
                    Width = 160,
                    Height = 30,
                    HorizontalAlignment = HorizontalAlignment.Left,
                    Content = "Run selected tests",
                    Margin = new Thickness(320, 0, 0, 0)
                };

                reRunButton.Click += ReRunButton_Click;
                pageContent.Children.Add(reRunButton);
            }
        }

        private void AttachCopyTagsButton()
        {
            Grid pageContent = this.testPage.Content as Grid;
            if (pageContent != null)
            {
                Button reRunButton = new Button()
                {
                    Width = 180,
                    Height = 30,
                    HorizontalAlignment = HorizontalAlignment.Left,
                    Content = "Copy selected tests tags ",
                    Margin = new Thickness(500, 0, 0, 0)
                };

                reRunButton.Click += CopyTagsButton_Click;
                pageContent.Children.Add(reRunButton);
            }
        }

        private void ReRunButton_Click(object sender, RoutedEventArgs e)
        {
            string tag = GetTagExpression();

            //clear current tests
            var data = this.testPage.TreeView.DataContext as TestRunData;
            data.TestAssemblies[0].TestClasses.Clear();
            data.TotalScenarios = 0;
            data.RunScenarios = 0;
            data.FailedScenarios = 0;
            data.RunScenarios = 0;

            //set the tag expression and fire new test run
            this.testPage.UnitTestHarness.Settings.TagExpression = tag;
            this.testPage.UnitTestHarness.Run();
        }

        private void CopyTagsButton_Click(object sender, RoutedEventArgs e)
        {
            string tag = GetTagExpression();
            Clipboard.SetText(tag);
        }

        private string GetTagExpression()
        {
            //create tag expression for the current selected items
            StringBuilder sb = new StringBuilder();
            foreach (KeyValuePair<object, Microsoft.Silverlight.Testing.Controls.TreeViewItem> item in this.testPage.TreeView.GetCheckedItemsAndContainers())
            {
                TestMethodData tmd = item.Key as TestMethodData;
                if (tmd != null)
                {
                    sb.Append(tmd.Name);
                    sb.Append("+");
                }
            }
            string tag = sb.ToString().TrimEnd('+');
            return tag;
        }
    }
}