﻿using System;
using System.Globalization;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    public abstract class MultiCulturalTestClassBase
    {
        private readonly string[] defaultCultures = new string[] { "en-US", "ro-RO", "bg-BG", "ar-DZ", "ar-QA", "da-DK", "de-DE", "es-ES", "fr-BE", "fr-FR", "it-IT", "ko-KR", "nl-BE", "pa-IN", "ru-RU", "sv-FI" };
        //"zh-CHS" not supported in Silverlight 

        public virtual string[] DefaultCultures
        {
            get
            {
                return defaultCultures;
            }
        }

        public void RunTestInMultipleCultures(Action action, string[] cultures = null)
        {
            if(cultures == null)
            {
                cultures = DefaultCultures;
            }

            SpreadsheetCultureHelper previousCulture = FormatHelper.DefaultSpreadsheetCulture;

            try
            {
                foreach (string culture in cultures)
                {
                    FormatHelper.DefaultSpreadsheetCulture = new SpreadsheetCultureHelper(new CultureInfo(culture));
                    action();
                }
            }
            finally
            {
                FormatHelper.DefaultSpreadsheetCulture = previousCulture;
            }
        }
    }
}
