﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    [TestClass]
    public class CellRangeTests
    {
        [TestMethod]
        public void GetCellRangesDifferenceNotIntersectingRangesTest()
        {
            AssertGetDifferenceMethod(new CellRange(0, 0, 10, 10), new CellRange(0, 11, 4, 13), new CellRange[] { new CellRange(0, 0, 10, 10) });            
        }

        [TestMethod]
        public void GetCellRangesDifferenceEqualRangesTest()
        {
            AssertGetDifferenceMethod(new CellRange(0, 0, 10, 10), new CellRange(0, 0, 10, 10), new CellRange[] { });
        }

        [TestMethod]
        public void GetCellRangesDifferenceFirstInSecondTest()
        {
            AssertGetDifferenceMethod(new CellRange(3, 2, 7, 6), new CellRange(0, 0, 10, 10), new CellRange[] { });
        }

        [TestMethod]
        public void GetCellRangesDifferenceFirstSurroundingSecondTest()
        {
            AssertGetDifferenceMethod(new CellRange(0, 0, 10, 10), new CellRange(3, 2, 7, 6), new CellRange[] 
            {
                new CellRange(0, 0, 7, 1),
                new CellRange(0, 2, 2, 10),
                new CellRange(3, 7, 10, 10),
                new CellRange(8, 0, 10, 6),
            });            
        }

        [TestMethod]
        public void GetCellRangesDifferenceSecondInCornerTest()
        {
            AssertGetDifferenceMethod(new CellRange(0, 0, 10, 10), new CellRange(0, 0, 3, 4), new CellRange[] 
            {
                new CellRange(0, 5, 10, 10),
                new CellRange(4, 0, 10, 4),
            });
        }

        [TestMethod]
        public void GetCellRangesDifferenceComplexIntersectionTest()
        {
            AssertGetDifferenceMethod(new CellRange(3, 2, 10, 11), new CellRange(5, 6, 13, 15), new CellRange[] 
            {
                new CellRange(3, 2, 10, 5),
                new CellRange(3, 6, 4, 11),
            });
        }

        private static void AssertGetDifferenceMethod(CellRange first, CellRange second, IEnumerable<CellRange> expectedResult)
        {
            CellRange[] expectedRanges = GetSortedRanges(expectedResult);
            CellRange[] actualRanges = GetSortedRanges(CellRange.GetDifference(first, second));

            Assert.AreEqual(expectedRanges.Length, actualRanges.Length);

            for (int i = 0; i < expectedRanges.Length; i++)
            {
                CellRange expected = expectedRanges[i];
                CellRange actual = actualRanges[i];

                Assert.AreEqual(expected, actual);
            }
        }

        private static CellRange[] GetSortedRanges(IEnumerable<CellRange> ranges)
        {
            return ranges.OrderBy(x => x.FromIndex.RowIndex)
                .ThenBy(x => x.FromIndex.ColumnIndex)
                .ThenBy(x => x.RowCount)
                .ThenBy(x => x.ColumnCount)
                .ToArray();
        }
    }
}
