﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellReferenceRanges
{
    [TestClass]
    public class CellAreaReferenceCreationTests
    {
        // The start index of the relative CellReferences is 0, e.g. A1
        private static readonly RowColumnReference relativeOffset1 = RowColumnReference.CreateRelativeRowColumnReference(0, 1);
        private static readonly RowColumnReference relativeOffset3 = RowColumnReference.CreateRelativeRowColumnReference(0, 3);
        private static readonly CellReference B2 = new CellReference(relativeOffset1, relativeOffset1);
        private static readonly CellReference B4 = new CellReference(relativeOffset3, relativeOffset1);
        private static readonly CellReference D2 = new CellReference(relativeOffset1, relativeOffset3);
        private static readonly CellReference D4 = new CellReference(relativeOffset3, relativeOffset3);

        // The small "s" denotes absolute reference (the symbol is used instead of $)
        private static readonly RowColumnReference absoluteIndex0 = RowColumnReference.CreateAbsoluteRowColumnReference(0);
        private static readonly RowColumnReference absoluteIndex2 = RowColumnReference.CreateAbsoluteRowColumnReference(2);
        private static readonly RowColumnReference absoluteIndex4 = RowColumnReference.CreateAbsoluteRowColumnReference(4);
        private static readonly CellReference sAs1 = new CellReference(absoluteIndex0, absoluteIndex0);
        private static readonly CellReference sAs3 = new CellReference(absoluteIndex2, absoluteIndex0);
        private static readonly CellReference sEs1 = new CellReference(absoluteIndex0, absoluteIndex4);
        private static readonly CellReference sEs3 = new CellReference(absoluteIndex2, absoluteIndex4);

        // The start index of the relative CellReferences is 5, e.g. F6
        private static readonly RowColumnReference relativeOffsetMinus1 = RowColumnReference.CreateRelativeRowColumnReference(5, -1);
        private static readonly RowColumnReference relativeOffsetMinus3 = RowColumnReference.CreateRelativeRowColumnReference(5, -3);
        private static readonly RowColumnReference relativeOffsetMinus5 = RowColumnReference.CreateRelativeRowColumnReference(5, -5);
        private static readonly CellReference sA1 = new CellReference(relativeOffsetMinus5, absoluteIndex0);
        private static readonly CellReference sA5 = new CellReference(relativeOffsetMinus1, absoluteIndex0);
        private static readonly CellReference As5 = new CellReference(absoluteIndex4, relativeOffsetMinus5);
        private static readonly CellReference As1 = new CellReference(absoluteIndex0, relativeOffsetMinus5);
        private static readonly CellReference sC1 = new CellReference(relativeOffsetMinus5, absoluteIndex2);
        private static readonly CellReference sC5 = new CellReference(relativeOffsetMinus1, absoluteIndex2);
        private static readonly CellReference Cs5 = new CellReference(absoluteIndex4, relativeOffsetMinus3);
        private static readonly CellReference Cs1 = new CellReference(absoluteIndex0, relativeOffsetMinus3);
      
        [TestMethod]
        public void CreateB2ToD4RelativeCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(B2, D4);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, false, 3, 3);
        }

        [TestMethod]
        public void CreateD2ToB4RelativeCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(D2, B4);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, false, 3, 3);
        }

        [TestMethod]
        public void CreateD4ToB2RelativeCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(D4, B2);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, false, 3, 3);
        }

        [TestMethod]
        public void CreateB4ToD2RelativeCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(D4, B2);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, false, 3, 3);
        }

        [TestMethod]
        public void CreateA1ToE3AbsoluteCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sAs1, sEs3);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 5, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, true, 4, 2);
        }

        [TestMethod]
        public void CreateE1ToA3AbsoluteCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sEs1, sAs3);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 5, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, true, 4, 2);
        }

        [TestMethod]
        public void CreateE3ToA1AbsoluteCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sEs3, sAs1);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 5, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, true, 4, 2);
        }

        [TestMethod]
        public void CreateA3ToE1AbsoluteCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sEs3, sAs1);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 5, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, true, 4, 2);
        }

        [TestMethod]
        public void CreatesA1ToCs5MixedCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sA1, Cs5);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, false, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, true, 2, 4);
        }

        [TestMethod]
        public void CreatesC1ToAs5MixedCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sC1, As5);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, false, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, true, 2, 4);
        }

        [TestMethod]
        public void CreatesC5ToAs1MixedCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sC5, As1);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, false, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, true, false, 2, 4);
        }

        [TestMethod]
        public void CreatesA5ToCs1MixedCellReferenceAreaTest()
        {
            CellAreaReference cellAreaReference = new CellAreaReference(sA5, Cs1);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(cellAreaReference, 3, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.FromCellReference, true, true, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(cellAreaReference.ToCellReference, false, false, 2, 4);
        }
    }
}
