﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellReferenceRanges
{
    [TestClass]
    public class CellAreaReferenceRangeAndIntersectionTests
    {
        private static readonly CellIndex startIndex = new CellIndex(0, 0);
        private static readonly CellReference B2 = CellReference.CreateFromCellName("B2", startIndex);
        private static readonly CellReference B7 = CellReference.CreateFromCellName("B7", startIndex);
        private static readonly CellReference C4 = CellReference.CreateFromCellName("C4", startIndex);
        private static readonly CellReference D5 = CellReference.CreateFromCellName("D5", startIndex);
        private static readonly CellReference E4 = CellReference.CreateFromCellName("E4", startIndex);
        private static readonly CellReference E7 = CellReference.CreateFromCellName("E7", startIndex);
        private static readonly CellReference F3 = CellReference.CreateFromCellName("F3", startIndex);
        private static readonly RowColumnReference invalidRowColumnReference = RowColumnReference.CreateRelativeRowColumnReference(0, -1);
        private static readonly CellReference invalidReference = new CellReference(invalidRowColumnReference, invalidRowColumnReference);

        [TestMethod]
        public void CreateValidRangeFromB2ToF3CellReferences()
        {
            CellAreaReference first = new CellAreaReference(B2);
            CellAreaReference second = new CellAreaReference(F3);

            CellAreaReference result = CellAreaReference.CreateRange(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 5, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 5, 2);
        }

        [TestMethod]
        public void CreateValidRangeFromB2C4ToD5E7CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, C4);
            CellAreaReference second = new CellAreaReference(D5, E7);

            CellAreaReference result = CellAreaReference.CreateRange(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 4, 6);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 4, 6);
        }

        [TestMethod]
        public void CreateValidRangeFromB7C4ToD5F3CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B7, C4);
            CellAreaReference second = new CellAreaReference(D5, F3);

            CellAreaReference result = CellAreaReference.CreateRange(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 5, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 5, 6);
        }

        [TestMethod]
        public void CreateValidRangeFromB2E7ToC4D5CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, E7);
            CellAreaReference second = new CellAreaReference(C4, D5);

            CellAreaReference result = CellAreaReference.CreateRange(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 4, 6);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 4, 6);
        }

        [TestMethod]
        public void CreateValidRangeFromB2C4ToB2C4CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, C4);

            CellAreaReference result = CellAreaReference.CreateRange(first, first);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 2, 3);
        }

        [TestMethod]
        public void CreateInvalidRangeFromREFToREFCellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(invalidReference, invalidReference);

            CellAreaReference result = CellAreaReference.CreateRange(first, first);

            Assert.IsFalse(result.IsValid);
        }

        [TestMethod]
        public void CreateInvalidRangeFromB2ToREFCellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, F3);
            CellAreaReference second = new CellAreaReference(B2, invalidReference);

            CellAreaReference result = CellAreaReference.CreateRange(first, second);

            Assert.IsFalse(result.IsValid);
        }

        [TestMethod]
        public void CreateValidIntersectionFromB2D5ToC4E7CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, D5);
            CellAreaReference second = new CellAreaReference(C4, E7);

            CellAreaReference result = CellAreaReference.CreateIntersection(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 2, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 3, 4);
        }

        [TestMethod]
        public void CreateValidIntersectionFromB7E4ToD5F3CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B7, E4);
            CellAreaReference second = new CellAreaReference(D5, F3);

            CellAreaReference result = CellAreaReference.CreateIntersection(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 4, 4);
        }

        [TestMethod]
        public void CreateValidIntersectionFromB2E7ToC4D5CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, E7);
            CellAreaReference second = new CellAreaReference(C4, D5);

            CellAreaReference result = CellAreaReference.CreateIntersection(first, second);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 2, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 3, 4);
        }

        [TestMethod]
        public void CreateValidIntersectionFromB2C4ToB2C4CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, C4);

            CellAreaReference result = CellAreaReference.CreateIntersection(first, first);

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 2, 3);
        }

        [TestMethod]
        public void CreateInvalidIntersectionFromB2C4ToD5F3CellAreaReferences()
        {
            CellAreaReference first = new CellAreaReference(B2, C4);
            CellAreaReference second = new CellAreaReference(D5, F3);

            CellAreaReference result = CellAreaReference.CreateIntersection(first, second);

            Assert.IsNull(result);
        }
    }
}
