﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellReferenceRanges
{
    [TestClass]
    public class CellReferenceRangeExpressionRangeAndInsertionTests
    {
        private static Workbook workbook;

        private static Workbook Workbook
        {
            get
            {
                if(workbook == null)
                {
                    workbook = new Workbook();
                    workbook.Worksheets.Add();
                }

                return workbook;
            }
        }

        private CellReferenceRangeExpression GetCellReferenceRangeExpression(CellIndex startIndex, params string[] ranges)
        {
            List<CellReferenceRange> cellReferenceRanges = new List<CellReferenceRange>();
            
            for (int i = 0; i < ranges.Length; i++)
            {
                WorksheetReference reference = new WorksheetReference(Workbook, Workbook.ActiveWorksheet.Name, true);
                cellReferenceRanges.Add(new CellReferenceRange(reference, GetCellAreaReference(startIndex, ranges[i])));
            }

            return new CellReferenceRangeExpression(cellReferenceRanges);
        }

        private CellAreaReference GetCellAreaReference(CellIndex startIndex, string range)
        {
            string[] cellNames = range.Split(':');
            
            if(cellNames.Length == 2)
            {
                CellReference left = CellReference.CreateFromCellName(cellNames[0], startIndex);
                CellReference right = CellReference.CreateFromCellName(cellNames[1], startIndex);

                return new CellAreaReference(left, right);
            }
            
            CellReference cellReference = CellReference.CreateFromCellName(range, startIndex);
            return new CellAreaReference(cellReference);
        }

        [TestMethod]
        public void CreateRangeFromSingleCellReferenceExpressions()
        {
            CellIndex cellIndex = new CellIndex(0, 0);
            CellReferenceRangeExpression leftExpression = GetCellReferenceRangeExpression(cellIndex, "A1");
            CellReferenceRangeExpression rightExpression = GetCellReferenceRangeExpression(cellIndex, "C3");

            CellReferenceRangeExpression resultExpression;
            CellReferenceRangeExpression.TryCreateRange(leftExpression, rightExpression, out resultExpression);

            CellAreaReference result = resultExpression.CellReferenceRanges[0].CellAreaReference;

            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 3, 3);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 2, 2);
        }

        [TestMethod]
        public void CreateRangeFromAreaCellReferenceExpressions()
        {
            CellIndex cellIndex = new CellIndex(0, 0);
            CellReferenceRangeExpression leftExpression = GetCellReferenceRangeExpression(cellIndex, "A1:A2");
            CellReferenceRangeExpression rightExpression = GetCellReferenceRangeExpression(cellIndex, "C3:D4");

            CellReferenceRangeExpression resultExpression;
            CellReferenceRangeExpression.TryCreateRange(leftExpression, rightExpression, out resultExpression);

            CellAreaReference result = resultExpression.CellReferenceRanges[0].CellAreaReference;

            Assert.AreEqual(1, resultExpression.CellReferenceRanges.Count);
            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 4, 4);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 3, 3);
        }

        [TestMethod]
        public void CreateRangeFromMultipleAreasCellReferenceExpressions()
        {
            CellIndex cellIndex = new CellIndex(0, 0);
            CellReferenceRangeExpression leftExpression = GetCellReferenceRangeExpression(cellIndex, "A1:A2", "B2:B3");
            CellReferenceRangeExpression rightExpression = GetCellReferenceRangeExpression(cellIndex, "C3:C4", "D4:D5");

            CellReferenceRangeExpression resultExpression;
            CellReferenceRangeExpression.TryCreateRange(leftExpression, rightExpression, out resultExpression);

            CellAreaReference result = resultExpression.CellReferenceRanges[0].CellAreaReference;

            Assert.AreEqual(1, resultExpression.CellReferenceRanges.Count);
            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 4, 5);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 0, 0);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 3, 4);
        }

        [TestMethod]
        public void CreateIntersectionFromSingleAreasCellReferenceExpressions()
        {
            CellIndex cellIndex = new CellIndex(0, 0);
            CellReferenceRangeExpression leftExpression = GetCellReferenceRangeExpression(cellIndex, "A1:C3");
            CellReferenceRangeExpression rightExpression = GetCellReferenceRangeExpression(cellIndex, "B2:D4");

            CellReferenceRangeExpression resultExpression;
            CellReferenceRangeExpression.TryCreateIntersection(leftExpression, rightExpression, out resultExpression);

            CellAreaReference result = resultExpression.CellReferenceRanges[0].CellAreaReference;

            Assert.AreEqual(1, resultExpression.CellReferenceRanges.Count);
            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(result, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(result.ToCellReference, false, false, 2, 2);
        }

        [TestMethod]
        public void CreateIntersectionFromMultipleAreasCellReferenceExpressions()
        {
            CellIndex cellIndex = new CellIndex(0, 0);

            CellReferenceRangeExpression leftExpression = GetCellReferenceRangeExpression(cellIndex, "A1:C3", "F2:H4");
            CellReferenceRangeExpression rightExpression = GetCellReferenceRangeExpression(cellIndex, "B2:D4", "G3:I5");

            CellReferenceRangeExpression resultExpression;
            CellReferenceRangeExpression.TryCreateIntersection(leftExpression, rightExpression, out resultExpression);

            Assert.AreEqual(2, resultExpression.CellReferenceRanges.Count);

            CellAreaReference firstResult = resultExpression.CellReferenceRanges[0].CellAreaReference;
            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(firstResult, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(firstResult.FromCellReference, false, false, 1, 1);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(firstResult.ToCellReference, false, false, 2, 2);

            CellAreaReference secondResult = resultExpression.CellReferenceRanges[1].CellAreaReference;
            CellReferenceTestHelper.AssertCellAreaReferenceValidityAndSize(secondResult, 2, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(secondResult.FromCellReference, false, false, 6, 2);
            CellReferenceTestHelper.AssertCellReferenceCorrectness(secondResult.ToCellReference, false, false, 7, 3);
        }
    }
}
