﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class CellReferenceRangeTests
    {
        #region Fields

        private Worksheet worksheet;

        private static readonly RowColumnReference rowColumnReference0 = RowColumnReference.CreateAbsoluteRowColumnReference(0);
        private static readonly RowColumnReference rowColumnReference1 = RowColumnReference.CreateAbsoluteRowColumnReference(1);
        private static readonly RowColumnReference rowColumnReference2 = RowColumnReference.CreateAbsoluteRowColumnReference(2);

        #endregion


        #region TestInitialize

        [TestInitialize]
        public void TestInitialize()
        {
            this.worksheet = TestHelper.CreateWorksheet();
        }

        [TestCleanup]
        public void TestCleanup()
        {
            this.worksheet = null;
        }

        #endregion


        #region Translation Tests of Absolute CellReferenceRanges

        [TestMethod]
        public void TranslateAbsoluteCellReference1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A$1", 0, 0, 0, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A$1", 0, 0, 0, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A$1", 0, 0, 0, 0);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReference2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J$10", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$J$10", 9, 9, 9, 9);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$J$10", 9, 9, 9, 9);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$J$10", 9, 9, 9, 9);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReference3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B$1", this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount / 2, SpreadsheetDefaultValues.ColumnCount / 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, 0));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B$1", 0, 1, 0, 1);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReference4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A$1:$A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$A$1:$A$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$A$1", 0, 0, 0, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$A$1:$A$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$A$1", 0, 0, 0, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "$A$1:$A$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$A$1", 0, 0, 0, 0);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReference5()
        {
            RadExpression expression;
            RadExpression translatedExpression;
            string toString = "$J$10:$J$10";

            RadExpression.TryParse("=$J$10:$J$10", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 9, 9, 9, 9);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 9, 9, 9, 9);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 9, 9, 9, 9);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReference6()
        {
            RadExpression expression;
            RadExpression translatedExpression;
            string toString = "$B$1:$B$1";

            RadExpression.TryParse("=$B$1:$B$1", this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount / 2, SpreadsheetDefaultValues.ColumnCount / 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, 0));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, SpreadsheetDefaultValues.ColumnCount - 1));
            Assert.AreEqual(translatedExpression.ToString(), toString);
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), toString, 0, 1, 0, 1);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReferenceRange1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B$1:$A$2", this.worksheet, new CellIndex(5, 5), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$B$2", 0, 0, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$B$2", 0, 0, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A$1:$B$2", 0, 0, 1, 1);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReferenceRange2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J$10:$K$9", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J$9:$K$10", 8, 9, 9, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J$9:$K$10", 8, 9, 9, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J$9:$K$10", 8, 9, 9, 10);
        }

        [TestMethod]
        public void TranslateAbsoluteCellReferenceRange3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B$1:$C$3", this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount / 2, SpreadsheetDefaultValues.ColumnCount / 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B$1:$C$3", 0, 1, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B$1:$C$3", 0, 1, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, 0));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B$1:$C$3", 0, 1, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B$1:$C$3", 0, 1, 2, 2);
        }

        #endregion


        #region Translation Tests of Relative CellReferenceRanges

        [TestMethod]
        public void TranslateRelativeCellReference1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = -1
            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B2", 1, 1, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B3", 2, 1, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C2", 1, 2, 1, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 0, columnOffset = -1
            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(0, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B3", 2, 1, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B4", 3, 1, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C3", 2, 2, 2, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(0, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(1, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = 0
            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(1, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C2", 1, 2, 1, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C3", 2, 2, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D2", 1, 3, 1, 3);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(1, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 0, columnOffset = 1
            RadExpression.TryParse("=B1", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D3", 2, 3, 2, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D4", 3, 3, 3, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "E3", 2, 4, 2, 4);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B1", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = 0
            RadExpression.TryParse("=A2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C4", 3, 2, 3, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C5", 4, 2, 4, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D4", 3, 3, 3, 3);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = 1
            RadExpression.TryParse("=B1", this.worksheet, new CellIndex(1, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D2", 1, 3, 1, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D3", 2, 3, 2, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "E2", 1, 4, 1, 4);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B1", this.worksheet, new CellIndex(1, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = -1
            RadExpression.TryParse("=A2", this.worksheet, new CellIndex(0, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B4", 3, 1, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B5", 4, 1, 4, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C4", 3, 2, 3, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A2", this.worksheet, new CellIndex(0, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = 1
            RadExpression.TryParse("=B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D4", 3, 3, 3, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D5", 4, 3, 4, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "E4", 3, 4, 3, 4);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference9()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 0, columnOffset = 0
            RadExpression.TryParse("=A1", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C3", 2, 2, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "XFD1048576", SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1, SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1);
        }

        [TestMethod]
        public void TranslateRelativeCellReference10()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = -1
            RadExpression.TryParse("=A1:A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "B2:B2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B2:B2", 1, 1, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "B3:B3");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B3:B3", 2, 1, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "C2:C2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C2:C2", 1, 2, 1, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError9()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReference11()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = 1
            RadExpression.TryParse("=B2:B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "D4:D4");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D4:D4", 3, 3, 3, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "D5:D5");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D5:D5", 4, 3, 4, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "E4:E4");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "E4:E4", 3, 4, 3, 4);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceError10()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B2:B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = -1
            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B2:C3", 1, 1, 2, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(4, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B3:C4", 2, 1, 3, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C2:D3", 1, 2, 2, 3);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 0, columnOffset = -1
            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(0, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A3:B4", 2, 0, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A4:B5", 3, 0, 4, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B3:C4", 2, 1, 3, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(0, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(1, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = 0
            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(1, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A2:B3", 1, 0, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A3:B4", 2, 0, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B2:C3", 1, 1, 2, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:B2", this.worksheet, new CellIndex(1, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 0, columnOffset = 1
            RadExpression.TryParse("=B1:C2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D3:E4", 2, 3, 3, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D4:E5", 3, 3, 4, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "E3:F4", 2, 4, 3, 5);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B1:C2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = 0
            RadExpression.TryParse("=A2:B3", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C4:D5", 3, 2, 4, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C5:D6", 4, 2, 5, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D4:E5", 3, 3, 4, 4);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A2:B3", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = -1, columnOffset = 1
            RadExpression.TryParse("=B1:C2", this.worksheet, new CellIndex(2, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D1:E2", 0, 3, 1, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D2:E3", 1, 3, 2, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "E1:F2", 0, 4, 1, 5);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B1:C2", this.worksheet, new CellIndex(2, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            //rowOffset = 1, columnOffset = -1
            RadExpression.TryParse("=A2:B3", this.worksheet, new CellIndex(0, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A4:B5", 3, 0, 4, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A5:B6", 4, 0, 5, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B4:C5", 3, 1, 4, 2);
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRangeError7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A2:B3", this.worksheet, new CellIndex(0, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateRelativeCellReferenceRange8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B2:C3", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D4:E5", 3, 3, 4, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D5:E6", 4, 3, 5, 4);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "E4:F5", 3, 4, 4, 5);
        }

        #endregion


        #region Translation Tests of Half-Absolute Half-Relative CellReferenceRanges

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "C$1", 0, 2, 0, 2);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B$2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D$2", 1, 3, 1, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "D$2", 1, 3, 1, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "E$2", 1, 4, 1, 4);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B$2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A2", 1, 0, 1, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A3", 2, 0, 2, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$A2", 1, 0, 1, 0);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B4", 3, 1, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B5", 4, 1, 4, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression, "$B4", 3, 1, 3, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "B$1:B$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "B$1:B$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:B$1", 0, 1, 0, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "C$1:C$1");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C$1:C$1", 0, 2, 0, 2);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:A$1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B$2:B$2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "D$2:D$2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D$2:D$2", 1, 3, 1, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "D$2:D$2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D$2:D$2", 1, 3, 1, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "E$2:E$2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "E$2:E$2", 1, 4, 1, 4);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=B$2:B$2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:$A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$A2:$A2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:$A2", 1, 0, 1, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$A3:$A3");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A3:$A3", 2, 0, 2, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "$A2:$A2");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:$A2", 1, 0, 1, 0);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:$A1", this.worksheet, new CellIndex(1, 1), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReference8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B2:$B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$B4:$B4");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B4:$B4", 3, 1, 3, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            Assert.AreEqual(translatedExpression.ToString(), "$B5:$B5");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B5:$B5", 4, 1, 4, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            Assert.AreEqual(translatedExpression.ToString(), "$B4:$B4");
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$B4:$B4", 3, 1, 3, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceError8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$B2:$B2", this.worksheet, new CellIndex(0, 0), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:B$2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:C$2", 0, 1, 1, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A$1:B$2", 0, 0, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:C$2", 0, 1, 1, 2);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError1()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:B$2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=J$10:K$11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(15, 15));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "N$10:O$11", 9, 13, 10, 14);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 12));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "K$10:L$11", 9, 10, 10, 11);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(5, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C$10:D$11", 9, 2, 10, 3);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(19, 5));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D$10:E$11", 9, 3, 10, 4);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError2()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=J$10:K$11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:$B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:$B3", 1, 0, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:$B3", 1, 0, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A1:$B2", 0, 0, 1, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError3()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:$B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J10:$K11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(15, 15));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J14:$K15", 13, 9, 14, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 12));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J2:$K3", 1, 9, 2, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(5, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J4:$K5", 3, 9, 4, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(19, 5));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J18:$K19", 17, 9, 18, 10);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError4()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J10:$K11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:B$2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:C$2", 1, 0, 1, 2);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:B$2", 1, 0, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A1:C$2", 0, 0, 1, 2);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError5()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$A1:B$2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J10:K$11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(15, 15));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J$11:O14", 10, 9, 13, 14);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 12));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$J2:L$11", 1, 9, 10, 11);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(5, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D4:$J$11", 3, 3, 10, 9);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(19, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "D$11:$J18", 10, 3, 17, 9);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError6()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=$J10:K$11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:$B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:$B3", 0, 1, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A$1:$B3", 0, 0, 2, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:$B2", 0, 1, 1, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError7()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:$B2", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=J$10:$K11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(15, 15));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$K$10:N15", 9, 10, 14, 13);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 12));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "K3:$K$10", 2, 10, 9, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(5, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B5:$K$10", 4, 1, 9, 10);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(19, 4));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "C$10:$K19", 9, 2, 18, 10);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError8()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=J$10:$K11", this.worksheet, new CellIndex(11, 11), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange9()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:A1", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:B2", 0, 1, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A$1:A2", 0, 0, 1, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "B$1:B1", 0, 1, 0, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError9()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A$1:A1", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRange10()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:$A1", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A2:B2", 1, 0, 1, 1);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(3, 2));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "A2:$A2", 1, 0, 1, 0);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(2, 3));
            CellReferenceTestHelper.AssertTranslation(translatedExpression.GetValueAsConstantOrCellReference(), "$A1:B1", 0, 0, 0, 1);
        }

        [TestMethod]
        public void TranslateHalfAbsoluteHalfRelativeCellReferenceRangeError10()
        {
            RadExpression expression;
            RadExpression translatedExpression;

            RadExpression.TryParse("=A1:$A1", this.worksheet, new CellIndex(2, 2), out expression);

            translatedExpression = expression.CloneAndTranslate(this.worksheet, new CellIndex(0, 0));
            CellReferenceTestHelper.AssertToString(translatedExpression, "#REF!");
        }

        #endregion


        #region CellAreaReference Creation Tests

        [TestMethod]
        public void CreateIntersectionA1andA1()
        {
            CellAreaReference A1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0));

            CellReferenceTestHelper.CreateIntersectionAndAssertResult(A1, A1, A1);
        }

        [TestMethod]
        public void CreateIntersectionA1andA1B1()
        {
            CellAreaReference A1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0));

            CellAreaReference A1B1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0),
                new CellReference(rowColumnReference0, rowColumnReference1));

            CellReferenceTestHelper.CreateIntersectionAndAssertResult(A1, A1, A1B1);
        }

        [TestMethod]
        public void CreateIntersectionB2C3andA1B2()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                new CellReference(rowColumnReference1, rowColumnReference1));

            CellAreaReference B2C3 = new CellAreaReference(
                new CellReference(rowColumnReference1, rowColumnReference1),
                new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A1B2 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0),
                new CellReference(rowColumnReference1, rowColumnReference1));

            CellReferenceTestHelper.CreateIntersectionAndAssertResult(expectedRange, B2C3, A1B2);
        }

        [TestMethod]
        public void CreateIntersectionC3B2andB2A1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                new CellReference(rowColumnReference1, rowColumnReference1));

            CellAreaReference C3B2 = new CellAreaReference(
                new CellReference(rowColumnReference2, rowColumnReference2), 
                new CellReference(rowColumnReference1, rowColumnReference1));

            CellAreaReference B2A1 = new CellAreaReference(
                new CellReference(rowColumnReference1, rowColumnReference1),
                new CellReference(rowColumnReference0, rowColumnReference0));

            CellReferenceTestHelper.CreateIntersectionAndAssertResult(expectedRange, C3B2, B2A1);
        }

        [TestMethod]
        public void CreateIntersectionA1andB2()
        {
            CellAreaReference A1 = new CellAreaReference(
               new CellReference(rowColumnReference0, rowColumnReference0));

            CellAreaReference B2 = new CellAreaReference(
                new CellReference(rowColumnReference1, rowColumnReference1));

            CellReferenceTestHelper.CreateIntersectionAndAssertError(A1, B2);
        }

        [TestMethod]
        public void CreateIntersectionA1B1andB2C3()
        {
            CellAreaReference A1B1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0),
                new CellReference(rowColumnReference0, rowColumnReference1));

            CellAreaReference B2C3 = new CellAreaReference(
                   new CellReference(rowColumnReference1, rowColumnReference1),
                   new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateIntersectionAndAssertError(A1B1, B2C3);
            CellReferenceTestHelper.CreateIntersectionAndAssertError(B2C3, A1B1);
        }

        [TestMethod]
        public void CreateIntersectionA1B1andB1C1()
        {
            CellAreaReference expectedRange = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));

            CellAreaReference A1B1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference0),
                new CellReference(rowColumnReference0, rowColumnReference1));

            CellAreaReference B1C1 = new CellAreaReference(
                new CellReference(rowColumnReference0, rowColumnReference1),
                new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateIntersectionAndAssertResult(expectedRange, A1B1, B1C1);
        }

        [TestMethod]
        public void CreateRangeFromA1ToC3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference0));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A1, C3);
        }

        [TestMethod]
        public void CreateRangeFromA3ToC1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A3, C1);
        }

        [TestMethod]
        public void CreateRangeA3C1AndB2()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B2 = new CellAreaReference(new CellReference(rowColumnReference1, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A3, C1, B2);
        }

        [TestMethod]
        public void CreateRangeA3B2AndC1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B2 = new CellAreaReference(new CellReference(rowColumnReference1, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A3, B2, C1);
        }

        [TestMethod]
        public void CreateRangeB2A1AndC1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B2 = new CellAreaReference(new CellReference(rowColumnReference1, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, B2, A3, C1);
        }

        [TestMethod]
        public void CreateRangeC1B2AndA3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B2 = new CellAreaReference(new CellReference(rowColumnReference1, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, C1, B2, A3);
        }

        [TestMethod]
        public void CreateRangeC1A3AndB2()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B2 = new CellAreaReference(new CellReference(rowColumnReference1, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, C1, A3, B2);
        }

        [TestMethod]
        public void CreateRangeC3A3AndB1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, C3, A3, B1);
        }

        [TestMethod]
        public void CreateRangeC3A1AndB1()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, C3, A1, B1);
        }

        [TestMethod]
        public void CreateRangeA1B1AndC3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A1, B1, C3);
        }

        [TestMethod]
        public void CreateRangeA3B1AndC3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, A3, B1, C3);
        }

        [TestMethod]
        public void CreateRangeB1C3AndA3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference2));

            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, B1, C3, A3);
        }

        [TestMethod]
        public void CreateRangeB1C1AndA3()
        {
            CellAreaReference expectedRange = new CellAreaReference(
                           new CellReference(rowColumnReference0, rowColumnReference0),
                           new CellReference(rowColumnReference2, rowColumnReference2));

            CellAreaReference A3 = new CellAreaReference(new CellReference(rowColumnReference2, rowColumnReference0));
            CellAreaReference B1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference1));
            CellAreaReference C1 = new CellAreaReference(new CellReference(rowColumnReference0, rowColumnReference2));
            
            CellReferenceTestHelper.CreateRangeAndAssertResult(expectedRange, B1, C1, A3);
        }

        [TestMethod]
        public void CreateRangesInFunctions()
        {
            this.worksheet.Cells[0, 0].SetValue(1);
            this.worksheet.Cells[1, 0].SetValue(4);
            this.worksheet.Cells[2, 0].SetValue(7);

            this.worksheet.Cells[new CellRange(new CellIndex(0, 0), new CellIndex(0, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);
            this.worksheet.Cells[new CellRange(new CellIndex(1, 0), new CellIndex(1, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);
            this.worksheet.Cells[new CellRange(new CellIndex(2, 0), new CellIndex(2, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);

            RadExpression expression;
            
            RadExpression.TryParse("=SUM(A1:C3)", this.worksheet,  new CellIndex(4, 4),  out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(A1:B2:C3)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(C3:B2:A1)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(C1:B1:A3)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(B3:C3:A1)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(A3:B2:C1)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(A3:C1:B2)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(A1:C3:B2)", this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);

            RadExpression.TryParse("=SUM(A1:C3:B2:D4)", this.worksheet, new CellIndex(5, 5), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 45);
        }

        [TestMethod]
        public void CreateUnionInFunctions()
        {
            this.worksheet.Cells[0, 0].SetValue(1);
            this.worksheet.Cells[1, 0].SetValue(4);
            this.worksheet.Cells[2, 0].SetValue(7);

            this.worksheet.Cells[new CellRange(new CellIndex(0, 0), new CellIndex(0, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);
            this.worksheet.Cells[new CellRange(new CellIndex(1, 0), new CellIndex(1, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);
            this.worksheet.Cells[new CellRange(new CellIndex(2, 0), new CellIndex(2, 2))].FillDataSeriesLinear(CellOrientation.Horizontal, 1);

            RadExpression expression;

            RadExpression.TryParse("=SUM(A1{ls}C3)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 10);

            RadExpression.TryParse("=SUM(A1{ls}B2{ls}C3)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 15);

            RadExpression.TryParse("=SUM(A1{ls}A1)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 2);

            RadExpression.TryParse("=SUM(A1:B2{ls}C1)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 15);

            RadExpression.TryParse("=SUM(A1:B2{ls}B2:C1)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 28);

            RadExpression.TryParse("=SUM(A3:C1{ls}A1:C3)".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 90);

            RadExpression.TryParse("=SUM((A1:B2{ls}B2) (B2{ls}B2:C3))".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 20);

            RadExpression.TryParse("=SUM((A1:B2) (B2{ls}B2:C3))".ReplaceListSeparator(), this.worksheet, new CellIndex(4, 4), out expression);
            CellReferenceTestHelper.AssertNumberExpressionValue(expression, 10);
        }

        #endregion


        #region Translation Tests of CellReferencesRanges in Formulas

        [TestMethod]
        public void TranslationOfFormulas()
        {
            this.worksheet.Cells[0, 0].SetValue(1);
            this.worksheet.Cells[1, 0].SetValue(2);
            this.worksheet.Cells[2, 0].SetValue(3);
            this.worksheet.Cells[3, 0].SetValue(4);

            this.worksheet.Cells[0, 1].SetValue("=SUM(A$1:A1)");
            FormulaCellValue cellValue = this.worksheet.Cells[0, 1].GetValue().Value as FormulaCellValue;
         
            Assert.IsNotNull(cellValue);
            Assert.AreEqual(1, cellValue.Value.GetValueAsConstantExpression().NumberValue());

            CellReferenceTestHelper.AssertTranslatedFormula(worksheet, cellValue, new CellIndex(1, 1), 3);
            CellReferenceTestHelper.AssertTranslatedFormula(worksheet, cellValue, new CellIndex(2, 1), 6);
            CellReferenceTestHelper.AssertTranslatedFormula(worksheet, cellValue, new CellIndex(3, 1), 10);
        }

        #endregion
    }
}
