﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Copying;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    [TestClass]
    public class CellStyleCollectionsMergerTests
    {
        [TestMethod]
        public void Merge_WhenTargetCellStyleCollectionAndSourceCellStyleCollectionContainAbsolutelyEqualStyle_SourceStyleIsNotAddedToTheTargetStyleCollection()
        {
            // Arrange
            Workbook targetWorkbook = new Workbook();
            int initialStylesCount = targetWorkbook.Styles.Count;
            CellStyle targetStyle = targetWorkbook.Styles.Add("Style1");

            Workbook sourceWorkbook = new Workbook();
            sourceWorkbook.Styles.Add("Style1");

            // Act
            CellStyleCollectionsMerger stylesMerger = new CellStyleCollectionsMerger(targetWorkbook.Styles, sourceWorkbook.Styles, CreateCopyContext(targetWorkbook.Worksheets.Add()));
            stylesMerger.Merge();

            // Assert
            Assert.AreEqual(1 + initialStylesCount, targetWorkbook.Styles.Count);
            Assert.AreEqual(targetStyle, targetWorkbook.Styles["Style1"]);
        }

        [TestMethod]
        public void Merge_WhenTargetCellStyleCollectionAndSourceCellStyleCollectionContainDifferentStylesButWithSameStyleName_SourceStyleIsRenamedAndAddedToTheTargetStyleCollection()
        {
            // Arrange
            Workbook targetWorkbook = new Workbook();
            int initialStylesCount = targetWorkbook.Styles.Count;
            CellStyle targetStyle = targetWorkbook.Styles.Add("Style1");
            targetStyle.FontSize = 20;

            Workbook sourceWorkbook = new Workbook();
            CellStyle sourceStyle = sourceWorkbook.Styles.Add("Style1");

            // Act
            CellStyleCollectionsMerger stylesMerger = new CellStyleCollectionsMerger(targetWorkbook.Styles, sourceWorkbook.Styles, CreateCopyContext(targetWorkbook.Worksheets.Add()));
            stylesMerger.Merge();

            // Assert
            Assert.AreEqual(2 + initialStylesCount, targetWorkbook.Styles.Count);
            Assert.AreEqual(targetStyle, targetWorkbook.Styles["Style1"]);

            CellStyle renanmedStyle = targetWorkbook.Styles.GetByName("Style1 1");
            Assert.IsNotNull(renanmedStyle);
            Assert.IsTrue(renanmedStyle.HasSameProperties(sourceStyle));
        }

        [TestMethod]
        public void Merge_WhenSourceCellStyleCollectionContainsStyleWhichDoesntExistInTargetCellStyleCollection_SourceStyleIsAddedToTheTargetStyleCollection()
        {
            // Arrange
            Workbook targetWorkbook = new Workbook();
            int initialStylesCount = targetWorkbook.Styles.Count;

            Workbook sourceWorkbook = new Workbook();
            CellStyle sourceStyle = sourceWorkbook.Styles.Add("Style1");
            sourceStyle.FontSize = 20;

            // Act
            CellStyleCollectionsMerger stylesMerger = new CellStyleCollectionsMerger(targetWorkbook.Styles, sourceWorkbook.Styles, CreateCopyContext(targetWorkbook.Worksheets.Add()));
            stylesMerger.Merge();

            // Assert
            Assert.AreEqual(1 + initialStylesCount, targetWorkbook.Styles.Count);
            Assert.AreEqual(sourceStyle, targetWorkbook.Styles["Style1"]);
        }

        [TestMethod]
        public void Merge_WhenSourceCellStyleCollectionContainsModifiedNormalStyle_SourceStyleNormalIsRenamedAndAddedToTheTargetStyleCollection()
        {
            // Arrange
            Workbook targetWorkbook = new Workbook();
            int initialStylesCount = targetWorkbook.Styles.Count;

            Workbook sourceWorkbook = new Workbook();
            CellStyle sourceStyle = sourceWorkbook.Styles["Normal"];
            sourceStyle.FontSize = 20;

            // Act
            CellStyleCollectionsMerger stylesMerger = new CellStyleCollectionsMerger(targetWorkbook.Styles, sourceWorkbook.Styles, CreateCopyContext(targetWorkbook.Worksheets.Add()));
            stylesMerger.Merge();

            // Assert
            Assert.AreEqual(1 + initialStylesCount, targetWorkbook.Styles.Count);
            Assert.IsTrue(targetWorkbook.Styles["Normal 1"].HasSameProperties(sourceStyle));
        }

        private static CopyContext CreateCopyContext(Worksheet targetWorksheet)
        {
            return new CopyContext(targetWorksheet, new Worksheet("source", new Workbook()));
        }
    }
}