﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    public class CellValueFormatTestsBase : MultiCulturalTestClassBase
    {
        #region Fields

        public static readonly string TestNumberFormat = "#{gs}##0{ds}00";
        public static readonly string TestNumberToString = "123{gs}456{ds}79";
        public static readonly string FractionShortFormat = "# ?/?";
        public static readonly string FractionLongFormat = "# ??/??";
        public static readonly string[] EnUSCulture = new string[] { "en-US" };
        public static readonly string[] DeDECulture = new string[] { "de-DE" };
        public static readonly string[] ItITCulture = new string[] { "it-IT" };
        public static readonly string[] BgBGCulture = new string[] { "bg-BG" };
        public static readonly string[] EsESCulture = new string[] { "es-ES" };
        public static readonly string[] EsVECulture = new string[] { "es-VE" };
        public static readonly string[] KoKRCulture = new string[] { "ko-KR" };
        public static readonly string[] PaINCulture = new string[] { "pa-IN" };
        public static readonly string[] NlNLCulture = new string[] { "nl-NL" };
        public static readonly string[] NonSpaceGroupSeparatorCultures = new string[] { "en-US", "ar-DZ", "ar-QA", "da-DK", "de-DE", "es-ES", "fr-BE", "it-IT", "ko-KR" };
        public static readonly string[] NonPunjabiCultures = new string[] { "en-US", "bg-BG", "ar-DZ", "ar-QA", "da-DK", "de-DE", "es-ES", "fr-BE", "fr-FR", "it-IT", "ko-KR", "nl-BE", "ru-RU", "sv-FI" };
        public static readonly string[] NonArabicCultures = new string[] { "en-US", "bg-BG", "da-DK", "de-DE", "es-ES", "fr-BE", "fr-FR", "it-IT", "ko-KR", "nl-BE", "pa-IN", "ru-RU", "sv-FI" };
        public static readonly string DefaultDateString = "6/10/1987 3:44:55.678";

        public static string NumberToString2
        {
            get
            {
                return "123456{ds}7899".ReplaceGroupDecimalSeparators();
            }
        }

        #endregion


        #region Helper Methods

        internal string ReplaceCurrencySymbol(string input, string currencySymbol)
        {
            return input.Replace("{cs}", currencySymbol);
        }

        // Set a value that requires a format to be applied
        internal void SetValueAndAssertAppliedFormat(string cellValueString, string expectedFormatString, double expectedValue, string exptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            RunTestInMultipleCultures(() =>
            {
                SetValueAndAssertAppliedFormatInternal(cellValueString, expectedFormatString, 
                    expectedValue.ToDefaultSpreadsheetCultureString(), exptectedResultValueString, expectedType);

            }, cultures);
        }

        internal void SetValueAndAssertAppliedFormat(string cellValueString, string expectedFormatString, string expectedValueString, string exptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            RunTestInMultipleCultures(() =>
            {
                SetValueAndAssertAppliedFormatInternal(cellValueString, expectedFormatString, expectedValueString, exptectedResultValueString, expectedType);

            }, cultures);
        }

        private void SetValueAndAssertAppliedFormatInternal(string cellValueString, string expectedFormatString, string expectedValueString, string exptectedResultValueString, Type expectedType = null)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[0, 0].SetValue(cellValueString.ReplaceGroupDecimalSeparators());

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;

            if (expectedType == null)
            {
                expectedType = typeof(NumberCellValue);
            }

            Assert.IsInstanceOfType(cellValue, expectedType, GetErrorMessage());
            Assert.AreEqual(expectedFormatString.ReplaceGroupDecimalSeparators(), format.FormatString, true, GetErrorMessage());
            Assert.AreEqual(expectedValueString.ReplaceGroupDecimalSeparators(), cellValue.GetValueAsString(format), GetErrorMessage());
            string cellStringValueIncludingInvisibleSymbols = LayoutHelper.GetResultValueStringIncludingInvisibleSymbols(cellValue, format);
            Assert.AreEqual(exptectedResultValueString.ReplaceGroupDecimalSeparators(), cellStringValueIncludingInvisibleSymbols, GetErrorMessage());
        }

        // Set value and format; assert the format is applied correctly
        internal void SetValueAndFormatAssertResult(string cellValueString, string cellFormatString, string expectedFormatString,
            string expectedValueString, string exptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = CreateWorksheetSetValueThenFormat(cellValueString, cellFormatString);

                SetValueAndFormatAssertResultInternal(worksheet, expectedFormatString, expectedValueString, exptectedResultValueString, expectedType);

            }, cultures);
        }

        internal void SetValueAndFormatAssertResult(DateTime cellInput, string cellFormatString, string expectedFormatString,
           string expectedValueString, string exptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            Worksheet worksheet = CreateWorksheetSetValueThenFormat(cellInput, cellFormatString);

            SetValueAndFormatAssertResultInternal(worksheet, expectedFormatString, expectedValueString, exptectedResultValueString, expectedType);
        }

        internal void SetValueThenFormatAppliedCorrectly(string formatString, string cellValueInput, Type expectedCellValueType, string expectedValueString,
            string exptectedResultValueString, Color? expectedForeground, Predicate<double> expectedCondition, bool expectedShouldExpand, string[] cultures = null)
        {
            AssertValueAndFormatAppliedCorrectly(false, formatString, cellValueInput, expectedCellValueType, expectedValueString, exptectedResultValueString,
                expectedForeground, expectedCondition, expectedShouldExpand, cultures);
        }

        // Set format and value; assert the format is applied correctly
        internal void SetFormatAndValueAssertResult(string cellFormatString, string cellValueString, string expectedFormatString,
            string expectedValueString, string exptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = CreateWorksheetSetFormatThenValue(cellValueString, cellFormatString);

                SetValueAndFormatAssertResultInternal(worksheet, expectedFormatString, expectedValueString, exptectedResultValueString, expectedType);
            }, cultures);
        }

        
        /// <summary>
        /// Set format on a single cell; set value on a range including this cell; assert format is applied correctly on both
        /// this cell and the rest of the range.
        /// </summary>
        /// <param name="cellFormatString">The format string set on the single ce;;.</param>
        /// <param name="cellValueString">The value set to the whole range.</param>
        /// <param name="expectedFormatString">The expected format string of the single cell.</param>
        /// <param name="expectedValueString">The expected value string of the single cell.</param>
        /// <param name="exptectedResultValueString">The exptected result value string of the single cell.</param>
        /// <param name="otherExpectedFormatString">The expected format string of the rest of the range.</param>
        /// <param name="otherExpectedValueString">The expected value string of the rest of the range.</param>
        /// <param name="otherExptectedResultValueString">The exptected result value string of the rest of the range.</param>
        /// <param name="cultures">The cultures.</param>
        /// <param name="expectedType">The expected type.</param>
        internal void SetFormatAndValueAsRangeAssertResult(string cellFormatString, string cellValueString, string expectedFormatString,
            string expectedValueString, string exptectedResultValueString, string otherExpectedFormatString, string otherExpectedValueString,
            string otherExptectedResultValueString, string[] cultures = null, Type expectedType = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = CreateWorksheetSetFormatThenValueAsPartOfRange(cellValueString, cellFormatString);

                SetValueAndFormatAssertResultInternal(worksheet, expectedFormatString, expectedValueString, exptectedResultValueString, expectedType);
                SetValueAndFormatAssertResultInternal(worksheet, otherExpectedFormatString, otherExpectedValueString, otherExptectedResultValueString, expectedType, range: new CellRange(1, 0, 3, 0));
            }, cultures);
        }

        internal void SetFormatThenValueAppliedCorrectly(string formatString, string cellValueInput, Type expectedCellValueType,
          string expectedValueString, string exptectedResultValueString, Color? expectedForeground, Predicate<double> expectedCondition, bool expectedShouldExpand)
        {
            AssertValueAndFormatAppliedCorrectly(true, formatString, cellValueInput, expectedCellValueType, expectedValueString, exptectedResultValueString,
                expectedForeground, expectedCondition, expectedShouldExpand);
        }

        internal void SetValueTwiceAssertAppliedFormat(string firstCellValue, string firstExpectedFormat, Type firstExpectedType, string secondCellValue,
            string secondExpectedCellValue, string secondExpectedResultValue, Type secondExpectedType, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue(firstCellValue);

                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.AreEqual(firstExpectedFormat.ReplaceGroupDecimalSeparators(), format.FormatString);
                Assert.IsInstanceOfType(cellValue, firstExpectedType);

                worksheet.Cells[0, 0].SetValue(secondCellValue);
                cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.AreEqual(secondExpectedCellValue.ReplaceGroupDecimalSeparators(), cellValue.GetValueAsString(format));
                Assert.AreEqual(secondExpectedResultValue.ReplaceGroupDecimalSeparators(), cellValue.GetResultValueAsString(format));
                Assert.IsInstanceOfType(cellValue, secondExpectedType);
            }, cultures);
        }

        internal void AssertDateFormatAppliedCorrectly(string formatString, string cellInput, Type expectedType,
            int numberOfItems, bool applyFormat, bool isTransparent, bool shouldExpand, string expectedText, string[] cultures = null, FormatPatternInfo expectedFormatInfo = null)
        {
            RunTestInMultipleCultures(() =>
                {
                    List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
                    items.Add(new CellValueFormatResultItem(expectedText, isTransparent, shouldExpand, applyFormat));
                    FormatPatternInfo formatInfo = expectedFormatInfo ?? new FormatPatternInfo(expectedText, items);
                    AssertCellValueFormatResultAndItems(formatString, cellInput, formatInfo, expectedType);
                }, cultures);
        }
        
        internal void TestScientific(string formatString, string value, FormatPatternInfo formatInfo)
        {
            RunTestInMultipleCultures(() =>
                {
                    AssertCellValueFormatResultAndItems(formatString, value, formatInfo);
                });
        }

        internal void TestAccounting(string format, FormatPatternInfo positive, FormatPatternInfo negative, FormatPatternInfo zero, FormatPatternInfo text)
        {
            RunTestInMultipleCultures(() =>
            {
                this.AssertCellValueFormatResultAndItems(format, (123456.7899).ToDefaultSpreadsheetCultureString(), positive);
                this.AssertCellValueFormatResultAndItems(format, (-123456.7899).ToDefaultSpreadsheetCultureString(), negative);
                this.AssertCellValueFormatResultAndItems(format, "0", zero);
                this.AssertCellValueFormatResultAndItems(format, "test", text, expectedValueType: typeof(TextCellValue));
            }, EnUSCulture);
        }

        internal void SetFormatAndAssertCategory(string formatString, FormatStringCategory expectedCategory, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
            {
                SetFormatAndAssertCategoryInternal(formatString, expectedCategory);

            }, cultures);
        }

        internal void SetFormatAndAssertCategoryInternal(string formatString, FormatStringCategory expectedCategory)
        {
            Worksheet worksheet = CreateWorksheetSetFormat(formatString);

            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;

            Assert.AreEqual(expectedCategory, format.FormatStringInfo.Category);
        }

        internal void TestFormulaCellValue(string newValue, string expectedFormat, string expectedValueAsString,
           string expectedResultValueAsString, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue(newValue.ReplaceListSeparator());

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                FormulaCellValue formulaCellValue = cellValue as FormulaCellValue;

                Assert.IsNotNull(formulaCellValue);

                Assert.AreEqual(expectedFormat.ReplaceListSeparator().ReplaceGroupDecimalSeparators(), format.FormatString, true, GetErrorMessage());
                Assert.AreEqual(expectedValueAsString.ReplaceListSeparator().ReplaceGroupDecimalSeparators(), cellValue.GetValueAsString(format), true);
                Assert.AreEqual(expectedResultValueAsString.ReplaceGroupDecimalSeparators(),
                    format.GetFormatResult(formulaCellValue.GetResultValueAsCellValue()).VisibleInfosText);
            
            }, cultures);
        }

        private string GetErrorMessage()
        {
            return FormatHelper.DefaultSpreadsheetCulture.CultureInfo.Name;
        }

        private void AssertValueAndFormatAppliedCorrectly(bool setFormatFirst, string formatString, string cellValueInput, Type expectedCellValueType, string expectedValueString,
            string exptectedResultValueString, Color? expectedForeground, Predicate<double> expectedCondition, bool expectedShouldExpand, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet;
                if (setFormatFirst)
                {
                    worksheet = CreateWorksheetSetFormatThenValue(cellValueInput, formatString);
                }
                else
                {
                    worksheet = CreateWorksheetSetValueThenFormat(cellValueInput, formatString);
                }

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
                Assert.IsInstanceOfType(cellValue, expectedCellValueType, FormatHelper.DefaultSpreadsheetCulture.CultureInfo.Name);

                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);

                Assert.AreEqual(expectedValueString.ReplaceGroupDecimalSeparators(), cellValue.GetValueAsString(format));
                string cellValueResultStringIncludingInvisibleSymbols = LayoutHelper.GetResultValueStringIncludingInvisibleSymbols(cellValue, format);
                Assert.AreEqual(exptectedResultValueString.ReplaceGroupDecimalSeparators(), cellValueResultStringIncludingInvisibleSymbols);
                Assert.AreEqual(expectedForeground, cellValueFormatResult.Foreground);
                Assert.AreEqual(expectedCondition, cellValueFormatResult.Condition);
                Assert.AreEqual(expectedShouldExpand, cellValueFormatResult.ContainsExpandableInfo);
            }, cultures);
        }

        private Worksheet CreateWorksheetSetValueThenFormat(DateTime cellInput, string cellFormatString)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[0, 0].SetWidth(new ColumnWidth(250, true));
            worksheet.Cells[0, 0].SetValue(cellInput);
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(cellFormatString.ReplaceGroupDecimalSeparators()));

            return worksheet;
        }

        private Worksheet CreateWorksheetSetValueThenFormat(string cellInput, string cellFormatString)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[0, 0].SetWidth(new ColumnWidth(250, true));
            worksheet.Cells[0, 0].SetValue(cellInput.ReplaceGroupDecimalSeparators());
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(cellFormatString.ReplaceGroupDecimalSeparators()));

            return worksheet;
        }

        private Worksheet CreateWorksheetSetFormatThenValue(string cellInput, string cellFormatString)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[0, 0].SetWidth(new ColumnWidth(250, true));
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(cellFormatString.ReplaceGroupDecimalSeparators()));
            worksheet.Cells[0, 0].SetValue(cellInput.ReplaceGroupDecimalSeparators().ReplaceListSeparator());

            return worksheet;
        }

        private Worksheet CreateWorksheetSetFormatThenValueAsPartOfRange(string cellInput, string cellFormatString)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[0, 0].SetWidth(new ColumnWidth(250, true));
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(cellFormatString.ReplaceGroupDecimalSeparators()));
            worksheet.Cells[0, 0, 3, 0].SetValue(cellInput.ReplaceGroupDecimalSeparators().ReplaceListSeparator());

            return worksheet;
        }

        private Worksheet CreateWorksheetSetFormat(string cellFormatString)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[0, 0].SetWidth(new ColumnWidth(250, true));
            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(cellFormatString.ReplaceGroupDecimalSeparators()));

            return worksheet;
        }

        private void SetValueAndFormatAssertResultInternal(Worksheet worksheet, string expectedFormatString, string expectedValueString, string exptectedResultValueString, Type expectedType = null, CellRange range = null)
        {
            if (range == null)
            {
                range = new CellRange(0, 0, 0, 0);
            }

            ICellValue cellValue = worksheet.Cells[range].GetValue().Value;
            CellValueFormat format = worksheet.Cells[range].GetFormat().Value;

            if (expectedType == null)
            {
                expectedType = typeof(NumberCellValue);
            }

            Assert.IsInstanceOfType(cellValue, expectedType);
            Assert.AreEqual(expectedFormatString.ReplaceGroupDecimalSeparators(), format.FormatString);
            Assert.AreEqual(expectedValueString.ReplaceGroupDecimalSeparators(), cellValue.GetValueAsString(format));
            string valueAsStringIncludingInvisibleSymbols = LayoutHelper.GetResultValueStringIncludingInvisibleSymbols(cellValue, format);
            Assert.AreEqual(exptectedResultValueString.ReplaceGroupDecimalSeparators(), valueAsStringIncludingInvisibleSymbols);
        }

        #endregion


        #region Helper Methods Testing CellValueFormatResultItems

        internal void AssertCellValueFormatResultAndItems(string formatString, string value, FormatPatternInfo formatInfo, Type expectedValueType = null)
        {
            Worksheet worksheet = CreateWorksheetSetFormatThenValue(value, formatString);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            if (expectedValueType == null)
            {
                expectedValueType = typeof(NumberCellValue);
            }

            Assert.IsInstanceOfType(cellValue, expectedValueType);

            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;

            Assert.AreEqual(formatInfo.ExpectedText.ReplaceGroupDecimalSeparators(), infosText);
            Assert.AreEqual(formatInfo.Infos.Count, infos.Count());

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                Assert.AreEqual(formatInfo.Infos[i].ApplyFormat, info.ApplyFormat);
                Assert.AreEqual(formatInfo.Infos[i].IsTransparent, info.IsTransparent);
                Assert.AreEqual(formatInfo.Infos[i].ShouldExpand, info.ShouldExpand);
                Assert.AreEqual(formatInfo.Infos[i].Text.ReplaceGroupDecimalSeparators(), info.Text);

                i++;
            }
        }

        public void TestCurrency(bool isPositive, string formatString, int itemCount, Color? foreground, string text,
        string item1Text, bool isItem1Transparent = false, bool isItem1Symbol = true, string item2Text = null, bool isItem2Transparent = false,
        bool isItem2Symbol = true, string item3Text = null, bool isItem3Transparent = false, bool isItem3Symbol = true)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();

                worksheet.Cells[0, 0].SetFormat(new CellValueFormat(formatString.ReplaceGroupDecimalSeparators()));
                double testValue = isPositive ? 123456.7899 : -123456.7899;
                worksheet.Cells[0, 0].SetValue(testValue.ToDefaultSpreadsheetCultureString());

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
                Assert.IsTrue(cellValue is NumberCellValue);

                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
                IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                string infosText = cellValueFormatResult.InfosText;
                bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                Assert.AreEqual(foreground, cellValueFormatResult.Foreground);
                Assert.AreEqual(null, cellValueFormatResult.Condition);
                Assert.AreEqual(null, cellValueFormatResult.Culture);
                Assert.AreEqual(false, shouldExpand);
                Assert.AreEqual(text.ReplaceGroupDecimalSeparators(), infosText);

                int i = 0;
                foreach (CellValueFormatResultItem info in infos)
                {
                    if (i == 0 && item1Text != null)
                    {
                        Assert.AreEqual(isItem1Symbol, info.ApplyFormat);
                        Assert.AreEqual(isItem1Transparent, info.IsTransparent);
                        Assert.AreEqual(false, info.ShouldExpand);
                        Assert.AreEqual(item1Text.ReplaceGroupDecimalSeparators(), info.Text);
                    }

                    if (i == 1 && item2Text != null)
                    {
                        Assert.AreEqual(isItem2Symbol, info.ApplyFormat);
                        Assert.AreEqual(isItem2Transparent, info.IsTransparent);
                        Assert.AreEqual(false, info.ShouldExpand);
                        Assert.AreEqual(item2Text.ReplaceGroupDecimalSeparators(), info.Text);
                    }

                    if (i == 2 && item3Text != null)
                    {
                        Assert.AreEqual(isItem3Symbol, info.ApplyFormat);
                        Assert.AreEqual(isItem3Transparent, info.IsTransparent);
                        Assert.AreEqual(false, info.ShouldExpand);
                        Assert.AreEqual(item3Text.ReplaceGroupDecimalSeparators(), info.Text);
                    }
                    i++;
                }
            }, EnUSCulture);
        }

        #endregion
    }
}
