﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Accounting : CellValueFormatTestsBase
    {
        #region Fields

        public static readonly string AccountingFormatEnUS1 = @"_(""$""* #{gs}##0_);_(""$""* (#{gs}##0);_(""$""* ""-""_);_(@_)";
        public static readonly string AccountingFormatEnUS2 = @"_(* #{gs}##0_);_(* (#{gs}##0);_(* ""-""_);_(@_)";
        public static readonly string AccountingFormatEnUS3 = @"_(""$""* #{gs}##0{ds}00_);_(""$""* (#{gs}##0{ds}00);_(""$""* ""-""??_);_(@_)";
        public static readonly string AccountingFormatEnUS4 = @"_(* #{gs}##0.00_);_(* (#{gs}##0.00);_(* ""-""??_);_(@_)";
        public static readonly string AccountingFormatEnUS5 = "_(\"$\"* #,##0.00_);_(\"$\"* \\(#,##0.00\\);_(\"$&\"* \"-\"??_);_(@_)";

        public static readonly string AccountingFormatEsES1 = @"_-* #{gs}##0 ""€""_-;-* #{gs}##0 ""€""_-;_-* ""-"" ""€""_-;_-@_-";
        public static readonly string AccountingFormatEsES2 = @"_-* #{gs}##0 _€_-;-* #{gs}##0 _€_-;_-* ""-"" _€_-;_-@_-";
        public static readonly string AccountingFormatEsES3 = @"_-* #{gs}##0{ds}00 ""€""_-;-* #{gs}##0{ds}00 ""€""_-;_-* ""-""?? ""€""_-;_-@_-";
        public static readonly string AccountingFormatEsES4 = @"_-* #{gs}##0{ds}00 _€_-;-* #{gs}##0{ds}00 _€_-;_-* ""-""?? _€_-;_-@_-";

        public static readonly string AccountingFormatKoKR1 = @"_-₩* #{gs}##0_-;-₩* #{gs}##0_-;_-₩* ""-""_-;_-@_-";
        public static readonly string AccountingFormatKoKR2 = @"_-* #{gs}##0_-;-* #{gs}##0_-;_-* ""-""_-;_-@_-";
        public static readonly string AccountingFormatKoKR3 = @"_-₩* #{gs}##0{ds}00_-;-₩* #{gs}##0{ds}00_-;_-₩* ""-""??_-;_-@_-";
        public static readonly string AccountingFormatKoKR4 = @"_-* #{gs}##0{ds}00_-;-* #{gs}##0{ds}00_-;_-* ""-""??_-;_-@_-";

        public static readonly string AccountingFormatEsVE1 = @"_ {cs} * #{gs}##0_ ;_ {cs} * -#{gs}##0_ ;_ {cs} * ""-""_ ;_ @_ ";
        public static readonly string AccountingFormatEsVE2 = @"_ * #{gs}##0_ ;_ * -#{gs}##0_ ;_ * ""-""_ ;_ @_ ";
        public static readonly string AccountingFormatEsVE3 = @"_ {cs} * #{gs}##0{ds}00_ ;_ {cs} * -#{gs}##0{ds}00_ ;_ {cs} * ""-""??_ ;_ @_ ";
        public static readonly string AccountingFormatEsVE4 = @"_ * #{gs}##0{ds}00_ ;_ * -#{gs}##0{ds}00_ ;_ * ""-""??_ ;_ @_ ";

        #endregion


        #region Set Value Then Accounting Format and Assert Format Applied Correctly in en-US

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat1EnUS()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEnUS1, AccountingFormatEnUS1, "15963", "($ 15{gs}963)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat2EnUS()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEnUS2, AccountingFormatEnUS2, "15963", "( 15{gs}963)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat3EnUS()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEnUS3, AccountingFormatEnUS3, "15963", "($ 15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat4EnUS()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEnUS4, AccountingFormatEnUS4, "15963", "( 15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat1EnUS()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEnUS1, AccountingFormatEnUS1, "-15963", "($ (15{gs}963)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat2EnUS()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEnUS2, AccountingFormatEnUS2, "-15963", "( (15{gs}963)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat3EnUS()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEnUS3, AccountingFormatEnUS3, "-15963", "($ (15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat4EnUS()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEnUS4, AccountingFormatEnUS4, "-15963", "( (15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormatContainingEscapedSymbolsEnUS()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEnUS5, AccountingFormatEnUS5, "15963", "($ 15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPositiveDecimalNumberThenAccountingFormatContainingEscapedSymbolsEnUS()
        {
            SetValueAndFormatAssertResult("15{ds}963", AccountingFormatEnUS5, AccountingFormatEnUS5, "15{ds}963", "($ 15{ds}96)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormatContainingEscapedSymbolsEnUS()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEnUS5, AccountingFormatEnUS5, "-15963", "($ (15{gs}963{ds}00)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeDecimalNumberThenAccountingFormatContainingEscapedSymbolsEnUS()
        {
            SetValueAndFormatAssertResult("-15{ds}963", AccountingFormatEnUS5, AccountingFormatEnUS5, "-15{ds}963", "($ (15{ds}96)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat1EnUS()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEnUS1, AccountingFormatEnUS1, "0", "($ -)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat2EnUS()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEnUS2, AccountingFormatEnUS2, "0", "( -)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat3EnUS()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEnUS3, AccountingFormatEnUS3, "0", "($ -??)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat4EnUS()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEnUS4, AccountingFormatEnUS4, "0", "( -??)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetTextThenAccountingFormat1EnUS()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEnUS1, AccountingFormatEnUS1, "string", "(string)",
                cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat2EnUS()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEnUS2, AccountingFormatEnUS2, "string", "(string)",
                cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat3EnUS()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEnUS3, AccountingFormatEnUS3, "string", "(string)",
                cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat4EnUS()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEnUS4, AccountingFormatEnUS4, "string", "(string)",
                cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        #endregion


        #region Set Value Then Accounting Format and Assert Format Applied Correctly in es-ES

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat1EsES()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsES1, AccountingFormatEsES1, "15963", "- 15{gs}963 €-", cultures: new string[] { "ko-KR" });
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat2EsES()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsES2, AccountingFormatEsES2, "15963", "- 15{gs}963 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat3EsES()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsES3, AccountingFormatEsES3, "15963", "- 15{gs}963{ds}00 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat4EsES()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsES4, AccountingFormatEsES4, "15963", "- 15{gs}963{ds}00 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat1EsES()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsES1, AccountingFormatEsES1, "-15963", "- 15{gs}963 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat2EsES()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsES2, AccountingFormatEsES2, "-15963", "- 15{gs}963 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat3EsES()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsES3, AccountingFormatEsES3, "-15963", "- 15{gs}963{ds}00 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat4EsES()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsES4, AccountingFormatEsES4, "-15963", "- 15{gs}963{ds}00 €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat1EsES()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsES1, AccountingFormatEsES1, "0", "- - €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat2EsES()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsES2, AccountingFormatEsES2, "0", "- - €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat3EsES()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsES3, AccountingFormatEsES3, "0", "- -?? €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat4EsES()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsES4, AccountingFormatEsES4, "0", "- -?? €-", cultures: EsESCulture);
        }

        [TestMethod]
        public void SetTextThenAccountingFormat1EsES()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsES1, AccountingFormatEsES1, "string", "-string-",
                cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat2EsES()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsES2, AccountingFormatEsES2, "string", "-string-",
                cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat3EsES()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsES3, AccountingFormatEsES3, "string", "-string-",
                cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat4EsES()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsES4, AccountingFormatEsES4, "string", "-string-",
                cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        #endregion


        #region Set Value Then Accounting Format and Assert Format Applied Correctly in ko-KR

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat1KoKR()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatKoKR1, AccountingFormatKoKR1, "15963", "-₩ 15{gs}963-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat2KoKR()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatKoKR2, AccountingFormatKoKR2, "15963", "- 15{gs}963-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat3KoKR()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatKoKR3, AccountingFormatKoKR3, "15963", "-₩ 15{gs}963{ds}00-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat4KoKR()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatKoKR4, AccountingFormatKoKR4, "15963", "- 15{gs}963{ds}00-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat1KoKR()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatKoKR1, AccountingFormatKoKR1, "-15963", "-₩ 15{gs}963-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat2KoKR()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatKoKR2, AccountingFormatKoKR2, "-15963", "- 15{gs}963-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat3KoKR()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatKoKR3, AccountingFormatKoKR3, "-15963", "-₩ 15{gs}963{ds}00-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat4KoKR()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatKoKR4, AccountingFormatKoKR4, "-15963", "- 15{gs}963{ds}00-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat1KoKR()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatKoKR1, AccountingFormatKoKR1, "0", "-₩ --", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat2KoKR()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatKoKR2, AccountingFormatKoKR2, "0", "- --", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat3KoKR()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatKoKR3, AccountingFormatKoKR3, "0", "-₩ -??-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat4KoKR()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatKoKR4, AccountingFormatKoKR4, "0", "- -??-", cultures: KoKRCulture);
        }

        [TestMethod]
        public void SetTextThenAccountingFormat1KoKR()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatKoKR1, AccountingFormatKoKR1, "string", "-string-",
                cultures: KoKRCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat2KoKR()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatKoKR2, AccountingFormatKoKR2, "string", "-string-",
                cultures: KoKRCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat3KoKR()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatKoKR3, AccountingFormatKoKR3, "string", "-string-",
                cultures: KoKRCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat4KoKR()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatKoKR4, AccountingFormatKoKR4, "string", "-string-",
                cultures: KoKRCulture, expectedType: typeof(TextCellValue));
        }

        #endregion


        #region Set Value Then Accounting Format and Assert Format Applied Correctly in es-VE

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat1EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE1;
            string testExpectedValueString = " {cs}  15{gs}963 ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("15963", formatString, formatString, "15963", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat2EsVE()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsVE2, AccountingFormatEsVE2, "15963", "  15{gs}963 ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat3EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE3;
            string testExpectedValueString = " {cs}  15{gs}963{ds}00 ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("15963", formatString, formatString, "15963", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetPositiveWholeNumberThenAccountingFormat4EsVE()
        {
            SetValueAndFormatAssertResult("15963", AccountingFormatEsVE4, AccountingFormatEsVE4, "15963", "  15{gs}963{ds}00 ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat1EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE1;
            string testExpectedValueString = " {cs}  -15.963 ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("-15963", formatString, formatString, "-15963", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat2EsVE()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsVE2, AccountingFormatEsVE2, "-15963", "  -15.963 ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat3EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE3;
            string testExpectedValueString = " {cs}  -15.963,00 ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("-15963", formatString, formatString, "-15963", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenAccountingFormat4EsVE()
        {
            SetValueAndFormatAssertResult("-15963", AccountingFormatEsVE4, AccountingFormatEsVE4, "-15963", "  -15.963,00 ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat1EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE1;
            string testExpectedValueString = " {cs}  - ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("0", formatString, formatString, "0", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat2EsVE()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsVE2, AccountingFormatEsVE2, "0", "  - ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat3EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE3;
            string testExpectedValueString = " {cs}  -?? ";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);
            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetValueAndFormatAssertResult("0", formatString, formatString, "0", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetZeroThenAccountingFormat4EsVE()
        {
            SetValueAndFormatAssertResult("0", AccountingFormatEsVE4, AccountingFormatEsVE4, "0", "  -?? ", cultures: EsVECulture);
        }

        [TestMethod]
        public void SetTextThenAccountingFormat1EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE1;

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            SetValueAndFormatAssertResult("string", formatString, formatString, "string", " string ",
                cultures: EsVECulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat2EsVE()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsVE2, AccountingFormatEsVE2, "string", " string ",
                cultures: EsVECulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat3EsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = AccountingFormatEsVE3;

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            SetValueAndFormatAssertResult("string", formatString, formatString, "string", " string ",
                cultures: EsVECulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextThenAccountingFormat4EsVE()
        {
            SetValueAndFormatAssertResult("string", AccountingFormatEsVE4, AccountingFormatEsVE4, "string", " string ",
                cultures: EsVECulture, expectedType: typeof(TextCellValue));
        }

        #endregion


        #region Set Accounting Format Then Value and Assert Format Applied Correctly

        [TestMethod]
        public void SetAccountingFormatThenNumberValue()
        {
            SetFormatAndValueAssertResult(AccountingFormatEnUS3, "1234{ds}56789", AccountingFormatEnUS3, "1234{ds}56789", "($ 1{gs}234{ds}57)",
                cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetAccountingFormatThenPercentValue()
        {
            SetFormatAndValueAssertResult(AccountingFormatEnUS3, "10{ds}567%", AccountingFormatEnUS3, "0{ds}10567", "($ 0.11)",
                cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetAccountingFormatThenFractionValue()
        {
            SetFormatAndValueAssertResult(AccountingFormatEnUS3, "0 1/2", AccountingFormatEnUS3, "0{ds}5", "($ 0.50)",
                cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetAccountingFormatThenTextValue()
        {
            SetFormatAndValueAssertResult(AccountingFormatEnUS3, "any string", AccountingFormatEnUS3, "any string", "(any string)",
                expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenFunctionValue()
        {
            SetFormatAndValueAssertResult(AccountingFormatEnUS3, "=ABS(-10)", AccountingFormatEnUS3, "=ABS(-10)", "($ 10{ds}00)",
                expectedType: typeof(FormulaCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenNumberValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(AccountingFormatEnUS3, "1{gs}234", AccountingFormatEnUS3, "1234", "($ 1{gs}234{ds}00)",
                "#{gs}##0", "1234", "1{gs}234", expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenCurrencyValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(AccountingFormatEnUS3, "$1234", AccountingFormatEnUS3, "1234", "($ 1{gs}234{ds}00)",
                "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenDateValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(AccountingFormatEnUS3, "10/6/1987", AccountingFormatEnUS3, "32056", "($ 32{gs}056{ds}00)",
                "m/d/yyyy", "10/6/1987", "10/6/1987", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenTimeValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(AccountingFormatEnUS3, "12:30", AccountingFormatEnUS3, "0{ds}520833333333333", "($ 0{ds}52)",
                "h:mm", "12:30:00 PM", "12:30", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenPercentageValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(AccountingFormatEnUS3, "15%", AccountingFormatEnUS3, "0{ds}15", "($ 0{ds}15)",
                "0%", "15%", "15%", expectedType: typeof(NumberCellValue));
        }

        #endregion


        #region Set Accounting Format and Assert Category is Correct

        [TestMethod]
        public void SetAccountingFormatEnUS()
        {
            SetFormatAndAssertCategory("_($* #,##0.00_);_($* (#,##0.00);_($* \"-\"??_);_(@_)", FormatStringCategory.Accounting, EnUSCulture);
        }

        [TestMethod]
        public void SetAccountingFormatBgBG()
        {
            SetFormatAndAssertCategory("_-* # ##0,00 лв._-;-* # ##0,00 лв._-;_-* \"-\"?? лв._-;_-@_-", FormatStringCategory.Accounting, BgBGCulture);
        }

        [TestMethod]
        public void SetAccountingFormatDeDE()
        {
            SetFormatAndAssertCategory("_-* #.##0,00 €_-;-* #.##0,00 €_-;_-* \"-\"?? €_-;_-@_-", FormatStringCategory.Accounting, DeDECulture);
        }

        [TestMethod]
        public void SetAccountingFormatItIT()
        {
            SetFormatAndAssertCategory("_-€*  #.##0,00_-;-€*  #.##0,00_-;_-€*  \"-\"??_-;_-@_-", FormatStringCategory.Accounting, ItITCulture);
        }

        [TestMethod]
        public void SetAccountingFormatEsES()
        {
            SetFormatAndAssertCategory("_-* #.##0,00 €_-;-* #.##0,00 €_-;_-* \"-\"?? €_-;_-@_-", FormatStringCategory.Accounting, EsESCulture);
        }

        //[TestMethod]
        //public void SetAccountingFormatEsVE()
        //{
        //    string cultureName = EnUSCulture[0];
        //    CultureInfo culture = new CultureInfo(cultureName);
        //    string currencySymbol = culture.NumberFormat.CurrencySymbol;

        //    string formatString = this.ReplaceCurrencySymbol(@"_ {cs}*  #.##0,00_ ;_ {cs}*  -#.##0,00_ ;_ {cs}*  ""-""??_ ;_ @_ ", currencySymbol);

        //    SetFormatAndAssertCategory(formatString, FormatStringCategory.Accounting, EsVECulture);
        //}

        //TODO: This test passes locally and fails on the build machine.
        //See TeamPulse item 97040 Some CellValueFormat tests for pa-in culture fail only on the build machine and pass locally.
        //[TestMethod]
        //public void SetAccountingFormatPaIN()
        //{
        //    SetFormatAndAssertCategory("_ ₹*  #,##0.00_ ;_ ₹*  -#,##0.00_ ;_ ₹*  \"-\"??_ ;_ @_ ", FormatStringCategory.Accounting, PaINCulture);
        //}

        #endregion


        #region CellValueFormatResultItem Tests

        [TestMethod]
        public void TestDollarSign()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("$", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("($ " + TestNumberToString + ")", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("$", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("(" + TestNumberToString + ")", false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("($ (" + TestNumberToString + ")", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("$", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("($ -??)", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("(test)", itemsText);

            this.TestAccounting(string.Format(@"_($* {0}_);_($* ({0});_($* ""-""??_);_(@_)", TestNumberFormat), positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Spanish_Chile_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("$ ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("-$  " + TestNumberToString + "-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-$ ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("-$  " + TestNumberToString + "-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("$ ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("-$  -??-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-[$$-340A] * {0}_-;-[$$-340A] * {0}_-;_-[$$-340A] * ""-""??_-;_-@_-", TestNumberFormat), positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Spanish_Uruguay_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("$U ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("($U  " + TestNumberToString + ")", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("$U ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("(" + TestNumberToString + ")", false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("($U  (" + TestNumberToString + ")", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("$U ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("($U  -??)", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("(test)", itemsText);

            this.TestAccounting(string.Format(@"_([$$U-380A] * {0}_);_([$$U-380A] * ({0});_([$$U-380A] * ""-""??_);_(@_)", TestNumberFormat), positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_English_UK_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("£", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("-£ " + TestNumberToString + "-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-£", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("-£ " + TestNumberToString + "-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("£", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("-£ -??-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-[$£-809]* {0}_-;-[$£-809]* {0}_-;_-[$£-809]* ""-""??_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Yi_PRC_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("¥", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo(" ¥ " + TestNumberToString + " ", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("¥", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("-" + TestNumberToString, false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo(" ¥ -" + TestNumberToString + " ", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("¥", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo(" ¥ -?? ", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo(" test ", itemsText);

            this.TestAccounting(string.Format(@"_ [$¥-478]* {0}_ ;_ [$¥-478]* -{0}_ ;_ [$¥-478]* ""-""??_ ;_ @_ ", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Lao_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(")", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("₭", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("  " + TestNumberToString + ")₭ ", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("(" + TestNumberToString + ")₭", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("  (" + TestNumberToString + ")₭ ", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(")", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("₭", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("  -??)₭ ", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo(" test ", itemsText);

            this.TestAccounting(string.Format(@"_ * {0}_)[$₭-454]_ ;_ * ({0})[$₭-454]_ ;_ * ""-""??_)[$₭-454]_ ;_ @_ ", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_French_Belgium_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("€ ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo(" €  " + TestNumberToString + " ", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("€ ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("-" + TestNumberToString, false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo(" €  -" + TestNumberToString + " ", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("€ ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo(" €  -?? ", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo(" test ", itemsText);

            this.TestAccounting(string.Format(@"_ [$€-80C] * {0}_ ;_ [$€-80C] * -{0}_ ;_ [$€-80C] * ""-""??_ ;_ @_ ", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Khmer_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString + "៛", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("- " + TestNumberToString + "៛-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + "៛", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("- " + TestNumberToString + "៛-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("៛", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("- -??៛-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-* {0}[$៛-453]_-;-* {0}[$៛-453]_-;_-* ""-""??[$៛-453]_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Pashto_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("؋", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo(string.Format("-؋ {0}-", TestNumberToString), itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("؋", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + "-", false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo(string.Format("-؋ {0}-", TestNumberToString), itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("؋", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("-؋ -??-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-[$؋-463]* {0}_-;_-[$؋-463]* {0}-;_-[$؋-463]* ""-""??_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Serbian_Latin_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("Din.", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("- " + TestNumberToString + " Din.-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("Din.", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("- " + TestNumberToString + " Din.-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("Din.", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("- -?? Din.-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-* {0} [$Din.-241A]_-;-* {0} [$Din.-241A]_-;_-* ""-""?? [$Din.-241A]_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Azeri_Latin_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("man.", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("- " + TestNumberToString + " man.-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("man.", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("- " + TestNumberToString + " man.-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("man.", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("- -?? man.-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-* {0} [$man.-42C]_-;-* {0} [$man.-42C]_-;_-* ""-""?? [$man.-42C]_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_English_India_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("Rs.", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo(" Rs.  " + TestNumberToString + " ", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("Rs.", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("-" + TestNumberToString, false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo(" Rs.  -" + TestNumberToString + " ", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("Rs.", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo(" Rs.  -?? ", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(" ", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo(" test ", itemsText);

            this.TestAccounting(string.Format(@"_ [$Rs.-4009] * {0}_ ;_ [$Rs.-4009] * -{0}_ ;_ [$Rs.-4009] * ""-""??_ ;_ @_ ", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Bulgarian_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("лв.", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("- " + TestNumberToString + " лв.-", itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + " ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("лв.", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("- " + TestNumberToString + " лв.-", itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("лв.", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("- -?? лв.-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-* {0} [$лв.-402]_-;-* {0} [$лв.-402]_-;_-* ""-""?? [$лв.-402]_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Arabic_Qatar_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("ر.ق.‏", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("-ر.ق.‏  123,456.79-".ReplaceGroupDecimalSeparators(), itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("ر.ق.‏", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + "-", false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("-ر.ق.‏  123,456.79-".ReplaceGroupDecimalSeparators(), itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("ر.ق.‏", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("-ر.ق.‏  -??-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-[$ر.ق.‏-4001] * {0}_-;_-[$ر.ق.‏-4001] * {0}-;_-[$ر.ق.‏-4001] * ""-""??_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_Persian_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("ريال", false, false, false));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo(string.Format("-ريال  {0}-", TestNumberToString), itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("ريال", false, false, false));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem(TestNumberToString + "-", false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo(string.Format("-ريال  {0}-", TestNumberToString), itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("ريال", false, false, false));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("-ريال  -??-", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem("-", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("-test-", itemsText);

            this.TestAccounting(string.Format(@"_-[$ريال-429] * {0}_-;_-[$ريال-429] * {0}-;_-[$ريال-429] * ""-""??_-;_-@_-", TestNumberFormat),
                positive, negative, zero, text);
        }

        [TestMethod]
        public void Test_ADP_Accounting()
        {
            List<CellValueFormatResultItem> itemsPositive = new List<CellValueFormatResultItem>();
            itemsPositive.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsPositive.Add(new CellValueFormatResultItem("ADP ", false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsPositive.Add(new CellValueFormatResultItem(TestNumberToString, false, false, true));
            itemsPositive.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo positive = new FormatPatternInfo("(ADP  " + TestNumberToString + ")".ReplaceGroupDecimalSeparators(), itemsPositive);

            List<CellValueFormatResultItem> itemsNegative = new List<CellValueFormatResultItem>();
            itemsNegative.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsNegative.Add(new CellValueFormatResultItem("ADP ", false, false, true));
            itemsNegative.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsNegative.Add(new CellValueFormatResultItem("(" + TestNumberToString + ")".ReplaceGroupDecimalSeparators(), false, false, true));
            FormatPatternInfo negative = new FormatPatternInfo("(ADP  (" + TestNumberToString + ")".ReplaceGroupDecimalSeparators(), itemsNegative);

            List<CellValueFormatResultItem> itemsZero = new List<CellValueFormatResultItem>();
            itemsZero.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem("ADP ", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem(" ", false, true, true));
            itemsZero.Add(new CellValueFormatResultItem("-", false, false, true));
            itemsZero.Add(new CellValueFormatResultItem("??", true, false, true));
            itemsZero.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo zero = new FormatPatternInfo("(ADP  -??)", itemsZero);

            List<CellValueFormatResultItem> itemsText = new List<CellValueFormatResultItem>();
            itemsText.Add(new CellValueFormatResultItem("(", true, false, true));
            itemsText.Add(new CellValueFormatResultItem("test", false, false, true));
            itemsText.Add(new CellValueFormatResultItem(")", true, false, true));
            FormatPatternInfo text = new FormatPatternInfo("(test)", itemsText);

            this.TestAccounting(string.Format(@"_([$ADP] * {0}_);_([$ADP] * ({0});_([$ADP] * ""-""??_);_(@_)", TestNumberFormat),
                positive, negative, zero, text);
        }

        #endregion
    }
}
