﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Currency : CellValueFormatTestsBase
    {
        #region Fields

        public static readonly string TestCurrencyWholeNumberFormatEnUS = "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)";
        public static readonly string TestCurrencyDecimalNumberFormatEnUS = "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)";
        public static readonly string TestCurrencyWholeNumberFormatDeDE = "#{gs}##0 \"€\";[Red]-#{gs}##0 \"€\"";
        public static readonly string TestCurrencyDecimalNumberFormatDeDE = "#{gs}##0{ds}00 \"€\";[Red]-#{gs}##0{ds}00 \"€\"";
        public static readonly string TestCurrencyWholeNumberFormatItIT = "\"€\" #{gs}##0;[Red]-\"€\" #{gs}##0";
        public static readonly string TestCurrencyDecimalNumberFormatItIT = "\"€\" #{gs}##0{ds}00;[Red]-\"€\" #{gs}##0{ds}00";
        public static readonly string TestCurrencyWholeNumberFormatBgBG = "#{gs}##0 \"лв.\";[Red]-#{gs}##0 \"лв.\"";
        public static readonly string TestCurrencyDecimalNumberFormatBgBG = "#{gs}##0{ds}00 \"лв.\";[Red]-#{gs}##0{ds}00 \"лв.\"";
        public static readonly string TestCurrencyWholeNumberFormatEsVE = "\"{cs}\" #{gs}##0;[Red]\"{cs}\" -#{gs}##0";
        public static readonly string TestCurrencyDecimalNumberFormatEsVE = "\"{cs}\" #{gs}##0{ds}00;[Red]\"{cs}\" -#{gs}##0{ds}00";

        #endregion


        #region Set Value and Assert Currency Format in en-US

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS()
        {
            this.SetValueAndAssertAppliedFormat("$123", TestCurrencyWholeNumberFormatEnUS, "123", "$123)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS1()
        {
            this.SetValueAndFormatAssertResult("12.39", "$#,##0.00", "$#,##0.00", "12.39", "$12.39", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatZeroEnUS()
        {
            this.SetValueAndAssertAppliedFormat("$0", TestCurrencyWholeNumberFormatEnUS, "0", "$0)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatDecimalNumberEnUS()
        {
            this.SetValueAndAssertAppliedFormat("$0{ds}1567", TestCurrencyDecimalNumberFormatEnUS, "0{ds}1567", "$0{ds}16)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS2()
        {
            this.SetValueAndAssertAppliedFormat("$1{gs}234", TestCurrencyWholeNumberFormatEnUS, "1234", "$1{gs}234)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS3()
        {
            this.SetValueAndAssertAppliedFormat("$1{gs}234{ds}567", TestCurrencyDecimalNumberFormatEnUS, "1234{ds}567", "$1{gs}234{ds}57)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS4()
        {
            this.SetValueAndAssertAppliedFormat("$1{gs}234{gs}567{ds}567", TestCurrencyDecimalNumberFormatEnUS, "1234567{ds}567", "$1{gs}234{gs}567{ds}57)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEnUS5()
        {
            this.SetValueAndAssertAppliedFormat("$-1{gs}234{ds}567", TestCurrencyDecimalNumberFormatEnUS, "-1234{ds}567", "($1{gs}234{ds}57)", cultures: EnUSCulture);
        }

        #endregion


        #region Set Value and Assert Currency Format in de-DE

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE1()
        {
            this.SetValueAndAssertAppliedFormat("€123", TestCurrencyWholeNumberFormatDeDE, "123", "123 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE2()
        {
            this.SetValueAndAssertAppliedFormat("€-123", TestCurrencyWholeNumberFormatDeDE, "-123", "-123 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE3()
        {
            this.SetValueAndAssertAppliedFormat("€123{ds}456", TestCurrencyDecimalNumberFormatDeDE, "123{ds}456", "123{ds}46 €", cultures: DeDECulture);
        }
       
        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE4()
        {
            this.SetValueAndAssertAppliedFormat("123€", TestCurrencyWholeNumberFormatDeDE, "123", "123 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE5()
        {
            this.SetValueAndAssertAppliedFormat("-123€", TestCurrencyWholeNumberFormatDeDE, "-123", "-123 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE6()
        {
            this.SetValueAndAssertAppliedFormat("123{ds}456€", TestCurrencyDecimalNumberFormatDeDE, "123{ds}456", "123{ds}46 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE7()
        {
            this.SetValueAndAssertAppliedFormat("123{gs}456€", TestCurrencyWholeNumberFormatDeDE, "123456", "123{gs}456 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberDeDE8()
        {
            this.SetValueAndAssertAppliedFormat("123{gs}456 €", TestCurrencyWholeNumberFormatDeDE, "123456", "123{gs}456 €", cultures: DeDECulture);
        }

        #endregion


        #region Set Value and Assert Currency Format in it-IT

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT1()
        {
            this.SetValueAndAssertAppliedFormat("€123", TestCurrencyWholeNumberFormatItIT, "123", "€ 123", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT2()
        {
            this.SetValueAndAssertAppliedFormat("€-123", TestCurrencyWholeNumberFormatItIT, "-123", "-€ 123", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT3()
        {
            this.SetValueAndAssertAppliedFormat("€123{ds}456", TestCurrencyDecimalNumberFormatItIT, "123{ds}456", "€ 123{ds}46", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT4()
        {
            this.SetValueAndAssertAppliedFormat("123€", TestCurrencyWholeNumberFormatItIT, "123", "€ 123", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT5()
        {
            this.SetValueAndAssertAppliedFormat("123{gs}456 €", TestCurrencyWholeNumberFormatItIT, "123456", "€ 123{gs}456", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT7()
        {
            this.SetValueAndAssertAppliedFormat("€123{gs}456", TestCurrencyWholeNumberFormatItIT, "123456", "€ 123{gs}456", cultures: ItITCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberItIT6()
        {
            this.SetValueAndAssertAppliedFormat("€ 123{gs}456", TestCurrencyWholeNumberFormatItIT, "123456", "€ 123{gs}456", cultures: ItITCulture);
        }

        #endregion


        #region Set Value and Assert Currency Format in bg-BG

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberBgBG()
        {
            this.SetValueAndAssertAppliedFormat("123лв.", TestCurrencyWholeNumberFormatBgBG, "123", "123 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatZeroBgBG()
        {
            this.SetValueAndAssertAppliedFormat("0лв.", TestCurrencyWholeNumberFormatBgBG, "0", "0 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatDecimalNumberBgBG()
        {
            this.SetValueAndAssertAppliedFormat("0{ds}1567лв.", TestCurrencyDecimalNumberFormatBgBG, "0{ds}1567", "0{ds}16 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberBgBG2()
        {
            this.SetValueAndAssertAppliedFormat("1{gs}234лв.", TestCurrencyWholeNumberFormatBgBG, "1234", "1{gs}234 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberBgBG3()
        {
            this.SetValueAndAssertAppliedFormat("1{gs}234{ds}567 лв.", TestCurrencyDecimalNumberFormatBgBG, "1234{ds}567", "1{gs}234{ds}57 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberBgBG4()
        {
            this.SetValueAndAssertAppliedFormat("1{gs}234{gs}567{ds}567 лв.", TestCurrencyDecimalNumberFormatBgBG,
                "1234567{ds}567", "1{gs}234{gs}567{ds}57 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberBgBG5()
        {
            this.SetValueAndAssertAppliedFormat("-1{gs}234{ds}567 лв.", TestCurrencyDecimalNumberFormatBgBG, "-1234{ds}567",
                "-1{gs}234{ds}57 лв.", cultures: BgBGCulture);
        }

        #endregion


        #region Set Value and Assert Currency Format in es-VE

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 123";
            string testExpectedValueString = "{cs} 123";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyWholeNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "123", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatZeroEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 0";
            string testExpectedValueString = "{cs} 0";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyWholeNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "0", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatDecimalNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 0{ds}1567";
            string testExpectedValueString = "{cs} 0{ds}16";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyDecimalNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "0{ds}1567", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEsVE2()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 1{gs}234";
            string testExpectedValueString = "{cs} 1{gs}234";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyWholeNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "1234", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEsVE3()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 1{gs}234{ds}567";
            string testExpectedValueString = "{cs} 1{gs}234{ds}57";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyDecimalNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "1234{ds}567", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEsVE4()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} 1{gs}234{gs}567{ds}567";
            string testExpectedValueString = "{cs} 1{gs}234{gs}567{ds}57";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyDecimalNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat,
                "1234567{ds}567", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetValueThatRequiresCurrencyFormatWholeNumberEsVE5()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = "{cs} -1{gs}234{ds}567";
            string testExpectedValueString = "{cs} -1{gs}234{ds}57";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testFormatString, currencySymbol));
            string testCurrencyFormat = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(TestCurrencyDecimalNumberFormatEsVE, currencySymbol));
            string expectedValueString = this.ConvertCurrencyForWindows10(this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol));

            this.SetValueAndAssertAppliedFormat(formatString, testCurrencyFormat, "-1234{ds}567",
                expectedValueString, cultures: EsVECulture);
        }

        private string ConvertCurrencyForWindows10(string value)
        {
            string cultureName = EsVECulture[0];
            CultureInfo culture = new CultureInfo(cultureName);

            if (culture.NumberFormat.CurrencySymbol.Equals("Bs."))
            {
                return value.Replace(" ", string.Empty);
            }

            return value;
        }

        #endregion


        #region Set Value & Format and Assert Currency Format Applied Correctly in en-US

        [TestMethod]
        public void SetWholeNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("96731", TestCurrencyWholeNumberFormatEnUS, TestCurrencyWholeNumberFormatEnUS, 
                "96731", "$96{gs}731)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetZeroThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("0", TestCurrencyWholeNumberFormatEnUS, TestCurrencyWholeNumberFormatEnUS,
                "0", "$0)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("-96731", TestCurrencyWholeNumberFormatEnUS, TestCurrencyWholeNumberFormatEnUS,
                "-96731", "($96{gs}731)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetDecimalNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("{ds}64729", TestCurrencyDecimalNumberFormatEnUS, TestCurrencyDecimalNumberFormatEnUS,
                "0{ds}64729", "$0{ds}65)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetDecimalNumberThenCurrencyFormatContainingEscapedCurrencySymbolEnUS1()
        {
            SetValueAndFormatAssertResult("{ds}64729", "\\$#,##0.00", "\\$#,##0.00",
                "0{ds}64729", "$0{ds}65", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetDecimalNumberThenCurrencyFormatContainingEscapedCurrencySymbolEnUS2()
        {
            SetValueAndFormatAssertResult("{ds}64729", "\\\\\\$#,##0.00", "\\\\\\$#,##0.00",
                "0{ds}64729", "\\$0{ds}65", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetWholeNumberThenCurrencyFormatContainingEscapedCurrencySymbolEnUS1()
        {
            SetValueAndFormatAssertResult("64729", "\\$#,##0.00", "\\$#,##0.00",
                "64729", "$64{gs}729{ds}00", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeDecimalNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("-{ds}64729", TestCurrencyDecimalNumberFormatEnUS, TestCurrencyDecimalNumberFormatEnUS,
                "-0{ds}64729", "($0{ds}65)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetFractionNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("0 1/2", TestCurrencyDecimalNumberFormatEnUS, TestCurrencyDecimalNumberFormatEnUS,
                "0{ds}5", "$0{ds}50)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetNegativeFractionNumberThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("-0 1/2", TestCurrencyDecimalNumberFormatEnUS, TestCurrencyDecimalNumberFormatEnUS,
                "-0{ds}5", "($0{ds}50)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetPercentThenCurrencyFormatEnUS()
        {
            SetValueAndFormatAssertResult("10%", TestCurrencyWholeNumberFormatEnUS, TestCurrencyWholeNumberFormatEnUS,
                "0{ds}1", "$0)", cultures: EnUSCulture);
        }

        #endregion


        #region Set Value & Format and Assert Currency Format Applied Correctly in de-DE

        [TestMethod]
        public void SetWholeNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("96731", TestCurrencyWholeNumberFormatDeDE, TestCurrencyWholeNumberFormatDeDE,
                "96731", "96{gs}731 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetZeroThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("0", TestCurrencyWholeNumberFormatDeDE, TestCurrencyWholeNumberFormatDeDE,
                "0", "0 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("-96731", TestCurrencyWholeNumberFormatDeDE, TestCurrencyWholeNumberFormatDeDE,
                "-96731", "-96{gs}731 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetDecimalNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("{ds}64729", TestCurrencyDecimalNumberFormatDeDE, TestCurrencyDecimalNumberFormatDeDE,
                "0{ds}64729", "0{ds}65 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetNegativeDecimalNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("-{ds}64729", TestCurrencyDecimalNumberFormatDeDE, TestCurrencyDecimalNumberFormatDeDE,
                "-0{ds}64729", "-0{ds}65 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetFractionNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("0 1/2", TestCurrencyDecimalNumberFormatDeDE, TestCurrencyDecimalNumberFormatDeDE,
                "0{ds}5", "0{ds}50 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetNegativeFractionNumberThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("-0 1/2", TestCurrencyDecimalNumberFormatDeDE, TestCurrencyDecimalNumberFormatDeDE,
                "-0{ds}5", "-0{ds}50 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetPercentThenCurrencyFormatDeDE()
        {
            SetValueAndFormatAssertResult("10%", TestCurrencyWholeNumberFormatDeDE, TestCurrencyWholeNumberFormatDeDE,
                "0{ds}1", "0 €", cultures: DeDECulture);
        }

        #endregion


        #region Set Value & Format and Assert Currency Format Applied Correctly in bg-BG

        [TestMethod]
        public void SetWholeNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("96731", TestCurrencyWholeNumberFormatBgBG, TestCurrencyWholeNumberFormatBgBG,
                "96731", "96{gs}731 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetZeroThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("0", TestCurrencyWholeNumberFormatBgBG, TestCurrencyWholeNumberFormatBgBG,
                "0", "0 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetNegativeWholeNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("-96731", TestCurrencyWholeNumberFormatBgBG, TestCurrencyWholeNumberFormatBgBG,
                "-96731", "-96{gs}731 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetDecimalNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("{ds}64729", TestCurrencyDecimalNumberFormatBgBG, TestCurrencyDecimalNumberFormatBgBG,
                "0{ds}64729", "0{ds}65 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetNegativeDecimalNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("-{ds}64729", TestCurrencyDecimalNumberFormatBgBG, TestCurrencyDecimalNumberFormatBgBG,
                "-0{ds}64729", "-0{ds}65 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetFractionNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("0 1/2", TestCurrencyDecimalNumberFormatBgBG, TestCurrencyDecimalNumberFormatBgBG,
                "0{ds}5", "0{ds}50 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetNegativeFractionNumberThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("-0 1/2", TestCurrencyDecimalNumberFormatBgBG, TestCurrencyDecimalNumberFormatBgBG,
                "-0{ds}5", "-0{ds}50 лв.", cultures: BgBGCulture);
        }

        [TestMethod]
        public void SetPercentThenCurrencyFormatBgBG()
        {
            SetValueAndFormatAssertResult("10%", TestCurrencyWholeNumberFormatBgBG, TestCurrencyWholeNumberFormatBgBG,
                "0{ds}1", "0 лв.", cultures: BgBGCulture);
        }

        #endregion


        #region Set Format & Value and Assert Currency Format Applied Correctly in en-US

        [TestMethod]
        public void SetCurrencyFormatThenWholeNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatEnUS, "96731", TestCurrencyWholeNumberFormatEnUS,
                "96731", "$96{gs}731)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenZeroNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatEnUS, "0", TestCurrencyWholeNumberFormatEnUS,
                "0", "$0)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeWholeNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatEnUS, "-96731", TestCurrencyWholeNumberFormatEnUS,
                "-96731", "($96{gs}731)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenDecimalNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatEnUS, "{ds}64729", TestCurrencyDecimalNumberFormatEnUS,
                "0{ds}64729", "$0{ds}65)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeDecimalNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatEnUS, "-{ds}64729", TestCurrencyDecimalNumberFormatEnUS,
                "-0{ds}64729", "($0{ds}65)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFractionNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatEnUS, "0 1/2", TestCurrencyDecimalNumberFormatEnUS,
                "0{ds}5", "$0{ds}50)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeFractionNumberEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatEnUS, "-0 1/2", TestCurrencyDecimalNumberFormatEnUS,
                "-0{ds}5", "($0{ds}50)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenPercentEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatEnUS, "10%", TestCurrencyWholeNumberFormatEnUS,
                "0{ds}1", "$0)", cultures: EnUSCulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFormulaEnUS()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatEnUS, "=ABS(-10)", TestCurrencyWholeNumberFormatEnUS,
                "=ABS(-10)", "$10)", cultures: EnUSCulture, expectedType: typeof(FormulaCellValue));
        }

        [TestMethod]
        public void SetCurrencyFormatThenNumberValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestCurrencyDecimalNumberFormatEnUS, "1{gs}234", TestCurrencyDecimalNumberFormatEnUS, "1234", "$1{gs}234{ds}00)",
                "#{gs}##0", "1234", "1{gs}234", expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetCurrencyFormatThenDateValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestCurrencyDecimalNumberFormatEnUS, "10/6/1987", TestCurrencyDecimalNumberFormatEnUS, "32056", "$32{gs}056{ds}00)",
                "m/d/yyyy", "10/6/1987", "10/6/1987", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetCurrencyFormatThenTimeValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestCurrencyDecimalNumberFormatEnUS, "12:30", TestCurrencyDecimalNumberFormatEnUS, "0{ds}520833333333333", "$0{ds}52)",
                "h:mm", "12:30:00 PM", "12:30", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetCurrencyFormatThenPercentageValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestCurrencyDecimalNumberFormatEnUS, "15%", TestCurrencyDecimalNumberFormatEnUS, "0{ds}15", "$0{ds}15)",
                "0%", "15%", "15%", expectedType: typeof(NumberCellValue));
        }

        #endregion


        #region Set Format & Value and Assert Currency Format Applied Correctly in de-DE

        [TestMethod]
        public void SetCurrencyFormatThenWholeNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatDeDE, "96731", TestCurrencyWholeNumberFormatDeDE,
                "96731", "96{gs}731 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenZeroNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatDeDE, "0", TestCurrencyWholeNumberFormatDeDE,
                "0", "0 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeWholeNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatDeDE, "-96731", TestCurrencyWholeNumberFormatDeDE,
                "-96731", "-96{gs}731 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenDecimalNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatDeDE, "{ds}64729", TestCurrencyDecimalNumberFormatDeDE,
                "0{ds}64729", "0{ds}65 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeDecimalNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatDeDE, "-{ds}64729", TestCurrencyDecimalNumberFormatDeDE,
                "-0{ds}64729", "-0{ds}65 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFractionNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatDeDE, "0 1/2", TestCurrencyDecimalNumberFormatDeDE,
                "0{ds}5", "0{ds}50 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeFractionNumberDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyDecimalNumberFormatDeDE, "-0 1/2", TestCurrencyDecimalNumberFormatDeDE,
                "-0{ds}5", "-0{ds}50 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenPercentDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatDeDE, "10%", TestCurrencyWholeNumberFormatDeDE,
                "0{ds}1", "0 €", cultures: DeDECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFormulaDeDE()
        {
            SetFormatAndValueAssertResult(TestCurrencyWholeNumberFormatDeDE, "=ABS(-10)", TestCurrencyWholeNumberFormatDeDE,
                "=ABS(-10)", "10 €", cultures: DeDECulture, expectedType: typeof(FormulaCellValue));
        }

        #endregion


        #region Set Format & Value and Assert Currency Format Applied Correctly in es-VE

        [TestMethod]
        public void SetCurrencyFormatThenWholeNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyWholeNumberFormatEsVE;
            string testExpectedValueString = "{cs} 96{gs}731";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "96731", formatString,
                "96731", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenZeroNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyWholeNumberFormatEsVE;
            string testExpectedValueString = "{cs} 0";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "0", formatString,
                "0", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeWholeNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyWholeNumberFormatEsVE;
            string testExpectedValueString = "{cs} -96{gs}731";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "-96731", formatString,
                "-96731", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenDecimalNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyDecimalNumberFormatEsVE;
            string testExpectedValueString = "{cs} 0{ds}65";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "{ds}64729", formatString,
                "0{ds}64729", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeDecimalNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyDecimalNumberFormatEsVE;
            string testExpectedValueString = "{cs} -0{ds}65";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "-{ds}64729", formatString,
                "-0{ds}64729", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFractionNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyDecimalNumberFormatEsVE;
            string testExpectedValueString = "{cs} 0{ds}50";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "0 1/2", formatString,
                "0{ds}5", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenNegativeFractionNumberEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyDecimalNumberFormatEsVE;
            string testExpectedValueString = "{cs} -0{ds}50";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "-0 1/2", formatString,
                "-0{ds}5", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenPercentEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyDecimalNumberFormatEsVE;
            string testExpectedValueString = "{cs} 0{ds}10";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "10%", formatString,
                "0{ds}1", expectedValueString, cultures: EsVECulture);
        }

        [TestMethod]
        public void SetCurrencyFormatThenFormulaEsVE()
        {
            string cultureName = EsVECulture[0];
            string testFormatString = TestCurrencyWholeNumberFormatEsVE;
            string testExpectedValueString = "{cs} 10";

            CultureInfo culture = new CultureInfo(cultureName);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;

            string formatString = this.ReplaceCurrencySymbol(testFormatString, currencySymbol);

            string expectedValueString = this.ReplaceCurrencySymbol(testExpectedValueString, currencySymbol);

            SetFormatAndValueAssertResult(formatString, "=ABS(-10)", formatString,
                "=ABS(-10)", expectedValueString, cultures: EsVECulture, expectedType: typeof(FormulaCellValue));
        }

        #endregion


        #region Set Format and Assert Category is Correct

        [TestMethod]
        public void SetFormatSimplePositiveEnUS()
        {
            SetFormatAndAssertCategory("$#,##0.00", FormatStringCategory.Currency, EnUSCulture);
        }

        [TestMethod]
        public void SetFormatSimplePositiveBgBG()
        {
            SetFormatAndAssertCategory("# ##0,00 лв.", FormatStringCategory.Currency, BgBGCulture);
        }

        [TestMethod]
        public void SetFormatSimplePositiveDeDE()
        {
            SetFormatAndAssertCategory("#.##0,00 €", FormatStringCategory.Currency, DeDECulture);
        }

        [TestMethod]
        public void SetFormatSimplePositiveItIT()
        {
            SetFormatAndAssertCategory("€ #.##0,00", FormatStringCategory.Currency, ItITCulture);
        }

        [TestMethod]
        public void SetFormatSimplePositiveEsES()
        {
            SetFormatAndAssertCategory("#.##0,00 €", FormatStringCategory.Currency, EsESCulture);
        }

        [TestMethod]
        public void SetFormatSimplePositiveEsVE()
        {
            CultureInfo culture = new CultureInfo(EsVECulture[0]);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;
            string formatString = this.ConvertCurrencyForWindows10("{cs} #.##0,00".Replace("{cs}", currencySymbol));
            SetFormatAndAssertCategory(formatString, FormatStringCategory.Currency, EsVECulture);
        }

        //TODO: This test passes locally and fails on the build machine.
        //See TeamPulse item 97040 Some CellValueFormat tests for pa-in culture fail only on the build machine and pass locally.
        //[TestMethod]
        //public void SetFormatSimplePositivePaIN()
        //{
        //    SetFormatAndAssertCategory("₹ #,##0.00", FormatStringCategory.Currency, PaINCulture);
        //}

        [TestMethod]
        public void SetFormatNegativeRedEnUS()
        {
            SetFormatAndAssertCategory("$#,##0.00_);[Red]($#,##0.00)", FormatStringCategory.Currency, EnUSCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedBgBG()
        {
            SetFormatAndAssertCategory("# ##0,00 \\л\\в\\.;[Red]-# ##0,00 \\л\\в\\.", FormatStringCategory.Currency, BgBGCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedDeDE()
        {
            SetFormatAndAssertCategory("#.##0,00 €;[Red]-#.##0,00 €", FormatStringCategory.Currency, DeDECulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedItIT()
        {
            SetFormatAndAssertCategory("€ #.##0,00;[Red]-€ #.##0,00", FormatStringCategory.Currency, ItITCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedEsES()
        {
            SetFormatAndAssertCategory("#.##0,00 €;[Red]-#.##0,00 €", FormatStringCategory.Currency, EsESCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedEsVe()
        {
            CultureInfo culture = new CultureInfo(EsVECulture[0]);
            string currencySymbol = culture.NumberFormat.CurrencySymbol;
            string formatString = this.ConvertCurrencyForWindows10("{cs} #.##0,00;[Red]{cs} -#.##0,00".Replace("{cs}", currencySymbol));
            SetFormatAndAssertCategory(formatString, FormatStringCategory.Currency, EsVECulture);
        }

        //TODO: This test passes locally and fails on the build machine.
        //See TeamPulse item 97040 Some CellValueFormat tests for pa-in culture fail only on the build machine and pass locally.
        //[TestMethod]
        //public void SetFormatNegativeRedPaIN()
        //{
        //    SetFormatAndAssertCategory("₹ #,##0.00;[Red]₹ -#,##0.00", FormatStringCategory.Currency, PaINCulture);
        //}

        #endregion


        #region ResultItem Tests

        [TestMethod]
        public void CurrencyTest1Positive()
        {
            RunTestInMultipleCultures(() =>
                {
                    Worksheet worksheet = TestHelper.CreateWorksheet();

                    worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("${0}_);[Red](${0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
                    worksheet.Cells[0, 0].SetValue(NumberToString2);

                    ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                    Assert.IsTrue(cellValue is NumberCellValue);

                    CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
                    CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
                    IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                    string infosText = cellValueFormatResult.InfosText;
                    bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                    Assert.AreEqual(2, infos.Count());
                    Assert.AreEqual(null, cellValueFormatResult.Foreground);
                    Assert.AreEqual(null, cellValueFormatResult.Condition);
                    Assert.AreEqual(null, cellValueFormatResult.Culture);
                    Assert.AreEqual(false, shouldExpand);
                    Assert.AreEqual(string.Format("${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

                    int i = 0;
                    foreach (CellValueFormatResultItem info in infos)
                    {
                        if (i == 0)
                        {
                            Assert.AreEqual(true, info.ApplyFormat);
                            Assert.AreEqual(false, info.IsTransparent);
                            Assert.AreEqual(false, info.ShouldExpand);
                            Assert.AreEqual(string.Format("${0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                        }

                        if (i == 1)
                        {
                            Assert.AreEqual(true, info.ApplyFormat);
                            Assert.AreEqual(true, info.IsTransparent);
                            Assert.AreEqual(false, info.ShouldExpand);
                            Assert.AreEqual(")", info.Text);
                        }
                        i++;
                    }
                }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest1Negative()
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();

                worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("${0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
                worksheet.Cells[0, 0].SetValue("-" + NumberToString2);

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.IsTrue(cellValue is NumberCellValue);
                CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
                CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
                IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                string infosText = cellValueFormatResult.InfosText;
                bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;
                Assert.AreEqual(1, infos.Count());

                Assert.AreEqual(null, cellValueFormatResult.Foreground);
                Assert.AreEqual(null, cellValueFormatResult.Condition);
                Assert.AreEqual(null, cellValueFormatResult.Culture);
                Assert.AreEqual(false, shouldExpand);
                Assert.AreEqual("-$" + TestNumberToString.ReplaceGroupDecimalSeparators(), infosText);

                int i = 0;
                foreach (CellValueFormatResultItem info in infos)
                {
                    if (i == 0)
                    {
                        Assert.AreEqual(true, info.ApplyFormat);
                        Assert.AreEqual(false, info.IsTransparent);
                        Assert.AreEqual(false, info.ShouldExpand);
                        Assert.AreEqual("-$" + TestNumberToString.ReplaceGroupDecimalSeparators(), info.Text);
                    }
                }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest1NegativeRed()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("${0}_);[Red](${0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue("-" + NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);

            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("(${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("(${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
                        }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Chinese_Singapore_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$-1004]{0}_);([$$-1004]{0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);

            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(2, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual("$" + TestNumberToString.ReplaceGroupDecimalSeparators(), info.Text);
                }

                if (i == 1)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(true, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(")", info.Text);
                }
                i++;
            }
                }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Chinese_Singapore_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$-1004]{0}_);([$$-1004]{0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("(${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("(${0})", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Malay_BruneiDarussalam_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$-83E]{0};[Red][$$-83E]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("${0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("${0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Malay_BruneiDarussalam_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$-83E]{0};[Red][$$-83E]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("${0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("${0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Quechua_Bolivia_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$b-46B] {0}_);[Red]([$$b-46B] {0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;
            Assert.AreEqual(2, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("$b {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("$b {0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }

                if (i == 1)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(true, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(")", info.Text);
                }
                i++;
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Quechua_Bolivia_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$$b-46B] {0}_);[Red]([$$b-46B] {0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("($b {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("($b {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Welsh_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$£-452]{0};[Red]-[$£-452]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("£{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("£{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Welsh_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$£-452]{0};[Red]-[$£-452]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("-£{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("-£{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Tibetan_PRC_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$¥-451]{0};[$¥-451]-{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("¥{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("¥{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Tibetan_PRC_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$¥-451]{0};[$¥-451]-{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat cellValueFormat = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = cellValueFormat.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("¥-{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("¥-{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Euro_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$€-2] {0}_);[Red]([$€-2] {0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(2, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("€ {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("€ {0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }

                if (i == 1)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(true, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(")", info.Text);
                }
                i++;
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Euro_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$€-2] {0}_);[Red]([$€-2] {0})", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("(€ {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("(€ {0})", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Euro2_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("{0} [$€-1]_);[Red]({0} [$€-1])", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(2, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("{0} €)", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("{0} €", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }

                if (i == 1)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(true, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(")", info.Text);
                }
                i++;
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Euro2_Negative()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("{0} [$€-1]_);[Red]({0} [$€-1])", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("({0} €)", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("({0} €)", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Thai_Positive()
        {
             RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$฿-41E]{0};[Red]-[$฿-41E]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(NumberToString2);

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(null, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("฿{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("฿{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }

        [TestMethod]
        public void CurrencyTest_Thai_Negative()
        {
            RunTestInMultipleCultures(() =>
            {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat(string.Format("[$฿-41E]{0};[Red]-[$฿-41E]{0}", TestNumberFormat.ReplaceGroupDecimalSeparators())));
            worksheet.Cells[0, 0].SetValue(string.Format("-{0}", NumberToString2));

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

            Assert.IsTrue(cellValue is NumberCellValue);
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
            CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
            IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
            string infosText = cellValueFormatResult.InfosText;
            bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

            Assert.AreEqual(1, infos.Count());
            Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
            Assert.AreEqual(null, cellValueFormatResult.Condition);
            Assert.AreEqual(null, cellValueFormatResult.Culture);
            Assert.AreEqual(false, shouldExpand);
            Assert.AreEqual(string.Format("-฿{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), infosText);

            int i = 0;
            foreach (CellValueFormatResultItem info in infos)
            {
                if (i == 0)
                {
                    Assert.AreEqual(true, info.ApplyFormat);
                    Assert.AreEqual(false, info.IsTransparent);
                    Assert.AreEqual(false, info.ShouldExpand);
                    Assert.AreEqual(string.Format("-฿{0}", TestNumberToString.ReplaceGroupDecimalSeparators()), info.Text);
                }
            }
            }, EnUSCulture);
        }
        
        [TestMethod]
        public void CurrencyTest_Dari_Afghanistan_Positive()
        {
            this.TestCurrency(true, string.Format("[$؋-48C]{0}_-;[Red][$؋-48C]{0}-", TestNumberFormat), 3, null,
                string.Format("؋{0}-", TestNumberToString), "؋", false, false, TestNumberToString, false, true, "-", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Dari_Afghanistan_Negative()
        {
            this.TestCurrency(false, string.Format("[$؋-48C]{0}_-;[Red][$؋-48C]{0}-", TestNumberFormat), 2, Colors.Red,
                string.Format("؋{0}-", TestNumberToString), "؋", false, false, string.Format("{0}-", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Hebrew_Positive()
        {
            this.TestCurrency(true, string.Format("[$₪-40D] {0};[Red][$₪-40D] {0}", TestNumberFormat), 1, null,
                string.Format("₪ {0}", TestNumberToString),
                string.Format("₪ {0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Hebrew_Negative()
        {
            this.TestCurrency(false, string.Format("[$₪-40D] {0};[Red][$₪-40D] {0}", TestNumberFormat), 1, Colors.Red,
                string.Format("₪ {0}", TestNumberToString),
                string.Format("₪ {0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Spanish_Panama_Positive()
        {
            this.TestCurrency(true, string.Format("[$B/.-180A] {0}_);([$B/.-180A] {0})", TestNumberFormat), 3, null, 
                string.Format("B/. {0})", TestNumberToString),
                "B/.", false, false, string.Format(" {0}", TestNumberToString), false, true, ")", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Spanish_Panama_Negative()
        {
            this.TestCurrency(false, string.Format("[$B/.-180A] {0}_);([$B/.-180A] {0})", TestNumberFormat), 3, null,
                string.Format("(B/. {0})", TestNumberToString),
                "(", false, true, "B/.", false, false, string.Format(" {0})", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_English_Belize_Positive()
        {
            this.TestCurrency(true, string.Format("[$BZ$-2809]{0}_);[Red]([$BZ$-2809]{0})", TestNumberFormat), 2, null, 
                string.Format("BZ${0})", TestNumberToString),
                string.Format("BZ${0}", TestNumberToString), false, true,")", true, true);
        }

        [TestMethod]
        public void CurrencyTest_English_Belize_Negative()
        {
            this.TestCurrency(false, string.Format("[$BZ$-2809]{0}_);[Red]([$BZ$-2809]{0})", TestNumberFormat), 1, Colors.Red, 
                string.Format("(BZ${0})", TestNumberToString),
                 string.Format("(BZ${0})", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Italian_Switzerland_Positive()
        {
            this.TestCurrency(true,  string.Format("[$fr.-810] {0};[$fr.-810] -{0}", TestNumberFormat), 2, null, 
                string.Format("fr. {0}", TestNumberToString),
                "fr.", false, false,  string.Format(" {0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Italian_Switzerland_Negative()
        {
            this.TestCurrency(false,  string.Format("[$fr.-810] {0};[$fr.-810] -{0}", TestNumberFormat), 2, null, 
                string.Format("fr. -{0}", TestNumberToString),
                "fr.", false, false,  string.Format(" -{0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Chinese_MacaoSAR_Positive()
        {
            this.TestCurrency(true,  string.Format("[$MOP-1404]{0}_);([$MOP-1404]{0})", TestNumberFormat), 2, null, 
                string.Format("MOP{0})", TestNumberToString),
                 string.Format("MOP{0}", TestNumberToString), false, true, ")", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Chinese_MacaoSAR_Negative()
        {
            this.TestCurrency(false,  string.Format("[$MOP-1404]{0}_);([$MOP-1404]{0})", TestNumberFormat), 1, null, 
                string.Format("(MOP{0})", TestNumberToString),
                 string.Format("(MOP{0})", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Uzbek_Latin_Positive()
        {
            this.TestCurrency(true,  string.Format("{0} [$so'm-443];[Red]-{0} [$so'm-443]", TestNumberFormat), 2, null,
                string.Format("{0} so'm", TestNumberToString),
                 string.Format("{0} ", TestNumberToString), false, true, "so'm", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Uzbek_Latin_Negative()
        {
            this.TestCurrency(false,  string.Format("{0} [$so'm-443];[Red]-{0} [$so'm-443]", TestNumberFormat), 2, Colors.Red,  
                string.Format("-{0} so'm", TestNumberToString), 
                string.Format("-{0} ", TestNumberToString), false, true, "so'm", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Bulgarian_Positive()
        {
            this.TestCurrency(true, string.Format("{0} [$лв.-402]", TestNumberFormat), 2, null, string.Format("{0} лв.", TestNumberToString),
                 string.Format("{0} ", TestNumberToString), false, true, "лв.", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Bulgarian_Negative()
        {
            this.TestCurrency(false,  string.Format("{0} [$лв.-402]", TestNumberFormat), 2, null,  string.Format("-{0} лв.", TestNumberToString),
                 string.Format("-{0} ", TestNumberToString), false, true, "лв.", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Bashkir_Positive()
        {
            this.TestCurrency(true,  string.Format("{0} [$һ.-46D];[Red]{0} [$һ.-46D]", TestNumberFormat), 2, null,
                string.Format("{0} һ.", TestNumberToString),
                 string.Format("{0} ", TestNumberToString), false, true, "һ.", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Bashkir_Negative()
        {
            this.TestCurrency(false,  string.Format("{0} [$һ.-46D];[Red]{0} [$һ.-46D]", TestNumberFormat), 2, Colors.Red, 
                string.Format("{0} һ.", TestNumberToString),
                 string.Format("{0} ", TestNumberToString), false, true, "һ.", false, false);
        }

        [TestMethod]
        public void CurrencyTest_Arabic_Egypt_Positive()
        {
            this.TestCurrency(true,  string.Format("[$ج.م.‏-C01] {0}_-;[$ج.م.‏-C01] {0}-", TestNumberFormat), 3, null,  
                string.Format("ج.م.‏ {0}-", TestNumberToString), "ج.م.‏", false, false, string.Format(" {0}", TestNumberToString),
                false, true, "-", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Arabic_Egypt_Negative()
        {
            this.TestCurrency(false,  string.Format("[$ج.م.‏-C01] {0}_-;[$ج.م.‏-C01] {0}-", TestNumberFormat), 2, null, 
                string.Format("ج.م.‏ {0}-", TestNumberToString), "ج.م.‏", false, false,  string.Format(" {0}-", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Arabic_SaudiArabia_Positive()
        {
            this.TestCurrency(true,  string.Format("[$ر.س.‏-401] {0}_-;[Red][$ر.س.‏-401] {0}-", TestNumberFormat), 3, null, 
                 string.Format("ر.س.‏ {0}-", TestNumberToString), "ر.س.‏", false, false,  
                 string.Format(" {0}", TestNumberToString), false, true, "-", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Arabic_SaudiArabia_Negative()
        {
            this.TestCurrency(false,  string.Format("[$ر.س.‏-401] {0}_-;[Red][$ر.س.‏-401] {0}-", TestNumberFormat), 2, Colors.Red, 
                 string.Format("ر.س.‏ {0}-", TestNumberToString), "ر.س.‏", false, false,
                 string.Format(" {0}-", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Marathi_Positive()
        {
            this.TestCurrency(true,  string.Format("[$रु-44E] {0};[Red][$रु-44E] -{0}", TestNumberFormat), 1, null, 
                 string.Format("रु {0}", TestNumberToString),  string.Format("रु {0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Marathi_Negative()
        {
            this.TestCurrency(false,  string.Format("[$रु-44E] {0};[Red][$रु-44E] -{0}", TestNumberFormat), 1, Colors.Red, 
                 string.Format("रु -{0}", TestNumberToString),  string.Format("रु -{0}", TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_Sinhala_Positive()
        {
            this.TestCurrency(true,  string.Format("[$රු.-45B] {0}_);[Red]([$රු.-45B] {0})", TestNumberFormat), 3, null, 
                 string.Format("රු. {0})", TestNumberToString), "රු.", false, false, 
                 string.Format(" {0}", TestNumberToString), false, true, ")", true, true);
        }

        [TestMethod]
        public void CurrencyTest_Sinhala_Negative()
        {
            this.TestCurrency(false,  string.Format("[$රු.-45B] {0}_);[Red]([$රු.-45B] {0})", TestNumberFormat), 3, Colors.Red, 
                 string.Format("(රු. {0})", TestNumberToString), "(", false, true, "රු.", false, false, 
                 string.Format(" {0})", TestNumberToString), false, true);
        }

        #endregion


        #region Three Letters Tests

        public void CurrencyTest_3Letter(string letters)
        {
            this.CurrencyTest_3Letters_Positive(letters);
            this.CurrencyTest_3Letters_Negative(letters);
        }

        public void CurrencyTest_3Letters_Positive(string letters)
        {
            this.TestCurrency(true, string.Format("[${0}] {1}_);[Red]([${0}] {1})", letters, TestNumberFormat), 2, null,
                string.Format("{0} {1})", letters, TestNumberToString),
                string.Format("{0} {1}", letters, TestNumberToString), false, true, ")", true, true);
        }

        public void CurrencyTest_3Letters_Negative(string letters)
        {
            this.TestCurrency(false, string.Format("[${0}] {1}_);[Red]([${0}] {1})", letters, TestNumberFormat), 1, Colors.Red,
                string.Format("({0} {1})", letters, TestNumberToString),
                string.Format("({0} {1})", letters, TestNumberToString), false, true);
        }

        [TestMethod]
        public void CurrencyTest_All_3Letters()
        {
            string[] threeLetterCurrencies = new string[] { "ADP", "AED", "AFA", "AFN", "ALL", "AMD", "ANG", "AOA", "AOK", "AON", "ARS", "ATS", "AUD", "AWG", "AZM", "AZN", "AZS", "BAD", "BAM", "BBD", "BDT", "BEF", "BES", "BGL", "BGN", "BHD", "BIF", "BMD", "BND", "BOB", "BOP", "BOV", "BPS", "BRL", "BSD", "BTN", "BWP", "BYB", "BYR", "BZD", "CAD", "CDF", "CFP", "CHF", "CKI", "CLF", "CLP", "CNH", "CNY", "COP", "COU", "CRC", "CSD", "CUP", "CVE", "CYP", "CZK", "DEM", "DJF", "DKK", "DOP", "DZD", "ECS", "EEK", "EGP", "ERN", "ESP", "ETB", "EUR", "FIM", "FJD", "FKP", "FRF", "GBP", "GEL", "GES", "GGY", "GHC", "GHS", "GIP", "GMD", "GNF", "GRD", "GTQ", "GWP", "GYD", "HKD", "HNL", "HRK", "HTG", "HUF", "IDR", "IEP", "ILS", "INR", "IQD", "IRR", "ISK", "ITL", "JMD", "JOD", "JPY", "KES", "KGS", "KHR", "KMF", "KPW", "KRW", "KTS", "KWD", "KYD", "KZT", "LAK", "LBP", "LKR", "LRD", "LSL", "LTL", "LTT", "LUF", "LVL", "LVR", "LYD", "MAD", "MDL", "MGA", "MGF", "MKD", "MMK", "MNT", "MOP", "MRO", "MTL", "MUR", "MVR", "MVS", "MWK", "MXN", "MXP", "MXV", "MYR", "MZM", "MZN", "NAD", "NGN", "NIC", "NIO", "NLG", "NOK", "NPR", "NZD", "OMR", "PAB", "PEN", "PGK", "PHP", "PKR", "PLN", "PLZ", "PTE", "PYG", "QAR", "ROL", "RON", "RSD", "RUB", "RUR", "RWF", "SAR", "SBD", "SCR", "SDD", "SDG", "SEK", "SGD", "SHP", "SIT", "SKK", "SLL", "SOS", "SRD", "SRG", "SSP", "STD", "SVC", "SYP", "SZL", "THB", "TJR", "TJS", "TMM", "TND", "TOP", "TPE", "TRL", "TRS", "TRY", "TTD", "TUR", "TWD", "TZS", "UAH", "UAK", "UGX", "UNR", "USD", "UYP", "UYU", "UZS", "VEB", "VEF", "VND", "VUV", "WST", "XAF", "XAG", "XAU", "XBA", "XBB", "XBC", "XBD", "XCD", "XDR", "XEU", "XFO", "XFU", "XOF", "XPD", "XPF", "XPT", "XTS", "XXX", "YER", "YUM", "YUN", "YUX", "ZAR", "ZMK", "ZRN", "ZRZ", "ZWD", "ZWL" };

            for (int i = 0; i < threeLetterCurrencies.Length; i++)
            {
                this.CurrencyTest_3Letter(threeLetterCurrencies[i]);
            }
        }

        #endregion
    }
}
