﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Date : CellValueFormatTestsBase
    {
        #region Fields

        public static readonly string DateFormatEnUS1 = "m/d/yyyy";
        public static readonly string DateFormatEnUS2 = "d-mmm-yy";
        public static readonly string DateFormatEnUS3 = "d-mmm";
        public static readonly string DateTimeFormatEnUS1 = "m/d/yyyy h:mm";
        public static readonly string DateFormat4 = "mmm-yy";

        public static readonly string DateFormatEsES1 = "dd/mm/yyyy";
        public static readonly string DateFormatEsES2 = "dd-mmm-yy";
        public static readonly string DateFormatEsES3 = "dd-mmm";
        public static readonly string DateTimeFormatEsES1 = "dd/mm/yyyy h:mm";

        public static readonly string DateFormatDeDE1 = "dd.mm.yyyy";
        public static readonly string DateFormatDeDE2 = "dd. mmm yy";
        public static readonly string DateFormatDeDE3 = "dd. mmm";
        public static readonly string DateFormatDeDE4 = "mmm yy";

        public static readonly string DateFormatBgBG1 = "d.m.yyyy 'г.'";

        #endregion


        #region Set Value and Assert Date Format in es-US

        [TestMethod]
        public void SimpleDateTestEnUS1()
        {
            SetValueAndAssertAppliedFormat("6/10/1987", DateFormatEnUS1, "6/10/1987", "6/10/1987", EnUSCulture);
        }

        // TODO: See TeamPulse item 154274 The string "6 June 2013" is not parsed in correct format.
        //[TestMethod]
        //public void SimpleDateTestEnUS2()
        //{
        //    SetValueAndAssertAppliedFormat("6 June 2013", DateFormatEnUS2, "6-Jun-13", "6/6/2013", EnUSCulture);
        //}

        [TestMethod]
        public void SimpleDateTestEnUS3()
        {
            SetValueAndAssertAppliedFormat("10/1987", DateFormat4, "10/1/1987", "Oct-87", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS4()
        {
            SetValueAndAssertAppliedFormat("2013/5/31", DateFormatEnUS1, "5/31/2013", "5/31/2013", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS5()
        {
            SetValueAndAssertAppliedFormat("9/10", DateFormatEnUS3, string.Format("9/10/{0}", DateTime.Now.Year), "10-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS6()
        {
            SetValueAndAssertAppliedFormat("15 May", DateFormatEnUS3, string.Format("5/15/{0}", DateTime.Now.Year), "15-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS7()
        {
            SetValueAndAssertAppliedFormat("May 2013", DateFormat4, "5/1/2013", "May-13", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS71()
        {
            SetValueAndAssertAppliedFormat("May/2013", DateFormat4, "5/1/2013", "May-13", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS8()
        {
            SetValueAndAssertAppliedFormat("May-2013", DateFormat4, "5/1/2013", "May-13", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS9()
        {
            SetValueAndAssertAppliedFormat("May-13", DateFormatEnUS3, string.Format("5/13/{0}", DateTime.Now.Year), "13-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS91()
        {
            SetValueAndAssertAppliedFormat("May 13", DateFormatEnUS3, string.Format("5/13/{0}", DateTime.Now.Year), "13-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS92()
        {
            SetValueAndAssertAppliedFormat("May/13", DateFormatEnUS3, string.Format("5/13/{0}", DateTime.Now.Year), "13-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS10()
        {
            SetValueAndAssertAppliedFormat("8-Sep", DateFormatEnUS3, string.Format("9/8/{0}", DateTime.Now.Year), "8-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS101()
        {
            SetValueAndAssertAppliedFormat("8/Sep", DateFormatEnUS3, string.Format("9/8/{0}", DateTime.Now.Year), "8-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS102()
        {
            SetValueAndAssertAppliedFormat("8 Sep", DateFormatEnUS3, string.Format("9/8/{0}", DateTime.Now.Year), "8-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTestEnUS12()
        {
            SetValueAndAssertAppliedFormat("8-September", DateFormatEnUS3, string.Format("9/8/{0}", DateTime.Now.Year), "8-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest23()
        {
            SetValueAndAssertAppliedFormat("31-Dec", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest26()
        {
            SetValueAndAssertAppliedFormat("15-Feb", DateFormatEnUS3, "2/15/" + DateTime.Now.Year.ToString(), "15-Feb", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest27()
        {
            SetValueAndAssertAppliedFormat("15-Mar", DateFormatEnUS3, "3/15/" + DateTime.Now.Year.ToString(), "15-Mar", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest28()
        {
            SetValueAndAssertAppliedFormat("15-Apr", DateFormatEnUS3, "4/15/" + DateTime.Now.Year.ToString(), "15-Apr", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest29()
        {
            SetValueAndAssertAppliedFormat("15-May", DateFormatEnUS3, "5/15/" + DateTime.Now.Year.ToString(), "15-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest311()
        {
            SetValueAndAssertAppliedFormat("15-Jun", DateFormatEnUS3, "6/15/" + DateTime.Now.Year.ToString(), "15-Jun", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest31()
        {
            SetValueAndAssertAppliedFormat("15-Jul", DateFormatEnUS3, "7/15/" + DateTime.Now.Year.ToString(), "15-Jul", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest32()
        {
            SetValueAndAssertAppliedFormat("15-Aug", DateFormatEnUS3, "8/15/" + DateTime.Now.Year.ToString(), "15-Aug", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest33()
        {
            SetValueAndAssertAppliedFormat("15-Sep", DateFormatEnUS3, "9/15/" + DateTime.Now.Year.ToString(), "15-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest34()
        {
            SetValueAndAssertAppliedFormat("15-Oct", DateFormatEnUS3, "10/15/" + DateTime.Now.Year.ToString(), "15-Oct", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest35()
        {
            SetValueAndAssertAppliedFormat("15-Nov", DateFormatEnUS3, "11/15/" + DateTime.Now.Year.ToString(), "15-Nov", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest37()
        {
            SetValueAndAssertAppliedFormat("Dec-31", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest38()
        {
            SetValueAndAssertAppliedFormat("Dec-32", DateFormat4, "12/1/1932", "Dec-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest39()
        {
            SetValueAndAssertAppliedFormat("dec-31", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest40()
        {
            SetValueAndAssertAppliedFormat("Feb-28", DateFormatEnUS3, "2/28/" + DateTime.Now.Year.ToString(), "28-Feb", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest41()
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue("Feb-29");

                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.IsTrue(cellValue is NumberCellValue);

                if (DateTime.IsLeapYear(DateTime.Now.Year))
                {
                    Assert.AreEqual(DateFormatEnUS3, format.FormatString);
                }
                else
                {
                    Assert.AreEqual(DateFormat4, format.FormatString);
                }

                if (DateTime.IsLeapYear(DateTime.Now.Year))
                {
                    Assert.AreEqual(string.Format("2/29/{0}", DateTime.Now.Year), cellValue.GetValueAsString(format));
                    Assert.AreEqual("29-Feb", cellValue.GetResultValueAsString(format));
                }
                else
                {
                    Assert.AreEqual("2/1/1929", cellValue.GetValueAsString(format));
                    Assert.AreEqual("Feb-29", cellValue.GetResultValueAsString(format));
                }
            }, EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest42()
        {
            SetValueAndAssertAppliedFormat("Feb-30", DateFormat4, "2/1/1930", "Feb-30", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest43()
        {
            SetValueAndAssertAppliedFormat("13-mar-87", DateFormatEnUS2, "3/13/1987", "13-Mar-87", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest44()
        {
            SetValueAndAssertAppliedFormat("13-mar-1987", DateFormatEnUS2, "3/13/1987", "13-Mar-87", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest48()
        {
            SetValueAndAssertAppliedFormat("13-mar-0", DateFormatEnUS2, "3/13/2000", "13-Mar-00", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest49()
        {
            SetValueAndAssertAppliedFormat("13-march-0", DateFormatEnUS2, "3/13/2000", "13-Mar-00", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest50()
        {
            SetValueAndAssertAppliedFormat("31-december", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest52()
        {
            SetValueAndAssertAppliedFormat("15-february", DateFormatEnUS3, "2/15/" + DateTime.Now.Year.ToString(), "15-Feb", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest53()
        {
            SetValueAndAssertAppliedFormat("15-march", DateFormatEnUS3, "3/15/" + DateTime.Now.Year.ToString(), "15-Mar", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest54()
        {
            SetValueAndAssertAppliedFormat("15-april", DateFormatEnUS3, "4/15/" + DateTime.Now.Year.ToString(), "15-Apr", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest55()
        {
            SetValueAndAssertAppliedFormat("15-may", DateFormatEnUS3, "5/15/" + DateTime.Now.Year.ToString(), "15-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest56()
        {
            SetValueAndAssertAppliedFormat("15-July", DateFormatEnUS3, "7/15/" + DateTime.Now.Year.ToString(), "15-Jul", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest57()
        {
            SetValueAndAssertAppliedFormat("15-July", DateFormatEnUS3, "7/15/" + DateTime.Now.Year.ToString(), "15-Jul", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest58()
        {
            SetValueAndAssertAppliedFormat("15-septEmber", DateFormatEnUS3, "9/15/" + DateTime.Now.Year.ToString(), "15-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest59()
        {
            SetValueAndAssertAppliedFormat("15-october", DateFormatEnUS3, "10/15/" + DateTime.Now.Year.ToString(), "15-Oct", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest60()
        {
            SetValueAndAssertAppliedFormat("15-Nov", DateFormatEnUS3, "11/15/" + DateTime.Now.Year.ToString(), "15-Nov", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest61()
        {
            SetValueAndAssertAppliedFormat("december-31", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest62()
        {
            SetValueAndAssertAppliedFormat("december-32", DateFormat4, "12/1/1932", "Dec-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest64()
        {
            SetValueAndAssertAppliedFormat("february-28", DateFormatEnUS3, "2/28/" + DateTime.Now.Year.ToString(), "28-Feb", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest65()
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue("february-29");

                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.IsTrue(cellValue is NumberCellValue);

                if (DateTime.IsLeapYear(DateTime.Now.Year))
                {
                    Assert.AreEqual(DateFormatEnUS3, format.FormatString);
                }
                else
                {
                    Assert.AreEqual(DateFormat4, format.FormatString);
                }

                if (DateTime.IsLeapYear(DateTime.Now.Year))
                {
                    Assert.AreEqual(string.Format("2/29/{0}", DateTime.Now.Year), cellValue.GetValueAsString(format));
                    Assert.AreEqual("29-Feb", cellValue.GetResultValueAsString(format));
                }
                else
                {
                    Assert.AreEqual("2/1/1929", cellValue.GetValueAsString(format));
                    Assert.AreEqual("Feb-29", cellValue.GetResultValueAsString(format));
                }
            }, EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest66()
        {
            SetValueAndAssertAppliedFormat("february-30", DateFormat4, "2/1/1930", "Feb-30", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest67()
        {
            SetValueAndAssertAppliedFormat("13-march-87", DateFormatEnUS2, "3/13/1987", "13-Mar-87", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest68()
        {
            SetValueAndAssertAppliedFormat("13-march-1987", DateFormatEnUS2, "3/13/1987", "13-Mar-87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormatMonthJan()
        {
            SetValueAndAssertAppliedFormat("1/13/2007", DateFormatEnUS1, "1/13/2007", "1/13/2007", cultures: EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormatDayOne()
        {
            SetValueAndAssertAppliedFormat("2/1/2007", DateFormatEnUS1, "2/1/2007", "2/1/2007", cultures: EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormatYearIsCurrent()
        {
            RunTestInMultipleCultures(() =>
            {
                string currentYear = DateTime.Now.Year.ToString();

                string newValue = "02-" + currentYear;
                string expectedFormat = "mmm-yy";
                string expectedValueAsString = "2/1/" + currentYear;
                string expectedResultValueAsString = "Feb-" + currentYear.Substring(2);

                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue(newValue);

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
                CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;

                Assert.IsTrue(cellValue is NumberCellValue);
                Assert.IsTrue(format.Equals(new CellValueFormat(expectedFormat)));
                Assert.AreEqual(expectedValueAsString, cellValue.GetValueAsString(format));
                Assert.AreEqual(expectedResultValueAsString, cellValue.GetResultValueAsString(format));
            }, EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormat01Dash01()
        {
            SetValueAndAssertAppliedFormat("01-01", DateFormatEnUS3, "1/1/" + DateTime.Now.Year.ToString(), "1-Jan", cultures: EnUSCulture);
        }

        #endregion


        #region Set Value and Assert Date Format in es-US

        [TestMethod]
        public void SimpleDateTestEsES1()
        {
            SetValueAndAssertAppliedFormat("13/12/2013", DateFormatEsES1, "13/12/2013", "13/12/2013", EsESCulture);
        }

        [TestMethod]
        public void SimpleDateTestEsES2()
        {
            SetValueAndAssertAppliedFormat("15-4-2013", DateFormatEsES1, "15/04/2013", "15/04/2013", EsESCulture);
        }

        // TODO: See TeamPulse item 154173 String of type 15-4 results in 15/04/2015 instead of 15-abr in es-ES culture
        //[TestMethod]
        //public void SimpleDateTestEsES3()
        //{
        //    SetValueAndAssertAppliedFormat("15-4", DateFormatEsES3, "15/04/2013", "15-abr", EsESCulture);
        //}

        // TODO: See TeamPulse item 154173 String of type 15-4 results in 15/04/2015 instead of 15-abr in es-ES culture
        //[TestMethod]
        //public void SimpleDateTestEsES4()
        //{
        //    SetValueAndAssertAppliedFormat("3-15", DateFormat4, "mar-15", "01/03/2015", EsESCulture);
        //}

        // TODO: See TeamPulse item 154175 GetValueAsString returns unexpected results
        //[TestMethod]
        //public void SimpleDateTestEsES5()
        //{
        //    SetValueAndAssertAppliedFormat("mar 2015", DateFormat4, "mar-15", "01/03/2015", EsESCulture);
        //}

        [TestMethod]
        public void SimpleDateTestEsES6()
        {
            SetValueAndAssertAppliedFormat("2013/2/15", DateFormatEsES1, "15/02/2013", "15/02/2013", EsESCulture);
        }

        // TODO: See TeamPulse item 154177 Setting 10/9 in Spanish culture results in 9 October, instead of 10 September
        //[TestMethod]
        //public void SimpleDateTestEsES7()
        //{
        //    SetValueAndAssertAppliedFormat("10/9", DateFormatEsES3, "10-sep", string.Format("10/09/{0}", DateTime.Now.Year), EsESCulture);
        //}

        #endregion


        #region Set Value and Assert Date Format in de-DE

        [TestMethod]
        public void SimpleDateTestDeDE0()
        {
            SetValueAndAssertAppliedFormat("6.10.1987", DateFormatDeDE1, "06.10.1987", "06.10.1987", DeDECulture);
        }

        [TestMethod]
        public void SimpleDateTestDeDE1()
        {
            SetValueAndAssertAppliedFormat("6/10/1987", DateFormatDeDE1, "06.10.1987", "06.10.1987", DeDECulture);
        }

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE2()
        //{
        //    SetValueAndAssertAppliedFormat("6 Juni 2013", DateFormatDeDE2, "06.06.2013", "06. Jun 13", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE3()
        //{
        //    SetValueAndAssertAppliedFormat("10/1987", DateFormatDeDE4, "01.10.1987", "Okt 87", DeDECulture);
        //}

        [TestMethod]
        public void SimpleDateTestDeDE4()
        {
            SetValueAndAssertAppliedFormat("2013/5/31", DateFormatDeDE1, "31.05.2013", "31.05.2013", DeDECulture);
        }

        // TODO: See TeamPulse item 154177 Setting 10/9 in Spanish and German culture results in 9 October, instead of 10 September
        //[TestMethod]
        //public void SimpleDateTestDeDE5()
        //{
        //    SetValueAndAssertAppliedFormat("9/10", DateFormatDeDE3, string.Format("9.10.{0}", DateTime.Now.Year), "09. Okt", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE12()
        //{
        //    SetValueAndAssertAppliedFormat("15 Mai", DateFormatDeDE3, string.Format("15.05.{0}", DateTime.Now.Year), "15. Mai", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE6()
        //{
        //    SetValueAndAssertAppliedFormat("Mai 2013", DateFormatDeDE4, "01.05.2013", "Mai 13", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE7()
        //{
        //    SetValueAndAssertAppliedFormat("Mai-2013", DateFormatDeDE4, "01.05.2013", "Mai 13", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE8()
        //{
        //    SetValueAndAssertAppliedFormat("Mai-13", DateFormatDeDE4, "01.05.2013", "Mai 13", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE9()
        //{
        //    SetValueAndAssertAppliedFormat("8-Sep", DateFormatDeDE3, string.Format("08.09.{0}", DateTime.Now.Year), "08. Sep", DeDECulture);
        //}

        // TODO: See TeamPulse item 154193 Some dates are not parsed in correct formats in de-De culture
        //[TestMethod]
        //public void SimpleDateTestDeDE10()
        //{
        //    SetValueAndAssertAppliedFormat("8-September", DateFormatDeDE3, string.Format("08.09.{0}", DateTime.Now.Year), "08. Sep", DeDECulture);
        //}

        [TestMethod]
        public void SimpleDateTestDeDE13()
        {
            SetValueAndAssertAppliedFormat("13.12.11", DateFormatDeDE1, "13.12.2011", "13.12.2011", DeDECulture);
        }

        #endregion


        #region Set Value and Assert Date Format in bg-BG

        [TestMethod]
        public void SimpleDateTestBgBG1()
        {
            SetValueAndAssertAppliedFormat("13.12.11", DateFormatBgBG1, "13.12.2011 г.", "13.12.2011 г.", BgBGCulture);
            // The purpose of the test is to check the correct ordering of the day and month part of the date format in
            // Bulgarian culture. At the moment the default Bulgarian date format of the Spreadsheet has the suffix 'г.'
            // which differs from the behavior of Excel and might change in the future.
        }

        #endregion


        #region Set Value & Format and Assert Date Format Applied Correctly

        [TestMethod]
        public void SetDateThenEscapedSeparatorDateFormatTest1()
        {
            RunTestInMultipleCultures(() =>
            {
                string format = "\"m\"/d/yyyy";
                DateTime testDate = new DateTime(2013, 10, 5);
                SetValueAndFormatAssertResult(testDate, format, format, "10/5/2013", "M/5/2013");
            }, new string[] { "En-US" });
        }

        [TestMethod]
        public void SetDateThenEscapedSeparatorDateFormatTest2()
        {
            RunTestInMultipleCultures(() =>
            {
                string format = "m\"/\"d/yyyy";
                DateTime testDate = new DateTime(2013, 10, 5);
                SetValueAndFormatAssertResult(testDate, format, format, "10/5/2013", "10/5/2013");
            }, new string[] { "En-US" });
        }

        [TestMethod]
        public void SetDateThenEscapedSeparatorDateFormatTest3()
        {
            RunTestInMultipleCultures(() =>
            {
                string format = "m\\/d/yyyy";
                DateTime testDate = new DateTime(2013, 10, 5);
                SetValueAndFormatAssertResult(testDate, format, format, "10/5/2013", "10/5/2013");
            }, new string[] { "En-US" });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest1()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedCellValue = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS1, DateFormatEnUS1, expectedCellValue, "10/5/2013");
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest2()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedCellValue = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS1, DateFormatEnUS1, expectedCellValue, "12/31/2013");
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest3()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("d-MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS2, DateFormatEnUS2, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest4()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("d-MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS2, DateFormatEnUS2, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest5()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("d-MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS3, DateFormatEnUS3, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest6()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("d-MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEnUS3, DateFormatEnUS3, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest7()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormat4, DateFormat4, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest8()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormat4, DateFormat4, expectedValueString, exptectedResultValueString);

            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest9()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString(@"dd""/""MM""/""yyyy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES1, DateFormatEsES1, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest10()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString(@"dd""/""MM""/""yyyy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES1, DateFormatEsES1, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest11()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd-MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES2, DateFormatEsES2, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest12()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd-MMM-yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES2, DateFormatEsES2, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest13()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd-MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES3, DateFormatEsES3, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest14()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd-MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatEsES3, DateFormatEsES3, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest15()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd.MM.yyyy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE1, DateFormatDeDE1, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest16()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd.MM.yyyy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE1, DateFormatDeDE1, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest17()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd. MMM yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE2, DateFormatDeDE2, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest18()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd. MMM yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE2, DateFormatDeDE2, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest19()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd. MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE3, DateFormatDeDE3, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest20()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("dd. MMM", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE3, DateFormatDeDE3, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest21()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 10, 5);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("MMM yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE4, DateFormatDeDE4, expectedValueString, exptectedResultValueString);
            });
        }

        [TestMethod]
        public void SetDateThenDateFormatTest22()
        {
            RunTestInMultipleCultures(() =>
            {
                DateTime testDate = new DateTime(2013, 12, 31);
                string expectedValueString = testDate.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo.DateTimeFormat.ShortDatePattern, FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                string exptectedResultValueString = testDate.ToString("MMM yy", FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
                SetValueAndFormatAssertResult(testDate, DateFormatDeDE4, DateFormatDeDE4, expectedValueString, exptectedResultValueString);
            });
        }

        #endregion


        #region Set Format & Value and Assert Date Format Applied Correctly

        [TestMethod]
        public void TestDateEnUS1()
        {
            string format = "m/d/yyyy";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue), 1, true, false, false,
                "6/10/1987", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS2()
        {
            string format = "[$-F800]dddd, mmmm dd, yyyy";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "Wednesday, June 10, 1987", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS3()
        {
            string format = "m/d;(@)";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue), 1, true, false, false, "6/10", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS3_Text()
        {
            string format = "m/d;(@)";

            AssertDateFormatAppliedCorrectly(format, "test", typeof(TextCellValue), 1, true, false, false, "(test)", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS4()
        {
            string format = "m/d/yy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue), 1, true, false, false, "6/10/87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS5()
        {
            string format = "mm/dd/yy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue), 1, true, false, false, "06/10/87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS6()
        {
            string format = "[$-409]d-mmm;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                 1, true, false, false, "10-Jun", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS7()
        {
            string format = "[$-409]d-mmm-yy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                 1, true, false, false, "10-Jun-87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS8()
        {
            string format = "[$-409]mmm-yy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "Jun-87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS9()
        {
            string format = "[$-409]mmmm-yy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "June-87", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS10()
        {
            string format = "[$-409]mmmm d, yyyy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "June 10, 1987", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS11()
        {
            string format = "[$-409]m/d/yy h:mm AM/PM;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "6/10/87 3:44 AM", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS12()
        {
            string format = "m/d/yy h:mm;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "6/10/87 3:44", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS13()
        {
            string format = "[$-409]mmmmm;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "J", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS14()
        {
            string format = "[$-409]mmmmm-yy;@";

            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("J", false, false, true));
            items.Add(new CellValueFormatResultItem("-87", false, false, true));
            FormatPatternInfo formatInfo = new FormatPatternInfo("J-87", items);

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                2, true, false, false, "J-87", EnUSCulture, formatInfo);
        }

        [TestMethod]
        public void TestDateEnUS15()
        {
            string format = "[$-409]d-mmm-yyyy;@";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, "10-Jun-1987", EnUSCulture);
        }

        [TestMethod]
        public void TestDateEnUS16()
        {
            string format = "m/d/yy h:mm;@";
            string inputCellValue = "1234{ds}56789";

            AssertDateFormatAppliedCorrectly(format, inputCellValue, typeof(NumberCellValue),
                1, true, false, false, "5/18/03 13:37", EnUSCulture);
        }

        [TestMethod]
        public void SetDateFormatThenNumberValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("mm/dd/yy;@", "1{gs}234", "mm/dd/yy;@", "5/18/1903", "05/18/03",
                "#{gs}##0", "1234", "1{gs}234", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetDateFormatThenCurrencyValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("mm/dd/yy;@", "$1234", "mm/dd/yy;@", "5/18/1903", "05/18/03",
                "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetDateFormatThenPercentageValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("mm/dd/yy;@", "15%", "mm/dd/yy;@", "3:36:00 AM", "12/31/99",
                "0%", "15%", "15%", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetFormatAndThenValue_WhenSetValueWithDateTimeOverloadIsUsed_DateFormatIsAppliedCorrectly()
        {
            // Arrange
            Worksheet worksheet = TestHelper.CreateWorksheet();
            var expectedFormat = new CellValueFormat("dd yyyy MMM");

            // Act
            worksheet.Cells[1, 1].SetFormat(expectedFormat);
            worksheet.Cells[1, 1].SetValue(new DateTime(2015, 6, 23));
            var cellFormat = worksheet.Cells[1, 1].GetFormat().Value;

            // Assert
            Assert.AreEqual(expectedFormat.FormatString, cellFormat.FormatString);
            Assert.AreEqual("23 2015 Jun", worksheet.Cells[1, 1].GetValue().Value.GetResultValueAsString(cellFormat));
        }

        [TestMethod]
        public void SetFormatAndThenValue_WhenSetValueWithNumberOverloadIsUsed_DateFormatIsAppliedCorrectly()
        {
            // Arrange
            Worksheet worksheet = TestHelper.CreateWorksheet();
            var expectedFormat = new CellValueFormat("ddd yyyy MMM");

            // Act
            worksheet.Cells[1, 1].SetFormat(expectedFormat);
            worksheet.Cells[1, 1].SetValue(5);
            var cellFormat = worksheet.Cells[1, 1].GetFormat().Value;

            // Assert
            Assert.AreEqual(expectedFormat.FormatString, cellFormat.FormatString);
            Assert.AreEqual("Fri 1900 Jan", worksheet.Cells[1, 1].GetValue().Value.GetResultValueAsString(cellFormat));
        }

        #endregion


        #region Tests

        [TestMethod]
        public void SimpleDateTest()
        {
            SetValueAndAssertAppliedFormat("6/10/1987", DateFormatEnUS1, "6/10/1987", "6/10/1987", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest1()
        {
            SetValueAndAssertAppliedFormat("6/10/1987  3:44", DateTimeFormatEnUS1, "6/10/1987 3:44:00 AM", "6/10/1987 3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest2()
        {
            SetValueAndAssertAppliedFormat("6/10/1987  3:44:55", DateTimeFormatEnUS1, "6/10/1987 3:44:55 AM", "6/10/1987 3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest3()
        {
            SetValueAndAssertAppliedFormat("6/10/1987  3:44:55.678", "mm:ss.0", "6/10/1987 3:44:56 AM", "44:55.7", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest4()
        {
            SetValueAndAssertAppliedFormat("1/2", DateFormatEnUS3, "1/2/" + DateTime.Now.Year.ToString(), "2-Jan", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest5()
        {
            SetValueAndAssertAppliedFormat("1-2", DateFormatEnUS3, "1/2/" + DateTime.Now.Year.ToString(), "2-Jan", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest7()
        {
            SetValueAndAssertAppliedFormat("1/31", DateFormatEnUS3, "1/31/" + DateTime.Now.Year.ToString(), "31-Jan", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest8()
        {
            SetValueAndAssertAppliedFormat("1/32", DateFormat4, "1/1/1932", "Jan-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest29()
        {
            SetValueAndAssertAppliedFormat("2/28", DateFormatEnUS3, "2/28/" + DateTime.Now.Year.ToString(), "28-Feb", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest9()
        {
            SetValueAndAssertAppliedFormat("3/31", DateFormatEnUS3, "3/31/" + DateTime.Now.Year.ToString(), "31-Mar", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest10()
        {
            SetValueAndAssertAppliedFormat("3/32", DateFormat4, "3/1/1932", "Mar-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest11()
        {
            SetValueAndAssertAppliedFormat("4/30", DateFormatEnUS3, "4/30/" + DateTime.Now.Year.ToString(), "30-Apr", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest12()
        {
            SetValueAndAssertAppliedFormat("4/31", DateFormat4, "4/1/1931", "Apr-31", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest13()
        {
            SetValueAndAssertAppliedFormat("5/31", DateFormatEnUS3, "5/31/" + DateTime.Now.Year.ToString(), "31-May", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest14()
        {
            SetValueAndAssertAppliedFormat("5/32", DateFormat4, "5/1/1932", "May-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest15()
        {
            SetValueAndAssertAppliedFormat("6/30", DateFormatEnUS3, "6/30/" + DateTime.Now.Year.ToString(), "30-Jun", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest16()
        {
            SetValueAndAssertAppliedFormat("6/31", DateFormat4, "6/1/1931", "Jun-31", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest17()
        {
            SetValueAndAssertAppliedFormat("7/31", DateFormatEnUS3, "7/31/" + DateTime.Now.Year.ToString(), "31-Jul", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest18()
        {
            SetValueAndAssertAppliedFormat("7/32", DateFormat4, "7/1/1932", "Jul-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest19()
        {
            SetValueAndAssertAppliedFormat("8/31", DateFormatEnUS3, "8/31/" + DateTime.Now.Year.ToString(), "31-Aug", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest20()
        {
            SetValueAndAssertAppliedFormat("8/32", DateFormat4, "8/1/1932", "Aug-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest21()
        {
            SetValueAndAssertAppliedFormat("9/30", DateFormatEnUS3, "9/30/" + DateTime.Now.Year.ToString(), "30-Sep", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest22()
        {
            SetValueAndAssertAppliedFormat("9/31", DateFormat4, "9/1/1931", "Sep-31", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest23()
        {
            SetValueAndAssertAppliedFormat("10/31", DateFormatEnUS3, "10/31/" + DateTime.Now.Year.ToString(), "31-Oct", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest24()
        {
            SetValueAndAssertAppliedFormat("10/32", DateFormat4, "10/1/1932", "Oct-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest25()
        {
            SetValueAndAssertAppliedFormat("11/30", DateFormatEnUS3, "11/30/" + DateTime.Now.Year.ToString(), "30-Nov", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest26()
        {
            SetValueAndAssertAppliedFormat("11/31", DateFormat4, "11/1/1931", "Nov-31", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest27()
        {
            SetValueAndAssertAppliedFormat("12/31", DateFormatEnUS3, "12/31/" + DateTime.Now.Year.ToString(), "31-Dec", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest28()
        {
            SetValueAndAssertAppliedFormat("12/32", DateFormat4, "12/1/1932", "Dec-32", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest30()
        {
            RunTestInMultipleCultures(() =>
                {
                    Worksheet worksheet = TestHelper.CreateWorksheet();
                    worksheet.Cells[0, 0].SetValue("2/29");

                    ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
                    CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;

                    Assert.IsTrue(cellValue is NumberCellValue);

                    if (DateTime.IsLeapYear(DateTime.Now.Year))
                    {
                        Assert.AreEqual(DateFormatEnUS3, format.FormatString);
                    }
                    else
                    {
                        Assert.AreEqual(DateFormat4, format.FormatString);
                    }

                    if (DateTime.IsLeapYear(DateTime.Now.Year))
                    {
                        Assert.AreEqual(string.Format("2/29/{0}", DateTime.Now.Year), cellValue.GetValueAsString(format));
                        Assert.AreEqual("29-Feb", cellValue.GetResultValueAsString(format));
                    }
                    else
                    {
                        Assert.AreEqual("2/1/1929", cellValue.GetValueAsString(format));
                        Assert.AreEqual("Feb-29", cellValue.GetResultValueAsString(format));
                    }
                }, EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest31()
        {
            SetValueAndAssertAppliedFormat("2/30", DateFormat4, "2/1/1930", "Feb-30", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest32()
        {
            SetValueAndAssertAppliedFormat("6/1987", DateFormat4, "6/1/1987", "Jun-87", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest34()
        {
            SetValueAndAssertAppliedFormat("6/9999", DateFormat4, "6/1/9999", "Jun-99", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest36()
        {
            SetValueAndAssertAppliedFormat("6/10/87", DateFormatEnUS1, "6/10/1987", "6/10/1987", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest37()
        {
            SetValueAndAssertAppliedFormat("6/10/87  3:44", DateTimeFormatEnUS1, "6/10/1987 3:44:00 AM", "6/10/1987 3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest38()
        {
            SetValueAndAssertAppliedFormat("6/10/87  3:44:55", DateTimeFormatEnUS1, "6/10/1987 3:44:55 AM", "6/10/1987 3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleDateTest39()
        {
            SetValueAndAssertAppliedFormat("6/10/87  3:44:55.678", "mm:ss.0", "6/10/1987 3:44:56 AM", "44:55.7", EnUSCulture);
        }

        [TestMethod]
        public void TestResultValueWhenHourFormatIsUpper()
        {
            RunTestInMultipleCultures(() =>
            {
                string newValue = "2/2/2013 12:34";
                string newFormat = "m/d/yyyy HH:mm";
                string expectedResultValueAsString = "2/2/2013 12:34";

                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue(newValue);

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.IsTrue(cellValue is NumberCellValue);
                Assert.AreEqual(expectedResultValueAsString, cellValue.GetResultValueAsString(new CellValueFormat(newFormat)));
            }, EnUSCulture);
        }

        [TestMethod]
        public void TestFormatWithDifferentCasing()
        {
            RunTestInMultipleCultures(() =>
            {
                string newValue = "1/2/2013 13:34";
                string newFormat = "m/dDd/yYyY HH:mm Am/pM";
                string expectedResultValueAsString = "1/Wed/2013 01:34 PM";

                Worksheet worksheet = TestHelper.CreateWorksheet();
                worksheet.Cells[0, 0].SetValue(newValue);

                ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                Assert.IsTrue(cellValue is NumberCellValue);
                Assert.AreEqual(expectedResultValueAsString, cellValue.GetResultValueAsString(new CellValueFormat(newFormat)));
            }, EnUSCulture);
        }

        #endregion

    }
}
