﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Fractions : CellValueFormatTestsBase
    {
        #region Fields

        internal static string FractionsOne = string.Concat(Enumerable.Repeat(FormatHelper.FractionsInvisibleChar, 1));
        internal static string FractionsTwo = string.Concat(Enumerable.Repeat(FormatHelper.FractionsInvisibleChar, 2));
        internal static string FractionsThree = string.Concat(Enumerable.Repeat(FormatHelper.FractionsInvisibleChar, 3));

        #endregion


        #region Set Value and Assert Fractions Format is Applied

        [TestMethod]
        public void SimpleFractionsTest()
        {
            SetValueAndAssertAppliedFormat("1 1/2", CellValueFormatTestsBase.FractionShortFormat, 1.5, "1 1/2");
        }

        [TestMethod]
        public void SimpleFractionsTest1()
        {
            SetValueAndAssertAppliedFormat("1 2/4", CellValueFormatTestsBase.FractionShortFormat, 1.5, "1 1/2");
        }

        [TestMethod]
        public void SimpleFractionsTest2()
        {
            SetValueAndAssertAppliedFormat("1 2/2", CellValueFormatTestsBase.FractionShortFormat, 2, string.Format("2 {0}/{0}", FractionsOne));
        }

        [TestMethod]
        public void SimpleFractionsTest3()
        {
            SetValueAndAssertAppliedFormat("1 12/35", CellValueFormatTestsBase.FractionLongFormat, 1.34285714285714, "1 12/35");
        }

        [TestMethod]
        public void SimpleFractionsTest4()
        {
            SetValueAndAssertAppliedFormat("1 17/63", CellValueFormatTestsBase.FractionLongFormat, 1.26984126984127, "1 17/63");
        }

        [TestMethod]
        public void SimpleFractionsTest5()
        {
            SetValueAndAssertAppliedFormat("1 24/89", CellValueFormatTestsBase.FractionLongFormat, 1.26966292134831, "1 24/89");
        }

        [TestMethod]
        public void SimpleFractionsTest6()
        {
            SetValueAndAssertAppliedFormat("1 23/32", CellValueFormatTestsBase.FractionLongFormat, 1.71875, "1 23/32");
        }

        [TestMethod]
        public void SimpleFractionsTest7()
        {
            string expectedResultValueString = string.Format("1 {0}1/4{0}", FractionsOne);
            SetValueAndAssertAppliedFormat("1 25/100", CellValueFormatTestsBase.FractionLongFormat, 1.25, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest8()
        {
            string expectedResultValueString = string.Format("1 {0}1/2{0}", FractionsOne);
            SetValueAndAssertAppliedFormat("1 50/100", CellValueFormatTestsBase.FractionLongFormat, 1.5, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest9()
        {
            string expectedResultValueString = string.Format("-1 {0}1/50", FractionsOne);
            SetValueAndAssertAppliedFormat("-1 2/100", CellValueFormatTestsBase.FractionLongFormat, -1.02, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest10()
        {
            string expectedResultValueString = string.Format("-2 {0}/{0}", FractionsOne);
            SetValueAndAssertAppliedFormat("-1 2/2", CellValueFormatTestsBase.FractionShortFormat, -2d, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest11()
        {
            string expectedResultValueString = string.Format("-2 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("-1 100/100", CellValueFormatTestsBase.FractionLongFormat, -2d, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest12()
        {
            string expectedResultValueString = string.Format("-2 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("-1 50/50", CellValueFormatTestsBase.FractionLongFormat, -2d, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest13()
        {
            string expectedResultValueString = string.Format("-8 {0}7/68", FractionsOne);
            SetValueAndAssertAppliedFormat("-1 8765/1234", CellValueFormatTestsBase.FractionLongFormat, -8.10291734197731, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest14()
        {
            string expectedResultValueString = string.Format("-1 {0}5/23", FractionsOne);
            SetValueAndAssertAppliedFormat("-1 1234/5678", CellValueFormatTestsBase.FractionLongFormat, -1.21733004579077, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest15()
        {
            string expectedResultValueString = string.Format("1 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("1 1/101", CellValueFormatTestsBase.FractionLongFormat, 1.00990099009901, expectedResultValueString);
        }

        #region Old Comment
        //in Excel this code (entering 1 1/100) will display 1 1/99, we display 1
        #endregion
        [TestMethod]
        public void SimpleFractionsTest16()
        {
            string expectedResultValueString = string.Format("1 {0}1/99", FractionsOne);
            SetValueAndAssertAppliedFormat("1 1/100", CellValueFormatTestsBase.FractionLongFormat, 1.01, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest17()
        {
            string expectedResultValueString = string.Format("2 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("1 101/100", CellValueFormatTestsBase.FractionLongFormat, 2.01, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest18()
        {
            string expectedResultValueString = string.Format("31 {0}5/12", FractionsOne);
            SetValueAndAssertAppliedFormat("1 365/12", CellValueFormatTestsBase.FractionLongFormat, 31.4166666666667, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest19()
        {
            string expectedResultValueString = string.Format("-1 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("-1 1/101", CellValueFormatTestsBase.FractionLongFormat, -1.00990099009901, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest20()
        {
            SetValueAndAssertAppliedFormat("-1 99/100", CellValueFormatTestsBase.FractionLongFormat, -1.99, "-1 98/99");
        }

        [TestMethod]
        public void SimpleFractionsTest21()
        {
            string expectedResultValueString = string.Format("2 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("1 100/101", CellValueFormatTestsBase.FractionLongFormat, 1.99009900990099, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest22()
        {
            string expectedResultValueString = string.Format("-2 {0}/{0}", FractionsTwo);
            SetValueAndAssertAppliedFormat("-1 100/101", CellValueFormatTestsBase.FractionLongFormat, -1.99009900990099, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest23()
        {
            string expectedResultValueString = string.Format("1 {0}2/7{0}", FractionsOne);
            SetValueAndAssertAppliedFormat("1 1234/4321", CellValueFormatTestsBase.FractionLongFormat, 1.28558204119417, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest24()
        {
            string expectedResultValueString = string.Format("2 {0}5/11", FractionsOne);
            SetValueAndAssertAppliedFormat("1 9876/6789", CellValueFormatTestsBase.FractionLongFormat, 2.454706142289, expectedResultValueString);
        }

        [TestMethod]
        public void SimpleFractionsTest25()
        {
            string expectedResultValueString = string.Format("1 11/16", FractionsOne);
            SetValueAndAssertAppliedFormat("1 6789/9876", CellValueFormatTestsBase.FractionLongFormat, 1.68742405832321, expectedResultValueString);
        }

        #endregion


        #region Set Fraction Format and Apply Value

        [TestMethod]
        public void TestFractionsUpToOneDigit()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/2", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1 123/234", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1 23/567", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/6", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/6", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1 123/789", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1 998/999", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1 153/165", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1/9", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1/9", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1/9", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1/10", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1/2", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "1/2", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit8()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("- 1/2", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "-1/2", info);
        }

        [TestMethod]
        public void TestFractionsUpToOneDigit9()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/3", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("-1 1/3", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "-1 1/3", info);
        }

        #region Old Comment
        //in Excel 11/10 returns 1 1/9 which is strange because 1/10 returns 0 when formatted with # ?/?. Here 11/10 returns 1 which is correct i think
        #endregion
        [TestMethod]
        public void TestFractionsUpToOneDigit10()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/9", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/9", items);
            TestScientific(CellValueFormatTestsBase.FractionShortFormat, "11/10", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("23/45", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 23/45", items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 23/45", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("3/74", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}3/74", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 23/567", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("7/11", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}7/11", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 567/890", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("1/69", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}1/69", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 13/899", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsTwo), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 899/900", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsTwo), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 7/899", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("1/69", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}1/69", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 13/899", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsTwo), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1 900/899", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits8()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-9 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            items.Add(new CellValueFormatResultItem("1/3", false, false, false));
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-9 {0}1/3{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "-1 100/12", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits9()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("{0}1/2{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1/2", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits10()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            items.Add(new CellValueFormatResultItem(string.Format("{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("- {0}1/2{0}", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "-1/2", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits11()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{0}", FractionsTwo), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "1/100", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits12()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{0}", FractionsTwo), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "-1/100", info);
        }

        [TestMethod]
        public void TestFractionsUpToTwoDigits13()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-8 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem(string.Format("7/68", FractionsOne), false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-8 {0}7/68", FractionsOne), items);
            TestScientific(CellValueFormatTestsBase.FractionLongFormat, "-1 8765/1234", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("23/456", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}23/456", FractionsOne), items);
            TestScientific("# ???/???", "1 23/456", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("115/226", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 115/226", items);
            TestScientific("# ???/???", "1 345/678", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsThree), items);
            TestScientific("# ???/???", "1 1/1000", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("53/121", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}53/121", FractionsOne), items);
            TestScientific("# ???/???", "1 3456/7890", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("28/45", false, false, false));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}28/45{0}", FractionsOne), items);
            TestScientific("# ???/???", "1 56/90", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsTwo, true, false, false));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            items.Add(new CellValueFormatResultItem(FractionsTwo, true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}1/2{0}", FractionsTwo), items);
            TestScientific("# ???/???", "1 1/2", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsTwo, true, false, false));
            items.Add(new CellValueFormatResultItem("1/13", false, false, false));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}1/13{1}", FractionsTwo, FractionsOne), items);
            TestScientific("# ???/???", "1 1/13", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsTwo, true, false, false));
            items.Add(new CellValueFormatResultItem("2/999", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}2/999", FractionsTwo), items);
            TestScientific("# ???/???", "1 1001/999", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits8()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("4 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("4 {0}/{0}", FractionsThree), items);
            TestScientific("# ???/???", "1 45/15", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits9()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsTwo, true, false, false));
            items.Add(new CellValueFormatResultItem("1/15", false, false, false));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("- {0}1/15{1}", FractionsTwo, FractionsOne), items);
            TestScientific("# ???/???", "-3/45", info);
        }

        [TestMethod]
        public void TestFractionsUpToThreeDigits10()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{0}", FractionsThree), items);
            TestScientific("# ???/???", "-1/1000", info);
        }

        #region Old Comment
        //when inputted -8765/1234 with format "# ???/???" in Excel it displays "-7 60/583", which i think is bug because when you double click the 
        //cell and enter in edit mode and then exit it displays -7 67/651, which we calculate from the first time.
        #endregion
        [TestMethod]
        public void TestFractionsUpToThreeDigits11()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-7 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("60/583", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-7 {0}60/583", FractionsOne), items);
            TestScientific("# ???/???", "-8765/1234", info);
        }

        [TestMethod]
        public void TestFractionsAsHalves()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/2", items);
            TestScientific("# ?/2", "1 123/456", info);
        }

        [TestMethod]
        public void TestFractionsAsHalves1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/2", "1 1/5", info);
        }

        [TestMethod]
        public void TestFractionsAsHalves2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/2", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/2", items);
            TestScientific("# ?/2", "1 2/3", info);
        }

        [TestMethod]
        public void TestFractionsAsHalves3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/2", "1 3/4", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/4", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 1/4", FractionsOne), items);
            TestScientific("# ?/4", "1 13/46", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/4", "1 1/12", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("3/4", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 3/4", items);
            TestScientific("# ?/4", "1 56/78", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("2 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/4", "1 7/8", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("3/4", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 3/4", items);
            TestScientific("# ?/4", "1 6/7", info);
        }

        [TestMethod]
        public void TestFractionsAsQuarters5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("2 ", false, false, true));
            items.Add(new CellValueFormatResultItem("3/4", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("2 3/4", items);
            TestScientific("# ?/4", "1 9876/5432", info);
        }

        [TestMethod]
        public void TestFractionsAsEights()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("4/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 4/8", items);
            TestScientific("# ?/8", "1 23/45", info);
        }

        [TestMethod]
        public void TestFractionsAsEights1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/8", items);
            TestScientific("# ?/8", "1 12/90", info);
        }

        [TestMethod]
        public void TestFractionsAsEights2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem("1/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 1/8", items);
            TestScientific("# ?/8", "1 1/16", info);
        }

        [TestMethod]
        public void TestFractionsAsEights3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/8", "1 1/17", info);
        }

        [TestMethod]
        public void TestFractionsAsEights4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("7/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("7/8", items);
            TestScientific("# ?/8", "12/13", info);
        }

        [TestMethod]
        public void TestFractionsAsEights5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsOne), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{0}", FractionsOne), items);
            TestScientific("# ?/8", "1/17", info);
        }

        [TestMethod]
        public void TestFractionsAsEights6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem("7/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("- 7/8", items);
            TestScientific("# ?/8", "-12/13", info);
        }

        [TestMethod]
        public void TestFractionsAsEights7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem("7/8", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("- 7/8", items);
            TestScientific("# ?/8", "- 12/13", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("6/16", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("16/16", items);
            TestScientific("# ??/16", "123/321", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("6/16", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("- 16/16", items);
            TestScientific("# ??/16", "-123/321", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 11/11", items);
            TestScientific("# ??/16", "1 1/33", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo("0 11/11", items);
            TestScientific("# ??/16", "-1/33", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("1/16", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("11/16", items);
            TestScientific("# ??/16", "1/32", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo("1 11/11", items);
            TestScientific("# ??/16", "31/32", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("3/16", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("-1 13/16", items);
            TestScientific("# ??/16", "-1 1234/5678", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("7 ", false, false, true));
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("2/16", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("7 12/16", items);
            TestScientific("# ??/16", "8765/1234", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths8()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{0}", FractionsTwo), items);
            TestScientific("# ??/16", "33/33", info);
        }

        [TestMethod]
        public void TestFractionsAsSixteenths9()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{0}", FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-1 {0}/{0}", FractionsTwo), items);
            TestScientific("# ??/16", "- 33/33", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("5/10", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("5/10", items);
            TestScientific("# ?/10", "1111/2222", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("- ", false, false, true));
            items.Add(new CellValueFormatResultItem("4/10", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("- 4/10", items);
            TestScientific("# ?/10", "-234/567", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsOne, FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-1 {0}/{1}", FractionsOne, FractionsTwo), items);
            TestScientific("# ?/10", "-    3/3", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("3/10", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("3/10", items);
            TestScientific("# ?/10", "3/10", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-85 ", false, false, true));
            items.Add(new CellValueFormatResultItem("3/10", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo("-85 3/10", items);
            TestScientific("# ?/10", "-3 1234/15", info);
        }

        [TestMethod]
        public void TestFractionsAsTenths5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsOne, FractionsTwo), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{1}", FractionsOne, FractionsTwo), items);
            TestScientific("# ?/10", "1/21", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("1/100", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("{0}1/100", FractionsOne), items);
            TestScientific("# ??/100", "1/100", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(FractionsOne, true, false, false));
            items.Add(new CellValueFormatResultItem("1/100", false, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("{0}1/100", FractionsOne), items);
            TestScientific("# ??/100", "1/200", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "1/201", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("1 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "1 1/201", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("-1 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("-1 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "-1 1/201", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths5()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "-1/201", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths6()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("0 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("0 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "0", info);
        }

        [TestMethod]
        public void TestFractionsAsHundredths7()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("123 ", false, false, true));
            items.Add(new CellValueFormatResultItem(string.Format("{0}/{1}", FractionsTwo, FractionsThree), true, false, false));
            FormatPatternInfo info = new FormatPatternInfo(string.Format("123 {0}/{1}", FractionsTwo, FractionsThree), items);
            TestScientific("# ??/100", "123", info);
        }

        #endregion
    }
}
