﻿using System;
using System.Globalization;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Functions : CellValueFormatTestsBase
    {
        [TestMethod]
        public void DateFunctionFormatTest()
        {
            string cellInput = "=DATE(2013{ls} 10{ls} 5)";
            string expectedFormat = "m/d/yyyy";

            TestFormulaCellValue(cellInput, expectedFormat, cellInput, "10/5/2013", EnUSCulture);
        }

        [TestMethod]
        public void TodayFunctionFormatTest()
        {
            string cellInput = "=TODAY()";
            string expectedFormat = "m/d/yyyy";

            TestFormulaCellValue(cellInput, expectedFormat, cellInput,
                DateTime.Now.ToString("M/d/yyyy", new CultureInfo("en-US")), EnUSCulture);
        }

        [TestMethod]
        public void RateFunctionFormatTest()
        {
            string cellInput = "=RATE(48{ls} -200{ls} 8000{ls} 0{ls} 0)";
            TestFormulaCellValue(cellInput, "0{ds}00%", cellInput, "0{ds}77%", EnUSCulture);
        }

        [TestMethod]
        public void DbFunctionFormatTest()
        {
            TestFormulaCellValue("=Db(1000000{ls} 100000{ls} 6{ls} 1{ls} 7)",
                "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)",
                "=DB(1000000{ls} 100000{ls} 6{ls} 1{ls} 7)",
                "$186{gs}083{ds}33",
                EnUSCulture);
        }

        [TestMethod]
        public void IpmtFunctionFormatTest()
        {
            TestFormulaCellValue("=Ipmt(0{ds}1{ls} 3{ls} 3{ls} 8000{ls} 500{ls} 0)",
                "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)",
                "=IPMT(0{ds}1{ls} 3{ls} 3{ls} 8000{ls} 500{ls} 0)",
                "($260{ds}73)",
                EnUSCulture);
        }

        [TestMethod]
        public void FvFunctionFormatTest()
        {
            TestFormulaCellValue("=Fv(0{ds}005{ls} 10{ls} -200{ls} -500{ls} 0)",
                "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)",
                "=FV(0{ds}005{ls} 10{ls} -200{ls} -500{ls} 0)",
                "$2{gs}571{ds}18",
                EnUSCulture);
        }

        [TestMethod]
        public void NpvFunctionFormatTest()
        {
            TestFormulaCellValue("=NPV(0{ls} -10000{ls} 3000{ls} 4200{ls} 6800)",
                "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)",
                "=NPV(0{ls} -10000{ls} 3000{ls} 4200{ls} 6800)",
                "$4{gs}000{ds}00",
                EnUSCulture);
        }

        [TestMethod]
        public void PpmtFunctionFormatTest()
        {
            TestFormulaCellValue("=Ppmt(0{ds}08{ls} 10{ls} 10{ls} 200000)",
                 CellValueFormatTests_Currency.TestCurrencyDecimalNumberFormatEnUS,
                "=PPMT(0{ds}08{ls} 10{ls} 10{ls} 200000)",
                "($27{gs}598{ds}05)",
                EnUSCulture);
        }

        [TestMethod]
        public void PvFunctionFormatTest()
        {
            TestFormulaCellValue("=Pv(0{ls} 240{ls} 500000{ls} 0{ls} 1)",
                "\"$\"#{gs}##0{ds}00_);[Red](\"$\"#{gs}##0{ds}00)",
                "=PV(0{ls} 240{ls} 500000{ls} 0{ls} 1)",
                "($120{gs}000{gs}000{ds}00)",
                EnUSCulture);
        }

        [TestMethod]
        public void SlnFunctionFormatTest()
        {
            TestFormulaCellValue("=Sln(30000{ls} 7500{ls} 10)",
                CellValueFormatTests_Currency.TestCurrencyDecimalNumberFormatEnUS,
                "=SLN(30000{ls} 7500{ls} 10)",
                "$2{gs}250{ds}00",
                EnUSCulture);
        }

        [TestMethod]
        public void VdbFunctionFormatTest()
        {
            TestFormulaCellValue("=Vdb(2400{ls} 300{ls} 3650{ls} 0{ls} 1)",
                CellValueFormatTests_Currency.TestCurrencyDecimalNumberFormatEnUS,
                "=VDB(2400{ls} 300{ls} 3650{ls} 0{ls} 1)",
                "$1{ds}32",
                EnUSCulture);
        }
    }
}
