﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_General : CellValueFormatTestsBase
    {
        [TestMethod]
        public void SimpleGeneralNumberTest()
        {
            string testValue = "123";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue);
        }

        [TestMethod]
        public void SimpleGeneralNumberTest1()
        {
            string testValue = "1{ds}2345";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue);
        }

        [TestMethod]
        public void SetGeneralNumberPlusWhiteSpace()
        {
            string testValue = "123    ";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, "123", "123", expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetValueThatDoesNotRequireNumberFormatTest1()
        {
            string testValue = "234{ds}567";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue);
        }

        [TestMethod]
        public void SetValueThatDoesNotRequireNumberFormatTest2()
        {
            string testValue = "1234{ds}567";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue);
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyNumberFormat()
        {
            string testValue = "1" + Environment.NewLine + "2";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyCurrencyFormatEnUS()
        {
            string testValue = "$1{gs}23{gs}456";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyCurrencyFormatEnUS2()
        {
            string testValue = "$1{gs}23456";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyCurrencyFormatEnUS3()
        {
            string testValue = "$1{gs}23{ds}456";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyCurrencyFormatEnUS4()
        {
            string testValue = "$123456{gs}";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyCurrencyFormatEnUS5()
        {
            string testValue = "$123{gs}456{gs}";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetDateThatNotApplyDateFormatEnUS()
        {
            string testValue = "13-1";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetDateThatNotApplyDateFormatEnUS2()
        {
            string testValue = "6/10000";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetDateThatNotApplyDateFormatEnUS3()
        {
            string testValue = "6/1899";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetDateThatNotApplyDateFormatEnUS4()
        {
            string testValue = "1,2";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        // TODO: The test fails in pa-IN culture
        // See item in TeamPulse 154146 The string 123,45 is parased to 12345 in pa-IN culture
        [TestMethod]
        public void SetNumberValueWithIncorrectGroupSeparators()
        {
            string testValue = "123{gs}45";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue,  expectedType: typeof(TextCellValue), 
                cultures: new string[] { "en-US", "ar-DZ", "ar-QA", "da-DK", "de-DE", "es-ES", "fr-BE", "it-IT", "ko-KR", "bg-BG", "fr-FR", "nl-BE", "ru-RU", "sv-FI" });
        }

        // TODO: The test fails in pa-IN culture
        // See item in TeamPulse 154146 The string 123,45 is parased to 12345 in pa-IN culture
        [TestMethod]
        public void SetNumberValueWithIncorrectGroupSeparators2()
        {
            string testValue = "12{gs}34{ds}567";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue)
            , cultures: new string[] { "en-US", "ar-DZ", "ar-QA", "da-DK", "de-DE", "es-ES", "fr-BE", "it-IT", "ko-KR", "bg-BG", "fr-FR", "nl-BE", "ru-RU", "sv-FI" });
        }

        // TODO: The value is parsed as date time, 
        // See item in TeamPulse 154150 Certain strings result in a date instead of a text value
        //[TestMethod]
        //public void SetNumberValueWithIncorrectGroupSeparators3()
        //{
        //    string testValue = "12{gs}3456";
        //    SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        //}

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS1()
        {
            string testValue = "31/2013";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        // TODO: See item in TeamPulse 154154 The string 2013/5 is parsed to date instead of to a text value in en-US culture
        //[TestMethod]
        //public void SetValueThatShouldNotApplyDateFormatEnUS2()
        //{
        //    string testValue = "2013/5";
        //    this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        //}

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS3()
        {
            string testValue = "2013/31/5";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }
        
        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS4()
        {
            string testValue = "32-Dec";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS5()
        {
            string testValue = "15-Asd";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS6()
        {
            string testValue = "13-feb-123";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS7()
        {
            string testValue = "32-jan-32";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS8()
        {
            string testValue = "123-mar-12";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS9()
        {
            string testValue = "32-december";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS10()
        {
            string testValue = "13-february-123";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS11()
        {
            string testValue = "32-january-32";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS12()
        {
            string testValue = "123-march-12";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EnUSCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEnUS13()
        {
            string testValue = "23/" + Environment.NewLine + "12/2013";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEsES1()
        {
            string testValue = "12/13/2013";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotApplyDateFormatEsES2()
        {
            string testValue = "2013/15/2";
            this.SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: EsESCulture, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetNumberValueWithIncorrectGroupSeparators4()
        {
            string testValue = "123456{gs}";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: NonSpaceGroupSeparatorCultures, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetNumberValueWithIncorrectGroupSeparators5()
        {
            string testValue = "{gs}123456";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, cultures: NonSpaceGroupSeparatorCultures, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestBooleanTextFormatString()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[0, 0].SetValue("true");

            ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;
            CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;

            Assert.IsTrue(cellValue is BooleanCellValue);
            Assert.IsTrue(format.Equals(CellValueFormat.GeneralFormat));

            worksheet.Cells[0, 0].SetFormat(new CellValueFormat("@"));

            cellValue = worksheet.Cells[0, 0].GetValue().Value;
            format = worksheet.Cells[0, 0].GetFormat().Value;

            Assert.IsTrue(cellValue is TextCellValue);
            Assert.IsTrue(format.Equals(new CellValueFormat("@")));
        }

        [TestMethod]
        public void TestNumberWithTrailingSignIsText()
        {
            SetValueAndAssertAppliedFormat("1-", string.Empty, "1-", "1-", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText001Dash01()
        {
            string testValue = "001-01";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText01Dash001()
        {
            string testValue = "01-001";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText00001Dash000001()
        {
            string testValue = "000001-000001";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText0001Dash01()
        {
            string testValue = "0001-01";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText01Dash0001()
        {
            string testValue = "01-0001";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsText06Dash0113()
        {
            string testValue = "06-0113";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void TestIsTextNumberAndNewLine()
        {
            string testValue = "1" + Environment.NewLine;
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        //TODO: See item in TeamPulse 154150 Certain strings result in a date instead of a text value
        //[TestMethod]
        //public void TestIsTextNumberWithSpaceInMiddle()
        //{
        //    string testValue = "1 23";
        //    SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        //}

        [TestMethod]
        public void TestIsTextWithPercentInMiddle()
        {
            string testValue = "12%3";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, testValue, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific()
        {
            string testValue = "0{ds}000000000000005";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, "5E-15");
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific2()
        {
            string testValue = "4273874923749827398749823749723894";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, "4{ds}27387492374983E+33", "4{ds}27387E+33");
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific3()
        {
            string testValue = "324234234234{ds}42384729374823749";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, "324234234234{ds}424", "3{ds}24234E+11");
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific4()
        {
            string testValue = "5389475893475987553";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, "5389475893475990000", "5{ds}38948E+18");
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific5()
        {
            string testValue = "0{ds}0000000000000000005";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, testValue, "5E-19");
        }

        [TestMethod]
        public void SetValueThatShouldNotBeScientific6()
        {
            string testValue = "0{ds}00000000000000000005";
            string displayValue = "5E-20";
            SetValueAndAssertAppliedFormat(testValue, string.Empty, displayValue, displayValue);
        }
    }
}
