﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Number : CellValueFormatTestsBase
    {
        #region Constants

        public static readonly string TestDecimalNumberFormat = "0{ds}00";
        public static readonly string TestWholeNumberFormat = "#{gs}##0";
        public static readonly string TestDecimalNumberNegativeFormat = "0{ds}00;[Red]0{ds}00"; 
        public static readonly string TestDecimalNumberNegativeFormat2 = @"0{ds}00_ ;[Red]-0{ds}00\ ";

        #endregion


        #region Set Value and Assert Number Format

        [TestMethod]
        public void SetValueThatRequiresNumberFormatWholeNumber()
        {
            SetValueAndAssertAppliedFormat("1{gs}234", TestWholeNumberFormat, "1234", "1{gs}234");
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatLargeWholeNumber()
        {
            SetValueAndAssertAppliedFormat("1{gs}234{gs}567", TestWholeNumberFormat, "1234567", "1{gs}234{gs}567", cultures: NonPunjabiCultures);
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatLargeWholeNumberPaIn()
        {
            SetValueAndAssertAppliedFormat("1{gs}234{gs}567", TestWholeNumberFormat, "1234567", "12{gs}34{gs}567", cultures: PaINCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatTest()
        {
            SetValueAndAssertAppliedFormat("1{gs}234{ds}567", TestNumberFormat, "1234{ds}567", "1{gs}234{ds}57");
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatTest2()
        {
            SetValueAndAssertAppliedFormat("123{gs}456{ds}7899", TestNumberFormat, 123456.7899, "123{gs}456{ds}79", cultures: NonPunjabiCultures);
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatTest2PaIn()
        {
            SetValueAndAssertAppliedFormat("123{gs}456{ds}7899", TestNumberFormat, 123456.7899, "1{gs}23{gs}456{ds}79", cultures: PaINCulture);
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatTest3()
        {
            SetValueAndAssertAppliedFormat("100{gs}000{gs}000{gs}000", TestWholeNumberFormat, 100000000000, "100{gs}000{gs}000{gs}000", cultures: NonPunjabiCultures);
        }

        [TestMethod]
        public void SetValueThatRequiresNumberFormatTest3PaIn()
        {
            SetValueAndAssertAppliedFormat("100{gs}000{gs}000{gs}000", TestWholeNumberFormat, 100000000000, "1{gs}00{gs}00{gs}00{gs}00{gs}000", cultures: PaINCulture);
        }

        #endregion


        #region Set Value & Format and Assert Number Format Applied Correctly

        [TestMethod]
        public void SetValueThenNumberFormat1()
        {
            SetValueThenFormatAppliedCorrectly(TestWholeNumberFormat, "15963", typeof(NumberCellValue), "15963", "15{gs}963", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat2()
        {
            SetValueThenFormatAppliedCorrectly(TestWholeNumberFormat, "{ds}123", typeof(NumberCellValue), "0{ds}123", "0", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat3()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberFormat, "{ds}123", typeof(NumberCellValue), "0{ds}123", "0{ds}12", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat4()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberFormat, "123456", typeof(NumberCellValue), "123456", "123456{ds}00", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat5()
        {
            SetValueThenFormatAppliedCorrectly(TestNumberFormat, "123456", typeof(NumberCellValue), "123456", "123{gs}456{ds}00", null, null, false, cultures: NonPunjabiCultures);
        }

        [TestMethod]
        public void SetValueThenNumberFormat5PaIn()
        {
            SetValueThenFormatAppliedCorrectly(TestNumberFormat, "123456", typeof(NumberCellValue), "123456", "1{gs}23{gs}456{ds}00", null, null, false, cultures: PaINCulture);
        }

        [TestMethod]
        public void SetValueThenNumberFormat6()
        {
            SetValueThenFormatAppliedCorrectly(TestNumberFormat, "0{ds}123", typeof(NumberCellValue), "0{ds}123", "0{ds}12", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat7()
        {
            SetValueThenFormatAppliedCorrectly(TestNumberFormat, "123456{ds}123", typeof(NumberCellValue), "123456{ds}123", 
                "123{gs}456{ds}12", null, null, false, cultures: NonPunjabiCultures);
        }

        [TestMethod]
        public void SetValueThenNumberFormat7PaIn()
        {
            SetValueThenFormatAppliedCorrectly(TestNumberFormat, "123456{ds}123", typeof(NumberCellValue), "123456{ds}123",
                "1{gs}23{gs}456{ds}12", null, null, false, cultures: PaINCulture);
        }

        [TestMethod]
        public void SetValueThenNumberFormat9()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat, "{ds}123", typeof(NumberCellValue), "0{ds}123", "0{ds}12", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat10()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat, "123456", typeof(NumberCellValue), "123456", "123456{ds}00", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat11()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat, "-0{ds}123", typeof(NumberCellValue), "-0{ds}123", "0{ds}12", Colors.Red, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat12()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat, "-123456", typeof(NumberCellValue), "-123456", "123456{ds}00", Colors.Red, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat13()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat, "-123456{ds}123", typeof(NumberCellValue), "-123456{ds}123",
                "123456{ds}12", Colors.Red, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat14()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat2, "{ds}123", typeof(NumberCellValue), "0{ds}123", "0{ds}12 ", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat15()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat2, "123456", typeof(NumberCellValue), "123456", "123456{ds}00 ", null, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat16()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat2, "-0{ds}123", typeof(NumberCellValue), "-0{ds}123", "-0{ds}12 ", Colors.Red, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat17()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat2, "-123456", typeof(NumberCellValue), "-123456", "-123456{ds}00 ", Colors.Red, null, false);
        }

        [TestMethod]
        public void SetValueThenNumberFormat18()
        {
            SetValueThenFormatAppliedCorrectly(TestDecimalNumberNegativeFormat2, "-123456{ds}123", typeof(NumberCellValue), "-123456{ds}123",
                "-123456{ds}12 ", Colors.Red, null, false);
        }

        #endregion


        #region Set Format & Value and Assert Number Format Applied Correctly

        [TestMethod]
        public void SetFormatThenValueNumberFormat()
        {
            SetFormatThenValueAppliedCorrectly(TestWholeNumberFormat, "15963", typeof(NumberCellValue), "15963", "15{gs}963", null, null, false);
        }

        [TestMethod]
        public void SetFormatThenValueNumberThatSetsFormat()
        {
            SetFormatThenValueAppliedCorrectly(TestDecimalNumberFormat, "1{gs}234", typeof(NumberCellValue), "1234", "1234{ds}00", null, null, false);
        }

        [TestMethod]
        public void SetFormatThenValueFractionValue()
        {
            SetFormatThenValueAppliedCorrectly(TestDecimalNumberFormat, "1 1/2", typeof(NumberCellValue), "1{ds}5", "1{ds}50", null, null, false);
        }

        [TestMethod]
        public void SetFormatThenValueTextValue()
        {
            string cellValueInput = "text";
            SetFormatThenValueAppliedCorrectly(TestDecimalNumberFormat, cellValueInput, typeof(TextCellValue), cellValueInput, 
                cellValueInput, null, null, false);
        }

        [TestMethod]
        public void SetFormatThenValueFunctionValue()
        {
            string cellValueInput = "=SUM(3)";
            SetFormatThenValueAppliedCorrectly(TestDecimalNumberFormat, cellValueInput, typeof(FormulaCellValue), cellValueInput, 
                "3{ds}00", null, null, false);
        }

        [TestMethod]
        public void SetNumberFormatThenCurrencyValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestDecimalNumberFormat, "$1234", TestDecimalNumberFormat, "1234", "1234{ds}00",
                "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetNumberFormatThenDateValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestDecimalNumberFormat, "10/6/1987", TestDecimalNumberFormat, "32056", "32056{ds}00",
                "m/d/yyyy", "10/6/1987", "10/6/1987", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetNumberFormatThenTimeValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestDecimalNumberFormat, "12:30", TestDecimalNumberFormat, "0{ds}520833333333333", "0{ds}52",
                "h:mm", "12:30:00 PM", "12:30", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetNumberFormatThenPercentageValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(TestDecimalNumberFormat, "15%", TestDecimalNumberFormat, "0{ds}15", "0{ds}15",
                "0%", "15%", "15%", expectedType: typeof(NumberCellValue));
        }

        #endregion


        #region Set Format and Assert Category is Correct

        [TestMethod]
        public void SetFormatSimplePositive()
        {
            SetFormatAndAssertCategory("0{ds}00", FormatStringCategory.Number, NonArabicCultures);
        }

        [TestMethod]
        public void SetFormatNegativeRedEnUS()
        {
            SetFormatAndAssertCategory("0.00_);[Red](0.00)", FormatStringCategory.Number, EnUSCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedBgBG()
        {
            SetFormatAndAssertCategory(@"0,00;[Red]-0,00", FormatStringCategory.Number, BgBGCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedDeDE()
        {
            SetFormatAndAssertCategory(@"0,00;[Red]-0,00", FormatStringCategory.Number, DeDECulture);
        }

       // [TestMethod]
       // public void SetFormatNegativeRedItIT()
       // {
       //     SetFormatAndAssertCategory(@"0,00;[Red]-0,00", FormatStringCategory.Number, ItITCulture);
       // }

        [TestMethod]
        public void SetFormatNegativeRedEsES()
        {
            SetFormatAndAssertCategory(@"0,00;[Red]-0,00", FormatStringCategory.Number, EsESCulture);
        }

        [TestMethod]
        public void SetFormatNegativeRedPaIN()
        {
            SetFormatAndAssertCategory(@"0.00;[Red]-0.00", FormatStringCategory.Number, PaINCulture);
        }

        #endregion


        #region Old tests

        [TestMethod]
        public void NumberTest3Positive()
        {
            RunTestInMultipleCultures(() =>
              {
                  string appliedFormat = string.Format("{0}_);({0})", TestNumberFormat).ReplaceGroupDecimalSeparators();

                  Worksheet worksheet = TestHelper.CreateWorksheet();

                  worksheet.Cells[0, 0].SetFormat(new CellValueFormat(appliedFormat));
                  worksheet.Cells[0, 0].SetValue((123.4567).ToDefaultSpreadsheetCultureString());

                  ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                  Assert.IsTrue(cellValue is NumberCellValue);

                  CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                  CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
                  IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                  string infosText = cellValueFormatResult.InfosText;
                  bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                  Assert.AreEqual(2, infos.Count());

                  Assert.AreEqual(null, cellValueFormatResult.Foreground);
                  Assert.AreEqual(null, cellValueFormatResult.Condition);
                  Assert.AreEqual(null, cellValueFormatResult.Culture);
                  Assert.AreEqual(false, shouldExpand);

                  string expectedString = "123{ds}46".ReplaceGroupDecimalSeparators();
                  Assert.AreEqual(expectedString + ")", infosText);

                  int i = 0;
                  foreach (CellValueFormatResultItem info in infos)
                  {
                      if (i == 0)
                      {
                          Assert.AreEqual(true, info.ApplyFormat);
                          Assert.AreEqual(false, info.IsTransparent);
                          Assert.AreEqual(false, info.ShouldExpand);
                          Assert.AreEqual(expectedString, info.Text);
                      }

                      if (i == 1)
                      {
                          Assert.AreEqual(true, info.ApplyFormat);
                          Assert.AreEqual(true, info.IsTransparent);
                          Assert.AreEqual(false, info.ShouldExpand);
                          Assert.AreEqual(")", info.Text);
                      }
                      i++;
                  }
              }, new string[] {"en-US"});
        }

        [TestMethod]
        public void NumberTest3Negative()
        {
            RunTestInMultipleCultures(() =>
                {
                    string appliedFormat = string.Format("{0}_);({0})", TestNumberFormat).ReplaceGroupDecimalSeparators();

                    Worksheet worksheet = TestHelper.CreateWorksheet();

                    worksheet.Cells[0, 0].SetFormat(new CellValueFormat(appliedFormat));
                    worksheet.Cells[0, 0].SetValue((-123.4567).ToDefaultSpreadsheetCultureString());

                    ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                    Assert.IsTrue(cellValue is NumberCellValue);

                    CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                    CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
                    IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                    string infosText = cellValueFormatResult.InfosText;
                    bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                    Assert.AreEqual(1, infos.Count());

                    Assert.AreEqual(null, cellValueFormatResult.Foreground);
                    Assert.AreEqual(null, cellValueFormatResult.Condition);
                    Assert.AreEqual(null, cellValueFormatResult.Culture);
                    Assert.AreEqual(false, shouldExpand);
            
                    string expectedString = "(123{ds}46)".ReplaceGroupDecimalSeparators();
                    Assert.AreEqual(expectedString, infosText);

                    int i = 0;
                    foreach (CellValueFormatResultItem info in infos)
                    {
                        if (i == 0)
                        {
                            Assert.AreEqual(true, info.ApplyFormat);
                            Assert.AreEqual(false, info.IsTransparent);
                            Assert.AreEqual(false, info.ShouldExpand);
                            Assert.AreEqual(expectedString, info.Text);
                        }
                    }
                }, new string[] { "en-US" });
        }

        [TestMethod]
        public void NumberTest4Positive()
        {
            RunTestInMultipleCultures(() =>
              {
                  string appliedFormat = string.Format("{0}_);[Red]({0})", TestNumberFormat).ReplaceGroupDecimalSeparators();

                  Worksheet worksheet = TestHelper.CreateWorksheet();

                  worksheet.Cells[0, 0].SetFormat(new CellValueFormat(appliedFormat));
                  worksheet.Cells[0, 0].SetValue((123.4567).ToDefaultSpreadsheetCultureString());

                  ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                  Assert.IsTrue(cellValue is NumberCellValue);

                  CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                  CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
                  IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                  string infosText = cellValueFormatResult.InfosText;
                  bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                  Assert.AreEqual(2, infos.Count());

                  Assert.AreEqual(null, cellValueFormatResult.Foreground);
                  Assert.AreEqual(null, cellValueFormatResult.Condition);
                  Assert.AreEqual(null, cellValueFormatResult.Culture);
                  Assert.AreEqual(false, shouldExpand);

                  string expectedString = "123{ds}46".ReplaceGroupDecimalSeparators(); 
                  Assert.AreEqual(expectedString + ")", infosText);

                  int i = 0;
                  foreach (CellValueFormatResultItem info in infos)
                  {
                      if (i == 0)
                      {
                          Assert.AreEqual(true, info.ApplyFormat);
                          Assert.AreEqual(false, info.IsTransparent);
                          Assert.AreEqual(false, info.ShouldExpand);
                          Assert.AreEqual(expectedString, info.Text);
                      }

                      if (i == 1)
                      {
                          Assert.AreEqual(true, info.ApplyFormat);
                          Assert.AreEqual(true, info.IsTransparent);
                          Assert.AreEqual(false, info.ShouldExpand);
                          Assert.AreEqual(")", info.Text);
                      }
                      i++;
                  }
              }, new string[] { "en-US" });
        }

        [TestMethod]
        public void NumberTest4Negative()
        {
            RunTestInMultipleCultures(() =>
               {
                   string appliedFormat = string.Format("{0}_);[Red]({0})", TestNumberFormat).ReplaceGroupDecimalSeparators();

                   Worksheet worksheet = TestHelper.CreateWorksheet();

                   worksheet.Cells[0, 0].SetFormat(new CellValueFormat(appliedFormat));
                   worksheet.Cells[0, 0].SetValue((-123.4567).ToDefaultSpreadsheetCultureString());

                   ICellValue cellValue = worksheet.Cells[0, 0].GetValue().Value;

                   Assert.IsTrue(cellValue is NumberCellValue);

                   CellValueFormat format = worksheet.Cells[0, 0].GetFormat().Value;
                   CellValueFormatResult cellValueFormatResult = format.GetFormatResult(cellValue);
                   IEnumerable<CellValueFormatResultItem> infos = cellValueFormatResult.Infos;
                   string infosText = cellValueFormatResult.InfosText;
                   bool shouldExpand = cellValueFormatResult.ContainsExpandableInfo;

                   Assert.AreEqual(1, infos.Count());

                   Assert.AreEqual(Colors.Red, cellValueFormatResult.Foreground);
                   Assert.AreEqual(null, cellValueFormatResult.Condition);
                   Assert.AreEqual(null, cellValueFormatResult.Culture);
                   Assert.AreEqual(false, shouldExpand);

                   string expectedString = "(123{ds}46)".ReplaceGroupDecimalSeparators(); 
                   Assert.AreEqual(expectedString, infosText);

                   int i = 0;
                   foreach (CellValueFormatResultItem info in infos)
                   {
                       if (i == 0)
                       {
                           Assert.AreEqual(true, info.ApplyFormat);
                           Assert.AreEqual(false, info.IsTransparent);
                           Assert.AreEqual(false, info.ShouldExpand);
                           Assert.AreEqual(expectedString, info.Text);
                       }
                   }
               }, new string[] { "en-US" });
        }

        #endregion

    }
}
