﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Percent : CellValueFormatTestsBase
    {
        #region Constants

        public static readonly string PercentFormatNoDS = "0%";
        public static readonly string PercentFormatOneDS = "0{ds}0%";
        public static readonly string PercentFormatTwoDS = "0{ds}00%";
        public static readonly string PercentFormatFourDS = "0{ds}0000%";

        #endregion


        #region Set Value and Assert Percent Format

        [TestMethod]
        public void SetValueThatRequiresPercentFormatTest()
        {
            SetValueAndAssertAppliedFormat("13%", PercentFormatNoDS, "13%", "13%");
        }

        [TestMethod]
        public void SetValueThatRequiresPercentFormatTest1()
        {
            string cellValueString = "12{ds}1%";
            SetValueAndAssertAppliedFormat(cellValueString, PercentFormatTwoDS, cellValueString, "12{ds}10%");
        }

        [TestMethod]
        public void SetValueThatRequiresPercentFormatTest2()
        {
            string cellValueString = "{ds}123%";
            SetValueAndAssertAppliedFormat(cellValueString, PercentFormatTwoDS, "0{ds}123%", "0{ds}12%");
        }

        [TestMethod]
        public void SetValueThatRequiresPercentFormatTest3()
        {
            SetValueAndAssertAppliedFormat("0%", PercentFormatNoDS, PercentFormatNoDS, "0%");
        }

        [TestMethod]
        public void SetValueThatRequiresPercentFormatTest4()
        {
            string cellValueString = "1234{ds}56789%";
            SetValueAndAssertAppliedFormat(cellValueString, PercentFormatTwoDS, cellValueString, "1234{ds}57%");
        }

        #endregion


        #region Set Value & Format and Assert Percent Format Applied Correctly

        [TestMethod]
        public void SetValueThenPercentFormat1()
        {
            SetValueAndFormatAssertResult("15963", PercentFormatNoDS, PercentFormatNoDS, "1596300%", "1596300%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat2()
        {
            SetValueAndFormatAssertResult("15963", PercentFormatTwoDS, PercentFormatTwoDS, "1596300%", "1596300{ds}00%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat3()
        {
            SetValueAndFormatAssertResult("15963{ds}3456789", PercentFormatNoDS, PercentFormatNoDS, "1596334{ds}56789%", "1596335%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat4()
        {
            SetValueAndFormatAssertResult("15963{ds}3456789", PercentFormatOneDS, PercentFormatOneDS, "1596334{ds}56789%", "1596334{ds}6%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat5()
        {
            SetValueAndFormatAssertResult("15963{ds}3456789", PercentFormatTwoDS, PercentFormatTwoDS, "1596334{ds}56789%", "1596334{ds}57%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat6()
        {
            SetValueAndFormatAssertResult("15963{ds}3456789", PercentFormatFourDS, PercentFormatFourDS, "1596334{ds}56789%", "1596334{ds}5679%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat7()
        {
            SetValueAndFormatAssertResult("0{ds}56781234", PercentFormatNoDS, PercentFormatNoDS, "56{ds}781234%", "57%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat8()
        {
            SetValueAndFormatAssertResult("0{ds}56781234", PercentFormatOneDS, PercentFormatOneDS, "56{ds}781234%", "56{ds}8%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat9()
        {
            SetValueAndFormatAssertResult("0{ds}56781234", PercentFormatTwoDS, PercentFormatTwoDS, "56{ds}781234%", "56{ds}78%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat10()
        {
            SetValueAndFormatAssertResult("0{ds}56781234", PercentFormatFourDS, PercentFormatFourDS, "56{ds}781234%", "56{ds}7812%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat11()
        {
            SetValueAndFormatAssertResult("13", PercentFormatTwoDS, PercentFormatTwoDS, "1300%", "1300{ds}00%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat12()
        {
            SetValueAndFormatAssertResult("0{ds}3456789", PercentFormatFourDS, PercentFormatFourDS, "34{ds}56789%", "34{ds}5679%");
        }

        [TestMethod]
        public void SetValueThenPercentFormat13()
        {
            SetValueAndFormatAssertResult("string value", PercentFormatFourDS, PercentFormatFourDS, "string value", "string value", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetValueThenPercentFormat14()
        {
            SetValueAndFormatAssertResult("0 1/2", PercentFormatTwoDS, PercentFormatTwoDS, "50%", "50{ds}00%");
        }

        #endregion


        #region Set Format & Value and Assert Percent Format Applied Correctly

        [TestMethod]
        public void SetPercentFormatThenValueResultItemTest()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("34{ds}5679%", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("34{ds}5679%", items);

            TestScientific("0{ds}0000%", "0{ds}3456789", info);
        }

        [TestMethod]
        public void SetPercentFormatThenNumberValue()
        {
            SetFormatAndValueAssertResult(PercentFormatTwoDS, "1234{ds}56789", PercentFormatTwoDS, "123456{ds}789%", "123456{ds}79%");
        }

        [TestMethod]
        public void SetPercentFormatThenNumberThatSetsFormatValue()
        {
            SetFormatAndValueAssertResult(PercentFormatTwoDS, "1234{ds}56789%", PercentFormatTwoDS, "1234{ds}56789%", "1234{ds}57%");
        }

        [TestMethod]
        public void SetPercentFormatThenFractionValue()
        {
            SetFormatAndValueAssertResult(PercentFormatTwoDS, "0 1/2", PercentFormatTwoDS, "50%", "50{ds}00%");
        }

        [TestMethod]
        public void SetPercentFormatThenTextValue()
        {
            SetFormatAndValueAssertResult(PercentFormatTwoDS, "any string", PercentFormatTwoDS, "any string", "any string", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetPercentFormatThenFunctionValue()
        {
            string cellValueInput = "=SUM(1{ls} 2)";
            SetFormatAndValueAssertResult(PercentFormatTwoDS, cellValueInput, PercentFormatTwoDS, cellValueInput, "300{ds}00%", expectedType: typeof(FormulaCellValue));
        }

        [TestMethod]
        public void SetPercentFormatThenFunctionThatSetsFormat()
        {
            string cellValueInput = "=RATE(48{ls} -200{ls} 8000{ls} 0{ls} 0)";
            SetFormatAndValueAssertResult(PercentFormatOneDS, cellValueInput, PercentFormatOneDS, cellValueInput, "0{ds}8%", expectedType: typeof(FormulaCellValue));
        }

        [TestMethod]
        public void SetPercentFormatThenNumberValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(PercentFormatNoDS, "1{gs}234", PercentFormatNoDS, "123400%", "123400%",
                "#{gs}##0", "1234", "1{gs}234", expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetPercentFormatThenCurrencyValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(PercentFormatNoDS, "$1234", PercentFormatNoDS, "123400%", "123400%",
                "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetPercentFormatThenDateValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(PercentFormatNoDS, "10/6/1987", PercentFormatNoDS, "3205600%", "3205600%",
                "m/d/yyyy", "10/6/1987", "10/6/1987", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetAccountingFormatThenTimeValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult(PercentFormatNoDS, "12:30", PercentFormatNoDS, "52.0833333333333%", "52%",
                "h:mm", "12:30:00 PM", "12:30", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        #endregion
    }
}
