﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Scientific : CellValueFormatTestsBase
    {
        #region Constants

        public static readonly string Scientific0DS1Format = "0E+0";
        public static readonly string Scientific0DSFormat = "0E+00";
        public static readonly string Scientific1DSFormat = "0{ds}0E+00";
        public static readonly string Scientific2DSFormat = "0{ds}00E+00";
        public static readonly string Scientific5DSFormat = "0{ds}00000E+00";
        public static readonly string Scientific7DSFormat = "0{ds}0000000E+00";
       
        #endregion


        #region Set Value and Assert Scientific Format is Applied

        [TestMethod]
        public void SimpleScientificTest2()
        {
            SetValueAndAssertAppliedFormat("123123123123123", string.Empty, 123123123123123, "1{ds}23123E+14");
        }

        [TestMethod]
        public void SimpleScientificTest3()
        {
            SetValueAndAssertAppliedFormat("100000000000", string.Empty, 100000000000, "1E+11");
        }

        [TestMethod]
        public void SimpleScientificTest4()
        {
            SetValueAndAssertAppliedFormat("110000000000", string.Empty, 110000000000, "1{ds}1E+11");
        }

        [TestMethod]
        public void SimpleScientificTest5()
        {
            SetValueAndAssertAppliedFormat("99999999999", string.Empty, 99999999999, "99999999999");
        }

        [TestMethod]
        public void SimpleScientificTest6()
        {
            SetValueAndAssertAppliedFormat("123456789123", string.Empty, 123456789123, "1{ds}23457E+11");
        }

        [TestMethod]
        public void SimpleScientificTest7()
        {
            SetValueAndAssertAppliedFormat("123000000000", string.Empty, 123000000000, "1{ds}23E+11");
        }

        [TestMethod]
        public void SimpleScientificTest8()
        {
            SetValueAndAssertAppliedFormat("123400000000", string.Empty, 123400000000, "1{ds}234E+11");
        }

        [TestMethod]
        public void SimpleScientificTest9()
        {
            SetValueAndAssertAppliedFormat("123450000000", string.Empty, 123450000000, "1{ds}2345E+11");
        }

        [TestMethod]
        public void SimpleScientificTest10()
        {
            SetValueAndAssertAppliedFormat("123456000000", string.Empty, 123456000000, "1{ds}23456E+11");
        }

        #endregion


        #region Set Format, Then Value and Assert Format Applied Correctly

        [TestMethod]
        public void SimpleScientificTest()
        {
            SetFormatThenValueAppliedCorrectly(Scientific2DSFormat, "1200000", typeof(NumberCellValue), "1200000", "1{ds}20E+06", null,
                null, false);
        }

        [TestMethod]
        public void SimpleScientificTest1()
        {
            SetFormatThenValueAppliedCorrectly(Scientific2DSFormat, "12320000000000", typeof(NumberCellValue), "12320000000000", 
                "1{ds}23E+13", null, null, false);
        }

        [TestMethod]
        public void Scientific0DS1WholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific0DS1Format, "123456789", typeof(NumberCellValue), "123456789", "1E+8", null,
                null, false);
        }

        [TestMethod]
        public void Scientific0DSWholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific0DSFormat, "123456789", typeof(NumberCellValue), "123456789", "1E+08", null,
                null, false);
        }

        [TestMethod]
        public void Scientific1DSWholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific1DSFormat, "123456789", typeof(NumberCellValue), "123456789", "1{ds}2E+08", null,
                null, false);
        }

        [TestMethod]
        public void Scientific2DSWholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific2DSFormat, "123456789", typeof(NumberCellValue), "123456789", "1{ds}23E+08", null,
                null, false);
        }

        [TestMethod]
        public void Scientific5DSWholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific5DSFormat, "123456789", typeof(NumberCellValue), "123456789", "1{ds}23457E+08", null,
                null, false);
        }

        [TestMethod]
        public void Scientific7DSWholeNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific7DSFormat, "123456789", typeof(NumberCellValue), "123456789", "1{ds}2345679E+08", null,
                null, false);
        }

        [TestMethod]
        public void Scientific0DS1DecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific0DS1Format, "0{ds}000123456789", typeof(NumberCellValue), "0{ds}000123456789", "1E-4", null,
                null, false);
        }

        [TestMethod]
        public void Scientific0DSDecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific0DSFormat, "0{ds}000123456789", typeof(NumberCellValue), "0{ds}000123456789", "1E-04", null,
                null, false);
        }

        [TestMethod]
        public void Scientific1DSDecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific1DSFormat, "0{ds}000123456789", typeof(NumberCellValue),
                "0{ds}000123456789", "1{ds}2E-04", null, null, false);
        }

        [TestMethod]
        public void Scientific2DSDecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific2DSFormat, "0{ds}000123456789", typeof(NumberCellValue),
                "0{ds}000123456789", "1{ds}23E-04", null, null, false);
        }

        [TestMethod]
        public void Scientific5DSDecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific5DSFormat, "0{ds}000123456789", typeof(NumberCellValue),
                "0{ds}000123456789", "1{ds}23457E-04", null, null, false);
        }

        [TestMethod]
        public void Scientific7DSDecimalNumber()
        {
            SetFormatThenValueAppliedCorrectly(Scientific7DSFormat, "0{ds}000123456789", typeof(NumberCellValue),
                "0{ds}000123456789", "1{ds}2345679E-04", null, null, false);
        }

        #endregion
    }
}
