﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Special : CellValueFormatTestsBase
    {
        [TestMethod]
        public void TestSpecialZipCode()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("123456789", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("123456789", items);
            TestScientific("00000", "123456789", info);
        }

        [TestMethod]
        public void TestSpecialZipCodePlus4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("12345-6789", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("12345-6789", items);
            TestScientific("00000-0000", "123456789", info);
        }

        [TestMethod]
        public void TestSpecialPhoneNumber()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("(12) 345-6789", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("(12) 345-6789", items);
            TestScientific("[<=9999999]###-####;(###) ###-####", "123456789", info);
        }

        [TestMethod]
        public void TestSpecialPhoneNumber1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("123-4567", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("123-4567", items);
            TestScientific("[<=9999999]###-####;(###) ###-####", "1234567", info);
        }

        [TestMethod]
        public void TestSpecialPhoneNumber2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("(123456) 789-0123", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("(123456) 789-0123", items);
            TestScientific("[<=9999999]###-####;(###) ###-####", "1234567890123", info);
        }

        [TestMethod]
        public void TestSpecialSocialSecurityNumber()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("123-45-6789", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("123-45-6789", items);
            TestScientific("000-00-0000", "123456789", info);
        }

        [TestMethod]
        public void TestSpecialSocialSecurityNumber1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("000-00-0001", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("000-00-0001", items);
            TestScientific("000-00-0000", "1{ds}23456789", info);
        }

        [TestMethod]
        public void TestSpecialSocialSecurityNumber2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1234567-89-0123", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("1234567-89-0123", items);
            TestScientific("000-00-0000", "1234567890123", info);
        }

        [TestMethod]
        public void TestSpecial()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("012-345 67 89", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("012-345 67 89", items);
            TestScientific("0##-000\" \"00\" \"00", "123456789", info);
        }

        [TestMethod]
        public void TestSpecial1()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("  1 23 456 789", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("  1 23 456 789", items);
            TestScientific("[>=3000000000000]#\" \"##\" \"##\" \"##\" \"###\" \"###\" | \"##;#\" \"##\" \"##\" \"##\" \"###\" \"###", 
                "123456789", info);
        }

        [TestMethod]
        public void TestSpecial2()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 23 45 67 890 123", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("1 23 45 67 890 123", items);
            TestScientific("[>=3000000000000]#\" \"##\" \"##\" \"##\" \"###\" \"###\" | \"##;#\" \"##\" \"##\" \"##\" \"###\" \"###",
                "1234567890123", info);
        }

        [TestMethod]
        public void TestSpecial3()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem(" 3 00 00 000 000 | 01", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo(" 3 00 00 000 000 | 01", items);
            TestScientific("[>=3000000000000]#\" \"##\" \"##\" \"##\" \"###\" \"###\" | \"##;#\" \"##\" \"##\" \"##\" \"###\" \"###",
                "3000000000001", info);
        }

        [TestMethod]
        public void TestSpecial4()
        {
            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("1 23 45 67 890 123 | 45", false, false, true));
            FormatPatternInfo info = new FormatPatternInfo("1 23 45 67 890 123 | 45", items);
            TestScientific("[>=3000000000000]#\" \"##\" \"##\" \"##\" \"###\" \"###\" | \"##;#\" \"##\" \"##\" \"##\" \"###\" \"###",
                "123456789012345", info);
        }

        [TestMethod]
        public void SetValueThenFormatCoincidingWithTimeCharacter()
        {
            this.SetValueAndFormatAssertResult("1{gs}234{ds}567", "#{ds}00\" m\";-#{ds}00\" m\"", "#{ds}00\" m\";-#{ds}00\" m\"",
                "1234{ds}567", "1234{ds}57 m", cultures: NlNLCulture);
        }
    }
}
