﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Text : CellValueFormatTestsBase
    {
        #region Fields

        public static readonly string TextFormat = "@";

        #endregion


        #region Set Value, then Apply Text Format and Assert Format Applied Correctly

        [TestMethod]
        public void SetNumberAndTextFormat()
        {
            SetValueAndFormatAssertResult("1{gs}234", TextFormat, TextFormat, "1234", "1234", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetDecimalNumberAndTextFormat()
        {
            SetValueAndFormatAssertResult("1{gs}234{ds}56", TextFormat, TextFormat, "1234{ds}56", "1234{ds}56",expectedType:  typeof(TextCellValue));
        }

        [TestMethod]
        public void SetNumberValueSetTextFormat()
        {
            SetValueAndFormatAssertResult("123", TextFormat, TextFormat, "123", "123",expectedType:  typeof(TextCellValue));
        }

        [TestMethod]
        public void SetFractionAndTextFormat()
        {
            SetValueAndFormatAssertResult("1 1/2", TextFormat, TextFormat, "1{ds}5", "1{ds}5", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetTextAndTextFormat()
        {
            SetValueAndFormatAssertResult("text", TextFormat, TextFormat, "text", "text", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetPercentAndTextFormat()
        {
            SetValueAndFormatAssertResult("0{ds}12%", TextFormat, TextFormat, "0{ds}0012", "0{ds}0012", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void SetFunctionAndTextFormat()
        {
            SetValueAndFormatAssertResult("=ABS(-10)", TextFormat, TextFormat, "=ABS(-10)", "=ABS(-10)", expectedType: typeof(TextCellValue));
        }
        
        #endregion


        #region Apply Text Format, then Set Value and Assert Format Applied Correctly

        [TestMethod]
        public void SimpleTextTest()
        {
            SetFormatAndValueAssertResult(@"@\s\a\s", "123", @"@\s\a\s", "123", "123sas", expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFormatAndSetNumberValue()
        {
            string cellInput = "1{gs}234";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFromatAndSetDecimalNumber()
        {
            string cellInput = "1{gs}234{ds}56";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFromatAndSetNumberValue()
        {
            string cellInput = "123";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFromatAndSetFraction()
        {
            string cellInput = "1 1/2";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFromatAndSetText()
        {
            string cellInput = "text";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFromatAndSetPercent()
        {
            string cellInput = "0{ds}12%";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue));
        }

        [TestMethod]
        public void ApplyTextFormatAndSetCurrencyEnUS()
        {
            string cellInput = "$123";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue),
                cultures: EnUSCulture);
        }

        [TestMethod]
        public void ApplyTextFormatAndSetCurrencyDeDE()
        {
            string cellInput = "123 €";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue),
                cultures: DeDECulture);
        }

        [TestMethod]
        public void ApplyTextFormatAndSetCurrencyBgBG()
        {
            string cellInput = "123 лв.";
            SetFormatAndValueAssertResult(TextFormat, cellInput, TextFormat, cellInput, cellInput, expectedType: typeof(TextCellValue),
                cultures: BgBGCulture);
        }

        #endregion


        #region Set Value That Requires Format and Set Value To Text

        [TestMethod]
        public void SetPercentValueSetTextValue()
        {
            SetValueTwiceAssertAppliedFormat("13%", "0%", typeof(NumberCellValue), "asd", "asd", "asd", typeof(TextCellValue));
        }

        [TestMethod]
        public void SetFractionValueSetTextValue()
        {
            SetValueTwiceAssertAppliedFormat("1 1/2", CellValueFormatTestsBase.FractionShortFormat, typeof(NumberCellValue), "asd", "asd", "asd", typeof(TextCellValue));
        }

        [TestMethod]
        public void SetScientificValueSetTextValue()
        {
            SetValueTwiceAssertAppliedFormat("123E+3", "0{ds}00E+00", typeof(NumberCellValue), "asd", "asd", "asd", typeof(TextCellValue));
        }

        [TestMethod]
        public void SetCurrencyValueSetTextValue()
        {
            SetValueTwiceAssertAppliedFormat("$123", "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", typeof(NumberCellValue), "asd", "asd", "asd",
                typeof(TextCellValue), new string[] { "en-US" });
        }

        #endregion


        #region Set Text Format Then Set Value That Requires Format

        [TestMethod]
        public void SetTextFormatThenGeneralValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "1234", "@", "1234", "1234",
                "", "1234", "1234", expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetTextFormatThenNumberValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "1{gs}234", "@", "1234", "1234",
                "#{gs}##0", "1234", "1{gs}234", NonArabicCultures, typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetTextFormatThenCurrencyValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "$1234", "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)",
                "\"$\"#{gs}##0_);[Red](\"$\"#{gs}##0)", "1234", "$1{gs}234)", cultures:EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetTextFormatThenDateValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "06/10/1987", "m/d/yyyy", "6/10/1987", "6/10/1987",
                "m/d/yyyy", "6/10/1987", "6/10/1987", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetTextFormatThenPercentageValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "15%", "0%", "15%", "15%",
                "0%", "15%", "15%", cultures: EnUSCulture, expectedType: typeof(NumberCellValue));
        }

        [TestMethod]
        public void SetTextFormatThenScientificValueOnRangeWithIndeterminateFormat()
        {
            SetFormatAndValueAsRangeAssertResult("@", "1{ds}00E+00", "0{ds}00E+00", "1", "1{ds}00E+00",
                "0{ds}00E+00", "1", "1{ds}00E+00", expectedType: typeof(NumberCellValue));
        }

        #endregion


        #region Set Text Format and Assert Category is Correct

        [TestMethod]
        public void SetTextFormatAndAssertCategory()
        {
            SetFormatAndAssertCategory("@", FormatStringCategory.Text);
        }

        #endregion
    }
}
