﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Formatting.FormatStrings;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats
{
    [TestClass]
    public class CellValueFormatTests_Time : CellValueFormatTestsBase
    {
        #region Fields

        public static readonly string TimeFormatEnUS1 = "h:mm";
        public static readonly string TimeFormatEnUS2 = "h:mm:ss";
        public static readonly string TimeFormatEnUS3 = "mm:ss.0";
        public static readonly string TimeFormat1AmpmEnUS = "h:mm AM/PM";
        public static readonly string TimeFormat2AmpmEnUS = "h:mm:ss AM/PM";
        public static readonly string DateTimeFormatEnUS1 = "m/d/yyyy h:mm";

        #endregion


        #region Set value that requires time format

        [TestMethod]
        public void SimpleTimeTest()
        {
            SetValueAndAssertAppliedFormat("3:44", TimeFormatEnUS1, "3:44:00 AM", "3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest1()
        {
            SetValueAndAssertAppliedFormat("3:44:55", TimeFormatEnUS2, "3:44:55 AM", "3:44:55", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest2()
        {
            SetValueAndAssertAppliedFormat("3:44.567", TimeFormatEnUS3, "12:03:45 AM", "03:44.6", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest3()
        {
            SetValueAndAssertAppliedFormat("3:44:55.678", TimeFormatEnUS3, "3:44:56 AM", "44:55.7", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest4()
        {
            SetValueAndAssertAppliedFormat("3:44 AM", TimeFormat1AmpmEnUS, "3:44:00 AM", "3:44 AM", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest5()
        {
            SetValueAndAssertAppliedFormat("3:44 PM", TimeFormat1AmpmEnUS, "3:44:00 PM", "3:44 PM", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest6()
        {
            SetValueAndAssertAppliedFormat("3:44:55 AM", TimeFormat2AmpmEnUS, "3:44:55 AM", "3:44:55 AM", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest7()
        {
            SetValueAndAssertAppliedFormat("3:44:55 PM", TimeFormat2AmpmEnUS, "3:44:55 PM", "3:44:55 PM", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest8()
        {
            SetValueAndAssertAppliedFormat("3:44.567 AM", TimeFormatEnUS3, "12:03:45 AM", "03:44.6", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest9()
        {
            SetValueAndAssertAppliedFormat("3:44.567 PM", TimeFormatEnUS3, "12:03:45 PM", "03:44.6", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest10()
        {
            SetValueAndAssertAppliedFormat("3:44:55.678 AM", TimeFormatEnUS3, "3:44:56 AM", "44:55.7", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest11()
        {
            SetValueAndAssertAppliedFormat("3:44:55.678 PM", TimeFormatEnUS3, "3:44:56 PM", "44:55.7", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest12()
        {
            SetValueAndAssertAppliedFormat("3:44.56 AM", TimeFormatEnUS3, "12:03:45 AM", "03:44.6", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest13()
        {
            SetValueAndAssertAppliedFormat("3:44.56 PM", TimeFormatEnUS3, "12:03:45 PM", "03:44.6", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest14()
        {
            SetValueAndAssertAppliedFormat("3:44.5", TimeFormatEnUS3, "12:03:45 AM", "03:44.5", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest15()
        {
            SetValueAndAssertAppliedFormat("3:44.5 PM", TimeFormatEnUS3, "12:03:45 PM", "03:44.5", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest16()
        {
            SetValueAndAssertAppliedFormat("6/10/1987 3:44 AM", DateTimeFormatEnUS1, "6/10/1987 3:44:00 AM", "6/10/1987 3:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest17()
        {
            SetValueAndAssertAppliedFormat("6/10/1987 3:44 PM", DateTimeFormatEnUS1, "6/10/1987 3:44:00 PM", "6/10/1987 15:44", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest18()
        {
            SetValueAndAssertAppliedFormat("15:30", TimeFormatEnUS1, "3:30:00 PM", "15:30", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest19()
        {
            SetValueAndAssertAppliedFormat("15:33:55", TimeFormatEnUS2, "3:33:55 PM", "15:33:55", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest20()
        {
            SetValueAndAssertAppliedFormat("6/10/87 15:33", DateTimeFormatEnUS1, "6/10/1987 3:33:00 PM", "6/10/1987 15:33", EnUSCulture);
        }

        [TestMethod]
        public void SimpleTimeTest22()
        {
            SetValueAndAssertAppliedFormat("6/10/1987 15:44 PM", DateTimeFormatEnUS1, "6/10/1987 3:44:00 PM", "6/10/1987 15:44", EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormatMidnight()
        {
            SetValueAndAssertAppliedFormat("00:00", TimeFormatEnUS1, "12:00:00 AM", "0:00", EnUSCulture);
        }

        [TestMethod]
        public void TestDateFormatMidnightPlusSeconds()
        {
            SetValueAndAssertAppliedFormat("00:00:00", TimeFormatEnUS2, "12:00:00 AM", "0:00:00", EnUSCulture);
        }

        #endregion


        #region CellFormatResultItem Tests

        [TestMethod]
        public void TestTime1()
        {
            string format = "[$-F400]h:mm:ss AM/PM";
            string expectedString = "3:44:56 AM";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime2()
        {
            string format = "h:mm;@";
            string expectedString = "3:44";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime3()
        {
            string format = "[$-409]h:mm AM/PM;@";
            string expectedString = "3:44 AM";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime4()
        {
            string format = "h:mm:ss;@";
            string expectedString = "3:44:56";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue), 1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime5()
        {
            string format = "mm:ss{ds}0;@";
            string expectedString = "44:55.7";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime6()
        {
            string format = "[h]:mm:ss;@".ReplaceGroupDecimalSeparators();
            string expectedString = string.Format("766515:44:56");

            List<CellValueFormatResultItem> items = new List<CellValueFormatResultItem>();
            items.Add(new CellValueFormatResultItem("766515", false, false, true));
            items.Add(new CellValueFormatResultItem(":44:56", false, false, true));
            FormatPatternInfo formatInfo = new FormatPatternInfo("766515:44:56", items);

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                2, true, false, false, expectedString, EnUSCulture, formatInfo);
        }

        [TestMethod]
        public void TestTime7()
        {
            string format = "[$-409]m/d/yy h:mm AM/PM;@";
            string expectedString = "6/10/87 3:44 AM";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime8()
        {
            string format = "m/d/yy h:mm;@";
            string expectedString = "6/10/87 3:44";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        //[TestMethod]
        //public void TestTime9()
        //{
        //    string format = "[m]:ss;@";
        //    string expectedString = string.Format(":{0}", TestHelper.DefaultDate.Second);

        //    AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
        //        2, true, false, false, "45996704", expectedString);
        //}

        [TestMethod]
        public void TestTime10()
        {
            string format = "[s];@";
            string expectedString = "2759456696";

            AssertDateFormatAppliedCorrectly(format, DefaultDateString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime101()
        {
            string format = "[s];@";
            string expectedString = "26784";

            AssertDateFormatAppliedCorrectly(format, "7:26:24", typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTime11()
        {
            string afternoonTimeString = "6/10/1987 16:44:55.678";
            string format = "h:mm:ss tt";
            string expectedString = "4:44:56 PM";

            AssertDateFormatAppliedCorrectly(format, afternoonTimeString, typeof(NumberCellValue),
                1, true, false, false, expectedString, EnUSCulture);
        }

        [TestMethod]
        public void TestTimeWithNegativeDoubleValue()
        {
            string format = "[$-409]h:mm AM/PM;@";
            string expectedString = "#";

            AssertDateFormatAppliedCorrectly(format, "-1", typeof(NumberCellValue),
                1, true, false, true, expectedString, EnUSCulture);
        }

        #endregion
    }
}
