﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Model.Drawing.Charts;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Charts;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Charts
{
    [TestClass]
    public class ChartFactoryTests
    {
        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRowAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            (seriesGroup as BarSeriesGroup).BarDirection = BarDirection.Bar;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRowAsPieChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Pie);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new PieSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));
            expectedChart.SeriesGroups.Add(seriesGroup);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRowAsBubbleChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(
                worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bubble);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BubbleSeriesGroup();
            seriesGroup.Series.AddBubble(
                     new NumericChartData(Enumerable.Range(1, 4).Select(s => (double)s)),
                     new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)),
                     new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)),
                     null);

            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new ValueAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeNumbersAndTextLeftColumnAsColumnChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Column);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)));
            (seriesGroup as BarSeriesGroup).BarDirection = BarDirection.Column;
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithNumbersOnlyAsLineChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 1, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Line);

            DocumentChart expectedChart = new DocumentChart();

            var numericChartData = new double[] { 1, 2, 3, 4 };

            SeriesGroup seriesGroup = new LineSeriesGroup();
            seriesGroup.Series.Add(new NumericChartData(numericChartData), new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)));
            seriesGroup.Series.Add(new NumericChartData(numericChartData), new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithDatesInLeftColumnAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 0, 2, 0].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 0, 3, 0].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 0, 4, 0].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new DateAxis();
            horizontalAxis.NumberFormat = FormatHelper.ShortDatePattern;
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithUnstructuredDataNumbersMixedWithTextAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("1");
            worksheet.Cells[2, 0, 2, 0].SetValue("text 1");
            worksheet.Cells[3, 0, 3, 0].SetValue("2");
            worksheet.Cells[4, 0, 4, 0].SetValue("text 2");

            worksheet.Cells[1, 1, 1, 1].SetValue("text 5");
            worksheet.Cells[2, 1, 2, 1].SetValue("3");
            worksheet.Cells[3, 1, 3, 1].SetValue("4");
            worksheet.Cells[4, 1, 4, 1].SetValue("5");

            worksheet.Cells[1, 2, 1, 2].SetValue("4");
            worksheet.Cells[2, 2, 2, 2].SetValue("text 4");
            worksheet.Cells[3, 2, 3, 2].SetValue("5");
            worksheet.Cells[4, 2, 4, 2].SetValue("5");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 2, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(3, 1, 3, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(3, 0, 3, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 2, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(4, 1, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(4, 0, 4, 0))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithUnstructuredDataNumbersMixedWithDatesAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("1");
            worksheet.Cells[2, 0, 2, 0].SetValue("3");
            worksheet.Cells[3, 0, 3, 0].SetValue("2");
            worksheet.Cells[4, 0, 4, 0].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 1, 1, 1].SetValue("3");
            worksheet.Cells[2, 1, 2, 1].SetValue("3");
            worksheet.Cells[3, 1, 3, 1].SetValue("4");
            worksheet.Cells[4, 1, 4, 1].SetValue("5");

            worksheet.Cells[1, 2, 1, 2].SetValue("4");
            worksheet.Cells[2, 2, 2, 2].SetValue(new DateTime(2018, 1, 1));
            worksheet.Cells[3, 2, 3, 2].SetValue("5");
            worksheet.Cells[4, 2, 4, 2].SetValue("5");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 2, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(3, 1, 3, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(3, 0, 3, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 2, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(4, 1, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(4, 0, 4, 0))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseEmptyRangeAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 3, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 3, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 3, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 3, 0)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 3, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithTextDataAtTheBottomAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[4, 0, 4, 0].SetValue("test");
            worksheet.Cells[4, 2, 4, 2].SetValue("test");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 3, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(4, 1, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(4, 0, 4, 0))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithDataJaggedAtTheTop_TitlesAreDeterminedAccordingToFirstCellofTheTopRowThatHasText()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[4, 0, 4, 0].SetValue("test");
            worksheet.Cells[5, 0, 5, 0].SetValue("test");
            worksheet.Cells[6, 0, 6, 0].SetValue("test");

            worksheet.Cells[3, 1, 3, 1].SetValue("test");
            worksheet.Cells[4, 1, 4, 1].SetValue("test");
            worksheet.Cells[5, 1, 5, 1].SetValue("test");
            worksheet.Cells[6, 1, 6, 1].SetValue("test");

            worksheet.Cells[2, 2, 2, 2].SetValue("test");
            worksheet.Cells[3, 2, 3, 2].SetValue("test");
            worksheet.Cells[4, 2, 4, 2].SetValue("test");
            worksheet.Cells[5, 2, 5, 2].SetValue("test");
            worksheet.Cells[6, 2, 6, 2].SetValue("test");

            worksheet.Cells[1, 3, 1, 3].SetValue("test");
            worksheet.Cells[2, 3, 2, 3].SetValue("test");
            worksheet.Cells[3, 3, 3, 3].SetValue("test");
            worksheet.Cells[4, 3, 4, 3].SetValue("test");
            worksheet.Cells[5, 3, 5, 3].SetValue("test");
            worksheet.Cells[6, 3, 6, 3].SetValue("test");

            worksheet.Cells[0, 4, 0, 4].SetValue("test");
            worksheet.Cells[1, 4, 1, 4].SetValue("test");
            worksheet.Cells[2, 4, 2, 4].SetValue("test");
            worksheet.Cells[3, 4, 3, 4].SetValue("test");
            worksheet.Cells[4, 4, 4, 4].SetValue("test");
            worksheet.Cells[5, 4, 5, 4].SetValue("test");
            worksheet.Cells[6, 4, 6, 4].SetValue("test");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 6, 4), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 6, 3)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(1, 4, 6, 4)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 4, 0, 4))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithTextOnlyAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("text 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("text 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("text 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("text 4");

            worksheet.Cells[1, 1, 1, 1].SetValue("text 10");
            worksheet.Cells[2, 1, 2, 1].SetValue("text 20");
            worksheet.Cells[3, 1, 3, 1].SetValue("text 30");
            worksheet.Cells[4, 1, 4, 1].SetValue("text 40");

            worksheet.Cells[1, 2, 1, 2].SetValue("text 100");
            worksheet.Cells[2, 2, 2, 2].SetValue("text 200");
            worksheet.Cells[3, 2, 3, 2].SetValue("text 300");
            worksheet.Cells[4, 2, 4, 2].SetValue("text 400");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 3, 2)),
                                      new WorkbookFormulaChartData(worksheet, new CellRange(4, 2, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(4, 0, 4, 1))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithAllDatesAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 1, 2, 1].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 1, 3, 1].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 1, 4, 1].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 2, 1, 2].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 2, 2, 2].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 2, 3, 2].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 2, 4, 2].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 3, 1, 3].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 3, 2, 3].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 3, 3, 3].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 3, 4, 3].SetValue(new DateTime(2018, 1, 1));

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 1, 4, 3), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            var numericChartData = new double[] { 1, 2, 3, 4 };
            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new NumericChartData(numericChartData), new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)));
            seriesGroup.Series.Add(new NumericChartData(numericChartData), new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)));
            seriesGroup.Series.Add(new NumericChartData(numericChartData), new WorkbookFormulaChartData(worksheet, new CellRange(1, 3, 4, 3)));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();
            verticalAxis.NumberFormat = FormatHelper.ShortDatePattern;

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParsesDataRangeIsWiderThanTaller()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            worksheet.Cells[0, 4, 0, 4].SetValue("category 2");
            worksheet.Cells[1, 4, 1, 4].SetValue("1000");
            worksheet.Cells[2, 4, 2, 4].SetValue("2000");
            worksheet.Cells[3, 4, 3, 4].SetValue("3000");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 3, 4), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 4)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 1, 4)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 1, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 4)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(2, 1, 2, 4)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(2, 0, 2, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 4)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(3, 1, 3, 4)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(3, 0, 3, 0))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParsedDataRangeIsTallerThanWider()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");
            worksheet.Cells[5, 0, 5, 0].SetValue("category 5");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");
            worksheet.Cells[5, 1, 5, 1].SetValue("5");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");
            worksheet.Cells[5, 2, 5, 2].SetValue("50");

            worksheet.Cells[0, 3, 0, 3].SetValue("series 3");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");
            worksheet.Cells[4, 3, 4, 3].SetValue("400");
            worksheet.Cells[5, 3, 5, 3].SetValue("500");


            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 5, 3), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 5, 0)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 5, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 5, 0)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 5, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 5, 0)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 3, 5, 3)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 3, 0, 3))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParsedDataRangeIsIsAsTallAsWide()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 3, 3), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 3)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 1, 3)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 1, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 3)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(2, 1, 2, 3)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(2, 0, 2, 0))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 3)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(3, 1, 3, 3)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(3, 0, 3, 0))));
            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithNumbersAndTwoLeftColumnsOfTextAndTwoTopRowsOfTextAsBarChart()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[2, 0, 2, 0].SetValue("text");
            worksheet.Cells[3, 0, 3, 0].SetValue("text");
            worksheet.Cells[4, 0, 4, 0].SetValue("text");
            worksheet.Cells[5, 0, 5, 0].SetValue("text");

            worksheet.Cells[2, 1, 2, 1].SetValue("category 1");
            worksheet.Cells[3, 1, 3, 1].SetValue("category 2");
            worksheet.Cells[4, 1, 4, 1].SetValue("category 3");
            worksheet.Cells[5, 1, 5, 1].SetValue("category 4");

            worksheet.Cells[0, 2, 0, 2].SetValue("text");
            worksheet.Cells[1, 2, 1, 2].SetValue("series 1");
            worksheet.Cells[2, 2, 2, 2].SetValue("1");
            worksheet.Cells[3, 2, 3, 2].SetValue("2");
            worksheet.Cells[4, 2, 4, 2].SetValue("3");
            worksheet.Cells[5, 2, 5, 2].SetValue("4");

            worksheet.Cells[0, 3, 0, 3].SetValue("text");
            worksheet.Cells[1, 3, 1, 3].SetValue("series 2");
            worksheet.Cells[2, 3, 2, 3].SetValue("1");
            worksheet.Cells[3, 3, 3, 3].SetValue("2");
            worksheet.Cells[4, 3, 4, 3].SetValue("3");
            worksheet.Cells[5, 3, 5, 3].SetValue("4");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 5, 3), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(2, 0, 5, 1)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(2, 2, 5, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 1, 2))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(2, 0, 5, 1)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(2, 3, 5, 3)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 3, 1, 3))));

            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithNumbersAndThirdColumnIsText()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            worksheet.Cells[1, 3, 1, 3].SetValue("text");
            worksheet.Cells[2, 3, 2, 3].SetValue("text");
            worksheet.Cells[3, 3, 3, 3].SetValue("text");
            worksheet.Cells[4, 3, 4, 3].SetValue("text");

            worksheet.Cells[1, 4, 1, 4].SetValue("100");
            worksheet.Cells[2, 4, 2, 4].SetValue("200");
            worksheet.Cells[3, 4, 3, 4].SetValue("300");
            worksheet.Cells[4, 4, 4, 4].SetValue("400");

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(1, 1, 4, 4), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 3)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 4, 4, 4)));

            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRowAndTheRestIsBooleans_LeftColumnAndTopRowAreTitlesBooleansAreData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue(true);
            worksheet.Cells[2, 1, 2, 1].SetValue(true);
            worksheet.Cells[3, 1, 3, 1].SetValue(true);
            worksheet.Cells[4, 1, 4, 1].SetValue(true);

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue(false);
            worksheet.Cells[2, 2, 2, 2].SetValue(false);
            worksheet.Cells[3, 2, 3, 2].SetValue(false);
            worksheet.Cells[4, 2, 4, 2].SetValue(false);

            DocumentChart resultChart = DocumentChartFactory.CreateChartFromRangeOfType(worksheet, new CellRange(0, 0, 4, 2), SeriesRangesOrientation.Automatic, ChartType.Bar);

            DocumentChart expectedChart = new DocumentChart();

            SeriesGroup seriesGroup = new BarSeriesGroup();
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 1, 4, 1)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 1, 0, 1))));
            seriesGroup.Series.Add(new WorkbookFormulaChartData(worksheet, new CellRange(1, 0, 4, 0)),
                                   new WorkbookFormulaChartData(worksheet, new CellRange(1, 2, 4, 2)),
                                      new FormulaTitle(new WorkbookFormulaChartData(worksheet, new CellRange(0, 2, 0, 2))));

            (seriesGroup as ISupportAxes).AxisGroupName = AxisGroupName.Primary;
            expectedChart.SeriesGroups.Add(seriesGroup);

            Axis horizontalAxis = new CategoryAxis();
            Axis verticalAxis = new ValueAxis();

            expectedChart.PrimaryAxes = new AxisGroup(horizontalAxis, verticalAxis);

            TestHelper.AssertChartsAreEqual(expectedChart, resultChart);
        }

    }
}
