﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Charts;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Charts
{
    [TestClass]
    public class RangeChartParserTests
    {
        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRow_LeftColumnAndTopRowAreTitlesRestIsData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 0, 4, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 2), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeNumbersAndtextLeftColumn_LeftColumnIsCategoriesNoSeriesTitlesRestIsData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 0, 4, 0), parseResult.CategoriesRange);
            Assert.IsNull(parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithNumbersOnly_AllIsDataWithoutCategoriesOrSeriesTitles()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 1, 4, 2));

            Assert.IsNull(parseResult.CategoriesRange);
            Assert.IsNull(parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithDatesInLeftColumn_TheLeftColumnIsCategoriesRange()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 0, 2, 0].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 0, 3, 0].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 0, 4, 0].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 0, 4, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 2), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithUnstructuredDataNumbersMixedWithText_DataRangeWillStartAfterTheFirstTextCellsOnTheLastRowAndLastColumn()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("1");
            worksheet.Cells[2, 0, 2, 0].SetValue("text 1");
            worksheet.Cells[3, 0, 3, 0].SetValue("2");
            worksheet.Cells[4, 0, 4, 0].SetValue("text 2");

            worksheet.Cells[1, 1, 1, 1].SetValue("text 5");
            worksheet.Cells[2, 1, 2, 1].SetValue("3");
            worksheet.Cells[3, 1, 3, 1].SetValue("4");
            worksheet.Cells[4, 1, 4, 1].SetValue("5");

            worksheet.Cells[1, 2, 1, 2].SetValue("4");
            worksheet.Cells[2, 2, 2, 2].SetValue("text 4");
            worksheet.Cells[3, 2, 3, 2].SetValue("5");
            worksheet.Cells[4, 2, 4, 2].SetValue("5");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 1, 2, 2), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(3, 0, 4, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(3, 1, 4, 2), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithUnstructuredDataNumbersMixedWithDates_DataRangeWillStartAfterTheFirstDateCellsOnTheLastRowAndLastColumn()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("1");
            worksheet.Cells[2, 0, 2, 0].SetValue("3");
            worksheet.Cells[3, 0, 3, 0].SetValue("2");
            worksheet.Cells[4, 0, 4, 0].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 1, 1, 1].SetValue("3");
            worksheet.Cells[2, 1, 2, 1].SetValue("3");
            worksheet.Cells[3, 1, 3, 1].SetValue("4");
            worksheet.Cells[4, 1, 4, 1].SetValue("5");

            worksheet.Cells[1, 2, 1, 2].SetValue("4");
            worksheet.Cells[2, 2, 2, 2].SetValue(new DateTime(2018, 1, 1));
            worksheet.Cells[3, 2, 3, 2].SetValue("5");
            worksheet.Cells[4, 2, 4, 2].SetValue("5");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 1, 2, 2), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(3, 0, 4, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(3, 1, 4, 2), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseEmptyRange_TopRowAndLeftColumnAreReservedForTitlesTheRestIsData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 3, 2));

            Assert.AreEqual(new CellRange(1, 0, 3, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 2), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 3, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeTextDataAtTheBottom()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[4, 0, 4, 0].SetValue("test");
            worksheet.Cells[4, 2, 4, 2].SetValue("test");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 1, 3, 2), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(4, 0, 4, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(4, 1, 4, 2), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithDataJaggedAtTheTop_TitlesAreDeterminedAccordingToFirstCellofTheTopRowThatHasText()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[4, 0, 4, 0].SetValue("test");
            worksheet.Cells[5, 0, 5, 0].SetValue("test");
            worksheet.Cells[6, 0, 6, 0].SetValue("test");

            worksheet.Cells[3, 1, 3, 1].SetValue("test");
            worksheet.Cells[4, 1, 4, 1].SetValue("test");
            worksheet.Cells[5, 1, 5, 1].SetValue("test");
            worksheet.Cells[6, 1, 6, 1].SetValue("test");

            worksheet.Cells[2, 2, 2, 2].SetValue("test");
            worksheet.Cells[3, 2, 3, 2].SetValue("test");
            worksheet.Cells[4, 2, 4, 2].SetValue("test");
            worksheet.Cells[5, 2, 5, 2].SetValue("test");
            worksheet.Cells[6, 2, 6, 2].SetValue("test");

            worksheet.Cells[1, 3, 1, 3].SetValue("test");
            worksheet.Cells[2, 3, 2, 3].SetValue("test");
            worksheet.Cells[3, 3, 3, 3].SetValue("test");
            worksheet.Cells[4, 3, 4, 3].SetValue("test");
            worksheet.Cells[5, 3, 5, 3].SetValue("test");
            worksheet.Cells[6, 3, 6, 3].SetValue("test");

            worksheet.Cells[0, 4, 0, 4].SetValue("test");
            worksheet.Cells[1, 4, 1, 4].SetValue("test");
            worksheet.Cells[2, 4, 2, 4].SetValue("test");
            worksheet.Cells[3, 4, 3, 4].SetValue("test");
            worksheet.Cells[4, 4, 4, 4].SetValue("test");
            worksheet.Cells[5, 4, 5, 4].SetValue("test");
            worksheet.Cells[6, 4, 6, 4].SetValue("test");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 6, 4));

            Assert.AreEqual(new CellRange(1, 0, 6, 3), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 4, 0, 4), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 4, 6, 4), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithTextOnly_BottomRightCellIsDataAndTheRestIsTitles()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("text 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("text 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("text 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("text 4");

            worksheet.Cells[1, 1, 1, 1].SetValue("text 10");
            worksheet.Cells[2, 1, 2, 1].SetValue("text 20");
            worksheet.Cells[3, 1, 3, 1].SetValue("text 30");
            worksheet.Cells[4, 1, 4, 1].SetValue("text 40");

            worksheet.Cells[1, 2, 1, 2].SetValue("text 100");
            worksheet.Cells[2, 2, 2, 2].SetValue("text 200");
            worksheet.Cells[3, 2, 3, 2].SetValue("text 300");
            worksheet.Cells[4, 2, 4, 2].SetValue("text 400");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 2, 3, 2), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(4, 0, 4, 1), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(4, 2, 4, 2), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithAllDates_DatesAreDataNoTitles()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 1, 2, 1].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 1, 3, 1].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 1, 4, 1].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 2, 1, 2].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 2, 2, 2].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 2, 3, 2].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 2, 4, 2].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 3, 1, 3].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 3, 2, 3].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 3, 3, 3].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 3, 4, 3].SetValue(new DateTime(2018, 1, 1));

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 1, 4, 3));

            Assert.IsNull(parseResult.CategoriesRange);
            Assert.IsNull(parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 3), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithAllDatesAndOneGeneralNumberInBottomRight_DatesAreTitlesAndNumberIsData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 1, 2, 1].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 1, 3, 1].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 1, 4, 1].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 2, 1, 2].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 2, 2, 2].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 2, 3, 2].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 2, 4, 2].SetValue(new DateTime(2018, 1, 1));

            worksheet.Cells[1, 3, 1, 3].SetValue(new DateTime(2015, 1, 1));
            worksheet.Cells[2, 3, 2, 3].SetValue(new DateTime(2016, 1, 1));
            worksheet.Cells[3, 3, 3, 3].SetValue(new DateTime(2017, 1, 1));
            worksheet.Cells[4, 3, 4, 3].SetValue(1);

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 1, 4, 3));

            Assert.AreEqual(new CellRange(1, 3, 3, 3), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(4, 1, 4, 2), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(4, 3, 4, 3), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsWiderThanTaller_SeriesAreDeterminedToBeHorizontal()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            worksheet.Cells[0, 4, 0, 4].SetValue("category 2");
            worksheet.Cells[1, 4, 1, 4].SetValue("1000");
            worksheet.Cells[2, 4, 2, 4].SetValue("2000");
            worksheet.Cells[3, 4, 3, 4].SetValue("3000");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 3, 4));

            Assert.AreEqual(new CellRange(0, 1, 0, 4), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(1, 0, 3, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 3, 4), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsWiderThanTallerParserIsCalledWithVerticalParameter_SeriesAreVertical()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            worksheet.Cells[0, 4, 0, 4].SetValue("category 2");
            worksheet.Cells[1, 4, 1, 4].SetValue("1000");
            worksheet.Cells[2, 4, 2, 4].SetValue("2000");
            worksheet.Cells[3, 4, 3, 4].SetValue("3000");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 3, 4), true);

            Assert.AreEqual(new CellRange(0, 1, 0, 4), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 0, 3, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(1, 1, 3, 4), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsTallerThanWider_SeriesAreVertical()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");
            worksheet.Cells[5, 0, 5, 0].SetValue("category 5");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");
            worksheet.Cells[5, 1, 5, 1].SetValue("5");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");
            worksheet.Cells[5, 2, 5, 2].SetValue("50");

            worksheet.Cells[0, 3, 0, 3].SetValue("series 3");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");
            worksheet.Cells[4, 3, 4, 3].SetValue("400");
            worksheet.Cells[5, 3, 5, 3].SetValue("500");


            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 5, 3));

            Assert.AreEqual(new CellRange(1, 0, 5, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 3), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 5, 3), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsTallerThanWiderParserIsCalledWIthHorizontalParameter_SeriesAreHorizontal()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");
            worksheet.Cells[5, 0, 5, 0].SetValue("category 5");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");
            worksheet.Cells[5, 1, 5, 1].SetValue("5");

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");
            worksheet.Cells[5, 2, 5, 2].SetValue("50");

            worksheet.Cells[0, 3, 0, 3].SetValue("series 3");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");
            worksheet.Cells[4, 3, 4, 3].SetValue("400");
            worksheet.Cells[5, 3, 5, 3].SetValue("500");


            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 5, 3), false);

            Assert.AreEqual(new CellRange(1, 0, 5, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 3), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(1, 1, 5, 3), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsIsAsTallAsWide_SeriesAreHorizontal()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 3, 3));

            Assert.AreEqual(new CellRange(0, 1, 0, 3), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(1, 0, 3, 0), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 3, 3), parseResult.DataRange);
            Assert.IsFalse(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParsedDataRangeIsIsAsTallAsWideParserIsCalledWithVerticalParameter_SeriesAreVertical()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("series 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("series 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("series 3");

            worksheet.Cells[0, 1, 0, 1].SetValue("category 1");
            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");

            worksheet.Cells[0, 2, 0, 2].SetValue("category 2");
            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");

            worksheet.Cells[0, 3, 0, 3].SetValue("category 2");
            worksheet.Cells[1, 3, 1, 3].SetValue("100");
            worksheet.Cells[2, 3, 2, 3].SetValue("200");
            worksheet.Cells[3, 3, 3, 3].SetValue("300");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 3, 3), true);

            Assert.AreEqual(new CellRange(0, 1, 0, 3), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 0, 3, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(1, 1, 3, 3), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithNumbersAndTwoLeftColumnsOfTextAndTwoTopRowsOfText_TextIsTitlesAndNumbersAreData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[2, 0, 2, 0].SetValue("text");
            worksheet.Cells[3, 0, 3, 0].SetValue("text");
            worksheet.Cells[4, 0, 4, 0].SetValue("text");
            worksheet.Cells[5, 0, 5, 0].SetValue("text");

            worksheet.Cells[2, 1, 2, 1].SetValue("category 1");
            worksheet.Cells[3, 1, 3, 1].SetValue("category 2");
            worksheet.Cells[4, 1, 4, 1].SetValue("category 3");
            worksheet.Cells[5, 1, 5, 1].SetValue("category 4");

            worksheet.Cells[0, 2, 0, 2].SetValue("text");
            worksheet.Cells[1, 2, 1, 2].SetValue("series 1");
            worksheet.Cells[2, 2, 2, 2].SetValue("1");
            worksheet.Cells[3, 2, 3, 2].SetValue("2");
            worksheet.Cells[4, 2, 4, 2].SetValue("3");
            worksheet.Cells[5, 2, 5, 2].SetValue("4");

            worksheet.Cells[0, 3, 0, 3].SetValue("text");
            worksheet.Cells[1, 3, 1, 3].SetValue("series 2");
            worksheet.Cells[2, 3, 2, 3].SetValue("1");
            worksheet.Cells[3, 3, 3, 3].SetValue("2");
            worksheet.Cells[4, 3, 4, 3].SetValue("3");
            worksheet.Cells[5, 3, 5, 3].SetValue("4");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 5, 3));

            Assert.AreEqual(new CellRange(2, 0, 5, 1), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 2, 1, 3), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(2, 2, 5, 3), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithNumbersAndThirdColumnIsText_FirstToThirdColumnIsCategoriesRangeTheRestIsData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 1, 1, 1].SetValue("1");
            worksheet.Cells[2, 1, 2, 1].SetValue("2");
            worksheet.Cells[3, 1, 3, 1].SetValue("3");
            worksheet.Cells[4, 1, 4, 1].SetValue("4");

            worksheet.Cells[1, 2, 1, 2].SetValue("10");
            worksheet.Cells[2, 2, 2, 2].SetValue("20");
            worksheet.Cells[3, 2, 3, 2].SetValue("30");
            worksheet.Cells[4, 2, 4, 2].SetValue("40");

            worksheet.Cells[1, 3, 1, 3].SetValue("text");
            worksheet.Cells[2, 3, 2, 3].SetValue("text");
            worksheet.Cells[3, 3, 3, 3].SetValue("text");
            worksheet.Cells[4, 3, 4, 3].SetValue("text");

            worksheet.Cells[1, 4, 1, 4].SetValue("100");
            worksheet.Cells[2, 4, 2, 4].SetValue("200");
            worksheet.Cells[3, 4, 3, 4].SetValue("300");
            worksheet.Cells[4, 4, 4, 4].SetValue("400");

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(1, 1, 4, 4));

            Assert.AreEqual(new CellRange(1, 1, 4, 3), parseResult.CategoriesRange);
            Assert.IsNull(parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 4, 4, 4), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }

        [TestMethod]
        public void ParseRangeWithTextOnLeftColumnAndTopRowAndTheRestIsBooleans_LeftColumnAndTopRowAreTitlesBooleansAreData()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            worksheet.Cells[1, 0, 1, 0].SetValue("category 1");
            worksheet.Cells[2, 0, 2, 0].SetValue("category 2");
            worksheet.Cells[3, 0, 3, 0].SetValue("category 3");
            worksheet.Cells[4, 0, 4, 0].SetValue("category 4");

            worksheet.Cells[0, 1, 0, 1].SetValue("series 1");
            worksheet.Cells[1, 1, 1, 1].SetValue(true);
            worksheet.Cells[2, 1, 2, 1].SetValue(true);
            worksheet.Cells[3, 1, 3, 1].SetValue(true);
            worksheet.Cells[4, 1, 4, 1].SetValue(true);

            worksheet.Cells[0, 2, 0, 2].SetValue("series 2");
            worksheet.Cells[1, 2, 1, 2].SetValue(false);
            worksheet.Cells[2, 2, 2, 2].SetValue(false);
            worksheet.Cells[3, 2, 3, 2].SetValue(false);
            worksheet.Cells[4, 2, 4, 2].SetValue(false);

            RangeParseResultInfo parseResult = RangeChartParser.ParseRange(worksheet, new CellRange(0, 0, 4, 2));

            Assert.AreEqual(new CellRange(1, 0, 4, 0), parseResult.CategoriesRange);
            Assert.AreEqual(new CellRange(0, 1, 0, 2), parseResult.SeriesTitlesRange);
            Assert.AreEqual(new CellRange(1, 1, 4, 2), parseResult.DataRange);
            Assert.IsTrue(parseResult.SeriesRangesAreVertical);
        }
    }
}
