﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Charts;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests.ChartsTests
{
    [TestClass]
    public class WorkbookChartDataTests
    {
        [TestMethod]
        public void CreateChartDataWithCellRange_ResultFormulaIsCorrect()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            worksheet.Name = "TestWorksheet";

            WorkbookFormulaChartData spreadsheetChartData = new WorkbookFormulaChartData(worksheet, new CellRange(0, 0, 1, 0), new CellRange(2, 0, 3, 0));

            string expectedFormula = "TestWorksheet!$A$1:$A$2,TestWorksheet!$A$3:$A$4";
            Assert.AreEqual(expectedFormula, spreadsheetChartData.Formula);
        }

        [TestMethod]
        public void CreateChartDataWithFormula_ResultRangesAreCorrect()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            worksheet.Name = "TestWorksheet";

            WorkbookFormulaChartData spreadsheetChartData = new WorkbookFormulaChartData(worksheet.Workbook, "TestWorksheet!$A$1:$A$2,TestWorksheet!$A$3:$A$4");

            IEnumerable<CellRange> expectedRanges = new List<CellRange> () { new CellRange(0, 0, 1, 0), new CellRange(2, 0, 3, 0) };

            Worksheet actualWorksheet = worksheet;
            IEnumerable<CellRange> actualRanges = spreadsheetChartData.EnumerateCellRanges(out worksheet);

            TestHelper.AssertCollectionsAreEqual(expectedRanges, actualRanges);
            Assert.AreEqual(worksheet, actualWorksheet);
        }

        [TestMethod]
        [ExpectedException(typeof(FormatException))]
        public void CreateChartDataWithArrayFormula_RangesCannotBeEnumerated()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();
            worksheet.Name = "TestWorksheet";

            WorkbookFormulaChartData spreadsheetChartData = new WorkbookFormulaChartData(worksheet.Workbook, "{1, 2, 3, 4}");

            IEnumerable<CellRange> expectedRanges = new List<CellRange>() { };
            IEnumerable<CellRange> actualRanges;
            Worksheet actualWorksheet;
            bool result = spreadsheetChartData.TryEnumerateCellRanges(out actualRanges, out actualWorksheet);

            TestHelper.AssertCollectionsAreEqual(expectedRanges, actualRanges);
            Assert.IsNull(actualWorksheet);
            Assert.IsFalse(result);

            spreadsheetChartData.EnumerateCellRanges(out actualWorksheet);
        }
    }
}
