﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities.Performance;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
	[TestClass]
	public class CompressedListPerformanceTests
    {
        [TestMethod]
        public void TestCompressedListWithInsertDifferentValuesSpeedAndCorrectness()
        {
            int size = 10000;

            Action action = new Action(() =>
            {
                CompressedList<int> list = CompressedListTestHelper.CreateCompressedListWithDifferentValues(size);

                for (int i = 0; i < size; i++)
                {
                    Assert.AreEqual(i, list.GetValue(i));
                }
            });

            ExecuteWithSpeedCounter(action);
        }

        [TestMethod]
        public void TestCompressedListInsertSameValueSpeedAndCorrectness()
        {
            int size = 10000;
            int value = 1000;

            Action action = new Action(() =>
            {
                CompressedList<int> list = CompressedListTestHelper.CreateCompressedListWithSameValue(size, value);

                for (int i = 0; i < size; i++)
                {
                    Assert.AreEqual(value, list.GetValue(i));
                }
            });

            ExecuteWithSpeedCounter(action);
        }

        [TestMethod]
        public void TestCompressedListInsertRangesSpeedAndCorrectness()
        {
            int ranges = 100;
            int rangeSize = 100;
            int size = ranges * rangeSize;

            Action action = new Action(() =>
            {
                CompressedList<int> list = CompressedListTestHelper.CreateCompressedListWithRanges(size, rangeSize);

                for (int i = 0; i < size; i++)
                {
                    Assert.AreEqual(i / ranges, list.GetValue(i));
                }
            });

            ExecuteWithSpeedCounter(action);
        }

        [TestMethod]
        public void TestSetValueSpeed()
        {
            int miliseconds = 1550;

            int size = 10000;
            Workbook wb = new Workbook();
            wb.History.IsEnabled = false;
            Worksheet sheet = wb.Worksheets.Add();

            Action action = new Action(() =>
            {
                for (int i = 0; i < size; i++)
                {
                    sheet.Cells[i, 0].SetValue(i.ToString());
                }
            });

            ExecuteWithSpeedCounter(action, miliseconds);
        }

        private void ExecuteWithSpeedCounter(Action action, double maxMilliseconds = 100)
        {
            SnippetPerformanceCounter performanceCounter = new SnippetPerformanceCounter("");

            performanceCounter.Start();
            action();
            performanceCounter.End();

            Assert.IsTrue(performanceCounter.TotalMilliseconds < maxMilliseconds, 
                string.Format("Executed for {0}; Must be for {1}", performanceCounter.TotalMilliseconds, maxMilliseconds));
        }
	}
}