﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CompressedLists.CompressedListTests
{
    internal class CompressedListBuilder<T>
    {
        private readonly long fromIndex;
        private readonly long toIndex;
        private bool isDefaultValueAssigned = false;
        private T defaultValue;

        public CompressedListBuilder(long fromIndex, long toIndex)
        {
            this.fromIndex = fromIndex;
            this.toIndex = toIndex;
        }

        public CompressedListBuilder<T> WithDefaultValue(T defaultValue)
        {
            this.defaultValue = defaultValue;
            this.isDefaultValueAssigned = true;

            return this;
        }

        public CompressedList<T> Build()
        {
            CompressedList<T> list;

            if (this.isDefaultValueAssigned)
            {
                list = new CompressedList<T>(this.fromIndex, this.toIndex, this.defaultValue);
            }
            else
            {
                list = new CompressedList<T>(this.fromIndex, this.toIndex);
            }

            return list;
        }

        internal static void AssertValueCorrectness(CompressedList<T> list, long fromIndex, IEnumerable<T> expectedValues)
        {
            List<T> expectedValuesList = expectedValues.ToList();

            for (int i = 0; i < expectedValuesList.Count; i++)
            {
                T value = list.GetValue(fromIndex + i);
                Assert.AreEqual(expectedValuesList[i], value);
            }
        }
    }
}
