﻿using System;
using System.Linq;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CompressedLists.CompressedListTests
{
    [TestClass]
    public class GetNonDefaultRanges_Should
    {
        [TestMethod]
        public void ReturnNullForEmptyCompressedList()
        {
            var sut = new CompressedListBuilder<string>(0, 100).Build();

            List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
            Assert.AreEqual(0, ranges.Count);
        }

        [TestMethod]
        public void ReturnNullForEmptyCompressedListWithSetDefaultValue()
        {
            var sut = new CompressedListBuilder<string>(0, 100).WithDefaultValue("New Default Value").Build();

            List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
            Assert.AreEqual(0, ranges.Count);
        }

        [TestMethod]
        public void ReturnOneRangeForCompressedListWithOneValue()
        {
            var sut = new CompressedListBuilder<string>(0, 100).Build();
            sut.SetValue(0, 100, "value");

            List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
            Assert.AreEqual(1, ranges.Count);
            Assert.AreEqual(0, ranges[0].Start);
            Assert.AreEqual(100, ranges[0].End);
            Assert.AreEqual("value", ranges[0].Value);
        }

        [TestMethod]
        public void ReturnTwoRangesForCompressedListTwoValues()
        {
            var sut = new CompressedListBuilder<string>(0, 100).Build();
            sut.SetValue(5, 10, "value");
            sut.SetValue(15, 20, "value");

            List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
            Assert.AreEqual(2, ranges.Count);

            Assert.AreEqual(5, ranges[0].Start);
            Assert.AreEqual(10, ranges[0].End);
            Assert.AreEqual("value", ranges[0].Value);

            Assert.AreEqual(15, ranges[1].Start);
            Assert.AreEqual(20, ranges[1].End);
            Assert.AreEqual("value", ranges[1].Value);
        }

        [TestMethod]
        public void ReturnTwoRangesForCompressedListWithEmptyAndValueRanges()
        {
            var sut = new CompressedListBuilder<string>(0, 100).Build();
            sut.SetValue(5, 50, "value");
            sut.SetValue(51, 90, "value");
  
            List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
            Assert.AreEqual(1, ranges.Count);

            Assert.AreEqual(5, ranges[0].Start);
            Assert.AreEqual(90, ranges[0].End);
            Assert.AreEqual("value", ranges[0].Value);
        }

        // TODO: Start with an empty compressed list and add set a given value to a range. Then set the inputed value to be default.
        // as a result the list returns range as a non-default, where as it should treat it as default
        //[TestMethod]
        //public void ReturnZeroRangesForCompressedListWithOneRangesAndNewDefaultValue()
        //{
        //    string newDefaultValue = "value";
        //    var sut = new CompressedListBuilder<string>(0, 10).Build();
        //    sut.SetValue(5, 7, newDefaultValue);
        //    sut.SetDefaultValue(newDefaultValue);

        //    List<Range<long, string>> ranges = sut.GetNonDefaultRanges().ToList();
        //    Assert.AreEqual(0, ranges.Count);
        //}
    }
}
