﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CompressedLists.CompressedListTests
{
    [TestClass]
    public class SetValue_Should
    {
        [TestMethod]
        public void SetTheValueOfSingleIndex()
        {
            var sut = new CompressedListBuilder<string>(0, 2).Build();
            string newValue = "marker";

            sut.SetValue(1, newValue);

            string[] expectedValues = new [] { default(string), newValue, default(string) };
            CompressedListBuilder<string>.AssertValueCorrectness(sut, 0, expectedValues);
        }

        [TestMethod]
        public void SetTheValueOfMultipleIndexes()
        {
            var sut = new CompressedListBuilder<string>(0, 3).Build();
            string newValue = "marker";

            sut.SetValue(1, 2, newValue);

            string[] expectedValues = new[] { default(string), newValue, newValue, default(string) };
            CompressedListBuilder<string>.AssertValueCorrectness(sut, 0, expectedValues);
        }

        [TestMethod]
        public void SetMultipleValues()
        {
            var sut = new CompressedListBuilder<string>(0, 3).Build();
            
            List<string> newValues = new List<string>() { "marker", "marker2" };
            sut.SetValue(1, newValues);

            string[] expectedValues = new[] { default(string), "marker", "marker2", default(string) };
            CompressedListBuilder<string>.AssertValueCorrectness(sut, 0, expectedValues);
        }

        [TestMethod]
        public void SetCompressedList()
        {
            var compressedList = new CompressedListBuilder<string>(0, 3).Build();
            compressedList.SetValue(1, "marker");
            compressedList.SetValue(2, "marker2");
            
            var sut = new CompressedListBuilder<string>(0, 3).Build();

            sut.SetValue(compressedList);

            string[] expectedValues = new[] { default(string), "marker", "marker2", default(string) };
            CompressedListBuilder<string>.AssertValueCorrectness(sut, 0, expectedValues);
        }
    }
}
