﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CompressedLists.RangeClassTests
{
    [TestClass]
    public class Range_Contains_Should
    {
        [TestMethod]
        public void ReturnTrueWhenItContainsThePosition()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            bool contains = sut.Contains(15);

            Assert.IsTrue(contains, "The Range does not contain the specified position");
        }

        [TestMethod]
        public void ReturnTrueWhenThePositionIsStart()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            bool contains = sut.Contains(0);

            Assert.IsTrue(contains, "The Range does not contain the specified position");
        }

        [TestMethod]
        public void ReturnTrueWhenThePositionIsEnd()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            bool contains = sut.Contains(42);

            Assert.IsTrue(contains, "The Range does not contain the specified position");
        }

        [TestMethod]
        public void ReturnFalseWhenItDoesNotContainThePosition()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            bool contains = sut.Contains(43);

            Assert.IsFalse(contains, "The Range contains the specified position");
        }

        [TestMethod]
        public void ReturnTrueWhenItContainsTheRange()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            var range = new RangeBuilder<long, int>().Build(10, 40);
            bool contains = sut.Contains(range);

            Assert.IsTrue(contains, "The Range does not contain the specified range");
        }

        [TestMethod]
        public void ReturnTrueWhenItCoincidesWithTheRange()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            var range = new RangeBuilder<long, int>().Build(0, 42);
            bool contains = sut.Contains(range);

            Assert.IsTrue(contains, "The Range does not contain the specified range");
        }

        [TestMethod]
        public void ReturnFalseWhenTheRangeEndExceedsTheEnd()
        {
            var sut = new RangeBuilder<long, int>().Build(0, 42);

            var range = new RangeBuilder<long, int>().Build(10, 43);
            bool contains = sut.Contains(range);

            Assert.IsFalse(contains, "The Range contains the specified range");
        }

        [TestMethod]
        public void ReturnFalseWhenTheRangeStartIsBeforeTheStart()
        {
            var sut = new RangeBuilder<long, int>().Build(10, 42);

            var range = new RangeBuilder<long, int>().Build(0, 40);
            bool contains = sut.Contains(range);

            Assert.IsFalse(contains, "The Range contains the specified range");
        }

        [TestMethod]
        public void ReturnFalseWhenTheRangeParameterContainsTheRange()
        {
            var sut = new RangeBuilder<long, int>().Build(10, 42);

            var range = new RangeBuilder<long, int>().Build(9, 43);
            bool contains = sut.Contains(range);

            Assert.IsFalse(contains, "The Range contains the specified range");
        }
    }
}
