﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.CopyPaste
{
    [TestClass]
    public class CopyPasteContextTests
    {
        #region Test copy/paste of hidden rows/column with filtering applied to the sheet.

        [TestMethod]
        public void TestRangeNoHiddenRowsColumns()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 2, 2);
            List<CellRange> expectedRanges = new List<CellRange>() { range };
            this.ApplyFilteringToSheet(fromWorksheet);

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(), new List<int>());
        }

        [TestMethod]
        public void TestRangeOneHiddenRowMiddle()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(1, 1, 3, 3);

            List<CellRange> hiddenRows = new List<CellRange>()
            { 
                new CellRange(2, 2, 2, 2) 
            };

            this.HideRows(fromWorksheet, hiddenRows);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(1, 1, 1, 3),
                new CellRange(3, 1, 3, 3)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(){ 2 }, new List<int>());
        }

        [TestMethod]
        public void TestRangeMultipleHiddenRowsMiddle()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 2);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(1, 1, 2, 1) 
            };

            this.HideRows(fromWorksheet, hiddenRows);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 0, 0, 2),
                new CellRange(3, 0, 3, 2)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(){ 1, 2 }, new List<int>());
        }

        [TestMethod]
        public void TestRangeMultipleHiddenRowsBeginning()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 2);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(0, 0, 0, 0),
                new CellRange(1, 1, 1, 1)
            };

            this.HideRows(fromWorksheet, hiddenRows);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(2, 0, 3, 2)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(){ 0, 1 }, new List<int>());
        }

        [TestMethod]
        public void TestRangeMultipleHiddenRowsEnd()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 2);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(2, 0, 2, 0),
                new CellRange(3, 0, 3, 0)
            };

            this.HideRows(fromWorksheet, hiddenRows);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 0, 1, 2)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(){ 2, 3 }, new List<int>());
        }

        [TestMethod]
        public void TestRangeMultipleHiddenColumnsMiddle()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 3);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 1, 0, 1),
                new CellRange(0, 2, 0, 2)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 0, 3, 0),
                new CellRange(0, 3, 3, 3)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(), new List<int>() { 1, 2 });
        }

        [TestMethod]
        public void TestRangeMultipleHiddenColumnsBeginning()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 3);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 0, 0, 0),
                new CellRange(0, 1, 0, 1)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 2, 3, 3)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(), new List<int>() { 0, 1 });
        }

        [TestMethod]
        public void TestRangeMultipleHiddenColumnsEnd()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 3, 3);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 2, 0, 2),
                new CellRange(0, 3, 0, 3)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 0, 3, 1)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(), new List<int>() { 2, 3 });
        }

        [TestMethod]
        public void TestRangeMultipleHiddenRowsColumnsMiddle()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 5, 5);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 1, 0, 1),
                new CellRange(0, 3, 0, 3)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(1, 0, 1, 0),
                new CellRange(3, 0, 3, 0)
            };

            RowSelection rowSelection = new RowSelection(fromWorksheet, hiddenRows);
            rowSelection.SetHidden(true);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(0, 0, 0, 0),
                new CellRange(2, 0, 2, 0),
                new CellRange(4, 0, 5, 0),
                new CellRange(0, 2, 0, 2),
                new CellRange(2, 2, 2, 2),
                new CellRange(4, 2, 5, 2),
                new CellRange(0, 4, 0, 5),
                new CellRange(2, 4, 2, 5),
                new CellRange(4, 4, 5, 5),
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>() { 1, 3 }, new List<int>() { 1, 3 });
        }

        [TestMethod]
        public void TestRangeMultipleHiddenRowsColumnsBorders()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 5, 5);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 0, 0, 0),
                new CellRange(0, 5, 0, 5)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);
            this.ApplyFilteringToSheet(fromWorksheet);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(0, 0, 0, 0),
                new CellRange(5, 0, 5, 0)
            };

            this.HideRows(fromWorksheet, hiddenRows);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                new CellRange(1, 1, 4, 4)
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>() { 0, 5 }, new List<int>() { 0, 5 });
        }

        #endregion

        #region Test copy/paste of hidden rows/column without filtering applied to the sheet.

        [TestMethod]
        public void TestRangeMultipleHiddenRowsColumnsMiddleWithoutFiltering()
        {
            Worksheet fromWorksheet = TestHelper.CreateWorksheet();
            CellRange range = new CellRange(0, 0, 5, 5);

            List<CellRange> hiddenColumns = new List<CellRange>() 
            {
                new CellRange(0, 1, 0, 1),
                new CellRange(0, 3, 0, 3)
            };

            this.HideColumns(fromWorksheet, hiddenColumns);

            List<CellRange> hiddenRows = new List<CellRange>() 
            {
                new CellRange(1, 0, 1, 0),
                new CellRange(3, 0, 3, 0)
            };

            RowSelection rowSelection = new RowSelection(fromWorksheet, hiddenRows);
            rowSelection.SetHidden(true);

            List<CellRange> expectedRanges = new List<CellRange>()
            { 
                range
            };

            this.TestCellRangeSplit(fromWorksheet, range, expectedRanges, new List<int>(), new List<int>());
        }

        #endregion

        // Depending on whether filtering is applied anywhere on the sheet copying of rows and columns has different behaviour:
        // 1. With filtering, when copying all hidden rows/columns are skipped
        // 2. Without filtering, when copying all hidden rows/columns are included
        private void ApplyFilteringToSheet(Worksheet worksheet)
        {
            worksheet.Filter.FilterRange = new CellRange(0, 0, 0, 0);
            IFilter filter = new DynamicFilter(0, DynamicFilterType.April);
            worksheet.Filter.SetFilter(filter);
        }

        private void HideColumns(Worksheet worksheet, List<CellRange> hiddenColumns)
        {
            ColumnSelection columnSelection = new ColumnSelection(worksheet, hiddenColumns);
            columnSelection.SetHidden(true);
        }

        private void HideRows(Worksheet worksheet, List<CellRange> hiddenRows)
        {
            RowSelection rowSelection = new RowSelection(worksheet, hiddenRows);
            rowSelection.SetHidden(true);
        }

        private void TestCellRangeSplit(Worksheet fromWorksheet, CellRange cellRange, List<CellRange> expectedRanges,
            List<int> expectedHiddenRows, List<int> expectedHiddenColumns)
        {
            Worksheet toWorksheet = TestHelper.CreateWorksheet();
            CellRange toCellRange = new CellRange(0,0,0,0);

            CopyPasteContext context = new CopyPasteContext(fromWorksheet, cellRange, toWorksheet, toCellRange, null, null, PasteOptions.All, true, true, false);

            Assert.AreEqual(expectedRanges.Count, context.FromCellRange.Count);
            for (int i = 0; i < expectedRanges.Count; i++)
            {
                Assert.AreEqual(expectedRanges[i], context.FromCellRange[i]);
            }

            Assert.AreEqual(expectedHiddenRows.Count, context.HiddenRows.Count());
            for (int i = 0; i < expectedHiddenRows.Count; i++)
            {
                Assert.AreEqual(expectedHiddenRows[i], context.HiddenRows.ElementAt(i));
            }

            Assert.AreEqual(expectedHiddenColumns.Count, context.HiddenColumns.Count());
            for (int i = 0; i < expectedHiddenColumns.Count; i++)
            {
                Assert.AreEqual(expectedHiddenColumns[i], context.HiddenColumns.ElementAt(i));
            }
        }
    }
}
