﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.DataValidation
{
    [TestClass]
    public class ListDataValidationTests
    {
        [TestMethod]
        [TestCategory("DataValidation")]
        public void AllowValueContainedInTheListTest()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            CellIndex cellIndex = new CellIndex(5, 5);

            worksheet.Cells[0, 0].SetValue("test");
            worksheet.Cells[1, 0].SetValue("Test");
            worksheet.Cells[2, 0].SetValue("ala bala");
            worksheet.Cells[3, 0].SetValue("bala");
            worksheet.Cells[4, 0].SetValue("tralalaa");

            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = "=$A$1:$A$5";
            ListDataValidationRule rule = new ListDataValidationRule(context);

            var result = rule.Evaluate(worksheet, cellIndex, "bala");
            Assert.IsTrue(result);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AllowValueContainedInTheListFromAnotherWorksheetTest()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            Worksheet worksheet1 = workbook.Worksheets.Add();
            CellIndex cellIndex = new CellIndex(5, 5);

            worksheet1.Cells[0, 0].SetValue("test");
            worksheet1.Cells[1, 0].SetValue("Test");
            worksheet1.Cells[2, 0].SetValue("ala bala");
            worksheet1.Cells[3, 0].SetValue("bala");
            worksheet1.Cells[4, 0].SetValue("tralalaa");

            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = string.Format("={0}!$A$1:$A$5", worksheet1.Name);
            ListDataValidationRule rule = new ListDataValidationRule(context);

            var result = rule.Evaluate(worksheet, cellIndex, "bala");
            Assert.IsTrue(result);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AllowValueContainedInTheListFromAnotherWorksheetDefinedNameTest()
        {
            Workbook workbook = new Workbook();
            Worksheet worksheet = workbook.Worksheets.Add();
            Worksheet worksheet1 = workbook.Worksheets.Add();
            CellIndex cellIndex = new CellIndex(5, 5);

            worksheet1.Cells[0, 0].SetValue("test");
            worksheet1.Cells[1, 0].SetValue("Test");
            worksheet1.Cells[2, 0].SetValue("ala bala");
            worksheet1.Cells[3, 0].SetValue("bala");
            worksheet1.Cells[4, 0].SetValue("tralalaa");

            worksheet.Names.Add("listSource", string.Format("={0}!$A$1:$A$5", worksheet1.Name), cellIndex);

            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = string.Format("=listSource");
            ListDataValidationRule rule = new ListDataValidationRule(context);

            var result = rule.Evaluate(worksheet, cellIndex, "bala");
            Assert.IsTrue(result);
        }
    }
}