﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.DataValidation;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.DataValidation
{
    [TestClass]
    public class MissingArgumentsDataValidationTests
    {
        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertWholeNumberDataValidationRule_WithMissingArguments_AllowsOnlyWholeNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = null;

                WholeNumberDataValidationRule rule = new WholeNumberDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsFalse(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsFalse(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsFalse(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDecimalDataValidationRule_WithMissingArguments_AllowsOnlyDecimalNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = null;

                DecimalDataValidationRule rule = new DecimalDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDateDataValidationRule_WithMissingArguments_AllowsOnlyDates()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = null;

                DateDataValidationRule rule = new DateDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTimeDataValidationRule_WithMissingArguments_AllowsOnlyTimes()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = null;

                TimeDataValidationRule rule = new TimeDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTextLenghtDataValidationRule_WithMissingArguments_AllowsAnyTextLength()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = null;

                TextLengthDataValidationRule rule = new TextLengthDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsTrue(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertListDataValidationRule_WithMissingArguments_AllowsValue()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            ListDataValidationRuleContext context = new ListDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = null;

            ListDataValidationRule rule = new ListDataValidationRule(context);
            bool result = rule.Evaluate(worksheet, cellIndex, "1");

            Assert.IsTrue(result);

            bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

            Assert.IsTrue(result1);

            bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

            Assert.IsTrue(result2);

            bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

            Assert.IsTrue(result3);

            bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

            Assert.IsTrue(result4);

            bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

            Assert.IsTrue(result5);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertCustomDataValidationRule_WithMissingArguments_AllowsValue()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            SingleArgumentDataValidationRuleContext context = new SingleArgumentDataValidationRuleContext(worksheet, cellIndex);
            context.Argument1 = null;

            CustomDataValidationRule rule = new CustomDataValidationRule(context);
            bool result = rule.Evaluate(worksheet, cellIndex, "1");

            Assert.IsTrue(result);

            bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

            Assert.IsTrue(result1);

            bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

            Assert.IsTrue(result2);

            bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

            Assert.IsTrue(result3);

            bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

            Assert.IsTrue(result4);

            bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

            Assert.IsTrue(result5);
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertWholeNumberDataValidationRule_WithMissingFirstArguments_AllowsOnlyWholeNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = "30";

                WholeNumberDataValidationRule rule = new WholeNumberDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsFalse(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsFalse(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsFalse(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDecimalDataValidationRule_WithMissingFirstArguments_AllowsOnlyDecimalNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = "30";

                DecimalDataValidationRule rule = new DecimalDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDateDataValidationRule_WithMissingFirstArguments_AllowsOnlyDates()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = "10/10/2019";

                DateDataValidationRule rule = new DateDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTimeDataValidationRule_WithMissingFirstArguments_AllowsOnlyTimes()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = "10:30";

                TimeDataValidationRule rule = new TimeDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTextLenghtDataValidationRule_WithMissingFirstArguments_AllowsAnyTextLength()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in Enum.GetValues(typeof(ComparisonOperator)))
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = null;
                context.Argument2 = "5";

                TextLengthDataValidationRule rule = new TextLengthDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsTrue(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertWholeNumberDataValidationRule_WithMissingSecondArguments_AllowsOnlyWholeNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in new ComparisonOperator[] { ComparisonOperator.Between, ComparisonOperator.NotBetween })
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = "30";
                context.Argument2 = null;

                WholeNumberDataValidationRule rule = new WholeNumberDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsFalse(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsFalse(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsFalse(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDecimalDataValidationRule_WithMissingSecondArguments_AllowsOnlyDecimalNumbers()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in new ComparisonOperator[] { ComparisonOperator.Between, ComparisonOperator.NotBetween })
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = "30";
                context.Argument2 = null;

                DecimalDataValidationRule rule = new DecimalDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertDateDataValidationRule_WithMissingSecondArguments_AllowsOnlyDates()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in new ComparisonOperator[] { ComparisonOperator.Between, ComparisonOperator.NotBetween })
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = "10/10/2019";
                context.Argument2 = null;

                DateDataValidationRule rule = new DateDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTimeDataValidationRule_WithMissingSecondArguments_AllowsOnlyTimes()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in new ComparisonOperator[] { ComparisonOperator.Between, ComparisonOperator.NotBetween })
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = "10:30";
                context.Argument2 = null;

                TimeDataValidationRule rule = new TimeDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsFalse(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }

        [TestMethod]
        [TestCategory("DataValidation")]
        public void AssertTextLenghtDataValidationRule_WithMissingSecondArguments_AllowsAnyTextLength()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.Worksheets[0];
            CellIndex cellIndex = new CellIndex(2, 0);

            foreach (ComparisonOperator comparisonOperator in new ComparisonOperator[] { ComparisonOperator.Between, ComparisonOperator.NotBetween })
            {
                NumberDataValidationRuleContext context = new NumberDataValidationRuleContext(worksheet, cellIndex);
                context.ComparisonOperator = comparisonOperator;
                context.Argument1 = "5";
                context.Argument2 = null;

                TextLengthDataValidationRule rule = new TextLengthDataValidationRule(context);
                bool result = rule.Evaluate(worksheet, cellIndex, "1");

                Assert.IsTrue(result);

                bool result1 = rule.Evaluate(worksheet, cellIndex, "1.2");

                Assert.IsTrue(result1);

                bool result2 = rule.Evaluate(worksheet, cellIndex, "abc");

                Assert.IsTrue(result2);

                bool result3 = rule.Evaluate(worksheet, cellIndex, "11/11/2019");

                Assert.IsTrue(result3);

                bool result4 = rule.Evaluate(worksheet, cellIndex, "11/11/2019 11:30");

                Assert.IsTrue(result4);

                bool result5 = rule.Evaluate(worksheet, cellIndex, "11:30");

                Assert.IsTrue(result5);
            }
        }
    }
}
