﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.DefinedNameTests
{
    [TestClass]
    public class DefinedName_IsNameValid_Should
    {
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void ReturnFalse_WhenNameIsNull()
        {
            DefinedName.IsNameValid(null);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void ReturnFalse_WhenNameIsEmptyString()
        {
            DefinedName.IsNameValid(string.Empty);
        }

        [TestMethod]
        public void ReturnFalse_WhenNameFirstCharIsADigit()
        {
            Assert.IsFalse(DefinedName.IsNameValid("0name"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameFirstCharIsListSeparator()
        {
            Assert.IsFalse(DefinedName.IsNameValid(",name"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameFirstCharIsQuestionMark()
        {
            Assert.IsFalse(DefinedName.IsNameValid("?name"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsValidCellName()
        {
            Assert.IsFalse(DefinedName.IsNameValid("A1"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameContainsPercentSymbol()
        {
            Assert.IsFalse(DefinedName.IsNameValid("name%"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameContainsEqualsSymbol()
        {
            Assert.IsFalse(DefinedName.IsNameValid("na=me"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameContainsSpaceSymbol()
        {
            Assert.IsFalse(DefinedName.IsNameValid("na me"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameContainsNewLineSymbol()
        {
            Assert.IsFalse(DefinedName.IsNameValid("na\nme"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsAnInteger()
        {
            Assert.IsFalse(DefinedName.IsNameValid("15"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsADouble()
        {
            Assert.IsFalse(DefinedName.IsNameValid("0.5"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsAScientificNumber()
        {
            Assert.IsFalse(DefinedName.IsNameValid("1.0E+01"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsRUpperCase()
        {
            Assert.IsFalse(DefinedName.IsNameValid("R"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsRLowerCase()
        {
            Assert.IsFalse(DefinedName.IsNameValid("r"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsCUpperCase()
        {
            Assert.IsFalse(DefinedName.IsNameValid("C"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsCLowerCase()
        {
            Assert.IsFalse(DefinedName.IsNameValid("c"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsTrue()
        {
            Assert.IsFalse(DefinedName.IsNameValid("TRUE"));
        }

        [TestMethod]
        public void ReturnFalse_WhenNameIsFalse()
        {
            Assert.IsFalse(DefinedName.IsNameValid("FALSE"));
        }

        [TestMethod]
        public void ReturnTrue_WhenNameContainsOnlyLetters()
        {
            Assert.IsTrue(DefinedName.IsNameValid("name"));
        }

        [TestMethod]
        public void ReturnTrue_WhenNameStartsWithUnderscoreAndContainsLetters()
        {
            Assert.IsTrue(DefinedName.IsNameValid("_name"));
        }

        [TestMethod]
        public void ReturnTrue_WhenNameStartsWithBackslashAndContainsLetters()
        {
            Assert.IsTrue(DefinedName.IsNameValid(@"\name"));
        }

        [TestMethod]
        public void ReturnTrue_WhenNameIsNotValidCellName()
        {
            Assert.IsTrue(DefinedName.IsNameValid("ZZZ1"));
        }
    }
}
