﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.DefinedNames
{
    [TestClass]
    public class DefinedNameInvalidationTests : DefinedNameReferringTests
    {
        [TestMethod]
        public void ReferToNameWithIntValueUpdateNameTest()
        {
            ISpreadsheetName name = this.AddDefinedName(Workbook, "name", "=5", new CellIndex(0, 0));
            this.Sheet1.Cells[0, 0].SetValue("=name");

            Assert.AreEqual("5", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            Workbook.NameManager.Update(name, name.Name, "10", new CellIndex(0, 0));

            Assert.AreEqual("10", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void ReferToNameWithIntValueRemoveNameTest()
        {
            ISpreadsheetName name = this.AddDefinedName(Workbook, "name", "=5", new CellIndex(0, 0));
            this.Sheet1.Cells[0, 0].SetValue("=name");

            Assert.AreEqual("5", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            Workbook.Names.Remove(name.Name);

            Assert.AreEqual(ErrorExpressions.NameError.Value, this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat), false);
        }

        [TestMethod]
        public void ReferToNameWithNameValueAddSecondNameTest()
        {
            this.AddDefinedName(Workbook, "name", "=secondName", new CellIndex(0, 0));
            
            this.Sheet1.Cells[0, 0].SetValue("=name");
            Assert.AreEqual(ErrorExpressions.NameError.Value, this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat), false);

            this.AddDefinedName(Workbook, "secondName", "=10", new CellIndex(0, 0));
            
            Assert.AreEqual("10", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void ReferToNameWithNameValueAddAndUpdateSecondNameTest()
        {
            this.AddDefinedName(Workbook, "name", "=secondName", new CellIndex(0, 0));
            
            this.Sheet1.Cells[0, 0].SetValue("=name");
            Assert.AreEqual(ErrorExpressions.NameError.Value, this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat), false);

            ISpreadsheetName secondName = this.AddDefinedName(Workbook, "secondName", "=10", new CellIndex(0, 0));

            Assert.AreEqual("10", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            this.Workbook.NameManager.Update(secondName, secondName.Name, "=15", new CellIndex(0, 0));

            Assert.AreEqual("15", this.Sheet1.Cells[0, 0].GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void ReferToNameWithCellRefValueUpdateCellTest1()
        {
            this.AddDefinedName(Workbook, "name", "=Sheet1!$A$1", new CellIndex(1, 1));

            CellSelection b2 = this.Sheet1.Cells[1, 1];
            b2.SetValue("=name");

            Assert.AreEqual("0", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat), false);

            Sheet1.Cells[0, 0].SetValue("10");

            Assert.AreEqual("10", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat), false);
        }

        [TestMethod]
        public void ReferToNameWithCellRefValueUpdateCellTest2()
        {
            Sheet1.Cells[0, 0].SetValue("5");

            this.AddDefinedName(Workbook, "name", "=Sheet1!$A$1", new CellIndex(1, 1));

            CellSelection b2 = this.Sheet1.Cells[1, 1];
            b2.SetValue("=name");

            Assert.AreEqual("5", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            Sheet1.Cells[0, 0].SetValue("10");

            Assert.AreEqual("10", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        [TestMethod]
        public void ReferToNameWithCellRefValueUpdateCellTest3()
        {
            ISpreadsheetName name = this.AddDefinedName(Workbook, "name", "=Sheet1!$A$1", new CellIndex(0, 0));

            CellSelection a1 = Sheet1.Cells[0, 0];
            a1.SetValue("=name");
            //Assert.AreEqual("5", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            CellSelection b2 = Sheet1.Cells[1, 1];
            b2.SetValue("=name");
            //Assert.AreEqual("5", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));

            this.Workbook.NameManager.Update(name, name.Name, "=5", new CellIndex(0, 0));

            Assert.AreEqual("5", a1.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
            Assert.AreEqual("5", b2.GetValue().Value.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }
    }
}
