﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.DefinedNames
{
    [TestClass]
    public class SanitazingRefersToTests
    {
        private void AssertRefersToFieldCorrectness(string expectedRefersTo, string inputRefersTo)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();

            ISpreadsheetName name = worksheet.Names.Add("name", inputRefersTo, new CellIndex(0, 0));
            Assert.AreEqual(expectedRefersTo, name.RefersTo, false);
        }

        private void AssertRefersToFieldException(string inputRefersTo)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            
            try
            {
                worksheet.Names.Add("name", inputRefersTo, new CellIndex(0, 0));
                Assert.Fail();
            }
            catch(Exception ex)
            {
                Assert.IsNotNull(ex);
            }
        }

        [TestMethod]
        public void RefersToWithNoEqualsSignDigitTest()
        {
            AssertRefersToFieldCorrectness("=3", "3");
        }

        [TestMethod]
        public void RefersToWithNoEqualsSignDigitsAndLettersTest()
        {
            AssertRefersToFieldCorrectness(@"=""343gfd""", "343gfd");
        }

        [TestMethod]
        public void RefersToWithNoEqualsSignLettersTest()
        {
            AssertRefersToFieldCorrectness(@"=""abc""", "abc");
        }

        [TestMethod]
        public void RefersToWithNoEqualsSignSymbolTest()
        {
            AssertRefersToFieldCorrectness(@"=""!""", "!");
        }

        [TestMethod]
        public void RefersToWithNoEqualsSignSymbolAndLettersTest()
        {
            AssertRefersToFieldCorrectness(@"=""!abc""", "!abc");
        }

        [TestMethod]
        public void RefersToWithNoEqualSignCurrencyValueTest()
        {
            AssertRefersToFieldCorrectness(@"=""$5""", "$5");
        }

        [TestMethod]
        public void RefersToWithNoEqualSignWrongValueTest()
        {
            AssertRefersToFieldCorrectness(@"=""3-""", "3-");
        }

        [TestMethod]
        public void RefersToWithEqualsSignTest()
        {
            AssertRefersToFieldCorrectness(@"=""=""", "=");
        }

        [TestMethod]
        public void RefersToWithEqualSignDigitsTest()
        {
            AssertRefersToFieldCorrectness(@"=15", "=15");
        }

        [TestMethod]
        public void RefersToWithEqualSignLettersTest()
        {
            AssertRefersToFieldCorrectness(@"=abc", "=abc");
        }

        [TestMethod]
        public void RefersToWithEqualWrongValueTest()
        {
            AssertRefersToFieldException("=3-");
        }
    }
}
