﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class ExpressionParserTests : MultiCulturalTestClassBase
    {
        #region Helper Methods

        public void AssertStringNotParsed(string inputString, CellIndex cellIndex = null)
        {  
            RunTestInMultipleCultures(() =>
            {
                ParseResult result = this.Parse(inputString, cellIndex);

                Assert.AreEqual(ParseResult.Error, result);
            });
        }

        public void AssertStringParsedCorrectly(string inputString, CellIndex cellIndex = null)
        {
            RunTestInMultipleCultures(() =>
            {
                ParseResult result = this.Parse(inputString, cellIndex);
                
                Assert.AreEqual(ParseResult.Successful, result);
            });
        }

        private ParseResult Parse(string inputString, CellIndex cellIndex = null)
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            cellIndex = cellIndex ?? new CellIndex(0, 0);

            InputStringCollection collection;
            RadExpression expression;

            return RadExpression.TryParse(SpreadsheetCultureHelper.FunctionStart + inputString.ReplaceListSeparator(),
                worksheet, cellIndex.RowIndex, cellIndex.ColumnIndex, out expression, out collection);
        }

        #endregion


        #region Parse CellReference Operators

        [TestMethod]
        public void UnionNotInFunctionParseTest()
        {
            AssertStringNotParsed("{ls}");
        }

        [TestMethod]
        public void UnionWithoutLeftOperandParseTest()
        {
            AssertStringNotParsed("{ls}A1");
        }

        [TestMethod]
        public void UnionWithoutRightOperandParseTest()
        {
            AssertStringNotParsed("A1{ls}");
        }

        [TestMethod]
        public void MultipleUnionsNotInFunctionParseTest()
        {
            AssertStringParsedCorrectly("A1{ls}B1{ls}C1{ls}D1");
        }

        [TestMethod]
        public void UnionInParenthesisParseTest()
        {
            AssertStringParsedCorrectly("(  A1  {ls} B1 )");
        }

        [TestMethod]
        public void MultipleUnionsWithSpacesNotInFunctionParseTest()
        {
            AssertStringParsedCorrectly("A1 {ls}  B1  {ls} C1    {ls} D1");
        }

        [TestMethod]
        public void UnionInFunctionParseTest()
        {
            AssertStringParsedCorrectly("SUM((A1{ls}B1))");
        }
        
        [TestMethod]
        public void UnionWithSpacesInFunctionParseTest()
        {
            AssertStringParsedCorrectly("SUM(  (  A1  {ls} B1 ) )");
        }

        [TestMethod]
        public void UnionInFunctionAndListSeparatorsParseTest()
        {
            AssertStringParsedCorrectly("SUM((A1{ls}B1){ls}A1{ls}(A1{ls}B1))");
        }

        [TestMethod]
        public void MissingOperandInFunction()
        {
            AssertStringParsedCorrectly("SUM({ls})");
        }

        [TestMethod]
        public void TwoMissingOperandsInFunction()
        {
            AssertStringParsedCorrectly("SUM({ls}{ls})");
        }

        [TestMethod]
        public void TwoMissingOperandsWithSpacesInFunction()
        {
            AssertStringParsedCorrectly("SUM(  {ls}  {ls}  )");
        }

        [TestMethod]
        public void UnionWithNoLeftOperandInFunction()
        {
            AssertStringNotParsed("SUM(({ls}A1))");
        }

        [TestMethod]
        public void UnionWithNoRightOperandInFunction()
        {
            AssertStringNotParsed("SUM((A1{ls}))");
        }

        [TestMethod]
        public void UnionWithEmptyParenthesisLeftOperandInFunction()
        {
            AssertStringNotParsed("SUM(((){ls}A1))");
        }

        [TestMethod]
        public void UnionWithEmptyParenthesisRightOperandInFunction()
        {
            AssertStringNotParsed("SUM((A1{ls}()))");
        }

        [TestMethod]
        public void UnionWithNoOperandsInFunction()
        {
            AssertStringNotParsed("SUM(({ls}))");
        }

        [TestMethod]
        public void TwoUnionWithNoOperandsInFunction()
        {
            AssertStringNotParsed("SUM(({ls}{ls}))");
        }

        [TestMethod]
        public void TwoUnionWithNoOperandsAndSpacesInFunction()
        {
            AssertStringNotParsed("SUM(( {ls} {ls} ))");
        }

        #endregion
    }
}
