﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.ExpressionParser
{
    [TestClass]
    public class ParserTestsAgainstExcel : MultiCulturalTestClassBase
    {
        #region Helper Methods

        private void AssertValuesAreParsed(string fileName, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();

                using (StreamReader reader = new StreamReader(TestHelperInternal.GetResourceStream(fileName)))
                {
                    StringBuilder builder = new StringBuilder();

                    while (!reader.EndOfStream)
                    {
                        string line = reader.ReadLine();
                        string[] values = line.Split('\t');

                        string input = values[0].ReplaceGroupDecimalSeparators().ReplaceListSeparator();
                        bool shouldBeParsed = values.Length > 1;


                        FormulaCellValue formulaCellValue;
                        bool isActuallyParsed = TryParseExpression(input, out formulaCellValue, worksheet, new CellIndex(10, 10));

                        if (shouldBeParsed != isActuallyParsed)
                        {
                            builder.AppendLine(string.Format("The value {0} is {1}parsed in excel and {2}parsed in RadSpreadsheet",
                                input, shouldBeParsed ? "" : "not ", isActuallyParsed ? "" : "not "));
                        }
                        else if (shouldBeParsed)
                        {
                            string expectedValue = values[1].ReplaceGroupDecimalSeparators().ReplaceListSeparator();
                            string actualValue = formulaCellValue.GetResultValueAsString(CellValueFormat.GeneralFormat);

                            if (!string.Equals(expectedValue, actualValue))
                            {
                                builder.AppendLine(string.Format("The result of the value {0} in excel is {1} and in RadSpreadsheet it is {2}",
                                    input, expectedValue, actualValue));
                            }
                        }
                    }

                    string errors = builder.ToString();
                    Assert.IsTrue(string.Equals(errors, string.Empty), errors);
                }
            }, cultures);
        }

        public static bool TryParseExpression(string cellInput, out FormulaCellValue formulaCellValue, Worksheet worksheet = null, CellIndex cellIndex = null)
        {
            formulaCellValue = null;

            if (worksheet == null)
            {
                worksheet = TestHelper.CreateWorksheet();
            }

            if (cellIndex == null)
            {
                cellIndex = new CellIndex(0, 0);
            }

            ICellValue cellValue;
            CellValueFormat format;
            CellValueFactory.TryCreate(SpreadsheetCultureHelper.FunctionStart + cellInput, 
                worksheet, cellIndex.RowIndex, cellIndex.ColumnIndex, CellValueFormat.GeneralFormat, out cellValue, out format);

            if (cellValue != null)
            {
                formulaCellValue = cellValue as FormulaCellValue;
                return formulaCellValue != null;
            }

            return false;
        }

        #endregion


        #region Test Methods

#if !(DEBUG && SILVERLIGHT)
        //These test are extremely slow only in silverlight debug

        [TestMethod]
        public void ParseOneToFourCharValues()
        {
            AssertValuesAreParsed("TestDataResources/ParsedOneToFourCharValues.txt", new string[] { "en-US" });
        }

        [TestMethod]
        public void ParsePatternsValues()
        {
            AssertValuesAreParsed("TestDataResources/ParsedPatternValues.txt", new string[] { "en-US" });
        }

#endif

        #endregion
    }
}
