﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class RadExpressionToStringTests : MultiCulturalTestClassBase
    {
        private void AssertRadExpressionStringBuildCorrectly(string expectedString, string input)
        {
            RunTestInMultipleCultures(() =>
            {
                Worksheet worksheet = TestHelper.CreateWorksheet();
                CellIndex cellIndex = new CellIndex(0, 0);
                RadExpression expression;

                RadExpression.TryParse(input.ReplaceGroupDecimalSeparators().ReplaceListSeparator(), worksheet, cellIndex, out expression);

                string exptectedStringLocalized = SpreadsheetCultureHelper.ClearFormulaValue(expectedString.ReplaceGroupDecimalSeparators().ReplaceListSeparator());
                string actualString = expression.ToString();
                Assert.AreEqual(exptectedStringLocalized, actualString);
            });
        }

        [TestMethod]
        public void AdditionNoParenthesisStringTest()
        {
            string input = "=1+2+3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void SubtractionNoParenthesisStringTest()
        {
            string input = "=1-2-3";
            string expectedString = "=(1-2)-3";
            AssertRadExpressionStringBuildCorrectly(expectedString, input);
        }

        [TestMethod]
        public void SubtractionLeftOperandInParenthesisStringTest()
        {
            string input = "=(1-2)-3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void SubtractionRightOperandInParenthesisStringTest()
        {
            string input = "=1-(2-3)";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void SubtractionAndAdditionNoParenthesisStringTest()
        {
            string input = "=1-2+3";
            string exptectedString = "=(1-2)+3";
            AssertRadExpressionStringBuildCorrectly(exptectedString, input);
        }

        [TestMethod]
        public void SubtractionAndAdditionRightOperandParenthesisStringTest()
        {
            string input = "=1-(2+3)";
            string exptectedString = "=1-(2+3)";
            AssertRadExpressionStringBuildCorrectly(exptectedString, input);
        }

        [TestMethod]
        public void AdditionAndSubtractionNoParenthesisStringTest()
        {
            string input = "=1+2-3";
            string exptectedString = "=(1+2)-3";
            AssertRadExpressionStringBuildCorrectly(exptectedString, input);
        }

        [TestMethod]
        public void MultiplicationNoParenthesisStringTest()
        {
            string input = "=1*2*3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void DivisionLeftOperandInParenthesisStringTest()
        {
            string input = "=(1/2)/3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void DivisionRightOperandInParenthesisStringTest()
        {
            string input = "=1/(2/3)";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void MultiplicationAndAdditionRightOperandInParenthesisStringTest()
        {
            string input = "=1*(2+3)";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void AdditionAndMultiplicationRightOperandInParenthesisStringTest()
        {
            string input = "=1+(2*3)";
            string expectedString = "=1+2*3";
            AssertRadExpressionStringBuildCorrectly(expectedString, input);
        }

        [TestMethod]
        public void DivisionAndMultiplicationRightOperandInParenthesisStringTest()
        {
            string input = "=1/(2*3)";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void MultiplicationAndDivisionRightOperandInParenthesisStringTest()
        {
            string input = "=1*(2/3)";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void DivisionAndMultiplicationLeftOperandInParenthesisStringTest()
        {
            string input = "=(1/2)*3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void MultiplicationAndDivisionLeftOperandInParenthesisStringTest()
        {
            string input = "=(1*2)/3";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void ThreeUnionsStringTest()
        {
            string input = "=A2:B2{ls}C2:D2{ls}E2:F2";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void UnionAndIntersectionStringTest()
        {
            string input = "=A2:B2{ls}C2:D2 E2:F2";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }

        [TestMethod]
        public void ThreeIntersectionStringTest()
        {
            string input = "=A2:B2 C2:D2 E2:F2";
            AssertRadExpressionStringBuildCorrectly(input, input);
        }
    }
}
