﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.DataSeries;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class FillSeriesGrowthTests
    {
        #region Helper Methods

        private static void AssertExponentialFillTrendResult(double?[] initialValues, double[] expectedValues)
        {
            double[] result = FillDataSeriesHelper.FillExponentialTrend(initialValues, expectedValues.Length);

            Assert.AreEqual(expectedValues.Length, result.Length);

            for (int i = 0; i < expectedValues.Length; i++)
            {
                Assert.IsTrue(expectedValues[i].EqualsDouble(result[i]));
            }
        }

        private static void AssertExponentialFillResult(double?[] initialValues, double[] expectedValues, double stepValue, double? stopValue = null)
        {
            double[] result = FillDataSeriesHelper.FillExponential(initialValues[0].Value, expectedValues.Length, stepValue, stopValue);

            Assert.AreEqual(expectedValues.Length, result.Length);

            for (int i = 0; i < expectedValues.Length; i++)
            {
                Assert.IsTrue(expectedValues[i].EqualsDouble(result[i]));
            }
        }

        private static void AssertStepResult(double?[] simpleInitialValues, double expectedStep)
        {
            double defaultStep = FillDataSeriesHelper.GetStepValue(simpleInitialValues);

            Assert.IsTrue(expectedStep.EqualsDouble(defaultStep));
        }

        #endregion


        #region Growth Trend Tests

        [TestMethod]
        public void SimpleFillSeriesTrendGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                2, 3, 5, 7
            };

            var expectedResult = new double[]
            {
                 2.006535701, 3.07504998440296, 4.71256624109901, 7.22208766992057, 11.0679718105893, 16.9618544662927, 25.9943295717859,
                 39.8367508239981, 61.0504961026499, 93.5609204386817, 143.383696974632, 219.737946802128, 336.752129311867, 516.078348079741, 790.898818965028
            };

            AssertExponentialFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesTrendGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                7, 5, 3, 2
            };

            var expectedResult = new double[]
            {
                7.22208767, 4.71256624109901, 3.07504998440296, 2.00653570110273, 1.30930734141595, 0.854350965867987, 0.557482227274609, 0.363769043570182,
                0.237367059586604, 0.154887069069472, 0.101067116080521, 0.0659484488550299, 0.0430327694610327, 0.0280797998988137, 0.0183226683346839
            };

            AssertExponentialFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillSeriesTrendGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 20, 100
            };

            var expectedResult = new double[]
            {
                1.02256518256357, 4.67624223911311, 21.3846919998238, 97.7932768542929, 447.213595499958, 2045.13036512715, 9352.48447822622, 42769.3839996476,
                195586.553708586, 894427.190999917, 4090260.7302543, 18704968.9564525, 85538767.9992952, 391173107.417172, 1788854381.99983
            };

            AssertExponentialFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void ReversedFillSeriesTrendGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                100, 20, 5, 1
            };

            var expectedResult = new double[]
            {
                97.7932768542928, 21.3846919998237, 4.6762422391131, 1.02256518256357, 0.223606797749979, 0.0488966384271463, 0.0106923459999119, 0.00233812111955655,
                0.000511282591281785, 0.000111803398874989, 0.0000244483192135731
            };

            AssertExponentialFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Growth Tests

        [TestMethod]
        public void SimpleFillSeriesGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                2, 3, 5, 7
            };

            var expectedStep = 1.7;

            var expectedResult = new double[]
            {
                 2, 3.4, 5.78, 9.826, 16.7042, 28.39714, 48.275138, 82.0677346, 139.51514882, 237.175752994, 403.1987800898, 685.43792615266,
                 1165.24447445952, 1980.91560658119, 3367.55653118802
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                7, 5, 3, 2
            };

            var expectedStep = -1.7;

            var expectedResult = new double[]
            {
                7, -11.9, 20.23, -34.391, 58.4647, -99.38999, 168.962983, -287.2370711,
                488.30302087, -830.115135479, 1411.1957303143, -2399.03274153431, 4078.35566060832, -6933.20462303415, 11786.4478591581
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        [TestMethod]
        public void FillSeriesGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 20, 100
            };

            var expectedStep = 31.2;

            var expectedResult = new double[]
            {
                1, 31.2, 973.44, 30371.328, 947585.4336, 29564665.52832, 922417564.483584, 28779428011.8878, 897918153970.9, 28015046403892.1, 874069447801433,
                27270966771404700.0
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        [TestMethod]
        public void ReversedFillSeriesGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                100, 20, 5, 1
            };

            var expectedStep = -31.2;

            var expectedResult = new double[]
            {
                100, -3120, 97344, -3037132.8, 94758543.36, -2956466552.832, 92241756448.3584, -2877942801188.78, 89791815397090
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        #endregion


        #region Step Growth Tests

        [TestMethod]
        public void SimpleFillSeriesStepGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                2, 3, 5, 7
            };

            var step = 2;

            var expectedResult = new double[]
            {
                 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesStepGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                7, 5, 3, 2
            };

            var step = -2;

            var expectedResult = new double[]
            {
                7, -14, 28, -56, 112, -224, 448, -896, 1792, -3584, 7168, -14336, 28672, -57344, 114688
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void FillSeriesStepGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 20, 100
            };

            var step = 33;

            var expectedResult = new double[]
            {
                1, 33, 1089, 35937, 1185921, 39135393, 1291467969, 42618442977, 1406408618241
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void ReversedFillSeriesStepGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                100, 20, 5, 1
            };

            var step = -33;

            var expectedResult = new double[]
            {
                100, -3300, 108900, -3593700, 118592100, -3913539300, 129146796900, -4261844297700, 140640861824100
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step);
        }

        #endregion


        #region Step StopValue Growth Tests

        [TestMethod]
        public void SimpleFillSeriesStepStopValueGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                2, 3, 5, 7
            };

            double step = 2;
            double stopValue = 5000;

            var expectedResult = new double[]
            {
                 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        [TestMethod]
        public void FillSeriesStepStopValueGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 20, 100
            };

            double step = 33;
            double stopValue = 50000000000;

            var expectedResult = new double[]
            {
                1, 33, 1089, 35937, 1185921, 39135393, 1291467969, 42618442977
            };

            AssertExponentialFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        #endregion


        #region Stop Value Growth Tests

        [TestMethod]
        public void SimpleFillSeriesStopValueGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                2, 3, 5, 7
            };

            var expectedStep = 1.7;
            double stopValue = 3000;

            var expectedResult = new double[]
            {
                 2, 3.4, 5.78, 9.826, 16.7042, 28.39714, 48.275138, 82.0677346, 139.51514882, 237.175752994, 403.1987800898, 685.43792615266,
                 1165.24447445952, 1980.91560658119
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        }

        [TestMethod]
        public void FillSeriesStopValueGrowthTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 20, 100
            };

            var expectedStep = 31.2;

            double stopValue = 50000000000;

            var expectedResult = new double[]
            {
                1, 31.2, 973.44, 30371.328, 947585.4336, 29564665.52832, 922417564.483584, 28779428011.8878
            };

            AssertStepResult(simpleInitialValues, expectedStep);

            AssertExponentialFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        }

        #endregion


        #region Growth Trend Empty Values Tests

        [TestMethod]
        public void SimpleFillSeriesGrowthTrendEmptyValuesTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, null, null, 2, 3
            };

            var expectedResult = new double[]
            {
                0.980076226, 1.276938836, 1.66372038, 2.167657074, 2.824234919, 3.679688533, 4.794256882, 6.246425165, 8.138451547,
                10.60356794, 13.81536184, 18, 23.45215447, 30.55575274, 39.81101296
            };

            AssertExponentialFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion
    }
}