﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.DataSeries;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.Tests
{
    [TestClass]
    public class FillSeriesLinearTests
    {
        #region Helper Methods

        private static void AssertLinearFillTrendResult(double?[] initialValues, double[] expectedValues)
        {
            double[] result = FillDataSeriesHelper.FillLinearTrend(initialValues, expectedValues.Length);

            Assert.AreEqual(expectedValues.Length, result.Length);

            for (int i = 0; i < expectedValues.Length; i++)
            {
                Assert.IsTrue(expectedValues[i].EqualsDouble(result[i]));
            }
        }

        private static void AssertLinearFillResult(double?[] initialValues, double[] expectedValues, double stepValue, double? stopValue = null)
        {
            double[] result = FillDataSeriesHelper.FillLinear(initialValues[0].Value, expectedValues.Length, stepValue, stopValue);

            Assert.AreEqual(expectedValues.Length, result.Length);

            for (int i = 0; i < expectedValues.Length; i++)
            {
                Assert.IsTrue(expectedValues[i].EqualsDouble(result[i]));
            }
        }

        private static void AssertStepResult(double?[] simpleInitialValues, double expectedStep)
        {
            double defaultStep = FillDataSeriesHelper.GetStepValue(simpleInitialValues);

            Assert.AreEqual(expectedStep, defaultStep);
        }

        #endregion


        #region Linear Trend One Initial Value Tests

        [TestMethod]
        public void FillLinearTrendOneInitialValue1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1
            };

            var expectedResult = new double[]
            {
                1, 2, 3, 4, 5, 6, 7, 8, 9, 10
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue2()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0.5
            };

            var expectedResult = new double[]
            {
                0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5, 9.5, 10.5, 11.5, 12.5, 13.5, 14.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue3()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0.0000001
            };

            var expectedResult = new double[]
            {
                0.0000001, 1.0000001, 2.0000001, 3.0000001, 4.0000001, 5.0000001, 6.0000001, 7.0000001, 8.0000001, 9.0000001, 10.0000001, 11.0000001, 12.0000001
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue4()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0.0000001
            };

            var expectedResult = new double[]
            {
                0.0000001, 1.0000001, 2.0000001, 3.0000001, 4.0000001, 5.0000001, 6.0000001, 7.0000001, 8.0000001, 9.0000001, 10.0000001, 11.0000001, 12.0000001
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue5()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1.0000001
            };

            var expectedResult = new double[]
            {
                1.0000001, 2.0000001, 3.0000001, 4.0000001, 5.0000001, 6.0000001, 7.0000001, 8.0000001, 9.0000001, 10.0000001, 11.0000001, 12.0000001, 13.0000001
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue6()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0
            };

            var expectedResult = new double[]
            {
                0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue7()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -0.5
            };

            var expectedResult = new double[]
            {
                -0.5, 0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5, 9.5, 10.5, 11.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue8()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -0.0000010
            };

            var expectedResult = new double[]
            {
                -0.0000010, 0.9999990, 1.9999990, 2.9999990, 3.9999990, 4.9999990, 5.9999990, 6.9999990, 7.9999990, 8.9999990, 9.9999990, 10.9999990, 11.9999990
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue9()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -1.0000001
            };

            var expectedResult = new double[]
            {
                -1.0000001, -0.0000001, 0.9999999, 1.9999999, 2.9999999, 3.9999999, 4.9999999, 5.9999999, 6.9999999, 7.9999999, 8.9999999, 9.9999999, 10.9999999
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendOneInitialValue10()
        {
            double?[] simpleInitialValues = new double?[]
            {
                99999999
            };

            var expectedResult = new double[]
            {
                99999999, 100000000, 100000001, 100000002, 100000003, 100000004, 100000005, 100000006, 100000007, 100000008, 100000009, 100000010, 100000011
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear Trend Two Initial Values Tests

        [TestMethod]
        public void FillLinearTrendTwoInitialValue1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 2
            };

            var expectedResult = new double[]
            {
                1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue2()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 1.5
            };

            var expectedResult = new double[]
            {
                1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5, 7, 7.5, 8, 8.5, 9, 9.5, 10, 10.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue3()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 3
            };

            var expectedResult = new double[]
            {
                1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue4()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 1
            };

            var expectedResult = new double[]
            {
                5, 1, -3, -7, -11, -15, -19, -23, -27, -31, -35, -39, -43, -47, -51, -55, -59, -63, -67, -71
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue5()
        {
            double?[] simpleInitialValues = new double?[]
            {
                7, 11
            };

            var expectedResult = new double[]
            {
                7, 11, 15, 19, 23, 27, 31, 35, 39, 43, 47, 51, 55, 59, 63, 67, 71, 75, 79, 83
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        // TODO:
        //[TestMethod]
        //public void FillLinearTrendTwoInitialValue6()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        -22, -11
        //    };

        //    var expectedResult = new double[]
        //    {
        //        -22, -11, -0.000000000000007105427357601, 11, 22, 33, 44, 55, 66, 77, 88, 99, 110, 121, 132, 143, 154, 165, 176, 187
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        [TestMethod]
        public void FillLinearTrendTwoInitialValue7()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -22, 15
            };

            var expectedResult = new double[]
            {
                -22, 15, 52, 89, 126, 163, 200, 237, 274, 311, 348, 385, 422, 459, 496, 533, 570, 607, 644, 681
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue8()
        {
            double?[] simpleInitialValues = new double?[]
            {
                15, -22
            };

            var expectedResult = new double[]
            {
                15, -22, -59, -96, -133, -170, -207, -244, -281, -318, -355, -392, -429, -466, -503, -540, -577, -614, -651, -688
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        // TODO:
        //[TestMethod]
        //public void FillLinearTrendTwoInitialValue9()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        0, -11
        //    };

        //    var expectedResult = new double[]
        //    {
        //        -0.00000000000000177635683940025, -11, -22, -33, -44, -55, -66, -77, -88, -99, -110, -121, -132, -143, -154, -165, -176, -187, -198, -209
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        [TestMethod]
        public void FillLinearTrendTwoInitialValue10()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0, 0
            };

            var expectedResult = new double[]
            {
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue11()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -5,0
            };

            var expectedResult = new double[]
            {
                -5, 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue12()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 1
            };

            var expectedResult = new double[]
            {
                1, 1, 1, 1, 1, 1
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendTwoInitialValue13()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -5, -5
            };

            var expectedResult = new double[]
            {
                -5, -5, -5, -5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear Trend Three Initial Values Tests

        [TestMethod]
        public void FillLinearTrendThreeInitialValue1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 2, 3
            };

            var expectedResult = new double[]
            {
                1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue2()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 5, 10
            };

            var expectedResult = new double[]
            {
                0.833333333333333, 5.33333333333333, 9.83333333333333, 14.3333333333333, 18.8333333333333, 23.3333333333333, 27.8333333333333, 32.3333333333333, 36.8333333333333, 41.3333333333333, 45.8333333333333, 50.3333333333333, 54.8333333333333, 59.3333333333333, 63.8333333333333
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue3()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, 3, 5
            };

            var expectedResult = new double[]
            {
                1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue4()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 1, 10
            };

            var expectedResult = new double[]
            {
                2.83333333333333, 5.33333333333333, 7.83333333333333, 10.3333333333333, 12.8333333333333, 15.3333333333333, 17.8333333333333, 20.3333333333333, 22.8333333333333, 25.3333333333333, 27.8333333333333, 30.3333333333333, 32.8333333333333, 35.3333333333333, 37.8333333333333
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue5()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0, 0, 1
            };

            var expectedResult = new double[]
            {
                -0.166666666666667, 0.333333333333333, 0.833333333333333, 1.333333333333330, 1.833333333333330, 2.333333333333330, 2.833333333333330, 3.333333333333330, 3.833333333333330, 4.333333333333330, 4.833333333333330, 5.333333333333330, 5.833333333333330, 6.333333333333330, 6.833333333333330
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        // TODO:
        //[TestMethod]
        //public void FillLinearTrendThreeInitialValue6()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        1, 0, 5
        //    };

        //    var expectedResult = new double[]
        //    {
        //        6.66133814775094E-16, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        [TestMethod]
        public void FillLinearTrendThreeInitialValue7()
        {
            double?[] simpleInitialValues = new double?[]
            {
                10, 1, 5
            };

            var expectedResult = new double[]
            {
                7.833333333333330, 5.333333333333330, 2.833333333333330, 0.333333333333332, -2.166666666666670, -4.666666666666670, -7.166666666666670, -9.666666666666670, -12.166666666666700, -14.666666666666700, -17.166666666666700, -19.666666666666700, -22.166666666666700, -24.666666666666700, -27.166666666666700
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue8()
        {
            double?[] simpleInitialValues = new double?[]
            {
                10, 7, 1
            };

            var expectedResult = new double[]
            {
                10.5, 6.0, 1.5, -3.0, -7.5, -12.0, -16.5, -21.0, -25.5, -30.0, -34.5, -39.0, -43.5, -48.0, -52.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue9()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -1, 0, 1
            };

            var expectedResult = new double[]
            {
                -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue10()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -5, -7, -10
            };

            var expectedResult = new double[]
            {
               -4.833333333333330, -7.333333333333330, -9.833333333333330, -12.333333333333300, -14.833333333333300, -17.333333333333300, -19.833333333333300, -22.333333333333300, -24.833333333333300, -27.333333333333300, -29.833333333333300, -32.333333333333300, -34.833333333333300, -37.333333333333300, -39.833333333333300
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue11()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -8, -20, -1
            };

            var expectedResult = new double[]
            {
                -13.166666666666700, -9.666666666666670, -6.166666666666670, -2.666666666666670, 0.833333333333329, 4.333333333333330, 7.833333333333330, 11.333333333333300, 14.833333333333300, 18.333333333333300, 21.833333333333300, 25.333333333333300, 28.833333333333300, 32.333333333333300, 35.833333333333300
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue12()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -50, 0, -100
            };

            var expectedResult = new double[]
            {
                -25, -50, -75, -100, -125, -150, -175, -200, -225, -250
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue13()
        {
            double?[] simpleInitialValues = new double?[]
            {
                90, -67, 0
            };

            var expectedResult = new double[]
            {
                52.666666666666700, 7.666666666666670, -37.333333333333300, -82.333333333333300, -127.333333333333000, -172.333333333333000, -217.333333333333000, -262.333333333333000, -307.333333333333000, -352.333333333333000, -397.333333333333000, -442.333333333333000, -487.333333333333000, -532.333333333333000, -577.333333333333000
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValue14()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0, -60, -20
            };

            var expectedResult = new double[]
            {
                -16.666666666666700, -26.666666666666700, -36.666666666666700, -46.666666666666700, -56.666666666666700, -66.666666666666700, -76.666666666666700, -86.666666666666700, -96.666666666666700, -106.666666666667000, -116.666666666667000, -126.666666666667000, -136.666666666667000, -146.666666666667000, -156.666666666667000
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear Trend Three Initial Values Containg Nulls Tests

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, null, 5
            };

            var expectedResult = new double[]
            {
                1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls2()
        {
            double?[] simpleInitialValues = new double?[]
            {
                10, null, 4
            };

            var expectedResult = new double[]
            {
                10, 7, 4, 1, -2, -5, -8, -11, -14, -17, -20, -23, -26, -29, -32, -35, -38, -41, -44, -47, -50, -53, -56
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls3()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -50, null, 50
            };

            var expectedResult = new double[]
            {
                -50, 0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 950, 1000, 1050
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls4()
        {
            double?[] simpleInitialValues = new double?[]
            {
                23, null, -44
            };

            var expectedResult = new double[]
            {
                23, -10.5, -44, -77.5, -111, -144.5, -178, -211.5, -245, -278.5, -312, -345.5, -379, -412.5, -446, -479.5, -513, -546.5, -580, -613.5, -647, -680.5, -714
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        // TODO:
        //[TestMethod]
        //public void FillLinearTrendThreeInitialValuesContainingNulls5()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        0, null, 14
        //    };

        //    var expectedResult = new double[]
        //    {
        //        -8.88178E-16, 7, 14, 21, 28, 35, 42, 49, 56, 63, 70, 77, 84, 91, 98, 105, 112, 119, 126, 133, 140, 147, 154
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        //[TestMethod]
        //public void FillLinearTrendThreeInitialValuesContainingNulls6()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        0, null, -15
        //    };

        //    var expectedResult = new double[]
        //    {
        //        3.55271E-15, -7.5, -15, -22.5, -30, -37.5, -45, -52.5, -60, -67.5, -75, -82.5, -90, -97.5, -105, -112.5, -120, -127.5, -135, -142.5, -150, -157.5, -165
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        //[TestMethod]
        //public void FillLinearTrendThreeInitialValuesContainingNulls7()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        -15, null, 0
        //    };

        //    var expectedResult = new double[]
        //    {
        //        -15, -7.5, 3.55271E-15, 7.5, 15, 22.5, 30, 37.5, 45, 52.5, 60, 67.5, 75, 82.5, 90, 97.5, 105, 112.5, 120, 127.5, 135, 142.5, 150
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        //[TestMethod]
        //public void FillLinearTrendThreeInitialValuesContainingNulls8()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        55, null, 0
        //    };

        //    var expectedResult = new double[]
        //    {
        //        55, 27.5, -1.42109E-14, -27.5, -55, -82.5, -110, -137.5, -165, -192.5, -220, -247.5, -275, -302.5, -330, -357.5, -385, -412.5, -440, -467.5, -495, -522.5, -550
        //    };

        //    AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        //}

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls9()
        {
            double?[] simpleInitialValues = new double?[]
            {
                100000, null, 100000
            };

            var expectedResult = new double[]
            {
                100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000, 100000
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls10()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, null, 5
            };

            var expectedResult = new double[]
            {
                5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls11()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0, null, 0
            };

            var expectedResult = new double[]
            {
                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls12()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -100000, null, -100000
            };

            var expectedResult = new double[]
            {
                -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000, -100000
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls13()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0.12345, null, 0.12345
            };

            var expectedResult = new double[]
            {
                0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345, 0.12345
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls14()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0.12345, null, 55.1231234
            };

            var expectedResult = new double[]
            {
                0.12345, 27.6232867, 55.1231234, 82.6229601, 110.1227968, 137.6226335, 165.1224702, 192.6223069, 220.1221436, 247.6219803, 275.121817, 302.6216537, 330.1214904, 357.6213271, 385.1211638, 412.6210005, 440.1208372, 467.6206739, 495.1205106, 522.6203473, 550.120184, 577.6200207, 605.1198574
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls15()
        {
            double?[] simpleInitialValues = new double?[]
            {
                55.1231234, null, 0.12345
            };

            var expectedResult = new double[]
            {
                55.1231234, 27.6232867, 0.12345, -27.3763867, -54.8762234, -82.3760601, -109.8758968, -137.3757335, -164.8755702, -192.3754069, -219.8752436, -247.3750803, -274.874917, -302.3747537, -329.8745904, -357.3744271, -384.8742638, -412.3741005, -439.8739372, -467.3737739, -494.8736106, -522.3734473, -549.873284
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls16()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -0.12345, null, 55.1231234
            };

            var expectedResult = new double[]
            {
                -0.12345, 27.4998367, 55.1231234, 82.7464101, 110.3696968, 137.9929835, 165.6162702, 193.2395569, 220.8628436, 248.4861303, 276.109417, 303.7327037, 331.3559904, 358.9792771, 386.6025638, 414.2258505, 441.8491372, 469.4724239, 497.0957106, 524.7189973, 552.342284, 579.9655707, 607.5888574
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls17()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -55.1231234, null, 0.12345
            };

            var expectedResult = new double[]
            {
                -55.1231234, -27.4998367, 0.12345, 27.7467367, 55.3700234, 82.9933101, 110.6165968, 138.2398835, 165.8631702, 193.4864569, 221.1097436, 248.7330303, 276.356317, 303.9796037, 331.6028904, 359.2261771, 386.8494638, 414.4727505, 442.0960372, 469.7193239, 497.3426106, 524.9658973, 552.589184
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls18()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -0.97645, null, -100.94234
            };

            var expectedResult = new double[]
            {
                -0.97645, -50.959395, -100.94234, -150.925285, -200.90823, -250.891175, -300.87412, -350.857065, -400.84001, -450.822955, -500.8059, -550.788845, -600.77179, -650.754735, -700.73768, -750.720625, -800.70357, -850.686515, -900.66946, -950.652405, -1000.63535, -1050.618295, -1100.60124
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendThreeInitialValuesContainingNulls19()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -55.1231234, null, -1.312567
            };

            var expectedResult = new double[]
            {
                -55.1231234, -28.2178452, -1.312567, 25.5927112, 52.4979894, 79.4032676, 106.3085458, 133.213824, 160.1191022, 187.0243804, 213.9296586, 240.8349368, 267.740215, 294.6454932, 321.5507714, 348.4560496, 375.3613278, 402.266606, 429.1718842, 456.0771624, 482.9824406, 509.8877188, 536.792997
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear Trend Misc

        [TestMethod]
        public void FillLinearTrendMisc1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, null, null, 2, 5, null, 1, null
            };

            var expectedResult = new double[]
            {
                1.773333333, 1.92, 2.066666667, 2.213333333, 2.36, 2.506666667, 2.653333333, 2.8, 2.946666667, 3.093333333, 3.24, 3.386666667, 3.533333333, 3.68, 3.826666667, 3.973333333, 4.12, 4.266666667, 4.413333333, 4.56, 4.706666667, 4.853333333, 5, 5.146666667, 5.293333333
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc2()
        {
            double?[] simpleInitialValues = new double?[]
            {
                4, 6, null, 2, null, null, 10, null
            };

            var expectedResult = new double[]
            {
                3.476190476, 4.285714286, 5.095238095, 5.904761905, 6.714285714, 7.523809524, 8.333333333, 9.142857143, 9.952380952, 10.76190476, 11.57142857, 12.38095238, 13.19047619, 14, 14.80952381, 15.61904762, 16.42857143, 17.23809524, 18.04761905, 18.85714286, 19.66666667, 20.47619048, 21.28571429, 22.0952381, 22.9047619
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc3()
        {
            double?[] simpleInitialValues = new double?[]
            {
                345, 123, null, null, null, -123, null, 65
            };

            var expectedResult = new double[]
            {
                238.4045802, 196.5877863, 154.7709924, 112.9541985, 71.13740458, 29.32061069, -12.49618321, -54.3129771, -96.12977099, -137.9465649, -179.7633588, -221.5801527, -263.3969466, -305.2137405, -347.0305344, -388.8473282, -430.6641221, -472.480916, -514.2977099, -556.1145038, -597.9312977, -639.7480916, -681.5648855, -723.3816794, -765.1984733
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc4()
        {
            double?[] simpleInitialValues = new double?[]
            {
                -34523, 7456, null, null, 123, null, 123, 568
            };

            var expectedResult = new double[]
            {
                -15622.16129, -12741.17204, -9860.182796, -6979.193548, -4098.204301, -1217.215054, 1663.774194, 4544.763441, 7425.752688, 10306.74194, 13187.73118, 16068.72043, 18949.70968, 21830.69892, 24711.68817, 27592.67742, 30473.66667, 33354.65591, 36235.64516, 39116.63441, 41997.62366, 44878.6129, 47759.60215, 50640.5914, 53521.58065
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc5()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, null, -4345, null, null, 957, null, -783
            };

            var expectedResult = new double[]
            {
                -1671.172414, -1491.551724, -1311.931034, -1132.310345, -952.6896552, -773.0689655, -593.4482759, -413.8275862, -234.2068966, -54.5862069, 125.0344828, 304.6551724, 484.2758621, 663.8965517, 843.5172414, 1023.137931, 1202.758621, 1382.37931, 1562, 1741.62069, 1921.241379, 2100.862069, 2280.482759, 2460.103448, 2639.724138
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc6()
        {
            double?[] simpleInitialValues = new double?[]
            {
                456, null, 0, null, null, -763, null, 136
            };

            var expectedResult = new double[]
            {
                230.5517241, 152.4655172, 74.37931034, -3.706896552, -81.79310345, -159.8793103, -237.9655172, -316.0517241, -394.137931, -472.2241379, -550.3103448, -628.3965517, -706.4827586, -784.5689655, -862.6551724, -940.7413793, -1018.827586, -1096.913793, -1175, -1253.086207, -1331.172414, -1409.258621, -1487.344828, -1565.431034, -1643.517241
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc7()
        {
            double?[] simpleInitialValues = new double?[]
            {
                0, null, null, 63456, null, -123, null, -555
            };

            var expectedResult = new double[]
            {
                22640.69159, 20788.37383, 18936.05607, 17083.73832, 15231.42056, 13379.1028, 11526.78505, 9674.46729, 7822.149533, 5969.831776, 4117.514019, 2265.196262, 412.8785047, -1439.439252, -3291.757009, -5144.074766, -6996.392523, -8848.71028, -10701.02804, -12553.34579, -14405.66355, -16257.98131, -18110.29907, -19962.61682, -21814.93458
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillLinearTrendMisc8()
        {
            double?[] simpleInitialValues = new double?[]
            {
                76, null, 1234, 6, null, null, null, 0
            };

            var expectedResult = new double[]
            {
                497.6923077, 441.4615385, 385.2307692, 329, 272.7692308, 216.5384615, 160.3076923, 104.0769231, 47.84615385, -8.384615385, -64.61538462, -120.8461538, -177.0769231, -233.3076923, -289.5384615, -345.7692308, -402, -458.2307692, -514.4615385, -570.6923077, -626.9230769, -683.1538462, -739.3846154, -795.6153846, -851.8461538
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear One Initial Value With Step

        [TestMethod]
        public void LinearOneInitialValueWithDefaultStep1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1
            };

            double expectedStep = 1;
            AssertStepResult(simpleInitialValues, expectedStep);

            var expectedResultWithDefaultStep = new double[]
            {
                1, 2, 3, 4, 5, 6, 7, 8, 9, 10
            };

            AssertLinearFillResult(simpleInitialValues, expectedResultWithDefaultStep, expectedStep);
        }

        [TestMethod]
        public void LinearOneInitialValueWithDefaultStepAndStopValue1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1
            };

            double expectedStep = 1;
            double stopValue = 7;

            AssertStepResult(simpleInitialValues, expectedStep);

            var expectedResultWithDefaultStep = new double[]
            {
                1, 2, 3, 4, 5, 6, 7
            };

            AssertLinearFillResult(simpleInitialValues, expectedResultWithDefaultStep, expectedStep, stopValue);
        }

        [TestMethod]
        public void LinearOneInitialValueWithExplicitStep1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1
            };

            double step = 1.4;

            var expectedResultWithChangedStep = new double[]
            {
                1, 2.4, 3.8, 5.2, 6.6, 8, 9.4, 10.8, 12.2, 13.6, 15, 16.4, 17.8, 19.2, 20.6, 22
            };

            AssertLinearFillResult(simpleInitialValues, expectedResultWithChangedStep, step);
        }

        [TestMethod]
        public void LinearOneInitialValueWithExplicitStepAndStopValue1()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1
            };

            double step = 1.4;
            double stopValue = 15;

            var expectedResultWithChangedStep = new double[]
            {
                1, 2.4, 3.8, 5.2, 6.6, 8, 9.4, 10.8, 12.2, 13.6, 15
            };

            AssertLinearFillResult(simpleInitialValues, expectedResultWithChangedStep, step, stopValue);
        }

        #endregion


        #region Others

        public void FillLinearTrendSingleInitialValue()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 10, 20, 30
            };

            var expectedResult = new double[]
            {
                3.5, 12, 20.5, 29, 37.5, 46, 54.5, 63, 71.5, 80, 88.5, 97, 105.5, 114, 122.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesLinearTrendTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                30, 20, 10, 5
            };

            var expectedResult = new double[]
            {
                29, 20.5, 12, 3.5, -5, -13.5, -22, -30.5, -39, -47.5, -56, -64.5, -73, -81.5, -90
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void FillSeriesLinearTrendTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 100, 20, 312
            };

            var expectedResult = new double[]
            {
                -16.9, 67.2, 151.3, 235.4, 319.5, 403.6, 487.7, 571.8, 655.9, 740, 824.1, 908.2, 992.3, 1076.4, 1160.5
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        [TestMethod]
        public void ReversedFillSeriesLinearTrendTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                312, 20, 100, 5
            };

            var expectedResult = new double[]
            {
                235.4, 151.3, 67.2, -16.9, -101, -185.1, -269.2, -353.3, -437.4, -521.5, -605.6, -689.7, -773.8, -857.9, -942
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion


        #region Linear Default Tests

        [TestMethod]
        public void SimpleFillSeriesLinearDefaultTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 10, 20, 30
            };

            double expectedStep = 8.5;

            AssertStepResult(simpleInitialValues, expectedStep);

            var expectedResult = new double[]
            {
                5, 13.5, 22, 30.5, 39, 47.5, 56, 64.5, 73, 81.5, 90, 98.5, 107, 115.5, 124
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesLinearDefaultTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                30, 20, 10, 5
            };

            double expectedStep = -8.5;
            AssertStepResult(simpleInitialValues, expectedStep);

            var expectedResult = new double[]
            {
                30, 21.5, 13, 4.5, -4, -12.5, -21, -29.5, -38, -46.5, -55, -63.5, -72, -80.5, -89
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep);
        }

        // TODO:
        //[TestMethod]
        //public void FillSeriesLinearDefaultTest()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        5, 100, 20, 312
        //    };

        //    double expectedStep = 84.1;
        //    AssertStepResult(simpleInitialValues, expectedStep);

        //    var expectedResult = new double[]
        //    {
        //        5, 89.1, 173.2, 257.3, 341.4, 425.5, 509.6, 593.7, 677.8, 761.9, 846, 930.1, 1014.2, 1098.3, 1182.4
        //    };

        //    AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep);
        //}

        //[TestMethod]
        //public void ReversedFillSeriesLinearDefaultTest()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        312, 20, 100, 5
        //    };

        //    double expectedStep = -84.1;
        //    AssertStepResult(simpleInitialValues, expectedStep);

        //    var expectedResult = new double[]
        //    {
        //        312, 227.9, 143.8, 59.7, -24.4, -108.5, -192.6, -276.7, -360.8, -444.9, -529, -613.1, -697.2, -781.3, -865.4
        //    };

        //    AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep);
        //}

        #endregion


        #region Linear Step Tests

        [TestMethod]
        public void SimpleFillSeriesLinearStepTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 10, 20, 30
            };

            double step = 7.5;

            var expectedResult = new double[]
            {
                5, 12.5, 20, 27.5, 35, 42.5, 50, 57.5, 65, 72.5, 80, 87.5, 95, 102.5, 110
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesLinearStepTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                30, 20, 10, 5
            };

            double step = -7.5;

            var expectedResult = new double[]
            {
                30, 22.5, 15, 7.5, 0, -7.5, -15, -22.5, -30, -37.5, -45, -52.5, -60, -67.5, -75
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void FillSeriesLinearStepTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 100, 20, 312
            };

            double step = 64.1;

            var expectedResult = new double[]
            {
                5, 69.1, 133.2, 197.3, 261.4, 325.5, 389.6, 453.7, 517.8, 581.9, 646, 710.1, 774.2, 838.3, 902.4
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step);
        }

        [TestMethod]
        public void ReversedFillSeriesLinearStepTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                312, 20, 100, 5
            };

            double step = -64.1;

            var expectedResult = new double[]
            {
                312, 247.9, 183.8, 119.7, 55.6, -8.5, -72.6, -136.7, -200.8, -264.9, -329, -393.1, -457.2, -521.3, -585.4
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step);
        }

        #endregion


        #region Linear Stop Value Tests

        [TestMethod]
        public void SimpleFillSeriesLinearStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 10, 20, 30
            };

            double expectedStep = 8.5;

            AssertStepResult(simpleInitialValues, expectedStep);

            double stopValue = 100;

            var expectedResult = new double[]
            {
                5, 13.5, 22, 30.5, 39, 47.5, 56, 64.5, 73, 81.5, 90, 98.5
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesLinearStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                30, 20, 10, 5
            };

            double expectedStep = -8.5;
            AssertStepResult(simpleInitialValues, expectedStep);

            double stopValue = -60;

            var expectedResult = new double[]
            {
                30, 21.5, 13, 4.5, -4, -12.5, -21, -29.5, -38, -46.5, -55
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        }

        // TODO:
        //[TestMethod]
        //public void FillSeriesLinearStopValueTest()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        5, 100, 20, 312
        //    };

        //    double expectedStep = 84.1;
        //    AssertStepResult(simpleInitialValues, expectedStep);

        //    double stopValue = 1000;

        //    var expectedResult = new double[]
        //    {
        //        5, 89.1, 173.2, 257.3, 341.4, 425.5, 509.6, 593.7, 677.8, 761.9, 846, 930.1
        //    };

        //    AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        //}

        //[TestMethod]
        //public void ReversedFillSeriesLinearStopValueTest()
        //{
        //    double?[] simpleInitialValues = new double?[]
        //    {
        //        312, 20, 100, 5
        //    };

        //    double expectedStep = -84.1;
        //    AssertStepResult(simpleInitialValues, expectedStep);

        //    double stopValue = -650;

        //    var expectedResult = new double[]
        //    {
        //        312, 227.9, 143.8, 59.7, -24.4, -108.5, -192.6, -276.7, -360.8, -444.9, -529, -613.1
        //    };

        //    AssertLinearFillResult(simpleInitialValues, expectedResult, expectedStep, stopValue);
        //}

        #endregion


        #region Linear Step Stop Value Tests

        [TestMethod]
        public void SimpleFillSeriesLinearStepStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 10, 20, 30
            };

            double step = 7.5;
            double stopValue = 100;

            var expectedResult = new double[]
            {
                 5, 12.5, 20, 27.5, 35, 42.5, 50, 57.5, 65, 72.5, 80, 87.5, 95
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        [TestMethod]
        public void ReversedSimpleFillSeriesLinearStepStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                30, 20, 10, 5
            };

            double step = -7.5;
            double stopValue = -60;

            var expectedResult = new double[]
            {
                30, 22.5, 15, 7.5, 0, -7.5, -15, -22.5, -30, -37.5, -45, -52.5, -60
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        [TestMethod]
        public void FillSeriesLinearStepStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                5, 100, 20, 312
            };

            double step = 64.1;
            double stopValue = 1000;

            var expectedResult = new double[]
            {
                5, 69.1, 133.2, 197.3, 261.4, 325.5, 389.6, 453.7, 517.8, 581.9, 646, 710.1, 774.2, 838.3, 902.4
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        [TestMethod]
        public void ReversedFillSeriesLinearStepStopValueTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                312, 20, 100, 5
            };

            double step = -64.1;
            double stopValue = -650;

            var expectedResult = new double[]
            {
                312, 247.9, 183.8, 119.7, 55.6, -8.5, -72.6, -136.7, -200.8, -264.9, -329, -393.1, -457.2, -521.3, -585.4
            };

            AssertLinearFillResult(simpleInitialValues, expectedResult, step, stopValue);
        }

        #endregion


        #region Linear Trend Empty Values Tests

        [TestMethod]
        public void SimpleFillSeriesLinearTrendEmptyValuesTest()
        {
            double?[] simpleInitialValues = new double?[]
            {
                1, null, null, 2, 3
            };

            var expectedResult = new double[]
            {
                0.923076923, 1.384615385, 1.846153846, 2.307692308, 2.769230769, 3.230769231, 3.692307692, 4.153846154, 4.615384615,
                5.076923077, 5.538461538, 6, 6.461538462, 6.923076923, 7.384615385
            };

            AssertLinearFillTrendResult(simpleInitialValues, expectedResult);
        }

        #endregion
    }
}