﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Filtering
{
    [TestClass]
    public class ColorFiltersTests
    {
        private static readonly IFill fill1 = new PatternFill(PatternType.Solid, Color.FromArgb(255, 255, 0, 0), Color.FromArgb(255, 255, 0, 0));
        private static readonly IFill fill2 = new PatternFill(PatternType.Solid, Color.FromArgb(255, 255, 0, 0), Color.FromArgb(255, 0, 0, 0));
        private static readonly IFill fill3 = new PatternFill(PatternType.HorizontalStripe, Color.FromArgb(255, 255, 0, 0), Color.FromArgb(255, 255, 0, 0));

        private static readonly ThemableColor foreColor1 = new ThemableColor(Color.FromArgb(255, 255, 0, 0));
        private static readonly ThemableColor foreColor2 = new ThemableColor(Color.FromArgb(255, 0, 255, 0));
        private static readonly ThemableColor foreColor3 = new ThemableColor(Color.FromArgb(255, 0, 0, 255));

        [TestMethod]
        public void FillColorFilterBasicTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[6, 5, 6, 5].SetFill(fill1);
            worksheet.Cells[7, 5, 7, 5].SetFill(fill2);
            worksheet.Cells[8, 5, 8, 5].SetFill(fill3);

            IFilter fillFilter = new FillColorFilter(0, fill1);
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            worksheet.Filter.SetFilter(fillFilter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {6, false},
                {7, true},
                {8, true}
            });
        }

        [TestMethod]
        public void ForeColorFilterBasicTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[6, 5, 6, 5].SetForeColor(foreColor1);
            worksheet.Cells[7, 5, 7, 5].SetForeColor(foreColor2);
            worksheet.Cells[8, 5, 8, 5].SetForeColor(foreColor3);

            IFilter foreColorFilter = new ForeColorFilter(0, foreColor2);
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            worksheet.Filter.SetFilter(foreColorFilter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {6, true},
                {7, false},
                {8, true}
            });
        }
    }
}
