﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Filtering
{
    [TestClass]
    public class InsertCellsWithFilteringTests
    {
        #region Filter range set without filter applied

        [TestMethod]
        public void InsertSmallerRangeAboveFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(3, 6, 3, 6), InsertShiftType.Down);

            Assert.IsFalse(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeAboveFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(3, 6, 3, 6));   
            worksheet.Cells.Remove(new CellRange(3, 6, 3, 6), RemoveShiftType.Up);

            Assert.IsFalse(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertSmallerRangeToLeftOfFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(6, 3, 6, 3), InsertShiftType.Right);

            Assert.IsFalse(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeToLeftOfFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(6, 3, 6, 3));
            worksheet.Cells.Remove(new CellRange(6, 3, 6, 3), RemoveShiftType.Left);

            Assert.IsFalse(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertSmallerRangeToRightOfFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(6, 13, 6, 13), InsertShiftType.Right);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeToRightOfFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(6, 13, 6, 13));
            worksheet.Cells.Remove(new CellRange(6, 13, 6, 13), RemoveShiftType.Left);

            Assert.IsTrue(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertSmallerRangeBelowFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(13, 6, 13, 6), InsertShiftType.Down);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeBelowFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(13, 6, 13, 6));
            worksheet.Cells.Remove(new CellRange(13, 6, 13, 6), RemoveShiftType.Up);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertSmallerRangeWithinFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(6, 6, 6, 6), InsertShiftType.Down);

            Assert.IsFalse(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(6, 6, 6, 6));
            worksheet.Cells.Remove(new CellRange(6, 6, 6, 6), RemoveShiftType.Up);

            Assert.IsFalse(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeAboveFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(3, 3, 3, 11), InsertShiftType.Down);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(1, 0), worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(1, 0), worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeAboveFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(3, 3, 3, 11));
            worksheet.Cells.Remove(new CellRange(3, 3, 3, 11), RemoveShiftType.Up);

            Assert.IsTrue(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(-1, 0), worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(-1, 0), worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeToLeftOfFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(3, 3, 11, 3), InsertShiftType.Right);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, 1), worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, 1), worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeToLeftOfFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 3, 11, 3));
            worksheet.Cells.Remove(new CellRange(3, 3, 11, 3), RemoveShiftType.Left);

            Assert.IsTrue(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, -1), worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, -1), worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeToRightOfFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(3, 13, 13, 13), InsertShiftType.Right);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeToRightOfFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 13, 13, 13));
            worksheet.Cells.Remove(new CellRange(3, 13, 13, 13), RemoveShiftType.Left);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeBelowFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(13, 3, 13, 13), InsertShiftType.Down);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeBelowFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(13, 3, 13, 13));
            worksheet.Cells.Remove(new CellRange(13, 3, 13, 13), RemoveShiftType.Up);

            Assert.IsTrue(canRemove);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeWithinFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(3, 6, 13, 6), InsertShiftType.Right);

            CellRange expectedRange = new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(0, 1));
            Assert.IsTrue(success);
            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 6, 13, 6));
            worksheet.Cells.Remove(new CellRange(3, 6, 13, 6), RemoveShiftType.Left);

            CellRange expectedRange = new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(0, -1));
            Assert.IsTrue(canRemove);
            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertLargerRangeWithinFilteredWithoutAppliedFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(6, 3, 6, 13), InsertShiftType.Down);

            CellRange expectedRange = new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(1, 0));
            Assert.IsTrue(success);
            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinFilteredWithoutAppliedFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(6, 3, 6, 13));
            worksheet.Cells.Remove(new CellRange(6, 3, 6, 13), RemoveShiftType.Up);

            CellRange expectedRange = new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(-1, 0));
            Assert.IsTrue(canRemove);
            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.AreEqual(expectedRange, worksheet.Filter.FilterRange);
        }

        [TestMethod]
        public void InsertSameHeightRangeToLeftOfFilteredWithoutAppliedFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool success = worksheet.Cells.Insert(new CellRange(5, 3, 10, 3), InsertShiftType.Right);

            Assert.IsTrue(success);
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, 1), worksheet.Filter.FilterRange);
        }

        public void DeleteEntireFilteredWithoutAppliedFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, FilterTestHelperInternal.DefaultCellRange);
            worksheet.Cells.Remove(FilterTestHelperInternal.DefaultCellRange, RemoveShiftType.Left);

            Assert.IsTrue(canRemove);
            Assert.IsNull(worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Undo();

            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);

            worksheet.Workbook.History.Redo();

            Assert.IsNull(worksheet.Filter.FilterRange);
        }

        #endregion


        #region Filter range set with filters applied

        [TestMethod]
        public void InsertSmallerRangeAboveFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(3, 6, 3, 6), InsertShiftType.Down);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteSmallerRangeAboveFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(3, 6, 3, 6));
            worksheet.Cells.Remove(new CellRange(3, 6, 3, 6), RemoveShiftType.Up);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(canRemove, worksheet);
        }

        [TestMethod]
        public void InsertSmallerRangeToLeftOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(6, 3, 6, 3), InsertShiftType.Right);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteSmallerRangeToLeftOfFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(6, 3, 6, 3));
            worksheet.Cells.Remove(new CellRange(6, 3, 6, 3), RemoveShiftType.Left);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(canRemove, worksheet);
        }

        [TestMethod]
        public void InsertSmallerRangeToRightOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(6, 13, 6, 13), InsertShiftType.Right);

            Assert.IsTrue(success);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void DeleteSmallerRangeToRightOfFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(6, 13, 6, 13));
            worksheet.Cells.Remove(new CellRange(6, 13, 6, 13), RemoveShiftType.Left);

            Assert.IsTrue(canRemove);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void InsertSmallerRangeBelowFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(13, 6, 13, 6), InsertShiftType.Down);

            Assert.IsTrue(success);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void DeleteSmallerRangeBelowFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(13, 6, 13, 6));
            worksheet.Cells.Remove(new CellRange(13, 6, 13, 6), RemoveShiftType.Up);

            Assert.IsTrue(success);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void InsertSmallerRangeWithinFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(6, 6, 6, 6), InsertShiftType.Down);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteSmallerRangeWithinFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(6, 6, 6, 6));
            worksheet.Cells.Remove(new CellRange(6, 6, 6, 6), RemoveShiftType.Up);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(canRemove, worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeAboveFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(3, 3, 3, 13), InsertShiftType.Down);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeAboveFilteredWithFilterShifUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(3, 3, 3, 13));
            worksheet.Cells.Remove(new CellRange(3, 3, 3, 13), RemoveShiftType.Up);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(canRemove, worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeToLeftOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(3, 3, 13, 3), InsertShiftType.Right);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeToLeftOfFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 3, 13, 3));
            worksheet.Cells.Remove(new CellRange(3, 3, 13, 3), RemoveShiftType.Left);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeToRightOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(3, 13, 13, 13), InsertShiftType.Right);

            Assert.IsTrue(success);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeToRightOfFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 13, 13, 13));
            worksheet.Cells.Remove(new CellRange(3, 13, 13, 13), RemoveShiftType.Left);

            Assert.IsTrue(canRemove);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeBelowFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(13, 3, 13, 13), InsertShiftType.Down);

            Assert.IsTrue(success);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeBelowFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(13, 3, 13, 13));
            worksheet.Cells.Remove(new CellRange(13, 3, 13, 13), RemoveShiftType.Up);

            Assert.IsTrue(canRemove);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeWithinFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(3, 6, 13, 6), InsertShiftType.Right);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Left, new CellRange(3, 6, 13, 6));
            worksheet.Cells.Remove(new CellRange(3, 6, 13, 6), RemoveShiftType.Left);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void InsertLargerRangeWithinFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(7, 3, 7, 13), InsertShiftType.Down);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteLargerRangeWithinFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, new CellRange(7, 3, 7, 13));
            worksheet.Cells.Remove(new CellRange(7, 3, 7, 13), RemoveShiftType.Up);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void InsertRowRangeAboveFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Rows.Insert(3);
            Assert.IsTrue(success);
            this.AssertResultInsertRowRangeAboveFilteredWithFilterShiftDown(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultInsertRowRangeAboveFilteredWithFilterShiftDown(worksheet);
        }

        private void AssertResultInsertRowRangeAboveFilteredWithFilterShiftDown(Worksheet worksheet)
        {
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(1, 0), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, false},
                {7, true},
                {8, true},
                {9, false},
                {10, false},
                {11, true},
            });
        }

        [TestMethod]
        public void DeleteRowRangeRangeAboveFilteredWithFilterShifUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            worksheet.Rows.Remove(3);

            this.AssertResultDeleteRowRangeRangeAboveFilteredWithFilterShifUp(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultDeleteRowRangeRangeAboveFilteredWithFilterShifUp(worksheet);
        }

        private void AssertResultDeleteRowRangeRangeAboveFilteredWithFilterShifUp(Worksheet worksheet)
        {
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(-1, 0), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, true},
                {6, true},
                {7, false},
                {8, false},
                {9, true},
                {10, false},
            });
        }

        [TestMethod]
        public void InsertColumnRangeToLeftOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Columns.Insert(3);

            Assert.IsTrue(success);
            this.AssertResultInsertColumnRangeToLeftOfFilteredWithFilterShiftRight(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultInsertColumnRangeToLeftOfFilteredWithFilterShiftRight(worksheet);
        }

        private void AssertResultInsertColumnRangeToLeftOfFilteredWithFilterShiftRight(Worksheet worksheet)
        {
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, 1), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, true},
                {8, false},
                {9, false},
                {10, true},
            });
        }

        [TestMethod]
        public void DeleteColumnRangeToLeftOfFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            worksheet.Columns.Remove(3);

            this.AssertResultDeleteColumnRangeToLeftOfFilteredWithFilterShiftLeft(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultDeleteColumnRangeToLeftOfFilteredWithFilterShiftLeft(worksheet);
        }

        private void AssertResultDeleteColumnRangeToLeftOfFilteredWithFilterShiftLeft(Worksheet worksheet)
        {
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange.Offset(0, -1), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, true},
                {8, false},
                {9, false},
                {10, true},
            });
        }

        [TestMethod]
        public void InsertColumnRangeWithinFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Columns.Insert(6);

            Assert.IsTrue(success);
            this.AssertResultInsertColumnRangeWithinFilteredWithFilterShiftRight(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultInsertColumnRangeWithinFilteredWithFilterShiftRight(worksheet);
        }

        private void AssertResultInsertColumnRangeWithinFilteredWithFilterShiftRight(Worksheet worksheet)
        {
            Assert.AreEqual(new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(0, 1)), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(6, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, true},
                {8, false},
                {9, false},
                {10, true},
            });
        }

        [TestMethod]
        public void DeleteColumnRangeWithinFilteredWithFilterShiftLeft()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            worksheet.Columns.Remove(6);

            this.AssertResultDeleteColumnRangeWithinFilteredWithFilterShiftLeft(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultDeleteColumnRangeWithinFilteredWithFilterShiftLeft(worksheet);
        }

        private void AssertResultDeleteColumnRangeWithinFilteredWithFilterShiftLeft(Worksheet worksheet)
        {
            Assert.AreEqual(new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(0, -1)), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(4, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, true},
                {8, false},
                {9, false},
                {10, true},
            });
        }

        [TestMethod]
        public void InsertRowRangeWithinFilteredWithFilterShiftDown()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Rows.Insert(7);

            Assert.IsTrue(success);
            this.AssertResultInsertRowRangeWithinFilteredWithFilterShiftDown(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultInsertRowRangeWithinFilteredWithFilterShiftDown(worksheet);
        }

        private void AssertResultInsertRowRangeWithinFilteredWithFilterShiftDown(Worksheet worksheet)
        {
            Assert.AreEqual(new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(1, 0)), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, false},
                {8, true},
                {9, false},
                {10, false},
                {11, true},
            });
        }

        [TestMethod]
        public void DeleteRowRangeWithinFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            worksheet.Rows.Remove(7);

            this.AssertResultDeleteRowRangeWithinFilteredWithFilterShiftUp(worksheet);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            this.AssertResultDeleteRowRangeWithinFilteredWithFilterShiftUp(worksheet);
        }

        private void AssertResultDeleteRowRangeWithinFilteredWithFilterShiftUp(Worksheet worksheet)
        {
            Assert.AreEqual(new CellRange(FilterTestHelperInternal.DefaultCellRange.FromIndex,
                FilterTestHelperInternal.DefaultCellRange.ToIndex.Offset(-1, 0)), worksheet.Filter.FilterRange);
            IFilter firstFilter = worksheet.Filter.Filters[0];
            IFilter secondFilter = worksheet.Filter.Filters[1];
            Assert.AreEqual(0, firstFilter.RelativeColumnIndex);
            Assert.AreEqual(5, secondFilter.RelativeColumnIndex);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, false},
                {8, false},
                {9, true},
                {10, false},
            });
        }

        [TestMethod]
        public void InsertSameHeightRangeToLeftOfFilteredWithFilterShiftRight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool success = worksheet.Cells.Insert(new CellRange(5, 3, 10, 3), InsertShiftType.Down);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(success, worksheet);
        }

        [TestMethod]
        public void DeleteEntireFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            bool canRemove = worksheet.Filter.CanInsertOrRemove(ShiftType.Up, FilterTestHelperInternal.DefaultCellRange);
            worksheet.Cells.Remove(FilterTestHelperInternal.DefaultCellRange, RemoveShiftType.Up);

            this.AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(canRemove, worksheet);
        }

        [TestMethod]
        public void DeleteRowsEntireFilteredWithFilterShiftUp()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Filter.FilterRange = FilterTestHelperInternal.DefaultCellRange;
            this.EnterValuesInRange(worksheet, worksheet.Filter.FilterRange);
            this.SetFilters(worksheet);

            worksheet.Rows.Remove(FilterTestHelperInternal.DefaultCellRange.FromIndex.RowIndex, FilterTestHelperInternal.DefaultCellRange.RowCount);

            Assert.IsNull(worksheet.Filter.FilterRange);
            FilterTestHelperInternal.AssertAllRowsAreUnhidden(worksheet, FilterTestHelperInternal.DefaultCellRange);

            worksheet.Workbook.History.Undo();

            this.AssertFilteringSetUpIsUnchanged(worksheet);

            worksheet.Workbook.History.Redo();

            Assert.IsNull(worksheet.Filter.FilterRange);
            FilterTestHelperInternal.AssertAllRowsAreUnhidden(worksheet, FilterTestHelperInternal.DefaultCellRange);
        }

        #endregion

        private void EnterValuesInRange(Worksheet worksheet, CellRange range)
        {
            for (int col = range.FromIndex.ColumnIndex; col <= range.ToIndex.ColumnIndex; col++)
            {
                for (int row = range.FromIndex.RowIndex; row <= range.ToIndex.RowIndex; row++)
                {
                    worksheet.Cells[row, col, row, col].SetValue(row);
                }
            }
        }

        private void SetFilters(Worksheet worksheet)
        {
            IFilter firstFilter = new ValuesCollectionFilter(0, new string[] { "7", "8", "9" });
            IFilter secondFilter = new ValuesCollectionFilter(5, new string[] { "6", "8", "9", "10" });

            worksheet.Filter.SetFilter(firstFilter);
            worksheet.Filter.SetFilter(secondFilter);
        }

        private void AssertRangeCannotBeDeletedOrInsertedAndFilteringIsUnchanged(bool canInsertRemove, Worksheet worksheet)
        {
            Assert.IsFalse(canInsertRemove);
            this.AssertFilteringSetUpIsUnchanged(worksheet);
        }

        private void AssertFilteringSetUpIsUnchanged(Worksheet worksheet)
        {
            Assert.AreEqual(FilterTestHelperInternal.DefaultCellRange, worksheet.Filter.FilterRange);
            Assert.AreEqual(2, worksheet.Filter.Filters.Count());
            IFilter firstFilter = worksheet.Filter.GetFilter(0);
            IFilter secondFilter = worksheet.Filter.GetFilter(5);
            Assert.IsNotNull(firstFilter);
            Assert.IsNotNull(secondFilter);
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {5, false},
                {6, true},
                {7, true},
                {8, false},
                {9, false},
                {10, true},
            });
        }

    }
}
