﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Filtering
{
    [TestClass]
    public class MergedCellsAndHyperlinksFilterTests
    {
        [TestMethod]
        public void MergedCellWithinOneColumnTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 0, 0, new List<object>() { 0d, 1d, 2d, 3d, 4d, 5d });
            worksheet.Cells[2, 0, 4, 0].Merge();

            CellRange filterRange = new CellRange(0, 0, 5, 0);
            IFilter filter = new ValuesCollectionFilter(0, new List<string>() { "0", "1", "5" }, true);
            worksheet.Filter.FilterRange = filterRange;
            worksheet.Filter.SetFilter(filter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {0, false},
                {1, false},
                {2, true},
                {3, false},
                {4, false},
                {5, false},
            });
        }

        [TestMethod]
        public void MergedCellWithinOneColumnTestHideBlanks()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 0, 0, new List<object>() { 0d, 1d, 2d, 3d, 4d, 5d });
            worksheet.Cells[2, 0, 4, 0].Merge();

            CellRange filterRange = new CellRange(0, 0, 5, 0);
            IFilter filter = new ValuesCollectionFilter(0, new List<string>() { "0", "1", "3", "4", "5" }, false);
            worksheet.Filter.FilterRange = filterRange;
            worksheet.Filter.SetFilter(filter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {0, false},
                {1, false},
                {2, true},
                {3, true},
                {4, true},
                {5, false},
            });
        }

        [TestMethod]
        public void FilterSplitsMergedCellWhichIsOutsideFilteredRange()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 0, 0, new List<object>() { 0d, 1d, 2d, 3d, 4d, 5d });
            worksheet.Cells[1, 1, 5, 2].Merge();

            CellRange filterRange = new CellRange(0, 0, 5, 0);
            IFilter filter = new ValuesCollectionFilter(0, new List<string>() { "0", "1", "3", "5" }, false);
            worksheet.Filter.FilterRange = filterRange;
            worksheet.Filter.SetFilter(filter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {0, false},
                {1, false},
                {2, true},
                {3, false},
                {4, true},
                {5, false},
            });
        }

        [TestMethod]
        public void FilterWithIntersectingHyperlink()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 0, 0, new List<object>() { 0d, 1d, 2d, 3d, 4d, 5d });
            worksheet.Hyperlinks.Add(new CellRange(1, 0, 4, 1), HyperlinkInfo.CreateHyperlink("test"));

            CellRange filterRange = new CellRange(0, 0, 5, 0);
            IFilter filter = new ValuesCollectionFilter(0, new List<string>() { "0", "1", "3", "5" }, false);
            worksheet.Filter.FilterRange = filterRange;
            worksheet.Filter.SetFilter(filter);

            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, new Dictionary<int, bool>()
            {
                {0, false},
                {1, false},
                {2, true},
                {3, false},
                {4, true},
                {5, false},
            });
        }
    }
}
