﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Filtering
{
    [TestClass]
    public class ValuesCollectionFilterTests
    {
        [TestMethod]
        public void TestCorrectRowsHiddenBasic()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            List<object> values = new List<object>() { 1d, 2d, 3d };
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 2, 5, values);
            worksheet.Filter.FilterRange = new CellRange(1, 5, 4, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, true);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {2, false},
               {3, false},
               {4, true}
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenStringNumberAndBlank()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            List<object> values = new List<object>() { 1.2, "1.2" };
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 3, 5, values);
            worksheet.Filter.FilterRange = new CellRange(1, 5, 4, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {1, false},
               {2, true},
               {3, false},
               {4, false}
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenDates()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            List<object> values = new List<object>() 
            { 
                new DateTime(1986, 4, 2),
                new DateTime(1986, 4, 3),
                1d,
                2d,
                new DateTime(2013, 1, 1),
                new DateTime(2013, 12, 5, 5, 5, 30),
                new DateTime(2014, 12, 5, 5, 5, 30)
            };
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 2, 5, values);
            worksheet.Filter.FilterRange = new CellRange(1, 5, 8, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {1, false},
               {2, false},
               {3, true},
               {4, false},
               {5, false},
               {6, false},
               {7, false},
               {8, true},
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenDatesWithCustomFormat()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[2, 5, 2, 5].SetValue(31504); // 02/04/1986
            worksheet.Cells[2, 5, 2, 5].SetFormat(new CellValueFormat("dd/mm/yyyy"));
            worksheet.Cells[3, 5, 3, 5].SetValue(31748); // 02/12/1986
            worksheet.Cells[3, 5, 3, 5].SetFormat(new CellValueFormat("dd/mm/yyyy"));

            worksheet.Filter.FilterRange = new CellRange(1, 5, 8, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {1, false},
               {2, false},
               {3, true},
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenTime()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[2, 5, 2, 5].SetValue(0.6);
            worksheet.Cells[2, 5, 2, 5].SetFormat(new CellValueFormat("h:mm:ss AM/PM"));
            worksheet.Cells[3, 5, 3, 5].SetValue(0.6);
            worksheet.Cells[3, 5, 3, 5].SetFormat(new CellValueFormat("hh:mm"));

            worksheet.Filter.FilterRange = new CellRange(1, 5, 8, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {2, false},
               {3, true},
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenBlank()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[2, 5, 2, 5].SetValue(1);
            worksheet.Cells[4, 5, 4, 5].SetValue(2);
            worksheet.Cells[8, 5, 8, 5].SetValue("   ");

            worksheet.Filter.FilterRange = new CellRange(1, 5, 8, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {2, false},
               {3, true},
               {4, false},
               {5, true},
               {6, true},
               {7, true},
               {8, true},
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }

        [TestMethod]
        public void TestCorrectRowsHiddenTwoColumns()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            List<object> values = new List<object>() 
            { 
                new DateTime(1986, 4, 2),
                new DateTime(1986, 4, 3),
                1d,
                2d,
                new DateTime(2013, 1, 1),
                new DateTime(2013, 12, 5, 5, 5, 30),
                new DateTime(2014, 12, 5, 5, 5, 30)
            };
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 2, 5, values);

            List<object> values2 = new List<object>() 
            { 
                1d, 2d, 3d, 4d, 5d, 6d, 7d
            };
            FilterTestHelperInternal.SetValuesToWorksheetColumn(worksheet, 2, 6, values2);

            worksheet.Filter.FilterRange = new CellRange(1, 5, 8, 6);

            IFilter filter = new ValuesCollectionFilter(0, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter);
            IFilter filter2 = new ValuesCollectionFilter(1, FilterTestHelperInternal.StringValues, FilterTestHelperInternal.DateValues, false);
            worksheet.Filter.SetFilter(filter2);

            Dictionary<int, bool> expectedRows = new Dictionary<int, bool>()
            {
               {1, false},
               {2, false},
               {3, true},
               {4, true},
               {5, true},
               {6, true},
               {7, true},
               {8, true},
            };
            FilterTestHelperInternal.AssertCorrectRowsHidden(worksheet, expectedRows);
        }
    }
}
