﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.FormatProviders
{
    [TestClass]
    public class CsvCultureDependentTests : CsvTestClassBase
    {
        #region Fields

        private readonly string[,] inputCellValues = new string[,]
            {
                {"100,20", "100.20", "100;20"},
                {"300.140;", "ab cd", "160-230-420"},
            };

        #endregion


        #region CsvListSeparator tests

        [TestMethod]
        public void CsvListSeparatorEnUsTests()
        {
            this.WriteAndTestCultureDependentListSeparator("en-US", this.inputCellValues,
@"""100,20"",100.2,100;20
300.140;,ab cd,160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorBgBgTests()
        {
            this.WriteAndTestCultureDependentListSeparator("bg-BG", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorArDzTests()
        {
            this.WriteAndTestCultureDependentListSeparator("ar-DZ", this.inputCellValues,
@"100,20;100.2;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorArQaTests()
        {
            this.WriteAndTestCultureDependentListSeparator("ar-QA", this.inputCellValues,
@"100,20;100.2;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorDaDkTests()
        {
            this.WriteAndTestCultureDependentListSeparator("da-DK", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorDeDeTests()
        {
            this.WriteAndTestCultureDependentListSeparator("de-DE", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorEsEsTests()
        {
            this.WriteAndTestCultureDependentListSeparator("es-ES", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorFrBeTests()
        {
            this.WriteAndTestCultureDependentListSeparator("fr-BE", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorFrFrTests()
        {
            this.WriteAndTestCultureDependentListSeparator("fr-FR", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorItItTests()
        {
            this.WriteAndTestCultureDependentListSeparator("it-IT", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorKoKrTests()
        {
            this.WriteAndTestCultureDependentListSeparator("ko-KR", this.inputCellValues,
@"""100,20"",100.2,100;20
300.140;,ab cd,160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorNlBeTests()
        {
            this.WriteAndTestCultureDependentListSeparator("nl-BE", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorPaInTests()
        {
            this.WriteAndTestCultureDependentListSeparator("pa-IN", this.inputCellValues,
@"""10,020"",100.2,100;20
300.140;,ab cd,160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorRuRuTests()
        {
            this.WriteAndTestCultureDependentListSeparator("ru-RU", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        [TestMethod]
        public void CsvListSeparatorSvFiTests()
        {
            this.WriteAndTestCultureDependentListSeparator("sv-FI", this.inputCellValues,
@"100,2;100.20;""100;20""
""300.140;"";ab cd;160-230-420
");
        }

        #endregion
    }
}
