﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Core;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Csv;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.FormatProviders
{
    public class CsvTestClassBase : MultiCulturalTestClassBase
    {
        #region Methods

        private static IEnumerable<string> GetInputValues(string line, char? splitter)
        {
            char charSplitter = splitter ?? FormatHelper.DefaultSpreadsheetCulture.ListSeparator[0];

            string[] values = line.ReplaceGroupDecimalSeparators().ReplaceListSeparator().Split(charSplitter);
            for (int i = 0; i < values.Length; i++)
            {
                string result = values[i].Replace('*', '\n');
                yield return result;
            }
        }

        public void ParseAndCompareFiles(string inputDataFilename, string resultDataFilename, CsvSettings settings = null,
            char? endOfLine = null, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
              {
                  using (StreamReader testReader = new StreamReader(TestHelperInternal.GetResourceStream("TestDataResources/" + inputDataFilename)))
                  using (StreamReader resultReader = new StreamReader(TestHelperInternal.GetResourceStream("TestDataResources/" + resultDataFilename)))
                  {
                      string input = testReader.ReadToEnd().ReplaceGroupDecimalSeparators().ReplaceListSeparator();

                      CsvSettings csvSettings = settings ?? new CsvSettings();
                      CsvParser parser = new CsvParser(new StringReader(input), csvSettings);
                      foreach (CsvRecord record in parser.Parse())
                      {
                          foreach (string value in record.GetValues())
                          {
                              string testValue = endOfLine == null ? value : value.Replace('\n', (char)endOfLine);
                              string expectedValue = resultReader.ReadLine().ReplaceGroupDecimalSeparators().ReplaceListSeparator();

                              Assert.AreEqual(testValue, expectedValue);
                          }
                      }
                  }
              }, cultures);
        }

        public void WriteAndCompareFiles(string inputDataFilename, string resultDataFilename, CsvSettings settings = null,
            char? splitter = null, string[] cultures = null)
        {
            RunTestInMultipleCultures(() =>
                {
                    StringBuilder builder = new StringBuilder();
                    StringWriter outputWriter = new StringWriter(builder);
                    CsvWriter csvWriter = new CsvWriter(outputWriter, settings);

                    StreamReader inputFileReader = new StreamReader(TestHelperInternal.GetResourceStream("TestDataResources/" + inputDataFilename));
                    string line = inputFileReader.ReadLine();

                    while (line != null)
                    {
                        IEnumerable<string> inputValues = GetInputValues(line, splitter);
                        csvWriter.WriteRecord(inputValues);
                        line = inputFileReader.ReadLine();
                    }

                    StringReader outputReader = new StringReader(builder.ToString());

                    using (StreamReader outputFileReader = new StreamReader(TestHelperInternal.GetResourceStream("TestDataResources/" + resultDataFilename)))
                    {
                        while (!outputFileReader.EndOfStream)
                        {
                            string outputLine = outputReader.ReadLine() ?? string.Empty;
                            string expectedline = outputFileReader.ReadLine().ReplaceGroupDecimalSeparators().ReplaceListSeparator();

                            Assert.AreEqual(expectedline, outputLine);
                        }

                        Assert.AreEqual<int>(-1, outputReader.Peek());
                    }
                }, cultures);
        }

        public void WriteAndTestCultureDependentListSeparator(string culture, string[,] inputCellValues, string expectedOutput)
        {
            RunTestInMultipleCultures(() =>
                {
                    Worksheet worksheet = TestHelper.CreateWorksheet();

                    for (int i = 0; i < inputCellValues.GetLength(0); i++)
                    {
                        for (int j = 0; j < inputCellValues.GetLength(1); j++)
                        {
                            worksheet.Cells[i, j].SetValue(inputCellValues[i, j]);
                        }
                    }

                    string exportedCsv = new CsvFormatProvider().Export(worksheet.Workbook);

                    Assert.AreEqual(expectedOutput, exportedCsv);
                }, new string[] { culture });
        }

        #endregion
    }
}
