﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Functions
{
    [TestClass]
    public class CyclicReferencePropagationTests
    {
        private CellReferenceRangeExpression A1CellRefereceExpression;

        [TestInitialize]
        public void TestInitialize()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Cells[0, 0].SetValue("=A1");

            this.A1CellRefereceExpression = new CellReferenceRangeExpression(
                worksheet.Workbook, worksheet.Name, false, CellAreaReference.CreateFromCellName("A1", new CellIndex(10, 10)));
        }

        [TestCleanup]
        public void TestCleanup()
        {
            this.A1CellRefereceExpression = null;
        }

        [TestMethod]
        public void AbsFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Abs(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void SumFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Sum(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsBlankFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsBlank(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsErrFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsErr(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsErrorFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsError(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsNumberFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsErr(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsTextFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsText(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsNAFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsNA(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsEvenFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsEven(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void TypeFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new TypeFunction(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void ErrorTypeFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new ErrorType(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void LeftFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Left(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void SearchFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Search(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void ConcatenateFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Concatenate(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void NotFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Not(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void XorFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new Xor(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void AndFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new And(), ErrorExpressions.CyclicReference, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsRefFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsRef(), BooleanExpression.True, A1CellRefereceExpression);
        }

        [TestMethod]
        public void IsFormulaFunctionCyclicReferencePropagationTest()
        {
            FunctionsTestHelper.AssertFunctionResult(new IsFormula(), BooleanExpression.True, A1CellRefereceExpression);
        }
    }
}
