﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class DateFunctionsInternalTests
    {
        [TestMethod]
        public void Date()
        {
            Date date = new Date();

            FunctionsTestHelper.AssertFunctionResult(date, 367, "F1", "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(date, 1, "A2", "B2", "B2");
            FunctionsTestHelper.AssertFunctionResult(date, 20, 0, 1, "B1");
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.ValueError, "C1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.DivisionByZero, "D1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NumberError, "E1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(date, ErrorExpressions.NotAvailableError, "G1", 1, 1);
        }

        [TestMethod]
        public void Day()
        {
            Day day = new Day();

            FunctionsTestHelper.AssertFunctionResult(day, 31, "A2");
            FunctionsTestHelper.AssertFunctionResult(day, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(day, 20, "B1");
            FunctionsTestHelper.AssertFunctionResult(day, 3, "C2");
            FunctionsTestHelper.AssertFunctionResult(day, 4, "D2");
            FunctionsTestHelper.AssertFunctionResult(day, 5, "E2");
            FunctionsTestHelper.AssertFunctionResult(day, 6, "F2");
            FunctionsTestHelper.AssertFunctionResult(day, 15, "G2");
            FunctionsTestHelper.AssertFunctionResult(day, 1, "B2");
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(day, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void Month()
        {
            Month month = new Month();

            FunctionsTestHelper.AssertFunctionResult(month, 12, "A2");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "B2");
            FunctionsTestHelper.AssertFunctionResult(month, 2, "G2");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "C2");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "D2");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "E2");
            FunctionsTestHelper.AssertFunctionResult(month, 1, "F2");
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(month, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void Year()
        {
            Year year = new Year();
            FunctionsTestHelper.AssertFunctionResult(year, 1899, "A2");

            FunctionsTestHelper.AssertFunctionResult(year, 1900, "A1");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "B1");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "C2");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "D2");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "E2");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "F2");
            FunctionsTestHelper.AssertFunctionResult(year, 1900, "B2");
            FunctionsTestHelper.AssertFunctionResult(year, 2013, "G2");


            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(year, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void DateValue()
        {
            DateValue dateValue = new DateValue();
            Date date = new Date();

            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "A1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "A2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "B2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "C2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "D2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "E2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "F2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.ValueError, "G2");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(dateValue, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void Time()
        {
            Time time = new Time();
            FunctionsTestHelper.AssertFunctionResult(time, 0, "A2", "A2", "A2");
            FunctionsTestHelper.AssertFunctionResult(time, 0.0423726851851852, "B2", "B2", "B2");
            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.ValueError, "C1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.DivisionByZero, "D1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.NumberError, "E1", 1, 1);
            FunctionsTestHelper.AssertFunctionError(time, ErrorExpressions.NotAvailableError, "G1", 1, 1);
        }

        [TestMethod]
        public void Second()
        {
            Second second = new Second();
            FunctionsTestHelper.AssertFunctionResult(second, 59, "A3");
            FunctionsTestHelper.AssertFunctionResult(second, 59, "B3");
            FunctionsTestHelper.AssertFunctionResult(second, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(second, 0, "B2");


            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(second, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void Minute()
        {
            Minute minute = new Minute();

            FunctionsTestHelper.AssertFunctionResult(minute, 45, "A3");
            FunctionsTestHelper.AssertFunctionResult(minute, 45, "B3");

            FunctionsTestHelper.AssertFunctionResult(minute, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(minute, 0, "B2");
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(minute, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void Hour()
        {
            Hour hour = new Hour();
            FunctionsTestHelper.AssertFunctionResult(hour, 13, "B3"); new Hour();
            FunctionsTestHelper.AssertFunctionResult(hour, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(hour, 0, "B2");

            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(hour, ErrorExpressions.NotAvailableError, "G1");
        }
        [TestMethod]
        public void TimeValue()
        {
            TimeValue timeValue = new TimeValue();
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "A1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "A2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "B2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "C2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "D2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "E2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "F2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.ValueError, "G2");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(timeValue, ErrorExpressions.NotAvailableError, "G1");
        }
    }
}
