﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class FinancialFunctionsTests
    {
        [TestMethod]
        public void AccrIntMArgumentConversionTest()
        {
            AccrIntM accrIntM = new AccrIntM();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 0.205555555555556, 39539, 39614, 0.1, new object[,] { { 10 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 0.205555555555556, 39539, 39614, 0.1, "A1", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 20.5555555555556, "39539", "39614", 0.1, "1000", "0");
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 0.411111111111111, 39539, 39614, 0.1, "B1", 0);
            FunctionsTestHelper.AssertFunctionResult(accrIntM, 0.411111111111111, 39539, 39614, 0.1, new object[,] { { "20" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, "string");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, "string", 0);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, "string", 0.1, 0);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, "C1");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, "C1", 0.1, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, true);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, false);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, true, 1);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, "F1");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, 0.1, "F1");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.ValueError, 39539, 39614, "F1", 1);

        }

        [TestMethod]
        public void AccrIntMErrorPropagationTest()
        {
            AccrIntM accrIntM = new AccrIntM();

            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, -1, 39614, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.DivisionByZero, "D1", 39614, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.DivisionByZero, "D1", -1, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, "E1", -1, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NumberError, 39614, 39615, 0.1, 1000, "E1");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NotAvailableError, "G1", 39614, 0.1, 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NotAvailableError, 39614, 39615, 0.1, 1000, "G1");
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 1000);
            FunctionsTestHelper.AssertFunctionError(accrIntM, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 1000);
        }

        [TestMethod]
        public void AmorDegrcArgumentConversionTest()
        {
            AmorDegrc amorDegrc = new AmorDegrc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, new object[,] { { 2400 } }, 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, "A1", 1, 0.15, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, "2400", "39679", "39813", "20", "1", (0.15).ToString(), "0");
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, new object[,] { { "20" } }, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorDegrc, 776, 2400, 39679, 39813, "B1", 1, 0.15, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, "string", 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "string", 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, "string", 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, "C1", 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, "C1", 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "C1", 0.15, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, 0.15, false);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, true, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, false, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, 0.15, "F1");
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "F1", 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.ValueError, "F1", 39679, 39813, 300, 1, 0.15, 0);
        }
        [TestMethod]
        public void AmorDegrcErrorPropagationTest()
        {
            AmorDegrc amorDegrc = new AmorDegrc();

            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.DivisionByZero, "D1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.DivisionByZero, 2400, "D1", 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, "E1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, "E1");
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NotAvailableError, "G1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NotAvailableError, 2400, 39679, 39813, 300, 1, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorDegrc, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300, 1, 0.15, 2);
        }
        [TestMethod]
        public void AmorLincArgumentConversionTest()
        {
            AmorLinc amorLinc = new AmorLinc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, new object[,] { { 2400 } }, 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, "A1", 1, 0.15, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, "2400", "39679", "39813", "20", "1", (0.15).ToDefaultSpreadsheetCultureString(), "0");
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, new object[,] { { "20" } }, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionResult(amorLinc, 360, 2400, 39679, 39813, "B1", 1, 0.15, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, "string", 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "string", 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, "string", 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, "C1", 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, "C1", 39679, 39813, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "C1", 0.15, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, 0.15, false);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, true, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, false, 300, 1, 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, 1, 0.15, "F1");
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, 2400, 39679, 39813, 300, "F1", 0.15, 0);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.ValueError, "F1", 39679, 39813, 300, 1, 0.15, 0);
        }

        [TestMethod]
        public void AmorLincErrorPropagationTest()
        {
            AmorLinc amorLinc = new AmorLinc();

            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.DivisionByZero, "D1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.DivisionByZero, 2400, "D1", 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, "E1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NumberError, 2400, 39679, 39813, 300, 1, 0.15, "E1");
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NotAvailableError, "G1", 39679, 39813, 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NotAvailableError, 2400, 39679, 39813, 300, 1, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300, 1, 0.15, 2);
            FunctionsTestHelper.AssertFunctionError(amorLinc, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300, 1, 0.15, 2);
        }

        [TestMethod]
        public void CoupDayBsArgumentConversionTest()
        {
            CoupDayBs coupDayBs = new CoupDayBs();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupDayBs, 70, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CoupDayBsErrorPropagationTest()
        {
            CoupDayBs coupDayBs = new CoupDayBs();

            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.DivisionByZero, "D1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.DivisionByZero, 40568, "D1", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, "E1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NumberError, 40568, 40862, 2, "E1");
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NotAvailableError, "G1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NotAvailableError, 40568, 40862, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(coupDayBs, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300);
        }
        [TestMethod]
        public void CoupDaysArgumentConversionTest()
        {
            CoupDays coupDays = new CoupDays();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 90, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupDays, 90, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupDays, 180, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }
        [TestMethod]
        public void CoupDaysErrorPropagationTest()
        {
            CoupDays coupDays = new CoupDays();

            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.DivisionByZero, "D1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.DivisionByZero, 40568, "D1", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, "E1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NumberError, 40568, 40862, 2, "E1");
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NotAvailableError, "G1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NotAvailableError, 40568, 40862, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(coupDays, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300);
        }

        [TestMethod]
        public void CoupDaysNcArgumentConversionTest()
        {
            CoupDaysNc coupDaysNc = new CoupDaysNc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 20, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 20, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupDaysNc, 110, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }
        [TestMethod]
        public void CoupNcdArgumentConversionTest()
        {
            CoupNcd coupNcd = new CoupNcd();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40589, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupNcd, 40678, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNcd, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CoupDaysNcErrorPropagationTest()
        {
            CoupDaysNc coupDaysNc = new CoupDaysNc();

            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.DivisionByZero, "D1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.DivisionByZero, 40568, "D1", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, "E1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NumberError, 40568, 40862, 2, "E1");
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NotAvailableError, "G1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NotAvailableError, 40568, 40862, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(coupDaysNc, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300);
        }

        [TestMethod]
        public void CoupNumArgumentConversionTest()
        {
            CoupNum coupNum = new CoupNum();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupNum, 4, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupNum, 2, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CoupNumErrorPropagationTest()
        {
            CoupNum coupNum = new CoupNum();

            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.DivisionByZero, "D1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.DivisionByZero, 40568, "D1", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, "E1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NumberError, 40568, 40862, 2, "E1");
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NotAvailableError, "G1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NotAvailableError, 40568, 40862, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(coupNum, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300);
        }

        [TestMethod]
        public void CoupPcdArgumentConversionTest()
        {
            CoupPcd coupPcd = new CoupPcd();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40568, 40862, new object[,] { { 2 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40568, 40862, "D2", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, "40568", "40862", "2", "0");
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40568, 40862, "D2", 0);
            FunctionsTestHelper.AssertFunctionResult(coupPcd, 40497, 40568, 40862, new object[,] { { "2" } }, 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CoupPcdErrorPropagationTest()
        {
            CoupPcd coupPcd = new CoupPcd();

            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.DivisionByZero, "D1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.DivisionByZero, 40568, "D1", 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, "E1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NumberError, 40568, 40862, 2, "E1");
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NotAvailableError, "G1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NotAvailableError, 40568, 40862, "G1", 2);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(coupPcd, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 300);
        }
        [TestMethod]
        public void CumIPmtArgumentConversionTest()
        {
            CumIPmt cumIPmt = new CumIPmt();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -21413.0750118792, 0.0075, 360, 125000, new object[,] { { 2 } }, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -19539.6151136335, 0.0075, 360, 125000, "D2", 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -13931.8364167034, 0.0075, 360, 125000, "A1", 24, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -11135.2321307508, (0.0075).ToDefaultSpreadsheetCultureString(), "360", "125000", "13", "24", "0");
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -21413.0750118792, 0.0075, 360, 125000, new object[,] { { "2" } }, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumIPmt, -4629.47769161841, 0.0075, 360, 125000, "B1", 24, 0);
            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CumIPmtErrorPropagationTest()
        {
            CumIPmt cumIPmt = new CumIPmt();

            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.DivisionByZero, "D1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.DivisionByZero, 0.0075, "D1", 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, "E1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, "E1");
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NotAvailableError, "G1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NotAvailableError, 0.0075, 360, 125000, 13, "G1", 0);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(cumIPmt, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 13, 24, 0);
        }
        [TestMethod]
        public void CumPrincArgumentConversionTest()
        {
            CumPrinc cumPrinc = new CumPrinc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -1719.8252252833, 0.0075, 360, 125000, new object[,] { { 2 } }, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -1581.72858116702, 0.0075, 360, 125000, "D2", 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -1154.83765101127, 0.0075, 360, 125000, "A1", 24, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -934.107123420877, (0.0075).ToDefaultSpreadsheetCultureString(), "360", "125000", "13", "24", "0");
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -1719.8252252833, 0.0075, 360, 125000, new object[,] { { "2" } }, 24, 0);
            FunctionsTestHelper.AssertFunctionResult(cumPrinc, -399.413664286471, 0.0075, 360, 125000, "B1", 24, 0);
            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, 2, true);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, false, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, true, 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, "F1", 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, "F1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.ValueError, 40568, 40862, 2, "F1");
        }

        [TestMethod]
        public void CumPrincErrorPropagationTest()
        {
            CumPrinc cumPrinc = new CumPrinc();

            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.DivisionByZero, "D1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.DivisionByZero, 0.0075, "D1", 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, "E1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NumberError, 0.0075, 360, 125000, 13, 24, "E1");
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NotAvailableError, "G1", 360, 125000, 13, 24, 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NotAvailableError, 0.0075, 360, 125000, 13, "G1", 0);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 300);
            FunctionsTestHelper.AssertFunctionError(cumPrinc, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 13, 24, 0);
        }

        [TestMethod]
        public void DbArgumentConversionTest()
        {
            Db db = new Db();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(db, 265833.333333333, 1000000, 100000, 6, 1, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(db, 265833.333333333, 1000000, 100000, 6, 1, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, "20", 1, 10);
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, new object[,] { { "20" } }, 1, 10);
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, "B1", 1, 10);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, 20, true, 10);
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, 20, new object[,] { { true } }, 10);
            FunctionsTestHelper.AssertFunctionResult(db, 90833.3333333333, 1000000, 100000, 20, "F1", 10);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");
        }
        [TestMethod]
        public void DbErrorPropagationTest()
        {
            Db db = new Db();

            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.DivisionByZero, "D1", 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.DivisionByZero, 100000, "D1", 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, "E1", 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NumberError, 1000000, 100000, 6, 1, "E1");
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NotAvailableError, "G1", 100000, 6, 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NotAvailableError, 1000000, 100000, 6, 1, "G1", 0);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.NotAvailableError, "G1", "D1", "E1", 1, 7);
            FunctionsTestHelper.AssertFunctionError(db, ErrorExpressions.DivisionByZero, "D1", "G1", "E1", 1, 7);
        }
        [TestMethod]
        public void DiscArgumentConversionTest()
        {
            Disc disc = new Disc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(disc, 2.31428571428571, 39107, 39248, new object[,] { { 10 } }, 100, 0);
            FunctionsTestHelper.AssertFunctionResult(disc, 2.31428571428571, 39107, 39248, "A1", 100, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(disc, 2.05714285714286, 39107, 39248, "20", 100, 0);
            FunctionsTestHelper.AssertFunctionResult(disc, 2.05714285714286, 39107, 39248, new object[,] { { "20" } }, 100, 0);
            FunctionsTestHelper.AssertFunctionResult(disc, 2.05714285714286, 39107, 39248, "B1", 100, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 1000000, 100000, 20, true, 10);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 1000000, 100000, 20, new object[,] { { true } }, 10);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 1000000, 100000, 20, "F1", 10);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 40568, 40862, 2, "string");
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, "string", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 40568, "string", 2, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, "C1", 40862, 2, 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 40568, 40862, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(disc, ErrorExpressions.ValueError, 40568, 40862, 2, "C1");
        }

        [TestMethod]
        public void DollarDeArgumentConversionTest()
        {
            DollarDe dollarDe = new DollarDe();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.1, 1.1, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.1, 1.1, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.5, 1.1, "20");
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.5, 1.1, new object[,] { { "20" } });
            FunctionsTestHelper.AssertFunctionResult(dollarDe, 1.5, 1.1, "B1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, 1.1, true);
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, 1.1, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, 1.1, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, 1.1, "string");
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, "string", 5);
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, "C1", 10);
            FunctionsTestHelper.AssertFunctionError(dollarDe, ErrorExpressions.ValueError, 1.1, "C1");
        }
        [TestMethod]
        public void DollarFrArgumentConversionTest()
        {
            DollarFr dollarFr = new DollarFr();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.1, 1.1, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.1, 1.1, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.02, 1.1, "20");
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.02, 1.1, new object[,] { { "20" } });
            FunctionsTestHelper.AssertFunctionResult(dollarFr, 1.02, 1.1, "B1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, 1.1, true);
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, 1.1, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, 1.1, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, 1.1, "string");
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, "string", 5);
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, "C1", 10);
            FunctionsTestHelper.AssertFunctionError(dollarFr, ErrorExpressions.ValueError, 1.1, "C1");
        }

        [TestMethod]
        public void DurationArgumentConversionTest()
        {
            Duration duration = new Duration();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(duration, 4.08665046785357, 39448, 42370, new object[,] { { 10 } }, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 4.08665046785357, 39448, 42370, "A1", 0.09, 1, 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(duration, 4.06898851547633, 39448, 42370, "20", 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 4.06898851547633, 39448, 42370, new object[,] { { "20" } }, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(duration, 4.06898851547633, 39448, 42370, "B1", 0.09, 1, 1);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, true);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "string");
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, "string", 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, "C1", 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionError(duration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "C1");
        }

        [TestMethod]
        public void EffectArgumentConversionTest()
        {
            Effect effect = new Effect();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(effect, 149.0625, new object[,] { { 10 } }, 4);
            FunctionsTestHelper.AssertFunctionResult(effect, 149.0625, "A1", 4);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(effect, 1295, "20", 4);
            FunctionsTestHelper.AssertFunctionResult(effect, 1295, new object[,] { { "20" } }, 4);
            FunctionsTestHelper.AssertFunctionResult(effect, 1295, "B1", 4);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, 10, true);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, 10, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, 10, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, 5, "string");
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, "string", 25);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, "C1", 100);
            FunctionsTestHelper.AssertFunctionError(effect, ErrorExpressions.ValueError, 8, "C1");
        }
        [TestMethod]
        public void FvArgumentConversionTest()
        {
            Fv fv = new Fv();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(fv, 2571.17534765198, 0.005, new object[,] { { 10 } }, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 2571.17534765198, 0.005, "A1", -200, -500, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(fv, 4748.27087606246, 0.005, "20", -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 4748.27087606246, 0.005, new object[,] { { "20" } }, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionResult(fv, 4748.27087606246, 0.005, "B1", -200, -500, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(fv, 734003000, true, 20, -200, -500, false);
            FunctionsTestHelper.AssertFunctionResult(fv, 734003000, new object[,] { { true } }, 20, -200, -500, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionResult(fv, 734003000, "F1", 20, -200, -500, "A2");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.ValueError, 5, "string", -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.ValueError, "string", 5, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.ValueError, "C1", 5, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(fv, ErrorExpressions.ValueError, 8, "C1", -200, -500, 0);
        }
        [TestMethod]
        public void IntRateArgumentConversionTest()
        {
            IntRate intRate = new IntRate();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(intRate, 4, 39493, 39583, new object[,] { { 10 } }, new object[,] { { 20 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(intRate, 4, 39493, 39583, "A1", 20, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(intRate, 4, 39493, 39583, 10, "20", 0);
            FunctionsTestHelper.AssertFunctionResult(intRate, 4, 39493, 39583, 10, new object[,] { { "20" } }, 0);
            FunctionsTestHelper.AssertFunctionResult(intRate, 4, 39493, 39583, 10, "B1", 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, 39493, 39583, true, 20, false);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, 39493, 39583, new object[,] { { true } }, 20, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, 39493, 39583, "F1", 20, "A2");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, 20, "string", 10, 20, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, "string", 20, 10, 20, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, "C1", 5, 20, 10, 0);
            FunctionsTestHelper.AssertFunctionError(intRate, ErrorExpressions.ValueError, 8, "C1", 20, 10, 0);
        }
        [TestMethod]
        public void IpmtArgumentConversionTest()
        {
            Ipmt ipmt = new Ipmt();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(ipmt, -36.2417030877094, 0.00833333333333333, new object[,] { { 10 } }, new object[,] { { 20 } }, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -36.2417030877094, 0.00833333333333333, "A1", 20, 8000, 500, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(ipmt, -36.2417030877094, 0.00833333333333333, 10, "20", 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -36.2417030877094, 0.00833333333333333, 10, new object[,] { { "20" } }, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -36.2417030877094, 0.00833333333333333, 10, "B1", 8000, 500, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(ipmt, -66.6666666666667, 0.00833333333333333, true, 20, 8000, 500, false);
            FunctionsTestHelper.AssertFunctionResult(ipmt, -66.6666666666667, 0.00833333333333333, new object[,] { { true } }, 20, 8000, 500, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionResult(ipmt, -66.6666666666667, 0.00833333333333333, "F1", 20, 8000, 500, "A2");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.ValueError, 20, "string", 20, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.ValueError, "string", 10, 20, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.ValueError, "C1", 10, 20, 8000, 500, 0);
            FunctionsTestHelper.AssertFunctionError(ipmt, ErrorExpressions.ValueError, 8, "C1", 20, 8000, 500, 0);
        }
        [TestMethod]
        public void IspmtArgumentConversionTest()
        {
            Ispmt ispmt = new Ispmt();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(ispmt, -60000, 0.00833333333333333, 1, new object[,] { { 10 } }, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -60000, 0.00833333333333333, 1, "A1", 8000000);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, 1, "20", 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, 1, new object[,] { { "20" } }, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, 1, "B1", 8000000);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, true, 20, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, new object[,] { { true } }, 20, 8000000);
            FunctionsTestHelper.AssertFunctionResult(ispmt, -63333.3333333333, 0.00833333333333333, "F1", 20, 8000000);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(ispmt, ErrorExpressions.ValueError, 20, "string", 20, 8000);
            FunctionsTestHelper.AssertFunctionError(ispmt, ErrorExpressions.ValueError, "string", 10, 20, 8000);
            FunctionsTestHelper.AssertFunctionError(ispmt, ErrorExpressions.ValueError, "C1", 10, 20, 8000);
            FunctionsTestHelper.AssertFunctionError(ispmt, ErrorExpressions.ValueError, 8, "C1", 20, 8000);
        }

        [TestMethod]
        public void MDurationArgumentConversionTest()
        {
            MDuration mDuration = new MDuration();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(mDuration, 3.7492206127097, 39448, 42370, new object[,] { { 10 } }, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 3.7492206127097, 39448, 42370, "A1", 0.09, 1, 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(mDuration, 3.73301698667553, 39448, 42370, "20", 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 3.73301698667553, 39448, 42370, new object[,] { { "20" } }, 0.09, 1, 1);
            FunctionsTestHelper.AssertFunctionResult(mDuration, 3.73301698667553, 39448, 42370, "B1", 0.09, 1, 1);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, true);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "string");
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, "string", 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, "C1", 42370, 0.08, 0.09, 2, 1);
            FunctionsTestHelper.AssertFunctionError(mDuration, ErrorExpressions.ValueError, 39448, 42370, 0.08, 0.09, 2, "C1");
        }
        [TestMethod]
        public void NominalArgumentConversionTest()
        {
            Nominal nominal = new Nominal();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(nominal, 3.28464114735149, new object[,] { { 10 } }, 4);
            FunctionsTestHelper.AssertFunctionResult(nominal, 3.28464114735149, "A1", 4);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(nominal, 4.56278057171229, "20", 4);
            FunctionsTestHelper.AssertFunctionResult(nominal, 4.56278057171229, new object[,] { { "20" } }, 4);
            FunctionsTestHelper.AssertFunctionResult(nominal, 4.56278057171229, "B1", 4);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, 10, true);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, 10, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, 10, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, 5, "string");
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, "string", 25);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, "C1", 100);
            FunctionsTestHelper.AssertFunctionError(nominal, ErrorExpressions.ValueError, 8, "C1");
        }

        [TestMethod]
        public void NPerArgumentConversionTest()
        {
            NPer nPer = new NPer();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(nPer, 463.815785117522, 0.01, new object[,] { { 10 } }, -1000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(nPer, 463.815785117522, 0.01, "A1", -1000, 0, 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(nPer, 68.6705692705061, 0.01, "20", -1000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(nPer, 68.67056927050616, 0.01, new object[,] { { "20" } }, -1000, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(nPer, 68.67056927050616, 0.01, "B1", -1000, 0, 1);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(nPer, 463.815785117522, 0.01, 10, -1000, false, true);
            FunctionsTestHelper.AssertFunctionResult(nPer, 463.815785117522, 0.01, 10, -1000, new object[,] { { false } }, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionResult(nPer, 463.815785117522, 0.01, 10, -1000, false, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.ValueError, 5, "string", -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.ValueError, "string", 5, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.ValueError, "C1", 5, -200, -500, 0);
            FunctionsTestHelper.AssertFunctionError(nPer, ErrorExpressions.ValueError, 8, "C1", -200, -500, 0);
        }

        [TestMethod]
        public void NpvArgumentConversionTest()
        {
            Npv npv = new Npv();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(npv, 5, 1, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(npv, 5, 1, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(npv, 10, 1, "20");
            FunctionsTestHelper.AssertFunctionResult(npv, 10, 1, new object[,] { { "20" } });
            FunctionsTestHelper.AssertFunctionResult(npv, 10, 1, "B1");

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionResult(npv, 10, true, 20);
            FunctionsTestHelper.AssertFunctionResult(npv, 10, new object[,] { { true } }, 20);
            FunctionsTestHelper.AssertFunctionResult(npv, 10, "F1", 20);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(npv, ErrorExpressions.ValueError, "string", 20);
            FunctionsTestHelper.AssertFunctionError(npv, ErrorExpressions.ValueError, 1, "string");
            FunctionsTestHelper.AssertFunctionError(npv, ErrorExpressions.ValueError, "C1", 20);
            FunctionsTestHelper.AssertFunctionError(npv, ErrorExpressions.ValueError, 1, "C1");
        }

        [TestMethod]
        public void OddFPriceArgumentConversionTest()
        {
            OddFPrice oddFPrice = new OddFPrice();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 70.815605279549, 39763, 44256, 39736, 39873, 0.0785, 0.0625, new object[,] { { 10 } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 70.815605279549, 39763, 44256, 39736, 39873, 0.0785, 0.0625, "A1", 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 75.5581101091728, 39763, 44256, 39736, 39873, 0.0785, 0.0625, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 75.5581101091728, 39763, 44256, 39736, 39873, 0.0785, 0.0625, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFPrice, 75.5581101091728, 39763, 44256, 39736, 39873, 0.0785, 0.0625, "B1", 1, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, "F1", 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "string");
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, "string", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, "C1", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "C1");
        }
     
        [TestMethod]
        public void OddFYieldArgumentConversionTest()
        {
            OddFYield oddFYield = new OddFYield();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.602582149689104, 39763, 44256, 39736, 39873, 0.0575, new object[,] { { 10 } }, 100, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.602582149689104, 39763, 44256, 39736, 39873, 0.0575, "A1", 100, 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.00917322131453278, 39763, 44256, 39736, 39873, 0.0575, 84.5, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.00917322131453278, 39763, 44256, 39736, 39873, 0.0575, 84.5, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddFYield, 0.00917322131453278, 39763, 44256, 39736, 39873, 0.0575, 84.5, "B1", 1, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, "F1", 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "string");
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, "string", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, "C1", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddFYield, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "C1");
        }

        [TestMethod]
        public void OddLPriceArgumentConversionTest()
        {
            OddLPrice oddLPrice = new OddLPrice();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 11.1558885383807, 39485, 39614, 39370, 0.0375, 0.0405, new object[,] { { 10 } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 11.1558885383807, 39485, 39614, 39370, 0.0375, 0.0405, "A1", 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 21.0139327024185, 39485, 39614, 39370, 0.0375, 0.0405, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 21.0139327024185, 39485, 39614, 39370, 0.0375, 0.0405, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLPrice, 21.0139327024185, 39485, 39614, 39370, 0.0375, 0.0405, "B1", 1, 0);

            //Direct and indirect bool
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0575, 84.5, 10, "F1", 0);

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "string");
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, "string", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, "C1", 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLPrice, ErrorExpressions.ValueError, 39763, 44256, 39736, 39873, 0.0785, 0.0625, 10, 1, "C1");
        }
        [TestMethod]
        public void OddLYieldArgumentConversionTest()
        {
            OddLYield oddLYield = new OddLYield();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(oddLYield, -5.78258262759499, 39558, 39614, 39440, 0.0375, 99.875, new object[,] { { 10 } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, -5.78258262759499, 39558, 39614, 39440, 0.0375, 99.875, "A1", 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(oddLYield, -5.13505208723675, 39558, 39614, 39440, 0.0375, 99.875, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, -5.13505208723675, 39558, 39614, 39440, 0.0375, 99.875, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(oddLYield, -5.13505208723675, 39558, 39614, 39440, 0.0375, 99.875, "B1", 1, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, 39558, 39614, 39440, 0.0375, 99.875, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, 39558, 39614, 39440, 0.0375, 99.875, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, 39558, 39614, 39440, 0.0375, 99.875, 10, "F1", 0);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, 39558, 39614, 39440, 0.0375, 99.875, 10, 1, "string");
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, "string", 39614, 39440, 0.0375, 99.875, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, "C1", 39614, 39440, 0.0375, 99.875, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(oddLYield, ErrorExpressions.ValueError, 39558, 39614, 39440, 0.0375, 99.875, 10, 1, "C1");
        }
        [TestMethod]
        public void PDurationArgumentConversionTest()
        {
            PDuration pDuration = new PDuration();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.0397474322108725, new object[,] { { 10 } }, 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.0397474322108725, "A1", 2000, 2200);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.031305461448855, "20", 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.031305461448855, new object[,] { { "20" } }, 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.031305461448855, "B1", 2000, 2200);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.137503523749935, true, 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.137503523749935, new object[,] { { true } }, 2000, 2200);
            FunctionsTestHelper.AssertFunctionResult(pDuration, 0.137503523749935, "F1", 2000, 2200);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.ValueError, 1, 2000, "string");
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.ValueError, "string", 2000, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.ValueError, "C1", 2000, 2200);
            FunctionsTestHelper.AssertFunctionError(pDuration, ErrorExpressions.ValueError, 1, 2000, "C1");
        }

        [TestMethod]
        public void PpmtArgumentConversionTest()
        {
            Ppmt ppmt = new Ppmt();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(ppmt, -27598.1914526887, 0.08, new object[,] { { 10 } }, 10, 200000, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -27598.1914526887, 0.08, "A1", 10, 200000, 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, "20", 200000, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, new object[,] { { "20" } }, 200000, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, "B1", 200000, 1, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, 20, 200000, true, false);
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, 20, 200000, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionResult(ppmt, -8736.57699265263, 0.08, 10, 20, 200000, "F1", false);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.ValueError, 0.08, 1, 2000, "string", 1);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.ValueError, 0.08, "string", 2000, 2200, 3);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.ValueError, 0.08, "C1", 2000, 2200, 3);
            FunctionsTestHelper.AssertFunctionError(ppmt, ErrorExpressions.ValueError, 0.08, 1, 2000, 1, "C1");
        }
        [TestMethod]
        public void PriceArgumentConversionTest()
        {
            Price price = new Price();
            
            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(price, 45.9659964505817, 39493, 43054, 0.0575, 0.065, new object[,] { { 10 } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 45.9659964505817, 39493, 43054, 0.0575, 0.065, "A1", 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(price, 51.3777912902154, 39493, 43054, 0.0575, 0.065, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 51.3777912902154, 39493, 43054, 0.0575, 0.065, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(price, 51.3777912902154, 39493, 43054, 0.0575, 0.065, "B1", 1, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 39493, 43054, 0.0575, 0.065, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 39493, 43054, 0.0575, 0.065, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 39493, 43054, 0.0575, 0.065, 10, "F1", 0);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 39493, 43054, 0.0575, 0.065, 10, "string", 1);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 0.08, "string", 43054, 0.0575, 0.065, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 0.08, "C1", 43054, 0.0575, 0.065, 10, 1, 0);
            FunctionsTestHelper.AssertFunctionError(price, ErrorExpressions.ValueError, 0.08, 39493, 43054, 0.0575, 0.065, 10, 1, "C1");
        }

        [TestMethod]
        public void PriceDiscArgumentConversionTest()
        {
            PriceDisc priceDisc = new PriceDisc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 9.978125, 39494, 39508, 0.0525, new object[,] { { 10 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 9.978125, 39494, 39508, 0.0525, "A1", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 19.95625, 39494, 39508, 0.0525, "20", 0);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 19.95625, 39494, 39508, 0.0525, new object[,] { { "20" } }, 0);
            FunctionsTestHelper.AssertFunctionResult(priceDisc, 19.95625, 39494, 39508, 0.0525, "B1", 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, 39493, 43054, 0.0525, 20, true);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, 39493, 43054, 0.0525, 20, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, 39493, 43054, 0.0525, 20, "F1", 0);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, 39493, 43054, 0.0525, 20, "string", 1);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, "string", 39493, 43054, 0.0525, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, "C1", 39493, 43054, 0.0525, 0);
            FunctionsTestHelper.AssertFunctionError(priceDisc, ErrorExpressions.ValueError, 39493, 43054, 0.0525, 20, "C1");
        }
        [TestMethod]
        public void PvDiscArgumentConversionTest()
        {
            Pv pv = new Pv();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, new object[,] { { 10 } }, 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, "A1", 5000, 9000, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(pv, -109000.012643536, -0.00000001, "20", 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -109000.012643536, -0.00000001, new object[,] { { "20" } }, 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionResult(pv, -109000.012643536, -0.00000001, "B1", 5000, 9000, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, 10, 5000, 9000, false);
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, 10, 5000, 9000, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionResult(pv, -59000.0038752809, -0.00000001, 10, 5000, 9000, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.ValueError, -0.00000001, 10, 5000, 9000, "string");
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.ValueError, "string", 10, 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.ValueError, "C1", 10, 5000, 9000, 0);
            FunctionsTestHelper.AssertFunctionError(pv, ErrorExpressions.ValueError, -0.00000001, 10, 5000, 9000, "C1");
        }

        [TestMethod]
        public void RateArgumentConversionTest()
        {
            Rate rate = new Rate();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(rate, 0.150984144771126, new object[,] { { 10 } }, -200, 1000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(rate, 0.150984144771126, "A1", -200, 1000, 0, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(rate, 0.0775468953001139, "20", -200, 2000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(rate, 0.0775468953001139, new object[,] { { "20" } }, -200, 2000, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(rate, 0.0775468953001139, "B1", -200, 2000, 0, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(rate, 0.00770147248820137, 48, -200, 8000, 0, false);
            FunctionsTestHelper.AssertFunctionResult(rate, 0.00770147248820137, 48, -200, 8000, 0, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionResult(rate, 0.00770147248820137, 48, -200, 8000, 0, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.ValueError, 10, -100, 800, 1, "string");
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.ValueError, "string", -100, 800, 1, 0);
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.ValueError, "C1", -100, 800, 1, 0);
            FunctionsTestHelper.AssertFunctionError(rate, ErrorExpressions.ValueError, 10, -100, 800, 1, "C1");
        }
        [TestMethod]
        public void ReceivedArgumentConversionTest()
        {
            Received received = new Received();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(received, 10.145846544071, 39493, 39583, new object[,] { { 10 } }, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionResult(received, 10.145846544071, 39493, 39583, "A1", 0.0575, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(received, 20.291693088142, 39493, 39583, "20", 0.0575, 0);
            FunctionsTestHelper.AssertFunctionResult(received, 20.291693088142, 39493, 39583, new object[,] { { "20" } }, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionResult(received, 20.291693088142, 39493, 39583, "B1", 0.0575, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, 39493, 39583, 10, 0.0575, false);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, 39493, 39583, 10, 0.0575, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, 39493, 39583, 10, 0.0575, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, 39493, 39583, 10, 0.0575, "string");
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, "string", 39583, 10, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, "C1", 39583, 10, 0.0575, 0);
            FunctionsTestHelper.AssertFunctionError(received, ErrorExpressions.ValueError, 39493, 39583, 10, 0.0575, "C1");
        }
        [TestMethod]
        public void RriArgumentConversionTest()
        {
            Rri rri = new Rri();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(rri, 0.009576582776887, new object[,] { { 10 } }, 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.009576582776887, "A1", 10000, 11000);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(rri, 0.00477688208720606, "20", 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.00477688208720606, new object[,] { { "20" } }, 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.00477688208720606, "B1", 10000, 11000);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(rri, 0.1, true, 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.1, new object[,] { { true } }, 10000, 11000);
            FunctionsTestHelper.AssertFunctionResult(rri, 0.1, "F1", 10000, 11000);

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.ValueError, 8, 1000, "string");
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.ValueError, "string", 10000, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.ValueError, "C1", 10000, 11000);
            FunctionsTestHelper.AssertFunctionError(rri, ErrorExpressions.ValueError, 8, 1000, "C1");
        }
        [TestMethod]
        public void SlnArgumentConversionTest()
        {
            Sln sln = new Sln();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(sln, 2250, 30000, 7500, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(sln, 2250, 30000, 7500, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(sln, 1125, 30000, 7500, "20");
            FunctionsTestHelper.AssertFunctionResult(sln, 1125, 30000, 7500, new object[,] { { "20" } });
            FunctionsTestHelper.AssertFunctionResult(sln, 1125, 30000, 7500, "B1");

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(sln, 22500, 30000, 7500, true);
            FunctionsTestHelper.AssertFunctionResult(sln, 22500, 30000, 7500, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionResult(sln, 22500, 30000, 7500, "F1");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(sln, ErrorExpressions.ValueError, 30000, 7500, "string");
            FunctionsTestHelper.AssertFunctionError(sln, ErrorExpressions.ValueError, "string", 30000, 7500);
            FunctionsTestHelper.AssertFunctionError(sln, ErrorExpressions.ValueError, "C1", 30000, 7500);
            FunctionsTestHelper.AssertFunctionError(sln, ErrorExpressions.ValueError, 30000, 7500, "C1");
        }
        [TestMethod]
        public void SydArgumentConversionTest()
        {
            Syd syd = new Syd();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(syd, 454.545454545455, 10000, 7500, new object[,] { { 10 } }, 1);
            FunctionsTestHelper.AssertFunctionResult(syd, 454.545454545455, 10000, 7500, "A1", 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(syd, 238.095238095238, 10000, 7500, "20", 1);
            FunctionsTestHelper.AssertFunctionResult(syd, 238.095238095238, 10000, 7500, new object[,] { { "20" } }, 1);
            FunctionsTestHelper.AssertFunctionResult(syd, 238.095238095238, 10000, 7500, "B1", 1);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(syd, 454.545454545455, 10000, 7500, 10, true);
            FunctionsTestHelper.AssertFunctionResult(syd, 454.545454545455, 10000, 7500, 10, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionResult(syd, 454.545454545455, 10000, 7500, 10, "F1");

            //Direct and indirect text
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.ValueError, 10000, 7500, 10, "string");
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.ValueError, "string", 10000, 7500, 10);
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.ValueError, "C1", 10000, 7500, 10);
            FunctionsTestHelper.AssertFunctionError(syd, ErrorExpressions.ValueError, 10000, 7500, 10, "C1");
        }

        [TestMethod]
        public void TBillEqArgumentConversionTest()
        {
            TBillEq tBillEq = new TBillEq();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(tBillEq, 0.554711246200608, new object[,] { { 39538 } }, 39600, 0.5);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(tBillEq, 0.554711246200608, 39538, "39600", 0.5);
            FunctionsTestHelper.AssertFunctionResult(tBillEq, 0.554711246200608, 39538, new object[,] { { "39600" } }, 0.5);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, 39538, 39600, false);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, 39538, 39600, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, 39538, 39600, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, 39538, 39600, "string");
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, "string", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, "C1", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillEq, ErrorExpressions.ValueError, 39538, 39600, "C1");
        }


        [TestMethod]
        public void TBillPriceArgumentConversionTest()
        {
            TBillPrice tBillPrice = new TBillPrice();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 82.7777777777778, new object[,] { { 39538 } }, 39600, 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 82.7777777777778, 39538, "39600", 1);
            FunctionsTestHelper.AssertFunctionResult(tBillPrice, 82.7777777777778, 39538, new object[,] { { "39600" } }, 1);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, 39538, 39600, true);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, 39538, 39600, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, 39538, 39600, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, 39538, 39600, "string");
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, "string", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, "C1", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillPrice, ErrorExpressions.ValueError, 39538, 39600, "C1");
        }
        [TestMethod]
        public void TBillYieldArgumentConversionTest()
        {
            TBillYield tBillYield = new TBillYield();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 52.258064516129, 39538, 39600, new object[,] { { 10 } });
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 52.258064516129, 39538, 39600, "A1");

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 23.2258064516129, 39538, 39600, "20");
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 23.2258064516129, 39538, 39600, new object[,] { { "20" } });
            FunctionsTestHelper.AssertFunctionResult(tBillYield, 23.2258064516129, 39538, 39600, "B1");

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, 39538, 39600, true);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, 39538, 39600, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, 39538, 39600, "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, 39538, 39600, "string");
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, "string", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, "C1", 39583, 1);
            FunctionsTestHelper.AssertFunctionError(tBillYield, ErrorExpressions.ValueError, 39538, 39600, "C1");
        }
        [TestMethod]
        public void VdbArgumentConversionTest()
        {
            Vdb vdb = new Vdb();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, new object[,] { { 10 } }, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, "A1", 0, 1);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(vdb, 240, 2400, 300, "20", 0, 1);
            FunctionsTestHelper.AssertFunctionResult(vdb, 240, 2400, 300, new object[,] { { "20" } }, 0, 1);
            FunctionsTestHelper.AssertFunctionResult(vdb, 240, 2400, 300, "B1", 0, 1);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, 10, false, true);
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, 10, new object[,] { { false } }, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionResult(vdb, 480, 2400, 300, 10, "A2", "B2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.ValueError, 80, 2400, 300, 10, 0, "string");
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.ValueError, "string", 2400, 300, 10, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.ValueError, "C1", 2400, 300, 10, 0, 1);
            FunctionsTestHelper.AssertFunctionError(vdb, ErrorExpressions.ValueError, 80, 2400, 300, 10, 0, "C1");
        }

        [TestMethod]
        public void YieldArgumentConversionTest()
        {
            Yield yield = new Yield();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(yield, -0.0753646430106732, 39493, 42689, 0.0575, 95.04287, new object[,] { { 10 } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, -0.0753646430106732, 39493, 42689, 0.0575, 95.04287, "A1", 1, 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(yield, -0.0477205309259267, 39493, 42689, 0.0575, 95.04287, "20", 1, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, -0.0477205309259267, 39493, 42689, 0.0575, 95.04287, new object[,] { { "20" } }, 1, 0);
            FunctionsTestHelper.AssertFunctionResult(yield, -0.0477205309259267, 39493, 42689, 0.0575, 95.04287, "B1", 1, 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, true, false);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, new object[,] { { true } }, new object[,] { { false } });
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, "B2", "A2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, 0, "string");
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, "string", 39493, 42689, 0.0575, 95.04287, 10, 0, 1);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, "C1", 39493, 42689, 0.0575, 95.04287, 10, 0, 1);
            FunctionsTestHelper.AssertFunctionError(yield, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, 0, "C1");
        }
        [TestMethod]
        public void YieldDiscArgumentConversionTest()
        {
            YieldDisc yieldDisc = new YieldDisc();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, -21.5950698932812, 39494, 39508, 99.7950, new object[,] { { 10 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, -21.5950698932812, 39494, 39508, 99.7950, "A1", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, -19.1901397865625, 39494, 39508, 99.7950, "20", 0);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, -19.1901397865625, 39494, 39508, 99.7950, new object[,] { { "20" } }, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldDisc, -19.1901397865625, 39494, 39508, 99.7950, "B1", 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, 39494, 39508, 99.7950, 10, true);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, 39494, 39508, 99.7950, 10, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, 39494, 39508, 99.7950, 10, "B2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, "string");
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, "string", 39493, 42689, 0.0575, 95.04287, 10, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, "C1", 39493, 42689, 0.0575, 95.04287, 10, 0);
            FunctionsTestHelper.AssertFunctionError(yieldDisc, ErrorExpressions.ValueError, 39493, 42689, 0.0575, 95.04287, 10, "C1");
        }

        [TestMethod]
        public void YieldMatArgumentConversionTest()
        {
            YieldMat yieldMat = new YieldMat();

            //Direct and indirect numbers
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 14.8098148317067, 39522, 39755, 39521, 0.0625, new object[,] { { 10 } }, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 14.8098148317067, 39522, 39755, 39521, 0.0625, "A1", 0);

            //Direct and indirect text numbers
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 6.62254074040261, 39522, 39755, 39521, 0.0625, "20", 0);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 6.62254074040261, 39522, 39755, 39521, 0.0625, new object[,] { { "20" } }, 0);
            FunctionsTestHelper.AssertFunctionResult(yieldMat, 6.62254074040261, 39522, 39755, 39521, 0.0625, "B1", 0);

            //Direct and indirect bool                 
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, 39522, 39755, 39394, 0.0625, 10, true);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, 39522, 39755, 39394, 0.0625, 10, new object[,] { { true } });
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, 39522, 39755, 39394, 0.0625, 10, "B2");

            //Direct and indirect text                 
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, 39522, 39755, 39394, 0.0625, 10, "string");
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, "string", 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, "C1", 39755, 39394, 0.0625, 100.0123, 0);
            FunctionsTestHelper.AssertFunctionError(yieldMat, ErrorExpressions.ValueError, 39522, 39755, 39394, 0.0625, 10, "C1");
        }
    }
}
