﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Functions
{
    [TestClass]
    public class LogicalArgumentConversion
    {
        [TestMethod]
        public void ConvertEmptyExpressionDirectIgnoreToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectUseAsIsToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectConvertToDefaultToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectTreatAsErrorToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectIgnoreToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectUseAsIsToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectConvertToDefaultToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectTreatAsErrorToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }   

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectIgnoreToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectUseAsIsToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectConvertToDefaultToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectTreatAsErrorToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectIgnoreToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectUseAsIsToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectConvertToDefaultToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectTreatAsErrorToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectIgnoreToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectUseAsIsToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectConvertToDefaultToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectTreatAsErrorToLogical()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectIgnoreToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectUseAsIsToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectConvertToDefaultToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectTreatAsErrorToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectIgnoreToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectUseAsIsToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectConvertToDefaultToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectTreatAsErrorToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }


        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectIgnoreToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectUseAsIsToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectConvertToDefaultToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectTreatAsErrorToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectIgnoreToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectUseAsIsToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, true);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectConvertToDefaultToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectTreatAsErrorToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectIgnoreToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectUseAsIsToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectConvertToDefaultToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, null, 1, false);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectTreatAsErrorToLogical()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Logical, ErrorExpressions.ValueError, 0, null);
        }
    }
}
