﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Functions.FunctionHelperClassTests
{
    [TestClass]
    public class NumberArgumentConversion
    {
        [TestMethod]
        public void ConvertEmptyExpressionDirectIgnoreToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectUseAsIsToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectConvertToDefaultToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectTreatAsErrorToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectIgnoreToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectUseAsIsToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Number, null, 1, Math.PI);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectConvertToDefaultToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectTreatAsErrorToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectIgnoreToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectUseAsIsToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Number, null, 1, 1d);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectConvertToDefaultToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectTreatAsErrorToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectIgnoreToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectUseAsIsToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Number, null, 1, 123d);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectConvertToDefaultToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectTreatAsErrorToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectIgnoreToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectUseAsIsToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectConvertToDefaultToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionDirectTreatAsErrorToNumber()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("string"), rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectIgnoreToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectUseAsIsToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectConvertToDefaultToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectTreatAsErrorToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectIgnoreToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectUseAsIsToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 123d);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectConvertToDefaultToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectTreatAsErrorToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }


        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectIgnoreToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectUseAsIsToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 1d);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectConvertToDefaultToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectTreatAsErrorToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectIgnoreToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectUseAsIsToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 123d);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectConvertToDefaultToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectTreatAsErrorToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectIgnoreToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.Ignore);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 0, null);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectUseAsIsToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectConvertToDefaultToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, null, 1, 0d);
        }

        [TestMethod]
        public void ConvertNonTextNumberExpressionIndirectTreatAsErrorToNumber()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("string");

            ArgumentConversionRules rules = new ArgumentConversionRules(nonTextNumberIndirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Number, ErrorExpressions.ValueError, 0, null);
        }
    }
}
