﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Functions.FunctionHelperClassTests
{
    [TestClass]
    public class TextArgumentConversion
    {
        [TestMethod]
        public void ConvertEmptyExpressionDirectUseAsIsToText()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Text, null, 1, string.Empty);
        }

        [TestMethod]
        public void ConvertEmptyExpressionDirectConvertToDefaultToText()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(emptyDirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(EmptyExpression.Empty, rules, ArgumentType.Text, null, 1, string.Empty);
        }

        [TestMethod]
        public void ConvertNumberExpressionDirectUseAsIsToText()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(numberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(NumberExpression.PI, rules, ArgumentType.Text, null, 1, Math.PI.ToString());
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolDirectTreatAsErrorToText()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(boolDirectArgument: ArgumentInterpretation.TreatAsError);
            FunctionsTestHelper.AssertConversionCorrectness(BooleanExpression.True, rules, ArgumentType.Text, ErrorExpressions.ValueError, 0, null);
        }

        [TestMethod]
        public void ConvertTextNumberExpressionDirectUseAsIsToText()
        {
            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberDirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(new StringExpression("123"), rules, ArgumentType.Text, null, 1, 123.ToString());
        }

        [TestMethod]
        public void ConvertEmptyExpressionIndirectConvertToDefaultToText()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(string.Empty);

            ArgumentConversionRules rules = new ArgumentConversionRules(emptyIndirectArgument: ArgumentInterpretation.ConvertToDefault);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Text, null, 1, string.Empty);
        }

        [TestMethod]
        public void ConvertNumberExpressionIndirectUseAsIsToText()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("123");

            ArgumentConversionRules rules = new ArgumentConversionRules(numberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Text, null, 1, "123");
        }

        [TestMethod]
        public void ConvertBooleanExpressionBoolIndirectUseAsIsToText()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument("TRUE");

            ArgumentConversionRules rules = new ArgumentConversionRules(boolIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Text, null, 1, BooleanExpression.True.ToString());
        }

        [TestMethod]
        public void ConvertTextNumberExpressionIndirectUseAsIsToText()
        {
            RadExpression expression = FunctionsTestHelper.GetIndirectArgument(@"=""123""");

            ArgumentConversionRules rules = new ArgumentConversionRules(textNumberIndirectArgument: ArgumentInterpretation.UseAsIs);
            FunctionsTestHelper.AssertConversionCorrectness(expression, rules, ArgumentType.Text, null, 1, "123");
        }
    }
}
