﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class InformationFunctionsTests
    {
        [TestMethod]
        public void ErrorType()
        {
            ErrorType errorType = new ErrorType();

            FunctionsTestHelper.AssertFunctionResult(errorType, 2, "D1");
            FunctionsTestHelper.AssertFunctionResult(errorType, 6, "E1");

            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, "A1");
            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, "B1");
            FunctionsTestHelper.AssertFunctionError(errorType, ErrorExpressions.NotAvailableError, "C1");
        }

        [TestMethod]
        public void IsBlank()
        {
            IsBlank isBlank = new IsBlank();
            FunctionsTestHelper.AssertFunctionResult(isBlank, true, "AB1");

            FunctionsTestHelper.AssertFunctionResult(isBlank, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isBlank, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isBlank, false, "C1");
        }

        [TestMethod]
        public void IsFormula()
        {
            IsFormula isFormula = new IsFormula();

            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "D1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "E1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "G1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "E2");
            FunctionsTestHelper.AssertFunctionResult(isFormula, true, "F2");

            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "AB1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "B2");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "C2");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "D2");
            FunctionsTestHelper.AssertFunctionResult(isFormula, false, "G2");

        }

        [TestMethod]
        public void IsNumber()
        {
            IsNumber isNumber = new IsNumber();

            FunctionsTestHelper.AssertFunctionResult(isNumber, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(isNumber, true, "C2");
            FunctionsTestHelper.AssertFunctionResult(isNumber, true, "D2");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "F2");

            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "B2");

            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "C1");

            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "D1");
            FunctionsTestHelper.AssertFunctionResult(isNumber, false, "E1");
        }

        [TestMethod]
        public void IsText()
        {
            IsText isText = new IsText();


            FunctionsTestHelper.AssertFunctionResult(isText, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(isText, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isText, false, "B2");

            FunctionsTestHelper.AssertFunctionResult(isText, false, "D1");
            FunctionsTestHelper.AssertFunctionResult(isText, false, "E1");

            FunctionsTestHelper.AssertFunctionResult(isText, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isText, false, "C2");

            FunctionsTestHelper.AssertFunctionResult(isText, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "E2");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "F2");
            FunctionsTestHelper.AssertFunctionResult(isText, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(isText, false, "D2");
        }

        [TestMethod]
        public void IsNonText()
        {
            IsNonText isNonText = new IsNonText();
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "F2");
            FunctionsTestHelper.AssertFunctionResult(isNonText, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "C2");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "D2");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "F1");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "A2");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "B2");

            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "D1");
            FunctionsTestHelper.AssertFunctionResult(isNonText, true, "E1");
        }

        [TestMethod]
        public void IsLogical()
        {
            IsLogical isLogical = new IsLogical();

            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "C2");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "D2");

            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "D1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "E1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "F2");
            FunctionsTestHelper.AssertFunctionResult(isLogical, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, true, "F1");
            FunctionsTestHelper.AssertFunctionResult(isLogical, true, "A2");
            FunctionsTestHelper.AssertFunctionResult(isLogical, true, "B2");

        }
        [TestMethod]
        public void IsRef()
        {
            IsRef isRef = new IsRef();
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "D1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "E1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "F1");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "A2");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "B2");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "C2");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "D2");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "E2");
            FunctionsTestHelper.AssertFunctionResult(isRef, true, "F2");

        }

        [TestMethod]
        public void IsError()
        {
            IsError isError = new IsError();

            FunctionsTestHelper.AssertFunctionResult(isError, true, "D1");
            FunctionsTestHelper.AssertFunctionResult(isError, true, "E1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "B2");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "C2");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "D2");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isError, false, "F2");
        }

        [TestMethod]
        public void IsErr()
        {
            IsErr isErr = new IsErr();

            FunctionsTestHelper.AssertFunctionResult(isErr, true, "D1");
            FunctionsTestHelper.AssertFunctionResult(isErr, true, "E1");

            FunctionsTestHelper.AssertFunctionResult(isErr, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "B2");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "C2");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "D2");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isErr, false, "F2");
        }

        [TestMethod]
        public void IsNA()
        {
            IsNA isNa = new IsNA();
            FunctionsTestHelper.AssertFunctionResult(isNa, true, "G1");

            FunctionsTestHelper.AssertFunctionResult(isNa, false, "D1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "E1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "A2");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "B2");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "C2");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "D2");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "E2");
            FunctionsTestHelper.AssertFunctionResult(isNa, false, "F2");
        }

        [TestMethod]
        public void IsEven()
        {
            IsEven isEven = new IsEven();
            FunctionsTestHelper.AssertFunctionResult(isEven, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(isEven, true, "D2");
            FunctionsTestHelper.AssertFunctionResult(isEven, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(isEven, true, "F2");
            FunctionsTestHelper.AssertFunctionResult(isEven, false, "E2");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, "A2");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, "B2");

            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(isEven, ErrorExpressions.NotAvailableError, "G1");
        }


        [TestMethod]
        public void IsOdd()
        {
            IsOdd isOdd = new IsOdd();
            FunctionsTestHelper.AssertFunctionResult(isOdd, true, "E2");

            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "D2");

            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "B1");
            FunctionsTestHelper.AssertFunctionResult(isOdd, false, "F2");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, "A2");

            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.NotAvailableError, "G1");
            FunctionsTestHelper.AssertFunctionError(isOdd, ErrorExpressions.ValueError, "B2");
        }
        [TestMethod]
        public void N()
        {
            N nFunction = new N();

            FunctionsTestHelper.AssertFunctionResult(nFunction, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(nFunction, 0, "B1");

            FunctionsTestHelper.AssertFunctionError(nFunction, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(nFunction, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(nFunction, ErrorExpressions.NotAvailableError, "G1");
        }

        [TestMethod]
        public void TypeFunction()
        {
            TypeFunction typeFunction = new TypeFunction();

            FunctionsTestHelper.AssertFunctionResult(typeFunction, 16, "D1");
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 16, "E1");
            FunctionsTestHelper.AssertFunctionResult(typeFunction, 16, "G1");
        }
    }
}
