﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class LogicalFunctionsTests
    {
        [TestMethod]
        public void AndFunctionArgumentConversionTest()
        {
            And and = new And();

            FunctionsTestHelper.AssertFunctionResult(and, false, EmptyExpression.Empty, EmptyExpression.Empty);
            FunctionsTestHelper.AssertFunctionResult(and, false, true, true, 0);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, true, 1);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, true, 100);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, true, -50);
            FunctionsTestHelper.AssertFunctionResult(and, false, 100, 0, -50);
            FunctionsTestHelper.AssertFunctionResult(and, true, 5, -1, -2);
            FunctionsTestHelper.AssertFunctionResult(and, true, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(and, false, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(and, true, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(and, false, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(and, true, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(and, false, false, "B1");

            FunctionsTestHelper.AssertFunctionResult(and, true, true, "C3");
            FunctionsTestHelper.AssertFunctionResult(and, false, false, "C3");

            FunctionsTestHelper.AssertFunctionResult(and, true, new object[,] { { true, true } });
            FunctionsTestHelper.AssertFunctionResult(and, true, new object[,] { { 3, 1 } });
            FunctionsTestHelper.AssertFunctionResult(and, true, new object[,] { { "3", 1 } });
        }

        [TestMethod]
        public void AndFunctionErrorTest()
        {
            And and = new And();

            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "0");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, true, "1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, true, "0");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, "string");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, 1, "string");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, false, "string");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.NumberError, 0, "E1");
            FunctionsTestHelper.AssertFunctionError(and, ErrorExpressions.ValueError, new object[,] { { "3", "1" } });
        }        
        
        [TestMethod]
        public void XorFunctionErrorTest()
        {
            Xor xor = new Xor();

            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, new object[,] { { "3", "1" } });

            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "1");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "0");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, true, "1");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, false, "0");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "string");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, "string", "text");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, true, "text");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(xor, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void XorFunctionArgumentConversionTest()
        {
            Xor xor = new Xor();

            FunctionsTestHelper.AssertFunctionResult(xor, false, EmptyExpression.Empty, EmptyExpression.Empty);
            FunctionsTestHelper.AssertFunctionResult(xor, false, 0, false);
            FunctionsTestHelper.AssertFunctionResult(xor, true, 1, false);
            FunctionsTestHelper.AssertFunctionResult(xor, true, -1, false);
            FunctionsTestHelper.AssertFunctionResult(xor, true, -1, 0);
            FunctionsTestHelper.AssertFunctionResult(xor, false, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(xor, true, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(xor, true, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(xor, false, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(xor, false, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(xor, true, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(xor, true, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(xor, false, true, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(xor, true, true, true, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(xor, false, false, "B1");

            FunctionsTestHelper.AssertFunctionResult(xor, true, true, "C3");
            FunctionsTestHelper.AssertFunctionResult(xor, false, false, "C3");


            FunctionsTestHelper.AssertFunctionResult(xor, true, new object[,] { { true, false } });
            FunctionsTestHelper.AssertFunctionResult(xor, true, new object[,] { { 0, 1 } });
            FunctionsTestHelper.AssertFunctionResult(xor, false, new object[,] { { 0, 0 } });
            FunctionsTestHelper.AssertFunctionResult(xor, true, new object[,] { { "3", 1 } });
            FunctionsTestHelper.AssertFunctionResult(xor, false, new object[,] { { "3", 0 } });
        }
 
        [TestMethod]
        public void NotFunctionArgumentConversionTest()
        {
            Not not = new Not();

            FunctionsTestHelper.AssertFunctionResult(not, true, 0);
            FunctionsTestHelper.AssertFunctionResult(not, false, 1);
            FunctionsTestHelper.AssertFunctionResult(not, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(not, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(not, true, "C3");

            FunctionsTestHelper.AssertFunctionResult(not, false, new object[,] { { true, false } });
            FunctionsTestHelper.AssertFunctionResult(not, true, new object[,] { { 0, 1 } });
            FunctionsTestHelper.AssertFunctionResult(not, true, new object[,] { { 0, 0 } });
        }

        [TestMethod]
        public void NotFunctionErrorTest()
        {
            Not not = new Not();

            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, new object[,] { { "3", 1 } });
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, new object[,] { { "3", "1" } });

            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, "1");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, "text");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(not, ErrorExpressions.NumberError, "E1");

        }

        [TestMethod]
        public void OrFunctionErrorTest()
        {
            Or or = new Or();

            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, new object[,] { { "3", "1" } });
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "1");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "0");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, true, "1");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, false, "0");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "string");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, "string", "text");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, true, "text");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.ValueError, false, "text");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(or, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void OrFunctionArgumentConversionTest()
        {
            Or or = new Or();

            FunctionsTestHelper.AssertFunctionResult(or, false, EmptyExpression.Empty, EmptyExpression.Empty);
            FunctionsTestHelper.AssertFunctionResult(or, false, 0, false);
            FunctionsTestHelper.AssertFunctionResult(or, true, 1, false);
            FunctionsTestHelper.AssertFunctionResult(or, true, -1, false);
            FunctionsTestHelper.AssertFunctionResult(or, true, -1, 0);
            FunctionsTestHelper.AssertFunctionResult(or, false, 0, 0);
            FunctionsTestHelper.AssertFunctionResult(or, true, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(or, true, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(or, false, false, "C1");
            FunctionsTestHelper.AssertFunctionResult(or, true, true, "A1");
            FunctionsTestHelper.AssertFunctionResult(or, true, false, "A1");
            FunctionsTestHelper.AssertFunctionResult(or, true, true, "B1");
            FunctionsTestHelper.AssertFunctionResult(or, false, false, "B1");

            FunctionsTestHelper.AssertFunctionResult(or, true, true, "C3");
            FunctionsTestHelper.AssertFunctionResult(or, false, false, "C3");


            FunctionsTestHelper.AssertFunctionResult(or, true, new object[,] { { true, false } });
            FunctionsTestHelper.AssertFunctionResult(or, true, new object[,] { { 0, 1 } });
            FunctionsTestHelper.AssertFunctionResult(or, false, new object[,] { { 0, 0 } });
            FunctionsTestHelper.AssertFunctionResult(or, true, new object[,] { { "3", 1 } });
            FunctionsTestHelper.AssertFunctionResult(or, false, new object[,] { { "3", 0 } });
        }
    }
}
