﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class MathAndTrigonometryFunctionsTests
    {
        [TestMethod]
        public void AbsFunctionTest()
        {
            Abs abs = new Abs();
            FunctionsTestHelper.AssertFunctionResult(abs, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(abs, 20, "B1");
            FunctionsTestHelper.AssertFunctionResult(abs, 0, "C3");

            FunctionsTestHelper.AssertFunctionResult(abs, 2, "A11");
            FunctionsTestHelper.AssertFunctionResult(abs, 2, "B11");
            FunctionsTestHelper.AssertFunctionResult(abs, 2, "A12");
            FunctionsTestHelper.AssertFunctionResult(abs, 3, "B12");
        }

        [TestMethod]
        public void AbsFunctionErrorTest()
        {
            Abs abs = new Abs();

            FunctionsTestHelper.AssertFunctionError(abs, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(abs, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(abs, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void AcosFunctionTest()
        {
            Acos acos = new Acos();

            FunctionsTestHelper.AssertFunctionResult(acos, 0, "B2");
            FunctionsTestHelper.AssertFunctionResult(acos, 1.5707963267949, "A2");

            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, "A1");
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, "B1");

            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(acos, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void AcoshFucntionTest()
        {
            Acosh acosh = new Acosh();

            FunctionsTestHelper.AssertFunctionResult(acosh, 3.6882538673613, "B1"); 
            FunctionsTestHelper.AssertFunctionResult(acosh, 0, "B2");
            FunctionsTestHelper.AssertFunctionResult(acosh, 2.99322284612638, "A1");

            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, "A2");
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(acosh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void AcotFunctionTest()
        {
            Acot acot = new Acot();

            FunctionsTestHelper.AssertFunctionResult(acot, 0.099668652491162, "A1");
            FunctionsTestHelper.AssertFunctionResult(acot, 0.0499583957219428, "B1");
            FunctionsTestHelper.AssertFunctionError(acot, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(acot, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(acot, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void AcothFunctionTest()
        {
            Acoth acoth = new Acoth();

            FunctionsTestHelper.AssertFunctionResult(acoth, 0.100335347731076, "A1");
            FunctionsTestHelper.AssertFunctionResult(acoth, 0.0500417292784913, "B1");
        }

        [TestMethod]
        public void AcothFunctionErrorTest()
        {
            Acoth acoth = new Acoth();

            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(acoth, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void AsinFunctionTest()
        {
            Asin asin = new Asin();

            FunctionsTestHelper.AssertFunctionResult(asin, 1.5707963267949, "B2");
            FunctionsTestHelper.AssertFunctionResult(asin, 0, "A2");
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, "A1");
            FunctionsTestHelper.AssertFunctionError(asin, ErrorExpressions.NumberError, "B1");
        }

        [TestMethod]
        public void AsinhFunctionTest()
        {
            Asinh asinh = new Asinh();

            FunctionsTestHelper.AssertFunctionResult(asinh, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(asinh, 3.68950386898891, "B1");
            FunctionsTestHelper.AssertFunctionResult(asinh, 2.99822295029797, "A1");
            FunctionsTestHelper.AssertFunctionResult(asinh, 0.881373587019543, "B2");

            FunctionsTestHelper.AssertFunctionError(asinh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(asinh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(asinh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Atan()
        {
            Atan atan = new Atan();
            FunctionsTestHelper.AssertFunctionResult(atan, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(atan, 0.785398163397448, "B2");
            FunctionsTestHelper.AssertFunctionResult(atan, 1.52083793107295, "B1");
            FunctionsTestHelper.AssertFunctionResult(atan, 1.47112767430373, "A1");

            FunctionsTestHelper.AssertFunctionError(atan, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(atan, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(atan, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Atan2()
        {
            Atan2 atan2 = new Atan2();

            FunctionsTestHelper.AssertFunctionResult(atan2, 1.5707963267949, "A2", "B2");
            FunctionsTestHelper.AssertFunctionResult(atan2, 0.463647609000806, "B1", "A1");
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.DivisionByZero, "A2", "A2");

            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.DivisionByZero, "D1", 2);
            FunctionsTestHelper.AssertFunctionError(atan2, ErrorExpressions.NumberError, "E1", 2);
        }

        [TestMethod]
        public void Atanh()
        {
            Atanh atanh = new Atanh();
            FunctionsTestHelper.AssertFunctionResult(atanh, 0, "A2");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, "B2");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, "A1");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, "B1");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(atanh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Ceiling()
        {
            Ceiling ceiling = new Ceiling();

            FunctionsTestHelper.AssertFunctionResult(ceiling, 3, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(ceiling, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(ceiling, 660, (654.321).ToDefaultSpreadsheetCultureString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(ceiling, 660, (654.321).ToDefaultSpreadsheetCultureString(), "B1");

            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(ceiling, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void CeilingMath()
        {
            CeilingMath ceilingMath = new CeilingMath();

            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 660, (654.321).ToDefaultSpreadsheetCultureString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 660, (654.321).ToDefaultSpreadsheetCultureString(), "B1");
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 3, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(ceilingMath, 1, "F1", "F1");

            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(ceilingMath, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void CeilingPrecise()
        {
            CeilingPrecise ceilingPrecise = new CeilingPrecise();

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 3, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 1, "F1", "F1");

            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 660, (654.321).ToDefaultSpreadsheetCultureString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(ceilingPrecise, 660, (654.321).ToDefaultSpreadsheetCultureString(), "B1");

            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(ceilingPrecise, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void Combin()
        {
            Combin combin = new Combin();
            FunctionsTestHelper.AssertFunctionResult(combin, 252, "A1", "5");
            FunctionsTestHelper.AssertFunctionResult(combin, 15504, "B1", "5");
            FunctionsTestHelper.AssertFunctionResult(combin, 20, 20, "F1");
            FunctionsTestHelper.AssertFunctionResult(combin, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(combin, ErrorExpressions.NumberError, 1, "E1");
        }


        [TestMethod]
        public void Cos()
        {
            Cos cos = new Cos();
            FunctionsTestHelper.AssertFunctionResult(cos, 0.54030230586814, "B2");
            FunctionsTestHelper.AssertFunctionResult(cos, 1, "A2");
            FunctionsTestHelper.AssertFunctionResult(cos, 0.408082061813392, "B1");
            FunctionsTestHelper.AssertFunctionResult(cos, -0.839071529076452, "A1");
            FunctionsTestHelper.AssertFunctionError(cos, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(cos, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(cos, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Cosh()
        {
            Cosh cosh = new Cosh();

            FunctionsTestHelper.AssertFunctionResult(cosh, 1.54308063481524, "B2");
            FunctionsTestHelper.AssertFunctionResult(cosh, 1, "A2");
            FunctionsTestHelper.AssertFunctionResult(cosh, 242582597.704895, "B1");
            FunctionsTestHelper.AssertFunctionResult(cosh, 11013.2329201033, "A1");

            FunctionsTestHelper.AssertFunctionError(cosh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(cosh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(cosh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Cot()
        {
            Cot cot = new Cot();
            FunctionsTestHelper.AssertFunctionResult(cot, 1.54235104535692, "A1");
            FunctionsTestHelper.AssertFunctionResult(cot, 0.446995108948917, "B1");
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(cot, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Coth()
        {
            Coth coth = new Coth();
            FunctionsTestHelper.AssertFunctionResult(coth, 1.00000000412231, "A1");
            FunctionsTestHelper.AssertFunctionResult(coth, 1, "B1");
            
            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(coth, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Csc()
        {
            Csc csc = new Csc();
            FunctionsTestHelper.AssertFunctionResult(csc, -1.83816396088967, "A1");
            FunctionsTestHelper.AssertFunctionResult(csc, 1.095355936408, "B1");

            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(csc, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Csch()
        {
            Csch csch = new Csch();
            FunctionsTestHelper.AssertFunctionResult(csch, 0.0000907998597121222, "A1");
            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(csch, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Degrees()
        {
            Degrees degrees = new Degrees();

            FunctionsTestHelper.AssertFunctionResult(degrees, 57.2957795130823, "F1");
            FunctionsTestHelper.AssertFunctionResult(degrees, 572.957795130823, "A1");
            FunctionsTestHelper.AssertFunctionError(degrees, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(degrees, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(degrees, ErrorExpressions.NumberError, "E1");
        }


        [TestMethod]
        public void Even()
        {
            Even even = new Even();

            FunctionsTestHelper.AssertFunctionResult(even, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(even, 2, "F1");
            FunctionsTestHelper.AssertFunctionResult(even, 20, "B1");
            FunctionsTestHelper.AssertFunctionError(even, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(even, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(even, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Exp()
        {
            Exp exp = new Exp();
            
            FunctionsTestHelper.AssertFunctionResult(exp, Math.E, "F1");
            FunctionsTestHelper.AssertFunctionResult(exp, 22026.4657948067, "10");
            FunctionsTestHelper.AssertFunctionResult(exp, 22026.4657948067, "A1");

            FunctionsTestHelper.AssertFunctionError(exp, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(exp, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(exp, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Fact()
        {
            Fact fact = new Fact();

            FunctionsTestHelper.AssertFunctionResult(fact, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(fact, 3628800, "A1");

            FunctionsTestHelper.AssertFunctionError(fact, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(fact, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(fact, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void FactDouble()
        {
            FactDouble factDouble = new FactDouble();

            FunctionsTestHelper.AssertFunctionResult(factDouble, 3840, "A1");
            FunctionsTestHelper.AssertFunctionResult(factDouble, 1, "F1");

            FunctionsTestHelper.AssertFunctionError(factDouble, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(factDouble, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(factDouble, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Floor()
        {
            Floor floor = new Floor();

            FunctionsTestHelper.AssertFunctionResult(floor, 2, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(floor, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(floor, 650, (654.321).ToDefaultSpreadsheetCultureString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(floor, 640, (654.321).ToDefaultSpreadsheetCultureString(), "B1");

            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(floor, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void FloorMath()
        {
            FloorMath floorMath = new FloorMath();

            FunctionsTestHelper.AssertFunctionResult(floorMath, 2, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(floorMath, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(floorMath, 650, (654.321).ToDefaultSpreadsheetCultureString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(floorMath, 640, (654.321).ToDefaultSpreadsheetCultureString(), "B1");

            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(floorMath, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void FloorPrecise()
        {
            FloorPrecise floorPrecise = new FloorPrecise();
    
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 2, 2.5, "F1");
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 650, (654.321).ToString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(floorPrecise, 640, (654.321).ToString(), "B1");

            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(floorPrecise, ErrorExpressions.NumberError, 1, "E1");
        }

        [TestMethod]
        public void Gcd()
        {
            Gcd gcd = new Gcd();

            FunctionsTestHelper.AssertFunctionResult(gcd, 1, "A1", 7);
            FunctionsTestHelper.AssertFunctionResult(gcd, 20, "B1", 40);
            FunctionsTestHelper.AssertFunctionResult(gcd, 20, "B1");

            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 7, "F1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.ValueError, 10, 5, "C1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(gcd, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Int()
        {
            Int intFunction = new Int();

            FunctionsTestHelper.AssertFunctionResult(intFunction, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(intFunction, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(intFunction, 20, "B1");

            FunctionsTestHelper.AssertFunctionError(intFunction, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(intFunction, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(intFunction, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Lcm()
        {
            Lcm lcm = new Lcm();

            FunctionsTestHelper.AssertFunctionResult(lcm, 70, "A1", 7);
            FunctionsTestHelper.AssertFunctionResult(lcm, 140, "B1", 7);
            FunctionsTestHelper.AssertFunctionResult(lcm, 20, "B1");

            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 7, "F1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.ValueError, 10, 5, "C1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(lcm, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Ln()
        {
            Ln ln = new Ln();

            FunctionsTestHelper.AssertFunctionResult(ln, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(ln, 2.30258509299405, "A1");

            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(ln, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Log()
        {
            Log log = new Log();

            FunctionsTestHelper.AssertFunctionResult(log, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(log, 1, "A1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.NumberError, "E1");

            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(log, ErrorExpressions.ValueError, "C1", "C1");
        }

        [TestMethod]
        public void Log10()
        {
            Log10 log10 = new Log10();

            FunctionsTestHelper.AssertFunctionResult(log10, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(log10, 0, "F1");
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(log10, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Mod()
        {
            Mod mod = new Mod();

            FunctionsTestHelper.AssertFunctionResult(mod, 0.34, (12.34).ToDefaultSpreadsheetCultureString(), "F1");
            FunctionsTestHelper.AssertFunctionResult(mod, 0, "A1", 1);
            FunctionsTestHelper.AssertFunctionResult(mod, 0, "A1", "F1");

            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(mod, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Multinomial()
        {
            Multinomial multinomial = new Multinomial();

            FunctionsTestHelper.AssertFunctionResult(multinomial, 11, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(multinomial, 21, 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(multinomial, 30045015, "A1", "B1");
            FunctionsTestHelper.AssertFunctionResult(multinomial, 1, "B1");

            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, "F1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, 1, "F1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(multinomial, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void MRound()
        {
            MRound mRound = new MRound();

            FunctionsTestHelper.AssertFunctionResult(mRound, 120, 123.456, "A1");
            FunctionsTestHelper.AssertFunctionResult(mRound, 120, 123.456, "B1");

            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, 1, "F1");
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(mRound, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Odd()
        {
            Odd odd = new Odd();

            FunctionsTestHelper.AssertFunctionResult(odd, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(odd, 11, "A1");
            FunctionsTestHelper.AssertFunctionResult(odd, 21, "B1");

            FunctionsTestHelper.AssertFunctionError(odd, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(odd, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(odd, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Power()
        {
            Power power = new Power();

            FunctionsTestHelper.AssertFunctionResult(power, 0, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(power, 100, "A1", 2);
            FunctionsTestHelper.AssertFunctionResult(power, 400, "B1", 2);

            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(power, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Product()
        {
            Product product = new Product();

            FunctionsTestHelper.AssertFunctionResult(product, 20, 2, "A1");
            FunctionsTestHelper.AssertFunctionResult(product, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(product, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(product, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(product, -1, -1, "F1");
            FunctionsTestHelper.AssertFunctionResult(product, 1, "F1", true);
            FunctionsTestHelper.AssertFunctionResult(product, 0, "F1", false);
            FunctionsTestHelper.AssertFunctionResult(product, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(product, 0, "F1", "F1");

            FunctionsTestHelper.AssertFunctionResult(product, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(product, 4, 4, "B1");
            FunctionsTestHelper.AssertFunctionResult(product, 10, "A1", "B1");
            FunctionsTestHelper.AssertFunctionResult(product, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(product, 10, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(product, 0, "C3");

            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(product, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Quotient()
        {
            Quotient quotient = new Quotient();

            FunctionsTestHelper.AssertFunctionResult(quotient, 3, "A1", 3);
            FunctionsTestHelper.AssertFunctionResult(quotient, 6, "B1", 3);

            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.NumberError, 0, "E1");

            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, "F1", 5);
            FunctionsTestHelper.AssertFunctionError(quotient, ErrorExpressions.ValueError, 2, "F1");
        }

        [TestMethod]
        public void Radians()
        {
            Radians radians = new Radians();

            FunctionsTestHelper.AssertFunctionResult(radians, 0.0174532925199433, "F1");
            FunctionsTestHelper.AssertFunctionResult(radians, 0.174532925199433, "A1");

            FunctionsTestHelper.AssertFunctionError(radians, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(radians, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(radians, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Round()
        {
            Round round = new Round();

            FunctionsTestHelper.AssertFunctionResult(round, 1234.6, 1234.56789, "F1");
            FunctionsTestHelper.AssertFunctionResult(round, 0, "A1", "-2");
            FunctionsTestHelper.AssertFunctionResult(round, 0, "B1", "-2");

            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(round, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void RoundDown()
        {
            RoundDown roundDown = new RoundDown();

            FunctionsTestHelper.AssertFunctionResult(roundDown, 10, "A1", 1);
            FunctionsTestHelper.AssertFunctionResult(roundDown, 20, "B1", 1);
            FunctionsTestHelper.AssertFunctionResult(roundDown, -3.1, -3.14159, "F1");

            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(roundDown, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void RoundUp()
        {
            RoundUp roundUp = new RoundUp();

            FunctionsTestHelper.AssertFunctionResult(roundUp, -3.2, -3.14159, "F1");
            FunctionsTestHelper.AssertFunctionResult(roundUp, 10, "A1", 1);
            FunctionsTestHelper.AssertFunctionResult(roundUp, 20, "B1", 1);
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(roundUp, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Sec()
        {
            Sec sec = new Sec();

            FunctionsTestHelper.AssertFunctionResult(sec, -1.1917935066879, "A1");
            FunctionsTestHelper.AssertFunctionResult(sec, 2.45048752095671, "B1");
            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sec, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Sech()
        {
            Sech sech = new Sech();

            FunctionsTestHelper.AssertFunctionResult(sech, 0.0000907998593378173, "A1");

            FunctionsTestHelper.AssertFunctionError(sech, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sech, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sech, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Sign()
        {
            Sign sign = new Sign();

            FunctionsTestHelper.AssertFunctionResult(sign, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(sign, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(sign, 1, "B1");

            FunctionsTestHelper.AssertFunctionError(sign, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sign, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sign, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Sin()
        {
            Sin sin = new Sin();

            FunctionsTestHelper.AssertFunctionResult(sin, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(sin, 0.841470984807897, "B2");
            FunctionsTestHelper.AssertFunctionResult(sin, 0.912945250727628, "B1");
            FunctionsTestHelper.AssertFunctionResult(sin, -0.54402111088937, "A1");
            FunctionsTestHelper.AssertFunctionError(sin, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sin, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sin, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Sinh()
        {
            Sinh sinh = new Sinh();
            FunctionsTestHelper.AssertFunctionResult(sinh, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(sinh, 1.1752011936438, "B2");
            FunctionsTestHelper.AssertFunctionResult(sinh, 242582597.704895, "B1");
            FunctionsTestHelper.AssertFunctionResult(sinh, 11013.2328747034, "A1");
            FunctionsTestHelper.AssertFunctionError(sinh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sinh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sinh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Sqrt()
        {
            Sqrt sqrt = new Sqrt();

            FunctionsTestHelper.AssertFunctionResult(sqrt, 1, "F1");
            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sqrt, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void SqrtPi()
        {
            SqrtPi sqrtPi = new SqrtPi();

            FunctionsTestHelper.AssertFunctionResult(sqrtPi, 5.60499121639793, "A1");
            FunctionsTestHelper.AssertFunctionResult(sqrtPi, 7.926654595212020, "B1");
            FunctionsTestHelper.AssertFunctionError(sqrtPi, ErrorExpressions.ValueError, "F1");
        }

        [TestMethod]
        public void Sum()
        {
            Sum sum = new Sum();

            FunctionsTestHelper.AssertFunctionResult(sum, 1, true, "F1");
            FunctionsTestHelper.AssertFunctionResult(sum, 0, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(sum, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(sum, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(sum, 0, "F1", "F1");

            FunctionsTestHelper.AssertFunctionResult(sum, 11, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(sum, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sum, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(sum, 2, 2, "C1");
            FunctionsTestHelper.AssertFunctionResult(sum, 10, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sum, 1, 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(sum, 10, "A1", "B1");
            FunctionsTestHelper.AssertFunctionResult(sum, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sum, 0, "C3");

            FunctionsTestHelper.AssertFunctionResult(sum, 2, "A11");
            // TODO:
            //FunctionsTestHelper.AssertFunctionResult(sum, 2, "B11");
            FunctionsTestHelper.AssertFunctionResult(sum, 2, "A12");
            FunctionsTestHelper.AssertFunctionResult(sum, 3, "B12");

            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(sum, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void SumSq()
        {
            SumSq sumSq = new SumSq();

            FunctionsTestHelper.AssertFunctionResult(sumSq, 1, true, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 25, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 25, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "F1", "F1");

            FunctionsTestHelper.AssertFunctionResult(sumSq, 101, 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 100, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "C1");

            FunctionsTestHelper.AssertFunctionResult(sumSq, 1, 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 100, "A1", "B1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 100, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 4, 2, "C1");

            FunctionsTestHelper.AssertFunctionResult(sumSq, 100, 10, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 4, 2, "F1");
            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "F1");

            FunctionsTestHelper.AssertFunctionResult(sumSq, 0, "C3");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(sumSq, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Tan()
        {
            Tan tan = new Tan();

            FunctionsTestHelper.AssertFunctionResult(tan, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(tan, 1.5574077246549, "B2");
            FunctionsTestHelper.AssertFunctionResult(tan, 2.23716094422474, "B1");
            FunctionsTestHelper.AssertFunctionResult(tan, 0.648360827459087, "A1");

            FunctionsTestHelper.AssertFunctionError(tan, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(tan, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(tan, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Tanh()
        {
            Tanh tanh = new Tanh();

            FunctionsTestHelper.AssertFunctionResult(tanh, 0, "A2");
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.761594155955765, "B2");
            FunctionsTestHelper.AssertFunctionResult(tanh, 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(tanh, 0.999999995877693, "A1");

            FunctionsTestHelper.AssertFunctionError(tanh, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(tanh, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(tanh, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void Trunc()
        {
            Trunc trunc = new Trunc();

            FunctionsTestHelper.AssertFunctionResult(trunc, 1234.5, 1234.5678, "F1");
            FunctionsTestHelper.AssertFunctionResult(trunc, 1.0000012345, 1.0000012345678, "10");
            FunctionsTestHelper.AssertFunctionResult(trunc, 1.0000012345, 1.0000012345678, "A1");

            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, "C1", 1);
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, 1, "C1");
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(trunc, ErrorExpressions.NumberError, 0, "E1");
        }



    }
}
