﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class MathUtilityTests
    {
        [TestMethod]
        public void IndexOfBiggestBitTests()
        {
            const int LongBits = 64;

            for (int i = 0; i <= 2 * LongBits; i++)
            {
                long number = 1L << i;
                long actualIndex = MathUtility.IndexOfBiggestBit(number);
                long expectedIndex = i % LongBits;

                Assert.AreEqual(expectedIndex, actualIndex);
            }

            long zeroResult = MathUtility.IndexOfBiggestBit(0);
            Assert.AreEqual(-1, zeroResult);
        }

#if WPF
        [TestMethod]
        public void RoundFunction_IsCorrect_WhenNumDigitsIsPositive()
        {
            Assert.AreEqual(MathUtility.Round(1.365, 2), 1.37);
            Assert.AreEqual(MathUtility.Round(-1.365, 2), -1.37);

            Assert.AreEqual(MathUtility.Round(2.15, 1), 2.2);
            Assert.AreEqual(MathUtility.Round(2.149, 1), 2.1);

            Assert.AreEqual(MathUtility.Round(-1.475, 2), -1.48);

            Assert.AreEqual(MathUtility.Round(1.500, 2), 1.5);
            Assert.AreEqual(MathUtility.Round(1.505, 2), 1.51);

            Assert.AreEqual(MathUtility.Round(1.015, 2), 1.02);
        }

        [TestMethod]
        public void RoundFunction_IsCorrect_WhenNumDigitsIsZero()
        {
            Assert.AreEqual(MathUtility.Round(1.365, 0), 1);
            Assert.AreEqual(MathUtility.Round(-1.365, 0), -1);

            Assert.AreEqual(MathUtility.Round(2.15, 0), 2);
            Assert.AreEqual(MathUtility.Round(2.149, 0), 2);

            Assert.AreEqual(MathUtility.Round(-1.475, 0), -1);

            Assert.AreEqual(MathUtility.Round(1.500, 0), 2);
            Assert.AreEqual(MathUtility.Round(1.505, 0), 2);
        }

        [TestMethod]
        public void RoundFunction_IsCorrect_WhenNumDigitsIsNegative()
        {
            Assert.AreEqual(MathUtility.Round(626.3, -3), 1000);
            Assert.AreEqual(MathUtility.Round(21.5, -1), 20);

            Assert.AreEqual(MathUtility.Round(1.98, -1), 0);
            Assert.AreEqual(MathUtility.Round(1.365, -2), 0);

            Assert.AreEqual(MathUtility.Round(-1.475, -2), 0);

            Assert.AreEqual(MathUtility.Round(1.500, -2), 0);
            Assert.AreEqual(MathUtility.Round(1.505, -1), 0);
        }
#endif
    }
}
