﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class StatisticalFunctionsTests
    {
        [TestMethod]
        public void Average()
        {
            Average average = new Average();

            FunctionsTestHelper.AssertFunctionResult(average, 7.5, 5, "A1");
            FunctionsTestHelper.AssertFunctionResult(average, 10, 10, "B1");
            FunctionsTestHelper.AssertFunctionResult(average, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(average, 10, "A1", "B1", "C1");      
            FunctionsTestHelper.AssertFunctionResult(average, -1, -1, "F1");   
            
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "C1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "B1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "B1", "C1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "F1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.DivisionByZero, "F1", "F1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(average, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void AverageA()
        {
            AverageA averagea = new AverageA();

            FunctionsTestHelper.AssertFunctionResult(averagea, 0.5, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 1, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 0, -1, "F1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 7.5, 5, "A1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 5, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 5, 10, "B1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 3.33333333333333, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(averagea, 0, "B1");

            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(averagea, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Max()
        {
            Max max = new Max();

            FunctionsTestHelper.AssertFunctionResult(max, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(max, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(max, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(max, 50, 50, "C1");
            FunctionsTestHelper.AssertFunctionResult(max, 1, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(max, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(max, -5, -5, "B1");
            FunctionsTestHelper.AssertFunctionResult(max, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(max, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(max, -10, -10, "F1");
            FunctionsTestHelper.AssertFunctionResult(max, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(max, 0, "C3");

            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(max, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void MaxA()
        {
            MaxA maxa = new MaxA();

            FunctionsTestHelper.AssertFunctionResult(maxa, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 50, 50, "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 1, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 0, -5, "B1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 1, -10, "F1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 10, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(maxa, 0, "C3");

            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(maxa, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Median()
        {
            Median median = new Median();

            FunctionsTestHelper.AssertFunctionResult(median, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(median, 15, "A1", 20);
            FunctionsTestHelper.AssertFunctionResult(median, 10, 5, "A1", 15);
            FunctionsTestHelper.AssertFunctionResult(median, 10, 10, "F1");
            FunctionsTestHelper.AssertFunctionResult(median, 10, 10, "C1");
            FunctionsTestHelper.AssertFunctionResult(median, 10, "5", "A1", "15");
            FunctionsTestHelper.AssertFunctionResult(median, 10, 10, "B1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "C3");

            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "C1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "B1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "B1", "B1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "F1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "F1", "F1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(median, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Min()
        {
            Min min = new Min();

            FunctionsTestHelper.AssertFunctionResult(min, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(min, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(min, 10, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(min, 50, 50, "C1");
            FunctionsTestHelper.AssertFunctionResult(min, 1, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(min, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(min, -5, -5, "B1");
            FunctionsTestHelper.AssertFunctionResult(min, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(min, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(min, -10, -10, "F1");
            FunctionsTestHelper.AssertFunctionResult(min, 5, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(min, 0, "C3");

            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(min, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void MinA()
        {
            MinA mina = new MinA();

            FunctionsTestHelper.AssertFunctionResult(mina, 0, 0, "F1");
            FunctionsTestHelper.AssertFunctionResult(mina, 10, "A1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, 50, "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, true, "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "B1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, 30, "B1");
            FunctionsTestHelper.AssertFunctionResult(mina, -5, -5, "B1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 1, "F1");
            FunctionsTestHelper.AssertFunctionResult(mina, -10, -10, "F1");
            FunctionsTestHelper.AssertFunctionResult(mina, 1, 5, "F1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "A1", "B1", "C1");
            FunctionsTestHelper.AssertFunctionResult(mina, 0, "C3");     
            
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(mina, ErrorExpressions.NumberError, 0, "E1");
        }

        [TestMethod]
        public void Mode()
        {
            Mode mode = new Mode();
            FunctionsTestHelper.AssertFunctionResult(mode, 10, "A1", "A1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(mode, 2, 2, "A1", "A1", 2);
            FunctionsTestHelper.AssertFunctionResult(mode, 10, 1, "A1", "A1", 2, 3);

            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "C1", 0);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, 0, "C1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "C1", "C1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.DivisionByZero, 1, "D1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NumberError, 0, "E1");

            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NotAvailableError, "A1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.NotAvailableError, 5, "A1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "B1");
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "B1", 1);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "B1", 2, 2);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "F1", 2, 2);
            FunctionsTestHelper.AssertFunctionError(mode, ErrorExpressions.ValueError, "F1");
        }

        [TestMethod]
        public void Stdev()
        {
            Stdev stdev = new Stdev();

            FunctionsTestHelper.AssertFunctionResult(stdev, 7.07106781186548, "A1", 20);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0, "F1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0, "B1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdev, 0, "C1", 2, 2);

            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "B1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "B1", "B1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "F1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "F1", 2);
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "C1");
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "C1", 2);
            FunctionsTestHelper.AssertFunctionError(stdev, ErrorExpressions.DivisionByZero, "C3");
        }

        [TestMethod]
        public void StdevP()
        {
            StdevP stdevp = new StdevP();

            FunctionsTestHelper.AssertFunctionResult(stdevp, 5, "A1", 20);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0, "F1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0, "B1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevp, 0, "C1", 2, 2);
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "C1");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "C1", 2);
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "C3");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "B1");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "B1", "B1");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "F1");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "F1", 2);
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(stdevp, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void StdevA()
        {
            StdevA stdeva = new StdevA();

            FunctionsTestHelper.AssertFunctionResult(stdeva, 7.07106781186548, "A1", 20);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, "F1", 0);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0, "B1", "B1");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.577350269189626, "B1", "B1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, "B1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.707106781186548, "C1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0, "C1", "C1");
            FunctionsTestHelper.AssertFunctionResult(stdeva, 0.577350269189626, "F1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 1.15470053837925, "B1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdeva, 1.15470053837925, "C1", 2, 2);

            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, "C3");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, "C1");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, 1);
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, "B1");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, "F1");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(stdeva, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void StdevPA()
        {
            StdevPA stdevpa = new StdevPA();

            FunctionsTestHelper.AssertFunctionResult(stdevpa, 5, "A1", 20);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, false, "F1");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, "F1", 0);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0, "B1", "B1");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.471404520791032, "B1", "B1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, "B1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.5, "C1", 1);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0, "C1", "C1");
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.471404520791032, "F1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.942809041582063, "B1", 2, 2);
            FunctionsTestHelper.AssertFunctionResult(stdevpa, 0.942809041582063, "C1", 2, 2);           
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, "C3");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, "C1");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, 1);
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, "B1");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, "F1");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(stdevpa, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void StdevInputDifferencesTest()
        {
            Stdev stdev = new Stdev();
            StdevP stdevP = new StdevP();
            StdevA stdevA = new StdevA();
            StdevPA stdevPA = new StdevPA();

            FunctionsTestHelper.AssertFunctionResult(stdev, 0.707106781186548, "A2", "B2", "C2", "D2", "E2", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevP, 0.5, "A2", "B2", "C2", "D2", "E2", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevA, 1.75119007154183, "A2", "B2", "C2", "D2", "E2", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevPA, 1.59861050777091, "A2", "B2", "C2", "D2", "E2", "F2");

            FunctionsTestHelper.AssertFunctionResult(stdev, 2.16024689946929, false, "B2", 3, "D2", "5", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevP, 1.87082869338697, false, "B2", 3, "D2", "5", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevA, 2.13697605664328, false, "B2", 3, "D2", "5", "F2");
            FunctionsTestHelper.AssertFunctionResult(stdevPA, 1.95078331845327, false, "B2", 3, "D2", "5", "F2");

            FunctionsTestHelper.AssertFunctionResult(stdev, 2.08166599946613, "A2", true, "C2", 4, "E2", "6");
            FunctionsTestHelper.AssertFunctionResult(stdevP, 1.80277563773199, "A2", true, "C2", 4, "E2", "6");
            FunctionsTestHelper.AssertFunctionResult(stdevA, 2.42212028327799, "A2", true, "C2", 4, "E2", "6");
            FunctionsTestHelper.AssertFunctionResult(stdevPA, 2.21108319357027, "A2", true, "C2", 4, "E2", "6");
        }
    }
}
