﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.ImportExport
{
    [TestClass]
    public class CultureDependentCellValuesTests : MultiCulturalImportExportTestClassBase
    {
        private void AssertNumberCellValueImportString(ICellValue value, string expectedValue)
        {
            NumberCellValue numberCellValue = value as NumberCellValue;
            Assert.IsNotNull(numberCellValue);
            Assert.AreEqual(expectedValue, numberCellValue.GetResultValueAsString(CellValueFormat.GeneralFormat));
        }

        private void AssertFormulaCellValueImportString(ICellValue value, string expectedValue)
        {
            FormulaCellValue formulaCellValue = value as FormulaCellValue;
            Assert.IsNotNull(formulaCellValue);
            Assert.AreEqual(expectedValue, formulaCellValue.RawValue);
        }

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Worksheet worksheet = workbook.ActiveWorksheet;

            worksheet.Cells[0, 0].SetValue("1{ds}2".ReplaceGroupDecimalSeparators());
            worksheet.Cells[1, 0].SetValue("=3{ds}4".ReplaceGroupDecimalSeparators());
            worksheet.Cells[2, 0].SetValue("=SUM(1{ds}2{ls} 3{ds}4)".ReplaceGroupDecimalSeparators());
            worksheet.Cells[3, 0].SetValue("=SUM((A1:B1{ls} A1:B2))".ReplaceGroupDecimalSeparators());
            worksheet.Cells[4, 0].SetValue("={3{als}4{ars}1{als}2}".ReplaceGroupDecimalSeparators());

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Cells cells = workbook.ActiveWorksheet.Cells;

            this.AssertNumberCellValueImportString(cells[0, 0].GetValue().Value, "1{ds}2".ReplaceGroupDecimalSeparators());

            this.AssertFormulaCellValueImportString(cells[1, 0].GetValue().Value, "=3{ds}4".ReplaceGroupDecimalSeparators());
            this.AssertFormulaCellValueImportString(cells[2, 0].GetValue().Value, "=SUM(1{ds}2{ls} 3{ds}4)".ReplaceGroupDecimalSeparators());
            this.AssertFormulaCellValueImportString(cells[3, 0].GetValue().Value, "=SUM((A1:B1{ls} A1:B2))".ReplaceGroupDecimalSeparators());
            this.AssertFormulaCellValueImportString(cells[4, 0].GetValue().Value, "={3{als}4{ars}1{als}2}".ReplaceGroupDecimalSeparators());
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ImportCultureDependentCellValuesTest()
        {
            ImportXlsxFileAndAssert("ImportCultureDependentCellValues.xlsx");
        }

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportCultureDependentCellValuesTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
