﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.ImportExport
{
    // Normally, this test will be in the public Test project. However, because currently imported number
    // formats are not converted to the current culture, this test runs only in en-US culture
    [TestClass]
    public class NumberFormatTests : MultiCulturalImportExportTestClassBase
    {
        private static readonly string[] formatStrings = new string[] {
            "",
            "0{ds}00", 
            "$#{gs}##0{ds}00", 
            @"_($* #{gs}##0{ds}00_);_($* (#{gs}##0{ds}00);_($* "" - ""??_);_(@_)",
            "m/d/yyyy",
            "[$-F800]dddd, mmmm dd, yyyy",
            "[$-F400]h:mm:ss AM/PM",
            "0{ds}00%",
            "# ?/?",
            "0{ds}00E+00",
            "@",
            "#0{ds}00#"
        };

        public override Workbook ArrangeWorkbook()
        {
            Workbook workbook = TestHelper.CreateNonEmptyWorkbook();
            Cells cells = workbook.Worksheets["Sheet1"].Cells;
            
            for (int i = 0; i < formatStrings.Length; i++)
            {
                cells[0, i].SetFormat(new CellValueFormat(formatStrings[i].ReplaceGroupDecimalSeparators()));
            }

            return workbook;
        }

        public override void AssertWorkbookImportedCorrectly(Workbook workbook)
        {
            Cells cells = workbook.Worksheets["Sheet1"].Cells;

            for (int i = 0; i < formatStrings.Length; i++)
            {
                CellValueFormat format = cells[0, i].GetFormat().Value;
                CellValueFormat expectedFormat = new CellValueFormat(formatStrings[i]);
                Assert.AreEqual(expectedFormat, format);
            }
        }

        // TODO:
        //[TestMethod]
        //public void ImportNumberFormatsTest()
        //{
        //    ImportXlsxFileAndAssert("ImportNumberFormatTestDocument.xlsx");
        //}

        [TestMethod]
        [TestCategory("ImportExport")]
        public void ExportAndImportNumberFormatsTest()
        {
            ArrangeWorkbookExportImportAssert();
        }
    }
}
