﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.InsertRemoveCells
{
    public class PropertyChangedEventAsserter
    {
        private readonly Cells cells;
        private readonly Dictionary<IPropertyDefinition, CellRange> propertyToRange;

        public PropertyChangedEventAsserter(Cells cells)
        {
            Guard.ThrowExceptionIfNull(cells, "cells");
            this.cells = cells;
            this.propertyToRange = new Dictionary<IPropertyDefinition, CellRange>();
            this.cells.CellPropertyChanged += Cells_CellPropertyChanged;
        }

        private void Cells_CellPropertyChanged(object sender, CellPropertyChangedEventArgs e)
        {
            if (this.propertyToRange.ContainsKey(e.Property))
            {
                CellRange oldRange = this.propertyToRange[e.Property];
                this.propertyToRange[e.Property] = CellRange.Max(oldRange, e.CellRange);
            }
            else
            {
                this.propertyToRange.Add(e.Property, e.CellRange);
            }
        }

        public void AssertCellPropertyChangedNotFired()
        {
            Assert.AreEqual(0, this.propertyToRange.Count);
        }

        public void AssertCellPropertyChanged(IPropertyDefinition definition, CellRange range)
        {
            Assert.AreEqual(1, this.propertyToRange.Count);
            Assert.IsTrue(this.propertyToRange.ContainsKey(definition));
            Assert.AreEqual(range, this.propertyToRange[definition]);
        }
    }
}
