﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class CellBordersLayerTests : CellBordersLayerTestsBase
    {
        private enum CellBorderType
        {
            Horizontal,
            Vertical,
            DiagonalUp,
            DiagonalDown
        }
        
        private readonly List<Func<ViewportPaneType, CellRange, IEnumerable<LineRenderable>>> getExpectedRenderables;
        private readonly List<Action<Worksheet>> addBordersToWorksheet;

        public CellBordersLayerTests()
        {
            this.getExpectedRenderables = new List<Func<ViewportPaneType, CellRange, IEnumerable<LineRenderable>>>();
            this.addBordersToWorksheet = new List<Action<Worksheet>>();

            this.RegisterTestBorders(new CellRange(2, 3, 6, 7), CellBorderStyle.Dotted, Colors.Orange, CellBorderType.Horizontal);
            this.RegisterTestBorders(new CellRange(2, 3, 6, 7), CellBorderStyle.DashDot, Colors.Red, CellBorderType.Vertical);
            this.RegisterTestBorders(new CellRange(25, 19, 30, 30), CellBorderStyle.Thin, Colors.Green, CellBorderType.DiagonalDown);
            this.RegisterTestBorders(new CellRange(25, 19, 30, 30), CellBorderStyle.Hair, Colors.Blue, CellBorderType.DiagonalUp);
        }

        [TestMethod]
        public void SimpleScenarioBordersLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable, 
                new KeyValuePair<int, CellRange>(0, new CellRange(2, 3, 6, 7)),
                new KeyValuePair<int, CellRange>(1, new CellRange(2, 3, 6, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(25, 19, 27, 20)),
                new KeyValuePair<int, CellRange>(3, new CellRange(25, 19, 27, 20))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioBordersLayerTest()
        {
            this.Initialize(new Point(300, 90), 1);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(4, 4, 6, 7)),
                new KeyValuePair<int, CellRange>(1, new CellRange(4, 4, 6, 7)),
                new KeyValuePair<int, CellRange>(2, null),
                new KeyValuePair<int, CellRange>(3, null)
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioBordersLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(2, 3, 6, 6)),
                new KeyValuePair<int, CellRange>(1, new CellRange(2, 3, 6, 6)),
                new KeyValuePair<int, CellRange>(2, null),
                new KeyValuePair<int, CellRange>(3, null)
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioBordersLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(2, 3, 6, 7)),
                new KeyValuePair<int, CellRange>(1, new CellRange(2, 3, 6, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(25, 19, 30, 23)),
                new KeyValuePair<int, CellRange>(3, new CellRange(25, 19, 30, 23))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioBordersLayerTest()
        {
            this.Initialize(new Point(600, 250), 2);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, null),
                new KeyValuePair<int, CellRange>(1, null),
                new KeyValuePair<int, CellRange>(2, new CellRange(25, 19, 26, 19)),
                new KeyValuePair<int, CellRange>(3, new CellRange(25, 19, 26, 19))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioBordersLayerTest()
        {
            this.Initialize(new Point(400, 90), 0.5);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(4, 6, 6, 7)),
                new KeyValuePair<int, CellRange>(1, new CellRange(4, 6, 6, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(25, 19, 30, 29)),
                new KeyValuePair<int, CellRange>(3, new CellRange(25, 19, 30, 29))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioBordersLayerTest()
        {
            this.Initialize(new Point(300, 200), 1.3);

            IEnumerable<LineRenderable> expectedScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, null),
                new KeyValuePair<int, CellRange>(1, null),
                new KeyValuePair<int, CellRange>(2, new CellRange(25, 19, 30, 20)),
                new KeyValuePair<int, CellRange>(3, new CellRange(25, 19, 30, 20))
                );

            IEnumerable<LineRenderable> expectedHorizontalScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.HorizontalScrollable);

            IEnumerable<LineRenderable> expectedVerticalScrollableBorders = this.GetExpectedBorderRenderables(ViewportPaneType.VerticalScrollable);

            IEnumerable<LineRenderable> expectedFixedBorders = this.GetExpectedBorderRenderables(ViewportPaneType.Fixed,
                new KeyValuePair<int, CellRange>(0, new CellRange(2, 3, 4, 5)),
                new KeyValuePair<int, CellRange>(1, new CellRange(2, 3, 4, 5)),
                new KeyValuePair<int, CellRange>(2, null),
                new KeyValuePair<int, CellRange>(3, null)
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellBordersLayer bordersLayer = (CellBordersLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(bordersLayer, expectedScrollableBorders, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(bordersLayer, expectedHorizontalScrollableBorders, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(bordersLayer, expectedVerticalScrollableBorders, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(bordersLayer, expectedFixedBorders, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }   

        private IEnumerable<LineRenderable> GetExpectedBorderRenderables(ViewportPaneType paneType, params KeyValuePair<int, CellRange>[] registeredIndexesToExpectedVisibleRanges)
        {
            List<LineRenderable> expectedRenderables = new List<LineRenderable>();

            foreach (KeyValuePair<int, CellRange> indexToRange in registeredIndexesToExpectedVisibleRanges)
            {
                int registeredIndex = indexToRange.Key;
                CellRange expectedVisibleRange = indexToRange.Value;

                if (expectedVisibleRange != null)
                {
                    expectedRenderables.AddRange(this.getExpectedRenderables[registeredIndex](paneType, expectedVisibleRange));
                }
            }

            return expectedRenderables;
        }

        private Worksheet CreateWorksheetScenario()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();

            foreach (Action<Worksheet> addBordersAction in this.addBordersToWorksheet)
            {
                addBordersAction(worksheet);
            }

            return worksheet;
        }

        private void RegisterTestBorders(CellRange cellRange, CellBorderStyle style, Color color, CellBorderType borderType)
        {
            CellBorder border = new CellBorder(style, new ThemableColor(color));

            this.addBordersToWorksheet.Add((worksheet) =>
                {
                    CellBorder vertical = borderType == CellBorderType.Vertical ? border : null;
                    CellBorder horizontal = borderType == CellBorderType.Horizontal ? border : null;
                    CellBorder diagonalUp = borderType == CellBorderType.DiagonalUp ? border : null;
                    CellBorder diagonalDown = borderType == CellBorderType.DiagonalDown ? border : null;

                    worksheet.Cells[cellRange].SetBorders(new CellBorders(vertical, horizontal, vertical, horizontal, horizontal, vertical, diagonalUp, diagonalDown));
                });

            BorderProperties properties = new BorderProperties()
            {
                CellBorderStyle = style,
                Stroke = color,
                StrokeThickness = border.Thickness
            };

            this.getExpectedRenderables.Add((paneType, expectedRange) =>
                {
                    Point topLeftBordersOffset = new Point();
                    Rect boundingBox = base.GetLinesBoundingBox(paneType, expectedRange, topLeftBordersOffset);

                    switch (borderType)
                    {
                        case CellBorderType.Horizontal:
                            return base.GetExpectedHorizontalLines(expectedRange, boundingBox, topLeftBordersOffset, properties);
                        case CellBorderType.Vertical:
                            return base.GetExpectedVerticalLines(expectedRange, boundingBox, topLeftBordersOffset, properties);
                        case CellBorderType.DiagonalDown:
                            return base.GetExpectedDiagonalDownLines(expectedRange.RowCount, expectedRange.ColumnCount, boundingBox, properties);
                        case CellBorderType.DiagonalUp:
                            return base.GetExpectedDiagonalUpLines(expectedRange.RowCount, expectedRange.ColumnCount, boundingBox, properties);
                        default:
                            throw new NotSupportedException(String.Format("Border type {0} is not supported!", borderType));
                    }
                
                });
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new CellBordersLayer(new TestRenderer<LineRenderable>(RenderableComparer.AreEqual), true));
        }
    }
}
