﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    public abstract class CellBordersLayerTestsBase : LayerTesterBase
    {
        internal class BorderProperties
        {
            public double StrokeThickness { get; set; }
            public Color Stroke { get; set; }
            public CellBorderStyle CellBorderStyle { get; set; }
        }
                        
        protected Rect GetLinesBoundingBox(ViewportPaneType paneType, CellRange range, Point topLeftLinesOffset)
        {
            double left = range.FromIndex.ColumnIndex * ColumnWidth - topLeftLinesOffset.X;
            double top = range.FromIndex.RowIndex * RowHeight - topLeftLinesOffset.Y;
            double width = range.ColumnCount * ColumnWidth + topLeftLinesOffset.X;
            double height = range.RowCount * RowHeight + topLeftLinesOffset.Y;
            Rect scaledBoundingBox = this.TranslateAndScale(new Rect(left, top, width, height), paneType);

            return scaledBoundingBox;
        }

        internal IEnumerable<LineRenderable> GetExpectedDiagonalDownLines(int rowsCount, int columnsCount, Rect scaledBoundingBox, BorderProperties properties)
        {
            double scaledColumnWidth = this.ScaleFactor * ColumnWidth;
            double scaledRowHeight = this.ScaleFactor * RowHeight;
            Rect startRect = new Rect(scaledBoundingBox.Left, scaledBoundingBox.Top, scaledColumnWidth, scaledRowHeight);

            for (int row = 0; row < rowsCount; row++)
            {
                for (int column = 0; column < columnsCount; column++)
                {
                    Point offset = new Point(column * scaledColumnWidth, row * scaledRowHeight);

                    yield return new LineRenderable()
                    {
                        X1 = startRect.Left + offset.X,
                        Y1 = startRect.Top + offset.Y,
                        X2 = startRect.Right + offset.X,
                        Y2 = startRect.Bottom + offset.Y,
                        CellBorderStyle = properties.CellBorderStyle,
                        StrokeThickness = properties.StrokeThickness,
                        Stroke = properties.Stroke
                    };
                }
            }
        }

        internal IEnumerable<LineRenderable> GetExpectedDiagonalUpLines(int rowsCount, int columnsCount, Rect scaledBoundingBox, BorderProperties properties)
        {
            double scaledColumnWidth = this.ScaleFactor * ColumnWidth;
            double scaledRowHeight = this.ScaleFactor * RowHeight;
            Rect startRect = new Rect(scaledBoundingBox.Left, scaledBoundingBox.Top, scaledColumnWidth, scaledRowHeight);

            for (int row = 0; row < rowsCount; row++)
            {
                for (int column = 0; column < columnsCount; column++)
                {
                    Point offset = new Point(column * scaledColumnWidth, row * scaledRowHeight);

                    yield return new LineRenderable()
                    {
                        X1 = startRect.Left + offset.X,
                        Y1 = startRect.Bottom + offset.Y,
                        X2 = startRect.Right + offset.X,
                        Y2 = startRect.Top + offset.Y,
                        CellBorderStyle = properties.CellBorderStyle,
                        StrokeThickness = properties.StrokeThickness,
                        Stroke = properties.Stroke
                    };
                }
            }
        }

        internal IEnumerable<LineRenderable> GetExpectedHorizontalLines(CellRange range, Rect scaledBoundingBox, Point topLeftGridlineOffset, BorderProperties properties)
        {
            double halfStrokeThickness = this.ScaleFactor * properties.StrokeThickness / 2;
            int firstColumnIndex = topLeftGridlineOffset.X > 0 ? (range.FromIndex.ColumnIndex - 1) : (range.FromIndex.ColumnIndex);
            bool shouldAddLeftHalfStrokeThickness = firstColumnIndex > 0;

            LineRenderable firstHorizontal = new LineRenderable()
            {
                X1 = scaledBoundingBox.Left - (shouldAddLeftHalfStrokeThickness ? halfStrokeThickness : 0),
                X2 = scaledBoundingBox.Right + halfStrokeThickness,
                Y1 = scaledBoundingBox.Top + topLeftGridlineOffset.Y,
                Y2 = scaledBoundingBox.Top + topLeftGridlineOffset.Y,
                CellBorderStyle = properties.CellBorderStyle,
                StrokeThickness = properties.StrokeThickness,
                Stroke = properties.Stroke
            };

            yield return firstHorizontal;

            for (int row = 1; row <= range.RowCount; row++)
            {
                double rowOffset = row * SpreadsheetDefaultValues.DefaultRowHeight * this.ScaleFactor;

                LineRenderable horizonalLine = new LineRenderable()
                {
                    X1 = firstHorizontal.X1,
                    X2 = firstHorizontal.X2,
                    Y1 = firstHorizontal.Y1 + rowOffset,
                    Y2 = firstHorizontal.Y2 + rowOffset,
                    CellBorderStyle = firstHorizontal.CellBorderStyle,
                    Stroke = firstHorizontal.Stroke,
                    StrokeThickness = firstHorizontal.StrokeThickness
                };

                yield return horizonalLine;
            }
        }

        internal IEnumerable<LineRenderable> GetExpectedVerticalLines(CellRange range, Rect scaledBoundingBox, Point topLeftGridlineOffset, BorderProperties properties)
        {
            double halfStrokeThickness = this.ScaleFactor * properties.StrokeThickness / 2;
            int firstRowIndex = topLeftGridlineOffset.Y > 0 ? (range.FromIndex.RowIndex - 1) : (range.FromIndex.RowIndex);
            bool shouldAddTopHalfStrokeThickness = firstRowIndex > 0;

            LineRenderable firstVertical = new LineRenderable()
            {
                X1 = scaledBoundingBox.Left + topLeftGridlineOffset.X,
                X2 = scaledBoundingBox.Left + topLeftGridlineOffset.X,
                Y1 = scaledBoundingBox.Top - (shouldAddTopHalfStrokeThickness ? halfStrokeThickness : 0),
                Y2 = scaledBoundingBox.Bottom + halfStrokeThickness,
                CellBorderStyle = properties.CellBorderStyle,
                StrokeThickness = properties.StrokeThickness,
                Stroke = properties.Stroke
            };

            yield return firstVertical;

            for (int column = 1; column <= range.ColumnCount; column++)
            {
                double columnOffset = column * SpreadsheetDefaultValues.DefaultColumnWidth * this.ScaleFactor;

                LineRenderable verticalLine = new LineRenderable()
                {
                    X1 = firstVertical.X1 + columnOffset,
                    X2 = firstVertical.X2 + columnOffset,
                    Y1 = firstVertical.Y1,
                    Y2 = firstVertical.Y2,
                    CellBorderStyle = firstVertical.CellBorderStyle,
                    Stroke = firstVertical.Stroke,
                    StrokeThickness = firstVertical.StrokeThickness
                };

                yield return verticalLine;
            }
        }

        internal void AssertLayerUpdate(CellBordersLayerBase layer, IEnumerable<LineRenderable> expectedRenderables, ViewportPaneType paneType)
        {
            TestRenderer<LineRenderable> renderer = (TestRenderer<LineRenderable>)layer.LineRenderer;
            renderer.AssertAreRendered(expectedRenderables, paneType);
        }
    }
}
