﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class CellFillLayerTests : LayerTesterBase
    {
        private readonly List<Action<Worksheet>> addPatternFillAction;
        private readonly List<Action<Worksheet>> addGradientFillAction;
        private readonly List<Func<ViewportPaneType, IEnumerable<CellRange>, PatternFillRenderable>> getExpectedPatternFillRenderable;
        private readonly List<Func<ViewportPaneType, CellRange, IEnumerable<GradientFillRenderable>>> getExpectedGradientFillRenderables;

        public CellFillLayerTests()
        {
            this.addPatternFillAction = new List<Action<Worksheet>>();
            this.addGradientFillAction = new List<Action<Worksheet>>();
            this.getExpectedPatternFillRenderable = new List<Func<ViewportPaneType, IEnumerable<CellRange>, PatternFillRenderable>>();
            this.getExpectedGradientFillRenderables = new List<Func<ViewportPaneType, CellRange, IEnumerable<GradientFillRenderable>>>();

            this.RegisterTestPatternFill(new CellRange[]
            {
                new CellRange(2, 3, 6, 7),
                new CellRange(25, 19, 30, 30),
            }, PatternType.DiagonalCrosshatch, Colors.Blue, Colors.Orange);

            this.RegisterTestPatternFill(new CellRange[]
            {
                new CellRange(24, 1, 29, 5),
                new CellRange(30, 6, 35, 9),
                new CellRange(3, 18, 7, 22),
            }, PatternType.Solid, Colors.Cyan, Colors.Black);

            this.RegisterTestGradientFill(new CellRange(8, 9, 12, 15), GradientType.FromCenter, Colors.Red, Colors.Green);
            this.RegisterTestGradientFill(new CellRange(16, 4, 21, 7), GradientType.DiagonalDownRepeated, Colors.Black, Colors.White);
            this.RegisterTestGradientFill(new CellRange(17, 16, 23, 22), GradientType.Horizontal, Colors.Purple, Colors.Magenta);
        }

        [TestMethod]
        public void SimpleScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(2, 3, 6, 7),
                    new CellRange(25, 19, 27, 20),
                }),
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(24, 1, 27, 5),
                    new CellRange(3, 18, 7, 20),
                })
                );

            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0,  new CellRange(8, 9, 12, 15)),
                new KeyValuePair<int, CellRange>(1,  new CellRange(16, 4, 21, 7)),
                new KeyValuePair<int, CellRange>(2,  new CellRange(17, 16, 23, 20))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(300, 90), 1);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(4, 4, 6, 7)
                }),
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(24, 4, 29, 5),
                    new CellRange(30, 6, 34, 9),
                })
                );

            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(8, 9, 12, 15)),
                new KeyValuePair<int, CellRange>(1, new CellRange(16, 4, 21, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(17, 16, 23, 16))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(2, 3, 6, 6)
                })
                );

            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(2, 3, 6, 7),
                    new CellRange(25, 19, 30, 23),
                }),
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(24, 1, 29, 5),
                    new CellRange(30, 6, 35, 9),
                    new CellRange(3, 18, 7, 22),
                })
                );

            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(8, 9, 12, 15)),
                new KeyValuePair<int, CellRange>(1, new CellRange(16, 4, 21, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(17, 16, 23, 22))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(600, 250), 2);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(25, 19, 26, 19)
                })
                );

            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(12, 9, 12, 15)),
                new KeyValuePair<int, CellRange>(2, new CellRange(17, 16, 23, 19))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(400, 90), 0.5);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(4, 6, 6, 7),
                    new CellRange(25, 19, 30, 29),
                }),
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(30, 6, 35, 9),
                    new CellRange(4, 18, 7, 22),
                })
                );
            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(0, new CellRange(8, 9, 12, 15)),
                new KeyValuePair<int, CellRange>(1, new CellRange(16, 6, 21, 7)),
                new KeyValuePair<int, CellRange>(2, new CellRange(17, 16, 23, 22))
                );

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioCellFillLayerTest()
        {
            this.Initialize(new Point(300, 200), 1.3);

            IEnumerable<PatternFillRenderable> expectedScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(25, 19, 30, 20)
                })
                );
            IEnumerable<GradientFillRenderable> expectedScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Scrollable,
                new KeyValuePair<int, CellRange>(2, new CellRange(17, 16, 23, 20))
                );

            IEnumerable<PatternFillRenderable> expectedHorizontalScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.HorizontalScrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(3, 18, 4, 20)
                })
                );
            IEnumerable<GradientFillRenderable> expectedHorizontalScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.HorizontalScrollable);

            IEnumerable<PatternFillRenderable> expectedVerticalScrollablePatterns = this.GetExpectedPatternRenderables(ViewportPaneType.VerticalScrollable,
                new KeyValuePair<int, IEnumerable<CellRange>>(1, new CellRange[] 
                {
                    new CellRange(24, 1, 29, 5)
                })
                );
            IEnumerable<GradientFillRenderable> expectedVerticalScrollableGradients = this.GetExpectedGradientRenderables(ViewportPaneType.VerticalScrollable,
                new KeyValuePair<int, CellRange>(1, new CellRange(16, 4, 21, 5))
                );

            IEnumerable<PatternFillRenderable> expectedFixedPattens = this.GetExpectedPatternRenderables(ViewportPaneType.Fixed,
                new KeyValuePair<int, IEnumerable<CellRange>>(0, new CellRange[] 
                {
                    new CellRange(2, 3, 4, 5)
                })
                );
            IEnumerable<GradientFillRenderable> expectedFixedGradients = this.GetExpectedGradientRenderables(ViewportPaneType.Fixed);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellFillLayer cellFillLayer = (CellFillLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellFillLayer, expectedScrollablePatterns, expectedScrollableGradients, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(cellFillLayer, expectedHorizontalScrollablePatterns, expectedHorizontalScrollableGradients, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(cellFillLayer, expectedVerticalScrollablePatterns, expectedVerticalScrollableGradients, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(cellFillLayer, expectedFixedPattens, expectedFixedGradients, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }   

        private Worksheet CreateWorksheetScenario()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();

            foreach (Action<Worksheet> addPatternAction in this.addPatternFillAction)
            {
                addPatternAction(worksheet);
            }

            foreach (Action<Worksheet> addGradientAction in this.addGradientFillAction)
            {
                addGradientAction(worksheet);
            }

            return worksheet;
        }

        private void RegisterTestPatternFill(IEnumerable<CellRange> cellRanges, PatternType patternType, Color patternColor, Color background)
        {
            PatternFill patternFill = new PatternFill(patternType, patternColor, background);

            this.addPatternFillAction.Add((worksheet) =>
                {
                    worksheet.Cells[cellRanges].SetFill(patternFill);
                });

            this.getExpectedPatternFillRenderable.Add((paneType, expectedRanges) =>
                {
                    PatternFillRenderable renderable =  new PatternFillRenderable();
                    renderable.PatternFill = patternFill;

                    foreach (var expectedRange in expectedRanges)
                    {
                        renderable.AddVisibleBoxes(this.GetVisibleCellBoxRectangles(expectedRange, paneType));
                    }

                    return renderable;
                });
        }

        private void RegisterTestGradientFill(CellRange cellRange, GradientType gradientType, Color color1, Color color2)
        {
            GradientFill gradientFill = new GradientFill(gradientType, color1, color2);

            this.addGradientFillAction.Add((worksheet) =>
                {
                    worksheet.Cells[cellRange].SetFill(gradientFill);
                });

            this.getExpectedGradientFillRenderables.Add((paneType, expectedRange) =>
                {
                    List<GradientFillRenderable> expectedRenderables = new List<GradientFillRenderable>();
                    IEnumerable<Rect> visibleCellBoxRectangles = this.GetVisibleCellBoxRectangles(expectedRange, paneType);

                    foreach (Rect visibleRect in visibleCellBoxRectangles)
                    {
                        GradientFillRenderable renderable = new GradientFillRenderable()
                        {
                            GradientFill = gradientFill,
                            BoundingRectangle = visibleRect
                        };

                        expectedRenderables.Add(renderable);
                    }

                    return expectedRenderables;
                });
        }

        private IEnumerable<Rect> GetVisibleCellBoxRectangles(CellRange range, ViewportPaneType paneType)
        {
            CellRange firstRange = new CellRange(range.FromIndex, range.FromIndex);
            Rect firstRect = this.GetVisibleRect(firstRange, paneType);

            for (int row = 0; row < range.RowCount; row++)
            {
                for (int column = 0; column < range.ColumnCount; column++)
                {
                    yield return firstRect.Translate(column * ColumnWidth * this.ScaleFactor, row * RowHeight * this.ScaleFactor);
                }
            }
        }

        private Rect GetVisibleRect(CellRange range, ViewportPaneType paneType)
        {
            Rect layoutRect = new Rect(range.FromIndex.ColumnIndex * ColumnWidth, range.FromIndex.RowIndex * RowHeight, range.ColumnCount * ColumnWidth, range.RowCount * RowHeight);
            Rect translatedAndScaledRect = this.TranslateAndScale(layoutRect, paneType);

            return translatedAndScaledRect;
        }

        private IEnumerable<PatternFillRenderable> GetExpectedPatternRenderables(ViewportPaneType paneType, params KeyValuePair<int, IEnumerable<CellRange>>[] registeredIndexesToVisibleRanges)
        {
            foreach (KeyValuePair<int, IEnumerable<CellRange>> indexToRanges in registeredIndexesToVisibleRanges)
            {
                int registeredIndex = indexToRanges.Key;
                IEnumerable<CellRange> expectedVisibleRanges = indexToRanges.Value;

                yield return this.getExpectedPatternFillRenderable[registeredIndex](paneType, expectedVisibleRanges);
            }
        }

        private IEnumerable<GradientFillRenderable> GetExpectedGradientRenderables(ViewportPaneType paneType, params KeyValuePair<int, CellRange>[] registeredIndexesToVisibleRanges)
        {
            List<GradientFillRenderable> expectedGradientFills = new List<GradientFillRenderable>();

            foreach (KeyValuePair<int, CellRange> indexToRange in registeredIndexesToVisibleRanges)
            {
                int registeredIndex = indexToRange.Key;
                CellRange expectedVisibleRange = indexToRange.Value;

                expectedGradientFills.AddRange(this.getExpectedGradientFillRenderables[registeredIndex](paneType, expectedVisibleRange));
            }

            return expectedGradientFills;
        }

        private void AssertLayerUpdate(CellFillLayer layer, IEnumerable<PatternFillRenderable> expectedPatterns, IEnumerable<GradientFillRenderable> expectedGradients, ViewportPaneType paneType)
        {
            TestRenderer<PatternFillRenderable> patternsRenderer = (TestRenderer<PatternFillRenderable>)layer.PatternFillRenderer;
            patternsRenderer.AssertAreRendered(expectedPatterns, paneType);  

            TestRenderer<GradientFillRenderable> gradientsRenderer = (TestRenderer<GradientFillRenderable>)layer.GradientFillRenderer;
            gradientsRenderer.AssertAreRendered(expectedGradients, paneType);          
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new CellFillLayer(new TestRenderer<PatternFillRenderable>(RenderableComparer.AreEqual), new TestRenderer<GradientFillRenderable>(RenderableComparer.AreEqual)));
        }
    }
}
