﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.CellValueFormats;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class CellValuesLayerTests : LayerTesterBase
    {
        private static readonly string generalFormat;
        private static readonly string accountingFormat;
        private static readonly Color transparentColor;
        private readonly List<Action<Worksheet>> addCellValueAction;
        private readonly List<Func<ViewportPaneType, TextBlockRenderable>> getExpectedTextBlockRenderable;

        static CellValuesLayerTests()
        {
            generalFormat = "@";
            accountingFormat = CellValueFormatTests_Accounting.AccountingFormatEnUS3.ReplaceGroupDecimalSeparators();
            transparentColor = Colors.Transparent;
        }

        public CellValuesLayerTests()
        {            
            this.addCellValueAction = new List<Action<Worksheet>>();
            this.getExpectedTextBlockRenderable = new List<Func<ViewportPaneType, TextBlockRenderable>>();

            this.RegisterTestCellValue(new CellIndex(0, 0), "Arial", new CellValueFormat(generalFormat),
                CreateFontProperties(Colors.Orange, "Arial", 10, true, false, false), false, TextAlignment.Center);

            this.RegisterTestCellValue(new CellIndex(2, 3), "3", new CellValueFormat(generalFormat),
                CreateFontProperties(Colors.Red, "Calibri", 16, false, true, true), true, TextAlignment.Right);

            this.RegisterTestCellValue(new CellIndex(25, 19), "3", new CellValueFormat(accountingFormat),
                CreateFontProperties(Colors.Green, "Cambria", 12, false, true, false), false, TextAlignment.Left, new RunRenderable[]
                {
                    new RunRenderable() { Text = "(", Foreground = new SolidColorBrush(transparentColor) },
                    new RunRenderable() { Text = "$" },
                    new RunRenderable() { Text = new String(' ', 8) },
                    new RunRenderable() { Text = "+", Foreground = new SolidColorBrush(transparentColor), FontFamily = new FontFamily("Arial"), FontSize=0.397945205479452 },
                    new RunRenderable() { Text = "3{ds}00".ReplaceGroupDecimalSeparators() },
                    new RunRenderable() { Text = ")", Foreground = new SolidColorBrush(transparentColor) },
                });

            this.RegisterTestCellValue(new CellIndex(30, 6), "Bauhaus", new CellValueFormat(generalFormat),
                CreateFontProperties(Colors.Blue, "Bauhaus 93", 13, false, false, false), false, TextAlignment.Left);

            this.RegisterTestCellValue(new CellIndex(6, 26), "Consolas", new CellValueFormat(generalFormat),
                CreateFontProperties(Colors.Purple, "Consolas", 11, true, false, false), true, TextAlignment.Right);
        }

        [TestMethod]
        public void SimpleScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 0, 1, 2);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(300, 90), 1);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 3);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(), 3);
            
            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 0, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 0, 1, 2, 3);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(600, 250), 2);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 2);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(400, 90), 0.5);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 2, 3, 4);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioCellValuesLayerTest()
        {
            this.Initialize(new Point(700, 200), 1.3);

            IEnumerable<TextBlockRenderable> expectedScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Scrollable, 2);

            IEnumerable<TextBlockRenderable> expectedHorizontalScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.HorizontalScrollable, 4);

            IEnumerable<TextBlockRenderable> expectedVerticalScrollableValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.VerticalScrollable);

            IEnumerable<TextBlockRenderable> expectedFixedValues = this.GetExpectedTextBlockRenderables(ViewportPaneType.Fixed, 0, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedScrollableValues, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(cellValuesLayer, expectedHorizontalScrollableValues, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(cellValuesLayer, expectedVerticalScrollableValues, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(cellValuesLayer, expectedFixedValues, ViewportPaneType.Fixed);
            }, new CellIndex(7, 6));
        }

        [TestMethod]
        public void TestFilteredRowsNotRenderingCellValues()
        {
            this.Initialize(new Point(), 1);

            Worksheet worksheet = new Workbook().Worksheets.Add();
            FontProperties fontProperties = CreateFontProperties(Colors.Orange, "Arial", 10, true, false, false);

            for (int i = 0; i <= 15; i++)
            {
                this.AddCellValue(worksheet, new CellIndex(i, 0), i.ToString(), CellValueFormat.GeneralFormat,
                    fontProperties, false, TextAlignment.Right);
            }

            worksheet.Filter.FilterRange = worksheet.UsedCellRange;
            worksheet.Filter.SetFilter(new ValuesCollectionFilter(0, new string[] { "0", "15" }));

            List<TextBlockRenderable> expectedRenderables = new List<TextBlockRenderable>();
            expectedRenderables.Add(this.GetExpectedRenderable(new CellIndex(0, 0), "0", 
                null, fontProperties, TextAlignment.Right, false, CellValueFormat.GeneralFormat));
            expectedRenderables.Add(this.GetExpectedRenderable(new CellIndex(15, 0), "15",
                null, fontProperties, TextAlignment.Right, false, CellValueFormat.GeneralFormat));
            expectedRenderables.Last().CellBox = new CellLayoutBox(15, 0, new Rect(0, 20, 65, 20), CellMergeState.NotMerged);

            base.TestLayer(worksheet, new Size(1350, 550), (layerWrapper) =>
            {
                CellValuesLayer cellValuesLayer = (CellValuesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(cellValuesLayer, expectedRenderables, ViewportPaneType.Scrollable);
            });
        }

        private void AssertLayerUpdate(CellValuesLayer layer, IEnumerable<TextBlockRenderable> expectedTextBlocks, ViewportPaneType paneType)
        {
            TestRenderer<TextBlockRenderable> renderer = (TestRenderer<TextBlockRenderable>)layer.TextRenderer;
            renderer.AssertAreRendered(expectedTextBlocks, paneType);
        }

        private IEnumerable<TextBlockRenderable> GetExpectedTextBlockRenderables(ViewportPaneType paneType, params int[] expectedTextBlockIndexes)
        {
            foreach (int index in expectedTextBlockIndexes)
            {
                yield return this.getExpectedTextBlockRenderable[index](paneType);
            }
        }

        private Worksheet CreateWorksheetScenario()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();

            foreach (Action<Worksheet> addImageAction in this.addCellValueAction)
            {
                addImageAction(worksheet);
            }

            return worksheet;
        }

        private static FontProperties CreateFontProperties(Color foreground, string fontFamilly, double fontSize, bool bold, bool italic, bool underline)
        {
            return new FontProperties(new FontFamily(fontFamilly), fontSize, bold)
            {
                IsItalic = italic,
                ForeColor = new ThemableColor(foreground),
                Underline = underline ? UnderlineType.Single : UnderlineType.None
            };
        }

        private void RegisterTestCellValue(CellIndex cellIndex, string inputText, CellValueFormat format, FontProperties fontProperties, bool isWrapped, TextAlignment alignment, RunRenderable[] resultRunCollection = null)
        {
            this.addCellValueAction.Add((worksheet) =>
            {
                AddCellValue(worksheet, cellIndex, inputText, format, fontProperties, isWrapped, alignment);
            });

            this.getExpectedTextBlockRenderable.Add((paneType) =>
            {
                return GetExpectedRenderable(cellIndex, inputText, resultRunCollection, fontProperties, alignment, isWrapped, format);
            });
        }

        private void AddCellValue(Worksheet worksheet, CellIndex cellIndex, string inputText, CellValueFormat format, FontProperties fontProperties, bool isWrapped, TextAlignment alignment)
        {
            CellSelection cell = worksheet.Cells[cellIndex];
            cell.SetValue(inputText);
            cell.SetFormat(format);
            cell.SetForeColor(fontProperties.ForeColor);
            cell.SetFontFamily(new ThemableFontFamily(fontProperties.FontFamily));
            cell.SetFontSize(fontProperties.FontSize);
            cell.SetIsBold(fontProperties.IsBold);
            cell.SetIsItalic(fontProperties.IsItalic);
            cell.SetUnderline(fontProperties.Underline);
            cell.SetIsWrapped(isWrapped);
            cell.SetHorizontalAlignment(GetRadAlignment(alignment));
        }

        private TextBlockRenderable GetExpectedRenderable(CellIndex cellIndex, string inputText, RunRenderable[] resultRunCollection, FontProperties fontProperties, TextAlignment alignment, bool isWrapped, CellValueFormat format)
        {
            TextBlockRenderable renderable = new TextBlockRenderable();
                  
            renderable.CellBox = this.GetLayoutBox(cellIndex);                
            this.SetText(renderable, inputText, resultRunCollection);
            renderable.FontSize = fontProperties.FontSize;
            renderable.FontFamily = fontProperties.FontFamily;
            renderable.FontWeight = fontProperties.FontWeight;
            renderable.FontStyle = fontProperties.FontStyle;
            renderable.TextDecorations = (fontProperties.Underline != UnderlineType.None) ? TextDecorations.Underline : null;
            renderable.Foreground = fontProperties.ForeColor.LocalValue;
            renderable.TextAlignment = alignment;
            renderable.TextWrapping = isWrapped ? TextWrapping.Wrap : TextWrapping.NoWrap;
            renderable.ScaleFactor = new Size(this.ScaleFactor, this.ScaleFactor);
            renderable.CellValueFormat = format;

            return renderable;
        }

        private CellLayoutBox GetLayoutBox(CellIndex cellIndex)
        {
            Rect boxRect = new Rect(cellIndex.ColumnIndex * ColumnWidth, cellIndex.RowIndex * RowHeight, ColumnWidth, RowHeight);
            CellLayoutBox box = new CellLayoutBox(cellIndex.RowIndex, cellIndex.ColumnIndex, boxRect, CellMergeState.NotMerged);

            return box;
        }

        private void SetText(TextBlockRenderable renderable, string text, RunRenderable[] runs)
        {
            if (runs == null)
            {
                renderable.Text = text;
            }
            else
            {
                foreach (RunRenderable run in runs)
                {
                    renderable.AddRun(run);
                }
            }
        }

        private static RadHorizontalAlignment GetRadAlignment(TextAlignment alignment)
        {
            switch (alignment)
            {
                case TextAlignment.Center:
                    return RadHorizontalAlignment.Center;
                case TextAlignment.Left:
                    return RadHorizontalAlignment.Left;
                case TextAlignment.Right:
                    return RadHorizontalAlignment.Right;
                default:
                    throw new NotSupportedException(String.Format("Not supported alignment: {0}!", alignment));
            }
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new CellValuesLayer(new TestRenderer<TextBlockRenderable>(RenderableComparer.AreEqual)));
        }
    }
}
