﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class ColumnHeadingMarginLayerTests : LayerTesterBase
    {
        private readonly Worksheet emptyWorksheet;

        public ColumnHeadingMarginLayerTests()
        {
            this.emptyWorksheet = new Workbook().Worksheets.Add();
        }

        [TestMethod]
        public void SimpleScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 0, 20);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(301, 205), 1);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 4, 16);

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 0, 6);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 0, 23);

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(210, 250), 2);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 3, 9);

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(210, 250), 0.5);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Scrollable, 3, 26);

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioColumnHeadingMarginLayerTest()
        {
            this.Initialize(new Point(250, 130), 1.3);

            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedScrollableHeadings = this.GetExpectedColumnHeadings();
            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedHorizontalScrollableHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.HorizontalScrollable, 9, 19);
            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedVerticalScrollableHeadings = this.GetExpectedColumnHeadings();
            IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedFixedHeadings = this.GetExpectedColumnHeadings(ViewportPaneType.Fixed, 0, 5);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                ColumnHeadingMarginLayer columnHeadingMarginLayer = (ColumnHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedHorizontalScrollableHeadings, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedVerticalScrollableHeadings, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(columnHeadingMarginLayer, expectedFixedHeadings, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }

        private IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> GetExpectedColumnHeadings()
        {
            yield break;
        }

        private IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> GetExpectedColumnHeadings(ViewportPaneType paneType, int fromColumnIndex, int toColumnIndex)
        {
            Rect startLayoutRect = new Rect(fromColumnIndex * ColumnWidth, 0, ColumnWidth, RowHeight * SpreadsheetDefaultValues.RowCount);

            double leftStart = fromColumnIndex * ColumnWidth - this.GetHorizontalScroll(paneType);
            double width = ColumnWidth;
            double height = SpreadsheetDefaultValues.RowColumnHeadingMinimumSize.Height;
            Rect firstRectangle = new Rect(leftStart, 0, width, height);
            Size scaleFactor = new Size(this.ScaleFactor, this.ScaleFactor);

            for (int column = fromColumnIndex; column <= toColumnIndex; column++)
            {
                double offset = (column - fromColumnIndex) * ColumnWidth;

                yield return new RowColumnHeadingRenderable<ColumnLayoutBox>
                {
                    LayoutBox = new ColumnLayoutBox(column, startLayoutRect.Translate(offset, 0)),
                    Rectangle = firstRectangle.Translate(offset, 0),
                    ScaleFactor = scaleFactor,
                    Text = NameConverter.ConvertColumnIndexToName(column)
                };
            }
        }

        private void AssertLayerUpdate(ColumnHeadingMarginLayer layer, IEnumerable<RowColumnHeadingRenderable<ColumnLayoutBox>> expectedRenderables, ViewportPaneType paneType)
        {
            TestRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>> renderer = (TestRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>>)layer.ColumnHeadingRenderer;
            renderer.AssertAreRendered(expectedRenderables, paneType);
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new ColumnHeadingMarginLayer(new TestRenderer<RowColumnHeadingRenderable<ColumnLayoutBox>>(RenderableComparer.AreEqual)));
        }
    }
}
