﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class GridlinesLayerTests : CellBordersLayerTestsBase
    {
        private readonly Worksheet emptyWorksheet;
        private static readonly BorderProperties ExpectedGridlineProperties;

        static GridlinesLayerTests()
        {
            GridlinesLayerTests.ExpectedGridlineProperties = new BorderProperties()
            {
                CellBorderStyle = CellBorderStyle.None,
                Stroke = Color.FromArgb(0xFF, 0xD3, 0xD3, 0xD3),
                StrokeThickness = 1
            };
        }

        public GridlinesLayerTests()
        {
            this.emptyWorksheet = new Workbook().Worksheets.Add();            
        }

        [TestMethod]
        public void SimpleScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(1, 1, 27, 20), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(301, 205), 1);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(11, 5, 40, 16), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(1, 1, 9, 6), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(1, 1, 60, 23), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(210, 250), 2);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(13, 4, 27, 9), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(210, 250), 0.5);

            IEnumerable<LineRenderable> expectedScrollableLines = this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(13, 4, 72, 26), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioGridlinesLayerTest()
        {
            this.Initialize(new Point(250, 130), 1.3);

            IEnumerable<LineRenderable> expectedScrollableLines = 
                this.GetExpectedGridlines(ViewportPaneType.Scrollable, new CellRange(12, 10, 27, 19), new Point(65, 20));
            IEnumerable<LineRenderable> expectedHorizontalScrollableLines = 
                this.GetExpectedGridlines(ViewportPaneType.HorizontalScrollable, new CellRange(1, 10, 4, 19), new Point(65, 20));
            IEnumerable<LineRenderable> expectedVerticalScrollableLines = 
                this.GetExpectedGridlines(ViewportPaneType.VerticalScrollable, new CellRange(12, 1, 27, 5), new Point(65, 20));
            IEnumerable<LineRenderable> expectedFixedLines = 
                this.GetExpectedGridlines(ViewportPaneType.Fixed, new CellRange(1, 1, 4, 5), new Point(65, 20));

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                GridlinesLayer gridlinesLayer = (GridlinesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(gridlinesLayer, expectedScrollableLines, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(gridlinesLayer, expectedHorizontalScrollableLines, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(gridlinesLayer, expectedVerticalScrollableLines, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(gridlinesLayer, expectedFixedLines, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }

        private IEnumerable<LineRenderable> GetExpectedGridlines(ViewportPaneType paneType, CellRange range, Point topLeftLinesOffset)
        {
            Point scaledTopLeftOffset = topLeftLinesOffset.Scale(this.ScaleFactor);
            Rect scaledBoundingBox = this.GetLinesBoundingBox(paneType, range, topLeftLinesOffset);

            foreach (LineRenderable horizontalLine in base.GetExpectedHorizontalLines(range, scaledBoundingBox, scaledTopLeftOffset, ExpectedGridlineProperties))
            {
                yield return horizontalLine;
            }

            foreach (LineRenderable verticalLine in base.GetExpectedVerticalLines(range, scaledBoundingBox, scaledTopLeftOffset, ExpectedGridlineProperties))
            {
                yield return verticalLine;
            }
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new GridlinesLayer(new TestRenderer<LineRenderable>(RenderableComparer.AreEqual)));
        }
    }
}
