﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class RowHeadingMarginLayerTests : LayerTesterBase
    {
        private readonly Worksheet emptyWorksheet;

        public RowHeadingMarginLayerTests()
        {
            this.emptyWorksheet = new Workbook().Worksheets.Add();
        }

        [TestMethod]
        public void SimpleScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 0, 27, 27);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrolledScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(301, 205), 1);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 10, 40, 40);

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 0, 9, 9);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 0, 60, 60);

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(210, 250), 2);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 12, 27, 27);

            base.TestLayer(this.emptyWorksheet, new Size(760, 605), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(210, 250), 0.5);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Scrollable, 12, 72, 72);

            base.TestLayer(this.emptyWorksheet, new Size(760, 600), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioRowHeadingMarginLayerTest()
        {
            this.Initialize(new Point(250, 130), 1.3);

            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedScrollableHeadings = this.GetExpectedRowHeadings();
            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedHorizontalScrollableHeadings = this.GetExpectedRowHeadings();
            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedVerticalScrollableHeadings = this.GetExpectedRowHeadings(ViewportPaneType.VerticalScrollable, 11, 27, 27);
            IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedFixedHeadings = this.GetExpectedRowHeadings(ViewportPaneType.Fixed, 0, 4, 27);

            base.TestLayer(this.emptyWorksheet, new Size(1350, 550), (layerWrapper) =>
            {
                RowHeadingMarginLayer rowHeadingMarginLayer = (RowHeadingMarginLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedScrollableHeadings, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedHorizontalScrollableHeadings, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedVerticalScrollableHeadings, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(rowHeadingMarginLayer, expectedFixedHeadings, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }

        private IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> GetExpectedRowHeadings()
        {
            yield break;
        }

        private IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> GetExpectedRowHeadings(ViewportPaneType paneType, int fromRowIndex, int toRowIndex, int maxVisibleRowIndex)
        {
            Rect startLayoutRect = new Rect(0, fromRowIndex * RowHeight, ColumnWidth * SpreadsheetDefaultValues.ColumnCount, RowHeight);

            double topStart = fromRowIndex * RowHeight - this.GetVerticalScroll(paneType);
            string rowName = NameConverter.ConvertRowIndexToName(maxVisibleRowIndex);
            double width = RadWorksheetLayout.GetRowHeadingDesiredWidth(rowName);
            double height = RowHeight;
            Rect firstRectangle = new Rect(0, topStart, width, height);
            Size scaleFactor = new Size(this.ScaleFactor, this.ScaleFactor);

            for (int row = fromRowIndex; row <= toRowIndex; row++)
            {
                double offset = (row - fromRowIndex) * RowHeight;

                yield return new RowColumnHeadingRenderable<RowLayoutBox>
                {
                    LayoutBox = new RowLayoutBox(row, startLayoutRect.Translate(0, offset)),
                    Rectangle = firstRectangle.Translate(0, offset),
                    ScaleFactor = scaleFactor,
                    Text = NameConverter.ConvertRowIndexToName(row)
                };
            }            
        }

        private void AssertLayerUpdate(RowHeadingMarginLayer layer, IEnumerable<RowColumnHeadingRenderable<RowLayoutBox>> expectedRenderables, ViewportPaneType paneType)
        {
            TestRenderer<RowColumnHeadingRenderable<RowLayoutBox>> renderer = (TestRenderer<RowColumnHeadingRenderable<RowLayoutBox>>)layer.RowHeadingRenderer;
            renderer.AssertAreRendered(expectedRenderables, paneType);
        }

        protected override LayerWrapper CreateTestLayer()
        {
            return new LayerWrapper(new RowHeadingMarginLayer(new TestRenderer<RowColumnHeadingRenderable<RowLayoutBox>>(RenderableComparer.AreEqual)));
        }
    }
}
