﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    [TestClass]
    public class ShapesLayerTests : LayerTesterBase
    {
        private readonly Worksheet emptyWorksheet;
        private readonly List<Func<ViewportPaneType, ImageRenderable>> getExpectedRenderable;
        private readonly List<Action<Worksheet>> addImageToWorksheet;

        public ShapesLayerTests()
        {
            this.getExpectedRenderable = new List<Func<ViewportPaneType, ImageRenderable>>();
            this.addImageToWorksheet = new List<Action<Worksheet>>();
            this.emptyWorksheet = new Workbook().Worksheets.Add();

            this.RegisterTestImage(new CellIndex(0, 0), 200, 300, 0, false, false);
            this.RegisterTestImage(new CellIndex(5, 5), 200, 150, 32, false, true);
            this.RegisterTestImage(new CellIndex(25, 20), 60, 100, 120, true, true);
        }

        [TestMethod]
        public void SimpleScenarioShapesLayerTest()
        {
            this.Initialize(new Point(), 1);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 0, 1, 2);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
                {
                    ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                    this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
                });
        }

        [TestMethod]
        public void ScrolledScenarioShapesLayerTest()
        {
            this.Initialize(new Point(301, 200), 1);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedInScenarioShapesLayerTest()
        {
            this.Initialize(new Point(), 3);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 0, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ZoomedOutScenarioShapesLayerTest()
        {
            this.Initialize(new Point(), 0.5);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 0, 1, 2);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomInScenarioShapesLayerTest()
        {
            this.Initialize(new Point(210, 250), 2);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void ScrollAndZoomOutScenarioShapesLayerTest()
        {
            this.Initialize(new Point(210, 250), 0.5);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 1, 2);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(760, 600), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
            });
        }

        [TestMethod]
        public void FreezePanesScenarioShapesLayerTest()
        {
            this.Initialize(new Point(260, 120), 1.3);

            IEnumerable<ImageRenderable> expectedScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.Scrollable, 2);
            IEnumerable<ImageRenderable> expectedHorizontalScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.HorizontalScrollable);
            IEnumerable<ImageRenderable> expectedVerticalScrollableImages = this.GetExpectedImageRenderables(ViewportPaneType.VerticalScrollable, 0, 1);
            IEnumerable<ImageRenderable> expectedFixedImages = this.GetExpectedImageRenderables(ViewportPaneType.Fixed, 0, 1);

            base.TestLayer(this.CreateWorksheetScenario(), new Size(1350, 550), (layerWrapper) =>
            {
                ShapesLayer shapesLayer = (ShapesLayer)layerWrapper.Layer;
                this.AssertLayerUpdate(shapesLayer, expectedScrollableImages, ViewportPaneType.Scrollable);
                this.AssertLayerUpdate(shapesLayer, expectedHorizontalScrollableImages, ViewportPaneType.HorizontalScrollable);
                this.AssertLayerUpdate(shapesLayer, expectedVerticalScrollableImages, ViewportPaneType.VerticalScrollable);
                this.AssertLayerUpdate(shapesLayer, expectedFixedImages, ViewportPaneType.Fixed);
            }, new CellIndex(5, 6));
        }        

        protected override LayerWrapper CreateTestLayer()
        {
#if !SILVERLIGHT
            return new LayerWrapper(new ShapesLayer(new TestRenderer<ShapeRenderable>(RenderableComparer.AreEqual), new ShapeSourceFactory(new ImageSourceCache(), new TestChartShapeSource())));
#else
            return new LayerWrapper(new ShapesLayer(new TestRenderer<ShapeRenderable>(RenderableComparer.AreEqual), new ShapeSourceFactory(new ImageSourceCache())));
#endif
        }

        private void AssertLayerUpdate(ShapesLayer layer, IEnumerable<ImageRenderable> expectedRenderables, ViewportPaneType paneType)
        {
            TestRenderer<ShapeRenderable> renderer = (TestRenderer<ShapeRenderable>)layer.ShapeRenderer;
            renderer.AssertAreRendered(expectedRenderables, paneType);
        }

        private IEnumerable<ImageRenderable> GetExpectedImageRenderables(ViewportPaneType paneType, params int[] expectedImagesIndexes)
        {
            foreach (int index in expectedImagesIndexes)
            {
                yield return this.getExpectedRenderable[index](paneType);
            }
        }

        private Worksheet CreateWorksheetScenario()
        {
            Worksheet worksheet = new Workbook().Worksheets.Add();

            foreach (Action<Worksheet> addImageAction in this.addImageToWorksheet)
            {
                addImageAction(worksheet);
            }
            
            return worksheet;
        }

        private void RegisterTestImage(CellIndex cellIndex, double width, double height, double rotation, bool flipHorizontally, bool flipVertically)
        {
            this.addImageToWorksheet.Add((worksheet) =>
                {
                    ShapesLayerTests.AddImage(worksheet, cellIndex, width, height, rotation, flipHorizontally, flipVertically);
                });

            this.getExpectedRenderable.Add((paneType) =>
                {
                    Worksheet worksheet = this.emptyWorksheet;
                    RadWorksheetLayout layout = worksheet.Workbook.GetWorksheetLayout(worksheet, true);
                    Rect documentRect = new Rect(layout.GetColumnLeft(cellIndex.ColumnIndex), layout.GetRowTop(cellIndex.RowIndex), width, height);

                    return ShapesLayerTests.CreateExpectedImageRenderable(worksheet, cellIndex, this.TranslateAndScale(documentRect, paneType), rotation, flipHorizontally, flipVertically);
                });
        }

        private static ImageRenderable CreateExpectedImageRenderable(Worksheet worksheet, CellIndex cellIndex, Rect rect, double rotation, bool horizontallyFlipped, bool verticallyFlipped)
        {
            FloatingImage picture = new FloatingImage(worksheet, cellIndex, 0, 0)
            {
                IsHorizontallyFlipped = horizontallyFlipped,
                IsVerticallyFlipped = verticallyFlipped,
                RotationAngle = rotation
            };

            return new ImageRenderable()
            {
                TopLeft = new Point(rect.Left, rect.Top),
                Width = rect.Width,
                Height = rect.Height,
                Shape = picture
            };
        }

        private static void AddImage(Worksheet worksheet, CellIndex cellIndex, double width, double height, double rotation, bool flipHorizontally, bool flipVertically)
        {
            FloatingImage image = new FloatingImage(worksheet, cellIndex, 0, 0);

            using (Stream stream = TestHelperInternal.GetStreamFromFile("Penguins.jpg"))
            {
                image.ImageSource = new Telerik.Windows.Documents.Media.ImageSource(stream, "jpg");
            }

            image.Width = width;
            image.Height = height;
            image.SetRotationAngle(rotation, true);
            image.IsHorizontallyFlipped = flipHorizontally;
            image.IsVerticallyFlipped = flipVertically;

            worksheet.Shapes.Add(image);
        }
    }
}
