﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Layout.Layers;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayersTests
{
    internal class TestRenderer<T> : IRenderer<T>
        where T : IRenderable
    {
        private readonly Func<T, T, bool> areEqual;
        private readonly Dictionary<ViewportPaneType, List<T>> paneTypeToRenderedSets;

        public TestRenderer(Func<T, T, bool> areEqual)
        {
            this.areEqual = areEqual;
            this.paneTypeToRenderedSets = new Dictionary<ViewportPaneType, List<T>>();

            foreach (ViewportPaneType paneType in Enum.GetValues(typeof(ViewportPaneType)))
            {
                this.paneTypeToRenderedSets.Add(paneType, new List<T>());
            }
        }

        public void Render(T renderable, ViewportPaneType paneType)
        {
            if (this.Contains(renderable, paneType))
            {
                throw new InvalidOperationException("Cannot render same object twice!");
            }

            this.paneTypeToRenderedSets[paneType].Add(renderable);
        }

        public void AssertAreRendered(IEnumerable<T> expectedRenderables, ViewportPaneType paneType)
        {
            List<T> renderedSet = this.paneTypeToRenderedSets[paneType];
            List<T> expectedSet = new List<T>(expectedRenderables);
            Assert.AreEqual(expectedSet.Count, renderedSet.Count);

            for (int i = 0; i < expectedSet.Count; i++)
            {
                T expectedRenderable = expectedSet[i];
                Assert.IsTrue(this.Contains(expectedRenderable, paneType));
            }
        }

        private bool Contains(T renderable, ViewportPaneType paneType)
        {
            List<T> renderedSet = this.paneTypeToRenderedSets[paneType];

            for (int i = 0; i < renderedSet.Count; i++)
            {
                if(this.areEqual(renderedSet[i], renderable))
                {
                    return true;
                }
            }

            return false;
        }
    }
}
