﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayoutTests
{
    [TestClass]
    public class LayoutHelperTests
    {
        [TestMethod]
        public void CalculateCellContentSize_WithZeroWrappingWidth()
        {
            // Arrange
            double? wrappingWidth = 0;

            CellProperties cellProperties = GetCellProperties();

            // Act
            Size cellSize = LayoutHelper.CalculateCellContentSize(cellProperties.Value, cellProperties.Format, cellProperties.FontProperties,
                                cellProperties.Indent, wrappingWidth, LayoutHelper.DefaultTextMeasuringMethod);

            // Assert
            Size expectedCellSize = new Size(24.503333333333334, 14.4);
            Assert.AreEqual(expectedCellSize, cellSize);
        }

        [TestMethod]
        public void CalculateCellContentSize_WithNegativeWrappingWidth()
        {
            // Arrange
            double? wrappingWidth = -10;

            CellProperties cellProperties = GetCellProperties();

            // Act
            Size cellSize = LayoutHelper.CalculateCellContentSize(cellProperties.Value, cellProperties.Format, cellProperties.FontProperties,
                                cellProperties.Indent, wrappingWidth, LayoutHelper.DefaultTextMeasuringMethod);

            // Assert
            Size expectedCellSize = new Size(24.503333333333334, 14.4);
            Assert.AreEqual(expectedCellSize, cellSize);
        }

        [TestMethod]
        public void CalculateCellContentSize_WithPositiveWrappingWidth()
        {
            // Arrange
            double? wrappingWidth = 10;

            CellProperties cellProperties = GetCellProperties();

            // Act
            Size actualCellSize = LayoutHelper.CalculateCellContentSize(cellProperties.Value, cellProperties.Format, cellProperties.FontProperties,
                                cellProperties.Indent, wrappingWidth, LayoutHelper.DefaultTextMeasuringMethod);

            // Assert
            Size expectedCellSize = new Size(11.276666666666667, 57.6);
            Assert.AreEqual(expectedCellSize, actualCellSize);
        }

        [TestMethod]
        public void CalculateCellContentSize_WithNullWrappingWidth()
        {
            double? wrappingWidth = null;

            CellProperties cellProperties = GetCellProperties();

            // Act
            Size cellSize = LayoutHelper.CalculateCellContentSize(cellProperties.Value, cellProperties.Format, cellProperties.FontProperties,
                                cellProperties.Indent, wrappingWidth, LayoutHelper.DefaultTextMeasuringMethod);

            // Assert
            Size expectedCellSize = new Size(24.4996, 14.4);
            Assert.AreEqual(expectedCellSize, cellSize);
        }

        private static CellProperties GetCellProperties()
        {
            return new CellProperties
            {
                Value = new TextCellValue("test"),
                Format = new CellValueFormat("test"),
                FontProperties = new FontProperties(new System.Windows.Media.FontFamily("test"), 12, false),
                IsWrapped = true,
                Indent = 0
            };
        }
    }
}
