﻿using System;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.LayoutTests
{
    [TestClass]
    public class WorksheetLayoutTests
    {
        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetBottomCellIndexFromPoint()
        {
            bool getLeftCellWithPriority = false;
            bool getTopCellWithPriority = false;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[1, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, 20);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(4, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetTopCellIndexFromPointAllTopRowsAreHidden()
        {
            bool getLeftCellWithPriority = true;
            bool getTopCellWithPriority = true;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[0, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, 0);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(4, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetBottomCellIndexFromPointAllTopRowsAreHidden()
        {
            bool getLeftCellWithPriority = false;
            bool getTopCellWithPriority = false;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[0, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, 0);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(4, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetTopCellIndexFromPointAllBottomRowsAreHidden()
        {
            bool getLeftCellWithPriority = true;
            bool getTopCellWithPriority = true;
            int bottomRowIndex = SpreadsheetDefaultValues.RowCount - 1;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[SpreadsheetDefaultValues.RowCount - 3, SpreadsheetDefaultValues.RowCount - 1].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, layout.GetRowTop(bottomRowIndex) + layout.GetRowHeight(bottomRowIndex));
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(SpreadsheetDefaultValues.RowCount - 4, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetBottomCellIndexFromPointAllBottomRowsAreHidden()
        {
            bool getLeftCellWithPriority = false;
            bool getTopCellWithPriority = false;
            int bottomRowIndex = SpreadsheetDefaultValues.RowCount - 1;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[SpreadsheetDefaultValues.RowCount - 3, SpreadsheetDefaultValues.RowCount - 1].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, layout.GetRowTop(bottomRowIndex) + layout.GetRowHeight(bottomRowIndex));
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(SpreadsheetDefaultValues.RowCount - 4, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetTopCellIndexFromPoint()
        {
            bool getLeftCellWithPriority = true;
            bool getTopCellWithPriority = true;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Rows[1, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(0, 20);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(0, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetRightCellIndexFromPoint()
        {
            bool getLeftCellWithPriority = false;
            bool getTopCellWithPriority = false;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[1, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(65, 0);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(0, rowIndex);
            Assert.AreEqual(4, columnIndex);
        }

        [TestMethod]
        [TestCategory("GetCellIndexFromPoint")]
        public void GetLeftCellIndexFromPoint()
        {
            bool getLeftCellWithPriority = true;
            bool getTopCellWithPriority = true;

            Worksheet worksheet = TestHelper.CreateWorksheet();
            worksheet.Columns[1, 3].SetHidden(true);

            RadWorksheetLayout layout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            layout.SetWorksheet(worksheet, false);
            layout.Measure();

            Point point = new Point(65, 0);
            int rowIndex;
            int columnIndex;
            layout.GetCellIndexFromPoint(point.X, point.Y, getLeftCellWithPriority, getTopCellWithPriority, out rowIndex, out columnIndex, true, true);

            Assert.AreEqual(0, rowIndex);
            Assert.AreEqual(0, columnIndex);
        }
    }
}