﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Windows.Documents.Spreadsheet.Model.Printing;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Printing
{
    [TestClass]
    public class HeaderFooterTests
    {
        [TestMethod]
        public void TestSanitizerRemovingUnnecessaryColorChangeAtTheEnd()
        {
            string unfinishedColorAtTheEnd = @"Let's test all fields:
        Ampersand-&&

Color-&KFF0000red
Automatic color-&K000000black
Theme color-&K05-024theme
&""Algerian,Regular""font family change
&""-,Bold Italic""normal font bold italic
&""-,Bold""normal font bold
&K12345";

            string actualResult = new HeaderFooterSectionTextSanitizer().Sanitize(unfinishedColorAtTheEnd);
            string expectedResult = @"Let's test all fields:
        Ampersand-&&

Color-&KFF0000red
Automatic color-&K000000black
Theme color-&K05-024theme
&""Algerian,Regular""font family change
&""-,Bold Italic""normal font bold italic
&""-,Bold""normal font bold
";

            Assert.AreEqual(expectedResult, actualResult);
        }

        [TestMethod]
        public void TestSanitizerRemovingFalseAmpersand()
        {
            string falseAmpersand = @"
&""-,Italic""normal font italic,
&""-,Regular""normal font
&false ampersand
&Uunderline
&Edouble underline
        tabs        and spaces    and new lines
&Eunderline off


new lines";

            string actualResult = new HeaderFooterSectionTextSanitizer().Sanitize(falseAmpersand);
            string expectedResult = @"
&""-,Italic""normal font italic,
&""-,Regular""normal font
false ampersand
&Uunderline
&Edouble underline
        tabs        and spaces    and new lines
&Eunderline off


new lines";

            Assert.AreEqual(expectedResult, actualResult);
        }

        [TestMethod]
        public void TestSanitizerRemovingUnsupportedPictureElementAndFontStyleAtTheEnd()
        {
            string unsupportedPictureElement = @"&Sstrikethrough
&S&Ysubscript
&Xsuperscript
&Xsuperscript off
&17.17 font size
&Ddate
&Ttime
&Zfolder
&Ffile
&Aworksheet
&Gpicture
different new lines" + "\n\r\r\n\n\rthe end&\"-,Italic\"";

            string actualResult = new HeaderFooterSectionTextSanitizer().Sanitize(unsupportedPictureElement);
            string expectedResult = @"&Sstrikethrough
&S&Ysubscript
&Xsuperscript
&Xsuperscript off
&17.17 font size
&Ddate
&Ttime
&Zfolder
&Ffile
&Aworksheet
picture
different new lines" + "\n\r\r\n\n\rthe end";

            Assert.AreEqual(expectedResult, actualResult);

        }

        [TestMethod]
        public void TestSanitizerRemovingUnnecessaryFontSizeChangeAtTheEnd()
        {
            string fontSizeAtTheEnd = "Page &\"-,Bold Italic\"&P &\"-,Regular\"from &\"-,Bold Italic\"&N&234";
            
            string actualResult = new HeaderFooterSectionTextSanitizer().Sanitize(fontSizeAtTheEnd);
            string expectedResult = "Page &\"-,Bold Italic\"&P &\"-,Regular\"from &\"-,Bold Italic\"&N";

            Assert.AreEqual(expectedResult, actualResult);
        }
    }
}
