﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model.Protection;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Protection
{
    [TestClass]
    public class ProtectionHelperTests
    {
        private const int DefaultSpinCount = 100000;
        private const string Salt = "PW1HZzaGz4QXLjHyskhu9g==";
        private const string Password = "123";

        [TestMethod]
        public void Sha1Test()
        {
            string expectedHash = "QGdU3XnDUWj2+ic07xXGwhkLFyQ=";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "SHA-1", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void Sha256Test()
        {
            string expectedHash = "njMU1icSEb9y2L9LwcQ2Rmiu9IUTOwbQgAYc8CO8dKE=";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "SHA-256", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void Sha384Test()
        {
            string expectedHash = "Dhi6+vUAbKJfXJPA3iPIjHshD5udoIEg5afb8e1u9oOt82y0I4VasMv8qS3fmCm8";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "SHA-384", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void Sha512Test()
        {
            string expectedHash = "iJaBekjKn6k9XrZxWgKVcE+Znbt6bPBvtPUnDk0uKOSdsP3mnjqpkyhnHG4NDtb2xe84g/cviorhdBO0vl4keQ==";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "SHA-512", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void RipeMD160Test()
        {
            string expectedHash = "t+iNBU/ZobgWbIVwJlOphqmi4mQ=";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "RIPEMD-160", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void WhirlpoolTest()
        {
            string expectedHash = "Yj5441HWMY0xF4uhuz496yOBhGnMT5bHVpiu95PbyWUhxpfLLN/gXAAekGxz1KmCPEEG03GtdFatcyZXlqBnIg==";

            string hash = SpreadsheetProtectionHelper.Instance.GenerateHashBase64(Password, Salt, "WHIRLPOOL", DefaultSpinCount);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void LegacyHashShortPasswordTest()
        {
            string expectedHash = "CF7A";

            string hash = SpreadsheetProtectionHelper.Instance.GeneratePasswordLegacyScheme(Password);
            Assert.AreEqual(expectedHash, hash);
        }

        [TestMethod]
        public void LegacyHashLongPasswordTest()
        {
            string password = "QWErty123";
            string expectedHash = "96F4";

            string hash = SpreadsheetProtectionHelper.Instance.GeneratePasswordLegacyScheme(password);
            Assert.AreEqual(expectedHash, hash);
        }
    }
}
