﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Measurement;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;
using Telerik.Windows.Documents.Spreadsheet.Utilities.Performance;
using TextMeasurementInfo = Telerik.Windows.Documents.Core.TextMeasurer.TextMeasurementInfo;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    [TestClass]
    public class RadTextMeasurerTests
    {
        #region Constants

        private static readonly string[] FONTS = { "Arial", "Calibri", "Times New Roman" };
        private static readonly double SPEED_FACTOR = 1.2;
        private static readonly string SAMPLE_STRING = "’_-–/|:[]().,!~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
        private static readonly string SAMPLE_LONG_STRING =
@"2.Project Tesseract – Motivation and History
Tesseract is an open-source OCR engine that was developed at HP between 1984 and 1994. Like a supernova, it appeared from nowhere for the 1995 UNLV Annual Test of OCR Accuracy, shone brightly with its results, and then vanished back under the same cloak of secrecy under which it had been developed. Now for the first time, details of the architecture and algorithms can be revealed.
Tesseract began as a PhD research project in HP Labs, Bristol, and gained momentum as a possible software and/or hardware add-on for HP’s line of flatbed scanners. Motivation was provided by the fact that the commercial OCR engines of the day were in their infancy, and failed miserably on anything but the best quality print. After a joint project between HP Labs Bristol, and HP’s scanner division in Colorado, Tesseract had a significant lead in accuracy over the commercial engines, but did not become a product. The next stage of its development was back in HP Labs Bristol as an investigation of OCR for compression. Work concentrated more on improving rejection efficiency than on base-level accuracy. At the end of this project, at the end of 1994, development ceased entirely. The engine was sent to UNLV for the 1995 Annual Test of OCR Accuracy, where it proved its worth against the commercial engines of the time. In late 2005, HP released Tesseract for open source. It is now available at a. 
3.Architecture
Since HP had independently-developed page layout analysis technology that was used in products, (and therefore not released for open-source) Tesseract never needed its own page layout analysis. Tesseract therefore assumes that its input is a binary image with optional polygonal text regions defined.
 
Processing follows a traditional step-by-step pipeline, but some of the stages were unusual in their day, and possibly remain so even now. The first step is a connected component analysis in which outlines of the components are stored. This was a computationally expensive design decision at the time, but had a significant advantage: by inspection of the nesting of outlines, and the number of child and grandchild outlines, it is simple to detect inverse text and recognize it as easily as black-on-white text. Tesseract was probably the first OCR engine able to handle white-on-black text so trivially. At this stage, outlines are gathered together, purely by nesting, into Blobs.
Blobs are organized into text lines, and the lines and regions are analyzed for fixed pitch or proportional text. Text lines are broken into words differently according to the kind of character spacing. Fixed pitch text is chopped immediately by character cells. Proportional text is broken into words using definite spaces and fuzzy spaces.
Recognition then proceeds as a two-pass process. In the first pass, an attempt is made to recognize each word in turn. Each word that is satisfactory is passed to an adaptive classifier as training data. The adaptive classifier then gets a chance to more accurately recognize text lower down the page. 
Since the adaptive classifier may have learned something useful too late to make a contribution near the top of the page, a second pass is run over the page, in which words that were not recognized well enough are recognized again. 
A final phase resolves fuzzy spaces, and checks alternative hypotheses for the x-height to locate smallcap text.
4.Adaptive Tresholding
Thresholding is the simplest method of image segmentation. From a grayscale image, thresholding can be used to create binary images. 
During the thresholding process, individual pixels in an image are marked as object pixels if their value is greater than some threshold value (assuming an object to be brighter than the background) and as background pixels otherwise. This convention is known as threshold above. Variants include threshold below, which is opposite of threshold above; threshold inside, where a pixel is labeled object if its value is between two thresholds; and threshold outside, which is the opposite of threshold inside (Shapiro, et al. 2001:83). Typically, an object pixel is given a value of 1 while a background pixel is given a value of 0. Finally, a binary image is created by coloring each pixel white or black, depending on a pixel's label.
Thresholding is called adaptive thresholding when a different threshold is used for different regions in the image. This may also be known as local or dynamic thresholding. 
The key parameter in the thresholding process is the choice of the threshold value (or values, as mentioned earlier). Several different methods for choosing a threshold exist; users can manually choose a threshold value, or a thresholding algorithm can compute a value automatically, which is known as automatic thresholding. A simple method would be to choose the mean or median value, the rationale being that if the object pixels are brighter than the background, they should also be brighter than the average. In a noiseless image with uniform background and object values, the mean or median will work well as the threshold, however, this will generally not be the case. A more sophisticated approach might be to create a histogram of the image pixel intensities and use the valley point as the threshold. The histogram approach assumes that there is some average value for the background and object pixels, but that the actual pixel values have some variation around these average values. However, this may be computationally expensive, and image histograms may not have clearly defined valley points, often making the selection of an accurate threshold difficult. One method that is relatively simple, does not require much specific knowledge of the image, and is robust against image noise, is the following iterative method:2.Project Tesseract – Motivation and History
Tesseract is an open-source OCR engine that was developed at HP between 1984 and 1994. Like a supernova, it appeared from nowhere for the 1995 UNLV Annual Test of OCR Accuracy, shone brightly with its results, and then vanished back under the same cloak of secrecy under which it had been developed. Now for the first time, details of the architecture and algorithms can be revealed.
Tesseract began as a PhD research project in HP Labs, Bristol, and gained momentum as a possible software and/or hardware add-on for HP’s line of flatbed scanners. Motivation was provided by the fact that the commercial OCR engines of the day were in their infancy, and failed miserably on anything but the best quality print. After a joint project between HP Labs Bristol, and HP’s scanner division in Colorado, Tesseract had a significant lead in accuracy over the commercial engines, but did not become a product. The next stage of its development was back in HP Labs Bristol as an investigation of OCR for compression. Work concentrated more on improving rejection efficiency than on base-level accuracy. At the end of this project, at the end of 1994, development ceased entirely. The engine was sent to UNLV for the 1995 Annual Test of OCR Accuracy, where it proved its worth against the commercial engines of the time. In late 2005, HP released Tesseract for open source. It is now available at a.";

        #endregion


        #region HelperMethods

        private bool AreClose(double f1, double f2)
        {
            return Math.Abs(f1 - f2) < 0.5;
        }

        public int Compare(KeyValuePair<string, double> a, KeyValuePair<string, double> b)
        {
            return Math.Abs(b.Value).CompareTo(Math.Abs(a.Value));
        }

        #endregion


        #region Methods

        [TestMethod]
        public void TestMeasureWithUnknownFont()
        {
            FontProperties fontProperties = new FontProperties();
            fontProperties.FontSize = 12;
            fontProperties.FontFamily = new FontFamily("SomeFontThatIsNotPresentForSure");
            try
            {
                TextMeasurementInfo result = RadTextMeasurer.Measure(SAMPLE_STRING, fontProperties);
            }
            catch
            {
                Assert.Fail("Measure with unknown font problem.");
            }
        }

        [TestMethod]
        public void TestABCWarmUpMeasurementSystem()
        {
            FontProperties fontProperties = new FontProperties();
            fontProperties.FontSize = 12;
            foreach (var font in FONTS)
            {
                fontProperties.IsBold = false;
                fontProperties.FontFamily = new FontFamily(font);
                RadTextMeasurer.Measure(SAMPLE_STRING, fontProperties);

                fontProperties.IsBold = true;
                RadTextMeasurer.Measure(SAMPLE_STRING, fontProperties);
            }
        }

        [TestMethod]
        public void TestCorrectness()
        {
            FontProperties fontProperties = new FontProperties();
            fontProperties.FontFamily = new FontFamily("Verdana");
            fontProperties.FontSize = 12;

            TextBlock block = new TextBlock();
            block.Text = SAMPLE_STRING;
            block.FontFamily = fontProperties.FontFamily;
            block.FontSize = fontProperties.FontSize;

            Size blockSize = block.GetSize();

            Size radBlockSize = RadTextMeasurer.Measure(SAMPLE_STRING, fontProperties).Size;

            Assert.IsTrue(
                AreClose(blockSize.Width, radBlockSize.Width) &&
                AreClose(blockSize.Height, radBlockSize.Height));
        }

        [TestMethod]
        public void TestSpeedWithLotsOfWords()
        {
            string text = SAMPLE_LONG_STRING.Replace(Environment.NewLine, " ");
            string[] words = text.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

            foreach (var font in FONTS)
            {
                double smallestError = double.MaxValue;
                double biggestError = 0;
                double averageError = 0;
                string worstString = string.Empty;
                FontProperties fontProperties = new FontProperties();
                fontProperties.FontFamily = new FontFamily(font);
                fontProperties.FontSize = 12;

                TextBlock block = new TextBlock();
                block.Text = SAMPLE_STRING;
                block.FontFamily = fontProperties.FontFamily;
                block.FontSize = fontProperties.FontSize;

                SnippetPerformanceCounter textBlockCounter = new SnippetPerformanceCounter("TextBlockMeasure");
                SnippetPerformanceCounter radMeasureCounter = new SnippetPerformanceCounter("radMeasureCounter");
                foreach (string word in words)
                {
                    textBlockCounter.Start();
                    block.Text = word;

                    Size blockSize = block.GetSize();

                    textBlockCounter.Stop();

                    radMeasureCounter.Start();
                    Size radBlockSize = RadTextMeasurer.Measure(word, fontProperties).Size;
                    radMeasureCounter.Stop();

                    if (!AreClose(blockSize.Width, radBlockSize.Width) ||
                        !AreClose(blockSize.Height, radBlockSize.Height))
                    {
                        Assert.IsTrue(
                            AreClose(blockSize.Width, radBlockSize.Width) &&
                            AreClose(blockSize.Height, radBlockSize.Height));
                    }

                    var err = Math.Abs(blockSize.Width - radBlockSize.Width);
                    smallestError = Math.Min(smallestError, err);
                    if (err > biggestError)
                    {
                        biggestError = err;
                        worstString = word;
                    }
                    averageError += err;
                }
                textBlockCounter.End();
                radMeasureCounter.End();

                averageError /= words.Length;

                Assert.IsTrue(radMeasureCounter.TotalMilliseconds * SPEED_FACTOR <= textBlockCounter.TotalMilliseconds);
            }
        }

        // TODO: This test is not stable!
        //[TestMethod]
        //public void TestBoldCorrectnessWithOneWord()
        //{
        //    string text = SAMPLE_STRING;

        //    foreach (var font in FONTS)
        //    {
        //        FontProperties fontProperties = new FontProperties();
        //        fontProperties.FontFamily = new FontFamily(font);
        //        fontProperties.FontSize = 12;
        //        fontProperties.IsBold = true;

        //        TextBlock block = new TextBlock();
        //        block.Text = SAMPLE_STRING;
        //        block.FontFamily = fontProperties.FontFamily;
        //        block.FontSize = fontProperties.FontSize;
        //        block.FontWeight = FontWeights.Bold;

        //        SnippetPerformanceCounter textBlockCounter = new SnippetPerformanceCounter("TextBlockMeasure");
        //        SnippetPerformanceCounter radMeasureCounter = new SnippetPerformanceCounter("radMeasureCounter");
        //        foreach (char ch in text)
        //        {
        //            textBlockCounter.Start();
        //            block.Text = ch.ToString();
        //            Size blockSize = block.GetSize();
        //            textBlockCounter.Stop();

        //            radMeasureCounter.Start();
        //            Size radBlockSize = RadTextMeasurer.Measure(ch.ToString(), fontProperties).Size;
        //            radMeasureCounter.Stop();

        //            Assert.IsTrue(
        //                AreClose(blockSize.Width, radBlockSize.Width) &&
        //                AreClose(blockSize.Height, radBlockSize.Height));
        //        }
        //        textBlockCounter.End();
        //        radMeasureCounter.End();

        //        Assert.IsTrue(radMeasureCounter.TotalMilliseconds <= textBlockCounter.TotalMilliseconds);
        //    }
        //}

        [TestMethod]
        public void TestBoldSpeedAndCorrectnessWithLotsOfWords()
        {
            string text = SAMPLE_LONG_STRING.Replace(Environment.NewLine, " ");
            string[] words = text.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

            foreach (var font in FONTS)
            {
                FontProperties fontProperties = new FontProperties();
                fontProperties.FontFamily = new FontFamily(font);
                fontProperties.FontSize = 12;
                fontProperties.IsBold = true;

                TextBlock block = new TextBlock();
                block.Text = SAMPLE_STRING;
                block.FontFamily = fontProperties.FontFamily;
                block.FontSize = fontProperties.FontSize;
                block.FontWeight = FontWeights.Bold;

                SnippetPerformanceCounter textBlockCounter = new SnippetPerformanceCounter("TextBlockMeasure");
                SnippetPerformanceCounter radMeasureCounter = new SnippetPerformanceCounter("radMeasureCounter");
                foreach (string word in words)
                {
                    textBlockCounter.Start();
                    block.Text = word;
                    Size blockSize = block.GetSize();
                    textBlockCounter.Stop();

                    radMeasureCounter.Start();
                    Size radBlockSize = RadTextMeasurer.Measure(word, fontProperties).Size;
                    radMeasureCounter.Stop();

                    Assert.IsTrue(
                        AreClose(blockSize.Width, radBlockSize.Width) &&
                        AreClose(blockSize.Height, radBlockSize.Height));
                }
                textBlockCounter.End();
                radMeasureCounter.End();

                Assert.IsTrue(radMeasureCounter.TotalMilliseconds * SPEED_FACTOR <= textBlockCounter.TotalMilliseconds);
            }
        }

        [TestMethod]
        public void TestBaselineOffsetCorrectness()
        {
            string text = SAMPLE_LONG_STRING.Replace(Environment.NewLine, " ");
            string[] words = text.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
            foreach (var font in FONTS)
            {
                FontProperties fontProperties = new FontProperties();
                fontProperties.FontFamily = new FontFamily(font);
                fontProperties.FontSize = 12;
                fontProperties.IsBold = true;

                TextBlock block = new TextBlock();
                block.Text = SAMPLE_STRING;
                block.FontFamily = fontProperties.FontFamily;
                block.FontSize = fontProperties.FontSize;
                block.FontWeight = FontWeights.Bold;
                block.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));

                foreach (string word in words)
                {
                    block.Text = word;
                    double baseLineOffset = block.BaselineOffset;
                    double radBaseLineOffset = RadTextMeasurer.Measure(word, fontProperties).BaselineOffset;

                    Assert.IsTrue(AreClose(baseLineOffset, radBaseLineOffset));
                }
            }
        }

        [TestMethod]
        public void TestPairSymbolMeasureCorrectness()
        {
            TextBlock block = new TextBlock();
            foreach (var font in FONTS)
            {
                FontProperties fontProperties = new FontProperties();
                fontProperties.FontFamily = new FontFamily(font);
                fontProperties.FontSize = 12;


                block.FontFamily = fontProperties.FontFamily;
                block.FontSize = fontProperties.FontSize;

                double biggestError = 0;
                char errCh1 = 'a';
                char errCh2 = 'a';
                char ch1;
                char ch2;
                List<KeyValuePair<string, double>> list = new List<KeyValuePair<string, double>>();
                for (int i = 0; i < SAMPLE_STRING.Length; i++)
                {
                    ch1 = SAMPLE_STRING[i];
                    for (int j = 0; j < SAMPLE_STRING.Length; j++)
                    {
                        ch2 = SAMPLE_STRING[j];
                        block.Text = ch1.ToString() + ch2.ToString();
                        var blockWidth = block.GetSize().Width;

                        double radWidth = RadTextMeasurer.Measure(block.Text, fontProperties).Size.Width;


                        double currError = Math.Abs(blockWidth - radWidth);
                        if (biggestError < currError)
                        {
                            biggestError = Math.Max(biggestError, currError);
                            errCh1 = ch1;
                            errCh2 = ch2;
                        }

                        list.Add(new KeyValuePair<string, double>(ch1.ToString() + ch2.ToString(), blockWidth - radWidth));
                    }
                }

                list.Sort(Compare);
            }
        }

        //[TestMethod]
        //public void TestSpecialWordCorrectness()
        //{
        //    TextBlock block = new TextBlock();
        //    foreach (var font in Telerik.Windows.Documents.Core.Fonts.SystemFontsManager.GetAvailableFonts())
        //    {
        //        FontProperties fontProperties = new FontProperties();
        //        fontProperties.FontFamily = new FontFamily(font);
        //        fontProperties.FontSize = 12;


        //        block.FontFamily = fontProperties.FontFamily;
        //        block.FontSize = fontProperties.FontSize;

        //        block.Text = "average,average,average,average,average,average,average,";
        //        var blockSize = block.GetSize();
        //        var radBlockSize = RadTextMeasurer.Measure(block.Text, fontProperties).Size;

        //        Assert.IsTrue(AreClose(blockSize.Width, radBlockSize.Width) &&
        //            AreClose(blockSize.Height, radBlockSize.Height));
        //    }
        //}

        //[TestMethod]
        //public void TestKerningCorrectness()
        //{
        //    TextBlock block = new TextBlock();
        //    foreach (var font in Telerik.Windows.Documents.Core.Fonts.SystemFontsManager.GetAvailableFonts())
        //    {
        //        FontProperties fontProperties = new FontProperties();
        //        fontProperties.FontFamily = new FontFamily(font);
        //        fontProperties.FontSize = 12;


        //        block.FontFamily = fontProperties.FontFamily;
        //        block.FontSize = fontProperties.FontSize;

        //        block.Text = "T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,";
        //        var blockSize = block.GetSize();
        //        var radBlockSize = RadTextMeasurer.Measure(block.Text, fontProperties).Size;

        //        Assert.IsTrue(AreClose(blockSize.Width, radBlockSize.Width) &&
        //                        AreClose(blockSize.Height, radBlockSize.Height));
        //    }
        //}

        // NOTE: This test is not 100% correct. For example Wingdings 2 has two symbols that have greater width than the other.
        //[TestMethod]
        //public void TestFontPropertiesCorrectnes()
        //{
        //    TextBlock block = new TextBlock();
        //    foreach (var font in FontManager.GetAvailableFonts())
        //    {
        //        FontProperties fontProperties = new FontProperties();
        //        fontProperties.FontFamily = new FontFamily(font);
        //        fontProperties.FontSize = 12;


        //        block.FontFamily = fontProperties.FontFamily;
        //        block.FontSize = fontProperties.FontSize;

        //        block.Text = "iii";
        //        var firstSize = block.GetSize();

        //        block.Text = "WWW";
        //        var secondSize = block.GetSize();

        //        if (FontManager.IsMonospaced(font) !=
        //                AreClose(firstSize.Width, secondSize.Width))
        //        {
        //            Debug.WriteLine(font);
        //        }
        //        else
        //        {
        //            Assert.AreEqual(FontManager.IsMonospaced(font),
        //                    AreClose(firstSize.Width, secondSize.Width));
        //        }
        //    }
        //}

        #endregion
    }
}
