﻿using System;
using System.IO;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Media;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Shapes;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;
using Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Shapes
{
    [TestClass]
    public class ShapeTests
    {
        private static readonly CellIndex standardCaseIndex = new CellIndex(6, 2);
        private static readonly double offsetX = 0.999;
        private static readonly double offsetY = 0.249;
        private static readonly double width = 195;
        private static readonly double height = 100;

        private static readonly CellIndex rotatedStandardCaseIndex = new CellIndex(3, 2);
        private static readonly double rotatedOffsetX = 48.5;
        private static readonly double rotatedOffsetY = 12.75;

        [TestMethod]
        public void AdjustCellIndexToRotation40DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(40, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation45DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(45, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation60DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(60, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation90DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(90, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation120DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(120, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation135DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(135, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation150DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(150, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation200DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(200, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation225DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(225, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation260DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(260, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation300DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(300, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation315DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(315, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation350DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(350, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation400DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(400, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotation600DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(600, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexToRotationNegative90DegreesTest()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(-90, true);

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexRotateTo90DegreesAndBack()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(90, true);

            this.AssertChangedCellIndex(image);

            image.SetRotationAngle(0, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexRotateTo90DegreesUndoRedo()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(90, true);

            this.AssertChangedCellIndex(image);

            worksheet.Workbook.History.Undo();

            this.AssertSameCellIndex(image);

            worksheet.Workbook.History.Redo();

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexRotateTo90CausingToGoOutOfSpreadsheet()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingImage image = new FloatingImage(worksheet, new CellIndex(0,0), 0, 0);
            Stream stream = TestHelperInternal.GetStreamFromFile("Penguins.jpg");
            using (stream)
            {
                image.ImageSource = new ImageSource(stream, "jpg");
            }

            image.Width = 260;
            image.Height = height;

            image.SetRotationAngle(90, true);

            Assert.AreEqual(new CellIndex(0, 1), image.CellIndex);
        }

        [TestMethod]
        public void ChangeRotationAngleWithoutAdjustingIndexProperty()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.RotationAngle = 90;

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void ChangeRotationAngleWithoutAdjustingIndexMethod()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(90, false);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void ChangeRotationAngleWithoutAdjustingIndexThenSetWithAdjusting()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetRotationAngle(90, false);

            this.AssertSameCellIndex(image);

            image.SetRotationAngle(0, true);

            Assert.AreEqual(new CellIndex(8, 1), image.CellIndex);
            Assert.AreEqual(18.5, Math.Round(image.OffsetX, 2));
            Assert.AreEqual(7.75, Math.Round(image.OffsetY, 2));
        }

        [TestMethod]
        public void ShapesWidthCausesItToGoOutOfSpreadsheet()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingImage image = new FloatingImage(worksheet, new CellIndex(SpreadsheetDefaultValues.RowCount - 1, SpreadsheetDefaultValues.ColumnCount - 1), 0, 0);
            Stream stream = TestHelperInternal.GetStreamFromFile("Penguins.jpg");
            using (stream)
            {
                image.ImageSource = new ImageSource(stream, "jpg");
            }

            image.SetWidth(false, 2000, true);
            image.SetHeight(false, 1000, true);

            int columnsOffset = (int)(Math.Ceiling(image.Width / SpreadsheetDefaultValues.DefaultColumnWidth));
            int rowsOffset = (int)(Math.Ceiling(image.Height / SpreadsheetDefaultValues.DefaultRowHeight));

            int expectedColumnCount = SpreadsheetDefaultValues.ColumnCount - columnsOffset;
            int expectedRowCount = SpreadsheetDefaultValues.RowCount - rowsOffset;

            Assert.AreEqual(new CellIndex(expectedRowCount, expectedColumnCount), image.CellIndex);
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged170DegreesRotationGoesOutOfSpreadsheet()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.SetRotationAngle(170, true);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(1, 0), image.CellIndex);
            Assert.AreEqual(0, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(18, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged270DegreesRotationGoesOutOfSpreadsheet()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.SetRotationAngle(270, true);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(0, 2), image.CellIndex);
            Assert.AreEqual(48, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(0, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChangedZeroDegreesRotation()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            this.AssertSameCellIndex(image);
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged40DegreesRotation()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.RotationAngle = 40;

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(8, 1), image.CellIndex);
            Assert.AreEqual(11, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(11, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged90DegreesRotation()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.SetRotationAngle(90, true);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(3, 1), image.CellIndex);
            Assert.AreEqual(13, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(13, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged170DegreesRotation()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet, new CellIndex(10, 10));
            image.SetRotationAngle(170, true);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(5, 6), image.CellIndex);
            Assert.AreEqual(59, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(18, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged270DegreesRotation()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet, new CellIndex(15, 10));
            image.SetRotationAngle(270, true);

            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(2, 10), image.CellIndex);
            Assert.AreEqual(48, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(18, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void AdjustCellIndexSizeChanged90DegreesRotationUndoRedo()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.SetRotationAngle(90, true);

            this.AssertChangedCellIndex(image);

            worksheet.BeginUndoGroup();
            image.SetWidth(false, image.Width * 2, true);
            image.SetHeight(false, image.Height * 2, true);
            worksheet.EndUndoGroup();

            Assert.AreEqual(width * 2, image.Width);
            Assert.AreEqual(height * 2, image.Height);
            Assert.AreEqual(new CellIndex(3, 1), image.CellIndex);
            Assert.AreEqual(13, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(13, Math.Round(image.OffsetY, 0));

            worksheet.Workbook.History.Undo();

            this.AssertChangedCellIndex(image);
        }

        [TestMethod]
        public void LockAspectRatioDefaultIsTrue()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingImage image = new FloatingImage(worksheet, standardCaseIndex, offsetX, offsetY);

            Assert.IsTrue(image.LockAspectRatio);
        }

        [TestMethod]
        public void LockAspectRatioSetWidth()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetWidth(false, image.Width / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioSetWidthThroughtMethodTrue()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetWidth(true, image.Width / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioSetWidthThroughtMethodFalse()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetWidth(false, image.Width / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioSetHeight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetHeight(false, image.Height / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioSetHeightThroughMethodTrue()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetHeight(true, image.Height / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioSetHeightThroughMethodFalse()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;

            image.SetHeight(false, image.Height / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
        }

        [TestMethod]
        public void LockAspectRatioWithRotationSetWidth()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;
            image.RotationAngle = 44;

            image.SetWidth(true, image.Width / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
            Assert.AreEqual(new CellIndex(4, 2), image.CellIndex);
            Assert.AreEqual(32, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(13, Math.Round(image.OffsetY, 0));
        }

        [TestMethod]
        public void LockAspectRatioWithRotationSetHeight()
        {
            Worksheet worksheet = TestHelper.CreateWorksheet();
            FloatingShapeBase image = this.CreateStandardImage(worksheet);
            image.LockAspectRatio = true;
            image.RotationAngle = 44;

            image.SetHeight(true, image.Height / 2, true);

            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(width / 2, image.Width));
            Assert.IsTrue(TestHelper.AreDoubleValuesEssentiallyEqual(height / 2, image.Height));
            Assert.AreEqual(new CellIndex(4, 2), image.CellIndex);
            Assert.AreEqual(32, Math.Round(image.OffsetX, 0));
            Assert.AreEqual(13, Math.Round(image.OffsetY, 0));
        }

        private FloatingImage CreateStandardImage(Worksheet worksheet, CellIndex index = null)
        {
            if (index == null)
            {
                index = standardCaseIndex;
            }

            FloatingImage image = new FloatingImage(worksheet, index, offsetX, offsetY);
            Stream stream = TestHelperInternal.GetStreamFromFile("Penguins.jpg");
            using (stream)
            {
                image.ImageSource = new ImageSource(stream, "jpg");
            }

            image.LockAspectRatio = false;

            image.Width = width;
            image.Height = height;

            return image;
        }

        private void AssertSameCellIndex(FloatingShapeBase shape)
        {
            Assert.AreEqual(new CellIndex(6, 2), shape.CellIndex);
            Assert.AreEqual(1, Math.Round(shape.OffsetX, 2));
            Assert.AreEqual(0.25, Math.Round(shape.OffsetY, 2));
        }

        private void AssertChangedCellIndex(FloatingShapeBase shape)
        {
            Assert.AreEqual(rotatedStandardCaseIndex, shape.CellIndex);
            Assert.AreEqual(rotatedOffsetX, Math.Round(shape.OffsetX, 2));
            Assert.AreEqual(rotatedOffsetY, Math.Round(shape.OffsetY, 2));
        }
    }
}