﻿using System;
using System.Linq;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.SheetViewportTests
{
    internal static class SheetViewportTestsHelper
    {
        public static readonly Size ViewportSize = new Size(1200, 700);

        private static void SetViewport(SheetViewportTestsArgs args, double horizontalOffset, double verticalOffset)
        {
            Point offset = new Point(horizontalOffset, verticalOffset);
            args.SheetViewport = SheetViewportFactory.CreateSheetViewport(
                args.RadWorksheetLayout,
                args.SheetViewport,
                args.ScaleFactor,
                ViewportSize,
                SpreadsheetDefaultValues.VisibleSize,
                args.FrozenCellIndex,
                offset,
                true, true);
        }

        public static void SetViewportTopLeftCellIndex(SheetViewportTestsArgs args, CellIndex topLeftCellIndex)
        {
            args.RadWorksheetLayout.Measure(args.FrozenCellIndex);
            CellLayoutBox topLeftCellBox = args.RadWorksheetLayout.GetCellLayoutBox(topLeftCellIndex);
            Rect topLeftCellRect = topLeftCellBox.BoundingRectangle;

            SetViewport(args, topLeftCellRect.X, topLeftCellRect.Y);
        }

        public static void FreezePanes(SheetViewportTestsArgs args)
        {
            SheetViewport newViewport;
            if (SheetViewportFactory.GetFrozenViewport(args.RadWorksheetLayout, args.SheetViewport, SpreadsheetDefaultValues.VisibleSize, args.FrozenCellIndex, out newViewport))
            {
                args.SheetViewport = newViewport;
            }

            args.RadWorksheetLayout.Measure(args.FrozenCellIndex);
        }

        public static void InitializeViewport(SheetViewportTestsArgs args)
        {
            args.ScaleFactor = new Size(1, 1);
            args.FrozenCellIndex = new CellIndex(0, 0);
            args.RadWorksheetLayout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            args.RadWorksheetLayout.SetWorksheet(TestHelper.CreateWorksheet(), false);
            args.RadWorksheetLayout.Measure();
            args.SheetViewport = new SheetViewport();
            SetViewport(args, 0, 0);
        }
    }
}