﻿using System;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Layout;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.SheetViewportTests
{
    [TestClass]
    public class SheetViewport_GetViewportPaneFromDocumentPoint_Should
    {
        #region Constants

        private static readonly CellIndex FrozenCellIndex = new CellIndex(10, 10);

        #endregion


        #region Properties

        private SheetViewportTestsArgs Args { get; set; }

        private SheetViewport SheetViewport
        {
            get
            {
                return this.Args.SheetViewport;
            }
        }

        #endregion


        #region Methods

        [TestInitialize]
        public void InitializeTest()
        {
            RadWorksheetLayout radWorksheetLayout = new RadWorksheetLayout(new SizeI(SpreadsheetDefaultValues.ColumnCount, SpreadsheetDefaultValues.RowCount));
            radWorksheetLayout.SetWorksheet(TestHelper.CreateWorksheet(), false);
            radWorksheetLayout.Measure();

            this.Args = new SheetViewportTestsArgs(new SheetViewport(), radWorksheetLayout, new Size(1, 1), new CellIndex(0, 0));
            SheetViewportTestsHelper.InitializeViewport(this.Args);

            this.Args.FrozenCellIndex = FrozenCellIndex;
            SheetViewportTestsHelper.FreezePanes(this.Args);
        }

        #endregion


        #region Fixed pane tests

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopLeftFixedPoint()
        {
            Point point = new Point(0, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopRightFixedPoint()
        {
            Point point = new Point(649, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomRightFixedPoint()
        {
            Point point = new Point(649, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomLeftFixedPoint()
        {
            Point point = new Point(0, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopFixedEdgePoint()
        {
            Point point = new Point(100, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheRightFixedEdgePoint()
        {
            Point point = new Point(649, 100);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomFixedEdgePoint()
        {
            Point point = new Point(100, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheLeftFixedEdgePoint()
        {
            Point point = new Point(0, 100);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Fixed, result);
        }

        #endregion


        #region Horizontal scrollable pane tests

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopLeftHorizontalScrollablePoint()
        {
            Point point = new Point(650, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopRightHorizontalScrollablePoint()
        {
            Point point = new Point(1199, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomRightHorizontalScrollablePoint()
        {
            Point point = new Point(1199, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomLeftHorizontalScrollablePoint()
        {
            Point point = new Point(650, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopHorizontalScrollableEdgePoint()
        {
            Point point = new Point(800, 0);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheRightHorizontalScrollableEdgePoint()
        {
            Point point = new Point(1199, 100);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomHorizontalScrollableEdgePoint()
        {
            Point point = new Point(800, 199);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheLeftHorizontalScrollableEdgePoint()
        {
            Point point = new Point(650, 100);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.HorizontalScrollable, result);
        }

        #endregion


        #region Vertical scrollable pane tests

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopLeftVerticalScrollablePoint()
        {
            Point point = new Point(0, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopRightVerticalScrollablePoint()
        {
            Point point = new Point(649, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomRightVerticalScrollablePoint()
        {
            Point point = new Point(649, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomLeftVerticalScrollablePoint()
        {
            Point point = new Point(0, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopVerticalScrollableEdgePoint()
        {
            Point point = new Point(100, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheRightVerticalScrollableEdgePoint()
        {
            Point point = new Point(649, 300);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomVerticalScrollableEdgePoint()
        {
            Point point = new Point(100, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheLeftVerticalScrollableEdgePoint()
        {
            Point point = new Point(0, 300);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.VerticalScrollable, result);
        }

        #endregion


        #region  Scrollable pane tests

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopLeftScrollablePoint()
        {
            Point point = new Point(650, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopRightScrollablePoint()
        {
            Point point = new Point(1199, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomRightScrollablePoint()
        {
            Point point = new Point(1199, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomLeftScrollablePoint()
        {
            Point point = new Point(650, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheTopScrollableEdgePoint()
        {
            Point point = new Point(800, 200);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheRightScrollableEdgePoint()
        {
            Point point = new Point(1199, 300);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheBottomScrollableEdgePoint()
        {
            Point point = new Point(800, 699);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        [TestMethod]
        public void GetViewportPaneFromDocumentPoint_WhenPointIsTheLeftScrollableEdgePoint()
        {
            Point point = new Point(650, 300);
            ViewportPaneType result = this.SheetViewport.GetViewportPaneFromDocumentPoint(point).ViewportPaneType;

            Assert.AreEqual(ViewportPaneType.Scrollable, result);
        }

        #endregion
    }
}