﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Expressions;
using Telerik.Windows.Documents.Spreadsheet.Expressions.Functions;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Expressions
{
    [TestClass]
    public class TextFunctionsTests
    {
        [TestMethod]
        public void CharFunctionArgumentConversionTest()
        {
            CharFunction charFunction = new CharFunction();

            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)1).ToString(), true);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)1).ToString(), "F1");
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)10).ToString(), 10);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)10).ToString(), "10");
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)10).ToString(), "A1");
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)20).ToString(), 20);
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)20).ToString(), "20");
            FunctionsTestHelper.AssertFunctionResult(charFunction, ((char)20).ToString(), "B1");
        }

        [TestMethod]
        public void CharFunctionErrorTest()
        {
            CharFunction charFunction = new CharFunction();

            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.ValueError, 256);
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.ValueError, -1);
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.ValueError, 0);
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.ValueError, false);
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.ValueError, "C1");
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(charFunction, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void CodeFunctionCorrectnessTest()
        {
            Code code = new Code();

            FunctionsTestHelper.AssertFunctionResult(code, (int)'2', "200");
            FunctionsTestHelper.AssertFunctionResult(code, (int)'2', "B1");
            FunctionsTestHelper.AssertFunctionResult(code, (int)'1', 1);
            FunctionsTestHelper.AssertFunctionResult(code, (int)'1', "A1");
            FunctionsTestHelper.AssertFunctionResult(code, (int)'s', "string");
            FunctionsTestHelper.AssertFunctionResult(code, (int)'s', "C1");
            FunctionsTestHelper.AssertFunctionResult(code, (int)'T', true);
            FunctionsTestHelper.AssertFunctionResult(code, (int)'F', false);
            FunctionsTestHelper.AssertFunctionResult(code, (int)'T', "F1");
            FunctionsTestHelper.AssertFunctionResult(code, (int)' ', " ");
            FunctionsTestHelper.AssertFunctionResult(code, (int)' ', " abc");
        }

        [TestMethod]
        public void CodeFunctionErrorTest()
        {
            Code code = new Code();

            FunctionsTestHelper.AssertFunctionError(code, ErrorExpressions.ValueError, "C3");
            FunctionsTestHelper.AssertFunctionError(code, ErrorExpressions.ValueError, "");
            FunctionsTestHelper.AssertFunctionError(code, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(code, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void ConcatenateFunctionCorrectnessTest()
        {
            Concatenate concatenate = new Concatenate();

            FunctionsTestHelper.AssertFunctionResult(concatenate, "123123", "123", "123");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "123abc", "123", "abc");

            FunctionsTestHelper.AssertFunctionResult(concatenate, "abcdef", "abc", "def");
            FunctionsTestHelper.AssertFunctionResult(concatenate, @"abc""def", @"abc""", @"def");
            FunctionsTestHelper.AssertFunctionResult(concatenate, @"""abc""def", @"""abc", @"""def");
            FunctionsTestHelper.AssertFunctionResult(concatenate, @"""""", @"""", @"""");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "abc", "abc", "");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "abc", "", "abc");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "", "", "");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "-20", "-", "20");

            FunctionsTestHelper.AssertFunctionResult(concatenate, "qwertyuio", "qwe", "rty", "uio");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "stringstring", "C1", "C1");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "10string", "A1", "C1");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "string20", "C1", "B1");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "TRUE", "F1");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "TRUE10", "F1", "A1");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "", "C3", "C3");
        }

        [TestMethod]
        public void ConcatenateFunctionArgumentConversionTest()
        {
            Concatenate concatenate = new Concatenate();

            FunctionsTestHelper.AssertFunctionResult(concatenate, "", EmptyExpression.Empty, EmptyExpression.Empty);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "123abc", 123, "abc");
            FunctionsTestHelper.AssertFunctionResult(concatenate, "123TRUE", 123, true);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "FALSE", false);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "TRUE", true);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "TRUEFALSE", true, false);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "TRUE1FALSE", true, 1, false);
            FunctionsTestHelper.AssertFunctionResult(concatenate, "123", 123, "C3");
        }

        [TestMethod]
        public void ConcatenateFunctionErrorTest()
        {
            Concatenate concatenate = new Concatenate();

            FunctionsTestHelper.AssertFunctionError(concatenate, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(concatenate, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void ExactFunctionArgumentsConversionTest()
        {
            Exact exact = new Exact();

            FunctionsTestHelper.AssertFunctionResult(exact, true, "C1", "string");
            FunctionsTestHelper.AssertFunctionResult(exact, false, "A1", "B1");
            FunctionsTestHelper.AssertFunctionResult(exact, true, "A1", 10);
            FunctionsTestHelper.AssertFunctionResult(exact, true, true, true);
            FunctionsTestHelper.AssertFunctionResult(exact, true, "F1", "F1");
            FunctionsTestHelper.AssertFunctionResult(exact, true, false, false);
            FunctionsTestHelper.AssertFunctionResult(exact, true, "C3", "C3");
        }

        [TestMethod]
        public void ExactFunctionErrorTest()
        {
            Exact exact = new Exact();

            FunctionsTestHelper.AssertFunctionError(exact, ErrorExpressions.DivisionByZero, "", "D1");
            FunctionsTestHelper.AssertFunctionError(exact, ErrorExpressions.NumberError, "E1", "");
        }

        [TestMethod]
        public void FindFunctionArgumentConversionTest()
        {
            Find find = new Find();

            FunctionsTestHelper.AssertFunctionResult(find, 4, "i", "C1");
            FunctionsTestHelper.AssertFunctionResult(find, 2, "0", 10);
            FunctionsTestHelper.AssertFunctionResult(find, 2, "0", "A1");
            FunctionsTestHelper.AssertFunctionResult(find, 2, "0", 20);
            FunctionsTestHelper.AssertFunctionResult(find, 2, "0", "B1");
            FunctionsTestHelper.AssertFunctionResult(find, 3, "U", true);
            FunctionsTestHelper.AssertFunctionResult(find, 3, "U", "F1");
            FunctionsTestHelper.AssertFunctionResult(find, 5, "E", false);
        }

        [TestMethod]
        public void FindFunctionErrorTest()
        {
            Find find = new Find();

            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "", "", 10);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "M", "Miriam McGovern", 9);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "M", "Miriam McGovern", 0);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "M", "Miriam McGovern", -1);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "M", "Miriam McGovern", 16);

            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "a", "");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "tyu", "qwerty");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "P", true);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "r", false);
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.ValueError, "n", "F1");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.DivisionByZero, "D1", "A1");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.NumberError, "E1", "A1");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.DivisionByZero, "B1", "D1");
            FunctionsTestHelper.AssertFunctionError(find, ErrorExpressions.NumberError, "B1", "E1");
        }

        [TestMethod]
        public void LeftFunctionArgumentConversionTest()
        {
            Left left = new Left();

            FunctionsTestHelper.AssertFunctionResult(left, "T", true);
            FunctionsTestHelper.AssertFunctionResult(left, "T", "F1");
            FunctionsTestHelper.AssertFunctionResult(left, "F", false);
            FunctionsTestHelper.AssertFunctionResult(left, "1", 10);
            FunctionsTestHelper.AssertFunctionResult(left, "1", "A1");
            FunctionsTestHelper.AssertFunctionResult(left, "2", 20);
            FunctionsTestHelper.AssertFunctionResult(left, "2", "B1");
            FunctionsTestHelper.AssertFunctionResult(left, "s", "C1");
            FunctionsTestHelper.AssertFunctionResult(left, "strin", "C1", 5);
            FunctionsTestHelper.AssertFunctionResult(left, "TR", true, 2);
            FunctionsTestHelper.AssertFunctionResult(left, "TR", "F1", 2);
            FunctionsTestHelper.AssertFunctionResult(left, "FAL", false, 3);
            FunctionsTestHelper.AssertFunctionResult(left, "1", 10, 1);
            FunctionsTestHelper.AssertFunctionResult(left, "1", "A1", 1);
            FunctionsTestHelper.AssertFunctionResult(left, "20", 20, 3);
            FunctionsTestHelper.AssertFunctionResult(left, "20", "B1", 3);
        }

        [TestMethod]
        public void LeftFunctionErrorTest()
        {
            Left left = new Left();

            FunctionsTestHelper.AssertFunctionError(left, ErrorExpressions.ValueError, "qwerty", -1);
            FunctionsTestHelper.AssertFunctionError(left, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(left, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(left, ErrorExpressions.DivisionByZero, "", "D1");
            FunctionsTestHelper.AssertFunctionError(left, ErrorExpressions.NumberError, "", "E1");
        }

        [TestMethod]
        public void LenFunctionCorrectnessTest()
        {
            Len len = new Len();

            FunctionsTestHelper.AssertFunctionResult(len, 0, "");
            FunctionsTestHelper.AssertFunctionResult(len, 6, "string");
            FunctionsTestHelper.AssertFunctionResult(len, 6, "C1");
        }

        [TestMethod]
        public void LenFunctionArgumentConversionTest()
        {
            Len len = new Len();

            FunctionsTestHelper.AssertFunctionResult(len, 4, true);
            FunctionsTestHelper.AssertFunctionResult(len, 4, "F1");
            FunctionsTestHelper.AssertFunctionResult(len, 5, false);
            FunctionsTestHelper.AssertFunctionResult(len, 2, 10);
            FunctionsTestHelper.AssertFunctionResult(len, 2, "A1");
            FunctionsTestHelper.AssertFunctionResult(len, 2, "10");
            FunctionsTestHelper.AssertFunctionResult(len, 2, "B1");
            FunctionsTestHelper.AssertFunctionResult(len, 0, "C3");
        }

        [TestMethod]
        public void LenFunctionErrorTest()
        {
            Len len = new Len();

            FunctionsTestHelper.AssertFunctionError(len, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(len, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void LowerFunctionArgumentConversionTest()
        {
            Lower lower = new Lower();

            FunctionsTestHelper.AssertFunctionResult(lower, "true", true);
            FunctionsTestHelper.AssertFunctionResult(lower, "false", false);
            FunctionsTestHelper.AssertFunctionResult(lower, "true", "F1");
            FunctionsTestHelper.AssertFunctionResult(lower, "10", 10);
            FunctionsTestHelper.AssertFunctionResult(lower, "20", 20);
            FunctionsTestHelper.AssertFunctionResult(lower, "10", "A1");
            FunctionsTestHelper.AssertFunctionResult(lower, "20", "B1");
            FunctionsTestHelper.AssertFunctionResult(lower, "string", "C1");
            FunctionsTestHelper.AssertFunctionResult(lower, "", "C3");
        }

        [TestMethod]
        public void LowerFunctionErrorTest()
        {
            Lower lower = new Lower();

            FunctionsTestHelper.AssertFunctionError(lower, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(lower, ErrorExpressions.NumberError, "E1");
        }

        [TestMethod]
        public void ReptFunctionCorrectnessTest()
        {
            Rept rept = new Rept();

            FunctionsTestHelper.AssertFunctionResult(rept, "101010", "A1", 3);
            FunctionsTestHelper.AssertFunctionResult(rept, "202020", "B1", 3);
        }

        [TestMethod]
        public void ReptFunctionErrorTest()
        {
            Rept rept = new Rept();

            FunctionsTestHelper.AssertFunctionError(rept, ErrorExpressions.ValueError, "a", -1);
            FunctionsTestHelper.AssertFunctionError(rept, ErrorExpressions.ValueError, "a", -100);
            FunctionsTestHelper.AssertFunctionError(rept, ErrorExpressions.ValueError, "a", 32768);
            FunctionsTestHelper.AssertFunctionError(rept, ErrorExpressions.DivisionByZero, "D1", 2);
            FunctionsTestHelper.AssertFunctionError(rept, ErrorExpressions.NumberError, "", "E1");
        }

        [TestMethod]
        public void ReptFunctionArgumentConversionTest()
        {
            Rept rept = new Rept();

            FunctionsTestHelper.AssertFunctionResult(rept, "20202020202020202020", "B1", "A1");
            FunctionsTestHelper.AssertFunctionResult(rept, "stringstring", "C1", 2);
            FunctionsTestHelper.AssertFunctionResult(rept, "TRUETRUETRUE", true, 3);
            FunctionsTestHelper.AssertFunctionResult(rept, "TRUETRUETRUE", "F1", 3);
            FunctionsTestHelper.AssertFunctionResult(rept, "FALSEFALSEFALSE", false, 3);
            FunctionsTestHelper.AssertFunctionResult(rept, "1111111111", 1, "A1");
            FunctionsTestHelper.AssertFunctionResult(rept, "11111111111111111111", 1, "B1");
            FunctionsTestHelper.AssertFunctionResult(rept, "1", 1, true);
            FunctionsTestHelper.AssertFunctionResult(rept, "1", 1, "F1");
        }

        [TestMethod]
        public void RightFunctionCorrectnessTest()
        {
            Right right = new Right();

            FunctionsTestHelper.AssertFunctionResult(right, "g", "C1");
            FunctionsTestHelper.AssertFunctionResult(right, "tring", "C1", 5);
        }

        [TestMethod]
        public void RightFunctionArgumentConversionTest()
        {
            Right right = new Right();

            FunctionsTestHelper.AssertFunctionResult(right, "E", true);
            FunctionsTestHelper.AssertFunctionResult(right, "E", "F1");
            FunctionsTestHelper.AssertFunctionResult(right, "E", false);
            FunctionsTestHelper.AssertFunctionResult(right, "0", 10);
            FunctionsTestHelper.AssertFunctionResult(right, "0", "A1");
            FunctionsTestHelper.AssertFunctionResult(right, "0", 20);
            FunctionsTestHelper.AssertFunctionResult(right, "0", "B1");

            FunctionsTestHelper.AssertFunctionResult(right, "UE", true, 2);
            FunctionsTestHelper.AssertFunctionResult(right, "UE", "F1", 2);
            FunctionsTestHelper.AssertFunctionResult(right, "LSE", false, 3);
            FunctionsTestHelper.AssertFunctionResult(right, "0", 10, 1);
            FunctionsTestHelper.AssertFunctionResult(right, "0", "A1", 1);
            FunctionsTestHelper.AssertFunctionResult(right, "20", 20, 3);
            FunctionsTestHelper.AssertFunctionResult(right, "20", "B1", 3);
        }

        [TestMethod]
        public void RightFunctionErrorTest()
        {
            Right right = new Right();

            FunctionsTestHelper.AssertFunctionError(right, ErrorExpressions.ValueError, "qwerty", -1);
            FunctionsTestHelper.AssertFunctionError(right, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(right, ErrorExpressions.NumberError, "E1");
            FunctionsTestHelper.AssertFunctionError(right, ErrorExpressions.DivisionByZero, "", "D1");
            FunctionsTestHelper.AssertFunctionError(right, ErrorExpressions.NumberError, "", "E1");
        }

        [TestMethod]
        public void SearchFunctionArgumentConversionTest()
        {
            Search search = new Search();

            FunctionsTestHelper.AssertFunctionResult(search, 4, "i", "C1");
            FunctionsTestHelper.AssertFunctionResult(search, 2, "0", 10);
            FunctionsTestHelper.AssertFunctionResult(search, 2, "0", "A1");
            FunctionsTestHelper.AssertFunctionResult(search, 2, "0", 20);
            FunctionsTestHelper.AssertFunctionResult(search, 2, "0", "B1");
            FunctionsTestHelper.AssertFunctionResult(search, 3, "U", true);
            FunctionsTestHelper.AssertFunctionResult(search, 3, "U", "F1");
            FunctionsTestHelper.AssertFunctionResult(search, 5, "E", false);
        }

        [TestMethod]
        public void SearchFunctionErrorTest()
        {
            Search search = new Search();

            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "a", "");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "tyu", "qwerty");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "P", true);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "r", false);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "n", "F1");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.DivisionByZero, "D1", "A1");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.NumberError, "E1", "A1");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.DivisionByZero, "B1", "D1");
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.NumberError, "B1", "E1");

            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "", "", 10);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "M", "Miriam McGovern", 9);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "M", "Miriam McGovern", 0);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "M", "Miriam McGovern", -1);
            FunctionsTestHelper.AssertFunctionError(search, ErrorExpressions.ValueError, "M", "Miriam McGovern", 16);
        }

        [TestMethod]
        public void UpperFunctionArgumentsConversionTest()
        {
            Upper upper = new Upper();

            FunctionsTestHelper.AssertFunctionResult(upper, "TRUE", true);
            FunctionsTestHelper.AssertFunctionResult(upper, "FALSE", false);
            FunctionsTestHelper.AssertFunctionResult(upper, "TRUE", "F1");
            FunctionsTestHelper.AssertFunctionResult(upper, "10", 10);
            FunctionsTestHelper.AssertFunctionResult(upper, "20", 20);
            FunctionsTestHelper.AssertFunctionResult(upper, "10", "A1");
            FunctionsTestHelper.AssertFunctionResult(upper, "20", "B1");
            FunctionsTestHelper.AssertFunctionResult(upper, "STRING", "C1");
            FunctionsTestHelper.AssertFunctionResult(upper, "", "C3");
        }

        [TestMethod]
        public void UpperFunctionErrorTest()
        {
            Upper upper = new Upper();

            FunctionsTestHelper.AssertFunctionError(upper, ErrorExpressions.DivisionByZero, "D1");
            FunctionsTestHelper.AssertFunctionError(upper, ErrorExpressions.NumberError, "E1");
        }
    }
}
