﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.Core.DataStructures;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Model.Filtering;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities
{
    internal static class FilterTestHelperInternal
    {
        public static readonly CellRange DefaultCellRange = new CellRange(5, 5, 10, 10);
        public static readonly List<string> StringValues = new List<string>() {"1", "1.2", "2", "test", "2:24:00 PM"};
        public static readonly List<DateGroupItem> DateValues = new List<DateGroupItem>() 
        {
            new DateGroupItem(1986, 4, 2),
            new DateGroupItem(2013)
        };
        
        public static void AssertAllRowsAreUnhidden(Worksheet worksheet, CellRange range)
        {
            for (int row = range.FromIndex.RowIndex; row <= range.ToIndex.RowIndex; row++)
            {
                bool hidden = worksheet.Rows[row].GetHidden().Value;
                Assert.IsFalse(hidden);
            }
        }

        public static void AssertCorrectRowsHidden(Worksheet worksheet, Dictionary<int, bool> expectedRows)
        {
            foreach (var pair in expectedRows)
            {
                int row = pair.Key;
                bool expectedHidden = pair.Value;
                bool hidden = worksheet.Rows[row].GetHidden().Value;

                Assert.AreEqual(expectedHidden, hidden);
            }
        }

        public static void SetValuesToWorksheetColumn(Worksheet worksheet, int startRow, int startColumn, List<object> values)
        {
            FilterTestHelper.SetValuesToWorksheetColumn(worksheet, startRow, startColumn, values);
        }   

        public static int CountRowsFromRanges(ICompressedList<bool> hiddenRows)
        {
            IEnumerable<Range<long, bool>> hiddenRanges = hiddenRows.GetNonDefaultRanges();
            int rowCount = 0;

            foreach (Range<long, bool> range in hiddenRanges)
            {
                int start = (int)range.Start;
                int end = (int)range.End;

                rowCount += end - start + 1;
            }

            return rowCount;
        }
    }
}
