﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows;
using Telerik.Windows.Documents.Spreadsheet.Formatting;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Tests.Utilities;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal.Utilities
{
    public static class TestHelperInternal
    {
        #region Constants

        private const string TestResourcesFolderName = "TestDataResources";

        #endregion


        #region Helper Methods

        public static Stream GetStreamFromFile(string fileName)
        {
            return TestHelper.GetStreamFromFile(fileName, typeof(TestHelperInternal).Assembly.FullName);
        }

        public static Stream GetResourceStream(string resource)
        {
            return TestHelper.GetResourceStream(resource, typeof(TestHelperInternal).Assembly.FullName);
        }

        public static Workbook GetWorkbookFromFile(string fileName)
        {
            return TestHelper.GetWorkbookFromFile(fileName, typeof(TestHelperInternal).Assembly.FullName);
        }

        public static string ReplaceGroupDecimalSeparators(this string value)
        {
            return CultureStringHelperInternal.Replace(value);
        }

        public static string ReplaceListSeparator(this string value)
        {
            return CultureStringHelperInternal.Replace(value);
        }

        public static string ReplaceArraySeparators(this string value)
        {
            return CultureStringHelperInternal.Replace(value);
        }

        public static string ToDefaultSpreadsheetCultureString(this double value)
        {
            return value.ToString(FormatHelper.DefaultSpreadsheetCulture.CultureInfo);
        }

        public static Rect Translate(this Rect rect, Point delta)
        {
            return new Rect(rect.Left + delta.X, rect.Top + delta.Y, rect.Width, rect.Height);
        }

        public static Rect Translate(this Rect rect, double deltaX, double deltaY)
        {
            return new Rect(rect.Left + deltaX, rect.Top + deltaY, rect.Width, rect.Height);
        }

        public static Rect Scale(this Rect rect, double scaleFactor)
        {
            return new Rect(rect.Left * scaleFactor, rect.Top * scaleFactor, rect.Width * scaleFactor, rect.Height * scaleFactor);
        }

        public static Point Translate(this Point point, Point delta)
        {
            return new Point(point.X + delta.X, point.Y + delta.Y);
        }

        public static Point Translate(this Point point, double deltaX, double deltaY)
        {
            return new Point(point.X + deltaX, point.Y + deltaY);
        }

        public static Point Scale(this Point point, double scaleFactor)
        {
            return new Point(point.X * scaleFactor, point.Y * scaleFactor);
        }

        #endregion


        #region Inner Class

        /// <summary>
        /// This class differs from the CultureStringHelper in the Spreadsheet_Tests project as it uses FormatHelper.DefaultSpreadsheetCulture property, 
        /// which is heavily used in the multi-cultural tests. Note that the public equivalent of the class uses CurrentCulture to produce culture-specific strings.
        /// </summary>
        private static class CultureStringHelperInternal
        {
            private static readonly Dictionary<string, Func<string>> mapping;

            static CultureStringHelperInternal()
            {
                mapping = new Dictionary<string, Func<string>>();
                RegisterMapping("{gs}", () => { return FormatHelper.DefaultSpreadsheetCulture.NumberGroupSeparator; });
                RegisterMapping("{ds}", () => { return FormatHelper.DefaultSpreadsheetCulture.NumberDecimalSeparator; });
                RegisterMapping("{ls}", () => { return FormatHelper.DefaultSpreadsheetCulture.ListSeparator; });
                RegisterMapping("{als}", () => { return FormatHelper.DefaultSpreadsheetCulture.ArrayListSeparator; });
                RegisterMapping("{ars}", () => { return FormatHelper.DefaultSpreadsheetCulture.ArrayRowSeparator; });
            }

            private static void RegisterMapping(string mappingId, Func<string> mappingFunc)
            {
                mapping.Add(mappingId, mappingFunc);
            }

            public static string Replace(string value)
            {
                StringBuilder builder = new StringBuilder(value);

                foreach (string key in mapping.Keys)
                {
                    builder = builder.Replace(key, mapping[key]());
                }

                return builder.ToString();
            }
        }

        #endregion
    }
}