﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Windows.Documents.Spreadsheet.TestsInternal
{
    [TestClass]
    public class WorkbookSheetCollectionTests
    {
        [TestMethod]
        public void ValidSheetNameTests()
        {
            string[] validNames = { "Pesho", "Pesho's", "-+=&@#%^()!.<>~;  __", " ", "1111122222333334444455555666667", "_Sheet_Ябадабадуу64_", "=EqualSign" };

            foreach (string validName in validNames)
            {
                Assert.IsTrue(SheetCollection.IsValidSheetName(validName));
            }
        }

        [TestMethod]
        public void InvalidSheetNameTests()
        {
            string[] invalidNames = { "Pesho'", "'Pesho'", "11111222223333344444555556666677", null, "", 
                                        @"P/esho", @"P\esho", @"P/\esho", @"P?esho", @"P*esho", @"P:esho", @"P[esho", @"P]esho", @"'P/\?*:[]esho"};

            foreach (string invalidName in invalidNames)
            {
                Assert.IsFalse(SheetCollection.IsValidSheetName(invalidName));
            }
        }

        [TestMethod]
        public void RepeatedSheetNameTests()
        {
            Workbook workbook = new Workbook();
            workbook.Worksheets.Add();
            workbook.Worksheets.Add().Name = "Иван Иванов";

            string[] repeatedNames = { "ShEeT1", "иВан иВаноВ" };

            foreach (string repeatedName in repeatedNames)
            {
                Assert.IsFalse(workbook.Sheets.IsUniqueSheetName(repeatedName));
            }
        }
    }
}
