﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.Model.ContentTypes;

namespace Telerik.Documents.SpreadsheetStreaming
{
    internal class ContentTypesRepository
    {
        private readonly Dictionary<string, DefaultContentType> defaults;
        private readonly HashSet<OverrideContentType> overrides;

        public ContentTypesRepository()
        {
            this.defaults = new Dictionary<string, DefaultContentType>();
            this.overrides = new HashSet<OverrideContentType>();
            this.RegisterDefaultContentType("xml");
        }

        public IEnumerable<DefaultContentType> Defaults
        {
            get
            {
                return this.defaults.Values;
            }
        }

        public IEnumerable<OverrideContentType> Overrides
        {
            get
            {
                return this.overrides;
            }
        }

        public void Register(PartBase part)
        {
            this.Register(part.GetPartFullPath(), part.ContentType);
        }

        public void Register(DefaultContentType value)
        {
            this.defaults[value.Extension] = value;
        }

        public void Register(OverrideContentType value)
        {
            if (!this.overrides.Contains(value))
            {
                this.overrides.Add(value);
            }
        }

        internal void Register(string path, string contentType)
        {
            path = '/' + path;
            this.RegisterOverrideContentType(path, contentType);
            this.RegisterDefaultContentType(path);
        }

        private void RegisterOverrideContentType(string path, string contentType)
        {
            OverrideContentType overrideContentType = new OverrideContentType(path, contentType);
            if (!this.overrides.Contains(overrideContentType))
            {
                this.overrides.Add(overrideContentType);
            }
        }

        private void RegisterDefaultContentType(string path)
        {
            string contentType;
            string extension = OpenXmlHelper.GetExtension(path);
            if (OpenXmlHelper.TryGetContentTypeByExtension(extension, out contentType))
            {
                DefaultContentType defaultContentType = new DefaultContentType(extension, contentType);
                if (!this.defaults.ContainsKey(extension))
                {
                    this.defaults.Add(extension, defaultContentType);
                }
            }
        }
    }
}