﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Telerik.Documents.SpreadsheetStreaming.Core
{
    internal class CsvStreamWriter : StreamWriter
    {
        private readonly bool shouldDisposeStream;

        /// <summary>
        /// Initializes a new instance of the <see cref="CsvStreamWriter" /> class.
        /// </summary>
        /// <param name="stream">The underlying stream to write to.</param>
        /// <param name="encoding">The encoding for the stream.</param>
        public CsvStreamWriter(Stream stream, Encoding encoding)
        : base(stream, encoding)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CsvStreamWriter" /> class.
        /// </summary>
        /// <param name="stream">The stream.</param>
        public CsvStreamWriter(Stream stream)
        : this(stream, Encoding.UTF8, true)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CsvStreamWriter" /> class using default encoding.
        /// </summary>
        /// <param name="stream">The underlying stream to write to.</param>
        /// <param name="encoding">The encoding.</param>
        /// <param name="shouldDisposeStream">Determines whether the passed stream should be disposed when 
        /// <see cref="CsvStreamWriter"/>is disposed.</param> 
        public CsvStreamWriter(Stream stream, Encoding encoding, bool shouldDisposeStream)
            : base(stream, encoding)
        {
            this.shouldDisposeStream = shouldDisposeStream;
        }

        /// <summary>
        /// Disposes of the stream writer.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected override void Dispose(bool disposeManaged)
        {
            // Dispose the stream writer but pass false to the dispose
            // method to stop it from closing the underlying stream
            base.Dispose(this.shouldDisposeStream);
        }
    }
}
