﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Csv
{
    internal class CsvCellExporter : EntityBase, ICellExporter
    {
        private const string Quote = "\"";
        private const string EscapedQoute = "\"\""; 
        private static char[] charactersToEscape = { ',', '"', '\n' };

        private readonly StreamWriter writer;
        private string value;

        internal CsvCellExporter(StreamWriter writer)
        {
            this.writer = writer;
        }

        public void SetValue(string value)
        {
            Guard.ThrowExceptionIfNull(value, "value");

            this.value = value;
        }

        public void SetValue(double value)
        {
            Guard.ThrowExceptionIfNaN(value, "value");
            Guard.ThrowExceptionIfInfinity(value, "value");

            this.SetValue(value.ToString(CultureInfo.InvariantCulture));
        }

        public void SetValue(bool value)
        {
            string stringValue = value.ToString().ToUpperInvariant();
            this.SetValue(stringValue);
        }

        public void SetValue(DateTime value)
        {
            this.SetValue(value.ToString());
        }

        public void SetFormula(string value)
        {
            Guard.ThrowExceptionIfNull(value, "value");

            this.SetValue(value);
        }

        public void SetFormat(SpreadCellFormat cellFormat)
        {
        }

        internal sealed override void CompleteWriteOverride()
        {
            if (this.value == null)
            {
                return;
            }

            string escapedValue = Escape(this.value);
            this.writer.Write(escapedValue);
        }

        private static string Escape(string text)
        {
            if (text.Contains(Quote))
            {
                text = text.Replace(Quote, EscapedQoute);
            }

            if (text.IndexOfAny(charactersToEscape) > -1)
            {
                text = Quote + text + Quote;
            }

            return text;
        }
    }
}