﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Csv
{
    internal class CsvRowExporter : EntityBase, IRowExporter
    {
        private static readonly char valuesSeparator = ',';
        private readonly StreamWriter writer;
        private bool isFirstCellInRow;

        internal CsvRowExporter(StreamWriter writer)
        {
            this.isFirstCellInRow = true;
            this.writer = writer;
        }

        public ICellExporter CreateCellExporter()
        {
            if (!isFirstCellInRow)
            {
                this.Write(valuesSeparator.ToString());
            }

            CsvCellExporter cellExporter = new CsvCellExporter(this.writer);
            this.isFirstCellInRow = false;

            return cellExporter;
        }

        public void SkipCells(int count)
        {
            Guard.ThrowExceptionIfLessThan(0, count, "count");

            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < count; i++)
            {
                builder.Append(valuesSeparator);
            }

            this.Write(builder.ToString());
        }

        public void SetOutlineLevel(int value)
        {
        }

        public void SetHeightInPixels(double value)
        {
        }

        public void SetHeightInPoints(double value)
        {
        }

        public void SetHidden(bool value)
        {
        }

        internal override void CompleteWriteOverride()
        {
            this.writer.WriteLine();

            base.CompleteWriteOverride();
        }

        private void Write(string value)
        {
            this.writer.Write(value);
        }
    }
}