﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Csv
{
    internal class CsvWorkbookExporter : WorkbookExporterBase
    {
        private readonly Stream stream;
        private readonly SpreadCellStyleCollection cellStyles;
        private readonly CsvStreamWriter writer;
        private bool shouldCheckLicense;

        public CsvWorkbookExporter(Stream stream, bool isAppending)
        {
            this.stream = stream;
            if (isAppending)
            {
                this.stream.Seek(0, SeekOrigin.End);
            }
            else
            {
                this.stream.Seek(0, SeekOrigin.Begin);
                this.stream.SetLength(0);
            }
            this.writer = new CsvStreamWriter(this.stream, Encoding.UTF8, false);

            this.cellStyles = new SpreadCellStyleCollection();

            this.shouldCheckLicense = true;
        }

        public override SpreadCellStyleCollection CellStyles
        {
            get
            {
                return this.cellStyles;
            }
        }

        public override IWorksheetExporter CreateWorksheetExporterOverride(string name)
        {
            CsvWorksheetExporter exporter = new CsvWorksheetExporter(this, this.writer, this.shouldCheckLicense);

            this.shouldCheckLicense = false;

            return exporter;
        }

        public override IEnumerable<SheetInfo> GetSheetInfos()
        {
            return Enumerable.Empty<SheetInfo>();
        }

        internal override void CompleteWriteOverride()
        {
            this.writer.Flush();
            this.writer.Dispose();
            base.CompleteWriteOverride();
        }
    }
}