﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.Model;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Xlsx
{
    internal sealed class XlsxColumnExporter : EntityBase, IColumnExporter
    {
        private readonly ColumnRange columnRange;
        private readonly Queue<ColumnRange> columnRangesQueue;
        private readonly Action notifyDisposeAction;

        internal XlsxColumnExporter(Queue<ColumnRange> columnRangesQueue, int columnIndex, Action notifyDisposeAction)
        {
            this.notifyDisposeAction = notifyDisposeAction;
            columnIndex = columnIndex + 1; // 1 based column indexes in the document

            this.columnRange = new ColumnRange(columnIndex);
            this.columnRangesQueue = columnRangesQueue;
        }

        public void SetWidthInPixels(double width)
        {
            Guard.ThrowExceptionIfLessThan(0, width, "width");

            double excelWidth = UnitHelper.PixelWidthToExcelColumnWidth(width);

            this.columnRange.CustomWidth = true;
            this.columnRange.Width = excelWidth;
        }

        public void SetWidthInCharacters(double count)
        {
            Guard.ThrowExceptionIfOutOfRange(0, 255, count, "count");

            double pixelWidth = UnitHelper.CharactersCountToPixelWidth(count);
            pixelWidth += DefaultValues.LeftCellMargin + DefaultValues.RightCellMargin;

            this.SetWidthInPixels(pixelWidth);
        }

        public void SetOutlineLevel(int value)
        {
            Guard.ThrowExceptionIfLessThan(0, value, "value");

            this.columnRange.OutlineLevel = value;
        }

        public void SetHidden(bool value)
        {
            this.columnRange.Hidden = value;
        }

        internal sealed override void CompleteWriteOverride()
        {
            if (columnRangesQueue.Count > 0)
            {
                ColumnRange lastColumn = this.columnRangesQueue.Peek();
                if (lastColumn.LastColumnNumber + 1 == this.columnRange.LastColumnNumber &&
                    lastColumn.HasEqualColumnProperties(this.columnRange))
                {
                    lastColumn.MergeWith(this.columnRange);
                    return;
                }
            }

            this.columnRangesQueue.Enqueue(this.columnRange);
        }

        internal override void DisposeOverride()
        {
            this.notifyDisposeAction();

            base.DisposeOverride();
        }
    }
}