﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.Model;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Xlsx
{
    internal class XlsxMergedCellsExporter : EntityBase, IDisposable
    {
        private readonly List<SpreadCellRange> mergedCellsRanges;
        private readonly MergedCellsElementWriter mergedCellsCollectionWriter;

        public XlsxMergedCellsExporter(MergedCellsElementWriter mergedCellsCollectionWriter, List<SpreadCellRange> mergedCellsRanges)
        {
            this.mergedCellsCollectionWriter = mergedCellsCollectionWriter;
            this.mergedCellsRanges = mergedCellsRanges;
        }

        public void MergeCells(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex)
        {
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.RowCount - 1, fromRowIndex, "fromRowIndex");
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.ColumnCount - 1, fromColumnIndex, "fromColumnIndex");
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.RowCount - 1, toRowIndex, "toRowIndex");
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.ColumnCount - 1, toColumnIndex, "toColumnIndex");

            Guard.ThrowExceptionIfLessThan(fromRowIndex, toRowIndex, "toRowIndex");
            Guard.ThrowExceptionIfLessThan(fromColumnIndex, toColumnIndex, "toColumnIndex");

            this.ValidateMergeRegion(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex);

            MergedCellElementWriter mergedCell = mergedCellsCollectionWriter.CreateMergedCellElementWriter();
            mergedCell.Write(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex);
        }

        private static bool ValidateRange(int firstFromIndex, int firstToIndex, int secondFromIndex, int secondToIndex)
        {
            bool isValidRange =
                (firstFromIndex >= secondFromIndex && firstFromIndex <= secondToIndex) ||
                (firstToIndex >= secondFromIndex && firstToIndex <= secondToIndex) ||
                (firstFromIndex <= secondFromIndex && firstToIndex >= secondFromIndex);

            return isValidRange;
        }

        private void ValidateMergeRegion(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex)
        {
            foreach (var mergedCellsRange in this.mergedCellsRanges)
            {
                // check for intersection of rows
                bool hasRowsIntersection = ValidateRange(fromRowIndex, toRowIndex, mergedCellsRange.FromRowIndex, mergedCellsRange.ToRowIndex);

                // check for intersection of columns
                bool hasColumnsInterseciton = ValidateRange(fromColumnIndex, toColumnIndex, mergedCellsRange.FromColumnIndex, mergedCellsRange.ToColumnIndex);

                if (hasColumnsInterseciton && hasRowsIntersection)
                {
                    string exceptionMessage =
                        string.Format("There is an intersection with previously merged cells range. FromRowIdex: {0}, fromColumnIndex:{1}, toRowIndex:{2}, toColumnIndex:{3} ", mergedCellsRange.FromRowIndex, mergedCellsRange.FromColumnIndex, mergedCellsRange.ToRowIndex, mergedCellsRange.ToColumnIndex);

                    throw new InvalidOperationException(exceptionMessage);
                }
            }

            this.mergedCellsRanges.Add(new SpreadCellRange(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex));
        }
    }
}