﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.Model.Formatting;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Xlsx
{
    internal sealed class XlsxRowExporter : EntityBase, IRowExporter
    {
        private readonly StylesRepository stylesElement;
        private readonly RowElement rowElementWriter;
        private readonly int rowIndex;
        private int columnIndex;
        private bool hasAliveCell;

        internal XlsxRowExporter(SheetDataElement sheetData, StylesRepository stylesElement, int rowIndex)
        {
            Guard.ThrowExceptionIfGreaterThan(DefaultValues.RowCount - 1, rowIndex, "rowIndex");

            this.stylesElement = stylesElement;
            this.columnIndex = 0;
            this.rowIndex = rowIndex;

            this.rowElementWriter = sheetData.CreateRowElementWriter();
            this.rowElementWriter.RowIndex = this.rowIndex + 1; // rows have 1 based indexes
        }

        public void SkipCells(int count)
        {
            Guard.ThrowExceptionIfLessThan(0, count, "count");

            if (this.hasAliveCell)
            {
                throw new InvalidOperationException("Only one ICellExporter is allowed at a time. Please dispose any previously created ICellExporter instances.");
            }

            this.columnIndex += count;
        }

        public ICellExporter CreateCellExporter()
        {
            if (this.hasAliveCell)
            {
                throw new InvalidOperationException("Only one ICellExporter is allowed at a time. Please dispose any previously created ICellExporter instances.");
            }

            this.rowElementWriter.EnsureWritingStarted();
            this.hasAliveCell = true;

            return new XlsxCellExporter(this, this.rowElementWriter, this.stylesElement, this.rowIndex, this.columnIndex++);
        }

        public void SetOutlineLevel(int value)
        {
            Guard.ThrowExceptionIfLessThan(0, value, "value");

            this.rowElementWriter.OutlineLevel = value;
        }

        public void SetHeightInPixels(double value)
        {
            Guard.ThrowExceptionIfLessThan(0, value, "value");

            double valueInPoints = UnitHelper.DipToPoint(value);
            this.SetHeightInPoints(valueInPoints);
        }

        public void SetHeightInPoints(double value)
        {
            Guard.ThrowExceptionIfOutOfRange(0, 409, value, "value");

            this.rowElementWriter.CustomHeight = true;
            this.rowElementWriter.RowHeight = value;
        }

        public void SetHidden(bool value)
        {
            this.rowElementWriter.Hidden = value;
        }

        internal void NotifyCellDisposing()
        {
            this.hasAliveCell = false;
        }

        internal sealed override void CompleteWriteOverride()
        {
            this.rowElementWriter.EnsureWritingStarted();
            this.rowElementWriter.EnsureWritingEnded();
        }
    }
}