﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.Model;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Exporters.Xlsx
{
    internal class XlsxWorksheetViewExporter : EntityBase, IWorksheetViewExporter
    {
        private readonly SheetViewsElement sheetViewsElement;
        private SheetView sheetView;

        public XlsxWorksheetViewExporter(SheetViewsElement sheetViewsElement)
        {
            this.sheetViewsElement = sheetViewsElement;
        }

        private SheetView SheetView
        {
            get
            {
                if (this.sheetView == null)
                {
                    this.sheetView = new SheetView();
                }

                return this.sheetView;
            }
        }

        public void SetFirstVisibleCell(int rowIndex, int columnIndex)
        {
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            if (this.SheetView.HasFreezePanes)
            {
                Pane pane = this.SheetView.Pane;
                if (pane.YSplit.HasValue)
                {
                    int rowsCount = pane.YSplit.Value;
                    Guard.ThrowExceptionIfGreaterThan(this.SheetView.BottomRightSelection.ActiveCellRowIndex - rowsCount, rowIndex, "rowIndex");
                }

                if (pane.XSplit.HasValue)
                {
                    int columnsCount = pane.XSplit.Value;
                    Guard.ThrowExceptionIfGreaterThan(this.SheetView.BottomRightSelection.ActiveCellColumnIndex - columnsCount, columnIndex, "columnIndex");
                }
            }

            this.SheetView.SetTopLeftCell(rowIndex, columnIndex);
        }

        public void SetFreezePanes(int rowsCount, int columnsCount)
        {
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.RowCount - 1, rowsCount, "rowsCount");
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.ColumnCount - 1, columnsCount, "columnsCount");

            this.SheetView.SetSplitOffset(rowsCount, columnsCount);
        }

        public void SetFreezePanes(int rowsCount, int columnsCount, int scrollablePaneFirstVisibleCellRowIndex, int scrollablePaneFirstVisibleCellColumnIndex)
        {
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.RowCount - 1, rowsCount, "rowsCount");
            Guard.ThrowExceptionIfOutOfRange(0, DefaultValues.ColumnCount - 1, columnsCount, "columnsCount");

            int topLeftRowIndex = this.SheetView.TopLeftSelection.ActiveCellRowIndex;
            int topLeftPaneBottomRightCellRowIndex = topLeftRowIndex + rowsCount;
            Guard.ThrowExceptionIfLessThan(topLeftPaneBottomRightCellRowIndex, scrollablePaneFirstVisibleCellRowIndex, "scrollablePaneFirstVisibleCellRowIndex");

            int topLeftColumnIndex = this.sheetView.TopLeftSelection.ActiveCellColumnIndex;
            int topLeftPaneBottomRightCellColumnIndex = topLeftColumnIndex + columnsCount;
            Guard.ThrowExceptionIfLessThan(topLeftPaneBottomRightCellColumnIndex, scrollablePaneFirstVisibleCellColumnIndex, "scrollablePaneFirstVisibleCellColumnIndex");

            this.SheetView.SetSplitOffset(rowsCount, columnsCount);
            this.SheetView.SetTopLeftCell(SpreadPane.BottomRight, scrollablePaneFirstVisibleCellRowIndex, scrollablePaneFirstVisibleCellColumnIndex);
        }

        public void SetScaleFactor(double percent)
        {
            Guard.ThrowExceptionIfLessThan(0, percent, "percent");

            this.SheetView.SetScaleFactor(percent);
        }

        public void SetShouldShowGridLines(bool value)
        {
            this.SheetView.SetShouldShowGridLines(value);
        }

        public void SetShouldShowRowColumnHeaders(bool value)
        {
            this.SheetView.SetShouldShowRowColumnHeaders(value);
        }

        public void SetActiveSelectionCell(int rowIndex, int columnIndex)
        {
            Guard.ThrowExceptionIfInvalidRowIndex(rowIndex);
            Guard.ThrowExceptionIfInvalidColumnIndex(columnIndex);

            this.SheetView.SetActiveSelectionCell(rowIndex, columnIndex);
        }

        public void AddSelectionRange(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex)
        {
            Guard.ThrowExceptionIfLessThan(0, fromRowIndex, "fromRowIndex");
            Guard.ThrowExceptionIfLessThan(0, fromColumnIndex, "fromColumnIndex");

            Guard.ThrowExceptionIfLessThan(fromRowIndex, toRowIndex, "toRowIndex");
            Guard.ThrowExceptionIfLessThan(fromColumnIndex, toColumnIndex, "toColumnIndex");

            SpreadCellRange cellRange = new SpreadCellRange(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex);
            this.SheetView.AddSelectedRange(cellRange);
        }

        internal override void CompleteWriteOverride()
        {
            if (this.sheetView != null)
            {
                this.sheetViewsElement.EnsureWritingStarted();

                SheetViewElement sheetViewElement = this.sheetViewsElement.CreateSheetViewWriter();
                sheetViewElement.Write(this.SheetView);

                this.sheetViewsElement.EnsureWritingEnded();
            }
        }
    }
}