﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members used to import a column.
    /// </summary>
    internal interface IColumnImporter
    {
        /// <summary>
        /// Gets the first index of the column range with same properties.
        /// </summary>
        /// <value>The index.</value>
        int FromIndex { get; }

        /// <summary>
        /// Gets the last index of the column range with same properties.
        /// </summary>
        /// <value>The index.</value>
        int ToIndex { get; }

        /// <summary>
        /// Gets a value indicating if the width of the column is custom.
        /// </summary>
        /// <value>A value indicating if the width of the column is custom.</value>
        bool IsCustomWidth { get; }

        /// <summary>
        /// Gets the width in pixels.
        /// </summary>
        /// <value>The width in pixels.</value>
        double WidthInPixels { get; }

        /// <summary>
        /// Gets the width in characters.
        /// </summary>
        /// <value>The width in characters.</value>
        double WidthInCharacters { get; }

        /// <summary>
        /// Gets the outline level.
        /// </summary>
        /// <value>The outline level.</value>
        int OutlineLevel { get; }

        /// <summary>
        /// Gets a value indicating if the column is hidden.
        /// </summary>
        /// <value>A value indicating if the column is hidden.</value>
        bool IsHidden { get; }
    }
}