﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members for the row exporter classes.
    /// </summary>
    public interface IRowExporter : IDisposable
    {
        /// <summary>
        /// Creates a cell exporter.
        /// </summary>
        /// <returns>The cell exporter.</returns>
        ICellExporter CreateCellExporter();

        /// <summary>
        /// Skips a specified number of cells.
        /// </summary>
        /// <param name="count">The count of the cells to skip.</param>
        void SkipCells(int count);

        /// <summary>
        /// Sets the row outline level.
        /// </summary>
        /// <param name="value">The row outline level value.</param>
        void SetOutlineLevel(int value);

        /// <summary>
        /// Sets the row height in pixels.
        /// </summary>
        /// <param name="value">The row height value in pixels.</param>
        void SetHeightInPixels(double value);

        /// <summary>
        /// Sets the row height in points.
        /// </summary>
        /// <param name="value">The row height value in points.</param>
        void SetHeightInPoints(double value);

        /// <summary>
        /// Sets a value indicating whether the row should be hidden.
        /// </summary>
        /// <param name="value">True for hidden; otherwise false.</param>
        void SetHidden(bool value);
    }
}