﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members for the row importer classes.
    /// </summary>
    internal interface IRowImporter
    {
        /// <summary>
        /// Gets the index of the row.
        /// </summary>
        /// <value>The index of the row.</value>
        int RowIndex { get; }

        /// <summary>
        /// Gets the outline level.
        /// </summary>
        /// <value>The outline level.</value>
        int OutlineLevel { get; }

        /// <summary>
        /// Gets a value indicating if the height is custom.
        /// </summary>
        /// <value>A value indicating if the height is custom.</value>
        bool IsCustomHeight { get; }

        /// <summary>
        /// Gets the height in pixels.
        /// </summary>
        /// <value>The height in pixels.</value>
        double HeightInPixels { get; }

        /// <summary>
        /// Gets the height in points.
        /// </summary>
        /// <value>The height in points.</value>
        double HeightInPoints { get; }

        /// <summary>
        /// Gets a value indicating if the row is hidden.
        /// </summary>
        /// <value>A value indicating if the row is hidden.</value>
        bool IsHidden { get; }

        /// <summary>
        /// Gets the cells.
        /// </summary>
        /// <value>The cells.</value>
        IEnumerable<ICellImporter> Cells { get; }
    }
}