﻿using System;
using System.Collections.Generic;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members used to export a workbook.
    /// </summary>
    public interface IWorkbookExporter : IDisposable
    {
        /// <summary>
        /// Gets the cell style collection.
        /// </summary>
        /// <value>The cell style collection.</value>
        SpreadCellStyleCollection CellStyles { get; }

        /// <summary>
        /// Creates a worksheet exporter.
        /// </summary>
        /// <param name="name">The name of the worksheet.</param>
        /// <returns>The worksheet exporter.</returns>
        IWorksheetExporter CreateWorksheetExporter(string name);

        /// <summary>
        /// Provides information about the sheets in the workbook.
        /// </summary>
        /// <returns></returns>
        /// <returns>Returns an <see cref="IEnumerable{SheetInfo}">IEnumerable&lt;SheetInfo&gt;</see> containing information about all sheets present in the workbook.</returns>
        IEnumerable<SheetInfo> GetSheetInfos();
    }
}