﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members used to export a worksheet.
    /// </summary>
    public interface IWorksheetExporter : IDisposable
    {
        /// <summary>
        /// Creates a worksheet view exporter.
        /// </summary>
        /// <returns>The worksheet view exporter.</returns>
        IWorksheetViewExporter CreateWorksheetViewExporter();

        /// <summary>
        /// Creates a row exporter.
        /// </summary>
        /// <returns>The row exporter.</returns>
        IRowExporter CreateRowExporter();

        /// <summary>
        /// Creates a column exporter.
        /// </summary>
        /// <returns>The column exporter.</returns>
        IColumnExporter CreateColumnExporter();

        /// <summary>
        /// Skips a specified number of rows.
        /// </summary>
        /// <param name="count">The count of the rows to skip.</param>
        void SkipRows(int count);

        /// <summary>
        /// Skips a specified number of columns.
        /// </summary>
        /// <param name="count">The count of the columns to skip.</param>
        void SkipColumns(int count);

        /// <summary>
        /// Merges cells.
        /// </summary>
        /// <param name="fromRowIndex">The start row index of the area.</param>
        /// <param name="fromColumnIndex">The start column index of the area.</param>
        /// <param name="toRowIndex">The end row index of the area.</param>
        /// <param name="toColumnIndex">The end column index of the area.</param>
        void MergeCells(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex);
    }
}