﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Defines members used to export the worksheet view state.
    /// </summary>
    public interface IWorksheetViewExporter : IDisposable
    {
        /// <summary>
        /// Sets the first visible cell of the viewport.
        /// </summary>
        /// <param name="rowIndex">The row index.</param>
        /// <param name="columnIndex">The column index.</param>
        void SetFirstVisibleCell(int rowIndex, int columnIndex);

        /// <summary>
        /// Sets the freeze panes.
        /// </summary>
        /// <param name="rowsCount">The number of frozen rows.</param>
        /// <param name="columnsCount">The number of frozen column.</param>
        void SetFreezePanes(int rowsCount, int columnsCount);

        /// <summary>
        /// Sets the freeze panes.
        /// </summary>
        /// <param name="rowsCount">The number of frozen rows.</param>
        /// <param name="columnsCount">The number of frozen column.</param>
        /// <param name="scrollablePaneFirstVisibleCellRowIndex">The scrollable pane first visible cell row index.</param>
        /// <param name="scrollablePaneFirstVisibleCellColumnIndex">The scrollable pane first visible cell column index.</param>
        void SetFreezePanes(int rowsCount, int columnsCount, int scrollablePaneFirstVisibleCellRowIndex, int scrollablePaneFirstVisibleCellColumnIndex);

        /// <summary>
        /// Sets the scale factor.
        /// </summary>
        /// <param name="percent">The percentage that the viewport will be scaled to.</param>
        void SetScaleFactor(double percent);

        /// <summary>
        /// Sets a value indicating whether gridlines should be visible.
        /// </summary>
        /// <param name="value">True for visible gridlines, otherwise False.</param>
        void SetShouldShowGridLines(bool value);

        /// <summary>
        /// Sets a value indicating whether row and column headers should be visible.
        /// </summary>
        /// <param name="value">True for visible row and column headers, otherwise False.</param>
        void SetShouldShowRowColumnHeaders(bool value);

        /// <summary>
        /// Sets the active cell of the selection.
        /// </summary>
        /// <param name="rowIndex">The row index of the active cell.</param>
        /// <param name="columnIndex">The column index of the active cell.</param>
        void SetActiveSelectionCell(int rowIndex, int columnIndex);

        /// <summary>
        /// Adds a range to the selection.
        /// </summary>
        /// <param name="fromRowIndex">The row index of the top left cell of the range.</param>
        /// <param name="fromColumnIndex">The column index of the top left cell of the range.</param>
        /// <param name="toRowIndex">The row index of the bottom right cell of the range.</param>
        /// <param name="toColumnIndex">The column index of the bottom right cell of the range.</param>
        void AddSelectionRange(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex);
    }
}