﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes
{
    internal class ConvertedOpenXmlAttribute<T> : OpenXmlAttributeBase<T>
    {
        private readonly IStringConverter<T> converter;

        public ConvertedOpenXmlAttribute(string name, IStringConverter<T> converter, T defaultValue, bool isRequired = false)
            : this(name, null, converter, defaultValue, isRequired)
        { 
        }

        public ConvertedOpenXmlAttribute(string name, IStringConverter<T> converter, bool isRequired = false)
            : this(name, null, converter, isRequired)
        { 
        }

        public ConvertedOpenXmlAttribute(string name, OpenXmlNamespace ns, IStringConverter<T> converter, bool isRequired = false)
            : this(name, ns, converter, default(T), isRequired)
        {
        }

        public ConvertedOpenXmlAttribute(string name, OpenXmlNamespace ns, IStringConverter<T> converter, T defaultValue, bool isRequired = false)
            : base(name, ns, defaultValue, isRequired)
        {
            this.converter = converter;
        }

        public override string GetValueAsString()
        {
            return this.converter.ConvertToString(this.Value);
        }

        protected override void SetStringValueOverride(string value)
        {
            this.Value = this.converter.ConvertFromString(value);
        }
    }
}