﻿using System;
using System.Linq;
using System.Xml;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core
{
    internal abstract class DirectElementBase<T> : ElementBase
    {
        public void Write(T value)
        {
            this.OnBeforeWrite(value);
            this.InitializeAttributesOverride(value);

            if (this.ShouldExport())
            {
                this.BeginWrite();
                this.WriteChildElementsOverride(value);
                this.EndWrite();
            }
        }

        public void Read(ref T value)
        {
            this.BeginRead();

            if (!this.IsEmptyElement)
            {
                XmlNodeType nodeType = this.Reader.GetNodeType();
                if (nodeType == XmlNodeType.Text || nodeType == XmlNodeType.SignificantWhitespace)
                {
                    this.ReadInnerText();
                    this.Reader.Read();
                }
                else
                {
                    this.ReadChildElements(ref value);
                }
            }

            this.CopyAttributesOverride(ref value);

            this.EndRead();
        }

        protected virtual void OnBeforeWrite(T value)
        {
        }

        protected abstract void InitializeAttributesOverride(T value);
        protected abstract void WriteChildElementsOverride(T value);

        protected virtual void CopyAttributesOverride(ref T value)
        {
            throw new NotImplementedException();
        }

        protected virtual void ReadChildElementOverride(ElementBase element, ref T value)
        {
            throw new NotImplementedException();
        }

        protected TResult CreateChildElement<TResult>()
            where TResult : ElementBase, new()
        {
            ElementContext context = new ElementContext(this.Writer, this.Reader, this.Theme);
            TResult instance = new TResult();
            instance.SetContext(context);

            return instance;
        }

        private void ReadChildElements(ref T value)
        {
            string elementName;
            ElementBase element;
            while (!this.Reader.IsElementOfType(XmlNodeType.EndElement))
            {
                switch (this.Reader.GetNodeType())
                {
                    case XmlNodeType.Element:
                       if (this.Reader.GetElementName(out elementName))
                        {
                            element = this.CreateElement(elementName);
                            if (element != null)
                            {
                                this.ReadChildElementOverride(element, ref value);
                            }
                            else
                            {
                                this.Reader.SkipElement();
                            }
                        }
                        else
                        {
                            this.Reader.SkipElement();
                        }

                        break;

                    default:
                        this.Reader.SkipElement();

                        break;
                }
            }

            this.Reader.Read();
        }
    }
}