﻿using System;
using System.IO;
using System.Linq;
using System.Xml;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core
{
    internal class OpenXmlReader
    {
        private readonly XmlReader reader;

        public OpenXmlReader(Stream stream)
        {
            this.reader = XmlReader.Create(stream);
        }

        public bool IsEndOfElement()
        {
            return this.IsElementOfType(XmlNodeType.EndElement);
        }

        public XmlNodeType GetNodeType()
        {
            if (this.reader == null)
            {
                return XmlNodeType.None;
            }

            return this.reader.NodeType;
        }

        public bool IsEmptyElement()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.IsEmptyElement;
        }

        public bool HasAttributes()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.HasAttributes;
        }

        public bool MoveToFirstAttribute()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.MoveToFirstAttribute();
        }

        public bool MoveToNextAttribute()
        {
            Guard.ThrowExceptionIfNull(this.reader, "currentPartReader");

            return this.reader.MoveToNextAttribute();
        }

        public bool IsElementOfType(XmlNodeType type)
        {
            if (this.reader == null)
            {
                return false;
            }

            return this.reader.NodeType == type;
        }

        public bool Read()
        {
            if (this.reader == null)
            {
                return false;
            }

            if (this.reader.Read())
            {
                return true;
            }

            return false;
        }

        public void SkipElement()
        {
            if (this.reader == null)
            {
                return;
            }

            this.reader.Skip();
        }

        public bool GetElementName(out string elementName)
        {
            elementName = string.Empty;

            if (this.reader == null)
            {
                return false;
            }

            if (this.IsElementOfType(XmlNodeType.Element))
            {
                elementName = this.reader.LocalName;
                return true;
            }

            return false;
        }

        public bool GetElementValue(out string value)
        {
            if (this.reader == null)
            {
                value = string.Empty;

                return false;
            }
            else
            {
                value = this.reader.Value;

                return true;
            }
        }

        public bool GetAttributeNameAndValue(out string attributeName, out string value)
        {
            attributeName = string.Empty;
            value = string.Empty;

            if (this.reader == null)
            {
                return false;
            }

            if (this.IsElementOfType(XmlNodeType.Attribute))
            {
                attributeName = this.reader.Name;
                value = this.reader.Value;

                return true;
            }

            return false;
        }
    }
}