﻿using System;
using System.IO;
using System.Linq;

#if NETSTANDARD
using Telerik.Zip;
#else
using Telerik.Windows.Zip;
#endif

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core
{
    internal abstract class PartBase<T> : PartBase
        where T : ElementBase, new()
    {
        private readonly ZipArchiveEntry entry;
        private readonly Stream stream;
        private readonly OpenXmlWriter writer;
        private readonly OpenXmlReader reader;

        internal PartBase(PartContext context)
            : base(context.Name)
        {
            string path = context.Path;
            if (string.IsNullOrEmpty(path))
            {
                path = this.GetPartFullPath();
            }

            var archive = context.Archive;
            this.entry = archive.Entries.Where(p => p.FullName == path).FirstOrDefault();
            if (this.entry != null)
            {
                this.stream = this.entry.Open();
                this.reader = new OpenXmlReader(stream);
            }
            else
            {
                this.entry = context.Archive.CreateEntry(path);
                this.stream = this.entry.Open();
                this.writer = new OpenXmlWriter(stream);
            }

            ElementContext elementContext = new ElementContext(this.writer, this.reader, context.Theme);

            this.RootElement = new T();
            this.RootElement.SetContext(elementContext);
        }

        public T RootElement { get; private set; }

        public OpenXmlReader Reader
        {
            get
            {
                return this.reader;
            }
        }

        protected OpenXmlWriter Writer
        {
            get
            {
                return this.writer;
            }
        }

        internal override void CompleteWriteOverride()
        {
            if (this.Writer != null)
            {
                this.Writer.Flush();
            }

            this.entry.Dispose();
            this.stream.Dispose();
        }
    }
}