﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types
{
    internal static class CellTypesMapper
    {
        public static readonly string Boolean = "b";
        public static readonly string Error = "e";
        public static readonly string InlineString = "inlineStr";
        public static readonly string Number = "n";
        public static readonly string Date = "d";

        // public static readonly string SharedString = "s";
        public static readonly string String = "str";

        private static readonly Dictionary<string, CellValueType> nameToCellValueType;
        private static readonly Dictionary<CellValueType, string> cellValueTypeToName;

        static CellTypesMapper()
        {
            nameToCellValueType = new Dictionary<string, CellValueType>();
            cellValueTypeToName = new Dictionary<CellValueType, string>();

            RegisterPair(Boolean, CellValueType.Boolean);
            RegisterPair(Error, CellValueType.Error);
            RegisterPair(InlineString, CellValueType.RichText);
            RegisterPair(Number, CellValueType.Number);
            RegisterPair(Date, CellValueType.Date);
            RegisterPair(String, CellValueType.Text);
        }

        public static string GetCellTypeName(CellValueType valueType)
        {
            return cellValueTypeToName[valueType];
        }

        public static CellValueType GetCellValueType(string name)
        {
            return nameToCellValueType[name];
        }

        private static void RegisterPair(string name, CellValueType value)
        {
            nameToCellValueType.Add(name, value);
            cellValueTypeToName.Add(value, name);
        }
    }
}