﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types
{
    internal static class FontSchemesMapper
    {
        public static readonly string MajorFont = "major";
        public static readonly string MinorFont = "minor";
        public static readonly string None = "none";

        private static readonly ValueMapper<string, SpreadThemeFontType> nameToValueMapper;

        static FontSchemesMapper()
        {
            nameToValueMapper = new ValueMapper<string, SpreadThemeFontType>();
            nameToValueMapper.DefaultFromValue = new ValueBox<string>(FontSchemesMapper.None);

            nameToValueMapper.AddPair(MajorFont, SpreadThemeFontType.Major);
            nameToValueMapper.AddPair(MinorFont, SpreadThemeFontType.Minor);
        }

        public static string GetFontSchemeName(SpreadThemeFontType scheme)
        {
            return nameToValueMapper.GetFromValue(scheme);
        }

        public static SpreadThemeFontType? GetFontScheme(string name)
        {
            if (name.Equals(None, StringComparison.OrdinalIgnoreCase))
            {
                return null;
            }

            return nameToValueMapper.GetToValue(name);
        }
    }
}