﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types
{
    internal static class PatternTypesMapper
    {
        public static readonly string DarkDown = "darkDown";
        public static readonly string DarkGray = "darkGray";
        public static readonly string DarkGrid = "darkGrid";
        public static readonly string DarkHorizontal = "darkHorizontal";
        public static readonly string DarkTrellis = "darkTrellis";
        public static readonly string DarkUp = "darkUp";
        public static readonly string DarkVertical = "darkVertical";
        public static readonly string Gray0625 = "gray0625";
        public static readonly string Gray125 = "gray125";
        public static readonly string LightDown = "lightDown";
        public static readonly string LightGray = "lightGray";
        public static readonly string LightGrid = "lightGrid";
        public static readonly string LightHorizontal = "lightHorizontal";
        public static readonly string LightTrellis = "lightTrellis";
        public static readonly string LightUp = "lightUp";
        public static readonly string LightVertical = "lightVertical";
        public static readonly string MediumGray = "mediumGray";
        public static readonly string None = "none";
        public static readonly string Solid = "solid";

        private static readonly ValueMapper<string, SpreadPatternType> nameToValueMapper;

        static PatternTypesMapper()
        {
            nameToValueMapper = new ValueMapper<string, SpreadPatternType>();

            nameToValueMapper.AddPair(Solid, SpreadPatternType.Solid);
            nameToValueMapper.AddPair(DarkGray, SpreadPatternType.Gray75Percent);
            nameToValueMapper.AddPair(MediumGray, SpreadPatternType.Gray50Percent);
            nameToValueMapper.AddPair(LightGray, SpreadPatternType.Gray25Percent);
            nameToValueMapper.AddPair(Gray125, SpreadPatternType.Gray12Percent);
            nameToValueMapper.AddPair(Gray0625, SpreadPatternType.Gray6Percent);
            nameToValueMapper.AddPair(DarkHorizontal, SpreadPatternType.HorizontalStripe);
            nameToValueMapper.AddPair(DarkVertical, SpreadPatternType.VerticalStripe);
            nameToValueMapper.AddPair(DarkDown, SpreadPatternType.ReverseDiagonalStripe);
            nameToValueMapper.AddPair(DarkUp, SpreadPatternType.DiagonalStripe);
            nameToValueMapper.AddPair(DarkGrid, SpreadPatternType.DiagonalCrosshatch);
            nameToValueMapper.AddPair(DarkTrellis, SpreadPatternType.ThickDiagonalCrosshatch);
            nameToValueMapper.AddPair(LightHorizontal, SpreadPatternType.ThinHorizontalStripe);
            nameToValueMapper.AddPair(LightVertical, SpreadPatternType.ThinVerticalStripe);
            nameToValueMapper.AddPair(LightDown, SpreadPatternType.ThinReverseDiagonalStripe);
            nameToValueMapper.AddPair(LightUp, SpreadPatternType.ThinDiagonalStripe);
            nameToValueMapper.AddPair(LightGrid, SpreadPatternType.ThinHorizontalCrossHatch);
            nameToValueMapper.AddPair(LightTrellis, SpreadPatternType.ThinDiagonalCrosshatch);
        }

        public static string GetPatternTypeName(SpreadPatternType pattern)
        {
            return nameToValueMapper.GetFromValue(pattern);
        }

        public static SpreadPatternType GetPatternType(string name)
        {
            return nameToValueMapper.GetToValue(name);
        }
    }
}