﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements
{
    internal abstract class ColorElementBase : DirectElementBase<SpreadThemableColor>
    {
        ////private readonly IntOpenXmlAttribute indexed;
        private readonly ConvertedOpenXmlAttribute<UnsignedIntHex> rgb;
        private readonly IntOpenXmlAttribute themeColorId;
        private readonly OpenXmlAttribute<double> tint;

        public ColorElementBase()
        {
            ////this.indexed = this.RegisterAttribute(new IntOpenXmlAttribute("indexed"));
            this.rgb = this.RegisterAttribute(new ConvertedOpenXmlAttribute<UnsignedIntHex>("rgb", Converters.UnsignedIntHexConverter));
            this.themeColorId = this.RegisterAttribute(new IntOpenXmlAttribute("theme"));
            this.tint = this.RegisterAttribute<double>("tint", 0.0);
        }

        ////private int Indexed
        ////{
        ////    set
        ////    {
        ////        this.indexed.Value = value;
        ////    }
        ////}

        private UnsignedIntHex Rgb
        {
            get
            {
                return this.rgb.Value;
            }
            set
            {
                this.rgb.Value = value;
            }
        }

        private int ThemeColorId
        {
            get
            {
                return this.themeColorId.Value;
            }
            set
            {
                this.themeColorId.Value = value;
            }
        }

        private double Tint
        {
            get
            {
                return this.tint.Value;
            }
            set
            {
                this.tint.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(SpreadThemableColor value)
        {
            if (value.IsFromTheme)
            {
                this.ThemeColorId = (int)value.ThemeColorType;
                if (value.ColorShadeType.HasValue)
                {
                    this.Tint = this.Theme.ColorScheme.GetTintAndShade(value.ThemeColorType, value.ColorShadeType.Value);
                }
                else if (value.TintAndShade.HasValue)
                {
                    this.Tint = value.TintAndShade.Value;
                }
            }
            else
            {
                this.Rgb = new UnsignedIntHex(value.GetActualValue(this.Theme));
            }
        }

        protected override void CopyAttributesOverride(ref SpreadThemableColor value)
        {
            if (this.themeColorId.HasValue)
            {
                SpreadThemeColorType colorType = (SpreadThemeColorType)this.ThemeColorId;
                SpreadColor baseColor = this.Theme.ColorScheme[colorType].Color;
                SpreadColorShadeType? shadeType = ColorsHelper.GetColorShadeTypeForColorAndTint(baseColor, this.Tint);
                
                if (shadeType.HasValue)
                {
                    value = new SpreadThemableColor(colorType, shadeType);
                    return;
                }
               
                if (this.tint.HasValue)
                {
                    value = new SpreadThemableColor(colorType, this.Tint);
                    return;
                }

                value = new SpreadThemableColor(colorType);
                return;
            }
            else if (this.rgb.HasValue)
            {
                value = new SpreadThemableColor(this.Rgb.Color);
                return;
            }
            ////else if (this.indexed.HasValue)
            ////{
            ////    ResourceIndexedTable<Color> indexedColorTable = context.StyleSheet.IndexedColorTable;
            ////    int indexedColorTableCount = indexedColorTable.Count();
            ////    int indexed = this.Indexed;

            ////    if (indexed < 0 || indexed >= indexedColorTableCount)
            ////    {
            ////        indexed = (indexed % 2 == 0) ? indexedColorTableCount - 2 : indexedColorTableCount - 1;
            ////    }

            ////    return new SpreadThemableColor(context.StyleSheet.IndexedColorTable[indexed]);
            ////}
            
            value = new SpreadThemableColor(new SpreadColor(0, 0, 0));
        }

        protected override void WriteChildElementsOverride(SpreadThemableColor value)
        {
        }
    }
}