﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.Model.Relations;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers
{
    internal class RelationshipElement : DirectElementBase<Relationship>
    {
        private readonly OpenXmlAttribute<string> targetAttribute;
        private readonly OpenXmlAttribute<string> typeAttribute;
        private readonly OpenXmlAttribute<string> idAttribute;
        private readonly OpenXmlAttribute<string> targetModeAttribute;

        public RelationshipElement()
        {
            this.idAttribute = this.RegisterAttribute<string>("Id", true);
            this.typeAttribute = this.RegisterAttribute<string>("Type", true);
            this.targetAttribute = this.RegisterAttribute<string>("Target", true);
            this.targetModeAttribute = this.RegisterAttribute<string>("TargetMode", string.Empty, false);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.RelationshipElementName;
            }
        }

        private string Target
        {
            get
            {
                return this.targetAttribute.Value;
            }
            set
            {
                this.targetAttribute.Value = value;
            }
        }

        private string Type
        {
            get
            {
                return this.typeAttribute.Value;
            }
            set
            {
                this.typeAttribute.Value = value;
            }
        }

        private string Id
        {
            get
            {
                return this.idAttribute.Value;
            }
            set
            {
                this.idAttribute.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private string TargetMode
        {
            set
            {
                this.targetModeAttribute.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(Relationship value)
        {
            this.Id = value.RelationshipId;
            this.Type = value.Type;
            this.Target = value.Target;
        }

        protected override void CopyAttributesOverride(ref Relationship value)
        {
            value = new Relationship(this.Id, this.Type, this.Target);
        }

        protected override void WriteChildElementsOverride(Relationship value)
        {
        }

        protected override void ReadChildElementOverride(ElementBase element, ref Relationship value)
        {
        }
    }
}