﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet.Styles
{
    internal class AlignmentElement : DirectElementBase<DiferentialFormat>
    {
        private readonly OpenXmlAttribute<string> horizontalAlignment;
        private readonly OpenXmlAttribute<string> verticalAlignment;
        private readonly BoolOpenXmlAttribute wrapText;
        private readonly IntOpenXmlAttribute indent;

        public AlignmentElement()
        {
            this.horizontalAlignment = this.RegisterAttribute(new OpenXmlAttribute<string>("horizontal"));
            this.verticalAlignment = this.RegisterAttribute(new OpenXmlAttribute<string>("vertical"));
            this.indent = this.RegisterAttribute(new IntOpenXmlAttribute("indent"));
            this.wrapText = this.RegisterAttribute(new BoolOpenXmlAttribute("wrapText"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.AlignmentElementName;
            }
        }

        protected override bool AlwaysExport
        {
            get
            {
                return false;
            }
        }

        private string HorizontalAlignment
        {
            get
            {
                return this.horizontalAlignment.Value;
            }
            set
            {
                this.horizontalAlignment.Value = value;
            }
        }

        private string VerticalAlignment
        {
            get
            {
                return this.verticalAlignment.Value;
            }
            set
            {
                this.verticalAlignment.Value = value;
            }
        }

        private bool WrapText
        {
            get
            {
                return this.wrapText.Value;
            }
            set
            {
                this.wrapText.Value = value;
            }
        }

        private int Indent
        {
            get
            {
                return this.indent.Value;
            }
            set
            {
                this.indent.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(DiferentialFormat value)
        {
            if (value.HorizontalAlignment.HasValue)
            {
                this.HorizontalAlignment = HorizontalAlignmentsMapper.GetHorizontalAlignmentName(value.HorizontalAlignment.Value);
            }

            if (value.VerticalAlignment.HasValue)
            {
                this.VerticalAlignment = VerticalAlignmentsMapper.GetVerticalAlignmentName(value.VerticalAlignment.Value);
            }

            if (value.Indent.HasValue)
            {
                this.Indent = value.Indent.Value;
            }

            if (value.WrapText.HasValue)
            {
                this.WrapText = value.WrapText.Value;
            }
        }

        protected override void WriteChildElementsOverride(DiferentialFormat value)
        {
        }

        protected override void CopyAttributesOverride(ref DiferentialFormat value)
        {
            if (this.horizontalAlignment.HasValue)
            {
                value.HorizontalAlignment = HorizontalAlignmentsMapper.GetHorizontalAlignmentValue(this.HorizontalAlignment);
            }

            if (this.verticalAlignment.HasValue)
            {
                value.VerticalAlignment = VerticalAlignmentsMapper.GetVerticalAlignmentValue(this.VerticalAlignment);
            }

            if (this.indent.HasValue)
            {
                value.Indent = this.Indent;
            }

            if (this.wrapText.HasValue)
            {
                value.WrapText = this.WrapText;
            }
        }

        protected override void ReadChildElementOverride(ElementBase element, ref DiferentialFormat value)
        {
        }
    }
}