﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.Model.Formatting;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Styles
{
    internal class BorderElement : DirectElementBase<SpreadCellBorders>
    {
        private readonly BoolOpenXmlAttribute diagonalDown;
        private readonly BoolOpenXmlAttribute diagonalUp;

        public BorderElement()
        {
            this.diagonalDown = this.RegisterAttribute(new BoolOpenXmlAttribute("diagonalDown"));
            this.diagonalUp = this.RegisterAttribute(new BoolOpenXmlAttribute("diagonalUp"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.BorderElementName;
            }
        }

        private bool DiagonalDown
        {
            get
            {
                return this.diagonalDown.Value;
            }
            set
            {
                this.diagonalDown.Value = value;
            }
        }

        private bool DiagonalUp
        {
            get
            {
                return this.diagonalUp.Value;
            }
            set
            {
                this.diagonalUp.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(SpreadCellBorders value)
        {
            if (!IsNullOrNone(value.DiagonalDown))
            {
                this.DiagonalDown = true;
            }

            if (!IsNullOrNone(value.DiagonalUp))
            {
                this.DiagonalUp = true;
            }
        }

        protected override void WriteChildElementsOverride(SpreadCellBorders value)
        {
            LeftBorderElement leftBorderElement = this.CreateChildElement<LeftBorderElement>();
            leftBorderElement.Write(value.Left);

            RightBorderElement rightBorderElement = this.CreateChildElement<RightBorderElement>();
            rightBorderElement.Write(value.Right);

            TopBorderElement topBorderElement = this.CreateChildElement<TopBorderElement>();
            topBorderElement.Write(value.Top);

            BottomBorderElement bottomBorderElement = this.CreateChildElement<BottomBorderElement>();
            bottomBorderElement.Write(value.Bottom);

            SpreadBorder diagonalBorder = value.DiagonalUp;
            if (diagonalBorder == null)
            {
                diagonalBorder = value.DiagonalDown;
            }

            DiagonalBorderElement diagonalBorderElement = this.CreateChildElement<DiagonalBorderElement>();
            diagonalBorderElement.Write(diagonalBorder);
        }

        protected override void CopyAttributesOverride(ref SpreadCellBorders value)
        {
        }

        protected override void ReadChildElementOverride(ElementBase element, ref SpreadCellBorders value)
        {
            switch (element.ElementName)
            {
                case XlsxElementNames.LeftBorderElementName:
                    {
                        SpreadBorder border = ReadBorderElement(element);

                        value.Left = border;
                    }
                    break;

                case XlsxElementNames.RightBorderElementName:
                    {
                        SpreadBorder border = ReadBorderElement(element);

                        value.Right = border;
                    }
                    break;

                case XlsxElementNames.TopBorderElementName:
                    {
                        SpreadBorder border = ReadBorderElement(element);

                        value.Top = border;
                    }
                    break;

                case XlsxElementNames.BottomBorderElementName:
                    {
                        SpreadBorder border = ReadBorderElement(element);

                        value.Bottom = border;
                    }
                    break;

                case XlsxElementNames.DiagonalBorderElementName:
                    {
                        SpreadBorder border = ReadBorderElement(element);

                        if (this.DiagonalDown)
                        {
                            value.DiagonalDown = border;
                        }

                        if (this.DiagonalUp)
                        {
                            value.DiagonalUp = border;
                        }
                    }
                    break;

                default:
                    throw new InvalidOperationException();
            }
        }

        private static SpreadBorder ReadBorderElement(ElementBase element)
        {
            BorderTypeElementBase borderElement = element as BorderTypeElementBase;
            SpreadBorder border = null;

            borderElement.Read(ref border);
            return border;
        }

        private static bool IsNullOrNone(SpreadBorder border)
        {
            return border == null || border.Style == SpreadBorderStyle.None;
        }
    }
}