﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Styles
{
    internal abstract class BorderTypeElementBase : DirectElementBase<SpreadBorder>
    {
        private readonly OpenXmlAttribute<string> style;

        public BorderTypeElementBase()
        {
            this.style = this.RegisterAttribute<string>("style", BorderStylesMapper.None);
        }

        private string Style
        {
            get
            {
                return this.style.Value;
            }
            set
            {
                this.style.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(SpreadBorder value)
        {
            if (value != null)
            {
                this.Style = BorderStylesMapper.GetBorderStyleName(value.Style);
            }
        }

        protected override void WriteChildElementsOverride(SpreadBorder value)
        {
            if (value != null)
            {
                if (value.Style != SpreadBorderStyle.None && value.Color != null)
                {
                    ColorElement color = this.CreateChildElement<ColorElement>();
                    color.Write(value.Color);
                }
            }
        }

        protected override void CopyAttributesOverride(ref SpreadBorder value)
        {
            if (this.style.HasValue)
            {
                value = value ?? new SpreadBorder();

                value.Style = BorderStylesMapper.GetBorderStyle(this.Style);
            }
        }

        protected override void ReadChildElementOverride(ElementBase element, ref SpreadBorder value)
        {
            ColorElement colorElement = element as ColorElement;
            if (colorElement != null)
            {
                value = value ?? new SpreadBorder();

                SpreadThemableColor color = null;
                colorElement.Read(ref color);
                value.Color = color;
            }
        }
    }
}