﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Styles
{
    internal class CellStyleElement : DirectElementBase<CellStyleInfo>
    {
        private readonly IntOpenXmlAttribute builtinId;
        private readonly BoolOpenXmlAttribute customBuiltin;
        private readonly OpenXmlAttribute<string> name;
        private readonly IntOpenXmlAttribute formatId;

        public CellStyleElement()
        {
            this.name = this.RegisterAttribute(new OpenXmlAttribute<string>("name"));
            this.formatId = this.RegisterAttribute(new IntOpenXmlAttribute("xfId", true));
            this.builtinId = this.RegisterAttribute(new IntOpenXmlAttribute("builtinId"));
            this.customBuiltin = this.RegisterAttribute(new BoolOpenXmlAttribute("customBuiltin"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.CellStyleElementName;
            }
        }

        private int BuiltinId
        {
            get
            {
                return this.builtinId.Value;
            }
            set
            {
                this.builtinId.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private bool CustomBuiltin
        {
            set
            {
                this.customBuiltin.Value = value;
            }
        }

        private string Name
        {
            get
            {
                return this.name.Value;
            }
            set
            {
                this.name.Value = value;
            }
        }

        private int FormatId
        {
            get
            {
                return this.formatId.Value;
            }
            set
            {
                this.formatId.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(CellStyleInfo value)
        {
            this.Name = value.Name;
            this.FormatId = value.FormattingRecordId;

            if (value.BuiltInId.HasValue)
            {
                this.BuiltinId = value.BuiltInId.Value;
            }
        }

        protected override void WriteChildElementsOverride(CellStyleInfo value)
        {
        }

        protected override void CopyAttributesOverride(ref CellStyleInfo value)
        {
            value.Name = this.Name;
            value.FormattingRecordId = this.FormatId;

            if (this.builtinId.HasValue)
            {
                value.BuiltInId = this.BuiltinId;
            }
        }

        protected override void ReadChildElementOverride(ElementBase element, ref CellStyleInfo value)
        {
        }
    }
}