﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet.Styles;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Styles
{
    internal class FormatElement : DirectElementBase<DiferentialFormat>
    {
        private readonly BoolOpenXmlAttribute applyAlignment;
        private readonly BoolOpenXmlAttribute applyBorder;
        private readonly BoolOpenXmlAttribute applyFill;
        private readonly BoolOpenXmlAttribute applyFont;
        private readonly BoolOpenXmlAttribute applyNumberFormat;
        private readonly BoolOpenXmlAttribute applyProtection;
        private readonly IntOpenXmlAttribute borderId;
        private readonly IntOpenXmlAttribute fillId;
        private readonly IntOpenXmlAttribute fontId;
        private readonly IntOpenXmlAttribute numberFormatId;
        private readonly IntOpenXmlAttribute formatId;

        public FormatElement()
        {
            this.numberFormatId = this.RegisterAttribute(new IntOpenXmlAttribute("numFmtId"));
            this.fontId = this.RegisterAttribute(new IntOpenXmlAttribute("fontId"));
            this.fillId = this.RegisterAttribute(new IntOpenXmlAttribute("fillId"));
            this.borderId = this.RegisterAttribute(new IntOpenXmlAttribute("borderId"));
            this.formatId = this.RegisterAttribute(new IntOpenXmlAttribute("xfId"));
            this.applyNumberFormat = this.RegisterAttribute(new BoolOpenXmlAttribute("applyNumberFormat"));
            this.applyFont = this.RegisterAttribute(new BoolOpenXmlAttribute("applyFont"));
            this.applyFill = this.RegisterAttribute(new BoolOpenXmlAttribute("applyFill"));
            this.applyBorder = this.RegisterAttribute(new BoolOpenXmlAttribute("applyBorder"));
            this.applyAlignment = this.RegisterAttribute(new BoolOpenXmlAttribute("applyAlignment"));
            this.applyProtection = this.RegisterAttribute(new BoolOpenXmlAttribute("applyProtection"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FormatElementName;
            }
        }

        private bool ApplyAlignment
        {
            get
            {
                return this.applyAlignment.Value;
            }
            set
            {
                this.applyAlignment.Value = value;
            }
        }

        private bool ApplyBorder
        {
            get
            {
                return this.applyBorder.Value;
            }
            set
            {
                this.applyBorder.Value = value;
            }
        }

        private bool ApplyFill
        {
            get
            {
                return this.applyFill.Value;
            }
            set
            {
                this.applyFill.Value = value;
            }
        }

        private bool ApplyFont
        {
            get
            {
                return this.applyFont.Value;
            }
            set
            {
                this.applyFont.Value = value;
            }
        }

        private bool ApplyNumberFormat
        {
            get
            {
                return this.applyNumberFormat.Value;
            }
            set
            {
                this.applyNumberFormat.Value = value;
            }
        }

        private bool ApplyProtection
        {
            get
            {
                return this.applyProtection.Value;
            }
            set
            {
                this.applyProtection.Value = value;
            }
        }

        private int BorderId
        {
            get
            {
                return this.borderId.Value;
            }
            set
            {
                this.borderId.Value = value;
            }
        }

        private int FillId
        {
            get
            {
                return this.fillId.Value;
            }
            set
            {
                this.fillId.Value = value;
            }
        }

        private int FontId
        {
            get
            {
                return this.fontId.Value;
            }
            set
            {
                this.fontId.Value = value;
            }
        }

        private int NumberFormatId
        {
            get
            {
                return this.numberFormatId.Value;
            }
            set
            {
                this.numberFormatId.Value = value;
            }
        }

        private int FormatId
        {
            get
            {
                return this.formatId.Value;
            }
            set
            {
                this.formatId.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(DiferentialFormat value)
        {
            if (value.FormatId.HasValue)
            {
                this.FormatId = value.FormatId.Value;
            }

            if (value.FontId.HasValue)
            {
                this.FontId = value.FontId.Value;
            }

            if (value.BorderId.HasValue)
            {
                this.BorderId = value.BorderId.Value;
            }

            if (value.FillId.HasValue)
            {
                this.FillId = value.FillId.Value;
            }

            if (value.NumFmtId.HasValue)
            {
                this.NumberFormatId = value.NumFmtId.Value;
            }

            if (value.ApplyNumberFormat.HasValue)
            {
                this.ApplyNumberFormat = value.ApplyNumberFormat.Value;
            }

            if (value.ApplyAlignment.HasValue)
            {
                this.ApplyAlignment = value.ApplyAlignment.Value;
            }

            if (value.ApplyFont.HasValue)
            {
                this.ApplyFont = value.ApplyFont.Value;
            }

            if (value.ApplyBorder.HasValue)
            {
                this.ApplyBorder = value.ApplyBorder.Value;
            }

            if (value.ApplyFill.HasValue)
            {
                this.ApplyFill = value.ApplyFill.Value;
            }

            if (value.ApplyProtection.HasValue)
            {
                this.ApplyProtection = value.ApplyProtection.Value;
            }
        }

        protected override void WriteChildElementsOverride(DiferentialFormat value)
        {
            AlignmentElement alignmentsElement = this.CreateChildElement<AlignmentElement>();
            alignmentsElement.Write(value);

            if (value.IsLocked.HasValue)
            {
                ProtectionElement protectionElement = this.CreateChildElement<ProtectionElement>();
                protectionElement.Write(value);
            }
        }

        protected override void CopyAttributesOverride(ref DiferentialFormat value)
        {
            if (this.formatId.HasValue)
            {
                value.FormatId = this.FormatId;
            }

            if (this.fontId.HasValue)
            {
                value.FontId = this.FontId;
            }

            if (this.borderId.HasValue)
            {
                value.BorderId = this.BorderId;
            }

            if (this.fillId.HasValue)
            {
                value.FillId = this.FillId;
            }

            if (this.numberFormatId.HasValue)
            {
                value.NumFmtId = this.NumberFormatId;
            }

            if (this.applyNumberFormat.HasValue)
            {
                value.ApplyNumberFormat = this.ApplyNumberFormat;
            }

            if (this.applyAlignment.HasValue)
            {
                value.ApplyAlignment = this.ApplyAlignment;
            }

            if (this.applyFont.HasValue)
            {
                value.ApplyFont = this.ApplyFont;
            }

            if (this.applyBorder.HasValue)
            {
                value.ApplyBorder = this.ApplyBorder;
            }

            if (this.applyFill.HasValue)
            {
                value.ApplyFill = this.ApplyFill;
            }

            if (this.applyProtection.HasValue)
            {
                value.ApplyProtection = this.ApplyProtection;
            }
        }

        protected override void ReadChildElementOverride(ElementBase element, ref DiferentialFormat value)
        {
            switch (element.ElementName)
            {
                case XlsxElementNames.AlignmentElementName:
                    {
                        AlignmentElement alignmentElement = element as AlignmentElement;
                        alignmentElement.Read(ref value);
                    }
                    break;

                case XlsxElementNames.ProtectionElementName:
                    {
                        ProtectionElement protectionElement = element as ProtectionElement;
                        protectionElement.Read(ref value);
                    }
                    break;

                default:
                    throw new InvalidOperationException();
            }
        }
    }
}