﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet.Styles
{
    internal class GradientFillElement : DirectElementBase<ISpreadFill>
    {
        private readonly OpenXmlAttribute<double> bottom;
        private readonly OpenXmlAttribute<double> degree;
        private readonly OpenXmlAttribute<double> left;
        private readonly OpenXmlAttribute<double> right;
        private readonly OpenXmlAttribute<double> top;
        private readonly OpenXmlAttribute<string> type;

        private GradientFillInfo gradientFillInfo;

        public GradientFillElement()
        {
            this.type = this.RegisterAttribute<string>("type", GradientTypes.Linear);
            this.degree = this.RegisterAttribute<double>("degree", 0);
            this.left = this.RegisterAttribute<double>("left", 0);
            this.right = this.RegisterAttribute<double>("right", 0);
            this.top = this.RegisterAttribute<double>("top", 0);
            this.bottom = this.RegisterAttribute<double>("bottom", 0);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.GradientFillElementName;
            }
        }

        private string Type
        {
            get
            {
                return this.type.Value;
            }
            set
            {
                this.type.Value = value;
            }
        }

        private double Bottom
        {
            get
            {
                return this.bottom.Value;
            }
            set
            {
                this.bottom.Value = value;
            }
        }

        private double Degree
        {
            get
            {
                return this.degree.Value;
            }
            set
            {
                this.degree.Value = value;
            }
        }

        private double Left
        {
            get
            {
                return this.left.Value;
            }
            set
            {
                this.left.Value = value;
            }
        }

        private double Right
        {
            get
            {
                return this.right.Value;
            }
            set
            {
                this.right.Value = value;
            }
        }

        private double Top
        {
            get
            {
                return this.top.Value;
            }
            set
            {
                this.top.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(ISpreadFill value)
        {
            SpreadGradientFill fill = value as SpreadGradientFill;

            this.gradientFillInfo = GradientFillInfo.Create(fill);

            if (this.gradientFillInfo.Stops.Count == 0)
            {
                return;
            }

            if (this.gradientFillInfo.InfoType.HasValue)
            {
                this.Type = GradientTypes.GetGradientTypeName(this.gradientFillInfo.InfoType.Value);
            }

            if (this.gradientFillInfo.Degree.HasValue)
            {
                this.Degree = this.gradientFillInfo.Degree.Value;
            }

            if (this.gradientFillInfo.Left.HasValue)
            {
                this.Left = this.gradientFillInfo.Left.Value;
            }

            if (this.gradientFillInfo.Right.HasValue)
            {
                this.Right = this.gradientFillInfo.Right.Value;
            }

            if (this.gradientFillInfo.Top.HasValue)
            {
                this.Top = this.gradientFillInfo.Top.Value;
            }

            if (this.gradientFillInfo.Bottom.HasValue)
            {
                this.Bottom = this.gradientFillInfo.Bottom.Value;
            }
        }

        protected override void WriteChildElementsOverride(ISpreadFill value)
        {
            List<GradientStop> gradientStops = this.gradientFillInfo.Stops;

            foreach (GradientStop stop in gradientStops)
            {
                StopElement stopElement = this.CreateChildElement<StopElement>();
                stopElement.Write(stop);
            }
        }

        protected override void CopyAttributesOverride(ref ISpreadFill value)
        {
            if (this.type.HasValue)
            {
                this.gradientFillInfo.InfoType = GradientTypes.GetGradientTypeValue(this.Type);
            }

            if (this.degree.HasValue)
            {
                this.gradientFillInfo.Degree = this.Degree;
            }

            if (this.left.HasValue)
            {
                this.gradientFillInfo.Left = this.Left;
            }

            if (this.right.HasValue)
            {
                this.gradientFillInfo.Right = this.Right;
            }

            if (this.top.HasValue)
            {
                this.gradientFillInfo.Top = this.Top;
            }

            if (this.bottom.HasValue)
            {
                this.gradientFillInfo.Bottom = this.Bottom;
            }

            value = this.gradientFillInfo.ToFill();
        }

        protected override void ReadChildElementOverride(ElementBase element, ref ISpreadFill value)
        {
            if (this.gradientFillInfo == null)
            {
                this.gradientFillInfo = new GradientFillInfo();
            }

            switch (element.ElementName)
            {
                case XlsxElementNames.StopElementName:
                    {
                        GradientStop stop = new GradientStop();
                        StopElement stopElement = element as StopElement;
                        stopElement.Read(ref stop);

                        this.gradientFillInfo.Stops.Add(stop);
                    }
                    break;

                default:
                    throw new NotImplementedException();
            }
        }
    }
}