﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet.Styles;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Styles
{
    internal class PatternFillElement : DirectElementBase<ISpreadFill>
    {
        private readonly OpenXmlAttribute<string> patternType;
        private SpreadThemableColor foregroundColor = null;
        private SpreadThemableColor backgroundColor = null;

        public PatternFillElement()
        {
            this.patternType = this.RegisterAttribute<string>("patternType", PatternTypesMapper.None);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PatternFillElementName;
            }
        }

        private string PatternType
        {
            get
            {
                return this.patternType.Value;
            }
            set
            {
                this.patternType.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(ISpreadFill value)
        {
            NoneFill noneFill = value as NoneFill;
            if (noneFill != null)
            {
                this.PatternType = PatternTypesMapper.None;
            }

            SpreadPatternFill patternFill = value as SpreadPatternFill;
            if (patternFill != null)
            {
                this.PatternType = PatternTypesMapper.GetPatternTypeName(patternFill.PatternType);
            }
        }

        protected override void WriteChildElementsOverride(ISpreadFill value)
        {
            SpreadPatternFill fill = value as SpreadPatternFill;

            if (fill == null)
            {
                return;
            }

            SpreadThemableColor backgroundColor = fill.BackgroundColor;
            SpreadThemableColor patternColor = fill.PatternColor;

            if (fill.PatternType == SpreadPatternType.Solid)
            {
                SpreadThemableColor color = backgroundColor;
                backgroundColor = patternColor;
                patternColor = color;
            }

            if (patternColor != null)
            {
                ForegroundColorElement foregroundColorElement = this.CreateChildElement<ForegroundColorElement>();
                foregroundColorElement.Write(patternColor);
            }

            if (backgroundColor != null)
            {
                BackgroundColorElement backgroundColorElement = this.CreateChildElement<BackgroundColorElement>();
                backgroundColorElement.Write(backgroundColor);
            }
        }

        protected override void CopyAttributesOverride(ref ISpreadFill value)
        {
            if (this.PatternType == PatternTypesMapper.None)
            {
                value = NoneFill.Instance;
                return;
            }

            SpreadPatternType patternType = PatternTypesMapper.GetPatternType(this.PatternType);

            if (patternType == SpreadPatternType.Solid)
            {
                value = new SpreadPatternFill(patternType, this.backgroundColor, this.foregroundColor);
            }
            else
            {
                value = new SpreadPatternFill(patternType, this.foregroundColor, this.backgroundColor);
            }
        }

        protected override void ReadChildElementOverride(ElementBase element, ref ISpreadFill value)
        {
            switch (element.ElementName)
            {
                case XlsxElementNames.ForegroundColorElementName:
                    {
                        ForegroundColorElement foregroundColorElement = element as ForegroundColorElement;
                        foregroundColorElement.Read(ref this.foregroundColor);
                    }
                    break;

                case XlsxElementNames.BackgroundColorElementName:
                    {
                        BackgroundColorElement backgroundColorElement = element as BackgroundColorElement;
                        backgroundColorElement.Read(ref this.backgroundColor);
                    }
                    break;

                default:
                    throw new InvalidOperationException();
            }
        }
    }
}