﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet.Styles
{
    internal class StopElement : DirectElementBase<GradientStop>
    {
        private readonly OpenXmlAttribute<double> position;

        public StopElement()
        {
            this.position = this.RegisterAttribute<double>("position", true);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.StopElementName;
            }
        }

        private double Position
        {
            get
            {
                return this.position.Value;
            }
            set
            {
                this.position.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(GradientStop value)
        {
            this.Position = value.Position;
        }

        protected override void WriteChildElementsOverride(GradientStop value)
        {
            ColorElement color = this.CreateChildElement<ColorElement>();
            color.Write(value.ThemableColor);
        }

        protected override void CopyAttributesOverride(ref GradientStop value)
        {
            value.Position = this.Position;
        }

        protected override void ReadChildElementOverride(ElementBase element, ref GradientStop value)
        {
            switch (element.ElementName)
            {
                case XlsxElementNames.ColorElementName:
                    {
                        ColorElement colorElement = (ColorElement)element;
                        SpreadThemableColor color = null;
                        colorElement.Read(ref color);

                        value.ThemableColor = color;
                    }
                    break;

                default:
                    throw new InvalidOperationException();
            }
        }
    }
}