﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers;
using Telerik.Documents.SpreadsheetStreaming.Model.ContentTypes;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements
{
    internal class TypesElement : DirectElementBase<ContentTypesRepository>
    {
        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.ContentTypesNamespace;
            }
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.TypesElementName;
            }
        }

        protected override void InitializeAttributesOverride(ContentTypesRepository contentTypesRepository)
        {
        }

        protected override void WriteChildElementsOverride(ContentTypesRepository contentTypesRepository)
        {
            this.WriteDefaultElements(contentTypesRepository.Defaults);
            this.WriteOverrideElements(contentTypesRepository.Overrides);
        }

        protected override void CopyAttributesOverride(ref ContentTypesRepository value)
        {
        }

        protected override void ReadChildElementOverride(ElementBase element, ref ContentTypesRepository value)
        {
            switch (element.ElementName)
            {
                case XlsxElementNames.DefaultElementName:
                    {
                        DefaultElement defaultElement = element as DefaultElement;
                        DefaultContentType contentType = new DefaultContentType();
                        defaultElement.Read(ref contentType);
                        value.Register(contentType);
                    }
                    break;

                case XlsxElementNames.OverrideElementName:
                    {
                        OverrideElement overrideElement = element as OverrideElement;
                        OverrideContentType contentType = new OverrideContentType();
                        overrideElement.Read(ref contentType);
                        value.Register(contentType);
                    }
                    break;

                default:
                    throw new NotSupportedException();
            }
        }

        private void WriteDefaultElements(IEnumerable<DefaultContentType> defaults)
        {
            foreach (DefaultContentType defaultContentType in defaults)
            {
                DefaultElement element = this.CreateChildElement<DefaultElement>();
                element.Write(defaultContentType);
            }
        }

        private void WriteOverrideElements(IEnumerable<OverrideContentType> overrides)
        {
            foreach (OverrideContentType overrideContentType in overrides)
            {
                OverrideElement element = this.CreateChildElement<OverrideElement>();
                element.Write(overrideContentType);
            }
        }
    }
}