﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class CellElement : ConsecutiveElementBase
    {
        private readonly ConvertedOpenXmlAttribute<CellRef> reference;
        private readonly IntOpenXmlAttribute styleIndex;
        private readonly OpenXmlAttribute<string> cellDataType;

        public CellElement()
        {
            this.RegisterChildElement<CellValueElement>();
            this.RegisterChildElement<FormulaElement>();

            this.reference = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("r", Converters.CellRefConverter));
            this.styleIndex = this.RegisterAttribute(new IntOpenXmlAttribute("s", 0));
            this.cellDataType = this.RegisterAttribute<string>("t", CellTypesMapper.Number);
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.CellElementName;
            }
        }

        public CellRef Reference
        {
            get
            {
                return this.reference.Value;
            }
            set
            {
                this.reference.Value = value;
            }
        }

        public int StyleIndex
        {
            set
            {
                this.styleIndex.Value = value;
            }
        }

        public string CellDataType
        {
            get
            {
                return this.cellDataType.Value;
            }
            set
            {
                this.cellDataType.Value = value;
            }
        }

        public void WriteFormula(string value)
        {
            FormulaElement formulaElementWriter = this.CreateChildElement<FormulaElement>();
            formulaElementWriter.Write(value);
        }

        public void WriteValue(string value)
        {
            CellValueElement cellValueElementWriter = this.CreateChildElement<CellValueElement>();
            cellValueElementWriter.Write(value);
        }

        public void ReadFormula(ref string value)
        {
            FormulaElement formulaElementWriter = this.CreateChildElement<FormulaElement>();
            formulaElementWriter.Read(ref value);
        }

        public void ReadValue(ref string value)
        {
            CellValueElement cellValueElementWriter = this.CreateChildElement<CellValueElement>();
            cellValueElementWriter.Read(ref value);
        }
    }
}