﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class ColumnElement : DirectElementBase<ColumnRange>
    {
        private readonly IntOpenXmlAttribute min;
        private readonly IntOpenXmlAttribute max;
        private readonly BoolOpenXmlAttribute bestFit;
        private readonly BoolOpenXmlAttribute customWidth;
        private readonly IntOpenXmlAttribute styleIndex;
        private readonly OpenXmlAttribute<double> width;
        private readonly BoolOpenXmlAttribute hidden;
        private readonly IntOpenXmlAttribute outlineLevel;

        public ColumnElement()
        {
            this.min = this.RegisterAttribute(new IntOpenXmlAttribute("min", true));
            this.max = this.RegisterAttribute(new IntOpenXmlAttribute("max", true));
            this.bestFit = this.RegisterAttribute(new BoolOpenXmlAttribute("bestFit"));
            this.customWidth = this.RegisterAttribute(new BoolOpenXmlAttribute("customWidth"));
            this.styleIndex = this.RegisterAttribute(new IntOpenXmlAttribute("style", 0));
            this.width = this.RegisterAttribute<double>("width");
            this.hidden = this.RegisterAttribute(new BoolOpenXmlAttribute("hidden"));
            this.outlineLevel = this.RegisterAttribute(new IntOpenXmlAttribute("outlineLevel"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.ColumnElementName;
            }
        }

        public int Min
        {
            get
            {
                return this.min.Value;
            }
            set
            {
                this.min.Value = value;
            }
        }

        public int Max
        {
            get
            {
                return this.max.Value;
            }
            set
            {
                this.max.Value = value;
            }
        }

        public bool CustomWidth
        {
            get
            {
                return this.customWidth.Value;
            }
            set
            {
                this.customWidth.Value = value;
            }
        }

        public double Width
        {
            get
            {
                return this.width.Value;
            }
            set
            {
                this.width.Value = value;
            }
        }

        public int OutlineLevel
        {
            get
            {
                return this.outlineLevel.Value;
            }
            set
            {
                this.outlineLevel.Value = value;
            }
        }

        public bool Hidden
        {
            get
            {
                return this.hidden.Value;
            }
            set
            {
                this.hidden.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private bool BestFit
        {
            set
            {
                this.bestFit.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private int StyleIndex
        {
            set
            {
                this.styleIndex.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(ColumnRange value)
        {
            if (value.CustomWidth.HasValue)
            {
                this.CustomWidth = value.CustomWidth.Value;
            }

            if (value.Hidden.HasValue)
            {
                this.Hidden = value.Hidden.Value;
            }

            this.Max = value.LastColumnNumber;
            this.Min = value.FirstColumnNumber;

            if (value.OutlineLevel.HasValue)
            {
                this.OutlineLevel = value.OutlineLevel.Value;
            }

            if (value.Width.HasValue)
            {
                this.Width = value.Width.Value;
            }
        }

        protected override void CopyAttributesOverride(ref ColumnRange value)
        {
            value = new ColumnRange(this.Min);
            value.LastColumnNumber = this.Max;

            if (this.customWidth.HasValue)
            {
                value.CustomWidth = this.CustomWidth;
            }

            if (this.hidden.HasValue)
            {
                value.Hidden = this.Hidden;
            }

            if (this.outlineLevel.HasValue)
            {
                value.OutlineLevel = this.OutlineLevel;
            }

            if (this.width.HasValue)
            {
                value.Width = this.Width;
            }
        }

        protected override void WriteChildElementsOverride(ColumnRange value)
        {
        }
    }
}