﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class FormulaElement : DirectElementBase<string>
    {
        private readonly OpenXmlAttribute<string> formulaType;
        private readonly IntOpenXmlAttribute sharedGroupIndex;
        private readonly ConvertedOpenXmlAttribute<Ref> reference;

        public FormulaElement()
        {
            this.formulaType = this.RegisterAttribute<string>("t", FormulaTypes.Normal);
            this.reference = this.RegisterAttribute(new ConvertedOpenXmlAttribute<Ref>("ref", Converters.RefConverter));
            this.sharedGroupIndex = this.RegisterAttribute(new IntOpenXmlAttribute("si"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.FormulaElementName;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private string FormulaType
        {
            set
            {
                this.formulaType.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private Ref Reference
        {
            set
            {
                this.reference.Value = value;
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private int SharedGroupIndex
        {
            set
            {
                this.sharedGroupIndex.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(string value)
        {
            // TODO: The formula should be writen in InvariantCulture!
            this.InnerText = SpreadsheetCultureHelper.ClearFormulaValue(value);
        }

        protected override void WriteChildElementsOverride(string value)
        {
        }

        protected override void CopyAttributesOverride(ref string value)
        {
            value = this.InnerText;
        }

        protected override void ReadChildElementOverride(ElementBase element, ref string value)
        {
        }
    }
}