﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class PaneElement : DirectElementBase<Pane>
    {
        private readonly IntOpenXmlAttribute xSplit;
        private readonly IntOpenXmlAttribute ySplit;
        private readonly ConvertedOpenXmlAttribute<CellRef> topLeftCell;
        private readonly MappedOpenXmlAttribute<PaneState> state;
        private readonly MappedOpenXmlAttribute<SpreadPane> activePane;

        public PaneElement()
        {
            this.xSplit = this.RegisterAttribute(new IntOpenXmlAttribute("xSplit", 0));
            this.ySplit = this.RegisterAttribute(new IntOpenXmlAttribute("ySplit", 0));
            this.topLeftCell = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("topLeftCell", Converters.CellRefConverter, true));

            this.activePane = this.RegisterAttribute(new MappedOpenXmlAttribute<SpreadPane>("activePane", null, SpreadPaneMapper.Instance));
            this.state = this.RegisterAttribute(new MappedOpenXmlAttribute<PaneState>("state", null, ViewportStateMapper.Instance));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.PaneElementName;
            }
        }

        private int XSplit
        {
            set
            {
                this.xSplit.Value = value;
            }
        }

        private int YSplit
        {
            set
            {
                this.ySplit.Value = value;
            }
        }

        private CellRef TopLeftCell
        {
            set
            {
                this.topLeftCell.Value = value;
            }
        }

        private SpreadPane ActivePane
        {
            set
            {
                this.activePane.Value = value;
            }
        }

        private PaneState State
        {
            set
            {
                this.state.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(Pane value)
        {
            if (value.XSplit.HasValue)
            {
                this.XSplit = value.XSplit.Value;
            }

            if (value.YSplit.HasValue)
            {
                this.YSplit = value.YSplit.Value;
            }

            int rowIndex;
            int columnIndex;
            value.GetTopLeftCell(out rowIndex, out columnIndex);
            this.TopLeftCell = new CellRef(rowIndex, columnIndex);

            if (value.ActivePane.HasValue)
            {
                this.ActivePane = value.ActivePane.Value;
            }

            if (value.State.HasValue)
            {
                this.State = value.State.Value;
            }
        }

        protected override void WriteChildElementsOverride(Pane value)
        {
        }
    }
}