﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class RowElement : ConsecutiveElementBase
    {
        private readonly OpenXmlAttribute<double> rowHeight;
        private readonly ConvertedOpenXmlAttribute<int?> rowIndex;
        private readonly BoolOpenXmlAttribute customFormat;
        private readonly BoolOpenXmlAttribute customHeight;
        private readonly IntOpenXmlAttribute styleIndex;
        private readonly BoolOpenXmlAttribute hidden;
        private readonly IntOpenXmlAttribute outlineLevel;

        public RowElement()
        {
            this.RegisterChildElement<CellElement>();

            this.rowIndex = this.RegisterAttribute(new ConvertedOpenXmlAttribute<int?>("r", Converters.NullableIntValueConverter, false));
            this.rowHeight = this.RegisterAttribute<double>("ht");
            this.styleIndex = this.RegisterAttribute(new IntOpenXmlAttribute("s", 0));
            this.customFormat = this.RegisterAttribute(new BoolOpenXmlAttribute("customFormat"));
            this.customHeight = this.RegisterAttribute(new BoolOpenXmlAttribute("customHeight"));
            this.hidden = this.RegisterAttribute(new BoolOpenXmlAttribute("hidden"));
            this.outlineLevel = this.RegisterAttribute(new IntOpenXmlAttribute("outlineLevel"));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.RowElementName;
            }
        }

        public double RowHeight
        {
            get
            {
                return this.rowHeight.Value;
            }
            set
            {
                this.rowHeight.Value = value;
            }
        }

        public int? RowIndex
        {
            get
            {
                return this.rowIndex.Value;
            }
            set
            {
                this.rowIndex.Value = value;
            }
        }

        public bool CustomFormat
        {
            set
            {
                this.customFormat.Value = value;
            }
        }

        public bool CustomHeight
        {
            get
            {
                return this.customHeight.Value;
            }
            set
            {
                this.customHeight.Value = value;
            }
        }

        public int StyleIndex
        {
            set
            {
                this.styleIndex.Value = value;
            }
        }

        public bool Hidden
        {
            get
            {
                return this.hidden.Value;
            }
            set
            {
                this.hidden.Value = value;
            }
        }

        public int OutlineLevel
        {
            get
            {
                return this.outlineLevel.Value;
            }
            set
            {
                this.outlineLevel.Value = value;
            }
        }

        public IEnumerable<CellElement> Cells
        {
            get
            {
                while (this.ReadToElement<CellElement>())
                {
                    CellElement cellElement = this.CreateChildElement<CellElement>();
                    cellElement.BeginReadElement();

                    yield return cellElement;

                    cellElement.EndReadElement();
                }

                yield break;
            }
        }

        public CellElement CreateCellElementWriter()
        {
            return this.CreateChildElement<CellElement>();
        }
    }
}