﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class SelectionElement : DirectElementBase<Selection>
    {
        private readonly ConvertedOpenXmlAttribute<CellRef> activeCell;
        private readonly ConvertedOpenXmlAttribute<CellRefRangeSequence> sqref;
        private readonly MappedOpenXmlAttribute<SpreadPane> pane;

        public SelectionElement()
        {
            this.activeCell = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("activeCell", Converters.CellRefConverter, true));
            this.sqref = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRefRangeSequence>("sqref", Converters.CellRefRangeSequenceConverter, true));

            this.pane = this.RegisterAttribute(new MappedOpenXmlAttribute<SpreadPane>("pane", null, SpreadPaneMapper.Instance));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SelectionElementName;
            }
        }

        public CellRef ActiveCell
        {
            get
            {
                return this.activeCell.Value;
            }
            set
            {
                this.activeCell.Value = value;
            }
        }

        public CellRefRangeSequence SqRef
        {
            get
            {
                return this.sqref.Value;
            }
            set
            {
                this.sqref.Value = value;
            }
        }

        public SpreadPane Pane
        {
            get
            {
                return this.pane.Value;
            }
            set
            {
                this.pane.Value = value;
            }
        }

        protected override void InitializeAttributesOverride(Selection value)
        {
            int rowIndex;
            int columnIndex;
            value.GetActiveCell(out rowIndex, out columnIndex);
            this.ActiveCell = new CellRef(rowIndex, columnIndex);

            if (value.SelectionRanges == null)
            {
                SpreadCellRange selectedRange = new SpreadCellRange(rowIndex, columnIndex, rowIndex, columnIndex);
                CellRefRange selectedCellRefRange = new CellRefRange(selectedRange);
                this.SqRef = new CellRefRangeSequence(new CellRefRange[] { selectedCellRefRange });
            }
            else
            {
                IEnumerable<CellRefRange> cellRanges = value.SelectionRanges.Select(p => new CellRefRange(p));
                this.SqRef = new CellRefRangeSequence(cellRanges);
            }

            this.Pane = value.Pane;
        }

        protected override void WriteChildElementsOverride(Selection value)
        {
        }
    }
}