﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Attributes;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet
{
    internal class SheetViewElement : DirectElementBase<SheetView>
    {
        private readonly OpenXmlAttribute<double> zoomScale;
        private readonly ConvertedOpenXmlAttribute<CellRef> topLeftCell;
        private readonly IntOpenXmlAttribute workbookViewId;
        private readonly BoolOpenXmlAttribute showGridLines;
        private readonly BoolOpenXmlAttribute showRowColHeaders;

        public SheetViewElement()
        {
            this.zoomScale = this.RegisterAttribute<double>("zoomScale", 100);
            this.topLeftCell = this.RegisterAttribute(new ConvertedOpenXmlAttribute<CellRef>("topLeftCell", Converters.CellRefConverter, new CellRef(0, 0)));
            this.workbookViewId = this.RegisterAttribute(new IntOpenXmlAttribute("workbookViewId", 0, true));
            this.showGridLines = this.RegisterAttribute(new BoolOpenXmlAttribute("showGridLines", true, false));
            this.showRowColHeaders = this.RegisterAttribute(new BoolOpenXmlAttribute("showRowColHeaders", true, false));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.SheetViewElementName;
            }
        }

        public double ZoomScale
        {
            get
            {
                return this.zoomScale.Value;
            }
            set
            {
                this.zoomScale.Value = value;
            }
        }

        public CellRef TopLeftCell
        {
            get
            {
                return this.topLeftCell.Value;
            }
            set
            {
                this.topLeftCell.Value = value;
            }
        }

        public int WorkbookViewId
        {
            get
            {
                return this.workbookViewId.Value;
            }
            set
            {
                this.workbookViewId.Value = value;
            }
        }

        public bool ShowGridLines
        {
            get
            {
                return this.showGridLines.Value;
            }
            set
            {
                this.showGridLines.Value = value;
            }
        }

        public bool ShowRowColHeaders
        {
            get
            {
                return this.showRowColHeaders.Value;
            }
            set
            {
                this.showRowColHeaders.Value = value;
            }
        }

        protected override void OnBeforeWrite(SheetView value)
        {
            value.PrepareForExport();
        }

        protected override void InitializeAttributesOverride(SheetView value)
        {
            if (value.ScaleFactor.HasValue)
            {
                this.ZoomScale = value.ScaleFactor.Value;
            }

            int rowIndex;
            int columnIndex;
            value.GetTopLeftCell(SpreadPane.TopLeft, out rowIndex, out columnIndex);
            this.TopLeftCell = new CellRef(rowIndex, columnIndex);

            if (value.ShouldShowGridLines.HasValue)
            {
                this.ShowGridLines = value.ShouldShowGridLines.Value;
            }

            if (value.ShouldShowRowColumnHeaders.HasValue)
            {
                this.ShowRowColHeaders = value.ShouldShowRowColumnHeaders.Value;
            }
        }

        protected override void WriteChildElementsOverride(SheetView value)
        {
            if (value.HasFreezePanes)
            {
                PaneElement paneElementWriter = this.CreateChildElement<PaneElement>();
                paneElementWriter.Write(value.Pane);

                if (value.IsFirstColumnFreeze || (!value.IsFirstColumnFreeze && !value.IsFirstRowFreeze))
                {
                    SelectionElement topRightSelection = this.CreateChildElement<SelectionElement>();
                    topRightSelection.Write(value.TopRightSelection);
                }

                if (value.IsFirstRowFreeze || (!value.IsFirstColumnFreeze && !value.IsFirstRowFreeze))
                {
                    SelectionElement bottomLeftSelection = this.CreateChildElement<SelectionElement>();
                    bottomLeftSelection.Write(value.BottomLeftSelection);
                }

                if (!value.IsFirstColumnFreeze && !value.IsFirstRowFreeze)
                {
                    SelectionElement bottomRightSelection = this.CreateChildElement<SelectionElement>();
                    bottomRightSelection.Write(value.BottomRightSelection);
                }
            }
            else
            {
                SelectionElement topLeftSelection = this.CreateChildElement<SelectionElement>();
                topLeftSelection.Write(value.TopLeftSelection);
            }
        }
    }
}