﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model
{
    internal class FontProperties
    {
        internal FontProperties()
        {
        }

        internal FontProperties(int fontSize, SpreadThemableColor foreColor, SpreadThemableFontFamily fontFamily)
        {
            this.FontSize = fontSize;
            this.ForeColor = foreColor;
            this.FontFamily = fontFamily;
        }

        public double? FontSize { get; set; }

        public SpreadThemableColor ForeColor { get; set; }

        public bool? IsBold { get; set; }

        public bool? IsItalic { get; set; }

        public SpreadUnderlineType? Underline { get; set; }

        public SpreadThemableFontFamily FontFamily { get; set; }

        public bool HasSetProperties
        {
            get
            {
                return this.FontSize.HasValue ||
                    this.ForeColor != null ||
                    this.IsBold.HasValue ||
                    this.IsItalic.HasValue ||
                    this.Underline.HasValue ||
                    this.VerticalAlignment != null ||
                    this.Strike.HasValue ||
                    this.FontFamily != null;
            }
        }

        public string VerticalAlignment { get; set; }

        public bool? Strike { get; set; }

        public static FontProperties FromCellFormat(SpreadCellFormatBase cellFormat)
        {
            FontProperties result = new FontProperties();

            result.FontSize = cellFormat.FontSize;
            result.ForeColor = cellFormat.ForeColor;
            result.IsBold = cellFormat.IsBold;
            result.IsItalic = cellFormat.IsItalic;
            result.Underline = cellFormat.Underline;
            result.FontFamily = cellFormat.FontFamily;

            return result;
        }

        public override bool Equals(object obj)
        {
            FontProperties other = obj as FontProperties;
            if (other == null)
            {
                return false;
            }

            return ObjectExtensions.EqualsOfT(this.FontSize, other.FontSize) &&
                ObjectExtensions.EqualsOfT(this.ForeColor, other.ForeColor) &&
                ObjectExtensions.EqualsOfT(this.IsBold, other.IsBold) &&
                ObjectExtensions.EqualsOfT(this.IsItalic, other.IsItalic) &&
                ObjectExtensions.EqualsOfT(this.Underline, other.Underline) &&
                ObjectExtensions.EqualsOfT(this.VerticalAlignment, other.VerticalAlignment) &&
                ObjectExtensions.EqualsOfT(this.Strike, other.Strike) &&
                ObjectExtensions.EqualsOfT(this.FontFamily, other.FontFamily);
        }

        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(
                this.FontSize.GetHashCodeOrZero(),
                this.ForeColor.GetHashCodeOrZero(),
                this.IsBold.GetHashCodeOrZero(),
                this.IsItalic.GetHashCodeOrZero(),
                this.Underline.GetHashCodeOrZero(),
                this.VerticalAlignment.GetHashCodeOrZero(),
                this.Strike.GetHashCodeOrZero(),
                this.FontFamily.GetHashCodeOrZero());
        }
    }
}