﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers.Worksheet;

namespace Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Writers
{
    internal class WorksheetElement : ConsecutiveElementBase
    {
        public WorksheetElement()
        {
            this.RegisterChildElement<SheetViewsElement>();
            this.RegisterChildElement<ColumnsElement>();
            this.RegisterChildElement<SheetDataElement>();
            this.RegisterChildElement<MergedCellsElementWriter>(typeof(SheetDataElement));
        }

        public override string ElementName
        {
            get
            {
                return XlsxElementNames.WorksheetElementName;
            }
        }

        public override OpenXmlNamespace Namespace
        {
            get
            {
                return OpenXmlNamespaces.SpreadsheetMLNamespace;
            }
        }

        public override IEnumerable<OpenXmlNamespace> Namespaces
        {
            get
            {
                yield return OpenXmlNamespaces.OfficeDocumentRelationshipsNamespace;
            }
        }

        public SheetViewsElement GetSheetViewsElementWriter()
        {
            return this.GetRegisteredChildElement<SheetViewsElement>();
        }

        public ColumnsElement GetColumnsElementWriter()
        {
            return this.GetRegisteredChildElement<ColumnsElement>();
        }

        public ColumnsElement GetColumnsElementReader()
        {
            if (this.ReadToElement<ColumnsElement>())
            {
                return this.GetRegisteredChildElement<ColumnsElement>();
            }

            return null;
        }

        public SheetDataElement GetSheetDataElementWriter()
        {
            return this.GetRegisteredChildElement<SheetDataElement>();
        }

        public SheetDataElement GetSheetDataElementReader()
        {
            if (this.ReadToElement<SheetDataElement>())
            {
                return this.GetRegisteredChildElement<SheetDataElement>();
            }

            return null;
        }

        public MergedCellsElementWriter GetMergedCellsElementWriter()
        {
            return this.GetRegisteredChildElement<MergedCellsElementWriter>();
        }

        public MergedCellsElementWriter GetMergedCellsElementReader()
        {
            if (this.ReadToElement<MergedCellsElementWriter>())
            {
                return this.GetRegisteredChildElement<MergedCellsElementWriter>();
            }

            return null;
        }
    }
}