﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core.Types;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;

namespace Telerik.Documents.SpreadsheetStreaming.Importers
{
    internal class XlsxCellImporter : ICellImporter
    {
        private readonly CellElement cellElement;

        internal XlsxCellImporter(CellElement cellElement)
        {
            this.cellElement = cellElement;
            this.InitializeValue();
        }

        public int RowIndex
        {
            get
            {
                return this.cellElement.Reference.RowIndex;
            }
        }

        public int ColumnIndex
        {
            get
            {
                return this.cellElement.Reference.ColumnIndex;
            }
        }

        public SpreadCellFormat Format
        {
            get
            {
                // TODO: Implement this property getter
                throw new NotImplementedException();
            }
        }

        public string Value { get; private set; }

        private void InitializeValue()
        {
            CellValueType dataType = CellTypesMapper.GetCellValueType(this.cellElement.CellDataType);
            switch (dataType)
            {
                case CellValueType.Boolean:
                    {
                        string value = this.ReadValue();
                        this.Value = (value == "1").ToString();
                    }
                    break;

                case CellValueType.Date:
                case CellValueType.Number:
                case CellValueType.Text:
                    {
                        string value = this.ReadValue();
                        this.Value = value;
                    }
                    break;

                case CellValueType.Formula:
                    {
                        string value = this.ReadFormula();
                        this.Value = value;
                    }
                    break;

                case CellValueType.Error:
                case CellValueType.RichText:
                case CellValueType.SharedString:

                default:
                    throw new NotImplementedException();
            }
        }
 
        private string ReadFormula()
        {
            string value = null;
            this.cellElement.ReadFormula(ref value);
            return value;
        }
 
        private string ReadValue()
        {
            string value = null;
            this.cellElement.ReadValue(ref value);
            return value;
        }
    }
}