﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model.Elements.Worksheet;
using Telerik.Documents.SpreadsheetStreaming.Model;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Importers
{
    internal class XlsxColumnImporter : IColumnImporter
    {
        private readonly ColumnRange columnRange;

        internal XlsxColumnImporter(ColumnRange columnRange)
        {
            this.columnRange = columnRange;
        }

        public int FromIndex
        {
            get
            {
                return this.columnRange.FirstColumnNumber - 1;
            }
        }

        public int ToIndex
        {
            get
            {
                return this.columnRange.LastColumnNumber - 1;
            }
        }

        public bool IsCustomWidth
        {
            get
            {
                return this.columnRange.CustomWidth.Value;
            }
        }

        public double WidthInPixels
        {
            get
            {
                if (this.columnRange.CustomWidth.Value)
                {
                    double excelWidth = this.columnRange.Width.Value;
                    return UnitHelper.ExcelColumnWidthToPixelWidth(excelWidth);
                }

                return UnitHelper.CharactersCountToPixelWidth(DefaultValues.DefaultColumnWidthInCharacters);
            }
        }

        public double WidthInCharacters
        {
            get
            {
                return UnitHelper.PixelsToCharacterCount(this.WidthInPixels);
            }
        }

        public int OutlineLevel
        {
            get
            {
                return this.columnRange.OutlineLevel.Value;
            }
        }

        public bool IsHidden
        {
            get
            {
                return this.columnRange.Hidden.Value;
            }
        }
    }
}