﻿using Telerik.Windows.Documents.Licensing;

namespace Telerik.Documents.SpreadsheetStreaming.Licensing
{
    internal static class LicenseCheck
    {
        private static readonly string licenseSheetName = "RadSpreadStreamProcessing";

        internal static void Validate(IWorkbookExporter workbookExporter)
        {
            string licenseMessage;
            if (TryGetLicenseMessage(out licenseMessage))
            {
                using (IWorksheetExporter worksheetExporter = workbookExporter.CreateWorksheetExporter(licenseSheetName))
                {
                    ExportLicenseMessage(licenseMessage, worksheetExporter);
                }
            }
        }

        internal static void Validate(IWorksheetExporter worksheetExporter)
        {
            string licenseMessage;
            if (TryGetLicenseMessage(out licenseMessage))
            {
                ExportLicenseMessage(licenseMessage, worksheetExporter);
            }
        }

        private static void ExportLicenseMessage(string licenseMessage, IWorksheetExporter worksheetExporter)
        {
            using (IRowExporter rowExporter = worksheetExporter.CreateRowExporter())
            {
                using (ICellExporter cellExporter = rowExporter.CreateCellExporter())
                {
                    SpreadCellFormat spreadCellFormat = new SpreadCellFormat();
                    spreadCellFormat.ForeColor = new SpreadThemableColor(new SpreadColor(255, 0, 0));
                    cellExporter.SetFormat(spreadCellFormat);

                    cellExporter.SetValue(licenseMessage);
                }
            }
        }

        private static bool TryGetLicenseMessage(out string licenseMessage)
        {
            LicensingMode licensingMode = TelerikLicense.GetLicensingMode();
            bool result;

            switch (licensingMode)
            {
                case LicensingMode.Trial:
                    licenseMessage = TrialMessages.DocumentProcessingTrialMessage;
                    result = true;
                    break;
                case LicensingMode.Unlicensed:
                    licenseMessage = TrialMessages.DocumentProcessingUnlicencedMessage;
                    result = true;
                    break;
                case LicensingMode.Dev:
                default:
                    licenseMessage = null;
                    result = false;
                    break;
            }

            return result;
        }
    }
}