﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Model;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Model.Formatting
{
    internal class UniqueItemsStore<T> : IEnumerable<T>
    {
        //// OrderedDictionary could be use for better performance
        private readonly Dictionary<int, T> indexToValue;
        private readonly Dictionary<T, int> valueToIndex;

        private int startIndex;

        public UniqueItemsStore(int startIndex = 0)
        {
            this.startIndex = startIndex;
            this.indexToValue = new Dictionary<int, T>();
            this.valueToIndex = new Dictionary<T, int>();
        }

        public int Count
        {
            get
            {
                return this.indexToValue.Count;
            }
        }

        public int StartIndex
        {
            get
            {
                return this.startIndex;
            }
            set
            {
                this.startIndex = value;
            }
        }

        public T GetValueByIndex(int index)
        {
            return this.indexToValue[index];
        }

        public int GetIndexByValue(T value)
        {
            return this.valueToIndex[value];
        }

        public int GetNextIndex(T value)
        {
            if (this.ContainsValue(value))
            {
                return this.valueToIndex[value];
            }

            return this.GetNextIndexInternal();
        }

        public int Add(T value)
        {
            if (this.ContainsValue(value))
            {
                return this.valueToIndex[value];
            }

            int index = this.GetNextIndexInternal();

            this.Add(index, value);

            return index;
        }

        public void Add(int index, T value)
        {
            if (this.ContainsIndex(index))
            {
                if (!ObjectExtensions.EqualsOfT(this.indexToValue[index], value))
                {
                    throw new InvalidOperationException("Different item with the same index exists.");
                }

                return;
            }

            if (this.ContainsValue(value))
            {
                if (!ObjectExtensions.EqualsOfT(this.valueToIndex[value], index))
                {
                    throw new InvalidOperationException("Same item with different index exists.");
                }

                return;
            }

            this.indexToValue.Add(index, value);
            this.valueToIndex.Add(value, index);
        }

        public void Remove(T value)
        {
            int index = this.valueToIndex[value];
            this.Remove(index, value);
        }

        public void RemoveAtIndex(int index)
        {
            T value = this.indexToValue[index];
            this.Remove(index, value);
        }

        public bool ContainsValue(T value)
        {
            return this.valueToIndex.ContainsKey(value);
        }

        public bool ContainsIndex(int index)
        {
            return this.indexToValue.ContainsKey(index);
        }

        public IEnumerator<T> GetEnumerator()
        {
            // OrderedDictionary could be use for better performance
            return this.indexToValue.OrderBy(p => p.Key).Select(p => p.Value).GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        internal void Remove(int index, T value)
        {
            this.indexToValue.Remove(index);
            this.valueToIndex.Remove(value);
        }

        private int GetNextIndexInternal()
        {
            return this.startIndex++;
        }
    }
}