﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.ImportExport.Core;

namespace Telerik.Documents.SpreadsheetStreaming.Model.Relations
{
    internal class Relationships : IEnumerable<Relationship>
    {
        private static readonly string ThemeTargetFormat = "theme/{0}.xml";
        private static readonly string WorksheetTargetFormat = "worksheets/sheet{0}.xml";
        private static readonly string StylesTargetFormat = "{0}.xml";

        private readonly Dictionary<string, Relationship> relationships;

        internal Relationships(bool import = false)
        {
            this.relationships = new Dictionary<string, Relationship>();

            if (!import)
            {
                this.AddTheme("theme1");
                this.AddStyles("styles");
            }
        }

        public IEnumerable<Relationship> GetRelationships(string type)
        {
            return this.Where(relationship => relationship.Type == type);
        }

        public void AddRelationship(Relationship relationship)
        {
            Relationship relationshipWithTheSameId;
            if (this.relationships.TryGetValue(relationship.RelationshipId, out relationshipWithTheSameId))
            {
                relationshipWithTheSameId.MergeWith(relationship);

                return;
            }
            else
            {
                this.relationships.Add(relationship.RelationshipId, relationship);
            }
        }

        public int AddWorksheet(string targetName)
        {
            string worksheetRelationshipType = XlsxRelationshipTypes.WorksheetRelationshipType;

            int id = this.GetNextTypeId(worksheetRelationshipType);
            string target = string.Format(Relationships.WorksheetTargetFormat, id);

            this.AddRelationship(targetName, worksheetRelationshipType, target);

            return id;
        }

        public bool ContainsWorksheetName(string sheetName)
        {
            bool containsWorksheetName = this.Any(relationship => relationship.Type == XlsxRelationshipTypes.WorksheetRelationshipType && relationship.Name == sheetName);

            return containsWorksheetName;
        }

        public IEnumerator<Relationship> GetEnumerator()
        {
            return this.relationships.Values.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        private void AddTheme(string targetName)
        {
            string target = string.Format(Relationships.ThemeTargetFormat, targetName);

            this.AddRelationship(targetName, XlsxRelationshipTypes.ThemeRelationshipType, target);
        }

        private void AddStyles(string targetName)
        {
            string target = string.Format(Relationships.StylesTargetFormat, targetName);

            this.AddRelationship(targetName, XlsxRelationshipTypes.StylesRelationshipType, target);
        }

        private void AddRelationship(string targetName, string type, string target)
        {
            int relationshipId = this.GetNextRelationshipId();
            int id = this.GetNextTypeId(type);

            Relationship relationship = new Relationship(targetName, id, relationshipId, type, target);
            if (relationship.Type == XlsxRelationshipTypes.WorksheetRelationshipType)
            {
                relationship.Index = id;
            }

            this.relationships.Add(relationshipId.ToString(), relationship);
        }

        private int GetNextTypeId(string type)
        {
            return this.GetRelationships(type).Count() + 1;
        }

        private int GetNextRelationshipId()
        {
            return this.relationships.Count + 1;
        }
    }
}