﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Model
{
    internal class SpreadCellRange
    {
        public SpreadCellRange(int fromRowIndex, int fromColumnIndex, int toRowIndex, int toColumnIndex)
        {
            Guard.ThrowExceptionIfLessThan(0, fromRowIndex, "fromRowIndex");
            Guard.ThrowExceptionIfLessThan(0, fromColumnIndex, "fromColumnIndex");

            Guard.ThrowExceptionIfLessThan(fromRowIndex, toRowIndex, "toRowIndex");
            Guard.ThrowExceptionIfLessThan(fromColumnIndex, toColumnIndex, "toColumnIndex");

            this.FromRowIndex = fromRowIndex;
            this.FromColumnIndex = fromColumnIndex;
            this.ToRowIndex = toRowIndex;
            this.ToColumnIndex = toColumnIndex;
        }

        public int FromRowIndex { get; private set; }
        public int FromColumnIndex { get; private set; }
        public int ToRowIndex { get; private set; }
        public int ToColumnIndex { get; private set; }

        public override bool Equals(object obj)
        {
            SpreadCellRange other = obj as SpreadCellRange;

            if (other == null)
            {
                return false;
            }

            return this.FromRowIndex == other.FromRowIndex &&
                this.FromColumnIndex == other.FromColumnIndex &&
                this.ToRowIndex == other.ToRowIndex &&
                this.ToColumnIndex == other.ToColumnIndex;
        }

        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(
                this.FromRowIndex.GetHashCodeOrZero(),
                this.FromColumnIndex.GetHashCodeOrZero(),
                this.ToRowIndex.GetHashCodeOrZero(),
                this.ToColumnIndex.GetHashCodeOrZero());
        }

        internal bool ContainsCell(int rowIndex, int columnIndex)
        {
            if (this.FromRowIndex <= rowIndex && rowIndex <= this.ToRowIndex &&
                this.FromColumnIndex <= columnIndex && columnIndex <= this.ToColumnIndex)
            {
                return true;
            }

            return false;
        }
    }
}