﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Core;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Model.Themes
{
    internal class SpreadThemeFontScheme : NamedObjectBase
    {
        private readonly Dictionary<SpreadThemeFontType, SpreadThemeFonts> themeFontTypeToFont;

        private int? hashCode;

        public SpreadThemeFontScheme(string name,
            string latinMajorFontName = null,
            string latinMinorFontName = null,
            string eastAsianMajorFontName = null,
            string eastAsianMinorFontName = null,
            string complexScriptMajorFontName = null,
            string complexScriptMinorFontName = null)
            : base(name)
        {
            this.themeFontTypeToFont = new Dictionary<SpreadThemeFontType, SpreadThemeFonts>();
            this.themeFontTypeToFont.Add(SpreadThemeFontType.Major, new SpreadThemeFonts(latinMajorFontName, eastAsianMajorFontName, complexScriptMajorFontName));
            this.themeFontTypeToFont.Add(SpreadThemeFontType.Minor, new SpreadThemeFonts(latinMinorFontName, eastAsianMinorFontName, complexScriptMinorFontName));
        }

        public SpreadThemeFonts this[SpreadThemeFontType fontType]
        {
            get
            {
                return this.themeFontTypeToFont[fontType];
            }
        }

        public override bool Equals(object obj)
        {
            SpreadThemeFontScheme other = obj as SpreadThemeFontScheme;
            if (other == null || this.Name != other.Name)
            {
                return false;
            }

            foreach (var pair in this.themeFontTypeToFont)
            {
                if (!ObjectExtensions.EqualsOfT(pair.Value, other[pair.Key]))
                {
                    return false;
                }
            }

            return true;
        }

        public override int GetHashCode()
        {
            if (!this.hashCode.HasValue)
            {
                this.hashCode = this.Name.GetHashCode();

                foreach (var pair in this.themeFontTypeToFont)
                {
                    this.hashCode = ObjectExtensions.CombineHashCodes(hashCode.Value, pair.Value.GetHashCode());
                }
            }

            return this.hashCode.Value;
        }
    }
}