﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Model.Themes
{
    internal class SpreadThemeFonts
    {
        private readonly Dictionary<SpreadFontLanguageType, SpreadThemeFont> fontLanguageTypeToFont;

        public SpreadThemeFonts(string latinFontName = null, string eastAsianFontName = null, string complexScriptFontName = null)
        {
            this.fontLanguageTypeToFont = new Dictionary<SpreadFontLanguageType, SpreadThemeFont>();

            this.AddThemeFont(latinFontName, SpreadFontLanguageType.Latin);
            this.AddThemeFont(eastAsianFontName, SpreadFontLanguageType.EastAsian);
            this.AddThemeFont(complexScriptFontName, SpreadFontLanguageType.ComplexScript);
        }

        public SpreadThemeFont this[SpreadFontLanguageType fontLanguageType]
        {
            get
            {
                return this.fontLanguageTypeToFont[fontLanguageType];
            }
        }

        public override bool Equals(object obj)
        {
            SpreadThemeFonts other = obj as SpreadThemeFonts;
            foreach (var pair in this.fontLanguageTypeToFont)
            {
                if (!ObjectExtensions.EqualsOfT(pair.Value, other[pair.Key]))
                {
                    return false;
                }
            }

            return true;
        }

        public override int GetHashCode()
        {
            int hashCode = 0;
            foreach (var pair in this.fontLanguageTypeToFont)
            {
                hashCode = ObjectExtensions.CombineHashCodes(hashCode, pair.Value.GetHashCode());
            }

            return hashCode;
        }

        private void AddThemeFont(string fontName, SpreadFontLanguageType fontLanguageType)
        {
            SpreadThemeFont themeFont = null;
            if (!string.IsNullOrEmpty(fontName))
            {
                themeFont = new SpreadThemeFont(fontName, fontLanguageType);
            }

            this.fontLanguageTypeToFont.Add(fontLanguageType, themeFont);
        }
    }
}