﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Describes the settings which are used to determine how the border will appear in the document.
    /// </summary>
    public class SpreadBorder
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SpreadBorder" /> class.
        /// </summary>
        /// <param name="style">The border style.</param>
        /// <param name="color">The border color.</param>
        public SpreadBorder(SpreadBorderStyle style, SpreadThemableColor color)
        {
            Guard.ThrowExceptionIfNull(color, "color");

            this.Style = style;
            this.Color = color;
        }

        internal SpreadBorder()
        {
        }

        /// <summary>
        /// Gets the border style.
        /// </summary>
        /// <value>The border style.</value>
        public SpreadBorderStyle Style { get; internal set; }

        /// <summary>
        /// Gets the themable color.
        /// </summary>
        /// <value>The themable color.</value>
        public SpreadThemableColor Color { get; internal set; }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// Returns true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            SpreadBorder other = obj as SpreadBorder;

            if (other == null)
            {
                return false;
            }

            return ObjectExtensions.EqualsOfT(this.Style, other.Style) &&
                ObjectExtensions.EqualsOfT(this.Color, other.Color);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(
                this.Style.GetHashCodeOrZero(),
                this.Color.GetHashCodeOrZero());
        }
    }
}