﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Represents a cell style.
    /// </summary>
    public class SpreadCellStyle : SpreadCellFormatBase
    {
        private readonly string styleName;
        private readonly int? builtinId;

        internal SpreadCellStyle(string styleName, int? builtinId)
        {
            this.styleName = styleName;
            this.builtinId = builtinId;

            this.ApplyBorder = true;
            this.ApplyFill = true;
            this.ApplyFont = true;
            this.ApplyNumberFormat = true;
            this.ApplyAlignment = true;
            this.ApplyProtection = true;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the border properties should be applied.
        /// </summary>
        /// <value>True if the border properties should be applied; otherwise, false.</value>
        public bool ApplyBorder
        {
            get
            {
                return this.ApplyBorderInternal.Value;
            }
            set
            {
                this.ApplyBorderInternal = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the fill property should be applied.
        /// </summary>
        /// <value>True if the fill property should be applied; otherwise, false.</value>
        public bool ApplyFill
        {
            get
            {
                return this.ApplyFillInternal.Value;
            }
            set
            {
                this.ApplyFillInternal = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the font properties should be applied.
        /// </summary>
        /// <value>True if the font properties should be applied; otherwise, false.</value>
        public bool ApplyFont
        {
            get
            {
                return this.ApplyFontInternal.Value;
            }
            set
            {
                this.ApplyFontInternal = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the number format property should be applied.
        /// </summary>
        /// <value>True if the number format property should be applied; otherwise, false.</value>
        public bool ApplyNumberFormat
        {
            get
            {
                return this.ApplyNumberFormatInternal.Value;
            }
            set
            {
                this.ApplyNumberFormatInternal = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alignment properties should be applied.
        /// </summary>
        /// <value>True if the alignment properties should be applied; otherwise, false.</value>
        public bool ApplyAlignment
        {
            get
            {
                return this.ApplyAlignmentInternal.Value;
            }
            set
            {
                this.ApplyAlignmentInternal = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the protection property should be applied.
        /// </summary>
        /// <value>True if the protection property should be applied; otherwise, false.</value>
        public bool ApplyProtection
        {
            get
            {
                return this.ApplyProtectionInternal.Value;
            }
            set
            {
                this.ApplyProtectionInternal = value;
            }
        }

        /// <summary>
        /// Gets the name of the cell style.
        /// </summary>
        /// <value>The name of the cell style.</value>
        public string Name
        {
            get
            {
                return this.styleName;
            }
        }

        internal int? BuiltinId
        {
            get
            {
                return this.builtinId;
            }
        }
    }
}