﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Represents RGB color.
    /// </summary>
    public class SpreadColor
    {
        private readonly byte a;
        private readonly byte r;
        private readonly byte g;
        private readonly byte b;

        /// <summary>
        /// Initializes a new instance of the <see cref="SpreadColor" /> class.
        /// </summary>
        /// <param name="r">The red component value.</param>
        /// <param name="g">The green component value.</param>
        /// <param name="b">The blue component value.</param>
        public SpreadColor(byte r, byte g, byte b) : this(255, r, g, b)
        {
        }

        internal SpreadColor(byte a, byte r, byte g, byte b)
        {
            Guard.ThrowExceptionIfOutOfRange(0, 255, a, "a");
            Guard.ThrowExceptionIfOutOfRange(0, 255, r, "r");
            Guard.ThrowExceptionIfOutOfRange(0, 255, g, "g");
            Guard.ThrowExceptionIfOutOfRange(0, 255, b, "b");

            this.a = a;
            this.r = r;
            this.g = g;
            this.b = b;
        }

        /// <summary>
        /// Gets the green component value.
        /// </summary>
        /// <value>The green component value.</value>
        public byte G
        {
            get
            {
                return this.g;
            }
        }

        /// <summary>
        /// Gets the red component value.
        /// </summary>
        /// <value>The red component value.</value>
        public byte R
        {
            get
            {
                return this.r;
            }
        }

        /// <summary>
        /// Gets the blue component value.
        /// </summary>
        /// <value>The blue component value.</value>
        public byte B
        {
            get
            {
                return this.b;
            }
        }

        internal byte A
        {
            get
            {
                return this.a;
            }
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// Returns true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            SpreadColor other = obj as SpreadColor;

            if (other == null)
            {
                return false;
            }

            return this.A == other.A &&
                this.R == other.R &&
                this.G == other.G &&
                this.B == other.B;
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(this.A, this.R, this.G, this.B);
        }
    }
}