﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Describes a gradient type of fill.
    /// </summary>
    public class SpreadGradientFill : ISpreadFill
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SpreadGradientFill" /> class.
        /// </summary>
        /// <param name="gradientType">Type of the gradient.</param>
        /// <param name="color1">The color1 themable color.</param>
        /// <param name="color2">The color2 themable color.</param>
        public SpreadGradientFill(SpreadGradientType gradientType, SpreadThemableColor color1, SpreadThemableColor color2)
        {
            Guard.ThrowExceptionIfNull(color1, "color1");
            Guard.ThrowExceptionIfNull(color2, "color2");

            this.GradientType = gradientType;
            this.Color1 = color1;
            this.Color2 = color2;
        }

        /// <summary>
        /// Gets the color1 themable color.
        /// </summary>
        /// <value>The color1 themable color.</value>
        public SpreadThemableColor Color1 { get; private set; }

        /// <summary>
        /// Gets the color2 themable color.
        /// </summary>
        /// <value>The color2 themable color.</value>
        public SpreadThemableColor Color2 { get; private set; }

        /// <summary>
        /// Gets the type of the gradient.
        /// </summary>
        /// <value>The type of the gradient.</value>
        public SpreadGradientType GradientType { get; private set; }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object" /> is equal
        /// to the current <see cref="T:System.Object" />.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object" /> to compare with the current
        /// <see cref="T:System.Object" />.</param>
        /// <returns>
        /// Returns true if the specified <see cref="T:System.Object" /> is equal to the
        /// current <see cref="T:System.Object" />; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            SpreadGradientFill other = obj as SpreadGradientFill;

            if (other == null)
            {
                return false;
            }

            return ObjectExtensions.EqualsOfT(this.Color1, other.Color1) &&
                ObjectExtensions.EqualsOfT(this.Color2, other.Color2) &&
                ObjectExtensions.EqualsOfT(this.GradientType, other.GradientType);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object" />.</returns>
        public override int GetHashCode()
        {
            return ObjectExtensions.CombineHashCodes(
                this.Color1.GetHashCodeOrZero(),
                this.Color2.GetHashCodeOrZero(),
                this.GradientType.GetHashCodeOrZero());
        }
    }
}