﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming
{
    /// <summary>
    /// Describes the possible types of horizontal alignments.
    /// </summary>
    public enum SpreadHorizontalAlignment
    {
        /// <summary>
        /// General horizontal alignment. Text data is left-aligned. Numbers, dates, and times are right-aligned. Boolean types are centered. 
        /// Changing the alignment does not change the type of data.
        /// </summary>
        General,

        /// <summary>
        /// Specifies that the text should align with the left edge of the cell.
        /// </summary>
        Left,

        /// <summary>
        /// Specifies that the text should be centered across the cell.
        /// </summary>
        Center,

        /// <summary>
        /// Specifies that the text should align with the right edge of the cell.
        /// </summary>
        Right,

        /// <summary>
        /// Specifies that the text should be justified between the left and right edges of the cell except the last line. 
        /// </summary>
        Justify,

        /// <summary>
        /// Specifies that the value of the cell should be filled across the entire width of the cell.
        /// </summary>
        Fill,

        /// <summary>
        /// Specifies that the text should be centered across multiple cells.
        /// </summary>
        CenterContinuous,

        /// <summary>
        /// Specifies that the text should be evenly distributed between the left and right edges of the cell. 
        /// </summary>
        Distributed
    }
}