﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using Telerik.Documents.SpreadsheetStreaming.Model;

#if NETSTANDARD
using Telerik.Zip;
#else
using Telerik.Windows.Zip;
#endif

namespace Telerik.Documents.SpreadsheetStreaming.Utilities
{
    internal static class ResourcesHelper
    {
        private static readonly Dictionary<string, string> resources;

        static ResourcesHelper()
        {
#if  NETSTANDARD
            Assembly assembly = typeof(ResourcesHelper).GetTypeInfo().Assembly;
#else
            Assembly assembly = Assembly.GetExecutingAssembly();
#endif
            string assemblyName = assembly.FullName;
            int indexOfFirstSpace = assemblyName.IndexOf(',');
            assemblyName = assemblyName.Substring(0, indexOfFirstSpace);

            resources = new Dictionary<string, string>();
            resources.Add(assemblyName + ".Resources.xlsxContents.xl.theme.theme1.xml", "xl/theme/theme1.xml");
            resources.Add(assemblyName + ".Resources.xlsxContents._rels..rels", "_rels/.rels");
        }

        internal static void CopyResourcesToZipArchive(ZipArchive archive, ContentTypesRepository contentTypes)
        {
            contentTypes.Register("xl/theme/theme1.xml", "application/vnd.openxmlformats-officedocument.theme+xml");

            foreach (KeyValuePair<string, string> pair in resources)
            {
                if (archive.Entries.Where(p => p.FullName == pair.Value).Any())
                {
                    continue;
                }

#if NETSTANDARD
                Assembly assembly = typeof(ResourcesHelper).GetTypeInfo().Assembly;
#else
                Assembly assembly = Assembly.GetExecutingAssembly();
#endif
                using (ZipArchiveEntry entry = archive.CreateEntry(pair.Value))
                using (Stream entryStream = entry.Open())
                using (Stream resourceStream = assembly.GetManifestResourceStream(pair.Key))
                {
                    resourceStream.CopyTo(entryStream);
                }
            }
        }
    }
}