﻿using System;
using System.Linq;

namespace Telerik.Documents.SpreadsheetStreaming.Utilities
{
    internal static class UnitHelper
    {
        private static readonly double PointToInch = 72.0;
        private static readonly double DPI = 96.0;

        public static double PointToDip(double value)
        {
            return (value * UnitHelper.DPI) / UnitHelper.PointToInch;
        }

        public static double DipToPoint(double value)
        {
            return (value * UnitHelper.PointToInch) / UnitHelper.DPI;
        }

        public static double ExcelColumnWidthToPixelWidth(double width)
        {
            double maxDigitWidth = Math.Floor(DefaultValues.MaxDigitWidthInNormalStyle);
            return Math.Floor((((256 * width) + Math.Floor(128 / maxDigitWidth)) / 256) * maxDigitWidth);
        }

        public static double PixelWidthToExcelColumnWidth(double pixels)
        {
            // the Math.Floor is called if the width is returned from measuring
            double maxDigitWidth = Math.Floor(DefaultValues.MaxDigitWidthInNormalStyle);
            double charCount = PixelsToCharacterCount(pixels);
            return CharacterCountToExcelColumnWidth(charCount, maxDigitWidth);
        }

        public static double CharactersCountToPixelWidth(double width)
        {
            // the Math.Floor is called if the width is returned from measuring
            double maxDigitWidth = Math.Floor(DefaultValues.MaxDigitWidthInNormalStyle);
            return Math.Floor((((256 * width) + Math.Floor(128 / maxDigitWidth)) / 256) * maxDigitWidth);
        }

        internal static double PixelsToCharacterCount(double pixels)
        {
            double maxDigitWidth = Math.Floor(DefaultValues.MaxDigitWidthInNormalStyle);
            double marginPixels = DefaultValues.LeftCellMargin + DefaultValues.RightCellMargin;
            return Math.Floor((((pixels - marginPixels) / maxDigitWidth) * 100) + 0.5) / 100;
        }

        private static double CharacterCountToExcelColumnWidth(double charCount, double maxDigitWidth)
        {
            double marginPixels = DefaultValues.LeftCellMargin + DefaultValues.RightCellMargin;
            return Math.Floor(((charCount * maxDigitWidth) + marginPixels) / maxDigitWidth * 256) / 256;
        }
    }
}