﻿using Microsoft.Office.Interop.Excel;
using System.Collections.Generic;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class BorderWrapper
    {
        private readonly Border instance;
        private static readonly Dictionary<int, Dictionary<int, SpreadBorderStyle>> weightToMapper;

        static BorderWrapper()
        {
            weightToMapper = new Dictionary<int, Dictionary<int, SpreadBorderStyle>>();

            Dictionary<int, SpreadBorderStyle> hairlinelineStyleIndexToCellBorderStyle = new Dictionary<int, SpreadBorderStyle>();
            weightToMapper.Add((int)XlBorderWeight.xlHairline, hairlinelineStyleIndexToCellBorderStyle);
            hairlinelineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlLineStyleNone, SpreadBorderStyle.None);
            hairlinelineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlContinuous, SpreadBorderStyle.Hair);

            Dictionary<int, SpreadBorderStyle> thinlineStyleIndexToCellBorderStyle = new Dictionary<int, SpreadBorderStyle>();
            weightToMapper.Add((int)XlBorderWeight.xlThin, thinlineStyleIndexToCellBorderStyle);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlLineStyleNone, SpreadBorderStyle.None);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDot, SpreadBorderStyle.Dotted);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDashDotDot, SpreadBorderStyle.DashDotDot);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDashDot, SpreadBorderStyle.DashDot);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDash, SpreadBorderStyle.Dashed);
            thinlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlContinuous, SpreadBorderStyle.Thin);

            Dictionary<int, SpreadBorderStyle> mediumlineStyleIndexToCellBorderStyle = new Dictionary<int, SpreadBorderStyle>();
            weightToMapper.Add((int)XlBorderWeight.xlMedium, mediumlineStyleIndexToCellBorderStyle);
            mediumlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDashDotDot, SpreadBorderStyle.MediumDashDotDot);
            mediumlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDashDot, SpreadBorderStyle.MediumDashDot);
            mediumlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDash, SpreadBorderStyle.MediumDashed);
            mediumlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlContinuous, SpreadBorderStyle.Medium);
            mediumlineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlSlantDashDot, SpreadBorderStyle.SlantDashDot);

            Dictionary<int, SpreadBorderStyle> thicklineStyleIndexToCellBorderStyle = new Dictionary<int, SpreadBorderStyle>();
            weightToMapper.Add((int)XlBorderWeight.xlThick, thicklineStyleIndexToCellBorderStyle);
            thicklineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlDouble, SpreadBorderStyle.Double);
            thicklineStyleIndexToCellBorderStyle.Add((int)XlLineStyle.xlContinuous, SpreadBorderStyle.Thick);
        }

        public BorderWrapper(Border instance)
        {
            this.instance = instance;
        }

        public bool HasDefaultLineStyle
        {
            get
            {
                System.DBNull value = this.instance.LineStyle as System.DBNull;
                if (value != null)
                {
                    return true;
                }

                return weightToMapper[(int)XlBorderWeight.xlThin][(int)this.instance.LineStyle] == TestsHelper.DefaultBorderLineStyle;
            }
        }

        public SpreadBorderStyle Style
        {
            get
            {
                System.DBNull value = this.instance.Weight as System.DBNull;
                if (value != null)
                {
                    return SpreadBorderStyle.None;
                }

                int weight = this.instance.Weight;
                return weightToMapper[weight][this.instance.LineStyle];
            }
        }

        public SpreadThemableColor Color
        {
            get
            {
                return ExcelInteropTestsHelper.ExcelColorToThemableColor(this.instance.Color);
            }
        }

        public SpreadThemableColor ColorIndex
        {
            get
            {
                return ExcelInteropTestsHelper.ExcelColorToThemableColor(this.instance.ColorIndex);
            }
        }
    }
}