﻿using Microsoft.Office.Interop.Excel;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class CellStyleWrapper
    {
        private readonly Style style;
        private readonly BordersWrapper bordersWrapper;

        public CellStyleWrapper(Style style)
        {
            this.style = style;
            this.bordersWrapper = new BordersWrapper(style.Borders);
        }

        public bool ApplyAlignment
        {
            get
            {
                return this.style.IncludeAlignment;
            }
        }

        public bool ApplyFont
        {
            get
            {
                return this.style.IncludeFont;
            }
        }

        public bool ApplyNumberFormat
        {
            get
            {
                return this.style.IncludeNumber;
            }
        }

        public BorderWrapper InsideHorizontal
        {
            get
            {
                return this.bordersWrapper.InsideHorizontal;
            }
        }

        public BorderWrapper InsideVertical
        {
            get
            {
                return this.bordersWrapper.InsideVertical;
            }
        }

        public BorderWrapper Bottom
        {
            get
            {
                return this.bordersWrapper.Bottom;
            }
        }

        public BorderWrapper DiagonalDown
        {
            get
            {
                return this.bordersWrapper.DiagonalDown;
            }
        }

        public BorderWrapper DiagonalUp
        {
            get
            {
                return this.bordersWrapper.DiagonalUp;
            }
        }

        public SpreadPatternFill Fill
        {
            get
            {
                return this.style.Interior.ToPatternFill();
            }
        }

        public SpreadThemableFontFamily FontFamily
        {
            get
            {
                return this.style.Font.ToThemeFontFamily();
            }
        }

        public double FontSize
        {
            get
            {
                return this.style.Font.Size;
            }
        }

        public SpreadThemableColor ForeColor
        {
            get
            {
                SpreadThemableColor themableColor;
                try
                {
                    XlThemeColor color = (XlThemeColor)this.style.Font.ThemeColor;
                    double titntAndShade = this.style.Font.TintAndShade;
                    themableColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(color, titntAndShade);
                }
                catch
                {
                    themableColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(this.style.Font.Color);
                }

                if (themableColor == null)
                {
                    themableColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(this.style.Font.Color);
                }

                return themableColor;
            }
        }

        public SpreadHorizontalAlignment HorizontalAlignment
        {
            get
            {
                return this.style.HorizontalAlignment.ToHorizontalAlignment();
            }
        }

        public bool ApplyBorder
        {
            get
            {
                return this.style.IncludeBorder;
            }
        }

        public bool ApplyFill
        {
            get
            {
                return this.style.IncludePatterns;
            }
        }

        public bool ApplyProtection
        {
            get
            {
                return this.style.IncludeProtection;
            }
        }

        public int Indent
        {
            get
            {
                try
                {
                    return style.IndentLevel;
                }
                catch
                {
                    return 0;
                }
            }
        }

        public bool IsBold
        {
            get
            {
                return this.style.Font.Bold;
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.style.Font.Italic;
            }
        }

        public BorderWrapper LeftBorder
        {
            get
            {
                return this.bordersWrapper.Left;
            }
        }

        public string Name
        {
            get
            {
                return this.style.Name;
            }
        }

        public string NumberFormat
        {
            get
            {
                return this.style.NumberFormat == "General" ? string.Empty : this.style.NumberFormat;
            }
        }

        public BorderWrapper RightBorder
        {
            get
            {
                return this.bordersWrapper.Right;
            }
        }

        public BorderWrapper TopBorder
        {
            get
            {
                return this.bordersWrapper.Top;
            }
        }

        public SpreadUnderlineType Underline
        {
            get
            {
                return ((XlUnderlineStyle)this.style.Font.Underline).ToUnderlineType();
            }
        }

        public SpreadVerticalAlignment VerticalAlignment
        {
            get
            {
                return this.style.VerticalAlignment.ToVerticalAlignment();
            }
        }

        public bool WrapText
        {
            get
            {
                return this.style.WrapText;
            }
        }
    }
}