﻿using Microsoft.Office.Interop.Excel;
using System.Collections;
using System.Collections.Generic;
using Telerik.Documents.SpreadsheetStreaming.Tests.CommonWrappers;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class MergedCellsWrapper : IEnumerable<CellsWrapper>
    {
        private readonly Worksheet instance;

        public MergedCellsWrapper(Worksheet instance)
        {
            this.instance = instance;
        }

        private IEnumerable<CellsWrapper> GetMergedCells()
        {
            HashSet<CellRangeWrapper> knownMergedCells = new HashSet<CellRangeWrapper>();
            foreach (Range range in instance.UsedRange)
            {
                if (range.MergeCells)
                {
                    Range mergeArea = range.MergeArea;

                    CellRangeWrapper mergedCellRange = mergeArea.ToCellRange();

                    if (!knownMergedCells.Contains(mergedCellRange))
                    {
                        knownMergedCells.Add(mergedCellRange);

                        yield return new CellsWrapper(mergeArea);
                    }
                }
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        public IEnumerator<CellsWrapper> GetEnumerator()
        {
            return this.GetMergedCells().GetEnumerator();
        }
    }
}