﻿using Microsoft.Office.Interop.Excel;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class TextPropertiesWrapper
    {
        private readonly Range instance;

        public TextPropertiesWrapper(Range instance)
        {
            this.instance = instance;
        }

        public double FontSize
        {
            get
            {
                return this.instance.Font.Size;
            }
        }

        public SpreadThemableColor ForeColor
        {
            get
            {
                return ExcelInteropTestsHelper.ExcelColorToThemableColor(this.instance.Font.Color);
            }
        }

        public bool IsBold
        {
            get
            {
                return this.instance.Font.Bold;
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.instance.Font.Italic;
            }
        }

        public SpreadUnderlineType Underline
        {
            get
            {
                return ((XlUnderlineStyle)this.instance.Font.Underline).ToUnderlineType();
            }
        }

        public SpreadThemableFontFamily FontFamily
        {
            get
            {
                return this.instance.Font.ToThemeFontFamily();
            }
        }

        public SpreadHorizontalAlignment HorizontalAlignment
        {
            get
            {
                return ((XlHAlign)this.instance.HorizontalAlignment).ToHorizontalAlignment();
            }
        }

        public SpreadVerticalAlignment VerticalAlignment
        {
            get
            {
                return ((XlVAlign)this.instance.VerticalAlignment).ToVerticalAlignment();
            }
        }

        public int Indent
        {
            get
            {
                return this.instance.IndentLevel;
            }
        }

        public bool WrapText
        {
            get
            {
                return this.instance.WrapText;
            }
        }

        public bool HasDefaultFontFamily
        {
            get
            {
                return this.FontFamily.Equals(TestsHelper.DefaultFontFamily);
            }
        }

        public bool HasDefaultFontSize
        {
            get
            {
                return this.FontSize.Equals(TestsHelper.DefaultFontSize);
            }
        }

        public bool HasDefaultForeColor
        {
            get
            {
                return this.ForeColor.Equals(TestsHelper.DefaultBlack);
            }
        }

        public bool HasDefaultHorizontalAlignment
        {
            get
            {
                return this.HorizontalAlignment.Equals(TestsHelper.DefaultHorizontalAlignment);
            }
        }

        public bool HasDefaultIndent
        {
            get
            {
                return this.Indent.Equals(TestsHelper.DefaultIndent);
            }
        }

        public bool HasDefaultIsBold
        {
            get
            {
                return this.IsBold.Equals(TestsHelper.DefaultIsBold);
            }
        }

        public bool HasDefaultIsItalic
        {
            get
            {
                return this.IsItalic.Equals(TestsHelper.DefaultIsItalic);
            }
        }

        public bool HasDefaultUnderline
        {
            get
            {
                return this.Underline.Equals(TestsHelper.DefaultUnderline);
            }
        }

        public bool HasDefaultVerticalAlignment
        {
            get
            {
                return this.VerticalAlignment.Equals(TestsHelper.DefaultVerticalAlignment);
            }
        }

        public bool HasDefaultWrapText
        {
            get
            {
                return this.WrapText.Equals(TestsHelper.DefaultWrapText);
            }
        }

        public bool HasDefaultValues
        {
            get
            {
                if (!this.HasDefaultFontFamily)
                {
                    return false;
                }

                if (!this.HasDefaultFontSize)
                {
                    return false;
                }

                if (!this.HasDefaultForeColor)
                {
                    return false;
                }

                if (!this.HasDefaultHorizontalAlignment)
                {
                    return false;
                }

                if (!this.HasDefaultIndent)
                {
                    return false;
                }

                if (!this.HasDefaultIsBold)
                {
                    return false;
                }

                if (!this.HasDefaultIsItalic)
                {
                    return false;
                }

                if (!this.HasDefaultUnderline)
                {
                    return false;
                }

                if (!this.HasDefaultVerticalAlignment)
                {
                    return false;
                }

                if (!this.HasDefaultWrapText)
                {
                    return false;
                }

                return true;
            }
        }
    }
}