﻿using Microsoft.Office.Interop.Excel;
using System.Collections.Generic;
using Telerik.Documents.SpreadsheetStreaming.Tests.CommonWrappers;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class ViewStateWrapper
    {
        private Worksheet worksheet;

        public ViewStateWrapper(Worksheet worksheet)
        {
            this.worksheet = worksheet;
        }

        public int ActiveSelectionCellRowIndex
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.ActiveCell.Row - 1;
            }
        }

        public int ActiveSelectionCellColumnIndex
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.ActiveCell.Column - 1;
            }
        }

        public double ScaleFactor
        {
            get
            {
                return worksheet.Application.ActiveWindow.Zoom / 100;
            }
        }

        internal IEnumerable<CellRangeWrapper> SelectedRanges
        {
            get
            {
                return this.GetSelectedRanges();
            }
        }

        private IEnumerable<CellRangeWrapper> GetSelectedRanges()
        {
            Range selectionRange = worksheet.Application.Selection;

            if (selectionRange == null)
            {
                yield break;
            }

            foreach (Range range in selectionRange.Areas)
            {
                yield return range.ToCellRange();
            }
        }

        public bool ShouldShowGridLines
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.DisplayGridlines;
            }
        }

        public int SplitRowOffset
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.SplitRow;
            }
        }

        public int SplitColumnOffset
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.SplitColumn;
            }
        }

        internal CellRangeWrapper TopLeftCellIndex
        {
            get
            {
                var topLeftPane = this.worksheet.Application.ActiveWindow.Panes[1];

                int row = topLeftPane.ScrollRow - 1;
                int column = topLeftPane.ScrollColumn - 1;

                return new CellRangeWrapper(row, column, row, column);
            }
        }

        public BottomRightPaneWrapper BottomRightPane
        {
            get
            {
                return new BottomRightPaneWrapper(this.worksheet);
            }
        }

        public bool ShouldShowRowColumnHeaders
        {
            get
            {
                return this.worksheet.Application.ActiveWindow.DisplayHeadings;
            }
        }
    }
}