﻿using Microsoft.Office.Interop.Excel;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class WorkbookWrapper : IDisposable
    {
        private readonly Workbook instance;
        private readonly string fileName;

        private readonly WorksheetsWrapper worksheets;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="action"> This param is not using in ExcelInterop tests it is only for compatibily with Spreadsheet api tests</param>
        /// <returns></returns>
        public static WorkbookWrapper Open(Stream stream, SpreadDocumentFormat documentFormat)
        {
            string fileExtenssion = documentFormat == SpreadDocumentFormat.Csv ? "csv" : "xlsx";

            var fileName = ExcelInteropTestsHelper.GetTestFileName(fileExtenssion);

            Microsoft.Office.Interop.Excel.Application xlApp = new Microsoft.Office.Interop.Excel.Application();
            Microsoft.Office.Interop.Excel.Workbook excelWorkbook = xlApp.Open(stream, fileName);

            return new WorkbookWrapper(excelWorkbook, fileName);
        }

        public WorksheetsWrapper Worksheets
        {
            get
            {
                return this.worksheets;
            }
        }

        public CellStylesCollectionWrapper CellStyles
        {
            get
            {
                return new CellStylesCollectionWrapper(this.instance);
            }
        }

        private WorkbookWrapper(Workbook workbook, string fileName)
        {
            this.instance = workbook;
            this.fileName = fileName;
            this.worksheets = new WorksheetsWrapper(this.instance.Worksheets);
        }

        public void Dispose()
        {
            this.instance.Close(false);
            var processes = Process.GetProcesses().Where(p => p.ProcessName.ToLower() == "excel");
            foreach (var process in processes)
            {
                try
                {
                    process.Kill();
                }
                catch
                {
                }
            }

            File.Delete(this.fileName);
        }
    }
}