﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class CellStyleWrapper
    {
        private Telerik.Windows.Documents.Spreadsheet.PropertySystem.CellStyle style;

        public CellStyleWrapper(Telerik.Windows.Documents.Spreadsheet.PropertySystem.CellStyle style)
        {
            this.style = style;
        }

        public bool ApplyAlignment
        {
            get
            {
                return this.style.IncludeAlignment;
            }
        }

        public bool ApplyFont
        {
            get
            {
                return this.style.IncludeFont;
            }
        }

        public bool ApplyNumberFormat
        {
            get
            {
                return this.style.IncludeNumber;
            }
        }

        public BorderWrapper Bottom
        {
            get
            {
                return new BorderWrapper(this.style.BottomBorder);
            }
        }

        public BorderWrapper DiagonalDown
        {
            get
            {
                return new BorderWrapper(this.style.DiagonalDownBorder);
            }
        }

        public BorderWrapper DiagonalUp
        {
            get
            {
                return new BorderWrapper(this.style.DiagonalUpBorder);
            }
        }

        public BorderWrapper LeftBorder
        {
            get
            {
                return new BorderWrapper(this.style.LeftBorder);
            }
        }

        public BorderWrapper RightBorder
        {
            get
            {
                return new BorderWrapper(this.style.RightBorder);
            }
        }

        public BorderWrapper TopBorder
        {
            get
            {
                return new BorderWrapper(this.style.TopBorder);
            }
        }

        public SpreadPatternFill Fill
        {
            get
            {
                return this.style.Fill.SpreadFillToPatternFill();
            }
        }

        public SpreadThemableFontFamily FontFamily
        {
            get
            {
                var fontFamily = this.style.FontFamily;
                return fontFamily.SpreadFontFamilyToThemableFontFamily(this.style.Workbook.Theme);
            }
        }

        public double FontSize
        {
            get
            {
                return UnitHelper.DipToPoint(this.style.FontSize);
            }
        }

        public SpreadThemableColor ForeColor
        {
            get
            {
                return this.style.ForeColor.SpreadColorToThemableColor();
            }
        }

        public SpreadHorizontalAlignment HorizontalAlignment
        {
            get
            {
                return this.style.HorizontalAlignment.SpreadHorizontalAlignmentToHorizontalAlignment();
            }
        }

        public bool ApplyBorder
        {
            get
            {
                return this.style.IncludeBorder;
            }
        }

        public bool ApplyFill
        {
            get
            {
                return this.style.IncludeFill;
            }
        }

        public bool ApplyProtection
        {
            get
            {
                return this.style.IncludeProtection;
            }
        }

        public int Indent
        {
            get
            {
                return this.style.Indent;
            }
        }

        public bool IsBold
        {
            get
            {
                return this.style.IsBold;
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.style.IsItalic;
            }
        }

        public string Name
        {
            get
            {
                return this.style.Name;
            }
        }

        public string NumberFormat
        {
            get
            {
                return this.style.Format.FormatString;
            }
        }

        public SpreadUnderlineType Underline
        {
            get
            {
                return this.style.Underline.SpreadUnderlineTypeToUnderlineType();
            }
        }

        public SpreadVerticalAlignment VerticalAlignment
        {
            get
            {
                return this.style.VerticalAlignment.SpreadVerticalAlignmentToVerticalAlignment();
            }
        }

        public bool WrapText
        {
            get
            {
                return this.style.IsWrapped;
            }
        }
    }
}