﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.PropertySystem;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class CellWrapper
    {
        private readonly CellSelection instance;

        static CellWrapper()
        {
        }

        public CellWrapper(CellSelection cell)
        {
            this.instance = cell;
        }

        public string Value
        {
            get
            {
                ICellValue value = this.instance.GetValue().Value;
                return value.GetResultValueAsString(CellValueFormat.GeneralFormat);
            }
        }

        public BordersWrapper Borders
        {
            get
            {
                return new BordersWrapper(this.instance.GetBorders());
            }
        }

        public SpreadPatternFill PatternFill
        {
            get
            {
                PatternFill fill = this.instance.GetFill().Value as PatternFill;

                return fill.SpreadFillToPatternFill();
            }
        }

        public string NumberFormat
        {
            get
            {
                string format = this.instance.GetFormat().Value.FormatString;

                if (string.IsNullOrEmpty(format))
                {
                    return null;
                }

                return format;
            }
        }

        public TextPropertiesWrapper TextProperties
        {
            get
            {
                return new TextPropertiesWrapper(this.instance);
            }
        }

        public CellStyleWrapper CellStyle
        {
            get
            {
                string styleName = this.instance.GetStyleName().Value;
                CellStyle cellStyle = this.instance.Worksheet.Workbook.Styles.GetByName(styleName);

                return new CellStyleWrapper(cellStyle);
            }
        }
    }
}