﻿using System;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class TextPropertiesWrapper
    {
        private readonly CellSelection instance;

        public TextPropertiesWrapper(CellSelection instance)
        {
            this.instance = instance;
        }

        public double FontSize
        {
            get
            {
                return UnitHelper.DipToPoint(this.instance.GetFontSize().Value);
            }
        }

        public SpreadThemableColor ForeColor
        {
            get
            {
                var foreColor = this.instance.GetForeColor().Value.GetActualValue(this.instance.Worksheet.Workbook.Theme);
                return new SpreadThemableColor(
                    new SpreadColor(foreColor.R, foreColor.G, foreColor.B));
            }
        }

        public bool IsBold
        {
            get
            {
                return this.instance.GetIsBold().Value;
            }
        }

        public bool IsItalic
        {
            get
            {
                return this.instance.GetIsItalic().Value;
            }
        }

        public SpreadUnderlineType Underline
        {
            get
            {
                return this.instance.GetUnderline().Value.SpreadUnderlineTypeToUnderlineType();
            }
        }

        public SpreadThemableFontFamily FontFamily
        {
            get
            {
                ThemableFontFamily fontFamily = this.instance.GetFontFamily().Value;
                return fontFamily.SpreadFontFamilyToThemableFontFamily(this.instance.Worksheet.Workbook.Theme);
            }
        }

        public SpreadHorizontalAlignment HorizontalAlignment
        {
            get
            {
                return this.instance.GetHorizontalAlignment().Value.SpreadHorizontalAlignmentToHorizontalAlignment();
            }
        }

        public SpreadVerticalAlignment VerticalAlignment
        {
            get
            {
                return this.instance.GetVerticalAlignment().Value.SpreadVerticalAlignmentToVerticalAlignment();
            }
        }

        public int Indent
        {
            get
            {
                return this.instance.GetIndent().Value;
            }
        }

        public bool WrapText
        {
            get
            {
                return this.instance.GetIsWrapped().Value;
            }
        }

        public bool HasDefaultFontFamily
        {
            get
            {
                return this.FontFamily.Equals(TestsHelper.DefaultFontFamily);
            }
        }

        public bool HasDefaultFontSize
        {
            get
            {
                return this.FontSize.Equals(TestsHelper.DefaultFontSize);
            }
        }

        public bool HasDefaultForeColor
        {
            get
            {
                return this.ForeColor.Equals(TestsHelper.DefaultBlack);
            }
        }

        public bool HasDefaultHorizontalAlignment
        {
            get
            {
                return this.HorizontalAlignment.Equals(TestsHelper.DefaultHorizontalAlignment);
            }
        }

        public bool HasDefaultIndent
        {
            get
            {
                return this.Indent.Equals(TestsHelper.DefaultIndent);
            }
        }

        public bool HasDefaultIsBold
        {
            get
            {
                return this.IsBold.Equals(TestsHelper.DefaultIsBold);
            }
        }

        public bool HasDefaultIsItalic
        {
            get
            {
                return this.IsItalic.Equals(TestsHelper.DefaultIsItalic);
            }
        }

        public bool HasDefaultUnderline
        {
            get
            {
                return this.Underline.Equals(TestsHelper.DefaultUnderline);
            }
        }

        public bool HasDefaultVerticalAlignment
        {
            get
            {
                return this.VerticalAlignment.Equals(TestsHelper.DefaultVerticalAlignment);
            }
        }

        public bool HasDefaultWrapText
        {
            get
            {
                return this.WrapText.Equals(TestsHelper.DefaultWrapText);
            }
        }

        public bool HasDefaultValues
        {
            get
            {
                if (!this.HasDefaultFontFamily)
                {
                    return false;
                }

                if (!this.HasDefaultFontSize)
                {
                    return false;
                }

                if (!this.HasDefaultForeColor)
                {
                    return false;
                }

                if (!this.HasDefaultHorizontalAlignment)
                {
                    return false;
                }

                if (!this.HasDefaultIndent)
                {
                    return false;
                }

                if (!this.HasDefaultIsBold)
                {
                    return false;
                }

                if (!this.HasDefaultIsItalic)
                {
                    return false;
                }

                if (!this.HasDefaultUnderline)
                {
                    return false;
                }

                if (!this.HasDefaultVerticalAlignment)
                {
                    return false;
                }

                if (!this.HasDefaultWrapText)
                {
                    return false;
                }

                return true;
            }
        }
    }
}