﻿using System.Collections.Generic;
using Telerik.Documents.SpreadsheetStreaming.Tests.CommonWrappers;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class ViewStateWrapper
    {
        private WorksheetViewState worksheetViewState;

        public ViewStateWrapper(WorksheetViewState worksheetViewState)
        {
            this.worksheetViewState = worksheetViewState;
        }

        public int ActiveSelectionCellRowIndex
        {
            get
            {
                return this.worksheetViewState.SelectionState.ActiveCellIndex.RowIndex;
            }
        }

        public int ActiveSelectionCellColumnIndex
        {
            get
            {
                return this.worksheetViewState.SelectionState.ActiveCellIndex.ColumnIndex;
            }
        }

        public double ScaleFactor
        {
            get
            {
                return this.worksheetViewState.ScaleFactor.Width;
            }
        }

        internal IEnumerable<CellRangeWrapper> SelectedRanges
        {
            get
            {
                return this.GetSpreadSelectedRanges(this.worksheetViewState.SelectionState);
            }
        }

        public bool ShouldShowGridLines
        {
            get
            {
                return this.worksheetViewState.ShowGridLines;
            }
        }

        internal CellRangeWrapper TopLeftCellIndex
        {
            get
            {
                var cellIndex = this.worksheetViewState.TopLeftCellIndex;
                return new CellRangeWrapper(cellIndex.RowIndex, cellIndex.ColumnIndex, cellIndex.RowIndex, cellIndex.ColumnIndex);
            }
        }

        public BottomRightPaneWrapper BottomRightPane
        {
            get
            {
                return new BottomRightPaneWrapper(this.worksheetViewState);
            }
        }

        private IEnumerable<CellRangeWrapper> GetSpreadSelectedRanges(SelectionState selectionState)
        {
            foreach (var range in selectionState.SelectedRanges)
            {
                yield return new CellRangeWrapper(range.FromIndex.RowIndex, range.FromIndex.ColumnIndex, range.ToIndex.RowIndex, range.ToIndex.ColumnIndex);
            }
        }

        public int SplitRowOffset
        {
            get
            {
                return this.worksheetViewState.Pane.YSplit;
            }
        }

        public int SplitColumnOffset
        {
            get
            {
                return this.worksheetViewState.Pane.XSplit;
            }
        }

        public bool ShouldShowRowColumnHeaders
        {
            get
            {
                return this.worksheetViewState.ShowRowColHeaders;
            }
        }
    }
}