﻿using System;
using System.IO;
using System.Linq;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Csv;
using Telerik.Windows.Documents.Spreadsheet.Model;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers
{
    public class WorkbookWrapper : IDisposable
    {
        private readonly Workbook instance;

        private readonly WorksheetsWrapper worksheets;

        public static WorkbookWrapper Open(Stream stream, SpreadDocumentFormat documentFormat)
        {
            IWorkbookFormatProvider formatProvider;

            switch (documentFormat)
            {
                case SpreadDocumentFormat.Xlsx:
                    formatProvider = new XlsxFormatProvider();
                    break;
                case SpreadDocumentFormat.Csv:
                    formatProvider = new CsvFormatProvider();
                    break;
                default :
                    throw new NotSupportedException();
            }
            // For export and save the stream
            //string fileName = ExcelInteropTests.Utils.TestsConfig.TempFolder + ExcelInteropTests.Utils.TestsConfig.FileName + formatProvider.SupportedExtensions.FirstOrDefault();
            //File.WriteAllBytes(fileName, ReadFully(stream));

            stream.Seek(0, SeekOrigin.Begin);
            var workbook = formatProvider.Import(stream);

            return new WorkbookWrapper(workbook);
        }

        public WorksheetsWrapper Worksheets
        {
            get
            {
                return this.worksheets;
            }
        }

        public CellStylesCollectionWrapper CellStyles
        {
            get
            {
                return new CellStylesCollectionWrapper(this.instance);
            }
        }

        private WorkbookWrapper(Workbook workbook)
        {
            this.instance = workbook;
            this.worksheets = new WorksheetsWrapper(this.instance.Worksheets);
        }

        public void Dispose()
        {
        }

        private static byte[] ReadFully(Stream input)
        {
            input.Seek(0, SeekOrigin.Begin);

            byte[] buffer = new byte[input.Length];
            using (MemoryStream ms = new MemoryStream())
            {
                int read;
                while ((read = input.Read(buffer, 0, buffer.Length)) > 0)
                {
                    ms.Write(buffer, 0, read);
                }
                return ms.ToArray();
            }
        }
    }
}