﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.CsvExport
{
    [TestClass]
    public class CellValsueTest : CellValuesBaseTest
    {
        protected override SpreadDocumentFormat SpreadDocumentFormat
        {
            get
            {
                return SpreadDocumentFormat.Csv;
            }
        }

        protected override string TestCategory
        {
            get
            {
                return SpreadsheetStreamingCategory.CsvExport;
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        public void SetTextToCellTestCsv()
        {
            this.SetTextToCellAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetTextToCellWithNumberFormattingTestCsv()
        {
            this.SetTextToCellWithNumberFormattingAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetNumberAsStringToCellWithNumberFormattingTestCsv()
        {
            this.SetNumberAsStringToCellWithNumberFormattingAndAssert();
        }

#if !SPREAD
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.Bug)]
        [Description("The issue is reproducible when the csv is imported in Spreadsheet and actual value is 3/14/2015 but should be 3/14/2015 12:00:00 AM. The bug is valid only for RadSpreadsheet.")]
        public void SetDateValueWtihDateFormattingTestCsv()
        {
            this.SetDateValueWtihDateFormattingAndVerifyTheData();
        }
        
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.Bug)]
        [Description("The issue is reproducible when the csv is imported in Spreadsheet and actual value is 3/14/2015 but should be 3/14/2015 12:00:00 AM. The bug is valid only for RadSpreadsheet.")]
        public void SetDateValueWithoutFormattingTestCsv()
        {
            this.SetDateValueWithoutFormattingAndVerifyDocument();
        }
#endif

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetBooleanValueTestCsv()
        {
            this.SetBooleanValueAndVerifyDocument();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetNumeberValueToACellWithoutFormattingTestCsv()
        {
            this.SetNumeberToACellWithoutFormattingAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetNumeberToACellWithNumberFormattingTestCsv()
        {
            this.SetNumeberToACellWithNumberFormattingAndAssert();
        }

#if !EXCEL
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetFormatToACellAfterSettingValueTestCsv()
        {
            this.SetFormatToACellAfterSettingValueAndAssert();
        }
#endif

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetFormatToACellBeforeSettingValueTestCsv()
        {
            this.SetFormatToACellBeforeSettingValueAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        public void SetMultipleCellValuesWithFormatsCsvTests()
        {
            this.SetMultipleCellValuesWithFormatsAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        public void SetAllCellBordersCsvTest()
        {
            this.SetAllCellBordersAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        public void SetAllCellsPatternTypesCsvTest()
        {
            this.SetAllCellsPatternTypesAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        public void SkipCellsCsvTest()
        {
            this.SkipCellsAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetDiffValuesToASingleCellCsvTest()
        {
            this.SetMultipleValuesToASingleCellAndVerifyTheDocument();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithCommasTest()
        {
            this.SetStringWithCommasAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithDoubleQuotesTest()
        {
            this.SetStringWithDoubleQoutesAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithQuotesOnlyTest()
        {
            this.SetStringWithDoubleQoutesOnlyAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithNewLinesTest()
        {
            this.SetStringNewLinesAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithNewLineTest()
        {
            this.SetStringWithNewLineAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithNewLineAsciiSymbolTest()
        {
            this.SetStringWithNewLineAsciiSymbolAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringNewLinesRNTest()
        {
            this.SetStringNewLinesRNAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringNewLinesWithAtTest()
        {
            this.SetStringNewLinesWithAtSymbolAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithQuoteNewLineTest()
        {
            this.SetStringWithQuoteNewLineAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithQuotesAndCommasTest()
        {
            this.SetStringWithQuotesAndCommasAndAssert();
        }        

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithMoreSpecialCharactersTest()
        {
            this.SetStringWithMoreSpecialCharactersAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellValueCsv)]
        public void SetStringWithUmlautCharactersTest()
        {
            this.SetStringWithUmlautCharactersAndAssert();
        }    
    }
}