﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.CsvExport
{
    [TestClass]
    public class ElementsExportOrderCsvTests : ElementsOrderTestBase
    {
        protected override SpreadDocumentFormat SpreadDocumentFormat
        {
            get
            {
                return SpreadDocumentFormat.Csv;
            }
        }

        protected override string TestCategory
        {
            get
            {
                return SpreadsheetStreamingCategory.CsvExport;
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Columns is not supported in CSV. However we should not throw exception.")]
        public void ColumnsExportOrderIsBeforeRowsCsv()
        {
            this.AddRowsExportBeforeColumnsExport();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Columns is not supported in CSV. However we should not throw exception.")]
        public void ColumnsExportOrderCannotBeAddedAfterRowExportOrderTestCsv()
        {
            this.ExportColumnBeforeAndAfterRowExportOrder();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergeCellsOrderIsNotBeforeColumnExportTestCsv()
        {
            // THIS TEST WILL START FAILING ONCE BUG WITH TFS ID 287056 IS FIXED
            this.MergeCellsBeforeColumnExport();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergeCellsOrderIsAfterColumnsOrderCsv()
        {
            // THIS TEST WILL START FAILING ONCE BUG WITH TFS ID 287056 IS FIXED
            this.MergeCellsAfterColumnsExport();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Columns is not supported in CSV. However we should not throw exception.")]
        public void ColumnsExportOrderCannotBeWithinRowCsv()
        {
            this.ExportColumnWithinRow();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SkipCellsCannotAcceptNegativeValueTestCsv()
        {
            this.SkipCellsCannotAcceptNegativeNumberAndAssert();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SkipRowsCannotAcceptNegativeNumberTestCsv()
        {
            this.SkipRowsCannotAcceptNegativeNumber();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Columns is not supported in CSV. However we should not throw exception.")]
        public void SkipColumnsCannotAcceptNegativeNumberCsv()
        {
            // THIS TEST WILL START FAILING ONCE BUG WITH TFS ID 287056 IS FIXED
            this.SkipColumnsCannotAcceptNegativeNumber();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrder)]
        [Description("Columns is not supported in CSV. However we should not throw exception.")]
        public void RowsExportOrderCannotBeWithinColumnsTestCsv()
        {
            this.ExportRowsWithinColumns();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergingCellsWithoutCreatingRowExportAndAfterColumnExportCsvTest()
        {
            this.MergeCellsAfterColumnExportWithoutCreatingRowsAndVerifyDocumentIsValid();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergingCellsOrderIsNotWithinColumnExportCsvTest()
        {
            this.MergeCellsWithinColumnExport();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergingCellsOrderIsNotBeforeRowExporterCsvTest()
        {
            this.MergingCellsBeforeRowExporter();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.ElementsExportOrderCsv)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [Description("Merge cells is not supported in CSV. However we should not throw exception.")]
        public void MergingCellsOrderIsNotWithinRowExporterCsvTest()
        {
            this.MergingCellsOrderIsNotWithinRowExporter();
        }
    }
}
