﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.CsvExport
{
    [TestClass]
    public class WorksheetsCsvTests : WorksheetTestBase
    {
        protected override SpreadDocumentFormat SpreadDocumentFormat
        {
            get
            {
                return SpreadDocumentFormat.Csv;
            }
        }

        protected override string TestCategory
        {
            get
            {
                return SpreadsheetStreamingCategory.CsvExport;
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.WorksheetCsv)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void CreateWorksheetWithinAWorksheetTestCsv()
        {
            MemoryStream stream = null;

            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";
                string secondWorksheetName = "Second";

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                        }

                        using (IWorksheetExporter worksheet1 = workbook.CreateWorksheetExporter(secondWorksheetName))
                        {
                            using (worksheet1.CreateRowExporter())
                            {
                            }
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.WorksheetCsv)]
        public void CreateDocumentContainingMultipleWorksheetsCsv()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";
                string secondWorksheetName = "Second";
                string thirdWorksheetName = "Sheet3";

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    string sheetName = firstWorksheetName;

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }

                    sheetName = secondWorksheetName;
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }

                    sheetName = thirdWorksheetName;
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

#if VISUAL
                this.AssertTestVisual(stream);
#else

                using (StreamReader streamReader = new StreamReader(stream))
                {
                    string csvDocument = streamReader.ReadToEnd();

                    Assert.IsTrue(csvDocument.Contains(firstWorksheetName));

                    Assert.IsTrue(csvDocument.Contains(secondWorksheetName));
                    Assert.IsTrue(csvDocument.Contains(thirdWorksheetName));
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.WorksheetCsv)]
        public void OverrideExistingDocumentCsv()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";
                string secondWorksheetName = "Second";

                string sheetName = firstWorksheetName;
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                sheetName = secondWorksheetName;
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

#if VISUAL
                this.AssertTestVisual(stream);
#else

                using (StreamReader streamReader = new StreamReader(stream))
                {
                    string csvDocument = streamReader.ReadToEnd();

                    Assert.IsFalse(csvDocument.Contains(firstWorksheetName));

                    Assert.IsTrue(csvDocument.Contains(secondWorksheetName));
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.CsvExport)]
        [TestCategory(SpreadsheetStreamingCategory.WorksheetCsv)]
        public void AppendToExistingDocumentCsv()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";
                string secondWorksheetName = "Second";
                string thirdWorksheetName = "Sheet3";

                string sheetName = firstWorksheetName;
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                sheetName = secondWorksheetName;
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream, SpreadExportMode.Append))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                sheetName = thirdWorksheetName;
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream, SpreadExportMode.Append))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheetName))
                    {
                        using (IRowExporter rowExported = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cellExported = rowExported.CreateCellExporter())
                            {
                                cellExported.SetValue(sheetName);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

#if VISUAL
                this.AssertTestVisual(stream);
#else

                using (StreamReader streamReader = new StreamReader(stream))
                {
                    string csvDocument = streamReader.ReadToEnd();

                    Assert.IsTrue(csvDocument.Contains(firstWorksheetName));

                    Assert.IsTrue(csvDocument.Contains(secondWorksheetName));
                    Assert.IsTrue(csvDocument.Contains(thirdWorksheetName));
                }
#endif
            }
        }
    }
}
