﻿#if EXCEL || SPREAD
using SpreadsheetStreaming.Tests.ExcelWrappers;
#endif
using SpreadsheetStreaming.Tests;
using SpreadsheetStreaming.Tests.Utils;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming;

namespace SpreadsheetStreaming.ExcelInteropTests.Tests
{
    [TestClass]
    public class ElementsExportOrder : StreamFormatProvidersTestBase
    {
        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureColumnsExportOrderIsBeforeRows()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {

                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureColumnsExportOrderIsBeforeRowsAndCannotBeAddedAfterThat()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergeCellsOrderIsBeAfterRowsOrder()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 5, 5);
                        using (IRowExporter column = worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergeCellsOrderIsAfterColumnsOrder()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 5, 5);

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureColumnsExportOrderCannotBeWithinRow()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (IColumnExporter column = worksheet.CreateColumnExporter())
                            {

                            }
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void EnsureSkipCellsCannotAcceptNegativeNumber()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(-1);
                        }
                    }
                }
            }
        }


        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void EnsureSkipRowsCannotAcceptNegativeNumber()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.SkipRows(-1);
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void EnsureSkipColumnsCannotAcceptNegativeNumber()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.SkipColumns(-1);
                    }
                }
            }
        }

        //[TestMethod]
        //[TestCategory(XlsxToStreamCategory.XlsxToStream)]
        //[TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        //[ExpectedException(typeof(InvalidOperationException))]
        public void EnsureRowsExportOrderCannotBeWithinColumnsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {

                            }
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergingCellsOrderIsNotBeforColumnExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 2, 2);

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
            }
        }

        //[TestMethod]
        //[TestCategory(XlsxToStreamCategory.XlsxToStream)]
        //[TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        public void EnsureMergingCellsOrderIsAfterColumnExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }

                        worksheet.MergeCells(1, 1, 2, 2);
                    }
                }

#if !VISUAL
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    var mergedCells = worksheet.MergedCells.ToArray();
                    Assert.AreEqual(new CellRangeWrapper(1, 1, 2, 2), mergedCells[0].Range);
                }
#endif
            }
        }

        //[TestMethod]
        //[TestCategory(XlsxToStreamCategory.XlsxToStream)]
        //[TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        //[ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergingCellsOrderIsNotWithinColumnExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            worksheet.MergeCells(1, 1, 2, 2);
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergingCellsOrderIsNotBeforeRowExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 2, 2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(XlsxToStreamCategory.XlsxToStream)]
        [TestCategory(XlsxToStreamCategory.ElementsExportOrder)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void EnsureMergingCellsOrderIsNotWithinRowExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 2, 2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
            }
        }
    }
}
