﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.ExcelWrappers;
#endif
#if SPREAD
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.TextBased.Csv;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
#endif
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming;

namespace Telerik.Documents.SpreadsheetStreaming.VisualTests.Tests.CellValuesBaseTest
{
    public abstract class CellValuesBaseTest : ExportTestBase
    {
        protected void SetTextToCellAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetTextToCellWithNumberFormattingAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                SpreadCellFormat format = new SpreadCellFormat();
                                format.NumberFormat = "@";
                                cell.SetValue("test");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetNumberAsStringToCellWithNumberFormattingAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                SpreadCellFormat format = new SpreadCellFormat();
                                format.NumberFormat = "@";
                                cell.SetValue("123");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("123", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetNumeberToACellWithoutFormattingAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(123);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("123", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetDecimalNumberToACellWithoutFormattingAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(123.34);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetNumeberToACellWithNumberFormattingAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                SpreadCellFormat cellFormat = new SpreadCellFormat();
                                cellFormat.NumberFormat = "-123.123";
                                cell.SetValue(123.34);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("123.34", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetDateValueWtihDateFormattingAndVerifyTheData()
        {
            MemoryStream stream = null;
            SpreadCellFormat cellFormat = new SpreadCellFormat();

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cellFormat.NumberFormat = "[$-409]d-mmm-yyyy;@";

                                DateTime time = new DateTime(2015, 3, 14);

                                cell.SetValue(time);
                                cell.SetFormat(cellFormat);

                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

#if SPREAD
                    Assert.AreEqual("42077", value);
#else
                    Assert.AreEqual("3/14/2015 12:00:00 AM", value);
#endif

                    if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                    {
                        cellFormat.NumberFormat = "m/d/yyyy h:mm";
                    }

                    Assert.IsTrue(cell.IsCellFormatEqual(cellFormat), "The cell formating is not the same!");
                }
#endif
            }
        }

        protected void SetDateValueWithoutFormattingAndVerifyDocument()
        {
            MemoryStream stream = null;
            DateTime time = new DateTime(2015, 3, 14);

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(time);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;
                    string expextedValue = @"3/14/2015 12:00:00 AM";
                    var cellFormat = new SpreadCellFormat();

                    if (this.TestCategory == SpreadsheetStreamingCategory.XlsxExport)
                    {
                        expextedValue = "42077";
                    }
                    else
                    {
                        cellFormat.NumberFormat = @"m/d/yyyy h:mm";
                    }

                    Assert.AreEqual(expextedValue, value);

                    Assert.IsTrue(cell.IsCellFormatEqual(cellFormat));
                }
#endif
            }
        }

        protected void SetBooleanValueAndVerifyDocument()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(true);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;
                    string expectedValue = "True";
#if SPREAD
                    expectedValue = "TRUE";
#endif
                    Assert.AreEqual(expectedValue, value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        public void SetMultipleValuesToASingleCellAndVerifyTheDocument()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("MultipleValues"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test1");
                                cell.SetValue("123");
                                cell.SetValue("02.02.2012");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("02.02.2012", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetFormatToACellAfterSettingValueAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                SpreadCellFormat format = GetSpreadCellNumberFormatWithSetAllProperies();

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        worksheet.SkipRows(2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(2);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(format);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[2, 2];
                    string value = cell.Value;
                    string assertString = "test";

                    if (this.TestCategory != SpreadsheetStreamingCategory.CsvExport)
                    {
                        Assert.AreEqual(assertString, value);
                        Assert.IsTrue(cell.IsCellFormatEqual(format));
                    }
                    else
                    {
                        assertString = null;
                        Assert.AreEqual(assertString, assertString);
                    }

                }
#endif
            }
        }

        protected void SetFormatToACellBeforeSettingValueAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                SpreadCellFormat format = GetSpreadCellNumberFormatWithSetAllProperies();

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        worksheet.SkipRows(2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(2);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetFormat(format);
                                cell.SetValue("test");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[2, 2];
                    string value = cell.Value;
                    string assertString = "test";

                    if (this.TestCategory != SpreadsheetStreamingCategory.CsvExport)
                    {
                        Assert.AreEqual(assertString, value);
                        Assert.IsTrue(cell.IsCellFormatEqual(format));
                    }
                    else
                    {
                        assertString = null;
                        Assert.AreEqual(assertString, assertString);
                    }

                }
#endif
            }
        }

        protected void SetMultipleCellValuesWithFormatsAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                SpreadCellFormat bordersFormat = new SpreadCellFormat();

                SpreadBorder border = new SpreadBorder(SpreadBorderStyle.Hair, new SpreadThemableColor(new SpreadColor(123, 131, 111)));
                bordersFormat.LeftBorder = border;
                bordersFormat.RightBorder = border;
                bordersFormat.BottomBorder = border;

                SpreadCellFormat fillFormat = new SpreadCellFormat();
                fillFormat.Fill = new SpreadPatternFill(SpreadPatternType.ReverseDiagonalStripe);

                SpreadCellFormat numberFormatformat = new SpreadCellFormat();
                numberFormatformat.NumberFormat = "dd.mm.yyyy";

                SpreadCellFormat textPropertiesFormat = new SpreadCellFormat();
                textPropertiesFormat.FontFamily = new SpreadThemableFontFamily("Times New Roman");
                textPropertiesFormat.FontSize = 11;
                textPropertiesFormat.ForeColor = new SpreadThemableColor(new SpreadColor(111, 255, 222));
                textPropertiesFormat.HorizontalAlignment = SpreadHorizontalAlignment.Distributed;
                textPropertiesFormat.Indent = 2;
                textPropertiesFormat.IsBold = false;
                textPropertiesFormat.IsItalic = true;
                textPropertiesFormat.Underline = SpreadUnderlineType.SingleAccounting;
                textPropertiesFormat.VerticalAlignment = SpreadVerticalAlignment.Top;
                textPropertiesFormat.WrapText = false;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInPixels(100);
                        }

                        worksheet.SkipRows(2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(2);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("Borders");
                                cell.SetFormat(bordersFormat);
                            }
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("Borders1");
                                cell.SetFormat(bordersFormat);
                            }

                            row.SkipCells(2);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("Fill");
                                cell.SetFormat(fillFormat);
                            }
                        }

                        worksheet.SkipRows(2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(new DateTime(2015, 12, 1));
                                cell.SetFormat(numberFormatformat);
                            }

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("TextProperties");
                                cell.SetFormat(textPropertiesFormat);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                if (this.TestCategory == SpreadsheetStreamingCategory.SpreadsheetStreaming)
                {
                    using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                    {
                        WorksheetWrapper worksheet = workbookWraper.Worksheets[0];

                        CellWrapper bordersCell = worksheet.Cells[2, 2];
                        string bordersCellValue = bordersCell.Value;

                        Assert.AreEqual("Borders", bordersCellValue);
                        Assert.IsTrue(bordersCell.IsCellFormatEqual(bordersFormat));


                        CellWrapper borders1Cell = worksheet.Cells[2, 3];
                        string borders1CellValue = borders1Cell.Value;

                        Assert.AreEqual("Borders1", borders1CellValue);
                        Assert.IsTrue(borders1Cell.IsCellFormatEqual(bordersFormat));


                        CellWrapper fillCell = worksheet.Cells[2, 6];
                        string fillCellValue = fillCell.Value;

                        Assert.AreEqual("Fill", fillCellValue);
                        Assert.IsTrue(fillCell.IsCellFormatEqual(fillFormat));


                        CellWrapper numberFormatCell = worksheet.Cells[5, 0];

                        // Here the date is set as date value in the sheet. This is not supported by RadSpreadProcessing
#if EXCEL
                        string numberFormatCellValue = numberFormatCell.Value;
                        Assert.AreEqual("12/1/2015 12:00:00 AM", numberFormatCellValue);
#endif
                        Assert.IsTrue(numberFormatCell.IsCellFormatEqual(numberFormatformat));


                        CellWrapper textPropertiesCell = worksheet.Cells[5, 1];
                        string textPropertiesCellValue = textPropertiesCell.Value;

                        Assert.AreEqual("TextProperties", textPropertiesCellValue);
                        Assert.IsTrue(textPropertiesCell.IsCellFormatEqual(textPropertiesFormat));
                    }
                }
#endif
            }
        }

        protected void SetAllCellBordersAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    int counter = 0;
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        worksheet.SkipRows(2);

                        IRowExporter row = worksheet.CreateRowExporter();
                        row.SkipCells(2);

                        foreach (SpreadBorderStyle style in Enum.GetValues(typeof(SpreadBorderStyle)))
                        {
                            SpreadCellFormat format = new SpreadCellFormat();

                            SpreadBorder border = new SpreadBorder(style, new SpreadThemableColor(new SpreadColor(123, 131, 0)));
                            format.LeftBorder = border;
                            format.TopBorder = border;
                            format.RightBorder = border;
                            format.BottomBorder = border;

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(format);
                            }
                            counter++;

                            row.SkipCells(1);

                            if (counter % 5 == 0)
                            {
                                row.Dispose();
                                worksheet.SkipRows(1);
                                row = worksheet.CreateRowExporter();
                                row.SkipCells(2);
                            }
                        }

                        row.Dispose();
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                if (this.TestCategory == SpreadsheetStreamingCategory.SpreadsheetStreaming)
                {
                    using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                    {
                        int cellIndexCounter = 2;
                        int cellCounter = 0;
                        int rowIndex = 2;
                        foreach (SpreadBorderStyle style in Enum.GetValues(typeof(SpreadBorderStyle)))
                        {
                            SpreadCellFormat format = new SpreadCellFormat();

                            SpreadBorder border = new SpreadBorder(style, new SpreadThemableColor(new SpreadColor(123, 131, 0)));
                            format.LeftBorder = border;
                            format.TopBorder = border;
                            format.RightBorder = border;
                            format.BottomBorder = border;

                            WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                            CellWrapper cell = worksheet.Cells[rowIndex, cellIndexCounter];
                            string value = cell.Value;

                            Assert.AreEqual("test", value);
                            Assert.IsTrue(cell.IsCellFormatEqual(format));

                            cellCounter++;

                            if (cellCounter % 5 == 0)
                            {
                                cellIndexCounter = 0;
                                rowIndex += 2;
                            }
                            cellIndexCounter += 2;
                        }
                    }
                }
#endif
            }
        }

        protected void SetAllCellsPatternTypesAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                var patternColor = new SpreadThemableColor(new SpreadColor(255, 0, 0));
                var backgroundColor = new SpreadThemableColor(new SpreadColor(0, 0, 255));

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            foreach (SpreadPatternType patternType in Enum.GetValues(typeof(SpreadPatternType)))
                            {
                                SpreadCellFormat format = new SpreadCellFormat();
                                format.Fill = new SpreadPatternFill(patternType, patternColor, backgroundColor);

                                using (ICellExporter cell = row.CreateCellExporter())
                                {
                                    cell.SetFormat(format);
                                }
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];

                    string sheetName = "Sheet1";

#if EXCEL
                    if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                    {
                        sheetName = "test";
                    }
#endif

                    Assert.AreEqual(sheetName, worksheet.Name);

                    int cellIndex = 0;
                    foreach (SpreadPatternType patternType in Enum.GetValues(typeof(SpreadPatternType)))
                    {
                        SpreadCellFormat format = new SpreadCellFormat();
                        format.Fill = new SpreadPatternFill(patternType, patternColor, backgroundColor);
                        CellWrapper cell = worksheet.Cells[0, cellIndex];

                        cellIndex++;
                        if (this.TestCategory == SpreadsheetStreamingCategory.SpreadsheetStreaming)
                        {
                            Assert.IsTrue(cell.IsCellFormatEqual(format));
                        }
#if SPREAD
                        string value = "";
#elif EXCEL
                        string value = null;
#endif
                        Assert.IsTrue(cell.Value == value);
                    }
                }
#endif
            }
        }

        public void SkipCellsAndAssert()
        {
            int rowsCount = 5;
            int cellPerRow = 5;

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        for (int rowIndex = 0; rowIndex <= rowsCount; rowIndex++)
                        {
                            worksheet.SkipRows(1);

                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                                for (int cellsCount = 0; cellsCount < cellPerRow; cellsCount++)
                                {
                                    row.SkipCells(cellsCount);

                                    using (ICellExporter cell = row.CreateCellExporter())
                                    {
                                        cell.SetValue("value " + (rowIndex) + @" / " + cellsCount);
                                    }
                                }
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                if (this.TestCategory == SpreadsheetStreamingCategory.SpreadsheetStreaming)
                {
                    using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                    {
                        WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                        int checkRowIndex = 1;
                        int rowValue = 0;

                        for (int rowIndex = 0; rowIndex <= rowsCount; rowIndex++)
                        {
                            int checkCellIndex = 0;

                            if (rowIndex > 0)
                            {
                                checkRowIndex = rowIndex + 2;
                            }

                            if (checkRowIndex % 2 != 0 && checkRowIndex > 1)
                            {
                                rowValue++;
                            }

                            for (int cellsCount = 0; cellsCount < cellPerRow; cellsCount++)
                            {
                                if (cellsCount > 0)
                                {
                                    checkCellIndex = cellsCount + checkCellIndex + 1;
                                }

                                if (checkRowIndex % 2 != 0)
                                {
                                    CellWrapper cell = worksheet.Cells[checkRowIndex, checkCellIndex];
                                    Assert.AreEqual(cell.Value, "value " + rowValue + @" / " + cellsCount);

                                    if (checkCellIndex > 1)
                                    {
                                        CellWrapper nullabaleCellLeft = worksheet.Cells[rowIndex + 1, checkCellIndex - 1];
#if EXCEL
                                        Assert.AreEqual(nullabaleCellLeft.Value, null);
                                        CellWrapper nullabaleCellRight = worksheet.Cells[rowIndex + 1, checkCellIndex + 1];
                                        Assert.AreEqual(nullabaleCellRight.Value, null);
#elif SPREAD
                                        Assert.AreEqual(nullabaleCellLeft.Value, "");
                                        CellWrapper nullabaleCellRight = worksheet.Cells[rowIndex + 1, checkCellIndex + 1];
                                        Assert.AreEqual(nullabaleCellRight.Value, "");
#endif
                                    }
                                }
                            }
                        }
                    }
                }
#endif
            }
        }

        protected void SetMultipleFormatsToASingleCellAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                SpreadCellFormat format = GetSpreadCellNumberFormatWithSetAllProperies();

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        worksheet.SkipRows(2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(2);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(format);
                                format.IsBold = false;
                                format.IsItalic = false;
                                format.Underline = SpreadUnderlineType.None;
                                cell.SetFormat(format);
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[2, 2];
                    string value = cell.Value;
                    string assertString = "test";

                    if (this.TestCategory != SpreadsheetStreamingCategory.CsvExport)
                    {
                        Assert.AreEqual(assertString, value);
                        Assert.IsTrue(cell.IsCellFormatEqual(format));
                    }
                    else
                    {
                        assertString = null;
                        Assert.AreEqual(assertString, assertString);
                    }

                }
#endif
            }
        }

        private static SpreadCellFormat GetSpreadCellNumberFormatWithSetAllProperies()
        {
            SpreadCellFormat format = new SpreadCellFormat();

            SpreadBorder border = new SpreadBorder(SpreadBorderStyle.Medium, new SpreadThemableColor(new SpreadColor(123, 131, 0)));
            format.LeftBorder = border;
            format.TopBorder = border;
            format.RightBorder = border;
            format.BottomBorder = border;

            format.Fill = new SpreadPatternFill(SpreadPatternType.Gray6Percent);
            format.NumberFormat = "@";

            format.FontFamily = new SpreadThemableFontFamily("Arial");
            format.FontSize = 30;
            format.ForeColor = new SpreadThemableColor(new SpreadColor(0, 255, 0));
            format.HorizontalAlignment = SpreadHorizontalAlignment.Center;
            format.Indent = 5;
            format.IsBold = true;
            format.IsItalic = true;
            format.Underline = SpreadUnderlineType.Single;
            format.VerticalAlignment = SpreadVerticalAlignment.Center;
            format.WrapText = true;
            return format;
        }

        protected void SetStringWithCommasAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("1,2,3");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("1,2,3", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithDoubleQoutesAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("One \"Quoted\" Two");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("One \"Quoted\" Two", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithDoubleQoutesOnlyAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("\"\"");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("\"\"", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithNewLineAndAssert()
        {

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("aaa \n word");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("aaa \n word", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithNewLineAsciiSymbolAndAssert()
        {

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("\u000A line");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("\u000A line", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringNewLinesAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("1\n\r\"2\"\n\r3");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("1\n\r\"2\"\n\r3", value);
                }
#endif
            }
        }

        protected void SetStringNewLinesRNAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("one\r\ntwo\r\nthree");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("one\r\ntwo\r\nthree", value);
                }
#endif
            }
        }

        protected void SetStringNewLinesWithAtSymbolAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(@"one
two
three");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual(@"one
two
three", value);
                }
#endif
            }
        }

        protected void SetStringWithQuotesAndCommasAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("\"First\", \"Second\",\"Third\"");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("\"First\", \"Second\",\"Third\"", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithQuoteNewLineAndAssert()
        {

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("\"test\r\ntest\"");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("\"test\r\ntest\"", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }

        protected void SetStringWithMoreSpecialCharactersAndAssert()
        {

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("5 \\ \r \t \' \"");
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("5 \\ \r \t \' \"", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(new SpreadCellFormat()));
                }
#endif
            }
        }
    }
}
