﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.ExcelWrappers;
#endif
using Telerik.Documents.SpreadsheetStreaming.VisualTests.Tests;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming;

namespace Telerik.Documents.SpreadsheetStreaming.Tests
{
    public abstract class ElementsOrderTestBase : ExportTestBase
    {
        public void AddRowsExportBeforeColumnsExport()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#endif
            }
        }

        public void ExportColumnBeforeAndAfterRowExportOrder()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(5);
                            }
                        }

                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void MergeCellsBeforeColumnExport()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 5, 5);
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void MergeCellsAfterColumnsExport()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }
                        worksheet.MergeCells(1, 1, 5, 5);
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void ExportColumnWithinRow()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (IColumnExporter column = worksheet.CreateColumnExporter())
                            {

                            }
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void SkipCellsCannotAcceptNegativeNumberAndAssert()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(-1);
                        }
                    }
                }
            }
        }

        public void SkipRowsCannotAcceptNegativeNumber()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1");

                    worksheet.SkipRows(-1);

                    worksheet.Dispose();
                }
            }
        }

        public void SkipColumnsCannotAcceptNegativeNumber()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1");

                    worksheet.SkipColumns(-1);

                    worksheet.Dispose();
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void ExportRowsWithinColumns()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {

                            }
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void MergeCellsAfterColumnExportWithoutCreatingRowsAndVerifyDocumentIsValid()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                        }

                        worksheet.MergeCells(1, 1, 2, 2);
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    var mergedCells = worksheet.MergedCells.ToArray();
                    if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                    {
                        Assert.AreEqual(0, mergedCells.Length);
                    }
                    else
                    {
                        Assert.AreEqual(new CellRangeWrapper(1, 1, 2, 2), mergedCells[0].Range);
                    }
                }
#endif
            }
        }

        public void MergeCellsWithinColumnExport()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            worksheet.MergeCells(1, 1, 2, 2);
                        }
                    }
                }
            }
        }

        public void MergingCellsBeforeRowExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 2, 2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }

        public void MergingCellsOrderIsNotWithinRowExporter()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        worksheet.MergeCells(1, 1, 2, 2);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
#if VISUAL
                if (this.TestCategory == SpreadsheetStreamingCategory.CsvExport)
                {
                    this.AssertTestVisual(stream);
                }
#endif
            }
        }
    }
}
