﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Telerik.Documents.SpreadsheetStreaming.SpreadStreamImport
{
    [TestClass]
    public class ColumnsTests
    {
        [TestMethod]
        public void AssertColumnsAreGroupedByHiddenPropertyOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                bool value = true;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetHidden(value);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                Assert.AreEqual(0, columnImporter.FromIndex);
                                Assert.AreEqual(4, columnImporter.ToIndex);
                                Assert.AreEqual(value, columnImporter.IsHidden);
                                importedColumnsCount++;
                            }

                            Assert.AreEqual(1, importedColumnsCount);
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void AssertColumnsAreGroupedByOutlinePropertyOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                int value = 1;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetOutlineLevel(value);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                Assert.AreEqual(value, columnImporter.OutlineLevel);
                                Assert.AreEqual(0, columnImporter.FromIndex);
                                Assert.AreEqual(4, columnImporter.ToIndex);
                                importedColumnsCount++;
                            }

                            Assert.AreEqual(1, importedColumnsCount);
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void AssertColumnsAreGroupedByWidthInCharactersPropertyOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                int value = 10;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetWidthInCharacters(value);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                Assert.AreEqual(value, columnImporter.WidthInCharacters);
                                Assert.AreEqual(0, columnImporter.FromIndex);
                                Assert.AreEqual(4, columnImporter.ToIndex);
                                importedColumnsCount++;
                            }

                            Assert.AreEqual(1, importedColumnsCount);
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void AssertColumnsAreGroupedByWidthInPixelsPropertyOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                int value = 10;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetWidthInPixels(value);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                Assert.AreEqual(value, columnImporter.WidthInPixels);
                                Assert.AreEqual(0, columnImporter.FromIndex);
                                Assert.AreEqual(4, columnImporter.ToIndex);
                                importedColumnsCount++;
                            }

                            Assert.AreEqual(1, importedColumnsCount);
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void AssertColumnsAreGroupedByAllPropertiesOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                bool isHidden = true;
                int outlineLevel = 1;
                double width = 15;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetHidden(isHidden);
                                columnExproter.SetOutlineLevel(outlineLevel);
                                columnExproter.SetWidthInCharacters(width);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                Assert.AreEqual(isHidden, columnImporter.IsHidden);
                                Assert.AreEqual(outlineLevel, columnImporter.OutlineLevel);
                                Assert.AreEqual(width, columnImporter.WidthInCharacters);
                                Assert.AreEqual(0, columnImporter.FromIndex);
                                Assert.AreEqual(4, columnImporter.ToIndex);
                                importedColumnsCount++;
                            }

                            Assert.AreEqual(1, importedColumnsCount);
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void AssertColumnsAreGroupedByAllPropertiesInTwoGroupsOnExport()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                int columnsCount = 5;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;
                bool isHidden = true;
                int outlineLevel = 1;
                double width = 15;
                double secondWidth = 10;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetHidden(isHidden);
                                columnExproter.SetOutlineLevel(outlineLevel);
                                columnExproter.SetWidthInCharacters(width);
                            }
                        }

                        for (int i = 0; i < columnsCount; i++)
                        {
                            using (IColumnExporter columnExproter = worksheet.CreateColumnExporter())
                            {
                                columnExproter.SetHidden(isHidden);
                                columnExproter.SetOutlineLevel(outlineLevel);
                                columnExproter.SetWidthInCharacters(secondWidth);
                            }
                        }

                        using (IRowExporter rowExporter = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = rowExporter.CreateCellExporter())
                            {
                                cell.SetValue(false);
                            }
                        }
                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookImporter workbook = SpreadImporter.CreateWorkbookImporter(documentFormat, stream))
                {
                    foreach (IWorksheetImporter worksheetImporter in workbook.WorksheetImporters)
                    {
                        using (worksheetImporter)
                        {
                            int importedColumnsCount = 0;
                            foreach (IColumnImporter columnImporter in worksheetImporter.Columns)
                            {
                                if (importedColumnsCount == 0)
                                {
                                    Assert.AreEqual(isHidden, columnImporter.IsHidden);
                                    Assert.AreEqual(outlineLevel, columnImporter.OutlineLevel);
                                    Assert.AreEqual(width, columnImporter.WidthInCharacters);
                                    Assert.AreEqual(0, columnImporter.FromIndex);
                                    Assert.AreEqual(4, columnImporter.ToIndex);
                                }
                                else
                                {
                                    Assert.AreEqual(isHidden, columnImporter.IsHidden);
                                    Assert.AreEqual(outlineLevel, columnImporter.OutlineLevel);
                                    Assert.AreEqual(secondWidth, columnImporter.WidthInCharacters);
                                    Assert.AreEqual(5, columnImporter.FromIndex);
                                    Assert.AreEqual(9, columnImporter.ToIndex);
                                }

                                importedColumnsCount++;
                            }

                            Assert.AreEqual(2, importedColumnsCount);
                        }
                    }
                }
            }
        }
    }
}