﻿#if SPREAD
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

namespace Telerik.Documents.SpreadsheetStreaming.Tests
{
    [TestClass]
    public class WorkbookExporterGetSheetInfosTests
    {
        [TestMethod]
        public void XlxsWorkbookExporterGetSheetInfo_WithEmptyWorkbook_ReturnsEmptyCollection()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
            {
                var sheetInfos = workbookExporter.GetSheetInfos();
                Assert.IsFalse(sheetInfos.Any());

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                } 
            }
        }

        [TestMethod]
        public void XlxsWorkbookExporterGetSheetInfo_AfterWorksheetIsAdded_ReturnsSheetInfo()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
            {
                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    var sheetInfos = workbookExporter.GetSheetInfos();
                    Assert.AreEqual(1, sheetInfos.Count());
                    Assert.AreEqual("Sheet1", sheetInfos.First().Name);

                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }

                var sheetInfos1 = workbookExporter.GetSheetInfos();
                Assert.AreEqual(1, sheetInfos1.Count());
                Assert.AreEqual("Sheet1", sheetInfos1.First().Name);

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet2"))
                {
                    var sheetInfos2 = workbookExporter.GetSheetInfos();
                    Assert.AreEqual(2, sheetInfos2.Count());
                    Assert.AreEqual("Sheet2", sheetInfos2.Last().Name);

                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }

                var sheetInfos3 = workbookExporter.GetSheetInfos();
                Assert.AreEqual(2, sheetInfos3.Count());
                Assert.AreEqual("Sheet2", sheetInfos3.Last().Name);
            }
        }

        [TestMethod]
        public void CvsWorkbookExporterGetSheetInfo_WithEmptyWorkbook_ReturnsEmptyCollection()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Csv, stream))
            {
                var sheetInfos = workbookExporter.GetSheetInfos();
                Assert.IsFalse(sheetInfos.Any());

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }
            }
        }

        [TestMethod]
        public void CsvWorkbookExporterGetSheetInfo_AfterWorksheetIsAdded_ReturnsEmptyCollection()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Csv, stream))
            {
                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    var sheetInfos = workbookExporter.GetSheetInfos();
                    Assert.IsFalse(sheetInfos.Any());

                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }

                var sheetInfos1 = workbookExporter.GetSheetInfos();
                Assert.IsFalse(sheetInfos1.Any());

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet2"))
                {
                    var sheetInfos2 = workbookExporter.GetSheetInfos();
                    Assert.IsFalse(sheetInfos2.Any());

                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }

                var sheetInfos3 = workbookExporter.GetSheetInfos();
                Assert.IsFalse(sheetInfos3.Any());
            }
        }

        [TestMethod]
        public void XlsxAppendWorkbookExporterGetSheetInfo_WithEmptyWorkbook_ReturnsEmptyCollection()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
            {
                var sheetInfos = workbookExporter.GetSheetInfos();
                Assert.IsFalse(sheetInfos.Any());

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }
            }
        }

        [TestMethod]
        public void XlsxAppendWorkbookExporterGetSheetInfo_WithExistingWorkbook_ReturnsSheetInfo()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("ThreeWorksheets.xlsx"))
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                    {
                        var sheetInfos = workbookExporter.GetSheetInfos().ToList();
                        Assert.AreEqual(3, sheetInfos.Count);
                        Assert.AreEqual("Sheet1", sheetInfos[0].Name);
                        Assert.AreEqual("Sheet2", sheetInfos[1].Name);
                        Assert.AreEqual("Sheet3", sheetInfos[2].Name);

                        using (IWorksheetExporter worksheet = workbookExporter.CreateWorksheetExporter("Sheet4"))
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                            }
                        }
                    }
                }
            }
        }

        [TestMethod]
        public void XlsxAppendWorkbookExporterGetSheetInfo_AfterWorksheetIsAddedToExistingWorkbook_ReturnsSheetInfo()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("ThreeWorksheets.xlsx"))
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                    {
                        using (IWorksheetExporter worksheet = workbookExporter.CreateWorksheetExporter("Sheet4"))
                        {
                            var sheetInfos = workbookExporter.GetSheetInfos().ToList();
                            Assert.AreEqual(4, sheetInfos.Count);
                            Assert.AreEqual("Sheet1", sheetInfos[0].Name);
                            Assert.AreEqual("Sheet2", sheetInfos[1].Name);
                            Assert.AreEqual("Sheet3", sheetInfos[2].Name);
                            Assert.AreEqual("Sheet4", sheetInfos[3].Name);

                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                            }
                        }

                        var sheetInfos2 = workbookExporter.GetSheetInfos().ToList();
                        Assert.AreEqual(4, sheetInfos2.Count);
                        Assert.AreEqual("Sheet1", sheetInfos2[0].Name);
                        Assert.AreEqual("Sheet2", sheetInfos2[1].Name);
                        Assert.AreEqual("Sheet3", sheetInfos2[2].Name);
                        Assert.AreEqual("Sheet4", sheetInfos2[3].Name);
                    }
                }
            }
        }

        [TestMethod]
        public void CsvAppendWorkbookExporterGetSheetInfo_WithEmptyWorkbook_ReturnsEmptyCollection()
        {
            using (var stream = new MemoryStream())
            using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Csv, stream, SpreadExportMode.Append))
            {
                Assert.IsFalse(workbookExporter.GetSheetInfos().Any());

                using (IWorksheetExporter worksheetExpoter = workbookExporter.CreateWorksheetExporter("Sheet1"))
                {
                    using (var rowExporter = worksheetExpoter.CreateRowExporter())
                    {
                    }
                }
            }
        }

        [TestMethod]
        public void CsvAppendWorkbookExporterGetSheetInfo_WithExistingWorkbook_ReturnsEmptyCollection()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("File.csv"))
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbookExporter = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Csv, stream, SpreadExportMode.Append))
                    {
                        Assert.IsFalse(workbookExporter.GetSheetInfos().Any());
                        using (IWorksheetExporter worksheet = workbookExporter.CreateWorksheetExporter("Sheet1"))
                        {
                            Assert.IsFalse(workbookExporter.GetSheetInfos().Any());

                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                            }
                        }

                        Assert.IsFalse(workbookExporter.GetSheetInfos().Any());
                    }
                }
            }
        }
    }
}
#endif
