﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;

#if !VISUAL
using Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers;
#endif

namespace Telerik.Documents.SpreadsheetStreaming.Tests.XlsxExport
{
    [TestClass]
    public class AppendWorksheetTests : XlsxTestBase
    {
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.Append)]
        public void TestAppendWorksheetContainingFormats()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                SpreadBorder border1 = new SpreadBorder(SpreadBorderStyle.Hair, new SpreadThemableColor(new SpreadColor(255, 0, 0)));
                ISpreadFill fill1 = new SpreadPatternFill(
                    SpreadPatternType.HorizontalStripe,
                    new SpreadThemableColor(new SpreadColor(255, 0, 0)),
                    new SpreadThemableColor(new SpreadColor(0, 255, 0)));

                SpreadCellFormat firstWorksheetFirstCellFormat = new SpreadCellFormat()
                {
                    ApplyAlignmentInternal = true,
                    ApplyBorderInternal = true,
                    ApplyFillInternal = true,
                    ApplyFontInternal = true,
                    ApplyNumberFormatInternal = true,
                    ApplyProtectionInternal = true,
                    BottomBorder = border1,
                    DiagonalDownBorder = border1,
                    DiagonalUpBorder = border1,
                    Fill = fill1,
                    FontFamily = new SpreadThemableFontFamily("Verdana"),
                    FontSize = 30,
                    ForeColor = new SpreadThemableColor(SpreadThemeColorType.Accent5),
                    HorizontalAlignment = SpreadHorizontalAlignment.Center,
                    Indent = 3,
                    IsBold = true,
                    IsItalic = true,
                    LeftBorder = border1,
                    NumberFormat = @"#,##0.00",
                    RightBorder = border1,
                    TopBorder = border1,
                    Underline = SpreadUnderlineType.DoubleAccounting,
                    VerticalAlignment = SpreadVerticalAlignment.Justify,
                    WrapText = true
                };

                SpreadBorder border2 = new SpreadBorder(SpreadBorderStyle.Medium, new SpreadThemableColor(new SpreadColor(0, 0, 255)));
                ISpreadFill fill2 = new SpreadPatternFill(
                    SpreadPatternType.ThinVerticalStripe,
                    new SpreadThemableColor(new SpreadColor(0, 255, 0)),
                    new SpreadThemableColor(new SpreadColor(0, 0, 255)));

                SpreadCellFormat secondWorksheetFirstCellFormat = new SpreadCellFormat()
                {
                    ApplyAlignmentInternal = true,
                    ApplyBorderInternal = true,
                    ApplyFillInternal = true,
                    ApplyFontInternal = true,
                    ApplyNumberFormatInternal = true,
                    ApplyProtectionInternal = true,
                    BottomBorder = border2,
                    DiagonalDownBorder = border2,
                    DiagonalUpBorder = border2,
                    Fill = fill2,
                    FontFamily = new SpreadThemableFontFamily("Arial"),
                    FontSize = 8,
                    ForeColor = new SpreadThemableColor(SpreadThemeColorType.Accent2),
                    HorizontalAlignment = SpreadHorizontalAlignment.Right,
                    Indent = 7,
                    IsBold = false,
                    IsItalic = false,
                    LeftBorder = border2,
                    NumberFormat = @"_(_$* #,##0.00_);_(_$* (#,##0.00);_(_$* "" - ""??_);_(@_)",
                    RightBorder = border2,
                    TopBorder = border2,
                    Underline = SpreadUnderlineType.Single,
                    VerticalAlignment = SpreadVerticalAlignment.Top,
                    WrapText = false
                };

                SpreadCellFormat firstWorksheetSecondCellFormat = new SpreadCellFormat();
                SpreadCellFormat secondWorksheetSecondCellFormat = new SpreadCellFormat();

                int firstWorksheetFirstCellValue = 24;
                int firstWorksheetSecondCellValue = 402;
                int secondWorksheetFirstCellValue = 41;
                int secondWorksheetSecondCellValue = 921312;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                {
                    firstWorksheetSecondCellFormat.CellStyle = workbook.CellStyles["Bad"];
                    secondWorksheetSecondCellFormat.CellStyle = workbook.CellStyles["Good"];

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetFormat(firstWorksheetFirstCellFormat);
                                cell.SetValue(firstWorksheetFirstCellValue);
                            }

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetFormat(firstWorksheetSecondCellFormat);
                                cell.SetValue(firstWorksheetSecondCellValue);
                            }
                        }

                    }
                }

                stream.Seek(0, SeekOrigin.Begin);

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetFormat(secondWorksheetFirstCellFormat);
                                cell.SetValue(secondWorksheetFirstCellValue);
                            }

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetFormat(secondWorksheetSecondCellFormat);
                                cell.SetValue(secondWorksheetSecondCellValue);
                            }
                        }

                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = firstWorksheet.Cells[0, 0];

                    Assert.AreEqual(firstWorksheetFirstCellValue.ToString(), cell.Value);
                    Assert.IsTrue(cell.IsCellFormatEqual(firstWorksheetFirstCellFormat));

                    CellWrapper cell1 = firstWorksheet.Cells[0, 1];

                    Assert.AreEqual(firstWorksheetSecondCellValue.ToString(), cell1.Value);
                    Assert.IsTrue(cell1.CellStyle.IsCellStyleEqual(firstWorksheetSecondCellFormat.CellStyle));


                    WorksheetWrapper secondWorksheet = workbookWraper.Worksheets[1];
                    CellWrapper cell3 = secondWorksheet.Cells[0, 0];

                    Assert.AreEqual(secondWorksheetFirstCellValue.ToString(), cell3.Value);
                    Assert.IsTrue(cell3.IsCellFormatEqual(secondWorksheetFirstCellFormat));

                    CellWrapper cell4 = secondWorksheet.Cells[0, 1];

                    Assert.AreEqual(secondWorksheetSecondCellValue.ToString(), cell4.Value);
                    Assert.IsTrue(cell4.CellStyle.IsCellStyleEqual(secondWorksheetSecondCellFormat.CellStyle));
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.Append)]
        public void TestAppendWorksheetToWorksheetContainingFormats()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                string styleName = string.Empty;
                SpreadDocumentFormat documentFormat = SpreadDocumentFormat.Xlsx;

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream, SpreadExportMode.Create))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Date"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(3123);
                            }
                        }
                    }
                }

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream, SpreadExportMode.Append))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("CellStyles"))
                    {
                        SpreadCellStyle cellStyle = workbook.CellStyles.First();
                        styleName = cellStyle.Name;

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue(cellStyle.Name);
                                cell.SetFormat(new SpreadCellFormat()
                                {
                                    CellStyle = cellStyle
                                });
                            }
                        }
                    }
                }

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(documentFormat, stream, SpreadExportMode.Append))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Date5"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("gh");
                            }
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[1];
                    CellWrapper cell = firstWorksheet.Cells[0, 0];
                    Assert.AreEqual(styleName, cell.CellStyle.Name);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.Append)]
        public void TestAppendWorksheetDoNotReorderTheOldOnes()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("ThreeWorksheets.xlsx"))
            {
                string sheet4Name = "Sheet4";


                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                    {
                        using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(sheet4Name))
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                                using (ICellExporter cell = row.CreateCellExporter())
                                {
                                    cell.SetValue("gh");
                                }
                            }
                        }
                    }

#if !VISUAL
                    string sheet1Name = "Sheet1";
                    string sheet2Name = "Sheet2";
                    string sheet3Name = "Sheet3";

                    using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                    {
                        WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                        Assert.AreEqual(sheet1Name, firstWorksheet.Name);

                        WorksheetWrapper secondWorksheet = workbookWraper.Worksheets[1];
                        Assert.AreEqual(sheet2Name, secondWorksheet.Name);

                        WorksheetWrapper thirdWorksheet = workbookWraper.Worksheets[2];
                        Assert.AreEqual(sheet3Name, thirdWorksheet.Name);

                        WorksheetWrapper fourthWorksheet = workbookWraper.Worksheets[3];
                        Assert.AreEqual(sheet4Name, fourthWorksheet.Name);
                    }
#else
                    this.AssertTestVisual(stream);
#endif
                }
            }
        }

#if VISUAL
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.Append)]
        public void TestAppendWorksheetToWorksheetCreatedByExcel()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("TestAppendManyFeatures.xlsx"))
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                    {
                        using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Date5"))
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                                using (ICellExporter cell = row.CreateCellExporter())
                                {
                                    cell.SetValue("gh");
                                }
                            }
                        }
                    }

                    this.AssertTestVisual(stream);
                }
            }
        }
#else
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.Append)]
        public void TestAppendSameStyleAsExistingOneInTheDocument()
        {
            using (var resourceStream = ResourceHelper.GetResourceStream("TestAppendSameStyleAsExistingOneInTheDocument.xlsx"))
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    resourceStream.CopyTo(stream);
                    stream.Seek(0, SeekOrigin.Begin);

                    using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream, SpreadExportMode.Append))
                    {
                        using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("test"))
                        {
                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                                using (ICellExporter cell = row.CreateCellExporter())
                                {
                                    cell.SetValue("gh");
                                    cell.SetFormat(new SpreadCellFormat()
                                    {
                                        Fill = SpreadPatternFill.CreateSolidFill(new SpreadColor(0, 255, 0))
                                    });
                                }
                            }
                        }
                    }

                    using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                    {
                        WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[1];
                        var fill = firstWorksheet.Cells[0, 0].PatternFill;
                        var color = fill.BackgroundColor.LocalValue;

                        Assert.AreEqual(0, color.R);
                        Assert.AreEqual(255, color.G);
                        Assert.AreEqual(0, color.B);
                    }
                }
            }
        }
#endif
    }
}