﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers;
#endif

using System.Collections.Generic;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.XlsxExport
{
    [TestClass]
    public class CellStylesXlsxTests : XlsxTestBase
    {
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellStyles)]
        public void SetAllCellStylesTest()
        {
            int cellStylesPerRow = 5;

            SpreadCellStyleCollection cellStylesCollection;
            List<SpreadCellStyle> cellStyles;
            int rowsCount;

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    cellStylesCollection = workbook.CellStyles;
                    cellStyles = workbook.CellStyles.ToList();
                    rowsCount = cellStyles.Count / cellStylesPerRow;

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        SpreadCellStyle style = workbook.CellStyles.Add("MyStyle");

                        style.Fill = SpreadPatternFill.CreateSolidFill(new SpreadColor(0, 255, 0));

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                SpreadCellStyle cellStyle = style;
                                cell.SetValue(cellStyle.Name);
                                cell.SetFormat(new SpreadCellFormat()
                                {
                                    CellStyle = cellStyle
                                });
                            }
                        }

                        for (int rowIndex = 0; rowIndex <= rowsCount; rowIndex++)
                        {
                            worksheet.SkipRows(1);

                            using (IRowExporter row = worksheet.CreateRowExporter())
                            {
                                for (int cellsCount = 0; cellsCount < cellStylesPerRow; cellsCount++)
                                {
                                    row.SkipCells(1);

                                    using (ICellExporter cell = row.CreateCellExporter())
                                    {
                                        int cellStyleIndex = rowIndex * cellStylesPerRow + cellsCount;
                                        if (cellStyleIndex == cellStyles.Count)
                                        {
                                            break;
                                        }

                                        SpreadCellStyle cellStyle = cellStyles[cellStyleIndex];
                                        cell.SetValue(cellStyle.Name);
                                        cell.SetFormat(new SpreadCellFormat()
                                        {
                                            CellStyle = cellStyle
                                        });
                                    }
                                }
                            }
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    string customStyleName = "MyStyle";
                    CellStyleWrapper actualCellStyle = workbookWraper.CellStyles.GetByName(customStyleName);
                    SpreadCellStyle cellStyle = cellStylesCollection.GetByName(customStyleName);

                    Assert.AreEqual(cellStyle.Name, value);
                    Assert.IsTrue(actualCellStyle.IsCellStyleEqual(cellStyle));

                    int cellStyleIndex = 0;
                    
                    for (int rowIndex = 2; rowIndex <= (rowsCount + 2) * 2; rowIndex += 2)
                    {
                        for (int columnIndex = 1; columnIndex <= cellStylesPerRow * 2; columnIndex += 2)
                        {
                            if (cellStyleIndex == cellStyles.Count)
                            {
                                break;
                            }

                            SpreadCellStyle expectedCellStyle = cellStyles[cellStyleIndex];
                            cellStyleIndex++;

                            string actualStyleName = worksheet.Cells[rowIndex, columnIndex].Value;
                            var actualStyle = workbookWraper.CellStyles.GetByName(expectedCellStyle.Name);

                            Assert.AreEqual(expectedCellStyle.Name, actualStyleName);
                            Assert.IsTrue(actualStyle.IsCellStyleEqual(expectedCellStyle));
                        }
                    }
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.CellStyles)]
        public void SetCustomStyleToACellThenModifyTheStyleAndApplyToAnotherCell()
        {
            SpreadCellStyleCollection cellStylesCollection;
            SpreadCellStyle style;

            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    cellStylesCollection = workbook.CellStyles;

                    SpreadThemableColor color1 = new SpreadThemableColor(new SpreadColor(255, 0, 0));
                    SpreadThemableColor color2 = new SpreadThemableColor(new SpreadColor(255, 0, 255));
                    style = workbook.CellStyles.Add("MyStyle");
                    style.Fill = new SpreadPatternFill(SpreadPatternType.ThickDiagonalCrosshatch, color1, color2);
                    style.FontFamily = new SpreadThemableFontFamily(SpreadThemeFontType.Minor);
                    style.FontSize = 20;
                    style.ForeColor = new SpreadThemableColor(new SpreadColor(255, 255, 255));
                    style.HorizontalAlignment = SpreadHorizontalAlignment.Right;
                    style.VerticalAlignment = SpreadVerticalAlignment.Bottom;
                    style.IsBold = true;
                    style.IsItalic = true;
                    style.Underline = SpreadUnderlineType.Single;
                    style.WrapText = true;

                    SpreadCellFormat cellFormat = new SpreadCellFormat();
                    cellFormat.CellStyle = style;

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("CellStyle"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInPixels(110);
                        }

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("This is some loooong long text");
                                cell.SetFormat(cellFormat);
                    }

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("Test text");
                                style.Fill = new SpreadPatternFill(SpreadPatternType.VerticalStripe, color1, color2);
                                style.ForeColor = new SpreadThemableColor(new SpreadColor(0, 0, 0));
                                cell.SetFormat(cellFormat);
                            }
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper firstCell = worksheet.Cells[0, 0];
                    CellWrapper secondCell = worksheet.Cells[0, 1];
                    Assert.AreEqual(firstCell.TextProperties.ForeColor, secondCell.TextProperties.ForeColor);
                    Assert.IsTrue(TestsHelper.IsCellStyleEqual(firstCell.CellStyle, style));
                    Assert.IsTrue(TestsHelper.IsCellStyleEqual(secondCell.CellStyle, style));
                }
#endif
            }
        }
    }
}