﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers;
#endif

using System;
using System.IO;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.XlsxExport
{
    [TestClass]
    public class RowsColumnsXlsxTests : XlsxTestBase
    {
        private static double DefaultColumnWidth = 8.43;
#if EXCEL || SPREAD
        private static double DefaultRowHeight = 15;
#endif

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SetNegativeOutlineLevelOfRowTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetOutlineLevel(-1);
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SetNegativeOutlineLevelOfColumnTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1");

                    using (IColumnExporter column = worksheet.CreateColumnExporter())
                    {
                        column.SetOutlineLevel(-1);
                    }

                    worksheet.Dispose();
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void SetRowHeightColumnWidthOutlineLevels()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInCharacters(30);
                        }

                        this.GroupColumns(worksheet, false);

                        worksheet.SkipColumns(2);
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInCharacters(DefaultColumnWidth);
                        }

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetHeightInPoints(30);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                            }
                        }

                        this.GroupRows(worksheet, false);
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);

                    Assert.AreEqual(30, Math.Round(worksheet.Columns[0].WidthInCharacters));
                    Assert.AreEqual(30, worksheet.Rows[0].HeightInPoints);

                    for (int i = 2; i <= 16; i++)
                    {
                        Assert.AreEqual(Math.Round(DefaultColumnWidth), Math.Round(worksheet.Columns[i].WidthInCharacters));
                    }

                    for (int i = 2; i <= 28; i++)
                    {
                        Assert.AreEqual(DefaultRowHeight, worksheet.Rows[i].HeightInPoints);
                    }

                    Assert.AreEqual(Math.Floor(DefaultColumnWidth), Math.Floor(worksheet.Columns[17].WidthInCharacters));
                    Assert.AreEqual(DefaultRowHeight, worksheet.Rows[29].HeightInPoints);

                    this.AssertColumnsOutlineLevels(worksheet.Columns);
                    this.AssertRowsOutlineLevels(worksheet.Rows);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void SetRowHeightColumnWidthOutlineLevelsCollapsedTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInCharacters(30);
                        }

                        this.GroupColumns(worksheet, true);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetHeightInPoints(30);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                            }
                        }

                        this.GroupRows(worksheet, true);
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[0, 0];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);

                    Assert.AreEqual(30, Math.Round(worksheet.Columns[0].WidthInCharacters));
                    Assert.AreEqual(30, worksheet.Rows[0].HeightInPoints);

                    for (int i = 2; i <= 16; i++)
                    {
                        Assert.AreEqual(0, worksheet.Columns[i].WidthInCharacters);
                    }

                    for (int i = 2; i <= 28; i++)
                    {
                        // The height of the row is not set, they are just hidden.
                        // In Excel the width is the default
                        // In SpreadProcessing is 0. This might be not right.
#if EXCEL
                        Assert.AreEqual(0, worksheet.Rows[i].HeightInPoints);
#else
                        Assert.AreEqual(DefaultRowHeight, worksheet.Rows[i].HeightInPoints);
#endif
                    }

                    Assert.AreEqual(Math.Floor(DefaultColumnWidth), Math.Floor(worksheet.Columns[17].WidthInCharacters));
                    Assert.AreEqual(DefaultRowHeight, worksheet.Rows[29].HeightInPoints);

                    this.AssertColumnsOutlineLevels(worksheet.Columns);
                    this.AssertRowsOutlineLevels(worksheet.Rows);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void HideRowsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        HideRows(worksheet);
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                AssertRowsVisibility(stream, this.SpreadDocumentFormat);
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void HideColumnsTes()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        HideColumns(worksheet);

                        // Should be removed after fix http://tpdogfood.telerik.com/view#item/185580
                        CreateEmptyRow(worksheet);
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                AssertColumnsVisibility(stream, this.SpreadDocumentFormat);
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void HideRowsColumnsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        HideColumns(worksheet);

                        HideRows(worksheet);
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                AssertColumnsVisibility(stream, this.SpreadDocumentFormat);
                AssertRowsVisibility(stream, this.SpreadDocumentFormat);
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void SetNegativeColumnWidthInPixels()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1");

                    using (IColumnExporter column = worksheet.CreateColumnExporter())
                    {
                        column.SetWidthInPixels(-1);
                    }

                    worksheet.Dispose();
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void ColumnWithNegativeWidthInCharacters()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1");
                    using (IColumnExporter column = worksheet.CreateColumnExporter())
                    {
                        column.SetWidthInCharacters(-1);
                    }

                    using (IRowExporter row = worksheet.CreateRowExporter())
                    {
                    }

                    worksheet.Dispose();
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void RowWithNegativeHeighrInPixelsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetHeightInPixels(-1);
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void RowWithNegativeHeightInPointsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetHeightInPoints(-1);
                        }
                    }
                }
            }
        }

#if VISUAL
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.RowsColumns)]
        public void RowAndColumnWithZeroHeightWidthInPointsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetWidthInPixels(0);
                        }
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SetHeightInPoints(0);
                        }
                    }
                }

                this.AssertTestVisual(stream);
            }
        }
#endif

        private static void CreateEmptyRow(IWorksheetExporter worksheet)
        {
            using (IRowExporter row = worksheet.CreateRowExporter())
            {

            }
        }

        private static void HideColumns(IWorksheetExporter worksheet)
        {
            for (int i = 0; i < 6; i++)
            {
                if (i % 2 != 0)
                {
                    worksheet.SkipColumns(1);
                }
                else
                {
                    using (IColumnExporter column = worksheet.CreateColumnExporter())
                    {
                        if (i % 2 == 0)
                        {
                            column.SetHidden(true);
                        }
                    }
                }

            }
        }

        private static void HideRows(IWorksheetExporter worksheet)
        {
            for (int i = 0; i < 6; i++)
            {
                if (i % 2 != 0)
                {
                    worksheet.SkipRows(1);
                }
                else
                {
                    using (IRowExporter row = worksheet.CreateRowExporter())
                    {
                        if (i % 2 == 0)
                        {
                            row.SetHidden(true);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("hidden" + i);
                            }
                        }
                    }
                }
            }
        }

        private void GroupColumns(IWorksheetExporter worksheet, bool hidden)
        {
            worksheet.SkipColumns(1);

            for (int i = 0; i < 2; i++)
            {
                using (IColumnExporter column = worksheet.CreateColumnExporter())
                {
                    column.SetOutlineLevel(1);
                    if (!hidden)
                    {
                        column.SetWidthInCharacters(DefaultColumnWidth);
                    }
                    column.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 3; i++)
            {
                using (IColumnExporter column = worksheet.CreateColumnExporter())
                {
                    column.SetOutlineLevel(2);
                    if (!hidden)
                    {
                        column.SetWidthInCharacters(DefaultColumnWidth);
                    }
                    column.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 5; i++)
            {
                using (IColumnExporter column = worksheet.CreateColumnExporter())
                {
                    column.SetOutlineLevel(3);
                    if (!hidden)
                    {
                        column.SetWidthInCharacters(DefaultColumnWidth);
                    }
                    column.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 3; i++)
            {
                using (IColumnExporter column = worksheet.CreateColumnExporter())
                {
                    column.SetOutlineLevel(2);
                    if (!hidden)
                    {
                        column.SetWidthInCharacters(DefaultColumnWidth);
                    }
                    column.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 2; i++)
            {
                using (IColumnExporter column = worksheet.CreateColumnExporter())
                {
                    column.SetOutlineLevel(1);
                    if (!hidden)
                    {
                        column.SetWidthInCharacters(DefaultColumnWidth);
                    }
                    column.SetHidden(hidden);
                }
            }
        }

        private void GroupRows(IWorksheetExporter worksheet, bool hidden)
        {
            worksheet.SkipRows(1);

            for (int i = 0; i < 2; i++)
            {
                using (IRowExporter row = worksheet.CreateRowExporter())
                {
                    row.SetOutlineLevel(1);
                    row.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 5; i++)
            {
                using (IRowExporter row = worksheet.CreateRowExporter())
                {
                    row.SetOutlineLevel(2);
                    row.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 15; i++)
            {
                using (IRowExporter row = worksheet.CreateRowExporter())
                {
                    row.SetOutlineLevel(3);
                    row.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 3; i++)
            {
                using (IRowExporter row = worksheet.CreateRowExporter())
                {
                    row.SetOutlineLevel(2);
                    row.SetHidden(hidden);
                }
            }

            for (int i = 0; i < 2; i++)
            {
                using (IRowExporter row = worksheet.CreateRowExporter())
                {
                    row.SetOutlineLevel(1);
                    row.SetHidden(hidden);
                }
            }
        }

#if EXCEL || SPREAD
        private static void AssertRowsVisibility(MemoryStream stream, SpreadDocumentFormat documentFormat)
        {
            using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, documentFormat))
            {
                WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                RowWrapper row1 = worksheet.Rows[0];
                Assert.AreEqual(true, row1.IsHidden, "Row 1 is not hidden");

                RowWrapper row3 = worksheet.Rows[2];
                Assert.AreEqual(true, row3.IsHidden, "Row 3 is not hidden");

                RowWrapper row5 = worksheet.Rows[4];
                Assert.AreEqual(true, row5.IsHidden, "Row 6 is not hidden");

            }
        }

        private static void AssertColumnsVisibility(MemoryStream stream, SpreadDocumentFormat documentFormat)
        {
            using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, documentFormat))
            {
                WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                ColumnWrapper columnA = worksheet.Columns[0];
                Assert.AreEqual(true, columnA.IsHidden, "Column A is not hidden");

                ColumnWrapper columnC = worksheet.Columns[2];
                Assert.AreEqual(true, columnA.IsHidden, "Column C is not hidden");

                ColumnWrapper columnE = worksheet.Columns[4];
                Assert.AreEqual(true, columnA.IsHidden, "Column B is not hidden");

            }
        }

        private void AssertColumnsOutlineLevels(ColumnsWrapper columns)
        {
            for (int i = 0; i <= 1; i++)
            {
                Assert.AreEqual(0, columns[i].OutlineLevel);
            }

            for (int i = 2; i <= 3; i++)
            {
                Assert.AreEqual(1, columns[i].OutlineLevel);
            }

            for (int i = 4; i <= 6; i++)
            {
                Assert.AreEqual(2, columns[i].OutlineLevel);
            }

            for (int i = 7; i <= 11; i++)
            {
                Assert.AreEqual(3, columns[i].OutlineLevel);
            }

            for (int i = 12; i <= 14; i++)
            {
                Assert.AreEqual(2, columns[i].OutlineLevel);
            }

            for (int i = 15; i <= 16; i++)
            {
                Assert.AreEqual(1, columns[i].OutlineLevel);
            }

            for (int i = 17; i <= 20; i++)
            {
                Assert.AreEqual(0, columns[i].OutlineLevel);
            }
        }

        private void AssertRowsOutlineLevels(RowsWrapper rows)
        {
            for (int i = 0; i <= 1; i++)
            {
                Assert.AreEqual(0, rows[i].OutlineLevel);
            }

            for (int i = 2; i <= 3; i++)
            {
                Assert.AreEqual(1, rows[i].OutlineLevel);
            }

            for (int i = 4; i <= 8; i++)
            {
                Assert.AreEqual(2, rows[i].OutlineLevel);
            }

            for (int i = 9; i <= 23; i++)
            {
                Assert.AreEqual(3, rows[i].OutlineLevel);
            }

            for (int i = 24; i <= 26; i++)
            {
                Assert.AreEqual(2, rows[i].OutlineLevel);
            }

            for (int i = 27; i <= 28; i++)
            {
                Assert.AreEqual(1, rows[i].OutlineLevel);
            }

            for (int i = 29; i <= 32; i++)
            {
                Assert.AreEqual(0, rows[i].OutlineLevel);
            }
        }
#endif
    }
}