﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers;
#endif
using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming.Tests.ExportBase;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.XlsxExport
{
    [TestClass]
    public class WorksheetsXlsxTest : WorksheetTestBase
    {
        protected override SpreadDocumentFormat SpreadDocumentFormat
        {
            get
            {
                return SpreadDocumentFormat.Xlsx;
            }
        }

        protected override string TestCategory
        {
            get
            {
                return SpreadsheetStreamingCategory.XlsxExport;
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.Worksheet)]
        public void CreateDocumentContainingMultipleWorksheetsXlsx()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "First";
            string secondWorksheetName = "Second";
            string thirdWorksheetName = "Sheet3";

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(secondWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(thirdWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    var firstWorksheet = workbookWraper.Worksheets[0];
                    Assert.AreEqual(firstWorksheet.Name, firstWorksheetName);

                    var secondWorksheet = workbookWraper.Worksheets[1];
                    Assert.AreEqual(secondWorksheet.Name, secondWorksheetName);

                    var thirdWorksheet = workbookWraper.Worksheets[2];
                    Assert.AreEqual(thirdWorksheet.Name, thirdWorksheetName);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.Worksheet)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void CreateWorksheetWithinAWorksheetXlsxTest()
        {
            MemoryStream stream = null;

            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";
                string secondWorksheetName = "Second";

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                        }

                        using (IWorksheetExporter worksheet1 = workbook.CreateWorksheetExporter(secondWorksheetName))
                        {
                            using (worksheet1.CreateRowExporter())
                            {
                            }
                        }
                    }
                }
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.Worksheet)]
        [ExpectedException(typeof(ArgumentException))]
        public void CreateWorksheetsWithSameNames()
        {
            MemoryStream stream = null;

            using (stream = new MemoryStream())
            {
                string firstWorksheetName = "First";

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        { }
                    }

                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (worksheet.CreateRowExporter())
                        {
                            using (worksheet.CreateRowExporter())
                            { }
                        }
                    }
                }
            }
        }

#if SPREAD
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void ExportWorkbookWithEmptyWorksheetTest()
        {
            this.CreateWorkbookWithEmptyWorksheet();
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void ExportWorkbookContainingWorksheetWithOnlyOneColumnTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Date"))
                    {
                        using (IColumnExporter column = worksheet.CreateColumnExporter())
                        {
                            column.SetHidden(true);
                        }
                    }
                }
            }
        }
#endif
    }
}
