﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.ExcelWrappers;
#endif

using Telerik.Documents.SpreadsheetStreaming.Tests;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Linq;
using Telerik.Documents.SpreadsheetStreaming;
using Telerik.Documents.SpreadsheetStreaming.VisualTests.Tests;
using System;
using Telerik.Documents.SpreadsheetStreaming.Model;
using System.Collections.Generic;

namespace Telerik.Documents.SpreadsheetStreaming.ExcelInteropTests.Tests
{
    [TestClass]
    public class MergedCellsXlsxTests : XlsxTestBase
    {
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergedCellsTest()
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                SpreadThemableColor color = new SpreadThemableColor(new SpreadColor(255, 255, 100));

                SpreadCellFormat centerAlignmentFormat = new SpreadCellFormat();
                centerAlignmentFormat.HorizontalAlignment = SpreadHorizontalAlignment.Center;
                centerAlignmentFormat.Fill = new SpreadPatternFill(SpreadPatternType.Solid, color, color);

                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet2"))
                    {
                        worksheet.SkipRows(1);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(1);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(centerAlignmentFormat);
                            }

                            row.SkipCells(3);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(centerAlignmentFormat);
                            }
                        }

                        worksheet.SkipRows(5);

                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {
                            row.SkipCells(1);

                            using (ICellExporter cell = row.CreateCellExporter())
                            {
                                cell.SetValue("test");
                                cell.SetFormat(centerAlignmentFormat);
                            }
                        }

                        worksheet.MergeCells(1, 1, 5, 3);
                        worksheet.MergeCells(1, 5, 5, 5);
                        worksheet.MergeCells(7, 1, 7, 4);
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper worksheet = workbookWraper.Worksheets[0];
                    CellWrapper cell = worksheet.Cells[1, 1];
                    string value = cell.Value;

                    Assert.AreEqual("test", value);
                    Assert.IsTrue(cell.IsCellFormatEqual(centerAlignmentFormat));

                    CellWrapper cell1 = worksheet.Cells[1, 5];
                    string value1 = cell1.Value;

                    Assert.AreEqual("test", value1);
                    Assert.IsTrue(cell1.IsCellFormatEqual(centerAlignmentFormat));

                    CellWrapper cell2 = worksheet.Cells[1, 5];
                    string value2 = cell2.Value;

                    Assert.AreEqual("test", value2);
                    Assert.IsTrue(cell2.IsCellFormatEqual(centerAlignmentFormat));

                    var mergedCells = worksheet.MergedCells.ToArray();

                    Assert.AreEqual(new CellRangeWrapper(1, 1, 5, 3), mergedCells[0].Range);
                    Assert.AreEqual(new CellRangeWrapper(1, 5, 5, 5), mergedCells[1].Range);
                    Assert.AreEqual(new CellRangeWrapper(7, 1, 7, 4), mergedCells[2].Range);
                }
#endif
            }
        }

        // rows

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexEqual_ToRowIndexEqual_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(15, 3, 20, 6);
            SpreadCellRange secondRange = new SpreadCellRange(15, 3, 20, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexEqual_ToRowIndexSmaller_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(5, 0, 10, 6);
            SpreadCellRange secondRange = new SpreadCellRange(5, 0, 8, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexEqual_ToRowIndexGreater_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(5, 0, 10, 6);
            SpreadCellRange secondRange = new SpreadCellRange(5, 0, 12, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexSmaller_ToRowIndexSmaller_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(10, 0, 20, 6);
            SpreadCellRange secondRange = new SpreadCellRange(1, 0, 5, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexIsSmaller_ToRowIndexEqual_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(3, 0, 5, 6);
            SpreadCellRange secondRange = new SpreadCellRange(0, 0, 5, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexIsSmaller_ToRowIndexGreater_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(3, 0, 5, 6);
            SpreadCellRange secondRange = new SpreadCellRange(0, 0, 6, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexGreater_ToRowIndexSmaller_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 4, 6);
            SpreadCellRange secondRange = new SpreadCellRange(1, 0, 2, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexIsGreater_ToRowIndexEqual_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(3, 0, 5, 6);
            SpreadCellRange secondRange = new SpreadCellRange(4, 0, 5, 6);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromRowIndexIsGreater_ToRowIndexGreater_ColumnsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(3, 0, 5, 6);
            SpreadCellRange secondRange = new SpreadCellRange(4, 0, 6, 6);
            MergeRanges(firstRange, secondRange);
        }

        // Columns

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexEqual_ToColumnIndexSmaller_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 0, 10, 5);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexEqual_ToColumnIndexGreater_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 5, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 0, 5, 15);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexSmaller_ToColumnIndexSmaller_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 0, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 2, 0, 8);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexSmaller_ToColumnIndexEqual_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 2, 10, 10);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexSmaller_ToColumnIndexGreater_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 2, 10, 15);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexGreater_ToColumnIndexSmaller_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 7, 10, 8);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexGreater_ToColumnIndexEqual_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 7, 10, 10);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void MergeCellsThatAreAlreadyMerged_FromColumnIndexGreater_ToColumnIndexGreater_RowsAreTheSameTest()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 5, 10, 10);
            SpreadCellRange secondRange = new SpreadCellRange(0, 7, 10, 15);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeCells_SingleRow()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 0, 10);
            SpreadCellRange secondRange = new SpreadCellRange(3, 0, 3, 10);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeCells_SingleColumn()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 10, 0);
            SpreadCellRange secondRange = new SpreadCellRange(0, 3, 10, 3);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeTwoRanges_OnDifferentColumnAndRows()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 4, 4);
            SpreadCellRange secondRange = new SpreadCellRange(6, 6, 10, 10);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeTwoRanges_OnSameRows()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 4, 4);
            SpreadCellRange secondRange = new SpreadCellRange(0, 5, 4, 10);
            MergeRanges(firstRange, secondRange);
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.MergedCells)]
        public void MergeTwoRanges_OnSameColumns()
        {
            SpreadCellRange firstRange = new SpreadCellRange(0, 0, 4, 4);
            SpreadCellRange secondRange = new SpreadCellRange(5, 0, 10, 4);
            MergeRanges(firstRange, secondRange);
        }

        private void MergeRanges(SpreadCellRange firstRange, SpreadCellRange secondRange)
        {
            List<SpreadCellRange> ranges = new List<SpreadCellRange>();
            ranges.Add(firstRange);
            ranges.Add(secondRange);

            MergeRanges(ranges);
        }

        private void MergeRanges(IEnumerable<SpreadCellRange> mergeRanges)
        {
            MemoryStream stream = null;
            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(SpreadDocumentFormat.Xlsx, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter("Sheet1"))
                    {
                        // this using is needed to create valid document.
                        using (IRowExporter row = worksheet.CreateRowExporter())
                        {

                        }

                        foreach (SpreadCellRange range in mergeRanges)
                        {
                            worksheet.MergeCells(range.FromRowIndex, range.FromColumnIndex, range.ToRowIndex, range.ToColumnIndex);
                        }
                    }
                }
#if VISUAL
                this.AssertTestVisual(stream);
#endif
            }
        }
    }
}