﻿#if EXCEL || SPREAD
using Telerik.Documents.SpreadsheetStreaming.Tests.ExcelWrappers;
#endif

using System;
using System.IO;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.Documents.SpreadsheetStreaming.Tests.Utils;
using Telerik.Documents.SpreadsheetStreaming.VisualTests.Tests;
using Telerik.Documents.SpreadsheetStreaming.Model;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Tests.XlsxExportTests
{
    [TestClass]
    public class WorksheetViewStateXlsxTests : WorksheetTestBase
    {
        protected override SpreadDocumentFormat SpreadDocumentFormat
        {
            get
            {
                return SpreadDocumentFormat.Xlsx;
            }
        }

        protected override string TestCategory
        {
            get
            {
                return SpreadsheetStreamingCategory.XlsxExport;
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithActiveSelectionCellSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int activeSelectionCellRowIndex = 10;
            int activeSelectionCellColumnIndex = 5;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetActiveSelectionCell(activeSelectionCellRowIndex, activeSelectionCellColumnIndex);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    var viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(activeSelectionCellRowIndex, viewState.ActiveSelectionCellRowIndex);
                    Assert.AreEqual(activeSelectionCellColumnIndex, viewState.ActiveSelectionCellColumnIndex);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithScaleFactorSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            double scaleFactor = 2.5;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetScaleFactor(scaleFactor);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(scaleFactor, viewState.ScaleFactor);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithSingleSelectedRangeSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            SpreadCellRange selectedRange = new SpreadCellRange(10, 11, 20, 21);

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.AddSelectionRange(
                                selectedRange.FromRowIndex,
                                selectedRange.FromColumnIndex,
                                selectedRange.ToRowIndex,
                                selectedRange.ToColumnIndex);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(1, viewState.SelectedRanges.Count());
                    Assert.AreEqual(new CellRangeWrapper(selectedRange), viewState.SelectedRanges.First());
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithMultipleSelectedRangeSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            var selectedRanges = new SpreadCellRange[]
                                {
                                    new SpreadCellRange(10, 11, 12, 13),
                                    new SpreadCellRange(1, 1, 10, 10),
                                    new SpreadCellRange(2, 5, 55, 13)
                                };

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            foreach (var range in selectedRanges)
                            {
                                worksheetView.AddSelectionRange(
                                    range.FromRowIndex,
                                    range.FromColumnIndex,
                                    range.ToRowIndex,
                                    range.ToColumnIndex);
                            }
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(selectedRanges.Length, viewState.SelectedRanges.Count());

                    int index = 0;

                    foreach (CellRangeWrapper actualRange in viewState.SelectedRanges)
                    {
                        Assert.AreEqual(new CellRangeWrapper(selectedRanges[index]), actualRange);
                        index++;
                    }

                    Assert.AreEqual(selectedRanges[0].FromRowIndex, viewState.ActiveSelectionCellRowIndex);
                    Assert.AreEqual(selectedRanges[0].FromColumnIndex, viewState.ActiveSelectionCellColumnIndex);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentNotShowingGridlines()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            bool shouldShowGridLines = false;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetShouldShowGridLines(shouldShowGridLines);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(shouldShowGridLines, viewState.ShouldShowGridLines);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentShowingGridlines()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            bool shouldShowGridLines = true;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetShouldShowGridLines(shouldShowGridLines);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(shouldShowGridLines, viewState.ShouldShowGridLines);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithSplitOffsetSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int splitRowOffset = 10;
            int splitColumnOffset = 6;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetFreezePanes(splitRowOffset, splitColumnOffset);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(splitRowOffset, viewState.SplitRowOffset);
                    Assert.AreEqual(splitColumnOffset, viewState.SplitColumnOffset);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentFrozenFirstRow()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int splitRowOffset = 1;
            int splitColumnOffset = 0;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetFreezePanes(splitRowOffset, splitColumnOffset);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(splitRowOffset, viewState.SplitRowOffset);
                    Assert.AreEqual(splitColumnOffset, viewState.SplitColumnOffset);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentFrozenFirstColumn()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int splitRowOffset = 0;
            int splitColumnOffset = 1;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetFreezePanes(splitRowOffset, splitColumnOffset);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(splitRowOffset, viewState.SplitRowOffset);
                    Assert.AreEqual(splitColumnOffset, viewState.SplitColumnOffset);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithTopLeftCellSetSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int rowIndex = 5;
            int columnIndex = 7;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetFirstVisibleCell(rowIndex, columnIndex);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(rowIndex, viewState.TopLeftCellIndex.FromRowIndex);
                    Assert.AreEqual(columnIndex, viewState.TopLeftCellIndex.FromColumnIndex);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentWithTopLeftCellSetAndSplitPanesSet()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int rowIndex = 5;
            int columnIndex = 7;

            int splitRowOffset = 2;
            int splitColumnOffset = 2;

            int frozenRowIndex = 12;
            int frozenColumnIndex = 13;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetFirstVisibleCell(rowIndex, columnIndex);
                            worksheetView.SetFreezePanes(splitRowOffset, splitColumnOffset, frozenRowIndex, frozenColumnIndex);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;

                    Assert.AreEqual(rowIndex, viewState.TopLeftCellIndex.FromRowIndex);
                    Assert.AreEqual(columnIndex, viewState.TopLeftCellIndex.FromColumnIndex);

                    Assert.AreEqual(splitRowOffset, viewState.SplitRowOffset);
                    Assert.AreEqual(splitColumnOffset, viewState.SplitColumnOffset);

                    Assert.AreEqual(frozenRowIndex, viewState.BottomRightPane.TopLeftCellIndex.FromRowIndex);
                    Assert.AreEqual(frozenColumnIndex, viewState.BottomRightPane.TopLeftCellIndex.FromColumnIndex);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentShowingRowColumnHeaders()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            bool shouldShowRowColumnHeaders = true;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetShouldShowRowColumnHeaders(shouldShowRowColumnHeaders);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(shouldShowRowColumnHeaders, viewState.ShouldShowRowColumnHeaders);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentNotShowingRowColumnHeaders()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            bool shouldShowRowColumnHeaders = false;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetShouldShowRowColumnHeaders(shouldShowRowColumnHeaders);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;
                    Assert.AreEqual(shouldShowRowColumnHeaders, viewState.ShouldShowRowColumnHeaders);
                }
#endif
            }
        }

        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        public void CreateDocumentSetAllViewStateProperties()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";
            int activeSelectionCellRowIndex = 1;
            int activeSelectionCellColumnIndex = 5;

            double scaleFactor = 0.75;

            var selectedRanges = new SpreadCellRange[]
                                {
                                    new SpreadCellRange(10, 11, 12, 13),
                                    new SpreadCellRange(1, 1, 10, 10),
                                    new SpreadCellRange(2, 5, 55, 13)
                                };

            bool shouldShowGridLines = false;

            int rowIndex = 5;
            int columnIndex = 7;

            int splitRowOffset = 2;
            int splitColumnOffset = 2;

            int frozenRowIndex = 12;
            int frozenColumnIndex = 13;

            bool shouldShowRowColumnHeaders = false;

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter worksheetView = worksheet.CreateWorksheetViewExporter())
                        {
                            worksheetView.SetActiveSelectionCell(activeSelectionCellRowIndex, activeSelectionCellColumnIndex);
                            worksheetView.SetScaleFactor(scaleFactor);
                            foreach (var range in selectedRanges)
                            {
                                worksheetView.AddSelectionRange(
                                    range.FromRowIndex,
                                    range.FromColumnIndex,
                                    range.ToRowIndex,
                                    range.ToColumnIndex);
                            }
                            worksheetView.SetShouldShowGridLines(shouldShowGridLines);
                            worksheetView.SetFirstVisibleCell(rowIndex, columnIndex);
                            worksheetView.SetFreezePanes(splitRowOffset, splitColumnOffset, frozenRowIndex, frozenColumnIndex);
                            worksheetView.SetFirstVisibleCell(rowIndex, columnIndex);
                            worksheetView.SetShouldShowRowColumnHeaders(shouldShowRowColumnHeaders);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }

#if VISUAL
                this.AssertTestVisual(stream);
#else
                using (WorkbookWrapper workbookWraper = WorkbookWrapper.Open(stream, this.SpreadDocumentFormat))
                {
                    WorksheetWrapper firstWorksheet = workbookWraper.Worksheets[0];
                    ViewStateWrapper viewState = firstWorksheet.ViewState;

                    Assert.AreEqual(activeSelectionCellRowIndex, viewState.ActiveSelectionCellRowIndex);
                    Assert.AreEqual(activeSelectionCellColumnIndex, viewState.ActiveSelectionCellColumnIndex);

                    Assert.AreEqual(scaleFactor, viewState.ScaleFactor);

                    Assert.AreEqual(selectedRanges.Length, viewState.SelectedRanges.Count());

                    int index = 0;

                    foreach (var actualRange in viewState.SelectedRanges)
                    {
                        Assert.AreEqual(new CellRangeWrapper(selectedRanges[index]), actualRange);
                        index++;
                    }

                    Assert.AreEqual(shouldShowGridLines, viewState.ShouldShowGridLines);

                    Assert.AreEqual(rowIndex, viewState.TopLeftCellIndex.FromRowIndex);
                    Assert.AreEqual(columnIndex, viewState.TopLeftCellIndex.FromColumnIndex);

                    Assert.AreEqual(splitRowOffset, viewState.SplitRowOffset);
                    Assert.AreEqual(splitColumnOffset, viewState.SplitColumnOffset);

                    Assert.AreEqual(frozenRowIndex, viewState.BottomRightPane.TopLeftCellIndex.FromRowIndex);
                    Assert.AreEqual(frozenColumnIndex, viewState.BottomRightPane.TopLeftCellIndex.FromColumnIndex);

                    Assert.AreEqual(shouldShowRowColumnHeaders, viewState.ShouldShowRowColumnHeaders);
                }
#endif
            }
        }

#if !VISUAL
        [TestMethod]
        [TestCategory(SpreadsheetStreamingCategory.SpreadsheetStreaming)]
        [TestCategory(SpreadsheetStreamingCategory.XlsxExport)]
        [TestCategory(SpreadsheetStreamingCategory.ViewState)]
        [ExpectedException(typeof(InvalidOperationException))]
        public void CreateDocumentSetInvalidActiveCell()
        {
            MemoryStream stream = null;
            string firstWorksheetName = "Sheet1";

            using (stream = new MemoryStream())
            {
                using (IWorkbookExporter workbook = SpreadExporter.CreateWorkbookExporter(this.SpreadDocumentFormat, stream))
                {
                    using (IWorksheetExporter worksheet = workbook.CreateWorksheetExporter(firstWorksheetName))
                    {
                        using (IWorksheetViewExporter viewExporter = worksheet.CreateWorksheetViewExporter())
                        {
                            viewExporter.AddSelectionRange(0, 0, 2, 2);
                            viewExporter.SetActiveSelectionCell(20, 20);
                        }

                        using (worksheet.CreateRowExporter())
                        {
                        }
                    }
                }
            }
        }
#endif
    }
}