﻿#if EXCEL
using Microsoft.Office.Interop.Excel;
using System;
using System.Collections.Generic;
using System.IO;
using Telerik.Documents.SpreadsheetStreaming.Model.Themes;
using Telerik.Documents.SpreadsheetStreaming.Tests.CommonWrappers;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Utils
{
    public static class ExcelInteropTestsHelper
    {
        private static readonly Dictionary<XlPattern, SpreadPatternType> xlPatternToPatternType;
        private static readonly Dictionary<XlHAlign, SpreadHorizontalAlignment> xlHAlignToHorizontalAlignment;
        private static readonly Dictionary<XlUnderlineStyle, SpreadUnderlineType> xlUndelineStyleToUnderline;
        private static readonly Dictionary<XlVAlign, SpreadVerticalAlignment> xlValignToVerticalAlignment;
        private static readonly Dictionary<XlThemeColor, SpreadThemeColorType> xlThemeColorToThemeColorType;

        static ExcelInteropTestsHelper()
        {
            xlPatternToPatternType = new Dictionary<XlPattern, SpreadPatternType>();
            xlPatternToPatternType.Add(XlPattern.xlPatternSolid, SpreadPatternType.Solid);
            xlPatternToPatternType.Add(XlPattern.xlPatternGray75, SpreadPatternType.Gray75Percent);
            xlPatternToPatternType.Add(XlPattern.xlPatternGray50, SpreadPatternType.Gray50Percent);
            xlPatternToPatternType.Add(XlPattern.xlPatternGray25, SpreadPatternType.Gray25Percent);
            xlPatternToPatternType.Add(XlPattern.xlPatternGray16, SpreadPatternType.Gray12Percent);
            xlPatternToPatternType.Add(XlPattern.xlPatternGray8, SpreadPatternType.Gray6Percent);
            xlPatternToPatternType.Add(XlPattern.xlPatternHorizontal, SpreadPatternType.HorizontalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternVertical, SpreadPatternType.VerticalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternDown, SpreadPatternType.ReverseDiagonalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternUp, SpreadPatternType.DiagonalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternChecker, SpreadPatternType.DiagonalCrosshatch);
            xlPatternToPatternType.Add(XlPattern.xlPatternSemiGray75, SpreadPatternType.ThickDiagonalCrosshatch);
            xlPatternToPatternType.Add(XlPattern.xlPatternLightHorizontal, SpreadPatternType.ThinHorizontalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternLightVertical, SpreadPatternType.ThinVerticalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternLightDown, SpreadPatternType.ThinReverseDiagonalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternLightUp, SpreadPatternType.ThinDiagonalStripe);
            xlPatternToPatternType.Add(XlPattern.xlPatternGrid, SpreadPatternType.ThinHorizontalCrossHatch);
            xlPatternToPatternType.Add(XlPattern.xlPatternCrissCross, SpreadPatternType.ThinDiagonalCrosshatch);

            xlHAlignToHorizontalAlignment = new Dictionary<XlHAlign, SpreadHorizontalAlignment>();
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignCenter, SpreadHorizontalAlignment.Center);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignCenterAcrossSelection, SpreadHorizontalAlignment.CenterContinuous);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignDistributed, SpreadHorizontalAlignment.Distributed);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignFill, SpreadHorizontalAlignment.Fill);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignGeneral, SpreadHorizontalAlignment.General);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignJustify, SpreadHorizontalAlignment.Justify);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignLeft, SpreadHorizontalAlignment.Left);
            xlHAlignToHorizontalAlignment.Add(XlHAlign.xlHAlignRight, SpreadHorizontalAlignment.Right);

            xlUndelineStyleToUnderline = new Dictionary<XlUnderlineStyle, SpreadUnderlineType>();
            xlUndelineStyleToUnderline.Add(XlUnderlineStyle.xlUnderlineStyleDouble, SpreadUnderlineType.Double);
            xlUndelineStyleToUnderline.Add(XlUnderlineStyle.xlUnderlineStyleDoubleAccounting, SpreadUnderlineType.DoubleAccounting);
            xlUndelineStyleToUnderline.Add(XlUnderlineStyle.xlUnderlineStyleNone, SpreadUnderlineType.None);
            xlUndelineStyleToUnderline.Add(XlUnderlineStyle.xlUnderlineStyleSingle, SpreadUnderlineType.Single);
            xlUndelineStyleToUnderline.Add(XlUnderlineStyle.xlUnderlineStyleSingleAccounting, SpreadUnderlineType.SingleAccounting);

            xlValignToVerticalAlignment = new Dictionary<XlVAlign, SpreadVerticalAlignment>();
            xlValignToVerticalAlignment.Add(XlVAlign.xlVAlignBottom, SpreadVerticalAlignment.Bottom);
            xlValignToVerticalAlignment.Add(XlVAlign.xlVAlignCenter, SpreadVerticalAlignment.Center);
            xlValignToVerticalAlignment.Add(XlVAlign.xlVAlignDistributed, SpreadVerticalAlignment.Distributed);
            xlValignToVerticalAlignment.Add(XlVAlign.xlVAlignJustify, SpreadVerticalAlignment.Justify);
            xlValignToVerticalAlignment.Add(XlVAlign.xlVAlignTop, SpreadVerticalAlignment.Top);

            xlThemeColorToThemeColorType = new Dictionary<XlThemeColor, SpreadThemeColorType>();
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent1, SpreadThemeColorType.Accent1);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent2, SpreadThemeColorType.Accent2);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent3, SpreadThemeColorType.Accent3);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent4, SpreadThemeColorType.Accent4);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent5, SpreadThemeColorType.Accent5);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorAccent6, SpreadThemeColorType.Accent6);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorDark1, SpreadThemeColorType.Light1);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorDark2, SpreadThemeColorType.Light2);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorFollowedHyperlink, SpreadThemeColorType.FollowedHyperlink);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorHyperlink, SpreadThemeColorType.Hyperlink);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorLight1, SpreadThemeColorType.Dark1);
            xlThemeColorToThemeColorType.Add(XlThemeColor.xlThemeColorLight2, SpreadThemeColorType.Dark2);
        }

        public static Microsoft.Office.Interop.Excel.Workbook Open(this Microsoft.Office.Interop.Excel.Application app, Stream stream, string fileName)
        {
            stream.Seek(0, SeekOrigin.Begin);

            if (!Directory.Exists(TestsConfig.TempFolder))
            {
                Directory.CreateDirectory(TestsConfig.TempFolder);
            }

            using (FileStream fileStream = File.OpenWrite(fileName))
            {
                stream.CopyTo(fileStream);
            }

            return app.Workbooks.Open(fileName);
        }

        public static string GetTestFileName(string extenssion)
        {
            return Path.Combine(TestsConfig.TempFolder, string.Format("{0}.{1}", TestsConfig.FileName, extenssion));
        }

        public static SpreadThemableColor ExcelColorToThemableColor(double color)
        {
            System.Drawing.Color col = System.Drawing.ColorTranslator.FromOle((int)color);
            SpreadColor rgbColor = new SpreadColor(col.R, col.G, col.B);

            return new SpreadThemableColor(rgbColor);
        }

        public static SpreadThemableColor ExcelColorToThemableColor(XlThemeColor color, double titntAndShade)
        {
            if (xlThemeColorToThemeColorType.ContainsKey(color))
            {
                var colorType = xlThemeColorToThemeColorType[color];
                if (titntAndShade == 0)
                {
                    return new SpreadThemableColor(colorType);
                }
                else
                {
                    foreach (SpreadColorShadeType shadeType in Enum.GetValues(typeof(SpreadColorShadeType)))
                    {
                        double ts = SpreadPredefinedThemeSchemes.DefaultTheme.ColorScheme.GetTintAndShade(colorType, shadeType);
                        if (Math.Round(ts, 2).EqualsDouble(Math.Round(titntAndShade, 2)))
                        {
                            titntAndShade = ts;
                            break;
                        }
                    }

                    return new SpreadThemableColor(colorType, titntAndShade);
                }
            }

            return null;
        }

        internal static CellRangeWrapper ToCellRange(this Microsoft.Office.Interop.Excel.Range range)
        {
            int fromColumnIndex = range.Column - 1;
            int toColumnIndex = fromColumnIndex + range.Columns.Count - 1;
            int fromRowIndex = range.Row - 1;
            int toRowIndex = fromRowIndex + range.Rows.Count - 1;

            return new CellRangeWrapper(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex);
        }

        public static SpreadPatternFill ToPatternFill(this Interior interior)
        {
            XlPattern pattern = (XlPattern)interior.Pattern;
            SpreadThemableColor themableColor = null;
            SpreadThemableColor themablePatternColor = null;

            try
            {
                XlThemeColor color = (XlThemeColor)interior.ThemeColor;
                double titntAndShade = interior.TintAndShade;
                themableColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(color, titntAndShade);
            }
            catch
            {
            }

            if (themableColor == null)
            {
                var color = interior.Color;
                themableColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(color);
            }

            try
            {
                XlThemeColor color = (XlThemeColor)interior.PatternThemeColor;
                double titntAndShade = interior.PatternTintAndShade;
                themablePatternColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(color, titntAndShade);
            }
            catch
            {
            }

            if (themablePatternColor == null)
            {
                var patternColor = interior.PatternColor;
                themablePatternColor = ExcelInteropTestsHelper.ExcelColorToThemableColor(patternColor);
            }

            if (pattern != XlPattern.xlPatternNone)
            {
                var patternType = xlPatternToPatternType[pattern];
                return new SpreadPatternFill(patternType, themablePatternColor, themableColor);
            }

            return null;
        }

        public static SpreadUnderlineType ToUnderlineType(this XlUnderlineStyle underline)
        {
            return xlUndelineStyleToUnderline[underline];
        }

        public static SpreadHorizontalAlignment ToHorizontalAlignment(this XlHAlign horizontalAlignment)
        {
            return xlHAlignToHorizontalAlignment[horizontalAlignment];
        }

        public static SpreadVerticalAlignment ToVerticalAlignment(this XlVAlign verticalAlignment)
        {
            return xlValignToVerticalAlignment[verticalAlignment];
        }

        public static SpreadThemableFontFamily ToThemeFontFamily(this Font font)
        {
            XlThemeFont fontType = font.ThemeFont;
            switch (fontType)
            {
                case XlThemeFont.xlThemeFontNone:
                    return new SpreadThemableFontFamily(font.Name);
                case XlThemeFont.xlThemeFontMajor:
                    return new SpreadThemableFontFamily(SpreadThemeFontType.Major);
                case XlThemeFont.xlThemeFontMinor:
                    return new SpreadThemableFontFamily(SpreadThemeFontType.Minor);
            }

            throw new InvalidOperationException();
        }
    }
}
#endif