﻿using System;
using System.Collections.Generic;
using System.Windows.Media;
using Telerik.Documents.SpreadsheetStreaming.Tests.CommonWrappers;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Utils
{
#if SPREAD
    public static class SpreadProcessingTestsHelper
    {
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.PatternType, SpreadPatternType> spreadPatternTypeToPatternType;
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment, SpreadHorizontalAlignment> spreadHorizontalAlignmentToHorizontalAlignment;
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType, SpreadUnderlineType> shreadUnderlineTypeToUnderline;
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment, SpreadVerticalAlignment> spreadVerticalAlignmentToVerticalAlignment;
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType, SpreadThemeColorType> spreadColorTypeToColorType;
        private static readonly Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType, SpreadColorShadeType> spreadColorShadeTypeToColorShadeType;

        static SpreadProcessingTestsHelper()
        {
            spreadPatternTypeToPatternType = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.PatternType, SpreadPatternType>();
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Solid, SpreadPatternType.Solid);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Gray75Percent, SpreadPatternType.Gray75Percent);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Gray50Percent, SpreadPatternType.Gray50Percent);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Gray25Percent, SpreadPatternType.Gray25Percent);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Gray12Percent, SpreadPatternType.Gray12Percent);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Gray6Percent, SpreadPatternType.Gray6Percent);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.HorizontalStripe, SpreadPatternType.HorizontalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.VerticalStripe, SpreadPatternType.VerticalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ReverseDiagonalStripe, SpreadPatternType.ReverseDiagonalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.DiagonalStripe, SpreadPatternType.DiagonalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.DiagonalCrosshatch, SpreadPatternType.DiagonalCrosshatch);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThickDiagonalCrosshatch, SpreadPatternType.ThickDiagonalCrosshatch);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinHorizontalStripe, SpreadPatternType.ThinHorizontalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinVerticalStripe, SpreadPatternType.ThinVerticalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinReverseDiagonalStripe, SpreadPatternType.ThinReverseDiagonalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinDiagonalStripe, SpreadPatternType.ThinDiagonalStripe);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinHorizontalCrossHatch, SpreadPatternType.ThinHorizontalCrossHatch);
            spreadPatternTypeToPatternType.Add(Telerik.Windows.Documents.Spreadsheet.Model.PatternType.ThinDiagonalCrosshatch, SpreadPatternType.ThinDiagonalCrosshatch);

            spreadHorizontalAlignmentToHorizontalAlignment = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment, SpreadHorizontalAlignment>();
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Center, SpreadHorizontalAlignment.Center);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.CenterContinuous, SpreadHorizontalAlignment.CenterContinuous);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Distributed, SpreadHorizontalAlignment.Distributed);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Fill, SpreadHorizontalAlignment.Fill);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.General, SpreadHorizontalAlignment.General);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Justify, SpreadHorizontalAlignment.Justify);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Left, SpreadHorizontalAlignment.Left);
            spreadHorizontalAlignmentToHorizontalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment.Right, SpreadHorizontalAlignment.Right);

            shreadUnderlineTypeToUnderline = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType, SpreadUnderlineType>();
            shreadUnderlineTypeToUnderline.Add(Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.Double, SpreadUnderlineType.Double);
            shreadUnderlineTypeToUnderline.Add(Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.DoubleAccounting, SpreadUnderlineType.DoubleAccounting);
            shreadUnderlineTypeToUnderline.Add(Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.None, SpreadUnderlineType.None);
            shreadUnderlineTypeToUnderline.Add(Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.Single, SpreadUnderlineType.Single);
            shreadUnderlineTypeToUnderline.Add(Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType.SingleAccounting, SpreadUnderlineType.SingleAccounting);

            spreadVerticalAlignmentToVerticalAlignment = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment, SpreadVerticalAlignment>();
            spreadVerticalAlignmentToVerticalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment.Bottom, SpreadVerticalAlignment.Bottom);
            spreadVerticalAlignmentToVerticalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment.Center, SpreadVerticalAlignment.Center);
            spreadVerticalAlignmentToVerticalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment.Distributed, SpreadVerticalAlignment.Distributed);
            spreadVerticalAlignmentToVerticalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment.Justify, SpreadVerticalAlignment.Justify);
            spreadVerticalAlignmentToVerticalAlignment.Add(Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment.Top, SpreadVerticalAlignment.Top);

            spreadColorTypeToColorType = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType, SpreadThemeColorType>();
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent1, SpreadThemeColorType.Accent1);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent2, SpreadThemeColorType.Accent2);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent3, SpreadThemeColorType.Accent3);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent4, SpreadThemeColorType.Accent4);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent5, SpreadThemeColorType.Accent5);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Accent6, SpreadThemeColorType.Accent6);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Background1, SpreadThemeColorType.Light1);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Background2, SpreadThemeColorType.Light2);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.FollowedHyperlink, SpreadThemeColorType.FollowedHyperlink);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Hyperlink, SpreadThemeColorType.Hyperlink);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Text1, SpreadThemeColorType.Dark1);
            spreadColorTypeToColorType.Add(Telerik.Windows.Documents.Spreadsheet.Theming.ThemeColorType.Text2, SpreadThemeColorType.Dark2);

            spreadColorShadeTypeToColorShadeType = new Dictionary<Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType, SpreadColorShadeType>();
            spreadColorShadeTypeToColorShadeType.Add(Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType.Shade1, SpreadColorShadeType.Shade1);
            spreadColorShadeTypeToColorShadeType.Add(Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType.Shade2, SpreadColorShadeType.Shade2);
            spreadColorShadeTypeToColorShadeType.Add(Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType.Shade3, SpreadColorShadeType.Shade3);
            spreadColorShadeTypeToColorShadeType.Add(Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType.Shade4, SpreadColorShadeType.Shade4);
            spreadColorShadeTypeToColorShadeType.Add(Telerik.Windows.Documents.Spreadsheet.Model.ColorShadeType.Shade5, SpreadColorShadeType.Shade5);
        }

        internal static CellRangeWrapper ToCellRange(this Telerik.Windows.Documents.Spreadsheet.Model.CellRange range)
        {
            int fromColumnIndex = range.FromIndex.ColumnIndex;
            int toColumnIndex = range.ToIndex.ColumnIndex;
            int fromRowIndex = range.FromIndex.RowIndex;
            int toRowIndex = range.ToIndex.RowIndex;

            return new CellRangeWrapper(fromRowIndex, fromColumnIndex, toRowIndex, toColumnIndex);
        }

        public static SpreadThemableColor SpreadColorToThemableColor(this Telerik.Windows.Documents.Spreadsheet.Model.ThemableColor color)
        {
            if (color.IsFromTheme)
            {
                SpreadThemeColorType colorType = spreadColorTypeToColorType[color.ThemeColorType];

                if (color.ColorShadeType.HasValue)
                {
                    var shadeType = spreadColorShadeTypeToColorShadeType[color.ColorShadeType.Value];

                    return new SpreadThemableColor(colorType, shadeType);
                }

                if (color.TintAndShade.HasValue)
                {
                    return new SpreadThemableColor(colorType, color.TintAndShade.Value);
                }

                return new SpreadThemableColor(colorType);
            }
            else
            {
                Color patternColor = color.LocalValue;
                var patternRadColor = new SpreadColor(patternColor.A, patternColor.R, patternColor.G, patternColor.B);
                return new SpreadThemableColor(patternRadColor);
            }
        }

        public static SpreadPatternFill SpreadFillToPatternFill(this Telerik.Windows.Documents.Spreadsheet.Model.IFill fill)
        {
            var patternFill = fill as Telerik.Windows.Documents.Spreadsheet.Model.PatternFill;
            if (patternFill == null || patternFill.Equals(Telerik.Windows.Documents.Spreadsheet.Model.PatternFill.Default))
            {
                return null;
            }

            var patternThemableColor = patternFill.PatternColor.SpreadColorToThemableColor();
            var backgroundThemableColor = patternFill.BackgroundColor.SpreadColorToThemableColor();

            if (patternFill.PatternType == Telerik.Windows.Documents.Spreadsheet.Model.PatternType.Solid)
            {
                patternThemableColor = patternFill.BackgroundColor.SpreadColorToThemableColor();
                backgroundThemableColor = patternFill.PatternColor.SpreadColorToThemableColor();
            }

            return new SpreadPatternFill(
                spreadPatternTypeToPatternType[patternFill.PatternType],
                patternThemableColor,
                backgroundThemableColor);
        }

        public static SpreadThemableFontFamily SpreadFontFamilyToThemableFontFamily(this Telerik.Windows.Documents.Spreadsheet.Model.ThemableFontFamily fontFamily, Telerik.Windows.Documents.Spreadsheet.Theming.DocumentTheme theme)
        {
            if (fontFamily.IsFromTheme)
            {
                switch (fontFamily.ThemeFontType)
                {
                    case Telerik.Windows.Documents.Spreadsheet.Theming.ThemeFontType.Minor:
                        return new SpreadThemableFontFamily(SpreadThemeFontType.Minor);
                    case Telerik.Windows.Documents.Spreadsheet.Theming.ThemeFontType.Major:
                        return new SpreadThemableFontFamily(SpreadThemeFontType.Major);
                }

                throw new InvalidOperationException();
            }
            else
            {
                string fontFamilyName = fontFamily.LocalValue.Source;

                return new SpreadThemableFontFamily(fontFamilyName);
            }
        }

        public static SpreadUnderlineType SpreadUnderlineTypeToUnderlineType(this Telerik.Windows.Documents.Spreadsheet.Model.UnderlineType underline)
        {
            return shreadUnderlineTypeToUnderline[underline];
        }

        public static SpreadHorizontalAlignment SpreadHorizontalAlignmentToHorizontalAlignment(this Telerik.Windows.Documents.Spreadsheet.Model.RadHorizontalAlignment alignment)
        {
            return spreadHorizontalAlignmentToHorizontalAlignment[alignment];
        }

        public static SpreadVerticalAlignment SpreadVerticalAlignmentToVerticalAlignment(this Telerik.Windows.Documents.Spreadsheet.Model.RadVerticalAlignment alignment)
        {
            return spreadVerticalAlignmentToVerticalAlignment[alignment];
        }
    }
#endif
}