﻿using System;
using Telerik.Documents.SpreadsheetStreaming.Tests.Wrappers;
using Telerik.Documents.SpreadsheetStreaming;
using Telerik.Documents.SpreadsheetStreaming.Model.Themes;
using Telerik.Documents.SpreadsheetStreaming.Utilities;

namespace Telerik.Documents.SpreadsheetStreaming.Tests.Utils
{
    public static class TestsHelper
    {
        public static readonly SpreadThemableColor DefaultBlack;
        public static readonly SpreadThemableColor DefaultWhite;
        public static readonly SpreadThemableFontFamily DefaultFontFamily;
        public static readonly double DefaultFontSize;
        public static readonly SpreadHorizontalAlignment DefaultHorizontalAlignment;
        public static readonly int DefaultIndent;
        public static readonly bool DefaultIsBold;
        public static readonly bool DefaultIsItalic;
        public static readonly SpreadUnderlineType DefaultUnderline;
        public static readonly SpreadVerticalAlignment DefaultVerticalAlignment;
        public static readonly bool DefaultWrapText;
        public static readonly SpreadBorderStyle DefaultBorderLineStyle;

        static TestsHelper()
        {
            DefaultBlack = new SpreadThemableColor(new SpreadColor(0, 0, 0));
#if SPREAD
            DefaultWhite = new SpreadThemableColor(new SpreadColor(0, 255, 255, 255));
#else
            DefaultWhite = new SpreadThemableColor(new SpreadColor(255, 255, 255));
#endif
            DefaultFontFamily = new SpreadThemableFontFamily(SpreadThemeFontType.Minor);
            DefaultFontSize = 11;
            DefaultHorizontalAlignment = SpreadHorizontalAlignment.General;
            DefaultIndent = 0;
            DefaultIsBold = false;
            DefaultIsItalic = false;
            DefaultUnderline = SpreadUnderlineType.None;
            DefaultVerticalAlignment = SpreadVerticalAlignment.Bottom;
            DefaultWrapText = false;
            DefaultBorderLineStyle = SpreadBorderStyle.None;
        }

        public static bool IsCellFormatEqual(this CellWrapper cellWrapper, SpreadCellFormat format)
        {
            if (!BordersEquals(cellWrapper.Borders, format))
            {
                return false;
            }

            if (!FillEquals(cellWrapper.PatternFill, format.Fill))
            {
                return false;
            }

            if (!NumberFormatEquals(cellWrapper.NumberFormat, format.NumberFormat))
            {
                return false;
            }

            if (!TextPropertiesEquals(cellWrapper.TextProperties, format))
            {
                return false;
            }

            return true;
        }

        private static bool BordersEquals(BordersWrapper cellBordersWrapper, SpreadCellFormat borders)
        {
            if (borders == null)
            {
                return cellBordersWrapper.HasDefaultBorders;
            }

            if (!BorderEquals(cellBordersWrapper.Left, borders.LeftBorder))
            {
                return false;
            }

            if (!BorderEquals(cellBordersWrapper.Top, borders.TopBorder))
            {
                return false;
            }

            if (!BorderEquals(cellBordersWrapper.Right, borders.RightBorder))
            {
                return false;
            }

            if (!BorderEquals(cellBordersWrapper.Bottom, borders.BottomBorder))
            {
                return false;
            }

            if (!BorderEquals(cellBordersWrapper.DiagonalDown, borders.DiagonalDownBorder))
            {
                return false;
            }

            if (!BorderEquals(cellBordersWrapper.DiagonalUp, borders.DiagonalUpBorder))
            {
                return false;
            }

            return true;
        }

        private static bool BorderEquals(BorderWrapper cellBorderWrapper, SpreadBorder border)
        {
            if (border == null)
            {
                return cellBorderWrapper.HasDefaultLineStyle;
            }

            if (!cellBorderWrapper.Style.Equals(border.Style))
            {
                return false;
            }

            if (cellBorderWrapper.Style == SpreadBorderStyle.None &&
                border.Style == SpreadBorderStyle.None)
            {
                return true;
            }

            if (cellBorderWrapper.Style != SpreadBorderStyle.None)
            {
                if (!ColorEquals(cellBorderWrapper.Color, border.Color))
                {
                    return false;
                }
            }

            return true;
        }

        private static bool FillEquals(SpreadPatternFill cellFillFromWrapper, ISpreadFill fill)
        {
            SpreadPatternFill patternFill = fill as SpreadPatternFill;

            if (cellFillFromWrapper == null && patternFill == null)
            {
                return true;
            }

            if (cellFillFromWrapper == null)
            {
                return false;
            }

            return PatternFillEquals(cellFillFromWrapper, patternFill);
        }

        private static bool PatternFillEquals(SpreadPatternFill cellFillFromWrapper, SpreadPatternFill patternFill)
        {
            if (cellFillFromWrapper.PatternType != patternFill.PatternType)
            {
                return false;
            }

            if (patternFill.BackgroundColor != null)
            {
                if (!ColorEquals(cellFillFromWrapper.BackgroundColor, patternFill.BackgroundColor))
                {
                    return false;
                }
            }
            else
            {
                if (!ColorEquals(cellFillFromWrapper.BackgroundColor, DefaultWhite))
                {
                    return false;
                }
            }

            if (patternFill.PatternColor != null)
            {
                if (!ColorEquals(cellFillFromWrapper.PatternColor, patternFill.PatternColor))
                {
                    return false;
                }
            }
            else
            {
                if (!ColorEquals(cellFillFromWrapper.PatternColor, DefaultBlack))
                {
                    return false;
                }
            }

            return true;
        }

        private static bool NumberFormatEquals(string numberFormatFromWrapper, string numberFormat)
        {
            if (numberFormat == null)
            {
                return numberFormatFromWrapper == "General" || string.IsNullOrEmpty(numberFormatFromWrapper);
            }

            if (!ObjectExtensions.EqualsOfT(numberFormatFromWrapper, numberFormat))
            {
                return false;
            }

            return true;
        }

        private static bool TextPropertiesEquals(TextPropertiesWrapper textPropertiesWrapper, SpreadCellFormat format)
        {
            if (format == null && textPropertiesWrapper.HasDefaultValues)
            {
                return true;
            }

            if ((format.FontFamily == null && !textPropertiesWrapper.HasDefaultFontFamily) &&
                !FontsEquals(textPropertiesWrapper.FontFamily, format.FontFamily))
            {
                return false;
            }

            if ((format.FontSize == null && !textPropertiesWrapper.HasDefaultFontSize) &&
                !textPropertiesWrapper.FontSize.Equals(format.FontSize))
            {
                return false;
            }

            if ((format.ForeColor == null && !textPropertiesWrapper.HasDefaultForeColor) &&
                !ColorEquals(textPropertiesWrapper.ForeColor, format.ForeColor))
            {
                return false;
            }

            if ((format.HorizontalAlignment == null && !textPropertiesWrapper.HasDefaultHorizontalAlignment) &&
                !textPropertiesWrapper.HorizontalAlignment.Equals(format.HorizontalAlignment))
            {
                return false;
            }

            if ((format.Indent == null && !textPropertiesWrapper.HasDefaultIndent) &&
                !textPropertiesWrapper.Indent.Equals(format.Indent))
            {
                return false;
            }

            if ((format.IsBold == null && !textPropertiesWrapper.HasDefaultIsBold) &&
                !textPropertiesWrapper.IsBold.Equals(format.IsBold))
            {
                return false;
            }

            if ((format.IsItalic == null && !textPropertiesWrapper.HasDefaultIsItalic) &&
                !textPropertiesWrapper.IsItalic.Equals(format.IsItalic))
            {
                return false;
            }

            if ((format.Underline == null && !textPropertiesWrapper.HasDefaultUnderline) &&
                !textPropertiesWrapper.Underline.Equals(format.Underline))
            {
                return false;
            }

            if ((format.VerticalAlignment == null && !textPropertiesWrapper.HasDefaultVerticalAlignment) &&
                !textPropertiesWrapper.VerticalAlignment.Equals(format.VerticalAlignment))
            {
                return false;
            }

            if ((format.WrapText == null && !textPropertiesWrapper.HasDefaultWrapText) &&
                !textPropertiesWrapper.WrapText.Equals(format.WrapText))
            {
                return false;
            }

            return true;
        }
 
        private static bool FontsEquals(SpreadThemableFontFamily fontFamily, SpreadThemableFontFamily fontFamily1)
        {
            if (fontFamily1 == null)
            {
                var font = fontFamily.GetActualValue(SpreadPredefinedThemeSchemes.DefaultTheme);
                var defaultFont = DefaultFontFamily.GetActualValue(SpreadPredefinedThemeSchemes.DefaultTheme);

                return font.Equals(defaultFont);
            }

            return ObjectExtensions.EqualsOfT(fontFamily, fontFamily1);
        }

        public static bool IsCellStyleEqual(this CellStyleWrapper cellStyleWrapper, SpreadCellStyle cellStyle)
        {
            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyAlignment, cellStyle.ApplyAlignmentInternal))
            {
                return false;
            }

            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyFont, cellStyle.ApplyFontInternal))
            {
                return false;
            }

            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyNumberFormat, cellStyle.ApplyNumberFormatInternal))
            {
                return false;
            }

#if SPREAD
            if (!BorderEquals(cellStyleWrapper.Bottom, cellStyle.BottomBorder))
            {
                return false;
            }

            if (!BorderEquals(cellStyleWrapper.DiagonalDown, cellStyle.DiagonalDownBorder))
            {
                return false;
            }

            if (!BorderEquals(cellStyleWrapper.DiagonalUp, cellStyle.DiagonalUpBorder))
            {
                return false;
            }

            if (!BorderEquals(cellStyleWrapper.LeftBorder, cellStyle.LeftBorder))
            {
                return false;
            }

            if (!BorderEquals(cellStyleWrapper.RightBorder, cellStyle.RightBorder))
            {
                return false;
            }

            if (!BorderEquals(cellStyleWrapper.TopBorder, cellStyle.TopBorder))
            {
                return false;
            }
#endif

            if (!FillEquals(cellStyleWrapper.Fill, cellStyle.Fill))
            {
                return false;
            }

            if (!FontsEquals(cellStyleWrapper.FontFamily, cellStyle.FontFamily))
            {
                return false;
            }

            if (!FontSizeEquals(cellStyleWrapper.FontSize, cellStyle.FontSize))
            {
                return false;
            }

            if (!ColorEquals(cellStyleWrapper.ForeColor, cellStyle.ForeColor))
            {
                return false;
            }

            if (!HorizontalAlignmentEquals(cellStyleWrapper.HorizontalAlignment, cellStyle.HorizontalAlignment))
            {
                return false;
            }

            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyBorder, cellStyle.ApplyBorderInternal))
            {
                return false;
            }

            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyFill, cellStyle.ApplyFillInternal))
            {
                return false;
            }

            if (!CellStyleApplyPropertyEquals(cellStyleWrapper.ApplyProtection, cellStyle.ApplyProtectionInternal))
            {
                return false;
            }

            if (!IndentEquals(cellStyleWrapper.Indent, cellStyle.Indent))
            {
                return false;
            }

            if (!IsBoldEquals(cellStyleWrapper.IsBold, cellStyle.IsBold))
            {
                return false;
            }

            if (!IsItalicEquals(cellStyleWrapper.IsItalic, cellStyle.IsItalic))
            {
                return false;
            }

            if (!ObjectExtensions.EqualsOfT(cellStyleWrapper.Name, cellStyle.Name))
            {
                return false;
            }

            if (!NumberFormatEquals(cellStyleWrapper.NumberFormat, cellStyle.NumberFormat))
            {
                return false;
            }

            if (!UnderlineEquals(cellStyleWrapper.Underline, cellStyle.Underline))
            {
                return false;
            }

            if (!VerticalAlignmentEquals(cellStyleWrapper.VerticalAlignment, cellStyle.VerticalAlignment))
            {
                return false;
            }

            if (!WrapTextEquals(cellStyleWrapper.WrapText, cellStyle.WrapText))
            {
                return false;
            }

            return true;
        }
 
        private static bool WrapTextEquals(bool wrapText, bool? wrapText1)
        {
            if (!wrapText1.HasValue)
            {
                return wrapText.Equals(DefaultWrapText);
            }

            return wrapText.Equals(wrapText1.Value);
        }
 
        private static bool VerticalAlignmentEquals(SpreadVerticalAlignment verticalAlignment, SpreadVerticalAlignment? verticalAlignment1)
        {
            if (!verticalAlignment1.HasValue)
            {
                return verticalAlignment.Equals(DefaultVerticalAlignment);
            }

            return verticalAlignment.Equals(verticalAlignment1.Value);
        }

        private static bool UnderlineEquals(SpreadUnderlineType underline, SpreadUnderlineType? underline1)
        {
            if (!underline1.HasValue)
            {
                return underline.Equals(DefaultUnderline);
            }

            return underline.Equals(underline1.Value);
        }
 
        private static bool IsItalicEquals(bool isItalic, bool? isItalic1)
        {
            if (!isItalic1.HasValue)
            {
                return isItalic.Equals(DefaultIsItalic);
            }

            return isItalic.Equals(isItalic1.Value);
        }
 
        private static bool IsBoldEquals(bool isBold, bool? isBold1)
        {
            if (!isBold1.HasValue)
            {
                return isBold.Equals(DefaultIsBold);
            }

            return isBold.Equals(isBold1.Value);
        }
 
        private static bool IndentEquals(int indent, int? indent1)
        {
            if (!indent1.HasValue)
            {
                return indent.Equals(DefaultIndent);
            }

            return indent.Equals(indent1.Value);
        }
 
        private static bool HorizontalAlignmentEquals(SpreadHorizontalAlignment horizontalAlignment, SpreadHorizontalAlignment? horizontalAlignment1)
        {
            if (!horizontalAlignment1.HasValue)
            {
                return horizontalAlignment.Equals(DefaultHorizontalAlignment);
            }

            return horizontalAlignment.Equals(horizontalAlignment1.Value);
        }
 
        private static bool FontSizeEquals(double fontSize, double? fontSize1)
        {
            if (!fontSize1.HasValue)
            {
                return fontSize == DefaultFontSize;
            }

            return fontSize.Equals(fontSize1.Value);
        }
 
        private static bool CellStyleApplyPropertyEquals(bool applyAlignment, bool? applyAlignment1)
        {
            if (!applyAlignment1.HasValue)
            {
                return applyAlignment;
            }

            return applyAlignment.Equals(applyAlignment1.Value);
        }
 
        private static bool ColorEquals(SpreadThemableColor foreColor, SpreadThemableColor foreColor1)
        {
            var color = foreColor.GetActualValue(SpreadPredefinedThemeSchemes.DefaultTheme);

            if (foreColor1 == null)
            {
                var defaultColor = DefaultBlack.GetActualValue(SpreadPredefinedThemeSchemes.DefaultTheme);
                return color.Equals(defaultColor);
            }
            
            var color1 = foreColor1.GetActualValue(SpreadPredefinedThemeSchemes.DefaultTheme);

            return ObjectExtensions.EqualsOfT(color, color1);
        }
    }
}