﻿using System;
using System.ComponentModel;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Base class for the compression settings.
    /// </summary>
    public class CompressionSettings : INotifyPropertyChanged  
    {
        private CompressionMethod compressionMethod = CompressionMethod.Stored;
        
        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets compression method.
        /// </summary>
        public CompressionMethod Method
        {
            get
            {
                return this.compressionMethod;
            }

            protected set
            {
                this.compressionMethod = value;
                this.OnPropertyChanged("Method");
            }
        }

        /// <summary>
        /// Copy settings from the given base settings.
        /// </summary>
        /// <param name="baseSettings">Base settings to copy from.</param>
        internal virtual void CopyFrom(CompressionSettings baseSettings)
        {
        }

        /// <summary>
        /// Prepare settings for usage in zip archive entries.
        /// </summary>
        /// <param name="header">Central directory header.</param>
        internal virtual void PrepareForZip(CentralDirectoryHeader header = null)
        {
        }

        /// <summary>
        /// Called when property value is changed.
        /// </summary>
        /// <param name="propertyName">Property name.</param>
        protected void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
