﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// The compression level to be used for compression of data.
    /// </summary>
    public enum CompressionLevel
    {
        /// <summary>
        /// The data will be simply stored,
        /// no compression should be performed.
        /// </summary>
        NoCompression = 0,

        /// <summary>
        /// Same as NoCompression.
        /// </summary>
        Level0 = 0,

        /// <summary>
        /// The fastest but least effective compression.
        /// </summary>
        Fastest = 1,

        /// <summary>
        /// A synonym for Fastest.
        /// </summary>
        Level1 = 1,

        /// <summary>
        /// A little slower, but better, than level 1.
        /// </summary>
        Level2 = 2,

        /// <summary>
        /// A little slower, but better, than level 2.
        /// </summary>
        Level3 = 3,

        /// <summary>
        /// A little slower, but better, than level 3.
        /// </summary>
        Level4 = 4,

        /// <summary>
        /// A little slower than level 4, but with better compression.
        /// </summary>
        Level5 = 5,

        /// <summary>
        /// The default compression level with
        /// a good balance of speed and compression efficiency.
        /// </summary>
        Optimal = 6,

        /// <summary>
        /// A synonym for Optimal.
        /// </summary>
        Level6 = 6,

        /// <summary>
        /// Pretty good compression.
        /// </summary>
        Level7 = 7,

        /// <summary>
        ///  Better compression than Level7.
        /// </summary>
        Level8 = 8,

        /// <summary>
        /// The best compression, where best means
        /// greatest reduction in size of the input data.
        /// This is also the slowest compression.
        /// </summary>
        Best = 9,

        /// <summary>
        /// A synonym for Best compression level.
        /// </summary>
        Level9 = 9
    }
}
