﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Class which implements Deflate compression algorithm.
    /// </summary>
    internal class DeflateAlgorithm : ICompressionAlgorithm
    {
        private DeflateSettings deflateSettings;

        /// <summary>
        /// Creates a compressor object. 
        /// </summary>
        /// <returns>A compressor object.</returns>
        public IBlockTransform CreateCompressor()
        {
            return new DeflateCompressor(this.deflateSettings);
        }

        /// <summary>
        /// Creates a decompressor object.
        /// </summary>
        /// <returns>A decompressor object.</returns>
        public IBlockTransform CreateDecompressor()
        {
            return new DeflateDecompressor(this.deflateSettings);
        }

        /// <summary>
        /// Initialize compression algorithm using given compression settings.
        /// </summary>
        /// <param name="settings">Compression settings.</param>
        public void Initialize(CompressionSettings settings)
        {
            this.deflateSettings = settings as DeflateSettings;
            if (this.deflateSettings == null)
            {
                throw new ArgumentException("Wrong settings type.");
            }
        }
    }
}
