﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents configuration of deflate algorithm.
    /// </summary>
    internal class DeflateConfiguration
    {
        private static readonly DeflateConfiguration[] table;

        static DeflateConfiguration()
        {
            table = new DeflateConfiguration[]
                {
                    new DeflateConfiguration(0, 0, 0, 0),

                    new DeflateConfiguration(4, 4, 8, 4),
                    new DeflateConfiguration(4, 5, 16, 8),
                    new DeflateConfiguration(4, 6, 32, 32),

                    new DeflateConfiguration(4, 4, 16, 16),
                    new DeflateConfiguration(8, 16, 32, 32),
                    new DeflateConfiguration(8, 16, 128, 128),
                    new DeflateConfiguration(8, 32, 128, 256),
                    new DeflateConfiguration(32, 128, 258, 1024),
                    new DeflateConfiguration(32, 258, 258, 4096),
                };
        }

        private DeflateConfiguration(int goodLength, int maxLazy, int niceLength, int maxChainLength)
        {
            this.GoodLength = goodLength;
            this.MaxLazy = maxLazy;
            this.NiceLength = niceLength;
            this.MaxChainLength = maxChainLength;
        }

        /// <summary>
        /// Use a faster search when the previous match is longer
        /// than this reduce lazy search above this match length.
        /// </summary>
        internal int GoodLength
        {
            get;
            private set;
        }

        /// <summary>
        /// Attempt to find a better match only when the current match is
        /// strictly smaller than this value. This mechanism is used only for
        /// compression levels >= 4.  For levels 1,2,3: MaxLazy is actually
        /// MaxInsertLength (See DeflateFast).
        /// Do not perform lazy search above this match length.
        /// </summary>
        internal int MaxLazy
        {
            get;
            private set;
        }

        /// <summary>
        /// Quit search above this match length.
        /// </summary>
        internal int NiceLength
        {
            get;
            private set;
        }

        /// <summary>
        /// To speed up deflation, hash chains are never searched beyond this length.
        /// A higher limit improves compression ratio but degrades the speed.
        /// </summary>
        internal int MaxChainLength
        {
            get;
            private set;
        }

        /// <summary>
        /// Returns instance of Config class by the compression level.
        /// </summary>
        /// <param name="compressionLevel">Compression level.</param>
        /// <returns>Instance of Config class.</returns>
        public static DeflateConfiguration Lookup(int compressionLevel)
        {
            return table[compressionLevel];
        }
    }
}
