﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Compression settings of the Deflate method.
    /// </summary>
    public class DeflateSettings : CompressionSettings
    {
        private CompressionLevel compressionLevel;
        private CompressedStreamHeader headerType;

        /// <summary>
        /// Initializes a new instance of the DeflateSettings class.
        /// </summary>
        public DeflateSettings()
        {
            this.Method = CompressionMethod.Deflate;
            this.CompressionLevel = CompressionLevel.Optimal;
            this.HeaderType = CompressedStreamHeader.ZLib;
        }

        /// <summary>
        /// The compression level of deflate algorithm to be used for deflating by a CompressedStream.
        /// </summary>
        public CompressionLevel CompressionLevel
        {
            get
            {
                return this.compressionLevel;
            }

            set
            {
                this.compressionLevel = value;
                this.OnPropertyChanged("CompressionLevel");
            }
        }

        /// <summary>
        /// Gets or sets compression stream header type.
        /// </summary>
        public CompressedStreamHeader HeaderType
        {
            get
            {
                return this.headerType;
            }

            set
            {
                this.headerType = value;
                this.OnPropertyChanged("HeaderType");
            }
        }

        /// <summary>
        /// Copy settings from the given base settings.
        /// </summary>
        /// <param name="baseSettings">Base settings to copy from.</param>
        internal override void CopyFrom(CompressionSettings baseSettings)
        {
            DeflateSettings deflateSettings = baseSettings as DeflateSettings;
            if (deflateSettings != null)
            {
                this.HeaderType = deflateSettings.HeaderType;
                this.CompressionLevel = deflateSettings.CompressionLevel;
            }
        }

        /// <summary>
        /// Prepare settings for usage in zip archive entries.
        /// </summary>
        /// <param name="header">Central directory header.</param>
        internal override void PrepareForZip(CentralDirectoryHeader header = null)
        {
            base.PrepareForZip(header);

            this.HeaderType = CompressedStreamHeader.None;
        }
    }
}
