﻿using System;
using System.Collections.Generic;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Interface which must be implemented by all implementations of the compression algorithm.
    /// </summary>
    internal interface ICompressionAlgorithm
    {
        /// <summary>
        /// Creates a compressor object. 
        /// </summary>
        /// <returns>A compressor object.</returns>
        IBlockTransform CreateCompressor();

        /// <summary>
        /// Creates a decompressor object.
        /// </summary>
        /// <returns>A decompressor object.</returns>
        IBlockTransform CreateDecompressor();

        /// <summary>
        /// Initialize compression algorithm using given compression settings.
        /// </summary>
        /// <param name="settings">Compression settings.</param>
        void Initialize(CompressionSettings settings);
    }
}
