#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaBitTreeDecoder
    {
        private LzmaRangeBitDecoder[] models;
        private int bitLevels;

        public LzmaBitTreeDecoder(int levels)
        {
            this.bitLevels = levels;
            this.models = new LzmaRangeBitDecoder[1 << this.bitLevels];
        }

        public static uint ReverseDecode(LzmaRangeBitDecoder[] models,
            uint startIndex,
            LzmaRangeDecoder rangeDecoder,
            int numBitLevels)
        {
            uint mask = 1;
            uint symbol = 0;
            if (rangeDecoder.CheckInputRequired(numBitLevels))
            {
                for (int bitIndex = 0; bitIndex < numBitLevels; bitIndex++)
                {
                    uint bit = models[startIndex + mask].Decode(rangeDecoder);
                    if (rangeDecoder.InputRequired)
                    {
                        break;
                    }

                    mask <<= 1;
                    mask += bit;
                    symbol |= bit << bitIndex;
                }
            }

            return symbol;
        }

        public void Init()
        {
            int modelsCounter = 1 << this.bitLevels;
            for (uint i = 1; i < modelsCounter; i++)
            {
                this.models[i].Init();
            }
        }

        public uint Decode(LzmaRangeDecoder rangeDecoder)
        {
            uint mask = 1;
            if (rangeDecoder.CheckInputRequired(this.bitLevels))
            {
                for (int bitIndex = this.bitLevels; bitIndex > 0; bitIndex--)
                {
                    mask = (mask << 1) + this.models[mask].Decode(rangeDecoder);
                }
            }

            return mask - ((uint)1 << this.bitLevels);
        }

        public uint ReverseDecode(LzmaRangeDecoder rangeDecoder)
        {
            uint symbol = ReverseDecode(this.models, 0, rangeDecoder, this.bitLevels);

            return symbol;
        }
    }
}
