﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal struct LzmaBitTreeEncoder
    {
        private LzmaRangeBitEncoder[] models;
        private int bitLevels;

        public LzmaBitTreeEncoder(int bitLevelsNumber)
        {
            this.bitLevels = bitLevelsNumber;
            this.models = new LzmaRangeBitEncoder[1 << this.bitLevels];
        }

        public static uint ReverseGetPrice(LzmaRangeBitEncoder[] models,
            uint startIndex,
            int bitLevelsNumber,
            uint symbol)
        {
            uint price = 0;
            uint offset = 1;
            for (int i = bitLevelsNumber; i > 0; i--)
            {
                uint bit = symbol & 1;
                symbol >>= 1;
                price += models[startIndex + offset].GetPrice(bit);
                offset = (offset << 1) | bit;
            }

            return price;
        }

        public static void ReverseEncode(LzmaRangeBitEncoder[] models,
            uint startIndex,
            LzmaRangeEncoder rangeEncoder,
            int bitLevelsNumber,
            uint symbol)
        {
            uint m = 1;
            for (int i = 0; i < bitLevelsNumber; i++)
            {
                uint bit = symbol & 1;
                models[startIndex + m].Encode(rangeEncoder, bit);
                m = (m << 1) | bit;
                symbol >>= 1;
            }
        }

        public void Init()
        {
            for (uint i = 1; i < (1 << this.bitLevels); i++)
            {
                this.models[i].Init();
            }
        }

        public void Encode(LzmaRangeEncoder rangeEncoder, uint symbol)
        {
            uint m = 1;
            for (int bitIndex = this.bitLevels; bitIndex > 0;)
            {
                bitIndex--;
                uint bit = (symbol >> bitIndex) & 1;
                this.models[m].Encode(rangeEncoder, bit);
                m = (m << 1) | bit;
            }
        }

        public void ReverseEncode(LzmaRangeEncoder rangeEncoder, uint symbol)
        {
            uint index = 1;
            for (uint i = 0; i < this.bitLevels; i++)
            {
                uint bit = symbol & 1;
                this.models[index].Encode(rangeEncoder, bit);
                index = (index << 1) | bit;
                symbol >>= 1;
            }
        }

        public uint GetPrice(uint symbol)
        {
            uint price = 0;
            uint index = 1;
            for (int bitIndex = this.bitLevels; bitIndex > 0;)
            {
                bitIndex--;
                uint bit = (symbol >> bitIndex) & 1;
                price += this.models[index].GetPrice(bit);
                index = (index << 1) + bit;
            }

            return price;
        }

        public uint ReverseGetPrice(uint symbol)
        {
            uint price = 0;
            uint index = 1;
            for (int i = this.bitLevels; i > 0; i--)
            {
                uint bit = symbol & 1;
                symbol >>= 1;
                price += this.models[index].GetPrice(bit);
                index = (index << 1) | bit;
            }

            return price;
        }
    }
}
