﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaLengthDecoder
    {
        private LzmaRangeBitDecoder choice = new LzmaRangeBitDecoder();
        private LzmaRangeBitDecoder choice2 = new LzmaRangeBitDecoder();
        private LzmaBitTreeDecoder[] lowCoder = new LzmaBitTreeDecoder[LzmaTransformBase.PosStatesMaxNumber];
        private LzmaBitTreeDecoder[] middleCoder = new LzmaBitTreeDecoder[LzmaTransformBase.PosStatesMaxNumber];
        private LzmaBitTreeDecoder highCoder = new LzmaBitTreeDecoder(LzmaTransformBase.HighLengthBitsNumber);
        private uint numberPositionStates = 0;

        public LzmaLengthDecoder(uint positionStates)
        {
            for (uint positionState = this.numberPositionStates; positionState < positionStates; positionState++)
            {
                this.lowCoder[positionState] = new LzmaBitTreeDecoder(LzmaTransformBase.LowLengthBitsNumber);
                this.middleCoder[positionState] = new LzmaBitTreeDecoder(LzmaTransformBase.MiddleLengthBitsNumber);
            }

            this.numberPositionStates = positionStates;
        }

        public void Init()
        {
            this.choice.Init();
            for (uint positionState = 0; positionState < this.numberPositionStates; positionState++)
            {
                this.lowCoder[positionState].Init();
                this.middleCoder[positionState].Init();
            }

            this.choice2.Init();
            this.highCoder.Init();
        }

        public uint Decode(LzmaRangeDecoder rangeDecoder, uint positionState)
        {
            rangeDecoder.SaveState();
            uint decoded = this.choice.Decode(rangeDecoder);
            if (rangeDecoder.InputRequired)
            {
                return 0;
            }

            uint symbol;
            if (decoded == 0)
            {
                symbol = this.lowCoder[positionState].Decode(rangeDecoder);
            }
            else
            {
                symbol = LzmaTransformBase.LowLengthSymbolsNumber;
                decoded = this.choice2.Decode(rangeDecoder);
                if (!rangeDecoder.InputRequired)
                {
                    if (decoded == 0)
                    {
                        symbol += this.middleCoder[positionState].Decode(rangeDecoder);
                    }
                    else
                    {
                        symbol += LzmaTransformBase.MiddleLengthSymbolsNumber;
                        symbol += this.highCoder.Decode(rangeDecoder);
                    }
                }

                if (rangeDecoder.InputRequired)
                {
                    this.choice2.RestoreState();
                }
            }

            if (rangeDecoder.InputRequired)
            {
                this.choice.RestoreState();
                rangeDecoder.RestoreState();
            }
            
            return symbol;
        }
    }
}
