﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaLengthEncoder
    {
        private LzmaRangeBitEncoder choice = new LzmaRangeBitEncoder();
        private LzmaRangeBitEncoder choice2 = new LzmaRangeBitEncoder();
        private LzmaBitTreeEncoder[] lowCoder = new LzmaBitTreeEncoder[LzmaTransformBase.PosStatesEncodingMax];
        private LzmaBitTreeEncoder[] middleCoder = new LzmaBitTreeEncoder[LzmaTransformBase.PosStatesEncodingMax];
        private LzmaBitTreeEncoder highCoder = new LzmaBitTreeEncoder(LzmaTransformBase.HighLengthBitsNumber);

        public LzmaLengthEncoder()
        {
            for (uint posState = 0; posState < LzmaTransformBase.PosStatesEncodingMax; posState++)
            {
                this.lowCoder[posState] = new LzmaBitTreeEncoder(LzmaTransformBase.LowLengthBitsNumber);
                this.middleCoder[posState] = new LzmaBitTreeEncoder(LzmaTransformBase.MiddleLengthBitsNumber);
            }
        }

        public void Init(uint posStates)
        {
            this.choice.Init();
            this.choice2.Init();
            for (uint posState = 0; posState < posStates; posState++)
            {
                this.lowCoder[posState].Init();
                this.middleCoder[posState].Init();
            }

            this.highCoder.Init();
        }

        public void Encode(LzmaRangeEncoder rangeEncoder, uint symbol, uint posState)
        {
            if (symbol < LzmaTransformBase.LowLengthSymbolsNumber)
            {
                this.choice.Encode(rangeEncoder, 0);
                this.lowCoder[posState].Encode(rangeEncoder, symbol);
            }
            else
            {
                symbol -= LzmaTransformBase.LowLengthSymbolsNumber;
                this.choice.Encode(rangeEncoder, 1);
                if (symbol < LzmaTransformBase.MiddleLengthSymbolsNumber)
                {
                    this.choice2.Encode(rangeEncoder, 0);
                    this.middleCoder[posState].Encode(rangeEncoder, symbol);
                }
                else
                {
                    this.choice2.Encode(rangeEncoder, 1);
                    this.highCoder.Encode(rangeEncoder, symbol - LzmaTransformBase.MiddleLengthSymbolsNumber);
                }
            }
        }

        public void SetPrices(uint posState, uint symbolsCounter, uint[] prices, uint st)
        {
            uint a0 = this.choice.GetPrice0();
            uint a1 = this.choice.GetPrice1();
            uint b0 = a1 + this.choice2.GetPrice0();
            uint b1 = a1 + this.choice2.GetPrice1();
            uint i = 0;
            for (i = 0; i < LzmaTransformBase.LowLengthSymbolsNumber; i++)
            {
                if (i >= symbolsCounter)
                {
                    return;
                }

                prices[st + i] = a0 + this.lowCoder[posState].GetPrice(i);
            }

            for (; i < LzmaTransformBase.LowLengthSymbolsNumber + LzmaTransformBase.MiddleLengthSymbolsNumber; i++)
            {
                if (i >= symbolsCounter)
                {
                    return;
                }

                prices[st + i] = b0 + this.middleCoder[posState].GetPrice(i - LzmaTransformBase.LowLengthSymbolsNumber);
            }

            for (; i < symbolsCounter; i++)
            {
                uint price = this.highCoder.GetPrice(
                    i - LzmaTransformBase.LowLengthSymbolsNumber - LzmaTransformBase.MiddleLengthSymbolsNumber);
                prices[st + i] = b1 + price;
            }
        }
    }
}
