﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaLiteralDecoder
    {
        private InternalDecoder[] coders;
        private int numerPreviousBits;
        private int numberPositionBits;
        private uint positionMask;

        public LzmaLiteralDecoder(int positionBits, int previousBits)
        {
            if (this.coders == null
                || this.numerPreviousBits != previousBits
                || this.numberPositionBits != positionBits)
            {
                this.numberPositionBits = positionBits;
                this.positionMask = ((uint)1 << positionBits) - 1;
                this.numerPreviousBits = previousBits;
                uint numStates = (uint)1 << (this.numerPreviousBits + this.numberPositionBits);

                this.coders = new InternalDecoder[numStates];
                for (uint i = 0; i < numStates; i++)
                {
                    this.coders[i].Create();
                }
            }
        }

        public void Init()
        {
            uint numStates = (uint)1 << (this.numerPreviousBits + this.numberPositionBits);
            for (uint i = 0; i < numStates; i++)
            {
                this.coders[i].Init();
            }
        }

        public byte DecodeNormal(LzmaRangeDecoder rangeDecoder, uint position, byte previousByte)
        {
            return this.coders[this.GetState(position, previousByte)].DecodeNormal(rangeDecoder);
        }

        public byte DecodeWithMatchByte(LzmaRangeDecoder rangeDecoder, uint pos, byte previousByte, byte matchByte)
        {
            return this.coders[this.GetState(pos, previousByte)].DecodeWithMatchByte(rangeDecoder, matchByte);
        }

        private uint GetState(uint position, byte previousByte)
        {
            return ((position & this.positionMask) << this.numerPreviousBits) + (uint)(previousByte >> (8 - this.numerPreviousBits));
        }

        private struct InternalDecoder
        {
            private LzmaRangeBitDecoder[] decoders;

            public void Create()
            {
                this.decoders = new LzmaRangeBitDecoder[0x300];
            }

            public void Init()
            {
                for (int i = 0; i < 0x300; i++)
                {
                    this.decoders[i].Init();
                }
            }

            public byte DecodeNormal(LzmaRangeDecoder rangeDecoder)
            {
                uint symbol = 1;
                do
                {
                    symbol = (symbol << 1) | this.decoders[symbol].Decode(rangeDecoder);
                }
                while (symbol < 0x100);

                return (byte)symbol;
            }

            public byte DecodeWithMatchByte(LzmaRangeDecoder rangeDecoder, byte matchByte)
            {
                uint symbol = 1;
                do
                {
                    uint matchBit = (uint)(matchByte >> 7) & 1;
                    matchByte <<= 1;
                    uint bit = this.decoders[((1 + matchBit) << 8) + symbol].Decode(rangeDecoder);
                    symbol = (symbol << 1) | bit;
                    if (matchBit != bit)
                    {
                        while (symbol < 0x100)
                        {
                            symbol = (symbol << 1) | this.decoders[symbol].Decode(rangeDecoder);
                        }

                        break;
                    }
                }
                while (symbol < 0x100);

                return (byte)symbol;
            }
        }
    }
}
