﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaLiteralEncoder
    {
        private Encoder[] encoders;
        private int previousBits;
        private int positionBits;
        private uint positionMask;

        public LzmaLiteralEncoder(int posBits, int previousBits)
        {
            this.positionBits = posBits;
            this.positionMask = ((uint)1 << posBits) - 1;
            this.previousBits = previousBits;
            uint statesCounter = (uint)1 << (this.previousBits + this.positionBits);
            this.encoders = new Encoder[statesCounter];
            for (uint i = 0; i < statesCounter; i++)
            {
                this.encoders[i].Create();
            }

            for (uint i = 0; i < statesCounter; i++)
            {
                this.encoders[i].Init();
            }
        }

        public Encoder GetSubCoder(uint position, byte previousByte)
        {
            uint index = ((position & this.positionMask) << this.previousBits)
                + (uint)(previousByte >> (8 - this.previousBits));

            return this.encoders[index];
        }

        public struct Encoder
        {
            internal LzmaRangeBitEncoder[] Encoders
            {
                get;
                private set;
            }

            public void Create()
            {
                this.Encoders = new LzmaRangeBitEncoder[0x300];
            }

            public void Init()
            {
                for (int i = 0; i < 0x300; i++)
                {
                    this.Encoders[i].Init();
                }
            }

            public void Encode(LzmaRangeEncoder rangeEncoder, byte symbol)
            {
                uint context = 1;
                for (int i = 7; i >= 0; i--)
                {
                    uint bit = (uint)((symbol >> i) & 1);
                    this.Encoders[context].Encode(rangeEncoder, bit);
                    context = (context << 1) | bit;
                }
            }

            public void EncodeMatched(LzmaRangeEncoder rangeEncoder, byte matchByte, byte symbol)
            {
                uint context = 1;
                bool same = true;
                for (int i = 7; i >= 0; i--)
                {
                    uint bit = (uint)((symbol >> i) & 1);
                    uint state = context;
                    if (same)
                    {
                        uint matchBit = (uint)((matchByte >> i) & 1);
                        state += (1 + matchBit) << 8;
                        same = matchBit == bit;
                    }

                    this.Encoders[state].Encode(rangeEncoder, bit);
                    context = (context << 1) | bit;
                }
            }

            public uint GetPrice(bool matchMode, byte matchByte, byte symbol)
            {
                uint price = 0;
                uint context = 1;
                int i = 7;
                if (matchMode)
                {
                    for (; i >= 0; i--)
                    {
                        uint matchBit = (uint)(matchByte >> i) & 1;
                        uint bit = (uint)(symbol >> i) & 1;
                        price += this.Encoders[((1 + matchBit) << 8) + context].GetPrice(bit);
                        context = (context << 1) | bit;
                        if (matchBit != bit)
                        {
                            i--;
                            break;
                        }
                    }
                }

                for (; i >= 0; i--)
                {
                    uint bit = (uint)(symbol >> i) & 1;
                    price += this.Encoders[context].GetPrice(bit);
                    context = (context << 1) | bit;
                }

                return price;
            }
        }
    }
}
