#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal struct LzmaRangeBitDecoder
    {
        private const int ModelTotalBits = 11;
        private const uint ModelTotal = 1 << ModelTotalBits;
        private const int MoveBits = 5;

        private uint bitMask;
        private uint savedBitMask;

        public void Init()
        {
            this.bitMask = ModelTotal >> 1;
        }

        public uint DecodeState(LzmaRangeDecoder rangeDecoder)
        {
            rangeDecoder.SaveState();
            uint value = this.Decode(rangeDecoder);
            if (rangeDecoder.InputRequired)
            {
                rangeDecoder.RestoreState();
            }

            return value;
        }

        public uint Decode(LzmaRangeDecoder rangeDecoder)
        {
            uint value;
            uint newBound = (uint)(rangeDecoder.Range >> ModelTotalBits) * (uint)this.bitMask;
            this.SaveState();

            if (rangeDecoder.Code < newBound)
            {
                this.bitMask += (ModelTotal - this.bitMask) >> MoveBits;
                rangeDecoder.UpdateRange(newBound);

                value = 0;
            }
            else
            {
                this.bitMask -= this.bitMask >> MoveBits;
                rangeDecoder.MoveRange(newBound);

                value = 1;
            }

            if (rangeDecoder.InputRequired)
            {
                this.RestoreState();
            }

            return value;
        }

        public void RestoreState()
        {
            this.bitMask = this.savedBitMask;
        }

        private void SaveState()
        {
            this.savedBitMask = this.bitMask;
        }
    }
}
