﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal struct LzmaRangeBitEncoder
    {
        public const int BitPriceShiftBits = 6;
        
        private const int BitModelTotalBits = 11;
        private const uint BitModelTotal = 1 << BitModelTotalBits;
        private const int MoveBitsNumber = 5;
        private const int MoveReducingBitsNumber = 2;
        private const int BitsNumber = BitModelTotalBits - MoveReducingBitsNumber;

        private static uint[] prices;
        private uint probability;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2207:InitializeValueTypeStaticFieldsInline")]
        static LzmaRangeBitEncoder()
        {
            prices = new uint[BitModelTotal >> MoveReducingBitsNumber];

            for (int i = BitsNumber - 1; i >= 0; i--)
            {
                uint start = (uint)1 << (BitsNumber - i - 1);
                uint end = (uint)1 << (BitsNumber - i);
                for (uint j = start; j < end; j++)
                {
                    prices[j] = ((uint)i << BitPriceShiftBits) +
                        (((end - j) << BitPriceShiftBits) >> (BitsNumber - i - 1));
                }
            }
        }

        public void Init()
        {
            this.probability = BitModelTotal >> 1;
        }

        public void Encode(LzmaRangeEncoder encoder, uint symbol)
        {
            uint newBound = (encoder.Range >> BitModelTotalBits) * this.probability;
            if (symbol == 0)
            {
                encoder.Range = newBound;
                this.probability += (BitModelTotal - this.probability) >> MoveBitsNumber;
            }
            else
            {
                encoder.Low += newBound;
                encoder.Range -= newBound;
                this.probability -= this.probability >> MoveBitsNumber;
            }

            if (encoder.Range < LzmaRangeEncoder.TopValue)
            {
                encoder.Range <<= 8;
                encoder.ShiftLow();
            }
        }

        public uint GetPrice(uint symbol)
        {
            return prices[(((this.probability - symbol) ^ (-(int)symbol)) & (BitModelTotal - 1)) >> MoveReducingBitsNumber];
        }

        public uint GetPrice0()
        {
            return prices[this.probability >> MoveReducingBitsNumber];
        }

        public uint GetPrice1()
        {
            return prices[(BitModelTotal - this.probability) >> MoveReducingBitsNumber];
        }
    }
}
