using System;
using System.Collections.ObjectModel;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaRangeDecoder
    {
        public const uint TopValue = 1 << 24;

        private uint range;
        private uint code;
        private Collection<byte[]> buffers = new Collection<byte[]>();
        private byte[] buffer;
        private int lastBufferSize;
        private int index = 0;
        private bool inputRequired = true;
        private uint savedRange;
        private uint savedCode;
        private int savedIndex;

        public bool FinalBlock
        {
            get;
            set;
        }

        public uint Range
        {
            get
            {
                return this.range;
            }
        }

        public uint Code
        {
            get
            {
                return this.code;
            }
        }

        public bool InputRequired
        {
            get
            {
                return this.inputRequired;
            }
        }

        public void Init(byte[] inputBuffer, int startIndex)
        {
            this.code = 0;
            this.range = 0xFFFFFFFF;
            for (int i = 0; i < 5; i++)
            {
                this.code = (this.code << 8) | inputBuffer[startIndex + i];
            }
        }

        public bool CheckInputRequired(int length, bool ignoreFinalBlock = false)
        {
            if (this.buffers.Count < 2 && (!this.FinalBlock || ignoreFinalBlock))
            {
                this.inputRequired = (this.lastBufferSize - this.index) < length;
            }

            return !this.inputRequired;
        }

        public void SetBuffer(byte[] inputBuffer, int length)
        {
            if (length > 0)
            {
                this.lastBufferSize = length;
                this.buffers.Add(inputBuffer);
                if (this.buffers.Count == 1)
                {
                    this.buffer = this.buffers[0];
                    this.index = 0;
                }

                this.inputRequired = false;
            }
        }

        public uint DecodeDirectBits(int numTotalBits)
        {
            uint result = 0;
            for (int i = numTotalBits; i > 0; i--)
            {
                this.range >>= 1;

                uint t = (this.code - this.range) >> 31;
                this.code -= this.range & (t - 1);
                result = (result << 1) | (1 - t);

                if (this.range < TopValue)
                {
                    this.code = (this.code << 8) | this.GetNextByte();
                    this.range <<= 8;
                }
            }

            return result;
        }

        public void MoveRange(uint newBound)
        {
            this.range -= newBound;
            this.code -= newBound;
            this.CheckRange();
        }

        public void UpdateRange(uint newBound)
        {
            this.range = newBound;
            this.CheckRange();
        }

        public void SaveState()
        {
            this.savedIndex = this.index;
            this.savedRange = this.range;
            this.savedCode = this.code;
        }

        public void RestoreState()
        {
            this.index = this.savedIndex;
            this.range = this.savedRange;
            this.code = this.savedCode;
        }

        private void CheckRange()
        {
            if (this.range < LzmaRangeDecoder.TopValue)
            {
                this.code = (this.code << 8) | this.GetNextByte();
                this.range <<= 8;
            }
        }

        private byte GetNextByte()
        {
            if ((this.buffers.Count == 1 && this.index >= this.lastBufferSize)
            || this.index >= this.buffer.Length)
            {
                if (this.buffers.Count < 2)
                {
                    if (this.inputRequired)
                    {
                        throw new InvalidOperationException("Decoder must check for request next block.");
                    }

                    this.inputRequired = true;
                    return 0;
                }
                else
                {
                    this.buffers.RemoveAt(0);
                }

                this.buffer = this.buffers[0];
                this.index = 0;
            }

            return this.buffer[this.index++];
        }
    }
}
