﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Class which implements Deflate compression algorithm.
    /// </summary>
    internal class LzmaAlgorithm : ICompressionAlgorithm
    {
        private LzmaSettings lzmaSettings;

        /// <summary>
        /// Creates a compressor object. 
        /// </summary>
        /// <returns>A compressor object.</returns>
        public IBlockTransform CreateCompressor()
        {
            return new LzmaCompressor(this.lzmaSettings);
        }

        /// <summary>
        /// Creates a decompressor object.
        /// </summary>
        /// <returns>A decompressor object.</returns>
        public IBlockTransform CreateDecompressor()
        {
            return new LzmaDecompressor(this.lzmaSettings);
        }

        /// <summary>
        /// Initialize compression algorithm using given compression settings.
        /// </summary>
        /// <param name="settings">Compression settings.</param>
        public void Initialize(CompressionSettings settings)
        {
            this.lzmaSettings = settings as LzmaSettings;
            if (this.lzmaSettings == null)
            {
                throw new ArgumentException("Wrong settings type.");
            }
        }
    }
}
