using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    internal class LzmaOutputWindow
    {
        private byte[] window = null;
        private uint end = 0;
        private uint windowSize = 0;
        private uint start = 0;

        private byte[] outputBuffer;
        private int outputOffset;

        public LzmaOutputWindow(uint blockSize)
        {
            this.windowSize = blockSize;
            this.window = new byte[blockSize];
        }

        public int AvailableBytes
        {
            get
            {
                int size = (int)(this.end - this.start);
                if (size < 0)
                {
                    size = (int)(this.end + this.windowSize - this.start);
                }

                return size;
            }
        }

        public int Copied
        {
            get
            {
                return this.outputOffset;
            }
        }

        public bool Full
        {
            get
            {
                return this.outputBuffer.Length - this.outputOffset < 1;
            }
        }

        public void SetOutputBuffer(byte[] buffer)
        {
            this.outputBuffer = buffer;
            this.outputOffset = 0;
        }

        public void Flush()
        {
            int length = this.outputBuffer.Length - this.outputOffset;
            if (length > 0)
            {
                int available = this.AvailableBytes;
                if (available > 0)
                {
                    if (length > available)
                    {
                        length = available;
                    }

                    this.Flush(length);
                }
            }
        }

        public void CopyBlock(uint distance, uint length)
        {
            uint position = this.end - distance - 1;
            if (position >= this.windowSize)
            {
                position += this.windowSize;
            }

            for (; length > 0; length--)
            {
                if (position >= this.windowSize)
                {
                    position = 0;
                }

                this.window[this.end++] = this.window[position++];
                if (this.end >= this.windowSize)
                {
                    this.Flush();
                    this.end = 0;
                }
            }
        }

        public void PutByte(byte value)
        {
            this.window[this.end++] = value;
            if (this.end >= this.windowSize)
            {
                this.Flush();
                this.end = 0;
            }
        }

        public byte GetByte(uint distance)
        {
            uint position = this.end - distance - 1;
            if (position >= this.windowSize)
            {
                position += this.windowSize;
            }

            return this.window[position];
        }

        private void Flush(int length)
        {
            if ((int)this.start + length > this.windowSize)
            {
                int rest = (int)(this.windowSize - this.start);
                Array.Copy(this.window, (int)this.start, this.outputBuffer, this.outputOffset, rest);
                this.outputOffset += rest;
                length -= rest;
                this.UpdatePosition(rest);
            }

            Array.Copy(this.window, (int)this.start, this.outputBuffer, this.outputOffset, length);
            this.outputOffset += length;

            this.UpdatePosition(length);
        }

        private void UpdatePosition(int length)
        {
            uint size = this.end - this.start;
            if (size != 0)
            {
                this.start += (uint)length;
                if (this.start >= this.windowSize)
                {
                    this.start -= this.windowSize;
                }
            }
        }
    }
}
