﻿#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents base class for LZMA compression and decompression functionality.
    /// </summary>
    internal abstract class LzmaTransformBase : CompressionTransformBase
    {
        public const uint RepeaterDistancesNumber = 4;
        public const uint StatesNumber = 12;

        public const int PosSlotBitsNumber = 6;
        public const int DictionaryLogSizeMin = 0;

        public const int LengthToPosStatesBits = 2;
        public const uint LengthToPosStates = 1 << LengthToPosStatesBits;

        public const uint MatchMinLength = 2;

        public const int AlignBitsNumber = 4;
        public const uint AlignTableSize = 1 << AlignBitsNumber;
        public const uint AlignMask = AlignTableSize - 1;

        public const uint StartPosModelIndex = 4;
        public const uint EndPosModelIndex = 14;
        public const uint PosModelsNumber = EndPosModelIndex - StartPosModelIndex;

        public const uint FullDistancesNumber = 1 << ((int)EndPosModelIndex / 2);

        public const uint LiteralPosStatesBitsEncodingMax = 4;
        public const int LiteralContextBitsMax = 8;

        public const int PosStatesBitsMax = 4;
        public const uint PosStatesMaxNumber = 1 << PosStatesBitsMax;
        public const int PosStatesBitsEncodingMax = 4;
        public const uint PosStatesEncodingMax = 1 << PosStatesBitsEncodingMax;

        public const int LowLengthBitsNumber = 3;
        public const int MiddleLengthBitsNumber = 3;
        public const int HighLengthBitsNumber = 8;
        public const uint LowLengthSymbolsNumber = 1 << LowLengthBitsNumber;
        public const uint MiddleLengthSymbolsNumber = 1 << MiddleLengthBitsNumber;
        public const uint LengthSymbolsNumber = LowLengthSymbolsNumber + MiddleLengthSymbolsNumber +
                (1 << HighLengthBitsNumber);
        public const uint MatchMaxLen = MatchMinLength + LengthSymbolsNumber - 1;

        /// <summary>
        /// Initializes a new instance of the <see cref="LzmaTransformBase"/> class.
        /// </summary>
        /// <param name="settings">Settings.</param>
        public LzmaTransformBase(LzmaSettings settings)
        {
            this.Settings = settings;
        }

        protected LzmaSettings Settings
        {
            get;
            private set;
        }
    }
}
