﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Crypto provider which implements traditional PKWARE encryption.
    /// </summary>
    public class DefaultCryptoProvider : ICryptoProvider 
    {
        private DefaultEncryptionSettings cryptoSettings = null;

        /// <summary>
        /// Creates an decryptor object. 
        /// </summary>
        /// <returns>A decryptor object.</returns>
        public IBlockTransform CreateDecryptor()
        {
            DefaultDecryptor decryptor = null;
            DefaultDecryptor tempDecryptor = null;
            try
            {
                tempDecryptor = new DefaultDecryptor();
                tempDecryptor.InitKeys(this.cryptoSettings.Password);
                decryptor = tempDecryptor;
                tempDecryptor = null;
            }
            finally
            {
                if (tempDecryptor != null)
                {
                    tempDecryptor.Dispose();
                }
            }
            return decryptor;
        }

        /// <summary>
        /// Creates an encryptor object.
        /// </summary>
        /// <returns></returns>
        public IBlockTransform CreateEncryptor()
        {
            DefaultEncryptor encryptor = null;
            DefaultEncryptor tempEncryptor = null;
            try
            {
                tempEncryptor = new DefaultEncryptor();
                tempEncryptor.InitKeys(this.cryptoSettings.Password);

                if (this.cryptoSettings.FileTime > 0)
                {
                    byte[] initData = new byte[]
                    {
                        (byte)((this.cryptoSettings.FileTime >> 8) & 0xff)
                    };
                    tempEncryptor.Header.InitData = initData;
                }

                encryptor = tempEncryptor;
                tempEncryptor = null;
            }
            finally
            {
                if (tempEncryptor != null)
                {
                    tempEncryptor.Dispose();
                }
            }
            return encryptor;
        }

        /// <summary>
        /// Initialize crypto provider using given encryption settings.
        /// </summary>
        /// <param name="settings">Encryption settings.</param>
        public void Initialize(EncryptionSettings settings)
        {
            if (settings == null)
            {
                throw new ArgumentNullException("settings");
            }

            this.cryptoSettings = settings as DefaultEncryptionSettings;

            if (this.cryptoSettings == null)
            {
                throw new ArgumentException("Invalid argument type", "settings");
            }
        }
    }
}
