﻿using System;
using System.Collections.Generic;
using System.Text;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Crypto transformation which implements traditional PKWARE decryption.
    /// </summary>
    internal class DefaultDecryptor : DefaultCryptoTransformBase
    {
        /// <summary>
        /// Initializes a new instance of the DefaultDecryptor class.
        /// </summary>
        public DefaultDecryptor()
            : base()
        {
        }

        /// <summary>Transforms the specified region of the input byte array and copies the resulting transform to the specified region of the output byte array.</summary>
        /// <returns>The number of bytes written.</returns>
        /// <param name="inputBuffer">The input for which to compute the transform. </param>
        /// <param name="inputOffset">The offset into the input byte array from which to begin using data. </param>
        /// <param name="inputCount">The number of bytes in the input byte array to use as data. </param>
        /// <param name="outputBuffer">The output to which to write the transform. </param>
        /// <param name="outputOffset">The offset into the output byte array from which to begin writing data. </param>
        public override int TransformBlock(byte[] inputBuffer, int inputOffset, int inputCount, byte[] outputBuffer, int outputOffset)
        {
            this.ValidateParameters(inputBuffer, inputOffset, inputCount, outputBuffer, outputOffset, true);

            for (int i = 0; i < inputCount; i++)
            {
                byte decrypted = (byte)(inputBuffer[inputOffset + i] ^ this.EncodingByte);
                UpdateKeys(decrypted);
                outputBuffer[outputOffset + i] = decrypted;
            }

            return inputCount;
        }

        /// <summary>Transforms the specified region of the specified byte array.</summary>
        /// <returns>The computed transform.</returns>
        /// <param name="inputBuffer">The input for which to compute the transform. </param>
        /// <param name="inputOffset">The offset into the byte array from which to begin using data. </param>
        /// <param name="inputCount">The number of bytes in the byte array to use as data. </param>
        public override byte[] TransformFinalBlock(byte[] inputBuffer, int inputOffset, int inputCount)
        {
            this.ValidateInputBufferParameters(inputBuffer, inputOffset, inputCount, false, true);

            byte[] outputBuffer = new byte[inputCount];
            this.TransformBlock(inputBuffer, inputOffset, inputCount, outputBuffer, 0);
            return outputBuffer;
        }
    }
}
