﻿using System;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Interface which provides method to encrypt/decrypt data in the ZIP archive.
    /// </summary>
    public interface ICryptoProvider
    {
        /// <summary>
        /// Creates an decryptor object. 
        /// </summary>
        /// <returns>A decryptor object.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Decryptor")]
        IBlockTransform CreateDecryptor();

        /// <summary>
        /// Creates an encryptor object.
        /// </summary>
        /// <returns>An encryptor object.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Encryptor")]
        IBlockTransform CreateEncryptor();

        /// <summary>
        /// Initialize crypto provider using given encryption settings.
        /// </summary>
        /// <param name="settings">Encryption settings.</param>
        void Initialize(EncryptionSettings settings);
    }
}
