﻿using System;
using System.IO;
using System.Text;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Interface which provides platform-specific operations. 
    /// </summary>
    public interface IPlatformManager
    {
        /// <summary>
        /// Gets a platform-specific alternate character used to separate directory levels in a path string that reflects a hierarchical file system organization.
        /// </summary>
        string AltDirectorySeparatorChar
        {
            get;
        }

        /// <summary>
        /// Gets default encoding for this platform.
        /// </summary>
        Encoding DefaultEncoding
        {
            get;
        }

        /// <summary>
        /// Gets a platform-specific character used to separate directory levels in a path string that reflects a hierarchical file system organization.
        /// </summary>
        string DirectorySeparatorChar
        {
            get;
        }

        /// <summary>
        /// Creates temporary stream.
        /// </summary>
        /// <returns>Stream will be used for temporary operations.</returns>
        Stream CreateTemporaryStream();

        /// <summary>
        /// Deletes temporary stream.
        /// </summary>
        /// <param name="stream">Stream to delete.</param>
        void DeleteTemporaryStream(Stream stream);

        /// <summary>
        /// Gets crypto provider initialized using given encryption settings.
        /// </summary>
        /// <param name="settings">Encryption settings.</param>
        /// <returns>Crypto provider.</returns>
        /// <exception cref="NotSupportedException">Specified crypto algorithm is not supported.</exception>
        ICryptoProvider GetCryptoProvider(EncryptionSettings settings);

        /// <summary>
        /// Indicates whether specified encoding is supported for this platform.
        /// </summary>
        /// <param name="encoding"></param>
        /// <returns></returns>
        bool IsEncodingSupported(Encoding encoding);
    }
}