﻿using System;
using System.ComponentModel;
using System.IO;

namespace Telerik.Windows.Zip
{
    /// <summary>
    /// Represents a stream that can write into a compressed stream.
    /// </summary>
    [Obsolete("This class has been deprecated. Use CompressedStream instead of ZipOutputStream.")]
    [EditorBrowsable(EditorBrowsableState.Never)]
    public class ZipOutputStream : CompressedStream
    {
        private byte[] rb = new byte[1];

        /// <summary>
        /// Initializes a new instance of the ZipOutputStream class.
        /// </summary>
        /// <param name="baseStream">
        /// The stream that will be compressed.
        /// </param>
        public ZipOutputStream(Stream baseStream)
            : this(baseStream, CompressionMethod.Deflate)
        {
        }

        /// <summary>
        /// Initializes a new instance of the ZipOutputStream class.
        /// </summary>
        /// <param name="baseStream">
        /// The stream that will be compressed.
        /// </param>
        /// /// <param name="method">
        /// The compression method.
        /// </param>
        public ZipOutputStream(Stream baseStream, CompressionMethod method)
            : base(baseStream, StreamOperationMode.Write, ZipHelper.GetCompressionSettings(method, null), false, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the ZipOutputStream class.
        /// </summary>
        /// <param name="baseStream">
        /// The stream that will be compressed.
        /// </param>
        /// /// <param name="compressionLevel">
        /// The compression level.
        /// </param>
        public ZipOutputStream(Stream baseStream, ZipCompression compressionLevel)
            : base(baseStream, StreamOperationMode.Write, ZipOutputStream.CreateDeflateSettings(compressionLevel), false, null)
        {
        }

        /// <summary>
        /// The stream that is decompressed.
        /// </summary>
        public new Stream BaseStream
        {
            get
            {
                return base.BaseStream;
            }
        }

        /// <summary>
        /// Gets the uncompressed size of the stream.
        /// </summary>
        public int UncompressedSize
        {
            get
            {
                return (int)this.TotalPlainCount;
            }
        }

        /// <summary>
        /// Writes a byte to the current position in the stream and advances the
        /// position within the stream by one byte.
        /// </summary>
        /// <param name="value">The byte to write to the stream. </param>
        public override void WriteByte(byte value)
        {
            this.rb[0] = value;
            this.Write(this.rb, 0, 1);
        }

        /// <summary>
        /// Create DeflateSettings for specified compression level.
        /// </summary>
        /// <param name="compressionLevel">Compression level.</param>
        /// <returns>DeflateSettings.</returns>
        internal static DeflateSettings CreateDeflateSettings(ZipCompression compressionLevel)
        {
            DeflateSettings settings = new DeflateSettings();
            settings.CompressionLevel = compressionLevel == ZipCompression.Default ? CompressionLevel.Optimal : (CompressionLevel)(int)compressionLevel;

            return settings;
        }
    }
}
