﻿using System;

#if PORTABLE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
	/// <summary>
	/// Test work item type that does not complete until the sleep time has 
	/// elapsed.  This is NOT a blocking Sleep.
	/// </summary>
	public class AsyncSleepWorkItem : AsyncWorkItem
	{
		/// <summary>
		/// The amount of milliseconds to wait for.
		/// </summary>
		private int sleepTime;

		/// <summary>
		/// The DateTime that marks the point in time the task is complete.
		/// </summary>
		private DateTime? expires;

		/// <summary>
		/// Create a new Sleep work item, including the number of 
		/// milliseconds to wait until continuing.
		/// </summary>
		/// <param name="sleepMilliseconds">Milliseconds to wait.</param>
		public AsyncSleepWorkItem(int sleepMilliseconds)
		{
			this.sleepTime = sleepMilliseconds;
			this.expires = null;
		}

		/// <summary>
		/// On the first time, will calculate the final DateTime.  Otherwise, 
		/// null operation (returns) until that time.
		/// </summary>
		public override void Invoke()
		{
			if (this.expires == null)
			{
				this.expires = DateTime.Now.AddMilliseconds(this.sleepTime);
			}
			if (DateTime.Now > this.expires)
			{
				this.IsComplete = true;
			}
		}
	}
}