﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;

#if NETCORE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
    [TestClass]
    public class CompressedStreamTest
    {
        private const string FilePath = "/Telerik.Windows.Zip.Tests;component/TestData/test.xml";

        [TestMethod]
        public void ZipZLibStreamTest()
        {
            CompressionSettings settings = new DeflateSettings();
            StreamTest(FilePath, settings);
        }

        [TestMethod]
        public void LzmaStreamTest()
        {
            CompressionSettings settings = new LzmaSettings()
            {
                DictionarySize = 11,
                FastBytes = 10,
                LiteralContextBits = 0,
                LiteralPositionBits = 0,
                PositionStateBits = 0
            };

            StreamTest(FilePath, settings);
        }

        [TestMethod]
        public void LzmaUsingEndOfStreamTest()
        {
            LzmaSettings settings = new LzmaSettings();

            using (MemoryStream stream = ZipTestsHepler.GetResourceStream(FilePath))
            {
                using (MemoryStream baseStream = new MemoryStream())
                {
                    CheckStreamHeader(stream, settings, baseStream, -1);

                    settings.StreamLength = stream.Length;
                    CheckStreamHeader(stream, settings, baseStream, stream.Length);
                }
            }
        }

        private static void CheckStreamHeader(Stream stream, LzmaSettings settings, Stream baseStream, long expectedLength)
        {
            baseStream.SetLength(0);
            stream.Position = 0;

            using (CompressedStream compressedStream = new CompressedStream(baseStream, StreamOperationMode.Write, settings))
            {
                stream.CopyTo(compressedStream);
            }

            InspectStreams(stream, settings, baseStream);

            baseStream.Position = 0;
            byte[] header = new byte[13];
            baseStream.Read(header, 0, header.Length);

            long uncompressedStreamLength = GetLzmaUncompressedSize(header);

            Assert.AreEqual(uncompressedStreamLength, expectedLength,
                "Lzma stream header contains incorrect length.");
        }

        private static long GetLzmaUncompressedSize(byte[] header)
        {
            long uncompressedSize = BitConverter.ToInt64(header, 5);

            return uncompressedSize;
        }

        private static long GetResourceStreamLength(string filePath)
        {
            using (MemoryStream stream = ZipTestsHepler.GetResourceStream(filePath))
            {
                return stream.Length;
            }
        }

        private static void StreamTest(string filePath, CompressionSettings settings)
        {
            using (MemoryStream stream = ZipTestsHepler.GetResourceStream(filePath))
            {
                using (MemoryStream baseStream = new MemoryStream())
                {
                    using (CompressedStream compressedStream = new CompressedStream(baseStream, StreamOperationMode.Write, settings))
                    {
                        stream.CopyTo(compressedStream);
                    }

                    InspectStreams(stream, settings, baseStream);
                }
            }
        }

        private static void InspectStreams(Stream stream, CompressionSettings settings, Stream baseStream)
        {
            using (Stream inspectedStream = new MemoryStream())
            {
                baseStream.Position = 0;
                using (CompressedStream compressedStream = new CompressedStream(baseStream, StreamOperationMode.Read, settings))
                {
                    compressedStream.CopyTo(inspectedStream);
                }

                Assert.IsTrue(ZipTestsHepler.CompareStreams(stream, inspectedStream),
                    "Stream is broken after compress/decompress process.");
            }
        }
    }
}
