﻿using System;
using System.IO;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

#if NETCORE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
	[TestClass]
	public class CryptoStreamTest
	{
		[TestMethod]
		public void TestStoreAlgorithmWithEncryption()
		{
			StoreSettings compressionSettings = new StoreSettings();

			DefaultEncryptionSettings encryptionSettings = new DefaultEncryptionSettings()
			{
				Password = "TestZip"
			};

			string testString = this.GetTestString();

			using (MemoryStream bufferStream = new MemoryStream())
			{
				using (Stream outputStream = new CompressedStream(
					bufferStream,
					StreamOperationMode.Write,
					compressionSettings,
					true,
					encryptionSettings))
				{
					using (StreamWriter writer = new StreamWriter(outputStream))
					{
						writer.Write(testString);
					}
				}

				bufferStream.Seek(0L, SeekOrigin.Begin);

				using (Stream inputStream = new CompressedStream(
					bufferStream,
					StreamOperationMode.Read,
					compressionSettings,
					true,
					encryptionSettings))
				{
					using (StreamReader reader = new StreamReader(inputStream))
					{
						string line = reader.ReadLine();
						Assert.AreEqual(testString, line, "Decrypted string is wrong");
					}
				}
			}
		}

		[TestMethod]
		public void TestCompressedSizeForStoreAlgorithmWithEncryption()
		{
			StoreSettings compressionSettings = new StoreSettings();

			DefaultEncryptionSettings encryptionSettings = new DefaultEncryptionSettings()
			{
				Password = "TestZip"
			};

			string testString = this.GetTestString();
			long compressedSize = 0L;

			using (MemoryStream bufferStream = new MemoryStream())
			{
				using (CompressedStream outputStream = new CompressedStream(
					bufferStream,
					StreamOperationMode.Write,
					compressionSettings,
					true,
					encryptionSettings))
				{
					using (StreamWriter writer = new StreamWriter(outputStream))
					{
						writer.Write(testString);
					}
					compressedSize = outputStream.CompressedSize;
				}

				// Traditional PKWARE encryption adds 12 bytes 
				// to the size of the encrypted data.
				Assert.AreEqual(testString.Length + 12, compressedSize, "Compressed size of the output stream is wrong");

				bufferStream.Seek(0L, SeekOrigin.Begin);

				using (CompressedStream inputStream = new CompressedStream(
					bufferStream,
					StreamOperationMode.Read,
					compressionSettings,
					true,
					encryptionSettings))
				{
					using (StreamReader reader = new StreamReader(inputStream))
					{
						string line = reader.ReadToEnd();
						Assert.AreEqual(testString, line, "Decrypted string is wrong");
					}
					Assert.AreEqual(compressedSize, inputStream.CompressedSize, "Compressed size of the input stream is wrong");
				}
			}
		}

		private string GetTestString()
		{
			int charCount = 8200;
			string testString = string.Empty;
			for (int i = 0; i < charCount; i++)
			{
				testString += (i % 10).ToString();
			}

			return testString;
		}
	}
}
