﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;

#if NETCORE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
    [TestClass]
    public class EncryptedZipArchiveTest
    {
        [TestMethod]
        public void ZipPasswordTest()
        {
            const string password = "PaSsWoRd";
            const string fileName = "LoremIpsum.txt";
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/" + fileName;

            DefaultEncryptionSettings encryptionSettings = new DefaultEncryptionSettings();
            encryptionSettings.Password = password;

            using (MemoryStream uncompressedStream = ZipTestsHepler.GetResourceStream(filePath))
            {
                MemoryStream compressedStream = new MemoryStream();
                using (ZipArchive zipArchive = new ZipArchive(compressedStream, ZipArchiveMode.Create, true, null, null, encryptionSettings))
                {
                    using (ZipArchiveEntry entry = zipArchive.CreateEntry(fileName))
                    {
                        Stream entryStream = entry.Open();
                        uncompressedStream.CopyTo(entryStream);
                    }
                }

                using (Stream inspectedStream = new MemoryStream())
                {
                    using (ZipArchive zipArchive = new ZipArchive(compressedStream, ZipArchiveMode.Read, true, null))
                    {
                        ZipArchiveEntry entry = zipArchive.GetEntry(fileName);
                        Assert.IsNotNull(entry, "Entry is not found.");
                        try
                        {
                            Stream stream = entry.Open();
                            stream.CopyTo(inspectedStream);
                            Assert.Fail("Password protection failed");
                        }
                        catch
                        {
                        }
                        finally
                        {
                            try
                            {
                                entry.Dispose();
                            }
                            catch
                            {
                            }
                        }
                    }

                    inspectedStream.Position = 0;

                    using (ZipArchive zipArchive = new ZipArchive(compressedStream, ZipArchiveMode.Read, false, null, null, encryptionSettings))
                    {
                        ZipArchiveEntry entry = zipArchive.GetEntry(fileName);
                        Assert.IsNotNull(entry, "Entry is not found.");

                        Stream stream = entry.Open();
                        stream.CopyTo(inspectedStream);
                    }

                    Assert.IsTrue(ZipTestsHepler.CompareStreams(uncompressedStream, inspectedStream),
                        "Stream is broken after compress/decompress process.");
                }
            }
        }
    }
}
