﻿using System;
using System.IO;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

#if NETCORE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
    [TestClass]
    public class LzmaZipArchiveTest
    {
        [TestMethod]
        public void LzmaZipCompressionTest()
        {
            const string fileName = "LoremIpsum.txt";
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/" + fileName;

            using (MemoryStream uncompressedStream = ZipTestsHepler.GetResourceStream(filePath))
            {
                MemoryStream compressedStream = new MemoryStream();
                CompressionSettings settings = new LzmaSettings();

                using (ZipArchive zipArchive = new ZipArchive(compressedStream, ZipArchiveMode.Create, true, null, settings, null))
                {
                    using (ZipArchiveEntry entry = zipArchive.CreateEntry(fileName))
                    {
                        Stream entryStream = entry.Open();
                        uncompressedStream.CopyTo(entryStream);
                    }
                }

                using (Stream inspectedStream = new MemoryStream())
                {
                    using (ZipArchive zipArchive = new ZipArchive(compressedStream))
                    {
                        ZipArchiveEntry entry = zipArchive.GetEntry(fileName);
                        Assert.IsNotNull(entry, "Entry is not found.");

                        Stream stream = entry.Open();
                        stream.CopyTo(inspectedStream);
                    }

                    Assert.IsTrue(ZipTestsHepler.CompareStreams(uncompressedStream, inspectedStream),
                        "Stream is broken after compress/decompress process.");
                }
            }
        }

        [TestMethod]
        public void LzmaZipCompressedFileTest()
        {
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/";
            const string archivePath = filePath + "LzmaTestZip.zip";
            const string fileName = "test.xml";
            const string entryName = "Data/" + fileName;
            const string fileToComparePath = filePath + fileName;

            MemoryStream zipStream = ZipTestsHepler.GetResourceStream(archivePath);
            EntryData[] entriesData = LzmaZipArchiveTest.GetCompressedFileData();

            using (ZipArchive zipArchive = new ZipArchive(zipStream))
            {
                var entries = zipArchive.Entries.ToArray();
                Assert.AreEqual<int>(entries.Length,
                    entriesData.Length,
                    "Count of created entries is incorrect.");

                for (int i = 0; i < entriesData.Length; i++)
                {
                    EntryData data = entriesData[i];
                    data.CompareToZipArchiveEntry(entries[i]);
                }

                using (Stream inspectedStream = new MemoryStream())
                {
                    ZipArchiveEntry entry = zipArchive.GetEntry(entryName);
                    Assert.IsNotNull(entry, "Entry is not found.");

                    Stream stream = entry.Open();
                    stream.CopyTo(inspectedStream);
                    using (MemoryStream uncompressedStream = ZipTestsHepler.GetResourceStream(fileToComparePath))
                    {
                        Assert.IsTrue(ZipTestsHepler.CompareStreams(uncompressedStream, inspectedStream),
                            "File in archive is broken.");
                    }
                }
            }
        }

        private static EntryData[] GetCompressedFileData()
        {
            EntryData[] entriesData = new EntryData[]
            {
                new EntryData()
                {
                    FullName = "Data/",
                    Attributes = FileAttributes.Directory,
                    CompressedLength = 0,
                    Length = 0
                },

                new EntryData()
                {
                    FullName = "Data/telerik.png",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 1252,
                    Length = 1252
                },

                new EntryData()
                {
                    FullName = "Data/test.xml",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 611,
                    Length = 1728
                },

                new EntryData()
                {
                    FullName = "ZIP File Format Specification v6.3.3.txt",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 36407,
                    Length = 161414
                }
            };

            return entriesData;
        }

    }
}
