﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Windows;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Net;
#if WPF
using System.Net.Sockets;
#endif
using System.Threading;
using System.Text;

namespace Telerik.Windows.Zip.Tests
{
#pragma warning disable 618
    [TestClass]
    public partial class OldApiTests
    {
#if WPF
        private const string LocalHost = "127.0.0.1";
        private const int Port = 2055;
        private TcpListener listener;
        private object lockObject = new object();
        private string serverContent = null;
        private long inputChecksum;
#endif

        [TestMethod]
        public void OldApiZLibStreamTest()
        {
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/test.xml";
            using (MemoryStream stream = ZipTestsHepler.GetResourceStream(filePath))
            {
                using (MemoryStream baseStream = new MemoryStream())
                {
                    using (ZipOutputStream compressedStream = new ZipOutputStream(baseStream))
                    {
                        stream.CopyTo(compressedStream);
                    }

                    using (Stream inspectedStream = new MemoryStream())
                    {
                        baseStream.Position = 0;
                        using (ZipInputStream compressedStream = new ZipInputStream(baseStream))
                        {
                            compressedStream.CopyTo(inspectedStream);
                        }

                        Assert.IsTrue(ZipTestsHepler.CompareStreams(stream, inspectedStream),
                            "Stream is broken after compress/decompress process.");
                    }
                }
            }
        }

        [TestMethod]
        public void OldApiCompressionTest()
        {
            const string fileName = "LoremIpsum.txt";
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/" + fileName;

            using (MemoryStream uncompressedStream = ZipTestsHepler.GetResourceStream(filePath))
            {
                using (MemoryStream compressedStream = new MemoryStream())
                {
                    ZipPackage zipPackage = ZipPackage.Create(compressedStream);
                    zipPackage.AddStream(uncompressedStream, fileName);
                    zipPackage.Close(false);

                    using (Stream inspectedStream = new MemoryStream())
                    {
                        using (zipPackage = ZipPackage.Open(compressedStream))
                        {
                            int index = zipPackage.IndexOf(fileName);

                            Assert.AreEqual<int>(0, index, "Entry index is incorrect.");

                            ZipPackageEntry entry = zipPackage.ZipPackageEntries[index];
                            Assert.IsNotNull(entry, "Entry is not found.");

                            Stream stream = entry.OpenInputStream();
                            stream.CopyTo(inspectedStream);
                        }

                        Assert.IsTrue(ZipTestsHepler.CompareStreams(uncompressedStream, inspectedStream),
                            "Stream is broken after compress/decompress process.");
                    }
                }
            }
		}

        [TestMethod]
        public void OldApiCompressedFileTest()
        {
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/TestZip.zip";
            EntryData[] entriesData = ZipTestsHepler.GetCompressedFileData();
            MemoryStream zipStream = ZipTestsHepler.GetResourceStream(filePath);

            using (ZipPackage zipPackage = ZipPackage.Open(zipStream))
            {
                var entries = zipPackage.ZipPackageEntries;

                Assert.AreEqual<int>(entries.Count,
                    entriesData.Length,
                    "Count of created entries is incorrect.");

                for (int i = 0; i < entriesData.Length; i++)
                {
                    EntryData data = entriesData[i];
                    data.CompareToZipPackageEntry(entries[i]);
                }
            }
        }

        [TestMethod]
        public void OldApiEntriesTest()
        {
            const int fileCount = 10;
            const string fileNameFormat = "Hello World #{0}.txt";
            MemoryStream zipStream = new MemoryStream();
            ZipPackage zipPackage = ZipPackage.Create(zipStream);
            for (int i = 0; i < fileCount; i++)
            {
                AddTestEntry(zipPackage, fileNameFormat, i);
            }
            zipPackage.Close(false);

            using (zipPackage = ZipPackage.Open(zipStream))
            {
                Assert.AreEqual<int>(zipPackage.ZipPackageEntries.Count,
                    fileCount,
                    "Count of created entries is incorrect.");

                this.AddTestEntry(zipPackage, fileNameFormat, fileCount);

                Assert.AreEqual<int>(zipPackage.ZipPackageEntries.Count,
                    fileCount + 1,
                    "Count of created entries is incorrect.");

                zipPackage.RemoveEntry(zipPackage.ZipPackageEntries[0]);
                zipPackage.RemoveEntry(zipPackage.ZipPackageEntries[0]);
                zipPackage.RemoveEntry(zipPackage.ZipPackageEntries[0]);

                Assert.AreEqual<int>(zipPackage.ZipPackageEntries.Count,
                    fileCount - 2,
                    "Count of created entries is incorrect.");
            }
        }

        [TestMethod]
        public void OldApiReadWordDocxTest()
        {
            const string zipFilePath = "/Telerik.Windows.Zip.Tests;component/TestData/LongDoc.docx";
            const string fileName = "document.xml";
            const string compressedFilePath = "word/" + fileName;
            const string originalFilePath = "/Telerik.Windows.Zip.Tests;component/TestData/" + fileName;

            MemoryStream zipStream = ZipTestsHepler.GetResourceStream(zipFilePath);
            using (ZipPackage zipPackage = ZipPackage.Open(zipStream))
            {
                ZipPackageEntry entry = zipPackage.ZipPackageEntries.First(p => p.FileNameInZip == compressedFilePath);
                using (Stream input = entry.OpenInputStream())
                {
                    MemoryStream uncompressedStream = new MemoryStream();
                    input.CopyTo(uncompressedStream);

                    MemoryStream originalStream = ZipTestsHepler.GetResourceStream(originalFilePath);

                    Assert.IsTrue(ZipTestsHepler.CompareStreams(uncompressedStream, originalStream),
                        "File in the Word document is broken.");
                }
            }
        }

        private void AddTestEntry(ZipPackage zipPackage, string fileNameFormat, int fileIndex)
        {
            MemoryStream testStream = new MemoryStream();
            using (StreamWriter writter = new StreamWriter(testStream))
            {
                string fileName = string.Format(fileNameFormat, fileIndex.ToString());
                writter.Write(fileName);
                testStream.Position = 0;
                zipPackage.AddStream(testStream, fileName);
            }
        }

#if WPF

        [TestMethod]
        public void OldApiNonSeekableStreamTest()
        {
            const string fileName = "LoremIpsum.txt";
            const string filePath = "/Telerik.Windows.Zip.Tests;component/TestData/" + fileName;

            this.listener = new TcpListener(IPAddress.Parse(LocalHost), Port);
            this.listener.Start();

            Thread thread = new Thread(new ThreadStart(ServiceThread));
            thread.IsBackground = true;
            thread.Start();

            string originalContent = string.Empty;
            using (MemoryStream originalStream = ZipTestsHepler.GetResourceStream(filePath))
            {
                StreamReader reader = new StreamReader(originalStream, Encoding.UTF8);
                originalContent = reader.ReadToEnd();
            }

            long outputChecksum = 0;
            TcpClient client = new TcpClient();
            client.SendTimeout = 1500;
            client.Connect(LocalHost, Port);
            try
            {
                NetworkStream socketStream = new NetworkStream(client.Client, FileAccess.Write);
                ZipOutputStream output = new ZipOutputStream(socketStream);
                byte[] buffer = Encoding.UTF8.GetBytes(originalContent);
                output.Write(buffer, 0, buffer.Length);
                output.Flush();
                outputChecksum = output.Checksum;
            }
            catch (Exception ex)
            {
                Assert.Fail("Write exception: " + ex.ToString());
            }

            client.Close();

            lock (this.lockObject)
            {
                Monitor.Wait(this.lockObject);
            }

            Assert.AreEqual(originalContent, this.serverContent, "Content is broken");
            Assert.AreEqual(outputChecksum, this.inputChecksum, "Checksum isn't the same.");

            this.listener.Stop();
            this.listener = null;
        }

        public void ServiceThread()
        {
            bool execute = true;
            while (execute)
            {
                Socket socket = this.listener.AcceptSocket();
                socket.ReceiveTimeout = 5000;
                try
                {
                    NetworkStream socketStream = new NetworkStream(socket, FileAccess.Read);
                    while (!socketStream.DataAvailable)
                    {
                        Thread.Sleep(100);
                    }

                    ZipInputStream input = new ZipInputStream(socketStream);

                    StreamReader reader = new StreamReader(input, Encoding.UTF8);
                    this.serverContent = reader.ReadToEnd();

                    socketStream.Close();
                    this.inputChecksum = input.Checksum;
                    execute = false;
                }
                catch (Exception ex)
                {
                    Assert.Fail("Read exception: " + ex.ToString());
                }

                socket.Close();
            }

            lock (this.lockObject)
            {
                Monitor.PulseAll(this.lockObject);
            }
        }
#endif

#pragma warning restore 618
    }
}