﻿using System;
using System.IO;
using System.Linq;
using System.Reflection;

#if NETCORE
namespace Telerik.Zip.Tests
#else
namespace Telerik.Windows.Zip.Tests
#endif
{
    public static class ZipTestsHepler
    {
        public static bool CompareStreams(Stream uncompressedStream, Stream inspectedStream)
        {
            const int bufferLength = 4096;

            uncompressedStream.Position = 0;
            inspectedStream.Position = 0;

            byte[] buffer = new byte[bufferLength];
            byte[] inspectedBuffer = new byte[bufferLength];
            int read;
            while ((read = uncompressedStream.Read(buffer, 0, bufferLength)) > 0)
            {
                int inspected = inspectedStream.Read(inspectedBuffer, 0, read);
                if (inspected != read)
                {
                    return false;
                }

                for (int i = 0; i < read; i++)
                {
                    if (buffer[i] != inspectedBuffer[i])
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        public static EntryData[] GetCompressedFileData()
        {
            EntryData[] entriesData = new EntryData[]
            {
                new EntryData()
                {
                    FullName = "Data/",
                    Attributes = FileAttributes.Directory,
                    CompressedLength = 0,
                    Length = 0
                },

                new EntryData()
                {
                    FullName = "Data/telerik.png",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 1252,
                    Length = 1252
                },

                new EntryData()
                {
                    FullName = "Data/test.xml",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 601,
                    Length = 1728
                },

                new EntryData()
                {
                    FullName = "ZIP File Format Specification v6.3.3.txt",
                    Attributes = FileAttributes.Archive,
                    CompressedLength = 40375,
                    Length = 161414
                }
            };

            return entriesData;
        }

        internal static MemoryStream GetResourceStream(string filePath, bool readOnly = true)
        {
            MemoryStream stream = new MemoryStream();
            Stream fileStream;
#if NETCORE
            var path = filePath.Split(new string[] { "component/" }, StringSplitOptions.RemoveEmptyEntries)[1];
            path = path.Replace('/', '.');
            Assembly assembly = typeof(ZipTestsHepler).Assembly;
            var resourceName = assembly.GetManifestResourceNames().Where(p => p.Contains(path)).FirstOrDefault();
            fileStream = assembly.GetManifestResourceStream(resourceName);
#else
            fileStream = System.Windows.Application.GetResourceStream(new Uri(filePath, UriKind.Relative)).Stream;
#endif
            fileStream.CopyTo(stream);
            stream.Position = 0;

            if (!readOnly)
            {
                return stream;
            }

            MemoryStream readOnlyStream = new MemoryStream(stream.ToArray(), false);
            ((IDisposable)stream).Dispose();

            return readOnlyStream;
        }
    }
}
