﻿using System;
using System.Collections.Generic;
using System.Text;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents header of the transformation.
    /// The extra data precedes the transformed data which provides
    /// some additional information about transformation (compression or encryption).
    /// </summary>
    public class TransformationHeader
    {
        /// <summary>
        /// Initializes a new instance of the TransformationHeader class.
        /// </summary>
        internal TransformationHeader()
        {
            this.Buffer = null;
            this.BytesToRead = 0;
        }

        /// <summary>
        /// Gets or sets buffer to store header information.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
        public byte[] Buffer
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets number of byte to read.
        /// </summary>
        public int BytesToRead
        {
            get;
            set;
        }

        /// <summary>
        /// Gets initialization data of the header.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
        public byte[] InitData
        {
            get;
            set;
        }

        /// <summary>
        /// Gets length of the transformation header.
        /// </summary>
        public int Length
        {
            get;
            internal set;
        }

        /// <summary>
        /// Gets or sets the flag which indicates
        /// that the compressed size should include the header size.
        /// </summary>
        public bool CountHeaderInCompressedSize
        {
            get;
            set;
        }
    }
}
