﻿using System;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents data descriptor record described in the
    /// ZIP File Format Specification v6.3.3, #4.3.9.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
    internal class Zip64DataDescriptor : Zip64DataDescriptorBase, ISpecData
    {
        /// <summary>
        /// Data descriptor header signature.
        /// </summary>
        public const uint Signature = 0x08074b50;

        /// <summary>
        /// Read data from the binary reader.
        /// </summary>
        /// <param name="reader">Binary reader to read data from.</param>
        /// <returns>True if success, otherwise false.</returns>
        public bool TryReadBlock(BinaryReader reader)
        {
            byte[] buffer = new byte[4];
            ZipHelper.ReadBytes(reader.BaseStream, buffer, buffer.Length);
            uint crcOrSignature = BitConverter.ToUInt32(buffer, 0);
            if (crcOrSignature == Zip64DataDescriptor.Signature)
            {
                this.ReadFields(reader);
            }
            else
            {
                this.Crc = crcOrSignature;
                this.ReadSize(reader);
            }

            return true;
        }

        /// <summary>
        /// Write data to the binary writer.
        /// </summary>
        /// <param name="writer">Binary writer to write data to.</param>
        public void WriteBlock(BinaryWriter writer)
        {
            writer.Write(DataDescriptor.Signature);
            this.WriteFields(writer);
        }
    }
}
