﻿using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents base fields of data descriptor record described in the
    /// ZIP File Format Specification v6.3.3, #4.3.9.
    /// </summary>
    internal class Zip64DataDescriptorBase
    {
        /// <summary>
        /// Gets or sets crc-32.
        /// </summary>
        public uint Crc
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets compressed size.
        /// </summary>
        public ulong CompressedSize
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets uncompressed size.
        /// </summary>
        public ulong UncompressedSize
        {
            get;
            set;
        }

        /// <summary>
        /// Read data from the binary reader.
        /// </summary>
        /// <param name="reader">Binary reader to read data from.</param>
        protected virtual void ReadFields(BinaryReader reader)
        {
            this.Crc = reader.ReadUInt32();
            this.ReadSize(reader);
        }

        /// <summary>
        /// Read data from the binary reader.
        /// </summary>
        /// <param name="reader">Binary reader to read data from.</param>
        protected void ReadSize(BinaryReader reader)
        {
            this.CompressedSize = reader.ReadUInt64();
            this.UncompressedSize = reader.ReadUInt64();
        }

        /// <summary>
        /// Write data to the binary writer.
        /// </summary>
        /// <param name="writer">Binary writer to write data to.</param>
        protected virtual void WriteFields(BinaryWriter writer)
        {
            writer.Write(this.Crc);
            writer.Write(this.CompressedSize);
            writer.Write(this.UncompressedSize);
        }
    }
}
