﻿using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents Zip64 end of central directory locator described in the
    /// ZIP File Format Specification v6.3.3, #4.3.15.
    /// </summary>
    internal class Zip64EndOfCentralDirectoryLocator : ISpecData
    {
        /// <summary>
        /// Zip64 end of central directory locator signature.
        /// </summary>
        public const uint Signature = 0x07064b50;

        /// <summary>
        /// Size of the data block without signature and variable size fields.
        /// </summary>
        public const int StaticBlockLength = 16;

        /// <summary>
        /// Gets or sets number of the disk with the
        /// start of the zip64 end of
        /// central directory.
        /// </summary>
        public uint NumberOfTheDiskWithTheStartOfTheZip64EndOfCentralDirectory
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets relative offset of the zip64
        /// end of central directory record.
        /// </summary>
        public ulong RelativeOffsetOfTheZip64EndOfCentralDirectoryRecord
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets number of disks.
        /// </summary>
        public uint NumberOfDisks
        {
            get;
            set;
        }

        /// <summary>
        /// Read data from the binary reader.
        /// </summary>
        /// <param name="reader">Binary reader to read data from.</param>
        /// <returns>True if success, otherwise false.</returns>
        public bool TryReadBlock(BinaryReader reader)
        {
            if (reader.ReadUInt32() != Zip64EndOfCentralDirectoryLocator.Signature)
            {
                return false;
            }

            this.NumberOfTheDiskWithTheStartOfTheZip64EndOfCentralDirectory = reader.ReadUInt32();
            this.RelativeOffsetOfTheZip64EndOfCentralDirectoryRecord = reader.ReadUInt64();
            this.NumberOfDisks = reader.ReadUInt32();

            return true;
        }

        /// <summary>
        /// Write data to the binary writer.
        /// </summary>
        /// <param name="writer">Binary writer to write data to.</param>
        public void WriteBlock(BinaryWriter writer)
        {
            writer.Write(Zip64EndOfCentralDirectoryLocator.Signature);
            writer.Write(this.NumberOfTheDiskWithTheStartOfTheZip64EndOfCentralDirectory);
            writer.Write(this.RelativeOffsetOfTheZip64EndOfCentralDirectoryRecord);
            writer.Write(this.NumberOfDisks);
        }
    }
}
