﻿using System;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents base class for extra field described in the
    /// ZIP File Format Specification v6.3.3, #4.5.2.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
    internal class AesEncryptionExtraField : ExtraFieldBase
    {
        /// <summary>
        /// Gets or sets vendor version for this record.
        /// </summary>
        public ushort VendorVersion
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets signature (AE).
        /// </summary>
        public ushort Signature
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets bit length of encryption key.
        /// 1 - 128-bit , 2 - 192-bit , 3 - 256-bit.
        /// </summary>
        public byte KeyLength
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets method.
        /// </summary>
        public ushort Method
        {
            get;
            set;
        }

        /// <summary>
        /// Gets extra field type (Header ID).
        /// </summary>
        protected override ushort HeaderId
        {
            get
            {
                return (ushort)ExtraFieldType.AesEncryption;
            }
        }

        /// <summary>
        /// Gets extra field data.
        /// </summary>
        /// <returns>Byte array of extra field data.</returns>
        protected override byte[] GetBlock()
        {
            byte[] fieldsData;
            using (MemoryStream stream = new MemoryStream())
            {
                using (BinaryWriter writer = new BinaryWriter(stream))
                {
                    writer.Write(this.VendorVersion);
                    writer.Write(this.Signature);
                    writer.Write(this.KeyLength);
                    writer.Write(this.Method);

                    fieldsData = stream.ToArray();
                }
            }

            return fieldsData;
        }

        /// <summary>
        /// Implements parsing of extra field data.
        /// </summary>
        /// <param name="extraData">Extra field data.</param>
        protected override void ParseBlock(byte[] extraData)
        {
            this.VendorVersion = BitConverter.ToUInt16(extraData, 0);
            this.Signature = BitConverter.ToUInt16(extraData, 2);
            this.KeyLength = extraData[4];
            this.Method = BitConverter.ToUInt16(extraData, 5);
        }
    }
}
