﻿using System;
using System.Collections.Generic;
using System.Text;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents identifier of the encryption algorithm described in the
    /// ZIP File Format Specification v6.3.3, #7.2.3.2.
    /// </summary>
    internal enum EncryptionAlgorithmType
    {
        /// <summary>
        /// Unknown algorithm.
        /// </summary>
        Unknown = -1,

        /// <summary>
        /// DES algorithm.
        /// </summary>
        Des = 0x6601,

        /// <summary>
        /// RC2 algorithm.
        /// The version needed to extract up to 5.2.
        /// </summary>
        Rc2Old = 0x6602,

        /// <summary>
        /// 3DES 168 bit algorithm.
        /// </summary>
        TripleDes168 = 0x6603,

        /// <summary>
        /// 3DES 112 bit algorithm.
        /// </summary>
        TripleDes112 = 0x6609,

        /// <summary>
        /// AES 128 bit algorithm.
        /// </summary>
        Aes128 = 0x660E,

        /// <summary>
        /// AES 192 bit algorithm.
        /// </summary>
        Aes192 = 0x660F,

        /// <summary>
        /// AES 256 bit algorithm.
        /// </summary>
        Aes256 = 0x6610,

        /// <summary>
        /// RC2 algorithm.
        /// The version needed to extract 5.2 and higher.
        /// </summary>
        Rc2 = 0x6702,

        /// <summary>
        /// Blowfish algorithm.
        /// </summary>
        Blowfish = 0x6720,

        /// <summary>
        /// Twofish algorithm.
        /// </summary>
        Twofish = 0x6721,

        /// <summary>
        /// RC4 algorithm.
        /// </summary>
        Rc4 = 0x6801
    }
}
