﻿using System;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents strong encryption extra field described in the
    /// ZIP File Format Specification v6.3.3, #4.5.12.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
    internal class StrongEncryptionExtraField : ExtraFieldBase
    {
        /// <summary>
        /// Gets or sets format definition for this record.
        /// </summary>
        public ushort Format
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets encryption algorithm identifier.
        /// </summary>
        public ushort AlgorithmId
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets bit length of encryption key.
        /// </summary>
        public ushort KeyLength
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets processing flags.
        /// </summary>
        public ushort Flags
        {
            get;
            set;
        }

        /// <summary>
        /// Gets extra field type (Header ID).
        /// </summary>
        protected override ushort HeaderId
        {
            get
            {
                return (ushort)ExtraFieldType.StrongEncryption;
            }
        }

        /// <summary>
        /// Gets extra field data.
        /// </summary>
        /// <returns>Byte array of extra field data.</returns>
        protected override byte[] GetBlock()
        {
            byte[] fieldsData;
            using (MemoryStream stream = new MemoryStream())
            {
                using (BinaryWriter writer = new BinaryWriter(stream))
                {
                    writer.Write(this.Format);
                    writer.Write(this.AlgorithmId);
                    writer.Write(this.KeyLength);
                    writer.Write(this.Flags);

                    fieldsData = stream.ToArray();
                }
            }

            return fieldsData;
        }

        /// <summary>
        /// Implements parsing of extra field data.
        /// </summary>
        /// <param name="extraData">Extra field data.</param>
        protected override void ParseBlock(byte[] extraData)
        {
            this.Format = BitConverter.ToUInt16(extraData, 0);
            this.AlgorithmId = BitConverter.ToUInt16(extraData, 2);
            this.KeyLength = BitConverter.ToUInt16(extraData, 4);
            this.Flags = BitConverter.ToUInt16(extraData, 6);
        }
    }
}
