﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents base class for extra field described in the
    /// ZIP File Format Specification v6.3.3, #4.5.2.
    /// </summary>
    internal class UnknownExtraField : ExtraFieldBase
    {
        private ushort headerId;

        /// <summary>
        /// Initializes a new instance of the UnknownExtraField class.
        /// </summary>
        /// <param name="fieldHeaderId">Header Id.</param>
        public UnknownExtraField(ushort fieldHeaderId)
        {
            this.headerId = fieldHeaderId;
        }

        /// <summary>
        /// Gets extra field type (Header ID).
        /// </summary>
        protected override ushort HeaderId
        {
            get
            {
                return this.headerId;
            }
        }

        /// <summary>
        /// Gets or sets extra field data.
        /// </summary>
        private byte[] ExtraFieldData
        {
            get;
            set;
        }

        /// <summary>
        /// Gets extra field data.
        /// </summary>
        /// <returns>Byte array of extra field data.</returns>
        protected override byte[] GetBlock()
        {
            return this.ExtraFieldData;
        }

        /// <summary>
        /// Implements parsing of extra field data.
        /// </summary>
        /// <param name="extraData">Extra field data.</param>
        protected override void ParseBlock(byte[] extraData)
        {
            this.ExtraFieldData = extraData;
        }
    }
}
