﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents general purpose bit flag in the 
    /// ZIP File Format Specification v6.3.3, #4.4.4.
    /// </summary>
    [Flags]
    internal enum GeneralPurposeBitFlag : ushort
    {
        /// <summary>
        /// Bit 0: If set, indicates that the file is encrypted.
        /// </summary>
        FileIsEncrypted = 0x0001,
              
        /// <summary>
        /// Bit 3: If this bit is set, the fields crc-32, compressed 
        /// size and uncompressed size are set to zero in the 
        /// local header.  The correct values are put in the 
        /// data descriptor immediately following the compressed
        /// data. 
        /// </summary>
        /// <remarks>
        /// Note: PKZIP version 2.04g for DOS only 
        /// recognizes this bit for method 8 compression, newer 
        /// versions of PKZIP recognize this bit for any 
        /// compression method.
        /// </remarks>
        ZeroLocalHeader = 0x0008,
        
        /// <summary>
        /// Bit 4: Reserved for use with method 8, for enhanced
        /// deflating. 
        /// </summary>
        ReservedForEnhancedDeflating = 0x0010,
        
        /// <summary>
        /// Bit 5: If this bit is set, this indicates that the file is 
        /// compressed patched data.  
        /// </summary>
        /// <remarks>
        /// Note: Requires PKZIP version 2.70 or greater.
        /// </remarks>
        CompressedPatchedData = 0x0020,
        
        /// <summary>
        /// Bit 6: Strong encryption.  If this bit is set, you MUST
        /// set the version needed to extract value to at least
        /// 50 and you MUST also set bit 0.  If AES encryption
        /// is used, the version needed to extract value MUST 
        /// be at least 51. See the section describing the Strong
        /// Encryption Specification for details.  Refer to the 
        /// section in this document entitled "Incorporating PKWARE 
        /// Proprietary Technology into Your Product" for more 
        /// information.
        /// </summary>
        StrongEncryption = 0x0040,
        
        /// <summary>
        /// Bit 11: Language encoding flag (EFS).  If this bit is set,
        /// the filename and comment fields for this file
        /// MUST be encoded using UTF-8 (see APPENDIX D).
        /// </summary>
        EncodingUtf8 = 0x0800,       

        /// <summary>
        /// Bit 13: Set when encrypting the Central Directory to indicate 
        /// selected data values in the Local Header are masked to
        /// hide their actual values.  See the section describing 
        /// the Strong Encryption Specification for details.  Refer
        /// to the section in this document entitled "Incorporating 
        /// PKWARE Proprietary Technology into Your Product" for 
        /// more information.
        /// </summary>
        HideLocalHeader = 0x2000,
    }
}
