﻿using System;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents general purpose bit flag for the Method 6 - Imploding 
    /// ZIP File Format Specification v6.3.3, #4.4.4.
    /// </summary>
    [Flags]
    internal enum ImplodingFlags : ushort
    {
        /// <summary>
        /// For Method 6 - Imploding.
        /// Bit 1: If the compression method used was type 6,
        /// Imploding, then this bit, if set, indicates
        /// an 8K sliding dictionary was used.  If clear,
        /// then a 4K sliding dictionary was used.
        /// </summary>
        SlidingDictionary8K = 0x0002,

        /// <summary>
        /// For Method 6 - Imploding.
        ///  Bit 2: If the compression method used was type 6,
        ///  Imploding, then this bit, if set, indicates
        ///  3 Shannon-Fano trees were used to encode the
        ///  sliding dictionary output.  If clear, then 2
        ///  Shannon-Fano trees were used.
        /// </summary>
        ShannonFano3 = 0x0004
    }
}
