﻿using System;
using System.IO;

#if NETSTANDARD
namespace Telerik.Zip
#else
namespace Telerik.Windows.Zip
#endif
{
    /// <summary>
    /// Represents local file header record described in the
    /// ZIP File Format Specification v6.3.3, #4.3.7.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1812:AvoidUninstantiatedInternalClasses")]
    internal class LocalFileHeader : FileHeaderBase, ISpecData
    {
        /// <summary>
        /// Local file header signature.
        /// </summary>
        public const uint Signature = 0x04034b50;

        /// <summary>
        /// Size of the data block without signature and variable size fields.
        /// </summary>
        public const int StaticBlockLength = 26;

        /// <summary>
        /// Initializes a new instance of the LocalFileHeader class.
        /// </summary>
        public LocalFileHeader()
        {
        }

        /// <summary>
        /// Initializes a new instance of the LocalFileHeader class.
        /// </summary>
        /// <param name="fileHeader">File header to copy properties from.</param>
        internal LocalFileHeader(FileHeaderBase fileHeader)
        {
            this.FromFileHeader(fileHeader);
        }

        /// <summary>
        /// Read data from the binary reader.
        /// </summary>
        /// <param name="reader">Binary reader to read data from.</param>
        /// <returns>True if success, otherwise false.</returns>
        public bool TryReadBlock(BinaryReader reader)
        {
            if (reader.ReadUInt32() != LocalFileHeader.Signature)
            {
                return false;
            }

            this.ReadFields(reader);
            this.ReadExtraData(reader);

            return true;
        }

        /// <summary>
        /// Write data to the binary writer.
        /// </summary>
        /// <param name="writer">Binary writer to write data to.</param>
        public void WriteBlock(BinaryWriter writer)
        {
            writer.Write(LocalFileHeader.Signature);
            this.WriteFields(writer);
            this.WriteExtraData(writer);
        }

        protected override FileHeaderInfoBase GetHeaderInfo()
        {
            LocalFileHeaderInfo headerInfo = new LocalFileHeaderInfo(this);

            return headerInfo;
        }
    }
}
